__all__ = ["B", "D", "F", "I", "I8", "I16", "I32", "I64", "U", "U8", "U16", "U32", "U64"]

# python
from typing import NewType

B = NewType("B", bool)
D = NewType("D", float)
F = NewType("F", float)
I = NewType("I", int)
I8 = NewType("I8", int)
I16 = NewType("I16", int)
I32 = NewType("I32", int)
I64 = NewType("I64", int)
U = NewType("U", int)
U8 = NewType("U8", int)
U16 = NewType("U16", int)
U32 = NewType("U32", int)
U64 = NewType("U64", int)
