# generated from codegen/templates/_emath.pyi
# pyright: reportGeneralTypeIssues=false

__all__ = [
    "BVector1",
    "DVector1",
    "FVector1",
    "I8Vector1",
    "U8Vector1",
    "I16Vector1",
    "U16Vector1",
    "I32Vector1",
    "U32Vector1",
    "IVector1",
    "UVector1",
    "I64Vector1",
    "U64Vector1",
    "BVector2",
    "DVector2",
    "FVector2",
    "I8Vector2",
    "U8Vector2",
    "I16Vector2",
    "U16Vector2",
    "I32Vector2",
    "U32Vector2",
    "IVector2",
    "UVector2",
    "I64Vector2",
    "U64Vector2",
    "BVector3",
    "DVector3",
    "FVector3",
    "I8Vector3",
    "U8Vector3",
    "I16Vector3",
    "U16Vector3",
    "I32Vector3",
    "U32Vector3",
    "IVector3",
    "UVector3",
    "I64Vector3",
    "U64Vector3",
    "BVector4",
    "DVector4",
    "FVector4",
    "I8Vector4",
    "U8Vector4",
    "I16Vector4",
    "U16Vector4",
    "I32Vector4",
    "U32Vector4",
    "IVector4",
    "UVector4",
    "I64Vector4",
    "U64Vector4",
]

import ctypes
from collections.abc import Buffer
from typing import Any
from typing import Iterator
from typing import SupportsFloat
from typing import SupportsInt
from typing import final
from typing import overload

from emath._podtyping import I8
from emath._podtyping import I16
from emath._podtyping import I32
from emath._podtyping import I64
from emath._podtyping import U8
from emath._podtyping import U16
from emath._podtyping import U32
from emath._podtyping import U64
from emath._podtyping import B
from emath._podtyping import D
from emath._podtyping import F
from emath._podtyping import I
from emath._podtyping import U

Number = SupportsFloat | SupportsInt

# use _bool instead of bool because it can be shadowed by swizzles
_bool = bool

@final
class BVector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[_bool]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> _bool: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: BVector1) -> BVector1: ...
    @overload
    def __add__(self, other: Number) -> BVector1: ...
    @overload
    def __radd__(self, other: BVector1) -> BVector1: ...
    @overload
    def __radd__(self, other: Number) -> BVector1: ...
    @overload
    def __sub__(self, other: BVector1) -> BVector1: ...
    @overload
    def __sub__(self, other: Number) -> BVector1: ...
    @overload
    def __rsub__(self, other: BVector1) -> BVector1: ...
    @overload
    def __rsub__(self, other: Number) -> BVector1: ...
    @overload
    def __mul__(self, other: BVector1) -> BVector1: ...
    @overload
    def __mul__(self, other: Number) -> BVector1: ...
    @overload
    def __rmul__(self, other: BVector1) -> BVector1: ...
    @overload
    def __rmul__(self, other: Number) -> BVector1: ...
    def __neg__(self) -> BVector1: ...
    def __abs__(self) -> BVector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> _bool: ...
    @property
    def xx(self) -> BVector2: ...
    @property
    def xo(self) -> BVector2: ...
    @property
    def xl(self) -> BVector2: ...
    @property
    def ox(self) -> BVector2: ...
    @property
    def lx(self) -> BVector2: ...
    @property
    def xxx(self) -> BVector3: ...
    @property
    def xxo(self) -> BVector3: ...
    @property
    def xxl(self) -> BVector3: ...
    @property
    def xox(self) -> BVector3: ...
    @property
    def xoo(self) -> BVector3: ...
    @property
    def xol(self) -> BVector3: ...
    @property
    def xlx(self) -> BVector3: ...
    @property
    def xlo(self) -> BVector3: ...
    @property
    def xll(self) -> BVector3: ...
    @property
    def oxx(self) -> BVector3: ...
    @property
    def oxo(self) -> BVector3: ...
    @property
    def oxl(self) -> BVector3: ...
    @property
    def oox(self) -> BVector3: ...
    @property
    def olx(self) -> BVector3: ...
    @property
    def lxx(self) -> BVector3: ...
    @property
    def lxo(self) -> BVector3: ...
    @property
    def lxl(self) -> BVector3: ...
    @property
    def lox(self) -> BVector3: ...
    @property
    def llx(self) -> BVector3: ...
    @property
    def xxxx(self) -> BVector4: ...
    @property
    def xxxo(self) -> BVector4: ...
    @property
    def xxxl(self) -> BVector4: ...
    @property
    def xxox(self) -> BVector4: ...
    @property
    def xxoo(self) -> BVector4: ...
    @property
    def xxol(self) -> BVector4: ...
    @property
    def xxlx(self) -> BVector4: ...
    @property
    def xxlo(self) -> BVector4: ...
    @property
    def xxll(self) -> BVector4: ...
    @property
    def xoxx(self) -> BVector4: ...
    @property
    def xoxo(self) -> BVector4: ...
    @property
    def xoxl(self) -> BVector4: ...
    @property
    def xoox(self) -> BVector4: ...
    @property
    def xooo(self) -> BVector4: ...
    @property
    def xool(self) -> BVector4: ...
    @property
    def xolx(self) -> BVector4: ...
    @property
    def xolo(self) -> BVector4: ...
    @property
    def xoll(self) -> BVector4: ...
    @property
    def xlxx(self) -> BVector4: ...
    @property
    def xlxo(self) -> BVector4: ...
    @property
    def xlxl(self) -> BVector4: ...
    @property
    def xlox(self) -> BVector4: ...
    @property
    def xloo(self) -> BVector4: ...
    @property
    def xlol(self) -> BVector4: ...
    @property
    def xllx(self) -> BVector4: ...
    @property
    def xllo(self) -> BVector4: ...
    @property
    def xlll(self) -> BVector4: ...
    @property
    def oxxx(self) -> BVector4: ...
    @property
    def oxxo(self) -> BVector4: ...
    @property
    def oxxl(self) -> BVector4: ...
    @property
    def oxox(self) -> BVector4: ...
    @property
    def oxoo(self) -> BVector4: ...
    @property
    def oxol(self) -> BVector4: ...
    @property
    def oxlx(self) -> BVector4: ...
    @property
    def oxlo(self) -> BVector4: ...
    @property
    def oxll(self) -> BVector4: ...
    @property
    def ooxx(self) -> BVector4: ...
    @property
    def ooxo(self) -> BVector4: ...
    @property
    def ooxl(self) -> BVector4: ...
    @property
    def ooox(self) -> BVector4: ...
    @property
    def oolx(self) -> BVector4: ...
    @property
    def olxx(self) -> BVector4: ...
    @property
    def olxo(self) -> BVector4: ...
    @property
    def olxl(self) -> BVector4: ...
    @property
    def olox(self) -> BVector4: ...
    @property
    def ollx(self) -> BVector4: ...
    @property
    def lxxx(self) -> BVector4: ...
    @property
    def lxxo(self) -> BVector4: ...
    @property
    def lxxl(self) -> BVector4: ...
    @property
    def lxox(self) -> BVector4: ...
    @property
    def lxoo(self) -> BVector4: ...
    @property
    def lxol(self) -> BVector4: ...
    @property
    def lxlx(self) -> BVector4: ...
    @property
    def lxlo(self) -> BVector4: ...
    @property
    def lxll(self) -> BVector4: ...
    @property
    def loxx(self) -> BVector4: ...
    @property
    def loxo(self) -> BVector4: ...
    @property
    def loxl(self) -> BVector4: ...
    @property
    def loox(self) -> BVector4: ...
    @property
    def lolx(self) -> BVector4: ...
    @property
    def llxx(self) -> BVector4: ...
    @property
    def llxo(self) -> BVector4: ...
    @property
    def llxl(self) -> BVector4: ...
    @property
    def llox(self) -> BVector4: ...
    @property
    def lllx(self) -> BVector4: ...
    @property
    def r(self) -> _bool: ...
    @property
    def rr(self) -> BVector2: ...
    @property
    def ro(self) -> BVector2: ...
    @property
    def rl(self) -> BVector2: ...
    @property
    def lr(self) -> BVector2: ...
    @property
    def rrr(self) -> BVector3: ...
    @property
    def rro(self) -> BVector3: ...
    @property
    def rrl(self) -> BVector3: ...
    @property
    def ror(self) -> BVector3: ...
    @property
    def roo(self) -> BVector3: ...
    @property
    def rol(self) -> BVector3: ...
    @property
    def rlr(self) -> BVector3: ...
    @property
    def rlo(self) -> BVector3: ...
    @property
    def rll(self) -> BVector3: ...
    @property
    def orr(self) -> BVector3: ...
    @property
    def oro(self) -> BVector3: ...
    @property
    def orl(self) -> BVector3: ...
    @property
    def oor(self) -> BVector3: ...
    @property
    def olr(self) -> BVector3: ...
    @property
    def lrr(self) -> BVector3: ...
    @property
    def lro(self) -> BVector3: ...
    @property
    def lrl(self) -> BVector3: ...
    @property
    def lor(self) -> BVector3: ...
    @property
    def llr(self) -> BVector3: ...
    @property
    def rrrr(self) -> BVector4: ...
    @property
    def rrro(self) -> BVector4: ...
    @property
    def rrrl(self) -> BVector4: ...
    @property
    def rror(self) -> BVector4: ...
    @property
    def rroo(self) -> BVector4: ...
    @property
    def rrol(self) -> BVector4: ...
    @property
    def rrlr(self) -> BVector4: ...
    @property
    def rrlo(self) -> BVector4: ...
    @property
    def rrll(self) -> BVector4: ...
    @property
    def rorr(self) -> BVector4: ...
    @property
    def roro(self) -> BVector4: ...
    @property
    def rorl(self) -> BVector4: ...
    @property
    def roor(self) -> BVector4: ...
    @property
    def rooo(self) -> BVector4: ...
    @property
    def rool(self) -> BVector4: ...
    @property
    def rolr(self) -> BVector4: ...
    @property
    def rolo(self) -> BVector4: ...
    @property
    def roll(self) -> BVector4: ...
    @property
    def rlrr(self) -> BVector4: ...
    @property
    def rlro(self) -> BVector4: ...
    @property
    def rlrl(self) -> BVector4: ...
    @property
    def rlor(self) -> BVector4: ...
    @property
    def rloo(self) -> BVector4: ...
    @property
    def rlol(self) -> BVector4: ...
    @property
    def rllr(self) -> BVector4: ...
    @property
    def rllo(self) -> BVector4: ...
    @property
    def rlll(self) -> BVector4: ...
    @property
    def orrr(self) -> BVector4: ...
    @property
    def orro(self) -> BVector4: ...
    @property
    def orrl(self) -> BVector4: ...
    @property
    def oror(self) -> BVector4: ...
    @property
    def oroo(self) -> BVector4: ...
    @property
    def orol(self) -> BVector4: ...
    @property
    def orlr(self) -> BVector4: ...
    @property
    def orlo(self) -> BVector4: ...
    @property
    def orll(self) -> BVector4: ...
    @property
    def oorr(self) -> BVector4: ...
    @property
    def ooro(self) -> BVector4: ...
    @property
    def oorl(self) -> BVector4: ...
    @property
    def ooor(self) -> BVector4: ...
    @property
    def oolr(self) -> BVector4: ...
    @property
    def olrr(self) -> BVector4: ...
    @property
    def olro(self) -> BVector4: ...
    @property
    def olrl(self) -> BVector4: ...
    @property
    def olor(self) -> BVector4: ...
    @property
    def ollr(self) -> BVector4: ...
    @property
    def lrrr(self) -> BVector4: ...
    @property
    def lrro(self) -> BVector4: ...
    @property
    def lrrl(self) -> BVector4: ...
    @property
    def lror(self) -> BVector4: ...
    @property
    def lroo(self) -> BVector4: ...
    @property
    def lrol(self) -> BVector4: ...
    @property
    def lrlr(self) -> BVector4: ...
    @property
    def lrlo(self) -> BVector4: ...
    @property
    def lrll(self) -> BVector4: ...
    @property
    def lorr(self) -> BVector4: ...
    @property
    def loro(self) -> BVector4: ...
    @property
    def lorl(self) -> BVector4: ...
    @property
    def loor(self) -> BVector4: ...
    @property
    def lolr(self) -> BVector4: ...
    @property
    def llrr(self) -> BVector4: ...
    @property
    def llro(self) -> BVector4: ...
    @property
    def llrl(self) -> BVector4: ...
    @property
    def llor(self) -> BVector4: ...
    @property
    def lllr(self) -> BVector4: ...
    @property
    def s(self) -> _bool: ...
    @property
    def ss(self) -> BVector2: ...
    @property
    def so(self) -> BVector2: ...
    @property
    def sl(self) -> BVector2: ...
    @property
    def os(self) -> BVector2: ...
    @property
    def ls(self) -> BVector2: ...
    @property
    def sss(self) -> BVector3: ...
    @property
    def sso(self) -> BVector3: ...
    @property
    def ssl(self) -> BVector3: ...
    @property
    def sos(self) -> BVector3: ...
    @property
    def soo(self) -> BVector3: ...
    @property
    def sol(self) -> BVector3: ...
    @property
    def sls(self) -> BVector3: ...
    @property
    def slo(self) -> BVector3: ...
    @property
    def sll(self) -> BVector3: ...
    @property
    def oss(self) -> BVector3: ...
    @property
    def oso(self) -> BVector3: ...
    @property
    def osl(self) -> BVector3: ...
    @property
    def oos(self) -> BVector3: ...
    @property
    def ols(self) -> BVector3: ...
    @property
    def lss(self) -> BVector3: ...
    @property
    def lso(self) -> BVector3: ...
    @property
    def lsl(self) -> BVector3: ...
    @property
    def los(self) -> BVector3: ...
    @property
    def lls(self) -> BVector3: ...
    @property
    def ssss(self) -> BVector4: ...
    @property
    def ssso(self) -> BVector4: ...
    @property
    def sssl(self) -> BVector4: ...
    @property
    def ssos(self) -> BVector4: ...
    @property
    def ssoo(self) -> BVector4: ...
    @property
    def ssol(self) -> BVector4: ...
    @property
    def ssls(self) -> BVector4: ...
    @property
    def sslo(self) -> BVector4: ...
    @property
    def ssll(self) -> BVector4: ...
    @property
    def soss(self) -> BVector4: ...
    @property
    def soso(self) -> BVector4: ...
    @property
    def sosl(self) -> BVector4: ...
    @property
    def soos(self) -> BVector4: ...
    @property
    def sooo(self) -> BVector4: ...
    @property
    def sool(self) -> BVector4: ...
    @property
    def sols(self) -> BVector4: ...
    @property
    def solo(self) -> BVector4: ...
    @property
    def soll(self) -> BVector4: ...
    @property
    def slss(self) -> BVector4: ...
    @property
    def slso(self) -> BVector4: ...
    @property
    def slsl(self) -> BVector4: ...
    @property
    def slos(self) -> BVector4: ...
    @property
    def sloo(self) -> BVector4: ...
    @property
    def slol(self) -> BVector4: ...
    @property
    def slls(self) -> BVector4: ...
    @property
    def sllo(self) -> BVector4: ...
    @property
    def slll(self) -> BVector4: ...
    @property
    def osss(self) -> BVector4: ...
    @property
    def osso(self) -> BVector4: ...
    @property
    def ossl(self) -> BVector4: ...
    @property
    def osos(self) -> BVector4: ...
    @property
    def osoo(self) -> BVector4: ...
    @property
    def osol(self) -> BVector4: ...
    @property
    def osls(self) -> BVector4: ...
    @property
    def oslo(self) -> BVector4: ...
    @property
    def osll(self) -> BVector4: ...
    @property
    def ooss(self) -> BVector4: ...
    @property
    def ooso(self) -> BVector4: ...
    @property
    def oosl(self) -> BVector4: ...
    @property
    def ooos(self) -> BVector4: ...
    @property
    def ools(self) -> BVector4: ...
    @property
    def olss(self) -> BVector4: ...
    @property
    def olso(self) -> BVector4: ...
    @property
    def olsl(self) -> BVector4: ...
    @property
    def olos(self) -> BVector4: ...
    @property
    def olls(self) -> BVector4: ...
    @property
    def lsss(self) -> BVector4: ...
    @property
    def lsso(self) -> BVector4: ...
    @property
    def lssl(self) -> BVector4: ...
    @property
    def lsos(self) -> BVector4: ...
    @property
    def lsoo(self) -> BVector4: ...
    @property
    def lsol(self) -> BVector4: ...
    @property
    def lsls(self) -> BVector4: ...
    @property
    def lslo(self) -> BVector4: ...
    @property
    def lsll(self) -> BVector4: ...
    @property
    def loss(self) -> BVector4: ...
    @property
    def loso(self) -> BVector4: ...
    @property
    def losl(self) -> BVector4: ...
    @property
    def loos(self) -> BVector4: ...
    @property
    def lols(self) -> BVector4: ...
    @property
    def llss(self) -> BVector4: ...
    @property
    def llso(self) -> BVector4: ...
    @property
    def llsl(self) -> BVector4: ...
    @property
    def llos(self) -> BVector4: ...
    @property
    def llls(self) -> BVector4: ...
    @property
    def u(self) -> _bool: ...
    @property
    def uu(self) -> BVector2: ...
    @property
    def uo(self) -> BVector2: ...
    @property
    def ul(self) -> BVector2: ...
    @property
    def ou(self) -> BVector2: ...
    @property
    def lu(self) -> BVector2: ...
    @property
    def uuu(self) -> BVector3: ...
    @property
    def uuo(self) -> BVector3: ...
    @property
    def uul(self) -> BVector3: ...
    @property
    def uou(self) -> BVector3: ...
    @property
    def uoo(self) -> BVector3: ...
    @property
    def uol(self) -> BVector3: ...
    @property
    def ulu(self) -> BVector3: ...
    @property
    def ulo(self) -> BVector3: ...
    @property
    def ull(self) -> BVector3: ...
    @property
    def ouu(self) -> BVector3: ...
    @property
    def ouo(self) -> BVector3: ...
    @property
    def oul(self) -> BVector3: ...
    @property
    def oou(self) -> BVector3: ...
    @property
    def olu(self) -> BVector3: ...
    @property
    def luu(self) -> BVector3: ...
    @property
    def luo(self) -> BVector3: ...
    @property
    def lul(self) -> BVector3: ...
    @property
    def lou(self) -> BVector3: ...
    @property
    def llu(self) -> BVector3: ...
    @property
    def uuuu(self) -> BVector4: ...
    @property
    def uuuo(self) -> BVector4: ...
    @property
    def uuul(self) -> BVector4: ...
    @property
    def uuou(self) -> BVector4: ...
    @property
    def uuoo(self) -> BVector4: ...
    @property
    def uuol(self) -> BVector4: ...
    @property
    def uulu(self) -> BVector4: ...
    @property
    def uulo(self) -> BVector4: ...
    @property
    def uull(self) -> BVector4: ...
    @property
    def uouu(self) -> BVector4: ...
    @property
    def uouo(self) -> BVector4: ...
    @property
    def uoul(self) -> BVector4: ...
    @property
    def uoou(self) -> BVector4: ...
    @property
    def uooo(self) -> BVector4: ...
    @property
    def uool(self) -> BVector4: ...
    @property
    def uolu(self) -> BVector4: ...
    @property
    def uolo(self) -> BVector4: ...
    @property
    def uoll(self) -> BVector4: ...
    @property
    def uluu(self) -> BVector4: ...
    @property
    def uluo(self) -> BVector4: ...
    @property
    def ulul(self) -> BVector4: ...
    @property
    def ulou(self) -> BVector4: ...
    @property
    def uloo(self) -> BVector4: ...
    @property
    def ulol(self) -> BVector4: ...
    @property
    def ullu(self) -> BVector4: ...
    @property
    def ullo(self) -> BVector4: ...
    @property
    def ulll(self) -> BVector4: ...
    @property
    def ouuu(self) -> BVector4: ...
    @property
    def ouuo(self) -> BVector4: ...
    @property
    def ouul(self) -> BVector4: ...
    @property
    def ouou(self) -> BVector4: ...
    @property
    def ouoo(self) -> BVector4: ...
    @property
    def ouol(self) -> BVector4: ...
    @property
    def oulu(self) -> BVector4: ...
    @property
    def oulo(self) -> BVector4: ...
    @property
    def oull(self) -> BVector4: ...
    @property
    def oouu(self) -> BVector4: ...
    @property
    def oouo(self) -> BVector4: ...
    @property
    def ooul(self) -> BVector4: ...
    @property
    def ooou(self) -> BVector4: ...
    @property
    def oolu(self) -> BVector4: ...
    @property
    def oluu(self) -> BVector4: ...
    @property
    def oluo(self) -> BVector4: ...
    @property
    def olul(self) -> BVector4: ...
    @property
    def olou(self) -> BVector4: ...
    @property
    def ollu(self) -> BVector4: ...
    @property
    def luuu(self) -> BVector4: ...
    @property
    def luuo(self) -> BVector4: ...
    @property
    def luul(self) -> BVector4: ...
    @property
    def luou(self) -> BVector4: ...
    @property
    def luoo(self) -> BVector4: ...
    @property
    def luol(self) -> BVector4: ...
    @property
    def lulu(self) -> BVector4: ...
    @property
    def lulo(self) -> BVector4: ...
    @property
    def lull(self) -> BVector4: ...
    @property
    def louu(self) -> BVector4: ...
    @property
    def louo(self) -> BVector4: ...
    @property
    def loul(self) -> BVector4: ...
    @property
    def loou(self) -> BVector4: ...
    @property
    def lolu(self) -> BVector4: ...
    @property
    def lluu(self) -> BVector4: ...
    @property
    def lluo(self) -> BVector4: ...
    @property
    def llul(self) -> BVector4: ...
    @property
    def llou(self) -> BVector4: ...
    @property
    def lllu(self) -> BVector4: ...
    @property
    def oo(self) -> BVector2: ...
    @property
    def ol(self) -> BVector2: ...
    @property
    def lo(self) -> BVector2: ...
    @property
    def ll(self) -> BVector2: ...
    @property
    def ooo(self) -> BVector3: ...
    @property
    def ool(self) -> BVector3: ...
    @property
    def olo(self) -> BVector3: ...
    @property
    def oll(self) -> BVector3: ...
    @property
    def loo(self) -> BVector3: ...
    @property
    def lol(self) -> BVector3: ...
    @property
    def llo(self) -> BVector3: ...
    @property
    def lll(self) -> BVector3: ...
    @property
    def oooo(self) -> BVector4: ...
    @property
    def oool(self) -> BVector4: ...
    @property
    def oolo(self) -> BVector4: ...
    @property
    def ooll(self) -> BVector4: ...
    @property
    def oloo(self) -> BVector4: ...
    @property
    def olol(self) -> BVector4: ...
    @property
    def ollo(self) -> BVector4: ...
    @property
    def olll(self) -> BVector4: ...
    @property
    def looo(self) -> BVector4: ...
    @property
    def lool(self) -> BVector4: ...
    @property
    def lolo(self) -> BVector4: ...
    @property
    def loll(self) -> BVector4: ...
    @property
    def lloo(self) -> BVector4: ...
    @property
    def llol(self) -> BVector4: ...
    @property
    def lllo(self) -> BVector4: ...
    @property
    def llll(self) -> BVector4: ...
    def min(self, n: Number, /) -> BVector1: ...
    def max(self, n: Number, /) -> BVector1: ...
    def clamp(self, min: Number, max: Number, /) -> BVector1: ...
    @classmethod
    def get_limits(cls) -> tuple[_bool, _bool]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[BVector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector1: ...

@final
class BVector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: BVector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[BVector1]: ...
    @overload
    def __getitem__(self, index: int) -> BVector1: ...
    @overload
    def __getitem__(self, index: slice) -> BVector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[BVector1]: ...

@final
class DVector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DVector1) -> DVector1: ...
    @overload
    def __add__(self, other: Number) -> DVector1: ...
    @overload
    def __radd__(self, other: DVector1) -> DVector1: ...
    @overload
    def __radd__(self, other: Number) -> DVector1: ...
    @overload
    def __sub__(self, other: DVector1) -> DVector1: ...
    @overload
    def __sub__(self, other: Number) -> DVector1: ...
    @overload
    def __rsub__(self, other: DVector1) -> DVector1: ...
    @overload
    def __rsub__(self, other: Number) -> DVector1: ...
    @overload
    def __mul__(self, other: DVector1) -> DVector1: ...
    @overload
    def __mul__(self, other: Number) -> DVector1: ...
    @overload
    def __rmul__(self, other: DVector1) -> DVector1: ...
    @overload
    def __rmul__(self, other: Number) -> DVector1: ...
    def __matmul__(self, other: DVector1) -> float: ...
    def __rmatmul__(self, other: DVector1) -> float: ...
    @overload
    def __mod__(self, other: DVector1) -> DVector1: ...
    @overload
    def __mod__(self, other: Number) -> DVector1: ...
    @overload
    def __rmod__(self, other: DVector1) -> DVector1: ...
    @overload
    def __rmod__(self, other: Number) -> DVector1: ...
    @overload
    def __pow__(self, other: DVector1) -> DVector1: ...
    @overload
    def __pow__(self, other: Number) -> DVector1: ...
    @overload
    def __rpow__(self, other: DVector1) -> DVector1: ...
    @overload
    def __rpow__(self, other: Number) -> DVector1: ...
    @overload
    def __truediv__(self, other: DVector1) -> DVector1: ...
    @overload
    def __truediv__(self, other: Number) -> DVector1: ...
    @overload
    def __rtruediv__(self, other: DVector1) -> DVector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> DVector1: ...
    def __neg__(self) -> DVector1: ...
    def __abs__(self) -> DVector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def xx(self) -> DVector2: ...
    @property
    def xo(self) -> DVector2: ...
    @property
    def xl(self) -> DVector2: ...
    @property
    def ox(self) -> DVector2: ...
    @property
    def lx(self) -> DVector2: ...
    @property
    def xxx(self) -> DVector3: ...
    @property
    def xxo(self) -> DVector3: ...
    @property
    def xxl(self) -> DVector3: ...
    @property
    def xox(self) -> DVector3: ...
    @property
    def xoo(self) -> DVector3: ...
    @property
    def xol(self) -> DVector3: ...
    @property
    def xlx(self) -> DVector3: ...
    @property
    def xlo(self) -> DVector3: ...
    @property
    def xll(self) -> DVector3: ...
    @property
    def oxx(self) -> DVector3: ...
    @property
    def oxo(self) -> DVector3: ...
    @property
    def oxl(self) -> DVector3: ...
    @property
    def oox(self) -> DVector3: ...
    @property
    def olx(self) -> DVector3: ...
    @property
    def lxx(self) -> DVector3: ...
    @property
    def lxo(self) -> DVector3: ...
    @property
    def lxl(self) -> DVector3: ...
    @property
    def lox(self) -> DVector3: ...
    @property
    def llx(self) -> DVector3: ...
    @property
    def xxxx(self) -> DVector4: ...
    @property
    def xxxo(self) -> DVector4: ...
    @property
    def xxxl(self) -> DVector4: ...
    @property
    def xxox(self) -> DVector4: ...
    @property
    def xxoo(self) -> DVector4: ...
    @property
    def xxol(self) -> DVector4: ...
    @property
    def xxlx(self) -> DVector4: ...
    @property
    def xxlo(self) -> DVector4: ...
    @property
    def xxll(self) -> DVector4: ...
    @property
    def xoxx(self) -> DVector4: ...
    @property
    def xoxo(self) -> DVector4: ...
    @property
    def xoxl(self) -> DVector4: ...
    @property
    def xoox(self) -> DVector4: ...
    @property
    def xooo(self) -> DVector4: ...
    @property
    def xool(self) -> DVector4: ...
    @property
    def xolx(self) -> DVector4: ...
    @property
    def xolo(self) -> DVector4: ...
    @property
    def xoll(self) -> DVector4: ...
    @property
    def xlxx(self) -> DVector4: ...
    @property
    def xlxo(self) -> DVector4: ...
    @property
    def xlxl(self) -> DVector4: ...
    @property
    def xlox(self) -> DVector4: ...
    @property
    def xloo(self) -> DVector4: ...
    @property
    def xlol(self) -> DVector4: ...
    @property
    def xllx(self) -> DVector4: ...
    @property
    def xllo(self) -> DVector4: ...
    @property
    def xlll(self) -> DVector4: ...
    @property
    def oxxx(self) -> DVector4: ...
    @property
    def oxxo(self) -> DVector4: ...
    @property
    def oxxl(self) -> DVector4: ...
    @property
    def oxox(self) -> DVector4: ...
    @property
    def oxoo(self) -> DVector4: ...
    @property
    def oxol(self) -> DVector4: ...
    @property
    def oxlx(self) -> DVector4: ...
    @property
    def oxlo(self) -> DVector4: ...
    @property
    def oxll(self) -> DVector4: ...
    @property
    def ooxx(self) -> DVector4: ...
    @property
    def ooxo(self) -> DVector4: ...
    @property
    def ooxl(self) -> DVector4: ...
    @property
    def ooox(self) -> DVector4: ...
    @property
    def oolx(self) -> DVector4: ...
    @property
    def olxx(self) -> DVector4: ...
    @property
    def olxo(self) -> DVector4: ...
    @property
    def olxl(self) -> DVector4: ...
    @property
    def olox(self) -> DVector4: ...
    @property
    def ollx(self) -> DVector4: ...
    @property
    def lxxx(self) -> DVector4: ...
    @property
    def lxxo(self) -> DVector4: ...
    @property
    def lxxl(self) -> DVector4: ...
    @property
    def lxox(self) -> DVector4: ...
    @property
    def lxoo(self) -> DVector4: ...
    @property
    def lxol(self) -> DVector4: ...
    @property
    def lxlx(self) -> DVector4: ...
    @property
    def lxlo(self) -> DVector4: ...
    @property
    def lxll(self) -> DVector4: ...
    @property
    def loxx(self) -> DVector4: ...
    @property
    def loxo(self) -> DVector4: ...
    @property
    def loxl(self) -> DVector4: ...
    @property
    def loox(self) -> DVector4: ...
    @property
    def lolx(self) -> DVector4: ...
    @property
    def llxx(self) -> DVector4: ...
    @property
    def llxo(self) -> DVector4: ...
    @property
    def llxl(self) -> DVector4: ...
    @property
    def llox(self) -> DVector4: ...
    @property
    def lllx(self) -> DVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def rr(self) -> DVector2: ...
    @property
    def ro(self) -> DVector2: ...
    @property
    def rl(self) -> DVector2: ...
    @property
    def lr(self) -> DVector2: ...
    @property
    def rrr(self) -> DVector3: ...
    @property
    def rro(self) -> DVector3: ...
    @property
    def rrl(self) -> DVector3: ...
    @property
    def ror(self) -> DVector3: ...
    @property
    def roo(self) -> DVector3: ...
    @property
    def rol(self) -> DVector3: ...
    @property
    def rlr(self) -> DVector3: ...
    @property
    def rlo(self) -> DVector3: ...
    @property
    def rll(self) -> DVector3: ...
    @property
    def orr(self) -> DVector3: ...
    @property
    def oro(self) -> DVector3: ...
    @property
    def orl(self) -> DVector3: ...
    @property
    def oor(self) -> DVector3: ...
    @property
    def olr(self) -> DVector3: ...
    @property
    def lrr(self) -> DVector3: ...
    @property
    def lro(self) -> DVector3: ...
    @property
    def lrl(self) -> DVector3: ...
    @property
    def lor(self) -> DVector3: ...
    @property
    def llr(self) -> DVector3: ...
    @property
    def rrrr(self) -> DVector4: ...
    @property
    def rrro(self) -> DVector4: ...
    @property
    def rrrl(self) -> DVector4: ...
    @property
    def rror(self) -> DVector4: ...
    @property
    def rroo(self) -> DVector4: ...
    @property
    def rrol(self) -> DVector4: ...
    @property
    def rrlr(self) -> DVector4: ...
    @property
    def rrlo(self) -> DVector4: ...
    @property
    def rrll(self) -> DVector4: ...
    @property
    def rorr(self) -> DVector4: ...
    @property
    def roro(self) -> DVector4: ...
    @property
    def rorl(self) -> DVector4: ...
    @property
    def roor(self) -> DVector4: ...
    @property
    def rooo(self) -> DVector4: ...
    @property
    def rool(self) -> DVector4: ...
    @property
    def rolr(self) -> DVector4: ...
    @property
    def rolo(self) -> DVector4: ...
    @property
    def roll(self) -> DVector4: ...
    @property
    def rlrr(self) -> DVector4: ...
    @property
    def rlro(self) -> DVector4: ...
    @property
    def rlrl(self) -> DVector4: ...
    @property
    def rlor(self) -> DVector4: ...
    @property
    def rloo(self) -> DVector4: ...
    @property
    def rlol(self) -> DVector4: ...
    @property
    def rllr(self) -> DVector4: ...
    @property
    def rllo(self) -> DVector4: ...
    @property
    def rlll(self) -> DVector4: ...
    @property
    def orrr(self) -> DVector4: ...
    @property
    def orro(self) -> DVector4: ...
    @property
    def orrl(self) -> DVector4: ...
    @property
    def oror(self) -> DVector4: ...
    @property
    def oroo(self) -> DVector4: ...
    @property
    def orol(self) -> DVector4: ...
    @property
    def orlr(self) -> DVector4: ...
    @property
    def orlo(self) -> DVector4: ...
    @property
    def orll(self) -> DVector4: ...
    @property
    def oorr(self) -> DVector4: ...
    @property
    def ooro(self) -> DVector4: ...
    @property
    def oorl(self) -> DVector4: ...
    @property
    def ooor(self) -> DVector4: ...
    @property
    def oolr(self) -> DVector4: ...
    @property
    def olrr(self) -> DVector4: ...
    @property
    def olro(self) -> DVector4: ...
    @property
    def olrl(self) -> DVector4: ...
    @property
    def olor(self) -> DVector4: ...
    @property
    def ollr(self) -> DVector4: ...
    @property
    def lrrr(self) -> DVector4: ...
    @property
    def lrro(self) -> DVector4: ...
    @property
    def lrrl(self) -> DVector4: ...
    @property
    def lror(self) -> DVector4: ...
    @property
    def lroo(self) -> DVector4: ...
    @property
    def lrol(self) -> DVector4: ...
    @property
    def lrlr(self) -> DVector4: ...
    @property
    def lrlo(self) -> DVector4: ...
    @property
    def lrll(self) -> DVector4: ...
    @property
    def lorr(self) -> DVector4: ...
    @property
    def loro(self) -> DVector4: ...
    @property
    def lorl(self) -> DVector4: ...
    @property
    def loor(self) -> DVector4: ...
    @property
    def lolr(self) -> DVector4: ...
    @property
    def llrr(self) -> DVector4: ...
    @property
    def llro(self) -> DVector4: ...
    @property
    def llrl(self) -> DVector4: ...
    @property
    def llor(self) -> DVector4: ...
    @property
    def lllr(self) -> DVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def ss(self) -> DVector2: ...
    @property
    def so(self) -> DVector2: ...
    @property
    def sl(self) -> DVector2: ...
    @property
    def os(self) -> DVector2: ...
    @property
    def ls(self) -> DVector2: ...
    @property
    def sss(self) -> DVector3: ...
    @property
    def sso(self) -> DVector3: ...
    @property
    def ssl(self) -> DVector3: ...
    @property
    def sos(self) -> DVector3: ...
    @property
    def soo(self) -> DVector3: ...
    @property
    def sol(self) -> DVector3: ...
    @property
    def sls(self) -> DVector3: ...
    @property
    def slo(self) -> DVector3: ...
    @property
    def sll(self) -> DVector3: ...
    @property
    def oss(self) -> DVector3: ...
    @property
    def oso(self) -> DVector3: ...
    @property
    def osl(self) -> DVector3: ...
    @property
    def oos(self) -> DVector3: ...
    @property
    def ols(self) -> DVector3: ...
    @property
    def lss(self) -> DVector3: ...
    @property
    def lso(self) -> DVector3: ...
    @property
    def lsl(self) -> DVector3: ...
    @property
    def los(self) -> DVector3: ...
    @property
    def lls(self) -> DVector3: ...
    @property
    def ssss(self) -> DVector4: ...
    @property
    def ssso(self) -> DVector4: ...
    @property
    def sssl(self) -> DVector4: ...
    @property
    def ssos(self) -> DVector4: ...
    @property
    def ssoo(self) -> DVector4: ...
    @property
    def ssol(self) -> DVector4: ...
    @property
    def ssls(self) -> DVector4: ...
    @property
    def sslo(self) -> DVector4: ...
    @property
    def ssll(self) -> DVector4: ...
    @property
    def soss(self) -> DVector4: ...
    @property
    def soso(self) -> DVector4: ...
    @property
    def sosl(self) -> DVector4: ...
    @property
    def soos(self) -> DVector4: ...
    @property
    def sooo(self) -> DVector4: ...
    @property
    def sool(self) -> DVector4: ...
    @property
    def sols(self) -> DVector4: ...
    @property
    def solo(self) -> DVector4: ...
    @property
    def soll(self) -> DVector4: ...
    @property
    def slss(self) -> DVector4: ...
    @property
    def slso(self) -> DVector4: ...
    @property
    def slsl(self) -> DVector4: ...
    @property
    def slos(self) -> DVector4: ...
    @property
    def sloo(self) -> DVector4: ...
    @property
    def slol(self) -> DVector4: ...
    @property
    def slls(self) -> DVector4: ...
    @property
    def sllo(self) -> DVector4: ...
    @property
    def slll(self) -> DVector4: ...
    @property
    def osss(self) -> DVector4: ...
    @property
    def osso(self) -> DVector4: ...
    @property
    def ossl(self) -> DVector4: ...
    @property
    def osos(self) -> DVector4: ...
    @property
    def osoo(self) -> DVector4: ...
    @property
    def osol(self) -> DVector4: ...
    @property
    def osls(self) -> DVector4: ...
    @property
    def oslo(self) -> DVector4: ...
    @property
    def osll(self) -> DVector4: ...
    @property
    def ooss(self) -> DVector4: ...
    @property
    def ooso(self) -> DVector4: ...
    @property
    def oosl(self) -> DVector4: ...
    @property
    def ooos(self) -> DVector4: ...
    @property
    def ools(self) -> DVector4: ...
    @property
    def olss(self) -> DVector4: ...
    @property
    def olso(self) -> DVector4: ...
    @property
    def olsl(self) -> DVector4: ...
    @property
    def olos(self) -> DVector4: ...
    @property
    def olls(self) -> DVector4: ...
    @property
    def lsss(self) -> DVector4: ...
    @property
    def lsso(self) -> DVector4: ...
    @property
    def lssl(self) -> DVector4: ...
    @property
    def lsos(self) -> DVector4: ...
    @property
    def lsoo(self) -> DVector4: ...
    @property
    def lsol(self) -> DVector4: ...
    @property
    def lsls(self) -> DVector4: ...
    @property
    def lslo(self) -> DVector4: ...
    @property
    def lsll(self) -> DVector4: ...
    @property
    def loss(self) -> DVector4: ...
    @property
    def loso(self) -> DVector4: ...
    @property
    def losl(self) -> DVector4: ...
    @property
    def loos(self) -> DVector4: ...
    @property
    def lols(self) -> DVector4: ...
    @property
    def llss(self) -> DVector4: ...
    @property
    def llso(self) -> DVector4: ...
    @property
    def llsl(self) -> DVector4: ...
    @property
    def llos(self) -> DVector4: ...
    @property
    def llls(self) -> DVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def uu(self) -> DVector2: ...
    @property
    def uo(self) -> DVector2: ...
    @property
    def ul(self) -> DVector2: ...
    @property
    def ou(self) -> DVector2: ...
    @property
    def lu(self) -> DVector2: ...
    @property
    def uuu(self) -> DVector3: ...
    @property
    def uuo(self) -> DVector3: ...
    @property
    def uul(self) -> DVector3: ...
    @property
    def uou(self) -> DVector3: ...
    @property
    def uoo(self) -> DVector3: ...
    @property
    def uol(self) -> DVector3: ...
    @property
    def ulu(self) -> DVector3: ...
    @property
    def ulo(self) -> DVector3: ...
    @property
    def ull(self) -> DVector3: ...
    @property
    def ouu(self) -> DVector3: ...
    @property
    def ouo(self) -> DVector3: ...
    @property
    def oul(self) -> DVector3: ...
    @property
    def oou(self) -> DVector3: ...
    @property
    def olu(self) -> DVector3: ...
    @property
    def luu(self) -> DVector3: ...
    @property
    def luo(self) -> DVector3: ...
    @property
    def lul(self) -> DVector3: ...
    @property
    def lou(self) -> DVector3: ...
    @property
    def llu(self) -> DVector3: ...
    @property
    def uuuu(self) -> DVector4: ...
    @property
    def uuuo(self) -> DVector4: ...
    @property
    def uuul(self) -> DVector4: ...
    @property
    def uuou(self) -> DVector4: ...
    @property
    def uuoo(self) -> DVector4: ...
    @property
    def uuol(self) -> DVector4: ...
    @property
    def uulu(self) -> DVector4: ...
    @property
    def uulo(self) -> DVector4: ...
    @property
    def uull(self) -> DVector4: ...
    @property
    def uouu(self) -> DVector4: ...
    @property
    def uouo(self) -> DVector4: ...
    @property
    def uoul(self) -> DVector4: ...
    @property
    def uoou(self) -> DVector4: ...
    @property
    def uooo(self) -> DVector4: ...
    @property
    def uool(self) -> DVector4: ...
    @property
    def uolu(self) -> DVector4: ...
    @property
    def uolo(self) -> DVector4: ...
    @property
    def uoll(self) -> DVector4: ...
    @property
    def uluu(self) -> DVector4: ...
    @property
    def uluo(self) -> DVector4: ...
    @property
    def ulul(self) -> DVector4: ...
    @property
    def ulou(self) -> DVector4: ...
    @property
    def uloo(self) -> DVector4: ...
    @property
    def ulol(self) -> DVector4: ...
    @property
    def ullu(self) -> DVector4: ...
    @property
    def ullo(self) -> DVector4: ...
    @property
    def ulll(self) -> DVector4: ...
    @property
    def ouuu(self) -> DVector4: ...
    @property
    def ouuo(self) -> DVector4: ...
    @property
    def ouul(self) -> DVector4: ...
    @property
    def ouou(self) -> DVector4: ...
    @property
    def ouoo(self) -> DVector4: ...
    @property
    def ouol(self) -> DVector4: ...
    @property
    def oulu(self) -> DVector4: ...
    @property
    def oulo(self) -> DVector4: ...
    @property
    def oull(self) -> DVector4: ...
    @property
    def oouu(self) -> DVector4: ...
    @property
    def oouo(self) -> DVector4: ...
    @property
    def ooul(self) -> DVector4: ...
    @property
    def ooou(self) -> DVector4: ...
    @property
    def oolu(self) -> DVector4: ...
    @property
    def oluu(self) -> DVector4: ...
    @property
    def oluo(self) -> DVector4: ...
    @property
    def olul(self) -> DVector4: ...
    @property
    def olou(self) -> DVector4: ...
    @property
    def ollu(self) -> DVector4: ...
    @property
    def luuu(self) -> DVector4: ...
    @property
    def luuo(self) -> DVector4: ...
    @property
    def luul(self) -> DVector4: ...
    @property
    def luou(self) -> DVector4: ...
    @property
    def luoo(self) -> DVector4: ...
    @property
    def luol(self) -> DVector4: ...
    @property
    def lulu(self) -> DVector4: ...
    @property
    def lulo(self) -> DVector4: ...
    @property
    def lull(self) -> DVector4: ...
    @property
    def louu(self) -> DVector4: ...
    @property
    def louo(self) -> DVector4: ...
    @property
    def loul(self) -> DVector4: ...
    @property
    def loou(self) -> DVector4: ...
    @property
    def lolu(self) -> DVector4: ...
    @property
    def lluu(self) -> DVector4: ...
    @property
    def lluo(self) -> DVector4: ...
    @property
    def llul(self) -> DVector4: ...
    @property
    def llou(self) -> DVector4: ...
    @property
    def lllu(self) -> DVector4: ...
    @property
    def oo(self) -> DVector2: ...
    @property
    def ol(self) -> DVector2: ...
    @property
    def lo(self) -> DVector2: ...
    @property
    def ll(self) -> DVector2: ...
    @property
    def ooo(self) -> DVector3: ...
    @property
    def ool(self) -> DVector3: ...
    @property
    def olo(self) -> DVector3: ...
    @property
    def oll(self) -> DVector3: ...
    @property
    def loo(self) -> DVector3: ...
    @property
    def lol(self) -> DVector3: ...
    @property
    def llo(self) -> DVector3: ...
    @property
    def lll(self) -> DVector3: ...
    @property
    def oooo(self) -> DVector4: ...
    @property
    def oool(self) -> DVector4: ...
    @property
    def oolo(self) -> DVector4: ...
    @property
    def ooll(self) -> DVector4: ...
    @property
    def oloo(self) -> DVector4: ...
    @property
    def olol(self) -> DVector4: ...
    @property
    def ollo(self) -> DVector4: ...
    @property
    def olll(self) -> DVector4: ...
    @property
    def looo(self) -> DVector4: ...
    @property
    def lool(self) -> DVector4: ...
    @property
    def lolo(self) -> DVector4: ...
    @property
    def loll(self) -> DVector4: ...
    @property
    def lloo(self) -> DVector4: ...
    @property
    def llol(self) -> DVector4: ...
    @property
    def lllo(self) -> DVector4: ...
    @property
    def llll(self) -> DVector4: ...
    @property
    def magnitude(self) -> float: ...
    def normalize(self) -> DVector1: ...
    def distance(self, other: DVector1, /) -> float: ...
    def lerp(self, other: DVector1, x: float, /) -> DVector1: ...
    def min(self, n: Number, /) -> DVector1: ...
    def max(self, n: Number, /) -> DVector1: ...
    def clamp(self, min: Number, max: Number, /) -> DVector1: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[DVector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector1: ...

@final
class DVector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: DVector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector1]: ...
    @overload
    def __getitem__(self, index: int) -> DVector1: ...
    @overload
    def __getitem__(self, index: slice) -> DVector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[DVector1]: ...

@final
class FVector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FVector1) -> FVector1: ...
    @overload
    def __add__(self, other: Number) -> FVector1: ...
    @overload
    def __radd__(self, other: FVector1) -> FVector1: ...
    @overload
    def __radd__(self, other: Number) -> FVector1: ...
    @overload
    def __sub__(self, other: FVector1) -> FVector1: ...
    @overload
    def __sub__(self, other: Number) -> FVector1: ...
    @overload
    def __rsub__(self, other: FVector1) -> FVector1: ...
    @overload
    def __rsub__(self, other: Number) -> FVector1: ...
    @overload
    def __mul__(self, other: FVector1) -> FVector1: ...
    @overload
    def __mul__(self, other: Number) -> FVector1: ...
    @overload
    def __rmul__(self, other: FVector1) -> FVector1: ...
    @overload
    def __rmul__(self, other: Number) -> FVector1: ...
    def __matmul__(self, other: FVector1) -> float: ...
    def __rmatmul__(self, other: FVector1) -> float: ...
    @overload
    def __mod__(self, other: FVector1) -> FVector1: ...
    @overload
    def __mod__(self, other: Number) -> FVector1: ...
    @overload
    def __rmod__(self, other: FVector1) -> FVector1: ...
    @overload
    def __rmod__(self, other: Number) -> FVector1: ...
    @overload
    def __pow__(self, other: FVector1) -> FVector1: ...
    @overload
    def __pow__(self, other: Number) -> FVector1: ...
    @overload
    def __rpow__(self, other: FVector1) -> FVector1: ...
    @overload
    def __rpow__(self, other: Number) -> FVector1: ...
    @overload
    def __truediv__(self, other: FVector1) -> FVector1: ...
    @overload
    def __truediv__(self, other: Number) -> FVector1: ...
    @overload
    def __rtruediv__(self, other: FVector1) -> FVector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> FVector1: ...
    def __neg__(self) -> FVector1: ...
    def __abs__(self) -> FVector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def xx(self) -> FVector2: ...
    @property
    def xo(self) -> FVector2: ...
    @property
    def xl(self) -> FVector2: ...
    @property
    def ox(self) -> FVector2: ...
    @property
    def lx(self) -> FVector2: ...
    @property
    def xxx(self) -> FVector3: ...
    @property
    def xxo(self) -> FVector3: ...
    @property
    def xxl(self) -> FVector3: ...
    @property
    def xox(self) -> FVector3: ...
    @property
    def xoo(self) -> FVector3: ...
    @property
    def xol(self) -> FVector3: ...
    @property
    def xlx(self) -> FVector3: ...
    @property
    def xlo(self) -> FVector3: ...
    @property
    def xll(self) -> FVector3: ...
    @property
    def oxx(self) -> FVector3: ...
    @property
    def oxo(self) -> FVector3: ...
    @property
    def oxl(self) -> FVector3: ...
    @property
    def oox(self) -> FVector3: ...
    @property
    def olx(self) -> FVector3: ...
    @property
    def lxx(self) -> FVector3: ...
    @property
    def lxo(self) -> FVector3: ...
    @property
    def lxl(self) -> FVector3: ...
    @property
    def lox(self) -> FVector3: ...
    @property
    def llx(self) -> FVector3: ...
    @property
    def xxxx(self) -> FVector4: ...
    @property
    def xxxo(self) -> FVector4: ...
    @property
    def xxxl(self) -> FVector4: ...
    @property
    def xxox(self) -> FVector4: ...
    @property
    def xxoo(self) -> FVector4: ...
    @property
    def xxol(self) -> FVector4: ...
    @property
    def xxlx(self) -> FVector4: ...
    @property
    def xxlo(self) -> FVector4: ...
    @property
    def xxll(self) -> FVector4: ...
    @property
    def xoxx(self) -> FVector4: ...
    @property
    def xoxo(self) -> FVector4: ...
    @property
    def xoxl(self) -> FVector4: ...
    @property
    def xoox(self) -> FVector4: ...
    @property
    def xooo(self) -> FVector4: ...
    @property
    def xool(self) -> FVector4: ...
    @property
    def xolx(self) -> FVector4: ...
    @property
    def xolo(self) -> FVector4: ...
    @property
    def xoll(self) -> FVector4: ...
    @property
    def xlxx(self) -> FVector4: ...
    @property
    def xlxo(self) -> FVector4: ...
    @property
    def xlxl(self) -> FVector4: ...
    @property
    def xlox(self) -> FVector4: ...
    @property
    def xloo(self) -> FVector4: ...
    @property
    def xlol(self) -> FVector4: ...
    @property
    def xllx(self) -> FVector4: ...
    @property
    def xllo(self) -> FVector4: ...
    @property
    def xlll(self) -> FVector4: ...
    @property
    def oxxx(self) -> FVector4: ...
    @property
    def oxxo(self) -> FVector4: ...
    @property
    def oxxl(self) -> FVector4: ...
    @property
    def oxox(self) -> FVector4: ...
    @property
    def oxoo(self) -> FVector4: ...
    @property
    def oxol(self) -> FVector4: ...
    @property
    def oxlx(self) -> FVector4: ...
    @property
    def oxlo(self) -> FVector4: ...
    @property
    def oxll(self) -> FVector4: ...
    @property
    def ooxx(self) -> FVector4: ...
    @property
    def ooxo(self) -> FVector4: ...
    @property
    def ooxl(self) -> FVector4: ...
    @property
    def ooox(self) -> FVector4: ...
    @property
    def oolx(self) -> FVector4: ...
    @property
    def olxx(self) -> FVector4: ...
    @property
    def olxo(self) -> FVector4: ...
    @property
    def olxl(self) -> FVector4: ...
    @property
    def olox(self) -> FVector4: ...
    @property
    def ollx(self) -> FVector4: ...
    @property
    def lxxx(self) -> FVector4: ...
    @property
    def lxxo(self) -> FVector4: ...
    @property
    def lxxl(self) -> FVector4: ...
    @property
    def lxox(self) -> FVector4: ...
    @property
    def lxoo(self) -> FVector4: ...
    @property
    def lxol(self) -> FVector4: ...
    @property
    def lxlx(self) -> FVector4: ...
    @property
    def lxlo(self) -> FVector4: ...
    @property
    def lxll(self) -> FVector4: ...
    @property
    def loxx(self) -> FVector4: ...
    @property
    def loxo(self) -> FVector4: ...
    @property
    def loxl(self) -> FVector4: ...
    @property
    def loox(self) -> FVector4: ...
    @property
    def lolx(self) -> FVector4: ...
    @property
    def llxx(self) -> FVector4: ...
    @property
    def llxo(self) -> FVector4: ...
    @property
    def llxl(self) -> FVector4: ...
    @property
    def llox(self) -> FVector4: ...
    @property
    def lllx(self) -> FVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def rr(self) -> FVector2: ...
    @property
    def ro(self) -> FVector2: ...
    @property
    def rl(self) -> FVector2: ...
    @property
    def lr(self) -> FVector2: ...
    @property
    def rrr(self) -> FVector3: ...
    @property
    def rro(self) -> FVector3: ...
    @property
    def rrl(self) -> FVector3: ...
    @property
    def ror(self) -> FVector3: ...
    @property
    def roo(self) -> FVector3: ...
    @property
    def rol(self) -> FVector3: ...
    @property
    def rlr(self) -> FVector3: ...
    @property
    def rlo(self) -> FVector3: ...
    @property
    def rll(self) -> FVector3: ...
    @property
    def orr(self) -> FVector3: ...
    @property
    def oro(self) -> FVector3: ...
    @property
    def orl(self) -> FVector3: ...
    @property
    def oor(self) -> FVector3: ...
    @property
    def olr(self) -> FVector3: ...
    @property
    def lrr(self) -> FVector3: ...
    @property
    def lro(self) -> FVector3: ...
    @property
    def lrl(self) -> FVector3: ...
    @property
    def lor(self) -> FVector3: ...
    @property
    def llr(self) -> FVector3: ...
    @property
    def rrrr(self) -> FVector4: ...
    @property
    def rrro(self) -> FVector4: ...
    @property
    def rrrl(self) -> FVector4: ...
    @property
    def rror(self) -> FVector4: ...
    @property
    def rroo(self) -> FVector4: ...
    @property
    def rrol(self) -> FVector4: ...
    @property
    def rrlr(self) -> FVector4: ...
    @property
    def rrlo(self) -> FVector4: ...
    @property
    def rrll(self) -> FVector4: ...
    @property
    def rorr(self) -> FVector4: ...
    @property
    def roro(self) -> FVector4: ...
    @property
    def rorl(self) -> FVector4: ...
    @property
    def roor(self) -> FVector4: ...
    @property
    def rooo(self) -> FVector4: ...
    @property
    def rool(self) -> FVector4: ...
    @property
    def rolr(self) -> FVector4: ...
    @property
    def rolo(self) -> FVector4: ...
    @property
    def roll(self) -> FVector4: ...
    @property
    def rlrr(self) -> FVector4: ...
    @property
    def rlro(self) -> FVector4: ...
    @property
    def rlrl(self) -> FVector4: ...
    @property
    def rlor(self) -> FVector4: ...
    @property
    def rloo(self) -> FVector4: ...
    @property
    def rlol(self) -> FVector4: ...
    @property
    def rllr(self) -> FVector4: ...
    @property
    def rllo(self) -> FVector4: ...
    @property
    def rlll(self) -> FVector4: ...
    @property
    def orrr(self) -> FVector4: ...
    @property
    def orro(self) -> FVector4: ...
    @property
    def orrl(self) -> FVector4: ...
    @property
    def oror(self) -> FVector4: ...
    @property
    def oroo(self) -> FVector4: ...
    @property
    def orol(self) -> FVector4: ...
    @property
    def orlr(self) -> FVector4: ...
    @property
    def orlo(self) -> FVector4: ...
    @property
    def orll(self) -> FVector4: ...
    @property
    def oorr(self) -> FVector4: ...
    @property
    def ooro(self) -> FVector4: ...
    @property
    def oorl(self) -> FVector4: ...
    @property
    def ooor(self) -> FVector4: ...
    @property
    def oolr(self) -> FVector4: ...
    @property
    def olrr(self) -> FVector4: ...
    @property
    def olro(self) -> FVector4: ...
    @property
    def olrl(self) -> FVector4: ...
    @property
    def olor(self) -> FVector4: ...
    @property
    def ollr(self) -> FVector4: ...
    @property
    def lrrr(self) -> FVector4: ...
    @property
    def lrro(self) -> FVector4: ...
    @property
    def lrrl(self) -> FVector4: ...
    @property
    def lror(self) -> FVector4: ...
    @property
    def lroo(self) -> FVector4: ...
    @property
    def lrol(self) -> FVector4: ...
    @property
    def lrlr(self) -> FVector4: ...
    @property
    def lrlo(self) -> FVector4: ...
    @property
    def lrll(self) -> FVector4: ...
    @property
    def lorr(self) -> FVector4: ...
    @property
    def loro(self) -> FVector4: ...
    @property
    def lorl(self) -> FVector4: ...
    @property
    def loor(self) -> FVector4: ...
    @property
    def lolr(self) -> FVector4: ...
    @property
    def llrr(self) -> FVector4: ...
    @property
    def llro(self) -> FVector4: ...
    @property
    def llrl(self) -> FVector4: ...
    @property
    def llor(self) -> FVector4: ...
    @property
    def lllr(self) -> FVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def ss(self) -> FVector2: ...
    @property
    def so(self) -> FVector2: ...
    @property
    def sl(self) -> FVector2: ...
    @property
    def os(self) -> FVector2: ...
    @property
    def ls(self) -> FVector2: ...
    @property
    def sss(self) -> FVector3: ...
    @property
    def sso(self) -> FVector3: ...
    @property
    def ssl(self) -> FVector3: ...
    @property
    def sos(self) -> FVector3: ...
    @property
    def soo(self) -> FVector3: ...
    @property
    def sol(self) -> FVector3: ...
    @property
    def sls(self) -> FVector3: ...
    @property
    def slo(self) -> FVector3: ...
    @property
    def sll(self) -> FVector3: ...
    @property
    def oss(self) -> FVector3: ...
    @property
    def oso(self) -> FVector3: ...
    @property
    def osl(self) -> FVector3: ...
    @property
    def oos(self) -> FVector3: ...
    @property
    def ols(self) -> FVector3: ...
    @property
    def lss(self) -> FVector3: ...
    @property
    def lso(self) -> FVector3: ...
    @property
    def lsl(self) -> FVector3: ...
    @property
    def los(self) -> FVector3: ...
    @property
    def lls(self) -> FVector3: ...
    @property
    def ssss(self) -> FVector4: ...
    @property
    def ssso(self) -> FVector4: ...
    @property
    def sssl(self) -> FVector4: ...
    @property
    def ssos(self) -> FVector4: ...
    @property
    def ssoo(self) -> FVector4: ...
    @property
    def ssol(self) -> FVector4: ...
    @property
    def ssls(self) -> FVector4: ...
    @property
    def sslo(self) -> FVector4: ...
    @property
    def ssll(self) -> FVector4: ...
    @property
    def soss(self) -> FVector4: ...
    @property
    def soso(self) -> FVector4: ...
    @property
    def sosl(self) -> FVector4: ...
    @property
    def soos(self) -> FVector4: ...
    @property
    def sooo(self) -> FVector4: ...
    @property
    def sool(self) -> FVector4: ...
    @property
    def sols(self) -> FVector4: ...
    @property
    def solo(self) -> FVector4: ...
    @property
    def soll(self) -> FVector4: ...
    @property
    def slss(self) -> FVector4: ...
    @property
    def slso(self) -> FVector4: ...
    @property
    def slsl(self) -> FVector4: ...
    @property
    def slos(self) -> FVector4: ...
    @property
    def sloo(self) -> FVector4: ...
    @property
    def slol(self) -> FVector4: ...
    @property
    def slls(self) -> FVector4: ...
    @property
    def sllo(self) -> FVector4: ...
    @property
    def slll(self) -> FVector4: ...
    @property
    def osss(self) -> FVector4: ...
    @property
    def osso(self) -> FVector4: ...
    @property
    def ossl(self) -> FVector4: ...
    @property
    def osos(self) -> FVector4: ...
    @property
    def osoo(self) -> FVector4: ...
    @property
    def osol(self) -> FVector4: ...
    @property
    def osls(self) -> FVector4: ...
    @property
    def oslo(self) -> FVector4: ...
    @property
    def osll(self) -> FVector4: ...
    @property
    def ooss(self) -> FVector4: ...
    @property
    def ooso(self) -> FVector4: ...
    @property
    def oosl(self) -> FVector4: ...
    @property
    def ooos(self) -> FVector4: ...
    @property
    def ools(self) -> FVector4: ...
    @property
    def olss(self) -> FVector4: ...
    @property
    def olso(self) -> FVector4: ...
    @property
    def olsl(self) -> FVector4: ...
    @property
    def olos(self) -> FVector4: ...
    @property
    def olls(self) -> FVector4: ...
    @property
    def lsss(self) -> FVector4: ...
    @property
    def lsso(self) -> FVector4: ...
    @property
    def lssl(self) -> FVector4: ...
    @property
    def lsos(self) -> FVector4: ...
    @property
    def lsoo(self) -> FVector4: ...
    @property
    def lsol(self) -> FVector4: ...
    @property
    def lsls(self) -> FVector4: ...
    @property
    def lslo(self) -> FVector4: ...
    @property
    def lsll(self) -> FVector4: ...
    @property
    def loss(self) -> FVector4: ...
    @property
    def loso(self) -> FVector4: ...
    @property
    def losl(self) -> FVector4: ...
    @property
    def loos(self) -> FVector4: ...
    @property
    def lols(self) -> FVector4: ...
    @property
    def llss(self) -> FVector4: ...
    @property
    def llso(self) -> FVector4: ...
    @property
    def llsl(self) -> FVector4: ...
    @property
    def llos(self) -> FVector4: ...
    @property
    def llls(self) -> FVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def uu(self) -> FVector2: ...
    @property
    def uo(self) -> FVector2: ...
    @property
    def ul(self) -> FVector2: ...
    @property
    def ou(self) -> FVector2: ...
    @property
    def lu(self) -> FVector2: ...
    @property
    def uuu(self) -> FVector3: ...
    @property
    def uuo(self) -> FVector3: ...
    @property
    def uul(self) -> FVector3: ...
    @property
    def uou(self) -> FVector3: ...
    @property
    def uoo(self) -> FVector3: ...
    @property
    def uol(self) -> FVector3: ...
    @property
    def ulu(self) -> FVector3: ...
    @property
    def ulo(self) -> FVector3: ...
    @property
    def ull(self) -> FVector3: ...
    @property
    def ouu(self) -> FVector3: ...
    @property
    def ouo(self) -> FVector3: ...
    @property
    def oul(self) -> FVector3: ...
    @property
    def oou(self) -> FVector3: ...
    @property
    def olu(self) -> FVector3: ...
    @property
    def luu(self) -> FVector3: ...
    @property
    def luo(self) -> FVector3: ...
    @property
    def lul(self) -> FVector3: ...
    @property
    def lou(self) -> FVector3: ...
    @property
    def llu(self) -> FVector3: ...
    @property
    def uuuu(self) -> FVector4: ...
    @property
    def uuuo(self) -> FVector4: ...
    @property
    def uuul(self) -> FVector4: ...
    @property
    def uuou(self) -> FVector4: ...
    @property
    def uuoo(self) -> FVector4: ...
    @property
    def uuol(self) -> FVector4: ...
    @property
    def uulu(self) -> FVector4: ...
    @property
    def uulo(self) -> FVector4: ...
    @property
    def uull(self) -> FVector4: ...
    @property
    def uouu(self) -> FVector4: ...
    @property
    def uouo(self) -> FVector4: ...
    @property
    def uoul(self) -> FVector4: ...
    @property
    def uoou(self) -> FVector4: ...
    @property
    def uooo(self) -> FVector4: ...
    @property
    def uool(self) -> FVector4: ...
    @property
    def uolu(self) -> FVector4: ...
    @property
    def uolo(self) -> FVector4: ...
    @property
    def uoll(self) -> FVector4: ...
    @property
    def uluu(self) -> FVector4: ...
    @property
    def uluo(self) -> FVector4: ...
    @property
    def ulul(self) -> FVector4: ...
    @property
    def ulou(self) -> FVector4: ...
    @property
    def uloo(self) -> FVector4: ...
    @property
    def ulol(self) -> FVector4: ...
    @property
    def ullu(self) -> FVector4: ...
    @property
    def ullo(self) -> FVector4: ...
    @property
    def ulll(self) -> FVector4: ...
    @property
    def ouuu(self) -> FVector4: ...
    @property
    def ouuo(self) -> FVector4: ...
    @property
    def ouul(self) -> FVector4: ...
    @property
    def ouou(self) -> FVector4: ...
    @property
    def ouoo(self) -> FVector4: ...
    @property
    def ouol(self) -> FVector4: ...
    @property
    def oulu(self) -> FVector4: ...
    @property
    def oulo(self) -> FVector4: ...
    @property
    def oull(self) -> FVector4: ...
    @property
    def oouu(self) -> FVector4: ...
    @property
    def oouo(self) -> FVector4: ...
    @property
    def ooul(self) -> FVector4: ...
    @property
    def ooou(self) -> FVector4: ...
    @property
    def oolu(self) -> FVector4: ...
    @property
    def oluu(self) -> FVector4: ...
    @property
    def oluo(self) -> FVector4: ...
    @property
    def olul(self) -> FVector4: ...
    @property
    def olou(self) -> FVector4: ...
    @property
    def ollu(self) -> FVector4: ...
    @property
    def luuu(self) -> FVector4: ...
    @property
    def luuo(self) -> FVector4: ...
    @property
    def luul(self) -> FVector4: ...
    @property
    def luou(self) -> FVector4: ...
    @property
    def luoo(self) -> FVector4: ...
    @property
    def luol(self) -> FVector4: ...
    @property
    def lulu(self) -> FVector4: ...
    @property
    def lulo(self) -> FVector4: ...
    @property
    def lull(self) -> FVector4: ...
    @property
    def louu(self) -> FVector4: ...
    @property
    def louo(self) -> FVector4: ...
    @property
    def loul(self) -> FVector4: ...
    @property
    def loou(self) -> FVector4: ...
    @property
    def lolu(self) -> FVector4: ...
    @property
    def lluu(self) -> FVector4: ...
    @property
    def lluo(self) -> FVector4: ...
    @property
    def llul(self) -> FVector4: ...
    @property
    def llou(self) -> FVector4: ...
    @property
    def lllu(self) -> FVector4: ...
    @property
    def oo(self) -> FVector2: ...
    @property
    def ol(self) -> FVector2: ...
    @property
    def lo(self) -> FVector2: ...
    @property
    def ll(self) -> FVector2: ...
    @property
    def ooo(self) -> FVector3: ...
    @property
    def ool(self) -> FVector3: ...
    @property
    def olo(self) -> FVector3: ...
    @property
    def oll(self) -> FVector3: ...
    @property
    def loo(self) -> FVector3: ...
    @property
    def lol(self) -> FVector3: ...
    @property
    def llo(self) -> FVector3: ...
    @property
    def lll(self) -> FVector3: ...
    @property
    def oooo(self) -> FVector4: ...
    @property
    def oool(self) -> FVector4: ...
    @property
    def oolo(self) -> FVector4: ...
    @property
    def ooll(self) -> FVector4: ...
    @property
    def oloo(self) -> FVector4: ...
    @property
    def olol(self) -> FVector4: ...
    @property
    def ollo(self) -> FVector4: ...
    @property
    def olll(self) -> FVector4: ...
    @property
    def looo(self) -> FVector4: ...
    @property
    def lool(self) -> FVector4: ...
    @property
    def lolo(self) -> FVector4: ...
    @property
    def loll(self) -> FVector4: ...
    @property
    def lloo(self) -> FVector4: ...
    @property
    def llol(self) -> FVector4: ...
    @property
    def lllo(self) -> FVector4: ...
    @property
    def llll(self) -> FVector4: ...
    @property
    def magnitude(self) -> float: ...
    def normalize(self) -> FVector1: ...
    def distance(self, other: FVector1, /) -> float: ...
    def lerp(self, other: FVector1, x: float, /) -> FVector1: ...
    def min(self, n: Number, /) -> FVector1: ...
    def max(self, n: Number, /) -> FVector1: ...
    def clamp(self, min: Number, max: Number, /) -> FVector1: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[FVector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector1: ...

@final
class FVector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: FVector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector1]: ...
    @overload
    def __getitem__(self, index: int) -> FVector1: ...
    @overload
    def __getitem__(self, index: slice) -> FVector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[FVector1]: ...

@final
class I8Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __add__(self, other: Number) -> I8Vector1: ...
    @overload
    def __radd__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __radd__(self, other: Number) -> I8Vector1: ...
    @overload
    def __sub__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __sub__(self, other: Number) -> I8Vector1: ...
    @overload
    def __rsub__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> I8Vector1: ...
    @overload
    def __mul__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __mul__(self, other: Number) -> I8Vector1: ...
    @overload
    def __rmul__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> I8Vector1: ...
    @overload
    def __truediv__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> I8Vector1: ...
    @overload
    def __rtruediv__(self, other: I8Vector1) -> I8Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> I8Vector1: ...
    def __neg__(self) -> I8Vector1: ...
    def __abs__(self) -> I8Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> I8Vector2: ...
    @property
    def xo(self) -> I8Vector2: ...
    @property
    def xl(self) -> I8Vector2: ...
    @property
    def ox(self) -> I8Vector2: ...
    @property
    def lx(self) -> I8Vector2: ...
    @property
    def xxx(self) -> I8Vector3: ...
    @property
    def xxo(self) -> I8Vector3: ...
    @property
    def xxl(self) -> I8Vector3: ...
    @property
    def xox(self) -> I8Vector3: ...
    @property
    def xoo(self) -> I8Vector3: ...
    @property
    def xol(self) -> I8Vector3: ...
    @property
    def xlx(self) -> I8Vector3: ...
    @property
    def xlo(self) -> I8Vector3: ...
    @property
    def xll(self) -> I8Vector3: ...
    @property
    def oxx(self) -> I8Vector3: ...
    @property
    def oxo(self) -> I8Vector3: ...
    @property
    def oxl(self) -> I8Vector3: ...
    @property
    def oox(self) -> I8Vector3: ...
    @property
    def olx(self) -> I8Vector3: ...
    @property
    def lxx(self) -> I8Vector3: ...
    @property
    def lxo(self) -> I8Vector3: ...
    @property
    def lxl(self) -> I8Vector3: ...
    @property
    def lox(self) -> I8Vector3: ...
    @property
    def llx(self) -> I8Vector3: ...
    @property
    def xxxx(self) -> I8Vector4: ...
    @property
    def xxxo(self) -> I8Vector4: ...
    @property
    def xxxl(self) -> I8Vector4: ...
    @property
    def xxox(self) -> I8Vector4: ...
    @property
    def xxoo(self) -> I8Vector4: ...
    @property
    def xxol(self) -> I8Vector4: ...
    @property
    def xxlx(self) -> I8Vector4: ...
    @property
    def xxlo(self) -> I8Vector4: ...
    @property
    def xxll(self) -> I8Vector4: ...
    @property
    def xoxx(self) -> I8Vector4: ...
    @property
    def xoxo(self) -> I8Vector4: ...
    @property
    def xoxl(self) -> I8Vector4: ...
    @property
    def xoox(self) -> I8Vector4: ...
    @property
    def xooo(self) -> I8Vector4: ...
    @property
    def xool(self) -> I8Vector4: ...
    @property
    def xolx(self) -> I8Vector4: ...
    @property
    def xolo(self) -> I8Vector4: ...
    @property
    def xoll(self) -> I8Vector4: ...
    @property
    def xlxx(self) -> I8Vector4: ...
    @property
    def xlxo(self) -> I8Vector4: ...
    @property
    def xlxl(self) -> I8Vector4: ...
    @property
    def xlox(self) -> I8Vector4: ...
    @property
    def xloo(self) -> I8Vector4: ...
    @property
    def xlol(self) -> I8Vector4: ...
    @property
    def xllx(self) -> I8Vector4: ...
    @property
    def xllo(self) -> I8Vector4: ...
    @property
    def xlll(self) -> I8Vector4: ...
    @property
    def oxxx(self) -> I8Vector4: ...
    @property
    def oxxo(self) -> I8Vector4: ...
    @property
    def oxxl(self) -> I8Vector4: ...
    @property
    def oxox(self) -> I8Vector4: ...
    @property
    def oxoo(self) -> I8Vector4: ...
    @property
    def oxol(self) -> I8Vector4: ...
    @property
    def oxlx(self) -> I8Vector4: ...
    @property
    def oxlo(self) -> I8Vector4: ...
    @property
    def oxll(self) -> I8Vector4: ...
    @property
    def ooxx(self) -> I8Vector4: ...
    @property
    def ooxo(self) -> I8Vector4: ...
    @property
    def ooxl(self) -> I8Vector4: ...
    @property
    def ooox(self) -> I8Vector4: ...
    @property
    def oolx(self) -> I8Vector4: ...
    @property
    def olxx(self) -> I8Vector4: ...
    @property
    def olxo(self) -> I8Vector4: ...
    @property
    def olxl(self) -> I8Vector4: ...
    @property
    def olox(self) -> I8Vector4: ...
    @property
    def ollx(self) -> I8Vector4: ...
    @property
    def lxxx(self) -> I8Vector4: ...
    @property
    def lxxo(self) -> I8Vector4: ...
    @property
    def lxxl(self) -> I8Vector4: ...
    @property
    def lxox(self) -> I8Vector4: ...
    @property
    def lxoo(self) -> I8Vector4: ...
    @property
    def lxol(self) -> I8Vector4: ...
    @property
    def lxlx(self) -> I8Vector4: ...
    @property
    def lxlo(self) -> I8Vector4: ...
    @property
    def lxll(self) -> I8Vector4: ...
    @property
    def loxx(self) -> I8Vector4: ...
    @property
    def loxo(self) -> I8Vector4: ...
    @property
    def loxl(self) -> I8Vector4: ...
    @property
    def loox(self) -> I8Vector4: ...
    @property
    def lolx(self) -> I8Vector4: ...
    @property
    def llxx(self) -> I8Vector4: ...
    @property
    def llxo(self) -> I8Vector4: ...
    @property
    def llxl(self) -> I8Vector4: ...
    @property
    def llox(self) -> I8Vector4: ...
    @property
    def lllx(self) -> I8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> I8Vector2: ...
    @property
    def ro(self) -> I8Vector2: ...
    @property
    def rl(self) -> I8Vector2: ...
    @property
    def lr(self) -> I8Vector2: ...
    @property
    def rrr(self) -> I8Vector3: ...
    @property
    def rro(self) -> I8Vector3: ...
    @property
    def rrl(self) -> I8Vector3: ...
    @property
    def ror(self) -> I8Vector3: ...
    @property
    def roo(self) -> I8Vector3: ...
    @property
    def rol(self) -> I8Vector3: ...
    @property
    def rlr(self) -> I8Vector3: ...
    @property
    def rlo(self) -> I8Vector3: ...
    @property
    def rll(self) -> I8Vector3: ...
    @property
    def orr(self) -> I8Vector3: ...
    @property
    def oro(self) -> I8Vector3: ...
    @property
    def orl(self) -> I8Vector3: ...
    @property
    def oor(self) -> I8Vector3: ...
    @property
    def olr(self) -> I8Vector3: ...
    @property
    def lrr(self) -> I8Vector3: ...
    @property
    def lro(self) -> I8Vector3: ...
    @property
    def lrl(self) -> I8Vector3: ...
    @property
    def lor(self) -> I8Vector3: ...
    @property
    def llr(self) -> I8Vector3: ...
    @property
    def rrrr(self) -> I8Vector4: ...
    @property
    def rrro(self) -> I8Vector4: ...
    @property
    def rrrl(self) -> I8Vector4: ...
    @property
    def rror(self) -> I8Vector4: ...
    @property
    def rroo(self) -> I8Vector4: ...
    @property
    def rrol(self) -> I8Vector4: ...
    @property
    def rrlr(self) -> I8Vector4: ...
    @property
    def rrlo(self) -> I8Vector4: ...
    @property
    def rrll(self) -> I8Vector4: ...
    @property
    def rorr(self) -> I8Vector4: ...
    @property
    def roro(self) -> I8Vector4: ...
    @property
    def rorl(self) -> I8Vector4: ...
    @property
    def roor(self) -> I8Vector4: ...
    @property
    def rooo(self) -> I8Vector4: ...
    @property
    def rool(self) -> I8Vector4: ...
    @property
    def rolr(self) -> I8Vector4: ...
    @property
    def rolo(self) -> I8Vector4: ...
    @property
    def roll(self) -> I8Vector4: ...
    @property
    def rlrr(self) -> I8Vector4: ...
    @property
    def rlro(self) -> I8Vector4: ...
    @property
    def rlrl(self) -> I8Vector4: ...
    @property
    def rlor(self) -> I8Vector4: ...
    @property
    def rloo(self) -> I8Vector4: ...
    @property
    def rlol(self) -> I8Vector4: ...
    @property
    def rllr(self) -> I8Vector4: ...
    @property
    def rllo(self) -> I8Vector4: ...
    @property
    def rlll(self) -> I8Vector4: ...
    @property
    def orrr(self) -> I8Vector4: ...
    @property
    def orro(self) -> I8Vector4: ...
    @property
    def orrl(self) -> I8Vector4: ...
    @property
    def oror(self) -> I8Vector4: ...
    @property
    def oroo(self) -> I8Vector4: ...
    @property
    def orol(self) -> I8Vector4: ...
    @property
    def orlr(self) -> I8Vector4: ...
    @property
    def orlo(self) -> I8Vector4: ...
    @property
    def orll(self) -> I8Vector4: ...
    @property
    def oorr(self) -> I8Vector4: ...
    @property
    def ooro(self) -> I8Vector4: ...
    @property
    def oorl(self) -> I8Vector4: ...
    @property
    def ooor(self) -> I8Vector4: ...
    @property
    def oolr(self) -> I8Vector4: ...
    @property
    def olrr(self) -> I8Vector4: ...
    @property
    def olro(self) -> I8Vector4: ...
    @property
    def olrl(self) -> I8Vector4: ...
    @property
    def olor(self) -> I8Vector4: ...
    @property
    def ollr(self) -> I8Vector4: ...
    @property
    def lrrr(self) -> I8Vector4: ...
    @property
    def lrro(self) -> I8Vector4: ...
    @property
    def lrrl(self) -> I8Vector4: ...
    @property
    def lror(self) -> I8Vector4: ...
    @property
    def lroo(self) -> I8Vector4: ...
    @property
    def lrol(self) -> I8Vector4: ...
    @property
    def lrlr(self) -> I8Vector4: ...
    @property
    def lrlo(self) -> I8Vector4: ...
    @property
    def lrll(self) -> I8Vector4: ...
    @property
    def lorr(self) -> I8Vector4: ...
    @property
    def loro(self) -> I8Vector4: ...
    @property
    def lorl(self) -> I8Vector4: ...
    @property
    def loor(self) -> I8Vector4: ...
    @property
    def lolr(self) -> I8Vector4: ...
    @property
    def llrr(self) -> I8Vector4: ...
    @property
    def llro(self) -> I8Vector4: ...
    @property
    def llrl(self) -> I8Vector4: ...
    @property
    def llor(self) -> I8Vector4: ...
    @property
    def lllr(self) -> I8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> I8Vector2: ...
    @property
    def so(self) -> I8Vector2: ...
    @property
    def sl(self) -> I8Vector2: ...
    @property
    def os(self) -> I8Vector2: ...
    @property
    def ls(self) -> I8Vector2: ...
    @property
    def sss(self) -> I8Vector3: ...
    @property
    def sso(self) -> I8Vector3: ...
    @property
    def ssl(self) -> I8Vector3: ...
    @property
    def sos(self) -> I8Vector3: ...
    @property
    def soo(self) -> I8Vector3: ...
    @property
    def sol(self) -> I8Vector3: ...
    @property
    def sls(self) -> I8Vector3: ...
    @property
    def slo(self) -> I8Vector3: ...
    @property
    def sll(self) -> I8Vector3: ...
    @property
    def oss(self) -> I8Vector3: ...
    @property
    def oso(self) -> I8Vector3: ...
    @property
    def osl(self) -> I8Vector3: ...
    @property
    def oos(self) -> I8Vector3: ...
    @property
    def ols(self) -> I8Vector3: ...
    @property
    def lss(self) -> I8Vector3: ...
    @property
    def lso(self) -> I8Vector3: ...
    @property
    def lsl(self) -> I8Vector3: ...
    @property
    def los(self) -> I8Vector3: ...
    @property
    def lls(self) -> I8Vector3: ...
    @property
    def ssss(self) -> I8Vector4: ...
    @property
    def ssso(self) -> I8Vector4: ...
    @property
    def sssl(self) -> I8Vector4: ...
    @property
    def ssos(self) -> I8Vector4: ...
    @property
    def ssoo(self) -> I8Vector4: ...
    @property
    def ssol(self) -> I8Vector4: ...
    @property
    def ssls(self) -> I8Vector4: ...
    @property
    def sslo(self) -> I8Vector4: ...
    @property
    def ssll(self) -> I8Vector4: ...
    @property
    def soss(self) -> I8Vector4: ...
    @property
    def soso(self) -> I8Vector4: ...
    @property
    def sosl(self) -> I8Vector4: ...
    @property
    def soos(self) -> I8Vector4: ...
    @property
    def sooo(self) -> I8Vector4: ...
    @property
    def sool(self) -> I8Vector4: ...
    @property
    def sols(self) -> I8Vector4: ...
    @property
    def solo(self) -> I8Vector4: ...
    @property
    def soll(self) -> I8Vector4: ...
    @property
    def slss(self) -> I8Vector4: ...
    @property
    def slso(self) -> I8Vector4: ...
    @property
    def slsl(self) -> I8Vector4: ...
    @property
    def slos(self) -> I8Vector4: ...
    @property
    def sloo(self) -> I8Vector4: ...
    @property
    def slol(self) -> I8Vector4: ...
    @property
    def slls(self) -> I8Vector4: ...
    @property
    def sllo(self) -> I8Vector4: ...
    @property
    def slll(self) -> I8Vector4: ...
    @property
    def osss(self) -> I8Vector4: ...
    @property
    def osso(self) -> I8Vector4: ...
    @property
    def ossl(self) -> I8Vector4: ...
    @property
    def osos(self) -> I8Vector4: ...
    @property
    def osoo(self) -> I8Vector4: ...
    @property
    def osol(self) -> I8Vector4: ...
    @property
    def osls(self) -> I8Vector4: ...
    @property
    def oslo(self) -> I8Vector4: ...
    @property
    def osll(self) -> I8Vector4: ...
    @property
    def ooss(self) -> I8Vector4: ...
    @property
    def ooso(self) -> I8Vector4: ...
    @property
    def oosl(self) -> I8Vector4: ...
    @property
    def ooos(self) -> I8Vector4: ...
    @property
    def ools(self) -> I8Vector4: ...
    @property
    def olss(self) -> I8Vector4: ...
    @property
    def olso(self) -> I8Vector4: ...
    @property
    def olsl(self) -> I8Vector4: ...
    @property
    def olos(self) -> I8Vector4: ...
    @property
    def olls(self) -> I8Vector4: ...
    @property
    def lsss(self) -> I8Vector4: ...
    @property
    def lsso(self) -> I8Vector4: ...
    @property
    def lssl(self) -> I8Vector4: ...
    @property
    def lsos(self) -> I8Vector4: ...
    @property
    def lsoo(self) -> I8Vector4: ...
    @property
    def lsol(self) -> I8Vector4: ...
    @property
    def lsls(self) -> I8Vector4: ...
    @property
    def lslo(self) -> I8Vector4: ...
    @property
    def lsll(self) -> I8Vector4: ...
    @property
    def loss(self) -> I8Vector4: ...
    @property
    def loso(self) -> I8Vector4: ...
    @property
    def losl(self) -> I8Vector4: ...
    @property
    def loos(self) -> I8Vector4: ...
    @property
    def lols(self) -> I8Vector4: ...
    @property
    def llss(self) -> I8Vector4: ...
    @property
    def llso(self) -> I8Vector4: ...
    @property
    def llsl(self) -> I8Vector4: ...
    @property
    def llos(self) -> I8Vector4: ...
    @property
    def llls(self) -> I8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> I8Vector2: ...
    @property
    def uo(self) -> I8Vector2: ...
    @property
    def ul(self) -> I8Vector2: ...
    @property
    def ou(self) -> I8Vector2: ...
    @property
    def lu(self) -> I8Vector2: ...
    @property
    def uuu(self) -> I8Vector3: ...
    @property
    def uuo(self) -> I8Vector3: ...
    @property
    def uul(self) -> I8Vector3: ...
    @property
    def uou(self) -> I8Vector3: ...
    @property
    def uoo(self) -> I8Vector3: ...
    @property
    def uol(self) -> I8Vector3: ...
    @property
    def ulu(self) -> I8Vector3: ...
    @property
    def ulo(self) -> I8Vector3: ...
    @property
    def ull(self) -> I8Vector3: ...
    @property
    def ouu(self) -> I8Vector3: ...
    @property
    def ouo(self) -> I8Vector3: ...
    @property
    def oul(self) -> I8Vector3: ...
    @property
    def oou(self) -> I8Vector3: ...
    @property
    def olu(self) -> I8Vector3: ...
    @property
    def luu(self) -> I8Vector3: ...
    @property
    def luo(self) -> I8Vector3: ...
    @property
    def lul(self) -> I8Vector3: ...
    @property
    def lou(self) -> I8Vector3: ...
    @property
    def llu(self) -> I8Vector3: ...
    @property
    def uuuu(self) -> I8Vector4: ...
    @property
    def uuuo(self) -> I8Vector4: ...
    @property
    def uuul(self) -> I8Vector4: ...
    @property
    def uuou(self) -> I8Vector4: ...
    @property
    def uuoo(self) -> I8Vector4: ...
    @property
    def uuol(self) -> I8Vector4: ...
    @property
    def uulu(self) -> I8Vector4: ...
    @property
    def uulo(self) -> I8Vector4: ...
    @property
    def uull(self) -> I8Vector4: ...
    @property
    def uouu(self) -> I8Vector4: ...
    @property
    def uouo(self) -> I8Vector4: ...
    @property
    def uoul(self) -> I8Vector4: ...
    @property
    def uoou(self) -> I8Vector4: ...
    @property
    def uooo(self) -> I8Vector4: ...
    @property
    def uool(self) -> I8Vector4: ...
    @property
    def uolu(self) -> I8Vector4: ...
    @property
    def uolo(self) -> I8Vector4: ...
    @property
    def uoll(self) -> I8Vector4: ...
    @property
    def uluu(self) -> I8Vector4: ...
    @property
    def uluo(self) -> I8Vector4: ...
    @property
    def ulul(self) -> I8Vector4: ...
    @property
    def ulou(self) -> I8Vector4: ...
    @property
    def uloo(self) -> I8Vector4: ...
    @property
    def ulol(self) -> I8Vector4: ...
    @property
    def ullu(self) -> I8Vector4: ...
    @property
    def ullo(self) -> I8Vector4: ...
    @property
    def ulll(self) -> I8Vector4: ...
    @property
    def ouuu(self) -> I8Vector4: ...
    @property
    def ouuo(self) -> I8Vector4: ...
    @property
    def ouul(self) -> I8Vector4: ...
    @property
    def ouou(self) -> I8Vector4: ...
    @property
    def ouoo(self) -> I8Vector4: ...
    @property
    def ouol(self) -> I8Vector4: ...
    @property
    def oulu(self) -> I8Vector4: ...
    @property
    def oulo(self) -> I8Vector4: ...
    @property
    def oull(self) -> I8Vector4: ...
    @property
    def oouu(self) -> I8Vector4: ...
    @property
    def oouo(self) -> I8Vector4: ...
    @property
    def ooul(self) -> I8Vector4: ...
    @property
    def ooou(self) -> I8Vector4: ...
    @property
    def oolu(self) -> I8Vector4: ...
    @property
    def oluu(self) -> I8Vector4: ...
    @property
    def oluo(self) -> I8Vector4: ...
    @property
    def olul(self) -> I8Vector4: ...
    @property
    def olou(self) -> I8Vector4: ...
    @property
    def ollu(self) -> I8Vector4: ...
    @property
    def luuu(self) -> I8Vector4: ...
    @property
    def luuo(self) -> I8Vector4: ...
    @property
    def luul(self) -> I8Vector4: ...
    @property
    def luou(self) -> I8Vector4: ...
    @property
    def luoo(self) -> I8Vector4: ...
    @property
    def luol(self) -> I8Vector4: ...
    @property
    def lulu(self) -> I8Vector4: ...
    @property
    def lulo(self) -> I8Vector4: ...
    @property
    def lull(self) -> I8Vector4: ...
    @property
    def louu(self) -> I8Vector4: ...
    @property
    def louo(self) -> I8Vector4: ...
    @property
    def loul(self) -> I8Vector4: ...
    @property
    def loou(self) -> I8Vector4: ...
    @property
    def lolu(self) -> I8Vector4: ...
    @property
    def lluu(self) -> I8Vector4: ...
    @property
    def lluo(self) -> I8Vector4: ...
    @property
    def llul(self) -> I8Vector4: ...
    @property
    def llou(self) -> I8Vector4: ...
    @property
    def lllu(self) -> I8Vector4: ...
    @property
    def oo(self) -> I8Vector2: ...
    @property
    def ol(self) -> I8Vector2: ...
    @property
    def lo(self) -> I8Vector2: ...
    @property
    def ll(self) -> I8Vector2: ...
    @property
    def ooo(self) -> I8Vector3: ...
    @property
    def ool(self) -> I8Vector3: ...
    @property
    def olo(self) -> I8Vector3: ...
    @property
    def oll(self) -> I8Vector3: ...
    @property
    def loo(self) -> I8Vector3: ...
    @property
    def lol(self) -> I8Vector3: ...
    @property
    def llo(self) -> I8Vector3: ...
    @property
    def lll(self) -> I8Vector3: ...
    @property
    def oooo(self) -> I8Vector4: ...
    @property
    def oool(self) -> I8Vector4: ...
    @property
    def oolo(self) -> I8Vector4: ...
    @property
    def ooll(self) -> I8Vector4: ...
    @property
    def oloo(self) -> I8Vector4: ...
    @property
    def olol(self) -> I8Vector4: ...
    @property
    def ollo(self) -> I8Vector4: ...
    @property
    def olll(self) -> I8Vector4: ...
    @property
    def looo(self) -> I8Vector4: ...
    @property
    def lool(self) -> I8Vector4: ...
    @property
    def lolo(self) -> I8Vector4: ...
    @property
    def loll(self) -> I8Vector4: ...
    @property
    def lloo(self) -> I8Vector4: ...
    @property
    def llol(self) -> I8Vector4: ...
    @property
    def lllo(self) -> I8Vector4: ...
    @property
    def llll(self) -> I8Vector4: ...
    def min(self, n: Number, /) -> I8Vector1: ...
    def max(self, n: Number, /) -> I8Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> I8Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I8Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector1: ...

@final
class I8Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I8Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I8Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> I8Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> I8Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[I8Vector1]: ...

@final
class U8Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __add__(self, other: Number) -> U8Vector1: ...
    @overload
    def __radd__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __radd__(self, other: Number) -> U8Vector1: ...
    @overload
    def __sub__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __sub__(self, other: Number) -> U8Vector1: ...
    @overload
    def __rsub__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> U8Vector1: ...
    @overload
    def __mul__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __mul__(self, other: Number) -> U8Vector1: ...
    @overload
    def __rmul__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> U8Vector1: ...
    @overload
    def __truediv__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> U8Vector1: ...
    @overload
    def __rtruediv__(self, other: U8Vector1) -> U8Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> U8Vector1: ...
    def __abs__(self) -> U8Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> U8Vector2: ...
    @property
    def xo(self) -> U8Vector2: ...
    @property
    def xl(self) -> U8Vector2: ...
    @property
    def ox(self) -> U8Vector2: ...
    @property
    def lx(self) -> U8Vector2: ...
    @property
    def xxx(self) -> U8Vector3: ...
    @property
    def xxo(self) -> U8Vector3: ...
    @property
    def xxl(self) -> U8Vector3: ...
    @property
    def xox(self) -> U8Vector3: ...
    @property
    def xoo(self) -> U8Vector3: ...
    @property
    def xol(self) -> U8Vector3: ...
    @property
    def xlx(self) -> U8Vector3: ...
    @property
    def xlo(self) -> U8Vector3: ...
    @property
    def xll(self) -> U8Vector3: ...
    @property
    def oxx(self) -> U8Vector3: ...
    @property
    def oxo(self) -> U8Vector3: ...
    @property
    def oxl(self) -> U8Vector3: ...
    @property
    def oox(self) -> U8Vector3: ...
    @property
    def olx(self) -> U8Vector3: ...
    @property
    def lxx(self) -> U8Vector3: ...
    @property
    def lxo(self) -> U8Vector3: ...
    @property
    def lxl(self) -> U8Vector3: ...
    @property
    def lox(self) -> U8Vector3: ...
    @property
    def llx(self) -> U8Vector3: ...
    @property
    def xxxx(self) -> U8Vector4: ...
    @property
    def xxxo(self) -> U8Vector4: ...
    @property
    def xxxl(self) -> U8Vector4: ...
    @property
    def xxox(self) -> U8Vector4: ...
    @property
    def xxoo(self) -> U8Vector4: ...
    @property
    def xxol(self) -> U8Vector4: ...
    @property
    def xxlx(self) -> U8Vector4: ...
    @property
    def xxlo(self) -> U8Vector4: ...
    @property
    def xxll(self) -> U8Vector4: ...
    @property
    def xoxx(self) -> U8Vector4: ...
    @property
    def xoxo(self) -> U8Vector4: ...
    @property
    def xoxl(self) -> U8Vector4: ...
    @property
    def xoox(self) -> U8Vector4: ...
    @property
    def xooo(self) -> U8Vector4: ...
    @property
    def xool(self) -> U8Vector4: ...
    @property
    def xolx(self) -> U8Vector4: ...
    @property
    def xolo(self) -> U8Vector4: ...
    @property
    def xoll(self) -> U8Vector4: ...
    @property
    def xlxx(self) -> U8Vector4: ...
    @property
    def xlxo(self) -> U8Vector4: ...
    @property
    def xlxl(self) -> U8Vector4: ...
    @property
    def xlox(self) -> U8Vector4: ...
    @property
    def xloo(self) -> U8Vector4: ...
    @property
    def xlol(self) -> U8Vector4: ...
    @property
    def xllx(self) -> U8Vector4: ...
    @property
    def xllo(self) -> U8Vector4: ...
    @property
    def xlll(self) -> U8Vector4: ...
    @property
    def oxxx(self) -> U8Vector4: ...
    @property
    def oxxo(self) -> U8Vector4: ...
    @property
    def oxxl(self) -> U8Vector4: ...
    @property
    def oxox(self) -> U8Vector4: ...
    @property
    def oxoo(self) -> U8Vector4: ...
    @property
    def oxol(self) -> U8Vector4: ...
    @property
    def oxlx(self) -> U8Vector4: ...
    @property
    def oxlo(self) -> U8Vector4: ...
    @property
    def oxll(self) -> U8Vector4: ...
    @property
    def ooxx(self) -> U8Vector4: ...
    @property
    def ooxo(self) -> U8Vector4: ...
    @property
    def ooxl(self) -> U8Vector4: ...
    @property
    def ooox(self) -> U8Vector4: ...
    @property
    def oolx(self) -> U8Vector4: ...
    @property
    def olxx(self) -> U8Vector4: ...
    @property
    def olxo(self) -> U8Vector4: ...
    @property
    def olxl(self) -> U8Vector4: ...
    @property
    def olox(self) -> U8Vector4: ...
    @property
    def ollx(self) -> U8Vector4: ...
    @property
    def lxxx(self) -> U8Vector4: ...
    @property
    def lxxo(self) -> U8Vector4: ...
    @property
    def lxxl(self) -> U8Vector4: ...
    @property
    def lxox(self) -> U8Vector4: ...
    @property
    def lxoo(self) -> U8Vector4: ...
    @property
    def lxol(self) -> U8Vector4: ...
    @property
    def lxlx(self) -> U8Vector4: ...
    @property
    def lxlo(self) -> U8Vector4: ...
    @property
    def lxll(self) -> U8Vector4: ...
    @property
    def loxx(self) -> U8Vector4: ...
    @property
    def loxo(self) -> U8Vector4: ...
    @property
    def loxl(self) -> U8Vector4: ...
    @property
    def loox(self) -> U8Vector4: ...
    @property
    def lolx(self) -> U8Vector4: ...
    @property
    def llxx(self) -> U8Vector4: ...
    @property
    def llxo(self) -> U8Vector4: ...
    @property
    def llxl(self) -> U8Vector4: ...
    @property
    def llox(self) -> U8Vector4: ...
    @property
    def lllx(self) -> U8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> U8Vector2: ...
    @property
    def ro(self) -> U8Vector2: ...
    @property
    def rl(self) -> U8Vector2: ...
    @property
    def lr(self) -> U8Vector2: ...
    @property
    def rrr(self) -> U8Vector3: ...
    @property
    def rro(self) -> U8Vector3: ...
    @property
    def rrl(self) -> U8Vector3: ...
    @property
    def ror(self) -> U8Vector3: ...
    @property
    def roo(self) -> U8Vector3: ...
    @property
    def rol(self) -> U8Vector3: ...
    @property
    def rlr(self) -> U8Vector3: ...
    @property
    def rlo(self) -> U8Vector3: ...
    @property
    def rll(self) -> U8Vector3: ...
    @property
    def orr(self) -> U8Vector3: ...
    @property
    def oro(self) -> U8Vector3: ...
    @property
    def orl(self) -> U8Vector3: ...
    @property
    def oor(self) -> U8Vector3: ...
    @property
    def olr(self) -> U8Vector3: ...
    @property
    def lrr(self) -> U8Vector3: ...
    @property
    def lro(self) -> U8Vector3: ...
    @property
    def lrl(self) -> U8Vector3: ...
    @property
    def lor(self) -> U8Vector3: ...
    @property
    def llr(self) -> U8Vector3: ...
    @property
    def rrrr(self) -> U8Vector4: ...
    @property
    def rrro(self) -> U8Vector4: ...
    @property
    def rrrl(self) -> U8Vector4: ...
    @property
    def rror(self) -> U8Vector4: ...
    @property
    def rroo(self) -> U8Vector4: ...
    @property
    def rrol(self) -> U8Vector4: ...
    @property
    def rrlr(self) -> U8Vector4: ...
    @property
    def rrlo(self) -> U8Vector4: ...
    @property
    def rrll(self) -> U8Vector4: ...
    @property
    def rorr(self) -> U8Vector4: ...
    @property
    def roro(self) -> U8Vector4: ...
    @property
    def rorl(self) -> U8Vector4: ...
    @property
    def roor(self) -> U8Vector4: ...
    @property
    def rooo(self) -> U8Vector4: ...
    @property
    def rool(self) -> U8Vector4: ...
    @property
    def rolr(self) -> U8Vector4: ...
    @property
    def rolo(self) -> U8Vector4: ...
    @property
    def roll(self) -> U8Vector4: ...
    @property
    def rlrr(self) -> U8Vector4: ...
    @property
    def rlro(self) -> U8Vector4: ...
    @property
    def rlrl(self) -> U8Vector4: ...
    @property
    def rlor(self) -> U8Vector4: ...
    @property
    def rloo(self) -> U8Vector4: ...
    @property
    def rlol(self) -> U8Vector4: ...
    @property
    def rllr(self) -> U8Vector4: ...
    @property
    def rllo(self) -> U8Vector4: ...
    @property
    def rlll(self) -> U8Vector4: ...
    @property
    def orrr(self) -> U8Vector4: ...
    @property
    def orro(self) -> U8Vector4: ...
    @property
    def orrl(self) -> U8Vector4: ...
    @property
    def oror(self) -> U8Vector4: ...
    @property
    def oroo(self) -> U8Vector4: ...
    @property
    def orol(self) -> U8Vector4: ...
    @property
    def orlr(self) -> U8Vector4: ...
    @property
    def orlo(self) -> U8Vector4: ...
    @property
    def orll(self) -> U8Vector4: ...
    @property
    def oorr(self) -> U8Vector4: ...
    @property
    def ooro(self) -> U8Vector4: ...
    @property
    def oorl(self) -> U8Vector4: ...
    @property
    def ooor(self) -> U8Vector4: ...
    @property
    def oolr(self) -> U8Vector4: ...
    @property
    def olrr(self) -> U8Vector4: ...
    @property
    def olro(self) -> U8Vector4: ...
    @property
    def olrl(self) -> U8Vector4: ...
    @property
    def olor(self) -> U8Vector4: ...
    @property
    def ollr(self) -> U8Vector4: ...
    @property
    def lrrr(self) -> U8Vector4: ...
    @property
    def lrro(self) -> U8Vector4: ...
    @property
    def lrrl(self) -> U8Vector4: ...
    @property
    def lror(self) -> U8Vector4: ...
    @property
    def lroo(self) -> U8Vector4: ...
    @property
    def lrol(self) -> U8Vector4: ...
    @property
    def lrlr(self) -> U8Vector4: ...
    @property
    def lrlo(self) -> U8Vector4: ...
    @property
    def lrll(self) -> U8Vector4: ...
    @property
    def lorr(self) -> U8Vector4: ...
    @property
    def loro(self) -> U8Vector4: ...
    @property
    def lorl(self) -> U8Vector4: ...
    @property
    def loor(self) -> U8Vector4: ...
    @property
    def lolr(self) -> U8Vector4: ...
    @property
    def llrr(self) -> U8Vector4: ...
    @property
    def llro(self) -> U8Vector4: ...
    @property
    def llrl(self) -> U8Vector4: ...
    @property
    def llor(self) -> U8Vector4: ...
    @property
    def lllr(self) -> U8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> U8Vector2: ...
    @property
    def so(self) -> U8Vector2: ...
    @property
    def sl(self) -> U8Vector2: ...
    @property
    def os(self) -> U8Vector2: ...
    @property
    def ls(self) -> U8Vector2: ...
    @property
    def sss(self) -> U8Vector3: ...
    @property
    def sso(self) -> U8Vector3: ...
    @property
    def ssl(self) -> U8Vector3: ...
    @property
    def sos(self) -> U8Vector3: ...
    @property
    def soo(self) -> U8Vector3: ...
    @property
    def sol(self) -> U8Vector3: ...
    @property
    def sls(self) -> U8Vector3: ...
    @property
    def slo(self) -> U8Vector3: ...
    @property
    def sll(self) -> U8Vector3: ...
    @property
    def oss(self) -> U8Vector3: ...
    @property
    def oso(self) -> U8Vector3: ...
    @property
    def osl(self) -> U8Vector3: ...
    @property
    def oos(self) -> U8Vector3: ...
    @property
    def ols(self) -> U8Vector3: ...
    @property
    def lss(self) -> U8Vector3: ...
    @property
    def lso(self) -> U8Vector3: ...
    @property
    def lsl(self) -> U8Vector3: ...
    @property
    def los(self) -> U8Vector3: ...
    @property
    def lls(self) -> U8Vector3: ...
    @property
    def ssss(self) -> U8Vector4: ...
    @property
    def ssso(self) -> U8Vector4: ...
    @property
    def sssl(self) -> U8Vector4: ...
    @property
    def ssos(self) -> U8Vector4: ...
    @property
    def ssoo(self) -> U8Vector4: ...
    @property
    def ssol(self) -> U8Vector4: ...
    @property
    def ssls(self) -> U8Vector4: ...
    @property
    def sslo(self) -> U8Vector4: ...
    @property
    def ssll(self) -> U8Vector4: ...
    @property
    def soss(self) -> U8Vector4: ...
    @property
    def soso(self) -> U8Vector4: ...
    @property
    def sosl(self) -> U8Vector4: ...
    @property
    def soos(self) -> U8Vector4: ...
    @property
    def sooo(self) -> U8Vector4: ...
    @property
    def sool(self) -> U8Vector4: ...
    @property
    def sols(self) -> U8Vector4: ...
    @property
    def solo(self) -> U8Vector4: ...
    @property
    def soll(self) -> U8Vector4: ...
    @property
    def slss(self) -> U8Vector4: ...
    @property
    def slso(self) -> U8Vector4: ...
    @property
    def slsl(self) -> U8Vector4: ...
    @property
    def slos(self) -> U8Vector4: ...
    @property
    def sloo(self) -> U8Vector4: ...
    @property
    def slol(self) -> U8Vector4: ...
    @property
    def slls(self) -> U8Vector4: ...
    @property
    def sllo(self) -> U8Vector4: ...
    @property
    def slll(self) -> U8Vector4: ...
    @property
    def osss(self) -> U8Vector4: ...
    @property
    def osso(self) -> U8Vector4: ...
    @property
    def ossl(self) -> U8Vector4: ...
    @property
    def osos(self) -> U8Vector4: ...
    @property
    def osoo(self) -> U8Vector4: ...
    @property
    def osol(self) -> U8Vector4: ...
    @property
    def osls(self) -> U8Vector4: ...
    @property
    def oslo(self) -> U8Vector4: ...
    @property
    def osll(self) -> U8Vector4: ...
    @property
    def ooss(self) -> U8Vector4: ...
    @property
    def ooso(self) -> U8Vector4: ...
    @property
    def oosl(self) -> U8Vector4: ...
    @property
    def ooos(self) -> U8Vector4: ...
    @property
    def ools(self) -> U8Vector4: ...
    @property
    def olss(self) -> U8Vector4: ...
    @property
    def olso(self) -> U8Vector4: ...
    @property
    def olsl(self) -> U8Vector4: ...
    @property
    def olos(self) -> U8Vector4: ...
    @property
    def olls(self) -> U8Vector4: ...
    @property
    def lsss(self) -> U8Vector4: ...
    @property
    def lsso(self) -> U8Vector4: ...
    @property
    def lssl(self) -> U8Vector4: ...
    @property
    def lsos(self) -> U8Vector4: ...
    @property
    def lsoo(self) -> U8Vector4: ...
    @property
    def lsol(self) -> U8Vector4: ...
    @property
    def lsls(self) -> U8Vector4: ...
    @property
    def lslo(self) -> U8Vector4: ...
    @property
    def lsll(self) -> U8Vector4: ...
    @property
    def loss(self) -> U8Vector4: ...
    @property
    def loso(self) -> U8Vector4: ...
    @property
    def losl(self) -> U8Vector4: ...
    @property
    def loos(self) -> U8Vector4: ...
    @property
    def lols(self) -> U8Vector4: ...
    @property
    def llss(self) -> U8Vector4: ...
    @property
    def llso(self) -> U8Vector4: ...
    @property
    def llsl(self) -> U8Vector4: ...
    @property
    def llos(self) -> U8Vector4: ...
    @property
    def llls(self) -> U8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> U8Vector2: ...
    @property
    def uo(self) -> U8Vector2: ...
    @property
    def ul(self) -> U8Vector2: ...
    @property
    def ou(self) -> U8Vector2: ...
    @property
    def lu(self) -> U8Vector2: ...
    @property
    def uuu(self) -> U8Vector3: ...
    @property
    def uuo(self) -> U8Vector3: ...
    @property
    def uul(self) -> U8Vector3: ...
    @property
    def uou(self) -> U8Vector3: ...
    @property
    def uoo(self) -> U8Vector3: ...
    @property
    def uol(self) -> U8Vector3: ...
    @property
    def ulu(self) -> U8Vector3: ...
    @property
    def ulo(self) -> U8Vector3: ...
    @property
    def ull(self) -> U8Vector3: ...
    @property
    def ouu(self) -> U8Vector3: ...
    @property
    def ouo(self) -> U8Vector3: ...
    @property
    def oul(self) -> U8Vector3: ...
    @property
    def oou(self) -> U8Vector3: ...
    @property
    def olu(self) -> U8Vector3: ...
    @property
    def luu(self) -> U8Vector3: ...
    @property
    def luo(self) -> U8Vector3: ...
    @property
    def lul(self) -> U8Vector3: ...
    @property
    def lou(self) -> U8Vector3: ...
    @property
    def llu(self) -> U8Vector3: ...
    @property
    def uuuu(self) -> U8Vector4: ...
    @property
    def uuuo(self) -> U8Vector4: ...
    @property
    def uuul(self) -> U8Vector4: ...
    @property
    def uuou(self) -> U8Vector4: ...
    @property
    def uuoo(self) -> U8Vector4: ...
    @property
    def uuol(self) -> U8Vector4: ...
    @property
    def uulu(self) -> U8Vector4: ...
    @property
    def uulo(self) -> U8Vector4: ...
    @property
    def uull(self) -> U8Vector4: ...
    @property
    def uouu(self) -> U8Vector4: ...
    @property
    def uouo(self) -> U8Vector4: ...
    @property
    def uoul(self) -> U8Vector4: ...
    @property
    def uoou(self) -> U8Vector4: ...
    @property
    def uooo(self) -> U8Vector4: ...
    @property
    def uool(self) -> U8Vector4: ...
    @property
    def uolu(self) -> U8Vector4: ...
    @property
    def uolo(self) -> U8Vector4: ...
    @property
    def uoll(self) -> U8Vector4: ...
    @property
    def uluu(self) -> U8Vector4: ...
    @property
    def uluo(self) -> U8Vector4: ...
    @property
    def ulul(self) -> U8Vector4: ...
    @property
    def ulou(self) -> U8Vector4: ...
    @property
    def uloo(self) -> U8Vector4: ...
    @property
    def ulol(self) -> U8Vector4: ...
    @property
    def ullu(self) -> U8Vector4: ...
    @property
    def ullo(self) -> U8Vector4: ...
    @property
    def ulll(self) -> U8Vector4: ...
    @property
    def ouuu(self) -> U8Vector4: ...
    @property
    def ouuo(self) -> U8Vector4: ...
    @property
    def ouul(self) -> U8Vector4: ...
    @property
    def ouou(self) -> U8Vector4: ...
    @property
    def ouoo(self) -> U8Vector4: ...
    @property
    def ouol(self) -> U8Vector4: ...
    @property
    def oulu(self) -> U8Vector4: ...
    @property
    def oulo(self) -> U8Vector4: ...
    @property
    def oull(self) -> U8Vector4: ...
    @property
    def oouu(self) -> U8Vector4: ...
    @property
    def oouo(self) -> U8Vector4: ...
    @property
    def ooul(self) -> U8Vector4: ...
    @property
    def ooou(self) -> U8Vector4: ...
    @property
    def oolu(self) -> U8Vector4: ...
    @property
    def oluu(self) -> U8Vector4: ...
    @property
    def oluo(self) -> U8Vector4: ...
    @property
    def olul(self) -> U8Vector4: ...
    @property
    def olou(self) -> U8Vector4: ...
    @property
    def ollu(self) -> U8Vector4: ...
    @property
    def luuu(self) -> U8Vector4: ...
    @property
    def luuo(self) -> U8Vector4: ...
    @property
    def luul(self) -> U8Vector4: ...
    @property
    def luou(self) -> U8Vector4: ...
    @property
    def luoo(self) -> U8Vector4: ...
    @property
    def luol(self) -> U8Vector4: ...
    @property
    def lulu(self) -> U8Vector4: ...
    @property
    def lulo(self) -> U8Vector4: ...
    @property
    def lull(self) -> U8Vector4: ...
    @property
    def louu(self) -> U8Vector4: ...
    @property
    def louo(self) -> U8Vector4: ...
    @property
    def loul(self) -> U8Vector4: ...
    @property
    def loou(self) -> U8Vector4: ...
    @property
    def lolu(self) -> U8Vector4: ...
    @property
    def lluu(self) -> U8Vector4: ...
    @property
    def lluo(self) -> U8Vector4: ...
    @property
    def llul(self) -> U8Vector4: ...
    @property
    def llou(self) -> U8Vector4: ...
    @property
    def lllu(self) -> U8Vector4: ...
    @property
    def oo(self) -> U8Vector2: ...
    @property
    def ol(self) -> U8Vector2: ...
    @property
    def lo(self) -> U8Vector2: ...
    @property
    def ll(self) -> U8Vector2: ...
    @property
    def ooo(self) -> U8Vector3: ...
    @property
    def ool(self) -> U8Vector3: ...
    @property
    def olo(self) -> U8Vector3: ...
    @property
    def oll(self) -> U8Vector3: ...
    @property
    def loo(self) -> U8Vector3: ...
    @property
    def lol(self) -> U8Vector3: ...
    @property
    def llo(self) -> U8Vector3: ...
    @property
    def lll(self) -> U8Vector3: ...
    @property
    def oooo(self) -> U8Vector4: ...
    @property
    def oool(self) -> U8Vector4: ...
    @property
    def oolo(self) -> U8Vector4: ...
    @property
    def ooll(self) -> U8Vector4: ...
    @property
    def oloo(self) -> U8Vector4: ...
    @property
    def olol(self) -> U8Vector4: ...
    @property
    def ollo(self) -> U8Vector4: ...
    @property
    def olll(self) -> U8Vector4: ...
    @property
    def looo(self) -> U8Vector4: ...
    @property
    def lool(self) -> U8Vector4: ...
    @property
    def lolo(self) -> U8Vector4: ...
    @property
    def loll(self) -> U8Vector4: ...
    @property
    def lloo(self) -> U8Vector4: ...
    @property
    def llol(self) -> U8Vector4: ...
    @property
    def lllo(self) -> U8Vector4: ...
    @property
    def llll(self) -> U8Vector4: ...
    def min(self, n: Number, /) -> U8Vector1: ...
    def max(self, n: Number, /) -> U8Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> U8Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U8Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector1: ...

@final
class U8Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U8Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U8Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> U8Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> U8Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[U8Vector1]: ...

@final
class I16Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __add__(self, other: Number) -> I16Vector1: ...
    @overload
    def __radd__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __radd__(self, other: Number) -> I16Vector1: ...
    @overload
    def __sub__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __sub__(self, other: Number) -> I16Vector1: ...
    @overload
    def __rsub__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> I16Vector1: ...
    @overload
    def __mul__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __mul__(self, other: Number) -> I16Vector1: ...
    @overload
    def __rmul__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> I16Vector1: ...
    @overload
    def __truediv__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> I16Vector1: ...
    @overload
    def __rtruediv__(self, other: I16Vector1) -> I16Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> I16Vector1: ...
    def __neg__(self) -> I16Vector1: ...
    def __abs__(self) -> I16Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> I16Vector2: ...
    @property
    def xo(self) -> I16Vector2: ...
    @property
    def xl(self) -> I16Vector2: ...
    @property
    def ox(self) -> I16Vector2: ...
    @property
    def lx(self) -> I16Vector2: ...
    @property
    def xxx(self) -> I16Vector3: ...
    @property
    def xxo(self) -> I16Vector3: ...
    @property
    def xxl(self) -> I16Vector3: ...
    @property
    def xox(self) -> I16Vector3: ...
    @property
    def xoo(self) -> I16Vector3: ...
    @property
    def xol(self) -> I16Vector3: ...
    @property
    def xlx(self) -> I16Vector3: ...
    @property
    def xlo(self) -> I16Vector3: ...
    @property
    def xll(self) -> I16Vector3: ...
    @property
    def oxx(self) -> I16Vector3: ...
    @property
    def oxo(self) -> I16Vector3: ...
    @property
    def oxl(self) -> I16Vector3: ...
    @property
    def oox(self) -> I16Vector3: ...
    @property
    def olx(self) -> I16Vector3: ...
    @property
    def lxx(self) -> I16Vector3: ...
    @property
    def lxo(self) -> I16Vector3: ...
    @property
    def lxl(self) -> I16Vector3: ...
    @property
    def lox(self) -> I16Vector3: ...
    @property
    def llx(self) -> I16Vector3: ...
    @property
    def xxxx(self) -> I16Vector4: ...
    @property
    def xxxo(self) -> I16Vector4: ...
    @property
    def xxxl(self) -> I16Vector4: ...
    @property
    def xxox(self) -> I16Vector4: ...
    @property
    def xxoo(self) -> I16Vector4: ...
    @property
    def xxol(self) -> I16Vector4: ...
    @property
    def xxlx(self) -> I16Vector4: ...
    @property
    def xxlo(self) -> I16Vector4: ...
    @property
    def xxll(self) -> I16Vector4: ...
    @property
    def xoxx(self) -> I16Vector4: ...
    @property
    def xoxo(self) -> I16Vector4: ...
    @property
    def xoxl(self) -> I16Vector4: ...
    @property
    def xoox(self) -> I16Vector4: ...
    @property
    def xooo(self) -> I16Vector4: ...
    @property
    def xool(self) -> I16Vector4: ...
    @property
    def xolx(self) -> I16Vector4: ...
    @property
    def xolo(self) -> I16Vector4: ...
    @property
    def xoll(self) -> I16Vector4: ...
    @property
    def xlxx(self) -> I16Vector4: ...
    @property
    def xlxo(self) -> I16Vector4: ...
    @property
    def xlxl(self) -> I16Vector4: ...
    @property
    def xlox(self) -> I16Vector4: ...
    @property
    def xloo(self) -> I16Vector4: ...
    @property
    def xlol(self) -> I16Vector4: ...
    @property
    def xllx(self) -> I16Vector4: ...
    @property
    def xllo(self) -> I16Vector4: ...
    @property
    def xlll(self) -> I16Vector4: ...
    @property
    def oxxx(self) -> I16Vector4: ...
    @property
    def oxxo(self) -> I16Vector4: ...
    @property
    def oxxl(self) -> I16Vector4: ...
    @property
    def oxox(self) -> I16Vector4: ...
    @property
    def oxoo(self) -> I16Vector4: ...
    @property
    def oxol(self) -> I16Vector4: ...
    @property
    def oxlx(self) -> I16Vector4: ...
    @property
    def oxlo(self) -> I16Vector4: ...
    @property
    def oxll(self) -> I16Vector4: ...
    @property
    def ooxx(self) -> I16Vector4: ...
    @property
    def ooxo(self) -> I16Vector4: ...
    @property
    def ooxl(self) -> I16Vector4: ...
    @property
    def ooox(self) -> I16Vector4: ...
    @property
    def oolx(self) -> I16Vector4: ...
    @property
    def olxx(self) -> I16Vector4: ...
    @property
    def olxo(self) -> I16Vector4: ...
    @property
    def olxl(self) -> I16Vector4: ...
    @property
    def olox(self) -> I16Vector4: ...
    @property
    def ollx(self) -> I16Vector4: ...
    @property
    def lxxx(self) -> I16Vector4: ...
    @property
    def lxxo(self) -> I16Vector4: ...
    @property
    def lxxl(self) -> I16Vector4: ...
    @property
    def lxox(self) -> I16Vector4: ...
    @property
    def lxoo(self) -> I16Vector4: ...
    @property
    def lxol(self) -> I16Vector4: ...
    @property
    def lxlx(self) -> I16Vector4: ...
    @property
    def lxlo(self) -> I16Vector4: ...
    @property
    def lxll(self) -> I16Vector4: ...
    @property
    def loxx(self) -> I16Vector4: ...
    @property
    def loxo(self) -> I16Vector4: ...
    @property
    def loxl(self) -> I16Vector4: ...
    @property
    def loox(self) -> I16Vector4: ...
    @property
    def lolx(self) -> I16Vector4: ...
    @property
    def llxx(self) -> I16Vector4: ...
    @property
    def llxo(self) -> I16Vector4: ...
    @property
    def llxl(self) -> I16Vector4: ...
    @property
    def llox(self) -> I16Vector4: ...
    @property
    def lllx(self) -> I16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> I16Vector2: ...
    @property
    def ro(self) -> I16Vector2: ...
    @property
    def rl(self) -> I16Vector2: ...
    @property
    def lr(self) -> I16Vector2: ...
    @property
    def rrr(self) -> I16Vector3: ...
    @property
    def rro(self) -> I16Vector3: ...
    @property
    def rrl(self) -> I16Vector3: ...
    @property
    def ror(self) -> I16Vector3: ...
    @property
    def roo(self) -> I16Vector3: ...
    @property
    def rol(self) -> I16Vector3: ...
    @property
    def rlr(self) -> I16Vector3: ...
    @property
    def rlo(self) -> I16Vector3: ...
    @property
    def rll(self) -> I16Vector3: ...
    @property
    def orr(self) -> I16Vector3: ...
    @property
    def oro(self) -> I16Vector3: ...
    @property
    def orl(self) -> I16Vector3: ...
    @property
    def oor(self) -> I16Vector3: ...
    @property
    def olr(self) -> I16Vector3: ...
    @property
    def lrr(self) -> I16Vector3: ...
    @property
    def lro(self) -> I16Vector3: ...
    @property
    def lrl(self) -> I16Vector3: ...
    @property
    def lor(self) -> I16Vector3: ...
    @property
    def llr(self) -> I16Vector3: ...
    @property
    def rrrr(self) -> I16Vector4: ...
    @property
    def rrro(self) -> I16Vector4: ...
    @property
    def rrrl(self) -> I16Vector4: ...
    @property
    def rror(self) -> I16Vector4: ...
    @property
    def rroo(self) -> I16Vector4: ...
    @property
    def rrol(self) -> I16Vector4: ...
    @property
    def rrlr(self) -> I16Vector4: ...
    @property
    def rrlo(self) -> I16Vector4: ...
    @property
    def rrll(self) -> I16Vector4: ...
    @property
    def rorr(self) -> I16Vector4: ...
    @property
    def roro(self) -> I16Vector4: ...
    @property
    def rorl(self) -> I16Vector4: ...
    @property
    def roor(self) -> I16Vector4: ...
    @property
    def rooo(self) -> I16Vector4: ...
    @property
    def rool(self) -> I16Vector4: ...
    @property
    def rolr(self) -> I16Vector4: ...
    @property
    def rolo(self) -> I16Vector4: ...
    @property
    def roll(self) -> I16Vector4: ...
    @property
    def rlrr(self) -> I16Vector4: ...
    @property
    def rlro(self) -> I16Vector4: ...
    @property
    def rlrl(self) -> I16Vector4: ...
    @property
    def rlor(self) -> I16Vector4: ...
    @property
    def rloo(self) -> I16Vector4: ...
    @property
    def rlol(self) -> I16Vector4: ...
    @property
    def rllr(self) -> I16Vector4: ...
    @property
    def rllo(self) -> I16Vector4: ...
    @property
    def rlll(self) -> I16Vector4: ...
    @property
    def orrr(self) -> I16Vector4: ...
    @property
    def orro(self) -> I16Vector4: ...
    @property
    def orrl(self) -> I16Vector4: ...
    @property
    def oror(self) -> I16Vector4: ...
    @property
    def oroo(self) -> I16Vector4: ...
    @property
    def orol(self) -> I16Vector4: ...
    @property
    def orlr(self) -> I16Vector4: ...
    @property
    def orlo(self) -> I16Vector4: ...
    @property
    def orll(self) -> I16Vector4: ...
    @property
    def oorr(self) -> I16Vector4: ...
    @property
    def ooro(self) -> I16Vector4: ...
    @property
    def oorl(self) -> I16Vector4: ...
    @property
    def ooor(self) -> I16Vector4: ...
    @property
    def oolr(self) -> I16Vector4: ...
    @property
    def olrr(self) -> I16Vector4: ...
    @property
    def olro(self) -> I16Vector4: ...
    @property
    def olrl(self) -> I16Vector4: ...
    @property
    def olor(self) -> I16Vector4: ...
    @property
    def ollr(self) -> I16Vector4: ...
    @property
    def lrrr(self) -> I16Vector4: ...
    @property
    def lrro(self) -> I16Vector4: ...
    @property
    def lrrl(self) -> I16Vector4: ...
    @property
    def lror(self) -> I16Vector4: ...
    @property
    def lroo(self) -> I16Vector4: ...
    @property
    def lrol(self) -> I16Vector4: ...
    @property
    def lrlr(self) -> I16Vector4: ...
    @property
    def lrlo(self) -> I16Vector4: ...
    @property
    def lrll(self) -> I16Vector4: ...
    @property
    def lorr(self) -> I16Vector4: ...
    @property
    def loro(self) -> I16Vector4: ...
    @property
    def lorl(self) -> I16Vector4: ...
    @property
    def loor(self) -> I16Vector4: ...
    @property
    def lolr(self) -> I16Vector4: ...
    @property
    def llrr(self) -> I16Vector4: ...
    @property
    def llro(self) -> I16Vector4: ...
    @property
    def llrl(self) -> I16Vector4: ...
    @property
    def llor(self) -> I16Vector4: ...
    @property
    def lllr(self) -> I16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> I16Vector2: ...
    @property
    def so(self) -> I16Vector2: ...
    @property
    def sl(self) -> I16Vector2: ...
    @property
    def os(self) -> I16Vector2: ...
    @property
    def ls(self) -> I16Vector2: ...
    @property
    def sss(self) -> I16Vector3: ...
    @property
    def sso(self) -> I16Vector3: ...
    @property
    def ssl(self) -> I16Vector3: ...
    @property
    def sos(self) -> I16Vector3: ...
    @property
    def soo(self) -> I16Vector3: ...
    @property
    def sol(self) -> I16Vector3: ...
    @property
    def sls(self) -> I16Vector3: ...
    @property
    def slo(self) -> I16Vector3: ...
    @property
    def sll(self) -> I16Vector3: ...
    @property
    def oss(self) -> I16Vector3: ...
    @property
    def oso(self) -> I16Vector3: ...
    @property
    def osl(self) -> I16Vector3: ...
    @property
    def oos(self) -> I16Vector3: ...
    @property
    def ols(self) -> I16Vector3: ...
    @property
    def lss(self) -> I16Vector3: ...
    @property
    def lso(self) -> I16Vector3: ...
    @property
    def lsl(self) -> I16Vector3: ...
    @property
    def los(self) -> I16Vector3: ...
    @property
    def lls(self) -> I16Vector3: ...
    @property
    def ssss(self) -> I16Vector4: ...
    @property
    def ssso(self) -> I16Vector4: ...
    @property
    def sssl(self) -> I16Vector4: ...
    @property
    def ssos(self) -> I16Vector4: ...
    @property
    def ssoo(self) -> I16Vector4: ...
    @property
    def ssol(self) -> I16Vector4: ...
    @property
    def ssls(self) -> I16Vector4: ...
    @property
    def sslo(self) -> I16Vector4: ...
    @property
    def ssll(self) -> I16Vector4: ...
    @property
    def soss(self) -> I16Vector4: ...
    @property
    def soso(self) -> I16Vector4: ...
    @property
    def sosl(self) -> I16Vector4: ...
    @property
    def soos(self) -> I16Vector4: ...
    @property
    def sooo(self) -> I16Vector4: ...
    @property
    def sool(self) -> I16Vector4: ...
    @property
    def sols(self) -> I16Vector4: ...
    @property
    def solo(self) -> I16Vector4: ...
    @property
    def soll(self) -> I16Vector4: ...
    @property
    def slss(self) -> I16Vector4: ...
    @property
    def slso(self) -> I16Vector4: ...
    @property
    def slsl(self) -> I16Vector4: ...
    @property
    def slos(self) -> I16Vector4: ...
    @property
    def sloo(self) -> I16Vector4: ...
    @property
    def slol(self) -> I16Vector4: ...
    @property
    def slls(self) -> I16Vector4: ...
    @property
    def sllo(self) -> I16Vector4: ...
    @property
    def slll(self) -> I16Vector4: ...
    @property
    def osss(self) -> I16Vector4: ...
    @property
    def osso(self) -> I16Vector4: ...
    @property
    def ossl(self) -> I16Vector4: ...
    @property
    def osos(self) -> I16Vector4: ...
    @property
    def osoo(self) -> I16Vector4: ...
    @property
    def osol(self) -> I16Vector4: ...
    @property
    def osls(self) -> I16Vector4: ...
    @property
    def oslo(self) -> I16Vector4: ...
    @property
    def osll(self) -> I16Vector4: ...
    @property
    def ooss(self) -> I16Vector4: ...
    @property
    def ooso(self) -> I16Vector4: ...
    @property
    def oosl(self) -> I16Vector4: ...
    @property
    def ooos(self) -> I16Vector4: ...
    @property
    def ools(self) -> I16Vector4: ...
    @property
    def olss(self) -> I16Vector4: ...
    @property
    def olso(self) -> I16Vector4: ...
    @property
    def olsl(self) -> I16Vector4: ...
    @property
    def olos(self) -> I16Vector4: ...
    @property
    def olls(self) -> I16Vector4: ...
    @property
    def lsss(self) -> I16Vector4: ...
    @property
    def lsso(self) -> I16Vector4: ...
    @property
    def lssl(self) -> I16Vector4: ...
    @property
    def lsos(self) -> I16Vector4: ...
    @property
    def lsoo(self) -> I16Vector4: ...
    @property
    def lsol(self) -> I16Vector4: ...
    @property
    def lsls(self) -> I16Vector4: ...
    @property
    def lslo(self) -> I16Vector4: ...
    @property
    def lsll(self) -> I16Vector4: ...
    @property
    def loss(self) -> I16Vector4: ...
    @property
    def loso(self) -> I16Vector4: ...
    @property
    def losl(self) -> I16Vector4: ...
    @property
    def loos(self) -> I16Vector4: ...
    @property
    def lols(self) -> I16Vector4: ...
    @property
    def llss(self) -> I16Vector4: ...
    @property
    def llso(self) -> I16Vector4: ...
    @property
    def llsl(self) -> I16Vector4: ...
    @property
    def llos(self) -> I16Vector4: ...
    @property
    def llls(self) -> I16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> I16Vector2: ...
    @property
    def uo(self) -> I16Vector2: ...
    @property
    def ul(self) -> I16Vector2: ...
    @property
    def ou(self) -> I16Vector2: ...
    @property
    def lu(self) -> I16Vector2: ...
    @property
    def uuu(self) -> I16Vector3: ...
    @property
    def uuo(self) -> I16Vector3: ...
    @property
    def uul(self) -> I16Vector3: ...
    @property
    def uou(self) -> I16Vector3: ...
    @property
    def uoo(self) -> I16Vector3: ...
    @property
    def uol(self) -> I16Vector3: ...
    @property
    def ulu(self) -> I16Vector3: ...
    @property
    def ulo(self) -> I16Vector3: ...
    @property
    def ull(self) -> I16Vector3: ...
    @property
    def ouu(self) -> I16Vector3: ...
    @property
    def ouo(self) -> I16Vector3: ...
    @property
    def oul(self) -> I16Vector3: ...
    @property
    def oou(self) -> I16Vector3: ...
    @property
    def olu(self) -> I16Vector3: ...
    @property
    def luu(self) -> I16Vector3: ...
    @property
    def luo(self) -> I16Vector3: ...
    @property
    def lul(self) -> I16Vector3: ...
    @property
    def lou(self) -> I16Vector3: ...
    @property
    def llu(self) -> I16Vector3: ...
    @property
    def uuuu(self) -> I16Vector4: ...
    @property
    def uuuo(self) -> I16Vector4: ...
    @property
    def uuul(self) -> I16Vector4: ...
    @property
    def uuou(self) -> I16Vector4: ...
    @property
    def uuoo(self) -> I16Vector4: ...
    @property
    def uuol(self) -> I16Vector4: ...
    @property
    def uulu(self) -> I16Vector4: ...
    @property
    def uulo(self) -> I16Vector4: ...
    @property
    def uull(self) -> I16Vector4: ...
    @property
    def uouu(self) -> I16Vector4: ...
    @property
    def uouo(self) -> I16Vector4: ...
    @property
    def uoul(self) -> I16Vector4: ...
    @property
    def uoou(self) -> I16Vector4: ...
    @property
    def uooo(self) -> I16Vector4: ...
    @property
    def uool(self) -> I16Vector4: ...
    @property
    def uolu(self) -> I16Vector4: ...
    @property
    def uolo(self) -> I16Vector4: ...
    @property
    def uoll(self) -> I16Vector4: ...
    @property
    def uluu(self) -> I16Vector4: ...
    @property
    def uluo(self) -> I16Vector4: ...
    @property
    def ulul(self) -> I16Vector4: ...
    @property
    def ulou(self) -> I16Vector4: ...
    @property
    def uloo(self) -> I16Vector4: ...
    @property
    def ulol(self) -> I16Vector4: ...
    @property
    def ullu(self) -> I16Vector4: ...
    @property
    def ullo(self) -> I16Vector4: ...
    @property
    def ulll(self) -> I16Vector4: ...
    @property
    def ouuu(self) -> I16Vector4: ...
    @property
    def ouuo(self) -> I16Vector4: ...
    @property
    def ouul(self) -> I16Vector4: ...
    @property
    def ouou(self) -> I16Vector4: ...
    @property
    def ouoo(self) -> I16Vector4: ...
    @property
    def ouol(self) -> I16Vector4: ...
    @property
    def oulu(self) -> I16Vector4: ...
    @property
    def oulo(self) -> I16Vector4: ...
    @property
    def oull(self) -> I16Vector4: ...
    @property
    def oouu(self) -> I16Vector4: ...
    @property
    def oouo(self) -> I16Vector4: ...
    @property
    def ooul(self) -> I16Vector4: ...
    @property
    def ooou(self) -> I16Vector4: ...
    @property
    def oolu(self) -> I16Vector4: ...
    @property
    def oluu(self) -> I16Vector4: ...
    @property
    def oluo(self) -> I16Vector4: ...
    @property
    def olul(self) -> I16Vector4: ...
    @property
    def olou(self) -> I16Vector4: ...
    @property
    def ollu(self) -> I16Vector4: ...
    @property
    def luuu(self) -> I16Vector4: ...
    @property
    def luuo(self) -> I16Vector4: ...
    @property
    def luul(self) -> I16Vector4: ...
    @property
    def luou(self) -> I16Vector4: ...
    @property
    def luoo(self) -> I16Vector4: ...
    @property
    def luol(self) -> I16Vector4: ...
    @property
    def lulu(self) -> I16Vector4: ...
    @property
    def lulo(self) -> I16Vector4: ...
    @property
    def lull(self) -> I16Vector4: ...
    @property
    def louu(self) -> I16Vector4: ...
    @property
    def louo(self) -> I16Vector4: ...
    @property
    def loul(self) -> I16Vector4: ...
    @property
    def loou(self) -> I16Vector4: ...
    @property
    def lolu(self) -> I16Vector4: ...
    @property
    def lluu(self) -> I16Vector4: ...
    @property
    def lluo(self) -> I16Vector4: ...
    @property
    def llul(self) -> I16Vector4: ...
    @property
    def llou(self) -> I16Vector4: ...
    @property
    def lllu(self) -> I16Vector4: ...
    @property
    def oo(self) -> I16Vector2: ...
    @property
    def ol(self) -> I16Vector2: ...
    @property
    def lo(self) -> I16Vector2: ...
    @property
    def ll(self) -> I16Vector2: ...
    @property
    def ooo(self) -> I16Vector3: ...
    @property
    def ool(self) -> I16Vector3: ...
    @property
    def olo(self) -> I16Vector3: ...
    @property
    def oll(self) -> I16Vector3: ...
    @property
    def loo(self) -> I16Vector3: ...
    @property
    def lol(self) -> I16Vector3: ...
    @property
    def llo(self) -> I16Vector3: ...
    @property
    def lll(self) -> I16Vector3: ...
    @property
    def oooo(self) -> I16Vector4: ...
    @property
    def oool(self) -> I16Vector4: ...
    @property
    def oolo(self) -> I16Vector4: ...
    @property
    def ooll(self) -> I16Vector4: ...
    @property
    def oloo(self) -> I16Vector4: ...
    @property
    def olol(self) -> I16Vector4: ...
    @property
    def ollo(self) -> I16Vector4: ...
    @property
    def olll(self) -> I16Vector4: ...
    @property
    def looo(self) -> I16Vector4: ...
    @property
    def lool(self) -> I16Vector4: ...
    @property
    def lolo(self) -> I16Vector4: ...
    @property
    def loll(self) -> I16Vector4: ...
    @property
    def lloo(self) -> I16Vector4: ...
    @property
    def llol(self) -> I16Vector4: ...
    @property
    def lllo(self) -> I16Vector4: ...
    @property
    def llll(self) -> I16Vector4: ...
    def min(self, n: Number, /) -> I16Vector1: ...
    def max(self, n: Number, /) -> I16Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> I16Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I16Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector1: ...

@final
class I16Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I16Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I16Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> I16Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> I16Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[I16Vector1]: ...

@final
class U16Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __add__(self, other: Number) -> U16Vector1: ...
    @overload
    def __radd__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __radd__(self, other: Number) -> U16Vector1: ...
    @overload
    def __sub__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __sub__(self, other: Number) -> U16Vector1: ...
    @overload
    def __rsub__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> U16Vector1: ...
    @overload
    def __mul__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __mul__(self, other: Number) -> U16Vector1: ...
    @overload
    def __rmul__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> U16Vector1: ...
    @overload
    def __truediv__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> U16Vector1: ...
    @overload
    def __rtruediv__(self, other: U16Vector1) -> U16Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> U16Vector1: ...
    def __abs__(self) -> U16Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> U16Vector2: ...
    @property
    def xo(self) -> U16Vector2: ...
    @property
    def xl(self) -> U16Vector2: ...
    @property
    def ox(self) -> U16Vector2: ...
    @property
    def lx(self) -> U16Vector2: ...
    @property
    def xxx(self) -> U16Vector3: ...
    @property
    def xxo(self) -> U16Vector3: ...
    @property
    def xxl(self) -> U16Vector3: ...
    @property
    def xox(self) -> U16Vector3: ...
    @property
    def xoo(self) -> U16Vector3: ...
    @property
    def xol(self) -> U16Vector3: ...
    @property
    def xlx(self) -> U16Vector3: ...
    @property
    def xlo(self) -> U16Vector3: ...
    @property
    def xll(self) -> U16Vector3: ...
    @property
    def oxx(self) -> U16Vector3: ...
    @property
    def oxo(self) -> U16Vector3: ...
    @property
    def oxl(self) -> U16Vector3: ...
    @property
    def oox(self) -> U16Vector3: ...
    @property
    def olx(self) -> U16Vector3: ...
    @property
    def lxx(self) -> U16Vector3: ...
    @property
    def lxo(self) -> U16Vector3: ...
    @property
    def lxl(self) -> U16Vector3: ...
    @property
    def lox(self) -> U16Vector3: ...
    @property
    def llx(self) -> U16Vector3: ...
    @property
    def xxxx(self) -> U16Vector4: ...
    @property
    def xxxo(self) -> U16Vector4: ...
    @property
    def xxxl(self) -> U16Vector4: ...
    @property
    def xxox(self) -> U16Vector4: ...
    @property
    def xxoo(self) -> U16Vector4: ...
    @property
    def xxol(self) -> U16Vector4: ...
    @property
    def xxlx(self) -> U16Vector4: ...
    @property
    def xxlo(self) -> U16Vector4: ...
    @property
    def xxll(self) -> U16Vector4: ...
    @property
    def xoxx(self) -> U16Vector4: ...
    @property
    def xoxo(self) -> U16Vector4: ...
    @property
    def xoxl(self) -> U16Vector4: ...
    @property
    def xoox(self) -> U16Vector4: ...
    @property
    def xooo(self) -> U16Vector4: ...
    @property
    def xool(self) -> U16Vector4: ...
    @property
    def xolx(self) -> U16Vector4: ...
    @property
    def xolo(self) -> U16Vector4: ...
    @property
    def xoll(self) -> U16Vector4: ...
    @property
    def xlxx(self) -> U16Vector4: ...
    @property
    def xlxo(self) -> U16Vector4: ...
    @property
    def xlxl(self) -> U16Vector4: ...
    @property
    def xlox(self) -> U16Vector4: ...
    @property
    def xloo(self) -> U16Vector4: ...
    @property
    def xlol(self) -> U16Vector4: ...
    @property
    def xllx(self) -> U16Vector4: ...
    @property
    def xllo(self) -> U16Vector4: ...
    @property
    def xlll(self) -> U16Vector4: ...
    @property
    def oxxx(self) -> U16Vector4: ...
    @property
    def oxxo(self) -> U16Vector4: ...
    @property
    def oxxl(self) -> U16Vector4: ...
    @property
    def oxox(self) -> U16Vector4: ...
    @property
    def oxoo(self) -> U16Vector4: ...
    @property
    def oxol(self) -> U16Vector4: ...
    @property
    def oxlx(self) -> U16Vector4: ...
    @property
    def oxlo(self) -> U16Vector4: ...
    @property
    def oxll(self) -> U16Vector4: ...
    @property
    def ooxx(self) -> U16Vector4: ...
    @property
    def ooxo(self) -> U16Vector4: ...
    @property
    def ooxl(self) -> U16Vector4: ...
    @property
    def ooox(self) -> U16Vector4: ...
    @property
    def oolx(self) -> U16Vector4: ...
    @property
    def olxx(self) -> U16Vector4: ...
    @property
    def olxo(self) -> U16Vector4: ...
    @property
    def olxl(self) -> U16Vector4: ...
    @property
    def olox(self) -> U16Vector4: ...
    @property
    def ollx(self) -> U16Vector4: ...
    @property
    def lxxx(self) -> U16Vector4: ...
    @property
    def lxxo(self) -> U16Vector4: ...
    @property
    def lxxl(self) -> U16Vector4: ...
    @property
    def lxox(self) -> U16Vector4: ...
    @property
    def lxoo(self) -> U16Vector4: ...
    @property
    def lxol(self) -> U16Vector4: ...
    @property
    def lxlx(self) -> U16Vector4: ...
    @property
    def lxlo(self) -> U16Vector4: ...
    @property
    def lxll(self) -> U16Vector4: ...
    @property
    def loxx(self) -> U16Vector4: ...
    @property
    def loxo(self) -> U16Vector4: ...
    @property
    def loxl(self) -> U16Vector4: ...
    @property
    def loox(self) -> U16Vector4: ...
    @property
    def lolx(self) -> U16Vector4: ...
    @property
    def llxx(self) -> U16Vector4: ...
    @property
    def llxo(self) -> U16Vector4: ...
    @property
    def llxl(self) -> U16Vector4: ...
    @property
    def llox(self) -> U16Vector4: ...
    @property
    def lllx(self) -> U16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> U16Vector2: ...
    @property
    def ro(self) -> U16Vector2: ...
    @property
    def rl(self) -> U16Vector2: ...
    @property
    def lr(self) -> U16Vector2: ...
    @property
    def rrr(self) -> U16Vector3: ...
    @property
    def rro(self) -> U16Vector3: ...
    @property
    def rrl(self) -> U16Vector3: ...
    @property
    def ror(self) -> U16Vector3: ...
    @property
    def roo(self) -> U16Vector3: ...
    @property
    def rol(self) -> U16Vector3: ...
    @property
    def rlr(self) -> U16Vector3: ...
    @property
    def rlo(self) -> U16Vector3: ...
    @property
    def rll(self) -> U16Vector3: ...
    @property
    def orr(self) -> U16Vector3: ...
    @property
    def oro(self) -> U16Vector3: ...
    @property
    def orl(self) -> U16Vector3: ...
    @property
    def oor(self) -> U16Vector3: ...
    @property
    def olr(self) -> U16Vector3: ...
    @property
    def lrr(self) -> U16Vector3: ...
    @property
    def lro(self) -> U16Vector3: ...
    @property
    def lrl(self) -> U16Vector3: ...
    @property
    def lor(self) -> U16Vector3: ...
    @property
    def llr(self) -> U16Vector3: ...
    @property
    def rrrr(self) -> U16Vector4: ...
    @property
    def rrro(self) -> U16Vector4: ...
    @property
    def rrrl(self) -> U16Vector4: ...
    @property
    def rror(self) -> U16Vector4: ...
    @property
    def rroo(self) -> U16Vector4: ...
    @property
    def rrol(self) -> U16Vector4: ...
    @property
    def rrlr(self) -> U16Vector4: ...
    @property
    def rrlo(self) -> U16Vector4: ...
    @property
    def rrll(self) -> U16Vector4: ...
    @property
    def rorr(self) -> U16Vector4: ...
    @property
    def roro(self) -> U16Vector4: ...
    @property
    def rorl(self) -> U16Vector4: ...
    @property
    def roor(self) -> U16Vector4: ...
    @property
    def rooo(self) -> U16Vector4: ...
    @property
    def rool(self) -> U16Vector4: ...
    @property
    def rolr(self) -> U16Vector4: ...
    @property
    def rolo(self) -> U16Vector4: ...
    @property
    def roll(self) -> U16Vector4: ...
    @property
    def rlrr(self) -> U16Vector4: ...
    @property
    def rlro(self) -> U16Vector4: ...
    @property
    def rlrl(self) -> U16Vector4: ...
    @property
    def rlor(self) -> U16Vector4: ...
    @property
    def rloo(self) -> U16Vector4: ...
    @property
    def rlol(self) -> U16Vector4: ...
    @property
    def rllr(self) -> U16Vector4: ...
    @property
    def rllo(self) -> U16Vector4: ...
    @property
    def rlll(self) -> U16Vector4: ...
    @property
    def orrr(self) -> U16Vector4: ...
    @property
    def orro(self) -> U16Vector4: ...
    @property
    def orrl(self) -> U16Vector4: ...
    @property
    def oror(self) -> U16Vector4: ...
    @property
    def oroo(self) -> U16Vector4: ...
    @property
    def orol(self) -> U16Vector4: ...
    @property
    def orlr(self) -> U16Vector4: ...
    @property
    def orlo(self) -> U16Vector4: ...
    @property
    def orll(self) -> U16Vector4: ...
    @property
    def oorr(self) -> U16Vector4: ...
    @property
    def ooro(self) -> U16Vector4: ...
    @property
    def oorl(self) -> U16Vector4: ...
    @property
    def ooor(self) -> U16Vector4: ...
    @property
    def oolr(self) -> U16Vector4: ...
    @property
    def olrr(self) -> U16Vector4: ...
    @property
    def olro(self) -> U16Vector4: ...
    @property
    def olrl(self) -> U16Vector4: ...
    @property
    def olor(self) -> U16Vector4: ...
    @property
    def ollr(self) -> U16Vector4: ...
    @property
    def lrrr(self) -> U16Vector4: ...
    @property
    def lrro(self) -> U16Vector4: ...
    @property
    def lrrl(self) -> U16Vector4: ...
    @property
    def lror(self) -> U16Vector4: ...
    @property
    def lroo(self) -> U16Vector4: ...
    @property
    def lrol(self) -> U16Vector4: ...
    @property
    def lrlr(self) -> U16Vector4: ...
    @property
    def lrlo(self) -> U16Vector4: ...
    @property
    def lrll(self) -> U16Vector4: ...
    @property
    def lorr(self) -> U16Vector4: ...
    @property
    def loro(self) -> U16Vector4: ...
    @property
    def lorl(self) -> U16Vector4: ...
    @property
    def loor(self) -> U16Vector4: ...
    @property
    def lolr(self) -> U16Vector4: ...
    @property
    def llrr(self) -> U16Vector4: ...
    @property
    def llro(self) -> U16Vector4: ...
    @property
    def llrl(self) -> U16Vector4: ...
    @property
    def llor(self) -> U16Vector4: ...
    @property
    def lllr(self) -> U16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> U16Vector2: ...
    @property
    def so(self) -> U16Vector2: ...
    @property
    def sl(self) -> U16Vector2: ...
    @property
    def os(self) -> U16Vector2: ...
    @property
    def ls(self) -> U16Vector2: ...
    @property
    def sss(self) -> U16Vector3: ...
    @property
    def sso(self) -> U16Vector3: ...
    @property
    def ssl(self) -> U16Vector3: ...
    @property
    def sos(self) -> U16Vector3: ...
    @property
    def soo(self) -> U16Vector3: ...
    @property
    def sol(self) -> U16Vector3: ...
    @property
    def sls(self) -> U16Vector3: ...
    @property
    def slo(self) -> U16Vector3: ...
    @property
    def sll(self) -> U16Vector3: ...
    @property
    def oss(self) -> U16Vector3: ...
    @property
    def oso(self) -> U16Vector3: ...
    @property
    def osl(self) -> U16Vector3: ...
    @property
    def oos(self) -> U16Vector3: ...
    @property
    def ols(self) -> U16Vector3: ...
    @property
    def lss(self) -> U16Vector3: ...
    @property
    def lso(self) -> U16Vector3: ...
    @property
    def lsl(self) -> U16Vector3: ...
    @property
    def los(self) -> U16Vector3: ...
    @property
    def lls(self) -> U16Vector3: ...
    @property
    def ssss(self) -> U16Vector4: ...
    @property
    def ssso(self) -> U16Vector4: ...
    @property
    def sssl(self) -> U16Vector4: ...
    @property
    def ssos(self) -> U16Vector4: ...
    @property
    def ssoo(self) -> U16Vector4: ...
    @property
    def ssol(self) -> U16Vector4: ...
    @property
    def ssls(self) -> U16Vector4: ...
    @property
    def sslo(self) -> U16Vector4: ...
    @property
    def ssll(self) -> U16Vector4: ...
    @property
    def soss(self) -> U16Vector4: ...
    @property
    def soso(self) -> U16Vector4: ...
    @property
    def sosl(self) -> U16Vector4: ...
    @property
    def soos(self) -> U16Vector4: ...
    @property
    def sooo(self) -> U16Vector4: ...
    @property
    def sool(self) -> U16Vector4: ...
    @property
    def sols(self) -> U16Vector4: ...
    @property
    def solo(self) -> U16Vector4: ...
    @property
    def soll(self) -> U16Vector4: ...
    @property
    def slss(self) -> U16Vector4: ...
    @property
    def slso(self) -> U16Vector4: ...
    @property
    def slsl(self) -> U16Vector4: ...
    @property
    def slos(self) -> U16Vector4: ...
    @property
    def sloo(self) -> U16Vector4: ...
    @property
    def slol(self) -> U16Vector4: ...
    @property
    def slls(self) -> U16Vector4: ...
    @property
    def sllo(self) -> U16Vector4: ...
    @property
    def slll(self) -> U16Vector4: ...
    @property
    def osss(self) -> U16Vector4: ...
    @property
    def osso(self) -> U16Vector4: ...
    @property
    def ossl(self) -> U16Vector4: ...
    @property
    def osos(self) -> U16Vector4: ...
    @property
    def osoo(self) -> U16Vector4: ...
    @property
    def osol(self) -> U16Vector4: ...
    @property
    def osls(self) -> U16Vector4: ...
    @property
    def oslo(self) -> U16Vector4: ...
    @property
    def osll(self) -> U16Vector4: ...
    @property
    def ooss(self) -> U16Vector4: ...
    @property
    def ooso(self) -> U16Vector4: ...
    @property
    def oosl(self) -> U16Vector4: ...
    @property
    def ooos(self) -> U16Vector4: ...
    @property
    def ools(self) -> U16Vector4: ...
    @property
    def olss(self) -> U16Vector4: ...
    @property
    def olso(self) -> U16Vector4: ...
    @property
    def olsl(self) -> U16Vector4: ...
    @property
    def olos(self) -> U16Vector4: ...
    @property
    def olls(self) -> U16Vector4: ...
    @property
    def lsss(self) -> U16Vector4: ...
    @property
    def lsso(self) -> U16Vector4: ...
    @property
    def lssl(self) -> U16Vector4: ...
    @property
    def lsos(self) -> U16Vector4: ...
    @property
    def lsoo(self) -> U16Vector4: ...
    @property
    def lsol(self) -> U16Vector4: ...
    @property
    def lsls(self) -> U16Vector4: ...
    @property
    def lslo(self) -> U16Vector4: ...
    @property
    def lsll(self) -> U16Vector4: ...
    @property
    def loss(self) -> U16Vector4: ...
    @property
    def loso(self) -> U16Vector4: ...
    @property
    def losl(self) -> U16Vector4: ...
    @property
    def loos(self) -> U16Vector4: ...
    @property
    def lols(self) -> U16Vector4: ...
    @property
    def llss(self) -> U16Vector4: ...
    @property
    def llso(self) -> U16Vector4: ...
    @property
    def llsl(self) -> U16Vector4: ...
    @property
    def llos(self) -> U16Vector4: ...
    @property
    def llls(self) -> U16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> U16Vector2: ...
    @property
    def uo(self) -> U16Vector2: ...
    @property
    def ul(self) -> U16Vector2: ...
    @property
    def ou(self) -> U16Vector2: ...
    @property
    def lu(self) -> U16Vector2: ...
    @property
    def uuu(self) -> U16Vector3: ...
    @property
    def uuo(self) -> U16Vector3: ...
    @property
    def uul(self) -> U16Vector3: ...
    @property
    def uou(self) -> U16Vector3: ...
    @property
    def uoo(self) -> U16Vector3: ...
    @property
    def uol(self) -> U16Vector3: ...
    @property
    def ulu(self) -> U16Vector3: ...
    @property
    def ulo(self) -> U16Vector3: ...
    @property
    def ull(self) -> U16Vector3: ...
    @property
    def ouu(self) -> U16Vector3: ...
    @property
    def ouo(self) -> U16Vector3: ...
    @property
    def oul(self) -> U16Vector3: ...
    @property
    def oou(self) -> U16Vector3: ...
    @property
    def olu(self) -> U16Vector3: ...
    @property
    def luu(self) -> U16Vector3: ...
    @property
    def luo(self) -> U16Vector3: ...
    @property
    def lul(self) -> U16Vector3: ...
    @property
    def lou(self) -> U16Vector3: ...
    @property
    def llu(self) -> U16Vector3: ...
    @property
    def uuuu(self) -> U16Vector4: ...
    @property
    def uuuo(self) -> U16Vector4: ...
    @property
    def uuul(self) -> U16Vector4: ...
    @property
    def uuou(self) -> U16Vector4: ...
    @property
    def uuoo(self) -> U16Vector4: ...
    @property
    def uuol(self) -> U16Vector4: ...
    @property
    def uulu(self) -> U16Vector4: ...
    @property
    def uulo(self) -> U16Vector4: ...
    @property
    def uull(self) -> U16Vector4: ...
    @property
    def uouu(self) -> U16Vector4: ...
    @property
    def uouo(self) -> U16Vector4: ...
    @property
    def uoul(self) -> U16Vector4: ...
    @property
    def uoou(self) -> U16Vector4: ...
    @property
    def uooo(self) -> U16Vector4: ...
    @property
    def uool(self) -> U16Vector4: ...
    @property
    def uolu(self) -> U16Vector4: ...
    @property
    def uolo(self) -> U16Vector4: ...
    @property
    def uoll(self) -> U16Vector4: ...
    @property
    def uluu(self) -> U16Vector4: ...
    @property
    def uluo(self) -> U16Vector4: ...
    @property
    def ulul(self) -> U16Vector4: ...
    @property
    def ulou(self) -> U16Vector4: ...
    @property
    def uloo(self) -> U16Vector4: ...
    @property
    def ulol(self) -> U16Vector4: ...
    @property
    def ullu(self) -> U16Vector4: ...
    @property
    def ullo(self) -> U16Vector4: ...
    @property
    def ulll(self) -> U16Vector4: ...
    @property
    def ouuu(self) -> U16Vector4: ...
    @property
    def ouuo(self) -> U16Vector4: ...
    @property
    def ouul(self) -> U16Vector4: ...
    @property
    def ouou(self) -> U16Vector4: ...
    @property
    def ouoo(self) -> U16Vector4: ...
    @property
    def ouol(self) -> U16Vector4: ...
    @property
    def oulu(self) -> U16Vector4: ...
    @property
    def oulo(self) -> U16Vector4: ...
    @property
    def oull(self) -> U16Vector4: ...
    @property
    def oouu(self) -> U16Vector4: ...
    @property
    def oouo(self) -> U16Vector4: ...
    @property
    def ooul(self) -> U16Vector4: ...
    @property
    def ooou(self) -> U16Vector4: ...
    @property
    def oolu(self) -> U16Vector4: ...
    @property
    def oluu(self) -> U16Vector4: ...
    @property
    def oluo(self) -> U16Vector4: ...
    @property
    def olul(self) -> U16Vector4: ...
    @property
    def olou(self) -> U16Vector4: ...
    @property
    def ollu(self) -> U16Vector4: ...
    @property
    def luuu(self) -> U16Vector4: ...
    @property
    def luuo(self) -> U16Vector4: ...
    @property
    def luul(self) -> U16Vector4: ...
    @property
    def luou(self) -> U16Vector4: ...
    @property
    def luoo(self) -> U16Vector4: ...
    @property
    def luol(self) -> U16Vector4: ...
    @property
    def lulu(self) -> U16Vector4: ...
    @property
    def lulo(self) -> U16Vector4: ...
    @property
    def lull(self) -> U16Vector4: ...
    @property
    def louu(self) -> U16Vector4: ...
    @property
    def louo(self) -> U16Vector4: ...
    @property
    def loul(self) -> U16Vector4: ...
    @property
    def loou(self) -> U16Vector4: ...
    @property
    def lolu(self) -> U16Vector4: ...
    @property
    def lluu(self) -> U16Vector4: ...
    @property
    def lluo(self) -> U16Vector4: ...
    @property
    def llul(self) -> U16Vector4: ...
    @property
    def llou(self) -> U16Vector4: ...
    @property
    def lllu(self) -> U16Vector4: ...
    @property
    def oo(self) -> U16Vector2: ...
    @property
    def ol(self) -> U16Vector2: ...
    @property
    def lo(self) -> U16Vector2: ...
    @property
    def ll(self) -> U16Vector2: ...
    @property
    def ooo(self) -> U16Vector3: ...
    @property
    def ool(self) -> U16Vector3: ...
    @property
    def olo(self) -> U16Vector3: ...
    @property
    def oll(self) -> U16Vector3: ...
    @property
    def loo(self) -> U16Vector3: ...
    @property
    def lol(self) -> U16Vector3: ...
    @property
    def llo(self) -> U16Vector3: ...
    @property
    def lll(self) -> U16Vector3: ...
    @property
    def oooo(self) -> U16Vector4: ...
    @property
    def oool(self) -> U16Vector4: ...
    @property
    def oolo(self) -> U16Vector4: ...
    @property
    def ooll(self) -> U16Vector4: ...
    @property
    def oloo(self) -> U16Vector4: ...
    @property
    def olol(self) -> U16Vector4: ...
    @property
    def ollo(self) -> U16Vector4: ...
    @property
    def olll(self) -> U16Vector4: ...
    @property
    def looo(self) -> U16Vector4: ...
    @property
    def lool(self) -> U16Vector4: ...
    @property
    def lolo(self) -> U16Vector4: ...
    @property
    def loll(self) -> U16Vector4: ...
    @property
    def lloo(self) -> U16Vector4: ...
    @property
    def llol(self) -> U16Vector4: ...
    @property
    def lllo(self) -> U16Vector4: ...
    @property
    def llll(self) -> U16Vector4: ...
    def min(self, n: Number, /) -> U16Vector1: ...
    def max(self, n: Number, /) -> U16Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> U16Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U16Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector1: ...

@final
class U16Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U16Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U16Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> U16Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> U16Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[U16Vector1]: ...

@final
class I32Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __add__(self, other: Number) -> I32Vector1: ...
    @overload
    def __radd__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __radd__(self, other: Number) -> I32Vector1: ...
    @overload
    def __sub__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __sub__(self, other: Number) -> I32Vector1: ...
    @overload
    def __rsub__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> I32Vector1: ...
    @overload
    def __mul__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __mul__(self, other: Number) -> I32Vector1: ...
    @overload
    def __rmul__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> I32Vector1: ...
    @overload
    def __truediv__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> I32Vector1: ...
    @overload
    def __rtruediv__(self, other: I32Vector1) -> I32Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> I32Vector1: ...
    def __neg__(self) -> I32Vector1: ...
    def __abs__(self) -> I32Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> I32Vector2: ...
    @property
    def xo(self) -> I32Vector2: ...
    @property
    def xl(self) -> I32Vector2: ...
    @property
    def ox(self) -> I32Vector2: ...
    @property
    def lx(self) -> I32Vector2: ...
    @property
    def xxx(self) -> I32Vector3: ...
    @property
    def xxo(self) -> I32Vector3: ...
    @property
    def xxl(self) -> I32Vector3: ...
    @property
    def xox(self) -> I32Vector3: ...
    @property
    def xoo(self) -> I32Vector3: ...
    @property
    def xol(self) -> I32Vector3: ...
    @property
    def xlx(self) -> I32Vector3: ...
    @property
    def xlo(self) -> I32Vector3: ...
    @property
    def xll(self) -> I32Vector3: ...
    @property
    def oxx(self) -> I32Vector3: ...
    @property
    def oxo(self) -> I32Vector3: ...
    @property
    def oxl(self) -> I32Vector3: ...
    @property
    def oox(self) -> I32Vector3: ...
    @property
    def olx(self) -> I32Vector3: ...
    @property
    def lxx(self) -> I32Vector3: ...
    @property
    def lxo(self) -> I32Vector3: ...
    @property
    def lxl(self) -> I32Vector3: ...
    @property
    def lox(self) -> I32Vector3: ...
    @property
    def llx(self) -> I32Vector3: ...
    @property
    def xxxx(self) -> I32Vector4: ...
    @property
    def xxxo(self) -> I32Vector4: ...
    @property
    def xxxl(self) -> I32Vector4: ...
    @property
    def xxox(self) -> I32Vector4: ...
    @property
    def xxoo(self) -> I32Vector4: ...
    @property
    def xxol(self) -> I32Vector4: ...
    @property
    def xxlx(self) -> I32Vector4: ...
    @property
    def xxlo(self) -> I32Vector4: ...
    @property
    def xxll(self) -> I32Vector4: ...
    @property
    def xoxx(self) -> I32Vector4: ...
    @property
    def xoxo(self) -> I32Vector4: ...
    @property
    def xoxl(self) -> I32Vector4: ...
    @property
    def xoox(self) -> I32Vector4: ...
    @property
    def xooo(self) -> I32Vector4: ...
    @property
    def xool(self) -> I32Vector4: ...
    @property
    def xolx(self) -> I32Vector4: ...
    @property
    def xolo(self) -> I32Vector4: ...
    @property
    def xoll(self) -> I32Vector4: ...
    @property
    def xlxx(self) -> I32Vector4: ...
    @property
    def xlxo(self) -> I32Vector4: ...
    @property
    def xlxl(self) -> I32Vector4: ...
    @property
    def xlox(self) -> I32Vector4: ...
    @property
    def xloo(self) -> I32Vector4: ...
    @property
    def xlol(self) -> I32Vector4: ...
    @property
    def xllx(self) -> I32Vector4: ...
    @property
    def xllo(self) -> I32Vector4: ...
    @property
    def xlll(self) -> I32Vector4: ...
    @property
    def oxxx(self) -> I32Vector4: ...
    @property
    def oxxo(self) -> I32Vector4: ...
    @property
    def oxxl(self) -> I32Vector4: ...
    @property
    def oxox(self) -> I32Vector4: ...
    @property
    def oxoo(self) -> I32Vector4: ...
    @property
    def oxol(self) -> I32Vector4: ...
    @property
    def oxlx(self) -> I32Vector4: ...
    @property
    def oxlo(self) -> I32Vector4: ...
    @property
    def oxll(self) -> I32Vector4: ...
    @property
    def ooxx(self) -> I32Vector4: ...
    @property
    def ooxo(self) -> I32Vector4: ...
    @property
    def ooxl(self) -> I32Vector4: ...
    @property
    def ooox(self) -> I32Vector4: ...
    @property
    def oolx(self) -> I32Vector4: ...
    @property
    def olxx(self) -> I32Vector4: ...
    @property
    def olxo(self) -> I32Vector4: ...
    @property
    def olxl(self) -> I32Vector4: ...
    @property
    def olox(self) -> I32Vector4: ...
    @property
    def ollx(self) -> I32Vector4: ...
    @property
    def lxxx(self) -> I32Vector4: ...
    @property
    def lxxo(self) -> I32Vector4: ...
    @property
    def lxxl(self) -> I32Vector4: ...
    @property
    def lxox(self) -> I32Vector4: ...
    @property
    def lxoo(self) -> I32Vector4: ...
    @property
    def lxol(self) -> I32Vector4: ...
    @property
    def lxlx(self) -> I32Vector4: ...
    @property
    def lxlo(self) -> I32Vector4: ...
    @property
    def lxll(self) -> I32Vector4: ...
    @property
    def loxx(self) -> I32Vector4: ...
    @property
    def loxo(self) -> I32Vector4: ...
    @property
    def loxl(self) -> I32Vector4: ...
    @property
    def loox(self) -> I32Vector4: ...
    @property
    def lolx(self) -> I32Vector4: ...
    @property
    def llxx(self) -> I32Vector4: ...
    @property
    def llxo(self) -> I32Vector4: ...
    @property
    def llxl(self) -> I32Vector4: ...
    @property
    def llox(self) -> I32Vector4: ...
    @property
    def lllx(self) -> I32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> I32Vector2: ...
    @property
    def ro(self) -> I32Vector2: ...
    @property
    def rl(self) -> I32Vector2: ...
    @property
    def lr(self) -> I32Vector2: ...
    @property
    def rrr(self) -> I32Vector3: ...
    @property
    def rro(self) -> I32Vector3: ...
    @property
    def rrl(self) -> I32Vector3: ...
    @property
    def ror(self) -> I32Vector3: ...
    @property
    def roo(self) -> I32Vector3: ...
    @property
    def rol(self) -> I32Vector3: ...
    @property
    def rlr(self) -> I32Vector3: ...
    @property
    def rlo(self) -> I32Vector3: ...
    @property
    def rll(self) -> I32Vector3: ...
    @property
    def orr(self) -> I32Vector3: ...
    @property
    def oro(self) -> I32Vector3: ...
    @property
    def orl(self) -> I32Vector3: ...
    @property
    def oor(self) -> I32Vector3: ...
    @property
    def olr(self) -> I32Vector3: ...
    @property
    def lrr(self) -> I32Vector3: ...
    @property
    def lro(self) -> I32Vector3: ...
    @property
    def lrl(self) -> I32Vector3: ...
    @property
    def lor(self) -> I32Vector3: ...
    @property
    def llr(self) -> I32Vector3: ...
    @property
    def rrrr(self) -> I32Vector4: ...
    @property
    def rrro(self) -> I32Vector4: ...
    @property
    def rrrl(self) -> I32Vector4: ...
    @property
    def rror(self) -> I32Vector4: ...
    @property
    def rroo(self) -> I32Vector4: ...
    @property
    def rrol(self) -> I32Vector4: ...
    @property
    def rrlr(self) -> I32Vector4: ...
    @property
    def rrlo(self) -> I32Vector4: ...
    @property
    def rrll(self) -> I32Vector4: ...
    @property
    def rorr(self) -> I32Vector4: ...
    @property
    def roro(self) -> I32Vector4: ...
    @property
    def rorl(self) -> I32Vector4: ...
    @property
    def roor(self) -> I32Vector4: ...
    @property
    def rooo(self) -> I32Vector4: ...
    @property
    def rool(self) -> I32Vector4: ...
    @property
    def rolr(self) -> I32Vector4: ...
    @property
    def rolo(self) -> I32Vector4: ...
    @property
    def roll(self) -> I32Vector4: ...
    @property
    def rlrr(self) -> I32Vector4: ...
    @property
    def rlro(self) -> I32Vector4: ...
    @property
    def rlrl(self) -> I32Vector4: ...
    @property
    def rlor(self) -> I32Vector4: ...
    @property
    def rloo(self) -> I32Vector4: ...
    @property
    def rlol(self) -> I32Vector4: ...
    @property
    def rllr(self) -> I32Vector4: ...
    @property
    def rllo(self) -> I32Vector4: ...
    @property
    def rlll(self) -> I32Vector4: ...
    @property
    def orrr(self) -> I32Vector4: ...
    @property
    def orro(self) -> I32Vector4: ...
    @property
    def orrl(self) -> I32Vector4: ...
    @property
    def oror(self) -> I32Vector4: ...
    @property
    def oroo(self) -> I32Vector4: ...
    @property
    def orol(self) -> I32Vector4: ...
    @property
    def orlr(self) -> I32Vector4: ...
    @property
    def orlo(self) -> I32Vector4: ...
    @property
    def orll(self) -> I32Vector4: ...
    @property
    def oorr(self) -> I32Vector4: ...
    @property
    def ooro(self) -> I32Vector4: ...
    @property
    def oorl(self) -> I32Vector4: ...
    @property
    def ooor(self) -> I32Vector4: ...
    @property
    def oolr(self) -> I32Vector4: ...
    @property
    def olrr(self) -> I32Vector4: ...
    @property
    def olro(self) -> I32Vector4: ...
    @property
    def olrl(self) -> I32Vector4: ...
    @property
    def olor(self) -> I32Vector4: ...
    @property
    def ollr(self) -> I32Vector4: ...
    @property
    def lrrr(self) -> I32Vector4: ...
    @property
    def lrro(self) -> I32Vector4: ...
    @property
    def lrrl(self) -> I32Vector4: ...
    @property
    def lror(self) -> I32Vector4: ...
    @property
    def lroo(self) -> I32Vector4: ...
    @property
    def lrol(self) -> I32Vector4: ...
    @property
    def lrlr(self) -> I32Vector4: ...
    @property
    def lrlo(self) -> I32Vector4: ...
    @property
    def lrll(self) -> I32Vector4: ...
    @property
    def lorr(self) -> I32Vector4: ...
    @property
    def loro(self) -> I32Vector4: ...
    @property
    def lorl(self) -> I32Vector4: ...
    @property
    def loor(self) -> I32Vector4: ...
    @property
    def lolr(self) -> I32Vector4: ...
    @property
    def llrr(self) -> I32Vector4: ...
    @property
    def llro(self) -> I32Vector4: ...
    @property
    def llrl(self) -> I32Vector4: ...
    @property
    def llor(self) -> I32Vector4: ...
    @property
    def lllr(self) -> I32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> I32Vector2: ...
    @property
    def so(self) -> I32Vector2: ...
    @property
    def sl(self) -> I32Vector2: ...
    @property
    def os(self) -> I32Vector2: ...
    @property
    def ls(self) -> I32Vector2: ...
    @property
    def sss(self) -> I32Vector3: ...
    @property
    def sso(self) -> I32Vector3: ...
    @property
    def ssl(self) -> I32Vector3: ...
    @property
    def sos(self) -> I32Vector3: ...
    @property
    def soo(self) -> I32Vector3: ...
    @property
    def sol(self) -> I32Vector3: ...
    @property
    def sls(self) -> I32Vector3: ...
    @property
    def slo(self) -> I32Vector3: ...
    @property
    def sll(self) -> I32Vector3: ...
    @property
    def oss(self) -> I32Vector3: ...
    @property
    def oso(self) -> I32Vector3: ...
    @property
    def osl(self) -> I32Vector3: ...
    @property
    def oos(self) -> I32Vector3: ...
    @property
    def ols(self) -> I32Vector3: ...
    @property
    def lss(self) -> I32Vector3: ...
    @property
    def lso(self) -> I32Vector3: ...
    @property
    def lsl(self) -> I32Vector3: ...
    @property
    def los(self) -> I32Vector3: ...
    @property
    def lls(self) -> I32Vector3: ...
    @property
    def ssss(self) -> I32Vector4: ...
    @property
    def ssso(self) -> I32Vector4: ...
    @property
    def sssl(self) -> I32Vector4: ...
    @property
    def ssos(self) -> I32Vector4: ...
    @property
    def ssoo(self) -> I32Vector4: ...
    @property
    def ssol(self) -> I32Vector4: ...
    @property
    def ssls(self) -> I32Vector4: ...
    @property
    def sslo(self) -> I32Vector4: ...
    @property
    def ssll(self) -> I32Vector4: ...
    @property
    def soss(self) -> I32Vector4: ...
    @property
    def soso(self) -> I32Vector4: ...
    @property
    def sosl(self) -> I32Vector4: ...
    @property
    def soos(self) -> I32Vector4: ...
    @property
    def sooo(self) -> I32Vector4: ...
    @property
    def sool(self) -> I32Vector4: ...
    @property
    def sols(self) -> I32Vector4: ...
    @property
    def solo(self) -> I32Vector4: ...
    @property
    def soll(self) -> I32Vector4: ...
    @property
    def slss(self) -> I32Vector4: ...
    @property
    def slso(self) -> I32Vector4: ...
    @property
    def slsl(self) -> I32Vector4: ...
    @property
    def slos(self) -> I32Vector4: ...
    @property
    def sloo(self) -> I32Vector4: ...
    @property
    def slol(self) -> I32Vector4: ...
    @property
    def slls(self) -> I32Vector4: ...
    @property
    def sllo(self) -> I32Vector4: ...
    @property
    def slll(self) -> I32Vector4: ...
    @property
    def osss(self) -> I32Vector4: ...
    @property
    def osso(self) -> I32Vector4: ...
    @property
    def ossl(self) -> I32Vector4: ...
    @property
    def osos(self) -> I32Vector4: ...
    @property
    def osoo(self) -> I32Vector4: ...
    @property
    def osol(self) -> I32Vector4: ...
    @property
    def osls(self) -> I32Vector4: ...
    @property
    def oslo(self) -> I32Vector4: ...
    @property
    def osll(self) -> I32Vector4: ...
    @property
    def ooss(self) -> I32Vector4: ...
    @property
    def ooso(self) -> I32Vector4: ...
    @property
    def oosl(self) -> I32Vector4: ...
    @property
    def ooos(self) -> I32Vector4: ...
    @property
    def ools(self) -> I32Vector4: ...
    @property
    def olss(self) -> I32Vector4: ...
    @property
    def olso(self) -> I32Vector4: ...
    @property
    def olsl(self) -> I32Vector4: ...
    @property
    def olos(self) -> I32Vector4: ...
    @property
    def olls(self) -> I32Vector4: ...
    @property
    def lsss(self) -> I32Vector4: ...
    @property
    def lsso(self) -> I32Vector4: ...
    @property
    def lssl(self) -> I32Vector4: ...
    @property
    def lsos(self) -> I32Vector4: ...
    @property
    def lsoo(self) -> I32Vector4: ...
    @property
    def lsol(self) -> I32Vector4: ...
    @property
    def lsls(self) -> I32Vector4: ...
    @property
    def lslo(self) -> I32Vector4: ...
    @property
    def lsll(self) -> I32Vector4: ...
    @property
    def loss(self) -> I32Vector4: ...
    @property
    def loso(self) -> I32Vector4: ...
    @property
    def losl(self) -> I32Vector4: ...
    @property
    def loos(self) -> I32Vector4: ...
    @property
    def lols(self) -> I32Vector4: ...
    @property
    def llss(self) -> I32Vector4: ...
    @property
    def llso(self) -> I32Vector4: ...
    @property
    def llsl(self) -> I32Vector4: ...
    @property
    def llos(self) -> I32Vector4: ...
    @property
    def llls(self) -> I32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> I32Vector2: ...
    @property
    def uo(self) -> I32Vector2: ...
    @property
    def ul(self) -> I32Vector2: ...
    @property
    def ou(self) -> I32Vector2: ...
    @property
    def lu(self) -> I32Vector2: ...
    @property
    def uuu(self) -> I32Vector3: ...
    @property
    def uuo(self) -> I32Vector3: ...
    @property
    def uul(self) -> I32Vector3: ...
    @property
    def uou(self) -> I32Vector3: ...
    @property
    def uoo(self) -> I32Vector3: ...
    @property
    def uol(self) -> I32Vector3: ...
    @property
    def ulu(self) -> I32Vector3: ...
    @property
    def ulo(self) -> I32Vector3: ...
    @property
    def ull(self) -> I32Vector3: ...
    @property
    def ouu(self) -> I32Vector3: ...
    @property
    def ouo(self) -> I32Vector3: ...
    @property
    def oul(self) -> I32Vector3: ...
    @property
    def oou(self) -> I32Vector3: ...
    @property
    def olu(self) -> I32Vector3: ...
    @property
    def luu(self) -> I32Vector3: ...
    @property
    def luo(self) -> I32Vector3: ...
    @property
    def lul(self) -> I32Vector3: ...
    @property
    def lou(self) -> I32Vector3: ...
    @property
    def llu(self) -> I32Vector3: ...
    @property
    def uuuu(self) -> I32Vector4: ...
    @property
    def uuuo(self) -> I32Vector4: ...
    @property
    def uuul(self) -> I32Vector4: ...
    @property
    def uuou(self) -> I32Vector4: ...
    @property
    def uuoo(self) -> I32Vector4: ...
    @property
    def uuol(self) -> I32Vector4: ...
    @property
    def uulu(self) -> I32Vector4: ...
    @property
    def uulo(self) -> I32Vector4: ...
    @property
    def uull(self) -> I32Vector4: ...
    @property
    def uouu(self) -> I32Vector4: ...
    @property
    def uouo(self) -> I32Vector4: ...
    @property
    def uoul(self) -> I32Vector4: ...
    @property
    def uoou(self) -> I32Vector4: ...
    @property
    def uooo(self) -> I32Vector4: ...
    @property
    def uool(self) -> I32Vector4: ...
    @property
    def uolu(self) -> I32Vector4: ...
    @property
    def uolo(self) -> I32Vector4: ...
    @property
    def uoll(self) -> I32Vector4: ...
    @property
    def uluu(self) -> I32Vector4: ...
    @property
    def uluo(self) -> I32Vector4: ...
    @property
    def ulul(self) -> I32Vector4: ...
    @property
    def ulou(self) -> I32Vector4: ...
    @property
    def uloo(self) -> I32Vector4: ...
    @property
    def ulol(self) -> I32Vector4: ...
    @property
    def ullu(self) -> I32Vector4: ...
    @property
    def ullo(self) -> I32Vector4: ...
    @property
    def ulll(self) -> I32Vector4: ...
    @property
    def ouuu(self) -> I32Vector4: ...
    @property
    def ouuo(self) -> I32Vector4: ...
    @property
    def ouul(self) -> I32Vector4: ...
    @property
    def ouou(self) -> I32Vector4: ...
    @property
    def ouoo(self) -> I32Vector4: ...
    @property
    def ouol(self) -> I32Vector4: ...
    @property
    def oulu(self) -> I32Vector4: ...
    @property
    def oulo(self) -> I32Vector4: ...
    @property
    def oull(self) -> I32Vector4: ...
    @property
    def oouu(self) -> I32Vector4: ...
    @property
    def oouo(self) -> I32Vector4: ...
    @property
    def ooul(self) -> I32Vector4: ...
    @property
    def ooou(self) -> I32Vector4: ...
    @property
    def oolu(self) -> I32Vector4: ...
    @property
    def oluu(self) -> I32Vector4: ...
    @property
    def oluo(self) -> I32Vector4: ...
    @property
    def olul(self) -> I32Vector4: ...
    @property
    def olou(self) -> I32Vector4: ...
    @property
    def ollu(self) -> I32Vector4: ...
    @property
    def luuu(self) -> I32Vector4: ...
    @property
    def luuo(self) -> I32Vector4: ...
    @property
    def luul(self) -> I32Vector4: ...
    @property
    def luou(self) -> I32Vector4: ...
    @property
    def luoo(self) -> I32Vector4: ...
    @property
    def luol(self) -> I32Vector4: ...
    @property
    def lulu(self) -> I32Vector4: ...
    @property
    def lulo(self) -> I32Vector4: ...
    @property
    def lull(self) -> I32Vector4: ...
    @property
    def louu(self) -> I32Vector4: ...
    @property
    def louo(self) -> I32Vector4: ...
    @property
    def loul(self) -> I32Vector4: ...
    @property
    def loou(self) -> I32Vector4: ...
    @property
    def lolu(self) -> I32Vector4: ...
    @property
    def lluu(self) -> I32Vector4: ...
    @property
    def lluo(self) -> I32Vector4: ...
    @property
    def llul(self) -> I32Vector4: ...
    @property
    def llou(self) -> I32Vector4: ...
    @property
    def lllu(self) -> I32Vector4: ...
    @property
    def oo(self) -> I32Vector2: ...
    @property
    def ol(self) -> I32Vector2: ...
    @property
    def lo(self) -> I32Vector2: ...
    @property
    def ll(self) -> I32Vector2: ...
    @property
    def ooo(self) -> I32Vector3: ...
    @property
    def ool(self) -> I32Vector3: ...
    @property
    def olo(self) -> I32Vector3: ...
    @property
    def oll(self) -> I32Vector3: ...
    @property
    def loo(self) -> I32Vector3: ...
    @property
    def lol(self) -> I32Vector3: ...
    @property
    def llo(self) -> I32Vector3: ...
    @property
    def lll(self) -> I32Vector3: ...
    @property
    def oooo(self) -> I32Vector4: ...
    @property
    def oool(self) -> I32Vector4: ...
    @property
    def oolo(self) -> I32Vector4: ...
    @property
    def ooll(self) -> I32Vector4: ...
    @property
    def oloo(self) -> I32Vector4: ...
    @property
    def olol(self) -> I32Vector4: ...
    @property
    def ollo(self) -> I32Vector4: ...
    @property
    def olll(self) -> I32Vector4: ...
    @property
    def looo(self) -> I32Vector4: ...
    @property
    def lool(self) -> I32Vector4: ...
    @property
    def lolo(self) -> I32Vector4: ...
    @property
    def loll(self) -> I32Vector4: ...
    @property
    def lloo(self) -> I32Vector4: ...
    @property
    def llol(self) -> I32Vector4: ...
    @property
    def lllo(self) -> I32Vector4: ...
    @property
    def llll(self) -> I32Vector4: ...
    def min(self, n: Number, /) -> I32Vector1: ...
    def max(self, n: Number, /) -> I32Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> I32Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I32Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector1: ...

@final
class I32Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I32Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I32Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> I32Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> I32Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[I32Vector1]: ...

@final
class U32Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __add__(self, other: Number) -> U32Vector1: ...
    @overload
    def __radd__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __radd__(self, other: Number) -> U32Vector1: ...
    @overload
    def __sub__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __sub__(self, other: Number) -> U32Vector1: ...
    @overload
    def __rsub__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> U32Vector1: ...
    @overload
    def __mul__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __mul__(self, other: Number) -> U32Vector1: ...
    @overload
    def __rmul__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> U32Vector1: ...
    @overload
    def __truediv__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> U32Vector1: ...
    @overload
    def __rtruediv__(self, other: U32Vector1) -> U32Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> U32Vector1: ...
    def __abs__(self) -> U32Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> U32Vector2: ...
    @property
    def xo(self) -> U32Vector2: ...
    @property
    def xl(self) -> U32Vector2: ...
    @property
    def ox(self) -> U32Vector2: ...
    @property
    def lx(self) -> U32Vector2: ...
    @property
    def xxx(self) -> U32Vector3: ...
    @property
    def xxo(self) -> U32Vector3: ...
    @property
    def xxl(self) -> U32Vector3: ...
    @property
    def xox(self) -> U32Vector3: ...
    @property
    def xoo(self) -> U32Vector3: ...
    @property
    def xol(self) -> U32Vector3: ...
    @property
    def xlx(self) -> U32Vector3: ...
    @property
    def xlo(self) -> U32Vector3: ...
    @property
    def xll(self) -> U32Vector3: ...
    @property
    def oxx(self) -> U32Vector3: ...
    @property
    def oxo(self) -> U32Vector3: ...
    @property
    def oxl(self) -> U32Vector3: ...
    @property
    def oox(self) -> U32Vector3: ...
    @property
    def olx(self) -> U32Vector3: ...
    @property
    def lxx(self) -> U32Vector3: ...
    @property
    def lxo(self) -> U32Vector3: ...
    @property
    def lxl(self) -> U32Vector3: ...
    @property
    def lox(self) -> U32Vector3: ...
    @property
    def llx(self) -> U32Vector3: ...
    @property
    def xxxx(self) -> U32Vector4: ...
    @property
    def xxxo(self) -> U32Vector4: ...
    @property
    def xxxl(self) -> U32Vector4: ...
    @property
    def xxox(self) -> U32Vector4: ...
    @property
    def xxoo(self) -> U32Vector4: ...
    @property
    def xxol(self) -> U32Vector4: ...
    @property
    def xxlx(self) -> U32Vector4: ...
    @property
    def xxlo(self) -> U32Vector4: ...
    @property
    def xxll(self) -> U32Vector4: ...
    @property
    def xoxx(self) -> U32Vector4: ...
    @property
    def xoxo(self) -> U32Vector4: ...
    @property
    def xoxl(self) -> U32Vector4: ...
    @property
    def xoox(self) -> U32Vector4: ...
    @property
    def xooo(self) -> U32Vector4: ...
    @property
    def xool(self) -> U32Vector4: ...
    @property
    def xolx(self) -> U32Vector4: ...
    @property
    def xolo(self) -> U32Vector4: ...
    @property
    def xoll(self) -> U32Vector4: ...
    @property
    def xlxx(self) -> U32Vector4: ...
    @property
    def xlxo(self) -> U32Vector4: ...
    @property
    def xlxl(self) -> U32Vector4: ...
    @property
    def xlox(self) -> U32Vector4: ...
    @property
    def xloo(self) -> U32Vector4: ...
    @property
    def xlol(self) -> U32Vector4: ...
    @property
    def xllx(self) -> U32Vector4: ...
    @property
    def xllo(self) -> U32Vector4: ...
    @property
    def xlll(self) -> U32Vector4: ...
    @property
    def oxxx(self) -> U32Vector4: ...
    @property
    def oxxo(self) -> U32Vector4: ...
    @property
    def oxxl(self) -> U32Vector4: ...
    @property
    def oxox(self) -> U32Vector4: ...
    @property
    def oxoo(self) -> U32Vector4: ...
    @property
    def oxol(self) -> U32Vector4: ...
    @property
    def oxlx(self) -> U32Vector4: ...
    @property
    def oxlo(self) -> U32Vector4: ...
    @property
    def oxll(self) -> U32Vector4: ...
    @property
    def ooxx(self) -> U32Vector4: ...
    @property
    def ooxo(self) -> U32Vector4: ...
    @property
    def ooxl(self) -> U32Vector4: ...
    @property
    def ooox(self) -> U32Vector4: ...
    @property
    def oolx(self) -> U32Vector4: ...
    @property
    def olxx(self) -> U32Vector4: ...
    @property
    def olxo(self) -> U32Vector4: ...
    @property
    def olxl(self) -> U32Vector4: ...
    @property
    def olox(self) -> U32Vector4: ...
    @property
    def ollx(self) -> U32Vector4: ...
    @property
    def lxxx(self) -> U32Vector4: ...
    @property
    def lxxo(self) -> U32Vector4: ...
    @property
    def lxxl(self) -> U32Vector4: ...
    @property
    def lxox(self) -> U32Vector4: ...
    @property
    def lxoo(self) -> U32Vector4: ...
    @property
    def lxol(self) -> U32Vector4: ...
    @property
    def lxlx(self) -> U32Vector4: ...
    @property
    def lxlo(self) -> U32Vector4: ...
    @property
    def lxll(self) -> U32Vector4: ...
    @property
    def loxx(self) -> U32Vector4: ...
    @property
    def loxo(self) -> U32Vector4: ...
    @property
    def loxl(self) -> U32Vector4: ...
    @property
    def loox(self) -> U32Vector4: ...
    @property
    def lolx(self) -> U32Vector4: ...
    @property
    def llxx(self) -> U32Vector4: ...
    @property
    def llxo(self) -> U32Vector4: ...
    @property
    def llxl(self) -> U32Vector4: ...
    @property
    def llox(self) -> U32Vector4: ...
    @property
    def lllx(self) -> U32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> U32Vector2: ...
    @property
    def ro(self) -> U32Vector2: ...
    @property
    def rl(self) -> U32Vector2: ...
    @property
    def lr(self) -> U32Vector2: ...
    @property
    def rrr(self) -> U32Vector3: ...
    @property
    def rro(self) -> U32Vector3: ...
    @property
    def rrl(self) -> U32Vector3: ...
    @property
    def ror(self) -> U32Vector3: ...
    @property
    def roo(self) -> U32Vector3: ...
    @property
    def rol(self) -> U32Vector3: ...
    @property
    def rlr(self) -> U32Vector3: ...
    @property
    def rlo(self) -> U32Vector3: ...
    @property
    def rll(self) -> U32Vector3: ...
    @property
    def orr(self) -> U32Vector3: ...
    @property
    def oro(self) -> U32Vector3: ...
    @property
    def orl(self) -> U32Vector3: ...
    @property
    def oor(self) -> U32Vector3: ...
    @property
    def olr(self) -> U32Vector3: ...
    @property
    def lrr(self) -> U32Vector3: ...
    @property
    def lro(self) -> U32Vector3: ...
    @property
    def lrl(self) -> U32Vector3: ...
    @property
    def lor(self) -> U32Vector3: ...
    @property
    def llr(self) -> U32Vector3: ...
    @property
    def rrrr(self) -> U32Vector4: ...
    @property
    def rrro(self) -> U32Vector4: ...
    @property
    def rrrl(self) -> U32Vector4: ...
    @property
    def rror(self) -> U32Vector4: ...
    @property
    def rroo(self) -> U32Vector4: ...
    @property
    def rrol(self) -> U32Vector4: ...
    @property
    def rrlr(self) -> U32Vector4: ...
    @property
    def rrlo(self) -> U32Vector4: ...
    @property
    def rrll(self) -> U32Vector4: ...
    @property
    def rorr(self) -> U32Vector4: ...
    @property
    def roro(self) -> U32Vector4: ...
    @property
    def rorl(self) -> U32Vector4: ...
    @property
    def roor(self) -> U32Vector4: ...
    @property
    def rooo(self) -> U32Vector4: ...
    @property
    def rool(self) -> U32Vector4: ...
    @property
    def rolr(self) -> U32Vector4: ...
    @property
    def rolo(self) -> U32Vector4: ...
    @property
    def roll(self) -> U32Vector4: ...
    @property
    def rlrr(self) -> U32Vector4: ...
    @property
    def rlro(self) -> U32Vector4: ...
    @property
    def rlrl(self) -> U32Vector4: ...
    @property
    def rlor(self) -> U32Vector4: ...
    @property
    def rloo(self) -> U32Vector4: ...
    @property
    def rlol(self) -> U32Vector4: ...
    @property
    def rllr(self) -> U32Vector4: ...
    @property
    def rllo(self) -> U32Vector4: ...
    @property
    def rlll(self) -> U32Vector4: ...
    @property
    def orrr(self) -> U32Vector4: ...
    @property
    def orro(self) -> U32Vector4: ...
    @property
    def orrl(self) -> U32Vector4: ...
    @property
    def oror(self) -> U32Vector4: ...
    @property
    def oroo(self) -> U32Vector4: ...
    @property
    def orol(self) -> U32Vector4: ...
    @property
    def orlr(self) -> U32Vector4: ...
    @property
    def orlo(self) -> U32Vector4: ...
    @property
    def orll(self) -> U32Vector4: ...
    @property
    def oorr(self) -> U32Vector4: ...
    @property
    def ooro(self) -> U32Vector4: ...
    @property
    def oorl(self) -> U32Vector4: ...
    @property
    def ooor(self) -> U32Vector4: ...
    @property
    def oolr(self) -> U32Vector4: ...
    @property
    def olrr(self) -> U32Vector4: ...
    @property
    def olro(self) -> U32Vector4: ...
    @property
    def olrl(self) -> U32Vector4: ...
    @property
    def olor(self) -> U32Vector4: ...
    @property
    def ollr(self) -> U32Vector4: ...
    @property
    def lrrr(self) -> U32Vector4: ...
    @property
    def lrro(self) -> U32Vector4: ...
    @property
    def lrrl(self) -> U32Vector4: ...
    @property
    def lror(self) -> U32Vector4: ...
    @property
    def lroo(self) -> U32Vector4: ...
    @property
    def lrol(self) -> U32Vector4: ...
    @property
    def lrlr(self) -> U32Vector4: ...
    @property
    def lrlo(self) -> U32Vector4: ...
    @property
    def lrll(self) -> U32Vector4: ...
    @property
    def lorr(self) -> U32Vector4: ...
    @property
    def loro(self) -> U32Vector4: ...
    @property
    def lorl(self) -> U32Vector4: ...
    @property
    def loor(self) -> U32Vector4: ...
    @property
    def lolr(self) -> U32Vector4: ...
    @property
    def llrr(self) -> U32Vector4: ...
    @property
    def llro(self) -> U32Vector4: ...
    @property
    def llrl(self) -> U32Vector4: ...
    @property
    def llor(self) -> U32Vector4: ...
    @property
    def lllr(self) -> U32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> U32Vector2: ...
    @property
    def so(self) -> U32Vector2: ...
    @property
    def sl(self) -> U32Vector2: ...
    @property
    def os(self) -> U32Vector2: ...
    @property
    def ls(self) -> U32Vector2: ...
    @property
    def sss(self) -> U32Vector3: ...
    @property
    def sso(self) -> U32Vector3: ...
    @property
    def ssl(self) -> U32Vector3: ...
    @property
    def sos(self) -> U32Vector3: ...
    @property
    def soo(self) -> U32Vector3: ...
    @property
    def sol(self) -> U32Vector3: ...
    @property
    def sls(self) -> U32Vector3: ...
    @property
    def slo(self) -> U32Vector3: ...
    @property
    def sll(self) -> U32Vector3: ...
    @property
    def oss(self) -> U32Vector3: ...
    @property
    def oso(self) -> U32Vector3: ...
    @property
    def osl(self) -> U32Vector3: ...
    @property
    def oos(self) -> U32Vector3: ...
    @property
    def ols(self) -> U32Vector3: ...
    @property
    def lss(self) -> U32Vector3: ...
    @property
    def lso(self) -> U32Vector3: ...
    @property
    def lsl(self) -> U32Vector3: ...
    @property
    def los(self) -> U32Vector3: ...
    @property
    def lls(self) -> U32Vector3: ...
    @property
    def ssss(self) -> U32Vector4: ...
    @property
    def ssso(self) -> U32Vector4: ...
    @property
    def sssl(self) -> U32Vector4: ...
    @property
    def ssos(self) -> U32Vector4: ...
    @property
    def ssoo(self) -> U32Vector4: ...
    @property
    def ssol(self) -> U32Vector4: ...
    @property
    def ssls(self) -> U32Vector4: ...
    @property
    def sslo(self) -> U32Vector4: ...
    @property
    def ssll(self) -> U32Vector4: ...
    @property
    def soss(self) -> U32Vector4: ...
    @property
    def soso(self) -> U32Vector4: ...
    @property
    def sosl(self) -> U32Vector4: ...
    @property
    def soos(self) -> U32Vector4: ...
    @property
    def sooo(self) -> U32Vector4: ...
    @property
    def sool(self) -> U32Vector4: ...
    @property
    def sols(self) -> U32Vector4: ...
    @property
    def solo(self) -> U32Vector4: ...
    @property
    def soll(self) -> U32Vector4: ...
    @property
    def slss(self) -> U32Vector4: ...
    @property
    def slso(self) -> U32Vector4: ...
    @property
    def slsl(self) -> U32Vector4: ...
    @property
    def slos(self) -> U32Vector4: ...
    @property
    def sloo(self) -> U32Vector4: ...
    @property
    def slol(self) -> U32Vector4: ...
    @property
    def slls(self) -> U32Vector4: ...
    @property
    def sllo(self) -> U32Vector4: ...
    @property
    def slll(self) -> U32Vector4: ...
    @property
    def osss(self) -> U32Vector4: ...
    @property
    def osso(self) -> U32Vector4: ...
    @property
    def ossl(self) -> U32Vector4: ...
    @property
    def osos(self) -> U32Vector4: ...
    @property
    def osoo(self) -> U32Vector4: ...
    @property
    def osol(self) -> U32Vector4: ...
    @property
    def osls(self) -> U32Vector4: ...
    @property
    def oslo(self) -> U32Vector4: ...
    @property
    def osll(self) -> U32Vector4: ...
    @property
    def ooss(self) -> U32Vector4: ...
    @property
    def ooso(self) -> U32Vector4: ...
    @property
    def oosl(self) -> U32Vector4: ...
    @property
    def ooos(self) -> U32Vector4: ...
    @property
    def ools(self) -> U32Vector4: ...
    @property
    def olss(self) -> U32Vector4: ...
    @property
    def olso(self) -> U32Vector4: ...
    @property
    def olsl(self) -> U32Vector4: ...
    @property
    def olos(self) -> U32Vector4: ...
    @property
    def olls(self) -> U32Vector4: ...
    @property
    def lsss(self) -> U32Vector4: ...
    @property
    def lsso(self) -> U32Vector4: ...
    @property
    def lssl(self) -> U32Vector4: ...
    @property
    def lsos(self) -> U32Vector4: ...
    @property
    def lsoo(self) -> U32Vector4: ...
    @property
    def lsol(self) -> U32Vector4: ...
    @property
    def lsls(self) -> U32Vector4: ...
    @property
    def lslo(self) -> U32Vector4: ...
    @property
    def lsll(self) -> U32Vector4: ...
    @property
    def loss(self) -> U32Vector4: ...
    @property
    def loso(self) -> U32Vector4: ...
    @property
    def losl(self) -> U32Vector4: ...
    @property
    def loos(self) -> U32Vector4: ...
    @property
    def lols(self) -> U32Vector4: ...
    @property
    def llss(self) -> U32Vector4: ...
    @property
    def llso(self) -> U32Vector4: ...
    @property
    def llsl(self) -> U32Vector4: ...
    @property
    def llos(self) -> U32Vector4: ...
    @property
    def llls(self) -> U32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> U32Vector2: ...
    @property
    def uo(self) -> U32Vector2: ...
    @property
    def ul(self) -> U32Vector2: ...
    @property
    def ou(self) -> U32Vector2: ...
    @property
    def lu(self) -> U32Vector2: ...
    @property
    def uuu(self) -> U32Vector3: ...
    @property
    def uuo(self) -> U32Vector3: ...
    @property
    def uul(self) -> U32Vector3: ...
    @property
    def uou(self) -> U32Vector3: ...
    @property
    def uoo(self) -> U32Vector3: ...
    @property
    def uol(self) -> U32Vector3: ...
    @property
    def ulu(self) -> U32Vector3: ...
    @property
    def ulo(self) -> U32Vector3: ...
    @property
    def ull(self) -> U32Vector3: ...
    @property
    def ouu(self) -> U32Vector3: ...
    @property
    def ouo(self) -> U32Vector3: ...
    @property
    def oul(self) -> U32Vector3: ...
    @property
    def oou(self) -> U32Vector3: ...
    @property
    def olu(self) -> U32Vector3: ...
    @property
    def luu(self) -> U32Vector3: ...
    @property
    def luo(self) -> U32Vector3: ...
    @property
    def lul(self) -> U32Vector3: ...
    @property
    def lou(self) -> U32Vector3: ...
    @property
    def llu(self) -> U32Vector3: ...
    @property
    def uuuu(self) -> U32Vector4: ...
    @property
    def uuuo(self) -> U32Vector4: ...
    @property
    def uuul(self) -> U32Vector4: ...
    @property
    def uuou(self) -> U32Vector4: ...
    @property
    def uuoo(self) -> U32Vector4: ...
    @property
    def uuol(self) -> U32Vector4: ...
    @property
    def uulu(self) -> U32Vector4: ...
    @property
    def uulo(self) -> U32Vector4: ...
    @property
    def uull(self) -> U32Vector4: ...
    @property
    def uouu(self) -> U32Vector4: ...
    @property
    def uouo(self) -> U32Vector4: ...
    @property
    def uoul(self) -> U32Vector4: ...
    @property
    def uoou(self) -> U32Vector4: ...
    @property
    def uooo(self) -> U32Vector4: ...
    @property
    def uool(self) -> U32Vector4: ...
    @property
    def uolu(self) -> U32Vector4: ...
    @property
    def uolo(self) -> U32Vector4: ...
    @property
    def uoll(self) -> U32Vector4: ...
    @property
    def uluu(self) -> U32Vector4: ...
    @property
    def uluo(self) -> U32Vector4: ...
    @property
    def ulul(self) -> U32Vector4: ...
    @property
    def ulou(self) -> U32Vector4: ...
    @property
    def uloo(self) -> U32Vector4: ...
    @property
    def ulol(self) -> U32Vector4: ...
    @property
    def ullu(self) -> U32Vector4: ...
    @property
    def ullo(self) -> U32Vector4: ...
    @property
    def ulll(self) -> U32Vector4: ...
    @property
    def ouuu(self) -> U32Vector4: ...
    @property
    def ouuo(self) -> U32Vector4: ...
    @property
    def ouul(self) -> U32Vector4: ...
    @property
    def ouou(self) -> U32Vector4: ...
    @property
    def ouoo(self) -> U32Vector4: ...
    @property
    def ouol(self) -> U32Vector4: ...
    @property
    def oulu(self) -> U32Vector4: ...
    @property
    def oulo(self) -> U32Vector4: ...
    @property
    def oull(self) -> U32Vector4: ...
    @property
    def oouu(self) -> U32Vector4: ...
    @property
    def oouo(self) -> U32Vector4: ...
    @property
    def ooul(self) -> U32Vector4: ...
    @property
    def ooou(self) -> U32Vector4: ...
    @property
    def oolu(self) -> U32Vector4: ...
    @property
    def oluu(self) -> U32Vector4: ...
    @property
    def oluo(self) -> U32Vector4: ...
    @property
    def olul(self) -> U32Vector4: ...
    @property
    def olou(self) -> U32Vector4: ...
    @property
    def ollu(self) -> U32Vector4: ...
    @property
    def luuu(self) -> U32Vector4: ...
    @property
    def luuo(self) -> U32Vector4: ...
    @property
    def luul(self) -> U32Vector4: ...
    @property
    def luou(self) -> U32Vector4: ...
    @property
    def luoo(self) -> U32Vector4: ...
    @property
    def luol(self) -> U32Vector4: ...
    @property
    def lulu(self) -> U32Vector4: ...
    @property
    def lulo(self) -> U32Vector4: ...
    @property
    def lull(self) -> U32Vector4: ...
    @property
    def louu(self) -> U32Vector4: ...
    @property
    def louo(self) -> U32Vector4: ...
    @property
    def loul(self) -> U32Vector4: ...
    @property
    def loou(self) -> U32Vector4: ...
    @property
    def lolu(self) -> U32Vector4: ...
    @property
    def lluu(self) -> U32Vector4: ...
    @property
    def lluo(self) -> U32Vector4: ...
    @property
    def llul(self) -> U32Vector4: ...
    @property
    def llou(self) -> U32Vector4: ...
    @property
    def lllu(self) -> U32Vector4: ...
    @property
    def oo(self) -> U32Vector2: ...
    @property
    def ol(self) -> U32Vector2: ...
    @property
    def lo(self) -> U32Vector2: ...
    @property
    def ll(self) -> U32Vector2: ...
    @property
    def ooo(self) -> U32Vector3: ...
    @property
    def ool(self) -> U32Vector3: ...
    @property
    def olo(self) -> U32Vector3: ...
    @property
    def oll(self) -> U32Vector3: ...
    @property
    def loo(self) -> U32Vector3: ...
    @property
    def lol(self) -> U32Vector3: ...
    @property
    def llo(self) -> U32Vector3: ...
    @property
    def lll(self) -> U32Vector3: ...
    @property
    def oooo(self) -> U32Vector4: ...
    @property
    def oool(self) -> U32Vector4: ...
    @property
    def oolo(self) -> U32Vector4: ...
    @property
    def ooll(self) -> U32Vector4: ...
    @property
    def oloo(self) -> U32Vector4: ...
    @property
    def olol(self) -> U32Vector4: ...
    @property
    def ollo(self) -> U32Vector4: ...
    @property
    def olll(self) -> U32Vector4: ...
    @property
    def looo(self) -> U32Vector4: ...
    @property
    def lool(self) -> U32Vector4: ...
    @property
    def lolo(self) -> U32Vector4: ...
    @property
    def loll(self) -> U32Vector4: ...
    @property
    def lloo(self) -> U32Vector4: ...
    @property
    def llol(self) -> U32Vector4: ...
    @property
    def lllo(self) -> U32Vector4: ...
    @property
    def llll(self) -> U32Vector4: ...
    def min(self, n: Number, /) -> U32Vector1: ...
    def max(self, n: Number, /) -> U32Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> U32Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U32Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector1: ...

@final
class U32Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U32Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U32Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> U32Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> U32Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[U32Vector1]: ...

@final
class IVector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: IVector1) -> IVector1: ...
    @overload
    def __add__(self, other: Number) -> IVector1: ...
    @overload
    def __radd__(self, other: IVector1) -> IVector1: ...
    @overload
    def __radd__(self, other: Number) -> IVector1: ...
    @overload
    def __sub__(self, other: IVector1) -> IVector1: ...
    @overload
    def __sub__(self, other: Number) -> IVector1: ...
    @overload
    def __rsub__(self, other: IVector1) -> IVector1: ...
    @overload
    def __rsub__(self, other: Number) -> IVector1: ...
    @overload
    def __mul__(self, other: IVector1) -> IVector1: ...
    @overload
    def __mul__(self, other: Number) -> IVector1: ...
    @overload
    def __rmul__(self, other: IVector1) -> IVector1: ...
    @overload
    def __rmul__(self, other: Number) -> IVector1: ...
    @overload
    def __truediv__(self, other: IVector1) -> IVector1: ...
    @overload
    def __truediv__(self, other: Number) -> IVector1: ...
    @overload
    def __rtruediv__(self, other: IVector1) -> IVector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> IVector1: ...
    def __neg__(self) -> IVector1: ...
    def __abs__(self) -> IVector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> IVector2: ...
    @property
    def xo(self) -> IVector2: ...
    @property
    def xl(self) -> IVector2: ...
    @property
    def ox(self) -> IVector2: ...
    @property
    def lx(self) -> IVector2: ...
    @property
    def xxx(self) -> IVector3: ...
    @property
    def xxo(self) -> IVector3: ...
    @property
    def xxl(self) -> IVector3: ...
    @property
    def xox(self) -> IVector3: ...
    @property
    def xoo(self) -> IVector3: ...
    @property
    def xol(self) -> IVector3: ...
    @property
    def xlx(self) -> IVector3: ...
    @property
    def xlo(self) -> IVector3: ...
    @property
    def xll(self) -> IVector3: ...
    @property
    def oxx(self) -> IVector3: ...
    @property
    def oxo(self) -> IVector3: ...
    @property
    def oxl(self) -> IVector3: ...
    @property
    def oox(self) -> IVector3: ...
    @property
    def olx(self) -> IVector3: ...
    @property
    def lxx(self) -> IVector3: ...
    @property
    def lxo(self) -> IVector3: ...
    @property
    def lxl(self) -> IVector3: ...
    @property
    def lox(self) -> IVector3: ...
    @property
    def llx(self) -> IVector3: ...
    @property
    def xxxx(self) -> IVector4: ...
    @property
    def xxxo(self) -> IVector4: ...
    @property
    def xxxl(self) -> IVector4: ...
    @property
    def xxox(self) -> IVector4: ...
    @property
    def xxoo(self) -> IVector4: ...
    @property
    def xxol(self) -> IVector4: ...
    @property
    def xxlx(self) -> IVector4: ...
    @property
    def xxlo(self) -> IVector4: ...
    @property
    def xxll(self) -> IVector4: ...
    @property
    def xoxx(self) -> IVector4: ...
    @property
    def xoxo(self) -> IVector4: ...
    @property
    def xoxl(self) -> IVector4: ...
    @property
    def xoox(self) -> IVector4: ...
    @property
    def xooo(self) -> IVector4: ...
    @property
    def xool(self) -> IVector4: ...
    @property
    def xolx(self) -> IVector4: ...
    @property
    def xolo(self) -> IVector4: ...
    @property
    def xoll(self) -> IVector4: ...
    @property
    def xlxx(self) -> IVector4: ...
    @property
    def xlxo(self) -> IVector4: ...
    @property
    def xlxl(self) -> IVector4: ...
    @property
    def xlox(self) -> IVector4: ...
    @property
    def xloo(self) -> IVector4: ...
    @property
    def xlol(self) -> IVector4: ...
    @property
    def xllx(self) -> IVector4: ...
    @property
    def xllo(self) -> IVector4: ...
    @property
    def xlll(self) -> IVector4: ...
    @property
    def oxxx(self) -> IVector4: ...
    @property
    def oxxo(self) -> IVector4: ...
    @property
    def oxxl(self) -> IVector4: ...
    @property
    def oxox(self) -> IVector4: ...
    @property
    def oxoo(self) -> IVector4: ...
    @property
    def oxol(self) -> IVector4: ...
    @property
    def oxlx(self) -> IVector4: ...
    @property
    def oxlo(self) -> IVector4: ...
    @property
    def oxll(self) -> IVector4: ...
    @property
    def ooxx(self) -> IVector4: ...
    @property
    def ooxo(self) -> IVector4: ...
    @property
    def ooxl(self) -> IVector4: ...
    @property
    def ooox(self) -> IVector4: ...
    @property
    def oolx(self) -> IVector4: ...
    @property
    def olxx(self) -> IVector4: ...
    @property
    def olxo(self) -> IVector4: ...
    @property
    def olxl(self) -> IVector4: ...
    @property
    def olox(self) -> IVector4: ...
    @property
    def ollx(self) -> IVector4: ...
    @property
    def lxxx(self) -> IVector4: ...
    @property
    def lxxo(self) -> IVector4: ...
    @property
    def lxxl(self) -> IVector4: ...
    @property
    def lxox(self) -> IVector4: ...
    @property
    def lxoo(self) -> IVector4: ...
    @property
    def lxol(self) -> IVector4: ...
    @property
    def lxlx(self) -> IVector4: ...
    @property
    def lxlo(self) -> IVector4: ...
    @property
    def lxll(self) -> IVector4: ...
    @property
    def loxx(self) -> IVector4: ...
    @property
    def loxo(self) -> IVector4: ...
    @property
    def loxl(self) -> IVector4: ...
    @property
    def loox(self) -> IVector4: ...
    @property
    def lolx(self) -> IVector4: ...
    @property
    def llxx(self) -> IVector4: ...
    @property
    def llxo(self) -> IVector4: ...
    @property
    def llxl(self) -> IVector4: ...
    @property
    def llox(self) -> IVector4: ...
    @property
    def lllx(self) -> IVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> IVector2: ...
    @property
    def ro(self) -> IVector2: ...
    @property
    def rl(self) -> IVector2: ...
    @property
    def lr(self) -> IVector2: ...
    @property
    def rrr(self) -> IVector3: ...
    @property
    def rro(self) -> IVector3: ...
    @property
    def rrl(self) -> IVector3: ...
    @property
    def ror(self) -> IVector3: ...
    @property
    def roo(self) -> IVector3: ...
    @property
    def rol(self) -> IVector3: ...
    @property
    def rlr(self) -> IVector3: ...
    @property
    def rlo(self) -> IVector3: ...
    @property
    def rll(self) -> IVector3: ...
    @property
    def orr(self) -> IVector3: ...
    @property
    def oro(self) -> IVector3: ...
    @property
    def orl(self) -> IVector3: ...
    @property
    def oor(self) -> IVector3: ...
    @property
    def olr(self) -> IVector3: ...
    @property
    def lrr(self) -> IVector3: ...
    @property
    def lro(self) -> IVector3: ...
    @property
    def lrl(self) -> IVector3: ...
    @property
    def lor(self) -> IVector3: ...
    @property
    def llr(self) -> IVector3: ...
    @property
    def rrrr(self) -> IVector4: ...
    @property
    def rrro(self) -> IVector4: ...
    @property
    def rrrl(self) -> IVector4: ...
    @property
    def rror(self) -> IVector4: ...
    @property
    def rroo(self) -> IVector4: ...
    @property
    def rrol(self) -> IVector4: ...
    @property
    def rrlr(self) -> IVector4: ...
    @property
    def rrlo(self) -> IVector4: ...
    @property
    def rrll(self) -> IVector4: ...
    @property
    def rorr(self) -> IVector4: ...
    @property
    def roro(self) -> IVector4: ...
    @property
    def rorl(self) -> IVector4: ...
    @property
    def roor(self) -> IVector4: ...
    @property
    def rooo(self) -> IVector4: ...
    @property
    def rool(self) -> IVector4: ...
    @property
    def rolr(self) -> IVector4: ...
    @property
    def rolo(self) -> IVector4: ...
    @property
    def roll(self) -> IVector4: ...
    @property
    def rlrr(self) -> IVector4: ...
    @property
    def rlro(self) -> IVector4: ...
    @property
    def rlrl(self) -> IVector4: ...
    @property
    def rlor(self) -> IVector4: ...
    @property
    def rloo(self) -> IVector4: ...
    @property
    def rlol(self) -> IVector4: ...
    @property
    def rllr(self) -> IVector4: ...
    @property
    def rllo(self) -> IVector4: ...
    @property
    def rlll(self) -> IVector4: ...
    @property
    def orrr(self) -> IVector4: ...
    @property
    def orro(self) -> IVector4: ...
    @property
    def orrl(self) -> IVector4: ...
    @property
    def oror(self) -> IVector4: ...
    @property
    def oroo(self) -> IVector4: ...
    @property
    def orol(self) -> IVector4: ...
    @property
    def orlr(self) -> IVector4: ...
    @property
    def orlo(self) -> IVector4: ...
    @property
    def orll(self) -> IVector4: ...
    @property
    def oorr(self) -> IVector4: ...
    @property
    def ooro(self) -> IVector4: ...
    @property
    def oorl(self) -> IVector4: ...
    @property
    def ooor(self) -> IVector4: ...
    @property
    def oolr(self) -> IVector4: ...
    @property
    def olrr(self) -> IVector4: ...
    @property
    def olro(self) -> IVector4: ...
    @property
    def olrl(self) -> IVector4: ...
    @property
    def olor(self) -> IVector4: ...
    @property
    def ollr(self) -> IVector4: ...
    @property
    def lrrr(self) -> IVector4: ...
    @property
    def lrro(self) -> IVector4: ...
    @property
    def lrrl(self) -> IVector4: ...
    @property
    def lror(self) -> IVector4: ...
    @property
    def lroo(self) -> IVector4: ...
    @property
    def lrol(self) -> IVector4: ...
    @property
    def lrlr(self) -> IVector4: ...
    @property
    def lrlo(self) -> IVector4: ...
    @property
    def lrll(self) -> IVector4: ...
    @property
    def lorr(self) -> IVector4: ...
    @property
    def loro(self) -> IVector4: ...
    @property
    def lorl(self) -> IVector4: ...
    @property
    def loor(self) -> IVector4: ...
    @property
    def lolr(self) -> IVector4: ...
    @property
    def llrr(self) -> IVector4: ...
    @property
    def llro(self) -> IVector4: ...
    @property
    def llrl(self) -> IVector4: ...
    @property
    def llor(self) -> IVector4: ...
    @property
    def lllr(self) -> IVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> IVector2: ...
    @property
    def so(self) -> IVector2: ...
    @property
    def sl(self) -> IVector2: ...
    @property
    def os(self) -> IVector2: ...
    @property
    def ls(self) -> IVector2: ...
    @property
    def sss(self) -> IVector3: ...
    @property
    def sso(self) -> IVector3: ...
    @property
    def ssl(self) -> IVector3: ...
    @property
    def sos(self) -> IVector3: ...
    @property
    def soo(self) -> IVector3: ...
    @property
    def sol(self) -> IVector3: ...
    @property
    def sls(self) -> IVector3: ...
    @property
    def slo(self) -> IVector3: ...
    @property
    def sll(self) -> IVector3: ...
    @property
    def oss(self) -> IVector3: ...
    @property
    def oso(self) -> IVector3: ...
    @property
    def osl(self) -> IVector3: ...
    @property
    def oos(self) -> IVector3: ...
    @property
    def ols(self) -> IVector3: ...
    @property
    def lss(self) -> IVector3: ...
    @property
    def lso(self) -> IVector3: ...
    @property
    def lsl(self) -> IVector3: ...
    @property
    def los(self) -> IVector3: ...
    @property
    def lls(self) -> IVector3: ...
    @property
    def ssss(self) -> IVector4: ...
    @property
    def ssso(self) -> IVector4: ...
    @property
    def sssl(self) -> IVector4: ...
    @property
    def ssos(self) -> IVector4: ...
    @property
    def ssoo(self) -> IVector4: ...
    @property
    def ssol(self) -> IVector4: ...
    @property
    def ssls(self) -> IVector4: ...
    @property
    def sslo(self) -> IVector4: ...
    @property
    def ssll(self) -> IVector4: ...
    @property
    def soss(self) -> IVector4: ...
    @property
    def soso(self) -> IVector4: ...
    @property
    def sosl(self) -> IVector4: ...
    @property
    def soos(self) -> IVector4: ...
    @property
    def sooo(self) -> IVector4: ...
    @property
    def sool(self) -> IVector4: ...
    @property
    def sols(self) -> IVector4: ...
    @property
    def solo(self) -> IVector4: ...
    @property
    def soll(self) -> IVector4: ...
    @property
    def slss(self) -> IVector4: ...
    @property
    def slso(self) -> IVector4: ...
    @property
    def slsl(self) -> IVector4: ...
    @property
    def slos(self) -> IVector4: ...
    @property
    def sloo(self) -> IVector4: ...
    @property
    def slol(self) -> IVector4: ...
    @property
    def slls(self) -> IVector4: ...
    @property
    def sllo(self) -> IVector4: ...
    @property
    def slll(self) -> IVector4: ...
    @property
    def osss(self) -> IVector4: ...
    @property
    def osso(self) -> IVector4: ...
    @property
    def ossl(self) -> IVector4: ...
    @property
    def osos(self) -> IVector4: ...
    @property
    def osoo(self) -> IVector4: ...
    @property
    def osol(self) -> IVector4: ...
    @property
    def osls(self) -> IVector4: ...
    @property
    def oslo(self) -> IVector4: ...
    @property
    def osll(self) -> IVector4: ...
    @property
    def ooss(self) -> IVector4: ...
    @property
    def ooso(self) -> IVector4: ...
    @property
    def oosl(self) -> IVector4: ...
    @property
    def ooos(self) -> IVector4: ...
    @property
    def ools(self) -> IVector4: ...
    @property
    def olss(self) -> IVector4: ...
    @property
    def olso(self) -> IVector4: ...
    @property
    def olsl(self) -> IVector4: ...
    @property
    def olos(self) -> IVector4: ...
    @property
    def olls(self) -> IVector4: ...
    @property
    def lsss(self) -> IVector4: ...
    @property
    def lsso(self) -> IVector4: ...
    @property
    def lssl(self) -> IVector4: ...
    @property
    def lsos(self) -> IVector4: ...
    @property
    def lsoo(self) -> IVector4: ...
    @property
    def lsol(self) -> IVector4: ...
    @property
    def lsls(self) -> IVector4: ...
    @property
    def lslo(self) -> IVector4: ...
    @property
    def lsll(self) -> IVector4: ...
    @property
    def loss(self) -> IVector4: ...
    @property
    def loso(self) -> IVector4: ...
    @property
    def losl(self) -> IVector4: ...
    @property
    def loos(self) -> IVector4: ...
    @property
    def lols(self) -> IVector4: ...
    @property
    def llss(self) -> IVector4: ...
    @property
    def llso(self) -> IVector4: ...
    @property
    def llsl(self) -> IVector4: ...
    @property
    def llos(self) -> IVector4: ...
    @property
    def llls(self) -> IVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> IVector2: ...
    @property
    def uo(self) -> IVector2: ...
    @property
    def ul(self) -> IVector2: ...
    @property
    def ou(self) -> IVector2: ...
    @property
    def lu(self) -> IVector2: ...
    @property
    def uuu(self) -> IVector3: ...
    @property
    def uuo(self) -> IVector3: ...
    @property
    def uul(self) -> IVector3: ...
    @property
    def uou(self) -> IVector3: ...
    @property
    def uoo(self) -> IVector3: ...
    @property
    def uol(self) -> IVector3: ...
    @property
    def ulu(self) -> IVector3: ...
    @property
    def ulo(self) -> IVector3: ...
    @property
    def ull(self) -> IVector3: ...
    @property
    def ouu(self) -> IVector3: ...
    @property
    def ouo(self) -> IVector3: ...
    @property
    def oul(self) -> IVector3: ...
    @property
    def oou(self) -> IVector3: ...
    @property
    def olu(self) -> IVector3: ...
    @property
    def luu(self) -> IVector3: ...
    @property
    def luo(self) -> IVector3: ...
    @property
    def lul(self) -> IVector3: ...
    @property
    def lou(self) -> IVector3: ...
    @property
    def llu(self) -> IVector3: ...
    @property
    def uuuu(self) -> IVector4: ...
    @property
    def uuuo(self) -> IVector4: ...
    @property
    def uuul(self) -> IVector4: ...
    @property
    def uuou(self) -> IVector4: ...
    @property
    def uuoo(self) -> IVector4: ...
    @property
    def uuol(self) -> IVector4: ...
    @property
    def uulu(self) -> IVector4: ...
    @property
    def uulo(self) -> IVector4: ...
    @property
    def uull(self) -> IVector4: ...
    @property
    def uouu(self) -> IVector4: ...
    @property
    def uouo(self) -> IVector4: ...
    @property
    def uoul(self) -> IVector4: ...
    @property
    def uoou(self) -> IVector4: ...
    @property
    def uooo(self) -> IVector4: ...
    @property
    def uool(self) -> IVector4: ...
    @property
    def uolu(self) -> IVector4: ...
    @property
    def uolo(self) -> IVector4: ...
    @property
    def uoll(self) -> IVector4: ...
    @property
    def uluu(self) -> IVector4: ...
    @property
    def uluo(self) -> IVector4: ...
    @property
    def ulul(self) -> IVector4: ...
    @property
    def ulou(self) -> IVector4: ...
    @property
    def uloo(self) -> IVector4: ...
    @property
    def ulol(self) -> IVector4: ...
    @property
    def ullu(self) -> IVector4: ...
    @property
    def ullo(self) -> IVector4: ...
    @property
    def ulll(self) -> IVector4: ...
    @property
    def ouuu(self) -> IVector4: ...
    @property
    def ouuo(self) -> IVector4: ...
    @property
    def ouul(self) -> IVector4: ...
    @property
    def ouou(self) -> IVector4: ...
    @property
    def ouoo(self) -> IVector4: ...
    @property
    def ouol(self) -> IVector4: ...
    @property
    def oulu(self) -> IVector4: ...
    @property
    def oulo(self) -> IVector4: ...
    @property
    def oull(self) -> IVector4: ...
    @property
    def oouu(self) -> IVector4: ...
    @property
    def oouo(self) -> IVector4: ...
    @property
    def ooul(self) -> IVector4: ...
    @property
    def ooou(self) -> IVector4: ...
    @property
    def oolu(self) -> IVector4: ...
    @property
    def oluu(self) -> IVector4: ...
    @property
    def oluo(self) -> IVector4: ...
    @property
    def olul(self) -> IVector4: ...
    @property
    def olou(self) -> IVector4: ...
    @property
    def ollu(self) -> IVector4: ...
    @property
    def luuu(self) -> IVector4: ...
    @property
    def luuo(self) -> IVector4: ...
    @property
    def luul(self) -> IVector4: ...
    @property
    def luou(self) -> IVector4: ...
    @property
    def luoo(self) -> IVector4: ...
    @property
    def luol(self) -> IVector4: ...
    @property
    def lulu(self) -> IVector4: ...
    @property
    def lulo(self) -> IVector4: ...
    @property
    def lull(self) -> IVector4: ...
    @property
    def louu(self) -> IVector4: ...
    @property
    def louo(self) -> IVector4: ...
    @property
    def loul(self) -> IVector4: ...
    @property
    def loou(self) -> IVector4: ...
    @property
    def lolu(self) -> IVector4: ...
    @property
    def lluu(self) -> IVector4: ...
    @property
    def lluo(self) -> IVector4: ...
    @property
    def llul(self) -> IVector4: ...
    @property
    def llou(self) -> IVector4: ...
    @property
    def lllu(self) -> IVector4: ...
    @property
    def oo(self) -> IVector2: ...
    @property
    def ol(self) -> IVector2: ...
    @property
    def lo(self) -> IVector2: ...
    @property
    def ll(self) -> IVector2: ...
    @property
    def ooo(self) -> IVector3: ...
    @property
    def ool(self) -> IVector3: ...
    @property
    def olo(self) -> IVector3: ...
    @property
    def oll(self) -> IVector3: ...
    @property
    def loo(self) -> IVector3: ...
    @property
    def lol(self) -> IVector3: ...
    @property
    def llo(self) -> IVector3: ...
    @property
    def lll(self) -> IVector3: ...
    @property
    def oooo(self) -> IVector4: ...
    @property
    def oool(self) -> IVector4: ...
    @property
    def oolo(self) -> IVector4: ...
    @property
    def ooll(self) -> IVector4: ...
    @property
    def oloo(self) -> IVector4: ...
    @property
    def olol(self) -> IVector4: ...
    @property
    def ollo(self) -> IVector4: ...
    @property
    def olll(self) -> IVector4: ...
    @property
    def looo(self) -> IVector4: ...
    @property
    def lool(self) -> IVector4: ...
    @property
    def lolo(self) -> IVector4: ...
    @property
    def loll(self) -> IVector4: ...
    @property
    def lloo(self) -> IVector4: ...
    @property
    def llol(self) -> IVector4: ...
    @property
    def lllo(self) -> IVector4: ...
    @property
    def llll(self) -> IVector4: ...
    def min(self, n: Number, /) -> IVector1: ...
    def max(self, n: Number, /) -> IVector1: ...
    def clamp(self, min: Number, max: Number, /) -> IVector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[IVector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector1: ...

@final
class IVector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: IVector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[IVector1]: ...
    @overload
    def __getitem__(self, index: int) -> IVector1: ...
    @overload
    def __getitem__(self, index: slice) -> IVector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[IVector1]: ...

@final
class UVector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: UVector1) -> UVector1: ...
    @overload
    def __add__(self, other: Number) -> UVector1: ...
    @overload
    def __radd__(self, other: UVector1) -> UVector1: ...
    @overload
    def __radd__(self, other: Number) -> UVector1: ...
    @overload
    def __sub__(self, other: UVector1) -> UVector1: ...
    @overload
    def __sub__(self, other: Number) -> UVector1: ...
    @overload
    def __rsub__(self, other: UVector1) -> UVector1: ...
    @overload
    def __rsub__(self, other: Number) -> UVector1: ...
    @overload
    def __mul__(self, other: UVector1) -> UVector1: ...
    @overload
    def __mul__(self, other: Number) -> UVector1: ...
    @overload
    def __rmul__(self, other: UVector1) -> UVector1: ...
    @overload
    def __rmul__(self, other: Number) -> UVector1: ...
    @overload
    def __truediv__(self, other: UVector1) -> UVector1: ...
    @overload
    def __truediv__(self, other: Number) -> UVector1: ...
    @overload
    def __rtruediv__(self, other: UVector1) -> UVector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> UVector1: ...
    def __abs__(self) -> UVector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> UVector2: ...
    @property
    def xo(self) -> UVector2: ...
    @property
    def xl(self) -> UVector2: ...
    @property
    def ox(self) -> UVector2: ...
    @property
    def lx(self) -> UVector2: ...
    @property
    def xxx(self) -> UVector3: ...
    @property
    def xxo(self) -> UVector3: ...
    @property
    def xxl(self) -> UVector3: ...
    @property
    def xox(self) -> UVector3: ...
    @property
    def xoo(self) -> UVector3: ...
    @property
    def xol(self) -> UVector3: ...
    @property
    def xlx(self) -> UVector3: ...
    @property
    def xlo(self) -> UVector3: ...
    @property
    def xll(self) -> UVector3: ...
    @property
    def oxx(self) -> UVector3: ...
    @property
    def oxo(self) -> UVector3: ...
    @property
    def oxl(self) -> UVector3: ...
    @property
    def oox(self) -> UVector3: ...
    @property
    def olx(self) -> UVector3: ...
    @property
    def lxx(self) -> UVector3: ...
    @property
    def lxo(self) -> UVector3: ...
    @property
    def lxl(self) -> UVector3: ...
    @property
    def lox(self) -> UVector3: ...
    @property
    def llx(self) -> UVector3: ...
    @property
    def xxxx(self) -> UVector4: ...
    @property
    def xxxo(self) -> UVector4: ...
    @property
    def xxxl(self) -> UVector4: ...
    @property
    def xxox(self) -> UVector4: ...
    @property
    def xxoo(self) -> UVector4: ...
    @property
    def xxol(self) -> UVector4: ...
    @property
    def xxlx(self) -> UVector4: ...
    @property
    def xxlo(self) -> UVector4: ...
    @property
    def xxll(self) -> UVector4: ...
    @property
    def xoxx(self) -> UVector4: ...
    @property
    def xoxo(self) -> UVector4: ...
    @property
    def xoxl(self) -> UVector4: ...
    @property
    def xoox(self) -> UVector4: ...
    @property
    def xooo(self) -> UVector4: ...
    @property
    def xool(self) -> UVector4: ...
    @property
    def xolx(self) -> UVector4: ...
    @property
    def xolo(self) -> UVector4: ...
    @property
    def xoll(self) -> UVector4: ...
    @property
    def xlxx(self) -> UVector4: ...
    @property
    def xlxo(self) -> UVector4: ...
    @property
    def xlxl(self) -> UVector4: ...
    @property
    def xlox(self) -> UVector4: ...
    @property
    def xloo(self) -> UVector4: ...
    @property
    def xlol(self) -> UVector4: ...
    @property
    def xllx(self) -> UVector4: ...
    @property
    def xllo(self) -> UVector4: ...
    @property
    def xlll(self) -> UVector4: ...
    @property
    def oxxx(self) -> UVector4: ...
    @property
    def oxxo(self) -> UVector4: ...
    @property
    def oxxl(self) -> UVector4: ...
    @property
    def oxox(self) -> UVector4: ...
    @property
    def oxoo(self) -> UVector4: ...
    @property
    def oxol(self) -> UVector4: ...
    @property
    def oxlx(self) -> UVector4: ...
    @property
    def oxlo(self) -> UVector4: ...
    @property
    def oxll(self) -> UVector4: ...
    @property
    def ooxx(self) -> UVector4: ...
    @property
    def ooxo(self) -> UVector4: ...
    @property
    def ooxl(self) -> UVector4: ...
    @property
    def ooox(self) -> UVector4: ...
    @property
    def oolx(self) -> UVector4: ...
    @property
    def olxx(self) -> UVector4: ...
    @property
    def olxo(self) -> UVector4: ...
    @property
    def olxl(self) -> UVector4: ...
    @property
    def olox(self) -> UVector4: ...
    @property
    def ollx(self) -> UVector4: ...
    @property
    def lxxx(self) -> UVector4: ...
    @property
    def lxxo(self) -> UVector4: ...
    @property
    def lxxl(self) -> UVector4: ...
    @property
    def lxox(self) -> UVector4: ...
    @property
    def lxoo(self) -> UVector4: ...
    @property
    def lxol(self) -> UVector4: ...
    @property
    def lxlx(self) -> UVector4: ...
    @property
    def lxlo(self) -> UVector4: ...
    @property
    def lxll(self) -> UVector4: ...
    @property
    def loxx(self) -> UVector4: ...
    @property
    def loxo(self) -> UVector4: ...
    @property
    def loxl(self) -> UVector4: ...
    @property
    def loox(self) -> UVector4: ...
    @property
    def lolx(self) -> UVector4: ...
    @property
    def llxx(self) -> UVector4: ...
    @property
    def llxo(self) -> UVector4: ...
    @property
    def llxl(self) -> UVector4: ...
    @property
    def llox(self) -> UVector4: ...
    @property
    def lllx(self) -> UVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> UVector2: ...
    @property
    def ro(self) -> UVector2: ...
    @property
    def rl(self) -> UVector2: ...
    @property
    def lr(self) -> UVector2: ...
    @property
    def rrr(self) -> UVector3: ...
    @property
    def rro(self) -> UVector3: ...
    @property
    def rrl(self) -> UVector3: ...
    @property
    def ror(self) -> UVector3: ...
    @property
    def roo(self) -> UVector3: ...
    @property
    def rol(self) -> UVector3: ...
    @property
    def rlr(self) -> UVector3: ...
    @property
    def rlo(self) -> UVector3: ...
    @property
    def rll(self) -> UVector3: ...
    @property
    def orr(self) -> UVector3: ...
    @property
    def oro(self) -> UVector3: ...
    @property
    def orl(self) -> UVector3: ...
    @property
    def oor(self) -> UVector3: ...
    @property
    def olr(self) -> UVector3: ...
    @property
    def lrr(self) -> UVector3: ...
    @property
    def lro(self) -> UVector3: ...
    @property
    def lrl(self) -> UVector3: ...
    @property
    def lor(self) -> UVector3: ...
    @property
    def llr(self) -> UVector3: ...
    @property
    def rrrr(self) -> UVector4: ...
    @property
    def rrro(self) -> UVector4: ...
    @property
    def rrrl(self) -> UVector4: ...
    @property
    def rror(self) -> UVector4: ...
    @property
    def rroo(self) -> UVector4: ...
    @property
    def rrol(self) -> UVector4: ...
    @property
    def rrlr(self) -> UVector4: ...
    @property
    def rrlo(self) -> UVector4: ...
    @property
    def rrll(self) -> UVector4: ...
    @property
    def rorr(self) -> UVector4: ...
    @property
    def roro(self) -> UVector4: ...
    @property
    def rorl(self) -> UVector4: ...
    @property
    def roor(self) -> UVector4: ...
    @property
    def rooo(self) -> UVector4: ...
    @property
    def rool(self) -> UVector4: ...
    @property
    def rolr(self) -> UVector4: ...
    @property
    def rolo(self) -> UVector4: ...
    @property
    def roll(self) -> UVector4: ...
    @property
    def rlrr(self) -> UVector4: ...
    @property
    def rlro(self) -> UVector4: ...
    @property
    def rlrl(self) -> UVector4: ...
    @property
    def rlor(self) -> UVector4: ...
    @property
    def rloo(self) -> UVector4: ...
    @property
    def rlol(self) -> UVector4: ...
    @property
    def rllr(self) -> UVector4: ...
    @property
    def rllo(self) -> UVector4: ...
    @property
    def rlll(self) -> UVector4: ...
    @property
    def orrr(self) -> UVector4: ...
    @property
    def orro(self) -> UVector4: ...
    @property
    def orrl(self) -> UVector4: ...
    @property
    def oror(self) -> UVector4: ...
    @property
    def oroo(self) -> UVector4: ...
    @property
    def orol(self) -> UVector4: ...
    @property
    def orlr(self) -> UVector4: ...
    @property
    def orlo(self) -> UVector4: ...
    @property
    def orll(self) -> UVector4: ...
    @property
    def oorr(self) -> UVector4: ...
    @property
    def ooro(self) -> UVector4: ...
    @property
    def oorl(self) -> UVector4: ...
    @property
    def ooor(self) -> UVector4: ...
    @property
    def oolr(self) -> UVector4: ...
    @property
    def olrr(self) -> UVector4: ...
    @property
    def olro(self) -> UVector4: ...
    @property
    def olrl(self) -> UVector4: ...
    @property
    def olor(self) -> UVector4: ...
    @property
    def ollr(self) -> UVector4: ...
    @property
    def lrrr(self) -> UVector4: ...
    @property
    def lrro(self) -> UVector4: ...
    @property
    def lrrl(self) -> UVector4: ...
    @property
    def lror(self) -> UVector4: ...
    @property
    def lroo(self) -> UVector4: ...
    @property
    def lrol(self) -> UVector4: ...
    @property
    def lrlr(self) -> UVector4: ...
    @property
    def lrlo(self) -> UVector4: ...
    @property
    def lrll(self) -> UVector4: ...
    @property
    def lorr(self) -> UVector4: ...
    @property
    def loro(self) -> UVector4: ...
    @property
    def lorl(self) -> UVector4: ...
    @property
    def loor(self) -> UVector4: ...
    @property
    def lolr(self) -> UVector4: ...
    @property
    def llrr(self) -> UVector4: ...
    @property
    def llro(self) -> UVector4: ...
    @property
    def llrl(self) -> UVector4: ...
    @property
    def llor(self) -> UVector4: ...
    @property
    def lllr(self) -> UVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> UVector2: ...
    @property
    def so(self) -> UVector2: ...
    @property
    def sl(self) -> UVector2: ...
    @property
    def os(self) -> UVector2: ...
    @property
    def ls(self) -> UVector2: ...
    @property
    def sss(self) -> UVector3: ...
    @property
    def sso(self) -> UVector3: ...
    @property
    def ssl(self) -> UVector3: ...
    @property
    def sos(self) -> UVector3: ...
    @property
    def soo(self) -> UVector3: ...
    @property
    def sol(self) -> UVector3: ...
    @property
    def sls(self) -> UVector3: ...
    @property
    def slo(self) -> UVector3: ...
    @property
    def sll(self) -> UVector3: ...
    @property
    def oss(self) -> UVector3: ...
    @property
    def oso(self) -> UVector3: ...
    @property
    def osl(self) -> UVector3: ...
    @property
    def oos(self) -> UVector3: ...
    @property
    def ols(self) -> UVector3: ...
    @property
    def lss(self) -> UVector3: ...
    @property
    def lso(self) -> UVector3: ...
    @property
    def lsl(self) -> UVector3: ...
    @property
    def los(self) -> UVector3: ...
    @property
    def lls(self) -> UVector3: ...
    @property
    def ssss(self) -> UVector4: ...
    @property
    def ssso(self) -> UVector4: ...
    @property
    def sssl(self) -> UVector4: ...
    @property
    def ssos(self) -> UVector4: ...
    @property
    def ssoo(self) -> UVector4: ...
    @property
    def ssol(self) -> UVector4: ...
    @property
    def ssls(self) -> UVector4: ...
    @property
    def sslo(self) -> UVector4: ...
    @property
    def ssll(self) -> UVector4: ...
    @property
    def soss(self) -> UVector4: ...
    @property
    def soso(self) -> UVector4: ...
    @property
    def sosl(self) -> UVector4: ...
    @property
    def soos(self) -> UVector4: ...
    @property
    def sooo(self) -> UVector4: ...
    @property
    def sool(self) -> UVector4: ...
    @property
    def sols(self) -> UVector4: ...
    @property
    def solo(self) -> UVector4: ...
    @property
    def soll(self) -> UVector4: ...
    @property
    def slss(self) -> UVector4: ...
    @property
    def slso(self) -> UVector4: ...
    @property
    def slsl(self) -> UVector4: ...
    @property
    def slos(self) -> UVector4: ...
    @property
    def sloo(self) -> UVector4: ...
    @property
    def slol(self) -> UVector4: ...
    @property
    def slls(self) -> UVector4: ...
    @property
    def sllo(self) -> UVector4: ...
    @property
    def slll(self) -> UVector4: ...
    @property
    def osss(self) -> UVector4: ...
    @property
    def osso(self) -> UVector4: ...
    @property
    def ossl(self) -> UVector4: ...
    @property
    def osos(self) -> UVector4: ...
    @property
    def osoo(self) -> UVector4: ...
    @property
    def osol(self) -> UVector4: ...
    @property
    def osls(self) -> UVector4: ...
    @property
    def oslo(self) -> UVector4: ...
    @property
    def osll(self) -> UVector4: ...
    @property
    def ooss(self) -> UVector4: ...
    @property
    def ooso(self) -> UVector4: ...
    @property
    def oosl(self) -> UVector4: ...
    @property
    def ooos(self) -> UVector4: ...
    @property
    def ools(self) -> UVector4: ...
    @property
    def olss(self) -> UVector4: ...
    @property
    def olso(self) -> UVector4: ...
    @property
    def olsl(self) -> UVector4: ...
    @property
    def olos(self) -> UVector4: ...
    @property
    def olls(self) -> UVector4: ...
    @property
    def lsss(self) -> UVector4: ...
    @property
    def lsso(self) -> UVector4: ...
    @property
    def lssl(self) -> UVector4: ...
    @property
    def lsos(self) -> UVector4: ...
    @property
    def lsoo(self) -> UVector4: ...
    @property
    def lsol(self) -> UVector4: ...
    @property
    def lsls(self) -> UVector4: ...
    @property
    def lslo(self) -> UVector4: ...
    @property
    def lsll(self) -> UVector4: ...
    @property
    def loss(self) -> UVector4: ...
    @property
    def loso(self) -> UVector4: ...
    @property
    def losl(self) -> UVector4: ...
    @property
    def loos(self) -> UVector4: ...
    @property
    def lols(self) -> UVector4: ...
    @property
    def llss(self) -> UVector4: ...
    @property
    def llso(self) -> UVector4: ...
    @property
    def llsl(self) -> UVector4: ...
    @property
    def llos(self) -> UVector4: ...
    @property
    def llls(self) -> UVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> UVector2: ...
    @property
    def uo(self) -> UVector2: ...
    @property
    def ul(self) -> UVector2: ...
    @property
    def ou(self) -> UVector2: ...
    @property
    def lu(self) -> UVector2: ...
    @property
    def uuu(self) -> UVector3: ...
    @property
    def uuo(self) -> UVector3: ...
    @property
    def uul(self) -> UVector3: ...
    @property
    def uou(self) -> UVector3: ...
    @property
    def uoo(self) -> UVector3: ...
    @property
    def uol(self) -> UVector3: ...
    @property
    def ulu(self) -> UVector3: ...
    @property
    def ulo(self) -> UVector3: ...
    @property
    def ull(self) -> UVector3: ...
    @property
    def ouu(self) -> UVector3: ...
    @property
    def ouo(self) -> UVector3: ...
    @property
    def oul(self) -> UVector3: ...
    @property
    def oou(self) -> UVector3: ...
    @property
    def olu(self) -> UVector3: ...
    @property
    def luu(self) -> UVector3: ...
    @property
    def luo(self) -> UVector3: ...
    @property
    def lul(self) -> UVector3: ...
    @property
    def lou(self) -> UVector3: ...
    @property
    def llu(self) -> UVector3: ...
    @property
    def uuuu(self) -> UVector4: ...
    @property
    def uuuo(self) -> UVector4: ...
    @property
    def uuul(self) -> UVector4: ...
    @property
    def uuou(self) -> UVector4: ...
    @property
    def uuoo(self) -> UVector4: ...
    @property
    def uuol(self) -> UVector4: ...
    @property
    def uulu(self) -> UVector4: ...
    @property
    def uulo(self) -> UVector4: ...
    @property
    def uull(self) -> UVector4: ...
    @property
    def uouu(self) -> UVector4: ...
    @property
    def uouo(self) -> UVector4: ...
    @property
    def uoul(self) -> UVector4: ...
    @property
    def uoou(self) -> UVector4: ...
    @property
    def uooo(self) -> UVector4: ...
    @property
    def uool(self) -> UVector4: ...
    @property
    def uolu(self) -> UVector4: ...
    @property
    def uolo(self) -> UVector4: ...
    @property
    def uoll(self) -> UVector4: ...
    @property
    def uluu(self) -> UVector4: ...
    @property
    def uluo(self) -> UVector4: ...
    @property
    def ulul(self) -> UVector4: ...
    @property
    def ulou(self) -> UVector4: ...
    @property
    def uloo(self) -> UVector4: ...
    @property
    def ulol(self) -> UVector4: ...
    @property
    def ullu(self) -> UVector4: ...
    @property
    def ullo(self) -> UVector4: ...
    @property
    def ulll(self) -> UVector4: ...
    @property
    def ouuu(self) -> UVector4: ...
    @property
    def ouuo(self) -> UVector4: ...
    @property
    def ouul(self) -> UVector4: ...
    @property
    def ouou(self) -> UVector4: ...
    @property
    def ouoo(self) -> UVector4: ...
    @property
    def ouol(self) -> UVector4: ...
    @property
    def oulu(self) -> UVector4: ...
    @property
    def oulo(self) -> UVector4: ...
    @property
    def oull(self) -> UVector4: ...
    @property
    def oouu(self) -> UVector4: ...
    @property
    def oouo(self) -> UVector4: ...
    @property
    def ooul(self) -> UVector4: ...
    @property
    def ooou(self) -> UVector4: ...
    @property
    def oolu(self) -> UVector4: ...
    @property
    def oluu(self) -> UVector4: ...
    @property
    def oluo(self) -> UVector4: ...
    @property
    def olul(self) -> UVector4: ...
    @property
    def olou(self) -> UVector4: ...
    @property
    def ollu(self) -> UVector4: ...
    @property
    def luuu(self) -> UVector4: ...
    @property
    def luuo(self) -> UVector4: ...
    @property
    def luul(self) -> UVector4: ...
    @property
    def luou(self) -> UVector4: ...
    @property
    def luoo(self) -> UVector4: ...
    @property
    def luol(self) -> UVector4: ...
    @property
    def lulu(self) -> UVector4: ...
    @property
    def lulo(self) -> UVector4: ...
    @property
    def lull(self) -> UVector4: ...
    @property
    def louu(self) -> UVector4: ...
    @property
    def louo(self) -> UVector4: ...
    @property
    def loul(self) -> UVector4: ...
    @property
    def loou(self) -> UVector4: ...
    @property
    def lolu(self) -> UVector4: ...
    @property
    def lluu(self) -> UVector4: ...
    @property
    def lluo(self) -> UVector4: ...
    @property
    def llul(self) -> UVector4: ...
    @property
    def llou(self) -> UVector4: ...
    @property
    def lllu(self) -> UVector4: ...
    @property
    def oo(self) -> UVector2: ...
    @property
    def ol(self) -> UVector2: ...
    @property
    def lo(self) -> UVector2: ...
    @property
    def ll(self) -> UVector2: ...
    @property
    def ooo(self) -> UVector3: ...
    @property
    def ool(self) -> UVector3: ...
    @property
    def olo(self) -> UVector3: ...
    @property
    def oll(self) -> UVector3: ...
    @property
    def loo(self) -> UVector3: ...
    @property
    def lol(self) -> UVector3: ...
    @property
    def llo(self) -> UVector3: ...
    @property
    def lll(self) -> UVector3: ...
    @property
    def oooo(self) -> UVector4: ...
    @property
    def oool(self) -> UVector4: ...
    @property
    def oolo(self) -> UVector4: ...
    @property
    def ooll(self) -> UVector4: ...
    @property
    def oloo(self) -> UVector4: ...
    @property
    def olol(self) -> UVector4: ...
    @property
    def ollo(self) -> UVector4: ...
    @property
    def olll(self) -> UVector4: ...
    @property
    def looo(self) -> UVector4: ...
    @property
    def lool(self) -> UVector4: ...
    @property
    def lolo(self) -> UVector4: ...
    @property
    def loll(self) -> UVector4: ...
    @property
    def lloo(self) -> UVector4: ...
    @property
    def llol(self) -> UVector4: ...
    @property
    def lllo(self) -> UVector4: ...
    @property
    def llll(self) -> UVector4: ...
    def min(self, n: Number, /) -> UVector1: ...
    def max(self, n: Number, /) -> UVector1: ...
    def clamp(self, min: Number, max: Number, /) -> UVector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[UVector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector1: ...

@final
class UVector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: UVector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[UVector1]: ...
    @overload
    def __getitem__(self, index: int) -> UVector1: ...
    @overload
    def __getitem__(self, index: slice) -> UVector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[UVector1]: ...

@final
class I64Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __add__(self, other: Number) -> I64Vector1: ...
    @overload
    def __radd__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __radd__(self, other: Number) -> I64Vector1: ...
    @overload
    def __sub__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __sub__(self, other: Number) -> I64Vector1: ...
    @overload
    def __rsub__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> I64Vector1: ...
    @overload
    def __mul__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __mul__(self, other: Number) -> I64Vector1: ...
    @overload
    def __rmul__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> I64Vector1: ...
    @overload
    def __truediv__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> I64Vector1: ...
    @overload
    def __rtruediv__(self, other: I64Vector1) -> I64Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> I64Vector1: ...
    def __neg__(self) -> I64Vector1: ...
    def __abs__(self) -> I64Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> I64Vector2: ...
    @property
    def xo(self) -> I64Vector2: ...
    @property
    def xl(self) -> I64Vector2: ...
    @property
    def ox(self) -> I64Vector2: ...
    @property
    def lx(self) -> I64Vector2: ...
    @property
    def xxx(self) -> I64Vector3: ...
    @property
    def xxo(self) -> I64Vector3: ...
    @property
    def xxl(self) -> I64Vector3: ...
    @property
    def xox(self) -> I64Vector3: ...
    @property
    def xoo(self) -> I64Vector3: ...
    @property
    def xol(self) -> I64Vector3: ...
    @property
    def xlx(self) -> I64Vector3: ...
    @property
    def xlo(self) -> I64Vector3: ...
    @property
    def xll(self) -> I64Vector3: ...
    @property
    def oxx(self) -> I64Vector3: ...
    @property
    def oxo(self) -> I64Vector3: ...
    @property
    def oxl(self) -> I64Vector3: ...
    @property
    def oox(self) -> I64Vector3: ...
    @property
    def olx(self) -> I64Vector3: ...
    @property
    def lxx(self) -> I64Vector3: ...
    @property
    def lxo(self) -> I64Vector3: ...
    @property
    def lxl(self) -> I64Vector3: ...
    @property
    def lox(self) -> I64Vector3: ...
    @property
    def llx(self) -> I64Vector3: ...
    @property
    def xxxx(self) -> I64Vector4: ...
    @property
    def xxxo(self) -> I64Vector4: ...
    @property
    def xxxl(self) -> I64Vector4: ...
    @property
    def xxox(self) -> I64Vector4: ...
    @property
    def xxoo(self) -> I64Vector4: ...
    @property
    def xxol(self) -> I64Vector4: ...
    @property
    def xxlx(self) -> I64Vector4: ...
    @property
    def xxlo(self) -> I64Vector4: ...
    @property
    def xxll(self) -> I64Vector4: ...
    @property
    def xoxx(self) -> I64Vector4: ...
    @property
    def xoxo(self) -> I64Vector4: ...
    @property
    def xoxl(self) -> I64Vector4: ...
    @property
    def xoox(self) -> I64Vector4: ...
    @property
    def xooo(self) -> I64Vector4: ...
    @property
    def xool(self) -> I64Vector4: ...
    @property
    def xolx(self) -> I64Vector4: ...
    @property
    def xolo(self) -> I64Vector4: ...
    @property
    def xoll(self) -> I64Vector4: ...
    @property
    def xlxx(self) -> I64Vector4: ...
    @property
    def xlxo(self) -> I64Vector4: ...
    @property
    def xlxl(self) -> I64Vector4: ...
    @property
    def xlox(self) -> I64Vector4: ...
    @property
    def xloo(self) -> I64Vector4: ...
    @property
    def xlol(self) -> I64Vector4: ...
    @property
    def xllx(self) -> I64Vector4: ...
    @property
    def xllo(self) -> I64Vector4: ...
    @property
    def xlll(self) -> I64Vector4: ...
    @property
    def oxxx(self) -> I64Vector4: ...
    @property
    def oxxo(self) -> I64Vector4: ...
    @property
    def oxxl(self) -> I64Vector4: ...
    @property
    def oxox(self) -> I64Vector4: ...
    @property
    def oxoo(self) -> I64Vector4: ...
    @property
    def oxol(self) -> I64Vector4: ...
    @property
    def oxlx(self) -> I64Vector4: ...
    @property
    def oxlo(self) -> I64Vector4: ...
    @property
    def oxll(self) -> I64Vector4: ...
    @property
    def ooxx(self) -> I64Vector4: ...
    @property
    def ooxo(self) -> I64Vector4: ...
    @property
    def ooxl(self) -> I64Vector4: ...
    @property
    def ooox(self) -> I64Vector4: ...
    @property
    def oolx(self) -> I64Vector4: ...
    @property
    def olxx(self) -> I64Vector4: ...
    @property
    def olxo(self) -> I64Vector4: ...
    @property
    def olxl(self) -> I64Vector4: ...
    @property
    def olox(self) -> I64Vector4: ...
    @property
    def ollx(self) -> I64Vector4: ...
    @property
    def lxxx(self) -> I64Vector4: ...
    @property
    def lxxo(self) -> I64Vector4: ...
    @property
    def lxxl(self) -> I64Vector4: ...
    @property
    def lxox(self) -> I64Vector4: ...
    @property
    def lxoo(self) -> I64Vector4: ...
    @property
    def lxol(self) -> I64Vector4: ...
    @property
    def lxlx(self) -> I64Vector4: ...
    @property
    def lxlo(self) -> I64Vector4: ...
    @property
    def lxll(self) -> I64Vector4: ...
    @property
    def loxx(self) -> I64Vector4: ...
    @property
    def loxo(self) -> I64Vector4: ...
    @property
    def loxl(self) -> I64Vector4: ...
    @property
    def loox(self) -> I64Vector4: ...
    @property
    def lolx(self) -> I64Vector4: ...
    @property
    def llxx(self) -> I64Vector4: ...
    @property
    def llxo(self) -> I64Vector4: ...
    @property
    def llxl(self) -> I64Vector4: ...
    @property
    def llox(self) -> I64Vector4: ...
    @property
    def lllx(self) -> I64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> I64Vector2: ...
    @property
    def ro(self) -> I64Vector2: ...
    @property
    def rl(self) -> I64Vector2: ...
    @property
    def lr(self) -> I64Vector2: ...
    @property
    def rrr(self) -> I64Vector3: ...
    @property
    def rro(self) -> I64Vector3: ...
    @property
    def rrl(self) -> I64Vector3: ...
    @property
    def ror(self) -> I64Vector3: ...
    @property
    def roo(self) -> I64Vector3: ...
    @property
    def rol(self) -> I64Vector3: ...
    @property
    def rlr(self) -> I64Vector3: ...
    @property
    def rlo(self) -> I64Vector3: ...
    @property
    def rll(self) -> I64Vector3: ...
    @property
    def orr(self) -> I64Vector3: ...
    @property
    def oro(self) -> I64Vector3: ...
    @property
    def orl(self) -> I64Vector3: ...
    @property
    def oor(self) -> I64Vector3: ...
    @property
    def olr(self) -> I64Vector3: ...
    @property
    def lrr(self) -> I64Vector3: ...
    @property
    def lro(self) -> I64Vector3: ...
    @property
    def lrl(self) -> I64Vector3: ...
    @property
    def lor(self) -> I64Vector3: ...
    @property
    def llr(self) -> I64Vector3: ...
    @property
    def rrrr(self) -> I64Vector4: ...
    @property
    def rrro(self) -> I64Vector4: ...
    @property
    def rrrl(self) -> I64Vector4: ...
    @property
    def rror(self) -> I64Vector4: ...
    @property
    def rroo(self) -> I64Vector4: ...
    @property
    def rrol(self) -> I64Vector4: ...
    @property
    def rrlr(self) -> I64Vector4: ...
    @property
    def rrlo(self) -> I64Vector4: ...
    @property
    def rrll(self) -> I64Vector4: ...
    @property
    def rorr(self) -> I64Vector4: ...
    @property
    def roro(self) -> I64Vector4: ...
    @property
    def rorl(self) -> I64Vector4: ...
    @property
    def roor(self) -> I64Vector4: ...
    @property
    def rooo(self) -> I64Vector4: ...
    @property
    def rool(self) -> I64Vector4: ...
    @property
    def rolr(self) -> I64Vector4: ...
    @property
    def rolo(self) -> I64Vector4: ...
    @property
    def roll(self) -> I64Vector4: ...
    @property
    def rlrr(self) -> I64Vector4: ...
    @property
    def rlro(self) -> I64Vector4: ...
    @property
    def rlrl(self) -> I64Vector4: ...
    @property
    def rlor(self) -> I64Vector4: ...
    @property
    def rloo(self) -> I64Vector4: ...
    @property
    def rlol(self) -> I64Vector4: ...
    @property
    def rllr(self) -> I64Vector4: ...
    @property
    def rllo(self) -> I64Vector4: ...
    @property
    def rlll(self) -> I64Vector4: ...
    @property
    def orrr(self) -> I64Vector4: ...
    @property
    def orro(self) -> I64Vector4: ...
    @property
    def orrl(self) -> I64Vector4: ...
    @property
    def oror(self) -> I64Vector4: ...
    @property
    def oroo(self) -> I64Vector4: ...
    @property
    def orol(self) -> I64Vector4: ...
    @property
    def orlr(self) -> I64Vector4: ...
    @property
    def orlo(self) -> I64Vector4: ...
    @property
    def orll(self) -> I64Vector4: ...
    @property
    def oorr(self) -> I64Vector4: ...
    @property
    def ooro(self) -> I64Vector4: ...
    @property
    def oorl(self) -> I64Vector4: ...
    @property
    def ooor(self) -> I64Vector4: ...
    @property
    def oolr(self) -> I64Vector4: ...
    @property
    def olrr(self) -> I64Vector4: ...
    @property
    def olro(self) -> I64Vector4: ...
    @property
    def olrl(self) -> I64Vector4: ...
    @property
    def olor(self) -> I64Vector4: ...
    @property
    def ollr(self) -> I64Vector4: ...
    @property
    def lrrr(self) -> I64Vector4: ...
    @property
    def lrro(self) -> I64Vector4: ...
    @property
    def lrrl(self) -> I64Vector4: ...
    @property
    def lror(self) -> I64Vector4: ...
    @property
    def lroo(self) -> I64Vector4: ...
    @property
    def lrol(self) -> I64Vector4: ...
    @property
    def lrlr(self) -> I64Vector4: ...
    @property
    def lrlo(self) -> I64Vector4: ...
    @property
    def lrll(self) -> I64Vector4: ...
    @property
    def lorr(self) -> I64Vector4: ...
    @property
    def loro(self) -> I64Vector4: ...
    @property
    def lorl(self) -> I64Vector4: ...
    @property
    def loor(self) -> I64Vector4: ...
    @property
    def lolr(self) -> I64Vector4: ...
    @property
    def llrr(self) -> I64Vector4: ...
    @property
    def llro(self) -> I64Vector4: ...
    @property
    def llrl(self) -> I64Vector4: ...
    @property
    def llor(self) -> I64Vector4: ...
    @property
    def lllr(self) -> I64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> I64Vector2: ...
    @property
    def so(self) -> I64Vector2: ...
    @property
    def sl(self) -> I64Vector2: ...
    @property
    def os(self) -> I64Vector2: ...
    @property
    def ls(self) -> I64Vector2: ...
    @property
    def sss(self) -> I64Vector3: ...
    @property
    def sso(self) -> I64Vector3: ...
    @property
    def ssl(self) -> I64Vector3: ...
    @property
    def sos(self) -> I64Vector3: ...
    @property
    def soo(self) -> I64Vector3: ...
    @property
    def sol(self) -> I64Vector3: ...
    @property
    def sls(self) -> I64Vector3: ...
    @property
    def slo(self) -> I64Vector3: ...
    @property
    def sll(self) -> I64Vector3: ...
    @property
    def oss(self) -> I64Vector3: ...
    @property
    def oso(self) -> I64Vector3: ...
    @property
    def osl(self) -> I64Vector3: ...
    @property
    def oos(self) -> I64Vector3: ...
    @property
    def ols(self) -> I64Vector3: ...
    @property
    def lss(self) -> I64Vector3: ...
    @property
    def lso(self) -> I64Vector3: ...
    @property
    def lsl(self) -> I64Vector3: ...
    @property
    def los(self) -> I64Vector3: ...
    @property
    def lls(self) -> I64Vector3: ...
    @property
    def ssss(self) -> I64Vector4: ...
    @property
    def ssso(self) -> I64Vector4: ...
    @property
    def sssl(self) -> I64Vector4: ...
    @property
    def ssos(self) -> I64Vector4: ...
    @property
    def ssoo(self) -> I64Vector4: ...
    @property
    def ssol(self) -> I64Vector4: ...
    @property
    def ssls(self) -> I64Vector4: ...
    @property
    def sslo(self) -> I64Vector4: ...
    @property
    def ssll(self) -> I64Vector4: ...
    @property
    def soss(self) -> I64Vector4: ...
    @property
    def soso(self) -> I64Vector4: ...
    @property
    def sosl(self) -> I64Vector4: ...
    @property
    def soos(self) -> I64Vector4: ...
    @property
    def sooo(self) -> I64Vector4: ...
    @property
    def sool(self) -> I64Vector4: ...
    @property
    def sols(self) -> I64Vector4: ...
    @property
    def solo(self) -> I64Vector4: ...
    @property
    def soll(self) -> I64Vector4: ...
    @property
    def slss(self) -> I64Vector4: ...
    @property
    def slso(self) -> I64Vector4: ...
    @property
    def slsl(self) -> I64Vector4: ...
    @property
    def slos(self) -> I64Vector4: ...
    @property
    def sloo(self) -> I64Vector4: ...
    @property
    def slol(self) -> I64Vector4: ...
    @property
    def slls(self) -> I64Vector4: ...
    @property
    def sllo(self) -> I64Vector4: ...
    @property
    def slll(self) -> I64Vector4: ...
    @property
    def osss(self) -> I64Vector4: ...
    @property
    def osso(self) -> I64Vector4: ...
    @property
    def ossl(self) -> I64Vector4: ...
    @property
    def osos(self) -> I64Vector4: ...
    @property
    def osoo(self) -> I64Vector4: ...
    @property
    def osol(self) -> I64Vector4: ...
    @property
    def osls(self) -> I64Vector4: ...
    @property
    def oslo(self) -> I64Vector4: ...
    @property
    def osll(self) -> I64Vector4: ...
    @property
    def ooss(self) -> I64Vector4: ...
    @property
    def ooso(self) -> I64Vector4: ...
    @property
    def oosl(self) -> I64Vector4: ...
    @property
    def ooos(self) -> I64Vector4: ...
    @property
    def ools(self) -> I64Vector4: ...
    @property
    def olss(self) -> I64Vector4: ...
    @property
    def olso(self) -> I64Vector4: ...
    @property
    def olsl(self) -> I64Vector4: ...
    @property
    def olos(self) -> I64Vector4: ...
    @property
    def olls(self) -> I64Vector4: ...
    @property
    def lsss(self) -> I64Vector4: ...
    @property
    def lsso(self) -> I64Vector4: ...
    @property
    def lssl(self) -> I64Vector4: ...
    @property
    def lsos(self) -> I64Vector4: ...
    @property
    def lsoo(self) -> I64Vector4: ...
    @property
    def lsol(self) -> I64Vector4: ...
    @property
    def lsls(self) -> I64Vector4: ...
    @property
    def lslo(self) -> I64Vector4: ...
    @property
    def lsll(self) -> I64Vector4: ...
    @property
    def loss(self) -> I64Vector4: ...
    @property
    def loso(self) -> I64Vector4: ...
    @property
    def losl(self) -> I64Vector4: ...
    @property
    def loos(self) -> I64Vector4: ...
    @property
    def lols(self) -> I64Vector4: ...
    @property
    def llss(self) -> I64Vector4: ...
    @property
    def llso(self) -> I64Vector4: ...
    @property
    def llsl(self) -> I64Vector4: ...
    @property
    def llos(self) -> I64Vector4: ...
    @property
    def llls(self) -> I64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> I64Vector2: ...
    @property
    def uo(self) -> I64Vector2: ...
    @property
    def ul(self) -> I64Vector2: ...
    @property
    def ou(self) -> I64Vector2: ...
    @property
    def lu(self) -> I64Vector2: ...
    @property
    def uuu(self) -> I64Vector3: ...
    @property
    def uuo(self) -> I64Vector3: ...
    @property
    def uul(self) -> I64Vector3: ...
    @property
    def uou(self) -> I64Vector3: ...
    @property
    def uoo(self) -> I64Vector3: ...
    @property
    def uol(self) -> I64Vector3: ...
    @property
    def ulu(self) -> I64Vector3: ...
    @property
    def ulo(self) -> I64Vector3: ...
    @property
    def ull(self) -> I64Vector3: ...
    @property
    def ouu(self) -> I64Vector3: ...
    @property
    def ouo(self) -> I64Vector3: ...
    @property
    def oul(self) -> I64Vector3: ...
    @property
    def oou(self) -> I64Vector3: ...
    @property
    def olu(self) -> I64Vector3: ...
    @property
    def luu(self) -> I64Vector3: ...
    @property
    def luo(self) -> I64Vector3: ...
    @property
    def lul(self) -> I64Vector3: ...
    @property
    def lou(self) -> I64Vector3: ...
    @property
    def llu(self) -> I64Vector3: ...
    @property
    def uuuu(self) -> I64Vector4: ...
    @property
    def uuuo(self) -> I64Vector4: ...
    @property
    def uuul(self) -> I64Vector4: ...
    @property
    def uuou(self) -> I64Vector4: ...
    @property
    def uuoo(self) -> I64Vector4: ...
    @property
    def uuol(self) -> I64Vector4: ...
    @property
    def uulu(self) -> I64Vector4: ...
    @property
    def uulo(self) -> I64Vector4: ...
    @property
    def uull(self) -> I64Vector4: ...
    @property
    def uouu(self) -> I64Vector4: ...
    @property
    def uouo(self) -> I64Vector4: ...
    @property
    def uoul(self) -> I64Vector4: ...
    @property
    def uoou(self) -> I64Vector4: ...
    @property
    def uooo(self) -> I64Vector4: ...
    @property
    def uool(self) -> I64Vector4: ...
    @property
    def uolu(self) -> I64Vector4: ...
    @property
    def uolo(self) -> I64Vector4: ...
    @property
    def uoll(self) -> I64Vector4: ...
    @property
    def uluu(self) -> I64Vector4: ...
    @property
    def uluo(self) -> I64Vector4: ...
    @property
    def ulul(self) -> I64Vector4: ...
    @property
    def ulou(self) -> I64Vector4: ...
    @property
    def uloo(self) -> I64Vector4: ...
    @property
    def ulol(self) -> I64Vector4: ...
    @property
    def ullu(self) -> I64Vector4: ...
    @property
    def ullo(self) -> I64Vector4: ...
    @property
    def ulll(self) -> I64Vector4: ...
    @property
    def ouuu(self) -> I64Vector4: ...
    @property
    def ouuo(self) -> I64Vector4: ...
    @property
    def ouul(self) -> I64Vector4: ...
    @property
    def ouou(self) -> I64Vector4: ...
    @property
    def ouoo(self) -> I64Vector4: ...
    @property
    def ouol(self) -> I64Vector4: ...
    @property
    def oulu(self) -> I64Vector4: ...
    @property
    def oulo(self) -> I64Vector4: ...
    @property
    def oull(self) -> I64Vector4: ...
    @property
    def oouu(self) -> I64Vector4: ...
    @property
    def oouo(self) -> I64Vector4: ...
    @property
    def ooul(self) -> I64Vector4: ...
    @property
    def ooou(self) -> I64Vector4: ...
    @property
    def oolu(self) -> I64Vector4: ...
    @property
    def oluu(self) -> I64Vector4: ...
    @property
    def oluo(self) -> I64Vector4: ...
    @property
    def olul(self) -> I64Vector4: ...
    @property
    def olou(self) -> I64Vector4: ...
    @property
    def ollu(self) -> I64Vector4: ...
    @property
    def luuu(self) -> I64Vector4: ...
    @property
    def luuo(self) -> I64Vector4: ...
    @property
    def luul(self) -> I64Vector4: ...
    @property
    def luou(self) -> I64Vector4: ...
    @property
    def luoo(self) -> I64Vector4: ...
    @property
    def luol(self) -> I64Vector4: ...
    @property
    def lulu(self) -> I64Vector4: ...
    @property
    def lulo(self) -> I64Vector4: ...
    @property
    def lull(self) -> I64Vector4: ...
    @property
    def louu(self) -> I64Vector4: ...
    @property
    def louo(self) -> I64Vector4: ...
    @property
    def loul(self) -> I64Vector4: ...
    @property
    def loou(self) -> I64Vector4: ...
    @property
    def lolu(self) -> I64Vector4: ...
    @property
    def lluu(self) -> I64Vector4: ...
    @property
    def lluo(self) -> I64Vector4: ...
    @property
    def llul(self) -> I64Vector4: ...
    @property
    def llou(self) -> I64Vector4: ...
    @property
    def lllu(self) -> I64Vector4: ...
    @property
    def oo(self) -> I64Vector2: ...
    @property
    def ol(self) -> I64Vector2: ...
    @property
    def lo(self) -> I64Vector2: ...
    @property
    def ll(self) -> I64Vector2: ...
    @property
    def ooo(self) -> I64Vector3: ...
    @property
    def ool(self) -> I64Vector3: ...
    @property
    def olo(self) -> I64Vector3: ...
    @property
    def oll(self) -> I64Vector3: ...
    @property
    def loo(self) -> I64Vector3: ...
    @property
    def lol(self) -> I64Vector3: ...
    @property
    def llo(self) -> I64Vector3: ...
    @property
    def lll(self) -> I64Vector3: ...
    @property
    def oooo(self) -> I64Vector4: ...
    @property
    def oool(self) -> I64Vector4: ...
    @property
    def oolo(self) -> I64Vector4: ...
    @property
    def ooll(self) -> I64Vector4: ...
    @property
    def oloo(self) -> I64Vector4: ...
    @property
    def olol(self) -> I64Vector4: ...
    @property
    def ollo(self) -> I64Vector4: ...
    @property
    def olll(self) -> I64Vector4: ...
    @property
    def looo(self) -> I64Vector4: ...
    @property
    def lool(self) -> I64Vector4: ...
    @property
    def lolo(self) -> I64Vector4: ...
    @property
    def loll(self) -> I64Vector4: ...
    @property
    def lloo(self) -> I64Vector4: ...
    @property
    def llol(self) -> I64Vector4: ...
    @property
    def lllo(self) -> I64Vector4: ...
    @property
    def llll(self) -> I64Vector4: ...
    def min(self, n: Number, /) -> I64Vector1: ...
    def max(self, n: Number, /) -> I64Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> I64Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I64Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector1: ...

@final
class I64Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I64Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I64Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> I64Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> I64Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[I64Vector1]: ...

@final
class U64Vector1:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __add__(self, other: Number) -> U64Vector1: ...
    @overload
    def __radd__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __radd__(self, other: Number) -> U64Vector1: ...
    @overload
    def __sub__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __sub__(self, other: Number) -> U64Vector1: ...
    @overload
    def __rsub__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __rsub__(self, other: Number) -> U64Vector1: ...
    @overload
    def __mul__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __mul__(self, other: Number) -> U64Vector1: ...
    @overload
    def __rmul__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __rmul__(self, other: Number) -> U64Vector1: ...
    @overload
    def __truediv__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __truediv__(self, other: Number) -> U64Vector1: ...
    @overload
    def __rtruediv__(self, other: U64Vector1) -> U64Vector1: ...
    @overload
    def __rtruediv__(self, other: Number) -> U64Vector1: ...
    def __abs__(self) -> U64Vector1: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def xx(self) -> U64Vector2: ...
    @property
    def xo(self) -> U64Vector2: ...
    @property
    def xl(self) -> U64Vector2: ...
    @property
    def ox(self) -> U64Vector2: ...
    @property
    def lx(self) -> U64Vector2: ...
    @property
    def xxx(self) -> U64Vector3: ...
    @property
    def xxo(self) -> U64Vector3: ...
    @property
    def xxl(self) -> U64Vector3: ...
    @property
    def xox(self) -> U64Vector3: ...
    @property
    def xoo(self) -> U64Vector3: ...
    @property
    def xol(self) -> U64Vector3: ...
    @property
    def xlx(self) -> U64Vector3: ...
    @property
    def xlo(self) -> U64Vector3: ...
    @property
    def xll(self) -> U64Vector3: ...
    @property
    def oxx(self) -> U64Vector3: ...
    @property
    def oxo(self) -> U64Vector3: ...
    @property
    def oxl(self) -> U64Vector3: ...
    @property
    def oox(self) -> U64Vector3: ...
    @property
    def olx(self) -> U64Vector3: ...
    @property
    def lxx(self) -> U64Vector3: ...
    @property
    def lxo(self) -> U64Vector3: ...
    @property
    def lxl(self) -> U64Vector3: ...
    @property
    def lox(self) -> U64Vector3: ...
    @property
    def llx(self) -> U64Vector3: ...
    @property
    def xxxx(self) -> U64Vector4: ...
    @property
    def xxxo(self) -> U64Vector4: ...
    @property
    def xxxl(self) -> U64Vector4: ...
    @property
    def xxox(self) -> U64Vector4: ...
    @property
    def xxoo(self) -> U64Vector4: ...
    @property
    def xxol(self) -> U64Vector4: ...
    @property
    def xxlx(self) -> U64Vector4: ...
    @property
    def xxlo(self) -> U64Vector4: ...
    @property
    def xxll(self) -> U64Vector4: ...
    @property
    def xoxx(self) -> U64Vector4: ...
    @property
    def xoxo(self) -> U64Vector4: ...
    @property
    def xoxl(self) -> U64Vector4: ...
    @property
    def xoox(self) -> U64Vector4: ...
    @property
    def xooo(self) -> U64Vector4: ...
    @property
    def xool(self) -> U64Vector4: ...
    @property
    def xolx(self) -> U64Vector4: ...
    @property
    def xolo(self) -> U64Vector4: ...
    @property
    def xoll(self) -> U64Vector4: ...
    @property
    def xlxx(self) -> U64Vector4: ...
    @property
    def xlxo(self) -> U64Vector4: ...
    @property
    def xlxl(self) -> U64Vector4: ...
    @property
    def xlox(self) -> U64Vector4: ...
    @property
    def xloo(self) -> U64Vector4: ...
    @property
    def xlol(self) -> U64Vector4: ...
    @property
    def xllx(self) -> U64Vector4: ...
    @property
    def xllo(self) -> U64Vector4: ...
    @property
    def xlll(self) -> U64Vector4: ...
    @property
    def oxxx(self) -> U64Vector4: ...
    @property
    def oxxo(self) -> U64Vector4: ...
    @property
    def oxxl(self) -> U64Vector4: ...
    @property
    def oxox(self) -> U64Vector4: ...
    @property
    def oxoo(self) -> U64Vector4: ...
    @property
    def oxol(self) -> U64Vector4: ...
    @property
    def oxlx(self) -> U64Vector4: ...
    @property
    def oxlo(self) -> U64Vector4: ...
    @property
    def oxll(self) -> U64Vector4: ...
    @property
    def ooxx(self) -> U64Vector4: ...
    @property
    def ooxo(self) -> U64Vector4: ...
    @property
    def ooxl(self) -> U64Vector4: ...
    @property
    def ooox(self) -> U64Vector4: ...
    @property
    def oolx(self) -> U64Vector4: ...
    @property
    def olxx(self) -> U64Vector4: ...
    @property
    def olxo(self) -> U64Vector4: ...
    @property
    def olxl(self) -> U64Vector4: ...
    @property
    def olox(self) -> U64Vector4: ...
    @property
    def ollx(self) -> U64Vector4: ...
    @property
    def lxxx(self) -> U64Vector4: ...
    @property
    def lxxo(self) -> U64Vector4: ...
    @property
    def lxxl(self) -> U64Vector4: ...
    @property
    def lxox(self) -> U64Vector4: ...
    @property
    def lxoo(self) -> U64Vector4: ...
    @property
    def lxol(self) -> U64Vector4: ...
    @property
    def lxlx(self) -> U64Vector4: ...
    @property
    def lxlo(self) -> U64Vector4: ...
    @property
    def lxll(self) -> U64Vector4: ...
    @property
    def loxx(self) -> U64Vector4: ...
    @property
    def loxo(self) -> U64Vector4: ...
    @property
    def loxl(self) -> U64Vector4: ...
    @property
    def loox(self) -> U64Vector4: ...
    @property
    def lolx(self) -> U64Vector4: ...
    @property
    def llxx(self) -> U64Vector4: ...
    @property
    def llxo(self) -> U64Vector4: ...
    @property
    def llxl(self) -> U64Vector4: ...
    @property
    def llox(self) -> U64Vector4: ...
    @property
    def lllx(self) -> U64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def rr(self) -> U64Vector2: ...
    @property
    def ro(self) -> U64Vector2: ...
    @property
    def rl(self) -> U64Vector2: ...
    @property
    def lr(self) -> U64Vector2: ...
    @property
    def rrr(self) -> U64Vector3: ...
    @property
    def rro(self) -> U64Vector3: ...
    @property
    def rrl(self) -> U64Vector3: ...
    @property
    def ror(self) -> U64Vector3: ...
    @property
    def roo(self) -> U64Vector3: ...
    @property
    def rol(self) -> U64Vector3: ...
    @property
    def rlr(self) -> U64Vector3: ...
    @property
    def rlo(self) -> U64Vector3: ...
    @property
    def rll(self) -> U64Vector3: ...
    @property
    def orr(self) -> U64Vector3: ...
    @property
    def oro(self) -> U64Vector3: ...
    @property
    def orl(self) -> U64Vector3: ...
    @property
    def oor(self) -> U64Vector3: ...
    @property
    def olr(self) -> U64Vector3: ...
    @property
    def lrr(self) -> U64Vector3: ...
    @property
    def lro(self) -> U64Vector3: ...
    @property
    def lrl(self) -> U64Vector3: ...
    @property
    def lor(self) -> U64Vector3: ...
    @property
    def llr(self) -> U64Vector3: ...
    @property
    def rrrr(self) -> U64Vector4: ...
    @property
    def rrro(self) -> U64Vector4: ...
    @property
    def rrrl(self) -> U64Vector4: ...
    @property
    def rror(self) -> U64Vector4: ...
    @property
    def rroo(self) -> U64Vector4: ...
    @property
    def rrol(self) -> U64Vector4: ...
    @property
    def rrlr(self) -> U64Vector4: ...
    @property
    def rrlo(self) -> U64Vector4: ...
    @property
    def rrll(self) -> U64Vector4: ...
    @property
    def rorr(self) -> U64Vector4: ...
    @property
    def roro(self) -> U64Vector4: ...
    @property
    def rorl(self) -> U64Vector4: ...
    @property
    def roor(self) -> U64Vector4: ...
    @property
    def rooo(self) -> U64Vector4: ...
    @property
    def rool(self) -> U64Vector4: ...
    @property
    def rolr(self) -> U64Vector4: ...
    @property
    def rolo(self) -> U64Vector4: ...
    @property
    def roll(self) -> U64Vector4: ...
    @property
    def rlrr(self) -> U64Vector4: ...
    @property
    def rlro(self) -> U64Vector4: ...
    @property
    def rlrl(self) -> U64Vector4: ...
    @property
    def rlor(self) -> U64Vector4: ...
    @property
    def rloo(self) -> U64Vector4: ...
    @property
    def rlol(self) -> U64Vector4: ...
    @property
    def rllr(self) -> U64Vector4: ...
    @property
    def rllo(self) -> U64Vector4: ...
    @property
    def rlll(self) -> U64Vector4: ...
    @property
    def orrr(self) -> U64Vector4: ...
    @property
    def orro(self) -> U64Vector4: ...
    @property
    def orrl(self) -> U64Vector4: ...
    @property
    def oror(self) -> U64Vector4: ...
    @property
    def oroo(self) -> U64Vector4: ...
    @property
    def orol(self) -> U64Vector4: ...
    @property
    def orlr(self) -> U64Vector4: ...
    @property
    def orlo(self) -> U64Vector4: ...
    @property
    def orll(self) -> U64Vector4: ...
    @property
    def oorr(self) -> U64Vector4: ...
    @property
    def ooro(self) -> U64Vector4: ...
    @property
    def oorl(self) -> U64Vector4: ...
    @property
    def ooor(self) -> U64Vector4: ...
    @property
    def oolr(self) -> U64Vector4: ...
    @property
    def olrr(self) -> U64Vector4: ...
    @property
    def olro(self) -> U64Vector4: ...
    @property
    def olrl(self) -> U64Vector4: ...
    @property
    def olor(self) -> U64Vector4: ...
    @property
    def ollr(self) -> U64Vector4: ...
    @property
    def lrrr(self) -> U64Vector4: ...
    @property
    def lrro(self) -> U64Vector4: ...
    @property
    def lrrl(self) -> U64Vector4: ...
    @property
    def lror(self) -> U64Vector4: ...
    @property
    def lroo(self) -> U64Vector4: ...
    @property
    def lrol(self) -> U64Vector4: ...
    @property
    def lrlr(self) -> U64Vector4: ...
    @property
    def lrlo(self) -> U64Vector4: ...
    @property
    def lrll(self) -> U64Vector4: ...
    @property
    def lorr(self) -> U64Vector4: ...
    @property
    def loro(self) -> U64Vector4: ...
    @property
    def lorl(self) -> U64Vector4: ...
    @property
    def loor(self) -> U64Vector4: ...
    @property
    def lolr(self) -> U64Vector4: ...
    @property
    def llrr(self) -> U64Vector4: ...
    @property
    def llro(self) -> U64Vector4: ...
    @property
    def llrl(self) -> U64Vector4: ...
    @property
    def llor(self) -> U64Vector4: ...
    @property
    def lllr(self) -> U64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def ss(self) -> U64Vector2: ...
    @property
    def so(self) -> U64Vector2: ...
    @property
    def sl(self) -> U64Vector2: ...
    @property
    def os(self) -> U64Vector2: ...
    @property
    def ls(self) -> U64Vector2: ...
    @property
    def sss(self) -> U64Vector3: ...
    @property
    def sso(self) -> U64Vector3: ...
    @property
    def ssl(self) -> U64Vector3: ...
    @property
    def sos(self) -> U64Vector3: ...
    @property
    def soo(self) -> U64Vector3: ...
    @property
    def sol(self) -> U64Vector3: ...
    @property
    def sls(self) -> U64Vector3: ...
    @property
    def slo(self) -> U64Vector3: ...
    @property
    def sll(self) -> U64Vector3: ...
    @property
    def oss(self) -> U64Vector3: ...
    @property
    def oso(self) -> U64Vector3: ...
    @property
    def osl(self) -> U64Vector3: ...
    @property
    def oos(self) -> U64Vector3: ...
    @property
    def ols(self) -> U64Vector3: ...
    @property
    def lss(self) -> U64Vector3: ...
    @property
    def lso(self) -> U64Vector3: ...
    @property
    def lsl(self) -> U64Vector3: ...
    @property
    def los(self) -> U64Vector3: ...
    @property
    def lls(self) -> U64Vector3: ...
    @property
    def ssss(self) -> U64Vector4: ...
    @property
    def ssso(self) -> U64Vector4: ...
    @property
    def sssl(self) -> U64Vector4: ...
    @property
    def ssos(self) -> U64Vector4: ...
    @property
    def ssoo(self) -> U64Vector4: ...
    @property
    def ssol(self) -> U64Vector4: ...
    @property
    def ssls(self) -> U64Vector4: ...
    @property
    def sslo(self) -> U64Vector4: ...
    @property
    def ssll(self) -> U64Vector4: ...
    @property
    def soss(self) -> U64Vector4: ...
    @property
    def soso(self) -> U64Vector4: ...
    @property
    def sosl(self) -> U64Vector4: ...
    @property
    def soos(self) -> U64Vector4: ...
    @property
    def sooo(self) -> U64Vector4: ...
    @property
    def sool(self) -> U64Vector4: ...
    @property
    def sols(self) -> U64Vector4: ...
    @property
    def solo(self) -> U64Vector4: ...
    @property
    def soll(self) -> U64Vector4: ...
    @property
    def slss(self) -> U64Vector4: ...
    @property
    def slso(self) -> U64Vector4: ...
    @property
    def slsl(self) -> U64Vector4: ...
    @property
    def slos(self) -> U64Vector4: ...
    @property
    def sloo(self) -> U64Vector4: ...
    @property
    def slol(self) -> U64Vector4: ...
    @property
    def slls(self) -> U64Vector4: ...
    @property
    def sllo(self) -> U64Vector4: ...
    @property
    def slll(self) -> U64Vector4: ...
    @property
    def osss(self) -> U64Vector4: ...
    @property
    def osso(self) -> U64Vector4: ...
    @property
    def ossl(self) -> U64Vector4: ...
    @property
    def osos(self) -> U64Vector4: ...
    @property
    def osoo(self) -> U64Vector4: ...
    @property
    def osol(self) -> U64Vector4: ...
    @property
    def osls(self) -> U64Vector4: ...
    @property
    def oslo(self) -> U64Vector4: ...
    @property
    def osll(self) -> U64Vector4: ...
    @property
    def ooss(self) -> U64Vector4: ...
    @property
    def ooso(self) -> U64Vector4: ...
    @property
    def oosl(self) -> U64Vector4: ...
    @property
    def ooos(self) -> U64Vector4: ...
    @property
    def ools(self) -> U64Vector4: ...
    @property
    def olss(self) -> U64Vector4: ...
    @property
    def olso(self) -> U64Vector4: ...
    @property
    def olsl(self) -> U64Vector4: ...
    @property
    def olos(self) -> U64Vector4: ...
    @property
    def olls(self) -> U64Vector4: ...
    @property
    def lsss(self) -> U64Vector4: ...
    @property
    def lsso(self) -> U64Vector4: ...
    @property
    def lssl(self) -> U64Vector4: ...
    @property
    def lsos(self) -> U64Vector4: ...
    @property
    def lsoo(self) -> U64Vector4: ...
    @property
    def lsol(self) -> U64Vector4: ...
    @property
    def lsls(self) -> U64Vector4: ...
    @property
    def lslo(self) -> U64Vector4: ...
    @property
    def lsll(self) -> U64Vector4: ...
    @property
    def loss(self) -> U64Vector4: ...
    @property
    def loso(self) -> U64Vector4: ...
    @property
    def losl(self) -> U64Vector4: ...
    @property
    def loos(self) -> U64Vector4: ...
    @property
    def lols(self) -> U64Vector4: ...
    @property
    def llss(self) -> U64Vector4: ...
    @property
    def llso(self) -> U64Vector4: ...
    @property
    def llsl(self) -> U64Vector4: ...
    @property
    def llos(self) -> U64Vector4: ...
    @property
    def llls(self) -> U64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def uu(self) -> U64Vector2: ...
    @property
    def uo(self) -> U64Vector2: ...
    @property
    def ul(self) -> U64Vector2: ...
    @property
    def ou(self) -> U64Vector2: ...
    @property
    def lu(self) -> U64Vector2: ...
    @property
    def uuu(self) -> U64Vector3: ...
    @property
    def uuo(self) -> U64Vector3: ...
    @property
    def uul(self) -> U64Vector3: ...
    @property
    def uou(self) -> U64Vector3: ...
    @property
    def uoo(self) -> U64Vector3: ...
    @property
    def uol(self) -> U64Vector3: ...
    @property
    def ulu(self) -> U64Vector3: ...
    @property
    def ulo(self) -> U64Vector3: ...
    @property
    def ull(self) -> U64Vector3: ...
    @property
    def ouu(self) -> U64Vector3: ...
    @property
    def ouo(self) -> U64Vector3: ...
    @property
    def oul(self) -> U64Vector3: ...
    @property
    def oou(self) -> U64Vector3: ...
    @property
    def olu(self) -> U64Vector3: ...
    @property
    def luu(self) -> U64Vector3: ...
    @property
    def luo(self) -> U64Vector3: ...
    @property
    def lul(self) -> U64Vector3: ...
    @property
    def lou(self) -> U64Vector3: ...
    @property
    def llu(self) -> U64Vector3: ...
    @property
    def uuuu(self) -> U64Vector4: ...
    @property
    def uuuo(self) -> U64Vector4: ...
    @property
    def uuul(self) -> U64Vector4: ...
    @property
    def uuou(self) -> U64Vector4: ...
    @property
    def uuoo(self) -> U64Vector4: ...
    @property
    def uuol(self) -> U64Vector4: ...
    @property
    def uulu(self) -> U64Vector4: ...
    @property
    def uulo(self) -> U64Vector4: ...
    @property
    def uull(self) -> U64Vector4: ...
    @property
    def uouu(self) -> U64Vector4: ...
    @property
    def uouo(self) -> U64Vector4: ...
    @property
    def uoul(self) -> U64Vector4: ...
    @property
    def uoou(self) -> U64Vector4: ...
    @property
    def uooo(self) -> U64Vector4: ...
    @property
    def uool(self) -> U64Vector4: ...
    @property
    def uolu(self) -> U64Vector4: ...
    @property
    def uolo(self) -> U64Vector4: ...
    @property
    def uoll(self) -> U64Vector4: ...
    @property
    def uluu(self) -> U64Vector4: ...
    @property
    def uluo(self) -> U64Vector4: ...
    @property
    def ulul(self) -> U64Vector4: ...
    @property
    def ulou(self) -> U64Vector4: ...
    @property
    def uloo(self) -> U64Vector4: ...
    @property
    def ulol(self) -> U64Vector4: ...
    @property
    def ullu(self) -> U64Vector4: ...
    @property
    def ullo(self) -> U64Vector4: ...
    @property
    def ulll(self) -> U64Vector4: ...
    @property
    def ouuu(self) -> U64Vector4: ...
    @property
    def ouuo(self) -> U64Vector4: ...
    @property
    def ouul(self) -> U64Vector4: ...
    @property
    def ouou(self) -> U64Vector4: ...
    @property
    def ouoo(self) -> U64Vector4: ...
    @property
    def ouol(self) -> U64Vector4: ...
    @property
    def oulu(self) -> U64Vector4: ...
    @property
    def oulo(self) -> U64Vector4: ...
    @property
    def oull(self) -> U64Vector4: ...
    @property
    def oouu(self) -> U64Vector4: ...
    @property
    def oouo(self) -> U64Vector4: ...
    @property
    def ooul(self) -> U64Vector4: ...
    @property
    def ooou(self) -> U64Vector4: ...
    @property
    def oolu(self) -> U64Vector4: ...
    @property
    def oluu(self) -> U64Vector4: ...
    @property
    def oluo(self) -> U64Vector4: ...
    @property
    def olul(self) -> U64Vector4: ...
    @property
    def olou(self) -> U64Vector4: ...
    @property
    def ollu(self) -> U64Vector4: ...
    @property
    def luuu(self) -> U64Vector4: ...
    @property
    def luuo(self) -> U64Vector4: ...
    @property
    def luul(self) -> U64Vector4: ...
    @property
    def luou(self) -> U64Vector4: ...
    @property
    def luoo(self) -> U64Vector4: ...
    @property
    def luol(self) -> U64Vector4: ...
    @property
    def lulu(self) -> U64Vector4: ...
    @property
    def lulo(self) -> U64Vector4: ...
    @property
    def lull(self) -> U64Vector4: ...
    @property
    def louu(self) -> U64Vector4: ...
    @property
    def louo(self) -> U64Vector4: ...
    @property
    def loul(self) -> U64Vector4: ...
    @property
    def loou(self) -> U64Vector4: ...
    @property
    def lolu(self) -> U64Vector4: ...
    @property
    def lluu(self) -> U64Vector4: ...
    @property
    def lluo(self) -> U64Vector4: ...
    @property
    def llul(self) -> U64Vector4: ...
    @property
    def llou(self) -> U64Vector4: ...
    @property
    def lllu(self) -> U64Vector4: ...
    @property
    def oo(self) -> U64Vector2: ...
    @property
    def ol(self) -> U64Vector2: ...
    @property
    def lo(self) -> U64Vector2: ...
    @property
    def ll(self) -> U64Vector2: ...
    @property
    def ooo(self) -> U64Vector3: ...
    @property
    def ool(self) -> U64Vector3: ...
    @property
    def olo(self) -> U64Vector3: ...
    @property
    def oll(self) -> U64Vector3: ...
    @property
    def loo(self) -> U64Vector3: ...
    @property
    def lol(self) -> U64Vector3: ...
    @property
    def llo(self) -> U64Vector3: ...
    @property
    def lll(self) -> U64Vector3: ...
    @property
    def oooo(self) -> U64Vector4: ...
    @property
    def oool(self) -> U64Vector4: ...
    @property
    def oolo(self) -> U64Vector4: ...
    @property
    def ooll(self) -> U64Vector4: ...
    @property
    def oloo(self) -> U64Vector4: ...
    @property
    def olol(self) -> U64Vector4: ...
    @property
    def ollo(self) -> U64Vector4: ...
    @property
    def olll(self) -> U64Vector4: ...
    @property
    def looo(self) -> U64Vector4: ...
    @property
    def lool(self) -> U64Vector4: ...
    @property
    def lolo(self) -> U64Vector4: ...
    @property
    def loll(self) -> U64Vector4: ...
    @property
    def lloo(self) -> U64Vector4: ...
    @property
    def llol(self) -> U64Vector4: ...
    @property
    def lllo(self) -> U64Vector4: ...
    @property
    def llll(self) -> U64Vector4: ...
    def min(self, n: Number, /) -> U64Vector1: ...
    def max(self, n: Number, /) -> U64Vector1: ...
    def clamp(self, min: Number, max: Number, /) -> U64Vector1: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U64Vector1Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector1: ...

@final
class U64Vector1Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U64Vector1): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U64Vector1]: ...
    @overload
    def __getitem__(self, index: int) -> U64Vector1: ...
    @overload
    def __getitem__(self, index: slice) -> U64Vector1Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector1Array: ...
    @classmethod
    def get_component_type(cls) -> type[U64Vector1]: ...

@final
class BVector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[_bool]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> _bool: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: BVector2) -> BVector2: ...
    @overload
    def __add__(self, other: Number) -> BVector2: ...
    @overload
    def __radd__(self, other: BVector2) -> BVector2: ...
    @overload
    def __radd__(self, other: Number) -> BVector2: ...
    @overload
    def __sub__(self, other: BVector2) -> BVector2: ...
    @overload
    def __sub__(self, other: Number) -> BVector2: ...
    @overload
    def __rsub__(self, other: BVector2) -> BVector2: ...
    @overload
    def __rsub__(self, other: Number) -> BVector2: ...
    @overload
    def __mul__(self, other: BVector2) -> BVector2: ...
    @overload
    def __mul__(self, other: Number) -> BVector2: ...
    @overload
    def __rmul__(self, other: BVector2) -> BVector2: ...
    @overload
    def __rmul__(self, other: Number) -> BVector2: ...
    def __neg__(self) -> BVector2: ...
    def __abs__(self) -> BVector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> _bool: ...
    @property
    def y(self) -> _bool: ...
    @property
    def xx(self) -> BVector2: ...
    @property
    def xy(self) -> BVector2: ...
    @property
    def xo(self) -> BVector2: ...
    @property
    def xl(self) -> BVector2: ...
    @property
    def yx(self) -> BVector2: ...
    @property
    def yy(self) -> BVector2: ...
    @property
    def yo(self) -> BVector2: ...
    @property
    def yl(self) -> BVector2: ...
    @property
    def ox(self) -> BVector2: ...
    @property
    def oy(self) -> BVector2: ...
    @property
    def lx(self) -> BVector2: ...
    @property
    def ly(self) -> BVector2: ...
    @property
    def xxx(self) -> BVector3: ...
    @property
    def xxy(self) -> BVector3: ...
    @property
    def xxo(self) -> BVector3: ...
    @property
    def xxl(self) -> BVector3: ...
    @property
    def xyx(self) -> BVector3: ...
    @property
    def xyy(self) -> BVector3: ...
    @property
    def xyo(self) -> BVector3: ...
    @property
    def xyl(self) -> BVector3: ...
    @property
    def xox(self) -> BVector3: ...
    @property
    def xoy(self) -> BVector3: ...
    @property
    def xoo(self) -> BVector3: ...
    @property
    def xol(self) -> BVector3: ...
    @property
    def xlx(self) -> BVector3: ...
    @property
    def xly(self) -> BVector3: ...
    @property
    def xlo(self) -> BVector3: ...
    @property
    def xll(self) -> BVector3: ...
    @property
    def yxx(self) -> BVector3: ...
    @property
    def yxy(self) -> BVector3: ...
    @property
    def yxo(self) -> BVector3: ...
    @property
    def yxl(self) -> BVector3: ...
    @property
    def yyx(self) -> BVector3: ...
    @property
    def yyy(self) -> BVector3: ...
    @property
    def yyo(self) -> BVector3: ...
    @property
    def yyl(self) -> BVector3: ...
    @property
    def yox(self) -> BVector3: ...
    @property
    def yoy(self) -> BVector3: ...
    @property
    def yoo(self) -> BVector3: ...
    @property
    def yol(self) -> BVector3: ...
    @property
    def ylx(self) -> BVector3: ...
    @property
    def yly(self) -> BVector3: ...
    @property
    def ylo(self) -> BVector3: ...
    @property
    def yll(self) -> BVector3: ...
    @property
    def oxx(self) -> BVector3: ...
    @property
    def oxy(self) -> BVector3: ...
    @property
    def oxo(self) -> BVector3: ...
    @property
    def oxl(self) -> BVector3: ...
    @property
    def oyx(self) -> BVector3: ...
    @property
    def oyy(self) -> BVector3: ...
    @property
    def oyo(self) -> BVector3: ...
    @property
    def oyl(self) -> BVector3: ...
    @property
    def oox(self) -> BVector3: ...
    @property
    def ooy(self) -> BVector3: ...
    @property
    def olx(self) -> BVector3: ...
    @property
    def oly(self) -> BVector3: ...
    @property
    def lxx(self) -> BVector3: ...
    @property
    def lxy(self) -> BVector3: ...
    @property
    def lxo(self) -> BVector3: ...
    @property
    def lxl(self) -> BVector3: ...
    @property
    def lyx(self) -> BVector3: ...
    @property
    def lyy(self) -> BVector3: ...
    @property
    def lyo(self) -> BVector3: ...
    @property
    def lyl(self) -> BVector3: ...
    @property
    def lox(self) -> BVector3: ...
    @property
    def loy(self) -> BVector3: ...
    @property
    def llx(self) -> BVector3: ...
    @property
    def lly(self) -> BVector3: ...
    @property
    def xxxx(self) -> BVector4: ...
    @property
    def xxxy(self) -> BVector4: ...
    @property
    def xxxo(self) -> BVector4: ...
    @property
    def xxxl(self) -> BVector4: ...
    @property
    def xxyx(self) -> BVector4: ...
    @property
    def xxyy(self) -> BVector4: ...
    @property
    def xxyo(self) -> BVector4: ...
    @property
    def xxyl(self) -> BVector4: ...
    @property
    def xxox(self) -> BVector4: ...
    @property
    def xxoy(self) -> BVector4: ...
    @property
    def xxoo(self) -> BVector4: ...
    @property
    def xxol(self) -> BVector4: ...
    @property
    def xxlx(self) -> BVector4: ...
    @property
    def xxly(self) -> BVector4: ...
    @property
    def xxlo(self) -> BVector4: ...
    @property
    def xxll(self) -> BVector4: ...
    @property
    def xyxx(self) -> BVector4: ...
    @property
    def xyxy(self) -> BVector4: ...
    @property
    def xyxo(self) -> BVector4: ...
    @property
    def xyxl(self) -> BVector4: ...
    @property
    def xyyx(self) -> BVector4: ...
    @property
    def xyyy(self) -> BVector4: ...
    @property
    def xyyo(self) -> BVector4: ...
    @property
    def xyyl(self) -> BVector4: ...
    @property
    def xyox(self) -> BVector4: ...
    @property
    def xyoy(self) -> BVector4: ...
    @property
    def xyoo(self) -> BVector4: ...
    @property
    def xyol(self) -> BVector4: ...
    @property
    def xylx(self) -> BVector4: ...
    @property
    def xyly(self) -> BVector4: ...
    @property
    def xylo(self) -> BVector4: ...
    @property
    def xyll(self) -> BVector4: ...
    @property
    def xoxx(self) -> BVector4: ...
    @property
    def xoxy(self) -> BVector4: ...
    @property
    def xoxo(self) -> BVector4: ...
    @property
    def xoxl(self) -> BVector4: ...
    @property
    def xoyx(self) -> BVector4: ...
    @property
    def xoyy(self) -> BVector4: ...
    @property
    def xoyo(self) -> BVector4: ...
    @property
    def xoyl(self) -> BVector4: ...
    @property
    def xoox(self) -> BVector4: ...
    @property
    def xooy(self) -> BVector4: ...
    @property
    def xooo(self) -> BVector4: ...
    @property
    def xool(self) -> BVector4: ...
    @property
    def xolx(self) -> BVector4: ...
    @property
    def xoly(self) -> BVector4: ...
    @property
    def xolo(self) -> BVector4: ...
    @property
    def xoll(self) -> BVector4: ...
    @property
    def xlxx(self) -> BVector4: ...
    @property
    def xlxy(self) -> BVector4: ...
    @property
    def xlxo(self) -> BVector4: ...
    @property
    def xlxl(self) -> BVector4: ...
    @property
    def xlyx(self) -> BVector4: ...
    @property
    def xlyy(self) -> BVector4: ...
    @property
    def xlyo(self) -> BVector4: ...
    @property
    def xlyl(self) -> BVector4: ...
    @property
    def xlox(self) -> BVector4: ...
    @property
    def xloy(self) -> BVector4: ...
    @property
    def xloo(self) -> BVector4: ...
    @property
    def xlol(self) -> BVector4: ...
    @property
    def xllx(self) -> BVector4: ...
    @property
    def xlly(self) -> BVector4: ...
    @property
    def xllo(self) -> BVector4: ...
    @property
    def xlll(self) -> BVector4: ...
    @property
    def yxxx(self) -> BVector4: ...
    @property
    def yxxy(self) -> BVector4: ...
    @property
    def yxxo(self) -> BVector4: ...
    @property
    def yxxl(self) -> BVector4: ...
    @property
    def yxyx(self) -> BVector4: ...
    @property
    def yxyy(self) -> BVector4: ...
    @property
    def yxyo(self) -> BVector4: ...
    @property
    def yxyl(self) -> BVector4: ...
    @property
    def yxox(self) -> BVector4: ...
    @property
    def yxoy(self) -> BVector4: ...
    @property
    def yxoo(self) -> BVector4: ...
    @property
    def yxol(self) -> BVector4: ...
    @property
    def yxlx(self) -> BVector4: ...
    @property
    def yxly(self) -> BVector4: ...
    @property
    def yxlo(self) -> BVector4: ...
    @property
    def yxll(self) -> BVector4: ...
    @property
    def yyxx(self) -> BVector4: ...
    @property
    def yyxy(self) -> BVector4: ...
    @property
    def yyxo(self) -> BVector4: ...
    @property
    def yyxl(self) -> BVector4: ...
    @property
    def yyyx(self) -> BVector4: ...
    @property
    def yyyy(self) -> BVector4: ...
    @property
    def yyyo(self) -> BVector4: ...
    @property
    def yyyl(self) -> BVector4: ...
    @property
    def yyox(self) -> BVector4: ...
    @property
    def yyoy(self) -> BVector4: ...
    @property
    def yyoo(self) -> BVector4: ...
    @property
    def yyol(self) -> BVector4: ...
    @property
    def yylx(self) -> BVector4: ...
    @property
    def yyly(self) -> BVector4: ...
    @property
    def yylo(self) -> BVector4: ...
    @property
    def yyll(self) -> BVector4: ...
    @property
    def yoxx(self) -> BVector4: ...
    @property
    def yoxy(self) -> BVector4: ...
    @property
    def yoxo(self) -> BVector4: ...
    @property
    def yoxl(self) -> BVector4: ...
    @property
    def yoyx(self) -> BVector4: ...
    @property
    def yoyy(self) -> BVector4: ...
    @property
    def yoyo(self) -> BVector4: ...
    @property
    def yoyl(self) -> BVector4: ...
    @property
    def yoox(self) -> BVector4: ...
    @property
    def yooy(self) -> BVector4: ...
    @property
    def yooo(self) -> BVector4: ...
    @property
    def yool(self) -> BVector4: ...
    @property
    def yolx(self) -> BVector4: ...
    @property
    def yoly(self) -> BVector4: ...
    @property
    def yolo(self) -> BVector4: ...
    @property
    def yoll(self) -> BVector4: ...
    @property
    def ylxx(self) -> BVector4: ...
    @property
    def ylxy(self) -> BVector4: ...
    @property
    def ylxo(self) -> BVector4: ...
    @property
    def ylxl(self) -> BVector4: ...
    @property
    def ylyx(self) -> BVector4: ...
    @property
    def ylyy(self) -> BVector4: ...
    @property
    def ylyo(self) -> BVector4: ...
    @property
    def ylyl(self) -> BVector4: ...
    @property
    def ylox(self) -> BVector4: ...
    @property
    def yloy(self) -> BVector4: ...
    @property
    def yloo(self) -> BVector4: ...
    @property
    def ylol(self) -> BVector4: ...
    @property
    def yllx(self) -> BVector4: ...
    @property
    def ylly(self) -> BVector4: ...
    @property
    def yllo(self) -> BVector4: ...
    @property
    def ylll(self) -> BVector4: ...
    @property
    def oxxx(self) -> BVector4: ...
    @property
    def oxxy(self) -> BVector4: ...
    @property
    def oxxo(self) -> BVector4: ...
    @property
    def oxxl(self) -> BVector4: ...
    @property
    def oxyx(self) -> BVector4: ...
    @property
    def oxyy(self) -> BVector4: ...
    @property
    def oxyo(self) -> BVector4: ...
    @property
    def oxyl(self) -> BVector4: ...
    @property
    def oxox(self) -> BVector4: ...
    @property
    def oxoy(self) -> BVector4: ...
    @property
    def oxoo(self) -> BVector4: ...
    @property
    def oxol(self) -> BVector4: ...
    @property
    def oxlx(self) -> BVector4: ...
    @property
    def oxly(self) -> BVector4: ...
    @property
    def oxlo(self) -> BVector4: ...
    @property
    def oxll(self) -> BVector4: ...
    @property
    def oyxx(self) -> BVector4: ...
    @property
    def oyxy(self) -> BVector4: ...
    @property
    def oyxo(self) -> BVector4: ...
    @property
    def oyxl(self) -> BVector4: ...
    @property
    def oyyx(self) -> BVector4: ...
    @property
    def oyyy(self) -> BVector4: ...
    @property
    def oyyo(self) -> BVector4: ...
    @property
    def oyyl(self) -> BVector4: ...
    @property
    def oyox(self) -> BVector4: ...
    @property
    def oyoy(self) -> BVector4: ...
    @property
    def oyoo(self) -> BVector4: ...
    @property
    def oyol(self) -> BVector4: ...
    @property
    def oylx(self) -> BVector4: ...
    @property
    def oyly(self) -> BVector4: ...
    @property
    def oylo(self) -> BVector4: ...
    @property
    def oyll(self) -> BVector4: ...
    @property
    def ooxx(self) -> BVector4: ...
    @property
    def ooxy(self) -> BVector4: ...
    @property
    def ooxo(self) -> BVector4: ...
    @property
    def ooxl(self) -> BVector4: ...
    @property
    def ooyx(self) -> BVector4: ...
    @property
    def ooyy(self) -> BVector4: ...
    @property
    def ooyo(self) -> BVector4: ...
    @property
    def ooyl(self) -> BVector4: ...
    @property
    def ooox(self) -> BVector4: ...
    @property
    def oooy(self) -> BVector4: ...
    @property
    def oolx(self) -> BVector4: ...
    @property
    def ooly(self) -> BVector4: ...
    @property
    def olxx(self) -> BVector4: ...
    @property
    def olxy(self) -> BVector4: ...
    @property
    def olxo(self) -> BVector4: ...
    @property
    def olxl(self) -> BVector4: ...
    @property
    def olyx(self) -> BVector4: ...
    @property
    def olyy(self) -> BVector4: ...
    @property
    def olyo(self) -> BVector4: ...
    @property
    def olyl(self) -> BVector4: ...
    @property
    def olox(self) -> BVector4: ...
    @property
    def oloy(self) -> BVector4: ...
    @property
    def ollx(self) -> BVector4: ...
    @property
    def olly(self) -> BVector4: ...
    @property
    def lxxx(self) -> BVector4: ...
    @property
    def lxxy(self) -> BVector4: ...
    @property
    def lxxo(self) -> BVector4: ...
    @property
    def lxxl(self) -> BVector4: ...
    @property
    def lxyx(self) -> BVector4: ...
    @property
    def lxyy(self) -> BVector4: ...
    @property
    def lxyo(self) -> BVector4: ...
    @property
    def lxyl(self) -> BVector4: ...
    @property
    def lxox(self) -> BVector4: ...
    @property
    def lxoy(self) -> BVector4: ...
    @property
    def lxoo(self) -> BVector4: ...
    @property
    def lxol(self) -> BVector4: ...
    @property
    def lxlx(self) -> BVector4: ...
    @property
    def lxly(self) -> BVector4: ...
    @property
    def lxlo(self) -> BVector4: ...
    @property
    def lxll(self) -> BVector4: ...
    @property
    def lyxx(self) -> BVector4: ...
    @property
    def lyxy(self) -> BVector4: ...
    @property
    def lyxo(self) -> BVector4: ...
    @property
    def lyxl(self) -> BVector4: ...
    @property
    def lyyx(self) -> BVector4: ...
    @property
    def lyyy(self) -> BVector4: ...
    @property
    def lyyo(self) -> BVector4: ...
    @property
    def lyyl(self) -> BVector4: ...
    @property
    def lyox(self) -> BVector4: ...
    @property
    def lyoy(self) -> BVector4: ...
    @property
    def lyoo(self) -> BVector4: ...
    @property
    def lyol(self) -> BVector4: ...
    @property
    def lylx(self) -> BVector4: ...
    @property
    def lyly(self) -> BVector4: ...
    @property
    def lylo(self) -> BVector4: ...
    @property
    def lyll(self) -> BVector4: ...
    @property
    def loxx(self) -> BVector4: ...
    @property
    def loxy(self) -> BVector4: ...
    @property
    def loxo(self) -> BVector4: ...
    @property
    def loxl(self) -> BVector4: ...
    @property
    def loyx(self) -> BVector4: ...
    @property
    def loyy(self) -> BVector4: ...
    @property
    def loyo(self) -> BVector4: ...
    @property
    def loyl(self) -> BVector4: ...
    @property
    def loox(self) -> BVector4: ...
    @property
    def looy(self) -> BVector4: ...
    @property
    def lolx(self) -> BVector4: ...
    @property
    def loly(self) -> BVector4: ...
    @property
    def llxx(self) -> BVector4: ...
    @property
    def llxy(self) -> BVector4: ...
    @property
    def llxo(self) -> BVector4: ...
    @property
    def llxl(self) -> BVector4: ...
    @property
    def llyx(self) -> BVector4: ...
    @property
    def llyy(self) -> BVector4: ...
    @property
    def llyo(self) -> BVector4: ...
    @property
    def llyl(self) -> BVector4: ...
    @property
    def llox(self) -> BVector4: ...
    @property
    def lloy(self) -> BVector4: ...
    @property
    def lllx(self) -> BVector4: ...
    @property
    def llly(self) -> BVector4: ...
    @property
    def r(self) -> _bool: ...
    @property
    def g(self) -> _bool: ...
    @property
    def rr(self) -> BVector2: ...
    @property
    def rg(self) -> BVector2: ...
    @property
    def ro(self) -> BVector2: ...
    @property
    def rl(self) -> BVector2: ...
    @property
    def gr(self) -> BVector2: ...
    @property
    def gg(self) -> BVector2: ...
    @property
    def go(self) -> BVector2: ...
    @property
    def gl(self) -> BVector2: ...
    @property
    def og(self) -> BVector2: ...
    @property
    def lr(self) -> BVector2: ...
    @property
    def lg(self) -> BVector2: ...
    @property
    def rrr(self) -> BVector3: ...
    @property
    def rrg(self) -> BVector3: ...
    @property
    def rro(self) -> BVector3: ...
    @property
    def rrl(self) -> BVector3: ...
    @property
    def rgr(self) -> BVector3: ...
    @property
    def rgg(self) -> BVector3: ...
    @property
    def rgo(self) -> BVector3: ...
    @property
    def rgl(self) -> BVector3: ...
    @property
    def ror(self) -> BVector3: ...
    @property
    def rog(self) -> BVector3: ...
    @property
    def roo(self) -> BVector3: ...
    @property
    def rol(self) -> BVector3: ...
    @property
    def rlr(self) -> BVector3: ...
    @property
    def rlg(self) -> BVector3: ...
    @property
    def rlo(self) -> BVector3: ...
    @property
    def rll(self) -> BVector3: ...
    @property
    def grr(self) -> BVector3: ...
    @property
    def grg(self) -> BVector3: ...
    @property
    def gro(self) -> BVector3: ...
    @property
    def grl(self) -> BVector3: ...
    @property
    def ggr(self) -> BVector3: ...
    @property
    def ggg(self) -> BVector3: ...
    @property
    def ggo(self) -> BVector3: ...
    @property
    def ggl(self) -> BVector3: ...
    @property
    def gor(self) -> BVector3: ...
    @property
    def gog(self) -> BVector3: ...
    @property
    def goo(self) -> BVector3: ...
    @property
    def gol(self) -> BVector3: ...
    @property
    def glr(self) -> BVector3: ...
    @property
    def glg(self) -> BVector3: ...
    @property
    def glo(self) -> BVector3: ...
    @property
    def gll(self) -> BVector3: ...
    @property
    def orr(self) -> BVector3: ...
    @property
    def org(self) -> BVector3: ...
    @property
    def oro(self) -> BVector3: ...
    @property
    def orl(self) -> BVector3: ...
    @property
    def ogr(self) -> BVector3: ...
    @property
    def ogg(self) -> BVector3: ...
    @property
    def ogo(self) -> BVector3: ...
    @property
    def ogl(self) -> BVector3: ...
    @property
    def oor(self) -> BVector3: ...
    @property
    def oog(self) -> BVector3: ...
    @property
    def olr(self) -> BVector3: ...
    @property
    def olg(self) -> BVector3: ...
    @property
    def lrr(self) -> BVector3: ...
    @property
    def lrg(self) -> BVector3: ...
    @property
    def lro(self) -> BVector3: ...
    @property
    def lrl(self) -> BVector3: ...
    @property
    def lgr(self) -> BVector3: ...
    @property
    def lgg(self) -> BVector3: ...
    @property
    def lgo(self) -> BVector3: ...
    @property
    def lgl(self) -> BVector3: ...
    @property
    def lor(self) -> BVector3: ...
    @property
    def log(self) -> BVector3: ...
    @property
    def llr(self) -> BVector3: ...
    @property
    def llg(self) -> BVector3: ...
    @property
    def rrrr(self) -> BVector4: ...
    @property
    def rrrg(self) -> BVector4: ...
    @property
    def rrro(self) -> BVector4: ...
    @property
    def rrrl(self) -> BVector4: ...
    @property
    def rrgr(self) -> BVector4: ...
    @property
    def rrgg(self) -> BVector4: ...
    @property
    def rrgo(self) -> BVector4: ...
    @property
    def rrgl(self) -> BVector4: ...
    @property
    def rror(self) -> BVector4: ...
    @property
    def rrog(self) -> BVector4: ...
    @property
    def rroo(self) -> BVector4: ...
    @property
    def rrol(self) -> BVector4: ...
    @property
    def rrlr(self) -> BVector4: ...
    @property
    def rrlg(self) -> BVector4: ...
    @property
    def rrlo(self) -> BVector4: ...
    @property
    def rrll(self) -> BVector4: ...
    @property
    def rgrr(self) -> BVector4: ...
    @property
    def rgrg(self) -> BVector4: ...
    @property
    def rgro(self) -> BVector4: ...
    @property
    def rgrl(self) -> BVector4: ...
    @property
    def rggr(self) -> BVector4: ...
    @property
    def rggg(self) -> BVector4: ...
    @property
    def rggo(self) -> BVector4: ...
    @property
    def rggl(self) -> BVector4: ...
    @property
    def rgor(self) -> BVector4: ...
    @property
    def rgog(self) -> BVector4: ...
    @property
    def rgoo(self) -> BVector4: ...
    @property
    def rgol(self) -> BVector4: ...
    @property
    def rglr(self) -> BVector4: ...
    @property
    def rglg(self) -> BVector4: ...
    @property
    def rglo(self) -> BVector4: ...
    @property
    def rgll(self) -> BVector4: ...
    @property
    def rorr(self) -> BVector4: ...
    @property
    def rorg(self) -> BVector4: ...
    @property
    def roro(self) -> BVector4: ...
    @property
    def rorl(self) -> BVector4: ...
    @property
    def rogr(self) -> BVector4: ...
    @property
    def rogg(self) -> BVector4: ...
    @property
    def rogo(self) -> BVector4: ...
    @property
    def rogl(self) -> BVector4: ...
    @property
    def roor(self) -> BVector4: ...
    @property
    def roog(self) -> BVector4: ...
    @property
    def rooo(self) -> BVector4: ...
    @property
    def rool(self) -> BVector4: ...
    @property
    def rolr(self) -> BVector4: ...
    @property
    def rolg(self) -> BVector4: ...
    @property
    def rolo(self) -> BVector4: ...
    @property
    def roll(self) -> BVector4: ...
    @property
    def rlrr(self) -> BVector4: ...
    @property
    def rlrg(self) -> BVector4: ...
    @property
    def rlro(self) -> BVector4: ...
    @property
    def rlrl(self) -> BVector4: ...
    @property
    def rlgr(self) -> BVector4: ...
    @property
    def rlgg(self) -> BVector4: ...
    @property
    def rlgo(self) -> BVector4: ...
    @property
    def rlgl(self) -> BVector4: ...
    @property
    def rlor(self) -> BVector4: ...
    @property
    def rlog(self) -> BVector4: ...
    @property
    def rloo(self) -> BVector4: ...
    @property
    def rlol(self) -> BVector4: ...
    @property
    def rllr(self) -> BVector4: ...
    @property
    def rllg(self) -> BVector4: ...
    @property
    def rllo(self) -> BVector4: ...
    @property
    def rlll(self) -> BVector4: ...
    @property
    def grrr(self) -> BVector4: ...
    @property
    def grrg(self) -> BVector4: ...
    @property
    def grro(self) -> BVector4: ...
    @property
    def grrl(self) -> BVector4: ...
    @property
    def grgr(self) -> BVector4: ...
    @property
    def grgg(self) -> BVector4: ...
    @property
    def grgo(self) -> BVector4: ...
    @property
    def grgl(self) -> BVector4: ...
    @property
    def gror(self) -> BVector4: ...
    @property
    def grog(self) -> BVector4: ...
    @property
    def groo(self) -> BVector4: ...
    @property
    def grol(self) -> BVector4: ...
    @property
    def grlr(self) -> BVector4: ...
    @property
    def grlg(self) -> BVector4: ...
    @property
    def grlo(self) -> BVector4: ...
    @property
    def grll(self) -> BVector4: ...
    @property
    def ggrr(self) -> BVector4: ...
    @property
    def ggrg(self) -> BVector4: ...
    @property
    def ggro(self) -> BVector4: ...
    @property
    def ggrl(self) -> BVector4: ...
    @property
    def gggr(self) -> BVector4: ...
    @property
    def gggg(self) -> BVector4: ...
    @property
    def gggo(self) -> BVector4: ...
    @property
    def gggl(self) -> BVector4: ...
    @property
    def ggor(self) -> BVector4: ...
    @property
    def ggog(self) -> BVector4: ...
    @property
    def ggoo(self) -> BVector4: ...
    @property
    def ggol(self) -> BVector4: ...
    @property
    def gglr(self) -> BVector4: ...
    @property
    def gglg(self) -> BVector4: ...
    @property
    def gglo(self) -> BVector4: ...
    @property
    def ggll(self) -> BVector4: ...
    @property
    def gorr(self) -> BVector4: ...
    @property
    def gorg(self) -> BVector4: ...
    @property
    def goro(self) -> BVector4: ...
    @property
    def gorl(self) -> BVector4: ...
    @property
    def gogr(self) -> BVector4: ...
    @property
    def gogg(self) -> BVector4: ...
    @property
    def gogo(self) -> BVector4: ...
    @property
    def gogl(self) -> BVector4: ...
    @property
    def goor(self) -> BVector4: ...
    @property
    def goog(self) -> BVector4: ...
    @property
    def gooo(self) -> BVector4: ...
    @property
    def gool(self) -> BVector4: ...
    @property
    def golr(self) -> BVector4: ...
    @property
    def golg(self) -> BVector4: ...
    @property
    def golo(self) -> BVector4: ...
    @property
    def goll(self) -> BVector4: ...
    @property
    def glrr(self) -> BVector4: ...
    @property
    def glrg(self) -> BVector4: ...
    @property
    def glro(self) -> BVector4: ...
    @property
    def glrl(self) -> BVector4: ...
    @property
    def glgr(self) -> BVector4: ...
    @property
    def glgg(self) -> BVector4: ...
    @property
    def glgo(self) -> BVector4: ...
    @property
    def glgl(self) -> BVector4: ...
    @property
    def glor(self) -> BVector4: ...
    @property
    def glog(self) -> BVector4: ...
    @property
    def gloo(self) -> BVector4: ...
    @property
    def glol(self) -> BVector4: ...
    @property
    def gllr(self) -> BVector4: ...
    @property
    def gllg(self) -> BVector4: ...
    @property
    def gllo(self) -> BVector4: ...
    @property
    def glll(self) -> BVector4: ...
    @property
    def orrr(self) -> BVector4: ...
    @property
    def orrg(self) -> BVector4: ...
    @property
    def orro(self) -> BVector4: ...
    @property
    def orrl(self) -> BVector4: ...
    @property
    def orgr(self) -> BVector4: ...
    @property
    def orgg(self) -> BVector4: ...
    @property
    def orgo(self) -> BVector4: ...
    @property
    def orgl(self) -> BVector4: ...
    @property
    def oror(self) -> BVector4: ...
    @property
    def orog(self) -> BVector4: ...
    @property
    def oroo(self) -> BVector4: ...
    @property
    def orol(self) -> BVector4: ...
    @property
    def orlr(self) -> BVector4: ...
    @property
    def orlg(self) -> BVector4: ...
    @property
    def orlo(self) -> BVector4: ...
    @property
    def orll(self) -> BVector4: ...
    @property
    def ogrr(self) -> BVector4: ...
    @property
    def ogrg(self) -> BVector4: ...
    @property
    def ogro(self) -> BVector4: ...
    @property
    def ogrl(self) -> BVector4: ...
    @property
    def oggr(self) -> BVector4: ...
    @property
    def oggg(self) -> BVector4: ...
    @property
    def oggo(self) -> BVector4: ...
    @property
    def oggl(self) -> BVector4: ...
    @property
    def ogor(self) -> BVector4: ...
    @property
    def ogog(self) -> BVector4: ...
    @property
    def ogoo(self) -> BVector4: ...
    @property
    def ogol(self) -> BVector4: ...
    @property
    def oglr(self) -> BVector4: ...
    @property
    def oglg(self) -> BVector4: ...
    @property
    def oglo(self) -> BVector4: ...
    @property
    def ogll(self) -> BVector4: ...
    @property
    def oorr(self) -> BVector4: ...
    @property
    def oorg(self) -> BVector4: ...
    @property
    def ooro(self) -> BVector4: ...
    @property
    def oorl(self) -> BVector4: ...
    @property
    def oogr(self) -> BVector4: ...
    @property
    def oogg(self) -> BVector4: ...
    @property
    def oogo(self) -> BVector4: ...
    @property
    def oogl(self) -> BVector4: ...
    @property
    def ooor(self) -> BVector4: ...
    @property
    def ooog(self) -> BVector4: ...
    @property
    def oolr(self) -> BVector4: ...
    @property
    def oolg(self) -> BVector4: ...
    @property
    def olrr(self) -> BVector4: ...
    @property
    def olrg(self) -> BVector4: ...
    @property
    def olro(self) -> BVector4: ...
    @property
    def olrl(self) -> BVector4: ...
    @property
    def olgr(self) -> BVector4: ...
    @property
    def olgg(self) -> BVector4: ...
    @property
    def olgo(self) -> BVector4: ...
    @property
    def olgl(self) -> BVector4: ...
    @property
    def olor(self) -> BVector4: ...
    @property
    def olog(self) -> BVector4: ...
    @property
    def ollr(self) -> BVector4: ...
    @property
    def ollg(self) -> BVector4: ...
    @property
    def lrrr(self) -> BVector4: ...
    @property
    def lrrg(self) -> BVector4: ...
    @property
    def lrro(self) -> BVector4: ...
    @property
    def lrrl(self) -> BVector4: ...
    @property
    def lrgr(self) -> BVector4: ...
    @property
    def lrgg(self) -> BVector4: ...
    @property
    def lrgo(self) -> BVector4: ...
    @property
    def lrgl(self) -> BVector4: ...
    @property
    def lror(self) -> BVector4: ...
    @property
    def lrog(self) -> BVector4: ...
    @property
    def lroo(self) -> BVector4: ...
    @property
    def lrol(self) -> BVector4: ...
    @property
    def lrlr(self) -> BVector4: ...
    @property
    def lrlg(self) -> BVector4: ...
    @property
    def lrlo(self) -> BVector4: ...
    @property
    def lrll(self) -> BVector4: ...
    @property
    def lgrr(self) -> BVector4: ...
    @property
    def lgrg(self) -> BVector4: ...
    @property
    def lgro(self) -> BVector4: ...
    @property
    def lgrl(self) -> BVector4: ...
    @property
    def lggr(self) -> BVector4: ...
    @property
    def lggg(self) -> BVector4: ...
    @property
    def lggo(self) -> BVector4: ...
    @property
    def lggl(self) -> BVector4: ...
    @property
    def lgor(self) -> BVector4: ...
    @property
    def lgog(self) -> BVector4: ...
    @property
    def lgoo(self) -> BVector4: ...
    @property
    def lgol(self) -> BVector4: ...
    @property
    def lglr(self) -> BVector4: ...
    @property
    def lglg(self) -> BVector4: ...
    @property
    def lglo(self) -> BVector4: ...
    @property
    def lgll(self) -> BVector4: ...
    @property
    def lorr(self) -> BVector4: ...
    @property
    def lorg(self) -> BVector4: ...
    @property
    def loro(self) -> BVector4: ...
    @property
    def lorl(self) -> BVector4: ...
    @property
    def logr(self) -> BVector4: ...
    @property
    def logg(self) -> BVector4: ...
    @property
    def logo(self) -> BVector4: ...
    @property
    def logl(self) -> BVector4: ...
    @property
    def loor(self) -> BVector4: ...
    @property
    def loog(self) -> BVector4: ...
    @property
    def lolr(self) -> BVector4: ...
    @property
    def lolg(self) -> BVector4: ...
    @property
    def llrr(self) -> BVector4: ...
    @property
    def llrg(self) -> BVector4: ...
    @property
    def llro(self) -> BVector4: ...
    @property
    def llrl(self) -> BVector4: ...
    @property
    def llgr(self) -> BVector4: ...
    @property
    def llgg(self) -> BVector4: ...
    @property
    def llgo(self) -> BVector4: ...
    @property
    def llgl(self) -> BVector4: ...
    @property
    def llor(self) -> BVector4: ...
    @property
    def llog(self) -> BVector4: ...
    @property
    def lllr(self) -> BVector4: ...
    @property
    def lllg(self) -> BVector4: ...
    @property
    def s(self) -> _bool: ...
    @property
    def t(self) -> _bool: ...
    @property
    def ss(self) -> BVector2: ...
    @property
    def st(self) -> BVector2: ...
    @property
    def so(self) -> BVector2: ...
    @property
    def sl(self) -> BVector2: ...
    @property
    def ts(self) -> BVector2: ...
    @property
    def tt(self) -> BVector2: ...
    @property
    def to(self) -> BVector2: ...
    @property
    def tl(self) -> BVector2: ...
    @property
    def os(self) -> BVector2: ...
    @property
    def ot(self) -> BVector2: ...
    @property
    def ls(self) -> BVector2: ...
    @property
    def lt(self) -> BVector2: ...
    @property
    def sss(self) -> BVector3: ...
    @property
    def sst(self) -> BVector3: ...
    @property
    def sso(self) -> BVector3: ...
    @property
    def ssl(self) -> BVector3: ...
    @property
    def sts(self) -> BVector3: ...
    @property
    def stt(self) -> BVector3: ...
    @property
    def sto(self) -> BVector3: ...
    @property
    def stl(self) -> BVector3: ...
    @property
    def sos(self) -> BVector3: ...
    @property
    def sot(self) -> BVector3: ...
    @property
    def soo(self) -> BVector3: ...
    @property
    def sol(self) -> BVector3: ...
    @property
    def sls(self) -> BVector3: ...
    @property
    def slt(self) -> BVector3: ...
    @property
    def slo(self) -> BVector3: ...
    @property
    def sll(self) -> BVector3: ...
    @property
    def tss(self) -> BVector3: ...
    @property
    def tst(self) -> BVector3: ...
    @property
    def tso(self) -> BVector3: ...
    @property
    def tsl(self) -> BVector3: ...
    @property
    def tts(self) -> BVector3: ...
    @property
    def ttt(self) -> BVector3: ...
    @property
    def tto(self) -> BVector3: ...
    @property
    def ttl(self) -> BVector3: ...
    @property
    def tos(self) -> BVector3: ...
    @property
    def tot(self) -> BVector3: ...
    @property
    def too(self) -> BVector3: ...
    @property
    def tol(self) -> BVector3: ...
    @property
    def tls(self) -> BVector3: ...
    @property
    def tlt(self) -> BVector3: ...
    @property
    def tlo(self) -> BVector3: ...
    @property
    def tll(self) -> BVector3: ...
    @property
    def oss(self) -> BVector3: ...
    @property
    def ost(self) -> BVector3: ...
    @property
    def oso(self) -> BVector3: ...
    @property
    def osl(self) -> BVector3: ...
    @property
    def ots(self) -> BVector3: ...
    @property
    def ott(self) -> BVector3: ...
    @property
    def oto(self) -> BVector3: ...
    @property
    def otl(self) -> BVector3: ...
    @property
    def oos(self) -> BVector3: ...
    @property
    def oot(self) -> BVector3: ...
    @property
    def ols(self) -> BVector3: ...
    @property
    def olt(self) -> BVector3: ...
    @property
    def lss(self) -> BVector3: ...
    @property
    def lst(self) -> BVector3: ...
    @property
    def lso(self) -> BVector3: ...
    @property
    def lsl(self) -> BVector3: ...
    @property
    def lts(self) -> BVector3: ...
    @property
    def ltt(self) -> BVector3: ...
    @property
    def lto(self) -> BVector3: ...
    @property
    def ltl(self) -> BVector3: ...
    @property
    def los(self) -> BVector3: ...
    @property
    def lot(self) -> BVector3: ...
    @property
    def lls(self) -> BVector3: ...
    @property
    def llt(self) -> BVector3: ...
    @property
    def ssss(self) -> BVector4: ...
    @property
    def ssst(self) -> BVector4: ...
    @property
    def ssso(self) -> BVector4: ...
    @property
    def sssl(self) -> BVector4: ...
    @property
    def ssts(self) -> BVector4: ...
    @property
    def sstt(self) -> BVector4: ...
    @property
    def ssto(self) -> BVector4: ...
    @property
    def sstl(self) -> BVector4: ...
    @property
    def ssos(self) -> BVector4: ...
    @property
    def ssot(self) -> BVector4: ...
    @property
    def ssoo(self) -> BVector4: ...
    @property
    def ssol(self) -> BVector4: ...
    @property
    def ssls(self) -> BVector4: ...
    @property
    def sslt(self) -> BVector4: ...
    @property
    def sslo(self) -> BVector4: ...
    @property
    def ssll(self) -> BVector4: ...
    @property
    def stss(self) -> BVector4: ...
    @property
    def stst(self) -> BVector4: ...
    @property
    def stso(self) -> BVector4: ...
    @property
    def stsl(self) -> BVector4: ...
    @property
    def stts(self) -> BVector4: ...
    @property
    def sttt(self) -> BVector4: ...
    @property
    def stto(self) -> BVector4: ...
    @property
    def sttl(self) -> BVector4: ...
    @property
    def stos(self) -> BVector4: ...
    @property
    def stot(self) -> BVector4: ...
    @property
    def stoo(self) -> BVector4: ...
    @property
    def stol(self) -> BVector4: ...
    @property
    def stls(self) -> BVector4: ...
    @property
    def stlt(self) -> BVector4: ...
    @property
    def stlo(self) -> BVector4: ...
    @property
    def stll(self) -> BVector4: ...
    @property
    def soss(self) -> BVector4: ...
    @property
    def sost(self) -> BVector4: ...
    @property
    def soso(self) -> BVector4: ...
    @property
    def sosl(self) -> BVector4: ...
    @property
    def sots(self) -> BVector4: ...
    @property
    def sott(self) -> BVector4: ...
    @property
    def soto(self) -> BVector4: ...
    @property
    def sotl(self) -> BVector4: ...
    @property
    def soos(self) -> BVector4: ...
    @property
    def soot(self) -> BVector4: ...
    @property
    def sooo(self) -> BVector4: ...
    @property
    def sool(self) -> BVector4: ...
    @property
    def sols(self) -> BVector4: ...
    @property
    def solt(self) -> BVector4: ...
    @property
    def solo(self) -> BVector4: ...
    @property
    def soll(self) -> BVector4: ...
    @property
    def slss(self) -> BVector4: ...
    @property
    def slst(self) -> BVector4: ...
    @property
    def slso(self) -> BVector4: ...
    @property
    def slsl(self) -> BVector4: ...
    @property
    def slts(self) -> BVector4: ...
    @property
    def sltt(self) -> BVector4: ...
    @property
    def slto(self) -> BVector4: ...
    @property
    def sltl(self) -> BVector4: ...
    @property
    def slos(self) -> BVector4: ...
    @property
    def slot(self) -> BVector4: ...
    @property
    def sloo(self) -> BVector4: ...
    @property
    def slol(self) -> BVector4: ...
    @property
    def slls(self) -> BVector4: ...
    @property
    def sllt(self) -> BVector4: ...
    @property
    def sllo(self) -> BVector4: ...
    @property
    def slll(self) -> BVector4: ...
    @property
    def tsss(self) -> BVector4: ...
    @property
    def tsst(self) -> BVector4: ...
    @property
    def tsso(self) -> BVector4: ...
    @property
    def tssl(self) -> BVector4: ...
    @property
    def tsts(self) -> BVector4: ...
    @property
    def tstt(self) -> BVector4: ...
    @property
    def tsto(self) -> BVector4: ...
    @property
    def tstl(self) -> BVector4: ...
    @property
    def tsos(self) -> BVector4: ...
    @property
    def tsot(self) -> BVector4: ...
    @property
    def tsoo(self) -> BVector4: ...
    @property
    def tsol(self) -> BVector4: ...
    @property
    def tsls(self) -> BVector4: ...
    @property
    def tslt(self) -> BVector4: ...
    @property
    def tslo(self) -> BVector4: ...
    @property
    def tsll(self) -> BVector4: ...
    @property
    def ttss(self) -> BVector4: ...
    @property
    def ttst(self) -> BVector4: ...
    @property
    def ttso(self) -> BVector4: ...
    @property
    def ttsl(self) -> BVector4: ...
    @property
    def ttts(self) -> BVector4: ...
    @property
    def tttt(self) -> BVector4: ...
    @property
    def ttto(self) -> BVector4: ...
    @property
    def tttl(self) -> BVector4: ...
    @property
    def ttos(self) -> BVector4: ...
    @property
    def ttot(self) -> BVector4: ...
    @property
    def ttoo(self) -> BVector4: ...
    @property
    def ttol(self) -> BVector4: ...
    @property
    def ttls(self) -> BVector4: ...
    @property
    def ttlt(self) -> BVector4: ...
    @property
    def ttlo(self) -> BVector4: ...
    @property
    def ttll(self) -> BVector4: ...
    @property
    def toss(self) -> BVector4: ...
    @property
    def tost(self) -> BVector4: ...
    @property
    def toso(self) -> BVector4: ...
    @property
    def tosl(self) -> BVector4: ...
    @property
    def tots(self) -> BVector4: ...
    @property
    def tott(self) -> BVector4: ...
    @property
    def toto(self) -> BVector4: ...
    @property
    def totl(self) -> BVector4: ...
    @property
    def toos(self) -> BVector4: ...
    @property
    def toot(self) -> BVector4: ...
    @property
    def tooo(self) -> BVector4: ...
    @property
    def tool(self) -> BVector4: ...
    @property
    def tols(self) -> BVector4: ...
    @property
    def tolt(self) -> BVector4: ...
    @property
    def tolo(self) -> BVector4: ...
    @property
    def toll(self) -> BVector4: ...
    @property
    def tlss(self) -> BVector4: ...
    @property
    def tlst(self) -> BVector4: ...
    @property
    def tlso(self) -> BVector4: ...
    @property
    def tlsl(self) -> BVector4: ...
    @property
    def tlts(self) -> BVector4: ...
    @property
    def tltt(self) -> BVector4: ...
    @property
    def tlto(self) -> BVector4: ...
    @property
    def tltl(self) -> BVector4: ...
    @property
    def tlos(self) -> BVector4: ...
    @property
    def tlot(self) -> BVector4: ...
    @property
    def tloo(self) -> BVector4: ...
    @property
    def tlol(self) -> BVector4: ...
    @property
    def tlls(self) -> BVector4: ...
    @property
    def tllt(self) -> BVector4: ...
    @property
    def tllo(self) -> BVector4: ...
    @property
    def tlll(self) -> BVector4: ...
    @property
    def osss(self) -> BVector4: ...
    @property
    def osst(self) -> BVector4: ...
    @property
    def osso(self) -> BVector4: ...
    @property
    def ossl(self) -> BVector4: ...
    @property
    def osts(self) -> BVector4: ...
    @property
    def ostt(self) -> BVector4: ...
    @property
    def osto(self) -> BVector4: ...
    @property
    def ostl(self) -> BVector4: ...
    @property
    def osos(self) -> BVector4: ...
    @property
    def osot(self) -> BVector4: ...
    @property
    def osoo(self) -> BVector4: ...
    @property
    def osol(self) -> BVector4: ...
    @property
    def osls(self) -> BVector4: ...
    @property
    def oslt(self) -> BVector4: ...
    @property
    def oslo(self) -> BVector4: ...
    @property
    def osll(self) -> BVector4: ...
    @property
    def otss(self) -> BVector4: ...
    @property
    def otst(self) -> BVector4: ...
    @property
    def otso(self) -> BVector4: ...
    @property
    def otsl(self) -> BVector4: ...
    @property
    def otts(self) -> BVector4: ...
    @property
    def ottt(self) -> BVector4: ...
    @property
    def otto(self) -> BVector4: ...
    @property
    def ottl(self) -> BVector4: ...
    @property
    def otos(self) -> BVector4: ...
    @property
    def otot(self) -> BVector4: ...
    @property
    def otoo(self) -> BVector4: ...
    @property
    def otol(self) -> BVector4: ...
    @property
    def otls(self) -> BVector4: ...
    @property
    def otlt(self) -> BVector4: ...
    @property
    def otlo(self) -> BVector4: ...
    @property
    def otll(self) -> BVector4: ...
    @property
    def ooss(self) -> BVector4: ...
    @property
    def oost(self) -> BVector4: ...
    @property
    def ooso(self) -> BVector4: ...
    @property
    def oosl(self) -> BVector4: ...
    @property
    def oots(self) -> BVector4: ...
    @property
    def oott(self) -> BVector4: ...
    @property
    def ooto(self) -> BVector4: ...
    @property
    def ootl(self) -> BVector4: ...
    @property
    def ooos(self) -> BVector4: ...
    @property
    def ooot(self) -> BVector4: ...
    @property
    def ools(self) -> BVector4: ...
    @property
    def oolt(self) -> BVector4: ...
    @property
    def olss(self) -> BVector4: ...
    @property
    def olst(self) -> BVector4: ...
    @property
    def olso(self) -> BVector4: ...
    @property
    def olsl(self) -> BVector4: ...
    @property
    def olts(self) -> BVector4: ...
    @property
    def oltt(self) -> BVector4: ...
    @property
    def olto(self) -> BVector4: ...
    @property
    def oltl(self) -> BVector4: ...
    @property
    def olos(self) -> BVector4: ...
    @property
    def olot(self) -> BVector4: ...
    @property
    def olls(self) -> BVector4: ...
    @property
    def ollt(self) -> BVector4: ...
    @property
    def lsss(self) -> BVector4: ...
    @property
    def lsst(self) -> BVector4: ...
    @property
    def lsso(self) -> BVector4: ...
    @property
    def lssl(self) -> BVector4: ...
    @property
    def lsts(self) -> BVector4: ...
    @property
    def lstt(self) -> BVector4: ...
    @property
    def lsto(self) -> BVector4: ...
    @property
    def lstl(self) -> BVector4: ...
    @property
    def lsos(self) -> BVector4: ...
    @property
    def lsot(self) -> BVector4: ...
    @property
    def lsoo(self) -> BVector4: ...
    @property
    def lsol(self) -> BVector4: ...
    @property
    def lsls(self) -> BVector4: ...
    @property
    def lslt(self) -> BVector4: ...
    @property
    def lslo(self) -> BVector4: ...
    @property
    def lsll(self) -> BVector4: ...
    @property
    def ltss(self) -> BVector4: ...
    @property
    def ltst(self) -> BVector4: ...
    @property
    def ltso(self) -> BVector4: ...
    @property
    def ltsl(self) -> BVector4: ...
    @property
    def ltts(self) -> BVector4: ...
    @property
    def lttt(self) -> BVector4: ...
    @property
    def ltto(self) -> BVector4: ...
    @property
    def lttl(self) -> BVector4: ...
    @property
    def ltos(self) -> BVector4: ...
    @property
    def ltot(self) -> BVector4: ...
    @property
    def ltoo(self) -> BVector4: ...
    @property
    def ltol(self) -> BVector4: ...
    @property
    def ltls(self) -> BVector4: ...
    @property
    def ltlt(self) -> BVector4: ...
    @property
    def ltlo(self) -> BVector4: ...
    @property
    def ltll(self) -> BVector4: ...
    @property
    def loss(self) -> BVector4: ...
    @property
    def lost(self) -> BVector4: ...
    @property
    def loso(self) -> BVector4: ...
    @property
    def losl(self) -> BVector4: ...
    @property
    def lots(self) -> BVector4: ...
    @property
    def lott(self) -> BVector4: ...
    @property
    def loto(self) -> BVector4: ...
    @property
    def lotl(self) -> BVector4: ...
    @property
    def loos(self) -> BVector4: ...
    @property
    def loot(self) -> BVector4: ...
    @property
    def lols(self) -> BVector4: ...
    @property
    def lolt(self) -> BVector4: ...
    @property
    def llss(self) -> BVector4: ...
    @property
    def llst(self) -> BVector4: ...
    @property
    def llso(self) -> BVector4: ...
    @property
    def llsl(self) -> BVector4: ...
    @property
    def llts(self) -> BVector4: ...
    @property
    def lltt(self) -> BVector4: ...
    @property
    def llto(self) -> BVector4: ...
    @property
    def lltl(self) -> BVector4: ...
    @property
    def llos(self) -> BVector4: ...
    @property
    def llot(self) -> BVector4: ...
    @property
    def llls(self) -> BVector4: ...
    @property
    def lllt(self) -> BVector4: ...
    @property
    def u(self) -> _bool: ...
    @property
    def v(self) -> _bool: ...
    @property
    def uu(self) -> BVector2: ...
    @property
    def uv(self) -> BVector2: ...
    @property
    def uo(self) -> BVector2: ...
    @property
    def ul(self) -> BVector2: ...
    @property
    def vu(self) -> BVector2: ...
    @property
    def vv(self) -> BVector2: ...
    @property
    def vo(self) -> BVector2: ...
    @property
    def vl(self) -> BVector2: ...
    @property
    def ou(self) -> BVector2: ...
    @property
    def ov(self) -> BVector2: ...
    @property
    def lu(self) -> BVector2: ...
    @property
    def lv(self) -> BVector2: ...
    @property
    def uuu(self) -> BVector3: ...
    @property
    def uuv(self) -> BVector3: ...
    @property
    def uuo(self) -> BVector3: ...
    @property
    def uul(self) -> BVector3: ...
    @property
    def uvu(self) -> BVector3: ...
    @property
    def uvv(self) -> BVector3: ...
    @property
    def uvo(self) -> BVector3: ...
    @property
    def uvl(self) -> BVector3: ...
    @property
    def uou(self) -> BVector3: ...
    @property
    def uov(self) -> BVector3: ...
    @property
    def uoo(self) -> BVector3: ...
    @property
    def uol(self) -> BVector3: ...
    @property
    def ulu(self) -> BVector3: ...
    @property
    def ulv(self) -> BVector3: ...
    @property
    def ulo(self) -> BVector3: ...
    @property
    def ull(self) -> BVector3: ...
    @property
    def vuu(self) -> BVector3: ...
    @property
    def vuv(self) -> BVector3: ...
    @property
    def vuo(self) -> BVector3: ...
    @property
    def vul(self) -> BVector3: ...
    @property
    def vvu(self) -> BVector3: ...
    @property
    def vvv(self) -> BVector3: ...
    @property
    def vvo(self) -> BVector3: ...
    @property
    def vvl(self) -> BVector3: ...
    @property
    def vou(self) -> BVector3: ...
    @property
    def vov(self) -> BVector3: ...
    @property
    def voo(self) -> BVector3: ...
    @property
    def vol(self) -> BVector3: ...
    @property
    def vlu(self) -> BVector3: ...
    @property
    def vlv(self) -> BVector3: ...
    @property
    def vlo(self) -> BVector3: ...
    @property
    def vll(self) -> BVector3: ...
    @property
    def ouu(self) -> BVector3: ...
    @property
    def ouv(self) -> BVector3: ...
    @property
    def ouo(self) -> BVector3: ...
    @property
    def oul(self) -> BVector3: ...
    @property
    def ovu(self) -> BVector3: ...
    @property
    def ovv(self) -> BVector3: ...
    @property
    def ovo(self) -> BVector3: ...
    @property
    def ovl(self) -> BVector3: ...
    @property
    def oou(self) -> BVector3: ...
    @property
    def oov(self) -> BVector3: ...
    @property
    def olu(self) -> BVector3: ...
    @property
    def olv(self) -> BVector3: ...
    @property
    def luu(self) -> BVector3: ...
    @property
    def luv(self) -> BVector3: ...
    @property
    def luo(self) -> BVector3: ...
    @property
    def lul(self) -> BVector3: ...
    @property
    def lvu(self) -> BVector3: ...
    @property
    def lvv(self) -> BVector3: ...
    @property
    def lvo(self) -> BVector3: ...
    @property
    def lvl(self) -> BVector3: ...
    @property
    def lou(self) -> BVector3: ...
    @property
    def lov(self) -> BVector3: ...
    @property
    def llu(self) -> BVector3: ...
    @property
    def llv(self) -> BVector3: ...
    @property
    def uuuu(self) -> BVector4: ...
    @property
    def uuuv(self) -> BVector4: ...
    @property
    def uuuo(self) -> BVector4: ...
    @property
    def uuul(self) -> BVector4: ...
    @property
    def uuvu(self) -> BVector4: ...
    @property
    def uuvv(self) -> BVector4: ...
    @property
    def uuvo(self) -> BVector4: ...
    @property
    def uuvl(self) -> BVector4: ...
    @property
    def uuou(self) -> BVector4: ...
    @property
    def uuov(self) -> BVector4: ...
    @property
    def uuoo(self) -> BVector4: ...
    @property
    def uuol(self) -> BVector4: ...
    @property
    def uulu(self) -> BVector4: ...
    @property
    def uulv(self) -> BVector4: ...
    @property
    def uulo(self) -> BVector4: ...
    @property
    def uull(self) -> BVector4: ...
    @property
    def uvuu(self) -> BVector4: ...
    @property
    def uvuv(self) -> BVector4: ...
    @property
    def uvuo(self) -> BVector4: ...
    @property
    def uvul(self) -> BVector4: ...
    @property
    def uvvu(self) -> BVector4: ...
    @property
    def uvvv(self) -> BVector4: ...
    @property
    def uvvo(self) -> BVector4: ...
    @property
    def uvvl(self) -> BVector4: ...
    @property
    def uvou(self) -> BVector4: ...
    @property
    def uvov(self) -> BVector4: ...
    @property
    def uvoo(self) -> BVector4: ...
    @property
    def uvol(self) -> BVector4: ...
    @property
    def uvlu(self) -> BVector4: ...
    @property
    def uvlv(self) -> BVector4: ...
    @property
    def uvlo(self) -> BVector4: ...
    @property
    def uvll(self) -> BVector4: ...
    @property
    def uouu(self) -> BVector4: ...
    @property
    def uouv(self) -> BVector4: ...
    @property
    def uouo(self) -> BVector4: ...
    @property
    def uoul(self) -> BVector4: ...
    @property
    def uovu(self) -> BVector4: ...
    @property
    def uovv(self) -> BVector4: ...
    @property
    def uovo(self) -> BVector4: ...
    @property
    def uovl(self) -> BVector4: ...
    @property
    def uoou(self) -> BVector4: ...
    @property
    def uoov(self) -> BVector4: ...
    @property
    def uooo(self) -> BVector4: ...
    @property
    def uool(self) -> BVector4: ...
    @property
    def uolu(self) -> BVector4: ...
    @property
    def uolv(self) -> BVector4: ...
    @property
    def uolo(self) -> BVector4: ...
    @property
    def uoll(self) -> BVector4: ...
    @property
    def uluu(self) -> BVector4: ...
    @property
    def uluv(self) -> BVector4: ...
    @property
    def uluo(self) -> BVector4: ...
    @property
    def ulul(self) -> BVector4: ...
    @property
    def ulvu(self) -> BVector4: ...
    @property
    def ulvv(self) -> BVector4: ...
    @property
    def ulvo(self) -> BVector4: ...
    @property
    def ulvl(self) -> BVector4: ...
    @property
    def ulou(self) -> BVector4: ...
    @property
    def ulov(self) -> BVector4: ...
    @property
    def uloo(self) -> BVector4: ...
    @property
    def ulol(self) -> BVector4: ...
    @property
    def ullu(self) -> BVector4: ...
    @property
    def ullv(self) -> BVector4: ...
    @property
    def ullo(self) -> BVector4: ...
    @property
    def ulll(self) -> BVector4: ...
    @property
    def vuuu(self) -> BVector4: ...
    @property
    def vuuv(self) -> BVector4: ...
    @property
    def vuuo(self) -> BVector4: ...
    @property
    def vuul(self) -> BVector4: ...
    @property
    def vuvu(self) -> BVector4: ...
    @property
    def vuvv(self) -> BVector4: ...
    @property
    def vuvo(self) -> BVector4: ...
    @property
    def vuvl(self) -> BVector4: ...
    @property
    def vuou(self) -> BVector4: ...
    @property
    def vuov(self) -> BVector4: ...
    @property
    def vuoo(self) -> BVector4: ...
    @property
    def vuol(self) -> BVector4: ...
    @property
    def vulu(self) -> BVector4: ...
    @property
    def vulv(self) -> BVector4: ...
    @property
    def vulo(self) -> BVector4: ...
    @property
    def vull(self) -> BVector4: ...
    @property
    def vvuu(self) -> BVector4: ...
    @property
    def vvuv(self) -> BVector4: ...
    @property
    def vvuo(self) -> BVector4: ...
    @property
    def vvul(self) -> BVector4: ...
    @property
    def vvvu(self) -> BVector4: ...
    @property
    def vvvv(self) -> BVector4: ...
    @property
    def vvvo(self) -> BVector4: ...
    @property
    def vvvl(self) -> BVector4: ...
    @property
    def vvou(self) -> BVector4: ...
    @property
    def vvov(self) -> BVector4: ...
    @property
    def vvoo(self) -> BVector4: ...
    @property
    def vvol(self) -> BVector4: ...
    @property
    def vvlu(self) -> BVector4: ...
    @property
    def vvlv(self) -> BVector4: ...
    @property
    def vvlo(self) -> BVector4: ...
    @property
    def vvll(self) -> BVector4: ...
    @property
    def vouu(self) -> BVector4: ...
    @property
    def vouv(self) -> BVector4: ...
    @property
    def vouo(self) -> BVector4: ...
    @property
    def voul(self) -> BVector4: ...
    @property
    def vovu(self) -> BVector4: ...
    @property
    def vovv(self) -> BVector4: ...
    @property
    def vovo(self) -> BVector4: ...
    @property
    def vovl(self) -> BVector4: ...
    @property
    def voou(self) -> BVector4: ...
    @property
    def voov(self) -> BVector4: ...
    @property
    def vooo(self) -> BVector4: ...
    @property
    def vool(self) -> BVector4: ...
    @property
    def volu(self) -> BVector4: ...
    @property
    def volv(self) -> BVector4: ...
    @property
    def volo(self) -> BVector4: ...
    @property
    def voll(self) -> BVector4: ...
    @property
    def vluu(self) -> BVector4: ...
    @property
    def vluv(self) -> BVector4: ...
    @property
    def vluo(self) -> BVector4: ...
    @property
    def vlul(self) -> BVector4: ...
    @property
    def vlvu(self) -> BVector4: ...
    @property
    def vlvv(self) -> BVector4: ...
    @property
    def vlvo(self) -> BVector4: ...
    @property
    def vlvl(self) -> BVector4: ...
    @property
    def vlou(self) -> BVector4: ...
    @property
    def vlov(self) -> BVector4: ...
    @property
    def vloo(self) -> BVector4: ...
    @property
    def vlol(self) -> BVector4: ...
    @property
    def vllu(self) -> BVector4: ...
    @property
    def vllv(self) -> BVector4: ...
    @property
    def vllo(self) -> BVector4: ...
    @property
    def vlll(self) -> BVector4: ...
    @property
    def ouuu(self) -> BVector4: ...
    @property
    def ouuv(self) -> BVector4: ...
    @property
    def ouuo(self) -> BVector4: ...
    @property
    def ouul(self) -> BVector4: ...
    @property
    def ouvu(self) -> BVector4: ...
    @property
    def ouvv(self) -> BVector4: ...
    @property
    def ouvo(self) -> BVector4: ...
    @property
    def ouvl(self) -> BVector4: ...
    @property
    def ouou(self) -> BVector4: ...
    @property
    def ouov(self) -> BVector4: ...
    @property
    def ouoo(self) -> BVector4: ...
    @property
    def ouol(self) -> BVector4: ...
    @property
    def oulu(self) -> BVector4: ...
    @property
    def oulv(self) -> BVector4: ...
    @property
    def oulo(self) -> BVector4: ...
    @property
    def oull(self) -> BVector4: ...
    @property
    def ovuu(self) -> BVector4: ...
    @property
    def ovuv(self) -> BVector4: ...
    @property
    def ovuo(self) -> BVector4: ...
    @property
    def ovul(self) -> BVector4: ...
    @property
    def ovvu(self) -> BVector4: ...
    @property
    def ovvv(self) -> BVector4: ...
    @property
    def ovvo(self) -> BVector4: ...
    @property
    def ovvl(self) -> BVector4: ...
    @property
    def ovou(self) -> BVector4: ...
    @property
    def ovov(self) -> BVector4: ...
    @property
    def ovoo(self) -> BVector4: ...
    @property
    def ovol(self) -> BVector4: ...
    @property
    def ovlu(self) -> BVector4: ...
    @property
    def ovlv(self) -> BVector4: ...
    @property
    def ovlo(self) -> BVector4: ...
    @property
    def ovll(self) -> BVector4: ...
    @property
    def oouu(self) -> BVector4: ...
    @property
    def oouv(self) -> BVector4: ...
    @property
    def oouo(self) -> BVector4: ...
    @property
    def ooul(self) -> BVector4: ...
    @property
    def oovu(self) -> BVector4: ...
    @property
    def oovv(self) -> BVector4: ...
    @property
    def oovo(self) -> BVector4: ...
    @property
    def oovl(self) -> BVector4: ...
    @property
    def ooou(self) -> BVector4: ...
    @property
    def ooov(self) -> BVector4: ...
    @property
    def oolu(self) -> BVector4: ...
    @property
    def oolv(self) -> BVector4: ...
    @property
    def oluu(self) -> BVector4: ...
    @property
    def oluv(self) -> BVector4: ...
    @property
    def oluo(self) -> BVector4: ...
    @property
    def olul(self) -> BVector4: ...
    @property
    def olvu(self) -> BVector4: ...
    @property
    def olvv(self) -> BVector4: ...
    @property
    def olvo(self) -> BVector4: ...
    @property
    def olvl(self) -> BVector4: ...
    @property
    def olou(self) -> BVector4: ...
    @property
    def olov(self) -> BVector4: ...
    @property
    def ollu(self) -> BVector4: ...
    @property
    def ollv(self) -> BVector4: ...
    @property
    def luuu(self) -> BVector4: ...
    @property
    def luuv(self) -> BVector4: ...
    @property
    def luuo(self) -> BVector4: ...
    @property
    def luul(self) -> BVector4: ...
    @property
    def luvu(self) -> BVector4: ...
    @property
    def luvv(self) -> BVector4: ...
    @property
    def luvo(self) -> BVector4: ...
    @property
    def luvl(self) -> BVector4: ...
    @property
    def luou(self) -> BVector4: ...
    @property
    def luov(self) -> BVector4: ...
    @property
    def luoo(self) -> BVector4: ...
    @property
    def luol(self) -> BVector4: ...
    @property
    def lulu(self) -> BVector4: ...
    @property
    def lulv(self) -> BVector4: ...
    @property
    def lulo(self) -> BVector4: ...
    @property
    def lull(self) -> BVector4: ...
    @property
    def lvuu(self) -> BVector4: ...
    @property
    def lvuv(self) -> BVector4: ...
    @property
    def lvuo(self) -> BVector4: ...
    @property
    def lvul(self) -> BVector4: ...
    @property
    def lvvu(self) -> BVector4: ...
    @property
    def lvvv(self) -> BVector4: ...
    @property
    def lvvo(self) -> BVector4: ...
    @property
    def lvvl(self) -> BVector4: ...
    @property
    def lvou(self) -> BVector4: ...
    @property
    def lvov(self) -> BVector4: ...
    @property
    def lvoo(self) -> BVector4: ...
    @property
    def lvol(self) -> BVector4: ...
    @property
    def lvlu(self) -> BVector4: ...
    @property
    def lvlv(self) -> BVector4: ...
    @property
    def lvlo(self) -> BVector4: ...
    @property
    def lvll(self) -> BVector4: ...
    @property
    def louu(self) -> BVector4: ...
    @property
    def louv(self) -> BVector4: ...
    @property
    def louo(self) -> BVector4: ...
    @property
    def loul(self) -> BVector4: ...
    @property
    def lovu(self) -> BVector4: ...
    @property
    def lovv(self) -> BVector4: ...
    @property
    def lovo(self) -> BVector4: ...
    @property
    def lovl(self) -> BVector4: ...
    @property
    def loou(self) -> BVector4: ...
    @property
    def loov(self) -> BVector4: ...
    @property
    def lolu(self) -> BVector4: ...
    @property
    def lolv(self) -> BVector4: ...
    @property
    def lluu(self) -> BVector4: ...
    @property
    def lluv(self) -> BVector4: ...
    @property
    def lluo(self) -> BVector4: ...
    @property
    def llul(self) -> BVector4: ...
    @property
    def llvu(self) -> BVector4: ...
    @property
    def llvv(self) -> BVector4: ...
    @property
    def llvo(self) -> BVector4: ...
    @property
    def llvl(self) -> BVector4: ...
    @property
    def llou(self) -> BVector4: ...
    @property
    def llov(self) -> BVector4: ...
    @property
    def lllu(self) -> BVector4: ...
    @property
    def lllv(self) -> BVector4: ...
    @property
    def oo(self) -> BVector2: ...
    @property
    def ol(self) -> BVector2: ...
    @property
    def lo(self) -> BVector2: ...
    @property
    def ll(self) -> BVector2: ...
    @property
    def ooo(self) -> BVector3: ...
    @property
    def ool(self) -> BVector3: ...
    @property
    def olo(self) -> BVector3: ...
    @property
    def oll(self) -> BVector3: ...
    @property
    def loo(self) -> BVector3: ...
    @property
    def lol(self) -> BVector3: ...
    @property
    def llo(self) -> BVector3: ...
    @property
    def lll(self) -> BVector3: ...
    @property
    def oooo(self) -> BVector4: ...
    @property
    def oool(self) -> BVector4: ...
    @property
    def oolo(self) -> BVector4: ...
    @property
    def ooll(self) -> BVector4: ...
    @property
    def oloo(self) -> BVector4: ...
    @property
    def olol(self) -> BVector4: ...
    @property
    def ollo(self) -> BVector4: ...
    @property
    def olll(self) -> BVector4: ...
    @property
    def looo(self) -> BVector4: ...
    @property
    def lool(self) -> BVector4: ...
    @property
    def lolo(self) -> BVector4: ...
    @property
    def loll(self) -> BVector4: ...
    @property
    def lloo(self) -> BVector4: ...
    @property
    def llol(self) -> BVector4: ...
    @property
    def lllo(self) -> BVector4: ...
    @property
    def llll(self) -> BVector4: ...
    def min(self, n: Number, /) -> BVector2: ...
    def max(self, n: Number, /) -> BVector2: ...
    def clamp(self, min: Number, max: Number, /) -> BVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[_bool, _bool]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[BVector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector2: ...

@final
class BVector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: BVector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[BVector2]: ...
    @overload
    def __getitem__(self, index: int) -> BVector2: ...
    @overload
    def __getitem__(self, index: slice) -> BVector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[BVector2]: ...

@final
class DVector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DVector2) -> DVector2: ...
    @overload
    def __add__(self, other: Number) -> DVector2: ...
    @overload
    def __radd__(self, other: DVector2) -> DVector2: ...
    @overload
    def __radd__(self, other: Number) -> DVector2: ...
    @overload
    def __sub__(self, other: DVector2) -> DVector2: ...
    @overload
    def __sub__(self, other: Number) -> DVector2: ...
    @overload
    def __rsub__(self, other: DVector2) -> DVector2: ...
    @overload
    def __rsub__(self, other: Number) -> DVector2: ...
    @overload
    def __mul__(self, other: DVector2) -> DVector2: ...
    @overload
    def __mul__(self, other: Number) -> DVector2: ...
    @overload
    def __rmul__(self, other: DVector2) -> DVector2: ...
    @overload
    def __rmul__(self, other: Number) -> DVector2: ...
    def __matmul__(self, other: DVector2) -> float: ...
    def __rmatmul__(self, other: DVector2) -> float: ...
    @overload
    def __mod__(self, other: DVector2) -> DVector2: ...
    @overload
    def __mod__(self, other: Number) -> DVector2: ...
    @overload
    def __rmod__(self, other: DVector2) -> DVector2: ...
    @overload
    def __rmod__(self, other: Number) -> DVector2: ...
    @overload
    def __pow__(self, other: DVector2) -> DVector2: ...
    @overload
    def __pow__(self, other: Number) -> DVector2: ...
    @overload
    def __rpow__(self, other: DVector2) -> DVector2: ...
    @overload
    def __rpow__(self, other: Number) -> DVector2: ...
    @overload
    def __truediv__(self, other: DVector2) -> DVector2: ...
    @overload
    def __truediv__(self, other: Number) -> DVector2: ...
    @overload
    def __rtruediv__(self, other: DVector2) -> DVector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> DVector2: ...
    def __neg__(self) -> DVector2: ...
    def __abs__(self) -> DVector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def xx(self) -> DVector2: ...
    @property
    def xy(self) -> DVector2: ...
    @property
    def xo(self) -> DVector2: ...
    @property
    def xl(self) -> DVector2: ...
    @property
    def yx(self) -> DVector2: ...
    @property
    def yy(self) -> DVector2: ...
    @property
    def yo(self) -> DVector2: ...
    @property
    def yl(self) -> DVector2: ...
    @property
    def ox(self) -> DVector2: ...
    @property
    def oy(self) -> DVector2: ...
    @property
    def lx(self) -> DVector2: ...
    @property
    def ly(self) -> DVector2: ...
    @property
    def xxx(self) -> DVector3: ...
    @property
    def xxy(self) -> DVector3: ...
    @property
    def xxo(self) -> DVector3: ...
    @property
    def xxl(self) -> DVector3: ...
    @property
    def xyx(self) -> DVector3: ...
    @property
    def xyy(self) -> DVector3: ...
    @property
    def xyo(self) -> DVector3: ...
    @property
    def xyl(self) -> DVector3: ...
    @property
    def xox(self) -> DVector3: ...
    @property
    def xoy(self) -> DVector3: ...
    @property
    def xoo(self) -> DVector3: ...
    @property
    def xol(self) -> DVector3: ...
    @property
    def xlx(self) -> DVector3: ...
    @property
    def xly(self) -> DVector3: ...
    @property
    def xlo(self) -> DVector3: ...
    @property
    def xll(self) -> DVector3: ...
    @property
    def yxx(self) -> DVector3: ...
    @property
    def yxy(self) -> DVector3: ...
    @property
    def yxo(self) -> DVector3: ...
    @property
    def yxl(self) -> DVector3: ...
    @property
    def yyx(self) -> DVector3: ...
    @property
    def yyy(self) -> DVector3: ...
    @property
    def yyo(self) -> DVector3: ...
    @property
    def yyl(self) -> DVector3: ...
    @property
    def yox(self) -> DVector3: ...
    @property
    def yoy(self) -> DVector3: ...
    @property
    def yoo(self) -> DVector3: ...
    @property
    def yol(self) -> DVector3: ...
    @property
    def ylx(self) -> DVector3: ...
    @property
    def yly(self) -> DVector3: ...
    @property
    def ylo(self) -> DVector3: ...
    @property
    def yll(self) -> DVector3: ...
    @property
    def oxx(self) -> DVector3: ...
    @property
    def oxy(self) -> DVector3: ...
    @property
    def oxo(self) -> DVector3: ...
    @property
    def oxl(self) -> DVector3: ...
    @property
    def oyx(self) -> DVector3: ...
    @property
    def oyy(self) -> DVector3: ...
    @property
    def oyo(self) -> DVector3: ...
    @property
    def oyl(self) -> DVector3: ...
    @property
    def oox(self) -> DVector3: ...
    @property
    def ooy(self) -> DVector3: ...
    @property
    def olx(self) -> DVector3: ...
    @property
    def oly(self) -> DVector3: ...
    @property
    def lxx(self) -> DVector3: ...
    @property
    def lxy(self) -> DVector3: ...
    @property
    def lxo(self) -> DVector3: ...
    @property
    def lxl(self) -> DVector3: ...
    @property
    def lyx(self) -> DVector3: ...
    @property
    def lyy(self) -> DVector3: ...
    @property
    def lyo(self) -> DVector3: ...
    @property
    def lyl(self) -> DVector3: ...
    @property
    def lox(self) -> DVector3: ...
    @property
    def loy(self) -> DVector3: ...
    @property
    def llx(self) -> DVector3: ...
    @property
    def lly(self) -> DVector3: ...
    @property
    def xxxx(self) -> DVector4: ...
    @property
    def xxxy(self) -> DVector4: ...
    @property
    def xxxo(self) -> DVector4: ...
    @property
    def xxxl(self) -> DVector4: ...
    @property
    def xxyx(self) -> DVector4: ...
    @property
    def xxyy(self) -> DVector4: ...
    @property
    def xxyo(self) -> DVector4: ...
    @property
    def xxyl(self) -> DVector4: ...
    @property
    def xxox(self) -> DVector4: ...
    @property
    def xxoy(self) -> DVector4: ...
    @property
    def xxoo(self) -> DVector4: ...
    @property
    def xxol(self) -> DVector4: ...
    @property
    def xxlx(self) -> DVector4: ...
    @property
    def xxly(self) -> DVector4: ...
    @property
    def xxlo(self) -> DVector4: ...
    @property
    def xxll(self) -> DVector4: ...
    @property
    def xyxx(self) -> DVector4: ...
    @property
    def xyxy(self) -> DVector4: ...
    @property
    def xyxo(self) -> DVector4: ...
    @property
    def xyxl(self) -> DVector4: ...
    @property
    def xyyx(self) -> DVector4: ...
    @property
    def xyyy(self) -> DVector4: ...
    @property
    def xyyo(self) -> DVector4: ...
    @property
    def xyyl(self) -> DVector4: ...
    @property
    def xyox(self) -> DVector4: ...
    @property
    def xyoy(self) -> DVector4: ...
    @property
    def xyoo(self) -> DVector4: ...
    @property
    def xyol(self) -> DVector4: ...
    @property
    def xylx(self) -> DVector4: ...
    @property
    def xyly(self) -> DVector4: ...
    @property
    def xylo(self) -> DVector4: ...
    @property
    def xyll(self) -> DVector4: ...
    @property
    def xoxx(self) -> DVector4: ...
    @property
    def xoxy(self) -> DVector4: ...
    @property
    def xoxo(self) -> DVector4: ...
    @property
    def xoxl(self) -> DVector4: ...
    @property
    def xoyx(self) -> DVector4: ...
    @property
    def xoyy(self) -> DVector4: ...
    @property
    def xoyo(self) -> DVector4: ...
    @property
    def xoyl(self) -> DVector4: ...
    @property
    def xoox(self) -> DVector4: ...
    @property
    def xooy(self) -> DVector4: ...
    @property
    def xooo(self) -> DVector4: ...
    @property
    def xool(self) -> DVector4: ...
    @property
    def xolx(self) -> DVector4: ...
    @property
    def xoly(self) -> DVector4: ...
    @property
    def xolo(self) -> DVector4: ...
    @property
    def xoll(self) -> DVector4: ...
    @property
    def xlxx(self) -> DVector4: ...
    @property
    def xlxy(self) -> DVector4: ...
    @property
    def xlxo(self) -> DVector4: ...
    @property
    def xlxl(self) -> DVector4: ...
    @property
    def xlyx(self) -> DVector4: ...
    @property
    def xlyy(self) -> DVector4: ...
    @property
    def xlyo(self) -> DVector4: ...
    @property
    def xlyl(self) -> DVector4: ...
    @property
    def xlox(self) -> DVector4: ...
    @property
    def xloy(self) -> DVector4: ...
    @property
    def xloo(self) -> DVector4: ...
    @property
    def xlol(self) -> DVector4: ...
    @property
    def xllx(self) -> DVector4: ...
    @property
    def xlly(self) -> DVector4: ...
    @property
    def xllo(self) -> DVector4: ...
    @property
    def xlll(self) -> DVector4: ...
    @property
    def yxxx(self) -> DVector4: ...
    @property
    def yxxy(self) -> DVector4: ...
    @property
    def yxxo(self) -> DVector4: ...
    @property
    def yxxl(self) -> DVector4: ...
    @property
    def yxyx(self) -> DVector4: ...
    @property
    def yxyy(self) -> DVector4: ...
    @property
    def yxyo(self) -> DVector4: ...
    @property
    def yxyl(self) -> DVector4: ...
    @property
    def yxox(self) -> DVector4: ...
    @property
    def yxoy(self) -> DVector4: ...
    @property
    def yxoo(self) -> DVector4: ...
    @property
    def yxol(self) -> DVector4: ...
    @property
    def yxlx(self) -> DVector4: ...
    @property
    def yxly(self) -> DVector4: ...
    @property
    def yxlo(self) -> DVector4: ...
    @property
    def yxll(self) -> DVector4: ...
    @property
    def yyxx(self) -> DVector4: ...
    @property
    def yyxy(self) -> DVector4: ...
    @property
    def yyxo(self) -> DVector4: ...
    @property
    def yyxl(self) -> DVector4: ...
    @property
    def yyyx(self) -> DVector4: ...
    @property
    def yyyy(self) -> DVector4: ...
    @property
    def yyyo(self) -> DVector4: ...
    @property
    def yyyl(self) -> DVector4: ...
    @property
    def yyox(self) -> DVector4: ...
    @property
    def yyoy(self) -> DVector4: ...
    @property
    def yyoo(self) -> DVector4: ...
    @property
    def yyol(self) -> DVector4: ...
    @property
    def yylx(self) -> DVector4: ...
    @property
    def yyly(self) -> DVector4: ...
    @property
    def yylo(self) -> DVector4: ...
    @property
    def yyll(self) -> DVector4: ...
    @property
    def yoxx(self) -> DVector4: ...
    @property
    def yoxy(self) -> DVector4: ...
    @property
    def yoxo(self) -> DVector4: ...
    @property
    def yoxl(self) -> DVector4: ...
    @property
    def yoyx(self) -> DVector4: ...
    @property
    def yoyy(self) -> DVector4: ...
    @property
    def yoyo(self) -> DVector4: ...
    @property
    def yoyl(self) -> DVector4: ...
    @property
    def yoox(self) -> DVector4: ...
    @property
    def yooy(self) -> DVector4: ...
    @property
    def yooo(self) -> DVector4: ...
    @property
    def yool(self) -> DVector4: ...
    @property
    def yolx(self) -> DVector4: ...
    @property
    def yoly(self) -> DVector4: ...
    @property
    def yolo(self) -> DVector4: ...
    @property
    def yoll(self) -> DVector4: ...
    @property
    def ylxx(self) -> DVector4: ...
    @property
    def ylxy(self) -> DVector4: ...
    @property
    def ylxo(self) -> DVector4: ...
    @property
    def ylxl(self) -> DVector4: ...
    @property
    def ylyx(self) -> DVector4: ...
    @property
    def ylyy(self) -> DVector4: ...
    @property
    def ylyo(self) -> DVector4: ...
    @property
    def ylyl(self) -> DVector4: ...
    @property
    def ylox(self) -> DVector4: ...
    @property
    def yloy(self) -> DVector4: ...
    @property
    def yloo(self) -> DVector4: ...
    @property
    def ylol(self) -> DVector4: ...
    @property
    def yllx(self) -> DVector4: ...
    @property
    def ylly(self) -> DVector4: ...
    @property
    def yllo(self) -> DVector4: ...
    @property
    def ylll(self) -> DVector4: ...
    @property
    def oxxx(self) -> DVector4: ...
    @property
    def oxxy(self) -> DVector4: ...
    @property
    def oxxo(self) -> DVector4: ...
    @property
    def oxxl(self) -> DVector4: ...
    @property
    def oxyx(self) -> DVector4: ...
    @property
    def oxyy(self) -> DVector4: ...
    @property
    def oxyo(self) -> DVector4: ...
    @property
    def oxyl(self) -> DVector4: ...
    @property
    def oxox(self) -> DVector4: ...
    @property
    def oxoy(self) -> DVector4: ...
    @property
    def oxoo(self) -> DVector4: ...
    @property
    def oxol(self) -> DVector4: ...
    @property
    def oxlx(self) -> DVector4: ...
    @property
    def oxly(self) -> DVector4: ...
    @property
    def oxlo(self) -> DVector4: ...
    @property
    def oxll(self) -> DVector4: ...
    @property
    def oyxx(self) -> DVector4: ...
    @property
    def oyxy(self) -> DVector4: ...
    @property
    def oyxo(self) -> DVector4: ...
    @property
    def oyxl(self) -> DVector4: ...
    @property
    def oyyx(self) -> DVector4: ...
    @property
    def oyyy(self) -> DVector4: ...
    @property
    def oyyo(self) -> DVector4: ...
    @property
    def oyyl(self) -> DVector4: ...
    @property
    def oyox(self) -> DVector4: ...
    @property
    def oyoy(self) -> DVector4: ...
    @property
    def oyoo(self) -> DVector4: ...
    @property
    def oyol(self) -> DVector4: ...
    @property
    def oylx(self) -> DVector4: ...
    @property
    def oyly(self) -> DVector4: ...
    @property
    def oylo(self) -> DVector4: ...
    @property
    def oyll(self) -> DVector4: ...
    @property
    def ooxx(self) -> DVector4: ...
    @property
    def ooxy(self) -> DVector4: ...
    @property
    def ooxo(self) -> DVector4: ...
    @property
    def ooxl(self) -> DVector4: ...
    @property
    def ooyx(self) -> DVector4: ...
    @property
    def ooyy(self) -> DVector4: ...
    @property
    def ooyo(self) -> DVector4: ...
    @property
    def ooyl(self) -> DVector4: ...
    @property
    def ooox(self) -> DVector4: ...
    @property
    def oooy(self) -> DVector4: ...
    @property
    def oolx(self) -> DVector4: ...
    @property
    def ooly(self) -> DVector4: ...
    @property
    def olxx(self) -> DVector4: ...
    @property
    def olxy(self) -> DVector4: ...
    @property
    def olxo(self) -> DVector4: ...
    @property
    def olxl(self) -> DVector4: ...
    @property
    def olyx(self) -> DVector4: ...
    @property
    def olyy(self) -> DVector4: ...
    @property
    def olyo(self) -> DVector4: ...
    @property
    def olyl(self) -> DVector4: ...
    @property
    def olox(self) -> DVector4: ...
    @property
    def oloy(self) -> DVector4: ...
    @property
    def ollx(self) -> DVector4: ...
    @property
    def olly(self) -> DVector4: ...
    @property
    def lxxx(self) -> DVector4: ...
    @property
    def lxxy(self) -> DVector4: ...
    @property
    def lxxo(self) -> DVector4: ...
    @property
    def lxxl(self) -> DVector4: ...
    @property
    def lxyx(self) -> DVector4: ...
    @property
    def lxyy(self) -> DVector4: ...
    @property
    def lxyo(self) -> DVector4: ...
    @property
    def lxyl(self) -> DVector4: ...
    @property
    def lxox(self) -> DVector4: ...
    @property
    def lxoy(self) -> DVector4: ...
    @property
    def lxoo(self) -> DVector4: ...
    @property
    def lxol(self) -> DVector4: ...
    @property
    def lxlx(self) -> DVector4: ...
    @property
    def lxly(self) -> DVector4: ...
    @property
    def lxlo(self) -> DVector4: ...
    @property
    def lxll(self) -> DVector4: ...
    @property
    def lyxx(self) -> DVector4: ...
    @property
    def lyxy(self) -> DVector4: ...
    @property
    def lyxo(self) -> DVector4: ...
    @property
    def lyxl(self) -> DVector4: ...
    @property
    def lyyx(self) -> DVector4: ...
    @property
    def lyyy(self) -> DVector4: ...
    @property
    def lyyo(self) -> DVector4: ...
    @property
    def lyyl(self) -> DVector4: ...
    @property
    def lyox(self) -> DVector4: ...
    @property
    def lyoy(self) -> DVector4: ...
    @property
    def lyoo(self) -> DVector4: ...
    @property
    def lyol(self) -> DVector4: ...
    @property
    def lylx(self) -> DVector4: ...
    @property
    def lyly(self) -> DVector4: ...
    @property
    def lylo(self) -> DVector4: ...
    @property
    def lyll(self) -> DVector4: ...
    @property
    def loxx(self) -> DVector4: ...
    @property
    def loxy(self) -> DVector4: ...
    @property
    def loxo(self) -> DVector4: ...
    @property
    def loxl(self) -> DVector4: ...
    @property
    def loyx(self) -> DVector4: ...
    @property
    def loyy(self) -> DVector4: ...
    @property
    def loyo(self) -> DVector4: ...
    @property
    def loyl(self) -> DVector4: ...
    @property
    def loox(self) -> DVector4: ...
    @property
    def looy(self) -> DVector4: ...
    @property
    def lolx(self) -> DVector4: ...
    @property
    def loly(self) -> DVector4: ...
    @property
    def llxx(self) -> DVector4: ...
    @property
    def llxy(self) -> DVector4: ...
    @property
    def llxo(self) -> DVector4: ...
    @property
    def llxl(self) -> DVector4: ...
    @property
    def llyx(self) -> DVector4: ...
    @property
    def llyy(self) -> DVector4: ...
    @property
    def llyo(self) -> DVector4: ...
    @property
    def llyl(self) -> DVector4: ...
    @property
    def llox(self) -> DVector4: ...
    @property
    def lloy(self) -> DVector4: ...
    @property
    def lllx(self) -> DVector4: ...
    @property
    def llly(self) -> DVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def g(self) -> float: ...
    @property
    def rr(self) -> DVector2: ...
    @property
    def rg(self) -> DVector2: ...
    @property
    def ro(self) -> DVector2: ...
    @property
    def rl(self) -> DVector2: ...
    @property
    def gr(self) -> DVector2: ...
    @property
    def gg(self) -> DVector2: ...
    @property
    def go(self) -> DVector2: ...
    @property
    def gl(self) -> DVector2: ...
    @property
    def og(self) -> DVector2: ...
    @property
    def lr(self) -> DVector2: ...
    @property
    def lg(self) -> DVector2: ...
    @property
    def rrr(self) -> DVector3: ...
    @property
    def rrg(self) -> DVector3: ...
    @property
    def rro(self) -> DVector3: ...
    @property
    def rrl(self) -> DVector3: ...
    @property
    def rgr(self) -> DVector3: ...
    @property
    def rgg(self) -> DVector3: ...
    @property
    def rgo(self) -> DVector3: ...
    @property
    def rgl(self) -> DVector3: ...
    @property
    def ror(self) -> DVector3: ...
    @property
    def rog(self) -> DVector3: ...
    @property
    def roo(self) -> DVector3: ...
    @property
    def rol(self) -> DVector3: ...
    @property
    def rlr(self) -> DVector3: ...
    @property
    def rlg(self) -> DVector3: ...
    @property
    def rlo(self) -> DVector3: ...
    @property
    def rll(self) -> DVector3: ...
    @property
    def grr(self) -> DVector3: ...
    @property
    def grg(self) -> DVector3: ...
    @property
    def gro(self) -> DVector3: ...
    @property
    def grl(self) -> DVector3: ...
    @property
    def ggr(self) -> DVector3: ...
    @property
    def ggg(self) -> DVector3: ...
    @property
    def ggo(self) -> DVector3: ...
    @property
    def ggl(self) -> DVector3: ...
    @property
    def gor(self) -> DVector3: ...
    @property
    def gog(self) -> DVector3: ...
    @property
    def goo(self) -> DVector3: ...
    @property
    def gol(self) -> DVector3: ...
    @property
    def glr(self) -> DVector3: ...
    @property
    def glg(self) -> DVector3: ...
    @property
    def glo(self) -> DVector3: ...
    @property
    def gll(self) -> DVector3: ...
    @property
    def orr(self) -> DVector3: ...
    @property
    def org(self) -> DVector3: ...
    @property
    def oro(self) -> DVector3: ...
    @property
    def orl(self) -> DVector3: ...
    @property
    def ogr(self) -> DVector3: ...
    @property
    def ogg(self) -> DVector3: ...
    @property
    def ogo(self) -> DVector3: ...
    @property
    def ogl(self) -> DVector3: ...
    @property
    def oor(self) -> DVector3: ...
    @property
    def oog(self) -> DVector3: ...
    @property
    def olr(self) -> DVector3: ...
    @property
    def olg(self) -> DVector3: ...
    @property
    def lrr(self) -> DVector3: ...
    @property
    def lrg(self) -> DVector3: ...
    @property
    def lro(self) -> DVector3: ...
    @property
    def lrl(self) -> DVector3: ...
    @property
    def lgr(self) -> DVector3: ...
    @property
    def lgg(self) -> DVector3: ...
    @property
    def lgo(self) -> DVector3: ...
    @property
    def lgl(self) -> DVector3: ...
    @property
    def lor(self) -> DVector3: ...
    @property
    def log(self) -> DVector3: ...
    @property
    def llr(self) -> DVector3: ...
    @property
    def llg(self) -> DVector3: ...
    @property
    def rrrr(self) -> DVector4: ...
    @property
    def rrrg(self) -> DVector4: ...
    @property
    def rrro(self) -> DVector4: ...
    @property
    def rrrl(self) -> DVector4: ...
    @property
    def rrgr(self) -> DVector4: ...
    @property
    def rrgg(self) -> DVector4: ...
    @property
    def rrgo(self) -> DVector4: ...
    @property
    def rrgl(self) -> DVector4: ...
    @property
    def rror(self) -> DVector4: ...
    @property
    def rrog(self) -> DVector4: ...
    @property
    def rroo(self) -> DVector4: ...
    @property
    def rrol(self) -> DVector4: ...
    @property
    def rrlr(self) -> DVector4: ...
    @property
    def rrlg(self) -> DVector4: ...
    @property
    def rrlo(self) -> DVector4: ...
    @property
    def rrll(self) -> DVector4: ...
    @property
    def rgrr(self) -> DVector4: ...
    @property
    def rgrg(self) -> DVector4: ...
    @property
    def rgro(self) -> DVector4: ...
    @property
    def rgrl(self) -> DVector4: ...
    @property
    def rggr(self) -> DVector4: ...
    @property
    def rggg(self) -> DVector4: ...
    @property
    def rggo(self) -> DVector4: ...
    @property
    def rggl(self) -> DVector4: ...
    @property
    def rgor(self) -> DVector4: ...
    @property
    def rgog(self) -> DVector4: ...
    @property
    def rgoo(self) -> DVector4: ...
    @property
    def rgol(self) -> DVector4: ...
    @property
    def rglr(self) -> DVector4: ...
    @property
    def rglg(self) -> DVector4: ...
    @property
    def rglo(self) -> DVector4: ...
    @property
    def rgll(self) -> DVector4: ...
    @property
    def rorr(self) -> DVector4: ...
    @property
    def rorg(self) -> DVector4: ...
    @property
    def roro(self) -> DVector4: ...
    @property
    def rorl(self) -> DVector4: ...
    @property
    def rogr(self) -> DVector4: ...
    @property
    def rogg(self) -> DVector4: ...
    @property
    def rogo(self) -> DVector4: ...
    @property
    def rogl(self) -> DVector4: ...
    @property
    def roor(self) -> DVector4: ...
    @property
    def roog(self) -> DVector4: ...
    @property
    def rooo(self) -> DVector4: ...
    @property
    def rool(self) -> DVector4: ...
    @property
    def rolr(self) -> DVector4: ...
    @property
    def rolg(self) -> DVector4: ...
    @property
    def rolo(self) -> DVector4: ...
    @property
    def roll(self) -> DVector4: ...
    @property
    def rlrr(self) -> DVector4: ...
    @property
    def rlrg(self) -> DVector4: ...
    @property
    def rlro(self) -> DVector4: ...
    @property
    def rlrl(self) -> DVector4: ...
    @property
    def rlgr(self) -> DVector4: ...
    @property
    def rlgg(self) -> DVector4: ...
    @property
    def rlgo(self) -> DVector4: ...
    @property
    def rlgl(self) -> DVector4: ...
    @property
    def rlor(self) -> DVector4: ...
    @property
    def rlog(self) -> DVector4: ...
    @property
    def rloo(self) -> DVector4: ...
    @property
    def rlol(self) -> DVector4: ...
    @property
    def rllr(self) -> DVector4: ...
    @property
    def rllg(self) -> DVector4: ...
    @property
    def rllo(self) -> DVector4: ...
    @property
    def rlll(self) -> DVector4: ...
    @property
    def grrr(self) -> DVector4: ...
    @property
    def grrg(self) -> DVector4: ...
    @property
    def grro(self) -> DVector4: ...
    @property
    def grrl(self) -> DVector4: ...
    @property
    def grgr(self) -> DVector4: ...
    @property
    def grgg(self) -> DVector4: ...
    @property
    def grgo(self) -> DVector4: ...
    @property
    def grgl(self) -> DVector4: ...
    @property
    def gror(self) -> DVector4: ...
    @property
    def grog(self) -> DVector4: ...
    @property
    def groo(self) -> DVector4: ...
    @property
    def grol(self) -> DVector4: ...
    @property
    def grlr(self) -> DVector4: ...
    @property
    def grlg(self) -> DVector4: ...
    @property
    def grlo(self) -> DVector4: ...
    @property
    def grll(self) -> DVector4: ...
    @property
    def ggrr(self) -> DVector4: ...
    @property
    def ggrg(self) -> DVector4: ...
    @property
    def ggro(self) -> DVector4: ...
    @property
    def ggrl(self) -> DVector4: ...
    @property
    def gggr(self) -> DVector4: ...
    @property
    def gggg(self) -> DVector4: ...
    @property
    def gggo(self) -> DVector4: ...
    @property
    def gggl(self) -> DVector4: ...
    @property
    def ggor(self) -> DVector4: ...
    @property
    def ggog(self) -> DVector4: ...
    @property
    def ggoo(self) -> DVector4: ...
    @property
    def ggol(self) -> DVector4: ...
    @property
    def gglr(self) -> DVector4: ...
    @property
    def gglg(self) -> DVector4: ...
    @property
    def gglo(self) -> DVector4: ...
    @property
    def ggll(self) -> DVector4: ...
    @property
    def gorr(self) -> DVector4: ...
    @property
    def gorg(self) -> DVector4: ...
    @property
    def goro(self) -> DVector4: ...
    @property
    def gorl(self) -> DVector4: ...
    @property
    def gogr(self) -> DVector4: ...
    @property
    def gogg(self) -> DVector4: ...
    @property
    def gogo(self) -> DVector4: ...
    @property
    def gogl(self) -> DVector4: ...
    @property
    def goor(self) -> DVector4: ...
    @property
    def goog(self) -> DVector4: ...
    @property
    def gooo(self) -> DVector4: ...
    @property
    def gool(self) -> DVector4: ...
    @property
    def golr(self) -> DVector4: ...
    @property
    def golg(self) -> DVector4: ...
    @property
    def golo(self) -> DVector4: ...
    @property
    def goll(self) -> DVector4: ...
    @property
    def glrr(self) -> DVector4: ...
    @property
    def glrg(self) -> DVector4: ...
    @property
    def glro(self) -> DVector4: ...
    @property
    def glrl(self) -> DVector4: ...
    @property
    def glgr(self) -> DVector4: ...
    @property
    def glgg(self) -> DVector4: ...
    @property
    def glgo(self) -> DVector4: ...
    @property
    def glgl(self) -> DVector4: ...
    @property
    def glor(self) -> DVector4: ...
    @property
    def glog(self) -> DVector4: ...
    @property
    def gloo(self) -> DVector4: ...
    @property
    def glol(self) -> DVector4: ...
    @property
    def gllr(self) -> DVector4: ...
    @property
    def gllg(self) -> DVector4: ...
    @property
    def gllo(self) -> DVector4: ...
    @property
    def glll(self) -> DVector4: ...
    @property
    def orrr(self) -> DVector4: ...
    @property
    def orrg(self) -> DVector4: ...
    @property
    def orro(self) -> DVector4: ...
    @property
    def orrl(self) -> DVector4: ...
    @property
    def orgr(self) -> DVector4: ...
    @property
    def orgg(self) -> DVector4: ...
    @property
    def orgo(self) -> DVector4: ...
    @property
    def orgl(self) -> DVector4: ...
    @property
    def oror(self) -> DVector4: ...
    @property
    def orog(self) -> DVector4: ...
    @property
    def oroo(self) -> DVector4: ...
    @property
    def orol(self) -> DVector4: ...
    @property
    def orlr(self) -> DVector4: ...
    @property
    def orlg(self) -> DVector4: ...
    @property
    def orlo(self) -> DVector4: ...
    @property
    def orll(self) -> DVector4: ...
    @property
    def ogrr(self) -> DVector4: ...
    @property
    def ogrg(self) -> DVector4: ...
    @property
    def ogro(self) -> DVector4: ...
    @property
    def ogrl(self) -> DVector4: ...
    @property
    def oggr(self) -> DVector4: ...
    @property
    def oggg(self) -> DVector4: ...
    @property
    def oggo(self) -> DVector4: ...
    @property
    def oggl(self) -> DVector4: ...
    @property
    def ogor(self) -> DVector4: ...
    @property
    def ogog(self) -> DVector4: ...
    @property
    def ogoo(self) -> DVector4: ...
    @property
    def ogol(self) -> DVector4: ...
    @property
    def oglr(self) -> DVector4: ...
    @property
    def oglg(self) -> DVector4: ...
    @property
    def oglo(self) -> DVector4: ...
    @property
    def ogll(self) -> DVector4: ...
    @property
    def oorr(self) -> DVector4: ...
    @property
    def oorg(self) -> DVector4: ...
    @property
    def ooro(self) -> DVector4: ...
    @property
    def oorl(self) -> DVector4: ...
    @property
    def oogr(self) -> DVector4: ...
    @property
    def oogg(self) -> DVector4: ...
    @property
    def oogo(self) -> DVector4: ...
    @property
    def oogl(self) -> DVector4: ...
    @property
    def ooor(self) -> DVector4: ...
    @property
    def ooog(self) -> DVector4: ...
    @property
    def oolr(self) -> DVector4: ...
    @property
    def oolg(self) -> DVector4: ...
    @property
    def olrr(self) -> DVector4: ...
    @property
    def olrg(self) -> DVector4: ...
    @property
    def olro(self) -> DVector4: ...
    @property
    def olrl(self) -> DVector4: ...
    @property
    def olgr(self) -> DVector4: ...
    @property
    def olgg(self) -> DVector4: ...
    @property
    def olgo(self) -> DVector4: ...
    @property
    def olgl(self) -> DVector4: ...
    @property
    def olor(self) -> DVector4: ...
    @property
    def olog(self) -> DVector4: ...
    @property
    def ollr(self) -> DVector4: ...
    @property
    def ollg(self) -> DVector4: ...
    @property
    def lrrr(self) -> DVector4: ...
    @property
    def lrrg(self) -> DVector4: ...
    @property
    def lrro(self) -> DVector4: ...
    @property
    def lrrl(self) -> DVector4: ...
    @property
    def lrgr(self) -> DVector4: ...
    @property
    def lrgg(self) -> DVector4: ...
    @property
    def lrgo(self) -> DVector4: ...
    @property
    def lrgl(self) -> DVector4: ...
    @property
    def lror(self) -> DVector4: ...
    @property
    def lrog(self) -> DVector4: ...
    @property
    def lroo(self) -> DVector4: ...
    @property
    def lrol(self) -> DVector4: ...
    @property
    def lrlr(self) -> DVector4: ...
    @property
    def lrlg(self) -> DVector4: ...
    @property
    def lrlo(self) -> DVector4: ...
    @property
    def lrll(self) -> DVector4: ...
    @property
    def lgrr(self) -> DVector4: ...
    @property
    def lgrg(self) -> DVector4: ...
    @property
    def lgro(self) -> DVector4: ...
    @property
    def lgrl(self) -> DVector4: ...
    @property
    def lggr(self) -> DVector4: ...
    @property
    def lggg(self) -> DVector4: ...
    @property
    def lggo(self) -> DVector4: ...
    @property
    def lggl(self) -> DVector4: ...
    @property
    def lgor(self) -> DVector4: ...
    @property
    def lgog(self) -> DVector4: ...
    @property
    def lgoo(self) -> DVector4: ...
    @property
    def lgol(self) -> DVector4: ...
    @property
    def lglr(self) -> DVector4: ...
    @property
    def lglg(self) -> DVector4: ...
    @property
    def lglo(self) -> DVector4: ...
    @property
    def lgll(self) -> DVector4: ...
    @property
    def lorr(self) -> DVector4: ...
    @property
    def lorg(self) -> DVector4: ...
    @property
    def loro(self) -> DVector4: ...
    @property
    def lorl(self) -> DVector4: ...
    @property
    def logr(self) -> DVector4: ...
    @property
    def logg(self) -> DVector4: ...
    @property
    def logo(self) -> DVector4: ...
    @property
    def logl(self) -> DVector4: ...
    @property
    def loor(self) -> DVector4: ...
    @property
    def loog(self) -> DVector4: ...
    @property
    def lolr(self) -> DVector4: ...
    @property
    def lolg(self) -> DVector4: ...
    @property
    def llrr(self) -> DVector4: ...
    @property
    def llrg(self) -> DVector4: ...
    @property
    def llro(self) -> DVector4: ...
    @property
    def llrl(self) -> DVector4: ...
    @property
    def llgr(self) -> DVector4: ...
    @property
    def llgg(self) -> DVector4: ...
    @property
    def llgo(self) -> DVector4: ...
    @property
    def llgl(self) -> DVector4: ...
    @property
    def llor(self) -> DVector4: ...
    @property
    def llog(self) -> DVector4: ...
    @property
    def lllr(self) -> DVector4: ...
    @property
    def lllg(self) -> DVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def t(self) -> float: ...
    @property
    def ss(self) -> DVector2: ...
    @property
    def st(self) -> DVector2: ...
    @property
    def so(self) -> DVector2: ...
    @property
    def sl(self) -> DVector2: ...
    @property
    def ts(self) -> DVector2: ...
    @property
    def tt(self) -> DVector2: ...
    @property
    def to(self) -> DVector2: ...
    @property
    def tl(self) -> DVector2: ...
    @property
    def os(self) -> DVector2: ...
    @property
    def ot(self) -> DVector2: ...
    @property
    def ls(self) -> DVector2: ...
    @property
    def lt(self) -> DVector2: ...
    @property
    def sss(self) -> DVector3: ...
    @property
    def sst(self) -> DVector3: ...
    @property
    def sso(self) -> DVector3: ...
    @property
    def ssl(self) -> DVector3: ...
    @property
    def sts(self) -> DVector3: ...
    @property
    def stt(self) -> DVector3: ...
    @property
    def sto(self) -> DVector3: ...
    @property
    def stl(self) -> DVector3: ...
    @property
    def sos(self) -> DVector3: ...
    @property
    def sot(self) -> DVector3: ...
    @property
    def soo(self) -> DVector3: ...
    @property
    def sol(self) -> DVector3: ...
    @property
    def sls(self) -> DVector3: ...
    @property
    def slt(self) -> DVector3: ...
    @property
    def slo(self) -> DVector3: ...
    @property
    def sll(self) -> DVector3: ...
    @property
    def tss(self) -> DVector3: ...
    @property
    def tst(self) -> DVector3: ...
    @property
    def tso(self) -> DVector3: ...
    @property
    def tsl(self) -> DVector3: ...
    @property
    def tts(self) -> DVector3: ...
    @property
    def ttt(self) -> DVector3: ...
    @property
    def tto(self) -> DVector3: ...
    @property
    def ttl(self) -> DVector3: ...
    @property
    def tos(self) -> DVector3: ...
    @property
    def tot(self) -> DVector3: ...
    @property
    def too(self) -> DVector3: ...
    @property
    def tol(self) -> DVector3: ...
    @property
    def tls(self) -> DVector3: ...
    @property
    def tlt(self) -> DVector3: ...
    @property
    def tlo(self) -> DVector3: ...
    @property
    def tll(self) -> DVector3: ...
    @property
    def oss(self) -> DVector3: ...
    @property
    def ost(self) -> DVector3: ...
    @property
    def oso(self) -> DVector3: ...
    @property
    def osl(self) -> DVector3: ...
    @property
    def ots(self) -> DVector3: ...
    @property
    def ott(self) -> DVector3: ...
    @property
    def oto(self) -> DVector3: ...
    @property
    def otl(self) -> DVector3: ...
    @property
    def oos(self) -> DVector3: ...
    @property
    def oot(self) -> DVector3: ...
    @property
    def ols(self) -> DVector3: ...
    @property
    def olt(self) -> DVector3: ...
    @property
    def lss(self) -> DVector3: ...
    @property
    def lst(self) -> DVector3: ...
    @property
    def lso(self) -> DVector3: ...
    @property
    def lsl(self) -> DVector3: ...
    @property
    def lts(self) -> DVector3: ...
    @property
    def ltt(self) -> DVector3: ...
    @property
    def lto(self) -> DVector3: ...
    @property
    def ltl(self) -> DVector3: ...
    @property
    def los(self) -> DVector3: ...
    @property
    def lot(self) -> DVector3: ...
    @property
    def lls(self) -> DVector3: ...
    @property
    def llt(self) -> DVector3: ...
    @property
    def ssss(self) -> DVector4: ...
    @property
    def ssst(self) -> DVector4: ...
    @property
    def ssso(self) -> DVector4: ...
    @property
    def sssl(self) -> DVector4: ...
    @property
    def ssts(self) -> DVector4: ...
    @property
    def sstt(self) -> DVector4: ...
    @property
    def ssto(self) -> DVector4: ...
    @property
    def sstl(self) -> DVector4: ...
    @property
    def ssos(self) -> DVector4: ...
    @property
    def ssot(self) -> DVector4: ...
    @property
    def ssoo(self) -> DVector4: ...
    @property
    def ssol(self) -> DVector4: ...
    @property
    def ssls(self) -> DVector4: ...
    @property
    def sslt(self) -> DVector4: ...
    @property
    def sslo(self) -> DVector4: ...
    @property
    def ssll(self) -> DVector4: ...
    @property
    def stss(self) -> DVector4: ...
    @property
    def stst(self) -> DVector4: ...
    @property
    def stso(self) -> DVector4: ...
    @property
    def stsl(self) -> DVector4: ...
    @property
    def stts(self) -> DVector4: ...
    @property
    def sttt(self) -> DVector4: ...
    @property
    def stto(self) -> DVector4: ...
    @property
    def sttl(self) -> DVector4: ...
    @property
    def stos(self) -> DVector4: ...
    @property
    def stot(self) -> DVector4: ...
    @property
    def stoo(self) -> DVector4: ...
    @property
    def stol(self) -> DVector4: ...
    @property
    def stls(self) -> DVector4: ...
    @property
    def stlt(self) -> DVector4: ...
    @property
    def stlo(self) -> DVector4: ...
    @property
    def stll(self) -> DVector4: ...
    @property
    def soss(self) -> DVector4: ...
    @property
    def sost(self) -> DVector4: ...
    @property
    def soso(self) -> DVector4: ...
    @property
    def sosl(self) -> DVector4: ...
    @property
    def sots(self) -> DVector4: ...
    @property
    def sott(self) -> DVector4: ...
    @property
    def soto(self) -> DVector4: ...
    @property
    def sotl(self) -> DVector4: ...
    @property
    def soos(self) -> DVector4: ...
    @property
    def soot(self) -> DVector4: ...
    @property
    def sooo(self) -> DVector4: ...
    @property
    def sool(self) -> DVector4: ...
    @property
    def sols(self) -> DVector4: ...
    @property
    def solt(self) -> DVector4: ...
    @property
    def solo(self) -> DVector4: ...
    @property
    def soll(self) -> DVector4: ...
    @property
    def slss(self) -> DVector4: ...
    @property
    def slst(self) -> DVector4: ...
    @property
    def slso(self) -> DVector4: ...
    @property
    def slsl(self) -> DVector4: ...
    @property
    def slts(self) -> DVector4: ...
    @property
    def sltt(self) -> DVector4: ...
    @property
    def slto(self) -> DVector4: ...
    @property
    def sltl(self) -> DVector4: ...
    @property
    def slos(self) -> DVector4: ...
    @property
    def slot(self) -> DVector4: ...
    @property
    def sloo(self) -> DVector4: ...
    @property
    def slol(self) -> DVector4: ...
    @property
    def slls(self) -> DVector4: ...
    @property
    def sllt(self) -> DVector4: ...
    @property
    def sllo(self) -> DVector4: ...
    @property
    def slll(self) -> DVector4: ...
    @property
    def tsss(self) -> DVector4: ...
    @property
    def tsst(self) -> DVector4: ...
    @property
    def tsso(self) -> DVector4: ...
    @property
    def tssl(self) -> DVector4: ...
    @property
    def tsts(self) -> DVector4: ...
    @property
    def tstt(self) -> DVector4: ...
    @property
    def tsto(self) -> DVector4: ...
    @property
    def tstl(self) -> DVector4: ...
    @property
    def tsos(self) -> DVector4: ...
    @property
    def tsot(self) -> DVector4: ...
    @property
    def tsoo(self) -> DVector4: ...
    @property
    def tsol(self) -> DVector4: ...
    @property
    def tsls(self) -> DVector4: ...
    @property
    def tslt(self) -> DVector4: ...
    @property
    def tslo(self) -> DVector4: ...
    @property
    def tsll(self) -> DVector4: ...
    @property
    def ttss(self) -> DVector4: ...
    @property
    def ttst(self) -> DVector4: ...
    @property
    def ttso(self) -> DVector4: ...
    @property
    def ttsl(self) -> DVector4: ...
    @property
    def ttts(self) -> DVector4: ...
    @property
    def tttt(self) -> DVector4: ...
    @property
    def ttto(self) -> DVector4: ...
    @property
    def tttl(self) -> DVector4: ...
    @property
    def ttos(self) -> DVector4: ...
    @property
    def ttot(self) -> DVector4: ...
    @property
    def ttoo(self) -> DVector4: ...
    @property
    def ttol(self) -> DVector4: ...
    @property
    def ttls(self) -> DVector4: ...
    @property
    def ttlt(self) -> DVector4: ...
    @property
    def ttlo(self) -> DVector4: ...
    @property
    def ttll(self) -> DVector4: ...
    @property
    def toss(self) -> DVector4: ...
    @property
    def tost(self) -> DVector4: ...
    @property
    def toso(self) -> DVector4: ...
    @property
    def tosl(self) -> DVector4: ...
    @property
    def tots(self) -> DVector4: ...
    @property
    def tott(self) -> DVector4: ...
    @property
    def toto(self) -> DVector4: ...
    @property
    def totl(self) -> DVector4: ...
    @property
    def toos(self) -> DVector4: ...
    @property
    def toot(self) -> DVector4: ...
    @property
    def tooo(self) -> DVector4: ...
    @property
    def tool(self) -> DVector4: ...
    @property
    def tols(self) -> DVector4: ...
    @property
    def tolt(self) -> DVector4: ...
    @property
    def tolo(self) -> DVector4: ...
    @property
    def toll(self) -> DVector4: ...
    @property
    def tlss(self) -> DVector4: ...
    @property
    def tlst(self) -> DVector4: ...
    @property
    def tlso(self) -> DVector4: ...
    @property
    def tlsl(self) -> DVector4: ...
    @property
    def tlts(self) -> DVector4: ...
    @property
    def tltt(self) -> DVector4: ...
    @property
    def tlto(self) -> DVector4: ...
    @property
    def tltl(self) -> DVector4: ...
    @property
    def tlos(self) -> DVector4: ...
    @property
    def tlot(self) -> DVector4: ...
    @property
    def tloo(self) -> DVector4: ...
    @property
    def tlol(self) -> DVector4: ...
    @property
    def tlls(self) -> DVector4: ...
    @property
    def tllt(self) -> DVector4: ...
    @property
    def tllo(self) -> DVector4: ...
    @property
    def tlll(self) -> DVector4: ...
    @property
    def osss(self) -> DVector4: ...
    @property
    def osst(self) -> DVector4: ...
    @property
    def osso(self) -> DVector4: ...
    @property
    def ossl(self) -> DVector4: ...
    @property
    def osts(self) -> DVector4: ...
    @property
    def ostt(self) -> DVector4: ...
    @property
    def osto(self) -> DVector4: ...
    @property
    def ostl(self) -> DVector4: ...
    @property
    def osos(self) -> DVector4: ...
    @property
    def osot(self) -> DVector4: ...
    @property
    def osoo(self) -> DVector4: ...
    @property
    def osol(self) -> DVector4: ...
    @property
    def osls(self) -> DVector4: ...
    @property
    def oslt(self) -> DVector4: ...
    @property
    def oslo(self) -> DVector4: ...
    @property
    def osll(self) -> DVector4: ...
    @property
    def otss(self) -> DVector4: ...
    @property
    def otst(self) -> DVector4: ...
    @property
    def otso(self) -> DVector4: ...
    @property
    def otsl(self) -> DVector4: ...
    @property
    def otts(self) -> DVector4: ...
    @property
    def ottt(self) -> DVector4: ...
    @property
    def otto(self) -> DVector4: ...
    @property
    def ottl(self) -> DVector4: ...
    @property
    def otos(self) -> DVector4: ...
    @property
    def otot(self) -> DVector4: ...
    @property
    def otoo(self) -> DVector4: ...
    @property
    def otol(self) -> DVector4: ...
    @property
    def otls(self) -> DVector4: ...
    @property
    def otlt(self) -> DVector4: ...
    @property
    def otlo(self) -> DVector4: ...
    @property
    def otll(self) -> DVector4: ...
    @property
    def ooss(self) -> DVector4: ...
    @property
    def oost(self) -> DVector4: ...
    @property
    def ooso(self) -> DVector4: ...
    @property
    def oosl(self) -> DVector4: ...
    @property
    def oots(self) -> DVector4: ...
    @property
    def oott(self) -> DVector4: ...
    @property
    def ooto(self) -> DVector4: ...
    @property
    def ootl(self) -> DVector4: ...
    @property
    def ooos(self) -> DVector4: ...
    @property
    def ooot(self) -> DVector4: ...
    @property
    def ools(self) -> DVector4: ...
    @property
    def oolt(self) -> DVector4: ...
    @property
    def olss(self) -> DVector4: ...
    @property
    def olst(self) -> DVector4: ...
    @property
    def olso(self) -> DVector4: ...
    @property
    def olsl(self) -> DVector4: ...
    @property
    def olts(self) -> DVector4: ...
    @property
    def oltt(self) -> DVector4: ...
    @property
    def olto(self) -> DVector4: ...
    @property
    def oltl(self) -> DVector4: ...
    @property
    def olos(self) -> DVector4: ...
    @property
    def olot(self) -> DVector4: ...
    @property
    def olls(self) -> DVector4: ...
    @property
    def ollt(self) -> DVector4: ...
    @property
    def lsss(self) -> DVector4: ...
    @property
    def lsst(self) -> DVector4: ...
    @property
    def lsso(self) -> DVector4: ...
    @property
    def lssl(self) -> DVector4: ...
    @property
    def lsts(self) -> DVector4: ...
    @property
    def lstt(self) -> DVector4: ...
    @property
    def lsto(self) -> DVector4: ...
    @property
    def lstl(self) -> DVector4: ...
    @property
    def lsos(self) -> DVector4: ...
    @property
    def lsot(self) -> DVector4: ...
    @property
    def lsoo(self) -> DVector4: ...
    @property
    def lsol(self) -> DVector4: ...
    @property
    def lsls(self) -> DVector4: ...
    @property
    def lslt(self) -> DVector4: ...
    @property
    def lslo(self) -> DVector4: ...
    @property
    def lsll(self) -> DVector4: ...
    @property
    def ltss(self) -> DVector4: ...
    @property
    def ltst(self) -> DVector4: ...
    @property
    def ltso(self) -> DVector4: ...
    @property
    def ltsl(self) -> DVector4: ...
    @property
    def ltts(self) -> DVector4: ...
    @property
    def lttt(self) -> DVector4: ...
    @property
    def ltto(self) -> DVector4: ...
    @property
    def lttl(self) -> DVector4: ...
    @property
    def ltos(self) -> DVector4: ...
    @property
    def ltot(self) -> DVector4: ...
    @property
    def ltoo(self) -> DVector4: ...
    @property
    def ltol(self) -> DVector4: ...
    @property
    def ltls(self) -> DVector4: ...
    @property
    def ltlt(self) -> DVector4: ...
    @property
    def ltlo(self) -> DVector4: ...
    @property
    def ltll(self) -> DVector4: ...
    @property
    def loss(self) -> DVector4: ...
    @property
    def lost(self) -> DVector4: ...
    @property
    def loso(self) -> DVector4: ...
    @property
    def losl(self) -> DVector4: ...
    @property
    def lots(self) -> DVector4: ...
    @property
    def lott(self) -> DVector4: ...
    @property
    def loto(self) -> DVector4: ...
    @property
    def lotl(self) -> DVector4: ...
    @property
    def loos(self) -> DVector4: ...
    @property
    def loot(self) -> DVector4: ...
    @property
    def lols(self) -> DVector4: ...
    @property
    def lolt(self) -> DVector4: ...
    @property
    def llss(self) -> DVector4: ...
    @property
    def llst(self) -> DVector4: ...
    @property
    def llso(self) -> DVector4: ...
    @property
    def llsl(self) -> DVector4: ...
    @property
    def llts(self) -> DVector4: ...
    @property
    def lltt(self) -> DVector4: ...
    @property
    def llto(self) -> DVector4: ...
    @property
    def lltl(self) -> DVector4: ...
    @property
    def llos(self) -> DVector4: ...
    @property
    def llot(self) -> DVector4: ...
    @property
    def llls(self) -> DVector4: ...
    @property
    def lllt(self) -> DVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def v(self) -> float: ...
    @property
    def uu(self) -> DVector2: ...
    @property
    def uv(self) -> DVector2: ...
    @property
    def uo(self) -> DVector2: ...
    @property
    def ul(self) -> DVector2: ...
    @property
    def vu(self) -> DVector2: ...
    @property
    def vv(self) -> DVector2: ...
    @property
    def vo(self) -> DVector2: ...
    @property
    def vl(self) -> DVector2: ...
    @property
    def ou(self) -> DVector2: ...
    @property
    def ov(self) -> DVector2: ...
    @property
    def lu(self) -> DVector2: ...
    @property
    def lv(self) -> DVector2: ...
    @property
    def uuu(self) -> DVector3: ...
    @property
    def uuv(self) -> DVector3: ...
    @property
    def uuo(self) -> DVector3: ...
    @property
    def uul(self) -> DVector3: ...
    @property
    def uvu(self) -> DVector3: ...
    @property
    def uvv(self) -> DVector3: ...
    @property
    def uvo(self) -> DVector3: ...
    @property
    def uvl(self) -> DVector3: ...
    @property
    def uou(self) -> DVector3: ...
    @property
    def uov(self) -> DVector3: ...
    @property
    def uoo(self) -> DVector3: ...
    @property
    def uol(self) -> DVector3: ...
    @property
    def ulu(self) -> DVector3: ...
    @property
    def ulv(self) -> DVector3: ...
    @property
    def ulo(self) -> DVector3: ...
    @property
    def ull(self) -> DVector3: ...
    @property
    def vuu(self) -> DVector3: ...
    @property
    def vuv(self) -> DVector3: ...
    @property
    def vuo(self) -> DVector3: ...
    @property
    def vul(self) -> DVector3: ...
    @property
    def vvu(self) -> DVector3: ...
    @property
    def vvv(self) -> DVector3: ...
    @property
    def vvo(self) -> DVector3: ...
    @property
    def vvl(self) -> DVector3: ...
    @property
    def vou(self) -> DVector3: ...
    @property
    def vov(self) -> DVector3: ...
    @property
    def voo(self) -> DVector3: ...
    @property
    def vol(self) -> DVector3: ...
    @property
    def vlu(self) -> DVector3: ...
    @property
    def vlv(self) -> DVector3: ...
    @property
    def vlo(self) -> DVector3: ...
    @property
    def vll(self) -> DVector3: ...
    @property
    def ouu(self) -> DVector3: ...
    @property
    def ouv(self) -> DVector3: ...
    @property
    def ouo(self) -> DVector3: ...
    @property
    def oul(self) -> DVector3: ...
    @property
    def ovu(self) -> DVector3: ...
    @property
    def ovv(self) -> DVector3: ...
    @property
    def ovo(self) -> DVector3: ...
    @property
    def ovl(self) -> DVector3: ...
    @property
    def oou(self) -> DVector3: ...
    @property
    def oov(self) -> DVector3: ...
    @property
    def olu(self) -> DVector3: ...
    @property
    def olv(self) -> DVector3: ...
    @property
    def luu(self) -> DVector3: ...
    @property
    def luv(self) -> DVector3: ...
    @property
    def luo(self) -> DVector3: ...
    @property
    def lul(self) -> DVector3: ...
    @property
    def lvu(self) -> DVector3: ...
    @property
    def lvv(self) -> DVector3: ...
    @property
    def lvo(self) -> DVector3: ...
    @property
    def lvl(self) -> DVector3: ...
    @property
    def lou(self) -> DVector3: ...
    @property
    def lov(self) -> DVector3: ...
    @property
    def llu(self) -> DVector3: ...
    @property
    def llv(self) -> DVector3: ...
    @property
    def uuuu(self) -> DVector4: ...
    @property
    def uuuv(self) -> DVector4: ...
    @property
    def uuuo(self) -> DVector4: ...
    @property
    def uuul(self) -> DVector4: ...
    @property
    def uuvu(self) -> DVector4: ...
    @property
    def uuvv(self) -> DVector4: ...
    @property
    def uuvo(self) -> DVector4: ...
    @property
    def uuvl(self) -> DVector4: ...
    @property
    def uuou(self) -> DVector4: ...
    @property
    def uuov(self) -> DVector4: ...
    @property
    def uuoo(self) -> DVector4: ...
    @property
    def uuol(self) -> DVector4: ...
    @property
    def uulu(self) -> DVector4: ...
    @property
    def uulv(self) -> DVector4: ...
    @property
    def uulo(self) -> DVector4: ...
    @property
    def uull(self) -> DVector4: ...
    @property
    def uvuu(self) -> DVector4: ...
    @property
    def uvuv(self) -> DVector4: ...
    @property
    def uvuo(self) -> DVector4: ...
    @property
    def uvul(self) -> DVector4: ...
    @property
    def uvvu(self) -> DVector4: ...
    @property
    def uvvv(self) -> DVector4: ...
    @property
    def uvvo(self) -> DVector4: ...
    @property
    def uvvl(self) -> DVector4: ...
    @property
    def uvou(self) -> DVector4: ...
    @property
    def uvov(self) -> DVector4: ...
    @property
    def uvoo(self) -> DVector4: ...
    @property
    def uvol(self) -> DVector4: ...
    @property
    def uvlu(self) -> DVector4: ...
    @property
    def uvlv(self) -> DVector4: ...
    @property
    def uvlo(self) -> DVector4: ...
    @property
    def uvll(self) -> DVector4: ...
    @property
    def uouu(self) -> DVector4: ...
    @property
    def uouv(self) -> DVector4: ...
    @property
    def uouo(self) -> DVector4: ...
    @property
    def uoul(self) -> DVector4: ...
    @property
    def uovu(self) -> DVector4: ...
    @property
    def uovv(self) -> DVector4: ...
    @property
    def uovo(self) -> DVector4: ...
    @property
    def uovl(self) -> DVector4: ...
    @property
    def uoou(self) -> DVector4: ...
    @property
    def uoov(self) -> DVector4: ...
    @property
    def uooo(self) -> DVector4: ...
    @property
    def uool(self) -> DVector4: ...
    @property
    def uolu(self) -> DVector4: ...
    @property
    def uolv(self) -> DVector4: ...
    @property
    def uolo(self) -> DVector4: ...
    @property
    def uoll(self) -> DVector4: ...
    @property
    def uluu(self) -> DVector4: ...
    @property
    def uluv(self) -> DVector4: ...
    @property
    def uluo(self) -> DVector4: ...
    @property
    def ulul(self) -> DVector4: ...
    @property
    def ulvu(self) -> DVector4: ...
    @property
    def ulvv(self) -> DVector4: ...
    @property
    def ulvo(self) -> DVector4: ...
    @property
    def ulvl(self) -> DVector4: ...
    @property
    def ulou(self) -> DVector4: ...
    @property
    def ulov(self) -> DVector4: ...
    @property
    def uloo(self) -> DVector4: ...
    @property
    def ulol(self) -> DVector4: ...
    @property
    def ullu(self) -> DVector4: ...
    @property
    def ullv(self) -> DVector4: ...
    @property
    def ullo(self) -> DVector4: ...
    @property
    def ulll(self) -> DVector4: ...
    @property
    def vuuu(self) -> DVector4: ...
    @property
    def vuuv(self) -> DVector4: ...
    @property
    def vuuo(self) -> DVector4: ...
    @property
    def vuul(self) -> DVector4: ...
    @property
    def vuvu(self) -> DVector4: ...
    @property
    def vuvv(self) -> DVector4: ...
    @property
    def vuvo(self) -> DVector4: ...
    @property
    def vuvl(self) -> DVector4: ...
    @property
    def vuou(self) -> DVector4: ...
    @property
    def vuov(self) -> DVector4: ...
    @property
    def vuoo(self) -> DVector4: ...
    @property
    def vuol(self) -> DVector4: ...
    @property
    def vulu(self) -> DVector4: ...
    @property
    def vulv(self) -> DVector4: ...
    @property
    def vulo(self) -> DVector4: ...
    @property
    def vull(self) -> DVector4: ...
    @property
    def vvuu(self) -> DVector4: ...
    @property
    def vvuv(self) -> DVector4: ...
    @property
    def vvuo(self) -> DVector4: ...
    @property
    def vvul(self) -> DVector4: ...
    @property
    def vvvu(self) -> DVector4: ...
    @property
    def vvvv(self) -> DVector4: ...
    @property
    def vvvo(self) -> DVector4: ...
    @property
    def vvvl(self) -> DVector4: ...
    @property
    def vvou(self) -> DVector4: ...
    @property
    def vvov(self) -> DVector4: ...
    @property
    def vvoo(self) -> DVector4: ...
    @property
    def vvol(self) -> DVector4: ...
    @property
    def vvlu(self) -> DVector4: ...
    @property
    def vvlv(self) -> DVector4: ...
    @property
    def vvlo(self) -> DVector4: ...
    @property
    def vvll(self) -> DVector4: ...
    @property
    def vouu(self) -> DVector4: ...
    @property
    def vouv(self) -> DVector4: ...
    @property
    def vouo(self) -> DVector4: ...
    @property
    def voul(self) -> DVector4: ...
    @property
    def vovu(self) -> DVector4: ...
    @property
    def vovv(self) -> DVector4: ...
    @property
    def vovo(self) -> DVector4: ...
    @property
    def vovl(self) -> DVector4: ...
    @property
    def voou(self) -> DVector4: ...
    @property
    def voov(self) -> DVector4: ...
    @property
    def vooo(self) -> DVector4: ...
    @property
    def vool(self) -> DVector4: ...
    @property
    def volu(self) -> DVector4: ...
    @property
    def volv(self) -> DVector4: ...
    @property
    def volo(self) -> DVector4: ...
    @property
    def voll(self) -> DVector4: ...
    @property
    def vluu(self) -> DVector4: ...
    @property
    def vluv(self) -> DVector4: ...
    @property
    def vluo(self) -> DVector4: ...
    @property
    def vlul(self) -> DVector4: ...
    @property
    def vlvu(self) -> DVector4: ...
    @property
    def vlvv(self) -> DVector4: ...
    @property
    def vlvo(self) -> DVector4: ...
    @property
    def vlvl(self) -> DVector4: ...
    @property
    def vlou(self) -> DVector4: ...
    @property
    def vlov(self) -> DVector4: ...
    @property
    def vloo(self) -> DVector4: ...
    @property
    def vlol(self) -> DVector4: ...
    @property
    def vllu(self) -> DVector4: ...
    @property
    def vllv(self) -> DVector4: ...
    @property
    def vllo(self) -> DVector4: ...
    @property
    def vlll(self) -> DVector4: ...
    @property
    def ouuu(self) -> DVector4: ...
    @property
    def ouuv(self) -> DVector4: ...
    @property
    def ouuo(self) -> DVector4: ...
    @property
    def ouul(self) -> DVector4: ...
    @property
    def ouvu(self) -> DVector4: ...
    @property
    def ouvv(self) -> DVector4: ...
    @property
    def ouvo(self) -> DVector4: ...
    @property
    def ouvl(self) -> DVector4: ...
    @property
    def ouou(self) -> DVector4: ...
    @property
    def ouov(self) -> DVector4: ...
    @property
    def ouoo(self) -> DVector4: ...
    @property
    def ouol(self) -> DVector4: ...
    @property
    def oulu(self) -> DVector4: ...
    @property
    def oulv(self) -> DVector4: ...
    @property
    def oulo(self) -> DVector4: ...
    @property
    def oull(self) -> DVector4: ...
    @property
    def ovuu(self) -> DVector4: ...
    @property
    def ovuv(self) -> DVector4: ...
    @property
    def ovuo(self) -> DVector4: ...
    @property
    def ovul(self) -> DVector4: ...
    @property
    def ovvu(self) -> DVector4: ...
    @property
    def ovvv(self) -> DVector4: ...
    @property
    def ovvo(self) -> DVector4: ...
    @property
    def ovvl(self) -> DVector4: ...
    @property
    def ovou(self) -> DVector4: ...
    @property
    def ovov(self) -> DVector4: ...
    @property
    def ovoo(self) -> DVector4: ...
    @property
    def ovol(self) -> DVector4: ...
    @property
    def ovlu(self) -> DVector4: ...
    @property
    def ovlv(self) -> DVector4: ...
    @property
    def ovlo(self) -> DVector4: ...
    @property
    def ovll(self) -> DVector4: ...
    @property
    def oouu(self) -> DVector4: ...
    @property
    def oouv(self) -> DVector4: ...
    @property
    def oouo(self) -> DVector4: ...
    @property
    def ooul(self) -> DVector4: ...
    @property
    def oovu(self) -> DVector4: ...
    @property
    def oovv(self) -> DVector4: ...
    @property
    def oovo(self) -> DVector4: ...
    @property
    def oovl(self) -> DVector4: ...
    @property
    def ooou(self) -> DVector4: ...
    @property
    def ooov(self) -> DVector4: ...
    @property
    def oolu(self) -> DVector4: ...
    @property
    def oolv(self) -> DVector4: ...
    @property
    def oluu(self) -> DVector4: ...
    @property
    def oluv(self) -> DVector4: ...
    @property
    def oluo(self) -> DVector4: ...
    @property
    def olul(self) -> DVector4: ...
    @property
    def olvu(self) -> DVector4: ...
    @property
    def olvv(self) -> DVector4: ...
    @property
    def olvo(self) -> DVector4: ...
    @property
    def olvl(self) -> DVector4: ...
    @property
    def olou(self) -> DVector4: ...
    @property
    def olov(self) -> DVector4: ...
    @property
    def ollu(self) -> DVector4: ...
    @property
    def ollv(self) -> DVector4: ...
    @property
    def luuu(self) -> DVector4: ...
    @property
    def luuv(self) -> DVector4: ...
    @property
    def luuo(self) -> DVector4: ...
    @property
    def luul(self) -> DVector4: ...
    @property
    def luvu(self) -> DVector4: ...
    @property
    def luvv(self) -> DVector4: ...
    @property
    def luvo(self) -> DVector4: ...
    @property
    def luvl(self) -> DVector4: ...
    @property
    def luou(self) -> DVector4: ...
    @property
    def luov(self) -> DVector4: ...
    @property
    def luoo(self) -> DVector4: ...
    @property
    def luol(self) -> DVector4: ...
    @property
    def lulu(self) -> DVector4: ...
    @property
    def lulv(self) -> DVector4: ...
    @property
    def lulo(self) -> DVector4: ...
    @property
    def lull(self) -> DVector4: ...
    @property
    def lvuu(self) -> DVector4: ...
    @property
    def lvuv(self) -> DVector4: ...
    @property
    def lvuo(self) -> DVector4: ...
    @property
    def lvul(self) -> DVector4: ...
    @property
    def lvvu(self) -> DVector4: ...
    @property
    def lvvv(self) -> DVector4: ...
    @property
    def lvvo(self) -> DVector4: ...
    @property
    def lvvl(self) -> DVector4: ...
    @property
    def lvou(self) -> DVector4: ...
    @property
    def lvov(self) -> DVector4: ...
    @property
    def lvoo(self) -> DVector4: ...
    @property
    def lvol(self) -> DVector4: ...
    @property
    def lvlu(self) -> DVector4: ...
    @property
    def lvlv(self) -> DVector4: ...
    @property
    def lvlo(self) -> DVector4: ...
    @property
    def lvll(self) -> DVector4: ...
    @property
    def louu(self) -> DVector4: ...
    @property
    def louv(self) -> DVector4: ...
    @property
    def louo(self) -> DVector4: ...
    @property
    def loul(self) -> DVector4: ...
    @property
    def lovu(self) -> DVector4: ...
    @property
    def lovv(self) -> DVector4: ...
    @property
    def lovo(self) -> DVector4: ...
    @property
    def lovl(self) -> DVector4: ...
    @property
    def loou(self) -> DVector4: ...
    @property
    def loov(self) -> DVector4: ...
    @property
    def lolu(self) -> DVector4: ...
    @property
    def lolv(self) -> DVector4: ...
    @property
    def lluu(self) -> DVector4: ...
    @property
    def lluv(self) -> DVector4: ...
    @property
    def lluo(self) -> DVector4: ...
    @property
    def llul(self) -> DVector4: ...
    @property
    def llvu(self) -> DVector4: ...
    @property
    def llvv(self) -> DVector4: ...
    @property
    def llvo(self) -> DVector4: ...
    @property
    def llvl(self) -> DVector4: ...
    @property
    def llou(self) -> DVector4: ...
    @property
    def llov(self) -> DVector4: ...
    @property
    def lllu(self) -> DVector4: ...
    @property
    def lllv(self) -> DVector4: ...
    @property
    def oo(self) -> DVector2: ...
    @property
    def ol(self) -> DVector2: ...
    @property
    def lo(self) -> DVector2: ...
    @property
    def ll(self) -> DVector2: ...
    @property
    def ooo(self) -> DVector3: ...
    @property
    def ool(self) -> DVector3: ...
    @property
    def olo(self) -> DVector3: ...
    @property
    def oll(self) -> DVector3: ...
    @property
    def loo(self) -> DVector3: ...
    @property
    def lol(self) -> DVector3: ...
    @property
    def llo(self) -> DVector3: ...
    @property
    def lll(self) -> DVector3: ...
    @property
    def oooo(self) -> DVector4: ...
    @property
    def oool(self) -> DVector4: ...
    @property
    def oolo(self) -> DVector4: ...
    @property
    def ooll(self) -> DVector4: ...
    @property
    def oloo(self) -> DVector4: ...
    @property
    def olol(self) -> DVector4: ...
    @property
    def ollo(self) -> DVector4: ...
    @property
    def olll(self) -> DVector4: ...
    @property
    def looo(self) -> DVector4: ...
    @property
    def lool(self) -> DVector4: ...
    @property
    def lolo(self) -> DVector4: ...
    @property
    def loll(self) -> DVector4: ...
    @property
    def lloo(self) -> DVector4: ...
    @property
    def llol(self) -> DVector4: ...
    @property
    def lllo(self) -> DVector4: ...
    @property
    def llll(self) -> DVector4: ...
    @property
    def magnitude(self) -> float: ...
    def normalize(self) -> DVector2: ...
    def distance(self, other: DVector2, /) -> float: ...
    def lerp(self, other: DVector2, x: float, /) -> DVector2: ...
    def min(self, n: Number, /) -> DVector2: ...
    def max(self, n: Number, /) -> DVector2: ...
    def clamp(self, min: Number, max: Number, /) -> DVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[DVector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector2: ...

@final
class DVector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: DVector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector2]: ...
    @overload
    def __getitem__(self, index: int) -> DVector2: ...
    @overload
    def __getitem__(self, index: slice) -> DVector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[DVector2]: ...

@final
class FVector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FVector2) -> FVector2: ...
    @overload
    def __add__(self, other: Number) -> FVector2: ...
    @overload
    def __radd__(self, other: FVector2) -> FVector2: ...
    @overload
    def __radd__(self, other: Number) -> FVector2: ...
    @overload
    def __sub__(self, other: FVector2) -> FVector2: ...
    @overload
    def __sub__(self, other: Number) -> FVector2: ...
    @overload
    def __rsub__(self, other: FVector2) -> FVector2: ...
    @overload
    def __rsub__(self, other: Number) -> FVector2: ...
    @overload
    def __mul__(self, other: FVector2) -> FVector2: ...
    @overload
    def __mul__(self, other: Number) -> FVector2: ...
    @overload
    def __rmul__(self, other: FVector2) -> FVector2: ...
    @overload
    def __rmul__(self, other: Number) -> FVector2: ...
    def __matmul__(self, other: FVector2) -> float: ...
    def __rmatmul__(self, other: FVector2) -> float: ...
    @overload
    def __mod__(self, other: FVector2) -> FVector2: ...
    @overload
    def __mod__(self, other: Number) -> FVector2: ...
    @overload
    def __rmod__(self, other: FVector2) -> FVector2: ...
    @overload
    def __rmod__(self, other: Number) -> FVector2: ...
    @overload
    def __pow__(self, other: FVector2) -> FVector2: ...
    @overload
    def __pow__(self, other: Number) -> FVector2: ...
    @overload
    def __rpow__(self, other: FVector2) -> FVector2: ...
    @overload
    def __rpow__(self, other: Number) -> FVector2: ...
    @overload
    def __truediv__(self, other: FVector2) -> FVector2: ...
    @overload
    def __truediv__(self, other: Number) -> FVector2: ...
    @overload
    def __rtruediv__(self, other: FVector2) -> FVector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> FVector2: ...
    def __neg__(self) -> FVector2: ...
    def __abs__(self) -> FVector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def xx(self) -> FVector2: ...
    @property
    def xy(self) -> FVector2: ...
    @property
    def xo(self) -> FVector2: ...
    @property
    def xl(self) -> FVector2: ...
    @property
    def yx(self) -> FVector2: ...
    @property
    def yy(self) -> FVector2: ...
    @property
    def yo(self) -> FVector2: ...
    @property
    def yl(self) -> FVector2: ...
    @property
    def ox(self) -> FVector2: ...
    @property
    def oy(self) -> FVector2: ...
    @property
    def lx(self) -> FVector2: ...
    @property
    def ly(self) -> FVector2: ...
    @property
    def xxx(self) -> FVector3: ...
    @property
    def xxy(self) -> FVector3: ...
    @property
    def xxo(self) -> FVector3: ...
    @property
    def xxl(self) -> FVector3: ...
    @property
    def xyx(self) -> FVector3: ...
    @property
    def xyy(self) -> FVector3: ...
    @property
    def xyo(self) -> FVector3: ...
    @property
    def xyl(self) -> FVector3: ...
    @property
    def xox(self) -> FVector3: ...
    @property
    def xoy(self) -> FVector3: ...
    @property
    def xoo(self) -> FVector3: ...
    @property
    def xol(self) -> FVector3: ...
    @property
    def xlx(self) -> FVector3: ...
    @property
    def xly(self) -> FVector3: ...
    @property
    def xlo(self) -> FVector3: ...
    @property
    def xll(self) -> FVector3: ...
    @property
    def yxx(self) -> FVector3: ...
    @property
    def yxy(self) -> FVector3: ...
    @property
    def yxo(self) -> FVector3: ...
    @property
    def yxl(self) -> FVector3: ...
    @property
    def yyx(self) -> FVector3: ...
    @property
    def yyy(self) -> FVector3: ...
    @property
    def yyo(self) -> FVector3: ...
    @property
    def yyl(self) -> FVector3: ...
    @property
    def yox(self) -> FVector3: ...
    @property
    def yoy(self) -> FVector3: ...
    @property
    def yoo(self) -> FVector3: ...
    @property
    def yol(self) -> FVector3: ...
    @property
    def ylx(self) -> FVector3: ...
    @property
    def yly(self) -> FVector3: ...
    @property
    def ylo(self) -> FVector3: ...
    @property
    def yll(self) -> FVector3: ...
    @property
    def oxx(self) -> FVector3: ...
    @property
    def oxy(self) -> FVector3: ...
    @property
    def oxo(self) -> FVector3: ...
    @property
    def oxl(self) -> FVector3: ...
    @property
    def oyx(self) -> FVector3: ...
    @property
    def oyy(self) -> FVector3: ...
    @property
    def oyo(self) -> FVector3: ...
    @property
    def oyl(self) -> FVector3: ...
    @property
    def oox(self) -> FVector3: ...
    @property
    def ooy(self) -> FVector3: ...
    @property
    def olx(self) -> FVector3: ...
    @property
    def oly(self) -> FVector3: ...
    @property
    def lxx(self) -> FVector3: ...
    @property
    def lxy(self) -> FVector3: ...
    @property
    def lxo(self) -> FVector3: ...
    @property
    def lxl(self) -> FVector3: ...
    @property
    def lyx(self) -> FVector3: ...
    @property
    def lyy(self) -> FVector3: ...
    @property
    def lyo(self) -> FVector3: ...
    @property
    def lyl(self) -> FVector3: ...
    @property
    def lox(self) -> FVector3: ...
    @property
    def loy(self) -> FVector3: ...
    @property
    def llx(self) -> FVector3: ...
    @property
    def lly(self) -> FVector3: ...
    @property
    def xxxx(self) -> FVector4: ...
    @property
    def xxxy(self) -> FVector4: ...
    @property
    def xxxo(self) -> FVector4: ...
    @property
    def xxxl(self) -> FVector4: ...
    @property
    def xxyx(self) -> FVector4: ...
    @property
    def xxyy(self) -> FVector4: ...
    @property
    def xxyo(self) -> FVector4: ...
    @property
    def xxyl(self) -> FVector4: ...
    @property
    def xxox(self) -> FVector4: ...
    @property
    def xxoy(self) -> FVector4: ...
    @property
    def xxoo(self) -> FVector4: ...
    @property
    def xxol(self) -> FVector4: ...
    @property
    def xxlx(self) -> FVector4: ...
    @property
    def xxly(self) -> FVector4: ...
    @property
    def xxlo(self) -> FVector4: ...
    @property
    def xxll(self) -> FVector4: ...
    @property
    def xyxx(self) -> FVector4: ...
    @property
    def xyxy(self) -> FVector4: ...
    @property
    def xyxo(self) -> FVector4: ...
    @property
    def xyxl(self) -> FVector4: ...
    @property
    def xyyx(self) -> FVector4: ...
    @property
    def xyyy(self) -> FVector4: ...
    @property
    def xyyo(self) -> FVector4: ...
    @property
    def xyyl(self) -> FVector4: ...
    @property
    def xyox(self) -> FVector4: ...
    @property
    def xyoy(self) -> FVector4: ...
    @property
    def xyoo(self) -> FVector4: ...
    @property
    def xyol(self) -> FVector4: ...
    @property
    def xylx(self) -> FVector4: ...
    @property
    def xyly(self) -> FVector4: ...
    @property
    def xylo(self) -> FVector4: ...
    @property
    def xyll(self) -> FVector4: ...
    @property
    def xoxx(self) -> FVector4: ...
    @property
    def xoxy(self) -> FVector4: ...
    @property
    def xoxo(self) -> FVector4: ...
    @property
    def xoxl(self) -> FVector4: ...
    @property
    def xoyx(self) -> FVector4: ...
    @property
    def xoyy(self) -> FVector4: ...
    @property
    def xoyo(self) -> FVector4: ...
    @property
    def xoyl(self) -> FVector4: ...
    @property
    def xoox(self) -> FVector4: ...
    @property
    def xooy(self) -> FVector4: ...
    @property
    def xooo(self) -> FVector4: ...
    @property
    def xool(self) -> FVector4: ...
    @property
    def xolx(self) -> FVector4: ...
    @property
    def xoly(self) -> FVector4: ...
    @property
    def xolo(self) -> FVector4: ...
    @property
    def xoll(self) -> FVector4: ...
    @property
    def xlxx(self) -> FVector4: ...
    @property
    def xlxy(self) -> FVector4: ...
    @property
    def xlxo(self) -> FVector4: ...
    @property
    def xlxl(self) -> FVector4: ...
    @property
    def xlyx(self) -> FVector4: ...
    @property
    def xlyy(self) -> FVector4: ...
    @property
    def xlyo(self) -> FVector4: ...
    @property
    def xlyl(self) -> FVector4: ...
    @property
    def xlox(self) -> FVector4: ...
    @property
    def xloy(self) -> FVector4: ...
    @property
    def xloo(self) -> FVector4: ...
    @property
    def xlol(self) -> FVector4: ...
    @property
    def xllx(self) -> FVector4: ...
    @property
    def xlly(self) -> FVector4: ...
    @property
    def xllo(self) -> FVector4: ...
    @property
    def xlll(self) -> FVector4: ...
    @property
    def yxxx(self) -> FVector4: ...
    @property
    def yxxy(self) -> FVector4: ...
    @property
    def yxxo(self) -> FVector4: ...
    @property
    def yxxl(self) -> FVector4: ...
    @property
    def yxyx(self) -> FVector4: ...
    @property
    def yxyy(self) -> FVector4: ...
    @property
    def yxyo(self) -> FVector4: ...
    @property
    def yxyl(self) -> FVector4: ...
    @property
    def yxox(self) -> FVector4: ...
    @property
    def yxoy(self) -> FVector4: ...
    @property
    def yxoo(self) -> FVector4: ...
    @property
    def yxol(self) -> FVector4: ...
    @property
    def yxlx(self) -> FVector4: ...
    @property
    def yxly(self) -> FVector4: ...
    @property
    def yxlo(self) -> FVector4: ...
    @property
    def yxll(self) -> FVector4: ...
    @property
    def yyxx(self) -> FVector4: ...
    @property
    def yyxy(self) -> FVector4: ...
    @property
    def yyxo(self) -> FVector4: ...
    @property
    def yyxl(self) -> FVector4: ...
    @property
    def yyyx(self) -> FVector4: ...
    @property
    def yyyy(self) -> FVector4: ...
    @property
    def yyyo(self) -> FVector4: ...
    @property
    def yyyl(self) -> FVector4: ...
    @property
    def yyox(self) -> FVector4: ...
    @property
    def yyoy(self) -> FVector4: ...
    @property
    def yyoo(self) -> FVector4: ...
    @property
    def yyol(self) -> FVector4: ...
    @property
    def yylx(self) -> FVector4: ...
    @property
    def yyly(self) -> FVector4: ...
    @property
    def yylo(self) -> FVector4: ...
    @property
    def yyll(self) -> FVector4: ...
    @property
    def yoxx(self) -> FVector4: ...
    @property
    def yoxy(self) -> FVector4: ...
    @property
    def yoxo(self) -> FVector4: ...
    @property
    def yoxl(self) -> FVector4: ...
    @property
    def yoyx(self) -> FVector4: ...
    @property
    def yoyy(self) -> FVector4: ...
    @property
    def yoyo(self) -> FVector4: ...
    @property
    def yoyl(self) -> FVector4: ...
    @property
    def yoox(self) -> FVector4: ...
    @property
    def yooy(self) -> FVector4: ...
    @property
    def yooo(self) -> FVector4: ...
    @property
    def yool(self) -> FVector4: ...
    @property
    def yolx(self) -> FVector4: ...
    @property
    def yoly(self) -> FVector4: ...
    @property
    def yolo(self) -> FVector4: ...
    @property
    def yoll(self) -> FVector4: ...
    @property
    def ylxx(self) -> FVector4: ...
    @property
    def ylxy(self) -> FVector4: ...
    @property
    def ylxo(self) -> FVector4: ...
    @property
    def ylxl(self) -> FVector4: ...
    @property
    def ylyx(self) -> FVector4: ...
    @property
    def ylyy(self) -> FVector4: ...
    @property
    def ylyo(self) -> FVector4: ...
    @property
    def ylyl(self) -> FVector4: ...
    @property
    def ylox(self) -> FVector4: ...
    @property
    def yloy(self) -> FVector4: ...
    @property
    def yloo(self) -> FVector4: ...
    @property
    def ylol(self) -> FVector4: ...
    @property
    def yllx(self) -> FVector4: ...
    @property
    def ylly(self) -> FVector4: ...
    @property
    def yllo(self) -> FVector4: ...
    @property
    def ylll(self) -> FVector4: ...
    @property
    def oxxx(self) -> FVector4: ...
    @property
    def oxxy(self) -> FVector4: ...
    @property
    def oxxo(self) -> FVector4: ...
    @property
    def oxxl(self) -> FVector4: ...
    @property
    def oxyx(self) -> FVector4: ...
    @property
    def oxyy(self) -> FVector4: ...
    @property
    def oxyo(self) -> FVector4: ...
    @property
    def oxyl(self) -> FVector4: ...
    @property
    def oxox(self) -> FVector4: ...
    @property
    def oxoy(self) -> FVector4: ...
    @property
    def oxoo(self) -> FVector4: ...
    @property
    def oxol(self) -> FVector4: ...
    @property
    def oxlx(self) -> FVector4: ...
    @property
    def oxly(self) -> FVector4: ...
    @property
    def oxlo(self) -> FVector4: ...
    @property
    def oxll(self) -> FVector4: ...
    @property
    def oyxx(self) -> FVector4: ...
    @property
    def oyxy(self) -> FVector4: ...
    @property
    def oyxo(self) -> FVector4: ...
    @property
    def oyxl(self) -> FVector4: ...
    @property
    def oyyx(self) -> FVector4: ...
    @property
    def oyyy(self) -> FVector4: ...
    @property
    def oyyo(self) -> FVector4: ...
    @property
    def oyyl(self) -> FVector4: ...
    @property
    def oyox(self) -> FVector4: ...
    @property
    def oyoy(self) -> FVector4: ...
    @property
    def oyoo(self) -> FVector4: ...
    @property
    def oyol(self) -> FVector4: ...
    @property
    def oylx(self) -> FVector4: ...
    @property
    def oyly(self) -> FVector4: ...
    @property
    def oylo(self) -> FVector4: ...
    @property
    def oyll(self) -> FVector4: ...
    @property
    def ooxx(self) -> FVector4: ...
    @property
    def ooxy(self) -> FVector4: ...
    @property
    def ooxo(self) -> FVector4: ...
    @property
    def ooxl(self) -> FVector4: ...
    @property
    def ooyx(self) -> FVector4: ...
    @property
    def ooyy(self) -> FVector4: ...
    @property
    def ooyo(self) -> FVector4: ...
    @property
    def ooyl(self) -> FVector4: ...
    @property
    def ooox(self) -> FVector4: ...
    @property
    def oooy(self) -> FVector4: ...
    @property
    def oolx(self) -> FVector4: ...
    @property
    def ooly(self) -> FVector4: ...
    @property
    def olxx(self) -> FVector4: ...
    @property
    def olxy(self) -> FVector4: ...
    @property
    def olxo(self) -> FVector4: ...
    @property
    def olxl(self) -> FVector4: ...
    @property
    def olyx(self) -> FVector4: ...
    @property
    def olyy(self) -> FVector4: ...
    @property
    def olyo(self) -> FVector4: ...
    @property
    def olyl(self) -> FVector4: ...
    @property
    def olox(self) -> FVector4: ...
    @property
    def oloy(self) -> FVector4: ...
    @property
    def ollx(self) -> FVector4: ...
    @property
    def olly(self) -> FVector4: ...
    @property
    def lxxx(self) -> FVector4: ...
    @property
    def lxxy(self) -> FVector4: ...
    @property
    def lxxo(self) -> FVector4: ...
    @property
    def lxxl(self) -> FVector4: ...
    @property
    def lxyx(self) -> FVector4: ...
    @property
    def lxyy(self) -> FVector4: ...
    @property
    def lxyo(self) -> FVector4: ...
    @property
    def lxyl(self) -> FVector4: ...
    @property
    def lxox(self) -> FVector4: ...
    @property
    def lxoy(self) -> FVector4: ...
    @property
    def lxoo(self) -> FVector4: ...
    @property
    def lxol(self) -> FVector4: ...
    @property
    def lxlx(self) -> FVector4: ...
    @property
    def lxly(self) -> FVector4: ...
    @property
    def lxlo(self) -> FVector4: ...
    @property
    def lxll(self) -> FVector4: ...
    @property
    def lyxx(self) -> FVector4: ...
    @property
    def lyxy(self) -> FVector4: ...
    @property
    def lyxo(self) -> FVector4: ...
    @property
    def lyxl(self) -> FVector4: ...
    @property
    def lyyx(self) -> FVector4: ...
    @property
    def lyyy(self) -> FVector4: ...
    @property
    def lyyo(self) -> FVector4: ...
    @property
    def lyyl(self) -> FVector4: ...
    @property
    def lyox(self) -> FVector4: ...
    @property
    def lyoy(self) -> FVector4: ...
    @property
    def lyoo(self) -> FVector4: ...
    @property
    def lyol(self) -> FVector4: ...
    @property
    def lylx(self) -> FVector4: ...
    @property
    def lyly(self) -> FVector4: ...
    @property
    def lylo(self) -> FVector4: ...
    @property
    def lyll(self) -> FVector4: ...
    @property
    def loxx(self) -> FVector4: ...
    @property
    def loxy(self) -> FVector4: ...
    @property
    def loxo(self) -> FVector4: ...
    @property
    def loxl(self) -> FVector4: ...
    @property
    def loyx(self) -> FVector4: ...
    @property
    def loyy(self) -> FVector4: ...
    @property
    def loyo(self) -> FVector4: ...
    @property
    def loyl(self) -> FVector4: ...
    @property
    def loox(self) -> FVector4: ...
    @property
    def looy(self) -> FVector4: ...
    @property
    def lolx(self) -> FVector4: ...
    @property
    def loly(self) -> FVector4: ...
    @property
    def llxx(self) -> FVector4: ...
    @property
    def llxy(self) -> FVector4: ...
    @property
    def llxo(self) -> FVector4: ...
    @property
    def llxl(self) -> FVector4: ...
    @property
    def llyx(self) -> FVector4: ...
    @property
    def llyy(self) -> FVector4: ...
    @property
    def llyo(self) -> FVector4: ...
    @property
    def llyl(self) -> FVector4: ...
    @property
    def llox(self) -> FVector4: ...
    @property
    def lloy(self) -> FVector4: ...
    @property
    def lllx(self) -> FVector4: ...
    @property
    def llly(self) -> FVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def g(self) -> float: ...
    @property
    def rr(self) -> FVector2: ...
    @property
    def rg(self) -> FVector2: ...
    @property
    def ro(self) -> FVector2: ...
    @property
    def rl(self) -> FVector2: ...
    @property
    def gr(self) -> FVector2: ...
    @property
    def gg(self) -> FVector2: ...
    @property
    def go(self) -> FVector2: ...
    @property
    def gl(self) -> FVector2: ...
    @property
    def og(self) -> FVector2: ...
    @property
    def lr(self) -> FVector2: ...
    @property
    def lg(self) -> FVector2: ...
    @property
    def rrr(self) -> FVector3: ...
    @property
    def rrg(self) -> FVector3: ...
    @property
    def rro(self) -> FVector3: ...
    @property
    def rrl(self) -> FVector3: ...
    @property
    def rgr(self) -> FVector3: ...
    @property
    def rgg(self) -> FVector3: ...
    @property
    def rgo(self) -> FVector3: ...
    @property
    def rgl(self) -> FVector3: ...
    @property
    def ror(self) -> FVector3: ...
    @property
    def rog(self) -> FVector3: ...
    @property
    def roo(self) -> FVector3: ...
    @property
    def rol(self) -> FVector3: ...
    @property
    def rlr(self) -> FVector3: ...
    @property
    def rlg(self) -> FVector3: ...
    @property
    def rlo(self) -> FVector3: ...
    @property
    def rll(self) -> FVector3: ...
    @property
    def grr(self) -> FVector3: ...
    @property
    def grg(self) -> FVector3: ...
    @property
    def gro(self) -> FVector3: ...
    @property
    def grl(self) -> FVector3: ...
    @property
    def ggr(self) -> FVector3: ...
    @property
    def ggg(self) -> FVector3: ...
    @property
    def ggo(self) -> FVector3: ...
    @property
    def ggl(self) -> FVector3: ...
    @property
    def gor(self) -> FVector3: ...
    @property
    def gog(self) -> FVector3: ...
    @property
    def goo(self) -> FVector3: ...
    @property
    def gol(self) -> FVector3: ...
    @property
    def glr(self) -> FVector3: ...
    @property
    def glg(self) -> FVector3: ...
    @property
    def glo(self) -> FVector3: ...
    @property
    def gll(self) -> FVector3: ...
    @property
    def orr(self) -> FVector3: ...
    @property
    def org(self) -> FVector3: ...
    @property
    def oro(self) -> FVector3: ...
    @property
    def orl(self) -> FVector3: ...
    @property
    def ogr(self) -> FVector3: ...
    @property
    def ogg(self) -> FVector3: ...
    @property
    def ogo(self) -> FVector3: ...
    @property
    def ogl(self) -> FVector3: ...
    @property
    def oor(self) -> FVector3: ...
    @property
    def oog(self) -> FVector3: ...
    @property
    def olr(self) -> FVector3: ...
    @property
    def olg(self) -> FVector3: ...
    @property
    def lrr(self) -> FVector3: ...
    @property
    def lrg(self) -> FVector3: ...
    @property
    def lro(self) -> FVector3: ...
    @property
    def lrl(self) -> FVector3: ...
    @property
    def lgr(self) -> FVector3: ...
    @property
    def lgg(self) -> FVector3: ...
    @property
    def lgo(self) -> FVector3: ...
    @property
    def lgl(self) -> FVector3: ...
    @property
    def lor(self) -> FVector3: ...
    @property
    def log(self) -> FVector3: ...
    @property
    def llr(self) -> FVector3: ...
    @property
    def llg(self) -> FVector3: ...
    @property
    def rrrr(self) -> FVector4: ...
    @property
    def rrrg(self) -> FVector4: ...
    @property
    def rrro(self) -> FVector4: ...
    @property
    def rrrl(self) -> FVector4: ...
    @property
    def rrgr(self) -> FVector4: ...
    @property
    def rrgg(self) -> FVector4: ...
    @property
    def rrgo(self) -> FVector4: ...
    @property
    def rrgl(self) -> FVector4: ...
    @property
    def rror(self) -> FVector4: ...
    @property
    def rrog(self) -> FVector4: ...
    @property
    def rroo(self) -> FVector4: ...
    @property
    def rrol(self) -> FVector4: ...
    @property
    def rrlr(self) -> FVector4: ...
    @property
    def rrlg(self) -> FVector4: ...
    @property
    def rrlo(self) -> FVector4: ...
    @property
    def rrll(self) -> FVector4: ...
    @property
    def rgrr(self) -> FVector4: ...
    @property
    def rgrg(self) -> FVector4: ...
    @property
    def rgro(self) -> FVector4: ...
    @property
    def rgrl(self) -> FVector4: ...
    @property
    def rggr(self) -> FVector4: ...
    @property
    def rggg(self) -> FVector4: ...
    @property
    def rggo(self) -> FVector4: ...
    @property
    def rggl(self) -> FVector4: ...
    @property
    def rgor(self) -> FVector4: ...
    @property
    def rgog(self) -> FVector4: ...
    @property
    def rgoo(self) -> FVector4: ...
    @property
    def rgol(self) -> FVector4: ...
    @property
    def rglr(self) -> FVector4: ...
    @property
    def rglg(self) -> FVector4: ...
    @property
    def rglo(self) -> FVector4: ...
    @property
    def rgll(self) -> FVector4: ...
    @property
    def rorr(self) -> FVector4: ...
    @property
    def rorg(self) -> FVector4: ...
    @property
    def roro(self) -> FVector4: ...
    @property
    def rorl(self) -> FVector4: ...
    @property
    def rogr(self) -> FVector4: ...
    @property
    def rogg(self) -> FVector4: ...
    @property
    def rogo(self) -> FVector4: ...
    @property
    def rogl(self) -> FVector4: ...
    @property
    def roor(self) -> FVector4: ...
    @property
    def roog(self) -> FVector4: ...
    @property
    def rooo(self) -> FVector4: ...
    @property
    def rool(self) -> FVector4: ...
    @property
    def rolr(self) -> FVector4: ...
    @property
    def rolg(self) -> FVector4: ...
    @property
    def rolo(self) -> FVector4: ...
    @property
    def roll(self) -> FVector4: ...
    @property
    def rlrr(self) -> FVector4: ...
    @property
    def rlrg(self) -> FVector4: ...
    @property
    def rlro(self) -> FVector4: ...
    @property
    def rlrl(self) -> FVector4: ...
    @property
    def rlgr(self) -> FVector4: ...
    @property
    def rlgg(self) -> FVector4: ...
    @property
    def rlgo(self) -> FVector4: ...
    @property
    def rlgl(self) -> FVector4: ...
    @property
    def rlor(self) -> FVector4: ...
    @property
    def rlog(self) -> FVector4: ...
    @property
    def rloo(self) -> FVector4: ...
    @property
    def rlol(self) -> FVector4: ...
    @property
    def rllr(self) -> FVector4: ...
    @property
    def rllg(self) -> FVector4: ...
    @property
    def rllo(self) -> FVector4: ...
    @property
    def rlll(self) -> FVector4: ...
    @property
    def grrr(self) -> FVector4: ...
    @property
    def grrg(self) -> FVector4: ...
    @property
    def grro(self) -> FVector4: ...
    @property
    def grrl(self) -> FVector4: ...
    @property
    def grgr(self) -> FVector4: ...
    @property
    def grgg(self) -> FVector4: ...
    @property
    def grgo(self) -> FVector4: ...
    @property
    def grgl(self) -> FVector4: ...
    @property
    def gror(self) -> FVector4: ...
    @property
    def grog(self) -> FVector4: ...
    @property
    def groo(self) -> FVector4: ...
    @property
    def grol(self) -> FVector4: ...
    @property
    def grlr(self) -> FVector4: ...
    @property
    def grlg(self) -> FVector4: ...
    @property
    def grlo(self) -> FVector4: ...
    @property
    def grll(self) -> FVector4: ...
    @property
    def ggrr(self) -> FVector4: ...
    @property
    def ggrg(self) -> FVector4: ...
    @property
    def ggro(self) -> FVector4: ...
    @property
    def ggrl(self) -> FVector4: ...
    @property
    def gggr(self) -> FVector4: ...
    @property
    def gggg(self) -> FVector4: ...
    @property
    def gggo(self) -> FVector4: ...
    @property
    def gggl(self) -> FVector4: ...
    @property
    def ggor(self) -> FVector4: ...
    @property
    def ggog(self) -> FVector4: ...
    @property
    def ggoo(self) -> FVector4: ...
    @property
    def ggol(self) -> FVector4: ...
    @property
    def gglr(self) -> FVector4: ...
    @property
    def gglg(self) -> FVector4: ...
    @property
    def gglo(self) -> FVector4: ...
    @property
    def ggll(self) -> FVector4: ...
    @property
    def gorr(self) -> FVector4: ...
    @property
    def gorg(self) -> FVector4: ...
    @property
    def goro(self) -> FVector4: ...
    @property
    def gorl(self) -> FVector4: ...
    @property
    def gogr(self) -> FVector4: ...
    @property
    def gogg(self) -> FVector4: ...
    @property
    def gogo(self) -> FVector4: ...
    @property
    def gogl(self) -> FVector4: ...
    @property
    def goor(self) -> FVector4: ...
    @property
    def goog(self) -> FVector4: ...
    @property
    def gooo(self) -> FVector4: ...
    @property
    def gool(self) -> FVector4: ...
    @property
    def golr(self) -> FVector4: ...
    @property
    def golg(self) -> FVector4: ...
    @property
    def golo(self) -> FVector4: ...
    @property
    def goll(self) -> FVector4: ...
    @property
    def glrr(self) -> FVector4: ...
    @property
    def glrg(self) -> FVector4: ...
    @property
    def glro(self) -> FVector4: ...
    @property
    def glrl(self) -> FVector4: ...
    @property
    def glgr(self) -> FVector4: ...
    @property
    def glgg(self) -> FVector4: ...
    @property
    def glgo(self) -> FVector4: ...
    @property
    def glgl(self) -> FVector4: ...
    @property
    def glor(self) -> FVector4: ...
    @property
    def glog(self) -> FVector4: ...
    @property
    def gloo(self) -> FVector4: ...
    @property
    def glol(self) -> FVector4: ...
    @property
    def gllr(self) -> FVector4: ...
    @property
    def gllg(self) -> FVector4: ...
    @property
    def gllo(self) -> FVector4: ...
    @property
    def glll(self) -> FVector4: ...
    @property
    def orrr(self) -> FVector4: ...
    @property
    def orrg(self) -> FVector4: ...
    @property
    def orro(self) -> FVector4: ...
    @property
    def orrl(self) -> FVector4: ...
    @property
    def orgr(self) -> FVector4: ...
    @property
    def orgg(self) -> FVector4: ...
    @property
    def orgo(self) -> FVector4: ...
    @property
    def orgl(self) -> FVector4: ...
    @property
    def oror(self) -> FVector4: ...
    @property
    def orog(self) -> FVector4: ...
    @property
    def oroo(self) -> FVector4: ...
    @property
    def orol(self) -> FVector4: ...
    @property
    def orlr(self) -> FVector4: ...
    @property
    def orlg(self) -> FVector4: ...
    @property
    def orlo(self) -> FVector4: ...
    @property
    def orll(self) -> FVector4: ...
    @property
    def ogrr(self) -> FVector4: ...
    @property
    def ogrg(self) -> FVector4: ...
    @property
    def ogro(self) -> FVector4: ...
    @property
    def ogrl(self) -> FVector4: ...
    @property
    def oggr(self) -> FVector4: ...
    @property
    def oggg(self) -> FVector4: ...
    @property
    def oggo(self) -> FVector4: ...
    @property
    def oggl(self) -> FVector4: ...
    @property
    def ogor(self) -> FVector4: ...
    @property
    def ogog(self) -> FVector4: ...
    @property
    def ogoo(self) -> FVector4: ...
    @property
    def ogol(self) -> FVector4: ...
    @property
    def oglr(self) -> FVector4: ...
    @property
    def oglg(self) -> FVector4: ...
    @property
    def oglo(self) -> FVector4: ...
    @property
    def ogll(self) -> FVector4: ...
    @property
    def oorr(self) -> FVector4: ...
    @property
    def oorg(self) -> FVector4: ...
    @property
    def ooro(self) -> FVector4: ...
    @property
    def oorl(self) -> FVector4: ...
    @property
    def oogr(self) -> FVector4: ...
    @property
    def oogg(self) -> FVector4: ...
    @property
    def oogo(self) -> FVector4: ...
    @property
    def oogl(self) -> FVector4: ...
    @property
    def ooor(self) -> FVector4: ...
    @property
    def ooog(self) -> FVector4: ...
    @property
    def oolr(self) -> FVector4: ...
    @property
    def oolg(self) -> FVector4: ...
    @property
    def olrr(self) -> FVector4: ...
    @property
    def olrg(self) -> FVector4: ...
    @property
    def olro(self) -> FVector4: ...
    @property
    def olrl(self) -> FVector4: ...
    @property
    def olgr(self) -> FVector4: ...
    @property
    def olgg(self) -> FVector4: ...
    @property
    def olgo(self) -> FVector4: ...
    @property
    def olgl(self) -> FVector4: ...
    @property
    def olor(self) -> FVector4: ...
    @property
    def olog(self) -> FVector4: ...
    @property
    def ollr(self) -> FVector4: ...
    @property
    def ollg(self) -> FVector4: ...
    @property
    def lrrr(self) -> FVector4: ...
    @property
    def lrrg(self) -> FVector4: ...
    @property
    def lrro(self) -> FVector4: ...
    @property
    def lrrl(self) -> FVector4: ...
    @property
    def lrgr(self) -> FVector4: ...
    @property
    def lrgg(self) -> FVector4: ...
    @property
    def lrgo(self) -> FVector4: ...
    @property
    def lrgl(self) -> FVector4: ...
    @property
    def lror(self) -> FVector4: ...
    @property
    def lrog(self) -> FVector4: ...
    @property
    def lroo(self) -> FVector4: ...
    @property
    def lrol(self) -> FVector4: ...
    @property
    def lrlr(self) -> FVector4: ...
    @property
    def lrlg(self) -> FVector4: ...
    @property
    def lrlo(self) -> FVector4: ...
    @property
    def lrll(self) -> FVector4: ...
    @property
    def lgrr(self) -> FVector4: ...
    @property
    def lgrg(self) -> FVector4: ...
    @property
    def lgro(self) -> FVector4: ...
    @property
    def lgrl(self) -> FVector4: ...
    @property
    def lggr(self) -> FVector4: ...
    @property
    def lggg(self) -> FVector4: ...
    @property
    def lggo(self) -> FVector4: ...
    @property
    def lggl(self) -> FVector4: ...
    @property
    def lgor(self) -> FVector4: ...
    @property
    def lgog(self) -> FVector4: ...
    @property
    def lgoo(self) -> FVector4: ...
    @property
    def lgol(self) -> FVector4: ...
    @property
    def lglr(self) -> FVector4: ...
    @property
    def lglg(self) -> FVector4: ...
    @property
    def lglo(self) -> FVector4: ...
    @property
    def lgll(self) -> FVector4: ...
    @property
    def lorr(self) -> FVector4: ...
    @property
    def lorg(self) -> FVector4: ...
    @property
    def loro(self) -> FVector4: ...
    @property
    def lorl(self) -> FVector4: ...
    @property
    def logr(self) -> FVector4: ...
    @property
    def logg(self) -> FVector4: ...
    @property
    def logo(self) -> FVector4: ...
    @property
    def logl(self) -> FVector4: ...
    @property
    def loor(self) -> FVector4: ...
    @property
    def loog(self) -> FVector4: ...
    @property
    def lolr(self) -> FVector4: ...
    @property
    def lolg(self) -> FVector4: ...
    @property
    def llrr(self) -> FVector4: ...
    @property
    def llrg(self) -> FVector4: ...
    @property
    def llro(self) -> FVector4: ...
    @property
    def llrl(self) -> FVector4: ...
    @property
    def llgr(self) -> FVector4: ...
    @property
    def llgg(self) -> FVector4: ...
    @property
    def llgo(self) -> FVector4: ...
    @property
    def llgl(self) -> FVector4: ...
    @property
    def llor(self) -> FVector4: ...
    @property
    def llog(self) -> FVector4: ...
    @property
    def lllr(self) -> FVector4: ...
    @property
    def lllg(self) -> FVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def t(self) -> float: ...
    @property
    def ss(self) -> FVector2: ...
    @property
    def st(self) -> FVector2: ...
    @property
    def so(self) -> FVector2: ...
    @property
    def sl(self) -> FVector2: ...
    @property
    def ts(self) -> FVector2: ...
    @property
    def tt(self) -> FVector2: ...
    @property
    def to(self) -> FVector2: ...
    @property
    def tl(self) -> FVector2: ...
    @property
    def os(self) -> FVector2: ...
    @property
    def ot(self) -> FVector2: ...
    @property
    def ls(self) -> FVector2: ...
    @property
    def lt(self) -> FVector2: ...
    @property
    def sss(self) -> FVector3: ...
    @property
    def sst(self) -> FVector3: ...
    @property
    def sso(self) -> FVector3: ...
    @property
    def ssl(self) -> FVector3: ...
    @property
    def sts(self) -> FVector3: ...
    @property
    def stt(self) -> FVector3: ...
    @property
    def sto(self) -> FVector3: ...
    @property
    def stl(self) -> FVector3: ...
    @property
    def sos(self) -> FVector3: ...
    @property
    def sot(self) -> FVector3: ...
    @property
    def soo(self) -> FVector3: ...
    @property
    def sol(self) -> FVector3: ...
    @property
    def sls(self) -> FVector3: ...
    @property
    def slt(self) -> FVector3: ...
    @property
    def slo(self) -> FVector3: ...
    @property
    def sll(self) -> FVector3: ...
    @property
    def tss(self) -> FVector3: ...
    @property
    def tst(self) -> FVector3: ...
    @property
    def tso(self) -> FVector3: ...
    @property
    def tsl(self) -> FVector3: ...
    @property
    def tts(self) -> FVector3: ...
    @property
    def ttt(self) -> FVector3: ...
    @property
    def tto(self) -> FVector3: ...
    @property
    def ttl(self) -> FVector3: ...
    @property
    def tos(self) -> FVector3: ...
    @property
    def tot(self) -> FVector3: ...
    @property
    def too(self) -> FVector3: ...
    @property
    def tol(self) -> FVector3: ...
    @property
    def tls(self) -> FVector3: ...
    @property
    def tlt(self) -> FVector3: ...
    @property
    def tlo(self) -> FVector3: ...
    @property
    def tll(self) -> FVector3: ...
    @property
    def oss(self) -> FVector3: ...
    @property
    def ost(self) -> FVector3: ...
    @property
    def oso(self) -> FVector3: ...
    @property
    def osl(self) -> FVector3: ...
    @property
    def ots(self) -> FVector3: ...
    @property
    def ott(self) -> FVector3: ...
    @property
    def oto(self) -> FVector3: ...
    @property
    def otl(self) -> FVector3: ...
    @property
    def oos(self) -> FVector3: ...
    @property
    def oot(self) -> FVector3: ...
    @property
    def ols(self) -> FVector3: ...
    @property
    def olt(self) -> FVector3: ...
    @property
    def lss(self) -> FVector3: ...
    @property
    def lst(self) -> FVector3: ...
    @property
    def lso(self) -> FVector3: ...
    @property
    def lsl(self) -> FVector3: ...
    @property
    def lts(self) -> FVector3: ...
    @property
    def ltt(self) -> FVector3: ...
    @property
    def lto(self) -> FVector3: ...
    @property
    def ltl(self) -> FVector3: ...
    @property
    def los(self) -> FVector3: ...
    @property
    def lot(self) -> FVector3: ...
    @property
    def lls(self) -> FVector3: ...
    @property
    def llt(self) -> FVector3: ...
    @property
    def ssss(self) -> FVector4: ...
    @property
    def ssst(self) -> FVector4: ...
    @property
    def ssso(self) -> FVector4: ...
    @property
    def sssl(self) -> FVector4: ...
    @property
    def ssts(self) -> FVector4: ...
    @property
    def sstt(self) -> FVector4: ...
    @property
    def ssto(self) -> FVector4: ...
    @property
    def sstl(self) -> FVector4: ...
    @property
    def ssos(self) -> FVector4: ...
    @property
    def ssot(self) -> FVector4: ...
    @property
    def ssoo(self) -> FVector4: ...
    @property
    def ssol(self) -> FVector4: ...
    @property
    def ssls(self) -> FVector4: ...
    @property
    def sslt(self) -> FVector4: ...
    @property
    def sslo(self) -> FVector4: ...
    @property
    def ssll(self) -> FVector4: ...
    @property
    def stss(self) -> FVector4: ...
    @property
    def stst(self) -> FVector4: ...
    @property
    def stso(self) -> FVector4: ...
    @property
    def stsl(self) -> FVector4: ...
    @property
    def stts(self) -> FVector4: ...
    @property
    def sttt(self) -> FVector4: ...
    @property
    def stto(self) -> FVector4: ...
    @property
    def sttl(self) -> FVector4: ...
    @property
    def stos(self) -> FVector4: ...
    @property
    def stot(self) -> FVector4: ...
    @property
    def stoo(self) -> FVector4: ...
    @property
    def stol(self) -> FVector4: ...
    @property
    def stls(self) -> FVector4: ...
    @property
    def stlt(self) -> FVector4: ...
    @property
    def stlo(self) -> FVector4: ...
    @property
    def stll(self) -> FVector4: ...
    @property
    def soss(self) -> FVector4: ...
    @property
    def sost(self) -> FVector4: ...
    @property
    def soso(self) -> FVector4: ...
    @property
    def sosl(self) -> FVector4: ...
    @property
    def sots(self) -> FVector4: ...
    @property
    def sott(self) -> FVector4: ...
    @property
    def soto(self) -> FVector4: ...
    @property
    def sotl(self) -> FVector4: ...
    @property
    def soos(self) -> FVector4: ...
    @property
    def soot(self) -> FVector4: ...
    @property
    def sooo(self) -> FVector4: ...
    @property
    def sool(self) -> FVector4: ...
    @property
    def sols(self) -> FVector4: ...
    @property
    def solt(self) -> FVector4: ...
    @property
    def solo(self) -> FVector4: ...
    @property
    def soll(self) -> FVector4: ...
    @property
    def slss(self) -> FVector4: ...
    @property
    def slst(self) -> FVector4: ...
    @property
    def slso(self) -> FVector4: ...
    @property
    def slsl(self) -> FVector4: ...
    @property
    def slts(self) -> FVector4: ...
    @property
    def sltt(self) -> FVector4: ...
    @property
    def slto(self) -> FVector4: ...
    @property
    def sltl(self) -> FVector4: ...
    @property
    def slos(self) -> FVector4: ...
    @property
    def slot(self) -> FVector4: ...
    @property
    def sloo(self) -> FVector4: ...
    @property
    def slol(self) -> FVector4: ...
    @property
    def slls(self) -> FVector4: ...
    @property
    def sllt(self) -> FVector4: ...
    @property
    def sllo(self) -> FVector4: ...
    @property
    def slll(self) -> FVector4: ...
    @property
    def tsss(self) -> FVector4: ...
    @property
    def tsst(self) -> FVector4: ...
    @property
    def tsso(self) -> FVector4: ...
    @property
    def tssl(self) -> FVector4: ...
    @property
    def tsts(self) -> FVector4: ...
    @property
    def tstt(self) -> FVector4: ...
    @property
    def tsto(self) -> FVector4: ...
    @property
    def tstl(self) -> FVector4: ...
    @property
    def tsos(self) -> FVector4: ...
    @property
    def tsot(self) -> FVector4: ...
    @property
    def tsoo(self) -> FVector4: ...
    @property
    def tsol(self) -> FVector4: ...
    @property
    def tsls(self) -> FVector4: ...
    @property
    def tslt(self) -> FVector4: ...
    @property
    def tslo(self) -> FVector4: ...
    @property
    def tsll(self) -> FVector4: ...
    @property
    def ttss(self) -> FVector4: ...
    @property
    def ttst(self) -> FVector4: ...
    @property
    def ttso(self) -> FVector4: ...
    @property
    def ttsl(self) -> FVector4: ...
    @property
    def ttts(self) -> FVector4: ...
    @property
    def tttt(self) -> FVector4: ...
    @property
    def ttto(self) -> FVector4: ...
    @property
    def tttl(self) -> FVector4: ...
    @property
    def ttos(self) -> FVector4: ...
    @property
    def ttot(self) -> FVector4: ...
    @property
    def ttoo(self) -> FVector4: ...
    @property
    def ttol(self) -> FVector4: ...
    @property
    def ttls(self) -> FVector4: ...
    @property
    def ttlt(self) -> FVector4: ...
    @property
    def ttlo(self) -> FVector4: ...
    @property
    def ttll(self) -> FVector4: ...
    @property
    def toss(self) -> FVector4: ...
    @property
    def tost(self) -> FVector4: ...
    @property
    def toso(self) -> FVector4: ...
    @property
    def tosl(self) -> FVector4: ...
    @property
    def tots(self) -> FVector4: ...
    @property
    def tott(self) -> FVector4: ...
    @property
    def toto(self) -> FVector4: ...
    @property
    def totl(self) -> FVector4: ...
    @property
    def toos(self) -> FVector4: ...
    @property
    def toot(self) -> FVector4: ...
    @property
    def tooo(self) -> FVector4: ...
    @property
    def tool(self) -> FVector4: ...
    @property
    def tols(self) -> FVector4: ...
    @property
    def tolt(self) -> FVector4: ...
    @property
    def tolo(self) -> FVector4: ...
    @property
    def toll(self) -> FVector4: ...
    @property
    def tlss(self) -> FVector4: ...
    @property
    def tlst(self) -> FVector4: ...
    @property
    def tlso(self) -> FVector4: ...
    @property
    def tlsl(self) -> FVector4: ...
    @property
    def tlts(self) -> FVector4: ...
    @property
    def tltt(self) -> FVector4: ...
    @property
    def tlto(self) -> FVector4: ...
    @property
    def tltl(self) -> FVector4: ...
    @property
    def tlos(self) -> FVector4: ...
    @property
    def tlot(self) -> FVector4: ...
    @property
    def tloo(self) -> FVector4: ...
    @property
    def tlol(self) -> FVector4: ...
    @property
    def tlls(self) -> FVector4: ...
    @property
    def tllt(self) -> FVector4: ...
    @property
    def tllo(self) -> FVector4: ...
    @property
    def tlll(self) -> FVector4: ...
    @property
    def osss(self) -> FVector4: ...
    @property
    def osst(self) -> FVector4: ...
    @property
    def osso(self) -> FVector4: ...
    @property
    def ossl(self) -> FVector4: ...
    @property
    def osts(self) -> FVector4: ...
    @property
    def ostt(self) -> FVector4: ...
    @property
    def osto(self) -> FVector4: ...
    @property
    def ostl(self) -> FVector4: ...
    @property
    def osos(self) -> FVector4: ...
    @property
    def osot(self) -> FVector4: ...
    @property
    def osoo(self) -> FVector4: ...
    @property
    def osol(self) -> FVector4: ...
    @property
    def osls(self) -> FVector4: ...
    @property
    def oslt(self) -> FVector4: ...
    @property
    def oslo(self) -> FVector4: ...
    @property
    def osll(self) -> FVector4: ...
    @property
    def otss(self) -> FVector4: ...
    @property
    def otst(self) -> FVector4: ...
    @property
    def otso(self) -> FVector4: ...
    @property
    def otsl(self) -> FVector4: ...
    @property
    def otts(self) -> FVector4: ...
    @property
    def ottt(self) -> FVector4: ...
    @property
    def otto(self) -> FVector4: ...
    @property
    def ottl(self) -> FVector4: ...
    @property
    def otos(self) -> FVector4: ...
    @property
    def otot(self) -> FVector4: ...
    @property
    def otoo(self) -> FVector4: ...
    @property
    def otol(self) -> FVector4: ...
    @property
    def otls(self) -> FVector4: ...
    @property
    def otlt(self) -> FVector4: ...
    @property
    def otlo(self) -> FVector4: ...
    @property
    def otll(self) -> FVector4: ...
    @property
    def ooss(self) -> FVector4: ...
    @property
    def oost(self) -> FVector4: ...
    @property
    def ooso(self) -> FVector4: ...
    @property
    def oosl(self) -> FVector4: ...
    @property
    def oots(self) -> FVector4: ...
    @property
    def oott(self) -> FVector4: ...
    @property
    def ooto(self) -> FVector4: ...
    @property
    def ootl(self) -> FVector4: ...
    @property
    def ooos(self) -> FVector4: ...
    @property
    def ooot(self) -> FVector4: ...
    @property
    def ools(self) -> FVector4: ...
    @property
    def oolt(self) -> FVector4: ...
    @property
    def olss(self) -> FVector4: ...
    @property
    def olst(self) -> FVector4: ...
    @property
    def olso(self) -> FVector4: ...
    @property
    def olsl(self) -> FVector4: ...
    @property
    def olts(self) -> FVector4: ...
    @property
    def oltt(self) -> FVector4: ...
    @property
    def olto(self) -> FVector4: ...
    @property
    def oltl(self) -> FVector4: ...
    @property
    def olos(self) -> FVector4: ...
    @property
    def olot(self) -> FVector4: ...
    @property
    def olls(self) -> FVector4: ...
    @property
    def ollt(self) -> FVector4: ...
    @property
    def lsss(self) -> FVector4: ...
    @property
    def lsst(self) -> FVector4: ...
    @property
    def lsso(self) -> FVector4: ...
    @property
    def lssl(self) -> FVector4: ...
    @property
    def lsts(self) -> FVector4: ...
    @property
    def lstt(self) -> FVector4: ...
    @property
    def lsto(self) -> FVector4: ...
    @property
    def lstl(self) -> FVector4: ...
    @property
    def lsos(self) -> FVector4: ...
    @property
    def lsot(self) -> FVector4: ...
    @property
    def lsoo(self) -> FVector4: ...
    @property
    def lsol(self) -> FVector4: ...
    @property
    def lsls(self) -> FVector4: ...
    @property
    def lslt(self) -> FVector4: ...
    @property
    def lslo(self) -> FVector4: ...
    @property
    def lsll(self) -> FVector4: ...
    @property
    def ltss(self) -> FVector4: ...
    @property
    def ltst(self) -> FVector4: ...
    @property
    def ltso(self) -> FVector4: ...
    @property
    def ltsl(self) -> FVector4: ...
    @property
    def ltts(self) -> FVector4: ...
    @property
    def lttt(self) -> FVector4: ...
    @property
    def ltto(self) -> FVector4: ...
    @property
    def lttl(self) -> FVector4: ...
    @property
    def ltos(self) -> FVector4: ...
    @property
    def ltot(self) -> FVector4: ...
    @property
    def ltoo(self) -> FVector4: ...
    @property
    def ltol(self) -> FVector4: ...
    @property
    def ltls(self) -> FVector4: ...
    @property
    def ltlt(self) -> FVector4: ...
    @property
    def ltlo(self) -> FVector4: ...
    @property
    def ltll(self) -> FVector4: ...
    @property
    def loss(self) -> FVector4: ...
    @property
    def lost(self) -> FVector4: ...
    @property
    def loso(self) -> FVector4: ...
    @property
    def losl(self) -> FVector4: ...
    @property
    def lots(self) -> FVector4: ...
    @property
    def lott(self) -> FVector4: ...
    @property
    def loto(self) -> FVector4: ...
    @property
    def lotl(self) -> FVector4: ...
    @property
    def loos(self) -> FVector4: ...
    @property
    def loot(self) -> FVector4: ...
    @property
    def lols(self) -> FVector4: ...
    @property
    def lolt(self) -> FVector4: ...
    @property
    def llss(self) -> FVector4: ...
    @property
    def llst(self) -> FVector4: ...
    @property
    def llso(self) -> FVector4: ...
    @property
    def llsl(self) -> FVector4: ...
    @property
    def llts(self) -> FVector4: ...
    @property
    def lltt(self) -> FVector4: ...
    @property
    def llto(self) -> FVector4: ...
    @property
    def lltl(self) -> FVector4: ...
    @property
    def llos(self) -> FVector4: ...
    @property
    def llot(self) -> FVector4: ...
    @property
    def llls(self) -> FVector4: ...
    @property
    def lllt(self) -> FVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def v(self) -> float: ...
    @property
    def uu(self) -> FVector2: ...
    @property
    def uv(self) -> FVector2: ...
    @property
    def uo(self) -> FVector2: ...
    @property
    def ul(self) -> FVector2: ...
    @property
    def vu(self) -> FVector2: ...
    @property
    def vv(self) -> FVector2: ...
    @property
    def vo(self) -> FVector2: ...
    @property
    def vl(self) -> FVector2: ...
    @property
    def ou(self) -> FVector2: ...
    @property
    def ov(self) -> FVector2: ...
    @property
    def lu(self) -> FVector2: ...
    @property
    def lv(self) -> FVector2: ...
    @property
    def uuu(self) -> FVector3: ...
    @property
    def uuv(self) -> FVector3: ...
    @property
    def uuo(self) -> FVector3: ...
    @property
    def uul(self) -> FVector3: ...
    @property
    def uvu(self) -> FVector3: ...
    @property
    def uvv(self) -> FVector3: ...
    @property
    def uvo(self) -> FVector3: ...
    @property
    def uvl(self) -> FVector3: ...
    @property
    def uou(self) -> FVector3: ...
    @property
    def uov(self) -> FVector3: ...
    @property
    def uoo(self) -> FVector3: ...
    @property
    def uol(self) -> FVector3: ...
    @property
    def ulu(self) -> FVector3: ...
    @property
    def ulv(self) -> FVector3: ...
    @property
    def ulo(self) -> FVector3: ...
    @property
    def ull(self) -> FVector3: ...
    @property
    def vuu(self) -> FVector3: ...
    @property
    def vuv(self) -> FVector3: ...
    @property
    def vuo(self) -> FVector3: ...
    @property
    def vul(self) -> FVector3: ...
    @property
    def vvu(self) -> FVector3: ...
    @property
    def vvv(self) -> FVector3: ...
    @property
    def vvo(self) -> FVector3: ...
    @property
    def vvl(self) -> FVector3: ...
    @property
    def vou(self) -> FVector3: ...
    @property
    def vov(self) -> FVector3: ...
    @property
    def voo(self) -> FVector3: ...
    @property
    def vol(self) -> FVector3: ...
    @property
    def vlu(self) -> FVector3: ...
    @property
    def vlv(self) -> FVector3: ...
    @property
    def vlo(self) -> FVector3: ...
    @property
    def vll(self) -> FVector3: ...
    @property
    def ouu(self) -> FVector3: ...
    @property
    def ouv(self) -> FVector3: ...
    @property
    def ouo(self) -> FVector3: ...
    @property
    def oul(self) -> FVector3: ...
    @property
    def ovu(self) -> FVector3: ...
    @property
    def ovv(self) -> FVector3: ...
    @property
    def ovo(self) -> FVector3: ...
    @property
    def ovl(self) -> FVector3: ...
    @property
    def oou(self) -> FVector3: ...
    @property
    def oov(self) -> FVector3: ...
    @property
    def olu(self) -> FVector3: ...
    @property
    def olv(self) -> FVector3: ...
    @property
    def luu(self) -> FVector3: ...
    @property
    def luv(self) -> FVector3: ...
    @property
    def luo(self) -> FVector3: ...
    @property
    def lul(self) -> FVector3: ...
    @property
    def lvu(self) -> FVector3: ...
    @property
    def lvv(self) -> FVector3: ...
    @property
    def lvo(self) -> FVector3: ...
    @property
    def lvl(self) -> FVector3: ...
    @property
    def lou(self) -> FVector3: ...
    @property
    def lov(self) -> FVector3: ...
    @property
    def llu(self) -> FVector3: ...
    @property
    def llv(self) -> FVector3: ...
    @property
    def uuuu(self) -> FVector4: ...
    @property
    def uuuv(self) -> FVector4: ...
    @property
    def uuuo(self) -> FVector4: ...
    @property
    def uuul(self) -> FVector4: ...
    @property
    def uuvu(self) -> FVector4: ...
    @property
    def uuvv(self) -> FVector4: ...
    @property
    def uuvo(self) -> FVector4: ...
    @property
    def uuvl(self) -> FVector4: ...
    @property
    def uuou(self) -> FVector4: ...
    @property
    def uuov(self) -> FVector4: ...
    @property
    def uuoo(self) -> FVector4: ...
    @property
    def uuol(self) -> FVector4: ...
    @property
    def uulu(self) -> FVector4: ...
    @property
    def uulv(self) -> FVector4: ...
    @property
    def uulo(self) -> FVector4: ...
    @property
    def uull(self) -> FVector4: ...
    @property
    def uvuu(self) -> FVector4: ...
    @property
    def uvuv(self) -> FVector4: ...
    @property
    def uvuo(self) -> FVector4: ...
    @property
    def uvul(self) -> FVector4: ...
    @property
    def uvvu(self) -> FVector4: ...
    @property
    def uvvv(self) -> FVector4: ...
    @property
    def uvvo(self) -> FVector4: ...
    @property
    def uvvl(self) -> FVector4: ...
    @property
    def uvou(self) -> FVector4: ...
    @property
    def uvov(self) -> FVector4: ...
    @property
    def uvoo(self) -> FVector4: ...
    @property
    def uvol(self) -> FVector4: ...
    @property
    def uvlu(self) -> FVector4: ...
    @property
    def uvlv(self) -> FVector4: ...
    @property
    def uvlo(self) -> FVector4: ...
    @property
    def uvll(self) -> FVector4: ...
    @property
    def uouu(self) -> FVector4: ...
    @property
    def uouv(self) -> FVector4: ...
    @property
    def uouo(self) -> FVector4: ...
    @property
    def uoul(self) -> FVector4: ...
    @property
    def uovu(self) -> FVector4: ...
    @property
    def uovv(self) -> FVector4: ...
    @property
    def uovo(self) -> FVector4: ...
    @property
    def uovl(self) -> FVector4: ...
    @property
    def uoou(self) -> FVector4: ...
    @property
    def uoov(self) -> FVector4: ...
    @property
    def uooo(self) -> FVector4: ...
    @property
    def uool(self) -> FVector4: ...
    @property
    def uolu(self) -> FVector4: ...
    @property
    def uolv(self) -> FVector4: ...
    @property
    def uolo(self) -> FVector4: ...
    @property
    def uoll(self) -> FVector4: ...
    @property
    def uluu(self) -> FVector4: ...
    @property
    def uluv(self) -> FVector4: ...
    @property
    def uluo(self) -> FVector4: ...
    @property
    def ulul(self) -> FVector4: ...
    @property
    def ulvu(self) -> FVector4: ...
    @property
    def ulvv(self) -> FVector4: ...
    @property
    def ulvo(self) -> FVector4: ...
    @property
    def ulvl(self) -> FVector4: ...
    @property
    def ulou(self) -> FVector4: ...
    @property
    def ulov(self) -> FVector4: ...
    @property
    def uloo(self) -> FVector4: ...
    @property
    def ulol(self) -> FVector4: ...
    @property
    def ullu(self) -> FVector4: ...
    @property
    def ullv(self) -> FVector4: ...
    @property
    def ullo(self) -> FVector4: ...
    @property
    def ulll(self) -> FVector4: ...
    @property
    def vuuu(self) -> FVector4: ...
    @property
    def vuuv(self) -> FVector4: ...
    @property
    def vuuo(self) -> FVector4: ...
    @property
    def vuul(self) -> FVector4: ...
    @property
    def vuvu(self) -> FVector4: ...
    @property
    def vuvv(self) -> FVector4: ...
    @property
    def vuvo(self) -> FVector4: ...
    @property
    def vuvl(self) -> FVector4: ...
    @property
    def vuou(self) -> FVector4: ...
    @property
    def vuov(self) -> FVector4: ...
    @property
    def vuoo(self) -> FVector4: ...
    @property
    def vuol(self) -> FVector4: ...
    @property
    def vulu(self) -> FVector4: ...
    @property
    def vulv(self) -> FVector4: ...
    @property
    def vulo(self) -> FVector4: ...
    @property
    def vull(self) -> FVector4: ...
    @property
    def vvuu(self) -> FVector4: ...
    @property
    def vvuv(self) -> FVector4: ...
    @property
    def vvuo(self) -> FVector4: ...
    @property
    def vvul(self) -> FVector4: ...
    @property
    def vvvu(self) -> FVector4: ...
    @property
    def vvvv(self) -> FVector4: ...
    @property
    def vvvo(self) -> FVector4: ...
    @property
    def vvvl(self) -> FVector4: ...
    @property
    def vvou(self) -> FVector4: ...
    @property
    def vvov(self) -> FVector4: ...
    @property
    def vvoo(self) -> FVector4: ...
    @property
    def vvol(self) -> FVector4: ...
    @property
    def vvlu(self) -> FVector4: ...
    @property
    def vvlv(self) -> FVector4: ...
    @property
    def vvlo(self) -> FVector4: ...
    @property
    def vvll(self) -> FVector4: ...
    @property
    def vouu(self) -> FVector4: ...
    @property
    def vouv(self) -> FVector4: ...
    @property
    def vouo(self) -> FVector4: ...
    @property
    def voul(self) -> FVector4: ...
    @property
    def vovu(self) -> FVector4: ...
    @property
    def vovv(self) -> FVector4: ...
    @property
    def vovo(self) -> FVector4: ...
    @property
    def vovl(self) -> FVector4: ...
    @property
    def voou(self) -> FVector4: ...
    @property
    def voov(self) -> FVector4: ...
    @property
    def vooo(self) -> FVector4: ...
    @property
    def vool(self) -> FVector4: ...
    @property
    def volu(self) -> FVector4: ...
    @property
    def volv(self) -> FVector4: ...
    @property
    def volo(self) -> FVector4: ...
    @property
    def voll(self) -> FVector4: ...
    @property
    def vluu(self) -> FVector4: ...
    @property
    def vluv(self) -> FVector4: ...
    @property
    def vluo(self) -> FVector4: ...
    @property
    def vlul(self) -> FVector4: ...
    @property
    def vlvu(self) -> FVector4: ...
    @property
    def vlvv(self) -> FVector4: ...
    @property
    def vlvo(self) -> FVector4: ...
    @property
    def vlvl(self) -> FVector4: ...
    @property
    def vlou(self) -> FVector4: ...
    @property
    def vlov(self) -> FVector4: ...
    @property
    def vloo(self) -> FVector4: ...
    @property
    def vlol(self) -> FVector4: ...
    @property
    def vllu(self) -> FVector4: ...
    @property
    def vllv(self) -> FVector4: ...
    @property
    def vllo(self) -> FVector4: ...
    @property
    def vlll(self) -> FVector4: ...
    @property
    def ouuu(self) -> FVector4: ...
    @property
    def ouuv(self) -> FVector4: ...
    @property
    def ouuo(self) -> FVector4: ...
    @property
    def ouul(self) -> FVector4: ...
    @property
    def ouvu(self) -> FVector4: ...
    @property
    def ouvv(self) -> FVector4: ...
    @property
    def ouvo(self) -> FVector4: ...
    @property
    def ouvl(self) -> FVector4: ...
    @property
    def ouou(self) -> FVector4: ...
    @property
    def ouov(self) -> FVector4: ...
    @property
    def ouoo(self) -> FVector4: ...
    @property
    def ouol(self) -> FVector4: ...
    @property
    def oulu(self) -> FVector4: ...
    @property
    def oulv(self) -> FVector4: ...
    @property
    def oulo(self) -> FVector4: ...
    @property
    def oull(self) -> FVector4: ...
    @property
    def ovuu(self) -> FVector4: ...
    @property
    def ovuv(self) -> FVector4: ...
    @property
    def ovuo(self) -> FVector4: ...
    @property
    def ovul(self) -> FVector4: ...
    @property
    def ovvu(self) -> FVector4: ...
    @property
    def ovvv(self) -> FVector4: ...
    @property
    def ovvo(self) -> FVector4: ...
    @property
    def ovvl(self) -> FVector4: ...
    @property
    def ovou(self) -> FVector4: ...
    @property
    def ovov(self) -> FVector4: ...
    @property
    def ovoo(self) -> FVector4: ...
    @property
    def ovol(self) -> FVector4: ...
    @property
    def ovlu(self) -> FVector4: ...
    @property
    def ovlv(self) -> FVector4: ...
    @property
    def ovlo(self) -> FVector4: ...
    @property
    def ovll(self) -> FVector4: ...
    @property
    def oouu(self) -> FVector4: ...
    @property
    def oouv(self) -> FVector4: ...
    @property
    def oouo(self) -> FVector4: ...
    @property
    def ooul(self) -> FVector4: ...
    @property
    def oovu(self) -> FVector4: ...
    @property
    def oovv(self) -> FVector4: ...
    @property
    def oovo(self) -> FVector4: ...
    @property
    def oovl(self) -> FVector4: ...
    @property
    def ooou(self) -> FVector4: ...
    @property
    def ooov(self) -> FVector4: ...
    @property
    def oolu(self) -> FVector4: ...
    @property
    def oolv(self) -> FVector4: ...
    @property
    def oluu(self) -> FVector4: ...
    @property
    def oluv(self) -> FVector4: ...
    @property
    def oluo(self) -> FVector4: ...
    @property
    def olul(self) -> FVector4: ...
    @property
    def olvu(self) -> FVector4: ...
    @property
    def olvv(self) -> FVector4: ...
    @property
    def olvo(self) -> FVector4: ...
    @property
    def olvl(self) -> FVector4: ...
    @property
    def olou(self) -> FVector4: ...
    @property
    def olov(self) -> FVector4: ...
    @property
    def ollu(self) -> FVector4: ...
    @property
    def ollv(self) -> FVector4: ...
    @property
    def luuu(self) -> FVector4: ...
    @property
    def luuv(self) -> FVector4: ...
    @property
    def luuo(self) -> FVector4: ...
    @property
    def luul(self) -> FVector4: ...
    @property
    def luvu(self) -> FVector4: ...
    @property
    def luvv(self) -> FVector4: ...
    @property
    def luvo(self) -> FVector4: ...
    @property
    def luvl(self) -> FVector4: ...
    @property
    def luou(self) -> FVector4: ...
    @property
    def luov(self) -> FVector4: ...
    @property
    def luoo(self) -> FVector4: ...
    @property
    def luol(self) -> FVector4: ...
    @property
    def lulu(self) -> FVector4: ...
    @property
    def lulv(self) -> FVector4: ...
    @property
    def lulo(self) -> FVector4: ...
    @property
    def lull(self) -> FVector4: ...
    @property
    def lvuu(self) -> FVector4: ...
    @property
    def lvuv(self) -> FVector4: ...
    @property
    def lvuo(self) -> FVector4: ...
    @property
    def lvul(self) -> FVector4: ...
    @property
    def lvvu(self) -> FVector4: ...
    @property
    def lvvv(self) -> FVector4: ...
    @property
    def lvvo(self) -> FVector4: ...
    @property
    def lvvl(self) -> FVector4: ...
    @property
    def lvou(self) -> FVector4: ...
    @property
    def lvov(self) -> FVector4: ...
    @property
    def lvoo(self) -> FVector4: ...
    @property
    def lvol(self) -> FVector4: ...
    @property
    def lvlu(self) -> FVector4: ...
    @property
    def lvlv(self) -> FVector4: ...
    @property
    def lvlo(self) -> FVector4: ...
    @property
    def lvll(self) -> FVector4: ...
    @property
    def louu(self) -> FVector4: ...
    @property
    def louv(self) -> FVector4: ...
    @property
    def louo(self) -> FVector4: ...
    @property
    def loul(self) -> FVector4: ...
    @property
    def lovu(self) -> FVector4: ...
    @property
    def lovv(self) -> FVector4: ...
    @property
    def lovo(self) -> FVector4: ...
    @property
    def lovl(self) -> FVector4: ...
    @property
    def loou(self) -> FVector4: ...
    @property
    def loov(self) -> FVector4: ...
    @property
    def lolu(self) -> FVector4: ...
    @property
    def lolv(self) -> FVector4: ...
    @property
    def lluu(self) -> FVector4: ...
    @property
    def lluv(self) -> FVector4: ...
    @property
    def lluo(self) -> FVector4: ...
    @property
    def llul(self) -> FVector4: ...
    @property
    def llvu(self) -> FVector4: ...
    @property
    def llvv(self) -> FVector4: ...
    @property
    def llvo(self) -> FVector4: ...
    @property
    def llvl(self) -> FVector4: ...
    @property
    def llou(self) -> FVector4: ...
    @property
    def llov(self) -> FVector4: ...
    @property
    def lllu(self) -> FVector4: ...
    @property
    def lllv(self) -> FVector4: ...
    @property
    def oo(self) -> FVector2: ...
    @property
    def ol(self) -> FVector2: ...
    @property
    def lo(self) -> FVector2: ...
    @property
    def ll(self) -> FVector2: ...
    @property
    def ooo(self) -> FVector3: ...
    @property
    def ool(self) -> FVector3: ...
    @property
    def olo(self) -> FVector3: ...
    @property
    def oll(self) -> FVector3: ...
    @property
    def loo(self) -> FVector3: ...
    @property
    def lol(self) -> FVector3: ...
    @property
    def llo(self) -> FVector3: ...
    @property
    def lll(self) -> FVector3: ...
    @property
    def oooo(self) -> FVector4: ...
    @property
    def oool(self) -> FVector4: ...
    @property
    def oolo(self) -> FVector4: ...
    @property
    def ooll(self) -> FVector4: ...
    @property
    def oloo(self) -> FVector4: ...
    @property
    def olol(self) -> FVector4: ...
    @property
    def ollo(self) -> FVector4: ...
    @property
    def olll(self) -> FVector4: ...
    @property
    def looo(self) -> FVector4: ...
    @property
    def lool(self) -> FVector4: ...
    @property
    def lolo(self) -> FVector4: ...
    @property
    def loll(self) -> FVector4: ...
    @property
    def lloo(self) -> FVector4: ...
    @property
    def llol(self) -> FVector4: ...
    @property
    def lllo(self) -> FVector4: ...
    @property
    def llll(self) -> FVector4: ...
    @property
    def magnitude(self) -> float: ...
    def normalize(self) -> FVector2: ...
    def distance(self, other: FVector2, /) -> float: ...
    def lerp(self, other: FVector2, x: float, /) -> FVector2: ...
    def min(self, n: Number, /) -> FVector2: ...
    def max(self, n: Number, /) -> FVector2: ...
    def clamp(self, min: Number, max: Number, /) -> FVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[FVector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector2: ...

@final
class FVector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: FVector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector2]: ...
    @overload
    def __getitem__(self, index: int) -> FVector2: ...
    @overload
    def __getitem__(self, index: slice) -> FVector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[FVector2]: ...

@final
class I8Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __add__(self, other: Number) -> I8Vector2: ...
    @overload
    def __radd__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __radd__(self, other: Number) -> I8Vector2: ...
    @overload
    def __sub__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __sub__(self, other: Number) -> I8Vector2: ...
    @overload
    def __rsub__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> I8Vector2: ...
    @overload
    def __mul__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __mul__(self, other: Number) -> I8Vector2: ...
    @overload
    def __rmul__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> I8Vector2: ...
    @overload
    def __truediv__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> I8Vector2: ...
    @overload
    def __rtruediv__(self, other: I8Vector2) -> I8Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> I8Vector2: ...
    def __neg__(self) -> I8Vector2: ...
    def __abs__(self) -> I8Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> I8Vector2: ...
    @property
    def xy(self) -> I8Vector2: ...
    @property
    def xo(self) -> I8Vector2: ...
    @property
    def xl(self) -> I8Vector2: ...
    @property
    def yx(self) -> I8Vector2: ...
    @property
    def yy(self) -> I8Vector2: ...
    @property
    def yo(self) -> I8Vector2: ...
    @property
    def yl(self) -> I8Vector2: ...
    @property
    def ox(self) -> I8Vector2: ...
    @property
    def oy(self) -> I8Vector2: ...
    @property
    def lx(self) -> I8Vector2: ...
    @property
    def ly(self) -> I8Vector2: ...
    @property
    def xxx(self) -> I8Vector3: ...
    @property
    def xxy(self) -> I8Vector3: ...
    @property
    def xxo(self) -> I8Vector3: ...
    @property
    def xxl(self) -> I8Vector3: ...
    @property
    def xyx(self) -> I8Vector3: ...
    @property
    def xyy(self) -> I8Vector3: ...
    @property
    def xyo(self) -> I8Vector3: ...
    @property
    def xyl(self) -> I8Vector3: ...
    @property
    def xox(self) -> I8Vector3: ...
    @property
    def xoy(self) -> I8Vector3: ...
    @property
    def xoo(self) -> I8Vector3: ...
    @property
    def xol(self) -> I8Vector3: ...
    @property
    def xlx(self) -> I8Vector3: ...
    @property
    def xly(self) -> I8Vector3: ...
    @property
    def xlo(self) -> I8Vector3: ...
    @property
    def xll(self) -> I8Vector3: ...
    @property
    def yxx(self) -> I8Vector3: ...
    @property
    def yxy(self) -> I8Vector3: ...
    @property
    def yxo(self) -> I8Vector3: ...
    @property
    def yxl(self) -> I8Vector3: ...
    @property
    def yyx(self) -> I8Vector3: ...
    @property
    def yyy(self) -> I8Vector3: ...
    @property
    def yyo(self) -> I8Vector3: ...
    @property
    def yyl(self) -> I8Vector3: ...
    @property
    def yox(self) -> I8Vector3: ...
    @property
    def yoy(self) -> I8Vector3: ...
    @property
    def yoo(self) -> I8Vector3: ...
    @property
    def yol(self) -> I8Vector3: ...
    @property
    def ylx(self) -> I8Vector3: ...
    @property
    def yly(self) -> I8Vector3: ...
    @property
    def ylo(self) -> I8Vector3: ...
    @property
    def yll(self) -> I8Vector3: ...
    @property
    def oxx(self) -> I8Vector3: ...
    @property
    def oxy(self) -> I8Vector3: ...
    @property
    def oxo(self) -> I8Vector3: ...
    @property
    def oxl(self) -> I8Vector3: ...
    @property
    def oyx(self) -> I8Vector3: ...
    @property
    def oyy(self) -> I8Vector3: ...
    @property
    def oyo(self) -> I8Vector3: ...
    @property
    def oyl(self) -> I8Vector3: ...
    @property
    def oox(self) -> I8Vector3: ...
    @property
    def ooy(self) -> I8Vector3: ...
    @property
    def olx(self) -> I8Vector3: ...
    @property
    def oly(self) -> I8Vector3: ...
    @property
    def lxx(self) -> I8Vector3: ...
    @property
    def lxy(self) -> I8Vector3: ...
    @property
    def lxo(self) -> I8Vector3: ...
    @property
    def lxl(self) -> I8Vector3: ...
    @property
    def lyx(self) -> I8Vector3: ...
    @property
    def lyy(self) -> I8Vector3: ...
    @property
    def lyo(self) -> I8Vector3: ...
    @property
    def lyl(self) -> I8Vector3: ...
    @property
    def lox(self) -> I8Vector3: ...
    @property
    def loy(self) -> I8Vector3: ...
    @property
    def llx(self) -> I8Vector3: ...
    @property
    def lly(self) -> I8Vector3: ...
    @property
    def xxxx(self) -> I8Vector4: ...
    @property
    def xxxy(self) -> I8Vector4: ...
    @property
    def xxxo(self) -> I8Vector4: ...
    @property
    def xxxl(self) -> I8Vector4: ...
    @property
    def xxyx(self) -> I8Vector4: ...
    @property
    def xxyy(self) -> I8Vector4: ...
    @property
    def xxyo(self) -> I8Vector4: ...
    @property
    def xxyl(self) -> I8Vector4: ...
    @property
    def xxox(self) -> I8Vector4: ...
    @property
    def xxoy(self) -> I8Vector4: ...
    @property
    def xxoo(self) -> I8Vector4: ...
    @property
    def xxol(self) -> I8Vector4: ...
    @property
    def xxlx(self) -> I8Vector4: ...
    @property
    def xxly(self) -> I8Vector4: ...
    @property
    def xxlo(self) -> I8Vector4: ...
    @property
    def xxll(self) -> I8Vector4: ...
    @property
    def xyxx(self) -> I8Vector4: ...
    @property
    def xyxy(self) -> I8Vector4: ...
    @property
    def xyxo(self) -> I8Vector4: ...
    @property
    def xyxl(self) -> I8Vector4: ...
    @property
    def xyyx(self) -> I8Vector4: ...
    @property
    def xyyy(self) -> I8Vector4: ...
    @property
    def xyyo(self) -> I8Vector4: ...
    @property
    def xyyl(self) -> I8Vector4: ...
    @property
    def xyox(self) -> I8Vector4: ...
    @property
    def xyoy(self) -> I8Vector4: ...
    @property
    def xyoo(self) -> I8Vector4: ...
    @property
    def xyol(self) -> I8Vector4: ...
    @property
    def xylx(self) -> I8Vector4: ...
    @property
    def xyly(self) -> I8Vector4: ...
    @property
    def xylo(self) -> I8Vector4: ...
    @property
    def xyll(self) -> I8Vector4: ...
    @property
    def xoxx(self) -> I8Vector4: ...
    @property
    def xoxy(self) -> I8Vector4: ...
    @property
    def xoxo(self) -> I8Vector4: ...
    @property
    def xoxl(self) -> I8Vector4: ...
    @property
    def xoyx(self) -> I8Vector4: ...
    @property
    def xoyy(self) -> I8Vector4: ...
    @property
    def xoyo(self) -> I8Vector4: ...
    @property
    def xoyl(self) -> I8Vector4: ...
    @property
    def xoox(self) -> I8Vector4: ...
    @property
    def xooy(self) -> I8Vector4: ...
    @property
    def xooo(self) -> I8Vector4: ...
    @property
    def xool(self) -> I8Vector4: ...
    @property
    def xolx(self) -> I8Vector4: ...
    @property
    def xoly(self) -> I8Vector4: ...
    @property
    def xolo(self) -> I8Vector4: ...
    @property
    def xoll(self) -> I8Vector4: ...
    @property
    def xlxx(self) -> I8Vector4: ...
    @property
    def xlxy(self) -> I8Vector4: ...
    @property
    def xlxo(self) -> I8Vector4: ...
    @property
    def xlxl(self) -> I8Vector4: ...
    @property
    def xlyx(self) -> I8Vector4: ...
    @property
    def xlyy(self) -> I8Vector4: ...
    @property
    def xlyo(self) -> I8Vector4: ...
    @property
    def xlyl(self) -> I8Vector4: ...
    @property
    def xlox(self) -> I8Vector4: ...
    @property
    def xloy(self) -> I8Vector4: ...
    @property
    def xloo(self) -> I8Vector4: ...
    @property
    def xlol(self) -> I8Vector4: ...
    @property
    def xllx(self) -> I8Vector4: ...
    @property
    def xlly(self) -> I8Vector4: ...
    @property
    def xllo(self) -> I8Vector4: ...
    @property
    def xlll(self) -> I8Vector4: ...
    @property
    def yxxx(self) -> I8Vector4: ...
    @property
    def yxxy(self) -> I8Vector4: ...
    @property
    def yxxo(self) -> I8Vector4: ...
    @property
    def yxxl(self) -> I8Vector4: ...
    @property
    def yxyx(self) -> I8Vector4: ...
    @property
    def yxyy(self) -> I8Vector4: ...
    @property
    def yxyo(self) -> I8Vector4: ...
    @property
    def yxyl(self) -> I8Vector4: ...
    @property
    def yxox(self) -> I8Vector4: ...
    @property
    def yxoy(self) -> I8Vector4: ...
    @property
    def yxoo(self) -> I8Vector4: ...
    @property
    def yxol(self) -> I8Vector4: ...
    @property
    def yxlx(self) -> I8Vector4: ...
    @property
    def yxly(self) -> I8Vector4: ...
    @property
    def yxlo(self) -> I8Vector4: ...
    @property
    def yxll(self) -> I8Vector4: ...
    @property
    def yyxx(self) -> I8Vector4: ...
    @property
    def yyxy(self) -> I8Vector4: ...
    @property
    def yyxo(self) -> I8Vector4: ...
    @property
    def yyxl(self) -> I8Vector4: ...
    @property
    def yyyx(self) -> I8Vector4: ...
    @property
    def yyyy(self) -> I8Vector4: ...
    @property
    def yyyo(self) -> I8Vector4: ...
    @property
    def yyyl(self) -> I8Vector4: ...
    @property
    def yyox(self) -> I8Vector4: ...
    @property
    def yyoy(self) -> I8Vector4: ...
    @property
    def yyoo(self) -> I8Vector4: ...
    @property
    def yyol(self) -> I8Vector4: ...
    @property
    def yylx(self) -> I8Vector4: ...
    @property
    def yyly(self) -> I8Vector4: ...
    @property
    def yylo(self) -> I8Vector4: ...
    @property
    def yyll(self) -> I8Vector4: ...
    @property
    def yoxx(self) -> I8Vector4: ...
    @property
    def yoxy(self) -> I8Vector4: ...
    @property
    def yoxo(self) -> I8Vector4: ...
    @property
    def yoxl(self) -> I8Vector4: ...
    @property
    def yoyx(self) -> I8Vector4: ...
    @property
    def yoyy(self) -> I8Vector4: ...
    @property
    def yoyo(self) -> I8Vector4: ...
    @property
    def yoyl(self) -> I8Vector4: ...
    @property
    def yoox(self) -> I8Vector4: ...
    @property
    def yooy(self) -> I8Vector4: ...
    @property
    def yooo(self) -> I8Vector4: ...
    @property
    def yool(self) -> I8Vector4: ...
    @property
    def yolx(self) -> I8Vector4: ...
    @property
    def yoly(self) -> I8Vector4: ...
    @property
    def yolo(self) -> I8Vector4: ...
    @property
    def yoll(self) -> I8Vector4: ...
    @property
    def ylxx(self) -> I8Vector4: ...
    @property
    def ylxy(self) -> I8Vector4: ...
    @property
    def ylxo(self) -> I8Vector4: ...
    @property
    def ylxl(self) -> I8Vector4: ...
    @property
    def ylyx(self) -> I8Vector4: ...
    @property
    def ylyy(self) -> I8Vector4: ...
    @property
    def ylyo(self) -> I8Vector4: ...
    @property
    def ylyl(self) -> I8Vector4: ...
    @property
    def ylox(self) -> I8Vector4: ...
    @property
    def yloy(self) -> I8Vector4: ...
    @property
    def yloo(self) -> I8Vector4: ...
    @property
    def ylol(self) -> I8Vector4: ...
    @property
    def yllx(self) -> I8Vector4: ...
    @property
    def ylly(self) -> I8Vector4: ...
    @property
    def yllo(self) -> I8Vector4: ...
    @property
    def ylll(self) -> I8Vector4: ...
    @property
    def oxxx(self) -> I8Vector4: ...
    @property
    def oxxy(self) -> I8Vector4: ...
    @property
    def oxxo(self) -> I8Vector4: ...
    @property
    def oxxl(self) -> I8Vector4: ...
    @property
    def oxyx(self) -> I8Vector4: ...
    @property
    def oxyy(self) -> I8Vector4: ...
    @property
    def oxyo(self) -> I8Vector4: ...
    @property
    def oxyl(self) -> I8Vector4: ...
    @property
    def oxox(self) -> I8Vector4: ...
    @property
    def oxoy(self) -> I8Vector4: ...
    @property
    def oxoo(self) -> I8Vector4: ...
    @property
    def oxol(self) -> I8Vector4: ...
    @property
    def oxlx(self) -> I8Vector4: ...
    @property
    def oxly(self) -> I8Vector4: ...
    @property
    def oxlo(self) -> I8Vector4: ...
    @property
    def oxll(self) -> I8Vector4: ...
    @property
    def oyxx(self) -> I8Vector4: ...
    @property
    def oyxy(self) -> I8Vector4: ...
    @property
    def oyxo(self) -> I8Vector4: ...
    @property
    def oyxl(self) -> I8Vector4: ...
    @property
    def oyyx(self) -> I8Vector4: ...
    @property
    def oyyy(self) -> I8Vector4: ...
    @property
    def oyyo(self) -> I8Vector4: ...
    @property
    def oyyl(self) -> I8Vector4: ...
    @property
    def oyox(self) -> I8Vector4: ...
    @property
    def oyoy(self) -> I8Vector4: ...
    @property
    def oyoo(self) -> I8Vector4: ...
    @property
    def oyol(self) -> I8Vector4: ...
    @property
    def oylx(self) -> I8Vector4: ...
    @property
    def oyly(self) -> I8Vector4: ...
    @property
    def oylo(self) -> I8Vector4: ...
    @property
    def oyll(self) -> I8Vector4: ...
    @property
    def ooxx(self) -> I8Vector4: ...
    @property
    def ooxy(self) -> I8Vector4: ...
    @property
    def ooxo(self) -> I8Vector4: ...
    @property
    def ooxl(self) -> I8Vector4: ...
    @property
    def ooyx(self) -> I8Vector4: ...
    @property
    def ooyy(self) -> I8Vector4: ...
    @property
    def ooyo(self) -> I8Vector4: ...
    @property
    def ooyl(self) -> I8Vector4: ...
    @property
    def ooox(self) -> I8Vector4: ...
    @property
    def oooy(self) -> I8Vector4: ...
    @property
    def oolx(self) -> I8Vector4: ...
    @property
    def ooly(self) -> I8Vector4: ...
    @property
    def olxx(self) -> I8Vector4: ...
    @property
    def olxy(self) -> I8Vector4: ...
    @property
    def olxo(self) -> I8Vector4: ...
    @property
    def olxl(self) -> I8Vector4: ...
    @property
    def olyx(self) -> I8Vector4: ...
    @property
    def olyy(self) -> I8Vector4: ...
    @property
    def olyo(self) -> I8Vector4: ...
    @property
    def olyl(self) -> I8Vector4: ...
    @property
    def olox(self) -> I8Vector4: ...
    @property
    def oloy(self) -> I8Vector4: ...
    @property
    def ollx(self) -> I8Vector4: ...
    @property
    def olly(self) -> I8Vector4: ...
    @property
    def lxxx(self) -> I8Vector4: ...
    @property
    def lxxy(self) -> I8Vector4: ...
    @property
    def lxxo(self) -> I8Vector4: ...
    @property
    def lxxl(self) -> I8Vector4: ...
    @property
    def lxyx(self) -> I8Vector4: ...
    @property
    def lxyy(self) -> I8Vector4: ...
    @property
    def lxyo(self) -> I8Vector4: ...
    @property
    def lxyl(self) -> I8Vector4: ...
    @property
    def lxox(self) -> I8Vector4: ...
    @property
    def lxoy(self) -> I8Vector4: ...
    @property
    def lxoo(self) -> I8Vector4: ...
    @property
    def lxol(self) -> I8Vector4: ...
    @property
    def lxlx(self) -> I8Vector4: ...
    @property
    def lxly(self) -> I8Vector4: ...
    @property
    def lxlo(self) -> I8Vector4: ...
    @property
    def lxll(self) -> I8Vector4: ...
    @property
    def lyxx(self) -> I8Vector4: ...
    @property
    def lyxy(self) -> I8Vector4: ...
    @property
    def lyxo(self) -> I8Vector4: ...
    @property
    def lyxl(self) -> I8Vector4: ...
    @property
    def lyyx(self) -> I8Vector4: ...
    @property
    def lyyy(self) -> I8Vector4: ...
    @property
    def lyyo(self) -> I8Vector4: ...
    @property
    def lyyl(self) -> I8Vector4: ...
    @property
    def lyox(self) -> I8Vector4: ...
    @property
    def lyoy(self) -> I8Vector4: ...
    @property
    def lyoo(self) -> I8Vector4: ...
    @property
    def lyol(self) -> I8Vector4: ...
    @property
    def lylx(self) -> I8Vector4: ...
    @property
    def lyly(self) -> I8Vector4: ...
    @property
    def lylo(self) -> I8Vector4: ...
    @property
    def lyll(self) -> I8Vector4: ...
    @property
    def loxx(self) -> I8Vector4: ...
    @property
    def loxy(self) -> I8Vector4: ...
    @property
    def loxo(self) -> I8Vector4: ...
    @property
    def loxl(self) -> I8Vector4: ...
    @property
    def loyx(self) -> I8Vector4: ...
    @property
    def loyy(self) -> I8Vector4: ...
    @property
    def loyo(self) -> I8Vector4: ...
    @property
    def loyl(self) -> I8Vector4: ...
    @property
    def loox(self) -> I8Vector4: ...
    @property
    def looy(self) -> I8Vector4: ...
    @property
    def lolx(self) -> I8Vector4: ...
    @property
    def loly(self) -> I8Vector4: ...
    @property
    def llxx(self) -> I8Vector4: ...
    @property
    def llxy(self) -> I8Vector4: ...
    @property
    def llxo(self) -> I8Vector4: ...
    @property
    def llxl(self) -> I8Vector4: ...
    @property
    def llyx(self) -> I8Vector4: ...
    @property
    def llyy(self) -> I8Vector4: ...
    @property
    def llyo(self) -> I8Vector4: ...
    @property
    def llyl(self) -> I8Vector4: ...
    @property
    def llox(self) -> I8Vector4: ...
    @property
    def lloy(self) -> I8Vector4: ...
    @property
    def lllx(self) -> I8Vector4: ...
    @property
    def llly(self) -> I8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> I8Vector2: ...
    @property
    def rg(self) -> I8Vector2: ...
    @property
    def ro(self) -> I8Vector2: ...
    @property
    def rl(self) -> I8Vector2: ...
    @property
    def gr(self) -> I8Vector2: ...
    @property
    def gg(self) -> I8Vector2: ...
    @property
    def go(self) -> I8Vector2: ...
    @property
    def gl(self) -> I8Vector2: ...
    @property
    def og(self) -> I8Vector2: ...
    @property
    def lr(self) -> I8Vector2: ...
    @property
    def lg(self) -> I8Vector2: ...
    @property
    def rrr(self) -> I8Vector3: ...
    @property
    def rrg(self) -> I8Vector3: ...
    @property
    def rro(self) -> I8Vector3: ...
    @property
    def rrl(self) -> I8Vector3: ...
    @property
    def rgr(self) -> I8Vector3: ...
    @property
    def rgg(self) -> I8Vector3: ...
    @property
    def rgo(self) -> I8Vector3: ...
    @property
    def rgl(self) -> I8Vector3: ...
    @property
    def ror(self) -> I8Vector3: ...
    @property
    def rog(self) -> I8Vector3: ...
    @property
    def roo(self) -> I8Vector3: ...
    @property
    def rol(self) -> I8Vector3: ...
    @property
    def rlr(self) -> I8Vector3: ...
    @property
    def rlg(self) -> I8Vector3: ...
    @property
    def rlo(self) -> I8Vector3: ...
    @property
    def rll(self) -> I8Vector3: ...
    @property
    def grr(self) -> I8Vector3: ...
    @property
    def grg(self) -> I8Vector3: ...
    @property
    def gro(self) -> I8Vector3: ...
    @property
    def grl(self) -> I8Vector3: ...
    @property
    def ggr(self) -> I8Vector3: ...
    @property
    def ggg(self) -> I8Vector3: ...
    @property
    def ggo(self) -> I8Vector3: ...
    @property
    def ggl(self) -> I8Vector3: ...
    @property
    def gor(self) -> I8Vector3: ...
    @property
    def gog(self) -> I8Vector3: ...
    @property
    def goo(self) -> I8Vector3: ...
    @property
    def gol(self) -> I8Vector3: ...
    @property
    def glr(self) -> I8Vector3: ...
    @property
    def glg(self) -> I8Vector3: ...
    @property
    def glo(self) -> I8Vector3: ...
    @property
    def gll(self) -> I8Vector3: ...
    @property
    def orr(self) -> I8Vector3: ...
    @property
    def org(self) -> I8Vector3: ...
    @property
    def oro(self) -> I8Vector3: ...
    @property
    def orl(self) -> I8Vector3: ...
    @property
    def ogr(self) -> I8Vector3: ...
    @property
    def ogg(self) -> I8Vector3: ...
    @property
    def ogo(self) -> I8Vector3: ...
    @property
    def ogl(self) -> I8Vector3: ...
    @property
    def oor(self) -> I8Vector3: ...
    @property
    def oog(self) -> I8Vector3: ...
    @property
    def olr(self) -> I8Vector3: ...
    @property
    def olg(self) -> I8Vector3: ...
    @property
    def lrr(self) -> I8Vector3: ...
    @property
    def lrg(self) -> I8Vector3: ...
    @property
    def lro(self) -> I8Vector3: ...
    @property
    def lrl(self) -> I8Vector3: ...
    @property
    def lgr(self) -> I8Vector3: ...
    @property
    def lgg(self) -> I8Vector3: ...
    @property
    def lgo(self) -> I8Vector3: ...
    @property
    def lgl(self) -> I8Vector3: ...
    @property
    def lor(self) -> I8Vector3: ...
    @property
    def log(self) -> I8Vector3: ...
    @property
    def llr(self) -> I8Vector3: ...
    @property
    def llg(self) -> I8Vector3: ...
    @property
    def rrrr(self) -> I8Vector4: ...
    @property
    def rrrg(self) -> I8Vector4: ...
    @property
    def rrro(self) -> I8Vector4: ...
    @property
    def rrrl(self) -> I8Vector4: ...
    @property
    def rrgr(self) -> I8Vector4: ...
    @property
    def rrgg(self) -> I8Vector4: ...
    @property
    def rrgo(self) -> I8Vector4: ...
    @property
    def rrgl(self) -> I8Vector4: ...
    @property
    def rror(self) -> I8Vector4: ...
    @property
    def rrog(self) -> I8Vector4: ...
    @property
    def rroo(self) -> I8Vector4: ...
    @property
    def rrol(self) -> I8Vector4: ...
    @property
    def rrlr(self) -> I8Vector4: ...
    @property
    def rrlg(self) -> I8Vector4: ...
    @property
    def rrlo(self) -> I8Vector4: ...
    @property
    def rrll(self) -> I8Vector4: ...
    @property
    def rgrr(self) -> I8Vector4: ...
    @property
    def rgrg(self) -> I8Vector4: ...
    @property
    def rgro(self) -> I8Vector4: ...
    @property
    def rgrl(self) -> I8Vector4: ...
    @property
    def rggr(self) -> I8Vector4: ...
    @property
    def rggg(self) -> I8Vector4: ...
    @property
    def rggo(self) -> I8Vector4: ...
    @property
    def rggl(self) -> I8Vector4: ...
    @property
    def rgor(self) -> I8Vector4: ...
    @property
    def rgog(self) -> I8Vector4: ...
    @property
    def rgoo(self) -> I8Vector4: ...
    @property
    def rgol(self) -> I8Vector4: ...
    @property
    def rglr(self) -> I8Vector4: ...
    @property
    def rglg(self) -> I8Vector4: ...
    @property
    def rglo(self) -> I8Vector4: ...
    @property
    def rgll(self) -> I8Vector4: ...
    @property
    def rorr(self) -> I8Vector4: ...
    @property
    def rorg(self) -> I8Vector4: ...
    @property
    def roro(self) -> I8Vector4: ...
    @property
    def rorl(self) -> I8Vector4: ...
    @property
    def rogr(self) -> I8Vector4: ...
    @property
    def rogg(self) -> I8Vector4: ...
    @property
    def rogo(self) -> I8Vector4: ...
    @property
    def rogl(self) -> I8Vector4: ...
    @property
    def roor(self) -> I8Vector4: ...
    @property
    def roog(self) -> I8Vector4: ...
    @property
    def rooo(self) -> I8Vector4: ...
    @property
    def rool(self) -> I8Vector4: ...
    @property
    def rolr(self) -> I8Vector4: ...
    @property
    def rolg(self) -> I8Vector4: ...
    @property
    def rolo(self) -> I8Vector4: ...
    @property
    def roll(self) -> I8Vector4: ...
    @property
    def rlrr(self) -> I8Vector4: ...
    @property
    def rlrg(self) -> I8Vector4: ...
    @property
    def rlro(self) -> I8Vector4: ...
    @property
    def rlrl(self) -> I8Vector4: ...
    @property
    def rlgr(self) -> I8Vector4: ...
    @property
    def rlgg(self) -> I8Vector4: ...
    @property
    def rlgo(self) -> I8Vector4: ...
    @property
    def rlgl(self) -> I8Vector4: ...
    @property
    def rlor(self) -> I8Vector4: ...
    @property
    def rlog(self) -> I8Vector4: ...
    @property
    def rloo(self) -> I8Vector4: ...
    @property
    def rlol(self) -> I8Vector4: ...
    @property
    def rllr(self) -> I8Vector4: ...
    @property
    def rllg(self) -> I8Vector4: ...
    @property
    def rllo(self) -> I8Vector4: ...
    @property
    def rlll(self) -> I8Vector4: ...
    @property
    def grrr(self) -> I8Vector4: ...
    @property
    def grrg(self) -> I8Vector4: ...
    @property
    def grro(self) -> I8Vector4: ...
    @property
    def grrl(self) -> I8Vector4: ...
    @property
    def grgr(self) -> I8Vector4: ...
    @property
    def grgg(self) -> I8Vector4: ...
    @property
    def grgo(self) -> I8Vector4: ...
    @property
    def grgl(self) -> I8Vector4: ...
    @property
    def gror(self) -> I8Vector4: ...
    @property
    def grog(self) -> I8Vector4: ...
    @property
    def groo(self) -> I8Vector4: ...
    @property
    def grol(self) -> I8Vector4: ...
    @property
    def grlr(self) -> I8Vector4: ...
    @property
    def grlg(self) -> I8Vector4: ...
    @property
    def grlo(self) -> I8Vector4: ...
    @property
    def grll(self) -> I8Vector4: ...
    @property
    def ggrr(self) -> I8Vector4: ...
    @property
    def ggrg(self) -> I8Vector4: ...
    @property
    def ggro(self) -> I8Vector4: ...
    @property
    def ggrl(self) -> I8Vector4: ...
    @property
    def gggr(self) -> I8Vector4: ...
    @property
    def gggg(self) -> I8Vector4: ...
    @property
    def gggo(self) -> I8Vector4: ...
    @property
    def gggl(self) -> I8Vector4: ...
    @property
    def ggor(self) -> I8Vector4: ...
    @property
    def ggog(self) -> I8Vector4: ...
    @property
    def ggoo(self) -> I8Vector4: ...
    @property
    def ggol(self) -> I8Vector4: ...
    @property
    def gglr(self) -> I8Vector4: ...
    @property
    def gglg(self) -> I8Vector4: ...
    @property
    def gglo(self) -> I8Vector4: ...
    @property
    def ggll(self) -> I8Vector4: ...
    @property
    def gorr(self) -> I8Vector4: ...
    @property
    def gorg(self) -> I8Vector4: ...
    @property
    def goro(self) -> I8Vector4: ...
    @property
    def gorl(self) -> I8Vector4: ...
    @property
    def gogr(self) -> I8Vector4: ...
    @property
    def gogg(self) -> I8Vector4: ...
    @property
    def gogo(self) -> I8Vector4: ...
    @property
    def gogl(self) -> I8Vector4: ...
    @property
    def goor(self) -> I8Vector4: ...
    @property
    def goog(self) -> I8Vector4: ...
    @property
    def gooo(self) -> I8Vector4: ...
    @property
    def gool(self) -> I8Vector4: ...
    @property
    def golr(self) -> I8Vector4: ...
    @property
    def golg(self) -> I8Vector4: ...
    @property
    def golo(self) -> I8Vector4: ...
    @property
    def goll(self) -> I8Vector4: ...
    @property
    def glrr(self) -> I8Vector4: ...
    @property
    def glrg(self) -> I8Vector4: ...
    @property
    def glro(self) -> I8Vector4: ...
    @property
    def glrl(self) -> I8Vector4: ...
    @property
    def glgr(self) -> I8Vector4: ...
    @property
    def glgg(self) -> I8Vector4: ...
    @property
    def glgo(self) -> I8Vector4: ...
    @property
    def glgl(self) -> I8Vector4: ...
    @property
    def glor(self) -> I8Vector4: ...
    @property
    def glog(self) -> I8Vector4: ...
    @property
    def gloo(self) -> I8Vector4: ...
    @property
    def glol(self) -> I8Vector4: ...
    @property
    def gllr(self) -> I8Vector4: ...
    @property
    def gllg(self) -> I8Vector4: ...
    @property
    def gllo(self) -> I8Vector4: ...
    @property
    def glll(self) -> I8Vector4: ...
    @property
    def orrr(self) -> I8Vector4: ...
    @property
    def orrg(self) -> I8Vector4: ...
    @property
    def orro(self) -> I8Vector4: ...
    @property
    def orrl(self) -> I8Vector4: ...
    @property
    def orgr(self) -> I8Vector4: ...
    @property
    def orgg(self) -> I8Vector4: ...
    @property
    def orgo(self) -> I8Vector4: ...
    @property
    def orgl(self) -> I8Vector4: ...
    @property
    def oror(self) -> I8Vector4: ...
    @property
    def orog(self) -> I8Vector4: ...
    @property
    def oroo(self) -> I8Vector4: ...
    @property
    def orol(self) -> I8Vector4: ...
    @property
    def orlr(self) -> I8Vector4: ...
    @property
    def orlg(self) -> I8Vector4: ...
    @property
    def orlo(self) -> I8Vector4: ...
    @property
    def orll(self) -> I8Vector4: ...
    @property
    def ogrr(self) -> I8Vector4: ...
    @property
    def ogrg(self) -> I8Vector4: ...
    @property
    def ogro(self) -> I8Vector4: ...
    @property
    def ogrl(self) -> I8Vector4: ...
    @property
    def oggr(self) -> I8Vector4: ...
    @property
    def oggg(self) -> I8Vector4: ...
    @property
    def oggo(self) -> I8Vector4: ...
    @property
    def oggl(self) -> I8Vector4: ...
    @property
    def ogor(self) -> I8Vector4: ...
    @property
    def ogog(self) -> I8Vector4: ...
    @property
    def ogoo(self) -> I8Vector4: ...
    @property
    def ogol(self) -> I8Vector4: ...
    @property
    def oglr(self) -> I8Vector4: ...
    @property
    def oglg(self) -> I8Vector4: ...
    @property
    def oglo(self) -> I8Vector4: ...
    @property
    def ogll(self) -> I8Vector4: ...
    @property
    def oorr(self) -> I8Vector4: ...
    @property
    def oorg(self) -> I8Vector4: ...
    @property
    def ooro(self) -> I8Vector4: ...
    @property
    def oorl(self) -> I8Vector4: ...
    @property
    def oogr(self) -> I8Vector4: ...
    @property
    def oogg(self) -> I8Vector4: ...
    @property
    def oogo(self) -> I8Vector4: ...
    @property
    def oogl(self) -> I8Vector4: ...
    @property
    def ooor(self) -> I8Vector4: ...
    @property
    def ooog(self) -> I8Vector4: ...
    @property
    def oolr(self) -> I8Vector4: ...
    @property
    def oolg(self) -> I8Vector4: ...
    @property
    def olrr(self) -> I8Vector4: ...
    @property
    def olrg(self) -> I8Vector4: ...
    @property
    def olro(self) -> I8Vector4: ...
    @property
    def olrl(self) -> I8Vector4: ...
    @property
    def olgr(self) -> I8Vector4: ...
    @property
    def olgg(self) -> I8Vector4: ...
    @property
    def olgo(self) -> I8Vector4: ...
    @property
    def olgl(self) -> I8Vector4: ...
    @property
    def olor(self) -> I8Vector4: ...
    @property
    def olog(self) -> I8Vector4: ...
    @property
    def ollr(self) -> I8Vector4: ...
    @property
    def ollg(self) -> I8Vector4: ...
    @property
    def lrrr(self) -> I8Vector4: ...
    @property
    def lrrg(self) -> I8Vector4: ...
    @property
    def lrro(self) -> I8Vector4: ...
    @property
    def lrrl(self) -> I8Vector4: ...
    @property
    def lrgr(self) -> I8Vector4: ...
    @property
    def lrgg(self) -> I8Vector4: ...
    @property
    def lrgo(self) -> I8Vector4: ...
    @property
    def lrgl(self) -> I8Vector4: ...
    @property
    def lror(self) -> I8Vector4: ...
    @property
    def lrog(self) -> I8Vector4: ...
    @property
    def lroo(self) -> I8Vector4: ...
    @property
    def lrol(self) -> I8Vector4: ...
    @property
    def lrlr(self) -> I8Vector4: ...
    @property
    def lrlg(self) -> I8Vector4: ...
    @property
    def lrlo(self) -> I8Vector4: ...
    @property
    def lrll(self) -> I8Vector4: ...
    @property
    def lgrr(self) -> I8Vector4: ...
    @property
    def lgrg(self) -> I8Vector4: ...
    @property
    def lgro(self) -> I8Vector4: ...
    @property
    def lgrl(self) -> I8Vector4: ...
    @property
    def lggr(self) -> I8Vector4: ...
    @property
    def lggg(self) -> I8Vector4: ...
    @property
    def lggo(self) -> I8Vector4: ...
    @property
    def lggl(self) -> I8Vector4: ...
    @property
    def lgor(self) -> I8Vector4: ...
    @property
    def lgog(self) -> I8Vector4: ...
    @property
    def lgoo(self) -> I8Vector4: ...
    @property
    def lgol(self) -> I8Vector4: ...
    @property
    def lglr(self) -> I8Vector4: ...
    @property
    def lglg(self) -> I8Vector4: ...
    @property
    def lglo(self) -> I8Vector4: ...
    @property
    def lgll(self) -> I8Vector4: ...
    @property
    def lorr(self) -> I8Vector4: ...
    @property
    def lorg(self) -> I8Vector4: ...
    @property
    def loro(self) -> I8Vector4: ...
    @property
    def lorl(self) -> I8Vector4: ...
    @property
    def logr(self) -> I8Vector4: ...
    @property
    def logg(self) -> I8Vector4: ...
    @property
    def logo(self) -> I8Vector4: ...
    @property
    def logl(self) -> I8Vector4: ...
    @property
    def loor(self) -> I8Vector4: ...
    @property
    def loog(self) -> I8Vector4: ...
    @property
    def lolr(self) -> I8Vector4: ...
    @property
    def lolg(self) -> I8Vector4: ...
    @property
    def llrr(self) -> I8Vector4: ...
    @property
    def llrg(self) -> I8Vector4: ...
    @property
    def llro(self) -> I8Vector4: ...
    @property
    def llrl(self) -> I8Vector4: ...
    @property
    def llgr(self) -> I8Vector4: ...
    @property
    def llgg(self) -> I8Vector4: ...
    @property
    def llgo(self) -> I8Vector4: ...
    @property
    def llgl(self) -> I8Vector4: ...
    @property
    def llor(self) -> I8Vector4: ...
    @property
    def llog(self) -> I8Vector4: ...
    @property
    def lllr(self) -> I8Vector4: ...
    @property
    def lllg(self) -> I8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> I8Vector2: ...
    @property
    def st(self) -> I8Vector2: ...
    @property
    def so(self) -> I8Vector2: ...
    @property
    def sl(self) -> I8Vector2: ...
    @property
    def ts(self) -> I8Vector2: ...
    @property
    def tt(self) -> I8Vector2: ...
    @property
    def to(self) -> I8Vector2: ...
    @property
    def tl(self) -> I8Vector2: ...
    @property
    def os(self) -> I8Vector2: ...
    @property
    def ot(self) -> I8Vector2: ...
    @property
    def ls(self) -> I8Vector2: ...
    @property
    def lt(self) -> I8Vector2: ...
    @property
    def sss(self) -> I8Vector3: ...
    @property
    def sst(self) -> I8Vector3: ...
    @property
    def sso(self) -> I8Vector3: ...
    @property
    def ssl(self) -> I8Vector3: ...
    @property
    def sts(self) -> I8Vector3: ...
    @property
    def stt(self) -> I8Vector3: ...
    @property
    def sto(self) -> I8Vector3: ...
    @property
    def stl(self) -> I8Vector3: ...
    @property
    def sos(self) -> I8Vector3: ...
    @property
    def sot(self) -> I8Vector3: ...
    @property
    def soo(self) -> I8Vector3: ...
    @property
    def sol(self) -> I8Vector3: ...
    @property
    def sls(self) -> I8Vector3: ...
    @property
    def slt(self) -> I8Vector3: ...
    @property
    def slo(self) -> I8Vector3: ...
    @property
    def sll(self) -> I8Vector3: ...
    @property
    def tss(self) -> I8Vector3: ...
    @property
    def tst(self) -> I8Vector3: ...
    @property
    def tso(self) -> I8Vector3: ...
    @property
    def tsl(self) -> I8Vector3: ...
    @property
    def tts(self) -> I8Vector3: ...
    @property
    def ttt(self) -> I8Vector3: ...
    @property
    def tto(self) -> I8Vector3: ...
    @property
    def ttl(self) -> I8Vector3: ...
    @property
    def tos(self) -> I8Vector3: ...
    @property
    def tot(self) -> I8Vector3: ...
    @property
    def too(self) -> I8Vector3: ...
    @property
    def tol(self) -> I8Vector3: ...
    @property
    def tls(self) -> I8Vector3: ...
    @property
    def tlt(self) -> I8Vector3: ...
    @property
    def tlo(self) -> I8Vector3: ...
    @property
    def tll(self) -> I8Vector3: ...
    @property
    def oss(self) -> I8Vector3: ...
    @property
    def ost(self) -> I8Vector3: ...
    @property
    def oso(self) -> I8Vector3: ...
    @property
    def osl(self) -> I8Vector3: ...
    @property
    def ots(self) -> I8Vector3: ...
    @property
    def ott(self) -> I8Vector3: ...
    @property
    def oto(self) -> I8Vector3: ...
    @property
    def otl(self) -> I8Vector3: ...
    @property
    def oos(self) -> I8Vector3: ...
    @property
    def oot(self) -> I8Vector3: ...
    @property
    def ols(self) -> I8Vector3: ...
    @property
    def olt(self) -> I8Vector3: ...
    @property
    def lss(self) -> I8Vector3: ...
    @property
    def lst(self) -> I8Vector3: ...
    @property
    def lso(self) -> I8Vector3: ...
    @property
    def lsl(self) -> I8Vector3: ...
    @property
    def lts(self) -> I8Vector3: ...
    @property
    def ltt(self) -> I8Vector3: ...
    @property
    def lto(self) -> I8Vector3: ...
    @property
    def ltl(self) -> I8Vector3: ...
    @property
    def los(self) -> I8Vector3: ...
    @property
    def lot(self) -> I8Vector3: ...
    @property
    def lls(self) -> I8Vector3: ...
    @property
    def llt(self) -> I8Vector3: ...
    @property
    def ssss(self) -> I8Vector4: ...
    @property
    def ssst(self) -> I8Vector4: ...
    @property
    def ssso(self) -> I8Vector4: ...
    @property
    def sssl(self) -> I8Vector4: ...
    @property
    def ssts(self) -> I8Vector4: ...
    @property
    def sstt(self) -> I8Vector4: ...
    @property
    def ssto(self) -> I8Vector4: ...
    @property
    def sstl(self) -> I8Vector4: ...
    @property
    def ssos(self) -> I8Vector4: ...
    @property
    def ssot(self) -> I8Vector4: ...
    @property
    def ssoo(self) -> I8Vector4: ...
    @property
    def ssol(self) -> I8Vector4: ...
    @property
    def ssls(self) -> I8Vector4: ...
    @property
    def sslt(self) -> I8Vector4: ...
    @property
    def sslo(self) -> I8Vector4: ...
    @property
    def ssll(self) -> I8Vector4: ...
    @property
    def stss(self) -> I8Vector4: ...
    @property
    def stst(self) -> I8Vector4: ...
    @property
    def stso(self) -> I8Vector4: ...
    @property
    def stsl(self) -> I8Vector4: ...
    @property
    def stts(self) -> I8Vector4: ...
    @property
    def sttt(self) -> I8Vector4: ...
    @property
    def stto(self) -> I8Vector4: ...
    @property
    def sttl(self) -> I8Vector4: ...
    @property
    def stos(self) -> I8Vector4: ...
    @property
    def stot(self) -> I8Vector4: ...
    @property
    def stoo(self) -> I8Vector4: ...
    @property
    def stol(self) -> I8Vector4: ...
    @property
    def stls(self) -> I8Vector4: ...
    @property
    def stlt(self) -> I8Vector4: ...
    @property
    def stlo(self) -> I8Vector4: ...
    @property
    def stll(self) -> I8Vector4: ...
    @property
    def soss(self) -> I8Vector4: ...
    @property
    def sost(self) -> I8Vector4: ...
    @property
    def soso(self) -> I8Vector4: ...
    @property
    def sosl(self) -> I8Vector4: ...
    @property
    def sots(self) -> I8Vector4: ...
    @property
    def sott(self) -> I8Vector4: ...
    @property
    def soto(self) -> I8Vector4: ...
    @property
    def sotl(self) -> I8Vector4: ...
    @property
    def soos(self) -> I8Vector4: ...
    @property
    def soot(self) -> I8Vector4: ...
    @property
    def sooo(self) -> I8Vector4: ...
    @property
    def sool(self) -> I8Vector4: ...
    @property
    def sols(self) -> I8Vector4: ...
    @property
    def solt(self) -> I8Vector4: ...
    @property
    def solo(self) -> I8Vector4: ...
    @property
    def soll(self) -> I8Vector4: ...
    @property
    def slss(self) -> I8Vector4: ...
    @property
    def slst(self) -> I8Vector4: ...
    @property
    def slso(self) -> I8Vector4: ...
    @property
    def slsl(self) -> I8Vector4: ...
    @property
    def slts(self) -> I8Vector4: ...
    @property
    def sltt(self) -> I8Vector4: ...
    @property
    def slto(self) -> I8Vector4: ...
    @property
    def sltl(self) -> I8Vector4: ...
    @property
    def slos(self) -> I8Vector4: ...
    @property
    def slot(self) -> I8Vector4: ...
    @property
    def sloo(self) -> I8Vector4: ...
    @property
    def slol(self) -> I8Vector4: ...
    @property
    def slls(self) -> I8Vector4: ...
    @property
    def sllt(self) -> I8Vector4: ...
    @property
    def sllo(self) -> I8Vector4: ...
    @property
    def slll(self) -> I8Vector4: ...
    @property
    def tsss(self) -> I8Vector4: ...
    @property
    def tsst(self) -> I8Vector4: ...
    @property
    def tsso(self) -> I8Vector4: ...
    @property
    def tssl(self) -> I8Vector4: ...
    @property
    def tsts(self) -> I8Vector4: ...
    @property
    def tstt(self) -> I8Vector4: ...
    @property
    def tsto(self) -> I8Vector4: ...
    @property
    def tstl(self) -> I8Vector4: ...
    @property
    def tsos(self) -> I8Vector4: ...
    @property
    def tsot(self) -> I8Vector4: ...
    @property
    def tsoo(self) -> I8Vector4: ...
    @property
    def tsol(self) -> I8Vector4: ...
    @property
    def tsls(self) -> I8Vector4: ...
    @property
    def tslt(self) -> I8Vector4: ...
    @property
    def tslo(self) -> I8Vector4: ...
    @property
    def tsll(self) -> I8Vector4: ...
    @property
    def ttss(self) -> I8Vector4: ...
    @property
    def ttst(self) -> I8Vector4: ...
    @property
    def ttso(self) -> I8Vector4: ...
    @property
    def ttsl(self) -> I8Vector4: ...
    @property
    def ttts(self) -> I8Vector4: ...
    @property
    def tttt(self) -> I8Vector4: ...
    @property
    def ttto(self) -> I8Vector4: ...
    @property
    def tttl(self) -> I8Vector4: ...
    @property
    def ttos(self) -> I8Vector4: ...
    @property
    def ttot(self) -> I8Vector4: ...
    @property
    def ttoo(self) -> I8Vector4: ...
    @property
    def ttol(self) -> I8Vector4: ...
    @property
    def ttls(self) -> I8Vector4: ...
    @property
    def ttlt(self) -> I8Vector4: ...
    @property
    def ttlo(self) -> I8Vector4: ...
    @property
    def ttll(self) -> I8Vector4: ...
    @property
    def toss(self) -> I8Vector4: ...
    @property
    def tost(self) -> I8Vector4: ...
    @property
    def toso(self) -> I8Vector4: ...
    @property
    def tosl(self) -> I8Vector4: ...
    @property
    def tots(self) -> I8Vector4: ...
    @property
    def tott(self) -> I8Vector4: ...
    @property
    def toto(self) -> I8Vector4: ...
    @property
    def totl(self) -> I8Vector4: ...
    @property
    def toos(self) -> I8Vector4: ...
    @property
    def toot(self) -> I8Vector4: ...
    @property
    def tooo(self) -> I8Vector4: ...
    @property
    def tool(self) -> I8Vector4: ...
    @property
    def tols(self) -> I8Vector4: ...
    @property
    def tolt(self) -> I8Vector4: ...
    @property
    def tolo(self) -> I8Vector4: ...
    @property
    def toll(self) -> I8Vector4: ...
    @property
    def tlss(self) -> I8Vector4: ...
    @property
    def tlst(self) -> I8Vector4: ...
    @property
    def tlso(self) -> I8Vector4: ...
    @property
    def tlsl(self) -> I8Vector4: ...
    @property
    def tlts(self) -> I8Vector4: ...
    @property
    def tltt(self) -> I8Vector4: ...
    @property
    def tlto(self) -> I8Vector4: ...
    @property
    def tltl(self) -> I8Vector4: ...
    @property
    def tlos(self) -> I8Vector4: ...
    @property
    def tlot(self) -> I8Vector4: ...
    @property
    def tloo(self) -> I8Vector4: ...
    @property
    def tlol(self) -> I8Vector4: ...
    @property
    def tlls(self) -> I8Vector4: ...
    @property
    def tllt(self) -> I8Vector4: ...
    @property
    def tllo(self) -> I8Vector4: ...
    @property
    def tlll(self) -> I8Vector4: ...
    @property
    def osss(self) -> I8Vector4: ...
    @property
    def osst(self) -> I8Vector4: ...
    @property
    def osso(self) -> I8Vector4: ...
    @property
    def ossl(self) -> I8Vector4: ...
    @property
    def osts(self) -> I8Vector4: ...
    @property
    def ostt(self) -> I8Vector4: ...
    @property
    def osto(self) -> I8Vector4: ...
    @property
    def ostl(self) -> I8Vector4: ...
    @property
    def osos(self) -> I8Vector4: ...
    @property
    def osot(self) -> I8Vector4: ...
    @property
    def osoo(self) -> I8Vector4: ...
    @property
    def osol(self) -> I8Vector4: ...
    @property
    def osls(self) -> I8Vector4: ...
    @property
    def oslt(self) -> I8Vector4: ...
    @property
    def oslo(self) -> I8Vector4: ...
    @property
    def osll(self) -> I8Vector4: ...
    @property
    def otss(self) -> I8Vector4: ...
    @property
    def otst(self) -> I8Vector4: ...
    @property
    def otso(self) -> I8Vector4: ...
    @property
    def otsl(self) -> I8Vector4: ...
    @property
    def otts(self) -> I8Vector4: ...
    @property
    def ottt(self) -> I8Vector4: ...
    @property
    def otto(self) -> I8Vector4: ...
    @property
    def ottl(self) -> I8Vector4: ...
    @property
    def otos(self) -> I8Vector4: ...
    @property
    def otot(self) -> I8Vector4: ...
    @property
    def otoo(self) -> I8Vector4: ...
    @property
    def otol(self) -> I8Vector4: ...
    @property
    def otls(self) -> I8Vector4: ...
    @property
    def otlt(self) -> I8Vector4: ...
    @property
    def otlo(self) -> I8Vector4: ...
    @property
    def otll(self) -> I8Vector4: ...
    @property
    def ooss(self) -> I8Vector4: ...
    @property
    def oost(self) -> I8Vector4: ...
    @property
    def ooso(self) -> I8Vector4: ...
    @property
    def oosl(self) -> I8Vector4: ...
    @property
    def oots(self) -> I8Vector4: ...
    @property
    def oott(self) -> I8Vector4: ...
    @property
    def ooto(self) -> I8Vector4: ...
    @property
    def ootl(self) -> I8Vector4: ...
    @property
    def ooos(self) -> I8Vector4: ...
    @property
    def ooot(self) -> I8Vector4: ...
    @property
    def ools(self) -> I8Vector4: ...
    @property
    def oolt(self) -> I8Vector4: ...
    @property
    def olss(self) -> I8Vector4: ...
    @property
    def olst(self) -> I8Vector4: ...
    @property
    def olso(self) -> I8Vector4: ...
    @property
    def olsl(self) -> I8Vector4: ...
    @property
    def olts(self) -> I8Vector4: ...
    @property
    def oltt(self) -> I8Vector4: ...
    @property
    def olto(self) -> I8Vector4: ...
    @property
    def oltl(self) -> I8Vector4: ...
    @property
    def olos(self) -> I8Vector4: ...
    @property
    def olot(self) -> I8Vector4: ...
    @property
    def olls(self) -> I8Vector4: ...
    @property
    def ollt(self) -> I8Vector4: ...
    @property
    def lsss(self) -> I8Vector4: ...
    @property
    def lsst(self) -> I8Vector4: ...
    @property
    def lsso(self) -> I8Vector4: ...
    @property
    def lssl(self) -> I8Vector4: ...
    @property
    def lsts(self) -> I8Vector4: ...
    @property
    def lstt(self) -> I8Vector4: ...
    @property
    def lsto(self) -> I8Vector4: ...
    @property
    def lstl(self) -> I8Vector4: ...
    @property
    def lsos(self) -> I8Vector4: ...
    @property
    def lsot(self) -> I8Vector4: ...
    @property
    def lsoo(self) -> I8Vector4: ...
    @property
    def lsol(self) -> I8Vector4: ...
    @property
    def lsls(self) -> I8Vector4: ...
    @property
    def lslt(self) -> I8Vector4: ...
    @property
    def lslo(self) -> I8Vector4: ...
    @property
    def lsll(self) -> I8Vector4: ...
    @property
    def ltss(self) -> I8Vector4: ...
    @property
    def ltst(self) -> I8Vector4: ...
    @property
    def ltso(self) -> I8Vector4: ...
    @property
    def ltsl(self) -> I8Vector4: ...
    @property
    def ltts(self) -> I8Vector4: ...
    @property
    def lttt(self) -> I8Vector4: ...
    @property
    def ltto(self) -> I8Vector4: ...
    @property
    def lttl(self) -> I8Vector4: ...
    @property
    def ltos(self) -> I8Vector4: ...
    @property
    def ltot(self) -> I8Vector4: ...
    @property
    def ltoo(self) -> I8Vector4: ...
    @property
    def ltol(self) -> I8Vector4: ...
    @property
    def ltls(self) -> I8Vector4: ...
    @property
    def ltlt(self) -> I8Vector4: ...
    @property
    def ltlo(self) -> I8Vector4: ...
    @property
    def ltll(self) -> I8Vector4: ...
    @property
    def loss(self) -> I8Vector4: ...
    @property
    def lost(self) -> I8Vector4: ...
    @property
    def loso(self) -> I8Vector4: ...
    @property
    def losl(self) -> I8Vector4: ...
    @property
    def lots(self) -> I8Vector4: ...
    @property
    def lott(self) -> I8Vector4: ...
    @property
    def loto(self) -> I8Vector4: ...
    @property
    def lotl(self) -> I8Vector4: ...
    @property
    def loos(self) -> I8Vector4: ...
    @property
    def loot(self) -> I8Vector4: ...
    @property
    def lols(self) -> I8Vector4: ...
    @property
    def lolt(self) -> I8Vector4: ...
    @property
    def llss(self) -> I8Vector4: ...
    @property
    def llst(self) -> I8Vector4: ...
    @property
    def llso(self) -> I8Vector4: ...
    @property
    def llsl(self) -> I8Vector4: ...
    @property
    def llts(self) -> I8Vector4: ...
    @property
    def lltt(self) -> I8Vector4: ...
    @property
    def llto(self) -> I8Vector4: ...
    @property
    def lltl(self) -> I8Vector4: ...
    @property
    def llos(self) -> I8Vector4: ...
    @property
    def llot(self) -> I8Vector4: ...
    @property
    def llls(self) -> I8Vector4: ...
    @property
    def lllt(self) -> I8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I8Vector2: ...
    @property
    def uv(self) -> I8Vector2: ...
    @property
    def uo(self) -> I8Vector2: ...
    @property
    def ul(self) -> I8Vector2: ...
    @property
    def vu(self) -> I8Vector2: ...
    @property
    def vv(self) -> I8Vector2: ...
    @property
    def vo(self) -> I8Vector2: ...
    @property
    def vl(self) -> I8Vector2: ...
    @property
    def ou(self) -> I8Vector2: ...
    @property
    def ov(self) -> I8Vector2: ...
    @property
    def lu(self) -> I8Vector2: ...
    @property
    def lv(self) -> I8Vector2: ...
    @property
    def uuu(self) -> I8Vector3: ...
    @property
    def uuv(self) -> I8Vector3: ...
    @property
    def uuo(self) -> I8Vector3: ...
    @property
    def uul(self) -> I8Vector3: ...
    @property
    def uvu(self) -> I8Vector3: ...
    @property
    def uvv(self) -> I8Vector3: ...
    @property
    def uvo(self) -> I8Vector3: ...
    @property
    def uvl(self) -> I8Vector3: ...
    @property
    def uou(self) -> I8Vector3: ...
    @property
    def uov(self) -> I8Vector3: ...
    @property
    def uoo(self) -> I8Vector3: ...
    @property
    def uol(self) -> I8Vector3: ...
    @property
    def ulu(self) -> I8Vector3: ...
    @property
    def ulv(self) -> I8Vector3: ...
    @property
    def ulo(self) -> I8Vector3: ...
    @property
    def ull(self) -> I8Vector3: ...
    @property
    def vuu(self) -> I8Vector3: ...
    @property
    def vuv(self) -> I8Vector3: ...
    @property
    def vuo(self) -> I8Vector3: ...
    @property
    def vul(self) -> I8Vector3: ...
    @property
    def vvu(self) -> I8Vector3: ...
    @property
    def vvv(self) -> I8Vector3: ...
    @property
    def vvo(self) -> I8Vector3: ...
    @property
    def vvl(self) -> I8Vector3: ...
    @property
    def vou(self) -> I8Vector3: ...
    @property
    def vov(self) -> I8Vector3: ...
    @property
    def voo(self) -> I8Vector3: ...
    @property
    def vol(self) -> I8Vector3: ...
    @property
    def vlu(self) -> I8Vector3: ...
    @property
    def vlv(self) -> I8Vector3: ...
    @property
    def vlo(self) -> I8Vector3: ...
    @property
    def vll(self) -> I8Vector3: ...
    @property
    def ouu(self) -> I8Vector3: ...
    @property
    def ouv(self) -> I8Vector3: ...
    @property
    def ouo(self) -> I8Vector3: ...
    @property
    def oul(self) -> I8Vector3: ...
    @property
    def ovu(self) -> I8Vector3: ...
    @property
    def ovv(self) -> I8Vector3: ...
    @property
    def ovo(self) -> I8Vector3: ...
    @property
    def ovl(self) -> I8Vector3: ...
    @property
    def oou(self) -> I8Vector3: ...
    @property
    def oov(self) -> I8Vector3: ...
    @property
    def olu(self) -> I8Vector3: ...
    @property
    def olv(self) -> I8Vector3: ...
    @property
    def luu(self) -> I8Vector3: ...
    @property
    def luv(self) -> I8Vector3: ...
    @property
    def luo(self) -> I8Vector3: ...
    @property
    def lul(self) -> I8Vector3: ...
    @property
    def lvu(self) -> I8Vector3: ...
    @property
    def lvv(self) -> I8Vector3: ...
    @property
    def lvo(self) -> I8Vector3: ...
    @property
    def lvl(self) -> I8Vector3: ...
    @property
    def lou(self) -> I8Vector3: ...
    @property
    def lov(self) -> I8Vector3: ...
    @property
    def llu(self) -> I8Vector3: ...
    @property
    def llv(self) -> I8Vector3: ...
    @property
    def uuuu(self) -> I8Vector4: ...
    @property
    def uuuv(self) -> I8Vector4: ...
    @property
    def uuuo(self) -> I8Vector4: ...
    @property
    def uuul(self) -> I8Vector4: ...
    @property
    def uuvu(self) -> I8Vector4: ...
    @property
    def uuvv(self) -> I8Vector4: ...
    @property
    def uuvo(self) -> I8Vector4: ...
    @property
    def uuvl(self) -> I8Vector4: ...
    @property
    def uuou(self) -> I8Vector4: ...
    @property
    def uuov(self) -> I8Vector4: ...
    @property
    def uuoo(self) -> I8Vector4: ...
    @property
    def uuol(self) -> I8Vector4: ...
    @property
    def uulu(self) -> I8Vector4: ...
    @property
    def uulv(self) -> I8Vector4: ...
    @property
    def uulo(self) -> I8Vector4: ...
    @property
    def uull(self) -> I8Vector4: ...
    @property
    def uvuu(self) -> I8Vector4: ...
    @property
    def uvuv(self) -> I8Vector4: ...
    @property
    def uvuo(self) -> I8Vector4: ...
    @property
    def uvul(self) -> I8Vector4: ...
    @property
    def uvvu(self) -> I8Vector4: ...
    @property
    def uvvv(self) -> I8Vector4: ...
    @property
    def uvvo(self) -> I8Vector4: ...
    @property
    def uvvl(self) -> I8Vector4: ...
    @property
    def uvou(self) -> I8Vector4: ...
    @property
    def uvov(self) -> I8Vector4: ...
    @property
    def uvoo(self) -> I8Vector4: ...
    @property
    def uvol(self) -> I8Vector4: ...
    @property
    def uvlu(self) -> I8Vector4: ...
    @property
    def uvlv(self) -> I8Vector4: ...
    @property
    def uvlo(self) -> I8Vector4: ...
    @property
    def uvll(self) -> I8Vector4: ...
    @property
    def uouu(self) -> I8Vector4: ...
    @property
    def uouv(self) -> I8Vector4: ...
    @property
    def uouo(self) -> I8Vector4: ...
    @property
    def uoul(self) -> I8Vector4: ...
    @property
    def uovu(self) -> I8Vector4: ...
    @property
    def uovv(self) -> I8Vector4: ...
    @property
    def uovo(self) -> I8Vector4: ...
    @property
    def uovl(self) -> I8Vector4: ...
    @property
    def uoou(self) -> I8Vector4: ...
    @property
    def uoov(self) -> I8Vector4: ...
    @property
    def uooo(self) -> I8Vector4: ...
    @property
    def uool(self) -> I8Vector4: ...
    @property
    def uolu(self) -> I8Vector4: ...
    @property
    def uolv(self) -> I8Vector4: ...
    @property
    def uolo(self) -> I8Vector4: ...
    @property
    def uoll(self) -> I8Vector4: ...
    @property
    def uluu(self) -> I8Vector4: ...
    @property
    def uluv(self) -> I8Vector4: ...
    @property
    def uluo(self) -> I8Vector4: ...
    @property
    def ulul(self) -> I8Vector4: ...
    @property
    def ulvu(self) -> I8Vector4: ...
    @property
    def ulvv(self) -> I8Vector4: ...
    @property
    def ulvo(self) -> I8Vector4: ...
    @property
    def ulvl(self) -> I8Vector4: ...
    @property
    def ulou(self) -> I8Vector4: ...
    @property
    def ulov(self) -> I8Vector4: ...
    @property
    def uloo(self) -> I8Vector4: ...
    @property
    def ulol(self) -> I8Vector4: ...
    @property
    def ullu(self) -> I8Vector4: ...
    @property
    def ullv(self) -> I8Vector4: ...
    @property
    def ullo(self) -> I8Vector4: ...
    @property
    def ulll(self) -> I8Vector4: ...
    @property
    def vuuu(self) -> I8Vector4: ...
    @property
    def vuuv(self) -> I8Vector4: ...
    @property
    def vuuo(self) -> I8Vector4: ...
    @property
    def vuul(self) -> I8Vector4: ...
    @property
    def vuvu(self) -> I8Vector4: ...
    @property
    def vuvv(self) -> I8Vector4: ...
    @property
    def vuvo(self) -> I8Vector4: ...
    @property
    def vuvl(self) -> I8Vector4: ...
    @property
    def vuou(self) -> I8Vector4: ...
    @property
    def vuov(self) -> I8Vector4: ...
    @property
    def vuoo(self) -> I8Vector4: ...
    @property
    def vuol(self) -> I8Vector4: ...
    @property
    def vulu(self) -> I8Vector4: ...
    @property
    def vulv(self) -> I8Vector4: ...
    @property
    def vulo(self) -> I8Vector4: ...
    @property
    def vull(self) -> I8Vector4: ...
    @property
    def vvuu(self) -> I8Vector4: ...
    @property
    def vvuv(self) -> I8Vector4: ...
    @property
    def vvuo(self) -> I8Vector4: ...
    @property
    def vvul(self) -> I8Vector4: ...
    @property
    def vvvu(self) -> I8Vector4: ...
    @property
    def vvvv(self) -> I8Vector4: ...
    @property
    def vvvo(self) -> I8Vector4: ...
    @property
    def vvvl(self) -> I8Vector4: ...
    @property
    def vvou(self) -> I8Vector4: ...
    @property
    def vvov(self) -> I8Vector4: ...
    @property
    def vvoo(self) -> I8Vector4: ...
    @property
    def vvol(self) -> I8Vector4: ...
    @property
    def vvlu(self) -> I8Vector4: ...
    @property
    def vvlv(self) -> I8Vector4: ...
    @property
    def vvlo(self) -> I8Vector4: ...
    @property
    def vvll(self) -> I8Vector4: ...
    @property
    def vouu(self) -> I8Vector4: ...
    @property
    def vouv(self) -> I8Vector4: ...
    @property
    def vouo(self) -> I8Vector4: ...
    @property
    def voul(self) -> I8Vector4: ...
    @property
    def vovu(self) -> I8Vector4: ...
    @property
    def vovv(self) -> I8Vector4: ...
    @property
    def vovo(self) -> I8Vector4: ...
    @property
    def vovl(self) -> I8Vector4: ...
    @property
    def voou(self) -> I8Vector4: ...
    @property
    def voov(self) -> I8Vector4: ...
    @property
    def vooo(self) -> I8Vector4: ...
    @property
    def vool(self) -> I8Vector4: ...
    @property
    def volu(self) -> I8Vector4: ...
    @property
    def volv(self) -> I8Vector4: ...
    @property
    def volo(self) -> I8Vector4: ...
    @property
    def voll(self) -> I8Vector4: ...
    @property
    def vluu(self) -> I8Vector4: ...
    @property
    def vluv(self) -> I8Vector4: ...
    @property
    def vluo(self) -> I8Vector4: ...
    @property
    def vlul(self) -> I8Vector4: ...
    @property
    def vlvu(self) -> I8Vector4: ...
    @property
    def vlvv(self) -> I8Vector4: ...
    @property
    def vlvo(self) -> I8Vector4: ...
    @property
    def vlvl(self) -> I8Vector4: ...
    @property
    def vlou(self) -> I8Vector4: ...
    @property
    def vlov(self) -> I8Vector4: ...
    @property
    def vloo(self) -> I8Vector4: ...
    @property
    def vlol(self) -> I8Vector4: ...
    @property
    def vllu(self) -> I8Vector4: ...
    @property
    def vllv(self) -> I8Vector4: ...
    @property
    def vllo(self) -> I8Vector4: ...
    @property
    def vlll(self) -> I8Vector4: ...
    @property
    def ouuu(self) -> I8Vector4: ...
    @property
    def ouuv(self) -> I8Vector4: ...
    @property
    def ouuo(self) -> I8Vector4: ...
    @property
    def ouul(self) -> I8Vector4: ...
    @property
    def ouvu(self) -> I8Vector4: ...
    @property
    def ouvv(self) -> I8Vector4: ...
    @property
    def ouvo(self) -> I8Vector4: ...
    @property
    def ouvl(self) -> I8Vector4: ...
    @property
    def ouou(self) -> I8Vector4: ...
    @property
    def ouov(self) -> I8Vector4: ...
    @property
    def ouoo(self) -> I8Vector4: ...
    @property
    def ouol(self) -> I8Vector4: ...
    @property
    def oulu(self) -> I8Vector4: ...
    @property
    def oulv(self) -> I8Vector4: ...
    @property
    def oulo(self) -> I8Vector4: ...
    @property
    def oull(self) -> I8Vector4: ...
    @property
    def ovuu(self) -> I8Vector4: ...
    @property
    def ovuv(self) -> I8Vector4: ...
    @property
    def ovuo(self) -> I8Vector4: ...
    @property
    def ovul(self) -> I8Vector4: ...
    @property
    def ovvu(self) -> I8Vector4: ...
    @property
    def ovvv(self) -> I8Vector4: ...
    @property
    def ovvo(self) -> I8Vector4: ...
    @property
    def ovvl(self) -> I8Vector4: ...
    @property
    def ovou(self) -> I8Vector4: ...
    @property
    def ovov(self) -> I8Vector4: ...
    @property
    def ovoo(self) -> I8Vector4: ...
    @property
    def ovol(self) -> I8Vector4: ...
    @property
    def ovlu(self) -> I8Vector4: ...
    @property
    def ovlv(self) -> I8Vector4: ...
    @property
    def ovlo(self) -> I8Vector4: ...
    @property
    def ovll(self) -> I8Vector4: ...
    @property
    def oouu(self) -> I8Vector4: ...
    @property
    def oouv(self) -> I8Vector4: ...
    @property
    def oouo(self) -> I8Vector4: ...
    @property
    def ooul(self) -> I8Vector4: ...
    @property
    def oovu(self) -> I8Vector4: ...
    @property
    def oovv(self) -> I8Vector4: ...
    @property
    def oovo(self) -> I8Vector4: ...
    @property
    def oovl(self) -> I8Vector4: ...
    @property
    def ooou(self) -> I8Vector4: ...
    @property
    def ooov(self) -> I8Vector4: ...
    @property
    def oolu(self) -> I8Vector4: ...
    @property
    def oolv(self) -> I8Vector4: ...
    @property
    def oluu(self) -> I8Vector4: ...
    @property
    def oluv(self) -> I8Vector4: ...
    @property
    def oluo(self) -> I8Vector4: ...
    @property
    def olul(self) -> I8Vector4: ...
    @property
    def olvu(self) -> I8Vector4: ...
    @property
    def olvv(self) -> I8Vector4: ...
    @property
    def olvo(self) -> I8Vector4: ...
    @property
    def olvl(self) -> I8Vector4: ...
    @property
    def olou(self) -> I8Vector4: ...
    @property
    def olov(self) -> I8Vector4: ...
    @property
    def ollu(self) -> I8Vector4: ...
    @property
    def ollv(self) -> I8Vector4: ...
    @property
    def luuu(self) -> I8Vector4: ...
    @property
    def luuv(self) -> I8Vector4: ...
    @property
    def luuo(self) -> I8Vector4: ...
    @property
    def luul(self) -> I8Vector4: ...
    @property
    def luvu(self) -> I8Vector4: ...
    @property
    def luvv(self) -> I8Vector4: ...
    @property
    def luvo(self) -> I8Vector4: ...
    @property
    def luvl(self) -> I8Vector4: ...
    @property
    def luou(self) -> I8Vector4: ...
    @property
    def luov(self) -> I8Vector4: ...
    @property
    def luoo(self) -> I8Vector4: ...
    @property
    def luol(self) -> I8Vector4: ...
    @property
    def lulu(self) -> I8Vector4: ...
    @property
    def lulv(self) -> I8Vector4: ...
    @property
    def lulo(self) -> I8Vector4: ...
    @property
    def lull(self) -> I8Vector4: ...
    @property
    def lvuu(self) -> I8Vector4: ...
    @property
    def lvuv(self) -> I8Vector4: ...
    @property
    def lvuo(self) -> I8Vector4: ...
    @property
    def lvul(self) -> I8Vector4: ...
    @property
    def lvvu(self) -> I8Vector4: ...
    @property
    def lvvv(self) -> I8Vector4: ...
    @property
    def lvvo(self) -> I8Vector4: ...
    @property
    def lvvl(self) -> I8Vector4: ...
    @property
    def lvou(self) -> I8Vector4: ...
    @property
    def lvov(self) -> I8Vector4: ...
    @property
    def lvoo(self) -> I8Vector4: ...
    @property
    def lvol(self) -> I8Vector4: ...
    @property
    def lvlu(self) -> I8Vector4: ...
    @property
    def lvlv(self) -> I8Vector4: ...
    @property
    def lvlo(self) -> I8Vector4: ...
    @property
    def lvll(self) -> I8Vector4: ...
    @property
    def louu(self) -> I8Vector4: ...
    @property
    def louv(self) -> I8Vector4: ...
    @property
    def louo(self) -> I8Vector4: ...
    @property
    def loul(self) -> I8Vector4: ...
    @property
    def lovu(self) -> I8Vector4: ...
    @property
    def lovv(self) -> I8Vector4: ...
    @property
    def lovo(self) -> I8Vector4: ...
    @property
    def lovl(self) -> I8Vector4: ...
    @property
    def loou(self) -> I8Vector4: ...
    @property
    def loov(self) -> I8Vector4: ...
    @property
    def lolu(self) -> I8Vector4: ...
    @property
    def lolv(self) -> I8Vector4: ...
    @property
    def lluu(self) -> I8Vector4: ...
    @property
    def lluv(self) -> I8Vector4: ...
    @property
    def lluo(self) -> I8Vector4: ...
    @property
    def llul(self) -> I8Vector4: ...
    @property
    def llvu(self) -> I8Vector4: ...
    @property
    def llvv(self) -> I8Vector4: ...
    @property
    def llvo(self) -> I8Vector4: ...
    @property
    def llvl(self) -> I8Vector4: ...
    @property
    def llou(self) -> I8Vector4: ...
    @property
    def llov(self) -> I8Vector4: ...
    @property
    def lllu(self) -> I8Vector4: ...
    @property
    def lllv(self) -> I8Vector4: ...
    @property
    def oo(self) -> I8Vector2: ...
    @property
    def ol(self) -> I8Vector2: ...
    @property
    def lo(self) -> I8Vector2: ...
    @property
    def ll(self) -> I8Vector2: ...
    @property
    def ooo(self) -> I8Vector3: ...
    @property
    def ool(self) -> I8Vector3: ...
    @property
    def olo(self) -> I8Vector3: ...
    @property
    def oll(self) -> I8Vector3: ...
    @property
    def loo(self) -> I8Vector3: ...
    @property
    def lol(self) -> I8Vector3: ...
    @property
    def llo(self) -> I8Vector3: ...
    @property
    def lll(self) -> I8Vector3: ...
    @property
    def oooo(self) -> I8Vector4: ...
    @property
    def oool(self) -> I8Vector4: ...
    @property
    def oolo(self) -> I8Vector4: ...
    @property
    def ooll(self) -> I8Vector4: ...
    @property
    def oloo(self) -> I8Vector4: ...
    @property
    def olol(self) -> I8Vector4: ...
    @property
    def ollo(self) -> I8Vector4: ...
    @property
    def olll(self) -> I8Vector4: ...
    @property
    def looo(self) -> I8Vector4: ...
    @property
    def lool(self) -> I8Vector4: ...
    @property
    def lolo(self) -> I8Vector4: ...
    @property
    def loll(self) -> I8Vector4: ...
    @property
    def lloo(self) -> I8Vector4: ...
    @property
    def llol(self) -> I8Vector4: ...
    @property
    def lllo(self) -> I8Vector4: ...
    @property
    def llll(self) -> I8Vector4: ...
    def min(self, n: Number, /) -> I8Vector2: ...
    def max(self, n: Number, /) -> I8Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> I8Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I8Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector2: ...

@final
class I8Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I8Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I8Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> I8Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> I8Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[I8Vector2]: ...

@final
class U8Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __add__(self, other: Number) -> U8Vector2: ...
    @overload
    def __radd__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __radd__(self, other: Number) -> U8Vector2: ...
    @overload
    def __sub__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __sub__(self, other: Number) -> U8Vector2: ...
    @overload
    def __rsub__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> U8Vector2: ...
    @overload
    def __mul__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __mul__(self, other: Number) -> U8Vector2: ...
    @overload
    def __rmul__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> U8Vector2: ...
    @overload
    def __truediv__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> U8Vector2: ...
    @overload
    def __rtruediv__(self, other: U8Vector2) -> U8Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> U8Vector2: ...
    def __abs__(self) -> U8Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> U8Vector2: ...
    @property
    def xy(self) -> U8Vector2: ...
    @property
    def xo(self) -> U8Vector2: ...
    @property
    def xl(self) -> U8Vector2: ...
    @property
    def yx(self) -> U8Vector2: ...
    @property
    def yy(self) -> U8Vector2: ...
    @property
    def yo(self) -> U8Vector2: ...
    @property
    def yl(self) -> U8Vector2: ...
    @property
    def ox(self) -> U8Vector2: ...
    @property
    def oy(self) -> U8Vector2: ...
    @property
    def lx(self) -> U8Vector2: ...
    @property
    def ly(self) -> U8Vector2: ...
    @property
    def xxx(self) -> U8Vector3: ...
    @property
    def xxy(self) -> U8Vector3: ...
    @property
    def xxo(self) -> U8Vector3: ...
    @property
    def xxl(self) -> U8Vector3: ...
    @property
    def xyx(self) -> U8Vector3: ...
    @property
    def xyy(self) -> U8Vector3: ...
    @property
    def xyo(self) -> U8Vector3: ...
    @property
    def xyl(self) -> U8Vector3: ...
    @property
    def xox(self) -> U8Vector3: ...
    @property
    def xoy(self) -> U8Vector3: ...
    @property
    def xoo(self) -> U8Vector3: ...
    @property
    def xol(self) -> U8Vector3: ...
    @property
    def xlx(self) -> U8Vector3: ...
    @property
    def xly(self) -> U8Vector3: ...
    @property
    def xlo(self) -> U8Vector3: ...
    @property
    def xll(self) -> U8Vector3: ...
    @property
    def yxx(self) -> U8Vector3: ...
    @property
    def yxy(self) -> U8Vector3: ...
    @property
    def yxo(self) -> U8Vector3: ...
    @property
    def yxl(self) -> U8Vector3: ...
    @property
    def yyx(self) -> U8Vector3: ...
    @property
    def yyy(self) -> U8Vector3: ...
    @property
    def yyo(self) -> U8Vector3: ...
    @property
    def yyl(self) -> U8Vector3: ...
    @property
    def yox(self) -> U8Vector3: ...
    @property
    def yoy(self) -> U8Vector3: ...
    @property
    def yoo(self) -> U8Vector3: ...
    @property
    def yol(self) -> U8Vector3: ...
    @property
    def ylx(self) -> U8Vector3: ...
    @property
    def yly(self) -> U8Vector3: ...
    @property
    def ylo(self) -> U8Vector3: ...
    @property
    def yll(self) -> U8Vector3: ...
    @property
    def oxx(self) -> U8Vector3: ...
    @property
    def oxy(self) -> U8Vector3: ...
    @property
    def oxo(self) -> U8Vector3: ...
    @property
    def oxl(self) -> U8Vector3: ...
    @property
    def oyx(self) -> U8Vector3: ...
    @property
    def oyy(self) -> U8Vector3: ...
    @property
    def oyo(self) -> U8Vector3: ...
    @property
    def oyl(self) -> U8Vector3: ...
    @property
    def oox(self) -> U8Vector3: ...
    @property
    def ooy(self) -> U8Vector3: ...
    @property
    def olx(self) -> U8Vector3: ...
    @property
    def oly(self) -> U8Vector3: ...
    @property
    def lxx(self) -> U8Vector3: ...
    @property
    def lxy(self) -> U8Vector3: ...
    @property
    def lxo(self) -> U8Vector3: ...
    @property
    def lxl(self) -> U8Vector3: ...
    @property
    def lyx(self) -> U8Vector3: ...
    @property
    def lyy(self) -> U8Vector3: ...
    @property
    def lyo(self) -> U8Vector3: ...
    @property
    def lyl(self) -> U8Vector3: ...
    @property
    def lox(self) -> U8Vector3: ...
    @property
    def loy(self) -> U8Vector3: ...
    @property
    def llx(self) -> U8Vector3: ...
    @property
    def lly(self) -> U8Vector3: ...
    @property
    def xxxx(self) -> U8Vector4: ...
    @property
    def xxxy(self) -> U8Vector4: ...
    @property
    def xxxo(self) -> U8Vector4: ...
    @property
    def xxxl(self) -> U8Vector4: ...
    @property
    def xxyx(self) -> U8Vector4: ...
    @property
    def xxyy(self) -> U8Vector4: ...
    @property
    def xxyo(self) -> U8Vector4: ...
    @property
    def xxyl(self) -> U8Vector4: ...
    @property
    def xxox(self) -> U8Vector4: ...
    @property
    def xxoy(self) -> U8Vector4: ...
    @property
    def xxoo(self) -> U8Vector4: ...
    @property
    def xxol(self) -> U8Vector4: ...
    @property
    def xxlx(self) -> U8Vector4: ...
    @property
    def xxly(self) -> U8Vector4: ...
    @property
    def xxlo(self) -> U8Vector4: ...
    @property
    def xxll(self) -> U8Vector4: ...
    @property
    def xyxx(self) -> U8Vector4: ...
    @property
    def xyxy(self) -> U8Vector4: ...
    @property
    def xyxo(self) -> U8Vector4: ...
    @property
    def xyxl(self) -> U8Vector4: ...
    @property
    def xyyx(self) -> U8Vector4: ...
    @property
    def xyyy(self) -> U8Vector4: ...
    @property
    def xyyo(self) -> U8Vector4: ...
    @property
    def xyyl(self) -> U8Vector4: ...
    @property
    def xyox(self) -> U8Vector4: ...
    @property
    def xyoy(self) -> U8Vector4: ...
    @property
    def xyoo(self) -> U8Vector4: ...
    @property
    def xyol(self) -> U8Vector4: ...
    @property
    def xylx(self) -> U8Vector4: ...
    @property
    def xyly(self) -> U8Vector4: ...
    @property
    def xylo(self) -> U8Vector4: ...
    @property
    def xyll(self) -> U8Vector4: ...
    @property
    def xoxx(self) -> U8Vector4: ...
    @property
    def xoxy(self) -> U8Vector4: ...
    @property
    def xoxo(self) -> U8Vector4: ...
    @property
    def xoxl(self) -> U8Vector4: ...
    @property
    def xoyx(self) -> U8Vector4: ...
    @property
    def xoyy(self) -> U8Vector4: ...
    @property
    def xoyo(self) -> U8Vector4: ...
    @property
    def xoyl(self) -> U8Vector4: ...
    @property
    def xoox(self) -> U8Vector4: ...
    @property
    def xooy(self) -> U8Vector4: ...
    @property
    def xooo(self) -> U8Vector4: ...
    @property
    def xool(self) -> U8Vector4: ...
    @property
    def xolx(self) -> U8Vector4: ...
    @property
    def xoly(self) -> U8Vector4: ...
    @property
    def xolo(self) -> U8Vector4: ...
    @property
    def xoll(self) -> U8Vector4: ...
    @property
    def xlxx(self) -> U8Vector4: ...
    @property
    def xlxy(self) -> U8Vector4: ...
    @property
    def xlxo(self) -> U8Vector4: ...
    @property
    def xlxl(self) -> U8Vector4: ...
    @property
    def xlyx(self) -> U8Vector4: ...
    @property
    def xlyy(self) -> U8Vector4: ...
    @property
    def xlyo(self) -> U8Vector4: ...
    @property
    def xlyl(self) -> U8Vector4: ...
    @property
    def xlox(self) -> U8Vector4: ...
    @property
    def xloy(self) -> U8Vector4: ...
    @property
    def xloo(self) -> U8Vector4: ...
    @property
    def xlol(self) -> U8Vector4: ...
    @property
    def xllx(self) -> U8Vector4: ...
    @property
    def xlly(self) -> U8Vector4: ...
    @property
    def xllo(self) -> U8Vector4: ...
    @property
    def xlll(self) -> U8Vector4: ...
    @property
    def yxxx(self) -> U8Vector4: ...
    @property
    def yxxy(self) -> U8Vector4: ...
    @property
    def yxxo(self) -> U8Vector4: ...
    @property
    def yxxl(self) -> U8Vector4: ...
    @property
    def yxyx(self) -> U8Vector4: ...
    @property
    def yxyy(self) -> U8Vector4: ...
    @property
    def yxyo(self) -> U8Vector4: ...
    @property
    def yxyl(self) -> U8Vector4: ...
    @property
    def yxox(self) -> U8Vector4: ...
    @property
    def yxoy(self) -> U8Vector4: ...
    @property
    def yxoo(self) -> U8Vector4: ...
    @property
    def yxol(self) -> U8Vector4: ...
    @property
    def yxlx(self) -> U8Vector4: ...
    @property
    def yxly(self) -> U8Vector4: ...
    @property
    def yxlo(self) -> U8Vector4: ...
    @property
    def yxll(self) -> U8Vector4: ...
    @property
    def yyxx(self) -> U8Vector4: ...
    @property
    def yyxy(self) -> U8Vector4: ...
    @property
    def yyxo(self) -> U8Vector4: ...
    @property
    def yyxl(self) -> U8Vector4: ...
    @property
    def yyyx(self) -> U8Vector4: ...
    @property
    def yyyy(self) -> U8Vector4: ...
    @property
    def yyyo(self) -> U8Vector4: ...
    @property
    def yyyl(self) -> U8Vector4: ...
    @property
    def yyox(self) -> U8Vector4: ...
    @property
    def yyoy(self) -> U8Vector4: ...
    @property
    def yyoo(self) -> U8Vector4: ...
    @property
    def yyol(self) -> U8Vector4: ...
    @property
    def yylx(self) -> U8Vector4: ...
    @property
    def yyly(self) -> U8Vector4: ...
    @property
    def yylo(self) -> U8Vector4: ...
    @property
    def yyll(self) -> U8Vector4: ...
    @property
    def yoxx(self) -> U8Vector4: ...
    @property
    def yoxy(self) -> U8Vector4: ...
    @property
    def yoxo(self) -> U8Vector4: ...
    @property
    def yoxl(self) -> U8Vector4: ...
    @property
    def yoyx(self) -> U8Vector4: ...
    @property
    def yoyy(self) -> U8Vector4: ...
    @property
    def yoyo(self) -> U8Vector4: ...
    @property
    def yoyl(self) -> U8Vector4: ...
    @property
    def yoox(self) -> U8Vector4: ...
    @property
    def yooy(self) -> U8Vector4: ...
    @property
    def yooo(self) -> U8Vector4: ...
    @property
    def yool(self) -> U8Vector4: ...
    @property
    def yolx(self) -> U8Vector4: ...
    @property
    def yoly(self) -> U8Vector4: ...
    @property
    def yolo(self) -> U8Vector4: ...
    @property
    def yoll(self) -> U8Vector4: ...
    @property
    def ylxx(self) -> U8Vector4: ...
    @property
    def ylxy(self) -> U8Vector4: ...
    @property
    def ylxo(self) -> U8Vector4: ...
    @property
    def ylxl(self) -> U8Vector4: ...
    @property
    def ylyx(self) -> U8Vector4: ...
    @property
    def ylyy(self) -> U8Vector4: ...
    @property
    def ylyo(self) -> U8Vector4: ...
    @property
    def ylyl(self) -> U8Vector4: ...
    @property
    def ylox(self) -> U8Vector4: ...
    @property
    def yloy(self) -> U8Vector4: ...
    @property
    def yloo(self) -> U8Vector4: ...
    @property
    def ylol(self) -> U8Vector4: ...
    @property
    def yllx(self) -> U8Vector4: ...
    @property
    def ylly(self) -> U8Vector4: ...
    @property
    def yllo(self) -> U8Vector4: ...
    @property
    def ylll(self) -> U8Vector4: ...
    @property
    def oxxx(self) -> U8Vector4: ...
    @property
    def oxxy(self) -> U8Vector4: ...
    @property
    def oxxo(self) -> U8Vector4: ...
    @property
    def oxxl(self) -> U8Vector4: ...
    @property
    def oxyx(self) -> U8Vector4: ...
    @property
    def oxyy(self) -> U8Vector4: ...
    @property
    def oxyo(self) -> U8Vector4: ...
    @property
    def oxyl(self) -> U8Vector4: ...
    @property
    def oxox(self) -> U8Vector4: ...
    @property
    def oxoy(self) -> U8Vector4: ...
    @property
    def oxoo(self) -> U8Vector4: ...
    @property
    def oxol(self) -> U8Vector4: ...
    @property
    def oxlx(self) -> U8Vector4: ...
    @property
    def oxly(self) -> U8Vector4: ...
    @property
    def oxlo(self) -> U8Vector4: ...
    @property
    def oxll(self) -> U8Vector4: ...
    @property
    def oyxx(self) -> U8Vector4: ...
    @property
    def oyxy(self) -> U8Vector4: ...
    @property
    def oyxo(self) -> U8Vector4: ...
    @property
    def oyxl(self) -> U8Vector4: ...
    @property
    def oyyx(self) -> U8Vector4: ...
    @property
    def oyyy(self) -> U8Vector4: ...
    @property
    def oyyo(self) -> U8Vector4: ...
    @property
    def oyyl(self) -> U8Vector4: ...
    @property
    def oyox(self) -> U8Vector4: ...
    @property
    def oyoy(self) -> U8Vector4: ...
    @property
    def oyoo(self) -> U8Vector4: ...
    @property
    def oyol(self) -> U8Vector4: ...
    @property
    def oylx(self) -> U8Vector4: ...
    @property
    def oyly(self) -> U8Vector4: ...
    @property
    def oylo(self) -> U8Vector4: ...
    @property
    def oyll(self) -> U8Vector4: ...
    @property
    def ooxx(self) -> U8Vector4: ...
    @property
    def ooxy(self) -> U8Vector4: ...
    @property
    def ooxo(self) -> U8Vector4: ...
    @property
    def ooxl(self) -> U8Vector4: ...
    @property
    def ooyx(self) -> U8Vector4: ...
    @property
    def ooyy(self) -> U8Vector4: ...
    @property
    def ooyo(self) -> U8Vector4: ...
    @property
    def ooyl(self) -> U8Vector4: ...
    @property
    def ooox(self) -> U8Vector4: ...
    @property
    def oooy(self) -> U8Vector4: ...
    @property
    def oolx(self) -> U8Vector4: ...
    @property
    def ooly(self) -> U8Vector4: ...
    @property
    def olxx(self) -> U8Vector4: ...
    @property
    def olxy(self) -> U8Vector4: ...
    @property
    def olxo(self) -> U8Vector4: ...
    @property
    def olxl(self) -> U8Vector4: ...
    @property
    def olyx(self) -> U8Vector4: ...
    @property
    def olyy(self) -> U8Vector4: ...
    @property
    def olyo(self) -> U8Vector4: ...
    @property
    def olyl(self) -> U8Vector4: ...
    @property
    def olox(self) -> U8Vector4: ...
    @property
    def oloy(self) -> U8Vector4: ...
    @property
    def ollx(self) -> U8Vector4: ...
    @property
    def olly(self) -> U8Vector4: ...
    @property
    def lxxx(self) -> U8Vector4: ...
    @property
    def lxxy(self) -> U8Vector4: ...
    @property
    def lxxo(self) -> U8Vector4: ...
    @property
    def lxxl(self) -> U8Vector4: ...
    @property
    def lxyx(self) -> U8Vector4: ...
    @property
    def lxyy(self) -> U8Vector4: ...
    @property
    def lxyo(self) -> U8Vector4: ...
    @property
    def lxyl(self) -> U8Vector4: ...
    @property
    def lxox(self) -> U8Vector4: ...
    @property
    def lxoy(self) -> U8Vector4: ...
    @property
    def lxoo(self) -> U8Vector4: ...
    @property
    def lxol(self) -> U8Vector4: ...
    @property
    def lxlx(self) -> U8Vector4: ...
    @property
    def lxly(self) -> U8Vector4: ...
    @property
    def lxlo(self) -> U8Vector4: ...
    @property
    def lxll(self) -> U8Vector4: ...
    @property
    def lyxx(self) -> U8Vector4: ...
    @property
    def lyxy(self) -> U8Vector4: ...
    @property
    def lyxo(self) -> U8Vector4: ...
    @property
    def lyxl(self) -> U8Vector4: ...
    @property
    def lyyx(self) -> U8Vector4: ...
    @property
    def lyyy(self) -> U8Vector4: ...
    @property
    def lyyo(self) -> U8Vector4: ...
    @property
    def lyyl(self) -> U8Vector4: ...
    @property
    def lyox(self) -> U8Vector4: ...
    @property
    def lyoy(self) -> U8Vector4: ...
    @property
    def lyoo(self) -> U8Vector4: ...
    @property
    def lyol(self) -> U8Vector4: ...
    @property
    def lylx(self) -> U8Vector4: ...
    @property
    def lyly(self) -> U8Vector4: ...
    @property
    def lylo(self) -> U8Vector4: ...
    @property
    def lyll(self) -> U8Vector4: ...
    @property
    def loxx(self) -> U8Vector4: ...
    @property
    def loxy(self) -> U8Vector4: ...
    @property
    def loxo(self) -> U8Vector4: ...
    @property
    def loxl(self) -> U8Vector4: ...
    @property
    def loyx(self) -> U8Vector4: ...
    @property
    def loyy(self) -> U8Vector4: ...
    @property
    def loyo(self) -> U8Vector4: ...
    @property
    def loyl(self) -> U8Vector4: ...
    @property
    def loox(self) -> U8Vector4: ...
    @property
    def looy(self) -> U8Vector4: ...
    @property
    def lolx(self) -> U8Vector4: ...
    @property
    def loly(self) -> U8Vector4: ...
    @property
    def llxx(self) -> U8Vector4: ...
    @property
    def llxy(self) -> U8Vector4: ...
    @property
    def llxo(self) -> U8Vector4: ...
    @property
    def llxl(self) -> U8Vector4: ...
    @property
    def llyx(self) -> U8Vector4: ...
    @property
    def llyy(self) -> U8Vector4: ...
    @property
    def llyo(self) -> U8Vector4: ...
    @property
    def llyl(self) -> U8Vector4: ...
    @property
    def llox(self) -> U8Vector4: ...
    @property
    def lloy(self) -> U8Vector4: ...
    @property
    def lllx(self) -> U8Vector4: ...
    @property
    def llly(self) -> U8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> U8Vector2: ...
    @property
    def rg(self) -> U8Vector2: ...
    @property
    def ro(self) -> U8Vector2: ...
    @property
    def rl(self) -> U8Vector2: ...
    @property
    def gr(self) -> U8Vector2: ...
    @property
    def gg(self) -> U8Vector2: ...
    @property
    def go(self) -> U8Vector2: ...
    @property
    def gl(self) -> U8Vector2: ...
    @property
    def og(self) -> U8Vector2: ...
    @property
    def lr(self) -> U8Vector2: ...
    @property
    def lg(self) -> U8Vector2: ...
    @property
    def rrr(self) -> U8Vector3: ...
    @property
    def rrg(self) -> U8Vector3: ...
    @property
    def rro(self) -> U8Vector3: ...
    @property
    def rrl(self) -> U8Vector3: ...
    @property
    def rgr(self) -> U8Vector3: ...
    @property
    def rgg(self) -> U8Vector3: ...
    @property
    def rgo(self) -> U8Vector3: ...
    @property
    def rgl(self) -> U8Vector3: ...
    @property
    def ror(self) -> U8Vector3: ...
    @property
    def rog(self) -> U8Vector3: ...
    @property
    def roo(self) -> U8Vector3: ...
    @property
    def rol(self) -> U8Vector3: ...
    @property
    def rlr(self) -> U8Vector3: ...
    @property
    def rlg(self) -> U8Vector3: ...
    @property
    def rlo(self) -> U8Vector3: ...
    @property
    def rll(self) -> U8Vector3: ...
    @property
    def grr(self) -> U8Vector3: ...
    @property
    def grg(self) -> U8Vector3: ...
    @property
    def gro(self) -> U8Vector3: ...
    @property
    def grl(self) -> U8Vector3: ...
    @property
    def ggr(self) -> U8Vector3: ...
    @property
    def ggg(self) -> U8Vector3: ...
    @property
    def ggo(self) -> U8Vector3: ...
    @property
    def ggl(self) -> U8Vector3: ...
    @property
    def gor(self) -> U8Vector3: ...
    @property
    def gog(self) -> U8Vector3: ...
    @property
    def goo(self) -> U8Vector3: ...
    @property
    def gol(self) -> U8Vector3: ...
    @property
    def glr(self) -> U8Vector3: ...
    @property
    def glg(self) -> U8Vector3: ...
    @property
    def glo(self) -> U8Vector3: ...
    @property
    def gll(self) -> U8Vector3: ...
    @property
    def orr(self) -> U8Vector3: ...
    @property
    def org(self) -> U8Vector3: ...
    @property
    def oro(self) -> U8Vector3: ...
    @property
    def orl(self) -> U8Vector3: ...
    @property
    def ogr(self) -> U8Vector3: ...
    @property
    def ogg(self) -> U8Vector3: ...
    @property
    def ogo(self) -> U8Vector3: ...
    @property
    def ogl(self) -> U8Vector3: ...
    @property
    def oor(self) -> U8Vector3: ...
    @property
    def oog(self) -> U8Vector3: ...
    @property
    def olr(self) -> U8Vector3: ...
    @property
    def olg(self) -> U8Vector3: ...
    @property
    def lrr(self) -> U8Vector3: ...
    @property
    def lrg(self) -> U8Vector3: ...
    @property
    def lro(self) -> U8Vector3: ...
    @property
    def lrl(self) -> U8Vector3: ...
    @property
    def lgr(self) -> U8Vector3: ...
    @property
    def lgg(self) -> U8Vector3: ...
    @property
    def lgo(self) -> U8Vector3: ...
    @property
    def lgl(self) -> U8Vector3: ...
    @property
    def lor(self) -> U8Vector3: ...
    @property
    def log(self) -> U8Vector3: ...
    @property
    def llr(self) -> U8Vector3: ...
    @property
    def llg(self) -> U8Vector3: ...
    @property
    def rrrr(self) -> U8Vector4: ...
    @property
    def rrrg(self) -> U8Vector4: ...
    @property
    def rrro(self) -> U8Vector4: ...
    @property
    def rrrl(self) -> U8Vector4: ...
    @property
    def rrgr(self) -> U8Vector4: ...
    @property
    def rrgg(self) -> U8Vector4: ...
    @property
    def rrgo(self) -> U8Vector4: ...
    @property
    def rrgl(self) -> U8Vector4: ...
    @property
    def rror(self) -> U8Vector4: ...
    @property
    def rrog(self) -> U8Vector4: ...
    @property
    def rroo(self) -> U8Vector4: ...
    @property
    def rrol(self) -> U8Vector4: ...
    @property
    def rrlr(self) -> U8Vector4: ...
    @property
    def rrlg(self) -> U8Vector4: ...
    @property
    def rrlo(self) -> U8Vector4: ...
    @property
    def rrll(self) -> U8Vector4: ...
    @property
    def rgrr(self) -> U8Vector4: ...
    @property
    def rgrg(self) -> U8Vector4: ...
    @property
    def rgro(self) -> U8Vector4: ...
    @property
    def rgrl(self) -> U8Vector4: ...
    @property
    def rggr(self) -> U8Vector4: ...
    @property
    def rggg(self) -> U8Vector4: ...
    @property
    def rggo(self) -> U8Vector4: ...
    @property
    def rggl(self) -> U8Vector4: ...
    @property
    def rgor(self) -> U8Vector4: ...
    @property
    def rgog(self) -> U8Vector4: ...
    @property
    def rgoo(self) -> U8Vector4: ...
    @property
    def rgol(self) -> U8Vector4: ...
    @property
    def rglr(self) -> U8Vector4: ...
    @property
    def rglg(self) -> U8Vector4: ...
    @property
    def rglo(self) -> U8Vector4: ...
    @property
    def rgll(self) -> U8Vector4: ...
    @property
    def rorr(self) -> U8Vector4: ...
    @property
    def rorg(self) -> U8Vector4: ...
    @property
    def roro(self) -> U8Vector4: ...
    @property
    def rorl(self) -> U8Vector4: ...
    @property
    def rogr(self) -> U8Vector4: ...
    @property
    def rogg(self) -> U8Vector4: ...
    @property
    def rogo(self) -> U8Vector4: ...
    @property
    def rogl(self) -> U8Vector4: ...
    @property
    def roor(self) -> U8Vector4: ...
    @property
    def roog(self) -> U8Vector4: ...
    @property
    def rooo(self) -> U8Vector4: ...
    @property
    def rool(self) -> U8Vector4: ...
    @property
    def rolr(self) -> U8Vector4: ...
    @property
    def rolg(self) -> U8Vector4: ...
    @property
    def rolo(self) -> U8Vector4: ...
    @property
    def roll(self) -> U8Vector4: ...
    @property
    def rlrr(self) -> U8Vector4: ...
    @property
    def rlrg(self) -> U8Vector4: ...
    @property
    def rlro(self) -> U8Vector4: ...
    @property
    def rlrl(self) -> U8Vector4: ...
    @property
    def rlgr(self) -> U8Vector4: ...
    @property
    def rlgg(self) -> U8Vector4: ...
    @property
    def rlgo(self) -> U8Vector4: ...
    @property
    def rlgl(self) -> U8Vector4: ...
    @property
    def rlor(self) -> U8Vector4: ...
    @property
    def rlog(self) -> U8Vector4: ...
    @property
    def rloo(self) -> U8Vector4: ...
    @property
    def rlol(self) -> U8Vector4: ...
    @property
    def rllr(self) -> U8Vector4: ...
    @property
    def rllg(self) -> U8Vector4: ...
    @property
    def rllo(self) -> U8Vector4: ...
    @property
    def rlll(self) -> U8Vector4: ...
    @property
    def grrr(self) -> U8Vector4: ...
    @property
    def grrg(self) -> U8Vector4: ...
    @property
    def grro(self) -> U8Vector4: ...
    @property
    def grrl(self) -> U8Vector4: ...
    @property
    def grgr(self) -> U8Vector4: ...
    @property
    def grgg(self) -> U8Vector4: ...
    @property
    def grgo(self) -> U8Vector4: ...
    @property
    def grgl(self) -> U8Vector4: ...
    @property
    def gror(self) -> U8Vector4: ...
    @property
    def grog(self) -> U8Vector4: ...
    @property
    def groo(self) -> U8Vector4: ...
    @property
    def grol(self) -> U8Vector4: ...
    @property
    def grlr(self) -> U8Vector4: ...
    @property
    def grlg(self) -> U8Vector4: ...
    @property
    def grlo(self) -> U8Vector4: ...
    @property
    def grll(self) -> U8Vector4: ...
    @property
    def ggrr(self) -> U8Vector4: ...
    @property
    def ggrg(self) -> U8Vector4: ...
    @property
    def ggro(self) -> U8Vector4: ...
    @property
    def ggrl(self) -> U8Vector4: ...
    @property
    def gggr(self) -> U8Vector4: ...
    @property
    def gggg(self) -> U8Vector4: ...
    @property
    def gggo(self) -> U8Vector4: ...
    @property
    def gggl(self) -> U8Vector4: ...
    @property
    def ggor(self) -> U8Vector4: ...
    @property
    def ggog(self) -> U8Vector4: ...
    @property
    def ggoo(self) -> U8Vector4: ...
    @property
    def ggol(self) -> U8Vector4: ...
    @property
    def gglr(self) -> U8Vector4: ...
    @property
    def gglg(self) -> U8Vector4: ...
    @property
    def gglo(self) -> U8Vector4: ...
    @property
    def ggll(self) -> U8Vector4: ...
    @property
    def gorr(self) -> U8Vector4: ...
    @property
    def gorg(self) -> U8Vector4: ...
    @property
    def goro(self) -> U8Vector4: ...
    @property
    def gorl(self) -> U8Vector4: ...
    @property
    def gogr(self) -> U8Vector4: ...
    @property
    def gogg(self) -> U8Vector4: ...
    @property
    def gogo(self) -> U8Vector4: ...
    @property
    def gogl(self) -> U8Vector4: ...
    @property
    def goor(self) -> U8Vector4: ...
    @property
    def goog(self) -> U8Vector4: ...
    @property
    def gooo(self) -> U8Vector4: ...
    @property
    def gool(self) -> U8Vector4: ...
    @property
    def golr(self) -> U8Vector4: ...
    @property
    def golg(self) -> U8Vector4: ...
    @property
    def golo(self) -> U8Vector4: ...
    @property
    def goll(self) -> U8Vector4: ...
    @property
    def glrr(self) -> U8Vector4: ...
    @property
    def glrg(self) -> U8Vector4: ...
    @property
    def glro(self) -> U8Vector4: ...
    @property
    def glrl(self) -> U8Vector4: ...
    @property
    def glgr(self) -> U8Vector4: ...
    @property
    def glgg(self) -> U8Vector4: ...
    @property
    def glgo(self) -> U8Vector4: ...
    @property
    def glgl(self) -> U8Vector4: ...
    @property
    def glor(self) -> U8Vector4: ...
    @property
    def glog(self) -> U8Vector4: ...
    @property
    def gloo(self) -> U8Vector4: ...
    @property
    def glol(self) -> U8Vector4: ...
    @property
    def gllr(self) -> U8Vector4: ...
    @property
    def gllg(self) -> U8Vector4: ...
    @property
    def gllo(self) -> U8Vector4: ...
    @property
    def glll(self) -> U8Vector4: ...
    @property
    def orrr(self) -> U8Vector4: ...
    @property
    def orrg(self) -> U8Vector4: ...
    @property
    def orro(self) -> U8Vector4: ...
    @property
    def orrl(self) -> U8Vector4: ...
    @property
    def orgr(self) -> U8Vector4: ...
    @property
    def orgg(self) -> U8Vector4: ...
    @property
    def orgo(self) -> U8Vector4: ...
    @property
    def orgl(self) -> U8Vector4: ...
    @property
    def oror(self) -> U8Vector4: ...
    @property
    def orog(self) -> U8Vector4: ...
    @property
    def oroo(self) -> U8Vector4: ...
    @property
    def orol(self) -> U8Vector4: ...
    @property
    def orlr(self) -> U8Vector4: ...
    @property
    def orlg(self) -> U8Vector4: ...
    @property
    def orlo(self) -> U8Vector4: ...
    @property
    def orll(self) -> U8Vector4: ...
    @property
    def ogrr(self) -> U8Vector4: ...
    @property
    def ogrg(self) -> U8Vector4: ...
    @property
    def ogro(self) -> U8Vector4: ...
    @property
    def ogrl(self) -> U8Vector4: ...
    @property
    def oggr(self) -> U8Vector4: ...
    @property
    def oggg(self) -> U8Vector4: ...
    @property
    def oggo(self) -> U8Vector4: ...
    @property
    def oggl(self) -> U8Vector4: ...
    @property
    def ogor(self) -> U8Vector4: ...
    @property
    def ogog(self) -> U8Vector4: ...
    @property
    def ogoo(self) -> U8Vector4: ...
    @property
    def ogol(self) -> U8Vector4: ...
    @property
    def oglr(self) -> U8Vector4: ...
    @property
    def oglg(self) -> U8Vector4: ...
    @property
    def oglo(self) -> U8Vector4: ...
    @property
    def ogll(self) -> U8Vector4: ...
    @property
    def oorr(self) -> U8Vector4: ...
    @property
    def oorg(self) -> U8Vector4: ...
    @property
    def ooro(self) -> U8Vector4: ...
    @property
    def oorl(self) -> U8Vector4: ...
    @property
    def oogr(self) -> U8Vector4: ...
    @property
    def oogg(self) -> U8Vector4: ...
    @property
    def oogo(self) -> U8Vector4: ...
    @property
    def oogl(self) -> U8Vector4: ...
    @property
    def ooor(self) -> U8Vector4: ...
    @property
    def ooog(self) -> U8Vector4: ...
    @property
    def oolr(self) -> U8Vector4: ...
    @property
    def oolg(self) -> U8Vector4: ...
    @property
    def olrr(self) -> U8Vector4: ...
    @property
    def olrg(self) -> U8Vector4: ...
    @property
    def olro(self) -> U8Vector4: ...
    @property
    def olrl(self) -> U8Vector4: ...
    @property
    def olgr(self) -> U8Vector4: ...
    @property
    def olgg(self) -> U8Vector4: ...
    @property
    def olgo(self) -> U8Vector4: ...
    @property
    def olgl(self) -> U8Vector4: ...
    @property
    def olor(self) -> U8Vector4: ...
    @property
    def olog(self) -> U8Vector4: ...
    @property
    def ollr(self) -> U8Vector4: ...
    @property
    def ollg(self) -> U8Vector4: ...
    @property
    def lrrr(self) -> U8Vector4: ...
    @property
    def lrrg(self) -> U8Vector4: ...
    @property
    def lrro(self) -> U8Vector4: ...
    @property
    def lrrl(self) -> U8Vector4: ...
    @property
    def lrgr(self) -> U8Vector4: ...
    @property
    def lrgg(self) -> U8Vector4: ...
    @property
    def lrgo(self) -> U8Vector4: ...
    @property
    def lrgl(self) -> U8Vector4: ...
    @property
    def lror(self) -> U8Vector4: ...
    @property
    def lrog(self) -> U8Vector4: ...
    @property
    def lroo(self) -> U8Vector4: ...
    @property
    def lrol(self) -> U8Vector4: ...
    @property
    def lrlr(self) -> U8Vector4: ...
    @property
    def lrlg(self) -> U8Vector4: ...
    @property
    def lrlo(self) -> U8Vector4: ...
    @property
    def lrll(self) -> U8Vector4: ...
    @property
    def lgrr(self) -> U8Vector4: ...
    @property
    def lgrg(self) -> U8Vector4: ...
    @property
    def lgro(self) -> U8Vector4: ...
    @property
    def lgrl(self) -> U8Vector4: ...
    @property
    def lggr(self) -> U8Vector4: ...
    @property
    def lggg(self) -> U8Vector4: ...
    @property
    def lggo(self) -> U8Vector4: ...
    @property
    def lggl(self) -> U8Vector4: ...
    @property
    def lgor(self) -> U8Vector4: ...
    @property
    def lgog(self) -> U8Vector4: ...
    @property
    def lgoo(self) -> U8Vector4: ...
    @property
    def lgol(self) -> U8Vector4: ...
    @property
    def lglr(self) -> U8Vector4: ...
    @property
    def lglg(self) -> U8Vector4: ...
    @property
    def lglo(self) -> U8Vector4: ...
    @property
    def lgll(self) -> U8Vector4: ...
    @property
    def lorr(self) -> U8Vector4: ...
    @property
    def lorg(self) -> U8Vector4: ...
    @property
    def loro(self) -> U8Vector4: ...
    @property
    def lorl(self) -> U8Vector4: ...
    @property
    def logr(self) -> U8Vector4: ...
    @property
    def logg(self) -> U8Vector4: ...
    @property
    def logo(self) -> U8Vector4: ...
    @property
    def logl(self) -> U8Vector4: ...
    @property
    def loor(self) -> U8Vector4: ...
    @property
    def loog(self) -> U8Vector4: ...
    @property
    def lolr(self) -> U8Vector4: ...
    @property
    def lolg(self) -> U8Vector4: ...
    @property
    def llrr(self) -> U8Vector4: ...
    @property
    def llrg(self) -> U8Vector4: ...
    @property
    def llro(self) -> U8Vector4: ...
    @property
    def llrl(self) -> U8Vector4: ...
    @property
    def llgr(self) -> U8Vector4: ...
    @property
    def llgg(self) -> U8Vector4: ...
    @property
    def llgo(self) -> U8Vector4: ...
    @property
    def llgl(self) -> U8Vector4: ...
    @property
    def llor(self) -> U8Vector4: ...
    @property
    def llog(self) -> U8Vector4: ...
    @property
    def lllr(self) -> U8Vector4: ...
    @property
    def lllg(self) -> U8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> U8Vector2: ...
    @property
    def st(self) -> U8Vector2: ...
    @property
    def so(self) -> U8Vector2: ...
    @property
    def sl(self) -> U8Vector2: ...
    @property
    def ts(self) -> U8Vector2: ...
    @property
    def tt(self) -> U8Vector2: ...
    @property
    def to(self) -> U8Vector2: ...
    @property
    def tl(self) -> U8Vector2: ...
    @property
    def os(self) -> U8Vector2: ...
    @property
    def ot(self) -> U8Vector2: ...
    @property
    def ls(self) -> U8Vector2: ...
    @property
    def lt(self) -> U8Vector2: ...
    @property
    def sss(self) -> U8Vector3: ...
    @property
    def sst(self) -> U8Vector3: ...
    @property
    def sso(self) -> U8Vector3: ...
    @property
    def ssl(self) -> U8Vector3: ...
    @property
    def sts(self) -> U8Vector3: ...
    @property
    def stt(self) -> U8Vector3: ...
    @property
    def sto(self) -> U8Vector3: ...
    @property
    def stl(self) -> U8Vector3: ...
    @property
    def sos(self) -> U8Vector3: ...
    @property
    def sot(self) -> U8Vector3: ...
    @property
    def soo(self) -> U8Vector3: ...
    @property
    def sol(self) -> U8Vector3: ...
    @property
    def sls(self) -> U8Vector3: ...
    @property
    def slt(self) -> U8Vector3: ...
    @property
    def slo(self) -> U8Vector3: ...
    @property
    def sll(self) -> U8Vector3: ...
    @property
    def tss(self) -> U8Vector3: ...
    @property
    def tst(self) -> U8Vector3: ...
    @property
    def tso(self) -> U8Vector3: ...
    @property
    def tsl(self) -> U8Vector3: ...
    @property
    def tts(self) -> U8Vector3: ...
    @property
    def ttt(self) -> U8Vector3: ...
    @property
    def tto(self) -> U8Vector3: ...
    @property
    def ttl(self) -> U8Vector3: ...
    @property
    def tos(self) -> U8Vector3: ...
    @property
    def tot(self) -> U8Vector3: ...
    @property
    def too(self) -> U8Vector3: ...
    @property
    def tol(self) -> U8Vector3: ...
    @property
    def tls(self) -> U8Vector3: ...
    @property
    def tlt(self) -> U8Vector3: ...
    @property
    def tlo(self) -> U8Vector3: ...
    @property
    def tll(self) -> U8Vector3: ...
    @property
    def oss(self) -> U8Vector3: ...
    @property
    def ost(self) -> U8Vector3: ...
    @property
    def oso(self) -> U8Vector3: ...
    @property
    def osl(self) -> U8Vector3: ...
    @property
    def ots(self) -> U8Vector3: ...
    @property
    def ott(self) -> U8Vector3: ...
    @property
    def oto(self) -> U8Vector3: ...
    @property
    def otl(self) -> U8Vector3: ...
    @property
    def oos(self) -> U8Vector3: ...
    @property
    def oot(self) -> U8Vector3: ...
    @property
    def ols(self) -> U8Vector3: ...
    @property
    def olt(self) -> U8Vector3: ...
    @property
    def lss(self) -> U8Vector3: ...
    @property
    def lst(self) -> U8Vector3: ...
    @property
    def lso(self) -> U8Vector3: ...
    @property
    def lsl(self) -> U8Vector3: ...
    @property
    def lts(self) -> U8Vector3: ...
    @property
    def ltt(self) -> U8Vector3: ...
    @property
    def lto(self) -> U8Vector3: ...
    @property
    def ltl(self) -> U8Vector3: ...
    @property
    def los(self) -> U8Vector3: ...
    @property
    def lot(self) -> U8Vector3: ...
    @property
    def lls(self) -> U8Vector3: ...
    @property
    def llt(self) -> U8Vector3: ...
    @property
    def ssss(self) -> U8Vector4: ...
    @property
    def ssst(self) -> U8Vector4: ...
    @property
    def ssso(self) -> U8Vector4: ...
    @property
    def sssl(self) -> U8Vector4: ...
    @property
    def ssts(self) -> U8Vector4: ...
    @property
    def sstt(self) -> U8Vector4: ...
    @property
    def ssto(self) -> U8Vector4: ...
    @property
    def sstl(self) -> U8Vector4: ...
    @property
    def ssos(self) -> U8Vector4: ...
    @property
    def ssot(self) -> U8Vector4: ...
    @property
    def ssoo(self) -> U8Vector4: ...
    @property
    def ssol(self) -> U8Vector4: ...
    @property
    def ssls(self) -> U8Vector4: ...
    @property
    def sslt(self) -> U8Vector4: ...
    @property
    def sslo(self) -> U8Vector4: ...
    @property
    def ssll(self) -> U8Vector4: ...
    @property
    def stss(self) -> U8Vector4: ...
    @property
    def stst(self) -> U8Vector4: ...
    @property
    def stso(self) -> U8Vector4: ...
    @property
    def stsl(self) -> U8Vector4: ...
    @property
    def stts(self) -> U8Vector4: ...
    @property
    def sttt(self) -> U8Vector4: ...
    @property
    def stto(self) -> U8Vector4: ...
    @property
    def sttl(self) -> U8Vector4: ...
    @property
    def stos(self) -> U8Vector4: ...
    @property
    def stot(self) -> U8Vector4: ...
    @property
    def stoo(self) -> U8Vector4: ...
    @property
    def stol(self) -> U8Vector4: ...
    @property
    def stls(self) -> U8Vector4: ...
    @property
    def stlt(self) -> U8Vector4: ...
    @property
    def stlo(self) -> U8Vector4: ...
    @property
    def stll(self) -> U8Vector4: ...
    @property
    def soss(self) -> U8Vector4: ...
    @property
    def sost(self) -> U8Vector4: ...
    @property
    def soso(self) -> U8Vector4: ...
    @property
    def sosl(self) -> U8Vector4: ...
    @property
    def sots(self) -> U8Vector4: ...
    @property
    def sott(self) -> U8Vector4: ...
    @property
    def soto(self) -> U8Vector4: ...
    @property
    def sotl(self) -> U8Vector4: ...
    @property
    def soos(self) -> U8Vector4: ...
    @property
    def soot(self) -> U8Vector4: ...
    @property
    def sooo(self) -> U8Vector4: ...
    @property
    def sool(self) -> U8Vector4: ...
    @property
    def sols(self) -> U8Vector4: ...
    @property
    def solt(self) -> U8Vector4: ...
    @property
    def solo(self) -> U8Vector4: ...
    @property
    def soll(self) -> U8Vector4: ...
    @property
    def slss(self) -> U8Vector4: ...
    @property
    def slst(self) -> U8Vector4: ...
    @property
    def slso(self) -> U8Vector4: ...
    @property
    def slsl(self) -> U8Vector4: ...
    @property
    def slts(self) -> U8Vector4: ...
    @property
    def sltt(self) -> U8Vector4: ...
    @property
    def slto(self) -> U8Vector4: ...
    @property
    def sltl(self) -> U8Vector4: ...
    @property
    def slos(self) -> U8Vector4: ...
    @property
    def slot(self) -> U8Vector4: ...
    @property
    def sloo(self) -> U8Vector4: ...
    @property
    def slol(self) -> U8Vector4: ...
    @property
    def slls(self) -> U8Vector4: ...
    @property
    def sllt(self) -> U8Vector4: ...
    @property
    def sllo(self) -> U8Vector4: ...
    @property
    def slll(self) -> U8Vector4: ...
    @property
    def tsss(self) -> U8Vector4: ...
    @property
    def tsst(self) -> U8Vector4: ...
    @property
    def tsso(self) -> U8Vector4: ...
    @property
    def tssl(self) -> U8Vector4: ...
    @property
    def tsts(self) -> U8Vector4: ...
    @property
    def tstt(self) -> U8Vector4: ...
    @property
    def tsto(self) -> U8Vector4: ...
    @property
    def tstl(self) -> U8Vector4: ...
    @property
    def tsos(self) -> U8Vector4: ...
    @property
    def tsot(self) -> U8Vector4: ...
    @property
    def tsoo(self) -> U8Vector4: ...
    @property
    def tsol(self) -> U8Vector4: ...
    @property
    def tsls(self) -> U8Vector4: ...
    @property
    def tslt(self) -> U8Vector4: ...
    @property
    def tslo(self) -> U8Vector4: ...
    @property
    def tsll(self) -> U8Vector4: ...
    @property
    def ttss(self) -> U8Vector4: ...
    @property
    def ttst(self) -> U8Vector4: ...
    @property
    def ttso(self) -> U8Vector4: ...
    @property
    def ttsl(self) -> U8Vector4: ...
    @property
    def ttts(self) -> U8Vector4: ...
    @property
    def tttt(self) -> U8Vector4: ...
    @property
    def ttto(self) -> U8Vector4: ...
    @property
    def tttl(self) -> U8Vector4: ...
    @property
    def ttos(self) -> U8Vector4: ...
    @property
    def ttot(self) -> U8Vector4: ...
    @property
    def ttoo(self) -> U8Vector4: ...
    @property
    def ttol(self) -> U8Vector4: ...
    @property
    def ttls(self) -> U8Vector4: ...
    @property
    def ttlt(self) -> U8Vector4: ...
    @property
    def ttlo(self) -> U8Vector4: ...
    @property
    def ttll(self) -> U8Vector4: ...
    @property
    def toss(self) -> U8Vector4: ...
    @property
    def tost(self) -> U8Vector4: ...
    @property
    def toso(self) -> U8Vector4: ...
    @property
    def tosl(self) -> U8Vector4: ...
    @property
    def tots(self) -> U8Vector4: ...
    @property
    def tott(self) -> U8Vector4: ...
    @property
    def toto(self) -> U8Vector4: ...
    @property
    def totl(self) -> U8Vector4: ...
    @property
    def toos(self) -> U8Vector4: ...
    @property
    def toot(self) -> U8Vector4: ...
    @property
    def tooo(self) -> U8Vector4: ...
    @property
    def tool(self) -> U8Vector4: ...
    @property
    def tols(self) -> U8Vector4: ...
    @property
    def tolt(self) -> U8Vector4: ...
    @property
    def tolo(self) -> U8Vector4: ...
    @property
    def toll(self) -> U8Vector4: ...
    @property
    def tlss(self) -> U8Vector4: ...
    @property
    def tlst(self) -> U8Vector4: ...
    @property
    def tlso(self) -> U8Vector4: ...
    @property
    def tlsl(self) -> U8Vector4: ...
    @property
    def tlts(self) -> U8Vector4: ...
    @property
    def tltt(self) -> U8Vector4: ...
    @property
    def tlto(self) -> U8Vector4: ...
    @property
    def tltl(self) -> U8Vector4: ...
    @property
    def tlos(self) -> U8Vector4: ...
    @property
    def tlot(self) -> U8Vector4: ...
    @property
    def tloo(self) -> U8Vector4: ...
    @property
    def tlol(self) -> U8Vector4: ...
    @property
    def tlls(self) -> U8Vector4: ...
    @property
    def tllt(self) -> U8Vector4: ...
    @property
    def tllo(self) -> U8Vector4: ...
    @property
    def tlll(self) -> U8Vector4: ...
    @property
    def osss(self) -> U8Vector4: ...
    @property
    def osst(self) -> U8Vector4: ...
    @property
    def osso(self) -> U8Vector4: ...
    @property
    def ossl(self) -> U8Vector4: ...
    @property
    def osts(self) -> U8Vector4: ...
    @property
    def ostt(self) -> U8Vector4: ...
    @property
    def osto(self) -> U8Vector4: ...
    @property
    def ostl(self) -> U8Vector4: ...
    @property
    def osos(self) -> U8Vector4: ...
    @property
    def osot(self) -> U8Vector4: ...
    @property
    def osoo(self) -> U8Vector4: ...
    @property
    def osol(self) -> U8Vector4: ...
    @property
    def osls(self) -> U8Vector4: ...
    @property
    def oslt(self) -> U8Vector4: ...
    @property
    def oslo(self) -> U8Vector4: ...
    @property
    def osll(self) -> U8Vector4: ...
    @property
    def otss(self) -> U8Vector4: ...
    @property
    def otst(self) -> U8Vector4: ...
    @property
    def otso(self) -> U8Vector4: ...
    @property
    def otsl(self) -> U8Vector4: ...
    @property
    def otts(self) -> U8Vector4: ...
    @property
    def ottt(self) -> U8Vector4: ...
    @property
    def otto(self) -> U8Vector4: ...
    @property
    def ottl(self) -> U8Vector4: ...
    @property
    def otos(self) -> U8Vector4: ...
    @property
    def otot(self) -> U8Vector4: ...
    @property
    def otoo(self) -> U8Vector4: ...
    @property
    def otol(self) -> U8Vector4: ...
    @property
    def otls(self) -> U8Vector4: ...
    @property
    def otlt(self) -> U8Vector4: ...
    @property
    def otlo(self) -> U8Vector4: ...
    @property
    def otll(self) -> U8Vector4: ...
    @property
    def ooss(self) -> U8Vector4: ...
    @property
    def oost(self) -> U8Vector4: ...
    @property
    def ooso(self) -> U8Vector4: ...
    @property
    def oosl(self) -> U8Vector4: ...
    @property
    def oots(self) -> U8Vector4: ...
    @property
    def oott(self) -> U8Vector4: ...
    @property
    def ooto(self) -> U8Vector4: ...
    @property
    def ootl(self) -> U8Vector4: ...
    @property
    def ooos(self) -> U8Vector4: ...
    @property
    def ooot(self) -> U8Vector4: ...
    @property
    def ools(self) -> U8Vector4: ...
    @property
    def oolt(self) -> U8Vector4: ...
    @property
    def olss(self) -> U8Vector4: ...
    @property
    def olst(self) -> U8Vector4: ...
    @property
    def olso(self) -> U8Vector4: ...
    @property
    def olsl(self) -> U8Vector4: ...
    @property
    def olts(self) -> U8Vector4: ...
    @property
    def oltt(self) -> U8Vector4: ...
    @property
    def olto(self) -> U8Vector4: ...
    @property
    def oltl(self) -> U8Vector4: ...
    @property
    def olos(self) -> U8Vector4: ...
    @property
    def olot(self) -> U8Vector4: ...
    @property
    def olls(self) -> U8Vector4: ...
    @property
    def ollt(self) -> U8Vector4: ...
    @property
    def lsss(self) -> U8Vector4: ...
    @property
    def lsst(self) -> U8Vector4: ...
    @property
    def lsso(self) -> U8Vector4: ...
    @property
    def lssl(self) -> U8Vector4: ...
    @property
    def lsts(self) -> U8Vector4: ...
    @property
    def lstt(self) -> U8Vector4: ...
    @property
    def lsto(self) -> U8Vector4: ...
    @property
    def lstl(self) -> U8Vector4: ...
    @property
    def lsos(self) -> U8Vector4: ...
    @property
    def lsot(self) -> U8Vector4: ...
    @property
    def lsoo(self) -> U8Vector4: ...
    @property
    def lsol(self) -> U8Vector4: ...
    @property
    def lsls(self) -> U8Vector4: ...
    @property
    def lslt(self) -> U8Vector4: ...
    @property
    def lslo(self) -> U8Vector4: ...
    @property
    def lsll(self) -> U8Vector4: ...
    @property
    def ltss(self) -> U8Vector4: ...
    @property
    def ltst(self) -> U8Vector4: ...
    @property
    def ltso(self) -> U8Vector4: ...
    @property
    def ltsl(self) -> U8Vector4: ...
    @property
    def ltts(self) -> U8Vector4: ...
    @property
    def lttt(self) -> U8Vector4: ...
    @property
    def ltto(self) -> U8Vector4: ...
    @property
    def lttl(self) -> U8Vector4: ...
    @property
    def ltos(self) -> U8Vector4: ...
    @property
    def ltot(self) -> U8Vector4: ...
    @property
    def ltoo(self) -> U8Vector4: ...
    @property
    def ltol(self) -> U8Vector4: ...
    @property
    def ltls(self) -> U8Vector4: ...
    @property
    def ltlt(self) -> U8Vector4: ...
    @property
    def ltlo(self) -> U8Vector4: ...
    @property
    def ltll(self) -> U8Vector4: ...
    @property
    def loss(self) -> U8Vector4: ...
    @property
    def lost(self) -> U8Vector4: ...
    @property
    def loso(self) -> U8Vector4: ...
    @property
    def losl(self) -> U8Vector4: ...
    @property
    def lots(self) -> U8Vector4: ...
    @property
    def lott(self) -> U8Vector4: ...
    @property
    def loto(self) -> U8Vector4: ...
    @property
    def lotl(self) -> U8Vector4: ...
    @property
    def loos(self) -> U8Vector4: ...
    @property
    def loot(self) -> U8Vector4: ...
    @property
    def lols(self) -> U8Vector4: ...
    @property
    def lolt(self) -> U8Vector4: ...
    @property
    def llss(self) -> U8Vector4: ...
    @property
    def llst(self) -> U8Vector4: ...
    @property
    def llso(self) -> U8Vector4: ...
    @property
    def llsl(self) -> U8Vector4: ...
    @property
    def llts(self) -> U8Vector4: ...
    @property
    def lltt(self) -> U8Vector4: ...
    @property
    def llto(self) -> U8Vector4: ...
    @property
    def lltl(self) -> U8Vector4: ...
    @property
    def llos(self) -> U8Vector4: ...
    @property
    def llot(self) -> U8Vector4: ...
    @property
    def llls(self) -> U8Vector4: ...
    @property
    def lllt(self) -> U8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U8Vector2: ...
    @property
    def uv(self) -> U8Vector2: ...
    @property
    def uo(self) -> U8Vector2: ...
    @property
    def ul(self) -> U8Vector2: ...
    @property
    def vu(self) -> U8Vector2: ...
    @property
    def vv(self) -> U8Vector2: ...
    @property
    def vo(self) -> U8Vector2: ...
    @property
    def vl(self) -> U8Vector2: ...
    @property
    def ou(self) -> U8Vector2: ...
    @property
    def ov(self) -> U8Vector2: ...
    @property
    def lu(self) -> U8Vector2: ...
    @property
    def lv(self) -> U8Vector2: ...
    @property
    def uuu(self) -> U8Vector3: ...
    @property
    def uuv(self) -> U8Vector3: ...
    @property
    def uuo(self) -> U8Vector3: ...
    @property
    def uul(self) -> U8Vector3: ...
    @property
    def uvu(self) -> U8Vector3: ...
    @property
    def uvv(self) -> U8Vector3: ...
    @property
    def uvo(self) -> U8Vector3: ...
    @property
    def uvl(self) -> U8Vector3: ...
    @property
    def uou(self) -> U8Vector3: ...
    @property
    def uov(self) -> U8Vector3: ...
    @property
    def uoo(self) -> U8Vector3: ...
    @property
    def uol(self) -> U8Vector3: ...
    @property
    def ulu(self) -> U8Vector3: ...
    @property
    def ulv(self) -> U8Vector3: ...
    @property
    def ulo(self) -> U8Vector3: ...
    @property
    def ull(self) -> U8Vector3: ...
    @property
    def vuu(self) -> U8Vector3: ...
    @property
    def vuv(self) -> U8Vector3: ...
    @property
    def vuo(self) -> U8Vector3: ...
    @property
    def vul(self) -> U8Vector3: ...
    @property
    def vvu(self) -> U8Vector3: ...
    @property
    def vvv(self) -> U8Vector3: ...
    @property
    def vvo(self) -> U8Vector3: ...
    @property
    def vvl(self) -> U8Vector3: ...
    @property
    def vou(self) -> U8Vector3: ...
    @property
    def vov(self) -> U8Vector3: ...
    @property
    def voo(self) -> U8Vector3: ...
    @property
    def vol(self) -> U8Vector3: ...
    @property
    def vlu(self) -> U8Vector3: ...
    @property
    def vlv(self) -> U8Vector3: ...
    @property
    def vlo(self) -> U8Vector3: ...
    @property
    def vll(self) -> U8Vector3: ...
    @property
    def ouu(self) -> U8Vector3: ...
    @property
    def ouv(self) -> U8Vector3: ...
    @property
    def ouo(self) -> U8Vector3: ...
    @property
    def oul(self) -> U8Vector3: ...
    @property
    def ovu(self) -> U8Vector3: ...
    @property
    def ovv(self) -> U8Vector3: ...
    @property
    def ovo(self) -> U8Vector3: ...
    @property
    def ovl(self) -> U8Vector3: ...
    @property
    def oou(self) -> U8Vector3: ...
    @property
    def oov(self) -> U8Vector3: ...
    @property
    def olu(self) -> U8Vector3: ...
    @property
    def olv(self) -> U8Vector3: ...
    @property
    def luu(self) -> U8Vector3: ...
    @property
    def luv(self) -> U8Vector3: ...
    @property
    def luo(self) -> U8Vector3: ...
    @property
    def lul(self) -> U8Vector3: ...
    @property
    def lvu(self) -> U8Vector3: ...
    @property
    def lvv(self) -> U8Vector3: ...
    @property
    def lvo(self) -> U8Vector3: ...
    @property
    def lvl(self) -> U8Vector3: ...
    @property
    def lou(self) -> U8Vector3: ...
    @property
    def lov(self) -> U8Vector3: ...
    @property
    def llu(self) -> U8Vector3: ...
    @property
    def llv(self) -> U8Vector3: ...
    @property
    def uuuu(self) -> U8Vector4: ...
    @property
    def uuuv(self) -> U8Vector4: ...
    @property
    def uuuo(self) -> U8Vector4: ...
    @property
    def uuul(self) -> U8Vector4: ...
    @property
    def uuvu(self) -> U8Vector4: ...
    @property
    def uuvv(self) -> U8Vector4: ...
    @property
    def uuvo(self) -> U8Vector4: ...
    @property
    def uuvl(self) -> U8Vector4: ...
    @property
    def uuou(self) -> U8Vector4: ...
    @property
    def uuov(self) -> U8Vector4: ...
    @property
    def uuoo(self) -> U8Vector4: ...
    @property
    def uuol(self) -> U8Vector4: ...
    @property
    def uulu(self) -> U8Vector4: ...
    @property
    def uulv(self) -> U8Vector4: ...
    @property
    def uulo(self) -> U8Vector4: ...
    @property
    def uull(self) -> U8Vector4: ...
    @property
    def uvuu(self) -> U8Vector4: ...
    @property
    def uvuv(self) -> U8Vector4: ...
    @property
    def uvuo(self) -> U8Vector4: ...
    @property
    def uvul(self) -> U8Vector4: ...
    @property
    def uvvu(self) -> U8Vector4: ...
    @property
    def uvvv(self) -> U8Vector4: ...
    @property
    def uvvo(self) -> U8Vector4: ...
    @property
    def uvvl(self) -> U8Vector4: ...
    @property
    def uvou(self) -> U8Vector4: ...
    @property
    def uvov(self) -> U8Vector4: ...
    @property
    def uvoo(self) -> U8Vector4: ...
    @property
    def uvol(self) -> U8Vector4: ...
    @property
    def uvlu(self) -> U8Vector4: ...
    @property
    def uvlv(self) -> U8Vector4: ...
    @property
    def uvlo(self) -> U8Vector4: ...
    @property
    def uvll(self) -> U8Vector4: ...
    @property
    def uouu(self) -> U8Vector4: ...
    @property
    def uouv(self) -> U8Vector4: ...
    @property
    def uouo(self) -> U8Vector4: ...
    @property
    def uoul(self) -> U8Vector4: ...
    @property
    def uovu(self) -> U8Vector4: ...
    @property
    def uovv(self) -> U8Vector4: ...
    @property
    def uovo(self) -> U8Vector4: ...
    @property
    def uovl(self) -> U8Vector4: ...
    @property
    def uoou(self) -> U8Vector4: ...
    @property
    def uoov(self) -> U8Vector4: ...
    @property
    def uooo(self) -> U8Vector4: ...
    @property
    def uool(self) -> U8Vector4: ...
    @property
    def uolu(self) -> U8Vector4: ...
    @property
    def uolv(self) -> U8Vector4: ...
    @property
    def uolo(self) -> U8Vector4: ...
    @property
    def uoll(self) -> U8Vector4: ...
    @property
    def uluu(self) -> U8Vector4: ...
    @property
    def uluv(self) -> U8Vector4: ...
    @property
    def uluo(self) -> U8Vector4: ...
    @property
    def ulul(self) -> U8Vector4: ...
    @property
    def ulvu(self) -> U8Vector4: ...
    @property
    def ulvv(self) -> U8Vector4: ...
    @property
    def ulvo(self) -> U8Vector4: ...
    @property
    def ulvl(self) -> U8Vector4: ...
    @property
    def ulou(self) -> U8Vector4: ...
    @property
    def ulov(self) -> U8Vector4: ...
    @property
    def uloo(self) -> U8Vector4: ...
    @property
    def ulol(self) -> U8Vector4: ...
    @property
    def ullu(self) -> U8Vector4: ...
    @property
    def ullv(self) -> U8Vector4: ...
    @property
    def ullo(self) -> U8Vector4: ...
    @property
    def ulll(self) -> U8Vector4: ...
    @property
    def vuuu(self) -> U8Vector4: ...
    @property
    def vuuv(self) -> U8Vector4: ...
    @property
    def vuuo(self) -> U8Vector4: ...
    @property
    def vuul(self) -> U8Vector4: ...
    @property
    def vuvu(self) -> U8Vector4: ...
    @property
    def vuvv(self) -> U8Vector4: ...
    @property
    def vuvo(self) -> U8Vector4: ...
    @property
    def vuvl(self) -> U8Vector4: ...
    @property
    def vuou(self) -> U8Vector4: ...
    @property
    def vuov(self) -> U8Vector4: ...
    @property
    def vuoo(self) -> U8Vector4: ...
    @property
    def vuol(self) -> U8Vector4: ...
    @property
    def vulu(self) -> U8Vector4: ...
    @property
    def vulv(self) -> U8Vector4: ...
    @property
    def vulo(self) -> U8Vector4: ...
    @property
    def vull(self) -> U8Vector4: ...
    @property
    def vvuu(self) -> U8Vector4: ...
    @property
    def vvuv(self) -> U8Vector4: ...
    @property
    def vvuo(self) -> U8Vector4: ...
    @property
    def vvul(self) -> U8Vector4: ...
    @property
    def vvvu(self) -> U8Vector4: ...
    @property
    def vvvv(self) -> U8Vector4: ...
    @property
    def vvvo(self) -> U8Vector4: ...
    @property
    def vvvl(self) -> U8Vector4: ...
    @property
    def vvou(self) -> U8Vector4: ...
    @property
    def vvov(self) -> U8Vector4: ...
    @property
    def vvoo(self) -> U8Vector4: ...
    @property
    def vvol(self) -> U8Vector4: ...
    @property
    def vvlu(self) -> U8Vector4: ...
    @property
    def vvlv(self) -> U8Vector4: ...
    @property
    def vvlo(self) -> U8Vector4: ...
    @property
    def vvll(self) -> U8Vector4: ...
    @property
    def vouu(self) -> U8Vector4: ...
    @property
    def vouv(self) -> U8Vector4: ...
    @property
    def vouo(self) -> U8Vector4: ...
    @property
    def voul(self) -> U8Vector4: ...
    @property
    def vovu(self) -> U8Vector4: ...
    @property
    def vovv(self) -> U8Vector4: ...
    @property
    def vovo(self) -> U8Vector4: ...
    @property
    def vovl(self) -> U8Vector4: ...
    @property
    def voou(self) -> U8Vector4: ...
    @property
    def voov(self) -> U8Vector4: ...
    @property
    def vooo(self) -> U8Vector4: ...
    @property
    def vool(self) -> U8Vector4: ...
    @property
    def volu(self) -> U8Vector4: ...
    @property
    def volv(self) -> U8Vector4: ...
    @property
    def volo(self) -> U8Vector4: ...
    @property
    def voll(self) -> U8Vector4: ...
    @property
    def vluu(self) -> U8Vector4: ...
    @property
    def vluv(self) -> U8Vector4: ...
    @property
    def vluo(self) -> U8Vector4: ...
    @property
    def vlul(self) -> U8Vector4: ...
    @property
    def vlvu(self) -> U8Vector4: ...
    @property
    def vlvv(self) -> U8Vector4: ...
    @property
    def vlvo(self) -> U8Vector4: ...
    @property
    def vlvl(self) -> U8Vector4: ...
    @property
    def vlou(self) -> U8Vector4: ...
    @property
    def vlov(self) -> U8Vector4: ...
    @property
    def vloo(self) -> U8Vector4: ...
    @property
    def vlol(self) -> U8Vector4: ...
    @property
    def vllu(self) -> U8Vector4: ...
    @property
    def vllv(self) -> U8Vector4: ...
    @property
    def vllo(self) -> U8Vector4: ...
    @property
    def vlll(self) -> U8Vector4: ...
    @property
    def ouuu(self) -> U8Vector4: ...
    @property
    def ouuv(self) -> U8Vector4: ...
    @property
    def ouuo(self) -> U8Vector4: ...
    @property
    def ouul(self) -> U8Vector4: ...
    @property
    def ouvu(self) -> U8Vector4: ...
    @property
    def ouvv(self) -> U8Vector4: ...
    @property
    def ouvo(self) -> U8Vector4: ...
    @property
    def ouvl(self) -> U8Vector4: ...
    @property
    def ouou(self) -> U8Vector4: ...
    @property
    def ouov(self) -> U8Vector4: ...
    @property
    def ouoo(self) -> U8Vector4: ...
    @property
    def ouol(self) -> U8Vector4: ...
    @property
    def oulu(self) -> U8Vector4: ...
    @property
    def oulv(self) -> U8Vector4: ...
    @property
    def oulo(self) -> U8Vector4: ...
    @property
    def oull(self) -> U8Vector4: ...
    @property
    def ovuu(self) -> U8Vector4: ...
    @property
    def ovuv(self) -> U8Vector4: ...
    @property
    def ovuo(self) -> U8Vector4: ...
    @property
    def ovul(self) -> U8Vector4: ...
    @property
    def ovvu(self) -> U8Vector4: ...
    @property
    def ovvv(self) -> U8Vector4: ...
    @property
    def ovvo(self) -> U8Vector4: ...
    @property
    def ovvl(self) -> U8Vector4: ...
    @property
    def ovou(self) -> U8Vector4: ...
    @property
    def ovov(self) -> U8Vector4: ...
    @property
    def ovoo(self) -> U8Vector4: ...
    @property
    def ovol(self) -> U8Vector4: ...
    @property
    def ovlu(self) -> U8Vector4: ...
    @property
    def ovlv(self) -> U8Vector4: ...
    @property
    def ovlo(self) -> U8Vector4: ...
    @property
    def ovll(self) -> U8Vector4: ...
    @property
    def oouu(self) -> U8Vector4: ...
    @property
    def oouv(self) -> U8Vector4: ...
    @property
    def oouo(self) -> U8Vector4: ...
    @property
    def ooul(self) -> U8Vector4: ...
    @property
    def oovu(self) -> U8Vector4: ...
    @property
    def oovv(self) -> U8Vector4: ...
    @property
    def oovo(self) -> U8Vector4: ...
    @property
    def oovl(self) -> U8Vector4: ...
    @property
    def ooou(self) -> U8Vector4: ...
    @property
    def ooov(self) -> U8Vector4: ...
    @property
    def oolu(self) -> U8Vector4: ...
    @property
    def oolv(self) -> U8Vector4: ...
    @property
    def oluu(self) -> U8Vector4: ...
    @property
    def oluv(self) -> U8Vector4: ...
    @property
    def oluo(self) -> U8Vector4: ...
    @property
    def olul(self) -> U8Vector4: ...
    @property
    def olvu(self) -> U8Vector4: ...
    @property
    def olvv(self) -> U8Vector4: ...
    @property
    def olvo(self) -> U8Vector4: ...
    @property
    def olvl(self) -> U8Vector4: ...
    @property
    def olou(self) -> U8Vector4: ...
    @property
    def olov(self) -> U8Vector4: ...
    @property
    def ollu(self) -> U8Vector4: ...
    @property
    def ollv(self) -> U8Vector4: ...
    @property
    def luuu(self) -> U8Vector4: ...
    @property
    def luuv(self) -> U8Vector4: ...
    @property
    def luuo(self) -> U8Vector4: ...
    @property
    def luul(self) -> U8Vector4: ...
    @property
    def luvu(self) -> U8Vector4: ...
    @property
    def luvv(self) -> U8Vector4: ...
    @property
    def luvo(self) -> U8Vector4: ...
    @property
    def luvl(self) -> U8Vector4: ...
    @property
    def luou(self) -> U8Vector4: ...
    @property
    def luov(self) -> U8Vector4: ...
    @property
    def luoo(self) -> U8Vector4: ...
    @property
    def luol(self) -> U8Vector4: ...
    @property
    def lulu(self) -> U8Vector4: ...
    @property
    def lulv(self) -> U8Vector4: ...
    @property
    def lulo(self) -> U8Vector4: ...
    @property
    def lull(self) -> U8Vector4: ...
    @property
    def lvuu(self) -> U8Vector4: ...
    @property
    def lvuv(self) -> U8Vector4: ...
    @property
    def lvuo(self) -> U8Vector4: ...
    @property
    def lvul(self) -> U8Vector4: ...
    @property
    def lvvu(self) -> U8Vector4: ...
    @property
    def lvvv(self) -> U8Vector4: ...
    @property
    def lvvo(self) -> U8Vector4: ...
    @property
    def lvvl(self) -> U8Vector4: ...
    @property
    def lvou(self) -> U8Vector4: ...
    @property
    def lvov(self) -> U8Vector4: ...
    @property
    def lvoo(self) -> U8Vector4: ...
    @property
    def lvol(self) -> U8Vector4: ...
    @property
    def lvlu(self) -> U8Vector4: ...
    @property
    def lvlv(self) -> U8Vector4: ...
    @property
    def lvlo(self) -> U8Vector4: ...
    @property
    def lvll(self) -> U8Vector4: ...
    @property
    def louu(self) -> U8Vector4: ...
    @property
    def louv(self) -> U8Vector4: ...
    @property
    def louo(self) -> U8Vector4: ...
    @property
    def loul(self) -> U8Vector4: ...
    @property
    def lovu(self) -> U8Vector4: ...
    @property
    def lovv(self) -> U8Vector4: ...
    @property
    def lovo(self) -> U8Vector4: ...
    @property
    def lovl(self) -> U8Vector4: ...
    @property
    def loou(self) -> U8Vector4: ...
    @property
    def loov(self) -> U8Vector4: ...
    @property
    def lolu(self) -> U8Vector4: ...
    @property
    def lolv(self) -> U8Vector4: ...
    @property
    def lluu(self) -> U8Vector4: ...
    @property
    def lluv(self) -> U8Vector4: ...
    @property
    def lluo(self) -> U8Vector4: ...
    @property
    def llul(self) -> U8Vector4: ...
    @property
    def llvu(self) -> U8Vector4: ...
    @property
    def llvv(self) -> U8Vector4: ...
    @property
    def llvo(self) -> U8Vector4: ...
    @property
    def llvl(self) -> U8Vector4: ...
    @property
    def llou(self) -> U8Vector4: ...
    @property
    def llov(self) -> U8Vector4: ...
    @property
    def lllu(self) -> U8Vector4: ...
    @property
    def lllv(self) -> U8Vector4: ...
    @property
    def oo(self) -> U8Vector2: ...
    @property
    def ol(self) -> U8Vector2: ...
    @property
    def lo(self) -> U8Vector2: ...
    @property
    def ll(self) -> U8Vector2: ...
    @property
    def ooo(self) -> U8Vector3: ...
    @property
    def ool(self) -> U8Vector3: ...
    @property
    def olo(self) -> U8Vector3: ...
    @property
    def oll(self) -> U8Vector3: ...
    @property
    def loo(self) -> U8Vector3: ...
    @property
    def lol(self) -> U8Vector3: ...
    @property
    def llo(self) -> U8Vector3: ...
    @property
    def lll(self) -> U8Vector3: ...
    @property
    def oooo(self) -> U8Vector4: ...
    @property
    def oool(self) -> U8Vector4: ...
    @property
    def oolo(self) -> U8Vector4: ...
    @property
    def ooll(self) -> U8Vector4: ...
    @property
    def oloo(self) -> U8Vector4: ...
    @property
    def olol(self) -> U8Vector4: ...
    @property
    def ollo(self) -> U8Vector4: ...
    @property
    def olll(self) -> U8Vector4: ...
    @property
    def looo(self) -> U8Vector4: ...
    @property
    def lool(self) -> U8Vector4: ...
    @property
    def lolo(self) -> U8Vector4: ...
    @property
    def loll(self) -> U8Vector4: ...
    @property
    def lloo(self) -> U8Vector4: ...
    @property
    def llol(self) -> U8Vector4: ...
    @property
    def lllo(self) -> U8Vector4: ...
    @property
    def llll(self) -> U8Vector4: ...
    def min(self, n: Number, /) -> U8Vector2: ...
    def max(self, n: Number, /) -> U8Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> U8Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U8Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector2: ...

@final
class U8Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U8Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U8Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> U8Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> U8Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[U8Vector2]: ...

@final
class I16Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __add__(self, other: Number) -> I16Vector2: ...
    @overload
    def __radd__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __radd__(self, other: Number) -> I16Vector2: ...
    @overload
    def __sub__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __sub__(self, other: Number) -> I16Vector2: ...
    @overload
    def __rsub__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> I16Vector2: ...
    @overload
    def __mul__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __mul__(self, other: Number) -> I16Vector2: ...
    @overload
    def __rmul__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> I16Vector2: ...
    @overload
    def __truediv__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> I16Vector2: ...
    @overload
    def __rtruediv__(self, other: I16Vector2) -> I16Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> I16Vector2: ...
    def __neg__(self) -> I16Vector2: ...
    def __abs__(self) -> I16Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> I16Vector2: ...
    @property
    def xy(self) -> I16Vector2: ...
    @property
    def xo(self) -> I16Vector2: ...
    @property
    def xl(self) -> I16Vector2: ...
    @property
    def yx(self) -> I16Vector2: ...
    @property
    def yy(self) -> I16Vector2: ...
    @property
    def yo(self) -> I16Vector2: ...
    @property
    def yl(self) -> I16Vector2: ...
    @property
    def ox(self) -> I16Vector2: ...
    @property
    def oy(self) -> I16Vector2: ...
    @property
    def lx(self) -> I16Vector2: ...
    @property
    def ly(self) -> I16Vector2: ...
    @property
    def xxx(self) -> I16Vector3: ...
    @property
    def xxy(self) -> I16Vector3: ...
    @property
    def xxo(self) -> I16Vector3: ...
    @property
    def xxl(self) -> I16Vector3: ...
    @property
    def xyx(self) -> I16Vector3: ...
    @property
    def xyy(self) -> I16Vector3: ...
    @property
    def xyo(self) -> I16Vector3: ...
    @property
    def xyl(self) -> I16Vector3: ...
    @property
    def xox(self) -> I16Vector3: ...
    @property
    def xoy(self) -> I16Vector3: ...
    @property
    def xoo(self) -> I16Vector3: ...
    @property
    def xol(self) -> I16Vector3: ...
    @property
    def xlx(self) -> I16Vector3: ...
    @property
    def xly(self) -> I16Vector3: ...
    @property
    def xlo(self) -> I16Vector3: ...
    @property
    def xll(self) -> I16Vector3: ...
    @property
    def yxx(self) -> I16Vector3: ...
    @property
    def yxy(self) -> I16Vector3: ...
    @property
    def yxo(self) -> I16Vector3: ...
    @property
    def yxl(self) -> I16Vector3: ...
    @property
    def yyx(self) -> I16Vector3: ...
    @property
    def yyy(self) -> I16Vector3: ...
    @property
    def yyo(self) -> I16Vector3: ...
    @property
    def yyl(self) -> I16Vector3: ...
    @property
    def yox(self) -> I16Vector3: ...
    @property
    def yoy(self) -> I16Vector3: ...
    @property
    def yoo(self) -> I16Vector3: ...
    @property
    def yol(self) -> I16Vector3: ...
    @property
    def ylx(self) -> I16Vector3: ...
    @property
    def yly(self) -> I16Vector3: ...
    @property
    def ylo(self) -> I16Vector3: ...
    @property
    def yll(self) -> I16Vector3: ...
    @property
    def oxx(self) -> I16Vector3: ...
    @property
    def oxy(self) -> I16Vector3: ...
    @property
    def oxo(self) -> I16Vector3: ...
    @property
    def oxl(self) -> I16Vector3: ...
    @property
    def oyx(self) -> I16Vector3: ...
    @property
    def oyy(self) -> I16Vector3: ...
    @property
    def oyo(self) -> I16Vector3: ...
    @property
    def oyl(self) -> I16Vector3: ...
    @property
    def oox(self) -> I16Vector3: ...
    @property
    def ooy(self) -> I16Vector3: ...
    @property
    def olx(self) -> I16Vector3: ...
    @property
    def oly(self) -> I16Vector3: ...
    @property
    def lxx(self) -> I16Vector3: ...
    @property
    def lxy(self) -> I16Vector3: ...
    @property
    def lxo(self) -> I16Vector3: ...
    @property
    def lxl(self) -> I16Vector3: ...
    @property
    def lyx(self) -> I16Vector3: ...
    @property
    def lyy(self) -> I16Vector3: ...
    @property
    def lyo(self) -> I16Vector3: ...
    @property
    def lyl(self) -> I16Vector3: ...
    @property
    def lox(self) -> I16Vector3: ...
    @property
    def loy(self) -> I16Vector3: ...
    @property
    def llx(self) -> I16Vector3: ...
    @property
    def lly(self) -> I16Vector3: ...
    @property
    def xxxx(self) -> I16Vector4: ...
    @property
    def xxxy(self) -> I16Vector4: ...
    @property
    def xxxo(self) -> I16Vector4: ...
    @property
    def xxxl(self) -> I16Vector4: ...
    @property
    def xxyx(self) -> I16Vector4: ...
    @property
    def xxyy(self) -> I16Vector4: ...
    @property
    def xxyo(self) -> I16Vector4: ...
    @property
    def xxyl(self) -> I16Vector4: ...
    @property
    def xxox(self) -> I16Vector4: ...
    @property
    def xxoy(self) -> I16Vector4: ...
    @property
    def xxoo(self) -> I16Vector4: ...
    @property
    def xxol(self) -> I16Vector4: ...
    @property
    def xxlx(self) -> I16Vector4: ...
    @property
    def xxly(self) -> I16Vector4: ...
    @property
    def xxlo(self) -> I16Vector4: ...
    @property
    def xxll(self) -> I16Vector4: ...
    @property
    def xyxx(self) -> I16Vector4: ...
    @property
    def xyxy(self) -> I16Vector4: ...
    @property
    def xyxo(self) -> I16Vector4: ...
    @property
    def xyxl(self) -> I16Vector4: ...
    @property
    def xyyx(self) -> I16Vector4: ...
    @property
    def xyyy(self) -> I16Vector4: ...
    @property
    def xyyo(self) -> I16Vector4: ...
    @property
    def xyyl(self) -> I16Vector4: ...
    @property
    def xyox(self) -> I16Vector4: ...
    @property
    def xyoy(self) -> I16Vector4: ...
    @property
    def xyoo(self) -> I16Vector4: ...
    @property
    def xyol(self) -> I16Vector4: ...
    @property
    def xylx(self) -> I16Vector4: ...
    @property
    def xyly(self) -> I16Vector4: ...
    @property
    def xylo(self) -> I16Vector4: ...
    @property
    def xyll(self) -> I16Vector4: ...
    @property
    def xoxx(self) -> I16Vector4: ...
    @property
    def xoxy(self) -> I16Vector4: ...
    @property
    def xoxo(self) -> I16Vector4: ...
    @property
    def xoxl(self) -> I16Vector4: ...
    @property
    def xoyx(self) -> I16Vector4: ...
    @property
    def xoyy(self) -> I16Vector4: ...
    @property
    def xoyo(self) -> I16Vector4: ...
    @property
    def xoyl(self) -> I16Vector4: ...
    @property
    def xoox(self) -> I16Vector4: ...
    @property
    def xooy(self) -> I16Vector4: ...
    @property
    def xooo(self) -> I16Vector4: ...
    @property
    def xool(self) -> I16Vector4: ...
    @property
    def xolx(self) -> I16Vector4: ...
    @property
    def xoly(self) -> I16Vector4: ...
    @property
    def xolo(self) -> I16Vector4: ...
    @property
    def xoll(self) -> I16Vector4: ...
    @property
    def xlxx(self) -> I16Vector4: ...
    @property
    def xlxy(self) -> I16Vector4: ...
    @property
    def xlxo(self) -> I16Vector4: ...
    @property
    def xlxl(self) -> I16Vector4: ...
    @property
    def xlyx(self) -> I16Vector4: ...
    @property
    def xlyy(self) -> I16Vector4: ...
    @property
    def xlyo(self) -> I16Vector4: ...
    @property
    def xlyl(self) -> I16Vector4: ...
    @property
    def xlox(self) -> I16Vector4: ...
    @property
    def xloy(self) -> I16Vector4: ...
    @property
    def xloo(self) -> I16Vector4: ...
    @property
    def xlol(self) -> I16Vector4: ...
    @property
    def xllx(self) -> I16Vector4: ...
    @property
    def xlly(self) -> I16Vector4: ...
    @property
    def xllo(self) -> I16Vector4: ...
    @property
    def xlll(self) -> I16Vector4: ...
    @property
    def yxxx(self) -> I16Vector4: ...
    @property
    def yxxy(self) -> I16Vector4: ...
    @property
    def yxxo(self) -> I16Vector4: ...
    @property
    def yxxl(self) -> I16Vector4: ...
    @property
    def yxyx(self) -> I16Vector4: ...
    @property
    def yxyy(self) -> I16Vector4: ...
    @property
    def yxyo(self) -> I16Vector4: ...
    @property
    def yxyl(self) -> I16Vector4: ...
    @property
    def yxox(self) -> I16Vector4: ...
    @property
    def yxoy(self) -> I16Vector4: ...
    @property
    def yxoo(self) -> I16Vector4: ...
    @property
    def yxol(self) -> I16Vector4: ...
    @property
    def yxlx(self) -> I16Vector4: ...
    @property
    def yxly(self) -> I16Vector4: ...
    @property
    def yxlo(self) -> I16Vector4: ...
    @property
    def yxll(self) -> I16Vector4: ...
    @property
    def yyxx(self) -> I16Vector4: ...
    @property
    def yyxy(self) -> I16Vector4: ...
    @property
    def yyxo(self) -> I16Vector4: ...
    @property
    def yyxl(self) -> I16Vector4: ...
    @property
    def yyyx(self) -> I16Vector4: ...
    @property
    def yyyy(self) -> I16Vector4: ...
    @property
    def yyyo(self) -> I16Vector4: ...
    @property
    def yyyl(self) -> I16Vector4: ...
    @property
    def yyox(self) -> I16Vector4: ...
    @property
    def yyoy(self) -> I16Vector4: ...
    @property
    def yyoo(self) -> I16Vector4: ...
    @property
    def yyol(self) -> I16Vector4: ...
    @property
    def yylx(self) -> I16Vector4: ...
    @property
    def yyly(self) -> I16Vector4: ...
    @property
    def yylo(self) -> I16Vector4: ...
    @property
    def yyll(self) -> I16Vector4: ...
    @property
    def yoxx(self) -> I16Vector4: ...
    @property
    def yoxy(self) -> I16Vector4: ...
    @property
    def yoxo(self) -> I16Vector4: ...
    @property
    def yoxl(self) -> I16Vector4: ...
    @property
    def yoyx(self) -> I16Vector4: ...
    @property
    def yoyy(self) -> I16Vector4: ...
    @property
    def yoyo(self) -> I16Vector4: ...
    @property
    def yoyl(self) -> I16Vector4: ...
    @property
    def yoox(self) -> I16Vector4: ...
    @property
    def yooy(self) -> I16Vector4: ...
    @property
    def yooo(self) -> I16Vector4: ...
    @property
    def yool(self) -> I16Vector4: ...
    @property
    def yolx(self) -> I16Vector4: ...
    @property
    def yoly(self) -> I16Vector4: ...
    @property
    def yolo(self) -> I16Vector4: ...
    @property
    def yoll(self) -> I16Vector4: ...
    @property
    def ylxx(self) -> I16Vector4: ...
    @property
    def ylxy(self) -> I16Vector4: ...
    @property
    def ylxo(self) -> I16Vector4: ...
    @property
    def ylxl(self) -> I16Vector4: ...
    @property
    def ylyx(self) -> I16Vector4: ...
    @property
    def ylyy(self) -> I16Vector4: ...
    @property
    def ylyo(self) -> I16Vector4: ...
    @property
    def ylyl(self) -> I16Vector4: ...
    @property
    def ylox(self) -> I16Vector4: ...
    @property
    def yloy(self) -> I16Vector4: ...
    @property
    def yloo(self) -> I16Vector4: ...
    @property
    def ylol(self) -> I16Vector4: ...
    @property
    def yllx(self) -> I16Vector4: ...
    @property
    def ylly(self) -> I16Vector4: ...
    @property
    def yllo(self) -> I16Vector4: ...
    @property
    def ylll(self) -> I16Vector4: ...
    @property
    def oxxx(self) -> I16Vector4: ...
    @property
    def oxxy(self) -> I16Vector4: ...
    @property
    def oxxo(self) -> I16Vector4: ...
    @property
    def oxxl(self) -> I16Vector4: ...
    @property
    def oxyx(self) -> I16Vector4: ...
    @property
    def oxyy(self) -> I16Vector4: ...
    @property
    def oxyo(self) -> I16Vector4: ...
    @property
    def oxyl(self) -> I16Vector4: ...
    @property
    def oxox(self) -> I16Vector4: ...
    @property
    def oxoy(self) -> I16Vector4: ...
    @property
    def oxoo(self) -> I16Vector4: ...
    @property
    def oxol(self) -> I16Vector4: ...
    @property
    def oxlx(self) -> I16Vector4: ...
    @property
    def oxly(self) -> I16Vector4: ...
    @property
    def oxlo(self) -> I16Vector4: ...
    @property
    def oxll(self) -> I16Vector4: ...
    @property
    def oyxx(self) -> I16Vector4: ...
    @property
    def oyxy(self) -> I16Vector4: ...
    @property
    def oyxo(self) -> I16Vector4: ...
    @property
    def oyxl(self) -> I16Vector4: ...
    @property
    def oyyx(self) -> I16Vector4: ...
    @property
    def oyyy(self) -> I16Vector4: ...
    @property
    def oyyo(self) -> I16Vector4: ...
    @property
    def oyyl(self) -> I16Vector4: ...
    @property
    def oyox(self) -> I16Vector4: ...
    @property
    def oyoy(self) -> I16Vector4: ...
    @property
    def oyoo(self) -> I16Vector4: ...
    @property
    def oyol(self) -> I16Vector4: ...
    @property
    def oylx(self) -> I16Vector4: ...
    @property
    def oyly(self) -> I16Vector4: ...
    @property
    def oylo(self) -> I16Vector4: ...
    @property
    def oyll(self) -> I16Vector4: ...
    @property
    def ooxx(self) -> I16Vector4: ...
    @property
    def ooxy(self) -> I16Vector4: ...
    @property
    def ooxo(self) -> I16Vector4: ...
    @property
    def ooxl(self) -> I16Vector4: ...
    @property
    def ooyx(self) -> I16Vector4: ...
    @property
    def ooyy(self) -> I16Vector4: ...
    @property
    def ooyo(self) -> I16Vector4: ...
    @property
    def ooyl(self) -> I16Vector4: ...
    @property
    def ooox(self) -> I16Vector4: ...
    @property
    def oooy(self) -> I16Vector4: ...
    @property
    def oolx(self) -> I16Vector4: ...
    @property
    def ooly(self) -> I16Vector4: ...
    @property
    def olxx(self) -> I16Vector4: ...
    @property
    def olxy(self) -> I16Vector4: ...
    @property
    def olxo(self) -> I16Vector4: ...
    @property
    def olxl(self) -> I16Vector4: ...
    @property
    def olyx(self) -> I16Vector4: ...
    @property
    def olyy(self) -> I16Vector4: ...
    @property
    def olyo(self) -> I16Vector4: ...
    @property
    def olyl(self) -> I16Vector4: ...
    @property
    def olox(self) -> I16Vector4: ...
    @property
    def oloy(self) -> I16Vector4: ...
    @property
    def ollx(self) -> I16Vector4: ...
    @property
    def olly(self) -> I16Vector4: ...
    @property
    def lxxx(self) -> I16Vector4: ...
    @property
    def lxxy(self) -> I16Vector4: ...
    @property
    def lxxo(self) -> I16Vector4: ...
    @property
    def lxxl(self) -> I16Vector4: ...
    @property
    def lxyx(self) -> I16Vector4: ...
    @property
    def lxyy(self) -> I16Vector4: ...
    @property
    def lxyo(self) -> I16Vector4: ...
    @property
    def lxyl(self) -> I16Vector4: ...
    @property
    def lxox(self) -> I16Vector4: ...
    @property
    def lxoy(self) -> I16Vector4: ...
    @property
    def lxoo(self) -> I16Vector4: ...
    @property
    def lxol(self) -> I16Vector4: ...
    @property
    def lxlx(self) -> I16Vector4: ...
    @property
    def lxly(self) -> I16Vector4: ...
    @property
    def lxlo(self) -> I16Vector4: ...
    @property
    def lxll(self) -> I16Vector4: ...
    @property
    def lyxx(self) -> I16Vector4: ...
    @property
    def lyxy(self) -> I16Vector4: ...
    @property
    def lyxo(self) -> I16Vector4: ...
    @property
    def lyxl(self) -> I16Vector4: ...
    @property
    def lyyx(self) -> I16Vector4: ...
    @property
    def lyyy(self) -> I16Vector4: ...
    @property
    def lyyo(self) -> I16Vector4: ...
    @property
    def lyyl(self) -> I16Vector4: ...
    @property
    def lyox(self) -> I16Vector4: ...
    @property
    def lyoy(self) -> I16Vector4: ...
    @property
    def lyoo(self) -> I16Vector4: ...
    @property
    def lyol(self) -> I16Vector4: ...
    @property
    def lylx(self) -> I16Vector4: ...
    @property
    def lyly(self) -> I16Vector4: ...
    @property
    def lylo(self) -> I16Vector4: ...
    @property
    def lyll(self) -> I16Vector4: ...
    @property
    def loxx(self) -> I16Vector4: ...
    @property
    def loxy(self) -> I16Vector4: ...
    @property
    def loxo(self) -> I16Vector4: ...
    @property
    def loxl(self) -> I16Vector4: ...
    @property
    def loyx(self) -> I16Vector4: ...
    @property
    def loyy(self) -> I16Vector4: ...
    @property
    def loyo(self) -> I16Vector4: ...
    @property
    def loyl(self) -> I16Vector4: ...
    @property
    def loox(self) -> I16Vector4: ...
    @property
    def looy(self) -> I16Vector4: ...
    @property
    def lolx(self) -> I16Vector4: ...
    @property
    def loly(self) -> I16Vector4: ...
    @property
    def llxx(self) -> I16Vector4: ...
    @property
    def llxy(self) -> I16Vector4: ...
    @property
    def llxo(self) -> I16Vector4: ...
    @property
    def llxl(self) -> I16Vector4: ...
    @property
    def llyx(self) -> I16Vector4: ...
    @property
    def llyy(self) -> I16Vector4: ...
    @property
    def llyo(self) -> I16Vector4: ...
    @property
    def llyl(self) -> I16Vector4: ...
    @property
    def llox(self) -> I16Vector4: ...
    @property
    def lloy(self) -> I16Vector4: ...
    @property
    def lllx(self) -> I16Vector4: ...
    @property
    def llly(self) -> I16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> I16Vector2: ...
    @property
    def rg(self) -> I16Vector2: ...
    @property
    def ro(self) -> I16Vector2: ...
    @property
    def rl(self) -> I16Vector2: ...
    @property
    def gr(self) -> I16Vector2: ...
    @property
    def gg(self) -> I16Vector2: ...
    @property
    def go(self) -> I16Vector2: ...
    @property
    def gl(self) -> I16Vector2: ...
    @property
    def og(self) -> I16Vector2: ...
    @property
    def lr(self) -> I16Vector2: ...
    @property
    def lg(self) -> I16Vector2: ...
    @property
    def rrr(self) -> I16Vector3: ...
    @property
    def rrg(self) -> I16Vector3: ...
    @property
    def rro(self) -> I16Vector3: ...
    @property
    def rrl(self) -> I16Vector3: ...
    @property
    def rgr(self) -> I16Vector3: ...
    @property
    def rgg(self) -> I16Vector3: ...
    @property
    def rgo(self) -> I16Vector3: ...
    @property
    def rgl(self) -> I16Vector3: ...
    @property
    def ror(self) -> I16Vector3: ...
    @property
    def rog(self) -> I16Vector3: ...
    @property
    def roo(self) -> I16Vector3: ...
    @property
    def rol(self) -> I16Vector3: ...
    @property
    def rlr(self) -> I16Vector3: ...
    @property
    def rlg(self) -> I16Vector3: ...
    @property
    def rlo(self) -> I16Vector3: ...
    @property
    def rll(self) -> I16Vector3: ...
    @property
    def grr(self) -> I16Vector3: ...
    @property
    def grg(self) -> I16Vector3: ...
    @property
    def gro(self) -> I16Vector3: ...
    @property
    def grl(self) -> I16Vector3: ...
    @property
    def ggr(self) -> I16Vector3: ...
    @property
    def ggg(self) -> I16Vector3: ...
    @property
    def ggo(self) -> I16Vector3: ...
    @property
    def ggl(self) -> I16Vector3: ...
    @property
    def gor(self) -> I16Vector3: ...
    @property
    def gog(self) -> I16Vector3: ...
    @property
    def goo(self) -> I16Vector3: ...
    @property
    def gol(self) -> I16Vector3: ...
    @property
    def glr(self) -> I16Vector3: ...
    @property
    def glg(self) -> I16Vector3: ...
    @property
    def glo(self) -> I16Vector3: ...
    @property
    def gll(self) -> I16Vector3: ...
    @property
    def orr(self) -> I16Vector3: ...
    @property
    def org(self) -> I16Vector3: ...
    @property
    def oro(self) -> I16Vector3: ...
    @property
    def orl(self) -> I16Vector3: ...
    @property
    def ogr(self) -> I16Vector3: ...
    @property
    def ogg(self) -> I16Vector3: ...
    @property
    def ogo(self) -> I16Vector3: ...
    @property
    def ogl(self) -> I16Vector3: ...
    @property
    def oor(self) -> I16Vector3: ...
    @property
    def oog(self) -> I16Vector3: ...
    @property
    def olr(self) -> I16Vector3: ...
    @property
    def olg(self) -> I16Vector3: ...
    @property
    def lrr(self) -> I16Vector3: ...
    @property
    def lrg(self) -> I16Vector3: ...
    @property
    def lro(self) -> I16Vector3: ...
    @property
    def lrl(self) -> I16Vector3: ...
    @property
    def lgr(self) -> I16Vector3: ...
    @property
    def lgg(self) -> I16Vector3: ...
    @property
    def lgo(self) -> I16Vector3: ...
    @property
    def lgl(self) -> I16Vector3: ...
    @property
    def lor(self) -> I16Vector3: ...
    @property
    def log(self) -> I16Vector3: ...
    @property
    def llr(self) -> I16Vector3: ...
    @property
    def llg(self) -> I16Vector3: ...
    @property
    def rrrr(self) -> I16Vector4: ...
    @property
    def rrrg(self) -> I16Vector4: ...
    @property
    def rrro(self) -> I16Vector4: ...
    @property
    def rrrl(self) -> I16Vector4: ...
    @property
    def rrgr(self) -> I16Vector4: ...
    @property
    def rrgg(self) -> I16Vector4: ...
    @property
    def rrgo(self) -> I16Vector4: ...
    @property
    def rrgl(self) -> I16Vector4: ...
    @property
    def rror(self) -> I16Vector4: ...
    @property
    def rrog(self) -> I16Vector4: ...
    @property
    def rroo(self) -> I16Vector4: ...
    @property
    def rrol(self) -> I16Vector4: ...
    @property
    def rrlr(self) -> I16Vector4: ...
    @property
    def rrlg(self) -> I16Vector4: ...
    @property
    def rrlo(self) -> I16Vector4: ...
    @property
    def rrll(self) -> I16Vector4: ...
    @property
    def rgrr(self) -> I16Vector4: ...
    @property
    def rgrg(self) -> I16Vector4: ...
    @property
    def rgro(self) -> I16Vector4: ...
    @property
    def rgrl(self) -> I16Vector4: ...
    @property
    def rggr(self) -> I16Vector4: ...
    @property
    def rggg(self) -> I16Vector4: ...
    @property
    def rggo(self) -> I16Vector4: ...
    @property
    def rggl(self) -> I16Vector4: ...
    @property
    def rgor(self) -> I16Vector4: ...
    @property
    def rgog(self) -> I16Vector4: ...
    @property
    def rgoo(self) -> I16Vector4: ...
    @property
    def rgol(self) -> I16Vector4: ...
    @property
    def rglr(self) -> I16Vector4: ...
    @property
    def rglg(self) -> I16Vector4: ...
    @property
    def rglo(self) -> I16Vector4: ...
    @property
    def rgll(self) -> I16Vector4: ...
    @property
    def rorr(self) -> I16Vector4: ...
    @property
    def rorg(self) -> I16Vector4: ...
    @property
    def roro(self) -> I16Vector4: ...
    @property
    def rorl(self) -> I16Vector4: ...
    @property
    def rogr(self) -> I16Vector4: ...
    @property
    def rogg(self) -> I16Vector4: ...
    @property
    def rogo(self) -> I16Vector4: ...
    @property
    def rogl(self) -> I16Vector4: ...
    @property
    def roor(self) -> I16Vector4: ...
    @property
    def roog(self) -> I16Vector4: ...
    @property
    def rooo(self) -> I16Vector4: ...
    @property
    def rool(self) -> I16Vector4: ...
    @property
    def rolr(self) -> I16Vector4: ...
    @property
    def rolg(self) -> I16Vector4: ...
    @property
    def rolo(self) -> I16Vector4: ...
    @property
    def roll(self) -> I16Vector4: ...
    @property
    def rlrr(self) -> I16Vector4: ...
    @property
    def rlrg(self) -> I16Vector4: ...
    @property
    def rlro(self) -> I16Vector4: ...
    @property
    def rlrl(self) -> I16Vector4: ...
    @property
    def rlgr(self) -> I16Vector4: ...
    @property
    def rlgg(self) -> I16Vector4: ...
    @property
    def rlgo(self) -> I16Vector4: ...
    @property
    def rlgl(self) -> I16Vector4: ...
    @property
    def rlor(self) -> I16Vector4: ...
    @property
    def rlog(self) -> I16Vector4: ...
    @property
    def rloo(self) -> I16Vector4: ...
    @property
    def rlol(self) -> I16Vector4: ...
    @property
    def rllr(self) -> I16Vector4: ...
    @property
    def rllg(self) -> I16Vector4: ...
    @property
    def rllo(self) -> I16Vector4: ...
    @property
    def rlll(self) -> I16Vector4: ...
    @property
    def grrr(self) -> I16Vector4: ...
    @property
    def grrg(self) -> I16Vector4: ...
    @property
    def grro(self) -> I16Vector4: ...
    @property
    def grrl(self) -> I16Vector4: ...
    @property
    def grgr(self) -> I16Vector4: ...
    @property
    def grgg(self) -> I16Vector4: ...
    @property
    def grgo(self) -> I16Vector4: ...
    @property
    def grgl(self) -> I16Vector4: ...
    @property
    def gror(self) -> I16Vector4: ...
    @property
    def grog(self) -> I16Vector4: ...
    @property
    def groo(self) -> I16Vector4: ...
    @property
    def grol(self) -> I16Vector4: ...
    @property
    def grlr(self) -> I16Vector4: ...
    @property
    def grlg(self) -> I16Vector4: ...
    @property
    def grlo(self) -> I16Vector4: ...
    @property
    def grll(self) -> I16Vector4: ...
    @property
    def ggrr(self) -> I16Vector4: ...
    @property
    def ggrg(self) -> I16Vector4: ...
    @property
    def ggro(self) -> I16Vector4: ...
    @property
    def ggrl(self) -> I16Vector4: ...
    @property
    def gggr(self) -> I16Vector4: ...
    @property
    def gggg(self) -> I16Vector4: ...
    @property
    def gggo(self) -> I16Vector4: ...
    @property
    def gggl(self) -> I16Vector4: ...
    @property
    def ggor(self) -> I16Vector4: ...
    @property
    def ggog(self) -> I16Vector4: ...
    @property
    def ggoo(self) -> I16Vector4: ...
    @property
    def ggol(self) -> I16Vector4: ...
    @property
    def gglr(self) -> I16Vector4: ...
    @property
    def gglg(self) -> I16Vector4: ...
    @property
    def gglo(self) -> I16Vector4: ...
    @property
    def ggll(self) -> I16Vector4: ...
    @property
    def gorr(self) -> I16Vector4: ...
    @property
    def gorg(self) -> I16Vector4: ...
    @property
    def goro(self) -> I16Vector4: ...
    @property
    def gorl(self) -> I16Vector4: ...
    @property
    def gogr(self) -> I16Vector4: ...
    @property
    def gogg(self) -> I16Vector4: ...
    @property
    def gogo(self) -> I16Vector4: ...
    @property
    def gogl(self) -> I16Vector4: ...
    @property
    def goor(self) -> I16Vector4: ...
    @property
    def goog(self) -> I16Vector4: ...
    @property
    def gooo(self) -> I16Vector4: ...
    @property
    def gool(self) -> I16Vector4: ...
    @property
    def golr(self) -> I16Vector4: ...
    @property
    def golg(self) -> I16Vector4: ...
    @property
    def golo(self) -> I16Vector4: ...
    @property
    def goll(self) -> I16Vector4: ...
    @property
    def glrr(self) -> I16Vector4: ...
    @property
    def glrg(self) -> I16Vector4: ...
    @property
    def glro(self) -> I16Vector4: ...
    @property
    def glrl(self) -> I16Vector4: ...
    @property
    def glgr(self) -> I16Vector4: ...
    @property
    def glgg(self) -> I16Vector4: ...
    @property
    def glgo(self) -> I16Vector4: ...
    @property
    def glgl(self) -> I16Vector4: ...
    @property
    def glor(self) -> I16Vector4: ...
    @property
    def glog(self) -> I16Vector4: ...
    @property
    def gloo(self) -> I16Vector4: ...
    @property
    def glol(self) -> I16Vector4: ...
    @property
    def gllr(self) -> I16Vector4: ...
    @property
    def gllg(self) -> I16Vector4: ...
    @property
    def gllo(self) -> I16Vector4: ...
    @property
    def glll(self) -> I16Vector4: ...
    @property
    def orrr(self) -> I16Vector4: ...
    @property
    def orrg(self) -> I16Vector4: ...
    @property
    def orro(self) -> I16Vector4: ...
    @property
    def orrl(self) -> I16Vector4: ...
    @property
    def orgr(self) -> I16Vector4: ...
    @property
    def orgg(self) -> I16Vector4: ...
    @property
    def orgo(self) -> I16Vector4: ...
    @property
    def orgl(self) -> I16Vector4: ...
    @property
    def oror(self) -> I16Vector4: ...
    @property
    def orog(self) -> I16Vector4: ...
    @property
    def oroo(self) -> I16Vector4: ...
    @property
    def orol(self) -> I16Vector4: ...
    @property
    def orlr(self) -> I16Vector4: ...
    @property
    def orlg(self) -> I16Vector4: ...
    @property
    def orlo(self) -> I16Vector4: ...
    @property
    def orll(self) -> I16Vector4: ...
    @property
    def ogrr(self) -> I16Vector4: ...
    @property
    def ogrg(self) -> I16Vector4: ...
    @property
    def ogro(self) -> I16Vector4: ...
    @property
    def ogrl(self) -> I16Vector4: ...
    @property
    def oggr(self) -> I16Vector4: ...
    @property
    def oggg(self) -> I16Vector4: ...
    @property
    def oggo(self) -> I16Vector4: ...
    @property
    def oggl(self) -> I16Vector4: ...
    @property
    def ogor(self) -> I16Vector4: ...
    @property
    def ogog(self) -> I16Vector4: ...
    @property
    def ogoo(self) -> I16Vector4: ...
    @property
    def ogol(self) -> I16Vector4: ...
    @property
    def oglr(self) -> I16Vector4: ...
    @property
    def oglg(self) -> I16Vector4: ...
    @property
    def oglo(self) -> I16Vector4: ...
    @property
    def ogll(self) -> I16Vector4: ...
    @property
    def oorr(self) -> I16Vector4: ...
    @property
    def oorg(self) -> I16Vector4: ...
    @property
    def ooro(self) -> I16Vector4: ...
    @property
    def oorl(self) -> I16Vector4: ...
    @property
    def oogr(self) -> I16Vector4: ...
    @property
    def oogg(self) -> I16Vector4: ...
    @property
    def oogo(self) -> I16Vector4: ...
    @property
    def oogl(self) -> I16Vector4: ...
    @property
    def ooor(self) -> I16Vector4: ...
    @property
    def ooog(self) -> I16Vector4: ...
    @property
    def oolr(self) -> I16Vector4: ...
    @property
    def oolg(self) -> I16Vector4: ...
    @property
    def olrr(self) -> I16Vector4: ...
    @property
    def olrg(self) -> I16Vector4: ...
    @property
    def olro(self) -> I16Vector4: ...
    @property
    def olrl(self) -> I16Vector4: ...
    @property
    def olgr(self) -> I16Vector4: ...
    @property
    def olgg(self) -> I16Vector4: ...
    @property
    def olgo(self) -> I16Vector4: ...
    @property
    def olgl(self) -> I16Vector4: ...
    @property
    def olor(self) -> I16Vector4: ...
    @property
    def olog(self) -> I16Vector4: ...
    @property
    def ollr(self) -> I16Vector4: ...
    @property
    def ollg(self) -> I16Vector4: ...
    @property
    def lrrr(self) -> I16Vector4: ...
    @property
    def lrrg(self) -> I16Vector4: ...
    @property
    def lrro(self) -> I16Vector4: ...
    @property
    def lrrl(self) -> I16Vector4: ...
    @property
    def lrgr(self) -> I16Vector4: ...
    @property
    def lrgg(self) -> I16Vector4: ...
    @property
    def lrgo(self) -> I16Vector4: ...
    @property
    def lrgl(self) -> I16Vector4: ...
    @property
    def lror(self) -> I16Vector4: ...
    @property
    def lrog(self) -> I16Vector4: ...
    @property
    def lroo(self) -> I16Vector4: ...
    @property
    def lrol(self) -> I16Vector4: ...
    @property
    def lrlr(self) -> I16Vector4: ...
    @property
    def lrlg(self) -> I16Vector4: ...
    @property
    def lrlo(self) -> I16Vector4: ...
    @property
    def lrll(self) -> I16Vector4: ...
    @property
    def lgrr(self) -> I16Vector4: ...
    @property
    def lgrg(self) -> I16Vector4: ...
    @property
    def lgro(self) -> I16Vector4: ...
    @property
    def lgrl(self) -> I16Vector4: ...
    @property
    def lggr(self) -> I16Vector4: ...
    @property
    def lggg(self) -> I16Vector4: ...
    @property
    def lggo(self) -> I16Vector4: ...
    @property
    def lggl(self) -> I16Vector4: ...
    @property
    def lgor(self) -> I16Vector4: ...
    @property
    def lgog(self) -> I16Vector4: ...
    @property
    def lgoo(self) -> I16Vector4: ...
    @property
    def lgol(self) -> I16Vector4: ...
    @property
    def lglr(self) -> I16Vector4: ...
    @property
    def lglg(self) -> I16Vector4: ...
    @property
    def lglo(self) -> I16Vector4: ...
    @property
    def lgll(self) -> I16Vector4: ...
    @property
    def lorr(self) -> I16Vector4: ...
    @property
    def lorg(self) -> I16Vector4: ...
    @property
    def loro(self) -> I16Vector4: ...
    @property
    def lorl(self) -> I16Vector4: ...
    @property
    def logr(self) -> I16Vector4: ...
    @property
    def logg(self) -> I16Vector4: ...
    @property
    def logo(self) -> I16Vector4: ...
    @property
    def logl(self) -> I16Vector4: ...
    @property
    def loor(self) -> I16Vector4: ...
    @property
    def loog(self) -> I16Vector4: ...
    @property
    def lolr(self) -> I16Vector4: ...
    @property
    def lolg(self) -> I16Vector4: ...
    @property
    def llrr(self) -> I16Vector4: ...
    @property
    def llrg(self) -> I16Vector4: ...
    @property
    def llro(self) -> I16Vector4: ...
    @property
    def llrl(self) -> I16Vector4: ...
    @property
    def llgr(self) -> I16Vector4: ...
    @property
    def llgg(self) -> I16Vector4: ...
    @property
    def llgo(self) -> I16Vector4: ...
    @property
    def llgl(self) -> I16Vector4: ...
    @property
    def llor(self) -> I16Vector4: ...
    @property
    def llog(self) -> I16Vector4: ...
    @property
    def lllr(self) -> I16Vector4: ...
    @property
    def lllg(self) -> I16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> I16Vector2: ...
    @property
    def st(self) -> I16Vector2: ...
    @property
    def so(self) -> I16Vector2: ...
    @property
    def sl(self) -> I16Vector2: ...
    @property
    def ts(self) -> I16Vector2: ...
    @property
    def tt(self) -> I16Vector2: ...
    @property
    def to(self) -> I16Vector2: ...
    @property
    def tl(self) -> I16Vector2: ...
    @property
    def os(self) -> I16Vector2: ...
    @property
    def ot(self) -> I16Vector2: ...
    @property
    def ls(self) -> I16Vector2: ...
    @property
    def lt(self) -> I16Vector2: ...
    @property
    def sss(self) -> I16Vector3: ...
    @property
    def sst(self) -> I16Vector3: ...
    @property
    def sso(self) -> I16Vector3: ...
    @property
    def ssl(self) -> I16Vector3: ...
    @property
    def sts(self) -> I16Vector3: ...
    @property
    def stt(self) -> I16Vector3: ...
    @property
    def sto(self) -> I16Vector3: ...
    @property
    def stl(self) -> I16Vector3: ...
    @property
    def sos(self) -> I16Vector3: ...
    @property
    def sot(self) -> I16Vector3: ...
    @property
    def soo(self) -> I16Vector3: ...
    @property
    def sol(self) -> I16Vector3: ...
    @property
    def sls(self) -> I16Vector3: ...
    @property
    def slt(self) -> I16Vector3: ...
    @property
    def slo(self) -> I16Vector3: ...
    @property
    def sll(self) -> I16Vector3: ...
    @property
    def tss(self) -> I16Vector3: ...
    @property
    def tst(self) -> I16Vector3: ...
    @property
    def tso(self) -> I16Vector3: ...
    @property
    def tsl(self) -> I16Vector3: ...
    @property
    def tts(self) -> I16Vector3: ...
    @property
    def ttt(self) -> I16Vector3: ...
    @property
    def tto(self) -> I16Vector3: ...
    @property
    def ttl(self) -> I16Vector3: ...
    @property
    def tos(self) -> I16Vector3: ...
    @property
    def tot(self) -> I16Vector3: ...
    @property
    def too(self) -> I16Vector3: ...
    @property
    def tol(self) -> I16Vector3: ...
    @property
    def tls(self) -> I16Vector3: ...
    @property
    def tlt(self) -> I16Vector3: ...
    @property
    def tlo(self) -> I16Vector3: ...
    @property
    def tll(self) -> I16Vector3: ...
    @property
    def oss(self) -> I16Vector3: ...
    @property
    def ost(self) -> I16Vector3: ...
    @property
    def oso(self) -> I16Vector3: ...
    @property
    def osl(self) -> I16Vector3: ...
    @property
    def ots(self) -> I16Vector3: ...
    @property
    def ott(self) -> I16Vector3: ...
    @property
    def oto(self) -> I16Vector3: ...
    @property
    def otl(self) -> I16Vector3: ...
    @property
    def oos(self) -> I16Vector3: ...
    @property
    def oot(self) -> I16Vector3: ...
    @property
    def ols(self) -> I16Vector3: ...
    @property
    def olt(self) -> I16Vector3: ...
    @property
    def lss(self) -> I16Vector3: ...
    @property
    def lst(self) -> I16Vector3: ...
    @property
    def lso(self) -> I16Vector3: ...
    @property
    def lsl(self) -> I16Vector3: ...
    @property
    def lts(self) -> I16Vector3: ...
    @property
    def ltt(self) -> I16Vector3: ...
    @property
    def lto(self) -> I16Vector3: ...
    @property
    def ltl(self) -> I16Vector3: ...
    @property
    def los(self) -> I16Vector3: ...
    @property
    def lot(self) -> I16Vector3: ...
    @property
    def lls(self) -> I16Vector3: ...
    @property
    def llt(self) -> I16Vector3: ...
    @property
    def ssss(self) -> I16Vector4: ...
    @property
    def ssst(self) -> I16Vector4: ...
    @property
    def ssso(self) -> I16Vector4: ...
    @property
    def sssl(self) -> I16Vector4: ...
    @property
    def ssts(self) -> I16Vector4: ...
    @property
    def sstt(self) -> I16Vector4: ...
    @property
    def ssto(self) -> I16Vector4: ...
    @property
    def sstl(self) -> I16Vector4: ...
    @property
    def ssos(self) -> I16Vector4: ...
    @property
    def ssot(self) -> I16Vector4: ...
    @property
    def ssoo(self) -> I16Vector4: ...
    @property
    def ssol(self) -> I16Vector4: ...
    @property
    def ssls(self) -> I16Vector4: ...
    @property
    def sslt(self) -> I16Vector4: ...
    @property
    def sslo(self) -> I16Vector4: ...
    @property
    def ssll(self) -> I16Vector4: ...
    @property
    def stss(self) -> I16Vector4: ...
    @property
    def stst(self) -> I16Vector4: ...
    @property
    def stso(self) -> I16Vector4: ...
    @property
    def stsl(self) -> I16Vector4: ...
    @property
    def stts(self) -> I16Vector4: ...
    @property
    def sttt(self) -> I16Vector4: ...
    @property
    def stto(self) -> I16Vector4: ...
    @property
    def sttl(self) -> I16Vector4: ...
    @property
    def stos(self) -> I16Vector4: ...
    @property
    def stot(self) -> I16Vector4: ...
    @property
    def stoo(self) -> I16Vector4: ...
    @property
    def stol(self) -> I16Vector4: ...
    @property
    def stls(self) -> I16Vector4: ...
    @property
    def stlt(self) -> I16Vector4: ...
    @property
    def stlo(self) -> I16Vector4: ...
    @property
    def stll(self) -> I16Vector4: ...
    @property
    def soss(self) -> I16Vector4: ...
    @property
    def sost(self) -> I16Vector4: ...
    @property
    def soso(self) -> I16Vector4: ...
    @property
    def sosl(self) -> I16Vector4: ...
    @property
    def sots(self) -> I16Vector4: ...
    @property
    def sott(self) -> I16Vector4: ...
    @property
    def soto(self) -> I16Vector4: ...
    @property
    def sotl(self) -> I16Vector4: ...
    @property
    def soos(self) -> I16Vector4: ...
    @property
    def soot(self) -> I16Vector4: ...
    @property
    def sooo(self) -> I16Vector4: ...
    @property
    def sool(self) -> I16Vector4: ...
    @property
    def sols(self) -> I16Vector4: ...
    @property
    def solt(self) -> I16Vector4: ...
    @property
    def solo(self) -> I16Vector4: ...
    @property
    def soll(self) -> I16Vector4: ...
    @property
    def slss(self) -> I16Vector4: ...
    @property
    def slst(self) -> I16Vector4: ...
    @property
    def slso(self) -> I16Vector4: ...
    @property
    def slsl(self) -> I16Vector4: ...
    @property
    def slts(self) -> I16Vector4: ...
    @property
    def sltt(self) -> I16Vector4: ...
    @property
    def slto(self) -> I16Vector4: ...
    @property
    def sltl(self) -> I16Vector4: ...
    @property
    def slos(self) -> I16Vector4: ...
    @property
    def slot(self) -> I16Vector4: ...
    @property
    def sloo(self) -> I16Vector4: ...
    @property
    def slol(self) -> I16Vector4: ...
    @property
    def slls(self) -> I16Vector4: ...
    @property
    def sllt(self) -> I16Vector4: ...
    @property
    def sllo(self) -> I16Vector4: ...
    @property
    def slll(self) -> I16Vector4: ...
    @property
    def tsss(self) -> I16Vector4: ...
    @property
    def tsst(self) -> I16Vector4: ...
    @property
    def tsso(self) -> I16Vector4: ...
    @property
    def tssl(self) -> I16Vector4: ...
    @property
    def tsts(self) -> I16Vector4: ...
    @property
    def tstt(self) -> I16Vector4: ...
    @property
    def tsto(self) -> I16Vector4: ...
    @property
    def tstl(self) -> I16Vector4: ...
    @property
    def tsos(self) -> I16Vector4: ...
    @property
    def tsot(self) -> I16Vector4: ...
    @property
    def tsoo(self) -> I16Vector4: ...
    @property
    def tsol(self) -> I16Vector4: ...
    @property
    def tsls(self) -> I16Vector4: ...
    @property
    def tslt(self) -> I16Vector4: ...
    @property
    def tslo(self) -> I16Vector4: ...
    @property
    def tsll(self) -> I16Vector4: ...
    @property
    def ttss(self) -> I16Vector4: ...
    @property
    def ttst(self) -> I16Vector4: ...
    @property
    def ttso(self) -> I16Vector4: ...
    @property
    def ttsl(self) -> I16Vector4: ...
    @property
    def ttts(self) -> I16Vector4: ...
    @property
    def tttt(self) -> I16Vector4: ...
    @property
    def ttto(self) -> I16Vector4: ...
    @property
    def tttl(self) -> I16Vector4: ...
    @property
    def ttos(self) -> I16Vector4: ...
    @property
    def ttot(self) -> I16Vector4: ...
    @property
    def ttoo(self) -> I16Vector4: ...
    @property
    def ttol(self) -> I16Vector4: ...
    @property
    def ttls(self) -> I16Vector4: ...
    @property
    def ttlt(self) -> I16Vector4: ...
    @property
    def ttlo(self) -> I16Vector4: ...
    @property
    def ttll(self) -> I16Vector4: ...
    @property
    def toss(self) -> I16Vector4: ...
    @property
    def tost(self) -> I16Vector4: ...
    @property
    def toso(self) -> I16Vector4: ...
    @property
    def tosl(self) -> I16Vector4: ...
    @property
    def tots(self) -> I16Vector4: ...
    @property
    def tott(self) -> I16Vector4: ...
    @property
    def toto(self) -> I16Vector4: ...
    @property
    def totl(self) -> I16Vector4: ...
    @property
    def toos(self) -> I16Vector4: ...
    @property
    def toot(self) -> I16Vector4: ...
    @property
    def tooo(self) -> I16Vector4: ...
    @property
    def tool(self) -> I16Vector4: ...
    @property
    def tols(self) -> I16Vector4: ...
    @property
    def tolt(self) -> I16Vector4: ...
    @property
    def tolo(self) -> I16Vector4: ...
    @property
    def toll(self) -> I16Vector4: ...
    @property
    def tlss(self) -> I16Vector4: ...
    @property
    def tlst(self) -> I16Vector4: ...
    @property
    def tlso(self) -> I16Vector4: ...
    @property
    def tlsl(self) -> I16Vector4: ...
    @property
    def tlts(self) -> I16Vector4: ...
    @property
    def tltt(self) -> I16Vector4: ...
    @property
    def tlto(self) -> I16Vector4: ...
    @property
    def tltl(self) -> I16Vector4: ...
    @property
    def tlos(self) -> I16Vector4: ...
    @property
    def tlot(self) -> I16Vector4: ...
    @property
    def tloo(self) -> I16Vector4: ...
    @property
    def tlol(self) -> I16Vector4: ...
    @property
    def tlls(self) -> I16Vector4: ...
    @property
    def tllt(self) -> I16Vector4: ...
    @property
    def tllo(self) -> I16Vector4: ...
    @property
    def tlll(self) -> I16Vector4: ...
    @property
    def osss(self) -> I16Vector4: ...
    @property
    def osst(self) -> I16Vector4: ...
    @property
    def osso(self) -> I16Vector4: ...
    @property
    def ossl(self) -> I16Vector4: ...
    @property
    def osts(self) -> I16Vector4: ...
    @property
    def ostt(self) -> I16Vector4: ...
    @property
    def osto(self) -> I16Vector4: ...
    @property
    def ostl(self) -> I16Vector4: ...
    @property
    def osos(self) -> I16Vector4: ...
    @property
    def osot(self) -> I16Vector4: ...
    @property
    def osoo(self) -> I16Vector4: ...
    @property
    def osol(self) -> I16Vector4: ...
    @property
    def osls(self) -> I16Vector4: ...
    @property
    def oslt(self) -> I16Vector4: ...
    @property
    def oslo(self) -> I16Vector4: ...
    @property
    def osll(self) -> I16Vector4: ...
    @property
    def otss(self) -> I16Vector4: ...
    @property
    def otst(self) -> I16Vector4: ...
    @property
    def otso(self) -> I16Vector4: ...
    @property
    def otsl(self) -> I16Vector4: ...
    @property
    def otts(self) -> I16Vector4: ...
    @property
    def ottt(self) -> I16Vector4: ...
    @property
    def otto(self) -> I16Vector4: ...
    @property
    def ottl(self) -> I16Vector4: ...
    @property
    def otos(self) -> I16Vector4: ...
    @property
    def otot(self) -> I16Vector4: ...
    @property
    def otoo(self) -> I16Vector4: ...
    @property
    def otol(self) -> I16Vector4: ...
    @property
    def otls(self) -> I16Vector4: ...
    @property
    def otlt(self) -> I16Vector4: ...
    @property
    def otlo(self) -> I16Vector4: ...
    @property
    def otll(self) -> I16Vector4: ...
    @property
    def ooss(self) -> I16Vector4: ...
    @property
    def oost(self) -> I16Vector4: ...
    @property
    def ooso(self) -> I16Vector4: ...
    @property
    def oosl(self) -> I16Vector4: ...
    @property
    def oots(self) -> I16Vector4: ...
    @property
    def oott(self) -> I16Vector4: ...
    @property
    def ooto(self) -> I16Vector4: ...
    @property
    def ootl(self) -> I16Vector4: ...
    @property
    def ooos(self) -> I16Vector4: ...
    @property
    def ooot(self) -> I16Vector4: ...
    @property
    def ools(self) -> I16Vector4: ...
    @property
    def oolt(self) -> I16Vector4: ...
    @property
    def olss(self) -> I16Vector4: ...
    @property
    def olst(self) -> I16Vector4: ...
    @property
    def olso(self) -> I16Vector4: ...
    @property
    def olsl(self) -> I16Vector4: ...
    @property
    def olts(self) -> I16Vector4: ...
    @property
    def oltt(self) -> I16Vector4: ...
    @property
    def olto(self) -> I16Vector4: ...
    @property
    def oltl(self) -> I16Vector4: ...
    @property
    def olos(self) -> I16Vector4: ...
    @property
    def olot(self) -> I16Vector4: ...
    @property
    def olls(self) -> I16Vector4: ...
    @property
    def ollt(self) -> I16Vector4: ...
    @property
    def lsss(self) -> I16Vector4: ...
    @property
    def lsst(self) -> I16Vector4: ...
    @property
    def lsso(self) -> I16Vector4: ...
    @property
    def lssl(self) -> I16Vector4: ...
    @property
    def lsts(self) -> I16Vector4: ...
    @property
    def lstt(self) -> I16Vector4: ...
    @property
    def lsto(self) -> I16Vector4: ...
    @property
    def lstl(self) -> I16Vector4: ...
    @property
    def lsos(self) -> I16Vector4: ...
    @property
    def lsot(self) -> I16Vector4: ...
    @property
    def lsoo(self) -> I16Vector4: ...
    @property
    def lsol(self) -> I16Vector4: ...
    @property
    def lsls(self) -> I16Vector4: ...
    @property
    def lslt(self) -> I16Vector4: ...
    @property
    def lslo(self) -> I16Vector4: ...
    @property
    def lsll(self) -> I16Vector4: ...
    @property
    def ltss(self) -> I16Vector4: ...
    @property
    def ltst(self) -> I16Vector4: ...
    @property
    def ltso(self) -> I16Vector4: ...
    @property
    def ltsl(self) -> I16Vector4: ...
    @property
    def ltts(self) -> I16Vector4: ...
    @property
    def lttt(self) -> I16Vector4: ...
    @property
    def ltto(self) -> I16Vector4: ...
    @property
    def lttl(self) -> I16Vector4: ...
    @property
    def ltos(self) -> I16Vector4: ...
    @property
    def ltot(self) -> I16Vector4: ...
    @property
    def ltoo(self) -> I16Vector4: ...
    @property
    def ltol(self) -> I16Vector4: ...
    @property
    def ltls(self) -> I16Vector4: ...
    @property
    def ltlt(self) -> I16Vector4: ...
    @property
    def ltlo(self) -> I16Vector4: ...
    @property
    def ltll(self) -> I16Vector4: ...
    @property
    def loss(self) -> I16Vector4: ...
    @property
    def lost(self) -> I16Vector4: ...
    @property
    def loso(self) -> I16Vector4: ...
    @property
    def losl(self) -> I16Vector4: ...
    @property
    def lots(self) -> I16Vector4: ...
    @property
    def lott(self) -> I16Vector4: ...
    @property
    def loto(self) -> I16Vector4: ...
    @property
    def lotl(self) -> I16Vector4: ...
    @property
    def loos(self) -> I16Vector4: ...
    @property
    def loot(self) -> I16Vector4: ...
    @property
    def lols(self) -> I16Vector4: ...
    @property
    def lolt(self) -> I16Vector4: ...
    @property
    def llss(self) -> I16Vector4: ...
    @property
    def llst(self) -> I16Vector4: ...
    @property
    def llso(self) -> I16Vector4: ...
    @property
    def llsl(self) -> I16Vector4: ...
    @property
    def llts(self) -> I16Vector4: ...
    @property
    def lltt(self) -> I16Vector4: ...
    @property
    def llto(self) -> I16Vector4: ...
    @property
    def lltl(self) -> I16Vector4: ...
    @property
    def llos(self) -> I16Vector4: ...
    @property
    def llot(self) -> I16Vector4: ...
    @property
    def llls(self) -> I16Vector4: ...
    @property
    def lllt(self) -> I16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I16Vector2: ...
    @property
    def uv(self) -> I16Vector2: ...
    @property
    def uo(self) -> I16Vector2: ...
    @property
    def ul(self) -> I16Vector2: ...
    @property
    def vu(self) -> I16Vector2: ...
    @property
    def vv(self) -> I16Vector2: ...
    @property
    def vo(self) -> I16Vector2: ...
    @property
    def vl(self) -> I16Vector2: ...
    @property
    def ou(self) -> I16Vector2: ...
    @property
    def ov(self) -> I16Vector2: ...
    @property
    def lu(self) -> I16Vector2: ...
    @property
    def lv(self) -> I16Vector2: ...
    @property
    def uuu(self) -> I16Vector3: ...
    @property
    def uuv(self) -> I16Vector3: ...
    @property
    def uuo(self) -> I16Vector3: ...
    @property
    def uul(self) -> I16Vector3: ...
    @property
    def uvu(self) -> I16Vector3: ...
    @property
    def uvv(self) -> I16Vector3: ...
    @property
    def uvo(self) -> I16Vector3: ...
    @property
    def uvl(self) -> I16Vector3: ...
    @property
    def uou(self) -> I16Vector3: ...
    @property
    def uov(self) -> I16Vector3: ...
    @property
    def uoo(self) -> I16Vector3: ...
    @property
    def uol(self) -> I16Vector3: ...
    @property
    def ulu(self) -> I16Vector3: ...
    @property
    def ulv(self) -> I16Vector3: ...
    @property
    def ulo(self) -> I16Vector3: ...
    @property
    def ull(self) -> I16Vector3: ...
    @property
    def vuu(self) -> I16Vector3: ...
    @property
    def vuv(self) -> I16Vector3: ...
    @property
    def vuo(self) -> I16Vector3: ...
    @property
    def vul(self) -> I16Vector3: ...
    @property
    def vvu(self) -> I16Vector3: ...
    @property
    def vvv(self) -> I16Vector3: ...
    @property
    def vvo(self) -> I16Vector3: ...
    @property
    def vvl(self) -> I16Vector3: ...
    @property
    def vou(self) -> I16Vector3: ...
    @property
    def vov(self) -> I16Vector3: ...
    @property
    def voo(self) -> I16Vector3: ...
    @property
    def vol(self) -> I16Vector3: ...
    @property
    def vlu(self) -> I16Vector3: ...
    @property
    def vlv(self) -> I16Vector3: ...
    @property
    def vlo(self) -> I16Vector3: ...
    @property
    def vll(self) -> I16Vector3: ...
    @property
    def ouu(self) -> I16Vector3: ...
    @property
    def ouv(self) -> I16Vector3: ...
    @property
    def ouo(self) -> I16Vector3: ...
    @property
    def oul(self) -> I16Vector3: ...
    @property
    def ovu(self) -> I16Vector3: ...
    @property
    def ovv(self) -> I16Vector3: ...
    @property
    def ovo(self) -> I16Vector3: ...
    @property
    def ovl(self) -> I16Vector3: ...
    @property
    def oou(self) -> I16Vector3: ...
    @property
    def oov(self) -> I16Vector3: ...
    @property
    def olu(self) -> I16Vector3: ...
    @property
    def olv(self) -> I16Vector3: ...
    @property
    def luu(self) -> I16Vector3: ...
    @property
    def luv(self) -> I16Vector3: ...
    @property
    def luo(self) -> I16Vector3: ...
    @property
    def lul(self) -> I16Vector3: ...
    @property
    def lvu(self) -> I16Vector3: ...
    @property
    def lvv(self) -> I16Vector3: ...
    @property
    def lvo(self) -> I16Vector3: ...
    @property
    def lvl(self) -> I16Vector3: ...
    @property
    def lou(self) -> I16Vector3: ...
    @property
    def lov(self) -> I16Vector3: ...
    @property
    def llu(self) -> I16Vector3: ...
    @property
    def llv(self) -> I16Vector3: ...
    @property
    def uuuu(self) -> I16Vector4: ...
    @property
    def uuuv(self) -> I16Vector4: ...
    @property
    def uuuo(self) -> I16Vector4: ...
    @property
    def uuul(self) -> I16Vector4: ...
    @property
    def uuvu(self) -> I16Vector4: ...
    @property
    def uuvv(self) -> I16Vector4: ...
    @property
    def uuvo(self) -> I16Vector4: ...
    @property
    def uuvl(self) -> I16Vector4: ...
    @property
    def uuou(self) -> I16Vector4: ...
    @property
    def uuov(self) -> I16Vector4: ...
    @property
    def uuoo(self) -> I16Vector4: ...
    @property
    def uuol(self) -> I16Vector4: ...
    @property
    def uulu(self) -> I16Vector4: ...
    @property
    def uulv(self) -> I16Vector4: ...
    @property
    def uulo(self) -> I16Vector4: ...
    @property
    def uull(self) -> I16Vector4: ...
    @property
    def uvuu(self) -> I16Vector4: ...
    @property
    def uvuv(self) -> I16Vector4: ...
    @property
    def uvuo(self) -> I16Vector4: ...
    @property
    def uvul(self) -> I16Vector4: ...
    @property
    def uvvu(self) -> I16Vector4: ...
    @property
    def uvvv(self) -> I16Vector4: ...
    @property
    def uvvo(self) -> I16Vector4: ...
    @property
    def uvvl(self) -> I16Vector4: ...
    @property
    def uvou(self) -> I16Vector4: ...
    @property
    def uvov(self) -> I16Vector4: ...
    @property
    def uvoo(self) -> I16Vector4: ...
    @property
    def uvol(self) -> I16Vector4: ...
    @property
    def uvlu(self) -> I16Vector4: ...
    @property
    def uvlv(self) -> I16Vector4: ...
    @property
    def uvlo(self) -> I16Vector4: ...
    @property
    def uvll(self) -> I16Vector4: ...
    @property
    def uouu(self) -> I16Vector4: ...
    @property
    def uouv(self) -> I16Vector4: ...
    @property
    def uouo(self) -> I16Vector4: ...
    @property
    def uoul(self) -> I16Vector4: ...
    @property
    def uovu(self) -> I16Vector4: ...
    @property
    def uovv(self) -> I16Vector4: ...
    @property
    def uovo(self) -> I16Vector4: ...
    @property
    def uovl(self) -> I16Vector4: ...
    @property
    def uoou(self) -> I16Vector4: ...
    @property
    def uoov(self) -> I16Vector4: ...
    @property
    def uooo(self) -> I16Vector4: ...
    @property
    def uool(self) -> I16Vector4: ...
    @property
    def uolu(self) -> I16Vector4: ...
    @property
    def uolv(self) -> I16Vector4: ...
    @property
    def uolo(self) -> I16Vector4: ...
    @property
    def uoll(self) -> I16Vector4: ...
    @property
    def uluu(self) -> I16Vector4: ...
    @property
    def uluv(self) -> I16Vector4: ...
    @property
    def uluo(self) -> I16Vector4: ...
    @property
    def ulul(self) -> I16Vector4: ...
    @property
    def ulvu(self) -> I16Vector4: ...
    @property
    def ulvv(self) -> I16Vector4: ...
    @property
    def ulvo(self) -> I16Vector4: ...
    @property
    def ulvl(self) -> I16Vector4: ...
    @property
    def ulou(self) -> I16Vector4: ...
    @property
    def ulov(self) -> I16Vector4: ...
    @property
    def uloo(self) -> I16Vector4: ...
    @property
    def ulol(self) -> I16Vector4: ...
    @property
    def ullu(self) -> I16Vector4: ...
    @property
    def ullv(self) -> I16Vector4: ...
    @property
    def ullo(self) -> I16Vector4: ...
    @property
    def ulll(self) -> I16Vector4: ...
    @property
    def vuuu(self) -> I16Vector4: ...
    @property
    def vuuv(self) -> I16Vector4: ...
    @property
    def vuuo(self) -> I16Vector4: ...
    @property
    def vuul(self) -> I16Vector4: ...
    @property
    def vuvu(self) -> I16Vector4: ...
    @property
    def vuvv(self) -> I16Vector4: ...
    @property
    def vuvo(self) -> I16Vector4: ...
    @property
    def vuvl(self) -> I16Vector4: ...
    @property
    def vuou(self) -> I16Vector4: ...
    @property
    def vuov(self) -> I16Vector4: ...
    @property
    def vuoo(self) -> I16Vector4: ...
    @property
    def vuol(self) -> I16Vector4: ...
    @property
    def vulu(self) -> I16Vector4: ...
    @property
    def vulv(self) -> I16Vector4: ...
    @property
    def vulo(self) -> I16Vector4: ...
    @property
    def vull(self) -> I16Vector4: ...
    @property
    def vvuu(self) -> I16Vector4: ...
    @property
    def vvuv(self) -> I16Vector4: ...
    @property
    def vvuo(self) -> I16Vector4: ...
    @property
    def vvul(self) -> I16Vector4: ...
    @property
    def vvvu(self) -> I16Vector4: ...
    @property
    def vvvv(self) -> I16Vector4: ...
    @property
    def vvvo(self) -> I16Vector4: ...
    @property
    def vvvl(self) -> I16Vector4: ...
    @property
    def vvou(self) -> I16Vector4: ...
    @property
    def vvov(self) -> I16Vector4: ...
    @property
    def vvoo(self) -> I16Vector4: ...
    @property
    def vvol(self) -> I16Vector4: ...
    @property
    def vvlu(self) -> I16Vector4: ...
    @property
    def vvlv(self) -> I16Vector4: ...
    @property
    def vvlo(self) -> I16Vector4: ...
    @property
    def vvll(self) -> I16Vector4: ...
    @property
    def vouu(self) -> I16Vector4: ...
    @property
    def vouv(self) -> I16Vector4: ...
    @property
    def vouo(self) -> I16Vector4: ...
    @property
    def voul(self) -> I16Vector4: ...
    @property
    def vovu(self) -> I16Vector4: ...
    @property
    def vovv(self) -> I16Vector4: ...
    @property
    def vovo(self) -> I16Vector4: ...
    @property
    def vovl(self) -> I16Vector4: ...
    @property
    def voou(self) -> I16Vector4: ...
    @property
    def voov(self) -> I16Vector4: ...
    @property
    def vooo(self) -> I16Vector4: ...
    @property
    def vool(self) -> I16Vector4: ...
    @property
    def volu(self) -> I16Vector4: ...
    @property
    def volv(self) -> I16Vector4: ...
    @property
    def volo(self) -> I16Vector4: ...
    @property
    def voll(self) -> I16Vector4: ...
    @property
    def vluu(self) -> I16Vector4: ...
    @property
    def vluv(self) -> I16Vector4: ...
    @property
    def vluo(self) -> I16Vector4: ...
    @property
    def vlul(self) -> I16Vector4: ...
    @property
    def vlvu(self) -> I16Vector4: ...
    @property
    def vlvv(self) -> I16Vector4: ...
    @property
    def vlvo(self) -> I16Vector4: ...
    @property
    def vlvl(self) -> I16Vector4: ...
    @property
    def vlou(self) -> I16Vector4: ...
    @property
    def vlov(self) -> I16Vector4: ...
    @property
    def vloo(self) -> I16Vector4: ...
    @property
    def vlol(self) -> I16Vector4: ...
    @property
    def vllu(self) -> I16Vector4: ...
    @property
    def vllv(self) -> I16Vector4: ...
    @property
    def vllo(self) -> I16Vector4: ...
    @property
    def vlll(self) -> I16Vector4: ...
    @property
    def ouuu(self) -> I16Vector4: ...
    @property
    def ouuv(self) -> I16Vector4: ...
    @property
    def ouuo(self) -> I16Vector4: ...
    @property
    def ouul(self) -> I16Vector4: ...
    @property
    def ouvu(self) -> I16Vector4: ...
    @property
    def ouvv(self) -> I16Vector4: ...
    @property
    def ouvo(self) -> I16Vector4: ...
    @property
    def ouvl(self) -> I16Vector4: ...
    @property
    def ouou(self) -> I16Vector4: ...
    @property
    def ouov(self) -> I16Vector4: ...
    @property
    def ouoo(self) -> I16Vector4: ...
    @property
    def ouol(self) -> I16Vector4: ...
    @property
    def oulu(self) -> I16Vector4: ...
    @property
    def oulv(self) -> I16Vector4: ...
    @property
    def oulo(self) -> I16Vector4: ...
    @property
    def oull(self) -> I16Vector4: ...
    @property
    def ovuu(self) -> I16Vector4: ...
    @property
    def ovuv(self) -> I16Vector4: ...
    @property
    def ovuo(self) -> I16Vector4: ...
    @property
    def ovul(self) -> I16Vector4: ...
    @property
    def ovvu(self) -> I16Vector4: ...
    @property
    def ovvv(self) -> I16Vector4: ...
    @property
    def ovvo(self) -> I16Vector4: ...
    @property
    def ovvl(self) -> I16Vector4: ...
    @property
    def ovou(self) -> I16Vector4: ...
    @property
    def ovov(self) -> I16Vector4: ...
    @property
    def ovoo(self) -> I16Vector4: ...
    @property
    def ovol(self) -> I16Vector4: ...
    @property
    def ovlu(self) -> I16Vector4: ...
    @property
    def ovlv(self) -> I16Vector4: ...
    @property
    def ovlo(self) -> I16Vector4: ...
    @property
    def ovll(self) -> I16Vector4: ...
    @property
    def oouu(self) -> I16Vector4: ...
    @property
    def oouv(self) -> I16Vector4: ...
    @property
    def oouo(self) -> I16Vector4: ...
    @property
    def ooul(self) -> I16Vector4: ...
    @property
    def oovu(self) -> I16Vector4: ...
    @property
    def oovv(self) -> I16Vector4: ...
    @property
    def oovo(self) -> I16Vector4: ...
    @property
    def oovl(self) -> I16Vector4: ...
    @property
    def ooou(self) -> I16Vector4: ...
    @property
    def ooov(self) -> I16Vector4: ...
    @property
    def oolu(self) -> I16Vector4: ...
    @property
    def oolv(self) -> I16Vector4: ...
    @property
    def oluu(self) -> I16Vector4: ...
    @property
    def oluv(self) -> I16Vector4: ...
    @property
    def oluo(self) -> I16Vector4: ...
    @property
    def olul(self) -> I16Vector4: ...
    @property
    def olvu(self) -> I16Vector4: ...
    @property
    def olvv(self) -> I16Vector4: ...
    @property
    def olvo(self) -> I16Vector4: ...
    @property
    def olvl(self) -> I16Vector4: ...
    @property
    def olou(self) -> I16Vector4: ...
    @property
    def olov(self) -> I16Vector4: ...
    @property
    def ollu(self) -> I16Vector4: ...
    @property
    def ollv(self) -> I16Vector4: ...
    @property
    def luuu(self) -> I16Vector4: ...
    @property
    def luuv(self) -> I16Vector4: ...
    @property
    def luuo(self) -> I16Vector4: ...
    @property
    def luul(self) -> I16Vector4: ...
    @property
    def luvu(self) -> I16Vector4: ...
    @property
    def luvv(self) -> I16Vector4: ...
    @property
    def luvo(self) -> I16Vector4: ...
    @property
    def luvl(self) -> I16Vector4: ...
    @property
    def luou(self) -> I16Vector4: ...
    @property
    def luov(self) -> I16Vector4: ...
    @property
    def luoo(self) -> I16Vector4: ...
    @property
    def luol(self) -> I16Vector4: ...
    @property
    def lulu(self) -> I16Vector4: ...
    @property
    def lulv(self) -> I16Vector4: ...
    @property
    def lulo(self) -> I16Vector4: ...
    @property
    def lull(self) -> I16Vector4: ...
    @property
    def lvuu(self) -> I16Vector4: ...
    @property
    def lvuv(self) -> I16Vector4: ...
    @property
    def lvuo(self) -> I16Vector4: ...
    @property
    def lvul(self) -> I16Vector4: ...
    @property
    def lvvu(self) -> I16Vector4: ...
    @property
    def lvvv(self) -> I16Vector4: ...
    @property
    def lvvo(self) -> I16Vector4: ...
    @property
    def lvvl(self) -> I16Vector4: ...
    @property
    def lvou(self) -> I16Vector4: ...
    @property
    def lvov(self) -> I16Vector4: ...
    @property
    def lvoo(self) -> I16Vector4: ...
    @property
    def lvol(self) -> I16Vector4: ...
    @property
    def lvlu(self) -> I16Vector4: ...
    @property
    def lvlv(self) -> I16Vector4: ...
    @property
    def lvlo(self) -> I16Vector4: ...
    @property
    def lvll(self) -> I16Vector4: ...
    @property
    def louu(self) -> I16Vector4: ...
    @property
    def louv(self) -> I16Vector4: ...
    @property
    def louo(self) -> I16Vector4: ...
    @property
    def loul(self) -> I16Vector4: ...
    @property
    def lovu(self) -> I16Vector4: ...
    @property
    def lovv(self) -> I16Vector4: ...
    @property
    def lovo(self) -> I16Vector4: ...
    @property
    def lovl(self) -> I16Vector4: ...
    @property
    def loou(self) -> I16Vector4: ...
    @property
    def loov(self) -> I16Vector4: ...
    @property
    def lolu(self) -> I16Vector4: ...
    @property
    def lolv(self) -> I16Vector4: ...
    @property
    def lluu(self) -> I16Vector4: ...
    @property
    def lluv(self) -> I16Vector4: ...
    @property
    def lluo(self) -> I16Vector4: ...
    @property
    def llul(self) -> I16Vector4: ...
    @property
    def llvu(self) -> I16Vector4: ...
    @property
    def llvv(self) -> I16Vector4: ...
    @property
    def llvo(self) -> I16Vector4: ...
    @property
    def llvl(self) -> I16Vector4: ...
    @property
    def llou(self) -> I16Vector4: ...
    @property
    def llov(self) -> I16Vector4: ...
    @property
    def lllu(self) -> I16Vector4: ...
    @property
    def lllv(self) -> I16Vector4: ...
    @property
    def oo(self) -> I16Vector2: ...
    @property
    def ol(self) -> I16Vector2: ...
    @property
    def lo(self) -> I16Vector2: ...
    @property
    def ll(self) -> I16Vector2: ...
    @property
    def ooo(self) -> I16Vector3: ...
    @property
    def ool(self) -> I16Vector3: ...
    @property
    def olo(self) -> I16Vector3: ...
    @property
    def oll(self) -> I16Vector3: ...
    @property
    def loo(self) -> I16Vector3: ...
    @property
    def lol(self) -> I16Vector3: ...
    @property
    def llo(self) -> I16Vector3: ...
    @property
    def lll(self) -> I16Vector3: ...
    @property
    def oooo(self) -> I16Vector4: ...
    @property
    def oool(self) -> I16Vector4: ...
    @property
    def oolo(self) -> I16Vector4: ...
    @property
    def ooll(self) -> I16Vector4: ...
    @property
    def oloo(self) -> I16Vector4: ...
    @property
    def olol(self) -> I16Vector4: ...
    @property
    def ollo(self) -> I16Vector4: ...
    @property
    def olll(self) -> I16Vector4: ...
    @property
    def looo(self) -> I16Vector4: ...
    @property
    def lool(self) -> I16Vector4: ...
    @property
    def lolo(self) -> I16Vector4: ...
    @property
    def loll(self) -> I16Vector4: ...
    @property
    def lloo(self) -> I16Vector4: ...
    @property
    def llol(self) -> I16Vector4: ...
    @property
    def lllo(self) -> I16Vector4: ...
    @property
    def llll(self) -> I16Vector4: ...
    def min(self, n: Number, /) -> I16Vector2: ...
    def max(self, n: Number, /) -> I16Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> I16Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I16Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector2: ...

@final
class I16Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I16Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I16Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> I16Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> I16Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[I16Vector2]: ...

@final
class U16Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __add__(self, other: Number) -> U16Vector2: ...
    @overload
    def __radd__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __radd__(self, other: Number) -> U16Vector2: ...
    @overload
    def __sub__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __sub__(self, other: Number) -> U16Vector2: ...
    @overload
    def __rsub__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> U16Vector2: ...
    @overload
    def __mul__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __mul__(self, other: Number) -> U16Vector2: ...
    @overload
    def __rmul__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> U16Vector2: ...
    @overload
    def __truediv__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> U16Vector2: ...
    @overload
    def __rtruediv__(self, other: U16Vector2) -> U16Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> U16Vector2: ...
    def __abs__(self) -> U16Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> U16Vector2: ...
    @property
    def xy(self) -> U16Vector2: ...
    @property
    def xo(self) -> U16Vector2: ...
    @property
    def xl(self) -> U16Vector2: ...
    @property
    def yx(self) -> U16Vector2: ...
    @property
    def yy(self) -> U16Vector2: ...
    @property
    def yo(self) -> U16Vector2: ...
    @property
    def yl(self) -> U16Vector2: ...
    @property
    def ox(self) -> U16Vector2: ...
    @property
    def oy(self) -> U16Vector2: ...
    @property
    def lx(self) -> U16Vector2: ...
    @property
    def ly(self) -> U16Vector2: ...
    @property
    def xxx(self) -> U16Vector3: ...
    @property
    def xxy(self) -> U16Vector3: ...
    @property
    def xxo(self) -> U16Vector3: ...
    @property
    def xxl(self) -> U16Vector3: ...
    @property
    def xyx(self) -> U16Vector3: ...
    @property
    def xyy(self) -> U16Vector3: ...
    @property
    def xyo(self) -> U16Vector3: ...
    @property
    def xyl(self) -> U16Vector3: ...
    @property
    def xox(self) -> U16Vector3: ...
    @property
    def xoy(self) -> U16Vector3: ...
    @property
    def xoo(self) -> U16Vector3: ...
    @property
    def xol(self) -> U16Vector3: ...
    @property
    def xlx(self) -> U16Vector3: ...
    @property
    def xly(self) -> U16Vector3: ...
    @property
    def xlo(self) -> U16Vector3: ...
    @property
    def xll(self) -> U16Vector3: ...
    @property
    def yxx(self) -> U16Vector3: ...
    @property
    def yxy(self) -> U16Vector3: ...
    @property
    def yxo(self) -> U16Vector3: ...
    @property
    def yxl(self) -> U16Vector3: ...
    @property
    def yyx(self) -> U16Vector3: ...
    @property
    def yyy(self) -> U16Vector3: ...
    @property
    def yyo(self) -> U16Vector3: ...
    @property
    def yyl(self) -> U16Vector3: ...
    @property
    def yox(self) -> U16Vector3: ...
    @property
    def yoy(self) -> U16Vector3: ...
    @property
    def yoo(self) -> U16Vector3: ...
    @property
    def yol(self) -> U16Vector3: ...
    @property
    def ylx(self) -> U16Vector3: ...
    @property
    def yly(self) -> U16Vector3: ...
    @property
    def ylo(self) -> U16Vector3: ...
    @property
    def yll(self) -> U16Vector3: ...
    @property
    def oxx(self) -> U16Vector3: ...
    @property
    def oxy(self) -> U16Vector3: ...
    @property
    def oxo(self) -> U16Vector3: ...
    @property
    def oxl(self) -> U16Vector3: ...
    @property
    def oyx(self) -> U16Vector3: ...
    @property
    def oyy(self) -> U16Vector3: ...
    @property
    def oyo(self) -> U16Vector3: ...
    @property
    def oyl(self) -> U16Vector3: ...
    @property
    def oox(self) -> U16Vector3: ...
    @property
    def ooy(self) -> U16Vector3: ...
    @property
    def olx(self) -> U16Vector3: ...
    @property
    def oly(self) -> U16Vector3: ...
    @property
    def lxx(self) -> U16Vector3: ...
    @property
    def lxy(self) -> U16Vector3: ...
    @property
    def lxo(self) -> U16Vector3: ...
    @property
    def lxl(self) -> U16Vector3: ...
    @property
    def lyx(self) -> U16Vector3: ...
    @property
    def lyy(self) -> U16Vector3: ...
    @property
    def lyo(self) -> U16Vector3: ...
    @property
    def lyl(self) -> U16Vector3: ...
    @property
    def lox(self) -> U16Vector3: ...
    @property
    def loy(self) -> U16Vector3: ...
    @property
    def llx(self) -> U16Vector3: ...
    @property
    def lly(self) -> U16Vector3: ...
    @property
    def xxxx(self) -> U16Vector4: ...
    @property
    def xxxy(self) -> U16Vector4: ...
    @property
    def xxxo(self) -> U16Vector4: ...
    @property
    def xxxl(self) -> U16Vector4: ...
    @property
    def xxyx(self) -> U16Vector4: ...
    @property
    def xxyy(self) -> U16Vector4: ...
    @property
    def xxyo(self) -> U16Vector4: ...
    @property
    def xxyl(self) -> U16Vector4: ...
    @property
    def xxox(self) -> U16Vector4: ...
    @property
    def xxoy(self) -> U16Vector4: ...
    @property
    def xxoo(self) -> U16Vector4: ...
    @property
    def xxol(self) -> U16Vector4: ...
    @property
    def xxlx(self) -> U16Vector4: ...
    @property
    def xxly(self) -> U16Vector4: ...
    @property
    def xxlo(self) -> U16Vector4: ...
    @property
    def xxll(self) -> U16Vector4: ...
    @property
    def xyxx(self) -> U16Vector4: ...
    @property
    def xyxy(self) -> U16Vector4: ...
    @property
    def xyxo(self) -> U16Vector4: ...
    @property
    def xyxl(self) -> U16Vector4: ...
    @property
    def xyyx(self) -> U16Vector4: ...
    @property
    def xyyy(self) -> U16Vector4: ...
    @property
    def xyyo(self) -> U16Vector4: ...
    @property
    def xyyl(self) -> U16Vector4: ...
    @property
    def xyox(self) -> U16Vector4: ...
    @property
    def xyoy(self) -> U16Vector4: ...
    @property
    def xyoo(self) -> U16Vector4: ...
    @property
    def xyol(self) -> U16Vector4: ...
    @property
    def xylx(self) -> U16Vector4: ...
    @property
    def xyly(self) -> U16Vector4: ...
    @property
    def xylo(self) -> U16Vector4: ...
    @property
    def xyll(self) -> U16Vector4: ...
    @property
    def xoxx(self) -> U16Vector4: ...
    @property
    def xoxy(self) -> U16Vector4: ...
    @property
    def xoxo(self) -> U16Vector4: ...
    @property
    def xoxl(self) -> U16Vector4: ...
    @property
    def xoyx(self) -> U16Vector4: ...
    @property
    def xoyy(self) -> U16Vector4: ...
    @property
    def xoyo(self) -> U16Vector4: ...
    @property
    def xoyl(self) -> U16Vector4: ...
    @property
    def xoox(self) -> U16Vector4: ...
    @property
    def xooy(self) -> U16Vector4: ...
    @property
    def xooo(self) -> U16Vector4: ...
    @property
    def xool(self) -> U16Vector4: ...
    @property
    def xolx(self) -> U16Vector4: ...
    @property
    def xoly(self) -> U16Vector4: ...
    @property
    def xolo(self) -> U16Vector4: ...
    @property
    def xoll(self) -> U16Vector4: ...
    @property
    def xlxx(self) -> U16Vector4: ...
    @property
    def xlxy(self) -> U16Vector4: ...
    @property
    def xlxo(self) -> U16Vector4: ...
    @property
    def xlxl(self) -> U16Vector4: ...
    @property
    def xlyx(self) -> U16Vector4: ...
    @property
    def xlyy(self) -> U16Vector4: ...
    @property
    def xlyo(self) -> U16Vector4: ...
    @property
    def xlyl(self) -> U16Vector4: ...
    @property
    def xlox(self) -> U16Vector4: ...
    @property
    def xloy(self) -> U16Vector4: ...
    @property
    def xloo(self) -> U16Vector4: ...
    @property
    def xlol(self) -> U16Vector4: ...
    @property
    def xllx(self) -> U16Vector4: ...
    @property
    def xlly(self) -> U16Vector4: ...
    @property
    def xllo(self) -> U16Vector4: ...
    @property
    def xlll(self) -> U16Vector4: ...
    @property
    def yxxx(self) -> U16Vector4: ...
    @property
    def yxxy(self) -> U16Vector4: ...
    @property
    def yxxo(self) -> U16Vector4: ...
    @property
    def yxxl(self) -> U16Vector4: ...
    @property
    def yxyx(self) -> U16Vector4: ...
    @property
    def yxyy(self) -> U16Vector4: ...
    @property
    def yxyo(self) -> U16Vector4: ...
    @property
    def yxyl(self) -> U16Vector4: ...
    @property
    def yxox(self) -> U16Vector4: ...
    @property
    def yxoy(self) -> U16Vector4: ...
    @property
    def yxoo(self) -> U16Vector4: ...
    @property
    def yxol(self) -> U16Vector4: ...
    @property
    def yxlx(self) -> U16Vector4: ...
    @property
    def yxly(self) -> U16Vector4: ...
    @property
    def yxlo(self) -> U16Vector4: ...
    @property
    def yxll(self) -> U16Vector4: ...
    @property
    def yyxx(self) -> U16Vector4: ...
    @property
    def yyxy(self) -> U16Vector4: ...
    @property
    def yyxo(self) -> U16Vector4: ...
    @property
    def yyxl(self) -> U16Vector4: ...
    @property
    def yyyx(self) -> U16Vector4: ...
    @property
    def yyyy(self) -> U16Vector4: ...
    @property
    def yyyo(self) -> U16Vector4: ...
    @property
    def yyyl(self) -> U16Vector4: ...
    @property
    def yyox(self) -> U16Vector4: ...
    @property
    def yyoy(self) -> U16Vector4: ...
    @property
    def yyoo(self) -> U16Vector4: ...
    @property
    def yyol(self) -> U16Vector4: ...
    @property
    def yylx(self) -> U16Vector4: ...
    @property
    def yyly(self) -> U16Vector4: ...
    @property
    def yylo(self) -> U16Vector4: ...
    @property
    def yyll(self) -> U16Vector4: ...
    @property
    def yoxx(self) -> U16Vector4: ...
    @property
    def yoxy(self) -> U16Vector4: ...
    @property
    def yoxo(self) -> U16Vector4: ...
    @property
    def yoxl(self) -> U16Vector4: ...
    @property
    def yoyx(self) -> U16Vector4: ...
    @property
    def yoyy(self) -> U16Vector4: ...
    @property
    def yoyo(self) -> U16Vector4: ...
    @property
    def yoyl(self) -> U16Vector4: ...
    @property
    def yoox(self) -> U16Vector4: ...
    @property
    def yooy(self) -> U16Vector4: ...
    @property
    def yooo(self) -> U16Vector4: ...
    @property
    def yool(self) -> U16Vector4: ...
    @property
    def yolx(self) -> U16Vector4: ...
    @property
    def yoly(self) -> U16Vector4: ...
    @property
    def yolo(self) -> U16Vector4: ...
    @property
    def yoll(self) -> U16Vector4: ...
    @property
    def ylxx(self) -> U16Vector4: ...
    @property
    def ylxy(self) -> U16Vector4: ...
    @property
    def ylxo(self) -> U16Vector4: ...
    @property
    def ylxl(self) -> U16Vector4: ...
    @property
    def ylyx(self) -> U16Vector4: ...
    @property
    def ylyy(self) -> U16Vector4: ...
    @property
    def ylyo(self) -> U16Vector4: ...
    @property
    def ylyl(self) -> U16Vector4: ...
    @property
    def ylox(self) -> U16Vector4: ...
    @property
    def yloy(self) -> U16Vector4: ...
    @property
    def yloo(self) -> U16Vector4: ...
    @property
    def ylol(self) -> U16Vector4: ...
    @property
    def yllx(self) -> U16Vector4: ...
    @property
    def ylly(self) -> U16Vector4: ...
    @property
    def yllo(self) -> U16Vector4: ...
    @property
    def ylll(self) -> U16Vector4: ...
    @property
    def oxxx(self) -> U16Vector4: ...
    @property
    def oxxy(self) -> U16Vector4: ...
    @property
    def oxxo(self) -> U16Vector4: ...
    @property
    def oxxl(self) -> U16Vector4: ...
    @property
    def oxyx(self) -> U16Vector4: ...
    @property
    def oxyy(self) -> U16Vector4: ...
    @property
    def oxyo(self) -> U16Vector4: ...
    @property
    def oxyl(self) -> U16Vector4: ...
    @property
    def oxox(self) -> U16Vector4: ...
    @property
    def oxoy(self) -> U16Vector4: ...
    @property
    def oxoo(self) -> U16Vector4: ...
    @property
    def oxol(self) -> U16Vector4: ...
    @property
    def oxlx(self) -> U16Vector4: ...
    @property
    def oxly(self) -> U16Vector4: ...
    @property
    def oxlo(self) -> U16Vector4: ...
    @property
    def oxll(self) -> U16Vector4: ...
    @property
    def oyxx(self) -> U16Vector4: ...
    @property
    def oyxy(self) -> U16Vector4: ...
    @property
    def oyxo(self) -> U16Vector4: ...
    @property
    def oyxl(self) -> U16Vector4: ...
    @property
    def oyyx(self) -> U16Vector4: ...
    @property
    def oyyy(self) -> U16Vector4: ...
    @property
    def oyyo(self) -> U16Vector4: ...
    @property
    def oyyl(self) -> U16Vector4: ...
    @property
    def oyox(self) -> U16Vector4: ...
    @property
    def oyoy(self) -> U16Vector4: ...
    @property
    def oyoo(self) -> U16Vector4: ...
    @property
    def oyol(self) -> U16Vector4: ...
    @property
    def oylx(self) -> U16Vector4: ...
    @property
    def oyly(self) -> U16Vector4: ...
    @property
    def oylo(self) -> U16Vector4: ...
    @property
    def oyll(self) -> U16Vector4: ...
    @property
    def ooxx(self) -> U16Vector4: ...
    @property
    def ooxy(self) -> U16Vector4: ...
    @property
    def ooxo(self) -> U16Vector4: ...
    @property
    def ooxl(self) -> U16Vector4: ...
    @property
    def ooyx(self) -> U16Vector4: ...
    @property
    def ooyy(self) -> U16Vector4: ...
    @property
    def ooyo(self) -> U16Vector4: ...
    @property
    def ooyl(self) -> U16Vector4: ...
    @property
    def ooox(self) -> U16Vector4: ...
    @property
    def oooy(self) -> U16Vector4: ...
    @property
    def oolx(self) -> U16Vector4: ...
    @property
    def ooly(self) -> U16Vector4: ...
    @property
    def olxx(self) -> U16Vector4: ...
    @property
    def olxy(self) -> U16Vector4: ...
    @property
    def olxo(self) -> U16Vector4: ...
    @property
    def olxl(self) -> U16Vector4: ...
    @property
    def olyx(self) -> U16Vector4: ...
    @property
    def olyy(self) -> U16Vector4: ...
    @property
    def olyo(self) -> U16Vector4: ...
    @property
    def olyl(self) -> U16Vector4: ...
    @property
    def olox(self) -> U16Vector4: ...
    @property
    def oloy(self) -> U16Vector4: ...
    @property
    def ollx(self) -> U16Vector4: ...
    @property
    def olly(self) -> U16Vector4: ...
    @property
    def lxxx(self) -> U16Vector4: ...
    @property
    def lxxy(self) -> U16Vector4: ...
    @property
    def lxxo(self) -> U16Vector4: ...
    @property
    def lxxl(self) -> U16Vector4: ...
    @property
    def lxyx(self) -> U16Vector4: ...
    @property
    def lxyy(self) -> U16Vector4: ...
    @property
    def lxyo(self) -> U16Vector4: ...
    @property
    def lxyl(self) -> U16Vector4: ...
    @property
    def lxox(self) -> U16Vector4: ...
    @property
    def lxoy(self) -> U16Vector4: ...
    @property
    def lxoo(self) -> U16Vector4: ...
    @property
    def lxol(self) -> U16Vector4: ...
    @property
    def lxlx(self) -> U16Vector4: ...
    @property
    def lxly(self) -> U16Vector4: ...
    @property
    def lxlo(self) -> U16Vector4: ...
    @property
    def lxll(self) -> U16Vector4: ...
    @property
    def lyxx(self) -> U16Vector4: ...
    @property
    def lyxy(self) -> U16Vector4: ...
    @property
    def lyxo(self) -> U16Vector4: ...
    @property
    def lyxl(self) -> U16Vector4: ...
    @property
    def lyyx(self) -> U16Vector4: ...
    @property
    def lyyy(self) -> U16Vector4: ...
    @property
    def lyyo(self) -> U16Vector4: ...
    @property
    def lyyl(self) -> U16Vector4: ...
    @property
    def lyox(self) -> U16Vector4: ...
    @property
    def lyoy(self) -> U16Vector4: ...
    @property
    def lyoo(self) -> U16Vector4: ...
    @property
    def lyol(self) -> U16Vector4: ...
    @property
    def lylx(self) -> U16Vector4: ...
    @property
    def lyly(self) -> U16Vector4: ...
    @property
    def lylo(self) -> U16Vector4: ...
    @property
    def lyll(self) -> U16Vector4: ...
    @property
    def loxx(self) -> U16Vector4: ...
    @property
    def loxy(self) -> U16Vector4: ...
    @property
    def loxo(self) -> U16Vector4: ...
    @property
    def loxl(self) -> U16Vector4: ...
    @property
    def loyx(self) -> U16Vector4: ...
    @property
    def loyy(self) -> U16Vector4: ...
    @property
    def loyo(self) -> U16Vector4: ...
    @property
    def loyl(self) -> U16Vector4: ...
    @property
    def loox(self) -> U16Vector4: ...
    @property
    def looy(self) -> U16Vector4: ...
    @property
    def lolx(self) -> U16Vector4: ...
    @property
    def loly(self) -> U16Vector4: ...
    @property
    def llxx(self) -> U16Vector4: ...
    @property
    def llxy(self) -> U16Vector4: ...
    @property
    def llxo(self) -> U16Vector4: ...
    @property
    def llxl(self) -> U16Vector4: ...
    @property
    def llyx(self) -> U16Vector4: ...
    @property
    def llyy(self) -> U16Vector4: ...
    @property
    def llyo(self) -> U16Vector4: ...
    @property
    def llyl(self) -> U16Vector4: ...
    @property
    def llox(self) -> U16Vector4: ...
    @property
    def lloy(self) -> U16Vector4: ...
    @property
    def lllx(self) -> U16Vector4: ...
    @property
    def llly(self) -> U16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> U16Vector2: ...
    @property
    def rg(self) -> U16Vector2: ...
    @property
    def ro(self) -> U16Vector2: ...
    @property
    def rl(self) -> U16Vector2: ...
    @property
    def gr(self) -> U16Vector2: ...
    @property
    def gg(self) -> U16Vector2: ...
    @property
    def go(self) -> U16Vector2: ...
    @property
    def gl(self) -> U16Vector2: ...
    @property
    def og(self) -> U16Vector2: ...
    @property
    def lr(self) -> U16Vector2: ...
    @property
    def lg(self) -> U16Vector2: ...
    @property
    def rrr(self) -> U16Vector3: ...
    @property
    def rrg(self) -> U16Vector3: ...
    @property
    def rro(self) -> U16Vector3: ...
    @property
    def rrl(self) -> U16Vector3: ...
    @property
    def rgr(self) -> U16Vector3: ...
    @property
    def rgg(self) -> U16Vector3: ...
    @property
    def rgo(self) -> U16Vector3: ...
    @property
    def rgl(self) -> U16Vector3: ...
    @property
    def ror(self) -> U16Vector3: ...
    @property
    def rog(self) -> U16Vector3: ...
    @property
    def roo(self) -> U16Vector3: ...
    @property
    def rol(self) -> U16Vector3: ...
    @property
    def rlr(self) -> U16Vector3: ...
    @property
    def rlg(self) -> U16Vector3: ...
    @property
    def rlo(self) -> U16Vector3: ...
    @property
    def rll(self) -> U16Vector3: ...
    @property
    def grr(self) -> U16Vector3: ...
    @property
    def grg(self) -> U16Vector3: ...
    @property
    def gro(self) -> U16Vector3: ...
    @property
    def grl(self) -> U16Vector3: ...
    @property
    def ggr(self) -> U16Vector3: ...
    @property
    def ggg(self) -> U16Vector3: ...
    @property
    def ggo(self) -> U16Vector3: ...
    @property
    def ggl(self) -> U16Vector3: ...
    @property
    def gor(self) -> U16Vector3: ...
    @property
    def gog(self) -> U16Vector3: ...
    @property
    def goo(self) -> U16Vector3: ...
    @property
    def gol(self) -> U16Vector3: ...
    @property
    def glr(self) -> U16Vector3: ...
    @property
    def glg(self) -> U16Vector3: ...
    @property
    def glo(self) -> U16Vector3: ...
    @property
    def gll(self) -> U16Vector3: ...
    @property
    def orr(self) -> U16Vector3: ...
    @property
    def org(self) -> U16Vector3: ...
    @property
    def oro(self) -> U16Vector3: ...
    @property
    def orl(self) -> U16Vector3: ...
    @property
    def ogr(self) -> U16Vector3: ...
    @property
    def ogg(self) -> U16Vector3: ...
    @property
    def ogo(self) -> U16Vector3: ...
    @property
    def ogl(self) -> U16Vector3: ...
    @property
    def oor(self) -> U16Vector3: ...
    @property
    def oog(self) -> U16Vector3: ...
    @property
    def olr(self) -> U16Vector3: ...
    @property
    def olg(self) -> U16Vector3: ...
    @property
    def lrr(self) -> U16Vector3: ...
    @property
    def lrg(self) -> U16Vector3: ...
    @property
    def lro(self) -> U16Vector3: ...
    @property
    def lrl(self) -> U16Vector3: ...
    @property
    def lgr(self) -> U16Vector3: ...
    @property
    def lgg(self) -> U16Vector3: ...
    @property
    def lgo(self) -> U16Vector3: ...
    @property
    def lgl(self) -> U16Vector3: ...
    @property
    def lor(self) -> U16Vector3: ...
    @property
    def log(self) -> U16Vector3: ...
    @property
    def llr(self) -> U16Vector3: ...
    @property
    def llg(self) -> U16Vector3: ...
    @property
    def rrrr(self) -> U16Vector4: ...
    @property
    def rrrg(self) -> U16Vector4: ...
    @property
    def rrro(self) -> U16Vector4: ...
    @property
    def rrrl(self) -> U16Vector4: ...
    @property
    def rrgr(self) -> U16Vector4: ...
    @property
    def rrgg(self) -> U16Vector4: ...
    @property
    def rrgo(self) -> U16Vector4: ...
    @property
    def rrgl(self) -> U16Vector4: ...
    @property
    def rror(self) -> U16Vector4: ...
    @property
    def rrog(self) -> U16Vector4: ...
    @property
    def rroo(self) -> U16Vector4: ...
    @property
    def rrol(self) -> U16Vector4: ...
    @property
    def rrlr(self) -> U16Vector4: ...
    @property
    def rrlg(self) -> U16Vector4: ...
    @property
    def rrlo(self) -> U16Vector4: ...
    @property
    def rrll(self) -> U16Vector4: ...
    @property
    def rgrr(self) -> U16Vector4: ...
    @property
    def rgrg(self) -> U16Vector4: ...
    @property
    def rgro(self) -> U16Vector4: ...
    @property
    def rgrl(self) -> U16Vector4: ...
    @property
    def rggr(self) -> U16Vector4: ...
    @property
    def rggg(self) -> U16Vector4: ...
    @property
    def rggo(self) -> U16Vector4: ...
    @property
    def rggl(self) -> U16Vector4: ...
    @property
    def rgor(self) -> U16Vector4: ...
    @property
    def rgog(self) -> U16Vector4: ...
    @property
    def rgoo(self) -> U16Vector4: ...
    @property
    def rgol(self) -> U16Vector4: ...
    @property
    def rglr(self) -> U16Vector4: ...
    @property
    def rglg(self) -> U16Vector4: ...
    @property
    def rglo(self) -> U16Vector4: ...
    @property
    def rgll(self) -> U16Vector4: ...
    @property
    def rorr(self) -> U16Vector4: ...
    @property
    def rorg(self) -> U16Vector4: ...
    @property
    def roro(self) -> U16Vector4: ...
    @property
    def rorl(self) -> U16Vector4: ...
    @property
    def rogr(self) -> U16Vector4: ...
    @property
    def rogg(self) -> U16Vector4: ...
    @property
    def rogo(self) -> U16Vector4: ...
    @property
    def rogl(self) -> U16Vector4: ...
    @property
    def roor(self) -> U16Vector4: ...
    @property
    def roog(self) -> U16Vector4: ...
    @property
    def rooo(self) -> U16Vector4: ...
    @property
    def rool(self) -> U16Vector4: ...
    @property
    def rolr(self) -> U16Vector4: ...
    @property
    def rolg(self) -> U16Vector4: ...
    @property
    def rolo(self) -> U16Vector4: ...
    @property
    def roll(self) -> U16Vector4: ...
    @property
    def rlrr(self) -> U16Vector4: ...
    @property
    def rlrg(self) -> U16Vector4: ...
    @property
    def rlro(self) -> U16Vector4: ...
    @property
    def rlrl(self) -> U16Vector4: ...
    @property
    def rlgr(self) -> U16Vector4: ...
    @property
    def rlgg(self) -> U16Vector4: ...
    @property
    def rlgo(self) -> U16Vector4: ...
    @property
    def rlgl(self) -> U16Vector4: ...
    @property
    def rlor(self) -> U16Vector4: ...
    @property
    def rlog(self) -> U16Vector4: ...
    @property
    def rloo(self) -> U16Vector4: ...
    @property
    def rlol(self) -> U16Vector4: ...
    @property
    def rllr(self) -> U16Vector4: ...
    @property
    def rllg(self) -> U16Vector4: ...
    @property
    def rllo(self) -> U16Vector4: ...
    @property
    def rlll(self) -> U16Vector4: ...
    @property
    def grrr(self) -> U16Vector4: ...
    @property
    def grrg(self) -> U16Vector4: ...
    @property
    def grro(self) -> U16Vector4: ...
    @property
    def grrl(self) -> U16Vector4: ...
    @property
    def grgr(self) -> U16Vector4: ...
    @property
    def grgg(self) -> U16Vector4: ...
    @property
    def grgo(self) -> U16Vector4: ...
    @property
    def grgl(self) -> U16Vector4: ...
    @property
    def gror(self) -> U16Vector4: ...
    @property
    def grog(self) -> U16Vector4: ...
    @property
    def groo(self) -> U16Vector4: ...
    @property
    def grol(self) -> U16Vector4: ...
    @property
    def grlr(self) -> U16Vector4: ...
    @property
    def grlg(self) -> U16Vector4: ...
    @property
    def grlo(self) -> U16Vector4: ...
    @property
    def grll(self) -> U16Vector4: ...
    @property
    def ggrr(self) -> U16Vector4: ...
    @property
    def ggrg(self) -> U16Vector4: ...
    @property
    def ggro(self) -> U16Vector4: ...
    @property
    def ggrl(self) -> U16Vector4: ...
    @property
    def gggr(self) -> U16Vector4: ...
    @property
    def gggg(self) -> U16Vector4: ...
    @property
    def gggo(self) -> U16Vector4: ...
    @property
    def gggl(self) -> U16Vector4: ...
    @property
    def ggor(self) -> U16Vector4: ...
    @property
    def ggog(self) -> U16Vector4: ...
    @property
    def ggoo(self) -> U16Vector4: ...
    @property
    def ggol(self) -> U16Vector4: ...
    @property
    def gglr(self) -> U16Vector4: ...
    @property
    def gglg(self) -> U16Vector4: ...
    @property
    def gglo(self) -> U16Vector4: ...
    @property
    def ggll(self) -> U16Vector4: ...
    @property
    def gorr(self) -> U16Vector4: ...
    @property
    def gorg(self) -> U16Vector4: ...
    @property
    def goro(self) -> U16Vector4: ...
    @property
    def gorl(self) -> U16Vector4: ...
    @property
    def gogr(self) -> U16Vector4: ...
    @property
    def gogg(self) -> U16Vector4: ...
    @property
    def gogo(self) -> U16Vector4: ...
    @property
    def gogl(self) -> U16Vector4: ...
    @property
    def goor(self) -> U16Vector4: ...
    @property
    def goog(self) -> U16Vector4: ...
    @property
    def gooo(self) -> U16Vector4: ...
    @property
    def gool(self) -> U16Vector4: ...
    @property
    def golr(self) -> U16Vector4: ...
    @property
    def golg(self) -> U16Vector4: ...
    @property
    def golo(self) -> U16Vector4: ...
    @property
    def goll(self) -> U16Vector4: ...
    @property
    def glrr(self) -> U16Vector4: ...
    @property
    def glrg(self) -> U16Vector4: ...
    @property
    def glro(self) -> U16Vector4: ...
    @property
    def glrl(self) -> U16Vector4: ...
    @property
    def glgr(self) -> U16Vector4: ...
    @property
    def glgg(self) -> U16Vector4: ...
    @property
    def glgo(self) -> U16Vector4: ...
    @property
    def glgl(self) -> U16Vector4: ...
    @property
    def glor(self) -> U16Vector4: ...
    @property
    def glog(self) -> U16Vector4: ...
    @property
    def gloo(self) -> U16Vector4: ...
    @property
    def glol(self) -> U16Vector4: ...
    @property
    def gllr(self) -> U16Vector4: ...
    @property
    def gllg(self) -> U16Vector4: ...
    @property
    def gllo(self) -> U16Vector4: ...
    @property
    def glll(self) -> U16Vector4: ...
    @property
    def orrr(self) -> U16Vector4: ...
    @property
    def orrg(self) -> U16Vector4: ...
    @property
    def orro(self) -> U16Vector4: ...
    @property
    def orrl(self) -> U16Vector4: ...
    @property
    def orgr(self) -> U16Vector4: ...
    @property
    def orgg(self) -> U16Vector4: ...
    @property
    def orgo(self) -> U16Vector4: ...
    @property
    def orgl(self) -> U16Vector4: ...
    @property
    def oror(self) -> U16Vector4: ...
    @property
    def orog(self) -> U16Vector4: ...
    @property
    def oroo(self) -> U16Vector4: ...
    @property
    def orol(self) -> U16Vector4: ...
    @property
    def orlr(self) -> U16Vector4: ...
    @property
    def orlg(self) -> U16Vector4: ...
    @property
    def orlo(self) -> U16Vector4: ...
    @property
    def orll(self) -> U16Vector4: ...
    @property
    def ogrr(self) -> U16Vector4: ...
    @property
    def ogrg(self) -> U16Vector4: ...
    @property
    def ogro(self) -> U16Vector4: ...
    @property
    def ogrl(self) -> U16Vector4: ...
    @property
    def oggr(self) -> U16Vector4: ...
    @property
    def oggg(self) -> U16Vector4: ...
    @property
    def oggo(self) -> U16Vector4: ...
    @property
    def oggl(self) -> U16Vector4: ...
    @property
    def ogor(self) -> U16Vector4: ...
    @property
    def ogog(self) -> U16Vector4: ...
    @property
    def ogoo(self) -> U16Vector4: ...
    @property
    def ogol(self) -> U16Vector4: ...
    @property
    def oglr(self) -> U16Vector4: ...
    @property
    def oglg(self) -> U16Vector4: ...
    @property
    def oglo(self) -> U16Vector4: ...
    @property
    def ogll(self) -> U16Vector4: ...
    @property
    def oorr(self) -> U16Vector4: ...
    @property
    def oorg(self) -> U16Vector4: ...
    @property
    def ooro(self) -> U16Vector4: ...
    @property
    def oorl(self) -> U16Vector4: ...
    @property
    def oogr(self) -> U16Vector4: ...
    @property
    def oogg(self) -> U16Vector4: ...
    @property
    def oogo(self) -> U16Vector4: ...
    @property
    def oogl(self) -> U16Vector4: ...
    @property
    def ooor(self) -> U16Vector4: ...
    @property
    def ooog(self) -> U16Vector4: ...
    @property
    def oolr(self) -> U16Vector4: ...
    @property
    def oolg(self) -> U16Vector4: ...
    @property
    def olrr(self) -> U16Vector4: ...
    @property
    def olrg(self) -> U16Vector4: ...
    @property
    def olro(self) -> U16Vector4: ...
    @property
    def olrl(self) -> U16Vector4: ...
    @property
    def olgr(self) -> U16Vector4: ...
    @property
    def olgg(self) -> U16Vector4: ...
    @property
    def olgo(self) -> U16Vector4: ...
    @property
    def olgl(self) -> U16Vector4: ...
    @property
    def olor(self) -> U16Vector4: ...
    @property
    def olog(self) -> U16Vector4: ...
    @property
    def ollr(self) -> U16Vector4: ...
    @property
    def ollg(self) -> U16Vector4: ...
    @property
    def lrrr(self) -> U16Vector4: ...
    @property
    def lrrg(self) -> U16Vector4: ...
    @property
    def lrro(self) -> U16Vector4: ...
    @property
    def lrrl(self) -> U16Vector4: ...
    @property
    def lrgr(self) -> U16Vector4: ...
    @property
    def lrgg(self) -> U16Vector4: ...
    @property
    def lrgo(self) -> U16Vector4: ...
    @property
    def lrgl(self) -> U16Vector4: ...
    @property
    def lror(self) -> U16Vector4: ...
    @property
    def lrog(self) -> U16Vector4: ...
    @property
    def lroo(self) -> U16Vector4: ...
    @property
    def lrol(self) -> U16Vector4: ...
    @property
    def lrlr(self) -> U16Vector4: ...
    @property
    def lrlg(self) -> U16Vector4: ...
    @property
    def lrlo(self) -> U16Vector4: ...
    @property
    def lrll(self) -> U16Vector4: ...
    @property
    def lgrr(self) -> U16Vector4: ...
    @property
    def lgrg(self) -> U16Vector4: ...
    @property
    def lgro(self) -> U16Vector4: ...
    @property
    def lgrl(self) -> U16Vector4: ...
    @property
    def lggr(self) -> U16Vector4: ...
    @property
    def lggg(self) -> U16Vector4: ...
    @property
    def lggo(self) -> U16Vector4: ...
    @property
    def lggl(self) -> U16Vector4: ...
    @property
    def lgor(self) -> U16Vector4: ...
    @property
    def lgog(self) -> U16Vector4: ...
    @property
    def lgoo(self) -> U16Vector4: ...
    @property
    def lgol(self) -> U16Vector4: ...
    @property
    def lglr(self) -> U16Vector4: ...
    @property
    def lglg(self) -> U16Vector4: ...
    @property
    def lglo(self) -> U16Vector4: ...
    @property
    def lgll(self) -> U16Vector4: ...
    @property
    def lorr(self) -> U16Vector4: ...
    @property
    def lorg(self) -> U16Vector4: ...
    @property
    def loro(self) -> U16Vector4: ...
    @property
    def lorl(self) -> U16Vector4: ...
    @property
    def logr(self) -> U16Vector4: ...
    @property
    def logg(self) -> U16Vector4: ...
    @property
    def logo(self) -> U16Vector4: ...
    @property
    def logl(self) -> U16Vector4: ...
    @property
    def loor(self) -> U16Vector4: ...
    @property
    def loog(self) -> U16Vector4: ...
    @property
    def lolr(self) -> U16Vector4: ...
    @property
    def lolg(self) -> U16Vector4: ...
    @property
    def llrr(self) -> U16Vector4: ...
    @property
    def llrg(self) -> U16Vector4: ...
    @property
    def llro(self) -> U16Vector4: ...
    @property
    def llrl(self) -> U16Vector4: ...
    @property
    def llgr(self) -> U16Vector4: ...
    @property
    def llgg(self) -> U16Vector4: ...
    @property
    def llgo(self) -> U16Vector4: ...
    @property
    def llgl(self) -> U16Vector4: ...
    @property
    def llor(self) -> U16Vector4: ...
    @property
    def llog(self) -> U16Vector4: ...
    @property
    def lllr(self) -> U16Vector4: ...
    @property
    def lllg(self) -> U16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> U16Vector2: ...
    @property
    def st(self) -> U16Vector2: ...
    @property
    def so(self) -> U16Vector2: ...
    @property
    def sl(self) -> U16Vector2: ...
    @property
    def ts(self) -> U16Vector2: ...
    @property
    def tt(self) -> U16Vector2: ...
    @property
    def to(self) -> U16Vector2: ...
    @property
    def tl(self) -> U16Vector2: ...
    @property
    def os(self) -> U16Vector2: ...
    @property
    def ot(self) -> U16Vector2: ...
    @property
    def ls(self) -> U16Vector2: ...
    @property
    def lt(self) -> U16Vector2: ...
    @property
    def sss(self) -> U16Vector3: ...
    @property
    def sst(self) -> U16Vector3: ...
    @property
    def sso(self) -> U16Vector3: ...
    @property
    def ssl(self) -> U16Vector3: ...
    @property
    def sts(self) -> U16Vector3: ...
    @property
    def stt(self) -> U16Vector3: ...
    @property
    def sto(self) -> U16Vector3: ...
    @property
    def stl(self) -> U16Vector3: ...
    @property
    def sos(self) -> U16Vector3: ...
    @property
    def sot(self) -> U16Vector3: ...
    @property
    def soo(self) -> U16Vector3: ...
    @property
    def sol(self) -> U16Vector3: ...
    @property
    def sls(self) -> U16Vector3: ...
    @property
    def slt(self) -> U16Vector3: ...
    @property
    def slo(self) -> U16Vector3: ...
    @property
    def sll(self) -> U16Vector3: ...
    @property
    def tss(self) -> U16Vector3: ...
    @property
    def tst(self) -> U16Vector3: ...
    @property
    def tso(self) -> U16Vector3: ...
    @property
    def tsl(self) -> U16Vector3: ...
    @property
    def tts(self) -> U16Vector3: ...
    @property
    def ttt(self) -> U16Vector3: ...
    @property
    def tto(self) -> U16Vector3: ...
    @property
    def ttl(self) -> U16Vector3: ...
    @property
    def tos(self) -> U16Vector3: ...
    @property
    def tot(self) -> U16Vector3: ...
    @property
    def too(self) -> U16Vector3: ...
    @property
    def tol(self) -> U16Vector3: ...
    @property
    def tls(self) -> U16Vector3: ...
    @property
    def tlt(self) -> U16Vector3: ...
    @property
    def tlo(self) -> U16Vector3: ...
    @property
    def tll(self) -> U16Vector3: ...
    @property
    def oss(self) -> U16Vector3: ...
    @property
    def ost(self) -> U16Vector3: ...
    @property
    def oso(self) -> U16Vector3: ...
    @property
    def osl(self) -> U16Vector3: ...
    @property
    def ots(self) -> U16Vector3: ...
    @property
    def ott(self) -> U16Vector3: ...
    @property
    def oto(self) -> U16Vector3: ...
    @property
    def otl(self) -> U16Vector3: ...
    @property
    def oos(self) -> U16Vector3: ...
    @property
    def oot(self) -> U16Vector3: ...
    @property
    def ols(self) -> U16Vector3: ...
    @property
    def olt(self) -> U16Vector3: ...
    @property
    def lss(self) -> U16Vector3: ...
    @property
    def lst(self) -> U16Vector3: ...
    @property
    def lso(self) -> U16Vector3: ...
    @property
    def lsl(self) -> U16Vector3: ...
    @property
    def lts(self) -> U16Vector3: ...
    @property
    def ltt(self) -> U16Vector3: ...
    @property
    def lto(self) -> U16Vector3: ...
    @property
    def ltl(self) -> U16Vector3: ...
    @property
    def los(self) -> U16Vector3: ...
    @property
    def lot(self) -> U16Vector3: ...
    @property
    def lls(self) -> U16Vector3: ...
    @property
    def llt(self) -> U16Vector3: ...
    @property
    def ssss(self) -> U16Vector4: ...
    @property
    def ssst(self) -> U16Vector4: ...
    @property
    def ssso(self) -> U16Vector4: ...
    @property
    def sssl(self) -> U16Vector4: ...
    @property
    def ssts(self) -> U16Vector4: ...
    @property
    def sstt(self) -> U16Vector4: ...
    @property
    def ssto(self) -> U16Vector4: ...
    @property
    def sstl(self) -> U16Vector4: ...
    @property
    def ssos(self) -> U16Vector4: ...
    @property
    def ssot(self) -> U16Vector4: ...
    @property
    def ssoo(self) -> U16Vector4: ...
    @property
    def ssol(self) -> U16Vector4: ...
    @property
    def ssls(self) -> U16Vector4: ...
    @property
    def sslt(self) -> U16Vector4: ...
    @property
    def sslo(self) -> U16Vector4: ...
    @property
    def ssll(self) -> U16Vector4: ...
    @property
    def stss(self) -> U16Vector4: ...
    @property
    def stst(self) -> U16Vector4: ...
    @property
    def stso(self) -> U16Vector4: ...
    @property
    def stsl(self) -> U16Vector4: ...
    @property
    def stts(self) -> U16Vector4: ...
    @property
    def sttt(self) -> U16Vector4: ...
    @property
    def stto(self) -> U16Vector4: ...
    @property
    def sttl(self) -> U16Vector4: ...
    @property
    def stos(self) -> U16Vector4: ...
    @property
    def stot(self) -> U16Vector4: ...
    @property
    def stoo(self) -> U16Vector4: ...
    @property
    def stol(self) -> U16Vector4: ...
    @property
    def stls(self) -> U16Vector4: ...
    @property
    def stlt(self) -> U16Vector4: ...
    @property
    def stlo(self) -> U16Vector4: ...
    @property
    def stll(self) -> U16Vector4: ...
    @property
    def soss(self) -> U16Vector4: ...
    @property
    def sost(self) -> U16Vector4: ...
    @property
    def soso(self) -> U16Vector4: ...
    @property
    def sosl(self) -> U16Vector4: ...
    @property
    def sots(self) -> U16Vector4: ...
    @property
    def sott(self) -> U16Vector4: ...
    @property
    def soto(self) -> U16Vector4: ...
    @property
    def sotl(self) -> U16Vector4: ...
    @property
    def soos(self) -> U16Vector4: ...
    @property
    def soot(self) -> U16Vector4: ...
    @property
    def sooo(self) -> U16Vector4: ...
    @property
    def sool(self) -> U16Vector4: ...
    @property
    def sols(self) -> U16Vector4: ...
    @property
    def solt(self) -> U16Vector4: ...
    @property
    def solo(self) -> U16Vector4: ...
    @property
    def soll(self) -> U16Vector4: ...
    @property
    def slss(self) -> U16Vector4: ...
    @property
    def slst(self) -> U16Vector4: ...
    @property
    def slso(self) -> U16Vector4: ...
    @property
    def slsl(self) -> U16Vector4: ...
    @property
    def slts(self) -> U16Vector4: ...
    @property
    def sltt(self) -> U16Vector4: ...
    @property
    def slto(self) -> U16Vector4: ...
    @property
    def sltl(self) -> U16Vector4: ...
    @property
    def slos(self) -> U16Vector4: ...
    @property
    def slot(self) -> U16Vector4: ...
    @property
    def sloo(self) -> U16Vector4: ...
    @property
    def slol(self) -> U16Vector4: ...
    @property
    def slls(self) -> U16Vector4: ...
    @property
    def sllt(self) -> U16Vector4: ...
    @property
    def sllo(self) -> U16Vector4: ...
    @property
    def slll(self) -> U16Vector4: ...
    @property
    def tsss(self) -> U16Vector4: ...
    @property
    def tsst(self) -> U16Vector4: ...
    @property
    def tsso(self) -> U16Vector4: ...
    @property
    def tssl(self) -> U16Vector4: ...
    @property
    def tsts(self) -> U16Vector4: ...
    @property
    def tstt(self) -> U16Vector4: ...
    @property
    def tsto(self) -> U16Vector4: ...
    @property
    def tstl(self) -> U16Vector4: ...
    @property
    def tsos(self) -> U16Vector4: ...
    @property
    def tsot(self) -> U16Vector4: ...
    @property
    def tsoo(self) -> U16Vector4: ...
    @property
    def tsol(self) -> U16Vector4: ...
    @property
    def tsls(self) -> U16Vector4: ...
    @property
    def tslt(self) -> U16Vector4: ...
    @property
    def tslo(self) -> U16Vector4: ...
    @property
    def tsll(self) -> U16Vector4: ...
    @property
    def ttss(self) -> U16Vector4: ...
    @property
    def ttst(self) -> U16Vector4: ...
    @property
    def ttso(self) -> U16Vector4: ...
    @property
    def ttsl(self) -> U16Vector4: ...
    @property
    def ttts(self) -> U16Vector4: ...
    @property
    def tttt(self) -> U16Vector4: ...
    @property
    def ttto(self) -> U16Vector4: ...
    @property
    def tttl(self) -> U16Vector4: ...
    @property
    def ttos(self) -> U16Vector4: ...
    @property
    def ttot(self) -> U16Vector4: ...
    @property
    def ttoo(self) -> U16Vector4: ...
    @property
    def ttol(self) -> U16Vector4: ...
    @property
    def ttls(self) -> U16Vector4: ...
    @property
    def ttlt(self) -> U16Vector4: ...
    @property
    def ttlo(self) -> U16Vector4: ...
    @property
    def ttll(self) -> U16Vector4: ...
    @property
    def toss(self) -> U16Vector4: ...
    @property
    def tost(self) -> U16Vector4: ...
    @property
    def toso(self) -> U16Vector4: ...
    @property
    def tosl(self) -> U16Vector4: ...
    @property
    def tots(self) -> U16Vector4: ...
    @property
    def tott(self) -> U16Vector4: ...
    @property
    def toto(self) -> U16Vector4: ...
    @property
    def totl(self) -> U16Vector4: ...
    @property
    def toos(self) -> U16Vector4: ...
    @property
    def toot(self) -> U16Vector4: ...
    @property
    def tooo(self) -> U16Vector4: ...
    @property
    def tool(self) -> U16Vector4: ...
    @property
    def tols(self) -> U16Vector4: ...
    @property
    def tolt(self) -> U16Vector4: ...
    @property
    def tolo(self) -> U16Vector4: ...
    @property
    def toll(self) -> U16Vector4: ...
    @property
    def tlss(self) -> U16Vector4: ...
    @property
    def tlst(self) -> U16Vector4: ...
    @property
    def tlso(self) -> U16Vector4: ...
    @property
    def tlsl(self) -> U16Vector4: ...
    @property
    def tlts(self) -> U16Vector4: ...
    @property
    def tltt(self) -> U16Vector4: ...
    @property
    def tlto(self) -> U16Vector4: ...
    @property
    def tltl(self) -> U16Vector4: ...
    @property
    def tlos(self) -> U16Vector4: ...
    @property
    def tlot(self) -> U16Vector4: ...
    @property
    def tloo(self) -> U16Vector4: ...
    @property
    def tlol(self) -> U16Vector4: ...
    @property
    def tlls(self) -> U16Vector4: ...
    @property
    def tllt(self) -> U16Vector4: ...
    @property
    def tllo(self) -> U16Vector4: ...
    @property
    def tlll(self) -> U16Vector4: ...
    @property
    def osss(self) -> U16Vector4: ...
    @property
    def osst(self) -> U16Vector4: ...
    @property
    def osso(self) -> U16Vector4: ...
    @property
    def ossl(self) -> U16Vector4: ...
    @property
    def osts(self) -> U16Vector4: ...
    @property
    def ostt(self) -> U16Vector4: ...
    @property
    def osto(self) -> U16Vector4: ...
    @property
    def ostl(self) -> U16Vector4: ...
    @property
    def osos(self) -> U16Vector4: ...
    @property
    def osot(self) -> U16Vector4: ...
    @property
    def osoo(self) -> U16Vector4: ...
    @property
    def osol(self) -> U16Vector4: ...
    @property
    def osls(self) -> U16Vector4: ...
    @property
    def oslt(self) -> U16Vector4: ...
    @property
    def oslo(self) -> U16Vector4: ...
    @property
    def osll(self) -> U16Vector4: ...
    @property
    def otss(self) -> U16Vector4: ...
    @property
    def otst(self) -> U16Vector4: ...
    @property
    def otso(self) -> U16Vector4: ...
    @property
    def otsl(self) -> U16Vector4: ...
    @property
    def otts(self) -> U16Vector4: ...
    @property
    def ottt(self) -> U16Vector4: ...
    @property
    def otto(self) -> U16Vector4: ...
    @property
    def ottl(self) -> U16Vector4: ...
    @property
    def otos(self) -> U16Vector4: ...
    @property
    def otot(self) -> U16Vector4: ...
    @property
    def otoo(self) -> U16Vector4: ...
    @property
    def otol(self) -> U16Vector4: ...
    @property
    def otls(self) -> U16Vector4: ...
    @property
    def otlt(self) -> U16Vector4: ...
    @property
    def otlo(self) -> U16Vector4: ...
    @property
    def otll(self) -> U16Vector4: ...
    @property
    def ooss(self) -> U16Vector4: ...
    @property
    def oost(self) -> U16Vector4: ...
    @property
    def ooso(self) -> U16Vector4: ...
    @property
    def oosl(self) -> U16Vector4: ...
    @property
    def oots(self) -> U16Vector4: ...
    @property
    def oott(self) -> U16Vector4: ...
    @property
    def ooto(self) -> U16Vector4: ...
    @property
    def ootl(self) -> U16Vector4: ...
    @property
    def ooos(self) -> U16Vector4: ...
    @property
    def ooot(self) -> U16Vector4: ...
    @property
    def ools(self) -> U16Vector4: ...
    @property
    def oolt(self) -> U16Vector4: ...
    @property
    def olss(self) -> U16Vector4: ...
    @property
    def olst(self) -> U16Vector4: ...
    @property
    def olso(self) -> U16Vector4: ...
    @property
    def olsl(self) -> U16Vector4: ...
    @property
    def olts(self) -> U16Vector4: ...
    @property
    def oltt(self) -> U16Vector4: ...
    @property
    def olto(self) -> U16Vector4: ...
    @property
    def oltl(self) -> U16Vector4: ...
    @property
    def olos(self) -> U16Vector4: ...
    @property
    def olot(self) -> U16Vector4: ...
    @property
    def olls(self) -> U16Vector4: ...
    @property
    def ollt(self) -> U16Vector4: ...
    @property
    def lsss(self) -> U16Vector4: ...
    @property
    def lsst(self) -> U16Vector4: ...
    @property
    def lsso(self) -> U16Vector4: ...
    @property
    def lssl(self) -> U16Vector4: ...
    @property
    def lsts(self) -> U16Vector4: ...
    @property
    def lstt(self) -> U16Vector4: ...
    @property
    def lsto(self) -> U16Vector4: ...
    @property
    def lstl(self) -> U16Vector4: ...
    @property
    def lsos(self) -> U16Vector4: ...
    @property
    def lsot(self) -> U16Vector4: ...
    @property
    def lsoo(self) -> U16Vector4: ...
    @property
    def lsol(self) -> U16Vector4: ...
    @property
    def lsls(self) -> U16Vector4: ...
    @property
    def lslt(self) -> U16Vector4: ...
    @property
    def lslo(self) -> U16Vector4: ...
    @property
    def lsll(self) -> U16Vector4: ...
    @property
    def ltss(self) -> U16Vector4: ...
    @property
    def ltst(self) -> U16Vector4: ...
    @property
    def ltso(self) -> U16Vector4: ...
    @property
    def ltsl(self) -> U16Vector4: ...
    @property
    def ltts(self) -> U16Vector4: ...
    @property
    def lttt(self) -> U16Vector4: ...
    @property
    def ltto(self) -> U16Vector4: ...
    @property
    def lttl(self) -> U16Vector4: ...
    @property
    def ltos(self) -> U16Vector4: ...
    @property
    def ltot(self) -> U16Vector4: ...
    @property
    def ltoo(self) -> U16Vector4: ...
    @property
    def ltol(self) -> U16Vector4: ...
    @property
    def ltls(self) -> U16Vector4: ...
    @property
    def ltlt(self) -> U16Vector4: ...
    @property
    def ltlo(self) -> U16Vector4: ...
    @property
    def ltll(self) -> U16Vector4: ...
    @property
    def loss(self) -> U16Vector4: ...
    @property
    def lost(self) -> U16Vector4: ...
    @property
    def loso(self) -> U16Vector4: ...
    @property
    def losl(self) -> U16Vector4: ...
    @property
    def lots(self) -> U16Vector4: ...
    @property
    def lott(self) -> U16Vector4: ...
    @property
    def loto(self) -> U16Vector4: ...
    @property
    def lotl(self) -> U16Vector4: ...
    @property
    def loos(self) -> U16Vector4: ...
    @property
    def loot(self) -> U16Vector4: ...
    @property
    def lols(self) -> U16Vector4: ...
    @property
    def lolt(self) -> U16Vector4: ...
    @property
    def llss(self) -> U16Vector4: ...
    @property
    def llst(self) -> U16Vector4: ...
    @property
    def llso(self) -> U16Vector4: ...
    @property
    def llsl(self) -> U16Vector4: ...
    @property
    def llts(self) -> U16Vector4: ...
    @property
    def lltt(self) -> U16Vector4: ...
    @property
    def llto(self) -> U16Vector4: ...
    @property
    def lltl(self) -> U16Vector4: ...
    @property
    def llos(self) -> U16Vector4: ...
    @property
    def llot(self) -> U16Vector4: ...
    @property
    def llls(self) -> U16Vector4: ...
    @property
    def lllt(self) -> U16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U16Vector2: ...
    @property
    def uv(self) -> U16Vector2: ...
    @property
    def uo(self) -> U16Vector2: ...
    @property
    def ul(self) -> U16Vector2: ...
    @property
    def vu(self) -> U16Vector2: ...
    @property
    def vv(self) -> U16Vector2: ...
    @property
    def vo(self) -> U16Vector2: ...
    @property
    def vl(self) -> U16Vector2: ...
    @property
    def ou(self) -> U16Vector2: ...
    @property
    def ov(self) -> U16Vector2: ...
    @property
    def lu(self) -> U16Vector2: ...
    @property
    def lv(self) -> U16Vector2: ...
    @property
    def uuu(self) -> U16Vector3: ...
    @property
    def uuv(self) -> U16Vector3: ...
    @property
    def uuo(self) -> U16Vector3: ...
    @property
    def uul(self) -> U16Vector3: ...
    @property
    def uvu(self) -> U16Vector3: ...
    @property
    def uvv(self) -> U16Vector3: ...
    @property
    def uvo(self) -> U16Vector3: ...
    @property
    def uvl(self) -> U16Vector3: ...
    @property
    def uou(self) -> U16Vector3: ...
    @property
    def uov(self) -> U16Vector3: ...
    @property
    def uoo(self) -> U16Vector3: ...
    @property
    def uol(self) -> U16Vector3: ...
    @property
    def ulu(self) -> U16Vector3: ...
    @property
    def ulv(self) -> U16Vector3: ...
    @property
    def ulo(self) -> U16Vector3: ...
    @property
    def ull(self) -> U16Vector3: ...
    @property
    def vuu(self) -> U16Vector3: ...
    @property
    def vuv(self) -> U16Vector3: ...
    @property
    def vuo(self) -> U16Vector3: ...
    @property
    def vul(self) -> U16Vector3: ...
    @property
    def vvu(self) -> U16Vector3: ...
    @property
    def vvv(self) -> U16Vector3: ...
    @property
    def vvo(self) -> U16Vector3: ...
    @property
    def vvl(self) -> U16Vector3: ...
    @property
    def vou(self) -> U16Vector3: ...
    @property
    def vov(self) -> U16Vector3: ...
    @property
    def voo(self) -> U16Vector3: ...
    @property
    def vol(self) -> U16Vector3: ...
    @property
    def vlu(self) -> U16Vector3: ...
    @property
    def vlv(self) -> U16Vector3: ...
    @property
    def vlo(self) -> U16Vector3: ...
    @property
    def vll(self) -> U16Vector3: ...
    @property
    def ouu(self) -> U16Vector3: ...
    @property
    def ouv(self) -> U16Vector3: ...
    @property
    def ouo(self) -> U16Vector3: ...
    @property
    def oul(self) -> U16Vector3: ...
    @property
    def ovu(self) -> U16Vector3: ...
    @property
    def ovv(self) -> U16Vector3: ...
    @property
    def ovo(self) -> U16Vector3: ...
    @property
    def ovl(self) -> U16Vector3: ...
    @property
    def oou(self) -> U16Vector3: ...
    @property
    def oov(self) -> U16Vector3: ...
    @property
    def olu(self) -> U16Vector3: ...
    @property
    def olv(self) -> U16Vector3: ...
    @property
    def luu(self) -> U16Vector3: ...
    @property
    def luv(self) -> U16Vector3: ...
    @property
    def luo(self) -> U16Vector3: ...
    @property
    def lul(self) -> U16Vector3: ...
    @property
    def lvu(self) -> U16Vector3: ...
    @property
    def lvv(self) -> U16Vector3: ...
    @property
    def lvo(self) -> U16Vector3: ...
    @property
    def lvl(self) -> U16Vector3: ...
    @property
    def lou(self) -> U16Vector3: ...
    @property
    def lov(self) -> U16Vector3: ...
    @property
    def llu(self) -> U16Vector3: ...
    @property
    def llv(self) -> U16Vector3: ...
    @property
    def uuuu(self) -> U16Vector4: ...
    @property
    def uuuv(self) -> U16Vector4: ...
    @property
    def uuuo(self) -> U16Vector4: ...
    @property
    def uuul(self) -> U16Vector4: ...
    @property
    def uuvu(self) -> U16Vector4: ...
    @property
    def uuvv(self) -> U16Vector4: ...
    @property
    def uuvo(self) -> U16Vector4: ...
    @property
    def uuvl(self) -> U16Vector4: ...
    @property
    def uuou(self) -> U16Vector4: ...
    @property
    def uuov(self) -> U16Vector4: ...
    @property
    def uuoo(self) -> U16Vector4: ...
    @property
    def uuol(self) -> U16Vector4: ...
    @property
    def uulu(self) -> U16Vector4: ...
    @property
    def uulv(self) -> U16Vector4: ...
    @property
    def uulo(self) -> U16Vector4: ...
    @property
    def uull(self) -> U16Vector4: ...
    @property
    def uvuu(self) -> U16Vector4: ...
    @property
    def uvuv(self) -> U16Vector4: ...
    @property
    def uvuo(self) -> U16Vector4: ...
    @property
    def uvul(self) -> U16Vector4: ...
    @property
    def uvvu(self) -> U16Vector4: ...
    @property
    def uvvv(self) -> U16Vector4: ...
    @property
    def uvvo(self) -> U16Vector4: ...
    @property
    def uvvl(self) -> U16Vector4: ...
    @property
    def uvou(self) -> U16Vector4: ...
    @property
    def uvov(self) -> U16Vector4: ...
    @property
    def uvoo(self) -> U16Vector4: ...
    @property
    def uvol(self) -> U16Vector4: ...
    @property
    def uvlu(self) -> U16Vector4: ...
    @property
    def uvlv(self) -> U16Vector4: ...
    @property
    def uvlo(self) -> U16Vector4: ...
    @property
    def uvll(self) -> U16Vector4: ...
    @property
    def uouu(self) -> U16Vector4: ...
    @property
    def uouv(self) -> U16Vector4: ...
    @property
    def uouo(self) -> U16Vector4: ...
    @property
    def uoul(self) -> U16Vector4: ...
    @property
    def uovu(self) -> U16Vector4: ...
    @property
    def uovv(self) -> U16Vector4: ...
    @property
    def uovo(self) -> U16Vector4: ...
    @property
    def uovl(self) -> U16Vector4: ...
    @property
    def uoou(self) -> U16Vector4: ...
    @property
    def uoov(self) -> U16Vector4: ...
    @property
    def uooo(self) -> U16Vector4: ...
    @property
    def uool(self) -> U16Vector4: ...
    @property
    def uolu(self) -> U16Vector4: ...
    @property
    def uolv(self) -> U16Vector4: ...
    @property
    def uolo(self) -> U16Vector4: ...
    @property
    def uoll(self) -> U16Vector4: ...
    @property
    def uluu(self) -> U16Vector4: ...
    @property
    def uluv(self) -> U16Vector4: ...
    @property
    def uluo(self) -> U16Vector4: ...
    @property
    def ulul(self) -> U16Vector4: ...
    @property
    def ulvu(self) -> U16Vector4: ...
    @property
    def ulvv(self) -> U16Vector4: ...
    @property
    def ulvo(self) -> U16Vector4: ...
    @property
    def ulvl(self) -> U16Vector4: ...
    @property
    def ulou(self) -> U16Vector4: ...
    @property
    def ulov(self) -> U16Vector4: ...
    @property
    def uloo(self) -> U16Vector4: ...
    @property
    def ulol(self) -> U16Vector4: ...
    @property
    def ullu(self) -> U16Vector4: ...
    @property
    def ullv(self) -> U16Vector4: ...
    @property
    def ullo(self) -> U16Vector4: ...
    @property
    def ulll(self) -> U16Vector4: ...
    @property
    def vuuu(self) -> U16Vector4: ...
    @property
    def vuuv(self) -> U16Vector4: ...
    @property
    def vuuo(self) -> U16Vector4: ...
    @property
    def vuul(self) -> U16Vector4: ...
    @property
    def vuvu(self) -> U16Vector4: ...
    @property
    def vuvv(self) -> U16Vector4: ...
    @property
    def vuvo(self) -> U16Vector4: ...
    @property
    def vuvl(self) -> U16Vector4: ...
    @property
    def vuou(self) -> U16Vector4: ...
    @property
    def vuov(self) -> U16Vector4: ...
    @property
    def vuoo(self) -> U16Vector4: ...
    @property
    def vuol(self) -> U16Vector4: ...
    @property
    def vulu(self) -> U16Vector4: ...
    @property
    def vulv(self) -> U16Vector4: ...
    @property
    def vulo(self) -> U16Vector4: ...
    @property
    def vull(self) -> U16Vector4: ...
    @property
    def vvuu(self) -> U16Vector4: ...
    @property
    def vvuv(self) -> U16Vector4: ...
    @property
    def vvuo(self) -> U16Vector4: ...
    @property
    def vvul(self) -> U16Vector4: ...
    @property
    def vvvu(self) -> U16Vector4: ...
    @property
    def vvvv(self) -> U16Vector4: ...
    @property
    def vvvo(self) -> U16Vector4: ...
    @property
    def vvvl(self) -> U16Vector4: ...
    @property
    def vvou(self) -> U16Vector4: ...
    @property
    def vvov(self) -> U16Vector4: ...
    @property
    def vvoo(self) -> U16Vector4: ...
    @property
    def vvol(self) -> U16Vector4: ...
    @property
    def vvlu(self) -> U16Vector4: ...
    @property
    def vvlv(self) -> U16Vector4: ...
    @property
    def vvlo(self) -> U16Vector4: ...
    @property
    def vvll(self) -> U16Vector4: ...
    @property
    def vouu(self) -> U16Vector4: ...
    @property
    def vouv(self) -> U16Vector4: ...
    @property
    def vouo(self) -> U16Vector4: ...
    @property
    def voul(self) -> U16Vector4: ...
    @property
    def vovu(self) -> U16Vector4: ...
    @property
    def vovv(self) -> U16Vector4: ...
    @property
    def vovo(self) -> U16Vector4: ...
    @property
    def vovl(self) -> U16Vector4: ...
    @property
    def voou(self) -> U16Vector4: ...
    @property
    def voov(self) -> U16Vector4: ...
    @property
    def vooo(self) -> U16Vector4: ...
    @property
    def vool(self) -> U16Vector4: ...
    @property
    def volu(self) -> U16Vector4: ...
    @property
    def volv(self) -> U16Vector4: ...
    @property
    def volo(self) -> U16Vector4: ...
    @property
    def voll(self) -> U16Vector4: ...
    @property
    def vluu(self) -> U16Vector4: ...
    @property
    def vluv(self) -> U16Vector4: ...
    @property
    def vluo(self) -> U16Vector4: ...
    @property
    def vlul(self) -> U16Vector4: ...
    @property
    def vlvu(self) -> U16Vector4: ...
    @property
    def vlvv(self) -> U16Vector4: ...
    @property
    def vlvo(self) -> U16Vector4: ...
    @property
    def vlvl(self) -> U16Vector4: ...
    @property
    def vlou(self) -> U16Vector4: ...
    @property
    def vlov(self) -> U16Vector4: ...
    @property
    def vloo(self) -> U16Vector4: ...
    @property
    def vlol(self) -> U16Vector4: ...
    @property
    def vllu(self) -> U16Vector4: ...
    @property
    def vllv(self) -> U16Vector4: ...
    @property
    def vllo(self) -> U16Vector4: ...
    @property
    def vlll(self) -> U16Vector4: ...
    @property
    def ouuu(self) -> U16Vector4: ...
    @property
    def ouuv(self) -> U16Vector4: ...
    @property
    def ouuo(self) -> U16Vector4: ...
    @property
    def ouul(self) -> U16Vector4: ...
    @property
    def ouvu(self) -> U16Vector4: ...
    @property
    def ouvv(self) -> U16Vector4: ...
    @property
    def ouvo(self) -> U16Vector4: ...
    @property
    def ouvl(self) -> U16Vector4: ...
    @property
    def ouou(self) -> U16Vector4: ...
    @property
    def ouov(self) -> U16Vector4: ...
    @property
    def ouoo(self) -> U16Vector4: ...
    @property
    def ouol(self) -> U16Vector4: ...
    @property
    def oulu(self) -> U16Vector4: ...
    @property
    def oulv(self) -> U16Vector4: ...
    @property
    def oulo(self) -> U16Vector4: ...
    @property
    def oull(self) -> U16Vector4: ...
    @property
    def ovuu(self) -> U16Vector4: ...
    @property
    def ovuv(self) -> U16Vector4: ...
    @property
    def ovuo(self) -> U16Vector4: ...
    @property
    def ovul(self) -> U16Vector4: ...
    @property
    def ovvu(self) -> U16Vector4: ...
    @property
    def ovvv(self) -> U16Vector4: ...
    @property
    def ovvo(self) -> U16Vector4: ...
    @property
    def ovvl(self) -> U16Vector4: ...
    @property
    def ovou(self) -> U16Vector4: ...
    @property
    def ovov(self) -> U16Vector4: ...
    @property
    def ovoo(self) -> U16Vector4: ...
    @property
    def ovol(self) -> U16Vector4: ...
    @property
    def ovlu(self) -> U16Vector4: ...
    @property
    def ovlv(self) -> U16Vector4: ...
    @property
    def ovlo(self) -> U16Vector4: ...
    @property
    def ovll(self) -> U16Vector4: ...
    @property
    def oouu(self) -> U16Vector4: ...
    @property
    def oouv(self) -> U16Vector4: ...
    @property
    def oouo(self) -> U16Vector4: ...
    @property
    def ooul(self) -> U16Vector4: ...
    @property
    def oovu(self) -> U16Vector4: ...
    @property
    def oovv(self) -> U16Vector4: ...
    @property
    def oovo(self) -> U16Vector4: ...
    @property
    def oovl(self) -> U16Vector4: ...
    @property
    def ooou(self) -> U16Vector4: ...
    @property
    def ooov(self) -> U16Vector4: ...
    @property
    def oolu(self) -> U16Vector4: ...
    @property
    def oolv(self) -> U16Vector4: ...
    @property
    def oluu(self) -> U16Vector4: ...
    @property
    def oluv(self) -> U16Vector4: ...
    @property
    def oluo(self) -> U16Vector4: ...
    @property
    def olul(self) -> U16Vector4: ...
    @property
    def olvu(self) -> U16Vector4: ...
    @property
    def olvv(self) -> U16Vector4: ...
    @property
    def olvo(self) -> U16Vector4: ...
    @property
    def olvl(self) -> U16Vector4: ...
    @property
    def olou(self) -> U16Vector4: ...
    @property
    def olov(self) -> U16Vector4: ...
    @property
    def ollu(self) -> U16Vector4: ...
    @property
    def ollv(self) -> U16Vector4: ...
    @property
    def luuu(self) -> U16Vector4: ...
    @property
    def luuv(self) -> U16Vector4: ...
    @property
    def luuo(self) -> U16Vector4: ...
    @property
    def luul(self) -> U16Vector4: ...
    @property
    def luvu(self) -> U16Vector4: ...
    @property
    def luvv(self) -> U16Vector4: ...
    @property
    def luvo(self) -> U16Vector4: ...
    @property
    def luvl(self) -> U16Vector4: ...
    @property
    def luou(self) -> U16Vector4: ...
    @property
    def luov(self) -> U16Vector4: ...
    @property
    def luoo(self) -> U16Vector4: ...
    @property
    def luol(self) -> U16Vector4: ...
    @property
    def lulu(self) -> U16Vector4: ...
    @property
    def lulv(self) -> U16Vector4: ...
    @property
    def lulo(self) -> U16Vector4: ...
    @property
    def lull(self) -> U16Vector4: ...
    @property
    def lvuu(self) -> U16Vector4: ...
    @property
    def lvuv(self) -> U16Vector4: ...
    @property
    def lvuo(self) -> U16Vector4: ...
    @property
    def lvul(self) -> U16Vector4: ...
    @property
    def lvvu(self) -> U16Vector4: ...
    @property
    def lvvv(self) -> U16Vector4: ...
    @property
    def lvvo(self) -> U16Vector4: ...
    @property
    def lvvl(self) -> U16Vector4: ...
    @property
    def lvou(self) -> U16Vector4: ...
    @property
    def lvov(self) -> U16Vector4: ...
    @property
    def lvoo(self) -> U16Vector4: ...
    @property
    def lvol(self) -> U16Vector4: ...
    @property
    def lvlu(self) -> U16Vector4: ...
    @property
    def lvlv(self) -> U16Vector4: ...
    @property
    def lvlo(self) -> U16Vector4: ...
    @property
    def lvll(self) -> U16Vector4: ...
    @property
    def louu(self) -> U16Vector4: ...
    @property
    def louv(self) -> U16Vector4: ...
    @property
    def louo(self) -> U16Vector4: ...
    @property
    def loul(self) -> U16Vector4: ...
    @property
    def lovu(self) -> U16Vector4: ...
    @property
    def lovv(self) -> U16Vector4: ...
    @property
    def lovo(self) -> U16Vector4: ...
    @property
    def lovl(self) -> U16Vector4: ...
    @property
    def loou(self) -> U16Vector4: ...
    @property
    def loov(self) -> U16Vector4: ...
    @property
    def lolu(self) -> U16Vector4: ...
    @property
    def lolv(self) -> U16Vector4: ...
    @property
    def lluu(self) -> U16Vector4: ...
    @property
    def lluv(self) -> U16Vector4: ...
    @property
    def lluo(self) -> U16Vector4: ...
    @property
    def llul(self) -> U16Vector4: ...
    @property
    def llvu(self) -> U16Vector4: ...
    @property
    def llvv(self) -> U16Vector4: ...
    @property
    def llvo(self) -> U16Vector4: ...
    @property
    def llvl(self) -> U16Vector4: ...
    @property
    def llou(self) -> U16Vector4: ...
    @property
    def llov(self) -> U16Vector4: ...
    @property
    def lllu(self) -> U16Vector4: ...
    @property
    def lllv(self) -> U16Vector4: ...
    @property
    def oo(self) -> U16Vector2: ...
    @property
    def ol(self) -> U16Vector2: ...
    @property
    def lo(self) -> U16Vector2: ...
    @property
    def ll(self) -> U16Vector2: ...
    @property
    def ooo(self) -> U16Vector3: ...
    @property
    def ool(self) -> U16Vector3: ...
    @property
    def olo(self) -> U16Vector3: ...
    @property
    def oll(self) -> U16Vector3: ...
    @property
    def loo(self) -> U16Vector3: ...
    @property
    def lol(self) -> U16Vector3: ...
    @property
    def llo(self) -> U16Vector3: ...
    @property
    def lll(self) -> U16Vector3: ...
    @property
    def oooo(self) -> U16Vector4: ...
    @property
    def oool(self) -> U16Vector4: ...
    @property
    def oolo(self) -> U16Vector4: ...
    @property
    def ooll(self) -> U16Vector4: ...
    @property
    def oloo(self) -> U16Vector4: ...
    @property
    def olol(self) -> U16Vector4: ...
    @property
    def ollo(self) -> U16Vector4: ...
    @property
    def olll(self) -> U16Vector4: ...
    @property
    def looo(self) -> U16Vector4: ...
    @property
    def lool(self) -> U16Vector4: ...
    @property
    def lolo(self) -> U16Vector4: ...
    @property
    def loll(self) -> U16Vector4: ...
    @property
    def lloo(self) -> U16Vector4: ...
    @property
    def llol(self) -> U16Vector4: ...
    @property
    def lllo(self) -> U16Vector4: ...
    @property
    def llll(self) -> U16Vector4: ...
    def min(self, n: Number, /) -> U16Vector2: ...
    def max(self, n: Number, /) -> U16Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> U16Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U16Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector2: ...

@final
class U16Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U16Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U16Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> U16Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> U16Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[U16Vector2]: ...

@final
class I32Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __add__(self, other: Number) -> I32Vector2: ...
    @overload
    def __radd__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __radd__(self, other: Number) -> I32Vector2: ...
    @overload
    def __sub__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __sub__(self, other: Number) -> I32Vector2: ...
    @overload
    def __rsub__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> I32Vector2: ...
    @overload
    def __mul__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __mul__(self, other: Number) -> I32Vector2: ...
    @overload
    def __rmul__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> I32Vector2: ...
    @overload
    def __truediv__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> I32Vector2: ...
    @overload
    def __rtruediv__(self, other: I32Vector2) -> I32Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> I32Vector2: ...
    def __neg__(self) -> I32Vector2: ...
    def __abs__(self) -> I32Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> I32Vector2: ...
    @property
    def xy(self) -> I32Vector2: ...
    @property
    def xo(self) -> I32Vector2: ...
    @property
    def xl(self) -> I32Vector2: ...
    @property
    def yx(self) -> I32Vector2: ...
    @property
    def yy(self) -> I32Vector2: ...
    @property
    def yo(self) -> I32Vector2: ...
    @property
    def yl(self) -> I32Vector2: ...
    @property
    def ox(self) -> I32Vector2: ...
    @property
    def oy(self) -> I32Vector2: ...
    @property
    def lx(self) -> I32Vector2: ...
    @property
    def ly(self) -> I32Vector2: ...
    @property
    def xxx(self) -> I32Vector3: ...
    @property
    def xxy(self) -> I32Vector3: ...
    @property
    def xxo(self) -> I32Vector3: ...
    @property
    def xxl(self) -> I32Vector3: ...
    @property
    def xyx(self) -> I32Vector3: ...
    @property
    def xyy(self) -> I32Vector3: ...
    @property
    def xyo(self) -> I32Vector3: ...
    @property
    def xyl(self) -> I32Vector3: ...
    @property
    def xox(self) -> I32Vector3: ...
    @property
    def xoy(self) -> I32Vector3: ...
    @property
    def xoo(self) -> I32Vector3: ...
    @property
    def xol(self) -> I32Vector3: ...
    @property
    def xlx(self) -> I32Vector3: ...
    @property
    def xly(self) -> I32Vector3: ...
    @property
    def xlo(self) -> I32Vector3: ...
    @property
    def xll(self) -> I32Vector3: ...
    @property
    def yxx(self) -> I32Vector3: ...
    @property
    def yxy(self) -> I32Vector3: ...
    @property
    def yxo(self) -> I32Vector3: ...
    @property
    def yxl(self) -> I32Vector3: ...
    @property
    def yyx(self) -> I32Vector3: ...
    @property
    def yyy(self) -> I32Vector3: ...
    @property
    def yyo(self) -> I32Vector3: ...
    @property
    def yyl(self) -> I32Vector3: ...
    @property
    def yox(self) -> I32Vector3: ...
    @property
    def yoy(self) -> I32Vector3: ...
    @property
    def yoo(self) -> I32Vector3: ...
    @property
    def yol(self) -> I32Vector3: ...
    @property
    def ylx(self) -> I32Vector3: ...
    @property
    def yly(self) -> I32Vector3: ...
    @property
    def ylo(self) -> I32Vector3: ...
    @property
    def yll(self) -> I32Vector3: ...
    @property
    def oxx(self) -> I32Vector3: ...
    @property
    def oxy(self) -> I32Vector3: ...
    @property
    def oxo(self) -> I32Vector3: ...
    @property
    def oxl(self) -> I32Vector3: ...
    @property
    def oyx(self) -> I32Vector3: ...
    @property
    def oyy(self) -> I32Vector3: ...
    @property
    def oyo(self) -> I32Vector3: ...
    @property
    def oyl(self) -> I32Vector3: ...
    @property
    def oox(self) -> I32Vector3: ...
    @property
    def ooy(self) -> I32Vector3: ...
    @property
    def olx(self) -> I32Vector3: ...
    @property
    def oly(self) -> I32Vector3: ...
    @property
    def lxx(self) -> I32Vector3: ...
    @property
    def lxy(self) -> I32Vector3: ...
    @property
    def lxo(self) -> I32Vector3: ...
    @property
    def lxl(self) -> I32Vector3: ...
    @property
    def lyx(self) -> I32Vector3: ...
    @property
    def lyy(self) -> I32Vector3: ...
    @property
    def lyo(self) -> I32Vector3: ...
    @property
    def lyl(self) -> I32Vector3: ...
    @property
    def lox(self) -> I32Vector3: ...
    @property
    def loy(self) -> I32Vector3: ...
    @property
    def llx(self) -> I32Vector3: ...
    @property
    def lly(self) -> I32Vector3: ...
    @property
    def xxxx(self) -> I32Vector4: ...
    @property
    def xxxy(self) -> I32Vector4: ...
    @property
    def xxxo(self) -> I32Vector4: ...
    @property
    def xxxl(self) -> I32Vector4: ...
    @property
    def xxyx(self) -> I32Vector4: ...
    @property
    def xxyy(self) -> I32Vector4: ...
    @property
    def xxyo(self) -> I32Vector4: ...
    @property
    def xxyl(self) -> I32Vector4: ...
    @property
    def xxox(self) -> I32Vector4: ...
    @property
    def xxoy(self) -> I32Vector4: ...
    @property
    def xxoo(self) -> I32Vector4: ...
    @property
    def xxol(self) -> I32Vector4: ...
    @property
    def xxlx(self) -> I32Vector4: ...
    @property
    def xxly(self) -> I32Vector4: ...
    @property
    def xxlo(self) -> I32Vector4: ...
    @property
    def xxll(self) -> I32Vector4: ...
    @property
    def xyxx(self) -> I32Vector4: ...
    @property
    def xyxy(self) -> I32Vector4: ...
    @property
    def xyxo(self) -> I32Vector4: ...
    @property
    def xyxl(self) -> I32Vector4: ...
    @property
    def xyyx(self) -> I32Vector4: ...
    @property
    def xyyy(self) -> I32Vector4: ...
    @property
    def xyyo(self) -> I32Vector4: ...
    @property
    def xyyl(self) -> I32Vector4: ...
    @property
    def xyox(self) -> I32Vector4: ...
    @property
    def xyoy(self) -> I32Vector4: ...
    @property
    def xyoo(self) -> I32Vector4: ...
    @property
    def xyol(self) -> I32Vector4: ...
    @property
    def xylx(self) -> I32Vector4: ...
    @property
    def xyly(self) -> I32Vector4: ...
    @property
    def xylo(self) -> I32Vector4: ...
    @property
    def xyll(self) -> I32Vector4: ...
    @property
    def xoxx(self) -> I32Vector4: ...
    @property
    def xoxy(self) -> I32Vector4: ...
    @property
    def xoxo(self) -> I32Vector4: ...
    @property
    def xoxl(self) -> I32Vector4: ...
    @property
    def xoyx(self) -> I32Vector4: ...
    @property
    def xoyy(self) -> I32Vector4: ...
    @property
    def xoyo(self) -> I32Vector4: ...
    @property
    def xoyl(self) -> I32Vector4: ...
    @property
    def xoox(self) -> I32Vector4: ...
    @property
    def xooy(self) -> I32Vector4: ...
    @property
    def xooo(self) -> I32Vector4: ...
    @property
    def xool(self) -> I32Vector4: ...
    @property
    def xolx(self) -> I32Vector4: ...
    @property
    def xoly(self) -> I32Vector4: ...
    @property
    def xolo(self) -> I32Vector4: ...
    @property
    def xoll(self) -> I32Vector4: ...
    @property
    def xlxx(self) -> I32Vector4: ...
    @property
    def xlxy(self) -> I32Vector4: ...
    @property
    def xlxo(self) -> I32Vector4: ...
    @property
    def xlxl(self) -> I32Vector4: ...
    @property
    def xlyx(self) -> I32Vector4: ...
    @property
    def xlyy(self) -> I32Vector4: ...
    @property
    def xlyo(self) -> I32Vector4: ...
    @property
    def xlyl(self) -> I32Vector4: ...
    @property
    def xlox(self) -> I32Vector4: ...
    @property
    def xloy(self) -> I32Vector4: ...
    @property
    def xloo(self) -> I32Vector4: ...
    @property
    def xlol(self) -> I32Vector4: ...
    @property
    def xllx(self) -> I32Vector4: ...
    @property
    def xlly(self) -> I32Vector4: ...
    @property
    def xllo(self) -> I32Vector4: ...
    @property
    def xlll(self) -> I32Vector4: ...
    @property
    def yxxx(self) -> I32Vector4: ...
    @property
    def yxxy(self) -> I32Vector4: ...
    @property
    def yxxo(self) -> I32Vector4: ...
    @property
    def yxxl(self) -> I32Vector4: ...
    @property
    def yxyx(self) -> I32Vector4: ...
    @property
    def yxyy(self) -> I32Vector4: ...
    @property
    def yxyo(self) -> I32Vector4: ...
    @property
    def yxyl(self) -> I32Vector4: ...
    @property
    def yxox(self) -> I32Vector4: ...
    @property
    def yxoy(self) -> I32Vector4: ...
    @property
    def yxoo(self) -> I32Vector4: ...
    @property
    def yxol(self) -> I32Vector4: ...
    @property
    def yxlx(self) -> I32Vector4: ...
    @property
    def yxly(self) -> I32Vector4: ...
    @property
    def yxlo(self) -> I32Vector4: ...
    @property
    def yxll(self) -> I32Vector4: ...
    @property
    def yyxx(self) -> I32Vector4: ...
    @property
    def yyxy(self) -> I32Vector4: ...
    @property
    def yyxo(self) -> I32Vector4: ...
    @property
    def yyxl(self) -> I32Vector4: ...
    @property
    def yyyx(self) -> I32Vector4: ...
    @property
    def yyyy(self) -> I32Vector4: ...
    @property
    def yyyo(self) -> I32Vector4: ...
    @property
    def yyyl(self) -> I32Vector4: ...
    @property
    def yyox(self) -> I32Vector4: ...
    @property
    def yyoy(self) -> I32Vector4: ...
    @property
    def yyoo(self) -> I32Vector4: ...
    @property
    def yyol(self) -> I32Vector4: ...
    @property
    def yylx(self) -> I32Vector4: ...
    @property
    def yyly(self) -> I32Vector4: ...
    @property
    def yylo(self) -> I32Vector4: ...
    @property
    def yyll(self) -> I32Vector4: ...
    @property
    def yoxx(self) -> I32Vector4: ...
    @property
    def yoxy(self) -> I32Vector4: ...
    @property
    def yoxo(self) -> I32Vector4: ...
    @property
    def yoxl(self) -> I32Vector4: ...
    @property
    def yoyx(self) -> I32Vector4: ...
    @property
    def yoyy(self) -> I32Vector4: ...
    @property
    def yoyo(self) -> I32Vector4: ...
    @property
    def yoyl(self) -> I32Vector4: ...
    @property
    def yoox(self) -> I32Vector4: ...
    @property
    def yooy(self) -> I32Vector4: ...
    @property
    def yooo(self) -> I32Vector4: ...
    @property
    def yool(self) -> I32Vector4: ...
    @property
    def yolx(self) -> I32Vector4: ...
    @property
    def yoly(self) -> I32Vector4: ...
    @property
    def yolo(self) -> I32Vector4: ...
    @property
    def yoll(self) -> I32Vector4: ...
    @property
    def ylxx(self) -> I32Vector4: ...
    @property
    def ylxy(self) -> I32Vector4: ...
    @property
    def ylxo(self) -> I32Vector4: ...
    @property
    def ylxl(self) -> I32Vector4: ...
    @property
    def ylyx(self) -> I32Vector4: ...
    @property
    def ylyy(self) -> I32Vector4: ...
    @property
    def ylyo(self) -> I32Vector4: ...
    @property
    def ylyl(self) -> I32Vector4: ...
    @property
    def ylox(self) -> I32Vector4: ...
    @property
    def yloy(self) -> I32Vector4: ...
    @property
    def yloo(self) -> I32Vector4: ...
    @property
    def ylol(self) -> I32Vector4: ...
    @property
    def yllx(self) -> I32Vector4: ...
    @property
    def ylly(self) -> I32Vector4: ...
    @property
    def yllo(self) -> I32Vector4: ...
    @property
    def ylll(self) -> I32Vector4: ...
    @property
    def oxxx(self) -> I32Vector4: ...
    @property
    def oxxy(self) -> I32Vector4: ...
    @property
    def oxxo(self) -> I32Vector4: ...
    @property
    def oxxl(self) -> I32Vector4: ...
    @property
    def oxyx(self) -> I32Vector4: ...
    @property
    def oxyy(self) -> I32Vector4: ...
    @property
    def oxyo(self) -> I32Vector4: ...
    @property
    def oxyl(self) -> I32Vector4: ...
    @property
    def oxox(self) -> I32Vector4: ...
    @property
    def oxoy(self) -> I32Vector4: ...
    @property
    def oxoo(self) -> I32Vector4: ...
    @property
    def oxol(self) -> I32Vector4: ...
    @property
    def oxlx(self) -> I32Vector4: ...
    @property
    def oxly(self) -> I32Vector4: ...
    @property
    def oxlo(self) -> I32Vector4: ...
    @property
    def oxll(self) -> I32Vector4: ...
    @property
    def oyxx(self) -> I32Vector4: ...
    @property
    def oyxy(self) -> I32Vector4: ...
    @property
    def oyxo(self) -> I32Vector4: ...
    @property
    def oyxl(self) -> I32Vector4: ...
    @property
    def oyyx(self) -> I32Vector4: ...
    @property
    def oyyy(self) -> I32Vector4: ...
    @property
    def oyyo(self) -> I32Vector4: ...
    @property
    def oyyl(self) -> I32Vector4: ...
    @property
    def oyox(self) -> I32Vector4: ...
    @property
    def oyoy(self) -> I32Vector4: ...
    @property
    def oyoo(self) -> I32Vector4: ...
    @property
    def oyol(self) -> I32Vector4: ...
    @property
    def oylx(self) -> I32Vector4: ...
    @property
    def oyly(self) -> I32Vector4: ...
    @property
    def oylo(self) -> I32Vector4: ...
    @property
    def oyll(self) -> I32Vector4: ...
    @property
    def ooxx(self) -> I32Vector4: ...
    @property
    def ooxy(self) -> I32Vector4: ...
    @property
    def ooxo(self) -> I32Vector4: ...
    @property
    def ooxl(self) -> I32Vector4: ...
    @property
    def ooyx(self) -> I32Vector4: ...
    @property
    def ooyy(self) -> I32Vector4: ...
    @property
    def ooyo(self) -> I32Vector4: ...
    @property
    def ooyl(self) -> I32Vector4: ...
    @property
    def ooox(self) -> I32Vector4: ...
    @property
    def oooy(self) -> I32Vector4: ...
    @property
    def oolx(self) -> I32Vector4: ...
    @property
    def ooly(self) -> I32Vector4: ...
    @property
    def olxx(self) -> I32Vector4: ...
    @property
    def olxy(self) -> I32Vector4: ...
    @property
    def olxo(self) -> I32Vector4: ...
    @property
    def olxl(self) -> I32Vector4: ...
    @property
    def olyx(self) -> I32Vector4: ...
    @property
    def olyy(self) -> I32Vector4: ...
    @property
    def olyo(self) -> I32Vector4: ...
    @property
    def olyl(self) -> I32Vector4: ...
    @property
    def olox(self) -> I32Vector4: ...
    @property
    def oloy(self) -> I32Vector4: ...
    @property
    def ollx(self) -> I32Vector4: ...
    @property
    def olly(self) -> I32Vector4: ...
    @property
    def lxxx(self) -> I32Vector4: ...
    @property
    def lxxy(self) -> I32Vector4: ...
    @property
    def lxxo(self) -> I32Vector4: ...
    @property
    def lxxl(self) -> I32Vector4: ...
    @property
    def lxyx(self) -> I32Vector4: ...
    @property
    def lxyy(self) -> I32Vector4: ...
    @property
    def lxyo(self) -> I32Vector4: ...
    @property
    def lxyl(self) -> I32Vector4: ...
    @property
    def lxox(self) -> I32Vector4: ...
    @property
    def lxoy(self) -> I32Vector4: ...
    @property
    def lxoo(self) -> I32Vector4: ...
    @property
    def lxol(self) -> I32Vector4: ...
    @property
    def lxlx(self) -> I32Vector4: ...
    @property
    def lxly(self) -> I32Vector4: ...
    @property
    def lxlo(self) -> I32Vector4: ...
    @property
    def lxll(self) -> I32Vector4: ...
    @property
    def lyxx(self) -> I32Vector4: ...
    @property
    def lyxy(self) -> I32Vector4: ...
    @property
    def lyxo(self) -> I32Vector4: ...
    @property
    def lyxl(self) -> I32Vector4: ...
    @property
    def lyyx(self) -> I32Vector4: ...
    @property
    def lyyy(self) -> I32Vector4: ...
    @property
    def lyyo(self) -> I32Vector4: ...
    @property
    def lyyl(self) -> I32Vector4: ...
    @property
    def lyox(self) -> I32Vector4: ...
    @property
    def lyoy(self) -> I32Vector4: ...
    @property
    def lyoo(self) -> I32Vector4: ...
    @property
    def lyol(self) -> I32Vector4: ...
    @property
    def lylx(self) -> I32Vector4: ...
    @property
    def lyly(self) -> I32Vector4: ...
    @property
    def lylo(self) -> I32Vector4: ...
    @property
    def lyll(self) -> I32Vector4: ...
    @property
    def loxx(self) -> I32Vector4: ...
    @property
    def loxy(self) -> I32Vector4: ...
    @property
    def loxo(self) -> I32Vector4: ...
    @property
    def loxl(self) -> I32Vector4: ...
    @property
    def loyx(self) -> I32Vector4: ...
    @property
    def loyy(self) -> I32Vector4: ...
    @property
    def loyo(self) -> I32Vector4: ...
    @property
    def loyl(self) -> I32Vector4: ...
    @property
    def loox(self) -> I32Vector4: ...
    @property
    def looy(self) -> I32Vector4: ...
    @property
    def lolx(self) -> I32Vector4: ...
    @property
    def loly(self) -> I32Vector4: ...
    @property
    def llxx(self) -> I32Vector4: ...
    @property
    def llxy(self) -> I32Vector4: ...
    @property
    def llxo(self) -> I32Vector4: ...
    @property
    def llxl(self) -> I32Vector4: ...
    @property
    def llyx(self) -> I32Vector4: ...
    @property
    def llyy(self) -> I32Vector4: ...
    @property
    def llyo(self) -> I32Vector4: ...
    @property
    def llyl(self) -> I32Vector4: ...
    @property
    def llox(self) -> I32Vector4: ...
    @property
    def lloy(self) -> I32Vector4: ...
    @property
    def lllx(self) -> I32Vector4: ...
    @property
    def llly(self) -> I32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> I32Vector2: ...
    @property
    def rg(self) -> I32Vector2: ...
    @property
    def ro(self) -> I32Vector2: ...
    @property
    def rl(self) -> I32Vector2: ...
    @property
    def gr(self) -> I32Vector2: ...
    @property
    def gg(self) -> I32Vector2: ...
    @property
    def go(self) -> I32Vector2: ...
    @property
    def gl(self) -> I32Vector2: ...
    @property
    def og(self) -> I32Vector2: ...
    @property
    def lr(self) -> I32Vector2: ...
    @property
    def lg(self) -> I32Vector2: ...
    @property
    def rrr(self) -> I32Vector3: ...
    @property
    def rrg(self) -> I32Vector3: ...
    @property
    def rro(self) -> I32Vector3: ...
    @property
    def rrl(self) -> I32Vector3: ...
    @property
    def rgr(self) -> I32Vector3: ...
    @property
    def rgg(self) -> I32Vector3: ...
    @property
    def rgo(self) -> I32Vector3: ...
    @property
    def rgl(self) -> I32Vector3: ...
    @property
    def ror(self) -> I32Vector3: ...
    @property
    def rog(self) -> I32Vector3: ...
    @property
    def roo(self) -> I32Vector3: ...
    @property
    def rol(self) -> I32Vector3: ...
    @property
    def rlr(self) -> I32Vector3: ...
    @property
    def rlg(self) -> I32Vector3: ...
    @property
    def rlo(self) -> I32Vector3: ...
    @property
    def rll(self) -> I32Vector3: ...
    @property
    def grr(self) -> I32Vector3: ...
    @property
    def grg(self) -> I32Vector3: ...
    @property
    def gro(self) -> I32Vector3: ...
    @property
    def grl(self) -> I32Vector3: ...
    @property
    def ggr(self) -> I32Vector3: ...
    @property
    def ggg(self) -> I32Vector3: ...
    @property
    def ggo(self) -> I32Vector3: ...
    @property
    def ggl(self) -> I32Vector3: ...
    @property
    def gor(self) -> I32Vector3: ...
    @property
    def gog(self) -> I32Vector3: ...
    @property
    def goo(self) -> I32Vector3: ...
    @property
    def gol(self) -> I32Vector3: ...
    @property
    def glr(self) -> I32Vector3: ...
    @property
    def glg(self) -> I32Vector3: ...
    @property
    def glo(self) -> I32Vector3: ...
    @property
    def gll(self) -> I32Vector3: ...
    @property
    def orr(self) -> I32Vector3: ...
    @property
    def org(self) -> I32Vector3: ...
    @property
    def oro(self) -> I32Vector3: ...
    @property
    def orl(self) -> I32Vector3: ...
    @property
    def ogr(self) -> I32Vector3: ...
    @property
    def ogg(self) -> I32Vector3: ...
    @property
    def ogo(self) -> I32Vector3: ...
    @property
    def ogl(self) -> I32Vector3: ...
    @property
    def oor(self) -> I32Vector3: ...
    @property
    def oog(self) -> I32Vector3: ...
    @property
    def olr(self) -> I32Vector3: ...
    @property
    def olg(self) -> I32Vector3: ...
    @property
    def lrr(self) -> I32Vector3: ...
    @property
    def lrg(self) -> I32Vector3: ...
    @property
    def lro(self) -> I32Vector3: ...
    @property
    def lrl(self) -> I32Vector3: ...
    @property
    def lgr(self) -> I32Vector3: ...
    @property
    def lgg(self) -> I32Vector3: ...
    @property
    def lgo(self) -> I32Vector3: ...
    @property
    def lgl(self) -> I32Vector3: ...
    @property
    def lor(self) -> I32Vector3: ...
    @property
    def log(self) -> I32Vector3: ...
    @property
    def llr(self) -> I32Vector3: ...
    @property
    def llg(self) -> I32Vector3: ...
    @property
    def rrrr(self) -> I32Vector4: ...
    @property
    def rrrg(self) -> I32Vector4: ...
    @property
    def rrro(self) -> I32Vector4: ...
    @property
    def rrrl(self) -> I32Vector4: ...
    @property
    def rrgr(self) -> I32Vector4: ...
    @property
    def rrgg(self) -> I32Vector4: ...
    @property
    def rrgo(self) -> I32Vector4: ...
    @property
    def rrgl(self) -> I32Vector4: ...
    @property
    def rror(self) -> I32Vector4: ...
    @property
    def rrog(self) -> I32Vector4: ...
    @property
    def rroo(self) -> I32Vector4: ...
    @property
    def rrol(self) -> I32Vector4: ...
    @property
    def rrlr(self) -> I32Vector4: ...
    @property
    def rrlg(self) -> I32Vector4: ...
    @property
    def rrlo(self) -> I32Vector4: ...
    @property
    def rrll(self) -> I32Vector4: ...
    @property
    def rgrr(self) -> I32Vector4: ...
    @property
    def rgrg(self) -> I32Vector4: ...
    @property
    def rgro(self) -> I32Vector4: ...
    @property
    def rgrl(self) -> I32Vector4: ...
    @property
    def rggr(self) -> I32Vector4: ...
    @property
    def rggg(self) -> I32Vector4: ...
    @property
    def rggo(self) -> I32Vector4: ...
    @property
    def rggl(self) -> I32Vector4: ...
    @property
    def rgor(self) -> I32Vector4: ...
    @property
    def rgog(self) -> I32Vector4: ...
    @property
    def rgoo(self) -> I32Vector4: ...
    @property
    def rgol(self) -> I32Vector4: ...
    @property
    def rglr(self) -> I32Vector4: ...
    @property
    def rglg(self) -> I32Vector4: ...
    @property
    def rglo(self) -> I32Vector4: ...
    @property
    def rgll(self) -> I32Vector4: ...
    @property
    def rorr(self) -> I32Vector4: ...
    @property
    def rorg(self) -> I32Vector4: ...
    @property
    def roro(self) -> I32Vector4: ...
    @property
    def rorl(self) -> I32Vector4: ...
    @property
    def rogr(self) -> I32Vector4: ...
    @property
    def rogg(self) -> I32Vector4: ...
    @property
    def rogo(self) -> I32Vector4: ...
    @property
    def rogl(self) -> I32Vector4: ...
    @property
    def roor(self) -> I32Vector4: ...
    @property
    def roog(self) -> I32Vector4: ...
    @property
    def rooo(self) -> I32Vector4: ...
    @property
    def rool(self) -> I32Vector4: ...
    @property
    def rolr(self) -> I32Vector4: ...
    @property
    def rolg(self) -> I32Vector4: ...
    @property
    def rolo(self) -> I32Vector4: ...
    @property
    def roll(self) -> I32Vector4: ...
    @property
    def rlrr(self) -> I32Vector4: ...
    @property
    def rlrg(self) -> I32Vector4: ...
    @property
    def rlro(self) -> I32Vector4: ...
    @property
    def rlrl(self) -> I32Vector4: ...
    @property
    def rlgr(self) -> I32Vector4: ...
    @property
    def rlgg(self) -> I32Vector4: ...
    @property
    def rlgo(self) -> I32Vector4: ...
    @property
    def rlgl(self) -> I32Vector4: ...
    @property
    def rlor(self) -> I32Vector4: ...
    @property
    def rlog(self) -> I32Vector4: ...
    @property
    def rloo(self) -> I32Vector4: ...
    @property
    def rlol(self) -> I32Vector4: ...
    @property
    def rllr(self) -> I32Vector4: ...
    @property
    def rllg(self) -> I32Vector4: ...
    @property
    def rllo(self) -> I32Vector4: ...
    @property
    def rlll(self) -> I32Vector4: ...
    @property
    def grrr(self) -> I32Vector4: ...
    @property
    def grrg(self) -> I32Vector4: ...
    @property
    def grro(self) -> I32Vector4: ...
    @property
    def grrl(self) -> I32Vector4: ...
    @property
    def grgr(self) -> I32Vector4: ...
    @property
    def grgg(self) -> I32Vector4: ...
    @property
    def grgo(self) -> I32Vector4: ...
    @property
    def grgl(self) -> I32Vector4: ...
    @property
    def gror(self) -> I32Vector4: ...
    @property
    def grog(self) -> I32Vector4: ...
    @property
    def groo(self) -> I32Vector4: ...
    @property
    def grol(self) -> I32Vector4: ...
    @property
    def grlr(self) -> I32Vector4: ...
    @property
    def grlg(self) -> I32Vector4: ...
    @property
    def grlo(self) -> I32Vector4: ...
    @property
    def grll(self) -> I32Vector4: ...
    @property
    def ggrr(self) -> I32Vector4: ...
    @property
    def ggrg(self) -> I32Vector4: ...
    @property
    def ggro(self) -> I32Vector4: ...
    @property
    def ggrl(self) -> I32Vector4: ...
    @property
    def gggr(self) -> I32Vector4: ...
    @property
    def gggg(self) -> I32Vector4: ...
    @property
    def gggo(self) -> I32Vector4: ...
    @property
    def gggl(self) -> I32Vector4: ...
    @property
    def ggor(self) -> I32Vector4: ...
    @property
    def ggog(self) -> I32Vector4: ...
    @property
    def ggoo(self) -> I32Vector4: ...
    @property
    def ggol(self) -> I32Vector4: ...
    @property
    def gglr(self) -> I32Vector4: ...
    @property
    def gglg(self) -> I32Vector4: ...
    @property
    def gglo(self) -> I32Vector4: ...
    @property
    def ggll(self) -> I32Vector4: ...
    @property
    def gorr(self) -> I32Vector4: ...
    @property
    def gorg(self) -> I32Vector4: ...
    @property
    def goro(self) -> I32Vector4: ...
    @property
    def gorl(self) -> I32Vector4: ...
    @property
    def gogr(self) -> I32Vector4: ...
    @property
    def gogg(self) -> I32Vector4: ...
    @property
    def gogo(self) -> I32Vector4: ...
    @property
    def gogl(self) -> I32Vector4: ...
    @property
    def goor(self) -> I32Vector4: ...
    @property
    def goog(self) -> I32Vector4: ...
    @property
    def gooo(self) -> I32Vector4: ...
    @property
    def gool(self) -> I32Vector4: ...
    @property
    def golr(self) -> I32Vector4: ...
    @property
    def golg(self) -> I32Vector4: ...
    @property
    def golo(self) -> I32Vector4: ...
    @property
    def goll(self) -> I32Vector4: ...
    @property
    def glrr(self) -> I32Vector4: ...
    @property
    def glrg(self) -> I32Vector4: ...
    @property
    def glro(self) -> I32Vector4: ...
    @property
    def glrl(self) -> I32Vector4: ...
    @property
    def glgr(self) -> I32Vector4: ...
    @property
    def glgg(self) -> I32Vector4: ...
    @property
    def glgo(self) -> I32Vector4: ...
    @property
    def glgl(self) -> I32Vector4: ...
    @property
    def glor(self) -> I32Vector4: ...
    @property
    def glog(self) -> I32Vector4: ...
    @property
    def gloo(self) -> I32Vector4: ...
    @property
    def glol(self) -> I32Vector4: ...
    @property
    def gllr(self) -> I32Vector4: ...
    @property
    def gllg(self) -> I32Vector4: ...
    @property
    def gllo(self) -> I32Vector4: ...
    @property
    def glll(self) -> I32Vector4: ...
    @property
    def orrr(self) -> I32Vector4: ...
    @property
    def orrg(self) -> I32Vector4: ...
    @property
    def orro(self) -> I32Vector4: ...
    @property
    def orrl(self) -> I32Vector4: ...
    @property
    def orgr(self) -> I32Vector4: ...
    @property
    def orgg(self) -> I32Vector4: ...
    @property
    def orgo(self) -> I32Vector4: ...
    @property
    def orgl(self) -> I32Vector4: ...
    @property
    def oror(self) -> I32Vector4: ...
    @property
    def orog(self) -> I32Vector4: ...
    @property
    def oroo(self) -> I32Vector4: ...
    @property
    def orol(self) -> I32Vector4: ...
    @property
    def orlr(self) -> I32Vector4: ...
    @property
    def orlg(self) -> I32Vector4: ...
    @property
    def orlo(self) -> I32Vector4: ...
    @property
    def orll(self) -> I32Vector4: ...
    @property
    def ogrr(self) -> I32Vector4: ...
    @property
    def ogrg(self) -> I32Vector4: ...
    @property
    def ogro(self) -> I32Vector4: ...
    @property
    def ogrl(self) -> I32Vector4: ...
    @property
    def oggr(self) -> I32Vector4: ...
    @property
    def oggg(self) -> I32Vector4: ...
    @property
    def oggo(self) -> I32Vector4: ...
    @property
    def oggl(self) -> I32Vector4: ...
    @property
    def ogor(self) -> I32Vector4: ...
    @property
    def ogog(self) -> I32Vector4: ...
    @property
    def ogoo(self) -> I32Vector4: ...
    @property
    def ogol(self) -> I32Vector4: ...
    @property
    def oglr(self) -> I32Vector4: ...
    @property
    def oglg(self) -> I32Vector4: ...
    @property
    def oglo(self) -> I32Vector4: ...
    @property
    def ogll(self) -> I32Vector4: ...
    @property
    def oorr(self) -> I32Vector4: ...
    @property
    def oorg(self) -> I32Vector4: ...
    @property
    def ooro(self) -> I32Vector4: ...
    @property
    def oorl(self) -> I32Vector4: ...
    @property
    def oogr(self) -> I32Vector4: ...
    @property
    def oogg(self) -> I32Vector4: ...
    @property
    def oogo(self) -> I32Vector4: ...
    @property
    def oogl(self) -> I32Vector4: ...
    @property
    def ooor(self) -> I32Vector4: ...
    @property
    def ooog(self) -> I32Vector4: ...
    @property
    def oolr(self) -> I32Vector4: ...
    @property
    def oolg(self) -> I32Vector4: ...
    @property
    def olrr(self) -> I32Vector4: ...
    @property
    def olrg(self) -> I32Vector4: ...
    @property
    def olro(self) -> I32Vector4: ...
    @property
    def olrl(self) -> I32Vector4: ...
    @property
    def olgr(self) -> I32Vector4: ...
    @property
    def olgg(self) -> I32Vector4: ...
    @property
    def olgo(self) -> I32Vector4: ...
    @property
    def olgl(self) -> I32Vector4: ...
    @property
    def olor(self) -> I32Vector4: ...
    @property
    def olog(self) -> I32Vector4: ...
    @property
    def ollr(self) -> I32Vector4: ...
    @property
    def ollg(self) -> I32Vector4: ...
    @property
    def lrrr(self) -> I32Vector4: ...
    @property
    def lrrg(self) -> I32Vector4: ...
    @property
    def lrro(self) -> I32Vector4: ...
    @property
    def lrrl(self) -> I32Vector4: ...
    @property
    def lrgr(self) -> I32Vector4: ...
    @property
    def lrgg(self) -> I32Vector4: ...
    @property
    def lrgo(self) -> I32Vector4: ...
    @property
    def lrgl(self) -> I32Vector4: ...
    @property
    def lror(self) -> I32Vector4: ...
    @property
    def lrog(self) -> I32Vector4: ...
    @property
    def lroo(self) -> I32Vector4: ...
    @property
    def lrol(self) -> I32Vector4: ...
    @property
    def lrlr(self) -> I32Vector4: ...
    @property
    def lrlg(self) -> I32Vector4: ...
    @property
    def lrlo(self) -> I32Vector4: ...
    @property
    def lrll(self) -> I32Vector4: ...
    @property
    def lgrr(self) -> I32Vector4: ...
    @property
    def lgrg(self) -> I32Vector4: ...
    @property
    def lgro(self) -> I32Vector4: ...
    @property
    def lgrl(self) -> I32Vector4: ...
    @property
    def lggr(self) -> I32Vector4: ...
    @property
    def lggg(self) -> I32Vector4: ...
    @property
    def lggo(self) -> I32Vector4: ...
    @property
    def lggl(self) -> I32Vector4: ...
    @property
    def lgor(self) -> I32Vector4: ...
    @property
    def lgog(self) -> I32Vector4: ...
    @property
    def lgoo(self) -> I32Vector4: ...
    @property
    def lgol(self) -> I32Vector4: ...
    @property
    def lglr(self) -> I32Vector4: ...
    @property
    def lglg(self) -> I32Vector4: ...
    @property
    def lglo(self) -> I32Vector4: ...
    @property
    def lgll(self) -> I32Vector4: ...
    @property
    def lorr(self) -> I32Vector4: ...
    @property
    def lorg(self) -> I32Vector4: ...
    @property
    def loro(self) -> I32Vector4: ...
    @property
    def lorl(self) -> I32Vector4: ...
    @property
    def logr(self) -> I32Vector4: ...
    @property
    def logg(self) -> I32Vector4: ...
    @property
    def logo(self) -> I32Vector4: ...
    @property
    def logl(self) -> I32Vector4: ...
    @property
    def loor(self) -> I32Vector4: ...
    @property
    def loog(self) -> I32Vector4: ...
    @property
    def lolr(self) -> I32Vector4: ...
    @property
    def lolg(self) -> I32Vector4: ...
    @property
    def llrr(self) -> I32Vector4: ...
    @property
    def llrg(self) -> I32Vector4: ...
    @property
    def llro(self) -> I32Vector4: ...
    @property
    def llrl(self) -> I32Vector4: ...
    @property
    def llgr(self) -> I32Vector4: ...
    @property
    def llgg(self) -> I32Vector4: ...
    @property
    def llgo(self) -> I32Vector4: ...
    @property
    def llgl(self) -> I32Vector4: ...
    @property
    def llor(self) -> I32Vector4: ...
    @property
    def llog(self) -> I32Vector4: ...
    @property
    def lllr(self) -> I32Vector4: ...
    @property
    def lllg(self) -> I32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> I32Vector2: ...
    @property
    def st(self) -> I32Vector2: ...
    @property
    def so(self) -> I32Vector2: ...
    @property
    def sl(self) -> I32Vector2: ...
    @property
    def ts(self) -> I32Vector2: ...
    @property
    def tt(self) -> I32Vector2: ...
    @property
    def to(self) -> I32Vector2: ...
    @property
    def tl(self) -> I32Vector2: ...
    @property
    def os(self) -> I32Vector2: ...
    @property
    def ot(self) -> I32Vector2: ...
    @property
    def ls(self) -> I32Vector2: ...
    @property
    def lt(self) -> I32Vector2: ...
    @property
    def sss(self) -> I32Vector3: ...
    @property
    def sst(self) -> I32Vector3: ...
    @property
    def sso(self) -> I32Vector3: ...
    @property
    def ssl(self) -> I32Vector3: ...
    @property
    def sts(self) -> I32Vector3: ...
    @property
    def stt(self) -> I32Vector3: ...
    @property
    def sto(self) -> I32Vector3: ...
    @property
    def stl(self) -> I32Vector3: ...
    @property
    def sos(self) -> I32Vector3: ...
    @property
    def sot(self) -> I32Vector3: ...
    @property
    def soo(self) -> I32Vector3: ...
    @property
    def sol(self) -> I32Vector3: ...
    @property
    def sls(self) -> I32Vector3: ...
    @property
    def slt(self) -> I32Vector3: ...
    @property
    def slo(self) -> I32Vector3: ...
    @property
    def sll(self) -> I32Vector3: ...
    @property
    def tss(self) -> I32Vector3: ...
    @property
    def tst(self) -> I32Vector3: ...
    @property
    def tso(self) -> I32Vector3: ...
    @property
    def tsl(self) -> I32Vector3: ...
    @property
    def tts(self) -> I32Vector3: ...
    @property
    def ttt(self) -> I32Vector3: ...
    @property
    def tto(self) -> I32Vector3: ...
    @property
    def ttl(self) -> I32Vector3: ...
    @property
    def tos(self) -> I32Vector3: ...
    @property
    def tot(self) -> I32Vector3: ...
    @property
    def too(self) -> I32Vector3: ...
    @property
    def tol(self) -> I32Vector3: ...
    @property
    def tls(self) -> I32Vector3: ...
    @property
    def tlt(self) -> I32Vector3: ...
    @property
    def tlo(self) -> I32Vector3: ...
    @property
    def tll(self) -> I32Vector3: ...
    @property
    def oss(self) -> I32Vector3: ...
    @property
    def ost(self) -> I32Vector3: ...
    @property
    def oso(self) -> I32Vector3: ...
    @property
    def osl(self) -> I32Vector3: ...
    @property
    def ots(self) -> I32Vector3: ...
    @property
    def ott(self) -> I32Vector3: ...
    @property
    def oto(self) -> I32Vector3: ...
    @property
    def otl(self) -> I32Vector3: ...
    @property
    def oos(self) -> I32Vector3: ...
    @property
    def oot(self) -> I32Vector3: ...
    @property
    def ols(self) -> I32Vector3: ...
    @property
    def olt(self) -> I32Vector3: ...
    @property
    def lss(self) -> I32Vector3: ...
    @property
    def lst(self) -> I32Vector3: ...
    @property
    def lso(self) -> I32Vector3: ...
    @property
    def lsl(self) -> I32Vector3: ...
    @property
    def lts(self) -> I32Vector3: ...
    @property
    def ltt(self) -> I32Vector3: ...
    @property
    def lto(self) -> I32Vector3: ...
    @property
    def ltl(self) -> I32Vector3: ...
    @property
    def los(self) -> I32Vector3: ...
    @property
    def lot(self) -> I32Vector3: ...
    @property
    def lls(self) -> I32Vector3: ...
    @property
    def llt(self) -> I32Vector3: ...
    @property
    def ssss(self) -> I32Vector4: ...
    @property
    def ssst(self) -> I32Vector4: ...
    @property
    def ssso(self) -> I32Vector4: ...
    @property
    def sssl(self) -> I32Vector4: ...
    @property
    def ssts(self) -> I32Vector4: ...
    @property
    def sstt(self) -> I32Vector4: ...
    @property
    def ssto(self) -> I32Vector4: ...
    @property
    def sstl(self) -> I32Vector4: ...
    @property
    def ssos(self) -> I32Vector4: ...
    @property
    def ssot(self) -> I32Vector4: ...
    @property
    def ssoo(self) -> I32Vector4: ...
    @property
    def ssol(self) -> I32Vector4: ...
    @property
    def ssls(self) -> I32Vector4: ...
    @property
    def sslt(self) -> I32Vector4: ...
    @property
    def sslo(self) -> I32Vector4: ...
    @property
    def ssll(self) -> I32Vector4: ...
    @property
    def stss(self) -> I32Vector4: ...
    @property
    def stst(self) -> I32Vector4: ...
    @property
    def stso(self) -> I32Vector4: ...
    @property
    def stsl(self) -> I32Vector4: ...
    @property
    def stts(self) -> I32Vector4: ...
    @property
    def sttt(self) -> I32Vector4: ...
    @property
    def stto(self) -> I32Vector4: ...
    @property
    def sttl(self) -> I32Vector4: ...
    @property
    def stos(self) -> I32Vector4: ...
    @property
    def stot(self) -> I32Vector4: ...
    @property
    def stoo(self) -> I32Vector4: ...
    @property
    def stol(self) -> I32Vector4: ...
    @property
    def stls(self) -> I32Vector4: ...
    @property
    def stlt(self) -> I32Vector4: ...
    @property
    def stlo(self) -> I32Vector4: ...
    @property
    def stll(self) -> I32Vector4: ...
    @property
    def soss(self) -> I32Vector4: ...
    @property
    def sost(self) -> I32Vector4: ...
    @property
    def soso(self) -> I32Vector4: ...
    @property
    def sosl(self) -> I32Vector4: ...
    @property
    def sots(self) -> I32Vector4: ...
    @property
    def sott(self) -> I32Vector4: ...
    @property
    def soto(self) -> I32Vector4: ...
    @property
    def sotl(self) -> I32Vector4: ...
    @property
    def soos(self) -> I32Vector4: ...
    @property
    def soot(self) -> I32Vector4: ...
    @property
    def sooo(self) -> I32Vector4: ...
    @property
    def sool(self) -> I32Vector4: ...
    @property
    def sols(self) -> I32Vector4: ...
    @property
    def solt(self) -> I32Vector4: ...
    @property
    def solo(self) -> I32Vector4: ...
    @property
    def soll(self) -> I32Vector4: ...
    @property
    def slss(self) -> I32Vector4: ...
    @property
    def slst(self) -> I32Vector4: ...
    @property
    def slso(self) -> I32Vector4: ...
    @property
    def slsl(self) -> I32Vector4: ...
    @property
    def slts(self) -> I32Vector4: ...
    @property
    def sltt(self) -> I32Vector4: ...
    @property
    def slto(self) -> I32Vector4: ...
    @property
    def sltl(self) -> I32Vector4: ...
    @property
    def slos(self) -> I32Vector4: ...
    @property
    def slot(self) -> I32Vector4: ...
    @property
    def sloo(self) -> I32Vector4: ...
    @property
    def slol(self) -> I32Vector4: ...
    @property
    def slls(self) -> I32Vector4: ...
    @property
    def sllt(self) -> I32Vector4: ...
    @property
    def sllo(self) -> I32Vector4: ...
    @property
    def slll(self) -> I32Vector4: ...
    @property
    def tsss(self) -> I32Vector4: ...
    @property
    def tsst(self) -> I32Vector4: ...
    @property
    def tsso(self) -> I32Vector4: ...
    @property
    def tssl(self) -> I32Vector4: ...
    @property
    def tsts(self) -> I32Vector4: ...
    @property
    def tstt(self) -> I32Vector4: ...
    @property
    def tsto(self) -> I32Vector4: ...
    @property
    def tstl(self) -> I32Vector4: ...
    @property
    def tsos(self) -> I32Vector4: ...
    @property
    def tsot(self) -> I32Vector4: ...
    @property
    def tsoo(self) -> I32Vector4: ...
    @property
    def tsol(self) -> I32Vector4: ...
    @property
    def tsls(self) -> I32Vector4: ...
    @property
    def tslt(self) -> I32Vector4: ...
    @property
    def tslo(self) -> I32Vector4: ...
    @property
    def tsll(self) -> I32Vector4: ...
    @property
    def ttss(self) -> I32Vector4: ...
    @property
    def ttst(self) -> I32Vector4: ...
    @property
    def ttso(self) -> I32Vector4: ...
    @property
    def ttsl(self) -> I32Vector4: ...
    @property
    def ttts(self) -> I32Vector4: ...
    @property
    def tttt(self) -> I32Vector4: ...
    @property
    def ttto(self) -> I32Vector4: ...
    @property
    def tttl(self) -> I32Vector4: ...
    @property
    def ttos(self) -> I32Vector4: ...
    @property
    def ttot(self) -> I32Vector4: ...
    @property
    def ttoo(self) -> I32Vector4: ...
    @property
    def ttol(self) -> I32Vector4: ...
    @property
    def ttls(self) -> I32Vector4: ...
    @property
    def ttlt(self) -> I32Vector4: ...
    @property
    def ttlo(self) -> I32Vector4: ...
    @property
    def ttll(self) -> I32Vector4: ...
    @property
    def toss(self) -> I32Vector4: ...
    @property
    def tost(self) -> I32Vector4: ...
    @property
    def toso(self) -> I32Vector4: ...
    @property
    def tosl(self) -> I32Vector4: ...
    @property
    def tots(self) -> I32Vector4: ...
    @property
    def tott(self) -> I32Vector4: ...
    @property
    def toto(self) -> I32Vector4: ...
    @property
    def totl(self) -> I32Vector4: ...
    @property
    def toos(self) -> I32Vector4: ...
    @property
    def toot(self) -> I32Vector4: ...
    @property
    def tooo(self) -> I32Vector4: ...
    @property
    def tool(self) -> I32Vector4: ...
    @property
    def tols(self) -> I32Vector4: ...
    @property
    def tolt(self) -> I32Vector4: ...
    @property
    def tolo(self) -> I32Vector4: ...
    @property
    def toll(self) -> I32Vector4: ...
    @property
    def tlss(self) -> I32Vector4: ...
    @property
    def tlst(self) -> I32Vector4: ...
    @property
    def tlso(self) -> I32Vector4: ...
    @property
    def tlsl(self) -> I32Vector4: ...
    @property
    def tlts(self) -> I32Vector4: ...
    @property
    def tltt(self) -> I32Vector4: ...
    @property
    def tlto(self) -> I32Vector4: ...
    @property
    def tltl(self) -> I32Vector4: ...
    @property
    def tlos(self) -> I32Vector4: ...
    @property
    def tlot(self) -> I32Vector4: ...
    @property
    def tloo(self) -> I32Vector4: ...
    @property
    def tlol(self) -> I32Vector4: ...
    @property
    def tlls(self) -> I32Vector4: ...
    @property
    def tllt(self) -> I32Vector4: ...
    @property
    def tllo(self) -> I32Vector4: ...
    @property
    def tlll(self) -> I32Vector4: ...
    @property
    def osss(self) -> I32Vector4: ...
    @property
    def osst(self) -> I32Vector4: ...
    @property
    def osso(self) -> I32Vector4: ...
    @property
    def ossl(self) -> I32Vector4: ...
    @property
    def osts(self) -> I32Vector4: ...
    @property
    def ostt(self) -> I32Vector4: ...
    @property
    def osto(self) -> I32Vector4: ...
    @property
    def ostl(self) -> I32Vector4: ...
    @property
    def osos(self) -> I32Vector4: ...
    @property
    def osot(self) -> I32Vector4: ...
    @property
    def osoo(self) -> I32Vector4: ...
    @property
    def osol(self) -> I32Vector4: ...
    @property
    def osls(self) -> I32Vector4: ...
    @property
    def oslt(self) -> I32Vector4: ...
    @property
    def oslo(self) -> I32Vector4: ...
    @property
    def osll(self) -> I32Vector4: ...
    @property
    def otss(self) -> I32Vector4: ...
    @property
    def otst(self) -> I32Vector4: ...
    @property
    def otso(self) -> I32Vector4: ...
    @property
    def otsl(self) -> I32Vector4: ...
    @property
    def otts(self) -> I32Vector4: ...
    @property
    def ottt(self) -> I32Vector4: ...
    @property
    def otto(self) -> I32Vector4: ...
    @property
    def ottl(self) -> I32Vector4: ...
    @property
    def otos(self) -> I32Vector4: ...
    @property
    def otot(self) -> I32Vector4: ...
    @property
    def otoo(self) -> I32Vector4: ...
    @property
    def otol(self) -> I32Vector4: ...
    @property
    def otls(self) -> I32Vector4: ...
    @property
    def otlt(self) -> I32Vector4: ...
    @property
    def otlo(self) -> I32Vector4: ...
    @property
    def otll(self) -> I32Vector4: ...
    @property
    def ooss(self) -> I32Vector4: ...
    @property
    def oost(self) -> I32Vector4: ...
    @property
    def ooso(self) -> I32Vector4: ...
    @property
    def oosl(self) -> I32Vector4: ...
    @property
    def oots(self) -> I32Vector4: ...
    @property
    def oott(self) -> I32Vector4: ...
    @property
    def ooto(self) -> I32Vector4: ...
    @property
    def ootl(self) -> I32Vector4: ...
    @property
    def ooos(self) -> I32Vector4: ...
    @property
    def ooot(self) -> I32Vector4: ...
    @property
    def ools(self) -> I32Vector4: ...
    @property
    def oolt(self) -> I32Vector4: ...
    @property
    def olss(self) -> I32Vector4: ...
    @property
    def olst(self) -> I32Vector4: ...
    @property
    def olso(self) -> I32Vector4: ...
    @property
    def olsl(self) -> I32Vector4: ...
    @property
    def olts(self) -> I32Vector4: ...
    @property
    def oltt(self) -> I32Vector4: ...
    @property
    def olto(self) -> I32Vector4: ...
    @property
    def oltl(self) -> I32Vector4: ...
    @property
    def olos(self) -> I32Vector4: ...
    @property
    def olot(self) -> I32Vector4: ...
    @property
    def olls(self) -> I32Vector4: ...
    @property
    def ollt(self) -> I32Vector4: ...
    @property
    def lsss(self) -> I32Vector4: ...
    @property
    def lsst(self) -> I32Vector4: ...
    @property
    def lsso(self) -> I32Vector4: ...
    @property
    def lssl(self) -> I32Vector4: ...
    @property
    def lsts(self) -> I32Vector4: ...
    @property
    def lstt(self) -> I32Vector4: ...
    @property
    def lsto(self) -> I32Vector4: ...
    @property
    def lstl(self) -> I32Vector4: ...
    @property
    def lsos(self) -> I32Vector4: ...
    @property
    def lsot(self) -> I32Vector4: ...
    @property
    def lsoo(self) -> I32Vector4: ...
    @property
    def lsol(self) -> I32Vector4: ...
    @property
    def lsls(self) -> I32Vector4: ...
    @property
    def lslt(self) -> I32Vector4: ...
    @property
    def lslo(self) -> I32Vector4: ...
    @property
    def lsll(self) -> I32Vector4: ...
    @property
    def ltss(self) -> I32Vector4: ...
    @property
    def ltst(self) -> I32Vector4: ...
    @property
    def ltso(self) -> I32Vector4: ...
    @property
    def ltsl(self) -> I32Vector4: ...
    @property
    def ltts(self) -> I32Vector4: ...
    @property
    def lttt(self) -> I32Vector4: ...
    @property
    def ltto(self) -> I32Vector4: ...
    @property
    def lttl(self) -> I32Vector4: ...
    @property
    def ltos(self) -> I32Vector4: ...
    @property
    def ltot(self) -> I32Vector4: ...
    @property
    def ltoo(self) -> I32Vector4: ...
    @property
    def ltol(self) -> I32Vector4: ...
    @property
    def ltls(self) -> I32Vector4: ...
    @property
    def ltlt(self) -> I32Vector4: ...
    @property
    def ltlo(self) -> I32Vector4: ...
    @property
    def ltll(self) -> I32Vector4: ...
    @property
    def loss(self) -> I32Vector4: ...
    @property
    def lost(self) -> I32Vector4: ...
    @property
    def loso(self) -> I32Vector4: ...
    @property
    def losl(self) -> I32Vector4: ...
    @property
    def lots(self) -> I32Vector4: ...
    @property
    def lott(self) -> I32Vector4: ...
    @property
    def loto(self) -> I32Vector4: ...
    @property
    def lotl(self) -> I32Vector4: ...
    @property
    def loos(self) -> I32Vector4: ...
    @property
    def loot(self) -> I32Vector4: ...
    @property
    def lols(self) -> I32Vector4: ...
    @property
    def lolt(self) -> I32Vector4: ...
    @property
    def llss(self) -> I32Vector4: ...
    @property
    def llst(self) -> I32Vector4: ...
    @property
    def llso(self) -> I32Vector4: ...
    @property
    def llsl(self) -> I32Vector4: ...
    @property
    def llts(self) -> I32Vector4: ...
    @property
    def lltt(self) -> I32Vector4: ...
    @property
    def llto(self) -> I32Vector4: ...
    @property
    def lltl(self) -> I32Vector4: ...
    @property
    def llos(self) -> I32Vector4: ...
    @property
    def llot(self) -> I32Vector4: ...
    @property
    def llls(self) -> I32Vector4: ...
    @property
    def lllt(self) -> I32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I32Vector2: ...
    @property
    def uv(self) -> I32Vector2: ...
    @property
    def uo(self) -> I32Vector2: ...
    @property
    def ul(self) -> I32Vector2: ...
    @property
    def vu(self) -> I32Vector2: ...
    @property
    def vv(self) -> I32Vector2: ...
    @property
    def vo(self) -> I32Vector2: ...
    @property
    def vl(self) -> I32Vector2: ...
    @property
    def ou(self) -> I32Vector2: ...
    @property
    def ov(self) -> I32Vector2: ...
    @property
    def lu(self) -> I32Vector2: ...
    @property
    def lv(self) -> I32Vector2: ...
    @property
    def uuu(self) -> I32Vector3: ...
    @property
    def uuv(self) -> I32Vector3: ...
    @property
    def uuo(self) -> I32Vector3: ...
    @property
    def uul(self) -> I32Vector3: ...
    @property
    def uvu(self) -> I32Vector3: ...
    @property
    def uvv(self) -> I32Vector3: ...
    @property
    def uvo(self) -> I32Vector3: ...
    @property
    def uvl(self) -> I32Vector3: ...
    @property
    def uou(self) -> I32Vector3: ...
    @property
    def uov(self) -> I32Vector3: ...
    @property
    def uoo(self) -> I32Vector3: ...
    @property
    def uol(self) -> I32Vector3: ...
    @property
    def ulu(self) -> I32Vector3: ...
    @property
    def ulv(self) -> I32Vector3: ...
    @property
    def ulo(self) -> I32Vector3: ...
    @property
    def ull(self) -> I32Vector3: ...
    @property
    def vuu(self) -> I32Vector3: ...
    @property
    def vuv(self) -> I32Vector3: ...
    @property
    def vuo(self) -> I32Vector3: ...
    @property
    def vul(self) -> I32Vector3: ...
    @property
    def vvu(self) -> I32Vector3: ...
    @property
    def vvv(self) -> I32Vector3: ...
    @property
    def vvo(self) -> I32Vector3: ...
    @property
    def vvl(self) -> I32Vector3: ...
    @property
    def vou(self) -> I32Vector3: ...
    @property
    def vov(self) -> I32Vector3: ...
    @property
    def voo(self) -> I32Vector3: ...
    @property
    def vol(self) -> I32Vector3: ...
    @property
    def vlu(self) -> I32Vector3: ...
    @property
    def vlv(self) -> I32Vector3: ...
    @property
    def vlo(self) -> I32Vector3: ...
    @property
    def vll(self) -> I32Vector3: ...
    @property
    def ouu(self) -> I32Vector3: ...
    @property
    def ouv(self) -> I32Vector3: ...
    @property
    def ouo(self) -> I32Vector3: ...
    @property
    def oul(self) -> I32Vector3: ...
    @property
    def ovu(self) -> I32Vector3: ...
    @property
    def ovv(self) -> I32Vector3: ...
    @property
    def ovo(self) -> I32Vector3: ...
    @property
    def ovl(self) -> I32Vector3: ...
    @property
    def oou(self) -> I32Vector3: ...
    @property
    def oov(self) -> I32Vector3: ...
    @property
    def olu(self) -> I32Vector3: ...
    @property
    def olv(self) -> I32Vector3: ...
    @property
    def luu(self) -> I32Vector3: ...
    @property
    def luv(self) -> I32Vector3: ...
    @property
    def luo(self) -> I32Vector3: ...
    @property
    def lul(self) -> I32Vector3: ...
    @property
    def lvu(self) -> I32Vector3: ...
    @property
    def lvv(self) -> I32Vector3: ...
    @property
    def lvo(self) -> I32Vector3: ...
    @property
    def lvl(self) -> I32Vector3: ...
    @property
    def lou(self) -> I32Vector3: ...
    @property
    def lov(self) -> I32Vector3: ...
    @property
    def llu(self) -> I32Vector3: ...
    @property
    def llv(self) -> I32Vector3: ...
    @property
    def uuuu(self) -> I32Vector4: ...
    @property
    def uuuv(self) -> I32Vector4: ...
    @property
    def uuuo(self) -> I32Vector4: ...
    @property
    def uuul(self) -> I32Vector4: ...
    @property
    def uuvu(self) -> I32Vector4: ...
    @property
    def uuvv(self) -> I32Vector4: ...
    @property
    def uuvo(self) -> I32Vector4: ...
    @property
    def uuvl(self) -> I32Vector4: ...
    @property
    def uuou(self) -> I32Vector4: ...
    @property
    def uuov(self) -> I32Vector4: ...
    @property
    def uuoo(self) -> I32Vector4: ...
    @property
    def uuol(self) -> I32Vector4: ...
    @property
    def uulu(self) -> I32Vector4: ...
    @property
    def uulv(self) -> I32Vector4: ...
    @property
    def uulo(self) -> I32Vector4: ...
    @property
    def uull(self) -> I32Vector4: ...
    @property
    def uvuu(self) -> I32Vector4: ...
    @property
    def uvuv(self) -> I32Vector4: ...
    @property
    def uvuo(self) -> I32Vector4: ...
    @property
    def uvul(self) -> I32Vector4: ...
    @property
    def uvvu(self) -> I32Vector4: ...
    @property
    def uvvv(self) -> I32Vector4: ...
    @property
    def uvvo(self) -> I32Vector4: ...
    @property
    def uvvl(self) -> I32Vector4: ...
    @property
    def uvou(self) -> I32Vector4: ...
    @property
    def uvov(self) -> I32Vector4: ...
    @property
    def uvoo(self) -> I32Vector4: ...
    @property
    def uvol(self) -> I32Vector4: ...
    @property
    def uvlu(self) -> I32Vector4: ...
    @property
    def uvlv(self) -> I32Vector4: ...
    @property
    def uvlo(self) -> I32Vector4: ...
    @property
    def uvll(self) -> I32Vector4: ...
    @property
    def uouu(self) -> I32Vector4: ...
    @property
    def uouv(self) -> I32Vector4: ...
    @property
    def uouo(self) -> I32Vector4: ...
    @property
    def uoul(self) -> I32Vector4: ...
    @property
    def uovu(self) -> I32Vector4: ...
    @property
    def uovv(self) -> I32Vector4: ...
    @property
    def uovo(self) -> I32Vector4: ...
    @property
    def uovl(self) -> I32Vector4: ...
    @property
    def uoou(self) -> I32Vector4: ...
    @property
    def uoov(self) -> I32Vector4: ...
    @property
    def uooo(self) -> I32Vector4: ...
    @property
    def uool(self) -> I32Vector4: ...
    @property
    def uolu(self) -> I32Vector4: ...
    @property
    def uolv(self) -> I32Vector4: ...
    @property
    def uolo(self) -> I32Vector4: ...
    @property
    def uoll(self) -> I32Vector4: ...
    @property
    def uluu(self) -> I32Vector4: ...
    @property
    def uluv(self) -> I32Vector4: ...
    @property
    def uluo(self) -> I32Vector4: ...
    @property
    def ulul(self) -> I32Vector4: ...
    @property
    def ulvu(self) -> I32Vector4: ...
    @property
    def ulvv(self) -> I32Vector4: ...
    @property
    def ulvo(self) -> I32Vector4: ...
    @property
    def ulvl(self) -> I32Vector4: ...
    @property
    def ulou(self) -> I32Vector4: ...
    @property
    def ulov(self) -> I32Vector4: ...
    @property
    def uloo(self) -> I32Vector4: ...
    @property
    def ulol(self) -> I32Vector4: ...
    @property
    def ullu(self) -> I32Vector4: ...
    @property
    def ullv(self) -> I32Vector4: ...
    @property
    def ullo(self) -> I32Vector4: ...
    @property
    def ulll(self) -> I32Vector4: ...
    @property
    def vuuu(self) -> I32Vector4: ...
    @property
    def vuuv(self) -> I32Vector4: ...
    @property
    def vuuo(self) -> I32Vector4: ...
    @property
    def vuul(self) -> I32Vector4: ...
    @property
    def vuvu(self) -> I32Vector4: ...
    @property
    def vuvv(self) -> I32Vector4: ...
    @property
    def vuvo(self) -> I32Vector4: ...
    @property
    def vuvl(self) -> I32Vector4: ...
    @property
    def vuou(self) -> I32Vector4: ...
    @property
    def vuov(self) -> I32Vector4: ...
    @property
    def vuoo(self) -> I32Vector4: ...
    @property
    def vuol(self) -> I32Vector4: ...
    @property
    def vulu(self) -> I32Vector4: ...
    @property
    def vulv(self) -> I32Vector4: ...
    @property
    def vulo(self) -> I32Vector4: ...
    @property
    def vull(self) -> I32Vector4: ...
    @property
    def vvuu(self) -> I32Vector4: ...
    @property
    def vvuv(self) -> I32Vector4: ...
    @property
    def vvuo(self) -> I32Vector4: ...
    @property
    def vvul(self) -> I32Vector4: ...
    @property
    def vvvu(self) -> I32Vector4: ...
    @property
    def vvvv(self) -> I32Vector4: ...
    @property
    def vvvo(self) -> I32Vector4: ...
    @property
    def vvvl(self) -> I32Vector4: ...
    @property
    def vvou(self) -> I32Vector4: ...
    @property
    def vvov(self) -> I32Vector4: ...
    @property
    def vvoo(self) -> I32Vector4: ...
    @property
    def vvol(self) -> I32Vector4: ...
    @property
    def vvlu(self) -> I32Vector4: ...
    @property
    def vvlv(self) -> I32Vector4: ...
    @property
    def vvlo(self) -> I32Vector4: ...
    @property
    def vvll(self) -> I32Vector4: ...
    @property
    def vouu(self) -> I32Vector4: ...
    @property
    def vouv(self) -> I32Vector4: ...
    @property
    def vouo(self) -> I32Vector4: ...
    @property
    def voul(self) -> I32Vector4: ...
    @property
    def vovu(self) -> I32Vector4: ...
    @property
    def vovv(self) -> I32Vector4: ...
    @property
    def vovo(self) -> I32Vector4: ...
    @property
    def vovl(self) -> I32Vector4: ...
    @property
    def voou(self) -> I32Vector4: ...
    @property
    def voov(self) -> I32Vector4: ...
    @property
    def vooo(self) -> I32Vector4: ...
    @property
    def vool(self) -> I32Vector4: ...
    @property
    def volu(self) -> I32Vector4: ...
    @property
    def volv(self) -> I32Vector4: ...
    @property
    def volo(self) -> I32Vector4: ...
    @property
    def voll(self) -> I32Vector4: ...
    @property
    def vluu(self) -> I32Vector4: ...
    @property
    def vluv(self) -> I32Vector4: ...
    @property
    def vluo(self) -> I32Vector4: ...
    @property
    def vlul(self) -> I32Vector4: ...
    @property
    def vlvu(self) -> I32Vector4: ...
    @property
    def vlvv(self) -> I32Vector4: ...
    @property
    def vlvo(self) -> I32Vector4: ...
    @property
    def vlvl(self) -> I32Vector4: ...
    @property
    def vlou(self) -> I32Vector4: ...
    @property
    def vlov(self) -> I32Vector4: ...
    @property
    def vloo(self) -> I32Vector4: ...
    @property
    def vlol(self) -> I32Vector4: ...
    @property
    def vllu(self) -> I32Vector4: ...
    @property
    def vllv(self) -> I32Vector4: ...
    @property
    def vllo(self) -> I32Vector4: ...
    @property
    def vlll(self) -> I32Vector4: ...
    @property
    def ouuu(self) -> I32Vector4: ...
    @property
    def ouuv(self) -> I32Vector4: ...
    @property
    def ouuo(self) -> I32Vector4: ...
    @property
    def ouul(self) -> I32Vector4: ...
    @property
    def ouvu(self) -> I32Vector4: ...
    @property
    def ouvv(self) -> I32Vector4: ...
    @property
    def ouvo(self) -> I32Vector4: ...
    @property
    def ouvl(self) -> I32Vector4: ...
    @property
    def ouou(self) -> I32Vector4: ...
    @property
    def ouov(self) -> I32Vector4: ...
    @property
    def ouoo(self) -> I32Vector4: ...
    @property
    def ouol(self) -> I32Vector4: ...
    @property
    def oulu(self) -> I32Vector4: ...
    @property
    def oulv(self) -> I32Vector4: ...
    @property
    def oulo(self) -> I32Vector4: ...
    @property
    def oull(self) -> I32Vector4: ...
    @property
    def ovuu(self) -> I32Vector4: ...
    @property
    def ovuv(self) -> I32Vector4: ...
    @property
    def ovuo(self) -> I32Vector4: ...
    @property
    def ovul(self) -> I32Vector4: ...
    @property
    def ovvu(self) -> I32Vector4: ...
    @property
    def ovvv(self) -> I32Vector4: ...
    @property
    def ovvo(self) -> I32Vector4: ...
    @property
    def ovvl(self) -> I32Vector4: ...
    @property
    def ovou(self) -> I32Vector4: ...
    @property
    def ovov(self) -> I32Vector4: ...
    @property
    def ovoo(self) -> I32Vector4: ...
    @property
    def ovol(self) -> I32Vector4: ...
    @property
    def ovlu(self) -> I32Vector4: ...
    @property
    def ovlv(self) -> I32Vector4: ...
    @property
    def ovlo(self) -> I32Vector4: ...
    @property
    def ovll(self) -> I32Vector4: ...
    @property
    def oouu(self) -> I32Vector4: ...
    @property
    def oouv(self) -> I32Vector4: ...
    @property
    def oouo(self) -> I32Vector4: ...
    @property
    def ooul(self) -> I32Vector4: ...
    @property
    def oovu(self) -> I32Vector4: ...
    @property
    def oovv(self) -> I32Vector4: ...
    @property
    def oovo(self) -> I32Vector4: ...
    @property
    def oovl(self) -> I32Vector4: ...
    @property
    def ooou(self) -> I32Vector4: ...
    @property
    def ooov(self) -> I32Vector4: ...
    @property
    def oolu(self) -> I32Vector4: ...
    @property
    def oolv(self) -> I32Vector4: ...
    @property
    def oluu(self) -> I32Vector4: ...
    @property
    def oluv(self) -> I32Vector4: ...
    @property
    def oluo(self) -> I32Vector4: ...
    @property
    def olul(self) -> I32Vector4: ...
    @property
    def olvu(self) -> I32Vector4: ...
    @property
    def olvv(self) -> I32Vector4: ...
    @property
    def olvo(self) -> I32Vector4: ...
    @property
    def olvl(self) -> I32Vector4: ...
    @property
    def olou(self) -> I32Vector4: ...
    @property
    def olov(self) -> I32Vector4: ...
    @property
    def ollu(self) -> I32Vector4: ...
    @property
    def ollv(self) -> I32Vector4: ...
    @property
    def luuu(self) -> I32Vector4: ...
    @property
    def luuv(self) -> I32Vector4: ...
    @property
    def luuo(self) -> I32Vector4: ...
    @property
    def luul(self) -> I32Vector4: ...
    @property
    def luvu(self) -> I32Vector4: ...
    @property
    def luvv(self) -> I32Vector4: ...
    @property
    def luvo(self) -> I32Vector4: ...
    @property
    def luvl(self) -> I32Vector4: ...
    @property
    def luou(self) -> I32Vector4: ...
    @property
    def luov(self) -> I32Vector4: ...
    @property
    def luoo(self) -> I32Vector4: ...
    @property
    def luol(self) -> I32Vector4: ...
    @property
    def lulu(self) -> I32Vector4: ...
    @property
    def lulv(self) -> I32Vector4: ...
    @property
    def lulo(self) -> I32Vector4: ...
    @property
    def lull(self) -> I32Vector4: ...
    @property
    def lvuu(self) -> I32Vector4: ...
    @property
    def lvuv(self) -> I32Vector4: ...
    @property
    def lvuo(self) -> I32Vector4: ...
    @property
    def lvul(self) -> I32Vector4: ...
    @property
    def lvvu(self) -> I32Vector4: ...
    @property
    def lvvv(self) -> I32Vector4: ...
    @property
    def lvvo(self) -> I32Vector4: ...
    @property
    def lvvl(self) -> I32Vector4: ...
    @property
    def lvou(self) -> I32Vector4: ...
    @property
    def lvov(self) -> I32Vector4: ...
    @property
    def lvoo(self) -> I32Vector4: ...
    @property
    def lvol(self) -> I32Vector4: ...
    @property
    def lvlu(self) -> I32Vector4: ...
    @property
    def lvlv(self) -> I32Vector4: ...
    @property
    def lvlo(self) -> I32Vector4: ...
    @property
    def lvll(self) -> I32Vector4: ...
    @property
    def louu(self) -> I32Vector4: ...
    @property
    def louv(self) -> I32Vector4: ...
    @property
    def louo(self) -> I32Vector4: ...
    @property
    def loul(self) -> I32Vector4: ...
    @property
    def lovu(self) -> I32Vector4: ...
    @property
    def lovv(self) -> I32Vector4: ...
    @property
    def lovo(self) -> I32Vector4: ...
    @property
    def lovl(self) -> I32Vector4: ...
    @property
    def loou(self) -> I32Vector4: ...
    @property
    def loov(self) -> I32Vector4: ...
    @property
    def lolu(self) -> I32Vector4: ...
    @property
    def lolv(self) -> I32Vector4: ...
    @property
    def lluu(self) -> I32Vector4: ...
    @property
    def lluv(self) -> I32Vector4: ...
    @property
    def lluo(self) -> I32Vector4: ...
    @property
    def llul(self) -> I32Vector4: ...
    @property
    def llvu(self) -> I32Vector4: ...
    @property
    def llvv(self) -> I32Vector4: ...
    @property
    def llvo(self) -> I32Vector4: ...
    @property
    def llvl(self) -> I32Vector4: ...
    @property
    def llou(self) -> I32Vector4: ...
    @property
    def llov(self) -> I32Vector4: ...
    @property
    def lllu(self) -> I32Vector4: ...
    @property
    def lllv(self) -> I32Vector4: ...
    @property
    def oo(self) -> I32Vector2: ...
    @property
    def ol(self) -> I32Vector2: ...
    @property
    def lo(self) -> I32Vector2: ...
    @property
    def ll(self) -> I32Vector2: ...
    @property
    def ooo(self) -> I32Vector3: ...
    @property
    def ool(self) -> I32Vector3: ...
    @property
    def olo(self) -> I32Vector3: ...
    @property
    def oll(self) -> I32Vector3: ...
    @property
    def loo(self) -> I32Vector3: ...
    @property
    def lol(self) -> I32Vector3: ...
    @property
    def llo(self) -> I32Vector3: ...
    @property
    def lll(self) -> I32Vector3: ...
    @property
    def oooo(self) -> I32Vector4: ...
    @property
    def oool(self) -> I32Vector4: ...
    @property
    def oolo(self) -> I32Vector4: ...
    @property
    def ooll(self) -> I32Vector4: ...
    @property
    def oloo(self) -> I32Vector4: ...
    @property
    def olol(self) -> I32Vector4: ...
    @property
    def ollo(self) -> I32Vector4: ...
    @property
    def olll(self) -> I32Vector4: ...
    @property
    def looo(self) -> I32Vector4: ...
    @property
    def lool(self) -> I32Vector4: ...
    @property
    def lolo(self) -> I32Vector4: ...
    @property
    def loll(self) -> I32Vector4: ...
    @property
    def lloo(self) -> I32Vector4: ...
    @property
    def llol(self) -> I32Vector4: ...
    @property
    def lllo(self) -> I32Vector4: ...
    @property
    def llll(self) -> I32Vector4: ...
    def min(self, n: Number, /) -> I32Vector2: ...
    def max(self, n: Number, /) -> I32Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> I32Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I32Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector2: ...

@final
class I32Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I32Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I32Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> I32Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> I32Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[I32Vector2]: ...

@final
class U32Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __add__(self, other: Number) -> U32Vector2: ...
    @overload
    def __radd__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __radd__(self, other: Number) -> U32Vector2: ...
    @overload
    def __sub__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __sub__(self, other: Number) -> U32Vector2: ...
    @overload
    def __rsub__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> U32Vector2: ...
    @overload
    def __mul__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __mul__(self, other: Number) -> U32Vector2: ...
    @overload
    def __rmul__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> U32Vector2: ...
    @overload
    def __truediv__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> U32Vector2: ...
    @overload
    def __rtruediv__(self, other: U32Vector2) -> U32Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> U32Vector2: ...
    def __abs__(self) -> U32Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> U32Vector2: ...
    @property
    def xy(self) -> U32Vector2: ...
    @property
    def xo(self) -> U32Vector2: ...
    @property
    def xl(self) -> U32Vector2: ...
    @property
    def yx(self) -> U32Vector2: ...
    @property
    def yy(self) -> U32Vector2: ...
    @property
    def yo(self) -> U32Vector2: ...
    @property
    def yl(self) -> U32Vector2: ...
    @property
    def ox(self) -> U32Vector2: ...
    @property
    def oy(self) -> U32Vector2: ...
    @property
    def lx(self) -> U32Vector2: ...
    @property
    def ly(self) -> U32Vector2: ...
    @property
    def xxx(self) -> U32Vector3: ...
    @property
    def xxy(self) -> U32Vector3: ...
    @property
    def xxo(self) -> U32Vector3: ...
    @property
    def xxl(self) -> U32Vector3: ...
    @property
    def xyx(self) -> U32Vector3: ...
    @property
    def xyy(self) -> U32Vector3: ...
    @property
    def xyo(self) -> U32Vector3: ...
    @property
    def xyl(self) -> U32Vector3: ...
    @property
    def xox(self) -> U32Vector3: ...
    @property
    def xoy(self) -> U32Vector3: ...
    @property
    def xoo(self) -> U32Vector3: ...
    @property
    def xol(self) -> U32Vector3: ...
    @property
    def xlx(self) -> U32Vector3: ...
    @property
    def xly(self) -> U32Vector3: ...
    @property
    def xlo(self) -> U32Vector3: ...
    @property
    def xll(self) -> U32Vector3: ...
    @property
    def yxx(self) -> U32Vector3: ...
    @property
    def yxy(self) -> U32Vector3: ...
    @property
    def yxo(self) -> U32Vector3: ...
    @property
    def yxl(self) -> U32Vector3: ...
    @property
    def yyx(self) -> U32Vector3: ...
    @property
    def yyy(self) -> U32Vector3: ...
    @property
    def yyo(self) -> U32Vector3: ...
    @property
    def yyl(self) -> U32Vector3: ...
    @property
    def yox(self) -> U32Vector3: ...
    @property
    def yoy(self) -> U32Vector3: ...
    @property
    def yoo(self) -> U32Vector3: ...
    @property
    def yol(self) -> U32Vector3: ...
    @property
    def ylx(self) -> U32Vector3: ...
    @property
    def yly(self) -> U32Vector3: ...
    @property
    def ylo(self) -> U32Vector3: ...
    @property
    def yll(self) -> U32Vector3: ...
    @property
    def oxx(self) -> U32Vector3: ...
    @property
    def oxy(self) -> U32Vector3: ...
    @property
    def oxo(self) -> U32Vector3: ...
    @property
    def oxl(self) -> U32Vector3: ...
    @property
    def oyx(self) -> U32Vector3: ...
    @property
    def oyy(self) -> U32Vector3: ...
    @property
    def oyo(self) -> U32Vector3: ...
    @property
    def oyl(self) -> U32Vector3: ...
    @property
    def oox(self) -> U32Vector3: ...
    @property
    def ooy(self) -> U32Vector3: ...
    @property
    def olx(self) -> U32Vector3: ...
    @property
    def oly(self) -> U32Vector3: ...
    @property
    def lxx(self) -> U32Vector3: ...
    @property
    def lxy(self) -> U32Vector3: ...
    @property
    def lxo(self) -> U32Vector3: ...
    @property
    def lxl(self) -> U32Vector3: ...
    @property
    def lyx(self) -> U32Vector3: ...
    @property
    def lyy(self) -> U32Vector3: ...
    @property
    def lyo(self) -> U32Vector3: ...
    @property
    def lyl(self) -> U32Vector3: ...
    @property
    def lox(self) -> U32Vector3: ...
    @property
    def loy(self) -> U32Vector3: ...
    @property
    def llx(self) -> U32Vector3: ...
    @property
    def lly(self) -> U32Vector3: ...
    @property
    def xxxx(self) -> U32Vector4: ...
    @property
    def xxxy(self) -> U32Vector4: ...
    @property
    def xxxo(self) -> U32Vector4: ...
    @property
    def xxxl(self) -> U32Vector4: ...
    @property
    def xxyx(self) -> U32Vector4: ...
    @property
    def xxyy(self) -> U32Vector4: ...
    @property
    def xxyo(self) -> U32Vector4: ...
    @property
    def xxyl(self) -> U32Vector4: ...
    @property
    def xxox(self) -> U32Vector4: ...
    @property
    def xxoy(self) -> U32Vector4: ...
    @property
    def xxoo(self) -> U32Vector4: ...
    @property
    def xxol(self) -> U32Vector4: ...
    @property
    def xxlx(self) -> U32Vector4: ...
    @property
    def xxly(self) -> U32Vector4: ...
    @property
    def xxlo(self) -> U32Vector4: ...
    @property
    def xxll(self) -> U32Vector4: ...
    @property
    def xyxx(self) -> U32Vector4: ...
    @property
    def xyxy(self) -> U32Vector4: ...
    @property
    def xyxo(self) -> U32Vector4: ...
    @property
    def xyxl(self) -> U32Vector4: ...
    @property
    def xyyx(self) -> U32Vector4: ...
    @property
    def xyyy(self) -> U32Vector4: ...
    @property
    def xyyo(self) -> U32Vector4: ...
    @property
    def xyyl(self) -> U32Vector4: ...
    @property
    def xyox(self) -> U32Vector4: ...
    @property
    def xyoy(self) -> U32Vector4: ...
    @property
    def xyoo(self) -> U32Vector4: ...
    @property
    def xyol(self) -> U32Vector4: ...
    @property
    def xylx(self) -> U32Vector4: ...
    @property
    def xyly(self) -> U32Vector4: ...
    @property
    def xylo(self) -> U32Vector4: ...
    @property
    def xyll(self) -> U32Vector4: ...
    @property
    def xoxx(self) -> U32Vector4: ...
    @property
    def xoxy(self) -> U32Vector4: ...
    @property
    def xoxo(self) -> U32Vector4: ...
    @property
    def xoxl(self) -> U32Vector4: ...
    @property
    def xoyx(self) -> U32Vector4: ...
    @property
    def xoyy(self) -> U32Vector4: ...
    @property
    def xoyo(self) -> U32Vector4: ...
    @property
    def xoyl(self) -> U32Vector4: ...
    @property
    def xoox(self) -> U32Vector4: ...
    @property
    def xooy(self) -> U32Vector4: ...
    @property
    def xooo(self) -> U32Vector4: ...
    @property
    def xool(self) -> U32Vector4: ...
    @property
    def xolx(self) -> U32Vector4: ...
    @property
    def xoly(self) -> U32Vector4: ...
    @property
    def xolo(self) -> U32Vector4: ...
    @property
    def xoll(self) -> U32Vector4: ...
    @property
    def xlxx(self) -> U32Vector4: ...
    @property
    def xlxy(self) -> U32Vector4: ...
    @property
    def xlxo(self) -> U32Vector4: ...
    @property
    def xlxl(self) -> U32Vector4: ...
    @property
    def xlyx(self) -> U32Vector4: ...
    @property
    def xlyy(self) -> U32Vector4: ...
    @property
    def xlyo(self) -> U32Vector4: ...
    @property
    def xlyl(self) -> U32Vector4: ...
    @property
    def xlox(self) -> U32Vector4: ...
    @property
    def xloy(self) -> U32Vector4: ...
    @property
    def xloo(self) -> U32Vector4: ...
    @property
    def xlol(self) -> U32Vector4: ...
    @property
    def xllx(self) -> U32Vector4: ...
    @property
    def xlly(self) -> U32Vector4: ...
    @property
    def xllo(self) -> U32Vector4: ...
    @property
    def xlll(self) -> U32Vector4: ...
    @property
    def yxxx(self) -> U32Vector4: ...
    @property
    def yxxy(self) -> U32Vector4: ...
    @property
    def yxxo(self) -> U32Vector4: ...
    @property
    def yxxl(self) -> U32Vector4: ...
    @property
    def yxyx(self) -> U32Vector4: ...
    @property
    def yxyy(self) -> U32Vector4: ...
    @property
    def yxyo(self) -> U32Vector4: ...
    @property
    def yxyl(self) -> U32Vector4: ...
    @property
    def yxox(self) -> U32Vector4: ...
    @property
    def yxoy(self) -> U32Vector4: ...
    @property
    def yxoo(self) -> U32Vector4: ...
    @property
    def yxol(self) -> U32Vector4: ...
    @property
    def yxlx(self) -> U32Vector4: ...
    @property
    def yxly(self) -> U32Vector4: ...
    @property
    def yxlo(self) -> U32Vector4: ...
    @property
    def yxll(self) -> U32Vector4: ...
    @property
    def yyxx(self) -> U32Vector4: ...
    @property
    def yyxy(self) -> U32Vector4: ...
    @property
    def yyxo(self) -> U32Vector4: ...
    @property
    def yyxl(self) -> U32Vector4: ...
    @property
    def yyyx(self) -> U32Vector4: ...
    @property
    def yyyy(self) -> U32Vector4: ...
    @property
    def yyyo(self) -> U32Vector4: ...
    @property
    def yyyl(self) -> U32Vector4: ...
    @property
    def yyox(self) -> U32Vector4: ...
    @property
    def yyoy(self) -> U32Vector4: ...
    @property
    def yyoo(self) -> U32Vector4: ...
    @property
    def yyol(self) -> U32Vector4: ...
    @property
    def yylx(self) -> U32Vector4: ...
    @property
    def yyly(self) -> U32Vector4: ...
    @property
    def yylo(self) -> U32Vector4: ...
    @property
    def yyll(self) -> U32Vector4: ...
    @property
    def yoxx(self) -> U32Vector4: ...
    @property
    def yoxy(self) -> U32Vector4: ...
    @property
    def yoxo(self) -> U32Vector4: ...
    @property
    def yoxl(self) -> U32Vector4: ...
    @property
    def yoyx(self) -> U32Vector4: ...
    @property
    def yoyy(self) -> U32Vector4: ...
    @property
    def yoyo(self) -> U32Vector4: ...
    @property
    def yoyl(self) -> U32Vector4: ...
    @property
    def yoox(self) -> U32Vector4: ...
    @property
    def yooy(self) -> U32Vector4: ...
    @property
    def yooo(self) -> U32Vector4: ...
    @property
    def yool(self) -> U32Vector4: ...
    @property
    def yolx(self) -> U32Vector4: ...
    @property
    def yoly(self) -> U32Vector4: ...
    @property
    def yolo(self) -> U32Vector4: ...
    @property
    def yoll(self) -> U32Vector4: ...
    @property
    def ylxx(self) -> U32Vector4: ...
    @property
    def ylxy(self) -> U32Vector4: ...
    @property
    def ylxo(self) -> U32Vector4: ...
    @property
    def ylxl(self) -> U32Vector4: ...
    @property
    def ylyx(self) -> U32Vector4: ...
    @property
    def ylyy(self) -> U32Vector4: ...
    @property
    def ylyo(self) -> U32Vector4: ...
    @property
    def ylyl(self) -> U32Vector4: ...
    @property
    def ylox(self) -> U32Vector4: ...
    @property
    def yloy(self) -> U32Vector4: ...
    @property
    def yloo(self) -> U32Vector4: ...
    @property
    def ylol(self) -> U32Vector4: ...
    @property
    def yllx(self) -> U32Vector4: ...
    @property
    def ylly(self) -> U32Vector4: ...
    @property
    def yllo(self) -> U32Vector4: ...
    @property
    def ylll(self) -> U32Vector4: ...
    @property
    def oxxx(self) -> U32Vector4: ...
    @property
    def oxxy(self) -> U32Vector4: ...
    @property
    def oxxo(self) -> U32Vector4: ...
    @property
    def oxxl(self) -> U32Vector4: ...
    @property
    def oxyx(self) -> U32Vector4: ...
    @property
    def oxyy(self) -> U32Vector4: ...
    @property
    def oxyo(self) -> U32Vector4: ...
    @property
    def oxyl(self) -> U32Vector4: ...
    @property
    def oxox(self) -> U32Vector4: ...
    @property
    def oxoy(self) -> U32Vector4: ...
    @property
    def oxoo(self) -> U32Vector4: ...
    @property
    def oxol(self) -> U32Vector4: ...
    @property
    def oxlx(self) -> U32Vector4: ...
    @property
    def oxly(self) -> U32Vector4: ...
    @property
    def oxlo(self) -> U32Vector4: ...
    @property
    def oxll(self) -> U32Vector4: ...
    @property
    def oyxx(self) -> U32Vector4: ...
    @property
    def oyxy(self) -> U32Vector4: ...
    @property
    def oyxo(self) -> U32Vector4: ...
    @property
    def oyxl(self) -> U32Vector4: ...
    @property
    def oyyx(self) -> U32Vector4: ...
    @property
    def oyyy(self) -> U32Vector4: ...
    @property
    def oyyo(self) -> U32Vector4: ...
    @property
    def oyyl(self) -> U32Vector4: ...
    @property
    def oyox(self) -> U32Vector4: ...
    @property
    def oyoy(self) -> U32Vector4: ...
    @property
    def oyoo(self) -> U32Vector4: ...
    @property
    def oyol(self) -> U32Vector4: ...
    @property
    def oylx(self) -> U32Vector4: ...
    @property
    def oyly(self) -> U32Vector4: ...
    @property
    def oylo(self) -> U32Vector4: ...
    @property
    def oyll(self) -> U32Vector4: ...
    @property
    def ooxx(self) -> U32Vector4: ...
    @property
    def ooxy(self) -> U32Vector4: ...
    @property
    def ooxo(self) -> U32Vector4: ...
    @property
    def ooxl(self) -> U32Vector4: ...
    @property
    def ooyx(self) -> U32Vector4: ...
    @property
    def ooyy(self) -> U32Vector4: ...
    @property
    def ooyo(self) -> U32Vector4: ...
    @property
    def ooyl(self) -> U32Vector4: ...
    @property
    def ooox(self) -> U32Vector4: ...
    @property
    def oooy(self) -> U32Vector4: ...
    @property
    def oolx(self) -> U32Vector4: ...
    @property
    def ooly(self) -> U32Vector4: ...
    @property
    def olxx(self) -> U32Vector4: ...
    @property
    def olxy(self) -> U32Vector4: ...
    @property
    def olxo(self) -> U32Vector4: ...
    @property
    def olxl(self) -> U32Vector4: ...
    @property
    def olyx(self) -> U32Vector4: ...
    @property
    def olyy(self) -> U32Vector4: ...
    @property
    def olyo(self) -> U32Vector4: ...
    @property
    def olyl(self) -> U32Vector4: ...
    @property
    def olox(self) -> U32Vector4: ...
    @property
    def oloy(self) -> U32Vector4: ...
    @property
    def ollx(self) -> U32Vector4: ...
    @property
    def olly(self) -> U32Vector4: ...
    @property
    def lxxx(self) -> U32Vector4: ...
    @property
    def lxxy(self) -> U32Vector4: ...
    @property
    def lxxo(self) -> U32Vector4: ...
    @property
    def lxxl(self) -> U32Vector4: ...
    @property
    def lxyx(self) -> U32Vector4: ...
    @property
    def lxyy(self) -> U32Vector4: ...
    @property
    def lxyo(self) -> U32Vector4: ...
    @property
    def lxyl(self) -> U32Vector4: ...
    @property
    def lxox(self) -> U32Vector4: ...
    @property
    def lxoy(self) -> U32Vector4: ...
    @property
    def lxoo(self) -> U32Vector4: ...
    @property
    def lxol(self) -> U32Vector4: ...
    @property
    def lxlx(self) -> U32Vector4: ...
    @property
    def lxly(self) -> U32Vector4: ...
    @property
    def lxlo(self) -> U32Vector4: ...
    @property
    def lxll(self) -> U32Vector4: ...
    @property
    def lyxx(self) -> U32Vector4: ...
    @property
    def lyxy(self) -> U32Vector4: ...
    @property
    def lyxo(self) -> U32Vector4: ...
    @property
    def lyxl(self) -> U32Vector4: ...
    @property
    def lyyx(self) -> U32Vector4: ...
    @property
    def lyyy(self) -> U32Vector4: ...
    @property
    def lyyo(self) -> U32Vector4: ...
    @property
    def lyyl(self) -> U32Vector4: ...
    @property
    def lyox(self) -> U32Vector4: ...
    @property
    def lyoy(self) -> U32Vector4: ...
    @property
    def lyoo(self) -> U32Vector4: ...
    @property
    def lyol(self) -> U32Vector4: ...
    @property
    def lylx(self) -> U32Vector4: ...
    @property
    def lyly(self) -> U32Vector4: ...
    @property
    def lylo(self) -> U32Vector4: ...
    @property
    def lyll(self) -> U32Vector4: ...
    @property
    def loxx(self) -> U32Vector4: ...
    @property
    def loxy(self) -> U32Vector4: ...
    @property
    def loxo(self) -> U32Vector4: ...
    @property
    def loxl(self) -> U32Vector4: ...
    @property
    def loyx(self) -> U32Vector4: ...
    @property
    def loyy(self) -> U32Vector4: ...
    @property
    def loyo(self) -> U32Vector4: ...
    @property
    def loyl(self) -> U32Vector4: ...
    @property
    def loox(self) -> U32Vector4: ...
    @property
    def looy(self) -> U32Vector4: ...
    @property
    def lolx(self) -> U32Vector4: ...
    @property
    def loly(self) -> U32Vector4: ...
    @property
    def llxx(self) -> U32Vector4: ...
    @property
    def llxy(self) -> U32Vector4: ...
    @property
    def llxo(self) -> U32Vector4: ...
    @property
    def llxl(self) -> U32Vector4: ...
    @property
    def llyx(self) -> U32Vector4: ...
    @property
    def llyy(self) -> U32Vector4: ...
    @property
    def llyo(self) -> U32Vector4: ...
    @property
    def llyl(self) -> U32Vector4: ...
    @property
    def llox(self) -> U32Vector4: ...
    @property
    def lloy(self) -> U32Vector4: ...
    @property
    def lllx(self) -> U32Vector4: ...
    @property
    def llly(self) -> U32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> U32Vector2: ...
    @property
    def rg(self) -> U32Vector2: ...
    @property
    def ro(self) -> U32Vector2: ...
    @property
    def rl(self) -> U32Vector2: ...
    @property
    def gr(self) -> U32Vector2: ...
    @property
    def gg(self) -> U32Vector2: ...
    @property
    def go(self) -> U32Vector2: ...
    @property
    def gl(self) -> U32Vector2: ...
    @property
    def og(self) -> U32Vector2: ...
    @property
    def lr(self) -> U32Vector2: ...
    @property
    def lg(self) -> U32Vector2: ...
    @property
    def rrr(self) -> U32Vector3: ...
    @property
    def rrg(self) -> U32Vector3: ...
    @property
    def rro(self) -> U32Vector3: ...
    @property
    def rrl(self) -> U32Vector3: ...
    @property
    def rgr(self) -> U32Vector3: ...
    @property
    def rgg(self) -> U32Vector3: ...
    @property
    def rgo(self) -> U32Vector3: ...
    @property
    def rgl(self) -> U32Vector3: ...
    @property
    def ror(self) -> U32Vector3: ...
    @property
    def rog(self) -> U32Vector3: ...
    @property
    def roo(self) -> U32Vector3: ...
    @property
    def rol(self) -> U32Vector3: ...
    @property
    def rlr(self) -> U32Vector3: ...
    @property
    def rlg(self) -> U32Vector3: ...
    @property
    def rlo(self) -> U32Vector3: ...
    @property
    def rll(self) -> U32Vector3: ...
    @property
    def grr(self) -> U32Vector3: ...
    @property
    def grg(self) -> U32Vector3: ...
    @property
    def gro(self) -> U32Vector3: ...
    @property
    def grl(self) -> U32Vector3: ...
    @property
    def ggr(self) -> U32Vector3: ...
    @property
    def ggg(self) -> U32Vector3: ...
    @property
    def ggo(self) -> U32Vector3: ...
    @property
    def ggl(self) -> U32Vector3: ...
    @property
    def gor(self) -> U32Vector3: ...
    @property
    def gog(self) -> U32Vector3: ...
    @property
    def goo(self) -> U32Vector3: ...
    @property
    def gol(self) -> U32Vector3: ...
    @property
    def glr(self) -> U32Vector3: ...
    @property
    def glg(self) -> U32Vector3: ...
    @property
    def glo(self) -> U32Vector3: ...
    @property
    def gll(self) -> U32Vector3: ...
    @property
    def orr(self) -> U32Vector3: ...
    @property
    def org(self) -> U32Vector3: ...
    @property
    def oro(self) -> U32Vector3: ...
    @property
    def orl(self) -> U32Vector3: ...
    @property
    def ogr(self) -> U32Vector3: ...
    @property
    def ogg(self) -> U32Vector3: ...
    @property
    def ogo(self) -> U32Vector3: ...
    @property
    def ogl(self) -> U32Vector3: ...
    @property
    def oor(self) -> U32Vector3: ...
    @property
    def oog(self) -> U32Vector3: ...
    @property
    def olr(self) -> U32Vector3: ...
    @property
    def olg(self) -> U32Vector3: ...
    @property
    def lrr(self) -> U32Vector3: ...
    @property
    def lrg(self) -> U32Vector3: ...
    @property
    def lro(self) -> U32Vector3: ...
    @property
    def lrl(self) -> U32Vector3: ...
    @property
    def lgr(self) -> U32Vector3: ...
    @property
    def lgg(self) -> U32Vector3: ...
    @property
    def lgo(self) -> U32Vector3: ...
    @property
    def lgl(self) -> U32Vector3: ...
    @property
    def lor(self) -> U32Vector3: ...
    @property
    def log(self) -> U32Vector3: ...
    @property
    def llr(self) -> U32Vector3: ...
    @property
    def llg(self) -> U32Vector3: ...
    @property
    def rrrr(self) -> U32Vector4: ...
    @property
    def rrrg(self) -> U32Vector4: ...
    @property
    def rrro(self) -> U32Vector4: ...
    @property
    def rrrl(self) -> U32Vector4: ...
    @property
    def rrgr(self) -> U32Vector4: ...
    @property
    def rrgg(self) -> U32Vector4: ...
    @property
    def rrgo(self) -> U32Vector4: ...
    @property
    def rrgl(self) -> U32Vector4: ...
    @property
    def rror(self) -> U32Vector4: ...
    @property
    def rrog(self) -> U32Vector4: ...
    @property
    def rroo(self) -> U32Vector4: ...
    @property
    def rrol(self) -> U32Vector4: ...
    @property
    def rrlr(self) -> U32Vector4: ...
    @property
    def rrlg(self) -> U32Vector4: ...
    @property
    def rrlo(self) -> U32Vector4: ...
    @property
    def rrll(self) -> U32Vector4: ...
    @property
    def rgrr(self) -> U32Vector4: ...
    @property
    def rgrg(self) -> U32Vector4: ...
    @property
    def rgro(self) -> U32Vector4: ...
    @property
    def rgrl(self) -> U32Vector4: ...
    @property
    def rggr(self) -> U32Vector4: ...
    @property
    def rggg(self) -> U32Vector4: ...
    @property
    def rggo(self) -> U32Vector4: ...
    @property
    def rggl(self) -> U32Vector4: ...
    @property
    def rgor(self) -> U32Vector4: ...
    @property
    def rgog(self) -> U32Vector4: ...
    @property
    def rgoo(self) -> U32Vector4: ...
    @property
    def rgol(self) -> U32Vector4: ...
    @property
    def rglr(self) -> U32Vector4: ...
    @property
    def rglg(self) -> U32Vector4: ...
    @property
    def rglo(self) -> U32Vector4: ...
    @property
    def rgll(self) -> U32Vector4: ...
    @property
    def rorr(self) -> U32Vector4: ...
    @property
    def rorg(self) -> U32Vector4: ...
    @property
    def roro(self) -> U32Vector4: ...
    @property
    def rorl(self) -> U32Vector4: ...
    @property
    def rogr(self) -> U32Vector4: ...
    @property
    def rogg(self) -> U32Vector4: ...
    @property
    def rogo(self) -> U32Vector4: ...
    @property
    def rogl(self) -> U32Vector4: ...
    @property
    def roor(self) -> U32Vector4: ...
    @property
    def roog(self) -> U32Vector4: ...
    @property
    def rooo(self) -> U32Vector4: ...
    @property
    def rool(self) -> U32Vector4: ...
    @property
    def rolr(self) -> U32Vector4: ...
    @property
    def rolg(self) -> U32Vector4: ...
    @property
    def rolo(self) -> U32Vector4: ...
    @property
    def roll(self) -> U32Vector4: ...
    @property
    def rlrr(self) -> U32Vector4: ...
    @property
    def rlrg(self) -> U32Vector4: ...
    @property
    def rlro(self) -> U32Vector4: ...
    @property
    def rlrl(self) -> U32Vector4: ...
    @property
    def rlgr(self) -> U32Vector4: ...
    @property
    def rlgg(self) -> U32Vector4: ...
    @property
    def rlgo(self) -> U32Vector4: ...
    @property
    def rlgl(self) -> U32Vector4: ...
    @property
    def rlor(self) -> U32Vector4: ...
    @property
    def rlog(self) -> U32Vector4: ...
    @property
    def rloo(self) -> U32Vector4: ...
    @property
    def rlol(self) -> U32Vector4: ...
    @property
    def rllr(self) -> U32Vector4: ...
    @property
    def rllg(self) -> U32Vector4: ...
    @property
    def rllo(self) -> U32Vector4: ...
    @property
    def rlll(self) -> U32Vector4: ...
    @property
    def grrr(self) -> U32Vector4: ...
    @property
    def grrg(self) -> U32Vector4: ...
    @property
    def grro(self) -> U32Vector4: ...
    @property
    def grrl(self) -> U32Vector4: ...
    @property
    def grgr(self) -> U32Vector4: ...
    @property
    def grgg(self) -> U32Vector4: ...
    @property
    def grgo(self) -> U32Vector4: ...
    @property
    def grgl(self) -> U32Vector4: ...
    @property
    def gror(self) -> U32Vector4: ...
    @property
    def grog(self) -> U32Vector4: ...
    @property
    def groo(self) -> U32Vector4: ...
    @property
    def grol(self) -> U32Vector4: ...
    @property
    def grlr(self) -> U32Vector4: ...
    @property
    def grlg(self) -> U32Vector4: ...
    @property
    def grlo(self) -> U32Vector4: ...
    @property
    def grll(self) -> U32Vector4: ...
    @property
    def ggrr(self) -> U32Vector4: ...
    @property
    def ggrg(self) -> U32Vector4: ...
    @property
    def ggro(self) -> U32Vector4: ...
    @property
    def ggrl(self) -> U32Vector4: ...
    @property
    def gggr(self) -> U32Vector4: ...
    @property
    def gggg(self) -> U32Vector4: ...
    @property
    def gggo(self) -> U32Vector4: ...
    @property
    def gggl(self) -> U32Vector4: ...
    @property
    def ggor(self) -> U32Vector4: ...
    @property
    def ggog(self) -> U32Vector4: ...
    @property
    def ggoo(self) -> U32Vector4: ...
    @property
    def ggol(self) -> U32Vector4: ...
    @property
    def gglr(self) -> U32Vector4: ...
    @property
    def gglg(self) -> U32Vector4: ...
    @property
    def gglo(self) -> U32Vector4: ...
    @property
    def ggll(self) -> U32Vector4: ...
    @property
    def gorr(self) -> U32Vector4: ...
    @property
    def gorg(self) -> U32Vector4: ...
    @property
    def goro(self) -> U32Vector4: ...
    @property
    def gorl(self) -> U32Vector4: ...
    @property
    def gogr(self) -> U32Vector4: ...
    @property
    def gogg(self) -> U32Vector4: ...
    @property
    def gogo(self) -> U32Vector4: ...
    @property
    def gogl(self) -> U32Vector4: ...
    @property
    def goor(self) -> U32Vector4: ...
    @property
    def goog(self) -> U32Vector4: ...
    @property
    def gooo(self) -> U32Vector4: ...
    @property
    def gool(self) -> U32Vector4: ...
    @property
    def golr(self) -> U32Vector4: ...
    @property
    def golg(self) -> U32Vector4: ...
    @property
    def golo(self) -> U32Vector4: ...
    @property
    def goll(self) -> U32Vector4: ...
    @property
    def glrr(self) -> U32Vector4: ...
    @property
    def glrg(self) -> U32Vector4: ...
    @property
    def glro(self) -> U32Vector4: ...
    @property
    def glrl(self) -> U32Vector4: ...
    @property
    def glgr(self) -> U32Vector4: ...
    @property
    def glgg(self) -> U32Vector4: ...
    @property
    def glgo(self) -> U32Vector4: ...
    @property
    def glgl(self) -> U32Vector4: ...
    @property
    def glor(self) -> U32Vector4: ...
    @property
    def glog(self) -> U32Vector4: ...
    @property
    def gloo(self) -> U32Vector4: ...
    @property
    def glol(self) -> U32Vector4: ...
    @property
    def gllr(self) -> U32Vector4: ...
    @property
    def gllg(self) -> U32Vector4: ...
    @property
    def gllo(self) -> U32Vector4: ...
    @property
    def glll(self) -> U32Vector4: ...
    @property
    def orrr(self) -> U32Vector4: ...
    @property
    def orrg(self) -> U32Vector4: ...
    @property
    def orro(self) -> U32Vector4: ...
    @property
    def orrl(self) -> U32Vector4: ...
    @property
    def orgr(self) -> U32Vector4: ...
    @property
    def orgg(self) -> U32Vector4: ...
    @property
    def orgo(self) -> U32Vector4: ...
    @property
    def orgl(self) -> U32Vector4: ...
    @property
    def oror(self) -> U32Vector4: ...
    @property
    def orog(self) -> U32Vector4: ...
    @property
    def oroo(self) -> U32Vector4: ...
    @property
    def orol(self) -> U32Vector4: ...
    @property
    def orlr(self) -> U32Vector4: ...
    @property
    def orlg(self) -> U32Vector4: ...
    @property
    def orlo(self) -> U32Vector4: ...
    @property
    def orll(self) -> U32Vector4: ...
    @property
    def ogrr(self) -> U32Vector4: ...
    @property
    def ogrg(self) -> U32Vector4: ...
    @property
    def ogro(self) -> U32Vector4: ...
    @property
    def ogrl(self) -> U32Vector4: ...
    @property
    def oggr(self) -> U32Vector4: ...
    @property
    def oggg(self) -> U32Vector4: ...
    @property
    def oggo(self) -> U32Vector4: ...
    @property
    def oggl(self) -> U32Vector4: ...
    @property
    def ogor(self) -> U32Vector4: ...
    @property
    def ogog(self) -> U32Vector4: ...
    @property
    def ogoo(self) -> U32Vector4: ...
    @property
    def ogol(self) -> U32Vector4: ...
    @property
    def oglr(self) -> U32Vector4: ...
    @property
    def oglg(self) -> U32Vector4: ...
    @property
    def oglo(self) -> U32Vector4: ...
    @property
    def ogll(self) -> U32Vector4: ...
    @property
    def oorr(self) -> U32Vector4: ...
    @property
    def oorg(self) -> U32Vector4: ...
    @property
    def ooro(self) -> U32Vector4: ...
    @property
    def oorl(self) -> U32Vector4: ...
    @property
    def oogr(self) -> U32Vector4: ...
    @property
    def oogg(self) -> U32Vector4: ...
    @property
    def oogo(self) -> U32Vector4: ...
    @property
    def oogl(self) -> U32Vector4: ...
    @property
    def ooor(self) -> U32Vector4: ...
    @property
    def ooog(self) -> U32Vector4: ...
    @property
    def oolr(self) -> U32Vector4: ...
    @property
    def oolg(self) -> U32Vector4: ...
    @property
    def olrr(self) -> U32Vector4: ...
    @property
    def olrg(self) -> U32Vector4: ...
    @property
    def olro(self) -> U32Vector4: ...
    @property
    def olrl(self) -> U32Vector4: ...
    @property
    def olgr(self) -> U32Vector4: ...
    @property
    def olgg(self) -> U32Vector4: ...
    @property
    def olgo(self) -> U32Vector4: ...
    @property
    def olgl(self) -> U32Vector4: ...
    @property
    def olor(self) -> U32Vector4: ...
    @property
    def olog(self) -> U32Vector4: ...
    @property
    def ollr(self) -> U32Vector4: ...
    @property
    def ollg(self) -> U32Vector4: ...
    @property
    def lrrr(self) -> U32Vector4: ...
    @property
    def lrrg(self) -> U32Vector4: ...
    @property
    def lrro(self) -> U32Vector4: ...
    @property
    def lrrl(self) -> U32Vector4: ...
    @property
    def lrgr(self) -> U32Vector4: ...
    @property
    def lrgg(self) -> U32Vector4: ...
    @property
    def lrgo(self) -> U32Vector4: ...
    @property
    def lrgl(self) -> U32Vector4: ...
    @property
    def lror(self) -> U32Vector4: ...
    @property
    def lrog(self) -> U32Vector4: ...
    @property
    def lroo(self) -> U32Vector4: ...
    @property
    def lrol(self) -> U32Vector4: ...
    @property
    def lrlr(self) -> U32Vector4: ...
    @property
    def lrlg(self) -> U32Vector4: ...
    @property
    def lrlo(self) -> U32Vector4: ...
    @property
    def lrll(self) -> U32Vector4: ...
    @property
    def lgrr(self) -> U32Vector4: ...
    @property
    def lgrg(self) -> U32Vector4: ...
    @property
    def lgro(self) -> U32Vector4: ...
    @property
    def lgrl(self) -> U32Vector4: ...
    @property
    def lggr(self) -> U32Vector4: ...
    @property
    def lggg(self) -> U32Vector4: ...
    @property
    def lggo(self) -> U32Vector4: ...
    @property
    def lggl(self) -> U32Vector4: ...
    @property
    def lgor(self) -> U32Vector4: ...
    @property
    def lgog(self) -> U32Vector4: ...
    @property
    def lgoo(self) -> U32Vector4: ...
    @property
    def lgol(self) -> U32Vector4: ...
    @property
    def lglr(self) -> U32Vector4: ...
    @property
    def lglg(self) -> U32Vector4: ...
    @property
    def lglo(self) -> U32Vector4: ...
    @property
    def lgll(self) -> U32Vector4: ...
    @property
    def lorr(self) -> U32Vector4: ...
    @property
    def lorg(self) -> U32Vector4: ...
    @property
    def loro(self) -> U32Vector4: ...
    @property
    def lorl(self) -> U32Vector4: ...
    @property
    def logr(self) -> U32Vector4: ...
    @property
    def logg(self) -> U32Vector4: ...
    @property
    def logo(self) -> U32Vector4: ...
    @property
    def logl(self) -> U32Vector4: ...
    @property
    def loor(self) -> U32Vector4: ...
    @property
    def loog(self) -> U32Vector4: ...
    @property
    def lolr(self) -> U32Vector4: ...
    @property
    def lolg(self) -> U32Vector4: ...
    @property
    def llrr(self) -> U32Vector4: ...
    @property
    def llrg(self) -> U32Vector4: ...
    @property
    def llro(self) -> U32Vector4: ...
    @property
    def llrl(self) -> U32Vector4: ...
    @property
    def llgr(self) -> U32Vector4: ...
    @property
    def llgg(self) -> U32Vector4: ...
    @property
    def llgo(self) -> U32Vector4: ...
    @property
    def llgl(self) -> U32Vector4: ...
    @property
    def llor(self) -> U32Vector4: ...
    @property
    def llog(self) -> U32Vector4: ...
    @property
    def lllr(self) -> U32Vector4: ...
    @property
    def lllg(self) -> U32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> U32Vector2: ...
    @property
    def st(self) -> U32Vector2: ...
    @property
    def so(self) -> U32Vector2: ...
    @property
    def sl(self) -> U32Vector2: ...
    @property
    def ts(self) -> U32Vector2: ...
    @property
    def tt(self) -> U32Vector2: ...
    @property
    def to(self) -> U32Vector2: ...
    @property
    def tl(self) -> U32Vector2: ...
    @property
    def os(self) -> U32Vector2: ...
    @property
    def ot(self) -> U32Vector2: ...
    @property
    def ls(self) -> U32Vector2: ...
    @property
    def lt(self) -> U32Vector2: ...
    @property
    def sss(self) -> U32Vector3: ...
    @property
    def sst(self) -> U32Vector3: ...
    @property
    def sso(self) -> U32Vector3: ...
    @property
    def ssl(self) -> U32Vector3: ...
    @property
    def sts(self) -> U32Vector3: ...
    @property
    def stt(self) -> U32Vector3: ...
    @property
    def sto(self) -> U32Vector3: ...
    @property
    def stl(self) -> U32Vector3: ...
    @property
    def sos(self) -> U32Vector3: ...
    @property
    def sot(self) -> U32Vector3: ...
    @property
    def soo(self) -> U32Vector3: ...
    @property
    def sol(self) -> U32Vector3: ...
    @property
    def sls(self) -> U32Vector3: ...
    @property
    def slt(self) -> U32Vector3: ...
    @property
    def slo(self) -> U32Vector3: ...
    @property
    def sll(self) -> U32Vector3: ...
    @property
    def tss(self) -> U32Vector3: ...
    @property
    def tst(self) -> U32Vector3: ...
    @property
    def tso(self) -> U32Vector3: ...
    @property
    def tsl(self) -> U32Vector3: ...
    @property
    def tts(self) -> U32Vector3: ...
    @property
    def ttt(self) -> U32Vector3: ...
    @property
    def tto(self) -> U32Vector3: ...
    @property
    def ttl(self) -> U32Vector3: ...
    @property
    def tos(self) -> U32Vector3: ...
    @property
    def tot(self) -> U32Vector3: ...
    @property
    def too(self) -> U32Vector3: ...
    @property
    def tol(self) -> U32Vector3: ...
    @property
    def tls(self) -> U32Vector3: ...
    @property
    def tlt(self) -> U32Vector3: ...
    @property
    def tlo(self) -> U32Vector3: ...
    @property
    def tll(self) -> U32Vector3: ...
    @property
    def oss(self) -> U32Vector3: ...
    @property
    def ost(self) -> U32Vector3: ...
    @property
    def oso(self) -> U32Vector3: ...
    @property
    def osl(self) -> U32Vector3: ...
    @property
    def ots(self) -> U32Vector3: ...
    @property
    def ott(self) -> U32Vector3: ...
    @property
    def oto(self) -> U32Vector3: ...
    @property
    def otl(self) -> U32Vector3: ...
    @property
    def oos(self) -> U32Vector3: ...
    @property
    def oot(self) -> U32Vector3: ...
    @property
    def ols(self) -> U32Vector3: ...
    @property
    def olt(self) -> U32Vector3: ...
    @property
    def lss(self) -> U32Vector3: ...
    @property
    def lst(self) -> U32Vector3: ...
    @property
    def lso(self) -> U32Vector3: ...
    @property
    def lsl(self) -> U32Vector3: ...
    @property
    def lts(self) -> U32Vector3: ...
    @property
    def ltt(self) -> U32Vector3: ...
    @property
    def lto(self) -> U32Vector3: ...
    @property
    def ltl(self) -> U32Vector3: ...
    @property
    def los(self) -> U32Vector3: ...
    @property
    def lot(self) -> U32Vector3: ...
    @property
    def lls(self) -> U32Vector3: ...
    @property
    def llt(self) -> U32Vector3: ...
    @property
    def ssss(self) -> U32Vector4: ...
    @property
    def ssst(self) -> U32Vector4: ...
    @property
    def ssso(self) -> U32Vector4: ...
    @property
    def sssl(self) -> U32Vector4: ...
    @property
    def ssts(self) -> U32Vector4: ...
    @property
    def sstt(self) -> U32Vector4: ...
    @property
    def ssto(self) -> U32Vector4: ...
    @property
    def sstl(self) -> U32Vector4: ...
    @property
    def ssos(self) -> U32Vector4: ...
    @property
    def ssot(self) -> U32Vector4: ...
    @property
    def ssoo(self) -> U32Vector4: ...
    @property
    def ssol(self) -> U32Vector4: ...
    @property
    def ssls(self) -> U32Vector4: ...
    @property
    def sslt(self) -> U32Vector4: ...
    @property
    def sslo(self) -> U32Vector4: ...
    @property
    def ssll(self) -> U32Vector4: ...
    @property
    def stss(self) -> U32Vector4: ...
    @property
    def stst(self) -> U32Vector4: ...
    @property
    def stso(self) -> U32Vector4: ...
    @property
    def stsl(self) -> U32Vector4: ...
    @property
    def stts(self) -> U32Vector4: ...
    @property
    def sttt(self) -> U32Vector4: ...
    @property
    def stto(self) -> U32Vector4: ...
    @property
    def sttl(self) -> U32Vector4: ...
    @property
    def stos(self) -> U32Vector4: ...
    @property
    def stot(self) -> U32Vector4: ...
    @property
    def stoo(self) -> U32Vector4: ...
    @property
    def stol(self) -> U32Vector4: ...
    @property
    def stls(self) -> U32Vector4: ...
    @property
    def stlt(self) -> U32Vector4: ...
    @property
    def stlo(self) -> U32Vector4: ...
    @property
    def stll(self) -> U32Vector4: ...
    @property
    def soss(self) -> U32Vector4: ...
    @property
    def sost(self) -> U32Vector4: ...
    @property
    def soso(self) -> U32Vector4: ...
    @property
    def sosl(self) -> U32Vector4: ...
    @property
    def sots(self) -> U32Vector4: ...
    @property
    def sott(self) -> U32Vector4: ...
    @property
    def soto(self) -> U32Vector4: ...
    @property
    def sotl(self) -> U32Vector4: ...
    @property
    def soos(self) -> U32Vector4: ...
    @property
    def soot(self) -> U32Vector4: ...
    @property
    def sooo(self) -> U32Vector4: ...
    @property
    def sool(self) -> U32Vector4: ...
    @property
    def sols(self) -> U32Vector4: ...
    @property
    def solt(self) -> U32Vector4: ...
    @property
    def solo(self) -> U32Vector4: ...
    @property
    def soll(self) -> U32Vector4: ...
    @property
    def slss(self) -> U32Vector4: ...
    @property
    def slst(self) -> U32Vector4: ...
    @property
    def slso(self) -> U32Vector4: ...
    @property
    def slsl(self) -> U32Vector4: ...
    @property
    def slts(self) -> U32Vector4: ...
    @property
    def sltt(self) -> U32Vector4: ...
    @property
    def slto(self) -> U32Vector4: ...
    @property
    def sltl(self) -> U32Vector4: ...
    @property
    def slos(self) -> U32Vector4: ...
    @property
    def slot(self) -> U32Vector4: ...
    @property
    def sloo(self) -> U32Vector4: ...
    @property
    def slol(self) -> U32Vector4: ...
    @property
    def slls(self) -> U32Vector4: ...
    @property
    def sllt(self) -> U32Vector4: ...
    @property
    def sllo(self) -> U32Vector4: ...
    @property
    def slll(self) -> U32Vector4: ...
    @property
    def tsss(self) -> U32Vector4: ...
    @property
    def tsst(self) -> U32Vector4: ...
    @property
    def tsso(self) -> U32Vector4: ...
    @property
    def tssl(self) -> U32Vector4: ...
    @property
    def tsts(self) -> U32Vector4: ...
    @property
    def tstt(self) -> U32Vector4: ...
    @property
    def tsto(self) -> U32Vector4: ...
    @property
    def tstl(self) -> U32Vector4: ...
    @property
    def tsos(self) -> U32Vector4: ...
    @property
    def tsot(self) -> U32Vector4: ...
    @property
    def tsoo(self) -> U32Vector4: ...
    @property
    def tsol(self) -> U32Vector4: ...
    @property
    def tsls(self) -> U32Vector4: ...
    @property
    def tslt(self) -> U32Vector4: ...
    @property
    def tslo(self) -> U32Vector4: ...
    @property
    def tsll(self) -> U32Vector4: ...
    @property
    def ttss(self) -> U32Vector4: ...
    @property
    def ttst(self) -> U32Vector4: ...
    @property
    def ttso(self) -> U32Vector4: ...
    @property
    def ttsl(self) -> U32Vector4: ...
    @property
    def ttts(self) -> U32Vector4: ...
    @property
    def tttt(self) -> U32Vector4: ...
    @property
    def ttto(self) -> U32Vector4: ...
    @property
    def tttl(self) -> U32Vector4: ...
    @property
    def ttos(self) -> U32Vector4: ...
    @property
    def ttot(self) -> U32Vector4: ...
    @property
    def ttoo(self) -> U32Vector4: ...
    @property
    def ttol(self) -> U32Vector4: ...
    @property
    def ttls(self) -> U32Vector4: ...
    @property
    def ttlt(self) -> U32Vector4: ...
    @property
    def ttlo(self) -> U32Vector4: ...
    @property
    def ttll(self) -> U32Vector4: ...
    @property
    def toss(self) -> U32Vector4: ...
    @property
    def tost(self) -> U32Vector4: ...
    @property
    def toso(self) -> U32Vector4: ...
    @property
    def tosl(self) -> U32Vector4: ...
    @property
    def tots(self) -> U32Vector4: ...
    @property
    def tott(self) -> U32Vector4: ...
    @property
    def toto(self) -> U32Vector4: ...
    @property
    def totl(self) -> U32Vector4: ...
    @property
    def toos(self) -> U32Vector4: ...
    @property
    def toot(self) -> U32Vector4: ...
    @property
    def tooo(self) -> U32Vector4: ...
    @property
    def tool(self) -> U32Vector4: ...
    @property
    def tols(self) -> U32Vector4: ...
    @property
    def tolt(self) -> U32Vector4: ...
    @property
    def tolo(self) -> U32Vector4: ...
    @property
    def toll(self) -> U32Vector4: ...
    @property
    def tlss(self) -> U32Vector4: ...
    @property
    def tlst(self) -> U32Vector4: ...
    @property
    def tlso(self) -> U32Vector4: ...
    @property
    def tlsl(self) -> U32Vector4: ...
    @property
    def tlts(self) -> U32Vector4: ...
    @property
    def tltt(self) -> U32Vector4: ...
    @property
    def tlto(self) -> U32Vector4: ...
    @property
    def tltl(self) -> U32Vector4: ...
    @property
    def tlos(self) -> U32Vector4: ...
    @property
    def tlot(self) -> U32Vector4: ...
    @property
    def tloo(self) -> U32Vector4: ...
    @property
    def tlol(self) -> U32Vector4: ...
    @property
    def tlls(self) -> U32Vector4: ...
    @property
    def tllt(self) -> U32Vector4: ...
    @property
    def tllo(self) -> U32Vector4: ...
    @property
    def tlll(self) -> U32Vector4: ...
    @property
    def osss(self) -> U32Vector4: ...
    @property
    def osst(self) -> U32Vector4: ...
    @property
    def osso(self) -> U32Vector4: ...
    @property
    def ossl(self) -> U32Vector4: ...
    @property
    def osts(self) -> U32Vector4: ...
    @property
    def ostt(self) -> U32Vector4: ...
    @property
    def osto(self) -> U32Vector4: ...
    @property
    def ostl(self) -> U32Vector4: ...
    @property
    def osos(self) -> U32Vector4: ...
    @property
    def osot(self) -> U32Vector4: ...
    @property
    def osoo(self) -> U32Vector4: ...
    @property
    def osol(self) -> U32Vector4: ...
    @property
    def osls(self) -> U32Vector4: ...
    @property
    def oslt(self) -> U32Vector4: ...
    @property
    def oslo(self) -> U32Vector4: ...
    @property
    def osll(self) -> U32Vector4: ...
    @property
    def otss(self) -> U32Vector4: ...
    @property
    def otst(self) -> U32Vector4: ...
    @property
    def otso(self) -> U32Vector4: ...
    @property
    def otsl(self) -> U32Vector4: ...
    @property
    def otts(self) -> U32Vector4: ...
    @property
    def ottt(self) -> U32Vector4: ...
    @property
    def otto(self) -> U32Vector4: ...
    @property
    def ottl(self) -> U32Vector4: ...
    @property
    def otos(self) -> U32Vector4: ...
    @property
    def otot(self) -> U32Vector4: ...
    @property
    def otoo(self) -> U32Vector4: ...
    @property
    def otol(self) -> U32Vector4: ...
    @property
    def otls(self) -> U32Vector4: ...
    @property
    def otlt(self) -> U32Vector4: ...
    @property
    def otlo(self) -> U32Vector4: ...
    @property
    def otll(self) -> U32Vector4: ...
    @property
    def ooss(self) -> U32Vector4: ...
    @property
    def oost(self) -> U32Vector4: ...
    @property
    def ooso(self) -> U32Vector4: ...
    @property
    def oosl(self) -> U32Vector4: ...
    @property
    def oots(self) -> U32Vector4: ...
    @property
    def oott(self) -> U32Vector4: ...
    @property
    def ooto(self) -> U32Vector4: ...
    @property
    def ootl(self) -> U32Vector4: ...
    @property
    def ooos(self) -> U32Vector4: ...
    @property
    def ooot(self) -> U32Vector4: ...
    @property
    def ools(self) -> U32Vector4: ...
    @property
    def oolt(self) -> U32Vector4: ...
    @property
    def olss(self) -> U32Vector4: ...
    @property
    def olst(self) -> U32Vector4: ...
    @property
    def olso(self) -> U32Vector4: ...
    @property
    def olsl(self) -> U32Vector4: ...
    @property
    def olts(self) -> U32Vector4: ...
    @property
    def oltt(self) -> U32Vector4: ...
    @property
    def olto(self) -> U32Vector4: ...
    @property
    def oltl(self) -> U32Vector4: ...
    @property
    def olos(self) -> U32Vector4: ...
    @property
    def olot(self) -> U32Vector4: ...
    @property
    def olls(self) -> U32Vector4: ...
    @property
    def ollt(self) -> U32Vector4: ...
    @property
    def lsss(self) -> U32Vector4: ...
    @property
    def lsst(self) -> U32Vector4: ...
    @property
    def lsso(self) -> U32Vector4: ...
    @property
    def lssl(self) -> U32Vector4: ...
    @property
    def lsts(self) -> U32Vector4: ...
    @property
    def lstt(self) -> U32Vector4: ...
    @property
    def lsto(self) -> U32Vector4: ...
    @property
    def lstl(self) -> U32Vector4: ...
    @property
    def lsos(self) -> U32Vector4: ...
    @property
    def lsot(self) -> U32Vector4: ...
    @property
    def lsoo(self) -> U32Vector4: ...
    @property
    def lsol(self) -> U32Vector4: ...
    @property
    def lsls(self) -> U32Vector4: ...
    @property
    def lslt(self) -> U32Vector4: ...
    @property
    def lslo(self) -> U32Vector4: ...
    @property
    def lsll(self) -> U32Vector4: ...
    @property
    def ltss(self) -> U32Vector4: ...
    @property
    def ltst(self) -> U32Vector4: ...
    @property
    def ltso(self) -> U32Vector4: ...
    @property
    def ltsl(self) -> U32Vector4: ...
    @property
    def ltts(self) -> U32Vector4: ...
    @property
    def lttt(self) -> U32Vector4: ...
    @property
    def ltto(self) -> U32Vector4: ...
    @property
    def lttl(self) -> U32Vector4: ...
    @property
    def ltos(self) -> U32Vector4: ...
    @property
    def ltot(self) -> U32Vector4: ...
    @property
    def ltoo(self) -> U32Vector4: ...
    @property
    def ltol(self) -> U32Vector4: ...
    @property
    def ltls(self) -> U32Vector4: ...
    @property
    def ltlt(self) -> U32Vector4: ...
    @property
    def ltlo(self) -> U32Vector4: ...
    @property
    def ltll(self) -> U32Vector4: ...
    @property
    def loss(self) -> U32Vector4: ...
    @property
    def lost(self) -> U32Vector4: ...
    @property
    def loso(self) -> U32Vector4: ...
    @property
    def losl(self) -> U32Vector4: ...
    @property
    def lots(self) -> U32Vector4: ...
    @property
    def lott(self) -> U32Vector4: ...
    @property
    def loto(self) -> U32Vector4: ...
    @property
    def lotl(self) -> U32Vector4: ...
    @property
    def loos(self) -> U32Vector4: ...
    @property
    def loot(self) -> U32Vector4: ...
    @property
    def lols(self) -> U32Vector4: ...
    @property
    def lolt(self) -> U32Vector4: ...
    @property
    def llss(self) -> U32Vector4: ...
    @property
    def llst(self) -> U32Vector4: ...
    @property
    def llso(self) -> U32Vector4: ...
    @property
    def llsl(self) -> U32Vector4: ...
    @property
    def llts(self) -> U32Vector4: ...
    @property
    def lltt(self) -> U32Vector4: ...
    @property
    def llto(self) -> U32Vector4: ...
    @property
    def lltl(self) -> U32Vector4: ...
    @property
    def llos(self) -> U32Vector4: ...
    @property
    def llot(self) -> U32Vector4: ...
    @property
    def llls(self) -> U32Vector4: ...
    @property
    def lllt(self) -> U32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U32Vector2: ...
    @property
    def uv(self) -> U32Vector2: ...
    @property
    def uo(self) -> U32Vector2: ...
    @property
    def ul(self) -> U32Vector2: ...
    @property
    def vu(self) -> U32Vector2: ...
    @property
    def vv(self) -> U32Vector2: ...
    @property
    def vo(self) -> U32Vector2: ...
    @property
    def vl(self) -> U32Vector2: ...
    @property
    def ou(self) -> U32Vector2: ...
    @property
    def ov(self) -> U32Vector2: ...
    @property
    def lu(self) -> U32Vector2: ...
    @property
    def lv(self) -> U32Vector2: ...
    @property
    def uuu(self) -> U32Vector3: ...
    @property
    def uuv(self) -> U32Vector3: ...
    @property
    def uuo(self) -> U32Vector3: ...
    @property
    def uul(self) -> U32Vector3: ...
    @property
    def uvu(self) -> U32Vector3: ...
    @property
    def uvv(self) -> U32Vector3: ...
    @property
    def uvo(self) -> U32Vector3: ...
    @property
    def uvl(self) -> U32Vector3: ...
    @property
    def uou(self) -> U32Vector3: ...
    @property
    def uov(self) -> U32Vector3: ...
    @property
    def uoo(self) -> U32Vector3: ...
    @property
    def uol(self) -> U32Vector3: ...
    @property
    def ulu(self) -> U32Vector3: ...
    @property
    def ulv(self) -> U32Vector3: ...
    @property
    def ulo(self) -> U32Vector3: ...
    @property
    def ull(self) -> U32Vector3: ...
    @property
    def vuu(self) -> U32Vector3: ...
    @property
    def vuv(self) -> U32Vector3: ...
    @property
    def vuo(self) -> U32Vector3: ...
    @property
    def vul(self) -> U32Vector3: ...
    @property
    def vvu(self) -> U32Vector3: ...
    @property
    def vvv(self) -> U32Vector3: ...
    @property
    def vvo(self) -> U32Vector3: ...
    @property
    def vvl(self) -> U32Vector3: ...
    @property
    def vou(self) -> U32Vector3: ...
    @property
    def vov(self) -> U32Vector3: ...
    @property
    def voo(self) -> U32Vector3: ...
    @property
    def vol(self) -> U32Vector3: ...
    @property
    def vlu(self) -> U32Vector3: ...
    @property
    def vlv(self) -> U32Vector3: ...
    @property
    def vlo(self) -> U32Vector3: ...
    @property
    def vll(self) -> U32Vector3: ...
    @property
    def ouu(self) -> U32Vector3: ...
    @property
    def ouv(self) -> U32Vector3: ...
    @property
    def ouo(self) -> U32Vector3: ...
    @property
    def oul(self) -> U32Vector3: ...
    @property
    def ovu(self) -> U32Vector3: ...
    @property
    def ovv(self) -> U32Vector3: ...
    @property
    def ovo(self) -> U32Vector3: ...
    @property
    def ovl(self) -> U32Vector3: ...
    @property
    def oou(self) -> U32Vector3: ...
    @property
    def oov(self) -> U32Vector3: ...
    @property
    def olu(self) -> U32Vector3: ...
    @property
    def olv(self) -> U32Vector3: ...
    @property
    def luu(self) -> U32Vector3: ...
    @property
    def luv(self) -> U32Vector3: ...
    @property
    def luo(self) -> U32Vector3: ...
    @property
    def lul(self) -> U32Vector3: ...
    @property
    def lvu(self) -> U32Vector3: ...
    @property
    def lvv(self) -> U32Vector3: ...
    @property
    def lvo(self) -> U32Vector3: ...
    @property
    def lvl(self) -> U32Vector3: ...
    @property
    def lou(self) -> U32Vector3: ...
    @property
    def lov(self) -> U32Vector3: ...
    @property
    def llu(self) -> U32Vector3: ...
    @property
    def llv(self) -> U32Vector3: ...
    @property
    def uuuu(self) -> U32Vector4: ...
    @property
    def uuuv(self) -> U32Vector4: ...
    @property
    def uuuo(self) -> U32Vector4: ...
    @property
    def uuul(self) -> U32Vector4: ...
    @property
    def uuvu(self) -> U32Vector4: ...
    @property
    def uuvv(self) -> U32Vector4: ...
    @property
    def uuvo(self) -> U32Vector4: ...
    @property
    def uuvl(self) -> U32Vector4: ...
    @property
    def uuou(self) -> U32Vector4: ...
    @property
    def uuov(self) -> U32Vector4: ...
    @property
    def uuoo(self) -> U32Vector4: ...
    @property
    def uuol(self) -> U32Vector4: ...
    @property
    def uulu(self) -> U32Vector4: ...
    @property
    def uulv(self) -> U32Vector4: ...
    @property
    def uulo(self) -> U32Vector4: ...
    @property
    def uull(self) -> U32Vector4: ...
    @property
    def uvuu(self) -> U32Vector4: ...
    @property
    def uvuv(self) -> U32Vector4: ...
    @property
    def uvuo(self) -> U32Vector4: ...
    @property
    def uvul(self) -> U32Vector4: ...
    @property
    def uvvu(self) -> U32Vector4: ...
    @property
    def uvvv(self) -> U32Vector4: ...
    @property
    def uvvo(self) -> U32Vector4: ...
    @property
    def uvvl(self) -> U32Vector4: ...
    @property
    def uvou(self) -> U32Vector4: ...
    @property
    def uvov(self) -> U32Vector4: ...
    @property
    def uvoo(self) -> U32Vector4: ...
    @property
    def uvol(self) -> U32Vector4: ...
    @property
    def uvlu(self) -> U32Vector4: ...
    @property
    def uvlv(self) -> U32Vector4: ...
    @property
    def uvlo(self) -> U32Vector4: ...
    @property
    def uvll(self) -> U32Vector4: ...
    @property
    def uouu(self) -> U32Vector4: ...
    @property
    def uouv(self) -> U32Vector4: ...
    @property
    def uouo(self) -> U32Vector4: ...
    @property
    def uoul(self) -> U32Vector4: ...
    @property
    def uovu(self) -> U32Vector4: ...
    @property
    def uovv(self) -> U32Vector4: ...
    @property
    def uovo(self) -> U32Vector4: ...
    @property
    def uovl(self) -> U32Vector4: ...
    @property
    def uoou(self) -> U32Vector4: ...
    @property
    def uoov(self) -> U32Vector4: ...
    @property
    def uooo(self) -> U32Vector4: ...
    @property
    def uool(self) -> U32Vector4: ...
    @property
    def uolu(self) -> U32Vector4: ...
    @property
    def uolv(self) -> U32Vector4: ...
    @property
    def uolo(self) -> U32Vector4: ...
    @property
    def uoll(self) -> U32Vector4: ...
    @property
    def uluu(self) -> U32Vector4: ...
    @property
    def uluv(self) -> U32Vector4: ...
    @property
    def uluo(self) -> U32Vector4: ...
    @property
    def ulul(self) -> U32Vector4: ...
    @property
    def ulvu(self) -> U32Vector4: ...
    @property
    def ulvv(self) -> U32Vector4: ...
    @property
    def ulvo(self) -> U32Vector4: ...
    @property
    def ulvl(self) -> U32Vector4: ...
    @property
    def ulou(self) -> U32Vector4: ...
    @property
    def ulov(self) -> U32Vector4: ...
    @property
    def uloo(self) -> U32Vector4: ...
    @property
    def ulol(self) -> U32Vector4: ...
    @property
    def ullu(self) -> U32Vector4: ...
    @property
    def ullv(self) -> U32Vector4: ...
    @property
    def ullo(self) -> U32Vector4: ...
    @property
    def ulll(self) -> U32Vector4: ...
    @property
    def vuuu(self) -> U32Vector4: ...
    @property
    def vuuv(self) -> U32Vector4: ...
    @property
    def vuuo(self) -> U32Vector4: ...
    @property
    def vuul(self) -> U32Vector4: ...
    @property
    def vuvu(self) -> U32Vector4: ...
    @property
    def vuvv(self) -> U32Vector4: ...
    @property
    def vuvo(self) -> U32Vector4: ...
    @property
    def vuvl(self) -> U32Vector4: ...
    @property
    def vuou(self) -> U32Vector4: ...
    @property
    def vuov(self) -> U32Vector4: ...
    @property
    def vuoo(self) -> U32Vector4: ...
    @property
    def vuol(self) -> U32Vector4: ...
    @property
    def vulu(self) -> U32Vector4: ...
    @property
    def vulv(self) -> U32Vector4: ...
    @property
    def vulo(self) -> U32Vector4: ...
    @property
    def vull(self) -> U32Vector4: ...
    @property
    def vvuu(self) -> U32Vector4: ...
    @property
    def vvuv(self) -> U32Vector4: ...
    @property
    def vvuo(self) -> U32Vector4: ...
    @property
    def vvul(self) -> U32Vector4: ...
    @property
    def vvvu(self) -> U32Vector4: ...
    @property
    def vvvv(self) -> U32Vector4: ...
    @property
    def vvvo(self) -> U32Vector4: ...
    @property
    def vvvl(self) -> U32Vector4: ...
    @property
    def vvou(self) -> U32Vector4: ...
    @property
    def vvov(self) -> U32Vector4: ...
    @property
    def vvoo(self) -> U32Vector4: ...
    @property
    def vvol(self) -> U32Vector4: ...
    @property
    def vvlu(self) -> U32Vector4: ...
    @property
    def vvlv(self) -> U32Vector4: ...
    @property
    def vvlo(self) -> U32Vector4: ...
    @property
    def vvll(self) -> U32Vector4: ...
    @property
    def vouu(self) -> U32Vector4: ...
    @property
    def vouv(self) -> U32Vector4: ...
    @property
    def vouo(self) -> U32Vector4: ...
    @property
    def voul(self) -> U32Vector4: ...
    @property
    def vovu(self) -> U32Vector4: ...
    @property
    def vovv(self) -> U32Vector4: ...
    @property
    def vovo(self) -> U32Vector4: ...
    @property
    def vovl(self) -> U32Vector4: ...
    @property
    def voou(self) -> U32Vector4: ...
    @property
    def voov(self) -> U32Vector4: ...
    @property
    def vooo(self) -> U32Vector4: ...
    @property
    def vool(self) -> U32Vector4: ...
    @property
    def volu(self) -> U32Vector4: ...
    @property
    def volv(self) -> U32Vector4: ...
    @property
    def volo(self) -> U32Vector4: ...
    @property
    def voll(self) -> U32Vector4: ...
    @property
    def vluu(self) -> U32Vector4: ...
    @property
    def vluv(self) -> U32Vector4: ...
    @property
    def vluo(self) -> U32Vector4: ...
    @property
    def vlul(self) -> U32Vector4: ...
    @property
    def vlvu(self) -> U32Vector4: ...
    @property
    def vlvv(self) -> U32Vector4: ...
    @property
    def vlvo(self) -> U32Vector4: ...
    @property
    def vlvl(self) -> U32Vector4: ...
    @property
    def vlou(self) -> U32Vector4: ...
    @property
    def vlov(self) -> U32Vector4: ...
    @property
    def vloo(self) -> U32Vector4: ...
    @property
    def vlol(self) -> U32Vector4: ...
    @property
    def vllu(self) -> U32Vector4: ...
    @property
    def vllv(self) -> U32Vector4: ...
    @property
    def vllo(self) -> U32Vector4: ...
    @property
    def vlll(self) -> U32Vector4: ...
    @property
    def ouuu(self) -> U32Vector4: ...
    @property
    def ouuv(self) -> U32Vector4: ...
    @property
    def ouuo(self) -> U32Vector4: ...
    @property
    def ouul(self) -> U32Vector4: ...
    @property
    def ouvu(self) -> U32Vector4: ...
    @property
    def ouvv(self) -> U32Vector4: ...
    @property
    def ouvo(self) -> U32Vector4: ...
    @property
    def ouvl(self) -> U32Vector4: ...
    @property
    def ouou(self) -> U32Vector4: ...
    @property
    def ouov(self) -> U32Vector4: ...
    @property
    def ouoo(self) -> U32Vector4: ...
    @property
    def ouol(self) -> U32Vector4: ...
    @property
    def oulu(self) -> U32Vector4: ...
    @property
    def oulv(self) -> U32Vector4: ...
    @property
    def oulo(self) -> U32Vector4: ...
    @property
    def oull(self) -> U32Vector4: ...
    @property
    def ovuu(self) -> U32Vector4: ...
    @property
    def ovuv(self) -> U32Vector4: ...
    @property
    def ovuo(self) -> U32Vector4: ...
    @property
    def ovul(self) -> U32Vector4: ...
    @property
    def ovvu(self) -> U32Vector4: ...
    @property
    def ovvv(self) -> U32Vector4: ...
    @property
    def ovvo(self) -> U32Vector4: ...
    @property
    def ovvl(self) -> U32Vector4: ...
    @property
    def ovou(self) -> U32Vector4: ...
    @property
    def ovov(self) -> U32Vector4: ...
    @property
    def ovoo(self) -> U32Vector4: ...
    @property
    def ovol(self) -> U32Vector4: ...
    @property
    def ovlu(self) -> U32Vector4: ...
    @property
    def ovlv(self) -> U32Vector4: ...
    @property
    def ovlo(self) -> U32Vector4: ...
    @property
    def ovll(self) -> U32Vector4: ...
    @property
    def oouu(self) -> U32Vector4: ...
    @property
    def oouv(self) -> U32Vector4: ...
    @property
    def oouo(self) -> U32Vector4: ...
    @property
    def ooul(self) -> U32Vector4: ...
    @property
    def oovu(self) -> U32Vector4: ...
    @property
    def oovv(self) -> U32Vector4: ...
    @property
    def oovo(self) -> U32Vector4: ...
    @property
    def oovl(self) -> U32Vector4: ...
    @property
    def ooou(self) -> U32Vector4: ...
    @property
    def ooov(self) -> U32Vector4: ...
    @property
    def oolu(self) -> U32Vector4: ...
    @property
    def oolv(self) -> U32Vector4: ...
    @property
    def oluu(self) -> U32Vector4: ...
    @property
    def oluv(self) -> U32Vector4: ...
    @property
    def oluo(self) -> U32Vector4: ...
    @property
    def olul(self) -> U32Vector4: ...
    @property
    def olvu(self) -> U32Vector4: ...
    @property
    def olvv(self) -> U32Vector4: ...
    @property
    def olvo(self) -> U32Vector4: ...
    @property
    def olvl(self) -> U32Vector4: ...
    @property
    def olou(self) -> U32Vector4: ...
    @property
    def olov(self) -> U32Vector4: ...
    @property
    def ollu(self) -> U32Vector4: ...
    @property
    def ollv(self) -> U32Vector4: ...
    @property
    def luuu(self) -> U32Vector4: ...
    @property
    def luuv(self) -> U32Vector4: ...
    @property
    def luuo(self) -> U32Vector4: ...
    @property
    def luul(self) -> U32Vector4: ...
    @property
    def luvu(self) -> U32Vector4: ...
    @property
    def luvv(self) -> U32Vector4: ...
    @property
    def luvo(self) -> U32Vector4: ...
    @property
    def luvl(self) -> U32Vector4: ...
    @property
    def luou(self) -> U32Vector4: ...
    @property
    def luov(self) -> U32Vector4: ...
    @property
    def luoo(self) -> U32Vector4: ...
    @property
    def luol(self) -> U32Vector4: ...
    @property
    def lulu(self) -> U32Vector4: ...
    @property
    def lulv(self) -> U32Vector4: ...
    @property
    def lulo(self) -> U32Vector4: ...
    @property
    def lull(self) -> U32Vector4: ...
    @property
    def lvuu(self) -> U32Vector4: ...
    @property
    def lvuv(self) -> U32Vector4: ...
    @property
    def lvuo(self) -> U32Vector4: ...
    @property
    def lvul(self) -> U32Vector4: ...
    @property
    def lvvu(self) -> U32Vector4: ...
    @property
    def lvvv(self) -> U32Vector4: ...
    @property
    def lvvo(self) -> U32Vector4: ...
    @property
    def lvvl(self) -> U32Vector4: ...
    @property
    def lvou(self) -> U32Vector4: ...
    @property
    def lvov(self) -> U32Vector4: ...
    @property
    def lvoo(self) -> U32Vector4: ...
    @property
    def lvol(self) -> U32Vector4: ...
    @property
    def lvlu(self) -> U32Vector4: ...
    @property
    def lvlv(self) -> U32Vector4: ...
    @property
    def lvlo(self) -> U32Vector4: ...
    @property
    def lvll(self) -> U32Vector4: ...
    @property
    def louu(self) -> U32Vector4: ...
    @property
    def louv(self) -> U32Vector4: ...
    @property
    def louo(self) -> U32Vector4: ...
    @property
    def loul(self) -> U32Vector4: ...
    @property
    def lovu(self) -> U32Vector4: ...
    @property
    def lovv(self) -> U32Vector4: ...
    @property
    def lovo(self) -> U32Vector4: ...
    @property
    def lovl(self) -> U32Vector4: ...
    @property
    def loou(self) -> U32Vector4: ...
    @property
    def loov(self) -> U32Vector4: ...
    @property
    def lolu(self) -> U32Vector4: ...
    @property
    def lolv(self) -> U32Vector4: ...
    @property
    def lluu(self) -> U32Vector4: ...
    @property
    def lluv(self) -> U32Vector4: ...
    @property
    def lluo(self) -> U32Vector4: ...
    @property
    def llul(self) -> U32Vector4: ...
    @property
    def llvu(self) -> U32Vector4: ...
    @property
    def llvv(self) -> U32Vector4: ...
    @property
    def llvo(self) -> U32Vector4: ...
    @property
    def llvl(self) -> U32Vector4: ...
    @property
    def llou(self) -> U32Vector4: ...
    @property
    def llov(self) -> U32Vector4: ...
    @property
    def lllu(self) -> U32Vector4: ...
    @property
    def lllv(self) -> U32Vector4: ...
    @property
    def oo(self) -> U32Vector2: ...
    @property
    def ol(self) -> U32Vector2: ...
    @property
    def lo(self) -> U32Vector2: ...
    @property
    def ll(self) -> U32Vector2: ...
    @property
    def ooo(self) -> U32Vector3: ...
    @property
    def ool(self) -> U32Vector3: ...
    @property
    def olo(self) -> U32Vector3: ...
    @property
    def oll(self) -> U32Vector3: ...
    @property
    def loo(self) -> U32Vector3: ...
    @property
    def lol(self) -> U32Vector3: ...
    @property
    def llo(self) -> U32Vector3: ...
    @property
    def lll(self) -> U32Vector3: ...
    @property
    def oooo(self) -> U32Vector4: ...
    @property
    def oool(self) -> U32Vector4: ...
    @property
    def oolo(self) -> U32Vector4: ...
    @property
    def ooll(self) -> U32Vector4: ...
    @property
    def oloo(self) -> U32Vector4: ...
    @property
    def olol(self) -> U32Vector4: ...
    @property
    def ollo(self) -> U32Vector4: ...
    @property
    def olll(self) -> U32Vector4: ...
    @property
    def looo(self) -> U32Vector4: ...
    @property
    def lool(self) -> U32Vector4: ...
    @property
    def lolo(self) -> U32Vector4: ...
    @property
    def loll(self) -> U32Vector4: ...
    @property
    def lloo(self) -> U32Vector4: ...
    @property
    def llol(self) -> U32Vector4: ...
    @property
    def lllo(self) -> U32Vector4: ...
    @property
    def llll(self) -> U32Vector4: ...
    def min(self, n: Number, /) -> U32Vector2: ...
    def max(self, n: Number, /) -> U32Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> U32Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U32Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector2: ...

@final
class U32Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U32Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U32Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> U32Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> U32Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[U32Vector2]: ...

@final
class IVector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: IVector2) -> IVector2: ...
    @overload
    def __add__(self, other: Number) -> IVector2: ...
    @overload
    def __radd__(self, other: IVector2) -> IVector2: ...
    @overload
    def __radd__(self, other: Number) -> IVector2: ...
    @overload
    def __sub__(self, other: IVector2) -> IVector2: ...
    @overload
    def __sub__(self, other: Number) -> IVector2: ...
    @overload
    def __rsub__(self, other: IVector2) -> IVector2: ...
    @overload
    def __rsub__(self, other: Number) -> IVector2: ...
    @overload
    def __mul__(self, other: IVector2) -> IVector2: ...
    @overload
    def __mul__(self, other: Number) -> IVector2: ...
    @overload
    def __rmul__(self, other: IVector2) -> IVector2: ...
    @overload
    def __rmul__(self, other: Number) -> IVector2: ...
    @overload
    def __truediv__(self, other: IVector2) -> IVector2: ...
    @overload
    def __truediv__(self, other: Number) -> IVector2: ...
    @overload
    def __rtruediv__(self, other: IVector2) -> IVector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> IVector2: ...
    def __neg__(self) -> IVector2: ...
    def __abs__(self) -> IVector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> IVector2: ...
    @property
    def xy(self) -> IVector2: ...
    @property
    def xo(self) -> IVector2: ...
    @property
    def xl(self) -> IVector2: ...
    @property
    def yx(self) -> IVector2: ...
    @property
    def yy(self) -> IVector2: ...
    @property
    def yo(self) -> IVector2: ...
    @property
    def yl(self) -> IVector2: ...
    @property
    def ox(self) -> IVector2: ...
    @property
    def oy(self) -> IVector2: ...
    @property
    def lx(self) -> IVector2: ...
    @property
    def ly(self) -> IVector2: ...
    @property
    def xxx(self) -> IVector3: ...
    @property
    def xxy(self) -> IVector3: ...
    @property
    def xxo(self) -> IVector3: ...
    @property
    def xxl(self) -> IVector3: ...
    @property
    def xyx(self) -> IVector3: ...
    @property
    def xyy(self) -> IVector3: ...
    @property
    def xyo(self) -> IVector3: ...
    @property
    def xyl(self) -> IVector3: ...
    @property
    def xox(self) -> IVector3: ...
    @property
    def xoy(self) -> IVector3: ...
    @property
    def xoo(self) -> IVector3: ...
    @property
    def xol(self) -> IVector3: ...
    @property
    def xlx(self) -> IVector3: ...
    @property
    def xly(self) -> IVector3: ...
    @property
    def xlo(self) -> IVector3: ...
    @property
    def xll(self) -> IVector3: ...
    @property
    def yxx(self) -> IVector3: ...
    @property
    def yxy(self) -> IVector3: ...
    @property
    def yxo(self) -> IVector3: ...
    @property
    def yxl(self) -> IVector3: ...
    @property
    def yyx(self) -> IVector3: ...
    @property
    def yyy(self) -> IVector3: ...
    @property
    def yyo(self) -> IVector3: ...
    @property
    def yyl(self) -> IVector3: ...
    @property
    def yox(self) -> IVector3: ...
    @property
    def yoy(self) -> IVector3: ...
    @property
    def yoo(self) -> IVector3: ...
    @property
    def yol(self) -> IVector3: ...
    @property
    def ylx(self) -> IVector3: ...
    @property
    def yly(self) -> IVector3: ...
    @property
    def ylo(self) -> IVector3: ...
    @property
    def yll(self) -> IVector3: ...
    @property
    def oxx(self) -> IVector3: ...
    @property
    def oxy(self) -> IVector3: ...
    @property
    def oxo(self) -> IVector3: ...
    @property
    def oxl(self) -> IVector3: ...
    @property
    def oyx(self) -> IVector3: ...
    @property
    def oyy(self) -> IVector3: ...
    @property
    def oyo(self) -> IVector3: ...
    @property
    def oyl(self) -> IVector3: ...
    @property
    def oox(self) -> IVector3: ...
    @property
    def ooy(self) -> IVector3: ...
    @property
    def olx(self) -> IVector3: ...
    @property
    def oly(self) -> IVector3: ...
    @property
    def lxx(self) -> IVector3: ...
    @property
    def lxy(self) -> IVector3: ...
    @property
    def lxo(self) -> IVector3: ...
    @property
    def lxl(self) -> IVector3: ...
    @property
    def lyx(self) -> IVector3: ...
    @property
    def lyy(self) -> IVector3: ...
    @property
    def lyo(self) -> IVector3: ...
    @property
    def lyl(self) -> IVector3: ...
    @property
    def lox(self) -> IVector3: ...
    @property
    def loy(self) -> IVector3: ...
    @property
    def llx(self) -> IVector3: ...
    @property
    def lly(self) -> IVector3: ...
    @property
    def xxxx(self) -> IVector4: ...
    @property
    def xxxy(self) -> IVector4: ...
    @property
    def xxxo(self) -> IVector4: ...
    @property
    def xxxl(self) -> IVector4: ...
    @property
    def xxyx(self) -> IVector4: ...
    @property
    def xxyy(self) -> IVector4: ...
    @property
    def xxyo(self) -> IVector4: ...
    @property
    def xxyl(self) -> IVector4: ...
    @property
    def xxox(self) -> IVector4: ...
    @property
    def xxoy(self) -> IVector4: ...
    @property
    def xxoo(self) -> IVector4: ...
    @property
    def xxol(self) -> IVector4: ...
    @property
    def xxlx(self) -> IVector4: ...
    @property
    def xxly(self) -> IVector4: ...
    @property
    def xxlo(self) -> IVector4: ...
    @property
    def xxll(self) -> IVector4: ...
    @property
    def xyxx(self) -> IVector4: ...
    @property
    def xyxy(self) -> IVector4: ...
    @property
    def xyxo(self) -> IVector4: ...
    @property
    def xyxl(self) -> IVector4: ...
    @property
    def xyyx(self) -> IVector4: ...
    @property
    def xyyy(self) -> IVector4: ...
    @property
    def xyyo(self) -> IVector4: ...
    @property
    def xyyl(self) -> IVector4: ...
    @property
    def xyox(self) -> IVector4: ...
    @property
    def xyoy(self) -> IVector4: ...
    @property
    def xyoo(self) -> IVector4: ...
    @property
    def xyol(self) -> IVector4: ...
    @property
    def xylx(self) -> IVector4: ...
    @property
    def xyly(self) -> IVector4: ...
    @property
    def xylo(self) -> IVector4: ...
    @property
    def xyll(self) -> IVector4: ...
    @property
    def xoxx(self) -> IVector4: ...
    @property
    def xoxy(self) -> IVector4: ...
    @property
    def xoxo(self) -> IVector4: ...
    @property
    def xoxl(self) -> IVector4: ...
    @property
    def xoyx(self) -> IVector4: ...
    @property
    def xoyy(self) -> IVector4: ...
    @property
    def xoyo(self) -> IVector4: ...
    @property
    def xoyl(self) -> IVector4: ...
    @property
    def xoox(self) -> IVector4: ...
    @property
    def xooy(self) -> IVector4: ...
    @property
    def xooo(self) -> IVector4: ...
    @property
    def xool(self) -> IVector4: ...
    @property
    def xolx(self) -> IVector4: ...
    @property
    def xoly(self) -> IVector4: ...
    @property
    def xolo(self) -> IVector4: ...
    @property
    def xoll(self) -> IVector4: ...
    @property
    def xlxx(self) -> IVector4: ...
    @property
    def xlxy(self) -> IVector4: ...
    @property
    def xlxo(self) -> IVector4: ...
    @property
    def xlxl(self) -> IVector4: ...
    @property
    def xlyx(self) -> IVector4: ...
    @property
    def xlyy(self) -> IVector4: ...
    @property
    def xlyo(self) -> IVector4: ...
    @property
    def xlyl(self) -> IVector4: ...
    @property
    def xlox(self) -> IVector4: ...
    @property
    def xloy(self) -> IVector4: ...
    @property
    def xloo(self) -> IVector4: ...
    @property
    def xlol(self) -> IVector4: ...
    @property
    def xllx(self) -> IVector4: ...
    @property
    def xlly(self) -> IVector4: ...
    @property
    def xllo(self) -> IVector4: ...
    @property
    def xlll(self) -> IVector4: ...
    @property
    def yxxx(self) -> IVector4: ...
    @property
    def yxxy(self) -> IVector4: ...
    @property
    def yxxo(self) -> IVector4: ...
    @property
    def yxxl(self) -> IVector4: ...
    @property
    def yxyx(self) -> IVector4: ...
    @property
    def yxyy(self) -> IVector4: ...
    @property
    def yxyo(self) -> IVector4: ...
    @property
    def yxyl(self) -> IVector4: ...
    @property
    def yxox(self) -> IVector4: ...
    @property
    def yxoy(self) -> IVector4: ...
    @property
    def yxoo(self) -> IVector4: ...
    @property
    def yxol(self) -> IVector4: ...
    @property
    def yxlx(self) -> IVector4: ...
    @property
    def yxly(self) -> IVector4: ...
    @property
    def yxlo(self) -> IVector4: ...
    @property
    def yxll(self) -> IVector4: ...
    @property
    def yyxx(self) -> IVector4: ...
    @property
    def yyxy(self) -> IVector4: ...
    @property
    def yyxo(self) -> IVector4: ...
    @property
    def yyxl(self) -> IVector4: ...
    @property
    def yyyx(self) -> IVector4: ...
    @property
    def yyyy(self) -> IVector4: ...
    @property
    def yyyo(self) -> IVector4: ...
    @property
    def yyyl(self) -> IVector4: ...
    @property
    def yyox(self) -> IVector4: ...
    @property
    def yyoy(self) -> IVector4: ...
    @property
    def yyoo(self) -> IVector4: ...
    @property
    def yyol(self) -> IVector4: ...
    @property
    def yylx(self) -> IVector4: ...
    @property
    def yyly(self) -> IVector4: ...
    @property
    def yylo(self) -> IVector4: ...
    @property
    def yyll(self) -> IVector4: ...
    @property
    def yoxx(self) -> IVector4: ...
    @property
    def yoxy(self) -> IVector4: ...
    @property
    def yoxo(self) -> IVector4: ...
    @property
    def yoxl(self) -> IVector4: ...
    @property
    def yoyx(self) -> IVector4: ...
    @property
    def yoyy(self) -> IVector4: ...
    @property
    def yoyo(self) -> IVector4: ...
    @property
    def yoyl(self) -> IVector4: ...
    @property
    def yoox(self) -> IVector4: ...
    @property
    def yooy(self) -> IVector4: ...
    @property
    def yooo(self) -> IVector4: ...
    @property
    def yool(self) -> IVector4: ...
    @property
    def yolx(self) -> IVector4: ...
    @property
    def yoly(self) -> IVector4: ...
    @property
    def yolo(self) -> IVector4: ...
    @property
    def yoll(self) -> IVector4: ...
    @property
    def ylxx(self) -> IVector4: ...
    @property
    def ylxy(self) -> IVector4: ...
    @property
    def ylxo(self) -> IVector4: ...
    @property
    def ylxl(self) -> IVector4: ...
    @property
    def ylyx(self) -> IVector4: ...
    @property
    def ylyy(self) -> IVector4: ...
    @property
    def ylyo(self) -> IVector4: ...
    @property
    def ylyl(self) -> IVector4: ...
    @property
    def ylox(self) -> IVector4: ...
    @property
    def yloy(self) -> IVector4: ...
    @property
    def yloo(self) -> IVector4: ...
    @property
    def ylol(self) -> IVector4: ...
    @property
    def yllx(self) -> IVector4: ...
    @property
    def ylly(self) -> IVector4: ...
    @property
    def yllo(self) -> IVector4: ...
    @property
    def ylll(self) -> IVector4: ...
    @property
    def oxxx(self) -> IVector4: ...
    @property
    def oxxy(self) -> IVector4: ...
    @property
    def oxxo(self) -> IVector4: ...
    @property
    def oxxl(self) -> IVector4: ...
    @property
    def oxyx(self) -> IVector4: ...
    @property
    def oxyy(self) -> IVector4: ...
    @property
    def oxyo(self) -> IVector4: ...
    @property
    def oxyl(self) -> IVector4: ...
    @property
    def oxox(self) -> IVector4: ...
    @property
    def oxoy(self) -> IVector4: ...
    @property
    def oxoo(self) -> IVector4: ...
    @property
    def oxol(self) -> IVector4: ...
    @property
    def oxlx(self) -> IVector4: ...
    @property
    def oxly(self) -> IVector4: ...
    @property
    def oxlo(self) -> IVector4: ...
    @property
    def oxll(self) -> IVector4: ...
    @property
    def oyxx(self) -> IVector4: ...
    @property
    def oyxy(self) -> IVector4: ...
    @property
    def oyxo(self) -> IVector4: ...
    @property
    def oyxl(self) -> IVector4: ...
    @property
    def oyyx(self) -> IVector4: ...
    @property
    def oyyy(self) -> IVector4: ...
    @property
    def oyyo(self) -> IVector4: ...
    @property
    def oyyl(self) -> IVector4: ...
    @property
    def oyox(self) -> IVector4: ...
    @property
    def oyoy(self) -> IVector4: ...
    @property
    def oyoo(self) -> IVector4: ...
    @property
    def oyol(self) -> IVector4: ...
    @property
    def oylx(self) -> IVector4: ...
    @property
    def oyly(self) -> IVector4: ...
    @property
    def oylo(self) -> IVector4: ...
    @property
    def oyll(self) -> IVector4: ...
    @property
    def ooxx(self) -> IVector4: ...
    @property
    def ooxy(self) -> IVector4: ...
    @property
    def ooxo(self) -> IVector4: ...
    @property
    def ooxl(self) -> IVector4: ...
    @property
    def ooyx(self) -> IVector4: ...
    @property
    def ooyy(self) -> IVector4: ...
    @property
    def ooyo(self) -> IVector4: ...
    @property
    def ooyl(self) -> IVector4: ...
    @property
    def ooox(self) -> IVector4: ...
    @property
    def oooy(self) -> IVector4: ...
    @property
    def oolx(self) -> IVector4: ...
    @property
    def ooly(self) -> IVector4: ...
    @property
    def olxx(self) -> IVector4: ...
    @property
    def olxy(self) -> IVector4: ...
    @property
    def olxo(self) -> IVector4: ...
    @property
    def olxl(self) -> IVector4: ...
    @property
    def olyx(self) -> IVector4: ...
    @property
    def olyy(self) -> IVector4: ...
    @property
    def olyo(self) -> IVector4: ...
    @property
    def olyl(self) -> IVector4: ...
    @property
    def olox(self) -> IVector4: ...
    @property
    def oloy(self) -> IVector4: ...
    @property
    def ollx(self) -> IVector4: ...
    @property
    def olly(self) -> IVector4: ...
    @property
    def lxxx(self) -> IVector4: ...
    @property
    def lxxy(self) -> IVector4: ...
    @property
    def lxxo(self) -> IVector4: ...
    @property
    def lxxl(self) -> IVector4: ...
    @property
    def lxyx(self) -> IVector4: ...
    @property
    def lxyy(self) -> IVector4: ...
    @property
    def lxyo(self) -> IVector4: ...
    @property
    def lxyl(self) -> IVector4: ...
    @property
    def lxox(self) -> IVector4: ...
    @property
    def lxoy(self) -> IVector4: ...
    @property
    def lxoo(self) -> IVector4: ...
    @property
    def lxol(self) -> IVector4: ...
    @property
    def lxlx(self) -> IVector4: ...
    @property
    def lxly(self) -> IVector4: ...
    @property
    def lxlo(self) -> IVector4: ...
    @property
    def lxll(self) -> IVector4: ...
    @property
    def lyxx(self) -> IVector4: ...
    @property
    def lyxy(self) -> IVector4: ...
    @property
    def lyxo(self) -> IVector4: ...
    @property
    def lyxl(self) -> IVector4: ...
    @property
    def lyyx(self) -> IVector4: ...
    @property
    def lyyy(self) -> IVector4: ...
    @property
    def lyyo(self) -> IVector4: ...
    @property
    def lyyl(self) -> IVector4: ...
    @property
    def lyox(self) -> IVector4: ...
    @property
    def lyoy(self) -> IVector4: ...
    @property
    def lyoo(self) -> IVector4: ...
    @property
    def lyol(self) -> IVector4: ...
    @property
    def lylx(self) -> IVector4: ...
    @property
    def lyly(self) -> IVector4: ...
    @property
    def lylo(self) -> IVector4: ...
    @property
    def lyll(self) -> IVector4: ...
    @property
    def loxx(self) -> IVector4: ...
    @property
    def loxy(self) -> IVector4: ...
    @property
    def loxo(self) -> IVector4: ...
    @property
    def loxl(self) -> IVector4: ...
    @property
    def loyx(self) -> IVector4: ...
    @property
    def loyy(self) -> IVector4: ...
    @property
    def loyo(self) -> IVector4: ...
    @property
    def loyl(self) -> IVector4: ...
    @property
    def loox(self) -> IVector4: ...
    @property
    def looy(self) -> IVector4: ...
    @property
    def lolx(self) -> IVector4: ...
    @property
    def loly(self) -> IVector4: ...
    @property
    def llxx(self) -> IVector4: ...
    @property
    def llxy(self) -> IVector4: ...
    @property
    def llxo(self) -> IVector4: ...
    @property
    def llxl(self) -> IVector4: ...
    @property
    def llyx(self) -> IVector4: ...
    @property
    def llyy(self) -> IVector4: ...
    @property
    def llyo(self) -> IVector4: ...
    @property
    def llyl(self) -> IVector4: ...
    @property
    def llox(self) -> IVector4: ...
    @property
    def lloy(self) -> IVector4: ...
    @property
    def lllx(self) -> IVector4: ...
    @property
    def llly(self) -> IVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> IVector2: ...
    @property
    def rg(self) -> IVector2: ...
    @property
    def ro(self) -> IVector2: ...
    @property
    def rl(self) -> IVector2: ...
    @property
    def gr(self) -> IVector2: ...
    @property
    def gg(self) -> IVector2: ...
    @property
    def go(self) -> IVector2: ...
    @property
    def gl(self) -> IVector2: ...
    @property
    def og(self) -> IVector2: ...
    @property
    def lr(self) -> IVector2: ...
    @property
    def lg(self) -> IVector2: ...
    @property
    def rrr(self) -> IVector3: ...
    @property
    def rrg(self) -> IVector3: ...
    @property
    def rro(self) -> IVector3: ...
    @property
    def rrl(self) -> IVector3: ...
    @property
    def rgr(self) -> IVector3: ...
    @property
    def rgg(self) -> IVector3: ...
    @property
    def rgo(self) -> IVector3: ...
    @property
    def rgl(self) -> IVector3: ...
    @property
    def ror(self) -> IVector3: ...
    @property
    def rog(self) -> IVector3: ...
    @property
    def roo(self) -> IVector3: ...
    @property
    def rol(self) -> IVector3: ...
    @property
    def rlr(self) -> IVector3: ...
    @property
    def rlg(self) -> IVector3: ...
    @property
    def rlo(self) -> IVector3: ...
    @property
    def rll(self) -> IVector3: ...
    @property
    def grr(self) -> IVector3: ...
    @property
    def grg(self) -> IVector3: ...
    @property
    def gro(self) -> IVector3: ...
    @property
    def grl(self) -> IVector3: ...
    @property
    def ggr(self) -> IVector3: ...
    @property
    def ggg(self) -> IVector3: ...
    @property
    def ggo(self) -> IVector3: ...
    @property
    def ggl(self) -> IVector3: ...
    @property
    def gor(self) -> IVector3: ...
    @property
    def gog(self) -> IVector3: ...
    @property
    def goo(self) -> IVector3: ...
    @property
    def gol(self) -> IVector3: ...
    @property
    def glr(self) -> IVector3: ...
    @property
    def glg(self) -> IVector3: ...
    @property
    def glo(self) -> IVector3: ...
    @property
    def gll(self) -> IVector3: ...
    @property
    def orr(self) -> IVector3: ...
    @property
    def org(self) -> IVector3: ...
    @property
    def oro(self) -> IVector3: ...
    @property
    def orl(self) -> IVector3: ...
    @property
    def ogr(self) -> IVector3: ...
    @property
    def ogg(self) -> IVector3: ...
    @property
    def ogo(self) -> IVector3: ...
    @property
    def ogl(self) -> IVector3: ...
    @property
    def oor(self) -> IVector3: ...
    @property
    def oog(self) -> IVector3: ...
    @property
    def olr(self) -> IVector3: ...
    @property
    def olg(self) -> IVector3: ...
    @property
    def lrr(self) -> IVector3: ...
    @property
    def lrg(self) -> IVector3: ...
    @property
    def lro(self) -> IVector3: ...
    @property
    def lrl(self) -> IVector3: ...
    @property
    def lgr(self) -> IVector3: ...
    @property
    def lgg(self) -> IVector3: ...
    @property
    def lgo(self) -> IVector3: ...
    @property
    def lgl(self) -> IVector3: ...
    @property
    def lor(self) -> IVector3: ...
    @property
    def log(self) -> IVector3: ...
    @property
    def llr(self) -> IVector3: ...
    @property
    def llg(self) -> IVector3: ...
    @property
    def rrrr(self) -> IVector4: ...
    @property
    def rrrg(self) -> IVector4: ...
    @property
    def rrro(self) -> IVector4: ...
    @property
    def rrrl(self) -> IVector4: ...
    @property
    def rrgr(self) -> IVector4: ...
    @property
    def rrgg(self) -> IVector4: ...
    @property
    def rrgo(self) -> IVector4: ...
    @property
    def rrgl(self) -> IVector4: ...
    @property
    def rror(self) -> IVector4: ...
    @property
    def rrog(self) -> IVector4: ...
    @property
    def rroo(self) -> IVector4: ...
    @property
    def rrol(self) -> IVector4: ...
    @property
    def rrlr(self) -> IVector4: ...
    @property
    def rrlg(self) -> IVector4: ...
    @property
    def rrlo(self) -> IVector4: ...
    @property
    def rrll(self) -> IVector4: ...
    @property
    def rgrr(self) -> IVector4: ...
    @property
    def rgrg(self) -> IVector4: ...
    @property
    def rgro(self) -> IVector4: ...
    @property
    def rgrl(self) -> IVector4: ...
    @property
    def rggr(self) -> IVector4: ...
    @property
    def rggg(self) -> IVector4: ...
    @property
    def rggo(self) -> IVector4: ...
    @property
    def rggl(self) -> IVector4: ...
    @property
    def rgor(self) -> IVector4: ...
    @property
    def rgog(self) -> IVector4: ...
    @property
    def rgoo(self) -> IVector4: ...
    @property
    def rgol(self) -> IVector4: ...
    @property
    def rglr(self) -> IVector4: ...
    @property
    def rglg(self) -> IVector4: ...
    @property
    def rglo(self) -> IVector4: ...
    @property
    def rgll(self) -> IVector4: ...
    @property
    def rorr(self) -> IVector4: ...
    @property
    def rorg(self) -> IVector4: ...
    @property
    def roro(self) -> IVector4: ...
    @property
    def rorl(self) -> IVector4: ...
    @property
    def rogr(self) -> IVector4: ...
    @property
    def rogg(self) -> IVector4: ...
    @property
    def rogo(self) -> IVector4: ...
    @property
    def rogl(self) -> IVector4: ...
    @property
    def roor(self) -> IVector4: ...
    @property
    def roog(self) -> IVector4: ...
    @property
    def rooo(self) -> IVector4: ...
    @property
    def rool(self) -> IVector4: ...
    @property
    def rolr(self) -> IVector4: ...
    @property
    def rolg(self) -> IVector4: ...
    @property
    def rolo(self) -> IVector4: ...
    @property
    def roll(self) -> IVector4: ...
    @property
    def rlrr(self) -> IVector4: ...
    @property
    def rlrg(self) -> IVector4: ...
    @property
    def rlro(self) -> IVector4: ...
    @property
    def rlrl(self) -> IVector4: ...
    @property
    def rlgr(self) -> IVector4: ...
    @property
    def rlgg(self) -> IVector4: ...
    @property
    def rlgo(self) -> IVector4: ...
    @property
    def rlgl(self) -> IVector4: ...
    @property
    def rlor(self) -> IVector4: ...
    @property
    def rlog(self) -> IVector4: ...
    @property
    def rloo(self) -> IVector4: ...
    @property
    def rlol(self) -> IVector4: ...
    @property
    def rllr(self) -> IVector4: ...
    @property
    def rllg(self) -> IVector4: ...
    @property
    def rllo(self) -> IVector4: ...
    @property
    def rlll(self) -> IVector4: ...
    @property
    def grrr(self) -> IVector4: ...
    @property
    def grrg(self) -> IVector4: ...
    @property
    def grro(self) -> IVector4: ...
    @property
    def grrl(self) -> IVector4: ...
    @property
    def grgr(self) -> IVector4: ...
    @property
    def grgg(self) -> IVector4: ...
    @property
    def grgo(self) -> IVector4: ...
    @property
    def grgl(self) -> IVector4: ...
    @property
    def gror(self) -> IVector4: ...
    @property
    def grog(self) -> IVector4: ...
    @property
    def groo(self) -> IVector4: ...
    @property
    def grol(self) -> IVector4: ...
    @property
    def grlr(self) -> IVector4: ...
    @property
    def grlg(self) -> IVector4: ...
    @property
    def grlo(self) -> IVector4: ...
    @property
    def grll(self) -> IVector4: ...
    @property
    def ggrr(self) -> IVector4: ...
    @property
    def ggrg(self) -> IVector4: ...
    @property
    def ggro(self) -> IVector4: ...
    @property
    def ggrl(self) -> IVector4: ...
    @property
    def gggr(self) -> IVector4: ...
    @property
    def gggg(self) -> IVector4: ...
    @property
    def gggo(self) -> IVector4: ...
    @property
    def gggl(self) -> IVector4: ...
    @property
    def ggor(self) -> IVector4: ...
    @property
    def ggog(self) -> IVector4: ...
    @property
    def ggoo(self) -> IVector4: ...
    @property
    def ggol(self) -> IVector4: ...
    @property
    def gglr(self) -> IVector4: ...
    @property
    def gglg(self) -> IVector4: ...
    @property
    def gglo(self) -> IVector4: ...
    @property
    def ggll(self) -> IVector4: ...
    @property
    def gorr(self) -> IVector4: ...
    @property
    def gorg(self) -> IVector4: ...
    @property
    def goro(self) -> IVector4: ...
    @property
    def gorl(self) -> IVector4: ...
    @property
    def gogr(self) -> IVector4: ...
    @property
    def gogg(self) -> IVector4: ...
    @property
    def gogo(self) -> IVector4: ...
    @property
    def gogl(self) -> IVector4: ...
    @property
    def goor(self) -> IVector4: ...
    @property
    def goog(self) -> IVector4: ...
    @property
    def gooo(self) -> IVector4: ...
    @property
    def gool(self) -> IVector4: ...
    @property
    def golr(self) -> IVector4: ...
    @property
    def golg(self) -> IVector4: ...
    @property
    def golo(self) -> IVector4: ...
    @property
    def goll(self) -> IVector4: ...
    @property
    def glrr(self) -> IVector4: ...
    @property
    def glrg(self) -> IVector4: ...
    @property
    def glro(self) -> IVector4: ...
    @property
    def glrl(self) -> IVector4: ...
    @property
    def glgr(self) -> IVector4: ...
    @property
    def glgg(self) -> IVector4: ...
    @property
    def glgo(self) -> IVector4: ...
    @property
    def glgl(self) -> IVector4: ...
    @property
    def glor(self) -> IVector4: ...
    @property
    def glog(self) -> IVector4: ...
    @property
    def gloo(self) -> IVector4: ...
    @property
    def glol(self) -> IVector4: ...
    @property
    def gllr(self) -> IVector4: ...
    @property
    def gllg(self) -> IVector4: ...
    @property
    def gllo(self) -> IVector4: ...
    @property
    def glll(self) -> IVector4: ...
    @property
    def orrr(self) -> IVector4: ...
    @property
    def orrg(self) -> IVector4: ...
    @property
    def orro(self) -> IVector4: ...
    @property
    def orrl(self) -> IVector4: ...
    @property
    def orgr(self) -> IVector4: ...
    @property
    def orgg(self) -> IVector4: ...
    @property
    def orgo(self) -> IVector4: ...
    @property
    def orgl(self) -> IVector4: ...
    @property
    def oror(self) -> IVector4: ...
    @property
    def orog(self) -> IVector4: ...
    @property
    def oroo(self) -> IVector4: ...
    @property
    def orol(self) -> IVector4: ...
    @property
    def orlr(self) -> IVector4: ...
    @property
    def orlg(self) -> IVector4: ...
    @property
    def orlo(self) -> IVector4: ...
    @property
    def orll(self) -> IVector4: ...
    @property
    def ogrr(self) -> IVector4: ...
    @property
    def ogrg(self) -> IVector4: ...
    @property
    def ogro(self) -> IVector4: ...
    @property
    def ogrl(self) -> IVector4: ...
    @property
    def oggr(self) -> IVector4: ...
    @property
    def oggg(self) -> IVector4: ...
    @property
    def oggo(self) -> IVector4: ...
    @property
    def oggl(self) -> IVector4: ...
    @property
    def ogor(self) -> IVector4: ...
    @property
    def ogog(self) -> IVector4: ...
    @property
    def ogoo(self) -> IVector4: ...
    @property
    def ogol(self) -> IVector4: ...
    @property
    def oglr(self) -> IVector4: ...
    @property
    def oglg(self) -> IVector4: ...
    @property
    def oglo(self) -> IVector4: ...
    @property
    def ogll(self) -> IVector4: ...
    @property
    def oorr(self) -> IVector4: ...
    @property
    def oorg(self) -> IVector4: ...
    @property
    def ooro(self) -> IVector4: ...
    @property
    def oorl(self) -> IVector4: ...
    @property
    def oogr(self) -> IVector4: ...
    @property
    def oogg(self) -> IVector4: ...
    @property
    def oogo(self) -> IVector4: ...
    @property
    def oogl(self) -> IVector4: ...
    @property
    def ooor(self) -> IVector4: ...
    @property
    def ooog(self) -> IVector4: ...
    @property
    def oolr(self) -> IVector4: ...
    @property
    def oolg(self) -> IVector4: ...
    @property
    def olrr(self) -> IVector4: ...
    @property
    def olrg(self) -> IVector4: ...
    @property
    def olro(self) -> IVector4: ...
    @property
    def olrl(self) -> IVector4: ...
    @property
    def olgr(self) -> IVector4: ...
    @property
    def olgg(self) -> IVector4: ...
    @property
    def olgo(self) -> IVector4: ...
    @property
    def olgl(self) -> IVector4: ...
    @property
    def olor(self) -> IVector4: ...
    @property
    def olog(self) -> IVector4: ...
    @property
    def ollr(self) -> IVector4: ...
    @property
    def ollg(self) -> IVector4: ...
    @property
    def lrrr(self) -> IVector4: ...
    @property
    def lrrg(self) -> IVector4: ...
    @property
    def lrro(self) -> IVector4: ...
    @property
    def lrrl(self) -> IVector4: ...
    @property
    def lrgr(self) -> IVector4: ...
    @property
    def lrgg(self) -> IVector4: ...
    @property
    def lrgo(self) -> IVector4: ...
    @property
    def lrgl(self) -> IVector4: ...
    @property
    def lror(self) -> IVector4: ...
    @property
    def lrog(self) -> IVector4: ...
    @property
    def lroo(self) -> IVector4: ...
    @property
    def lrol(self) -> IVector4: ...
    @property
    def lrlr(self) -> IVector4: ...
    @property
    def lrlg(self) -> IVector4: ...
    @property
    def lrlo(self) -> IVector4: ...
    @property
    def lrll(self) -> IVector4: ...
    @property
    def lgrr(self) -> IVector4: ...
    @property
    def lgrg(self) -> IVector4: ...
    @property
    def lgro(self) -> IVector4: ...
    @property
    def lgrl(self) -> IVector4: ...
    @property
    def lggr(self) -> IVector4: ...
    @property
    def lggg(self) -> IVector4: ...
    @property
    def lggo(self) -> IVector4: ...
    @property
    def lggl(self) -> IVector4: ...
    @property
    def lgor(self) -> IVector4: ...
    @property
    def lgog(self) -> IVector4: ...
    @property
    def lgoo(self) -> IVector4: ...
    @property
    def lgol(self) -> IVector4: ...
    @property
    def lglr(self) -> IVector4: ...
    @property
    def lglg(self) -> IVector4: ...
    @property
    def lglo(self) -> IVector4: ...
    @property
    def lgll(self) -> IVector4: ...
    @property
    def lorr(self) -> IVector4: ...
    @property
    def lorg(self) -> IVector4: ...
    @property
    def loro(self) -> IVector4: ...
    @property
    def lorl(self) -> IVector4: ...
    @property
    def logr(self) -> IVector4: ...
    @property
    def logg(self) -> IVector4: ...
    @property
    def logo(self) -> IVector4: ...
    @property
    def logl(self) -> IVector4: ...
    @property
    def loor(self) -> IVector4: ...
    @property
    def loog(self) -> IVector4: ...
    @property
    def lolr(self) -> IVector4: ...
    @property
    def lolg(self) -> IVector4: ...
    @property
    def llrr(self) -> IVector4: ...
    @property
    def llrg(self) -> IVector4: ...
    @property
    def llro(self) -> IVector4: ...
    @property
    def llrl(self) -> IVector4: ...
    @property
    def llgr(self) -> IVector4: ...
    @property
    def llgg(self) -> IVector4: ...
    @property
    def llgo(self) -> IVector4: ...
    @property
    def llgl(self) -> IVector4: ...
    @property
    def llor(self) -> IVector4: ...
    @property
    def llog(self) -> IVector4: ...
    @property
    def lllr(self) -> IVector4: ...
    @property
    def lllg(self) -> IVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> IVector2: ...
    @property
    def st(self) -> IVector2: ...
    @property
    def so(self) -> IVector2: ...
    @property
    def sl(self) -> IVector2: ...
    @property
    def ts(self) -> IVector2: ...
    @property
    def tt(self) -> IVector2: ...
    @property
    def to(self) -> IVector2: ...
    @property
    def tl(self) -> IVector2: ...
    @property
    def os(self) -> IVector2: ...
    @property
    def ot(self) -> IVector2: ...
    @property
    def ls(self) -> IVector2: ...
    @property
    def lt(self) -> IVector2: ...
    @property
    def sss(self) -> IVector3: ...
    @property
    def sst(self) -> IVector3: ...
    @property
    def sso(self) -> IVector3: ...
    @property
    def ssl(self) -> IVector3: ...
    @property
    def sts(self) -> IVector3: ...
    @property
    def stt(self) -> IVector3: ...
    @property
    def sto(self) -> IVector3: ...
    @property
    def stl(self) -> IVector3: ...
    @property
    def sos(self) -> IVector3: ...
    @property
    def sot(self) -> IVector3: ...
    @property
    def soo(self) -> IVector3: ...
    @property
    def sol(self) -> IVector3: ...
    @property
    def sls(self) -> IVector3: ...
    @property
    def slt(self) -> IVector3: ...
    @property
    def slo(self) -> IVector3: ...
    @property
    def sll(self) -> IVector3: ...
    @property
    def tss(self) -> IVector3: ...
    @property
    def tst(self) -> IVector3: ...
    @property
    def tso(self) -> IVector3: ...
    @property
    def tsl(self) -> IVector3: ...
    @property
    def tts(self) -> IVector3: ...
    @property
    def ttt(self) -> IVector3: ...
    @property
    def tto(self) -> IVector3: ...
    @property
    def ttl(self) -> IVector3: ...
    @property
    def tos(self) -> IVector3: ...
    @property
    def tot(self) -> IVector3: ...
    @property
    def too(self) -> IVector3: ...
    @property
    def tol(self) -> IVector3: ...
    @property
    def tls(self) -> IVector3: ...
    @property
    def tlt(self) -> IVector3: ...
    @property
    def tlo(self) -> IVector3: ...
    @property
    def tll(self) -> IVector3: ...
    @property
    def oss(self) -> IVector3: ...
    @property
    def ost(self) -> IVector3: ...
    @property
    def oso(self) -> IVector3: ...
    @property
    def osl(self) -> IVector3: ...
    @property
    def ots(self) -> IVector3: ...
    @property
    def ott(self) -> IVector3: ...
    @property
    def oto(self) -> IVector3: ...
    @property
    def otl(self) -> IVector3: ...
    @property
    def oos(self) -> IVector3: ...
    @property
    def oot(self) -> IVector3: ...
    @property
    def ols(self) -> IVector3: ...
    @property
    def olt(self) -> IVector3: ...
    @property
    def lss(self) -> IVector3: ...
    @property
    def lst(self) -> IVector3: ...
    @property
    def lso(self) -> IVector3: ...
    @property
    def lsl(self) -> IVector3: ...
    @property
    def lts(self) -> IVector3: ...
    @property
    def ltt(self) -> IVector3: ...
    @property
    def lto(self) -> IVector3: ...
    @property
    def ltl(self) -> IVector3: ...
    @property
    def los(self) -> IVector3: ...
    @property
    def lot(self) -> IVector3: ...
    @property
    def lls(self) -> IVector3: ...
    @property
    def llt(self) -> IVector3: ...
    @property
    def ssss(self) -> IVector4: ...
    @property
    def ssst(self) -> IVector4: ...
    @property
    def ssso(self) -> IVector4: ...
    @property
    def sssl(self) -> IVector4: ...
    @property
    def ssts(self) -> IVector4: ...
    @property
    def sstt(self) -> IVector4: ...
    @property
    def ssto(self) -> IVector4: ...
    @property
    def sstl(self) -> IVector4: ...
    @property
    def ssos(self) -> IVector4: ...
    @property
    def ssot(self) -> IVector4: ...
    @property
    def ssoo(self) -> IVector4: ...
    @property
    def ssol(self) -> IVector4: ...
    @property
    def ssls(self) -> IVector4: ...
    @property
    def sslt(self) -> IVector4: ...
    @property
    def sslo(self) -> IVector4: ...
    @property
    def ssll(self) -> IVector4: ...
    @property
    def stss(self) -> IVector4: ...
    @property
    def stst(self) -> IVector4: ...
    @property
    def stso(self) -> IVector4: ...
    @property
    def stsl(self) -> IVector4: ...
    @property
    def stts(self) -> IVector4: ...
    @property
    def sttt(self) -> IVector4: ...
    @property
    def stto(self) -> IVector4: ...
    @property
    def sttl(self) -> IVector4: ...
    @property
    def stos(self) -> IVector4: ...
    @property
    def stot(self) -> IVector4: ...
    @property
    def stoo(self) -> IVector4: ...
    @property
    def stol(self) -> IVector4: ...
    @property
    def stls(self) -> IVector4: ...
    @property
    def stlt(self) -> IVector4: ...
    @property
    def stlo(self) -> IVector4: ...
    @property
    def stll(self) -> IVector4: ...
    @property
    def soss(self) -> IVector4: ...
    @property
    def sost(self) -> IVector4: ...
    @property
    def soso(self) -> IVector4: ...
    @property
    def sosl(self) -> IVector4: ...
    @property
    def sots(self) -> IVector4: ...
    @property
    def sott(self) -> IVector4: ...
    @property
    def soto(self) -> IVector4: ...
    @property
    def sotl(self) -> IVector4: ...
    @property
    def soos(self) -> IVector4: ...
    @property
    def soot(self) -> IVector4: ...
    @property
    def sooo(self) -> IVector4: ...
    @property
    def sool(self) -> IVector4: ...
    @property
    def sols(self) -> IVector4: ...
    @property
    def solt(self) -> IVector4: ...
    @property
    def solo(self) -> IVector4: ...
    @property
    def soll(self) -> IVector4: ...
    @property
    def slss(self) -> IVector4: ...
    @property
    def slst(self) -> IVector4: ...
    @property
    def slso(self) -> IVector4: ...
    @property
    def slsl(self) -> IVector4: ...
    @property
    def slts(self) -> IVector4: ...
    @property
    def sltt(self) -> IVector4: ...
    @property
    def slto(self) -> IVector4: ...
    @property
    def sltl(self) -> IVector4: ...
    @property
    def slos(self) -> IVector4: ...
    @property
    def slot(self) -> IVector4: ...
    @property
    def sloo(self) -> IVector4: ...
    @property
    def slol(self) -> IVector4: ...
    @property
    def slls(self) -> IVector4: ...
    @property
    def sllt(self) -> IVector4: ...
    @property
    def sllo(self) -> IVector4: ...
    @property
    def slll(self) -> IVector4: ...
    @property
    def tsss(self) -> IVector4: ...
    @property
    def tsst(self) -> IVector4: ...
    @property
    def tsso(self) -> IVector4: ...
    @property
    def tssl(self) -> IVector4: ...
    @property
    def tsts(self) -> IVector4: ...
    @property
    def tstt(self) -> IVector4: ...
    @property
    def tsto(self) -> IVector4: ...
    @property
    def tstl(self) -> IVector4: ...
    @property
    def tsos(self) -> IVector4: ...
    @property
    def tsot(self) -> IVector4: ...
    @property
    def tsoo(self) -> IVector4: ...
    @property
    def tsol(self) -> IVector4: ...
    @property
    def tsls(self) -> IVector4: ...
    @property
    def tslt(self) -> IVector4: ...
    @property
    def tslo(self) -> IVector4: ...
    @property
    def tsll(self) -> IVector4: ...
    @property
    def ttss(self) -> IVector4: ...
    @property
    def ttst(self) -> IVector4: ...
    @property
    def ttso(self) -> IVector4: ...
    @property
    def ttsl(self) -> IVector4: ...
    @property
    def ttts(self) -> IVector4: ...
    @property
    def tttt(self) -> IVector4: ...
    @property
    def ttto(self) -> IVector4: ...
    @property
    def tttl(self) -> IVector4: ...
    @property
    def ttos(self) -> IVector4: ...
    @property
    def ttot(self) -> IVector4: ...
    @property
    def ttoo(self) -> IVector4: ...
    @property
    def ttol(self) -> IVector4: ...
    @property
    def ttls(self) -> IVector4: ...
    @property
    def ttlt(self) -> IVector4: ...
    @property
    def ttlo(self) -> IVector4: ...
    @property
    def ttll(self) -> IVector4: ...
    @property
    def toss(self) -> IVector4: ...
    @property
    def tost(self) -> IVector4: ...
    @property
    def toso(self) -> IVector4: ...
    @property
    def tosl(self) -> IVector4: ...
    @property
    def tots(self) -> IVector4: ...
    @property
    def tott(self) -> IVector4: ...
    @property
    def toto(self) -> IVector4: ...
    @property
    def totl(self) -> IVector4: ...
    @property
    def toos(self) -> IVector4: ...
    @property
    def toot(self) -> IVector4: ...
    @property
    def tooo(self) -> IVector4: ...
    @property
    def tool(self) -> IVector4: ...
    @property
    def tols(self) -> IVector4: ...
    @property
    def tolt(self) -> IVector4: ...
    @property
    def tolo(self) -> IVector4: ...
    @property
    def toll(self) -> IVector4: ...
    @property
    def tlss(self) -> IVector4: ...
    @property
    def tlst(self) -> IVector4: ...
    @property
    def tlso(self) -> IVector4: ...
    @property
    def tlsl(self) -> IVector4: ...
    @property
    def tlts(self) -> IVector4: ...
    @property
    def tltt(self) -> IVector4: ...
    @property
    def tlto(self) -> IVector4: ...
    @property
    def tltl(self) -> IVector4: ...
    @property
    def tlos(self) -> IVector4: ...
    @property
    def tlot(self) -> IVector4: ...
    @property
    def tloo(self) -> IVector4: ...
    @property
    def tlol(self) -> IVector4: ...
    @property
    def tlls(self) -> IVector4: ...
    @property
    def tllt(self) -> IVector4: ...
    @property
    def tllo(self) -> IVector4: ...
    @property
    def tlll(self) -> IVector4: ...
    @property
    def osss(self) -> IVector4: ...
    @property
    def osst(self) -> IVector4: ...
    @property
    def osso(self) -> IVector4: ...
    @property
    def ossl(self) -> IVector4: ...
    @property
    def osts(self) -> IVector4: ...
    @property
    def ostt(self) -> IVector4: ...
    @property
    def osto(self) -> IVector4: ...
    @property
    def ostl(self) -> IVector4: ...
    @property
    def osos(self) -> IVector4: ...
    @property
    def osot(self) -> IVector4: ...
    @property
    def osoo(self) -> IVector4: ...
    @property
    def osol(self) -> IVector4: ...
    @property
    def osls(self) -> IVector4: ...
    @property
    def oslt(self) -> IVector4: ...
    @property
    def oslo(self) -> IVector4: ...
    @property
    def osll(self) -> IVector4: ...
    @property
    def otss(self) -> IVector4: ...
    @property
    def otst(self) -> IVector4: ...
    @property
    def otso(self) -> IVector4: ...
    @property
    def otsl(self) -> IVector4: ...
    @property
    def otts(self) -> IVector4: ...
    @property
    def ottt(self) -> IVector4: ...
    @property
    def otto(self) -> IVector4: ...
    @property
    def ottl(self) -> IVector4: ...
    @property
    def otos(self) -> IVector4: ...
    @property
    def otot(self) -> IVector4: ...
    @property
    def otoo(self) -> IVector4: ...
    @property
    def otol(self) -> IVector4: ...
    @property
    def otls(self) -> IVector4: ...
    @property
    def otlt(self) -> IVector4: ...
    @property
    def otlo(self) -> IVector4: ...
    @property
    def otll(self) -> IVector4: ...
    @property
    def ooss(self) -> IVector4: ...
    @property
    def oost(self) -> IVector4: ...
    @property
    def ooso(self) -> IVector4: ...
    @property
    def oosl(self) -> IVector4: ...
    @property
    def oots(self) -> IVector4: ...
    @property
    def oott(self) -> IVector4: ...
    @property
    def ooto(self) -> IVector4: ...
    @property
    def ootl(self) -> IVector4: ...
    @property
    def ooos(self) -> IVector4: ...
    @property
    def ooot(self) -> IVector4: ...
    @property
    def ools(self) -> IVector4: ...
    @property
    def oolt(self) -> IVector4: ...
    @property
    def olss(self) -> IVector4: ...
    @property
    def olst(self) -> IVector4: ...
    @property
    def olso(self) -> IVector4: ...
    @property
    def olsl(self) -> IVector4: ...
    @property
    def olts(self) -> IVector4: ...
    @property
    def oltt(self) -> IVector4: ...
    @property
    def olto(self) -> IVector4: ...
    @property
    def oltl(self) -> IVector4: ...
    @property
    def olos(self) -> IVector4: ...
    @property
    def olot(self) -> IVector4: ...
    @property
    def olls(self) -> IVector4: ...
    @property
    def ollt(self) -> IVector4: ...
    @property
    def lsss(self) -> IVector4: ...
    @property
    def lsst(self) -> IVector4: ...
    @property
    def lsso(self) -> IVector4: ...
    @property
    def lssl(self) -> IVector4: ...
    @property
    def lsts(self) -> IVector4: ...
    @property
    def lstt(self) -> IVector4: ...
    @property
    def lsto(self) -> IVector4: ...
    @property
    def lstl(self) -> IVector4: ...
    @property
    def lsos(self) -> IVector4: ...
    @property
    def lsot(self) -> IVector4: ...
    @property
    def lsoo(self) -> IVector4: ...
    @property
    def lsol(self) -> IVector4: ...
    @property
    def lsls(self) -> IVector4: ...
    @property
    def lslt(self) -> IVector4: ...
    @property
    def lslo(self) -> IVector4: ...
    @property
    def lsll(self) -> IVector4: ...
    @property
    def ltss(self) -> IVector4: ...
    @property
    def ltst(self) -> IVector4: ...
    @property
    def ltso(self) -> IVector4: ...
    @property
    def ltsl(self) -> IVector4: ...
    @property
    def ltts(self) -> IVector4: ...
    @property
    def lttt(self) -> IVector4: ...
    @property
    def ltto(self) -> IVector4: ...
    @property
    def lttl(self) -> IVector4: ...
    @property
    def ltos(self) -> IVector4: ...
    @property
    def ltot(self) -> IVector4: ...
    @property
    def ltoo(self) -> IVector4: ...
    @property
    def ltol(self) -> IVector4: ...
    @property
    def ltls(self) -> IVector4: ...
    @property
    def ltlt(self) -> IVector4: ...
    @property
    def ltlo(self) -> IVector4: ...
    @property
    def ltll(self) -> IVector4: ...
    @property
    def loss(self) -> IVector4: ...
    @property
    def lost(self) -> IVector4: ...
    @property
    def loso(self) -> IVector4: ...
    @property
    def losl(self) -> IVector4: ...
    @property
    def lots(self) -> IVector4: ...
    @property
    def lott(self) -> IVector4: ...
    @property
    def loto(self) -> IVector4: ...
    @property
    def lotl(self) -> IVector4: ...
    @property
    def loos(self) -> IVector4: ...
    @property
    def loot(self) -> IVector4: ...
    @property
    def lols(self) -> IVector4: ...
    @property
    def lolt(self) -> IVector4: ...
    @property
    def llss(self) -> IVector4: ...
    @property
    def llst(self) -> IVector4: ...
    @property
    def llso(self) -> IVector4: ...
    @property
    def llsl(self) -> IVector4: ...
    @property
    def llts(self) -> IVector4: ...
    @property
    def lltt(self) -> IVector4: ...
    @property
    def llto(self) -> IVector4: ...
    @property
    def lltl(self) -> IVector4: ...
    @property
    def llos(self) -> IVector4: ...
    @property
    def llot(self) -> IVector4: ...
    @property
    def llls(self) -> IVector4: ...
    @property
    def lllt(self) -> IVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> IVector2: ...
    @property
    def uv(self) -> IVector2: ...
    @property
    def uo(self) -> IVector2: ...
    @property
    def ul(self) -> IVector2: ...
    @property
    def vu(self) -> IVector2: ...
    @property
    def vv(self) -> IVector2: ...
    @property
    def vo(self) -> IVector2: ...
    @property
    def vl(self) -> IVector2: ...
    @property
    def ou(self) -> IVector2: ...
    @property
    def ov(self) -> IVector2: ...
    @property
    def lu(self) -> IVector2: ...
    @property
    def lv(self) -> IVector2: ...
    @property
    def uuu(self) -> IVector3: ...
    @property
    def uuv(self) -> IVector3: ...
    @property
    def uuo(self) -> IVector3: ...
    @property
    def uul(self) -> IVector3: ...
    @property
    def uvu(self) -> IVector3: ...
    @property
    def uvv(self) -> IVector3: ...
    @property
    def uvo(self) -> IVector3: ...
    @property
    def uvl(self) -> IVector3: ...
    @property
    def uou(self) -> IVector3: ...
    @property
    def uov(self) -> IVector3: ...
    @property
    def uoo(self) -> IVector3: ...
    @property
    def uol(self) -> IVector3: ...
    @property
    def ulu(self) -> IVector3: ...
    @property
    def ulv(self) -> IVector3: ...
    @property
    def ulo(self) -> IVector3: ...
    @property
    def ull(self) -> IVector3: ...
    @property
    def vuu(self) -> IVector3: ...
    @property
    def vuv(self) -> IVector3: ...
    @property
    def vuo(self) -> IVector3: ...
    @property
    def vul(self) -> IVector3: ...
    @property
    def vvu(self) -> IVector3: ...
    @property
    def vvv(self) -> IVector3: ...
    @property
    def vvo(self) -> IVector3: ...
    @property
    def vvl(self) -> IVector3: ...
    @property
    def vou(self) -> IVector3: ...
    @property
    def vov(self) -> IVector3: ...
    @property
    def voo(self) -> IVector3: ...
    @property
    def vol(self) -> IVector3: ...
    @property
    def vlu(self) -> IVector3: ...
    @property
    def vlv(self) -> IVector3: ...
    @property
    def vlo(self) -> IVector3: ...
    @property
    def vll(self) -> IVector3: ...
    @property
    def ouu(self) -> IVector3: ...
    @property
    def ouv(self) -> IVector3: ...
    @property
    def ouo(self) -> IVector3: ...
    @property
    def oul(self) -> IVector3: ...
    @property
    def ovu(self) -> IVector3: ...
    @property
    def ovv(self) -> IVector3: ...
    @property
    def ovo(self) -> IVector3: ...
    @property
    def ovl(self) -> IVector3: ...
    @property
    def oou(self) -> IVector3: ...
    @property
    def oov(self) -> IVector3: ...
    @property
    def olu(self) -> IVector3: ...
    @property
    def olv(self) -> IVector3: ...
    @property
    def luu(self) -> IVector3: ...
    @property
    def luv(self) -> IVector3: ...
    @property
    def luo(self) -> IVector3: ...
    @property
    def lul(self) -> IVector3: ...
    @property
    def lvu(self) -> IVector3: ...
    @property
    def lvv(self) -> IVector3: ...
    @property
    def lvo(self) -> IVector3: ...
    @property
    def lvl(self) -> IVector3: ...
    @property
    def lou(self) -> IVector3: ...
    @property
    def lov(self) -> IVector3: ...
    @property
    def llu(self) -> IVector3: ...
    @property
    def llv(self) -> IVector3: ...
    @property
    def uuuu(self) -> IVector4: ...
    @property
    def uuuv(self) -> IVector4: ...
    @property
    def uuuo(self) -> IVector4: ...
    @property
    def uuul(self) -> IVector4: ...
    @property
    def uuvu(self) -> IVector4: ...
    @property
    def uuvv(self) -> IVector4: ...
    @property
    def uuvo(self) -> IVector4: ...
    @property
    def uuvl(self) -> IVector4: ...
    @property
    def uuou(self) -> IVector4: ...
    @property
    def uuov(self) -> IVector4: ...
    @property
    def uuoo(self) -> IVector4: ...
    @property
    def uuol(self) -> IVector4: ...
    @property
    def uulu(self) -> IVector4: ...
    @property
    def uulv(self) -> IVector4: ...
    @property
    def uulo(self) -> IVector4: ...
    @property
    def uull(self) -> IVector4: ...
    @property
    def uvuu(self) -> IVector4: ...
    @property
    def uvuv(self) -> IVector4: ...
    @property
    def uvuo(self) -> IVector4: ...
    @property
    def uvul(self) -> IVector4: ...
    @property
    def uvvu(self) -> IVector4: ...
    @property
    def uvvv(self) -> IVector4: ...
    @property
    def uvvo(self) -> IVector4: ...
    @property
    def uvvl(self) -> IVector4: ...
    @property
    def uvou(self) -> IVector4: ...
    @property
    def uvov(self) -> IVector4: ...
    @property
    def uvoo(self) -> IVector4: ...
    @property
    def uvol(self) -> IVector4: ...
    @property
    def uvlu(self) -> IVector4: ...
    @property
    def uvlv(self) -> IVector4: ...
    @property
    def uvlo(self) -> IVector4: ...
    @property
    def uvll(self) -> IVector4: ...
    @property
    def uouu(self) -> IVector4: ...
    @property
    def uouv(self) -> IVector4: ...
    @property
    def uouo(self) -> IVector4: ...
    @property
    def uoul(self) -> IVector4: ...
    @property
    def uovu(self) -> IVector4: ...
    @property
    def uovv(self) -> IVector4: ...
    @property
    def uovo(self) -> IVector4: ...
    @property
    def uovl(self) -> IVector4: ...
    @property
    def uoou(self) -> IVector4: ...
    @property
    def uoov(self) -> IVector4: ...
    @property
    def uooo(self) -> IVector4: ...
    @property
    def uool(self) -> IVector4: ...
    @property
    def uolu(self) -> IVector4: ...
    @property
    def uolv(self) -> IVector4: ...
    @property
    def uolo(self) -> IVector4: ...
    @property
    def uoll(self) -> IVector4: ...
    @property
    def uluu(self) -> IVector4: ...
    @property
    def uluv(self) -> IVector4: ...
    @property
    def uluo(self) -> IVector4: ...
    @property
    def ulul(self) -> IVector4: ...
    @property
    def ulvu(self) -> IVector4: ...
    @property
    def ulvv(self) -> IVector4: ...
    @property
    def ulvo(self) -> IVector4: ...
    @property
    def ulvl(self) -> IVector4: ...
    @property
    def ulou(self) -> IVector4: ...
    @property
    def ulov(self) -> IVector4: ...
    @property
    def uloo(self) -> IVector4: ...
    @property
    def ulol(self) -> IVector4: ...
    @property
    def ullu(self) -> IVector4: ...
    @property
    def ullv(self) -> IVector4: ...
    @property
    def ullo(self) -> IVector4: ...
    @property
    def ulll(self) -> IVector4: ...
    @property
    def vuuu(self) -> IVector4: ...
    @property
    def vuuv(self) -> IVector4: ...
    @property
    def vuuo(self) -> IVector4: ...
    @property
    def vuul(self) -> IVector4: ...
    @property
    def vuvu(self) -> IVector4: ...
    @property
    def vuvv(self) -> IVector4: ...
    @property
    def vuvo(self) -> IVector4: ...
    @property
    def vuvl(self) -> IVector4: ...
    @property
    def vuou(self) -> IVector4: ...
    @property
    def vuov(self) -> IVector4: ...
    @property
    def vuoo(self) -> IVector4: ...
    @property
    def vuol(self) -> IVector4: ...
    @property
    def vulu(self) -> IVector4: ...
    @property
    def vulv(self) -> IVector4: ...
    @property
    def vulo(self) -> IVector4: ...
    @property
    def vull(self) -> IVector4: ...
    @property
    def vvuu(self) -> IVector4: ...
    @property
    def vvuv(self) -> IVector4: ...
    @property
    def vvuo(self) -> IVector4: ...
    @property
    def vvul(self) -> IVector4: ...
    @property
    def vvvu(self) -> IVector4: ...
    @property
    def vvvv(self) -> IVector4: ...
    @property
    def vvvo(self) -> IVector4: ...
    @property
    def vvvl(self) -> IVector4: ...
    @property
    def vvou(self) -> IVector4: ...
    @property
    def vvov(self) -> IVector4: ...
    @property
    def vvoo(self) -> IVector4: ...
    @property
    def vvol(self) -> IVector4: ...
    @property
    def vvlu(self) -> IVector4: ...
    @property
    def vvlv(self) -> IVector4: ...
    @property
    def vvlo(self) -> IVector4: ...
    @property
    def vvll(self) -> IVector4: ...
    @property
    def vouu(self) -> IVector4: ...
    @property
    def vouv(self) -> IVector4: ...
    @property
    def vouo(self) -> IVector4: ...
    @property
    def voul(self) -> IVector4: ...
    @property
    def vovu(self) -> IVector4: ...
    @property
    def vovv(self) -> IVector4: ...
    @property
    def vovo(self) -> IVector4: ...
    @property
    def vovl(self) -> IVector4: ...
    @property
    def voou(self) -> IVector4: ...
    @property
    def voov(self) -> IVector4: ...
    @property
    def vooo(self) -> IVector4: ...
    @property
    def vool(self) -> IVector4: ...
    @property
    def volu(self) -> IVector4: ...
    @property
    def volv(self) -> IVector4: ...
    @property
    def volo(self) -> IVector4: ...
    @property
    def voll(self) -> IVector4: ...
    @property
    def vluu(self) -> IVector4: ...
    @property
    def vluv(self) -> IVector4: ...
    @property
    def vluo(self) -> IVector4: ...
    @property
    def vlul(self) -> IVector4: ...
    @property
    def vlvu(self) -> IVector4: ...
    @property
    def vlvv(self) -> IVector4: ...
    @property
    def vlvo(self) -> IVector4: ...
    @property
    def vlvl(self) -> IVector4: ...
    @property
    def vlou(self) -> IVector4: ...
    @property
    def vlov(self) -> IVector4: ...
    @property
    def vloo(self) -> IVector4: ...
    @property
    def vlol(self) -> IVector4: ...
    @property
    def vllu(self) -> IVector4: ...
    @property
    def vllv(self) -> IVector4: ...
    @property
    def vllo(self) -> IVector4: ...
    @property
    def vlll(self) -> IVector4: ...
    @property
    def ouuu(self) -> IVector4: ...
    @property
    def ouuv(self) -> IVector4: ...
    @property
    def ouuo(self) -> IVector4: ...
    @property
    def ouul(self) -> IVector4: ...
    @property
    def ouvu(self) -> IVector4: ...
    @property
    def ouvv(self) -> IVector4: ...
    @property
    def ouvo(self) -> IVector4: ...
    @property
    def ouvl(self) -> IVector4: ...
    @property
    def ouou(self) -> IVector4: ...
    @property
    def ouov(self) -> IVector4: ...
    @property
    def ouoo(self) -> IVector4: ...
    @property
    def ouol(self) -> IVector4: ...
    @property
    def oulu(self) -> IVector4: ...
    @property
    def oulv(self) -> IVector4: ...
    @property
    def oulo(self) -> IVector4: ...
    @property
    def oull(self) -> IVector4: ...
    @property
    def ovuu(self) -> IVector4: ...
    @property
    def ovuv(self) -> IVector4: ...
    @property
    def ovuo(self) -> IVector4: ...
    @property
    def ovul(self) -> IVector4: ...
    @property
    def ovvu(self) -> IVector4: ...
    @property
    def ovvv(self) -> IVector4: ...
    @property
    def ovvo(self) -> IVector4: ...
    @property
    def ovvl(self) -> IVector4: ...
    @property
    def ovou(self) -> IVector4: ...
    @property
    def ovov(self) -> IVector4: ...
    @property
    def ovoo(self) -> IVector4: ...
    @property
    def ovol(self) -> IVector4: ...
    @property
    def ovlu(self) -> IVector4: ...
    @property
    def ovlv(self) -> IVector4: ...
    @property
    def ovlo(self) -> IVector4: ...
    @property
    def ovll(self) -> IVector4: ...
    @property
    def oouu(self) -> IVector4: ...
    @property
    def oouv(self) -> IVector4: ...
    @property
    def oouo(self) -> IVector4: ...
    @property
    def ooul(self) -> IVector4: ...
    @property
    def oovu(self) -> IVector4: ...
    @property
    def oovv(self) -> IVector4: ...
    @property
    def oovo(self) -> IVector4: ...
    @property
    def oovl(self) -> IVector4: ...
    @property
    def ooou(self) -> IVector4: ...
    @property
    def ooov(self) -> IVector4: ...
    @property
    def oolu(self) -> IVector4: ...
    @property
    def oolv(self) -> IVector4: ...
    @property
    def oluu(self) -> IVector4: ...
    @property
    def oluv(self) -> IVector4: ...
    @property
    def oluo(self) -> IVector4: ...
    @property
    def olul(self) -> IVector4: ...
    @property
    def olvu(self) -> IVector4: ...
    @property
    def olvv(self) -> IVector4: ...
    @property
    def olvo(self) -> IVector4: ...
    @property
    def olvl(self) -> IVector4: ...
    @property
    def olou(self) -> IVector4: ...
    @property
    def olov(self) -> IVector4: ...
    @property
    def ollu(self) -> IVector4: ...
    @property
    def ollv(self) -> IVector4: ...
    @property
    def luuu(self) -> IVector4: ...
    @property
    def luuv(self) -> IVector4: ...
    @property
    def luuo(self) -> IVector4: ...
    @property
    def luul(self) -> IVector4: ...
    @property
    def luvu(self) -> IVector4: ...
    @property
    def luvv(self) -> IVector4: ...
    @property
    def luvo(self) -> IVector4: ...
    @property
    def luvl(self) -> IVector4: ...
    @property
    def luou(self) -> IVector4: ...
    @property
    def luov(self) -> IVector4: ...
    @property
    def luoo(self) -> IVector4: ...
    @property
    def luol(self) -> IVector4: ...
    @property
    def lulu(self) -> IVector4: ...
    @property
    def lulv(self) -> IVector4: ...
    @property
    def lulo(self) -> IVector4: ...
    @property
    def lull(self) -> IVector4: ...
    @property
    def lvuu(self) -> IVector4: ...
    @property
    def lvuv(self) -> IVector4: ...
    @property
    def lvuo(self) -> IVector4: ...
    @property
    def lvul(self) -> IVector4: ...
    @property
    def lvvu(self) -> IVector4: ...
    @property
    def lvvv(self) -> IVector4: ...
    @property
    def lvvo(self) -> IVector4: ...
    @property
    def lvvl(self) -> IVector4: ...
    @property
    def lvou(self) -> IVector4: ...
    @property
    def lvov(self) -> IVector4: ...
    @property
    def lvoo(self) -> IVector4: ...
    @property
    def lvol(self) -> IVector4: ...
    @property
    def lvlu(self) -> IVector4: ...
    @property
    def lvlv(self) -> IVector4: ...
    @property
    def lvlo(self) -> IVector4: ...
    @property
    def lvll(self) -> IVector4: ...
    @property
    def louu(self) -> IVector4: ...
    @property
    def louv(self) -> IVector4: ...
    @property
    def louo(self) -> IVector4: ...
    @property
    def loul(self) -> IVector4: ...
    @property
    def lovu(self) -> IVector4: ...
    @property
    def lovv(self) -> IVector4: ...
    @property
    def lovo(self) -> IVector4: ...
    @property
    def lovl(self) -> IVector4: ...
    @property
    def loou(self) -> IVector4: ...
    @property
    def loov(self) -> IVector4: ...
    @property
    def lolu(self) -> IVector4: ...
    @property
    def lolv(self) -> IVector4: ...
    @property
    def lluu(self) -> IVector4: ...
    @property
    def lluv(self) -> IVector4: ...
    @property
    def lluo(self) -> IVector4: ...
    @property
    def llul(self) -> IVector4: ...
    @property
    def llvu(self) -> IVector4: ...
    @property
    def llvv(self) -> IVector4: ...
    @property
    def llvo(self) -> IVector4: ...
    @property
    def llvl(self) -> IVector4: ...
    @property
    def llou(self) -> IVector4: ...
    @property
    def llov(self) -> IVector4: ...
    @property
    def lllu(self) -> IVector4: ...
    @property
    def lllv(self) -> IVector4: ...
    @property
    def oo(self) -> IVector2: ...
    @property
    def ol(self) -> IVector2: ...
    @property
    def lo(self) -> IVector2: ...
    @property
    def ll(self) -> IVector2: ...
    @property
    def ooo(self) -> IVector3: ...
    @property
    def ool(self) -> IVector3: ...
    @property
    def olo(self) -> IVector3: ...
    @property
    def oll(self) -> IVector3: ...
    @property
    def loo(self) -> IVector3: ...
    @property
    def lol(self) -> IVector3: ...
    @property
    def llo(self) -> IVector3: ...
    @property
    def lll(self) -> IVector3: ...
    @property
    def oooo(self) -> IVector4: ...
    @property
    def oool(self) -> IVector4: ...
    @property
    def oolo(self) -> IVector4: ...
    @property
    def ooll(self) -> IVector4: ...
    @property
    def oloo(self) -> IVector4: ...
    @property
    def olol(self) -> IVector4: ...
    @property
    def ollo(self) -> IVector4: ...
    @property
    def olll(self) -> IVector4: ...
    @property
    def looo(self) -> IVector4: ...
    @property
    def lool(self) -> IVector4: ...
    @property
    def lolo(self) -> IVector4: ...
    @property
    def loll(self) -> IVector4: ...
    @property
    def lloo(self) -> IVector4: ...
    @property
    def llol(self) -> IVector4: ...
    @property
    def lllo(self) -> IVector4: ...
    @property
    def llll(self) -> IVector4: ...
    def min(self, n: Number, /) -> IVector2: ...
    def max(self, n: Number, /) -> IVector2: ...
    def clamp(self, min: Number, max: Number, /) -> IVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[IVector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector2: ...

@final
class IVector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: IVector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[IVector2]: ...
    @overload
    def __getitem__(self, index: int) -> IVector2: ...
    @overload
    def __getitem__(self, index: slice) -> IVector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[IVector2]: ...

@final
class UVector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: UVector2) -> UVector2: ...
    @overload
    def __add__(self, other: Number) -> UVector2: ...
    @overload
    def __radd__(self, other: UVector2) -> UVector2: ...
    @overload
    def __radd__(self, other: Number) -> UVector2: ...
    @overload
    def __sub__(self, other: UVector2) -> UVector2: ...
    @overload
    def __sub__(self, other: Number) -> UVector2: ...
    @overload
    def __rsub__(self, other: UVector2) -> UVector2: ...
    @overload
    def __rsub__(self, other: Number) -> UVector2: ...
    @overload
    def __mul__(self, other: UVector2) -> UVector2: ...
    @overload
    def __mul__(self, other: Number) -> UVector2: ...
    @overload
    def __rmul__(self, other: UVector2) -> UVector2: ...
    @overload
    def __rmul__(self, other: Number) -> UVector2: ...
    @overload
    def __truediv__(self, other: UVector2) -> UVector2: ...
    @overload
    def __truediv__(self, other: Number) -> UVector2: ...
    @overload
    def __rtruediv__(self, other: UVector2) -> UVector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> UVector2: ...
    def __abs__(self) -> UVector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> UVector2: ...
    @property
    def xy(self) -> UVector2: ...
    @property
    def xo(self) -> UVector2: ...
    @property
    def xl(self) -> UVector2: ...
    @property
    def yx(self) -> UVector2: ...
    @property
    def yy(self) -> UVector2: ...
    @property
    def yo(self) -> UVector2: ...
    @property
    def yl(self) -> UVector2: ...
    @property
    def ox(self) -> UVector2: ...
    @property
    def oy(self) -> UVector2: ...
    @property
    def lx(self) -> UVector2: ...
    @property
    def ly(self) -> UVector2: ...
    @property
    def xxx(self) -> UVector3: ...
    @property
    def xxy(self) -> UVector3: ...
    @property
    def xxo(self) -> UVector3: ...
    @property
    def xxl(self) -> UVector3: ...
    @property
    def xyx(self) -> UVector3: ...
    @property
    def xyy(self) -> UVector3: ...
    @property
    def xyo(self) -> UVector3: ...
    @property
    def xyl(self) -> UVector3: ...
    @property
    def xox(self) -> UVector3: ...
    @property
    def xoy(self) -> UVector3: ...
    @property
    def xoo(self) -> UVector3: ...
    @property
    def xol(self) -> UVector3: ...
    @property
    def xlx(self) -> UVector3: ...
    @property
    def xly(self) -> UVector3: ...
    @property
    def xlo(self) -> UVector3: ...
    @property
    def xll(self) -> UVector3: ...
    @property
    def yxx(self) -> UVector3: ...
    @property
    def yxy(self) -> UVector3: ...
    @property
    def yxo(self) -> UVector3: ...
    @property
    def yxl(self) -> UVector3: ...
    @property
    def yyx(self) -> UVector3: ...
    @property
    def yyy(self) -> UVector3: ...
    @property
    def yyo(self) -> UVector3: ...
    @property
    def yyl(self) -> UVector3: ...
    @property
    def yox(self) -> UVector3: ...
    @property
    def yoy(self) -> UVector3: ...
    @property
    def yoo(self) -> UVector3: ...
    @property
    def yol(self) -> UVector3: ...
    @property
    def ylx(self) -> UVector3: ...
    @property
    def yly(self) -> UVector3: ...
    @property
    def ylo(self) -> UVector3: ...
    @property
    def yll(self) -> UVector3: ...
    @property
    def oxx(self) -> UVector3: ...
    @property
    def oxy(self) -> UVector3: ...
    @property
    def oxo(self) -> UVector3: ...
    @property
    def oxl(self) -> UVector3: ...
    @property
    def oyx(self) -> UVector3: ...
    @property
    def oyy(self) -> UVector3: ...
    @property
    def oyo(self) -> UVector3: ...
    @property
    def oyl(self) -> UVector3: ...
    @property
    def oox(self) -> UVector3: ...
    @property
    def ooy(self) -> UVector3: ...
    @property
    def olx(self) -> UVector3: ...
    @property
    def oly(self) -> UVector3: ...
    @property
    def lxx(self) -> UVector3: ...
    @property
    def lxy(self) -> UVector3: ...
    @property
    def lxo(self) -> UVector3: ...
    @property
    def lxl(self) -> UVector3: ...
    @property
    def lyx(self) -> UVector3: ...
    @property
    def lyy(self) -> UVector3: ...
    @property
    def lyo(self) -> UVector3: ...
    @property
    def lyl(self) -> UVector3: ...
    @property
    def lox(self) -> UVector3: ...
    @property
    def loy(self) -> UVector3: ...
    @property
    def llx(self) -> UVector3: ...
    @property
    def lly(self) -> UVector3: ...
    @property
    def xxxx(self) -> UVector4: ...
    @property
    def xxxy(self) -> UVector4: ...
    @property
    def xxxo(self) -> UVector4: ...
    @property
    def xxxl(self) -> UVector4: ...
    @property
    def xxyx(self) -> UVector4: ...
    @property
    def xxyy(self) -> UVector4: ...
    @property
    def xxyo(self) -> UVector4: ...
    @property
    def xxyl(self) -> UVector4: ...
    @property
    def xxox(self) -> UVector4: ...
    @property
    def xxoy(self) -> UVector4: ...
    @property
    def xxoo(self) -> UVector4: ...
    @property
    def xxol(self) -> UVector4: ...
    @property
    def xxlx(self) -> UVector4: ...
    @property
    def xxly(self) -> UVector4: ...
    @property
    def xxlo(self) -> UVector4: ...
    @property
    def xxll(self) -> UVector4: ...
    @property
    def xyxx(self) -> UVector4: ...
    @property
    def xyxy(self) -> UVector4: ...
    @property
    def xyxo(self) -> UVector4: ...
    @property
    def xyxl(self) -> UVector4: ...
    @property
    def xyyx(self) -> UVector4: ...
    @property
    def xyyy(self) -> UVector4: ...
    @property
    def xyyo(self) -> UVector4: ...
    @property
    def xyyl(self) -> UVector4: ...
    @property
    def xyox(self) -> UVector4: ...
    @property
    def xyoy(self) -> UVector4: ...
    @property
    def xyoo(self) -> UVector4: ...
    @property
    def xyol(self) -> UVector4: ...
    @property
    def xylx(self) -> UVector4: ...
    @property
    def xyly(self) -> UVector4: ...
    @property
    def xylo(self) -> UVector4: ...
    @property
    def xyll(self) -> UVector4: ...
    @property
    def xoxx(self) -> UVector4: ...
    @property
    def xoxy(self) -> UVector4: ...
    @property
    def xoxo(self) -> UVector4: ...
    @property
    def xoxl(self) -> UVector4: ...
    @property
    def xoyx(self) -> UVector4: ...
    @property
    def xoyy(self) -> UVector4: ...
    @property
    def xoyo(self) -> UVector4: ...
    @property
    def xoyl(self) -> UVector4: ...
    @property
    def xoox(self) -> UVector4: ...
    @property
    def xooy(self) -> UVector4: ...
    @property
    def xooo(self) -> UVector4: ...
    @property
    def xool(self) -> UVector4: ...
    @property
    def xolx(self) -> UVector4: ...
    @property
    def xoly(self) -> UVector4: ...
    @property
    def xolo(self) -> UVector4: ...
    @property
    def xoll(self) -> UVector4: ...
    @property
    def xlxx(self) -> UVector4: ...
    @property
    def xlxy(self) -> UVector4: ...
    @property
    def xlxo(self) -> UVector4: ...
    @property
    def xlxl(self) -> UVector4: ...
    @property
    def xlyx(self) -> UVector4: ...
    @property
    def xlyy(self) -> UVector4: ...
    @property
    def xlyo(self) -> UVector4: ...
    @property
    def xlyl(self) -> UVector4: ...
    @property
    def xlox(self) -> UVector4: ...
    @property
    def xloy(self) -> UVector4: ...
    @property
    def xloo(self) -> UVector4: ...
    @property
    def xlol(self) -> UVector4: ...
    @property
    def xllx(self) -> UVector4: ...
    @property
    def xlly(self) -> UVector4: ...
    @property
    def xllo(self) -> UVector4: ...
    @property
    def xlll(self) -> UVector4: ...
    @property
    def yxxx(self) -> UVector4: ...
    @property
    def yxxy(self) -> UVector4: ...
    @property
    def yxxo(self) -> UVector4: ...
    @property
    def yxxl(self) -> UVector4: ...
    @property
    def yxyx(self) -> UVector4: ...
    @property
    def yxyy(self) -> UVector4: ...
    @property
    def yxyo(self) -> UVector4: ...
    @property
    def yxyl(self) -> UVector4: ...
    @property
    def yxox(self) -> UVector4: ...
    @property
    def yxoy(self) -> UVector4: ...
    @property
    def yxoo(self) -> UVector4: ...
    @property
    def yxol(self) -> UVector4: ...
    @property
    def yxlx(self) -> UVector4: ...
    @property
    def yxly(self) -> UVector4: ...
    @property
    def yxlo(self) -> UVector4: ...
    @property
    def yxll(self) -> UVector4: ...
    @property
    def yyxx(self) -> UVector4: ...
    @property
    def yyxy(self) -> UVector4: ...
    @property
    def yyxo(self) -> UVector4: ...
    @property
    def yyxl(self) -> UVector4: ...
    @property
    def yyyx(self) -> UVector4: ...
    @property
    def yyyy(self) -> UVector4: ...
    @property
    def yyyo(self) -> UVector4: ...
    @property
    def yyyl(self) -> UVector4: ...
    @property
    def yyox(self) -> UVector4: ...
    @property
    def yyoy(self) -> UVector4: ...
    @property
    def yyoo(self) -> UVector4: ...
    @property
    def yyol(self) -> UVector4: ...
    @property
    def yylx(self) -> UVector4: ...
    @property
    def yyly(self) -> UVector4: ...
    @property
    def yylo(self) -> UVector4: ...
    @property
    def yyll(self) -> UVector4: ...
    @property
    def yoxx(self) -> UVector4: ...
    @property
    def yoxy(self) -> UVector4: ...
    @property
    def yoxo(self) -> UVector4: ...
    @property
    def yoxl(self) -> UVector4: ...
    @property
    def yoyx(self) -> UVector4: ...
    @property
    def yoyy(self) -> UVector4: ...
    @property
    def yoyo(self) -> UVector4: ...
    @property
    def yoyl(self) -> UVector4: ...
    @property
    def yoox(self) -> UVector4: ...
    @property
    def yooy(self) -> UVector4: ...
    @property
    def yooo(self) -> UVector4: ...
    @property
    def yool(self) -> UVector4: ...
    @property
    def yolx(self) -> UVector4: ...
    @property
    def yoly(self) -> UVector4: ...
    @property
    def yolo(self) -> UVector4: ...
    @property
    def yoll(self) -> UVector4: ...
    @property
    def ylxx(self) -> UVector4: ...
    @property
    def ylxy(self) -> UVector4: ...
    @property
    def ylxo(self) -> UVector4: ...
    @property
    def ylxl(self) -> UVector4: ...
    @property
    def ylyx(self) -> UVector4: ...
    @property
    def ylyy(self) -> UVector4: ...
    @property
    def ylyo(self) -> UVector4: ...
    @property
    def ylyl(self) -> UVector4: ...
    @property
    def ylox(self) -> UVector4: ...
    @property
    def yloy(self) -> UVector4: ...
    @property
    def yloo(self) -> UVector4: ...
    @property
    def ylol(self) -> UVector4: ...
    @property
    def yllx(self) -> UVector4: ...
    @property
    def ylly(self) -> UVector4: ...
    @property
    def yllo(self) -> UVector4: ...
    @property
    def ylll(self) -> UVector4: ...
    @property
    def oxxx(self) -> UVector4: ...
    @property
    def oxxy(self) -> UVector4: ...
    @property
    def oxxo(self) -> UVector4: ...
    @property
    def oxxl(self) -> UVector4: ...
    @property
    def oxyx(self) -> UVector4: ...
    @property
    def oxyy(self) -> UVector4: ...
    @property
    def oxyo(self) -> UVector4: ...
    @property
    def oxyl(self) -> UVector4: ...
    @property
    def oxox(self) -> UVector4: ...
    @property
    def oxoy(self) -> UVector4: ...
    @property
    def oxoo(self) -> UVector4: ...
    @property
    def oxol(self) -> UVector4: ...
    @property
    def oxlx(self) -> UVector4: ...
    @property
    def oxly(self) -> UVector4: ...
    @property
    def oxlo(self) -> UVector4: ...
    @property
    def oxll(self) -> UVector4: ...
    @property
    def oyxx(self) -> UVector4: ...
    @property
    def oyxy(self) -> UVector4: ...
    @property
    def oyxo(self) -> UVector4: ...
    @property
    def oyxl(self) -> UVector4: ...
    @property
    def oyyx(self) -> UVector4: ...
    @property
    def oyyy(self) -> UVector4: ...
    @property
    def oyyo(self) -> UVector4: ...
    @property
    def oyyl(self) -> UVector4: ...
    @property
    def oyox(self) -> UVector4: ...
    @property
    def oyoy(self) -> UVector4: ...
    @property
    def oyoo(self) -> UVector4: ...
    @property
    def oyol(self) -> UVector4: ...
    @property
    def oylx(self) -> UVector4: ...
    @property
    def oyly(self) -> UVector4: ...
    @property
    def oylo(self) -> UVector4: ...
    @property
    def oyll(self) -> UVector4: ...
    @property
    def ooxx(self) -> UVector4: ...
    @property
    def ooxy(self) -> UVector4: ...
    @property
    def ooxo(self) -> UVector4: ...
    @property
    def ooxl(self) -> UVector4: ...
    @property
    def ooyx(self) -> UVector4: ...
    @property
    def ooyy(self) -> UVector4: ...
    @property
    def ooyo(self) -> UVector4: ...
    @property
    def ooyl(self) -> UVector4: ...
    @property
    def ooox(self) -> UVector4: ...
    @property
    def oooy(self) -> UVector4: ...
    @property
    def oolx(self) -> UVector4: ...
    @property
    def ooly(self) -> UVector4: ...
    @property
    def olxx(self) -> UVector4: ...
    @property
    def olxy(self) -> UVector4: ...
    @property
    def olxo(self) -> UVector4: ...
    @property
    def olxl(self) -> UVector4: ...
    @property
    def olyx(self) -> UVector4: ...
    @property
    def olyy(self) -> UVector4: ...
    @property
    def olyo(self) -> UVector4: ...
    @property
    def olyl(self) -> UVector4: ...
    @property
    def olox(self) -> UVector4: ...
    @property
    def oloy(self) -> UVector4: ...
    @property
    def ollx(self) -> UVector4: ...
    @property
    def olly(self) -> UVector4: ...
    @property
    def lxxx(self) -> UVector4: ...
    @property
    def lxxy(self) -> UVector4: ...
    @property
    def lxxo(self) -> UVector4: ...
    @property
    def lxxl(self) -> UVector4: ...
    @property
    def lxyx(self) -> UVector4: ...
    @property
    def lxyy(self) -> UVector4: ...
    @property
    def lxyo(self) -> UVector4: ...
    @property
    def lxyl(self) -> UVector4: ...
    @property
    def lxox(self) -> UVector4: ...
    @property
    def lxoy(self) -> UVector4: ...
    @property
    def lxoo(self) -> UVector4: ...
    @property
    def lxol(self) -> UVector4: ...
    @property
    def lxlx(self) -> UVector4: ...
    @property
    def lxly(self) -> UVector4: ...
    @property
    def lxlo(self) -> UVector4: ...
    @property
    def lxll(self) -> UVector4: ...
    @property
    def lyxx(self) -> UVector4: ...
    @property
    def lyxy(self) -> UVector4: ...
    @property
    def lyxo(self) -> UVector4: ...
    @property
    def lyxl(self) -> UVector4: ...
    @property
    def lyyx(self) -> UVector4: ...
    @property
    def lyyy(self) -> UVector4: ...
    @property
    def lyyo(self) -> UVector4: ...
    @property
    def lyyl(self) -> UVector4: ...
    @property
    def lyox(self) -> UVector4: ...
    @property
    def lyoy(self) -> UVector4: ...
    @property
    def lyoo(self) -> UVector4: ...
    @property
    def lyol(self) -> UVector4: ...
    @property
    def lylx(self) -> UVector4: ...
    @property
    def lyly(self) -> UVector4: ...
    @property
    def lylo(self) -> UVector4: ...
    @property
    def lyll(self) -> UVector4: ...
    @property
    def loxx(self) -> UVector4: ...
    @property
    def loxy(self) -> UVector4: ...
    @property
    def loxo(self) -> UVector4: ...
    @property
    def loxl(self) -> UVector4: ...
    @property
    def loyx(self) -> UVector4: ...
    @property
    def loyy(self) -> UVector4: ...
    @property
    def loyo(self) -> UVector4: ...
    @property
    def loyl(self) -> UVector4: ...
    @property
    def loox(self) -> UVector4: ...
    @property
    def looy(self) -> UVector4: ...
    @property
    def lolx(self) -> UVector4: ...
    @property
    def loly(self) -> UVector4: ...
    @property
    def llxx(self) -> UVector4: ...
    @property
    def llxy(self) -> UVector4: ...
    @property
    def llxo(self) -> UVector4: ...
    @property
    def llxl(self) -> UVector4: ...
    @property
    def llyx(self) -> UVector4: ...
    @property
    def llyy(self) -> UVector4: ...
    @property
    def llyo(self) -> UVector4: ...
    @property
    def llyl(self) -> UVector4: ...
    @property
    def llox(self) -> UVector4: ...
    @property
    def lloy(self) -> UVector4: ...
    @property
    def lllx(self) -> UVector4: ...
    @property
    def llly(self) -> UVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> UVector2: ...
    @property
    def rg(self) -> UVector2: ...
    @property
    def ro(self) -> UVector2: ...
    @property
    def rl(self) -> UVector2: ...
    @property
    def gr(self) -> UVector2: ...
    @property
    def gg(self) -> UVector2: ...
    @property
    def go(self) -> UVector2: ...
    @property
    def gl(self) -> UVector2: ...
    @property
    def og(self) -> UVector2: ...
    @property
    def lr(self) -> UVector2: ...
    @property
    def lg(self) -> UVector2: ...
    @property
    def rrr(self) -> UVector3: ...
    @property
    def rrg(self) -> UVector3: ...
    @property
    def rro(self) -> UVector3: ...
    @property
    def rrl(self) -> UVector3: ...
    @property
    def rgr(self) -> UVector3: ...
    @property
    def rgg(self) -> UVector3: ...
    @property
    def rgo(self) -> UVector3: ...
    @property
    def rgl(self) -> UVector3: ...
    @property
    def ror(self) -> UVector3: ...
    @property
    def rog(self) -> UVector3: ...
    @property
    def roo(self) -> UVector3: ...
    @property
    def rol(self) -> UVector3: ...
    @property
    def rlr(self) -> UVector3: ...
    @property
    def rlg(self) -> UVector3: ...
    @property
    def rlo(self) -> UVector3: ...
    @property
    def rll(self) -> UVector3: ...
    @property
    def grr(self) -> UVector3: ...
    @property
    def grg(self) -> UVector3: ...
    @property
    def gro(self) -> UVector3: ...
    @property
    def grl(self) -> UVector3: ...
    @property
    def ggr(self) -> UVector3: ...
    @property
    def ggg(self) -> UVector3: ...
    @property
    def ggo(self) -> UVector3: ...
    @property
    def ggl(self) -> UVector3: ...
    @property
    def gor(self) -> UVector3: ...
    @property
    def gog(self) -> UVector3: ...
    @property
    def goo(self) -> UVector3: ...
    @property
    def gol(self) -> UVector3: ...
    @property
    def glr(self) -> UVector3: ...
    @property
    def glg(self) -> UVector3: ...
    @property
    def glo(self) -> UVector3: ...
    @property
    def gll(self) -> UVector3: ...
    @property
    def orr(self) -> UVector3: ...
    @property
    def org(self) -> UVector3: ...
    @property
    def oro(self) -> UVector3: ...
    @property
    def orl(self) -> UVector3: ...
    @property
    def ogr(self) -> UVector3: ...
    @property
    def ogg(self) -> UVector3: ...
    @property
    def ogo(self) -> UVector3: ...
    @property
    def ogl(self) -> UVector3: ...
    @property
    def oor(self) -> UVector3: ...
    @property
    def oog(self) -> UVector3: ...
    @property
    def olr(self) -> UVector3: ...
    @property
    def olg(self) -> UVector3: ...
    @property
    def lrr(self) -> UVector3: ...
    @property
    def lrg(self) -> UVector3: ...
    @property
    def lro(self) -> UVector3: ...
    @property
    def lrl(self) -> UVector3: ...
    @property
    def lgr(self) -> UVector3: ...
    @property
    def lgg(self) -> UVector3: ...
    @property
    def lgo(self) -> UVector3: ...
    @property
    def lgl(self) -> UVector3: ...
    @property
    def lor(self) -> UVector3: ...
    @property
    def log(self) -> UVector3: ...
    @property
    def llr(self) -> UVector3: ...
    @property
    def llg(self) -> UVector3: ...
    @property
    def rrrr(self) -> UVector4: ...
    @property
    def rrrg(self) -> UVector4: ...
    @property
    def rrro(self) -> UVector4: ...
    @property
    def rrrl(self) -> UVector4: ...
    @property
    def rrgr(self) -> UVector4: ...
    @property
    def rrgg(self) -> UVector4: ...
    @property
    def rrgo(self) -> UVector4: ...
    @property
    def rrgl(self) -> UVector4: ...
    @property
    def rror(self) -> UVector4: ...
    @property
    def rrog(self) -> UVector4: ...
    @property
    def rroo(self) -> UVector4: ...
    @property
    def rrol(self) -> UVector4: ...
    @property
    def rrlr(self) -> UVector4: ...
    @property
    def rrlg(self) -> UVector4: ...
    @property
    def rrlo(self) -> UVector4: ...
    @property
    def rrll(self) -> UVector4: ...
    @property
    def rgrr(self) -> UVector4: ...
    @property
    def rgrg(self) -> UVector4: ...
    @property
    def rgro(self) -> UVector4: ...
    @property
    def rgrl(self) -> UVector4: ...
    @property
    def rggr(self) -> UVector4: ...
    @property
    def rggg(self) -> UVector4: ...
    @property
    def rggo(self) -> UVector4: ...
    @property
    def rggl(self) -> UVector4: ...
    @property
    def rgor(self) -> UVector4: ...
    @property
    def rgog(self) -> UVector4: ...
    @property
    def rgoo(self) -> UVector4: ...
    @property
    def rgol(self) -> UVector4: ...
    @property
    def rglr(self) -> UVector4: ...
    @property
    def rglg(self) -> UVector4: ...
    @property
    def rglo(self) -> UVector4: ...
    @property
    def rgll(self) -> UVector4: ...
    @property
    def rorr(self) -> UVector4: ...
    @property
    def rorg(self) -> UVector4: ...
    @property
    def roro(self) -> UVector4: ...
    @property
    def rorl(self) -> UVector4: ...
    @property
    def rogr(self) -> UVector4: ...
    @property
    def rogg(self) -> UVector4: ...
    @property
    def rogo(self) -> UVector4: ...
    @property
    def rogl(self) -> UVector4: ...
    @property
    def roor(self) -> UVector4: ...
    @property
    def roog(self) -> UVector4: ...
    @property
    def rooo(self) -> UVector4: ...
    @property
    def rool(self) -> UVector4: ...
    @property
    def rolr(self) -> UVector4: ...
    @property
    def rolg(self) -> UVector4: ...
    @property
    def rolo(self) -> UVector4: ...
    @property
    def roll(self) -> UVector4: ...
    @property
    def rlrr(self) -> UVector4: ...
    @property
    def rlrg(self) -> UVector4: ...
    @property
    def rlro(self) -> UVector4: ...
    @property
    def rlrl(self) -> UVector4: ...
    @property
    def rlgr(self) -> UVector4: ...
    @property
    def rlgg(self) -> UVector4: ...
    @property
    def rlgo(self) -> UVector4: ...
    @property
    def rlgl(self) -> UVector4: ...
    @property
    def rlor(self) -> UVector4: ...
    @property
    def rlog(self) -> UVector4: ...
    @property
    def rloo(self) -> UVector4: ...
    @property
    def rlol(self) -> UVector4: ...
    @property
    def rllr(self) -> UVector4: ...
    @property
    def rllg(self) -> UVector4: ...
    @property
    def rllo(self) -> UVector4: ...
    @property
    def rlll(self) -> UVector4: ...
    @property
    def grrr(self) -> UVector4: ...
    @property
    def grrg(self) -> UVector4: ...
    @property
    def grro(self) -> UVector4: ...
    @property
    def grrl(self) -> UVector4: ...
    @property
    def grgr(self) -> UVector4: ...
    @property
    def grgg(self) -> UVector4: ...
    @property
    def grgo(self) -> UVector4: ...
    @property
    def grgl(self) -> UVector4: ...
    @property
    def gror(self) -> UVector4: ...
    @property
    def grog(self) -> UVector4: ...
    @property
    def groo(self) -> UVector4: ...
    @property
    def grol(self) -> UVector4: ...
    @property
    def grlr(self) -> UVector4: ...
    @property
    def grlg(self) -> UVector4: ...
    @property
    def grlo(self) -> UVector4: ...
    @property
    def grll(self) -> UVector4: ...
    @property
    def ggrr(self) -> UVector4: ...
    @property
    def ggrg(self) -> UVector4: ...
    @property
    def ggro(self) -> UVector4: ...
    @property
    def ggrl(self) -> UVector4: ...
    @property
    def gggr(self) -> UVector4: ...
    @property
    def gggg(self) -> UVector4: ...
    @property
    def gggo(self) -> UVector4: ...
    @property
    def gggl(self) -> UVector4: ...
    @property
    def ggor(self) -> UVector4: ...
    @property
    def ggog(self) -> UVector4: ...
    @property
    def ggoo(self) -> UVector4: ...
    @property
    def ggol(self) -> UVector4: ...
    @property
    def gglr(self) -> UVector4: ...
    @property
    def gglg(self) -> UVector4: ...
    @property
    def gglo(self) -> UVector4: ...
    @property
    def ggll(self) -> UVector4: ...
    @property
    def gorr(self) -> UVector4: ...
    @property
    def gorg(self) -> UVector4: ...
    @property
    def goro(self) -> UVector4: ...
    @property
    def gorl(self) -> UVector4: ...
    @property
    def gogr(self) -> UVector4: ...
    @property
    def gogg(self) -> UVector4: ...
    @property
    def gogo(self) -> UVector4: ...
    @property
    def gogl(self) -> UVector4: ...
    @property
    def goor(self) -> UVector4: ...
    @property
    def goog(self) -> UVector4: ...
    @property
    def gooo(self) -> UVector4: ...
    @property
    def gool(self) -> UVector4: ...
    @property
    def golr(self) -> UVector4: ...
    @property
    def golg(self) -> UVector4: ...
    @property
    def golo(self) -> UVector4: ...
    @property
    def goll(self) -> UVector4: ...
    @property
    def glrr(self) -> UVector4: ...
    @property
    def glrg(self) -> UVector4: ...
    @property
    def glro(self) -> UVector4: ...
    @property
    def glrl(self) -> UVector4: ...
    @property
    def glgr(self) -> UVector4: ...
    @property
    def glgg(self) -> UVector4: ...
    @property
    def glgo(self) -> UVector4: ...
    @property
    def glgl(self) -> UVector4: ...
    @property
    def glor(self) -> UVector4: ...
    @property
    def glog(self) -> UVector4: ...
    @property
    def gloo(self) -> UVector4: ...
    @property
    def glol(self) -> UVector4: ...
    @property
    def gllr(self) -> UVector4: ...
    @property
    def gllg(self) -> UVector4: ...
    @property
    def gllo(self) -> UVector4: ...
    @property
    def glll(self) -> UVector4: ...
    @property
    def orrr(self) -> UVector4: ...
    @property
    def orrg(self) -> UVector4: ...
    @property
    def orro(self) -> UVector4: ...
    @property
    def orrl(self) -> UVector4: ...
    @property
    def orgr(self) -> UVector4: ...
    @property
    def orgg(self) -> UVector4: ...
    @property
    def orgo(self) -> UVector4: ...
    @property
    def orgl(self) -> UVector4: ...
    @property
    def oror(self) -> UVector4: ...
    @property
    def orog(self) -> UVector4: ...
    @property
    def oroo(self) -> UVector4: ...
    @property
    def orol(self) -> UVector4: ...
    @property
    def orlr(self) -> UVector4: ...
    @property
    def orlg(self) -> UVector4: ...
    @property
    def orlo(self) -> UVector4: ...
    @property
    def orll(self) -> UVector4: ...
    @property
    def ogrr(self) -> UVector4: ...
    @property
    def ogrg(self) -> UVector4: ...
    @property
    def ogro(self) -> UVector4: ...
    @property
    def ogrl(self) -> UVector4: ...
    @property
    def oggr(self) -> UVector4: ...
    @property
    def oggg(self) -> UVector4: ...
    @property
    def oggo(self) -> UVector4: ...
    @property
    def oggl(self) -> UVector4: ...
    @property
    def ogor(self) -> UVector4: ...
    @property
    def ogog(self) -> UVector4: ...
    @property
    def ogoo(self) -> UVector4: ...
    @property
    def ogol(self) -> UVector4: ...
    @property
    def oglr(self) -> UVector4: ...
    @property
    def oglg(self) -> UVector4: ...
    @property
    def oglo(self) -> UVector4: ...
    @property
    def ogll(self) -> UVector4: ...
    @property
    def oorr(self) -> UVector4: ...
    @property
    def oorg(self) -> UVector4: ...
    @property
    def ooro(self) -> UVector4: ...
    @property
    def oorl(self) -> UVector4: ...
    @property
    def oogr(self) -> UVector4: ...
    @property
    def oogg(self) -> UVector4: ...
    @property
    def oogo(self) -> UVector4: ...
    @property
    def oogl(self) -> UVector4: ...
    @property
    def ooor(self) -> UVector4: ...
    @property
    def ooog(self) -> UVector4: ...
    @property
    def oolr(self) -> UVector4: ...
    @property
    def oolg(self) -> UVector4: ...
    @property
    def olrr(self) -> UVector4: ...
    @property
    def olrg(self) -> UVector4: ...
    @property
    def olro(self) -> UVector4: ...
    @property
    def olrl(self) -> UVector4: ...
    @property
    def olgr(self) -> UVector4: ...
    @property
    def olgg(self) -> UVector4: ...
    @property
    def olgo(self) -> UVector4: ...
    @property
    def olgl(self) -> UVector4: ...
    @property
    def olor(self) -> UVector4: ...
    @property
    def olog(self) -> UVector4: ...
    @property
    def ollr(self) -> UVector4: ...
    @property
    def ollg(self) -> UVector4: ...
    @property
    def lrrr(self) -> UVector4: ...
    @property
    def lrrg(self) -> UVector4: ...
    @property
    def lrro(self) -> UVector4: ...
    @property
    def lrrl(self) -> UVector4: ...
    @property
    def lrgr(self) -> UVector4: ...
    @property
    def lrgg(self) -> UVector4: ...
    @property
    def lrgo(self) -> UVector4: ...
    @property
    def lrgl(self) -> UVector4: ...
    @property
    def lror(self) -> UVector4: ...
    @property
    def lrog(self) -> UVector4: ...
    @property
    def lroo(self) -> UVector4: ...
    @property
    def lrol(self) -> UVector4: ...
    @property
    def lrlr(self) -> UVector4: ...
    @property
    def lrlg(self) -> UVector4: ...
    @property
    def lrlo(self) -> UVector4: ...
    @property
    def lrll(self) -> UVector4: ...
    @property
    def lgrr(self) -> UVector4: ...
    @property
    def lgrg(self) -> UVector4: ...
    @property
    def lgro(self) -> UVector4: ...
    @property
    def lgrl(self) -> UVector4: ...
    @property
    def lggr(self) -> UVector4: ...
    @property
    def lggg(self) -> UVector4: ...
    @property
    def lggo(self) -> UVector4: ...
    @property
    def lggl(self) -> UVector4: ...
    @property
    def lgor(self) -> UVector4: ...
    @property
    def lgog(self) -> UVector4: ...
    @property
    def lgoo(self) -> UVector4: ...
    @property
    def lgol(self) -> UVector4: ...
    @property
    def lglr(self) -> UVector4: ...
    @property
    def lglg(self) -> UVector4: ...
    @property
    def lglo(self) -> UVector4: ...
    @property
    def lgll(self) -> UVector4: ...
    @property
    def lorr(self) -> UVector4: ...
    @property
    def lorg(self) -> UVector4: ...
    @property
    def loro(self) -> UVector4: ...
    @property
    def lorl(self) -> UVector4: ...
    @property
    def logr(self) -> UVector4: ...
    @property
    def logg(self) -> UVector4: ...
    @property
    def logo(self) -> UVector4: ...
    @property
    def logl(self) -> UVector4: ...
    @property
    def loor(self) -> UVector4: ...
    @property
    def loog(self) -> UVector4: ...
    @property
    def lolr(self) -> UVector4: ...
    @property
    def lolg(self) -> UVector4: ...
    @property
    def llrr(self) -> UVector4: ...
    @property
    def llrg(self) -> UVector4: ...
    @property
    def llro(self) -> UVector4: ...
    @property
    def llrl(self) -> UVector4: ...
    @property
    def llgr(self) -> UVector4: ...
    @property
    def llgg(self) -> UVector4: ...
    @property
    def llgo(self) -> UVector4: ...
    @property
    def llgl(self) -> UVector4: ...
    @property
    def llor(self) -> UVector4: ...
    @property
    def llog(self) -> UVector4: ...
    @property
    def lllr(self) -> UVector4: ...
    @property
    def lllg(self) -> UVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> UVector2: ...
    @property
    def st(self) -> UVector2: ...
    @property
    def so(self) -> UVector2: ...
    @property
    def sl(self) -> UVector2: ...
    @property
    def ts(self) -> UVector2: ...
    @property
    def tt(self) -> UVector2: ...
    @property
    def to(self) -> UVector2: ...
    @property
    def tl(self) -> UVector2: ...
    @property
    def os(self) -> UVector2: ...
    @property
    def ot(self) -> UVector2: ...
    @property
    def ls(self) -> UVector2: ...
    @property
    def lt(self) -> UVector2: ...
    @property
    def sss(self) -> UVector3: ...
    @property
    def sst(self) -> UVector3: ...
    @property
    def sso(self) -> UVector3: ...
    @property
    def ssl(self) -> UVector3: ...
    @property
    def sts(self) -> UVector3: ...
    @property
    def stt(self) -> UVector3: ...
    @property
    def sto(self) -> UVector3: ...
    @property
    def stl(self) -> UVector3: ...
    @property
    def sos(self) -> UVector3: ...
    @property
    def sot(self) -> UVector3: ...
    @property
    def soo(self) -> UVector3: ...
    @property
    def sol(self) -> UVector3: ...
    @property
    def sls(self) -> UVector3: ...
    @property
    def slt(self) -> UVector3: ...
    @property
    def slo(self) -> UVector3: ...
    @property
    def sll(self) -> UVector3: ...
    @property
    def tss(self) -> UVector3: ...
    @property
    def tst(self) -> UVector3: ...
    @property
    def tso(self) -> UVector3: ...
    @property
    def tsl(self) -> UVector3: ...
    @property
    def tts(self) -> UVector3: ...
    @property
    def ttt(self) -> UVector3: ...
    @property
    def tto(self) -> UVector3: ...
    @property
    def ttl(self) -> UVector3: ...
    @property
    def tos(self) -> UVector3: ...
    @property
    def tot(self) -> UVector3: ...
    @property
    def too(self) -> UVector3: ...
    @property
    def tol(self) -> UVector3: ...
    @property
    def tls(self) -> UVector3: ...
    @property
    def tlt(self) -> UVector3: ...
    @property
    def tlo(self) -> UVector3: ...
    @property
    def tll(self) -> UVector3: ...
    @property
    def oss(self) -> UVector3: ...
    @property
    def ost(self) -> UVector3: ...
    @property
    def oso(self) -> UVector3: ...
    @property
    def osl(self) -> UVector3: ...
    @property
    def ots(self) -> UVector3: ...
    @property
    def ott(self) -> UVector3: ...
    @property
    def oto(self) -> UVector3: ...
    @property
    def otl(self) -> UVector3: ...
    @property
    def oos(self) -> UVector3: ...
    @property
    def oot(self) -> UVector3: ...
    @property
    def ols(self) -> UVector3: ...
    @property
    def olt(self) -> UVector3: ...
    @property
    def lss(self) -> UVector3: ...
    @property
    def lst(self) -> UVector3: ...
    @property
    def lso(self) -> UVector3: ...
    @property
    def lsl(self) -> UVector3: ...
    @property
    def lts(self) -> UVector3: ...
    @property
    def ltt(self) -> UVector3: ...
    @property
    def lto(self) -> UVector3: ...
    @property
    def ltl(self) -> UVector3: ...
    @property
    def los(self) -> UVector3: ...
    @property
    def lot(self) -> UVector3: ...
    @property
    def lls(self) -> UVector3: ...
    @property
    def llt(self) -> UVector3: ...
    @property
    def ssss(self) -> UVector4: ...
    @property
    def ssst(self) -> UVector4: ...
    @property
    def ssso(self) -> UVector4: ...
    @property
    def sssl(self) -> UVector4: ...
    @property
    def ssts(self) -> UVector4: ...
    @property
    def sstt(self) -> UVector4: ...
    @property
    def ssto(self) -> UVector4: ...
    @property
    def sstl(self) -> UVector4: ...
    @property
    def ssos(self) -> UVector4: ...
    @property
    def ssot(self) -> UVector4: ...
    @property
    def ssoo(self) -> UVector4: ...
    @property
    def ssol(self) -> UVector4: ...
    @property
    def ssls(self) -> UVector4: ...
    @property
    def sslt(self) -> UVector4: ...
    @property
    def sslo(self) -> UVector4: ...
    @property
    def ssll(self) -> UVector4: ...
    @property
    def stss(self) -> UVector4: ...
    @property
    def stst(self) -> UVector4: ...
    @property
    def stso(self) -> UVector4: ...
    @property
    def stsl(self) -> UVector4: ...
    @property
    def stts(self) -> UVector4: ...
    @property
    def sttt(self) -> UVector4: ...
    @property
    def stto(self) -> UVector4: ...
    @property
    def sttl(self) -> UVector4: ...
    @property
    def stos(self) -> UVector4: ...
    @property
    def stot(self) -> UVector4: ...
    @property
    def stoo(self) -> UVector4: ...
    @property
    def stol(self) -> UVector4: ...
    @property
    def stls(self) -> UVector4: ...
    @property
    def stlt(self) -> UVector4: ...
    @property
    def stlo(self) -> UVector4: ...
    @property
    def stll(self) -> UVector4: ...
    @property
    def soss(self) -> UVector4: ...
    @property
    def sost(self) -> UVector4: ...
    @property
    def soso(self) -> UVector4: ...
    @property
    def sosl(self) -> UVector4: ...
    @property
    def sots(self) -> UVector4: ...
    @property
    def sott(self) -> UVector4: ...
    @property
    def soto(self) -> UVector4: ...
    @property
    def sotl(self) -> UVector4: ...
    @property
    def soos(self) -> UVector4: ...
    @property
    def soot(self) -> UVector4: ...
    @property
    def sooo(self) -> UVector4: ...
    @property
    def sool(self) -> UVector4: ...
    @property
    def sols(self) -> UVector4: ...
    @property
    def solt(self) -> UVector4: ...
    @property
    def solo(self) -> UVector4: ...
    @property
    def soll(self) -> UVector4: ...
    @property
    def slss(self) -> UVector4: ...
    @property
    def slst(self) -> UVector4: ...
    @property
    def slso(self) -> UVector4: ...
    @property
    def slsl(self) -> UVector4: ...
    @property
    def slts(self) -> UVector4: ...
    @property
    def sltt(self) -> UVector4: ...
    @property
    def slto(self) -> UVector4: ...
    @property
    def sltl(self) -> UVector4: ...
    @property
    def slos(self) -> UVector4: ...
    @property
    def slot(self) -> UVector4: ...
    @property
    def sloo(self) -> UVector4: ...
    @property
    def slol(self) -> UVector4: ...
    @property
    def slls(self) -> UVector4: ...
    @property
    def sllt(self) -> UVector4: ...
    @property
    def sllo(self) -> UVector4: ...
    @property
    def slll(self) -> UVector4: ...
    @property
    def tsss(self) -> UVector4: ...
    @property
    def tsst(self) -> UVector4: ...
    @property
    def tsso(self) -> UVector4: ...
    @property
    def tssl(self) -> UVector4: ...
    @property
    def tsts(self) -> UVector4: ...
    @property
    def tstt(self) -> UVector4: ...
    @property
    def tsto(self) -> UVector4: ...
    @property
    def tstl(self) -> UVector4: ...
    @property
    def tsos(self) -> UVector4: ...
    @property
    def tsot(self) -> UVector4: ...
    @property
    def tsoo(self) -> UVector4: ...
    @property
    def tsol(self) -> UVector4: ...
    @property
    def tsls(self) -> UVector4: ...
    @property
    def tslt(self) -> UVector4: ...
    @property
    def tslo(self) -> UVector4: ...
    @property
    def tsll(self) -> UVector4: ...
    @property
    def ttss(self) -> UVector4: ...
    @property
    def ttst(self) -> UVector4: ...
    @property
    def ttso(self) -> UVector4: ...
    @property
    def ttsl(self) -> UVector4: ...
    @property
    def ttts(self) -> UVector4: ...
    @property
    def tttt(self) -> UVector4: ...
    @property
    def ttto(self) -> UVector4: ...
    @property
    def tttl(self) -> UVector4: ...
    @property
    def ttos(self) -> UVector4: ...
    @property
    def ttot(self) -> UVector4: ...
    @property
    def ttoo(self) -> UVector4: ...
    @property
    def ttol(self) -> UVector4: ...
    @property
    def ttls(self) -> UVector4: ...
    @property
    def ttlt(self) -> UVector4: ...
    @property
    def ttlo(self) -> UVector4: ...
    @property
    def ttll(self) -> UVector4: ...
    @property
    def toss(self) -> UVector4: ...
    @property
    def tost(self) -> UVector4: ...
    @property
    def toso(self) -> UVector4: ...
    @property
    def tosl(self) -> UVector4: ...
    @property
    def tots(self) -> UVector4: ...
    @property
    def tott(self) -> UVector4: ...
    @property
    def toto(self) -> UVector4: ...
    @property
    def totl(self) -> UVector4: ...
    @property
    def toos(self) -> UVector4: ...
    @property
    def toot(self) -> UVector4: ...
    @property
    def tooo(self) -> UVector4: ...
    @property
    def tool(self) -> UVector4: ...
    @property
    def tols(self) -> UVector4: ...
    @property
    def tolt(self) -> UVector4: ...
    @property
    def tolo(self) -> UVector4: ...
    @property
    def toll(self) -> UVector4: ...
    @property
    def tlss(self) -> UVector4: ...
    @property
    def tlst(self) -> UVector4: ...
    @property
    def tlso(self) -> UVector4: ...
    @property
    def tlsl(self) -> UVector4: ...
    @property
    def tlts(self) -> UVector4: ...
    @property
    def tltt(self) -> UVector4: ...
    @property
    def tlto(self) -> UVector4: ...
    @property
    def tltl(self) -> UVector4: ...
    @property
    def tlos(self) -> UVector4: ...
    @property
    def tlot(self) -> UVector4: ...
    @property
    def tloo(self) -> UVector4: ...
    @property
    def tlol(self) -> UVector4: ...
    @property
    def tlls(self) -> UVector4: ...
    @property
    def tllt(self) -> UVector4: ...
    @property
    def tllo(self) -> UVector4: ...
    @property
    def tlll(self) -> UVector4: ...
    @property
    def osss(self) -> UVector4: ...
    @property
    def osst(self) -> UVector4: ...
    @property
    def osso(self) -> UVector4: ...
    @property
    def ossl(self) -> UVector4: ...
    @property
    def osts(self) -> UVector4: ...
    @property
    def ostt(self) -> UVector4: ...
    @property
    def osto(self) -> UVector4: ...
    @property
    def ostl(self) -> UVector4: ...
    @property
    def osos(self) -> UVector4: ...
    @property
    def osot(self) -> UVector4: ...
    @property
    def osoo(self) -> UVector4: ...
    @property
    def osol(self) -> UVector4: ...
    @property
    def osls(self) -> UVector4: ...
    @property
    def oslt(self) -> UVector4: ...
    @property
    def oslo(self) -> UVector4: ...
    @property
    def osll(self) -> UVector4: ...
    @property
    def otss(self) -> UVector4: ...
    @property
    def otst(self) -> UVector4: ...
    @property
    def otso(self) -> UVector4: ...
    @property
    def otsl(self) -> UVector4: ...
    @property
    def otts(self) -> UVector4: ...
    @property
    def ottt(self) -> UVector4: ...
    @property
    def otto(self) -> UVector4: ...
    @property
    def ottl(self) -> UVector4: ...
    @property
    def otos(self) -> UVector4: ...
    @property
    def otot(self) -> UVector4: ...
    @property
    def otoo(self) -> UVector4: ...
    @property
    def otol(self) -> UVector4: ...
    @property
    def otls(self) -> UVector4: ...
    @property
    def otlt(self) -> UVector4: ...
    @property
    def otlo(self) -> UVector4: ...
    @property
    def otll(self) -> UVector4: ...
    @property
    def ooss(self) -> UVector4: ...
    @property
    def oost(self) -> UVector4: ...
    @property
    def ooso(self) -> UVector4: ...
    @property
    def oosl(self) -> UVector4: ...
    @property
    def oots(self) -> UVector4: ...
    @property
    def oott(self) -> UVector4: ...
    @property
    def ooto(self) -> UVector4: ...
    @property
    def ootl(self) -> UVector4: ...
    @property
    def ooos(self) -> UVector4: ...
    @property
    def ooot(self) -> UVector4: ...
    @property
    def ools(self) -> UVector4: ...
    @property
    def oolt(self) -> UVector4: ...
    @property
    def olss(self) -> UVector4: ...
    @property
    def olst(self) -> UVector4: ...
    @property
    def olso(self) -> UVector4: ...
    @property
    def olsl(self) -> UVector4: ...
    @property
    def olts(self) -> UVector4: ...
    @property
    def oltt(self) -> UVector4: ...
    @property
    def olto(self) -> UVector4: ...
    @property
    def oltl(self) -> UVector4: ...
    @property
    def olos(self) -> UVector4: ...
    @property
    def olot(self) -> UVector4: ...
    @property
    def olls(self) -> UVector4: ...
    @property
    def ollt(self) -> UVector4: ...
    @property
    def lsss(self) -> UVector4: ...
    @property
    def lsst(self) -> UVector4: ...
    @property
    def lsso(self) -> UVector4: ...
    @property
    def lssl(self) -> UVector4: ...
    @property
    def lsts(self) -> UVector4: ...
    @property
    def lstt(self) -> UVector4: ...
    @property
    def lsto(self) -> UVector4: ...
    @property
    def lstl(self) -> UVector4: ...
    @property
    def lsos(self) -> UVector4: ...
    @property
    def lsot(self) -> UVector4: ...
    @property
    def lsoo(self) -> UVector4: ...
    @property
    def lsol(self) -> UVector4: ...
    @property
    def lsls(self) -> UVector4: ...
    @property
    def lslt(self) -> UVector4: ...
    @property
    def lslo(self) -> UVector4: ...
    @property
    def lsll(self) -> UVector4: ...
    @property
    def ltss(self) -> UVector4: ...
    @property
    def ltst(self) -> UVector4: ...
    @property
    def ltso(self) -> UVector4: ...
    @property
    def ltsl(self) -> UVector4: ...
    @property
    def ltts(self) -> UVector4: ...
    @property
    def lttt(self) -> UVector4: ...
    @property
    def ltto(self) -> UVector4: ...
    @property
    def lttl(self) -> UVector4: ...
    @property
    def ltos(self) -> UVector4: ...
    @property
    def ltot(self) -> UVector4: ...
    @property
    def ltoo(self) -> UVector4: ...
    @property
    def ltol(self) -> UVector4: ...
    @property
    def ltls(self) -> UVector4: ...
    @property
    def ltlt(self) -> UVector4: ...
    @property
    def ltlo(self) -> UVector4: ...
    @property
    def ltll(self) -> UVector4: ...
    @property
    def loss(self) -> UVector4: ...
    @property
    def lost(self) -> UVector4: ...
    @property
    def loso(self) -> UVector4: ...
    @property
    def losl(self) -> UVector4: ...
    @property
    def lots(self) -> UVector4: ...
    @property
    def lott(self) -> UVector4: ...
    @property
    def loto(self) -> UVector4: ...
    @property
    def lotl(self) -> UVector4: ...
    @property
    def loos(self) -> UVector4: ...
    @property
    def loot(self) -> UVector4: ...
    @property
    def lols(self) -> UVector4: ...
    @property
    def lolt(self) -> UVector4: ...
    @property
    def llss(self) -> UVector4: ...
    @property
    def llst(self) -> UVector4: ...
    @property
    def llso(self) -> UVector4: ...
    @property
    def llsl(self) -> UVector4: ...
    @property
    def llts(self) -> UVector4: ...
    @property
    def lltt(self) -> UVector4: ...
    @property
    def llto(self) -> UVector4: ...
    @property
    def lltl(self) -> UVector4: ...
    @property
    def llos(self) -> UVector4: ...
    @property
    def llot(self) -> UVector4: ...
    @property
    def llls(self) -> UVector4: ...
    @property
    def lllt(self) -> UVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> UVector2: ...
    @property
    def uv(self) -> UVector2: ...
    @property
    def uo(self) -> UVector2: ...
    @property
    def ul(self) -> UVector2: ...
    @property
    def vu(self) -> UVector2: ...
    @property
    def vv(self) -> UVector2: ...
    @property
    def vo(self) -> UVector2: ...
    @property
    def vl(self) -> UVector2: ...
    @property
    def ou(self) -> UVector2: ...
    @property
    def ov(self) -> UVector2: ...
    @property
    def lu(self) -> UVector2: ...
    @property
    def lv(self) -> UVector2: ...
    @property
    def uuu(self) -> UVector3: ...
    @property
    def uuv(self) -> UVector3: ...
    @property
    def uuo(self) -> UVector3: ...
    @property
    def uul(self) -> UVector3: ...
    @property
    def uvu(self) -> UVector3: ...
    @property
    def uvv(self) -> UVector3: ...
    @property
    def uvo(self) -> UVector3: ...
    @property
    def uvl(self) -> UVector3: ...
    @property
    def uou(self) -> UVector3: ...
    @property
    def uov(self) -> UVector3: ...
    @property
    def uoo(self) -> UVector3: ...
    @property
    def uol(self) -> UVector3: ...
    @property
    def ulu(self) -> UVector3: ...
    @property
    def ulv(self) -> UVector3: ...
    @property
    def ulo(self) -> UVector3: ...
    @property
    def ull(self) -> UVector3: ...
    @property
    def vuu(self) -> UVector3: ...
    @property
    def vuv(self) -> UVector3: ...
    @property
    def vuo(self) -> UVector3: ...
    @property
    def vul(self) -> UVector3: ...
    @property
    def vvu(self) -> UVector3: ...
    @property
    def vvv(self) -> UVector3: ...
    @property
    def vvo(self) -> UVector3: ...
    @property
    def vvl(self) -> UVector3: ...
    @property
    def vou(self) -> UVector3: ...
    @property
    def vov(self) -> UVector3: ...
    @property
    def voo(self) -> UVector3: ...
    @property
    def vol(self) -> UVector3: ...
    @property
    def vlu(self) -> UVector3: ...
    @property
    def vlv(self) -> UVector3: ...
    @property
    def vlo(self) -> UVector3: ...
    @property
    def vll(self) -> UVector3: ...
    @property
    def ouu(self) -> UVector3: ...
    @property
    def ouv(self) -> UVector3: ...
    @property
    def ouo(self) -> UVector3: ...
    @property
    def oul(self) -> UVector3: ...
    @property
    def ovu(self) -> UVector3: ...
    @property
    def ovv(self) -> UVector3: ...
    @property
    def ovo(self) -> UVector3: ...
    @property
    def ovl(self) -> UVector3: ...
    @property
    def oou(self) -> UVector3: ...
    @property
    def oov(self) -> UVector3: ...
    @property
    def olu(self) -> UVector3: ...
    @property
    def olv(self) -> UVector3: ...
    @property
    def luu(self) -> UVector3: ...
    @property
    def luv(self) -> UVector3: ...
    @property
    def luo(self) -> UVector3: ...
    @property
    def lul(self) -> UVector3: ...
    @property
    def lvu(self) -> UVector3: ...
    @property
    def lvv(self) -> UVector3: ...
    @property
    def lvo(self) -> UVector3: ...
    @property
    def lvl(self) -> UVector3: ...
    @property
    def lou(self) -> UVector3: ...
    @property
    def lov(self) -> UVector3: ...
    @property
    def llu(self) -> UVector3: ...
    @property
    def llv(self) -> UVector3: ...
    @property
    def uuuu(self) -> UVector4: ...
    @property
    def uuuv(self) -> UVector4: ...
    @property
    def uuuo(self) -> UVector4: ...
    @property
    def uuul(self) -> UVector4: ...
    @property
    def uuvu(self) -> UVector4: ...
    @property
    def uuvv(self) -> UVector4: ...
    @property
    def uuvo(self) -> UVector4: ...
    @property
    def uuvl(self) -> UVector4: ...
    @property
    def uuou(self) -> UVector4: ...
    @property
    def uuov(self) -> UVector4: ...
    @property
    def uuoo(self) -> UVector4: ...
    @property
    def uuol(self) -> UVector4: ...
    @property
    def uulu(self) -> UVector4: ...
    @property
    def uulv(self) -> UVector4: ...
    @property
    def uulo(self) -> UVector4: ...
    @property
    def uull(self) -> UVector4: ...
    @property
    def uvuu(self) -> UVector4: ...
    @property
    def uvuv(self) -> UVector4: ...
    @property
    def uvuo(self) -> UVector4: ...
    @property
    def uvul(self) -> UVector4: ...
    @property
    def uvvu(self) -> UVector4: ...
    @property
    def uvvv(self) -> UVector4: ...
    @property
    def uvvo(self) -> UVector4: ...
    @property
    def uvvl(self) -> UVector4: ...
    @property
    def uvou(self) -> UVector4: ...
    @property
    def uvov(self) -> UVector4: ...
    @property
    def uvoo(self) -> UVector4: ...
    @property
    def uvol(self) -> UVector4: ...
    @property
    def uvlu(self) -> UVector4: ...
    @property
    def uvlv(self) -> UVector4: ...
    @property
    def uvlo(self) -> UVector4: ...
    @property
    def uvll(self) -> UVector4: ...
    @property
    def uouu(self) -> UVector4: ...
    @property
    def uouv(self) -> UVector4: ...
    @property
    def uouo(self) -> UVector4: ...
    @property
    def uoul(self) -> UVector4: ...
    @property
    def uovu(self) -> UVector4: ...
    @property
    def uovv(self) -> UVector4: ...
    @property
    def uovo(self) -> UVector4: ...
    @property
    def uovl(self) -> UVector4: ...
    @property
    def uoou(self) -> UVector4: ...
    @property
    def uoov(self) -> UVector4: ...
    @property
    def uooo(self) -> UVector4: ...
    @property
    def uool(self) -> UVector4: ...
    @property
    def uolu(self) -> UVector4: ...
    @property
    def uolv(self) -> UVector4: ...
    @property
    def uolo(self) -> UVector4: ...
    @property
    def uoll(self) -> UVector4: ...
    @property
    def uluu(self) -> UVector4: ...
    @property
    def uluv(self) -> UVector4: ...
    @property
    def uluo(self) -> UVector4: ...
    @property
    def ulul(self) -> UVector4: ...
    @property
    def ulvu(self) -> UVector4: ...
    @property
    def ulvv(self) -> UVector4: ...
    @property
    def ulvo(self) -> UVector4: ...
    @property
    def ulvl(self) -> UVector4: ...
    @property
    def ulou(self) -> UVector4: ...
    @property
    def ulov(self) -> UVector4: ...
    @property
    def uloo(self) -> UVector4: ...
    @property
    def ulol(self) -> UVector4: ...
    @property
    def ullu(self) -> UVector4: ...
    @property
    def ullv(self) -> UVector4: ...
    @property
    def ullo(self) -> UVector4: ...
    @property
    def ulll(self) -> UVector4: ...
    @property
    def vuuu(self) -> UVector4: ...
    @property
    def vuuv(self) -> UVector4: ...
    @property
    def vuuo(self) -> UVector4: ...
    @property
    def vuul(self) -> UVector4: ...
    @property
    def vuvu(self) -> UVector4: ...
    @property
    def vuvv(self) -> UVector4: ...
    @property
    def vuvo(self) -> UVector4: ...
    @property
    def vuvl(self) -> UVector4: ...
    @property
    def vuou(self) -> UVector4: ...
    @property
    def vuov(self) -> UVector4: ...
    @property
    def vuoo(self) -> UVector4: ...
    @property
    def vuol(self) -> UVector4: ...
    @property
    def vulu(self) -> UVector4: ...
    @property
    def vulv(self) -> UVector4: ...
    @property
    def vulo(self) -> UVector4: ...
    @property
    def vull(self) -> UVector4: ...
    @property
    def vvuu(self) -> UVector4: ...
    @property
    def vvuv(self) -> UVector4: ...
    @property
    def vvuo(self) -> UVector4: ...
    @property
    def vvul(self) -> UVector4: ...
    @property
    def vvvu(self) -> UVector4: ...
    @property
    def vvvv(self) -> UVector4: ...
    @property
    def vvvo(self) -> UVector4: ...
    @property
    def vvvl(self) -> UVector4: ...
    @property
    def vvou(self) -> UVector4: ...
    @property
    def vvov(self) -> UVector4: ...
    @property
    def vvoo(self) -> UVector4: ...
    @property
    def vvol(self) -> UVector4: ...
    @property
    def vvlu(self) -> UVector4: ...
    @property
    def vvlv(self) -> UVector4: ...
    @property
    def vvlo(self) -> UVector4: ...
    @property
    def vvll(self) -> UVector4: ...
    @property
    def vouu(self) -> UVector4: ...
    @property
    def vouv(self) -> UVector4: ...
    @property
    def vouo(self) -> UVector4: ...
    @property
    def voul(self) -> UVector4: ...
    @property
    def vovu(self) -> UVector4: ...
    @property
    def vovv(self) -> UVector4: ...
    @property
    def vovo(self) -> UVector4: ...
    @property
    def vovl(self) -> UVector4: ...
    @property
    def voou(self) -> UVector4: ...
    @property
    def voov(self) -> UVector4: ...
    @property
    def vooo(self) -> UVector4: ...
    @property
    def vool(self) -> UVector4: ...
    @property
    def volu(self) -> UVector4: ...
    @property
    def volv(self) -> UVector4: ...
    @property
    def volo(self) -> UVector4: ...
    @property
    def voll(self) -> UVector4: ...
    @property
    def vluu(self) -> UVector4: ...
    @property
    def vluv(self) -> UVector4: ...
    @property
    def vluo(self) -> UVector4: ...
    @property
    def vlul(self) -> UVector4: ...
    @property
    def vlvu(self) -> UVector4: ...
    @property
    def vlvv(self) -> UVector4: ...
    @property
    def vlvo(self) -> UVector4: ...
    @property
    def vlvl(self) -> UVector4: ...
    @property
    def vlou(self) -> UVector4: ...
    @property
    def vlov(self) -> UVector4: ...
    @property
    def vloo(self) -> UVector4: ...
    @property
    def vlol(self) -> UVector4: ...
    @property
    def vllu(self) -> UVector4: ...
    @property
    def vllv(self) -> UVector4: ...
    @property
    def vllo(self) -> UVector4: ...
    @property
    def vlll(self) -> UVector4: ...
    @property
    def ouuu(self) -> UVector4: ...
    @property
    def ouuv(self) -> UVector4: ...
    @property
    def ouuo(self) -> UVector4: ...
    @property
    def ouul(self) -> UVector4: ...
    @property
    def ouvu(self) -> UVector4: ...
    @property
    def ouvv(self) -> UVector4: ...
    @property
    def ouvo(self) -> UVector4: ...
    @property
    def ouvl(self) -> UVector4: ...
    @property
    def ouou(self) -> UVector4: ...
    @property
    def ouov(self) -> UVector4: ...
    @property
    def ouoo(self) -> UVector4: ...
    @property
    def ouol(self) -> UVector4: ...
    @property
    def oulu(self) -> UVector4: ...
    @property
    def oulv(self) -> UVector4: ...
    @property
    def oulo(self) -> UVector4: ...
    @property
    def oull(self) -> UVector4: ...
    @property
    def ovuu(self) -> UVector4: ...
    @property
    def ovuv(self) -> UVector4: ...
    @property
    def ovuo(self) -> UVector4: ...
    @property
    def ovul(self) -> UVector4: ...
    @property
    def ovvu(self) -> UVector4: ...
    @property
    def ovvv(self) -> UVector4: ...
    @property
    def ovvo(self) -> UVector4: ...
    @property
    def ovvl(self) -> UVector4: ...
    @property
    def ovou(self) -> UVector4: ...
    @property
    def ovov(self) -> UVector4: ...
    @property
    def ovoo(self) -> UVector4: ...
    @property
    def ovol(self) -> UVector4: ...
    @property
    def ovlu(self) -> UVector4: ...
    @property
    def ovlv(self) -> UVector4: ...
    @property
    def ovlo(self) -> UVector4: ...
    @property
    def ovll(self) -> UVector4: ...
    @property
    def oouu(self) -> UVector4: ...
    @property
    def oouv(self) -> UVector4: ...
    @property
    def oouo(self) -> UVector4: ...
    @property
    def ooul(self) -> UVector4: ...
    @property
    def oovu(self) -> UVector4: ...
    @property
    def oovv(self) -> UVector4: ...
    @property
    def oovo(self) -> UVector4: ...
    @property
    def oovl(self) -> UVector4: ...
    @property
    def ooou(self) -> UVector4: ...
    @property
    def ooov(self) -> UVector4: ...
    @property
    def oolu(self) -> UVector4: ...
    @property
    def oolv(self) -> UVector4: ...
    @property
    def oluu(self) -> UVector4: ...
    @property
    def oluv(self) -> UVector4: ...
    @property
    def oluo(self) -> UVector4: ...
    @property
    def olul(self) -> UVector4: ...
    @property
    def olvu(self) -> UVector4: ...
    @property
    def olvv(self) -> UVector4: ...
    @property
    def olvo(self) -> UVector4: ...
    @property
    def olvl(self) -> UVector4: ...
    @property
    def olou(self) -> UVector4: ...
    @property
    def olov(self) -> UVector4: ...
    @property
    def ollu(self) -> UVector4: ...
    @property
    def ollv(self) -> UVector4: ...
    @property
    def luuu(self) -> UVector4: ...
    @property
    def luuv(self) -> UVector4: ...
    @property
    def luuo(self) -> UVector4: ...
    @property
    def luul(self) -> UVector4: ...
    @property
    def luvu(self) -> UVector4: ...
    @property
    def luvv(self) -> UVector4: ...
    @property
    def luvo(self) -> UVector4: ...
    @property
    def luvl(self) -> UVector4: ...
    @property
    def luou(self) -> UVector4: ...
    @property
    def luov(self) -> UVector4: ...
    @property
    def luoo(self) -> UVector4: ...
    @property
    def luol(self) -> UVector4: ...
    @property
    def lulu(self) -> UVector4: ...
    @property
    def lulv(self) -> UVector4: ...
    @property
    def lulo(self) -> UVector4: ...
    @property
    def lull(self) -> UVector4: ...
    @property
    def lvuu(self) -> UVector4: ...
    @property
    def lvuv(self) -> UVector4: ...
    @property
    def lvuo(self) -> UVector4: ...
    @property
    def lvul(self) -> UVector4: ...
    @property
    def lvvu(self) -> UVector4: ...
    @property
    def lvvv(self) -> UVector4: ...
    @property
    def lvvo(self) -> UVector4: ...
    @property
    def lvvl(self) -> UVector4: ...
    @property
    def lvou(self) -> UVector4: ...
    @property
    def lvov(self) -> UVector4: ...
    @property
    def lvoo(self) -> UVector4: ...
    @property
    def lvol(self) -> UVector4: ...
    @property
    def lvlu(self) -> UVector4: ...
    @property
    def lvlv(self) -> UVector4: ...
    @property
    def lvlo(self) -> UVector4: ...
    @property
    def lvll(self) -> UVector4: ...
    @property
    def louu(self) -> UVector4: ...
    @property
    def louv(self) -> UVector4: ...
    @property
    def louo(self) -> UVector4: ...
    @property
    def loul(self) -> UVector4: ...
    @property
    def lovu(self) -> UVector4: ...
    @property
    def lovv(self) -> UVector4: ...
    @property
    def lovo(self) -> UVector4: ...
    @property
    def lovl(self) -> UVector4: ...
    @property
    def loou(self) -> UVector4: ...
    @property
    def loov(self) -> UVector4: ...
    @property
    def lolu(self) -> UVector4: ...
    @property
    def lolv(self) -> UVector4: ...
    @property
    def lluu(self) -> UVector4: ...
    @property
    def lluv(self) -> UVector4: ...
    @property
    def lluo(self) -> UVector4: ...
    @property
    def llul(self) -> UVector4: ...
    @property
    def llvu(self) -> UVector4: ...
    @property
    def llvv(self) -> UVector4: ...
    @property
    def llvo(self) -> UVector4: ...
    @property
    def llvl(self) -> UVector4: ...
    @property
    def llou(self) -> UVector4: ...
    @property
    def llov(self) -> UVector4: ...
    @property
    def lllu(self) -> UVector4: ...
    @property
    def lllv(self) -> UVector4: ...
    @property
    def oo(self) -> UVector2: ...
    @property
    def ol(self) -> UVector2: ...
    @property
    def lo(self) -> UVector2: ...
    @property
    def ll(self) -> UVector2: ...
    @property
    def ooo(self) -> UVector3: ...
    @property
    def ool(self) -> UVector3: ...
    @property
    def olo(self) -> UVector3: ...
    @property
    def oll(self) -> UVector3: ...
    @property
    def loo(self) -> UVector3: ...
    @property
    def lol(self) -> UVector3: ...
    @property
    def llo(self) -> UVector3: ...
    @property
    def lll(self) -> UVector3: ...
    @property
    def oooo(self) -> UVector4: ...
    @property
    def oool(self) -> UVector4: ...
    @property
    def oolo(self) -> UVector4: ...
    @property
    def ooll(self) -> UVector4: ...
    @property
    def oloo(self) -> UVector4: ...
    @property
    def olol(self) -> UVector4: ...
    @property
    def ollo(self) -> UVector4: ...
    @property
    def olll(self) -> UVector4: ...
    @property
    def looo(self) -> UVector4: ...
    @property
    def lool(self) -> UVector4: ...
    @property
    def lolo(self) -> UVector4: ...
    @property
    def loll(self) -> UVector4: ...
    @property
    def lloo(self) -> UVector4: ...
    @property
    def llol(self) -> UVector4: ...
    @property
    def lllo(self) -> UVector4: ...
    @property
    def llll(self) -> UVector4: ...
    def min(self, n: Number, /) -> UVector2: ...
    def max(self, n: Number, /) -> UVector2: ...
    def clamp(self, min: Number, max: Number, /) -> UVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[UVector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector2: ...

@final
class UVector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: UVector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[UVector2]: ...
    @overload
    def __getitem__(self, index: int) -> UVector2: ...
    @overload
    def __getitem__(self, index: slice) -> UVector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[UVector2]: ...

@final
class I64Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __add__(self, other: Number) -> I64Vector2: ...
    @overload
    def __radd__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __radd__(self, other: Number) -> I64Vector2: ...
    @overload
    def __sub__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __sub__(self, other: Number) -> I64Vector2: ...
    @overload
    def __rsub__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> I64Vector2: ...
    @overload
    def __mul__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __mul__(self, other: Number) -> I64Vector2: ...
    @overload
    def __rmul__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> I64Vector2: ...
    @overload
    def __truediv__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> I64Vector2: ...
    @overload
    def __rtruediv__(self, other: I64Vector2) -> I64Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> I64Vector2: ...
    def __neg__(self) -> I64Vector2: ...
    def __abs__(self) -> I64Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> I64Vector2: ...
    @property
    def xy(self) -> I64Vector2: ...
    @property
    def xo(self) -> I64Vector2: ...
    @property
    def xl(self) -> I64Vector2: ...
    @property
    def yx(self) -> I64Vector2: ...
    @property
    def yy(self) -> I64Vector2: ...
    @property
    def yo(self) -> I64Vector2: ...
    @property
    def yl(self) -> I64Vector2: ...
    @property
    def ox(self) -> I64Vector2: ...
    @property
    def oy(self) -> I64Vector2: ...
    @property
    def lx(self) -> I64Vector2: ...
    @property
    def ly(self) -> I64Vector2: ...
    @property
    def xxx(self) -> I64Vector3: ...
    @property
    def xxy(self) -> I64Vector3: ...
    @property
    def xxo(self) -> I64Vector3: ...
    @property
    def xxl(self) -> I64Vector3: ...
    @property
    def xyx(self) -> I64Vector3: ...
    @property
    def xyy(self) -> I64Vector3: ...
    @property
    def xyo(self) -> I64Vector3: ...
    @property
    def xyl(self) -> I64Vector3: ...
    @property
    def xox(self) -> I64Vector3: ...
    @property
    def xoy(self) -> I64Vector3: ...
    @property
    def xoo(self) -> I64Vector3: ...
    @property
    def xol(self) -> I64Vector3: ...
    @property
    def xlx(self) -> I64Vector3: ...
    @property
    def xly(self) -> I64Vector3: ...
    @property
    def xlo(self) -> I64Vector3: ...
    @property
    def xll(self) -> I64Vector3: ...
    @property
    def yxx(self) -> I64Vector3: ...
    @property
    def yxy(self) -> I64Vector3: ...
    @property
    def yxo(self) -> I64Vector3: ...
    @property
    def yxl(self) -> I64Vector3: ...
    @property
    def yyx(self) -> I64Vector3: ...
    @property
    def yyy(self) -> I64Vector3: ...
    @property
    def yyo(self) -> I64Vector3: ...
    @property
    def yyl(self) -> I64Vector3: ...
    @property
    def yox(self) -> I64Vector3: ...
    @property
    def yoy(self) -> I64Vector3: ...
    @property
    def yoo(self) -> I64Vector3: ...
    @property
    def yol(self) -> I64Vector3: ...
    @property
    def ylx(self) -> I64Vector3: ...
    @property
    def yly(self) -> I64Vector3: ...
    @property
    def ylo(self) -> I64Vector3: ...
    @property
    def yll(self) -> I64Vector3: ...
    @property
    def oxx(self) -> I64Vector3: ...
    @property
    def oxy(self) -> I64Vector3: ...
    @property
    def oxo(self) -> I64Vector3: ...
    @property
    def oxl(self) -> I64Vector3: ...
    @property
    def oyx(self) -> I64Vector3: ...
    @property
    def oyy(self) -> I64Vector3: ...
    @property
    def oyo(self) -> I64Vector3: ...
    @property
    def oyl(self) -> I64Vector3: ...
    @property
    def oox(self) -> I64Vector3: ...
    @property
    def ooy(self) -> I64Vector3: ...
    @property
    def olx(self) -> I64Vector3: ...
    @property
    def oly(self) -> I64Vector3: ...
    @property
    def lxx(self) -> I64Vector3: ...
    @property
    def lxy(self) -> I64Vector3: ...
    @property
    def lxo(self) -> I64Vector3: ...
    @property
    def lxl(self) -> I64Vector3: ...
    @property
    def lyx(self) -> I64Vector3: ...
    @property
    def lyy(self) -> I64Vector3: ...
    @property
    def lyo(self) -> I64Vector3: ...
    @property
    def lyl(self) -> I64Vector3: ...
    @property
    def lox(self) -> I64Vector3: ...
    @property
    def loy(self) -> I64Vector3: ...
    @property
    def llx(self) -> I64Vector3: ...
    @property
    def lly(self) -> I64Vector3: ...
    @property
    def xxxx(self) -> I64Vector4: ...
    @property
    def xxxy(self) -> I64Vector4: ...
    @property
    def xxxo(self) -> I64Vector4: ...
    @property
    def xxxl(self) -> I64Vector4: ...
    @property
    def xxyx(self) -> I64Vector4: ...
    @property
    def xxyy(self) -> I64Vector4: ...
    @property
    def xxyo(self) -> I64Vector4: ...
    @property
    def xxyl(self) -> I64Vector4: ...
    @property
    def xxox(self) -> I64Vector4: ...
    @property
    def xxoy(self) -> I64Vector4: ...
    @property
    def xxoo(self) -> I64Vector4: ...
    @property
    def xxol(self) -> I64Vector4: ...
    @property
    def xxlx(self) -> I64Vector4: ...
    @property
    def xxly(self) -> I64Vector4: ...
    @property
    def xxlo(self) -> I64Vector4: ...
    @property
    def xxll(self) -> I64Vector4: ...
    @property
    def xyxx(self) -> I64Vector4: ...
    @property
    def xyxy(self) -> I64Vector4: ...
    @property
    def xyxo(self) -> I64Vector4: ...
    @property
    def xyxl(self) -> I64Vector4: ...
    @property
    def xyyx(self) -> I64Vector4: ...
    @property
    def xyyy(self) -> I64Vector4: ...
    @property
    def xyyo(self) -> I64Vector4: ...
    @property
    def xyyl(self) -> I64Vector4: ...
    @property
    def xyox(self) -> I64Vector4: ...
    @property
    def xyoy(self) -> I64Vector4: ...
    @property
    def xyoo(self) -> I64Vector4: ...
    @property
    def xyol(self) -> I64Vector4: ...
    @property
    def xylx(self) -> I64Vector4: ...
    @property
    def xyly(self) -> I64Vector4: ...
    @property
    def xylo(self) -> I64Vector4: ...
    @property
    def xyll(self) -> I64Vector4: ...
    @property
    def xoxx(self) -> I64Vector4: ...
    @property
    def xoxy(self) -> I64Vector4: ...
    @property
    def xoxo(self) -> I64Vector4: ...
    @property
    def xoxl(self) -> I64Vector4: ...
    @property
    def xoyx(self) -> I64Vector4: ...
    @property
    def xoyy(self) -> I64Vector4: ...
    @property
    def xoyo(self) -> I64Vector4: ...
    @property
    def xoyl(self) -> I64Vector4: ...
    @property
    def xoox(self) -> I64Vector4: ...
    @property
    def xooy(self) -> I64Vector4: ...
    @property
    def xooo(self) -> I64Vector4: ...
    @property
    def xool(self) -> I64Vector4: ...
    @property
    def xolx(self) -> I64Vector4: ...
    @property
    def xoly(self) -> I64Vector4: ...
    @property
    def xolo(self) -> I64Vector4: ...
    @property
    def xoll(self) -> I64Vector4: ...
    @property
    def xlxx(self) -> I64Vector4: ...
    @property
    def xlxy(self) -> I64Vector4: ...
    @property
    def xlxo(self) -> I64Vector4: ...
    @property
    def xlxl(self) -> I64Vector4: ...
    @property
    def xlyx(self) -> I64Vector4: ...
    @property
    def xlyy(self) -> I64Vector4: ...
    @property
    def xlyo(self) -> I64Vector4: ...
    @property
    def xlyl(self) -> I64Vector4: ...
    @property
    def xlox(self) -> I64Vector4: ...
    @property
    def xloy(self) -> I64Vector4: ...
    @property
    def xloo(self) -> I64Vector4: ...
    @property
    def xlol(self) -> I64Vector4: ...
    @property
    def xllx(self) -> I64Vector4: ...
    @property
    def xlly(self) -> I64Vector4: ...
    @property
    def xllo(self) -> I64Vector4: ...
    @property
    def xlll(self) -> I64Vector4: ...
    @property
    def yxxx(self) -> I64Vector4: ...
    @property
    def yxxy(self) -> I64Vector4: ...
    @property
    def yxxo(self) -> I64Vector4: ...
    @property
    def yxxl(self) -> I64Vector4: ...
    @property
    def yxyx(self) -> I64Vector4: ...
    @property
    def yxyy(self) -> I64Vector4: ...
    @property
    def yxyo(self) -> I64Vector4: ...
    @property
    def yxyl(self) -> I64Vector4: ...
    @property
    def yxox(self) -> I64Vector4: ...
    @property
    def yxoy(self) -> I64Vector4: ...
    @property
    def yxoo(self) -> I64Vector4: ...
    @property
    def yxol(self) -> I64Vector4: ...
    @property
    def yxlx(self) -> I64Vector4: ...
    @property
    def yxly(self) -> I64Vector4: ...
    @property
    def yxlo(self) -> I64Vector4: ...
    @property
    def yxll(self) -> I64Vector4: ...
    @property
    def yyxx(self) -> I64Vector4: ...
    @property
    def yyxy(self) -> I64Vector4: ...
    @property
    def yyxo(self) -> I64Vector4: ...
    @property
    def yyxl(self) -> I64Vector4: ...
    @property
    def yyyx(self) -> I64Vector4: ...
    @property
    def yyyy(self) -> I64Vector4: ...
    @property
    def yyyo(self) -> I64Vector4: ...
    @property
    def yyyl(self) -> I64Vector4: ...
    @property
    def yyox(self) -> I64Vector4: ...
    @property
    def yyoy(self) -> I64Vector4: ...
    @property
    def yyoo(self) -> I64Vector4: ...
    @property
    def yyol(self) -> I64Vector4: ...
    @property
    def yylx(self) -> I64Vector4: ...
    @property
    def yyly(self) -> I64Vector4: ...
    @property
    def yylo(self) -> I64Vector4: ...
    @property
    def yyll(self) -> I64Vector4: ...
    @property
    def yoxx(self) -> I64Vector4: ...
    @property
    def yoxy(self) -> I64Vector4: ...
    @property
    def yoxo(self) -> I64Vector4: ...
    @property
    def yoxl(self) -> I64Vector4: ...
    @property
    def yoyx(self) -> I64Vector4: ...
    @property
    def yoyy(self) -> I64Vector4: ...
    @property
    def yoyo(self) -> I64Vector4: ...
    @property
    def yoyl(self) -> I64Vector4: ...
    @property
    def yoox(self) -> I64Vector4: ...
    @property
    def yooy(self) -> I64Vector4: ...
    @property
    def yooo(self) -> I64Vector4: ...
    @property
    def yool(self) -> I64Vector4: ...
    @property
    def yolx(self) -> I64Vector4: ...
    @property
    def yoly(self) -> I64Vector4: ...
    @property
    def yolo(self) -> I64Vector4: ...
    @property
    def yoll(self) -> I64Vector4: ...
    @property
    def ylxx(self) -> I64Vector4: ...
    @property
    def ylxy(self) -> I64Vector4: ...
    @property
    def ylxo(self) -> I64Vector4: ...
    @property
    def ylxl(self) -> I64Vector4: ...
    @property
    def ylyx(self) -> I64Vector4: ...
    @property
    def ylyy(self) -> I64Vector4: ...
    @property
    def ylyo(self) -> I64Vector4: ...
    @property
    def ylyl(self) -> I64Vector4: ...
    @property
    def ylox(self) -> I64Vector4: ...
    @property
    def yloy(self) -> I64Vector4: ...
    @property
    def yloo(self) -> I64Vector4: ...
    @property
    def ylol(self) -> I64Vector4: ...
    @property
    def yllx(self) -> I64Vector4: ...
    @property
    def ylly(self) -> I64Vector4: ...
    @property
    def yllo(self) -> I64Vector4: ...
    @property
    def ylll(self) -> I64Vector4: ...
    @property
    def oxxx(self) -> I64Vector4: ...
    @property
    def oxxy(self) -> I64Vector4: ...
    @property
    def oxxo(self) -> I64Vector4: ...
    @property
    def oxxl(self) -> I64Vector4: ...
    @property
    def oxyx(self) -> I64Vector4: ...
    @property
    def oxyy(self) -> I64Vector4: ...
    @property
    def oxyo(self) -> I64Vector4: ...
    @property
    def oxyl(self) -> I64Vector4: ...
    @property
    def oxox(self) -> I64Vector4: ...
    @property
    def oxoy(self) -> I64Vector4: ...
    @property
    def oxoo(self) -> I64Vector4: ...
    @property
    def oxol(self) -> I64Vector4: ...
    @property
    def oxlx(self) -> I64Vector4: ...
    @property
    def oxly(self) -> I64Vector4: ...
    @property
    def oxlo(self) -> I64Vector4: ...
    @property
    def oxll(self) -> I64Vector4: ...
    @property
    def oyxx(self) -> I64Vector4: ...
    @property
    def oyxy(self) -> I64Vector4: ...
    @property
    def oyxo(self) -> I64Vector4: ...
    @property
    def oyxl(self) -> I64Vector4: ...
    @property
    def oyyx(self) -> I64Vector4: ...
    @property
    def oyyy(self) -> I64Vector4: ...
    @property
    def oyyo(self) -> I64Vector4: ...
    @property
    def oyyl(self) -> I64Vector4: ...
    @property
    def oyox(self) -> I64Vector4: ...
    @property
    def oyoy(self) -> I64Vector4: ...
    @property
    def oyoo(self) -> I64Vector4: ...
    @property
    def oyol(self) -> I64Vector4: ...
    @property
    def oylx(self) -> I64Vector4: ...
    @property
    def oyly(self) -> I64Vector4: ...
    @property
    def oylo(self) -> I64Vector4: ...
    @property
    def oyll(self) -> I64Vector4: ...
    @property
    def ooxx(self) -> I64Vector4: ...
    @property
    def ooxy(self) -> I64Vector4: ...
    @property
    def ooxo(self) -> I64Vector4: ...
    @property
    def ooxl(self) -> I64Vector4: ...
    @property
    def ooyx(self) -> I64Vector4: ...
    @property
    def ooyy(self) -> I64Vector4: ...
    @property
    def ooyo(self) -> I64Vector4: ...
    @property
    def ooyl(self) -> I64Vector4: ...
    @property
    def ooox(self) -> I64Vector4: ...
    @property
    def oooy(self) -> I64Vector4: ...
    @property
    def oolx(self) -> I64Vector4: ...
    @property
    def ooly(self) -> I64Vector4: ...
    @property
    def olxx(self) -> I64Vector4: ...
    @property
    def olxy(self) -> I64Vector4: ...
    @property
    def olxo(self) -> I64Vector4: ...
    @property
    def olxl(self) -> I64Vector4: ...
    @property
    def olyx(self) -> I64Vector4: ...
    @property
    def olyy(self) -> I64Vector4: ...
    @property
    def olyo(self) -> I64Vector4: ...
    @property
    def olyl(self) -> I64Vector4: ...
    @property
    def olox(self) -> I64Vector4: ...
    @property
    def oloy(self) -> I64Vector4: ...
    @property
    def ollx(self) -> I64Vector4: ...
    @property
    def olly(self) -> I64Vector4: ...
    @property
    def lxxx(self) -> I64Vector4: ...
    @property
    def lxxy(self) -> I64Vector4: ...
    @property
    def lxxo(self) -> I64Vector4: ...
    @property
    def lxxl(self) -> I64Vector4: ...
    @property
    def lxyx(self) -> I64Vector4: ...
    @property
    def lxyy(self) -> I64Vector4: ...
    @property
    def lxyo(self) -> I64Vector4: ...
    @property
    def lxyl(self) -> I64Vector4: ...
    @property
    def lxox(self) -> I64Vector4: ...
    @property
    def lxoy(self) -> I64Vector4: ...
    @property
    def lxoo(self) -> I64Vector4: ...
    @property
    def lxol(self) -> I64Vector4: ...
    @property
    def lxlx(self) -> I64Vector4: ...
    @property
    def lxly(self) -> I64Vector4: ...
    @property
    def lxlo(self) -> I64Vector4: ...
    @property
    def lxll(self) -> I64Vector4: ...
    @property
    def lyxx(self) -> I64Vector4: ...
    @property
    def lyxy(self) -> I64Vector4: ...
    @property
    def lyxo(self) -> I64Vector4: ...
    @property
    def lyxl(self) -> I64Vector4: ...
    @property
    def lyyx(self) -> I64Vector4: ...
    @property
    def lyyy(self) -> I64Vector4: ...
    @property
    def lyyo(self) -> I64Vector4: ...
    @property
    def lyyl(self) -> I64Vector4: ...
    @property
    def lyox(self) -> I64Vector4: ...
    @property
    def lyoy(self) -> I64Vector4: ...
    @property
    def lyoo(self) -> I64Vector4: ...
    @property
    def lyol(self) -> I64Vector4: ...
    @property
    def lylx(self) -> I64Vector4: ...
    @property
    def lyly(self) -> I64Vector4: ...
    @property
    def lylo(self) -> I64Vector4: ...
    @property
    def lyll(self) -> I64Vector4: ...
    @property
    def loxx(self) -> I64Vector4: ...
    @property
    def loxy(self) -> I64Vector4: ...
    @property
    def loxo(self) -> I64Vector4: ...
    @property
    def loxl(self) -> I64Vector4: ...
    @property
    def loyx(self) -> I64Vector4: ...
    @property
    def loyy(self) -> I64Vector4: ...
    @property
    def loyo(self) -> I64Vector4: ...
    @property
    def loyl(self) -> I64Vector4: ...
    @property
    def loox(self) -> I64Vector4: ...
    @property
    def looy(self) -> I64Vector4: ...
    @property
    def lolx(self) -> I64Vector4: ...
    @property
    def loly(self) -> I64Vector4: ...
    @property
    def llxx(self) -> I64Vector4: ...
    @property
    def llxy(self) -> I64Vector4: ...
    @property
    def llxo(self) -> I64Vector4: ...
    @property
    def llxl(self) -> I64Vector4: ...
    @property
    def llyx(self) -> I64Vector4: ...
    @property
    def llyy(self) -> I64Vector4: ...
    @property
    def llyo(self) -> I64Vector4: ...
    @property
    def llyl(self) -> I64Vector4: ...
    @property
    def llox(self) -> I64Vector4: ...
    @property
    def lloy(self) -> I64Vector4: ...
    @property
    def lllx(self) -> I64Vector4: ...
    @property
    def llly(self) -> I64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> I64Vector2: ...
    @property
    def rg(self) -> I64Vector2: ...
    @property
    def ro(self) -> I64Vector2: ...
    @property
    def rl(self) -> I64Vector2: ...
    @property
    def gr(self) -> I64Vector2: ...
    @property
    def gg(self) -> I64Vector2: ...
    @property
    def go(self) -> I64Vector2: ...
    @property
    def gl(self) -> I64Vector2: ...
    @property
    def og(self) -> I64Vector2: ...
    @property
    def lr(self) -> I64Vector2: ...
    @property
    def lg(self) -> I64Vector2: ...
    @property
    def rrr(self) -> I64Vector3: ...
    @property
    def rrg(self) -> I64Vector3: ...
    @property
    def rro(self) -> I64Vector3: ...
    @property
    def rrl(self) -> I64Vector3: ...
    @property
    def rgr(self) -> I64Vector3: ...
    @property
    def rgg(self) -> I64Vector3: ...
    @property
    def rgo(self) -> I64Vector3: ...
    @property
    def rgl(self) -> I64Vector3: ...
    @property
    def ror(self) -> I64Vector3: ...
    @property
    def rog(self) -> I64Vector3: ...
    @property
    def roo(self) -> I64Vector3: ...
    @property
    def rol(self) -> I64Vector3: ...
    @property
    def rlr(self) -> I64Vector3: ...
    @property
    def rlg(self) -> I64Vector3: ...
    @property
    def rlo(self) -> I64Vector3: ...
    @property
    def rll(self) -> I64Vector3: ...
    @property
    def grr(self) -> I64Vector3: ...
    @property
    def grg(self) -> I64Vector3: ...
    @property
    def gro(self) -> I64Vector3: ...
    @property
    def grl(self) -> I64Vector3: ...
    @property
    def ggr(self) -> I64Vector3: ...
    @property
    def ggg(self) -> I64Vector3: ...
    @property
    def ggo(self) -> I64Vector3: ...
    @property
    def ggl(self) -> I64Vector3: ...
    @property
    def gor(self) -> I64Vector3: ...
    @property
    def gog(self) -> I64Vector3: ...
    @property
    def goo(self) -> I64Vector3: ...
    @property
    def gol(self) -> I64Vector3: ...
    @property
    def glr(self) -> I64Vector3: ...
    @property
    def glg(self) -> I64Vector3: ...
    @property
    def glo(self) -> I64Vector3: ...
    @property
    def gll(self) -> I64Vector3: ...
    @property
    def orr(self) -> I64Vector3: ...
    @property
    def org(self) -> I64Vector3: ...
    @property
    def oro(self) -> I64Vector3: ...
    @property
    def orl(self) -> I64Vector3: ...
    @property
    def ogr(self) -> I64Vector3: ...
    @property
    def ogg(self) -> I64Vector3: ...
    @property
    def ogo(self) -> I64Vector3: ...
    @property
    def ogl(self) -> I64Vector3: ...
    @property
    def oor(self) -> I64Vector3: ...
    @property
    def oog(self) -> I64Vector3: ...
    @property
    def olr(self) -> I64Vector3: ...
    @property
    def olg(self) -> I64Vector3: ...
    @property
    def lrr(self) -> I64Vector3: ...
    @property
    def lrg(self) -> I64Vector3: ...
    @property
    def lro(self) -> I64Vector3: ...
    @property
    def lrl(self) -> I64Vector3: ...
    @property
    def lgr(self) -> I64Vector3: ...
    @property
    def lgg(self) -> I64Vector3: ...
    @property
    def lgo(self) -> I64Vector3: ...
    @property
    def lgl(self) -> I64Vector3: ...
    @property
    def lor(self) -> I64Vector3: ...
    @property
    def log(self) -> I64Vector3: ...
    @property
    def llr(self) -> I64Vector3: ...
    @property
    def llg(self) -> I64Vector3: ...
    @property
    def rrrr(self) -> I64Vector4: ...
    @property
    def rrrg(self) -> I64Vector4: ...
    @property
    def rrro(self) -> I64Vector4: ...
    @property
    def rrrl(self) -> I64Vector4: ...
    @property
    def rrgr(self) -> I64Vector4: ...
    @property
    def rrgg(self) -> I64Vector4: ...
    @property
    def rrgo(self) -> I64Vector4: ...
    @property
    def rrgl(self) -> I64Vector4: ...
    @property
    def rror(self) -> I64Vector4: ...
    @property
    def rrog(self) -> I64Vector4: ...
    @property
    def rroo(self) -> I64Vector4: ...
    @property
    def rrol(self) -> I64Vector4: ...
    @property
    def rrlr(self) -> I64Vector4: ...
    @property
    def rrlg(self) -> I64Vector4: ...
    @property
    def rrlo(self) -> I64Vector4: ...
    @property
    def rrll(self) -> I64Vector4: ...
    @property
    def rgrr(self) -> I64Vector4: ...
    @property
    def rgrg(self) -> I64Vector4: ...
    @property
    def rgro(self) -> I64Vector4: ...
    @property
    def rgrl(self) -> I64Vector4: ...
    @property
    def rggr(self) -> I64Vector4: ...
    @property
    def rggg(self) -> I64Vector4: ...
    @property
    def rggo(self) -> I64Vector4: ...
    @property
    def rggl(self) -> I64Vector4: ...
    @property
    def rgor(self) -> I64Vector4: ...
    @property
    def rgog(self) -> I64Vector4: ...
    @property
    def rgoo(self) -> I64Vector4: ...
    @property
    def rgol(self) -> I64Vector4: ...
    @property
    def rglr(self) -> I64Vector4: ...
    @property
    def rglg(self) -> I64Vector4: ...
    @property
    def rglo(self) -> I64Vector4: ...
    @property
    def rgll(self) -> I64Vector4: ...
    @property
    def rorr(self) -> I64Vector4: ...
    @property
    def rorg(self) -> I64Vector4: ...
    @property
    def roro(self) -> I64Vector4: ...
    @property
    def rorl(self) -> I64Vector4: ...
    @property
    def rogr(self) -> I64Vector4: ...
    @property
    def rogg(self) -> I64Vector4: ...
    @property
    def rogo(self) -> I64Vector4: ...
    @property
    def rogl(self) -> I64Vector4: ...
    @property
    def roor(self) -> I64Vector4: ...
    @property
    def roog(self) -> I64Vector4: ...
    @property
    def rooo(self) -> I64Vector4: ...
    @property
    def rool(self) -> I64Vector4: ...
    @property
    def rolr(self) -> I64Vector4: ...
    @property
    def rolg(self) -> I64Vector4: ...
    @property
    def rolo(self) -> I64Vector4: ...
    @property
    def roll(self) -> I64Vector4: ...
    @property
    def rlrr(self) -> I64Vector4: ...
    @property
    def rlrg(self) -> I64Vector4: ...
    @property
    def rlro(self) -> I64Vector4: ...
    @property
    def rlrl(self) -> I64Vector4: ...
    @property
    def rlgr(self) -> I64Vector4: ...
    @property
    def rlgg(self) -> I64Vector4: ...
    @property
    def rlgo(self) -> I64Vector4: ...
    @property
    def rlgl(self) -> I64Vector4: ...
    @property
    def rlor(self) -> I64Vector4: ...
    @property
    def rlog(self) -> I64Vector4: ...
    @property
    def rloo(self) -> I64Vector4: ...
    @property
    def rlol(self) -> I64Vector4: ...
    @property
    def rllr(self) -> I64Vector4: ...
    @property
    def rllg(self) -> I64Vector4: ...
    @property
    def rllo(self) -> I64Vector4: ...
    @property
    def rlll(self) -> I64Vector4: ...
    @property
    def grrr(self) -> I64Vector4: ...
    @property
    def grrg(self) -> I64Vector4: ...
    @property
    def grro(self) -> I64Vector4: ...
    @property
    def grrl(self) -> I64Vector4: ...
    @property
    def grgr(self) -> I64Vector4: ...
    @property
    def grgg(self) -> I64Vector4: ...
    @property
    def grgo(self) -> I64Vector4: ...
    @property
    def grgl(self) -> I64Vector4: ...
    @property
    def gror(self) -> I64Vector4: ...
    @property
    def grog(self) -> I64Vector4: ...
    @property
    def groo(self) -> I64Vector4: ...
    @property
    def grol(self) -> I64Vector4: ...
    @property
    def grlr(self) -> I64Vector4: ...
    @property
    def grlg(self) -> I64Vector4: ...
    @property
    def grlo(self) -> I64Vector4: ...
    @property
    def grll(self) -> I64Vector4: ...
    @property
    def ggrr(self) -> I64Vector4: ...
    @property
    def ggrg(self) -> I64Vector4: ...
    @property
    def ggro(self) -> I64Vector4: ...
    @property
    def ggrl(self) -> I64Vector4: ...
    @property
    def gggr(self) -> I64Vector4: ...
    @property
    def gggg(self) -> I64Vector4: ...
    @property
    def gggo(self) -> I64Vector4: ...
    @property
    def gggl(self) -> I64Vector4: ...
    @property
    def ggor(self) -> I64Vector4: ...
    @property
    def ggog(self) -> I64Vector4: ...
    @property
    def ggoo(self) -> I64Vector4: ...
    @property
    def ggol(self) -> I64Vector4: ...
    @property
    def gglr(self) -> I64Vector4: ...
    @property
    def gglg(self) -> I64Vector4: ...
    @property
    def gglo(self) -> I64Vector4: ...
    @property
    def ggll(self) -> I64Vector4: ...
    @property
    def gorr(self) -> I64Vector4: ...
    @property
    def gorg(self) -> I64Vector4: ...
    @property
    def goro(self) -> I64Vector4: ...
    @property
    def gorl(self) -> I64Vector4: ...
    @property
    def gogr(self) -> I64Vector4: ...
    @property
    def gogg(self) -> I64Vector4: ...
    @property
    def gogo(self) -> I64Vector4: ...
    @property
    def gogl(self) -> I64Vector4: ...
    @property
    def goor(self) -> I64Vector4: ...
    @property
    def goog(self) -> I64Vector4: ...
    @property
    def gooo(self) -> I64Vector4: ...
    @property
    def gool(self) -> I64Vector4: ...
    @property
    def golr(self) -> I64Vector4: ...
    @property
    def golg(self) -> I64Vector4: ...
    @property
    def golo(self) -> I64Vector4: ...
    @property
    def goll(self) -> I64Vector4: ...
    @property
    def glrr(self) -> I64Vector4: ...
    @property
    def glrg(self) -> I64Vector4: ...
    @property
    def glro(self) -> I64Vector4: ...
    @property
    def glrl(self) -> I64Vector4: ...
    @property
    def glgr(self) -> I64Vector4: ...
    @property
    def glgg(self) -> I64Vector4: ...
    @property
    def glgo(self) -> I64Vector4: ...
    @property
    def glgl(self) -> I64Vector4: ...
    @property
    def glor(self) -> I64Vector4: ...
    @property
    def glog(self) -> I64Vector4: ...
    @property
    def gloo(self) -> I64Vector4: ...
    @property
    def glol(self) -> I64Vector4: ...
    @property
    def gllr(self) -> I64Vector4: ...
    @property
    def gllg(self) -> I64Vector4: ...
    @property
    def gllo(self) -> I64Vector4: ...
    @property
    def glll(self) -> I64Vector4: ...
    @property
    def orrr(self) -> I64Vector4: ...
    @property
    def orrg(self) -> I64Vector4: ...
    @property
    def orro(self) -> I64Vector4: ...
    @property
    def orrl(self) -> I64Vector4: ...
    @property
    def orgr(self) -> I64Vector4: ...
    @property
    def orgg(self) -> I64Vector4: ...
    @property
    def orgo(self) -> I64Vector4: ...
    @property
    def orgl(self) -> I64Vector4: ...
    @property
    def oror(self) -> I64Vector4: ...
    @property
    def orog(self) -> I64Vector4: ...
    @property
    def oroo(self) -> I64Vector4: ...
    @property
    def orol(self) -> I64Vector4: ...
    @property
    def orlr(self) -> I64Vector4: ...
    @property
    def orlg(self) -> I64Vector4: ...
    @property
    def orlo(self) -> I64Vector4: ...
    @property
    def orll(self) -> I64Vector4: ...
    @property
    def ogrr(self) -> I64Vector4: ...
    @property
    def ogrg(self) -> I64Vector4: ...
    @property
    def ogro(self) -> I64Vector4: ...
    @property
    def ogrl(self) -> I64Vector4: ...
    @property
    def oggr(self) -> I64Vector4: ...
    @property
    def oggg(self) -> I64Vector4: ...
    @property
    def oggo(self) -> I64Vector4: ...
    @property
    def oggl(self) -> I64Vector4: ...
    @property
    def ogor(self) -> I64Vector4: ...
    @property
    def ogog(self) -> I64Vector4: ...
    @property
    def ogoo(self) -> I64Vector4: ...
    @property
    def ogol(self) -> I64Vector4: ...
    @property
    def oglr(self) -> I64Vector4: ...
    @property
    def oglg(self) -> I64Vector4: ...
    @property
    def oglo(self) -> I64Vector4: ...
    @property
    def ogll(self) -> I64Vector4: ...
    @property
    def oorr(self) -> I64Vector4: ...
    @property
    def oorg(self) -> I64Vector4: ...
    @property
    def ooro(self) -> I64Vector4: ...
    @property
    def oorl(self) -> I64Vector4: ...
    @property
    def oogr(self) -> I64Vector4: ...
    @property
    def oogg(self) -> I64Vector4: ...
    @property
    def oogo(self) -> I64Vector4: ...
    @property
    def oogl(self) -> I64Vector4: ...
    @property
    def ooor(self) -> I64Vector4: ...
    @property
    def ooog(self) -> I64Vector4: ...
    @property
    def oolr(self) -> I64Vector4: ...
    @property
    def oolg(self) -> I64Vector4: ...
    @property
    def olrr(self) -> I64Vector4: ...
    @property
    def olrg(self) -> I64Vector4: ...
    @property
    def olro(self) -> I64Vector4: ...
    @property
    def olrl(self) -> I64Vector4: ...
    @property
    def olgr(self) -> I64Vector4: ...
    @property
    def olgg(self) -> I64Vector4: ...
    @property
    def olgo(self) -> I64Vector4: ...
    @property
    def olgl(self) -> I64Vector4: ...
    @property
    def olor(self) -> I64Vector4: ...
    @property
    def olog(self) -> I64Vector4: ...
    @property
    def ollr(self) -> I64Vector4: ...
    @property
    def ollg(self) -> I64Vector4: ...
    @property
    def lrrr(self) -> I64Vector4: ...
    @property
    def lrrg(self) -> I64Vector4: ...
    @property
    def lrro(self) -> I64Vector4: ...
    @property
    def lrrl(self) -> I64Vector4: ...
    @property
    def lrgr(self) -> I64Vector4: ...
    @property
    def lrgg(self) -> I64Vector4: ...
    @property
    def lrgo(self) -> I64Vector4: ...
    @property
    def lrgl(self) -> I64Vector4: ...
    @property
    def lror(self) -> I64Vector4: ...
    @property
    def lrog(self) -> I64Vector4: ...
    @property
    def lroo(self) -> I64Vector4: ...
    @property
    def lrol(self) -> I64Vector4: ...
    @property
    def lrlr(self) -> I64Vector4: ...
    @property
    def lrlg(self) -> I64Vector4: ...
    @property
    def lrlo(self) -> I64Vector4: ...
    @property
    def lrll(self) -> I64Vector4: ...
    @property
    def lgrr(self) -> I64Vector4: ...
    @property
    def lgrg(self) -> I64Vector4: ...
    @property
    def lgro(self) -> I64Vector4: ...
    @property
    def lgrl(self) -> I64Vector4: ...
    @property
    def lggr(self) -> I64Vector4: ...
    @property
    def lggg(self) -> I64Vector4: ...
    @property
    def lggo(self) -> I64Vector4: ...
    @property
    def lggl(self) -> I64Vector4: ...
    @property
    def lgor(self) -> I64Vector4: ...
    @property
    def lgog(self) -> I64Vector4: ...
    @property
    def lgoo(self) -> I64Vector4: ...
    @property
    def lgol(self) -> I64Vector4: ...
    @property
    def lglr(self) -> I64Vector4: ...
    @property
    def lglg(self) -> I64Vector4: ...
    @property
    def lglo(self) -> I64Vector4: ...
    @property
    def lgll(self) -> I64Vector4: ...
    @property
    def lorr(self) -> I64Vector4: ...
    @property
    def lorg(self) -> I64Vector4: ...
    @property
    def loro(self) -> I64Vector4: ...
    @property
    def lorl(self) -> I64Vector4: ...
    @property
    def logr(self) -> I64Vector4: ...
    @property
    def logg(self) -> I64Vector4: ...
    @property
    def logo(self) -> I64Vector4: ...
    @property
    def logl(self) -> I64Vector4: ...
    @property
    def loor(self) -> I64Vector4: ...
    @property
    def loog(self) -> I64Vector4: ...
    @property
    def lolr(self) -> I64Vector4: ...
    @property
    def lolg(self) -> I64Vector4: ...
    @property
    def llrr(self) -> I64Vector4: ...
    @property
    def llrg(self) -> I64Vector4: ...
    @property
    def llro(self) -> I64Vector4: ...
    @property
    def llrl(self) -> I64Vector4: ...
    @property
    def llgr(self) -> I64Vector4: ...
    @property
    def llgg(self) -> I64Vector4: ...
    @property
    def llgo(self) -> I64Vector4: ...
    @property
    def llgl(self) -> I64Vector4: ...
    @property
    def llor(self) -> I64Vector4: ...
    @property
    def llog(self) -> I64Vector4: ...
    @property
    def lllr(self) -> I64Vector4: ...
    @property
    def lllg(self) -> I64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> I64Vector2: ...
    @property
    def st(self) -> I64Vector2: ...
    @property
    def so(self) -> I64Vector2: ...
    @property
    def sl(self) -> I64Vector2: ...
    @property
    def ts(self) -> I64Vector2: ...
    @property
    def tt(self) -> I64Vector2: ...
    @property
    def to(self) -> I64Vector2: ...
    @property
    def tl(self) -> I64Vector2: ...
    @property
    def os(self) -> I64Vector2: ...
    @property
    def ot(self) -> I64Vector2: ...
    @property
    def ls(self) -> I64Vector2: ...
    @property
    def lt(self) -> I64Vector2: ...
    @property
    def sss(self) -> I64Vector3: ...
    @property
    def sst(self) -> I64Vector3: ...
    @property
    def sso(self) -> I64Vector3: ...
    @property
    def ssl(self) -> I64Vector3: ...
    @property
    def sts(self) -> I64Vector3: ...
    @property
    def stt(self) -> I64Vector3: ...
    @property
    def sto(self) -> I64Vector3: ...
    @property
    def stl(self) -> I64Vector3: ...
    @property
    def sos(self) -> I64Vector3: ...
    @property
    def sot(self) -> I64Vector3: ...
    @property
    def soo(self) -> I64Vector3: ...
    @property
    def sol(self) -> I64Vector3: ...
    @property
    def sls(self) -> I64Vector3: ...
    @property
    def slt(self) -> I64Vector3: ...
    @property
    def slo(self) -> I64Vector3: ...
    @property
    def sll(self) -> I64Vector3: ...
    @property
    def tss(self) -> I64Vector3: ...
    @property
    def tst(self) -> I64Vector3: ...
    @property
    def tso(self) -> I64Vector3: ...
    @property
    def tsl(self) -> I64Vector3: ...
    @property
    def tts(self) -> I64Vector3: ...
    @property
    def ttt(self) -> I64Vector3: ...
    @property
    def tto(self) -> I64Vector3: ...
    @property
    def ttl(self) -> I64Vector3: ...
    @property
    def tos(self) -> I64Vector3: ...
    @property
    def tot(self) -> I64Vector3: ...
    @property
    def too(self) -> I64Vector3: ...
    @property
    def tol(self) -> I64Vector3: ...
    @property
    def tls(self) -> I64Vector3: ...
    @property
    def tlt(self) -> I64Vector3: ...
    @property
    def tlo(self) -> I64Vector3: ...
    @property
    def tll(self) -> I64Vector3: ...
    @property
    def oss(self) -> I64Vector3: ...
    @property
    def ost(self) -> I64Vector3: ...
    @property
    def oso(self) -> I64Vector3: ...
    @property
    def osl(self) -> I64Vector3: ...
    @property
    def ots(self) -> I64Vector3: ...
    @property
    def ott(self) -> I64Vector3: ...
    @property
    def oto(self) -> I64Vector3: ...
    @property
    def otl(self) -> I64Vector3: ...
    @property
    def oos(self) -> I64Vector3: ...
    @property
    def oot(self) -> I64Vector3: ...
    @property
    def ols(self) -> I64Vector3: ...
    @property
    def olt(self) -> I64Vector3: ...
    @property
    def lss(self) -> I64Vector3: ...
    @property
    def lst(self) -> I64Vector3: ...
    @property
    def lso(self) -> I64Vector3: ...
    @property
    def lsl(self) -> I64Vector3: ...
    @property
    def lts(self) -> I64Vector3: ...
    @property
    def ltt(self) -> I64Vector3: ...
    @property
    def lto(self) -> I64Vector3: ...
    @property
    def ltl(self) -> I64Vector3: ...
    @property
    def los(self) -> I64Vector3: ...
    @property
    def lot(self) -> I64Vector3: ...
    @property
    def lls(self) -> I64Vector3: ...
    @property
    def llt(self) -> I64Vector3: ...
    @property
    def ssss(self) -> I64Vector4: ...
    @property
    def ssst(self) -> I64Vector4: ...
    @property
    def ssso(self) -> I64Vector4: ...
    @property
    def sssl(self) -> I64Vector4: ...
    @property
    def ssts(self) -> I64Vector4: ...
    @property
    def sstt(self) -> I64Vector4: ...
    @property
    def ssto(self) -> I64Vector4: ...
    @property
    def sstl(self) -> I64Vector4: ...
    @property
    def ssos(self) -> I64Vector4: ...
    @property
    def ssot(self) -> I64Vector4: ...
    @property
    def ssoo(self) -> I64Vector4: ...
    @property
    def ssol(self) -> I64Vector4: ...
    @property
    def ssls(self) -> I64Vector4: ...
    @property
    def sslt(self) -> I64Vector4: ...
    @property
    def sslo(self) -> I64Vector4: ...
    @property
    def ssll(self) -> I64Vector4: ...
    @property
    def stss(self) -> I64Vector4: ...
    @property
    def stst(self) -> I64Vector4: ...
    @property
    def stso(self) -> I64Vector4: ...
    @property
    def stsl(self) -> I64Vector4: ...
    @property
    def stts(self) -> I64Vector4: ...
    @property
    def sttt(self) -> I64Vector4: ...
    @property
    def stto(self) -> I64Vector4: ...
    @property
    def sttl(self) -> I64Vector4: ...
    @property
    def stos(self) -> I64Vector4: ...
    @property
    def stot(self) -> I64Vector4: ...
    @property
    def stoo(self) -> I64Vector4: ...
    @property
    def stol(self) -> I64Vector4: ...
    @property
    def stls(self) -> I64Vector4: ...
    @property
    def stlt(self) -> I64Vector4: ...
    @property
    def stlo(self) -> I64Vector4: ...
    @property
    def stll(self) -> I64Vector4: ...
    @property
    def soss(self) -> I64Vector4: ...
    @property
    def sost(self) -> I64Vector4: ...
    @property
    def soso(self) -> I64Vector4: ...
    @property
    def sosl(self) -> I64Vector4: ...
    @property
    def sots(self) -> I64Vector4: ...
    @property
    def sott(self) -> I64Vector4: ...
    @property
    def soto(self) -> I64Vector4: ...
    @property
    def sotl(self) -> I64Vector4: ...
    @property
    def soos(self) -> I64Vector4: ...
    @property
    def soot(self) -> I64Vector4: ...
    @property
    def sooo(self) -> I64Vector4: ...
    @property
    def sool(self) -> I64Vector4: ...
    @property
    def sols(self) -> I64Vector4: ...
    @property
    def solt(self) -> I64Vector4: ...
    @property
    def solo(self) -> I64Vector4: ...
    @property
    def soll(self) -> I64Vector4: ...
    @property
    def slss(self) -> I64Vector4: ...
    @property
    def slst(self) -> I64Vector4: ...
    @property
    def slso(self) -> I64Vector4: ...
    @property
    def slsl(self) -> I64Vector4: ...
    @property
    def slts(self) -> I64Vector4: ...
    @property
    def sltt(self) -> I64Vector4: ...
    @property
    def slto(self) -> I64Vector4: ...
    @property
    def sltl(self) -> I64Vector4: ...
    @property
    def slos(self) -> I64Vector4: ...
    @property
    def slot(self) -> I64Vector4: ...
    @property
    def sloo(self) -> I64Vector4: ...
    @property
    def slol(self) -> I64Vector4: ...
    @property
    def slls(self) -> I64Vector4: ...
    @property
    def sllt(self) -> I64Vector4: ...
    @property
    def sllo(self) -> I64Vector4: ...
    @property
    def slll(self) -> I64Vector4: ...
    @property
    def tsss(self) -> I64Vector4: ...
    @property
    def tsst(self) -> I64Vector4: ...
    @property
    def tsso(self) -> I64Vector4: ...
    @property
    def tssl(self) -> I64Vector4: ...
    @property
    def tsts(self) -> I64Vector4: ...
    @property
    def tstt(self) -> I64Vector4: ...
    @property
    def tsto(self) -> I64Vector4: ...
    @property
    def tstl(self) -> I64Vector4: ...
    @property
    def tsos(self) -> I64Vector4: ...
    @property
    def tsot(self) -> I64Vector4: ...
    @property
    def tsoo(self) -> I64Vector4: ...
    @property
    def tsol(self) -> I64Vector4: ...
    @property
    def tsls(self) -> I64Vector4: ...
    @property
    def tslt(self) -> I64Vector4: ...
    @property
    def tslo(self) -> I64Vector4: ...
    @property
    def tsll(self) -> I64Vector4: ...
    @property
    def ttss(self) -> I64Vector4: ...
    @property
    def ttst(self) -> I64Vector4: ...
    @property
    def ttso(self) -> I64Vector4: ...
    @property
    def ttsl(self) -> I64Vector4: ...
    @property
    def ttts(self) -> I64Vector4: ...
    @property
    def tttt(self) -> I64Vector4: ...
    @property
    def ttto(self) -> I64Vector4: ...
    @property
    def tttl(self) -> I64Vector4: ...
    @property
    def ttos(self) -> I64Vector4: ...
    @property
    def ttot(self) -> I64Vector4: ...
    @property
    def ttoo(self) -> I64Vector4: ...
    @property
    def ttol(self) -> I64Vector4: ...
    @property
    def ttls(self) -> I64Vector4: ...
    @property
    def ttlt(self) -> I64Vector4: ...
    @property
    def ttlo(self) -> I64Vector4: ...
    @property
    def ttll(self) -> I64Vector4: ...
    @property
    def toss(self) -> I64Vector4: ...
    @property
    def tost(self) -> I64Vector4: ...
    @property
    def toso(self) -> I64Vector4: ...
    @property
    def tosl(self) -> I64Vector4: ...
    @property
    def tots(self) -> I64Vector4: ...
    @property
    def tott(self) -> I64Vector4: ...
    @property
    def toto(self) -> I64Vector4: ...
    @property
    def totl(self) -> I64Vector4: ...
    @property
    def toos(self) -> I64Vector4: ...
    @property
    def toot(self) -> I64Vector4: ...
    @property
    def tooo(self) -> I64Vector4: ...
    @property
    def tool(self) -> I64Vector4: ...
    @property
    def tols(self) -> I64Vector4: ...
    @property
    def tolt(self) -> I64Vector4: ...
    @property
    def tolo(self) -> I64Vector4: ...
    @property
    def toll(self) -> I64Vector4: ...
    @property
    def tlss(self) -> I64Vector4: ...
    @property
    def tlst(self) -> I64Vector4: ...
    @property
    def tlso(self) -> I64Vector4: ...
    @property
    def tlsl(self) -> I64Vector4: ...
    @property
    def tlts(self) -> I64Vector4: ...
    @property
    def tltt(self) -> I64Vector4: ...
    @property
    def tlto(self) -> I64Vector4: ...
    @property
    def tltl(self) -> I64Vector4: ...
    @property
    def tlos(self) -> I64Vector4: ...
    @property
    def tlot(self) -> I64Vector4: ...
    @property
    def tloo(self) -> I64Vector4: ...
    @property
    def tlol(self) -> I64Vector4: ...
    @property
    def tlls(self) -> I64Vector4: ...
    @property
    def tllt(self) -> I64Vector4: ...
    @property
    def tllo(self) -> I64Vector4: ...
    @property
    def tlll(self) -> I64Vector4: ...
    @property
    def osss(self) -> I64Vector4: ...
    @property
    def osst(self) -> I64Vector4: ...
    @property
    def osso(self) -> I64Vector4: ...
    @property
    def ossl(self) -> I64Vector4: ...
    @property
    def osts(self) -> I64Vector4: ...
    @property
    def ostt(self) -> I64Vector4: ...
    @property
    def osto(self) -> I64Vector4: ...
    @property
    def ostl(self) -> I64Vector4: ...
    @property
    def osos(self) -> I64Vector4: ...
    @property
    def osot(self) -> I64Vector4: ...
    @property
    def osoo(self) -> I64Vector4: ...
    @property
    def osol(self) -> I64Vector4: ...
    @property
    def osls(self) -> I64Vector4: ...
    @property
    def oslt(self) -> I64Vector4: ...
    @property
    def oslo(self) -> I64Vector4: ...
    @property
    def osll(self) -> I64Vector4: ...
    @property
    def otss(self) -> I64Vector4: ...
    @property
    def otst(self) -> I64Vector4: ...
    @property
    def otso(self) -> I64Vector4: ...
    @property
    def otsl(self) -> I64Vector4: ...
    @property
    def otts(self) -> I64Vector4: ...
    @property
    def ottt(self) -> I64Vector4: ...
    @property
    def otto(self) -> I64Vector4: ...
    @property
    def ottl(self) -> I64Vector4: ...
    @property
    def otos(self) -> I64Vector4: ...
    @property
    def otot(self) -> I64Vector4: ...
    @property
    def otoo(self) -> I64Vector4: ...
    @property
    def otol(self) -> I64Vector4: ...
    @property
    def otls(self) -> I64Vector4: ...
    @property
    def otlt(self) -> I64Vector4: ...
    @property
    def otlo(self) -> I64Vector4: ...
    @property
    def otll(self) -> I64Vector4: ...
    @property
    def ooss(self) -> I64Vector4: ...
    @property
    def oost(self) -> I64Vector4: ...
    @property
    def ooso(self) -> I64Vector4: ...
    @property
    def oosl(self) -> I64Vector4: ...
    @property
    def oots(self) -> I64Vector4: ...
    @property
    def oott(self) -> I64Vector4: ...
    @property
    def ooto(self) -> I64Vector4: ...
    @property
    def ootl(self) -> I64Vector4: ...
    @property
    def ooos(self) -> I64Vector4: ...
    @property
    def ooot(self) -> I64Vector4: ...
    @property
    def ools(self) -> I64Vector4: ...
    @property
    def oolt(self) -> I64Vector4: ...
    @property
    def olss(self) -> I64Vector4: ...
    @property
    def olst(self) -> I64Vector4: ...
    @property
    def olso(self) -> I64Vector4: ...
    @property
    def olsl(self) -> I64Vector4: ...
    @property
    def olts(self) -> I64Vector4: ...
    @property
    def oltt(self) -> I64Vector4: ...
    @property
    def olto(self) -> I64Vector4: ...
    @property
    def oltl(self) -> I64Vector4: ...
    @property
    def olos(self) -> I64Vector4: ...
    @property
    def olot(self) -> I64Vector4: ...
    @property
    def olls(self) -> I64Vector4: ...
    @property
    def ollt(self) -> I64Vector4: ...
    @property
    def lsss(self) -> I64Vector4: ...
    @property
    def lsst(self) -> I64Vector4: ...
    @property
    def lsso(self) -> I64Vector4: ...
    @property
    def lssl(self) -> I64Vector4: ...
    @property
    def lsts(self) -> I64Vector4: ...
    @property
    def lstt(self) -> I64Vector4: ...
    @property
    def lsto(self) -> I64Vector4: ...
    @property
    def lstl(self) -> I64Vector4: ...
    @property
    def lsos(self) -> I64Vector4: ...
    @property
    def lsot(self) -> I64Vector4: ...
    @property
    def lsoo(self) -> I64Vector4: ...
    @property
    def lsol(self) -> I64Vector4: ...
    @property
    def lsls(self) -> I64Vector4: ...
    @property
    def lslt(self) -> I64Vector4: ...
    @property
    def lslo(self) -> I64Vector4: ...
    @property
    def lsll(self) -> I64Vector4: ...
    @property
    def ltss(self) -> I64Vector4: ...
    @property
    def ltst(self) -> I64Vector4: ...
    @property
    def ltso(self) -> I64Vector4: ...
    @property
    def ltsl(self) -> I64Vector4: ...
    @property
    def ltts(self) -> I64Vector4: ...
    @property
    def lttt(self) -> I64Vector4: ...
    @property
    def ltto(self) -> I64Vector4: ...
    @property
    def lttl(self) -> I64Vector4: ...
    @property
    def ltos(self) -> I64Vector4: ...
    @property
    def ltot(self) -> I64Vector4: ...
    @property
    def ltoo(self) -> I64Vector4: ...
    @property
    def ltol(self) -> I64Vector4: ...
    @property
    def ltls(self) -> I64Vector4: ...
    @property
    def ltlt(self) -> I64Vector4: ...
    @property
    def ltlo(self) -> I64Vector4: ...
    @property
    def ltll(self) -> I64Vector4: ...
    @property
    def loss(self) -> I64Vector4: ...
    @property
    def lost(self) -> I64Vector4: ...
    @property
    def loso(self) -> I64Vector4: ...
    @property
    def losl(self) -> I64Vector4: ...
    @property
    def lots(self) -> I64Vector4: ...
    @property
    def lott(self) -> I64Vector4: ...
    @property
    def loto(self) -> I64Vector4: ...
    @property
    def lotl(self) -> I64Vector4: ...
    @property
    def loos(self) -> I64Vector4: ...
    @property
    def loot(self) -> I64Vector4: ...
    @property
    def lols(self) -> I64Vector4: ...
    @property
    def lolt(self) -> I64Vector4: ...
    @property
    def llss(self) -> I64Vector4: ...
    @property
    def llst(self) -> I64Vector4: ...
    @property
    def llso(self) -> I64Vector4: ...
    @property
    def llsl(self) -> I64Vector4: ...
    @property
    def llts(self) -> I64Vector4: ...
    @property
    def lltt(self) -> I64Vector4: ...
    @property
    def llto(self) -> I64Vector4: ...
    @property
    def lltl(self) -> I64Vector4: ...
    @property
    def llos(self) -> I64Vector4: ...
    @property
    def llot(self) -> I64Vector4: ...
    @property
    def llls(self) -> I64Vector4: ...
    @property
    def lllt(self) -> I64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I64Vector2: ...
    @property
    def uv(self) -> I64Vector2: ...
    @property
    def uo(self) -> I64Vector2: ...
    @property
    def ul(self) -> I64Vector2: ...
    @property
    def vu(self) -> I64Vector2: ...
    @property
    def vv(self) -> I64Vector2: ...
    @property
    def vo(self) -> I64Vector2: ...
    @property
    def vl(self) -> I64Vector2: ...
    @property
    def ou(self) -> I64Vector2: ...
    @property
    def ov(self) -> I64Vector2: ...
    @property
    def lu(self) -> I64Vector2: ...
    @property
    def lv(self) -> I64Vector2: ...
    @property
    def uuu(self) -> I64Vector3: ...
    @property
    def uuv(self) -> I64Vector3: ...
    @property
    def uuo(self) -> I64Vector3: ...
    @property
    def uul(self) -> I64Vector3: ...
    @property
    def uvu(self) -> I64Vector3: ...
    @property
    def uvv(self) -> I64Vector3: ...
    @property
    def uvo(self) -> I64Vector3: ...
    @property
    def uvl(self) -> I64Vector3: ...
    @property
    def uou(self) -> I64Vector3: ...
    @property
    def uov(self) -> I64Vector3: ...
    @property
    def uoo(self) -> I64Vector3: ...
    @property
    def uol(self) -> I64Vector3: ...
    @property
    def ulu(self) -> I64Vector3: ...
    @property
    def ulv(self) -> I64Vector3: ...
    @property
    def ulo(self) -> I64Vector3: ...
    @property
    def ull(self) -> I64Vector3: ...
    @property
    def vuu(self) -> I64Vector3: ...
    @property
    def vuv(self) -> I64Vector3: ...
    @property
    def vuo(self) -> I64Vector3: ...
    @property
    def vul(self) -> I64Vector3: ...
    @property
    def vvu(self) -> I64Vector3: ...
    @property
    def vvv(self) -> I64Vector3: ...
    @property
    def vvo(self) -> I64Vector3: ...
    @property
    def vvl(self) -> I64Vector3: ...
    @property
    def vou(self) -> I64Vector3: ...
    @property
    def vov(self) -> I64Vector3: ...
    @property
    def voo(self) -> I64Vector3: ...
    @property
    def vol(self) -> I64Vector3: ...
    @property
    def vlu(self) -> I64Vector3: ...
    @property
    def vlv(self) -> I64Vector3: ...
    @property
    def vlo(self) -> I64Vector3: ...
    @property
    def vll(self) -> I64Vector3: ...
    @property
    def ouu(self) -> I64Vector3: ...
    @property
    def ouv(self) -> I64Vector3: ...
    @property
    def ouo(self) -> I64Vector3: ...
    @property
    def oul(self) -> I64Vector3: ...
    @property
    def ovu(self) -> I64Vector3: ...
    @property
    def ovv(self) -> I64Vector3: ...
    @property
    def ovo(self) -> I64Vector3: ...
    @property
    def ovl(self) -> I64Vector3: ...
    @property
    def oou(self) -> I64Vector3: ...
    @property
    def oov(self) -> I64Vector3: ...
    @property
    def olu(self) -> I64Vector3: ...
    @property
    def olv(self) -> I64Vector3: ...
    @property
    def luu(self) -> I64Vector3: ...
    @property
    def luv(self) -> I64Vector3: ...
    @property
    def luo(self) -> I64Vector3: ...
    @property
    def lul(self) -> I64Vector3: ...
    @property
    def lvu(self) -> I64Vector3: ...
    @property
    def lvv(self) -> I64Vector3: ...
    @property
    def lvo(self) -> I64Vector3: ...
    @property
    def lvl(self) -> I64Vector3: ...
    @property
    def lou(self) -> I64Vector3: ...
    @property
    def lov(self) -> I64Vector3: ...
    @property
    def llu(self) -> I64Vector3: ...
    @property
    def llv(self) -> I64Vector3: ...
    @property
    def uuuu(self) -> I64Vector4: ...
    @property
    def uuuv(self) -> I64Vector4: ...
    @property
    def uuuo(self) -> I64Vector4: ...
    @property
    def uuul(self) -> I64Vector4: ...
    @property
    def uuvu(self) -> I64Vector4: ...
    @property
    def uuvv(self) -> I64Vector4: ...
    @property
    def uuvo(self) -> I64Vector4: ...
    @property
    def uuvl(self) -> I64Vector4: ...
    @property
    def uuou(self) -> I64Vector4: ...
    @property
    def uuov(self) -> I64Vector4: ...
    @property
    def uuoo(self) -> I64Vector4: ...
    @property
    def uuol(self) -> I64Vector4: ...
    @property
    def uulu(self) -> I64Vector4: ...
    @property
    def uulv(self) -> I64Vector4: ...
    @property
    def uulo(self) -> I64Vector4: ...
    @property
    def uull(self) -> I64Vector4: ...
    @property
    def uvuu(self) -> I64Vector4: ...
    @property
    def uvuv(self) -> I64Vector4: ...
    @property
    def uvuo(self) -> I64Vector4: ...
    @property
    def uvul(self) -> I64Vector4: ...
    @property
    def uvvu(self) -> I64Vector4: ...
    @property
    def uvvv(self) -> I64Vector4: ...
    @property
    def uvvo(self) -> I64Vector4: ...
    @property
    def uvvl(self) -> I64Vector4: ...
    @property
    def uvou(self) -> I64Vector4: ...
    @property
    def uvov(self) -> I64Vector4: ...
    @property
    def uvoo(self) -> I64Vector4: ...
    @property
    def uvol(self) -> I64Vector4: ...
    @property
    def uvlu(self) -> I64Vector4: ...
    @property
    def uvlv(self) -> I64Vector4: ...
    @property
    def uvlo(self) -> I64Vector4: ...
    @property
    def uvll(self) -> I64Vector4: ...
    @property
    def uouu(self) -> I64Vector4: ...
    @property
    def uouv(self) -> I64Vector4: ...
    @property
    def uouo(self) -> I64Vector4: ...
    @property
    def uoul(self) -> I64Vector4: ...
    @property
    def uovu(self) -> I64Vector4: ...
    @property
    def uovv(self) -> I64Vector4: ...
    @property
    def uovo(self) -> I64Vector4: ...
    @property
    def uovl(self) -> I64Vector4: ...
    @property
    def uoou(self) -> I64Vector4: ...
    @property
    def uoov(self) -> I64Vector4: ...
    @property
    def uooo(self) -> I64Vector4: ...
    @property
    def uool(self) -> I64Vector4: ...
    @property
    def uolu(self) -> I64Vector4: ...
    @property
    def uolv(self) -> I64Vector4: ...
    @property
    def uolo(self) -> I64Vector4: ...
    @property
    def uoll(self) -> I64Vector4: ...
    @property
    def uluu(self) -> I64Vector4: ...
    @property
    def uluv(self) -> I64Vector4: ...
    @property
    def uluo(self) -> I64Vector4: ...
    @property
    def ulul(self) -> I64Vector4: ...
    @property
    def ulvu(self) -> I64Vector4: ...
    @property
    def ulvv(self) -> I64Vector4: ...
    @property
    def ulvo(self) -> I64Vector4: ...
    @property
    def ulvl(self) -> I64Vector4: ...
    @property
    def ulou(self) -> I64Vector4: ...
    @property
    def ulov(self) -> I64Vector4: ...
    @property
    def uloo(self) -> I64Vector4: ...
    @property
    def ulol(self) -> I64Vector4: ...
    @property
    def ullu(self) -> I64Vector4: ...
    @property
    def ullv(self) -> I64Vector4: ...
    @property
    def ullo(self) -> I64Vector4: ...
    @property
    def ulll(self) -> I64Vector4: ...
    @property
    def vuuu(self) -> I64Vector4: ...
    @property
    def vuuv(self) -> I64Vector4: ...
    @property
    def vuuo(self) -> I64Vector4: ...
    @property
    def vuul(self) -> I64Vector4: ...
    @property
    def vuvu(self) -> I64Vector4: ...
    @property
    def vuvv(self) -> I64Vector4: ...
    @property
    def vuvo(self) -> I64Vector4: ...
    @property
    def vuvl(self) -> I64Vector4: ...
    @property
    def vuou(self) -> I64Vector4: ...
    @property
    def vuov(self) -> I64Vector4: ...
    @property
    def vuoo(self) -> I64Vector4: ...
    @property
    def vuol(self) -> I64Vector4: ...
    @property
    def vulu(self) -> I64Vector4: ...
    @property
    def vulv(self) -> I64Vector4: ...
    @property
    def vulo(self) -> I64Vector4: ...
    @property
    def vull(self) -> I64Vector4: ...
    @property
    def vvuu(self) -> I64Vector4: ...
    @property
    def vvuv(self) -> I64Vector4: ...
    @property
    def vvuo(self) -> I64Vector4: ...
    @property
    def vvul(self) -> I64Vector4: ...
    @property
    def vvvu(self) -> I64Vector4: ...
    @property
    def vvvv(self) -> I64Vector4: ...
    @property
    def vvvo(self) -> I64Vector4: ...
    @property
    def vvvl(self) -> I64Vector4: ...
    @property
    def vvou(self) -> I64Vector4: ...
    @property
    def vvov(self) -> I64Vector4: ...
    @property
    def vvoo(self) -> I64Vector4: ...
    @property
    def vvol(self) -> I64Vector4: ...
    @property
    def vvlu(self) -> I64Vector4: ...
    @property
    def vvlv(self) -> I64Vector4: ...
    @property
    def vvlo(self) -> I64Vector4: ...
    @property
    def vvll(self) -> I64Vector4: ...
    @property
    def vouu(self) -> I64Vector4: ...
    @property
    def vouv(self) -> I64Vector4: ...
    @property
    def vouo(self) -> I64Vector4: ...
    @property
    def voul(self) -> I64Vector4: ...
    @property
    def vovu(self) -> I64Vector4: ...
    @property
    def vovv(self) -> I64Vector4: ...
    @property
    def vovo(self) -> I64Vector4: ...
    @property
    def vovl(self) -> I64Vector4: ...
    @property
    def voou(self) -> I64Vector4: ...
    @property
    def voov(self) -> I64Vector4: ...
    @property
    def vooo(self) -> I64Vector4: ...
    @property
    def vool(self) -> I64Vector4: ...
    @property
    def volu(self) -> I64Vector4: ...
    @property
    def volv(self) -> I64Vector4: ...
    @property
    def volo(self) -> I64Vector4: ...
    @property
    def voll(self) -> I64Vector4: ...
    @property
    def vluu(self) -> I64Vector4: ...
    @property
    def vluv(self) -> I64Vector4: ...
    @property
    def vluo(self) -> I64Vector4: ...
    @property
    def vlul(self) -> I64Vector4: ...
    @property
    def vlvu(self) -> I64Vector4: ...
    @property
    def vlvv(self) -> I64Vector4: ...
    @property
    def vlvo(self) -> I64Vector4: ...
    @property
    def vlvl(self) -> I64Vector4: ...
    @property
    def vlou(self) -> I64Vector4: ...
    @property
    def vlov(self) -> I64Vector4: ...
    @property
    def vloo(self) -> I64Vector4: ...
    @property
    def vlol(self) -> I64Vector4: ...
    @property
    def vllu(self) -> I64Vector4: ...
    @property
    def vllv(self) -> I64Vector4: ...
    @property
    def vllo(self) -> I64Vector4: ...
    @property
    def vlll(self) -> I64Vector4: ...
    @property
    def ouuu(self) -> I64Vector4: ...
    @property
    def ouuv(self) -> I64Vector4: ...
    @property
    def ouuo(self) -> I64Vector4: ...
    @property
    def ouul(self) -> I64Vector4: ...
    @property
    def ouvu(self) -> I64Vector4: ...
    @property
    def ouvv(self) -> I64Vector4: ...
    @property
    def ouvo(self) -> I64Vector4: ...
    @property
    def ouvl(self) -> I64Vector4: ...
    @property
    def ouou(self) -> I64Vector4: ...
    @property
    def ouov(self) -> I64Vector4: ...
    @property
    def ouoo(self) -> I64Vector4: ...
    @property
    def ouol(self) -> I64Vector4: ...
    @property
    def oulu(self) -> I64Vector4: ...
    @property
    def oulv(self) -> I64Vector4: ...
    @property
    def oulo(self) -> I64Vector4: ...
    @property
    def oull(self) -> I64Vector4: ...
    @property
    def ovuu(self) -> I64Vector4: ...
    @property
    def ovuv(self) -> I64Vector4: ...
    @property
    def ovuo(self) -> I64Vector4: ...
    @property
    def ovul(self) -> I64Vector4: ...
    @property
    def ovvu(self) -> I64Vector4: ...
    @property
    def ovvv(self) -> I64Vector4: ...
    @property
    def ovvo(self) -> I64Vector4: ...
    @property
    def ovvl(self) -> I64Vector4: ...
    @property
    def ovou(self) -> I64Vector4: ...
    @property
    def ovov(self) -> I64Vector4: ...
    @property
    def ovoo(self) -> I64Vector4: ...
    @property
    def ovol(self) -> I64Vector4: ...
    @property
    def ovlu(self) -> I64Vector4: ...
    @property
    def ovlv(self) -> I64Vector4: ...
    @property
    def ovlo(self) -> I64Vector4: ...
    @property
    def ovll(self) -> I64Vector4: ...
    @property
    def oouu(self) -> I64Vector4: ...
    @property
    def oouv(self) -> I64Vector4: ...
    @property
    def oouo(self) -> I64Vector4: ...
    @property
    def ooul(self) -> I64Vector4: ...
    @property
    def oovu(self) -> I64Vector4: ...
    @property
    def oovv(self) -> I64Vector4: ...
    @property
    def oovo(self) -> I64Vector4: ...
    @property
    def oovl(self) -> I64Vector4: ...
    @property
    def ooou(self) -> I64Vector4: ...
    @property
    def ooov(self) -> I64Vector4: ...
    @property
    def oolu(self) -> I64Vector4: ...
    @property
    def oolv(self) -> I64Vector4: ...
    @property
    def oluu(self) -> I64Vector4: ...
    @property
    def oluv(self) -> I64Vector4: ...
    @property
    def oluo(self) -> I64Vector4: ...
    @property
    def olul(self) -> I64Vector4: ...
    @property
    def olvu(self) -> I64Vector4: ...
    @property
    def olvv(self) -> I64Vector4: ...
    @property
    def olvo(self) -> I64Vector4: ...
    @property
    def olvl(self) -> I64Vector4: ...
    @property
    def olou(self) -> I64Vector4: ...
    @property
    def olov(self) -> I64Vector4: ...
    @property
    def ollu(self) -> I64Vector4: ...
    @property
    def ollv(self) -> I64Vector4: ...
    @property
    def luuu(self) -> I64Vector4: ...
    @property
    def luuv(self) -> I64Vector4: ...
    @property
    def luuo(self) -> I64Vector4: ...
    @property
    def luul(self) -> I64Vector4: ...
    @property
    def luvu(self) -> I64Vector4: ...
    @property
    def luvv(self) -> I64Vector4: ...
    @property
    def luvo(self) -> I64Vector4: ...
    @property
    def luvl(self) -> I64Vector4: ...
    @property
    def luou(self) -> I64Vector4: ...
    @property
    def luov(self) -> I64Vector4: ...
    @property
    def luoo(self) -> I64Vector4: ...
    @property
    def luol(self) -> I64Vector4: ...
    @property
    def lulu(self) -> I64Vector4: ...
    @property
    def lulv(self) -> I64Vector4: ...
    @property
    def lulo(self) -> I64Vector4: ...
    @property
    def lull(self) -> I64Vector4: ...
    @property
    def lvuu(self) -> I64Vector4: ...
    @property
    def lvuv(self) -> I64Vector4: ...
    @property
    def lvuo(self) -> I64Vector4: ...
    @property
    def lvul(self) -> I64Vector4: ...
    @property
    def lvvu(self) -> I64Vector4: ...
    @property
    def lvvv(self) -> I64Vector4: ...
    @property
    def lvvo(self) -> I64Vector4: ...
    @property
    def lvvl(self) -> I64Vector4: ...
    @property
    def lvou(self) -> I64Vector4: ...
    @property
    def lvov(self) -> I64Vector4: ...
    @property
    def lvoo(self) -> I64Vector4: ...
    @property
    def lvol(self) -> I64Vector4: ...
    @property
    def lvlu(self) -> I64Vector4: ...
    @property
    def lvlv(self) -> I64Vector4: ...
    @property
    def lvlo(self) -> I64Vector4: ...
    @property
    def lvll(self) -> I64Vector4: ...
    @property
    def louu(self) -> I64Vector4: ...
    @property
    def louv(self) -> I64Vector4: ...
    @property
    def louo(self) -> I64Vector4: ...
    @property
    def loul(self) -> I64Vector4: ...
    @property
    def lovu(self) -> I64Vector4: ...
    @property
    def lovv(self) -> I64Vector4: ...
    @property
    def lovo(self) -> I64Vector4: ...
    @property
    def lovl(self) -> I64Vector4: ...
    @property
    def loou(self) -> I64Vector4: ...
    @property
    def loov(self) -> I64Vector4: ...
    @property
    def lolu(self) -> I64Vector4: ...
    @property
    def lolv(self) -> I64Vector4: ...
    @property
    def lluu(self) -> I64Vector4: ...
    @property
    def lluv(self) -> I64Vector4: ...
    @property
    def lluo(self) -> I64Vector4: ...
    @property
    def llul(self) -> I64Vector4: ...
    @property
    def llvu(self) -> I64Vector4: ...
    @property
    def llvv(self) -> I64Vector4: ...
    @property
    def llvo(self) -> I64Vector4: ...
    @property
    def llvl(self) -> I64Vector4: ...
    @property
    def llou(self) -> I64Vector4: ...
    @property
    def llov(self) -> I64Vector4: ...
    @property
    def lllu(self) -> I64Vector4: ...
    @property
    def lllv(self) -> I64Vector4: ...
    @property
    def oo(self) -> I64Vector2: ...
    @property
    def ol(self) -> I64Vector2: ...
    @property
    def lo(self) -> I64Vector2: ...
    @property
    def ll(self) -> I64Vector2: ...
    @property
    def ooo(self) -> I64Vector3: ...
    @property
    def ool(self) -> I64Vector3: ...
    @property
    def olo(self) -> I64Vector3: ...
    @property
    def oll(self) -> I64Vector3: ...
    @property
    def loo(self) -> I64Vector3: ...
    @property
    def lol(self) -> I64Vector3: ...
    @property
    def llo(self) -> I64Vector3: ...
    @property
    def lll(self) -> I64Vector3: ...
    @property
    def oooo(self) -> I64Vector4: ...
    @property
    def oool(self) -> I64Vector4: ...
    @property
    def oolo(self) -> I64Vector4: ...
    @property
    def ooll(self) -> I64Vector4: ...
    @property
    def oloo(self) -> I64Vector4: ...
    @property
    def olol(self) -> I64Vector4: ...
    @property
    def ollo(self) -> I64Vector4: ...
    @property
    def olll(self) -> I64Vector4: ...
    @property
    def looo(self) -> I64Vector4: ...
    @property
    def lool(self) -> I64Vector4: ...
    @property
    def lolo(self) -> I64Vector4: ...
    @property
    def loll(self) -> I64Vector4: ...
    @property
    def lloo(self) -> I64Vector4: ...
    @property
    def llol(self) -> I64Vector4: ...
    @property
    def lllo(self) -> I64Vector4: ...
    @property
    def llll(self) -> I64Vector4: ...
    def min(self, n: Number, /) -> I64Vector2: ...
    def max(self, n: Number, /) -> I64Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> I64Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I64Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector2: ...

@final
class I64Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I64Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I64Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> I64Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> I64Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[I64Vector2]: ...

@final
class U64Vector2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __add__(self, other: Number) -> U64Vector2: ...
    @overload
    def __radd__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __radd__(self, other: Number) -> U64Vector2: ...
    @overload
    def __sub__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __sub__(self, other: Number) -> U64Vector2: ...
    @overload
    def __rsub__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __rsub__(self, other: Number) -> U64Vector2: ...
    @overload
    def __mul__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __mul__(self, other: Number) -> U64Vector2: ...
    @overload
    def __rmul__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __rmul__(self, other: Number) -> U64Vector2: ...
    @overload
    def __truediv__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __truediv__(self, other: Number) -> U64Vector2: ...
    @overload
    def __rtruediv__(self, other: U64Vector2) -> U64Vector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> U64Vector2: ...
    def __abs__(self) -> U64Vector2: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def xx(self) -> U64Vector2: ...
    @property
    def xy(self) -> U64Vector2: ...
    @property
    def xo(self) -> U64Vector2: ...
    @property
    def xl(self) -> U64Vector2: ...
    @property
    def yx(self) -> U64Vector2: ...
    @property
    def yy(self) -> U64Vector2: ...
    @property
    def yo(self) -> U64Vector2: ...
    @property
    def yl(self) -> U64Vector2: ...
    @property
    def ox(self) -> U64Vector2: ...
    @property
    def oy(self) -> U64Vector2: ...
    @property
    def lx(self) -> U64Vector2: ...
    @property
    def ly(self) -> U64Vector2: ...
    @property
    def xxx(self) -> U64Vector3: ...
    @property
    def xxy(self) -> U64Vector3: ...
    @property
    def xxo(self) -> U64Vector3: ...
    @property
    def xxl(self) -> U64Vector3: ...
    @property
    def xyx(self) -> U64Vector3: ...
    @property
    def xyy(self) -> U64Vector3: ...
    @property
    def xyo(self) -> U64Vector3: ...
    @property
    def xyl(self) -> U64Vector3: ...
    @property
    def xox(self) -> U64Vector3: ...
    @property
    def xoy(self) -> U64Vector3: ...
    @property
    def xoo(self) -> U64Vector3: ...
    @property
    def xol(self) -> U64Vector3: ...
    @property
    def xlx(self) -> U64Vector3: ...
    @property
    def xly(self) -> U64Vector3: ...
    @property
    def xlo(self) -> U64Vector3: ...
    @property
    def xll(self) -> U64Vector3: ...
    @property
    def yxx(self) -> U64Vector3: ...
    @property
    def yxy(self) -> U64Vector3: ...
    @property
    def yxo(self) -> U64Vector3: ...
    @property
    def yxl(self) -> U64Vector3: ...
    @property
    def yyx(self) -> U64Vector3: ...
    @property
    def yyy(self) -> U64Vector3: ...
    @property
    def yyo(self) -> U64Vector3: ...
    @property
    def yyl(self) -> U64Vector3: ...
    @property
    def yox(self) -> U64Vector3: ...
    @property
    def yoy(self) -> U64Vector3: ...
    @property
    def yoo(self) -> U64Vector3: ...
    @property
    def yol(self) -> U64Vector3: ...
    @property
    def ylx(self) -> U64Vector3: ...
    @property
    def yly(self) -> U64Vector3: ...
    @property
    def ylo(self) -> U64Vector3: ...
    @property
    def yll(self) -> U64Vector3: ...
    @property
    def oxx(self) -> U64Vector3: ...
    @property
    def oxy(self) -> U64Vector3: ...
    @property
    def oxo(self) -> U64Vector3: ...
    @property
    def oxl(self) -> U64Vector3: ...
    @property
    def oyx(self) -> U64Vector3: ...
    @property
    def oyy(self) -> U64Vector3: ...
    @property
    def oyo(self) -> U64Vector3: ...
    @property
    def oyl(self) -> U64Vector3: ...
    @property
    def oox(self) -> U64Vector3: ...
    @property
    def ooy(self) -> U64Vector3: ...
    @property
    def olx(self) -> U64Vector3: ...
    @property
    def oly(self) -> U64Vector3: ...
    @property
    def lxx(self) -> U64Vector3: ...
    @property
    def lxy(self) -> U64Vector3: ...
    @property
    def lxo(self) -> U64Vector3: ...
    @property
    def lxl(self) -> U64Vector3: ...
    @property
    def lyx(self) -> U64Vector3: ...
    @property
    def lyy(self) -> U64Vector3: ...
    @property
    def lyo(self) -> U64Vector3: ...
    @property
    def lyl(self) -> U64Vector3: ...
    @property
    def lox(self) -> U64Vector3: ...
    @property
    def loy(self) -> U64Vector3: ...
    @property
    def llx(self) -> U64Vector3: ...
    @property
    def lly(self) -> U64Vector3: ...
    @property
    def xxxx(self) -> U64Vector4: ...
    @property
    def xxxy(self) -> U64Vector4: ...
    @property
    def xxxo(self) -> U64Vector4: ...
    @property
    def xxxl(self) -> U64Vector4: ...
    @property
    def xxyx(self) -> U64Vector4: ...
    @property
    def xxyy(self) -> U64Vector4: ...
    @property
    def xxyo(self) -> U64Vector4: ...
    @property
    def xxyl(self) -> U64Vector4: ...
    @property
    def xxox(self) -> U64Vector4: ...
    @property
    def xxoy(self) -> U64Vector4: ...
    @property
    def xxoo(self) -> U64Vector4: ...
    @property
    def xxol(self) -> U64Vector4: ...
    @property
    def xxlx(self) -> U64Vector4: ...
    @property
    def xxly(self) -> U64Vector4: ...
    @property
    def xxlo(self) -> U64Vector4: ...
    @property
    def xxll(self) -> U64Vector4: ...
    @property
    def xyxx(self) -> U64Vector4: ...
    @property
    def xyxy(self) -> U64Vector4: ...
    @property
    def xyxo(self) -> U64Vector4: ...
    @property
    def xyxl(self) -> U64Vector4: ...
    @property
    def xyyx(self) -> U64Vector4: ...
    @property
    def xyyy(self) -> U64Vector4: ...
    @property
    def xyyo(self) -> U64Vector4: ...
    @property
    def xyyl(self) -> U64Vector4: ...
    @property
    def xyox(self) -> U64Vector4: ...
    @property
    def xyoy(self) -> U64Vector4: ...
    @property
    def xyoo(self) -> U64Vector4: ...
    @property
    def xyol(self) -> U64Vector4: ...
    @property
    def xylx(self) -> U64Vector4: ...
    @property
    def xyly(self) -> U64Vector4: ...
    @property
    def xylo(self) -> U64Vector4: ...
    @property
    def xyll(self) -> U64Vector4: ...
    @property
    def xoxx(self) -> U64Vector4: ...
    @property
    def xoxy(self) -> U64Vector4: ...
    @property
    def xoxo(self) -> U64Vector4: ...
    @property
    def xoxl(self) -> U64Vector4: ...
    @property
    def xoyx(self) -> U64Vector4: ...
    @property
    def xoyy(self) -> U64Vector4: ...
    @property
    def xoyo(self) -> U64Vector4: ...
    @property
    def xoyl(self) -> U64Vector4: ...
    @property
    def xoox(self) -> U64Vector4: ...
    @property
    def xooy(self) -> U64Vector4: ...
    @property
    def xooo(self) -> U64Vector4: ...
    @property
    def xool(self) -> U64Vector4: ...
    @property
    def xolx(self) -> U64Vector4: ...
    @property
    def xoly(self) -> U64Vector4: ...
    @property
    def xolo(self) -> U64Vector4: ...
    @property
    def xoll(self) -> U64Vector4: ...
    @property
    def xlxx(self) -> U64Vector4: ...
    @property
    def xlxy(self) -> U64Vector4: ...
    @property
    def xlxo(self) -> U64Vector4: ...
    @property
    def xlxl(self) -> U64Vector4: ...
    @property
    def xlyx(self) -> U64Vector4: ...
    @property
    def xlyy(self) -> U64Vector4: ...
    @property
    def xlyo(self) -> U64Vector4: ...
    @property
    def xlyl(self) -> U64Vector4: ...
    @property
    def xlox(self) -> U64Vector4: ...
    @property
    def xloy(self) -> U64Vector4: ...
    @property
    def xloo(self) -> U64Vector4: ...
    @property
    def xlol(self) -> U64Vector4: ...
    @property
    def xllx(self) -> U64Vector4: ...
    @property
    def xlly(self) -> U64Vector4: ...
    @property
    def xllo(self) -> U64Vector4: ...
    @property
    def xlll(self) -> U64Vector4: ...
    @property
    def yxxx(self) -> U64Vector4: ...
    @property
    def yxxy(self) -> U64Vector4: ...
    @property
    def yxxo(self) -> U64Vector4: ...
    @property
    def yxxl(self) -> U64Vector4: ...
    @property
    def yxyx(self) -> U64Vector4: ...
    @property
    def yxyy(self) -> U64Vector4: ...
    @property
    def yxyo(self) -> U64Vector4: ...
    @property
    def yxyl(self) -> U64Vector4: ...
    @property
    def yxox(self) -> U64Vector4: ...
    @property
    def yxoy(self) -> U64Vector4: ...
    @property
    def yxoo(self) -> U64Vector4: ...
    @property
    def yxol(self) -> U64Vector4: ...
    @property
    def yxlx(self) -> U64Vector4: ...
    @property
    def yxly(self) -> U64Vector4: ...
    @property
    def yxlo(self) -> U64Vector4: ...
    @property
    def yxll(self) -> U64Vector4: ...
    @property
    def yyxx(self) -> U64Vector4: ...
    @property
    def yyxy(self) -> U64Vector4: ...
    @property
    def yyxo(self) -> U64Vector4: ...
    @property
    def yyxl(self) -> U64Vector4: ...
    @property
    def yyyx(self) -> U64Vector4: ...
    @property
    def yyyy(self) -> U64Vector4: ...
    @property
    def yyyo(self) -> U64Vector4: ...
    @property
    def yyyl(self) -> U64Vector4: ...
    @property
    def yyox(self) -> U64Vector4: ...
    @property
    def yyoy(self) -> U64Vector4: ...
    @property
    def yyoo(self) -> U64Vector4: ...
    @property
    def yyol(self) -> U64Vector4: ...
    @property
    def yylx(self) -> U64Vector4: ...
    @property
    def yyly(self) -> U64Vector4: ...
    @property
    def yylo(self) -> U64Vector4: ...
    @property
    def yyll(self) -> U64Vector4: ...
    @property
    def yoxx(self) -> U64Vector4: ...
    @property
    def yoxy(self) -> U64Vector4: ...
    @property
    def yoxo(self) -> U64Vector4: ...
    @property
    def yoxl(self) -> U64Vector4: ...
    @property
    def yoyx(self) -> U64Vector4: ...
    @property
    def yoyy(self) -> U64Vector4: ...
    @property
    def yoyo(self) -> U64Vector4: ...
    @property
    def yoyl(self) -> U64Vector4: ...
    @property
    def yoox(self) -> U64Vector4: ...
    @property
    def yooy(self) -> U64Vector4: ...
    @property
    def yooo(self) -> U64Vector4: ...
    @property
    def yool(self) -> U64Vector4: ...
    @property
    def yolx(self) -> U64Vector4: ...
    @property
    def yoly(self) -> U64Vector4: ...
    @property
    def yolo(self) -> U64Vector4: ...
    @property
    def yoll(self) -> U64Vector4: ...
    @property
    def ylxx(self) -> U64Vector4: ...
    @property
    def ylxy(self) -> U64Vector4: ...
    @property
    def ylxo(self) -> U64Vector4: ...
    @property
    def ylxl(self) -> U64Vector4: ...
    @property
    def ylyx(self) -> U64Vector4: ...
    @property
    def ylyy(self) -> U64Vector4: ...
    @property
    def ylyo(self) -> U64Vector4: ...
    @property
    def ylyl(self) -> U64Vector4: ...
    @property
    def ylox(self) -> U64Vector4: ...
    @property
    def yloy(self) -> U64Vector4: ...
    @property
    def yloo(self) -> U64Vector4: ...
    @property
    def ylol(self) -> U64Vector4: ...
    @property
    def yllx(self) -> U64Vector4: ...
    @property
    def ylly(self) -> U64Vector4: ...
    @property
    def yllo(self) -> U64Vector4: ...
    @property
    def ylll(self) -> U64Vector4: ...
    @property
    def oxxx(self) -> U64Vector4: ...
    @property
    def oxxy(self) -> U64Vector4: ...
    @property
    def oxxo(self) -> U64Vector4: ...
    @property
    def oxxl(self) -> U64Vector4: ...
    @property
    def oxyx(self) -> U64Vector4: ...
    @property
    def oxyy(self) -> U64Vector4: ...
    @property
    def oxyo(self) -> U64Vector4: ...
    @property
    def oxyl(self) -> U64Vector4: ...
    @property
    def oxox(self) -> U64Vector4: ...
    @property
    def oxoy(self) -> U64Vector4: ...
    @property
    def oxoo(self) -> U64Vector4: ...
    @property
    def oxol(self) -> U64Vector4: ...
    @property
    def oxlx(self) -> U64Vector4: ...
    @property
    def oxly(self) -> U64Vector4: ...
    @property
    def oxlo(self) -> U64Vector4: ...
    @property
    def oxll(self) -> U64Vector4: ...
    @property
    def oyxx(self) -> U64Vector4: ...
    @property
    def oyxy(self) -> U64Vector4: ...
    @property
    def oyxo(self) -> U64Vector4: ...
    @property
    def oyxl(self) -> U64Vector4: ...
    @property
    def oyyx(self) -> U64Vector4: ...
    @property
    def oyyy(self) -> U64Vector4: ...
    @property
    def oyyo(self) -> U64Vector4: ...
    @property
    def oyyl(self) -> U64Vector4: ...
    @property
    def oyox(self) -> U64Vector4: ...
    @property
    def oyoy(self) -> U64Vector4: ...
    @property
    def oyoo(self) -> U64Vector4: ...
    @property
    def oyol(self) -> U64Vector4: ...
    @property
    def oylx(self) -> U64Vector4: ...
    @property
    def oyly(self) -> U64Vector4: ...
    @property
    def oylo(self) -> U64Vector4: ...
    @property
    def oyll(self) -> U64Vector4: ...
    @property
    def ooxx(self) -> U64Vector4: ...
    @property
    def ooxy(self) -> U64Vector4: ...
    @property
    def ooxo(self) -> U64Vector4: ...
    @property
    def ooxl(self) -> U64Vector4: ...
    @property
    def ooyx(self) -> U64Vector4: ...
    @property
    def ooyy(self) -> U64Vector4: ...
    @property
    def ooyo(self) -> U64Vector4: ...
    @property
    def ooyl(self) -> U64Vector4: ...
    @property
    def ooox(self) -> U64Vector4: ...
    @property
    def oooy(self) -> U64Vector4: ...
    @property
    def oolx(self) -> U64Vector4: ...
    @property
    def ooly(self) -> U64Vector4: ...
    @property
    def olxx(self) -> U64Vector4: ...
    @property
    def olxy(self) -> U64Vector4: ...
    @property
    def olxo(self) -> U64Vector4: ...
    @property
    def olxl(self) -> U64Vector4: ...
    @property
    def olyx(self) -> U64Vector4: ...
    @property
    def olyy(self) -> U64Vector4: ...
    @property
    def olyo(self) -> U64Vector4: ...
    @property
    def olyl(self) -> U64Vector4: ...
    @property
    def olox(self) -> U64Vector4: ...
    @property
    def oloy(self) -> U64Vector4: ...
    @property
    def ollx(self) -> U64Vector4: ...
    @property
    def olly(self) -> U64Vector4: ...
    @property
    def lxxx(self) -> U64Vector4: ...
    @property
    def lxxy(self) -> U64Vector4: ...
    @property
    def lxxo(self) -> U64Vector4: ...
    @property
    def lxxl(self) -> U64Vector4: ...
    @property
    def lxyx(self) -> U64Vector4: ...
    @property
    def lxyy(self) -> U64Vector4: ...
    @property
    def lxyo(self) -> U64Vector4: ...
    @property
    def lxyl(self) -> U64Vector4: ...
    @property
    def lxox(self) -> U64Vector4: ...
    @property
    def lxoy(self) -> U64Vector4: ...
    @property
    def lxoo(self) -> U64Vector4: ...
    @property
    def lxol(self) -> U64Vector4: ...
    @property
    def lxlx(self) -> U64Vector4: ...
    @property
    def lxly(self) -> U64Vector4: ...
    @property
    def lxlo(self) -> U64Vector4: ...
    @property
    def lxll(self) -> U64Vector4: ...
    @property
    def lyxx(self) -> U64Vector4: ...
    @property
    def lyxy(self) -> U64Vector4: ...
    @property
    def lyxo(self) -> U64Vector4: ...
    @property
    def lyxl(self) -> U64Vector4: ...
    @property
    def lyyx(self) -> U64Vector4: ...
    @property
    def lyyy(self) -> U64Vector4: ...
    @property
    def lyyo(self) -> U64Vector4: ...
    @property
    def lyyl(self) -> U64Vector4: ...
    @property
    def lyox(self) -> U64Vector4: ...
    @property
    def lyoy(self) -> U64Vector4: ...
    @property
    def lyoo(self) -> U64Vector4: ...
    @property
    def lyol(self) -> U64Vector4: ...
    @property
    def lylx(self) -> U64Vector4: ...
    @property
    def lyly(self) -> U64Vector4: ...
    @property
    def lylo(self) -> U64Vector4: ...
    @property
    def lyll(self) -> U64Vector4: ...
    @property
    def loxx(self) -> U64Vector4: ...
    @property
    def loxy(self) -> U64Vector4: ...
    @property
    def loxo(self) -> U64Vector4: ...
    @property
    def loxl(self) -> U64Vector4: ...
    @property
    def loyx(self) -> U64Vector4: ...
    @property
    def loyy(self) -> U64Vector4: ...
    @property
    def loyo(self) -> U64Vector4: ...
    @property
    def loyl(self) -> U64Vector4: ...
    @property
    def loox(self) -> U64Vector4: ...
    @property
    def looy(self) -> U64Vector4: ...
    @property
    def lolx(self) -> U64Vector4: ...
    @property
    def loly(self) -> U64Vector4: ...
    @property
    def llxx(self) -> U64Vector4: ...
    @property
    def llxy(self) -> U64Vector4: ...
    @property
    def llxo(self) -> U64Vector4: ...
    @property
    def llxl(self) -> U64Vector4: ...
    @property
    def llyx(self) -> U64Vector4: ...
    @property
    def llyy(self) -> U64Vector4: ...
    @property
    def llyo(self) -> U64Vector4: ...
    @property
    def llyl(self) -> U64Vector4: ...
    @property
    def llox(self) -> U64Vector4: ...
    @property
    def lloy(self) -> U64Vector4: ...
    @property
    def lllx(self) -> U64Vector4: ...
    @property
    def llly(self) -> U64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def rr(self) -> U64Vector2: ...
    @property
    def rg(self) -> U64Vector2: ...
    @property
    def ro(self) -> U64Vector2: ...
    @property
    def rl(self) -> U64Vector2: ...
    @property
    def gr(self) -> U64Vector2: ...
    @property
    def gg(self) -> U64Vector2: ...
    @property
    def go(self) -> U64Vector2: ...
    @property
    def gl(self) -> U64Vector2: ...
    @property
    def og(self) -> U64Vector2: ...
    @property
    def lr(self) -> U64Vector2: ...
    @property
    def lg(self) -> U64Vector2: ...
    @property
    def rrr(self) -> U64Vector3: ...
    @property
    def rrg(self) -> U64Vector3: ...
    @property
    def rro(self) -> U64Vector3: ...
    @property
    def rrl(self) -> U64Vector3: ...
    @property
    def rgr(self) -> U64Vector3: ...
    @property
    def rgg(self) -> U64Vector3: ...
    @property
    def rgo(self) -> U64Vector3: ...
    @property
    def rgl(self) -> U64Vector3: ...
    @property
    def ror(self) -> U64Vector3: ...
    @property
    def rog(self) -> U64Vector3: ...
    @property
    def roo(self) -> U64Vector3: ...
    @property
    def rol(self) -> U64Vector3: ...
    @property
    def rlr(self) -> U64Vector3: ...
    @property
    def rlg(self) -> U64Vector3: ...
    @property
    def rlo(self) -> U64Vector3: ...
    @property
    def rll(self) -> U64Vector3: ...
    @property
    def grr(self) -> U64Vector3: ...
    @property
    def grg(self) -> U64Vector3: ...
    @property
    def gro(self) -> U64Vector3: ...
    @property
    def grl(self) -> U64Vector3: ...
    @property
    def ggr(self) -> U64Vector3: ...
    @property
    def ggg(self) -> U64Vector3: ...
    @property
    def ggo(self) -> U64Vector3: ...
    @property
    def ggl(self) -> U64Vector3: ...
    @property
    def gor(self) -> U64Vector3: ...
    @property
    def gog(self) -> U64Vector3: ...
    @property
    def goo(self) -> U64Vector3: ...
    @property
    def gol(self) -> U64Vector3: ...
    @property
    def glr(self) -> U64Vector3: ...
    @property
    def glg(self) -> U64Vector3: ...
    @property
    def glo(self) -> U64Vector3: ...
    @property
    def gll(self) -> U64Vector3: ...
    @property
    def orr(self) -> U64Vector3: ...
    @property
    def org(self) -> U64Vector3: ...
    @property
    def oro(self) -> U64Vector3: ...
    @property
    def orl(self) -> U64Vector3: ...
    @property
    def ogr(self) -> U64Vector3: ...
    @property
    def ogg(self) -> U64Vector3: ...
    @property
    def ogo(self) -> U64Vector3: ...
    @property
    def ogl(self) -> U64Vector3: ...
    @property
    def oor(self) -> U64Vector3: ...
    @property
    def oog(self) -> U64Vector3: ...
    @property
    def olr(self) -> U64Vector3: ...
    @property
    def olg(self) -> U64Vector3: ...
    @property
    def lrr(self) -> U64Vector3: ...
    @property
    def lrg(self) -> U64Vector3: ...
    @property
    def lro(self) -> U64Vector3: ...
    @property
    def lrl(self) -> U64Vector3: ...
    @property
    def lgr(self) -> U64Vector3: ...
    @property
    def lgg(self) -> U64Vector3: ...
    @property
    def lgo(self) -> U64Vector3: ...
    @property
    def lgl(self) -> U64Vector3: ...
    @property
    def lor(self) -> U64Vector3: ...
    @property
    def log(self) -> U64Vector3: ...
    @property
    def llr(self) -> U64Vector3: ...
    @property
    def llg(self) -> U64Vector3: ...
    @property
    def rrrr(self) -> U64Vector4: ...
    @property
    def rrrg(self) -> U64Vector4: ...
    @property
    def rrro(self) -> U64Vector4: ...
    @property
    def rrrl(self) -> U64Vector4: ...
    @property
    def rrgr(self) -> U64Vector4: ...
    @property
    def rrgg(self) -> U64Vector4: ...
    @property
    def rrgo(self) -> U64Vector4: ...
    @property
    def rrgl(self) -> U64Vector4: ...
    @property
    def rror(self) -> U64Vector4: ...
    @property
    def rrog(self) -> U64Vector4: ...
    @property
    def rroo(self) -> U64Vector4: ...
    @property
    def rrol(self) -> U64Vector4: ...
    @property
    def rrlr(self) -> U64Vector4: ...
    @property
    def rrlg(self) -> U64Vector4: ...
    @property
    def rrlo(self) -> U64Vector4: ...
    @property
    def rrll(self) -> U64Vector4: ...
    @property
    def rgrr(self) -> U64Vector4: ...
    @property
    def rgrg(self) -> U64Vector4: ...
    @property
    def rgro(self) -> U64Vector4: ...
    @property
    def rgrl(self) -> U64Vector4: ...
    @property
    def rggr(self) -> U64Vector4: ...
    @property
    def rggg(self) -> U64Vector4: ...
    @property
    def rggo(self) -> U64Vector4: ...
    @property
    def rggl(self) -> U64Vector4: ...
    @property
    def rgor(self) -> U64Vector4: ...
    @property
    def rgog(self) -> U64Vector4: ...
    @property
    def rgoo(self) -> U64Vector4: ...
    @property
    def rgol(self) -> U64Vector4: ...
    @property
    def rglr(self) -> U64Vector4: ...
    @property
    def rglg(self) -> U64Vector4: ...
    @property
    def rglo(self) -> U64Vector4: ...
    @property
    def rgll(self) -> U64Vector4: ...
    @property
    def rorr(self) -> U64Vector4: ...
    @property
    def rorg(self) -> U64Vector4: ...
    @property
    def roro(self) -> U64Vector4: ...
    @property
    def rorl(self) -> U64Vector4: ...
    @property
    def rogr(self) -> U64Vector4: ...
    @property
    def rogg(self) -> U64Vector4: ...
    @property
    def rogo(self) -> U64Vector4: ...
    @property
    def rogl(self) -> U64Vector4: ...
    @property
    def roor(self) -> U64Vector4: ...
    @property
    def roog(self) -> U64Vector4: ...
    @property
    def rooo(self) -> U64Vector4: ...
    @property
    def rool(self) -> U64Vector4: ...
    @property
    def rolr(self) -> U64Vector4: ...
    @property
    def rolg(self) -> U64Vector4: ...
    @property
    def rolo(self) -> U64Vector4: ...
    @property
    def roll(self) -> U64Vector4: ...
    @property
    def rlrr(self) -> U64Vector4: ...
    @property
    def rlrg(self) -> U64Vector4: ...
    @property
    def rlro(self) -> U64Vector4: ...
    @property
    def rlrl(self) -> U64Vector4: ...
    @property
    def rlgr(self) -> U64Vector4: ...
    @property
    def rlgg(self) -> U64Vector4: ...
    @property
    def rlgo(self) -> U64Vector4: ...
    @property
    def rlgl(self) -> U64Vector4: ...
    @property
    def rlor(self) -> U64Vector4: ...
    @property
    def rlog(self) -> U64Vector4: ...
    @property
    def rloo(self) -> U64Vector4: ...
    @property
    def rlol(self) -> U64Vector4: ...
    @property
    def rllr(self) -> U64Vector4: ...
    @property
    def rllg(self) -> U64Vector4: ...
    @property
    def rllo(self) -> U64Vector4: ...
    @property
    def rlll(self) -> U64Vector4: ...
    @property
    def grrr(self) -> U64Vector4: ...
    @property
    def grrg(self) -> U64Vector4: ...
    @property
    def grro(self) -> U64Vector4: ...
    @property
    def grrl(self) -> U64Vector4: ...
    @property
    def grgr(self) -> U64Vector4: ...
    @property
    def grgg(self) -> U64Vector4: ...
    @property
    def grgo(self) -> U64Vector4: ...
    @property
    def grgl(self) -> U64Vector4: ...
    @property
    def gror(self) -> U64Vector4: ...
    @property
    def grog(self) -> U64Vector4: ...
    @property
    def groo(self) -> U64Vector4: ...
    @property
    def grol(self) -> U64Vector4: ...
    @property
    def grlr(self) -> U64Vector4: ...
    @property
    def grlg(self) -> U64Vector4: ...
    @property
    def grlo(self) -> U64Vector4: ...
    @property
    def grll(self) -> U64Vector4: ...
    @property
    def ggrr(self) -> U64Vector4: ...
    @property
    def ggrg(self) -> U64Vector4: ...
    @property
    def ggro(self) -> U64Vector4: ...
    @property
    def ggrl(self) -> U64Vector4: ...
    @property
    def gggr(self) -> U64Vector4: ...
    @property
    def gggg(self) -> U64Vector4: ...
    @property
    def gggo(self) -> U64Vector4: ...
    @property
    def gggl(self) -> U64Vector4: ...
    @property
    def ggor(self) -> U64Vector4: ...
    @property
    def ggog(self) -> U64Vector4: ...
    @property
    def ggoo(self) -> U64Vector4: ...
    @property
    def ggol(self) -> U64Vector4: ...
    @property
    def gglr(self) -> U64Vector4: ...
    @property
    def gglg(self) -> U64Vector4: ...
    @property
    def gglo(self) -> U64Vector4: ...
    @property
    def ggll(self) -> U64Vector4: ...
    @property
    def gorr(self) -> U64Vector4: ...
    @property
    def gorg(self) -> U64Vector4: ...
    @property
    def goro(self) -> U64Vector4: ...
    @property
    def gorl(self) -> U64Vector4: ...
    @property
    def gogr(self) -> U64Vector4: ...
    @property
    def gogg(self) -> U64Vector4: ...
    @property
    def gogo(self) -> U64Vector4: ...
    @property
    def gogl(self) -> U64Vector4: ...
    @property
    def goor(self) -> U64Vector4: ...
    @property
    def goog(self) -> U64Vector4: ...
    @property
    def gooo(self) -> U64Vector4: ...
    @property
    def gool(self) -> U64Vector4: ...
    @property
    def golr(self) -> U64Vector4: ...
    @property
    def golg(self) -> U64Vector4: ...
    @property
    def golo(self) -> U64Vector4: ...
    @property
    def goll(self) -> U64Vector4: ...
    @property
    def glrr(self) -> U64Vector4: ...
    @property
    def glrg(self) -> U64Vector4: ...
    @property
    def glro(self) -> U64Vector4: ...
    @property
    def glrl(self) -> U64Vector4: ...
    @property
    def glgr(self) -> U64Vector4: ...
    @property
    def glgg(self) -> U64Vector4: ...
    @property
    def glgo(self) -> U64Vector4: ...
    @property
    def glgl(self) -> U64Vector4: ...
    @property
    def glor(self) -> U64Vector4: ...
    @property
    def glog(self) -> U64Vector4: ...
    @property
    def gloo(self) -> U64Vector4: ...
    @property
    def glol(self) -> U64Vector4: ...
    @property
    def gllr(self) -> U64Vector4: ...
    @property
    def gllg(self) -> U64Vector4: ...
    @property
    def gllo(self) -> U64Vector4: ...
    @property
    def glll(self) -> U64Vector4: ...
    @property
    def orrr(self) -> U64Vector4: ...
    @property
    def orrg(self) -> U64Vector4: ...
    @property
    def orro(self) -> U64Vector4: ...
    @property
    def orrl(self) -> U64Vector4: ...
    @property
    def orgr(self) -> U64Vector4: ...
    @property
    def orgg(self) -> U64Vector4: ...
    @property
    def orgo(self) -> U64Vector4: ...
    @property
    def orgl(self) -> U64Vector4: ...
    @property
    def oror(self) -> U64Vector4: ...
    @property
    def orog(self) -> U64Vector4: ...
    @property
    def oroo(self) -> U64Vector4: ...
    @property
    def orol(self) -> U64Vector4: ...
    @property
    def orlr(self) -> U64Vector4: ...
    @property
    def orlg(self) -> U64Vector4: ...
    @property
    def orlo(self) -> U64Vector4: ...
    @property
    def orll(self) -> U64Vector4: ...
    @property
    def ogrr(self) -> U64Vector4: ...
    @property
    def ogrg(self) -> U64Vector4: ...
    @property
    def ogro(self) -> U64Vector4: ...
    @property
    def ogrl(self) -> U64Vector4: ...
    @property
    def oggr(self) -> U64Vector4: ...
    @property
    def oggg(self) -> U64Vector4: ...
    @property
    def oggo(self) -> U64Vector4: ...
    @property
    def oggl(self) -> U64Vector4: ...
    @property
    def ogor(self) -> U64Vector4: ...
    @property
    def ogog(self) -> U64Vector4: ...
    @property
    def ogoo(self) -> U64Vector4: ...
    @property
    def ogol(self) -> U64Vector4: ...
    @property
    def oglr(self) -> U64Vector4: ...
    @property
    def oglg(self) -> U64Vector4: ...
    @property
    def oglo(self) -> U64Vector4: ...
    @property
    def ogll(self) -> U64Vector4: ...
    @property
    def oorr(self) -> U64Vector4: ...
    @property
    def oorg(self) -> U64Vector4: ...
    @property
    def ooro(self) -> U64Vector4: ...
    @property
    def oorl(self) -> U64Vector4: ...
    @property
    def oogr(self) -> U64Vector4: ...
    @property
    def oogg(self) -> U64Vector4: ...
    @property
    def oogo(self) -> U64Vector4: ...
    @property
    def oogl(self) -> U64Vector4: ...
    @property
    def ooor(self) -> U64Vector4: ...
    @property
    def ooog(self) -> U64Vector4: ...
    @property
    def oolr(self) -> U64Vector4: ...
    @property
    def oolg(self) -> U64Vector4: ...
    @property
    def olrr(self) -> U64Vector4: ...
    @property
    def olrg(self) -> U64Vector4: ...
    @property
    def olro(self) -> U64Vector4: ...
    @property
    def olrl(self) -> U64Vector4: ...
    @property
    def olgr(self) -> U64Vector4: ...
    @property
    def olgg(self) -> U64Vector4: ...
    @property
    def olgo(self) -> U64Vector4: ...
    @property
    def olgl(self) -> U64Vector4: ...
    @property
    def olor(self) -> U64Vector4: ...
    @property
    def olog(self) -> U64Vector4: ...
    @property
    def ollr(self) -> U64Vector4: ...
    @property
    def ollg(self) -> U64Vector4: ...
    @property
    def lrrr(self) -> U64Vector4: ...
    @property
    def lrrg(self) -> U64Vector4: ...
    @property
    def lrro(self) -> U64Vector4: ...
    @property
    def lrrl(self) -> U64Vector4: ...
    @property
    def lrgr(self) -> U64Vector4: ...
    @property
    def lrgg(self) -> U64Vector4: ...
    @property
    def lrgo(self) -> U64Vector4: ...
    @property
    def lrgl(self) -> U64Vector4: ...
    @property
    def lror(self) -> U64Vector4: ...
    @property
    def lrog(self) -> U64Vector4: ...
    @property
    def lroo(self) -> U64Vector4: ...
    @property
    def lrol(self) -> U64Vector4: ...
    @property
    def lrlr(self) -> U64Vector4: ...
    @property
    def lrlg(self) -> U64Vector4: ...
    @property
    def lrlo(self) -> U64Vector4: ...
    @property
    def lrll(self) -> U64Vector4: ...
    @property
    def lgrr(self) -> U64Vector4: ...
    @property
    def lgrg(self) -> U64Vector4: ...
    @property
    def lgro(self) -> U64Vector4: ...
    @property
    def lgrl(self) -> U64Vector4: ...
    @property
    def lggr(self) -> U64Vector4: ...
    @property
    def lggg(self) -> U64Vector4: ...
    @property
    def lggo(self) -> U64Vector4: ...
    @property
    def lggl(self) -> U64Vector4: ...
    @property
    def lgor(self) -> U64Vector4: ...
    @property
    def lgog(self) -> U64Vector4: ...
    @property
    def lgoo(self) -> U64Vector4: ...
    @property
    def lgol(self) -> U64Vector4: ...
    @property
    def lglr(self) -> U64Vector4: ...
    @property
    def lglg(self) -> U64Vector4: ...
    @property
    def lglo(self) -> U64Vector4: ...
    @property
    def lgll(self) -> U64Vector4: ...
    @property
    def lorr(self) -> U64Vector4: ...
    @property
    def lorg(self) -> U64Vector4: ...
    @property
    def loro(self) -> U64Vector4: ...
    @property
    def lorl(self) -> U64Vector4: ...
    @property
    def logr(self) -> U64Vector4: ...
    @property
    def logg(self) -> U64Vector4: ...
    @property
    def logo(self) -> U64Vector4: ...
    @property
    def logl(self) -> U64Vector4: ...
    @property
    def loor(self) -> U64Vector4: ...
    @property
    def loog(self) -> U64Vector4: ...
    @property
    def lolr(self) -> U64Vector4: ...
    @property
    def lolg(self) -> U64Vector4: ...
    @property
    def llrr(self) -> U64Vector4: ...
    @property
    def llrg(self) -> U64Vector4: ...
    @property
    def llro(self) -> U64Vector4: ...
    @property
    def llrl(self) -> U64Vector4: ...
    @property
    def llgr(self) -> U64Vector4: ...
    @property
    def llgg(self) -> U64Vector4: ...
    @property
    def llgo(self) -> U64Vector4: ...
    @property
    def llgl(self) -> U64Vector4: ...
    @property
    def llor(self) -> U64Vector4: ...
    @property
    def llog(self) -> U64Vector4: ...
    @property
    def lllr(self) -> U64Vector4: ...
    @property
    def lllg(self) -> U64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def ss(self) -> U64Vector2: ...
    @property
    def st(self) -> U64Vector2: ...
    @property
    def so(self) -> U64Vector2: ...
    @property
    def sl(self) -> U64Vector2: ...
    @property
    def ts(self) -> U64Vector2: ...
    @property
    def tt(self) -> U64Vector2: ...
    @property
    def to(self) -> U64Vector2: ...
    @property
    def tl(self) -> U64Vector2: ...
    @property
    def os(self) -> U64Vector2: ...
    @property
    def ot(self) -> U64Vector2: ...
    @property
    def ls(self) -> U64Vector2: ...
    @property
    def lt(self) -> U64Vector2: ...
    @property
    def sss(self) -> U64Vector3: ...
    @property
    def sst(self) -> U64Vector3: ...
    @property
    def sso(self) -> U64Vector3: ...
    @property
    def ssl(self) -> U64Vector3: ...
    @property
    def sts(self) -> U64Vector3: ...
    @property
    def stt(self) -> U64Vector3: ...
    @property
    def sto(self) -> U64Vector3: ...
    @property
    def stl(self) -> U64Vector3: ...
    @property
    def sos(self) -> U64Vector3: ...
    @property
    def sot(self) -> U64Vector3: ...
    @property
    def soo(self) -> U64Vector3: ...
    @property
    def sol(self) -> U64Vector3: ...
    @property
    def sls(self) -> U64Vector3: ...
    @property
    def slt(self) -> U64Vector3: ...
    @property
    def slo(self) -> U64Vector3: ...
    @property
    def sll(self) -> U64Vector3: ...
    @property
    def tss(self) -> U64Vector3: ...
    @property
    def tst(self) -> U64Vector3: ...
    @property
    def tso(self) -> U64Vector3: ...
    @property
    def tsl(self) -> U64Vector3: ...
    @property
    def tts(self) -> U64Vector3: ...
    @property
    def ttt(self) -> U64Vector3: ...
    @property
    def tto(self) -> U64Vector3: ...
    @property
    def ttl(self) -> U64Vector3: ...
    @property
    def tos(self) -> U64Vector3: ...
    @property
    def tot(self) -> U64Vector3: ...
    @property
    def too(self) -> U64Vector3: ...
    @property
    def tol(self) -> U64Vector3: ...
    @property
    def tls(self) -> U64Vector3: ...
    @property
    def tlt(self) -> U64Vector3: ...
    @property
    def tlo(self) -> U64Vector3: ...
    @property
    def tll(self) -> U64Vector3: ...
    @property
    def oss(self) -> U64Vector3: ...
    @property
    def ost(self) -> U64Vector3: ...
    @property
    def oso(self) -> U64Vector3: ...
    @property
    def osl(self) -> U64Vector3: ...
    @property
    def ots(self) -> U64Vector3: ...
    @property
    def ott(self) -> U64Vector3: ...
    @property
    def oto(self) -> U64Vector3: ...
    @property
    def otl(self) -> U64Vector3: ...
    @property
    def oos(self) -> U64Vector3: ...
    @property
    def oot(self) -> U64Vector3: ...
    @property
    def ols(self) -> U64Vector3: ...
    @property
    def olt(self) -> U64Vector3: ...
    @property
    def lss(self) -> U64Vector3: ...
    @property
    def lst(self) -> U64Vector3: ...
    @property
    def lso(self) -> U64Vector3: ...
    @property
    def lsl(self) -> U64Vector3: ...
    @property
    def lts(self) -> U64Vector3: ...
    @property
    def ltt(self) -> U64Vector3: ...
    @property
    def lto(self) -> U64Vector3: ...
    @property
    def ltl(self) -> U64Vector3: ...
    @property
    def los(self) -> U64Vector3: ...
    @property
    def lot(self) -> U64Vector3: ...
    @property
    def lls(self) -> U64Vector3: ...
    @property
    def llt(self) -> U64Vector3: ...
    @property
    def ssss(self) -> U64Vector4: ...
    @property
    def ssst(self) -> U64Vector4: ...
    @property
    def ssso(self) -> U64Vector4: ...
    @property
    def sssl(self) -> U64Vector4: ...
    @property
    def ssts(self) -> U64Vector4: ...
    @property
    def sstt(self) -> U64Vector4: ...
    @property
    def ssto(self) -> U64Vector4: ...
    @property
    def sstl(self) -> U64Vector4: ...
    @property
    def ssos(self) -> U64Vector4: ...
    @property
    def ssot(self) -> U64Vector4: ...
    @property
    def ssoo(self) -> U64Vector4: ...
    @property
    def ssol(self) -> U64Vector4: ...
    @property
    def ssls(self) -> U64Vector4: ...
    @property
    def sslt(self) -> U64Vector4: ...
    @property
    def sslo(self) -> U64Vector4: ...
    @property
    def ssll(self) -> U64Vector4: ...
    @property
    def stss(self) -> U64Vector4: ...
    @property
    def stst(self) -> U64Vector4: ...
    @property
    def stso(self) -> U64Vector4: ...
    @property
    def stsl(self) -> U64Vector4: ...
    @property
    def stts(self) -> U64Vector4: ...
    @property
    def sttt(self) -> U64Vector4: ...
    @property
    def stto(self) -> U64Vector4: ...
    @property
    def sttl(self) -> U64Vector4: ...
    @property
    def stos(self) -> U64Vector4: ...
    @property
    def stot(self) -> U64Vector4: ...
    @property
    def stoo(self) -> U64Vector4: ...
    @property
    def stol(self) -> U64Vector4: ...
    @property
    def stls(self) -> U64Vector4: ...
    @property
    def stlt(self) -> U64Vector4: ...
    @property
    def stlo(self) -> U64Vector4: ...
    @property
    def stll(self) -> U64Vector4: ...
    @property
    def soss(self) -> U64Vector4: ...
    @property
    def sost(self) -> U64Vector4: ...
    @property
    def soso(self) -> U64Vector4: ...
    @property
    def sosl(self) -> U64Vector4: ...
    @property
    def sots(self) -> U64Vector4: ...
    @property
    def sott(self) -> U64Vector4: ...
    @property
    def soto(self) -> U64Vector4: ...
    @property
    def sotl(self) -> U64Vector4: ...
    @property
    def soos(self) -> U64Vector4: ...
    @property
    def soot(self) -> U64Vector4: ...
    @property
    def sooo(self) -> U64Vector4: ...
    @property
    def sool(self) -> U64Vector4: ...
    @property
    def sols(self) -> U64Vector4: ...
    @property
    def solt(self) -> U64Vector4: ...
    @property
    def solo(self) -> U64Vector4: ...
    @property
    def soll(self) -> U64Vector4: ...
    @property
    def slss(self) -> U64Vector4: ...
    @property
    def slst(self) -> U64Vector4: ...
    @property
    def slso(self) -> U64Vector4: ...
    @property
    def slsl(self) -> U64Vector4: ...
    @property
    def slts(self) -> U64Vector4: ...
    @property
    def sltt(self) -> U64Vector4: ...
    @property
    def slto(self) -> U64Vector4: ...
    @property
    def sltl(self) -> U64Vector4: ...
    @property
    def slos(self) -> U64Vector4: ...
    @property
    def slot(self) -> U64Vector4: ...
    @property
    def sloo(self) -> U64Vector4: ...
    @property
    def slol(self) -> U64Vector4: ...
    @property
    def slls(self) -> U64Vector4: ...
    @property
    def sllt(self) -> U64Vector4: ...
    @property
    def sllo(self) -> U64Vector4: ...
    @property
    def slll(self) -> U64Vector4: ...
    @property
    def tsss(self) -> U64Vector4: ...
    @property
    def tsst(self) -> U64Vector4: ...
    @property
    def tsso(self) -> U64Vector4: ...
    @property
    def tssl(self) -> U64Vector4: ...
    @property
    def tsts(self) -> U64Vector4: ...
    @property
    def tstt(self) -> U64Vector4: ...
    @property
    def tsto(self) -> U64Vector4: ...
    @property
    def tstl(self) -> U64Vector4: ...
    @property
    def tsos(self) -> U64Vector4: ...
    @property
    def tsot(self) -> U64Vector4: ...
    @property
    def tsoo(self) -> U64Vector4: ...
    @property
    def tsol(self) -> U64Vector4: ...
    @property
    def tsls(self) -> U64Vector4: ...
    @property
    def tslt(self) -> U64Vector4: ...
    @property
    def tslo(self) -> U64Vector4: ...
    @property
    def tsll(self) -> U64Vector4: ...
    @property
    def ttss(self) -> U64Vector4: ...
    @property
    def ttst(self) -> U64Vector4: ...
    @property
    def ttso(self) -> U64Vector4: ...
    @property
    def ttsl(self) -> U64Vector4: ...
    @property
    def ttts(self) -> U64Vector4: ...
    @property
    def tttt(self) -> U64Vector4: ...
    @property
    def ttto(self) -> U64Vector4: ...
    @property
    def tttl(self) -> U64Vector4: ...
    @property
    def ttos(self) -> U64Vector4: ...
    @property
    def ttot(self) -> U64Vector4: ...
    @property
    def ttoo(self) -> U64Vector4: ...
    @property
    def ttol(self) -> U64Vector4: ...
    @property
    def ttls(self) -> U64Vector4: ...
    @property
    def ttlt(self) -> U64Vector4: ...
    @property
    def ttlo(self) -> U64Vector4: ...
    @property
    def ttll(self) -> U64Vector4: ...
    @property
    def toss(self) -> U64Vector4: ...
    @property
    def tost(self) -> U64Vector4: ...
    @property
    def toso(self) -> U64Vector4: ...
    @property
    def tosl(self) -> U64Vector4: ...
    @property
    def tots(self) -> U64Vector4: ...
    @property
    def tott(self) -> U64Vector4: ...
    @property
    def toto(self) -> U64Vector4: ...
    @property
    def totl(self) -> U64Vector4: ...
    @property
    def toos(self) -> U64Vector4: ...
    @property
    def toot(self) -> U64Vector4: ...
    @property
    def tooo(self) -> U64Vector4: ...
    @property
    def tool(self) -> U64Vector4: ...
    @property
    def tols(self) -> U64Vector4: ...
    @property
    def tolt(self) -> U64Vector4: ...
    @property
    def tolo(self) -> U64Vector4: ...
    @property
    def toll(self) -> U64Vector4: ...
    @property
    def tlss(self) -> U64Vector4: ...
    @property
    def tlst(self) -> U64Vector4: ...
    @property
    def tlso(self) -> U64Vector4: ...
    @property
    def tlsl(self) -> U64Vector4: ...
    @property
    def tlts(self) -> U64Vector4: ...
    @property
    def tltt(self) -> U64Vector4: ...
    @property
    def tlto(self) -> U64Vector4: ...
    @property
    def tltl(self) -> U64Vector4: ...
    @property
    def tlos(self) -> U64Vector4: ...
    @property
    def tlot(self) -> U64Vector4: ...
    @property
    def tloo(self) -> U64Vector4: ...
    @property
    def tlol(self) -> U64Vector4: ...
    @property
    def tlls(self) -> U64Vector4: ...
    @property
    def tllt(self) -> U64Vector4: ...
    @property
    def tllo(self) -> U64Vector4: ...
    @property
    def tlll(self) -> U64Vector4: ...
    @property
    def osss(self) -> U64Vector4: ...
    @property
    def osst(self) -> U64Vector4: ...
    @property
    def osso(self) -> U64Vector4: ...
    @property
    def ossl(self) -> U64Vector4: ...
    @property
    def osts(self) -> U64Vector4: ...
    @property
    def ostt(self) -> U64Vector4: ...
    @property
    def osto(self) -> U64Vector4: ...
    @property
    def ostl(self) -> U64Vector4: ...
    @property
    def osos(self) -> U64Vector4: ...
    @property
    def osot(self) -> U64Vector4: ...
    @property
    def osoo(self) -> U64Vector4: ...
    @property
    def osol(self) -> U64Vector4: ...
    @property
    def osls(self) -> U64Vector4: ...
    @property
    def oslt(self) -> U64Vector4: ...
    @property
    def oslo(self) -> U64Vector4: ...
    @property
    def osll(self) -> U64Vector4: ...
    @property
    def otss(self) -> U64Vector4: ...
    @property
    def otst(self) -> U64Vector4: ...
    @property
    def otso(self) -> U64Vector4: ...
    @property
    def otsl(self) -> U64Vector4: ...
    @property
    def otts(self) -> U64Vector4: ...
    @property
    def ottt(self) -> U64Vector4: ...
    @property
    def otto(self) -> U64Vector4: ...
    @property
    def ottl(self) -> U64Vector4: ...
    @property
    def otos(self) -> U64Vector4: ...
    @property
    def otot(self) -> U64Vector4: ...
    @property
    def otoo(self) -> U64Vector4: ...
    @property
    def otol(self) -> U64Vector4: ...
    @property
    def otls(self) -> U64Vector4: ...
    @property
    def otlt(self) -> U64Vector4: ...
    @property
    def otlo(self) -> U64Vector4: ...
    @property
    def otll(self) -> U64Vector4: ...
    @property
    def ooss(self) -> U64Vector4: ...
    @property
    def oost(self) -> U64Vector4: ...
    @property
    def ooso(self) -> U64Vector4: ...
    @property
    def oosl(self) -> U64Vector4: ...
    @property
    def oots(self) -> U64Vector4: ...
    @property
    def oott(self) -> U64Vector4: ...
    @property
    def ooto(self) -> U64Vector4: ...
    @property
    def ootl(self) -> U64Vector4: ...
    @property
    def ooos(self) -> U64Vector4: ...
    @property
    def ooot(self) -> U64Vector4: ...
    @property
    def ools(self) -> U64Vector4: ...
    @property
    def oolt(self) -> U64Vector4: ...
    @property
    def olss(self) -> U64Vector4: ...
    @property
    def olst(self) -> U64Vector4: ...
    @property
    def olso(self) -> U64Vector4: ...
    @property
    def olsl(self) -> U64Vector4: ...
    @property
    def olts(self) -> U64Vector4: ...
    @property
    def oltt(self) -> U64Vector4: ...
    @property
    def olto(self) -> U64Vector4: ...
    @property
    def oltl(self) -> U64Vector4: ...
    @property
    def olos(self) -> U64Vector4: ...
    @property
    def olot(self) -> U64Vector4: ...
    @property
    def olls(self) -> U64Vector4: ...
    @property
    def ollt(self) -> U64Vector4: ...
    @property
    def lsss(self) -> U64Vector4: ...
    @property
    def lsst(self) -> U64Vector4: ...
    @property
    def lsso(self) -> U64Vector4: ...
    @property
    def lssl(self) -> U64Vector4: ...
    @property
    def lsts(self) -> U64Vector4: ...
    @property
    def lstt(self) -> U64Vector4: ...
    @property
    def lsto(self) -> U64Vector4: ...
    @property
    def lstl(self) -> U64Vector4: ...
    @property
    def lsos(self) -> U64Vector4: ...
    @property
    def lsot(self) -> U64Vector4: ...
    @property
    def lsoo(self) -> U64Vector4: ...
    @property
    def lsol(self) -> U64Vector4: ...
    @property
    def lsls(self) -> U64Vector4: ...
    @property
    def lslt(self) -> U64Vector4: ...
    @property
    def lslo(self) -> U64Vector4: ...
    @property
    def lsll(self) -> U64Vector4: ...
    @property
    def ltss(self) -> U64Vector4: ...
    @property
    def ltst(self) -> U64Vector4: ...
    @property
    def ltso(self) -> U64Vector4: ...
    @property
    def ltsl(self) -> U64Vector4: ...
    @property
    def ltts(self) -> U64Vector4: ...
    @property
    def lttt(self) -> U64Vector4: ...
    @property
    def ltto(self) -> U64Vector4: ...
    @property
    def lttl(self) -> U64Vector4: ...
    @property
    def ltos(self) -> U64Vector4: ...
    @property
    def ltot(self) -> U64Vector4: ...
    @property
    def ltoo(self) -> U64Vector4: ...
    @property
    def ltol(self) -> U64Vector4: ...
    @property
    def ltls(self) -> U64Vector4: ...
    @property
    def ltlt(self) -> U64Vector4: ...
    @property
    def ltlo(self) -> U64Vector4: ...
    @property
    def ltll(self) -> U64Vector4: ...
    @property
    def loss(self) -> U64Vector4: ...
    @property
    def lost(self) -> U64Vector4: ...
    @property
    def loso(self) -> U64Vector4: ...
    @property
    def losl(self) -> U64Vector4: ...
    @property
    def lots(self) -> U64Vector4: ...
    @property
    def lott(self) -> U64Vector4: ...
    @property
    def loto(self) -> U64Vector4: ...
    @property
    def lotl(self) -> U64Vector4: ...
    @property
    def loos(self) -> U64Vector4: ...
    @property
    def loot(self) -> U64Vector4: ...
    @property
    def lols(self) -> U64Vector4: ...
    @property
    def lolt(self) -> U64Vector4: ...
    @property
    def llss(self) -> U64Vector4: ...
    @property
    def llst(self) -> U64Vector4: ...
    @property
    def llso(self) -> U64Vector4: ...
    @property
    def llsl(self) -> U64Vector4: ...
    @property
    def llts(self) -> U64Vector4: ...
    @property
    def lltt(self) -> U64Vector4: ...
    @property
    def llto(self) -> U64Vector4: ...
    @property
    def lltl(self) -> U64Vector4: ...
    @property
    def llos(self) -> U64Vector4: ...
    @property
    def llot(self) -> U64Vector4: ...
    @property
    def llls(self) -> U64Vector4: ...
    @property
    def lllt(self) -> U64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U64Vector2: ...
    @property
    def uv(self) -> U64Vector2: ...
    @property
    def uo(self) -> U64Vector2: ...
    @property
    def ul(self) -> U64Vector2: ...
    @property
    def vu(self) -> U64Vector2: ...
    @property
    def vv(self) -> U64Vector2: ...
    @property
    def vo(self) -> U64Vector2: ...
    @property
    def vl(self) -> U64Vector2: ...
    @property
    def ou(self) -> U64Vector2: ...
    @property
    def ov(self) -> U64Vector2: ...
    @property
    def lu(self) -> U64Vector2: ...
    @property
    def lv(self) -> U64Vector2: ...
    @property
    def uuu(self) -> U64Vector3: ...
    @property
    def uuv(self) -> U64Vector3: ...
    @property
    def uuo(self) -> U64Vector3: ...
    @property
    def uul(self) -> U64Vector3: ...
    @property
    def uvu(self) -> U64Vector3: ...
    @property
    def uvv(self) -> U64Vector3: ...
    @property
    def uvo(self) -> U64Vector3: ...
    @property
    def uvl(self) -> U64Vector3: ...
    @property
    def uou(self) -> U64Vector3: ...
    @property
    def uov(self) -> U64Vector3: ...
    @property
    def uoo(self) -> U64Vector3: ...
    @property
    def uol(self) -> U64Vector3: ...
    @property
    def ulu(self) -> U64Vector3: ...
    @property
    def ulv(self) -> U64Vector3: ...
    @property
    def ulo(self) -> U64Vector3: ...
    @property
    def ull(self) -> U64Vector3: ...
    @property
    def vuu(self) -> U64Vector3: ...
    @property
    def vuv(self) -> U64Vector3: ...
    @property
    def vuo(self) -> U64Vector3: ...
    @property
    def vul(self) -> U64Vector3: ...
    @property
    def vvu(self) -> U64Vector3: ...
    @property
    def vvv(self) -> U64Vector3: ...
    @property
    def vvo(self) -> U64Vector3: ...
    @property
    def vvl(self) -> U64Vector3: ...
    @property
    def vou(self) -> U64Vector3: ...
    @property
    def vov(self) -> U64Vector3: ...
    @property
    def voo(self) -> U64Vector3: ...
    @property
    def vol(self) -> U64Vector3: ...
    @property
    def vlu(self) -> U64Vector3: ...
    @property
    def vlv(self) -> U64Vector3: ...
    @property
    def vlo(self) -> U64Vector3: ...
    @property
    def vll(self) -> U64Vector3: ...
    @property
    def ouu(self) -> U64Vector3: ...
    @property
    def ouv(self) -> U64Vector3: ...
    @property
    def ouo(self) -> U64Vector3: ...
    @property
    def oul(self) -> U64Vector3: ...
    @property
    def ovu(self) -> U64Vector3: ...
    @property
    def ovv(self) -> U64Vector3: ...
    @property
    def ovo(self) -> U64Vector3: ...
    @property
    def ovl(self) -> U64Vector3: ...
    @property
    def oou(self) -> U64Vector3: ...
    @property
    def oov(self) -> U64Vector3: ...
    @property
    def olu(self) -> U64Vector3: ...
    @property
    def olv(self) -> U64Vector3: ...
    @property
    def luu(self) -> U64Vector3: ...
    @property
    def luv(self) -> U64Vector3: ...
    @property
    def luo(self) -> U64Vector3: ...
    @property
    def lul(self) -> U64Vector3: ...
    @property
    def lvu(self) -> U64Vector3: ...
    @property
    def lvv(self) -> U64Vector3: ...
    @property
    def lvo(self) -> U64Vector3: ...
    @property
    def lvl(self) -> U64Vector3: ...
    @property
    def lou(self) -> U64Vector3: ...
    @property
    def lov(self) -> U64Vector3: ...
    @property
    def llu(self) -> U64Vector3: ...
    @property
    def llv(self) -> U64Vector3: ...
    @property
    def uuuu(self) -> U64Vector4: ...
    @property
    def uuuv(self) -> U64Vector4: ...
    @property
    def uuuo(self) -> U64Vector4: ...
    @property
    def uuul(self) -> U64Vector4: ...
    @property
    def uuvu(self) -> U64Vector4: ...
    @property
    def uuvv(self) -> U64Vector4: ...
    @property
    def uuvo(self) -> U64Vector4: ...
    @property
    def uuvl(self) -> U64Vector4: ...
    @property
    def uuou(self) -> U64Vector4: ...
    @property
    def uuov(self) -> U64Vector4: ...
    @property
    def uuoo(self) -> U64Vector4: ...
    @property
    def uuol(self) -> U64Vector4: ...
    @property
    def uulu(self) -> U64Vector4: ...
    @property
    def uulv(self) -> U64Vector4: ...
    @property
    def uulo(self) -> U64Vector4: ...
    @property
    def uull(self) -> U64Vector4: ...
    @property
    def uvuu(self) -> U64Vector4: ...
    @property
    def uvuv(self) -> U64Vector4: ...
    @property
    def uvuo(self) -> U64Vector4: ...
    @property
    def uvul(self) -> U64Vector4: ...
    @property
    def uvvu(self) -> U64Vector4: ...
    @property
    def uvvv(self) -> U64Vector4: ...
    @property
    def uvvo(self) -> U64Vector4: ...
    @property
    def uvvl(self) -> U64Vector4: ...
    @property
    def uvou(self) -> U64Vector4: ...
    @property
    def uvov(self) -> U64Vector4: ...
    @property
    def uvoo(self) -> U64Vector4: ...
    @property
    def uvol(self) -> U64Vector4: ...
    @property
    def uvlu(self) -> U64Vector4: ...
    @property
    def uvlv(self) -> U64Vector4: ...
    @property
    def uvlo(self) -> U64Vector4: ...
    @property
    def uvll(self) -> U64Vector4: ...
    @property
    def uouu(self) -> U64Vector4: ...
    @property
    def uouv(self) -> U64Vector4: ...
    @property
    def uouo(self) -> U64Vector4: ...
    @property
    def uoul(self) -> U64Vector4: ...
    @property
    def uovu(self) -> U64Vector4: ...
    @property
    def uovv(self) -> U64Vector4: ...
    @property
    def uovo(self) -> U64Vector4: ...
    @property
    def uovl(self) -> U64Vector4: ...
    @property
    def uoou(self) -> U64Vector4: ...
    @property
    def uoov(self) -> U64Vector4: ...
    @property
    def uooo(self) -> U64Vector4: ...
    @property
    def uool(self) -> U64Vector4: ...
    @property
    def uolu(self) -> U64Vector4: ...
    @property
    def uolv(self) -> U64Vector4: ...
    @property
    def uolo(self) -> U64Vector4: ...
    @property
    def uoll(self) -> U64Vector4: ...
    @property
    def uluu(self) -> U64Vector4: ...
    @property
    def uluv(self) -> U64Vector4: ...
    @property
    def uluo(self) -> U64Vector4: ...
    @property
    def ulul(self) -> U64Vector4: ...
    @property
    def ulvu(self) -> U64Vector4: ...
    @property
    def ulvv(self) -> U64Vector4: ...
    @property
    def ulvo(self) -> U64Vector4: ...
    @property
    def ulvl(self) -> U64Vector4: ...
    @property
    def ulou(self) -> U64Vector4: ...
    @property
    def ulov(self) -> U64Vector4: ...
    @property
    def uloo(self) -> U64Vector4: ...
    @property
    def ulol(self) -> U64Vector4: ...
    @property
    def ullu(self) -> U64Vector4: ...
    @property
    def ullv(self) -> U64Vector4: ...
    @property
    def ullo(self) -> U64Vector4: ...
    @property
    def ulll(self) -> U64Vector4: ...
    @property
    def vuuu(self) -> U64Vector4: ...
    @property
    def vuuv(self) -> U64Vector4: ...
    @property
    def vuuo(self) -> U64Vector4: ...
    @property
    def vuul(self) -> U64Vector4: ...
    @property
    def vuvu(self) -> U64Vector4: ...
    @property
    def vuvv(self) -> U64Vector4: ...
    @property
    def vuvo(self) -> U64Vector4: ...
    @property
    def vuvl(self) -> U64Vector4: ...
    @property
    def vuou(self) -> U64Vector4: ...
    @property
    def vuov(self) -> U64Vector4: ...
    @property
    def vuoo(self) -> U64Vector4: ...
    @property
    def vuol(self) -> U64Vector4: ...
    @property
    def vulu(self) -> U64Vector4: ...
    @property
    def vulv(self) -> U64Vector4: ...
    @property
    def vulo(self) -> U64Vector4: ...
    @property
    def vull(self) -> U64Vector4: ...
    @property
    def vvuu(self) -> U64Vector4: ...
    @property
    def vvuv(self) -> U64Vector4: ...
    @property
    def vvuo(self) -> U64Vector4: ...
    @property
    def vvul(self) -> U64Vector4: ...
    @property
    def vvvu(self) -> U64Vector4: ...
    @property
    def vvvv(self) -> U64Vector4: ...
    @property
    def vvvo(self) -> U64Vector4: ...
    @property
    def vvvl(self) -> U64Vector4: ...
    @property
    def vvou(self) -> U64Vector4: ...
    @property
    def vvov(self) -> U64Vector4: ...
    @property
    def vvoo(self) -> U64Vector4: ...
    @property
    def vvol(self) -> U64Vector4: ...
    @property
    def vvlu(self) -> U64Vector4: ...
    @property
    def vvlv(self) -> U64Vector4: ...
    @property
    def vvlo(self) -> U64Vector4: ...
    @property
    def vvll(self) -> U64Vector4: ...
    @property
    def vouu(self) -> U64Vector4: ...
    @property
    def vouv(self) -> U64Vector4: ...
    @property
    def vouo(self) -> U64Vector4: ...
    @property
    def voul(self) -> U64Vector4: ...
    @property
    def vovu(self) -> U64Vector4: ...
    @property
    def vovv(self) -> U64Vector4: ...
    @property
    def vovo(self) -> U64Vector4: ...
    @property
    def vovl(self) -> U64Vector4: ...
    @property
    def voou(self) -> U64Vector4: ...
    @property
    def voov(self) -> U64Vector4: ...
    @property
    def vooo(self) -> U64Vector4: ...
    @property
    def vool(self) -> U64Vector4: ...
    @property
    def volu(self) -> U64Vector4: ...
    @property
    def volv(self) -> U64Vector4: ...
    @property
    def volo(self) -> U64Vector4: ...
    @property
    def voll(self) -> U64Vector4: ...
    @property
    def vluu(self) -> U64Vector4: ...
    @property
    def vluv(self) -> U64Vector4: ...
    @property
    def vluo(self) -> U64Vector4: ...
    @property
    def vlul(self) -> U64Vector4: ...
    @property
    def vlvu(self) -> U64Vector4: ...
    @property
    def vlvv(self) -> U64Vector4: ...
    @property
    def vlvo(self) -> U64Vector4: ...
    @property
    def vlvl(self) -> U64Vector4: ...
    @property
    def vlou(self) -> U64Vector4: ...
    @property
    def vlov(self) -> U64Vector4: ...
    @property
    def vloo(self) -> U64Vector4: ...
    @property
    def vlol(self) -> U64Vector4: ...
    @property
    def vllu(self) -> U64Vector4: ...
    @property
    def vllv(self) -> U64Vector4: ...
    @property
    def vllo(self) -> U64Vector4: ...
    @property
    def vlll(self) -> U64Vector4: ...
    @property
    def ouuu(self) -> U64Vector4: ...
    @property
    def ouuv(self) -> U64Vector4: ...
    @property
    def ouuo(self) -> U64Vector4: ...
    @property
    def ouul(self) -> U64Vector4: ...
    @property
    def ouvu(self) -> U64Vector4: ...
    @property
    def ouvv(self) -> U64Vector4: ...
    @property
    def ouvo(self) -> U64Vector4: ...
    @property
    def ouvl(self) -> U64Vector4: ...
    @property
    def ouou(self) -> U64Vector4: ...
    @property
    def ouov(self) -> U64Vector4: ...
    @property
    def ouoo(self) -> U64Vector4: ...
    @property
    def ouol(self) -> U64Vector4: ...
    @property
    def oulu(self) -> U64Vector4: ...
    @property
    def oulv(self) -> U64Vector4: ...
    @property
    def oulo(self) -> U64Vector4: ...
    @property
    def oull(self) -> U64Vector4: ...
    @property
    def ovuu(self) -> U64Vector4: ...
    @property
    def ovuv(self) -> U64Vector4: ...
    @property
    def ovuo(self) -> U64Vector4: ...
    @property
    def ovul(self) -> U64Vector4: ...
    @property
    def ovvu(self) -> U64Vector4: ...
    @property
    def ovvv(self) -> U64Vector4: ...
    @property
    def ovvo(self) -> U64Vector4: ...
    @property
    def ovvl(self) -> U64Vector4: ...
    @property
    def ovou(self) -> U64Vector4: ...
    @property
    def ovov(self) -> U64Vector4: ...
    @property
    def ovoo(self) -> U64Vector4: ...
    @property
    def ovol(self) -> U64Vector4: ...
    @property
    def ovlu(self) -> U64Vector4: ...
    @property
    def ovlv(self) -> U64Vector4: ...
    @property
    def ovlo(self) -> U64Vector4: ...
    @property
    def ovll(self) -> U64Vector4: ...
    @property
    def oouu(self) -> U64Vector4: ...
    @property
    def oouv(self) -> U64Vector4: ...
    @property
    def oouo(self) -> U64Vector4: ...
    @property
    def ooul(self) -> U64Vector4: ...
    @property
    def oovu(self) -> U64Vector4: ...
    @property
    def oovv(self) -> U64Vector4: ...
    @property
    def oovo(self) -> U64Vector4: ...
    @property
    def oovl(self) -> U64Vector4: ...
    @property
    def ooou(self) -> U64Vector4: ...
    @property
    def ooov(self) -> U64Vector4: ...
    @property
    def oolu(self) -> U64Vector4: ...
    @property
    def oolv(self) -> U64Vector4: ...
    @property
    def oluu(self) -> U64Vector4: ...
    @property
    def oluv(self) -> U64Vector4: ...
    @property
    def oluo(self) -> U64Vector4: ...
    @property
    def olul(self) -> U64Vector4: ...
    @property
    def olvu(self) -> U64Vector4: ...
    @property
    def olvv(self) -> U64Vector4: ...
    @property
    def olvo(self) -> U64Vector4: ...
    @property
    def olvl(self) -> U64Vector4: ...
    @property
    def olou(self) -> U64Vector4: ...
    @property
    def olov(self) -> U64Vector4: ...
    @property
    def ollu(self) -> U64Vector4: ...
    @property
    def ollv(self) -> U64Vector4: ...
    @property
    def luuu(self) -> U64Vector4: ...
    @property
    def luuv(self) -> U64Vector4: ...
    @property
    def luuo(self) -> U64Vector4: ...
    @property
    def luul(self) -> U64Vector4: ...
    @property
    def luvu(self) -> U64Vector4: ...
    @property
    def luvv(self) -> U64Vector4: ...
    @property
    def luvo(self) -> U64Vector4: ...
    @property
    def luvl(self) -> U64Vector4: ...
    @property
    def luou(self) -> U64Vector4: ...
    @property
    def luov(self) -> U64Vector4: ...
    @property
    def luoo(self) -> U64Vector4: ...
    @property
    def luol(self) -> U64Vector4: ...
    @property
    def lulu(self) -> U64Vector4: ...
    @property
    def lulv(self) -> U64Vector4: ...
    @property
    def lulo(self) -> U64Vector4: ...
    @property
    def lull(self) -> U64Vector4: ...
    @property
    def lvuu(self) -> U64Vector4: ...
    @property
    def lvuv(self) -> U64Vector4: ...
    @property
    def lvuo(self) -> U64Vector4: ...
    @property
    def lvul(self) -> U64Vector4: ...
    @property
    def lvvu(self) -> U64Vector4: ...
    @property
    def lvvv(self) -> U64Vector4: ...
    @property
    def lvvo(self) -> U64Vector4: ...
    @property
    def lvvl(self) -> U64Vector4: ...
    @property
    def lvou(self) -> U64Vector4: ...
    @property
    def lvov(self) -> U64Vector4: ...
    @property
    def lvoo(self) -> U64Vector4: ...
    @property
    def lvol(self) -> U64Vector4: ...
    @property
    def lvlu(self) -> U64Vector4: ...
    @property
    def lvlv(self) -> U64Vector4: ...
    @property
    def lvlo(self) -> U64Vector4: ...
    @property
    def lvll(self) -> U64Vector4: ...
    @property
    def louu(self) -> U64Vector4: ...
    @property
    def louv(self) -> U64Vector4: ...
    @property
    def louo(self) -> U64Vector4: ...
    @property
    def loul(self) -> U64Vector4: ...
    @property
    def lovu(self) -> U64Vector4: ...
    @property
    def lovv(self) -> U64Vector4: ...
    @property
    def lovo(self) -> U64Vector4: ...
    @property
    def lovl(self) -> U64Vector4: ...
    @property
    def loou(self) -> U64Vector4: ...
    @property
    def loov(self) -> U64Vector4: ...
    @property
    def lolu(self) -> U64Vector4: ...
    @property
    def lolv(self) -> U64Vector4: ...
    @property
    def lluu(self) -> U64Vector4: ...
    @property
    def lluv(self) -> U64Vector4: ...
    @property
    def lluo(self) -> U64Vector4: ...
    @property
    def llul(self) -> U64Vector4: ...
    @property
    def llvu(self) -> U64Vector4: ...
    @property
    def llvv(self) -> U64Vector4: ...
    @property
    def llvo(self) -> U64Vector4: ...
    @property
    def llvl(self) -> U64Vector4: ...
    @property
    def llou(self) -> U64Vector4: ...
    @property
    def llov(self) -> U64Vector4: ...
    @property
    def lllu(self) -> U64Vector4: ...
    @property
    def lllv(self) -> U64Vector4: ...
    @property
    def oo(self) -> U64Vector2: ...
    @property
    def ol(self) -> U64Vector2: ...
    @property
    def lo(self) -> U64Vector2: ...
    @property
    def ll(self) -> U64Vector2: ...
    @property
    def ooo(self) -> U64Vector3: ...
    @property
    def ool(self) -> U64Vector3: ...
    @property
    def olo(self) -> U64Vector3: ...
    @property
    def oll(self) -> U64Vector3: ...
    @property
    def loo(self) -> U64Vector3: ...
    @property
    def lol(self) -> U64Vector3: ...
    @property
    def llo(self) -> U64Vector3: ...
    @property
    def lll(self) -> U64Vector3: ...
    @property
    def oooo(self) -> U64Vector4: ...
    @property
    def oool(self) -> U64Vector4: ...
    @property
    def oolo(self) -> U64Vector4: ...
    @property
    def ooll(self) -> U64Vector4: ...
    @property
    def oloo(self) -> U64Vector4: ...
    @property
    def olol(self) -> U64Vector4: ...
    @property
    def ollo(self) -> U64Vector4: ...
    @property
    def olll(self) -> U64Vector4: ...
    @property
    def looo(self) -> U64Vector4: ...
    @property
    def lool(self) -> U64Vector4: ...
    @property
    def lolo(self) -> U64Vector4: ...
    @property
    def loll(self) -> U64Vector4: ...
    @property
    def lloo(self) -> U64Vector4: ...
    @property
    def llol(self) -> U64Vector4: ...
    @property
    def lllo(self) -> U64Vector4: ...
    @property
    def llll(self) -> U64Vector4: ...
    def min(self, n: Number, /) -> U64Vector2: ...
    def max(self, n: Number, /) -> U64Vector2: ...
    def clamp(self, min: Number, max: Number, /) -> U64Vector2: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U64Vector2Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector2: ...

@final
class U64Vector2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U64Vector2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U64Vector2]: ...
    @overload
    def __getitem__(self, index: int) -> U64Vector2: ...
    @overload
    def __getitem__(self, index: slice) -> U64Vector2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector2Array: ...
    @classmethod
    def get_component_type(cls) -> type[U64Vector2]: ...

@final
class BVector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[_bool]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> _bool: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: BVector3) -> BVector3: ...
    @overload
    def __add__(self, other: Number) -> BVector3: ...
    @overload
    def __radd__(self, other: BVector3) -> BVector3: ...
    @overload
    def __radd__(self, other: Number) -> BVector3: ...
    @overload
    def __sub__(self, other: BVector3) -> BVector3: ...
    @overload
    def __sub__(self, other: Number) -> BVector3: ...
    @overload
    def __rsub__(self, other: BVector3) -> BVector3: ...
    @overload
    def __rsub__(self, other: Number) -> BVector3: ...
    @overload
    def __mul__(self, other: BVector3) -> BVector3: ...
    @overload
    def __mul__(self, other: Number) -> BVector3: ...
    @overload
    def __rmul__(self, other: BVector3) -> BVector3: ...
    @overload
    def __rmul__(self, other: Number) -> BVector3: ...
    def __neg__(self) -> BVector3: ...
    def __abs__(self) -> BVector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> _bool: ...
    @property
    def y(self) -> _bool: ...
    @property
    def z(self) -> _bool: ...
    @property
    def xx(self) -> BVector2: ...
    @property
    def xy(self) -> BVector2: ...
    @property
    def xz(self) -> BVector2: ...
    @property
    def xo(self) -> BVector2: ...
    @property
    def xl(self) -> BVector2: ...
    @property
    def yx(self) -> BVector2: ...
    @property
    def yy(self) -> BVector2: ...
    @property
    def yz(self) -> BVector2: ...
    @property
    def yo(self) -> BVector2: ...
    @property
    def yl(self) -> BVector2: ...
    @property
    def zx(self) -> BVector2: ...
    @property
    def zy(self) -> BVector2: ...
    @property
    def zz(self) -> BVector2: ...
    @property
    def zo(self) -> BVector2: ...
    @property
    def zl(self) -> BVector2: ...
    @property
    def ox(self) -> BVector2: ...
    @property
    def oy(self) -> BVector2: ...
    @property
    def oz(self) -> BVector2: ...
    @property
    def lx(self) -> BVector2: ...
    @property
    def ly(self) -> BVector2: ...
    @property
    def lz(self) -> BVector2: ...
    @property
    def xxx(self) -> BVector3: ...
    @property
    def xxy(self) -> BVector3: ...
    @property
    def xxz(self) -> BVector3: ...
    @property
    def xxo(self) -> BVector3: ...
    @property
    def xxl(self) -> BVector3: ...
    @property
    def xyx(self) -> BVector3: ...
    @property
    def xyy(self) -> BVector3: ...
    @property
    def xyz(self) -> BVector3: ...
    @property
    def xyo(self) -> BVector3: ...
    @property
    def xyl(self) -> BVector3: ...
    @property
    def xzx(self) -> BVector3: ...
    @property
    def xzy(self) -> BVector3: ...
    @property
    def xzz(self) -> BVector3: ...
    @property
    def xzo(self) -> BVector3: ...
    @property
    def xzl(self) -> BVector3: ...
    @property
    def xox(self) -> BVector3: ...
    @property
    def xoy(self) -> BVector3: ...
    @property
    def xoz(self) -> BVector3: ...
    @property
    def xoo(self) -> BVector3: ...
    @property
    def xol(self) -> BVector3: ...
    @property
    def xlx(self) -> BVector3: ...
    @property
    def xly(self) -> BVector3: ...
    @property
    def xlz(self) -> BVector3: ...
    @property
    def xlo(self) -> BVector3: ...
    @property
    def xll(self) -> BVector3: ...
    @property
    def yxx(self) -> BVector3: ...
    @property
    def yxy(self) -> BVector3: ...
    @property
    def yxz(self) -> BVector3: ...
    @property
    def yxo(self) -> BVector3: ...
    @property
    def yxl(self) -> BVector3: ...
    @property
    def yyx(self) -> BVector3: ...
    @property
    def yyy(self) -> BVector3: ...
    @property
    def yyz(self) -> BVector3: ...
    @property
    def yyo(self) -> BVector3: ...
    @property
    def yyl(self) -> BVector3: ...
    @property
    def yzx(self) -> BVector3: ...
    @property
    def yzy(self) -> BVector3: ...
    @property
    def yzz(self) -> BVector3: ...
    @property
    def yzo(self) -> BVector3: ...
    @property
    def yzl(self) -> BVector3: ...
    @property
    def yox(self) -> BVector3: ...
    @property
    def yoy(self) -> BVector3: ...
    @property
    def yoz(self) -> BVector3: ...
    @property
    def yoo(self) -> BVector3: ...
    @property
    def yol(self) -> BVector3: ...
    @property
    def ylx(self) -> BVector3: ...
    @property
    def yly(self) -> BVector3: ...
    @property
    def ylz(self) -> BVector3: ...
    @property
    def ylo(self) -> BVector3: ...
    @property
    def yll(self) -> BVector3: ...
    @property
    def zxx(self) -> BVector3: ...
    @property
    def zxy(self) -> BVector3: ...
    @property
    def zxz(self) -> BVector3: ...
    @property
    def zxo(self) -> BVector3: ...
    @property
    def zxl(self) -> BVector3: ...
    @property
    def zyx(self) -> BVector3: ...
    @property
    def zyy(self) -> BVector3: ...
    @property
    def zyz(self) -> BVector3: ...
    @property
    def zyo(self) -> BVector3: ...
    @property
    def zyl(self) -> BVector3: ...
    @property
    def zzx(self) -> BVector3: ...
    @property
    def zzy(self) -> BVector3: ...
    @property
    def zzz(self) -> BVector3: ...
    @property
    def zzo(self) -> BVector3: ...
    @property
    def zzl(self) -> BVector3: ...
    @property
    def zox(self) -> BVector3: ...
    @property
    def zoy(self) -> BVector3: ...
    @property
    def zoz(self) -> BVector3: ...
    @property
    def zoo(self) -> BVector3: ...
    @property
    def zol(self) -> BVector3: ...
    @property
    def zlx(self) -> BVector3: ...
    @property
    def zly(self) -> BVector3: ...
    @property
    def zlz(self) -> BVector3: ...
    @property
    def zlo(self) -> BVector3: ...
    @property
    def zll(self) -> BVector3: ...
    @property
    def oxx(self) -> BVector3: ...
    @property
    def oxy(self) -> BVector3: ...
    @property
    def oxz(self) -> BVector3: ...
    @property
    def oxo(self) -> BVector3: ...
    @property
    def oxl(self) -> BVector3: ...
    @property
    def oyx(self) -> BVector3: ...
    @property
    def oyy(self) -> BVector3: ...
    @property
    def oyz(self) -> BVector3: ...
    @property
    def oyo(self) -> BVector3: ...
    @property
    def oyl(self) -> BVector3: ...
    @property
    def ozx(self) -> BVector3: ...
    @property
    def ozy(self) -> BVector3: ...
    @property
    def ozz(self) -> BVector3: ...
    @property
    def ozo(self) -> BVector3: ...
    @property
    def ozl(self) -> BVector3: ...
    @property
    def oox(self) -> BVector3: ...
    @property
    def ooy(self) -> BVector3: ...
    @property
    def ooz(self) -> BVector3: ...
    @property
    def olx(self) -> BVector3: ...
    @property
    def oly(self) -> BVector3: ...
    @property
    def olz(self) -> BVector3: ...
    @property
    def lxx(self) -> BVector3: ...
    @property
    def lxy(self) -> BVector3: ...
    @property
    def lxz(self) -> BVector3: ...
    @property
    def lxo(self) -> BVector3: ...
    @property
    def lxl(self) -> BVector3: ...
    @property
    def lyx(self) -> BVector3: ...
    @property
    def lyy(self) -> BVector3: ...
    @property
    def lyz(self) -> BVector3: ...
    @property
    def lyo(self) -> BVector3: ...
    @property
    def lyl(self) -> BVector3: ...
    @property
    def lzx(self) -> BVector3: ...
    @property
    def lzy(self) -> BVector3: ...
    @property
    def lzz(self) -> BVector3: ...
    @property
    def lzo(self) -> BVector3: ...
    @property
    def lzl(self) -> BVector3: ...
    @property
    def lox(self) -> BVector3: ...
    @property
    def loy(self) -> BVector3: ...
    @property
    def loz(self) -> BVector3: ...
    @property
    def llx(self) -> BVector3: ...
    @property
    def lly(self) -> BVector3: ...
    @property
    def llz(self) -> BVector3: ...
    @property
    def xxxx(self) -> BVector4: ...
    @property
    def xxxy(self) -> BVector4: ...
    @property
    def xxxz(self) -> BVector4: ...
    @property
    def xxxo(self) -> BVector4: ...
    @property
    def xxxl(self) -> BVector4: ...
    @property
    def xxyx(self) -> BVector4: ...
    @property
    def xxyy(self) -> BVector4: ...
    @property
    def xxyz(self) -> BVector4: ...
    @property
    def xxyo(self) -> BVector4: ...
    @property
    def xxyl(self) -> BVector4: ...
    @property
    def xxzx(self) -> BVector4: ...
    @property
    def xxzy(self) -> BVector4: ...
    @property
    def xxzz(self) -> BVector4: ...
    @property
    def xxzo(self) -> BVector4: ...
    @property
    def xxzl(self) -> BVector4: ...
    @property
    def xxox(self) -> BVector4: ...
    @property
    def xxoy(self) -> BVector4: ...
    @property
    def xxoz(self) -> BVector4: ...
    @property
    def xxoo(self) -> BVector4: ...
    @property
    def xxol(self) -> BVector4: ...
    @property
    def xxlx(self) -> BVector4: ...
    @property
    def xxly(self) -> BVector4: ...
    @property
    def xxlz(self) -> BVector4: ...
    @property
    def xxlo(self) -> BVector4: ...
    @property
    def xxll(self) -> BVector4: ...
    @property
    def xyxx(self) -> BVector4: ...
    @property
    def xyxy(self) -> BVector4: ...
    @property
    def xyxz(self) -> BVector4: ...
    @property
    def xyxo(self) -> BVector4: ...
    @property
    def xyxl(self) -> BVector4: ...
    @property
    def xyyx(self) -> BVector4: ...
    @property
    def xyyy(self) -> BVector4: ...
    @property
    def xyyz(self) -> BVector4: ...
    @property
    def xyyo(self) -> BVector4: ...
    @property
    def xyyl(self) -> BVector4: ...
    @property
    def xyzx(self) -> BVector4: ...
    @property
    def xyzy(self) -> BVector4: ...
    @property
    def xyzz(self) -> BVector4: ...
    @property
    def xyzo(self) -> BVector4: ...
    @property
    def xyzl(self) -> BVector4: ...
    @property
    def xyox(self) -> BVector4: ...
    @property
    def xyoy(self) -> BVector4: ...
    @property
    def xyoz(self) -> BVector4: ...
    @property
    def xyoo(self) -> BVector4: ...
    @property
    def xyol(self) -> BVector4: ...
    @property
    def xylx(self) -> BVector4: ...
    @property
    def xyly(self) -> BVector4: ...
    @property
    def xylz(self) -> BVector4: ...
    @property
    def xylo(self) -> BVector4: ...
    @property
    def xyll(self) -> BVector4: ...
    @property
    def xzxx(self) -> BVector4: ...
    @property
    def xzxy(self) -> BVector4: ...
    @property
    def xzxz(self) -> BVector4: ...
    @property
    def xzxo(self) -> BVector4: ...
    @property
    def xzxl(self) -> BVector4: ...
    @property
    def xzyx(self) -> BVector4: ...
    @property
    def xzyy(self) -> BVector4: ...
    @property
    def xzyz(self) -> BVector4: ...
    @property
    def xzyo(self) -> BVector4: ...
    @property
    def xzyl(self) -> BVector4: ...
    @property
    def xzzx(self) -> BVector4: ...
    @property
    def xzzy(self) -> BVector4: ...
    @property
    def xzzz(self) -> BVector4: ...
    @property
    def xzzo(self) -> BVector4: ...
    @property
    def xzzl(self) -> BVector4: ...
    @property
    def xzox(self) -> BVector4: ...
    @property
    def xzoy(self) -> BVector4: ...
    @property
    def xzoz(self) -> BVector4: ...
    @property
    def xzoo(self) -> BVector4: ...
    @property
    def xzol(self) -> BVector4: ...
    @property
    def xzlx(self) -> BVector4: ...
    @property
    def xzly(self) -> BVector4: ...
    @property
    def xzlz(self) -> BVector4: ...
    @property
    def xzlo(self) -> BVector4: ...
    @property
    def xzll(self) -> BVector4: ...
    @property
    def xoxx(self) -> BVector4: ...
    @property
    def xoxy(self) -> BVector4: ...
    @property
    def xoxz(self) -> BVector4: ...
    @property
    def xoxo(self) -> BVector4: ...
    @property
    def xoxl(self) -> BVector4: ...
    @property
    def xoyx(self) -> BVector4: ...
    @property
    def xoyy(self) -> BVector4: ...
    @property
    def xoyz(self) -> BVector4: ...
    @property
    def xoyo(self) -> BVector4: ...
    @property
    def xoyl(self) -> BVector4: ...
    @property
    def xozx(self) -> BVector4: ...
    @property
    def xozy(self) -> BVector4: ...
    @property
    def xozz(self) -> BVector4: ...
    @property
    def xozo(self) -> BVector4: ...
    @property
    def xozl(self) -> BVector4: ...
    @property
    def xoox(self) -> BVector4: ...
    @property
    def xooy(self) -> BVector4: ...
    @property
    def xooz(self) -> BVector4: ...
    @property
    def xooo(self) -> BVector4: ...
    @property
    def xool(self) -> BVector4: ...
    @property
    def xolx(self) -> BVector4: ...
    @property
    def xoly(self) -> BVector4: ...
    @property
    def xolz(self) -> BVector4: ...
    @property
    def xolo(self) -> BVector4: ...
    @property
    def xoll(self) -> BVector4: ...
    @property
    def xlxx(self) -> BVector4: ...
    @property
    def xlxy(self) -> BVector4: ...
    @property
    def xlxz(self) -> BVector4: ...
    @property
    def xlxo(self) -> BVector4: ...
    @property
    def xlxl(self) -> BVector4: ...
    @property
    def xlyx(self) -> BVector4: ...
    @property
    def xlyy(self) -> BVector4: ...
    @property
    def xlyz(self) -> BVector4: ...
    @property
    def xlyo(self) -> BVector4: ...
    @property
    def xlyl(self) -> BVector4: ...
    @property
    def xlzx(self) -> BVector4: ...
    @property
    def xlzy(self) -> BVector4: ...
    @property
    def xlzz(self) -> BVector4: ...
    @property
    def xlzo(self) -> BVector4: ...
    @property
    def xlzl(self) -> BVector4: ...
    @property
    def xlox(self) -> BVector4: ...
    @property
    def xloy(self) -> BVector4: ...
    @property
    def xloz(self) -> BVector4: ...
    @property
    def xloo(self) -> BVector4: ...
    @property
    def xlol(self) -> BVector4: ...
    @property
    def xllx(self) -> BVector4: ...
    @property
    def xlly(self) -> BVector4: ...
    @property
    def xllz(self) -> BVector4: ...
    @property
    def xllo(self) -> BVector4: ...
    @property
    def xlll(self) -> BVector4: ...
    @property
    def yxxx(self) -> BVector4: ...
    @property
    def yxxy(self) -> BVector4: ...
    @property
    def yxxz(self) -> BVector4: ...
    @property
    def yxxo(self) -> BVector4: ...
    @property
    def yxxl(self) -> BVector4: ...
    @property
    def yxyx(self) -> BVector4: ...
    @property
    def yxyy(self) -> BVector4: ...
    @property
    def yxyz(self) -> BVector4: ...
    @property
    def yxyo(self) -> BVector4: ...
    @property
    def yxyl(self) -> BVector4: ...
    @property
    def yxzx(self) -> BVector4: ...
    @property
    def yxzy(self) -> BVector4: ...
    @property
    def yxzz(self) -> BVector4: ...
    @property
    def yxzo(self) -> BVector4: ...
    @property
    def yxzl(self) -> BVector4: ...
    @property
    def yxox(self) -> BVector4: ...
    @property
    def yxoy(self) -> BVector4: ...
    @property
    def yxoz(self) -> BVector4: ...
    @property
    def yxoo(self) -> BVector4: ...
    @property
    def yxol(self) -> BVector4: ...
    @property
    def yxlx(self) -> BVector4: ...
    @property
    def yxly(self) -> BVector4: ...
    @property
    def yxlz(self) -> BVector4: ...
    @property
    def yxlo(self) -> BVector4: ...
    @property
    def yxll(self) -> BVector4: ...
    @property
    def yyxx(self) -> BVector4: ...
    @property
    def yyxy(self) -> BVector4: ...
    @property
    def yyxz(self) -> BVector4: ...
    @property
    def yyxo(self) -> BVector4: ...
    @property
    def yyxl(self) -> BVector4: ...
    @property
    def yyyx(self) -> BVector4: ...
    @property
    def yyyy(self) -> BVector4: ...
    @property
    def yyyz(self) -> BVector4: ...
    @property
    def yyyo(self) -> BVector4: ...
    @property
    def yyyl(self) -> BVector4: ...
    @property
    def yyzx(self) -> BVector4: ...
    @property
    def yyzy(self) -> BVector4: ...
    @property
    def yyzz(self) -> BVector4: ...
    @property
    def yyzo(self) -> BVector4: ...
    @property
    def yyzl(self) -> BVector4: ...
    @property
    def yyox(self) -> BVector4: ...
    @property
    def yyoy(self) -> BVector4: ...
    @property
    def yyoz(self) -> BVector4: ...
    @property
    def yyoo(self) -> BVector4: ...
    @property
    def yyol(self) -> BVector4: ...
    @property
    def yylx(self) -> BVector4: ...
    @property
    def yyly(self) -> BVector4: ...
    @property
    def yylz(self) -> BVector4: ...
    @property
    def yylo(self) -> BVector4: ...
    @property
    def yyll(self) -> BVector4: ...
    @property
    def yzxx(self) -> BVector4: ...
    @property
    def yzxy(self) -> BVector4: ...
    @property
    def yzxz(self) -> BVector4: ...
    @property
    def yzxo(self) -> BVector4: ...
    @property
    def yzxl(self) -> BVector4: ...
    @property
    def yzyx(self) -> BVector4: ...
    @property
    def yzyy(self) -> BVector4: ...
    @property
    def yzyz(self) -> BVector4: ...
    @property
    def yzyo(self) -> BVector4: ...
    @property
    def yzyl(self) -> BVector4: ...
    @property
    def yzzx(self) -> BVector4: ...
    @property
    def yzzy(self) -> BVector4: ...
    @property
    def yzzz(self) -> BVector4: ...
    @property
    def yzzo(self) -> BVector4: ...
    @property
    def yzzl(self) -> BVector4: ...
    @property
    def yzox(self) -> BVector4: ...
    @property
    def yzoy(self) -> BVector4: ...
    @property
    def yzoz(self) -> BVector4: ...
    @property
    def yzoo(self) -> BVector4: ...
    @property
    def yzol(self) -> BVector4: ...
    @property
    def yzlx(self) -> BVector4: ...
    @property
    def yzly(self) -> BVector4: ...
    @property
    def yzlz(self) -> BVector4: ...
    @property
    def yzlo(self) -> BVector4: ...
    @property
    def yzll(self) -> BVector4: ...
    @property
    def yoxx(self) -> BVector4: ...
    @property
    def yoxy(self) -> BVector4: ...
    @property
    def yoxz(self) -> BVector4: ...
    @property
    def yoxo(self) -> BVector4: ...
    @property
    def yoxl(self) -> BVector4: ...
    @property
    def yoyx(self) -> BVector4: ...
    @property
    def yoyy(self) -> BVector4: ...
    @property
    def yoyz(self) -> BVector4: ...
    @property
    def yoyo(self) -> BVector4: ...
    @property
    def yoyl(self) -> BVector4: ...
    @property
    def yozx(self) -> BVector4: ...
    @property
    def yozy(self) -> BVector4: ...
    @property
    def yozz(self) -> BVector4: ...
    @property
    def yozo(self) -> BVector4: ...
    @property
    def yozl(self) -> BVector4: ...
    @property
    def yoox(self) -> BVector4: ...
    @property
    def yooy(self) -> BVector4: ...
    @property
    def yooz(self) -> BVector4: ...
    @property
    def yooo(self) -> BVector4: ...
    @property
    def yool(self) -> BVector4: ...
    @property
    def yolx(self) -> BVector4: ...
    @property
    def yoly(self) -> BVector4: ...
    @property
    def yolz(self) -> BVector4: ...
    @property
    def yolo(self) -> BVector4: ...
    @property
    def yoll(self) -> BVector4: ...
    @property
    def ylxx(self) -> BVector4: ...
    @property
    def ylxy(self) -> BVector4: ...
    @property
    def ylxz(self) -> BVector4: ...
    @property
    def ylxo(self) -> BVector4: ...
    @property
    def ylxl(self) -> BVector4: ...
    @property
    def ylyx(self) -> BVector4: ...
    @property
    def ylyy(self) -> BVector4: ...
    @property
    def ylyz(self) -> BVector4: ...
    @property
    def ylyo(self) -> BVector4: ...
    @property
    def ylyl(self) -> BVector4: ...
    @property
    def ylzx(self) -> BVector4: ...
    @property
    def ylzy(self) -> BVector4: ...
    @property
    def ylzz(self) -> BVector4: ...
    @property
    def ylzo(self) -> BVector4: ...
    @property
    def ylzl(self) -> BVector4: ...
    @property
    def ylox(self) -> BVector4: ...
    @property
    def yloy(self) -> BVector4: ...
    @property
    def yloz(self) -> BVector4: ...
    @property
    def yloo(self) -> BVector4: ...
    @property
    def ylol(self) -> BVector4: ...
    @property
    def yllx(self) -> BVector4: ...
    @property
    def ylly(self) -> BVector4: ...
    @property
    def yllz(self) -> BVector4: ...
    @property
    def yllo(self) -> BVector4: ...
    @property
    def ylll(self) -> BVector4: ...
    @property
    def zxxx(self) -> BVector4: ...
    @property
    def zxxy(self) -> BVector4: ...
    @property
    def zxxz(self) -> BVector4: ...
    @property
    def zxxo(self) -> BVector4: ...
    @property
    def zxxl(self) -> BVector4: ...
    @property
    def zxyx(self) -> BVector4: ...
    @property
    def zxyy(self) -> BVector4: ...
    @property
    def zxyz(self) -> BVector4: ...
    @property
    def zxyo(self) -> BVector4: ...
    @property
    def zxyl(self) -> BVector4: ...
    @property
    def zxzx(self) -> BVector4: ...
    @property
    def zxzy(self) -> BVector4: ...
    @property
    def zxzz(self) -> BVector4: ...
    @property
    def zxzo(self) -> BVector4: ...
    @property
    def zxzl(self) -> BVector4: ...
    @property
    def zxox(self) -> BVector4: ...
    @property
    def zxoy(self) -> BVector4: ...
    @property
    def zxoz(self) -> BVector4: ...
    @property
    def zxoo(self) -> BVector4: ...
    @property
    def zxol(self) -> BVector4: ...
    @property
    def zxlx(self) -> BVector4: ...
    @property
    def zxly(self) -> BVector4: ...
    @property
    def zxlz(self) -> BVector4: ...
    @property
    def zxlo(self) -> BVector4: ...
    @property
    def zxll(self) -> BVector4: ...
    @property
    def zyxx(self) -> BVector4: ...
    @property
    def zyxy(self) -> BVector4: ...
    @property
    def zyxz(self) -> BVector4: ...
    @property
    def zyxo(self) -> BVector4: ...
    @property
    def zyxl(self) -> BVector4: ...
    @property
    def zyyx(self) -> BVector4: ...
    @property
    def zyyy(self) -> BVector4: ...
    @property
    def zyyz(self) -> BVector4: ...
    @property
    def zyyo(self) -> BVector4: ...
    @property
    def zyyl(self) -> BVector4: ...
    @property
    def zyzx(self) -> BVector4: ...
    @property
    def zyzy(self) -> BVector4: ...
    @property
    def zyzz(self) -> BVector4: ...
    @property
    def zyzo(self) -> BVector4: ...
    @property
    def zyzl(self) -> BVector4: ...
    @property
    def zyox(self) -> BVector4: ...
    @property
    def zyoy(self) -> BVector4: ...
    @property
    def zyoz(self) -> BVector4: ...
    @property
    def zyoo(self) -> BVector4: ...
    @property
    def zyol(self) -> BVector4: ...
    @property
    def zylx(self) -> BVector4: ...
    @property
    def zyly(self) -> BVector4: ...
    @property
    def zylz(self) -> BVector4: ...
    @property
    def zylo(self) -> BVector4: ...
    @property
    def zyll(self) -> BVector4: ...
    @property
    def zzxx(self) -> BVector4: ...
    @property
    def zzxy(self) -> BVector4: ...
    @property
    def zzxz(self) -> BVector4: ...
    @property
    def zzxo(self) -> BVector4: ...
    @property
    def zzxl(self) -> BVector4: ...
    @property
    def zzyx(self) -> BVector4: ...
    @property
    def zzyy(self) -> BVector4: ...
    @property
    def zzyz(self) -> BVector4: ...
    @property
    def zzyo(self) -> BVector4: ...
    @property
    def zzyl(self) -> BVector4: ...
    @property
    def zzzx(self) -> BVector4: ...
    @property
    def zzzy(self) -> BVector4: ...
    @property
    def zzzz(self) -> BVector4: ...
    @property
    def zzzo(self) -> BVector4: ...
    @property
    def zzzl(self) -> BVector4: ...
    @property
    def zzox(self) -> BVector4: ...
    @property
    def zzoy(self) -> BVector4: ...
    @property
    def zzoz(self) -> BVector4: ...
    @property
    def zzoo(self) -> BVector4: ...
    @property
    def zzol(self) -> BVector4: ...
    @property
    def zzlx(self) -> BVector4: ...
    @property
    def zzly(self) -> BVector4: ...
    @property
    def zzlz(self) -> BVector4: ...
    @property
    def zzlo(self) -> BVector4: ...
    @property
    def zzll(self) -> BVector4: ...
    @property
    def zoxx(self) -> BVector4: ...
    @property
    def zoxy(self) -> BVector4: ...
    @property
    def zoxz(self) -> BVector4: ...
    @property
    def zoxo(self) -> BVector4: ...
    @property
    def zoxl(self) -> BVector4: ...
    @property
    def zoyx(self) -> BVector4: ...
    @property
    def zoyy(self) -> BVector4: ...
    @property
    def zoyz(self) -> BVector4: ...
    @property
    def zoyo(self) -> BVector4: ...
    @property
    def zoyl(self) -> BVector4: ...
    @property
    def zozx(self) -> BVector4: ...
    @property
    def zozy(self) -> BVector4: ...
    @property
    def zozz(self) -> BVector4: ...
    @property
    def zozo(self) -> BVector4: ...
    @property
    def zozl(self) -> BVector4: ...
    @property
    def zoox(self) -> BVector4: ...
    @property
    def zooy(self) -> BVector4: ...
    @property
    def zooz(self) -> BVector4: ...
    @property
    def zooo(self) -> BVector4: ...
    @property
    def zool(self) -> BVector4: ...
    @property
    def zolx(self) -> BVector4: ...
    @property
    def zoly(self) -> BVector4: ...
    @property
    def zolz(self) -> BVector4: ...
    @property
    def zolo(self) -> BVector4: ...
    @property
    def zoll(self) -> BVector4: ...
    @property
    def zlxx(self) -> BVector4: ...
    @property
    def zlxy(self) -> BVector4: ...
    @property
    def zlxz(self) -> BVector4: ...
    @property
    def zlxo(self) -> BVector4: ...
    @property
    def zlxl(self) -> BVector4: ...
    @property
    def zlyx(self) -> BVector4: ...
    @property
    def zlyy(self) -> BVector4: ...
    @property
    def zlyz(self) -> BVector4: ...
    @property
    def zlyo(self) -> BVector4: ...
    @property
    def zlyl(self) -> BVector4: ...
    @property
    def zlzx(self) -> BVector4: ...
    @property
    def zlzy(self) -> BVector4: ...
    @property
    def zlzz(self) -> BVector4: ...
    @property
    def zlzo(self) -> BVector4: ...
    @property
    def zlzl(self) -> BVector4: ...
    @property
    def zlox(self) -> BVector4: ...
    @property
    def zloy(self) -> BVector4: ...
    @property
    def zloz(self) -> BVector4: ...
    @property
    def zloo(self) -> BVector4: ...
    @property
    def zlol(self) -> BVector4: ...
    @property
    def zllx(self) -> BVector4: ...
    @property
    def zlly(self) -> BVector4: ...
    @property
    def zllz(self) -> BVector4: ...
    @property
    def zllo(self) -> BVector4: ...
    @property
    def zlll(self) -> BVector4: ...
    @property
    def oxxx(self) -> BVector4: ...
    @property
    def oxxy(self) -> BVector4: ...
    @property
    def oxxz(self) -> BVector4: ...
    @property
    def oxxo(self) -> BVector4: ...
    @property
    def oxxl(self) -> BVector4: ...
    @property
    def oxyx(self) -> BVector4: ...
    @property
    def oxyy(self) -> BVector4: ...
    @property
    def oxyz(self) -> BVector4: ...
    @property
    def oxyo(self) -> BVector4: ...
    @property
    def oxyl(self) -> BVector4: ...
    @property
    def oxzx(self) -> BVector4: ...
    @property
    def oxzy(self) -> BVector4: ...
    @property
    def oxzz(self) -> BVector4: ...
    @property
    def oxzo(self) -> BVector4: ...
    @property
    def oxzl(self) -> BVector4: ...
    @property
    def oxox(self) -> BVector4: ...
    @property
    def oxoy(self) -> BVector4: ...
    @property
    def oxoz(self) -> BVector4: ...
    @property
    def oxoo(self) -> BVector4: ...
    @property
    def oxol(self) -> BVector4: ...
    @property
    def oxlx(self) -> BVector4: ...
    @property
    def oxly(self) -> BVector4: ...
    @property
    def oxlz(self) -> BVector4: ...
    @property
    def oxlo(self) -> BVector4: ...
    @property
    def oxll(self) -> BVector4: ...
    @property
    def oyxx(self) -> BVector4: ...
    @property
    def oyxy(self) -> BVector4: ...
    @property
    def oyxz(self) -> BVector4: ...
    @property
    def oyxo(self) -> BVector4: ...
    @property
    def oyxl(self) -> BVector4: ...
    @property
    def oyyx(self) -> BVector4: ...
    @property
    def oyyy(self) -> BVector4: ...
    @property
    def oyyz(self) -> BVector4: ...
    @property
    def oyyo(self) -> BVector4: ...
    @property
    def oyyl(self) -> BVector4: ...
    @property
    def oyzx(self) -> BVector4: ...
    @property
    def oyzy(self) -> BVector4: ...
    @property
    def oyzz(self) -> BVector4: ...
    @property
    def oyzo(self) -> BVector4: ...
    @property
    def oyzl(self) -> BVector4: ...
    @property
    def oyox(self) -> BVector4: ...
    @property
    def oyoy(self) -> BVector4: ...
    @property
    def oyoz(self) -> BVector4: ...
    @property
    def oyoo(self) -> BVector4: ...
    @property
    def oyol(self) -> BVector4: ...
    @property
    def oylx(self) -> BVector4: ...
    @property
    def oyly(self) -> BVector4: ...
    @property
    def oylz(self) -> BVector4: ...
    @property
    def oylo(self) -> BVector4: ...
    @property
    def oyll(self) -> BVector4: ...
    @property
    def ozxx(self) -> BVector4: ...
    @property
    def ozxy(self) -> BVector4: ...
    @property
    def ozxz(self) -> BVector4: ...
    @property
    def ozxo(self) -> BVector4: ...
    @property
    def ozxl(self) -> BVector4: ...
    @property
    def ozyx(self) -> BVector4: ...
    @property
    def ozyy(self) -> BVector4: ...
    @property
    def ozyz(self) -> BVector4: ...
    @property
    def ozyo(self) -> BVector4: ...
    @property
    def ozyl(self) -> BVector4: ...
    @property
    def ozzx(self) -> BVector4: ...
    @property
    def ozzy(self) -> BVector4: ...
    @property
    def ozzz(self) -> BVector4: ...
    @property
    def ozzo(self) -> BVector4: ...
    @property
    def ozzl(self) -> BVector4: ...
    @property
    def ozox(self) -> BVector4: ...
    @property
    def ozoy(self) -> BVector4: ...
    @property
    def ozoz(self) -> BVector4: ...
    @property
    def ozoo(self) -> BVector4: ...
    @property
    def ozol(self) -> BVector4: ...
    @property
    def ozlx(self) -> BVector4: ...
    @property
    def ozly(self) -> BVector4: ...
    @property
    def ozlz(self) -> BVector4: ...
    @property
    def ozlo(self) -> BVector4: ...
    @property
    def ozll(self) -> BVector4: ...
    @property
    def ooxx(self) -> BVector4: ...
    @property
    def ooxy(self) -> BVector4: ...
    @property
    def ooxz(self) -> BVector4: ...
    @property
    def ooxo(self) -> BVector4: ...
    @property
    def ooxl(self) -> BVector4: ...
    @property
    def ooyx(self) -> BVector4: ...
    @property
    def ooyy(self) -> BVector4: ...
    @property
    def ooyz(self) -> BVector4: ...
    @property
    def ooyo(self) -> BVector4: ...
    @property
    def ooyl(self) -> BVector4: ...
    @property
    def oozx(self) -> BVector4: ...
    @property
    def oozy(self) -> BVector4: ...
    @property
    def oozz(self) -> BVector4: ...
    @property
    def oozo(self) -> BVector4: ...
    @property
    def oozl(self) -> BVector4: ...
    @property
    def ooox(self) -> BVector4: ...
    @property
    def oooy(self) -> BVector4: ...
    @property
    def oooz(self) -> BVector4: ...
    @property
    def oolx(self) -> BVector4: ...
    @property
    def ooly(self) -> BVector4: ...
    @property
    def oolz(self) -> BVector4: ...
    @property
    def olxx(self) -> BVector4: ...
    @property
    def olxy(self) -> BVector4: ...
    @property
    def olxz(self) -> BVector4: ...
    @property
    def olxo(self) -> BVector4: ...
    @property
    def olxl(self) -> BVector4: ...
    @property
    def olyx(self) -> BVector4: ...
    @property
    def olyy(self) -> BVector4: ...
    @property
    def olyz(self) -> BVector4: ...
    @property
    def olyo(self) -> BVector4: ...
    @property
    def olyl(self) -> BVector4: ...
    @property
    def olzx(self) -> BVector4: ...
    @property
    def olzy(self) -> BVector4: ...
    @property
    def olzz(self) -> BVector4: ...
    @property
    def olzo(self) -> BVector4: ...
    @property
    def olzl(self) -> BVector4: ...
    @property
    def olox(self) -> BVector4: ...
    @property
    def oloy(self) -> BVector4: ...
    @property
    def oloz(self) -> BVector4: ...
    @property
    def ollx(self) -> BVector4: ...
    @property
    def olly(self) -> BVector4: ...
    @property
    def ollz(self) -> BVector4: ...
    @property
    def lxxx(self) -> BVector4: ...
    @property
    def lxxy(self) -> BVector4: ...
    @property
    def lxxz(self) -> BVector4: ...
    @property
    def lxxo(self) -> BVector4: ...
    @property
    def lxxl(self) -> BVector4: ...
    @property
    def lxyx(self) -> BVector4: ...
    @property
    def lxyy(self) -> BVector4: ...
    @property
    def lxyz(self) -> BVector4: ...
    @property
    def lxyo(self) -> BVector4: ...
    @property
    def lxyl(self) -> BVector4: ...
    @property
    def lxzx(self) -> BVector4: ...
    @property
    def lxzy(self) -> BVector4: ...
    @property
    def lxzz(self) -> BVector4: ...
    @property
    def lxzo(self) -> BVector4: ...
    @property
    def lxzl(self) -> BVector4: ...
    @property
    def lxox(self) -> BVector4: ...
    @property
    def lxoy(self) -> BVector4: ...
    @property
    def lxoz(self) -> BVector4: ...
    @property
    def lxoo(self) -> BVector4: ...
    @property
    def lxol(self) -> BVector4: ...
    @property
    def lxlx(self) -> BVector4: ...
    @property
    def lxly(self) -> BVector4: ...
    @property
    def lxlz(self) -> BVector4: ...
    @property
    def lxlo(self) -> BVector4: ...
    @property
    def lxll(self) -> BVector4: ...
    @property
    def lyxx(self) -> BVector4: ...
    @property
    def lyxy(self) -> BVector4: ...
    @property
    def lyxz(self) -> BVector4: ...
    @property
    def lyxo(self) -> BVector4: ...
    @property
    def lyxl(self) -> BVector4: ...
    @property
    def lyyx(self) -> BVector4: ...
    @property
    def lyyy(self) -> BVector4: ...
    @property
    def lyyz(self) -> BVector4: ...
    @property
    def lyyo(self) -> BVector4: ...
    @property
    def lyyl(self) -> BVector4: ...
    @property
    def lyzx(self) -> BVector4: ...
    @property
    def lyzy(self) -> BVector4: ...
    @property
    def lyzz(self) -> BVector4: ...
    @property
    def lyzo(self) -> BVector4: ...
    @property
    def lyzl(self) -> BVector4: ...
    @property
    def lyox(self) -> BVector4: ...
    @property
    def lyoy(self) -> BVector4: ...
    @property
    def lyoz(self) -> BVector4: ...
    @property
    def lyoo(self) -> BVector4: ...
    @property
    def lyol(self) -> BVector4: ...
    @property
    def lylx(self) -> BVector4: ...
    @property
    def lyly(self) -> BVector4: ...
    @property
    def lylz(self) -> BVector4: ...
    @property
    def lylo(self) -> BVector4: ...
    @property
    def lyll(self) -> BVector4: ...
    @property
    def lzxx(self) -> BVector4: ...
    @property
    def lzxy(self) -> BVector4: ...
    @property
    def lzxz(self) -> BVector4: ...
    @property
    def lzxo(self) -> BVector4: ...
    @property
    def lzxl(self) -> BVector4: ...
    @property
    def lzyx(self) -> BVector4: ...
    @property
    def lzyy(self) -> BVector4: ...
    @property
    def lzyz(self) -> BVector4: ...
    @property
    def lzyo(self) -> BVector4: ...
    @property
    def lzyl(self) -> BVector4: ...
    @property
    def lzzx(self) -> BVector4: ...
    @property
    def lzzy(self) -> BVector4: ...
    @property
    def lzzz(self) -> BVector4: ...
    @property
    def lzzo(self) -> BVector4: ...
    @property
    def lzzl(self) -> BVector4: ...
    @property
    def lzox(self) -> BVector4: ...
    @property
    def lzoy(self) -> BVector4: ...
    @property
    def lzoz(self) -> BVector4: ...
    @property
    def lzoo(self) -> BVector4: ...
    @property
    def lzol(self) -> BVector4: ...
    @property
    def lzlx(self) -> BVector4: ...
    @property
    def lzly(self) -> BVector4: ...
    @property
    def lzlz(self) -> BVector4: ...
    @property
    def lzlo(self) -> BVector4: ...
    @property
    def lzll(self) -> BVector4: ...
    @property
    def loxx(self) -> BVector4: ...
    @property
    def loxy(self) -> BVector4: ...
    @property
    def loxz(self) -> BVector4: ...
    @property
    def loxo(self) -> BVector4: ...
    @property
    def loxl(self) -> BVector4: ...
    @property
    def loyx(self) -> BVector4: ...
    @property
    def loyy(self) -> BVector4: ...
    @property
    def loyz(self) -> BVector4: ...
    @property
    def loyo(self) -> BVector4: ...
    @property
    def loyl(self) -> BVector4: ...
    @property
    def lozx(self) -> BVector4: ...
    @property
    def lozy(self) -> BVector4: ...
    @property
    def lozz(self) -> BVector4: ...
    @property
    def lozo(self) -> BVector4: ...
    @property
    def lozl(self) -> BVector4: ...
    @property
    def loox(self) -> BVector4: ...
    @property
    def looy(self) -> BVector4: ...
    @property
    def looz(self) -> BVector4: ...
    @property
    def lolx(self) -> BVector4: ...
    @property
    def loly(self) -> BVector4: ...
    @property
    def lolz(self) -> BVector4: ...
    @property
    def llxx(self) -> BVector4: ...
    @property
    def llxy(self) -> BVector4: ...
    @property
    def llxz(self) -> BVector4: ...
    @property
    def llxo(self) -> BVector4: ...
    @property
    def llxl(self) -> BVector4: ...
    @property
    def llyx(self) -> BVector4: ...
    @property
    def llyy(self) -> BVector4: ...
    @property
    def llyz(self) -> BVector4: ...
    @property
    def llyo(self) -> BVector4: ...
    @property
    def llyl(self) -> BVector4: ...
    @property
    def llzx(self) -> BVector4: ...
    @property
    def llzy(self) -> BVector4: ...
    @property
    def llzz(self) -> BVector4: ...
    @property
    def llzo(self) -> BVector4: ...
    @property
    def llzl(self) -> BVector4: ...
    @property
    def llox(self) -> BVector4: ...
    @property
    def lloy(self) -> BVector4: ...
    @property
    def lloz(self) -> BVector4: ...
    @property
    def lllx(self) -> BVector4: ...
    @property
    def llly(self) -> BVector4: ...
    @property
    def lllz(self) -> BVector4: ...
    @property
    def r(self) -> _bool: ...
    @property
    def g(self) -> _bool: ...
    @property
    def b(self) -> _bool: ...
    @property
    def rr(self) -> BVector2: ...
    @property
    def rg(self) -> BVector2: ...
    @property
    def rb(self) -> BVector2: ...
    @property
    def ro(self) -> BVector2: ...
    @property
    def rl(self) -> BVector2: ...
    @property
    def gr(self) -> BVector2: ...
    @property
    def gg(self) -> BVector2: ...
    @property
    def gb(self) -> BVector2: ...
    @property
    def go(self) -> BVector2: ...
    @property
    def gl(self) -> BVector2: ...
    @property
    def br(self) -> BVector2: ...
    @property
    def bg(self) -> BVector2: ...
    @property
    def bb(self) -> BVector2: ...
    @property
    def bo(self) -> BVector2: ...
    @property
    def bl(self) -> BVector2: ...
    @property
    def og(self) -> BVector2: ...
    @property
    def ob(self) -> BVector2: ...
    @property
    def lr(self) -> BVector2: ...
    @property
    def lg(self) -> BVector2: ...
    @property
    def lb(self) -> BVector2: ...
    @property
    def rrr(self) -> BVector3: ...
    @property
    def rrg(self) -> BVector3: ...
    @property
    def rrb(self) -> BVector3: ...
    @property
    def rro(self) -> BVector3: ...
    @property
    def rrl(self) -> BVector3: ...
    @property
    def rgr(self) -> BVector3: ...
    @property
    def rgg(self) -> BVector3: ...
    @property
    def rgb(self) -> BVector3: ...
    @property
    def rgo(self) -> BVector3: ...
    @property
    def rgl(self) -> BVector3: ...
    @property
    def rbr(self) -> BVector3: ...
    @property
    def rbg(self) -> BVector3: ...
    @property
    def rbb(self) -> BVector3: ...
    @property
    def rbo(self) -> BVector3: ...
    @property
    def rbl(self) -> BVector3: ...
    @property
    def ror(self) -> BVector3: ...
    @property
    def rog(self) -> BVector3: ...
    @property
    def rob(self) -> BVector3: ...
    @property
    def roo(self) -> BVector3: ...
    @property
    def rol(self) -> BVector3: ...
    @property
    def rlr(self) -> BVector3: ...
    @property
    def rlg(self) -> BVector3: ...
    @property
    def rlb(self) -> BVector3: ...
    @property
    def rlo(self) -> BVector3: ...
    @property
    def rll(self) -> BVector3: ...
    @property
    def grr(self) -> BVector3: ...
    @property
    def grg(self) -> BVector3: ...
    @property
    def grb(self) -> BVector3: ...
    @property
    def gro(self) -> BVector3: ...
    @property
    def grl(self) -> BVector3: ...
    @property
    def ggr(self) -> BVector3: ...
    @property
    def ggg(self) -> BVector3: ...
    @property
    def ggb(self) -> BVector3: ...
    @property
    def ggo(self) -> BVector3: ...
    @property
    def ggl(self) -> BVector3: ...
    @property
    def gbr(self) -> BVector3: ...
    @property
    def gbg(self) -> BVector3: ...
    @property
    def gbb(self) -> BVector3: ...
    @property
    def gbo(self) -> BVector3: ...
    @property
    def gbl(self) -> BVector3: ...
    @property
    def gor(self) -> BVector3: ...
    @property
    def gog(self) -> BVector3: ...
    @property
    def gob(self) -> BVector3: ...
    @property
    def goo(self) -> BVector3: ...
    @property
    def gol(self) -> BVector3: ...
    @property
    def glr(self) -> BVector3: ...
    @property
    def glg(self) -> BVector3: ...
    @property
    def glb(self) -> BVector3: ...
    @property
    def glo(self) -> BVector3: ...
    @property
    def gll(self) -> BVector3: ...
    @property
    def brr(self) -> BVector3: ...
    @property
    def brg(self) -> BVector3: ...
    @property
    def brb(self) -> BVector3: ...
    @property
    def bro(self) -> BVector3: ...
    @property
    def brl(self) -> BVector3: ...
    @property
    def bgr(self) -> BVector3: ...
    @property
    def bgg(self) -> BVector3: ...
    @property
    def bgb(self) -> BVector3: ...
    @property
    def bgo(self) -> BVector3: ...
    @property
    def bgl(self) -> BVector3: ...
    @property
    def bbr(self) -> BVector3: ...
    @property
    def bbg(self) -> BVector3: ...
    @property
    def bbb(self) -> BVector3: ...
    @property
    def bbo(self) -> BVector3: ...
    @property
    def bbl(self) -> BVector3: ...
    @property
    def bor(self) -> BVector3: ...
    @property
    def bog(self) -> BVector3: ...
    @property
    def bob(self) -> BVector3: ...
    @property
    def boo(self) -> BVector3: ...
    @property
    def bol(self) -> BVector3: ...
    @property
    def blr(self) -> BVector3: ...
    @property
    def blg(self) -> BVector3: ...
    @property
    def blb(self) -> BVector3: ...
    @property
    def blo(self) -> BVector3: ...
    @property
    def bll(self) -> BVector3: ...
    @property
    def orr(self) -> BVector3: ...
    @property
    def org(self) -> BVector3: ...
    @property
    def orb(self) -> BVector3: ...
    @property
    def oro(self) -> BVector3: ...
    @property
    def orl(self) -> BVector3: ...
    @property
    def ogr(self) -> BVector3: ...
    @property
    def ogg(self) -> BVector3: ...
    @property
    def ogb(self) -> BVector3: ...
    @property
    def ogo(self) -> BVector3: ...
    @property
    def ogl(self) -> BVector3: ...
    @property
    def obr(self) -> BVector3: ...
    @property
    def obg(self) -> BVector3: ...
    @property
    def obb(self) -> BVector3: ...
    @property
    def obo(self) -> BVector3: ...
    @property
    def obl(self) -> BVector3: ...
    @property
    def oor(self) -> BVector3: ...
    @property
    def oog(self) -> BVector3: ...
    @property
    def oob(self) -> BVector3: ...
    @property
    def olr(self) -> BVector3: ...
    @property
    def olg(self) -> BVector3: ...
    @property
    def olb(self) -> BVector3: ...
    @property
    def lrr(self) -> BVector3: ...
    @property
    def lrg(self) -> BVector3: ...
    @property
    def lrb(self) -> BVector3: ...
    @property
    def lro(self) -> BVector3: ...
    @property
    def lrl(self) -> BVector3: ...
    @property
    def lgr(self) -> BVector3: ...
    @property
    def lgg(self) -> BVector3: ...
    @property
    def lgb(self) -> BVector3: ...
    @property
    def lgo(self) -> BVector3: ...
    @property
    def lgl(self) -> BVector3: ...
    @property
    def lbr(self) -> BVector3: ...
    @property
    def lbg(self) -> BVector3: ...
    @property
    def lbb(self) -> BVector3: ...
    @property
    def lbo(self) -> BVector3: ...
    @property
    def lbl(self) -> BVector3: ...
    @property
    def lor(self) -> BVector3: ...
    @property
    def log(self) -> BVector3: ...
    @property
    def lob(self) -> BVector3: ...
    @property
    def llr(self) -> BVector3: ...
    @property
    def llg(self) -> BVector3: ...
    @property
    def llb(self) -> BVector3: ...
    @property
    def rrrr(self) -> BVector4: ...
    @property
    def rrrg(self) -> BVector4: ...
    @property
    def rrrb(self) -> BVector4: ...
    @property
    def rrro(self) -> BVector4: ...
    @property
    def rrrl(self) -> BVector4: ...
    @property
    def rrgr(self) -> BVector4: ...
    @property
    def rrgg(self) -> BVector4: ...
    @property
    def rrgb(self) -> BVector4: ...
    @property
    def rrgo(self) -> BVector4: ...
    @property
    def rrgl(self) -> BVector4: ...
    @property
    def rrbr(self) -> BVector4: ...
    @property
    def rrbg(self) -> BVector4: ...
    @property
    def rrbb(self) -> BVector4: ...
    @property
    def rrbo(self) -> BVector4: ...
    @property
    def rrbl(self) -> BVector4: ...
    @property
    def rror(self) -> BVector4: ...
    @property
    def rrog(self) -> BVector4: ...
    @property
    def rrob(self) -> BVector4: ...
    @property
    def rroo(self) -> BVector4: ...
    @property
    def rrol(self) -> BVector4: ...
    @property
    def rrlr(self) -> BVector4: ...
    @property
    def rrlg(self) -> BVector4: ...
    @property
    def rrlb(self) -> BVector4: ...
    @property
    def rrlo(self) -> BVector4: ...
    @property
    def rrll(self) -> BVector4: ...
    @property
    def rgrr(self) -> BVector4: ...
    @property
    def rgrg(self) -> BVector4: ...
    @property
    def rgrb(self) -> BVector4: ...
    @property
    def rgro(self) -> BVector4: ...
    @property
    def rgrl(self) -> BVector4: ...
    @property
    def rggr(self) -> BVector4: ...
    @property
    def rggg(self) -> BVector4: ...
    @property
    def rggb(self) -> BVector4: ...
    @property
    def rggo(self) -> BVector4: ...
    @property
    def rggl(self) -> BVector4: ...
    @property
    def rgbr(self) -> BVector4: ...
    @property
    def rgbg(self) -> BVector4: ...
    @property
    def rgbb(self) -> BVector4: ...
    @property
    def rgbo(self) -> BVector4: ...
    @property
    def rgbl(self) -> BVector4: ...
    @property
    def rgor(self) -> BVector4: ...
    @property
    def rgog(self) -> BVector4: ...
    @property
    def rgob(self) -> BVector4: ...
    @property
    def rgoo(self) -> BVector4: ...
    @property
    def rgol(self) -> BVector4: ...
    @property
    def rglr(self) -> BVector4: ...
    @property
    def rglg(self) -> BVector4: ...
    @property
    def rglb(self) -> BVector4: ...
    @property
    def rglo(self) -> BVector4: ...
    @property
    def rgll(self) -> BVector4: ...
    @property
    def rbrr(self) -> BVector4: ...
    @property
    def rbrg(self) -> BVector4: ...
    @property
    def rbrb(self) -> BVector4: ...
    @property
    def rbro(self) -> BVector4: ...
    @property
    def rbrl(self) -> BVector4: ...
    @property
    def rbgr(self) -> BVector4: ...
    @property
    def rbgg(self) -> BVector4: ...
    @property
    def rbgb(self) -> BVector4: ...
    @property
    def rbgo(self) -> BVector4: ...
    @property
    def rbgl(self) -> BVector4: ...
    @property
    def rbbr(self) -> BVector4: ...
    @property
    def rbbg(self) -> BVector4: ...
    @property
    def rbbb(self) -> BVector4: ...
    @property
    def rbbo(self) -> BVector4: ...
    @property
    def rbbl(self) -> BVector4: ...
    @property
    def rbor(self) -> BVector4: ...
    @property
    def rbog(self) -> BVector4: ...
    @property
    def rbob(self) -> BVector4: ...
    @property
    def rboo(self) -> BVector4: ...
    @property
    def rbol(self) -> BVector4: ...
    @property
    def rblr(self) -> BVector4: ...
    @property
    def rblg(self) -> BVector4: ...
    @property
    def rblb(self) -> BVector4: ...
    @property
    def rblo(self) -> BVector4: ...
    @property
    def rbll(self) -> BVector4: ...
    @property
    def rorr(self) -> BVector4: ...
    @property
    def rorg(self) -> BVector4: ...
    @property
    def rorb(self) -> BVector4: ...
    @property
    def roro(self) -> BVector4: ...
    @property
    def rorl(self) -> BVector4: ...
    @property
    def rogr(self) -> BVector4: ...
    @property
    def rogg(self) -> BVector4: ...
    @property
    def rogb(self) -> BVector4: ...
    @property
    def rogo(self) -> BVector4: ...
    @property
    def rogl(self) -> BVector4: ...
    @property
    def robr(self) -> BVector4: ...
    @property
    def robg(self) -> BVector4: ...
    @property
    def robb(self) -> BVector4: ...
    @property
    def robo(self) -> BVector4: ...
    @property
    def robl(self) -> BVector4: ...
    @property
    def roor(self) -> BVector4: ...
    @property
    def roog(self) -> BVector4: ...
    @property
    def roob(self) -> BVector4: ...
    @property
    def rooo(self) -> BVector4: ...
    @property
    def rool(self) -> BVector4: ...
    @property
    def rolr(self) -> BVector4: ...
    @property
    def rolg(self) -> BVector4: ...
    @property
    def rolb(self) -> BVector4: ...
    @property
    def rolo(self) -> BVector4: ...
    @property
    def roll(self) -> BVector4: ...
    @property
    def rlrr(self) -> BVector4: ...
    @property
    def rlrg(self) -> BVector4: ...
    @property
    def rlrb(self) -> BVector4: ...
    @property
    def rlro(self) -> BVector4: ...
    @property
    def rlrl(self) -> BVector4: ...
    @property
    def rlgr(self) -> BVector4: ...
    @property
    def rlgg(self) -> BVector4: ...
    @property
    def rlgb(self) -> BVector4: ...
    @property
    def rlgo(self) -> BVector4: ...
    @property
    def rlgl(self) -> BVector4: ...
    @property
    def rlbr(self) -> BVector4: ...
    @property
    def rlbg(self) -> BVector4: ...
    @property
    def rlbb(self) -> BVector4: ...
    @property
    def rlbo(self) -> BVector4: ...
    @property
    def rlbl(self) -> BVector4: ...
    @property
    def rlor(self) -> BVector4: ...
    @property
    def rlog(self) -> BVector4: ...
    @property
    def rlob(self) -> BVector4: ...
    @property
    def rloo(self) -> BVector4: ...
    @property
    def rlol(self) -> BVector4: ...
    @property
    def rllr(self) -> BVector4: ...
    @property
    def rllg(self) -> BVector4: ...
    @property
    def rllb(self) -> BVector4: ...
    @property
    def rllo(self) -> BVector4: ...
    @property
    def rlll(self) -> BVector4: ...
    @property
    def grrr(self) -> BVector4: ...
    @property
    def grrg(self) -> BVector4: ...
    @property
    def grrb(self) -> BVector4: ...
    @property
    def grro(self) -> BVector4: ...
    @property
    def grrl(self) -> BVector4: ...
    @property
    def grgr(self) -> BVector4: ...
    @property
    def grgg(self) -> BVector4: ...
    @property
    def grgb(self) -> BVector4: ...
    @property
    def grgo(self) -> BVector4: ...
    @property
    def grgl(self) -> BVector4: ...
    @property
    def grbr(self) -> BVector4: ...
    @property
    def grbg(self) -> BVector4: ...
    @property
    def grbb(self) -> BVector4: ...
    @property
    def grbo(self) -> BVector4: ...
    @property
    def grbl(self) -> BVector4: ...
    @property
    def gror(self) -> BVector4: ...
    @property
    def grog(self) -> BVector4: ...
    @property
    def grob(self) -> BVector4: ...
    @property
    def groo(self) -> BVector4: ...
    @property
    def grol(self) -> BVector4: ...
    @property
    def grlr(self) -> BVector4: ...
    @property
    def grlg(self) -> BVector4: ...
    @property
    def grlb(self) -> BVector4: ...
    @property
    def grlo(self) -> BVector4: ...
    @property
    def grll(self) -> BVector4: ...
    @property
    def ggrr(self) -> BVector4: ...
    @property
    def ggrg(self) -> BVector4: ...
    @property
    def ggrb(self) -> BVector4: ...
    @property
    def ggro(self) -> BVector4: ...
    @property
    def ggrl(self) -> BVector4: ...
    @property
    def gggr(self) -> BVector4: ...
    @property
    def gggg(self) -> BVector4: ...
    @property
    def gggb(self) -> BVector4: ...
    @property
    def gggo(self) -> BVector4: ...
    @property
    def gggl(self) -> BVector4: ...
    @property
    def ggbr(self) -> BVector4: ...
    @property
    def ggbg(self) -> BVector4: ...
    @property
    def ggbb(self) -> BVector4: ...
    @property
    def ggbo(self) -> BVector4: ...
    @property
    def ggbl(self) -> BVector4: ...
    @property
    def ggor(self) -> BVector4: ...
    @property
    def ggog(self) -> BVector4: ...
    @property
    def ggob(self) -> BVector4: ...
    @property
    def ggoo(self) -> BVector4: ...
    @property
    def ggol(self) -> BVector4: ...
    @property
    def gglr(self) -> BVector4: ...
    @property
    def gglg(self) -> BVector4: ...
    @property
    def gglb(self) -> BVector4: ...
    @property
    def gglo(self) -> BVector4: ...
    @property
    def ggll(self) -> BVector4: ...
    @property
    def gbrr(self) -> BVector4: ...
    @property
    def gbrg(self) -> BVector4: ...
    @property
    def gbrb(self) -> BVector4: ...
    @property
    def gbro(self) -> BVector4: ...
    @property
    def gbrl(self) -> BVector4: ...
    @property
    def gbgr(self) -> BVector4: ...
    @property
    def gbgg(self) -> BVector4: ...
    @property
    def gbgb(self) -> BVector4: ...
    @property
    def gbgo(self) -> BVector4: ...
    @property
    def gbgl(self) -> BVector4: ...
    @property
    def gbbr(self) -> BVector4: ...
    @property
    def gbbg(self) -> BVector4: ...
    @property
    def gbbb(self) -> BVector4: ...
    @property
    def gbbo(self) -> BVector4: ...
    @property
    def gbbl(self) -> BVector4: ...
    @property
    def gbor(self) -> BVector4: ...
    @property
    def gbog(self) -> BVector4: ...
    @property
    def gbob(self) -> BVector4: ...
    @property
    def gboo(self) -> BVector4: ...
    @property
    def gbol(self) -> BVector4: ...
    @property
    def gblr(self) -> BVector4: ...
    @property
    def gblg(self) -> BVector4: ...
    @property
    def gblb(self) -> BVector4: ...
    @property
    def gblo(self) -> BVector4: ...
    @property
    def gbll(self) -> BVector4: ...
    @property
    def gorr(self) -> BVector4: ...
    @property
    def gorg(self) -> BVector4: ...
    @property
    def gorb(self) -> BVector4: ...
    @property
    def goro(self) -> BVector4: ...
    @property
    def gorl(self) -> BVector4: ...
    @property
    def gogr(self) -> BVector4: ...
    @property
    def gogg(self) -> BVector4: ...
    @property
    def gogb(self) -> BVector4: ...
    @property
    def gogo(self) -> BVector4: ...
    @property
    def gogl(self) -> BVector4: ...
    @property
    def gobr(self) -> BVector4: ...
    @property
    def gobg(self) -> BVector4: ...
    @property
    def gobb(self) -> BVector4: ...
    @property
    def gobo(self) -> BVector4: ...
    @property
    def gobl(self) -> BVector4: ...
    @property
    def goor(self) -> BVector4: ...
    @property
    def goog(self) -> BVector4: ...
    @property
    def goob(self) -> BVector4: ...
    @property
    def gooo(self) -> BVector4: ...
    @property
    def gool(self) -> BVector4: ...
    @property
    def golr(self) -> BVector4: ...
    @property
    def golg(self) -> BVector4: ...
    @property
    def golb(self) -> BVector4: ...
    @property
    def golo(self) -> BVector4: ...
    @property
    def goll(self) -> BVector4: ...
    @property
    def glrr(self) -> BVector4: ...
    @property
    def glrg(self) -> BVector4: ...
    @property
    def glrb(self) -> BVector4: ...
    @property
    def glro(self) -> BVector4: ...
    @property
    def glrl(self) -> BVector4: ...
    @property
    def glgr(self) -> BVector4: ...
    @property
    def glgg(self) -> BVector4: ...
    @property
    def glgb(self) -> BVector4: ...
    @property
    def glgo(self) -> BVector4: ...
    @property
    def glgl(self) -> BVector4: ...
    @property
    def glbr(self) -> BVector4: ...
    @property
    def glbg(self) -> BVector4: ...
    @property
    def glbb(self) -> BVector4: ...
    @property
    def glbo(self) -> BVector4: ...
    @property
    def glbl(self) -> BVector4: ...
    @property
    def glor(self) -> BVector4: ...
    @property
    def glog(self) -> BVector4: ...
    @property
    def glob(self) -> BVector4: ...
    @property
    def gloo(self) -> BVector4: ...
    @property
    def glol(self) -> BVector4: ...
    @property
    def gllr(self) -> BVector4: ...
    @property
    def gllg(self) -> BVector4: ...
    @property
    def gllb(self) -> BVector4: ...
    @property
    def gllo(self) -> BVector4: ...
    @property
    def glll(self) -> BVector4: ...
    @property
    def brrr(self) -> BVector4: ...
    @property
    def brrg(self) -> BVector4: ...
    @property
    def brrb(self) -> BVector4: ...
    @property
    def brro(self) -> BVector4: ...
    @property
    def brrl(self) -> BVector4: ...
    @property
    def brgr(self) -> BVector4: ...
    @property
    def brgg(self) -> BVector4: ...
    @property
    def brgb(self) -> BVector4: ...
    @property
    def brgo(self) -> BVector4: ...
    @property
    def brgl(self) -> BVector4: ...
    @property
    def brbr(self) -> BVector4: ...
    @property
    def brbg(self) -> BVector4: ...
    @property
    def brbb(self) -> BVector4: ...
    @property
    def brbo(self) -> BVector4: ...
    @property
    def brbl(self) -> BVector4: ...
    @property
    def bror(self) -> BVector4: ...
    @property
    def brog(self) -> BVector4: ...
    @property
    def brob(self) -> BVector4: ...
    @property
    def broo(self) -> BVector4: ...
    @property
    def brol(self) -> BVector4: ...
    @property
    def brlr(self) -> BVector4: ...
    @property
    def brlg(self) -> BVector4: ...
    @property
    def brlb(self) -> BVector4: ...
    @property
    def brlo(self) -> BVector4: ...
    @property
    def brll(self) -> BVector4: ...
    @property
    def bgrr(self) -> BVector4: ...
    @property
    def bgrg(self) -> BVector4: ...
    @property
    def bgrb(self) -> BVector4: ...
    @property
    def bgro(self) -> BVector4: ...
    @property
    def bgrl(self) -> BVector4: ...
    @property
    def bggr(self) -> BVector4: ...
    @property
    def bggg(self) -> BVector4: ...
    @property
    def bggb(self) -> BVector4: ...
    @property
    def bggo(self) -> BVector4: ...
    @property
    def bggl(self) -> BVector4: ...
    @property
    def bgbr(self) -> BVector4: ...
    @property
    def bgbg(self) -> BVector4: ...
    @property
    def bgbb(self) -> BVector4: ...
    @property
    def bgbo(self) -> BVector4: ...
    @property
    def bgbl(self) -> BVector4: ...
    @property
    def bgor(self) -> BVector4: ...
    @property
    def bgog(self) -> BVector4: ...
    @property
    def bgob(self) -> BVector4: ...
    @property
    def bgoo(self) -> BVector4: ...
    @property
    def bgol(self) -> BVector4: ...
    @property
    def bglr(self) -> BVector4: ...
    @property
    def bglg(self) -> BVector4: ...
    @property
    def bglb(self) -> BVector4: ...
    @property
    def bglo(self) -> BVector4: ...
    @property
    def bgll(self) -> BVector4: ...
    @property
    def bbrr(self) -> BVector4: ...
    @property
    def bbrg(self) -> BVector4: ...
    @property
    def bbrb(self) -> BVector4: ...
    @property
    def bbro(self) -> BVector4: ...
    @property
    def bbrl(self) -> BVector4: ...
    @property
    def bbgr(self) -> BVector4: ...
    @property
    def bbgg(self) -> BVector4: ...
    @property
    def bbgb(self) -> BVector4: ...
    @property
    def bbgo(self) -> BVector4: ...
    @property
    def bbgl(self) -> BVector4: ...
    @property
    def bbbr(self) -> BVector4: ...
    @property
    def bbbg(self) -> BVector4: ...
    @property
    def bbbb(self) -> BVector4: ...
    @property
    def bbbo(self) -> BVector4: ...
    @property
    def bbbl(self) -> BVector4: ...
    @property
    def bbor(self) -> BVector4: ...
    @property
    def bbog(self) -> BVector4: ...
    @property
    def bbob(self) -> BVector4: ...
    @property
    def bboo(self) -> BVector4: ...
    @property
    def bbol(self) -> BVector4: ...
    @property
    def bblr(self) -> BVector4: ...
    @property
    def bblg(self) -> BVector4: ...
    @property
    def bblb(self) -> BVector4: ...
    @property
    def bblo(self) -> BVector4: ...
    @property
    def bbll(self) -> BVector4: ...
    @property
    def borr(self) -> BVector4: ...
    @property
    def borg(self) -> BVector4: ...
    @property
    def borb(self) -> BVector4: ...
    @property
    def boro(self) -> BVector4: ...
    @property
    def borl(self) -> BVector4: ...
    @property
    def bogr(self) -> BVector4: ...
    @property
    def bogg(self) -> BVector4: ...
    @property
    def bogb(self) -> BVector4: ...
    @property
    def bogo(self) -> BVector4: ...
    @property
    def bogl(self) -> BVector4: ...
    @property
    def bobr(self) -> BVector4: ...
    @property
    def bobg(self) -> BVector4: ...
    @property
    def bobb(self) -> BVector4: ...
    @property
    def bobo(self) -> BVector4: ...
    @property
    def bobl(self) -> BVector4: ...
    @property
    def boor(self) -> BVector4: ...
    @property
    def boog(self) -> BVector4: ...
    @property
    def boob(self) -> BVector4: ...
    @property
    def booo(self) -> BVector4: ...
    @property
    def bool(self) -> BVector4: ...
    @property
    def bolr(self) -> BVector4: ...
    @property
    def bolg(self) -> BVector4: ...
    @property
    def bolb(self) -> BVector4: ...
    @property
    def bolo(self) -> BVector4: ...
    @property
    def boll(self) -> BVector4: ...
    @property
    def blrr(self) -> BVector4: ...
    @property
    def blrg(self) -> BVector4: ...
    @property
    def blrb(self) -> BVector4: ...
    @property
    def blro(self) -> BVector4: ...
    @property
    def blrl(self) -> BVector4: ...
    @property
    def blgr(self) -> BVector4: ...
    @property
    def blgg(self) -> BVector4: ...
    @property
    def blgb(self) -> BVector4: ...
    @property
    def blgo(self) -> BVector4: ...
    @property
    def blgl(self) -> BVector4: ...
    @property
    def blbr(self) -> BVector4: ...
    @property
    def blbg(self) -> BVector4: ...
    @property
    def blbb(self) -> BVector4: ...
    @property
    def blbo(self) -> BVector4: ...
    @property
    def blbl(self) -> BVector4: ...
    @property
    def blor(self) -> BVector4: ...
    @property
    def blog(self) -> BVector4: ...
    @property
    def blob(self) -> BVector4: ...
    @property
    def bloo(self) -> BVector4: ...
    @property
    def blol(self) -> BVector4: ...
    @property
    def bllr(self) -> BVector4: ...
    @property
    def bllg(self) -> BVector4: ...
    @property
    def bllb(self) -> BVector4: ...
    @property
    def bllo(self) -> BVector4: ...
    @property
    def blll(self) -> BVector4: ...
    @property
    def orrr(self) -> BVector4: ...
    @property
    def orrg(self) -> BVector4: ...
    @property
    def orrb(self) -> BVector4: ...
    @property
    def orro(self) -> BVector4: ...
    @property
    def orrl(self) -> BVector4: ...
    @property
    def orgr(self) -> BVector4: ...
    @property
    def orgg(self) -> BVector4: ...
    @property
    def orgb(self) -> BVector4: ...
    @property
    def orgo(self) -> BVector4: ...
    @property
    def orgl(self) -> BVector4: ...
    @property
    def orbr(self) -> BVector4: ...
    @property
    def orbg(self) -> BVector4: ...
    @property
    def orbb(self) -> BVector4: ...
    @property
    def orbo(self) -> BVector4: ...
    @property
    def orbl(self) -> BVector4: ...
    @property
    def oror(self) -> BVector4: ...
    @property
    def orog(self) -> BVector4: ...
    @property
    def orob(self) -> BVector4: ...
    @property
    def oroo(self) -> BVector4: ...
    @property
    def orol(self) -> BVector4: ...
    @property
    def orlr(self) -> BVector4: ...
    @property
    def orlg(self) -> BVector4: ...
    @property
    def orlb(self) -> BVector4: ...
    @property
    def orlo(self) -> BVector4: ...
    @property
    def orll(self) -> BVector4: ...
    @property
    def ogrr(self) -> BVector4: ...
    @property
    def ogrg(self) -> BVector4: ...
    @property
    def ogrb(self) -> BVector4: ...
    @property
    def ogro(self) -> BVector4: ...
    @property
    def ogrl(self) -> BVector4: ...
    @property
    def oggr(self) -> BVector4: ...
    @property
    def oggg(self) -> BVector4: ...
    @property
    def oggb(self) -> BVector4: ...
    @property
    def oggo(self) -> BVector4: ...
    @property
    def oggl(self) -> BVector4: ...
    @property
    def ogbr(self) -> BVector4: ...
    @property
    def ogbg(self) -> BVector4: ...
    @property
    def ogbb(self) -> BVector4: ...
    @property
    def ogbo(self) -> BVector4: ...
    @property
    def ogbl(self) -> BVector4: ...
    @property
    def ogor(self) -> BVector4: ...
    @property
    def ogog(self) -> BVector4: ...
    @property
    def ogob(self) -> BVector4: ...
    @property
    def ogoo(self) -> BVector4: ...
    @property
    def ogol(self) -> BVector4: ...
    @property
    def oglr(self) -> BVector4: ...
    @property
    def oglg(self) -> BVector4: ...
    @property
    def oglb(self) -> BVector4: ...
    @property
    def oglo(self) -> BVector4: ...
    @property
    def ogll(self) -> BVector4: ...
    @property
    def obrr(self) -> BVector4: ...
    @property
    def obrg(self) -> BVector4: ...
    @property
    def obrb(self) -> BVector4: ...
    @property
    def obro(self) -> BVector4: ...
    @property
    def obrl(self) -> BVector4: ...
    @property
    def obgr(self) -> BVector4: ...
    @property
    def obgg(self) -> BVector4: ...
    @property
    def obgb(self) -> BVector4: ...
    @property
    def obgo(self) -> BVector4: ...
    @property
    def obgl(self) -> BVector4: ...
    @property
    def obbr(self) -> BVector4: ...
    @property
    def obbg(self) -> BVector4: ...
    @property
    def obbb(self) -> BVector4: ...
    @property
    def obbo(self) -> BVector4: ...
    @property
    def obbl(self) -> BVector4: ...
    @property
    def obor(self) -> BVector4: ...
    @property
    def obog(self) -> BVector4: ...
    @property
    def obob(self) -> BVector4: ...
    @property
    def oboo(self) -> BVector4: ...
    @property
    def obol(self) -> BVector4: ...
    @property
    def oblr(self) -> BVector4: ...
    @property
    def oblg(self) -> BVector4: ...
    @property
    def oblb(self) -> BVector4: ...
    @property
    def oblo(self) -> BVector4: ...
    @property
    def obll(self) -> BVector4: ...
    @property
    def oorr(self) -> BVector4: ...
    @property
    def oorg(self) -> BVector4: ...
    @property
    def oorb(self) -> BVector4: ...
    @property
    def ooro(self) -> BVector4: ...
    @property
    def oorl(self) -> BVector4: ...
    @property
    def oogr(self) -> BVector4: ...
    @property
    def oogg(self) -> BVector4: ...
    @property
    def oogb(self) -> BVector4: ...
    @property
    def oogo(self) -> BVector4: ...
    @property
    def oogl(self) -> BVector4: ...
    @property
    def oobr(self) -> BVector4: ...
    @property
    def oobg(self) -> BVector4: ...
    @property
    def oobb(self) -> BVector4: ...
    @property
    def oobo(self) -> BVector4: ...
    @property
    def oobl(self) -> BVector4: ...
    @property
    def ooor(self) -> BVector4: ...
    @property
    def ooog(self) -> BVector4: ...
    @property
    def ooob(self) -> BVector4: ...
    @property
    def oolr(self) -> BVector4: ...
    @property
    def oolg(self) -> BVector4: ...
    @property
    def oolb(self) -> BVector4: ...
    @property
    def olrr(self) -> BVector4: ...
    @property
    def olrg(self) -> BVector4: ...
    @property
    def olrb(self) -> BVector4: ...
    @property
    def olro(self) -> BVector4: ...
    @property
    def olrl(self) -> BVector4: ...
    @property
    def olgr(self) -> BVector4: ...
    @property
    def olgg(self) -> BVector4: ...
    @property
    def olgb(self) -> BVector4: ...
    @property
    def olgo(self) -> BVector4: ...
    @property
    def olgl(self) -> BVector4: ...
    @property
    def olbr(self) -> BVector4: ...
    @property
    def olbg(self) -> BVector4: ...
    @property
    def olbb(self) -> BVector4: ...
    @property
    def olbo(self) -> BVector4: ...
    @property
    def olbl(self) -> BVector4: ...
    @property
    def olor(self) -> BVector4: ...
    @property
    def olog(self) -> BVector4: ...
    @property
    def olob(self) -> BVector4: ...
    @property
    def ollr(self) -> BVector4: ...
    @property
    def ollg(self) -> BVector4: ...
    @property
    def ollb(self) -> BVector4: ...
    @property
    def lrrr(self) -> BVector4: ...
    @property
    def lrrg(self) -> BVector4: ...
    @property
    def lrrb(self) -> BVector4: ...
    @property
    def lrro(self) -> BVector4: ...
    @property
    def lrrl(self) -> BVector4: ...
    @property
    def lrgr(self) -> BVector4: ...
    @property
    def lrgg(self) -> BVector4: ...
    @property
    def lrgb(self) -> BVector4: ...
    @property
    def lrgo(self) -> BVector4: ...
    @property
    def lrgl(self) -> BVector4: ...
    @property
    def lrbr(self) -> BVector4: ...
    @property
    def lrbg(self) -> BVector4: ...
    @property
    def lrbb(self) -> BVector4: ...
    @property
    def lrbo(self) -> BVector4: ...
    @property
    def lrbl(self) -> BVector4: ...
    @property
    def lror(self) -> BVector4: ...
    @property
    def lrog(self) -> BVector4: ...
    @property
    def lrob(self) -> BVector4: ...
    @property
    def lroo(self) -> BVector4: ...
    @property
    def lrol(self) -> BVector4: ...
    @property
    def lrlr(self) -> BVector4: ...
    @property
    def lrlg(self) -> BVector4: ...
    @property
    def lrlb(self) -> BVector4: ...
    @property
    def lrlo(self) -> BVector4: ...
    @property
    def lrll(self) -> BVector4: ...
    @property
    def lgrr(self) -> BVector4: ...
    @property
    def lgrg(self) -> BVector4: ...
    @property
    def lgrb(self) -> BVector4: ...
    @property
    def lgro(self) -> BVector4: ...
    @property
    def lgrl(self) -> BVector4: ...
    @property
    def lggr(self) -> BVector4: ...
    @property
    def lggg(self) -> BVector4: ...
    @property
    def lggb(self) -> BVector4: ...
    @property
    def lggo(self) -> BVector4: ...
    @property
    def lggl(self) -> BVector4: ...
    @property
    def lgbr(self) -> BVector4: ...
    @property
    def lgbg(self) -> BVector4: ...
    @property
    def lgbb(self) -> BVector4: ...
    @property
    def lgbo(self) -> BVector4: ...
    @property
    def lgbl(self) -> BVector4: ...
    @property
    def lgor(self) -> BVector4: ...
    @property
    def lgog(self) -> BVector4: ...
    @property
    def lgob(self) -> BVector4: ...
    @property
    def lgoo(self) -> BVector4: ...
    @property
    def lgol(self) -> BVector4: ...
    @property
    def lglr(self) -> BVector4: ...
    @property
    def lglg(self) -> BVector4: ...
    @property
    def lglb(self) -> BVector4: ...
    @property
    def lglo(self) -> BVector4: ...
    @property
    def lgll(self) -> BVector4: ...
    @property
    def lbrr(self) -> BVector4: ...
    @property
    def lbrg(self) -> BVector4: ...
    @property
    def lbrb(self) -> BVector4: ...
    @property
    def lbro(self) -> BVector4: ...
    @property
    def lbrl(self) -> BVector4: ...
    @property
    def lbgr(self) -> BVector4: ...
    @property
    def lbgg(self) -> BVector4: ...
    @property
    def lbgb(self) -> BVector4: ...
    @property
    def lbgo(self) -> BVector4: ...
    @property
    def lbgl(self) -> BVector4: ...
    @property
    def lbbr(self) -> BVector4: ...
    @property
    def lbbg(self) -> BVector4: ...
    @property
    def lbbb(self) -> BVector4: ...
    @property
    def lbbo(self) -> BVector4: ...
    @property
    def lbbl(self) -> BVector4: ...
    @property
    def lbor(self) -> BVector4: ...
    @property
    def lbog(self) -> BVector4: ...
    @property
    def lbob(self) -> BVector4: ...
    @property
    def lboo(self) -> BVector4: ...
    @property
    def lbol(self) -> BVector4: ...
    @property
    def lblr(self) -> BVector4: ...
    @property
    def lblg(self) -> BVector4: ...
    @property
    def lblb(self) -> BVector4: ...
    @property
    def lblo(self) -> BVector4: ...
    @property
    def lbll(self) -> BVector4: ...
    @property
    def lorr(self) -> BVector4: ...
    @property
    def lorg(self) -> BVector4: ...
    @property
    def lorb(self) -> BVector4: ...
    @property
    def loro(self) -> BVector4: ...
    @property
    def lorl(self) -> BVector4: ...
    @property
    def logr(self) -> BVector4: ...
    @property
    def logg(self) -> BVector4: ...
    @property
    def logb(self) -> BVector4: ...
    @property
    def logo(self) -> BVector4: ...
    @property
    def logl(self) -> BVector4: ...
    @property
    def lobr(self) -> BVector4: ...
    @property
    def lobg(self) -> BVector4: ...
    @property
    def lobb(self) -> BVector4: ...
    @property
    def lobo(self) -> BVector4: ...
    @property
    def lobl(self) -> BVector4: ...
    @property
    def loor(self) -> BVector4: ...
    @property
    def loog(self) -> BVector4: ...
    @property
    def loob(self) -> BVector4: ...
    @property
    def lolr(self) -> BVector4: ...
    @property
    def lolg(self) -> BVector4: ...
    @property
    def lolb(self) -> BVector4: ...
    @property
    def llrr(self) -> BVector4: ...
    @property
    def llrg(self) -> BVector4: ...
    @property
    def llrb(self) -> BVector4: ...
    @property
    def llro(self) -> BVector4: ...
    @property
    def llrl(self) -> BVector4: ...
    @property
    def llgr(self) -> BVector4: ...
    @property
    def llgg(self) -> BVector4: ...
    @property
    def llgb(self) -> BVector4: ...
    @property
    def llgo(self) -> BVector4: ...
    @property
    def llgl(self) -> BVector4: ...
    @property
    def llbr(self) -> BVector4: ...
    @property
    def llbg(self) -> BVector4: ...
    @property
    def llbb(self) -> BVector4: ...
    @property
    def llbo(self) -> BVector4: ...
    @property
    def llbl(self) -> BVector4: ...
    @property
    def llor(self) -> BVector4: ...
    @property
    def llog(self) -> BVector4: ...
    @property
    def llob(self) -> BVector4: ...
    @property
    def lllr(self) -> BVector4: ...
    @property
    def lllg(self) -> BVector4: ...
    @property
    def lllb(self) -> BVector4: ...
    @property
    def s(self) -> _bool: ...
    @property
    def t(self) -> _bool: ...
    @property
    def q(self) -> _bool: ...
    @property
    def ss(self) -> BVector2: ...
    @property
    def st(self) -> BVector2: ...
    @property
    def sq(self) -> BVector2: ...
    @property
    def so(self) -> BVector2: ...
    @property
    def sl(self) -> BVector2: ...
    @property
    def ts(self) -> BVector2: ...
    @property
    def tt(self) -> BVector2: ...
    @property
    def tq(self) -> BVector2: ...
    @property
    def to(self) -> BVector2: ...
    @property
    def tl(self) -> BVector2: ...
    @property
    def qs(self) -> BVector2: ...
    @property
    def qt(self) -> BVector2: ...
    @property
    def qq(self) -> BVector2: ...
    @property
    def qo(self) -> BVector2: ...
    @property
    def ql(self) -> BVector2: ...
    @property
    def os(self) -> BVector2: ...
    @property
    def ot(self) -> BVector2: ...
    @property
    def oq(self) -> BVector2: ...
    @property
    def ls(self) -> BVector2: ...
    @property
    def lt(self) -> BVector2: ...
    @property
    def lq(self) -> BVector2: ...
    @property
    def sss(self) -> BVector3: ...
    @property
    def sst(self) -> BVector3: ...
    @property
    def ssq(self) -> BVector3: ...
    @property
    def sso(self) -> BVector3: ...
    @property
    def ssl(self) -> BVector3: ...
    @property
    def sts(self) -> BVector3: ...
    @property
    def stt(self) -> BVector3: ...
    @property
    def stq(self) -> BVector3: ...
    @property
    def sto(self) -> BVector3: ...
    @property
    def stl(self) -> BVector3: ...
    @property
    def sqs(self) -> BVector3: ...
    @property
    def sqt(self) -> BVector3: ...
    @property
    def sqq(self) -> BVector3: ...
    @property
    def sqo(self) -> BVector3: ...
    @property
    def sql(self) -> BVector3: ...
    @property
    def sos(self) -> BVector3: ...
    @property
    def sot(self) -> BVector3: ...
    @property
    def soq(self) -> BVector3: ...
    @property
    def soo(self) -> BVector3: ...
    @property
    def sol(self) -> BVector3: ...
    @property
    def sls(self) -> BVector3: ...
    @property
    def slt(self) -> BVector3: ...
    @property
    def slq(self) -> BVector3: ...
    @property
    def slo(self) -> BVector3: ...
    @property
    def sll(self) -> BVector3: ...
    @property
    def tss(self) -> BVector3: ...
    @property
    def tst(self) -> BVector3: ...
    @property
    def tsq(self) -> BVector3: ...
    @property
    def tso(self) -> BVector3: ...
    @property
    def tsl(self) -> BVector3: ...
    @property
    def tts(self) -> BVector3: ...
    @property
    def ttt(self) -> BVector3: ...
    @property
    def ttq(self) -> BVector3: ...
    @property
    def tto(self) -> BVector3: ...
    @property
    def ttl(self) -> BVector3: ...
    @property
    def tqs(self) -> BVector3: ...
    @property
    def tqt(self) -> BVector3: ...
    @property
    def tqq(self) -> BVector3: ...
    @property
    def tqo(self) -> BVector3: ...
    @property
    def tql(self) -> BVector3: ...
    @property
    def tos(self) -> BVector3: ...
    @property
    def tot(self) -> BVector3: ...
    @property
    def toq(self) -> BVector3: ...
    @property
    def too(self) -> BVector3: ...
    @property
    def tol(self) -> BVector3: ...
    @property
    def tls(self) -> BVector3: ...
    @property
    def tlt(self) -> BVector3: ...
    @property
    def tlq(self) -> BVector3: ...
    @property
    def tlo(self) -> BVector3: ...
    @property
    def tll(self) -> BVector3: ...
    @property
    def qss(self) -> BVector3: ...
    @property
    def qst(self) -> BVector3: ...
    @property
    def qsq(self) -> BVector3: ...
    @property
    def qso(self) -> BVector3: ...
    @property
    def qsl(self) -> BVector3: ...
    @property
    def qts(self) -> BVector3: ...
    @property
    def qtt(self) -> BVector3: ...
    @property
    def qtq(self) -> BVector3: ...
    @property
    def qto(self) -> BVector3: ...
    @property
    def qtl(self) -> BVector3: ...
    @property
    def qqs(self) -> BVector3: ...
    @property
    def qqt(self) -> BVector3: ...
    @property
    def qqq(self) -> BVector3: ...
    @property
    def qqo(self) -> BVector3: ...
    @property
    def qql(self) -> BVector3: ...
    @property
    def qos(self) -> BVector3: ...
    @property
    def qot(self) -> BVector3: ...
    @property
    def qoq(self) -> BVector3: ...
    @property
    def qoo(self) -> BVector3: ...
    @property
    def qol(self) -> BVector3: ...
    @property
    def qls(self) -> BVector3: ...
    @property
    def qlt(self) -> BVector3: ...
    @property
    def qlq(self) -> BVector3: ...
    @property
    def qlo(self) -> BVector3: ...
    @property
    def qll(self) -> BVector3: ...
    @property
    def oss(self) -> BVector3: ...
    @property
    def ost(self) -> BVector3: ...
    @property
    def osq(self) -> BVector3: ...
    @property
    def oso(self) -> BVector3: ...
    @property
    def osl(self) -> BVector3: ...
    @property
    def ots(self) -> BVector3: ...
    @property
    def ott(self) -> BVector3: ...
    @property
    def otq(self) -> BVector3: ...
    @property
    def oto(self) -> BVector3: ...
    @property
    def otl(self) -> BVector3: ...
    @property
    def oqs(self) -> BVector3: ...
    @property
    def oqt(self) -> BVector3: ...
    @property
    def oqq(self) -> BVector3: ...
    @property
    def oqo(self) -> BVector3: ...
    @property
    def oql(self) -> BVector3: ...
    @property
    def oos(self) -> BVector3: ...
    @property
    def oot(self) -> BVector3: ...
    @property
    def ooq(self) -> BVector3: ...
    @property
    def ols(self) -> BVector3: ...
    @property
    def olt(self) -> BVector3: ...
    @property
    def olq(self) -> BVector3: ...
    @property
    def lss(self) -> BVector3: ...
    @property
    def lst(self) -> BVector3: ...
    @property
    def lsq(self) -> BVector3: ...
    @property
    def lso(self) -> BVector3: ...
    @property
    def lsl(self) -> BVector3: ...
    @property
    def lts(self) -> BVector3: ...
    @property
    def ltt(self) -> BVector3: ...
    @property
    def ltq(self) -> BVector3: ...
    @property
    def lto(self) -> BVector3: ...
    @property
    def ltl(self) -> BVector3: ...
    @property
    def lqs(self) -> BVector3: ...
    @property
    def lqt(self) -> BVector3: ...
    @property
    def lqq(self) -> BVector3: ...
    @property
    def lqo(self) -> BVector3: ...
    @property
    def lql(self) -> BVector3: ...
    @property
    def los(self) -> BVector3: ...
    @property
    def lot(self) -> BVector3: ...
    @property
    def loq(self) -> BVector3: ...
    @property
    def lls(self) -> BVector3: ...
    @property
    def llt(self) -> BVector3: ...
    @property
    def llq(self) -> BVector3: ...
    @property
    def ssss(self) -> BVector4: ...
    @property
    def ssst(self) -> BVector4: ...
    @property
    def sssq(self) -> BVector4: ...
    @property
    def ssso(self) -> BVector4: ...
    @property
    def sssl(self) -> BVector4: ...
    @property
    def ssts(self) -> BVector4: ...
    @property
    def sstt(self) -> BVector4: ...
    @property
    def sstq(self) -> BVector4: ...
    @property
    def ssto(self) -> BVector4: ...
    @property
    def sstl(self) -> BVector4: ...
    @property
    def ssqs(self) -> BVector4: ...
    @property
    def ssqt(self) -> BVector4: ...
    @property
    def ssqq(self) -> BVector4: ...
    @property
    def ssqo(self) -> BVector4: ...
    @property
    def ssql(self) -> BVector4: ...
    @property
    def ssos(self) -> BVector4: ...
    @property
    def ssot(self) -> BVector4: ...
    @property
    def ssoq(self) -> BVector4: ...
    @property
    def ssoo(self) -> BVector4: ...
    @property
    def ssol(self) -> BVector4: ...
    @property
    def ssls(self) -> BVector4: ...
    @property
    def sslt(self) -> BVector4: ...
    @property
    def sslq(self) -> BVector4: ...
    @property
    def sslo(self) -> BVector4: ...
    @property
    def ssll(self) -> BVector4: ...
    @property
    def stss(self) -> BVector4: ...
    @property
    def stst(self) -> BVector4: ...
    @property
    def stsq(self) -> BVector4: ...
    @property
    def stso(self) -> BVector4: ...
    @property
    def stsl(self) -> BVector4: ...
    @property
    def stts(self) -> BVector4: ...
    @property
    def sttt(self) -> BVector4: ...
    @property
    def sttq(self) -> BVector4: ...
    @property
    def stto(self) -> BVector4: ...
    @property
    def sttl(self) -> BVector4: ...
    @property
    def stqs(self) -> BVector4: ...
    @property
    def stqt(self) -> BVector4: ...
    @property
    def stqq(self) -> BVector4: ...
    @property
    def stqo(self) -> BVector4: ...
    @property
    def stql(self) -> BVector4: ...
    @property
    def stos(self) -> BVector4: ...
    @property
    def stot(self) -> BVector4: ...
    @property
    def stoq(self) -> BVector4: ...
    @property
    def stoo(self) -> BVector4: ...
    @property
    def stol(self) -> BVector4: ...
    @property
    def stls(self) -> BVector4: ...
    @property
    def stlt(self) -> BVector4: ...
    @property
    def stlq(self) -> BVector4: ...
    @property
    def stlo(self) -> BVector4: ...
    @property
    def stll(self) -> BVector4: ...
    @property
    def sqss(self) -> BVector4: ...
    @property
    def sqst(self) -> BVector4: ...
    @property
    def sqsq(self) -> BVector4: ...
    @property
    def sqso(self) -> BVector4: ...
    @property
    def sqsl(self) -> BVector4: ...
    @property
    def sqts(self) -> BVector4: ...
    @property
    def sqtt(self) -> BVector4: ...
    @property
    def sqtq(self) -> BVector4: ...
    @property
    def sqto(self) -> BVector4: ...
    @property
    def sqtl(self) -> BVector4: ...
    @property
    def sqqs(self) -> BVector4: ...
    @property
    def sqqt(self) -> BVector4: ...
    @property
    def sqqq(self) -> BVector4: ...
    @property
    def sqqo(self) -> BVector4: ...
    @property
    def sqql(self) -> BVector4: ...
    @property
    def sqos(self) -> BVector4: ...
    @property
    def sqot(self) -> BVector4: ...
    @property
    def sqoq(self) -> BVector4: ...
    @property
    def sqoo(self) -> BVector4: ...
    @property
    def sqol(self) -> BVector4: ...
    @property
    def sqls(self) -> BVector4: ...
    @property
    def sqlt(self) -> BVector4: ...
    @property
    def sqlq(self) -> BVector4: ...
    @property
    def sqlo(self) -> BVector4: ...
    @property
    def sqll(self) -> BVector4: ...
    @property
    def soss(self) -> BVector4: ...
    @property
    def sost(self) -> BVector4: ...
    @property
    def sosq(self) -> BVector4: ...
    @property
    def soso(self) -> BVector4: ...
    @property
    def sosl(self) -> BVector4: ...
    @property
    def sots(self) -> BVector4: ...
    @property
    def sott(self) -> BVector4: ...
    @property
    def sotq(self) -> BVector4: ...
    @property
    def soto(self) -> BVector4: ...
    @property
    def sotl(self) -> BVector4: ...
    @property
    def soqs(self) -> BVector4: ...
    @property
    def soqt(self) -> BVector4: ...
    @property
    def soqq(self) -> BVector4: ...
    @property
    def soqo(self) -> BVector4: ...
    @property
    def soql(self) -> BVector4: ...
    @property
    def soos(self) -> BVector4: ...
    @property
    def soot(self) -> BVector4: ...
    @property
    def sooq(self) -> BVector4: ...
    @property
    def sooo(self) -> BVector4: ...
    @property
    def sool(self) -> BVector4: ...
    @property
    def sols(self) -> BVector4: ...
    @property
    def solt(self) -> BVector4: ...
    @property
    def solq(self) -> BVector4: ...
    @property
    def solo(self) -> BVector4: ...
    @property
    def soll(self) -> BVector4: ...
    @property
    def slss(self) -> BVector4: ...
    @property
    def slst(self) -> BVector4: ...
    @property
    def slsq(self) -> BVector4: ...
    @property
    def slso(self) -> BVector4: ...
    @property
    def slsl(self) -> BVector4: ...
    @property
    def slts(self) -> BVector4: ...
    @property
    def sltt(self) -> BVector4: ...
    @property
    def sltq(self) -> BVector4: ...
    @property
    def slto(self) -> BVector4: ...
    @property
    def sltl(self) -> BVector4: ...
    @property
    def slqs(self) -> BVector4: ...
    @property
    def slqt(self) -> BVector4: ...
    @property
    def slqq(self) -> BVector4: ...
    @property
    def slqo(self) -> BVector4: ...
    @property
    def slql(self) -> BVector4: ...
    @property
    def slos(self) -> BVector4: ...
    @property
    def slot(self) -> BVector4: ...
    @property
    def sloq(self) -> BVector4: ...
    @property
    def sloo(self) -> BVector4: ...
    @property
    def slol(self) -> BVector4: ...
    @property
    def slls(self) -> BVector4: ...
    @property
    def sllt(self) -> BVector4: ...
    @property
    def sllq(self) -> BVector4: ...
    @property
    def sllo(self) -> BVector4: ...
    @property
    def slll(self) -> BVector4: ...
    @property
    def tsss(self) -> BVector4: ...
    @property
    def tsst(self) -> BVector4: ...
    @property
    def tssq(self) -> BVector4: ...
    @property
    def tsso(self) -> BVector4: ...
    @property
    def tssl(self) -> BVector4: ...
    @property
    def tsts(self) -> BVector4: ...
    @property
    def tstt(self) -> BVector4: ...
    @property
    def tstq(self) -> BVector4: ...
    @property
    def tsto(self) -> BVector4: ...
    @property
    def tstl(self) -> BVector4: ...
    @property
    def tsqs(self) -> BVector4: ...
    @property
    def tsqt(self) -> BVector4: ...
    @property
    def tsqq(self) -> BVector4: ...
    @property
    def tsqo(self) -> BVector4: ...
    @property
    def tsql(self) -> BVector4: ...
    @property
    def tsos(self) -> BVector4: ...
    @property
    def tsot(self) -> BVector4: ...
    @property
    def tsoq(self) -> BVector4: ...
    @property
    def tsoo(self) -> BVector4: ...
    @property
    def tsol(self) -> BVector4: ...
    @property
    def tsls(self) -> BVector4: ...
    @property
    def tslt(self) -> BVector4: ...
    @property
    def tslq(self) -> BVector4: ...
    @property
    def tslo(self) -> BVector4: ...
    @property
    def tsll(self) -> BVector4: ...
    @property
    def ttss(self) -> BVector4: ...
    @property
    def ttst(self) -> BVector4: ...
    @property
    def ttsq(self) -> BVector4: ...
    @property
    def ttso(self) -> BVector4: ...
    @property
    def ttsl(self) -> BVector4: ...
    @property
    def ttts(self) -> BVector4: ...
    @property
    def tttt(self) -> BVector4: ...
    @property
    def tttq(self) -> BVector4: ...
    @property
    def ttto(self) -> BVector4: ...
    @property
    def tttl(self) -> BVector4: ...
    @property
    def ttqs(self) -> BVector4: ...
    @property
    def ttqt(self) -> BVector4: ...
    @property
    def ttqq(self) -> BVector4: ...
    @property
    def ttqo(self) -> BVector4: ...
    @property
    def ttql(self) -> BVector4: ...
    @property
    def ttos(self) -> BVector4: ...
    @property
    def ttot(self) -> BVector4: ...
    @property
    def ttoq(self) -> BVector4: ...
    @property
    def ttoo(self) -> BVector4: ...
    @property
    def ttol(self) -> BVector4: ...
    @property
    def ttls(self) -> BVector4: ...
    @property
    def ttlt(self) -> BVector4: ...
    @property
    def ttlq(self) -> BVector4: ...
    @property
    def ttlo(self) -> BVector4: ...
    @property
    def ttll(self) -> BVector4: ...
    @property
    def tqss(self) -> BVector4: ...
    @property
    def tqst(self) -> BVector4: ...
    @property
    def tqsq(self) -> BVector4: ...
    @property
    def tqso(self) -> BVector4: ...
    @property
    def tqsl(self) -> BVector4: ...
    @property
    def tqts(self) -> BVector4: ...
    @property
    def tqtt(self) -> BVector4: ...
    @property
    def tqtq(self) -> BVector4: ...
    @property
    def tqto(self) -> BVector4: ...
    @property
    def tqtl(self) -> BVector4: ...
    @property
    def tqqs(self) -> BVector4: ...
    @property
    def tqqt(self) -> BVector4: ...
    @property
    def tqqq(self) -> BVector4: ...
    @property
    def tqqo(self) -> BVector4: ...
    @property
    def tqql(self) -> BVector4: ...
    @property
    def tqos(self) -> BVector4: ...
    @property
    def tqot(self) -> BVector4: ...
    @property
    def tqoq(self) -> BVector4: ...
    @property
    def tqoo(self) -> BVector4: ...
    @property
    def tqol(self) -> BVector4: ...
    @property
    def tqls(self) -> BVector4: ...
    @property
    def tqlt(self) -> BVector4: ...
    @property
    def tqlq(self) -> BVector4: ...
    @property
    def tqlo(self) -> BVector4: ...
    @property
    def tqll(self) -> BVector4: ...
    @property
    def toss(self) -> BVector4: ...
    @property
    def tost(self) -> BVector4: ...
    @property
    def tosq(self) -> BVector4: ...
    @property
    def toso(self) -> BVector4: ...
    @property
    def tosl(self) -> BVector4: ...
    @property
    def tots(self) -> BVector4: ...
    @property
    def tott(self) -> BVector4: ...
    @property
    def totq(self) -> BVector4: ...
    @property
    def toto(self) -> BVector4: ...
    @property
    def totl(self) -> BVector4: ...
    @property
    def toqs(self) -> BVector4: ...
    @property
    def toqt(self) -> BVector4: ...
    @property
    def toqq(self) -> BVector4: ...
    @property
    def toqo(self) -> BVector4: ...
    @property
    def toql(self) -> BVector4: ...
    @property
    def toos(self) -> BVector4: ...
    @property
    def toot(self) -> BVector4: ...
    @property
    def tooq(self) -> BVector4: ...
    @property
    def tooo(self) -> BVector4: ...
    @property
    def tool(self) -> BVector4: ...
    @property
    def tols(self) -> BVector4: ...
    @property
    def tolt(self) -> BVector4: ...
    @property
    def tolq(self) -> BVector4: ...
    @property
    def tolo(self) -> BVector4: ...
    @property
    def toll(self) -> BVector4: ...
    @property
    def tlss(self) -> BVector4: ...
    @property
    def tlst(self) -> BVector4: ...
    @property
    def tlsq(self) -> BVector4: ...
    @property
    def tlso(self) -> BVector4: ...
    @property
    def tlsl(self) -> BVector4: ...
    @property
    def tlts(self) -> BVector4: ...
    @property
    def tltt(self) -> BVector4: ...
    @property
    def tltq(self) -> BVector4: ...
    @property
    def tlto(self) -> BVector4: ...
    @property
    def tltl(self) -> BVector4: ...
    @property
    def tlqs(self) -> BVector4: ...
    @property
    def tlqt(self) -> BVector4: ...
    @property
    def tlqq(self) -> BVector4: ...
    @property
    def tlqo(self) -> BVector4: ...
    @property
    def tlql(self) -> BVector4: ...
    @property
    def tlos(self) -> BVector4: ...
    @property
    def tlot(self) -> BVector4: ...
    @property
    def tloq(self) -> BVector4: ...
    @property
    def tloo(self) -> BVector4: ...
    @property
    def tlol(self) -> BVector4: ...
    @property
    def tlls(self) -> BVector4: ...
    @property
    def tllt(self) -> BVector4: ...
    @property
    def tllq(self) -> BVector4: ...
    @property
    def tllo(self) -> BVector4: ...
    @property
    def tlll(self) -> BVector4: ...
    @property
    def qsss(self) -> BVector4: ...
    @property
    def qsst(self) -> BVector4: ...
    @property
    def qssq(self) -> BVector4: ...
    @property
    def qsso(self) -> BVector4: ...
    @property
    def qssl(self) -> BVector4: ...
    @property
    def qsts(self) -> BVector4: ...
    @property
    def qstt(self) -> BVector4: ...
    @property
    def qstq(self) -> BVector4: ...
    @property
    def qsto(self) -> BVector4: ...
    @property
    def qstl(self) -> BVector4: ...
    @property
    def qsqs(self) -> BVector4: ...
    @property
    def qsqt(self) -> BVector4: ...
    @property
    def qsqq(self) -> BVector4: ...
    @property
    def qsqo(self) -> BVector4: ...
    @property
    def qsql(self) -> BVector4: ...
    @property
    def qsos(self) -> BVector4: ...
    @property
    def qsot(self) -> BVector4: ...
    @property
    def qsoq(self) -> BVector4: ...
    @property
    def qsoo(self) -> BVector4: ...
    @property
    def qsol(self) -> BVector4: ...
    @property
    def qsls(self) -> BVector4: ...
    @property
    def qslt(self) -> BVector4: ...
    @property
    def qslq(self) -> BVector4: ...
    @property
    def qslo(self) -> BVector4: ...
    @property
    def qsll(self) -> BVector4: ...
    @property
    def qtss(self) -> BVector4: ...
    @property
    def qtst(self) -> BVector4: ...
    @property
    def qtsq(self) -> BVector4: ...
    @property
    def qtso(self) -> BVector4: ...
    @property
    def qtsl(self) -> BVector4: ...
    @property
    def qtts(self) -> BVector4: ...
    @property
    def qttt(self) -> BVector4: ...
    @property
    def qttq(self) -> BVector4: ...
    @property
    def qtto(self) -> BVector4: ...
    @property
    def qttl(self) -> BVector4: ...
    @property
    def qtqs(self) -> BVector4: ...
    @property
    def qtqt(self) -> BVector4: ...
    @property
    def qtqq(self) -> BVector4: ...
    @property
    def qtqo(self) -> BVector4: ...
    @property
    def qtql(self) -> BVector4: ...
    @property
    def qtos(self) -> BVector4: ...
    @property
    def qtot(self) -> BVector4: ...
    @property
    def qtoq(self) -> BVector4: ...
    @property
    def qtoo(self) -> BVector4: ...
    @property
    def qtol(self) -> BVector4: ...
    @property
    def qtls(self) -> BVector4: ...
    @property
    def qtlt(self) -> BVector4: ...
    @property
    def qtlq(self) -> BVector4: ...
    @property
    def qtlo(self) -> BVector4: ...
    @property
    def qtll(self) -> BVector4: ...
    @property
    def qqss(self) -> BVector4: ...
    @property
    def qqst(self) -> BVector4: ...
    @property
    def qqsq(self) -> BVector4: ...
    @property
    def qqso(self) -> BVector4: ...
    @property
    def qqsl(self) -> BVector4: ...
    @property
    def qqts(self) -> BVector4: ...
    @property
    def qqtt(self) -> BVector4: ...
    @property
    def qqtq(self) -> BVector4: ...
    @property
    def qqto(self) -> BVector4: ...
    @property
    def qqtl(self) -> BVector4: ...
    @property
    def qqqs(self) -> BVector4: ...
    @property
    def qqqt(self) -> BVector4: ...
    @property
    def qqqq(self) -> BVector4: ...
    @property
    def qqqo(self) -> BVector4: ...
    @property
    def qqql(self) -> BVector4: ...
    @property
    def qqos(self) -> BVector4: ...
    @property
    def qqot(self) -> BVector4: ...
    @property
    def qqoq(self) -> BVector4: ...
    @property
    def qqoo(self) -> BVector4: ...
    @property
    def qqol(self) -> BVector4: ...
    @property
    def qqls(self) -> BVector4: ...
    @property
    def qqlt(self) -> BVector4: ...
    @property
    def qqlq(self) -> BVector4: ...
    @property
    def qqlo(self) -> BVector4: ...
    @property
    def qqll(self) -> BVector4: ...
    @property
    def qoss(self) -> BVector4: ...
    @property
    def qost(self) -> BVector4: ...
    @property
    def qosq(self) -> BVector4: ...
    @property
    def qoso(self) -> BVector4: ...
    @property
    def qosl(self) -> BVector4: ...
    @property
    def qots(self) -> BVector4: ...
    @property
    def qott(self) -> BVector4: ...
    @property
    def qotq(self) -> BVector4: ...
    @property
    def qoto(self) -> BVector4: ...
    @property
    def qotl(self) -> BVector4: ...
    @property
    def qoqs(self) -> BVector4: ...
    @property
    def qoqt(self) -> BVector4: ...
    @property
    def qoqq(self) -> BVector4: ...
    @property
    def qoqo(self) -> BVector4: ...
    @property
    def qoql(self) -> BVector4: ...
    @property
    def qoos(self) -> BVector4: ...
    @property
    def qoot(self) -> BVector4: ...
    @property
    def qooq(self) -> BVector4: ...
    @property
    def qooo(self) -> BVector4: ...
    @property
    def qool(self) -> BVector4: ...
    @property
    def qols(self) -> BVector4: ...
    @property
    def qolt(self) -> BVector4: ...
    @property
    def qolq(self) -> BVector4: ...
    @property
    def qolo(self) -> BVector4: ...
    @property
    def qoll(self) -> BVector4: ...
    @property
    def qlss(self) -> BVector4: ...
    @property
    def qlst(self) -> BVector4: ...
    @property
    def qlsq(self) -> BVector4: ...
    @property
    def qlso(self) -> BVector4: ...
    @property
    def qlsl(self) -> BVector4: ...
    @property
    def qlts(self) -> BVector4: ...
    @property
    def qltt(self) -> BVector4: ...
    @property
    def qltq(self) -> BVector4: ...
    @property
    def qlto(self) -> BVector4: ...
    @property
    def qltl(self) -> BVector4: ...
    @property
    def qlqs(self) -> BVector4: ...
    @property
    def qlqt(self) -> BVector4: ...
    @property
    def qlqq(self) -> BVector4: ...
    @property
    def qlqo(self) -> BVector4: ...
    @property
    def qlql(self) -> BVector4: ...
    @property
    def qlos(self) -> BVector4: ...
    @property
    def qlot(self) -> BVector4: ...
    @property
    def qloq(self) -> BVector4: ...
    @property
    def qloo(self) -> BVector4: ...
    @property
    def qlol(self) -> BVector4: ...
    @property
    def qlls(self) -> BVector4: ...
    @property
    def qllt(self) -> BVector4: ...
    @property
    def qllq(self) -> BVector4: ...
    @property
    def qllo(self) -> BVector4: ...
    @property
    def qlll(self) -> BVector4: ...
    @property
    def osss(self) -> BVector4: ...
    @property
    def osst(self) -> BVector4: ...
    @property
    def ossq(self) -> BVector4: ...
    @property
    def osso(self) -> BVector4: ...
    @property
    def ossl(self) -> BVector4: ...
    @property
    def osts(self) -> BVector4: ...
    @property
    def ostt(self) -> BVector4: ...
    @property
    def ostq(self) -> BVector4: ...
    @property
    def osto(self) -> BVector4: ...
    @property
    def ostl(self) -> BVector4: ...
    @property
    def osqs(self) -> BVector4: ...
    @property
    def osqt(self) -> BVector4: ...
    @property
    def osqq(self) -> BVector4: ...
    @property
    def osqo(self) -> BVector4: ...
    @property
    def osql(self) -> BVector4: ...
    @property
    def osos(self) -> BVector4: ...
    @property
    def osot(self) -> BVector4: ...
    @property
    def osoq(self) -> BVector4: ...
    @property
    def osoo(self) -> BVector4: ...
    @property
    def osol(self) -> BVector4: ...
    @property
    def osls(self) -> BVector4: ...
    @property
    def oslt(self) -> BVector4: ...
    @property
    def oslq(self) -> BVector4: ...
    @property
    def oslo(self) -> BVector4: ...
    @property
    def osll(self) -> BVector4: ...
    @property
    def otss(self) -> BVector4: ...
    @property
    def otst(self) -> BVector4: ...
    @property
    def otsq(self) -> BVector4: ...
    @property
    def otso(self) -> BVector4: ...
    @property
    def otsl(self) -> BVector4: ...
    @property
    def otts(self) -> BVector4: ...
    @property
    def ottt(self) -> BVector4: ...
    @property
    def ottq(self) -> BVector4: ...
    @property
    def otto(self) -> BVector4: ...
    @property
    def ottl(self) -> BVector4: ...
    @property
    def otqs(self) -> BVector4: ...
    @property
    def otqt(self) -> BVector4: ...
    @property
    def otqq(self) -> BVector4: ...
    @property
    def otqo(self) -> BVector4: ...
    @property
    def otql(self) -> BVector4: ...
    @property
    def otos(self) -> BVector4: ...
    @property
    def otot(self) -> BVector4: ...
    @property
    def otoq(self) -> BVector4: ...
    @property
    def otoo(self) -> BVector4: ...
    @property
    def otol(self) -> BVector4: ...
    @property
    def otls(self) -> BVector4: ...
    @property
    def otlt(self) -> BVector4: ...
    @property
    def otlq(self) -> BVector4: ...
    @property
    def otlo(self) -> BVector4: ...
    @property
    def otll(self) -> BVector4: ...
    @property
    def oqss(self) -> BVector4: ...
    @property
    def oqst(self) -> BVector4: ...
    @property
    def oqsq(self) -> BVector4: ...
    @property
    def oqso(self) -> BVector4: ...
    @property
    def oqsl(self) -> BVector4: ...
    @property
    def oqts(self) -> BVector4: ...
    @property
    def oqtt(self) -> BVector4: ...
    @property
    def oqtq(self) -> BVector4: ...
    @property
    def oqto(self) -> BVector4: ...
    @property
    def oqtl(self) -> BVector4: ...
    @property
    def oqqs(self) -> BVector4: ...
    @property
    def oqqt(self) -> BVector4: ...
    @property
    def oqqq(self) -> BVector4: ...
    @property
    def oqqo(self) -> BVector4: ...
    @property
    def oqql(self) -> BVector4: ...
    @property
    def oqos(self) -> BVector4: ...
    @property
    def oqot(self) -> BVector4: ...
    @property
    def oqoq(self) -> BVector4: ...
    @property
    def oqoo(self) -> BVector4: ...
    @property
    def oqol(self) -> BVector4: ...
    @property
    def oqls(self) -> BVector4: ...
    @property
    def oqlt(self) -> BVector4: ...
    @property
    def oqlq(self) -> BVector4: ...
    @property
    def oqlo(self) -> BVector4: ...
    @property
    def oqll(self) -> BVector4: ...
    @property
    def ooss(self) -> BVector4: ...
    @property
    def oost(self) -> BVector4: ...
    @property
    def oosq(self) -> BVector4: ...
    @property
    def ooso(self) -> BVector4: ...
    @property
    def oosl(self) -> BVector4: ...
    @property
    def oots(self) -> BVector4: ...
    @property
    def oott(self) -> BVector4: ...
    @property
    def ootq(self) -> BVector4: ...
    @property
    def ooto(self) -> BVector4: ...
    @property
    def ootl(self) -> BVector4: ...
    @property
    def ooqs(self) -> BVector4: ...
    @property
    def ooqt(self) -> BVector4: ...
    @property
    def ooqq(self) -> BVector4: ...
    @property
    def ooqo(self) -> BVector4: ...
    @property
    def ooql(self) -> BVector4: ...
    @property
    def ooos(self) -> BVector4: ...
    @property
    def ooot(self) -> BVector4: ...
    @property
    def oooq(self) -> BVector4: ...
    @property
    def ools(self) -> BVector4: ...
    @property
    def oolt(self) -> BVector4: ...
    @property
    def oolq(self) -> BVector4: ...
    @property
    def olss(self) -> BVector4: ...
    @property
    def olst(self) -> BVector4: ...
    @property
    def olsq(self) -> BVector4: ...
    @property
    def olso(self) -> BVector4: ...
    @property
    def olsl(self) -> BVector4: ...
    @property
    def olts(self) -> BVector4: ...
    @property
    def oltt(self) -> BVector4: ...
    @property
    def oltq(self) -> BVector4: ...
    @property
    def olto(self) -> BVector4: ...
    @property
    def oltl(self) -> BVector4: ...
    @property
    def olqs(self) -> BVector4: ...
    @property
    def olqt(self) -> BVector4: ...
    @property
    def olqq(self) -> BVector4: ...
    @property
    def olqo(self) -> BVector4: ...
    @property
    def olql(self) -> BVector4: ...
    @property
    def olos(self) -> BVector4: ...
    @property
    def olot(self) -> BVector4: ...
    @property
    def oloq(self) -> BVector4: ...
    @property
    def olls(self) -> BVector4: ...
    @property
    def ollt(self) -> BVector4: ...
    @property
    def ollq(self) -> BVector4: ...
    @property
    def lsss(self) -> BVector4: ...
    @property
    def lsst(self) -> BVector4: ...
    @property
    def lssq(self) -> BVector4: ...
    @property
    def lsso(self) -> BVector4: ...
    @property
    def lssl(self) -> BVector4: ...
    @property
    def lsts(self) -> BVector4: ...
    @property
    def lstt(self) -> BVector4: ...
    @property
    def lstq(self) -> BVector4: ...
    @property
    def lsto(self) -> BVector4: ...
    @property
    def lstl(self) -> BVector4: ...
    @property
    def lsqs(self) -> BVector4: ...
    @property
    def lsqt(self) -> BVector4: ...
    @property
    def lsqq(self) -> BVector4: ...
    @property
    def lsqo(self) -> BVector4: ...
    @property
    def lsql(self) -> BVector4: ...
    @property
    def lsos(self) -> BVector4: ...
    @property
    def lsot(self) -> BVector4: ...
    @property
    def lsoq(self) -> BVector4: ...
    @property
    def lsoo(self) -> BVector4: ...
    @property
    def lsol(self) -> BVector4: ...
    @property
    def lsls(self) -> BVector4: ...
    @property
    def lslt(self) -> BVector4: ...
    @property
    def lslq(self) -> BVector4: ...
    @property
    def lslo(self) -> BVector4: ...
    @property
    def lsll(self) -> BVector4: ...
    @property
    def ltss(self) -> BVector4: ...
    @property
    def ltst(self) -> BVector4: ...
    @property
    def ltsq(self) -> BVector4: ...
    @property
    def ltso(self) -> BVector4: ...
    @property
    def ltsl(self) -> BVector4: ...
    @property
    def ltts(self) -> BVector4: ...
    @property
    def lttt(self) -> BVector4: ...
    @property
    def lttq(self) -> BVector4: ...
    @property
    def ltto(self) -> BVector4: ...
    @property
    def lttl(self) -> BVector4: ...
    @property
    def ltqs(self) -> BVector4: ...
    @property
    def ltqt(self) -> BVector4: ...
    @property
    def ltqq(self) -> BVector4: ...
    @property
    def ltqo(self) -> BVector4: ...
    @property
    def ltql(self) -> BVector4: ...
    @property
    def ltos(self) -> BVector4: ...
    @property
    def ltot(self) -> BVector4: ...
    @property
    def ltoq(self) -> BVector4: ...
    @property
    def ltoo(self) -> BVector4: ...
    @property
    def ltol(self) -> BVector4: ...
    @property
    def ltls(self) -> BVector4: ...
    @property
    def ltlt(self) -> BVector4: ...
    @property
    def ltlq(self) -> BVector4: ...
    @property
    def ltlo(self) -> BVector4: ...
    @property
    def ltll(self) -> BVector4: ...
    @property
    def lqss(self) -> BVector4: ...
    @property
    def lqst(self) -> BVector4: ...
    @property
    def lqsq(self) -> BVector4: ...
    @property
    def lqso(self) -> BVector4: ...
    @property
    def lqsl(self) -> BVector4: ...
    @property
    def lqts(self) -> BVector4: ...
    @property
    def lqtt(self) -> BVector4: ...
    @property
    def lqtq(self) -> BVector4: ...
    @property
    def lqto(self) -> BVector4: ...
    @property
    def lqtl(self) -> BVector4: ...
    @property
    def lqqs(self) -> BVector4: ...
    @property
    def lqqt(self) -> BVector4: ...
    @property
    def lqqq(self) -> BVector4: ...
    @property
    def lqqo(self) -> BVector4: ...
    @property
    def lqql(self) -> BVector4: ...
    @property
    def lqos(self) -> BVector4: ...
    @property
    def lqot(self) -> BVector4: ...
    @property
    def lqoq(self) -> BVector4: ...
    @property
    def lqoo(self) -> BVector4: ...
    @property
    def lqol(self) -> BVector4: ...
    @property
    def lqls(self) -> BVector4: ...
    @property
    def lqlt(self) -> BVector4: ...
    @property
    def lqlq(self) -> BVector4: ...
    @property
    def lqlo(self) -> BVector4: ...
    @property
    def lqll(self) -> BVector4: ...
    @property
    def loss(self) -> BVector4: ...
    @property
    def lost(self) -> BVector4: ...
    @property
    def losq(self) -> BVector4: ...
    @property
    def loso(self) -> BVector4: ...
    @property
    def losl(self) -> BVector4: ...
    @property
    def lots(self) -> BVector4: ...
    @property
    def lott(self) -> BVector4: ...
    @property
    def lotq(self) -> BVector4: ...
    @property
    def loto(self) -> BVector4: ...
    @property
    def lotl(self) -> BVector4: ...
    @property
    def loqs(self) -> BVector4: ...
    @property
    def loqt(self) -> BVector4: ...
    @property
    def loqq(self) -> BVector4: ...
    @property
    def loqo(self) -> BVector4: ...
    @property
    def loql(self) -> BVector4: ...
    @property
    def loos(self) -> BVector4: ...
    @property
    def loot(self) -> BVector4: ...
    @property
    def looq(self) -> BVector4: ...
    @property
    def lols(self) -> BVector4: ...
    @property
    def lolt(self) -> BVector4: ...
    @property
    def lolq(self) -> BVector4: ...
    @property
    def llss(self) -> BVector4: ...
    @property
    def llst(self) -> BVector4: ...
    @property
    def llsq(self) -> BVector4: ...
    @property
    def llso(self) -> BVector4: ...
    @property
    def llsl(self) -> BVector4: ...
    @property
    def llts(self) -> BVector4: ...
    @property
    def lltt(self) -> BVector4: ...
    @property
    def lltq(self) -> BVector4: ...
    @property
    def llto(self) -> BVector4: ...
    @property
    def lltl(self) -> BVector4: ...
    @property
    def llqs(self) -> BVector4: ...
    @property
    def llqt(self) -> BVector4: ...
    @property
    def llqq(self) -> BVector4: ...
    @property
    def llqo(self) -> BVector4: ...
    @property
    def llql(self) -> BVector4: ...
    @property
    def llos(self) -> BVector4: ...
    @property
    def llot(self) -> BVector4: ...
    @property
    def lloq(self) -> BVector4: ...
    @property
    def llls(self) -> BVector4: ...
    @property
    def lllt(self) -> BVector4: ...
    @property
    def lllq(self) -> BVector4: ...
    @property
    def u(self) -> _bool: ...
    @property
    def v(self) -> _bool: ...
    @property
    def uu(self) -> BVector2: ...
    @property
    def uv(self) -> BVector2: ...
    @property
    def uo(self) -> BVector2: ...
    @property
    def ul(self) -> BVector2: ...
    @property
    def vu(self) -> BVector2: ...
    @property
    def vv(self) -> BVector2: ...
    @property
    def vo(self) -> BVector2: ...
    @property
    def vl(self) -> BVector2: ...
    @property
    def ou(self) -> BVector2: ...
    @property
    def ov(self) -> BVector2: ...
    @property
    def lu(self) -> BVector2: ...
    @property
    def lv(self) -> BVector2: ...
    @property
    def uuu(self) -> BVector3: ...
    @property
    def uuv(self) -> BVector3: ...
    @property
    def uuo(self) -> BVector3: ...
    @property
    def uul(self) -> BVector3: ...
    @property
    def uvu(self) -> BVector3: ...
    @property
    def uvv(self) -> BVector3: ...
    @property
    def uvo(self) -> BVector3: ...
    @property
    def uvl(self) -> BVector3: ...
    @property
    def uou(self) -> BVector3: ...
    @property
    def uov(self) -> BVector3: ...
    @property
    def uoo(self) -> BVector3: ...
    @property
    def uol(self) -> BVector3: ...
    @property
    def ulu(self) -> BVector3: ...
    @property
    def ulv(self) -> BVector3: ...
    @property
    def ulo(self) -> BVector3: ...
    @property
    def ull(self) -> BVector3: ...
    @property
    def vuu(self) -> BVector3: ...
    @property
    def vuv(self) -> BVector3: ...
    @property
    def vuo(self) -> BVector3: ...
    @property
    def vul(self) -> BVector3: ...
    @property
    def vvu(self) -> BVector3: ...
    @property
    def vvv(self) -> BVector3: ...
    @property
    def vvo(self) -> BVector3: ...
    @property
    def vvl(self) -> BVector3: ...
    @property
    def vou(self) -> BVector3: ...
    @property
    def vov(self) -> BVector3: ...
    @property
    def voo(self) -> BVector3: ...
    @property
    def vol(self) -> BVector3: ...
    @property
    def vlu(self) -> BVector3: ...
    @property
    def vlv(self) -> BVector3: ...
    @property
    def vlo(self) -> BVector3: ...
    @property
    def vll(self) -> BVector3: ...
    @property
    def ouu(self) -> BVector3: ...
    @property
    def ouv(self) -> BVector3: ...
    @property
    def ouo(self) -> BVector3: ...
    @property
    def oul(self) -> BVector3: ...
    @property
    def ovu(self) -> BVector3: ...
    @property
    def ovv(self) -> BVector3: ...
    @property
    def ovo(self) -> BVector3: ...
    @property
    def ovl(self) -> BVector3: ...
    @property
    def oou(self) -> BVector3: ...
    @property
    def oov(self) -> BVector3: ...
    @property
    def olu(self) -> BVector3: ...
    @property
    def olv(self) -> BVector3: ...
    @property
    def luu(self) -> BVector3: ...
    @property
    def luv(self) -> BVector3: ...
    @property
    def luo(self) -> BVector3: ...
    @property
    def lul(self) -> BVector3: ...
    @property
    def lvu(self) -> BVector3: ...
    @property
    def lvv(self) -> BVector3: ...
    @property
    def lvo(self) -> BVector3: ...
    @property
    def lvl(self) -> BVector3: ...
    @property
    def lou(self) -> BVector3: ...
    @property
    def lov(self) -> BVector3: ...
    @property
    def llu(self) -> BVector3: ...
    @property
    def llv(self) -> BVector3: ...
    @property
    def uuuu(self) -> BVector4: ...
    @property
    def uuuv(self) -> BVector4: ...
    @property
    def uuuo(self) -> BVector4: ...
    @property
    def uuul(self) -> BVector4: ...
    @property
    def uuvu(self) -> BVector4: ...
    @property
    def uuvv(self) -> BVector4: ...
    @property
    def uuvo(self) -> BVector4: ...
    @property
    def uuvl(self) -> BVector4: ...
    @property
    def uuou(self) -> BVector4: ...
    @property
    def uuov(self) -> BVector4: ...
    @property
    def uuoo(self) -> BVector4: ...
    @property
    def uuol(self) -> BVector4: ...
    @property
    def uulu(self) -> BVector4: ...
    @property
    def uulv(self) -> BVector4: ...
    @property
    def uulo(self) -> BVector4: ...
    @property
    def uull(self) -> BVector4: ...
    @property
    def uvuu(self) -> BVector4: ...
    @property
    def uvuv(self) -> BVector4: ...
    @property
    def uvuo(self) -> BVector4: ...
    @property
    def uvul(self) -> BVector4: ...
    @property
    def uvvu(self) -> BVector4: ...
    @property
    def uvvv(self) -> BVector4: ...
    @property
    def uvvo(self) -> BVector4: ...
    @property
    def uvvl(self) -> BVector4: ...
    @property
    def uvou(self) -> BVector4: ...
    @property
    def uvov(self) -> BVector4: ...
    @property
    def uvoo(self) -> BVector4: ...
    @property
    def uvol(self) -> BVector4: ...
    @property
    def uvlu(self) -> BVector4: ...
    @property
    def uvlv(self) -> BVector4: ...
    @property
    def uvlo(self) -> BVector4: ...
    @property
    def uvll(self) -> BVector4: ...
    @property
    def uouu(self) -> BVector4: ...
    @property
    def uouv(self) -> BVector4: ...
    @property
    def uouo(self) -> BVector4: ...
    @property
    def uoul(self) -> BVector4: ...
    @property
    def uovu(self) -> BVector4: ...
    @property
    def uovv(self) -> BVector4: ...
    @property
    def uovo(self) -> BVector4: ...
    @property
    def uovl(self) -> BVector4: ...
    @property
    def uoou(self) -> BVector4: ...
    @property
    def uoov(self) -> BVector4: ...
    @property
    def uooo(self) -> BVector4: ...
    @property
    def uool(self) -> BVector4: ...
    @property
    def uolu(self) -> BVector4: ...
    @property
    def uolv(self) -> BVector4: ...
    @property
    def uolo(self) -> BVector4: ...
    @property
    def uoll(self) -> BVector4: ...
    @property
    def uluu(self) -> BVector4: ...
    @property
    def uluv(self) -> BVector4: ...
    @property
    def uluo(self) -> BVector4: ...
    @property
    def ulul(self) -> BVector4: ...
    @property
    def ulvu(self) -> BVector4: ...
    @property
    def ulvv(self) -> BVector4: ...
    @property
    def ulvo(self) -> BVector4: ...
    @property
    def ulvl(self) -> BVector4: ...
    @property
    def ulou(self) -> BVector4: ...
    @property
    def ulov(self) -> BVector4: ...
    @property
    def uloo(self) -> BVector4: ...
    @property
    def ulol(self) -> BVector4: ...
    @property
    def ullu(self) -> BVector4: ...
    @property
    def ullv(self) -> BVector4: ...
    @property
    def ullo(self) -> BVector4: ...
    @property
    def ulll(self) -> BVector4: ...
    @property
    def vuuu(self) -> BVector4: ...
    @property
    def vuuv(self) -> BVector4: ...
    @property
    def vuuo(self) -> BVector4: ...
    @property
    def vuul(self) -> BVector4: ...
    @property
    def vuvu(self) -> BVector4: ...
    @property
    def vuvv(self) -> BVector4: ...
    @property
    def vuvo(self) -> BVector4: ...
    @property
    def vuvl(self) -> BVector4: ...
    @property
    def vuou(self) -> BVector4: ...
    @property
    def vuov(self) -> BVector4: ...
    @property
    def vuoo(self) -> BVector4: ...
    @property
    def vuol(self) -> BVector4: ...
    @property
    def vulu(self) -> BVector4: ...
    @property
    def vulv(self) -> BVector4: ...
    @property
    def vulo(self) -> BVector4: ...
    @property
    def vull(self) -> BVector4: ...
    @property
    def vvuu(self) -> BVector4: ...
    @property
    def vvuv(self) -> BVector4: ...
    @property
    def vvuo(self) -> BVector4: ...
    @property
    def vvul(self) -> BVector4: ...
    @property
    def vvvu(self) -> BVector4: ...
    @property
    def vvvv(self) -> BVector4: ...
    @property
    def vvvo(self) -> BVector4: ...
    @property
    def vvvl(self) -> BVector4: ...
    @property
    def vvou(self) -> BVector4: ...
    @property
    def vvov(self) -> BVector4: ...
    @property
    def vvoo(self) -> BVector4: ...
    @property
    def vvol(self) -> BVector4: ...
    @property
    def vvlu(self) -> BVector4: ...
    @property
    def vvlv(self) -> BVector4: ...
    @property
    def vvlo(self) -> BVector4: ...
    @property
    def vvll(self) -> BVector4: ...
    @property
    def vouu(self) -> BVector4: ...
    @property
    def vouv(self) -> BVector4: ...
    @property
    def vouo(self) -> BVector4: ...
    @property
    def voul(self) -> BVector4: ...
    @property
    def vovu(self) -> BVector4: ...
    @property
    def vovv(self) -> BVector4: ...
    @property
    def vovo(self) -> BVector4: ...
    @property
    def vovl(self) -> BVector4: ...
    @property
    def voou(self) -> BVector4: ...
    @property
    def voov(self) -> BVector4: ...
    @property
    def vooo(self) -> BVector4: ...
    @property
    def vool(self) -> BVector4: ...
    @property
    def volu(self) -> BVector4: ...
    @property
    def volv(self) -> BVector4: ...
    @property
    def volo(self) -> BVector4: ...
    @property
    def voll(self) -> BVector4: ...
    @property
    def vluu(self) -> BVector4: ...
    @property
    def vluv(self) -> BVector4: ...
    @property
    def vluo(self) -> BVector4: ...
    @property
    def vlul(self) -> BVector4: ...
    @property
    def vlvu(self) -> BVector4: ...
    @property
    def vlvv(self) -> BVector4: ...
    @property
    def vlvo(self) -> BVector4: ...
    @property
    def vlvl(self) -> BVector4: ...
    @property
    def vlou(self) -> BVector4: ...
    @property
    def vlov(self) -> BVector4: ...
    @property
    def vloo(self) -> BVector4: ...
    @property
    def vlol(self) -> BVector4: ...
    @property
    def vllu(self) -> BVector4: ...
    @property
    def vllv(self) -> BVector4: ...
    @property
    def vllo(self) -> BVector4: ...
    @property
    def vlll(self) -> BVector4: ...
    @property
    def ouuu(self) -> BVector4: ...
    @property
    def ouuv(self) -> BVector4: ...
    @property
    def ouuo(self) -> BVector4: ...
    @property
    def ouul(self) -> BVector4: ...
    @property
    def ouvu(self) -> BVector4: ...
    @property
    def ouvv(self) -> BVector4: ...
    @property
    def ouvo(self) -> BVector4: ...
    @property
    def ouvl(self) -> BVector4: ...
    @property
    def ouou(self) -> BVector4: ...
    @property
    def ouov(self) -> BVector4: ...
    @property
    def ouoo(self) -> BVector4: ...
    @property
    def ouol(self) -> BVector4: ...
    @property
    def oulu(self) -> BVector4: ...
    @property
    def oulv(self) -> BVector4: ...
    @property
    def oulo(self) -> BVector4: ...
    @property
    def oull(self) -> BVector4: ...
    @property
    def ovuu(self) -> BVector4: ...
    @property
    def ovuv(self) -> BVector4: ...
    @property
    def ovuo(self) -> BVector4: ...
    @property
    def ovul(self) -> BVector4: ...
    @property
    def ovvu(self) -> BVector4: ...
    @property
    def ovvv(self) -> BVector4: ...
    @property
    def ovvo(self) -> BVector4: ...
    @property
    def ovvl(self) -> BVector4: ...
    @property
    def ovou(self) -> BVector4: ...
    @property
    def ovov(self) -> BVector4: ...
    @property
    def ovoo(self) -> BVector4: ...
    @property
    def ovol(self) -> BVector4: ...
    @property
    def ovlu(self) -> BVector4: ...
    @property
    def ovlv(self) -> BVector4: ...
    @property
    def ovlo(self) -> BVector4: ...
    @property
    def ovll(self) -> BVector4: ...
    @property
    def oouu(self) -> BVector4: ...
    @property
    def oouv(self) -> BVector4: ...
    @property
    def oouo(self) -> BVector4: ...
    @property
    def ooul(self) -> BVector4: ...
    @property
    def oovu(self) -> BVector4: ...
    @property
    def oovv(self) -> BVector4: ...
    @property
    def oovo(self) -> BVector4: ...
    @property
    def oovl(self) -> BVector4: ...
    @property
    def ooou(self) -> BVector4: ...
    @property
    def ooov(self) -> BVector4: ...
    @property
    def oolu(self) -> BVector4: ...
    @property
    def oolv(self) -> BVector4: ...
    @property
    def oluu(self) -> BVector4: ...
    @property
    def oluv(self) -> BVector4: ...
    @property
    def oluo(self) -> BVector4: ...
    @property
    def olul(self) -> BVector4: ...
    @property
    def olvu(self) -> BVector4: ...
    @property
    def olvv(self) -> BVector4: ...
    @property
    def olvo(self) -> BVector4: ...
    @property
    def olvl(self) -> BVector4: ...
    @property
    def olou(self) -> BVector4: ...
    @property
    def olov(self) -> BVector4: ...
    @property
    def ollu(self) -> BVector4: ...
    @property
    def ollv(self) -> BVector4: ...
    @property
    def luuu(self) -> BVector4: ...
    @property
    def luuv(self) -> BVector4: ...
    @property
    def luuo(self) -> BVector4: ...
    @property
    def luul(self) -> BVector4: ...
    @property
    def luvu(self) -> BVector4: ...
    @property
    def luvv(self) -> BVector4: ...
    @property
    def luvo(self) -> BVector4: ...
    @property
    def luvl(self) -> BVector4: ...
    @property
    def luou(self) -> BVector4: ...
    @property
    def luov(self) -> BVector4: ...
    @property
    def luoo(self) -> BVector4: ...
    @property
    def luol(self) -> BVector4: ...
    @property
    def lulu(self) -> BVector4: ...
    @property
    def lulv(self) -> BVector4: ...
    @property
    def lulo(self) -> BVector4: ...
    @property
    def lull(self) -> BVector4: ...
    @property
    def lvuu(self) -> BVector4: ...
    @property
    def lvuv(self) -> BVector4: ...
    @property
    def lvuo(self) -> BVector4: ...
    @property
    def lvul(self) -> BVector4: ...
    @property
    def lvvu(self) -> BVector4: ...
    @property
    def lvvv(self) -> BVector4: ...
    @property
    def lvvo(self) -> BVector4: ...
    @property
    def lvvl(self) -> BVector4: ...
    @property
    def lvou(self) -> BVector4: ...
    @property
    def lvov(self) -> BVector4: ...
    @property
    def lvoo(self) -> BVector4: ...
    @property
    def lvol(self) -> BVector4: ...
    @property
    def lvlu(self) -> BVector4: ...
    @property
    def lvlv(self) -> BVector4: ...
    @property
    def lvlo(self) -> BVector4: ...
    @property
    def lvll(self) -> BVector4: ...
    @property
    def louu(self) -> BVector4: ...
    @property
    def louv(self) -> BVector4: ...
    @property
    def louo(self) -> BVector4: ...
    @property
    def loul(self) -> BVector4: ...
    @property
    def lovu(self) -> BVector4: ...
    @property
    def lovv(self) -> BVector4: ...
    @property
    def lovo(self) -> BVector4: ...
    @property
    def lovl(self) -> BVector4: ...
    @property
    def loou(self) -> BVector4: ...
    @property
    def loov(self) -> BVector4: ...
    @property
    def lolu(self) -> BVector4: ...
    @property
    def lolv(self) -> BVector4: ...
    @property
    def lluu(self) -> BVector4: ...
    @property
    def lluv(self) -> BVector4: ...
    @property
    def lluo(self) -> BVector4: ...
    @property
    def llul(self) -> BVector4: ...
    @property
    def llvu(self) -> BVector4: ...
    @property
    def llvv(self) -> BVector4: ...
    @property
    def llvo(self) -> BVector4: ...
    @property
    def llvl(self) -> BVector4: ...
    @property
    def llou(self) -> BVector4: ...
    @property
    def llov(self) -> BVector4: ...
    @property
    def lllu(self) -> BVector4: ...
    @property
    def lllv(self) -> BVector4: ...
    @property
    def oo(self) -> BVector2: ...
    @property
    def ol(self) -> BVector2: ...
    @property
    def lo(self) -> BVector2: ...
    @property
    def ll(self) -> BVector2: ...
    @property
    def ooo(self) -> BVector3: ...
    @property
    def ool(self) -> BVector3: ...
    @property
    def olo(self) -> BVector3: ...
    @property
    def oll(self) -> BVector3: ...
    @property
    def loo(self) -> BVector3: ...
    @property
    def lol(self) -> BVector3: ...
    @property
    def llo(self) -> BVector3: ...
    @property
    def lll(self) -> BVector3: ...
    @property
    def oooo(self) -> BVector4: ...
    @property
    def oool(self) -> BVector4: ...
    @property
    def oolo(self) -> BVector4: ...
    @property
    def ooll(self) -> BVector4: ...
    @property
    def oloo(self) -> BVector4: ...
    @property
    def olol(self) -> BVector4: ...
    @property
    def ollo(self) -> BVector4: ...
    @property
    def olll(self) -> BVector4: ...
    @property
    def looo(self) -> BVector4: ...
    @property
    def lool(self) -> BVector4: ...
    @property
    def lolo(self) -> BVector4: ...
    @property
    def loll(self) -> BVector4: ...
    @property
    def lloo(self) -> BVector4: ...
    @property
    def llol(self) -> BVector4: ...
    @property
    def lllo(self) -> BVector4: ...
    @property
    def llll(self) -> BVector4: ...
    def min(self, n: Number, /) -> BVector3: ...
    def max(self, n: Number, /) -> BVector3: ...
    def clamp(self, min: Number, max: Number, /) -> BVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[_bool, _bool]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[BVector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector3: ...

@final
class BVector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: BVector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[BVector3]: ...
    @overload
    def __getitem__(self, index: int) -> BVector3: ...
    @overload
    def __getitem__(self, index: slice) -> BVector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[BVector3]: ...

@final
class DVector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DVector3) -> DVector3: ...
    @overload
    def __add__(self, other: Number) -> DVector3: ...
    @overload
    def __radd__(self, other: DVector3) -> DVector3: ...
    @overload
    def __radd__(self, other: Number) -> DVector3: ...
    @overload
    def __sub__(self, other: DVector3) -> DVector3: ...
    @overload
    def __sub__(self, other: Number) -> DVector3: ...
    @overload
    def __rsub__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rsub__(self, other: Number) -> DVector3: ...
    @overload
    def __mul__(self, other: DVector3) -> DVector3: ...
    @overload
    def __mul__(self, other: Number) -> DVector3: ...
    @overload
    def __rmul__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rmul__(self, other: Number) -> DVector3: ...
    def __matmul__(self, other: DVector3) -> float: ...
    def __rmatmul__(self, other: DVector3) -> float: ...
    @overload
    def __mod__(self, other: DVector3) -> DVector3: ...
    @overload
    def __mod__(self, other: Number) -> DVector3: ...
    @overload
    def __rmod__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rmod__(self, other: Number) -> DVector3: ...
    @overload
    def __pow__(self, other: DVector3) -> DVector3: ...
    @overload
    def __pow__(self, other: Number) -> DVector3: ...
    @overload
    def __rpow__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rpow__(self, other: Number) -> DVector3: ...
    @overload
    def __truediv__(self, other: DVector3) -> DVector3: ...
    @overload
    def __truediv__(self, other: Number) -> DVector3: ...
    @overload
    def __rtruediv__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> DVector3: ...
    def __neg__(self) -> DVector3: ...
    def __abs__(self) -> DVector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float: ...
    @property
    def xx(self) -> DVector2: ...
    @property
    def xy(self) -> DVector2: ...
    @property
    def xz(self) -> DVector2: ...
    @property
    def xo(self) -> DVector2: ...
    @property
    def xl(self) -> DVector2: ...
    @property
    def yx(self) -> DVector2: ...
    @property
    def yy(self) -> DVector2: ...
    @property
    def yz(self) -> DVector2: ...
    @property
    def yo(self) -> DVector2: ...
    @property
    def yl(self) -> DVector2: ...
    @property
    def zx(self) -> DVector2: ...
    @property
    def zy(self) -> DVector2: ...
    @property
    def zz(self) -> DVector2: ...
    @property
    def zo(self) -> DVector2: ...
    @property
    def zl(self) -> DVector2: ...
    @property
    def ox(self) -> DVector2: ...
    @property
    def oy(self) -> DVector2: ...
    @property
    def oz(self) -> DVector2: ...
    @property
    def lx(self) -> DVector2: ...
    @property
    def ly(self) -> DVector2: ...
    @property
    def lz(self) -> DVector2: ...
    @property
    def xxx(self) -> DVector3: ...
    @property
    def xxy(self) -> DVector3: ...
    @property
    def xxz(self) -> DVector3: ...
    @property
    def xxo(self) -> DVector3: ...
    @property
    def xxl(self) -> DVector3: ...
    @property
    def xyx(self) -> DVector3: ...
    @property
    def xyy(self) -> DVector3: ...
    @property
    def xyz(self) -> DVector3: ...
    @property
    def xyo(self) -> DVector3: ...
    @property
    def xyl(self) -> DVector3: ...
    @property
    def xzx(self) -> DVector3: ...
    @property
    def xzy(self) -> DVector3: ...
    @property
    def xzz(self) -> DVector3: ...
    @property
    def xzo(self) -> DVector3: ...
    @property
    def xzl(self) -> DVector3: ...
    @property
    def xox(self) -> DVector3: ...
    @property
    def xoy(self) -> DVector3: ...
    @property
    def xoz(self) -> DVector3: ...
    @property
    def xoo(self) -> DVector3: ...
    @property
    def xol(self) -> DVector3: ...
    @property
    def xlx(self) -> DVector3: ...
    @property
    def xly(self) -> DVector3: ...
    @property
    def xlz(self) -> DVector3: ...
    @property
    def xlo(self) -> DVector3: ...
    @property
    def xll(self) -> DVector3: ...
    @property
    def yxx(self) -> DVector3: ...
    @property
    def yxy(self) -> DVector3: ...
    @property
    def yxz(self) -> DVector3: ...
    @property
    def yxo(self) -> DVector3: ...
    @property
    def yxl(self) -> DVector3: ...
    @property
    def yyx(self) -> DVector3: ...
    @property
    def yyy(self) -> DVector3: ...
    @property
    def yyz(self) -> DVector3: ...
    @property
    def yyo(self) -> DVector3: ...
    @property
    def yyl(self) -> DVector3: ...
    @property
    def yzx(self) -> DVector3: ...
    @property
    def yzy(self) -> DVector3: ...
    @property
    def yzz(self) -> DVector3: ...
    @property
    def yzo(self) -> DVector3: ...
    @property
    def yzl(self) -> DVector3: ...
    @property
    def yox(self) -> DVector3: ...
    @property
    def yoy(self) -> DVector3: ...
    @property
    def yoz(self) -> DVector3: ...
    @property
    def yoo(self) -> DVector3: ...
    @property
    def yol(self) -> DVector3: ...
    @property
    def ylx(self) -> DVector3: ...
    @property
    def yly(self) -> DVector3: ...
    @property
    def ylz(self) -> DVector3: ...
    @property
    def ylo(self) -> DVector3: ...
    @property
    def yll(self) -> DVector3: ...
    @property
    def zxx(self) -> DVector3: ...
    @property
    def zxy(self) -> DVector3: ...
    @property
    def zxz(self) -> DVector3: ...
    @property
    def zxo(self) -> DVector3: ...
    @property
    def zxl(self) -> DVector3: ...
    @property
    def zyx(self) -> DVector3: ...
    @property
    def zyy(self) -> DVector3: ...
    @property
    def zyz(self) -> DVector3: ...
    @property
    def zyo(self) -> DVector3: ...
    @property
    def zyl(self) -> DVector3: ...
    @property
    def zzx(self) -> DVector3: ...
    @property
    def zzy(self) -> DVector3: ...
    @property
    def zzz(self) -> DVector3: ...
    @property
    def zzo(self) -> DVector3: ...
    @property
    def zzl(self) -> DVector3: ...
    @property
    def zox(self) -> DVector3: ...
    @property
    def zoy(self) -> DVector3: ...
    @property
    def zoz(self) -> DVector3: ...
    @property
    def zoo(self) -> DVector3: ...
    @property
    def zol(self) -> DVector3: ...
    @property
    def zlx(self) -> DVector3: ...
    @property
    def zly(self) -> DVector3: ...
    @property
    def zlz(self) -> DVector3: ...
    @property
    def zlo(self) -> DVector3: ...
    @property
    def zll(self) -> DVector3: ...
    @property
    def oxx(self) -> DVector3: ...
    @property
    def oxy(self) -> DVector3: ...
    @property
    def oxz(self) -> DVector3: ...
    @property
    def oxo(self) -> DVector3: ...
    @property
    def oxl(self) -> DVector3: ...
    @property
    def oyx(self) -> DVector3: ...
    @property
    def oyy(self) -> DVector3: ...
    @property
    def oyz(self) -> DVector3: ...
    @property
    def oyo(self) -> DVector3: ...
    @property
    def oyl(self) -> DVector3: ...
    @property
    def ozx(self) -> DVector3: ...
    @property
    def ozy(self) -> DVector3: ...
    @property
    def ozz(self) -> DVector3: ...
    @property
    def ozo(self) -> DVector3: ...
    @property
    def ozl(self) -> DVector3: ...
    @property
    def oox(self) -> DVector3: ...
    @property
    def ooy(self) -> DVector3: ...
    @property
    def ooz(self) -> DVector3: ...
    @property
    def olx(self) -> DVector3: ...
    @property
    def oly(self) -> DVector3: ...
    @property
    def olz(self) -> DVector3: ...
    @property
    def lxx(self) -> DVector3: ...
    @property
    def lxy(self) -> DVector3: ...
    @property
    def lxz(self) -> DVector3: ...
    @property
    def lxo(self) -> DVector3: ...
    @property
    def lxl(self) -> DVector3: ...
    @property
    def lyx(self) -> DVector3: ...
    @property
    def lyy(self) -> DVector3: ...
    @property
    def lyz(self) -> DVector3: ...
    @property
    def lyo(self) -> DVector3: ...
    @property
    def lyl(self) -> DVector3: ...
    @property
    def lzx(self) -> DVector3: ...
    @property
    def lzy(self) -> DVector3: ...
    @property
    def lzz(self) -> DVector3: ...
    @property
    def lzo(self) -> DVector3: ...
    @property
    def lzl(self) -> DVector3: ...
    @property
    def lox(self) -> DVector3: ...
    @property
    def loy(self) -> DVector3: ...
    @property
    def loz(self) -> DVector3: ...
    @property
    def llx(self) -> DVector3: ...
    @property
    def lly(self) -> DVector3: ...
    @property
    def llz(self) -> DVector3: ...
    @property
    def xxxx(self) -> DVector4: ...
    @property
    def xxxy(self) -> DVector4: ...
    @property
    def xxxz(self) -> DVector4: ...
    @property
    def xxxo(self) -> DVector4: ...
    @property
    def xxxl(self) -> DVector4: ...
    @property
    def xxyx(self) -> DVector4: ...
    @property
    def xxyy(self) -> DVector4: ...
    @property
    def xxyz(self) -> DVector4: ...
    @property
    def xxyo(self) -> DVector4: ...
    @property
    def xxyl(self) -> DVector4: ...
    @property
    def xxzx(self) -> DVector4: ...
    @property
    def xxzy(self) -> DVector4: ...
    @property
    def xxzz(self) -> DVector4: ...
    @property
    def xxzo(self) -> DVector4: ...
    @property
    def xxzl(self) -> DVector4: ...
    @property
    def xxox(self) -> DVector4: ...
    @property
    def xxoy(self) -> DVector4: ...
    @property
    def xxoz(self) -> DVector4: ...
    @property
    def xxoo(self) -> DVector4: ...
    @property
    def xxol(self) -> DVector4: ...
    @property
    def xxlx(self) -> DVector4: ...
    @property
    def xxly(self) -> DVector4: ...
    @property
    def xxlz(self) -> DVector4: ...
    @property
    def xxlo(self) -> DVector4: ...
    @property
    def xxll(self) -> DVector4: ...
    @property
    def xyxx(self) -> DVector4: ...
    @property
    def xyxy(self) -> DVector4: ...
    @property
    def xyxz(self) -> DVector4: ...
    @property
    def xyxo(self) -> DVector4: ...
    @property
    def xyxl(self) -> DVector4: ...
    @property
    def xyyx(self) -> DVector4: ...
    @property
    def xyyy(self) -> DVector4: ...
    @property
    def xyyz(self) -> DVector4: ...
    @property
    def xyyo(self) -> DVector4: ...
    @property
    def xyyl(self) -> DVector4: ...
    @property
    def xyzx(self) -> DVector4: ...
    @property
    def xyzy(self) -> DVector4: ...
    @property
    def xyzz(self) -> DVector4: ...
    @property
    def xyzo(self) -> DVector4: ...
    @property
    def xyzl(self) -> DVector4: ...
    @property
    def xyox(self) -> DVector4: ...
    @property
    def xyoy(self) -> DVector4: ...
    @property
    def xyoz(self) -> DVector4: ...
    @property
    def xyoo(self) -> DVector4: ...
    @property
    def xyol(self) -> DVector4: ...
    @property
    def xylx(self) -> DVector4: ...
    @property
    def xyly(self) -> DVector4: ...
    @property
    def xylz(self) -> DVector4: ...
    @property
    def xylo(self) -> DVector4: ...
    @property
    def xyll(self) -> DVector4: ...
    @property
    def xzxx(self) -> DVector4: ...
    @property
    def xzxy(self) -> DVector4: ...
    @property
    def xzxz(self) -> DVector4: ...
    @property
    def xzxo(self) -> DVector4: ...
    @property
    def xzxl(self) -> DVector4: ...
    @property
    def xzyx(self) -> DVector4: ...
    @property
    def xzyy(self) -> DVector4: ...
    @property
    def xzyz(self) -> DVector4: ...
    @property
    def xzyo(self) -> DVector4: ...
    @property
    def xzyl(self) -> DVector4: ...
    @property
    def xzzx(self) -> DVector4: ...
    @property
    def xzzy(self) -> DVector4: ...
    @property
    def xzzz(self) -> DVector4: ...
    @property
    def xzzo(self) -> DVector4: ...
    @property
    def xzzl(self) -> DVector4: ...
    @property
    def xzox(self) -> DVector4: ...
    @property
    def xzoy(self) -> DVector4: ...
    @property
    def xzoz(self) -> DVector4: ...
    @property
    def xzoo(self) -> DVector4: ...
    @property
    def xzol(self) -> DVector4: ...
    @property
    def xzlx(self) -> DVector4: ...
    @property
    def xzly(self) -> DVector4: ...
    @property
    def xzlz(self) -> DVector4: ...
    @property
    def xzlo(self) -> DVector4: ...
    @property
    def xzll(self) -> DVector4: ...
    @property
    def xoxx(self) -> DVector4: ...
    @property
    def xoxy(self) -> DVector4: ...
    @property
    def xoxz(self) -> DVector4: ...
    @property
    def xoxo(self) -> DVector4: ...
    @property
    def xoxl(self) -> DVector4: ...
    @property
    def xoyx(self) -> DVector4: ...
    @property
    def xoyy(self) -> DVector4: ...
    @property
    def xoyz(self) -> DVector4: ...
    @property
    def xoyo(self) -> DVector4: ...
    @property
    def xoyl(self) -> DVector4: ...
    @property
    def xozx(self) -> DVector4: ...
    @property
    def xozy(self) -> DVector4: ...
    @property
    def xozz(self) -> DVector4: ...
    @property
    def xozo(self) -> DVector4: ...
    @property
    def xozl(self) -> DVector4: ...
    @property
    def xoox(self) -> DVector4: ...
    @property
    def xooy(self) -> DVector4: ...
    @property
    def xooz(self) -> DVector4: ...
    @property
    def xooo(self) -> DVector4: ...
    @property
    def xool(self) -> DVector4: ...
    @property
    def xolx(self) -> DVector4: ...
    @property
    def xoly(self) -> DVector4: ...
    @property
    def xolz(self) -> DVector4: ...
    @property
    def xolo(self) -> DVector4: ...
    @property
    def xoll(self) -> DVector4: ...
    @property
    def xlxx(self) -> DVector4: ...
    @property
    def xlxy(self) -> DVector4: ...
    @property
    def xlxz(self) -> DVector4: ...
    @property
    def xlxo(self) -> DVector4: ...
    @property
    def xlxl(self) -> DVector4: ...
    @property
    def xlyx(self) -> DVector4: ...
    @property
    def xlyy(self) -> DVector4: ...
    @property
    def xlyz(self) -> DVector4: ...
    @property
    def xlyo(self) -> DVector4: ...
    @property
    def xlyl(self) -> DVector4: ...
    @property
    def xlzx(self) -> DVector4: ...
    @property
    def xlzy(self) -> DVector4: ...
    @property
    def xlzz(self) -> DVector4: ...
    @property
    def xlzo(self) -> DVector4: ...
    @property
    def xlzl(self) -> DVector4: ...
    @property
    def xlox(self) -> DVector4: ...
    @property
    def xloy(self) -> DVector4: ...
    @property
    def xloz(self) -> DVector4: ...
    @property
    def xloo(self) -> DVector4: ...
    @property
    def xlol(self) -> DVector4: ...
    @property
    def xllx(self) -> DVector4: ...
    @property
    def xlly(self) -> DVector4: ...
    @property
    def xllz(self) -> DVector4: ...
    @property
    def xllo(self) -> DVector4: ...
    @property
    def xlll(self) -> DVector4: ...
    @property
    def yxxx(self) -> DVector4: ...
    @property
    def yxxy(self) -> DVector4: ...
    @property
    def yxxz(self) -> DVector4: ...
    @property
    def yxxo(self) -> DVector4: ...
    @property
    def yxxl(self) -> DVector4: ...
    @property
    def yxyx(self) -> DVector4: ...
    @property
    def yxyy(self) -> DVector4: ...
    @property
    def yxyz(self) -> DVector4: ...
    @property
    def yxyo(self) -> DVector4: ...
    @property
    def yxyl(self) -> DVector4: ...
    @property
    def yxzx(self) -> DVector4: ...
    @property
    def yxzy(self) -> DVector4: ...
    @property
    def yxzz(self) -> DVector4: ...
    @property
    def yxzo(self) -> DVector4: ...
    @property
    def yxzl(self) -> DVector4: ...
    @property
    def yxox(self) -> DVector4: ...
    @property
    def yxoy(self) -> DVector4: ...
    @property
    def yxoz(self) -> DVector4: ...
    @property
    def yxoo(self) -> DVector4: ...
    @property
    def yxol(self) -> DVector4: ...
    @property
    def yxlx(self) -> DVector4: ...
    @property
    def yxly(self) -> DVector4: ...
    @property
    def yxlz(self) -> DVector4: ...
    @property
    def yxlo(self) -> DVector4: ...
    @property
    def yxll(self) -> DVector4: ...
    @property
    def yyxx(self) -> DVector4: ...
    @property
    def yyxy(self) -> DVector4: ...
    @property
    def yyxz(self) -> DVector4: ...
    @property
    def yyxo(self) -> DVector4: ...
    @property
    def yyxl(self) -> DVector4: ...
    @property
    def yyyx(self) -> DVector4: ...
    @property
    def yyyy(self) -> DVector4: ...
    @property
    def yyyz(self) -> DVector4: ...
    @property
    def yyyo(self) -> DVector4: ...
    @property
    def yyyl(self) -> DVector4: ...
    @property
    def yyzx(self) -> DVector4: ...
    @property
    def yyzy(self) -> DVector4: ...
    @property
    def yyzz(self) -> DVector4: ...
    @property
    def yyzo(self) -> DVector4: ...
    @property
    def yyzl(self) -> DVector4: ...
    @property
    def yyox(self) -> DVector4: ...
    @property
    def yyoy(self) -> DVector4: ...
    @property
    def yyoz(self) -> DVector4: ...
    @property
    def yyoo(self) -> DVector4: ...
    @property
    def yyol(self) -> DVector4: ...
    @property
    def yylx(self) -> DVector4: ...
    @property
    def yyly(self) -> DVector4: ...
    @property
    def yylz(self) -> DVector4: ...
    @property
    def yylo(self) -> DVector4: ...
    @property
    def yyll(self) -> DVector4: ...
    @property
    def yzxx(self) -> DVector4: ...
    @property
    def yzxy(self) -> DVector4: ...
    @property
    def yzxz(self) -> DVector4: ...
    @property
    def yzxo(self) -> DVector4: ...
    @property
    def yzxl(self) -> DVector4: ...
    @property
    def yzyx(self) -> DVector4: ...
    @property
    def yzyy(self) -> DVector4: ...
    @property
    def yzyz(self) -> DVector4: ...
    @property
    def yzyo(self) -> DVector4: ...
    @property
    def yzyl(self) -> DVector4: ...
    @property
    def yzzx(self) -> DVector4: ...
    @property
    def yzzy(self) -> DVector4: ...
    @property
    def yzzz(self) -> DVector4: ...
    @property
    def yzzo(self) -> DVector4: ...
    @property
    def yzzl(self) -> DVector4: ...
    @property
    def yzox(self) -> DVector4: ...
    @property
    def yzoy(self) -> DVector4: ...
    @property
    def yzoz(self) -> DVector4: ...
    @property
    def yzoo(self) -> DVector4: ...
    @property
    def yzol(self) -> DVector4: ...
    @property
    def yzlx(self) -> DVector4: ...
    @property
    def yzly(self) -> DVector4: ...
    @property
    def yzlz(self) -> DVector4: ...
    @property
    def yzlo(self) -> DVector4: ...
    @property
    def yzll(self) -> DVector4: ...
    @property
    def yoxx(self) -> DVector4: ...
    @property
    def yoxy(self) -> DVector4: ...
    @property
    def yoxz(self) -> DVector4: ...
    @property
    def yoxo(self) -> DVector4: ...
    @property
    def yoxl(self) -> DVector4: ...
    @property
    def yoyx(self) -> DVector4: ...
    @property
    def yoyy(self) -> DVector4: ...
    @property
    def yoyz(self) -> DVector4: ...
    @property
    def yoyo(self) -> DVector4: ...
    @property
    def yoyl(self) -> DVector4: ...
    @property
    def yozx(self) -> DVector4: ...
    @property
    def yozy(self) -> DVector4: ...
    @property
    def yozz(self) -> DVector4: ...
    @property
    def yozo(self) -> DVector4: ...
    @property
    def yozl(self) -> DVector4: ...
    @property
    def yoox(self) -> DVector4: ...
    @property
    def yooy(self) -> DVector4: ...
    @property
    def yooz(self) -> DVector4: ...
    @property
    def yooo(self) -> DVector4: ...
    @property
    def yool(self) -> DVector4: ...
    @property
    def yolx(self) -> DVector4: ...
    @property
    def yoly(self) -> DVector4: ...
    @property
    def yolz(self) -> DVector4: ...
    @property
    def yolo(self) -> DVector4: ...
    @property
    def yoll(self) -> DVector4: ...
    @property
    def ylxx(self) -> DVector4: ...
    @property
    def ylxy(self) -> DVector4: ...
    @property
    def ylxz(self) -> DVector4: ...
    @property
    def ylxo(self) -> DVector4: ...
    @property
    def ylxl(self) -> DVector4: ...
    @property
    def ylyx(self) -> DVector4: ...
    @property
    def ylyy(self) -> DVector4: ...
    @property
    def ylyz(self) -> DVector4: ...
    @property
    def ylyo(self) -> DVector4: ...
    @property
    def ylyl(self) -> DVector4: ...
    @property
    def ylzx(self) -> DVector4: ...
    @property
    def ylzy(self) -> DVector4: ...
    @property
    def ylzz(self) -> DVector4: ...
    @property
    def ylzo(self) -> DVector4: ...
    @property
    def ylzl(self) -> DVector4: ...
    @property
    def ylox(self) -> DVector4: ...
    @property
    def yloy(self) -> DVector4: ...
    @property
    def yloz(self) -> DVector4: ...
    @property
    def yloo(self) -> DVector4: ...
    @property
    def ylol(self) -> DVector4: ...
    @property
    def yllx(self) -> DVector4: ...
    @property
    def ylly(self) -> DVector4: ...
    @property
    def yllz(self) -> DVector4: ...
    @property
    def yllo(self) -> DVector4: ...
    @property
    def ylll(self) -> DVector4: ...
    @property
    def zxxx(self) -> DVector4: ...
    @property
    def zxxy(self) -> DVector4: ...
    @property
    def zxxz(self) -> DVector4: ...
    @property
    def zxxo(self) -> DVector4: ...
    @property
    def zxxl(self) -> DVector4: ...
    @property
    def zxyx(self) -> DVector4: ...
    @property
    def zxyy(self) -> DVector4: ...
    @property
    def zxyz(self) -> DVector4: ...
    @property
    def zxyo(self) -> DVector4: ...
    @property
    def zxyl(self) -> DVector4: ...
    @property
    def zxzx(self) -> DVector4: ...
    @property
    def zxzy(self) -> DVector4: ...
    @property
    def zxzz(self) -> DVector4: ...
    @property
    def zxzo(self) -> DVector4: ...
    @property
    def zxzl(self) -> DVector4: ...
    @property
    def zxox(self) -> DVector4: ...
    @property
    def zxoy(self) -> DVector4: ...
    @property
    def zxoz(self) -> DVector4: ...
    @property
    def zxoo(self) -> DVector4: ...
    @property
    def zxol(self) -> DVector4: ...
    @property
    def zxlx(self) -> DVector4: ...
    @property
    def zxly(self) -> DVector4: ...
    @property
    def zxlz(self) -> DVector4: ...
    @property
    def zxlo(self) -> DVector4: ...
    @property
    def zxll(self) -> DVector4: ...
    @property
    def zyxx(self) -> DVector4: ...
    @property
    def zyxy(self) -> DVector4: ...
    @property
    def zyxz(self) -> DVector4: ...
    @property
    def zyxo(self) -> DVector4: ...
    @property
    def zyxl(self) -> DVector4: ...
    @property
    def zyyx(self) -> DVector4: ...
    @property
    def zyyy(self) -> DVector4: ...
    @property
    def zyyz(self) -> DVector4: ...
    @property
    def zyyo(self) -> DVector4: ...
    @property
    def zyyl(self) -> DVector4: ...
    @property
    def zyzx(self) -> DVector4: ...
    @property
    def zyzy(self) -> DVector4: ...
    @property
    def zyzz(self) -> DVector4: ...
    @property
    def zyzo(self) -> DVector4: ...
    @property
    def zyzl(self) -> DVector4: ...
    @property
    def zyox(self) -> DVector4: ...
    @property
    def zyoy(self) -> DVector4: ...
    @property
    def zyoz(self) -> DVector4: ...
    @property
    def zyoo(self) -> DVector4: ...
    @property
    def zyol(self) -> DVector4: ...
    @property
    def zylx(self) -> DVector4: ...
    @property
    def zyly(self) -> DVector4: ...
    @property
    def zylz(self) -> DVector4: ...
    @property
    def zylo(self) -> DVector4: ...
    @property
    def zyll(self) -> DVector4: ...
    @property
    def zzxx(self) -> DVector4: ...
    @property
    def zzxy(self) -> DVector4: ...
    @property
    def zzxz(self) -> DVector4: ...
    @property
    def zzxo(self) -> DVector4: ...
    @property
    def zzxl(self) -> DVector4: ...
    @property
    def zzyx(self) -> DVector4: ...
    @property
    def zzyy(self) -> DVector4: ...
    @property
    def zzyz(self) -> DVector4: ...
    @property
    def zzyo(self) -> DVector4: ...
    @property
    def zzyl(self) -> DVector4: ...
    @property
    def zzzx(self) -> DVector4: ...
    @property
    def zzzy(self) -> DVector4: ...
    @property
    def zzzz(self) -> DVector4: ...
    @property
    def zzzo(self) -> DVector4: ...
    @property
    def zzzl(self) -> DVector4: ...
    @property
    def zzox(self) -> DVector4: ...
    @property
    def zzoy(self) -> DVector4: ...
    @property
    def zzoz(self) -> DVector4: ...
    @property
    def zzoo(self) -> DVector4: ...
    @property
    def zzol(self) -> DVector4: ...
    @property
    def zzlx(self) -> DVector4: ...
    @property
    def zzly(self) -> DVector4: ...
    @property
    def zzlz(self) -> DVector4: ...
    @property
    def zzlo(self) -> DVector4: ...
    @property
    def zzll(self) -> DVector4: ...
    @property
    def zoxx(self) -> DVector4: ...
    @property
    def zoxy(self) -> DVector4: ...
    @property
    def zoxz(self) -> DVector4: ...
    @property
    def zoxo(self) -> DVector4: ...
    @property
    def zoxl(self) -> DVector4: ...
    @property
    def zoyx(self) -> DVector4: ...
    @property
    def zoyy(self) -> DVector4: ...
    @property
    def zoyz(self) -> DVector4: ...
    @property
    def zoyo(self) -> DVector4: ...
    @property
    def zoyl(self) -> DVector4: ...
    @property
    def zozx(self) -> DVector4: ...
    @property
    def zozy(self) -> DVector4: ...
    @property
    def zozz(self) -> DVector4: ...
    @property
    def zozo(self) -> DVector4: ...
    @property
    def zozl(self) -> DVector4: ...
    @property
    def zoox(self) -> DVector4: ...
    @property
    def zooy(self) -> DVector4: ...
    @property
    def zooz(self) -> DVector4: ...
    @property
    def zooo(self) -> DVector4: ...
    @property
    def zool(self) -> DVector4: ...
    @property
    def zolx(self) -> DVector4: ...
    @property
    def zoly(self) -> DVector4: ...
    @property
    def zolz(self) -> DVector4: ...
    @property
    def zolo(self) -> DVector4: ...
    @property
    def zoll(self) -> DVector4: ...
    @property
    def zlxx(self) -> DVector4: ...
    @property
    def zlxy(self) -> DVector4: ...
    @property
    def zlxz(self) -> DVector4: ...
    @property
    def zlxo(self) -> DVector4: ...
    @property
    def zlxl(self) -> DVector4: ...
    @property
    def zlyx(self) -> DVector4: ...
    @property
    def zlyy(self) -> DVector4: ...
    @property
    def zlyz(self) -> DVector4: ...
    @property
    def zlyo(self) -> DVector4: ...
    @property
    def zlyl(self) -> DVector4: ...
    @property
    def zlzx(self) -> DVector4: ...
    @property
    def zlzy(self) -> DVector4: ...
    @property
    def zlzz(self) -> DVector4: ...
    @property
    def zlzo(self) -> DVector4: ...
    @property
    def zlzl(self) -> DVector4: ...
    @property
    def zlox(self) -> DVector4: ...
    @property
    def zloy(self) -> DVector4: ...
    @property
    def zloz(self) -> DVector4: ...
    @property
    def zloo(self) -> DVector4: ...
    @property
    def zlol(self) -> DVector4: ...
    @property
    def zllx(self) -> DVector4: ...
    @property
    def zlly(self) -> DVector4: ...
    @property
    def zllz(self) -> DVector4: ...
    @property
    def zllo(self) -> DVector4: ...
    @property
    def zlll(self) -> DVector4: ...
    @property
    def oxxx(self) -> DVector4: ...
    @property
    def oxxy(self) -> DVector4: ...
    @property
    def oxxz(self) -> DVector4: ...
    @property
    def oxxo(self) -> DVector4: ...
    @property
    def oxxl(self) -> DVector4: ...
    @property
    def oxyx(self) -> DVector4: ...
    @property
    def oxyy(self) -> DVector4: ...
    @property
    def oxyz(self) -> DVector4: ...
    @property
    def oxyo(self) -> DVector4: ...
    @property
    def oxyl(self) -> DVector4: ...
    @property
    def oxzx(self) -> DVector4: ...
    @property
    def oxzy(self) -> DVector4: ...
    @property
    def oxzz(self) -> DVector4: ...
    @property
    def oxzo(self) -> DVector4: ...
    @property
    def oxzl(self) -> DVector4: ...
    @property
    def oxox(self) -> DVector4: ...
    @property
    def oxoy(self) -> DVector4: ...
    @property
    def oxoz(self) -> DVector4: ...
    @property
    def oxoo(self) -> DVector4: ...
    @property
    def oxol(self) -> DVector4: ...
    @property
    def oxlx(self) -> DVector4: ...
    @property
    def oxly(self) -> DVector4: ...
    @property
    def oxlz(self) -> DVector4: ...
    @property
    def oxlo(self) -> DVector4: ...
    @property
    def oxll(self) -> DVector4: ...
    @property
    def oyxx(self) -> DVector4: ...
    @property
    def oyxy(self) -> DVector4: ...
    @property
    def oyxz(self) -> DVector4: ...
    @property
    def oyxo(self) -> DVector4: ...
    @property
    def oyxl(self) -> DVector4: ...
    @property
    def oyyx(self) -> DVector4: ...
    @property
    def oyyy(self) -> DVector4: ...
    @property
    def oyyz(self) -> DVector4: ...
    @property
    def oyyo(self) -> DVector4: ...
    @property
    def oyyl(self) -> DVector4: ...
    @property
    def oyzx(self) -> DVector4: ...
    @property
    def oyzy(self) -> DVector4: ...
    @property
    def oyzz(self) -> DVector4: ...
    @property
    def oyzo(self) -> DVector4: ...
    @property
    def oyzl(self) -> DVector4: ...
    @property
    def oyox(self) -> DVector4: ...
    @property
    def oyoy(self) -> DVector4: ...
    @property
    def oyoz(self) -> DVector4: ...
    @property
    def oyoo(self) -> DVector4: ...
    @property
    def oyol(self) -> DVector4: ...
    @property
    def oylx(self) -> DVector4: ...
    @property
    def oyly(self) -> DVector4: ...
    @property
    def oylz(self) -> DVector4: ...
    @property
    def oylo(self) -> DVector4: ...
    @property
    def oyll(self) -> DVector4: ...
    @property
    def ozxx(self) -> DVector4: ...
    @property
    def ozxy(self) -> DVector4: ...
    @property
    def ozxz(self) -> DVector4: ...
    @property
    def ozxo(self) -> DVector4: ...
    @property
    def ozxl(self) -> DVector4: ...
    @property
    def ozyx(self) -> DVector4: ...
    @property
    def ozyy(self) -> DVector4: ...
    @property
    def ozyz(self) -> DVector4: ...
    @property
    def ozyo(self) -> DVector4: ...
    @property
    def ozyl(self) -> DVector4: ...
    @property
    def ozzx(self) -> DVector4: ...
    @property
    def ozzy(self) -> DVector4: ...
    @property
    def ozzz(self) -> DVector4: ...
    @property
    def ozzo(self) -> DVector4: ...
    @property
    def ozzl(self) -> DVector4: ...
    @property
    def ozox(self) -> DVector4: ...
    @property
    def ozoy(self) -> DVector4: ...
    @property
    def ozoz(self) -> DVector4: ...
    @property
    def ozoo(self) -> DVector4: ...
    @property
    def ozol(self) -> DVector4: ...
    @property
    def ozlx(self) -> DVector4: ...
    @property
    def ozly(self) -> DVector4: ...
    @property
    def ozlz(self) -> DVector4: ...
    @property
    def ozlo(self) -> DVector4: ...
    @property
    def ozll(self) -> DVector4: ...
    @property
    def ooxx(self) -> DVector4: ...
    @property
    def ooxy(self) -> DVector4: ...
    @property
    def ooxz(self) -> DVector4: ...
    @property
    def ooxo(self) -> DVector4: ...
    @property
    def ooxl(self) -> DVector4: ...
    @property
    def ooyx(self) -> DVector4: ...
    @property
    def ooyy(self) -> DVector4: ...
    @property
    def ooyz(self) -> DVector4: ...
    @property
    def ooyo(self) -> DVector4: ...
    @property
    def ooyl(self) -> DVector4: ...
    @property
    def oozx(self) -> DVector4: ...
    @property
    def oozy(self) -> DVector4: ...
    @property
    def oozz(self) -> DVector4: ...
    @property
    def oozo(self) -> DVector4: ...
    @property
    def oozl(self) -> DVector4: ...
    @property
    def ooox(self) -> DVector4: ...
    @property
    def oooy(self) -> DVector4: ...
    @property
    def oooz(self) -> DVector4: ...
    @property
    def oolx(self) -> DVector4: ...
    @property
    def ooly(self) -> DVector4: ...
    @property
    def oolz(self) -> DVector4: ...
    @property
    def olxx(self) -> DVector4: ...
    @property
    def olxy(self) -> DVector4: ...
    @property
    def olxz(self) -> DVector4: ...
    @property
    def olxo(self) -> DVector4: ...
    @property
    def olxl(self) -> DVector4: ...
    @property
    def olyx(self) -> DVector4: ...
    @property
    def olyy(self) -> DVector4: ...
    @property
    def olyz(self) -> DVector4: ...
    @property
    def olyo(self) -> DVector4: ...
    @property
    def olyl(self) -> DVector4: ...
    @property
    def olzx(self) -> DVector4: ...
    @property
    def olzy(self) -> DVector4: ...
    @property
    def olzz(self) -> DVector4: ...
    @property
    def olzo(self) -> DVector4: ...
    @property
    def olzl(self) -> DVector4: ...
    @property
    def olox(self) -> DVector4: ...
    @property
    def oloy(self) -> DVector4: ...
    @property
    def oloz(self) -> DVector4: ...
    @property
    def ollx(self) -> DVector4: ...
    @property
    def olly(self) -> DVector4: ...
    @property
    def ollz(self) -> DVector4: ...
    @property
    def lxxx(self) -> DVector4: ...
    @property
    def lxxy(self) -> DVector4: ...
    @property
    def lxxz(self) -> DVector4: ...
    @property
    def lxxo(self) -> DVector4: ...
    @property
    def lxxl(self) -> DVector4: ...
    @property
    def lxyx(self) -> DVector4: ...
    @property
    def lxyy(self) -> DVector4: ...
    @property
    def lxyz(self) -> DVector4: ...
    @property
    def lxyo(self) -> DVector4: ...
    @property
    def lxyl(self) -> DVector4: ...
    @property
    def lxzx(self) -> DVector4: ...
    @property
    def lxzy(self) -> DVector4: ...
    @property
    def lxzz(self) -> DVector4: ...
    @property
    def lxzo(self) -> DVector4: ...
    @property
    def lxzl(self) -> DVector4: ...
    @property
    def lxox(self) -> DVector4: ...
    @property
    def lxoy(self) -> DVector4: ...
    @property
    def lxoz(self) -> DVector4: ...
    @property
    def lxoo(self) -> DVector4: ...
    @property
    def lxol(self) -> DVector4: ...
    @property
    def lxlx(self) -> DVector4: ...
    @property
    def lxly(self) -> DVector4: ...
    @property
    def lxlz(self) -> DVector4: ...
    @property
    def lxlo(self) -> DVector4: ...
    @property
    def lxll(self) -> DVector4: ...
    @property
    def lyxx(self) -> DVector4: ...
    @property
    def lyxy(self) -> DVector4: ...
    @property
    def lyxz(self) -> DVector4: ...
    @property
    def lyxo(self) -> DVector4: ...
    @property
    def lyxl(self) -> DVector4: ...
    @property
    def lyyx(self) -> DVector4: ...
    @property
    def lyyy(self) -> DVector4: ...
    @property
    def lyyz(self) -> DVector4: ...
    @property
    def lyyo(self) -> DVector4: ...
    @property
    def lyyl(self) -> DVector4: ...
    @property
    def lyzx(self) -> DVector4: ...
    @property
    def lyzy(self) -> DVector4: ...
    @property
    def lyzz(self) -> DVector4: ...
    @property
    def lyzo(self) -> DVector4: ...
    @property
    def lyzl(self) -> DVector4: ...
    @property
    def lyox(self) -> DVector4: ...
    @property
    def lyoy(self) -> DVector4: ...
    @property
    def lyoz(self) -> DVector4: ...
    @property
    def lyoo(self) -> DVector4: ...
    @property
    def lyol(self) -> DVector4: ...
    @property
    def lylx(self) -> DVector4: ...
    @property
    def lyly(self) -> DVector4: ...
    @property
    def lylz(self) -> DVector4: ...
    @property
    def lylo(self) -> DVector4: ...
    @property
    def lyll(self) -> DVector4: ...
    @property
    def lzxx(self) -> DVector4: ...
    @property
    def lzxy(self) -> DVector4: ...
    @property
    def lzxz(self) -> DVector4: ...
    @property
    def lzxo(self) -> DVector4: ...
    @property
    def lzxl(self) -> DVector4: ...
    @property
    def lzyx(self) -> DVector4: ...
    @property
    def lzyy(self) -> DVector4: ...
    @property
    def lzyz(self) -> DVector4: ...
    @property
    def lzyo(self) -> DVector4: ...
    @property
    def lzyl(self) -> DVector4: ...
    @property
    def lzzx(self) -> DVector4: ...
    @property
    def lzzy(self) -> DVector4: ...
    @property
    def lzzz(self) -> DVector4: ...
    @property
    def lzzo(self) -> DVector4: ...
    @property
    def lzzl(self) -> DVector4: ...
    @property
    def lzox(self) -> DVector4: ...
    @property
    def lzoy(self) -> DVector4: ...
    @property
    def lzoz(self) -> DVector4: ...
    @property
    def lzoo(self) -> DVector4: ...
    @property
    def lzol(self) -> DVector4: ...
    @property
    def lzlx(self) -> DVector4: ...
    @property
    def lzly(self) -> DVector4: ...
    @property
    def lzlz(self) -> DVector4: ...
    @property
    def lzlo(self) -> DVector4: ...
    @property
    def lzll(self) -> DVector4: ...
    @property
    def loxx(self) -> DVector4: ...
    @property
    def loxy(self) -> DVector4: ...
    @property
    def loxz(self) -> DVector4: ...
    @property
    def loxo(self) -> DVector4: ...
    @property
    def loxl(self) -> DVector4: ...
    @property
    def loyx(self) -> DVector4: ...
    @property
    def loyy(self) -> DVector4: ...
    @property
    def loyz(self) -> DVector4: ...
    @property
    def loyo(self) -> DVector4: ...
    @property
    def loyl(self) -> DVector4: ...
    @property
    def lozx(self) -> DVector4: ...
    @property
    def lozy(self) -> DVector4: ...
    @property
    def lozz(self) -> DVector4: ...
    @property
    def lozo(self) -> DVector4: ...
    @property
    def lozl(self) -> DVector4: ...
    @property
    def loox(self) -> DVector4: ...
    @property
    def looy(self) -> DVector4: ...
    @property
    def looz(self) -> DVector4: ...
    @property
    def lolx(self) -> DVector4: ...
    @property
    def loly(self) -> DVector4: ...
    @property
    def lolz(self) -> DVector4: ...
    @property
    def llxx(self) -> DVector4: ...
    @property
    def llxy(self) -> DVector4: ...
    @property
    def llxz(self) -> DVector4: ...
    @property
    def llxo(self) -> DVector4: ...
    @property
    def llxl(self) -> DVector4: ...
    @property
    def llyx(self) -> DVector4: ...
    @property
    def llyy(self) -> DVector4: ...
    @property
    def llyz(self) -> DVector4: ...
    @property
    def llyo(self) -> DVector4: ...
    @property
    def llyl(self) -> DVector4: ...
    @property
    def llzx(self) -> DVector4: ...
    @property
    def llzy(self) -> DVector4: ...
    @property
    def llzz(self) -> DVector4: ...
    @property
    def llzo(self) -> DVector4: ...
    @property
    def llzl(self) -> DVector4: ...
    @property
    def llox(self) -> DVector4: ...
    @property
    def lloy(self) -> DVector4: ...
    @property
    def lloz(self) -> DVector4: ...
    @property
    def lllx(self) -> DVector4: ...
    @property
    def llly(self) -> DVector4: ...
    @property
    def lllz(self) -> DVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def g(self) -> float: ...
    @property
    def b(self) -> float: ...
    @property
    def rr(self) -> DVector2: ...
    @property
    def rg(self) -> DVector2: ...
    @property
    def rb(self) -> DVector2: ...
    @property
    def ro(self) -> DVector2: ...
    @property
    def rl(self) -> DVector2: ...
    @property
    def gr(self) -> DVector2: ...
    @property
    def gg(self) -> DVector2: ...
    @property
    def gb(self) -> DVector2: ...
    @property
    def go(self) -> DVector2: ...
    @property
    def gl(self) -> DVector2: ...
    @property
    def br(self) -> DVector2: ...
    @property
    def bg(self) -> DVector2: ...
    @property
    def bb(self) -> DVector2: ...
    @property
    def bo(self) -> DVector2: ...
    @property
    def bl(self) -> DVector2: ...
    @property
    def og(self) -> DVector2: ...
    @property
    def ob(self) -> DVector2: ...
    @property
    def lr(self) -> DVector2: ...
    @property
    def lg(self) -> DVector2: ...
    @property
    def lb(self) -> DVector2: ...
    @property
    def rrr(self) -> DVector3: ...
    @property
    def rrg(self) -> DVector3: ...
    @property
    def rrb(self) -> DVector3: ...
    @property
    def rro(self) -> DVector3: ...
    @property
    def rrl(self) -> DVector3: ...
    @property
    def rgr(self) -> DVector3: ...
    @property
    def rgg(self) -> DVector3: ...
    @property
    def rgb(self) -> DVector3: ...
    @property
    def rgo(self) -> DVector3: ...
    @property
    def rgl(self) -> DVector3: ...
    @property
    def rbr(self) -> DVector3: ...
    @property
    def rbg(self) -> DVector3: ...
    @property
    def rbb(self) -> DVector3: ...
    @property
    def rbo(self) -> DVector3: ...
    @property
    def rbl(self) -> DVector3: ...
    @property
    def ror(self) -> DVector3: ...
    @property
    def rog(self) -> DVector3: ...
    @property
    def rob(self) -> DVector3: ...
    @property
    def roo(self) -> DVector3: ...
    @property
    def rol(self) -> DVector3: ...
    @property
    def rlr(self) -> DVector3: ...
    @property
    def rlg(self) -> DVector3: ...
    @property
    def rlb(self) -> DVector3: ...
    @property
    def rlo(self) -> DVector3: ...
    @property
    def rll(self) -> DVector3: ...
    @property
    def grr(self) -> DVector3: ...
    @property
    def grg(self) -> DVector3: ...
    @property
    def grb(self) -> DVector3: ...
    @property
    def gro(self) -> DVector3: ...
    @property
    def grl(self) -> DVector3: ...
    @property
    def ggr(self) -> DVector3: ...
    @property
    def ggg(self) -> DVector3: ...
    @property
    def ggb(self) -> DVector3: ...
    @property
    def ggo(self) -> DVector3: ...
    @property
    def ggl(self) -> DVector3: ...
    @property
    def gbr(self) -> DVector3: ...
    @property
    def gbg(self) -> DVector3: ...
    @property
    def gbb(self) -> DVector3: ...
    @property
    def gbo(self) -> DVector3: ...
    @property
    def gbl(self) -> DVector3: ...
    @property
    def gor(self) -> DVector3: ...
    @property
    def gog(self) -> DVector3: ...
    @property
    def gob(self) -> DVector3: ...
    @property
    def goo(self) -> DVector3: ...
    @property
    def gol(self) -> DVector3: ...
    @property
    def glr(self) -> DVector3: ...
    @property
    def glg(self) -> DVector3: ...
    @property
    def glb(self) -> DVector3: ...
    @property
    def glo(self) -> DVector3: ...
    @property
    def gll(self) -> DVector3: ...
    @property
    def brr(self) -> DVector3: ...
    @property
    def brg(self) -> DVector3: ...
    @property
    def brb(self) -> DVector3: ...
    @property
    def bro(self) -> DVector3: ...
    @property
    def brl(self) -> DVector3: ...
    @property
    def bgr(self) -> DVector3: ...
    @property
    def bgg(self) -> DVector3: ...
    @property
    def bgb(self) -> DVector3: ...
    @property
    def bgo(self) -> DVector3: ...
    @property
    def bgl(self) -> DVector3: ...
    @property
    def bbr(self) -> DVector3: ...
    @property
    def bbg(self) -> DVector3: ...
    @property
    def bbb(self) -> DVector3: ...
    @property
    def bbo(self) -> DVector3: ...
    @property
    def bbl(self) -> DVector3: ...
    @property
    def bor(self) -> DVector3: ...
    @property
    def bog(self) -> DVector3: ...
    @property
    def bob(self) -> DVector3: ...
    @property
    def boo(self) -> DVector3: ...
    @property
    def bol(self) -> DVector3: ...
    @property
    def blr(self) -> DVector3: ...
    @property
    def blg(self) -> DVector3: ...
    @property
    def blb(self) -> DVector3: ...
    @property
    def blo(self) -> DVector3: ...
    @property
    def bll(self) -> DVector3: ...
    @property
    def orr(self) -> DVector3: ...
    @property
    def org(self) -> DVector3: ...
    @property
    def orb(self) -> DVector3: ...
    @property
    def oro(self) -> DVector3: ...
    @property
    def orl(self) -> DVector3: ...
    @property
    def ogr(self) -> DVector3: ...
    @property
    def ogg(self) -> DVector3: ...
    @property
    def ogb(self) -> DVector3: ...
    @property
    def ogo(self) -> DVector3: ...
    @property
    def ogl(self) -> DVector3: ...
    @property
    def obr(self) -> DVector3: ...
    @property
    def obg(self) -> DVector3: ...
    @property
    def obb(self) -> DVector3: ...
    @property
    def obo(self) -> DVector3: ...
    @property
    def obl(self) -> DVector3: ...
    @property
    def oor(self) -> DVector3: ...
    @property
    def oog(self) -> DVector3: ...
    @property
    def oob(self) -> DVector3: ...
    @property
    def olr(self) -> DVector3: ...
    @property
    def olg(self) -> DVector3: ...
    @property
    def olb(self) -> DVector3: ...
    @property
    def lrr(self) -> DVector3: ...
    @property
    def lrg(self) -> DVector3: ...
    @property
    def lrb(self) -> DVector3: ...
    @property
    def lro(self) -> DVector3: ...
    @property
    def lrl(self) -> DVector3: ...
    @property
    def lgr(self) -> DVector3: ...
    @property
    def lgg(self) -> DVector3: ...
    @property
    def lgb(self) -> DVector3: ...
    @property
    def lgo(self) -> DVector3: ...
    @property
    def lgl(self) -> DVector3: ...
    @property
    def lbr(self) -> DVector3: ...
    @property
    def lbg(self) -> DVector3: ...
    @property
    def lbb(self) -> DVector3: ...
    @property
    def lbo(self) -> DVector3: ...
    @property
    def lbl(self) -> DVector3: ...
    @property
    def lor(self) -> DVector3: ...
    @property
    def log(self) -> DVector3: ...
    @property
    def lob(self) -> DVector3: ...
    @property
    def llr(self) -> DVector3: ...
    @property
    def llg(self) -> DVector3: ...
    @property
    def llb(self) -> DVector3: ...
    @property
    def rrrr(self) -> DVector4: ...
    @property
    def rrrg(self) -> DVector4: ...
    @property
    def rrrb(self) -> DVector4: ...
    @property
    def rrro(self) -> DVector4: ...
    @property
    def rrrl(self) -> DVector4: ...
    @property
    def rrgr(self) -> DVector4: ...
    @property
    def rrgg(self) -> DVector4: ...
    @property
    def rrgb(self) -> DVector4: ...
    @property
    def rrgo(self) -> DVector4: ...
    @property
    def rrgl(self) -> DVector4: ...
    @property
    def rrbr(self) -> DVector4: ...
    @property
    def rrbg(self) -> DVector4: ...
    @property
    def rrbb(self) -> DVector4: ...
    @property
    def rrbo(self) -> DVector4: ...
    @property
    def rrbl(self) -> DVector4: ...
    @property
    def rror(self) -> DVector4: ...
    @property
    def rrog(self) -> DVector4: ...
    @property
    def rrob(self) -> DVector4: ...
    @property
    def rroo(self) -> DVector4: ...
    @property
    def rrol(self) -> DVector4: ...
    @property
    def rrlr(self) -> DVector4: ...
    @property
    def rrlg(self) -> DVector4: ...
    @property
    def rrlb(self) -> DVector4: ...
    @property
    def rrlo(self) -> DVector4: ...
    @property
    def rrll(self) -> DVector4: ...
    @property
    def rgrr(self) -> DVector4: ...
    @property
    def rgrg(self) -> DVector4: ...
    @property
    def rgrb(self) -> DVector4: ...
    @property
    def rgro(self) -> DVector4: ...
    @property
    def rgrl(self) -> DVector4: ...
    @property
    def rggr(self) -> DVector4: ...
    @property
    def rggg(self) -> DVector4: ...
    @property
    def rggb(self) -> DVector4: ...
    @property
    def rggo(self) -> DVector4: ...
    @property
    def rggl(self) -> DVector4: ...
    @property
    def rgbr(self) -> DVector4: ...
    @property
    def rgbg(self) -> DVector4: ...
    @property
    def rgbb(self) -> DVector4: ...
    @property
    def rgbo(self) -> DVector4: ...
    @property
    def rgbl(self) -> DVector4: ...
    @property
    def rgor(self) -> DVector4: ...
    @property
    def rgog(self) -> DVector4: ...
    @property
    def rgob(self) -> DVector4: ...
    @property
    def rgoo(self) -> DVector4: ...
    @property
    def rgol(self) -> DVector4: ...
    @property
    def rglr(self) -> DVector4: ...
    @property
    def rglg(self) -> DVector4: ...
    @property
    def rglb(self) -> DVector4: ...
    @property
    def rglo(self) -> DVector4: ...
    @property
    def rgll(self) -> DVector4: ...
    @property
    def rbrr(self) -> DVector4: ...
    @property
    def rbrg(self) -> DVector4: ...
    @property
    def rbrb(self) -> DVector4: ...
    @property
    def rbro(self) -> DVector4: ...
    @property
    def rbrl(self) -> DVector4: ...
    @property
    def rbgr(self) -> DVector4: ...
    @property
    def rbgg(self) -> DVector4: ...
    @property
    def rbgb(self) -> DVector4: ...
    @property
    def rbgo(self) -> DVector4: ...
    @property
    def rbgl(self) -> DVector4: ...
    @property
    def rbbr(self) -> DVector4: ...
    @property
    def rbbg(self) -> DVector4: ...
    @property
    def rbbb(self) -> DVector4: ...
    @property
    def rbbo(self) -> DVector4: ...
    @property
    def rbbl(self) -> DVector4: ...
    @property
    def rbor(self) -> DVector4: ...
    @property
    def rbog(self) -> DVector4: ...
    @property
    def rbob(self) -> DVector4: ...
    @property
    def rboo(self) -> DVector4: ...
    @property
    def rbol(self) -> DVector4: ...
    @property
    def rblr(self) -> DVector4: ...
    @property
    def rblg(self) -> DVector4: ...
    @property
    def rblb(self) -> DVector4: ...
    @property
    def rblo(self) -> DVector4: ...
    @property
    def rbll(self) -> DVector4: ...
    @property
    def rorr(self) -> DVector4: ...
    @property
    def rorg(self) -> DVector4: ...
    @property
    def rorb(self) -> DVector4: ...
    @property
    def roro(self) -> DVector4: ...
    @property
    def rorl(self) -> DVector4: ...
    @property
    def rogr(self) -> DVector4: ...
    @property
    def rogg(self) -> DVector4: ...
    @property
    def rogb(self) -> DVector4: ...
    @property
    def rogo(self) -> DVector4: ...
    @property
    def rogl(self) -> DVector4: ...
    @property
    def robr(self) -> DVector4: ...
    @property
    def robg(self) -> DVector4: ...
    @property
    def robb(self) -> DVector4: ...
    @property
    def robo(self) -> DVector4: ...
    @property
    def robl(self) -> DVector4: ...
    @property
    def roor(self) -> DVector4: ...
    @property
    def roog(self) -> DVector4: ...
    @property
    def roob(self) -> DVector4: ...
    @property
    def rooo(self) -> DVector4: ...
    @property
    def rool(self) -> DVector4: ...
    @property
    def rolr(self) -> DVector4: ...
    @property
    def rolg(self) -> DVector4: ...
    @property
    def rolb(self) -> DVector4: ...
    @property
    def rolo(self) -> DVector4: ...
    @property
    def roll(self) -> DVector4: ...
    @property
    def rlrr(self) -> DVector4: ...
    @property
    def rlrg(self) -> DVector4: ...
    @property
    def rlrb(self) -> DVector4: ...
    @property
    def rlro(self) -> DVector4: ...
    @property
    def rlrl(self) -> DVector4: ...
    @property
    def rlgr(self) -> DVector4: ...
    @property
    def rlgg(self) -> DVector4: ...
    @property
    def rlgb(self) -> DVector4: ...
    @property
    def rlgo(self) -> DVector4: ...
    @property
    def rlgl(self) -> DVector4: ...
    @property
    def rlbr(self) -> DVector4: ...
    @property
    def rlbg(self) -> DVector4: ...
    @property
    def rlbb(self) -> DVector4: ...
    @property
    def rlbo(self) -> DVector4: ...
    @property
    def rlbl(self) -> DVector4: ...
    @property
    def rlor(self) -> DVector4: ...
    @property
    def rlog(self) -> DVector4: ...
    @property
    def rlob(self) -> DVector4: ...
    @property
    def rloo(self) -> DVector4: ...
    @property
    def rlol(self) -> DVector4: ...
    @property
    def rllr(self) -> DVector4: ...
    @property
    def rllg(self) -> DVector4: ...
    @property
    def rllb(self) -> DVector4: ...
    @property
    def rllo(self) -> DVector4: ...
    @property
    def rlll(self) -> DVector4: ...
    @property
    def grrr(self) -> DVector4: ...
    @property
    def grrg(self) -> DVector4: ...
    @property
    def grrb(self) -> DVector4: ...
    @property
    def grro(self) -> DVector4: ...
    @property
    def grrl(self) -> DVector4: ...
    @property
    def grgr(self) -> DVector4: ...
    @property
    def grgg(self) -> DVector4: ...
    @property
    def grgb(self) -> DVector4: ...
    @property
    def grgo(self) -> DVector4: ...
    @property
    def grgl(self) -> DVector4: ...
    @property
    def grbr(self) -> DVector4: ...
    @property
    def grbg(self) -> DVector4: ...
    @property
    def grbb(self) -> DVector4: ...
    @property
    def grbo(self) -> DVector4: ...
    @property
    def grbl(self) -> DVector4: ...
    @property
    def gror(self) -> DVector4: ...
    @property
    def grog(self) -> DVector4: ...
    @property
    def grob(self) -> DVector4: ...
    @property
    def groo(self) -> DVector4: ...
    @property
    def grol(self) -> DVector4: ...
    @property
    def grlr(self) -> DVector4: ...
    @property
    def grlg(self) -> DVector4: ...
    @property
    def grlb(self) -> DVector4: ...
    @property
    def grlo(self) -> DVector4: ...
    @property
    def grll(self) -> DVector4: ...
    @property
    def ggrr(self) -> DVector4: ...
    @property
    def ggrg(self) -> DVector4: ...
    @property
    def ggrb(self) -> DVector4: ...
    @property
    def ggro(self) -> DVector4: ...
    @property
    def ggrl(self) -> DVector4: ...
    @property
    def gggr(self) -> DVector4: ...
    @property
    def gggg(self) -> DVector4: ...
    @property
    def gggb(self) -> DVector4: ...
    @property
    def gggo(self) -> DVector4: ...
    @property
    def gggl(self) -> DVector4: ...
    @property
    def ggbr(self) -> DVector4: ...
    @property
    def ggbg(self) -> DVector4: ...
    @property
    def ggbb(self) -> DVector4: ...
    @property
    def ggbo(self) -> DVector4: ...
    @property
    def ggbl(self) -> DVector4: ...
    @property
    def ggor(self) -> DVector4: ...
    @property
    def ggog(self) -> DVector4: ...
    @property
    def ggob(self) -> DVector4: ...
    @property
    def ggoo(self) -> DVector4: ...
    @property
    def ggol(self) -> DVector4: ...
    @property
    def gglr(self) -> DVector4: ...
    @property
    def gglg(self) -> DVector4: ...
    @property
    def gglb(self) -> DVector4: ...
    @property
    def gglo(self) -> DVector4: ...
    @property
    def ggll(self) -> DVector4: ...
    @property
    def gbrr(self) -> DVector4: ...
    @property
    def gbrg(self) -> DVector4: ...
    @property
    def gbrb(self) -> DVector4: ...
    @property
    def gbro(self) -> DVector4: ...
    @property
    def gbrl(self) -> DVector4: ...
    @property
    def gbgr(self) -> DVector4: ...
    @property
    def gbgg(self) -> DVector4: ...
    @property
    def gbgb(self) -> DVector4: ...
    @property
    def gbgo(self) -> DVector4: ...
    @property
    def gbgl(self) -> DVector4: ...
    @property
    def gbbr(self) -> DVector4: ...
    @property
    def gbbg(self) -> DVector4: ...
    @property
    def gbbb(self) -> DVector4: ...
    @property
    def gbbo(self) -> DVector4: ...
    @property
    def gbbl(self) -> DVector4: ...
    @property
    def gbor(self) -> DVector4: ...
    @property
    def gbog(self) -> DVector4: ...
    @property
    def gbob(self) -> DVector4: ...
    @property
    def gboo(self) -> DVector4: ...
    @property
    def gbol(self) -> DVector4: ...
    @property
    def gblr(self) -> DVector4: ...
    @property
    def gblg(self) -> DVector4: ...
    @property
    def gblb(self) -> DVector4: ...
    @property
    def gblo(self) -> DVector4: ...
    @property
    def gbll(self) -> DVector4: ...
    @property
    def gorr(self) -> DVector4: ...
    @property
    def gorg(self) -> DVector4: ...
    @property
    def gorb(self) -> DVector4: ...
    @property
    def goro(self) -> DVector4: ...
    @property
    def gorl(self) -> DVector4: ...
    @property
    def gogr(self) -> DVector4: ...
    @property
    def gogg(self) -> DVector4: ...
    @property
    def gogb(self) -> DVector4: ...
    @property
    def gogo(self) -> DVector4: ...
    @property
    def gogl(self) -> DVector4: ...
    @property
    def gobr(self) -> DVector4: ...
    @property
    def gobg(self) -> DVector4: ...
    @property
    def gobb(self) -> DVector4: ...
    @property
    def gobo(self) -> DVector4: ...
    @property
    def gobl(self) -> DVector4: ...
    @property
    def goor(self) -> DVector4: ...
    @property
    def goog(self) -> DVector4: ...
    @property
    def goob(self) -> DVector4: ...
    @property
    def gooo(self) -> DVector4: ...
    @property
    def gool(self) -> DVector4: ...
    @property
    def golr(self) -> DVector4: ...
    @property
    def golg(self) -> DVector4: ...
    @property
    def golb(self) -> DVector4: ...
    @property
    def golo(self) -> DVector4: ...
    @property
    def goll(self) -> DVector4: ...
    @property
    def glrr(self) -> DVector4: ...
    @property
    def glrg(self) -> DVector4: ...
    @property
    def glrb(self) -> DVector4: ...
    @property
    def glro(self) -> DVector4: ...
    @property
    def glrl(self) -> DVector4: ...
    @property
    def glgr(self) -> DVector4: ...
    @property
    def glgg(self) -> DVector4: ...
    @property
    def glgb(self) -> DVector4: ...
    @property
    def glgo(self) -> DVector4: ...
    @property
    def glgl(self) -> DVector4: ...
    @property
    def glbr(self) -> DVector4: ...
    @property
    def glbg(self) -> DVector4: ...
    @property
    def glbb(self) -> DVector4: ...
    @property
    def glbo(self) -> DVector4: ...
    @property
    def glbl(self) -> DVector4: ...
    @property
    def glor(self) -> DVector4: ...
    @property
    def glog(self) -> DVector4: ...
    @property
    def glob(self) -> DVector4: ...
    @property
    def gloo(self) -> DVector4: ...
    @property
    def glol(self) -> DVector4: ...
    @property
    def gllr(self) -> DVector4: ...
    @property
    def gllg(self) -> DVector4: ...
    @property
    def gllb(self) -> DVector4: ...
    @property
    def gllo(self) -> DVector4: ...
    @property
    def glll(self) -> DVector4: ...
    @property
    def brrr(self) -> DVector4: ...
    @property
    def brrg(self) -> DVector4: ...
    @property
    def brrb(self) -> DVector4: ...
    @property
    def brro(self) -> DVector4: ...
    @property
    def brrl(self) -> DVector4: ...
    @property
    def brgr(self) -> DVector4: ...
    @property
    def brgg(self) -> DVector4: ...
    @property
    def brgb(self) -> DVector4: ...
    @property
    def brgo(self) -> DVector4: ...
    @property
    def brgl(self) -> DVector4: ...
    @property
    def brbr(self) -> DVector4: ...
    @property
    def brbg(self) -> DVector4: ...
    @property
    def brbb(self) -> DVector4: ...
    @property
    def brbo(self) -> DVector4: ...
    @property
    def brbl(self) -> DVector4: ...
    @property
    def bror(self) -> DVector4: ...
    @property
    def brog(self) -> DVector4: ...
    @property
    def brob(self) -> DVector4: ...
    @property
    def broo(self) -> DVector4: ...
    @property
    def brol(self) -> DVector4: ...
    @property
    def brlr(self) -> DVector4: ...
    @property
    def brlg(self) -> DVector4: ...
    @property
    def brlb(self) -> DVector4: ...
    @property
    def brlo(self) -> DVector4: ...
    @property
    def brll(self) -> DVector4: ...
    @property
    def bgrr(self) -> DVector4: ...
    @property
    def bgrg(self) -> DVector4: ...
    @property
    def bgrb(self) -> DVector4: ...
    @property
    def bgro(self) -> DVector4: ...
    @property
    def bgrl(self) -> DVector4: ...
    @property
    def bggr(self) -> DVector4: ...
    @property
    def bggg(self) -> DVector4: ...
    @property
    def bggb(self) -> DVector4: ...
    @property
    def bggo(self) -> DVector4: ...
    @property
    def bggl(self) -> DVector4: ...
    @property
    def bgbr(self) -> DVector4: ...
    @property
    def bgbg(self) -> DVector4: ...
    @property
    def bgbb(self) -> DVector4: ...
    @property
    def bgbo(self) -> DVector4: ...
    @property
    def bgbl(self) -> DVector4: ...
    @property
    def bgor(self) -> DVector4: ...
    @property
    def bgog(self) -> DVector4: ...
    @property
    def bgob(self) -> DVector4: ...
    @property
    def bgoo(self) -> DVector4: ...
    @property
    def bgol(self) -> DVector4: ...
    @property
    def bglr(self) -> DVector4: ...
    @property
    def bglg(self) -> DVector4: ...
    @property
    def bglb(self) -> DVector4: ...
    @property
    def bglo(self) -> DVector4: ...
    @property
    def bgll(self) -> DVector4: ...
    @property
    def bbrr(self) -> DVector4: ...
    @property
    def bbrg(self) -> DVector4: ...
    @property
    def bbrb(self) -> DVector4: ...
    @property
    def bbro(self) -> DVector4: ...
    @property
    def bbrl(self) -> DVector4: ...
    @property
    def bbgr(self) -> DVector4: ...
    @property
    def bbgg(self) -> DVector4: ...
    @property
    def bbgb(self) -> DVector4: ...
    @property
    def bbgo(self) -> DVector4: ...
    @property
    def bbgl(self) -> DVector4: ...
    @property
    def bbbr(self) -> DVector4: ...
    @property
    def bbbg(self) -> DVector4: ...
    @property
    def bbbb(self) -> DVector4: ...
    @property
    def bbbo(self) -> DVector4: ...
    @property
    def bbbl(self) -> DVector4: ...
    @property
    def bbor(self) -> DVector4: ...
    @property
    def bbog(self) -> DVector4: ...
    @property
    def bbob(self) -> DVector4: ...
    @property
    def bboo(self) -> DVector4: ...
    @property
    def bbol(self) -> DVector4: ...
    @property
    def bblr(self) -> DVector4: ...
    @property
    def bblg(self) -> DVector4: ...
    @property
    def bblb(self) -> DVector4: ...
    @property
    def bblo(self) -> DVector4: ...
    @property
    def bbll(self) -> DVector4: ...
    @property
    def borr(self) -> DVector4: ...
    @property
    def borg(self) -> DVector4: ...
    @property
    def borb(self) -> DVector4: ...
    @property
    def boro(self) -> DVector4: ...
    @property
    def borl(self) -> DVector4: ...
    @property
    def bogr(self) -> DVector4: ...
    @property
    def bogg(self) -> DVector4: ...
    @property
    def bogb(self) -> DVector4: ...
    @property
    def bogo(self) -> DVector4: ...
    @property
    def bogl(self) -> DVector4: ...
    @property
    def bobr(self) -> DVector4: ...
    @property
    def bobg(self) -> DVector4: ...
    @property
    def bobb(self) -> DVector4: ...
    @property
    def bobo(self) -> DVector4: ...
    @property
    def bobl(self) -> DVector4: ...
    @property
    def boor(self) -> DVector4: ...
    @property
    def boog(self) -> DVector4: ...
    @property
    def boob(self) -> DVector4: ...
    @property
    def booo(self) -> DVector4: ...
    @property
    def bool(self) -> DVector4: ...
    @property
    def bolr(self) -> DVector4: ...
    @property
    def bolg(self) -> DVector4: ...
    @property
    def bolb(self) -> DVector4: ...
    @property
    def bolo(self) -> DVector4: ...
    @property
    def boll(self) -> DVector4: ...
    @property
    def blrr(self) -> DVector4: ...
    @property
    def blrg(self) -> DVector4: ...
    @property
    def blrb(self) -> DVector4: ...
    @property
    def blro(self) -> DVector4: ...
    @property
    def blrl(self) -> DVector4: ...
    @property
    def blgr(self) -> DVector4: ...
    @property
    def blgg(self) -> DVector4: ...
    @property
    def blgb(self) -> DVector4: ...
    @property
    def blgo(self) -> DVector4: ...
    @property
    def blgl(self) -> DVector4: ...
    @property
    def blbr(self) -> DVector4: ...
    @property
    def blbg(self) -> DVector4: ...
    @property
    def blbb(self) -> DVector4: ...
    @property
    def blbo(self) -> DVector4: ...
    @property
    def blbl(self) -> DVector4: ...
    @property
    def blor(self) -> DVector4: ...
    @property
    def blog(self) -> DVector4: ...
    @property
    def blob(self) -> DVector4: ...
    @property
    def bloo(self) -> DVector4: ...
    @property
    def blol(self) -> DVector4: ...
    @property
    def bllr(self) -> DVector4: ...
    @property
    def bllg(self) -> DVector4: ...
    @property
    def bllb(self) -> DVector4: ...
    @property
    def bllo(self) -> DVector4: ...
    @property
    def blll(self) -> DVector4: ...
    @property
    def orrr(self) -> DVector4: ...
    @property
    def orrg(self) -> DVector4: ...
    @property
    def orrb(self) -> DVector4: ...
    @property
    def orro(self) -> DVector4: ...
    @property
    def orrl(self) -> DVector4: ...
    @property
    def orgr(self) -> DVector4: ...
    @property
    def orgg(self) -> DVector4: ...
    @property
    def orgb(self) -> DVector4: ...
    @property
    def orgo(self) -> DVector4: ...
    @property
    def orgl(self) -> DVector4: ...
    @property
    def orbr(self) -> DVector4: ...
    @property
    def orbg(self) -> DVector4: ...
    @property
    def orbb(self) -> DVector4: ...
    @property
    def orbo(self) -> DVector4: ...
    @property
    def orbl(self) -> DVector4: ...
    @property
    def oror(self) -> DVector4: ...
    @property
    def orog(self) -> DVector4: ...
    @property
    def orob(self) -> DVector4: ...
    @property
    def oroo(self) -> DVector4: ...
    @property
    def orol(self) -> DVector4: ...
    @property
    def orlr(self) -> DVector4: ...
    @property
    def orlg(self) -> DVector4: ...
    @property
    def orlb(self) -> DVector4: ...
    @property
    def orlo(self) -> DVector4: ...
    @property
    def orll(self) -> DVector4: ...
    @property
    def ogrr(self) -> DVector4: ...
    @property
    def ogrg(self) -> DVector4: ...
    @property
    def ogrb(self) -> DVector4: ...
    @property
    def ogro(self) -> DVector4: ...
    @property
    def ogrl(self) -> DVector4: ...
    @property
    def oggr(self) -> DVector4: ...
    @property
    def oggg(self) -> DVector4: ...
    @property
    def oggb(self) -> DVector4: ...
    @property
    def oggo(self) -> DVector4: ...
    @property
    def oggl(self) -> DVector4: ...
    @property
    def ogbr(self) -> DVector4: ...
    @property
    def ogbg(self) -> DVector4: ...
    @property
    def ogbb(self) -> DVector4: ...
    @property
    def ogbo(self) -> DVector4: ...
    @property
    def ogbl(self) -> DVector4: ...
    @property
    def ogor(self) -> DVector4: ...
    @property
    def ogog(self) -> DVector4: ...
    @property
    def ogob(self) -> DVector4: ...
    @property
    def ogoo(self) -> DVector4: ...
    @property
    def ogol(self) -> DVector4: ...
    @property
    def oglr(self) -> DVector4: ...
    @property
    def oglg(self) -> DVector4: ...
    @property
    def oglb(self) -> DVector4: ...
    @property
    def oglo(self) -> DVector4: ...
    @property
    def ogll(self) -> DVector4: ...
    @property
    def obrr(self) -> DVector4: ...
    @property
    def obrg(self) -> DVector4: ...
    @property
    def obrb(self) -> DVector4: ...
    @property
    def obro(self) -> DVector4: ...
    @property
    def obrl(self) -> DVector4: ...
    @property
    def obgr(self) -> DVector4: ...
    @property
    def obgg(self) -> DVector4: ...
    @property
    def obgb(self) -> DVector4: ...
    @property
    def obgo(self) -> DVector4: ...
    @property
    def obgl(self) -> DVector4: ...
    @property
    def obbr(self) -> DVector4: ...
    @property
    def obbg(self) -> DVector4: ...
    @property
    def obbb(self) -> DVector4: ...
    @property
    def obbo(self) -> DVector4: ...
    @property
    def obbl(self) -> DVector4: ...
    @property
    def obor(self) -> DVector4: ...
    @property
    def obog(self) -> DVector4: ...
    @property
    def obob(self) -> DVector4: ...
    @property
    def oboo(self) -> DVector4: ...
    @property
    def obol(self) -> DVector4: ...
    @property
    def oblr(self) -> DVector4: ...
    @property
    def oblg(self) -> DVector4: ...
    @property
    def oblb(self) -> DVector4: ...
    @property
    def oblo(self) -> DVector4: ...
    @property
    def obll(self) -> DVector4: ...
    @property
    def oorr(self) -> DVector4: ...
    @property
    def oorg(self) -> DVector4: ...
    @property
    def oorb(self) -> DVector4: ...
    @property
    def ooro(self) -> DVector4: ...
    @property
    def oorl(self) -> DVector4: ...
    @property
    def oogr(self) -> DVector4: ...
    @property
    def oogg(self) -> DVector4: ...
    @property
    def oogb(self) -> DVector4: ...
    @property
    def oogo(self) -> DVector4: ...
    @property
    def oogl(self) -> DVector4: ...
    @property
    def oobr(self) -> DVector4: ...
    @property
    def oobg(self) -> DVector4: ...
    @property
    def oobb(self) -> DVector4: ...
    @property
    def oobo(self) -> DVector4: ...
    @property
    def oobl(self) -> DVector4: ...
    @property
    def ooor(self) -> DVector4: ...
    @property
    def ooog(self) -> DVector4: ...
    @property
    def ooob(self) -> DVector4: ...
    @property
    def oolr(self) -> DVector4: ...
    @property
    def oolg(self) -> DVector4: ...
    @property
    def oolb(self) -> DVector4: ...
    @property
    def olrr(self) -> DVector4: ...
    @property
    def olrg(self) -> DVector4: ...
    @property
    def olrb(self) -> DVector4: ...
    @property
    def olro(self) -> DVector4: ...
    @property
    def olrl(self) -> DVector4: ...
    @property
    def olgr(self) -> DVector4: ...
    @property
    def olgg(self) -> DVector4: ...
    @property
    def olgb(self) -> DVector4: ...
    @property
    def olgo(self) -> DVector4: ...
    @property
    def olgl(self) -> DVector4: ...
    @property
    def olbr(self) -> DVector4: ...
    @property
    def olbg(self) -> DVector4: ...
    @property
    def olbb(self) -> DVector4: ...
    @property
    def olbo(self) -> DVector4: ...
    @property
    def olbl(self) -> DVector4: ...
    @property
    def olor(self) -> DVector4: ...
    @property
    def olog(self) -> DVector4: ...
    @property
    def olob(self) -> DVector4: ...
    @property
    def ollr(self) -> DVector4: ...
    @property
    def ollg(self) -> DVector4: ...
    @property
    def ollb(self) -> DVector4: ...
    @property
    def lrrr(self) -> DVector4: ...
    @property
    def lrrg(self) -> DVector4: ...
    @property
    def lrrb(self) -> DVector4: ...
    @property
    def lrro(self) -> DVector4: ...
    @property
    def lrrl(self) -> DVector4: ...
    @property
    def lrgr(self) -> DVector4: ...
    @property
    def lrgg(self) -> DVector4: ...
    @property
    def lrgb(self) -> DVector4: ...
    @property
    def lrgo(self) -> DVector4: ...
    @property
    def lrgl(self) -> DVector4: ...
    @property
    def lrbr(self) -> DVector4: ...
    @property
    def lrbg(self) -> DVector4: ...
    @property
    def lrbb(self) -> DVector4: ...
    @property
    def lrbo(self) -> DVector4: ...
    @property
    def lrbl(self) -> DVector4: ...
    @property
    def lror(self) -> DVector4: ...
    @property
    def lrog(self) -> DVector4: ...
    @property
    def lrob(self) -> DVector4: ...
    @property
    def lroo(self) -> DVector4: ...
    @property
    def lrol(self) -> DVector4: ...
    @property
    def lrlr(self) -> DVector4: ...
    @property
    def lrlg(self) -> DVector4: ...
    @property
    def lrlb(self) -> DVector4: ...
    @property
    def lrlo(self) -> DVector4: ...
    @property
    def lrll(self) -> DVector4: ...
    @property
    def lgrr(self) -> DVector4: ...
    @property
    def lgrg(self) -> DVector4: ...
    @property
    def lgrb(self) -> DVector4: ...
    @property
    def lgro(self) -> DVector4: ...
    @property
    def lgrl(self) -> DVector4: ...
    @property
    def lggr(self) -> DVector4: ...
    @property
    def lggg(self) -> DVector4: ...
    @property
    def lggb(self) -> DVector4: ...
    @property
    def lggo(self) -> DVector4: ...
    @property
    def lggl(self) -> DVector4: ...
    @property
    def lgbr(self) -> DVector4: ...
    @property
    def lgbg(self) -> DVector4: ...
    @property
    def lgbb(self) -> DVector4: ...
    @property
    def lgbo(self) -> DVector4: ...
    @property
    def lgbl(self) -> DVector4: ...
    @property
    def lgor(self) -> DVector4: ...
    @property
    def lgog(self) -> DVector4: ...
    @property
    def lgob(self) -> DVector4: ...
    @property
    def lgoo(self) -> DVector4: ...
    @property
    def lgol(self) -> DVector4: ...
    @property
    def lglr(self) -> DVector4: ...
    @property
    def lglg(self) -> DVector4: ...
    @property
    def lglb(self) -> DVector4: ...
    @property
    def lglo(self) -> DVector4: ...
    @property
    def lgll(self) -> DVector4: ...
    @property
    def lbrr(self) -> DVector4: ...
    @property
    def lbrg(self) -> DVector4: ...
    @property
    def lbrb(self) -> DVector4: ...
    @property
    def lbro(self) -> DVector4: ...
    @property
    def lbrl(self) -> DVector4: ...
    @property
    def lbgr(self) -> DVector4: ...
    @property
    def lbgg(self) -> DVector4: ...
    @property
    def lbgb(self) -> DVector4: ...
    @property
    def lbgo(self) -> DVector4: ...
    @property
    def lbgl(self) -> DVector4: ...
    @property
    def lbbr(self) -> DVector4: ...
    @property
    def lbbg(self) -> DVector4: ...
    @property
    def lbbb(self) -> DVector4: ...
    @property
    def lbbo(self) -> DVector4: ...
    @property
    def lbbl(self) -> DVector4: ...
    @property
    def lbor(self) -> DVector4: ...
    @property
    def lbog(self) -> DVector4: ...
    @property
    def lbob(self) -> DVector4: ...
    @property
    def lboo(self) -> DVector4: ...
    @property
    def lbol(self) -> DVector4: ...
    @property
    def lblr(self) -> DVector4: ...
    @property
    def lblg(self) -> DVector4: ...
    @property
    def lblb(self) -> DVector4: ...
    @property
    def lblo(self) -> DVector4: ...
    @property
    def lbll(self) -> DVector4: ...
    @property
    def lorr(self) -> DVector4: ...
    @property
    def lorg(self) -> DVector4: ...
    @property
    def lorb(self) -> DVector4: ...
    @property
    def loro(self) -> DVector4: ...
    @property
    def lorl(self) -> DVector4: ...
    @property
    def logr(self) -> DVector4: ...
    @property
    def logg(self) -> DVector4: ...
    @property
    def logb(self) -> DVector4: ...
    @property
    def logo(self) -> DVector4: ...
    @property
    def logl(self) -> DVector4: ...
    @property
    def lobr(self) -> DVector4: ...
    @property
    def lobg(self) -> DVector4: ...
    @property
    def lobb(self) -> DVector4: ...
    @property
    def lobo(self) -> DVector4: ...
    @property
    def lobl(self) -> DVector4: ...
    @property
    def loor(self) -> DVector4: ...
    @property
    def loog(self) -> DVector4: ...
    @property
    def loob(self) -> DVector4: ...
    @property
    def lolr(self) -> DVector4: ...
    @property
    def lolg(self) -> DVector4: ...
    @property
    def lolb(self) -> DVector4: ...
    @property
    def llrr(self) -> DVector4: ...
    @property
    def llrg(self) -> DVector4: ...
    @property
    def llrb(self) -> DVector4: ...
    @property
    def llro(self) -> DVector4: ...
    @property
    def llrl(self) -> DVector4: ...
    @property
    def llgr(self) -> DVector4: ...
    @property
    def llgg(self) -> DVector4: ...
    @property
    def llgb(self) -> DVector4: ...
    @property
    def llgo(self) -> DVector4: ...
    @property
    def llgl(self) -> DVector4: ...
    @property
    def llbr(self) -> DVector4: ...
    @property
    def llbg(self) -> DVector4: ...
    @property
    def llbb(self) -> DVector4: ...
    @property
    def llbo(self) -> DVector4: ...
    @property
    def llbl(self) -> DVector4: ...
    @property
    def llor(self) -> DVector4: ...
    @property
    def llog(self) -> DVector4: ...
    @property
    def llob(self) -> DVector4: ...
    @property
    def lllr(self) -> DVector4: ...
    @property
    def lllg(self) -> DVector4: ...
    @property
    def lllb(self) -> DVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def t(self) -> float: ...
    @property
    def q(self) -> float: ...
    @property
    def ss(self) -> DVector2: ...
    @property
    def st(self) -> DVector2: ...
    @property
    def sq(self) -> DVector2: ...
    @property
    def so(self) -> DVector2: ...
    @property
    def sl(self) -> DVector2: ...
    @property
    def ts(self) -> DVector2: ...
    @property
    def tt(self) -> DVector2: ...
    @property
    def tq(self) -> DVector2: ...
    @property
    def to(self) -> DVector2: ...
    @property
    def tl(self) -> DVector2: ...
    @property
    def qs(self) -> DVector2: ...
    @property
    def qt(self) -> DVector2: ...
    @property
    def qq(self) -> DVector2: ...
    @property
    def qo(self) -> DVector2: ...
    @property
    def ql(self) -> DVector2: ...
    @property
    def os(self) -> DVector2: ...
    @property
    def ot(self) -> DVector2: ...
    @property
    def oq(self) -> DVector2: ...
    @property
    def ls(self) -> DVector2: ...
    @property
    def lt(self) -> DVector2: ...
    @property
    def lq(self) -> DVector2: ...
    @property
    def sss(self) -> DVector3: ...
    @property
    def sst(self) -> DVector3: ...
    @property
    def ssq(self) -> DVector3: ...
    @property
    def sso(self) -> DVector3: ...
    @property
    def ssl(self) -> DVector3: ...
    @property
    def sts(self) -> DVector3: ...
    @property
    def stt(self) -> DVector3: ...
    @property
    def stq(self) -> DVector3: ...
    @property
    def sto(self) -> DVector3: ...
    @property
    def stl(self) -> DVector3: ...
    @property
    def sqs(self) -> DVector3: ...
    @property
    def sqt(self) -> DVector3: ...
    @property
    def sqq(self) -> DVector3: ...
    @property
    def sqo(self) -> DVector3: ...
    @property
    def sql(self) -> DVector3: ...
    @property
    def sos(self) -> DVector3: ...
    @property
    def sot(self) -> DVector3: ...
    @property
    def soq(self) -> DVector3: ...
    @property
    def soo(self) -> DVector3: ...
    @property
    def sol(self) -> DVector3: ...
    @property
    def sls(self) -> DVector3: ...
    @property
    def slt(self) -> DVector3: ...
    @property
    def slq(self) -> DVector3: ...
    @property
    def slo(self) -> DVector3: ...
    @property
    def sll(self) -> DVector3: ...
    @property
    def tss(self) -> DVector3: ...
    @property
    def tst(self) -> DVector3: ...
    @property
    def tsq(self) -> DVector3: ...
    @property
    def tso(self) -> DVector3: ...
    @property
    def tsl(self) -> DVector3: ...
    @property
    def tts(self) -> DVector3: ...
    @property
    def ttt(self) -> DVector3: ...
    @property
    def ttq(self) -> DVector3: ...
    @property
    def tto(self) -> DVector3: ...
    @property
    def ttl(self) -> DVector3: ...
    @property
    def tqs(self) -> DVector3: ...
    @property
    def tqt(self) -> DVector3: ...
    @property
    def tqq(self) -> DVector3: ...
    @property
    def tqo(self) -> DVector3: ...
    @property
    def tql(self) -> DVector3: ...
    @property
    def tos(self) -> DVector3: ...
    @property
    def tot(self) -> DVector3: ...
    @property
    def toq(self) -> DVector3: ...
    @property
    def too(self) -> DVector3: ...
    @property
    def tol(self) -> DVector3: ...
    @property
    def tls(self) -> DVector3: ...
    @property
    def tlt(self) -> DVector3: ...
    @property
    def tlq(self) -> DVector3: ...
    @property
    def tlo(self) -> DVector3: ...
    @property
    def tll(self) -> DVector3: ...
    @property
    def qss(self) -> DVector3: ...
    @property
    def qst(self) -> DVector3: ...
    @property
    def qsq(self) -> DVector3: ...
    @property
    def qso(self) -> DVector3: ...
    @property
    def qsl(self) -> DVector3: ...
    @property
    def qts(self) -> DVector3: ...
    @property
    def qtt(self) -> DVector3: ...
    @property
    def qtq(self) -> DVector3: ...
    @property
    def qto(self) -> DVector3: ...
    @property
    def qtl(self) -> DVector3: ...
    @property
    def qqs(self) -> DVector3: ...
    @property
    def qqt(self) -> DVector3: ...
    @property
    def qqq(self) -> DVector3: ...
    @property
    def qqo(self) -> DVector3: ...
    @property
    def qql(self) -> DVector3: ...
    @property
    def qos(self) -> DVector3: ...
    @property
    def qot(self) -> DVector3: ...
    @property
    def qoq(self) -> DVector3: ...
    @property
    def qoo(self) -> DVector3: ...
    @property
    def qol(self) -> DVector3: ...
    @property
    def qls(self) -> DVector3: ...
    @property
    def qlt(self) -> DVector3: ...
    @property
    def qlq(self) -> DVector3: ...
    @property
    def qlo(self) -> DVector3: ...
    @property
    def qll(self) -> DVector3: ...
    @property
    def oss(self) -> DVector3: ...
    @property
    def ost(self) -> DVector3: ...
    @property
    def osq(self) -> DVector3: ...
    @property
    def oso(self) -> DVector3: ...
    @property
    def osl(self) -> DVector3: ...
    @property
    def ots(self) -> DVector3: ...
    @property
    def ott(self) -> DVector3: ...
    @property
    def otq(self) -> DVector3: ...
    @property
    def oto(self) -> DVector3: ...
    @property
    def otl(self) -> DVector3: ...
    @property
    def oqs(self) -> DVector3: ...
    @property
    def oqt(self) -> DVector3: ...
    @property
    def oqq(self) -> DVector3: ...
    @property
    def oqo(self) -> DVector3: ...
    @property
    def oql(self) -> DVector3: ...
    @property
    def oos(self) -> DVector3: ...
    @property
    def oot(self) -> DVector3: ...
    @property
    def ooq(self) -> DVector3: ...
    @property
    def ols(self) -> DVector3: ...
    @property
    def olt(self) -> DVector3: ...
    @property
    def olq(self) -> DVector3: ...
    @property
    def lss(self) -> DVector3: ...
    @property
    def lst(self) -> DVector3: ...
    @property
    def lsq(self) -> DVector3: ...
    @property
    def lso(self) -> DVector3: ...
    @property
    def lsl(self) -> DVector3: ...
    @property
    def lts(self) -> DVector3: ...
    @property
    def ltt(self) -> DVector3: ...
    @property
    def ltq(self) -> DVector3: ...
    @property
    def lto(self) -> DVector3: ...
    @property
    def ltl(self) -> DVector3: ...
    @property
    def lqs(self) -> DVector3: ...
    @property
    def lqt(self) -> DVector3: ...
    @property
    def lqq(self) -> DVector3: ...
    @property
    def lqo(self) -> DVector3: ...
    @property
    def lql(self) -> DVector3: ...
    @property
    def los(self) -> DVector3: ...
    @property
    def lot(self) -> DVector3: ...
    @property
    def loq(self) -> DVector3: ...
    @property
    def lls(self) -> DVector3: ...
    @property
    def llt(self) -> DVector3: ...
    @property
    def llq(self) -> DVector3: ...
    @property
    def ssss(self) -> DVector4: ...
    @property
    def ssst(self) -> DVector4: ...
    @property
    def sssq(self) -> DVector4: ...
    @property
    def ssso(self) -> DVector4: ...
    @property
    def sssl(self) -> DVector4: ...
    @property
    def ssts(self) -> DVector4: ...
    @property
    def sstt(self) -> DVector4: ...
    @property
    def sstq(self) -> DVector4: ...
    @property
    def ssto(self) -> DVector4: ...
    @property
    def sstl(self) -> DVector4: ...
    @property
    def ssqs(self) -> DVector4: ...
    @property
    def ssqt(self) -> DVector4: ...
    @property
    def ssqq(self) -> DVector4: ...
    @property
    def ssqo(self) -> DVector4: ...
    @property
    def ssql(self) -> DVector4: ...
    @property
    def ssos(self) -> DVector4: ...
    @property
    def ssot(self) -> DVector4: ...
    @property
    def ssoq(self) -> DVector4: ...
    @property
    def ssoo(self) -> DVector4: ...
    @property
    def ssol(self) -> DVector4: ...
    @property
    def ssls(self) -> DVector4: ...
    @property
    def sslt(self) -> DVector4: ...
    @property
    def sslq(self) -> DVector4: ...
    @property
    def sslo(self) -> DVector4: ...
    @property
    def ssll(self) -> DVector4: ...
    @property
    def stss(self) -> DVector4: ...
    @property
    def stst(self) -> DVector4: ...
    @property
    def stsq(self) -> DVector4: ...
    @property
    def stso(self) -> DVector4: ...
    @property
    def stsl(self) -> DVector4: ...
    @property
    def stts(self) -> DVector4: ...
    @property
    def sttt(self) -> DVector4: ...
    @property
    def sttq(self) -> DVector4: ...
    @property
    def stto(self) -> DVector4: ...
    @property
    def sttl(self) -> DVector4: ...
    @property
    def stqs(self) -> DVector4: ...
    @property
    def stqt(self) -> DVector4: ...
    @property
    def stqq(self) -> DVector4: ...
    @property
    def stqo(self) -> DVector4: ...
    @property
    def stql(self) -> DVector4: ...
    @property
    def stos(self) -> DVector4: ...
    @property
    def stot(self) -> DVector4: ...
    @property
    def stoq(self) -> DVector4: ...
    @property
    def stoo(self) -> DVector4: ...
    @property
    def stol(self) -> DVector4: ...
    @property
    def stls(self) -> DVector4: ...
    @property
    def stlt(self) -> DVector4: ...
    @property
    def stlq(self) -> DVector4: ...
    @property
    def stlo(self) -> DVector4: ...
    @property
    def stll(self) -> DVector4: ...
    @property
    def sqss(self) -> DVector4: ...
    @property
    def sqst(self) -> DVector4: ...
    @property
    def sqsq(self) -> DVector4: ...
    @property
    def sqso(self) -> DVector4: ...
    @property
    def sqsl(self) -> DVector4: ...
    @property
    def sqts(self) -> DVector4: ...
    @property
    def sqtt(self) -> DVector4: ...
    @property
    def sqtq(self) -> DVector4: ...
    @property
    def sqto(self) -> DVector4: ...
    @property
    def sqtl(self) -> DVector4: ...
    @property
    def sqqs(self) -> DVector4: ...
    @property
    def sqqt(self) -> DVector4: ...
    @property
    def sqqq(self) -> DVector4: ...
    @property
    def sqqo(self) -> DVector4: ...
    @property
    def sqql(self) -> DVector4: ...
    @property
    def sqos(self) -> DVector4: ...
    @property
    def sqot(self) -> DVector4: ...
    @property
    def sqoq(self) -> DVector4: ...
    @property
    def sqoo(self) -> DVector4: ...
    @property
    def sqol(self) -> DVector4: ...
    @property
    def sqls(self) -> DVector4: ...
    @property
    def sqlt(self) -> DVector4: ...
    @property
    def sqlq(self) -> DVector4: ...
    @property
    def sqlo(self) -> DVector4: ...
    @property
    def sqll(self) -> DVector4: ...
    @property
    def soss(self) -> DVector4: ...
    @property
    def sost(self) -> DVector4: ...
    @property
    def sosq(self) -> DVector4: ...
    @property
    def soso(self) -> DVector4: ...
    @property
    def sosl(self) -> DVector4: ...
    @property
    def sots(self) -> DVector4: ...
    @property
    def sott(self) -> DVector4: ...
    @property
    def sotq(self) -> DVector4: ...
    @property
    def soto(self) -> DVector4: ...
    @property
    def sotl(self) -> DVector4: ...
    @property
    def soqs(self) -> DVector4: ...
    @property
    def soqt(self) -> DVector4: ...
    @property
    def soqq(self) -> DVector4: ...
    @property
    def soqo(self) -> DVector4: ...
    @property
    def soql(self) -> DVector4: ...
    @property
    def soos(self) -> DVector4: ...
    @property
    def soot(self) -> DVector4: ...
    @property
    def sooq(self) -> DVector4: ...
    @property
    def sooo(self) -> DVector4: ...
    @property
    def sool(self) -> DVector4: ...
    @property
    def sols(self) -> DVector4: ...
    @property
    def solt(self) -> DVector4: ...
    @property
    def solq(self) -> DVector4: ...
    @property
    def solo(self) -> DVector4: ...
    @property
    def soll(self) -> DVector4: ...
    @property
    def slss(self) -> DVector4: ...
    @property
    def slst(self) -> DVector4: ...
    @property
    def slsq(self) -> DVector4: ...
    @property
    def slso(self) -> DVector4: ...
    @property
    def slsl(self) -> DVector4: ...
    @property
    def slts(self) -> DVector4: ...
    @property
    def sltt(self) -> DVector4: ...
    @property
    def sltq(self) -> DVector4: ...
    @property
    def slto(self) -> DVector4: ...
    @property
    def sltl(self) -> DVector4: ...
    @property
    def slqs(self) -> DVector4: ...
    @property
    def slqt(self) -> DVector4: ...
    @property
    def slqq(self) -> DVector4: ...
    @property
    def slqo(self) -> DVector4: ...
    @property
    def slql(self) -> DVector4: ...
    @property
    def slos(self) -> DVector4: ...
    @property
    def slot(self) -> DVector4: ...
    @property
    def sloq(self) -> DVector4: ...
    @property
    def sloo(self) -> DVector4: ...
    @property
    def slol(self) -> DVector4: ...
    @property
    def slls(self) -> DVector4: ...
    @property
    def sllt(self) -> DVector4: ...
    @property
    def sllq(self) -> DVector4: ...
    @property
    def sllo(self) -> DVector4: ...
    @property
    def slll(self) -> DVector4: ...
    @property
    def tsss(self) -> DVector4: ...
    @property
    def tsst(self) -> DVector4: ...
    @property
    def tssq(self) -> DVector4: ...
    @property
    def tsso(self) -> DVector4: ...
    @property
    def tssl(self) -> DVector4: ...
    @property
    def tsts(self) -> DVector4: ...
    @property
    def tstt(self) -> DVector4: ...
    @property
    def tstq(self) -> DVector4: ...
    @property
    def tsto(self) -> DVector4: ...
    @property
    def tstl(self) -> DVector4: ...
    @property
    def tsqs(self) -> DVector4: ...
    @property
    def tsqt(self) -> DVector4: ...
    @property
    def tsqq(self) -> DVector4: ...
    @property
    def tsqo(self) -> DVector4: ...
    @property
    def tsql(self) -> DVector4: ...
    @property
    def tsos(self) -> DVector4: ...
    @property
    def tsot(self) -> DVector4: ...
    @property
    def tsoq(self) -> DVector4: ...
    @property
    def tsoo(self) -> DVector4: ...
    @property
    def tsol(self) -> DVector4: ...
    @property
    def tsls(self) -> DVector4: ...
    @property
    def tslt(self) -> DVector4: ...
    @property
    def tslq(self) -> DVector4: ...
    @property
    def tslo(self) -> DVector4: ...
    @property
    def tsll(self) -> DVector4: ...
    @property
    def ttss(self) -> DVector4: ...
    @property
    def ttst(self) -> DVector4: ...
    @property
    def ttsq(self) -> DVector4: ...
    @property
    def ttso(self) -> DVector4: ...
    @property
    def ttsl(self) -> DVector4: ...
    @property
    def ttts(self) -> DVector4: ...
    @property
    def tttt(self) -> DVector4: ...
    @property
    def tttq(self) -> DVector4: ...
    @property
    def ttto(self) -> DVector4: ...
    @property
    def tttl(self) -> DVector4: ...
    @property
    def ttqs(self) -> DVector4: ...
    @property
    def ttqt(self) -> DVector4: ...
    @property
    def ttqq(self) -> DVector4: ...
    @property
    def ttqo(self) -> DVector4: ...
    @property
    def ttql(self) -> DVector4: ...
    @property
    def ttos(self) -> DVector4: ...
    @property
    def ttot(self) -> DVector4: ...
    @property
    def ttoq(self) -> DVector4: ...
    @property
    def ttoo(self) -> DVector4: ...
    @property
    def ttol(self) -> DVector4: ...
    @property
    def ttls(self) -> DVector4: ...
    @property
    def ttlt(self) -> DVector4: ...
    @property
    def ttlq(self) -> DVector4: ...
    @property
    def ttlo(self) -> DVector4: ...
    @property
    def ttll(self) -> DVector4: ...
    @property
    def tqss(self) -> DVector4: ...
    @property
    def tqst(self) -> DVector4: ...
    @property
    def tqsq(self) -> DVector4: ...
    @property
    def tqso(self) -> DVector4: ...
    @property
    def tqsl(self) -> DVector4: ...
    @property
    def tqts(self) -> DVector4: ...
    @property
    def tqtt(self) -> DVector4: ...
    @property
    def tqtq(self) -> DVector4: ...
    @property
    def tqto(self) -> DVector4: ...
    @property
    def tqtl(self) -> DVector4: ...
    @property
    def tqqs(self) -> DVector4: ...
    @property
    def tqqt(self) -> DVector4: ...
    @property
    def tqqq(self) -> DVector4: ...
    @property
    def tqqo(self) -> DVector4: ...
    @property
    def tqql(self) -> DVector4: ...
    @property
    def tqos(self) -> DVector4: ...
    @property
    def tqot(self) -> DVector4: ...
    @property
    def tqoq(self) -> DVector4: ...
    @property
    def tqoo(self) -> DVector4: ...
    @property
    def tqol(self) -> DVector4: ...
    @property
    def tqls(self) -> DVector4: ...
    @property
    def tqlt(self) -> DVector4: ...
    @property
    def tqlq(self) -> DVector4: ...
    @property
    def tqlo(self) -> DVector4: ...
    @property
    def tqll(self) -> DVector4: ...
    @property
    def toss(self) -> DVector4: ...
    @property
    def tost(self) -> DVector4: ...
    @property
    def tosq(self) -> DVector4: ...
    @property
    def toso(self) -> DVector4: ...
    @property
    def tosl(self) -> DVector4: ...
    @property
    def tots(self) -> DVector4: ...
    @property
    def tott(self) -> DVector4: ...
    @property
    def totq(self) -> DVector4: ...
    @property
    def toto(self) -> DVector4: ...
    @property
    def totl(self) -> DVector4: ...
    @property
    def toqs(self) -> DVector4: ...
    @property
    def toqt(self) -> DVector4: ...
    @property
    def toqq(self) -> DVector4: ...
    @property
    def toqo(self) -> DVector4: ...
    @property
    def toql(self) -> DVector4: ...
    @property
    def toos(self) -> DVector4: ...
    @property
    def toot(self) -> DVector4: ...
    @property
    def tooq(self) -> DVector4: ...
    @property
    def tooo(self) -> DVector4: ...
    @property
    def tool(self) -> DVector4: ...
    @property
    def tols(self) -> DVector4: ...
    @property
    def tolt(self) -> DVector4: ...
    @property
    def tolq(self) -> DVector4: ...
    @property
    def tolo(self) -> DVector4: ...
    @property
    def toll(self) -> DVector4: ...
    @property
    def tlss(self) -> DVector4: ...
    @property
    def tlst(self) -> DVector4: ...
    @property
    def tlsq(self) -> DVector4: ...
    @property
    def tlso(self) -> DVector4: ...
    @property
    def tlsl(self) -> DVector4: ...
    @property
    def tlts(self) -> DVector4: ...
    @property
    def tltt(self) -> DVector4: ...
    @property
    def tltq(self) -> DVector4: ...
    @property
    def tlto(self) -> DVector4: ...
    @property
    def tltl(self) -> DVector4: ...
    @property
    def tlqs(self) -> DVector4: ...
    @property
    def tlqt(self) -> DVector4: ...
    @property
    def tlqq(self) -> DVector4: ...
    @property
    def tlqo(self) -> DVector4: ...
    @property
    def tlql(self) -> DVector4: ...
    @property
    def tlos(self) -> DVector4: ...
    @property
    def tlot(self) -> DVector4: ...
    @property
    def tloq(self) -> DVector4: ...
    @property
    def tloo(self) -> DVector4: ...
    @property
    def tlol(self) -> DVector4: ...
    @property
    def tlls(self) -> DVector4: ...
    @property
    def tllt(self) -> DVector4: ...
    @property
    def tllq(self) -> DVector4: ...
    @property
    def tllo(self) -> DVector4: ...
    @property
    def tlll(self) -> DVector4: ...
    @property
    def qsss(self) -> DVector4: ...
    @property
    def qsst(self) -> DVector4: ...
    @property
    def qssq(self) -> DVector4: ...
    @property
    def qsso(self) -> DVector4: ...
    @property
    def qssl(self) -> DVector4: ...
    @property
    def qsts(self) -> DVector4: ...
    @property
    def qstt(self) -> DVector4: ...
    @property
    def qstq(self) -> DVector4: ...
    @property
    def qsto(self) -> DVector4: ...
    @property
    def qstl(self) -> DVector4: ...
    @property
    def qsqs(self) -> DVector4: ...
    @property
    def qsqt(self) -> DVector4: ...
    @property
    def qsqq(self) -> DVector4: ...
    @property
    def qsqo(self) -> DVector4: ...
    @property
    def qsql(self) -> DVector4: ...
    @property
    def qsos(self) -> DVector4: ...
    @property
    def qsot(self) -> DVector4: ...
    @property
    def qsoq(self) -> DVector4: ...
    @property
    def qsoo(self) -> DVector4: ...
    @property
    def qsol(self) -> DVector4: ...
    @property
    def qsls(self) -> DVector4: ...
    @property
    def qslt(self) -> DVector4: ...
    @property
    def qslq(self) -> DVector4: ...
    @property
    def qslo(self) -> DVector4: ...
    @property
    def qsll(self) -> DVector4: ...
    @property
    def qtss(self) -> DVector4: ...
    @property
    def qtst(self) -> DVector4: ...
    @property
    def qtsq(self) -> DVector4: ...
    @property
    def qtso(self) -> DVector4: ...
    @property
    def qtsl(self) -> DVector4: ...
    @property
    def qtts(self) -> DVector4: ...
    @property
    def qttt(self) -> DVector4: ...
    @property
    def qttq(self) -> DVector4: ...
    @property
    def qtto(self) -> DVector4: ...
    @property
    def qttl(self) -> DVector4: ...
    @property
    def qtqs(self) -> DVector4: ...
    @property
    def qtqt(self) -> DVector4: ...
    @property
    def qtqq(self) -> DVector4: ...
    @property
    def qtqo(self) -> DVector4: ...
    @property
    def qtql(self) -> DVector4: ...
    @property
    def qtos(self) -> DVector4: ...
    @property
    def qtot(self) -> DVector4: ...
    @property
    def qtoq(self) -> DVector4: ...
    @property
    def qtoo(self) -> DVector4: ...
    @property
    def qtol(self) -> DVector4: ...
    @property
    def qtls(self) -> DVector4: ...
    @property
    def qtlt(self) -> DVector4: ...
    @property
    def qtlq(self) -> DVector4: ...
    @property
    def qtlo(self) -> DVector4: ...
    @property
    def qtll(self) -> DVector4: ...
    @property
    def qqss(self) -> DVector4: ...
    @property
    def qqst(self) -> DVector4: ...
    @property
    def qqsq(self) -> DVector4: ...
    @property
    def qqso(self) -> DVector4: ...
    @property
    def qqsl(self) -> DVector4: ...
    @property
    def qqts(self) -> DVector4: ...
    @property
    def qqtt(self) -> DVector4: ...
    @property
    def qqtq(self) -> DVector4: ...
    @property
    def qqto(self) -> DVector4: ...
    @property
    def qqtl(self) -> DVector4: ...
    @property
    def qqqs(self) -> DVector4: ...
    @property
    def qqqt(self) -> DVector4: ...
    @property
    def qqqq(self) -> DVector4: ...
    @property
    def qqqo(self) -> DVector4: ...
    @property
    def qqql(self) -> DVector4: ...
    @property
    def qqos(self) -> DVector4: ...
    @property
    def qqot(self) -> DVector4: ...
    @property
    def qqoq(self) -> DVector4: ...
    @property
    def qqoo(self) -> DVector4: ...
    @property
    def qqol(self) -> DVector4: ...
    @property
    def qqls(self) -> DVector4: ...
    @property
    def qqlt(self) -> DVector4: ...
    @property
    def qqlq(self) -> DVector4: ...
    @property
    def qqlo(self) -> DVector4: ...
    @property
    def qqll(self) -> DVector4: ...
    @property
    def qoss(self) -> DVector4: ...
    @property
    def qost(self) -> DVector4: ...
    @property
    def qosq(self) -> DVector4: ...
    @property
    def qoso(self) -> DVector4: ...
    @property
    def qosl(self) -> DVector4: ...
    @property
    def qots(self) -> DVector4: ...
    @property
    def qott(self) -> DVector4: ...
    @property
    def qotq(self) -> DVector4: ...
    @property
    def qoto(self) -> DVector4: ...
    @property
    def qotl(self) -> DVector4: ...
    @property
    def qoqs(self) -> DVector4: ...
    @property
    def qoqt(self) -> DVector4: ...
    @property
    def qoqq(self) -> DVector4: ...
    @property
    def qoqo(self) -> DVector4: ...
    @property
    def qoql(self) -> DVector4: ...
    @property
    def qoos(self) -> DVector4: ...
    @property
    def qoot(self) -> DVector4: ...
    @property
    def qooq(self) -> DVector4: ...
    @property
    def qooo(self) -> DVector4: ...
    @property
    def qool(self) -> DVector4: ...
    @property
    def qols(self) -> DVector4: ...
    @property
    def qolt(self) -> DVector4: ...
    @property
    def qolq(self) -> DVector4: ...
    @property
    def qolo(self) -> DVector4: ...
    @property
    def qoll(self) -> DVector4: ...
    @property
    def qlss(self) -> DVector4: ...
    @property
    def qlst(self) -> DVector4: ...
    @property
    def qlsq(self) -> DVector4: ...
    @property
    def qlso(self) -> DVector4: ...
    @property
    def qlsl(self) -> DVector4: ...
    @property
    def qlts(self) -> DVector4: ...
    @property
    def qltt(self) -> DVector4: ...
    @property
    def qltq(self) -> DVector4: ...
    @property
    def qlto(self) -> DVector4: ...
    @property
    def qltl(self) -> DVector4: ...
    @property
    def qlqs(self) -> DVector4: ...
    @property
    def qlqt(self) -> DVector4: ...
    @property
    def qlqq(self) -> DVector4: ...
    @property
    def qlqo(self) -> DVector4: ...
    @property
    def qlql(self) -> DVector4: ...
    @property
    def qlos(self) -> DVector4: ...
    @property
    def qlot(self) -> DVector4: ...
    @property
    def qloq(self) -> DVector4: ...
    @property
    def qloo(self) -> DVector4: ...
    @property
    def qlol(self) -> DVector4: ...
    @property
    def qlls(self) -> DVector4: ...
    @property
    def qllt(self) -> DVector4: ...
    @property
    def qllq(self) -> DVector4: ...
    @property
    def qllo(self) -> DVector4: ...
    @property
    def qlll(self) -> DVector4: ...
    @property
    def osss(self) -> DVector4: ...
    @property
    def osst(self) -> DVector4: ...
    @property
    def ossq(self) -> DVector4: ...
    @property
    def osso(self) -> DVector4: ...
    @property
    def ossl(self) -> DVector4: ...
    @property
    def osts(self) -> DVector4: ...
    @property
    def ostt(self) -> DVector4: ...
    @property
    def ostq(self) -> DVector4: ...
    @property
    def osto(self) -> DVector4: ...
    @property
    def ostl(self) -> DVector4: ...
    @property
    def osqs(self) -> DVector4: ...
    @property
    def osqt(self) -> DVector4: ...
    @property
    def osqq(self) -> DVector4: ...
    @property
    def osqo(self) -> DVector4: ...
    @property
    def osql(self) -> DVector4: ...
    @property
    def osos(self) -> DVector4: ...
    @property
    def osot(self) -> DVector4: ...
    @property
    def osoq(self) -> DVector4: ...
    @property
    def osoo(self) -> DVector4: ...
    @property
    def osol(self) -> DVector4: ...
    @property
    def osls(self) -> DVector4: ...
    @property
    def oslt(self) -> DVector4: ...
    @property
    def oslq(self) -> DVector4: ...
    @property
    def oslo(self) -> DVector4: ...
    @property
    def osll(self) -> DVector4: ...
    @property
    def otss(self) -> DVector4: ...
    @property
    def otst(self) -> DVector4: ...
    @property
    def otsq(self) -> DVector4: ...
    @property
    def otso(self) -> DVector4: ...
    @property
    def otsl(self) -> DVector4: ...
    @property
    def otts(self) -> DVector4: ...
    @property
    def ottt(self) -> DVector4: ...
    @property
    def ottq(self) -> DVector4: ...
    @property
    def otto(self) -> DVector4: ...
    @property
    def ottl(self) -> DVector4: ...
    @property
    def otqs(self) -> DVector4: ...
    @property
    def otqt(self) -> DVector4: ...
    @property
    def otqq(self) -> DVector4: ...
    @property
    def otqo(self) -> DVector4: ...
    @property
    def otql(self) -> DVector4: ...
    @property
    def otos(self) -> DVector4: ...
    @property
    def otot(self) -> DVector4: ...
    @property
    def otoq(self) -> DVector4: ...
    @property
    def otoo(self) -> DVector4: ...
    @property
    def otol(self) -> DVector4: ...
    @property
    def otls(self) -> DVector4: ...
    @property
    def otlt(self) -> DVector4: ...
    @property
    def otlq(self) -> DVector4: ...
    @property
    def otlo(self) -> DVector4: ...
    @property
    def otll(self) -> DVector4: ...
    @property
    def oqss(self) -> DVector4: ...
    @property
    def oqst(self) -> DVector4: ...
    @property
    def oqsq(self) -> DVector4: ...
    @property
    def oqso(self) -> DVector4: ...
    @property
    def oqsl(self) -> DVector4: ...
    @property
    def oqts(self) -> DVector4: ...
    @property
    def oqtt(self) -> DVector4: ...
    @property
    def oqtq(self) -> DVector4: ...
    @property
    def oqto(self) -> DVector4: ...
    @property
    def oqtl(self) -> DVector4: ...
    @property
    def oqqs(self) -> DVector4: ...
    @property
    def oqqt(self) -> DVector4: ...
    @property
    def oqqq(self) -> DVector4: ...
    @property
    def oqqo(self) -> DVector4: ...
    @property
    def oqql(self) -> DVector4: ...
    @property
    def oqos(self) -> DVector4: ...
    @property
    def oqot(self) -> DVector4: ...
    @property
    def oqoq(self) -> DVector4: ...
    @property
    def oqoo(self) -> DVector4: ...
    @property
    def oqol(self) -> DVector4: ...
    @property
    def oqls(self) -> DVector4: ...
    @property
    def oqlt(self) -> DVector4: ...
    @property
    def oqlq(self) -> DVector4: ...
    @property
    def oqlo(self) -> DVector4: ...
    @property
    def oqll(self) -> DVector4: ...
    @property
    def ooss(self) -> DVector4: ...
    @property
    def oost(self) -> DVector4: ...
    @property
    def oosq(self) -> DVector4: ...
    @property
    def ooso(self) -> DVector4: ...
    @property
    def oosl(self) -> DVector4: ...
    @property
    def oots(self) -> DVector4: ...
    @property
    def oott(self) -> DVector4: ...
    @property
    def ootq(self) -> DVector4: ...
    @property
    def ooto(self) -> DVector4: ...
    @property
    def ootl(self) -> DVector4: ...
    @property
    def ooqs(self) -> DVector4: ...
    @property
    def ooqt(self) -> DVector4: ...
    @property
    def ooqq(self) -> DVector4: ...
    @property
    def ooqo(self) -> DVector4: ...
    @property
    def ooql(self) -> DVector4: ...
    @property
    def ooos(self) -> DVector4: ...
    @property
    def ooot(self) -> DVector4: ...
    @property
    def oooq(self) -> DVector4: ...
    @property
    def ools(self) -> DVector4: ...
    @property
    def oolt(self) -> DVector4: ...
    @property
    def oolq(self) -> DVector4: ...
    @property
    def olss(self) -> DVector4: ...
    @property
    def olst(self) -> DVector4: ...
    @property
    def olsq(self) -> DVector4: ...
    @property
    def olso(self) -> DVector4: ...
    @property
    def olsl(self) -> DVector4: ...
    @property
    def olts(self) -> DVector4: ...
    @property
    def oltt(self) -> DVector4: ...
    @property
    def oltq(self) -> DVector4: ...
    @property
    def olto(self) -> DVector4: ...
    @property
    def oltl(self) -> DVector4: ...
    @property
    def olqs(self) -> DVector4: ...
    @property
    def olqt(self) -> DVector4: ...
    @property
    def olqq(self) -> DVector4: ...
    @property
    def olqo(self) -> DVector4: ...
    @property
    def olql(self) -> DVector4: ...
    @property
    def olos(self) -> DVector4: ...
    @property
    def olot(self) -> DVector4: ...
    @property
    def oloq(self) -> DVector4: ...
    @property
    def olls(self) -> DVector4: ...
    @property
    def ollt(self) -> DVector4: ...
    @property
    def ollq(self) -> DVector4: ...
    @property
    def lsss(self) -> DVector4: ...
    @property
    def lsst(self) -> DVector4: ...
    @property
    def lssq(self) -> DVector4: ...
    @property
    def lsso(self) -> DVector4: ...
    @property
    def lssl(self) -> DVector4: ...
    @property
    def lsts(self) -> DVector4: ...
    @property
    def lstt(self) -> DVector4: ...
    @property
    def lstq(self) -> DVector4: ...
    @property
    def lsto(self) -> DVector4: ...
    @property
    def lstl(self) -> DVector4: ...
    @property
    def lsqs(self) -> DVector4: ...
    @property
    def lsqt(self) -> DVector4: ...
    @property
    def lsqq(self) -> DVector4: ...
    @property
    def lsqo(self) -> DVector4: ...
    @property
    def lsql(self) -> DVector4: ...
    @property
    def lsos(self) -> DVector4: ...
    @property
    def lsot(self) -> DVector4: ...
    @property
    def lsoq(self) -> DVector4: ...
    @property
    def lsoo(self) -> DVector4: ...
    @property
    def lsol(self) -> DVector4: ...
    @property
    def lsls(self) -> DVector4: ...
    @property
    def lslt(self) -> DVector4: ...
    @property
    def lslq(self) -> DVector4: ...
    @property
    def lslo(self) -> DVector4: ...
    @property
    def lsll(self) -> DVector4: ...
    @property
    def ltss(self) -> DVector4: ...
    @property
    def ltst(self) -> DVector4: ...
    @property
    def ltsq(self) -> DVector4: ...
    @property
    def ltso(self) -> DVector4: ...
    @property
    def ltsl(self) -> DVector4: ...
    @property
    def ltts(self) -> DVector4: ...
    @property
    def lttt(self) -> DVector4: ...
    @property
    def lttq(self) -> DVector4: ...
    @property
    def ltto(self) -> DVector4: ...
    @property
    def lttl(self) -> DVector4: ...
    @property
    def ltqs(self) -> DVector4: ...
    @property
    def ltqt(self) -> DVector4: ...
    @property
    def ltqq(self) -> DVector4: ...
    @property
    def ltqo(self) -> DVector4: ...
    @property
    def ltql(self) -> DVector4: ...
    @property
    def ltos(self) -> DVector4: ...
    @property
    def ltot(self) -> DVector4: ...
    @property
    def ltoq(self) -> DVector4: ...
    @property
    def ltoo(self) -> DVector4: ...
    @property
    def ltol(self) -> DVector4: ...
    @property
    def ltls(self) -> DVector4: ...
    @property
    def ltlt(self) -> DVector4: ...
    @property
    def ltlq(self) -> DVector4: ...
    @property
    def ltlo(self) -> DVector4: ...
    @property
    def ltll(self) -> DVector4: ...
    @property
    def lqss(self) -> DVector4: ...
    @property
    def lqst(self) -> DVector4: ...
    @property
    def lqsq(self) -> DVector4: ...
    @property
    def lqso(self) -> DVector4: ...
    @property
    def lqsl(self) -> DVector4: ...
    @property
    def lqts(self) -> DVector4: ...
    @property
    def lqtt(self) -> DVector4: ...
    @property
    def lqtq(self) -> DVector4: ...
    @property
    def lqto(self) -> DVector4: ...
    @property
    def lqtl(self) -> DVector4: ...
    @property
    def lqqs(self) -> DVector4: ...
    @property
    def lqqt(self) -> DVector4: ...
    @property
    def lqqq(self) -> DVector4: ...
    @property
    def lqqo(self) -> DVector4: ...
    @property
    def lqql(self) -> DVector4: ...
    @property
    def lqos(self) -> DVector4: ...
    @property
    def lqot(self) -> DVector4: ...
    @property
    def lqoq(self) -> DVector4: ...
    @property
    def lqoo(self) -> DVector4: ...
    @property
    def lqol(self) -> DVector4: ...
    @property
    def lqls(self) -> DVector4: ...
    @property
    def lqlt(self) -> DVector4: ...
    @property
    def lqlq(self) -> DVector4: ...
    @property
    def lqlo(self) -> DVector4: ...
    @property
    def lqll(self) -> DVector4: ...
    @property
    def loss(self) -> DVector4: ...
    @property
    def lost(self) -> DVector4: ...
    @property
    def losq(self) -> DVector4: ...
    @property
    def loso(self) -> DVector4: ...
    @property
    def losl(self) -> DVector4: ...
    @property
    def lots(self) -> DVector4: ...
    @property
    def lott(self) -> DVector4: ...
    @property
    def lotq(self) -> DVector4: ...
    @property
    def loto(self) -> DVector4: ...
    @property
    def lotl(self) -> DVector4: ...
    @property
    def loqs(self) -> DVector4: ...
    @property
    def loqt(self) -> DVector4: ...
    @property
    def loqq(self) -> DVector4: ...
    @property
    def loqo(self) -> DVector4: ...
    @property
    def loql(self) -> DVector4: ...
    @property
    def loos(self) -> DVector4: ...
    @property
    def loot(self) -> DVector4: ...
    @property
    def looq(self) -> DVector4: ...
    @property
    def lols(self) -> DVector4: ...
    @property
    def lolt(self) -> DVector4: ...
    @property
    def lolq(self) -> DVector4: ...
    @property
    def llss(self) -> DVector4: ...
    @property
    def llst(self) -> DVector4: ...
    @property
    def llsq(self) -> DVector4: ...
    @property
    def llso(self) -> DVector4: ...
    @property
    def llsl(self) -> DVector4: ...
    @property
    def llts(self) -> DVector4: ...
    @property
    def lltt(self) -> DVector4: ...
    @property
    def lltq(self) -> DVector4: ...
    @property
    def llto(self) -> DVector4: ...
    @property
    def lltl(self) -> DVector4: ...
    @property
    def llqs(self) -> DVector4: ...
    @property
    def llqt(self) -> DVector4: ...
    @property
    def llqq(self) -> DVector4: ...
    @property
    def llqo(self) -> DVector4: ...
    @property
    def llql(self) -> DVector4: ...
    @property
    def llos(self) -> DVector4: ...
    @property
    def llot(self) -> DVector4: ...
    @property
    def lloq(self) -> DVector4: ...
    @property
    def llls(self) -> DVector4: ...
    @property
    def lllt(self) -> DVector4: ...
    @property
    def lllq(self) -> DVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def v(self) -> float: ...
    @property
    def uu(self) -> DVector2: ...
    @property
    def uv(self) -> DVector2: ...
    @property
    def uo(self) -> DVector2: ...
    @property
    def ul(self) -> DVector2: ...
    @property
    def vu(self) -> DVector2: ...
    @property
    def vv(self) -> DVector2: ...
    @property
    def vo(self) -> DVector2: ...
    @property
    def vl(self) -> DVector2: ...
    @property
    def ou(self) -> DVector2: ...
    @property
    def ov(self) -> DVector2: ...
    @property
    def lu(self) -> DVector2: ...
    @property
    def lv(self) -> DVector2: ...
    @property
    def uuu(self) -> DVector3: ...
    @property
    def uuv(self) -> DVector3: ...
    @property
    def uuo(self) -> DVector3: ...
    @property
    def uul(self) -> DVector3: ...
    @property
    def uvu(self) -> DVector3: ...
    @property
    def uvv(self) -> DVector3: ...
    @property
    def uvo(self) -> DVector3: ...
    @property
    def uvl(self) -> DVector3: ...
    @property
    def uou(self) -> DVector3: ...
    @property
    def uov(self) -> DVector3: ...
    @property
    def uoo(self) -> DVector3: ...
    @property
    def uol(self) -> DVector3: ...
    @property
    def ulu(self) -> DVector3: ...
    @property
    def ulv(self) -> DVector3: ...
    @property
    def ulo(self) -> DVector3: ...
    @property
    def ull(self) -> DVector3: ...
    @property
    def vuu(self) -> DVector3: ...
    @property
    def vuv(self) -> DVector3: ...
    @property
    def vuo(self) -> DVector3: ...
    @property
    def vul(self) -> DVector3: ...
    @property
    def vvu(self) -> DVector3: ...
    @property
    def vvv(self) -> DVector3: ...
    @property
    def vvo(self) -> DVector3: ...
    @property
    def vvl(self) -> DVector3: ...
    @property
    def vou(self) -> DVector3: ...
    @property
    def vov(self) -> DVector3: ...
    @property
    def voo(self) -> DVector3: ...
    @property
    def vol(self) -> DVector3: ...
    @property
    def vlu(self) -> DVector3: ...
    @property
    def vlv(self) -> DVector3: ...
    @property
    def vlo(self) -> DVector3: ...
    @property
    def vll(self) -> DVector3: ...
    @property
    def ouu(self) -> DVector3: ...
    @property
    def ouv(self) -> DVector3: ...
    @property
    def ouo(self) -> DVector3: ...
    @property
    def oul(self) -> DVector3: ...
    @property
    def ovu(self) -> DVector3: ...
    @property
    def ovv(self) -> DVector3: ...
    @property
    def ovo(self) -> DVector3: ...
    @property
    def ovl(self) -> DVector3: ...
    @property
    def oou(self) -> DVector3: ...
    @property
    def oov(self) -> DVector3: ...
    @property
    def olu(self) -> DVector3: ...
    @property
    def olv(self) -> DVector3: ...
    @property
    def luu(self) -> DVector3: ...
    @property
    def luv(self) -> DVector3: ...
    @property
    def luo(self) -> DVector3: ...
    @property
    def lul(self) -> DVector3: ...
    @property
    def lvu(self) -> DVector3: ...
    @property
    def lvv(self) -> DVector3: ...
    @property
    def lvo(self) -> DVector3: ...
    @property
    def lvl(self) -> DVector3: ...
    @property
    def lou(self) -> DVector3: ...
    @property
    def lov(self) -> DVector3: ...
    @property
    def llu(self) -> DVector3: ...
    @property
    def llv(self) -> DVector3: ...
    @property
    def uuuu(self) -> DVector4: ...
    @property
    def uuuv(self) -> DVector4: ...
    @property
    def uuuo(self) -> DVector4: ...
    @property
    def uuul(self) -> DVector4: ...
    @property
    def uuvu(self) -> DVector4: ...
    @property
    def uuvv(self) -> DVector4: ...
    @property
    def uuvo(self) -> DVector4: ...
    @property
    def uuvl(self) -> DVector4: ...
    @property
    def uuou(self) -> DVector4: ...
    @property
    def uuov(self) -> DVector4: ...
    @property
    def uuoo(self) -> DVector4: ...
    @property
    def uuol(self) -> DVector4: ...
    @property
    def uulu(self) -> DVector4: ...
    @property
    def uulv(self) -> DVector4: ...
    @property
    def uulo(self) -> DVector4: ...
    @property
    def uull(self) -> DVector4: ...
    @property
    def uvuu(self) -> DVector4: ...
    @property
    def uvuv(self) -> DVector4: ...
    @property
    def uvuo(self) -> DVector4: ...
    @property
    def uvul(self) -> DVector4: ...
    @property
    def uvvu(self) -> DVector4: ...
    @property
    def uvvv(self) -> DVector4: ...
    @property
    def uvvo(self) -> DVector4: ...
    @property
    def uvvl(self) -> DVector4: ...
    @property
    def uvou(self) -> DVector4: ...
    @property
    def uvov(self) -> DVector4: ...
    @property
    def uvoo(self) -> DVector4: ...
    @property
    def uvol(self) -> DVector4: ...
    @property
    def uvlu(self) -> DVector4: ...
    @property
    def uvlv(self) -> DVector4: ...
    @property
    def uvlo(self) -> DVector4: ...
    @property
    def uvll(self) -> DVector4: ...
    @property
    def uouu(self) -> DVector4: ...
    @property
    def uouv(self) -> DVector4: ...
    @property
    def uouo(self) -> DVector4: ...
    @property
    def uoul(self) -> DVector4: ...
    @property
    def uovu(self) -> DVector4: ...
    @property
    def uovv(self) -> DVector4: ...
    @property
    def uovo(self) -> DVector4: ...
    @property
    def uovl(self) -> DVector4: ...
    @property
    def uoou(self) -> DVector4: ...
    @property
    def uoov(self) -> DVector4: ...
    @property
    def uooo(self) -> DVector4: ...
    @property
    def uool(self) -> DVector4: ...
    @property
    def uolu(self) -> DVector4: ...
    @property
    def uolv(self) -> DVector4: ...
    @property
    def uolo(self) -> DVector4: ...
    @property
    def uoll(self) -> DVector4: ...
    @property
    def uluu(self) -> DVector4: ...
    @property
    def uluv(self) -> DVector4: ...
    @property
    def uluo(self) -> DVector4: ...
    @property
    def ulul(self) -> DVector4: ...
    @property
    def ulvu(self) -> DVector4: ...
    @property
    def ulvv(self) -> DVector4: ...
    @property
    def ulvo(self) -> DVector4: ...
    @property
    def ulvl(self) -> DVector4: ...
    @property
    def ulou(self) -> DVector4: ...
    @property
    def ulov(self) -> DVector4: ...
    @property
    def uloo(self) -> DVector4: ...
    @property
    def ulol(self) -> DVector4: ...
    @property
    def ullu(self) -> DVector4: ...
    @property
    def ullv(self) -> DVector4: ...
    @property
    def ullo(self) -> DVector4: ...
    @property
    def ulll(self) -> DVector4: ...
    @property
    def vuuu(self) -> DVector4: ...
    @property
    def vuuv(self) -> DVector4: ...
    @property
    def vuuo(self) -> DVector4: ...
    @property
    def vuul(self) -> DVector4: ...
    @property
    def vuvu(self) -> DVector4: ...
    @property
    def vuvv(self) -> DVector4: ...
    @property
    def vuvo(self) -> DVector4: ...
    @property
    def vuvl(self) -> DVector4: ...
    @property
    def vuou(self) -> DVector4: ...
    @property
    def vuov(self) -> DVector4: ...
    @property
    def vuoo(self) -> DVector4: ...
    @property
    def vuol(self) -> DVector4: ...
    @property
    def vulu(self) -> DVector4: ...
    @property
    def vulv(self) -> DVector4: ...
    @property
    def vulo(self) -> DVector4: ...
    @property
    def vull(self) -> DVector4: ...
    @property
    def vvuu(self) -> DVector4: ...
    @property
    def vvuv(self) -> DVector4: ...
    @property
    def vvuo(self) -> DVector4: ...
    @property
    def vvul(self) -> DVector4: ...
    @property
    def vvvu(self) -> DVector4: ...
    @property
    def vvvv(self) -> DVector4: ...
    @property
    def vvvo(self) -> DVector4: ...
    @property
    def vvvl(self) -> DVector4: ...
    @property
    def vvou(self) -> DVector4: ...
    @property
    def vvov(self) -> DVector4: ...
    @property
    def vvoo(self) -> DVector4: ...
    @property
    def vvol(self) -> DVector4: ...
    @property
    def vvlu(self) -> DVector4: ...
    @property
    def vvlv(self) -> DVector4: ...
    @property
    def vvlo(self) -> DVector4: ...
    @property
    def vvll(self) -> DVector4: ...
    @property
    def vouu(self) -> DVector4: ...
    @property
    def vouv(self) -> DVector4: ...
    @property
    def vouo(self) -> DVector4: ...
    @property
    def voul(self) -> DVector4: ...
    @property
    def vovu(self) -> DVector4: ...
    @property
    def vovv(self) -> DVector4: ...
    @property
    def vovo(self) -> DVector4: ...
    @property
    def vovl(self) -> DVector4: ...
    @property
    def voou(self) -> DVector4: ...
    @property
    def voov(self) -> DVector4: ...
    @property
    def vooo(self) -> DVector4: ...
    @property
    def vool(self) -> DVector4: ...
    @property
    def volu(self) -> DVector4: ...
    @property
    def volv(self) -> DVector4: ...
    @property
    def volo(self) -> DVector4: ...
    @property
    def voll(self) -> DVector4: ...
    @property
    def vluu(self) -> DVector4: ...
    @property
    def vluv(self) -> DVector4: ...
    @property
    def vluo(self) -> DVector4: ...
    @property
    def vlul(self) -> DVector4: ...
    @property
    def vlvu(self) -> DVector4: ...
    @property
    def vlvv(self) -> DVector4: ...
    @property
    def vlvo(self) -> DVector4: ...
    @property
    def vlvl(self) -> DVector4: ...
    @property
    def vlou(self) -> DVector4: ...
    @property
    def vlov(self) -> DVector4: ...
    @property
    def vloo(self) -> DVector4: ...
    @property
    def vlol(self) -> DVector4: ...
    @property
    def vllu(self) -> DVector4: ...
    @property
    def vllv(self) -> DVector4: ...
    @property
    def vllo(self) -> DVector4: ...
    @property
    def vlll(self) -> DVector4: ...
    @property
    def ouuu(self) -> DVector4: ...
    @property
    def ouuv(self) -> DVector4: ...
    @property
    def ouuo(self) -> DVector4: ...
    @property
    def ouul(self) -> DVector4: ...
    @property
    def ouvu(self) -> DVector4: ...
    @property
    def ouvv(self) -> DVector4: ...
    @property
    def ouvo(self) -> DVector4: ...
    @property
    def ouvl(self) -> DVector4: ...
    @property
    def ouou(self) -> DVector4: ...
    @property
    def ouov(self) -> DVector4: ...
    @property
    def ouoo(self) -> DVector4: ...
    @property
    def ouol(self) -> DVector4: ...
    @property
    def oulu(self) -> DVector4: ...
    @property
    def oulv(self) -> DVector4: ...
    @property
    def oulo(self) -> DVector4: ...
    @property
    def oull(self) -> DVector4: ...
    @property
    def ovuu(self) -> DVector4: ...
    @property
    def ovuv(self) -> DVector4: ...
    @property
    def ovuo(self) -> DVector4: ...
    @property
    def ovul(self) -> DVector4: ...
    @property
    def ovvu(self) -> DVector4: ...
    @property
    def ovvv(self) -> DVector4: ...
    @property
    def ovvo(self) -> DVector4: ...
    @property
    def ovvl(self) -> DVector4: ...
    @property
    def ovou(self) -> DVector4: ...
    @property
    def ovov(self) -> DVector4: ...
    @property
    def ovoo(self) -> DVector4: ...
    @property
    def ovol(self) -> DVector4: ...
    @property
    def ovlu(self) -> DVector4: ...
    @property
    def ovlv(self) -> DVector4: ...
    @property
    def ovlo(self) -> DVector4: ...
    @property
    def ovll(self) -> DVector4: ...
    @property
    def oouu(self) -> DVector4: ...
    @property
    def oouv(self) -> DVector4: ...
    @property
    def oouo(self) -> DVector4: ...
    @property
    def ooul(self) -> DVector4: ...
    @property
    def oovu(self) -> DVector4: ...
    @property
    def oovv(self) -> DVector4: ...
    @property
    def oovo(self) -> DVector4: ...
    @property
    def oovl(self) -> DVector4: ...
    @property
    def ooou(self) -> DVector4: ...
    @property
    def ooov(self) -> DVector4: ...
    @property
    def oolu(self) -> DVector4: ...
    @property
    def oolv(self) -> DVector4: ...
    @property
    def oluu(self) -> DVector4: ...
    @property
    def oluv(self) -> DVector4: ...
    @property
    def oluo(self) -> DVector4: ...
    @property
    def olul(self) -> DVector4: ...
    @property
    def olvu(self) -> DVector4: ...
    @property
    def olvv(self) -> DVector4: ...
    @property
    def olvo(self) -> DVector4: ...
    @property
    def olvl(self) -> DVector4: ...
    @property
    def olou(self) -> DVector4: ...
    @property
    def olov(self) -> DVector4: ...
    @property
    def ollu(self) -> DVector4: ...
    @property
    def ollv(self) -> DVector4: ...
    @property
    def luuu(self) -> DVector4: ...
    @property
    def luuv(self) -> DVector4: ...
    @property
    def luuo(self) -> DVector4: ...
    @property
    def luul(self) -> DVector4: ...
    @property
    def luvu(self) -> DVector4: ...
    @property
    def luvv(self) -> DVector4: ...
    @property
    def luvo(self) -> DVector4: ...
    @property
    def luvl(self) -> DVector4: ...
    @property
    def luou(self) -> DVector4: ...
    @property
    def luov(self) -> DVector4: ...
    @property
    def luoo(self) -> DVector4: ...
    @property
    def luol(self) -> DVector4: ...
    @property
    def lulu(self) -> DVector4: ...
    @property
    def lulv(self) -> DVector4: ...
    @property
    def lulo(self) -> DVector4: ...
    @property
    def lull(self) -> DVector4: ...
    @property
    def lvuu(self) -> DVector4: ...
    @property
    def lvuv(self) -> DVector4: ...
    @property
    def lvuo(self) -> DVector4: ...
    @property
    def lvul(self) -> DVector4: ...
    @property
    def lvvu(self) -> DVector4: ...
    @property
    def lvvv(self) -> DVector4: ...
    @property
    def lvvo(self) -> DVector4: ...
    @property
    def lvvl(self) -> DVector4: ...
    @property
    def lvou(self) -> DVector4: ...
    @property
    def lvov(self) -> DVector4: ...
    @property
    def lvoo(self) -> DVector4: ...
    @property
    def lvol(self) -> DVector4: ...
    @property
    def lvlu(self) -> DVector4: ...
    @property
    def lvlv(self) -> DVector4: ...
    @property
    def lvlo(self) -> DVector4: ...
    @property
    def lvll(self) -> DVector4: ...
    @property
    def louu(self) -> DVector4: ...
    @property
    def louv(self) -> DVector4: ...
    @property
    def louo(self) -> DVector4: ...
    @property
    def loul(self) -> DVector4: ...
    @property
    def lovu(self) -> DVector4: ...
    @property
    def lovv(self) -> DVector4: ...
    @property
    def lovo(self) -> DVector4: ...
    @property
    def lovl(self) -> DVector4: ...
    @property
    def loou(self) -> DVector4: ...
    @property
    def loov(self) -> DVector4: ...
    @property
    def lolu(self) -> DVector4: ...
    @property
    def lolv(self) -> DVector4: ...
    @property
    def lluu(self) -> DVector4: ...
    @property
    def lluv(self) -> DVector4: ...
    @property
    def lluo(self) -> DVector4: ...
    @property
    def llul(self) -> DVector4: ...
    @property
    def llvu(self) -> DVector4: ...
    @property
    def llvv(self) -> DVector4: ...
    @property
    def llvo(self) -> DVector4: ...
    @property
    def llvl(self) -> DVector4: ...
    @property
    def llou(self) -> DVector4: ...
    @property
    def llov(self) -> DVector4: ...
    @property
    def lllu(self) -> DVector4: ...
    @property
    def lllv(self) -> DVector4: ...
    @property
    def oo(self) -> DVector2: ...
    @property
    def ol(self) -> DVector2: ...
    @property
    def lo(self) -> DVector2: ...
    @property
    def ll(self) -> DVector2: ...
    @property
    def ooo(self) -> DVector3: ...
    @property
    def ool(self) -> DVector3: ...
    @property
    def olo(self) -> DVector3: ...
    @property
    def oll(self) -> DVector3: ...
    @property
    def loo(self) -> DVector3: ...
    @property
    def lol(self) -> DVector3: ...
    @property
    def llo(self) -> DVector3: ...
    @property
    def lll(self) -> DVector3: ...
    @property
    def oooo(self) -> DVector4: ...
    @property
    def oool(self) -> DVector4: ...
    @property
    def oolo(self) -> DVector4: ...
    @property
    def ooll(self) -> DVector4: ...
    @property
    def oloo(self) -> DVector4: ...
    @property
    def olol(self) -> DVector4: ...
    @property
    def ollo(self) -> DVector4: ...
    @property
    def olll(self) -> DVector4: ...
    @property
    def looo(self) -> DVector4: ...
    @property
    def lool(self) -> DVector4: ...
    @property
    def lolo(self) -> DVector4: ...
    @property
    def loll(self) -> DVector4: ...
    @property
    def lloo(self) -> DVector4: ...
    @property
    def llol(self) -> DVector4: ...
    @property
    def lllo(self) -> DVector4: ...
    @property
    def llll(self) -> DVector4: ...
    @property
    def magnitude(self) -> float: ...
    def cross(self, other: DVector3, /) -> DVector3: ...
    def to_quaternion(self) -> DQuaternion: ...
    def normalize(self) -> DVector3: ...
    def distance(self, other: DVector3, /) -> float: ...
    def lerp(self, other: DVector3, x: float, /) -> DVector3: ...
    def min(self, n: Number, /) -> DVector3: ...
    def max(self, n: Number, /) -> DVector3: ...
    def clamp(self, min: Number, max: Number, /) -> DVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[DVector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector3: ...

@final
class DVector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: DVector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector3]: ...
    @overload
    def __getitem__(self, index: int) -> DVector3: ...
    @overload
    def __getitem__(self, index: slice) -> DVector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[DVector3]: ...

@final
class FVector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FVector3) -> FVector3: ...
    @overload
    def __add__(self, other: Number) -> FVector3: ...
    @overload
    def __radd__(self, other: FVector3) -> FVector3: ...
    @overload
    def __radd__(self, other: Number) -> FVector3: ...
    @overload
    def __sub__(self, other: FVector3) -> FVector3: ...
    @overload
    def __sub__(self, other: Number) -> FVector3: ...
    @overload
    def __rsub__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rsub__(self, other: Number) -> FVector3: ...
    @overload
    def __mul__(self, other: FVector3) -> FVector3: ...
    @overload
    def __mul__(self, other: Number) -> FVector3: ...
    @overload
    def __rmul__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rmul__(self, other: Number) -> FVector3: ...
    def __matmul__(self, other: FVector3) -> float: ...
    def __rmatmul__(self, other: FVector3) -> float: ...
    @overload
    def __mod__(self, other: FVector3) -> FVector3: ...
    @overload
    def __mod__(self, other: Number) -> FVector3: ...
    @overload
    def __rmod__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rmod__(self, other: Number) -> FVector3: ...
    @overload
    def __pow__(self, other: FVector3) -> FVector3: ...
    @overload
    def __pow__(self, other: Number) -> FVector3: ...
    @overload
    def __rpow__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rpow__(self, other: Number) -> FVector3: ...
    @overload
    def __truediv__(self, other: FVector3) -> FVector3: ...
    @overload
    def __truediv__(self, other: Number) -> FVector3: ...
    @overload
    def __rtruediv__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> FVector3: ...
    def __neg__(self) -> FVector3: ...
    def __abs__(self) -> FVector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float: ...
    @property
    def xx(self) -> FVector2: ...
    @property
    def xy(self) -> FVector2: ...
    @property
    def xz(self) -> FVector2: ...
    @property
    def xo(self) -> FVector2: ...
    @property
    def xl(self) -> FVector2: ...
    @property
    def yx(self) -> FVector2: ...
    @property
    def yy(self) -> FVector2: ...
    @property
    def yz(self) -> FVector2: ...
    @property
    def yo(self) -> FVector2: ...
    @property
    def yl(self) -> FVector2: ...
    @property
    def zx(self) -> FVector2: ...
    @property
    def zy(self) -> FVector2: ...
    @property
    def zz(self) -> FVector2: ...
    @property
    def zo(self) -> FVector2: ...
    @property
    def zl(self) -> FVector2: ...
    @property
    def ox(self) -> FVector2: ...
    @property
    def oy(self) -> FVector2: ...
    @property
    def oz(self) -> FVector2: ...
    @property
    def lx(self) -> FVector2: ...
    @property
    def ly(self) -> FVector2: ...
    @property
    def lz(self) -> FVector2: ...
    @property
    def xxx(self) -> FVector3: ...
    @property
    def xxy(self) -> FVector3: ...
    @property
    def xxz(self) -> FVector3: ...
    @property
    def xxo(self) -> FVector3: ...
    @property
    def xxl(self) -> FVector3: ...
    @property
    def xyx(self) -> FVector3: ...
    @property
    def xyy(self) -> FVector3: ...
    @property
    def xyz(self) -> FVector3: ...
    @property
    def xyo(self) -> FVector3: ...
    @property
    def xyl(self) -> FVector3: ...
    @property
    def xzx(self) -> FVector3: ...
    @property
    def xzy(self) -> FVector3: ...
    @property
    def xzz(self) -> FVector3: ...
    @property
    def xzo(self) -> FVector3: ...
    @property
    def xzl(self) -> FVector3: ...
    @property
    def xox(self) -> FVector3: ...
    @property
    def xoy(self) -> FVector3: ...
    @property
    def xoz(self) -> FVector3: ...
    @property
    def xoo(self) -> FVector3: ...
    @property
    def xol(self) -> FVector3: ...
    @property
    def xlx(self) -> FVector3: ...
    @property
    def xly(self) -> FVector3: ...
    @property
    def xlz(self) -> FVector3: ...
    @property
    def xlo(self) -> FVector3: ...
    @property
    def xll(self) -> FVector3: ...
    @property
    def yxx(self) -> FVector3: ...
    @property
    def yxy(self) -> FVector3: ...
    @property
    def yxz(self) -> FVector3: ...
    @property
    def yxo(self) -> FVector3: ...
    @property
    def yxl(self) -> FVector3: ...
    @property
    def yyx(self) -> FVector3: ...
    @property
    def yyy(self) -> FVector3: ...
    @property
    def yyz(self) -> FVector3: ...
    @property
    def yyo(self) -> FVector3: ...
    @property
    def yyl(self) -> FVector3: ...
    @property
    def yzx(self) -> FVector3: ...
    @property
    def yzy(self) -> FVector3: ...
    @property
    def yzz(self) -> FVector3: ...
    @property
    def yzo(self) -> FVector3: ...
    @property
    def yzl(self) -> FVector3: ...
    @property
    def yox(self) -> FVector3: ...
    @property
    def yoy(self) -> FVector3: ...
    @property
    def yoz(self) -> FVector3: ...
    @property
    def yoo(self) -> FVector3: ...
    @property
    def yol(self) -> FVector3: ...
    @property
    def ylx(self) -> FVector3: ...
    @property
    def yly(self) -> FVector3: ...
    @property
    def ylz(self) -> FVector3: ...
    @property
    def ylo(self) -> FVector3: ...
    @property
    def yll(self) -> FVector3: ...
    @property
    def zxx(self) -> FVector3: ...
    @property
    def zxy(self) -> FVector3: ...
    @property
    def zxz(self) -> FVector3: ...
    @property
    def zxo(self) -> FVector3: ...
    @property
    def zxl(self) -> FVector3: ...
    @property
    def zyx(self) -> FVector3: ...
    @property
    def zyy(self) -> FVector3: ...
    @property
    def zyz(self) -> FVector3: ...
    @property
    def zyo(self) -> FVector3: ...
    @property
    def zyl(self) -> FVector3: ...
    @property
    def zzx(self) -> FVector3: ...
    @property
    def zzy(self) -> FVector3: ...
    @property
    def zzz(self) -> FVector3: ...
    @property
    def zzo(self) -> FVector3: ...
    @property
    def zzl(self) -> FVector3: ...
    @property
    def zox(self) -> FVector3: ...
    @property
    def zoy(self) -> FVector3: ...
    @property
    def zoz(self) -> FVector3: ...
    @property
    def zoo(self) -> FVector3: ...
    @property
    def zol(self) -> FVector3: ...
    @property
    def zlx(self) -> FVector3: ...
    @property
    def zly(self) -> FVector3: ...
    @property
    def zlz(self) -> FVector3: ...
    @property
    def zlo(self) -> FVector3: ...
    @property
    def zll(self) -> FVector3: ...
    @property
    def oxx(self) -> FVector3: ...
    @property
    def oxy(self) -> FVector3: ...
    @property
    def oxz(self) -> FVector3: ...
    @property
    def oxo(self) -> FVector3: ...
    @property
    def oxl(self) -> FVector3: ...
    @property
    def oyx(self) -> FVector3: ...
    @property
    def oyy(self) -> FVector3: ...
    @property
    def oyz(self) -> FVector3: ...
    @property
    def oyo(self) -> FVector3: ...
    @property
    def oyl(self) -> FVector3: ...
    @property
    def ozx(self) -> FVector3: ...
    @property
    def ozy(self) -> FVector3: ...
    @property
    def ozz(self) -> FVector3: ...
    @property
    def ozo(self) -> FVector3: ...
    @property
    def ozl(self) -> FVector3: ...
    @property
    def oox(self) -> FVector3: ...
    @property
    def ooy(self) -> FVector3: ...
    @property
    def ooz(self) -> FVector3: ...
    @property
    def olx(self) -> FVector3: ...
    @property
    def oly(self) -> FVector3: ...
    @property
    def olz(self) -> FVector3: ...
    @property
    def lxx(self) -> FVector3: ...
    @property
    def lxy(self) -> FVector3: ...
    @property
    def lxz(self) -> FVector3: ...
    @property
    def lxo(self) -> FVector3: ...
    @property
    def lxl(self) -> FVector3: ...
    @property
    def lyx(self) -> FVector3: ...
    @property
    def lyy(self) -> FVector3: ...
    @property
    def lyz(self) -> FVector3: ...
    @property
    def lyo(self) -> FVector3: ...
    @property
    def lyl(self) -> FVector3: ...
    @property
    def lzx(self) -> FVector3: ...
    @property
    def lzy(self) -> FVector3: ...
    @property
    def lzz(self) -> FVector3: ...
    @property
    def lzo(self) -> FVector3: ...
    @property
    def lzl(self) -> FVector3: ...
    @property
    def lox(self) -> FVector3: ...
    @property
    def loy(self) -> FVector3: ...
    @property
    def loz(self) -> FVector3: ...
    @property
    def llx(self) -> FVector3: ...
    @property
    def lly(self) -> FVector3: ...
    @property
    def llz(self) -> FVector3: ...
    @property
    def xxxx(self) -> FVector4: ...
    @property
    def xxxy(self) -> FVector4: ...
    @property
    def xxxz(self) -> FVector4: ...
    @property
    def xxxo(self) -> FVector4: ...
    @property
    def xxxl(self) -> FVector4: ...
    @property
    def xxyx(self) -> FVector4: ...
    @property
    def xxyy(self) -> FVector4: ...
    @property
    def xxyz(self) -> FVector4: ...
    @property
    def xxyo(self) -> FVector4: ...
    @property
    def xxyl(self) -> FVector4: ...
    @property
    def xxzx(self) -> FVector4: ...
    @property
    def xxzy(self) -> FVector4: ...
    @property
    def xxzz(self) -> FVector4: ...
    @property
    def xxzo(self) -> FVector4: ...
    @property
    def xxzl(self) -> FVector4: ...
    @property
    def xxox(self) -> FVector4: ...
    @property
    def xxoy(self) -> FVector4: ...
    @property
    def xxoz(self) -> FVector4: ...
    @property
    def xxoo(self) -> FVector4: ...
    @property
    def xxol(self) -> FVector4: ...
    @property
    def xxlx(self) -> FVector4: ...
    @property
    def xxly(self) -> FVector4: ...
    @property
    def xxlz(self) -> FVector4: ...
    @property
    def xxlo(self) -> FVector4: ...
    @property
    def xxll(self) -> FVector4: ...
    @property
    def xyxx(self) -> FVector4: ...
    @property
    def xyxy(self) -> FVector4: ...
    @property
    def xyxz(self) -> FVector4: ...
    @property
    def xyxo(self) -> FVector4: ...
    @property
    def xyxl(self) -> FVector4: ...
    @property
    def xyyx(self) -> FVector4: ...
    @property
    def xyyy(self) -> FVector4: ...
    @property
    def xyyz(self) -> FVector4: ...
    @property
    def xyyo(self) -> FVector4: ...
    @property
    def xyyl(self) -> FVector4: ...
    @property
    def xyzx(self) -> FVector4: ...
    @property
    def xyzy(self) -> FVector4: ...
    @property
    def xyzz(self) -> FVector4: ...
    @property
    def xyzo(self) -> FVector4: ...
    @property
    def xyzl(self) -> FVector4: ...
    @property
    def xyox(self) -> FVector4: ...
    @property
    def xyoy(self) -> FVector4: ...
    @property
    def xyoz(self) -> FVector4: ...
    @property
    def xyoo(self) -> FVector4: ...
    @property
    def xyol(self) -> FVector4: ...
    @property
    def xylx(self) -> FVector4: ...
    @property
    def xyly(self) -> FVector4: ...
    @property
    def xylz(self) -> FVector4: ...
    @property
    def xylo(self) -> FVector4: ...
    @property
    def xyll(self) -> FVector4: ...
    @property
    def xzxx(self) -> FVector4: ...
    @property
    def xzxy(self) -> FVector4: ...
    @property
    def xzxz(self) -> FVector4: ...
    @property
    def xzxo(self) -> FVector4: ...
    @property
    def xzxl(self) -> FVector4: ...
    @property
    def xzyx(self) -> FVector4: ...
    @property
    def xzyy(self) -> FVector4: ...
    @property
    def xzyz(self) -> FVector4: ...
    @property
    def xzyo(self) -> FVector4: ...
    @property
    def xzyl(self) -> FVector4: ...
    @property
    def xzzx(self) -> FVector4: ...
    @property
    def xzzy(self) -> FVector4: ...
    @property
    def xzzz(self) -> FVector4: ...
    @property
    def xzzo(self) -> FVector4: ...
    @property
    def xzzl(self) -> FVector4: ...
    @property
    def xzox(self) -> FVector4: ...
    @property
    def xzoy(self) -> FVector4: ...
    @property
    def xzoz(self) -> FVector4: ...
    @property
    def xzoo(self) -> FVector4: ...
    @property
    def xzol(self) -> FVector4: ...
    @property
    def xzlx(self) -> FVector4: ...
    @property
    def xzly(self) -> FVector4: ...
    @property
    def xzlz(self) -> FVector4: ...
    @property
    def xzlo(self) -> FVector4: ...
    @property
    def xzll(self) -> FVector4: ...
    @property
    def xoxx(self) -> FVector4: ...
    @property
    def xoxy(self) -> FVector4: ...
    @property
    def xoxz(self) -> FVector4: ...
    @property
    def xoxo(self) -> FVector4: ...
    @property
    def xoxl(self) -> FVector4: ...
    @property
    def xoyx(self) -> FVector4: ...
    @property
    def xoyy(self) -> FVector4: ...
    @property
    def xoyz(self) -> FVector4: ...
    @property
    def xoyo(self) -> FVector4: ...
    @property
    def xoyl(self) -> FVector4: ...
    @property
    def xozx(self) -> FVector4: ...
    @property
    def xozy(self) -> FVector4: ...
    @property
    def xozz(self) -> FVector4: ...
    @property
    def xozo(self) -> FVector4: ...
    @property
    def xozl(self) -> FVector4: ...
    @property
    def xoox(self) -> FVector4: ...
    @property
    def xooy(self) -> FVector4: ...
    @property
    def xooz(self) -> FVector4: ...
    @property
    def xooo(self) -> FVector4: ...
    @property
    def xool(self) -> FVector4: ...
    @property
    def xolx(self) -> FVector4: ...
    @property
    def xoly(self) -> FVector4: ...
    @property
    def xolz(self) -> FVector4: ...
    @property
    def xolo(self) -> FVector4: ...
    @property
    def xoll(self) -> FVector4: ...
    @property
    def xlxx(self) -> FVector4: ...
    @property
    def xlxy(self) -> FVector4: ...
    @property
    def xlxz(self) -> FVector4: ...
    @property
    def xlxo(self) -> FVector4: ...
    @property
    def xlxl(self) -> FVector4: ...
    @property
    def xlyx(self) -> FVector4: ...
    @property
    def xlyy(self) -> FVector4: ...
    @property
    def xlyz(self) -> FVector4: ...
    @property
    def xlyo(self) -> FVector4: ...
    @property
    def xlyl(self) -> FVector4: ...
    @property
    def xlzx(self) -> FVector4: ...
    @property
    def xlzy(self) -> FVector4: ...
    @property
    def xlzz(self) -> FVector4: ...
    @property
    def xlzo(self) -> FVector4: ...
    @property
    def xlzl(self) -> FVector4: ...
    @property
    def xlox(self) -> FVector4: ...
    @property
    def xloy(self) -> FVector4: ...
    @property
    def xloz(self) -> FVector4: ...
    @property
    def xloo(self) -> FVector4: ...
    @property
    def xlol(self) -> FVector4: ...
    @property
    def xllx(self) -> FVector4: ...
    @property
    def xlly(self) -> FVector4: ...
    @property
    def xllz(self) -> FVector4: ...
    @property
    def xllo(self) -> FVector4: ...
    @property
    def xlll(self) -> FVector4: ...
    @property
    def yxxx(self) -> FVector4: ...
    @property
    def yxxy(self) -> FVector4: ...
    @property
    def yxxz(self) -> FVector4: ...
    @property
    def yxxo(self) -> FVector4: ...
    @property
    def yxxl(self) -> FVector4: ...
    @property
    def yxyx(self) -> FVector4: ...
    @property
    def yxyy(self) -> FVector4: ...
    @property
    def yxyz(self) -> FVector4: ...
    @property
    def yxyo(self) -> FVector4: ...
    @property
    def yxyl(self) -> FVector4: ...
    @property
    def yxzx(self) -> FVector4: ...
    @property
    def yxzy(self) -> FVector4: ...
    @property
    def yxzz(self) -> FVector4: ...
    @property
    def yxzo(self) -> FVector4: ...
    @property
    def yxzl(self) -> FVector4: ...
    @property
    def yxox(self) -> FVector4: ...
    @property
    def yxoy(self) -> FVector4: ...
    @property
    def yxoz(self) -> FVector4: ...
    @property
    def yxoo(self) -> FVector4: ...
    @property
    def yxol(self) -> FVector4: ...
    @property
    def yxlx(self) -> FVector4: ...
    @property
    def yxly(self) -> FVector4: ...
    @property
    def yxlz(self) -> FVector4: ...
    @property
    def yxlo(self) -> FVector4: ...
    @property
    def yxll(self) -> FVector4: ...
    @property
    def yyxx(self) -> FVector4: ...
    @property
    def yyxy(self) -> FVector4: ...
    @property
    def yyxz(self) -> FVector4: ...
    @property
    def yyxo(self) -> FVector4: ...
    @property
    def yyxl(self) -> FVector4: ...
    @property
    def yyyx(self) -> FVector4: ...
    @property
    def yyyy(self) -> FVector4: ...
    @property
    def yyyz(self) -> FVector4: ...
    @property
    def yyyo(self) -> FVector4: ...
    @property
    def yyyl(self) -> FVector4: ...
    @property
    def yyzx(self) -> FVector4: ...
    @property
    def yyzy(self) -> FVector4: ...
    @property
    def yyzz(self) -> FVector4: ...
    @property
    def yyzo(self) -> FVector4: ...
    @property
    def yyzl(self) -> FVector4: ...
    @property
    def yyox(self) -> FVector4: ...
    @property
    def yyoy(self) -> FVector4: ...
    @property
    def yyoz(self) -> FVector4: ...
    @property
    def yyoo(self) -> FVector4: ...
    @property
    def yyol(self) -> FVector4: ...
    @property
    def yylx(self) -> FVector4: ...
    @property
    def yyly(self) -> FVector4: ...
    @property
    def yylz(self) -> FVector4: ...
    @property
    def yylo(self) -> FVector4: ...
    @property
    def yyll(self) -> FVector4: ...
    @property
    def yzxx(self) -> FVector4: ...
    @property
    def yzxy(self) -> FVector4: ...
    @property
    def yzxz(self) -> FVector4: ...
    @property
    def yzxo(self) -> FVector4: ...
    @property
    def yzxl(self) -> FVector4: ...
    @property
    def yzyx(self) -> FVector4: ...
    @property
    def yzyy(self) -> FVector4: ...
    @property
    def yzyz(self) -> FVector4: ...
    @property
    def yzyo(self) -> FVector4: ...
    @property
    def yzyl(self) -> FVector4: ...
    @property
    def yzzx(self) -> FVector4: ...
    @property
    def yzzy(self) -> FVector4: ...
    @property
    def yzzz(self) -> FVector4: ...
    @property
    def yzzo(self) -> FVector4: ...
    @property
    def yzzl(self) -> FVector4: ...
    @property
    def yzox(self) -> FVector4: ...
    @property
    def yzoy(self) -> FVector4: ...
    @property
    def yzoz(self) -> FVector4: ...
    @property
    def yzoo(self) -> FVector4: ...
    @property
    def yzol(self) -> FVector4: ...
    @property
    def yzlx(self) -> FVector4: ...
    @property
    def yzly(self) -> FVector4: ...
    @property
    def yzlz(self) -> FVector4: ...
    @property
    def yzlo(self) -> FVector4: ...
    @property
    def yzll(self) -> FVector4: ...
    @property
    def yoxx(self) -> FVector4: ...
    @property
    def yoxy(self) -> FVector4: ...
    @property
    def yoxz(self) -> FVector4: ...
    @property
    def yoxo(self) -> FVector4: ...
    @property
    def yoxl(self) -> FVector4: ...
    @property
    def yoyx(self) -> FVector4: ...
    @property
    def yoyy(self) -> FVector4: ...
    @property
    def yoyz(self) -> FVector4: ...
    @property
    def yoyo(self) -> FVector4: ...
    @property
    def yoyl(self) -> FVector4: ...
    @property
    def yozx(self) -> FVector4: ...
    @property
    def yozy(self) -> FVector4: ...
    @property
    def yozz(self) -> FVector4: ...
    @property
    def yozo(self) -> FVector4: ...
    @property
    def yozl(self) -> FVector4: ...
    @property
    def yoox(self) -> FVector4: ...
    @property
    def yooy(self) -> FVector4: ...
    @property
    def yooz(self) -> FVector4: ...
    @property
    def yooo(self) -> FVector4: ...
    @property
    def yool(self) -> FVector4: ...
    @property
    def yolx(self) -> FVector4: ...
    @property
    def yoly(self) -> FVector4: ...
    @property
    def yolz(self) -> FVector4: ...
    @property
    def yolo(self) -> FVector4: ...
    @property
    def yoll(self) -> FVector4: ...
    @property
    def ylxx(self) -> FVector4: ...
    @property
    def ylxy(self) -> FVector4: ...
    @property
    def ylxz(self) -> FVector4: ...
    @property
    def ylxo(self) -> FVector4: ...
    @property
    def ylxl(self) -> FVector4: ...
    @property
    def ylyx(self) -> FVector4: ...
    @property
    def ylyy(self) -> FVector4: ...
    @property
    def ylyz(self) -> FVector4: ...
    @property
    def ylyo(self) -> FVector4: ...
    @property
    def ylyl(self) -> FVector4: ...
    @property
    def ylzx(self) -> FVector4: ...
    @property
    def ylzy(self) -> FVector4: ...
    @property
    def ylzz(self) -> FVector4: ...
    @property
    def ylzo(self) -> FVector4: ...
    @property
    def ylzl(self) -> FVector4: ...
    @property
    def ylox(self) -> FVector4: ...
    @property
    def yloy(self) -> FVector4: ...
    @property
    def yloz(self) -> FVector4: ...
    @property
    def yloo(self) -> FVector4: ...
    @property
    def ylol(self) -> FVector4: ...
    @property
    def yllx(self) -> FVector4: ...
    @property
    def ylly(self) -> FVector4: ...
    @property
    def yllz(self) -> FVector4: ...
    @property
    def yllo(self) -> FVector4: ...
    @property
    def ylll(self) -> FVector4: ...
    @property
    def zxxx(self) -> FVector4: ...
    @property
    def zxxy(self) -> FVector4: ...
    @property
    def zxxz(self) -> FVector4: ...
    @property
    def zxxo(self) -> FVector4: ...
    @property
    def zxxl(self) -> FVector4: ...
    @property
    def zxyx(self) -> FVector4: ...
    @property
    def zxyy(self) -> FVector4: ...
    @property
    def zxyz(self) -> FVector4: ...
    @property
    def zxyo(self) -> FVector4: ...
    @property
    def zxyl(self) -> FVector4: ...
    @property
    def zxzx(self) -> FVector4: ...
    @property
    def zxzy(self) -> FVector4: ...
    @property
    def zxzz(self) -> FVector4: ...
    @property
    def zxzo(self) -> FVector4: ...
    @property
    def zxzl(self) -> FVector4: ...
    @property
    def zxox(self) -> FVector4: ...
    @property
    def zxoy(self) -> FVector4: ...
    @property
    def zxoz(self) -> FVector4: ...
    @property
    def zxoo(self) -> FVector4: ...
    @property
    def zxol(self) -> FVector4: ...
    @property
    def zxlx(self) -> FVector4: ...
    @property
    def zxly(self) -> FVector4: ...
    @property
    def zxlz(self) -> FVector4: ...
    @property
    def zxlo(self) -> FVector4: ...
    @property
    def zxll(self) -> FVector4: ...
    @property
    def zyxx(self) -> FVector4: ...
    @property
    def zyxy(self) -> FVector4: ...
    @property
    def zyxz(self) -> FVector4: ...
    @property
    def zyxo(self) -> FVector4: ...
    @property
    def zyxl(self) -> FVector4: ...
    @property
    def zyyx(self) -> FVector4: ...
    @property
    def zyyy(self) -> FVector4: ...
    @property
    def zyyz(self) -> FVector4: ...
    @property
    def zyyo(self) -> FVector4: ...
    @property
    def zyyl(self) -> FVector4: ...
    @property
    def zyzx(self) -> FVector4: ...
    @property
    def zyzy(self) -> FVector4: ...
    @property
    def zyzz(self) -> FVector4: ...
    @property
    def zyzo(self) -> FVector4: ...
    @property
    def zyzl(self) -> FVector4: ...
    @property
    def zyox(self) -> FVector4: ...
    @property
    def zyoy(self) -> FVector4: ...
    @property
    def zyoz(self) -> FVector4: ...
    @property
    def zyoo(self) -> FVector4: ...
    @property
    def zyol(self) -> FVector4: ...
    @property
    def zylx(self) -> FVector4: ...
    @property
    def zyly(self) -> FVector4: ...
    @property
    def zylz(self) -> FVector4: ...
    @property
    def zylo(self) -> FVector4: ...
    @property
    def zyll(self) -> FVector4: ...
    @property
    def zzxx(self) -> FVector4: ...
    @property
    def zzxy(self) -> FVector4: ...
    @property
    def zzxz(self) -> FVector4: ...
    @property
    def zzxo(self) -> FVector4: ...
    @property
    def zzxl(self) -> FVector4: ...
    @property
    def zzyx(self) -> FVector4: ...
    @property
    def zzyy(self) -> FVector4: ...
    @property
    def zzyz(self) -> FVector4: ...
    @property
    def zzyo(self) -> FVector4: ...
    @property
    def zzyl(self) -> FVector4: ...
    @property
    def zzzx(self) -> FVector4: ...
    @property
    def zzzy(self) -> FVector4: ...
    @property
    def zzzz(self) -> FVector4: ...
    @property
    def zzzo(self) -> FVector4: ...
    @property
    def zzzl(self) -> FVector4: ...
    @property
    def zzox(self) -> FVector4: ...
    @property
    def zzoy(self) -> FVector4: ...
    @property
    def zzoz(self) -> FVector4: ...
    @property
    def zzoo(self) -> FVector4: ...
    @property
    def zzol(self) -> FVector4: ...
    @property
    def zzlx(self) -> FVector4: ...
    @property
    def zzly(self) -> FVector4: ...
    @property
    def zzlz(self) -> FVector4: ...
    @property
    def zzlo(self) -> FVector4: ...
    @property
    def zzll(self) -> FVector4: ...
    @property
    def zoxx(self) -> FVector4: ...
    @property
    def zoxy(self) -> FVector4: ...
    @property
    def zoxz(self) -> FVector4: ...
    @property
    def zoxo(self) -> FVector4: ...
    @property
    def zoxl(self) -> FVector4: ...
    @property
    def zoyx(self) -> FVector4: ...
    @property
    def zoyy(self) -> FVector4: ...
    @property
    def zoyz(self) -> FVector4: ...
    @property
    def zoyo(self) -> FVector4: ...
    @property
    def zoyl(self) -> FVector4: ...
    @property
    def zozx(self) -> FVector4: ...
    @property
    def zozy(self) -> FVector4: ...
    @property
    def zozz(self) -> FVector4: ...
    @property
    def zozo(self) -> FVector4: ...
    @property
    def zozl(self) -> FVector4: ...
    @property
    def zoox(self) -> FVector4: ...
    @property
    def zooy(self) -> FVector4: ...
    @property
    def zooz(self) -> FVector4: ...
    @property
    def zooo(self) -> FVector4: ...
    @property
    def zool(self) -> FVector4: ...
    @property
    def zolx(self) -> FVector4: ...
    @property
    def zoly(self) -> FVector4: ...
    @property
    def zolz(self) -> FVector4: ...
    @property
    def zolo(self) -> FVector4: ...
    @property
    def zoll(self) -> FVector4: ...
    @property
    def zlxx(self) -> FVector4: ...
    @property
    def zlxy(self) -> FVector4: ...
    @property
    def zlxz(self) -> FVector4: ...
    @property
    def zlxo(self) -> FVector4: ...
    @property
    def zlxl(self) -> FVector4: ...
    @property
    def zlyx(self) -> FVector4: ...
    @property
    def zlyy(self) -> FVector4: ...
    @property
    def zlyz(self) -> FVector4: ...
    @property
    def zlyo(self) -> FVector4: ...
    @property
    def zlyl(self) -> FVector4: ...
    @property
    def zlzx(self) -> FVector4: ...
    @property
    def zlzy(self) -> FVector4: ...
    @property
    def zlzz(self) -> FVector4: ...
    @property
    def zlzo(self) -> FVector4: ...
    @property
    def zlzl(self) -> FVector4: ...
    @property
    def zlox(self) -> FVector4: ...
    @property
    def zloy(self) -> FVector4: ...
    @property
    def zloz(self) -> FVector4: ...
    @property
    def zloo(self) -> FVector4: ...
    @property
    def zlol(self) -> FVector4: ...
    @property
    def zllx(self) -> FVector4: ...
    @property
    def zlly(self) -> FVector4: ...
    @property
    def zllz(self) -> FVector4: ...
    @property
    def zllo(self) -> FVector4: ...
    @property
    def zlll(self) -> FVector4: ...
    @property
    def oxxx(self) -> FVector4: ...
    @property
    def oxxy(self) -> FVector4: ...
    @property
    def oxxz(self) -> FVector4: ...
    @property
    def oxxo(self) -> FVector4: ...
    @property
    def oxxl(self) -> FVector4: ...
    @property
    def oxyx(self) -> FVector4: ...
    @property
    def oxyy(self) -> FVector4: ...
    @property
    def oxyz(self) -> FVector4: ...
    @property
    def oxyo(self) -> FVector4: ...
    @property
    def oxyl(self) -> FVector4: ...
    @property
    def oxzx(self) -> FVector4: ...
    @property
    def oxzy(self) -> FVector4: ...
    @property
    def oxzz(self) -> FVector4: ...
    @property
    def oxzo(self) -> FVector4: ...
    @property
    def oxzl(self) -> FVector4: ...
    @property
    def oxox(self) -> FVector4: ...
    @property
    def oxoy(self) -> FVector4: ...
    @property
    def oxoz(self) -> FVector4: ...
    @property
    def oxoo(self) -> FVector4: ...
    @property
    def oxol(self) -> FVector4: ...
    @property
    def oxlx(self) -> FVector4: ...
    @property
    def oxly(self) -> FVector4: ...
    @property
    def oxlz(self) -> FVector4: ...
    @property
    def oxlo(self) -> FVector4: ...
    @property
    def oxll(self) -> FVector4: ...
    @property
    def oyxx(self) -> FVector4: ...
    @property
    def oyxy(self) -> FVector4: ...
    @property
    def oyxz(self) -> FVector4: ...
    @property
    def oyxo(self) -> FVector4: ...
    @property
    def oyxl(self) -> FVector4: ...
    @property
    def oyyx(self) -> FVector4: ...
    @property
    def oyyy(self) -> FVector4: ...
    @property
    def oyyz(self) -> FVector4: ...
    @property
    def oyyo(self) -> FVector4: ...
    @property
    def oyyl(self) -> FVector4: ...
    @property
    def oyzx(self) -> FVector4: ...
    @property
    def oyzy(self) -> FVector4: ...
    @property
    def oyzz(self) -> FVector4: ...
    @property
    def oyzo(self) -> FVector4: ...
    @property
    def oyzl(self) -> FVector4: ...
    @property
    def oyox(self) -> FVector4: ...
    @property
    def oyoy(self) -> FVector4: ...
    @property
    def oyoz(self) -> FVector4: ...
    @property
    def oyoo(self) -> FVector4: ...
    @property
    def oyol(self) -> FVector4: ...
    @property
    def oylx(self) -> FVector4: ...
    @property
    def oyly(self) -> FVector4: ...
    @property
    def oylz(self) -> FVector4: ...
    @property
    def oylo(self) -> FVector4: ...
    @property
    def oyll(self) -> FVector4: ...
    @property
    def ozxx(self) -> FVector4: ...
    @property
    def ozxy(self) -> FVector4: ...
    @property
    def ozxz(self) -> FVector4: ...
    @property
    def ozxo(self) -> FVector4: ...
    @property
    def ozxl(self) -> FVector4: ...
    @property
    def ozyx(self) -> FVector4: ...
    @property
    def ozyy(self) -> FVector4: ...
    @property
    def ozyz(self) -> FVector4: ...
    @property
    def ozyo(self) -> FVector4: ...
    @property
    def ozyl(self) -> FVector4: ...
    @property
    def ozzx(self) -> FVector4: ...
    @property
    def ozzy(self) -> FVector4: ...
    @property
    def ozzz(self) -> FVector4: ...
    @property
    def ozzo(self) -> FVector4: ...
    @property
    def ozzl(self) -> FVector4: ...
    @property
    def ozox(self) -> FVector4: ...
    @property
    def ozoy(self) -> FVector4: ...
    @property
    def ozoz(self) -> FVector4: ...
    @property
    def ozoo(self) -> FVector4: ...
    @property
    def ozol(self) -> FVector4: ...
    @property
    def ozlx(self) -> FVector4: ...
    @property
    def ozly(self) -> FVector4: ...
    @property
    def ozlz(self) -> FVector4: ...
    @property
    def ozlo(self) -> FVector4: ...
    @property
    def ozll(self) -> FVector4: ...
    @property
    def ooxx(self) -> FVector4: ...
    @property
    def ooxy(self) -> FVector4: ...
    @property
    def ooxz(self) -> FVector4: ...
    @property
    def ooxo(self) -> FVector4: ...
    @property
    def ooxl(self) -> FVector4: ...
    @property
    def ooyx(self) -> FVector4: ...
    @property
    def ooyy(self) -> FVector4: ...
    @property
    def ooyz(self) -> FVector4: ...
    @property
    def ooyo(self) -> FVector4: ...
    @property
    def ooyl(self) -> FVector4: ...
    @property
    def oozx(self) -> FVector4: ...
    @property
    def oozy(self) -> FVector4: ...
    @property
    def oozz(self) -> FVector4: ...
    @property
    def oozo(self) -> FVector4: ...
    @property
    def oozl(self) -> FVector4: ...
    @property
    def ooox(self) -> FVector4: ...
    @property
    def oooy(self) -> FVector4: ...
    @property
    def oooz(self) -> FVector4: ...
    @property
    def oolx(self) -> FVector4: ...
    @property
    def ooly(self) -> FVector4: ...
    @property
    def oolz(self) -> FVector4: ...
    @property
    def olxx(self) -> FVector4: ...
    @property
    def olxy(self) -> FVector4: ...
    @property
    def olxz(self) -> FVector4: ...
    @property
    def olxo(self) -> FVector4: ...
    @property
    def olxl(self) -> FVector4: ...
    @property
    def olyx(self) -> FVector4: ...
    @property
    def olyy(self) -> FVector4: ...
    @property
    def olyz(self) -> FVector4: ...
    @property
    def olyo(self) -> FVector4: ...
    @property
    def olyl(self) -> FVector4: ...
    @property
    def olzx(self) -> FVector4: ...
    @property
    def olzy(self) -> FVector4: ...
    @property
    def olzz(self) -> FVector4: ...
    @property
    def olzo(self) -> FVector4: ...
    @property
    def olzl(self) -> FVector4: ...
    @property
    def olox(self) -> FVector4: ...
    @property
    def oloy(self) -> FVector4: ...
    @property
    def oloz(self) -> FVector4: ...
    @property
    def ollx(self) -> FVector4: ...
    @property
    def olly(self) -> FVector4: ...
    @property
    def ollz(self) -> FVector4: ...
    @property
    def lxxx(self) -> FVector4: ...
    @property
    def lxxy(self) -> FVector4: ...
    @property
    def lxxz(self) -> FVector4: ...
    @property
    def lxxo(self) -> FVector4: ...
    @property
    def lxxl(self) -> FVector4: ...
    @property
    def lxyx(self) -> FVector4: ...
    @property
    def lxyy(self) -> FVector4: ...
    @property
    def lxyz(self) -> FVector4: ...
    @property
    def lxyo(self) -> FVector4: ...
    @property
    def lxyl(self) -> FVector4: ...
    @property
    def lxzx(self) -> FVector4: ...
    @property
    def lxzy(self) -> FVector4: ...
    @property
    def lxzz(self) -> FVector4: ...
    @property
    def lxzo(self) -> FVector4: ...
    @property
    def lxzl(self) -> FVector4: ...
    @property
    def lxox(self) -> FVector4: ...
    @property
    def lxoy(self) -> FVector4: ...
    @property
    def lxoz(self) -> FVector4: ...
    @property
    def lxoo(self) -> FVector4: ...
    @property
    def lxol(self) -> FVector4: ...
    @property
    def lxlx(self) -> FVector4: ...
    @property
    def lxly(self) -> FVector4: ...
    @property
    def lxlz(self) -> FVector4: ...
    @property
    def lxlo(self) -> FVector4: ...
    @property
    def lxll(self) -> FVector4: ...
    @property
    def lyxx(self) -> FVector4: ...
    @property
    def lyxy(self) -> FVector4: ...
    @property
    def lyxz(self) -> FVector4: ...
    @property
    def lyxo(self) -> FVector4: ...
    @property
    def lyxl(self) -> FVector4: ...
    @property
    def lyyx(self) -> FVector4: ...
    @property
    def lyyy(self) -> FVector4: ...
    @property
    def lyyz(self) -> FVector4: ...
    @property
    def lyyo(self) -> FVector4: ...
    @property
    def lyyl(self) -> FVector4: ...
    @property
    def lyzx(self) -> FVector4: ...
    @property
    def lyzy(self) -> FVector4: ...
    @property
    def lyzz(self) -> FVector4: ...
    @property
    def lyzo(self) -> FVector4: ...
    @property
    def lyzl(self) -> FVector4: ...
    @property
    def lyox(self) -> FVector4: ...
    @property
    def lyoy(self) -> FVector4: ...
    @property
    def lyoz(self) -> FVector4: ...
    @property
    def lyoo(self) -> FVector4: ...
    @property
    def lyol(self) -> FVector4: ...
    @property
    def lylx(self) -> FVector4: ...
    @property
    def lyly(self) -> FVector4: ...
    @property
    def lylz(self) -> FVector4: ...
    @property
    def lylo(self) -> FVector4: ...
    @property
    def lyll(self) -> FVector4: ...
    @property
    def lzxx(self) -> FVector4: ...
    @property
    def lzxy(self) -> FVector4: ...
    @property
    def lzxz(self) -> FVector4: ...
    @property
    def lzxo(self) -> FVector4: ...
    @property
    def lzxl(self) -> FVector4: ...
    @property
    def lzyx(self) -> FVector4: ...
    @property
    def lzyy(self) -> FVector4: ...
    @property
    def lzyz(self) -> FVector4: ...
    @property
    def lzyo(self) -> FVector4: ...
    @property
    def lzyl(self) -> FVector4: ...
    @property
    def lzzx(self) -> FVector4: ...
    @property
    def lzzy(self) -> FVector4: ...
    @property
    def lzzz(self) -> FVector4: ...
    @property
    def lzzo(self) -> FVector4: ...
    @property
    def lzzl(self) -> FVector4: ...
    @property
    def lzox(self) -> FVector4: ...
    @property
    def lzoy(self) -> FVector4: ...
    @property
    def lzoz(self) -> FVector4: ...
    @property
    def lzoo(self) -> FVector4: ...
    @property
    def lzol(self) -> FVector4: ...
    @property
    def lzlx(self) -> FVector4: ...
    @property
    def lzly(self) -> FVector4: ...
    @property
    def lzlz(self) -> FVector4: ...
    @property
    def lzlo(self) -> FVector4: ...
    @property
    def lzll(self) -> FVector4: ...
    @property
    def loxx(self) -> FVector4: ...
    @property
    def loxy(self) -> FVector4: ...
    @property
    def loxz(self) -> FVector4: ...
    @property
    def loxo(self) -> FVector4: ...
    @property
    def loxl(self) -> FVector4: ...
    @property
    def loyx(self) -> FVector4: ...
    @property
    def loyy(self) -> FVector4: ...
    @property
    def loyz(self) -> FVector4: ...
    @property
    def loyo(self) -> FVector4: ...
    @property
    def loyl(self) -> FVector4: ...
    @property
    def lozx(self) -> FVector4: ...
    @property
    def lozy(self) -> FVector4: ...
    @property
    def lozz(self) -> FVector4: ...
    @property
    def lozo(self) -> FVector4: ...
    @property
    def lozl(self) -> FVector4: ...
    @property
    def loox(self) -> FVector4: ...
    @property
    def looy(self) -> FVector4: ...
    @property
    def looz(self) -> FVector4: ...
    @property
    def lolx(self) -> FVector4: ...
    @property
    def loly(self) -> FVector4: ...
    @property
    def lolz(self) -> FVector4: ...
    @property
    def llxx(self) -> FVector4: ...
    @property
    def llxy(self) -> FVector4: ...
    @property
    def llxz(self) -> FVector4: ...
    @property
    def llxo(self) -> FVector4: ...
    @property
    def llxl(self) -> FVector4: ...
    @property
    def llyx(self) -> FVector4: ...
    @property
    def llyy(self) -> FVector4: ...
    @property
    def llyz(self) -> FVector4: ...
    @property
    def llyo(self) -> FVector4: ...
    @property
    def llyl(self) -> FVector4: ...
    @property
    def llzx(self) -> FVector4: ...
    @property
    def llzy(self) -> FVector4: ...
    @property
    def llzz(self) -> FVector4: ...
    @property
    def llzo(self) -> FVector4: ...
    @property
    def llzl(self) -> FVector4: ...
    @property
    def llox(self) -> FVector4: ...
    @property
    def lloy(self) -> FVector4: ...
    @property
    def lloz(self) -> FVector4: ...
    @property
    def lllx(self) -> FVector4: ...
    @property
    def llly(self) -> FVector4: ...
    @property
    def lllz(self) -> FVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def g(self) -> float: ...
    @property
    def b(self) -> float: ...
    @property
    def rr(self) -> FVector2: ...
    @property
    def rg(self) -> FVector2: ...
    @property
    def rb(self) -> FVector2: ...
    @property
    def ro(self) -> FVector2: ...
    @property
    def rl(self) -> FVector2: ...
    @property
    def gr(self) -> FVector2: ...
    @property
    def gg(self) -> FVector2: ...
    @property
    def gb(self) -> FVector2: ...
    @property
    def go(self) -> FVector2: ...
    @property
    def gl(self) -> FVector2: ...
    @property
    def br(self) -> FVector2: ...
    @property
    def bg(self) -> FVector2: ...
    @property
    def bb(self) -> FVector2: ...
    @property
    def bo(self) -> FVector2: ...
    @property
    def bl(self) -> FVector2: ...
    @property
    def og(self) -> FVector2: ...
    @property
    def ob(self) -> FVector2: ...
    @property
    def lr(self) -> FVector2: ...
    @property
    def lg(self) -> FVector2: ...
    @property
    def lb(self) -> FVector2: ...
    @property
    def rrr(self) -> FVector3: ...
    @property
    def rrg(self) -> FVector3: ...
    @property
    def rrb(self) -> FVector3: ...
    @property
    def rro(self) -> FVector3: ...
    @property
    def rrl(self) -> FVector3: ...
    @property
    def rgr(self) -> FVector3: ...
    @property
    def rgg(self) -> FVector3: ...
    @property
    def rgb(self) -> FVector3: ...
    @property
    def rgo(self) -> FVector3: ...
    @property
    def rgl(self) -> FVector3: ...
    @property
    def rbr(self) -> FVector3: ...
    @property
    def rbg(self) -> FVector3: ...
    @property
    def rbb(self) -> FVector3: ...
    @property
    def rbo(self) -> FVector3: ...
    @property
    def rbl(self) -> FVector3: ...
    @property
    def ror(self) -> FVector3: ...
    @property
    def rog(self) -> FVector3: ...
    @property
    def rob(self) -> FVector3: ...
    @property
    def roo(self) -> FVector3: ...
    @property
    def rol(self) -> FVector3: ...
    @property
    def rlr(self) -> FVector3: ...
    @property
    def rlg(self) -> FVector3: ...
    @property
    def rlb(self) -> FVector3: ...
    @property
    def rlo(self) -> FVector3: ...
    @property
    def rll(self) -> FVector3: ...
    @property
    def grr(self) -> FVector3: ...
    @property
    def grg(self) -> FVector3: ...
    @property
    def grb(self) -> FVector3: ...
    @property
    def gro(self) -> FVector3: ...
    @property
    def grl(self) -> FVector3: ...
    @property
    def ggr(self) -> FVector3: ...
    @property
    def ggg(self) -> FVector3: ...
    @property
    def ggb(self) -> FVector3: ...
    @property
    def ggo(self) -> FVector3: ...
    @property
    def ggl(self) -> FVector3: ...
    @property
    def gbr(self) -> FVector3: ...
    @property
    def gbg(self) -> FVector3: ...
    @property
    def gbb(self) -> FVector3: ...
    @property
    def gbo(self) -> FVector3: ...
    @property
    def gbl(self) -> FVector3: ...
    @property
    def gor(self) -> FVector3: ...
    @property
    def gog(self) -> FVector3: ...
    @property
    def gob(self) -> FVector3: ...
    @property
    def goo(self) -> FVector3: ...
    @property
    def gol(self) -> FVector3: ...
    @property
    def glr(self) -> FVector3: ...
    @property
    def glg(self) -> FVector3: ...
    @property
    def glb(self) -> FVector3: ...
    @property
    def glo(self) -> FVector3: ...
    @property
    def gll(self) -> FVector3: ...
    @property
    def brr(self) -> FVector3: ...
    @property
    def brg(self) -> FVector3: ...
    @property
    def brb(self) -> FVector3: ...
    @property
    def bro(self) -> FVector3: ...
    @property
    def brl(self) -> FVector3: ...
    @property
    def bgr(self) -> FVector3: ...
    @property
    def bgg(self) -> FVector3: ...
    @property
    def bgb(self) -> FVector3: ...
    @property
    def bgo(self) -> FVector3: ...
    @property
    def bgl(self) -> FVector3: ...
    @property
    def bbr(self) -> FVector3: ...
    @property
    def bbg(self) -> FVector3: ...
    @property
    def bbb(self) -> FVector3: ...
    @property
    def bbo(self) -> FVector3: ...
    @property
    def bbl(self) -> FVector3: ...
    @property
    def bor(self) -> FVector3: ...
    @property
    def bog(self) -> FVector3: ...
    @property
    def bob(self) -> FVector3: ...
    @property
    def boo(self) -> FVector3: ...
    @property
    def bol(self) -> FVector3: ...
    @property
    def blr(self) -> FVector3: ...
    @property
    def blg(self) -> FVector3: ...
    @property
    def blb(self) -> FVector3: ...
    @property
    def blo(self) -> FVector3: ...
    @property
    def bll(self) -> FVector3: ...
    @property
    def orr(self) -> FVector3: ...
    @property
    def org(self) -> FVector3: ...
    @property
    def orb(self) -> FVector3: ...
    @property
    def oro(self) -> FVector3: ...
    @property
    def orl(self) -> FVector3: ...
    @property
    def ogr(self) -> FVector3: ...
    @property
    def ogg(self) -> FVector3: ...
    @property
    def ogb(self) -> FVector3: ...
    @property
    def ogo(self) -> FVector3: ...
    @property
    def ogl(self) -> FVector3: ...
    @property
    def obr(self) -> FVector3: ...
    @property
    def obg(self) -> FVector3: ...
    @property
    def obb(self) -> FVector3: ...
    @property
    def obo(self) -> FVector3: ...
    @property
    def obl(self) -> FVector3: ...
    @property
    def oor(self) -> FVector3: ...
    @property
    def oog(self) -> FVector3: ...
    @property
    def oob(self) -> FVector3: ...
    @property
    def olr(self) -> FVector3: ...
    @property
    def olg(self) -> FVector3: ...
    @property
    def olb(self) -> FVector3: ...
    @property
    def lrr(self) -> FVector3: ...
    @property
    def lrg(self) -> FVector3: ...
    @property
    def lrb(self) -> FVector3: ...
    @property
    def lro(self) -> FVector3: ...
    @property
    def lrl(self) -> FVector3: ...
    @property
    def lgr(self) -> FVector3: ...
    @property
    def lgg(self) -> FVector3: ...
    @property
    def lgb(self) -> FVector3: ...
    @property
    def lgo(self) -> FVector3: ...
    @property
    def lgl(self) -> FVector3: ...
    @property
    def lbr(self) -> FVector3: ...
    @property
    def lbg(self) -> FVector3: ...
    @property
    def lbb(self) -> FVector3: ...
    @property
    def lbo(self) -> FVector3: ...
    @property
    def lbl(self) -> FVector3: ...
    @property
    def lor(self) -> FVector3: ...
    @property
    def log(self) -> FVector3: ...
    @property
    def lob(self) -> FVector3: ...
    @property
    def llr(self) -> FVector3: ...
    @property
    def llg(self) -> FVector3: ...
    @property
    def llb(self) -> FVector3: ...
    @property
    def rrrr(self) -> FVector4: ...
    @property
    def rrrg(self) -> FVector4: ...
    @property
    def rrrb(self) -> FVector4: ...
    @property
    def rrro(self) -> FVector4: ...
    @property
    def rrrl(self) -> FVector4: ...
    @property
    def rrgr(self) -> FVector4: ...
    @property
    def rrgg(self) -> FVector4: ...
    @property
    def rrgb(self) -> FVector4: ...
    @property
    def rrgo(self) -> FVector4: ...
    @property
    def rrgl(self) -> FVector4: ...
    @property
    def rrbr(self) -> FVector4: ...
    @property
    def rrbg(self) -> FVector4: ...
    @property
    def rrbb(self) -> FVector4: ...
    @property
    def rrbo(self) -> FVector4: ...
    @property
    def rrbl(self) -> FVector4: ...
    @property
    def rror(self) -> FVector4: ...
    @property
    def rrog(self) -> FVector4: ...
    @property
    def rrob(self) -> FVector4: ...
    @property
    def rroo(self) -> FVector4: ...
    @property
    def rrol(self) -> FVector4: ...
    @property
    def rrlr(self) -> FVector4: ...
    @property
    def rrlg(self) -> FVector4: ...
    @property
    def rrlb(self) -> FVector4: ...
    @property
    def rrlo(self) -> FVector4: ...
    @property
    def rrll(self) -> FVector4: ...
    @property
    def rgrr(self) -> FVector4: ...
    @property
    def rgrg(self) -> FVector4: ...
    @property
    def rgrb(self) -> FVector4: ...
    @property
    def rgro(self) -> FVector4: ...
    @property
    def rgrl(self) -> FVector4: ...
    @property
    def rggr(self) -> FVector4: ...
    @property
    def rggg(self) -> FVector4: ...
    @property
    def rggb(self) -> FVector4: ...
    @property
    def rggo(self) -> FVector4: ...
    @property
    def rggl(self) -> FVector4: ...
    @property
    def rgbr(self) -> FVector4: ...
    @property
    def rgbg(self) -> FVector4: ...
    @property
    def rgbb(self) -> FVector4: ...
    @property
    def rgbo(self) -> FVector4: ...
    @property
    def rgbl(self) -> FVector4: ...
    @property
    def rgor(self) -> FVector4: ...
    @property
    def rgog(self) -> FVector4: ...
    @property
    def rgob(self) -> FVector4: ...
    @property
    def rgoo(self) -> FVector4: ...
    @property
    def rgol(self) -> FVector4: ...
    @property
    def rglr(self) -> FVector4: ...
    @property
    def rglg(self) -> FVector4: ...
    @property
    def rglb(self) -> FVector4: ...
    @property
    def rglo(self) -> FVector4: ...
    @property
    def rgll(self) -> FVector4: ...
    @property
    def rbrr(self) -> FVector4: ...
    @property
    def rbrg(self) -> FVector4: ...
    @property
    def rbrb(self) -> FVector4: ...
    @property
    def rbro(self) -> FVector4: ...
    @property
    def rbrl(self) -> FVector4: ...
    @property
    def rbgr(self) -> FVector4: ...
    @property
    def rbgg(self) -> FVector4: ...
    @property
    def rbgb(self) -> FVector4: ...
    @property
    def rbgo(self) -> FVector4: ...
    @property
    def rbgl(self) -> FVector4: ...
    @property
    def rbbr(self) -> FVector4: ...
    @property
    def rbbg(self) -> FVector4: ...
    @property
    def rbbb(self) -> FVector4: ...
    @property
    def rbbo(self) -> FVector4: ...
    @property
    def rbbl(self) -> FVector4: ...
    @property
    def rbor(self) -> FVector4: ...
    @property
    def rbog(self) -> FVector4: ...
    @property
    def rbob(self) -> FVector4: ...
    @property
    def rboo(self) -> FVector4: ...
    @property
    def rbol(self) -> FVector4: ...
    @property
    def rblr(self) -> FVector4: ...
    @property
    def rblg(self) -> FVector4: ...
    @property
    def rblb(self) -> FVector4: ...
    @property
    def rblo(self) -> FVector4: ...
    @property
    def rbll(self) -> FVector4: ...
    @property
    def rorr(self) -> FVector4: ...
    @property
    def rorg(self) -> FVector4: ...
    @property
    def rorb(self) -> FVector4: ...
    @property
    def roro(self) -> FVector4: ...
    @property
    def rorl(self) -> FVector4: ...
    @property
    def rogr(self) -> FVector4: ...
    @property
    def rogg(self) -> FVector4: ...
    @property
    def rogb(self) -> FVector4: ...
    @property
    def rogo(self) -> FVector4: ...
    @property
    def rogl(self) -> FVector4: ...
    @property
    def robr(self) -> FVector4: ...
    @property
    def robg(self) -> FVector4: ...
    @property
    def robb(self) -> FVector4: ...
    @property
    def robo(self) -> FVector4: ...
    @property
    def robl(self) -> FVector4: ...
    @property
    def roor(self) -> FVector4: ...
    @property
    def roog(self) -> FVector4: ...
    @property
    def roob(self) -> FVector4: ...
    @property
    def rooo(self) -> FVector4: ...
    @property
    def rool(self) -> FVector4: ...
    @property
    def rolr(self) -> FVector4: ...
    @property
    def rolg(self) -> FVector4: ...
    @property
    def rolb(self) -> FVector4: ...
    @property
    def rolo(self) -> FVector4: ...
    @property
    def roll(self) -> FVector4: ...
    @property
    def rlrr(self) -> FVector4: ...
    @property
    def rlrg(self) -> FVector4: ...
    @property
    def rlrb(self) -> FVector4: ...
    @property
    def rlro(self) -> FVector4: ...
    @property
    def rlrl(self) -> FVector4: ...
    @property
    def rlgr(self) -> FVector4: ...
    @property
    def rlgg(self) -> FVector4: ...
    @property
    def rlgb(self) -> FVector4: ...
    @property
    def rlgo(self) -> FVector4: ...
    @property
    def rlgl(self) -> FVector4: ...
    @property
    def rlbr(self) -> FVector4: ...
    @property
    def rlbg(self) -> FVector4: ...
    @property
    def rlbb(self) -> FVector4: ...
    @property
    def rlbo(self) -> FVector4: ...
    @property
    def rlbl(self) -> FVector4: ...
    @property
    def rlor(self) -> FVector4: ...
    @property
    def rlog(self) -> FVector4: ...
    @property
    def rlob(self) -> FVector4: ...
    @property
    def rloo(self) -> FVector4: ...
    @property
    def rlol(self) -> FVector4: ...
    @property
    def rllr(self) -> FVector4: ...
    @property
    def rllg(self) -> FVector4: ...
    @property
    def rllb(self) -> FVector4: ...
    @property
    def rllo(self) -> FVector4: ...
    @property
    def rlll(self) -> FVector4: ...
    @property
    def grrr(self) -> FVector4: ...
    @property
    def grrg(self) -> FVector4: ...
    @property
    def grrb(self) -> FVector4: ...
    @property
    def grro(self) -> FVector4: ...
    @property
    def grrl(self) -> FVector4: ...
    @property
    def grgr(self) -> FVector4: ...
    @property
    def grgg(self) -> FVector4: ...
    @property
    def grgb(self) -> FVector4: ...
    @property
    def grgo(self) -> FVector4: ...
    @property
    def grgl(self) -> FVector4: ...
    @property
    def grbr(self) -> FVector4: ...
    @property
    def grbg(self) -> FVector4: ...
    @property
    def grbb(self) -> FVector4: ...
    @property
    def grbo(self) -> FVector4: ...
    @property
    def grbl(self) -> FVector4: ...
    @property
    def gror(self) -> FVector4: ...
    @property
    def grog(self) -> FVector4: ...
    @property
    def grob(self) -> FVector4: ...
    @property
    def groo(self) -> FVector4: ...
    @property
    def grol(self) -> FVector4: ...
    @property
    def grlr(self) -> FVector4: ...
    @property
    def grlg(self) -> FVector4: ...
    @property
    def grlb(self) -> FVector4: ...
    @property
    def grlo(self) -> FVector4: ...
    @property
    def grll(self) -> FVector4: ...
    @property
    def ggrr(self) -> FVector4: ...
    @property
    def ggrg(self) -> FVector4: ...
    @property
    def ggrb(self) -> FVector4: ...
    @property
    def ggro(self) -> FVector4: ...
    @property
    def ggrl(self) -> FVector4: ...
    @property
    def gggr(self) -> FVector4: ...
    @property
    def gggg(self) -> FVector4: ...
    @property
    def gggb(self) -> FVector4: ...
    @property
    def gggo(self) -> FVector4: ...
    @property
    def gggl(self) -> FVector4: ...
    @property
    def ggbr(self) -> FVector4: ...
    @property
    def ggbg(self) -> FVector4: ...
    @property
    def ggbb(self) -> FVector4: ...
    @property
    def ggbo(self) -> FVector4: ...
    @property
    def ggbl(self) -> FVector4: ...
    @property
    def ggor(self) -> FVector4: ...
    @property
    def ggog(self) -> FVector4: ...
    @property
    def ggob(self) -> FVector4: ...
    @property
    def ggoo(self) -> FVector4: ...
    @property
    def ggol(self) -> FVector4: ...
    @property
    def gglr(self) -> FVector4: ...
    @property
    def gglg(self) -> FVector4: ...
    @property
    def gglb(self) -> FVector4: ...
    @property
    def gglo(self) -> FVector4: ...
    @property
    def ggll(self) -> FVector4: ...
    @property
    def gbrr(self) -> FVector4: ...
    @property
    def gbrg(self) -> FVector4: ...
    @property
    def gbrb(self) -> FVector4: ...
    @property
    def gbro(self) -> FVector4: ...
    @property
    def gbrl(self) -> FVector4: ...
    @property
    def gbgr(self) -> FVector4: ...
    @property
    def gbgg(self) -> FVector4: ...
    @property
    def gbgb(self) -> FVector4: ...
    @property
    def gbgo(self) -> FVector4: ...
    @property
    def gbgl(self) -> FVector4: ...
    @property
    def gbbr(self) -> FVector4: ...
    @property
    def gbbg(self) -> FVector4: ...
    @property
    def gbbb(self) -> FVector4: ...
    @property
    def gbbo(self) -> FVector4: ...
    @property
    def gbbl(self) -> FVector4: ...
    @property
    def gbor(self) -> FVector4: ...
    @property
    def gbog(self) -> FVector4: ...
    @property
    def gbob(self) -> FVector4: ...
    @property
    def gboo(self) -> FVector4: ...
    @property
    def gbol(self) -> FVector4: ...
    @property
    def gblr(self) -> FVector4: ...
    @property
    def gblg(self) -> FVector4: ...
    @property
    def gblb(self) -> FVector4: ...
    @property
    def gblo(self) -> FVector4: ...
    @property
    def gbll(self) -> FVector4: ...
    @property
    def gorr(self) -> FVector4: ...
    @property
    def gorg(self) -> FVector4: ...
    @property
    def gorb(self) -> FVector4: ...
    @property
    def goro(self) -> FVector4: ...
    @property
    def gorl(self) -> FVector4: ...
    @property
    def gogr(self) -> FVector4: ...
    @property
    def gogg(self) -> FVector4: ...
    @property
    def gogb(self) -> FVector4: ...
    @property
    def gogo(self) -> FVector4: ...
    @property
    def gogl(self) -> FVector4: ...
    @property
    def gobr(self) -> FVector4: ...
    @property
    def gobg(self) -> FVector4: ...
    @property
    def gobb(self) -> FVector4: ...
    @property
    def gobo(self) -> FVector4: ...
    @property
    def gobl(self) -> FVector4: ...
    @property
    def goor(self) -> FVector4: ...
    @property
    def goog(self) -> FVector4: ...
    @property
    def goob(self) -> FVector4: ...
    @property
    def gooo(self) -> FVector4: ...
    @property
    def gool(self) -> FVector4: ...
    @property
    def golr(self) -> FVector4: ...
    @property
    def golg(self) -> FVector4: ...
    @property
    def golb(self) -> FVector4: ...
    @property
    def golo(self) -> FVector4: ...
    @property
    def goll(self) -> FVector4: ...
    @property
    def glrr(self) -> FVector4: ...
    @property
    def glrg(self) -> FVector4: ...
    @property
    def glrb(self) -> FVector4: ...
    @property
    def glro(self) -> FVector4: ...
    @property
    def glrl(self) -> FVector4: ...
    @property
    def glgr(self) -> FVector4: ...
    @property
    def glgg(self) -> FVector4: ...
    @property
    def glgb(self) -> FVector4: ...
    @property
    def glgo(self) -> FVector4: ...
    @property
    def glgl(self) -> FVector4: ...
    @property
    def glbr(self) -> FVector4: ...
    @property
    def glbg(self) -> FVector4: ...
    @property
    def glbb(self) -> FVector4: ...
    @property
    def glbo(self) -> FVector4: ...
    @property
    def glbl(self) -> FVector4: ...
    @property
    def glor(self) -> FVector4: ...
    @property
    def glog(self) -> FVector4: ...
    @property
    def glob(self) -> FVector4: ...
    @property
    def gloo(self) -> FVector4: ...
    @property
    def glol(self) -> FVector4: ...
    @property
    def gllr(self) -> FVector4: ...
    @property
    def gllg(self) -> FVector4: ...
    @property
    def gllb(self) -> FVector4: ...
    @property
    def gllo(self) -> FVector4: ...
    @property
    def glll(self) -> FVector4: ...
    @property
    def brrr(self) -> FVector4: ...
    @property
    def brrg(self) -> FVector4: ...
    @property
    def brrb(self) -> FVector4: ...
    @property
    def brro(self) -> FVector4: ...
    @property
    def brrl(self) -> FVector4: ...
    @property
    def brgr(self) -> FVector4: ...
    @property
    def brgg(self) -> FVector4: ...
    @property
    def brgb(self) -> FVector4: ...
    @property
    def brgo(self) -> FVector4: ...
    @property
    def brgl(self) -> FVector4: ...
    @property
    def brbr(self) -> FVector4: ...
    @property
    def brbg(self) -> FVector4: ...
    @property
    def brbb(self) -> FVector4: ...
    @property
    def brbo(self) -> FVector4: ...
    @property
    def brbl(self) -> FVector4: ...
    @property
    def bror(self) -> FVector4: ...
    @property
    def brog(self) -> FVector4: ...
    @property
    def brob(self) -> FVector4: ...
    @property
    def broo(self) -> FVector4: ...
    @property
    def brol(self) -> FVector4: ...
    @property
    def brlr(self) -> FVector4: ...
    @property
    def brlg(self) -> FVector4: ...
    @property
    def brlb(self) -> FVector4: ...
    @property
    def brlo(self) -> FVector4: ...
    @property
    def brll(self) -> FVector4: ...
    @property
    def bgrr(self) -> FVector4: ...
    @property
    def bgrg(self) -> FVector4: ...
    @property
    def bgrb(self) -> FVector4: ...
    @property
    def bgro(self) -> FVector4: ...
    @property
    def bgrl(self) -> FVector4: ...
    @property
    def bggr(self) -> FVector4: ...
    @property
    def bggg(self) -> FVector4: ...
    @property
    def bggb(self) -> FVector4: ...
    @property
    def bggo(self) -> FVector4: ...
    @property
    def bggl(self) -> FVector4: ...
    @property
    def bgbr(self) -> FVector4: ...
    @property
    def bgbg(self) -> FVector4: ...
    @property
    def bgbb(self) -> FVector4: ...
    @property
    def bgbo(self) -> FVector4: ...
    @property
    def bgbl(self) -> FVector4: ...
    @property
    def bgor(self) -> FVector4: ...
    @property
    def bgog(self) -> FVector4: ...
    @property
    def bgob(self) -> FVector4: ...
    @property
    def bgoo(self) -> FVector4: ...
    @property
    def bgol(self) -> FVector4: ...
    @property
    def bglr(self) -> FVector4: ...
    @property
    def bglg(self) -> FVector4: ...
    @property
    def bglb(self) -> FVector4: ...
    @property
    def bglo(self) -> FVector4: ...
    @property
    def bgll(self) -> FVector4: ...
    @property
    def bbrr(self) -> FVector4: ...
    @property
    def bbrg(self) -> FVector4: ...
    @property
    def bbrb(self) -> FVector4: ...
    @property
    def bbro(self) -> FVector4: ...
    @property
    def bbrl(self) -> FVector4: ...
    @property
    def bbgr(self) -> FVector4: ...
    @property
    def bbgg(self) -> FVector4: ...
    @property
    def bbgb(self) -> FVector4: ...
    @property
    def bbgo(self) -> FVector4: ...
    @property
    def bbgl(self) -> FVector4: ...
    @property
    def bbbr(self) -> FVector4: ...
    @property
    def bbbg(self) -> FVector4: ...
    @property
    def bbbb(self) -> FVector4: ...
    @property
    def bbbo(self) -> FVector4: ...
    @property
    def bbbl(self) -> FVector4: ...
    @property
    def bbor(self) -> FVector4: ...
    @property
    def bbog(self) -> FVector4: ...
    @property
    def bbob(self) -> FVector4: ...
    @property
    def bboo(self) -> FVector4: ...
    @property
    def bbol(self) -> FVector4: ...
    @property
    def bblr(self) -> FVector4: ...
    @property
    def bblg(self) -> FVector4: ...
    @property
    def bblb(self) -> FVector4: ...
    @property
    def bblo(self) -> FVector4: ...
    @property
    def bbll(self) -> FVector4: ...
    @property
    def borr(self) -> FVector4: ...
    @property
    def borg(self) -> FVector4: ...
    @property
    def borb(self) -> FVector4: ...
    @property
    def boro(self) -> FVector4: ...
    @property
    def borl(self) -> FVector4: ...
    @property
    def bogr(self) -> FVector4: ...
    @property
    def bogg(self) -> FVector4: ...
    @property
    def bogb(self) -> FVector4: ...
    @property
    def bogo(self) -> FVector4: ...
    @property
    def bogl(self) -> FVector4: ...
    @property
    def bobr(self) -> FVector4: ...
    @property
    def bobg(self) -> FVector4: ...
    @property
    def bobb(self) -> FVector4: ...
    @property
    def bobo(self) -> FVector4: ...
    @property
    def bobl(self) -> FVector4: ...
    @property
    def boor(self) -> FVector4: ...
    @property
    def boog(self) -> FVector4: ...
    @property
    def boob(self) -> FVector4: ...
    @property
    def booo(self) -> FVector4: ...
    @property
    def bool(self) -> FVector4: ...
    @property
    def bolr(self) -> FVector4: ...
    @property
    def bolg(self) -> FVector4: ...
    @property
    def bolb(self) -> FVector4: ...
    @property
    def bolo(self) -> FVector4: ...
    @property
    def boll(self) -> FVector4: ...
    @property
    def blrr(self) -> FVector4: ...
    @property
    def blrg(self) -> FVector4: ...
    @property
    def blrb(self) -> FVector4: ...
    @property
    def blro(self) -> FVector4: ...
    @property
    def blrl(self) -> FVector4: ...
    @property
    def blgr(self) -> FVector4: ...
    @property
    def blgg(self) -> FVector4: ...
    @property
    def blgb(self) -> FVector4: ...
    @property
    def blgo(self) -> FVector4: ...
    @property
    def blgl(self) -> FVector4: ...
    @property
    def blbr(self) -> FVector4: ...
    @property
    def blbg(self) -> FVector4: ...
    @property
    def blbb(self) -> FVector4: ...
    @property
    def blbo(self) -> FVector4: ...
    @property
    def blbl(self) -> FVector4: ...
    @property
    def blor(self) -> FVector4: ...
    @property
    def blog(self) -> FVector4: ...
    @property
    def blob(self) -> FVector4: ...
    @property
    def bloo(self) -> FVector4: ...
    @property
    def blol(self) -> FVector4: ...
    @property
    def bllr(self) -> FVector4: ...
    @property
    def bllg(self) -> FVector4: ...
    @property
    def bllb(self) -> FVector4: ...
    @property
    def bllo(self) -> FVector4: ...
    @property
    def blll(self) -> FVector4: ...
    @property
    def orrr(self) -> FVector4: ...
    @property
    def orrg(self) -> FVector4: ...
    @property
    def orrb(self) -> FVector4: ...
    @property
    def orro(self) -> FVector4: ...
    @property
    def orrl(self) -> FVector4: ...
    @property
    def orgr(self) -> FVector4: ...
    @property
    def orgg(self) -> FVector4: ...
    @property
    def orgb(self) -> FVector4: ...
    @property
    def orgo(self) -> FVector4: ...
    @property
    def orgl(self) -> FVector4: ...
    @property
    def orbr(self) -> FVector4: ...
    @property
    def orbg(self) -> FVector4: ...
    @property
    def orbb(self) -> FVector4: ...
    @property
    def orbo(self) -> FVector4: ...
    @property
    def orbl(self) -> FVector4: ...
    @property
    def oror(self) -> FVector4: ...
    @property
    def orog(self) -> FVector4: ...
    @property
    def orob(self) -> FVector4: ...
    @property
    def oroo(self) -> FVector4: ...
    @property
    def orol(self) -> FVector4: ...
    @property
    def orlr(self) -> FVector4: ...
    @property
    def orlg(self) -> FVector4: ...
    @property
    def orlb(self) -> FVector4: ...
    @property
    def orlo(self) -> FVector4: ...
    @property
    def orll(self) -> FVector4: ...
    @property
    def ogrr(self) -> FVector4: ...
    @property
    def ogrg(self) -> FVector4: ...
    @property
    def ogrb(self) -> FVector4: ...
    @property
    def ogro(self) -> FVector4: ...
    @property
    def ogrl(self) -> FVector4: ...
    @property
    def oggr(self) -> FVector4: ...
    @property
    def oggg(self) -> FVector4: ...
    @property
    def oggb(self) -> FVector4: ...
    @property
    def oggo(self) -> FVector4: ...
    @property
    def oggl(self) -> FVector4: ...
    @property
    def ogbr(self) -> FVector4: ...
    @property
    def ogbg(self) -> FVector4: ...
    @property
    def ogbb(self) -> FVector4: ...
    @property
    def ogbo(self) -> FVector4: ...
    @property
    def ogbl(self) -> FVector4: ...
    @property
    def ogor(self) -> FVector4: ...
    @property
    def ogog(self) -> FVector4: ...
    @property
    def ogob(self) -> FVector4: ...
    @property
    def ogoo(self) -> FVector4: ...
    @property
    def ogol(self) -> FVector4: ...
    @property
    def oglr(self) -> FVector4: ...
    @property
    def oglg(self) -> FVector4: ...
    @property
    def oglb(self) -> FVector4: ...
    @property
    def oglo(self) -> FVector4: ...
    @property
    def ogll(self) -> FVector4: ...
    @property
    def obrr(self) -> FVector4: ...
    @property
    def obrg(self) -> FVector4: ...
    @property
    def obrb(self) -> FVector4: ...
    @property
    def obro(self) -> FVector4: ...
    @property
    def obrl(self) -> FVector4: ...
    @property
    def obgr(self) -> FVector4: ...
    @property
    def obgg(self) -> FVector4: ...
    @property
    def obgb(self) -> FVector4: ...
    @property
    def obgo(self) -> FVector4: ...
    @property
    def obgl(self) -> FVector4: ...
    @property
    def obbr(self) -> FVector4: ...
    @property
    def obbg(self) -> FVector4: ...
    @property
    def obbb(self) -> FVector4: ...
    @property
    def obbo(self) -> FVector4: ...
    @property
    def obbl(self) -> FVector4: ...
    @property
    def obor(self) -> FVector4: ...
    @property
    def obog(self) -> FVector4: ...
    @property
    def obob(self) -> FVector4: ...
    @property
    def oboo(self) -> FVector4: ...
    @property
    def obol(self) -> FVector4: ...
    @property
    def oblr(self) -> FVector4: ...
    @property
    def oblg(self) -> FVector4: ...
    @property
    def oblb(self) -> FVector4: ...
    @property
    def oblo(self) -> FVector4: ...
    @property
    def obll(self) -> FVector4: ...
    @property
    def oorr(self) -> FVector4: ...
    @property
    def oorg(self) -> FVector4: ...
    @property
    def oorb(self) -> FVector4: ...
    @property
    def ooro(self) -> FVector4: ...
    @property
    def oorl(self) -> FVector4: ...
    @property
    def oogr(self) -> FVector4: ...
    @property
    def oogg(self) -> FVector4: ...
    @property
    def oogb(self) -> FVector4: ...
    @property
    def oogo(self) -> FVector4: ...
    @property
    def oogl(self) -> FVector4: ...
    @property
    def oobr(self) -> FVector4: ...
    @property
    def oobg(self) -> FVector4: ...
    @property
    def oobb(self) -> FVector4: ...
    @property
    def oobo(self) -> FVector4: ...
    @property
    def oobl(self) -> FVector4: ...
    @property
    def ooor(self) -> FVector4: ...
    @property
    def ooog(self) -> FVector4: ...
    @property
    def ooob(self) -> FVector4: ...
    @property
    def oolr(self) -> FVector4: ...
    @property
    def oolg(self) -> FVector4: ...
    @property
    def oolb(self) -> FVector4: ...
    @property
    def olrr(self) -> FVector4: ...
    @property
    def olrg(self) -> FVector4: ...
    @property
    def olrb(self) -> FVector4: ...
    @property
    def olro(self) -> FVector4: ...
    @property
    def olrl(self) -> FVector4: ...
    @property
    def olgr(self) -> FVector4: ...
    @property
    def olgg(self) -> FVector4: ...
    @property
    def olgb(self) -> FVector4: ...
    @property
    def olgo(self) -> FVector4: ...
    @property
    def olgl(self) -> FVector4: ...
    @property
    def olbr(self) -> FVector4: ...
    @property
    def olbg(self) -> FVector4: ...
    @property
    def olbb(self) -> FVector4: ...
    @property
    def olbo(self) -> FVector4: ...
    @property
    def olbl(self) -> FVector4: ...
    @property
    def olor(self) -> FVector4: ...
    @property
    def olog(self) -> FVector4: ...
    @property
    def olob(self) -> FVector4: ...
    @property
    def ollr(self) -> FVector4: ...
    @property
    def ollg(self) -> FVector4: ...
    @property
    def ollb(self) -> FVector4: ...
    @property
    def lrrr(self) -> FVector4: ...
    @property
    def lrrg(self) -> FVector4: ...
    @property
    def lrrb(self) -> FVector4: ...
    @property
    def lrro(self) -> FVector4: ...
    @property
    def lrrl(self) -> FVector4: ...
    @property
    def lrgr(self) -> FVector4: ...
    @property
    def lrgg(self) -> FVector4: ...
    @property
    def lrgb(self) -> FVector4: ...
    @property
    def lrgo(self) -> FVector4: ...
    @property
    def lrgl(self) -> FVector4: ...
    @property
    def lrbr(self) -> FVector4: ...
    @property
    def lrbg(self) -> FVector4: ...
    @property
    def lrbb(self) -> FVector4: ...
    @property
    def lrbo(self) -> FVector4: ...
    @property
    def lrbl(self) -> FVector4: ...
    @property
    def lror(self) -> FVector4: ...
    @property
    def lrog(self) -> FVector4: ...
    @property
    def lrob(self) -> FVector4: ...
    @property
    def lroo(self) -> FVector4: ...
    @property
    def lrol(self) -> FVector4: ...
    @property
    def lrlr(self) -> FVector4: ...
    @property
    def lrlg(self) -> FVector4: ...
    @property
    def lrlb(self) -> FVector4: ...
    @property
    def lrlo(self) -> FVector4: ...
    @property
    def lrll(self) -> FVector4: ...
    @property
    def lgrr(self) -> FVector4: ...
    @property
    def lgrg(self) -> FVector4: ...
    @property
    def lgrb(self) -> FVector4: ...
    @property
    def lgro(self) -> FVector4: ...
    @property
    def lgrl(self) -> FVector4: ...
    @property
    def lggr(self) -> FVector4: ...
    @property
    def lggg(self) -> FVector4: ...
    @property
    def lggb(self) -> FVector4: ...
    @property
    def lggo(self) -> FVector4: ...
    @property
    def lggl(self) -> FVector4: ...
    @property
    def lgbr(self) -> FVector4: ...
    @property
    def lgbg(self) -> FVector4: ...
    @property
    def lgbb(self) -> FVector4: ...
    @property
    def lgbo(self) -> FVector4: ...
    @property
    def lgbl(self) -> FVector4: ...
    @property
    def lgor(self) -> FVector4: ...
    @property
    def lgog(self) -> FVector4: ...
    @property
    def lgob(self) -> FVector4: ...
    @property
    def lgoo(self) -> FVector4: ...
    @property
    def lgol(self) -> FVector4: ...
    @property
    def lglr(self) -> FVector4: ...
    @property
    def lglg(self) -> FVector4: ...
    @property
    def lglb(self) -> FVector4: ...
    @property
    def lglo(self) -> FVector4: ...
    @property
    def lgll(self) -> FVector4: ...
    @property
    def lbrr(self) -> FVector4: ...
    @property
    def lbrg(self) -> FVector4: ...
    @property
    def lbrb(self) -> FVector4: ...
    @property
    def lbro(self) -> FVector4: ...
    @property
    def lbrl(self) -> FVector4: ...
    @property
    def lbgr(self) -> FVector4: ...
    @property
    def lbgg(self) -> FVector4: ...
    @property
    def lbgb(self) -> FVector4: ...
    @property
    def lbgo(self) -> FVector4: ...
    @property
    def lbgl(self) -> FVector4: ...
    @property
    def lbbr(self) -> FVector4: ...
    @property
    def lbbg(self) -> FVector4: ...
    @property
    def lbbb(self) -> FVector4: ...
    @property
    def lbbo(self) -> FVector4: ...
    @property
    def lbbl(self) -> FVector4: ...
    @property
    def lbor(self) -> FVector4: ...
    @property
    def lbog(self) -> FVector4: ...
    @property
    def lbob(self) -> FVector4: ...
    @property
    def lboo(self) -> FVector4: ...
    @property
    def lbol(self) -> FVector4: ...
    @property
    def lblr(self) -> FVector4: ...
    @property
    def lblg(self) -> FVector4: ...
    @property
    def lblb(self) -> FVector4: ...
    @property
    def lblo(self) -> FVector4: ...
    @property
    def lbll(self) -> FVector4: ...
    @property
    def lorr(self) -> FVector4: ...
    @property
    def lorg(self) -> FVector4: ...
    @property
    def lorb(self) -> FVector4: ...
    @property
    def loro(self) -> FVector4: ...
    @property
    def lorl(self) -> FVector4: ...
    @property
    def logr(self) -> FVector4: ...
    @property
    def logg(self) -> FVector4: ...
    @property
    def logb(self) -> FVector4: ...
    @property
    def logo(self) -> FVector4: ...
    @property
    def logl(self) -> FVector4: ...
    @property
    def lobr(self) -> FVector4: ...
    @property
    def lobg(self) -> FVector4: ...
    @property
    def lobb(self) -> FVector4: ...
    @property
    def lobo(self) -> FVector4: ...
    @property
    def lobl(self) -> FVector4: ...
    @property
    def loor(self) -> FVector4: ...
    @property
    def loog(self) -> FVector4: ...
    @property
    def loob(self) -> FVector4: ...
    @property
    def lolr(self) -> FVector4: ...
    @property
    def lolg(self) -> FVector4: ...
    @property
    def lolb(self) -> FVector4: ...
    @property
    def llrr(self) -> FVector4: ...
    @property
    def llrg(self) -> FVector4: ...
    @property
    def llrb(self) -> FVector4: ...
    @property
    def llro(self) -> FVector4: ...
    @property
    def llrl(self) -> FVector4: ...
    @property
    def llgr(self) -> FVector4: ...
    @property
    def llgg(self) -> FVector4: ...
    @property
    def llgb(self) -> FVector4: ...
    @property
    def llgo(self) -> FVector4: ...
    @property
    def llgl(self) -> FVector4: ...
    @property
    def llbr(self) -> FVector4: ...
    @property
    def llbg(self) -> FVector4: ...
    @property
    def llbb(self) -> FVector4: ...
    @property
    def llbo(self) -> FVector4: ...
    @property
    def llbl(self) -> FVector4: ...
    @property
    def llor(self) -> FVector4: ...
    @property
    def llog(self) -> FVector4: ...
    @property
    def llob(self) -> FVector4: ...
    @property
    def lllr(self) -> FVector4: ...
    @property
    def lllg(self) -> FVector4: ...
    @property
    def lllb(self) -> FVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def t(self) -> float: ...
    @property
    def q(self) -> float: ...
    @property
    def ss(self) -> FVector2: ...
    @property
    def st(self) -> FVector2: ...
    @property
    def sq(self) -> FVector2: ...
    @property
    def so(self) -> FVector2: ...
    @property
    def sl(self) -> FVector2: ...
    @property
    def ts(self) -> FVector2: ...
    @property
    def tt(self) -> FVector2: ...
    @property
    def tq(self) -> FVector2: ...
    @property
    def to(self) -> FVector2: ...
    @property
    def tl(self) -> FVector2: ...
    @property
    def qs(self) -> FVector2: ...
    @property
    def qt(self) -> FVector2: ...
    @property
    def qq(self) -> FVector2: ...
    @property
    def qo(self) -> FVector2: ...
    @property
    def ql(self) -> FVector2: ...
    @property
    def os(self) -> FVector2: ...
    @property
    def ot(self) -> FVector2: ...
    @property
    def oq(self) -> FVector2: ...
    @property
    def ls(self) -> FVector2: ...
    @property
    def lt(self) -> FVector2: ...
    @property
    def lq(self) -> FVector2: ...
    @property
    def sss(self) -> FVector3: ...
    @property
    def sst(self) -> FVector3: ...
    @property
    def ssq(self) -> FVector3: ...
    @property
    def sso(self) -> FVector3: ...
    @property
    def ssl(self) -> FVector3: ...
    @property
    def sts(self) -> FVector3: ...
    @property
    def stt(self) -> FVector3: ...
    @property
    def stq(self) -> FVector3: ...
    @property
    def sto(self) -> FVector3: ...
    @property
    def stl(self) -> FVector3: ...
    @property
    def sqs(self) -> FVector3: ...
    @property
    def sqt(self) -> FVector3: ...
    @property
    def sqq(self) -> FVector3: ...
    @property
    def sqo(self) -> FVector3: ...
    @property
    def sql(self) -> FVector3: ...
    @property
    def sos(self) -> FVector3: ...
    @property
    def sot(self) -> FVector3: ...
    @property
    def soq(self) -> FVector3: ...
    @property
    def soo(self) -> FVector3: ...
    @property
    def sol(self) -> FVector3: ...
    @property
    def sls(self) -> FVector3: ...
    @property
    def slt(self) -> FVector3: ...
    @property
    def slq(self) -> FVector3: ...
    @property
    def slo(self) -> FVector3: ...
    @property
    def sll(self) -> FVector3: ...
    @property
    def tss(self) -> FVector3: ...
    @property
    def tst(self) -> FVector3: ...
    @property
    def tsq(self) -> FVector3: ...
    @property
    def tso(self) -> FVector3: ...
    @property
    def tsl(self) -> FVector3: ...
    @property
    def tts(self) -> FVector3: ...
    @property
    def ttt(self) -> FVector3: ...
    @property
    def ttq(self) -> FVector3: ...
    @property
    def tto(self) -> FVector3: ...
    @property
    def ttl(self) -> FVector3: ...
    @property
    def tqs(self) -> FVector3: ...
    @property
    def tqt(self) -> FVector3: ...
    @property
    def tqq(self) -> FVector3: ...
    @property
    def tqo(self) -> FVector3: ...
    @property
    def tql(self) -> FVector3: ...
    @property
    def tos(self) -> FVector3: ...
    @property
    def tot(self) -> FVector3: ...
    @property
    def toq(self) -> FVector3: ...
    @property
    def too(self) -> FVector3: ...
    @property
    def tol(self) -> FVector3: ...
    @property
    def tls(self) -> FVector3: ...
    @property
    def tlt(self) -> FVector3: ...
    @property
    def tlq(self) -> FVector3: ...
    @property
    def tlo(self) -> FVector3: ...
    @property
    def tll(self) -> FVector3: ...
    @property
    def qss(self) -> FVector3: ...
    @property
    def qst(self) -> FVector3: ...
    @property
    def qsq(self) -> FVector3: ...
    @property
    def qso(self) -> FVector3: ...
    @property
    def qsl(self) -> FVector3: ...
    @property
    def qts(self) -> FVector3: ...
    @property
    def qtt(self) -> FVector3: ...
    @property
    def qtq(self) -> FVector3: ...
    @property
    def qto(self) -> FVector3: ...
    @property
    def qtl(self) -> FVector3: ...
    @property
    def qqs(self) -> FVector3: ...
    @property
    def qqt(self) -> FVector3: ...
    @property
    def qqq(self) -> FVector3: ...
    @property
    def qqo(self) -> FVector3: ...
    @property
    def qql(self) -> FVector3: ...
    @property
    def qos(self) -> FVector3: ...
    @property
    def qot(self) -> FVector3: ...
    @property
    def qoq(self) -> FVector3: ...
    @property
    def qoo(self) -> FVector3: ...
    @property
    def qol(self) -> FVector3: ...
    @property
    def qls(self) -> FVector3: ...
    @property
    def qlt(self) -> FVector3: ...
    @property
    def qlq(self) -> FVector3: ...
    @property
    def qlo(self) -> FVector3: ...
    @property
    def qll(self) -> FVector3: ...
    @property
    def oss(self) -> FVector3: ...
    @property
    def ost(self) -> FVector3: ...
    @property
    def osq(self) -> FVector3: ...
    @property
    def oso(self) -> FVector3: ...
    @property
    def osl(self) -> FVector3: ...
    @property
    def ots(self) -> FVector3: ...
    @property
    def ott(self) -> FVector3: ...
    @property
    def otq(self) -> FVector3: ...
    @property
    def oto(self) -> FVector3: ...
    @property
    def otl(self) -> FVector3: ...
    @property
    def oqs(self) -> FVector3: ...
    @property
    def oqt(self) -> FVector3: ...
    @property
    def oqq(self) -> FVector3: ...
    @property
    def oqo(self) -> FVector3: ...
    @property
    def oql(self) -> FVector3: ...
    @property
    def oos(self) -> FVector3: ...
    @property
    def oot(self) -> FVector3: ...
    @property
    def ooq(self) -> FVector3: ...
    @property
    def ols(self) -> FVector3: ...
    @property
    def olt(self) -> FVector3: ...
    @property
    def olq(self) -> FVector3: ...
    @property
    def lss(self) -> FVector3: ...
    @property
    def lst(self) -> FVector3: ...
    @property
    def lsq(self) -> FVector3: ...
    @property
    def lso(self) -> FVector3: ...
    @property
    def lsl(self) -> FVector3: ...
    @property
    def lts(self) -> FVector3: ...
    @property
    def ltt(self) -> FVector3: ...
    @property
    def ltq(self) -> FVector3: ...
    @property
    def lto(self) -> FVector3: ...
    @property
    def ltl(self) -> FVector3: ...
    @property
    def lqs(self) -> FVector3: ...
    @property
    def lqt(self) -> FVector3: ...
    @property
    def lqq(self) -> FVector3: ...
    @property
    def lqo(self) -> FVector3: ...
    @property
    def lql(self) -> FVector3: ...
    @property
    def los(self) -> FVector3: ...
    @property
    def lot(self) -> FVector3: ...
    @property
    def loq(self) -> FVector3: ...
    @property
    def lls(self) -> FVector3: ...
    @property
    def llt(self) -> FVector3: ...
    @property
    def llq(self) -> FVector3: ...
    @property
    def ssss(self) -> FVector4: ...
    @property
    def ssst(self) -> FVector4: ...
    @property
    def sssq(self) -> FVector4: ...
    @property
    def ssso(self) -> FVector4: ...
    @property
    def sssl(self) -> FVector4: ...
    @property
    def ssts(self) -> FVector4: ...
    @property
    def sstt(self) -> FVector4: ...
    @property
    def sstq(self) -> FVector4: ...
    @property
    def ssto(self) -> FVector4: ...
    @property
    def sstl(self) -> FVector4: ...
    @property
    def ssqs(self) -> FVector4: ...
    @property
    def ssqt(self) -> FVector4: ...
    @property
    def ssqq(self) -> FVector4: ...
    @property
    def ssqo(self) -> FVector4: ...
    @property
    def ssql(self) -> FVector4: ...
    @property
    def ssos(self) -> FVector4: ...
    @property
    def ssot(self) -> FVector4: ...
    @property
    def ssoq(self) -> FVector4: ...
    @property
    def ssoo(self) -> FVector4: ...
    @property
    def ssol(self) -> FVector4: ...
    @property
    def ssls(self) -> FVector4: ...
    @property
    def sslt(self) -> FVector4: ...
    @property
    def sslq(self) -> FVector4: ...
    @property
    def sslo(self) -> FVector4: ...
    @property
    def ssll(self) -> FVector4: ...
    @property
    def stss(self) -> FVector4: ...
    @property
    def stst(self) -> FVector4: ...
    @property
    def stsq(self) -> FVector4: ...
    @property
    def stso(self) -> FVector4: ...
    @property
    def stsl(self) -> FVector4: ...
    @property
    def stts(self) -> FVector4: ...
    @property
    def sttt(self) -> FVector4: ...
    @property
    def sttq(self) -> FVector4: ...
    @property
    def stto(self) -> FVector4: ...
    @property
    def sttl(self) -> FVector4: ...
    @property
    def stqs(self) -> FVector4: ...
    @property
    def stqt(self) -> FVector4: ...
    @property
    def stqq(self) -> FVector4: ...
    @property
    def stqo(self) -> FVector4: ...
    @property
    def stql(self) -> FVector4: ...
    @property
    def stos(self) -> FVector4: ...
    @property
    def stot(self) -> FVector4: ...
    @property
    def stoq(self) -> FVector4: ...
    @property
    def stoo(self) -> FVector4: ...
    @property
    def stol(self) -> FVector4: ...
    @property
    def stls(self) -> FVector4: ...
    @property
    def stlt(self) -> FVector4: ...
    @property
    def stlq(self) -> FVector4: ...
    @property
    def stlo(self) -> FVector4: ...
    @property
    def stll(self) -> FVector4: ...
    @property
    def sqss(self) -> FVector4: ...
    @property
    def sqst(self) -> FVector4: ...
    @property
    def sqsq(self) -> FVector4: ...
    @property
    def sqso(self) -> FVector4: ...
    @property
    def sqsl(self) -> FVector4: ...
    @property
    def sqts(self) -> FVector4: ...
    @property
    def sqtt(self) -> FVector4: ...
    @property
    def sqtq(self) -> FVector4: ...
    @property
    def sqto(self) -> FVector4: ...
    @property
    def sqtl(self) -> FVector4: ...
    @property
    def sqqs(self) -> FVector4: ...
    @property
    def sqqt(self) -> FVector4: ...
    @property
    def sqqq(self) -> FVector4: ...
    @property
    def sqqo(self) -> FVector4: ...
    @property
    def sqql(self) -> FVector4: ...
    @property
    def sqos(self) -> FVector4: ...
    @property
    def sqot(self) -> FVector4: ...
    @property
    def sqoq(self) -> FVector4: ...
    @property
    def sqoo(self) -> FVector4: ...
    @property
    def sqol(self) -> FVector4: ...
    @property
    def sqls(self) -> FVector4: ...
    @property
    def sqlt(self) -> FVector4: ...
    @property
    def sqlq(self) -> FVector4: ...
    @property
    def sqlo(self) -> FVector4: ...
    @property
    def sqll(self) -> FVector4: ...
    @property
    def soss(self) -> FVector4: ...
    @property
    def sost(self) -> FVector4: ...
    @property
    def sosq(self) -> FVector4: ...
    @property
    def soso(self) -> FVector4: ...
    @property
    def sosl(self) -> FVector4: ...
    @property
    def sots(self) -> FVector4: ...
    @property
    def sott(self) -> FVector4: ...
    @property
    def sotq(self) -> FVector4: ...
    @property
    def soto(self) -> FVector4: ...
    @property
    def sotl(self) -> FVector4: ...
    @property
    def soqs(self) -> FVector4: ...
    @property
    def soqt(self) -> FVector4: ...
    @property
    def soqq(self) -> FVector4: ...
    @property
    def soqo(self) -> FVector4: ...
    @property
    def soql(self) -> FVector4: ...
    @property
    def soos(self) -> FVector4: ...
    @property
    def soot(self) -> FVector4: ...
    @property
    def sooq(self) -> FVector4: ...
    @property
    def sooo(self) -> FVector4: ...
    @property
    def sool(self) -> FVector4: ...
    @property
    def sols(self) -> FVector4: ...
    @property
    def solt(self) -> FVector4: ...
    @property
    def solq(self) -> FVector4: ...
    @property
    def solo(self) -> FVector4: ...
    @property
    def soll(self) -> FVector4: ...
    @property
    def slss(self) -> FVector4: ...
    @property
    def slst(self) -> FVector4: ...
    @property
    def slsq(self) -> FVector4: ...
    @property
    def slso(self) -> FVector4: ...
    @property
    def slsl(self) -> FVector4: ...
    @property
    def slts(self) -> FVector4: ...
    @property
    def sltt(self) -> FVector4: ...
    @property
    def sltq(self) -> FVector4: ...
    @property
    def slto(self) -> FVector4: ...
    @property
    def sltl(self) -> FVector4: ...
    @property
    def slqs(self) -> FVector4: ...
    @property
    def slqt(self) -> FVector4: ...
    @property
    def slqq(self) -> FVector4: ...
    @property
    def slqo(self) -> FVector4: ...
    @property
    def slql(self) -> FVector4: ...
    @property
    def slos(self) -> FVector4: ...
    @property
    def slot(self) -> FVector4: ...
    @property
    def sloq(self) -> FVector4: ...
    @property
    def sloo(self) -> FVector4: ...
    @property
    def slol(self) -> FVector4: ...
    @property
    def slls(self) -> FVector4: ...
    @property
    def sllt(self) -> FVector4: ...
    @property
    def sllq(self) -> FVector4: ...
    @property
    def sllo(self) -> FVector4: ...
    @property
    def slll(self) -> FVector4: ...
    @property
    def tsss(self) -> FVector4: ...
    @property
    def tsst(self) -> FVector4: ...
    @property
    def tssq(self) -> FVector4: ...
    @property
    def tsso(self) -> FVector4: ...
    @property
    def tssl(self) -> FVector4: ...
    @property
    def tsts(self) -> FVector4: ...
    @property
    def tstt(self) -> FVector4: ...
    @property
    def tstq(self) -> FVector4: ...
    @property
    def tsto(self) -> FVector4: ...
    @property
    def tstl(self) -> FVector4: ...
    @property
    def tsqs(self) -> FVector4: ...
    @property
    def tsqt(self) -> FVector4: ...
    @property
    def tsqq(self) -> FVector4: ...
    @property
    def tsqo(self) -> FVector4: ...
    @property
    def tsql(self) -> FVector4: ...
    @property
    def tsos(self) -> FVector4: ...
    @property
    def tsot(self) -> FVector4: ...
    @property
    def tsoq(self) -> FVector4: ...
    @property
    def tsoo(self) -> FVector4: ...
    @property
    def tsol(self) -> FVector4: ...
    @property
    def tsls(self) -> FVector4: ...
    @property
    def tslt(self) -> FVector4: ...
    @property
    def tslq(self) -> FVector4: ...
    @property
    def tslo(self) -> FVector4: ...
    @property
    def tsll(self) -> FVector4: ...
    @property
    def ttss(self) -> FVector4: ...
    @property
    def ttst(self) -> FVector4: ...
    @property
    def ttsq(self) -> FVector4: ...
    @property
    def ttso(self) -> FVector4: ...
    @property
    def ttsl(self) -> FVector4: ...
    @property
    def ttts(self) -> FVector4: ...
    @property
    def tttt(self) -> FVector4: ...
    @property
    def tttq(self) -> FVector4: ...
    @property
    def ttto(self) -> FVector4: ...
    @property
    def tttl(self) -> FVector4: ...
    @property
    def ttqs(self) -> FVector4: ...
    @property
    def ttqt(self) -> FVector4: ...
    @property
    def ttqq(self) -> FVector4: ...
    @property
    def ttqo(self) -> FVector4: ...
    @property
    def ttql(self) -> FVector4: ...
    @property
    def ttos(self) -> FVector4: ...
    @property
    def ttot(self) -> FVector4: ...
    @property
    def ttoq(self) -> FVector4: ...
    @property
    def ttoo(self) -> FVector4: ...
    @property
    def ttol(self) -> FVector4: ...
    @property
    def ttls(self) -> FVector4: ...
    @property
    def ttlt(self) -> FVector4: ...
    @property
    def ttlq(self) -> FVector4: ...
    @property
    def ttlo(self) -> FVector4: ...
    @property
    def ttll(self) -> FVector4: ...
    @property
    def tqss(self) -> FVector4: ...
    @property
    def tqst(self) -> FVector4: ...
    @property
    def tqsq(self) -> FVector4: ...
    @property
    def tqso(self) -> FVector4: ...
    @property
    def tqsl(self) -> FVector4: ...
    @property
    def tqts(self) -> FVector4: ...
    @property
    def tqtt(self) -> FVector4: ...
    @property
    def tqtq(self) -> FVector4: ...
    @property
    def tqto(self) -> FVector4: ...
    @property
    def tqtl(self) -> FVector4: ...
    @property
    def tqqs(self) -> FVector4: ...
    @property
    def tqqt(self) -> FVector4: ...
    @property
    def tqqq(self) -> FVector4: ...
    @property
    def tqqo(self) -> FVector4: ...
    @property
    def tqql(self) -> FVector4: ...
    @property
    def tqos(self) -> FVector4: ...
    @property
    def tqot(self) -> FVector4: ...
    @property
    def tqoq(self) -> FVector4: ...
    @property
    def tqoo(self) -> FVector4: ...
    @property
    def tqol(self) -> FVector4: ...
    @property
    def tqls(self) -> FVector4: ...
    @property
    def tqlt(self) -> FVector4: ...
    @property
    def tqlq(self) -> FVector4: ...
    @property
    def tqlo(self) -> FVector4: ...
    @property
    def tqll(self) -> FVector4: ...
    @property
    def toss(self) -> FVector4: ...
    @property
    def tost(self) -> FVector4: ...
    @property
    def tosq(self) -> FVector4: ...
    @property
    def toso(self) -> FVector4: ...
    @property
    def tosl(self) -> FVector4: ...
    @property
    def tots(self) -> FVector4: ...
    @property
    def tott(self) -> FVector4: ...
    @property
    def totq(self) -> FVector4: ...
    @property
    def toto(self) -> FVector4: ...
    @property
    def totl(self) -> FVector4: ...
    @property
    def toqs(self) -> FVector4: ...
    @property
    def toqt(self) -> FVector4: ...
    @property
    def toqq(self) -> FVector4: ...
    @property
    def toqo(self) -> FVector4: ...
    @property
    def toql(self) -> FVector4: ...
    @property
    def toos(self) -> FVector4: ...
    @property
    def toot(self) -> FVector4: ...
    @property
    def tooq(self) -> FVector4: ...
    @property
    def tooo(self) -> FVector4: ...
    @property
    def tool(self) -> FVector4: ...
    @property
    def tols(self) -> FVector4: ...
    @property
    def tolt(self) -> FVector4: ...
    @property
    def tolq(self) -> FVector4: ...
    @property
    def tolo(self) -> FVector4: ...
    @property
    def toll(self) -> FVector4: ...
    @property
    def tlss(self) -> FVector4: ...
    @property
    def tlst(self) -> FVector4: ...
    @property
    def tlsq(self) -> FVector4: ...
    @property
    def tlso(self) -> FVector4: ...
    @property
    def tlsl(self) -> FVector4: ...
    @property
    def tlts(self) -> FVector4: ...
    @property
    def tltt(self) -> FVector4: ...
    @property
    def tltq(self) -> FVector4: ...
    @property
    def tlto(self) -> FVector4: ...
    @property
    def tltl(self) -> FVector4: ...
    @property
    def tlqs(self) -> FVector4: ...
    @property
    def tlqt(self) -> FVector4: ...
    @property
    def tlqq(self) -> FVector4: ...
    @property
    def tlqo(self) -> FVector4: ...
    @property
    def tlql(self) -> FVector4: ...
    @property
    def tlos(self) -> FVector4: ...
    @property
    def tlot(self) -> FVector4: ...
    @property
    def tloq(self) -> FVector4: ...
    @property
    def tloo(self) -> FVector4: ...
    @property
    def tlol(self) -> FVector4: ...
    @property
    def tlls(self) -> FVector4: ...
    @property
    def tllt(self) -> FVector4: ...
    @property
    def tllq(self) -> FVector4: ...
    @property
    def tllo(self) -> FVector4: ...
    @property
    def tlll(self) -> FVector4: ...
    @property
    def qsss(self) -> FVector4: ...
    @property
    def qsst(self) -> FVector4: ...
    @property
    def qssq(self) -> FVector4: ...
    @property
    def qsso(self) -> FVector4: ...
    @property
    def qssl(self) -> FVector4: ...
    @property
    def qsts(self) -> FVector4: ...
    @property
    def qstt(self) -> FVector4: ...
    @property
    def qstq(self) -> FVector4: ...
    @property
    def qsto(self) -> FVector4: ...
    @property
    def qstl(self) -> FVector4: ...
    @property
    def qsqs(self) -> FVector4: ...
    @property
    def qsqt(self) -> FVector4: ...
    @property
    def qsqq(self) -> FVector4: ...
    @property
    def qsqo(self) -> FVector4: ...
    @property
    def qsql(self) -> FVector4: ...
    @property
    def qsos(self) -> FVector4: ...
    @property
    def qsot(self) -> FVector4: ...
    @property
    def qsoq(self) -> FVector4: ...
    @property
    def qsoo(self) -> FVector4: ...
    @property
    def qsol(self) -> FVector4: ...
    @property
    def qsls(self) -> FVector4: ...
    @property
    def qslt(self) -> FVector4: ...
    @property
    def qslq(self) -> FVector4: ...
    @property
    def qslo(self) -> FVector4: ...
    @property
    def qsll(self) -> FVector4: ...
    @property
    def qtss(self) -> FVector4: ...
    @property
    def qtst(self) -> FVector4: ...
    @property
    def qtsq(self) -> FVector4: ...
    @property
    def qtso(self) -> FVector4: ...
    @property
    def qtsl(self) -> FVector4: ...
    @property
    def qtts(self) -> FVector4: ...
    @property
    def qttt(self) -> FVector4: ...
    @property
    def qttq(self) -> FVector4: ...
    @property
    def qtto(self) -> FVector4: ...
    @property
    def qttl(self) -> FVector4: ...
    @property
    def qtqs(self) -> FVector4: ...
    @property
    def qtqt(self) -> FVector4: ...
    @property
    def qtqq(self) -> FVector4: ...
    @property
    def qtqo(self) -> FVector4: ...
    @property
    def qtql(self) -> FVector4: ...
    @property
    def qtos(self) -> FVector4: ...
    @property
    def qtot(self) -> FVector4: ...
    @property
    def qtoq(self) -> FVector4: ...
    @property
    def qtoo(self) -> FVector4: ...
    @property
    def qtol(self) -> FVector4: ...
    @property
    def qtls(self) -> FVector4: ...
    @property
    def qtlt(self) -> FVector4: ...
    @property
    def qtlq(self) -> FVector4: ...
    @property
    def qtlo(self) -> FVector4: ...
    @property
    def qtll(self) -> FVector4: ...
    @property
    def qqss(self) -> FVector4: ...
    @property
    def qqst(self) -> FVector4: ...
    @property
    def qqsq(self) -> FVector4: ...
    @property
    def qqso(self) -> FVector4: ...
    @property
    def qqsl(self) -> FVector4: ...
    @property
    def qqts(self) -> FVector4: ...
    @property
    def qqtt(self) -> FVector4: ...
    @property
    def qqtq(self) -> FVector4: ...
    @property
    def qqto(self) -> FVector4: ...
    @property
    def qqtl(self) -> FVector4: ...
    @property
    def qqqs(self) -> FVector4: ...
    @property
    def qqqt(self) -> FVector4: ...
    @property
    def qqqq(self) -> FVector4: ...
    @property
    def qqqo(self) -> FVector4: ...
    @property
    def qqql(self) -> FVector4: ...
    @property
    def qqos(self) -> FVector4: ...
    @property
    def qqot(self) -> FVector4: ...
    @property
    def qqoq(self) -> FVector4: ...
    @property
    def qqoo(self) -> FVector4: ...
    @property
    def qqol(self) -> FVector4: ...
    @property
    def qqls(self) -> FVector4: ...
    @property
    def qqlt(self) -> FVector4: ...
    @property
    def qqlq(self) -> FVector4: ...
    @property
    def qqlo(self) -> FVector4: ...
    @property
    def qqll(self) -> FVector4: ...
    @property
    def qoss(self) -> FVector4: ...
    @property
    def qost(self) -> FVector4: ...
    @property
    def qosq(self) -> FVector4: ...
    @property
    def qoso(self) -> FVector4: ...
    @property
    def qosl(self) -> FVector4: ...
    @property
    def qots(self) -> FVector4: ...
    @property
    def qott(self) -> FVector4: ...
    @property
    def qotq(self) -> FVector4: ...
    @property
    def qoto(self) -> FVector4: ...
    @property
    def qotl(self) -> FVector4: ...
    @property
    def qoqs(self) -> FVector4: ...
    @property
    def qoqt(self) -> FVector4: ...
    @property
    def qoqq(self) -> FVector4: ...
    @property
    def qoqo(self) -> FVector4: ...
    @property
    def qoql(self) -> FVector4: ...
    @property
    def qoos(self) -> FVector4: ...
    @property
    def qoot(self) -> FVector4: ...
    @property
    def qooq(self) -> FVector4: ...
    @property
    def qooo(self) -> FVector4: ...
    @property
    def qool(self) -> FVector4: ...
    @property
    def qols(self) -> FVector4: ...
    @property
    def qolt(self) -> FVector4: ...
    @property
    def qolq(self) -> FVector4: ...
    @property
    def qolo(self) -> FVector4: ...
    @property
    def qoll(self) -> FVector4: ...
    @property
    def qlss(self) -> FVector4: ...
    @property
    def qlst(self) -> FVector4: ...
    @property
    def qlsq(self) -> FVector4: ...
    @property
    def qlso(self) -> FVector4: ...
    @property
    def qlsl(self) -> FVector4: ...
    @property
    def qlts(self) -> FVector4: ...
    @property
    def qltt(self) -> FVector4: ...
    @property
    def qltq(self) -> FVector4: ...
    @property
    def qlto(self) -> FVector4: ...
    @property
    def qltl(self) -> FVector4: ...
    @property
    def qlqs(self) -> FVector4: ...
    @property
    def qlqt(self) -> FVector4: ...
    @property
    def qlqq(self) -> FVector4: ...
    @property
    def qlqo(self) -> FVector4: ...
    @property
    def qlql(self) -> FVector4: ...
    @property
    def qlos(self) -> FVector4: ...
    @property
    def qlot(self) -> FVector4: ...
    @property
    def qloq(self) -> FVector4: ...
    @property
    def qloo(self) -> FVector4: ...
    @property
    def qlol(self) -> FVector4: ...
    @property
    def qlls(self) -> FVector4: ...
    @property
    def qllt(self) -> FVector4: ...
    @property
    def qllq(self) -> FVector4: ...
    @property
    def qllo(self) -> FVector4: ...
    @property
    def qlll(self) -> FVector4: ...
    @property
    def osss(self) -> FVector4: ...
    @property
    def osst(self) -> FVector4: ...
    @property
    def ossq(self) -> FVector4: ...
    @property
    def osso(self) -> FVector4: ...
    @property
    def ossl(self) -> FVector4: ...
    @property
    def osts(self) -> FVector4: ...
    @property
    def ostt(self) -> FVector4: ...
    @property
    def ostq(self) -> FVector4: ...
    @property
    def osto(self) -> FVector4: ...
    @property
    def ostl(self) -> FVector4: ...
    @property
    def osqs(self) -> FVector4: ...
    @property
    def osqt(self) -> FVector4: ...
    @property
    def osqq(self) -> FVector4: ...
    @property
    def osqo(self) -> FVector4: ...
    @property
    def osql(self) -> FVector4: ...
    @property
    def osos(self) -> FVector4: ...
    @property
    def osot(self) -> FVector4: ...
    @property
    def osoq(self) -> FVector4: ...
    @property
    def osoo(self) -> FVector4: ...
    @property
    def osol(self) -> FVector4: ...
    @property
    def osls(self) -> FVector4: ...
    @property
    def oslt(self) -> FVector4: ...
    @property
    def oslq(self) -> FVector4: ...
    @property
    def oslo(self) -> FVector4: ...
    @property
    def osll(self) -> FVector4: ...
    @property
    def otss(self) -> FVector4: ...
    @property
    def otst(self) -> FVector4: ...
    @property
    def otsq(self) -> FVector4: ...
    @property
    def otso(self) -> FVector4: ...
    @property
    def otsl(self) -> FVector4: ...
    @property
    def otts(self) -> FVector4: ...
    @property
    def ottt(self) -> FVector4: ...
    @property
    def ottq(self) -> FVector4: ...
    @property
    def otto(self) -> FVector4: ...
    @property
    def ottl(self) -> FVector4: ...
    @property
    def otqs(self) -> FVector4: ...
    @property
    def otqt(self) -> FVector4: ...
    @property
    def otqq(self) -> FVector4: ...
    @property
    def otqo(self) -> FVector4: ...
    @property
    def otql(self) -> FVector4: ...
    @property
    def otos(self) -> FVector4: ...
    @property
    def otot(self) -> FVector4: ...
    @property
    def otoq(self) -> FVector4: ...
    @property
    def otoo(self) -> FVector4: ...
    @property
    def otol(self) -> FVector4: ...
    @property
    def otls(self) -> FVector4: ...
    @property
    def otlt(self) -> FVector4: ...
    @property
    def otlq(self) -> FVector4: ...
    @property
    def otlo(self) -> FVector4: ...
    @property
    def otll(self) -> FVector4: ...
    @property
    def oqss(self) -> FVector4: ...
    @property
    def oqst(self) -> FVector4: ...
    @property
    def oqsq(self) -> FVector4: ...
    @property
    def oqso(self) -> FVector4: ...
    @property
    def oqsl(self) -> FVector4: ...
    @property
    def oqts(self) -> FVector4: ...
    @property
    def oqtt(self) -> FVector4: ...
    @property
    def oqtq(self) -> FVector4: ...
    @property
    def oqto(self) -> FVector4: ...
    @property
    def oqtl(self) -> FVector4: ...
    @property
    def oqqs(self) -> FVector4: ...
    @property
    def oqqt(self) -> FVector4: ...
    @property
    def oqqq(self) -> FVector4: ...
    @property
    def oqqo(self) -> FVector4: ...
    @property
    def oqql(self) -> FVector4: ...
    @property
    def oqos(self) -> FVector4: ...
    @property
    def oqot(self) -> FVector4: ...
    @property
    def oqoq(self) -> FVector4: ...
    @property
    def oqoo(self) -> FVector4: ...
    @property
    def oqol(self) -> FVector4: ...
    @property
    def oqls(self) -> FVector4: ...
    @property
    def oqlt(self) -> FVector4: ...
    @property
    def oqlq(self) -> FVector4: ...
    @property
    def oqlo(self) -> FVector4: ...
    @property
    def oqll(self) -> FVector4: ...
    @property
    def ooss(self) -> FVector4: ...
    @property
    def oost(self) -> FVector4: ...
    @property
    def oosq(self) -> FVector4: ...
    @property
    def ooso(self) -> FVector4: ...
    @property
    def oosl(self) -> FVector4: ...
    @property
    def oots(self) -> FVector4: ...
    @property
    def oott(self) -> FVector4: ...
    @property
    def ootq(self) -> FVector4: ...
    @property
    def ooto(self) -> FVector4: ...
    @property
    def ootl(self) -> FVector4: ...
    @property
    def ooqs(self) -> FVector4: ...
    @property
    def ooqt(self) -> FVector4: ...
    @property
    def ooqq(self) -> FVector4: ...
    @property
    def ooqo(self) -> FVector4: ...
    @property
    def ooql(self) -> FVector4: ...
    @property
    def ooos(self) -> FVector4: ...
    @property
    def ooot(self) -> FVector4: ...
    @property
    def oooq(self) -> FVector4: ...
    @property
    def ools(self) -> FVector4: ...
    @property
    def oolt(self) -> FVector4: ...
    @property
    def oolq(self) -> FVector4: ...
    @property
    def olss(self) -> FVector4: ...
    @property
    def olst(self) -> FVector4: ...
    @property
    def olsq(self) -> FVector4: ...
    @property
    def olso(self) -> FVector4: ...
    @property
    def olsl(self) -> FVector4: ...
    @property
    def olts(self) -> FVector4: ...
    @property
    def oltt(self) -> FVector4: ...
    @property
    def oltq(self) -> FVector4: ...
    @property
    def olto(self) -> FVector4: ...
    @property
    def oltl(self) -> FVector4: ...
    @property
    def olqs(self) -> FVector4: ...
    @property
    def olqt(self) -> FVector4: ...
    @property
    def olqq(self) -> FVector4: ...
    @property
    def olqo(self) -> FVector4: ...
    @property
    def olql(self) -> FVector4: ...
    @property
    def olos(self) -> FVector4: ...
    @property
    def olot(self) -> FVector4: ...
    @property
    def oloq(self) -> FVector4: ...
    @property
    def olls(self) -> FVector4: ...
    @property
    def ollt(self) -> FVector4: ...
    @property
    def ollq(self) -> FVector4: ...
    @property
    def lsss(self) -> FVector4: ...
    @property
    def lsst(self) -> FVector4: ...
    @property
    def lssq(self) -> FVector4: ...
    @property
    def lsso(self) -> FVector4: ...
    @property
    def lssl(self) -> FVector4: ...
    @property
    def lsts(self) -> FVector4: ...
    @property
    def lstt(self) -> FVector4: ...
    @property
    def lstq(self) -> FVector4: ...
    @property
    def lsto(self) -> FVector4: ...
    @property
    def lstl(self) -> FVector4: ...
    @property
    def lsqs(self) -> FVector4: ...
    @property
    def lsqt(self) -> FVector4: ...
    @property
    def lsqq(self) -> FVector4: ...
    @property
    def lsqo(self) -> FVector4: ...
    @property
    def lsql(self) -> FVector4: ...
    @property
    def lsos(self) -> FVector4: ...
    @property
    def lsot(self) -> FVector4: ...
    @property
    def lsoq(self) -> FVector4: ...
    @property
    def lsoo(self) -> FVector4: ...
    @property
    def lsol(self) -> FVector4: ...
    @property
    def lsls(self) -> FVector4: ...
    @property
    def lslt(self) -> FVector4: ...
    @property
    def lslq(self) -> FVector4: ...
    @property
    def lslo(self) -> FVector4: ...
    @property
    def lsll(self) -> FVector4: ...
    @property
    def ltss(self) -> FVector4: ...
    @property
    def ltst(self) -> FVector4: ...
    @property
    def ltsq(self) -> FVector4: ...
    @property
    def ltso(self) -> FVector4: ...
    @property
    def ltsl(self) -> FVector4: ...
    @property
    def ltts(self) -> FVector4: ...
    @property
    def lttt(self) -> FVector4: ...
    @property
    def lttq(self) -> FVector4: ...
    @property
    def ltto(self) -> FVector4: ...
    @property
    def lttl(self) -> FVector4: ...
    @property
    def ltqs(self) -> FVector4: ...
    @property
    def ltqt(self) -> FVector4: ...
    @property
    def ltqq(self) -> FVector4: ...
    @property
    def ltqo(self) -> FVector4: ...
    @property
    def ltql(self) -> FVector4: ...
    @property
    def ltos(self) -> FVector4: ...
    @property
    def ltot(self) -> FVector4: ...
    @property
    def ltoq(self) -> FVector4: ...
    @property
    def ltoo(self) -> FVector4: ...
    @property
    def ltol(self) -> FVector4: ...
    @property
    def ltls(self) -> FVector4: ...
    @property
    def ltlt(self) -> FVector4: ...
    @property
    def ltlq(self) -> FVector4: ...
    @property
    def ltlo(self) -> FVector4: ...
    @property
    def ltll(self) -> FVector4: ...
    @property
    def lqss(self) -> FVector4: ...
    @property
    def lqst(self) -> FVector4: ...
    @property
    def lqsq(self) -> FVector4: ...
    @property
    def lqso(self) -> FVector4: ...
    @property
    def lqsl(self) -> FVector4: ...
    @property
    def lqts(self) -> FVector4: ...
    @property
    def lqtt(self) -> FVector4: ...
    @property
    def lqtq(self) -> FVector4: ...
    @property
    def lqto(self) -> FVector4: ...
    @property
    def lqtl(self) -> FVector4: ...
    @property
    def lqqs(self) -> FVector4: ...
    @property
    def lqqt(self) -> FVector4: ...
    @property
    def lqqq(self) -> FVector4: ...
    @property
    def lqqo(self) -> FVector4: ...
    @property
    def lqql(self) -> FVector4: ...
    @property
    def lqos(self) -> FVector4: ...
    @property
    def lqot(self) -> FVector4: ...
    @property
    def lqoq(self) -> FVector4: ...
    @property
    def lqoo(self) -> FVector4: ...
    @property
    def lqol(self) -> FVector4: ...
    @property
    def lqls(self) -> FVector4: ...
    @property
    def lqlt(self) -> FVector4: ...
    @property
    def lqlq(self) -> FVector4: ...
    @property
    def lqlo(self) -> FVector4: ...
    @property
    def lqll(self) -> FVector4: ...
    @property
    def loss(self) -> FVector4: ...
    @property
    def lost(self) -> FVector4: ...
    @property
    def losq(self) -> FVector4: ...
    @property
    def loso(self) -> FVector4: ...
    @property
    def losl(self) -> FVector4: ...
    @property
    def lots(self) -> FVector4: ...
    @property
    def lott(self) -> FVector4: ...
    @property
    def lotq(self) -> FVector4: ...
    @property
    def loto(self) -> FVector4: ...
    @property
    def lotl(self) -> FVector4: ...
    @property
    def loqs(self) -> FVector4: ...
    @property
    def loqt(self) -> FVector4: ...
    @property
    def loqq(self) -> FVector4: ...
    @property
    def loqo(self) -> FVector4: ...
    @property
    def loql(self) -> FVector4: ...
    @property
    def loos(self) -> FVector4: ...
    @property
    def loot(self) -> FVector4: ...
    @property
    def looq(self) -> FVector4: ...
    @property
    def lols(self) -> FVector4: ...
    @property
    def lolt(self) -> FVector4: ...
    @property
    def lolq(self) -> FVector4: ...
    @property
    def llss(self) -> FVector4: ...
    @property
    def llst(self) -> FVector4: ...
    @property
    def llsq(self) -> FVector4: ...
    @property
    def llso(self) -> FVector4: ...
    @property
    def llsl(self) -> FVector4: ...
    @property
    def llts(self) -> FVector4: ...
    @property
    def lltt(self) -> FVector4: ...
    @property
    def lltq(self) -> FVector4: ...
    @property
    def llto(self) -> FVector4: ...
    @property
    def lltl(self) -> FVector4: ...
    @property
    def llqs(self) -> FVector4: ...
    @property
    def llqt(self) -> FVector4: ...
    @property
    def llqq(self) -> FVector4: ...
    @property
    def llqo(self) -> FVector4: ...
    @property
    def llql(self) -> FVector4: ...
    @property
    def llos(self) -> FVector4: ...
    @property
    def llot(self) -> FVector4: ...
    @property
    def lloq(self) -> FVector4: ...
    @property
    def llls(self) -> FVector4: ...
    @property
    def lllt(self) -> FVector4: ...
    @property
    def lllq(self) -> FVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def v(self) -> float: ...
    @property
    def uu(self) -> FVector2: ...
    @property
    def uv(self) -> FVector2: ...
    @property
    def uo(self) -> FVector2: ...
    @property
    def ul(self) -> FVector2: ...
    @property
    def vu(self) -> FVector2: ...
    @property
    def vv(self) -> FVector2: ...
    @property
    def vo(self) -> FVector2: ...
    @property
    def vl(self) -> FVector2: ...
    @property
    def ou(self) -> FVector2: ...
    @property
    def ov(self) -> FVector2: ...
    @property
    def lu(self) -> FVector2: ...
    @property
    def lv(self) -> FVector2: ...
    @property
    def uuu(self) -> FVector3: ...
    @property
    def uuv(self) -> FVector3: ...
    @property
    def uuo(self) -> FVector3: ...
    @property
    def uul(self) -> FVector3: ...
    @property
    def uvu(self) -> FVector3: ...
    @property
    def uvv(self) -> FVector3: ...
    @property
    def uvo(self) -> FVector3: ...
    @property
    def uvl(self) -> FVector3: ...
    @property
    def uou(self) -> FVector3: ...
    @property
    def uov(self) -> FVector3: ...
    @property
    def uoo(self) -> FVector3: ...
    @property
    def uol(self) -> FVector3: ...
    @property
    def ulu(self) -> FVector3: ...
    @property
    def ulv(self) -> FVector3: ...
    @property
    def ulo(self) -> FVector3: ...
    @property
    def ull(self) -> FVector3: ...
    @property
    def vuu(self) -> FVector3: ...
    @property
    def vuv(self) -> FVector3: ...
    @property
    def vuo(self) -> FVector3: ...
    @property
    def vul(self) -> FVector3: ...
    @property
    def vvu(self) -> FVector3: ...
    @property
    def vvv(self) -> FVector3: ...
    @property
    def vvo(self) -> FVector3: ...
    @property
    def vvl(self) -> FVector3: ...
    @property
    def vou(self) -> FVector3: ...
    @property
    def vov(self) -> FVector3: ...
    @property
    def voo(self) -> FVector3: ...
    @property
    def vol(self) -> FVector3: ...
    @property
    def vlu(self) -> FVector3: ...
    @property
    def vlv(self) -> FVector3: ...
    @property
    def vlo(self) -> FVector3: ...
    @property
    def vll(self) -> FVector3: ...
    @property
    def ouu(self) -> FVector3: ...
    @property
    def ouv(self) -> FVector3: ...
    @property
    def ouo(self) -> FVector3: ...
    @property
    def oul(self) -> FVector3: ...
    @property
    def ovu(self) -> FVector3: ...
    @property
    def ovv(self) -> FVector3: ...
    @property
    def ovo(self) -> FVector3: ...
    @property
    def ovl(self) -> FVector3: ...
    @property
    def oou(self) -> FVector3: ...
    @property
    def oov(self) -> FVector3: ...
    @property
    def olu(self) -> FVector3: ...
    @property
    def olv(self) -> FVector3: ...
    @property
    def luu(self) -> FVector3: ...
    @property
    def luv(self) -> FVector3: ...
    @property
    def luo(self) -> FVector3: ...
    @property
    def lul(self) -> FVector3: ...
    @property
    def lvu(self) -> FVector3: ...
    @property
    def lvv(self) -> FVector3: ...
    @property
    def lvo(self) -> FVector3: ...
    @property
    def lvl(self) -> FVector3: ...
    @property
    def lou(self) -> FVector3: ...
    @property
    def lov(self) -> FVector3: ...
    @property
    def llu(self) -> FVector3: ...
    @property
    def llv(self) -> FVector3: ...
    @property
    def uuuu(self) -> FVector4: ...
    @property
    def uuuv(self) -> FVector4: ...
    @property
    def uuuo(self) -> FVector4: ...
    @property
    def uuul(self) -> FVector4: ...
    @property
    def uuvu(self) -> FVector4: ...
    @property
    def uuvv(self) -> FVector4: ...
    @property
    def uuvo(self) -> FVector4: ...
    @property
    def uuvl(self) -> FVector4: ...
    @property
    def uuou(self) -> FVector4: ...
    @property
    def uuov(self) -> FVector4: ...
    @property
    def uuoo(self) -> FVector4: ...
    @property
    def uuol(self) -> FVector4: ...
    @property
    def uulu(self) -> FVector4: ...
    @property
    def uulv(self) -> FVector4: ...
    @property
    def uulo(self) -> FVector4: ...
    @property
    def uull(self) -> FVector4: ...
    @property
    def uvuu(self) -> FVector4: ...
    @property
    def uvuv(self) -> FVector4: ...
    @property
    def uvuo(self) -> FVector4: ...
    @property
    def uvul(self) -> FVector4: ...
    @property
    def uvvu(self) -> FVector4: ...
    @property
    def uvvv(self) -> FVector4: ...
    @property
    def uvvo(self) -> FVector4: ...
    @property
    def uvvl(self) -> FVector4: ...
    @property
    def uvou(self) -> FVector4: ...
    @property
    def uvov(self) -> FVector4: ...
    @property
    def uvoo(self) -> FVector4: ...
    @property
    def uvol(self) -> FVector4: ...
    @property
    def uvlu(self) -> FVector4: ...
    @property
    def uvlv(self) -> FVector4: ...
    @property
    def uvlo(self) -> FVector4: ...
    @property
    def uvll(self) -> FVector4: ...
    @property
    def uouu(self) -> FVector4: ...
    @property
    def uouv(self) -> FVector4: ...
    @property
    def uouo(self) -> FVector4: ...
    @property
    def uoul(self) -> FVector4: ...
    @property
    def uovu(self) -> FVector4: ...
    @property
    def uovv(self) -> FVector4: ...
    @property
    def uovo(self) -> FVector4: ...
    @property
    def uovl(self) -> FVector4: ...
    @property
    def uoou(self) -> FVector4: ...
    @property
    def uoov(self) -> FVector4: ...
    @property
    def uooo(self) -> FVector4: ...
    @property
    def uool(self) -> FVector4: ...
    @property
    def uolu(self) -> FVector4: ...
    @property
    def uolv(self) -> FVector4: ...
    @property
    def uolo(self) -> FVector4: ...
    @property
    def uoll(self) -> FVector4: ...
    @property
    def uluu(self) -> FVector4: ...
    @property
    def uluv(self) -> FVector4: ...
    @property
    def uluo(self) -> FVector4: ...
    @property
    def ulul(self) -> FVector4: ...
    @property
    def ulvu(self) -> FVector4: ...
    @property
    def ulvv(self) -> FVector4: ...
    @property
    def ulvo(self) -> FVector4: ...
    @property
    def ulvl(self) -> FVector4: ...
    @property
    def ulou(self) -> FVector4: ...
    @property
    def ulov(self) -> FVector4: ...
    @property
    def uloo(self) -> FVector4: ...
    @property
    def ulol(self) -> FVector4: ...
    @property
    def ullu(self) -> FVector4: ...
    @property
    def ullv(self) -> FVector4: ...
    @property
    def ullo(self) -> FVector4: ...
    @property
    def ulll(self) -> FVector4: ...
    @property
    def vuuu(self) -> FVector4: ...
    @property
    def vuuv(self) -> FVector4: ...
    @property
    def vuuo(self) -> FVector4: ...
    @property
    def vuul(self) -> FVector4: ...
    @property
    def vuvu(self) -> FVector4: ...
    @property
    def vuvv(self) -> FVector4: ...
    @property
    def vuvo(self) -> FVector4: ...
    @property
    def vuvl(self) -> FVector4: ...
    @property
    def vuou(self) -> FVector4: ...
    @property
    def vuov(self) -> FVector4: ...
    @property
    def vuoo(self) -> FVector4: ...
    @property
    def vuol(self) -> FVector4: ...
    @property
    def vulu(self) -> FVector4: ...
    @property
    def vulv(self) -> FVector4: ...
    @property
    def vulo(self) -> FVector4: ...
    @property
    def vull(self) -> FVector4: ...
    @property
    def vvuu(self) -> FVector4: ...
    @property
    def vvuv(self) -> FVector4: ...
    @property
    def vvuo(self) -> FVector4: ...
    @property
    def vvul(self) -> FVector4: ...
    @property
    def vvvu(self) -> FVector4: ...
    @property
    def vvvv(self) -> FVector4: ...
    @property
    def vvvo(self) -> FVector4: ...
    @property
    def vvvl(self) -> FVector4: ...
    @property
    def vvou(self) -> FVector4: ...
    @property
    def vvov(self) -> FVector4: ...
    @property
    def vvoo(self) -> FVector4: ...
    @property
    def vvol(self) -> FVector4: ...
    @property
    def vvlu(self) -> FVector4: ...
    @property
    def vvlv(self) -> FVector4: ...
    @property
    def vvlo(self) -> FVector4: ...
    @property
    def vvll(self) -> FVector4: ...
    @property
    def vouu(self) -> FVector4: ...
    @property
    def vouv(self) -> FVector4: ...
    @property
    def vouo(self) -> FVector4: ...
    @property
    def voul(self) -> FVector4: ...
    @property
    def vovu(self) -> FVector4: ...
    @property
    def vovv(self) -> FVector4: ...
    @property
    def vovo(self) -> FVector4: ...
    @property
    def vovl(self) -> FVector4: ...
    @property
    def voou(self) -> FVector4: ...
    @property
    def voov(self) -> FVector4: ...
    @property
    def vooo(self) -> FVector4: ...
    @property
    def vool(self) -> FVector4: ...
    @property
    def volu(self) -> FVector4: ...
    @property
    def volv(self) -> FVector4: ...
    @property
    def volo(self) -> FVector4: ...
    @property
    def voll(self) -> FVector4: ...
    @property
    def vluu(self) -> FVector4: ...
    @property
    def vluv(self) -> FVector4: ...
    @property
    def vluo(self) -> FVector4: ...
    @property
    def vlul(self) -> FVector4: ...
    @property
    def vlvu(self) -> FVector4: ...
    @property
    def vlvv(self) -> FVector4: ...
    @property
    def vlvo(self) -> FVector4: ...
    @property
    def vlvl(self) -> FVector4: ...
    @property
    def vlou(self) -> FVector4: ...
    @property
    def vlov(self) -> FVector4: ...
    @property
    def vloo(self) -> FVector4: ...
    @property
    def vlol(self) -> FVector4: ...
    @property
    def vllu(self) -> FVector4: ...
    @property
    def vllv(self) -> FVector4: ...
    @property
    def vllo(self) -> FVector4: ...
    @property
    def vlll(self) -> FVector4: ...
    @property
    def ouuu(self) -> FVector4: ...
    @property
    def ouuv(self) -> FVector4: ...
    @property
    def ouuo(self) -> FVector4: ...
    @property
    def ouul(self) -> FVector4: ...
    @property
    def ouvu(self) -> FVector4: ...
    @property
    def ouvv(self) -> FVector4: ...
    @property
    def ouvo(self) -> FVector4: ...
    @property
    def ouvl(self) -> FVector4: ...
    @property
    def ouou(self) -> FVector4: ...
    @property
    def ouov(self) -> FVector4: ...
    @property
    def ouoo(self) -> FVector4: ...
    @property
    def ouol(self) -> FVector4: ...
    @property
    def oulu(self) -> FVector4: ...
    @property
    def oulv(self) -> FVector4: ...
    @property
    def oulo(self) -> FVector4: ...
    @property
    def oull(self) -> FVector4: ...
    @property
    def ovuu(self) -> FVector4: ...
    @property
    def ovuv(self) -> FVector4: ...
    @property
    def ovuo(self) -> FVector4: ...
    @property
    def ovul(self) -> FVector4: ...
    @property
    def ovvu(self) -> FVector4: ...
    @property
    def ovvv(self) -> FVector4: ...
    @property
    def ovvo(self) -> FVector4: ...
    @property
    def ovvl(self) -> FVector4: ...
    @property
    def ovou(self) -> FVector4: ...
    @property
    def ovov(self) -> FVector4: ...
    @property
    def ovoo(self) -> FVector4: ...
    @property
    def ovol(self) -> FVector4: ...
    @property
    def ovlu(self) -> FVector4: ...
    @property
    def ovlv(self) -> FVector4: ...
    @property
    def ovlo(self) -> FVector4: ...
    @property
    def ovll(self) -> FVector4: ...
    @property
    def oouu(self) -> FVector4: ...
    @property
    def oouv(self) -> FVector4: ...
    @property
    def oouo(self) -> FVector4: ...
    @property
    def ooul(self) -> FVector4: ...
    @property
    def oovu(self) -> FVector4: ...
    @property
    def oovv(self) -> FVector4: ...
    @property
    def oovo(self) -> FVector4: ...
    @property
    def oovl(self) -> FVector4: ...
    @property
    def ooou(self) -> FVector4: ...
    @property
    def ooov(self) -> FVector4: ...
    @property
    def oolu(self) -> FVector4: ...
    @property
    def oolv(self) -> FVector4: ...
    @property
    def oluu(self) -> FVector4: ...
    @property
    def oluv(self) -> FVector4: ...
    @property
    def oluo(self) -> FVector4: ...
    @property
    def olul(self) -> FVector4: ...
    @property
    def olvu(self) -> FVector4: ...
    @property
    def olvv(self) -> FVector4: ...
    @property
    def olvo(self) -> FVector4: ...
    @property
    def olvl(self) -> FVector4: ...
    @property
    def olou(self) -> FVector4: ...
    @property
    def olov(self) -> FVector4: ...
    @property
    def ollu(self) -> FVector4: ...
    @property
    def ollv(self) -> FVector4: ...
    @property
    def luuu(self) -> FVector4: ...
    @property
    def luuv(self) -> FVector4: ...
    @property
    def luuo(self) -> FVector4: ...
    @property
    def luul(self) -> FVector4: ...
    @property
    def luvu(self) -> FVector4: ...
    @property
    def luvv(self) -> FVector4: ...
    @property
    def luvo(self) -> FVector4: ...
    @property
    def luvl(self) -> FVector4: ...
    @property
    def luou(self) -> FVector4: ...
    @property
    def luov(self) -> FVector4: ...
    @property
    def luoo(self) -> FVector4: ...
    @property
    def luol(self) -> FVector4: ...
    @property
    def lulu(self) -> FVector4: ...
    @property
    def lulv(self) -> FVector4: ...
    @property
    def lulo(self) -> FVector4: ...
    @property
    def lull(self) -> FVector4: ...
    @property
    def lvuu(self) -> FVector4: ...
    @property
    def lvuv(self) -> FVector4: ...
    @property
    def lvuo(self) -> FVector4: ...
    @property
    def lvul(self) -> FVector4: ...
    @property
    def lvvu(self) -> FVector4: ...
    @property
    def lvvv(self) -> FVector4: ...
    @property
    def lvvo(self) -> FVector4: ...
    @property
    def lvvl(self) -> FVector4: ...
    @property
    def lvou(self) -> FVector4: ...
    @property
    def lvov(self) -> FVector4: ...
    @property
    def lvoo(self) -> FVector4: ...
    @property
    def lvol(self) -> FVector4: ...
    @property
    def lvlu(self) -> FVector4: ...
    @property
    def lvlv(self) -> FVector4: ...
    @property
    def lvlo(self) -> FVector4: ...
    @property
    def lvll(self) -> FVector4: ...
    @property
    def louu(self) -> FVector4: ...
    @property
    def louv(self) -> FVector4: ...
    @property
    def louo(self) -> FVector4: ...
    @property
    def loul(self) -> FVector4: ...
    @property
    def lovu(self) -> FVector4: ...
    @property
    def lovv(self) -> FVector4: ...
    @property
    def lovo(self) -> FVector4: ...
    @property
    def lovl(self) -> FVector4: ...
    @property
    def loou(self) -> FVector4: ...
    @property
    def loov(self) -> FVector4: ...
    @property
    def lolu(self) -> FVector4: ...
    @property
    def lolv(self) -> FVector4: ...
    @property
    def lluu(self) -> FVector4: ...
    @property
    def lluv(self) -> FVector4: ...
    @property
    def lluo(self) -> FVector4: ...
    @property
    def llul(self) -> FVector4: ...
    @property
    def llvu(self) -> FVector4: ...
    @property
    def llvv(self) -> FVector4: ...
    @property
    def llvo(self) -> FVector4: ...
    @property
    def llvl(self) -> FVector4: ...
    @property
    def llou(self) -> FVector4: ...
    @property
    def llov(self) -> FVector4: ...
    @property
    def lllu(self) -> FVector4: ...
    @property
    def lllv(self) -> FVector4: ...
    @property
    def oo(self) -> FVector2: ...
    @property
    def ol(self) -> FVector2: ...
    @property
    def lo(self) -> FVector2: ...
    @property
    def ll(self) -> FVector2: ...
    @property
    def ooo(self) -> FVector3: ...
    @property
    def ool(self) -> FVector3: ...
    @property
    def olo(self) -> FVector3: ...
    @property
    def oll(self) -> FVector3: ...
    @property
    def loo(self) -> FVector3: ...
    @property
    def lol(self) -> FVector3: ...
    @property
    def llo(self) -> FVector3: ...
    @property
    def lll(self) -> FVector3: ...
    @property
    def oooo(self) -> FVector4: ...
    @property
    def oool(self) -> FVector4: ...
    @property
    def oolo(self) -> FVector4: ...
    @property
    def ooll(self) -> FVector4: ...
    @property
    def oloo(self) -> FVector4: ...
    @property
    def olol(self) -> FVector4: ...
    @property
    def ollo(self) -> FVector4: ...
    @property
    def olll(self) -> FVector4: ...
    @property
    def looo(self) -> FVector4: ...
    @property
    def lool(self) -> FVector4: ...
    @property
    def lolo(self) -> FVector4: ...
    @property
    def loll(self) -> FVector4: ...
    @property
    def lloo(self) -> FVector4: ...
    @property
    def llol(self) -> FVector4: ...
    @property
    def lllo(self) -> FVector4: ...
    @property
    def llll(self) -> FVector4: ...
    @property
    def magnitude(self) -> float: ...
    def cross(self, other: FVector3, /) -> FVector3: ...
    def to_quaternion(self) -> FQuaternion: ...
    def normalize(self) -> FVector3: ...
    def distance(self, other: FVector3, /) -> float: ...
    def lerp(self, other: FVector3, x: float, /) -> FVector3: ...
    def min(self, n: Number, /) -> FVector3: ...
    def max(self, n: Number, /) -> FVector3: ...
    def clamp(self, min: Number, max: Number, /) -> FVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[FVector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector3: ...

@final
class FVector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: FVector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector3]: ...
    @overload
    def __getitem__(self, index: int) -> FVector3: ...
    @overload
    def __getitem__(self, index: slice) -> FVector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[FVector3]: ...

@final
class I8Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __add__(self, other: Number) -> I8Vector3: ...
    @overload
    def __radd__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __radd__(self, other: Number) -> I8Vector3: ...
    @overload
    def __sub__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __sub__(self, other: Number) -> I8Vector3: ...
    @overload
    def __rsub__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> I8Vector3: ...
    @overload
    def __mul__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __mul__(self, other: Number) -> I8Vector3: ...
    @overload
    def __rmul__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> I8Vector3: ...
    @overload
    def __truediv__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> I8Vector3: ...
    @overload
    def __rtruediv__(self, other: I8Vector3) -> I8Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> I8Vector3: ...
    def __neg__(self) -> I8Vector3: ...
    def __abs__(self) -> I8Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> I8Vector2: ...
    @property
    def xy(self) -> I8Vector2: ...
    @property
    def xz(self) -> I8Vector2: ...
    @property
    def xo(self) -> I8Vector2: ...
    @property
    def xl(self) -> I8Vector2: ...
    @property
    def yx(self) -> I8Vector2: ...
    @property
    def yy(self) -> I8Vector2: ...
    @property
    def yz(self) -> I8Vector2: ...
    @property
    def yo(self) -> I8Vector2: ...
    @property
    def yl(self) -> I8Vector2: ...
    @property
    def zx(self) -> I8Vector2: ...
    @property
    def zy(self) -> I8Vector2: ...
    @property
    def zz(self) -> I8Vector2: ...
    @property
    def zo(self) -> I8Vector2: ...
    @property
    def zl(self) -> I8Vector2: ...
    @property
    def ox(self) -> I8Vector2: ...
    @property
    def oy(self) -> I8Vector2: ...
    @property
    def oz(self) -> I8Vector2: ...
    @property
    def lx(self) -> I8Vector2: ...
    @property
    def ly(self) -> I8Vector2: ...
    @property
    def lz(self) -> I8Vector2: ...
    @property
    def xxx(self) -> I8Vector3: ...
    @property
    def xxy(self) -> I8Vector3: ...
    @property
    def xxz(self) -> I8Vector3: ...
    @property
    def xxo(self) -> I8Vector3: ...
    @property
    def xxl(self) -> I8Vector3: ...
    @property
    def xyx(self) -> I8Vector3: ...
    @property
    def xyy(self) -> I8Vector3: ...
    @property
    def xyz(self) -> I8Vector3: ...
    @property
    def xyo(self) -> I8Vector3: ...
    @property
    def xyl(self) -> I8Vector3: ...
    @property
    def xzx(self) -> I8Vector3: ...
    @property
    def xzy(self) -> I8Vector3: ...
    @property
    def xzz(self) -> I8Vector3: ...
    @property
    def xzo(self) -> I8Vector3: ...
    @property
    def xzl(self) -> I8Vector3: ...
    @property
    def xox(self) -> I8Vector3: ...
    @property
    def xoy(self) -> I8Vector3: ...
    @property
    def xoz(self) -> I8Vector3: ...
    @property
    def xoo(self) -> I8Vector3: ...
    @property
    def xol(self) -> I8Vector3: ...
    @property
    def xlx(self) -> I8Vector3: ...
    @property
    def xly(self) -> I8Vector3: ...
    @property
    def xlz(self) -> I8Vector3: ...
    @property
    def xlo(self) -> I8Vector3: ...
    @property
    def xll(self) -> I8Vector3: ...
    @property
    def yxx(self) -> I8Vector3: ...
    @property
    def yxy(self) -> I8Vector3: ...
    @property
    def yxz(self) -> I8Vector3: ...
    @property
    def yxo(self) -> I8Vector3: ...
    @property
    def yxl(self) -> I8Vector3: ...
    @property
    def yyx(self) -> I8Vector3: ...
    @property
    def yyy(self) -> I8Vector3: ...
    @property
    def yyz(self) -> I8Vector3: ...
    @property
    def yyo(self) -> I8Vector3: ...
    @property
    def yyl(self) -> I8Vector3: ...
    @property
    def yzx(self) -> I8Vector3: ...
    @property
    def yzy(self) -> I8Vector3: ...
    @property
    def yzz(self) -> I8Vector3: ...
    @property
    def yzo(self) -> I8Vector3: ...
    @property
    def yzl(self) -> I8Vector3: ...
    @property
    def yox(self) -> I8Vector3: ...
    @property
    def yoy(self) -> I8Vector3: ...
    @property
    def yoz(self) -> I8Vector3: ...
    @property
    def yoo(self) -> I8Vector3: ...
    @property
    def yol(self) -> I8Vector3: ...
    @property
    def ylx(self) -> I8Vector3: ...
    @property
    def yly(self) -> I8Vector3: ...
    @property
    def ylz(self) -> I8Vector3: ...
    @property
    def ylo(self) -> I8Vector3: ...
    @property
    def yll(self) -> I8Vector3: ...
    @property
    def zxx(self) -> I8Vector3: ...
    @property
    def zxy(self) -> I8Vector3: ...
    @property
    def zxz(self) -> I8Vector3: ...
    @property
    def zxo(self) -> I8Vector3: ...
    @property
    def zxl(self) -> I8Vector3: ...
    @property
    def zyx(self) -> I8Vector3: ...
    @property
    def zyy(self) -> I8Vector3: ...
    @property
    def zyz(self) -> I8Vector3: ...
    @property
    def zyo(self) -> I8Vector3: ...
    @property
    def zyl(self) -> I8Vector3: ...
    @property
    def zzx(self) -> I8Vector3: ...
    @property
    def zzy(self) -> I8Vector3: ...
    @property
    def zzz(self) -> I8Vector3: ...
    @property
    def zzo(self) -> I8Vector3: ...
    @property
    def zzl(self) -> I8Vector3: ...
    @property
    def zox(self) -> I8Vector3: ...
    @property
    def zoy(self) -> I8Vector3: ...
    @property
    def zoz(self) -> I8Vector3: ...
    @property
    def zoo(self) -> I8Vector3: ...
    @property
    def zol(self) -> I8Vector3: ...
    @property
    def zlx(self) -> I8Vector3: ...
    @property
    def zly(self) -> I8Vector3: ...
    @property
    def zlz(self) -> I8Vector3: ...
    @property
    def zlo(self) -> I8Vector3: ...
    @property
    def zll(self) -> I8Vector3: ...
    @property
    def oxx(self) -> I8Vector3: ...
    @property
    def oxy(self) -> I8Vector3: ...
    @property
    def oxz(self) -> I8Vector3: ...
    @property
    def oxo(self) -> I8Vector3: ...
    @property
    def oxl(self) -> I8Vector3: ...
    @property
    def oyx(self) -> I8Vector3: ...
    @property
    def oyy(self) -> I8Vector3: ...
    @property
    def oyz(self) -> I8Vector3: ...
    @property
    def oyo(self) -> I8Vector3: ...
    @property
    def oyl(self) -> I8Vector3: ...
    @property
    def ozx(self) -> I8Vector3: ...
    @property
    def ozy(self) -> I8Vector3: ...
    @property
    def ozz(self) -> I8Vector3: ...
    @property
    def ozo(self) -> I8Vector3: ...
    @property
    def ozl(self) -> I8Vector3: ...
    @property
    def oox(self) -> I8Vector3: ...
    @property
    def ooy(self) -> I8Vector3: ...
    @property
    def ooz(self) -> I8Vector3: ...
    @property
    def olx(self) -> I8Vector3: ...
    @property
    def oly(self) -> I8Vector3: ...
    @property
    def olz(self) -> I8Vector3: ...
    @property
    def lxx(self) -> I8Vector3: ...
    @property
    def lxy(self) -> I8Vector3: ...
    @property
    def lxz(self) -> I8Vector3: ...
    @property
    def lxo(self) -> I8Vector3: ...
    @property
    def lxl(self) -> I8Vector3: ...
    @property
    def lyx(self) -> I8Vector3: ...
    @property
    def lyy(self) -> I8Vector3: ...
    @property
    def lyz(self) -> I8Vector3: ...
    @property
    def lyo(self) -> I8Vector3: ...
    @property
    def lyl(self) -> I8Vector3: ...
    @property
    def lzx(self) -> I8Vector3: ...
    @property
    def lzy(self) -> I8Vector3: ...
    @property
    def lzz(self) -> I8Vector3: ...
    @property
    def lzo(self) -> I8Vector3: ...
    @property
    def lzl(self) -> I8Vector3: ...
    @property
    def lox(self) -> I8Vector3: ...
    @property
    def loy(self) -> I8Vector3: ...
    @property
    def loz(self) -> I8Vector3: ...
    @property
    def llx(self) -> I8Vector3: ...
    @property
    def lly(self) -> I8Vector3: ...
    @property
    def llz(self) -> I8Vector3: ...
    @property
    def xxxx(self) -> I8Vector4: ...
    @property
    def xxxy(self) -> I8Vector4: ...
    @property
    def xxxz(self) -> I8Vector4: ...
    @property
    def xxxo(self) -> I8Vector4: ...
    @property
    def xxxl(self) -> I8Vector4: ...
    @property
    def xxyx(self) -> I8Vector4: ...
    @property
    def xxyy(self) -> I8Vector4: ...
    @property
    def xxyz(self) -> I8Vector4: ...
    @property
    def xxyo(self) -> I8Vector4: ...
    @property
    def xxyl(self) -> I8Vector4: ...
    @property
    def xxzx(self) -> I8Vector4: ...
    @property
    def xxzy(self) -> I8Vector4: ...
    @property
    def xxzz(self) -> I8Vector4: ...
    @property
    def xxzo(self) -> I8Vector4: ...
    @property
    def xxzl(self) -> I8Vector4: ...
    @property
    def xxox(self) -> I8Vector4: ...
    @property
    def xxoy(self) -> I8Vector4: ...
    @property
    def xxoz(self) -> I8Vector4: ...
    @property
    def xxoo(self) -> I8Vector4: ...
    @property
    def xxol(self) -> I8Vector4: ...
    @property
    def xxlx(self) -> I8Vector4: ...
    @property
    def xxly(self) -> I8Vector4: ...
    @property
    def xxlz(self) -> I8Vector4: ...
    @property
    def xxlo(self) -> I8Vector4: ...
    @property
    def xxll(self) -> I8Vector4: ...
    @property
    def xyxx(self) -> I8Vector4: ...
    @property
    def xyxy(self) -> I8Vector4: ...
    @property
    def xyxz(self) -> I8Vector4: ...
    @property
    def xyxo(self) -> I8Vector4: ...
    @property
    def xyxl(self) -> I8Vector4: ...
    @property
    def xyyx(self) -> I8Vector4: ...
    @property
    def xyyy(self) -> I8Vector4: ...
    @property
    def xyyz(self) -> I8Vector4: ...
    @property
    def xyyo(self) -> I8Vector4: ...
    @property
    def xyyl(self) -> I8Vector4: ...
    @property
    def xyzx(self) -> I8Vector4: ...
    @property
    def xyzy(self) -> I8Vector4: ...
    @property
    def xyzz(self) -> I8Vector4: ...
    @property
    def xyzo(self) -> I8Vector4: ...
    @property
    def xyzl(self) -> I8Vector4: ...
    @property
    def xyox(self) -> I8Vector4: ...
    @property
    def xyoy(self) -> I8Vector4: ...
    @property
    def xyoz(self) -> I8Vector4: ...
    @property
    def xyoo(self) -> I8Vector4: ...
    @property
    def xyol(self) -> I8Vector4: ...
    @property
    def xylx(self) -> I8Vector4: ...
    @property
    def xyly(self) -> I8Vector4: ...
    @property
    def xylz(self) -> I8Vector4: ...
    @property
    def xylo(self) -> I8Vector4: ...
    @property
    def xyll(self) -> I8Vector4: ...
    @property
    def xzxx(self) -> I8Vector4: ...
    @property
    def xzxy(self) -> I8Vector4: ...
    @property
    def xzxz(self) -> I8Vector4: ...
    @property
    def xzxo(self) -> I8Vector4: ...
    @property
    def xzxl(self) -> I8Vector4: ...
    @property
    def xzyx(self) -> I8Vector4: ...
    @property
    def xzyy(self) -> I8Vector4: ...
    @property
    def xzyz(self) -> I8Vector4: ...
    @property
    def xzyo(self) -> I8Vector4: ...
    @property
    def xzyl(self) -> I8Vector4: ...
    @property
    def xzzx(self) -> I8Vector4: ...
    @property
    def xzzy(self) -> I8Vector4: ...
    @property
    def xzzz(self) -> I8Vector4: ...
    @property
    def xzzo(self) -> I8Vector4: ...
    @property
    def xzzl(self) -> I8Vector4: ...
    @property
    def xzox(self) -> I8Vector4: ...
    @property
    def xzoy(self) -> I8Vector4: ...
    @property
    def xzoz(self) -> I8Vector4: ...
    @property
    def xzoo(self) -> I8Vector4: ...
    @property
    def xzol(self) -> I8Vector4: ...
    @property
    def xzlx(self) -> I8Vector4: ...
    @property
    def xzly(self) -> I8Vector4: ...
    @property
    def xzlz(self) -> I8Vector4: ...
    @property
    def xzlo(self) -> I8Vector4: ...
    @property
    def xzll(self) -> I8Vector4: ...
    @property
    def xoxx(self) -> I8Vector4: ...
    @property
    def xoxy(self) -> I8Vector4: ...
    @property
    def xoxz(self) -> I8Vector4: ...
    @property
    def xoxo(self) -> I8Vector4: ...
    @property
    def xoxl(self) -> I8Vector4: ...
    @property
    def xoyx(self) -> I8Vector4: ...
    @property
    def xoyy(self) -> I8Vector4: ...
    @property
    def xoyz(self) -> I8Vector4: ...
    @property
    def xoyo(self) -> I8Vector4: ...
    @property
    def xoyl(self) -> I8Vector4: ...
    @property
    def xozx(self) -> I8Vector4: ...
    @property
    def xozy(self) -> I8Vector4: ...
    @property
    def xozz(self) -> I8Vector4: ...
    @property
    def xozo(self) -> I8Vector4: ...
    @property
    def xozl(self) -> I8Vector4: ...
    @property
    def xoox(self) -> I8Vector4: ...
    @property
    def xooy(self) -> I8Vector4: ...
    @property
    def xooz(self) -> I8Vector4: ...
    @property
    def xooo(self) -> I8Vector4: ...
    @property
    def xool(self) -> I8Vector4: ...
    @property
    def xolx(self) -> I8Vector4: ...
    @property
    def xoly(self) -> I8Vector4: ...
    @property
    def xolz(self) -> I8Vector4: ...
    @property
    def xolo(self) -> I8Vector4: ...
    @property
    def xoll(self) -> I8Vector4: ...
    @property
    def xlxx(self) -> I8Vector4: ...
    @property
    def xlxy(self) -> I8Vector4: ...
    @property
    def xlxz(self) -> I8Vector4: ...
    @property
    def xlxo(self) -> I8Vector4: ...
    @property
    def xlxl(self) -> I8Vector4: ...
    @property
    def xlyx(self) -> I8Vector4: ...
    @property
    def xlyy(self) -> I8Vector4: ...
    @property
    def xlyz(self) -> I8Vector4: ...
    @property
    def xlyo(self) -> I8Vector4: ...
    @property
    def xlyl(self) -> I8Vector4: ...
    @property
    def xlzx(self) -> I8Vector4: ...
    @property
    def xlzy(self) -> I8Vector4: ...
    @property
    def xlzz(self) -> I8Vector4: ...
    @property
    def xlzo(self) -> I8Vector4: ...
    @property
    def xlzl(self) -> I8Vector4: ...
    @property
    def xlox(self) -> I8Vector4: ...
    @property
    def xloy(self) -> I8Vector4: ...
    @property
    def xloz(self) -> I8Vector4: ...
    @property
    def xloo(self) -> I8Vector4: ...
    @property
    def xlol(self) -> I8Vector4: ...
    @property
    def xllx(self) -> I8Vector4: ...
    @property
    def xlly(self) -> I8Vector4: ...
    @property
    def xllz(self) -> I8Vector4: ...
    @property
    def xllo(self) -> I8Vector4: ...
    @property
    def xlll(self) -> I8Vector4: ...
    @property
    def yxxx(self) -> I8Vector4: ...
    @property
    def yxxy(self) -> I8Vector4: ...
    @property
    def yxxz(self) -> I8Vector4: ...
    @property
    def yxxo(self) -> I8Vector4: ...
    @property
    def yxxl(self) -> I8Vector4: ...
    @property
    def yxyx(self) -> I8Vector4: ...
    @property
    def yxyy(self) -> I8Vector4: ...
    @property
    def yxyz(self) -> I8Vector4: ...
    @property
    def yxyo(self) -> I8Vector4: ...
    @property
    def yxyl(self) -> I8Vector4: ...
    @property
    def yxzx(self) -> I8Vector4: ...
    @property
    def yxzy(self) -> I8Vector4: ...
    @property
    def yxzz(self) -> I8Vector4: ...
    @property
    def yxzo(self) -> I8Vector4: ...
    @property
    def yxzl(self) -> I8Vector4: ...
    @property
    def yxox(self) -> I8Vector4: ...
    @property
    def yxoy(self) -> I8Vector4: ...
    @property
    def yxoz(self) -> I8Vector4: ...
    @property
    def yxoo(self) -> I8Vector4: ...
    @property
    def yxol(self) -> I8Vector4: ...
    @property
    def yxlx(self) -> I8Vector4: ...
    @property
    def yxly(self) -> I8Vector4: ...
    @property
    def yxlz(self) -> I8Vector4: ...
    @property
    def yxlo(self) -> I8Vector4: ...
    @property
    def yxll(self) -> I8Vector4: ...
    @property
    def yyxx(self) -> I8Vector4: ...
    @property
    def yyxy(self) -> I8Vector4: ...
    @property
    def yyxz(self) -> I8Vector4: ...
    @property
    def yyxo(self) -> I8Vector4: ...
    @property
    def yyxl(self) -> I8Vector4: ...
    @property
    def yyyx(self) -> I8Vector4: ...
    @property
    def yyyy(self) -> I8Vector4: ...
    @property
    def yyyz(self) -> I8Vector4: ...
    @property
    def yyyo(self) -> I8Vector4: ...
    @property
    def yyyl(self) -> I8Vector4: ...
    @property
    def yyzx(self) -> I8Vector4: ...
    @property
    def yyzy(self) -> I8Vector4: ...
    @property
    def yyzz(self) -> I8Vector4: ...
    @property
    def yyzo(self) -> I8Vector4: ...
    @property
    def yyzl(self) -> I8Vector4: ...
    @property
    def yyox(self) -> I8Vector4: ...
    @property
    def yyoy(self) -> I8Vector4: ...
    @property
    def yyoz(self) -> I8Vector4: ...
    @property
    def yyoo(self) -> I8Vector4: ...
    @property
    def yyol(self) -> I8Vector4: ...
    @property
    def yylx(self) -> I8Vector4: ...
    @property
    def yyly(self) -> I8Vector4: ...
    @property
    def yylz(self) -> I8Vector4: ...
    @property
    def yylo(self) -> I8Vector4: ...
    @property
    def yyll(self) -> I8Vector4: ...
    @property
    def yzxx(self) -> I8Vector4: ...
    @property
    def yzxy(self) -> I8Vector4: ...
    @property
    def yzxz(self) -> I8Vector4: ...
    @property
    def yzxo(self) -> I8Vector4: ...
    @property
    def yzxl(self) -> I8Vector4: ...
    @property
    def yzyx(self) -> I8Vector4: ...
    @property
    def yzyy(self) -> I8Vector4: ...
    @property
    def yzyz(self) -> I8Vector4: ...
    @property
    def yzyo(self) -> I8Vector4: ...
    @property
    def yzyl(self) -> I8Vector4: ...
    @property
    def yzzx(self) -> I8Vector4: ...
    @property
    def yzzy(self) -> I8Vector4: ...
    @property
    def yzzz(self) -> I8Vector4: ...
    @property
    def yzzo(self) -> I8Vector4: ...
    @property
    def yzzl(self) -> I8Vector4: ...
    @property
    def yzox(self) -> I8Vector4: ...
    @property
    def yzoy(self) -> I8Vector4: ...
    @property
    def yzoz(self) -> I8Vector4: ...
    @property
    def yzoo(self) -> I8Vector4: ...
    @property
    def yzol(self) -> I8Vector4: ...
    @property
    def yzlx(self) -> I8Vector4: ...
    @property
    def yzly(self) -> I8Vector4: ...
    @property
    def yzlz(self) -> I8Vector4: ...
    @property
    def yzlo(self) -> I8Vector4: ...
    @property
    def yzll(self) -> I8Vector4: ...
    @property
    def yoxx(self) -> I8Vector4: ...
    @property
    def yoxy(self) -> I8Vector4: ...
    @property
    def yoxz(self) -> I8Vector4: ...
    @property
    def yoxo(self) -> I8Vector4: ...
    @property
    def yoxl(self) -> I8Vector4: ...
    @property
    def yoyx(self) -> I8Vector4: ...
    @property
    def yoyy(self) -> I8Vector4: ...
    @property
    def yoyz(self) -> I8Vector4: ...
    @property
    def yoyo(self) -> I8Vector4: ...
    @property
    def yoyl(self) -> I8Vector4: ...
    @property
    def yozx(self) -> I8Vector4: ...
    @property
    def yozy(self) -> I8Vector4: ...
    @property
    def yozz(self) -> I8Vector4: ...
    @property
    def yozo(self) -> I8Vector4: ...
    @property
    def yozl(self) -> I8Vector4: ...
    @property
    def yoox(self) -> I8Vector4: ...
    @property
    def yooy(self) -> I8Vector4: ...
    @property
    def yooz(self) -> I8Vector4: ...
    @property
    def yooo(self) -> I8Vector4: ...
    @property
    def yool(self) -> I8Vector4: ...
    @property
    def yolx(self) -> I8Vector4: ...
    @property
    def yoly(self) -> I8Vector4: ...
    @property
    def yolz(self) -> I8Vector4: ...
    @property
    def yolo(self) -> I8Vector4: ...
    @property
    def yoll(self) -> I8Vector4: ...
    @property
    def ylxx(self) -> I8Vector4: ...
    @property
    def ylxy(self) -> I8Vector4: ...
    @property
    def ylxz(self) -> I8Vector4: ...
    @property
    def ylxo(self) -> I8Vector4: ...
    @property
    def ylxl(self) -> I8Vector4: ...
    @property
    def ylyx(self) -> I8Vector4: ...
    @property
    def ylyy(self) -> I8Vector4: ...
    @property
    def ylyz(self) -> I8Vector4: ...
    @property
    def ylyo(self) -> I8Vector4: ...
    @property
    def ylyl(self) -> I8Vector4: ...
    @property
    def ylzx(self) -> I8Vector4: ...
    @property
    def ylzy(self) -> I8Vector4: ...
    @property
    def ylzz(self) -> I8Vector4: ...
    @property
    def ylzo(self) -> I8Vector4: ...
    @property
    def ylzl(self) -> I8Vector4: ...
    @property
    def ylox(self) -> I8Vector4: ...
    @property
    def yloy(self) -> I8Vector4: ...
    @property
    def yloz(self) -> I8Vector4: ...
    @property
    def yloo(self) -> I8Vector4: ...
    @property
    def ylol(self) -> I8Vector4: ...
    @property
    def yllx(self) -> I8Vector4: ...
    @property
    def ylly(self) -> I8Vector4: ...
    @property
    def yllz(self) -> I8Vector4: ...
    @property
    def yllo(self) -> I8Vector4: ...
    @property
    def ylll(self) -> I8Vector4: ...
    @property
    def zxxx(self) -> I8Vector4: ...
    @property
    def zxxy(self) -> I8Vector4: ...
    @property
    def zxxz(self) -> I8Vector4: ...
    @property
    def zxxo(self) -> I8Vector4: ...
    @property
    def zxxl(self) -> I8Vector4: ...
    @property
    def zxyx(self) -> I8Vector4: ...
    @property
    def zxyy(self) -> I8Vector4: ...
    @property
    def zxyz(self) -> I8Vector4: ...
    @property
    def zxyo(self) -> I8Vector4: ...
    @property
    def zxyl(self) -> I8Vector4: ...
    @property
    def zxzx(self) -> I8Vector4: ...
    @property
    def zxzy(self) -> I8Vector4: ...
    @property
    def zxzz(self) -> I8Vector4: ...
    @property
    def zxzo(self) -> I8Vector4: ...
    @property
    def zxzl(self) -> I8Vector4: ...
    @property
    def zxox(self) -> I8Vector4: ...
    @property
    def zxoy(self) -> I8Vector4: ...
    @property
    def zxoz(self) -> I8Vector4: ...
    @property
    def zxoo(self) -> I8Vector4: ...
    @property
    def zxol(self) -> I8Vector4: ...
    @property
    def zxlx(self) -> I8Vector4: ...
    @property
    def zxly(self) -> I8Vector4: ...
    @property
    def zxlz(self) -> I8Vector4: ...
    @property
    def zxlo(self) -> I8Vector4: ...
    @property
    def zxll(self) -> I8Vector4: ...
    @property
    def zyxx(self) -> I8Vector4: ...
    @property
    def zyxy(self) -> I8Vector4: ...
    @property
    def zyxz(self) -> I8Vector4: ...
    @property
    def zyxo(self) -> I8Vector4: ...
    @property
    def zyxl(self) -> I8Vector4: ...
    @property
    def zyyx(self) -> I8Vector4: ...
    @property
    def zyyy(self) -> I8Vector4: ...
    @property
    def zyyz(self) -> I8Vector4: ...
    @property
    def zyyo(self) -> I8Vector4: ...
    @property
    def zyyl(self) -> I8Vector4: ...
    @property
    def zyzx(self) -> I8Vector4: ...
    @property
    def zyzy(self) -> I8Vector4: ...
    @property
    def zyzz(self) -> I8Vector4: ...
    @property
    def zyzo(self) -> I8Vector4: ...
    @property
    def zyzl(self) -> I8Vector4: ...
    @property
    def zyox(self) -> I8Vector4: ...
    @property
    def zyoy(self) -> I8Vector4: ...
    @property
    def zyoz(self) -> I8Vector4: ...
    @property
    def zyoo(self) -> I8Vector4: ...
    @property
    def zyol(self) -> I8Vector4: ...
    @property
    def zylx(self) -> I8Vector4: ...
    @property
    def zyly(self) -> I8Vector4: ...
    @property
    def zylz(self) -> I8Vector4: ...
    @property
    def zylo(self) -> I8Vector4: ...
    @property
    def zyll(self) -> I8Vector4: ...
    @property
    def zzxx(self) -> I8Vector4: ...
    @property
    def zzxy(self) -> I8Vector4: ...
    @property
    def zzxz(self) -> I8Vector4: ...
    @property
    def zzxo(self) -> I8Vector4: ...
    @property
    def zzxl(self) -> I8Vector4: ...
    @property
    def zzyx(self) -> I8Vector4: ...
    @property
    def zzyy(self) -> I8Vector4: ...
    @property
    def zzyz(self) -> I8Vector4: ...
    @property
    def zzyo(self) -> I8Vector4: ...
    @property
    def zzyl(self) -> I8Vector4: ...
    @property
    def zzzx(self) -> I8Vector4: ...
    @property
    def zzzy(self) -> I8Vector4: ...
    @property
    def zzzz(self) -> I8Vector4: ...
    @property
    def zzzo(self) -> I8Vector4: ...
    @property
    def zzzl(self) -> I8Vector4: ...
    @property
    def zzox(self) -> I8Vector4: ...
    @property
    def zzoy(self) -> I8Vector4: ...
    @property
    def zzoz(self) -> I8Vector4: ...
    @property
    def zzoo(self) -> I8Vector4: ...
    @property
    def zzol(self) -> I8Vector4: ...
    @property
    def zzlx(self) -> I8Vector4: ...
    @property
    def zzly(self) -> I8Vector4: ...
    @property
    def zzlz(self) -> I8Vector4: ...
    @property
    def zzlo(self) -> I8Vector4: ...
    @property
    def zzll(self) -> I8Vector4: ...
    @property
    def zoxx(self) -> I8Vector4: ...
    @property
    def zoxy(self) -> I8Vector4: ...
    @property
    def zoxz(self) -> I8Vector4: ...
    @property
    def zoxo(self) -> I8Vector4: ...
    @property
    def zoxl(self) -> I8Vector4: ...
    @property
    def zoyx(self) -> I8Vector4: ...
    @property
    def zoyy(self) -> I8Vector4: ...
    @property
    def zoyz(self) -> I8Vector4: ...
    @property
    def zoyo(self) -> I8Vector4: ...
    @property
    def zoyl(self) -> I8Vector4: ...
    @property
    def zozx(self) -> I8Vector4: ...
    @property
    def zozy(self) -> I8Vector4: ...
    @property
    def zozz(self) -> I8Vector4: ...
    @property
    def zozo(self) -> I8Vector4: ...
    @property
    def zozl(self) -> I8Vector4: ...
    @property
    def zoox(self) -> I8Vector4: ...
    @property
    def zooy(self) -> I8Vector4: ...
    @property
    def zooz(self) -> I8Vector4: ...
    @property
    def zooo(self) -> I8Vector4: ...
    @property
    def zool(self) -> I8Vector4: ...
    @property
    def zolx(self) -> I8Vector4: ...
    @property
    def zoly(self) -> I8Vector4: ...
    @property
    def zolz(self) -> I8Vector4: ...
    @property
    def zolo(self) -> I8Vector4: ...
    @property
    def zoll(self) -> I8Vector4: ...
    @property
    def zlxx(self) -> I8Vector4: ...
    @property
    def zlxy(self) -> I8Vector4: ...
    @property
    def zlxz(self) -> I8Vector4: ...
    @property
    def zlxo(self) -> I8Vector4: ...
    @property
    def zlxl(self) -> I8Vector4: ...
    @property
    def zlyx(self) -> I8Vector4: ...
    @property
    def zlyy(self) -> I8Vector4: ...
    @property
    def zlyz(self) -> I8Vector4: ...
    @property
    def zlyo(self) -> I8Vector4: ...
    @property
    def zlyl(self) -> I8Vector4: ...
    @property
    def zlzx(self) -> I8Vector4: ...
    @property
    def zlzy(self) -> I8Vector4: ...
    @property
    def zlzz(self) -> I8Vector4: ...
    @property
    def zlzo(self) -> I8Vector4: ...
    @property
    def zlzl(self) -> I8Vector4: ...
    @property
    def zlox(self) -> I8Vector4: ...
    @property
    def zloy(self) -> I8Vector4: ...
    @property
    def zloz(self) -> I8Vector4: ...
    @property
    def zloo(self) -> I8Vector4: ...
    @property
    def zlol(self) -> I8Vector4: ...
    @property
    def zllx(self) -> I8Vector4: ...
    @property
    def zlly(self) -> I8Vector4: ...
    @property
    def zllz(self) -> I8Vector4: ...
    @property
    def zllo(self) -> I8Vector4: ...
    @property
    def zlll(self) -> I8Vector4: ...
    @property
    def oxxx(self) -> I8Vector4: ...
    @property
    def oxxy(self) -> I8Vector4: ...
    @property
    def oxxz(self) -> I8Vector4: ...
    @property
    def oxxo(self) -> I8Vector4: ...
    @property
    def oxxl(self) -> I8Vector4: ...
    @property
    def oxyx(self) -> I8Vector4: ...
    @property
    def oxyy(self) -> I8Vector4: ...
    @property
    def oxyz(self) -> I8Vector4: ...
    @property
    def oxyo(self) -> I8Vector4: ...
    @property
    def oxyl(self) -> I8Vector4: ...
    @property
    def oxzx(self) -> I8Vector4: ...
    @property
    def oxzy(self) -> I8Vector4: ...
    @property
    def oxzz(self) -> I8Vector4: ...
    @property
    def oxzo(self) -> I8Vector4: ...
    @property
    def oxzl(self) -> I8Vector4: ...
    @property
    def oxox(self) -> I8Vector4: ...
    @property
    def oxoy(self) -> I8Vector4: ...
    @property
    def oxoz(self) -> I8Vector4: ...
    @property
    def oxoo(self) -> I8Vector4: ...
    @property
    def oxol(self) -> I8Vector4: ...
    @property
    def oxlx(self) -> I8Vector4: ...
    @property
    def oxly(self) -> I8Vector4: ...
    @property
    def oxlz(self) -> I8Vector4: ...
    @property
    def oxlo(self) -> I8Vector4: ...
    @property
    def oxll(self) -> I8Vector4: ...
    @property
    def oyxx(self) -> I8Vector4: ...
    @property
    def oyxy(self) -> I8Vector4: ...
    @property
    def oyxz(self) -> I8Vector4: ...
    @property
    def oyxo(self) -> I8Vector4: ...
    @property
    def oyxl(self) -> I8Vector4: ...
    @property
    def oyyx(self) -> I8Vector4: ...
    @property
    def oyyy(self) -> I8Vector4: ...
    @property
    def oyyz(self) -> I8Vector4: ...
    @property
    def oyyo(self) -> I8Vector4: ...
    @property
    def oyyl(self) -> I8Vector4: ...
    @property
    def oyzx(self) -> I8Vector4: ...
    @property
    def oyzy(self) -> I8Vector4: ...
    @property
    def oyzz(self) -> I8Vector4: ...
    @property
    def oyzo(self) -> I8Vector4: ...
    @property
    def oyzl(self) -> I8Vector4: ...
    @property
    def oyox(self) -> I8Vector4: ...
    @property
    def oyoy(self) -> I8Vector4: ...
    @property
    def oyoz(self) -> I8Vector4: ...
    @property
    def oyoo(self) -> I8Vector4: ...
    @property
    def oyol(self) -> I8Vector4: ...
    @property
    def oylx(self) -> I8Vector4: ...
    @property
    def oyly(self) -> I8Vector4: ...
    @property
    def oylz(self) -> I8Vector4: ...
    @property
    def oylo(self) -> I8Vector4: ...
    @property
    def oyll(self) -> I8Vector4: ...
    @property
    def ozxx(self) -> I8Vector4: ...
    @property
    def ozxy(self) -> I8Vector4: ...
    @property
    def ozxz(self) -> I8Vector4: ...
    @property
    def ozxo(self) -> I8Vector4: ...
    @property
    def ozxl(self) -> I8Vector4: ...
    @property
    def ozyx(self) -> I8Vector4: ...
    @property
    def ozyy(self) -> I8Vector4: ...
    @property
    def ozyz(self) -> I8Vector4: ...
    @property
    def ozyo(self) -> I8Vector4: ...
    @property
    def ozyl(self) -> I8Vector4: ...
    @property
    def ozzx(self) -> I8Vector4: ...
    @property
    def ozzy(self) -> I8Vector4: ...
    @property
    def ozzz(self) -> I8Vector4: ...
    @property
    def ozzo(self) -> I8Vector4: ...
    @property
    def ozzl(self) -> I8Vector4: ...
    @property
    def ozox(self) -> I8Vector4: ...
    @property
    def ozoy(self) -> I8Vector4: ...
    @property
    def ozoz(self) -> I8Vector4: ...
    @property
    def ozoo(self) -> I8Vector4: ...
    @property
    def ozol(self) -> I8Vector4: ...
    @property
    def ozlx(self) -> I8Vector4: ...
    @property
    def ozly(self) -> I8Vector4: ...
    @property
    def ozlz(self) -> I8Vector4: ...
    @property
    def ozlo(self) -> I8Vector4: ...
    @property
    def ozll(self) -> I8Vector4: ...
    @property
    def ooxx(self) -> I8Vector4: ...
    @property
    def ooxy(self) -> I8Vector4: ...
    @property
    def ooxz(self) -> I8Vector4: ...
    @property
    def ooxo(self) -> I8Vector4: ...
    @property
    def ooxl(self) -> I8Vector4: ...
    @property
    def ooyx(self) -> I8Vector4: ...
    @property
    def ooyy(self) -> I8Vector4: ...
    @property
    def ooyz(self) -> I8Vector4: ...
    @property
    def ooyo(self) -> I8Vector4: ...
    @property
    def ooyl(self) -> I8Vector4: ...
    @property
    def oozx(self) -> I8Vector4: ...
    @property
    def oozy(self) -> I8Vector4: ...
    @property
    def oozz(self) -> I8Vector4: ...
    @property
    def oozo(self) -> I8Vector4: ...
    @property
    def oozl(self) -> I8Vector4: ...
    @property
    def ooox(self) -> I8Vector4: ...
    @property
    def oooy(self) -> I8Vector4: ...
    @property
    def oooz(self) -> I8Vector4: ...
    @property
    def oolx(self) -> I8Vector4: ...
    @property
    def ooly(self) -> I8Vector4: ...
    @property
    def oolz(self) -> I8Vector4: ...
    @property
    def olxx(self) -> I8Vector4: ...
    @property
    def olxy(self) -> I8Vector4: ...
    @property
    def olxz(self) -> I8Vector4: ...
    @property
    def olxo(self) -> I8Vector4: ...
    @property
    def olxl(self) -> I8Vector4: ...
    @property
    def olyx(self) -> I8Vector4: ...
    @property
    def olyy(self) -> I8Vector4: ...
    @property
    def olyz(self) -> I8Vector4: ...
    @property
    def olyo(self) -> I8Vector4: ...
    @property
    def olyl(self) -> I8Vector4: ...
    @property
    def olzx(self) -> I8Vector4: ...
    @property
    def olzy(self) -> I8Vector4: ...
    @property
    def olzz(self) -> I8Vector4: ...
    @property
    def olzo(self) -> I8Vector4: ...
    @property
    def olzl(self) -> I8Vector4: ...
    @property
    def olox(self) -> I8Vector4: ...
    @property
    def oloy(self) -> I8Vector4: ...
    @property
    def oloz(self) -> I8Vector4: ...
    @property
    def ollx(self) -> I8Vector4: ...
    @property
    def olly(self) -> I8Vector4: ...
    @property
    def ollz(self) -> I8Vector4: ...
    @property
    def lxxx(self) -> I8Vector4: ...
    @property
    def lxxy(self) -> I8Vector4: ...
    @property
    def lxxz(self) -> I8Vector4: ...
    @property
    def lxxo(self) -> I8Vector4: ...
    @property
    def lxxl(self) -> I8Vector4: ...
    @property
    def lxyx(self) -> I8Vector4: ...
    @property
    def lxyy(self) -> I8Vector4: ...
    @property
    def lxyz(self) -> I8Vector4: ...
    @property
    def lxyo(self) -> I8Vector4: ...
    @property
    def lxyl(self) -> I8Vector4: ...
    @property
    def lxzx(self) -> I8Vector4: ...
    @property
    def lxzy(self) -> I8Vector4: ...
    @property
    def lxzz(self) -> I8Vector4: ...
    @property
    def lxzo(self) -> I8Vector4: ...
    @property
    def lxzl(self) -> I8Vector4: ...
    @property
    def lxox(self) -> I8Vector4: ...
    @property
    def lxoy(self) -> I8Vector4: ...
    @property
    def lxoz(self) -> I8Vector4: ...
    @property
    def lxoo(self) -> I8Vector4: ...
    @property
    def lxol(self) -> I8Vector4: ...
    @property
    def lxlx(self) -> I8Vector4: ...
    @property
    def lxly(self) -> I8Vector4: ...
    @property
    def lxlz(self) -> I8Vector4: ...
    @property
    def lxlo(self) -> I8Vector4: ...
    @property
    def lxll(self) -> I8Vector4: ...
    @property
    def lyxx(self) -> I8Vector4: ...
    @property
    def lyxy(self) -> I8Vector4: ...
    @property
    def lyxz(self) -> I8Vector4: ...
    @property
    def lyxo(self) -> I8Vector4: ...
    @property
    def lyxl(self) -> I8Vector4: ...
    @property
    def lyyx(self) -> I8Vector4: ...
    @property
    def lyyy(self) -> I8Vector4: ...
    @property
    def lyyz(self) -> I8Vector4: ...
    @property
    def lyyo(self) -> I8Vector4: ...
    @property
    def lyyl(self) -> I8Vector4: ...
    @property
    def lyzx(self) -> I8Vector4: ...
    @property
    def lyzy(self) -> I8Vector4: ...
    @property
    def lyzz(self) -> I8Vector4: ...
    @property
    def lyzo(self) -> I8Vector4: ...
    @property
    def lyzl(self) -> I8Vector4: ...
    @property
    def lyox(self) -> I8Vector4: ...
    @property
    def lyoy(self) -> I8Vector4: ...
    @property
    def lyoz(self) -> I8Vector4: ...
    @property
    def lyoo(self) -> I8Vector4: ...
    @property
    def lyol(self) -> I8Vector4: ...
    @property
    def lylx(self) -> I8Vector4: ...
    @property
    def lyly(self) -> I8Vector4: ...
    @property
    def lylz(self) -> I8Vector4: ...
    @property
    def lylo(self) -> I8Vector4: ...
    @property
    def lyll(self) -> I8Vector4: ...
    @property
    def lzxx(self) -> I8Vector4: ...
    @property
    def lzxy(self) -> I8Vector4: ...
    @property
    def lzxz(self) -> I8Vector4: ...
    @property
    def lzxo(self) -> I8Vector4: ...
    @property
    def lzxl(self) -> I8Vector4: ...
    @property
    def lzyx(self) -> I8Vector4: ...
    @property
    def lzyy(self) -> I8Vector4: ...
    @property
    def lzyz(self) -> I8Vector4: ...
    @property
    def lzyo(self) -> I8Vector4: ...
    @property
    def lzyl(self) -> I8Vector4: ...
    @property
    def lzzx(self) -> I8Vector4: ...
    @property
    def lzzy(self) -> I8Vector4: ...
    @property
    def lzzz(self) -> I8Vector4: ...
    @property
    def lzzo(self) -> I8Vector4: ...
    @property
    def lzzl(self) -> I8Vector4: ...
    @property
    def lzox(self) -> I8Vector4: ...
    @property
    def lzoy(self) -> I8Vector4: ...
    @property
    def lzoz(self) -> I8Vector4: ...
    @property
    def lzoo(self) -> I8Vector4: ...
    @property
    def lzol(self) -> I8Vector4: ...
    @property
    def lzlx(self) -> I8Vector4: ...
    @property
    def lzly(self) -> I8Vector4: ...
    @property
    def lzlz(self) -> I8Vector4: ...
    @property
    def lzlo(self) -> I8Vector4: ...
    @property
    def lzll(self) -> I8Vector4: ...
    @property
    def loxx(self) -> I8Vector4: ...
    @property
    def loxy(self) -> I8Vector4: ...
    @property
    def loxz(self) -> I8Vector4: ...
    @property
    def loxo(self) -> I8Vector4: ...
    @property
    def loxl(self) -> I8Vector4: ...
    @property
    def loyx(self) -> I8Vector4: ...
    @property
    def loyy(self) -> I8Vector4: ...
    @property
    def loyz(self) -> I8Vector4: ...
    @property
    def loyo(self) -> I8Vector4: ...
    @property
    def loyl(self) -> I8Vector4: ...
    @property
    def lozx(self) -> I8Vector4: ...
    @property
    def lozy(self) -> I8Vector4: ...
    @property
    def lozz(self) -> I8Vector4: ...
    @property
    def lozo(self) -> I8Vector4: ...
    @property
    def lozl(self) -> I8Vector4: ...
    @property
    def loox(self) -> I8Vector4: ...
    @property
    def looy(self) -> I8Vector4: ...
    @property
    def looz(self) -> I8Vector4: ...
    @property
    def lolx(self) -> I8Vector4: ...
    @property
    def loly(self) -> I8Vector4: ...
    @property
    def lolz(self) -> I8Vector4: ...
    @property
    def llxx(self) -> I8Vector4: ...
    @property
    def llxy(self) -> I8Vector4: ...
    @property
    def llxz(self) -> I8Vector4: ...
    @property
    def llxo(self) -> I8Vector4: ...
    @property
    def llxl(self) -> I8Vector4: ...
    @property
    def llyx(self) -> I8Vector4: ...
    @property
    def llyy(self) -> I8Vector4: ...
    @property
    def llyz(self) -> I8Vector4: ...
    @property
    def llyo(self) -> I8Vector4: ...
    @property
    def llyl(self) -> I8Vector4: ...
    @property
    def llzx(self) -> I8Vector4: ...
    @property
    def llzy(self) -> I8Vector4: ...
    @property
    def llzz(self) -> I8Vector4: ...
    @property
    def llzo(self) -> I8Vector4: ...
    @property
    def llzl(self) -> I8Vector4: ...
    @property
    def llox(self) -> I8Vector4: ...
    @property
    def lloy(self) -> I8Vector4: ...
    @property
    def lloz(self) -> I8Vector4: ...
    @property
    def lllx(self) -> I8Vector4: ...
    @property
    def llly(self) -> I8Vector4: ...
    @property
    def lllz(self) -> I8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> I8Vector2: ...
    @property
    def rg(self) -> I8Vector2: ...
    @property
    def rb(self) -> I8Vector2: ...
    @property
    def ro(self) -> I8Vector2: ...
    @property
    def rl(self) -> I8Vector2: ...
    @property
    def gr(self) -> I8Vector2: ...
    @property
    def gg(self) -> I8Vector2: ...
    @property
    def gb(self) -> I8Vector2: ...
    @property
    def go(self) -> I8Vector2: ...
    @property
    def gl(self) -> I8Vector2: ...
    @property
    def br(self) -> I8Vector2: ...
    @property
    def bg(self) -> I8Vector2: ...
    @property
    def bb(self) -> I8Vector2: ...
    @property
    def bo(self) -> I8Vector2: ...
    @property
    def bl(self) -> I8Vector2: ...
    @property
    def og(self) -> I8Vector2: ...
    @property
    def ob(self) -> I8Vector2: ...
    @property
    def lr(self) -> I8Vector2: ...
    @property
    def lg(self) -> I8Vector2: ...
    @property
    def lb(self) -> I8Vector2: ...
    @property
    def rrr(self) -> I8Vector3: ...
    @property
    def rrg(self) -> I8Vector3: ...
    @property
    def rrb(self) -> I8Vector3: ...
    @property
    def rro(self) -> I8Vector3: ...
    @property
    def rrl(self) -> I8Vector3: ...
    @property
    def rgr(self) -> I8Vector3: ...
    @property
    def rgg(self) -> I8Vector3: ...
    @property
    def rgb(self) -> I8Vector3: ...
    @property
    def rgo(self) -> I8Vector3: ...
    @property
    def rgl(self) -> I8Vector3: ...
    @property
    def rbr(self) -> I8Vector3: ...
    @property
    def rbg(self) -> I8Vector3: ...
    @property
    def rbb(self) -> I8Vector3: ...
    @property
    def rbo(self) -> I8Vector3: ...
    @property
    def rbl(self) -> I8Vector3: ...
    @property
    def ror(self) -> I8Vector3: ...
    @property
    def rog(self) -> I8Vector3: ...
    @property
    def rob(self) -> I8Vector3: ...
    @property
    def roo(self) -> I8Vector3: ...
    @property
    def rol(self) -> I8Vector3: ...
    @property
    def rlr(self) -> I8Vector3: ...
    @property
    def rlg(self) -> I8Vector3: ...
    @property
    def rlb(self) -> I8Vector3: ...
    @property
    def rlo(self) -> I8Vector3: ...
    @property
    def rll(self) -> I8Vector3: ...
    @property
    def grr(self) -> I8Vector3: ...
    @property
    def grg(self) -> I8Vector3: ...
    @property
    def grb(self) -> I8Vector3: ...
    @property
    def gro(self) -> I8Vector3: ...
    @property
    def grl(self) -> I8Vector3: ...
    @property
    def ggr(self) -> I8Vector3: ...
    @property
    def ggg(self) -> I8Vector3: ...
    @property
    def ggb(self) -> I8Vector3: ...
    @property
    def ggo(self) -> I8Vector3: ...
    @property
    def ggl(self) -> I8Vector3: ...
    @property
    def gbr(self) -> I8Vector3: ...
    @property
    def gbg(self) -> I8Vector3: ...
    @property
    def gbb(self) -> I8Vector3: ...
    @property
    def gbo(self) -> I8Vector3: ...
    @property
    def gbl(self) -> I8Vector3: ...
    @property
    def gor(self) -> I8Vector3: ...
    @property
    def gog(self) -> I8Vector3: ...
    @property
    def gob(self) -> I8Vector3: ...
    @property
    def goo(self) -> I8Vector3: ...
    @property
    def gol(self) -> I8Vector3: ...
    @property
    def glr(self) -> I8Vector3: ...
    @property
    def glg(self) -> I8Vector3: ...
    @property
    def glb(self) -> I8Vector3: ...
    @property
    def glo(self) -> I8Vector3: ...
    @property
    def gll(self) -> I8Vector3: ...
    @property
    def brr(self) -> I8Vector3: ...
    @property
    def brg(self) -> I8Vector3: ...
    @property
    def brb(self) -> I8Vector3: ...
    @property
    def bro(self) -> I8Vector3: ...
    @property
    def brl(self) -> I8Vector3: ...
    @property
    def bgr(self) -> I8Vector3: ...
    @property
    def bgg(self) -> I8Vector3: ...
    @property
    def bgb(self) -> I8Vector3: ...
    @property
    def bgo(self) -> I8Vector3: ...
    @property
    def bgl(self) -> I8Vector3: ...
    @property
    def bbr(self) -> I8Vector3: ...
    @property
    def bbg(self) -> I8Vector3: ...
    @property
    def bbb(self) -> I8Vector3: ...
    @property
    def bbo(self) -> I8Vector3: ...
    @property
    def bbl(self) -> I8Vector3: ...
    @property
    def bor(self) -> I8Vector3: ...
    @property
    def bog(self) -> I8Vector3: ...
    @property
    def bob(self) -> I8Vector3: ...
    @property
    def boo(self) -> I8Vector3: ...
    @property
    def bol(self) -> I8Vector3: ...
    @property
    def blr(self) -> I8Vector3: ...
    @property
    def blg(self) -> I8Vector3: ...
    @property
    def blb(self) -> I8Vector3: ...
    @property
    def blo(self) -> I8Vector3: ...
    @property
    def bll(self) -> I8Vector3: ...
    @property
    def orr(self) -> I8Vector3: ...
    @property
    def org(self) -> I8Vector3: ...
    @property
    def orb(self) -> I8Vector3: ...
    @property
    def oro(self) -> I8Vector3: ...
    @property
    def orl(self) -> I8Vector3: ...
    @property
    def ogr(self) -> I8Vector3: ...
    @property
    def ogg(self) -> I8Vector3: ...
    @property
    def ogb(self) -> I8Vector3: ...
    @property
    def ogo(self) -> I8Vector3: ...
    @property
    def ogl(self) -> I8Vector3: ...
    @property
    def obr(self) -> I8Vector3: ...
    @property
    def obg(self) -> I8Vector3: ...
    @property
    def obb(self) -> I8Vector3: ...
    @property
    def obo(self) -> I8Vector3: ...
    @property
    def obl(self) -> I8Vector3: ...
    @property
    def oor(self) -> I8Vector3: ...
    @property
    def oog(self) -> I8Vector3: ...
    @property
    def oob(self) -> I8Vector3: ...
    @property
    def olr(self) -> I8Vector3: ...
    @property
    def olg(self) -> I8Vector3: ...
    @property
    def olb(self) -> I8Vector3: ...
    @property
    def lrr(self) -> I8Vector3: ...
    @property
    def lrg(self) -> I8Vector3: ...
    @property
    def lrb(self) -> I8Vector3: ...
    @property
    def lro(self) -> I8Vector3: ...
    @property
    def lrl(self) -> I8Vector3: ...
    @property
    def lgr(self) -> I8Vector3: ...
    @property
    def lgg(self) -> I8Vector3: ...
    @property
    def lgb(self) -> I8Vector3: ...
    @property
    def lgo(self) -> I8Vector3: ...
    @property
    def lgl(self) -> I8Vector3: ...
    @property
    def lbr(self) -> I8Vector3: ...
    @property
    def lbg(self) -> I8Vector3: ...
    @property
    def lbb(self) -> I8Vector3: ...
    @property
    def lbo(self) -> I8Vector3: ...
    @property
    def lbl(self) -> I8Vector3: ...
    @property
    def lor(self) -> I8Vector3: ...
    @property
    def log(self) -> I8Vector3: ...
    @property
    def lob(self) -> I8Vector3: ...
    @property
    def llr(self) -> I8Vector3: ...
    @property
    def llg(self) -> I8Vector3: ...
    @property
    def llb(self) -> I8Vector3: ...
    @property
    def rrrr(self) -> I8Vector4: ...
    @property
    def rrrg(self) -> I8Vector4: ...
    @property
    def rrrb(self) -> I8Vector4: ...
    @property
    def rrro(self) -> I8Vector4: ...
    @property
    def rrrl(self) -> I8Vector4: ...
    @property
    def rrgr(self) -> I8Vector4: ...
    @property
    def rrgg(self) -> I8Vector4: ...
    @property
    def rrgb(self) -> I8Vector4: ...
    @property
    def rrgo(self) -> I8Vector4: ...
    @property
    def rrgl(self) -> I8Vector4: ...
    @property
    def rrbr(self) -> I8Vector4: ...
    @property
    def rrbg(self) -> I8Vector4: ...
    @property
    def rrbb(self) -> I8Vector4: ...
    @property
    def rrbo(self) -> I8Vector4: ...
    @property
    def rrbl(self) -> I8Vector4: ...
    @property
    def rror(self) -> I8Vector4: ...
    @property
    def rrog(self) -> I8Vector4: ...
    @property
    def rrob(self) -> I8Vector4: ...
    @property
    def rroo(self) -> I8Vector4: ...
    @property
    def rrol(self) -> I8Vector4: ...
    @property
    def rrlr(self) -> I8Vector4: ...
    @property
    def rrlg(self) -> I8Vector4: ...
    @property
    def rrlb(self) -> I8Vector4: ...
    @property
    def rrlo(self) -> I8Vector4: ...
    @property
    def rrll(self) -> I8Vector4: ...
    @property
    def rgrr(self) -> I8Vector4: ...
    @property
    def rgrg(self) -> I8Vector4: ...
    @property
    def rgrb(self) -> I8Vector4: ...
    @property
    def rgro(self) -> I8Vector4: ...
    @property
    def rgrl(self) -> I8Vector4: ...
    @property
    def rggr(self) -> I8Vector4: ...
    @property
    def rggg(self) -> I8Vector4: ...
    @property
    def rggb(self) -> I8Vector4: ...
    @property
    def rggo(self) -> I8Vector4: ...
    @property
    def rggl(self) -> I8Vector4: ...
    @property
    def rgbr(self) -> I8Vector4: ...
    @property
    def rgbg(self) -> I8Vector4: ...
    @property
    def rgbb(self) -> I8Vector4: ...
    @property
    def rgbo(self) -> I8Vector4: ...
    @property
    def rgbl(self) -> I8Vector4: ...
    @property
    def rgor(self) -> I8Vector4: ...
    @property
    def rgog(self) -> I8Vector4: ...
    @property
    def rgob(self) -> I8Vector4: ...
    @property
    def rgoo(self) -> I8Vector4: ...
    @property
    def rgol(self) -> I8Vector4: ...
    @property
    def rglr(self) -> I8Vector4: ...
    @property
    def rglg(self) -> I8Vector4: ...
    @property
    def rglb(self) -> I8Vector4: ...
    @property
    def rglo(self) -> I8Vector4: ...
    @property
    def rgll(self) -> I8Vector4: ...
    @property
    def rbrr(self) -> I8Vector4: ...
    @property
    def rbrg(self) -> I8Vector4: ...
    @property
    def rbrb(self) -> I8Vector4: ...
    @property
    def rbro(self) -> I8Vector4: ...
    @property
    def rbrl(self) -> I8Vector4: ...
    @property
    def rbgr(self) -> I8Vector4: ...
    @property
    def rbgg(self) -> I8Vector4: ...
    @property
    def rbgb(self) -> I8Vector4: ...
    @property
    def rbgo(self) -> I8Vector4: ...
    @property
    def rbgl(self) -> I8Vector4: ...
    @property
    def rbbr(self) -> I8Vector4: ...
    @property
    def rbbg(self) -> I8Vector4: ...
    @property
    def rbbb(self) -> I8Vector4: ...
    @property
    def rbbo(self) -> I8Vector4: ...
    @property
    def rbbl(self) -> I8Vector4: ...
    @property
    def rbor(self) -> I8Vector4: ...
    @property
    def rbog(self) -> I8Vector4: ...
    @property
    def rbob(self) -> I8Vector4: ...
    @property
    def rboo(self) -> I8Vector4: ...
    @property
    def rbol(self) -> I8Vector4: ...
    @property
    def rblr(self) -> I8Vector4: ...
    @property
    def rblg(self) -> I8Vector4: ...
    @property
    def rblb(self) -> I8Vector4: ...
    @property
    def rblo(self) -> I8Vector4: ...
    @property
    def rbll(self) -> I8Vector4: ...
    @property
    def rorr(self) -> I8Vector4: ...
    @property
    def rorg(self) -> I8Vector4: ...
    @property
    def rorb(self) -> I8Vector4: ...
    @property
    def roro(self) -> I8Vector4: ...
    @property
    def rorl(self) -> I8Vector4: ...
    @property
    def rogr(self) -> I8Vector4: ...
    @property
    def rogg(self) -> I8Vector4: ...
    @property
    def rogb(self) -> I8Vector4: ...
    @property
    def rogo(self) -> I8Vector4: ...
    @property
    def rogl(self) -> I8Vector4: ...
    @property
    def robr(self) -> I8Vector4: ...
    @property
    def robg(self) -> I8Vector4: ...
    @property
    def robb(self) -> I8Vector4: ...
    @property
    def robo(self) -> I8Vector4: ...
    @property
    def robl(self) -> I8Vector4: ...
    @property
    def roor(self) -> I8Vector4: ...
    @property
    def roog(self) -> I8Vector4: ...
    @property
    def roob(self) -> I8Vector4: ...
    @property
    def rooo(self) -> I8Vector4: ...
    @property
    def rool(self) -> I8Vector4: ...
    @property
    def rolr(self) -> I8Vector4: ...
    @property
    def rolg(self) -> I8Vector4: ...
    @property
    def rolb(self) -> I8Vector4: ...
    @property
    def rolo(self) -> I8Vector4: ...
    @property
    def roll(self) -> I8Vector4: ...
    @property
    def rlrr(self) -> I8Vector4: ...
    @property
    def rlrg(self) -> I8Vector4: ...
    @property
    def rlrb(self) -> I8Vector4: ...
    @property
    def rlro(self) -> I8Vector4: ...
    @property
    def rlrl(self) -> I8Vector4: ...
    @property
    def rlgr(self) -> I8Vector4: ...
    @property
    def rlgg(self) -> I8Vector4: ...
    @property
    def rlgb(self) -> I8Vector4: ...
    @property
    def rlgo(self) -> I8Vector4: ...
    @property
    def rlgl(self) -> I8Vector4: ...
    @property
    def rlbr(self) -> I8Vector4: ...
    @property
    def rlbg(self) -> I8Vector4: ...
    @property
    def rlbb(self) -> I8Vector4: ...
    @property
    def rlbo(self) -> I8Vector4: ...
    @property
    def rlbl(self) -> I8Vector4: ...
    @property
    def rlor(self) -> I8Vector4: ...
    @property
    def rlog(self) -> I8Vector4: ...
    @property
    def rlob(self) -> I8Vector4: ...
    @property
    def rloo(self) -> I8Vector4: ...
    @property
    def rlol(self) -> I8Vector4: ...
    @property
    def rllr(self) -> I8Vector4: ...
    @property
    def rllg(self) -> I8Vector4: ...
    @property
    def rllb(self) -> I8Vector4: ...
    @property
    def rllo(self) -> I8Vector4: ...
    @property
    def rlll(self) -> I8Vector4: ...
    @property
    def grrr(self) -> I8Vector4: ...
    @property
    def grrg(self) -> I8Vector4: ...
    @property
    def grrb(self) -> I8Vector4: ...
    @property
    def grro(self) -> I8Vector4: ...
    @property
    def grrl(self) -> I8Vector4: ...
    @property
    def grgr(self) -> I8Vector4: ...
    @property
    def grgg(self) -> I8Vector4: ...
    @property
    def grgb(self) -> I8Vector4: ...
    @property
    def grgo(self) -> I8Vector4: ...
    @property
    def grgl(self) -> I8Vector4: ...
    @property
    def grbr(self) -> I8Vector4: ...
    @property
    def grbg(self) -> I8Vector4: ...
    @property
    def grbb(self) -> I8Vector4: ...
    @property
    def grbo(self) -> I8Vector4: ...
    @property
    def grbl(self) -> I8Vector4: ...
    @property
    def gror(self) -> I8Vector4: ...
    @property
    def grog(self) -> I8Vector4: ...
    @property
    def grob(self) -> I8Vector4: ...
    @property
    def groo(self) -> I8Vector4: ...
    @property
    def grol(self) -> I8Vector4: ...
    @property
    def grlr(self) -> I8Vector4: ...
    @property
    def grlg(self) -> I8Vector4: ...
    @property
    def grlb(self) -> I8Vector4: ...
    @property
    def grlo(self) -> I8Vector4: ...
    @property
    def grll(self) -> I8Vector4: ...
    @property
    def ggrr(self) -> I8Vector4: ...
    @property
    def ggrg(self) -> I8Vector4: ...
    @property
    def ggrb(self) -> I8Vector4: ...
    @property
    def ggro(self) -> I8Vector4: ...
    @property
    def ggrl(self) -> I8Vector4: ...
    @property
    def gggr(self) -> I8Vector4: ...
    @property
    def gggg(self) -> I8Vector4: ...
    @property
    def gggb(self) -> I8Vector4: ...
    @property
    def gggo(self) -> I8Vector4: ...
    @property
    def gggl(self) -> I8Vector4: ...
    @property
    def ggbr(self) -> I8Vector4: ...
    @property
    def ggbg(self) -> I8Vector4: ...
    @property
    def ggbb(self) -> I8Vector4: ...
    @property
    def ggbo(self) -> I8Vector4: ...
    @property
    def ggbl(self) -> I8Vector4: ...
    @property
    def ggor(self) -> I8Vector4: ...
    @property
    def ggog(self) -> I8Vector4: ...
    @property
    def ggob(self) -> I8Vector4: ...
    @property
    def ggoo(self) -> I8Vector4: ...
    @property
    def ggol(self) -> I8Vector4: ...
    @property
    def gglr(self) -> I8Vector4: ...
    @property
    def gglg(self) -> I8Vector4: ...
    @property
    def gglb(self) -> I8Vector4: ...
    @property
    def gglo(self) -> I8Vector4: ...
    @property
    def ggll(self) -> I8Vector4: ...
    @property
    def gbrr(self) -> I8Vector4: ...
    @property
    def gbrg(self) -> I8Vector4: ...
    @property
    def gbrb(self) -> I8Vector4: ...
    @property
    def gbro(self) -> I8Vector4: ...
    @property
    def gbrl(self) -> I8Vector4: ...
    @property
    def gbgr(self) -> I8Vector4: ...
    @property
    def gbgg(self) -> I8Vector4: ...
    @property
    def gbgb(self) -> I8Vector4: ...
    @property
    def gbgo(self) -> I8Vector4: ...
    @property
    def gbgl(self) -> I8Vector4: ...
    @property
    def gbbr(self) -> I8Vector4: ...
    @property
    def gbbg(self) -> I8Vector4: ...
    @property
    def gbbb(self) -> I8Vector4: ...
    @property
    def gbbo(self) -> I8Vector4: ...
    @property
    def gbbl(self) -> I8Vector4: ...
    @property
    def gbor(self) -> I8Vector4: ...
    @property
    def gbog(self) -> I8Vector4: ...
    @property
    def gbob(self) -> I8Vector4: ...
    @property
    def gboo(self) -> I8Vector4: ...
    @property
    def gbol(self) -> I8Vector4: ...
    @property
    def gblr(self) -> I8Vector4: ...
    @property
    def gblg(self) -> I8Vector4: ...
    @property
    def gblb(self) -> I8Vector4: ...
    @property
    def gblo(self) -> I8Vector4: ...
    @property
    def gbll(self) -> I8Vector4: ...
    @property
    def gorr(self) -> I8Vector4: ...
    @property
    def gorg(self) -> I8Vector4: ...
    @property
    def gorb(self) -> I8Vector4: ...
    @property
    def goro(self) -> I8Vector4: ...
    @property
    def gorl(self) -> I8Vector4: ...
    @property
    def gogr(self) -> I8Vector4: ...
    @property
    def gogg(self) -> I8Vector4: ...
    @property
    def gogb(self) -> I8Vector4: ...
    @property
    def gogo(self) -> I8Vector4: ...
    @property
    def gogl(self) -> I8Vector4: ...
    @property
    def gobr(self) -> I8Vector4: ...
    @property
    def gobg(self) -> I8Vector4: ...
    @property
    def gobb(self) -> I8Vector4: ...
    @property
    def gobo(self) -> I8Vector4: ...
    @property
    def gobl(self) -> I8Vector4: ...
    @property
    def goor(self) -> I8Vector4: ...
    @property
    def goog(self) -> I8Vector4: ...
    @property
    def goob(self) -> I8Vector4: ...
    @property
    def gooo(self) -> I8Vector4: ...
    @property
    def gool(self) -> I8Vector4: ...
    @property
    def golr(self) -> I8Vector4: ...
    @property
    def golg(self) -> I8Vector4: ...
    @property
    def golb(self) -> I8Vector4: ...
    @property
    def golo(self) -> I8Vector4: ...
    @property
    def goll(self) -> I8Vector4: ...
    @property
    def glrr(self) -> I8Vector4: ...
    @property
    def glrg(self) -> I8Vector4: ...
    @property
    def glrb(self) -> I8Vector4: ...
    @property
    def glro(self) -> I8Vector4: ...
    @property
    def glrl(self) -> I8Vector4: ...
    @property
    def glgr(self) -> I8Vector4: ...
    @property
    def glgg(self) -> I8Vector4: ...
    @property
    def glgb(self) -> I8Vector4: ...
    @property
    def glgo(self) -> I8Vector4: ...
    @property
    def glgl(self) -> I8Vector4: ...
    @property
    def glbr(self) -> I8Vector4: ...
    @property
    def glbg(self) -> I8Vector4: ...
    @property
    def glbb(self) -> I8Vector4: ...
    @property
    def glbo(self) -> I8Vector4: ...
    @property
    def glbl(self) -> I8Vector4: ...
    @property
    def glor(self) -> I8Vector4: ...
    @property
    def glog(self) -> I8Vector4: ...
    @property
    def glob(self) -> I8Vector4: ...
    @property
    def gloo(self) -> I8Vector4: ...
    @property
    def glol(self) -> I8Vector4: ...
    @property
    def gllr(self) -> I8Vector4: ...
    @property
    def gllg(self) -> I8Vector4: ...
    @property
    def gllb(self) -> I8Vector4: ...
    @property
    def gllo(self) -> I8Vector4: ...
    @property
    def glll(self) -> I8Vector4: ...
    @property
    def brrr(self) -> I8Vector4: ...
    @property
    def brrg(self) -> I8Vector4: ...
    @property
    def brrb(self) -> I8Vector4: ...
    @property
    def brro(self) -> I8Vector4: ...
    @property
    def brrl(self) -> I8Vector4: ...
    @property
    def brgr(self) -> I8Vector4: ...
    @property
    def brgg(self) -> I8Vector4: ...
    @property
    def brgb(self) -> I8Vector4: ...
    @property
    def brgo(self) -> I8Vector4: ...
    @property
    def brgl(self) -> I8Vector4: ...
    @property
    def brbr(self) -> I8Vector4: ...
    @property
    def brbg(self) -> I8Vector4: ...
    @property
    def brbb(self) -> I8Vector4: ...
    @property
    def brbo(self) -> I8Vector4: ...
    @property
    def brbl(self) -> I8Vector4: ...
    @property
    def bror(self) -> I8Vector4: ...
    @property
    def brog(self) -> I8Vector4: ...
    @property
    def brob(self) -> I8Vector4: ...
    @property
    def broo(self) -> I8Vector4: ...
    @property
    def brol(self) -> I8Vector4: ...
    @property
    def brlr(self) -> I8Vector4: ...
    @property
    def brlg(self) -> I8Vector4: ...
    @property
    def brlb(self) -> I8Vector4: ...
    @property
    def brlo(self) -> I8Vector4: ...
    @property
    def brll(self) -> I8Vector4: ...
    @property
    def bgrr(self) -> I8Vector4: ...
    @property
    def bgrg(self) -> I8Vector4: ...
    @property
    def bgrb(self) -> I8Vector4: ...
    @property
    def bgro(self) -> I8Vector4: ...
    @property
    def bgrl(self) -> I8Vector4: ...
    @property
    def bggr(self) -> I8Vector4: ...
    @property
    def bggg(self) -> I8Vector4: ...
    @property
    def bggb(self) -> I8Vector4: ...
    @property
    def bggo(self) -> I8Vector4: ...
    @property
    def bggl(self) -> I8Vector4: ...
    @property
    def bgbr(self) -> I8Vector4: ...
    @property
    def bgbg(self) -> I8Vector4: ...
    @property
    def bgbb(self) -> I8Vector4: ...
    @property
    def bgbo(self) -> I8Vector4: ...
    @property
    def bgbl(self) -> I8Vector4: ...
    @property
    def bgor(self) -> I8Vector4: ...
    @property
    def bgog(self) -> I8Vector4: ...
    @property
    def bgob(self) -> I8Vector4: ...
    @property
    def bgoo(self) -> I8Vector4: ...
    @property
    def bgol(self) -> I8Vector4: ...
    @property
    def bglr(self) -> I8Vector4: ...
    @property
    def bglg(self) -> I8Vector4: ...
    @property
    def bglb(self) -> I8Vector4: ...
    @property
    def bglo(self) -> I8Vector4: ...
    @property
    def bgll(self) -> I8Vector4: ...
    @property
    def bbrr(self) -> I8Vector4: ...
    @property
    def bbrg(self) -> I8Vector4: ...
    @property
    def bbrb(self) -> I8Vector4: ...
    @property
    def bbro(self) -> I8Vector4: ...
    @property
    def bbrl(self) -> I8Vector4: ...
    @property
    def bbgr(self) -> I8Vector4: ...
    @property
    def bbgg(self) -> I8Vector4: ...
    @property
    def bbgb(self) -> I8Vector4: ...
    @property
    def bbgo(self) -> I8Vector4: ...
    @property
    def bbgl(self) -> I8Vector4: ...
    @property
    def bbbr(self) -> I8Vector4: ...
    @property
    def bbbg(self) -> I8Vector4: ...
    @property
    def bbbb(self) -> I8Vector4: ...
    @property
    def bbbo(self) -> I8Vector4: ...
    @property
    def bbbl(self) -> I8Vector4: ...
    @property
    def bbor(self) -> I8Vector4: ...
    @property
    def bbog(self) -> I8Vector4: ...
    @property
    def bbob(self) -> I8Vector4: ...
    @property
    def bboo(self) -> I8Vector4: ...
    @property
    def bbol(self) -> I8Vector4: ...
    @property
    def bblr(self) -> I8Vector4: ...
    @property
    def bblg(self) -> I8Vector4: ...
    @property
    def bblb(self) -> I8Vector4: ...
    @property
    def bblo(self) -> I8Vector4: ...
    @property
    def bbll(self) -> I8Vector4: ...
    @property
    def borr(self) -> I8Vector4: ...
    @property
    def borg(self) -> I8Vector4: ...
    @property
    def borb(self) -> I8Vector4: ...
    @property
    def boro(self) -> I8Vector4: ...
    @property
    def borl(self) -> I8Vector4: ...
    @property
    def bogr(self) -> I8Vector4: ...
    @property
    def bogg(self) -> I8Vector4: ...
    @property
    def bogb(self) -> I8Vector4: ...
    @property
    def bogo(self) -> I8Vector4: ...
    @property
    def bogl(self) -> I8Vector4: ...
    @property
    def bobr(self) -> I8Vector4: ...
    @property
    def bobg(self) -> I8Vector4: ...
    @property
    def bobb(self) -> I8Vector4: ...
    @property
    def bobo(self) -> I8Vector4: ...
    @property
    def bobl(self) -> I8Vector4: ...
    @property
    def boor(self) -> I8Vector4: ...
    @property
    def boog(self) -> I8Vector4: ...
    @property
    def boob(self) -> I8Vector4: ...
    @property
    def booo(self) -> I8Vector4: ...
    @property
    def bool(self) -> I8Vector4: ...
    @property
    def bolr(self) -> I8Vector4: ...
    @property
    def bolg(self) -> I8Vector4: ...
    @property
    def bolb(self) -> I8Vector4: ...
    @property
    def bolo(self) -> I8Vector4: ...
    @property
    def boll(self) -> I8Vector4: ...
    @property
    def blrr(self) -> I8Vector4: ...
    @property
    def blrg(self) -> I8Vector4: ...
    @property
    def blrb(self) -> I8Vector4: ...
    @property
    def blro(self) -> I8Vector4: ...
    @property
    def blrl(self) -> I8Vector4: ...
    @property
    def blgr(self) -> I8Vector4: ...
    @property
    def blgg(self) -> I8Vector4: ...
    @property
    def blgb(self) -> I8Vector4: ...
    @property
    def blgo(self) -> I8Vector4: ...
    @property
    def blgl(self) -> I8Vector4: ...
    @property
    def blbr(self) -> I8Vector4: ...
    @property
    def blbg(self) -> I8Vector4: ...
    @property
    def blbb(self) -> I8Vector4: ...
    @property
    def blbo(self) -> I8Vector4: ...
    @property
    def blbl(self) -> I8Vector4: ...
    @property
    def blor(self) -> I8Vector4: ...
    @property
    def blog(self) -> I8Vector4: ...
    @property
    def blob(self) -> I8Vector4: ...
    @property
    def bloo(self) -> I8Vector4: ...
    @property
    def blol(self) -> I8Vector4: ...
    @property
    def bllr(self) -> I8Vector4: ...
    @property
    def bllg(self) -> I8Vector4: ...
    @property
    def bllb(self) -> I8Vector4: ...
    @property
    def bllo(self) -> I8Vector4: ...
    @property
    def blll(self) -> I8Vector4: ...
    @property
    def orrr(self) -> I8Vector4: ...
    @property
    def orrg(self) -> I8Vector4: ...
    @property
    def orrb(self) -> I8Vector4: ...
    @property
    def orro(self) -> I8Vector4: ...
    @property
    def orrl(self) -> I8Vector4: ...
    @property
    def orgr(self) -> I8Vector4: ...
    @property
    def orgg(self) -> I8Vector4: ...
    @property
    def orgb(self) -> I8Vector4: ...
    @property
    def orgo(self) -> I8Vector4: ...
    @property
    def orgl(self) -> I8Vector4: ...
    @property
    def orbr(self) -> I8Vector4: ...
    @property
    def orbg(self) -> I8Vector4: ...
    @property
    def orbb(self) -> I8Vector4: ...
    @property
    def orbo(self) -> I8Vector4: ...
    @property
    def orbl(self) -> I8Vector4: ...
    @property
    def oror(self) -> I8Vector4: ...
    @property
    def orog(self) -> I8Vector4: ...
    @property
    def orob(self) -> I8Vector4: ...
    @property
    def oroo(self) -> I8Vector4: ...
    @property
    def orol(self) -> I8Vector4: ...
    @property
    def orlr(self) -> I8Vector4: ...
    @property
    def orlg(self) -> I8Vector4: ...
    @property
    def orlb(self) -> I8Vector4: ...
    @property
    def orlo(self) -> I8Vector4: ...
    @property
    def orll(self) -> I8Vector4: ...
    @property
    def ogrr(self) -> I8Vector4: ...
    @property
    def ogrg(self) -> I8Vector4: ...
    @property
    def ogrb(self) -> I8Vector4: ...
    @property
    def ogro(self) -> I8Vector4: ...
    @property
    def ogrl(self) -> I8Vector4: ...
    @property
    def oggr(self) -> I8Vector4: ...
    @property
    def oggg(self) -> I8Vector4: ...
    @property
    def oggb(self) -> I8Vector4: ...
    @property
    def oggo(self) -> I8Vector4: ...
    @property
    def oggl(self) -> I8Vector4: ...
    @property
    def ogbr(self) -> I8Vector4: ...
    @property
    def ogbg(self) -> I8Vector4: ...
    @property
    def ogbb(self) -> I8Vector4: ...
    @property
    def ogbo(self) -> I8Vector4: ...
    @property
    def ogbl(self) -> I8Vector4: ...
    @property
    def ogor(self) -> I8Vector4: ...
    @property
    def ogog(self) -> I8Vector4: ...
    @property
    def ogob(self) -> I8Vector4: ...
    @property
    def ogoo(self) -> I8Vector4: ...
    @property
    def ogol(self) -> I8Vector4: ...
    @property
    def oglr(self) -> I8Vector4: ...
    @property
    def oglg(self) -> I8Vector4: ...
    @property
    def oglb(self) -> I8Vector4: ...
    @property
    def oglo(self) -> I8Vector4: ...
    @property
    def ogll(self) -> I8Vector4: ...
    @property
    def obrr(self) -> I8Vector4: ...
    @property
    def obrg(self) -> I8Vector4: ...
    @property
    def obrb(self) -> I8Vector4: ...
    @property
    def obro(self) -> I8Vector4: ...
    @property
    def obrl(self) -> I8Vector4: ...
    @property
    def obgr(self) -> I8Vector4: ...
    @property
    def obgg(self) -> I8Vector4: ...
    @property
    def obgb(self) -> I8Vector4: ...
    @property
    def obgo(self) -> I8Vector4: ...
    @property
    def obgl(self) -> I8Vector4: ...
    @property
    def obbr(self) -> I8Vector4: ...
    @property
    def obbg(self) -> I8Vector4: ...
    @property
    def obbb(self) -> I8Vector4: ...
    @property
    def obbo(self) -> I8Vector4: ...
    @property
    def obbl(self) -> I8Vector4: ...
    @property
    def obor(self) -> I8Vector4: ...
    @property
    def obog(self) -> I8Vector4: ...
    @property
    def obob(self) -> I8Vector4: ...
    @property
    def oboo(self) -> I8Vector4: ...
    @property
    def obol(self) -> I8Vector4: ...
    @property
    def oblr(self) -> I8Vector4: ...
    @property
    def oblg(self) -> I8Vector4: ...
    @property
    def oblb(self) -> I8Vector4: ...
    @property
    def oblo(self) -> I8Vector4: ...
    @property
    def obll(self) -> I8Vector4: ...
    @property
    def oorr(self) -> I8Vector4: ...
    @property
    def oorg(self) -> I8Vector4: ...
    @property
    def oorb(self) -> I8Vector4: ...
    @property
    def ooro(self) -> I8Vector4: ...
    @property
    def oorl(self) -> I8Vector4: ...
    @property
    def oogr(self) -> I8Vector4: ...
    @property
    def oogg(self) -> I8Vector4: ...
    @property
    def oogb(self) -> I8Vector4: ...
    @property
    def oogo(self) -> I8Vector4: ...
    @property
    def oogl(self) -> I8Vector4: ...
    @property
    def oobr(self) -> I8Vector4: ...
    @property
    def oobg(self) -> I8Vector4: ...
    @property
    def oobb(self) -> I8Vector4: ...
    @property
    def oobo(self) -> I8Vector4: ...
    @property
    def oobl(self) -> I8Vector4: ...
    @property
    def ooor(self) -> I8Vector4: ...
    @property
    def ooog(self) -> I8Vector4: ...
    @property
    def ooob(self) -> I8Vector4: ...
    @property
    def oolr(self) -> I8Vector4: ...
    @property
    def oolg(self) -> I8Vector4: ...
    @property
    def oolb(self) -> I8Vector4: ...
    @property
    def olrr(self) -> I8Vector4: ...
    @property
    def olrg(self) -> I8Vector4: ...
    @property
    def olrb(self) -> I8Vector4: ...
    @property
    def olro(self) -> I8Vector4: ...
    @property
    def olrl(self) -> I8Vector4: ...
    @property
    def olgr(self) -> I8Vector4: ...
    @property
    def olgg(self) -> I8Vector4: ...
    @property
    def olgb(self) -> I8Vector4: ...
    @property
    def olgo(self) -> I8Vector4: ...
    @property
    def olgl(self) -> I8Vector4: ...
    @property
    def olbr(self) -> I8Vector4: ...
    @property
    def olbg(self) -> I8Vector4: ...
    @property
    def olbb(self) -> I8Vector4: ...
    @property
    def olbo(self) -> I8Vector4: ...
    @property
    def olbl(self) -> I8Vector4: ...
    @property
    def olor(self) -> I8Vector4: ...
    @property
    def olog(self) -> I8Vector4: ...
    @property
    def olob(self) -> I8Vector4: ...
    @property
    def ollr(self) -> I8Vector4: ...
    @property
    def ollg(self) -> I8Vector4: ...
    @property
    def ollb(self) -> I8Vector4: ...
    @property
    def lrrr(self) -> I8Vector4: ...
    @property
    def lrrg(self) -> I8Vector4: ...
    @property
    def lrrb(self) -> I8Vector4: ...
    @property
    def lrro(self) -> I8Vector4: ...
    @property
    def lrrl(self) -> I8Vector4: ...
    @property
    def lrgr(self) -> I8Vector4: ...
    @property
    def lrgg(self) -> I8Vector4: ...
    @property
    def lrgb(self) -> I8Vector4: ...
    @property
    def lrgo(self) -> I8Vector4: ...
    @property
    def lrgl(self) -> I8Vector4: ...
    @property
    def lrbr(self) -> I8Vector4: ...
    @property
    def lrbg(self) -> I8Vector4: ...
    @property
    def lrbb(self) -> I8Vector4: ...
    @property
    def lrbo(self) -> I8Vector4: ...
    @property
    def lrbl(self) -> I8Vector4: ...
    @property
    def lror(self) -> I8Vector4: ...
    @property
    def lrog(self) -> I8Vector4: ...
    @property
    def lrob(self) -> I8Vector4: ...
    @property
    def lroo(self) -> I8Vector4: ...
    @property
    def lrol(self) -> I8Vector4: ...
    @property
    def lrlr(self) -> I8Vector4: ...
    @property
    def lrlg(self) -> I8Vector4: ...
    @property
    def lrlb(self) -> I8Vector4: ...
    @property
    def lrlo(self) -> I8Vector4: ...
    @property
    def lrll(self) -> I8Vector4: ...
    @property
    def lgrr(self) -> I8Vector4: ...
    @property
    def lgrg(self) -> I8Vector4: ...
    @property
    def lgrb(self) -> I8Vector4: ...
    @property
    def lgro(self) -> I8Vector4: ...
    @property
    def lgrl(self) -> I8Vector4: ...
    @property
    def lggr(self) -> I8Vector4: ...
    @property
    def lggg(self) -> I8Vector4: ...
    @property
    def lggb(self) -> I8Vector4: ...
    @property
    def lggo(self) -> I8Vector4: ...
    @property
    def lggl(self) -> I8Vector4: ...
    @property
    def lgbr(self) -> I8Vector4: ...
    @property
    def lgbg(self) -> I8Vector4: ...
    @property
    def lgbb(self) -> I8Vector4: ...
    @property
    def lgbo(self) -> I8Vector4: ...
    @property
    def lgbl(self) -> I8Vector4: ...
    @property
    def lgor(self) -> I8Vector4: ...
    @property
    def lgog(self) -> I8Vector4: ...
    @property
    def lgob(self) -> I8Vector4: ...
    @property
    def lgoo(self) -> I8Vector4: ...
    @property
    def lgol(self) -> I8Vector4: ...
    @property
    def lglr(self) -> I8Vector4: ...
    @property
    def lglg(self) -> I8Vector4: ...
    @property
    def lglb(self) -> I8Vector4: ...
    @property
    def lglo(self) -> I8Vector4: ...
    @property
    def lgll(self) -> I8Vector4: ...
    @property
    def lbrr(self) -> I8Vector4: ...
    @property
    def lbrg(self) -> I8Vector4: ...
    @property
    def lbrb(self) -> I8Vector4: ...
    @property
    def lbro(self) -> I8Vector4: ...
    @property
    def lbrl(self) -> I8Vector4: ...
    @property
    def lbgr(self) -> I8Vector4: ...
    @property
    def lbgg(self) -> I8Vector4: ...
    @property
    def lbgb(self) -> I8Vector4: ...
    @property
    def lbgo(self) -> I8Vector4: ...
    @property
    def lbgl(self) -> I8Vector4: ...
    @property
    def lbbr(self) -> I8Vector4: ...
    @property
    def lbbg(self) -> I8Vector4: ...
    @property
    def lbbb(self) -> I8Vector4: ...
    @property
    def lbbo(self) -> I8Vector4: ...
    @property
    def lbbl(self) -> I8Vector4: ...
    @property
    def lbor(self) -> I8Vector4: ...
    @property
    def lbog(self) -> I8Vector4: ...
    @property
    def lbob(self) -> I8Vector4: ...
    @property
    def lboo(self) -> I8Vector4: ...
    @property
    def lbol(self) -> I8Vector4: ...
    @property
    def lblr(self) -> I8Vector4: ...
    @property
    def lblg(self) -> I8Vector4: ...
    @property
    def lblb(self) -> I8Vector4: ...
    @property
    def lblo(self) -> I8Vector4: ...
    @property
    def lbll(self) -> I8Vector4: ...
    @property
    def lorr(self) -> I8Vector4: ...
    @property
    def lorg(self) -> I8Vector4: ...
    @property
    def lorb(self) -> I8Vector4: ...
    @property
    def loro(self) -> I8Vector4: ...
    @property
    def lorl(self) -> I8Vector4: ...
    @property
    def logr(self) -> I8Vector4: ...
    @property
    def logg(self) -> I8Vector4: ...
    @property
    def logb(self) -> I8Vector4: ...
    @property
    def logo(self) -> I8Vector4: ...
    @property
    def logl(self) -> I8Vector4: ...
    @property
    def lobr(self) -> I8Vector4: ...
    @property
    def lobg(self) -> I8Vector4: ...
    @property
    def lobb(self) -> I8Vector4: ...
    @property
    def lobo(self) -> I8Vector4: ...
    @property
    def lobl(self) -> I8Vector4: ...
    @property
    def loor(self) -> I8Vector4: ...
    @property
    def loog(self) -> I8Vector4: ...
    @property
    def loob(self) -> I8Vector4: ...
    @property
    def lolr(self) -> I8Vector4: ...
    @property
    def lolg(self) -> I8Vector4: ...
    @property
    def lolb(self) -> I8Vector4: ...
    @property
    def llrr(self) -> I8Vector4: ...
    @property
    def llrg(self) -> I8Vector4: ...
    @property
    def llrb(self) -> I8Vector4: ...
    @property
    def llro(self) -> I8Vector4: ...
    @property
    def llrl(self) -> I8Vector4: ...
    @property
    def llgr(self) -> I8Vector4: ...
    @property
    def llgg(self) -> I8Vector4: ...
    @property
    def llgb(self) -> I8Vector4: ...
    @property
    def llgo(self) -> I8Vector4: ...
    @property
    def llgl(self) -> I8Vector4: ...
    @property
    def llbr(self) -> I8Vector4: ...
    @property
    def llbg(self) -> I8Vector4: ...
    @property
    def llbb(self) -> I8Vector4: ...
    @property
    def llbo(self) -> I8Vector4: ...
    @property
    def llbl(self) -> I8Vector4: ...
    @property
    def llor(self) -> I8Vector4: ...
    @property
    def llog(self) -> I8Vector4: ...
    @property
    def llob(self) -> I8Vector4: ...
    @property
    def lllr(self) -> I8Vector4: ...
    @property
    def lllg(self) -> I8Vector4: ...
    @property
    def lllb(self) -> I8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> I8Vector2: ...
    @property
    def st(self) -> I8Vector2: ...
    @property
    def sq(self) -> I8Vector2: ...
    @property
    def so(self) -> I8Vector2: ...
    @property
    def sl(self) -> I8Vector2: ...
    @property
    def ts(self) -> I8Vector2: ...
    @property
    def tt(self) -> I8Vector2: ...
    @property
    def tq(self) -> I8Vector2: ...
    @property
    def to(self) -> I8Vector2: ...
    @property
    def tl(self) -> I8Vector2: ...
    @property
    def qs(self) -> I8Vector2: ...
    @property
    def qt(self) -> I8Vector2: ...
    @property
    def qq(self) -> I8Vector2: ...
    @property
    def qo(self) -> I8Vector2: ...
    @property
    def ql(self) -> I8Vector2: ...
    @property
    def os(self) -> I8Vector2: ...
    @property
    def ot(self) -> I8Vector2: ...
    @property
    def oq(self) -> I8Vector2: ...
    @property
    def ls(self) -> I8Vector2: ...
    @property
    def lt(self) -> I8Vector2: ...
    @property
    def lq(self) -> I8Vector2: ...
    @property
    def sss(self) -> I8Vector3: ...
    @property
    def sst(self) -> I8Vector3: ...
    @property
    def ssq(self) -> I8Vector3: ...
    @property
    def sso(self) -> I8Vector3: ...
    @property
    def ssl(self) -> I8Vector3: ...
    @property
    def sts(self) -> I8Vector3: ...
    @property
    def stt(self) -> I8Vector3: ...
    @property
    def stq(self) -> I8Vector3: ...
    @property
    def sto(self) -> I8Vector3: ...
    @property
    def stl(self) -> I8Vector3: ...
    @property
    def sqs(self) -> I8Vector3: ...
    @property
    def sqt(self) -> I8Vector3: ...
    @property
    def sqq(self) -> I8Vector3: ...
    @property
    def sqo(self) -> I8Vector3: ...
    @property
    def sql(self) -> I8Vector3: ...
    @property
    def sos(self) -> I8Vector3: ...
    @property
    def sot(self) -> I8Vector3: ...
    @property
    def soq(self) -> I8Vector3: ...
    @property
    def soo(self) -> I8Vector3: ...
    @property
    def sol(self) -> I8Vector3: ...
    @property
    def sls(self) -> I8Vector3: ...
    @property
    def slt(self) -> I8Vector3: ...
    @property
    def slq(self) -> I8Vector3: ...
    @property
    def slo(self) -> I8Vector3: ...
    @property
    def sll(self) -> I8Vector3: ...
    @property
    def tss(self) -> I8Vector3: ...
    @property
    def tst(self) -> I8Vector3: ...
    @property
    def tsq(self) -> I8Vector3: ...
    @property
    def tso(self) -> I8Vector3: ...
    @property
    def tsl(self) -> I8Vector3: ...
    @property
    def tts(self) -> I8Vector3: ...
    @property
    def ttt(self) -> I8Vector3: ...
    @property
    def ttq(self) -> I8Vector3: ...
    @property
    def tto(self) -> I8Vector3: ...
    @property
    def ttl(self) -> I8Vector3: ...
    @property
    def tqs(self) -> I8Vector3: ...
    @property
    def tqt(self) -> I8Vector3: ...
    @property
    def tqq(self) -> I8Vector3: ...
    @property
    def tqo(self) -> I8Vector3: ...
    @property
    def tql(self) -> I8Vector3: ...
    @property
    def tos(self) -> I8Vector3: ...
    @property
    def tot(self) -> I8Vector3: ...
    @property
    def toq(self) -> I8Vector3: ...
    @property
    def too(self) -> I8Vector3: ...
    @property
    def tol(self) -> I8Vector3: ...
    @property
    def tls(self) -> I8Vector3: ...
    @property
    def tlt(self) -> I8Vector3: ...
    @property
    def tlq(self) -> I8Vector3: ...
    @property
    def tlo(self) -> I8Vector3: ...
    @property
    def tll(self) -> I8Vector3: ...
    @property
    def qss(self) -> I8Vector3: ...
    @property
    def qst(self) -> I8Vector3: ...
    @property
    def qsq(self) -> I8Vector3: ...
    @property
    def qso(self) -> I8Vector3: ...
    @property
    def qsl(self) -> I8Vector3: ...
    @property
    def qts(self) -> I8Vector3: ...
    @property
    def qtt(self) -> I8Vector3: ...
    @property
    def qtq(self) -> I8Vector3: ...
    @property
    def qto(self) -> I8Vector3: ...
    @property
    def qtl(self) -> I8Vector3: ...
    @property
    def qqs(self) -> I8Vector3: ...
    @property
    def qqt(self) -> I8Vector3: ...
    @property
    def qqq(self) -> I8Vector3: ...
    @property
    def qqo(self) -> I8Vector3: ...
    @property
    def qql(self) -> I8Vector3: ...
    @property
    def qos(self) -> I8Vector3: ...
    @property
    def qot(self) -> I8Vector3: ...
    @property
    def qoq(self) -> I8Vector3: ...
    @property
    def qoo(self) -> I8Vector3: ...
    @property
    def qol(self) -> I8Vector3: ...
    @property
    def qls(self) -> I8Vector3: ...
    @property
    def qlt(self) -> I8Vector3: ...
    @property
    def qlq(self) -> I8Vector3: ...
    @property
    def qlo(self) -> I8Vector3: ...
    @property
    def qll(self) -> I8Vector3: ...
    @property
    def oss(self) -> I8Vector3: ...
    @property
    def ost(self) -> I8Vector3: ...
    @property
    def osq(self) -> I8Vector3: ...
    @property
    def oso(self) -> I8Vector3: ...
    @property
    def osl(self) -> I8Vector3: ...
    @property
    def ots(self) -> I8Vector3: ...
    @property
    def ott(self) -> I8Vector3: ...
    @property
    def otq(self) -> I8Vector3: ...
    @property
    def oto(self) -> I8Vector3: ...
    @property
    def otl(self) -> I8Vector3: ...
    @property
    def oqs(self) -> I8Vector3: ...
    @property
    def oqt(self) -> I8Vector3: ...
    @property
    def oqq(self) -> I8Vector3: ...
    @property
    def oqo(self) -> I8Vector3: ...
    @property
    def oql(self) -> I8Vector3: ...
    @property
    def oos(self) -> I8Vector3: ...
    @property
    def oot(self) -> I8Vector3: ...
    @property
    def ooq(self) -> I8Vector3: ...
    @property
    def ols(self) -> I8Vector3: ...
    @property
    def olt(self) -> I8Vector3: ...
    @property
    def olq(self) -> I8Vector3: ...
    @property
    def lss(self) -> I8Vector3: ...
    @property
    def lst(self) -> I8Vector3: ...
    @property
    def lsq(self) -> I8Vector3: ...
    @property
    def lso(self) -> I8Vector3: ...
    @property
    def lsl(self) -> I8Vector3: ...
    @property
    def lts(self) -> I8Vector3: ...
    @property
    def ltt(self) -> I8Vector3: ...
    @property
    def ltq(self) -> I8Vector3: ...
    @property
    def lto(self) -> I8Vector3: ...
    @property
    def ltl(self) -> I8Vector3: ...
    @property
    def lqs(self) -> I8Vector3: ...
    @property
    def lqt(self) -> I8Vector3: ...
    @property
    def lqq(self) -> I8Vector3: ...
    @property
    def lqo(self) -> I8Vector3: ...
    @property
    def lql(self) -> I8Vector3: ...
    @property
    def los(self) -> I8Vector3: ...
    @property
    def lot(self) -> I8Vector3: ...
    @property
    def loq(self) -> I8Vector3: ...
    @property
    def lls(self) -> I8Vector3: ...
    @property
    def llt(self) -> I8Vector3: ...
    @property
    def llq(self) -> I8Vector3: ...
    @property
    def ssss(self) -> I8Vector4: ...
    @property
    def ssst(self) -> I8Vector4: ...
    @property
    def sssq(self) -> I8Vector4: ...
    @property
    def ssso(self) -> I8Vector4: ...
    @property
    def sssl(self) -> I8Vector4: ...
    @property
    def ssts(self) -> I8Vector4: ...
    @property
    def sstt(self) -> I8Vector4: ...
    @property
    def sstq(self) -> I8Vector4: ...
    @property
    def ssto(self) -> I8Vector4: ...
    @property
    def sstl(self) -> I8Vector4: ...
    @property
    def ssqs(self) -> I8Vector4: ...
    @property
    def ssqt(self) -> I8Vector4: ...
    @property
    def ssqq(self) -> I8Vector4: ...
    @property
    def ssqo(self) -> I8Vector4: ...
    @property
    def ssql(self) -> I8Vector4: ...
    @property
    def ssos(self) -> I8Vector4: ...
    @property
    def ssot(self) -> I8Vector4: ...
    @property
    def ssoq(self) -> I8Vector4: ...
    @property
    def ssoo(self) -> I8Vector4: ...
    @property
    def ssol(self) -> I8Vector4: ...
    @property
    def ssls(self) -> I8Vector4: ...
    @property
    def sslt(self) -> I8Vector4: ...
    @property
    def sslq(self) -> I8Vector4: ...
    @property
    def sslo(self) -> I8Vector4: ...
    @property
    def ssll(self) -> I8Vector4: ...
    @property
    def stss(self) -> I8Vector4: ...
    @property
    def stst(self) -> I8Vector4: ...
    @property
    def stsq(self) -> I8Vector4: ...
    @property
    def stso(self) -> I8Vector4: ...
    @property
    def stsl(self) -> I8Vector4: ...
    @property
    def stts(self) -> I8Vector4: ...
    @property
    def sttt(self) -> I8Vector4: ...
    @property
    def sttq(self) -> I8Vector4: ...
    @property
    def stto(self) -> I8Vector4: ...
    @property
    def sttl(self) -> I8Vector4: ...
    @property
    def stqs(self) -> I8Vector4: ...
    @property
    def stqt(self) -> I8Vector4: ...
    @property
    def stqq(self) -> I8Vector4: ...
    @property
    def stqo(self) -> I8Vector4: ...
    @property
    def stql(self) -> I8Vector4: ...
    @property
    def stos(self) -> I8Vector4: ...
    @property
    def stot(self) -> I8Vector4: ...
    @property
    def stoq(self) -> I8Vector4: ...
    @property
    def stoo(self) -> I8Vector4: ...
    @property
    def stol(self) -> I8Vector4: ...
    @property
    def stls(self) -> I8Vector4: ...
    @property
    def stlt(self) -> I8Vector4: ...
    @property
    def stlq(self) -> I8Vector4: ...
    @property
    def stlo(self) -> I8Vector4: ...
    @property
    def stll(self) -> I8Vector4: ...
    @property
    def sqss(self) -> I8Vector4: ...
    @property
    def sqst(self) -> I8Vector4: ...
    @property
    def sqsq(self) -> I8Vector4: ...
    @property
    def sqso(self) -> I8Vector4: ...
    @property
    def sqsl(self) -> I8Vector4: ...
    @property
    def sqts(self) -> I8Vector4: ...
    @property
    def sqtt(self) -> I8Vector4: ...
    @property
    def sqtq(self) -> I8Vector4: ...
    @property
    def sqto(self) -> I8Vector4: ...
    @property
    def sqtl(self) -> I8Vector4: ...
    @property
    def sqqs(self) -> I8Vector4: ...
    @property
    def sqqt(self) -> I8Vector4: ...
    @property
    def sqqq(self) -> I8Vector4: ...
    @property
    def sqqo(self) -> I8Vector4: ...
    @property
    def sqql(self) -> I8Vector4: ...
    @property
    def sqos(self) -> I8Vector4: ...
    @property
    def sqot(self) -> I8Vector4: ...
    @property
    def sqoq(self) -> I8Vector4: ...
    @property
    def sqoo(self) -> I8Vector4: ...
    @property
    def sqol(self) -> I8Vector4: ...
    @property
    def sqls(self) -> I8Vector4: ...
    @property
    def sqlt(self) -> I8Vector4: ...
    @property
    def sqlq(self) -> I8Vector4: ...
    @property
    def sqlo(self) -> I8Vector4: ...
    @property
    def sqll(self) -> I8Vector4: ...
    @property
    def soss(self) -> I8Vector4: ...
    @property
    def sost(self) -> I8Vector4: ...
    @property
    def sosq(self) -> I8Vector4: ...
    @property
    def soso(self) -> I8Vector4: ...
    @property
    def sosl(self) -> I8Vector4: ...
    @property
    def sots(self) -> I8Vector4: ...
    @property
    def sott(self) -> I8Vector4: ...
    @property
    def sotq(self) -> I8Vector4: ...
    @property
    def soto(self) -> I8Vector4: ...
    @property
    def sotl(self) -> I8Vector4: ...
    @property
    def soqs(self) -> I8Vector4: ...
    @property
    def soqt(self) -> I8Vector4: ...
    @property
    def soqq(self) -> I8Vector4: ...
    @property
    def soqo(self) -> I8Vector4: ...
    @property
    def soql(self) -> I8Vector4: ...
    @property
    def soos(self) -> I8Vector4: ...
    @property
    def soot(self) -> I8Vector4: ...
    @property
    def sooq(self) -> I8Vector4: ...
    @property
    def sooo(self) -> I8Vector4: ...
    @property
    def sool(self) -> I8Vector4: ...
    @property
    def sols(self) -> I8Vector4: ...
    @property
    def solt(self) -> I8Vector4: ...
    @property
    def solq(self) -> I8Vector4: ...
    @property
    def solo(self) -> I8Vector4: ...
    @property
    def soll(self) -> I8Vector4: ...
    @property
    def slss(self) -> I8Vector4: ...
    @property
    def slst(self) -> I8Vector4: ...
    @property
    def slsq(self) -> I8Vector4: ...
    @property
    def slso(self) -> I8Vector4: ...
    @property
    def slsl(self) -> I8Vector4: ...
    @property
    def slts(self) -> I8Vector4: ...
    @property
    def sltt(self) -> I8Vector4: ...
    @property
    def sltq(self) -> I8Vector4: ...
    @property
    def slto(self) -> I8Vector4: ...
    @property
    def sltl(self) -> I8Vector4: ...
    @property
    def slqs(self) -> I8Vector4: ...
    @property
    def slqt(self) -> I8Vector4: ...
    @property
    def slqq(self) -> I8Vector4: ...
    @property
    def slqo(self) -> I8Vector4: ...
    @property
    def slql(self) -> I8Vector4: ...
    @property
    def slos(self) -> I8Vector4: ...
    @property
    def slot(self) -> I8Vector4: ...
    @property
    def sloq(self) -> I8Vector4: ...
    @property
    def sloo(self) -> I8Vector4: ...
    @property
    def slol(self) -> I8Vector4: ...
    @property
    def slls(self) -> I8Vector4: ...
    @property
    def sllt(self) -> I8Vector4: ...
    @property
    def sllq(self) -> I8Vector4: ...
    @property
    def sllo(self) -> I8Vector4: ...
    @property
    def slll(self) -> I8Vector4: ...
    @property
    def tsss(self) -> I8Vector4: ...
    @property
    def tsst(self) -> I8Vector4: ...
    @property
    def tssq(self) -> I8Vector4: ...
    @property
    def tsso(self) -> I8Vector4: ...
    @property
    def tssl(self) -> I8Vector4: ...
    @property
    def tsts(self) -> I8Vector4: ...
    @property
    def tstt(self) -> I8Vector4: ...
    @property
    def tstq(self) -> I8Vector4: ...
    @property
    def tsto(self) -> I8Vector4: ...
    @property
    def tstl(self) -> I8Vector4: ...
    @property
    def tsqs(self) -> I8Vector4: ...
    @property
    def tsqt(self) -> I8Vector4: ...
    @property
    def tsqq(self) -> I8Vector4: ...
    @property
    def tsqo(self) -> I8Vector4: ...
    @property
    def tsql(self) -> I8Vector4: ...
    @property
    def tsos(self) -> I8Vector4: ...
    @property
    def tsot(self) -> I8Vector4: ...
    @property
    def tsoq(self) -> I8Vector4: ...
    @property
    def tsoo(self) -> I8Vector4: ...
    @property
    def tsol(self) -> I8Vector4: ...
    @property
    def tsls(self) -> I8Vector4: ...
    @property
    def tslt(self) -> I8Vector4: ...
    @property
    def tslq(self) -> I8Vector4: ...
    @property
    def tslo(self) -> I8Vector4: ...
    @property
    def tsll(self) -> I8Vector4: ...
    @property
    def ttss(self) -> I8Vector4: ...
    @property
    def ttst(self) -> I8Vector4: ...
    @property
    def ttsq(self) -> I8Vector4: ...
    @property
    def ttso(self) -> I8Vector4: ...
    @property
    def ttsl(self) -> I8Vector4: ...
    @property
    def ttts(self) -> I8Vector4: ...
    @property
    def tttt(self) -> I8Vector4: ...
    @property
    def tttq(self) -> I8Vector4: ...
    @property
    def ttto(self) -> I8Vector4: ...
    @property
    def tttl(self) -> I8Vector4: ...
    @property
    def ttqs(self) -> I8Vector4: ...
    @property
    def ttqt(self) -> I8Vector4: ...
    @property
    def ttqq(self) -> I8Vector4: ...
    @property
    def ttqo(self) -> I8Vector4: ...
    @property
    def ttql(self) -> I8Vector4: ...
    @property
    def ttos(self) -> I8Vector4: ...
    @property
    def ttot(self) -> I8Vector4: ...
    @property
    def ttoq(self) -> I8Vector4: ...
    @property
    def ttoo(self) -> I8Vector4: ...
    @property
    def ttol(self) -> I8Vector4: ...
    @property
    def ttls(self) -> I8Vector4: ...
    @property
    def ttlt(self) -> I8Vector4: ...
    @property
    def ttlq(self) -> I8Vector4: ...
    @property
    def ttlo(self) -> I8Vector4: ...
    @property
    def ttll(self) -> I8Vector4: ...
    @property
    def tqss(self) -> I8Vector4: ...
    @property
    def tqst(self) -> I8Vector4: ...
    @property
    def tqsq(self) -> I8Vector4: ...
    @property
    def tqso(self) -> I8Vector4: ...
    @property
    def tqsl(self) -> I8Vector4: ...
    @property
    def tqts(self) -> I8Vector4: ...
    @property
    def tqtt(self) -> I8Vector4: ...
    @property
    def tqtq(self) -> I8Vector4: ...
    @property
    def tqto(self) -> I8Vector4: ...
    @property
    def tqtl(self) -> I8Vector4: ...
    @property
    def tqqs(self) -> I8Vector4: ...
    @property
    def tqqt(self) -> I8Vector4: ...
    @property
    def tqqq(self) -> I8Vector4: ...
    @property
    def tqqo(self) -> I8Vector4: ...
    @property
    def tqql(self) -> I8Vector4: ...
    @property
    def tqos(self) -> I8Vector4: ...
    @property
    def tqot(self) -> I8Vector4: ...
    @property
    def tqoq(self) -> I8Vector4: ...
    @property
    def tqoo(self) -> I8Vector4: ...
    @property
    def tqol(self) -> I8Vector4: ...
    @property
    def tqls(self) -> I8Vector4: ...
    @property
    def tqlt(self) -> I8Vector4: ...
    @property
    def tqlq(self) -> I8Vector4: ...
    @property
    def tqlo(self) -> I8Vector4: ...
    @property
    def tqll(self) -> I8Vector4: ...
    @property
    def toss(self) -> I8Vector4: ...
    @property
    def tost(self) -> I8Vector4: ...
    @property
    def tosq(self) -> I8Vector4: ...
    @property
    def toso(self) -> I8Vector4: ...
    @property
    def tosl(self) -> I8Vector4: ...
    @property
    def tots(self) -> I8Vector4: ...
    @property
    def tott(self) -> I8Vector4: ...
    @property
    def totq(self) -> I8Vector4: ...
    @property
    def toto(self) -> I8Vector4: ...
    @property
    def totl(self) -> I8Vector4: ...
    @property
    def toqs(self) -> I8Vector4: ...
    @property
    def toqt(self) -> I8Vector4: ...
    @property
    def toqq(self) -> I8Vector4: ...
    @property
    def toqo(self) -> I8Vector4: ...
    @property
    def toql(self) -> I8Vector4: ...
    @property
    def toos(self) -> I8Vector4: ...
    @property
    def toot(self) -> I8Vector4: ...
    @property
    def tooq(self) -> I8Vector4: ...
    @property
    def tooo(self) -> I8Vector4: ...
    @property
    def tool(self) -> I8Vector4: ...
    @property
    def tols(self) -> I8Vector4: ...
    @property
    def tolt(self) -> I8Vector4: ...
    @property
    def tolq(self) -> I8Vector4: ...
    @property
    def tolo(self) -> I8Vector4: ...
    @property
    def toll(self) -> I8Vector4: ...
    @property
    def tlss(self) -> I8Vector4: ...
    @property
    def tlst(self) -> I8Vector4: ...
    @property
    def tlsq(self) -> I8Vector4: ...
    @property
    def tlso(self) -> I8Vector4: ...
    @property
    def tlsl(self) -> I8Vector4: ...
    @property
    def tlts(self) -> I8Vector4: ...
    @property
    def tltt(self) -> I8Vector4: ...
    @property
    def tltq(self) -> I8Vector4: ...
    @property
    def tlto(self) -> I8Vector4: ...
    @property
    def tltl(self) -> I8Vector4: ...
    @property
    def tlqs(self) -> I8Vector4: ...
    @property
    def tlqt(self) -> I8Vector4: ...
    @property
    def tlqq(self) -> I8Vector4: ...
    @property
    def tlqo(self) -> I8Vector4: ...
    @property
    def tlql(self) -> I8Vector4: ...
    @property
    def tlos(self) -> I8Vector4: ...
    @property
    def tlot(self) -> I8Vector4: ...
    @property
    def tloq(self) -> I8Vector4: ...
    @property
    def tloo(self) -> I8Vector4: ...
    @property
    def tlol(self) -> I8Vector4: ...
    @property
    def tlls(self) -> I8Vector4: ...
    @property
    def tllt(self) -> I8Vector4: ...
    @property
    def tllq(self) -> I8Vector4: ...
    @property
    def tllo(self) -> I8Vector4: ...
    @property
    def tlll(self) -> I8Vector4: ...
    @property
    def qsss(self) -> I8Vector4: ...
    @property
    def qsst(self) -> I8Vector4: ...
    @property
    def qssq(self) -> I8Vector4: ...
    @property
    def qsso(self) -> I8Vector4: ...
    @property
    def qssl(self) -> I8Vector4: ...
    @property
    def qsts(self) -> I8Vector4: ...
    @property
    def qstt(self) -> I8Vector4: ...
    @property
    def qstq(self) -> I8Vector4: ...
    @property
    def qsto(self) -> I8Vector4: ...
    @property
    def qstl(self) -> I8Vector4: ...
    @property
    def qsqs(self) -> I8Vector4: ...
    @property
    def qsqt(self) -> I8Vector4: ...
    @property
    def qsqq(self) -> I8Vector4: ...
    @property
    def qsqo(self) -> I8Vector4: ...
    @property
    def qsql(self) -> I8Vector4: ...
    @property
    def qsos(self) -> I8Vector4: ...
    @property
    def qsot(self) -> I8Vector4: ...
    @property
    def qsoq(self) -> I8Vector4: ...
    @property
    def qsoo(self) -> I8Vector4: ...
    @property
    def qsol(self) -> I8Vector4: ...
    @property
    def qsls(self) -> I8Vector4: ...
    @property
    def qslt(self) -> I8Vector4: ...
    @property
    def qslq(self) -> I8Vector4: ...
    @property
    def qslo(self) -> I8Vector4: ...
    @property
    def qsll(self) -> I8Vector4: ...
    @property
    def qtss(self) -> I8Vector4: ...
    @property
    def qtst(self) -> I8Vector4: ...
    @property
    def qtsq(self) -> I8Vector4: ...
    @property
    def qtso(self) -> I8Vector4: ...
    @property
    def qtsl(self) -> I8Vector4: ...
    @property
    def qtts(self) -> I8Vector4: ...
    @property
    def qttt(self) -> I8Vector4: ...
    @property
    def qttq(self) -> I8Vector4: ...
    @property
    def qtto(self) -> I8Vector4: ...
    @property
    def qttl(self) -> I8Vector4: ...
    @property
    def qtqs(self) -> I8Vector4: ...
    @property
    def qtqt(self) -> I8Vector4: ...
    @property
    def qtqq(self) -> I8Vector4: ...
    @property
    def qtqo(self) -> I8Vector4: ...
    @property
    def qtql(self) -> I8Vector4: ...
    @property
    def qtos(self) -> I8Vector4: ...
    @property
    def qtot(self) -> I8Vector4: ...
    @property
    def qtoq(self) -> I8Vector4: ...
    @property
    def qtoo(self) -> I8Vector4: ...
    @property
    def qtol(self) -> I8Vector4: ...
    @property
    def qtls(self) -> I8Vector4: ...
    @property
    def qtlt(self) -> I8Vector4: ...
    @property
    def qtlq(self) -> I8Vector4: ...
    @property
    def qtlo(self) -> I8Vector4: ...
    @property
    def qtll(self) -> I8Vector4: ...
    @property
    def qqss(self) -> I8Vector4: ...
    @property
    def qqst(self) -> I8Vector4: ...
    @property
    def qqsq(self) -> I8Vector4: ...
    @property
    def qqso(self) -> I8Vector4: ...
    @property
    def qqsl(self) -> I8Vector4: ...
    @property
    def qqts(self) -> I8Vector4: ...
    @property
    def qqtt(self) -> I8Vector4: ...
    @property
    def qqtq(self) -> I8Vector4: ...
    @property
    def qqto(self) -> I8Vector4: ...
    @property
    def qqtl(self) -> I8Vector4: ...
    @property
    def qqqs(self) -> I8Vector4: ...
    @property
    def qqqt(self) -> I8Vector4: ...
    @property
    def qqqq(self) -> I8Vector4: ...
    @property
    def qqqo(self) -> I8Vector4: ...
    @property
    def qqql(self) -> I8Vector4: ...
    @property
    def qqos(self) -> I8Vector4: ...
    @property
    def qqot(self) -> I8Vector4: ...
    @property
    def qqoq(self) -> I8Vector4: ...
    @property
    def qqoo(self) -> I8Vector4: ...
    @property
    def qqol(self) -> I8Vector4: ...
    @property
    def qqls(self) -> I8Vector4: ...
    @property
    def qqlt(self) -> I8Vector4: ...
    @property
    def qqlq(self) -> I8Vector4: ...
    @property
    def qqlo(self) -> I8Vector4: ...
    @property
    def qqll(self) -> I8Vector4: ...
    @property
    def qoss(self) -> I8Vector4: ...
    @property
    def qost(self) -> I8Vector4: ...
    @property
    def qosq(self) -> I8Vector4: ...
    @property
    def qoso(self) -> I8Vector4: ...
    @property
    def qosl(self) -> I8Vector4: ...
    @property
    def qots(self) -> I8Vector4: ...
    @property
    def qott(self) -> I8Vector4: ...
    @property
    def qotq(self) -> I8Vector4: ...
    @property
    def qoto(self) -> I8Vector4: ...
    @property
    def qotl(self) -> I8Vector4: ...
    @property
    def qoqs(self) -> I8Vector4: ...
    @property
    def qoqt(self) -> I8Vector4: ...
    @property
    def qoqq(self) -> I8Vector4: ...
    @property
    def qoqo(self) -> I8Vector4: ...
    @property
    def qoql(self) -> I8Vector4: ...
    @property
    def qoos(self) -> I8Vector4: ...
    @property
    def qoot(self) -> I8Vector4: ...
    @property
    def qooq(self) -> I8Vector4: ...
    @property
    def qooo(self) -> I8Vector4: ...
    @property
    def qool(self) -> I8Vector4: ...
    @property
    def qols(self) -> I8Vector4: ...
    @property
    def qolt(self) -> I8Vector4: ...
    @property
    def qolq(self) -> I8Vector4: ...
    @property
    def qolo(self) -> I8Vector4: ...
    @property
    def qoll(self) -> I8Vector4: ...
    @property
    def qlss(self) -> I8Vector4: ...
    @property
    def qlst(self) -> I8Vector4: ...
    @property
    def qlsq(self) -> I8Vector4: ...
    @property
    def qlso(self) -> I8Vector4: ...
    @property
    def qlsl(self) -> I8Vector4: ...
    @property
    def qlts(self) -> I8Vector4: ...
    @property
    def qltt(self) -> I8Vector4: ...
    @property
    def qltq(self) -> I8Vector4: ...
    @property
    def qlto(self) -> I8Vector4: ...
    @property
    def qltl(self) -> I8Vector4: ...
    @property
    def qlqs(self) -> I8Vector4: ...
    @property
    def qlqt(self) -> I8Vector4: ...
    @property
    def qlqq(self) -> I8Vector4: ...
    @property
    def qlqo(self) -> I8Vector4: ...
    @property
    def qlql(self) -> I8Vector4: ...
    @property
    def qlos(self) -> I8Vector4: ...
    @property
    def qlot(self) -> I8Vector4: ...
    @property
    def qloq(self) -> I8Vector4: ...
    @property
    def qloo(self) -> I8Vector4: ...
    @property
    def qlol(self) -> I8Vector4: ...
    @property
    def qlls(self) -> I8Vector4: ...
    @property
    def qllt(self) -> I8Vector4: ...
    @property
    def qllq(self) -> I8Vector4: ...
    @property
    def qllo(self) -> I8Vector4: ...
    @property
    def qlll(self) -> I8Vector4: ...
    @property
    def osss(self) -> I8Vector4: ...
    @property
    def osst(self) -> I8Vector4: ...
    @property
    def ossq(self) -> I8Vector4: ...
    @property
    def osso(self) -> I8Vector4: ...
    @property
    def ossl(self) -> I8Vector4: ...
    @property
    def osts(self) -> I8Vector4: ...
    @property
    def ostt(self) -> I8Vector4: ...
    @property
    def ostq(self) -> I8Vector4: ...
    @property
    def osto(self) -> I8Vector4: ...
    @property
    def ostl(self) -> I8Vector4: ...
    @property
    def osqs(self) -> I8Vector4: ...
    @property
    def osqt(self) -> I8Vector4: ...
    @property
    def osqq(self) -> I8Vector4: ...
    @property
    def osqo(self) -> I8Vector4: ...
    @property
    def osql(self) -> I8Vector4: ...
    @property
    def osos(self) -> I8Vector4: ...
    @property
    def osot(self) -> I8Vector4: ...
    @property
    def osoq(self) -> I8Vector4: ...
    @property
    def osoo(self) -> I8Vector4: ...
    @property
    def osol(self) -> I8Vector4: ...
    @property
    def osls(self) -> I8Vector4: ...
    @property
    def oslt(self) -> I8Vector4: ...
    @property
    def oslq(self) -> I8Vector4: ...
    @property
    def oslo(self) -> I8Vector4: ...
    @property
    def osll(self) -> I8Vector4: ...
    @property
    def otss(self) -> I8Vector4: ...
    @property
    def otst(self) -> I8Vector4: ...
    @property
    def otsq(self) -> I8Vector4: ...
    @property
    def otso(self) -> I8Vector4: ...
    @property
    def otsl(self) -> I8Vector4: ...
    @property
    def otts(self) -> I8Vector4: ...
    @property
    def ottt(self) -> I8Vector4: ...
    @property
    def ottq(self) -> I8Vector4: ...
    @property
    def otto(self) -> I8Vector4: ...
    @property
    def ottl(self) -> I8Vector4: ...
    @property
    def otqs(self) -> I8Vector4: ...
    @property
    def otqt(self) -> I8Vector4: ...
    @property
    def otqq(self) -> I8Vector4: ...
    @property
    def otqo(self) -> I8Vector4: ...
    @property
    def otql(self) -> I8Vector4: ...
    @property
    def otos(self) -> I8Vector4: ...
    @property
    def otot(self) -> I8Vector4: ...
    @property
    def otoq(self) -> I8Vector4: ...
    @property
    def otoo(self) -> I8Vector4: ...
    @property
    def otol(self) -> I8Vector4: ...
    @property
    def otls(self) -> I8Vector4: ...
    @property
    def otlt(self) -> I8Vector4: ...
    @property
    def otlq(self) -> I8Vector4: ...
    @property
    def otlo(self) -> I8Vector4: ...
    @property
    def otll(self) -> I8Vector4: ...
    @property
    def oqss(self) -> I8Vector4: ...
    @property
    def oqst(self) -> I8Vector4: ...
    @property
    def oqsq(self) -> I8Vector4: ...
    @property
    def oqso(self) -> I8Vector4: ...
    @property
    def oqsl(self) -> I8Vector4: ...
    @property
    def oqts(self) -> I8Vector4: ...
    @property
    def oqtt(self) -> I8Vector4: ...
    @property
    def oqtq(self) -> I8Vector4: ...
    @property
    def oqto(self) -> I8Vector4: ...
    @property
    def oqtl(self) -> I8Vector4: ...
    @property
    def oqqs(self) -> I8Vector4: ...
    @property
    def oqqt(self) -> I8Vector4: ...
    @property
    def oqqq(self) -> I8Vector4: ...
    @property
    def oqqo(self) -> I8Vector4: ...
    @property
    def oqql(self) -> I8Vector4: ...
    @property
    def oqos(self) -> I8Vector4: ...
    @property
    def oqot(self) -> I8Vector4: ...
    @property
    def oqoq(self) -> I8Vector4: ...
    @property
    def oqoo(self) -> I8Vector4: ...
    @property
    def oqol(self) -> I8Vector4: ...
    @property
    def oqls(self) -> I8Vector4: ...
    @property
    def oqlt(self) -> I8Vector4: ...
    @property
    def oqlq(self) -> I8Vector4: ...
    @property
    def oqlo(self) -> I8Vector4: ...
    @property
    def oqll(self) -> I8Vector4: ...
    @property
    def ooss(self) -> I8Vector4: ...
    @property
    def oost(self) -> I8Vector4: ...
    @property
    def oosq(self) -> I8Vector4: ...
    @property
    def ooso(self) -> I8Vector4: ...
    @property
    def oosl(self) -> I8Vector4: ...
    @property
    def oots(self) -> I8Vector4: ...
    @property
    def oott(self) -> I8Vector4: ...
    @property
    def ootq(self) -> I8Vector4: ...
    @property
    def ooto(self) -> I8Vector4: ...
    @property
    def ootl(self) -> I8Vector4: ...
    @property
    def ooqs(self) -> I8Vector4: ...
    @property
    def ooqt(self) -> I8Vector4: ...
    @property
    def ooqq(self) -> I8Vector4: ...
    @property
    def ooqo(self) -> I8Vector4: ...
    @property
    def ooql(self) -> I8Vector4: ...
    @property
    def ooos(self) -> I8Vector4: ...
    @property
    def ooot(self) -> I8Vector4: ...
    @property
    def oooq(self) -> I8Vector4: ...
    @property
    def ools(self) -> I8Vector4: ...
    @property
    def oolt(self) -> I8Vector4: ...
    @property
    def oolq(self) -> I8Vector4: ...
    @property
    def olss(self) -> I8Vector4: ...
    @property
    def olst(self) -> I8Vector4: ...
    @property
    def olsq(self) -> I8Vector4: ...
    @property
    def olso(self) -> I8Vector4: ...
    @property
    def olsl(self) -> I8Vector4: ...
    @property
    def olts(self) -> I8Vector4: ...
    @property
    def oltt(self) -> I8Vector4: ...
    @property
    def oltq(self) -> I8Vector4: ...
    @property
    def olto(self) -> I8Vector4: ...
    @property
    def oltl(self) -> I8Vector4: ...
    @property
    def olqs(self) -> I8Vector4: ...
    @property
    def olqt(self) -> I8Vector4: ...
    @property
    def olqq(self) -> I8Vector4: ...
    @property
    def olqo(self) -> I8Vector4: ...
    @property
    def olql(self) -> I8Vector4: ...
    @property
    def olos(self) -> I8Vector4: ...
    @property
    def olot(self) -> I8Vector4: ...
    @property
    def oloq(self) -> I8Vector4: ...
    @property
    def olls(self) -> I8Vector4: ...
    @property
    def ollt(self) -> I8Vector4: ...
    @property
    def ollq(self) -> I8Vector4: ...
    @property
    def lsss(self) -> I8Vector4: ...
    @property
    def lsst(self) -> I8Vector4: ...
    @property
    def lssq(self) -> I8Vector4: ...
    @property
    def lsso(self) -> I8Vector4: ...
    @property
    def lssl(self) -> I8Vector4: ...
    @property
    def lsts(self) -> I8Vector4: ...
    @property
    def lstt(self) -> I8Vector4: ...
    @property
    def lstq(self) -> I8Vector4: ...
    @property
    def lsto(self) -> I8Vector4: ...
    @property
    def lstl(self) -> I8Vector4: ...
    @property
    def lsqs(self) -> I8Vector4: ...
    @property
    def lsqt(self) -> I8Vector4: ...
    @property
    def lsqq(self) -> I8Vector4: ...
    @property
    def lsqo(self) -> I8Vector4: ...
    @property
    def lsql(self) -> I8Vector4: ...
    @property
    def lsos(self) -> I8Vector4: ...
    @property
    def lsot(self) -> I8Vector4: ...
    @property
    def lsoq(self) -> I8Vector4: ...
    @property
    def lsoo(self) -> I8Vector4: ...
    @property
    def lsol(self) -> I8Vector4: ...
    @property
    def lsls(self) -> I8Vector4: ...
    @property
    def lslt(self) -> I8Vector4: ...
    @property
    def lslq(self) -> I8Vector4: ...
    @property
    def lslo(self) -> I8Vector4: ...
    @property
    def lsll(self) -> I8Vector4: ...
    @property
    def ltss(self) -> I8Vector4: ...
    @property
    def ltst(self) -> I8Vector4: ...
    @property
    def ltsq(self) -> I8Vector4: ...
    @property
    def ltso(self) -> I8Vector4: ...
    @property
    def ltsl(self) -> I8Vector4: ...
    @property
    def ltts(self) -> I8Vector4: ...
    @property
    def lttt(self) -> I8Vector4: ...
    @property
    def lttq(self) -> I8Vector4: ...
    @property
    def ltto(self) -> I8Vector4: ...
    @property
    def lttl(self) -> I8Vector4: ...
    @property
    def ltqs(self) -> I8Vector4: ...
    @property
    def ltqt(self) -> I8Vector4: ...
    @property
    def ltqq(self) -> I8Vector4: ...
    @property
    def ltqo(self) -> I8Vector4: ...
    @property
    def ltql(self) -> I8Vector4: ...
    @property
    def ltos(self) -> I8Vector4: ...
    @property
    def ltot(self) -> I8Vector4: ...
    @property
    def ltoq(self) -> I8Vector4: ...
    @property
    def ltoo(self) -> I8Vector4: ...
    @property
    def ltol(self) -> I8Vector4: ...
    @property
    def ltls(self) -> I8Vector4: ...
    @property
    def ltlt(self) -> I8Vector4: ...
    @property
    def ltlq(self) -> I8Vector4: ...
    @property
    def ltlo(self) -> I8Vector4: ...
    @property
    def ltll(self) -> I8Vector4: ...
    @property
    def lqss(self) -> I8Vector4: ...
    @property
    def lqst(self) -> I8Vector4: ...
    @property
    def lqsq(self) -> I8Vector4: ...
    @property
    def lqso(self) -> I8Vector4: ...
    @property
    def lqsl(self) -> I8Vector4: ...
    @property
    def lqts(self) -> I8Vector4: ...
    @property
    def lqtt(self) -> I8Vector4: ...
    @property
    def lqtq(self) -> I8Vector4: ...
    @property
    def lqto(self) -> I8Vector4: ...
    @property
    def lqtl(self) -> I8Vector4: ...
    @property
    def lqqs(self) -> I8Vector4: ...
    @property
    def lqqt(self) -> I8Vector4: ...
    @property
    def lqqq(self) -> I8Vector4: ...
    @property
    def lqqo(self) -> I8Vector4: ...
    @property
    def lqql(self) -> I8Vector4: ...
    @property
    def lqos(self) -> I8Vector4: ...
    @property
    def lqot(self) -> I8Vector4: ...
    @property
    def lqoq(self) -> I8Vector4: ...
    @property
    def lqoo(self) -> I8Vector4: ...
    @property
    def lqol(self) -> I8Vector4: ...
    @property
    def lqls(self) -> I8Vector4: ...
    @property
    def lqlt(self) -> I8Vector4: ...
    @property
    def lqlq(self) -> I8Vector4: ...
    @property
    def lqlo(self) -> I8Vector4: ...
    @property
    def lqll(self) -> I8Vector4: ...
    @property
    def loss(self) -> I8Vector4: ...
    @property
    def lost(self) -> I8Vector4: ...
    @property
    def losq(self) -> I8Vector4: ...
    @property
    def loso(self) -> I8Vector4: ...
    @property
    def losl(self) -> I8Vector4: ...
    @property
    def lots(self) -> I8Vector4: ...
    @property
    def lott(self) -> I8Vector4: ...
    @property
    def lotq(self) -> I8Vector4: ...
    @property
    def loto(self) -> I8Vector4: ...
    @property
    def lotl(self) -> I8Vector4: ...
    @property
    def loqs(self) -> I8Vector4: ...
    @property
    def loqt(self) -> I8Vector4: ...
    @property
    def loqq(self) -> I8Vector4: ...
    @property
    def loqo(self) -> I8Vector4: ...
    @property
    def loql(self) -> I8Vector4: ...
    @property
    def loos(self) -> I8Vector4: ...
    @property
    def loot(self) -> I8Vector4: ...
    @property
    def looq(self) -> I8Vector4: ...
    @property
    def lols(self) -> I8Vector4: ...
    @property
    def lolt(self) -> I8Vector4: ...
    @property
    def lolq(self) -> I8Vector4: ...
    @property
    def llss(self) -> I8Vector4: ...
    @property
    def llst(self) -> I8Vector4: ...
    @property
    def llsq(self) -> I8Vector4: ...
    @property
    def llso(self) -> I8Vector4: ...
    @property
    def llsl(self) -> I8Vector4: ...
    @property
    def llts(self) -> I8Vector4: ...
    @property
    def lltt(self) -> I8Vector4: ...
    @property
    def lltq(self) -> I8Vector4: ...
    @property
    def llto(self) -> I8Vector4: ...
    @property
    def lltl(self) -> I8Vector4: ...
    @property
    def llqs(self) -> I8Vector4: ...
    @property
    def llqt(self) -> I8Vector4: ...
    @property
    def llqq(self) -> I8Vector4: ...
    @property
    def llqo(self) -> I8Vector4: ...
    @property
    def llql(self) -> I8Vector4: ...
    @property
    def llos(self) -> I8Vector4: ...
    @property
    def llot(self) -> I8Vector4: ...
    @property
    def lloq(self) -> I8Vector4: ...
    @property
    def llls(self) -> I8Vector4: ...
    @property
    def lllt(self) -> I8Vector4: ...
    @property
    def lllq(self) -> I8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I8Vector2: ...
    @property
    def uv(self) -> I8Vector2: ...
    @property
    def uo(self) -> I8Vector2: ...
    @property
    def ul(self) -> I8Vector2: ...
    @property
    def vu(self) -> I8Vector2: ...
    @property
    def vv(self) -> I8Vector2: ...
    @property
    def vo(self) -> I8Vector2: ...
    @property
    def vl(self) -> I8Vector2: ...
    @property
    def ou(self) -> I8Vector2: ...
    @property
    def ov(self) -> I8Vector2: ...
    @property
    def lu(self) -> I8Vector2: ...
    @property
    def lv(self) -> I8Vector2: ...
    @property
    def uuu(self) -> I8Vector3: ...
    @property
    def uuv(self) -> I8Vector3: ...
    @property
    def uuo(self) -> I8Vector3: ...
    @property
    def uul(self) -> I8Vector3: ...
    @property
    def uvu(self) -> I8Vector3: ...
    @property
    def uvv(self) -> I8Vector3: ...
    @property
    def uvo(self) -> I8Vector3: ...
    @property
    def uvl(self) -> I8Vector3: ...
    @property
    def uou(self) -> I8Vector3: ...
    @property
    def uov(self) -> I8Vector3: ...
    @property
    def uoo(self) -> I8Vector3: ...
    @property
    def uol(self) -> I8Vector3: ...
    @property
    def ulu(self) -> I8Vector3: ...
    @property
    def ulv(self) -> I8Vector3: ...
    @property
    def ulo(self) -> I8Vector3: ...
    @property
    def ull(self) -> I8Vector3: ...
    @property
    def vuu(self) -> I8Vector3: ...
    @property
    def vuv(self) -> I8Vector3: ...
    @property
    def vuo(self) -> I8Vector3: ...
    @property
    def vul(self) -> I8Vector3: ...
    @property
    def vvu(self) -> I8Vector3: ...
    @property
    def vvv(self) -> I8Vector3: ...
    @property
    def vvo(self) -> I8Vector3: ...
    @property
    def vvl(self) -> I8Vector3: ...
    @property
    def vou(self) -> I8Vector3: ...
    @property
    def vov(self) -> I8Vector3: ...
    @property
    def voo(self) -> I8Vector3: ...
    @property
    def vol(self) -> I8Vector3: ...
    @property
    def vlu(self) -> I8Vector3: ...
    @property
    def vlv(self) -> I8Vector3: ...
    @property
    def vlo(self) -> I8Vector3: ...
    @property
    def vll(self) -> I8Vector3: ...
    @property
    def ouu(self) -> I8Vector3: ...
    @property
    def ouv(self) -> I8Vector3: ...
    @property
    def ouo(self) -> I8Vector3: ...
    @property
    def oul(self) -> I8Vector3: ...
    @property
    def ovu(self) -> I8Vector3: ...
    @property
    def ovv(self) -> I8Vector3: ...
    @property
    def ovo(self) -> I8Vector3: ...
    @property
    def ovl(self) -> I8Vector3: ...
    @property
    def oou(self) -> I8Vector3: ...
    @property
    def oov(self) -> I8Vector3: ...
    @property
    def olu(self) -> I8Vector3: ...
    @property
    def olv(self) -> I8Vector3: ...
    @property
    def luu(self) -> I8Vector3: ...
    @property
    def luv(self) -> I8Vector3: ...
    @property
    def luo(self) -> I8Vector3: ...
    @property
    def lul(self) -> I8Vector3: ...
    @property
    def lvu(self) -> I8Vector3: ...
    @property
    def lvv(self) -> I8Vector3: ...
    @property
    def lvo(self) -> I8Vector3: ...
    @property
    def lvl(self) -> I8Vector3: ...
    @property
    def lou(self) -> I8Vector3: ...
    @property
    def lov(self) -> I8Vector3: ...
    @property
    def llu(self) -> I8Vector3: ...
    @property
    def llv(self) -> I8Vector3: ...
    @property
    def uuuu(self) -> I8Vector4: ...
    @property
    def uuuv(self) -> I8Vector4: ...
    @property
    def uuuo(self) -> I8Vector4: ...
    @property
    def uuul(self) -> I8Vector4: ...
    @property
    def uuvu(self) -> I8Vector4: ...
    @property
    def uuvv(self) -> I8Vector4: ...
    @property
    def uuvo(self) -> I8Vector4: ...
    @property
    def uuvl(self) -> I8Vector4: ...
    @property
    def uuou(self) -> I8Vector4: ...
    @property
    def uuov(self) -> I8Vector4: ...
    @property
    def uuoo(self) -> I8Vector4: ...
    @property
    def uuol(self) -> I8Vector4: ...
    @property
    def uulu(self) -> I8Vector4: ...
    @property
    def uulv(self) -> I8Vector4: ...
    @property
    def uulo(self) -> I8Vector4: ...
    @property
    def uull(self) -> I8Vector4: ...
    @property
    def uvuu(self) -> I8Vector4: ...
    @property
    def uvuv(self) -> I8Vector4: ...
    @property
    def uvuo(self) -> I8Vector4: ...
    @property
    def uvul(self) -> I8Vector4: ...
    @property
    def uvvu(self) -> I8Vector4: ...
    @property
    def uvvv(self) -> I8Vector4: ...
    @property
    def uvvo(self) -> I8Vector4: ...
    @property
    def uvvl(self) -> I8Vector4: ...
    @property
    def uvou(self) -> I8Vector4: ...
    @property
    def uvov(self) -> I8Vector4: ...
    @property
    def uvoo(self) -> I8Vector4: ...
    @property
    def uvol(self) -> I8Vector4: ...
    @property
    def uvlu(self) -> I8Vector4: ...
    @property
    def uvlv(self) -> I8Vector4: ...
    @property
    def uvlo(self) -> I8Vector4: ...
    @property
    def uvll(self) -> I8Vector4: ...
    @property
    def uouu(self) -> I8Vector4: ...
    @property
    def uouv(self) -> I8Vector4: ...
    @property
    def uouo(self) -> I8Vector4: ...
    @property
    def uoul(self) -> I8Vector4: ...
    @property
    def uovu(self) -> I8Vector4: ...
    @property
    def uovv(self) -> I8Vector4: ...
    @property
    def uovo(self) -> I8Vector4: ...
    @property
    def uovl(self) -> I8Vector4: ...
    @property
    def uoou(self) -> I8Vector4: ...
    @property
    def uoov(self) -> I8Vector4: ...
    @property
    def uooo(self) -> I8Vector4: ...
    @property
    def uool(self) -> I8Vector4: ...
    @property
    def uolu(self) -> I8Vector4: ...
    @property
    def uolv(self) -> I8Vector4: ...
    @property
    def uolo(self) -> I8Vector4: ...
    @property
    def uoll(self) -> I8Vector4: ...
    @property
    def uluu(self) -> I8Vector4: ...
    @property
    def uluv(self) -> I8Vector4: ...
    @property
    def uluo(self) -> I8Vector4: ...
    @property
    def ulul(self) -> I8Vector4: ...
    @property
    def ulvu(self) -> I8Vector4: ...
    @property
    def ulvv(self) -> I8Vector4: ...
    @property
    def ulvo(self) -> I8Vector4: ...
    @property
    def ulvl(self) -> I8Vector4: ...
    @property
    def ulou(self) -> I8Vector4: ...
    @property
    def ulov(self) -> I8Vector4: ...
    @property
    def uloo(self) -> I8Vector4: ...
    @property
    def ulol(self) -> I8Vector4: ...
    @property
    def ullu(self) -> I8Vector4: ...
    @property
    def ullv(self) -> I8Vector4: ...
    @property
    def ullo(self) -> I8Vector4: ...
    @property
    def ulll(self) -> I8Vector4: ...
    @property
    def vuuu(self) -> I8Vector4: ...
    @property
    def vuuv(self) -> I8Vector4: ...
    @property
    def vuuo(self) -> I8Vector4: ...
    @property
    def vuul(self) -> I8Vector4: ...
    @property
    def vuvu(self) -> I8Vector4: ...
    @property
    def vuvv(self) -> I8Vector4: ...
    @property
    def vuvo(self) -> I8Vector4: ...
    @property
    def vuvl(self) -> I8Vector4: ...
    @property
    def vuou(self) -> I8Vector4: ...
    @property
    def vuov(self) -> I8Vector4: ...
    @property
    def vuoo(self) -> I8Vector4: ...
    @property
    def vuol(self) -> I8Vector4: ...
    @property
    def vulu(self) -> I8Vector4: ...
    @property
    def vulv(self) -> I8Vector4: ...
    @property
    def vulo(self) -> I8Vector4: ...
    @property
    def vull(self) -> I8Vector4: ...
    @property
    def vvuu(self) -> I8Vector4: ...
    @property
    def vvuv(self) -> I8Vector4: ...
    @property
    def vvuo(self) -> I8Vector4: ...
    @property
    def vvul(self) -> I8Vector4: ...
    @property
    def vvvu(self) -> I8Vector4: ...
    @property
    def vvvv(self) -> I8Vector4: ...
    @property
    def vvvo(self) -> I8Vector4: ...
    @property
    def vvvl(self) -> I8Vector4: ...
    @property
    def vvou(self) -> I8Vector4: ...
    @property
    def vvov(self) -> I8Vector4: ...
    @property
    def vvoo(self) -> I8Vector4: ...
    @property
    def vvol(self) -> I8Vector4: ...
    @property
    def vvlu(self) -> I8Vector4: ...
    @property
    def vvlv(self) -> I8Vector4: ...
    @property
    def vvlo(self) -> I8Vector4: ...
    @property
    def vvll(self) -> I8Vector4: ...
    @property
    def vouu(self) -> I8Vector4: ...
    @property
    def vouv(self) -> I8Vector4: ...
    @property
    def vouo(self) -> I8Vector4: ...
    @property
    def voul(self) -> I8Vector4: ...
    @property
    def vovu(self) -> I8Vector4: ...
    @property
    def vovv(self) -> I8Vector4: ...
    @property
    def vovo(self) -> I8Vector4: ...
    @property
    def vovl(self) -> I8Vector4: ...
    @property
    def voou(self) -> I8Vector4: ...
    @property
    def voov(self) -> I8Vector4: ...
    @property
    def vooo(self) -> I8Vector4: ...
    @property
    def vool(self) -> I8Vector4: ...
    @property
    def volu(self) -> I8Vector4: ...
    @property
    def volv(self) -> I8Vector4: ...
    @property
    def volo(self) -> I8Vector4: ...
    @property
    def voll(self) -> I8Vector4: ...
    @property
    def vluu(self) -> I8Vector4: ...
    @property
    def vluv(self) -> I8Vector4: ...
    @property
    def vluo(self) -> I8Vector4: ...
    @property
    def vlul(self) -> I8Vector4: ...
    @property
    def vlvu(self) -> I8Vector4: ...
    @property
    def vlvv(self) -> I8Vector4: ...
    @property
    def vlvo(self) -> I8Vector4: ...
    @property
    def vlvl(self) -> I8Vector4: ...
    @property
    def vlou(self) -> I8Vector4: ...
    @property
    def vlov(self) -> I8Vector4: ...
    @property
    def vloo(self) -> I8Vector4: ...
    @property
    def vlol(self) -> I8Vector4: ...
    @property
    def vllu(self) -> I8Vector4: ...
    @property
    def vllv(self) -> I8Vector4: ...
    @property
    def vllo(self) -> I8Vector4: ...
    @property
    def vlll(self) -> I8Vector4: ...
    @property
    def ouuu(self) -> I8Vector4: ...
    @property
    def ouuv(self) -> I8Vector4: ...
    @property
    def ouuo(self) -> I8Vector4: ...
    @property
    def ouul(self) -> I8Vector4: ...
    @property
    def ouvu(self) -> I8Vector4: ...
    @property
    def ouvv(self) -> I8Vector4: ...
    @property
    def ouvo(self) -> I8Vector4: ...
    @property
    def ouvl(self) -> I8Vector4: ...
    @property
    def ouou(self) -> I8Vector4: ...
    @property
    def ouov(self) -> I8Vector4: ...
    @property
    def ouoo(self) -> I8Vector4: ...
    @property
    def ouol(self) -> I8Vector4: ...
    @property
    def oulu(self) -> I8Vector4: ...
    @property
    def oulv(self) -> I8Vector4: ...
    @property
    def oulo(self) -> I8Vector4: ...
    @property
    def oull(self) -> I8Vector4: ...
    @property
    def ovuu(self) -> I8Vector4: ...
    @property
    def ovuv(self) -> I8Vector4: ...
    @property
    def ovuo(self) -> I8Vector4: ...
    @property
    def ovul(self) -> I8Vector4: ...
    @property
    def ovvu(self) -> I8Vector4: ...
    @property
    def ovvv(self) -> I8Vector4: ...
    @property
    def ovvo(self) -> I8Vector4: ...
    @property
    def ovvl(self) -> I8Vector4: ...
    @property
    def ovou(self) -> I8Vector4: ...
    @property
    def ovov(self) -> I8Vector4: ...
    @property
    def ovoo(self) -> I8Vector4: ...
    @property
    def ovol(self) -> I8Vector4: ...
    @property
    def ovlu(self) -> I8Vector4: ...
    @property
    def ovlv(self) -> I8Vector4: ...
    @property
    def ovlo(self) -> I8Vector4: ...
    @property
    def ovll(self) -> I8Vector4: ...
    @property
    def oouu(self) -> I8Vector4: ...
    @property
    def oouv(self) -> I8Vector4: ...
    @property
    def oouo(self) -> I8Vector4: ...
    @property
    def ooul(self) -> I8Vector4: ...
    @property
    def oovu(self) -> I8Vector4: ...
    @property
    def oovv(self) -> I8Vector4: ...
    @property
    def oovo(self) -> I8Vector4: ...
    @property
    def oovl(self) -> I8Vector4: ...
    @property
    def ooou(self) -> I8Vector4: ...
    @property
    def ooov(self) -> I8Vector4: ...
    @property
    def oolu(self) -> I8Vector4: ...
    @property
    def oolv(self) -> I8Vector4: ...
    @property
    def oluu(self) -> I8Vector4: ...
    @property
    def oluv(self) -> I8Vector4: ...
    @property
    def oluo(self) -> I8Vector4: ...
    @property
    def olul(self) -> I8Vector4: ...
    @property
    def olvu(self) -> I8Vector4: ...
    @property
    def olvv(self) -> I8Vector4: ...
    @property
    def olvo(self) -> I8Vector4: ...
    @property
    def olvl(self) -> I8Vector4: ...
    @property
    def olou(self) -> I8Vector4: ...
    @property
    def olov(self) -> I8Vector4: ...
    @property
    def ollu(self) -> I8Vector4: ...
    @property
    def ollv(self) -> I8Vector4: ...
    @property
    def luuu(self) -> I8Vector4: ...
    @property
    def luuv(self) -> I8Vector4: ...
    @property
    def luuo(self) -> I8Vector4: ...
    @property
    def luul(self) -> I8Vector4: ...
    @property
    def luvu(self) -> I8Vector4: ...
    @property
    def luvv(self) -> I8Vector4: ...
    @property
    def luvo(self) -> I8Vector4: ...
    @property
    def luvl(self) -> I8Vector4: ...
    @property
    def luou(self) -> I8Vector4: ...
    @property
    def luov(self) -> I8Vector4: ...
    @property
    def luoo(self) -> I8Vector4: ...
    @property
    def luol(self) -> I8Vector4: ...
    @property
    def lulu(self) -> I8Vector4: ...
    @property
    def lulv(self) -> I8Vector4: ...
    @property
    def lulo(self) -> I8Vector4: ...
    @property
    def lull(self) -> I8Vector4: ...
    @property
    def lvuu(self) -> I8Vector4: ...
    @property
    def lvuv(self) -> I8Vector4: ...
    @property
    def lvuo(self) -> I8Vector4: ...
    @property
    def lvul(self) -> I8Vector4: ...
    @property
    def lvvu(self) -> I8Vector4: ...
    @property
    def lvvv(self) -> I8Vector4: ...
    @property
    def lvvo(self) -> I8Vector4: ...
    @property
    def lvvl(self) -> I8Vector4: ...
    @property
    def lvou(self) -> I8Vector4: ...
    @property
    def lvov(self) -> I8Vector4: ...
    @property
    def lvoo(self) -> I8Vector4: ...
    @property
    def lvol(self) -> I8Vector4: ...
    @property
    def lvlu(self) -> I8Vector4: ...
    @property
    def lvlv(self) -> I8Vector4: ...
    @property
    def lvlo(self) -> I8Vector4: ...
    @property
    def lvll(self) -> I8Vector4: ...
    @property
    def louu(self) -> I8Vector4: ...
    @property
    def louv(self) -> I8Vector4: ...
    @property
    def louo(self) -> I8Vector4: ...
    @property
    def loul(self) -> I8Vector4: ...
    @property
    def lovu(self) -> I8Vector4: ...
    @property
    def lovv(self) -> I8Vector4: ...
    @property
    def lovo(self) -> I8Vector4: ...
    @property
    def lovl(self) -> I8Vector4: ...
    @property
    def loou(self) -> I8Vector4: ...
    @property
    def loov(self) -> I8Vector4: ...
    @property
    def lolu(self) -> I8Vector4: ...
    @property
    def lolv(self) -> I8Vector4: ...
    @property
    def lluu(self) -> I8Vector4: ...
    @property
    def lluv(self) -> I8Vector4: ...
    @property
    def lluo(self) -> I8Vector4: ...
    @property
    def llul(self) -> I8Vector4: ...
    @property
    def llvu(self) -> I8Vector4: ...
    @property
    def llvv(self) -> I8Vector4: ...
    @property
    def llvo(self) -> I8Vector4: ...
    @property
    def llvl(self) -> I8Vector4: ...
    @property
    def llou(self) -> I8Vector4: ...
    @property
    def llov(self) -> I8Vector4: ...
    @property
    def lllu(self) -> I8Vector4: ...
    @property
    def lllv(self) -> I8Vector4: ...
    @property
    def oo(self) -> I8Vector2: ...
    @property
    def ol(self) -> I8Vector2: ...
    @property
    def lo(self) -> I8Vector2: ...
    @property
    def ll(self) -> I8Vector2: ...
    @property
    def ooo(self) -> I8Vector3: ...
    @property
    def ool(self) -> I8Vector3: ...
    @property
    def olo(self) -> I8Vector3: ...
    @property
    def oll(self) -> I8Vector3: ...
    @property
    def loo(self) -> I8Vector3: ...
    @property
    def lol(self) -> I8Vector3: ...
    @property
    def llo(self) -> I8Vector3: ...
    @property
    def lll(self) -> I8Vector3: ...
    @property
    def oooo(self) -> I8Vector4: ...
    @property
    def oool(self) -> I8Vector4: ...
    @property
    def oolo(self) -> I8Vector4: ...
    @property
    def ooll(self) -> I8Vector4: ...
    @property
    def oloo(self) -> I8Vector4: ...
    @property
    def olol(self) -> I8Vector4: ...
    @property
    def ollo(self) -> I8Vector4: ...
    @property
    def olll(self) -> I8Vector4: ...
    @property
    def looo(self) -> I8Vector4: ...
    @property
    def lool(self) -> I8Vector4: ...
    @property
    def lolo(self) -> I8Vector4: ...
    @property
    def loll(self) -> I8Vector4: ...
    @property
    def lloo(self) -> I8Vector4: ...
    @property
    def llol(self) -> I8Vector4: ...
    @property
    def lllo(self) -> I8Vector4: ...
    @property
    def llll(self) -> I8Vector4: ...
    def min(self, n: Number, /) -> I8Vector3: ...
    def max(self, n: Number, /) -> I8Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> I8Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I8Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector3: ...

@final
class I8Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I8Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I8Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> I8Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> I8Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[I8Vector3]: ...

@final
class U8Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __add__(self, other: Number) -> U8Vector3: ...
    @overload
    def __radd__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __radd__(self, other: Number) -> U8Vector3: ...
    @overload
    def __sub__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __sub__(self, other: Number) -> U8Vector3: ...
    @overload
    def __rsub__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> U8Vector3: ...
    @overload
    def __mul__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __mul__(self, other: Number) -> U8Vector3: ...
    @overload
    def __rmul__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> U8Vector3: ...
    @overload
    def __truediv__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> U8Vector3: ...
    @overload
    def __rtruediv__(self, other: U8Vector3) -> U8Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> U8Vector3: ...
    def __abs__(self) -> U8Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> U8Vector2: ...
    @property
    def xy(self) -> U8Vector2: ...
    @property
    def xz(self) -> U8Vector2: ...
    @property
    def xo(self) -> U8Vector2: ...
    @property
    def xl(self) -> U8Vector2: ...
    @property
    def yx(self) -> U8Vector2: ...
    @property
    def yy(self) -> U8Vector2: ...
    @property
    def yz(self) -> U8Vector2: ...
    @property
    def yo(self) -> U8Vector2: ...
    @property
    def yl(self) -> U8Vector2: ...
    @property
    def zx(self) -> U8Vector2: ...
    @property
    def zy(self) -> U8Vector2: ...
    @property
    def zz(self) -> U8Vector2: ...
    @property
    def zo(self) -> U8Vector2: ...
    @property
    def zl(self) -> U8Vector2: ...
    @property
    def ox(self) -> U8Vector2: ...
    @property
    def oy(self) -> U8Vector2: ...
    @property
    def oz(self) -> U8Vector2: ...
    @property
    def lx(self) -> U8Vector2: ...
    @property
    def ly(self) -> U8Vector2: ...
    @property
    def lz(self) -> U8Vector2: ...
    @property
    def xxx(self) -> U8Vector3: ...
    @property
    def xxy(self) -> U8Vector3: ...
    @property
    def xxz(self) -> U8Vector3: ...
    @property
    def xxo(self) -> U8Vector3: ...
    @property
    def xxl(self) -> U8Vector3: ...
    @property
    def xyx(self) -> U8Vector3: ...
    @property
    def xyy(self) -> U8Vector3: ...
    @property
    def xyz(self) -> U8Vector3: ...
    @property
    def xyo(self) -> U8Vector3: ...
    @property
    def xyl(self) -> U8Vector3: ...
    @property
    def xzx(self) -> U8Vector3: ...
    @property
    def xzy(self) -> U8Vector3: ...
    @property
    def xzz(self) -> U8Vector3: ...
    @property
    def xzo(self) -> U8Vector3: ...
    @property
    def xzl(self) -> U8Vector3: ...
    @property
    def xox(self) -> U8Vector3: ...
    @property
    def xoy(self) -> U8Vector3: ...
    @property
    def xoz(self) -> U8Vector3: ...
    @property
    def xoo(self) -> U8Vector3: ...
    @property
    def xol(self) -> U8Vector3: ...
    @property
    def xlx(self) -> U8Vector3: ...
    @property
    def xly(self) -> U8Vector3: ...
    @property
    def xlz(self) -> U8Vector3: ...
    @property
    def xlo(self) -> U8Vector3: ...
    @property
    def xll(self) -> U8Vector3: ...
    @property
    def yxx(self) -> U8Vector3: ...
    @property
    def yxy(self) -> U8Vector3: ...
    @property
    def yxz(self) -> U8Vector3: ...
    @property
    def yxo(self) -> U8Vector3: ...
    @property
    def yxl(self) -> U8Vector3: ...
    @property
    def yyx(self) -> U8Vector3: ...
    @property
    def yyy(self) -> U8Vector3: ...
    @property
    def yyz(self) -> U8Vector3: ...
    @property
    def yyo(self) -> U8Vector3: ...
    @property
    def yyl(self) -> U8Vector3: ...
    @property
    def yzx(self) -> U8Vector3: ...
    @property
    def yzy(self) -> U8Vector3: ...
    @property
    def yzz(self) -> U8Vector3: ...
    @property
    def yzo(self) -> U8Vector3: ...
    @property
    def yzl(self) -> U8Vector3: ...
    @property
    def yox(self) -> U8Vector3: ...
    @property
    def yoy(self) -> U8Vector3: ...
    @property
    def yoz(self) -> U8Vector3: ...
    @property
    def yoo(self) -> U8Vector3: ...
    @property
    def yol(self) -> U8Vector3: ...
    @property
    def ylx(self) -> U8Vector3: ...
    @property
    def yly(self) -> U8Vector3: ...
    @property
    def ylz(self) -> U8Vector3: ...
    @property
    def ylo(self) -> U8Vector3: ...
    @property
    def yll(self) -> U8Vector3: ...
    @property
    def zxx(self) -> U8Vector3: ...
    @property
    def zxy(self) -> U8Vector3: ...
    @property
    def zxz(self) -> U8Vector3: ...
    @property
    def zxo(self) -> U8Vector3: ...
    @property
    def zxl(self) -> U8Vector3: ...
    @property
    def zyx(self) -> U8Vector3: ...
    @property
    def zyy(self) -> U8Vector3: ...
    @property
    def zyz(self) -> U8Vector3: ...
    @property
    def zyo(self) -> U8Vector3: ...
    @property
    def zyl(self) -> U8Vector3: ...
    @property
    def zzx(self) -> U8Vector3: ...
    @property
    def zzy(self) -> U8Vector3: ...
    @property
    def zzz(self) -> U8Vector3: ...
    @property
    def zzo(self) -> U8Vector3: ...
    @property
    def zzl(self) -> U8Vector3: ...
    @property
    def zox(self) -> U8Vector3: ...
    @property
    def zoy(self) -> U8Vector3: ...
    @property
    def zoz(self) -> U8Vector3: ...
    @property
    def zoo(self) -> U8Vector3: ...
    @property
    def zol(self) -> U8Vector3: ...
    @property
    def zlx(self) -> U8Vector3: ...
    @property
    def zly(self) -> U8Vector3: ...
    @property
    def zlz(self) -> U8Vector3: ...
    @property
    def zlo(self) -> U8Vector3: ...
    @property
    def zll(self) -> U8Vector3: ...
    @property
    def oxx(self) -> U8Vector3: ...
    @property
    def oxy(self) -> U8Vector3: ...
    @property
    def oxz(self) -> U8Vector3: ...
    @property
    def oxo(self) -> U8Vector3: ...
    @property
    def oxl(self) -> U8Vector3: ...
    @property
    def oyx(self) -> U8Vector3: ...
    @property
    def oyy(self) -> U8Vector3: ...
    @property
    def oyz(self) -> U8Vector3: ...
    @property
    def oyo(self) -> U8Vector3: ...
    @property
    def oyl(self) -> U8Vector3: ...
    @property
    def ozx(self) -> U8Vector3: ...
    @property
    def ozy(self) -> U8Vector3: ...
    @property
    def ozz(self) -> U8Vector3: ...
    @property
    def ozo(self) -> U8Vector3: ...
    @property
    def ozl(self) -> U8Vector3: ...
    @property
    def oox(self) -> U8Vector3: ...
    @property
    def ooy(self) -> U8Vector3: ...
    @property
    def ooz(self) -> U8Vector3: ...
    @property
    def olx(self) -> U8Vector3: ...
    @property
    def oly(self) -> U8Vector3: ...
    @property
    def olz(self) -> U8Vector3: ...
    @property
    def lxx(self) -> U8Vector3: ...
    @property
    def lxy(self) -> U8Vector3: ...
    @property
    def lxz(self) -> U8Vector3: ...
    @property
    def lxo(self) -> U8Vector3: ...
    @property
    def lxl(self) -> U8Vector3: ...
    @property
    def lyx(self) -> U8Vector3: ...
    @property
    def lyy(self) -> U8Vector3: ...
    @property
    def lyz(self) -> U8Vector3: ...
    @property
    def lyo(self) -> U8Vector3: ...
    @property
    def lyl(self) -> U8Vector3: ...
    @property
    def lzx(self) -> U8Vector3: ...
    @property
    def lzy(self) -> U8Vector3: ...
    @property
    def lzz(self) -> U8Vector3: ...
    @property
    def lzo(self) -> U8Vector3: ...
    @property
    def lzl(self) -> U8Vector3: ...
    @property
    def lox(self) -> U8Vector3: ...
    @property
    def loy(self) -> U8Vector3: ...
    @property
    def loz(self) -> U8Vector3: ...
    @property
    def llx(self) -> U8Vector3: ...
    @property
    def lly(self) -> U8Vector3: ...
    @property
    def llz(self) -> U8Vector3: ...
    @property
    def xxxx(self) -> U8Vector4: ...
    @property
    def xxxy(self) -> U8Vector4: ...
    @property
    def xxxz(self) -> U8Vector4: ...
    @property
    def xxxo(self) -> U8Vector4: ...
    @property
    def xxxl(self) -> U8Vector4: ...
    @property
    def xxyx(self) -> U8Vector4: ...
    @property
    def xxyy(self) -> U8Vector4: ...
    @property
    def xxyz(self) -> U8Vector4: ...
    @property
    def xxyo(self) -> U8Vector4: ...
    @property
    def xxyl(self) -> U8Vector4: ...
    @property
    def xxzx(self) -> U8Vector4: ...
    @property
    def xxzy(self) -> U8Vector4: ...
    @property
    def xxzz(self) -> U8Vector4: ...
    @property
    def xxzo(self) -> U8Vector4: ...
    @property
    def xxzl(self) -> U8Vector4: ...
    @property
    def xxox(self) -> U8Vector4: ...
    @property
    def xxoy(self) -> U8Vector4: ...
    @property
    def xxoz(self) -> U8Vector4: ...
    @property
    def xxoo(self) -> U8Vector4: ...
    @property
    def xxol(self) -> U8Vector4: ...
    @property
    def xxlx(self) -> U8Vector4: ...
    @property
    def xxly(self) -> U8Vector4: ...
    @property
    def xxlz(self) -> U8Vector4: ...
    @property
    def xxlo(self) -> U8Vector4: ...
    @property
    def xxll(self) -> U8Vector4: ...
    @property
    def xyxx(self) -> U8Vector4: ...
    @property
    def xyxy(self) -> U8Vector4: ...
    @property
    def xyxz(self) -> U8Vector4: ...
    @property
    def xyxo(self) -> U8Vector4: ...
    @property
    def xyxl(self) -> U8Vector4: ...
    @property
    def xyyx(self) -> U8Vector4: ...
    @property
    def xyyy(self) -> U8Vector4: ...
    @property
    def xyyz(self) -> U8Vector4: ...
    @property
    def xyyo(self) -> U8Vector4: ...
    @property
    def xyyl(self) -> U8Vector4: ...
    @property
    def xyzx(self) -> U8Vector4: ...
    @property
    def xyzy(self) -> U8Vector4: ...
    @property
    def xyzz(self) -> U8Vector4: ...
    @property
    def xyzo(self) -> U8Vector4: ...
    @property
    def xyzl(self) -> U8Vector4: ...
    @property
    def xyox(self) -> U8Vector4: ...
    @property
    def xyoy(self) -> U8Vector4: ...
    @property
    def xyoz(self) -> U8Vector4: ...
    @property
    def xyoo(self) -> U8Vector4: ...
    @property
    def xyol(self) -> U8Vector4: ...
    @property
    def xylx(self) -> U8Vector4: ...
    @property
    def xyly(self) -> U8Vector4: ...
    @property
    def xylz(self) -> U8Vector4: ...
    @property
    def xylo(self) -> U8Vector4: ...
    @property
    def xyll(self) -> U8Vector4: ...
    @property
    def xzxx(self) -> U8Vector4: ...
    @property
    def xzxy(self) -> U8Vector4: ...
    @property
    def xzxz(self) -> U8Vector4: ...
    @property
    def xzxo(self) -> U8Vector4: ...
    @property
    def xzxl(self) -> U8Vector4: ...
    @property
    def xzyx(self) -> U8Vector4: ...
    @property
    def xzyy(self) -> U8Vector4: ...
    @property
    def xzyz(self) -> U8Vector4: ...
    @property
    def xzyo(self) -> U8Vector4: ...
    @property
    def xzyl(self) -> U8Vector4: ...
    @property
    def xzzx(self) -> U8Vector4: ...
    @property
    def xzzy(self) -> U8Vector4: ...
    @property
    def xzzz(self) -> U8Vector4: ...
    @property
    def xzzo(self) -> U8Vector4: ...
    @property
    def xzzl(self) -> U8Vector4: ...
    @property
    def xzox(self) -> U8Vector4: ...
    @property
    def xzoy(self) -> U8Vector4: ...
    @property
    def xzoz(self) -> U8Vector4: ...
    @property
    def xzoo(self) -> U8Vector4: ...
    @property
    def xzol(self) -> U8Vector4: ...
    @property
    def xzlx(self) -> U8Vector4: ...
    @property
    def xzly(self) -> U8Vector4: ...
    @property
    def xzlz(self) -> U8Vector4: ...
    @property
    def xzlo(self) -> U8Vector4: ...
    @property
    def xzll(self) -> U8Vector4: ...
    @property
    def xoxx(self) -> U8Vector4: ...
    @property
    def xoxy(self) -> U8Vector4: ...
    @property
    def xoxz(self) -> U8Vector4: ...
    @property
    def xoxo(self) -> U8Vector4: ...
    @property
    def xoxl(self) -> U8Vector4: ...
    @property
    def xoyx(self) -> U8Vector4: ...
    @property
    def xoyy(self) -> U8Vector4: ...
    @property
    def xoyz(self) -> U8Vector4: ...
    @property
    def xoyo(self) -> U8Vector4: ...
    @property
    def xoyl(self) -> U8Vector4: ...
    @property
    def xozx(self) -> U8Vector4: ...
    @property
    def xozy(self) -> U8Vector4: ...
    @property
    def xozz(self) -> U8Vector4: ...
    @property
    def xozo(self) -> U8Vector4: ...
    @property
    def xozl(self) -> U8Vector4: ...
    @property
    def xoox(self) -> U8Vector4: ...
    @property
    def xooy(self) -> U8Vector4: ...
    @property
    def xooz(self) -> U8Vector4: ...
    @property
    def xooo(self) -> U8Vector4: ...
    @property
    def xool(self) -> U8Vector4: ...
    @property
    def xolx(self) -> U8Vector4: ...
    @property
    def xoly(self) -> U8Vector4: ...
    @property
    def xolz(self) -> U8Vector4: ...
    @property
    def xolo(self) -> U8Vector4: ...
    @property
    def xoll(self) -> U8Vector4: ...
    @property
    def xlxx(self) -> U8Vector4: ...
    @property
    def xlxy(self) -> U8Vector4: ...
    @property
    def xlxz(self) -> U8Vector4: ...
    @property
    def xlxo(self) -> U8Vector4: ...
    @property
    def xlxl(self) -> U8Vector4: ...
    @property
    def xlyx(self) -> U8Vector4: ...
    @property
    def xlyy(self) -> U8Vector4: ...
    @property
    def xlyz(self) -> U8Vector4: ...
    @property
    def xlyo(self) -> U8Vector4: ...
    @property
    def xlyl(self) -> U8Vector4: ...
    @property
    def xlzx(self) -> U8Vector4: ...
    @property
    def xlzy(self) -> U8Vector4: ...
    @property
    def xlzz(self) -> U8Vector4: ...
    @property
    def xlzo(self) -> U8Vector4: ...
    @property
    def xlzl(self) -> U8Vector4: ...
    @property
    def xlox(self) -> U8Vector4: ...
    @property
    def xloy(self) -> U8Vector4: ...
    @property
    def xloz(self) -> U8Vector4: ...
    @property
    def xloo(self) -> U8Vector4: ...
    @property
    def xlol(self) -> U8Vector4: ...
    @property
    def xllx(self) -> U8Vector4: ...
    @property
    def xlly(self) -> U8Vector4: ...
    @property
    def xllz(self) -> U8Vector4: ...
    @property
    def xllo(self) -> U8Vector4: ...
    @property
    def xlll(self) -> U8Vector4: ...
    @property
    def yxxx(self) -> U8Vector4: ...
    @property
    def yxxy(self) -> U8Vector4: ...
    @property
    def yxxz(self) -> U8Vector4: ...
    @property
    def yxxo(self) -> U8Vector4: ...
    @property
    def yxxl(self) -> U8Vector4: ...
    @property
    def yxyx(self) -> U8Vector4: ...
    @property
    def yxyy(self) -> U8Vector4: ...
    @property
    def yxyz(self) -> U8Vector4: ...
    @property
    def yxyo(self) -> U8Vector4: ...
    @property
    def yxyl(self) -> U8Vector4: ...
    @property
    def yxzx(self) -> U8Vector4: ...
    @property
    def yxzy(self) -> U8Vector4: ...
    @property
    def yxzz(self) -> U8Vector4: ...
    @property
    def yxzo(self) -> U8Vector4: ...
    @property
    def yxzl(self) -> U8Vector4: ...
    @property
    def yxox(self) -> U8Vector4: ...
    @property
    def yxoy(self) -> U8Vector4: ...
    @property
    def yxoz(self) -> U8Vector4: ...
    @property
    def yxoo(self) -> U8Vector4: ...
    @property
    def yxol(self) -> U8Vector4: ...
    @property
    def yxlx(self) -> U8Vector4: ...
    @property
    def yxly(self) -> U8Vector4: ...
    @property
    def yxlz(self) -> U8Vector4: ...
    @property
    def yxlo(self) -> U8Vector4: ...
    @property
    def yxll(self) -> U8Vector4: ...
    @property
    def yyxx(self) -> U8Vector4: ...
    @property
    def yyxy(self) -> U8Vector4: ...
    @property
    def yyxz(self) -> U8Vector4: ...
    @property
    def yyxo(self) -> U8Vector4: ...
    @property
    def yyxl(self) -> U8Vector4: ...
    @property
    def yyyx(self) -> U8Vector4: ...
    @property
    def yyyy(self) -> U8Vector4: ...
    @property
    def yyyz(self) -> U8Vector4: ...
    @property
    def yyyo(self) -> U8Vector4: ...
    @property
    def yyyl(self) -> U8Vector4: ...
    @property
    def yyzx(self) -> U8Vector4: ...
    @property
    def yyzy(self) -> U8Vector4: ...
    @property
    def yyzz(self) -> U8Vector4: ...
    @property
    def yyzo(self) -> U8Vector4: ...
    @property
    def yyzl(self) -> U8Vector4: ...
    @property
    def yyox(self) -> U8Vector4: ...
    @property
    def yyoy(self) -> U8Vector4: ...
    @property
    def yyoz(self) -> U8Vector4: ...
    @property
    def yyoo(self) -> U8Vector4: ...
    @property
    def yyol(self) -> U8Vector4: ...
    @property
    def yylx(self) -> U8Vector4: ...
    @property
    def yyly(self) -> U8Vector4: ...
    @property
    def yylz(self) -> U8Vector4: ...
    @property
    def yylo(self) -> U8Vector4: ...
    @property
    def yyll(self) -> U8Vector4: ...
    @property
    def yzxx(self) -> U8Vector4: ...
    @property
    def yzxy(self) -> U8Vector4: ...
    @property
    def yzxz(self) -> U8Vector4: ...
    @property
    def yzxo(self) -> U8Vector4: ...
    @property
    def yzxl(self) -> U8Vector4: ...
    @property
    def yzyx(self) -> U8Vector4: ...
    @property
    def yzyy(self) -> U8Vector4: ...
    @property
    def yzyz(self) -> U8Vector4: ...
    @property
    def yzyo(self) -> U8Vector4: ...
    @property
    def yzyl(self) -> U8Vector4: ...
    @property
    def yzzx(self) -> U8Vector4: ...
    @property
    def yzzy(self) -> U8Vector4: ...
    @property
    def yzzz(self) -> U8Vector4: ...
    @property
    def yzzo(self) -> U8Vector4: ...
    @property
    def yzzl(self) -> U8Vector4: ...
    @property
    def yzox(self) -> U8Vector4: ...
    @property
    def yzoy(self) -> U8Vector4: ...
    @property
    def yzoz(self) -> U8Vector4: ...
    @property
    def yzoo(self) -> U8Vector4: ...
    @property
    def yzol(self) -> U8Vector4: ...
    @property
    def yzlx(self) -> U8Vector4: ...
    @property
    def yzly(self) -> U8Vector4: ...
    @property
    def yzlz(self) -> U8Vector4: ...
    @property
    def yzlo(self) -> U8Vector4: ...
    @property
    def yzll(self) -> U8Vector4: ...
    @property
    def yoxx(self) -> U8Vector4: ...
    @property
    def yoxy(self) -> U8Vector4: ...
    @property
    def yoxz(self) -> U8Vector4: ...
    @property
    def yoxo(self) -> U8Vector4: ...
    @property
    def yoxl(self) -> U8Vector4: ...
    @property
    def yoyx(self) -> U8Vector4: ...
    @property
    def yoyy(self) -> U8Vector4: ...
    @property
    def yoyz(self) -> U8Vector4: ...
    @property
    def yoyo(self) -> U8Vector4: ...
    @property
    def yoyl(self) -> U8Vector4: ...
    @property
    def yozx(self) -> U8Vector4: ...
    @property
    def yozy(self) -> U8Vector4: ...
    @property
    def yozz(self) -> U8Vector4: ...
    @property
    def yozo(self) -> U8Vector4: ...
    @property
    def yozl(self) -> U8Vector4: ...
    @property
    def yoox(self) -> U8Vector4: ...
    @property
    def yooy(self) -> U8Vector4: ...
    @property
    def yooz(self) -> U8Vector4: ...
    @property
    def yooo(self) -> U8Vector4: ...
    @property
    def yool(self) -> U8Vector4: ...
    @property
    def yolx(self) -> U8Vector4: ...
    @property
    def yoly(self) -> U8Vector4: ...
    @property
    def yolz(self) -> U8Vector4: ...
    @property
    def yolo(self) -> U8Vector4: ...
    @property
    def yoll(self) -> U8Vector4: ...
    @property
    def ylxx(self) -> U8Vector4: ...
    @property
    def ylxy(self) -> U8Vector4: ...
    @property
    def ylxz(self) -> U8Vector4: ...
    @property
    def ylxo(self) -> U8Vector4: ...
    @property
    def ylxl(self) -> U8Vector4: ...
    @property
    def ylyx(self) -> U8Vector4: ...
    @property
    def ylyy(self) -> U8Vector4: ...
    @property
    def ylyz(self) -> U8Vector4: ...
    @property
    def ylyo(self) -> U8Vector4: ...
    @property
    def ylyl(self) -> U8Vector4: ...
    @property
    def ylzx(self) -> U8Vector4: ...
    @property
    def ylzy(self) -> U8Vector4: ...
    @property
    def ylzz(self) -> U8Vector4: ...
    @property
    def ylzo(self) -> U8Vector4: ...
    @property
    def ylzl(self) -> U8Vector4: ...
    @property
    def ylox(self) -> U8Vector4: ...
    @property
    def yloy(self) -> U8Vector4: ...
    @property
    def yloz(self) -> U8Vector4: ...
    @property
    def yloo(self) -> U8Vector4: ...
    @property
    def ylol(self) -> U8Vector4: ...
    @property
    def yllx(self) -> U8Vector4: ...
    @property
    def ylly(self) -> U8Vector4: ...
    @property
    def yllz(self) -> U8Vector4: ...
    @property
    def yllo(self) -> U8Vector4: ...
    @property
    def ylll(self) -> U8Vector4: ...
    @property
    def zxxx(self) -> U8Vector4: ...
    @property
    def zxxy(self) -> U8Vector4: ...
    @property
    def zxxz(self) -> U8Vector4: ...
    @property
    def zxxo(self) -> U8Vector4: ...
    @property
    def zxxl(self) -> U8Vector4: ...
    @property
    def zxyx(self) -> U8Vector4: ...
    @property
    def zxyy(self) -> U8Vector4: ...
    @property
    def zxyz(self) -> U8Vector4: ...
    @property
    def zxyo(self) -> U8Vector4: ...
    @property
    def zxyl(self) -> U8Vector4: ...
    @property
    def zxzx(self) -> U8Vector4: ...
    @property
    def zxzy(self) -> U8Vector4: ...
    @property
    def zxzz(self) -> U8Vector4: ...
    @property
    def zxzo(self) -> U8Vector4: ...
    @property
    def zxzl(self) -> U8Vector4: ...
    @property
    def zxox(self) -> U8Vector4: ...
    @property
    def zxoy(self) -> U8Vector4: ...
    @property
    def zxoz(self) -> U8Vector4: ...
    @property
    def zxoo(self) -> U8Vector4: ...
    @property
    def zxol(self) -> U8Vector4: ...
    @property
    def zxlx(self) -> U8Vector4: ...
    @property
    def zxly(self) -> U8Vector4: ...
    @property
    def zxlz(self) -> U8Vector4: ...
    @property
    def zxlo(self) -> U8Vector4: ...
    @property
    def zxll(self) -> U8Vector4: ...
    @property
    def zyxx(self) -> U8Vector4: ...
    @property
    def zyxy(self) -> U8Vector4: ...
    @property
    def zyxz(self) -> U8Vector4: ...
    @property
    def zyxo(self) -> U8Vector4: ...
    @property
    def zyxl(self) -> U8Vector4: ...
    @property
    def zyyx(self) -> U8Vector4: ...
    @property
    def zyyy(self) -> U8Vector4: ...
    @property
    def zyyz(self) -> U8Vector4: ...
    @property
    def zyyo(self) -> U8Vector4: ...
    @property
    def zyyl(self) -> U8Vector4: ...
    @property
    def zyzx(self) -> U8Vector4: ...
    @property
    def zyzy(self) -> U8Vector4: ...
    @property
    def zyzz(self) -> U8Vector4: ...
    @property
    def zyzo(self) -> U8Vector4: ...
    @property
    def zyzl(self) -> U8Vector4: ...
    @property
    def zyox(self) -> U8Vector4: ...
    @property
    def zyoy(self) -> U8Vector4: ...
    @property
    def zyoz(self) -> U8Vector4: ...
    @property
    def zyoo(self) -> U8Vector4: ...
    @property
    def zyol(self) -> U8Vector4: ...
    @property
    def zylx(self) -> U8Vector4: ...
    @property
    def zyly(self) -> U8Vector4: ...
    @property
    def zylz(self) -> U8Vector4: ...
    @property
    def zylo(self) -> U8Vector4: ...
    @property
    def zyll(self) -> U8Vector4: ...
    @property
    def zzxx(self) -> U8Vector4: ...
    @property
    def zzxy(self) -> U8Vector4: ...
    @property
    def zzxz(self) -> U8Vector4: ...
    @property
    def zzxo(self) -> U8Vector4: ...
    @property
    def zzxl(self) -> U8Vector4: ...
    @property
    def zzyx(self) -> U8Vector4: ...
    @property
    def zzyy(self) -> U8Vector4: ...
    @property
    def zzyz(self) -> U8Vector4: ...
    @property
    def zzyo(self) -> U8Vector4: ...
    @property
    def zzyl(self) -> U8Vector4: ...
    @property
    def zzzx(self) -> U8Vector4: ...
    @property
    def zzzy(self) -> U8Vector4: ...
    @property
    def zzzz(self) -> U8Vector4: ...
    @property
    def zzzo(self) -> U8Vector4: ...
    @property
    def zzzl(self) -> U8Vector4: ...
    @property
    def zzox(self) -> U8Vector4: ...
    @property
    def zzoy(self) -> U8Vector4: ...
    @property
    def zzoz(self) -> U8Vector4: ...
    @property
    def zzoo(self) -> U8Vector4: ...
    @property
    def zzol(self) -> U8Vector4: ...
    @property
    def zzlx(self) -> U8Vector4: ...
    @property
    def zzly(self) -> U8Vector4: ...
    @property
    def zzlz(self) -> U8Vector4: ...
    @property
    def zzlo(self) -> U8Vector4: ...
    @property
    def zzll(self) -> U8Vector4: ...
    @property
    def zoxx(self) -> U8Vector4: ...
    @property
    def zoxy(self) -> U8Vector4: ...
    @property
    def zoxz(self) -> U8Vector4: ...
    @property
    def zoxo(self) -> U8Vector4: ...
    @property
    def zoxl(self) -> U8Vector4: ...
    @property
    def zoyx(self) -> U8Vector4: ...
    @property
    def zoyy(self) -> U8Vector4: ...
    @property
    def zoyz(self) -> U8Vector4: ...
    @property
    def zoyo(self) -> U8Vector4: ...
    @property
    def zoyl(self) -> U8Vector4: ...
    @property
    def zozx(self) -> U8Vector4: ...
    @property
    def zozy(self) -> U8Vector4: ...
    @property
    def zozz(self) -> U8Vector4: ...
    @property
    def zozo(self) -> U8Vector4: ...
    @property
    def zozl(self) -> U8Vector4: ...
    @property
    def zoox(self) -> U8Vector4: ...
    @property
    def zooy(self) -> U8Vector4: ...
    @property
    def zooz(self) -> U8Vector4: ...
    @property
    def zooo(self) -> U8Vector4: ...
    @property
    def zool(self) -> U8Vector4: ...
    @property
    def zolx(self) -> U8Vector4: ...
    @property
    def zoly(self) -> U8Vector4: ...
    @property
    def zolz(self) -> U8Vector4: ...
    @property
    def zolo(self) -> U8Vector4: ...
    @property
    def zoll(self) -> U8Vector4: ...
    @property
    def zlxx(self) -> U8Vector4: ...
    @property
    def zlxy(self) -> U8Vector4: ...
    @property
    def zlxz(self) -> U8Vector4: ...
    @property
    def zlxo(self) -> U8Vector4: ...
    @property
    def zlxl(self) -> U8Vector4: ...
    @property
    def zlyx(self) -> U8Vector4: ...
    @property
    def zlyy(self) -> U8Vector4: ...
    @property
    def zlyz(self) -> U8Vector4: ...
    @property
    def zlyo(self) -> U8Vector4: ...
    @property
    def zlyl(self) -> U8Vector4: ...
    @property
    def zlzx(self) -> U8Vector4: ...
    @property
    def zlzy(self) -> U8Vector4: ...
    @property
    def zlzz(self) -> U8Vector4: ...
    @property
    def zlzo(self) -> U8Vector4: ...
    @property
    def zlzl(self) -> U8Vector4: ...
    @property
    def zlox(self) -> U8Vector4: ...
    @property
    def zloy(self) -> U8Vector4: ...
    @property
    def zloz(self) -> U8Vector4: ...
    @property
    def zloo(self) -> U8Vector4: ...
    @property
    def zlol(self) -> U8Vector4: ...
    @property
    def zllx(self) -> U8Vector4: ...
    @property
    def zlly(self) -> U8Vector4: ...
    @property
    def zllz(self) -> U8Vector4: ...
    @property
    def zllo(self) -> U8Vector4: ...
    @property
    def zlll(self) -> U8Vector4: ...
    @property
    def oxxx(self) -> U8Vector4: ...
    @property
    def oxxy(self) -> U8Vector4: ...
    @property
    def oxxz(self) -> U8Vector4: ...
    @property
    def oxxo(self) -> U8Vector4: ...
    @property
    def oxxl(self) -> U8Vector4: ...
    @property
    def oxyx(self) -> U8Vector4: ...
    @property
    def oxyy(self) -> U8Vector4: ...
    @property
    def oxyz(self) -> U8Vector4: ...
    @property
    def oxyo(self) -> U8Vector4: ...
    @property
    def oxyl(self) -> U8Vector4: ...
    @property
    def oxzx(self) -> U8Vector4: ...
    @property
    def oxzy(self) -> U8Vector4: ...
    @property
    def oxzz(self) -> U8Vector4: ...
    @property
    def oxzo(self) -> U8Vector4: ...
    @property
    def oxzl(self) -> U8Vector4: ...
    @property
    def oxox(self) -> U8Vector4: ...
    @property
    def oxoy(self) -> U8Vector4: ...
    @property
    def oxoz(self) -> U8Vector4: ...
    @property
    def oxoo(self) -> U8Vector4: ...
    @property
    def oxol(self) -> U8Vector4: ...
    @property
    def oxlx(self) -> U8Vector4: ...
    @property
    def oxly(self) -> U8Vector4: ...
    @property
    def oxlz(self) -> U8Vector4: ...
    @property
    def oxlo(self) -> U8Vector4: ...
    @property
    def oxll(self) -> U8Vector4: ...
    @property
    def oyxx(self) -> U8Vector4: ...
    @property
    def oyxy(self) -> U8Vector4: ...
    @property
    def oyxz(self) -> U8Vector4: ...
    @property
    def oyxo(self) -> U8Vector4: ...
    @property
    def oyxl(self) -> U8Vector4: ...
    @property
    def oyyx(self) -> U8Vector4: ...
    @property
    def oyyy(self) -> U8Vector4: ...
    @property
    def oyyz(self) -> U8Vector4: ...
    @property
    def oyyo(self) -> U8Vector4: ...
    @property
    def oyyl(self) -> U8Vector4: ...
    @property
    def oyzx(self) -> U8Vector4: ...
    @property
    def oyzy(self) -> U8Vector4: ...
    @property
    def oyzz(self) -> U8Vector4: ...
    @property
    def oyzo(self) -> U8Vector4: ...
    @property
    def oyzl(self) -> U8Vector4: ...
    @property
    def oyox(self) -> U8Vector4: ...
    @property
    def oyoy(self) -> U8Vector4: ...
    @property
    def oyoz(self) -> U8Vector4: ...
    @property
    def oyoo(self) -> U8Vector4: ...
    @property
    def oyol(self) -> U8Vector4: ...
    @property
    def oylx(self) -> U8Vector4: ...
    @property
    def oyly(self) -> U8Vector4: ...
    @property
    def oylz(self) -> U8Vector4: ...
    @property
    def oylo(self) -> U8Vector4: ...
    @property
    def oyll(self) -> U8Vector4: ...
    @property
    def ozxx(self) -> U8Vector4: ...
    @property
    def ozxy(self) -> U8Vector4: ...
    @property
    def ozxz(self) -> U8Vector4: ...
    @property
    def ozxo(self) -> U8Vector4: ...
    @property
    def ozxl(self) -> U8Vector4: ...
    @property
    def ozyx(self) -> U8Vector4: ...
    @property
    def ozyy(self) -> U8Vector4: ...
    @property
    def ozyz(self) -> U8Vector4: ...
    @property
    def ozyo(self) -> U8Vector4: ...
    @property
    def ozyl(self) -> U8Vector4: ...
    @property
    def ozzx(self) -> U8Vector4: ...
    @property
    def ozzy(self) -> U8Vector4: ...
    @property
    def ozzz(self) -> U8Vector4: ...
    @property
    def ozzo(self) -> U8Vector4: ...
    @property
    def ozzl(self) -> U8Vector4: ...
    @property
    def ozox(self) -> U8Vector4: ...
    @property
    def ozoy(self) -> U8Vector4: ...
    @property
    def ozoz(self) -> U8Vector4: ...
    @property
    def ozoo(self) -> U8Vector4: ...
    @property
    def ozol(self) -> U8Vector4: ...
    @property
    def ozlx(self) -> U8Vector4: ...
    @property
    def ozly(self) -> U8Vector4: ...
    @property
    def ozlz(self) -> U8Vector4: ...
    @property
    def ozlo(self) -> U8Vector4: ...
    @property
    def ozll(self) -> U8Vector4: ...
    @property
    def ooxx(self) -> U8Vector4: ...
    @property
    def ooxy(self) -> U8Vector4: ...
    @property
    def ooxz(self) -> U8Vector4: ...
    @property
    def ooxo(self) -> U8Vector4: ...
    @property
    def ooxl(self) -> U8Vector4: ...
    @property
    def ooyx(self) -> U8Vector4: ...
    @property
    def ooyy(self) -> U8Vector4: ...
    @property
    def ooyz(self) -> U8Vector4: ...
    @property
    def ooyo(self) -> U8Vector4: ...
    @property
    def ooyl(self) -> U8Vector4: ...
    @property
    def oozx(self) -> U8Vector4: ...
    @property
    def oozy(self) -> U8Vector4: ...
    @property
    def oozz(self) -> U8Vector4: ...
    @property
    def oozo(self) -> U8Vector4: ...
    @property
    def oozl(self) -> U8Vector4: ...
    @property
    def ooox(self) -> U8Vector4: ...
    @property
    def oooy(self) -> U8Vector4: ...
    @property
    def oooz(self) -> U8Vector4: ...
    @property
    def oolx(self) -> U8Vector4: ...
    @property
    def ooly(self) -> U8Vector4: ...
    @property
    def oolz(self) -> U8Vector4: ...
    @property
    def olxx(self) -> U8Vector4: ...
    @property
    def olxy(self) -> U8Vector4: ...
    @property
    def olxz(self) -> U8Vector4: ...
    @property
    def olxo(self) -> U8Vector4: ...
    @property
    def olxl(self) -> U8Vector4: ...
    @property
    def olyx(self) -> U8Vector4: ...
    @property
    def olyy(self) -> U8Vector4: ...
    @property
    def olyz(self) -> U8Vector4: ...
    @property
    def olyo(self) -> U8Vector4: ...
    @property
    def olyl(self) -> U8Vector4: ...
    @property
    def olzx(self) -> U8Vector4: ...
    @property
    def olzy(self) -> U8Vector4: ...
    @property
    def olzz(self) -> U8Vector4: ...
    @property
    def olzo(self) -> U8Vector4: ...
    @property
    def olzl(self) -> U8Vector4: ...
    @property
    def olox(self) -> U8Vector4: ...
    @property
    def oloy(self) -> U8Vector4: ...
    @property
    def oloz(self) -> U8Vector4: ...
    @property
    def ollx(self) -> U8Vector4: ...
    @property
    def olly(self) -> U8Vector4: ...
    @property
    def ollz(self) -> U8Vector4: ...
    @property
    def lxxx(self) -> U8Vector4: ...
    @property
    def lxxy(self) -> U8Vector4: ...
    @property
    def lxxz(self) -> U8Vector4: ...
    @property
    def lxxo(self) -> U8Vector4: ...
    @property
    def lxxl(self) -> U8Vector4: ...
    @property
    def lxyx(self) -> U8Vector4: ...
    @property
    def lxyy(self) -> U8Vector4: ...
    @property
    def lxyz(self) -> U8Vector4: ...
    @property
    def lxyo(self) -> U8Vector4: ...
    @property
    def lxyl(self) -> U8Vector4: ...
    @property
    def lxzx(self) -> U8Vector4: ...
    @property
    def lxzy(self) -> U8Vector4: ...
    @property
    def lxzz(self) -> U8Vector4: ...
    @property
    def lxzo(self) -> U8Vector4: ...
    @property
    def lxzl(self) -> U8Vector4: ...
    @property
    def lxox(self) -> U8Vector4: ...
    @property
    def lxoy(self) -> U8Vector4: ...
    @property
    def lxoz(self) -> U8Vector4: ...
    @property
    def lxoo(self) -> U8Vector4: ...
    @property
    def lxol(self) -> U8Vector4: ...
    @property
    def lxlx(self) -> U8Vector4: ...
    @property
    def lxly(self) -> U8Vector4: ...
    @property
    def lxlz(self) -> U8Vector4: ...
    @property
    def lxlo(self) -> U8Vector4: ...
    @property
    def lxll(self) -> U8Vector4: ...
    @property
    def lyxx(self) -> U8Vector4: ...
    @property
    def lyxy(self) -> U8Vector4: ...
    @property
    def lyxz(self) -> U8Vector4: ...
    @property
    def lyxo(self) -> U8Vector4: ...
    @property
    def lyxl(self) -> U8Vector4: ...
    @property
    def lyyx(self) -> U8Vector4: ...
    @property
    def lyyy(self) -> U8Vector4: ...
    @property
    def lyyz(self) -> U8Vector4: ...
    @property
    def lyyo(self) -> U8Vector4: ...
    @property
    def lyyl(self) -> U8Vector4: ...
    @property
    def lyzx(self) -> U8Vector4: ...
    @property
    def lyzy(self) -> U8Vector4: ...
    @property
    def lyzz(self) -> U8Vector4: ...
    @property
    def lyzo(self) -> U8Vector4: ...
    @property
    def lyzl(self) -> U8Vector4: ...
    @property
    def lyox(self) -> U8Vector4: ...
    @property
    def lyoy(self) -> U8Vector4: ...
    @property
    def lyoz(self) -> U8Vector4: ...
    @property
    def lyoo(self) -> U8Vector4: ...
    @property
    def lyol(self) -> U8Vector4: ...
    @property
    def lylx(self) -> U8Vector4: ...
    @property
    def lyly(self) -> U8Vector4: ...
    @property
    def lylz(self) -> U8Vector4: ...
    @property
    def lylo(self) -> U8Vector4: ...
    @property
    def lyll(self) -> U8Vector4: ...
    @property
    def lzxx(self) -> U8Vector4: ...
    @property
    def lzxy(self) -> U8Vector4: ...
    @property
    def lzxz(self) -> U8Vector4: ...
    @property
    def lzxo(self) -> U8Vector4: ...
    @property
    def lzxl(self) -> U8Vector4: ...
    @property
    def lzyx(self) -> U8Vector4: ...
    @property
    def lzyy(self) -> U8Vector4: ...
    @property
    def lzyz(self) -> U8Vector4: ...
    @property
    def lzyo(self) -> U8Vector4: ...
    @property
    def lzyl(self) -> U8Vector4: ...
    @property
    def lzzx(self) -> U8Vector4: ...
    @property
    def lzzy(self) -> U8Vector4: ...
    @property
    def lzzz(self) -> U8Vector4: ...
    @property
    def lzzo(self) -> U8Vector4: ...
    @property
    def lzzl(self) -> U8Vector4: ...
    @property
    def lzox(self) -> U8Vector4: ...
    @property
    def lzoy(self) -> U8Vector4: ...
    @property
    def lzoz(self) -> U8Vector4: ...
    @property
    def lzoo(self) -> U8Vector4: ...
    @property
    def lzol(self) -> U8Vector4: ...
    @property
    def lzlx(self) -> U8Vector4: ...
    @property
    def lzly(self) -> U8Vector4: ...
    @property
    def lzlz(self) -> U8Vector4: ...
    @property
    def lzlo(self) -> U8Vector4: ...
    @property
    def lzll(self) -> U8Vector4: ...
    @property
    def loxx(self) -> U8Vector4: ...
    @property
    def loxy(self) -> U8Vector4: ...
    @property
    def loxz(self) -> U8Vector4: ...
    @property
    def loxo(self) -> U8Vector4: ...
    @property
    def loxl(self) -> U8Vector4: ...
    @property
    def loyx(self) -> U8Vector4: ...
    @property
    def loyy(self) -> U8Vector4: ...
    @property
    def loyz(self) -> U8Vector4: ...
    @property
    def loyo(self) -> U8Vector4: ...
    @property
    def loyl(self) -> U8Vector4: ...
    @property
    def lozx(self) -> U8Vector4: ...
    @property
    def lozy(self) -> U8Vector4: ...
    @property
    def lozz(self) -> U8Vector4: ...
    @property
    def lozo(self) -> U8Vector4: ...
    @property
    def lozl(self) -> U8Vector4: ...
    @property
    def loox(self) -> U8Vector4: ...
    @property
    def looy(self) -> U8Vector4: ...
    @property
    def looz(self) -> U8Vector4: ...
    @property
    def lolx(self) -> U8Vector4: ...
    @property
    def loly(self) -> U8Vector4: ...
    @property
    def lolz(self) -> U8Vector4: ...
    @property
    def llxx(self) -> U8Vector4: ...
    @property
    def llxy(self) -> U8Vector4: ...
    @property
    def llxz(self) -> U8Vector4: ...
    @property
    def llxo(self) -> U8Vector4: ...
    @property
    def llxl(self) -> U8Vector4: ...
    @property
    def llyx(self) -> U8Vector4: ...
    @property
    def llyy(self) -> U8Vector4: ...
    @property
    def llyz(self) -> U8Vector4: ...
    @property
    def llyo(self) -> U8Vector4: ...
    @property
    def llyl(self) -> U8Vector4: ...
    @property
    def llzx(self) -> U8Vector4: ...
    @property
    def llzy(self) -> U8Vector4: ...
    @property
    def llzz(self) -> U8Vector4: ...
    @property
    def llzo(self) -> U8Vector4: ...
    @property
    def llzl(self) -> U8Vector4: ...
    @property
    def llox(self) -> U8Vector4: ...
    @property
    def lloy(self) -> U8Vector4: ...
    @property
    def lloz(self) -> U8Vector4: ...
    @property
    def lllx(self) -> U8Vector4: ...
    @property
    def llly(self) -> U8Vector4: ...
    @property
    def lllz(self) -> U8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> U8Vector2: ...
    @property
    def rg(self) -> U8Vector2: ...
    @property
    def rb(self) -> U8Vector2: ...
    @property
    def ro(self) -> U8Vector2: ...
    @property
    def rl(self) -> U8Vector2: ...
    @property
    def gr(self) -> U8Vector2: ...
    @property
    def gg(self) -> U8Vector2: ...
    @property
    def gb(self) -> U8Vector2: ...
    @property
    def go(self) -> U8Vector2: ...
    @property
    def gl(self) -> U8Vector2: ...
    @property
    def br(self) -> U8Vector2: ...
    @property
    def bg(self) -> U8Vector2: ...
    @property
    def bb(self) -> U8Vector2: ...
    @property
    def bo(self) -> U8Vector2: ...
    @property
    def bl(self) -> U8Vector2: ...
    @property
    def og(self) -> U8Vector2: ...
    @property
    def ob(self) -> U8Vector2: ...
    @property
    def lr(self) -> U8Vector2: ...
    @property
    def lg(self) -> U8Vector2: ...
    @property
    def lb(self) -> U8Vector2: ...
    @property
    def rrr(self) -> U8Vector3: ...
    @property
    def rrg(self) -> U8Vector3: ...
    @property
    def rrb(self) -> U8Vector3: ...
    @property
    def rro(self) -> U8Vector3: ...
    @property
    def rrl(self) -> U8Vector3: ...
    @property
    def rgr(self) -> U8Vector3: ...
    @property
    def rgg(self) -> U8Vector3: ...
    @property
    def rgb(self) -> U8Vector3: ...
    @property
    def rgo(self) -> U8Vector3: ...
    @property
    def rgl(self) -> U8Vector3: ...
    @property
    def rbr(self) -> U8Vector3: ...
    @property
    def rbg(self) -> U8Vector3: ...
    @property
    def rbb(self) -> U8Vector3: ...
    @property
    def rbo(self) -> U8Vector3: ...
    @property
    def rbl(self) -> U8Vector3: ...
    @property
    def ror(self) -> U8Vector3: ...
    @property
    def rog(self) -> U8Vector3: ...
    @property
    def rob(self) -> U8Vector3: ...
    @property
    def roo(self) -> U8Vector3: ...
    @property
    def rol(self) -> U8Vector3: ...
    @property
    def rlr(self) -> U8Vector3: ...
    @property
    def rlg(self) -> U8Vector3: ...
    @property
    def rlb(self) -> U8Vector3: ...
    @property
    def rlo(self) -> U8Vector3: ...
    @property
    def rll(self) -> U8Vector3: ...
    @property
    def grr(self) -> U8Vector3: ...
    @property
    def grg(self) -> U8Vector3: ...
    @property
    def grb(self) -> U8Vector3: ...
    @property
    def gro(self) -> U8Vector3: ...
    @property
    def grl(self) -> U8Vector3: ...
    @property
    def ggr(self) -> U8Vector3: ...
    @property
    def ggg(self) -> U8Vector3: ...
    @property
    def ggb(self) -> U8Vector3: ...
    @property
    def ggo(self) -> U8Vector3: ...
    @property
    def ggl(self) -> U8Vector3: ...
    @property
    def gbr(self) -> U8Vector3: ...
    @property
    def gbg(self) -> U8Vector3: ...
    @property
    def gbb(self) -> U8Vector3: ...
    @property
    def gbo(self) -> U8Vector3: ...
    @property
    def gbl(self) -> U8Vector3: ...
    @property
    def gor(self) -> U8Vector3: ...
    @property
    def gog(self) -> U8Vector3: ...
    @property
    def gob(self) -> U8Vector3: ...
    @property
    def goo(self) -> U8Vector3: ...
    @property
    def gol(self) -> U8Vector3: ...
    @property
    def glr(self) -> U8Vector3: ...
    @property
    def glg(self) -> U8Vector3: ...
    @property
    def glb(self) -> U8Vector3: ...
    @property
    def glo(self) -> U8Vector3: ...
    @property
    def gll(self) -> U8Vector3: ...
    @property
    def brr(self) -> U8Vector3: ...
    @property
    def brg(self) -> U8Vector3: ...
    @property
    def brb(self) -> U8Vector3: ...
    @property
    def bro(self) -> U8Vector3: ...
    @property
    def brl(self) -> U8Vector3: ...
    @property
    def bgr(self) -> U8Vector3: ...
    @property
    def bgg(self) -> U8Vector3: ...
    @property
    def bgb(self) -> U8Vector3: ...
    @property
    def bgo(self) -> U8Vector3: ...
    @property
    def bgl(self) -> U8Vector3: ...
    @property
    def bbr(self) -> U8Vector3: ...
    @property
    def bbg(self) -> U8Vector3: ...
    @property
    def bbb(self) -> U8Vector3: ...
    @property
    def bbo(self) -> U8Vector3: ...
    @property
    def bbl(self) -> U8Vector3: ...
    @property
    def bor(self) -> U8Vector3: ...
    @property
    def bog(self) -> U8Vector3: ...
    @property
    def bob(self) -> U8Vector3: ...
    @property
    def boo(self) -> U8Vector3: ...
    @property
    def bol(self) -> U8Vector3: ...
    @property
    def blr(self) -> U8Vector3: ...
    @property
    def blg(self) -> U8Vector3: ...
    @property
    def blb(self) -> U8Vector3: ...
    @property
    def blo(self) -> U8Vector3: ...
    @property
    def bll(self) -> U8Vector3: ...
    @property
    def orr(self) -> U8Vector3: ...
    @property
    def org(self) -> U8Vector3: ...
    @property
    def orb(self) -> U8Vector3: ...
    @property
    def oro(self) -> U8Vector3: ...
    @property
    def orl(self) -> U8Vector3: ...
    @property
    def ogr(self) -> U8Vector3: ...
    @property
    def ogg(self) -> U8Vector3: ...
    @property
    def ogb(self) -> U8Vector3: ...
    @property
    def ogo(self) -> U8Vector3: ...
    @property
    def ogl(self) -> U8Vector3: ...
    @property
    def obr(self) -> U8Vector3: ...
    @property
    def obg(self) -> U8Vector3: ...
    @property
    def obb(self) -> U8Vector3: ...
    @property
    def obo(self) -> U8Vector3: ...
    @property
    def obl(self) -> U8Vector3: ...
    @property
    def oor(self) -> U8Vector3: ...
    @property
    def oog(self) -> U8Vector3: ...
    @property
    def oob(self) -> U8Vector3: ...
    @property
    def olr(self) -> U8Vector3: ...
    @property
    def olg(self) -> U8Vector3: ...
    @property
    def olb(self) -> U8Vector3: ...
    @property
    def lrr(self) -> U8Vector3: ...
    @property
    def lrg(self) -> U8Vector3: ...
    @property
    def lrb(self) -> U8Vector3: ...
    @property
    def lro(self) -> U8Vector3: ...
    @property
    def lrl(self) -> U8Vector3: ...
    @property
    def lgr(self) -> U8Vector3: ...
    @property
    def lgg(self) -> U8Vector3: ...
    @property
    def lgb(self) -> U8Vector3: ...
    @property
    def lgo(self) -> U8Vector3: ...
    @property
    def lgl(self) -> U8Vector3: ...
    @property
    def lbr(self) -> U8Vector3: ...
    @property
    def lbg(self) -> U8Vector3: ...
    @property
    def lbb(self) -> U8Vector3: ...
    @property
    def lbo(self) -> U8Vector3: ...
    @property
    def lbl(self) -> U8Vector3: ...
    @property
    def lor(self) -> U8Vector3: ...
    @property
    def log(self) -> U8Vector3: ...
    @property
    def lob(self) -> U8Vector3: ...
    @property
    def llr(self) -> U8Vector3: ...
    @property
    def llg(self) -> U8Vector3: ...
    @property
    def llb(self) -> U8Vector3: ...
    @property
    def rrrr(self) -> U8Vector4: ...
    @property
    def rrrg(self) -> U8Vector4: ...
    @property
    def rrrb(self) -> U8Vector4: ...
    @property
    def rrro(self) -> U8Vector4: ...
    @property
    def rrrl(self) -> U8Vector4: ...
    @property
    def rrgr(self) -> U8Vector4: ...
    @property
    def rrgg(self) -> U8Vector4: ...
    @property
    def rrgb(self) -> U8Vector4: ...
    @property
    def rrgo(self) -> U8Vector4: ...
    @property
    def rrgl(self) -> U8Vector4: ...
    @property
    def rrbr(self) -> U8Vector4: ...
    @property
    def rrbg(self) -> U8Vector4: ...
    @property
    def rrbb(self) -> U8Vector4: ...
    @property
    def rrbo(self) -> U8Vector4: ...
    @property
    def rrbl(self) -> U8Vector4: ...
    @property
    def rror(self) -> U8Vector4: ...
    @property
    def rrog(self) -> U8Vector4: ...
    @property
    def rrob(self) -> U8Vector4: ...
    @property
    def rroo(self) -> U8Vector4: ...
    @property
    def rrol(self) -> U8Vector4: ...
    @property
    def rrlr(self) -> U8Vector4: ...
    @property
    def rrlg(self) -> U8Vector4: ...
    @property
    def rrlb(self) -> U8Vector4: ...
    @property
    def rrlo(self) -> U8Vector4: ...
    @property
    def rrll(self) -> U8Vector4: ...
    @property
    def rgrr(self) -> U8Vector4: ...
    @property
    def rgrg(self) -> U8Vector4: ...
    @property
    def rgrb(self) -> U8Vector4: ...
    @property
    def rgro(self) -> U8Vector4: ...
    @property
    def rgrl(self) -> U8Vector4: ...
    @property
    def rggr(self) -> U8Vector4: ...
    @property
    def rggg(self) -> U8Vector4: ...
    @property
    def rggb(self) -> U8Vector4: ...
    @property
    def rggo(self) -> U8Vector4: ...
    @property
    def rggl(self) -> U8Vector4: ...
    @property
    def rgbr(self) -> U8Vector4: ...
    @property
    def rgbg(self) -> U8Vector4: ...
    @property
    def rgbb(self) -> U8Vector4: ...
    @property
    def rgbo(self) -> U8Vector4: ...
    @property
    def rgbl(self) -> U8Vector4: ...
    @property
    def rgor(self) -> U8Vector4: ...
    @property
    def rgog(self) -> U8Vector4: ...
    @property
    def rgob(self) -> U8Vector4: ...
    @property
    def rgoo(self) -> U8Vector4: ...
    @property
    def rgol(self) -> U8Vector4: ...
    @property
    def rglr(self) -> U8Vector4: ...
    @property
    def rglg(self) -> U8Vector4: ...
    @property
    def rglb(self) -> U8Vector4: ...
    @property
    def rglo(self) -> U8Vector4: ...
    @property
    def rgll(self) -> U8Vector4: ...
    @property
    def rbrr(self) -> U8Vector4: ...
    @property
    def rbrg(self) -> U8Vector4: ...
    @property
    def rbrb(self) -> U8Vector4: ...
    @property
    def rbro(self) -> U8Vector4: ...
    @property
    def rbrl(self) -> U8Vector4: ...
    @property
    def rbgr(self) -> U8Vector4: ...
    @property
    def rbgg(self) -> U8Vector4: ...
    @property
    def rbgb(self) -> U8Vector4: ...
    @property
    def rbgo(self) -> U8Vector4: ...
    @property
    def rbgl(self) -> U8Vector4: ...
    @property
    def rbbr(self) -> U8Vector4: ...
    @property
    def rbbg(self) -> U8Vector4: ...
    @property
    def rbbb(self) -> U8Vector4: ...
    @property
    def rbbo(self) -> U8Vector4: ...
    @property
    def rbbl(self) -> U8Vector4: ...
    @property
    def rbor(self) -> U8Vector4: ...
    @property
    def rbog(self) -> U8Vector4: ...
    @property
    def rbob(self) -> U8Vector4: ...
    @property
    def rboo(self) -> U8Vector4: ...
    @property
    def rbol(self) -> U8Vector4: ...
    @property
    def rblr(self) -> U8Vector4: ...
    @property
    def rblg(self) -> U8Vector4: ...
    @property
    def rblb(self) -> U8Vector4: ...
    @property
    def rblo(self) -> U8Vector4: ...
    @property
    def rbll(self) -> U8Vector4: ...
    @property
    def rorr(self) -> U8Vector4: ...
    @property
    def rorg(self) -> U8Vector4: ...
    @property
    def rorb(self) -> U8Vector4: ...
    @property
    def roro(self) -> U8Vector4: ...
    @property
    def rorl(self) -> U8Vector4: ...
    @property
    def rogr(self) -> U8Vector4: ...
    @property
    def rogg(self) -> U8Vector4: ...
    @property
    def rogb(self) -> U8Vector4: ...
    @property
    def rogo(self) -> U8Vector4: ...
    @property
    def rogl(self) -> U8Vector4: ...
    @property
    def robr(self) -> U8Vector4: ...
    @property
    def robg(self) -> U8Vector4: ...
    @property
    def robb(self) -> U8Vector4: ...
    @property
    def robo(self) -> U8Vector4: ...
    @property
    def robl(self) -> U8Vector4: ...
    @property
    def roor(self) -> U8Vector4: ...
    @property
    def roog(self) -> U8Vector4: ...
    @property
    def roob(self) -> U8Vector4: ...
    @property
    def rooo(self) -> U8Vector4: ...
    @property
    def rool(self) -> U8Vector4: ...
    @property
    def rolr(self) -> U8Vector4: ...
    @property
    def rolg(self) -> U8Vector4: ...
    @property
    def rolb(self) -> U8Vector4: ...
    @property
    def rolo(self) -> U8Vector4: ...
    @property
    def roll(self) -> U8Vector4: ...
    @property
    def rlrr(self) -> U8Vector4: ...
    @property
    def rlrg(self) -> U8Vector4: ...
    @property
    def rlrb(self) -> U8Vector4: ...
    @property
    def rlro(self) -> U8Vector4: ...
    @property
    def rlrl(self) -> U8Vector4: ...
    @property
    def rlgr(self) -> U8Vector4: ...
    @property
    def rlgg(self) -> U8Vector4: ...
    @property
    def rlgb(self) -> U8Vector4: ...
    @property
    def rlgo(self) -> U8Vector4: ...
    @property
    def rlgl(self) -> U8Vector4: ...
    @property
    def rlbr(self) -> U8Vector4: ...
    @property
    def rlbg(self) -> U8Vector4: ...
    @property
    def rlbb(self) -> U8Vector4: ...
    @property
    def rlbo(self) -> U8Vector4: ...
    @property
    def rlbl(self) -> U8Vector4: ...
    @property
    def rlor(self) -> U8Vector4: ...
    @property
    def rlog(self) -> U8Vector4: ...
    @property
    def rlob(self) -> U8Vector4: ...
    @property
    def rloo(self) -> U8Vector4: ...
    @property
    def rlol(self) -> U8Vector4: ...
    @property
    def rllr(self) -> U8Vector4: ...
    @property
    def rllg(self) -> U8Vector4: ...
    @property
    def rllb(self) -> U8Vector4: ...
    @property
    def rllo(self) -> U8Vector4: ...
    @property
    def rlll(self) -> U8Vector4: ...
    @property
    def grrr(self) -> U8Vector4: ...
    @property
    def grrg(self) -> U8Vector4: ...
    @property
    def grrb(self) -> U8Vector4: ...
    @property
    def grro(self) -> U8Vector4: ...
    @property
    def grrl(self) -> U8Vector4: ...
    @property
    def grgr(self) -> U8Vector4: ...
    @property
    def grgg(self) -> U8Vector4: ...
    @property
    def grgb(self) -> U8Vector4: ...
    @property
    def grgo(self) -> U8Vector4: ...
    @property
    def grgl(self) -> U8Vector4: ...
    @property
    def grbr(self) -> U8Vector4: ...
    @property
    def grbg(self) -> U8Vector4: ...
    @property
    def grbb(self) -> U8Vector4: ...
    @property
    def grbo(self) -> U8Vector4: ...
    @property
    def grbl(self) -> U8Vector4: ...
    @property
    def gror(self) -> U8Vector4: ...
    @property
    def grog(self) -> U8Vector4: ...
    @property
    def grob(self) -> U8Vector4: ...
    @property
    def groo(self) -> U8Vector4: ...
    @property
    def grol(self) -> U8Vector4: ...
    @property
    def grlr(self) -> U8Vector4: ...
    @property
    def grlg(self) -> U8Vector4: ...
    @property
    def grlb(self) -> U8Vector4: ...
    @property
    def grlo(self) -> U8Vector4: ...
    @property
    def grll(self) -> U8Vector4: ...
    @property
    def ggrr(self) -> U8Vector4: ...
    @property
    def ggrg(self) -> U8Vector4: ...
    @property
    def ggrb(self) -> U8Vector4: ...
    @property
    def ggro(self) -> U8Vector4: ...
    @property
    def ggrl(self) -> U8Vector4: ...
    @property
    def gggr(self) -> U8Vector4: ...
    @property
    def gggg(self) -> U8Vector4: ...
    @property
    def gggb(self) -> U8Vector4: ...
    @property
    def gggo(self) -> U8Vector4: ...
    @property
    def gggl(self) -> U8Vector4: ...
    @property
    def ggbr(self) -> U8Vector4: ...
    @property
    def ggbg(self) -> U8Vector4: ...
    @property
    def ggbb(self) -> U8Vector4: ...
    @property
    def ggbo(self) -> U8Vector4: ...
    @property
    def ggbl(self) -> U8Vector4: ...
    @property
    def ggor(self) -> U8Vector4: ...
    @property
    def ggog(self) -> U8Vector4: ...
    @property
    def ggob(self) -> U8Vector4: ...
    @property
    def ggoo(self) -> U8Vector4: ...
    @property
    def ggol(self) -> U8Vector4: ...
    @property
    def gglr(self) -> U8Vector4: ...
    @property
    def gglg(self) -> U8Vector4: ...
    @property
    def gglb(self) -> U8Vector4: ...
    @property
    def gglo(self) -> U8Vector4: ...
    @property
    def ggll(self) -> U8Vector4: ...
    @property
    def gbrr(self) -> U8Vector4: ...
    @property
    def gbrg(self) -> U8Vector4: ...
    @property
    def gbrb(self) -> U8Vector4: ...
    @property
    def gbro(self) -> U8Vector4: ...
    @property
    def gbrl(self) -> U8Vector4: ...
    @property
    def gbgr(self) -> U8Vector4: ...
    @property
    def gbgg(self) -> U8Vector4: ...
    @property
    def gbgb(self) -> U8Vector4: ...
    @property
    def gbgo(self) -> U8Vector4: ...
    @property
    def gbgl(self) -> U8Vector4: ...
    @property
    def gbbr(self) -> U8Vector4: ...
    @property
    def gbbg(self) -> U8Vector4: ...
    @property
    def gbbb(self) -> U8Vector4: ...
    @property
    def gbbo(self) -> U8Vector4: ...
    @property
    def gbbl(self) -> U8Vector4: ...
    @property
    def gbor(self) -> U8Vector4: ...
    @property
    def gbog(self) -> U8Vector4: ...
    @property
    def gbob(self) -> U8Vector4: ...
    @property
    def gboo(self) -> U8Vector4: ...
    @property
    def gbol(self) -> U8Vector4: ...
    @property
    def gblr(self) -> U8Vector4: ...
    @property
    def gblg(self) -> U8Vector4: ...
    @property
    def gblb(self) -> U8Vector4: ...
    @property
    def gblo(self) -> U8Vector4: ...
    @property
    def gbll(self) -> U8Vector4: ...
    @property
    def gorr(self) -> U8Vector4: ...
    @property
    def gorg(self) -> U8Vector4: ...
    @property
    def gorb(self) -> U8Vector4: ...
    @property
    def goro(self) -> U8Vector4: ...
    @property
    def gorl(self) -> U8Vector4: ...
    @property
    def gogr(self) -> U8Vector4: ...
    @property
    def gogg(self) -> U8Vector4: ...
    @property
    def gogb(self) -> U8Vector4: ...
    @property
    def gogo(self) -> U8Vector4: ...
    @property
    def gogl(self) -> U8Vector4: ...
    @property
    def gobr(self) -> U8Vector4: ...
    @property
    def gobg(self) -> U8Vector4: ...
    @property
    def gobb(self) -> U8Vector4: ...
    @property
    def gobo(self) -> U8Vector4: ...
    @property
    def gobl(self) -> U8Vector4: ...
    @property
    def goor(self) -> U8Vector4: ...
    @property
    def goog(self) -> U8Vector4: ...
    @property
    def goob(self) -> U8Vector4: ...
    @property
    def gooo(self) -> U8Vector4: ...
    @property
    def gool(self) -> U8Vector4: ...
    @property
    def golr(self) -> U8Vector4: ...
    @property
    def golg(self) -> U8Vector4: ...
    @property
    def golb(self) -> U8Vector4: ...
    @property
    def golo(self) -> U8Vector4: ...
    @property
    def goll(self) -> U8Vector4: ...
    @property
    def glrr(self) -> U8Vector4: ...
    @property
    def glrg(self) -> U8Vector4: ...
    @property
    def glrb(self) -> U8Vector4: ...
    @property
    def glro(self) -> U8Vector4: ...
    @property
    def glrl(self) -> U8Vector4: ...
    @property
    def glgr(self) -> U8Vector4: ...
    @property
    def glgg(self) -> U8Vector4: ...
    @property
    def glgb(self) -> U8Vector4: ...
    @property
    def glgo(self) -> U8Vector4: ...
    @property
    def glgl(self) -> U8Vector4: ...
    @property
    def glbr(self) -> U8Vector4: ...
    @property
    def glbg(self) -> U8Vector4: ...
    @property
    def glbb(self) -> U8Vector4: ...
    @property
    def glbo(self) -> U8Vector4: ...
    @property
    def glbl(self) -> U8Vector4: ...
    @property
    def glor(self) -> U8Vector4: ...
    @property
    def glog(self) -> U8Vector4: ...
    @property
    def glob(self) -> U8Vector4: ...
    @property
    def gloo(self) -> U8Vector4: ...
    @property
    def glol(self) -> U8Vector4: ...
    @property
    def gllr(self) -> U8Vector4: ...
    @property
    def gllg(self) -> U8Vector4: ...
    @property
    def gllb(self) -> U8Vector4: ...
    @property
    def gllo(self) -> U8Vector4: ...
    @property
    def glll(self) -> U8Vector4: ...
    @property
    def brrr(self) -> U8Vector4: ...
    @property
    def brrg(self) -> U8Vector4: ...
    @property
    def brrb(self) -> U8Vector4: ...
    @property
    def brro(self) -> U8Vector4: ...
    @property
    def brrl(self) -> U8Vector4: ...
    @property
    def brgr(self) -> U8Vector4: ...
    @property
    def brgg(self) -> U8Vector4: ...
    @property
    def brgb(self) -> U8Vector4: ...
    @property
    def brgo(self) -> U8Vector4: ...
    @property
    def brgl(self) -> U8Vector4: ...
    @property
    def brbr(self) -> U8Vector4: ...
    @property
    def brbg(self) -> U8Vector4: ...
    @property
    def brbb(self) -> U8Vector4: ...
    @property
    def brbo(self) -> U8Vector4: ...
    @property
    def brbl(self) -> U8Vector4: ...
    @property
    def bror(self) -> U8Vector4: ...
    @property
    def brog(self) -> U8Vector4: ...
    @property
    def brob(self) -> U8Vector4: ...
    @property
    def broo(self) -> U8Vector4: ...
    @property
    def brol(self) -> U8Vector4: ...
    @property
    def brlr(self) -> U8Vector4: ...
    @property
    def brlg(self) -> U8Vector4: ...
    @property
    def brlb(self) -> U8Vector4: ...
    @property
    def brlo(self) -> U8Vector4: ...
    @property
    def brll(self) -> U8Vector4: ...
    @property
    def bgrr(self) -> U8Vector4: ...
    @property
    def bgrg(self) -> U8Vector4: ...
    @property
    def bgrb(self) -> U8Vector4: ...
    @property
    def bgro(self) -> U8Vector4: ...
    @property
    def bgrl(self) -> U8Vector4: ...
    @property
    def bggr(self) -> U8Vector4: ...
    @property
    def bggg(self) -> U8Vector4: ...
    @property
    def bggb(self) -> U8Vector4: ...
    @property
    def bggo(self) -> U8Vector4: ...
    @property
    def bggl(self) -> U8Vector4: ...
    @property
    def bgbr(self) -> U8Vector4: ...
    @property
    def bgbg(self) -> U8Vector4: ...
    @property
    def bgbb(self) -> U8Vector4: ...
    @property
    def bgbo(self) -> U8Vector4: ...
    @property
    def bgbl(self) -> U8Vector4: ...
    @property
    def bgor(self) -> U8Vector4: ...
    @property
    def bgog(self) -> U8Vector4: ...
    @property
    def bgob(self) -> U8Vector4: ...
    @property
    def bgoo(self) -> U8Vector4: ...
    @property
    def bgol(self) -> U8Vector4: ...
    @property
    def bglr(self) -> U8Vector4: ...
    @property
    def bglg(self) -> U8Vector4: ...
    @property
    def bglb(self) -> U8Vector4: ...
    @property
    def bglo(self) -> U8Vector4: ...
    @property
    def bgll(self) -> U8Vector4: ...
    @property
    def bbrr(self) -> U8Vector4: ...
    @property
    def bbrg(self) -> U8Vector4: ...
    @property
    def bbrb(self) -> U8Vector4: ...
    @property
    def bbro(self) -> U8Vector4: ...
    @property
    def bbrl(self) -> U8Vector4: ...
    @property
    def bbgr(self) -> U8Vector4: ...
    @property
    def bbgg(self) -> U8Vector4: ...
    @property
    def bbgb(self) -> U8Vector4: ...
    @property
    def bbgo(self) -> U8Vector4: ...
    @property
    def bbgl(self) -> U8Vector4: ...
    @property
    def bbbr(self) -> U8Vector4: ...
    @property
    def bbbg(self) -> U8Vector4: ...
    @property
    def bbbb(self) -> U8Vector4: ...
    @property
    def bbbo(self) -> U8Vector4: ...
    @property
    def bbbl(self) -> U8Vector4: ...
    @property
    def bbor(self) -> U8Vector4: ...
    @property
    def bbog(self) -> U8Vector4: ...
    @property
    def bbob(self) -> U8Vector4: ...
    @property
    def bboo(self) -> U8Vector4: ...
    @property
    def bbol(self) -> U8Vector4: ...
    @property
    def bblr(self) -> U8Vector4: ...
    @property
    def bblg(self) -> U8Vector4: ...
    @property
    def bblb(self) -> U8Vector4: ...
    @property
    def bblo(self) -> U8Vector4: ...
    @property
    def bbll(self) -> U8Vector4: ...
    @property
    def borr(self) -> U8Vector4: ...
    @property
    def borg(self) -> U8Vector4: ...
    @property
    def borb(self) -> U8Vector4: ...
    @property
    def boro(self) -> U8Vector4: ...
    @property
    def borl(self) -> U8Vector4: ...
    @property
    def bogr(self) -> U8Vector4: ...
    @property
    def bogg(self) -> U8Vector4: ...
    @property
    def bogb(self) -> U8Vector4: ...
    @property
    def bogo(self) -> U8Vector4: ...
    @property
    def bogl(self) -> U8Vector4: ...
    @property
    def bobr(self) -> U8Vector4: ...
    @property
    def bobg(self) -> U8Vector4: ...
    @property
    def bobb(self) -> U8Vector4: ...
    @property
    def bobo(self) -> U8Vector4: ...
    @property
    def bobl(self) -> U8Vector4: ...
    @property
    def boor(self) -> U8Vector4: ...
    @property
    def boog(self) -> U8Vector4: ...
    @property
    def boob(self) -> U8Vector4: ...
    @property
    def booo(self) -> U8Vector4: ...
    @property
    def bool(self) -> U8Vector4: ...
    @property
    def bolr(self) -> U8Vector4: ...
    @property
    def bolg(self) -> U8Vector4: ...
    @property
    def bolb(self) -> U8Vector4: ...
    @property
    def bolo(self) -> U8Vector4: ...
    @property
    def boll(self) -> U8Vector4: ...
    @property
    def blrr(self) -> U8Vector4: ...
    @property
    def blrg(self) -> U8Vector4: ...
    @property
    def blrb(self) -> U8Vector4: ...
    @property
    def blro(self) -> U8Vector4: ...
    @property
    def blrl(self) -> U8Vector4: ...
    @property
    def blgr(self) -> U8Vector4: ...
    @property
    def blgg(self) -> U8Vector4: ...
    @property
    def blgb(self) -> U8Vector4: ...
    @property
    def blgo(self) -> U8Vector4: ...
    @property
    def blgl(self) -> U8Vector4: ...
    @property
    def blbr(self) -> U8Vector4: ...
    @property
    def blbg(self) -> U8Vector4: ...
    @property
    def blbb(self) -> U8Vector4: ...
    @property
    def blbo(self) -> U8Vector4: ...
    @property
    def blbl(self) -> U8Vector4: ...
    @property
    def blor(self) -> U8Vector4: ...
    @property
    def blog(self) -> U8Vector4: ...
    @property
    def blob(self) -> U8Vector4: ...
    @property
    def bloo(self) -> U8Vector4: ...
    @property
    def blol(self) -> U8Vector4: ...
    @property
    def bllr(self) -> U8Vector4: ...
    @property
    def bllg(self) -> U8Vector4: ...
    @property
    def bllb(self) -> U8Vector4: ...
    @property
    def bllo(self) -> U8Vector4: ...
    @property
    def blll(self) -> U8Vector4: ...
    @property
    def orrr(self) -> U8Vector4: ...
    @property
    def orrg(self) -> U8Vector4: ...
    @property
    def orrb(self) -> U8Vector4: ...
    @property
    def orro(self) -> U8Vector4: ...
    @property
    def orrl(self) -> U8Vector4: ...
    @property
    def orgr(self) -> U8Vector4: ...
    @property
    def orgg(self) -> U8Vector4: ...
    @property
    def orgb(self) -> U8Vector4: ...
    @property
    def orgo(self) -> U8Vector4: ...
    @property
    def orgl(self) -> U8Vector4: ...
    @property
    def orbr(self) -> U8Vector4: ...
    @property
    def orbg(self) -> U8Vector4: ...
    @property
    def orbb(self) -> U8Vector4: ...
    @property
    def orbo(self) -> U8Vector4: ...
    @property
    def orbl(self) -> U8Vector4: ...
    @property
    def oror(self) -> U8Vector4: ...
    @property
    def orog(self) -> U8Vector4: ...
    @property
    def orob(self) -> U8Vector4: ...
    @property
    def oroo(self) -> U8Vector4: ...
    @property
    def orol(self) -> U8Vector4: ...
    @property
    def orlr(self) -> U8Vector4: ...
    @property
    def orlg(self) -> U8Vector4: ...
    @property
    def orlb(self) -> U8Vector4: ...
    @property
    def orlo(self) -> U8Vector4: ...
    @property
    def orll(self) -> U8Vector4: ...
    @property
    def ogrr(self) -> U8Vector4: ...
    @property
    def ogrg(self) -> U8Vector4: ...
    @property
    def ogrb(self) -> U8Vector4: ...
    @property
    def ogro(self) -> U8Vector4: ...
    @property
    def ogrl(self) -> U8Vector4: ...
    @property
    def oggr(self) -> U8Vector4: ...
    @property
    def oggg(self) -> U8Vector4: ...
    @property
    def oggb(self) -> U8Vector4: ...
    @property
    def oggo(self) -> U8Vector4: ...
    @property
    def oggl(self) -> U8Vector4: ...
    @property
    def ogbr(self) -> U8Vector4: ...
    @property
    def ogbg(self) -> U8Vector4: ...
    @property
    def ogbb(self) -> U8Vector4: ...
    @property
    def ogbo(self) -> U8Vector4: ...
    @property
    def ogbl(self) -> U8Vector4: ...
    @property
    def ogor(self) -> U8Vector4: ...
    @property
    def ogog(self) -> U8Vector4: ...
    @property
    def ogob(self) -> U8Vector4: ...
    @property
    def ogoo(self) -> U8Vector4: ...
    @property
    def ogol(self) -> U8Vector4: ...
    @property
    def oglr(self) -> U8Vector4: ...
    @property
    def oglg(self) -> U8Vector4: ...
    @property
    def oglb(self) -> U8Vector4: ...
    @property
    def oglo(self) -> U8Vector4: ...
    @property
    def ogll(self) -> U8Vector4: ...
    @property
    def obrr(self) -> U8Vector4: ...
    @property
    def obrg(self) -> U8Vector4: ...
    @property
    def obrb(self) -> U8Vector4: ...
    @property
    def obro(self) -> U8Vector4: ...
    @property
    def obrl(self) -> U8Vector4: ...
    @property
    def obgr(self) -> U8Vector4: ...
    @property
    def obgg(self) -> U8Vector4: ...
    @property
    def obgb(self) -> U8Vector4: ...
    @property
    def obgo(self) -> U8Vector4: ...
    @property
    def obgl(self) -> U8Vector4: ...
    @property
    def obbr(self) -> U8Vector4: ...
    @property
    def obbg(self) -> U8Vector4: ...
    @property
    def obbb(self) -> U8Vector4: ...
    @property
    def obbo(self) -> U8Vector4: ...
    @property
    def obbl(self) -> U8Vector4: ...
    @property
    def obor(self) -> U8Vector4: ...
    @property
    def obog(self) -> U8Vector4: ...
    @property
    def obob(self) -> U8Vector4: ...
    @property
    def oboo(self) -> U8Vector4: ...
    @property
    def obol(self) -> U8Vector4: ...
    @property
    def oblr(self) -> U8Vector4: ...
    @property
    def oblg(self) -> U8Vector4: ...
    @property
    def oblb(self) -> U8Vector4: ...
    @property
    def oblo(self) -> U8Vector4: ...
    @property
    def obll(self) -> U8Vector4: ...
    @property
    def oorr(self) -> U8Vector4: ...
    @property
    def oorg(self) -> U8Vector4: ...
    @property
    def oorb(self) -> U8Vector4: ...
    @property
    def ooro(self) -> U8Vector4: ...
    @property
    def oorl(self) -> U8Vector4: ...
    @property
    def oogr(self) -> U8Vector4: ...
    @property
    def oogg(self) -> U8Vector4: ...
    @property
    def oogb(self) -> U8Vector4: ...
    @property
    def oogo(self) -> U8Vector4: ...
    @property
    def oogl(self) -> U8Vector4: ...
    @property
    def oobr(self) -> U8Vector4: ...
    @property
    def oobg(self) -> U8Vector4: ...
    @property
    def oobb(self) -> U8Vector4: ...
    @property
    def oobo(self) -> U8Vector4: ...
    @property
    def oobl(self) -> U8Vector4: ...
    @property
    def ooor(self) -> U8Vector4: ...
    @property
    def ooog(self) -> U8Vector4: ...
    @property
    def ooob(self) -> U8Vector4: ...
    @property
    def oolr(self) -> U8Vector4: ...
    @property
    def oolg(self) -> U8Vector4: ...
    @property
    def oolb(self) -> U8Vector4: ...
    @property
    def olrr(self) -> U8Vector4: ...
    @property
    def olrg(self) -> U8Vector4: ...
    @property
    def olrb(self) -> U8Vector4: ...
    @property
    def olro(self) -> U8Vector4: ...
    @property
    def olrl(self) -> U8Vector4: ...
    @property
    def olgr(self) -> U8Vector4: ...
    @property
    def olgg(self) -> U8Vector4: ...
    @property
    def olgb(self) -> U8Vector4: ...
    @property
    def olgo(self) -> U8Vector4: ...
    @property
    def olgl(self) -> U8Vector4: ...
    @property
    def olbr(self) -> U8Vector4: ...
    @property
    def olbg(self) -> U8Vector4: ...
    @property
    def olbb(self) -> U8Vector4: ...
    @property
    def olbo(self) -> U8Vector4: ...
    @property
    def olbl(self) -> U8Vector4: ...
    @property
    def olor(self) -> U8Vector4: ...
    @property
    def olog(self) -> U8Vector4: ...
    @property
    def olob(self) -> U8Vector4: ...
    @property
    def ollr(self) -> U8Vector4: ...
    @property
    def ollg(self) -> U8Vector4: ...
    @property
    def ollb(self) -> U8Vector4: ...
    @property
    def lrrr(self) -> U8Vector4: ...
    @property
    def lrrg(self) -> U8Vector4: ...
    @property
    def lrrb(self) -> U8Vector4: ...
    @property
    def lrro(self) -> U8Vector4: ...
    @property
    def lrrl(self) -> U8Vector4: ...
    @property
    def lrgr(self) -> U8Vector4: ...
    @property
    def lrgg(self) -> U8Vector4: ...
    @property
    def lrgb(self) -> U8Vector4: ...
    @property
    def lrgo(self) -> U8Vector4: ...
    @property
    def lrgl(self) -> U8Vector4: ...
    @property
    def lrbr(self) -> U8Vector4: ...
    @property
    def lrbg(self) -> U8Vector4: ...
    @property
    def lrbb(self) -> U8Vector4: ...
    @property
    def lrbo(self) -> U8Vector4: ...
    @property
    def lrbl(self) -> U8Vector4: ...
    @property
    def lror(self) -> U8Vector4: ...
    @property
    def lrog(self) -> U8Vector4: ...
    @property
    def lrob(self) -> U8Vector4: ...
    @property
    def lroo(self) -> U8Vector4: ...
    @property
    def lrol(self) -> U8Vector4: ...
    @property
    def lrlr(self) -> U8Vector4: ...
    @property
    def lrlg(self) -> U8Vector4: ...
    @property
    def lrlb(self) -> U8Vector4: ...
    @property
    def lrlo(self) -> U8Vector4: ...
    @property
    def lrll(self) -> U8Vector4: ...
    @property
    def lgrr(self) -> U8Vector4: ...
    @property
    def lgrg(self) -> U8Vector4: ...
    @property
    def lgrb(self) -> U8Vector4: ...
    @property
    def lgro(self) -> U8Vector4: ...
    @property
    def lgrl(self) -> U8Vector4: ...
    @property
    def lggr(self) -> U8Vector4: ...
    @property
    def lggg(self) -> U8Vector4: ...
    @property
    def lggb(self) -> U8Vector4: ...
    @property
    def lggo(self) -> U8Vector4: ...
    @property
    def lggl(self) -> U8Vector4: ...
    @property
    def lgbr(self) -> U8Vector4: ...
    @property
    def lgbg(self) -> U8Vector4: ...
    @property
    def lgbb(self) -> U8Vector4: ...
    @property
    def lgbo(self) -> U8Vector4: ...
    @property
    def lgbl(self) -> U8Vector4: ...
    @property
    def lgor(self) -> U8Vector4: ...
    @property
    def lgog(self) -> U8Vector4: ...
    @property
    def lgob(self) -> U8Vector4: ...
    @property
    def lgoo(self) -> U8Vector4: ...
    @property
    def lgol(self) -> U8Vector4: ...
    @property
    def lglr(self) -> U8Vector4: ...
    @property
    def lglg(self) -> U8Vector4: ...
    @property
    def lglb(self) -> U8Vector4: ...
    @property
    def lglo(self) -> U8Vector4: ...
    @property
    def lgll(self) -> U8Vector4: ...
    @property
    def lbrr(self) -> U8Vector4: ...
    @property
    def lbrg(self) -> U8Vector4: ...
    @property
    def lbrb(self) -> U8Vector4: ...
    @property
    def lbro(self) -> U8Vector4: ...
    @property
    def lbrl(self) -> U8Vector4: ...
    @property
    def lbgr(self) -> U8Vector4: ...
    @property
    def lbgg(self) -> U8Vector4: ...
    @property
    def lbgb(self) -> U8Vector4: ...
    @property
    def lbgo(self) -> U8Vector4: ...
    @property
    def lbgl(self) -> U8Vector4: ...
    @property
    def lbbr(self) -> U8Vector4: ...
    @property
    def lbbg(self) -> U8Vector4: ...
    @property
    def lbbb(self) -> U8Vector4: ...
    @property
    def lbbo(self) -> U8Vector4: ...
    @property
    def lbbl(self) -> U8Vector4: ...
    @property
    def lbor(self) -> U8Vector4: ...
    @property
    def lbog(self) -> U8Vector4: ...
    @property
    def lbob(self) -> U8Vector4: ...
    @property
    def lboo(self) -> U8Vector4: ...
    @property
    def lbol(self) -> U8Vector4: ...
    @property
    def lblr(self) -> U8Vector4: ...
    @property
    def lblg(self) -> U8Vector4: ...
    @property
    def lblb(self) -> U8Vector4: ...
    @property
    def lblo(self) -> U8Vector4: ...
    @property
    def lbll(self) -> U8Vector4: ...
    @property
    def lorr(self) -> U8Vector4: ...
    @property
    def lorg(self) -> U8Vector4: ...
    @property
    def lorb(self) -> U8Vector4: ...
    @property
    def loro(self) -> U8Vector4: ...
    @property
    def lorl(self) -> U8Vector4: ...
    @property
    def logr(self) -> U8Vector4: ...
    @property
    def logg(self) -> U8Vector4: ...
    @property
    def logb(self) -> U8Vector4: ...
    @property
    def logo(self) -> U8Vector4: ...
    @property
    def logl(self) -> U8Vector4: ...
    @property
    def lobr(self) -> U8Vector4: ...
    @property
    def lobg(self) -> U8Vector4: ...
    @property
    def lobb(self) -> U8Vector4: ...
    @property
    def lobo(self) -> U8Vector4: ...
    @property
    def lobl(self) -> U8Vector4: ...
    @property
    def loor(self) -> U8Vector4: ...
    @property
    def loog(self) -> U8Vector4: ...
    @property
    def loob(self) -> U8Vector4: ...
    @property
    def lolr(self) -> U8Vector4: ...
    @property
    def lolg(self) -> U8Vector4: ...
    @property
    def lolb(self) -> U8Vector4: ...
    @property
    def llrr(self) -> U8Vector4: ...
    @property
    def llrg(self) -> U8Vector4: ...
    @property
    def llrb(self) -> U8Vector4: ...
    @property
    def llro(self) -> U8Vector4: ...
    @property
    def llrl(self) -> U8Vector4: ...
    @property
    def llgr(self) -> U8Vector4: ...
    @property
    def llgg(self) -> U8Vector4: ...
    @property
    def llgb(self) -> U8Vector4: ...
    @property
    def llgo(self) -> U8Vector4: ...
    @property
    def llgl(self) -> U8Vector4: ...
    @property
    def llbr(self) -> U8Vector4: ...
    @property
    def llbg(self) -> U8Vector4: ...
    @property
    def llbb(self) -> U8Vector4: ...
    @property
    def llbo(self) -> U8Vector4: ...
    @property
    def llbl(self) -> U8Vector4: ...
    @property
    def llor(self) -> U8Vector4: ...
    @property
    def llog(self) -> U8Vector4: ...
    @property
    def llob(self) -> U8Vector4: ...
    @property
    def lllr(self) -> U8Vector4: ...
    @property
    def lllg(self) -> U8Vector4: ...
    @property
    def lllb(self) -> U8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> U8Vector2: ...
    @property
    def st(self) -> U8Vector2: ...
    @property
    def sq(self) -> U8Vector2: ...
    @property
    def so(self) -> U8Vector2: ...
    @property
    def sl(self) -> U8Vector2: ...
    @property
    def ts(self) -> U8Vector2: ...
    @property
    def tt(self) -> U8Vector2: ...
    @property
    def tq(self) -> U8Vector2: ...
    @property
    def to(self) -> U8Vector2: ...
    @property
    def tl(self) -> U8Vector2: ...
    @property
    def qs(self) -> U8Vector2: ...
    @property
    def qt(self) -> U8Vector2: ...
    @property
    def qq(self) -> U8Vector2: ...
    @property
    def qo(self) -> U8Vector2: ...
    @property
    def ql(self) -> U8Vector2: ...
    @property
    def os(self) -> U8Vector2: ...
    @property
    def ot(self) -> U8Vector2: ...
    @property
    def oq(self) -> U8Vector2: ...
    @property
    def ls(self) -> U8Vector2: ...
    @property
    def lt(self) -> U8Vector2: ...
    @property
    def lq(self) -> U8Vector2: ...
    @property
    def sss(self) -> U8Vector3: ...
    @property
    def sst(self) -> U8Vector3: ...
    @property
    def ssq(self) -> U8Vector3: ...
    @property
    def sso(self) -> U8Vector3: ...
    @property
    def ssl(self) -> U8Vector3: ...
    @property
    def sts(self) -> U8Vector3: ...
    @property
    def stt(self) -> U8Vector3: ...
    @property
    def stq(self) -> U8Vector3: ...
    @property
    def sto(self) -> U8Vector3: ...
    @property
    def stl(self) -> U8Vector3: ...
    @property
    def sqs(self) -> U8Vector3: ...
    @property
    def sqt(self) -> U8Vector3: ...
    @property
    def sqq(self) -> U8Vector3: ...
    @property
    def sqo(self) -> U8Vector3: ...
    @property
    def sql(self) -> U8Vector3: ...
    @property
    def sos(self) -> U8Vector3: ...
    @property
    def sot(self) -> U8Vector3: ...
    @property
    def soq(self) -> U8Vector3: ...
    @property
    def soo(self) -> U8Vector3: ...
    @property
    def sol(self) -> U8Vector3: ...
    @property
    def sls(self) -> U8Vector3: ...
    @property
    def slt(self) -> U8Vector3: ...
    @property
    def slq(self) -> U8Vector3: ...
    @property
    def slo(self) -> U8Vector3: ...
    @property
    def sll(self) -> U8Vector3: ...
    @property
    def tss(self) -> U8Vector3: ...
    @property
    def tst(self) -> U8Vector3: ...
    @property
    def tsq(self) -> U8Vector3: ...
    @property
    def tso(self) -> U8Vector3: ...
    @property
    def tsl(self) -> U8Vector3: ...
    @property
    def tts(self) -> U8Vector3: ...
    @property
    def ttt(self) -> U8Vector3: ...
    @property
    def ttq(self) -> U8Vector3: ...
    @property
    def tto(self) -> U8Vector3: ...
    @property
    def ttl(self) -> U8Vector3: ...
    @property
    def tqs(self) -> U8Vector3: ...
    @property
    def tqt(self) -> U8Vector3: ...
    @property
    def tqq(self) -> U8Vector3: ...
    @property
    def tqo(self) -> U8Vector3: ...
    @property
    def tql(self) -> U8Vector3: ...
    @property
    def tos(self) -> U8Vector3: ...
    @property
    def tot(self) -> U8Vector3: ...
    @property
    def toq(self) -> U8Vector3: ...
    @property
    def too(self) -> U8Vector3: ...
    @property
    def tol(self) -> U8Vector3: ...
    @property
    def tls(self) -> U8Vector3: ...
    @property
    def tlt(self) -> U8Vector3: ...
    @property
    def tlq(self) -> U8Vector3: ...
    @property
    def tlo(self) -> U8Vector3: ...
    @property
    def tll(self) -> U8Vector3: ...
    @property
    def qss(self) -> U8Vector3: ...
    @property
    def qst(self) -> U8Vector3: ...
    @property
    def qsq(self) -> U8Vector3: ...
    @property
    def qso(self) -> U8Vector3: ...
    @property
    def qsl(self) -> U8Vector3: ...
    @property
    def qts(self) -> U8Vector3: ...
    @property
    def qtt(self) -> U8Vector3: ...
    @property
    def qtq(self) -> U8Vector3: ...
    @property
    def qto(self) -> U8Vector3: ...
    @property
    def qtl(self) -> U8Vector3: ...
    @property
    def qqs(self) -> U8Vector3: ...
    @property
    def qqt(self) -> U8Vector3: ...
    @property
    def qqq(self) -> U8Vector3: ...
    @property
    def qqo(self) -> U8Vector3: ...
    @property
    def qql(self) -> U8Vector3: ...
    @property
    def qos(self) -> U8Vector3: ...
    @property
    def qot(self) -> U8Vector3: ...
    @property
    def qoq(self) -> U8Vector3: ...
    @property
    def qoo(self) -> U8Vector3: ...
    @property
    def qol(self) -> U8Vector3: ...
    @property
    def qls(self) -> U8Vector3: ...
    @property
    def qlt(self) -> U8Vector3: ...
    @property
    def qlq(self) -> U8Vector3: ...
    @property
    def qlo(self) -> U8Vector3: ...
    @property
    def qll(self) -> U8Vector3: ...
    @property
    def oss(self) -> U8Vector3: ...
    @property
    def ost(self) -> U8Vector3: ...
    @property
    def osq(self) -> U8Vector3: ...
    @property
    def oso(self) -> U8Vector3: ...
    @property
    def osl(self) -> U8Vector3: ...
    @property
    def ots(self) -> U8Vector3: ...
    @property
    def ott(self) -> U8Vector3: ...
    @property
    def otq(self) -> U8Vector3: ...
    @property
    def oto(self) -> U8Vector3: ...
    @property
    def otl(self) -> U8Vector3: ...
    @property
    def oqs(self) -> U8Vector3: ...
    @property
    def oqt(self) -> U8Vector3: ...
    @property
    def oqq(self) -> U8Vector3: ...
    @property
    def oqo(self) -> U8Vector3: ...
    @property
    def oql(self) -> U8Vector3: ...
    @property
    def oos(self) -> U8Vector3: ...
    @property
    def oot(self) -> U8Vector3: ...
    @property
    def ooq(self) -> U8Vector3: ...
    @property
    def ols(self) -> U8Vector3: ...
    @property
    def olt(self) -> U8Vector3: ...
    @property
    def olq(self) -> U8Vector3: ...
    @property
    def lss(self) -> U8Vector3: ...
    @property
    def lst(self) -> U8Vector3: ...
    @property
    def lsq(self) -> U8Vector3: ...
    @property
    def lso(self) -> U8Vector3: ...
    @property
    def lsl(self) -> U8Vector3: ...
    @property
    def lts(self) -> U8Vector3: ...
    @property
    def ltt(self) -> U8Vector3: ...
    @property
    def ltq(self) -> U8Vector3: ...
    @property
    def lto(self) -> U8Vector3: ...
    @property
    def ltl(self) -> U8Vector3: ...
    @property
    def lqs(self) -> U8Vector3: ...
    @property
    def lqt(self) -> U8Vector3: ...
    @property
    def lqq(self) -> U8Vector3: ...
    @property
    def lqo(self) -> U8Vector3: ...
    @property
    def lql(self) -> U8Vector3: ...
    @property
    def los(self) -> U8Vector3: ...
    @property
    def lot(self) -> U8Vector3: ...
    @property
    def loq(self) -> U8Vector3: ...
    @property
    def lls(self) -> U8Vector3: ...
    @property
    def llt(self) -> U8Vector3: ...
    @property
    def llq(self) -> U8Vector3: ...
    @property
    def ssss(self) -> U8Vector4: ...
    @property
    def ssst(self) -> U8Vector4: ...
    @property
    def sssq(self) -> U8Vector4: ...
    @property
    def ssso(self) -> U8Vector4: ...
    @property
    def sssl(self) -> U8Vector4: ...
    @property
    def ssts(self) -> U8Vector4: ...
    @property
    def sstt(self) -> U8Vector4: ...
    @property
    def sstq(self) -> U8Vector4: ...
    @property
    def ssto(self) -> U8Vector4: ...
    @property
    def sstl(self) -> U8Vector4: ...
    @property
    def ssqs(self) -> U8Vector4: ...
    @property
    def ssqt(self) -> U8Vector4: ...
    @property
    def ssqq(self) -> U8Vector4: ...
    @property
    def ssqo(self) -> U8Vector4: ...
    @property
    def ssql(self) -> U8Vector4: ...
    @property
    def ssos(self) -> U8Vector4: ...
    @property
    def ssot(self) -> U8Vector4: ...
    @property
    def ssoq(self) -> U8Vector4: ...
    @property
    def ssoo(self) -> U8Vector4: ...
    @property
    def ssol(self) -> U8Vector4: ...
    @property
    def ssls(self) -> U8Vector4: ...
    @property
    def sslt(self) -> U8Vector4: ...
    @property
    def sslq(self) -> U8Vector4: ...
    @property
    def sslo(self) -> U8Vector4: ...
    @property
    def ssll(self) -> U8Vector4: ...
    @property
    def stss(self) -> U8Vector4: ...
    @property
    def stst(self) -> U8Vector4: ...
    @property
    def stsq(self) -> U8Vector4: ...
    @property
    def stso(self) -> U8Vector4: ...
    @property
    def stsl(self) -> U8Vector4: ...
    @property
    def stts(self) -> U8Vector4: ...
    @property
    def sttt(self) -> U8Vector4: ...
    @property
    def sttq(self) -> U8Vector4: ...
    @property
    def stto(self) -> U8Vector4: ...
    @property
    def sttl(self) -> U8Vector4: ...
    @property
    def stqs(self) -> U8Vector4: ...
    @property
    def stqt(self) -> U8Vector4: ...
    @property
    def stqq(self) -> U8Vector4: ...
    @property
    def stqo(self) -> U8Vector4: ...
    @property
    def stql(self) -> U8Vector4: ...
    @property
    def stos(self) -> U8Vector4: ...
    @property
    def stot(self) -> U8Vector4: ...
    @property
    def stoq(self) -> U8Vector4: ...
    @property
    def stoo(self) -> U8Vector4: ...
    @property
    def stol(self) -> U8Vector4: ...
    @property
    def stls(self) -> U8Vector4: ...
    @property
    def stlt(self) -> U8Vector4: ...
    @property
    def stlq(self) -> U8Vector4: ...
    @property
    def stlo(self) -> U8Vector4: ...
    @property
    def stll(self) -> U8Vector4: ...
    @property
    def sqss(self) -> U8Vector4: ...
    @property
    def sqst(self) -> U8Vector4: ...
    @property
    def sqsq(self) -> U8Vector4: ...
    @property
    def sqso(self) -> U8Vector4: ...
    @property
    def sqsl(self) -> U8Vector4: ...
    @property
    def sqts(self) -> U8Vector4: ...
    @property
    def sqtt(self) -> U8Vector4: ...
    @property
    def sqtq(self) -> U8Vector4: ...
    @property
    def sqto(self) -> U8Vector4: ...
    @property
    def sqtl(self) -> U8Vector4: ...
    @property
    def sqqs(self) -> U8Vector4: ...
    @property
    def sqqt(self) -> U8Vector4: ...
    @property
    def sqqq(self) -> U8Vector4: ...
    @property
    def sqqo(self) -> U8Vector4: ...
    @property
    def sqql(self) -> U8Vector4: ...
    @property
    def sqos(self) -> U8Vector4: ...
    @property
    def sqot(self) -> U8Vector4: ...
    @property
    def sqoq(self) -> U8Vector4: ...
    @property
    def sqoo(self) -> U8Vector4: ...
    @property
    def sqol(self) -> U8Vector4: ...
    @property
    def sqls(self) -> U8Vector4: ...
    @property
    def sqlt(self) -> U8Vector4: ...
    @property
    def sqlq(self) -> U8Vector4: ...
    @property
    def sqlo(self) -> U8Vector4: ...
    @property
    def sqll(self) -> U8Vector4: ...
    @property
    def soss(self) -> U8Vector4: ...
    @property
    def sost(self) -> U8Vector4: ...
    @property
    def sosq(self) -> U8Vector4: ...
    @property
    def soso(self) -> U8Vector4: ...
    @property
    def sosl(self) -> U8Vector4: ...
    @property
    def sots(self) -> U8Vector4: ...
    @property
    def sott(self) -> U8Vector4: ...
    @property
    def sotq(self) -> U8Vector4: ...
    @property
    def soto(self) -> U8Vector4: ...
    @property
    def sotl(self) -> U8Vector4: ...
    @property
    def soqs(self) -> U8Vector4: ...
    @property
    def soqt(self) -> U8Vector4: ...
    @property
    def soqq(self) -> U8Vector4: ...
    @property
    def soqo(self) -> U8Vector4: ...
    @property
    def soql(self) -> U8Vector4: ...
    @property
    def soos(self) -> U8Vector4: ...
    @property
    def soot(self) -> U8Vector4: ...
    @property
    def sooq(self) -> U8Vector4: ...
    @property
    def sooo(self) -> U8Vector4: ...
    @property
    def sool(self) -> U8Vector4: ...
    @property
    def sols(self) -> U8Vector4: ...
    @property
    def solt(self) -> U8Vector4: ...
    @property
    def solq(self) -> U8Vector4: ...
    @property
    def solo(self) -> U8Vector4: ...
    @property
    def soll(self) -> U8Vector4: ...
    @property
    def slss(self) -> U8Vector4: ...
    @property
    def slst(self) -> U8Vector4: ...
    @property
    def slsq(self) -> U8Vector4: ...
    @property
    def slso(self) -> U8Vector4: ...
    @property
    def slsl(self) -> U8Vector4: ...
    @property
    def slts(self) -> U8Vector4: ...
    @property
    def sltt(self) -> U8Vector4: ...
    @property
    def sltq(self) -> U8Vector4: ...
    @property
    def slto(self) -> U8Vector4: ...
    @property
    def sltl(self) -> U8Vector4: ...
    @property
    def slqs(self) -> U8Vector4: ...
    @property
    def slqt(self) -> U8Vector4: ...
    @property
    def slqq(self) -> U8Vector4: ...
    @property
    def slqo(self) -> U8Vector4: ...
    @property
    def slql(self) -> U8Vector4: ...
    @property
    def slos(self) -> U8Vector4: ...
    @property
    def slot(self) -> U8Vector4: ...
    @property
    def sloq(self) -> U8Vector4: ...
    @property
    def sloo(self) -> U8Vector4: ...
    @property
    def slol(self) -> U8Vector4: ...
    @property
    def slls(self) -> U8Vector4: ...
    @property
    def sllt(self) -> U8Vector4: ...
    @property
    def sllq(self) -> U8Vector4: ...
    @property
    def sllo(self) -> U8Vector4: ...
    @property
    def slll(self) -> U8Vector4: ...
    @property
    def tsss(self) -> U8Vector4: ...
    @property
    def tsst(self) -> U8Vector4: ...
    @property
    def tssq(self) -> U8Vector4: ...
    @property
    def tsso(self) -> U8Vector4: ...
    @property
    def tssl(self) -> U8Vector4: ...
    @property
    def tsts(self) -> U8Vector4: ...
    @property
    def tstt(self) -> U8Vector4: ...
    @property
    def tstq(self) -> U8Vector4: ...
    @property
    def tsto(self) -> U8Vector4: ...
    @property
    def tstl(self) -> U8Vector4: ...
    @property
    def tsqs(self) -> U8Vector4: ...
    @property
    def tsqt(self) -> U8Vector4: ...
    @property
    def tsqq(self) -> U8Vector4: ...
    @property
    def tsqo(self) -> U8Vector4: ...
    @property
    def tsql(self) -> U8Vector4: ...
    @property
    def tsos(self) -> U8Vector4: ...
    @property
    def tsot(self) -> U8Vector4: ...
    @property
    def tsoq(self) -> U8Vector4: ...
    @property
    def tsoo(self) -> U8Vector4: ...
    @property
    def tsol(self) -> U8Vector4: ...
    @property
    def tsls(self) -> U8Vector4: ...
    @property
    def tslt(self) -> U8Vector4: ...
    @property
    def tslq(self) -> U8Vector4: ...
    @property
    def tslo(self) -> U8Vector4: ...
    @property
    def tsll(self) -> U8Vector4: ...
    @property
    def ttss(self) -> U8Vector4: ...
    @property
    def ttst(self) -> U8Vector4: ...
    @property
    def ttsq(self) -> U8Vector4: ...
    @property
    def ttso(self) -> U8Vector4: ...
    @property
    def ttsl(self) -> U8Vector4: ...
    @property
    def ttts(self) -> U8Vector4: ...
    @property
    def tttt(self) -> U8Vector4: ...
    @property
    def tttq(self) -> U8Vector4: ...
    @property
    def ttto(self) -> U8Vector4: ...
    @property
    def tttl(self) -> U8Vector4: ...
    @property
    def ttqs(self) -> U8Vector4: ...
    @property
    def ttqt(self) -> U8Vector4: ...
    @property
    def ttqq(self) -> U8Vector4: ...
    @property
    def ttqo(self) -> U8Vector4: ...
    @property
    def ttql(self) -> U8Vector4: ...
    @property
    def ttos(self) -> U8Vector4: ...
    @property
    def ttot(self) -> U8Vector4: ...
    @property
    def ttoq(self) -> U8Vector4: ...
    @property
    def ttoo(self) -> U8Vector4: ...
    @property
    def ttol(self) -> U8Vector4: ...
    @property
    def ttls(self) -> U8Vector4: ...
    @property
    def ttlt(self) -> U8Vector4: ...
    @property
    def ttlq(self) -> U8Vector4: ...
    @property
    def ttlo(self) -> U8Vector4: ...
    @property
    def ttll(self) -> U8Vector4: ...
    @property
    def tqss(self) -> U8Vector4: ...
    @property
    def tqst(self) -> U8Vector4: ...
    @property
    def tqsq(self) -> U8Vector4: ...
    @property
    def tqso(self) -> U8Vector4: ...
    @property
    def tqsl(self) -> U8Vector4: ...
    @property
    def tqts(self) -> U8Vector4: ...
    @property
    def tqtt(self) -> U8Vector4: ...
    @property
    def tqtq(self) -> U8Vector4: ...
    @property
    def tqto(self) -> U8Vector4: ...
    @property
    def tqtl(self) -> U8Vector4: ...
    @property
    def tqqs(self) -> U8Vector4: ...
    @property
    def tqqt(self) -> U8Vector4: ...
    @property
    def tqqq(self) -> U8Vector4: ...
    @property
    def tqqo(self) -> U8Vector4: ...
    @property
    def tqql(self) -> U8Vector4: ...
    @property
    def tqos(self) -> U8Vector4: ...
    @property
    def tqot(self) -> U8Vector4: ...
    @property
    def tqoq(self) -> U8Vector4: ...
    @property
    def tqoo(self) -> U8Vector4: ...
    @property
    def tqol(self) -> U8Vector4: ...
    @property
    def tqls(self) -> U8Vector4: ...
    @property
    def tqlt(self) -> U8Vector4: ...
    @property
    def tqlq(self) -> U8Vector4: ...
    @property
    def tqlo(self) -> U8Vector4: ...
    @property
    def tqll(self) -> U8Vector4: ...
    @property
    def toss(self) -> U8Vector4: ...
    @property
    def tost(self) -> U8Vector4: ...
    @property
    def tosq(self) -> U8Vector4: ...
    @property
    def toso(self) -> U8Vector4: ...
    @property
    def tosl(self) -> U8Vector4: ...
    @property
    def tots(self) -> U8Vector4: ...
    @property
    def tott(self) -> U8Vector4: ...
    @property
    def totq(self) -> U8Vector4: ...
    @property
    def toto(self) -> U8Vector4: ...
    @property
    def totl(self) -> U8Vector4: ...
    @property
    def toqs(self) -> U8Vector4: ...
    @property
    def toqt(self) -> U8Vector4: ...
    @property
    def toqq(self) -> U8Vector4: ...
    @property
    def toqo(self) -> U8Vector4: ...
    @property
    def toql(self) -> U8Vector4: ...
    @property
    def toos(self) -> U8Vector4: ...
    @property
    def toot(self) -> U8Vector4: ...
    @property
    def tooq(self) -> U8Vector4: ...
    @property
    def tooo(self) -> U8Vector4: ...
    @property
    def tool(self) -> U8Vector4: ...
    @property
    def tols(self) -> U8Vector4: ...
    @property
    def tolt(self) -> U8Vector4: ...
    @property
    def tolq(self) -> U8Vector4: ...
    @property
    def tolo(self) -> U8Vector4: ...
    @property
    def toll(self) -> U8Vector4: ...
    @property
    def tlss(self) -> U8Vector4: ...
    @property
    def tlst(self) -> U8Vector4: ...
    @property
    def tlsq(self) -> U8Vector4: ...
    @property
    def tlso(self) -> U8Vector4: ...
    @property
    def tlsl(self) -> U8Vector4: ...
    @property
    def tlts(self) -> U8Vector4: ...
    @property
    def tltt(self) -> U8Vector4: ...
    @property
    def tltq(self) -> U8Vector4: ...
    @property
    def tlto(self) -> U8Vector4: ...
    @property
    def tltl(self) -> U8Vector4: ...
    @property
    def tlqs(self) -> U8Vector4: ...
    @property
    def tlqt(self) -> U8Vector4: ...
    @property
    def tlqq(self) -> U8Vector4: ...
    @property
    def tlqo(self) -> U8Vector4: ...
    @property
    def tlql(self) -> U8Vector4: ...
    @property
    def tlos(self) -> U8Vector4: ...
    @property
    def tlot(self) -> U8Vector4: ...
    @property
    def tloq(self) -> U8Vector4: ...
    @property
    def tloo(self) -> U8Vector4: ...
    @property
    def tlol(self) -> U8Vector4: ...
    @property
    def tlls(self) -> U8Vector4: ...
    @property
    def tllt(self) -> U8Vector4: ...
    @property
    def tllq(self) -> U8Vector4: ...
    @property
    def tllo(self) -> U8Vector4: ...
    @property
    def tlll(self) -> U8Vector4: ...
    @property
    def qsss(self) -> U8Vector4: ...
    @property
    def qsst(self) -> U8Vector4: ...
    @property
    def qssq(self) -> U8Vector4: ...
    @property
    def qsso(self) -> U8Vector4: ...
    @property
    def qssl(self) -> U8Vector4: ...
    @property
    def qsts(self) -> U8Vector4: ...
    @property
    def qstt(self) -> U8Vector4: ...
    @property
    def qstq(self) -> U8Vector4: ...
    @property
    def qsto(self) -> U8Vector4: ...
    @property
    def qstl(self) -> U8Vector4: ...
    @property
    def qsqs(self) -> U8Vector4: ...
    @property
    def qsqt(self) -> U8Vector4: ...
    @property
    def qsqq(self) -> U8Vector4: ...
    @property
    def qsqo(self) -> U8Vector4: ...
    @property
    def qsql(self) -> U8Vector4: ...
    @property
    def qsos(self) -> U8Vector4: ...
    @property
    def qsot(self) -> U8Vector4: ...
    @property
    def qsoq(self) -> U8Vector4: ...
    @property
    def qsoo(self) -> U8Vector4: ...
    @property
    def qsol(self) -> U8Vector4: ...
    @property
    def qsls(self) -> U8Vector4: ...
    @property
    def qslt(self) -> U8Vector4: ...
    @property
    def qslq(self) -> U8Vector4: ...
    @property
    def qslo(self) -> U8Vector4: ...
    @property
    def qsll(self) -> U8Vector4: ...
    @property
    def qtss(self) -> U8Vector4: ...
    @property
    def qtst(self) -> U8Vector4: ...
    @property
    def qtsq(self) -> U8Vector4: ...
    @property
    def qtso(self) -> U8Vector4: ...
    @property
    def qtsl(self) -> U8Vector4: ...
    @property
    def qtts(self) -> U8Vector4: ...
    @property
    def qttt(self) -> U8Vector4: ...
    @property
    def qttq(self) -> U8Vector4: ...
    @property
    def qtto(self) -> U8Vector4: ...
    @property
    def qttl(self) -> U8Vector4: ...
    @property
    def qtqs(self) -> U8Vector4: ...
    @property
    def qtqt(self) -> U8Vector4: ...
    @property
    def qtqq(self) -> U8Vector4: ...
    @property
    def qtqo(self) -> U8Vector4: ...
    @property
    def qtql(self) -> U8Vector4: ...
    @property
    def qtos(self) -> U8Vector4: ...
    @property
    def qtot(self) -> U8Vector4: ...
    @property
    def qtoq(self) -> U8Vector4: ...
    @property
    def qtoo(self) -> U8Vector4: ...
    @property
    def qtol(self) -> U8Vector4: ...
    @property
    def qtls(self) -> U8Vector4: ...
    @property
    def qtlt(self) -> U8Vector4: ...
    @property
    def qtlq(self) -> U8Vector4: ...
    @property
    def qtlo(self) -> U8Vector4: ...
    @property
    def qtll(self) -> U8Vector4: ...
    @property
    def qqss(self) -> U8Vector4: ...
    @property
    def qqst(self) -> U8Vector4: ...
    @property
    def qqsq(self) -> U8Vector4: ...
    @property
    def qqso(self) -> U8Vector4: ...
    @property
    def qqsl(self) -> U8Vector4: ...
    @property
    def qqts(self) -> U8Vector4: ...
    @property
    def qqtt(self) -> U8Vector4: ...
    @property
    def qqtq(self) -> U8Vector4: ...
    @property
    def qqto(self) -> U8Vector4: ...
    @property
    def qqtl(self) -> U8Vector4: ...
    @property
    def qqqs(self) -> U8Vector4: ...
    @property
    def qqqt(self) -> U8Vector4: ...
    @property
    def qqqq(self) -> U8Vector4: ...
    @property
    def qqqo(self) -> U8Vector4: ...
    @property
    def qqql(self) -> U8Vector4: ...
    @property
    def qqos(self) -> U8Vector4: ...
    @property
    def qqot(self) -> U8Vector4: ...
    @property
    def qqoq(self) -> U8Vector4: ...
    @property
    def qqoo(self) -> U8Vector4: ...
    @property
    def qqol(self) -> U8Vector4: ...
    @property
    def qqls(self) -> U8Vector4: ...
    @property
    def qqlt(self) -> U8Vector4: ...
    @property
    def qqlq(self) -> U8Vector4: ...
    @property
    def qqlo(self) -> U8Vector4: ...
    @property
    def qqll(self) -> U8Vector4: ...
    @property
    def qoss(self) -> U8Vector4: ...
    @property
    def qost(self) -> U8Vector4: ...
    @property
    def qosq(self) -> U8Vector4: ...
    @property
    def qoso(self) -> U8Vector4: ...
    @property
    def qosl(self) -> U8Vector4: ...
    @property
    def qots(self) -> U8Vector4: ...
    @property
    def qott(self) -> U8Vector4: ...
    @property
    def qotq(self) -> U8Vector4: ...
    @property
    def qoto(self) -> U8Vector4: ...
    @property
    def qotl(self) -> U8Vector4: ...
    @property
    def qoqs(self) -> U8Vector4: ...
    @property
    def qoqt(self) -> U8Vector4: ...
    @property
    def qoqq(self) -> U8Vector4: ...
    @property
    def qoqo(self) -> U8Vector4: ...
    @property
    def qoql(self) -> U8Vector4: ...
    @property
    def qoos(self) -> U8Vector4: ...
    @property
    def qoot(self) -> U8Vector4: ...
    @property
    def qooq(self) -> U8Vector4: ...
    @property
    def qooo(self) -> U8Vector4: ...
    @property
    def qool(self) -> U8Vector4: ...
    @property
    def qols(self) -> U8Vector4: ...
    @property
    def qolt(self) -> U8Vector4: ...
    @property
    def qolq(self) -> U8Vector4: ...
    @property
    def qolo(self) -> U8Vector4: ...
    @property
    def qoll(self) -> U8Vector4: ...
    @property
    def qlss(self) -> U8Vector4: ...
    @property
    def qlst(self) -> U8Vector4: ...
    @property
    def qlsq(self) -> U8Vector4: ...
    @property
    def qlso(self) -> U8Vector4: ...
    @property
    def qlsl(self) -> U8Vector4: ...
    @property
    def qlts(self) -> U8Vector4: ...
    @property
    def qltt(self) -> U8Vector4: ...
    @property
    def qltq(self) -> U8Vector4: ...
    @property
    def qlto(self) -> U8Vector4: ...
    @property
    def qltl(self) -> U8Vector4: ...
    @property
    def qlqs(self) -> U8Vector4: ...
    @property
    def qlqt(self) -> U8Vector4: ...
    @property
    def qlqq(self) -> U8Vector4: ...
    @property
    def qlqo(self) -> U8Vector4: ...
    @property
    def qlql(self) -> U8Vector4: ...
    @property
    def qlos(self) -> U8Vector4: ...
    @property
    def qlot(self) -> U8Vector4: ...
    @property
    def qloq(self) -> U8Vector4: ...
    @property
    def qloo(self) -> U8Vector4: ...
    @property
    def qlol(self) -> U8Vector4: ...
    @property
    def qlls(self) -> U8Vector4: ...
    @property
    def qllt(self) -> U8Vector4: ...
    @property
    def qllq(self) -> U8Vector4: ...
    @property
    def qllo(self) -> U8Vector4: ...
    @property
    def qlll(self) -> U8Vector4: ...
    @property
    def osss(self) -> U8Vector4: ...
    @property
    def osst(self) -> U8Vector4: ...
    @property
    def ossq(self) -> U8Vector4: ...
    @property
    def osso(self) -> U8Vector4: ...
    @property
    def ossl(self) -> U8Vector4: ...
    @property
    def osts(self) -> U8Vector4: ...
    @property
    def ostt(self) -> U8Vector4: ...
    @property
    def ostq(self) -> U8Vector4: ...
    @property
    def osto(self) -> U8Vector4: ...
    @property
    def ostl(self) -> U8Vector4: ...
    @property
    def osqs(self) -> U8Vector4: ...
    @property
    def osqt(self) -> U8Vector4: ...
    @property
    def osqq(self) -> U8Vector4: ...
    @property
    def osqo(self) -> U8Vector4: ...
    @property
    def osql(self) -> U8Vector4: ...
    @property
    def osos(self) -> U8Vector4: ...
    @property
    def osot(self) -> U8Vector4: ...
    @property
    def osoq(self) -> U8Vector4: ...
    @property
    def osoo(self) -> U8Vector4: ...
    @property
    def osol(self) -> U8Vector4: ...
    @property
    def osls(self) -> U8Vector4: ...
    @property
    def oslt(self) -> U8Vector4: ...
    @property
    def oslq(self) -> U8Vector4: ...
    @property
    def oslo(self) -> U8Vector4: ...
    @property
    def osll(self) -> U8Vector4: ...
    @property
    def otss(self) -> U8Vector4: ...
    @property
    def otst(self) -> U8Vector4: ...
    @property
    def otsq(self) -> U8Vector4: ...
    @property
    def otso(self) -> U8Vector4: ...
    @property
    def otsl(self) -> U8Vector4: ...
    @property
    def otts(self) -> U8Vector4: ...
    @property
    def ottt(self) -> U8Vector4: ...
    @property
    def ottq(self) -> U8Vector4: ...
    @property
    def otto(self) -> U8Vector4: ...
    @property
    def ottl(self) -> U8Vector4: ...
    @property
    def otqs(self) -> U8Vector4: ...
    @property
    def otqt(self) -> U8Vector4: ...
    @property
    def otqq(self) -> U8Vector4: ...
    @property
    def otqo(self) -> U8Vector4: ...
    @property
    def otql(self) -> U8Vector4: ...
    @property
    def otos(self) -> U8Vector4: ...
    @property
    def otot(self) -> U8Vector4: ...
    @property
    def otoq(self) -> U8Vector4: ...
    @property
    def otoo(self) -> U8Vector4: ...
    @property
    def otol(self) -> U8Vector4: ...
    @property
    def otls(self) -> U8Vector4: ...
    @property
    def otlt(self) -> U8Vector4: ...
    @property
    def otlq(self) -> U8Vector4: ...
    @property
    def otlo(self) -> U8Vector4: ...
    @property
    def otll(self) -> U8Vector4: ...
    @property
    def oqss(self) -> U8Vector4: ...
    @property
    def oqst(self) -> U8Vector4: ...
    @property
    def oqsq(self) -> U8Vector4: ...
    @property
    def oqso(self) -> U8Vector4: ...
    @property
    def oqsl(self) -> U8Vector4: ...
    @property
    def oqts(self) -> U8Vector4: ...
    @property
    def oqtt(self) -> U8Vector4: ...
    @property
    def oqtq(self) -> U8Vector4: ...
    @property
    def oqto(self) -> U8Vector4: ...
    @property
    def oqtl(self) -> U8Vector4: ...
    @property
    def oqqs(self) -> U8Vector4: ...
    @property
    def oqqt(self) -> U8Vector4: ...
    @property
    def oqqq(self) -> U8Vector4: ...
    @property
    def oqqo(self) -> U8Vector4: ...
    @property
    def oqql(self) -> U8Vector4: ...
    @property
    def oqos(self) -> U8Vector4: ...
    @property
    def oqot(self) -> U8Vector4: ...
    @property
    def oqoq(self) -> U8Vector4: ...
    @property
    def oqoo(self) -> U8Vector4: ...
    @property
    def oqol(self) -> U8Vector4: ...
    @property
    def oqls(self) -> U8Vector4: ...
    @property
    def oqlt(self) -> U8Vector4: ...
    @property
    def oqlq(self) -> U8Vector4: ...
    @property
    def oqlo(self) -> U8Vector4: ...
    @property
    def oqll(self) -> U8Vector4: ...
    @property
    def ooss(self) -> U8Vector4: ...
    @property
    def oost(self) -> U8Vector4: ...
    @property
    def oosq(self) -> U8Vector4: ...
    @property
    def ooso(self) -> U8Vector4: ...
    @property
    def oosl(self) -> U8Vector4: ...
    @property
    def oots(self) -> U8Vector4: ...
    @property
    def oott(self) -> U8Vector4: ...
    @property
    def ootq(self) -> U8Vector4: ...
    @property
    def ooto(self) -> U8Vector4: ...
    @property
    def ootl(self) -> U8Vector4: ...
    @property
    def ooqs(self) -> U8Vector4: ...
    @property
    def ooqt(self) -> U8Vector4: ...
    @property
    def ooqq(self) -> U8Vector4: ...
    @property
    def ooqo(self) -> U8Vector4: ...
    @property
    def ooql(self) -> U8Vector4: ...
    @property
    def ooos(self) -> U8Vector4: ...
    @property
    def ooot(self) -> U8Vector4: ...
    @property
    def oooq(self) -> U8Vector4: ...
    @property
    def ools(self) -> U8Vector4: ...
    @property
    def oolt(self) -> U8Vector4: ...
    @property
    def oolq(self) -> U8Vector4: ...
    @property
    def olss(self) -> U8Vector4: ...
    @property
    def olst(self) -> U8Vector4: ...
    @property
    def olsq(self) -> U8Vector4: ...
    @property
    def olso(self) -> U8Vector4: ...
    @property
    def olsl(self) -> U8Vector4: ...
    @property
    def olts(self) -> U8Vector4: ...
    @property
    def oltt(self) -> U8Vector4: ...
    @property
    def oltq(self) -> U8Vector4: ...
    @property
    def olto(self) -> U8Vector4: ...
    @property
    def oltl(self) -> U8Vector4: ...
    @property
    def olqs(self) -> U8Vector4: ...
    @property
    def olqt(self) -> U8Vector4: ...
    @property
    def olqq(self) -> U8Vector4: ...
    @property
    def olqo(self) -> U8Vector4: ...
    @property
    def olql(self) -> U8Vector4: ...
    @property
    def olos(self) -> U8Vector4: ...
    @property
    def olot(self) -> U8Vector4: ...
    @property
    def oloq(self) -> U8Vector4: ...
    @property
    def olls(self) -> U8Vector4: ...
    @property
    def ollt(self) -> U8Vector4: ...
    @property
    def ollq(self) -> U8Vector4: ...
    @property
    def lsss(self) -> U8Vector4: ...
    @property
    def lsst(self) -> U8Vector4: ...
    @property
    def lssq(self) -> U8Vector4: ...
    @property
    def lsso(self) -> U8Vector4: ...
    @property
    def lssl(self) -> U8Vector4: ...
    @property
    def lsts(self) -> U8Vector4: ...
    @property
    def lstt(self) -> U8Vector4: ...
    @property
    def lstq(self) -> U8Vector4: ...
    @property
    def lsto(self) -> U8Vector4: ...
    @property
    def lstl(self) -> U8Vector4: ...
    @property
    def lsqs(self) -> U8Vector4: ...
    @property
    def lsqt(self) -> U8Vector4: ...
    @property
    def lsqq(self) -> U8Vector4: ...
    @property
    def lsqo(self) -> U8Vector4: ...
    @property
    def lsql(self) -> U8Vector4: ...
    @property
    def lsos(self) -> U8Vector4: ...
    @property
    def lsot(self) -> U8Vector4: ...
    @property
    def lsoq(self) -> U8Vector4: ...
    @property
    def lsoo(self) -> U8Vector4: ...
    @property
    def lsol(self) -> U8Vector4: ...
    @property
    def lsls(self) -> U8Vector4: ...
    @property
    def lslt(self) -> U8Vector4: ...
    @property
    def lslq(self) -> U8Vector4: ...
    @property
    def lslo(self) -> U8Vector4: ...
    @property
    def lsll(self) -> U8Vector4: ...
    @property
    def ltss(self) -> U8Vector4: ...
    @property
    def ltst(self) -> U8Vector4: ...
    @property
    def ltsq(self) -> U8Vector4: ...
    @property
    def ltso(self) -> U8Vector4: ...
    @property
    def ltsl(self) -> U8Vector4: ...
    @property
    def ltts(self) -> U8Vector4: ...
    @property
    def lttt(self) -> U8Vector4: ...
    @property
    def lttq(self) -> U8Vector4: ...
    @property
    def ltto(self) -> U8Vector4: ...
    @property
    def lttl(self) -> U8Vector4: ...
    @property
    def ltqs(self) -> U8Vector4: ...
    @property
    def ltqt(self) -> U8Vector4: ...
    @property
    def ltqq(self) -> U8Vector4: ...
    @property
    def ltqo(self) -> U8Vector4: ...
    @property
    def ltql(self) -> U8Vector4: ...
    @property
    def ltos(self) -> U8Vector4: ...
    @property
    def ltot(self) -> U8Vector4: ...
    @property
    def ltoq(self) -> U8Vector4: ...
    @property
    def ltoo(self) -> U8Vector4: ...
    @property
    def ltol(self) -> U8Vector4: ...
    @property
    def ltls(self) -> U8Vector4: ...
    @property
    def ltlt(self) -> U8Vector4: ...
    @property
    def ltlq(self) -> U8Vector4: ...
    @property
    def ltlo(self) -> U8Vector4: ...
    @property
    def ltll(self) -> U8Vector4: ...
    @property
    def lqss(self) -> U8Vector4: ...
    @property
    def lqst(self) -> U8Vector4: ...
    @property
    def lqsq(self) -> U8Vector4: ...
    @property
    def lqso(self) -> U8Vector4: ...
    @property
    def lqsl(self) -> U8Vector4: ...
    @property
    def lqts(self) -> U8Vector4: ...
    @property
    def lqtt(self) -> U8Vector4: ...
    @property
    def lqtq(self) -> U8Vector4: ...
    @property
    def lqto(self) -> U8Vector4: ...
    @property
    def lqtl(self) -> U8Vector4: ...
    @property
    def lqqs(self) -> U8Vector4: ...
    @property
    def lqqt(self) -> U8Vector4: ...
    @property
    def lqqq(self) -> U8Vector4: ...
    @property
    def lqqo(self) -> U8Vector4: ...
    @property
    def lqql(self) -> U8Vector4: ...
    @property
    def lqos(self) -> U8Vector4: ...
    @property
    def lqot(self) -> U8Vector4: ...
    @property
    def lqoq(self) -> U8Vector4: ...
    @property
    def lqoo(self) -> U8Vector4: ...
    @property
    def lqol(self) -> U8Vector4: ...
    @property
    def lqls(self) -> U8Vector4: ...
    @property
    def lqlt(self) -> U8Vector4: ...
    @property
    def lqlq(self) -> U8Vector4: ...
    @property
    def lqlo(self) -> U8Vector4: ...
    @property
    def lqll(self) -> U8Vector4: ...
    @property
    def loss(self) -> U8Vector4: ...
    @property
    def lost(self) -> U8Vector4: ...
    @property
    def losq(self) -> U8Vector4: ...
    @property
    def loso(self) -> U8Vector4: ...
    @property
    def losl(self) -> U8Vector4: ...
    @property
    def lots(self) -> U8Vector4: ...
    @property
    def lott(self) -> U8Vector4: ...
    @property
    def lotq(self) -> U8Vector4: ...
    @property
    def loto(self) -> U8Vector4: ...
    @property
    def lotl(self) -> U8Vector4: ...
    @property
    def loqs(self) -> U8Vector4: ...
    @property
    def loqt(self) -> U8Vector4: ...
    @property
    def loqq(self) -> U8Vector4: ...
    @property
    def loqo(self) -> U8Vector4: ...
    @property
    def loql(self) -> U8Vector4: ...
    @property
    def loos(self) -> U8Vector4: ...
    @property
    def loot(self) -> U8Vector4: ...
    @property
    def looq(self) -> U8Vector4: ...
    @property
    def lols(self) -> U8Vector4: ...
    @property
    def lolt(self) -> U8Vector4: ...
    @property
    def lolq(self) -> U8Vector4: ...
    @property
    def llss(self) -> U8Vector4: ...
    @property
    def llst(self) -> U8Vector4: ...
    @property
    def llsq(self) -> U8Vector4: ...
    @property
    def llso(self) -> U8Vector4: ...
    @property
    def llsl(self) -> U8Vector4: ...
    @property
    def llts(self) -> U8Vector4: ...
    @property
    def lltt(self) -> U8Vector4: ...
    @property
    def lltq(self) -> U8Vector4: ...
    @property
    def llto(self) -> U8Vector4: ...
    @property
    def lltl(self) -> U8Vector4: ...
    @property
    def llqs(self) -> U8Vector4: ...
    @property
    def llqt(self) -> U8Vector4: ...
    @property
    def llqq(self) -> U8Vector4: ...
    @property
    def llqo(self) -> U8Vector4: ...
    @property
    def llql(self) -> U8Vector4: ...
    @property
    def llos(self) -> U8Vector4: ...
    @property
    def llot(self) -> U8Vector4: ...
    @property
    def lloq(self) -> U8Vector4: ...
    @property
    def llls(self) -> U8Vector4: ...
    @property
    def lllt(self) -> U8Vector4: ...
    @property
    def lllq(self) -> U8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U8Vector2: ...
    @property
    def uv(self) -> U8Vector2: ...
    @property
    def uo(self) -> U8Vector2: ...
    @property
    def ul(self) -> U8Vector2: ...
    @property
    def vu(self) -> U8Vector2: ...
    @property
    def vv(self) -> U8Vector2: ...
    @property
    def vo(self) -> U8Vector2: ...
    @property
    def vl(self) -> U8Vector2: ...
    @property
    def ou(self) -> U8Vector2: ...
    @property
    def ov(self) -> U8Vector2: ...
    @property
    def lu(self) -> U8Vector2: ...
    @property
    def lv(self) -> U8Vector2: ...
    @property
    def uuu(self) -> U8Vector3: ...
    @property
    def uuv(self) -> U8Vector3: ...
    @property
    def uuo(self) -> U8Vector3: ...
    @property
    def uul(self) -> U8Vector3: ...
    @property
    def uvu(self) -> U8Vector3: ...
    @property
    def uvv(self) -> U8Vector3: ...
    @property
    def uvo(self) -> U8Vector3: ...
    @property
    def uvl(self) -> U8Vector3: ...
    @property
    def uou(self) -> U8Vector3: ...
    @property
    def uov(self) -> U8Vector3: ...
    @property
    def uoo(self) -> U8Vector3: ...
    @property
    def uol(self) -> U8Vector3: ...
    @property
    def ulu(self) -> U8Vector3: ...
    @property
    def ulv(self) -> U8Vector3: ...
    @property
    def ulo(self) -> U8Vector3: ...
    @property
    def ull(self) -> U8Vector3: ...
    @property
    def vuu(self) -> U8Vector3: ...
    @property
    def vuv(self) -> U8Vector3: ...
    @property
    def vuo(self) -> U8Vector3: ...
    @property
    def vul(self) -> U8Vector3: ...
    @property
    def vvu(self) -> U8Vector3: ...
    @property
    def vvv(self) -> U8Vector3: ...
    @property
    def vvo(self) -> U8Vector3: ...
    @property
    def vvl(self) -> U8Vector3: ...
    @property
    def vou(self) -> U8Vector3: ...
    @property
    def vov(self) -> U8Vector3: ...
    @property
    def voo(self) -> U8Vector3: ...
    @property
    def vol(self) -> U8Vector3: ...
    @property
    def vlu(self) -> U8Vector3: ...
    @property
    def vlv(self) -> U8Vector3: ...
    @property
    def vlo(self) -> U8Vector3: ...
    @property
    def vll(self) -> U8Vector3: ...
    @property
    def ouu(self) -> U8Vector3: ...
    @property
    def ouv(self) -> U8Vector3: ...
    @property
    def ouo(self) -> U8Vector3: ...
    @property
    def oul(self) -> U8Vector3: ...
    @property
    def ovu(self) -> U8Vector3: ...
    @property
    def ovv(self) -> U8Vector3: ...
    @property
    def ovo(self) -> U8Vector3: ...
    @property
    def ovl(self) -> U8Vector3: ...
    @property
    def oou(self) -> U8Vector3: ...
    @property
    def oov(self) -> U8Vector3: ...
    @property
    def olu(self) -> U8Vector3: ...
    @property
    def olv(self) -> U8Vector3: ...
    @property
    def luu(self) -> U8Vector3: ...
    @property
    def luv(self) -> U8Vector3: ...
    @property
    def luo(self) -> U8Vector3: ...
    @property
    def lul(self) -> U8Vector3: ...
    @property
    def lvu(self) -> U8Vector3: ...
    @property
    def lvv(self) -> U8Vector3: ...
    @property
    def lvo(self) -> U8Vector3: ...
    @property
    def lvl(self) -> U8Vector3: ...
    @property
    def lou(self) -> U8Vector3: ...
    @property
    def lov(self) -> U8Vector3: ...
    @property
    def llu(self) -> U8Vector3: ...
    @property
    def llv(self) -> U8Vector3: ...
    @property
    def uuuu(self) -> U8Vector4: ...
    @property
    def uuuv(self) -> U8Vector4: ...
    @property
    def uuuo(self) -> U8Vector4: ...
    @property
    def uuul(self) -> U8Vector4: ...
    @property
    def uuvu(self) -> U8Vector4: ...
    @property
    def uuvv(self) -> U8Vector4: ...
    @property
    def uuvo(self) -> U8Vector4: ...
    @property
    def uuvl(self) -> U8Vector4: ...
    @property
    def uuou(self) -> U8Vector4: ...
    @property
    def uuov(self) -> U8Vector4: ...
    @property
    def uuoo(self) -> U8Vector4: ...
    @property
    def uuol(self) -> U8Vector4: ...
    @property
    def uulu(self) -> U8Vector4: ...
    @property
    def uulv(self) -> U8Vector4: ...
    @property
    def uulo(self) -> U8Vector4: ...
    @property
    def uull(self) -> U8Vector4: ...
    @property
    def uvuu(self) -> U8Vector4: ...
    @property
    def uvuv(self) -> U8Vector4: ...
    @property
    def uvuo(self) -> U8Vector4: ...
    @property
    def uvul(self) -> U8Vector4: ...
    @property
    def uvvu(self) -> U8Vector4: ...
    @property
    def uvvv(self) -> U8Vector4: ...
    @property
    def uvvo(self) -> U8Vector4: ...
    @property
    def uvvl(self) -> U8Vector4: ...
    @property
    def uvou(self) -> U8Vector4: ...
    @property
    def uvov(self) -> U8Vector4: ...
    @property
    def uvoo(self) -> U8Vector4: ...
    @property
    def uvol(self) -> U8Vector4: ...
    @property
    def uvlu(self) -> U8Vector4: ...
    @property
    def uvlv(self) -> U8Vector4: ...
    @property
    def uvlo(self) -> U8Vector4: ...
    @property
    def uvll(self) -> U8Vector4: ...
    @property
    def uouu(self) -> U8Vector4: ...
    @property
    def uouv(self) -> U8Vector4: ...
    @property
    def uouo(self) -> U8Vector4: ...
    @property
    def uoul(self) -> U8Vector4: ...
    @property
    def uovu(self) -> U8Vector4: ...
    @property
    def uovv(self) -> U8Vector4: ...
    @property
    def uovo(self) -> U8Vector4: ...
    @property
    def uovl(self) -> U8Vector4: ...
    @property
    def uoou(self) -> U8Vector4: ...
    @property
    def uoov(self) -> U8Vector4: ...
    @property
    def uooo(self) -> U8Vector4: ...
    @property
    def uool(self) -> U8Vector4: ...
    @property
    def uolu(self) -> U8Vector4: ...
    @property
    def uolv(self) -> U8Vector4: ...
    @property
    def uolo(self) -> U8Vector4: ...
    @property
    def uoll(self) -> U8Vector4: ...
    @property
    def uluu(self) -> U8Vector4: ...
    @property
    def uluv(self) -> U8Vector4: ...
    @property
    def uluo(self) -> U8Vector4: ...
    @property
    def ulul(self) -> U8Vector4: ...
    @property
    def ulvu(self) -> U8Vector4: ...
    @property
    def ulvv(self) -> U8Vector4: ...
    @property
    def ulvo(self) -> U8Vector4: ...
    @property
    def ulvl(self) -> U8Vector4: ...
    @property
    def ulou(self) -> U8Vector4: ...
    @property
    def ulov(self) -> U8Vector4: ...
    @property
    def uloo(self) -> U8Vector4: ...
    @property
    def ulol(self) -> U8Vector4: ...
    @property
    def ullu(self) -> U8Vector4: ...
    @property
    def ullv(self) -> U8Vector4: ...
    @property
    def ullo(self) -> U8Vector4: ...
    @property
    def ulll(self) -> U8Vector4: ...
    @property
    def vuuu(self) -> U8Vector4: ...
    @property
    def vuuv(self) -> U8Vector4: ...
    @property
    def vuuo(self) -> U8Vector4: ...
    @property
    def vuul(self) -> U8Vector4: ...
    @property
    def vuvu(self) -> U8Vector4: ...
    @property
    def vuvv(self) -> U8Vector4: ...
    @property
    def vuvo(self) -> U8Vector4: ...
    @property
    def vuvl(self) -> U8Vector4: ...
    @property
    def vuou(self) -> U8Vector4: ...
    @property
    def vuov(self) -> U8Vector4: ...
    @property
    def vuoo(self) -> U8Vector4: ...
    @property
    def vuol(self) -> U8Vector4: ...
    @property
    def vulu(self) -> U8Vector4: ...
    @property
    def vulv(self) -> U8Vector4: ...
    @property
    def vulo(self) -> U8Vector4: ...
    @property
    def vull(self) -> U8Vector4: ...
    @property
    def vvuu(self) -> U8Vector4: ...
    @property
    def vvuv(self) -> U8Vector4: ...
    @property
    def vvuo(self) -> U8Vector4: ...
    @property
    def vvul(self) -> U8Vector4: ...
    @property
    def vvvu(self) -> U8Vector4: ...
    @property
    def vvvv(self) -> U8Vector4: ...
    @property
    def vvvo(self) -> U8Vector4: ...
    @property
    def vvvl(self) -> U8Vector4: ...
    @property
    def vvou(self) -> U8Vector4: ...
    @property
    def vvov(self) -> U8Vector4: ...
    @property
    def vvoo(self) -> U8Vector4: ...
    @property
    def vvol(self) -> U8Vector4: ...
    @property
    def vvlu(self) -> U8Vector4: ...
    @property
    def vvlv(self) -> U8Vector4: ...
    @property
    def vvlo(self) -> U8Vector4: ...
    @property
    def vvll(self) -> U8Vector4: ...
    @property
    def vouu(self) -> U8Vector4: ...
    @property
    def vouv(self) -> U8Vector4: ...
    @property
    def vouo(self) -> U8Vector4: ...
    @property
    def voul(self) -> U8Vector4: ...
    @property
    def vovu(self) -> U8Vector4: ...
    @property
    def vovv(self) -> U8Vector4: ...
    @property
    def vovo(self) -> U8Vector4: ...
    @property
    def vovl(self) -> U8Vector4: ...
    @property
    def voou(self) -> U8Vector4: ...
    @property
    def voov(self) -> U8Vector4: ...
    @property
    def vooo(self) -> U8Vector4: ...
    @property
    def vool(self) -> U8Vector4: ...
    @property
    def volu(self) -> U8Vector4: ...
    @property
    def volv(self) -> U8Vector4: ...
    @property
    def volo(self) -> U8Vector4: ...
    @property
    def voll(self) -> U8Vector4: ...
    @property
    def vluu(self) -> U8Vector4: ...
    @property
    def vluv(self) -> U8Vector4: ...
    @property
    def vluo(self) -> U8Vector4: ...
    @property
    def vlul(self) -> U8Vector4: ...
    @property
    def vlvu(self) -> U8Vector4: ...
    @property
    def vlvv(self) -> U8Vector4: ...
    @property
    def vlvo(self) -> U8Vector4: ...
    @property
    def vlvl(self) -> U8Vector4: ...
    @property
    def vlou(self) -> U8Vector4: ...
    @property
    def vlov(self) -> U8Vector4: ...
    @property
    def vloo(self) -> U8Vector4: ...
    @property
    def vlol(self) -> U8Vector4: ...
    @property
    def vllu(self) -> U8Vector4: ...
    @property
    def vllv(self) -> U8Vector4: ...
    @property
    def vllo(self) -> U8Vector4: ...
    @property
    def vlll(self) -> U8Vector4: ...
    @property
    def ouuu(self) -> U8Vector4: ...
    @property
    def ouuv(self) -> U8Vector4: ...
    @property
    def ouuo(self) -> U8Vector4: ...
    @property
    def ouul(self) -> U8Vector4: ...
    @property
    def ouvu(self) -> U8Vector4: ...
    @property
    def ouvv(self) -> U8Vector4: ...
    @property
    def ouvo(self) -> U8Vector4: ...
    @property
    def ouvl(self) -> U8Vector4: ...
    @property
    def ouou(self) -> U8Vector4: ...
    @property
    def ouov(self) -> U8Vector4: ...
    @property
    def ouoo(self) -> U8Vector4: ...
    @property
    def ouol(self) -> U8Vector4: ...
    @property
    def oulu(self) -> U8Vector4: ...
    @property
    def oulv(self) -> U8Vector4: ...
    @property
    def oulo(self) -> U8Vector4: ...
    @property
    def oull(self) -> U8Vector4: ...
    @property
    def ovuu(self) -> U8Vector4: ...
    @property
    def ovuv(self) -> U8Vector4: ...
    @property
    def ovuo(self) -> U8Vector4: ...
    @property
    def ovul(self) -> U8Vector4: ...
    @property
    def ovvu(self) -> U8Vector4: ...
    @property
    def ovvv(self) -> U8Vector4: ...
    @property
    def ovvo(self) -> U8Vector4: ...
    @property
    def ovvl(self) -> U8Vector4: ...
    @property
    def ovou(self) -> U8Vector4: ...
    @property
    def ovov(self) -> U8Vector4: ...
    @property
    def ovoo(self) -> U8Vector4: ...
    @property
    def ovol(self) -> U8Vector4: ...
    @property
    def ovlu(self) -> U8Vector4: ...
    @property
    def ovlv(self) -> U8Vector4: ...
    @property
    def ovlo(self) -> U8Vector4: ...
    @property
    def ovll(self) -> U8Vector4: ...
    @property
    def oouu(self) -> U8Vector4: ...
    @property
    def oouv(self) -> U8Vector4: ...
    @property
    def oouo(self) -> U8Vector4: ...
    @property
    def ooul(self) -> U8Vector4: ...
    @property
    def oovu(self) -> U8Vector4: ...
    @property
    def oovv(self) -> U8Vector4: ...
    @property
    def oovo(self) -> U8Vector4: ...
    @property
    def oovl(self) -> U8Vector4: ...
    @property
    def ooou(self) -> U8Vector4: ...
    @property
    def ooov(self) -> U8Vector4: ...
    @property
    def oolu(self) -> U8Vector4: ...
    @property
    def oolv(self) -> U8Vector4: ...
    @property
    def oluu(self) -> U8Vector4: ...
    @property
    def oluv(self) -> U8Vector4: ...
    @property
    def oluo(self) -> U8Vector4: ...
    @property
    def olul(self) -> U8Vector4: ...
    @property
    def olvu(self) -> U8Vector4: ...
    @property
    def olvv(self) -> U8Vector4: ...
    @property
    def olvo(self) -> U8Vector4: ...
    @property
    def olvl(self) -> U8Vector4: ...
    @property
    def olou(self) -> U8Vector4: ...
    @property
    def olov(self) -> U8Vector4: ...
    @property
    def ollu(self) -> U8Vector4: ...
    @property
    def ollv(self) -> U8Vector4: ...
    @property
    def luuu(self) -> U8Vector4: ...
    @property
    def luuv(self) -> U8Vector4: ...
    @property
    def luuo(self) -> U8Vector4: ...
    @property
    def luul(self) -> U8Vector4: ...
    @property
    def luvu(self) -> U8Vector4: ...
    @property
    def luvv(self) -> U8Vector4: ...
    @property
    def luvo(self) -> U8Vector4: ...
    @property
    def luvl(self) -> U8Vector4: ...
    @property
    def luou(self) -> U8Vector4: ...
    @property
    def luov(self) -> U8Vector4: ...
    @property
    def luoo(self) -> U8Vector4: ...
    @property
    def luol(self) -> U8Vector4: ...
    @property
    def lulu(self) -> U8Vector4: ...
    @property
    def lulv(self) -> U8Vector4: ...
    @property
    def lulo(self) -> U8Vector4: ...
    @property
    def lull(self) -> U8Vector4: ...
    @property
    def lvuu(self) -> U8Vector4: ...
    @property
    def lvuv(self) -> U8Vector4: ...
    @property
    def lvuo(self) -> U8Vector4: ...
    @property
    def lvul(self) -> U8Vector4: ...
    @property
    def lvvu(self) -> U8Vector4: ...
    @property
    def lvvv(self) -> U8Vector4: ...
    @property
    def lvvo(self) -> U8Vector4: ...
    @property
    def lvvl(self) -> U8Vector4: ...
    @property
    def lvou(self) -> U8Vector4: ...
    @property
    def lvov(self) -> U8Vector4: ...
    @property
    def lvoo(self) -> U8Vector4: ...
    @property
    def lvol(self) -> U8Vector4: ...
    @property
    def lvlu(self) -> U8Vector4: ...
    @property
    def lvlv(self) -> U8Vector4: ...
    @property
    def lvlo(self) -> U8Vector4: ...
    @property
    def lvll(self) -> U8Vector4: ...
    @property
    def louu(self) -> U8Vector4: ...
    @property
    def louv(self) -> U8Vector4: ...
    @property
    def louo(self) -> U8Vector4: ...
    @property
    def loul(self) -> U8Vector4: ...
    @property
    def lovu(self) -> U8Vector4: ...
    @property
    def lovv(self) -> U8Vector4: ...
    @property
    def lovo(self) -> U8Vector4: ...
    @property
    def lovl(self) -> U8Vector4: ...
    @property
    def loou(self) -> U8Vector4: ...
    @property
    def loov(self) -> U8Vector4: ...
    @property
    def lolu(self) -> U8Vector4: ...
    @property
    def lolv(self) -> U8Vector4: ...
    @property
    def lluu(self) -> U8Vector4: ...
    @property
    def lluv(self) -> U8Vector4: ...
    @property
    def lluo(self) -> U8Vector4: ...
    @property
    def llul(self) -> U8Vector4: ...
    @property
    def llvu(self) -> U8Vector4: ...
    @property
    def llvv(self) -> U8Vector4: ...
    @property
    def llvo(self) -> U8Vector4: ...
    @property
    def llvl(self) -> U8Vector4: ...
    @property
    def llou(self) -> U8Vector4: ...
    @property
    def llov(self) -> U8Vector4: ...
    @property
    def lllu(self) -> U8Vector4: ...
    @property
    def lllv(self) -> U8Vector4: ...
    @property
    def oo(self) -> U8Vector2: ...
    @property
    def ol(self) -> U8Vector2: ...
    @property
    def lo(self) -> U8Vector2: ...
    @property
    def ll(self) -> U8Vector2: ...
    @property
    def ooo(self) -> U8Vector3: ...
    @property
    def ool(self) -> U8Vector3: ...
    @property
    def olo(self) -> U8Vector3: ...
    @property
    def oll(self) -> U8Vector3: ...
    @property
    def loo(self) -> U8Vector3: ...
    @property
    def lol(self) -> U8Vector3: ...
    @property
    def llo(self) -> U8Vector3: ...
    @property
    def lll(self) -> U8Vector3: ...
    @property
    def oooo(self) -> U8Vector4: ...
    @property
    def oool(self) -> U8Vector4: ...
    @property
    def oolo(self) -> U8Vector4: ...
    @property
    def ooll(self) -> U8Vector4: ...
    @property
    def oloo(self) -> U8Vector4: ...
    @property
    def olol(self) -> U8Vector4: ...
    @property
    def ollo(self) -> U8Vector4: ...
    @property
    def olll(self) -> U8Vector4: ...
    @property
    def looo(self) -> U8Vector4: ...
    @property
    def lool(self) -> U8Vector4: ...
    @property
    def lolo(self) -> U8Vector4: ...
    @property
    def loll(self) -> U8Vector4: ...
    @property
    def lloo(self) -> U8Vector4: ...
    @property
    def llol(self) -> U8Vector4: ...
    @property
    def lllo(self) -> U8Vector4: ...
    @property
    def llll(self) -> U8Vector4: ...
    def min(self, n: Number, /) -> U8Vector3: ...
    def max(self, n: Number, /) -> U8Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> U8Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U8Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector3: ...

@final
class U8Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U8Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U8Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> U8Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> U8Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[U8Vector3]: ...

@final
class I16Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __add__(self, other: Number) -> I16Vector3: ...
    @overload
    def __radd__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __radd__(self, other: Number) -> I16Vector3: ...
    @overload
    def __sub__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __sub__(self, other: Number) -> I16Vector3: ...
    @overload
    def __rsub__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> I16Vector3: ...
    @overload
    def __mul__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __mul__(self, other: Number) -> I16Vector3: ...
    @overload
    def __rmul__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> I16Vector3: ...
    @overload
    def __truediv__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> I16Vector3: ...
    @overload
    def __rtruediv__(self, other: I16Vector3) -> I16Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> I16Vector3: ...
    def __neg__(self) -> I16Vector3: ...
    def __abs__(self) -> I16Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> I16Vector2: ...
    @property
    def xy(self) -> I16Vector2: ...
    @property
    def xz(self) -> I16Vector2: ...
    @property
    def xo(self) -> I16Vector2: ...
    @property
    def xl(self) -> I16Vector2: ...
    @property
    def yx(self) -> I16Vector2: ...
    @property
    def yy(self) -> I16Vector2: ...
    @property
    def yz(self) -> I16Vector2: ...
    @property
    def yo(self) -> I16Vector2: ...
    @property
    def yl(self) -> I16Vector2: ...
    @property
    def zx(self) -> I16Vector2: ...
    @property
    def zy(self) -> I16Vector2: ...
    @property
    def zz(self) -> I16Vector2: ...
    @property
    def zo(self) -> I16Vector2: ...
    @property
    def zl(self) -> I16Vector2: ...
    @property
    def ox(self) -> I16Vector2: ...
    @property
    def oy(self) -> I16Vector2: ...
    @property
    def oz(self) -> I16Vector2: ...
    @property
    def lx(self) -> I16Vector2: ...
    @property
    def ly(self) -> I16Vector2: ...
    @property
    def lz(self) -> I16Vector2: ...
    @property
    def xxx(self) -> I16Vector3: ...
    @property
    def xxy(self) -> I16Vector3: ...
    @property
    def xxz(self) -> I16Vector3: ...
    @property
    def xxo(self) -> I16Vector3: ...
    @property
    def xxl(self) -> I16Vector3: ...
    @property
    def xyx(self) -> I16Vector3: ...
    @property
    def xyy(self) -> I16Vector3: ...
    @property
    def xyz(self) -> I16Vector3: ...
    @property
    def xyo(self) -> I16Vector3: ...
    @property
    def xyl(self) -> I16Vector3: ...
    @property
    def xzx(self) -> I16Vector3: ...
    @property
    def xzy(self) -> I16Vector3: ...
    @property
    def xzz(self) -> I16Vector3: ...
    @property
    def xzo(self) -> I16Vector3: ...
    @property
    def xzl(self) -> I16Vector3: ...
    @property
    def xox(self) -> I16Vector3: ...
    @property
    def xoy(self) -> I16Vector3: ...
    @property
    def xoz(self) -> I16Vector3: ...
    @property
    def xoo(self) -> I16Vector3: ...
    @property
    def xol(self) -> I16Vector3: ...
    @property
    def xlx(self) -> I16Vector3: ...
    @property
    def xly(self) -> I16Vector3: ...
    @property
    def xlz(self) -> I16Vector3: ...
    @property
    def xlo(self) -> I16Vector3: ...
    @property
    def xll(self) -> I16Vector3: ...
    @property
    def yxx(self) -> I16Vector3: ...
    @property
    def yxy(self) -> I16Vector3: ...
    @property
    def yxz(self) -> I16Vector3: ...
    @property
    def yxo(self) -> I16Vector3: ...
    @property
    def yxl(self) -> I16Vector3: ...
    @property
    def yyx(self) -> I16Vector3: ...
    @property
    def yyy(self) -> I16Vector3: ...
    @property
    def yyz(self) -> I16Vector3: ...
    @property
    def yyo(self) -> I16Vector3: ...
    @property
    def yyl(self) -> I16Vector3: ...
    @property
    def yzx(self) -> I16Vector3: ...
    @property
    def yzy(self) -> I16Vector3: ...
    @property
    def yzz(self) -> I16Vector3: ...
    @property
    def yzo(self) -> I16Vector3: ...
    @property
    def yzl(self) -> I16Vector3: ...
    @property
    def yox(self) -> I16Vector3: ...
    @property
    def yoy(self) -> I16Vector3: ...
    @property
    def yoz(self) -> I16Vector3: ...
    @property
    def yoo(self) -> I16Vector3: ...
    @property
    def yol(self) -> I16Vector3: ...
    @property
    def ylx(self) -> I16Vector3: ...
    @property
    def yly(self) -> I16Vector3: ...
    @property
    def ylz(self) -> I16Vector3: ...
    @property
    def ylo(self) -> I16Vector3: ...
    @property
    def yll(self) -> I16Vector3: ...
    @property
    def zxx(self) -> I16Vector3: ...
    @property
    def zxy(self) -> I16Vector3: ...
    @property
    def zxz(self) -> I16Vector3: ...
    @property
    def zxo(self) -> I16Vector3: ...
    @property
    def zxl(self) -> I16Vector3: ...
    @property
    def zyx(self) -> I16Vector3: ...
    @property
    def zyy(self) -> I16Vector3: ...
    @property
    def zyz(self) -> I16Vector3: ...
    @property
    def zyo(self) -> I16Vector3: ...
    @property
    def zyl(self) -> I16Vector3: ...
    @property
    def zzx(self) -> I16Vector3: ...
    @property
    def zzy(self) -> I16Vector3: ...
    @property
    def zzz(self) -> I16Vector3: ...
    @property
    def zzo(self) -> I16Vector3: ...
    @property
    def zzl(self) -> I16Vector3: ...
    @property
    def zox(self) -> I16Vector3: ...
    @property
    def zoy(self) -> I16Vector3: ...
    @property
    def zoz(self) -> I16Vector3: ...
    @property
    def zoo(self) -> I16Vector3: ...
    @property
    def zol(self) -> I16Vector3: ...
    @property
    def zlx(self) -> I16Vector3: ...
    @property
    def zly(self) -> I16Vector3: ...
    @property
    def zlz(self) -> I16Vector3: ...
    @property
    def zlo(self) -> I16Vector3: ...
    @property
    def zll(self) -> I16Vector3: ...
    @property
    def oxx(self) -> I16Vector3: ...
    @property
    def oxy(self) -> I16Vector3: ...
    @property
    def oxz(self) -> I16Vector3: ...
    @property
    def oxo(self) -> I16Vector3: ...
    @property
    def oxl(self) -> I16Vector3: ...
    @property
    def oyx(self) -> I16Vector3: ...
    @property
    def oyy(self) -> I16Vector3: ...
    @property
    def oyz(self) -> I16Vector3: ...
    @property
    def oyo(self) -> I16Vector3: ...
    @property
    def oyl(self) -> I16Vector3: ...
    @property
    def ozx(self) -> I16Vector3: ...
    @property
    def ozy(self) -> I16Vector3: ...
    @property
    def ozz(self) -> I16Vector3: ...
    @property
    def ozo(self) -> I16Vector3: ...
    @property
    def ozl(self) -> I16Vector3: ...
    @property
    def oox(self) -> I16Vector3: ...
    @property
    def ooy(self) -> I16Vector3: ...
    @property
    def ooz(self) -> I16Vector3: ...
    @property
    def olx(self) -> I16Vector3: ...
    @property
    def oly(self) -> I16Vector3: ...
    @property
    def olz(self) -> I16Vector3: ...
    @property
    def lxx(self) -> I16Vector3: ...
    @property
    def lxy(self) -> I16Vector3: ...
    @property
    def lxz(self) -> I16Vector3: ...
    @property
    def lxo(self) -> I16Vector3: ...
    @property
    def lxl(self) -> I16Vector3: ...
    @property
    def lyx(self) -> I16Vector3: ...
    @property
    def lyy(self) -> I16Vector3: ...
    @property
    def lyz(self) -> I16Vector3: ...
    @property
    def lyo(self) -> I16Vector3: ...
    @property
    def lyl(self) -> I16Vector3: ...
    @property
    def lzx(self) -> I16Vector3: ...
    @property
    def lzy(self) -> I16Vector3: ...
    @property
    def lzz(self) -> I16Vector3: ...
    @property
    def lzo(self) -> I16Vector3: ...
    @property
    def lzl(self) -> I16Vector3: ...
    @property
    def lox(self) -> I16Vector3: ...
    @property
    def loy(self) -> I16Vector3: ...
    @property
    def loz(self) -> I16Vector3: ...
    @property
    def llx(self) -> I16Vector3: ...
    @property
    def lly(self) -> I16Vector3: ...
    @property
    def llz(self) -> I16Vector3: ...
    @property
    def xxxx(self) -> I16Vector4: ...
    @property
    def xxxy(self) -> I16Vector4: ...
    @property
    def xxxz(self) -> I16Vector4: ...
    @property
    def xxxo(self) -> I16Vector4: ...
    @property
    def xxxl(self) -> I16Vector4: ...
    @property
    def xxyx(self) -> I16Vector4: ...
    @property
    def xxyy(self) -> I16Vector4: ...
    @property
    def xxyz(self) -> I16Vector4: ...
    @property
    def xxyo(self) -> I16Vector4: ...
    @property
    def xxyl(self) -> I16Vector4: ...
    @property
    def xxzx(self) -> I16Vector4: ...
    @property
    def xxzy(self) -> I16Vector4: ...
    @property
    def xxzz(self) -> I16Vector4: ...
    @property
    def xxzo(self) -> I16Vector4: ...
    @property
    def xxzl(self) -> I16Vector4: ...
    @property
    def xxox(self) -> I16Vector4: ...
    @property
    def xxoy(self) -> I16Vector4: ...
    @property
    def xxoz(self) -> I16Vector4: ...
    @property
    def xxoo(self) -> I16Vector4: ...
    @property
    def xxol(self) -> I16Vector4: ...
    @property
    def xxlx(self) -> I16Vector4: ...
    @property
    def xxly(self) -> I16Vector4: ...
    @property
    def xxlz(self) -> I16Vector4: ...
    @property
    def xxlo(self) -> I16Vector4: ...
    @property
    def xxll(self) -> I16Vector4: ...
    @property
    def xyxx(self) -> I16Vector4: ...
    @property
    def xyxy(self) -> I16Vector4: ...
    @property
    def xyxz(self) -> I16Vector4: ...
    @property
    def xyxo(self) -> I16Vector4: ...
    @property
    def xyxl(self) -> I16Vector4: ...
    @property
    def xyyx(self) -> I16Vector4: ...
    @property
    def xyyy(self) -> I16Vector4: ...
    @property
    def xyyz(self) -> I16Vector4: ...
    @property
    def xyyo(self) -> I16Vector4: ...
    @property
    def xyyl(self) -> I16Vector4: ...
    @property
    def xyzx(self) -> I16Vector4: ...
    @property
    def xyzy(self) -> I16Vector4: ...
    @property
    def xyzz(self) -> I16Vector4: ...
    @property
    def xyzo(self) -> I16Vector4: ...
    @property
    def xyzl(self) -> I16Vector4: ...
    @property
    def xyox(self) -> I16Vector4: ...
    @property
    def xyoy(self) -> I16Vector4: ...
    @property
    def xyoz(self) -> I16Vector4: ...
    @property
    def xyoo(self) -> I16Vector4: ...
    @property
    def xyol(self) -> I16Vector4: ...
    @property
    def xylx(self) -> I16Vector4: ...
    @property
    def xyly(self) -> I16Vector4: ...
    @property
    def xylz(self) -> I16Vector4: ...
    @property
    def xylo(self) -> I16Vector4: ...
    @property
    def xyll(self) -> I16Vector4: ...
    @property
    def xzxx(self) -> I16Vector4: ...
    @property
    def xzxy(self) -> I16Vector4: ...
    @property
    def xzxz(self) -> I16Vector4: ...
    @property
    def xzxo(self) -> I16Vector4: ...
    @property
    def xzxl(self) -> I16Vector4: ...
    @property
    def xzyx(self) -> I16Vector4: ...
    @property
    def xzyy(self) -> I16Vector4: ...
    @property
    def xzyz(self) -> I16Vector4: ...
    @property
    def xzyo(self) -> I16Vector4: ...
    @property
    def xzyl(self) -> I16Vector4: ...
    @property
    def xzzx(self) -> I16Vector4: ...
    @property
    def xzzy(self) -> I16Vector4: ...
    @property
    def xzzz(self) -> I16Vector4: ...
    @property
    def xzzo(self) -> I16Vector4: ...
    @property
    def xzzl(self) -> I16Vector4: ...
    @property
    def xzox(self) -> I16Vector4: ...
    @property
    def xzoy(self) -> I16Vector4: ...
    @property
    def xzoz(self) -> I16Vector4: ...
    @property
    def xzoo(self) -> I16Vector4: ...
    @property
    def xzol(self) -> I16Vector4: ...
    @property
    def xzlx(self) -> I16Vector4: ...
    @property
    def xzly(self) -> I16Vector4: ...
    @property
    def xzlz(self) -> I16Vector4: ...
    @property
    def xzlo(self) -> I16Vector4: ...
    @property
    def xzll(self) -> I16Vector4: ...
    @property
    def xoxx(self) -> I16Vector4: ...
    @property
    def xoxy(self) -> I16Vector4: ...
    @property
    def xoxz(self) -> I16Vector4: ...
    @property
    def xoxo(self) -> I16Vector4: ...
    @property
    def xoxl(self) -> I16Vector4: ...
    @property
    def xoyx(self) -> I16Vector4: ...
    @property
    def xoyy(self) -> I16Vector4: ...
    @property
    def xoyz(self) -> I16Vector4: ...
    @property
    def xoyo(self) -> I16Vector4: ...
    @property
    def xoyl(self) -> I16Vector4: ...
    @property
    def xozx(self) -> I16Vector4: ...
    @property
    def xozy(self) -> I16Vector4: ...
    @property
    def xozz(self) -> I16Vector4: ...
    @property
    def xozo(self) -> I16Vector4: ...
    @property
    def xozl(self) -> I16Vector4: ...
    @property
    def xoox(self) -> I16Vector4: ...
    @property
    def xooy(self) -> I16Vector4: ...
    @property
    def xooz(self) -> I16Vector4: ...
    @property
    def xooo(self) -> I16Vector4: ...
    @property
    def xool(self) -> I16Vector4: ...
    @property
    def xolx(self) -> I16Vector4: ...
    @property
    def xoly(self) -> I16Vector4: ...
    @property
    def xolz(self) -> I16Vector4: ...
    @property
    def xolo(self) -> I16Vector4: ...
    @property
    def xoll(self) -> I16Vector4: ...
    @property
    def xlxx(self) -> I16Vector4: ...
    @property
    def xlxy(self) -> I16Vector4: ...
    @property
    def xlxz(self) -> I16Vector4: ...
    @property
    def xlxo(self) -> I16Vector4: ...
    @property
    def xlxl(self) -> I16Vector4: ...
    @property
    def xlyx(self) -> I16Vector4: ...
    @property
    def xlyy(self) -> I16Vector4: ...
    @property
    def xlyz(self) -> I16Vector4: ...
    @property
    def xlyo(self) -> I16Vector4: ...
    @property
    def xlyl(self) -> I16Vector4: ...
    @property
    def xlzx(self) -> I16Vector4: ...
    @property
    def xlzy(self) -> I16Vector4: ...
    @property
    def xlzz(self) -> I16Vector4: ...
    @property
    def xlzo(self) -> I16Vector4: ...
    @property
    def xlzl(self) -> I16Vector4: ...
    @property
    def xlox(self) -> I16Vector4: ...
    @property
    def xloy(self) -> I16Vector4: ...
    @property
    def xloz(self) -> I16Vector4: ...
    @property
    def xloo(self) -> I16Vector4: ...
    @property
    def xlol(self) -> I16Vector4: ...
    @property
    def xllx(self) -> I16Vector4: ...
    @property
    def xlly(self) -> I16Vector4: ...
    @property
    def xllz(self) -> I16Vector4: ...
    @property
    def xllo(self) -> I16Vector4: ...
    @property
    def xlll(self) -> I16Vector4: ...
    @property
    def yxxx(self) -> I16Vector4: ...
    @property
    def yxxy(self) -> I16Vector4: ...
    @property
    def yxxz(self) -> I16Vector4: ...
    @property
    def yxxo(self) -> I16Vector4: ...
    @property
    def yxxl(self) -> I16Vector4: ...
    @property
    def yxyx(self) -> I16Vector4: ...
    @property
    def yxyy(self) -> I16Vector4: ...
    @property
    def yxyz(self) -> I16Vector4: ...
    @property
    def yxyo(self) -> I16Vector4: ...
    @property
    def yxyl(self) -> I16Vector4: ...
    @property
    def yxzx(self) -> I16Vector4: ...
    @property
    def yxzy(self) -> I16Vector4: ...
    @property
    def yxzz(self) -> I16Vector4: ...
    @property
    def yxzo(self) -> I16Vector4: ...
    @property
    def yxzl(self) -> I16Vector4: ...
    @property
    def yxox(self) -> I16Vector4: ...
    @property
    def yxoy(self) -> I16Vector4: ...
    @property
    def yxoz(self) -> I16Vector4: ...
    @property
    def yxoo(self) -> I16Vector4: ...
    @property
    def yxol(self) -> I16Vector4: ...
    @property
    def yxlx(self) -> I16Vector4: ...
    @property
    def yxly(self) -> I16Vector4: ...
    @property
    def yxlz(self) -> I16Vector4: ...
    @property
    def yxlo(self) -> I16Vector4: ...
    @property
    def yxll(self) -> I16Vector4: ...
    @property
    def yyxx(self) -> I16Vector4: ...
    @property
    def yyxy(self) -> I16Vector4: ...
    @property
    def yyxz(self) -> I16Vector4: ...
    @property
    def yyxo(self) -> I16Vector4: ...
    @property
    def yyxl(self) -> I16Vector4: ...
    @property
    def yyyx(self) -> I16Vector4: ...
    @property
    def yyyy(self) -> I16Vector4: ...
    @property
    def yyyz(self) -> I16Vector4: ...
    @property
    def yyyo(self) -> I16Vector4: ...
    @property
    def yyyl(self) -> I16Vector4: ...
    @property
    def yyzx(self) -> I16Vector4: ...
    @property
    def yyzy(self) -> I16Vector4: ...
    @property
    def yyzz(self) -> I16Vector4: ...
    @property
    def yyzo(self) -> I16Vector4: ...
    @property
    def yyzl(self) -> I16Vector4: ...
    @property
    def yyox(self) -> I16Vector4: ...
    @property
    def yyoy(self) -> I16Vector4: ...
    @property
    def yyoz(self) -> I16Vector4: ...
    @property
    def yyoo(self) -> I16Vector4: ...
    @property
    def yyol(self) -> I16Vector4: ...
    @property
    def yylx(self) -> I16Vector4: ...
    @property
    def yyly(self) -> I16Vector4: ...
    @property
    def yylz(self) -> I16Vector4: ...
    @property
    def yylo(self) -> I16Vector4: ...
    @property
    def yyll(self) -> I16Vector4: ...
    @property
    def yzxx(self) -> I16Vector4: ...
    @property
    def yzxy(self) -> I16Vector4: ...
    @property
    def yzxz(self) -> I16Vector4: ...
    @property
    def yzxo(self) -> I16Vector4: ...
    @property
    def yzxl(self) -> I16Vector4: ...
    @property
    def yzyx(self) -> I16Vector4: ...
    @property
    def yzyy(self) -> I16Vector4: ...
    @property
    def yzyz(self) -> I16Vector4: ...
    @property
    def yzyo(self) -> I16Vector4: ...
    @property
    def yzyl(self) -> I16Vector4: ...
    @property
    def yzzx(self) -> I16Vector4: ...
    @property
    def yzzy(self) -> I16Vector4: ...
    @property
    def yzzz(self) -> I16Vector4: ...
    @property
    def yzzo(self) -> I16Vector4: ...
    @property
    def yzzl(self) -> I16Vector4: ...
    @property
    def yzox(self) -> I16Vector4: ...
    @property
    def yzoy(self) -> I16Vector4: ...
    @property
    def yzoz(self) -> I16Vector4: ...
    @property
    def yzoo(self) -> I16Vector4: ...
    @property
    def yzol(self) -> I16Vector4: ...
    @property
    def yzlx(self) -> I16Vector4: ...
    @property
    def yzly(self) -> I16Vector4: ...
    @property
    def yzlz(self) -> I16Vector4: ...
    @property
    def yzlo(self) -> I16Vector4: ...
    @property
    def yzll(self) -> I16Vector4: ...
    @property
    def yoxx(self) -> I16Vector4: ...
    @property
    def yoxy(self) -> I16Vector4: ...
    @property
    def yoxz(self) -> I16Vector4: ...
    @property
    def yoxo(self) -> I16Vector4: ...
    @property
    def yoxl(self) -> I16Vector4: ...
    @property
    def yoyx(self) -> I16Vector4: ...
    @property
    def yoyy(self) -> I16Vector4: ...
    @property
    def yoyz(self) -> I16Vector4: ...
    @property
    def yoyo(self) -> I16Vector4: ...
    @property
    def yoyl(self) -> I16Vector4: ...
    @property
    def yozx(self) -> I16Vector4: ...
    @property
    def yozy(self) -> I16Vector4: ...
    @property
    def yozz(self) -> I16Vector4: ...
    @property
    def yozo(self) -> I16Vector4: ...
    @property
    def yozl(self) -> I16Vector4: ...
    @property
    def yoox(self) -> I16Vector4: ...
    @property
    def yooy(self) -> I16Vector4: ...
    @property
    def yooz(self) -> I16Vector4: ...
    @property
    def yooo(self) -> I16Vector4: ...
    @property
    def yool(self) -> I16Vector4: ...
    @property
    def yolx(self) -> I16Vector4: ...
    @property
    def yoly(self) -> I16Vector4: ...
    @property
    def yolz(self) -> I16Vector4: ...
    @property
    def yolo(self) -> I16Vector4: ...
    @property
    def yoll(self) -> I16Vector4: ...
    @property
    def ylxx(self) -> I16Vector4: ...
    @property
    def ylxy(self) -> I16Vector4: ...
    @property
    def ylxz(self) -> I16Vector4: ...
    @property
    def ylxo(self) -> I16Vector4: ...
    @property
    def ylxl(self) -> I16Vector4: ...
    @property
    def ylyx(self) -> I16Vector4: ...
    @property
    def ylyy(self) -> I16Vector4: ...
    @property
    def ylyz(self) -> I16Vector4: ...
    @property
    def ylyo(self) -> I16Vector4: ...
    @property
    def ylyl(self) -> I16Vector4: ...
    @property
    def ylzx(self) -> I16Vector4: ...
    @property
    def ylzy(self) -> I16Vector4: ...
    @property
    def ylzz(self) -> I16Vector4: ...
    @property
    def ylzo(self) -> I16Vector4: ...
    @property
    def ylzl(self) -> I16Vector4: ...
    @property
    def ylox(self) -> I16Vector4: ...
    @property
    def yloy(self) -> I16Vector4: ...
    @property
    def yloz(self) -> I16Vector4: ...
    @property
    def yloo(self) -> I16Vector4: ...
    @property
    def ylol(self) -> I16Vector4: ...
    @property
    def yllx(self) -> I16Vector4: ...
    @property
    def ylly(self) -> I16Vector4: ...
    @property
    def yllz(self) -> I16Vector4: ...
    @property
    def yllo(self) -> I16Vector4: ...
    @property
    def ylll(self) -> I16Vector4: ...
    @property
    def zxxx(self) -> I16Vector4: ...
    @property
    def zxxy(self) -> I16Vector4: ...
    @property
    def zxxz(self) -> I16Vector4: ...
    @property
    def zxxo(self) -> I16Vector4: ...
    @property
    def zxxl(self) -> I16Vector4: ...
    @property
    def zxyx(self) -> I16Vector4: ...
    @property
    def zxyy(self) -> I16Vector4: ...
    @property
    def zxyz(self) -> I16Vector4: ...
    @property
    def zxyo(self) -> I16Vector4: ...
    @property
    def zxyl(self) -> I16Vector4: ...
    @property
    def zxzx(self) -> I16Vector4: ...
    @property
    def zxzy(self) -> I16Vector4: ...
    @property
    def zxzz(self) -> I16Vector4: ...
    @property
    def zxzo(self) -> I16Vector4: ...
    @property
    def zxzl(self) -> I16Vector4: ...
    @property
    def zxox(self) -> I16Vector4: ...
    @property
    def zxoy(self) -> I16Vector4: ...
    @property
    def zxoz(self) -> I16Vector4: ...
    @property
    def zxoo(self) -> I16Vector4: ...
    @property
    def zxol(self) -> I16Vector4: ...
    @property
    def zxlx(self) -> I16Vector4: ...
    @property
    def zxly(self) -> I16Vector4: ...
    @property
    def zxlz(self) -> I16Vector4: ...
    @property
    def zxlo(self) -> I16Vector4: ...
    @property
    def zxll(self) -> I16Vector4: ...
    @property
    def zyxx(self) -> I16Vector4: ...
    @property
    def zyxy(self) -> I16Vector4: ...
    @property
    def zyxz(self) -> I16Vector4: ...
    @property
    def zyxo(self) -> I16Vector4: ...
    @property
    def zyxl(self) -> I16Vector4: ...
    @property
    def zyyx(self) -> I16Vector4: ...
    @property
    def zyyy(self) -> I16Vector4: ...
    @property
    def zyyz(self) -> I16Vector4: ...
    @property
    def zyyo(self) -> I16Vector4: ...
    @property
    def zyyl(self) -> I16Vector4: ...
    @property
    def zyzx(self) -> I16Vector4: ...
    @property
    def zyzy(self) -> I16Vector4: ...
    @property
    def zyzz(self) -> I16Vector4: ...
    @property
    def zyzo(self) -> I16Vector4: ...
    @property
    def zyzl(self) -> I16Vector4: ...
    @property
    def zyox(self) -> I16Vector4: ...
    @property
    def zyoy(self) -> I16Vector4: ...
    @property
    def zyoz(self) -> I16Vector4: ...
    @property
    def zyoo(self) -> I16Vector4: ...
    @property
    def zyol(self) -> I16Vector4: ...
    @property
    def zylx(self) -> I16Vector4: ...
    @property
    def zyly(self) -> I16Vector4: ...
    @property
    def zylz(self) -> I16Vector4: ...
    @property
    def zylo(self) -> I16Vector4: ...
    @property
    def zyll(self) -> I16Vector4: ...
    @property
    def zzxx(self) -> I16Vector4: ...
    @property
    def zzxy(self) -> I16Vector4: ...
    @property
    def zzxz(self) -> I16Vector4: ...
    @property
    def zzxo(self) -> I16Vector4: ...
    @property
    def zzxl(self) -> I16Vector4: ...
    @property
    def zzyx(self) -> I16Vector4: ...
    @property
    def zzyy(self) -> I16Vector4: ...
    @property
    def zzyz(self) -> I16Vector4: ...
    @property
    def zzyo(self) -> I16Vector4: ...
    @property
    def zzyl(self) -> I16Vector4: ...
    @property
    def zzzx(self) -> I16Vector4: ...
    @property
    def zzzy(self) -> I16Vector4: ...
    @property
    def zzzz(self) -> I16Vector4: ...
    @property
    def zzzo(self) -> I16Vector4: ...
    @property
    def zzzl(self) -> I16Vector4: ...
    @property
    def zzox(self) -> I16Vector4: ...
    @property
    def zzoy(self) -> I16Vector4: ...
    @property
    def zzoz(self) -> I16Vector4: ...
    @property
    def zzoo(self) -> I16Vector4: ...
    @property
    def zzol(self) -> I16Vector4: ...
    @property
    def zzlx(self) -> I16Vector4: ...
    @property
    def zzly(self) -> I16Vector4: ...
    @property
    def zzlz(self) -> I16Vector4: ...
    @property
    def zzlo(self) -> I16Vector4: ...
    @property
    def zzll(self) -> I16Vector4: ...
    @property
    def zoxx(self) -> I16Vector4: ...
    @property
    def zoxy(self) -> I16Vector4: ...
    @property
    def zoxz(self) -> I16Vector4: ...
    @property
    def zoxo(self) -> I16Vector4: ...
    @property
    def zoxl(self) -> I16Vector4: ...
    @property
    def zoyx(self) -> I16Vector4: ...
    @property
    def zoyy(self) -> I16Vector4: ...
    @property
    def zoyz(self) -> I16Vector4: ...
    @property
    def zoyo(self) -> I16Vector4: ...
    @property
    def zoyl(self) -> I16Vector4: ...
    @property
    def zozx(self) -> I16Vector4: ...
    @property
    def zozy(self) -> I16Vector4: ...
    @property
    def zozz(self) -> I16Vector4: ...
    @property
    def zozo(self) -> I16Vector4: ...
    @property
    def zozl(self) -> I16Vector4: ...
    @property
    def zoox(self) -> I16Vector4: ...
    @property
    def zooy(self) -> I16Vector4: ...
    @property
    def zooz(self) -> I16Vector4: ...
    @property
    def zooo(self) -> I16Vector4: ...
    @property
    def zool(self) -> I16Vector4: ...
    @property
    def zolx(self) -> I16Vector4: ...
    @property
    def zoly(self) -> I16Vector4: ...
    @property
    def zolz(self) -> I16Vector4: ...
    @property
    def zolo(self) -> I16Vector4: ...
    @property
    def zoll(self) -> I16Vector4: ...
    @property
    def zlxx(self) -> I16Vector4: ...
    @property
    def zlxy(self) -> I16Vector4: ...
    @property
    def zlxz(self) -> I16Vector4: ...
    @property
    def zlxo(self) -> I16Vector4: ...
    @property
    def zlxl(self) -> I16Vector4: ...
    @property
    def zlyx(self) -> I16Vector4: ...
    @property
    def zlyy(self) -> I16Vector4: ...
    @property
    def zlyz(self) -> I16Vector4: ...
    @property
    def zlyo(self) -> I16Vector4: ...
    @property
    def zlyl(self) -> I16Vector4: ...
    @property
    def zlzx(self) -> I16Vector4: ...
    @property
    def zlzy(self) -> I16Vector4: ...
    @property
    def zlzz(self) -> I16Vector4: ...
    @property
    def zlzo(self) -> I16Vector4: ...
    @property
    def zlzl(self) -> I16Vector4: ...
    @property
    def zlox(self) -> I16Vector4: ...
    @property
    def zloy(self) -> I16Vector4: ...
    @property
    def zloz(self) -> I16Vector4: ...
    @property
    def zloo(self) -> I16Vector4: ...
    @property
    def zlol(self) -> I16Vector4: ...
    @property
    def zllx(self) -> I16Vector4: ...
    @property
    def zlly(self) -> I16Vector4: ...
    @property
    def zllz(self) -> I16Vector4: ...
    @property
    def zllo(self) -> I16Vector4: ...
    @property
    def zlll(self) -> I16Vector4: ...
    @property
    def oxxx(self) -> I16Vector4: ...
    @property
    def oxxy(self) -> I16Vector4: ...
    @property
    def oxxz(self) -> I16Vector4: ...
    @property
    def oxxo(self) -> I16Vector4: ...
    @property
    def oxxl(self) -> I16Vector4: ...
    @property
    def oxyx(self) -> I16Vector4: ...
    @property
    def oxyy(self) -> I16Vector4: ...
    @property
    def oxyz(self) -> I16Vector4: ...
    @property
    def oxyo(self) -> I16Vector4: ...
    @property
    def oxyl(self) -> I16Vector4: ...
    @property
    def oxzx(self) -> I16Vector4: ...
    @property
    def oxzy(self) -> I16Vector4: ...
    @property
    def oxzz(self) -> I16Vector4: ...
    @property
    def oxzo(self) -> I16Vector4: ...
    @property
    def oxzl(self) -> I16Vector4: ...
    @property
    def oxox(self) -> I16Vector4: ...
    @property
    def oxoy(self) -> I16Vector4: ...
    @property
    def oxoz(self) -> I16Vector4: ...
    @property
    def oxoo(self) -> I16Vector4: ...
    @property
    def oxol(self) -> I16Vector4: ...
    @property
    def oxlx(self) -> I16Vector4: ...
    @property
    def oxly(self) -> I16Vector4: ...
    @property
    def oxlz(self) -> I16Vector4: ...
    @property
    def oxlo(self) -> I16Vector4: ...
    @property
    def oxll(self) -> I16Vector4: ...
    @property
    def oyxx(self) -> I16Vector4: ...
    @property
    def oyxy(self) -> I16Vector4: ...
    @property
    def oyxz(self) -> I16Vector4: ...
    @property
    def oyxo(self) -> I16Vector4: ...
    @property
    def oyxl(self) -> I16Vector4: ...
    @property
    def oyyx(self) -> I16Vector4: ...
    @property
    def oyyy(self) -> I16Vector4: ...
    @property
    def oyyz(self) -> I16Vector4: ...
    @property
    def oyyo(self) -> I16Vector4: ...
    @property
    def oyyl(self) -> I16Vector4: ...
    @property
    def oyzx(self) -> I16Vector4: ...
    @property
    def oyzy(self) -> I16Vector4: ...
    @property
    def oyzz(self) -> I16Vector4: ...
    @property
    def oyzo(self) -> I16Vector4: ...
    @property
    def oyzl(self) -> I16Vector4: ...
    @property
    def oyox(self) -> I16Vector4: ...
    @property
    def oyoy(self) -> I16Vector4: ...
    @property
    def oyoz(self) -> I16Vector4: ...
    @property
    def oyoo(self) -> I16Vector4: ...
    @property
    def oyol(self) -> I16Vector4: ...
    @property
    def oylx(self) -> I16Vector4: ...
    @property
    def oyly(self) -> I16Vector4: ...
    @property
    def oylz(self) -> I16Vector4: ...
    @property
    def oylo(self) -> I16Vector4: ...
    @property
    def oyll(self) -> I16Vector4: ...
    @property
    def ozxx(self) -> I16Vector4: ...
    @property
    def ozxy(self) -> I16Vector4: ...
    @property
    def ozxz(self) -> I16Vector4: ...
    @property
    def ozxo(self) -> I16Vector4: ...
    @property
    def ozxl(self) -> I16Vector4: ...
    @property
    def ozyx(self) -> I16Vector4: ...
    @property
    def ozyy(self) -> I16Vector4: ...
    @property
    def ozyz(self) -> I16Vector4: ...
    @property
    def ozyo(self) -> I16Vector4: ...
    @property
    def ozyl(self) -> I16Vector4: ...
    @property
    def ozzx(self) -> I16Vector4: ...
    @property
    def ozzy(self) -> I16Vector4: ...
    @property
    def ozzz(self) -> I16Vector4: ...
    @property
    def ozzo(self) -> I16Vector4: ...
    @property
    def ozzl(self) -> I16Vector4: ...
    @property
    def ozox(self) -> I16Vector4: ...
    @property
    def ozoy(self) -> I16Vector4: ...
    @property
    def ozoz(self) -> I16Vector4: ...
    @property
    def ozoo(self) -> I16Vector4: ...
    @property
    def ozol(self) -> I16Vector4: ...
    @property
    def ozlx(self) -> I16Vector4: ...
    @property
    def ozly(self) -> I16Vector4: ...
    @property
    def ozlz(self) -> I16Vector4: ...
    @property
    def ozlo(self) -> I16Vector4: ...
    @property
    def ozll(self) -> I16Vector4: ...
    @property
    def ooxx(self) -> I16Vector4: ...
    @property
    def ooxy(self) -> I16Vector4: ...
    @property
    def ooxz(self) -> I16Vector4: ...
    @property
    def ooxo(self) -> I16Vector4: ...
    @property
    def ooxl(self) -> I16Vector4: ...
    @property
    def ooyx(self) -> I16Vector4: ...
    @property
    def ooyy(self) -> I16Vector4: ...
    @property
    def ooyz(self) -> I16Vector4: ...
    @property
    def ooyo(self) -> I16Vector4: ...
    @property
    def ooyl(self) -> I16Vector4: ...
    @property
    def oozx(self) -> I16Vector4: ...
    @property
    def oozy(self) -> I16Vector4: ...
    @property
    def oozz(self) -> I16Vector4: ...
    @property
    def oozo(self) -> I16Vector4: ...
    @property
    def oozl(self) -> I16Vector4: ...
    @property
    def ooox(self) -> I16Vector4: ...
    @property
    def oooy(self) -> I16Vector4: ...
    @property
    def oooz(self) -> I16Vector4: ...
    @property
    def oolx(self) -> I16Vector4: ...
    @property
    def ooly(self) -> I16Vector4: ...
    @property
    def oolz(self) -> I16Vector4: ...
    @property
    def olxx(self) -> I16Vector4: ...
    @property
    def olxy(self) -> I16Vector4: ...
    @property
    def olxz(self) -> I16Vector4: ...
    @property
    def olxo(self) -> I16Vector4: ...
    @property
    def olxl(self) -> I16Vector4: ...
    @property
    def olyx(self) -> I16Vector4: ...
    @property
    def olyy(self) -> I16Vector4: ...
    @property
    def olyz(self) -> I16Vector4: ...
    @property
    def olyo(self) -> I16Vector4: ...
    @property
    def olyl(self) -> I16Vector4: ...
    @property
    def olzx(self) -> I16Vector4: ...
    @property
    def olzy(self) -> I16Vector4: ...
    @property
    def olzz(self) -> I16Vector4: ...
    @property
    def olzo(self) -> I16Vector4: ...
    @property
    def olzl(self) -> I16Vector4: ...
    @property
    def olox(self) -> I16Vector4: ...
    @property
    def oloy(self) -> I16Vector4: ...
    @property
    def oloz(self) -> I16Vector4: ...
    @property
    def ollx(self) -> I16Vector4: ...
    @property
    def olly(self) -> I16Vector4: ...
    @property
    def ollz(self) -> I16Vector4: ...
    @property
    def lxxx(self) -> I16Vector4: ...
    @property
    def lxxy(self) -> I16Vector4: ...
    @property
    def lxxz(self) -> I16Vector4: ...
    @property
    def lxxo(self) -> I16Vector4: ...
    @property
    def lxxl(self) -> I16Vector4: ...
    @property
    def lxyx(self) -> I16Vector4: ...
    @property
    def lxyy(self) -> I16Vector4: ...
    @property
    def lxyz(self) -> I16Vector4: ...
    @property
    def lxyo(self) -> I16Vector4: ...
    @property
    def lxyl(self) -> I16Vector4: ...
    @property
    def lxzx(self) -> I16Vector4: ...
    @property
    def lxzy(self) -> I16Vector4: ...
    @property
    def lxzz(self) -> I16Vector4: ...
    @property
    def lxzo(self) -> I16Vector4: ...
    @property
    def lxzl(self) -> I16Vector4: ...
    @property
    def lxox(self) -> I16Vector4: ...
    @property
    def lxoy(self) -> I16Vector4: ...
    @property
    def lxoz(self) -> I16Vector4: ...
    @property
    def lxoo(self) -> I16Vector4: ...
    @property
    def lxol(self) -> I16Vector4: ...
    @property
    def lxlx(self) -> I16Vector4: ...
    @property
    def lxly(self) -> I16Vector4: ...
    @property
    def lxlz(self) -> I16Vector4: ...
    @property
    def lxlo(self) -> I16Vector4: ...
    @property
    def lxll(self) -> I16Vector4: ...
    @property
    def lyxx(self) -> I16Vector4: ...
    @property
    def lyxy(self) -> I16Vector4: ...
    @property
    def lyxz(self) -> I16Vector4: ...
    @property
    def lyxo(self) -> I16Vector4: ...
    @property
    def lyxl(self) -> I16Vector4: ...
    @property
    def lyyx(self) -> I16Vector4: ...
    @property
    def lyyy(self) -> I16Vector4: ...
    @property
    def lyyz(self) -> I16Vector4: ...
    @property
    def lyyo(self) -> I16Vector4: ...
    @property
    def lyyl(self) -> I16Vector4: ...
    @property
    def lyzx(self) -> I16Vector4: ...
    @property
    def lyzy(self) -> I16Vector4: ...
    @property
    def lyzz(self) -> I16Vector4: ...
    @property
    def lyzo(self) -> I16Vector4: ...
    @property
    def lyzl(self) -> I16Vector4: ...
    @property
    def lyox(self) -> I16Vector4: ...
    @property
    def lyoy(self) -> I16Vector4: ...
    @property
    def lyoz(self) -> I16Vector4: ...
    @property
    def lyoo(self) -> I16Vector4: ...
    @property
    def lyol(self) -> I16Vector4: ...
    @property
    def lylx(self) -> I16Vector4: ...
    @property
    def lyly(self) -> I16Vector4: ...
    @property
    def lylz(self) -> I16Vector4: ...
    @property
    def lylo(self) -> I16Vector4: ...
    @property
    def lyll(self) -> I16Vector4: ...
    @property
    def lzxx(self) -> I16Vector4: ...
    @property
    def lzxy(self) -> I16Vector4: ...
    @property
    def lzxz(self) -> I16Vector4: ...
    @property
    def lzxo(self) -> I16Vector4: ...
    @property
    def lzxl(self) -> I16Vector4: ...
    @property
    def lzyx(self) -> I16Vector4: ...
    @property
    def lzyy(self) -> I16Vector4: ...
    @property
    def lzyz(self) -> I16Vector4: ...
    @property
    def lzyo(self) -> I16Vector4: ...
    @property
    def lzyl(self) -> I16Vector4: ...
    @property
    def lzzx(self) -> I16Vector4: ...
    @property
    def lzzy(self) -> I16Vector4: ...
    @property
    def lzzz(self) -> I16Vector4: ...
    @property
    def lzzo(self) -> I16Vector4: ...
    @property
    def lzzl(self) -> I16Vector4: ...
    @property
    def lzox(self) -> I16Vector4: ...
    @property
    def lzoy(self) -> I16Vector4: ...
    @property
    def lzoz(self) -> I16Vector4: ...
    @property
    def lzoo(self) -> I16Vector4: ...
    @property
    def lzol(self) -> I16Vector4: ...
    @property
    def lzlx(self) -> I16Vector4: ...
    @property
    def lzly(self) -> I16Vector4: ...
    @property
    def lzlz(self) -> I16Vector4: ...
    @property
    def lzlo(self) -> I16Vector4: ...
    @property
    def lzll(self) -> I16Vector4: ...
    @property
    def loxx(self) -> I16Vector4: ...
    @property
    def loxy(self) -> I16Vector4: ...
    @property
    def loxz(self) -> I16Vector4: ...
    @property
    def loxo(self) -> I16Vector4: ...
    @property
    def loxl(self) -> I16Vector4: ...
    @property
    def loyx(self) -> I16Vector4: ...
    @property
    def loyy(self) -> I16Vector4: ...
    @property
    def loyz(self) -> I16Vector4: ...
    @property
    def loyo(self) -> I16Vector4: ...
    @property
    def loyl(self) -> I16Vector4: ...
    @property
    def lozx(self) -> I16Vector4: ...
    @property
    def lozy(self) -> I16Vector4: ...
    @property
    def lozz(self) -> I16Vector4: ...
    @property
    def lozo(self) -> I16Vector4: ...
    @property
    def lozl(self) -> I16Vector4: ...
    @property
    def loox(self) -> I16Vector4: ...
    @property
    def looy(self) -> I16Vector4: ...
    @property
    def looz(self) -> I16Vector4: ...
    @property
    def lolx(self) -> I16Vector4: ...
    @property
    def loly(self) -> I16Vector4: ...
    @property
    def lolz(self) -> I16Vector4: ...
    @property
    def llxx(self) -> I16Vector4: ...
    @property
    def llxy(self) -> I16Vector4: ...
    @property
    def llxz(self) -> I16Vector4: ...
    @property
    def llxo(self) -> I16Vector4: ...
    @property
    def llxl(self) -> I16Vector4: ...
    @property
    def llyx(self) -> I16Vector4: ...
    @property
    def llyy(self) -> I16Vector4: ...
    @property
    def llyz(self) -> I16Vector4: ...
    @property
    def llyo(self) -> I16Vector4: ...
    @property
    def llyl(self) -> I16Vector4: ...
    @property
    def llzx(self) -> I16Vector4: ...
    @property
    def llzy(self) -> I16Vector4: ...
    @property
    def llzz(self) -> I16Vector4: ...
    @property
    def llzo(self) -> I16Vector4: ...
    @property
    def llzl(self) -> I16Vector4: ...
    @property
    def llox(self) -> I16Vector4: ...
    @property
    def lloy(self) -> I16Vector4: ...
    @property
    def lloz(self) -> I16Vector4: ...
    @property
    def lllx(self) -> I16Vector4: ...
    @property
    def llly(self) -> I16Vector4: ...
    @property
    def lllz(self) -> I16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> I16Vector2: ...
    @property
    def rg(self) -> I16Vector2: ...
    @property
    def rb(self) -> I16Vector2: ...
    @property
    def ro(self) -> I16Vector2: ...
    @property
    def rl(self) -> I16Vector2: ...
    @property
    def gr(self) -> I16Vector2: ...
    @property
    def gg(self) -> I16Vector2: ...
    @property
    def gb(self) -> I16Vector2: ...
    @property
    def go(self) -> I16Vector2: ...
    @property
    def gl(self) -> I16Vector2: ...
    @property
    def br(self) -> I16Vector2: ...
    @property
    def bg(self) -> I16Vector2: ...
    @property
    def bb(self) -> I16Vector2: ...
    @property
    def bo(self) -> I16Vector2: ...
    @property
    def bl(self) -> I16Vector2: ...
    @property
    def og(self) -> I16Vector2: ...
    @property
    def ob(self) -> I16Vector2: ...
    @property
    def lr(self) -> I16Vector2: ...
    @property
    def lg(self) -> I16Vector2: ...
    @property
    def lb(self) -> I16Vector2: ...
    @property
    def rrr(self) -> I16Vector3: ...
    @property
    def rrg(self) -> I16Vector3: ...
    @property
    def rrb(self) -> I16Vector3: ...
    @property
    def rro(self) -> I16Vector3: ...
    @property
    def rrl(self) -> I16Vector3: ...
    @property
    def rgr(self) -> I16Vector3: ...
    @property
    def rgg(self) -> I16Vector3: ...
    @property
    def rgb(self) -> I16Vector3: ...
    @property
    def rgo(self) -> I16Vector3: ...
    @property
    def rgl(self) -> I16Vector3: ...
    @property
    def rbr(self) -> I16Vector3: ...
    @property
    def rbg(self) -> I16Vector3: ...
    @property
    def rbb(self) -> I16Vector3: ...
    @property
    def rbo(self) -> I16Vector3: ...
    @property
    def rbl(self) -> I16Vector3: ...
    @property
    def ror(self) -> I16Vector3: ...
    @property
    def rog(self) -> I16Vector3: ...
    @property
    def rob(self) -> I16Vector3: ...
    @property
    def roo(self) -> I16Vector3: ...
    @property
    def rol(self) -> I16Vector3: ...
    @property
    def rlr(self) -> I16Vector3: ...
    @property
    def rlg(self) -> I16Vector3: ...
    @property
    def rlb(self) -> I16Vector3: ...
    @property
    def rlo(self) -> I16Vector3: ...
    @property
    def rll(self) -> I16Vector3: ...
    @property
    def grr(self) -> I16Vector3: ...
    @property
    def grg(self) -> I16Vector3: ...
    @property
    def grb(self) -> I16Vector3: ...
    @property
    def gro(self) -> I16Vector3: ...
    @property
    def grl(self) -> I16Vector3: ...
    @property
    def ggr(self) -> I16Vector3: ...
    @property
    def ggg(self) -> I16Vector3: ...
    @property
    def ggb(self) -> I16Vector3: ...
    @property
    def ggo(self) -> I16Vector3: ...
    @property
    def ggl(self) -> I16Vector3: ...
    @property
    def gbr(self) -> I16Vector3: ...
    @property
    def gbg(self) -> I16Vector3: ...
    @property
    def gbb(self) -> I16Vector3: ...
    @property
    def gbo(self) -> I16Vector3: ...
    @property
    def gbl(self) -> I16Vector3: ...
    @property
    def gor(self) -> I16Vector3: ...
    @property
    def gog(self) -> I16Vector3: ...
    @property
    def gob(self) -> I16Vector3: ...
    @property
    def goo(self) -> I16Vector3: ...
    @property
    def gol(self) -> I16Vector3: ...
    @property
    def glr(self) -> I16Vector3: ...
    @property
    def glg(self) -> I16Vector3: ...
    @property
    def glb(self) -> I16Vector3: ...
    @property
    def glo(self) -> I16Vector3: ...
    @property
    def gll(self) -> I16Vector3: ...
    @property
    def brr(self) -> I16Vector3: ...
    @property
    def brg(self) -> I16Vector3: ...
    @property
    def brb(self) -> I16Vector3: ...
    @property
    def bro(self) -> I16Vector3: ...
    @property
    def brl(self) -> I16Vector3: ...
    @property
    def bgr(self) -> I16Vector3: ...
    @property
    def bgg(self) -> I16Vector3: ...
    @property
    def bgb(self) -> I16Vector3: ...
    @property
    def bgo(self) -> I16Vector3: ...
    @property
    def bgl(self) -> I16Vector3: ...
    @property
    def bbr(self) -> I16Vector3: ...
    @property
    def bbg(self) -> I16Vector3: ...
    @property
    def bbb(self) -> I16Vector3: ...
    @property
    def bbo(self) -> I16Vector3: ...
    @property
    def bbl(self) -> I16Vector3: ...
    @property
    def bor(self) -> I16Vector3: ...
    @property
    def bog(self) -> I16Vector3: ...
    @property
    def bob(self) -> I16Vector3: ...
    @property
    def boo(self) -> I16Vector3: ...
    @property
    def bol(self) -> I16Vector3: ...
    @property
    def blr(self) -> I16Vector3: ...
    @property
    def blg(self) -> I16Vector3: ...
    @property
    def blb(self) -> I16Vector3: ...
    @property
    def blo(self) -> I16Vector3: ...
    @property
    def bll(self) -> I16Vector3: ...
    @property
    def orr(self) -> I16Vector3: ...
    @property
    def org(self) -> I16Vector3: ...
    @property
    def orb(self) -> I16Vector3: ...
    @property
    def oro(self) -> I16Vector3: ...
    @property
    def orl(self) -> I16Vector3: ...
    @property
    def ogr(self) -> I16Vector3: ...
    @property
    def ogg(self) -> I16Vector3: ...
    @property
    def ogb(self) -> I16Vector3: ...
    @property
    def ogo(self) -> I16Vector3: ...
    @property
    def ogl(self) -> I16Vector3: ...
    @property
    def obr(self) -> I16Vector3: ...
    @property
    def obg(self) -> I16Vector3: ...
    @property
    def obb(self) -> I16Vector3: ...
    @property
    def obo(self) -> I16Vector3: ...
    @property
    def obl(self) -> I16Vector3: ...
    @property
    def oor(self) -> I16Vector3: ...
    @property
    def oog(self) -> I16Vector3: ...
    @property
    def oob(self) -> I16Vector3: ...
    @property
    def olr(self) -> I16Vector3: ...
    @property
    def olg(self) -> I16Vector3: ...
    @property
    def olb(self) -> I16Vector3: ...
    @property
    def lrr(self) -> I16Vector3: ...
    @property
    def lrg(self) -> I16Vector3: ...
    @property
    def lrb(self) -> I16Vector3: ...
    @property
    def lro(self) -> I16Vector3: ...
    @property
    def lrl(self) -> I16Vector3: ...
    @property
    def lgr(self) -> I16Vector3: ...
    @property
    def lgg(self) -> I16Vector3: ...
    @property
    def lgb(self) -> I16Vector3: ...
    @property
    def lgo(self) -> I16Vector3: ...
    @property
    def lgl(self) -> I16Vector3: ...
    @property
    def lbr(self) -> I16Vector3: ...
    @property
    def lbg(self) -> I16Vector3: ...
    @property
    def lbb(self) -> I16Vector3: ...
    @property
    def lbo(self) -> I16Vector3: ...
    @property
    def lbl(self) -> I16Vector3: ...
    @property
    def lor(self) -> I16Vector3: ...
    @property
    def log(self) -> I16Vector3: ...
    @property
    def lob(self) -> I16Vector3: ...
    @property
    def llr(self) -> I16Vector3: ...
    @property
    def llg(self) -> I16Vector3: ...
    @property
    def llb(self) -> I16Vector3: ...
    @property
    def rrrr(self) -> I16Vector4: ...
    @property
    def rrrg(self) -> I16Vector4: ...
    @property
    def rrrb(self) -> I16Vector4: ...
    @property
    def rrro(self) -> I16Vector4: ...
    @property
    def rrrl(self) -> I16Vector4: ...
    @property
    def rrgr(self) -> I16Vector4: ...
    @property
    def rrgg(self) -> I16Vector4: ...
    @property
    def rrgb(self) -> I16Vector4: ...
    @property
    def rrgo(self) -> I16Vector4: ...
    @property
    def rrgl(self) -> I16Vector4: ...
    @property
    def rrbr(self) -> I16Vector4: ...
    @property
    def rrbg(self) -> I16Vector4: ...
    @property
    def rrbb(self) -> I16Vector4: ...
    @property
    def rrbo(self) -> I16Vector4: ...
    @property
    def rrbl(self) -> I16Vector4: ...
    @property
    def rror(self) -> I16Vector4: ...
    @property
    def rrog(self) -> I16Vector4: ...
    @property
    def rrob(self) -> I16Vector4: ...
    @property
    def rroo(self) -> I16Vector4: ...
    @property
    def rrol(self) -> I16Vector4: ...
    @property
    def rrlr(self) -> I16Vector4: ...
    @property
    def rrlg(self) -> I16Vector4: ...
    @property
    def rrlb(self) -> I16Vector4: ...
    @property
    def rrlo(self) -> I16Vector4: ...
    @property
    def rrll(self) -> I16Vector4: ...
    @property
    def rgrr(self) -> I16Vector4: ...
    @property
    def rgrg(self) -> I16Vector4: ...
    @property
    def rgrb(self) -> I16Vector4: ...
    @property
    def rgro(self) -> I16Vector4: ...
    @property
    def rgrl(self) -> I16Vector4: ...
    @property
    def rggr(self) -> I16Vector4: ...
    @property
    def rggg(self) -> I16Vector4: ...
    @property
    def rggb(self) -> I16Vector4: ...
    @property
    def rggo(self) -> I16Vector4: ...
    @property
    def rggl(self) -> I16Vector4: ...
    @property
    def rgbr(self) -> I16Vector4: ...
    @property
    def rgbg(self) -> I16Vector4: ...
    @property
    def rgbb(self) -> I16Vector4: ...
    @property
    def rgbo(self) -> I16Vector4: ...
    @property
    def rgbl(self) -> I16Vector4: ...
    @property
    def rgor(self) -> I16Vector4: ...
    @property
    def rgog(self) -> I16Vector4: ...
    @property
    def rgob(self) -> I16Vector4: ...
    @property
    def rgoo(self) -> I16Vector4: ...
    @property
    def rgol(self) -> I16Vector4: ...
    @property
    def rglr(self) -> I16Vector4: ...
    @property
    def rglg(self) -> I16Vector4: ...
    @property
    def rglb(self) -> I16Vector4: ...
    @property
    def rglo(self) -> I16Vector4: ...
    @property
    def rgll(self) -> I16Vector4: ...
    @property
    def rbrr(self) -> I16Vector4: ...
    @property
    def rbrg(self) -> I16Vector4: ...
    @property
    def rbrb(self) -> I16Vector4: ...
    @property
    def rbro(self) -> I16Vector4: ...
    @property
    def rbrl(self) -> I16Vector4: ...
    @property
    def rbgr(self) -> I16Vector4: ...
    @property
    def rbgg(self) -> I16Vector4: ...
    @property
    def rbgb(self) -> I16Vector4: ...
    @property
    def rbgo(self) -> I16Vector4: ...
    @property
    def rbgl(self) -> I16Vector4: ...
    @property
    def rbbr(self) -> I16Vector4: ...
    @property
    def rbbg(self) -> I16Vector4: ...
    @property
    def rbbb(self) -> I16Vector4: ...
    @property
    def rbbo(self) -> I16Vector4: ...
    @property
    def rbbl(self) -> I16Vector4: ...
    @property
    def rbor(self) -> I16Vector4: ...
    @property
    def rbog(self) -> I16Vector4: ...
    @property
    def rbob(self) -> I16Vector4: ...
    @property
    def rboo(self) -> I16Vector4: ...
    @property
    def rbol(self) -> I16Vector4: ...
    @property
    def rblr(self) -> I16Vector4: ...
    @property
    def rblg(self) -> I16Vector4: ...
    @property
    def rblb(self) -> I16Vector4: ...
    @property
    def rblo(self) -> I16Vector4: ...
    @property
    def rbll(self) -> I16Vector4: ...
    @property
    def rorr(self) -> I16Vector4: ...
    @property
    def rorg(self) -> I16Vector4: ...
    @property
    def rorb(self) -> I16Vector4: ...
    @property
    def roro(self) -> I16Vector4: ...
    @property
    def rorl(self) -> I16Vector4: ...
    @property
    def rogr(self) -> I16Vector4: ...
    @property
    def rogg(self) -> I16Vector4: ...
    @property
    def rogb(self) -> I16Vector4: ...
    @property
    def rogo(self) -> I16Vector4: ...
    @property
    def rogl(self) -> I16Vector4: ...
    @property
    def robr(self) -> I16Vector4: ...
    @property
    def robg(self) -> I16Vector4: ...
    @property
    def robb(self) -> I16Vector4: ...
    @property
    def robo(self) -> I16Vector4: ...
    @property
    def robl(self) -> I16Vector4: ...
    @property
    def roor(self) -> I16Vector4: ...
    @property
    def roog(self) -> I16Vector4: ...
    @property
    def roob(self) -> I16Vector4: ...
    @property
    def rooo(self) -> I16Vector4: ...
    @property
    def rool(self) -> I16Vector4: ...
    @property
    def rolr(self) -> I16Vector4: ...
    @property
    def rolg(self) -> I16Vector4: ...
    @property
    def rolb(self) -> I16Vector4: ...
    @property
    def rolo(self) -> I16Vector4: ...
    @property
    def roll(self) -> I16Vector4: ...
    @property
    def rlrr(self) -> I16Vector4: ...
    @property
    def rlrg(self) -> I16Vector4: ...
    @property
    def rlrb(self) -> I16Vector4: ...
    @property
    def rlro(self) -> I16Vector4: ...
    @property
    def rlrl(self) -> I16Vector4: ...
    @property
    def rlgr(self) -> I16Vector4: ...
    @property
    def rlgg(self) -> I16Vector4: ...
    @property
    def rlgb(self) -> I16Vector4: ...
    @property
    def rlgo(self) -> I16Vector4: ...
    @property
    def rlgl(self) -> I16Vector4: ...
    @property
    def rlbr(self) -> I16Vector4: ...
    @property
    def rlbg(self) -> I16Vector4: ...
    @property
    def rlbb(self) -> I16Vector4: ...
    @property
    def rlbo(self) -> I16Vector4: ...
    @property
    def rlbl(self) -> I16Vector4: ...
    @property
    def rlor(self) -> I16Vector4: ...
    @property
    def rlog(self) -> I16Vector4: ...
    @property
    def rlob(self) -> I16Vector4: ...
    @property
    def rloo(self) -> I16Vector4: ...
    @property
    def rlol(self) -> I16Vector4: ...
    @property
    def rllr(self) -> I16Vector4: ...
    @property
    def rllg(self) -> I16Vector4: ...
    @property
    def rllb(self) -> I16Vector4: ...
    @property
    def rllo(self) -> I16Vector4: ...
    @property
    def rlll(self) -> I16Vector4: ...
    @property
    def grrr(self) -> I16Vector4: ...
    @property
    def grrg(self) -> I16Vector4: ...
    @property
    def grrb(self) -> I16Vector4: ...
    @property
    def grro(self) -> I16Vector4: ...
    @property
    def grrl(self) -> I16Vector4: ...
    @property
    def grgr(self) -> I16Vector4: ...
    @property
    def grgg(self) -> I16Vector4: ...
    @property
    def grgb(self) -> I16Vector4: ...
    @property
    def grgo(self) -> I16Vector4: ...
    @property
    def grgl(self) -> I16Vector4: ...
    @property
    def grbr(self) -> I16Vector4: ...
    @property
    def grbg(self) -> I16Vector4: ...
    @property
    def grbb(self) -> I16Vector4: ...
    @property
    def grbo(self) -> I16Vector4: ...
    @property
    def grbl(self) -> I16Vector4: ...
    @property
    def gror(self) -> I16Vector4: ...
    @property
    def grog(self) -> I16Vector4: ...
    @property
    def grob(self) -> I16Vector4: ...
    @property
    def groo(self) -> I16Vector4: ...
    @property
    def grol(self) -> I16Vector4: ...
    @property
    def grlr(self) -> I16Vector4: ...
    @property
    def grlg(self) -> I16Vector4: ...
    @property
    def grlb(self) -> I16Vector4: ...
    @property
    def grlo(self) -> I16Vector4: ...
    @property
    def grll(self) -> I16Vector4: ...
    @property
    def ggrr(self) -> I16Vector4: ...
    @property
    def ggrg(self) -> I16Vector4: ...
    @property
    def ggrb(self) -> I16Vector4: ...
    @property
    def ggro(self) -> I16Vector4: ...
    @property
    def ggrl(self) -> I16Vector4: ...
    @property
    def gggr(self) -> I16Vector4: ...
    @property
    def gggg(self) -> I16Vector4: ...
    @property
    def gggb(self) -> I16Vector4: ...
    @property
    def gggo(self) -> I16Vector4: ...
    @property
    def gggl(self) -> I16Vector4: ...
    @property
    def ggbr(self) -> I16Vector4: ...
    @property
    def ggbg(self) -> I16Vector4: ...
    @property
    def ggbb(self) -> I16Vector4: ...
    @property
    def ggbo(self) -> I16Vector4: ...
    @property
    def ggbl(self) -> I16Vector4: ...
    @property
    def ggor(self) -> I16Vector4: ...
    @property
    def ggog(self) -> I16Vector4: ...
    @property
    def ggob(self) -> I16Vector4: ...
    @property
    def ggoo(self) -> I16Vector4: ...
    @property
    def ggol(self) -> I16Vector4: ...
    @property
    def gglr(self) -> I16Vector4: ...
    @property
    def gglg(self) -> I16Vector4: ...
    @property
    def gglb(self) -> I16Vector4: ...
    @property
    def gglo(self) -> I16Vector4: ...
    @property
    def ggll(self) -> I16Vector4: ...
    @property
    def gbrr(self) -> I16Vector4: ...
    @property
    def gbrg(self) -> I16Vector4: ...
    @property
    def gbrb(self) -> I16Vector4: ...
    @property
    def gbro(self) -> I16Vector4: ...
    @property
    def gbrl(self) -> I16Vector4: ...
    @property
    def gbgr(self) -> I16Vector4: ...
    @property
    def gbgg(self) -> I16Vector4: ...
    @property
    def gbgb(self) -> I16Vector4: ...
    @property
    def gbgo(self) -> I16Vector4: ...
    @property
    def gbgl(self) -> I16Vector4: ...
    @property
    def gbbr(self) -> I16Vector4: ...
    @property
    def gbbg(self) -> I16Vector4: ...
    @property
    def gbbb(self) -> I16Vector4: ...
    @property
    def gbbo(self) -> I16Vector4: ...
    @property
    def gbbl(self) -> I16Vector4: ...
    @property
    def gbor(self) -> I16Vector4: ...
    @property
    def gbog(self) -> I16Vector4: ...
    @property
    def gbob(self) -> I16Vector4: ...
    @property
    def gboo(self) -> I16Vector4: ...
    @property
    def gbol(self) -> I16Vector4: ...
    @property
    def gblr(self) -> I16Vector4: ...
    @property
    def gblg(self) -> I16Vector4: ...
    @property
    def gblb(self) -> I16Vector4: ...
    @property
    def gblo(self) -> I16Vector4: ...
    @property
    def gbll(self) -> I16Vector4: ...
    @property
    def gorr(self) -> I16Vector4: ...
    @property
    def gorg(self) -> I16Vector4: ...
    @property
    def gorb(self) -> I16Vector4: ...
    @property
    def goro(self) -> I16Vector4: ...
    @property
    def gorl(self) -> I16Vector4: ...
    @property
    def gogr(self) -> I16Vector4: ...
    @property
    def gogg(self) -> I16Vector4: ...
    @property
    def gogb(self) -> I16Vector4: ...
    @property
    def gogo(self) -> I16Vector4: ...
    @property
    def gogl(self) -> I16Vector4: ...
    @property
    def gobr(self) -> I16Vector4: ...
    @property
    def gobg(self) -> I16Vector4: ...
    @property
    def gobb(self) -> I16Vector4: ...
    @property
    def gobo(self) -> I16Vector4: ...
    @property
    def gobl(self) -> I16Vector4: ...
    @property
    def goor(self) -> I16Vector4: ...
    @property
    def goog(self) -> I16Vector4: ...
    @property
    def goob(self) -> I16Vector4: ...
    @property
    def gooo(self) -> I16Vector4: ...
    @property
    def gool(self) -> I16Vector4: ...
    @property
    def golr(self) -> I16Vector4: ...
    @property
    def golg(self) -> I16Vector4: ...
    @property
    def golb(self) -> I16Vector4: ...
    @property
    def golo(self) -> I16Vector4: ...
    @property
    def goll(self) -> I16Vector4: ...
    @property
    def glrr(self) -> I16Vector4: ...
    @property
    def glrg(self) -> I16Vector4: ...
    @property
    def glrb(self) -> I16Vector4: ...
    @property
    def glro(self) -> I16Vector4: ...
    @property
    def glrl(self) -> I16Vector4: ...
    @property
    def glgr(self) -> I16Vector4: ...
    @property
    def glgg(self) -> I16Vector4: ...
    @property
    def glgb(self) -> I16Vector4: ...
    @property
    def glgo(self) -> I16Vector4: ...
    @property
    def glgl(self) -> I16Vector4: ...
    @property
    def glbr(self) -> I16Vector4: ...
    @property
    def glbg(self) -> I16Vector4: ...
    @property
    def glbb(self) -> I16Vector4: ...
    @property
    def glbo(self) -> I16Vector4: ...
    @property
    def glbl(self) -> I16Vector4: ...
    @property
    def glor(self) -> I16Vector4: ...
    @property
    def glog(self) -> I16Vector4: ...
    @property
    def glob(self) -> I16Vector4: ...
    @property
    def gloo(self) -> I16Vector4: ...
    @property
    def glol(self) -> I16Vector4: ...
    @property
    def gllr(self) -> I16Vector4: ...
    @property
    def gllg(self) -> I16Vector4: ...
    @property
    def gllb(self) -> I16Vector4: ...
    @property
    def gllo(self) -> I16Vector4: ...
    @property
    def glll(self) -> I16Vector4: ...
    @property
    def brrr(self) -> I16Vector4: ...
    @property
    def brrg(self) -> I16Vector4: ...
    @property
    def brrb(self) -> I16Vector4: ...
    @property
    def brro(self) -> I16Vector4: ...
    @property
    def brrl(self) -> I16Vector4: ...
    @property
    def brgr(self) -> I16Vector4: ...
    @property
    def brgg(self) -> I16Vector4: ...
    @property
    def brgb(self) -> I16Vector4: ...
    @property
    def brgo(self) -> I16Vector4: ...
    @property
    def brgl(self) -> I16Vector4: ...
    @property
    def brbr(self) -> I16Vector4: ...
    @property
    def brbg(self) -> I16Vector4: ...
    @property
    def brbb(self) -> I16Vector4: ...
    @property
    def brbo(self) -> I16Vector4: ...
    @property
    def brbl(self) -> I16Vector4: ...
    @property
    def bror(self) -> I16Vector4: ...
    @property
    def brog(self) -> I16Vector4: ...
    @property
    def brob(self) -> I16Vector4: ...
    @property
    def broo(self) -> I16Vector4: ...
    @property
    def brol(self) -> I16Vector4: ...
    @property
    def brlr(self) -> I16Vector4: ...
    @property
    def brlg(self) -> I16Vector4: ...
    @property
    def brlb(self) -> I16Vector4: ...
    @property
    def brlo(self) -> I16Vector4: ...
    @property
    def brll(self) -> I16Vector4: ...
    @property
    def bgrr(self) -> I16Vector4: ...
    @property
    def bgrg(self) -> I16Vector4: ...
    @property
    def bgrb(self) -> I16Vector4: ...
    @property
    def bgro(self) -> I16Vector4: ...
    @property
    def bgrl(self) -> I16Vector4: ...
    @property
    def bggr(self) -> I16Vector4: ...
    @property
    def bggg(self) -> I16Vector4: ...
    @property
    def bggb(self) -> I16Vector4: ...
    @property
    def bggo(self) -> I16Vector4: ...
    @property
    def bggl(self) -> I16Vector4: ...
    @property
    def bgbr(self) -> I16Vector4: ...
    @property
    def bgbg(self) -> I16Vector4: ...
    @property
    def bgbb(self) -> I16Vector4: ...
    @property
    def bgbo(self) -> I16Vector4: ...
    @property
    def bgbl(self) -> I16Vector4: ...
    @property
    def bgor(self) -> I16Vector4: ...
    @property
    def bgog(self) -> I16Vector4: ...
    @property
    def bgob(self) -> I16Vector4: ...
    @property
    def bgoo(self) -> I16Vector4: ...
    @property
    def bgol(self) -> I16Vector4: ...
    @property
    def bglr(self) -> I16Vector4: ...
    @property
    def bglg(self) -> I16Vector4: ...
    @property
    def bglb(self) -> I16Vector4: ...
    @property
    def bglo(self) -> I16Vector4: ...
    @property
    def bgll(self) -> I16Vector4: ...
    @property
    def bbrr(self) -> I16Vector4: ...
    @property
    def bbrg(self) -> I16Vector4: ...
    @property
    def bbrb(self) -> I16Vector4: ...
    @property
    def bbro(self) -> I16Vector4: ...
    @property
    def bbrl(self) -> I16Vector4: ...
    @property
    def bbgr(self) -> I16Vector4: ...
    @property
    def bbgg(self) -> I16Vector4: ...
    @property
    def bbgb(self) -> I16Vector4: ...
    @property
    def bbgo(self) -> I16Vector4: ...
    @property
    def bbgl(self) -> I16Vector4: ...
    @property
    def bbbr(self) -> I16Vector4: ...
    @property
    def bbbg(self) -> I16Vector4: ...
    @property
    def bbbb(self) -> I16Vector4: ...
    @property
    def bbbo(self) -> I16Vector4: ...
    @property
    def bbbl(self) -> I16Vector4: ...
    @property
    def bbor(self) -> I16Vector4: ...
    @property
    def bbog(self) -> I16Vector4: ...
    @property
    def bbob(self) -> I16Vector4: ...
    @property
    def bboo(self) -> I16Vector4: ...
    @property
    def bbol(self) -> I16Vector4: ...
    @property
    def bblr(self) -> I16Vector4: ...
    @property
    def bblg(self) -> I16Vector4: ...
    @property
    def bblb(self) -> I16Vector4: ...
    @property
    def bblo(self) -> I16Vector4: ...
    @property
    def bbll(self) -> I16Vector4: ...
    @property
    def borr(self) -> I16Vector4: ...
    @property
    def borg(self) -> I16Vector4: ...
    @property
    def borb(self) -> I16Vector4: ...
    @property
    def boro(self) -> I16Vector4: ...
    @property
    def borl(self) -> I16Vector4: ...
    @property
    def bogr(self) -> I16Vector4: ...
    @property
    def bogg(self) -> I16Vector4: ...
    @property
    def bogb(self) -> I16Vector4: ...
    @property
    def bogo(self) -> I16Vector4: ...
    @property
    def bogl(self) -> I16Vector4: ...
    @property
    def bobr(self) -> I16Vector4: ...
    @property
    def bobg(self) -> I16Vector4: ...
    @property
    def bobb(self) -> I16Vector4: ...
    @property
    def bobo(self) -> I16Vector4: ...
    @property
    def bobl(self) -> I16Vector4: ...
    @property
    def boor(self) -> I16Vector4: ...
    @property
    def boog(self) -> I16Vector4: ...
    @property
    def boob(self) -> I16Vector4: ...
    @property
    def booo(self) -> I16Vector4: ...
    @property
    def bool(self) -> I16Vector4: ...
    @property
    def bolr(self) -> I16Vector4: ...
    @property
    def bolg(self) -> I16Vector4: ...
    @property
    def bolb(self) -> I16Vector4: ...
    @property
    def bolo(self) -> I16Vector4: ...
    @property
    def boll(self) -> I16Vector4: ...
    @property
    def blrr(self) -> I16Vector4: ...
    @property
    def blrg(self) -> I16Vector4: ...
    @property
    def blrb(self) -> I16Vector4: ...
    @property
    def blro(self) -> I16Vector4: ...
    @property
    def blrl(self) -> I16Vector4: ...
    @property
    def blgr(self) -> I16Vector4: ...
    @property
    def blgg(self) -> I16Vector4: ...
    @property
    def blgb(self) -> I16Vector4: ...
    @property
    def blgo(self) -> I16Vector4: ...
    @property
    def blgl(self) -> I16Vector4: ...
    @property
    def blbr(self) -> I16Vector4: ...
    @property
    def blbg(self) -> I16Vector4: ...
    @property
    def blbb(self) -> I16Vector4: ...
    @property
    def blbo(self) -> I16Vector4: ...
    @property
    def blbl(self) -> I16Vector4: ...
    @property
    def blor(self) -> I16Vector4: ...
    @property
    def blog(self) -> I16Vector4: ...
    @property
    def blob(self) -> I16Vector4: ...
    @property
    def bloo(self) -> I16Vector4: ...
    @property
    def blol(self) -> I16Vector4: ...
    @property
    def bllr(self) -> I16Vector4: ...
    @property
    def bllg(self) -> I16Vector4: ...
    @property
    def bllb(self) -> I16Vector4: ...
    @property
    def bllo(self) -> I16Vector4: ...
    @property
    def blll(self) -> I16Vector4: ...
    @property
    def orrr(self) -> I16Vector4: ...
    @property
    def orrg(self) -> I16Vector4: ...
    @property
    def orrb(self) -> I16Vector4: ...
    @property
    def orro(self) -> I16Vector4: ...
    @property
    def orrl(self) -> I16Vector4: ...
    @property
    def orgr(self) -> I16Vector4: ...
    @property
    def orgg(self) -> I16Vector4: ...
    @property
    def orgb(self) -> I16Vector4: ...
    @property
    def orgo(self) -> I16Vector4: ...
    @property
    def orgl(self) -> I16Vector4: ...
    @property
    def orbr(self) -> I16Vector4: ...
    @property
    def orbg(self) -> I16Vector4: ...
    @property
    def orbb(self) -> I16Vector4: ...
    @property
    def orbo(self) -> I16Vector4: ...
    @property
    def orbl(self) -> I16Vector4: ...
    @property
    def oror(self) -> I16Vector4: ...
    @property
    def orog(self) -> I16Vector4: ...
    @property
    def orob(self) -> I16Vector4: ...
    @property
    def oroo(self) -> I16Vector4: ...
    @property
    def orol(self) -> I16Vector4: ...
    @property
    def orlr(self) -> I16Vector4: ...
    @property
    def orlg(self) -> I16Vector4: ...
    @property
    def orlb(self) -> I16Vector4: ...
    @property
    def orlo(self) -> I16Vector4: ...
    @property
    def orll(self) -> I16Vector4: ...
    @property
    def ogrr(self) -> I16Vector4: ...
    @property
    def ogrg(self) -> I16Vector4: ...
    @property
    def ogrb(self) -> I16Vector4: ...
    @property
    def ogro(self) -> I16Vector4: ...
    @property
    def ogrl(self) -> I16Vector4: ...
    @property
    def oggr(self) -> I16Vector4: ...
    @property
    def oggg(self) -> I16Vector4: ...
    @property
    def oggb(self) -> I16Vector4: ...
    @property
    def oggo(self) -> I16Vector4: ...
    @property
    def oggl(self) -> I16Vector4: ...
    @property
    def ogbr(self) -> I16Vector4: ...
    @property
    def ogbg(self) -> I16Vector4: ...
    @property
    def ogbb(self) -> I16Vector4: ...
    @property
    def ogbo(self) -> I16Vector4: ...
    @property
    def ogbl(self) -> I16Vector4: ...
    @property
    def ogor(self) -> I16Vector4: ...
    @property
    def ogog(self) -> I16Vector4: ...
    @property
    def ogob(self) -> I16Vector4: ...
    @property
    def ogoo(self) -> I16Vector4: ...
    @property
    def ogol(self) -> I16Vector4: ...
    @property
    def oglr(self) -> I16Vector4: ...
    @property
    def oglg(self) -> I16Vector4: ...
    @property
    def oglb(self) -> I16Vector4: ...
    @property
    def oglo(self) -> I16Vector4: ...
    @property
    def ogll(self) -> I16Vector4: ...
    @property
    def obrr(self) -> I16Vector4: ...
    @property
    def obrg(self) -> I16Vector4: ...
    @property
    def obrb(self) -> I16Vector4: ...
    @property
    def obro(self) -> I16Vector4: ...
    @property
    def obrl(self) -> I16Vector4: ...
    @property
    def obgr(self) -> I16Vector4: ...
    @property
    def obgg(self) -> I16Vector4: ...
    @property
    def obgb(self) -> I16Vector4: ...
    @property
    def obgo(self) -> I16Vector4: ...
    @property
    def obgl(self) -> I16Vector4: ...
    @property
    def obbr(self) -> I16Vector4: ...
    @property
    def obbg(self) -> I16Vector4: ...
    @property
    def obbb(self) -> I16Vector4: ...
    @property
    def obbo(self) -> I16Vector4: ...
    @property
    def obbl(self) -> I16Vector4: ...
    @property
    def obor(self) -> I16Vector4: ...
    @property
    def obog(self) -> I16Vector4: ...
    @property
    def obob(self) -> I16Vector4: ...
    @property
    def oboo(self) -> I16Vector4: ...
    @property
    def obol(self) -> I16Vector4: ...
    @property
    def oblr(self) -> I16Vector4: ...
    @property
    def oblg(self) -> I16Vector4: ...
    @property
    def oblb(self) -> I16Vector4: ...
    @property
    def oblo(self) -> I16Vector4: ...
    @property
    def obll(self) -> I16Vector4: ...
    @property
    def oorr(self) -> I16Vector4: ...
    @property
    def oorg(self) -> I16Vector4: ...
    @property
    def oorb(self) -> I16Vector4: ...
    @property
    def ooro(self) -> I16Vector4: ...
    @property
    def oorl(self) -> I16Vector4: ...
    @property
    def oogr(self) -> I16Vector4: ...
    @property
    def oogg(self) -> I16Vector4: ...
    @property
    def oogb(self) -> I16Vector4: ...
    @property
    def oogo(self) -> I16Vector4: ...
    @property
    def oogl(self) -> I16Vector4: ...
    @property
    def oobr(self) -> I16Vector4: ...
    @property
    def oobg(self) -> I16Vector4: ...
    @property
    def oobb(self) -> I16Vector4: ...
    @property
    def oobo(self) -> I16Vector4: ...
    @property
    def oobl(self) -> I16Vector4: ...
    @property
    def ooor(self) -> I16Vector4: ...
    @property
    def ooog(self) -> I16Vector4: ...
    @property
    def ooob(self) -> I16Vector4: ...
    @property
    def oolr(self) -> I16Vector4: ...
    @property
    def oolg(self) -> I16Vector4: ...
    @property
    def oolb(self) -> I16Vector4: ...
    @property
    def olrr(self) -> I16Vector4: ...
    @property
    def olrg(self) -> I16Vector4: ...
    @property
    def olrb(self) -> I16Vector4: ...
    @property
    def olro(self) -> I16Vector4: ...
    @property
    def olrl(self) -> I16Vector4: ...
    @property
    def olgr(self) -> I16Vector4: ...
    @property
    def olgg(self) -> I16Vector4: ...
    @property
    def olgb(self) -> I16Vector4: ...
    @property
    def olgo(self) -> I16Vector4: ...
    @property
    def olgl(self) -> I16Vector4: ...
    @property
    def olbr(self) -> I16Vector4: ...
    @property
    def olbg(self) -> I16Vector4: ...
    @property
    def olbb(self) -> I16Vector4: ...
    @property
    def olbo(self) -> I16Vector4: ...
    @property
    def olbl(self) -> I16Vector4: ...
    @property
    def olor(self) -> I16Vector4: ...
    @property
    def olog(self) -> I16Vector4: ...
    @property
    def olob(self) -> I16Vector4: ...
    @property
    def ollr(self) -> I16Vector4: ...
    @property
    def ollg(self) -> I16Vector4: ...
    @property
    def ollb(self) -> I16Vector4: ...
    @property
    def lrrr(self) -> I16Vector4: ...
    @property
    def lrrg(self) -> I16Vector4: ...
    @property
    def lrrb(self) -> I16Vector4: ...
    @property
    def lrro(self) -> I16Vector4: ...
    @property
    def lrrl(self) -> I16Vector4: ...
    @property
    def lrgr(self) -> I16Vector4: ...
    @property
    def lrgg(self) -> I16Vector4: ...
    @property
    def lrgb(self) -> I16Vector4: ...
    @property
    def lrgo(self) -> I16Vector4: ...
    @property
    def lrgl(self) -> I16Vector4: ...
    @property
    def lrbr(self) -> I16Vector4: ...
    @property
    def lrbg(self) -> I16Vector4: ...
    @property
    def lrbb(self) -> I16Vector4: ...
    @property
    def lrbo(self) -> I16Vector4: ...
    @property
    def lrbl(self) -> I16Vector4: ...
    @property
    def lror(self) -> I16Vector4: ...
    @property
    def lrog(self) -> I16Vector4: ...
    @property
    def lrob(self) -> I16Vector4: ...
    @property
    def lroo(self) -> I16Vector4: ...
    @property
    def lrol(self) -> I16Vector4: ...
    @property
    def lrlr(self) -> I16Vector4: ...
    @property
    def lrlg(self) -> I16Vector4: ...
    @property
    def lrlb(self) -> I16Vector4: ...
    @property
    def lrlo(self) -> I16Vector4: ...
    @property
    def lrll(self) -> I16Vector4: ...
    @property
    def lgrr(self) -> I16Vector4: ...
    @property
    def lgrg(self) -> I16Vector4: ...
    @property
    def lgrb(self) -> I16Vector4: ...
    @property
    def lgro(self) -> I16Vector4: ...
    @property
    def lgrl(self) -> I16Vector4: ...
    @property
    def lggr(self) -> I16Vector4: ...
    @property
    def lggg(self) -> I16Vector4: ...
    @property
    def lggb(self) -> I16Vector4: ...
    @property
    def lggo(self) -> I16Vector4: ...
    @property
    def lggl(self) -> I16Vector4: ...
    @property
    def lgbr(self) -> I16Vector4: ...
    @property
    def lgbg(self) -> I16Vector4: ...
    @property
    def lgbb(self) -> I16Vector4: ...
    @property
    def lgbo(self) -> I16Vector4: ...
    @property
    def lgbl(self) -> I16Vector4: ...
    @property
    def lgor(self) -> I16Vector4: ...
    @property
    def lgog(self) -> I16Vector4: ...
    @property
    def lgob(self) -> I16Vector4: ...
    @property
    def lgoo(self) -> I16Vector4: ...
    @property
    def lgol(self) -> I16Vector4: ...
    @property
    def lglr(self) -> I16Vector4: ...
    @property
    def lglg(self) -> I16Vector4: ...
    @property
    def lglb(self) -> I16Vector4: ...
    @property
    def lglo(self) -> I16Vector4: ...
    @property
    def lgll(self) -> I16Vector4: ...
    @property
    def lbrr(self) -> I16Vector4: ...
    @property
    def lbrg(self) -> I16Vector4: ...
    @property
    def lbrb(self) -> I16Vector4: ...
    @property
    def lbro(self) -> I16Vector4: ...
    @property
    def lbrl(self) -> I16Vector4: ...
    @property
    def lbgr(self) -> I16Vector4: ...
    @property
    def lbgg(self) -> I16Vector4: ...
    @property
    def lbgb(self) -> I16Vector4: ...
    @property
    def lbgo(self) -> I16Vector4: ...
    @property
    def lbgl(self) -> I16Vector4: ...
    @property
    def lbbr(self) -> I16Vector4: ...
    @property
    def lbbg(self) -> I16Vector4: ...
    @property
    def lbbb(self) -> I16Vector4: ...
    @property
    def lbbo(self) -> I16Vector4: ...
    @property
    def lbbl(self) -> I16Vector4: ...
    @property
    def lbor(self) -> I16Vector4: ...
    @property
    def lbog(self) -> I16Vector4: ...
    @property
    def lbob(self) -> I16Vector4: ...
    @property
    def lboo(self) -> I16Vector4: ...
    @property
    def lbol(self) -> I16Vector4: ...
    @property
    def lblr(self) -> I16Vector4: ...
    @property
    def lblg(self) -> I16Vector4: ...
    @property
    def lblb(self) -> I16Vector4: ...
    @property
    def lblo(self) -> I16Vector4: ...
    @property
    def lbll(self) -> I16Vector4: ...
    @property
    def lorr(self) -> I16Vector4: ...
    @property
    def lorg(self) -> I16Vector4: ...
    @property
    def lorb(self) -> I16Vector4: ...
    @property
    def loro(self) -> I16Vector4: ...
    @property
    def lorl(self) -> I16Vector4: ...
    @property
    def logr(self) -> I16Vector4: ...
    @property
    def logg(self) -> I16Vector4: ...
    @property
    def logb(self) -> I16Vector4: ...
    @property
    def logo(self) -> I16Vector4: ...
    @property
    def logl(self) -> I16Vector4: ...
    @property
    def lobr(self) -> I16Vector4: ...
    @property
    def lobg(self) -> I16Vector4: ...
    @property
    def lobb(self) -> I16Vector4: ...
    @property
    def lobo(self) -> I16Vector4: ...
    @property
    def lobl(self) -> I16Vector4: ...
    @property
    def loor(self) -> I16Vector4: ...
    @property
    def loog(self) -> I16Vector4: ...
    @property
    def loob(self) -> I16Vector4: ...
    @property
    def lolr(self) -> I16Vector4: ...
    @property
    def lolg(self) -> I16Vector4: ...
    @property
    def lolb(self) -> I16Vector4: ...
    @property
    def llrr(self) -> I16Vector4: ...
    @property
    def llrg(self) -> I16Vector4: ...
    @property
    def llrb(self) -> I16Vector4: ...
    @property
    def llro(self) -> I16Vector4: ...
    @property
    def llrl(self) -> I16Vector4: ...
    @property
    def llgr(self) -> I16Vector4: ...
    @property
    def llgg(self) -> I16Vector4: ...
    @property
    def llgb(self) -> I16Vector4: ...
    @property
    def llgo(self) -> I16Vector4: ...
    @property
    def llgl(self) -> I16Vector4: ...
    @property
    def llbr(self) -> I16Vector4: ...
    @property
    def llbg(self) -> I16Vector4: ...
    @property
    def llbb(self) -> I16Vector4: ...
    @property
    def llbo(self) -> I16Vector4: ...
    @property
    def llbl(self) -> I16Vector4: ...
    @property
    def llor(self) -> I16Vector4: ...
    @property
    def llog(self) -> I16Vector4: ...
    @property
    def llob(self) -> I16Vector4: ...
    @property
    def lllr(self) -> I16Vector4: ...
    @property
    def lllg(self) -> I16Vector4: ...
    @property
    def lllb(self) -> I16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> I16Vector2: ...
    @property
    def st(self) -> I16Vector2: ...
    @property
    def sq(self) -> I16Vector2: ...
    @property
    def so(self) -> I16Vector2: ...
    @property
    def sl(self) -> I16Vector2: ...
    @property
    def ts(self) -> I16Vector2: ...
    @property
    def tt(self) -> I16Vector2: ...
    @property
    def tq(self) -> I16Vector2: ...
    @property
    def to(self) -> I16Vector2: ...
    @property
    def tl(self) -> I16Vector2: ...
    @property
    def qs(self) -> I16Vector2: ...
    @property
    def qt(self) -> I16Vector2: ...
    @property
    def qq(self) -> I16Vector2: ...
    @property
    def qo(self) -> I16Vector2: ...
    @property
    def ql(self) -> I16Vector2: ...
    @property
    def os(self) -> I16Vector2: ...
    @property
    def ot(self) -> I16Vector2: ...
    @property
    def oq(self) -> I16Vector2: ...
    @property
    def ls(self) -> I16Vector2: ...
    @property
    def lt(self) -> I16Vector2: ...
    @property
    def lq(self) -> I16Vector2: ...
    @property
    def sss(self) -> I16Vector3: ...
    @property
    def sst(self) -> I16Vector3: ...
    @property
    def ssq(self) -> I16Vector3: ...
    @property
    def sso(self) -> I16Vector3: ...
    @property
    def ssl(self) -> I16Vector3: ...
    @property
    def sts(self) -> I16Vector3: ...
    @property
    def stt(self) -> I16Vector3: ...
    @property
    def stq(self) -> I16Vector3: ...
    @property
    def sto(self) -> I16Vector3: ...
    @property
    def stl(self) -> I16Vector3: ...
    @property
    def sqs(self) -> I16Vector3: ...
    @property
    def sqt(self) -> I16Vector3: ...
    @property
    def sqq(self) -> I16Vector3: ...
    @property
    def sqo(self) -> I16Vector3: ...
    @property
    def sql(self) -> I16Vector3: ...
    @property
    def sos(self) -> I16Vector3: ...
    @property
    def sot(self) -> I16Vector3: ...
    @property
    def soq(self) -> I16Vector3: ...
    @property
    def soo(self) -> I16Vector3: ...
    @property
    def sol(self) -> I16Vector3: ...
    @property
    def sls(self) -> I16Vector3: ...
    @property
    def slt(self) -> I16Vector3: ...
    @property
    def slq(self) -> I16Vector3: ...
    @property
    def slo(self) -> I16Vector3: ...
    @property
    def sll(self) -> I16Vector3: ...
    @property
    def tss(self) -> I16Vector3: ...
    @property
    def tst(self) -> I16Vector3: ...
    @property
    def tsq(self) -> I16Vector3: ...
    @property
    def tso(self) -> I16Vector3: ...
    @property
    def tsl(self) -> I16Vector3: ...
    @property
    def tts(self) -> I16Vector3: ...
    @property
    def ttt(self) -> I16Vector3: ...
    @property
    def ttq(self) -> I16Vector3: ...
    @property
    def tto(self) -> I16Vector3: ...
    @property
    def ttl(self) -> I16Vector3: ...
    @property
    def tqs(self) -> I16Vector3: ...
    @property
    def tqt(self) -> I16Vector3: ...
    @property
    def tqq(self) -> I16Vector3: ...
    @property
    def tqo(self) -> I16Vector3: ...
    @property
    def tql(self) -> I16Vector3: ...
    @property
    def tos(self) -> I16Vector3: ...
    @property
    def tot(self) -> I16Vector3: ...
    @property
    def toq(self) -> I16Vector3: ...
    @property
    def too(self) -> I16Vector3: ...
    @property
    def tol(self) -> I16Vector3: ...
    @property
    def tls(self) -> I16Vector3: ...
    @property
    def tlt(self) -> I16Vector3: ...
    @property
    def tlq(self) -> I16Vector3: ...
    @property
    def tlo(self) -> I16Vector3: ...
    @property
    def tll(self) -> I16Vector3: ...
    @property
    def qss(self) -> I16Vector3: ...
    @property
    def qst(self) -> I16Vector3: ...
    @property
    def qsq(self) -> I16Vector3: ...
    @property
    def qso(self) -> I16Vector3: ...
    @property
    def qsl(self) -> I16Vector3: ...
    @property
    def qts(self) -> I16Vector3: ...
    @property
    def qtt(self) -> I16Vector3: ...
    @property
    def qtq(self) -> I16Vector3: ...
    @property
    def qto(self) -> I16Vector3: ...
    @property
    def qtl(self) -> I16Vector3: ...
    @property
    def qqs(self) -> I16Vector3: ...
    @property
    def qqt(self) -> I16Vector3: ...
    @property
    def qqq(self) -> I16Vector3: ...
    @property
    def qqo(self) -> I16Vector3: ...
    @property
    def qql(self) -> I16Vector3: ...
    @property
    def qos(self) -> I16Vector3: ...
    @property
    def qot(self) -> I16Vector3: ...
    @property
    def qoq(self) -> I16Vector3: ...
    @property
    def qoo(self) -> I16Vector3: ...
    @property
    def qol(self) -> I16Vector3: ...
    @property
    def qls(self) -> I16Vector3: ...
    @property
    def qlt(self) -> I16Vector3: ...
    @property
    def qlq(self) -> I16Vector3: ...
    @property
    def qlo(self) -> I16Vector3: ...
    @property
    def qll(self) -> I16Vector3: ...
    @property
    def oss(self) -> I16Vector3: ...
    @property
    def ost(self) -> I16Vector3: ...
    @property
    def osq(self) -> I16Vector3: ...
    @property
    def oso(self) -> I16Vector3: ...
    @property
    def osl(self) -> I16Vector3: ...
    @property
    def ots(self) -> I16Vector3: ...
    @property
    def ott(self) -> I16Vector3: ...
    @property
    def otq(self) -> I16Vector3: ...
    @property
    def oto(self) -> I16Vector3: ...
    @property
    def otl(self) -> I16Vector3: ...
    @property
    def oqs(self) -> I16Vector3: ...
    @property
    def oqt(self) -> I16Vector3: ...
    @property
    def oqq(self) -> I16Vector3: ...
    @property
    def oqo(self) -> I16Vector3: ...
    @property
    def oql(self) -> I16Vector3: ...
    @property
    def oos(self) -> I16Vector3: ...
    @property
    def oot(self) -> I16Vector3: ...
    @property
    def ooq(self) -> I16Vector3: ...
    @property
    def ols(self) -> I16Vector3: ...
    @property
    def olt(self) -> I16Vector3: ...
    @property
    def olq(self) -> I16Vector3: ...
    @property
    def lss(self) -> I16Vector3: ...
    @property
    def lst(self) -> I16Vector3: ...
    @property
    def lsq(self) -> I16Vector3: ...
    @property
    def lso(self) -> I16Vector3: ...
    @property
    def lsl(self) -> I16Vector3: ...
    @property
    def lts(self) -> I16Vector3: ...
    @property
    def ltt(self) -> I16Vector3: ...
    @property
    def ltq(self) -> I16Vector3: ...
    @property
    def lto(self) -> I16Vector3: ...
    @property
    def ltl(self) -> I16Vector3: ...
    @property
    def lqs(self) -> I16Vector3: ...
    @property
    def lqt(self) -> I16Vector3: ...
    @property
    def lqq(self) -> I16Vector3: ...
    @property
    def lqo(self) -> I16Vector3: ...
    @property
    def lql(self) -> I16Vector3: ...
    @property
    def los(self) -> I16Vector3: ...
    @property
    def lot(self) -> I16Vector3: ...
    @property
    def loq(self) -> I16Vector3: ...
    @property
    def lls(self) -> I16Vector3: ...
    @property
    def llt(self) -> I16Vector3: ...
    @property
    def llq(self) -> I16Vector3: ...
    @property
    def ssss(self) -> I16Vector4: ...
    @property
    def ssst(self) -> I16Vector4: ...
    @property
    def sssq(self) -> I16Vector4: ...
    @property
    def ssso(self) -> I16Vector4: ...
    @property
    def sssl(self) -> I16Vector4: ...
    @property
    def ssts(self) -> I16Vector4: ...
    @property
    def sstt(self) -> I16Vector4: ...
    @property
    def sstq(self) -> I16Vector4: ...
    @property
    def ssto(self) -> I16Vector4: ...
    @property
    def sstl(self) -> I16Vector4: ...
    @property
    def ssqs(self) -> I16Vector4: ...
    @property
    def ssqt(self) -> I16Vector4: ...
    @property
    def ssqq(self) -> I16Vector4: ...
    @property
    def ssqo(self) -> I16Vector4: ...
    @property
    def ssql(self) -> I16Vector4: ...
    @property
    def ssos(self) -> I16Vector4: ...
    @property
    def ssot(self) -> I16Vector4: ...
    @property
    def ssoq(self) -> I16Vector4: ...
    @property
    def ssoo(self) -> I16Vector4: ...
    @property
    def ssol(self) -> I16Vector4: ...
    @property
    def ssls(self) -> I16Vector4: ...
    @property
    def sslt(self) -> I16Vector4: ...
    @property
    def sslq(self) -> I16Vector4: ...
    @property
    def sslo(self) -> I16Vector4: ...
    @property
    def ssll(self) -> I16Vector4: ...
    @property
    def stss(self) -> I16Vector4: ...
    @property
    def stst(self) -> I16Vector4: ...
    @property
    def stsq(self) -> I16Vector4: ...
    @property
    def stso(self) -> I16Vector4: ...
    @property
    def stsl(self) -> I16Vector4: ...
    @property
    def stts(self) -> I16Vector4: ...
    @property
    def sttt(self) -> I16Vector4: ...
    @property
    def sttq(self) -> I16Vector4: ...
    @property
    def stto(self) -> I16Vector4: ...
    @property
    def sttl(self) -> I16Vector4: ...
    @property
    def stqs(self) -> I16Vector4: ...
    @property
    def stqt(self) -> I16Vector4: ...
    @property
    def stqq(self) -> I16Vector4: ...
    @property
    def stqo(self) -> I16Vector4: ...
    @property
    def stql(self) -> I16Vector4: ...
    @property
    def stos(self) -> I16Vector4: ...
    @property
    def stot(self) -> I16Vector4: ...
    @property
    def stoq(self) -> I16Vector4: ...
    @property
    def stoo(self) -> I16Vector4: ...
    @property
    def stol(self) -> I16Vector4: ...
    @property
    def stls(self) -> I16Vector4: ...
    @property
    def stlt(self) -> I16Vector4: ...
    @property
    def stlq(self) -> I16Vector4: ...
    @property
    def stlo(self) -> I16Vector4: ...
    @property
    def stll(self) -> I16Vector4: ...
    @property
    def sqss(self) -> I16Vector4: ...
    @property
    def sqst(self) -> I16Vector4: ...
    @property
    def sqsq(self) -> I16Vector4: ...
    @property
    def sqso(self) -> I16Vector4: ...
    @property
    def sqsl(self) -> I16Vector4: ...
    @property
    def sqts(self) -> I16Vector4: ...
    @property
    def sqtt(self) -> I16Vector4: ...
    @property
    def sqtq(self) -> I16Vector4: ...
    @property
    def sqto(self) -> I16Vector4: ...
    @property
    def sqtl(self) -> I16Vector4: ...
    @property
    def sqqs(self) -> I16Vector4: ...
    @property
    def sqqt(self) -> I16Vector4: ...
    @property
    def sqqq(self) -> I16Vector4: ...
    @property
    def sqqo(self) -> I16Vector4: ...
    @property
    def sqql(self) -> I16Vector4: ...
    @property
    def sqos(self) -> I16Vector4: ...
    @property
    def sqot(self) -> I16Vector4: ...
    @property
    def sqoq(self) -> I16Vector4: ...
    @property
    def sqoo(self) -> I16Vector4: ...
    @property
    def sqol(self) -> I16Vector4: ...
    @property
    def sqls(self) -> I16Vector4: ...
    @property
    def sqlt(self) -> I16Vector4: ...
    @property
    def sqlq(self) -> I16Vector4: ...
    @property
    def sqlo(self) -> I16Vector4: ...
    @property
    def sqll(self) -> I16Vector4: ...
    @property
    def soss(self) -> I16Vector4: ...
    @property
    def sost(self) -> I16Vector4: ...
    @property
    def sosq(self) -> I16Vector4: ...
    @property
    def soso(self) -> I16Vector4: ...
    @property
    def sosl(self) -> I16Vector4: ...
    @property
    def sots(self) -> I16Vector4: ...
    @property
    def sott(self) -> I16Vector4: ...
    @property
    def sotq(self) -> I16Vector4: ...
    @property
    def soto(self) -> I16Vector4: ...
    @property
    def sotl(self) -> I16Vector4: ...
    @property
    def soqs(self) -> I16Vector4: ...
    @property
    def soqt(self) -> I16Vector4: ...
    @property
    def soqq(self) -> I16Vector4: ...
    @property
    def soqo(self) -> I16Vector4: ...
    @property
    def soql(self) -> I16Vector4: ...
    @property
    def soos(self) -> I16Vector4: ...
    @property
    def soot(self) -> I16Vector4: ...
    @property
    def sooq(self) -> I16Vector4: ...
    @property
    def sooo(self) -> I16Vector4: ...
    @property
    def sool(self) -> I16Vector4: ...
    @property
    def sols(self) -> I16Vector4: ...
    @property
    def solt(self) -> I16Vector4: ...
    @property
    def solq(self) -> I16Vector4: ...
    @property
    def solo(self) -> I16Vector4: ...
    @property
    def soll(self) -> I16Vector4: ...
    @property
    def slss(self) -> I16Vector4: ...
    @property
    def slst(self) -> I16Vector4: ...
    @property
    def slsq(self) -> I16Vector4: ...
    @property
    def slso(self) -> I16Vector4: ...
    @property
    def slsl(self) -> I16Vector4: ...
    @property
    def slts(self) -> I16Vector4: ...
    @property
    def sltt(self) -> I16Vector4: ...
    @property
    def sltq(self) -> I16Vector4: ...
    @property
    def slto(self) -> I16Vector4: ...
    @property
    def sltl(self) -> I16Vector4: ...
    @property
    def slqs(self) -> I16Vector4: ...
    @property
    def slqt(self) -> I16Vector4: ...
    @property
    def slqq(self) -> I16Vector4: ...
    @property
    def slqo(self) -> I16Vector4: ...
    @property
    def slql(self) -> I16Vector4: ...
    @property
    def slos(self) -> I16Vector4: ...
    @property
    def slot(self) -> I16Vector4: ...
    @property
    def sloq(self) -> I16Vector4: ...
    @property
    def sloo(self) -> I16Vector4: ...
    @property
    def slol(self) -> I16Vector4: ...
    @property
    def slls(self) -> I16Vector4: ...
    @property
    def sllt(self) -> I16Vector4: ...
    @property
    def sllq(self) -> I16Vector4: ...
    @property
    def sllo(self) -> I16Vector4: ...
    @property
    def slll(self) -> I16Vector4: ...
    @property
    def tsss(self) -> I16Vector4: ...
    @property
    def tsst(self) -> I16Vector4: ...
    @property
    def tssq(self) -> I16Vector4: ...
    @property
    def tsso(self) -> I16Vector4: ...
    @property
    def tssl(self) -> I16Vector4: ...
    @property
    def tsts(self) -> I16Vector4: ...
    @property
    def tstt(self) -> I16Vector4: ...
    @property
    def tstq(self) -> I16Vector4: ...
    @property
    def tsto(self) -> I16Vector4: ...
    @property
    def tstl(self) -> I16Vector4: ...
    @property
    def tsqs(self) -> I16Vector4: ...
    @property
    def tsqt(self) -> I16Vector4: ...
    @property
    def tsqq(self) -> I16Vector4: ...
    @property
    def tsqo(self) -> I16Vector4: ...
    @property
    def tsql(self) -> I16Vector4: ...
    @property
    def tsos(self) -> I16Vector4: ...
    @property
    def tsot(self) -> I16Vector4: ...
    @property
    def tsoq(self) -> I16Vector4: ...
    @property
    def tsoo(self) -> I16Vector4: ...
    @property
    def tsol(self) -> I16Vector4: ...
    @property
    def tsls(self) -> I16Vector4: ...
    @property
    def tslt(self) -> I16Vector4: ...
    @property
    def tslq(self) -> I16Vector4: ...
    @property
    def tslo(self) -> I16Vector4: ...
    @property
    def tsll(self) -> I16Vector4: ...
    @property
    def ttss(self) -> I16Vector4: ...
    @property
    def ttst(self) -> I16Vector4: ...
    @property
    def ttsq(self) -> I16Vector4: ...
    @property
    def ttso(self) -> I16Vector4: ...
    @property
    def ttsl(self) -> I16Vector4: ...
    @property
    def ttts(self) -> I16Vector4: ...
    @property
    def tttt(self) -> I16Vector4: ...
    @property
    def tttq(self) -> I16Vector4: ...
    @property
    def ttto(self) -> I16Vector4: ...
    @property
    def tttl(self) -> I16Vector4: ...
    @property
    def ttqs(self) -> I16Vector4: ...
    @property
    def ttqt(self) -> I16Vector4: ...
    @property
    def ttqq(self) -> I16Vector4: ...
    @property
    def ttqo(self) -> I16Vector4: ...
    @property
    def ttql(self) -> I16Vector4: ...
    @property
    def ttos(self) -> I16Vector4: ...
    @property
    def ttot(self) -> I16Vector4: ...
    @property
    def ttoq(self) -> I16Vector4: ...
    @property
    def ttoo(self) -> I16Vector4: ...
    @property
    def ttol(self) -> I16Vector4: ...
    @property
    def ttls(self) -> I16Vector4: ...
    @property
    def ttlt(self) -> I16Vector4: ...
    @property
    def ttlq(self) -> I16Vector4: ...
    @property
    def ttlo(self) -> I16Vector4: ...
    @property
    def ttll(self) -> I16Vector4: ...
    @property
    def tqss(self) -> I16Vector4: ...
    @property
    def tqst(self) -> I16Vector4: ...
    @property
    def tqsq(self) -> I16Vector4: ...
    @property
    def tqso(self) -> I16Vector4: ...
    @property
    def tqsl(self) -> I16Vector4: ...
    @property
    def tqts(self) -> I16Vector4: ...
    @property
    def tqtt(self) -> I16Vector4: ...
    @property
    def tqtq(self) -> I16Vector4: ...
    @property
    def tqto(self) -> I16Vector4: ...
    @property
    def tqtl(self) -> I16Vector4: ...
    @property
    def tqqs(self) -> I16Vector4: ...
    @property
    def tqqt(self) -> I16Vector4: ...
    @property
    def tqqq(self) -> I16Vector4: ...
    @property
    def tqqo(self) -> I16Vector4: ...
    @property
    def tqql(self) -> I16Vector4: ...
    @property
    def tqos(self) -> I16Vector4: ...
    @property
    def tqot(self) -> I16Vector4: ...
    @property
    def tqoq(self) -> I16Vector4: ...
    @property
    def tqoo(self) -> I16Vector4: ...
    @property
    def tqol(self) -> I16Vector4: ...
    @property
    def tqls(self) -> I16Vector4: ...
    @property
    def tqlt(self) -> I16Vector4: ...
    @property
    def tqlq(self) -> I16Vector4: ...
    @property
    def tqlo(self) -> I16Vector4: ...
    @property
    def tqll(self) -> I16Vector4: ...
    @property
    def toss(self) -> I16Vector4: ...
    @property
    def tost(self) -> I16Vector4: ...
    @property
    def tosq(self) -> I16Vector4: ...
    @property
    def toso(self) -> I16Vector4: ...
    @property
    def tosl(self) -> I16Vector4: ...
    @property
    def tots(self) -> I16Vector4: ...
    @property
    def tott(self) -> I16Vector4: ...
    @property
    def totq(self) -> I16Vector4: ...
    @property
    def toto(self) -> I16Vector4: ...
    @property
    def totl(self) -> I16Vector4: ...
    @property
    def toqs(self) -> I16Vector4: ...
    @property
    def toqt(self) -> I16Vector4: ...
    @property
    def toqq(self) -> I16Vector4: ...
    @property
    def toqo(self) -> I16Vector4: ...
    @property
    def toql(self) -> I16Vector4: ...
    @property
    def toos(self) -> I16Vector4: ...
    @property
    def toot(self) -> I16Vector4: ...
    @property
    def tooq(self) -> I16Vector4: ...
    @property
    def tooo(self) -> I16Vector4: ...
    @property
    def tool(self) -> I16Vector4: ...
    @property
    def tols(self) -> I16Vector4: ...
    @property
    def tolt(self) -> I16Vector4: ...
    @property
    def tolq(self) -> I16Vector4: ...
    @property
    def tolo(self) -> I16Vector4: ...
    @property
    def toll(self) -> I16Vector4: ...
    @property
    def tlss(self) -> I16Vector4: ...
    @property
    def tlst(self) -> I16Vector4: ...
    @property
    def tlsq(self) -> I16Vector4: ...
    @property
    def tlso(self) -> I16Vector4: ...
    @property
    def tlsl(self) -> I16Vector4: ...
    @property
    def tlts(self) -> I16Vector4: ...
    @property
    def tltt(self) -> I16Vector4: ...
    @property
    def tltq(self) -> I16Vector4: ...
    @property
    def tlto(self) -> I16Vector4: ...
    @property
    def tltl(self) -> I16Vector4: ...
    @property
    def tlqs(self) -> I16Vector4: ...
    @property
    def tlqt(self) -> I16Vector4: ...
    @property
    def tlqq(self) -> I16Vector4: ...
    @property
    def tlqo(self) -> I16Vector4: ...
    @property
    def tlql(self) -> I16Vector4: ...
    @property
    def tlos(self) -> I16Vector4: ...
    @property
    def tlot(self) -> I16Vector4: ...
    @property
    def tloq(self) -> I16Vector4: ...
    @property
    def tloo(self) -> I16Vector4: ...
    @property
    def tlol(self) -> I16Vector4: ...
    @property
    def tlls(self) -> I16Vector4: ...
    @property
    def tllt(self) -> I16Vector4: ...
    @property
    def tllq(self) -> I16Vector4: ...
    @property
    def tllo(self) -> I16Vector4: ...
    @property
    def tlll(self) -> I16Vector4: ...
    @property
    def qsss(self) -> I16Vector4: ...
    @property
    def qsst(self) -> I16Vector4: ...
    @property
    def qssq(self) -> I16Vector4: ...
    @property
    def qsso(self) -> I16Vector4: ...
    @property
    def qssl(self) -> I16Vector4: ...
    @property
    def qsts(self) -> I16Vector4: ...
    @property
    def qstt(self) -> I16Vector4: ...
    @property
    def qstq(self) -> I16Vector4: ...
    @property
    def qsto(self) -> I16Vector4: ...
    @property
    def qstl(self) -> I16Vector4: ...
    @property
    def qsqs(self) -> I16Vector4: ...
    @property
    def qsqt(self) -> I16Vector4: ...
    @property
    def qsqq(self) -> I16Vector4: ...
    @property
    def qsqo(self) -> I16Vector4: ...
    @property
    def qsql(self) -> I16Vector4: ...
    @property
    def qsos(self) -> I16Vector4: ...
    @property
    def qsot(self) -> I16Vector4: ...
    @property
    def qsoq(self) -> I16Vector4: ...
    @property
    def qsoo(self) -> I16Vector4: ...
    @property
    def qsol(self) -> I16Vector4: ...
    @property
    def qsls(self) -> I16Vector4: ...
    @property
    def qslt(self) -> I16Vector4: ...
    @property
    def qslq(self) -> I16Vector4: ...
    @property
    def qslo(self) -> I16Vector4: ...
    @property
    def qsll(self) -> I16Vector4: ...
    @property
    def qtss(self) -> I16Vector4: ...
    @property
    def qtst(self) -> I16Vector4: ...
    @property
    def qtsq(self) -> I16Vector4: ...
    @property
    def qtso(self) -> I16Vector4: ...
    @property
    def qtsl(self) -> I16Vector4: ...
    @property
    def qtts(self) -> I16Vector4: ...
    @property
    def qttt(self) -> I16Vector4: ...
    @property
    def qttq(self) -> I16Vector4: ...
    @property
    def qtto(self) -> I16Vector4: ...
    @property
    def qttl(self) -> I16Vector4: ...
    @property
    def qtqs(self) -> I16Vector4: ...
    @property
    def qtqt(self) -> I16Vector4: ...
    @property
    def qtqq(self) -> I16Vector4: ...
    @property
    def qtqo(self) -> I16Vector4: ...
    @property
    def qtql(self) -> I16Vector4: ...
    @property
    def qtos(self) -> I16Vector4: ...
    @property
    def qtot(self) -> I16Vector4: ...
    @property
    def qtoq(self) -> I16Vector4: ...
    @property
    def qtoo(self) -> I16Vector4: ...
    @property
    def qtol(self) -> I16Vector4: ...
    @property
    def qtls(self) -> I16Vector4: ...
    @property
    def qtlt(self) -> I16Vector4: ...
    @property
    def qtlq(self) -> I16Vector4: ...
    @property
    def qtlo(self) -> I16Vector4: ...
    @property
    def qtll(self) -> I16Vector4: ...
    @property
    def qqss(self) -> I16Vector4: ...
    @property
    def qqst(self) -> I16Vector4: ...
    @property
    def qqsq(self) -> I16Vector4: ...
    @property
    def qqso(self) -> I16Vector4: ...
    @property
    def qqsl(self) -> I16Vector4: ...
    @property
    def qqts(self) -> I16Vector4: ...
    @property
    def qqtt(self) -> I16Vector4: ...
    @property
    def qqtq(self) -> I16Vector4: ...
    @property
    def qqto(self) -> I16Vector4: ...
    @property
    def qqtl(self) -> I16Vector4: ...
    @property
    def qqqs(self) -> I16Vector4: ...
    @property
    def qqqt(self) -> I16Vector4: ...
    @property
    def qqqq(self) -> I16Vector4: ...
    @property
    def qqqo(self) -> I16Vector4: ...
    @property
    def qqql(self) -> I16Vector4: ...
    @property
    def qqos(self) -> I16Vector4: ...
    @property
    def qqot(self) -> I16Vector4: ...
    @property
    def qqoq(self) -> I16Vector4: ...
    @property
    def qqoo(self) -> I16Vector4: ...
    @property
    def qqol(self) -> I16Vector4: ...
    @property
    def qqls(self) -> I16Vector4: ...
    @property
    def qqlt(self) -> I16Vector4: ...
    @property
    def qqlq(self) -> I16Vector4: ...
    @property
    def qqlo(self) -> I16Vector4: ...
    @property
    def qqll(self) -> I16Vector4: ...
    @property
    def qoss(self) -> I16Vector4: ...
    @property
    def qost(self) -> I16Vector4: ...
    @property
    def qosq(self) -> I16Vector4: ...
    @property
    def qoso(self) -> I16Vector4: ...
    @property
    def qosl(self) -> I16Vector4: ...
    @property
    def qots(self) -> I16Vector4: ...
    @property
    def qott(self) -> I16Vector4: ...
    @property
    def qotq(self) -> I16Vector4: ...
    @property
    def qoto(self) -> I16Vector4: ...
    @property
    def qotl(self) -> I16Vector4: ...
    @property
    def qoqs(self) -> I16Vector4: ...
    @property
    def qoqt(self) -> I16Vector4: ...
    @property
    def qoqq(self) -> I16Vector4: ...
    @property
    def qoqo(self) -> I16Vector4: ...
    @property
    def qoql(self) -> I16Vector4: ...
    @property
    def qoos(self) -> I16Vector4: ...
    @property
    def qoot(self) -> I16Vector4: ...
    @property
    def qooq(self) -> I16Vector4: ...
    @property
    def qooo(self) -> I16Vector4: ...
    @property
    def qool(self) -> I16Vector4: ...
    @property
    def qols(self) -> I16Vector4: ...
    @property
    def qolt(self) -> I16Vector4: ...
    @property
    def qolq(self) -> I16Vector4: ...
    @property
    def qolo(self) -> I16Vector4: ...
    @property
    def qoll(self) -> I16Vector4: ...
    @property
    def qlss(self) -> I16Vector4: ...
    @property
    def qlst(self) -> I16Vector4: ...
    @property
    def qlsq(self) -> I16Vector4: ...
    @property
    def qlso(self) -> I16Vector4: ...
    @property
    def qlsl(self) -> I16Vector4: ...
    @property
    def qlts(self) -> I16Vector4: ...
    @property
    def qltt(self) -> I16Vector4: ...
    @property
    def qltq(self) -> I16Vector4: ...
    @property
    def qlto(self) -> I16Vector4: ...
    @property
    def qltl(self) -> I16Vector4: ...
    @property
    def qlqs(self) -> I16Vector4: ...
    @property
    def qlqt(self) -> I16Vector4: ...
    @property
    def qlqq(self) -> I16Vector4: ...
    @property
    def qlqo(self) -> I16Vector4: ...
    @property
    def qlql(self) -> I16Vector4: ...
    @property
    def qlos(self) -> I16Vector4: ...
    @property
    def qlot(self) -> I16Vector4: ...
    @property
    def qloq(self) -> I16Vector4: ...
    @property
    def qloo(self) -> I16Vector4: ...
    @property
    def qlol(self) -> I16Vector4: ...
    @property
    def qlls(self) -> I16Vector4: ...
    @property
    def qllt(self) -> I16Vector4: ...
    @property
    def qllq(self) -> I16Vector4: ...
    @property
    def qllo(self) -> I16Vector4: ...
    @property
    def qlll(self) -> I16Vector4: ...
    @property
    def osss(self) -> I16Vector4: ...
    @property
    def osst(self) -> I16Vector4: ...
    @property
    def ossq(self) -> I16Vector4: ...
    @property
    def osso(self) -> I16Vector4: ...
    @property
    def ossl(self) -> I16Vector4: ...
    @property
    def osts(self) -> I16Vector4: ...
    @property
    def ostt(self) -> I16Vector4: ...
    @property
    def ostq(self) -> I16Vector4: ...
    @property
    def osto(self) -> I16Vector4: ...
    @property
    def ostl(self) -> I16Vector4: ...
    @property
    def osqs(self) -> I16Vector4: ...
    @property
    def osqt(self) -> I16Vector4: ...
    @property
    def osqq(self) -> I16Vector4: ...
    @property
    def osqo(self) -> I16Vector4: ...
    @property
    def osql(self) -> I16Vector4: ...
    @property
    def osos(self) -> I16Vector4: ...
    @property
    def osot(self) -> I16Vector4: ...
    @property
    def osoq(self) -> I16Vector4: ...
    @property
    def osoo(self) -> I16Vector4: ...
    @property
    def osol(self) -> I16Vector4: ...
    @property
    def osls(self) -> I16Vector4: ...
    @property
    def oslt(self) -> I16Vector4: ...
    @property
    def oslq(self) -> I16Vector4: ...
    @property
    def oslo(self) -> I16Vector4: ...
    @property
    def osll(self) -> I16Vector4: ...
    @property
    def otss(self) -> I16Vector4: ...
    @property
    def otst(self) -> I16Vector4: ...
    @property
    def otsq(self) -> I16Vector4: ...
    @property
    def otso(self) -> I16Vector4: ...
    @property
    def otsl(self) -> I16Vector4: ...
    @property
    def otts(self) -> I16Vector4: ...
    @property
    def ottt(self) -> I16Vector4: ...
    @property
    def ottq(self) -> I16Vector4: ...
    @property
    def otto(self) -> I16Vector4: ...
    @property
    def ottl(self) -> I16Vector4: ...
    @property
    def otqs(self) -> I16Vector4: ...
    @property
    def otqt(self) -> I16Vector4: ...
    @property
    def otqq(self) -> I16Vector4: ...
    @property
    def otqo(self) -> I16Vector4: ...
    @property
    def otql(self) -> I16Vector4: ...
    @property
    def otos(self) -> I16Vector4: ...
    @property
    def otot(self) -> I16Vector4: ...
    @property
    def otoq(self) -> I16Vector4: ...
    @property
    def otoo(self) -> I16Vector4: ...
    @property
    def otol(self) -> I16Vector4: ...
    @property
    def otls(self) -> I16Vector4: ...
    @property
    def otlt(self) -> I16Vector4: ...
    @property
    def otlq(self) -> I16Vector4: ...
    @property
    def otlo(self) -> I16Vector4: ...
    @property
    def otll(self) -> I16Vector4: ...
    @property
    def oqss(self) -> I16Vector4: ...
    @property
    def oqst(self) -> I16Vector4: ...
    @property
    def oqsq(self) -> I16Vector4: ...
    @property
    def oqso(self) -> I16Vector4: ...
    @property
    def oqsl(self) -> I16Vector4: ...
    @property
    def oqts(self) -> I16Vector4: ...
    @property
    def oqtt(self) -> I16Vector4: ...
    @property
    def oqtq(self) -> I16Vector4: ...
    @property
    def oqto(self) -> I16Vector4: ...
    @property
    def oqtl(self) -> I16Vector4: ...
    @property
    def oqqs(self) -> I16Vector4: ...
    @property
    def oqqt(self) -> I16Vector4: ...
    @property
    def oqqq(self) -> I16Vector4: ...
    @property
    def oqqo(self) -> I16Vector4: ...
    @property
    def oqql(self) -> I16Vector4: ...
    @property
    def oqos(self) -> I16Vector4: ...
    @property
    def oqot(self) -> I16Vector4: ...
    @property
    def oqoq(self) -> I16Vector4: ...
    @property
    def oqoo(self) -> I16Vector4: ...
    @property
    def oqol(self) -> I16Vector4: ...
    @property
    def oqls(self) -> I16Vector4: ...
    @property
    def oqlt(self) -> I16Vector4: ...
    @property
    def oqlq(self) -> I16Vector4: ...
    @property
    def oqlo(self) -> I16Vector4: ...
    @property
    def oqll(self) -> I16Vector4: ...
    @property
    def ooss(self) -> I16Vector4: ...
    @property
    def oost(self) -> I16Vector4: ...
    @property
    def oosq(self) -> I16Vector4: ...
    @property
    def ooso(self) -> I16Vector4: ...
    @property
    def oosl(self) -> I16Vector4: ...
    @property
    def oots(self) -> I16Vector4: ...
    @property
    def oott(self) -> I16Vector4: ...
    @property
    def ootq(self) -> I16Vector4: ...
    @property
    def ooto(self) -> I16Vector4: ...
    @property
    def ootl(self) -> I16Vector4: ...
    @property
    def ooqs(self) -> I16Vector4: ...
    @property
    def ooqt(self) -> I16Vector4: ...
    @property
    def ooqq(self) -> I16Vector4: ...
    @property
    def ooqo(self) -> I16Vector4: ...
    @property
    def ooql(self) -> I16Vector4: ...
    @property
    def ooos(self) -> I16Vector4: ...
    @property
    def ooot(self) -> I16Vector4: ...
    @property
    def oooq(self) -> I16Vector4: ...
    @property
    def ools(self) -> I16Vector4: ...
    @property
    def oolt(self) -> I16Vector4: ...
    @property
    def oolq(self) -> I16Vector4: ...
    @property
    def olss(self) -> I16Vector4: ...
    @property
    def olst(self) -> I16Vector4: ...
    @property
    def olsq(self) -> I16Vector4: ...
    @property
    def olso(self) -> I16Vector4: ...
    @property
    def olsl(self) -> I16Vector4: ...
    @property
    def olts(self) -> I16Vector4: ...
    @property
    def oltt(self) -> I16Vector4: ...
    @property
    def oltq(self) -> I16Vector4: ...
    @property
    def olto(self) -> I16Vector4: ...
    @property
    def oltl(self) -> I16Vector4: ...
    @property
    def olqs(self) -> I16Vector4: ...
    @property
    def olqt(self) -> I16Vector4: ...
    @property
    def olqq(self) -> I16Vector4: ...
    @property
    def olqo(self) -> I16Vector4: ...
    @property
    def olql(self) -> I16Vector4: ...
    @property
    def olos(self) -> I16Vector4: ...
    @property
    def olot(self) -> I16Vector4: ...
    @property
    def oloq(self) -> I16Vector4: ...
    @property
    def olls(self) -> I16Vector4: ...
    @property
    def ollt(self) -> I16Vector4: ...
    @property
    def ollq(self) -> I16Vector4: ...
    @property
    def lsss(self) -> I16Vector4: ...
    @property
    def lsst(self) -> I16Vector4: ...
    @property
    def lssq(self) -> I16Vector4: ...
    @property
    def lsso(self) -> I16Vector4: ...
    @property
    def lssl(self) -> I16Vector4: ...
    @property
    def lsts(self) -> I16Vector4: ...
    @property
    def lstt(self) -> I16Vector4: ...
    @property
    def lstq(self) -> I16Vector4: ...
    @property
    def lsto(self) -> I16Vector4: ...
    @property
    def lstl(self) -> I16Vector4: ...
    @property
    def lsqs(self) -> I16Vector4: ...
    @property
    def lsqt(self) -> I16Vector4: ...
    @property
    def lsqq(self) -> I16Vector4: ...
    @property
    def lsqo(self) -> I16Vector4: ...
    @property
    def lsql(self) -> I16Vector4: ...
    @property
    def lsos(self) -> I16Vector4: ...
    @property
    def lsot(self) -> I16Vector4: ...
    @property
    def lsoq(self) -> I16Vector4: ...
    @property
    def lsoo(self) -> I16Vector4: ...
    @property
    def lsol(self) -> I16Vector4: ...
    @property
    def lsls(self) -> I16Vector4: ...
    @property
    def lslt(self) -> I16Vector4: ...
    @property
    def lslq(self) -> I16Vector4: ...
    @property
    def lslo(self) -> I16Vector4: ...
    @property
    def lsll(self) -> I16Vector4: ...
    @property
    def ltss(self) -> I16Vector4: ...
    @property
    def ltst(self) -> I16Vector4: ...
    @property
    def ltsq(self) -> I16Vector4: ...
    @property
    def ltso(self) -> I16Vector4: ...
    @property
    def ltsl(self) -> I16Vector4: ...
    @property
    def ltts(self) -> I16Vector4: ...
    @property
    def lttt(self) -> I16Vector4: ...
    @property
    def lttq(self) -> I16Vector4: ...
    @property
    def ltto(self) -> I16Vector4: ...
    @property
    def lttl(self) -> I16Vector4: ...
    @property
    def ltqs(self) -> I16Vector4: ...
    @property
    def ltqt(self) -> I16Vector4: ...
    @property
    def ltqq(self) -> I16Vector4: ...
    @property
    def ltqo(self) -> I16Vector4: ...
    @property
    def ltql(self) -> I16Vector4: ...
    @property
    def ltos(self) -> I16Vector4: ...
    @property
    def ltot(self) -> I16Vector4: ...
    @property
    def ltoq(self) -> I16Vector4: ...
    @property
    def ltoo(self) -> I16Vector4: ...
    @property
    def ltol(self) -> I16Vector4: ...
    @property
    def ltls(self) -> I16Vector4: ...
    @property
    def ltlt(self) -> I16Vector4: ...
    @property
    def ltlq(self) -> I16Vector4: ...
    @property
    def ltlo(self) -> I16Vector4: ...
    @property
    def ltll(self) -> I16Vector4: ...
    @property
    def lqss(self) -> I16Vector4: ...
    @property
    def lqst(self) -> I16Vector4: ...
    @property
    def lqsq(self) -> I16Vector4: ...
    @property
    def lqso(self) -> I16Vector4: ...
    @property
    def lqsl(self) -> I16Vector4: ...
    @property
    def lqts(self) -> I16Vector4: ...
    @property
    def lqtt(self) -> I16Vector4: ...
    @property
    def lqtq(self) -> I16Vector4: ...
    @property
    def lqto(self) -> I16Vector4: ...
    @property
    def lqtl(self) -> I16Vector4: ...
    @property
    def lqqs(self) -> I16Vector4: ...
    @property
    def lqqt(self) -> I16Vector4: ...
    @property
    def lqqq(self) -> I16Vector4: ...
    @property
    def lqqo(self) -> I16Vector4: ...
    @property
    def lqql(self) -> I16Vector4: ...
    @property
    def lqos(self) -> I16Vector4: ...
    @property
    def lqot(self) -> I16Vector4: ...
    @property
    def lqoq(self) -> I16Vector4: ...
    @property
    def lqoo(self) -> I16Vector4: ...
    @property
    def lqol(self) -> I16Vector4: ...
    @property
    def lqls(self) -> I16Vector4: ...
    @property
    def lqlt(self) -> I16Vector4: ...
    @property
    def lqlq(self) -> I16Vector4: ...
    @property
    def lqlo(self) -> I16Vector4: ...
    @property
    def lqll(self) -> I16Vector4: ...
    @property
    def loss(self) -> I16Vector4: ...
    @property
    def lost(self) -> I16Vector4: ...
    @property
    def losq(self) -> I16Vector4: ...
    @property
    def loso(self) -> I16Vector4: ...
    @property
    def losl(self) -> I16Vector4: ...
    @property
    def lots(self) -> I16Vector4: ...
    @property
    def lott(self) -> I16Vector4: ...
    @property
    def lotq(self) -> I16Vector4: ...
    @property
    def loto(self) -> I16Vector4: ...
    @property
    def lotl(self) -> I16Vector4: ...
    @property
    def loqs(self) -> I16Vector4: ...
    @property
    def loqt(self) -> I16Vector4: ...
    @property
    def loqq(self) -> I16Vector4: ...
    @property
    def loqo(self) -> I16Vector4: ...
    @property
    def loql(self) -> I16Vector4: ...
    @property
    def loos(self) -> I16Vector4: ...
    @property
    def loot(self) -> I16Vector4: ...
    @property
    def looq(self) -> I16Vector4: ...
    @property
    def lols(self) -> I16Vector4: ...
    @property
    def lolt(self) -> I16Vector4: ...
    @property
    def lolq(self) -> I16Vector4: ...
    @property
    def llss(self) -> I16Vector4: ...
    @property
    def llst(self) -> I16Vector4: ...
    @property
    def llsq(self) -> I16Vector4: ...
    @property
    def llso(self) -> I16Vector4: ...
    @property
    def llsl(self) -> I16Vector4: ...
    @property
    def llts(self) -> I16Vector4: ...
    @property
    def lltt(self) -> I16Vector4: ...
    @property
    def lltq(self) -> I16Vector4: ...
    @property
    def llto(self) -> I16Vector4: ...
    @property
    def lltl(self) -> I16Vector4: ...
    @property
    def llqs(self) -> I16Vector4: ...
    @property
    def llqt(self) -> I16Vector4: ...
    @property
    def llqq(self) -> I16Vector4: ...
    @property
    def llqo(self) -> I16Vector4: ...
    @property
    def llql(self) -> I16Vector4: ...
    @property
    def llos(self) -> I16Vector4: ...
    @property
    def llot(self) -> I16Vector4: ...
    @property
    def lloq(self) -> I16Vector4: ...
    @property
    def llls(self) -> I16Vector4: ...
    @property
    def lllt(self) -> I16Vector4: ...
    @property
    def lllq(self) -> I16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I16Vector2: ...
    @property
    def uv(self) -> I16Vector2: ...
    @property
    def uo(self) -> I16Vector2: ...
    @property
    def ul(self) -> I16Vector2: ...
    @property
    def vu(self) -> I16Vector2: ...
    @property
    def vv(self) -> I16Vector2: ...
    @property
    def vo(self) -> I16Vector2: ...
    @property
    def vl(self) -> I16Vector2: ...
    @property
    def ou(self) -> I16Vector2: ...
    @property
    def ov(self) -> I16Vector2: ...
    @property
    def lu(self) -> I16Vector2: ...
    @property
    def lv(self) -> I16Vector2: ...
    @property
    def uuu(self) -> I16Vector3: ...
    @property
    def uuv(self) -> I16Vector3: ...
    @property
    def uuo(self) -> I16Vector3: ...
    @property
    def uul(self) -> I16Vector3: ...
    @property
    def uvu(self) -> I16Vector3: ...
    @property
    def uvv(self) -> I16Vector3: ...
    @property
    def uvo(self) -> I16Vector3: ...
    @property
    def uvl(self) -> I16Vector3: ...
    @property
    def uou(self) -> I16Vector3: ...
    @property
    def uov(self) -> I16Vector3: ...
    @property
    def uoo(self) -> I16Vector3: ...
    @property
    def uol(self) -> I16Vector3: ...
    @property
    def ulu(self) -> I16Vector3: ...
    @property
    def ulv(self) -> I16Vector3: ...
    @property
    def ulo(self) -> I16Vector3: ...
    @property
    def ull(self) -> I16Vector3: ...
    @property
    def vuu(self) -> I16Vector3: ...
    @property
    def vuv(self) -> I16Vector3: ...
    @property
    def vuo(self) -> I16Vector3: ...
    @property
    def vul(self) -> I16Vector3: ...
    @property
    def vvu(self) -> I16Vector3: ...
    @property
    def vvv(self) -> I16Vector3: ...
    @property
    def vvo(self) -> I16Vector3: ...
    @property
    def vvl(self) -> I16Vector3: ...
    @property
    def vou(self) -> I16Vector3: ...
    @property
    def vov(self) -> I16Vector3: ...
    @property
    def voo(self) -> I16Vector3: ...
    @property
    def vol(self) -> I16Vector3: ...
    @property
    def vlu(self) -> I16Vector3: ...
    @property
    def vlv(self) -> I16Vector3: ...
    @property
    def vlo(self) -> I16Vector3: ...
    @property
    def vll(self) -> I16Vector3: ...
    @property
    def ouu(self) -> I16Vector3: ...
    @property
    def ouv(self) -> I16Vector3: ...
    @property
    def ouo(self) -> I16Vector3: ...
    @property
    def oul(self) -> I16Vector3: ...
    @property
    def ovu(self) -> I16Vector3: ...
    @property
    def ovv(self) -> I16Vector3: ...
    @property
    def ovo(self) -> I16Vector3: ...
    @property
    def ovl(self) -> I16Vector3: ...
    @property
    def oou(self) -> I16Vector3: ...
    @property
    def oov(self) -> I16Vector3: ...
    @property
    def olu(self) -> I16Vector3: ...
    @property
    def olv(self) -> I16Vector3: ...
    @property
    def luu(self) -> I16Vector3: ...
    @property
    def luv(self) -> I16Vector3: ...
    @property
    def luo(self) -> I16Vector3: ...
    @property
    def lul(self) -> I16Vector3: ...
    @property
    def lvu(self) -> I16Vector3: ...
    @property
    def lvv(self) -> I16Vector3: ...
    @property
    def lvo(self) -> I16Vector3: ...
    @property
    def lvl(self) -> I16Vector3: ...
    @property
    def lou(self) -> I16Vector3: ...
    @property
    def lov(self) -> I16Vector3: ...
    @property
    def llu(self) -> I16Vector3: ...
    @property
    def llv(self) -> I16Vector3: ...
    @property
    def uuuu(self) -> I16Vector4: ...
    @property
    def uuuv(self) -> I16Vector4: ...
    @property
    def uuuo(self) -> I16Vector4: ...
    @property
    def uuul(self) -> I16Vector4: ...
    @property
    def uuvu(self) -> I16Vector4: ...
    @property
    def uuvv(self) -> I16Vector4: ...
    @property
    def uuvo(self) -> I16Vector4: ...
    @property
    def uuvl(self) -> I16Vector4: ...
    @property
    def uuou(self) -> I16Vector4: ...
    @property
    def uuov(self) -> I16Vector4: ...
    @property
    def uuoo(self) -> I16Vector4: ...
    @property
    def uuol(self) -> I16Vector4: ...
    @property
    def uulu(self) -> I16Vector4: ...
    @property
    def uulv(self) -> I16Vector4: ...
    @property
    def uulo(self) -> I16Vector4: ...
    @property
    def uull(self) -> I16Vector4: ...
    @property
    def uvuu(self) -> I16Vector4: ...
    @property
    def uvuv(self) -> I16Vector4: ...
    @property
    def uvuo(self) -> I16Vector4: ...
    @property
    def uvul(self) -> I16Vector4: ...
    @property
    def uvvu(self) -> I16Vector4: ...
    @property
    def uvvv(self) -> I16Vector4: ...
    @property
    def uvvo(self) -> I16Vector4: ...
    @property
    def uvvl(self) -> I16Vector4: ...
    @property
    def uvou(self) -> I16Vector4: ...
    @property
    def uvov(self) -> I16Vector4: ...
    @property
    def uvoo(self) -> I16Vector4: ...
    @property
    def uvol(self) -> I16Vector4: ...
    @property
    def uvlu(self) -> I16Vector4: ...
    @property
    def uvlv(self) -> I16Vector4: ...
    @property
    def uvlo(self) -> I16Vector4: ...
    @property
    def uvll(self) -> I16Vector4: ...
    @property
    def uouu(self) -> I16Vector4: ...
    @property
    def uouv(self) -> I16Vector4: ...
    @property
    def uouo(self) -> I16Vector4: ...
    @property
    def uoul(self) -> I16Vector4: ...
    @property
    def uovu(self) -> I16Vector4: ...
    @property
    def uovv(self) -> I16Vector4: ...
    @property
    def uovo(self) -> I16Vector4: ...
    @property
    def uovl(self) -> I16Vector4: ...
    @property
    def uoou(self) -> I16Vector4: ...
    @property
    def uoov(self) -> I16Vector4: ...
    @property
    def uooo(self) -> I16Vector4: ...
    @property
    def uool(self) -> I16Vector4: ...
    @property
    def uolu(self) -> I16Vector4: ...
    @property
    def uolv(self) -> I16Vector4: ...
    @property
    def uolo(self) -> I16Vector4: ...
    @property
    def uoll(self) -> I16Vector4: ...
    @property
    def uluu(self) -> I16Vector4: ...
    @property
    def uluv(self) -> I16Vector4: ...
    @property
    def uluo(self) -> I16Vector4: ...
    @property
    def ulul(self) -> I16Vector4: ...
    @property
    def ulvu(self) -> I16Vector4: ...
    @property
    def ulvv(self) -> I16Vector4: ...
    @property
    def ulvo(self) -> I16Vector4: ...
    @property
    def ulvl(self) -> I16Vector4: ...
    @property
    def ulou(self) -> I16Vector4: ...
    @property
    def ulov(self) -> I16Vector4: ...
    @property
    def uloo(self) -> I16Vector4: ...
    @property
    def ulol(self) -> I16Vector4: ...
    @property
    def ullu(self) -> I16Vector4: ...
    @property
    def ullv(self) -> I16Vector4: ...
    @property
    def ullo(self) -> I16Vector4: ...
    @property
    def ulll(self) -> I16Vector4: ...
    @property
    def vuuu(self) -> I16Vector4: ...
    @property
    def vuuv(self) -> I16Vector4: ...
    @property
    def vuuo(self) -> I16Vector4: ...
    @property
    def vuul(self) -> I16Vector4: ...
    @property
    def vuvu(self) -> I16Vector4: ...
    @property
    def vuvv(self) -> I16Vector4: ...
    @property
    def vuvo(self) -> I16Vector4: ...
    @property
    def vuvl(self) -> I16Vector4: ...
    @property
    def vuou(self) -> I16Vector4: ...
    @property
    def vuov(self) -> I16Vector4: ...
    @property
    def vuoo(self) -> I16Vector4: ...
    @property
    def vuol(self) -> I16Vector4: ...
    @property
    def vulu(self) -> I16Vector4: ...
    @property
    def vulv(self) -> I16Vector4: ...
    @property
    def vulo(self) -> I16Vector4: ...
    @property
    def vull(self) -> I16Vector4: ...
    @property
    def vvuu(self) -> I16Vector4: ...
    @property
    def vvuv(self) -> I16Vector4: ...
    @property
    def vvuo(self) -> I16Vector4: ...
    @property
    def vvul(self) -> I16Vector4: ...
    @property
    def vvvu(self) -> I16Vector4: ...
    @property
    def vvvv(self) -> I16Vector4: ...
    @property
    def vvvo(self) -> I16Vector4: ...
    @property
    def vvvl(self) -> I16Vector4: ...
    @property
    def vvou(self) -> I16Vector4: ...
    @property
    def vvov(self) -> I16Vector4: ...
    @property
    def vvoo(self) -> I16Vector4: ...
    @property
    def vvol(self) -> I16Vector4: ...
    @property
    def vvlu(self) -> I16Vector4: ...
    @property
    def vvlv(self) -> I16Vector4: ...
    @property
    def vvlo(self) -> I16Vector4: ...
    @property
    def vvll(self) -> I16Vector4: ...
    @property
    def vouu(self) -> I16Vector4: ...
    @property
    def vouv(self) -> I16Vector4: ...
    @property
    def vouo(self) -> I16Vector4: ...
    @property
    def voul(self) -> I16Vector4: ...
    @property
    def vovu(self) -> I16Vector4: ...
    @property
    def vovv(self) -> I16Vector4: ...
    @property
    def vovo(self) -> I16Vector4: ...
    @property
    def vovl(self) -> I16Vector4: ...
    @property
    def voou(self) -> I16Vector4: ...
    @property
    def voov(self) -> I16Vector4: ...
    @property
    def vooo(self) -> I16Vector4: ...
    @property
    def vool(self) -> I16Vector4: ...
    @property
    def volu(self) -> I16Vector4: ...
    @property
    def volv(self) -> I16Vector4: ...
    @property
    def volo(self) -> I16Vector4: ...
    @property
    def voll(self) -> I16Vector4: ...
    @property
    def vluu(self) -> I16Vector4: ...
    @property
    def vluv(self) -> I16Vector4: ...
    @property
    def vluo(self) -> I16Vector4: ...
    @property
    def vlul(self) -> I16Vector4: ...
    @property
    def vlvu(self) -> I16Vector4: ...
    @property
    def vlvv(self) -> I16Vector4: ...
    @property
    def vlvo(self) -> I16Vector4: ...
    @property
    def vlvl(self) -> I16Vector4: ...
    @property
    def vlou(self) -> I16Vector4: ...
    @property
    def vlov(self) -> I16Vector4: ...
    @property
    def vloo(self) -> I16Vector4: ...
    @property
    def vlol(self) -> I16Vector4: ...
    @property
    def vllu(self) -> I16Vector4: ...
    @property
    def vllv(self) -> I16Vector4: ...
    @property
    def vllo(self) -> I16Vector4: ...
    @property
    def vlll(self) -> I16Vector4: ...
    @property
    def ouuu(self) -> I16Vector4: ...
    @property
    def ouuv(self) -> I16Vector4: ...
    @property
    def ouuo(self) -> I16Vector4: ...
    @property
    def ouul(self) -> I16Vector4: ...
    @property
    def ouvu(self) -> I16Vector4: ...
    @property
    def ouvv(self) -> I16Vector4: ...
    @property
    def ouvo(self) -> I16Vector4: ...
    @property
    def ouvl(self) -> I16Vector4: ...
    @property
    def ouou(self) -> I16Vector4: ...
    @property
    def ouov(self) -> I16Vector4: ...
    @property
    def ouoo(self) -> I16Vector4: ...
    @property
    def ouol(self) -> I16Vector4: ...
    @property
    def oulu(self) -> I16Vector4: ...
    @property
    def oulv(self) -> I16Vector4: ...
    @property
    def oulo(self) -> I16Vector4: ...
    @property
    def oull(self) -> I16Vector4: ...
    @property
    def ovuu(self) -> I16Vector4: ...
    @property
    def ovuv(self) -> I16Vector4: ...
    @property
    def ovuo(self) -> I16Vector4: ...
    @property
    def ovul(self) -> I16Vector4: ...
    @property
    def ovvu(self) -> I16Vector4: ...
    @property
    def ovvv(self) -> I16Vector4: ...
    @property
    def ovvo(self) -> I16Vector4: ...
    @property
    def ovvl(self) -> I16Vector4: ...
    @property
    def ovou(self) -> I16Vector4: ...
    @property
    def ovov(self) -> I16Vector4: ...
    @property
    def ovoo(self) -> I16Vector4: ...
    @property
    def ovol(self) -> I16Vector4: ...
    @property
    def ovlu(self) -> I16Vector4: ...
    @property
    def ovlv(self) -> I16Vector4: ...
    @property
    def ovlo(self) -> I16Vector4: ...
    @property
    def ovll(self) -> I16Vector4: ...
    @property
    def oouu(self) -> I16Vector4: ...
    @property
    def oouv(self) -> I16Vector4: ...
    @property
    def oouo(self) -> I16Vector4: ...
    @property
    def ooul(self) -> I16Vector4: ...
    @property
    def oovu(self) -> I16Vector4: ...
    @property
    def oovv(self) -> I16Vector4: ...
    @property
    def oovo(self) -> I16Vector4: ...
    @property
    def oovl(self) -> I16Vector4: ...
    @property
    def ooou(self) -> I16Vector4: ...
    @property
    def ooov(self) -> I16Vector4: ...
    @property
    def oolu(self) -> I16Vector4: ...
    @property
    def oolv(self) -> I16Vector4: ...
    @property
    def oluu(self) -> I16Vector4: ...
    @property
    def oluv(self) -> I16Vector4: ...
    @property
    def oluo(self) -> I16Vector4: ...
    @property
    def olul(self) -> I16Vector4: ...
    @property
    def olvu(self) -> I16Vector4: ...
    @property
    def olvv(self) -> I16Vector4: ...
    @property
    def olvo(self) -> I16Vector4: ...
    @property
    def olvl(self) -> I16Vector4: ...
    @property
    def olou(self) -> I16Vector4: ...
    @property
    def olov(self) -> I16Vector4: ...
    @property
    def ollu(self) -> I16Vector4: ...
    @property
    def ollv(self) -> I16Vector4: ...
    @property
    def luuu(self) -> I16Vector4: ...
    @property
    def luuv(self) -> I16Vector4: ...
    @property
    def luuo(self) -> I16Vector4: ...
    @property
    def luul(self) -> I16Vector4: ...
    @property
    def luvu(self) -> I16Vector4: ...
    @property
    def luvv(self) -> I16Vector4: ...
    @property
    def luvo(self) -> I16Vector4: ...
    @property
    def luvl(self) -> I16Vector4: ...
    @property
    def luou(self) -> I16Vector4: ...
    @property
    def luov(self) -> I16Vector4: ...
    @property
    def luoo(self) -> I16Vector4: ...
    @property
    def luol(self) -> I16Vector4: ...
    @property
    def lulu(self) -> I16Vector4: ...
    @property
    def lulv(self) -> I16Vector4: ...
    @property
    def lulo(self) -> I16Vector4: ...
    @property
    def lull(self) -> I16Vector4: ...
    @property
    def lvuu(self) -> I16Vector4: ...
    @property
    def lvuv(self) -> I16Vector4: ...
    @property
    def lvuo(self) -> I16Vector4: ...
    @property
    def lvul(self) -> I16Vector4: ...
    @property
    def lvvu(self) -> I16Vector4: ...
    @property
    def lvvv(self) -> I16Vector4: ...
    @property
    def lvvo(self) -> I16Vector4: ...
    @property
    def lvvl(self) -> I16Vector4: ...
    @property
    def lvou(self) -> I16Vector4: ...
    @property
    def lvov(self) -> I16Vector4: ...
    @property
    def lvoo(self) -> I16Vector4: ...
    @property
    def lvol(self) -> I16Vector4: ...
    @property
    def lvlu(self) -> I16Vector4: ...
    @property
    def lvlv(self) -> I16Vector4: ...
    @property
    def lvlo(self) -> I16Vector4: ...
    @property
    def lvll(self) -> I16Vector4: ...
    @property
    def louu(self) -> I16Vector4: ...
    @property
    def louv(self) -> I16Vector4: ...
    @property
    def louo(self) -> I16Vector4: ...
    @property
    def loul(self) -> I16Vector4: ...
    @property
    def lovu(self) -> I16Vector4: ...
    @property
    def lovv(self) -> I16Vector4: ...
    @property
    def lovo(self) -> I16Vector4: ...
    @property
    def lovl(self) -> I16Vector4: ...
    @property
    def loou(self) -> I16Vector4: ...
    @property
    def loov(self) -> I16Vector4: ...
    @property
    def lolu(self) -> I16Vector4: ...
    @property
    def lolv(self) -> I16Vector4: ...
    @property
    def lluu(self) -> I16Vector4: ...
    @property
    def lluv(self) -> I16Vector4: ...
    @property
    def lluo(self) -> I16Vector4: ...
    @property
    def llul(self) -> I16Vector4: ...
    @property
    def llvu(self) -> I16Vector4: ...
    @property
    def llvv(self) -> I16Vector4: ...
    @property
    def llvo(self) -> I16Vector4: ...
    @property
    def llvl(self) -> I16Vector4: ...
    @property
    def llou(self) -> I16Vector4: ...
    @property
    def llov(self) -> I16Vector4: ...
    @property
    def lllu(self) -> I16Vector4: ...
    @property
    def lllv(self) -> I16Vector4: ...
    @property
    def oo(self) -> I16Vector2: ...
    @property
    def ol(self) -> I16Vector2: ...
    @property
    def lo(self) -> I16Vector2: ...
    @property
    def ll(self) -> I16Vector2: ...
    @property
    def ooo(self) -> I16Vector3: ...
    @property
    def ool(self) -> I16Vector3: ...
    @property
    def olo(self) -> I16Vector3: ...
    @property
    def oll(self) -> I16Vector3: ...
    @property
    def loo(self) -> I16Vector3: ...
    @property
    def lol(self) -> I16Vector3: ...
    @property
    def llo(self) -> I16Vector3: ...
    @property
    def lll(self) -> I16Vector3: ...
    @property
    def oooo(self) -> I16Vector4: ...
    @property
    def oool(self) -> I16Vector4: ...
    @property
    def oolo(self) -> I16Vector4: ...
    @property
    def ooll(self) -> I16Vector4: ...
    @property
    def oloo(self) -> I16Vector4: ...
    @property
    def olol(self) -> I16Vector4: ...
    @property
    def ollo(self) -> I16Vector4: ...
    @property
    def olll(self) -> I16Vector4: ...
    @property
    def looo(self) -> I16Vector4: ...
    @property
    def lool(self) -> I16Vector4: ...
    @property
    def lolo(self) -> I16Vector4: ...
    @property
    def loll(self) -> I16Vector4: ...
    @property
    def lloo(self) -> I16Vector4: ...
    @property
    def llol(self) -> I16Vector4: ...
    @property
    def lllo(self) -> I16Vector4: ...
    @property
    def llll(self) -> I16Vector4: ...
    def min(self, n: Number, /) -> I16Vector3: ...
    def max(self, n: Number, /) -> I16Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> I16Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I16Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector3: ...

@final
class I16Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I16Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I16Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> I16Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> I16Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[I16Vector3]: ...

@final
class U16Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __add__(self, other: Number) -> U16Vector3: ...
    @overload
    def __radd__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __radd__(self, other: Number) -> U16Vector3: ...
    @overload
    def __sub__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __sub__(self, other: Number) -> U16Vector3: ...
    @overload
    def __rsub__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> U16Vector3: ...
    @overload
    def __mul__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __mul__(self, other: Number) -> U16Vector3: ...
    @overload
    def __rmul__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> U16Vector3: ...
    @overload
    def __truediv__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> U16Vector3: ...
    @overload
    def __rtruediv__(self, other: U16Vector3) -> U16Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> U16Vector3: ...
    def __abs__(self) -> U16Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> U16Vector2: ...
    @property
    def xy(self) -> U16Vector2: ...
    @property
    def xz(self) -> U16Vector2: ...
    @property
    def xo(self) -> U16Vector2: ...
    @property
    def xl(self) -> U16Vector2: ...
    @property
    def yx(self) -> U16Vector2: ...
    @property
    def yy(self) -> U16Vector2: ...
    @property
    def yz(self) -> U16Vector2: ...
    @property
    def yo(self) -> U16Vector2: ...
    @property
    def yl(self) -> U16Vector2: ...
    @property
    def zx(self) -> U16Vector2: ...
    @property
    def zy(self) -> U16Vector2: ...
    @property
    def zz(self) -> U16Vector2: ...
    @property
    def zo(self) -> U16Vector2: ...
    @property
    def zl(self) -> U16Vector2: ...
    @property
    def ox(self) -> U16Vector2: ...
    @property
    def oy(self) -> U16Vector2: ...
    @property
    def oz(self) -> U16Vector2: ...
    @property
    def lx(self) -> U16Vector2: ...
    @property
    def ly(self) -> U16Vector2: ...
    @property
    def lz(self) -> U16Vector2: ...
    @property
    def xxx(self) -> U16Vector3: ...
    @property
    def xxy(self) -> U16Vector3: ...
    @property
    def xxz(self) -> U16Vector3: ...
    @property
    def xxo(self) -> U16Vector3: ...
    @property
    def xxl(self) -> U16Vector3: ...
    @property
    def xyx(self) -> U16Vector3: ...
    @property
    def xyy(self) -> U16Vector3: ...
    @property
    def xyz(self) -> U16Vector3: ...
    @property
    def xyo(self) -> U16Vector3: ...
    @property
    def xyl(self) -> U16Vector3: ...
    @property
    def xzx(self) -> U16Vector3: ...
    @property
    def xzy(self) -> U16Vector3: ...
    @property
    def xzz(self) -> U16Vector3: ...
    @property
    def xzo(self) -> U16Vector3: ...
    @property
    def xzl(self) -> U16Vector3: ...
    @property
    def xox(self) -> U16Vector3: ...
    @property
    def xoy(self) -> U16Vector3: ...
    @property
    def xoz(self) -> U16Vector3: ...
    @property
    def xoo(self) -> U16Vector3: ...
    @property
    def xol(self) -> U16Vector3: ...
    @property
    def xlx(self) -> U16Vector3: ...
    @property
    def xly(self) -> U16Vector3: ...
    @property
    def xlz(self) -> U16Vector3: ...
    @property
    def xlo(self) -> U16Vector3: ...
    @property
    def xll(self) -> U16Vector3: ...
    @property
    def yxx(self) -> U16Vector3: ...
    @property
    def yxy(self) -> U16Vector3: ...
    @property
    def yxz(self) -> U16Vector3: ...
    @property
    def yxo(self) -> U16Vector3: ...
    @property
    def yxl(self) -> U16Vector3: ...
    @property
    def yyx(self) -> U16Vector3: ...
    @property
    def yyy(self) -> U16Vector3: ...
    @property
    def yyz(self) -> U16Vector3: ...
    @property
    def yyo(self) -> U16Vector3: ...
    @property
    def yyl(self) -> U16Vector3: ...
    @property
    def yzx(self) -> U16Vector3: ...
    @property
    def yzy(self) -> U16Vector3: ...
    @property
    def yzz(self) -> U16Vector3: ...
    @property
    def yzo(self) -> U16Vector3: ...
    @property
    def yzl(self) -> U16Vector3: ...
    @property
    def yox(self) -> U16Vector3: ...
    @property
    def yoy(self) -> U16Vector3: ...
    @property
    def yoz(self) -> U16Vector3: ...
    @property
    def yoo(self) -> U16Vector3: ...
    @property
    def yol(self) -> U16Vector3: ...
    @property
    def ylx(self) -> U16Vector3: ...
    @property
    def yly(self) -> U16Vector3: ...
    @property
    def ylz(self) -> U16Vector3: ...
    @property
    def ylo(self) -> U16Vector3: ...
    @property
    def yll(self) -> U16Vector3: ...
    @property
    def zxx(self) -> U16Vector3: ...
    @property
    def zxy(self) -> U16Vector3: ...
    @property
    def zxz(self) -> U16Vector3: ...
    @property
    def zxo(self) -> U16Vector3: ...
    @property
    def zxl(self) -> U16Vector3: ...
    @property
    def zyx(self) -> U16Vector3: ...
    @property
    def zyy(self) -> U16Vector3: ...
    @property
    def zyz(self) -> U16Vector3: ...
    @property
    def zyo(self) -> U16Vector3: ...
    @property
    def zyl(self) -> U16Vector3: ...
    @property
    def zzx(self) -> U16Vector3: ...
    @property
    def zzy(self) -> U16Vector3: ...
    @property
    def zzz(self) -> U16Vector3: ...
    @property
    def zzo(self) -> U16Vector3: ...
    @property
    def zzl(self) -> U16Vector3: ...
    @property
    def zox(self) -> U16Vector3: ...
    @property
    def zoy(self) -> U16Vector3: ...
    @property
    def zoz(self) -> U16Vector3: ...
    @property
    def zoo(self) -> U16Vector3: ...
    @property
    def zol(self) -> U16Vector3: ...
    @property
    def zlx(self) -> U16Vector3: ...
    @property
    def zly(self) -> U16Vector3: ...
    @property
    def zlz(self) -> U16Vector3: ...
    @property
    def zlo(self) -> U16Vector3: ...
    @property
    def zll(self) -> U16Vector3: ...
    @property
    def oxx(self) -> U16Vector3: ...
    @property
    def oxy(self) -> U16Vector3: ...
    @property
    def oxz(self) -> U16Vector3: ...
    @property
    def oxo(self) -> U16Vector3: ...
    @property
    def oxl(self) -> U16Vector3: ...
    @property
    def oyx(self) -> U16Vector3: ...
    @property
    def oyy(self) -> U16Vector3: ...
    @property
    def oyz(self) -> U16Vector3: ...
    @property
    def oyo(self) -> U16Vector3: ...
    @property
    def oyl(self) -> U16Vector3: ...
    @property
    def ozx(self) -> U16Vector3: ...
    @property
    def ozy(self) -> U16Vector3: ...
    @property
    def ozz(self) -> U16Vector3: ...
    @property
    def ozo(self) -> U16Vector3: ...
    @property
    def ozl(self) -> U16Vector3: ...
    @property
    def oox(self) -> U16Vector3: ...
    @property
    def ooy(self) -> U16Vector3: ...
    @property
    def ooz(self) -> U16Vector3: ...
    @property
    def olx(self) -> U16Vector3: ...
    @property
    def oly(self) -> U16Vector3: ...
    @property
    def olz(self) -> U16Vector3: ...
    @property
    def lxx(self) -> U16Vector3: ...
    @property
    def lxy(self) -> U16Vector3: ...
    @property
    def lxz(self) -> U16Vector3: ...
    @property
    def lxo(self) -> U16Vector3: ...
    @property
    def lxl(self) -> U16Vector3: ...
    @property
    def lyx(self) -> U16Vector3: ...
    @property
    def lyy(self) -> U16Vector3: ...
    @property
    def lyz(self) -> U16Vector3: ...
    @property
    def lyo(self) -> U16Vector3: ...
    @property
    def lyl(self) -> U16Vector3: ...
    @property
    def lzx(self) -> U16Vector3: ...
    @property
    def lzy(self) -> U16Vector3: ...
    @property
    def lzz(self) -> U16Vector3: ...
    @property
    def lzo(self) -> U16Vector3: ...
    @property
    def lzl(self) -> U16Vector3: ...
    @property
    def lox(self) -> U16Vector3: ...
    @property
    def loy(self) -> U16Vector3: ...
    @property
    def loz(self) -> U16Vector3: ...
    @property
    def llx(self) -> U16Vector3: ...
    @property
    def lly(self) -> U16Vector3: ...
    @property
    def llz(self) -> U16Vector3: ...
    @property
    def xxxx(self) -> U16Vector4: ...
    @property
    def xxxy(self) -> U16Vector4: ...
    @property
    def xxxz(self) -> U16Vector4: ...
    @property
    def xxxo(self) -> U16Vector4: ...
    @property
    def xxxl(self) -> U16Vector4: ...
    @property
    def xxyx(self) -> U16Vector4: ...
    @property
    def xxyy(self) -> U16Vector4: ...
    @property
    def xxyz(self) -> U16Vector4: ...
    @property
    def xxyo(self) -> U16Vector4: ...
    @property
    def xxyl(self) -> U16Vector4: ...
    @property
    def xxzx(self) -> U16Vector4: ...
    @property
    def xxzy(self) -> U16Vector4: ...
    @property
    def xxzz(self) -> U16Vector4: ...
    @property
    def xxzo(self) -> U16Vector4: ...
    @property
    def xxzl(self) -> U16Vector4: ...
    @property
    def xxox(self) -> U16Vector4: ...
    @property
    def xxoy(self) -> U16Vector4: ...
    @property
    def xxoz(self) -> U16Vector4: ...
    @property
    def xxoo(self) -> U16Vector4: ...
    @property
    def xxol(self) -> U16Vector4: ...
    @property
    def xxlx(self) -> U16Vector4: ...
    @property
    def xxly(self) -> U16Vector4: ...
    @property
    def xxlz(self) -> U16Vector4: ...
    @property
    def xxlo(self) -> U16Vector4: ...
    @property
    def xxll(self) -> U16Vector4: ...
    @property
    def xyxx(self) -> U16Vector4: ...
    @property
    def xyxy(self) -> U16Vector4: ...
    @property
    def xyxz(self) -> U16Vector4: ...
    @property
    def xyxo(self) -> U16Vector4: ...
    @property
    def xyxl(self) -> U16Vector4: ...
    @property
    def xyyx(self) -> U16Vector4: ...
    @property
    def xyyy(self) -> U16Vector4: ...
    @property
    def xyyz(self) -> U16Vector4: ...
    @property
    def xyyo(self) -> U16Vector4: ...
    @property
    def xyyl(self) -> U16Vector4: ...
    @property
    def xyzx(self) -> U16Vector4: ...
    @property
    def xyzy(self) -> U16Vector4: ...
    @property
    def xyzz(self) -> U16Vector4: ...
    @property
    def xyzo(self) -> U16Vector4: ...
    @property
    def xyzl(self) -> U16Vector4: ...
    @property
    def xyox(self) -> U16Vector4: ...
    @property
    def xyoy(self) -> U16Vector4: ...
    @property
    def xyoz(self) -> U16Vector4: ...
    @property
    def xyoo(self) -> U16Vector4: ...
    @property
    def xyol(self) -> U16Vector4: ...
    @property
    def xylx(self) -> U16Vector4: ...
    @property
    def xyly(self) -> U16Vector4: ...
    @property
    def xylz(self) -> U16Vector4: ...
    @property
    def xylo(self) -> U16Vector4: ...
    @property
    def xyll(self) -> U16Vector4: ...
    @property
    def xzxx(self) -> U16Vector4: ...
    @property
    def xzxy(self) -> U16Vector4: ...
    @property
    def xzxz(self) -> U16Vector4: ...
    @property
    def xzxo(self) -> U16Vector4: ...
    @property
    def xzxl(self) -> U16Vector4: ...
    @property
    def xzyx(self) -> U16Vector4: ...
    @property
    def xzyy(self) -> U16Vector4: ...
    @property
    def xzyz(self) -> U16Vector4: ...
    @property
    def xzyo(self) -> U16Vector4: ...
    @property
    def xzyl(self) -> U16Vector4: ...
    @property
    def xzzx(self) -> U16Vector4: ...
    @property
    def xzzy(self) -> U16Vector4: ...
    @property
    def xzzz(self) -> U16Vector4: ...
    @property
    def xzzo(self) -> U16Vector4: ...
    @property
    def xzzl(self) -> U16Vector4: ...
    @property
    def xzox(self) -> U16Vector4: ...
    @property
    def xzoy(self) -> U16Vector4: ...
    @property
    def xzoz(self) -> U16Vector4: ...
    @property
    def xzoo(self) -> U16Vector4: ...
    @property
    def xzol(self) -> U16Vector4: ...
    @property
    def xzlx(self) -> U16Vector4: ...
    @property
    def xzly(self) -> U16Vector4: ...
    @property
    def xzlz(self) -> U16Vector4: ...
    @property
    def xzlo(self) -> U16Vector4: ...
    @property
    def xzll(self) -> U16Vector4: ...
    @property
    def xoxx(self) -> U16Vector4: ...
    @property
    def xoxy(self) -> U16Vector4: ...
    @property
    def xoxz(self) -> U16Vector4: ...
    @property
    def xoxo(self) -> U16Vector4: ...
    @property
    def xoxl(self) -> U16Vector4: ...
    @property
    def xoyx(self) -> U16Vector4: ...
    @property
    def xoyy(self) -> U16Vector4: ...
    @property
    def xoyz(self) -> U16Vector4: ...
    @property
    def xoyo(self) -> U16Vector4: ...
    @property
    def xoyl(self) -> U16Vector4: ...
    @property
    def xozx(self) -> U16Vector4: ...
    @property
    def xozy(self) -> U16Vector4: ...
    @property
    def xozz(self) -> U16Vector4: ...
    @property
    def xozo(self) -> U16Vector4: ...
    @property
    def xozl(self) -> U16Vector4: ...
    @property
    def xoox(self) -> U16Vector4: ...
    @property
    def xooy(self) -> U16Vector4: ...
    @property
    def xooz(self) -> U16Vector4: ...
    @property
    def xooo(self) -> U16Vector4: ...
    @property
    def xool(self) -> U16Vector4: ...
    @property
    def xolx(self) -> U16Vector4: ...
    @property
    def xoly(self) -> U16Vector4: ...
    @property
    def xolz(self) -> U16Vector4: ...
    @property
    def xolo(self) -> U16Vector4: ...
    @property
    def xoll(self) -> U16Vector4: ...
    @property
    def xlxx(self) -> U16Vector4: ...
    @property
    def xlxy(self) -> U16Vector4: ...
    @property
    def xlxz(self) -> U16Vector4: ...
    @property
    def xlxo(self) -> U16Vector4: ...
    @property
    def xlxl(self) -> U16Vector4: ...
    @property
    def xlyx(self) -> U16Vector4: ...
    @property
    def xlyy(self) -> U16Vector4: ...
    @property
    def xlyz(self) -> U16Vector4: ...
    @property
    def xlyo(self) -> U16Vector4: ...
    @property
    def xlyl(self) -> U16Vector4: ...
    @property
    def xlzx(self) -> U16Vector4: ...
    @property
    def xlzy(self) -> U16Vector4: ...
    @property
    def xlzz(self) -> U16Vector4: ...
    @property
    def xlzo(self) -> U16Vector4: ...
    @property
    def xlzl(self) -> U16Vector4: ...
    @property
    def xlox(self) -> U16Vector4: ...
    @property
    def xloy(self) -> U16Vector4: ...
    @property
    def xloz(self) -> U16Vector4: ...
    @property
    def xloo(self) -> U16Vector4: ...
    @property
    def xlol(self) -> U16Vector4: ...
    @property
    def xllx(self) -> U16Vector4: ...
    @property
    def xlly(self) -> U16Vector4: ...
    @property
    def xllz(self) -> U16Vector4: ...
    @property
    def xllo(self) -> U16Vector4: ...
    @property
    def xlll(self) -> U16Vector4: ...
    @property
    def yxxx(self) -> U16Vector4: ...
    @property
    def yxxy(self) -> U16Vector4: ...
    @property
    def yxxz(self) -> U16Vector4: ...
    @property
    def yxxo(self) -> U16Vector4: ...
    @property
    def yxxl(self) -> U16Vector4: ...
    @property
    def yxyx(self) -> U16Vector4: ...
    @property
    def yxyy(self) -> U16Vector4: ...
    @property
    def yxyz(self) -> U16Vector4: ...
    @property
    def yxyo(self) -> U16Vector4: ...
    @property
    def yxyl(self) -> U16Vector4: ...
    @property
    def yxzx(self) -> U16Vector4: ...
    @property
    def yxzy(self) -> U16Vector4: ...
    @property
    def yxzz(self) -> U16Vector4: ...
    @property
    def yxzo(self) -> U16Vector4: ...
    @property
    def yxzl(self) -> U16Vector4: ...
    @property
    def yxox(self) -> U16Vector4: ...
    @property
    def yxoy(self) -> U16Vector4: ...
    @property
    def yxoz(self) -> U16Vector4: ...
    @property
    def yxoo(self) -> U16Vector4: ...
    @property
    def yxol(self) -> U16Vector4: ...
    @property
    def yxlx(self) -> U16Vector4: ...
    @property
    def yxly(self) -> U16Vector4: ...
    @property
    def yxlz(self) -> U16Vector4: ...
    @property
    def yxlo(self) -> U16Vector4: ...
    @property
    def yxll(self) -> U16Vector4: ...
    @property
    def yyxx(self) -> U16Vector4: ...
    @property
    def yyxy(self) -> U16Vector4: ...
    @property
    def yyxz(self) -> U16Vector4: ...
    @property
    def yyxo(self) -> U16Vector4: ...
    @property
    def yyxl(self) -> U16Vector4: ...
    @property
    def yyyx(self) -> U16Vector4: ...
    @property
    def yyyy(self) -> U16Vector4: ...
    @property
    def yyyz(self) -> U16Vector4: ...
    @property
    def yyyo(self) -> U16Vector4: ...
    @property
    def yyyl(self) -> U16Vector4: ...
    @property
    def yyzx(self) -> U16Vector4: ...
    @property
    def yyzy(self) -> U16Vector4: ...
    @property
    def yyzz(self) -> U16Vector4: ...
    @property
    def yyzo(self) -> U16Vector4: ...
    @property
    def yyzl(self) -> U16Vector4: ...
    @property
    def yyox(self) -> U16Vector4: ...
    @property
    def yyoy(self) -> U16Vector4: ...
    @property
    def yyoz(self) -> U16Vector4: ...
    @property
    def yyoo(self) -> U16Vector4: ...
    @property
    def yyol(self) -> U16Vector4: ...
    @property
    def yylx(self) -> U16Vector4: ...
    @property
    def yyly(self) -> U16Vector4: ...
    @property
    def yylz(self) -> U16Vector4: ...
    @property
    def yylo(self) -> U16Vector4: ...
    @property
    def yyll(self) -> U16Vector4: ...
    @property
    def yzxx(self) -> U16Vector4: ...
    @property
    def yzxy(self) -> U16Vector4: ...
    @property
    def yzxz(self) -> U16Vector4: ...
    @property
    def yzxo(self) -> U16Vector4: ...
    @property
    def yzxl(self) -> U16Vector4: ...
    @property
    def yzyx(self) -> U16Vector4: ...
    @property
    def yzyy(self) -> U16Vector4: ...
    @property
    def yzyz(self) -> U16Vector4: ...
    @property
    def yzyo(self) -> U16Vector4: ...
    @property
    def yzyl(self) -> U16Vector4: ...
    @property
    def yzzx(self) -> U16Vector4: ...
    @property
    def yzzy(self) -> U16Vector4: ...
    @property
    def yzzz(self) -> U16Vector4: ...
    @property
    def yzzo(self) -> U16Vector4: ...
    @property
    def yzzl(self) -> U16Vector4: ...
    @property
    def yzox(self) -> U16Vector4: ...
    @property
    def yzoy(self) -> U16Vector4: ...
    @property
    def yzoz(self) -> U16Vector4: ...
    @property
    def yzoo(self) -> U16Vector4: ...
    @property
    def yzol(self) -> U16Vector4: ...
    @property
    def yzlx(self) -> U16Vector4: ...
    @property
    def yzly(self) -> U16Vector4: ...
    @property
    def yzlz(self) -> U16Vector4: ...
    @property
    def yzlo(self) -> U16Vector4: ...
    @property
    def yzll(self) -> U16Vector4: ...
    @property
    def yoxx(self) -> U16Vector4: ...
    @property
    def yoxy(self) -> U16Vector4: ...
    @property
    def yoxz(self) -> U16Vector4: ...
    @property
    def yoxo(self) -> U16Vector4: ...
    @property
    def yoxl(self) -> U16Vector4: ...
    @property
    def yoyx(self) -> U16Vector4: ...
    @property
    def yoyy(self) -> U16Vector4: ...
    @property
    def yoyz(self) -> U16Vector4: ...
    @property
    def yoyo(self) -> U16Vector4: ...
    @property
    def yoyl(self) -> U16Vector4: ...
    @property
    def yozx(self) -> U16Vector4: ...
    @property
    def yozy(self) -> U16Vector4: ...
    @property
    def yozz(self) -> U16Vector4: ...
    @property
    def yozo(self) -> U16Vector4: ...
    @property
    def yozl(self) -> U16Vector4: ...
    @property
    def yoox(self) -> U16Vector4: ...
    @property
    def yooy(self) -> U16Vector4: ...
    @property
    def yooz(self) -> U16Vector4: ...
    @property
    def yooo(self) -> U16Vector4: ...
    @property
    def yool(self) -> U16Vector4: ...
    @property
    def yolx(self) -> U16Vector4: ...
    @property
    def yoly(self) -> U16Vector4: ...
    @property
    def yolz(self) -> U16Vector4: ...
    @property
    def yolo(self) -> U16Vector4: ...
    @property
    def yoll(self) -> U16Vector4: ...
    @property
    def ylxx(self) -> U16Vector4: ...
    @property
    def ylxy(self) -> U16Vector4: ...
    @property
    def ylxz(self) -> U16Vector4: ...
    @property
    def ylxo(self) -> U16Vector4: ...
    @property
    def ylxl(self) -> U16Vector4: ...
    @property
    def ylyx(self) -> U16Vector4: ...
    @property
    def ylyy(self) -> U16Vector4: ...
    @property
    def ylyz(self) -> U16Vector4: ...
    @property
    def ylyo(self) -> U16Vector4: ...
    @property
    def ylyl(self) -> U16Vector4: ...
    @property
    def ylzx(self) -> U16Vector4: ...
    @property
    def ylzy(self) -> U16Vector4: ...
    @property
    def ylzz(self) -> U16Vector4: ...
    @property
    def ylzo(self) -> U16Vector4: ...
    @property
    def ylzl(self) -> U16Vector4: ...
    @property
    def ylox(self) -> U16Vector4: ...
    @property
    def yloy(self) -> U16Vector4: ...
    @property
    def yloz(self) -> U16Vector4: ...
    @property
    def yloo(self) -> U16Vector4: ...
    @property
    def ylol(self) -> U16Vector4: ...
    @property
    def yllx(self) -> U16Vector4: ...
    @property
    def ylly(self) -> U16Vector4: ...
    @property
    def yllz(self) -> U16Vector4: ...
    @property
    def yllo(self) -> U16Vector4: ...
    @property
    def ylll(self) -> U16Vector4: ...
    @property
    def zxxx(self) -> U16Vector4: ...
    @property
    def zxxy(self) -> U16Vector4: ...
    @property
    def zxxz(self) -> U16Vector4: ...
    @property
    def zxxo(self) -> U16Vector4: ...
    @property
    def zxxl(self) -> U16Vector4: ...
    @property
    def zxyx(self) -> U16Vector4: ...
    @property
    def zxyy(self) -> U16Vector4: ...
    @property
    def zxyz(self) -> U16Vector4: ...
    @property
    def zxyo(self) -> U16Vector4: ...
    @property
    def zxyl(self) -> U16Vector4: ...
    @property
    def zxzx(self) -> U16Vector4: ...
    @property
    def zxzy(self) -> U16Vector4: ...
    @property
    def zxzz(self) -> U16Vector4: ...
    @property
    def zxzo(self) -> U16Vector4: ...
    @property
    def zxzl(self) -> U16Vector4: ...
    @property
    def zxox(self) -> U16Vector4: ...
    @property
    def zxoy(self) -> U16Vector4: ...
    @property
    def zxoz(self) -> U16Vector4: ...
    @property
    def zxoo(self) -> U16Vector4: ...
    @property
    def zxol(self) -> U16Vector4: ...
    @property
    def zxlx(self) -> U16Vector4: ...
    @property
    def zxly(self) -> U16Vector4: ...
    @property
    def zxlz(self) -> U16Vector4: ...
    @property
    def zxlo(self) -> U16Vector4: ...
    @property
    def zxll(self) -> U16Vector4: ...
    @property
    def zyxx(self) -> U16Vector4: ...
    @property
    def zyxy(self) -> U16Vector4: ...
    @property
    def zyxz(self) -> U16Vector4: ...
    @property
    def zyxo(self) -> U16Vector4: ...
    @property
    def zyxl(self) -> U16Vector4: ...
    @property
    def zyyx(self) -> U16Vector4: ...
    @property
    def zyyy(self) -> U16Vector4: ...
    @property
    def zyyz(self) -> U16Vector4: ...
    @property
    def zyyo(self) -> U16Vector4: ...
    @property
    def zyyl(self) -> U16Vector4: ...
    @property
    def zyzx(self) -> U16Vector4: ...
    @property
    def zyzy(self) -> U16Vector4: ...
    @property
    def zyzz(self) -> U16Vector4: ...
    @property
    def zyzo(self) -> U16Vector4: ...
    @property
    def zyzl(self) -> U16Vector4: ...
    @property
    def zyox(self) -> U16Vector4: ...
    @property
    def zyoy(self) -> U16Vector4: ...
    @property
    def zyoz(self) -> U16Vector4: ...
    @property
    def zyoo(self) -> U16Vector4: ...
    @property
    def zyol(self) -> U16Vector4: ...
    @property
    def zylx(self) -> U16Vector4: ...
    @property
    def zyly(self) -> U16Vector4: ...
    @property
    def zylz(self) -> U16Vector4: ...
    @property
    def zylo(self) -> U16Vector4: ...
    @property
    def zyll(self) -> U16Vector4: ...
    @property
    def zzxx(self) -> U16Vector4: ...
    @property
    def zzxy(self) -> U16Vector4: ...
    @property
    def zzxz(self) -> U16Vector4: ...
    @property
    def zzxo(self) -> U16Vector4: ...
    @property
    def zzxl(self) -> U16Vector4: ...
    @property
    def zzyx(self) -> U16Vector4: ...
    @property
    def zzyy(self) -> U16Vector4: ...
    @property
    def zzyz(self) -> U16Vector4: ...
    @property
    def zzyo(self) -> U16Vector4: ...
    @property
    def zzyl(self) -> U16Vector4: ...
    @property
    def zzzx(self) -> U16Vector4: ...
    @property
    def zzzy(self) -> U16Vector4: ...
    @property
    def zzzz(self) -> U16Vector4: ...
    @property
    def zzzo(self) -> U16Vector4: ...
    @property
    def zzzl(self) -> U16Vector4: ...
    @property
    def zzox(self) -> U16Vector4: ...
    @property
    def zzoy(self) -> U16Vector4: ...
    @property
    def zzoz(self) -> U16Vector4: ...
    @property
    def zzoo(self) -> U16Vector4: ...
    @property
    def zzol(self) -> U16Vector4: ...
    @property
    def zzlx(self) -> U16Vector4: ...
    @property
    def zzly(self) -> U16Vector4: ...
    @property
    def zzlz(self) -> U16Vector4: ...
    @property
    def zzlo(self) -> U16Vector4: ...
    @property
    def zzll(self) -> U16Vector4: ...
    @property
    def zoxx(self) -> U16Vector4: ...
    @property
    def zoxy(self) -> U16Vector4: ...
    @property
    def zoxz(self) -> U16Vector4: ...
    @property
    def zoxo(self) -> U16Vector4: ...
    @property
    def zoxl(self) -> U16Vector4: ...
    @property
    def zoyx(self) -> U16Vector4: ...
    @property
    def zoyy(self) -> U16Vector4: ...
    @property
    def zoyz(self) -> U16Vector4: ...
    @property
    def zoyo(self) -> U16Vector4: ...
    @property
    def zoyl(self) -> U16Vector4: ...
    @property
    def zozx(self) -> U16Vector4: ...
    @property
    def zozy(self) -> U16Vector4: ...
    @property
    def zozz(self) -> U16Vector4: ...
    @property
    def zozo(self) -> U16Vector4: ...
    @property
    def zozl(self) -> U16Vector4: ...
    @property
    def zoox(self) -> U16Vector4: ...
    @property
    def zooy(self) -> U16Vector4: ...
    @property
    def zooz(self) -> U16Vector4: ...
    @property
    def zooo(self) -> U16Vector4: ...
    @property
    def zool(self) -> U16Vector4: ...
    @property
    def zolx(self) -> U16Vector4: ...
    @property
    def zoly(self) -> U16Vector4: ...
    @property
    def zolz(self) -> U16Vector4: ...
    @property
    def zolo(self) -> U16Vector4: ...
    @property
    def zoll(self) -> U16Vector4: ...
    @property
    def zlxx(self) -> U16Vector4: ...
    @property
    def zlxy(self) -> U16Vector4: ...
    @property
    def zlxz(self) -> U16Vector4: ...
    @property
    def zlxo(self) -> U16Vector4: ...
    @property
    def zlxl(self) -> U16Vector4: ...
    @property
    def zlyx(self) -> U16Vector4: ...
    @property
    def zlyy(self) -> U16Vector4: ...
    @property
    def zlyz(self) -> U16Vector4: ...
    @property
    def zlyo(self) -> U16Vector4: ...
    @property
    def zlyl(self) -> U16Vector4: ...
    @property
    def zlzx(self) -> U16Vector4: ...
    @property
    def zlzy(self) -> U16Vector4: ...
    @property
    def zlzz(self) -> U16Vector4: ...
    @property
    def zlzo(self) -> U16Vector4: ...
    @property
    def zlzl(self) -> U16Vector4: ...
    @property
    def zlox(self) -> U16Vector4: ...
    @property
    def zloy(self) -> U16Vector4: ...
    @property
    def zloz(self) -> U16Vector4: ...
    @property
    def zloo(self) -> U16Vector4: ...
    @property
    def zlol(self) -> U16Vector4: ...
    @property
    def zllx(self) -> U16Vector4: ...
    @property
    def zlly(self) -> U16Vector4: ...
    @property
    def zllz(self) -> U16Vector4: ...
    @property
    def zllo(self) -> U16Vector4: ...
    @property
    def zlll(self) -> U16Vector4: ...
    @property
    def oxxx(self) -> U16Vector4: ...
    @property
    def oxxy(self) -> U16Vector4: ...
    @property
    def oxxz(self) -> U16Vector4: ...
    @property
    def oxxo(self) -> U16Vector4: ...
    @property
    def oxxl(self) -> U16Vector4: ...
    @property
    def oxyx(self) -> U16Vector4: ...
    @property
    def oxyy(self) -> U16Vector4: ...
    @property
    def oxyz(self) -> U16Vector4: ...
    @property
    def oxyo(self) -> U16Vector4: ...
    @property
    def oxyl(self) -> U16Vector4: ...
    @property
    def oxzx(self) -> U16Vector4: ...
    @property
    def oxzy(self) -> U16Vector4: ...
    @property
    def oxzz(self) -> U16Vector4: ...
    @property
    def oxzo(self) -> U16Vector4: ...
    @property
    def oxzl(self) -> U16Vector4: ...
    @property
    def oxox(self) -> U16Vector4: ...
    @property
    def oxoy(self) -> U16Vector4: ...
    @property
    def oxoz(self) -> U16Vector4: ...
    @property
    def oxoo(self) -> U16Vector4: ...
    @property
    def oxol(self) -> U16Vector4: ...
    @property
    def oxlx(self) -> U16Vector4: ...
    @property
    def oxly(self) -> U16Vector4: ...
    @property
    def oxlz(self) -> U16Vector4: ...
    @property
    def oxlo(self) -> U16Vector4: ...
    @property
    def oxll(self) -> U16Vector4: ...
    @property
    def oyxx(self) -> U16Vector4: ...
    @property
    def oyxy(self) -> U16Vector4: ...
    @property
    def oyxz(self) -> U16Vector4: ...
    @property
    def oyxo(self) -> U16Vector4: ...
    @property
    def oyxl(self) -> U16Vector4: ...
    @property
    def oyyx(self) -> U16Vector4: ...
    @property
    def oyyy(self) -> U16Vector4: ...
    @property
    def oyyz(self) -> U16Vector4: ...
    @property
    def oyyo(self) -> U16Vector4: ...
    @property
    def oyyl(self) -> U16Vector4: ...
    @property
    def oyzx(self) -> U16Vector4: ...
    @property
    def oyzy(self) -> U16Vector4: ...
    @property
    def oyzz(self) -> U16Vector4: ...
    @property
    def oyzo(self) -> U16Vector4: ...
    @property
    def oyzl(self) -> U16Vector4: ...
    @property
    def oyox(self) -> U16Vector4: ...
    @property
    def oyoy(self) -> U16Vector4: ...
    @property
    def oyoz(self) -> U16Vector4: ...
    @property
    def oyoo(self) -> U16Vector4: ...
    @property
    def oyol(self) -> U16Vector4: ...
    @property
    def oylx(self) -> U16Vector4: ...
    @property
    def oyly(self) -> U16Vector4: ...
    @property
    def oylz(self) -> U16Vector4: ...
    @property
    def oylo(self) -> U16Vector4: ...
    @property
    def oyll(self) -> U16Vector4: ...
    @property
    def ozxx(self) -> U16Vector4: ...
    @property
    def ozxy(self) -> U16Vector4: ...
    @property
    def ozxz(self) -> U16Vector4: ...
    @property
    def ozxo(self) -> U16Vector4: ...
    @property
    def ozxl(self) -> U16Vector4: ...
    @property
    def ozyx(self) -> U16Vector4: ...
    @property
    def ozyy(self) -> U16Vector4: ...
    @property
    def ozyz(self) -> U16Vector4: ...
    @property
    def ozyo(self) -> U16Vector4: ...
    @property
    def ozyl(self) -> U16Vector4: ...
    @property
    def ozzx(self) -> U16Vector4: ...
    @property
    def ozzy(self) -> U16Vector4: ...
    @property
    def ozzz(self) -> U16Vector4: ...
    @property
    def ozzo(self) -> U16Vector4: ...
    @property
    def ozzl(self) -> U16Vector4: ...
    @property
    def ozox(self) -> U16Vector4: ...
    @property
    def ozoy(self) -> U16Vector4: ...
    @property
    def ozoz(self) -> U16Vector4: ...
    @property
    def ozoo(self) -> U16Vector4: ...
    @property
    def ozol(self) -> U16Vector4: ...
    @property
    def ozlx(self) -> U16Vector4: ...
    @property
    def ozly(self) -> U16Vector4: ...
    @property
    def ozlz(self) -> U16Vector4: ...
    @property
    def ozlo(self) -> U16Vector4: ...
    @property
    def ozll(self) -> U16Vector4: ...
    @property
    def ooxx(self) -> U16Vector4: ...
    @property
    def ooxy(self) -> U16Vector4: ...
    @property
    def ooxz(self) -> U16Vector4: ...
    @property
    def ooxo(self) -> U16Vector4: ...
    @property
    def ooxl(self) -> U16Vector4: ...
    @property
    def ooyx(self) -> U16Vector4: ...
    @property
    def ooyy(self) -> U16Vector4: ...
    @property
    def ooyz(self) -> U16Vector4: ...
    @property
    def ooyo(self) -> U16Vector4: ...
    @property
    def ooyl(self) -> U16Vector4: ...
    @property
    def oozx(self) -> U16Vector4: ...
    @property
    def oozy(self) -> U16Vector4: ...
    @property
    def oozz(self) -> U16Vector4: ...
    @property
    def oozo(self) -> U16Vector4: ...
    @property
    def oozl(self) -> U16Vector4: ...
    @property
    def ooox(self) -> U16Vector4: ...
    @property
    def oooy(self) -> U16Vector4: ...
    @property
    def oooz(self) -> U16Vector4: ...
    @property
    def oolx(self) -> U16Vector4: ...
    @property
    def ooly(self) -> U16Vector4: ...
    @property
    def oolz(self) -> U16Vector4: ...
    @property
    def olxx(self) -> U16Vector4: ...
    @property
    def olxy(self) -> U16Vector4: ...
    @property
    def olxz(self) -> U16Vector4: ...
    @property
    def olxo(self) -> U16Vector4: ...
    @property
    def olxl(self) -> U16Vector4: ...
    @property
    def olyx(self) -> U16Vector4: ...
    @property
    def olyy(self) -> U16Vector4: ...
    @property
    def olyz(self) -> U16Vector4: ...
    @property
    def olyo(self) -> U16Vector4: ...
    @property
    def olyl(self) -> U16Vector4: ...
    @property
    def olzx(self) -> U16Vector4: ...
    @property
    def olzy(self) -> U16Vector4: ...
    @property
    def olzz(self) -> U16Vector4: ...
    @property
    def olzo(self) -> U16Vector4: ...
    @property
    def olzl(self) -> U16Vector4: ...
    @property
    def olox(self) -> U16Vector4: ...
    @property
    def oloy(self) -> U16Vector4: ...
    @property
    def oloz(self) -> U16Vector4: ...
    @property
    def ollx(self) -> U16Vector4: ...
    @property
    def olly(self) -> U16Vector4: ...
    @property
    def ollz(self) -> U16Vector4: ...
    @property
    def lxxx(self) -> U16Vector4: ...
    @property
    def lxxy(self) -> U16Vector4: ...
    @property
    def lxxz(self) -> U16Vector4: ...
    @property
    def lxxo(self) -> U16Vector4: ...
    @property
    def lxxl(self) -> U16Vector4: ...
    @property
    def lxyx(self) -> U16Vector4: ...
    @property
    def lxyy(self) -> U16Vector4: ...
    @property
    def lxyz(self) -> U16Vector4: ...
    @property
    def lxyo(self) -> U16Vector4: ...
    @property
    def lxyl(self) -> U16Vector4: ...
    @property
    def lxzx(self) -> U16Vector4: ...
    @property
    def lxzy(self) -> U16Vector4: ...
    @property
    def lxzz(self) -> U16Vector4: ...
    @property
    def lxzo(self) -> U16Vector4: ...
    @property
    def lxzl(self) -> U16Vector4: ...
    @property
    def lxox(self) -> U16Vector4: ...
    @property
    def lxoy(self) -> U16Vector4: ...
    @property
    def lxoz(self) -> U16Vector4: ...
    @property
    def lxoo(self) -> U16Vector4: ...
    @property
    def lxol(self) -> U16Vector4: ...
    @property
    def lxlx(self) -> U16Vector4: ...
    @property
    def lxly(self) -> U16Vector4: ...
    @property
    def lxlz(self) -> U16Vector4: ...
    @property
    def lxlo(self) -> U16Vector4: ...
    @property
    def lxll(self) -> U16Vector4: ...
    @property
    def lyxx(self) -> U16Vector4: ...
    @property
    def lyxy(self) -> U16Vector4: ...
    @property
    def lyxz(self) -> U16Vector4: ...
    @property
    def lyxo(self) -> U16Vector4: ...
    @property
    def lyxl(self) -> U16Vector4: ...
    @property
    def lyyx(self) -> U16Vector4: ...
    @property
    def lyyy(self) -> U16Vector4: ...
    @property
    def lyyz(self) -> U16Vector4: ...
    @property
    def lyyo(self) -> U16Vector4: ...
    @property
    def lyyl(self) -> U16Vector4: ...
    @property
    def lyzx(self) -> U16Vector4: ...
    @property
    def lyzy(self) -> U16Vector4: ...
    @property
    def lyzz(self) -> U16Vector4: ...
    @property
    def lyzo(self) -> U16Vector4: ...
    @property
    def lyzl(self) -> U16Vector4: ...
    @property
    def lyox(self) -> U16Vector4: ...
    @property
    def lyoy(self) -> U16Vector4: ...
    @property
    def lyoz(self) -> U16Vector4: ...
    @property
    def lyoo(self) -> U16Vector4: ...
    @property
    def lyol(self) -> U16Vector4: ...
    @property
    def lylx(self) -> U16Vector4: ...
    @property
    def lyly(self) -> U16Vector4: ...
    @property
    def lylz(self) -> U16Vector4: ...
    @property
    def lylo(self) -> U16Vector4: ...
    @property
    def lyll(self) -> U16Vector4: ...
    @property
    def lzxx(self) -> U16Vector4: ...
    @property
    def lzxy(self) -> U16Vector4: ...
    @property
    def lzxz(self) -> U16Vector4: ...
    @property
    def lzxo(self) -> U16Vector4: ...
    @property
    def lzxl(self) -> U16Vector4: ...
    @property
    def lzyx(self) -> U16Vector4: ...
    @property
    def lzyy(self) -> U16Vector4: ...
    @property
    def lzyz(self) -> U16Vector4: ...
    @property
    def lzyo(self) -> U16Vector4: ...
    @property
    def lzyl(self) -> U16Vector4: ...
    @property
    def lzzx(self) -> U16Vector4: ...
    @property
    def lzzy(self) -> U16Vector4: ...
    @property
    def lzzz(self) -> U16Vector4: ...
    @property
    def lzzo(self) -> U16Vector4: ...
    @property
    def lzzl(self) -> U16Vector4: ...
    @property
    def lzox(self) -> U16Vector4: ...
    @property
    def lzoy(self) -> U16Vector4: ...
    @property
    def lzoz(self) -> U16Vector4: ...
    @property
    def lzoo(self) -> U16Vector4: ...
    @property
    def lzol(self) -> U16Vector4: ...
    @property
    def lzlx(self) -> U16Vector4: ...
    @property
    def lzly(self) -> U16Vector4: ...
    @property
    def lzlz(self) -> U16Vector4: ...
    @property
    def lzlo(self) -> U16Vector4: ...
    @property
    def lzll(self) -> U16Vector4: ...
    @property
    def loxx(self) -> U16Vector4: ...
    @property
    def loxy(self) -> U16Vector4: ...
    @property
    def loxz(self) -> U16Vector4: ...
    @property
    def loxo(self) -> U16Vector4: ...
    @property
    def loxl(self) -> U16Vector4: ...
    @property
    def loyx(self) -> U16Vector4: ...
    @property
    def loyy(self) -> U16Vector4: ...
    @property
    def loyz(self) -> U16Vector4: ...
    @property
    def loyo(self) -> U16Vector4: ...
    @property
    def loyl(self) -> U16Vector4: ...
    @property
    def lozx(self) -> U16Vector4: ...
    @property
    def lozy(self) -> U16Vector4: ...
    @property
    def lozz(self) -> U16Vector4: ...
    @property
    def lozo(self) -> U16Vector4: ...
    @property
    def lozl(self) -> U16Vector4: ...
    @property
    def loox(self) -> U16Vector4: ...
    @property
    def looy(self) -> U16Vector4: ...
    @property
    def looz(self) -> U16Vector4: ...
    @property
    def lolx(self) -> U16Vector4: ...
    @property
    def loly(self) -> U16Vector4: ...
    @property
    def lolz(self) -> U16Vector4: ...
    @property
    def llxx(self) -> U16Vector4: ...
    @property
    def llxy(self) -> U16Vector4: ...
    @property
    def llxz(self) -> U16Vector4: ...
    @property
    def llxo(self) -> U16Vector4: ...
    @property
    def llxl(self) -> U16Vector4: ...
    @property
    def llyx(self) -> U16Vector4: ...
    @property
    def llyy(self) -> U16Vector4: ...
    @property
    def llyz(self) -> U16Vector4: ...
    @property
    def llyo(self) -> U16Vector4: ...
    @property
    def llyl(self) -> U16Vector4: ...
    @property
    def llzx(self) -> U16Vector4: ...
    @property
    def llzy(self) -> U16Vector4: ...
    @property
    def llzz(self) -> U16Vector4: ...
    @property
    def llzo(self) -> U16Vector4: ...
    @property
    def llzl(self) -> U16Vector4: ...
    @property
    def llox(self) -> U16Vector4: ...
    @property
    def lloy(self) -> U16Vector4: ...
    @property
    def lloz(self) -> U16Vector4: ...
    @property
    def lllx(self) -> U16Vector4: ...
    @property
    def llly(self) -> U16Vector4: ...
    @property
    def lllz(self) -> U16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> U16Vector2: ...
    @property
    def rg(self) -> U16Vector2: ...
    @property
    def rb(self) -> U16Vector2: ...
    @property
    def ro(self) -> U16Vector2: ...
    @property
    def rl(self) -> U16Vector2: ...
    @property
    def gr(self) -> U16Vector2: ...
    @property
    def gg(self) -> U16Vector2: ...
    @property
    def gb(self) -> U16Vector2: ...
    @property
    def go(self) -> U16Vector2: ...
    @property
    def gl(self) -> U16Vector2: ...
    @property
    def br(self) -> U16Vector2: ...
    @property
    def bg(self) -> U16Vector2: ...
    @property
    def bb(self) -> U16Vector2: ...
    @property
    def bo(self) -> U16Vector2: ...
    @property
    def bl(self) -> U16Vector2: ...
    @property
    def og(self) -> U16Vector2: ...
    @property
    def ob(self) -> U16Vector2: ...
    @property
    def lr(self) -> U16Vector2: ...
    @property
    def lg(self) -> U16Vector2: ...
    @property
    def lb(self) -> U16Vector2: ...
    @property
    def rrr(self) -> U16Vector3: ...
    @property
    def rrg(self) -> U16Vector3: ...
    @property
    def rrb(self) -> U16Vector3: ...
    @property
    def rro(self) -> U16Vector3: ...
    @property
    def rrl(self) -> U16Vector3: ...
    @property
    def rgr(self) -> U16Vector3: ...
    @property
    def rgg(self) -> U16Vector3: ...
    @property
    def rgb(self) -> U16Vector3: ...
    @property
    def rgo(self) -> U16Vector3: ...
    @property
    def rgl(self) -> U16Vector3: ...
    @property
    def rbr(self) -> U16Vector3: ...
    @property
    def rbg(self) -> U16Vector3: ...
    @property
    def rbb(self) -> U16Vector3: ...
    @property
    def rbo(self) -> U16Vector3: ...
    @property
    def rbl(self) -> U16Vector3: ...
    @property
    def ror(self) -> U16Vector3: ...
    @property
    def rog(self) -> U16Vector3: ...
    @property
    def rob(self) -> U16Vector3: ...
    @property
    def roo(self) -> U16Vector3: ...
    @property
    def rol(self) -> U16Vector3: ...
    @property
    def rlr(self) -> U16Vector3: ...
    @property
    def rlg(self) -> U16Vector3: ...
    @property
    def rlb(self) -> U16Vector3: ...
    @property
    def rlo(self) -> U16Vector3: ...
    @property
    def rll(self) -> U16Vector3: ...
    @property
    def grr(self) -> U16Vector3: ...
    @property
    def grg(self) -> U16Vector3: ...
    @property
    def grb(self) -> U16Vector3: ...
    @property
    def gro(self) -> U16Vector3: ...
    @property
    def grl(self) -> U16Vector3: ...
    @property
    def ggr(self) -> U16Vector3: ...
    @property
    def ggg(self) -> U16Vector3: ...
    @property
    def ggb(self) -> U16Vector3: ...
    @property
    def ggo(self) -> U16Vector3: ...
    @property
    def ggl(self) -> U16Vector3: ...
    @property
    def gbr(self) -> U16Vector3: ...
    @property
    def gbg(self) -> U16Vector3: ...
    @property
    def gbb(self) -> U16Vector3: ...
    @property
    def gbo(self) -> U16Vector3: ...
    @property
    def gbl(self) -> U16Vector3: ...
    @property
    def gor(self) -> U16Vector3: ...
    @property
    def gog(self) -> U16Vector3: ...
    @property
    def gob(self) -> U16Vector3: ...
    @property
    def goo(self) -> U16Vector3: ...
    @property
    def gol(self) -> U16Vector3: ...
    @property
    def glr(self) -> U16Vector3: ...
    @property
    def glg(self) -> U16Vector3: ...
    @property
    def glb(self) -> U16Vector3: ...
    @property
    def glo(self) -> U16Vector3: ...
    @property
    def gll(self) -> U16Vector3: ...
    @property
    def brr(self) -> U16Vector3: ...
    @property
    def brg(self) -> U16Vector3: ...
    @property
    def brb(self) -> U16Vector3: ...
    @property
    def bro(self) -> U16Vector3: ...
    @property
    def brl(self) -> U16Vector3: ...
    @property
    def bgr(self) -> U16Vector3: ...
    @property
    def bgg(self) -> U16Vector3: ...
    @property
    def bgb(self) -> U16Vector3: ...
    @property
    def bgo(self) -> U16Vector3: ...
    @property
    def bgl(self) -> U16Vector3: ...
    @property
    def bbr(self) -> U16Vector3: ...
    @property
    def bbg(self) -> U16Vector3: ...
    @property
    def bbb(self) -> U16Vector3: ...
    @property
    def bbo(self) -> U16Vector3: ...
    @property
    def bbl(self) -> U16Vector3: ...
    @property
    def bor(self) -> U16Vector3: ...
    @property
    def bog(self) -> U16Vector3: ...
    @property
    def bob(self) -> U16Vector3: ...
    @property
    def boo(self) -> U16Vector3: ...
    @property
    def bol(self) -> U16Vector3: ...
    @property
    def blr(self) -> U16Vector3: ...
    @property
    def blg(self) -> U16Vector3: ...
    @property
    def blb(self) -> U16Vector3: ...
    @property
    def blo(self) -> U16Vector3: ...
    @property
    def bll(self) -> U16Vector3: ...
    @property
    def orr(self) -> U16Vector3: ...
    @property
    def org(self) -> U16Vector3: ...
    @property
    def orb(self) -> U16Vector3: ...
    @property
    def oro(self) -> U16Vector3: ...
    @property
    def orl(self) -> U16Vector3: ...
    @property
    def ogr(self) -> U16Vector3: ...
    @property
    def ogg(self) -> U16Vector3: ...
    @property
    def ogb(self) -> U16Vector3: ...
    @property
    def ogo(self) -> U16Vector3: ...
    @property
    def ogl(self) -> U16Vector3: ...
    @property
    def obr(self) -> U16Vector3: ...
    @property
    def obg(self) -> U16Vector3: ...
    @property
    def obb(self) -> U16Vector3: ...
    @property
    def obo(self) -> U16Vector3: ...
    @property
    def obl(self) -> U16Vector3: ...
    @property
    def oor(self) -> U16Vector3: ...
    @property
    def oog(self) -> U16Vector3: ...
    @property
    def oob(self) -> U16Vector3: ...
    @property
    def olr(self) -> U16Vector3: ...
    @property
    def olg(self) -> U16Vector3: ...
    @property
    def olb(self) -> U16Vector3: ...
    @property
    def lrr(self) -> U16Vector3: ...
    @property
    def lrg(self) -> U16Vector3: ...
    @property
    def lrb(self) -> U16Vector3: ...
    @property
    def lro(self) -> U16Vector3: ...
    @property
    def lrl(self) -> U16Vector3: ...
    @property
    def lgr(self) -> U16Vector3: ...
    @property
    def lgg(self) -> U16Vector3: ...
    @property
    def lgb(self) -> U16Vector3: ...
    @property
    def lgo(self) -> U16Vector3: ...
    @property
    def lgl(self) -> U16Vector3: ...
    @property
    def lbr(self) -> U16Vector3: ...
    @property
    def lbg(self) -> U16Vector3: ...
    @property
    def lbb(self) -> U16Vector3: ...
    @property
    def lbo(self) -> U16Vector3: ...
    @property
    def lbl(self) -> U16Vector3: ...
    @property
    def lor(self) -> U16Vector3: ...
    @property
    def log(self) -> U16Vector3: ...
    @property
    def lob(self) -> U16Vector3: ...
    @property
    def llr(self) -> U16Vector3: ...
    @property
    def llg(self) -> U16Vector3: ...
    @property
    def llb(self) -> U16Vector3: ...
    @property
    def rrrr(self) -> U16Vector4: ...
    @property
    def rrrg(self) -> U16Vector4: ...
    @property
    def rrrb(self) -> U16Vector4: ...
    @property
    def rrro(self) -> U16Vector4: ...
    @property
    def rrrl(self) -> U16Vector4: ...
    @property
    def rrgr(self) -> U16Vector4: ...
    @property
    def rrgg(self) -> U16Vector4: ...
    @property
    def rrgb(self) -> U16Vector4: ...
    @property
    def rrgo(self) -> U16Vector4: ...
    @property
    def rrgl(self) -> U16Vector4: ...
    @property
    def rrbr(self) -> U16Vector4: ...
    @property
    def rrbg(self) -> U16Vector4: ...
    @property
    def rrbb(self) -> U16Vector4: ...
    @property
    def rrbo(self) -> U16Vector4: ...
    @property
    def rrbl(self) -> U16Vector4: ...
    @property
    def rror(self) -> U16Vector4: ...
    @property
    def rrog(self) -> U16Vector4: ...
    @property
    def rrob(self) -> U16Vector4: ...
    @property
    def rroo(self) -> U16Vector4: ...
    @property
    def rrol(self) -> U16Vector4: ...
    @property
    def rrlr(self) -> U16Vector4: ...
    @property
    def rrlg(self) -> U16Vector4: ...
    @property
    def rrlb(self) -> U16Vector4: ...
    @property
    def rrlo(self) -> U16Vector4: ...
    @property
    def rrll(self) -> U16Vector4: ...
    @property
    def rgrr(self) -> U16Vector4: ...
    @property
    def rgrg(self) -> U16Vector4: ...
    @property
    def rgrb(self) -> U16Vector4: ...
    @property
    def rgro(self) -> U16Vector4: ...
    @property
    def rgrl(self) -> U16Vector4: ...
    @property
    def rggr(self) -> U16Vector4: ...
    @property
    def rggg(self) -> U16Vector4: ...
    @property
    def rggb(self) -> U16Vector4: ...
    @property
    def rggo(self) -> U16Vector4: ...
    @property
    def rggl(self) -> U16Vector4: ...
    @property
    def rgbr(self) -> U16Vector4: ...
    @property
    def rgbg(self) -> U16Vector4: ...
    @property
    def rgbb(self) -> U16Vector4: ...
    @property
    def rgbo(self) -> U16Vector4: ...
    @property
    def rgbl(self) -> U16Vector4: ...
    @property
    def rgor(self) -> U16Vector4: ...
    @property
    def rgog(self) -> U16Vector4: ...
    @property
    def rgob(self) -> U16Vector4: ...
    @property
    def rgoo(self) -> U16Vector4: ...
    @property
    def rgol(self) -> U16Vector4: ...
    @property
    def rglr(self) -> U16Vector4: ...
    @property
    def rglg(self) -> U16Vector4: ...
    @property
    def rglb(self) -> U16Vector4: ...
    @property
    def rglo(self) -> U16Vector4: ...
    @property
    def rgll(self) -> U16Vector4: ...
    @property
    def rbrr(self) -> U16Vector4: ...
    @property
    def rbrg(self) -> U16Vector4: ...
    @property
    def rbrb(self) -> U16Vector4: ...
    @property
    def rbro(self) -> U16Vector4: ...
    @property
    def rbrl(self) -> U16Vector4: ...
    @property
    def rbgr(self) -> U16Vector4: ...
    @property
    def rbgg(self) -> U16Vector4: ...
    @property
    def rbgb(self) -> U16Vector4: ...
    @property
    def rbgo(self) -> U16Vector4: ...
    @property
    def rbgl(self) -> U16Vector4: ...
    @property
    def rbbr(self) -> U16Vector4: ...
    @property
    def rbbg(self) -> U16Vector4: ...
    @property
    def rbbb(self) -> U16Vector4: ...
    @property
    def rbbo(self) -> U16Vector4: ...
    @property
    def rbbl(self) -> U16Vector4: ...
    @property
    def rbor(self) -> U16Vector4: ...
    @property
    def rbog(self) -> U16Vector4: ...
    @property
    def rbob(self) -> U16Vector4: ...
    @property
    def rboo(self) -> U16Vector4: ...
    @property
    def rbol(self) -> U16Vector4: ...
    @property
    def rblr(self) -> U16Vector4: ...
    @property
    def rblg(self) -> U16Vector4: ...
    @property
    def rblb(self) -> U16Vector4: ...
    @property
    def rblo(self) -> U16Vector4: ...
    @property
    def rbll(self) -> U16Vector4: ...
    @property
    def rorr(self) -> U16Vector4: ...
    @property
    def rorg(self) -> U16Vector4: ...
    @property
    def rorb(self) -> U16Vector4: ...
    @property
    def roro(self) -> U16Vector4: ...
    @property
    def rorl(self) -> U16Vector4: ...
    @property
    def rogr(self) -> U16Vector4: ...
    @property
    def rogg(self) -> U16Vector4: ...
    @property
    def rogb(self) -> U16Vector4: ...
    @property
    def rogo(self) -> U16Vector4: ...
    @property
    def rogl(self) -> U16Vector4: ...
    @property
    def robr(self) -> U16Vector4: ...
    @property
    def robg(self) -> U16Vector4: ...
    @property
    def robb(self) -> U16Vector4: ...
    @property
    def robo(self) -> U16Vector4: ...
    @property
    def robl(self) -> U16Vector4: ...
    @property
    def roor(self) -> U16Vector4: ...
    @property
    def roog(self) -> U16Vector4: ...
    @property
    def roob(self) -> U16Vector4: ...
    @property
    def rooo(self) -> U16Vector4: ...
    @property
    def rool(self) -> U16Vector4: ...
    @property
    def rolr(self) -> U16Vector4: ...
    @property
    def rolg(self) -> U16Vector4: ...
    @property
    def rolb(self) -> U16Vector4: ...
    @property
    def rolo(self) -> U16Vector4: ...
    @property
    def roll(self) -> U16Vector4: ...
    @property
    def rlrr(self) -> U16Vector4: ...
    @property
    def rlrg(self) -> U16Vector4: ...
    @property
    def rlrb(self) -> U16Vector4: ...
    @property
    def rlro(self) -> U16Vector4: ...
    @property
    def rlrl(self) -> U16Vector4: ...
    @property
    def rlgr(self) -> U16Vector4: ...
    @property
    def rlgg(self) -> U16Vector4: ...
    @property
    def rlgb(self) -> U16Vector4: ...
    @property
    def rlgo(self) -> U16Vector4: ...
    @property
    def rlgl(self) -> U16Vector4: ...
    @property
    def rlbr(self) -> U16Vector4: ...
    @property
    def rlbg(self) -> U16Vector4: ...
    @property
    def rlbb(self) -> U16Vector4: ...
    @property
    def rlbo(self) -> U16Vector4: ...
    @property
    def rlbl(self) -> U16Vector4: ...
    @property
    def rlor(self) -> U16Vector4: ...
    @property
    def rlog(self) -> U16Vector4: ...
    @property
    def rlob(self) -> U16Vector4: ...
    @property
    def rloo(self) -> U16Vector4: ...
    @property
    def rlol(self) -> U16Vector4: ...
    @property
    def rllr(self) -> U16Vector4: ...
    @property
    def rllg(self) -> U16Vector4: ...
    @property
    def rllb(self) -> U16Vector4: ...
    @property
    def rllo(self) -> U16Vector4: ...
    @property
    def rlll(self) -> U16Vector4: ...
    @property
    def grrr(self) -> U16Vector4: ...
    @property
    def grrg(self) -> U16Vector4: ...
    @property
    def grrb(self) -> U16Vector4: ...
    @property
    def grro(self) -> U16Vector4: ...
    @property
    def grrl(self) -> U16Vector4: ...
    @property
    def grgr(self) -> U16Vector4: ...
    @property
    def grgg(self) -> U16Vector4: ...
    @property
    def grgb(self) -> U16Vector4: ...
    @property
    def grgo(self) -> U16Vector4: ...
    @property
    def grgl(self) -> U16Vector4: ...
    @property
    def grbr(self) -> U16Vector4: ...
    @property
    def grbg(self) -> U16Vector4: ...
    @property
    def grbb(self) -> U16Vector4: ...
    @property
    def grbo(self) -> U16Vector4: ...
    @property
    def grbl(self) -> U16Vector4: ...
    @property
    def gror(self) -> U16Vector4: ...
    @property
    def grog(self) -> U16Vector4: ...
    @property
    def grob(self) -> U16Vector4: ...
    @property
    def groo(self) -> U16Vector4: ...
    @property
    def grol(self) -> U16Vector4: ...
    @property
    def grlr(self) -> U16Vector4: ...
    @property
    def grlg(self) -> U16Vector4: ...
    @property
    def grlb(self) -> U16Vector4: ...
    @property
    def grlo(self) -> U16Vector4: ...
    @property
    def grll(self) -> U16Vector4: ...
    @property
    def ggrr(self) -> U16Vector4: ...
    @property
    def ggrg(self) -> U16Vector4: ...
    @property
    def ggrb(self) -> U16Vector4: ...
    @property
    def ggro(self) -> U16Vector4: ...
    @property
    def ggrl(self) -> U16Vector4: ...
    @property
    def gggr(self) -> U16Vector4: ...
    @property
    def gggg(self) -> U16Vector4: ...
    @property
    def gggb(self) -> U16Vector4: ...
    @property
    def gggo(self) -> U16Vector4: ...
    @property
    def gggl(self) -> U16Vector4: ...
    @property
    def ggbr(self) -> U16Vector4: ...
    @property
    def ggbg(self) -> U16Vector4: ...
    @property
    def ggbb(self) -> U16Vector4: ...
    @property
    def ggbo(self) -> U16Vector4: ...
    @property
    def ggbl(self) -> U16Vector4: ...
    @property
    def ggor(self) -> U16Vector4: ...
    @property
    def ggog(self) -> U16Vector4: ...
    @property
    def ggob(self) -> U16Vector4: ...
    @property
    def ggoo(self) -> U16Vector4: ...
    @property
    def ggol(self) -> U16Vector4: ...
    @property
    def gglr(self) -> U16Vector4: ...
    @property
    def gglg(self) -> U16Vector4: ...
    @property
    def gglb(self) -> U16Vector4: ...
    @property
    def gglo(self) -> U16Vector4: ...
    @property
    def ggll(self) -> U16Vector4: ...
    @property
    def gbrr(self) -> U16Vector4: ...
    @property
    def gbrg(self) -> U16Vector4: ...
    @property
    def gbrb(self) -> U16Vector4: ...
    @property
    def gbro(self) -> U16Vector4: ...
    @property
    def gbrl(self) -> U16Vector4: ...
    @property
    def gbgr(self) -> U16Vector4: ...
    @property
    def gbgg(self) -> U16Vector4: ...
    @property
    def gbgb(self) -> U16Vector4: ...
    @property
    def gbgo(self) -> U16Vector4: ...
    @property
    def gbgl(self) -> U16Vector4: ...
    @property
    def gbbr(self) -> U16Vector4: ...
    @property
    def gbbg(self) -> U16Vector4: ...
    @property
    def gbbb(self) -> U16Vector4: ...
    @property
    def gbbo(self) -> U16Vector4: ...
    @property
    def gbbl(self) -> U16Vector4: ...
    @property
    def gbor(self) -> U16Vector4: ...
    @property
    def gbog(self) -> U16Vector4: ...
    @property
    def gbob(self) -> U16Vector4: ...
    @property
    def gboo(self) -> U16Vector4: ...
    @property
    def gbol(self) -> U16Vector4: ...
    @property
    def gblr(self) -> U16Vector4: ...
    @property
    def gblg(self) -> U16Vector4: ...
    @property
    def gblb(self) -> U16Vector4: ...
    @property
    def gblo(self) -> U16Vector4: ...
    @property
    def gbll(self) -> U16Vector4: ...
    @property
    def gorr(self) -> U16Vector4: ...
    @property
    def gorg(self) -> U16Vector4: ...
    @property
    def gorb(self) -> U16Vector4: ...
    @property
    def goro(self) -> U16Vector4: ...
    @property
    def gorl(self) -> U16Vector4: ...
    @property
    def gogr(self) -> U16Vector4: ...
    @property
    def gogg(self) -> U16Vector4: ...
    @property
    def gogb(self) -> U16Vector4: ...
    @property
    def gogo(self) -> U16Vector4: ...
    @property
    def gogl(self) -> U16Vector4: ...
    @property
    def gobr(self) -> U16Vector4: ...
    @property
    def gobg(self) -> U16Vector4: ...
    @property
    def gobb(self) -> U16Vector4: ...
    @property
    def gobo(self) -> U16Vector4: ...
    @property
    def gobl(self) -> U16Vector4: ...
    @property
    def goor(self) -> U16Vector4: ...
    @property
    def goog(self) -> U16Vector4: ...
    @property
    def goob(self) -> U16Vector4: ...
    @property
    def gooo(self) -> U16Vector4: ...
    @property
    def gool(self) -> U16Vector4: ...
    @property
    def golr(self) -> U16Vector4: ...
    @property
    def golg(self) -> U16Vector4: ...
    @property
    def golb(self) -> U16Vector4: ...
    @property
    def golo(self) -> U16Vector4: ...
    @property
    def goll(self) -> U16Vector4: ...
    @property
    def glrr(self) -> U16Vector4: ...
    @property
    def glrg(self) -> U16Vector4: ...
    @property
    def glrb(self) -> U16Vector4: ...
    @property
    def glro(self) -> U16Vector4: ...
    @property
    def glrl(self) -> U16Vector4: ...
    @property
    def glgr(self) -> U16Vector4: ...
    @property
    def glgg(self) -> U16Vector4: ...
    @property
    def glgb(self) -> U16Vector4: ...
    @property
    def glgo(self) -> U16Vector4: ...
    @property
    def glgl(self) -> U16Vector4: ...
    @property
    def glbr(self) -> U16Vector4: ...
    @property
    def glbg(self) -> U16Vector4: ...
    @property
    def glbb(self) -> U16Vector4: ...
    @property
    def glbo(self) -> U16Vector4: ...
    @property
    def glbl(self) -> U16Vector4: ...
    @property
    def glor(self) -> U16Vector4: ...
    @property
    def glog(self) -> U16Vector4: ...
    @property
    def glob(self) -> U16Vector4: ...
    @property
    def gloo(self) -> U16Vector4: ...
    @property
    def glol(self) -> U16Vector4: ...
    @property
    def gllr(self) -> U16Vector4: ...
    @property
    def gllg(self) -> U16Vector4: ...
    @property
    def gllb(self) -> U16Vector4: ...
    @property
    def gllo(self) -> U16Vector4: ...
    @property
    def glll(self) -> U16Vector4: ...
    @property
    def brrr(self) -> U16Vector4: ...
    @property
    def brrg(self) -> U16Vector4: ...
    @property
    def brrb(self) -> U16Vector4: ...
    @property
    def brro(self) -> U16Vector4: ...
    @property
    def brrl(self) -> U16Vector4: ...
    @property
    def brgr(self) -> U16Vector4: ...
    @property
    def brgg(self) -> U16Vector4: ...
    @property
    def brgb(self) -> U16Vector4: ...
    @property
    def brgo(self) -> U16Vector4: ...
    @property
    def brgl(self) -> U16Vector4: ...
    @property
    def brbr(self) -> U16Vector4: ...
    @property
    def brbg(self) -> U16Vector4: ...
    @property
    def brbb(self) -> U16Vector4: ...
    @property
    def brbo(self) -> U16Vector4: ...
    @property
    def brbl(self) -> U16Vector4: ...
    @property
    def bror(self) -> U16Vector4: ...
    @property
    def brog(self) -> U16Vector4: ...
    @property
    def brob(self) -> U16Vector4: ...
    @property
    def broo(self) -> U16Vector4: ...
    @property
    def brol(self) -> U16Vector4: ...
    @property
    def brlr(self) -> U16Vector4: ...
    @property
    def brlg(self) -> U16Vector4: ...
    @property
    def brlb(self) -> U16Vector4: ...
    @property
    def brlo(self) -> U16Vector4: ...
    @property
    def brll(self) -> U16Vector4: ...
    @property
    def bgrr(self) -> U16Vector4: ...
    @property
    def bgrg(self) -> U16Vector4: ...
    @property
    def bgrb(self) -> U16Vector4: ...
    @property
    def bgro(self) -> U16Vector4: ...
    @property
    def bgrl(self) -> U16Vector4: ...
    @property
    def bggr(self) -> U16Vector4: ...
    @property
    def bggg(self) -> U16Vector4: ...
    @property
    def bggb(self) -> U16Vector4: ...
    @property
    def bggo(self) -> U16Vector4: ...
    @property
    def bggl(self) -> U16Vector4: ...
    @property
    def bgbr(self) -> U16Vector4: ...
    @property
    def bgbg(self) -> U16Vector4: ...
    @property
    def bgbb(self) -> U16Vector4: ...
    @property
    def bgbo(self) -> U16Vector4: ...
    @property
    def bgbl(self) -> U16Vector4: ...
    @property
    def bgor(self) -> U16Vector4: ...
    @property
    def bgog(self) -> U16Vector4: ...
    @property
    def bgob(self) -> U16Vector4: ...
    @property
    def bgoo(self) -> U16Vector4: ...
    @property
    def bgol(self) -> U16Vector4: ...
    @property
    def bglr(self) -> U16Vector4: ...
    @property
    def bglg(self) -> U16Vector4: ...
    @property
    def bglb(self) -> U16Vector4: ...
    @property
    def bglo(self) -> U16Vector4: ...
    @property
    def bgll(self) -> U16Vector4: ...
    @property
    def bbrr(self) -> U16Vector4: ...
    @property
    def bbrg(self) -> U16Vector4: ...
    @property
    def bbrb(self) -> U16Vector4: ...
    @property
    def bbro(self) -> U16Vector4: ...
    @property
    def bbrl(self) -> U16Vector4: ...
    @property
    def bbgr(self) -> U16Vector4: ...
    @property
    def bbgg(self) -> U16Vector4: ...
    @property
    def bbgb(self) -> U16Vector4: ...
    @property
    def bbgo(self) -> U16Vector4: ...
    @property
    def bbgl(self) -> U16Vector4: ...
    @property
    def bbbr(self) -> U16Vector4: ...
    @property
    def bbbg(self) -> U16Vector4: ...
    @property
    def bbbb(self) -> U16Vector4: ...
    @property
    def bbbo(self) -> U16Vector4: ...
    @property
    def bbbl(self) -> U16Vector4: ...
    @property
    def bbor(self) -> U16Vector4: ...
    @property
    def bbog(self) -> U16Vector4: ...
    @property
    def bbob(self) -> U16Vector4: ...
    @property
    def bboo(self) -> U16Vector4: ...
    @property
    def bbol(self) -> U16Vector4: ...
    @property
    def bblr(self) -> U16Vector4: ...
    @property
    def bblg(self) -> U16Vector4: ...
    @property
    def bblb(self) -> U16Vector4: ...
    @property
    def bblo(self) -> U16Vector4: ...
    @property
    def bbll(self) -> U16Vector4: ...
    @property
    def borr(self) -> U16Vector4: ...
    @property
    def borg(self) -> U16Vector4: ...
    @property
    def borb(self) -> U16Vector4: ...
    @property
    def boro(self) -> U16Vector4: ...
    @property
    def borl(self) -> U16Vector4: ...
    @property
    def bogr(self) -> U16Vector4: ...
    @property
    def bogg(self) -> U16Vector4: ...
    @property
    def bogb(self) -> U16Vector4: ...
    @property
    def bogo(self) -> U16Vector4: ...
    @property
    def bogl(self) -> U16Vector4: ...
    @property
    def bobr(self) -> U16Vector4: ...
    @property
    def bobg(self) -> U16Vector4: ...
    @property
    def bobb(self) -> U16Vector4: ...
    @property
    def bobo(self) -> U16Vector4: ...
    @property
    def bobl(self) -> U16Vector4: ...
    @property
    def boor(self) -> U16Vector4: ...
    @property
    def boog(self) -> U16Vector4: ...
    @property
    def boob(self) -> U16Vector4: ...
    @property
    def booo(self) -> U16Vector4: ...
    @property
    def bool(self) -> U16Vector4: ...
    @property
    def bolr(self) -> U16Vector4: ...
    @property
    def bolg(self) -> U16Vector4: ...
    @property
    def bolb(self) -> U16Vector4: ...
    @property
    def bolo(self) -> U16Vector4: ...
    @property
    def boll(self) -> U16Vector4: ...
    @property
    def blrr(self) -> U16Vector4: ...
    @property
    def blrg(self) -> U16Vector4: ...
    @property
    def blrb(self) -> U16Vector4: ...
    @property
    def blro(self) -> U16Vector4: ...
    @property
    def blrl(self) -> U16Vector4: ...
    @property
    def blgr(self) -> U16Vector4: ...
    @property
    def blgg(self) -> U16Vector4: ...
    @property
    def blgb(self) -> U16Vector4: ...
    @property
    def blgo(self) -> U16Vector4: ...
    @property
    def blgl(self) -> U16Vector4: ...
    @property
    def blbr(self) -> U16Vector4: ...
    @property
    def blbg(self) -> U16Vector4: ...
    @property
    def blbb(self) -> U16Vector4: ...
    @property
    def blbo(self) -> U16Vector4: ...
    @property
    def blbl(self) -> U16Vector4: ...
    @property
    def blor(self) -> U16Vector4: ...
    @property
    def blog(self) -> U16Vector4: ...
    @property
    def blob(self) -> U16Vector4: ...
    @property
    def bloo(self) -> U16Vector4: ...
    @property
    def blol(self) -> U16Vector4: ...
    @property
    def bllr(self) -> U16Vector4: ...
    @property
    def bllg(self) -> U16Vector4: ...
    @property
    def bllb(self) -> U16Vector4: ...
    @property
    def bllo(self) -> U16Vector4: ...
    @property
    def blll(self) -> U16Vector4: ...
    @property
    def orrr(self) -> U16Vector4: ...
    @property
    def orrg(self) -> U16Vector4: ...
    @property
    def orrb(self) -> U16Vector4: ...
    @property
    def orro(self) -> U16Vector4: ...
    @property
    def orrl(self) -> U16Vector4: ...
    @property
    def orgr(self) -> U16Vector4: ...
    @property
    def orgg(self) -> U16Vector4: ...
    @property
    def orgb(self) -> U16Vector4: ...
    @property
    def orgo(self) -> U16Vector4: ...
    @property
    def orgl(self) -> U16Vector4: ...
    @property
    def orbr(self) -> U16Vector4: ...
    @property
    def orbg(self) -> U16Vector4: ...
    @property
    def orbb(self) -> U16Vector4: ...
    @property
    def orbo(self) -> U16Vector4: ...
    @property
    def orbl(self) -> U16Vector4: ...
    @property
    def oror(self) -> U16Vector4: ...
    @property
    def orog(self) -> U16Vector4: ...
    @property
    def orob(self) -> U16Vector4: ...
    @property
    def oroo(self) -> U16Vector4: ...
    @property
    def orol(self) -> U16Vector4: ...
    @property
    def orlr(self) -> U16Vector4: ...
    @property
    def orlg(self) -> U16Vector4: ...
    @property
    def orlb(self) -> U16Vector4: ...
    @property
    def orlo(self) -> U16Vector4: ...
    @property
    def orll(self) -> U16Vector4: ...
    @property
    def ogrr(self) -> U16Vector4: ...
    @property
    def ogrg(self) -> U16Vector4: ...
    @property
    def ogrb(self) -> U16Vector4: ...
    @property
    def ogro(self) -> U16Vector4: ...
    @property
    def ogrl(self) -> U16Vector4: ...
    @property
    def oggr(self) -> U16Vector4: ...
    @property
    def oggg(self) -> U16Vector4: ...
    @property
    def oggb(self) -> U16Vector4: ...
    @property
    def oggo(self) -> U16Vector4: ...
    @property
    def oggl(self) -> U16Vector4: ...
    @property
    def ogbr(self) -> U16Vector4: ...
    @property
    def ogbg(self) -> U16Vector4: ...
    @property
    def ogbb(self) -> U16Vector4: ...
    @property
    def ogbo(self) -> U16Vector4: ...
    @property
    def ogbl(self) -> U16Vector4: ...
    @property
    def ogor(self) -> U16Vector4: ...
    @property
    def ogog(self) -> U16Vector4: ...
    @property
    def ogob(self) -> U16Vector4: ...
    @property
    def ogoo(self) -> U16Vector4: ...
    @property
    def ogol(self) -> U16Vector4: ...
    @property
    def oglr(self) -> U16Vector4: ...
    @property
    def oglg(self) -> U16Vector4: ...
    @property
    def oglb(self) -> U16Vector4: ...
    @property
    def oglo(self) -> U16Vector4: ...
    @property
    def ogll(self) -> U16Vector4: ...
    @property
    def obrr(self) -> U16Vector4: ...
    @property
    def obrg(self) -> U16Vector4: ...
    @property
    def obrb(self) -> U16Vector4: ...
    @property
    def obro(self) -> U16Vector4: ...
    @property
    def obrl(self) -> U16Vector4: ...
    @property
    def obgr(self) -> U16Vector4: ...
    @property
    def obgg(self) -> U16Vector4: ...
    @property
    def obgb(self) -> U16Vector4: ...
    @property
    def obgo(self) -> U16Vector4: ...
    @property
    def obgl(self) -> U16Vector4: ...
    @property
    def obbr(self) -> U16Vector4: ...
    @property
    def obbg(self) -> U16Vector4: ...
    @property
    def obbb(self) -> U16Vector4: ...
    @property
    def obbo(self) -> U16Vector4: ...
    @property
    def obbl(self) -> U16Vector4: ...
    @property
    def obor(self) -> U16Vector4: ...
    @property
    def obog(self) -> U16Vector4: ...
    @property
    def obob(self) -> U16Vector4: ...
    @property
    def oboo(self) -> U16Vector4: ...
    @property
    def obol(self) -> U16Vector4: ...
    @property
    def oblr(self) -> U16Vector4: ...
    @property
    def oblg(self) -> U16Vector4: ...
    @property
    def oblb(self) -> U16Vector4: ...
    @property
    def oblo(self) -> U16Vector4: ...
    @property
    def obll(self) -> U16Vector4: ...
    @property
    def oorr(self) -> U16Vector4: ...
    @property
    def oorg(self) -> U16Vector4: ...
    @property
    def oorb(self) -> U16Vector4: ...
    @property
    def ooro(self) -> U16Vector4: ...
    @property
    def oorl(self) -> U16Vector4: ...
    @property
    def oogr(self) -> U16Vector4: ...
    @property
    def oogg(self) -> U16Vector4: ...
    @property
    def oogb(self) -> U16Vector4: ...
    @property
    def oogo(self) -> U16Vector4: ...
    @property
    def oogl(self) -> U16Vector4: ...
    @property
    def oobr(self) -> U16Vector4: ...
    @property
    def oobg(self) -> U16Vector4: ...
    @property
    def oobb(self) -> U16Vector4: ...
    @property
    def oobo(self) -> U16Vector4: ...
    @property
    def oobl(self) -> U16Vector4: ...
    @property
    def ooor(self) -> U16Vector4: ...
    @property
    def ooog(self) -> U16Vector4: ...
    @property
    def ooob(self) -> U16Vector4: ...
    @property
    def oolr(self) -> U16Vector4: ...
    @property
    def oolg(self) -> U16Vector4: ...
    @property
    def oolb(self) -> U16Vector4: ...
    @property
    def olrr(self) -> U16Vector4: ...
    @property
    def olrg(self) -> U16Vector4: ...
    @property
    def olrb(self) -> U16Vector4: ...
    @property
    def olro(self) -> U16Vector4: ...
    @property
    def olrl(self) -> U16Vector4: ...
    @property
    def olgr(self) -> U16Vector4: ...
    @property
    def olgg(self) -> U16Vector4: ...
    @property
    def olgb(self) -> U16Vector4: ...
    @property
    def olgo(self) -> U16Vector4: ...
    @property
    def olgl(self) -> U16Vector4: ...
    @property
    def olbr(self) -> U16Vector4: ...
    @property
    def olbg(self) -> U16Vector4: ...
    @property
    def olbb(self) -> U16Vector4: ...
    @property
    def olbo(self) -> U16Vector4: ...
    @property
    def olbl(self) -> U16Vector4: ...
    @property
    def olor(self) -> U16Vector4: ...
    @property
    def olog(self) -> U16Vector4: ...
    @property
    def olob(self) -> U16Vector4: ...
    @property
    def ollr(self) -> U16Vector4: ...
    @property
    def ollg(self) -> U16Vector4: ...
    @property
    def ollb(self) -> U16Vector4: ...
    @property
    def lrrr(self) -> U16Vector4: ...
    @property
    def lrrg(self) -> U16Vector4: ...
    @property
    def lrrb(self) -> U16Vector4: ...
    @property
    def lrro(self) -> U16Vector4: ...
    @property
    def lrrl(self) -> U16Vector4: ...
    @property
    def lrgr(self) -> U16Vector4: ...
    @property
    def lrgg(self) -> U16Vector4: ...
    @property
    def lrgb(self) -> U16Vector4: ...
    @property
    def lrgo(self) -> U16Vector4: ...
    @property
    def lrgl(self) -> U16Vector4: ...
    @property
    def lrbr(self) -> U16Vector4: ...
    @property
    def lrbg(self) -> U16Vector4: ...
    @property
    def lrbb(self) -> U16Vector4: ...
    @property
    def lrbo(self) -> U16Vector4: ...
    @property
    def lrbl(self) -> U16Vector4: ...
    @property
    def lror(self) -> U16Vector4: ...
    @property
    def lrog(self) -> U16Vector4: ...
    @property
    def lrob(self) -> U16Vector4: ...
    @property
    def lroo(self) -> U16Vector4: ...
    @property
    def lrol(self) -> U16Vector4: ...
    @property
    def lrlr(self) -> U16Vector4: ...
    @property
    def lrlg(self) -> U16Vector4: ...
    @property
    def lrlb(self) -> U16Vector4: ...
    @property
    def lrlo(self) -> U16Vector4: ...
    @property
    def lrll(self) -> U16Vector4: ...
    @property
    def lgrr(self) -> U16Vector4: ...
    @property
    def lgrg(self) -> U16Vector4: ...
    @property
    def lgrb(self) -> U16Vector4: ...
    @property
    def lgro(self) -> U16Vector4: ...
    @property
    def lgrl(self) -> U16Vector4: ...
    @property
    def lggr(self) -> U16Vector4: ...
    @property
    def lggg(self) -> U16Vector4: ...
    @property
    def lggb(self) -> U16Vector4: ...
    @property
    def lggo(self) -> U16Vector4: ...
    @property
    def lggl(self) -> U16Vector4: ...
    @property
    def lgbr(self) -> U16Vector4: ...
    @property
    def lgbg(self) -> U16Vector4: ...
    @property
    def lgbb(self) -> U16Vector4: ...
    @property
    def lgbo(self) -> U16Vector4: ...
    @property
    def lgbl(self) -> U16Vector4: ...
    @property
    def lgor(self) -> U16Vector4: ...
    @property
    def lgog(self) -> U16Vector4: ...
    @property
    def lgob(self) -> U16Vector4: ...
    @property
    def lgoo(self) -> U16Vector4: ...
    @property
    def lgol(self) -> U16Vector4: ...
    @property
    def lglr(self) -> U16Vector4: ...
    @property
    def lglg(self) -> U16Vector4: ...
    @property
    def lglb(self) -> U16Vector4: ...
    @property
    def lglo(self) -> U16Vector4: ...
    @property
    def lgll(self) -> U16Vector4: ...
    @property
    def lbrr(self) -> U16Vector4: ...
    @property
    def lbrg(self) -> U16Vector4: ...
    @property
    def lbrb(self) -> U16Vector4: ...
    @property
    def lbro(self) -> U16Vector4: ...
    @property
    def lbrl(self) -> U16Vector4: ...
    @property
    def lbgr(self) -> U16Vector4: ...
    @property
    def lbgg(self) -> U16Vector4: ...
    @property
    def lbgb(self) -> U16Vector4: ...
    @property
    def lbgo(self) -> U16Vector4: ...
    @property
    def lbgl(self) -> U16Vector4: ...
    @property
    def lbbr(self) -> U16Vector4: ...
    @property
    def lbbg(self) -> U16Vector4: ...
    @property
    def lbbb(self) -> U16Vector4: ...
    @property
    def lbbo(self) -> U16Vector4: ...
    @property
    def lbbl(self) -> U16Vector4: ...
    @property
    def lbor(self) -> U16Vector4: ...
    @property
    def lbog(self) -> U16Vector4: ...
    @property
    def lbob(self) -> U16Vector4: ...
    @property
    def lboo(self) -> U16Vector4: ...
    @property
    def lbol(self) -> U16Vector4: ...
    @property
    def lblr(self) -> U16Vector4: ...
    @property
    def lblg(self) -> U16Vector4: ...
    @property
    def lblb(self) -> U16Vector4: ...
    @property
    def lblo(self) -> U16Vector4: ...
    @property
    def lbll(self) -> U16Vector4: ...
    @property
    def lorr(self) -> U16Vector4: ...
    @property
    def lorg(self) -> U16Vector4: ...
    @property
    def lorb(self) -> U16Vector4: ...
    @property
    def loro(self) -> U16Vector4: ...
    @property
    def lorl(self) -> U16Vector4: ...
    @property
    def logr(self) -> U16Vector4: ...
    @property
    def logg(self) -> U16Vector4: ...
    @property
    def logb(self) -> U16Vector4: ...
    @property
    def logo(self) -> U16Vector4: ...
    @property
    def logl(self) -> U16Vector4: ...
    @property
    def lobr(self) -> U16Vector4: ...
    @property
    def lobg(self) -> U16Vector4: ...
    @property
    def lobb(self) -> U16Vector4: ...
    @property
    def lobo(self) -> U16Vector4: ...
    @property
    def lobl(self) -> U16Vector4: ...
    @property
    def loor(self) -> U16Vector4: ...
    @property
    def loog(self) -> U16Vector4: ...
    @property
    def loob(self) -> U16Vector4: ...
    @property
    def lolr(self) -> U16Vector4: ...
    @property
    def lolg(self) -> U16Vector4: ...
    @property
    def lolb(self) -> U16Vector4: ...
    @property
    def llrr(self) -> U16Vector4: ...
    @property
    def llrg(self) -> U16Vector4: ...
    @property
    def llrb(self) -> U16Vector4: ...
    @property
    def llro(self) -> U16Vector4: ...
    @property
    def llrl(self) -> U16Vector4: ...
    @property
    def llgr(self) -> U16Vector4: ...
    @property
    def llgg(self) -> U16Vector4: ...
    @property
    def llgb(self) -> U16Vector4: ...
    @property
    def llgo(self) -> U16Vector4: ...
    @property
    def llgl(self) -> U16Vector4: ...
    @property
    def llbr(self) -> U16Vector4: ...
    @property
    def llbg(self) -> U16Vector4: ...
    @property
    def llbb(self) -> U16Vector4: ...
    @property
    def llbo(self) -> U16Vector4: ...
    @property
    def llbl(self) -> U16Vector4: ...
    @property
    def llor(self) -> U16Vector4: ...
    @property
    def llog(self) -> U16Vector4: ...
    @property
    def llob(self) -> U16Vector4: ...
    @property
    def lllr(self) -> U16Vector4: ...
    @property
    def lllg(self) -> U16Vector4: ...
    @property
    def lllb(self) -> U16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> U16Vector2: ...
    @property
    def st(self) -> U16Vector2: ...
    @property
    def sq(self) -> U16Vector2: ...
    @property
    def so(self) -> U16Vector2: ...
    @property
    def sl(self) -> U16Vector2: ...
    @property
    def ts(self) -> U16Vector2: ...
    @property
    def tt(self) -> U16Vector2: ...
    @property
    def tq(self) -> U16Vector2: ...
    @property
    def to(self) -> U16Vector2: ...
    @property
    def tl(self) -> U16Vector2: ...
    @property
    def qs(self) -> U16Vector2: ...
    @property
    def qt(self) -> U16Vector2: ...
    @property
    def qq(self) -> U16Vector2: ...
    @property
    def qo(self) -> U16Vector2: ...
    @property
    def ql(self) -> U16Vector2: ...
    @property
    def os(self) -> U16Vector2: ...
    @property
    def ot(self) -> U16Vector2: ...
    @property
    def oq(self) -> U16Vector2: ...
    @property
    def ls(self) -> U16Vector2: ...
    @property
    def lt(self) -> U16Vector2: ...
    @property
    def lq(self) -> U16Vector2: ...
    @property
    def sss(self) -> U16Vector3: ...
    @property
    def sst(self) -> U16Vector3: ...
    @property
    def ssq(self) -> U16Vector3: ...
    @property
    def sso(self) -> U16Vector3: ...
    @property
    def ssl(self) -> U16Vector3: ...
    @property
    def sts(self) -> U16Vector3: ...
    @property
    def stt(self) -> U16Vector3: ...
    @property
    def stq(self) -> U16Vector3: ...
    @property
    def sto(self) -> U16Vector3: ...
    @property
    def stl(self) -> U16Vector3: ...
    @property
    def sqs(self) -> U16Vector3: ...
    @property
    def sqt(self) -> U16Vector3: ...
    @property
    def sqq(self) -> U16Vector3: ...
    @property
    def sqo(self) -> U16Vector3: ...
    @property
    def sql(self) -> U16Vector3: ...
    @property
    def sos(self) -> U16Vector3: ...
    @property
    def sot(self) -> U16Vector3: ...
    @property
    def soq(self) -> U16Vector3: ...
    @property
    def soo(self) -> U16Vector3: ...
    @property
    def sol(self) -> U16Vector3: ...
    @property
    def sls(self) -> U16Vector3: ...
    @property
    def slt(self) -> U16Vector3: ...
    @property
    def slq(self) -> U16Vector3: ...
    @property
    def slo(self) -> U16Vector3: ...
    @property
    def sll(self) -> U16Vector3: ...
    @property
    def tss(self) -> U16Vector3: ...
    @property
    def tst(self) -> U16Vector3: ...
    @property
    def tsq(self) -> U16Vector3: ...
    @property
    def tso(self) -> U16Vector3: ...
    @property
    def tsl(self) -> U16Vector3: ...
    @property
    def tts(self) -> U16Vector3: ...
    @property
    def ttt(self) -> U16Vector3: ...
    @property
    def ttq(self) -> U16Vector3: ...
    @property
    def tto(self) -> U16Vector3: ...
    @property
    def ttl(self) -> U16Vector3: ...
    @property
    def tqs(self) -> U16Vector3: ...
    @property
    def tqt(self) -> U16Vector3: ...
    @property
    def tqq(self) -> U16Vector3: ...
    @property
    def tqo(self) -> U16Vector3: ...
    @property
    def tql(self) -> U16Vector3: ...
    @property
    def tos(self) -> U16Vector3: ...
    @property
    def tot(self) -> U16Vector3: ...
    @property
    def toq(self) -> U16Vector3: ...
    @property
    def too(self) -> U16Vector3: ...
    @property
    def tol(self) -> U16Vector3: ...
    @property
    def tls(self) -> U16Vector3: ...
    @property
    def tlt(self) -> U16Vector3: ...
    @property
    def tlq(self) -> U16Vector3: ...
    @property
    def tlo(self) -> U16Vector3: ...
    @property
    def tll(self) -> U16Vector3: ...
    @property
    def qss(self) -> U16Vector3: ...
    @property
    def qst(self) -> U16Vector3: ...
    @property
    def qsq(self) -> U16Vector3: ...
    @property
    def qso(self) -> U16Vector3: ...
    @property
    def qsl(self) -> U16Vector3: ...
    @property
    def qts(self) -> U16Vector3: ...
    @property
    def qtt(self) -> U16Vector3: ...
    @property
    def qtq(self) -> U16Vector3: ...
    @property
    def qto(self) -> U16Vector3: ...
    @property
    def qtl(self) -> U16Vector3: ...
    @property
    def qqs(self) -> U16Vector3: ...
    @property
    def qqt(self) -> U16Vector3: ...
    @property
    def qqq(self) -> U16Vector3: ...
    @property
    def qqo(self) -> U16Vector3: ...
    @property
    def qql(self) -> U16Vector3: ...
    @property
    def qos(self) -> U16Vector3: ...
    @property
    def qot(self) -> U16Vector3: ...
    @property
    def qoq(self) -> U16Vector3: ...
    @property
    def qoo(self) -> U16Vector3: ...
    @property
    def qol(self) -> U16Vector3: ...
    @property
    def qls(self) -> U16Vector3: ...
    @property
    def qlt(self) -> U16Vector3: ...
    @property
    def qlq(self) -> U16Vector3: ...
    @property
    def qlo(self) -> U16Vector3: ...
    @property
    def qll(self) -> U16Vector3: ...
    @property
    def oss(self) -> U16Vector3: ...
    @property
    def ost(self) -> U16Vector3: ...
    @property
    def osq(self) -> U16Vector3: ...
    @property
    def oso(self) -> U16Vector3: ...
    @property
    def osl(self) -> U16Vector3: ...
    @property
    def ots(self) -> U16Vector3: ...
    @property
    def ott(self) -> U16Vector3: ...
    @property
    def otq(self) -> U16Vector3: ...
    @property
    def oto(self) -> U16Vector3: ...
    @property
    def otl(self) -> U16Vector3: ...
    @property
    def oqs(self) -> U16Vector3: ...
    @property
    def oqt(self) -> U16Vector3: ...
    @property
    def oqq(self) -> U16Vector3: ...
    @property
    def oqo(self) -> U16Vector3: ...
    @property
    def oql(self) -> U16Vector3: ...
    @property
    def oos(self) -> U16Vector3: ...
    @property
    def oot(self) -> U16Vector3: ...
    @property
    def ooq(self) -> U16Vector3: ...
    @property
    def ols(self) -> U16Vector3: ...
    @property
    def olt(self) -> U16Vector3: ...
    @property
    def olq(self) -> U16Vector3: ...
    @property
    def lss(self) -> U16Vector3: ...
    @property
    def lst(self) -> U16Vector3: ...
    @property
    def lsq(self) -> U16Vector3: ...
    @property
    def lso(self) -> U16Vector3: ...
    @property
    def lsl(self) -> U16Vector3: ...
    @property
    def lts(self) -> U16Vector3: ...
    @property
    def ltt(self) -> U16Vector3: ...
    @property
    def ltq(self) -> U16Vector3: ...
    @property
    def lto(self) -> U16Vector3: ...
    @property
    def ltl(self) -> U16Vector3: ...
    @property
    def lqs(self) -> U16Vector3: ...
    @property
    def lqt(self) -> U16Vector3: ...
    @property
    def lqq(self) -> U16Vector3: ...
    @property
    def lqo(self) -> U16Vector3: ...
    @property
    def lql(self) -> U16Vector3: ...
    @property
    def los(self) -> U16Vector3: ...
    @property
    def lot(self) -> U16Vector3: ...
    @property
    def loq(self) -> U16Vector3: ...
    @property
    def lls(self) -> U16Vector3: ...
    @property
    def llt(self) -> U16Vector3: ...
    @property
    def llq(self) -> U16Vector3: ...
    @property
    def ssss(self) -> U16Vector4: ...
    @property
    def ssst(self) -> U16Vector4: ...
    @property
    def sssq(self) -> U16Vector4: ...
    @property
    def ssso(self) -> U16Vector4: ...
    @property
    def sssl(self) -> U16Vector4: ...
    @property
    def ssts(self) -> U16Vector4: ...
    @property
    def sstt(self) -> U16Vector4: ...
    @property
    def sstq(self) -> U16Vector4: ...
    @property
    def ssto(self) -> U16Vector4: ...
    @property
    def sstl(self) -> U16Vector4: ...
    @property
    def ssqs(self) -> U16Vector4: ...
    @property
    def ssqt(self) -> U16Vector4: ...
    @property
    def ssqq(self) -> U16Vector4: ...
    @property
    def ssqo(self) -> U16Vector4: ...
    @property
    def ssql(self) -> U16Vector4: ...
    @property
    def ssos(self) -> U16Vector4: ...
    @property
    def ssot(self) -> U16Vector4: ...
    @property
    def ssoq(self) -> U16Vector4: ...
    @property
    def ssoo(self) -> U16Vector4: ...
    @property
    def ssol(self) -> U16Vector4: ...
    @property
    def ssls(self) -> U16Vector4: ...
    @property
    def sslt(self) -> U16Vector4: ...
    @property
    def sslq(self) -> U16Vector4: ...
    @property
    def sslo(self) -> U16Vector4: ...
    @property
    def ssll(self) -> U16Vector4: ...
    @property
    def stss(self) -> U16Vector4: ...
    @property
    def stst(self) -> U16Vector4: ...
    @property
    def stsq(self) -> U16Vector4: ...
    @property
    def stso(self) -> U16Vector4: ...
    @property
    def stsl(self) -> U16Vector4: ...
    @property
    def stts(self) -> U16Vector4: ...
    @property
    def sttt(self) -> U16Vector4: ...
    @property
    def sttq(self) -> U16Vector4: ...
    @property
    def stto(self) -> U16Vector4: ...
    @property
    def sttl(self) -> U16Vector4: ...
    @property
    def stqs(self) -> U16Vector4: ...
    @property
    def stqt(self) -> U16Vector4: ...
    @property
    def stqq(self) -> U16Vector4: ...
    @property
    def stqo(self) -> U16Vector4: ...
    @property
    def stql(self) -> U16Vector4: ...
    @property
    def stos(self) -> U16Vector4: ...
    @property
    def stot(self) -> U16Vector4: ...
    @property
    def stoq(self) -> U16Vector4: ...
    @property
    def stoo(self) -> U16Vector4: ...
    @property
    def stol(self) -> U16Vector4: ...
    @property
    def stls(self) -> U16Vector4: ...
    @property
    def stlt(self) -> U16Vector4: ...
    @property
    def stlq(self) -> U16Vector4: ...
    @property
    def stlo(self) -> U16Vector4: ...
    @property
    def stll(self) -> U16Vector4: ...
    @property
    def sqss(self) -> U16Vector4: ...
    @property
    def sqst(self) -> U16Vector4: ...
    @property
    def sqsq(self) -> U16Vector4: ...
    @property
    def sqso(self) -> U16Vector4: ...
    @property
    def sqsl(self) -> U16Vector4: ...
    @property
    def sqts(self) -> U16Vector4: ...
    @property
    def sqtt(self) -> U16Vector4: ...
    @property
    def sqtq(self) -> U16Vector4: ...
    @property
    def sqto(self) -> U16Vector4: ...
    @property
    def sqtl(self) -> U16Vector4: ...
    @property
    def sqqs(self) -> U16Vector4: ...
    @property
    def sqqt(self) -> U16Vector4: ...
    @property
    def sqqq(self) -> U16Vector4: ...
    @property
    def sqqo(self) -> U16Vector4: ...
    @property
    def sqql(self) -> U16Vector4: ...
    @property
    def sqos(self) -> U16Vector4: ...
    @property
    def sqot(self) -> U16Vector4: ...
    @property
    def sqoq(self) -> U16Vector4: ...
    @property
    def sqoo(self) -> U16Vector4: ...
    @property
    def sqol(self) -> U16Vector4: ...
    @property
    def sqls(self) -> U16Vector4: ...
    @property
    def sqlt(self) -> U16Vector4: ...
    @property
    def sqlq(self) -> U16Vector4: ...
    @property
    def sqlo(self) -> U16Vector4: ...
    @property
    def sqll(self) -> U16Vector4: ...
    @property
    def soss(self) -> U16Vector4: ...
    @property
    def sost(self) -> U16Vector4: ...
    @property
    def sosq(self) -> U16Vector4: ...
    @property
    def soso(self) -> U16Vector4: ...
    @property
    def sosl(self) -> U16Vector4: ...
    @property
    def sots(self) -> U16Vector4: ...
    @property
    def sott(self) -> U16Vector4: ...
    @property
    def sotq(self) -> U16Vector4: ...
    @property
    def soto(self) -> U16Vector4: ...
    @property
    def sotl(self) -> U16Vector4: ...
    @property
    def soqs(self) -> U16Vector4: ...
    @property
    def soqt(self) -> U16Vector4: ...
    @property
    def soqq(self) -> U16Vector4: ...
    @property
    def soqo(self) -> U16Vector4: ...
    @property
    def soql(self) -> U16Vector4: ...
    @property
    def soos(self) -> U16Vector4: ...
    @property
    def soot(self) -> U16Vector4: ...
    @property
    def sooq(self) -> U16Vector4: ...
    @property
    def sooo(self) -> U16Vector4: ...
    @property
    def sool(self) -> U16Vector4: ...
    @property
    def sols(self) -> U16Vector4: ...
    @property
    def solt(self) -> U16Vector4: ...
    @property
    def solq(self) -> U16Vector4: ...
    @property
    def solo(self) -> U16Vector4: ...
    @property
    def soll(self) -> U16Vector4: ...
    @property
    def slss(self) -> U16Vector4: ...
    @property
    def slst(self) -> U16Vector4: ...
    @property
    def slsq(self) -> U16Vector4: ...
    @property
    def slso(self) -> U16Vector4: ...
    @property
    def slsl(self) -> U16Vector4: ...
    @property
    def slts(self) -> U16Vector4: ...
    @property
    def sltt(self) -> U16Vector4: ...
    @property
    def sltq(self) -> U16Vector4: ...
    @property
    def slto(self) -> U16Vector4: ...
    @property
    def sltl(self) -> U16Vector4: ...
    @property
    def slqs(self) -> U16Vector4: ...
    @property
    def slqt(self) -> U16Vector4: ...
    @property
    def slqq(self) -> U16Vector4: ...
    @property
    def slqo(self) -> U16Vector4: ...
    @property
    def slql(self) -> U16Vector4: ...
    @property
    def slos(self) -> U16Vector4: ...
    @property
    def slot(self) -> U16Vector4: ...
    @property
    def sloq(self) -> U16Vector4: ...
    @property
    def sloo(self) -> U16Vector4: ...
    @property
    def slol(self) -> U16Vector4: ...
    @property
    def slls(self) -> U16Vector4: ...
    @property
    def sllt(self) -> U16Vector4: ...
    @property
    def sllq(self) -> U16Vector4: ...
    @property
    def sllo(self) -> U16Vector4: ...
    @property
    def slll(self) -> U16Vector4: ...
    @property
    def tsss(self) -> U16Vector4: ...
    @property
    def tsst(self) -> U16Vector4: ...
    @property
    def tssq(self) -> U16Vector4: ...
    @property
    def tsso(self) -> U16Vector4: ...
    @property
    def tssl(self) -> U16Vector4: ...
    @property
    def tsts(self) -> U16Vector4: ...
    @property
    def tstt(self) -> U16Vector4: ...
    @property
    def tstq(self) -> U16Vector4: ...
    @property
    def tsto(self) -> U16Vector4: ...
    @property
    def tstl(self) -> U16Vector4: ...
    @property
    def tsqs(self) -> U16Vector4: ...
    @property
    def tsqt(self) -> U16Vector4: ...
    @property
    def tsqq(self) -> U16Vector4: ...
    @property
    def tsqo(self) -> U16Vector4: ...
    @property
    def tsql(self) -> U16Vector4: ...
    @property
    def tsos(self) -> U16Vector4: ...
    @property
    def tsot(self) -> U16Vector4: ...
    @property
    def tsoq(self) -> U16Vector4: ...
    @property
    def tsoo(self) -> U16Vector4: ...
    @property
    def tsol(self) -> U16Vector4: ...
    @property
    def tsls(self) -> U16Vector4: ...
    @property
    def tslt(self) -> U16Vector4: ...
    @property
    def tslq(self) -> U16Vector4: ...
    @property
    def tslo(self) -> U16Vector4: ...
    @property
    def tsll(self) -> U16Vector4: ...
    @property
    def ttss(self) -> U16Vector4: ...
    @property
    def ttst(self) -> U16Vector4: ...
    @property
    def ttsq(self) -> U16Vector4: ...
    @property
    def ttso(self) -> U16Vector4: ...
    @property
    def ttsl(self) -> U16Vector4: ...
    @property
    def ttts(self) -> U16Vector4: ...
    @property
    def tttt(self) -> U16Vector4: ...
    @property
    def tttq(self) -> U16Vector4: ...
    @property
    def ttto(self) -> U16Vector4: ...
    @property
    def tttl(self) -> U16Vector4: ...
    @property
    def ttqs(self) -> U16Vector4: ...
    @property
    def ttqt(self) -> U16Vector4: ...
    @property
    def ttqq(self) -> U16Vector4: ...
    @property
    def ttqo(self) -> U16Vector4: ...
    @property
    def ttql(self) -> U16Vector4: ...
    @property
    def ttos(self) -> U16Vector4: ...
    @property
    def ttot(self) -> U16Vector4: ...
    @property
    def ttoq(self) -> U16Vector4: ...
    @property
    def ttoo(self) -> U16Vector4: ...
    @property
    def ttol(self) -> U16Vector4: ...
    @property
    def ttls(self) -> U16Vector4: ...
    @property
    def ttlt(self) -> U16Vector4: ...
    @property
    def ttlq(self) -> U16Vector4: ...
    @property
    def ttlo(self) -> U16Vector4: ...
    @property
    def ttll(self) -> U16Vector4: ...
    @property
    def tqss(self) -> U16Vector4: ...
    @property
    def tqst(self) -> U16Vector4: ...
    @property
    def tqsq(self) -> U16Vector4: ...
    @property
    def tqso(self) -> U16Vector4: ...
    @property
    def tqsl(self) -> U16Vector4: ...
    @property
    def tqts(self) -> U16Vector4: ...
    @property
    def tqtt(self) -> U16Vector4: ...
    @property
    def tqtq(self) -> U16Vector4: ...
    @property
    def tqto(self) -> U16Vector4: ...
    @property
    def tqtl(self) -> U16Vector4: ...
    @property
    def tqqs(self) -> U16Vector4: ...
    @property
    def tqqt(self) -> U16Vector4: ...
    @property
    def tqqq(self) -> U16Vector4: ...
    @property
    def tqqo(self) -> U16Vector4: ...
    @property
    def tqql(self) -> U16Vector4: ...
    @property
    def tqos(self) -> U16Vector4: ...
    @property
    def tqot(self) -> U16Vector4: ...
    @property
    def tqoq(self) -> U16Vector4: ...
    @property
    def tqoo(self) -> U16Vector4: ...
    @property
    def tqol(self) -> U16Vector4: ...
    @property
    def tqls(self) -> U16Vector4: ...
    @property
    def tqlt(self) -> U16Vector4: ...
    @property
    def tqlq(self) -> U16Vector4: ...
    @property
    def tqlo(self) -> U16Vector4: ...
    @property
    def tqll(self) -> U16Vector4: ...
    @property
    def toss(self) -> U16Vector4: ...
    @property
    def tost(self) -> U16Vector4: ...
    @property
    def tosq(self) -> U16Vector4: ...
    @property
    def toso(self) -> U16Vector4: ...
    @property
    def tosl(self) -> U16Vector4: ...
    @property
    def tots(self) -> U16Vector4: ...
    @property
    def tott(self) -> U16Vector4: ...
    @property
    def totq(self) -> U16Vector4: ...
    @property
    def toto(self) -> U16Vector4: ...
    @property
    def totl(self) -> U16Vector4: ...
    @property
    def toqs(self) -> U16Vector4: ...
    @property
    def toqt(self) -> U16Vector4: ...
    @property
    def toqq(self) -> U16Vector4: ...
    @property
    def toqo(self) -> U16Vector4: ...
    @property
    def toql(self) -> U16Vector4: ...
    @property
    def toos(self) -> U16Vector4: ...
    @property
    def toot(self) -> U16Vector4: ...
    @property
    def tooq(self) -> U16Vector4: ...
    @property
    def tooo(self) -> U16Vector4: ...
    @property
    def tool(self) -> U16Vector4: ...
    @property
    def tols(self) -> U16Vector4: ...
    @property
    def tolt(self) -> U16Vector4: ...
    @property
    def tolq(self) -> U16Vector4: ...
    @property
    def tolo(self) -> U16Vector4: ...
    @property
    def toll(self) -> U16Vector4: ...
    @property
    def tlss(self) -> U16Vector4: ...
    @property
    def tlst(self) -> U16Vector4: ...
    @property
    def tlsq(self) -> U16Vector4: ...
    @property
    def tlso(self) -> U16Vector4: ...
    @property
    def tlsl(self) -> U16Vector4: ...
    @property
    def tlts(self) -> U16Vector4: ...
    @property
    def tltt(self) -> U16Vector4: ...
    @property
    def tltq(self) -> U16Vector4: ...
    @property
    def tlto(self) -> U16Vector4: ...
    @property
    def tltl(self) -> U16Vector4: ...
    @property
    def tlqs(self) -> U16Vector4: ...
    @property
    def tlqt(self) -> U16Vector4: ...
    @property
    def tlqq(self) -> U16Vector4: ...
    @property
    def tlqo(self) -> U16Vector4: ...
    @property
    def tlql(self) -> U16Vector4: ...
    @property
    def tlos(self) -> U16Vector4: ...
    @property
    def tlot(self) -> U16Vector4: ...
    @property
    def tloq(self) -> U16Vector4: ...
    @property
    def tloo(self) -> U16Vector4: ...
    @property
    def tlol(self) -> U16Vector4: ...
    @property
    def tlls(self) -> U16Vector4: ...
    @property
    def tllt(self) -> U16Vector4: ...
    @property
    def tllq(self) -> U16Vector4: ...
    @property
    def tllo(self) -> U16Vector4: ...
    @property
    def tlll(self) -> U16Vector4: ...
    @property
    def qsss(self) -> U16Vector4: ...
    @property
    def qsst(self) -> U16Vector4: ...
    @property
    def qssq(self) -> U16Vector4: ...
    @property
    def qsso(self) -> U16Vector4: ...
    @property
    def qssl(self) -> U16Vector4: ...
    @property
    def qsts(self) -> U16Vector4: ...
    @property
    def qstt(self) -> U16Vector4: ...
    @property
    def qstq(self) -> U16Vector4: ...
    @property
    def qsto(self) -> U16Vector4: ...
    @property
    def qstl(self) -> U16Vector4: ...
    @property
    def qsqs(self) -> U16Vector4: ...
    @property
    def qsqt(self) -> U16Vector4: ...
    @property
    def qsqq(self) -> U16Vector4: ...
    @property
    def qsqo(self) -> U16Vector4: ...
    @property
    def qsql(self) -> U16Vector4: ...
    @property
    def qsos(self) -> U16Vector4: ...
    @property
    def qsot(self) -> U16Vector4: ...
    @property
    def qsoq(self) -> U16Vector4: ...
    @property
    def qsoo(self) -> U16Vector4: ...
    @property
    def qsol(self) -> U16Vector4: ...
    @property
    def qsls(self) -> U16Vector4: ...
    @property
    def qslt(self) -> U16Vector4: ...
    @property
    def qslq(self) -> U16Vector4: ...
    @property
    def qslo(self) -> U16Vector4: ...
    @property
    def qsll(self) -> U16Vector4: ...
    @property
    def qtss(self) -> U16Vector4: ...
    @property
    def qtst(self) -> U16Vector4: ...
    @property
    def qtsq(self) -> U16Vector4: ...
    @property
    def qtso(self) -> U16Vector4: ...
    @property
    def qtsl(self) -> U16Vector4: ...
    @property
    def qtts(self) -> U16Vector4: ...
    @property
    def qttt(self) -> U16Vector4: ...
    @property
    def qttq(self) -> U16Vector4: ...
    @property
    def qtto(self) -> U16Vector4: ...
    @property
    def qttl(self) -> U16Vector4: ...
    @property
    def qtqs(self) -> U16Vector4: ...
    @property
    def qtqt(self) -> U16Vector4: ...
    @property
    def qtqq(self) -> U16Vector4: ...
    @property
    def qtqo(self) -> U16Vector4: ...
    @property
    def qtql(self) -> U16Vector4: ...
    @property
    def qtos(self) -> U16Vector4: ...
    @property
    def qtot(self) -> U16Vector4: ...
    @property
    def qtoq(self) -> U16Vector4: ...
    @property
    def qtoo(self) -> U16Vector4: ...
    @property
    def qtol(self) -> U16Vector4: ...
    @property
    def qtls(self) -> U16Vector4: ...
    @property
    def qtlt(self) -> U16Vector4: ...
    @property
    def qtlq(self) -> U16Vector4: ...
    @property
    def qtlo(self) -> U16Vector4: ...
    @property
    def qtll(self) -> U16Vector4: ...
    @property
    def qqss(self) -> U16Vector4: ...
    @property
    def qqst(self) -> U16Vector4: ...
    @property
    def qqsq(self) -> U16Vector4: ...
    @property
    def qqso(self) -> U16Vector4: ...
    @property
    def qqsl(self) -> U16Vector4: ...
    @property
    def qqts(self) -> U16Vector4: ...
    @property
    def qqtt(self) -> U16Vector4: ...
    @property
    def qqtq(self) -> U16Vector4: ...
    @property
    def qqto(self) -> U16Vector4: ...
    @property
    def qqtl(self) -> U16Vector4: ...
    @property
    def qqqs(self) -> U16Vector4: ...
    @property
    def qqqt(self) -> U16Vector4: ...
    @property
    def qqqq(self) -> U16Vector4: ...
    @property
    def qqqo(self) -> U16Vector4: ...
    @property
    def qqql(self) -> U16Vector4: ...
    @property
    def qqos(self) -> U16Vector4: ...
    @property
    def qqot(self) -> U16Vector4: ...
    @property
    def qqoq(self) -> U16Vector4: ...
    @property
    def qqoo(self) -> U16Vector4: ...
    @property
    def qqol(self) -> U16Vector4: ...
    @property
    def qqls(self) -> U16Vector4: ...
    @property
    def qqlt(self) -> U16Vector4: ...
    @property
    def qqlq(self) -> U16Vector4: ...
    @property
    def qqlo(self) -> U16Vector4: ...
    @property
    def qqll(self) -> U16Vector4: ...
    @property
    def qoss(self) -> U16Vector4: ...
    @property
    def qost(self) -> U16Vector4: ...
    @property
    def qosq(self) -> U16Vector4: ...
    @property
    def qoso(self) -> U16Vector4: ...
    @property
    def qosl(self) -> U16Vector4: ...
    @property
    def qots(self) -> U16Vector4: ...
    @property
    def qott(self) -> U16Vector4: ...
    @property
    def qotq(self) -> U16Vector4: ...
    @property
    def qoto(self) -> U16Vector4: ...
    @property
    def qotl(self) -> U16Vector4: ...
    @property
    def qoqs(self) -> U16Vector4: ...
    @property
    def qoqt(self) -> U16Vector4: ...
    @property
    def qoqq(self) -> U16Vector4: ...
    @property
    def qoqo(self) -> U16Vector4: ...
    @property
    def qoql(self) -> U16Vector4: ...
    @property
    def qoos(self) -> U16Vector4: ...
    @property
    def qoot(self) -> U16Vector4: ...
    @property
    def qooq(self) -> U16Vector4: ...
    @property
    def qooo(self) -> U16Vector4: ...
    @property
    def qool(self) -> U16Vector4: ...
    @property
    def qols(self) -> U16Vector4: ...
    @property
    def qolt(self) -> U16Vector4: ...
    @property
    def qolq(self) -> U16Vector4: ...
    @property
    def qolo(self) -> U16Vector4: ...
    @property
    def qoll(self) -> U16Vector4: ...
    @property
    def qlss(self) -> U16Vector4: ...
    @property
    def qlst(self) -> U16Vector4: ...
    @property
    def qlsq(self) -> U16Vector4: ...
    @property
    def qlso(self) -> U16Vector4: ...
    @property
    def qlsl(self) -> U16Vector4: ...
    @property
    def qlts(self) -> U16Vector4: ...
    @property
    def qltt(self) -> U16Vector4: ...
    @property
    def qltq(self) -> U16Vector4: ...
    @property
    def qlto(self) -> U16Vector4: ...
    @property
    def qltl(self) -> U16Vector4: ...
    @property
    def qlqs(self) -> U16Vector4: ...
    @property
    def qlqt(self) -> U16Vector4: ...
    @property
    def qlqq(self) -> U16Vector4: ...
    @property
    def qlqo(self) -> U16Vector4: ...
    @property
    def qlql(self) -> U16Vector4: ...
    @property
    def qlos(self) -> U16Vector4: ...
    @property
    def qlot(self) -> U16Vector4: ...
    @property
    def qloq(self) -> U16Vector4: ...
    @property
    def qloo(self) -> U16Vector4: ...
    @property
    def qlol(self) -> U16Vector4: ...
    @property
    def qlls(self) -> U16Vector4: ...
    @property
    def qllt(self) -> U16Vector4: ...
    @property
    def qllq(self) -> U16Vector4: ...
    @property
    def qllo(self) -> U16Vector4: ...
    @property
    def qlll(self) -> U16Vector4: ...
    @property
    def osss(self) -> U16Vector4: ...
    @property
    def osst(self) -> U16Vector4: ...
    @property
    def ossq(self) -> U16Vector4: ...
    @property
    def osso(self) -> U16Vector4: ...
    @property
    def ossl(self) -> U16Vector4: ...
    @property
    def osts(self) -> U16Vector4: ...
    @property
    def ostt(self) -> U16Vector4: ...
    @property
    def ostq(self) -> U16Vector4: ...
    @property
    def osto(self) -> U16Vector4: ...
    @property
    def ostl(self) -> U16Vector4: ...
    @property
    def osqs(self) -> U16Vector4: ...
    @property
    def osqt(self) -> U16Vector4: ...
    @property
    def osqq(self) -> U16Vector4: ...
    @property
    def osqo(self) -> U16Vector4: ...
    @property
    def osql(self) -> U16Vector4: ...
    @property
    def osos(self) -> U16Vector4: ...
    @property
    def osot(self) -> U16Vector4: ...
    @property
    def osoq(self) -> U16Vector4: ...
    @property
    def osoo(self) -> U16Vector4: ...
    @property
    def osol(self) -> U16Vector4: ...
    @property
    def osls(self) -> U16Vector4: ...
    @property
    def oslt(self) -> U16Vector4: ...
    @property
    def oslq(self) -> U16Vector4: ...
    @property
    def oslo(self) -> U16Vector4: ...
    @property
    def osll(self) -> U16Vector4: ...
    @property
    def otss(self) -> U16Vector4: ...
    @property
    def otst(self) -> U16Vector4: ...
    @property
    def otsq(self) -> U16Vector4: ...
    @property
    def otso(self) -> U16Vector4: ...
    @property
    def otsl(self) -> U16Vector4: ...
    @property
    def otts(self) -> U16Vector4: ...
    @property
    def ottt(self) -> U16Vector4: ...
    @property
    def ottq(self) -> U16Vector4: ...
    @property
    def otto(self) -> U16Vector4: ...
    @property
    def ottl(self) -> U16Vector4: ...
    @property
    def otqs(self) -> U16Vector4: ...
    @property
    def otqt(self) -> U16Vector4: ...
    @property
    def otqq(self) -> U16Vector4: ...
    @property
    def otqo(self) -> U16Vector4: ...
    @property
    def otql(self) -> U16Vector4: ...
    @property
    def otos(self) -> U16Vector4: ...
    @property
    def otot(self) -> U16Vector4: ...
    @property
    def otoq(self) -> U16Vector4: ...
    @property
    def otoo(self) -> U16Vector4: ...
    @property
    def otol(self) -> U16Vector4: ...
    @property
    def otls(self) -> U16Vector4: ...
    @property
    def otlt(self) -> U16Vector4: ...
    @property
    def otlq(self) -> U16Vector4: ...
    @property
    def otlo(self) -> U16Vector4: ...
    @property
    def otll(self) -> U16Vector4: ...
    @property
    def oqss(self) -> U16Vector4: ...
    @property
    def oqst(self) -> U16Vector4: ...
    @property
    def oqsq(self) -> U16Vector4: ...
    @property
    def oqso(self) -> U16Vector4: ...
    @property
    def oqsl(self) -> U16Vector4: ...
    @property
    def oqts(self) -> U16Vector4: ...
    @property
    def oqtt(self) -> U16Vector4: ...
    @property
    def oqtq(self) -> U16Vector4: ...
    @property
    def oqto(self) -> U16Vector4: ...
    @property
    def oqtl(self) -> U16Vector4: ...
    @property
    def oqqs(self) -> U16Vector4: ...
    @property
    def oqqt(self) -> U16Vector4: ...
    @property
    def oqqq(self) -> U16Vector4: ...
    @property
    def oqqo(self) -> U16Vector4: ...
    @property
    def oqql(self) -> U16Vector4: ...
    @property
    def oqos(self) -> U16Vector4: ...
    @property
    def oqot(self) -> U16Vector4: ...
    @property
    def oqoq(self) -> U16Vector4: ...
    @property
    def oqoo(self) -> U16Vector4: ...
    @property
    def oqol(self) -> U16Vector4: ...
    @property
    def oqls(self) -> U16Vector4: ...
    @property
    def oqlt(self) -> U16Vector4: ...
    @property
    def oqlq(self) -> U16Vector4: ...
    @property
    def oqlo(self) -> U16Vector4: ...
    @property
    def oqll(self) -> U16Vector4: ...
    @property
    def ooss(self) -> U16Vector4: ...
    @property
    def oost(self) -> U16Vector4: ...
    @property
    def oosq(self) -> U16Vector4: ...
    @property
    def ooso(self) -> U16Vector4: ...
    @property
    def oosl(self) -> U16Vector4: ...
    @property
    def oots(self) -> U16Vector4: ...
    @property
    def oott(self) -> U16Vector4: ...
    @property
    def ootq(self) -> U16Vector4: ...
    @property
    def ooto(self) -> U16Vector4: ...
    @property
    def ootl(self) -> U16Vector4: ...
    @property
    def ooqs(self) -> U16Vector4: ...
    @property
    def ooqt(self) -> U16Vector4: ...
    @property
    def ooqq(self) -> U16Vector4: ...
    @property
    def ooqo(self) -> U16Vector4: ...
    @property
    def ooql(self) -> U16Vector4: ...
    @property
    def ooos(self) -> U16Vector4: ...
    @property
    def ooot(self) -> U16Vector4: ...
    @property
    def oooq(self) -> U16Vector4: ...
    @property
    def ools(self) -> U16Vector4: ...
    @property
    def oolt(self) -> U16Vector4: ...
    @property
    def oolq(self) -> U16Vector4: ...
    @property
    def olss(self) -> U16Vector4: ...
    @property
    def olst(self) -> U16Vector4: ...
    @property
    def olsq(self) -> U16Vector4: ...
    @property
    def olso(self) -> U16Vector4: ...
    @property
    def olsl(self) -> U16Vector4: ...
    @property
    def olts(self) -> U16Vector4: ...
    @property
    def oltt(self) -> U16Vector4: ...
    @property
    def oltq(self) -> U16Vector4: ...
    @property
    def olto(self) -> U16Vector4: ...
    @property
    def oltl(self) -> U16Vector4: ...
    @property
    def olqs(self) -> U16Vector4: ...
    @property
    def olqt(self) -> U16Vector4: ...
    @property
    def olqq(self) -> U16Vector4: ...
    @property
    def olqo(self) -> U16Vector4: ...
    @property
    def olql(self) -> U16Vector4: ...
    @property
    def olos(self) -> U16Vector4: ...
    @property
    def olot(self) -> U16Vector4: ...
    @property
    def oloq(self) -> U16Vector4: ...
    @property
    def olls(self) -> U16Vector4: ...
    @property
    def ollt(self) -> U16Vector4: ...
    @property
    def ollq(self) -> U16Vector4: ...
    @property
    def lsss(self) -> U16Vector4: ...
    @property
    def lsst(self) -> U16Vector4: ...
    @property
    def lssq(self) -> U16Vector4: ...
    @property
    def lsso(self) -> U16Vector4: ...
    @property
    def lssl(self) -> U16Vector4: ...
    @property
    def lsts(self) -> U16Vector4: ...
    @property
    def lstt(self) -> U16Vector4: ...
    @property
    def lstq(self) -> U16Vector4: ...
    @property
    def lsto(self) -> U16Vector4: ...
    @property
    def lstl(self) -> U16Vector4: ...
    @property
    def lsqs(self) -> U16Vector4: ...
    @property
    def lsqt(self) -> U16Vector4: ...
    @property
    def lsqq(self) -> U16Vector4: ...
    @property
    def lsqo(self) -> U16Vector4: ...
    @property
    def lsql(self) -> U16Vector4: ...
    @property
    def lsos(self) -> U16Vector4: ...
    @property
    def lsot(self) -> U16Vector4: ...
    @property
    def lsoq(self) -> U16Vector4: ...
    @property
    def lsoo(self) -> U16Vector4: ...
    @property
    def lsol(self) -> U16Vector4: ...
    @property
    def lsls(self) -> U16Vector4: ...
    @property
    def lslt(self) -> U16Vector4: ...
    @property
    def lslq(self) -> U16Vector4: ...
    @property
    def lslo(self) -> U16Vector4: ...
    @property
    def lsll(self) -> U16Vector4: ...
    @property
    def ltss(self) -> U16Vector4: ...
    @property
    def ltst(self) -> U16Vector4: ...
    @property
    def ltsq(self) -> U16Vector4: ...
    @property
    def ltso(self) -> U16Vector4: ...
    @property
    def ltsl(self) -> U16Vector4: ...
    @property
    def ltts(self) -> U16Vector4: ...
    @property
    def lttt(self) -> U16Vector4: ...
    @property
    def lttq(self) -> U16Vector4: ...
    @property
    def ltto(self) -> U16Vector4: ...
    @property
    def lttl(self) -> U16Vector4: ...
    @property
    def ltqs(self) -> U16Vector4: ...
    @property
    def ltqt(self) -> U16Vector4: ...
    @property
    def ltqq(self) -> U16Vector4: ...
    @property
    def ltqo(self) -> U16Vector4: ...
    @property
    def ltql(self) -> U16Vector4: ...
    @property
    def ltos(self) -> U16Vector4: ...
    @property
    def ltot(self) -> U16Vector4: ...
    @property
    def ltoq(self) -> U16Vector4: ...
    @property
    def ltoo(self) -> U16Vector4: ...
    @property
    def ltol(self) -> U16Vector4: ...
    @property
    def ltls(self) -> U16Vector4: ...
    @property
    def ltlt(self) -> U16Vector4: ...
    @property
    def ltlq(self) -> U16Vector4: ...
    @property
    def ltlo(self) -> U16Vector4: ...
    @property
    def ltll(self) -> U16Vector4: ...
    @property
    def lqss(self) -> U16Vector4: ...
    @property
    def lqst(self) -> U16Vector4: ...
    @property
    def lqsq(self) -> U16Vector4: ...
    @property
    def lqso(self) -> U16Vector4: ...
    @property
    def lqsl(self) -> U16Vector4: ...
    @property
    def lqts(self) -> U16Vector4: ...
    @property
    def lqtt(self) -> U16Vector4: ...
    @property
    def lqtq(self) -> U16Vector4: ...
    @property
    def lqto(self) -> U16Vector4: ...
    @property
    def lqtl(self) -> U16Vector4: ...
    @property
    def lqqs(self) -> U16Vector4: ...
    @property
    def lqqt(self) -> U16Vector4: ...
    @property
    def lqqq(self) -> U16Vector4: ...
    @property
    def lqqo(self) -> U16Vector4: ...
    @property
    def lqql(self) -> U16Vector4: ...
    @property
    def lqos(self) -> U16Vector4: ...
    @property
    def lqot(self) -> U16Vector4: ...
    @property
    def lqoq(self) -> U16Vector4: ...
    @property
    def lqoo(self) -> U16Vector4: ...
    @property
    def lqol(self) -> U16Vector4: ...
    @property
    def lqls(self) -> U16Vector4: ...
    @property
    def lqlt(self) -> U16Vector4: ...
    @property
    def lqlq(self) -> U16Vector4: ...
    @property
    def lqlo(self) -> U16Vector4: ...
    @property
    def lqll(self) -> U16Vector4: ...
    @property
    def loss(self) -> U16Vector4: ...
    @property
    def lost(self) -> U16Vector4: ...
    @property
    def losq(self) -> U16Vector4: ...
    @property
    def loso(self) -> U16Vector4: ...
    @property
    def losl(self) -> U16Vector4: ...
    @property
    def lots(self) -> U16Vector4: ...
    @property
    def lott(self) -> U16Vector4: ...
    @property
    def lotq(self) -> U16Vector4: ...
    @property
    def loto(self) -> U16Vector4: ...
    @property
    def lotl(self) -> U16Vector4: ...
    @property
    def loqs(self) -> U16Vector4: ...
    @property
    def loqt(self) -> U16Vector4: ...
    @property
    def loqq(self) -> U16Vector4: ...
    @property
    def loqo(self) -> U16Vector4: ...
    @property
    def loql(self) -> U16Vector4: ...
    @property
    def loos(self) -> U16Vector4: ...
    @property
    def loot(self) -> U16Vector4: ...
    @property
    def looq(self) -> U16Vector4: ...
    @property
    def lols(self) -> U16Vector4: ...
    @property
    def lolt(self) -> U16Vector4: ...
    @property
    def lolq(self) -> U16Vector4: ...
    @property
    def llss(self) -> U16Vector4: ...
    @property
    def llst(self) -> U16Vector4: ...
    @property
    def llsq(self) -> U16Vector4: ...
    @property
    def llso(self) -> U16Vector4: ...
    @property
    def llsl(self) -> U16Vector4: ...
    @property
    def llts(self) -> U16Vector4: ...
    @property
    def lltt(self) -> U16Vector4: ...
    @property
    def lltq(self) -> U16Vector4: ...
    @property
    def llto(self) -> U16Vector4: ...
    @property
    def lltl(self) -> U16Vector4: ...
    @property
    def llqs(self) -> U16Vector4: ...
    @property
    def llqt(self) -> U16Vector4: ...
    @property
    def llqq(self) -> U16Vector4: ...
    @property
    def llqo(self) -> U16Vector4: ...
    @property
    def llql(self) -> U16Vector4: ...
    @property
    def llos(self) -> U16Vector4: ...
    @property
    def llot(self) -> U16Vector4: ...
    @property
    def lloq(self) -> U16Vector4: ...
    @property
    def llls(self) -> U16Vector4: ...
    @property
    def lllt(self) -> U16Vector4: ...
    @property
    def lllq(self) -> U16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U16Vector2: ...
    @property
    def uv(self) -> U16Vector2: ...
    @property
    def uo(self) -> U16Vector2: ...
    @property
    def ul(self) -> U16Vector2: ...
    @property
    def vu(self) -> U16Vector2: ...
    @property
    def vv(self) -> U16Vector2: ...
    @property
    def vo(self) -> U16Vector2: ...
    @property
    def vl(self) -> U16Vector2: ...
    @property
    def ou(self) -> U16Vector2: ...
    @property
    def ov(self) -> U16Vector2: ...
    @property
    def lu(self) -> U16Vector2: ...
    @property
    def lv(self) -> U16Vector2: ...
    @property
    def uuu(self) -> U16Vector3: ...
    @property
    def uuv(self) -> U16Vector3: ...
    @property
    def uuo(self) -> U16Vector3: ...
    @property
    def uul(self) -> U16Vector3: ...
    @property
    def uvu(self) -> U16Vector3: ...
    @property
    def uvv(self) -> U16Vector3: ...
    @property
    def uvo(self) -> U16Vector3: ...
    @property
    def uvl(self) -> U16Vector3: ...
    @property
    def uou(self) -> U16Vector3: ...
    @property
    def uov(self) -> U16Vector3: ...
    @property
    def uoo(self) -> U16Vector3: ...
    @property
    def uol(self) -> U16Vector3: ...
    @property
    def ulu(self) -> U16Vector3: ...
    @property
    def ulv(self) -> U16Vector3: ...
    @property
    def ulo(self) -> U16Vector3: ...
    @property
    def ull(self) -> U16Vector3: ...
    @property
    def vuu(self) -> U16Vector3: ...
    @property
    def vuv(self) -> U16Vector3: ...
    @property
    def vuo(self) -> U16Vector3: ...
    @property
    def vul(self) -> U16Vector3: ...
    @property
    def vvu(self) -> U16Vector3: ...
    @property
    def vvv(self) -> U16Vector3: ...
    @property
    def vvo(self) -> U16Vector3: ...
    @property
    def vvl(self) -> U16Vector3: ...
    @property
    def vou(self) -> U16Vector3: ...
    @property
    def vov(self) -> U16Vector3: ...
    @property
    def voo(self) -> U16Vector3: ...
    @property
    def vol(self) -> U16Vector3: ...
    @property
    def vlu(self) -> U16Vector3: ...
    @property
    def vlv(self) -> U16Vector3: ...
    @property
    def vlo(self) -> U16Vector3: ...
    @property
    def vll(self) -> U16Vector3: ...
    @property
    def ouu(self) -> U16Vector3: ...
    @property
    def ouv(self) -> U16Vector3: ...
    @property
    def ouo(self) -> U16Vector3: ...
    @property
    def oul(self) -> U16Vector3: ...
    @property
    def ovu(self) -> U16Vector3: ...
    @property
    def ovv(self) -> U16Vector3: ...
    @property
    def ovo(self) -> U16Vector3: ...
    @property
    def ovl(self) -> U16Vector3: ...
    @property
    def oou(self) -> U16Vector3: ...
    @property
    def oov(self) -> U16Vector3: ...
    @property
    def olu(self) -> U16Vector3: ...
    @property
    def olv(self) -> U16Vector3: ...
    @property
    def luu(self) -> U16Vector3: ...
    @property
    def luv(self) -> U16Vector3: ...
    @property
    def luo(self) -> U16Vector3: ...
    @property
    def lul(self) -> U16Vector3: ...
    @property
    def lvu(self) -> U16Vector3: ...
    @property
    def lvv(self) -> U16Vector3: ...
    @property
    def lvo(self) -> U16Vector3: ...
    @property
    def lvl(self) -> U16Vector3: ...
    @property
    def lou(self) -> U16Vector3: ...
    @property
    def lov(self) -> U16Vector3: ...
    @property
    def llu(self) -> U16Vector3: ...
    @property
    def llv(self) -> U16Vector3: ...
    @property
    def uuuu(self) -> U16Vector4: ...
    @property
    def uuuv(self) -> U16Vector4: ...
    @property
    def uuuo(self) -> U16Vector4: ...
    @property
    def uuul(self) -> U16Vector4: ...
    @property
    def uuvu(self) -> U16Vector4: ...
    @property
    def uuvv(self) -> U16Vector4: ...
    @property
    def uuvo(self) -> U16Vector4: ...
    @property
    def uuvl(self) -> U16Vector4: ...
    @property
    def uuou(self) -> U16Vector4: ...
    @property
    def uuov(self) -> U16Vector4: ...
    @property
    def uuoo(self) -> U16Vector4: ...
    @property
    def uuol(self) -> U16Vector4: ...
    @property
    def uulu(self) -> U16Vector4: ...
    @property
    def uulv(self) -> U16Vector4: ...
    @property
    def uulo(self) -> U16Vector4: ...
    @property
    def uull(self) -> U16Vector4: ...
    @property
    def uvuu(self) -> U16Vector4: ...
    @property
    def uvuv(self) -> U16Vector4: ...
    @property
    def uvuo(self) -> U16Vector4: ...
    @property
    def uvul(self) -> U16Vector4: ...
    @property
    def uvvu(self) -> U16Vector4: ...
    @property
    def uvvv(self) -> U16Vector4: ...
    @property
    def uvvo(self) -> U16Vector4: ...
    @property
    def uvvl(self) -> U16Vector4: ...
    @property
    def uvou(self) -> U16Vector4: ...
    @property
    def uvov(self) -> U16Vector4: ...
    @property
    def uvoo(self) -> U16Vector4: ...
    @property
    def uvol(self) -> U16Vector4: ...
    @property
    def uvlu(self) -> U16Vector4: ...
    @property
    def uvlv(self) -> U16Vector4: ...
    @property
    def uvlo(self) -> U16Vector4: ...
    @property
    def uvll(self) -> U16Vector4: ...
    @property
    def uouu(self) -> U16Vector4: ...
    @property
    def uouv(self) -> U16Vector4: ...
    @property
    def uouo(self) -> U16Vector4: ...
    @property
    def uoul(self) -> U16Vector4: ...
    @property
    def uovu(self) -> U16Vector4: ...
    @property
    def uovv(self) -> U16Vector4: ...
    @property
    def uovo(self) -> U16Vector4: ...
    @property
    def uovl(self) -> U16Vector4: ...
    @property
    def uoou(self) -> U16Vector4: ...
    @property
    def uoov(self) -> U16Vector4: ...
    @property
    def uooo(self) -> U16Vector4: ...
    @property
    def uool(self) -> U16Vector4: ...
    @property
    def uolu(self) -> U16Vector4: ...
    @property
    def uolv(self) -> U16Vector4: ...
    @property
    def uolo(self) -> U16Vector4: ...
    @property
    def uoll(self) -> U16Vector4: ...
    @property
    def uluu(self) -> U16Vector4: ...
    @property
    def uluv(self) -> U16Vector4: ...
    @property
    def uluo(self) -> U16Vector4: ...
    @property
    def ulul(self) -> U16Vector4: ...
    @property
    def ulvu(self) -> U16Vector4: ...
    @property
    def ulvv(self) -> U16Vector4: ...
    @property
    def ulvo(self) -> U16Vector4: ...
    @property
    def ulvl(self) -> U16Vector4: ...
    @property
    def ulou(self) -> U16Vector4: ...
    @property
    def ulov(self) -> U16Vector4: ...
    @property
    def uloo(self) -> U16Vector4: ...
    @property
    def ulol(self) -> U16Vector4: ...
    @property
    def ullu(self) -> U16Vector4: ...
    @property
    def ullv(self) -> U16Vector4: ...
    @property
    def ullo(self) -> U16Vector4: ...
    @property
    def ulll(self) -> U16Vector4: ...
    @property
    def vuuu(self) -> U16Vector4: ...
    @property
    def vuuv(self) -> U16Vector4: ...
    @property
    def vuuo(self) -> U16Vector4: ...
    @property
    def vuul(self) -> U16Vector4: ...
    @property
    def vuvu(self) -> U16Vector4: ...
    @property
    def vuvv(self) -> U16Vector4: ...
    @property
    def vuvo(self) -> U16Vector4: ...
    @property
    def vuvl(self) -> U16Vector4: ...
    @property
    def vuou(self) -> U16Vector4: ...
    @property
    def vuov(self) -> U16Vector4: ...
    @property
    def vuoo(self) -> U16Vector4: ...
    @property
    def vuol(self) -> U16Vector4: ...
    @property
    def vulu(self) -> U16Vector4: ...
    @property
    def vulv(self) -> U16Vector4: ...
    @property
    def vulo(self) -> U16Vector4: ...
    @property
    def vull(self) -> U16Vector4: ...
    @property
    def vvuu(self) -> U16Vector4: ...
    @property
    def vvuv(self) -> U16Vector4: ...
    @property
    def vvuo(self) -> U16Vector4: ...
    @property
    def vvul(self) -> U16Vector4: ...
    @property
    def vvvu(self) -> U16Vector4: ...
    @property
    def vvvv(self) -> U16Vector4: ...
    @property
    def vvvo(self) -> U16Vector4: ...
    @property
    def vvvl(self) -> U16Vector4: ...
    @property
    def vvou(self) -> U16Vector4: ...
    @property
    def vvov(self) -> U16Vector4: ...
    @property
    def vvoo(self) -> U16Vector4: ...
    @property
    def vvol(self) -> U16Vector4: ...
    @property
    def vvlu(self) -> U16Vector4: ...
    @property
    def vvlv(self) -> U16Vector4: ...
    @property
    def vvlo(self) -> U16Vector4: ...
    @property
    def vvll(self) -> U16Vector4: ...
    @property
    def vouu(self) -> U16Vector4: ...
    @property
    def vouv(self) -> U16Vector4: ...
    @property
    def vouo(self) -> U16Vector4: ...
    @property
    def voul(self) -> U16Vector4: ...
    @property
    def vovu(self) -> U16Vector4: ...
    @property
    def vovv(self) -> U16Vector4: ...
    @property
    def vovo(self) -> U16Vector4: ...
    @property
    def vovl(self) -> U16Vector4: ...
    @property
    def voou(self) -> U16Vector4: ...
    @property
    def voov(self) -> U16Vector4: ...
    @property
    def vooo(self) -> U16Vector4: ...
    @property
    def vool(self) -> U16Vector4: ...
    @property
    def volu(self) -> U16Vector4: ...
    @property
    def volv(self) -> U16Vector4: ...
    @property
    def volo(self) -> U16Vector4: ...
    @property
    def voll(self) -> U16Vector4: ...
    @property
    def vluu(self) -> U16Vector4: ...
    @property
    def vluv(self) -> U16Vector4: ...
    @property
    def vluo(self) -> U16Vector4: ...
    @property
    def vlul(self) -> U16Vector4: ...
    @property
    def vlvu(self) -> U16Vector4: ...
    @property
    def vlvv(self) -> U16Vector4: ...
    @property
    def vlvo(self) -> U16Vector4: ...
    @property
    def vlvl(self) -> U16Vector4: ...
    @property
    def vlou(self) -> U16Vector4: ...
    @property
    def vlov(self) -> U16Vector4: ...
    @property
    def vloo(self) -> U16Vector4: ...
    @property
    def vlol(self) -> U16Vector4: ...
    @property
    def vllu(self) -> U16Vector4: ...
    @property
    def vllv(self) -> U16Vector4: ...
    @property
    def vllo(self) -> U16Vector4: ...
    @property
    def vlll(self) -> U16Vector4: ...
    @property
    def ouuu(self) -> U16Vector4: ...
    @property
    def ouuv(self) -> U16Vector4: ...
    @property
    def ouuo(self) -> U16Vector4: ...
    @property
    def ouul(self) -> U16Vector4: ...
    @property
    def ouvu(self) -> U16Vector4: ...
    @property
    def ouvv(self) -> U16Vector4: ...
    @property
    def ouvo(self) -> U16Vector4: ...
    @property
    def ouvl(self) -> U16Vector4: ...
    @property
    def ouou(self) -> U16Vector4: ...
    @property
    def ouov(self) -> U16Vector4: ...
    @property
    def ouoo(self) -> U16Vector4: ...
    @property
    def ouol(self) -> U16Vector4: ...
    @property
    def oulu(self) -> U16Vector4: ...
    @property
    def oulv(self) -> U16Vector4: ...
    @property
    def oulo(self) -> U16Vector4: ...
    @property
    def oull(self) -> U16Vector4: ...
    @property
    def ovuu(self) -> U16Vector4: ...
    @property
    def ovuv(self) -> U16Vector4: ...
    @property
    def ovuo(self) -> U16Vector4: ...
    @property
    def ovul(self) -> U16Vector4: ...
    @property
    def ovvu(self) -> U16Vector4: ...
    @property
    def ovvv(self) -> U16Vector4: ...
    @property
    def ovvo(self) -> U16Vector4: ...
    @property
    def ovvl(self) -> U16Vector4: ...
    @property
    def ovou(self) -> U16Vector4: ...
    @property
    def ovov(self) -> U16Vector4: ...
    @property
    def ovoo(self) -> U16Vector4: ...
    @property
    def ovol(self) -> U16Vector4: ...
    @property
    def ovlu(self) -> U16Vector4: ...
    @property
    def ovlv(self) -> U16Vector4: ...
    @property
    def ovlo(self) -> U16Vector4: ...
    @property
    def ovll(self) -> U16Vector4: ...
    @property
    def oouu(self) -> U16Vector4: ...
    @property
    def oouv(self) -> U16Vector4: ...
    @property
    def oouo(self) -> U16Vector4: ...
    @property
    def ooul(self) -> U16Vector4: ...
    @property
    def oovu(self) -> U16Vector4: ...
    @property
    def oovv(self) -> U16Vector4: ...
    @property
    def oovo(self) -> U16Vector4: ...
    @property
    def oovl(self) -> U16Vector4: ...
    @property
    def ooou(self) -> U16Vector4: ...
    @property
    def ooov(self) -> U16Vector4: ...
    @property
    def oolu(self) -> U16Vector4: ...
    @property
    def oolv(self) -> U16Vector4: ...
    @property
    def oluu(self) -> U16Vector4: ...
    @property
    def oluv(self) -> U16Vector4: ...
    @property
    def oluo(self) -> U16Vector4: ...
    @property
    def olul(self) -> U16Vector4: ...
    @property
    def olvu(self) -> U16Vector4: ...
    @property
    def olvv(self) -> U16Vector4: ...
    @property
    def olvo(self) -> U16Vector4: ...
    @property
    def olvl(self) -> U16Vector4: ...
    @property
    def olou(self) -> U16Vector4: ...
    @property
    def olov(self) -> U16Vector4: ...
    @property
    def ollu(self) -> U16Vector4: ...
    @property
    def ollv(self) -> U16Vector4: ...
    @property
    def luuu(self) -> U16Vector4: ...
    @property
    def luuv(self) -> U16Vector4: ...
    @property
    def luuo(self) -> U16Vector4: ...
    @property
    def luul(self) -> U16Vector4: ...
    @property
    def luvu(self) -> U16Vector4: ...
    @property
    def luvv(self) -> U16Vector4: ...
    @property
    def luvo(self) -> U16Vector4: ...
    @property
    def luvl(self) -> U16Vector4: ...
    @property
    def luou(self) -> U16Vector4: ...
    @property
    def luov(self) -> U16Vector4: ...
    @property
    def luoo(self) -> U16Vector4: ...
    @property
    def luol(self) -> U16Vector4: ...
    @property
    def lulu(self) -> U16Vector4: ...
    @property
    def lulv(self) -> U16Vector4: ...
    @property
    def lulo(self) -> U16Vector4: ...
    @property
    def lull(self) -> U16Vector4: ...
    @property
    def lvuu(self) -> U16Vector4: ...
    @property
    def lvuv(self) -> U16Vector4: ...
    @property
    def lvuo(self) -> U16Vector4: ...
    @property
    def lvul(self) -> U16Vector4: ...
    @property
    def lvvu(self) -> U16Vector4: ...
    @property
    def lvvv(self) -> U16Vector4: ...
    @property
    def lvvo(self) -> U16Vector4: ...
    @property
    def lvvl(self) -> U16Vector4: ...
    @property
    def lvou(self) -> U16Vector4: ...
    @property
    def lvov(self) -> U16Vector4: ...
    @property
    def lvoo(self) -> U16Vector4: ...
    @property
    def lvol(self) -> U16Vector4: ...
    @property
    def lvlu(self) -> U16Vector4: ...
    @property
    def lvlv(self) -> U16Vector4: ...
    @property
    def lvlo(self) -> U16Vector4: ...
    @property
    def lvll(self) -> U16Vector4: ...
    @property
    def louu(self) -> U16Vector4: ...
    @property
    def louv(self) -> U16Vector4: ...
    @property
    def louo(self) -> U16Vector4: ...
    @property
    def loul(self) -> U16Vector4: ...
    @property
    def lovu(self) -> U16Vector4: ...
    @property
    def lovv(self) -> U16Vector4: ...
    @property
    def lovo(self) -> U16Vector4: ...
    @property
    def lovl(self) -> U16Vector4: ...
    @property
    def loou(self) -> U16Vector4: ...
    @property
    def loov(self) -> U16Vector4: ...
    @property
    def lolu(self) -> U16Vector4: ...
    @property
    def lolv(self) -> U16Vector4: ...
    @property
    def lluu(self) -> U16Vector4: ...
    @property
    def lluv(self) -> U16Vector4: ...
    @property
    def lluo(self) -> U16Vector4: ...
    @property
    def llul(self) -> U16Vector4: ...
    @property
    def llvu(self) -> U16Vector4: ...
    @property
    def llvv(self) -> U16Vector4: ...
    @property
    def llvo(self) -> U16Vector4: ...
    @property
    def llvl(self) -> U16Vector4: ...
    @property
    def llou(self) -> U16Vector4: ...
    @property
    def llov(self) -> U16Vector4: ...
    @property
    def lllu(self) -> U16Vector4: ...
    @property
    def lllv(self) -> U16Vector4: ...
    @property
    def oo(self) -> U16Vector2: ...
    @property
    def ol(self) -> U16Vector2: ...
    @property
    def lo(self) -> U16Vector2: ...
    @property
    def ll(self) -> U16Vector2: ...
    @property
    def ooo(self) -> U16Vector3: ...
    @property
    def ool(self) -> U16Vector3: ...
    @property
    def olo(self) -> U16Vector3: ...
    @property
    def oll(self) -> U16Vector3: ...
    @property
    def loo(self) -> U16Vector3: ...
    @property
    def lol(self) -> U16Vector3: ...
    @property
    def llo(self) -> U16Vector3: ...
    @property
    def lll(self) -> U16Vector3: ...
    @property
    def oooo(self) -> U16Vector4: ...
    @property
    def oool(self) -> U16Vector4: ...
    @property
    def oolo(self) -> U16Vector4: ...
    @property
    def ooll(self) -> U16Vector4: ...
    @property
    def oloo(self) -> U16Vector4: ...
    @property
    def olol(self) -> U16Vector4: ...
    @property
    def ollo(self) -> U16Vector4: ...
    @property
    def olll(self) -> U16Vector4: ...
    @property
    def looo(self) -> U16Vector4: ...
    @property
    def lool(self) -> U16Vector4: ...
    @property
    def lolo(self) -> U16Vector4: ...
    @property
    def loll(self) -> U16Vector4: ...
    @property
    def lloo(self) -> U16Vector4: ...
    @property
    def llol(self) -> U16Vector4: ...
    @property
    def lllo(self) -> U16Vector4: ...
    @property
    def llll(self) -> U16Vector4: ...
    def min(self, n: Number, /) -> U16Vector3: ...
    def max(self, n: Number, /) -> U16Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> U16Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U16Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector3: ...

@final
class U16Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U16Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U16Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> U16Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> U16Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[U16Vector3]: ...

@final
class I32Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __add__(self, other: Number) -> I32Vector3: ...
    @overload
    def __radd__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __radd__(self, other: Number) -> I32Vector3: ...
    @overload
    def __sub__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __sub__(self, other: Number) -> I32Vector3: ...
    @overload
    def __rsub__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> I32Vector3: ...
    @overload
    def __mul__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __mul__(self, other: Number) -> I32Vector3: ...
    @overload
    def __rmul__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> I32Vector3: ...
    @overload
    def __truediv__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> I32Vector3: ...
    @overload
    def __rtruediv__(self, other: I32Vector3) -> I32Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> I32Vector3: ...
    def __neg__(self) -> I32Vector3: ...
    def __abs__(self) -> I32Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> I32Vector2: ...
    @property
    def xy(self) -> I32Vector2: ...
    @property
    def xz(self) -> I32Vector2: ...
    @property
    def xo(self) -> I32Vector2: ...
    @property
    def xl(self) -> I32Vector2: ...
    @property
    def yx(self) -> I32Vector2: ...
    @property
    def yy(self) -> I32Vector2: ...
    @property
    def yz(self) -> I32Vector2: ...
    @property
    def yo(self) -> I32Vector2: ...
    @property
    def yl(self) -> I32Vector2: ...
    @property
    def zx(self) -> I32Vector2: ...
    @property
    def zy(self) -> I32Vector2: ...
    @property
    def zz(self) -> I32Vector2: ...
    @property
    def zo(self) -> I32Vector2: ...
    @property
    def zl(self) -> I32Vector2: ...
    @property
    def ox(self) -> I32Vector2: ...
    @property
    def oy(self) -> I32Vector2: ...
    @property
    def oz(self) -> I32Vector2: ...
    @property
    def lx(self) -> I32Vector2: ...
    @property
    def ly(self) -> I32Vector2: ...
    @property
    def lz(self) -> I32Vector2: ...
    @property
    def xxx(self) -> I32Vector3: ...
    @property
    def xxy(self) -> I32Vector3: ...
    @property
    def xxz(self) -> I32Vector3: ...
    @property
    def xxo(self) -> I32Vector3: ...
    @property
    def xxl(self) -> I32Vector3: ...
    @property
    def xyx(self) -> I32Vector3: ...
    @property
    def xyy(self) -> I32Vector3: ...
    @property
    def xyz(self) -> I32Vector3: ...
    @property
    def xyo(self) -> I32Vector3: ...
    @property
    def xyl(self) -> I32Vector3: ...
    @property
    def xzx(self) -> I32Vector3: ...
    @property
    def xzy(self) -> I32Vector3: ...
    @property
    def xzz(self) -> I32Vector3: ...
    @property
    def xzo(self) -> I32Vector3: ...
    @property
    def xzl(self) -> I32Vector3: ...
    @property
    def xox(self) -> I32Vector3: ...
    @property
    def xoy(self) -> I32Vector3: ...
    @property
    def xoz(self) -> I32Vector3: ...
    @property
    def xoo(self) -> I32Vector3: ...
    @property
    def xol(self) -> I32Vector3: ...
    @property
    def xlx(self) -> I32Vector3: ...
    @property
    def xly(self) -> I32Vector3: ...
    @property
    def xlz(self) -> I32Vector3: ...
    @property
    def xlo(self) -> I32Vector3: ...
    @property
    def xll(self) -> I32Vector3: ...
    @property
    def yxx(self) -> I32Vector3: ...
    @property
    def yxy(self) -> I32Vector3: ...
    @property
    def yxz(self) -> I32Vector3: ...
    @property
    def yxo(self) -> I32Vector3: ...
    @property
    def yxl(self) -> I32Vector3: ...
    @property
    def yyx(self) -> I32Vector3: ...
    @property
    def yyy(self) -> I32Vector3: ...
    @property
    def yyz(self) -> I32Vector3: ...
    @property
    def yyo(self) -> I32Vector3: ...
    @property
    def yyl(self) -> I32Vector3: ...
    @property
    def yzx(self) -> I32Vector3: ...
    @property
    def yzy(self) -> I32Vector3: ...
    @property
    def yzz(self) -> I32Vector3: ...
    @property
    def yzo(self) -> I32Vector3: ...
    @property
    def yzl(self) -> I32Vector3: ...
    @property
    def yox(self) -> I32Vector3: ...
    @property
    def yoy(self) -> I32Vector3: ...
    @property
    def yoz(self) -> I32Vector3: ...
    @property
    def yoo(self) -> I32Vector3: ...
    @property
    def yol(self) -> I32Vector3: ...
    @property
    def ylx(self) -> I32Vector3: ...
    @property
    def yly(self) -> I32Vector3: ...
    @property
    def ylz(self) -> I32Vector3: ...
    @property
    def ylo(self) -> I32Vector3: ...
    @property
    def yll(self) -> I32Vector3: ...
    @property
    def zxx(self) -> I32Vector3: ...
    @property
    def zxy(self) -> I32Vector3: ...
    @property
    def zxz(self) -> I32Vector3: ...
    @property
    def zxo(self) -> I32Vector3: ...
    @property
    def zxl(self) -> I32Vector3: ...
    @property
    def zyx(self) -> I32Vector3: ...
    @property
    def zyy(self) -> I32Vector3: ...
    @property
    def zyz(self) -> I32Vector3: ...
    @property
    def zyo(self) -> I32Vector3: ...
    @property
    def zyl(self) -> I32Vector3: ...
    @property
    def zzx(self) -> I32Vector3: ...
    @property
    def zzy(self) -> I32Vector3: ...
    @property
    def zzz(self) -> I32Vector3: ...
    @property
    def zzo(self) -> I32Vector3: ...
    @property
    def zzl(self) -> I32Vector3: ...
    @property
    def zox(self) -> I32Vector3: ...
    @property
    def zoy(self) -> I32Vector3: ...
    @property
    def zoz(self) -> I32Vector3: ...
    @property
    def zoo(self) -> I32Vector3: ...
    @property
    def zol(self) -> I32Vector3: ...
    @property
    def zlx(self) -> I32Vector3: ...
    @property
    def zly(self) -> I32Vector3: ...
    @property
    def zlz(self) -> I32Vector3: ...
    @property
    def zlo(self) -> I32Vector3: ...
    @property
    def zll(self) -> I32Vector3: ...
    @property
    def oxx(self) -> I32Vector3: ...
    @property
    def oxy(self) -> I32Vector3: ...
    @property
    def oxz(self) -> I32Vector3: ...
    @property
    def oxo(self) -> I32Vector3: ...
    @property
    def oxl(self) -> I32Vector3: ...
    @property
    def oyx(self) -> I32Vector3: ...
    @property
    def oyy(self) -> I32Vector3: ...
    @property
    def oyz(self) -> I32Vector3: ...
    @property
    def oyo(self) -> I32Vector3: ...
    @property
    def oyl(self) -> I32Vector3: ...
    @property
    def ozx(self) -> I32Vector3: ...
    @property
    def ozy(self) -> I32Vector3: ...
    @property
    def ozz(self) -> I32Vector3: ...
    @property
    def ozo(self) -> I32Vector3: ...
    @property
    def ozl(self) -> I32Vector3: ...
    @property
    def oox(self) -> I32Vector3: ...
    @property
    def ooy(self) -> I32Vector3: ...
    @property
    def ooz(self) -> I32Vector3: ...
    @property
    def olx(self) -> I32Vector3: ...
    @property
    def oly(self) -> I32Vector3: ...
    @property
    def olz(self) -> I32Vector3: ...
    @property
    def lxx(self) -> I32Vector3: ...
    @property
    def lxy(self) -> I32Vector3: ...
    @property
    def lxz(self) -> I32Vector3: ...
    @property
    def lxo(self) -> I32Vector3: ...
    @property
    def lxl(self) -> I32Vector3: ...
    @property
    def lyx(self) -> I32Vector3: ...
    @property
    def lyy(self) -> I32Vector3: ...
    @property
    def lyz(self) -> I32Vector3: ...
    @property
    def lyo(self) -> I32Vector3: ...
    @property
    def lyl(self) -> I32Vector3: ...
    @property
    def lzx(self) -> I32Vector3: ...
    @property
    def lzy(self) -> I32Vector3: ...
    @property
    def lzz(self) -> I32Vector3: ...
    @property
    def lzo(self) -> I32Vector3: ...
    @property
    def lzl(self) -> I32Vector3: ...
    @property
    def lox(self) -> I32Vector3: ...
    @property
    def loy(self) -> I32Vector3: ...
    @property
    def loz(self) -> I32Vector3: ...
    @property
    def llx(self) -> I32Vector3: ...
    @property
    def lly(self) -> I32Vector3: ...
    @property
    def llz(self) -> I32Vector3: ...
    @property
    def xxxx(self) -> I32Vector4: ...
    @property
    def xxxy(self) -> I32Vector4: ...
    @property
    def xxxz(self) -> I32Vector4: ...
    @property
    def xxxo(self) -> I32Vector4: ...
    @property
    def xxxl(self) -> I32Vector4: ...
    @property
    def xxyx(self) -> I32Vector4: ...
    @property
    def xxyy(self) -> I32Vector4: ...
    @property
    def xxyz(self) -> I32Vector4: ...
    @property
    def xxyo(self) -> I32Vector4: ...
    @property
    def xxyl(self) -> I32Vector4: ...
    @property
    def xxzx(self) -> I32Vector4: ...
    @property
    def xxzy(self) -> I32Vector4: ...
    @property
    def xxzz(self) -> I32Vector4: ...
    @property
    def xxzo(self) -> I32Vector4: ...
    @property
    def xxzl(self) -> I32Vector4: ...
    @property
    def xxox(self) -> I32Vector4: ...
    @property
    def xxoy(self) -> I32Vector4: ...
    @property
    def xxoz(self) -> I32Vector4: ...
    @property
    def xxoo(self) -> I32Vector4: ...
    @property
    def xxol(self) -> I32Vector4: ...
    @property
    def xxlx(self) -> I32Vector4: ...
    @property
    def xxly(self) -> I32Vector4: ...
    @property
    def xxlz(self) -> I32Vector4: ...
    @property
    def xxlo(self) -> I32Vector4: ...
    @property
    def xxll(self) -> I32Vector4: ...
    @property
    def xyxx(self) -> I32Vector4: ...
    @property
    def xyxy(self) -> I32Vector4: ...
    @property
    def xyxz(self) -> I32Vector4: ...
    @property
    def xyxo(self) -> I32Vector4: ...
    @property
    def xyxl(self) -> I32Vector4: ...
    @property
    def xyyx(self) -> I32Vector4: ...
    @property
    def xyyy(self) -> I32Vector4: ...
    @property
    def xyyz(self) -> I32Vector4: ...
    @property
    def xyyo(self) -> I32Vector4: ...
    @property
    def xyyl(self) -> I32Vector4: ...
    @property
    def xyzx(self) -> I32Vector4: ...
    @property
    def xyzy(self) -> I32Vector4: ...
    @property
    def xyzz(self) -> I32Vector4: ...
    @property
    def xyzo(self) -> I32Vector4: ...
    @property
    def xyzl(self) -> I32Vector4: ...
    @property
    def xyox(self) -> I32Vector4: ...
    @property
    def xyoy(self) -> I32Vector4: ...
    @property
    def xyoz(self) -> I32Vector4: ...
    @property
    def xyoo(self) -> I32Vector4: ...
    @property
    def xyol(self) -> I32Vector4: ...
    @property
    def xylx(self) -> I32Vector4: ...
    @property
    def xyly(self) -> I32Vector4: ...
    @property
    def xylz(self) -> I32Vector4: ...
    @property
    def xylo(self) -> I32Vector4: ...
    @property
    def xyll(self) -> I32Vector4: ...
    @property
    def xzxx(self) -> I32Vector4: ...
    @property
    def xzxy(self) -> I32Vector4: ...
    @property
    def xzxz(self) -> I32Vector4: ...
    @property
    def xzxo(self) -> I32Vector4: ...
    @property
    def xzxl(self) -> I32Vector4: ...
    @property
    def xzyx(self) -> I32Vector4: ...
    @property
    def xzyy(self) -> I32Vector4: ...
    @property
    def xzyz(self) -> I32Vector4: ...
    @property
    def xzyo(self) -> I32Vector4: ...
    @property
    def xzyl(self) -> I32Vector4: ...
    @property
    def xzzx(self) -> I32Vector4: ...
    @property
    def xzzy(self) -> I32Vector4: ...
    @property
    def xzzz(self) -> I32Vector4: ...
    @property
    def xzzo(self) -> I32Vector4: ...
    @property
    def xzzl(self) -> I32Vector4: ...
    @property
    def xzox(self) -> I32Vector4: ...
    @property
    def xzoy(self) -> I32Vector4: ...
    @property
    def xzoz(self) -> I32Vector4: ...
    @property
    def xzoo(self) -> I32Vector4: ...
    @property
    def xzol(self) -> I32Vector4: ...
    @property
    def xzlx(self) -> I32Vector4: ...
    @property
    def xzly(self) -> I32Vector4: ...
    @property
    def xzlz(self) -> I32Vector4: ...
    @property
    def xzlo(self) -> I32Vector4: ...
    @property
    def xzll(self) -> I32Vector4: ...
    @property
    def xoxx(self) -> I32Vector4: ...
    @property
    def xoxy(self) -> I32Vector4: ...
    @property
    def xoxz(self) -> I32Vector4: ...
    @property
    def xoxo(self) -> I32Vector4: ...
    @property
    def xoxl(self) -> I32Vector4: ...
    @property
    def xoyx(self) -> I32Vector4: ...
    @property
    def xoyy(self) -> I32Vector4: ...
    @property
    def xoyz(self) -> I32Vector4: ...
    @property
    def xoyo(self) -> I32Vector4: ...
    @property
    def xoyl(self) -> I32Vector4: ...
    @property
    def xozx(self) -> I32Vector4: ...
    @property
    def xozy(self) -> I32Vector4: ...
    @property
    def xozz(self) -> I32Vector4: ...
    @property
    def xozo(self) -> I32Vector4: ...
    @property
    def xozl(self) -> I32Vector4: ...
    @property
    def xoox(self) -> I32Vector4: ...
    @property
    def xooy(self) -> I32Vector4: ...
    @property
    def xooz(self) -> I32Vector4: ...
    @property
    def xooo(self) -> I32Vector4: ...
    @property
    def xool(self) -> I32Vector4: ...
    @property
    def xolx(self) -> I32Vector4: ...
    @property
    def xoly(self) -> I32Vector4: ...
    @property
    def xolz(self) -> I32Vector4: ...
    @property
    def xolo(self) -> I32Vector4: ...
    @property
    def xoll(self) -> I32Vector4: ...
    @property
    def xlxx(self) -> I32Vector4: ...
    @property
    def xlxy(self) -> I32Vector4: ...
    @property
    def xlxz(self) -> I32Vector4: ...
    @property
    def xlxo(self) -> I32Vector4: ...
    @property
    def xlxl(self) -> I32Vector4: ...
    @property
    def xlyx(self) -> I32Vector4: ...
    @property
    def xlyy(self) -> I32Vector4: ...
    @property
    def xlyz(self) -> I32Vector4: ...
    @property
    def xlyo(self) -> I32Vector4: ...
    @property
    def xlyl(self) -> I32Vector4: ...
    @property
    def xlzx(self) -> I32Vector4: ...
    @property
    def xlzy(self) -> I32Vector4: ...
    @property
    def xlzz(self) -> I32Vector4: ...
    @property
    def xlzo(self) -> I32Vector4: ...
    @property
    def xlzl(self) -> I32Vector4: ...
    @property
    def xlox(self) -> I32Vector4: ...
    @property
    def xloy(self) -> I32Vector4: ...
    @property
    def xloz(self) -> I32Vector4: ...
    @property
    def xloo(self) -> I32Vector4: ...
    @property
    def xlol(self) -> I32Vector4: ...
    @property
    def xllx(self) -> I32Vector4: ...
    @property
    def xlly(self) -> I32Vector4: ...
    @property
    def xllz(self) -> I32Vector4: ...
    @property
    def xllo(self) -> I32Vector4: ...
    @property
    def xlll(self) -> I32Vector4: ...
    @property
    def yxxx(self) -> I32Vector4: ...
    @property
    def yxxy(self) -> I32Vector4: ...
    @property
    def yxxz(self) -> I32Vector4: ...
    @property
    def yxxo(self) -> I32Vector4: ...
    @property
    def yxxl(self) -> I32Vector4: ...
    @property
    def yxyx(self) -> I32Vector4: ...
    @property
    def yxyy(self) -> I32Vector4: ...
    @property
    def yxyz(self) -> I32Vector4: ...
    @property
    def yxyo(self) -> I32Vector4: ...
    @property
    def yxyl(self) -> I32Vector4: ...
    @property
    def yxzx(self) -> I32Vector4: ...
    @property
    def yxzy(self) -> I32Vector4: ...
    @property
    def yxzz(self) -> I32Vector4: ...
    @property
    def yxzo(self) -> I32Vector4: ...
    @property
    def yxzl(self) -> I32Vector4: ...
    @property
    def yxox(self) -> I32Vector4: ...
    @property
    def yxoy(self) -> I32Vector4: ...
    @property
    def yxoz(self) -> I32Vector4: ...
    @property
    def yxoo(self) -> I32Vector4: ...
    @property
    def yxol(self) -> I32Vector4: ...
    @property
    def yxlx(self) -> I32Vector4: ...
    @property
    def yxly(self) -> I32Vector4: ...
    @property
    def yxlz(self) -> I32Vector4: ...
    @property
    def yxlo(self) -> I32Vector4: ...
    @property
    def yxll(self) -> I32Vector4: ...
    @property
    def yyxx(self) -> I32Vector4: ...
    @property
    def yyxy(self) -> I32Vector4: ...
    @property
    def yyxz(self) -> I32Vector4: ...
    @property
    def yyxo(self) -> I32Vector4: ...
    @property
    def yyxl(self) -> I32Vector4: ...
    @property
    def yyyx(self) -> I32Vector4: ...
    @property
    def yyyy(self) -> I32Vector4: ...
    @property
    def yyyz(self) -> I32Vector4: ...
    @property
    def yyyo(self) -> I32Vector4: ...
    @property
    def yyyl(self) -> I32Vector4: ...
    @property
    def yyzx(self) -> I32Vector4: ...
    @property
    def yyzy(self) -> I32Vector4: ...
    @property
    def yyzz(self) -> I32Vector4: ...
    @property
    def yyzo(self) -> I32Vector4: ...
    @property
    def yyzl(self) -> I32Vector4: ...
    @property
    def yyox(self) -> I32Vector4: ...
    @property
    def yyoy(self) -> I32Vector4: ...
    @property
    def yyoz(self) -> I32Vector4: ...
    @property
    def yyoo(self) -> I32Vector4: ...
    @property
    def yyol(self) -> I32Vector4: ...
    @property
    def yylx(self) -> I32Vector4: ...
    @property
    def yyly(self) -> I32Vector4: ...
    @property
    def yylz(self) -> I32Vector4: ...
    @property
    def yylo(self) -> I32Vector4: ...
    @property
    def yyll(self) -> I32Vector4: ...
    @property
    def yzxx(self) -> I32Vector4: ...
    @property
    def yzxy(self) -> I32Vector4: ...
    @property
    def yzxz(self) -> I32Vector4: ...
    @property
    def yzxo(self) -> I32Vector4: ...
    @property
    def yzxl(self) -> I32Vector4: ...
    @property
    def yzyx(self) -> I32Vector4: ...
    @property
    def yzyy(self) -> I32Vector4: ...
    @property
    def yzyz(self) -> I32Vector4: ...
    @property
    def yzyo(self) -> I32Vector4: ...
    @property
    def yzyl(self) -> I32Vector4: ...
    @property
    def yzzx(self) -> I32Vector4: ...
    @property
    def yzzy(self) -> I32Vector4: ...
    @property
    def yzzz(self) -> I32Vector4: ...
    @property
    def yzzo(self) -> I32Vector4: ...
    @property
    def yzzl(self) -> I32Vector4: ...
    @property
    def yzox(self) -> I32Vector4: ...
    @property
    def yzoy(self) -> I32Vector4: ...
    @property
    def yzoz(self) -> I32Vector4: ...
    @property
    def yzoo(self) -> I32Vector4: ...
    @property
    def yzol(self) -> I32Vector4: ...
    @property
    def yzlx(self) -> I32Vector4: ...
    @property
    def yzly(self) -> I32Vector4: ...
    @property
    def yzlz(self) -> I32Vector4: ...
    @property
    def yzlo(self) -> I32Vector4: ...
    @property
    def yzll(self) -> I32Vector4: ...
    @property
    def yoxx(self) -> I32Vector4: ...
    @property
    def yoxy(self) -> I32Vector4: ...
    @property
    def yoxz(self) -> I32Vector4: ...
    @property
    def yoxo(self) -> I32Vector4: ...
    @property
    def yoxl(self) -> I32Vector4: ...
    @property
    def yoyx(self) -> I32Vector4: ...
    @property
    def yoyy(self) -> I32Vector4: ...
    @property
    def yoyz(self) -> I32Vector4: ...
    @property
    def yoyo(self) -> I32Vector4: ...
    @property
    def yoyl(self) -> I32Vector4: ...
    @property
    def yozx(self) -> I32Vector4: ...
    @property
    def yozy(self) -> I32Vector4: ...
    @property
    def yozz(self) -> I32Vector4: ...
    @property
    def yozo(self) -> I32Vector4: ...
    @property
    def yozl(self) -> I32Vector4: ...
    @property
    def yoox(self) -> I32Vector4: ...
    @property
    def yooy(self) -> I32Vector4: ...
    @property
    def yooz(self) -> I32Vector4: ...
    @property
    def yooo(self) -> I32Vector4: ...
    @property
    def yool(self) -> I32Vector4: ...
    @property
    def yolx(self) -> I32Vector4: ...
    @property
    def yoly(self) -> I32Vector4: ...
    @property
    def yolz(self) -> I32Vector4: ...
    @property
    def yolo(self) -> I32Vector4: ...
    @property
    def yoll(self) -> I32Vector4: ...
    @property
    def ylxx(self) -> I32Vector4: ...
    @property
    def ylxy(self) -> I32Vector4: ...
    @property
    def ylxz(self) -> I32Vector4: ...
    @property
    def ylxo(self) -> I32Vector4: ...
    @property
    def ylxl(self) -> I32Vector4: ...
    @property
    def ylyx(self) -> I32Vector4: ...
    @property
    def ylyy(self) -> I32Vector4: ...
    @property
    def ylyz(self) -> I32Vector4: ...
    @property
    def ylyo(self) -> I32Vector4: ...
    @property
    def ylyl(self) -> I32Vector4: ...
    @property
    def ylzx(self) -> I32Vector4: ...
    @property
    def ylzy(self) -> I32Vector4: ...
    @property
    def ylzz(self) -> I32Vector4: ...
    @property
    def ylzo(self) -> I32Vector4: ...
    @property
    def ylzl(self) -> I32Vector4: ...
    @property
    def ylox(self) -> I32Vector4: ...
    @property
    def yloy(self) -> I32Vector4: ...
    @property
    def yloz(self) -> I32Vector4: ...
    @property
    def yloo(self) -> I32Vector4: ...
    @property
    def ylol(self) -> I32Vector4: ...
    @property
    def yllx(self) -> I32Vector4: ...
    @property
    def ylly(self) -> I32Vector4: ...
    @property
    def yllz(self) -> I32Vector4: ...
    @property
    def yllo(self) -> I32Vector4: ...
    @property
    def ylll(self) -> I32Vector4: ...
    @property
    def zxxx(self) -> I32Vector4: ...
    @property
    def zxxy(self) -> I32Vector4: ...
    @property
    def zxxz(self) -> I32Vector4: ...
    @property
    def zxxo(self) -> I32Vector4: ...
    @property
    def zxxl(self) -> I32Vector4: ...
    @property
    def zxyx(self) -> I32Vector4: ...
    @property
    def zxyy(self) -> I32Vector4: ...
    @property
    def zxyz(self) -> I32Vector4: ...
    @property
    def zxyo(self) -> I32Vector4: ...
    @property
    def zxyl(self) -> I32Vector4: ...
    @property
    def zxzx(self) -> I32Vector4: ...
    @property
    def zxzy(self) -> I32Vector4: ...
    @property
    def zxzz(self) -> I32Vector4: ...
    @property
    def zxzo(self) -> I32Vector4: ...
    @property
    def zxzl(self) -> I32Vector4: ...
    @property
    def zxox(self) -> I32Vector4: ...
    @property
    def zxoy(self) -> I32Vector4: ...
    @property
    def zxoz(self) -> I32Vector4: ...
    @property
    def zxoo(self) -> I32Vector4: ...
    @property
    def zxol(self) -> I32Vector4: ...
    @property
    def zxlx(self) -> I32Vector4: ...
    @property
    def zxly(self) -> I32Vector4: ...
    @property
    def zxlz(self) -> I32Vector4: ...
    @property
    def zxlo(self) -> I32Vector4: ...
    @property
    def zxll(self) -> I32Vector4: ...
    @property
    def zyxx(self) -> I32Vector4: ...
    @property
    def zyxy(self) -> I32Vector4: ...
    @property
    def zyxz(self) -> I32Vector4: ...
    @property
    def zyxo(self) -> I32Vector4: ...
    @property
    def zyxl(self) -> I32Vector4: ...
    @property
    def zyyx(self) -> I32Vector4: ...
    @property
    def zyyy(self) -> I32Vector4: ...
    @property
    def zyyz(self) -> I32Vector4: ...
    @property
    def zyyo(self) -> I32Vector4: ...
    @property
    def zyyl(self) -> I32Vector4: ...
    @property
    def zyzx(self) -> I32Vector4: ...
    @property
    def zyzy(self) -> I32Vector4: ...
    @property
    def zyzz(self) -> I32Vector4: ...
    @property
    def zyzo(self) -> I32Vector4: ...
    @property
    def zyzl(self) -> I32Vector4: ...
    @property
    def zyox(self) -> I32Vector4: ...
    @property
    def zyoy(self) -> I32Vector4: ...
    @property
    def zyoz(self) -> I32Vector4: ...
    @property
    def zyoo(self) -> I32Vector4: ...
    @property
    def zyol(self) -> I32Vector4: ...
    @property
    def zylx(self) -> I32Vector4: ...
    @property
    def zyly(self) -> I32Vector4: ...
    @property
    def zylz(self) -> I32Vector4: ...
    @property
    def zylo(self) -> I32Vector4: ...
    @property
    def zyll(self) -> I32Vector4: ...
    @property
    def zzxx(self) -> I32Vector4: ...
    @property
    def zzxy(self) -> I32Vector4: ...
    @property
    def zzxz(self) -> I32Vector4: ...
    @property
    def zzxo(self) -> I32Vector4: ...
    @property
    def zzxl(self) -> I32Vector4: ...
    @property
    def zzyx(self) -> I32Vector4: ...
    @property
    def zzyy(self) -> I32Vector4: ...
    @property
    def zzyz(self) -> I32Vector4: ...
    @property
    def zzyo(self) -> I32Vector4: ...
    @property
    def zzyl(self) -> I32Vector4: ...
    @property
    def zzzx(self) -> I32Vector4: ...
    @property
    def zzzy(self) -> I32Vector4: ...
    @property
    def zzzz(self) -> I32Vector4: ...
    @property
    def zzzo(self) -> I32Vector4: ...
    @property
    def zzzl(self) -> I32Vector4: ...
    @property
    def zzox(self) -> I32Vector4: ...
    @property
    def zzoy(self) -> I32Vector4: ...
    @property
    def zzoz(self) -> I32Vector4: ...
    @property
    def zzoo(self) -> I32Vector4: ...
    @property
    def zzol(self) -> I32Vector4: ...
    @property
    def zzlx(self) -> I32Vector4: ...
    @property
    def zzly(self) -> I32Vector4: ...
    @property
    def zzlz(self) -> I32Vector4: ...
    @property
    def zzlo(self) -> I32Vector4: ...
    @property
    def zzll(self) -> I32Vector4: ...
    @property
    def zoxx(self) -> I32Vector4: ...
    @property
    def zoxy(self) -> I32Vector4: ...
    @property
    def zoxz(self) -> I32Vector4: ...
    @property
    def zoxo(self) -> I32Vector4: ...
    @property
    def zoxl(self) -> I32Vector4: ...
    @property
    def zoyx(self) -> I32Vector4: ...
    @property
    def zoyy(self) -> I32Vector4: ...
    @property
    def zoyz(self) -> I32Vector4: ...
    @property
    def zoyo(self) -> I32Vector4: ...
    @property
    def zoyl(self) -> I32Vector4: ...
    @property
    def zozx(self) -> I32Vector4: ...
    @property
    def zozy(self) -> I32Vector4: ...
    @property
    def zozz(self) -> I32Vector4: ...
    @property
    def zozo(self) -> I32Vector4: ...
    @property
    def zozl(self) -> I32Vector4: ...
    @property
    def zoox(self) -> I32Vector4: ...
    @property
    def zooy(self) -> I32Vector4: ...
    @property
    def zooz(self) -> I32Vector4: ...
    @property
    def zooo(self) -> I32Vector4: ...
    @property
    def zool(self) -> I32Vector4: ...
    @property
    def zolx(self) -> I32Vector4: ...
    @property
    def zoly(self) -> I32Vector4: ...
    @property
    def zolz(self) -> I32Vector4: ...
    @property
    def zolo(self) -> I32Vector4: ...
    @property
    def zoll(self) -> I32Vector4: ...
    @property
    def zlxx(self) -> I32Vector4: ...
    @property
    def zlxy(self) -> I32Vector4: ...
    @property
    def zlxz(self) -> I32Vector4: ...
    @property
    def zlxo(self) -> I32Vector4: ...
    @property
    def zlxl(self) -> I32Vector4: ...
    @property
    def zlyx(self) -> I32Vector4: ...
    @property
    def zlyy(self) -> I32Vector4: ...
    @property
    def zlyz(self) -> I32Vector4: ...
    @property
    def zlyo(self) -> I32Vector4: ...
    @property
    def zlyl(self) -> I32Vector4: ...
    @property
    def zlzx(self) -> I32Vector4: ...
    @property
    def zlzy(self) -> I32Vector4: ...
    @property
    def zlzz(self) -> I32Vector4: ...
    @property
    def zlzo(self) -> I32Vector4: ...
    @property
    def zlzl(self) -> I32Vector4: ...
    @property
    def zlox(self) -> I32Vector4: ...
    @property
    def zloy(self) -> I32Vector4: ...
    @property
    def zloz(self) -> I32Vector4: ...
    @property
    def zloo(self) -> I32Vector4: ...
    @property
    def zlol(self) -> I32Vector4: ...
    @property
    def zllx(self) -> I32Vector4: ...
    @property
    def zlly(self) -> I32Vector4: ...
    @property
    def zllz(self) -> I32Vector4: ...
    @property
    def zllo(self) -> I32Vector4: ...
    @property
    def zlll(self) -> I32Vector4: ...
    @property
    def oxxx(self) -> I32Vector4: ...
    @property
    def oxxy(self) -> I32Vector4: ...
    @property
    def oxxz(self) -> I32Vector4: ...
    @property
    def oxxo(self) -> I32Vector4: ...
    @property
    def oxxl(self) -> I32Vector4: ...
    @property
    def oxyx(self) -> I32Vector4: ...
    @property
    def oxyy(self) -> I32Vector4: ...
    @property
    def oxyz(self) -> I32Vector4: ...
    @property
    def oxyo(self) -> I32Vector4: ...
    @property
    def oxyl(self) -> I32Vector4: ...
    @property
    def oxzx(self) -> I32Vector4: ...
    @property
    def oxzy(self) -> I32Vector4: ...
    @property
    def oxzz(self) -> I32Vector4: ...
    @property
    def oxzo(self) -> I32Vector4: ...
    @property
    def oxzl(self) -> I32Vector4: ...
    @property
    def oxox(self) -> I32Vector4: ...
    @property
    def oxoy(self) -> I32Vector4: ...
    @property
    def oxoz(self) -> I32Vector4: ...
    @property
    def oxoo(self) -> I32Vector4: ...
    @property
    def oxol(self) -> I32Vector4: ...
    @property
    def oxlx(self) -> I32Vector4: ...
    @property
    def oxly(self) -> I32Vector4: ...
    @property
    def oxlz(self) -> I32Vector4: ...
    @property
    def oxlo(self) -> I32Vector4: ...
    @property
    def oxll(self) -> I32Vector4: ...
    @property
    def oyxx(self) -> I32Vector4: ...
    @property
    def oyxy(self) -> I32Vector4: ...
    @property
    def oyxz(self) -> I32Vector4: ...
    @property
    def oyxo(self) -> I32Vector4: ...
    @property
    def oyxl(self) -> I32Vector4: ...
    @property
    def oyyx(self) -> I32Vector4: ...
    @property
    def oyyy(self) -> I32Vector4: ...
    @property
    def oyyz(self) -> I32Vector4: ...
    @property
    def oyyo(self) -> I32Vector4: ...
    @property
    def oyyl(self) -> I32Vector4: ...
    @property
    def oyzx(self) -> I32Vector4: ...
    @property
    def oyzy(self) -> I32Vector4: ...
    @property
    def oyzz(self) -> I32Vector4: ...
    @property
    def oyzo(self) -> I32Vector4: ...
    @property
    def oyzl(self) -> I32Vector4: ...
    @property
    def oyox(self) -> I32Vector4: ...
    @property
    def oyoy(self) -> I32Vector4: ...
    @property
    def oyoz(self) -> I32Vector4: ...
    @property
    def oyoo(self) -> I32Vector4: ...
    @property
    def oyol(self) -> I32Vector4: ...
    @property
    def oylx(self) -> I32Vector4: ...
    @property
    def oyly(self) -> I32Vector4: ...
    @property
    def oylz(self) -> I32Vector4: ...
    @property
    def oylo(self) -> I32Vector4: ...
    @property
    def oyll(self) -> I32Vector4: ...
    @property
    def ozxx(self) -> I32Vector4: ...
    @property
    def ozxy(self) -> I32Vector4: ...
    @property
    def ozxz(self) -> I32Vector4: ...
    @property
    def ozxo(self) -> I32Vector4: ...
    @property
    def ozxl(self) -> I32Vector4: ...
    @property
    def ozyx(self) -> I32Vector4: ...
    @property
    def ozyy(self) -> I32Vector4: ...
    @property
    def ozyz(self) -> I32Vector4: ...
    @property
    def ozyo(self) -> I32Vector4: ...
    @property
    def ozyl(self) -> I32Vector4: ...
    @property
    def ozzx(self) -> I32Vector4: ...
    @property
    def ozzy(self) -> I32Vector4: ...
    @property
    def ozzz(self) -> I32Vector4: ...
    @property
    def ozzo(self) -> I32Vector4: ...
    @property
    def ozzl(self) -> I32Vector4: ...
    @property
    def ozox(self) -> I32Vector4: ...
    @property
    def ozoy(self) -> I32Vector4: ...
    @property
    def ozoz(self) -> I32Vector4: ...
    @property
    def ozoo(self) -> I32Vector4: ...
    @property
    def ozol(self) -> I32Vector4: ...
    @property
    def ozlx(self) -> I32Vector4: ...
    @property
    def ozly(self) -> I32Vector4: ...
    @property
    def ozlz(self) -> I32Vector4: ...
    @property
    def ozlo(self) -> I32Vector4: ...
    @property
    def ozll(self) -> I32Vector4: ...
    @property
    def ooxx(self) -> I32Vector4: ...
    @property
    def ooxy(self) -> I32Vector4: ...
    @property
    def ooxz(self) -> I32Vector4: ...
    @property
    def ooxo(self) -> I32Vector4: ...
    @property
    def ooxl(self) -> I32Vector4: ...
    @property
    def ooyx(self) -> I32Vector4: ...
    @property
    def ooyy(self) -> I32Vector4: ...
    @property
    def ooyz(self) -> I32Vector4: ...
    @property
    def ooyo(self) -> I32Vector4: ...
    @property
    def ooyl(self) -> I32Vector4: ...
    @property
    def oozx(self) -> I32Vector4: ...
    @property
    def oozy(self) -> I32Vector4: ...
    @property
    def oozz(self) -> I32Vector4: ...
    @property
    def oozo(self) -> I32Vector4: ...
    @property
    def oozl(self) -> I32Vector4: ...
    @property
    def ooox(self) -> I32Vector4: ...
    @property
    def oooy(self) -> I32Vector4: ...
    @property
    def oooz(self) -> I32Vector4: ...
    @property
    def oolx(self) -> I32Vector4: ...
    @property
    def ooly(self) -> I32Vector4: ...
    @property
    def oolz(self) -> I32Vector4: ...
    @property
    def olxx(self) -> I32Vector4: ...
    @property
    def olxy(self) -> I32Vector4: ...
    @property
    def olxz(self) -> I32Vector4: ...
    @property
    def olxo(self) -> I32Vector4: ...
    @property
    def olxl(self) -> I32Vector4: ...
    @property
    def olyx(self) -> I32Vector4: ...
    @property
    def olyy(self) -> I32Vector4: ...
    @property
    def olyz(self) -> I32Vector4: ...
    @property
    def olyo(self) -> I32Vector4: ...
    @property
    def olyl(self) -> I32Vector4: ...
    @property
    def olzx(self) -> I32Vector4: ...
    @property
    def olzy(self) -> I32Vector4: ...
    @property
    def olzz(self) -> I32Vector4: ...
    @property
    def olzo(self) -> I32Vector4: ...
    @property
    def olzl(self) -> I32Vector4: ...
    @property
    def olox(self) -> I32Vector4: ...
    @property
    def oloy(self) -> I32Vector4: ...
    @property
    def oloz(self) -> I32Vector4: ...
    @property
    def ollx(self) -> I32Vector4: ...
    @property
    def olly(self) -> I32Vector4: ...
    @property
    def ollz(self) -> I32Vector4: ...
    @property
    def lxxx(self) -> I32Vector4: ...
    @property
    def lxxy(self) -> I32Vector4: ...
    @property
    def lxxz(self) -> I32Vector4: ...
    @property
    def lxxo(self) -> I32Vector4: ...
    @property
    def lxxl(self) -> I32Vector4: ...
    @property
    def lxyx(self) -> I32Vector4: ...
    @property
    def lxyy(self) -> I32Vector4: ...
    @property
    def lxyz(self) -> I32Vector4: ...
    @property
    def lxyo(self) -> I32Vector4: ...
    @property
    def lxyl(self) -> I32Vector4: ...
    @property
    def lxzx(self) -> I32Vector4: ...
    @property
    def lxzy(self) -> I32Vector4: ...
    @property
    def lxzz(self) -> I32Vector4: ...
    @property
    def lxzo(self) -> I32Vector4: ...
    @property
    def lxzl(self) -> I32Vector4: ...
    @property
    def lxox(self) -> I32Vector4: ...
    @property
    def lxoy(self) -> I32Vector4: ...
    @property
    def lxoz(self) -> I32Vector4: ...
    @property
    def lxoo(self) -> I32Vector4: ...
    @property
    def lxol(self) -> I32Vector4: ...
    @property
    def lxlx(self) -> I32Vector4: ...
    @property
    def lxly(self) -> I32Vector4: ...
    @property
    def lxlz(self) -> I32Vector4: ...
    @property
    def lxlo(self) -> I32Vector4: ...
    @property
    def lxll(self) -> I32Vector4: ...
    @property
    def lyxx(self) -> I32Vector4: ...
    @property
    def lyxy(self) -> I32Vector4: ...
    @property
    def lyxz(self) -> I32Vector4: ...
    @property
    def lyxo(self) -> I32Vector4: ...
    @property
    def lyxl(self) -> I32Vector4: ...
    @property
    def lyyx(self) -> I32Vector4: ...
    @property
    def lyyy(self) -> I32Vector4: ...
    @property
    def lyyz(self) -> I32Vector4: ...
    @property
    def lyyo(self) -> I32Vector4: ...
    @property
    def lyyl(self) -> I32Vector4: ...
    @property
    def lyzx(self) -> I32Vector4: ...
    @property
    def lyzy(self) -> I32Vector4: ...
    @property
    def lyzz(self) -> I32Vector4: ...
    @property
    def lyzo(self) -> I32Vector4: ...
    @property
    def lyzl(self) -> I32Vector4: ...
    @property
    def lyox(self) -> I32Vector4: ...
    @property
    def lyoy(self) -> I32Vector4: ...
    @property
    def lyoz(self) -> I32Vector4: ...
    @property
    def lyoo(self) -> I32Vector4: ...
    @property
    def lyol(self) -> I32Vector4: ...
    @property
    def lylx(self) -> I32Vector4: ...
    @property
    def lyly(self) -> I32Vector4: ...
    @property
    def lylz(self) -> I32Vector4: ...
    @property
    def lylo(self) -> I32Vector4: ...
    @property
    def lyll(self) -> I32Vector4: ...
    @property
    def lzxx(self) -> I32Vector4: ...
    @property
    def lzxy(self) -> I32Vector4: ...
    @property
    def lzxz(self) -> I32Vector4: ...
    @property
    def lzxo(self) -> I32Vector4: ...
    @property
    def lzxl(self) -> I32Vector4: ...
    @property
    def lzyx(self) -> I32Vector4: ...
    @property
    def lzyy(self) -> I32Vector4: ...
    @property
    def lzyz(self) -> I32Vector4: ...
    @property
    def lzyo(self) -> I32Vector4: ...
    @property
    def lzyl(self) -> I32Vector4: ...
    @property
    def lzzx(self) -> I32Vector4: ...
    @property
    def lzzy(self) -> I32Vector4: ...
    @property
    def lzzz(self) -> I32Vector4: ...
    @property
    def lzzo(self) -> I32Vector4: ...
    @property
    def lzzl(self) -> I32Vector4: ...
    @property
    def lzox(self) -> I32Vector4: ...
    @property
    def lzoy(self) -> I32Vector4: ...
    @property
    def lzoz(self) -> I32Vector4: ...
    @property
    def lzoo(self) -> I32Vector4: ...
    @property
    def lzol(self) -> I32Vector4: ...
    @property
    def lzlx(self) -> I32Vector4: ...
    @property
    def lzly(self) -> I32Vector4: ...
    @property
    def lzlz(self) -> I32Vector4: ...
    @property
    def lzlo(self) -> I32Vector4: ...
    @property
    def lzll(self) -> I32Vector4: ...
    @property
    def loxx(self) -> I32Vector4: ...
    @property
    def loxy(self) -> I32Vector4: ...
    @property
    def loxz(self) -> I32Vector4: ...
    @property
    def loxo(self) -> I32Vector4: ...
    @property
    def loxl(self) -> I32Vector4: ...
    @property
    def loyx(self) -> I32Vector4: ...
    @property
    def loyy(self) -> I32Vector4: ...
    @property
    def loyz(self) -> I32Vector4: ...
    @property
    def loyo(self) -> I32Vector4: ...
    @property
    def loyl(self) -> I32Vector4: ...
    @property
    def lozx(self) -> I32Vector4: ...
    @property
    def lozy(self) -> I32Vector4: ...
    @property
    def lozz(self) -> I32Vector4: ...
    @property
    def lozo(self) -> I32Vector4: ...
    @property
    def lozl(self) -> I32Vector4: ...
    @property
    def loox(self) -> I32Vector4: ...
    @property
    def looy(self) -> I32Vector4: ...
    @property
    def looz(self) -> I32Vector4: ...
    @property
    def lolx(self) -> I32Vector4: ...
    @property
    def loly(self) -> I32Vector4: ...
    @property
    def lolz(self) -> I32Vector4: ...
    @property
    def llxx(self) -> I32Vector4: ...
    @property
    def llxy(self) -> I32Vector4: ...
    @property
    def llxz(self) -> I32Vector4: ...
    @property
    def llxo(self) -> I32Vector4: ...
    @property
    def llxl(self) -> I32Vector4: ...
    @property
    def llyx(self) -> I32Vector4: ...
    @property
    def llyy(self) -> I32Vector4: ...
    @property
    def llyz(self) -> I32Vector4: ...
    @property
    def llyo(self) -> I32Vector4: ...
    @property
    def llyl(self) -> I32Vector4: ...
    @property
    def llzx(self) -> I32Vector4: ...
    @property
    def llzy(self) -> I32Vector4: ...
    @property
    def llzz(self) -> I32Vector4: ...
    @property
    def llzo(self) -> I32Vector4: ...
    @property
    def llzl(self) -> I32Vector4: ...
    @property
    def llox(self) -> I32Vector4: ...
    @property
    def lloy(self) -> I32Vector4: ...
    @property
    def lloz(self) -> I32Vector4: ...
    @property
    def lllx(self) -> I32Vector4: ...
    @property
    def llly(self) -> I32Vector4: ...
    @property
    def lllz(self) -> I32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> I32Vector2: ...
    @property
    def rg(self) -> I32Vector2: ...
    @property
    def rb(self) -> I32Vector2: ...
    @property
    def ro(self) -> I32Vector2: ...
    @property
    def rl(self) -> I32Vector2: ...
    @property
    def gr(self) -> I32Vector2: ...
    @property
    def gg(self) -> I32Vector2: ...
    @property
    def gb(self) -> I32Vector2: ...
    @property
    def go(self) -> I32Vector2: ...
    @property
    def gl(self) -> I32Vector2: ...
    @property
    def br(self) -> I32Vector2: ...
    @property
    def bg(self) -> I32Vector2: ...
    @property
    def bb(self) -> I32Vector2: ...
    @property
    def bo(self) -> I32Vector2: ...
    @property
    def bl(self) -> I32Vector2: ...
    @property
    def og(self) -> I32Vector2: ...
    @property
    def ob(self) -> I32Vector2: ...
    @property
    def lr(self) -> I32Vector2: ...
    @property
    def lg(self) -> I32Vector2: ...
    @property
    def lb(self) -> I32Vector2: ...
    @property
    def rrr(self) -> I32Vector3: ...
    @property
    def rrg(self) -> I32Vector3: ...
    @property
    def rrb(self) -> I32Vector3: ...
    @property
    def rro(self) -> I32Vector3: ...
    @property
    def rrl(self) -> I32Vector3: ...
    @property
    def rgr(self) -> I32Vector3: ...
    @property
    def rgg(self) -> I32Vector3: ...
    @property
    def rgb(self) -> I32Vector3: ...
    @property
    def rgo(self) -> I32Vector3: ...
    @property
    def rgl(self) -> I32Vector3: ...
    @property
    def rbr(self) -> I32Vector3: ...
    @property
    def rbg(self) -> I32Vector3: ...
    @property
    def rbb(self) -> I32Vector3: ...
    @property
    def rbo(self) -> I32Vector3: ...
    @property
    def rbl(self) -> I32Vector3: ...
    @property
    def ror(self) -> I32Vector3: ...
    @property
    def rog(self) -> I32Vector3: ...
    @property
    def rob(self) -> I32Vector3: ...
    @property
    def roo(self) -> I32Vector3: ...
    @property
    def rol(self) -> I32Vector3: ...
    @property
    def rlr(self) -> I32Vector3: ...
    @property
    def rlg(self) -> I32Vector3: ...
    @property
    def rlb(self) -> I32Vector3: ...
    @property
    def rlo(self) -> I32Vector3: ...
    @property
    def rll(self) -> I32Vector3: ...
    @property
    def grr(self) -> I32Vector3: ...
    @property
    def grg(self) -> I32Vector3: ...
    @property
    def grb(self) -> I32Vector3: ...
    @property
    def gro(self) -> I32Vector3: ...
    @property
    def grl(self) -> I32Vector3: ...
    @property
    def ggr(self) -> I32Vector3: ...
    @property
    def ggg(self) -> I32Vector3: ...
    @property
    def ggb(self) -> I32Vector3: ...
    @property
    def ggo(self) -> I32Vector3: ...
    @property
    def ggl(self) -> I32Vector3: ...
    @property
    def gbr(self) -> I32Vector3: ...
    @property
    def gbg(self) -> I32Vector3: ...
    @property
    def gbb(self) -> I32Vector3: ...
    @property
    def gbo(self) -> I32Vector3: ...
    @property
    def gbl(self) -> I32Vector3: ...
    @property
    def gor(self) -> I32Vector3: ...
    @property
    def gog(self) -> I32Vector3: ...
    @property
    def gob(self) -> I32Vector3: ...
    @property
    def goo(self) -> I32Vector3: ...
    @property
    def gol(self) -> I32Vector3: ...
    @property
    def glr(self) -> I32Vector3: ...
    @property
    def glg(self) -> I32Vector3: ...
    @property
    def glb(self) -> I32Vector3: ...
    @property
    def glo(self) -> I32Vector3: ...
    @property
    def gll(self) -> I32Vector3: ...
    @property
    def brr(self) -> I32Vector3: ...
    @property
    def brg(self) -> I32Vector3: ...
    @property
    def brb(self) -> I32Vector3: ...
    @property
    def bro(self) -> I32Vector3: ...
    @property
    def brl(self) -> I32Vector3: ...
    @property
    def bgr(self) -> I32Vector3: ...
    @property
    def bgg(self) -> I32Vector3: ...
    @property
    def bgb(self) -> I32Vector3: ...
    @property
    def bgo(self) -> I32Vector3: ...
    @property
    def bgl(self) -> I32Vector3: ...
    @property
    def bbr(self) -> I32Vector3: ...
    @property
    def bbg(self) -> I32Vector3: ...
    @property
    def bbb(self) -> I32Vector3: ...
    @property
    def bbo(self) -> I32Vector3: ...
    @property
    def bbl(self) -> I32Vector3: ...
    @property
    def bor(self) -> I32Vector3: ...
    @property
    def bog(self) -> I32Vector3: ...
    @property
    def bob(self) -> I32Vector3: ...
    @property
    def boo(self) -> I32Vector3: ...
    @property
    def bol(self) -> I32Vector3: ...
    @property
    def blr(self) -> I32Vector3: ...
    @property
    def blg(self) -> I32Vector3: ...
    @property
    def blb(self) -> I32Vector3: ...
    @property
    def blo(self) -> I32Vector3: ...
    @property
    def bll(self) -> I32Vector3: ...
    @property
    def orr(self) -> I32Vector3: ...
    @property
    def org(self) -> I32Vector3: ...
    @property
    def orb(self) -> I32Vector3: ...
    @property
    def oro(self) -> I32Vector3: ...
    @property
    def orl(self) -> I32Vector3: ...
    @property
    def ogr(self) -> I32Vector3: ...
    @property
    def ogg(self) -> I32Vector3: ...
    @property
    def ogb(self) -> I32Vector3: ...
    @property
    def ogo(self) -> I32Vector3: ...
    @property
    def ogl(self) -> I32Vector3: ...
    @property
    def obr(self) -> I32Vector3: ...
    @property
    def obg(self) -> I32Vector3: ...
    @property
    def obb(self) -> I32Vector3: ...
    @property
    def obo(self) -> I32Vector3: ...
    @property
    def obl(self) -> I32Vector3: ...
    @property
    def oor(self) -> I32Vector3: ...
    @property
    def oog(self) -> I32Vector3: ...
    @property
    def oob(self) -> I32Vector3: ...
    @property
    def olr(self) -> I32Vector3: ...
    @property
    def olg(self) -> I32Vector3: ...
    @property
    def olb(self) -> I32Vector3: ...
    @property
    def lrr(self) -> I32Vector3: ...
    @property
    def lrg(self) -> I32Vector3: ...
    @property
    def lrb(self) -> I32Vector3: ...
    @property
    def lro(self) -> I32Vector3: ...
    @property
    def lrl(self) -> I32Vector3: ...
    @property
    def lgr(self) -> I32Vector3: ...
    @property
    def lgg(self) -> I32Vector3: ...
    @property
    def lgb(self) -> I32Vector3: ...
    @property
    def lgo(self) -> I32Vector3: ...
    @property
    def lgl(self) -> I32Vector3: ...
    @property
    def lbr(self) -> I32Vector3: ...
    @property
    def lbg(self) -> I32Vector3: ...
    @property
    def lbb(self) -> I32Vector3: ...
    @property
    def lbo(self) -> I32Vector3: ...
    @property
    def lbl(self) -> I32Vector3: ...
    @property
    def lor(self) -> I32Vector3: ...
    @property
    def log(self) -> I32Vector3: ...
    @property
    def lob(self) -> I32Vector3: ...
    @property
    def llr(self) -> I32Vector3: ...
    @property
    def llg(self) -> I32Vector3: ...
    @property
    def llb(self) -> I32Vector3: ...
    @property
    def rrrr(self) -> I32Vector4: ...
    @property
    def rrrg(self) -> I32Vector4: ...
    @property
    def rrrb(self) -> I32Vector4: ...
    @property
    def rrro(self) -> I32Vector4: ...
    @property
    def rrrl(self) -> I32Vector4: ...
    @property
    def rrgr(self) -> I32Vector4: ...
    @property
    def rrgg(self) -> I32Vector4: ...
    @property
    def rrgb(self) -> I32Vector4: ...
    @property
    def rrgo(self) -> I32Vector4: ...
    @property
    def rrgl(self) -> I32Vector4: ...
    @property
    def rrbr(self) -> I32Vector4: ...
    @property
    def rrbg(self) -> I32Vector4: ...
    @property
    def rrbb(self) -> I32Vector4: ...
    @property
    def rrbo(self) -> I32Vector4: ...
    @property
    def rrbl(self) -> I32Vector4: ...
    @property
    def rror(self) -> I32Vector4: ...
    @property
    def rrog(self) -> I32Vector4: ...
    @property
    def rrob(self) -> I32Vector4: ...
    @property
    def rroo(self) -> I32Vector4: ...
    @property
    def rrol(self) -> I32Vector4: ...
    @property
    def rrlr(self) -> I32Vector4: ...
    @property
    def rrlg(self) -> I32Vector4: ...
    @property
    def rrlb(self) -> I32Vector4: ...
    @property
    def rrlo(self) -> I32Vector4: ...
    @property
    def rrll(self) -> I32Vector4: ...
    @property
    def rgrr(self) -> I32Vector4: ...
    @property
    def rgrg(self) -> I32Vector4: ...
    @property
    def rgrb(self) -> I32Vector4: ...
    @property
    def rgro(self) -> I32Vector4: ...
    @property
    def rgrl(self) -> I32Vector4: ...
    @property
    def rggr(self) -> I32Vector4: ...
    @property
    def rggg(self) -> I32Vector4: ...
    @property
    def rggb(self) -> I32Vector4: ...
    @property
    def rggo(self) -> I32Vector4: ...
    @property
    def rggl(self) -> I32Vector4: ...
    @property
    def rgbr(self) -> I32Vector4: ...
    @property
    def rgbg(self) -> I32Vector4: ...
    @property
    def rgbb(self) -> I32Vector4: ...
    @property
    def rgbo(self) -> I32Vector4: ...
    @property
    def rgbl(self) -> I32Vector4: ...
    @property
    def rgor(self) -> I32Vector4: ...
    @property
    def rgog(self) -> I32Vector4: ...
    @property
    def rgob(self) -> I32Vector4: ...
    @property
    def rgoo(self) -> I32Vector4: ...
    @property
    def rgol(self) -> I32Vector4: ...
    @property
    def rglr(self) -> I32Vector4: ...
    @property
    def rglg(self) -> I32Vector4: ...
    @property
    def rglb(self) -> I32Vector4: ...
    @property
    def rglo(self) -> I32Vector4: ...
    @property
    def rgll(self) -> I32Vector4: ...
    @property
    def rbrr(self) -> I32Vector4: ...
    @property
    def rbrg(self) -> I32Vector4: ...
    @property
    def rbrb(self) -> I32Vector4: ...
    @property
    def rbro(self) -> I32Vector4: ...
    @property
    def rbrl(self) -> I32Vector4: ...
    @property
    def rbgr(self) -> I32Vector4: ...
    @property
    def rbgg(self) -> I32Vector4: ...
    @property
    def rbgb(self) -> I32Vector4: ...
    @property
    def rbgo(self) -> I32Vector4: ...
    @property
    def rbgl(self) -> I32Vector4: ...
    @property
    def rbbr(self) -> I32Vector4: ...
    @property
    def rbbg(self) -> I32Vector4: ...
    @property
    def rbbb(self) -> I32Vector4: ...
    @property
    def rbbo(self) -> I32Vector4: ...
    @property
    def rbbl(self) -> I32Vector4: ...
    @property
    def rbor(self) -> I32Vector4: ...
    @property
    def rbog(self) -> I32Vector4: ...
    @property
    def rbob(self) -> I32Vector4: ...
    @property
    def rboo(self) -> I32Vector4: ...
    @property
    def rbol(self) -> I32Vector4: ...
    @property
    def rblr(self) -> I32Vector4: ...
    @property
    def rblg(self) -> I32Vector4: ...
    @property
    def rblb(self) -> I32Vector4: ...
    @property
    def rblo(self) -> I32Vector4: ...
    @property
    def rbll(self) -> I32Vector4: ...
    @property
    def rorr(self) -> I32Vector4: ...
    @property
    def rorg(self) -> I32Vector4: ...
    @property
    def rorb(self) -> I32Vector4: ...
    @property
    def roro(self) -> I32Vector4: ...
    @property
    def rorl(self) -> I32Vector4: ...
    @property
    def rogr(self) -> I32Vector4: ...
    @property
    def rogg(self) -> I32Vector4: ...
    @property
    def rogb(self) -> I32Vector4: ...
    @property
    def rogo(self) -> I32Vector4: ...
    @property
    def rogl(self) -> I32Vector4: ...
    @property
    def robr(self) -> I32Vector4: ...
    @property
    def robg(self) -> I32Vector4: ...
    @property
    def robb(self) -> I32Vector4: ...
    @property
    def robo(self) -> I32Vector4: ...
    @property
    def robl(self) -> I32Vector4: ...
    @property
    def roor(self) -> I32Vector4: ...
    @property
    def roog(self) -> I32Vector4: ...
    @property
    def roob(self) -> I32Vector4: ...
    @property
    def rooo(self) -> I32Vector4: ...
    @property
    def rool(self) -> I32Vector4: ...
    @property
    def rolr(self) -> I32Vector4: ...
    @property
    def rolg(self) -> I32Vector4: ...
    @property
    def rolb(self) -> I32Vector4: ...
    @property
    def rolo(self) -> I32Vector4: ...
    @property
    def roll(self) -> I32Vector4: ...
    @property
    def rlrr(self) -> I32Vector4: ...
    @property
    def rlrg(self) -> I32Vector4: ...
    @property
    def rlrb(self) -> I32Vector4: ...
    @property
    def rlro(self) -> I32Vector4: ...
    @property
    def rlrl(self) -> I32Vector4: ...
    @property
    def rlgr(self) -> I32Vector4: ...
    @property
    def rlgg(self) -> I32Vector4: ...
    @property
    def rlgb(self) -> I32Vector4: ...
    @property
    def rlgo(self) -> I32Vector4: ...
    @property
    def rlgl(self) -> I32Vector4: ...
    @property
    def rlbr(self) -> I32Vector4: ...
    @property
    def rlbg(self) -> I32Vector4: ...
    @property
    def rlbb(self) -> I32Vector4: ...
    @property
    def rlbo(self) -> I32Vector4: ...
    @property
    def rlbl(self) -> I32Vector4: ...
    @property
    def rlor(self) -> I32Vector4: ...
    @property
    def rlog(self) -> I32Vector4: ...
    @property
    def rlob(self) -> I32Vector4: ...
    @property
    def rloo(self) -> I32Vector4: ...
    @property
    def rlol(self) -> I32Vector4: ...
    @property
    def rllr(self) -> I32Vector4: ...
    @property
    def rllg(self) -> I32Vector4: ...
    @property
    def rllb(self) -> I32Vector4: ...
    @property
    def rllo(self) -> I32Vector4: ...
    @property
    def rlll(self) -> I32Vector4: ...
    @property
    def grrr(self) -> I32Vector4: ...
    @property
    def grrg(self) -> I32Vector4: ...
    @property
    def grrb(self) -> I32Vector4: ...
    @property
    def grro(self) -> I32Vector4: ...
    @property
    def grrl(self) -> I32Vector4: ...
    @property
    def grgr(self) -> I32Vector4: ...
    @property
    def grgg(self) -> I32Vector4: ...
    @property
    def grgb(self) -> I32Vector4: ...
    @property
    def grgo(self) -> I32Vector4: ...
    @property
    def grgl(self) -> I32Vector4: ...
    @property
    def grbr(self) -> I32Vector4: ...
    @property
    def grbg(self) -> I32Vector4: ...
    @property
    def grbb(self) -> I32Vector4: ...
    @property
    def grbo(self) -> I32Vector4: ...
    @property
    def grbl(self) -> I32Vector4: ...
    @property
    def gror(self) -> I32Vector4: ...
    @property
    def grog(self) -> I32Vector4: ...
    @property
    def grob(self) -> I32Vector4: ...
    @property
    def groo(self) -> I32Vector4: ...
    @property
    def grol(self) -> I32Vector4: ...
    @property
    def grlr(self) -> I32Vector4: ...
    @property
    def grlg(self) -> I32Vector4: ...
    @property
    def grlb(self) -> I32Vector4: ...
    @property
    def grlo(self) -> I32Vector4: ...
    @property
    def grll(self) -> I32Vector4: ...
    @property
    def ggrr(self) -> I32Vector4: ...
    @property
    def ggrg(self) -> I32Vector4: ...
    @property
    def ggrb(self) -> I32Vector4: ...
    @property
    def ggro(self) -> I32Vector4: ...
    @property
    def ggrl(self) -> I32Vector4: ...
    @property
    def gggr(self) -> I32Vector4: ...
    @property
    def gggg(self) -> I32Vector4: ...
    @property
    def gggb(self) -> I32Vector4: ...
    @property
    def gggo(self) -> I32Vector4: ...
    @property
    def gggl(self) -> I32Vector4: ...
    @property
    def ggbr(self) -> I32Vector4: ...
    @property
    def ggbg(self) -> I32Vector4: ...
    @property
    def ggbb(self) -> I32Vector4: ...
    @property
    def ggbo(self) -> I32Vector4: ...
    @property
    def ggbl(self) -> I32Vector4: ...
    @property
    def ggor(self) -> I32Vector4: ...
    @property
    def ggog(self) -> I32Vector4: ...
    @property
    def ggob(self) -> I32Vector4: ...
    @property
    def ggoo(self) -> I32Vector4: ...
    @property
    def ggol(self) -> I32Vector4: ...
    @property
    def gglr(self) -> I32Vector4: ...
    @property
    def gglg(self) -> I32Vector4: ...
    @property
    def gglb(self) -> I32Vector4: ...
    @property
    def gglo(self) -> I32Vector4: ...
    @property
    def ggll(self) -> I32Vector4: ...
    @property
    def gbrr(self) -> I32Vector4: ...
    @property
    def gbrg(self) -> I32Vector4: ...
    @property
    def gbrb(self) -> I32Vector4: ...
    @property
    def gbro(self) -> I32Vector4: ...
    @property
    def gbrl(self) -> I32Vector4: ...
    @property
    def gbgr(self) -> I32Vector4: ...
    @property
    def gbgg(self) -> I32Vector4: ...
    @property
    def gbgb(self) -> I32Vector4: ...
    @property
    def gbgo(self) -> I32Vector4: ...
    @property
    def gbgl(self) -> I32Vector4: ...
    @property
    def gbbr(self) -> I32Vector4: ...
    @property
    def gbbg(self) -> I32Vector4: ...
    @property
    def gbbb(self) -> I32Vector4: ...
    @property
    def gbbo(self) -> I32Vector4: ...
    @property
    def gbbl(self) -> I32Vector4: ...
    @property
    def gbor(self) -> I32Vector4: ...
    @property
    def gbog(self) -> I32Vector4: ...
    @property
    def gbob(self) -> I32Vector4: ...
    @property
    def gboo(self) -> I32Vector4: ...
    @property
    def gbol(self) -> I32Vector4: ...
    @property
    def gblr(self) -> I32Vector4: ...
    @property
    def gblg(self) -> I32Vector4: ...
    @property
    def gblb(self) -> I32Vector4: ...
    @property
    def gblo(self) -> I32Vector4: ...
    @property
    def gbll(self) -> I32Vector4: ...
    @property
    def gorr(self) -> I32Vector4: ...
    @property
    def gorg(self) -> I32Vector4: ...
    @property
    def gorb(self) -> I32Vector4: ...
    @property
    def goro(self) -> I32Vector4: ...
    @property
    def gorl(self) -> I32Vector4: ...
    @property
    def gogr(self) -> I32Vector4: ...
    @property
    def gogg(self) -> I32Vector4: ...
    @property
    def gogb(self) -> I32Vector4: ...
    @property
    def gogo(self) -> I32Vector4: ...
    @property
    def gogl(self) -> I32Vector4: ...
    @property
    def gobr(self) -> I32Vector4: ...
    @property
    def gobg(self) -> I32Vector4: ...
    @property
    def gobb(self) -> I32Vector4: ...
    @property
    def gobo(self) -> I32Vector4: ...
    @property
    def gobl(self) -> I32Vector4: ...
    @property
    def goor(self) -> I32Vector4: ...
    @property
    def goog(self) -> I32Vector4: ...
    @property
    def goob(self) -> I32Vector4: ...
    @property
    def gooo(self) -> I32Vector4: ...
    @property
    def gool(self) -> I32Vector4: ...
    @property
    def golr(self) -> I32Vector4: ...
    @property
    def golg(self) -> I32Vector4: ...
    @property
    def golb(self) -> I32Vector4: ...
    @property
    def golo(self) -> I32Vector4: ...
    @property
    def goll(self) -> I32Vector4: ...
    @property
    def glrr(self) -> I32Vector4: ...
    @property
    def glrg(self) -> I32Vector4: ...
    @property
    def glrb(self) -> I32Vector4: ...
    @property
    def glro(self) -> I32Vector4: ...
    @property
    def glrl(self) -> I32Vector4: ...
    @property
    def glgr(self) -> I32Vector4: ...
    @property
    def glgg(self) -> I32Vector4: ...
    @property
    def glgb(self) -> I32Vector4: ...
    @property
    def glgo(self) -> I32Vector4: ...
    @property
    def glgl(self) -> I32Vector4: ...
    @property
    def glbr(self) -> I32Vector4: ...
    @property
    def glbg(self) -> I32Vector4: ...
    @property
    def glbb(self) -> I32Vector4: ...
    @property
    def glbo(self) -> I32Vector4: ...
    @property
    def glbl(self) -> I32Vector4: ...
    @property
    def glor(self) -> I32Vector4: ...
    @property
    def glog(self) -> I32Vector4: ...
    @property
    def glob(self) -> I32Vector4: ...
    @property
    def gloo(self) -> I32Vector4: ...
    @property
    def glol(self) -> I32Vector4: ...
    @property
    def gllr(self) -> I32Vector4: ...
    @property
    def gllg(self) -> I32Vector4: ...
    @property
    def gllb(self) -> I32Vector4: ...
    @property
    def gllo(self) -> I32Vector4: ...
    @property
    def glll(self) -> I32Vector4: ...
    @property
    def brrr(self) -> I32Vector4: ...
    @property
    def brrg(self) -> I32Vector4: ...
    @property
    def brrb(self) -> I32Vector4: ...
    @property
    def brro(self) -> I32Vector4: ...
    @property
    def brrl(self) -> I32Vector4: ...
    @property
    def brgr(self) -> I32Vector4: ...
    @property
    def brgg(self) -> I32Vector4: ...
    @property
    def brgb(self) -> I32Vector4: ...
    @property
    def brgo(self) -> I32Vector4: ...
    @property
    def brgl(self) -> I32Vector4: ...
    @property
    def brbr(self) -> I32Vector4: ...
    @property
    def brbg(self) -> I32Vector4: ...
    @property
    def brbb(self) -> I32Vector4: ...
    @property
    def brbo(self) -> I32Vector4: ...
    @property
    def brbl(self) -> I32Vector4: ...
    @property
    def bror(self) -> I32Vector4: ...
    @property
    def brog(self) -> I32Vector4: ...
    @property
    def brob(self) -> I32Vector4: ...
    @property
    def broo(self) -> I32Vector4: ...
    @property
    def brol(self) -> I32Vector4: ...
    @property
    def brlr(self) -> I32Vector4: ...
    @property
    def brlg(self) -> I32Vector4: ...
    @property
    def brlb(self) -> I32Vector4: ...
    @property
    def brlo(self) -> I32Vector4: ...
    @property
    def brll(self) -> I32Vector4: ...
    @property
    def bgrr(self) -> I32Vector4: ...
    @property
    def bgrg(self) -> I32Vector4: ...
    @property
    def bgrb(self) -> I32Vector4: ...
    @property
    def bgro(self) -> I32Vector4: ...
    @property
    def bgrl(self) -> I32Vector4: ...
    @property
    def bggr(self) -> I32Vector4: ...
    @property
    def bggg(self) -> I32Vector4: ...
    @property
    def bggb(self) -> I32Vector4: ...
    @property
    def bggo(self) -> I32Vector4: ...
    @property
    def bggl(self) -> I32Vector4: ...
    @property
    def bgbr(self) -> I32Vector4: ...
    @property
    def bgbg(self) -> I32Vector4: ...
    @property
    def bgbb(self) -> I32Vector4: ...
    @property
    def bgbo(self) -> I32Vector4: ...
    @property
    def bgbl(self) -> I32Vector4: ...
    @property
    def bgor(self) -> I32Vector4: ...
    @property
    def bgog(self) -> I32Vector4: ...
    @property
    def bgob(self) -> I32Vector4: ...
    @property
    def bgoo(self) -> I32Vector4: ...
    @property
    def bgol(self) -> I32Vector4: ...
    @property
    def bglr(self) -> I32Vector4: ...
    @property
    def bglg(self) -> I32Vector4: ...
    @property
    def bglb(self) -> I32Vector4: ...
    @property
    def bglo(self) -> I32Vector4: ...
    @property
    def bgll(self) -> I32Vector4: ...
    @property
    def bbrr(self) -> I32Vector4: ...
    @property
    def bbrg(self) -> I32Vector4: ...
    @property
    def bbrb(self) -> I32Vector4: ...
    @property
    def bbro(self) -> I32Vector4: ...
    @property
    def bbrl(self) -> I32Vector4: ...
    @property
    def bbgr(self) -> I32Vector4: ...
    @property
    def bbgg(self) -> I32Vector4: ...
    @property
    def bbgb(self) -> I32Vector4: ...
    @property
    def bbgo(self) -> I32Vector4: ...
    @property
    def bbgl(self) -> I32Vector4: ...
    @property
    def bbbr(self) -> I32Vector4: ...
    @property
    def bbbg(self) -> I32Vector4: ...
    @property
    def bbbb(self) -> I32Vector4: ...
    @property
    def bbbo(self) -> I32Vector4: ...
    @property
    def bbbl(self) -> I32Vector4: ...
    @property
    def bbor(self) -> I32Vector4: ...
    @property
    def bbog(self) -> I32Vector4: ...
    @property
    def bbob(self) -> I32Vector4: ...
    @property
    def bboo(self) -> I32Vector4: ...
    @property
    def bbol(self) -> I32Vector4: ...
    @property
    def bblr(self) -> I32Vector4: ...
    @property
    def bblg(self) -> I32Vector4: ...
    @property
    def bblb(self) -> I32Vector4: ...
    @property
    def bblo(self) -> I32Vector4: ...
    @property
    def bbll(self) -> I32Vector4: ...
    @property
    def borr(self) -> I32Vector4: ...
    @property
    def borg(self) -> I32Vector4: ...
    @property
    def borb(self) -> I32Vector4: ...
    @property
    def boro(self) -> I32Vector4: ...
    @property
    def borl(self) -> I32Vector4: ...
    @property
    def bogr(self) -> I32Vector4: ...
    @property
    def bogg(self) -> I32Vector4: ...
    @property
    def bogb(self) -> I32Vector4: ...
    @property
    def bogo(self) -> I32Vector4: ...
    @property
    def bogl(self) -> I32Vector4: ...
    @property
    def bobr(self) -> I32Vector4: ...
    @property
    def bobg(self) -> I32Vector4: ...
    @property
    def bobb(self) -> I32Vector4: ...
    @property
    def bobo(self) -> I32Vector4: ...
    @property
    def bobl(self) -> I32Vector4: ...
    @property
    def boor(self) -> I32Vector4: ...
    @property
    def boog(self) -> I32Vector4: ...
    @property
    def boob(self) -> I32Vector4: ...
    @property
    def booo(self) -> I32Vector4: ...
    @property
    def bool(self) -> I32Vector4: ...
    @property
    def bolr(self) -> I32Vector4: ...
    @property
    def bolg(self) -> I32Vector4: ...
    @property
    def bolb(self) -> I32Vector4: ...
    @property
    def bolo(self) -> I32Vector4: ...
    @property
    def boll(self) -> I32Vector4: ...
    @property
    def blrr(self) -> I32Vector4: ...
    @property
    def blrg(self) -> I32Vector4: ...
    @property
    def blrb(self) -> I32Vector4: ...
    @property
    def blro(self) -> I32Vector4: ...
    @property
    def blrl(self) -> I32Vector4: ...
    @property
    def blgr(self) -> I32Vector4: ...
    @property
    def blgg(self) -> I32Vector4: ...
    @property
    def blgb(self) -> I32Vector4: ...
    @property
    def blgo(self) -> I32Vector4: ...
    @property
    def blgl(self) -> I32Vector4: ...
    @property
    def blbr(self) -> I32Vector4: ...
    @property
    def blbg(self) -> I32Vector4: ...
    @property
    def blbb(self) -> I32Vector4: ...
    @property
    def blbo(self) -> I32Vector4: ...
    @property
    def blbl(self) -> I32Vector4: ...
    @property
    def blor(self) -> I32Vector4: ...
    @property
    def blog(self) -> I32Vector4: ...
    @property
    def blob(self) -> I32Vector4: ...
    @property
    def bloo(self) -> I32Vector4: ...
    @property
    def blol(self) -> I32Vector4: ...
    @property
    def bllr(self) -> I32Vector4: ...
    @property
    def bllg(self) -> I32Vector4: ...
    @property
    def bllb(self) -> I32Vector4: ...
    @property
    def bllo(self) -> I32Vector4: ...
    @property
    def blll(self) -> I32Vector4: ...
    @property
    def orrr(self) -> I32Vector4: ...
    @property
    def orrg(self) -> I32Vector4: ...
    @property
    def orrb(self) -> I32Vector4: ...
    @property
    def orro(self) -> I32Vector4: ...
    @property
    def orrl(self) -> I32Vector4: ...
    @property
    def orgr(self) -> I32Vector4: ...
    @property
    def orgg(self) -> I32Vector4: ...
    @property
    def orgb(self) -> I32Vector4: ...
    @property
    def orgo(self) -> I32Vector4: ...
    @property
    def orgl(self) -> I32Vector4: ...
    @property
    def orbr(self) -> I32Vector4: ...
    @property
    def orbg(self) -> I32Vector4: ...
    @property
    def orbb(self) -> I32Vector4: ...
    @property
    def orbo(self) -> I32Vector4: ...
    @property
    def orbl(self) -> I32Vector4: ...
    @property
    def oror(self) -> I32Vector4: ...
    @property
    def orog(self) -> I32Vector4: ...
    @property
    def orob(self) -> I32Vector4: ...
    @property
    def oroo(self) -> I32Vector4: ...
    @property
    def orol(self) -> I32Vector4: ...
    @property
    def orlr(self) -> I32Vector4: ...
    @property
    def orlg(self) -> I32Vector4: ...
    @property
    def orlb(self) -> I32Vector4: ...
    @property
    def orlo(self) -> I32Vector4: ...
    @property
    def orll(self) -> I32Vector4: ...
    @property
    def ogrr(self) -> I32Vector4: ...
    @property
    def ogrg(self) -> I32Vector4: ...
    @property
    def ogrb(self) -> I32Vector4: ...
    @property
    def ogro(self) -> I32Vector4: ...
    @property
    def ogrl(self) -> I32Vector4: ...
    @property
    def oggr(self) -> I32Vector4: ...
    @property
    def oggg(self) -> I32Vector4: ...
    @property
    def oggb(self) -> I32Vector4: ...
    @property
    def oggo(self) -> I32Vector4: ...
    @property
    def oggl(self) -> I32Vector4: ...
    @property
    def ogbr(self) -> I32Vector4: ...
    @property
    def ogbg(self) -> I32Vector4: ...
    @property
    def ogbb(self) -> I32Vector4: ...
    @property
    def ogbo(self) -> I32Vector4: ...
    @property
    def ogbl(self) -> I32Vector4: ...
    @property
    def ogor(self) -> I32Vector4: ...
    @property
    def ogog(self) -> I32Vector4: ...
    @property
    def ogob(self) -> I32Vector4: ...
    @property
    def ogoo(self) -> I32Vector4: ...
    @property
    def ogol(self) -> I32Vector4: ...
    @property
    def oglr(self) -> I32Vector4: ...
    @property
    def oglg(self) -> I32Vector4: ...
    @property
    def oglb(self) -> I32Vector4: ...
    @property
    def oglo(self) -> I32Vector4: ...
    @property
    def ogll(self) -> I32Vector4: ...
    @property
    def obrr(self) -> I32Vector4: ...
    @property
    def obrg(self) -> I32Vector4: ...
    @property
    def obrb(self) -> I32Vector4: ...
    @property
    def obro(self) -> I32Vector4: ...
    @property
    def obrl(self) -> I32Vector4: ...
    @property
    def obgr(self) -> I32Vector4: ...
    @property
    def obgg(self) -> I32Vector4: ...
    @property
    def obgb(self) -> I32Vector4: ...
    @property
    def obgo(self) -> I32Vector4: ...
    @property
    def obgl(self) -> I32Vector4: ...
    @property
    def obbr(self) -> I32Vector4: ...
    @property
    def obbg(self) -> I32Vector4: ...
    @property
    def obbb(self) -> I32Vector4: ...
    @property
    def obbo(self) -> I32Vector4: ...
    @property
    def obbl(self) -> I32Vector4: ...
    @property
    def obor(self) -> I32Vector4: ...
    @property
    def obog(self) -> I32Vector4: ...
    @property
    def obob(self) -> I32Vector4: ...
    @property
    def oboo(self) -> I32Vector4: ...
    @property
    def obol(self) -> I32Vector4: ...
    @property
    def oblr(self) -> I32Vector4: ...
    @property
    def oblg(self) -> I32Vector4: ...
    @property
    def oblb(self) -> I32Vector4: ...
    @property
    def oblo(self) -> I32Vector4: ...
    @property
    def obll(self) -> I32Vector4: ...
    @property
    def oorr(self) -> I32Vector4: ...
    @property
    def oorg(self) -> I32Vector4: ...
    @property
    def oorb(self) -> I32Vector4: ...
    @property
    def ooro(self) -> I32Vector4: ...
    @property
    def oorl(self) -> I32Vector4: ...
    @property
    def oogr(self) -> I32Vector4: ...
    @property
    def oogg(self) -> I32Vector4: ...
    @property
    def oogb(self) -> I32Vector4: ...
    @property
    def oogo(self) -> I32Vector4: ...
    @property
    def oogl(self) -> I32Vector4: ...
    @property
    def oobr(self) -> I32Vector4: ...
    @property
    def oobg(self) -> I32Vector4: ...
    @property
    def oobb(self) -> I32Vector4: ...
    @property
    def oobo(self) -> I32Vector4: ...
    @property
    def oobl(self) -> I32Vector4: ...
    @property
    def ooor(self) -> I32Vector4: ...
    @property
    def ooog(self) -> I32Vector4: ...
    @property
    def ooob(self) -> I32Vector4: ...
    @property
    def oolr(self) -> I32Vector4: ...
    @property
    def oolg(self) -> I32Vector4: ...
    @property
    def oolb(self) -> I32Vector4: ...
    @property
    def olrr(self) -> I32Vector4: ...
    @property
    def olrg(self) -> I32Vector4: ...
    @property
    def olrb(self) -> I32Vector4: ...
    @property
    def olro(self) -> I32Vector4: ...
    @property
    def olrl(self) -> I32Vector4: ...
    @property
    def olgr(self) -> I32Vector4: ...
    @property
    def olgg(self) -> I32Vector4: ...
    @property
    def olgb(self) -> I32Vector4: ...
    @property
    def olgo(self) -> I32Vector4: ...
    @property
    def olgl(self) -> I32Vector4: ...
    @property
    def olbr(self) -> I32Vector4: ...
    @property
    def olbg(self) -> I32Vector4: ...
    @property
    def olbb(self) -> I32Vector4: ...
    @property
    def olbo(self) -> I32Vector4: ...
    @property
    def olbl(self) -> I32Vector4: ...
    @property
    def olor(self) -> I32Vector4: ...
    @property
    def olog(self) -> I32Vector4: ...
    @property
    def olob(self) -> I32Vector4: ...
    @property
    def ollr(self) -> I32Vector4: ...
    @property
    def ollg(self) -> I32Vector4: ...
    @property
    def ollb(self) -> I32Vector4: ...
    @property
    def lrrr(self) -> I32Vector4: ...
    @property
    def lrrg(self) -> I32Vector4: ...
    @property
    def lrrb(self) -> I32Vector4: ...
    @property
    def lrro(self) -> I32Vector4: ...
    @property
    def lrrl(self) -> I32Vector4: ...
    @property
    def lrgr(self) -> I32Vector4: ...
    @property
    def lrgg(self) -> I32Vector4: ...
    @property
    def lrgb(self) -> I32Vector4: ...
    @property
    def lrgo(self) -> I32Vector4: ...
    @property
    def lrgl(self) -> I32Vector4: ...
    @property
    def lrbr(self) -> I32Vector4: ...
    @property
    def lrbg(self) -> I32Vector4: ...
    @property
    def lrbb(self) -> I32Vector4: ...
    @property
    def lrbo(self) -> I32Vector4: ...
    @property
    def lrbl(self) -> I32Vector4: ...
    @property
    def lror(self) -> I32Vector4: ...
    @property
    def lrog(self) -> I32Vector4: ...
    @property
    def lrob(self) -> I32Vector4: ...
    @property
    def lroo(self) -> I32Vector4: ...
    @property
    def lrol(self) -> I32Vector4: ...
    @property
    def lrlr(self) -> I32Vector4: ...
    @property
    def lrlg(self) -> I32Vector4: ...
    @property
    def lrlb(self) -> I32Vector4: ...
    @property
    def lrlo(self) -> I32Vector4: ...
    @property
    def lrll(self) -> I32Vector4: ...
    @property
    def lgrr(self) -> I32Vector4: ...
    @property
    def lgrg(self) -> I32Vector4: ...
    @property
    def lgrb(self) -> I32Vector4: ...
    @property
    def lgro(self) -> I32Vector4: ...
    @property
    def lgrl(self) -> I32Vector4: ...
    @property
    def lggr(self) -> I32Vector4: ...
    @property
    def lggg(self) -> I32Vector4: ...
    @property
    def lggb(self) -> I32Vector4: ...
    @property
    def lggo(self) -> I32Vector4: ...
    @property
    def lggl(self) -> I32Vector4: ...
    @property
    def lgbr(self) -> I32Vector4: ...
    @property
    def lgbg(self) -> I32Vector4: ...
    @property
    def lgbb(self) -> I32Vector4: ...
    @property
    def lgbo(self) -> I32Vector4: ...
    @property
    def lgbl(self) -> I32Vector4: ...
    @property
    def lgor(self) -> I32Vector4: ...
    @property
    def lgog(self) -> I32Vector4: ...
    @property
    def lgob(self) -> I32Vector4: ...
    @property
    def lgoo(self) -> I32Vector4: ...
    @property
    def lgol(self) -> I32Vector4: ...
    @property
    def lglr(self) -> I32Vector4: ...
    @property
    def lglg(self) -> I32Vector4: ...
    @property
    def lglb(self) -> I32Vector4: ...
    @property
    def lglo(self) -> I32Vector4: ...
    @property
    def lgll(self) -> I32Vector4: ...
    @property
    def lbrr(self) -> I32Vector4: ...
    @property
    def lbrg(self) -> I32Vector4: ...
    @property
    def lbrb(self) -> I32Vector4: ...
    @property
    def lbro(self) -> I32Vector4: ...
    @property
    def lbrl(self) -> I32Vector4: ...
    @property
    def lbgr(self) -> I32Vector4: ...
    @property
    def lbgg(self) -> I32Vector4: ...
    @property
    def lbgb(self) -> I32Vector4: ...
    @property
    def lbgo(self) -> I32Vector4: ...
    @property
    def lbgl(self) -> I32Vector4: ...
    @property
    def lbbr(self) -> I32Vector4: ...
    @property
    def lbbg(self) -> I32Vector4: ...
    @property
    def lbbb(self) -> I32Vector4: ...
    @property
    def lbbo(self) -> I32Vector4: ...
    @property
    def lbbl(self) -> I32Vector4: ...
    @property
    def lbor(self) -> I32Vector4: ...
    @property
    def lbog(self) -> I32Vector4: ...
    @property
    def lbob(self) -> I32Vector4: ...
    @property
    def lboo(self) -> I32Vector4: ...
    @property
    def lbol(self) -> I32Vector4: ...
    @property
    def lblr(self) -> I32Vector4: ...
    @property
    def lblg(self) -> I32Vector4: ...
    @property
    def lblb(self) -> I32Vector4: ...
    @property
    def lblo(self) -> I32Vector4: ...
    @property
    def lbll(self) -> I32Vector4: ...
    @property
    def lorr(self) -> I32Vector4: ...
    @property
    def lorg(self) -> I32Vector4: ...
    @property
    def lorb(self) -> I32Vector4: ...
    @property
    def loro(self) -> I32Vector4: ...
    @property
    def lorl(self) -> I32Vector4: ...
    @property
    def logr(self) -> I32Vector4: ...
    @property
    def logg(self) -> I32Vector4: ...
    @property
    def logb(self) -> I32Vector4: ...
    @property
    def logo(self) -> I32Vector4: ...
    @property
    def logl(self) -> I32Vector4: ...
    @property
    def lobr(self) -> I32Vector4: ...
    @property
    def lobg(self) -> I32Vector4: ...
    @property
    def lobb(self) -> I32Vector4: ...
    @property
    def lobo(self) -> I32Vector4: ...
    @property
    def lobl(self) -> I32Vector4: ...
    @property
    def loor(self) -> I32Vector4: ...
    @property
    def loog(self) -> I32Vector4: ...
    @property
    def loob(self) -> I32Vector4: ...
    @property
    def lolr(self) -> I32Vector4: ...
    @property
    def lolg(self) -> I32Vector4: ...
    @property
    def lolb(self) -> I32Vector4: ...
    @property
    def llrr(self) -> I32Vector4: ...
    @property
    def llrg(self) -> I32Vector4: ...
    @property
    def llrb(self) -> I32Vector4: ...
    @property
    def llro(self) -> I32Vector4: ...
    @property
    def llrl(self) -> I32Vector4: ...
    @property
    def llgr(self) -> I32Vector4: ...
    @property
    def llgg(self) -> I32Vector4: ...
    @property
    def llgb(self) -> I32Vector4: ...
    @property
    def llgo(self) -> I32Vector4: ...
    @property
    def llgl(self) -> I32Vector4: ...
    @property
    def llbr(self) -> I32Vector4: ...
    @property
    def llbg(self) -> I32Vector4: ...
    @property
    def llbb(self) -> I32Vector4: ...
    @property
    def llbo(self) -> I32Vector4: ...
    @property
    def llbl(self) -> I32Vector4: ...
    @property
    def llor(self) -> I32Vector4: ...
    @property
    def llog(self) -> I32Vector4: ...
    @property
    def llob(self) -> I32Vector4: ...
    @property
    def lllr(self) -> I32Vector4: ...
    @property
    def lllg(self) -> I32Vector4: ...
    @property
    def lllb(self) -> I32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> I32Vector2: ...
    @property
    def st(self) -> I32Vector2: ...
    @property
    def sq(self) -> I32Vector2: ...
    @property
    def so(self) -> I32Vector2: ...
    @property
    def sl(self) -> I32Vector2: ...
    @property
    def ts(self) -> I32Vector2: ...
    @property
    def tt(self) -> I32Vector2: ...
    @property
    def tq(self) -> I32Vector2: ...
    @property
    def to(self) -> I32Vector2: ...
    @property
    def tl(self) -> I32Vector2: ...
    @property
    def qs(self) -> I32Vector2: ...
    @property
    def qt(self) -> I32Vector2: ...
    @property
    def qq(self) -> I32Vector2: ...
    @property
    def qo(self) -> I32Vector2: ...
    @property
    def ql(self) -> I32Vector2: ...
    @property
    def os(self) -> I32Vector2: ...
    @property
    def ot(self) -> I32Vector2: ...
    @property
    def oq(self) -> I32Vector2: ...
    @property
    def ls(self) -> I32Vector2: ...
    @property
    def lt(self) -> I32Vector2: ...
    @property
    def lq(self) -> I32Vector2: ...
    @property
    def sss(self) -> I32Vector3: ...
    @property
    def sst(self) -> I32Vector3: ...
    @property
    def ssq(self) -> I32Vector3: ...
    @property
    def sso(self) -> I32Vector3: ...
    @property
    def ssl(self) -> I32Vector3: ...
    @property
    def sts(self) -> I32Vector3: ...
    @property
    def stt(self) -> I32Vector3: ...
    @property
    def stq(self) -> I32Vector3: ...
    @property
    def sto(self) -> I32Vector3: ...
    @property
    def stl(self) -> I32Vector3: ...
    @property
    def sqs(self) -> I32Vector3: ...
    @property
    def sqt(self) -> I32Vector3: ...
    @property
    def sqq(self) -> I32Vector3: ...
    @property
    def sqo(self) -> I32Vector3: ...
    @property
    def sql(self) -> I32Vector3: ...
    @property
    def sos(self) -> I32Vector3: ...
    @property
    def sot(self) -> I32Vector3: ...
    @property
    def soq(self) -> I32Vector3: ...
    @property
    def soo(self) -> I32Vector3: ...
    @property
    def sol(self) -> I32Vector3: ...
    @property
    def sls(self) -> I32Vector3: ...
    @property
    def slt(self) -> I32Vector3: ...
    @property
    def slq(self) -> I32Vector3: ...
    @property
    def slo(self) -> I32Vector3: ...
    @property
    def sll(self) -> I32Vector3: ...
    @property
    def tss(self) -> I32Vector3: ...
    @property
    def tst(self) -> I32Vector3: ...
    @property
    def tsq(self) -> I32Vector3: ...
    @property
    def tso(self) -> I32Vector3: ...
    @property
    def tsl(self) -> I32Vector3: ...
    @property
    def tts(self) -> I32Vector3: ...
    @property
    def ttt(self) -> I32Vector3: ...
    @property
    def ttq(self) -> I32Vector3: ...
    @property
    def tto(self) -> I32Vector3: ...
    @property
    def ttl(self) -> I32Vector3: ...
    @property
    def tqs(self) -> I32Vector3: ...
    @property
    def tqt(self) -> I32Vector3: ...
    @property
    def tqq(self) -> I32Vector3: ...
    @property
    def tqo(self) -> I32Vector3: ...
    @property
    def tql(self) -> I32Vector3: ...
    @property
    def tos(self) -> I32Vector3: ...
    @property
    def tot(self) -> I32Vector3: ...
    @property
    def toq(self) -> I32Vector3: ...
    @property
    def too(self) -> I32Vector3: ...
    @property
    def tol(self) -> I32Vector3: ...
    @property
    def tls(self) -> I32Vector3: ...
    @property
    def tlt(self) -> I32Vector3: ...
    @property
    def tlq(self) -> I32Vector3: ...
    @property
    def tlo(self) -> I32Vector3: ...
    @property
    def tll(self) -> I32Vector3: ...
    @property
    def qss(self) -> I32Vector3: ...
    @property
    def qst(self) -> I32Vector3: ...
    @property
    def qsq(self) -> I32Vector3: ...
    @property
    def qso(self) -> I32Vector3: ...
    @property
    def qsl(self) -> I32Vector3: ...
    @property
    def qts(self) -> I32Vector3: ...
    @property
    def qtt(self) -> I32Vector3: ...
    @property
    def qtq(self) -> I32Vector3: ...
    @property
    def qto(self) -> I32Vector3: ...
    @property
    def qtl(self) -> I32Vector3: ...
    @property
    def qqs(self) -> I32Vector3: ...
    @property
    def qqt(self) -> I32Vector3: ...
    @property
    def qqq(self) -> I32Vector3: ...
    @property
    def qqo(self) -> I32Vector3: ...
    @property
    def qql(self) -> I32Vector3: ...
    @property
    def qos(self) -> I32Vector3: ...
    @property
    def qot(self) -> I32Vector3: ...
    @property
    def qoq(self) -> I32Vector3: ...
    @property
    def qoo(self) -> I32Vector3: ...
    @property
    def qol(self) -> I32Vector3: ...
    @property
    def qls(self) -> I32Vector3: ...
    @property
    def qlt(self) -> I32Vector3: ...
    @property
    def qlq(self) -> I32Vector3: ...
    @property
    def qlo(self) -> I32Vector3: ...
    @property
    def qll(self) -> I32Vector3: ...
    @property
    def oss(self) -> I32Vector3: ...
    @property
    def ost(self) -> I32Vector3: ...
    @property
    def osq(self) -> I32Vector3: ...
    @property
    def oso(self) -> I32Vector3: ...
    @property
    def osl(self) -> I32Vector3: ...
    @property
    def ots(self) -> I32Vector3: ...
    @property
    def ott(self) -> I32Vector3: ...
    @property
    def otq(self) -> I32Vector3: ...
    @property
    def oto(self) -> I32Vector3: ...
    @property
    def otl(self) -> I32Vector3: ...
    @property
    def oqs(self) -> I32Vector3: ...
    @property
    def oqt(self) -> I32Vector3: ...
    @property
    def oqq(self) -> I32Vector3: ...
    @property
    def oqo(self) -> I32Vector3: ...
    @property
    def oql(self) -> I32Vector3: ...
    @property
    def oos(self) -> I32Vector3: ...
    @property
    def oot(self) -> I32Vector3: ...
    @property
    def ooq(self) -> I32Vector3: ...
    @property
    def ols(self) -> I32Vector3: ...
    @property
    def olt(self) -> I32Vector3: ...
    @property
    def olq(self) -> I32Vector3: ...
    @property
    def lss(self) -> I32Vector3: ...
    @property
    def lst(self) -> I32Vector3: ...
    @property
    def lsq(self) -> I32Vector3: ...
    @property
    def lso(self) -> I32Vector3: ...
    @property
    def lsl(self) -> I32Vector3: ...
    @property
    def lts(self) -> I32Vector3: ...
    @property
    def ltt(self) -> I32Vector3: ...
    @property
    def ltq(self) -> I32Vector3: ...
    @property
    def lto(self) -> I32Vector3: ...
    @property
    def ltl(self) -> I32Vector3: ...
    @property
    def lqs(self) -> I32Vector3: ...
    @property
    def lqt(self) -> I32Vector3: ...
    @property
    def lqq(self) -> I32Vector3: ...
    @property
    def lqo(self) -> I32Vector3: ...
    @property
    def lql(self) -> I32Vector3: ...
    @property
    def los(self) -> I32Vector3: ...
    @property
    def lot(self) -> I32Vector3: ...
    @property
    def loq(self) -> I32Vector3: ...
    @property
    def lls(self) -> I32Vector3: ...
    @property
    def llt(self) -> I32Vector3: ...
    @property
    def llq(self) -> I32Vector3: ...
    @property
    def ssss(self) -> I32Vector4: ...
    @property
    def ssst(self) -> I32Vector4: ...
    @property
    def sssq(self) -> I32Vector4: ...
    @property
    def ssso(self) -> I32Vector4: ...
    @property
    def sssl(self) -> I32Vector4: ...
    @property
    def ssts(self) -> I32Vector4: ...
    @property
    def sstt(self) -> I32Vector4: ...
    @property
    def sstq(self) -> I32Vector4: ...
    @property
    def ssto(self) -> I32Vector4: ...
    @property
    def sstl(self) -> I32Vector4: ...
    @property
    def ssqs(self) -> I32Vector4: ...
    @property
    def ssqt(self) -> I32Vector4: ...
    @property
    def ssqq(self) -> I32Vector4: ...
    @property
    def ssqo(self) -> I32Vector4: ...
    @property
    def ssql(self) -> I32Vector4: ...
    @property
    def ssos(self) -> I32Vector4: ...
    @property
    def ssot(self) -> I32Vector4: ...
    @property
    def ssoq(self) -> I32Vector4: ...
    @property
    def ssoo(self) -> I32Vector4: ...
    @property
    def ssol(self) -> I32Vector4: ...
    @property
    def ssls(self) -> I32Vector4: ...
    @property
    def sslt(self) -> I32Vector4: ...
    @property
    def sslq(self) -> I32Vector4: ...
    @property
    def sslo(self) -> I32Vector4: ...
    @property
    def ssll(self) -> I32Vector4: ...
    @property
    def stss(self) -> I32Vector4: ...
    @property
    def stst(self) -> I32Vector4: ...
    @property
    def stsq(self) -> I32Vector4: ...
    @property
    def stso(self) -> I32Vector4: ...
    @property
    def stsl(self) -> I32Vector4: ...
    @property
    def stts(self) -> I32Vector4: ...
    @property
    def sttt(self) -> I32Vector4: ...
    @property
    def sttq(self) -> I32Vector4: ...
    @property
    def stto(self) -> I32Vector4: ...
    @property
    def sttl(self) -> I32Vector4: ...
    @property
    def stqs(self) -> I32Vector4: ...
    @property
    def stqt(self) -> I32Vector4: ...
    @property
    def stqq(self) -> I32Vector4: ...
    @property
    def stqo(self) -> I32Vector4: ...
    @property
    def stql(self) -> I32Vector4: ...
    @property
    def stos(self) -> I32Vector4: ...
    @property
    def stot(self) -> I32Vector4: ...
    @property
    def stoq(self) -> I32Vector4: ...
    @property
    def stoo(self) -> I32Vector4: ...
    @property
    def stol(self) -> I32Vector4: ...
    @property
    def stls(self) -> I32Vector4: ...
    @property
    def stlt(self) -> I32Vector4: ...
    @property
    def stlq(self) -> I32Vector4: ...
    @property
    def stlo(self) -> I32Vector4: ...
    @property
    def stll(self) -> I32Vector4: ...
    @property
    def sqss(self) -> I32Vector4: ...
    @property
    def sqst(self) -> I32Vector4: ...
    @property
    def sqsq(self) -> I32Vector4: ...
    @property
    def sqso(self) -> I32Vector4: ...
    @property
    def sqsl(self) -> I32Vector4: ...
    @property
    def sqts(self) -> I32Vector4: ...
    @property
    def sqtt(self) -> I32Vector4: ...
    @property
    def sqtq(self) -> I32Vector4: ...
    @property
    def sqto(self) -> I32Vector4: ...
    @property
    def sqtl(self) -> I32Vector4: ...
    @property
    def sqqs(self) -> I32Vector4: ...
    @property
    def sqqt(self) -> I32Vector4: ...
    @property
    def sqqq(self) -> I32Vector4: ...
    @property
    def sqqo(self) -> I32Vector4: ...
    @property
    def sqql(self) -> I32Vector4: ...
    @property
    def sqos(self) -> I32Vector4: ...
    @property
    def sqot(self) -> I32Vector4: ...
    @property
    def sqoq(self) -> I32Vector4: ...
    @property
    def sqoo(self) -> I32Vector4: ...
    @property
    def sqol(self) -> I32Vector4: ...
    @property
    def sqls(self) -> I32Vector4: ...
    @property
    def sqlt(self) -> I32Vector4: ...
    @property
    def sqlq(self) -> I32Vector4: ...
    @property
    def sqlo(self) -> I32Vector4: ...
    @property
    def sqll(self) -> I32Vector4: ...
    @property
    def soss(self) -> I32Vector4: ...
    @property
    def sost(self) -> I32Vector4: ...
    @property
    def sosq(self) -> I32Vector4: ...
    @property
    def soso(self) -> I32Vector4: ...
    @property
    def sosl(self) -> I32Vector4: ...
    @property
    def sots(self) -> I32Vector4: ...
    @property
    def sott(self) -> I32Vector4: ...
    @property
    def sotq(self) -> I32Vector4: ...
    @property
    def soto(self) -> I32Vector4: ...
    @property
    def sotl(self) -> I32Vector4: ...
    @property
    def soqs(self) -> I32Vector4: ...
    @property
    def soqt(self) -> I32Vector4: ...
    @property
    def soqq(self) -> I32Vector4: ...
    @property
    def soqo(self) -> I32Vector4: ...
    @property
    def soql(self) -> I32Vector4: ...
    @property
    def soos(self) -> I32Vector4: ...
    @property
    def soot(self) -> I32Vector4: ...
    @property
    def sooq(self) -> I32Vector4: ...
    @property
    def sooo(self) -> I32Vector4: ...
    @property
    def sool(self) -> I32Vector4: ...
    @property
    def sols(self) -> I32Vector4: ...
    @property
    def solt(self) -> I32Vector4: ...
    @property
    def solq(self) -> I32Vector4: ...
    @property
    def solo(self) -> I32Vector4: ...
    @property
    def soll(self) -> I32Vector4: ...
    @property
    def slss(self) -> I32Vector4: ...
    @property
    def slst(self) -> I32Vector4: ...
    @property
    def slsq(self) -> I32Vector4: ...
    @property
    def slso(self) -> I32Vector4: ...
    @property
    def slsl(self) -> I32Vector4: ...
    @property
    def slts(self) -> I32Vector4: ...
    @property
    def sltt(self) -> I32Vector4: ...
    @property
    def sltq(self) -> I32Vector4: ...
    @property
    def slto(self) -> I32Vector4: ...
    @property
    def sltl(self) -> I32Vector4: ...
    @property
    def slqs(self) -> I32Vector4: ...
    @property
    def slqt(self) -> I32Vector4: ...
    @property
    def slqq(self) -> I32Vector4: ...
    @property
    def slqo(self) -> I32Vector4: ...
    @property
    def slql(self) -> I32Vector4: ...
    @property
    def slos(self) -> I32Vector4: ...
    @property
    def slot(self) -> I32Vector4: ...
    @property
    def sloq(self) -> I32Vector4: ...
    @property
    def sloo(self) -> I32Vector4: ...
    @property
    def slol(self) -> I32Vector4: ...
    @property
    def slls(self) -> I32Vector4: ...
    @property
    def sllt(self) -> I32Vector4: ...
    @property
    def sllq(self) -> I32Vector4: ...
    @property
    def sllo(self) -> I32Vector4: ...
    @property
    def slll(self) -> I32Vector4: ...
    @property
    def tsss(self) -> I32Vector4: ...
    @property
    def tsst(self) -> I32Vector4: ...
    @property
    def tssq(self) -> I32Vector4: ...
    @property
    def tsso(self) -> I32Vector4: ...
    @property
    def tssl(self) -> I32Vector4: ...
    @property
    def tsts(self) -> I32Vector4: ...
    @property
    def tstt(self) -> I32Vector4: ...
    @property
    def tstq(self) -> I32Vector4: ...
    @property
    def tsto(self) -> I32Vector4: ...
    @property
    def tstl(self) -> I32Vector4: ...
    @property
    def tsqs(self) -> I32Vector4: ...
    @property
    def tsqt(self) -> I32Vector4: ...
    @property
    def tsqq(self) -> I32Vector4: ...
    @property
    def tsqo(self) -> I32Vector4: ...
    @property
    def tsql(self) -> I32Vector4: ...
    @property
    def tsos(self) -> I32Vector4: ...
    @property
    def tsot(self) -> I32Vector4: ...
    @property
    def tsoq(self) -> I32Vector4: ...
    @property
    def tsoo(self) -> I32Vector4: ...
    @property
    def tsol(self) -> I32Vector4: ...
    @property
    def tsls(self) -> I32Vector4: ...
    @property
    def tslt(self) -> I32Vector4: ...
    @property
    def tslq(self) -> I32Vector4: ...
    @property
    def tslo(self) -> I32Vector4: ...
    @property
    def tsll(self) -> I32Vector4: ...
    @property
    def ttss(self) -> I32Vector4: ...
    @property
    def ttst(self) -> I32Vector4: ...
    @property
    def ttsq(self) -> I32Vector4: ...
    @property
    def ttso(self) -> I32Vector4: ...
    @property
    def ttsl(self) -> I32Vector4: ...
    @property
    def ttts(self) -> I32Vector4: ...
    @property
    def tttt(self) -> I32Vector4: ...
    @property
    def tttq(self) -> I32Vector4: ...
    @property
    def ttto(self) -> I32Vector4: ...
    @property
    def tttl(self) -> I32Vector4: ...
    @property
    def ttqs(self) -> I32Vector4: ...
    @property
    def ttqt(self) -> I32Vector4: ...
    @property
    def ttqq(self) -> I32Vector4: ...
    @property
    def ttqo(self) -> I32Vector4: ...
    @property
    def ttql(self) -> I32Vector4: ...
    @property
    def ttos(self) -> I32Vector4: ...
    @property
    def ttot(self) -> I32Vector4: ...
    @property
    def ttoq(self) -> I32Vector4: ...
    @property
    def ttoo(self) -> I32Vector4: ...
    @property
    def ttol(self) -> I32Vector4: ...
    @property
    def ttls(self) -> I32Vector4: ...
    @property
    def ttlt(self) -> I32Vector4: ...
    @property
    def ttlq(self) -> I32Vector4: ...
    @property
    def ttlo(self) -> I32Vector4: ...
    @property
    def ttll(self) -> I32Vector4: ...
    @property
    def tqss(self) -> I32Vector4: ...
    @property
    def tqst(self) -> I32Vector4: ...
    @property
    def tqsq(self) -> I32Vector4: ...
    @property
    def tqso(self) -> I32Vector4: ...
    @property
    def tqsl(self) -> I32Vector4: ...
    @property
    def tqts(self) -> I32Vector4: ...
    @property
    def tqtt(self) -> I32Vector4: ...
    @property
    def tqtq(self) -> I32Vector4: ...
    @property
    def tqto(self) -> I32Vector4: ...
    @property
    def tqtl(self) -> I32Vector4: ...
    @property
    def tqqs(self) -> I32Vector4: ...
    @property
    def tqqt(self) -> I32Vector4: ...
    @property
    def tqqq(self) -> I32Vector4: ...
    @property
    def tqqo(self) -> I32Vector4: ...
    @property
    def tqql(self) -> I32Vector4: ...
    @property
    def tqos(self) -> I32Vector4: ...
    @property
    def tqot(self) -> I32Vector4: ...
    @property
    def tqoq(self) -> I32Vector4: ...
    @property
    def tqoo(self) -> I32Vector4: ...
    @property
    def tqol(self) -> I32Vector4: ...
    @property
    def tqls(self) -> I32Vector4: ...
    @property
    def tqlt(self) -> I32Vector4: ...
    @property
    def tqlq(self) -> I32Vector4: ...
    @property
    def tqlo(self) -> I32Vector4: ...
    @property
    def tqll(self) -> I32Vector4: ...
    @property
    def toss(self) -> I32Vector4: ...
    @property
    def tost(self) -> I32Vector4: ...
    @property
    def tosq(self) -> I32Vector4: ...
    @property
    def toso(self) -> I32Vector4: ...
    @property
    def tosl(self) -> I32Vector4: ...
    @property
    def tots(self) -> I32Vector4: ...
    @property
    def tott(self) -> I32Vector4: ...
    @property
    def totq(self) -> I32Vector4: ...
    @property
    def toto(self) -> I32Vector4: ...
    @property
    def totl(self) -> I32Vector4: ...
    @property
    def toqs(self) -> I32Vector4: ...
    @property
    def toqt(self) -> I32Vector4: ...
    @property
    def toqq(self) -> I32Vector4: ...
    @property
    def toqo(self) -> I32Vector4: ...
    @property
    def toql(self) -> I32Vector4: ...
    @property
    def toos(self) -> I32Vector4: ...
    @property
    def toot(self) -> I32Vector4: ...
    @property
    def tooq(self) -> I32Vector4: ...
    @property
    def tooo(self) -> I32Vector4: ...
    @property
    def tool(self) -> I32Vector4: ...
    @property
    def tols(self) -> I32Vector4: ...
    @property
    def tolt(self) -> I32Vector4: ...
    @property
    def tolq(self) -> I32Vector4: ...
    @property
    def tolo(self) -> I32Vector4: ...
    @property
    def toll(self) -> I32Vector4: ...
    @property
    def tlss(self) -> I32Vector4: ...
    @property
    def tlst(self) -> I32Vector4: ...
    @property
    def tlsq(self) -> I32Vector4: ...
    @property
    def tlso(self) -> I32Vector4: ...
    @property
    def tlsl(self) -> I32Vector4: ...
    @property
    def tlts(self) -> I32Vector4: ...
    @property
    def tltt(self) -> I32Vector4: ...
    @property
    def tltq(self) -> I32Vector4: ...
    @property
    def tlto(self) -> I32Vector4: ...
    @property
    def tltl(self) -> I32Vector4: ...
    @property
    def tlqs(self) -> I32Vector4: ...
    @property
    def tlqt(self) -> I32Vector4: ...
    @property
    def tlqq(self) -> I32Vector4: ...
    @property
    def tlqo(self) -> I32Vector4: ...
    @property
    def tlql(self) -> I32Vector4: ...
    @property
    def tlos(self) -> I32Vector4: ...
    @property
    def tlot(self) -> I32Vector4: ...
    @property
    def tloq(self) -> I32Vector4: ...
    @property
    def tloo(self) -> I32Vector4: ...
    @property
    def tlol(self) -> I32Vector4: ...
    @property
    def tlls(self) -> I32Vector4: ...
    @property
    def tllt(self) -> I32Vector4: ...
    @property
    def tllq(self) -> I32Vector4: ...
    @property
    def tllo(self) -> I32Vector4: ...
    @property
    def tlll(self) -> I32Vector4: ...
    @property
    def qsss(self) -> I32Vector4: ...
    @property
    def qsst(self) -> I32Vector4: ...
    @property
    def qssq(self) -> I32Vector4: ...
    @property
    def qsso(self) -> I32Vector4: ...
    @property
    def qssl(self) -> I32Vector4: ...
    @property
    def qsts(self) -> I32Vector4: ...
    @property
    def qstt(self) -> I32Vector4: ...
    @property
    def qstq(self) -> I32Vector4: ...
    @property
    def qsto(self) -> I32Vector4: ...
    @property
    def qstl(self) -> I32Vector4: ...
    @property
    def qsqs(self) -> I32Vector4: ...
    @property
    def qsqt(self) -> I32Vector4: ...
    @property
    def qsqq(self) -> I32Vector4: ...
    @property
    def qsqo(self) -> I32Vector4: ...
    @property
    def qsql(self) -> I32Vector4: ...
    @property
    def qsos(self) -> I32Vector4: ...
    @property
    def qsot(self) -> I32Vector4: ...
    @property
    def qsoq(self) -> I32Vector4: ...
    @property
    def qsoo(self) -> I32Vector4: ...
    @property
    def qsol(self) -> I32Vector4: ...
    @property
    def qsls(self) -> I32Vector4: ...
    @property
    def qslt(self) -> I32Vector4: ...
    @property
    def qslq(self) -> I32Vector4: ...
    @property
    def qslo(self) -> I32Vector4: ...
    @property
    def qsll(self) -> I32Vector4: ...
    @property
    def qtss(self) -> I32Vector4: ...
    @property
    def qtst(self) -> I32Vector4: ...
    @property
    def qtsq(self) -> I32Vector4: ...
    @property
    def qtso(self) -> I32Vector4: ...
    @property
    def qtsl(self) -> I32Vector4: ...
    @property
    def qtts(self) -> I32Vector4: ...
    @property
    def qttt(self) -> I32Vector4: ...
    @property
    def qttq(self) -> I32Vector4: ...
    @property
    def qtto(self) -> I32Vector4: ...
    @property
    def qttl(self) -> I32Vector4: ...
    @property
    def qtqs(self) -> I32Vector4: ...
    @property
    def qtqt(self) -> I32Vector4: ...
    @property
    def qtqq(self) -> I32Vector4: ...
    @property
    def qtqo(self) -> I32Vector4: ...
    @property
    def qtql(self) -> I32Vector4: ...
    @property
    def qtos(self) -> I32Vector4: ...
    @property
    def qtot(self) -> I32Vector4: ...
    @property
    def qtoq(self) -> I32Vector4: ...
    @property
    def qtoo(self) -> I32Vector4: ...
    @property
    def qtol(self) -> I32Vector4: ...
    @property
    def qtls(self) -> I32Vector4: ...
    @property
    def qtlt(self) -> I32Vector4: ...
    @property
    def qtlq(self) -> I32Vector4: ...
    @property
    def qtlo(self) -> I32Vector4: ...
    @property
    def qtll(self) -> I32Vector4: ...
    @property
    def qqss(self) -> I32Vector4: ...
    @property
    def qqst(self) -> I32Vector4: ...
    @property
    def qqsq(self) -> I32Vector4: ...
    @property
    def qqso(self) -> I32Vector4: ...
    @property
    def qqsl(self) -> I32Vector4: ...
    @property
    def qqts(self) -> I32Vector4: ...
    @property
    def qqtt(self) -> I32Vector4: ...
    @property
    def qqtq(self) -> I32Vector4: ...
    @property
    def qqto(self) -> I32Vector4: ...
    @property
    def qqtl(self) -> I32Vector4: ...
    @property
    def qqqs(self) -> I32Vector4: ...
    @property
    def qqqt(self) -> I32Vector4: ...
    @property
    def qqqq(self) -> I32Vector4: ...
    @property
    def qqqo(self) -> I32Vector4: ...
    @property
    def qqql(self) -> I32Vector4: ...
    @property
    def qqos(self) -> I32Vector4: ...
    @property
    def qqot(self) -> I32Vector4: ...
    @property
    def qqoq(self) -> I32Vector4: ...
    @property
    def qqoo(self) -> I32Vector4: ...
    @property
    def qqol(self) -> I32Vector4: ...
    @property
    def qqls(self) -> I32Vector4: ...
    @property
    def qqlt(self) -> I32Vector4: ...
    @property
    def qqlq(self) -> I32Vector4: ...
    @property
    def qqlo(self) -> I32Vector4: ...
    @property
    def qqll(self) -> I32Vector4: ...
    @property
    def qoss(self) -> I32Vector4: ...
    @property
    def qost(self) -> I32Vector4: ...
    @property
    def qosq(self) -> I32Vector4: ...
    @property
    def qoso(self) -> I32Vector4: ...
    @property
    def qosl(self) -> I32Vector4: ...
    @property
    def qots(self) -> I32Vector4: ...
    @property
    def qott(self) -> I32Vector4: ...
    @property
    def qotq(self) -> I32Vector4: ...
    @property
    def qoto(self) -> I32Vector4: ...
    @property
    def qotl(self) -> I32Vector4: ...
    @property
    def qoqs(self) -> I32Vector4: ...
    @property
    def qoqt(self) -> I32Vector4: ...
    @property
    def qoqq(self) -> I32Vector4: ...
    @property
    def qoqo(self) -> I32Vector4: ...
    @property
    def qoql(self) -> I32Vector4: ...
    @property
    def qoos(self) -> I32Vector4: ...
    @property
    def qoot(self) -> I32Vector4: ...
    @property
    def qooq(self) -> I32Vector4: ...
    @property
    def qooo(self) -> I32Vector4: ...
    @property
    def qool(self) -> I32Vector4: ...
    @property
    def qols(self) -> I32Vector4: ...
    @property
    def qolt(self) -> I32Vector4: ...
    @property
    def qolq(self) -> I32Vector4: ...
    @property
    def qolo(self) -> I32Vector4: ...
    @property
    def qoll(self) -> I32Vector4: ...
    @property
    def qlss(self) -> I32Vector4: ...
    @property
    def qlst(self) -> I32Vector4: ...
    @property
    def qlsq(self) -> I32Vector4: ...
    @property
    def qlso(self) -> I32Vector4: ...
    @property
    def qlsl(self) -> I32Vector4: ...
    @property
    def qlts(self) -> I32Vector4: ...
    @property
    def qltt(self) -> I32Vector4: ...
    @property
    def qltq(self) -> I32Vector4: ...
    @property
    def qlto(self) -> I32Vector4: ...
    @property
    def qltl(self) -> I32Vector4: ...
    @property
    def qlqs(self) -> I32Vector4: ...
    @property
    def qlqt(self) -> I32Vector4: ...
    @property
    def qlqq(self) -> I32Vector4: ...
    @property
    def qlqo(self) -> I32Vector4: ...
    @property
    def qlql(self) -> I32Vector4: ...
    @property
    def qlos(self) -> I32Vector4: ...
    @property
    def qlot(self) -> I32Vector4: ...
    @property
    def qloq(self) -> I32Vector4: ...
    @property
    def qloo(self) -> I32Vector4: ...
    @property
    def qlol(self) -> I32Vector4: ...
    @property
    def qlls(self) -> I32Vector4: ...
    @property
    def qllt(self) -> I32Vector4: ...
    @property
    def qllq(self) -> I32Vector4: ...
    @property
    def qllo(self) -> I32Vector4: ...
    @property
    def qlll(self) -> I32Vector4: ...
    @property
    def osss(self) -> I32Vector4: ...
    @property
    def osst(self) -> I32Vector4: ...
    @property
    def ossq(self) -> I32Vector4: ...
    @property
    def osso(self) -> I32Vector4: ...
    @property
    def ossl(self) -> I32Vector4: ...
    @property
    def osts(self) -> I32Vector4: ...
    @property
    def ostt(self) -> I32Vector4: ...
    @property
    def ostq(self) -> I32Vector4: ...
    @property
    def osto(self) -> I32Vector4: ...
    @property
    def ostl(self) -> I32Vector4: ...
    @property
    def osqs(self) -> I32Vector4: ...
    @property
    def osqt(self) -> I32Vector4: ...
    @property
    def osqq(self) -> I32Vector4: ...
    @property
    def osqo(self) -> I32Vector4: ...
    @property
    def osql(self) -> I32Vector4: ...
    @property
    def osos(self) -> I32Vector4: ...
    @property
    def osot(self) -> I32Vector4: ...
    @property
    def osoq(self) -> I32Vector4: ...
    @property
    def osoo(self) -> I32Vector4: ...
    @property
    def osol(self) -> I32Vector4: ...
    @property
    def osls(self) -> I32Vector4: ...
    @property
    def oslt(self) -> I32Vector4: ...
    @property
    def oslq(self) -> I32Vector4: ...
    @property
    def oslo(self) -> I32Vector4: ...
    @property
    def osll(self) -> I32Vector4: ...
    @property
    def otss(self) -> I32Vector4: ...
    @property
    def otst(self) -> I32Vector4: ...
    @property
    def otsq(self) -> I32Vector4: ...
    @property
    def otso(self) -> I32Vector4: ...
    @property
    def otsl(self) -> I32Vector4: ...
    @property
    def otts(self) -> I32Vector4: ...
    @property
    def ottt(self) -> I32Vector4: ...
    @property
    def ottq(self) -> I32Vector4: ...
    @property
    def otto(self) -> I32Vector4: ...
    @property
    def ottl(self) -> I32Vector4: ...
    @property
    def otqs(self) -> I32Vector4: ...
    @property
    def otqt(self) -> I32Vector4: ...
    @property
    def otqq(self) -> I32Vector4: ...
    @property
    def otqo(self) -> I32Vector4: ...
    @property
    def otql(self) -> I32Vector4: ...
    @property
    def otos(self) -> I32Vector4: ...
    @property
    def otot(self) -> I32Vector4: ...
    @property
    def otoq(self) -> I32Vector4: ...
    @property
    def otoo(self) -> I32Vector4: ...
    @property
    def otol(self) -> I32Vector4: ...
    @property
    def otls(self) -> I32Vector4: ...
    @property
    def otlt(self) -> I32Vector4: ...
    @property
    def otlq(self) -> I32Vector4: ...
    @property
    def otlo(self) -> I32Vector4: ...
    @property
    def otll(self) -> I32Vector4: ...
    @property
    def oqss(self) -> I32Vector4: ...
    @property
    def oqst(self) -> I32Vector4: ...
    @property
    def oqsq(self) -> I32Vector4: ...
    @property
    def oqso(self) -> I32Vector4: ...
    @property
    def oqsl(self) -> I32Vector4: ...
    @property
    def oqts(self) -> I32Vector4: ...
    @property
    def oqtt(self) -> I32Vector4: ...
    @property
    def oqtq(self) -> I32Vector4: ...
    @property
    def oqto(self) -> I32Vector4: ...
    @property
    def oqtl(self) -> I32Vector4: ...
    @property
    def oqqs(self) -> I32Vector4: ...
    @property
    def oqqt(self) -> I32Vector4: ...
    @property
    def oqqq(self) -> I32Vector4: ...
    @property
    def oqqo(self) -> I32Vector4: ...
    @property
    def oqql(self) -> I32Vector4: ...
    @property
    def oqos(self) -> I32Vector4: ...
    @property
    def oqot(self) -> I32Vector4: ...
    @property
    def oqoq(self) -> I32Vector4: ...
    @property
    def oqoo(self) -> I32Vector4: ...
    @property
    def oqol(self) -> I32Vector4: ...
    @property
    def oqls(self) -> I32Vector4: ...
    @property
    def oqlt(self) -> I32Vector4: ...
    @property
    def oqlq(self) -> I32Vector4: ...
    @property
    def oqlo(self) -> I32Vector4: ...
    @property
    def oqll(self) -> I32Vector4: ...
    @property
    def ooss(self) -> I32Vector4: ...
    @property
    def oost(self) -> I32Vector4: ...
    @property
    def oosq(self) -> I32Vector4: ...
    @property
    def ooso(self) -> I32Vector4: ...
    @property
    def oosl(self) -> I32Vector4: ...
    @property
    def oots(self) -> I32Vector4: ...
    @property
    def oott(self) -> I32Vector4: ...
    @property
    def ootq(self) -> I32Vector4: ...
    @property
    def ooto(self) -> I32Vector4: ...
    @property
    def ootl(self) -> I32Vector4: ...
    @property
    def ooqs(self) -> I32Vector4: ...
    @property
    def ooqt(self) -> I32Vector4: ...
    @property
    def ooqq(self) -> I32Vector4: ...
    @property
    def ooqo(self) -> I32Vector4: ...
    @property
    def ooql(self) -> I32Vector4: ...
    @property
    def ooos(self) -> I32Vector4: ...
    @property
    def ooot(self) -> I32Vector4: ...
    @property
    def oooq(self) -> I32Vector4: ...
    @property
    def ools(self) -> I32Vector4: ...
    @property
    def oolt(self) -> I32Vector4: ...
    @property
    def oolq(self) -> I32Vector4: ...
    @property
    def olss(self) -> I32Vector4: ...
    @property
    def olst(self) -> I32Vector4: ...
    @property
    def olsq(self) -> I32Vector4: ...
    @property
    def olso(self) -> I32Vector4: ...
    @property
    def olsl(self) -> I32Vector4: ...
    @property
    def olts(self) -> I32Vector4: ...
    @property
    def oltt(self) -> I32Vector4: ...
    @property
    def oltq(self) -> I32Vector4: ...
    @property
    def olto(self) -> I32Vector4: ...
    @property
    def oltl(self) -> I32Vector4: ...
    @property
    def olqs(self) -> I32Vector4: ...
    @property
    def olqt(self) -> I32Vector4: ...
    @property
    def olqq(self) -> I32Vector4: ...
    @property
    def olqo(self) -> I32Vector4: ...
    @property
    def olql(self) -> I32Vector4: ...
    @property
    def olos(self) -> I32Vector4: ...
    @property
    def olot(self) -> I32Vector4: ...
    @property
    def oloq(self) -> I32Vector4: ...
    @property
    def olls(self) -> I32Vector4: ...
    @property
    def ollt(self) -> I32Vector4: ...
    @property
    def ollq(self) -> I32Vector4: ...
    @property
    def lsss(self) -> I32Vector4: ...
    @property
    def lsst(self) -> I32Vector4: ...
    @property
    def lssq(self) -> I32Vector4: ...
    @property
    def lsso(self) -> I32Vector4: ...
    @property
    def lssl(self) -> I32Vector4: ...
    @property
    def lsts(self) -> I32Vector4: ...
    @property
    def lstt(self) -> I32Vector4: ...
    @property
    def lstq(self) -> I32Vector4: ...
    @property
    def lsto(self) -> I32Vector4: ...
    @property
    def lstl(self) -> I32Vector4: ...
    @property
    def lsqs(self) -> I32Vector4: ...
    @property
    def lsqt(self) -> I32Vector4: ...
    @property
    def lsqq(self) -> I32Vector4: ...
    @property
    def lsqo(self) -> I32Vector4: ...
    @property
    def lsql(self) -> I32Vector4: ...
    @property
    def lsos(self) -> I32Vector4: ...
    @property
    def lsot(self) -> I32Vector4: ...
    @property
    def lsoq(self) -> I32Vector4: ...
    @property
    def lsoo(self) -> I32Vector4: ...
    @property
    def lsol(self) -> I32Vector4: ...
    @property
    def lsls(self) -> I32Vector4: ...
    @property
    def lslt(self) -> I32Vector4: ...
    @property
    def lslq(self) -> I32Vector4: ...
    @property
    def lslo(self) -> I32Vector4: ...
    @property
    def lsll(self) -> I32Vector4: ...
    @property
    def ltss(self) -> I32Vector4: ...
    @property
    def ltst(self) -> I32Vector4: ...
    @property
    def ltsq(self) -> I32Vector4: ...
    @property
    def ltso(self) -> I32Vector4: ...
    @property
    def ltsl(self) -> I32Vector4: ...
    @property
    def ltts(self) -> I32Vector4: ...
    @property
    def lttt(self) -> I32Vector4: ...
    @property
    def lttq(self) -> I32Vector4: ...
    @property
    def ltto(self) -> I32Vector4: ...
    @property
    def lttl(self) -> I32Vector4: ...
    @property
    def ltqs(self) -> I32Vector4: ...
    @property
    def ltqt(self) -> I32Vector4: ...
    @property
    def ltqq(self) -> I32Vector4: ...
    @property
    def ltqo(self) -> I32Vector4: ...
    @property
    def ltql(self) -> I32Vector4: ...
    @property
    def ltos(self) -> I32Vector4: ...
    @property
    def ltot(self) -> I32Vector4: ...
    @property
    def ltoq(self) -> I32Vector4: ...
    @property
    def ltoo(self) -> I32Vector4: ...
    @property
    def ltol(self) -> I32Vector4: ...
    @property
    def ltls(self) -> I32Vector4: ...
    @property
    def ltlt(self) -> I32Vector4: ...
    @property
    def ltlq(self) -> I32Vector4: ...
    @property
    def ltlo(self) -> I32Vector4: ...
    @property
    def ltll(self) -> I32Vector4: ...
    @property
    def lqss(self) -> I32Vector4: ...
    @property
    def lqst(self) -> I32Vector4: ...
    @property
    def lqsq(self) -> I32Vector4: ...
    @property
    def lqso(self) -> I32Vector4: ...
    @property
    def lqsl(self) -> I32Vector4: ...
    @property
    def lqts(self) -> I32Vector4: ...
    @property
    def lqtt(self) -> I32Vector4: ...
    @property
    def lqtq(self) -> I32Vector4: ...
    @property
    def lqto(self) -> I32Vector4: ...
    @property
    def lqtl(self) -> I32Vector4: ...
    @property
    def lqqs(self) -> I32Vector4: ...
    @property
    def lqqt(self) -> I32Vector4: ...
    @property
    def lqqq(self) -> I32Vector4: ...
    @property
    def lqqo(self) -> I32Vector4: ...
    @property
    def lqql(self) -> I32Vector4: ...
    @property
    def lqos(self) -> I32Vector4: ...
    @property
    def lqot(self) -> I32Vector4: ...
    @property
    def lqoq(self) -> I32Vector4: ...
    @property
    def lqoo(self) -> I32Vector4: ...
    @property
    def lqol(self) -> I32Vector4: ...
    @property
    def lqls(self) -> I32Vector4: ...
    @property
    def lqlt(self) -> I32Vector4: ...
    @property
    def lqlq(self) -> I32Vector4: ...
    @property
    def lqlo(self) -> I32Vector4: ...
    @property
    def lqll(self) -> I32Vector4: ...
    @property
    def loss(self) -> I32Vector4: ...
    @property
    def lost(self) -> I32Vector4: ...
    @property
    def losq(self) -> I32Vector4: ...
    @property
    def loso(self) -> I32Vector4: ...
    @property
    def losl(self) -> I32Vector4: ...
    @property
    def lots(self) -> I32Vector4: ...
    @property
    def lott(self) -> I32Vector4: ...
    @property
    def lotq(self) -> I32Vector4: ...
    @property
    def loto(self) -> I32Vector4: ...
    @property
    def lotl(self) -> I32Vector4: ...
    @property
    def loqs(self) -> I32Vector4: ...
    @property
    def loqt(self) -> I32Vector4: ...
    @property
    def loqq(self) -> I32Vector4: ...
    @property
    def loqo(self) -> I32Vector4: ...
    @property
    def loql(self) -> I32Vector4: ...
    @property
    def loos(self) -> I32Vector4: ...
    @property
    def loot(self) -> I32Vector4: ...
    @property
    def looq(self) -> I32Vector4: ...
    @property
    def lols(self) -> I32Vector4: ...
    @property
    def lolt(self) -> I32Vector4: ...
    @property
    def lolq(self) -> I32Vector4: ...
    @property
    def llss(self) -> I32Vector4: ...
    @property
    def llst(self) -> I32Vector4: ...
    @property
    def llsq(self) -> I32Vector4: ...
    @property
    def llso(self) -> I32Vector4: ...
    @property
    def llsl(self) -> I32Vector4: ...
    @property
    def llts(self) -> I32Vector4: ...
    @property
    def lltt(self) -> I32Vector4: ...
    @property
    def lltq(self) -> I32Vector4: ...
    @property
    def llto(self) -> I32Vector4: ...
    @property
    def lltl(self) -> I32Vector4: ...
    @property
    def llqs(self) -> I32Vector4: ...
    @property
    def llqt(self) -> I32Vector4: ...
    @property
    def llqq(self) -> I32Vector4: ...
    @property
    def llqo(self) -> I32Vector4: ...
    @property
    def llql(self) -> I32Vector4: ...
    @property
    def llos(self) -> I32Vector4: ...
    @property
    def llot(self) -> I32Vector4: ...
    @property
    def lloq(self) -> I32Vector4: ...
    @property
    def llls(self) -> I32Vector4: ...
    @property
    def lllt(self) -> I32Vector4: ...
    @property
    def lllq(self) -> I32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I32Vector2: ...
    @property
    def uv(self) -> I32Vector2: ...
    @property
    def uo(self) -> I32Vector2: ...
    @property
    def ul(self) -> I32Vector2: ...
    @property
    def vu(self) -> I32Vector2: ...
    @property
    def vv(self) -> I32Vector2: ...
    @property
    def vo(self) -> I32Vector2: ...
    @property
    def vl(self) -> I32Vector2: ...
    @property
    def ou(self) -> I32Vector2: ...
    @property
    def ov(self) -> I32Vector2: ...
    @property
    def lu(self) -> I32Vector2: ...
    @property
    def lv(self) -> I32Vector2: ...
    @property
    def uuu(self) -> I32Vector3: ...
    @property
    def uuv(self) -> I32Vector3: ...
    @property
    def uuo(self) -> I32Vector3: ...
    @property
    def uul(self) -> I32Vector3: ...
    @property
    def uvu(self) -> I32Vector3: ...
    @property
    def uvv(self) -> I32Vector3: ...
    @property
    def uvo(self) -> I32Vector3: ...
    @property
    def uvl(self) -> I32Vector3: ...
    @property
    def uou(self) -> I32Vector3: ...
    @property
    def uov(self) -> I32Vector3: ...
    @property
    def uoo(self) -> I32Vector3: ...
    @property
    def uol(self) -> I32Vector3: ...
    @property
    def ulu(self) -> I32Vector3: ...
    @property
    def ulv(self) -> I32Vector3: ...
    @property
    def ulo(self) -> I32Vector3: ...
    @property
    def ull(self) -> I32Vector3: ...
    @property
    def vuu(self) -> I32Vector3: ...
    @property
    def vuv(self) -> I32Vector3: ...
    @property
    def vuo(self) -> I32Vector3: ...
    @property
    def vul(self) -> I32Vector3: ...
    @property
    def vvu(self) -> I32Vector3: ...
    @property
    def vvv(self) -> I32Vector3: ...
    @property
    def vvo(self) -> I32Vector3: ...
    @property
    def vvl(self) -> I32Vector3: ...
    @property
    def vou(self) -> I32Vector3: ...
    @property
    def vov(self) -> I32Vector3: ...
    @property
    def voo(self) -> I32Vector3: ...
    @property
    def vol(self) -> I32Vector3: ...
    @property
    def vlu(self) -> I32Vector3: ...
    @property
    def vlv(self) -> I32Vector3: ...
    @property
    def vlo(self) -> I32Vector3: ...
    @property
    def vll(self) -> I32Vector3: ...
    @property
    def ouu(self) -> I32Vector3: ...
    @property
    def ouv(self) -> I32Vector3: ...
    @property
    def ouo(self) -> I32Vector3: ...
    @property
    def oul(self) -> I32Vector3: ...
    @property
    def ovu(self) -> I32Vector3: ...
    @property
    def ovv(self) -> I32Vector3: ...
    @property
    def ovo(self) -> I32Vector3: ...
    @property
    def ovl(self) -> I32Vector3: ...
    @property
    def oou(self) -> I32Vector3: ...
    @property
    def oov(self) -> I32Vector3: ...
    @property
    def olu(self) -> I32Vector3: ...
    @property
    def olv(self) -> I32Vector3: ...
    @property
    def luu(self) -> I32Vector3: ...
    @property
    def luv(self) -> I32Vector3: ...
    @property
    def luo(self) -> I32Vector3: ...
    @property
    def lul(self) -> I32Vector3: ...
    @property
    def lvu(self) -> I32Vector3: ...
    @property
    def lvv(self) -> I32Vector3: ...
    @property
    def lvo(self) -> I32Vector3: ...
    @property
    def lvl(self) -> I32Vector3: ...
    @property
    def lou(self) -> I32Vector3: ...
    @property
    def lov(self) -> I32Vector3: ...
    @property
    def llu(self) -> I32Vector3: ...
    @property
    def llv(self) -> I32Vector3: ...
    @property
    def uuuu(self) -> I32Vector4: ...
    @property
    def uuuv(self) -> I32Vector4: ...
    @property
    def uuuo(self) -> I32Vector4: ...
    @property
    def uuul(self) -> I32Vector4: ...
    @property
    def uuvu(self) -> I32Vector4: ...
    @property
    def uuvv(self) -> I32Vector4: ...
    @property
    def uuvo(self) -> I32Vector4: ...
    @property
    def uuvl(self) -> I32Vector4: ...
    @property
    def uuou(self) -> I32Vector4: ...
    @property
    def uuov(self) -> I32Vector4: ...
    @property
    def uuoo(self) -> I32Vector4: ...
    @property
    def uuol(self) -> I32Vector4: ...
    @property
    def uulu(self) -> I32Vector4: ...
    @property
    def uulv(self) -> I32Vector4: ...
    @property
    def uulo(self) -> I32Vector4: ...
    @property
    def uull(self) -> I32Vector4: ...
    @property
    def uvuu(self) -> I32Vector4: ...
    @property
    def uvuv(self) -> I32Vector4: ...
    @property
    def uvuo(self) -> I32Vector4: ...
    @property
    def uvul(self) -> I32Vector4: ...
    @property
    def uvvu(self) -> I32Vector4: ...
    @property
    def uvvv(self) -> I32Vector4: ...
    @property
    def uvvo(self) -> I32Vector4: ...
    @property
    def uvvl(self) -> I32Vector4: ...
    @property
    def uvou(self) -> I32Vector4: ...
    @property
    def uvov(self) -> I32Vector4: ...
    @property
    def uvoo(self) -> I32Vector4: ...
    @property
    def uvol(self) -> I32Vector4: ...
    @property
    def uvlu(self) -> I32Vector4: ...
    @property
    def uvlv(self) -> I32Vector4: ...
    @property
    def uvlo(self) -> I32Vector4: ...
    @property
    def uvll(self) -> I32Vector4: ...
    @property
    def uouu(self) -> I32Vector4: ...
    @property
    def uouv(self) -> I32Vector4: ...
    @property
    def uouo(self) -> I32Vector4: ...
    @property
    def uoul(self) -> I32Vector4: ...
    @property
    def uovu(self) -> I32Vector4: ...
    @property
    def uovv(self) -> I32Vector4: ...
    @property
    def uovo(self) -> I32Vector4: ...
    @property
    def uovl(self) -> I32Vector4: ...
    @property
    def uoou(self) -> I32Vector4: ...
    @property
    def uoov(self) -> I32Vector4: ...
    @property
    def uooo(self) -> I32Vector4: ...
    @property
    def uool(self) -> I32Vector4: ...
    @property
    def uolu(self) -> I32Vector4: ...
    @property
    def uolv(self) -> I32Vector4: ...
    @property
    def uolo(self) -> I32Vector4: ...
    @property
    def uoll(self) -> I32Vector4: ...
    @property
    def uluu(self) -> I32Vector4: ...
    @property
    def uluv(self) -> I32Vector4: ...
    @property
    def uluo(self) -> I32Vector4: ...
    @property
    def ulul(self) -> I32Vector4: ...
    @property
    def ulvu(self) -> I32Vector4: ...
    @property
    def ulvv(self) -> I32Vector4: ...
    @property
    def ulvo(self) -> I32Vector4: ...
    @property
    def ulvl(self) -> I32Vector4: ...
    @property
    def ulou(self) -> I32Vector4: ...
    @property
    def ulov(self) -> I32Vector4: ...
    @property
    def uloo(self) -> I32Vector4: ...
    @property
    def ulol(self) -> I32Vector4: ...
    @property
    def ullu(self) -> I32Vector4: ...
    @property
    def ullv(self) -> I32Vector4: ...
    @property
    def ullo(self) -> I32Vector4: ...
    @property
    def ulll(self) -> I32Vector4: ...
    @property
    def vuuu(self) -> I32Vector4: ...
    @property
    def vuuv(self) -> I32Vector4: ...
    @property
    def vuuo(self) -> I32Vector4: ...
    @property
    def vuul(self) -> I32Vector4: ...
    @property
    def vuvu(self) -> I32Vector4: ...
    @property
    def vuvv(self) -> I32Vector4: ...
    @property
    def vuvo(self) -> I32Vector4: ...
    @property
    def vuvl(self) -> I32Vector4: ...
    @property
    def vuou(self) -> I32Vector4: ...
    @property
    def vuov(self) -> I32Vector4: ...
    @property
    def vuoo(self) -> I32Vector4: ...
    @property
    def vuol(self) -> I32Vector4: ...
    @property
    def vulu(self) -> I32Vector4: ...
    @property
    def vulv(self) -> I32Vector4: ...
    @property
    def vulo(self) -> I32Vector4: ...
    @property
    def vull(self) -> I32Vector4: ...
    @property
    def vvuu(self) -> I32Vector4: ...
    @property
    def vvuv(self) -> I32Vector4: ...
    @property
    def vvuo(self) -> I32Vector4: ...
    @property
    def vvul(self) -> I32Vector4: ...
    @property
    def vvvu(self) -> I32Vector4: ...
    @property
    def vvvv(self) -> I32Vector4: ...
    @property
    def vvvo(self) -> I32Vector4: ...
    @property
    def vvvl(self) -> I32Vector4: ...
    @property
    def vvou(self) -> I32Vector4: ...
    @property
    def vvov(self) -> I32Vector4: ...
    @property
    def vvoo(self) -> I32Vector4: ...
    @property
    def vvol(self) -> I32Vector4: ...
    @property
    def vvlu(self) -> I32Vector4: ...
    @property
    def vvlv(self) -> I32Vector4: ...
    @property
    def vvlo(self) -> I32Vector4: ...
    @property
    def vvll(self) -> I32Vector4: ...
    @property
    def vouu(self) -> I32Vector4: ...
    @property
    def vouv(self) -> I32Vector4: ...
    @property
    def vouo(self) -> I32Vector4: ...
    @property
    def voul(self) -> I32Vector4: ...
    @property
    def vovu(self) -> I32Vector4: ...
    @property
    def vovv(self) -> I32Vector4: ...
    @property
    def vovo(self) -> I32Vector4: ...
    @property
    def vovl(self) -> I32Vector4: ...
    @property
    def voou(self) -> I32Vector4: ...
    @property
    def voov(self) -> I32Vector4: ...
    @property
    def vooo(self) -> I32Vector4: ...
    @property
    def vool(self) -> I32Vector4: ...
    @property
    def volu(self) -> I32Vector4: ...
    @property
    def volv(self) -> I32Vector4: ...
    @property
    def volo(self) -> I32Vector4: ...
    @property
    def voll(self) -> I32Vector4: ...
    @property
    def vluu(self) -> I32Vector4: ...
    @property
    def vluv(self) -> I32Vector4: ...
    @property
    def vluo(self) -> I32Vector4: ...
    @property
    def vlul(self) -> I32Vector4: ...
    @property
    def vlvu(self) -> I32Vector4: ...
    @property
    def vlvv(self) -> I32Vector4: ...
    @property
    def vlvo(self) -> I32Vector4: ...
    @property
    def vlvl(self) -> I32Vector4: ...
    @property
    def vlou(self) -> I32Vector4: ...
    @property
    def vlov(self) -> I32Vector4: ...
    @property
    def vloo(self) -> I32Vector4: ...
    @property
    def vlol(self) -> I32Vector4: ...
    @property
    def vllu(self) -> I32Vector4: ...
    @property
    def vllv(self) -> I32Vector4: ...
    @property
    def vllo(self) -> I32Vector4: ...
    @property
    def vlll(self) -> I32Vector4: ...
    @property
    def ouuu(self) -> I32Vector4: ...
    @property
    def ouuv(self) -> I32Vector4: ...
    @property
    def ouuo(self) -> I32Vector4: ...
    @property
    def ouul(self) -> I32Vector4: ...
    @property
    def ouvu(self) -> I32Vector4: ...
    @property
    def ouvv(self) -> I32Vector4: ...
    @property
    def ouvo(self) -> I32Vector4: ...
    @property
    def ouvl(self) -> I32Vector4: ...
    @property
    def ouou(self) -> I32Vector4: ...
    @property
    def ouov(self) -> I32Vector4: ...
    @property
    def ouoo(self) -> I32Vector4: ...
    @property
    def ouol(self) -> I32Vector4: ...
    @property
    def oulu(self) -> I32Vector4: ...
    @property
    def oulv(self) -> I32Vector4: ...
    @property
    def oulo(self) -> I32Vector4: ...
    @property
    def oull(self) -> I32Vector4: ...
    @property
    def ovuu(self) -> I32Vector4: ...
    @property
    def ovuv(self) -> I32Vector4: ...
    @property
    def ovuo(self) -> I32Vector4: ...
    @property
    def ovul(self) -> I32Vector4: ...
    @property
    def ovvu(self) -> I32Vector4: ...
    @property
    def ovvv(self) -> I32Vector4: ...
    @property
    def ovvo(self) -> I32Vector4: ...
    @property
    def ovvl(self) -> I32Vector4: ...
    @property
    def ovou(self) -> I32Vector4: ...
    @property
    def ovov(self) -> I32Vector4: ...
    @property
    def ovoo(self) -> I32Vector4: ...
    @property
    def ovol(self) -> I32Vector4: ...
    @property
    def ovlu(self) -> I32Vector4: ...
    @property
    def ovlv(self) -> I32Vector4: ...
    @property
    def ovlo(self) -> I32Vector4: ...
    @property
    def ovll(self) -> I32Vector4: ...
    @property
    def oouu(self) -> I32Vector4: ...
    @property
    def oouv(self) -> I32Vector4: ...
    @property
    def oouo(self) -> I32Vector4: ...
    @property
    def ooul(self) -> I32Vector4: ...
    @property
    def oovu(self) -> I32Vector4: ...
    @property
    def oovv(self) -> I32Vector4: ...
    @property
    def oovo(self) -> I32Vector4: ...
    @property
    def oovl(self) -> I32Vector4: ...
    @property
    def ooou(self) -> I32Vector4: ...
    @property
    def ooov(self) -> I32Vector4: ...
    @property
    def oolu(self) -> I32Vector4: ...
    @property
    def oolv(self) -> I32Vector4: ...
    @property
    def oluu(self) -> I32Vector4: ...
    @property
    def oluv(self) -> I32Vector4: ...
    @property
    def oluo(self) -> I32Vector4: ...
    @property
    def olul(self) -> I32Vector4: ...
    @property
    def olvu(self) -> I32Vector4: ...
    @property
    def olvv(self) -> I32Vector4: ...
    @property
    def olvo(self) -> I32Vector4: ...
    @property
    def olvl(self) -> I32Vector4: ...
    @property
    def olou(self) -> I32Vector4: ...
    @property
    def olov(self) -> I32Vector4: ...
    @property
    def ollu(self) -> I32Vector4: ...
    @property
    def ollv(self) -> I32Vector4: ...
    @property
    def luuu(self) -> I32Vector4: ...
    @property
    def luuv(self) -> I32Vector4: ...
    @property
    def luuo(self) -> I32Vector4: ...
    @property
    def luul(self) -> I32Vector4: ...
    @property
    def luvu(self) -> I32Vector4: ...
    @property
    def luvv(self) -> I32Vector4: ...
    @property
    def luvo(self) -> I32Vector4: ...
    @property
    def luvl(self) -> I32Vector4: ...
    @property
    def luou(self) -> I32Vector4: ...
    @property
    def luov(self) -> I32Vector4: ...
    @property
    def luoo(self) -> I32Vector4: ...
    @property
    def luol(self) -> I32Vector4: ...
    @property
    def lulu(self) -> I32Vector4: ...
    @property
    def lulv(self) -> I32Vector4: ...
    @property
    def lulo(self) -> I32Vector4: ...
    @property
    def lull(self) -> I32Vector4: ...
    @property
    def lvuu(self) -> I32Vector4: ...
    @property
    def lvuv(self) -> I32Vector4: ...
    @property
    def lvuo(self) -> I32Vector4: ...
    @property
    def lvul(self) -> I32Vector4: ...
    @property
    def lvvu(self) -> I32Vector4: ...
    @property
    def lvvv(self) -> I32Vector4: ...
    @property
    def lvvo(self) -> I32Vector4: ...
    @property
    def lvvl(self) -> I32Vector4: ...
    @property
    def lvou(self) -> I32Vector4: ...
    @property
    def lvov(self) -> I32Vector4: ...
    @property
    def lvoo(self) -> I32Vector4: ...
    @property
    def lvol(self) -> I32Vector4: ...
    @property
    def lvlu(self) -> I32Vector4: ...
    @property
    def lvlv(self) -> I32Vector4: ...
    @property
    def lvlo(self) -> I32Vector4: ...
    @property
    def lvll(self) -> I32Vector4: ...
    @property
    def louu(self) -> I32Vector4: ...
    @property
    def louv(self) -> I32Vector4: ...
    @property
    def louo(self) -> I32Vector4: ...
    @property
    def loul(self) -> I32Vector4: ...
    @property
    def lovu(self) -> I32Vector4: ...
    @property
    def lovv(self) -> I32Vector4: ...
    @property
    def lovo(self) -> I32Vector4: ...
    @property
    def lovl(self) -> I32Vector4: ...
    @property
    def loou(self) -> I32Vector4: ...
    @property
    def loov(self) -> I32Vector4: ...
    @property
    def lolu(self) -> I32Vector4: ...
    @property
    def lolv(self) -> I32Vector4: ...
    @property
    def lluu(self) -> I32Vector4: ...
    @property
    def lluv(self) -> I32Vector4: ...
    @property
    def lluo(self) -> I32Vector4: ...
    @property
    def llul(self) -> I32Vector4: ...
    @property
    def llvu(self) -> I32Vector4: ...
    @property
    def llvv(self) -> I32Vector4: ...
    @property
    def llvo(self) -> I32Vector4: ...
    @property
    def llvl(self) -> I32Vector4: ...
    @property
    def llou(self) -> I32Vector4: ...
    @property
    def llov(self) -> I32Vector4: ...
    @property
    def lllu(self) -> I32Vector4: ...
    @property
    def lllv(self) -> I32Vector4: ...
    @property
    def oo(self) -> I32Vector2: ...
    @property
    def ol(self) -> I32Vector2: ...
    @property
    def lo(self) -> I32Vector2: ...
    @property
    def ll(self) -> I32Vector2: ...
    @property
    def ooo(self) -> I32Vector3: ...
    @property
    def ool(self) -> I32Vector3: ...
    @property
    def olo(self) -> I32Vector3: ...
    @property
    def oll(self) -> I32Vector3: ...
    @property
    def loo(self) -> I32Vector3: ...
    @property
    def lol(self) -> I32Vector3: ...
    @property
    def llo(self) -> I32Vector3: ...
    @property
    def lll(self) -> I32Vector3: ...
    @property
    def oooo(self) -> I32Vector4: ...
    @property
    def oool(self) -> I32Vector4: ...
    @property
    def oolo(self) -> I32Vector4: ...
    @property
    def ooll(self) -> I32Vector4: ...
    @property
    def oloo(self) -> I32Vector4: ...
    @property
    def olol(self) -> I32Vector4: ...
    @property
    def ollo(self) -> I32Vector4: ...
    @property
    def olll(self) -> I32Vector4: ...
    @property
    def looo(self) -> I32Vector4: ...
    @property
    def lool(self) -> I32Vector4: ...
    @property
    def lolo(self) -> I32Vector4: ...
    @property
    def loll(self) -> I32Vector4: ...
    @property
    def lloo(self) -> I32Vector4: ...
    @property
    def llol(self) -> I32Vector4: ...
    @property
    def lllo(self) -> I32Vector4: ...
    @property
    def llll(self) -> I32Vector4: ...
    def min(self, n: Number, /) -> I32Vector3: ...
    def max(self, n: Number, /) -> I32Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> I32Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I32Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector3: ...

@final
class I32Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I32Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I32Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> I32Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> I32Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[I32Vector3]: ...

@final
class U32Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __add__(self, other: Number) -> U32Vector3: ...
    @overload
    def __radd__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __radd__(self, other: Number) -> U32Vector3: ...
    @overload
    def __sub__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __sub__(self, other: Number) -> U32Vector3: ...
    @overload
    def __rsub__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> U32Vector3: ...
    @overload
    def __mul__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __mul__(self, other: Number) -> U32Vector3: ...
    @overload
    def __rmul__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> U32Vector3: ...
    @overload
    def __truediv__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> U32Vector3: ...
    @overload
    def __rtruediv__(self, other: U32Vector3) -> U32Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> U32Vector3: ...
    def __abs__(self) -> U32Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> U32Vector2: ...
    @property
    def xy(self) -> U32Vector2: ...
    @property
    def xz(self) -> U32Vector2: ...
    @property
    def xo(self) -> U32Vector2: ...
    @property
    def xl(self) -> U32Vector2: ...
    @property
    def yx(self) -> U32Vector2: ...
    @property
    def yy(self) -> U32Vector2: ...
    @property
    def yz(self) -> U32Vector2: ...
    @property
    def yo(self) -> U32Vector2: ...
    @property
    def yl(self) -> U32Vector2: ...
    @property
    def zx(self) -> U32Vector2: ...
    @property
    def zy(self) -> U32Vector2: ...
    @property
    def zz(self) -> U32Vector2: ...
    @property
    def zo(self) -> U32Vector2: ...
    @property
    def zl(self) -> U32Vector2: ...
    @property
    def ox(self) -> U32Vector2: ...
    @property
    def oy(self) -> U32Vector2: ...
    @property
    def oz(self) -> U32Vector2: ...
    @property
    def lx(self) -> U32Vector2: ...
    @property
    def ly(self) -> U32Vector2: ...
    @property
    def lz(self) -> U32Vector2: ...
    @property
    def xxx(self) -> U32Vector3: ...
    @property
    def xxy(self) -> U32Vector3: ...
    @property
    def xxz(self) -> U32Vector3: ...
    @property
    def xxo(self) -> U32Vector3: ...
    @property
    def xxl(self) -> U32Vector3: ...
    @property
    def xyx(self) -> U32Vector3: ...
    @property
    def xyy(self) -> U32Vector3: ...
    @property
    def xyz(self) -> U32Vector3: ...
    @property
    def xyo(self) -> U32Vector3: ...
    @property
    def xyl(self) -> U32Vector3: ...
    @property
    def xzx(self) -> U32Vector3: ...
    @property
    def xzy(self) -> U32Vector3: ...
    @property
    def xzz(self) -> U32Vector3: ...
    @property
    def xzo(self) -> U32Vector3: ...
    @property
    def xzl(self) -> U32Vector3: ...
    @property
    def xox(self) -> U32Vector3: ...
    @property
    def xoy(self) -> U32Vector3: ...
    @property
    def xoz(self) -> U32Vector3: ...
    @property
    def xoo(self) -> U32Vector3: ...
    @property
    def xol(self) -> U32Vector3: ...
    @property
    def xlx(self) -> U32Vector3: ...
    @property
    def xly(self) -> U32Vector3: ...
    @property
    def xlz(self) -> U32Vector3: ...
    @property
    def xlo(self) -> U32Vector3: ...
    @property
    def xll(self) -> U32Vector3: ...
    @property
    def yxx(self) -> U32Vector3: ...
    @property
    def yxy(self) -> U32Vector3: ...
    @property
    def yxz(self) -> U32Vector3: ...
    @property
    def yxo(self) -> U32Vector3: ...
    @property
    def yxl(self) -> U32Vector3: ...
    @property
    def yyx(self) -> U32Vector3: ...
    @property
    def yyy(self) -> U32Vector3: ...
    @property
    def yyz(self) -> U32Vector3: ...
    @property
    def yyo(self) -> U32Vector3: ...
    @property
    def yyl(self) -> U32Vector3: ...
    @property
    def yzx(self) -> U32Vector3: ...
    @property
    def yzy(self) -> U32Vector3: ...
    @property
    def yzz(self) -> U32Vector3: ...
    @property
    def yzo(self) -> U32Vector3: ...
    @property
    def yzl(self) -> U32Vector3: ...
    @property
    def yox(self) -> U32Vector3: ...
    @property
    def yoy(self) -> U32Vector3: ...
    @property
    def yoz(self) -> U32Vector3: ...
    @property
    def yoo(self) -> U32Vector3: ...
    @property
    def yol(self) -> U32Vector3: ...
    @property
    def ylx(self) -> U32Vector3: ...
    @property
    def yly(self) -> U32Vector3: ...
    @property
    def ylz(self) -> U32Vector3: ...
    @property
    def ylo(self) -> U32Vector3: ...
    @property
    def yll(self) -> U32Vector3: ...
    @property
    def zxx(self) -> U32Vector3: ...
    @property
    def zxy(self) -> U32Vector3: ...
    @property
    def zxz(self) -> U32Vector3: ...
    @property
    def zxo(self) -> U32Vector3: ...
    @property
    def zxl(self) -> U32Vector3: ...
    @property
    def zyx(self) -> U32Vector3: ...
    @property
    def zyy(self) -> U32Vector3: ...
    @property
    def zyz(self) -> U32Vector3: ...
    @property
    def zyo(self) -> U32Vector3: ...
    @property
    def zyl(self) -> U32Vector3: ...
    @property
    def zzx(self) -> U32Vector3: ...
    @property
    def zzy(self) -> U32Vector3: ...
    @property
    def zzz(self) -> U32Vector3: ...
    @property
    def zzo(self) -> U32Vector3: ...
    @property
    def zzl(self) -> U32Vector3: ...
    @property
    def zox(self) -> U32Vector3: ...
    @property
    def zoy(self) -> U32Vector3: ...
    @property
    def zoz(self) -> U32Vector3: ...
    @property
    def zoo(self) -> U32Vector3: ...
    @property
    def zol(self) -> U32Vector3: ...
    @property
    def zlx(self) -> U32Vector3: ...
    @property
    def zly(self) -> U32Vector3: ...
    @property
    def zlz(self) -> U32Vector3: ...
    @property
    def zlo(self) -> U32Vector3: ...
    @property
    def zll(self) -> U32Vector3: ...
    @property
    def oxx(self) -> U32Vector3: ...
    @property
    def oxy(self) -> U32Vector3: ...
    @property
    def oxz(self) -> U32Vector3: ...
    @property
    def oxo(self) -> U32Vector3: ...
    @property
    def oxl(self) -> U32Vector3: ...
    @property
    def oyx(self) -> U32Vector3: ...
    @property
    def oyy(self) -> U32Vector3: ...
    @property
    def oyz(self) -> U32Vector3: ...
    @property
    def oyo(self) -> U32Vector3: ...
    @property
    def oyl(self) -> U32Vector3: ...
    @property
    def ozx(self) -> U32Vector3: ...
    @property
    def ozy(self) -> U32Vector3: ...
    @property
    def ozz(self) -> U32Vector3: ...
    @property
    def ozo(self) -> U32Vector3: ...
    @property
    def ozl(self) -> U32Vector3: ...
    @property
    def oox(self) -> U32Vector3: ...
    @property
    def ooy(self) -> U32Vector3: ...
    @property
    def ooz(self) -> U32Vector3: ...
    @property
    def olx(self) -> U32Vector3: ...
    @property
    def oly(self) -> U32Vector3: ...
    @property
    def olz(self) -> U32Vector3: ...
    @property
    def lxx(self) -> U32Vector3: ...
    @property
    def lxy(self) -> U32Vector3: ...
    @property
    def lxz(self) -> U32Vector3: ...
    @property
    def lxo(self) -> U32Vector3: ...
    @property
    def lxl(self) -> U32Vector3: ...
    @property
    def lyx(self) -> U32Vector3: ...
    @property
    def lyy(self) -> U32Vector3: ...
    @property
    def lyz(self) -> U32Vector3: ...
    @property
    def lyo(self) -> U32Vector3: ...
    @property
    def lyl(self) -> U32Vector3: ...
    @property
    def lzx(self) -> U32Vector3: ...
    @property
    def lzy(self) -> U32Vector3: ...
    @property
    def lzz(self) -> U32Vector3: ...
    @property
    def lzo(self) -> U32Vector3: ...
    @property
    def lzl(self) -> U32Vector3: ...
    @property
    def lox(self) -> U32Vector3: ...
    @property
    def loy(self) -> U32Vector3: ...
    @property
    def loz(self) -> U32Vector3: ...
    @property
    def llx(self) -> U32Vector3: ...
    @property
    def lly(self) -> U32Vector3: ...
    @property
    def llz(self) -> U32Vector3: ...
    @property
    def xxxx(self) -> U32Vector4: ...
    @property
    def xxxy(self) -> U32Vector4: ...
    @property
    def xxxz(self) -> U32Vector4: ...
    @property
    def xxxo(self) -> U32Vector4: ...
    @property
    def xxxl(self) -> U32Vector4: ...
    @property
    def xxyx(self) -> U32Vector4: ...
    @property
    def xxyy(self) -> U32Vector4: ...
    @property
    def xxyz(self) -> U32Vector4: ...
    @property
    def xxyo(self) -> U32Vector4: ...
    @property
    def xxyl(self) -> U32Vector4: ...
    @property
    def xxzx(self) -> U32Vector4: ...
    @property
    def xxzy(self) -> U32Vector4: ...
    @property
    def xxzz(self) -> U32Vector4: ...
    @property
    def xxzo(self) -> U32Vector4: ...
    @property
    def xxzl(self) -> U32Vector4: ...
    @property
    def xxox(self) -> U32Vector4: ...
    @property
    def xxoy(self) -> U32Vector4: ...
    @property
    def xxoz(self) -> U32Vector4: ...
    @property
    def xxoo(self) -> U32Vector4: ...
    @property
    def xxol(self) -> U32Vector4: ...
    @property
    def xxlx(self) -> U32Vector4: ...
    @property
    def xxly(self) -> U32Vector4: ...
    @property
    def xxlz(self) -> U32Vector4: ...
    @property
    def xxlo(self) -> U32Vector4: ...
    @property
    def xxll(self) -> U32Vector4: ...
    @property
    def xyxx(self) -> U32Vector4: ...
    @property
    def xyxy(self) -> U32Vector4: ...
    @property
    def xyxz(self) -> U32Vector4: ...
    @property
    def xyxo(self) -> U32Vector4: ...
    @property
    def xyxl(self) -> U32Vector4: ...
    @property
    def xyyx(self) -> U32Vector4: ...
    @property
    def xyyy(self) -> U32Vector4: ...
    @property
    def xyyz(self) -> U32Vector4: ...
    @property
    def xyyo(self) -> U32Vector4: ...
    @property
    def xyyl(self) -> U32Vector4: ...
    @property
    def xyzx(self) -> U32Vector4: ...
    @property
    def xyzy(self) -> U32Vector4: ...
    @property
    def xyzz(self) -> U32Vector4: ...
    @property
    def xyzo(self) -> U32Vector4: ...
    @property
    def xyzl(self) -> U32Vector4: ...
    @property
    def xyox(self) -> U32Vector4: ...
    @property
    def xyoy(self) -> U32Vector4: ...
    @property
    def xyoz(self) -> U32Vector4: ...
    @property
    def xyoo(self) -> U32Vector4: ...
    @property
    def xyol(self) -> U32Vector4: ...
    @property
    def xylx(self) -> U32Vector4: ...
    @property
    def xyly(self) -> U32Vector4: ...
    @property
    def xylz(self) -> U32Vector4: ...
    @property
    def xylo(self) -> U32Vector4: ...
    @property
    def xyll(self) -> U32Vector4: ...
    @property
    def xzxx(self) -> U32Vector4: ...
    @property
    def xzxy(self) -> U32Vector4: ...
    @property
    def xzxz(self) -> U32Vector4: ...
    @property
    def xzxo(self) -> U32Vector4: ...
    @property
    def xzxl(self) -> U32Vector4: ...
    @property
    def xzyx(self) -> U32Vector4: ...
    @property
    def xzyy(self) -> U32Vector4: ...
    @property
    def xzyz(self) -> U32Vector4: ...
    @property
    def xzyo(self) -> U32Vector4: ...
    @property
    def xzyl(self) -> U32Vector4: ...
    @property
    def xzzx(self) -> U32Vector4: ...
    @property
    def xzzy(self) -> U32Vector4: ...
    @property
    def xzzz(self) -> U32Vector4: ...
    @property
    def xzzo(self) -> U32Vector4: ...
    @property
    def xzzl(self) -> U32Vector4: ...
    @property
    def xzox(self) -> U32Vector4: ...
    @property
    def xzoy(self) -> U32Vector4: ...
    @property
    def xzoz(self) -> U32Vector4: ...
    @property
    def xzoo(self) -> U32Vector4: ...
    @property
    def xzol(self) -> U32Vector4: ...
    @property
    def xzlx(self) -> U32Vector4: ...
    @property
    def xzly(self) -> U32Vector4: ...
    @property
    def xzlz(self) -> U32Vector4: ...
    @property
    def xzlo(self) -> U32Vector4: ...
    @property
    def xzll(self) -> U32Vector4: ...
    @property
    def xoxx(self) -> U32Vector4: ...
    @property
    def xoxy(self) -> U32Vector4: ...
    @property
    def xoxz(self) -> U32Vector4: ...
    @property
    def xoxo(self) -> U32Vector4: ...
    @property
    def xoxl(self) -> U32Vector4: ...
    @property
    def xoyx(self) -> U32Vector4: ...
    @property
    def xoyy(self) -> U32Vector4: ...
    @property
    def xoyz(self) -> U32Vector4: ...
    @property
    def xoyo(self) -> U32Vector4: ...
    @property
    def xoyl(self) -> U32Vector4: ...
    @property
    def xozx(self) -> U32Vector4: ...
    @property
    def xozy(self) -> U32Vector4: ...
    @property
    def xozz(self) -> U32Vector4: ...
    @property
    def xozo(self) -> U32Vector4: ...
    @property
    def xozl(self) -> U32Vector4: ...
    @property
    def xoox(self) -> U32Vector4: ...
    @property
    def xooy(self) -> U32Vector4: ...
    @property
    def xooz(self) -> U32Vector4: ...
    @property
    def xooo(self) -> U32Vector4: ...
    @property
    def xool(self) -> U32Vector4: ...
    @property
    def xolx(self) -> U32Vector4: ...
    @property
    def xoly(self) -> U32Vector4: ...
    @property
    def xolz(self) -> U32Vector4: ...
    @property
    def xolo(self) -> U32Vector4: ...
    @property
    def xoll(self) -> U32Vector4: ...
    @property
    def xlxx(self) -> U32Vector4: ...
    @property
    def xlxy(self) -> U32Vector4: ...
    @property
    def xlxz(self) -> U32Vector4: ...
    @property
    def xlxo(self) -> U32Vector4: ...
    @property
    def xlxl(self) -> U32Vector4: ...
    @property
    def xlyx(self) -> U32Vector4: ...
    @property
    def xlyy(self) -> U32Vector4: ...
    @property
    def xlyz(self) -> U32Vector4: ...
    @property
    def xlyo(self) -> U32Vector4: ...
    @property
    def xlyl(self) -> U32Vector4: ...
    @property
    def xlzx(self) -> U32Vector4: ...
    @property
    def xlzy(self) -> U32Vector4: ...
    @property
    def xlzz(self) -> U32Vector4: ...
    @property
    def xlzo(self) -> U32Vector4: ...
    @property
    def xlzl(self) -> U32Vector4: ...
    @property
    def xlox(self) -> U32Vector4: ...
    @property
    def xloy(self) -> U32Vector4: ...
    @property
    def xloz(self) -> U32Vector4: ...
    @property
    def xloo(self) -> U32Vector4: ...
    @property
    def xlol(self) -> U32Vector4: ...
    @property
    def xllx(self) -> U32Vector4: ...
    @property
    def xlly(self) -> U32Vector4: ...
    @property
    def xllz(self) -> U32Vector4: ...
    @property
    def xllo(self) -> U32Vector4: ...
    @property
    def xlll(self) -> U32Vector4: ...
    @property
    def yxxx(self) -> U32Vector4: ...
    @property
    def yxxy(self) -> U32Vector4: ...
    @property
    def yxxz(self) -> U32Vector4: ...
    @property
    def yxxo(self) -> U32Vector4: ...
    @property
    def yxxl(self) -> U32Vector4: ...
    @property
    def yxyx(self) -> U32Vector4: ...
    @property
    def yxyy(self) -> U32Vector4: ...
    @property
    def yxyz(self) -> U32Vector4: ...
    @property
    def yxyo(self) -> U32Vector4: ...
    @property
    def yxyl(self) -> U32Vector4: ...
    @property
    def yxzx(self) -> U32Vector4: ...
    @property
    def yxzy(self) -> U32Vector4: ...
    @property
    def yxzz(self) -> U32Vector4: ...
    @property
    def yxzo(self) -> U32Vector4: ...
    @property
    def yxzl(self) -> U32Vector4: ...
    @property
    def yxox(self) -> U32Vector4: ...
    @property
    def yxoy(self) -> U32Vector4: ...
    @property
    def yxoz(self) -> U32Vector4: ...
    @property
    def yxoo(self) -> U32Vector4: ...
    @property
    def yxol(self) -> U32Vector4: ...
    @property
    def yxlx(self) -> U32Vector4: ...
    @property
    def yxly(self) -> U32Vector4: ...
    @property
    def yxlz(self) -> U32Vector4: ...
    @property
    def yxlo(self) -> U32Vector4: ...
    @property
    def yxll(self) -> U32Vector4: ...
    @property
    def yyxx(self) -> U32Vector4: ...
    @property
    def yyxy(self) -> U32Vector4: ...
    @property
    def yyxz(self) -> U32Vector4: ...
    @property
    def yyxo(self) -> U32Vector4: ...
    @property
    def yyxl(self) -> U32Vector4: ...
    @property
    def yyyx(self) -> U32Vector4: ...
    @property
    def yyyy(self) -> U32Vector4: ...
    @property
    def yyyz(self) -> U32Vector4: ...
    @property
    def yyyo(self) -> U32Vector4: ...
    @property
    def yyyl(self) -> U32Vector4: ...
    @property
    def yyzx(self) -> U32Vector4: ...
    @property
    def yyzy(self) -> U32Vector4: ...
    @property
    def yyzz(self) -> U32Vector4: ...
    @property
    def yyzo(self) -> U32Vector4: ...
    @property
    def yyzl(self) -> U32Vector4: ...
    @property
    def yyox(self) -> U32Vector4: ...
    @property
    def yyoy(self) -> U32Vector4: ...
    @property
    def yyoz(self) -> U32Vector4: ...
    @property
    def yyoo(self) -> U32Vector4: ...
    @property
    def yyol(self) -> U32Vector4: ...
    @property
    def yylx(self) -> U32Vector4: ...
    @property
    def yyly(self) -> U32Vector4: ...
    @property
    def yylz(self) -> U32Vector4: ...
    @property
    def yylo(self) -> U32Vector4: ...
    @property
    def yyll(self) -> U32Vector4: ...
    @property
    def yzxx(self) -> U32Vector4: ...
    @property
    def yzxy(self) -> U32Vector4: ...
    @property
    def yzxz(self) -> U32Vector4: ...
    @property
    def yzxo(self) -> U32Vector4: ...
    @property
    def yzxl(self) -> U32Vector4: ...
    @property
    def yzyx(self) -> U32Vector4: ...
    @property
    def yzyy(self) -> U32Vector4: ...
    @property
    def yzyz(self) -> U32Vector4: ...
    @property
    def yzyo(self) -> U32Vector4: ...
    @property
    def yzyl(self) -> U32Vector4: ...
    @property
    def yzzx(self) -> U32Vector4: ...
    @property
    def yzzy(self) -> U32Vector4: ...
    @property
    def yzzz(self) -> U32Vector4: ...
    @property
    def yzzo(self) -> U32Vector4: ...
    @property
    def yzzl(self) -> U32Vector4: ...
    @property
    def yzox(self) -> U32Vector4: ...
    @property
    def yzoy(self) -> U32Vector4: ...
    @property
    def yzoz(self) -> U32Vector4: ...
    @property
    def yzoo(self) -> U32Vector4: ...
    @property
    def yzol(self) -> U32Vector4: ...
    @property
    def yzlx(self) -> U32Vector4: ...
    @property
    def yzly(self) -> U32Vector4: ...
    @property
    def yzlz(self) -> U32Vector4: ...
    @property
    def yzlo(self) -> U32Vector4: ...
    @property
    def yzll(self) -> U32Vector4: ...
    @property
    def yoxx(self) -> U32Vector4: ...
    @property
    def yoxy(self) -> U32Vector4: ...
    @property
    def yoxz(self) -> U32Vector4: ...
    @property
    def yoxo(self) -> U32Vector4: ...
    @property
    def yoxl(self) -> U32Vector4: ...
    @property
    def yoyx(self) -> U32Vector4: ...
    @property
    def yoyy(self) -> U32Vector4: ...
    @property
    def yoyz(self) -> U32Vector4: ...
    @property
    def yoyo(self) -> U32Vector4: ...
    @property
    def yoyl(self) -> U32Vector4: ...
    @property
    def yozx(self) -> U32Vector4: ...
    @property
    def yozy(self) -> U32Vector4: ...
    @property
    def yozz(self) -> U32Vector4: ...
    @property
    def yozo(self) -> U32Vector4: ...
    @property
    def yozl(self) -> U32Vector4: ...
    @property
    def yoox(self) -> U32Vector4: ...
    @property
    def yooy(self) -> U32Vector4: ...
    @property
    def yooz(self) -> U32Vector4: ...
    @property
    def yooo(self) -> U32Vector4: ...
    @property
    def yool(self) -> U32Vector4: ...
    @property
    def yolx(self) -> U32Vector4: ...
    @property
    def yoly(self) -> U32Vector4: ...
    @property
    def yolz(self) -> U32Vector4: ...
    @property
    def yolo(self) -> U32Vector4: ...
    @property
    def yoll(self) -> U32Vector4: ...
    @property
    def ylxx(self) -> U32Vector4: ...
    @property
    def ylxy(self) -> U32Vector4: ...
    @property
    def ylxz(self) -> U32Vector4: ...
    @property
    def ylxo(self) -> U32Vector4: ...
    @property
    def ylxl(self) -> U32Vector4: ...
    @property
    def ylyx(self) -> U32Vector4: ...
    @property
    def ylyy(self) -> U32Vector4: ...
    @property
    def ylyz(self) -> U32Vector4: ...
    @property
    def ylyo(self) -> U32Vector4: ...
    @property
    def ylyl(self) -> U32Vector4: ...
    @property
    def ylzx(self) -> U32Vector4: ...
    @property
    def ylzy(self) -> U32Vector4: ...
    @property
    def ylzz(self) -> U32Vector4: ...
    @property
    def ylzo(self) -> U32Vector4: ...
    @property
    def ylzl(self) -> U32Vector4: ...
    @property
    def ylox(self) -> U32Vector4: ...
    @property
    def yloy(self) -> U32Vector4: ...
    @property
    def yloz(self) -> U32Vector4: ...
    @property
    def yloo(self) -> U32Vector4: ...
    @property
    def ylol(self) -> U32Vector4: ...
    @property
    def yllx(self) -> U32Vector4: ...
    @property
    def ylly(self) -> U32Vector4: ...
    @property
    def yllz(self) -> U32Vector4: ...
    @property
    def yllo(self) -> U32Vector4: ...
    @property
    def ylll(self) -> U32Vector4: ...
    @property
    def zxxx(self) -> U32Vector4: ...
    @property
    def zxxy(self) -> U32Vector4: ...
    @property
    def zxxz(self) -> U32Vector4: ...
    @property
    def zxxo(self) -> U32Vector4: ...
    @property
    def zxxl(self) -> U32Vector4: ...
    @property
    def zxyx(self) -> U32Vector4: ...
    @property
    def zxyy(self) -> U32Vector4: ...
    @property
    def zxyz(self) -> U32Vector4: ...
    @property
    def zxyo(self) -> U32Vector4: ...
    @property
    def zxyl(self) -> U32Vector4: ...
    @property
    def zxzx(self) -> U32Vector4: ...
    @property
    def zxzy(self) -> U32Vector4: ...
    @property
    def zxzz(self) -> U32Vector4: ...
    @property
    def zxzo(self) -> U32Vector4: ...
    @property
    def zxzl(self) -> U32Vector4: ...
    @property
    def zxox(self) -> U32Vector4: ...
    @property
    def zxoy(self) -> U32Vector4: ...
    @property
    def zxoz(self) -> U32Vector4: ...
    @property
    def zxoo(self) -> U32Vector4: ...
    @property
    def zxol(self) -> U32Vector4: ...
    @property
    def zxlx(self) -> U32Vector4: ...
    @property
    def zxly(self) -> U32Vector4: ...
    @property
    def zxlz(self) -> U32Vector4: ...
    @property
    def zxlo(self) -> U32Vector4: ...
    @property
    def zxll(self) -> U32Vector4: ...
    @property
    def zyxx(self) -> U32Vector4: ...
    @property
    def zyxy(self) -> U32Vector4: ...
    @property
    def zyxz(self) -> U32Vector4: ...
    @property
    def zyxo(self) -> U32Vector4: ...
    @property
    def zyxl(self) -> U32Vector4: ...
    @property
    def zyyx(self) -> U32Vector4: ...
    @property
    def zyyy(self) -> U32Vector4: ...
    @property
    def zyyz(self) -> U32Vector4: ...
    @property
    def zyyo(self) -> U32Vector4: ...
    @property
    def zyyl(self) -> U32Vector4: ...
    @property
    def zyzx(self) -> U32Vector4: ...
    @property
    def zyzy(self) -> U32Vector4: ...
    @property
    def zyzz(self) -> U32Vector4: ...
    @property
    def zyzo(self) -> U32Vector4: ...
    @property
    def zyzl(self) -> U32Vector4: ...
    @property
    def zyox(self) -> U32Vector4: ...
    @property
    def zyoy(self) -> U32Vector4: ...
    @property
    def zyoz(self) -> U32Vector4: ...
    @property
    def zyoo(self) -> U32Vector4: ...
    @property
    def zyol(self) -> U32Vector4: ...
    @property
    def zylx(self) -> U32Vector4: ...
    @property
    def zyly(self) -> U32Vector4: ...
    @property
    def zylz(self) -> U32Vector4: ...
    @property
    def zylo(self) -> U32Vector4: ...
    @property
    def zyll(self) -> U32Vector4: ...
    @property
    def zzxx(self) -> U32Vector4: ...
    @property
    def zzxy(self) -> U32Vector4: ...
    @property
    def zzxz(self) -> U32Vector4: ...
    @property
    def zzxo(self) -> U32Vector4: ...
    @property
    def zzxl(self) -> U32Vector4: ...
    @property
    def zzyx(self) -> U32Vector4: ...
    @property
    def zzyy(self) -> U32Vector4: ...
    @property
    def zzyz(self) -> U32Vector4: ...
    @property
    def zzyo(self) -> U32Vector4: ...
    @property
    def zzyl(self) -> U32Vector4: ...
    @property
    def zzzx(self) -> U32Vector4: ...
    @property
    def zzzy(self) -> U32Vector4: ...
    @property
    def zzzz(self) -> U32Vector4: ...
    @property
    def zzzo(self) -> U32Vector4: ...
    @property
    def zzzl(self) -> U32Vector4: ...
    @property
    def zzox(self) -> U32Vector4: ...
    @property
    def zzoy(self) -> U32Vector4: ...
    @property
    def zzoz(self) -> U32Vector4: ...
    @property
    def zzoo(self) -> U32Vector4: ...
    @property
    def zzol(self) -> U32Vector4: ...
    @property
    def zzlx(self) -> U32Vector4: ...
    @property
    def zzly(self) -> U32Vector4: ...
    @property
    def zzlz(self) -> U32Vector4: ...
    @property
    def zzlo(self) -> U32Vector4: ...
    @property
    def zzll(self) -> U32Vector4: ...
    @property
    def zoxx(self) -> U32Vector4: ...
    @property
    def zoxy(self) -> U32Vector4: ...
    @property
    def zoxz(self) -> U32Vector4: ...
    @property
    def zoxo(self) -> U32Vector4: ...
    @property
    def zoxl(self) -> U32Vector4: ...
    @property
    def zoyx(self) -> U32Vector4: ...
    @property
    def zoyy(self) -> U32Vector4: ...
    @property
    def zoyz(self) -> U32Vector4: ...
    @property
    def zoyo(self) -> U32Vector4: ...
    @property
    def zoyl(self) -> U32Vector4: ...
    @property
    def zozx(self) -> U32Vector4: ...
    @property
    def zozy(self) -> U32Vector4: ...
    @property
    def zozz(self) -> U32Vector4: ...
    @property
    def zozo(self) -> U32Vector4: ...
    @property
    def zozl(self) -> U32Vector4: ...
    @property
    def zoox(self) -> U32Vector4: ...
    @property
    def zooy(self) -> U32Vector4: ...
    @property
    def zooz(self) -> U32Vector4: ...
    @property
    def zooo(self) -> U32Vector4: ...
    @property
    def zool(self) -> U32Vector4: ...
    @property
    def zolx(self) -> U32Vector4: ...
    @property
    def zoly(self) -> U32Vector4: ...
    @property
    def zolz(self) -> U32Vector4: ...
    @property
    def zolo(self) -> U32Vector4: ...
    @property
    def zoll(self) -> U32Vector4: ...
    @property
    def zlxx(self) -> U32Vector4: ...
    @property
    def zlxy(self) -> U32Vector4: ...
    @property
    def zlxz(self) -> U32Vector4: ...
    @property
    def zlxo(self) -> U32Vector4: ...
    @property
    def zlxl(self) -> U32Vector4: ...
    @property
    def zlyx(self) -> U32Vector4: ...
    @property
    def zlyy(self) -> U32Vector4: ...
    @property
    def zlyz(self) -> U32Vector4: ...
    @property
    def zlyo(self) -> U32Vector4: ...
    @property
    def zlyl(self) -> U32Vector4: ...
    @property
    def zlzx(self) -> U32Vector4: ...
    @property
    def zlzy(self) -> U32Vector4: ...
    @property
    def zlzz(self) -> U32Vector4: ...
    @property
    def zlzo(self) -> U32Vector4: ...
    @property
    def zlzl(self) -> U32Vector4: ...
    @property
    def zlox(self) -> U32Vector4: ...
    @property
    def zloy(self) -> U32Vector4: ...
    @property
    def zloz(self) -> U32Vector4: ...
    @property
    def zloo(self) -> U32Vector4: ...
    @property
    def zlol(self) -> U32Vector4: ...
    @property
    def zllx(self) -> U32Vector4: ...
    @property
    def zlly(self) -> U32Vector4: ...
    @property
    def zllz(self) -> U32Vector4: ...
    @property
    def zllo(self) -> U32Vector4: ...
    @property
    def zlll(self) -> U32Vector4: ...
    @property
    def oxxx(self) -> U32Vector4: ...
    @property
    def oxxy(self) -> U32Vector4: ...
    @property
    def oxxz(self) -> U32Vector4: ...
    @property
    def oxxo(self) -> U32Vector4: ...
    @property
    def oxxl(self) -> U32Vector4: ...
    @property
    def oxyx(self) -> U32Vector4: ...
    @property
    def oxyy(self) -> U32Vector4: ...
    @property
    def oxyz(self) -> U32Vector4: ...
    @property
    def oxyo(self) -> U32Vector4: ...
    @property
    def oxyl(self) -> U32Vector4: ...
    @property
    def oxzx(self) -> U32Vector4: ...
    @property
    def oxzy(self) -> U32Vector4: ...
    @property
    def oxzz(self) -> U32Vector4: ...
    @property
    def oxzo(self) -> U32Vector4: ...
    @property
    def oxzl(self) -> U32Vector4: ...
    @property
    def oxox(self) -> U32Vector4: ...
    @property
    def oxoy(self) -> U32Vector4: ...
    @property
    def oxoz(self) -> U32Vector4: ...
    @property
    def oxoo(self) -> U32Vector4: ...
    @property
    def oxol(self) -> U32Vector4: ...
    @property
    def oxlx(self) -> U32Vector4: ...
    @property
    def oxly(self) -> U32Vector4: ...
    @property
    def oxlz(self) -> U32Vector4: ...
    @property
    def oxlo(self) -> U32Vector4: ...
    @property
    def oxll(self) -> U32Vector4: ...
    @property
    def oyxx(self) -> U32Vector4: ...
    @property
    def oyxy(self) -> U32Vector4: ...
    @property
    def oyxz(self) -> U32Vector4: ...
    @property
    def oyxo(self) -> U32Vector4: ...
    @property
    def oyxl(self) -> U32Vector4: ...
    @property
    def oyyx(self) -> U32Vector4: ...
    @property
    def oyyy(self) -> U32Vector4: ...
    @property
    def oyyz(self) -> U32Vector4: ...
    @property
    def oyyo(self) -> U32Vector4: ...
    @property
    def oyyl(self) -> U32Vector4: ...
    @property
    def oyzx(self) -> U32Vector4: ...
    @property
    def oyzy(self) -> U32Vector4: ...
    @property
    def oyzz(self) -> U32Vector4: ...
    @property
    def oyzo(self) -> U32Vector4: ...
    @property
    def oyzl(self) -> U32Vector4: ...
    @property
    def oyox(self) -> U32Vector4: ...
    @property
    def oyoy(self) -> U32Vector4: ...
    @property
    def oyoz(self) -> U32Vector4: ...
    @property
    def oyoo(self) -> U32Vector4: ...
    @property
    def oyol(self) -> U32Vector4: ...
    @property
    def oylx(self) -> U32Vector4: ...
    @property
    def oyly(self) -> U32Vector4: ...
    @property
    def oylz(self) -> U32Vector4: ...
    @property
    def oylo(self) -> U32Vector4: ...
    @property
    def oyll(self) -> U32Vector4: ...
    @property
    def ozxx(self) -> U32Vector4: ...
    @property
    def ozxy(self) -> U32Vector4: ...
    @property
    def ozxz(self) -> U32Vector4: ...
    @property
    def ozxo(self) -> U32Vector4: ...
    @property
    def ozxl(self) -> U32Vector4: ...
    @property
    def ozyx(self) -> U32Vector4: ...
    @property
    def ozyy(self) -> U32Vector4: ...
    @property
    def ozyz(self) -> U32Vector4: ...
    @property
    def ozyo(self) -> U32Vector4: ...
    @property
    def ozyl(self) -> U32Vector4: ...
    @property
    def ozzx(self) -> U32Vector4: ...
    @property
    def ozzy(self) -> U32Vector4: ...
    @property
    def ozzz(self) -> U32Vector4: ...
    @property
    def ozzo(self) -> U32Vector4: ...
    @property
    def ozzl(self) -> U32Vector4: ...
    @property
    def ozox(self) -> U32Vector4: ...
    @property
    def ozoy(self) -> U32Vector4: ...
    @property
    def ozoz(self) -> U32Vector4: ...
    @property
    def ozoo(self) -> U32Vector4: ...
    @property
    def ozol(self) -> U32Vector4: ...
    @property
    def ozlx(self) -> U32Vector4: ...
    @property
    def ozly(self) -> U32Vector4: ...
    @property
    def ozlz(self) -> U32Vector4: ...
    @property
    def ozlo(self) -> U32Vector4: ...
    @property
    def ozll(self) -> U32Vector4: ...
    @property
    def ooxx(self) -> U32Vector4: ...
    @property
    def ooxy(self) -> U32Vector4: ...
    @property
    def ooxz(self) -> U32Vector4: ...
    @property
    def ooxo(self) -> U32Vector4: ...
    @property
    def ooxl(self) -> U32Vector4: ...
    @property
    def ooyx(self) -> U32Vector4: ...
    @property
    def ooyy(self) -> U32Vector4: ...
    @property
    def ooyz(self) -> U32Vector4: ...
    @property
    def ooyo(self) -> U32Vector4: ...
    @property
    def ooyl(self) -> U32Vector4: ...
    @property
    def oozx(self) -> U32Vector4: ...
    @property
    def oozy(self) -> U32Vector4: ...
    @property
    def oozz(self) -> U32Vector4: ...
    @property
    def oozo(self) -> U32Vector4: ...
    @property
    def oozl(self) -> U32Vector4: ...
    @property
    def ooox(self) -> U32Vector4: ...
    @property
    def oooy(self) -> U32Vector4: ...
    @property
    def oooz(self) -> U32Vector4: ...
    @property
    def oolx(self) -> U32Vector4: ...
    @property
    def ooly(self) -> U32Vector4: ...
    @property
    def oolz(self) -> U32Vector4: ...
    @property
    def olxx(self) -> U32Vector4: ...
    @property
    def olxy(self) -> U32Vector4: ...
    @property
    def olxz(self) -> U32Vector4: ...
    @property
    def olxo(self) -> U32Vector4: ...
    @property
    def olxl(self) -> U32Vector4: ...
    @property
    def olyx(self) -> U32Vector4: ...
    @property
    def olyy(self) -> U32Vector4: ...
    @property
    def olyz(self) -> U32Vector4: ...
    @property
    def olyo(self) -> U32Vector4: ...
    @property
    def olyl(self) -> U32Vector4: ...
    @property
    def olzx(self) -> U32Vector4: ...
    @property
    def olzy(self) -> U32Vector4: ...
    @property
    def olzz(self) -> U32Vector4: ...
    @property
    def olzo(self) -> U32Vector4: ...
    @property
    def olzl(self) -> U32Vector4: ...
    @property
    def olox(self) -> U32Vector4: ...
    @property
    def oloy(self) -> U32Vector4: ...
    @property
    def oloz(self) -> U32Vector4: ...
    @property
    def ollx(self) -> U32Vector4: ...
    @property
    def olly(self) -> U32Vector4: ...
    @property
    def ollz(self) -> U32Vector4: ...
    @property
    def lxxx(self) -> U32Vector4: ...
    @property
    def lxxy(self) -> U32Vector4: ...
    @property
    def lxxz(self) -> U32Vector4: ...
    @property
    def lxxo(self) -> U32Vector4: ...
    @property
    def lxxl(self) -> U32Vector4: ...
    @property
    def lxyx(self) -> U32Vector4: ...
    @property
    def lxyy(self) -> U32Vector4: ...
    @property
    def lxyz(self) -> U32Vector4: ...
    @property
    def lxyo(self) -> U32Vector4: ...
    @property
    def lxyl(self) -> U32Vector4: ...
    @property
    def lxzx(self) -> U32Vector4: ...
    @property
    def lxzy(self) -> U32Vector4: ...
    @property
    def lxzz(self) -> U32Vector4: ...
    @property
    def lxzo(self) -> U32Vector4: ...
    @property
    def lxzl(self) -> U32Vector4: ...
    @property
    def lxox(self) -> U32Vector4: ...
    @property
    def lxoy(self) -> U32Vector4: ...
    @property
    def lxoz(self) -> U32Vector4: ...
    @property
    def lxoo(self) -> U32Vector4: ...
    @property
    def lxol(self) -> U32Vector4: ...
    @property
    def lxlx(self) -> U32Vector4: ...
    @property
    def lxly(self) -> U32Vector4: ...
    @property
    def lxlz(self) -> U32Vector4: ...
    @property
    def lxlo(self) -> U32Vector4: ...
    @property
    def lxll(self) -> U32Vector4: ...
    @property
    def lyxx(self) -> U32Vector4: ...
    @property
    def lyxy(self) -> U32Vector4: ...
    @property
    def lyxz(self) -> U32Vector4: ...
    @property
    def lyxo(self) -> U32Vector4: ...
    @property
    def lyxl(self) -> U32Vector4: ...
    @property
    def lyyx(self) -> U32Vector4: ...
    @property
    def lyyy(self) -> U32Vector4: ...
    @property
    def lyyz(self) -> U32Vector4: ...
    @property
    def lyyo(self) -> U32Vector4: ...
    @property
    def lyyl(self) -> U32Vector4: ...
    @property
    def lyzx(self) -> U32Vector4: ...
    @property
    def lyzy(self) -> U32Vector4: ...
    @property
    def lyzz(self) -> U32Vector4: ...
    @property
    def lyzo(self) -> U32Vector4: ...
    @property
    def lyzl(self) -> U32Vector4: ...
    @property
    def lyox(self) -> U32Vector4: ...
    @property
    def lyoy(self) -> U32Vector4: ...
    @property
    def lyoz(self) -> U32Vector4: ...
    @property
    def lyoo(self) -> U32Vector4: ...
    @property
    def lyol(self) -> U32Vector4: ...
    @property
    def lylx(self) -> U32Vector4: ...
    @property
    def lyly(self) -> U32Vector4: ...
    @property
    def lylz(self) -> U32Vector4: ...
    @property
    def lylo(self) -> U32Vector4: ...
    @property
    def lyll(self) -> U32Vector4: ...
    @property
    def lzxx(self) -> U32Vector4: ...
    @property
    def lzxy(self) -> U32Vector4: ...
    @property
    def lzxz(self) -> U32Vector4: ...
    @property
    def lzxo(self) -> U32Vector4: ...
    @property
    def lzxl(self) -> U32Vector4: ...
    @property
    def lzyx(self) -> U32Vector4: ...
    @property
    def lzyy(self) -> U32Vector4: ...
    @property
    def lzyz(self) -> U32Vector4: ...
    @property
    def lzyo(self) -> U32Vector4: ...
    @property
    def lzyl(self) -> U32Vector4: ...
    @property
    def lzzx(self) -> U32Vector4: ...
    @property
    def lzzy(self) -> U32Vector4: ...
    @property
    def lzzz(self) -> U32Vector4: ...
    @property
    def lzzo(self) -> U32Vector4: ...
    @property
    def lzzl(self) -> U32Vector4: ...
    @property
    def lzox(self) -> U32Vector4: ...
    @property
    def lzoy(self) -> U32Vector4: ...
    @property
    def lzoz(self) -> U32Vector4: ...
    @property
    def lzoo(self) -> U32Vector4: ...
    @property
    def lzol(self) -> U32Vector4: ...
    @property
    def lzlx(self) -> U32Vector4: ...
    @property
    def lzly(self) -> U32Vector4: ...
    @property
    def lzlz(self) -> U32Vector4: ...
    @property
    def lzlo(self) -> U32Vector4: ...
    @property
    def lzll(self) -> U32Vector4: ...
    @property
    def loxx(self) -> U32Vector4: ...
    @property
    def loxy(self) -> U32Vector4: ...
    @property
    def loxz(self) -> U32Vector4: ...
    @property
    def loxo(self) -> U32Vector4: ...
    @property
    def loxl(self) -> U32Vector4: ...
    @property
    def loyx(self) -> U32Vector4: ...
    @property
    def loyy(self) -> U32Vector4: ...
    @property
    def loyz(self) -> U32Vector4: ...
    @property
    def loyo(self) -> U32Vector4: ...
    @property
    def loyl(self) -> U32Vector4: ...
    @property
    def lozx(self) -> U32Vector4: ...
    @property
    def lozy(self) -> U32Vector4: ...
    @property
    def lozz(self) -> U32Vector4: ...
    @property
    def lozo(self) -> U32Vector4: ...
    @property
    def lozl(self) -> U32Vector4: ...
    @property
    def loox(self) -> U32Vector4: ...
    @property
    def looy(self) -> U32Vector4: ...
    @property
    def looz(self) -> U32Vector4: ...
    @property
    def lolx(self) -> U32Vector4: ...
    @property
    def loly(self) -> U32Vector4: ...
    @property
    def lolz(self) -> U32Vector4: ...
    @property
    def llxx(self) -> U32Vector4: ...
    @property
    def llxy(self) -> U32Vector4: ...
    @property
    def llxz(self) -> U32Vector4: ...
    @property
    def llxo(self) -> U32Vector4: ...
    @property
    def llxl(self) -> U32Vector4: ...
    @property
    def llyx(self) -> U32Vector4: ...
    @property
    def llyy(self) -> U32Vector4: ...
    @property
    def llyz(self) -> U32Vector4: ...
    @property
    def llyo(self) -> U32Vector4: ...
    @property
    def llyl(self) -> U32Vector4: ...
    @property
    def llzx(self) -> U32Vector4: ...
    @property
    def llzy(self) -> U32Vector4: ...
    @property
    def llzz(self) -> U32Vector4: ...
    @property
    def llzo(self) -> U32Vector4: ...
    @property
    def llzl(self) -> U32Vector4: ...
    @property
    def llox(self) -> U32Vector4: ...
    @property
    def lloy(self) -> U32Vector4: ...
    @property
    def lloz(self) -> U32Vector4: ...
    @property
    def lllx(self) -> U32Vector4: ...
    @property
    def llly(self) -> U32Vector4: ...
    @property
    def lllz(self) -> U32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> U32Vector2: ...
    @property
    def rg(self) -> U32Vector2: ...
    @property
    def rb(self) -> U32Vector2: ...
    @property
    def ro(self) -> U32Vector2: ...
    @property
    def rl(self) -> U32Vector2: ...
    @property
    def gr(self) -> U32Vector2: ...
    @property
    def gg(self) -> U32Vector2: ...
    @property
    def gb(self) -> U32Vector2: ...
    @property
    def go(self) -> U32Vector2: ...
    @property
    def gl(self) -> U32Vector2: ...
    @property
    def br(self) -> U32Vector2: ...
    @property
    def bg(self) -> U32Vector2: ...
    @property
    def bb(self) -> U32Vector2: ...
    @property
    def bo(self) -> U32Vector2: ...
    @property
    def bl(self) -> U32Vector2: ...
    @property
    def og(self) -> U32Vector2: ...
    @property
    def ob(self) -> U32Vector2: ...
    @property
    def lr(self) -> U32Vector2: ...
    @property
    def lg(self) -> U32Vector2: ...
    @property
    def lb(self) -> U32Vector2: ...
    @property
    def rrr(self) -> U32Vector3: ...
    @property
    def rrg(self) -> U32Vector3: ...
    @property
    def rrb(self) -> U32Vector3: ...
    @property
    def rro(self) -> U32Vector3: ...
    @property
    def rrl(self) -> U32Vector3: ...
    @property
    def rgr(self) -> U32Vector3: ...
    @property
    def rgg(self) -> U32Vector3: ...
    @property
    def rgb(self) -> U32Vector3: ...
    @property
    def rgo(self) -> U32Vector3: ...
    @property
    def rgl(self) -> U32Vector3: ...
    @property
    def rbr(self) -> U32Vector3: ...
    @property
    def rbg(self) -> U32Vector3: ...
    @property
    def rbb(self) -> U32Vector3: ...
    @property
    def rbo(self) -> U32Vector3: ...
    @property
    def rbl(self) -> U32Vector3: ...
    @property
    def ror(self) -> U32Vector3: ...
    @property
    def rog(self) -> U32Vector3: ...
    @property
    def rob(self) -> U32Vector3: ...
    @property
    def roo(self) -> U32Vector3: ...
    @property
    def rol(self) -> U32Vector3: ...
    @property
    def rlr(self) -> U32Vector3: ...
    @property
    def rlg(self) -> U32Vector3: ...
    @property
    def rlb(self) -> U32Vector3: ...
    @property
    def rlo(self) -> U32Vector3: ...
    @property
    def rll(self) -> U32Vector3: ...
    @property
    def grr(self) -> U32Vector3: ...
    @property
    def grg(self) -> U32Vector3: ...
    @property
    def grb(self) -> U32Vector3: ...
    @property
    def gro(self) -> U32Vector3: ...
    @property
    def grl(self) -> U32Vector3: ...
    @property
    def ggr(self) -> U32Vector3: ...
    @property
    def ggg(self) -> U32Vector3: ...
    @property
    def ggb(self) -> U32Vector3: ...
    @property
    def ggo(self) -> U32Vector3: ...
    @property
    def ggl(self) -> U32Vector3: ...
    @property
    def gbr(self) -> U32Vector3: ...
    @property
    def gbg(self) -> U32Vector3: ...
    @property
    def gbb(self) -> U32Vector3: ...
    @property
    def gbo(self) -> U32Vector3: ...
    @property
    def gbl(self) -> U32Vector3: ...
    @property
    def gor(self) -> U32Vector3: ...
    @property
    def gog(self) -> U32Vector3: ...
    @property
    def gob(self) -> U32Vector3: ...
    @property
    def goo(self) -> U32Vector3: ...
    @property
    def gol(self) -> U32Vector3: ...
    @property
    def glr(self) -> U32Vector3: ...
    @property
    def glg(self) -> U32Vector3: ...
    @property
    def glb(self) -> U32Vector3: ...
    @property
    def glo(self) -> U32Vector3: ...
    @property
    def gll(self) -> U32Vector3: ...
    @property
    def brr(self) -> U32Vector3: ...
    @property
    def brg(self) -> U32Vector3: ...
    @property
    def brb(self) -> U32Vector3: ...
    @property
    def bro(self) -> U32Vector3: ...
    @property
    def brl(self) -> U32Vector3: ...
    @property
    def bgr(self) -> U32Vector3: ...
    @property
    def bgg(self) -> U32Vector3: ...
    @property
    def bgb(self) -> U32Vector3: ...
    @property
    def bgo(self) -> U32Vector3: ...
    @property
    def bgl(self) -> U32Vector3: ...
    @property
    def bbr(self) -> U32Vector3: ...
    @property
    def bbg(self) -> U32Vector3: ...
    @property
    def bbb(self) -> U32Vector3: ...
    @property
    def bbo(self) -> U32Vector3: ...
    @property
    def bbl(self) -> U32Vector3: ...
    @property
    def bor(self) -> U32Vector3: ...
    @property
    def bog(self) -> U32Vector3: ...
    @property
    def bob(self) -> U32Vector3: ...
    @property
    def boo(self) -> U32Vector3: ...
    @property
    def bol(self) -> U32Vector3: ...
    @property
    def blr(self) -> U32Vector3: ...
    @property
    def blg(self) -> U32Vector3: ...
    @property
    def blb(self) -> U32Vector3: ...
    @property
    def blo(self) -> U32Vector3: ...
    @property
    def bll(self) -> U32Vector3: ...
    @property
    def orr(self) -> U32Vector3: ...
    @property
    def org(self) -> U32Vector3: ...
    @property
    def orb(self) -> U32Vector3: ...
    @property
    def oro(self) -> U32Vector3: ...
    @property
    def orl(self) -> U32Vector3: ...
    @property
    def ogr(self) -> U32Vector3: ...
    @property
    def ogg(self) -> U32Vector3: ...
    @property
    def ogb(self) -> U32Vector3: ...
    @property
    def ogo(self) -> U32Vector3: ...
    @property
    def ogl(self) -> U32Vector3: ...
    @property
    def obr(self) -> U32Vector3: ...
    @property
    def obg(self) -> U32Vector3: ...
    @property
    def obb(self) -> U32Vector3: ...
    @property
    def obo(self) -> U32Vector3: ...
    @property
    def obl(self) -> U32Vector3: ...
    @property
    def oor(self) -> U32Vector3: ...
    @property
    def oog(self) -> U32Vector3: ...
    @property
    def oob(self) -> U32Vector3: ...
    @property
    def olr(self) -> U32Vector3: ...
    @property
    def olg(self) -> U32Vector3: ...
    @property
    def olb(self) -> U32Vector3: ...
    @property
    def lrr(self) -> U32Vector3: ...
    @property
    def lrg(self) -> U32Vector3: ...
    @property
    def lrb(self) -> U32Vector3: ...
    @property
    def lro(self) -> U32Vector3: ...
    @property
    def lrl(self) -> U32Vector3: ...
    @property
    def lgr(self) -> U32Vector3: ...
    @property
    def lgg(self) -> U32Vector3: ...
    @property
    def lgb(self) -> U32Vector3: ...
    @property
    def lgo(self) -> U32Vector3: ...
    @property
    def lgl(self) -> U32Vector3: ...
    @property
    def lbr(self) -> U32Vector3: ...
    @property
    def lbg(self) -> U32Vector3: ...
    @property
    def lbb(self) -> U32Vector3: ...
    @property
    def lbo(self) -> U32Vector3: ...
    @property
    def lbl(self) -> U32Vector3: ...
    @property
    def lor(self) -> U32Vector3: ...
    @property
    def log(self) -> U32Vector3: ...
    @property
    def lob(self) -> U32Vector3: ...
    @property
    def llr(self) -> U32Vector3: ...
    @property
    def llg(self) -> U32Vector3: ...
    @property
    def llb(self) -> U32Vector3: ...
    @property
    def rrrr(self) -> U32Vector4: ...
    @property
    def rrrg(self) -> U32Vector4: ...
    @property
    def rrrb(self) -> U32Vector4: ...
    @property
    def rrro(self) -> U32Vector4: ...
    @property
    def rrrl(self) -> U32Vector4: ...
    @property
    def rrgr(self) -> U32Vector4: ...
    @property
    def rrgg(self) -> U32Vector4: ...
    @property
    def rrgb(self) -> U32Vector4: ...
    @property
    def rrgo(self) -> U32Vector4: ...
    @property
    def rrgl(self) -> U32Vector4: ...
    @property
    def rrbr(self) -> U32Vector4: ...
    @property
    def rrbg(self) -> U32Vector4: ...
    @property
    def rrbb(self) -> U32Vector4: ...
    @property
    def rrbo(self) -> U32Vector4: ...
    @property
    def rrbl(self) -> U32Vector4: ...
    @property
    def rror(self) -> U32Vector4: ...
    @property
    def rrog(self) -> U32Vector4: ...
    @property
    def rrob(self) -> U32Vector4: ...
    @property
    def rroo(self) -> U32Vector4: ...
    @property
    def rrol(self) -> U32Vector4: ...
    @property
    def rrlr(self) -> U32Vector4: ...
    @property
    def rrlg(self) -> U32Vector4: ...
    @property
    def rrlb(self) -> U32Vector4: ...
    @property
    def rrlo(self) -> U32Vector4: ...
    @property
    def rrll(self) -> U32Vector4: ...
    @property
    def rgrr(self) -> U32Vector4: ...
    @property
    def rgrg(self) -> U32Vector4: ...
    @property
    def rgrb(self) -> U32Vector4: ...
    @property
    def rgro(self) -> U32Vector4: ...
    @property
    def rgrl(self) -> U32Vector4: ...
    @property
    def rggr(self) -> U32Vector4: ...
    @property
    def rggg(self) -> U32Vector4: ...
    @property
    def rggb(self) -> U32Vector4: ...
    @property
    def rggo(self) -> U32Vector4: ...
    @property
    def rggl(self) -> U32Vector4: ...
    @property
    def rgbr(self) -> U32Vector4: ...
    @property
    def rgbg(self) -> U32Vector4: ...
    @property
    def rgbb(self) -> U32Vector4: ...
    @property
    def rgbo(self) -> U32Vector4: ...
    @property
    def rgbl(self) -> U32Vector4: ...
    @property
    def rgor(self) -> U32Vector4: ...
    @property
    def rgog(self) -> U32Vector4: ...
    @property
    def rgob(self) -> U32Vector4: ...
    @property
    def rgoo(self) -> U32Vector4: ...
    @property
    def rgol(self) -> U32Vector4: ...
    @property
    def rglr(self) -> U32Vector4: ...
    @property
    def rglg(self) -> U32Vector4: ...
    @property
    def rglb(self) -> U32Vector4: ...
    @property
    def rglo(self) -> U32Vector4: ...
    @property
    def rgll(self) -> U32Vector4: ...
    @property
    def rbrr(self) -> U32Vector4: ...
    @property
    def rbrg(self) -> U32Vector4: ...
    @property
    def rbrb(self) -> U32Vector4: ...
    @property
    def rbro(self) -> U32Vector4: ...
    @property
    def rbrl(self) -> U32Vector4: ...
    @property
    def rbgr(self) -> U32Vector4: ...
    @property
    def rbgg(self) -> U32Vector4: ...
    @property
    def rbgb(self) -> U32Vector4: ...
    @property
    def rbgo(self) -> U32Vector4: ...
    @property
    def rbgl(self) -> U32Vector4: ...
    @property
    def rbbr(self) -> U32Vector4: ...
    @property
    def rbbg(self) -> U32Vector4: ...
    @property
    def rbbb(self) -> U32Vector4: ...
    @property
    def rbbo(self) -> U32Vector4: ...
    @property
    def rbbl(self) -> U32Vector4: ...
    @property
    def rbor(self) -> U32Vector4: ...
    @property
    def rbog(self) -> U32Vector4: ...
    @property
    def rbob(self) -> U32Vector4: ...
    @property
    def rboo(self) -> U32Vector4: ...
    @property
    def rbol(self) -> U32Vector4: ...
    @property
    def rblr(self) -> U32Vector4: ...
    @property
    def rblg(self) -> U32Vector4: ...
    @property
    def rblb(self) -> U32Vector4: ...
    @property
    def rblo(self) -> U32Vector4: ...
    @property
    def rbll(self) -> U32Vector4: ...
    @property
    def rorr(self) -> U32Vector4: ...
    @property
    def rorg(self) -> U32Vector4: ...
    @property
    def rorb(self) -> U32Vector4: ...
    @property
    def roro(self) -> U32Vector4: ...
    @property
    def rorl(self) -> U32Vector4: ...
    @property
    def rogr(self) -> U32Vector4: ...
    @property
    def rogg(self) -> U32Vector4: ...
    @property
    def rogb(self) -> U32Vector4: ...
    @property
    def rogo(self) -> U32Vector4: ...
    @property
    def rogl(self) -> U32Vector4: ...
    @property
    def robr(self) -> U32Vector4: ...
    @property
    def robg(self) -> U32Vector4: ...
    @property
    def robb(self) -> U32Vector4: ...
    @property
    def robo(self) -> U32Vector4: ...
    @property
    def robl(self) -> U32Vector4: ...
    @property
    def roor(self) -> U32Vector4: ...
    @property
    def roog(self) -> U32Vector4: ...
    @property
    def roob(self) -> U32Vector4: ...
    @property
    def rooo(self) -> U32Vector4: ...
    @property
    def rool(self) -> U32Vector4: ...
    @property
    def rolr(self) -> U32Vector4: ...
    @property
    def rolg(self) -> U32Vector4: ...
    @property
    def rolb(self) -> U32Vector4: ...
    @property
    def rolo(self) -> U32Vector4: ...
    @property
    def roll(self) -> U32Vector4: ...
    @property
    def rlrr(self) -> U32Vector4: ...
    @property
    def rlrg(self) -> U32Vector4: ...
    @property
    def rlrb(self) -> U32Vector4: ...
    @property
    def rlro(self) -> U32Vector4: ...
    @property
    def rlrl(self) -> U32Vector4: ...
    @property
    def rlgr(self) -> U32Vector4: ...
    @property
    def rlgg(self) -> U32Vector4: ...
    @property
    def rlgb(self) -> U32Vector4: ...
    @property
    def rlgo(self) -> U32Vector4: ...
    @property
    def rlgl(self) -> U32Vector4: ...
    @property
    def rlbr(self) -> U32Vector4: ...
    @property
    def rlbg(self) -> U32Vector4: ...
    @property
    def rlbb(self) -> U32Vector4: ...
    @property
    def rlbo(self) -> U32Vector4: ...
    @property
    def rlbl(self) -> U32Vector4: ...
    @property
    def rlor(self) -> U32Vector4: ...
    @property
    def rlog(self) -> U32Vector4: ...
    @property
    def rlob(self) -> U32Vector4: ...
    @property
    def rloo(self) -> U32Vector4: ...
    @property
    def rlol(self) -> U32Vector4: ...
    @property
    def rllr(self) -> U32Vector4: ...
    @property
    def rllg(self) -> U32Vector4: ...
    @property
    def rllb(self) -> U32Vector4: ...
    @property
    def rllo(self) -> U32Vector4: ...
    @property
    def rlll(self) -> U32Vector4: ...
    @property
    def grrr(self) -> U32Vector4: ...
    @property
    def grrg(self) -> U32Vector4: ...
    @property
    def grrb(self) -> U32Vector4: ...
    @property
    def grro(self) -> U32Vector4: ...
    @property
    def grrl(self) -> U32Vector4: ...
    @property
    def grgr(self) -> U32Vector4: ...
    @property
    def grgg(self) -> U32Vector4: ...
    @property
    def grgb(self) -> U32Vector4: ...
    @property
    def grgo(self) -> U32Vector4: ...
    @property
    def grgl(self) -> U32Vector4: ...
    @property
    def grbr(self) -> U32Vector4: ...
    @property
    def grbg(self) -> U32Vector4: ...
    @property
    def grbb(self) -> U32Vector4: ...
    @property
    def grbo(self) -> U32Vector4: ...
    @property
    def grbl(self) -> U32Vector4: ...
    @property
    def gror(self) -> U32Vector4: ...
    @property
    def grog(self) -> U32Vector4: ...
    @property
    def grob(self) -> U32Vector4: ...
    @property
    def groo(self) -> U32Vector4: ...
    @property
    def grol(self) -> U32Vector4: ...
    @property
    def grlr(self) -> U32Vector4: ...
    @property
    def grlg(self) -> U32Vector4: ...
    @property
    def grlb(self) -> U32Vector4: ...
    @property
    def grlo(self) -> U32Vector4: ...
    @property
    def grll(self) -> U32Vector4: ...
    @property
    def ggrr(self) -> U32Vector4: ...
    @property
    def ggrg(self) -> U32Vector4: ...
    @property
    def ggrb(self) -> U32Vector4: ...
    @property
    def ggro(self) -> U32Vector4: ...
    @property
    def ggrl(self) -> U32Vector4: ...
    @property
    def gggr(self) -> U32Vector4: ...
    @property
    def gggg(self) -> U32Vector4: ...
    @property
    def gggb(self) -> U32Vector4: ...
    @property
    def gggo(self) -> U32Vector4: ...
    @property
    def gggl(self) -> U32Vector4: ...
    @property
    def ggbr(self) -> U32Vector4: ...
    @property
    def ggbg(self) -> U32Vector4: ...
    @property
    def ggbb(self) -> U32Vector4: ...
    @property
    def ggbo(self) -> U32Vector4: ...
    @property
    def ggbl(self) -> U32Vector4: ...
    @property
    def ggor(self) -> U32Vector4: ...
    @property
    def ggog(self) -> U32Vector4: ...
    @property
    def ggob(self) -> U32Vector4: ...
    @property
    def ggoo(self) -> U32Vector4: ...
    @property
    def ggol(self) -> U32Vector4: ...
    @property
    def gglr(self) -> U32Vector4: ...
    @property
    def gglg(self) -> U32Vector4: ...
    @property
    def gglb(self) -> U32Vector4: ...
    @property
    def gglo(self) -> U32Vector4: ...
    @property
    def ggll(self) -> U32Vector4: ...
    @property
    def gbrr(self) -> U32Vector4: ...
    @property
    def gbrg(self) -> U32Vector4: ...
    @property
    def gbrb(self) -> U32Vector4: ...
    @property
    def gbro(self) -> U32Vector4: ...
    @property
    def gbrl(self) -> U32Vector4: ...
    @property
    def gbgr(self) -> U32Vector4: ...
    @property
    def gbgg(self) -> U32Vector4: ...
    @property
    def gbgb(self) -> U32Vector4: ...
    @property
    def gbgo(self) -> U32Vector4: ...
    @property
    def gbgl(self) -> U32Vector4: ...
    @property
    def gbbr(self) -> U32Vector4: ...
    @property
    def gbbg(self) -> U32Vector4: ...
    @property
    def gbbb(self) -> U32Vector4: ...
    @property
    def gbbo(self) -> U32Vector4: ...
    @property
    def gbbl(self) -> U32Vector4: ...
    @property
    def gbor(self) -> U32Vector4: ...
    @property
    def gbog(self) -> U32Vector4: ...
    @property
    def gbob(self) -> U32Vector4: ...
    @property
    def gboo(self) -> U32Vector4: ...
    @property
    def gbol(self) -> U32Vector4: ...
    @property
    def gblr(self) -> U32Vector4: ...
    @property
    def gblg(self) -> U32Vector4: ...
    @property
    def gblb(self) -> U32Vector4: ...
    @property
    def gblo(self) -> U32Vector4: ...
    @property
    def gbll(self) -> U32Vector4: ...
    @property
    def gorr(self) -> U32Vector4: ...
    @property
    def gorg(self) -> U32Vector4: ...
    @property
    def gorb(self) -> U32Vector4: ...
    @property
    def goro(self) -> U32Vector4: ...
    @property
    def gorl(self) -> U32Vector4: ...
    @property
    def gogr(self) -> U32Vector4: ...
    @property
    def gogg(self) -> U32Vector4: ...
    @property
    def gogb(self) -> U32Vector4: ...
    @property
    def gogo(self) -> U32Vector4: ...
    @property
    def gogl(self) -> U32Vector4: ...
    @property
    def gobr(self) -> U32Vector4: ...
    @property
    def gobg(self) -> U32Vector4: ...
    @property
    def gobb(self) -> U32Vector4: ...
    @property
    def gobo(self) -> U32Vector4: ...
    @property
    def gobl(self) -> U32Vector4: ...
    @property
    def goor(self) -> U32Vector4: ...
    @property
    def goog(self) -> U32Vector4: ...
    @property
    def goob(self) -> U32Vector4: ...
    @property
    def gooo(self) -> U32Vector4: ...
    @property
    def gool(self) -> U32Vector4: ...
    @property
    def golr(self) -> U32Vector4: ...
    @property
    def golg(self) -> U32Vector4: ...
    @property
    def golb(self) -> U32Vector4: ...
    @property
    def golo(self) -> U32Vector4: ...
    @property
    def goll(self) -> U32Vector4: ...
    @property
    def glrr(self) -> U32Vector4: ...
    @property
    def glrg(self) -> U32Vector4: ...
    @property
    def glrb(self) -> U32Vector4: ...
    @property
    def glro(self) -> U32Vector4: ...
    @property
    def glrl(self) -> U32Vector4: ...
    @property
    def glgr(self) -> U32Vector4: ...
    @property
    def glgg(self) -> U32Vector4: ...
    @property
    def glgb(self) -> U32Vector4: ...
    @property
    def glgo(self) -> U32Vector4: ...
    @property
    def glgl(self) -> U32Vector4: ...
    @property
    def glbr(self) -> U32Vector4: ...
    @property
    def glbg(self) -> U32Vector4: ...
    @property
    def glbb(self) -> U32Vector4: ...
    @property
    def glbo(self) -> U32Vector4: ...
    @property
    def glbl(self) -> U32Vector4: ...
    @property
    def glor(self) -> U32Vector4: ...
    @property
    def glog(self) -> U32Vector4: ...
    @property
    def glob(self) -> U32Vector4: ...
    @property
    def gloo(self) -> U32Vector4: ...
    @property
    def glol(self) -> U32Vector4: ...
    @property
    def gllr(self) -> U32Vector4: ...
    @property
    def gllg(self) -> U32Vector4: ...
    @property
    def gllb(self) -> U32Vector4: ...
    @property
    def gllo(self) -> U32Vector4: ...
    @property
    def glll(self) -> U32Vector4: ...
    @property
    def brrr(self) -> U32Vector4: ...
    @property
    def brrg(self) -> U32Vector4: ...
    @property
    def brrb(self) -> U32Vector4: ...
    @property
    def brro(self) -> U32Vector4: ...
    @property
    def brrl(self) -> U32Vector4: ...
    @property
    def brgr(self) -> U32Vector4: ...
    @property
    def brgg(self) -> U32Vector4: ...
    @property
    def brgb(self) -> U32Vector4: ...
    @property
    def brgo(self) -> U32Vector4: ...
    @property
    def brgl(self) -> U32Vector4: ...
    @property
    def brbr(self) -> U32Vector4: ...
    @property
    def brbg(self) -> U32Vector4: ...
    @property
    def brbb(self) -> U32Vector4: ...
    @property
    def brbo(self) -> U32Vector4: ...
    @property
    def brbl(self) -> U32Vector4: ...
    @property
    def bror(self) -> U32Vector4: ...
    @property
    def brog(self) -> U32Vector4: ...
    @property
    def brob(self) -> U32Vector4: ...
    @property
    def broo(self) -> U32Vector4: ...
    @property
    def brol(self) -> U32Vector4: ...
    @property
    def brlr(self) -> U32Vector4: ...
    @property
    def brlg(self) -> U32Vector4: ...
    @property
    def brlb(self) -> U32Vector4: ...
    @property
    def brlo(self) -> U32Vector4: ...
    @property
    def brll(self) -> U32Vector4: ...
    @property
    def bgrr(self) -> U32Vector4: ...
    @property
    def bgrg(self) -> U32Vector4: ...
    @property
    def bgrb(self) -> U32Vector4: ...
    @property
    def bgro(self) -> U32Vector4: ...
    @property
    def bgrl(self) -> U32Vector4: ...
    @property
    def bggr(self) -> U32Vector4: ...
    @property
    def bggg(self) -> U32Vector4: ...
    @property
    def bggb(self) -> U32Vector4: ...
    @property
    def bggo(self) -> U32Vector4: ...
    @property
    def bggl(self) -> U32Vector4: ...
    @property
    def bgbr(self) -> U32Vector4: ...
    @property
    def bgbg(self) -> U32Vector4: ...
    @property
    def bgbb(self) -> U32Vector4: ...
    @property
    def bgbo(self) -> U32Vector4: ...
    @property
    def bgbl(self) -> U32Vector4: ...
    @property
    def bgor(self) -> U32Vector4: ...
    @property
    def bgog(self) -> U32Vector4: ...
    @property
    def bgob(self) -> U32Vector4: ...
    @property
    def bgoo(self) -> U32Vector4: ...
    @property
    def bgol(self) -> U32Vector4: ...
    @property
    def bglr(self) -> U32Vector4: ...
    @property
    def bglg(self) -> U32Vector4: ...
    @property
    def bglb(self) -> U32Vector4: ...
    @property
    def bglo(self) -> U32Vector4: ...
    @property
    def bgll(self) -> U32Vector4: ...
    @property
    def bbrr(self) -> U32Vector4: ...
    @property
    def bbrg(self) -> U32Vector4: ...
    @property
    def bbrb(self) -> U32Vector4: ...
    @property
    def bbro(self) -> U32Vector4: ...
    @property
    def bbrl(self) -> U32Vector4: ...
    @property
    def bbgr(self) -> U32Vector4: ...
    @property
    def bbgg(self) -> U32Vector4: ...
    @property
    def bbgb(self) -> U32Vector4: ...
    @property
    def bbgo(self) -> U32Vector4: ...
    @property
    def bbgl(self) -> U32Vector4: ...
    @property
    def bbbr(self) -> U32Vector4: ...
    @property
    def bbbg(self) -> U32Vector4: ...
    @property
    def bbbb(self) -> U32Vector4: ...
    @property
    def bbbo(self) -> U32Vector4: ...
    @property
    def bbbl(self) -> U32Vector4: ...
    @property
    def bbor(self) -> U32Vector4: ...
    @property
    def bbog(self) -> U32Vector4: ...
    @property
    def bbob(self) -> U32Vector4: ...
    @property
    def bboo(self) -> U32Vector4: ...
    @property
    def bbol(self) -> U32Vector4: ...
    @property
    def bblr(self) -> U32Vector4: ...
    @property
    def bblg(self) -> U32Vector4: ...
    @property
    def bblb(self) -> U32Vector4: ...
    @property
    def bblo(self) -> U32Vector4: ...
    @property
    def bbll(self) -> U32Vector4: ...
    @property
    def borr(self) -> U32Vector4: ...
    @property
    def borg(self) -> U32Vector4: ...
    @property
    def borb(self) -> U32Vector4: ...
    @property
    def boro(self) -> U32Vector4: ...
    @property
    def borl(self) -> U32Vector4: ...
    @property
    def bogr(self) -> U32Vector4: ...
    @property
    def bogg(self) -> U32Vector4: ...
    @property
    def bogb(self) -> U32Vector4: ...
    @property
    def bogo(self) -> U32Vector4: ...
    @property
    def bogl(self) -> U32Vector4: ...
    @property
    def bobr(self) -> U32Vector4: ...
    @property
    def bobg(self) -> U32Vector4: ...
    @property
    def bobb(self) -> U32Vector4: ...
    @property
    def bobo(self) -> U32Vector4: ...
    @property
    def bobl(self) -> U32Vector4: ...
    @property
    def boor(self) -> U32Vector4: ...
    @property
    def boog(self) -> U32Vector4: ...
    @property
    def boob(self) -> U32Vector4: ...
    @property
    def booo(self) -> U32Vector4: ...
    @property
    def bool(self) -> U32Vector4: ...
    @property
    def bolr(self) -> U32Vector4: ...
    @property
    def bolg(self) -> U32Vector4: ...
    @property
    def bolb(self) -> U32Vector4: ...
    @property
    def bolo(self) -> U32Vector4: ...
    @property
    def boll(self) -> U32Vector4: ...
    @property
    def blrr(self) -> U32Vector4: ...
    @property
    def blrg(self) -> U32Vector4: ...
    @property
    def blrb(self) -> U32Vector4: ...
    @property
    def blro(self) -> U32Vector4: ...
    @property
    def blrl(self) -> U32Vector4: ...
    @property
    def blgr(self) -> U32Vector4: ...
    @property
    def blgg(self) -> U32Vector4: ...
    @property
    def blgb(self) -> U32Vector4: ...
    @property
    def blgo(self) -> U32Vector4: ...
    @property
    def blgl(self) -> U32Vector4: ...
    @property
    def blbr(self) -> U32Vector4: ...
    @property
    def blbg(self) -> U32Vector4: ...
    @property
    def blbb(self) -> U32Vector4: ...
    @property
    def blbo(self) -> U32Vector4: ...
    @property
    def blbl(self) -> U32Vector4: ...
    @property
    def blor(self) -> U32Vector4: ...
    @property
    def blog(self) -> U32Vector4: ...
    @property
    def blob(self) -> U32Vector4: ...
    @property
    def bloo(self) -> U32Vector4: ...
    @property
    def blol(self) -> U32Vector4: ...
    @property
    def bllr(self) -> U32Vector4: ...
    @property
    def bllg(self) -> U32Vector4: ...
    @property
    def bllb(self) -> U32Vector4: ...
    @property
    def bllo(self) -> U32Vector4: ...
    @property
    def blll(self) -> U32Vector4: ...
    @property
    def orrr(self) -> U32Vector4: ...
    @property
    def orrg(self) -> U32Vector4: ...
    @property
    def orrb(self) -> U32Vector4: ...
    @property
    def orro(self) -> U32Vector4: ...
    @property
    def orrl(self) -> U32Vector4: ...
    @property
    def orgr(self) -> U32Vector4: ...
    @property
    def orgg(self) -> U32Vector4: ...
    @property
    def orgb(self) -> U32Vector4: ...
    @property
    def orgo(self) -> U32Vector4: ...
    @property
    def orgl(self) -> U32Vector4: ...
    @property
    def orbr(self) -> U32Vector4: ...
    @property
    def orbg(self) -> U32Vector4: ...
    @property
    def orbb(self) -> U32Vector4: ...
    @property
    def orbo(self) -> U32Vector4: ...
    @property
    def orbl(self) -> U32Vector4: ...
    @property
    def oror(self) -> U32Vector4: ...
    @property
    def orog(self) -> U32Vector4: ...
    @property
    def orob(self) -> U32Vector4: ...
    @property
    def oroo(self) -> U32Vector4: ...
    @property
    def orol(self) -> U32Vector4: ...
    @property
    def orlr(self) -> U32Vector4: ...
    @property
    def orlg(self) -> U32Vector4: ...
    @property
    def orlb(self) -> U32Vector4: ...
    @property
    def orlo(self) -> U32Vector4: ...
    @property
    def orll(self) -> U32Vector4: ...
    @property
    def ogrr(self) -> U32Vector4: ...
    @property
    def ogrg(self) -> U32Vector4: ...
    @property
    def ogrb(self) -> U32Vector4: ...
    @property
    def ogro(self) -> U32Vector4: ...
    @property
    def ogrl(self) -> U32Vector4: ...
    @property
    def oggr(self) -> U32Vector4: ...
    @property
    def oggg(self) -> U32Vector4: ...
    @property
    def oggb(self) -> U32Vector4: ...
    @property
    def oggo(self) -> U32Vector4: ...
    @property
    def oggl(self) -> U32Vector4: ...
    @property
    def ogbr(self) -> U32Vector4: ...
    @property
    def ogbg(self) -> U32Vector4: ...
    @property
    def ogbb(self) -> U32Vector4: ...
    @property
    def ogbo(self) -> U32Vector4: ...
    @property
    def ogbl(self) -> U32Vector4: ...
    @property
    def ogor(self) -> U32Vector4: ...
    @property
    def ogog(self) -> U32Vector4: ...
    @property
    def ogob(self) -> U32Vector4: ...
    @property
    def ogoo(self) -> U32Vector4: ...
    @property
    def ogol(self) -> U32Vector4: ...
    @property
    def oglr(self) -> U32Vector4: ...
    @property
    def oglg(self) -> U32Vector4: ...
    @property
    def oglb(self) -> U32Vector4: ...
    @property
    def oglo(self) -> U32Vector4: ...
    @property
    def ogll(self) -> U32Vector4: ...
    @property
    def obrr(self) -> U32Vector4: ...
    @property
    def obrg(self) -> U32Vector4: ...
    @property
    def obrb(self) -> U32Vector4: ...
    @property
    def obro(self) -> U32Vector4: ...
    @property
    def obrl(self) -> U32Vector4: ...
    @property
    def obgr(self) -> U32Vector4: ...
    @property
    def obgg(self) -> U32Vector4: ...
    @property
    def obgb(self) -> U32Vector4: ...
    @property
    def obgo(self) -> U32Vector4: ...
    @property
    def obgl(self) -> U32Vector4: ...
    @property
    def obbr(self) -> U32Vector4: ...
    @property
    def obbg(self) -> U32Vector4: ...
    @property
    def obbb(self) -> U32Vector4: ...
    @property
    def obbo(self) -> U32Vector4: ...
    @property
    def obbl(self) -> U32Vector4: ...
    @property
    def obor(self) -> U32Vector4: ...
    @property
    def obog(self) -> U32Vector4: ...
    @property
    def obob(self) -> U32Vector4: ...
    @property
    def oboo(self) -> U32Vector4: ...
    @property
    def obol(self) -> U32Vector4: ...
    @property
    def oblr(self) -> U32Vector4: ...
    @property
    def oblg(self) -> U32Vector4: ...
    @property
    def oblb(self) -> U32Vector4: ...
    @property
    def oblo(self) -> U32Vector4: ...
    @property
    def obll(self) -> U32Vector4: ...
    @property
    def oorr(self) -> U32Vector4: ...
    @property
    def oorg(self) -> U32Vector4: ...
    @property
    def oorb(self) -> U32Vector4: ...
    @property
    def ooro(self) -> U32Vector4: ...
    @property
    def oorl(self) -> U32Vector4: ...
    @property
    def oogr(self) -> U32Vector4: ...
    @property
    def oogg(self) -> U32Vector4: ...
    @property
    def oogb(self) -> U32Vector4: ...
    @property
    def oogo(self) -> U32Vector4: ...
    @property
    def oogl(self) -> U32Vector4: ...
    @property
    def oobr(self) -> U32Vector4: ...
    @property
    def oobg(self) -> U32Vector4: ...
    @property
    def oobb(self) -> U32Vector4: ...
    @property
    def oobo(self) -> U32Vector4: ...
    @property
    def oobl(self) -> U32Vector4: ...
    @property
    def ooor(self) -> U32Vector4: ...
    @property
    def ooog(self) -> U32Vector4: ...
    @property
    def ooob(self) -> U32Vector4: ...
    @property
    def oolr(self) -> U32Vector4: ...
    @property
    def oolg(self) -> U32Vector4: ...
    @property
    def oolb(self) -> U32Vector4: ...
    @property
    def olrr(self) -> U32Vector4: ...
    @property
    def olrg(self) -> U32Vector4: ...
    @property
    def olrb(self) -> U32Vector4: ...
    @property
    def olro(self) -> U32Vector4: ...
    @property
    def olrl(self) -> U32Vector4: ...
    @property
    def olgr(self) -> U32Vector4: ...
    @property
    def olgg(self) -> U32Vector4: ...
    @property
    def olgb(self) -> U32Vector4: ...
    @property
    def olgo(self) -> U32Vector4: ...
    @property
    def olgl(self) -> U32Vector4: ...
    @property
    def olbr(self) -> U32Vector4: ...
    @property
    def olbg(self) -> U32Vector4: ...
    @property
    def olbb(self) -> U32Vector4: ...
    @property
    def olbo(self) -> U32Vector4: ...
    @property
    def olbl(self) -> U32Vector4: ...
    @property
    def olor(self) -> U32Vector4: ...
    @property
    def olog(self) -> U32Vector4: ...
    @property
    def olob(self) -> U32Vector4: ...
    @property
    def ollr(self) -> U32Vector4: ...
    @property
    def ollg(self) -> U32Vector4: ...
    @property
    def ollb(self) -> U32Vector4: ...
    @property
    def lrrr(self) -> U32Vector4: ...
    @property
    def lrrg(self) -> U32Vector4: ...
    @property
    def lrrb(self) -> U32Vector4: ...
    @property
    def lrro(self) -> U32Vector4: ...
    @property
    def lrrl(self) -> U32Vector4: ...
    @property
    def lrgr(self) -> U32Vector4: ...
    @property
    def lrgg(self) -> U32Vector4: ...
    @property
    def lrgb(self) -> U32Vector4: ...
    @property
    def lrgo(self) -> U32Vector4: ...
    @property
    def lrgl(self) -> U32Vector4: ...
    @property
    def lrbr(self) -> U32Vector4: ...
    @property
    def lrbg(self) -> U32Vector4: ...
    @property
    def lrbb(self) -> U32Vector4: ...
    @property
    def lrbo(self) -> U32Vector4: ...
    @property
    def lrbl(self) -> U32Vector4: ...
    @property
    def lror(self) -> U32Vector4: ...
    @property
    def lrog(self) -> U32Vector4: ...
    @property
    def lrob(self) -> U32Vector4: ...
    @property
    def lroo(self) -> U32Vector4: ...
    @property
    def lrol(self) -> U32Vector4: ...
    @property
    def lrlr(self) -> U32Vector4: ...
    @property
    def lrlg(self) -> U32Vector4: ...
    @property
    def lrlb(self) -> U32Vector4: ...
    @property
    def lrlo(self) -> U32Vector4: ...
    @property
    def lrll(self) -> U32Vector4: ...
    @property
    def lgrr(self) -> U32Vector4: ...
    @property
    def lgrg(self) -> U32Vector4: ...
    @property
    def lgrb(self) -> U32Vector4: ...
    @property
    def lgro(self) -> U32Vector4: ...
    @property
    def lgrl(self) -> U32Vector4: ...
    @property
    def lggr(self) -> U32Vector4: ...
    @property
    def lggg(self) -> U32Vector4: ...
    @property
    def lggb(self) -> U32Vector4: ...
    @property
    def lggo(self) -> U32Vector4: ...
    @property
    def lggl(self) -> U32Vector4: ...
    @property
    def lgbr(self) -> U32Vector4: ...
    @property
    def lgbg(self) -> U32Vector4: ...
    @property
    def lgbb(self) -> U32Vector4: ...
    @property
    def lgbo(self) -> U32Vector4: ...
    @property
    def lgbl(self) -> U32Vector4: ...
    @property
    def lgor(self) -> U32Vector4: ...
    @property
    def lgog(self) -> U32Vector4: ...
    @property
    def lgob(self) -> U32Vector4: ...
    @property
    def lgoo(self) -> U32Vector4: ...
    @property
    def lgol(self) -> U32Vector4: ...
    @property
    def lglr(self) -> U32Vector4: ...
    @property
    def lglg(self) -> U32Vector4: ...
    @property
    def lglb(self) -> U32Vector4: ...
    @property
    def lglo(self) -> U32Vector4: ...
    @property
    def lgll(self) -> U32Vector4: ...
    @property
    def lbrr(self) -> U32Vector4: ...
    @property
    def lbrg(self) -> U32Vector4: ...
    @property
    def lbrb(self) -> U32Vector4: ...
    @property
    def lbro(self) -> U32Vector4: ...
    @property
    def lbrl(self) -> U32Vector4: ...
    @property
    def lbgr(self) -> U32Vector4: ...
    @property
    def lbgg(self) -> U32Vector4: ...
    @property
    def lbgb(self) -> U32Vector4: ...
    @property
    def lbgo(self) -> U32Vector4: ...
    @property
    def lbgl(self) -> U32Vector4: ...
    @property
    def lbbr(self) -> U32Vector4: ...
    @property
    def lbbg(self) -> U32Vector4: ...
    @property
    def lbbb(self) -> U32Vector4: ...
    @property
    def lbbo(self) -> U32Vector4: ...
    @property
    def lbbl(self) -> U32Vector4: ...
    @property
    def lbor(self) -> U32Vector4: ...
    @property
    def lbog(self) -> U32Vector4: ...
    @property
    def lbob(self) -> U32Vector4: ...
    @property
    def lboo(self) -> U32Vector4: ...
    @property
    def lbol(self) -> U32Vector4: ...
    @property
    def lblr(self) -> U32Vector4: ...
    @property
    def lblg(self) -> U32Vector4: ...
    @property
    def lblb(self) -> U32Vector4: ...
    @property
    def lblo(self) -> U32Vector4: ...
    @property
    def lbll(self) -> U32Vector4: ...
    @property
    def lorr(self) -> U32Vector4: ...
    @property
    def lorg(self) -> U32Vector4: ...
    @property
    def lorb(self) -> U32Vector4: ...
    @property
    def loro(self) -> U32Vector4: ...
    @property
    def lorl(self) -> U32Vector4: ...
    @property
    def logr(self) -> U32Vector4: ...
    @property
    def logg(self) -> U32Vector4: ...
    @property
    def logb(self) -> U32Vector4: ...
    @property
    def logo(self) -> U32Vector4: ...
    @property
    def logl(self) -> U32Vector4: ...
    @property
    def lobr(self) -> U32Vector4: ...
    @property
    def lobg(self) -> U32Vector4: ...
    @property
    def lobb(self) -> U32Vector4: ...
    @property
    def lobo(self) -> U32Vector4: ...
    @property
    def lobl(self) -> U32Vector4: ...
    @property
    def loor(self) -> U32Vector4: ...
    @property
    def loog(self) -> U32Vector4: ...
    @property
    def loob(self) -> U32Vector4: ...
    @property
    def lolr(self) -> U32Vector4: ...
    @property
    def lolg(self) -> U32Vector4: ...
    @property
    def lolb(self) -> U32Vector4: ...
    @property
    def llrr(self) -> U32Vector4: ...
    @property
    def llrg(self) -> U32Vector4: ...
    @property
    def llrb(self) -> U32Vector4: ...
    @property
    def llro(self) -> U32Vector4: ...
    @property
    def llrl(self) -> U32Vector4: ...
    @property
    def llgr(self) -> U32Vector4: ...
    @property
    def llgg(self) -> U32Vector4: ...
    @property
    def llgb(self) -> U32Vector4: ...
    @property
    def llgo(self) -> U32Vector4: ...
    @property
    def llgl(self) -> U32Vector4: ...
    @property
    def llbr(self) -> U32Vector4: ...
    @property
    def llbg(self) -> U32Vector4: ...
    @property
    def llbb(self) -> U32Vector4: ...
    @property
    def llbo(self) -> U32Vector4: ...
    @property
    def llbl(self) -> U32Vector4: ...
    @property
    def llor(self) -> U32Vector4: ...
    @property
    def llog(self) -> U32Vector4: ...
    @property
    def llob(self) -> U32Vector4: ...
    @property
    def lllr(self) -> U32Vector4: ...
    @property
    def lllg(self) -> U32Vector4: ...
    @property
    def lllb(self) -> U32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> U32Vector2: ...
    @property
    def st(self) -> U32Vector2: ...
    @property
    def sq(self) -> U32Vector2: ...
    @property
    def so(self) -> U32Vector2: ...
    @property
    def sl(self) -> U32Vector2: ...
    @property
    def ts(self) -> U32Vector2: ...
    @property
    def tt(self) -> U32Vector2: ...
    @property
    def tq(self) -> U32Vector2: ...
    @property
    def to(self) -> U32Vector2: ...
    @property
    def tl(self) -> U32Vector2: ...
    @property
    def qs(self) -> U32Vector2: ...
    @property
    def qt(self) -> U32Vector2: ...
    @property
    def qq(self) -> U32Vector2: ...
    @property
    def qo(self) -> U32Vector2: ...
    @property
    def ql(self) -> U32Vector2: ...
    @property
    def os(self) -> U32Vector2: ...
    @property
    def ot(self) -> U32Vector2: ...
    @property
    def oq(self) -> U32Vector2: ...
    @property
    def ls(self) -> U32Vector2: ...
    @property
    def lt(self) -> U32Vector2: ...
    @property
    def lq(self) -> U32Vector2: ...
    @property
    def sss(self) -> U32Vector3: ...
    @property
    def sst(self) -> U32Vector3: ...
    @property
    def ssq(self) -> U32Vector3: ...
    @property
    def sso(self) -> U32Vector3: ...
    @property
    def ssl(self) -> U32Vector3: ...
    @property
    def sts(self) -> U32Vector3: ...
    @property
    def stt(self) -> U32Vector3: ...
    @property
    def stq(self) -> U32Vector3: ...
    @property
    def sto(self) -> U32Vector3: ...
    @property
    def stl(self) -> U32Vector3: ...
    @property
    def sqs(self) -> U32Vector3: ...
    @property
    def sqt(self) -> U32Vector3: ...
    @property
    def sqq(self) -> U32Vector3: ...
    @property
    def sqo(self) -> U32Vector3: ...
    @property
    def sql(self) -> U32Vector3: ...
    @property
    def sos(self) -> U32Vector3: ...
    @property
    def sot(self) -> U32Vector3: ...
    @property
    def soq(self) -> U32Vector3: ...
    @property
    def soo(self) -> U32Vector3: ...
    @property
    def sol(self) -> U32Vector3: ...
    @property
    def sls(self) -> U32Vector3: ...
    @property
    def slt(self) -> U32Vector3: ...
    @property
    def slq(self) -> U32Vector3: ...
    @property
    def slo(self) -> U32Vector3: ...
    @property
    def sll(self) -> U32Vector3: ...
    @property
    def tss(self) -> U32Vector3: ...
    @property
    def tst(self) -> U32Vector3: ...
    @property
    def tsq(self) -> U32Vector3: ...
    @property
    def tso(self) -> U32Vector3: ...
    @property
    def tsl(self) -> U32Vector3: ...
    @property
    def tts(self) -> U32Vector3: ...
    @property
    def ttt(self) -> U32Vector3: ...
    @property
    def ttq(self) -> U32Vector3: ...
    @property
    def tto(self) -> U32Vector3: ...
    @property
    def ttl(self) -> U32Vector3: ...
    @property
    def tqs(self) -> U32Vector3: ...
    @property
    def tqt(self) -> U32Vector3: ...
    @property
    def tqq(self) -> U32Vector3: ...
    @property
    def tqo(self) -> U32Vector3: ...
    @property
    def tql(self) -> U32Vector3: ...
    @property
    def tos(self) -> U32Vector3: ...
    @property
    def tot(self) -> U32Vector3: ...
    @property
    def toq(self) -> U32Vector3: ...
    @property
    def too(self) -> U32Vector3: ...
    @property
    def tol(self) -> U32Vector3: ...
    @property
    def tls(self) -> U32Vector3: ...
    @property
    def tlt(self) -> U32Vector3: ...
    @property
    def tlq(self) -> U32Vector3: ...
    @property
    def tlo(self) -> U32Vector3: ...
    @property
    def tll(self) -> U32Vector3: ...
    @property
    def qss(self) -> U32Vector3: ...
    @property
    def qst(self) -> U32Vector3: ...
    @property
    def qsq(self) -> U32Vector3: ...
    @property
    def qso(self) -> U32Vector3: ...
    @property
    def qsl(self) -> U32Vector3: ...
    @property
    def qts(self) -> U32Vector3: ...
    @property
    def qtt(self) -> U32Vector3: ...
    @property
    def qtq(self) -> U32Vector3: ...
    @property
    def qto(self) -> U32Vector3: ...
    @property
    def qtl(self) -> U32Vector3: ...
    @property
    def qqs(self) -> U32Vector3: ...
    @property
    def qqt(self) -> U32Vector3: ...
    @property
    def qqq(self) -> U32Vector3: ...
    @property
    def qqo(self) -> U32Vector3: ...
    @property
    def qql(self) -> U32Vector3: ...
    @property
    def qos(self) -> U32Vector3: ...
    @property
    def qot(self) -> U32Vector3: ...
    @property
    def qoq(self) -> U32Vector3: ...
    @property
    def qoo(self) -> U32Vector3: ...
    @property
    def qol(self) -> U32Vector3: ...
    @property
    def qls(self) -> U32Vector3: ...
    @property
    def qlt(self) -> U32Vector3: ...
    @property
    def qlq(self) -> U32Vector3: ...
    @property
    def qlo(self) -> U32Vector3: ...
    @property
    def qll(self) -> U32Vector3: ...
    @property
    def oss(self) -> U32Vector3: ...
    @property
    def ost(self) -> U32Vector3: ...
    @property
    def osq(self) -> U32Vector3: ...
    @property
    def oso(self) -> U32Vector3: ...
    @property
    def osl(self) -> U32Vector3: ...
    @property
    def ots(self) -> U32Vector3: ...
    @property
    def ott(self) -> U32Vector3: ...
    @property
    def otq(self) -> U32Vector3: ...
    @property
    def oto(self) -> U32Vector3: ...
    @property
    def otl(self) -> U32Vector3: ...
    @property
    def oqs(self) -> U32Vector3: ...
    @property
    def oqt(self) -> U32Vector3: ...
    @property
    def oqq(self) -> U32Vector3: ...
    @property
    def oqo(self) -> U32Vector3: ...
    @property
    def oql(self) -> U32Vector3: ...
    @property
    def oos(self) -> U32Vector3: ...
    @property
    def oot(self) -> U32Vector3: ...
    @property
    def ooq(self) -> U32Vector3: ...
    @property
    def ols(self) -> U32Vector3: ...
    @property
    def olt(self) -> U32Vector3: ...
    @property
    def olq(self) -> U32Vector3: ...
    @property
    def lss(self) -> U32Vector3: ...
    @property
    def lst(self) -> U32Vector3: ...
    @property
    def lsq(self) -> U32Vector3: ...
    @property
    def lso(self) -> U32Vector3: ...
    @property
    def lsl(self) -> U32Vector3: ...
    @property
    def lts(self) -> U32Vector3: ...
    @property
    def ltt(self) -> U32Vector3: ...
    @property
    def ltq(self) -> U32Vector3: ...
    @property
    def lto(self) -> U32Vector3: ...
    @property
    def ltl(self) -> U32Vector3: ...
    @property
    def lqs(self) -> U32Vector3: ...
    @property
    def lqt(self) -> U32Vector3: ...
    @property
    def lqq(self) -> U32Vector3: ...
    @property
    def lqo(self) -> U32Vector3: ...
    @property
    def lql(self) -> U32Vector3: ...
    @property
    def los(self) -> U32Vector3: ...
    @property
    def lot(self) -> U32Vector3: ...
    @property
    def loq(self) -> U32Vector3: ...
    @property
    def lls(self) -> U32Vector3: ...
    @property
    def llt(self) -> U32Vector3: ...
    @property
    def llq(self) -> U32Vector3: ...
    @property
    def ssss(self) -> U32Vector4: ...
    @property
    def ssst(self) -> U32Vector4: ...
    @property
    def sssq(self) -> U32Vector4: ...
    @property
    def ssso(self) -> U32Vector4: ...
    @property
    def sssl(self) -> U32Vector4: ...
    @property
    def ssts(self) -> U32Vector4: ...
    @property
    def sstt(self) -> U32Vector4: ...
    @property
    def sstq(self) -> U32Vector4: ...
    @property
    def ssto(self) -> U32Vector4: ...
    @property
    def sstl(self) -> U32Vector4: ...
    @property
    def ssqs(self) -> U32Vector4: ...
    @property
    def ssqt(self) -> U32Vector4: ...
    @property
    def ssqq(self) -> U32Vector4: ...
    @property
    def ssqo(self) -> U32Vector4: ...
    @property
    def ssql(self) -> U32Vector4: ...
    @property
    def ssos(self) -> U32Vector4: ...
    @property
    def ssot(self) -> U32Vector4: ...
    @property
    def ssoq(self) -> U32Vector4: ...
    @property
    def ssoo(self) -> U32Vector4: ...
    @property
    def ssol(self) -> U32Vector4: ...
    @property
    def ssls(self) -> U32Vector4: ...
    @property
    def sslt(self) -> U32Vector4: ...
    @property
    def sslq(self) -> U32Vector4: ...
    @property
    def sslo(self) -> U32Vector4: ...
    @property
    def ssll(self) -> U32Vector4: ...
    @property
    def stss(self) -> U32Vector4: ...
    @property
    def stst(self) -> U32Vector4: ...
    @property
    def stsq(self) -> U32Vector4: ...
    @property
    def stso(self) -> U32Vector4: ...
    @property
    def stsl(self) -> U32Vector4: ...
    @property
    def stts(self) -> U32Vector4: ...
    @property
    def sttt(self) -> U32Vector4: ...
    @property
    def sttq(self) -> U32Vector4: ...
    @property
    def stto(self) -> U32Vector4: ...
    @property
    def sttl(self) -> U32Vector4: ...
    @property
    def stqs(self) -> U32Vector4: ...
    @property
    def stqt(self) -> U32Vector4: ...
    @property
    def stqq(self) -> U32Vector4: ...
    @property
    def stqo(self) -> U32Vector4: ...
    @property
    def stql(self) -> U32Vector4: ...
    @property
    def stos(self) -> U32Vector4: ...
    @property
    def stot(self) -> U32Vector4: ...
    @property
    def stoq(self) -> U32Vector4: ...
    @property
    def stoo(self) -> U32Vector4: ...
    @property
    def stol(self) -> U32Vector4: ...
    @property
    def stls(self) -> U32Vector4: ...
    @property
    def stlt(self) -> U32Vector4: ...
    @property
    def stlq(self) -> U32Vector4: ...
    @property
    def stlo(self) -> U32Vector4: ...
    @property
    def stll(self) -> U32Vector4: ...
    @property
    def sqss(self) -> U32Vector4: ...
    @property
    def sqst(self) -> U32Vector4: ...
    @property
    def sqsq(self) -> U32Vector4: ...
    @property
    def sqso(self) -> U32Vector4: ...
    @property
    def sqsl(self) -> U32Vector4: ...
    @property
    def sqts(self) -> U32Vector4: ...
    @property
    def sqtt(self) -> U32Vector4: ...
    @property
    def sqtq(self) -> U32Vector4: ...
    @property
    def sqto(self) -> U32Vector4: ...
    @property
    def sqtl(self) -> U32Vector4: ...
    @property
    def sqqs(self) -> U32Vector4: ...
    @property
    def sqqt(self) -> U32Vector4: ...
    @property
    def sqqq(self) -> U32Vector4: ...
    @property
    def sqqo(self) -> U32Vector4: ...
    @property
    def sqql(self) -> U32Vector4: ...
    @property
    def sqos(self) -> U32Vector4: ...
    @property
    def sqot(self) -> U32Vector4: ...
    @property
    def sqoq(self) -> U32Vector4: ...
    @property
    def sqoo(self) -> U32Vector4: ...
    @property
    def sqol(self) -> U32Vector4: ...
    @property
    def sqls(self) -> U32Vector4: ...
    @property
    def sqlt(self) -> U32Vector4: ...
    @property
    def sqlq(self) -> U32Vector4: ...
    @property
    def sqlo(self) -> U32Vector4: ...
    @property
    def sqll(self) -> U32Vector4: ...
    @property
    def soss(self) -> U32Vector4: ...
    @property
    def sost(self) -> U32Vector4: ...
    @property
    def sosq(self) -> U32Vector4: ...
    @property
    def soso(self) -> U32Vector4: ...
    @property
    def sosl(self) -> U32Vector4: ...
    @property
    def sots(self) -> U32Vector4: ...
    @property
    def sott(self) -> U32Vector4: ...
    @property
    def sotq(self) -> U32Vector4: ...
    @property
    def soto(self) -> U32Vector4: ...
    @property
    def sotl(self) -> U32Vector4: ...
    @property
    def soqs(self) -> U32Vector4: ...
    @property
    def soqt(self) -> U32Vector4: ...
    @property
    def soqq(self) -> U32Vector4: ...
    @property
    def soqo(self) -> U32Vector4: ...
    @property
    def soql(self) -> U32Vector4: ...
    @property
    def soos(self) -> U32Vector4: ...
    @property
    def soot(self) -> U32Vector4: ...
    @property
    def sooq(self) -> U32Vector4: ...
    @property
    def sooo(self) -> U32Vector4: ...
    @property
    def sool(self) -> U32Vector4: ...
    @property
    def sols(self) -> U32Vector4: ...
    @property
    def solt(self) -> U32Vector4: ...
    @property
    def solq(self) -> U32Vector4: ...
    @property
    def solo(self) -> U32Vector4: ...
    @property
    def soll(self) -> U32Vector4: ...
    @property
    def slss(self) -> U32Vector4: ...
    @property
    def slst(self) -> U32Vector4: ...
    @property
    def slsq(self) -> U32Vector4: ...
    @property
    def slso(self) -> U32Vector4: ...
    @property
    def slsl(self) -> U32Vector4: ...
    @property
    def slts(self) -> U32Vector4: ...
    @property
    def sltt(self) -> U32Vector4: ...
    @property
    def sltq(self) -> U32Vector4: ...
    @property
    def slto(self) -> U32Vector4: ...
    @property
    def sltl(self) -> U32Vector4: ...
    @property
    def slqs(self) -> U32Vector4: ...
    @property
    def slqt(self) -> U32Vector4: ...
    @property
    def slqq(self) -> U32Vector4: ...
    @property
    def slqo(self) -> U32Vector4: ...
    @property
    def slql(self) -> U32Vector4: ...
    @property
    def slos(self) -> U32Vector4: ...
    @property
    def slot(self) -> U32Vector4: ...
    @property
    def sloq(self) -> U32Vector4: ...
    @property
    def sloo(self) -> U32Vector4: ...
    @property
    def slol(self) -> U32Vector4: ...
    @property
    def slls(self) -> U32Vector4: ...
    @property
    def sllt(self) -> U32Vector4: ...
    @property
    def sllq(self) -> U32Vector4: ...
    @property
    def sllo(self) -> U32Vector4: ...
    @property
    def slll(self) -> U32Vector4: ...
    @property
    def tsss(self) -> U32Vector4: ...
    @property
    def tsst(self) -> U32Vector4: ...
    @property
    def tssq(self) -> U32Vector4: ...
    @property
    def tsso(self) -> U32Vector4: ...
    @property
    def tssl(self) -> U32Vector4: ...
    @property
    def tsts(self) -> U32Vector4: ...
    @property
    def tstt(self) -> U32Vector4: ...
    @property
    def tstq(self) -> U32Vector4: ...
    @property
    def tsto(self) -> U32Vector4: ...
    @property
    def tstl(self) -> U32Vector4: ...
    @property
    def tsqs(self) -> U32Vector4: ...
    @property
    def tsqt(self) -> U32Vector4: ...
    @property
    def tsqq(self) -> U32Vector4: ...
    @property
    def tsqo(self) -> U32Vector4: ...
    @property
    def tsql(self) -> U32Vector4: ...
    @property
    def tsos(self) -> U32Vector4: ...
    @property
    def tsot(self) -> U32Vector4: ...
    @property
    def tsoq(self) -> U32Vector4: ...
    @property
    def tsoo(self) -> U32Vector4: ...
    @property
    def tsol(self) -> U32Vector4: ...
    @property
    def tsls(self) -> U32Vector4: ...
    @property
    def tslt(self) -> U32Vector4: ...
    @property
    def tslq(self) -> U32Vector4: ...
    @property
    def tslo(self) -> U32Vector4: ...
    @property
    def tsll(self) -> U32Vector4: ...
    @property
    def ttss(self) -> U32Vector4: ...
    @property
    def ttst(self) -> U32Vector4: ...
    @property
    def ttsq(self) -> U32Vector4: ...
    @property
    def ttso(self) -> U32Vector4: ...
    @property
    def ttsl(self) -> U32Vector4: ...
    @property
    def ttts(self) -> U32Vector4: ...
    @property
    def tttt(self) -> U32Vector4: ...
    @property
    def tttq(self) -> U32Vector4: ...
    @property
    def ttto(self) -> U32Vector4: ...
    @property
    def tttl(self) -> U32Vector4: ...
    @property
    def ttqs(self) -> U32Vector4: ...
    @property
    def ttqt(self) -> U32Vector4: ...
    @property
    def ttqq(self) -> U32Vector4: ...
    @property
    def ttqo(self) -> U32Vector4: ...
    @property
    def ttql(self) -> U32Vector4: ...
    @property
    def ttos(self) -> U32Vector4: ...
    @property
    def ttot(self) -> U32Vector4: ...
    @property
    def ttoq(self) -> U32Vector4: ...
    @property
    def ttoo(self) -> U32Vector4: ...
    @property
    def ttol(self) -> U32Vector4: ...
    @property
    def ttls(self) -> U32Vector4: ...
    @property
    def ttlt(self) -> U32Vector4: ...
    @property
    def ttlq(self) -> U32Vector4: ...
    @property
    def ttlo(self) -> U32Vector4: ...
    @property
    def ttll(self) -> U32Vector4: ...
    @property
    def tqss(self) -> U32Vector4: ...
    @property
    def tqst(self) -> U32Vector4: ...
    @property
    def tqsq(self) -> U32Vector4: ...
    @property
    def tqso(self) -> U32Vector4: ...
    @property
    def tqsl(self) -> U32Vector4: ...
    @property
    def tqts(self) -> U32Vector4: ...
    @property
    def tqtt(self) -> U32Vector4: ...
    @property
    def tqtq(self) -> U32Vector4: ...
    @property
    def tqto(self) -> U32Vector4: ...
    @property
    def tqtl(self) -> U32Vector4: ...
    @property
    def tqqs(self) -> U32Vector4: ...
    @property
    def tqqt(self) -> U32Vector4: ...
    @property
    def tqqq(self) -> U32Vector4: ...
    @property
    def tqqo(self) -> U32Vector4: ...
    @property
    def tqql(self) -> U32Vector4: ...
    @property
    def tqos(self) -> U32Vector4: ...
    @property
    def tqot(self) -> U32Vector4: ...
    @property
    def tqoq(self) -> U32Vector4: ...
    @property
    def tqoo(self) -> U32Vector4: ...
    @property
    def tqol(self) -> U32Vector4: ...
    @property
    def tqls(self) -> U32Vector4: ...
    @property
    def tqlt(self) -> U32Vector4: ...
    @property
    def tqlq(self) -> U32Vector4: ...
    @property
    def tqlo(self) -> U32Vector4: ...
    @property
    def tqll(self) -> U32Vector4: ...
    @property
    def toss(self) -> U32Vector4: ...
    @property
    def tost(self) -> U32Vector4: ...
    @property
    def tosq(self) -> U32Vector4: ...
    @property
    def toso(self) -> U32Vector4: ...
    @property
    def tosl(self) -> U32Vector4: ...
    @property
    def tots(self) -> U32Vector4: ...
    @property
    def tott(self) -> U32Vector4: ...
    @property
    def totq(self) -> U32Vector4: ...
    @property
    def toto(self) -> U32Vector4: ...
    @property
    def totl(self) -> U32Vector4: ...
    @property
    def toqs(self) -> U32Vector4: ...
    @property
    def toqt(self) -> U32Vector4: ...
    @property
    def toqq(self) -> U32Vector4: ...
    @property
    def toqo(self) -> U32Vector4: ...
    @property
    def toql(self) -> U32Vector4: ...
    @property
    def toos(self) -> U32Vector4: ...
    @property
    def toot(self) -> U32Vector4: ...
    @property
    def tooq(self) -> U32Vector4: ...
    @property
    def tooo(self) -> U32Vector4: ...
    @property
    def tool(self) -> U32Vector4: ...
    @property
    def tols(self) -> U32Vector4: ...
    @property
    def tolt(self) -> U32Vector4: ...
    @property
    def tolq(self) -> U32Vector4: ...
    @property
    def tolo(self) -> U32Vector4: ...
    @property
    def toll(self) -> U32Vector4: ...
    @property
    def tlss(self) -> U32Vector4: ...
    @property
    def tlst(self) -> U32Vector4: ...
    @property
    def tlsq(self) -> U32Vector4: ...
    @property
    def tlso(self) -> U32Vector4: ...
    @property
    def tlsl(self) -> U32Vector4: ...
    @property
    def tlts(self) -> U32Vector4: ...
    @property
    def tltt(self) -> U32Vector4: ...
    @property
    def tltq(self) -> U32Vector4: ...
    @property
    def tlto(self) -> U32Vector4: ...
    @property
    def tltl(self) -> U32Vector4: ...
    @property
    def tlqs(self) -> U32Vector4: ...
    @property
    def tlqt(self) -> U32Vector4: ...
    @property
    def tlqq(self) -> U32Vector4: ...
    @property
    def tlqo(self) -> U32Vector4: ...
    @property
    def tlql(self) -> U32Vector4: ...
    @property
    def tlos(self) -> U32Vector4: ...
    @property
    def tlot(self) -> U32Vector4: ...
    @property
    def tloq(self) -> U32Vector4: ...
    @property
    def tloo(self) -> U32Vector4: ...
    @property
    def tlol(self) -> U32Vector4: ...
    @property
    def tlls(self) -> U32Vector4: ...
    @property
    def tllt(self) -> U32Vector4: ...
    @property
    def tllq(self) -> U32Vector4: ...
    @property
    def tllo(self) -> U32Vector4: ...
    @property
    def tlll(self) -> U32Vector4: ...
    @property
    def qsss(self) -> U32Vector4: ...
    @property
    def qsst(self) -> U32Vector4: ...
    @property
    def qssq(self) -> U32Vector4: ...
    @property
    def qsso(self) -> U32Vector4: ...
    @property
    def qssl(self) -> U32Vector4: ...
    @property
    def qsts(self) -> U32Vector4: ...
    @property
    def qstt(self) -> U32Vector4: ...
    @property
    def qstq(self) -> U32Vector4: ...
    @property
    def qsto(self) -> U32Vector4: ...
    @property
    def qstl(self) -> U32Vector4: ...
    @property
    def qsqs(self) -> U32Vector4: ...
    @property
    def qsqt(self) -> U32Vector4: ...
    @property
    def qsqq(self) -> U32Vector4: ...
    @property
    def qsqo(self) -> U32Vector4: ...
    @property
    def qsql(self) -> U32Vector4: ...
    @property
    def qsos(self) -> U32Vector4: ...
    @property
    def qsot(self) -> U32Vector4: ...
    @property
    def qsoq(self) -> U32Vector4: ...
    @property
    def qsoo(self) -> U32Vector4: ...
    @property
    def qsol(self) -> U32Vector4: ...
    @property
    def qsls(self) -> U32Vector4: ...
    @property
    def qslt(self) -> U32Vector4: ...
    @property
    def qslq(self) -> U32Vector4: ...
    @property
    def qslo(self) -> U32Vector4: ...
    @property
    def qsll(self) -> U32Vector4: ...
    @property
    def qtss(self) -> U32Vector4: ...
    @property
    def qtst(self) -> U32Vector4: ...
    @property
    def qtsq(self) -> U32Vector4: ...
    @property
    def qtso(self) -> U32Vector4: ...
    @property
    def qtsl(self) -> U32Vector4: ...
    @property
    def qtts(self) -> U32Vector4: ...
    @property
    def qttt(self) -> U32Vector4: ...
    @property
    def qttq(self) -> U32Vector4: ...
    @property
    def qtto(self) -> U32Vector4: ...
    @property
    def qttl(self) -> U32Vector4: ...
    @property
    def qtqs(self) -> U32Vector4: ...
    @property
    def qtqt(self) -> U32Vector4: ...
    @property
    def qtqq(self) -> U32Vector4: ...
    @property
    def qtqo(self) -> U32Vector4: ...
    @property
    def qtql(self) -> U32Vector4: ...
    @property
    def qtos(self) -> U32Vector4: ...
    @property
    def qtot(self) -> U32Vector4: ...
    @property
    def qtoq(self) -> U32Vector4: ...
    @property
    def qtoo(self) -> U32Vector4: ...
    @property
    def qtol(self) -> U32Vector4: ...
    @property
    def qtls(self) -> U32Vector4: ...
    @property
    def qtlt(self) -> U32Vector4: ...
    @property
    def qtlq(self) -> U32Vector4: ...
    @property
    def qtlo(self) -> U32Vector4: ...
    @property
    def qtll(self) -> U32Vector4: ...
    @property
    def qqss(self) -> U32Vector4: ...
    @property
    def qqst(self) -> U32Vector4: ...
    @property
    def qqsq(self) -> U32Vector4: ...
    @property
    def qqso(self) -> U32Vector4: ...
    @property
    def qqsl(self) -> U32Vector4: ...
    @property
    def qqts(self) -> U32Vector4: ...
    @property
    def qqtt(self) -> U32Vector4: ...
    @property
    def qqtq(self) -> U32Vector4: ...
    @property
    def qqto(self) -> U32Vector4: ...
    @property
    def qqtl(self) -> U32Vector4: ...
    @property
    def qqqs(self) -> U32Vector4: ...
    @property
    def qqqt(self) -> U32Vector4: ...
    @property
    def qqqq(self) -> U32Vector4: ...
    @property
    def qqqo(self) -> U32Vector4: ...
    @property
    def qqql(self) -> U32Vector4: ...
    @property
    def qqos(self) -> U32Vector4: ...
    @property
    def qqot(self) -> U32Vector4: ...
    @property
    def qqoq(self) -> U32Vector4: ...
    @property
    def qqoo(self) -> U32Vector4: ...
    @property
    def qqol(self) -> U32Vector4: ...
    @property
    def qqls(self) -> U32Vector4: ...
    @property
    def qqlt(self) -> U32Vector4: ...
    @property
    def qqlq(self) -> U32Vector4: ...
    @property
    def qqlo(self) -> U32Vector4: ...
    @property
    def qqll(self) -> U32Vector4: ...
    @property
    def qoss(self) -> U32Vector4: ...
    @property
    def qost(self) -> U32Vector4: ...
    @property
    def qosq(self) -> U32Vector4: ...
    @property
    def qoso(self) -> U32Vector4: ...
    @property
    def qosl(self) -> U32Vector4: ...
    @property
    def qots(self) -> U32Vector4: ...
    @property
    def qott(self) -> U32Vector4: ...
    @property
    def qotq(self) -> U32Vector4: ...
    @property
    def qoto(self) -> U32Vector4: ...
    @property
    def qotl(self) -> U32Vector4: ...
    @property
    def qoqs(self) -> U32Vector4: ...
    @property
    def qoqt(self) -> U32Vector4: ...
    @property
    def qoqq(self) -> U32Vector4: ...
    @property
    def qoqo(self) -> U32Vector4: ...
    @property
    def qoql(self) -> U32Vector4: ...
    @property
    def qoos(self) -> U32Vector4: ...
    @property
    def qoot(self) -> U32Vector4: ...
    @property
    def qooq(self) -> U32Vector4: ...
    @property
    def qooo(self) -> U32Vector4: ...
    @property
    def qool(self) -> U32Vector4: ...
    @property
    def qols(self) -> U32Vector4: ...
    @property
    def qolt(self) -> U32Vector4: ...
    @property
    def qolq(self) -> U32Vector4: ...
    @property
    def qolo(self) -> U32Vector4: ...
    @property
    def qoll(self) -> U32Vector4: ...
    @property
    def qlss(self) -> U32Vector4: ...
    @property
    def qlst(self) -> U32Vector4: ...
    @property
    def qlsq(self) -> U32Vector4: ...
    @property
    def qlso(self) -> U32Vector4: ...
    @property
    def qlsl(self) -> U32Vector4: ...
    @property
    def qlts(self) -> U32Vector4: ...
    @property
    def qltt(self) -> U32Vector4: ...
    @property
    def qltq(self) -> U32Vector4: ...
    @property
    def qlto(self) -> U32Vector4: ...
    @property
    def qltl(self) -> U32Vector4: ...
    @property
    def qlqs(self) -> U32Vector4: ...
    @property
    def qlqt(self) -> U32Vector4: ...
    @property
    def qlqq(self) -> U32Vector4: ...
    @property
    def qlqo(self) -> U32Vector4: ...
    @property
    def qlql(self) -> U32Vector4: ...
    @property
    def qlos(self) -> U32Vector4: ...
    @property
    def qlot(self) -> U32Vector4: ...
    @property
    def qloq(self) -> U32Vector4: ...
    @property
    def qloo(self) -> U32Vector4: ...
    @property
    def qlol(self) -> U32Vector4: ...
    @property
    def qlls(self) -> U32Vector4: ...
    @property
    def qllt(self) -> U32Vector4: ...
    @property
    def qllq(self) -> U32Vector4: ...
    @property
    def qllo(self) -> U32Vector4: ...
    @property
    def qlll(self) -> U32Vector4: ...
    @property
    def osss(self) -> U32Vector4: ...
    @property
    def osst(self) -> U32Vector4: ...
    @property
    def ossq(self) -> U32Vector4: ...
    @property
    def osso(self) -> U32Vector4: ...
    @property
    def ossl(self) -> U32Vector4: ...
    @property
    def osts(self) -> U32Vector4: ...
    @property
    def ostt(self) -> U32Vector4: ...
    @property
    def ostq(self) -> U32Vector4: ...
    @property
    def osto(self) -> U32Vector4: ...
    @property
    def ostl(self) -> U32Vector4: ...
    @property
    def osqs(self) -> U32Vector4: ...
    @property
    def osqt(self) -> U32Vector4: ...
    @property
    def osqq(self) -> U32Vector4: ...
    @property
    def osqo(self) -> U32Vector4: ...
    @property
    def osql(self) -> U32Vector4: ...
    @property
    def osos(self) -> U32Vector4: ...
    @property
    def osot(self) -> U32Vector4: ...
    @property
    def osoq(self) -> U32Vector4: ...
    @property
    def osoo(self) -> U32Vector4: ...
    @property
    def osol(self) -> U32Vector4: ...
    @property
    def osls(self) -> U32Vector4: ...
    @property
    def oslt(self) -> U32Vector4: ...
    @property
    def oslq(self) -> U32Vector4: ...
    @property
    def oslo(self) -> U32Vector4: ...
    @property
    def osll(self) -> U32Vector4: ...
    @property
    def otss(self) -> U32Vector4: ...
    @property
    def otst(self) -> U32Vector4: ...
    @property
    def otsq(self) -> U32Vector4: ...
    @property
    def otso(self) -> U32Vector4: ...
    @property
    def otsl(self) -> U32Vector4: ...
    @property
    def otts(self) -> U32Vector4: ...
    @property
    def ottt(self) -> U32Vector4: ...
    @property
    def ottq(self) -> U32Vector4: ...
    @property
    def otto(self) -> U32Vector4: ...
    @property
    def ottl(self) -> U32Vector4: ...
    @property
    def otqs(self) -> U32Vector4: ...
    @property
    def otqt(self) -> U32Vector4: ...
    @property
    def otqq(self) -> U32Vector4: ...
    @property
    def otqo(self) -> U32Vector4: ...
    @property
    def otql(self) -> U32Vector4: ...
    @property
    def otos(self) -> U32Vector4: ...
    @property
    def otot(self) -> U32Vector4: ...
    @property
    def otoq(self) -> U32Vector4: ...
    @property
    def otoo(self) -> U32Vector4: ...
    @property
    def otol(self) -> U32Vector4: ...
    @property
    def otls(self) -> U32Vector4: ...
    @property
    def otlt(self) -> U32Vector4: ...
    @property
    def otlq(self) -> U32Vector4: ...
    @property
    def otlo(self) -> U32Vector4: ...
    @property
    def otll(self) -> U32Vector4: ...
    @property
    def oqss(self) -> U32Vector4: ...
    @property
    def oqst(self) -> U32Vector4: ...
    @property
    def oqsq(self) -> U32Vector4: ...
    @property
    def oqso(self) -> U32Vector4: ...
    @property
    def oqsl(self) -> U32Vector4: ...
    @property
    def oqts(self) -> U32Vector4: ...
    @property
    def oqtt(self) -> U32Vector4: ...
    @property
    def oqtq(self) -> U32Vector4: ...
    @property
    def oqto(self) -> U32Vector4: ...
    @property
    def oqtl(self) -> U32Vector4: ...
    @property
    def oqqs(self) -> U32Vector4: ...
    @property
    def oqqt(self) -> U32Vector4: ...
    @property
    def oqqq(self) -> U32Vector4: ...
    @property
    def oqqo(self) -> U32Vector4: ...
    @property
    def oqql(self) -> U32Vector4: ...
    @property
    def oqos(self) -> U32Vector4: ...
    @property
    def oqot(self) -> U32Vector4: ...
    @property
    def oqoq(self) -> U32Vector4: ...
    @property
    def oqoo(self) -> U32Vector4: ...
    @property
    def oqol(self) -> U32Vector4: ...
    @property
    def oqls(self) -> U32Vector4: ...
    @property
    def oqlt(self) -> U32Vector4: ...
    @property
    def oqlq(self) -> U32Vector4: ...
    @property
    def oqlo(self) -> U32Vector4: ...
    @property
    def oqll(self) -> U32Vector4: ...
    @property
    def ooss(self) -> U32Vector4: ...
    @property
    def oost(self) -> U32Vector4: ...
    @property
    def oosq(self) -> U32Vector4: ...
    @property
    def ooso(self) -> U32Vector4: ...
    @property
    def oosl(self) -> U32Vector4: ...
    @property
    def oots(self) -> U32Vector4: ...
    @property
    def oott(self) -> U32Vector4: ...
    @property
    def ootq(self) -> U32Vector4: ...
    @property
    def ooto(self) -> U32Vector4: ...
    @property
    def ootl(self) -> U32Vector4: ...
    @property
    def ooqs(self) -> U32Vector4: ...
    @property
    def ooqt(self) -> U32Vector4: ...
    @property
    def ooqq(self) -> U32Vector4: ...
    @property
    def ooqo(self) -> U32Vector4: ...
    @property
    def ooql(self) -> U32Vector4: ...
    @property
    def ooos(self) -> U32Vector4: ...
    @property
    def ooot(self) -> U32Vector4: ...
    @property
    def oooq(self) -> U32Vector4: ...
    @property
    def ools(self) -> U32Vector4: ...
    @property
    def oolt(self) -> U32Vector4: ...
    @property
    def oolq(self) -> U32Vector4: ...
    @property
    def olss(self) -> U32Vector4: ...
    @property
    def olst(self) -> U32Vector4: ...
    @property
    def olsq(self) -> U32Vector4: ...
    @property
    def olso(self) -> U32Vector4: ...
    @property
    def olsl(self) -> U32Vector4: ...
    @property
    def olts(self) -> U32Vector4: ...
    @property
    def oltt(self) -> U32Vector4: ...
    @property
    def oltq(self) -> U32Vector4: ...
    @property
    def olto(self) -> U32Vector4: ...
    @property
    def oltl(self) -> U32Vector4: ...
    @property
    def olqs(self) -> U32Vector4: ...
    @property
    def olqt(self) -> U32Vector4: ...
    @property
    def olqq(self) -> U32Vector4: ...
    @property
    def olqo(self) -> U32Vector4: ...
    @property
    def olql(self) -> U32Vector4: ...
    @property
    def olos(self) -> U32Vector4: ...
    @property
    def olot(self) -> U32Vector4: ...
    @property
    def oloq(self) -> U32Vector4: ...
    @property
    def olls(self) -> U32Vector4: ...
    @property
    def ollt(self) -> U32Vector4: ...
    @property
    def ollq(self) -> U32Vector4: ...
    @property
    def lsss(self) -> U32Vector4: ...
    @property
    def lsst(self) -> U32Vector4: ...
    @property
    def lssq(self) -> U32Vector4: ...
    @property
    def lsso(self) -> U32Vector4: ...
    @property
    def lssl(self) -> U32Vector4: ...
    @property
    def lsts(self) -> U32Vector4: ...
    @property
    def lstt(self) -> U32Vector4: ...
    @property
    def lstq(self) -> U32Vector4: ...
    @property
    def lsto(self) -> U32Vector4: ...
    @property
    def lstl(self) -> U32Vector4: ...
    @property
    def lsqs(self) -> U32Vector4: ...
    @property
    def lsqt(self) -> U32Vector4: ...
    @property
    def lsqq(self) -> U32Vector4: ...
    @property
    def lsqo(self) -> U32Vector4: ...
    @property
    def lsql(self) -> U32Vector4: ...
    @property
    def lsos(self) -> U32Vector4: ...
    @property
    def lsot(self) -> U32Vector4: ...
    @property
    def lsoq(self) -> U32Vector4: ...
    @property
    def lsoo(self) -> U32Vector4: ...
    @property
    def lsol(self) -> U32Vector4: ...
    @property
    def lsls(self) -> U32Vector4: ...
    @property
    def lslt(self) -> U32Vector4: ...
    @property
    def lslq(self) -> U32Vector4: ...
    @property
    def lslo(self) -> U32Vector4: ...
    @property
    def lsll(self) -> U32Vector4: ...
    @property
    def ltss(self) -> U32Vector4: ...
    @property
    def ltst(self) -> U32Vector4: ...
    @property
    def ltsq(self) -> U32Vector4: ...
    @property
    def ltso(self) -> U32Vector4: ...
    @property
    def ltsl(self) -> U32Vector4: ...
    @property
    def ltts(self) -> U32Vector4: ...
    @property
    def lttt(self) -> U32Vector4: ...
    @property
    def lttq(self) -> U32Vector4: ...
    @property
    def ltto(self) -> U32Vector4: ...
    @property
    def lttl(self) -> U32Vector4: ...
    @property
    def ltqs(self) -> U32Vector4: ...
    @property
    def ltqt(self) -> U32Vector4: ...
    @property
    def ltqq(self) -> U32Vector4: ...
    @property
    def ltqo(self) -> U32Vector4: ...
    @property
    def ltql(self) -> U32Vector4: ...
    @property
    def ltos(self) -> U32Vector4: ...
    @property
    def ltot(self) -> U32Vector4: ...
    @property
    def ltoq(self) -> U32Vector4: ...
    @property
    def ltoo(self) -> U32Vector4: ...
    @property
    def ltol(self) -> U32Vector4: ...
    @property
    def ltls(self) -> U32Vector4: ...
    @property
    def ltlt(self) -> U32Vector4: ...
    @property
    def ltlq(self) -> U32Vector4: ...
    @property
    def ltlo(self) -> U32Vector4: ...
    @property
    def ltll(self) -> U32Vector4: ...
    @property
    def lqss(self) -> U32Vector4: ...
    @property
    def lqst(self) -> U32Vector4: ...
    @property
    def lqsq(self) -> U32Vector4: ...
    @property
    def lqso(self) -> U32Vector4: ...
    @property
    def lqsl(self) -> U32Vector4: ...
    @property
    def lqts(self) -> U32Vector4: ...
    @property
    def lqtt(self) -> U32Vector4: ...
    @property
    def lqtq(self) -> U32Vector4: ...
    @property
    def lqto(self) -> U32Vector4: ...
    @property
    def lqtl(self) -> U32Vector4: ...
    @property
    def lqqs(self) -> U32Vector4: ...
    @property
    def lqqt(self) -> U32Vector4: ...
    @property
    def lqqq(self) -> U32Vector4: ...
    @property
    def lqqo(self) -> U32Vector4: ...
    @property
    def lqql(self) -> U32Vector4: ...
    @property
    def lqos(self) -> U32Vector4: ...
    @property
    def lqot(self) -> U32Vector4: ...
    @property
    def lqoq(self) -> U32Vector4: ...
    @property
    def lqoo(self) -> U32Vector4: ...
    @property
    def lqol(self) -> U32Vector4: ...
    @property
    def lqls(self) -> U32Vector4: ...
    @property
    def lqlt(self) -> U32Vector4: ...
    @property
    def lqlq(self) -> U32Vector4: ...
    @property
    def lqlo(self) -> U32Vector4: ...
    @property
    def lqll(self) -> U32Vector4: ...
    @property
    def loss(self) -> U32Vector4: ...
    @property
    def lost(self) -> U32Vector4: ...
    @property
    def losq(self) -> U32Vector4: ...
    @property
    def loso(self) -> U32Vector4: ...
    @property
    def losl(self) -> U32Vector4: ...
    @property
    def lots(self) -> U32Vector4: ...
    @property
    def lott(self) -> U32Vector4: ...
    @property
    def lotq(self) -> U32Vector4: ...
    @property
    def loto(self) -> U32Vector4: ...
    @property
    def lotl(self) -> U32Vector4: ...
    @property
    def loqs(self) -> U32Vector4: ...
    @property
    def loqt(self) -> U32Vector4: ...
    @property
    def loqq(self) -> U32Vector4: ...
    @property
    def loqo(self) -> U32Vector4: ...
    @property
    def loql(self) -> U32Vector4: ...
    @property
    def loos(self) -> U32Vector4: ...
    @property
    def loot(self) -> U32Vector4: ...
    @property
    def looq(self) -> U32Vector4: ...
    @property
    def lols(self) -> U32Vector4: ...
    @property
    def lolt(self) -> U32Vector4: ...
    @property
    def lolq(self) -> U32Vector4: ...
    @property
    def llss(self) -> U32Vector4: ...
    @property
    def llst(self) -> U32Vector4: ...
    @property
    def llsq(self) -> U32Vector4: ...
    @property
    def llso(self) -> U32Vector4: ...
    @property
    def llsl(self) -> U32Vector4: ...
    @property
    def llts(self) -> U32Vector4: ...
    @property
    def lltt(self) -> U32Vector4: ...
    @property
    def lltq(self) -> U32Vector4: ...
    @property
    def llto(self) -> U32Vector4: ...
    @property
    def lltl(self) -> U32Vector4: ...
    @property
    def llqs(self) -> U32Vector4: ...
    @property
    def llqt(self) -> U32Vector4: ...
    @property
    def llqq(self) -> U32Vector4: ...
    @property
    def llqo(self) -> U32Vector4: ...
    @property
    def llql(self) -> U32Vector4: ...
    @property
    def llos(self) -> U32Vector4: ...
    @property
    def llot(self) -> U32Vector4: ...
    @property
    def lloq(self) -> U32Vector4: ...
    @property
    def llls(self) -> U32Vector4: ...
    @property
    def lllt(self) -> U32Vector4: ...
    @property
    def lllq(self) -> U32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U32Vector2: ...
    @property
    def uv(self) -> U32Vector2: ...
    @property
    def uo(self) -> U32Vector2: ...
    @property
    def ul(self) -> U32Vector2: ...
    @property
    def vu(self) -> U32Vector2: ...
    @property
    def vv(self) -> U32Vector2: ...
    @property
    def vo(self) -> U32Vector2: ...
    @property
    def vl(self) -> U32Vector2: ...
    @property
    def ou(self) -> U32Vector2: ...
    @property
    def ov(self) -> U32Vector2: ...
    @property
    def lu(self) -> U32Vector2: ...
    @property
    def lv(self) -> U32Vector2: ...
    @property
    def uuu(self) -> U32Vector3: ...
    @property
    def uuv(self) -> U32Vector3: ...
    @property
    def uuo(self) -> U32Vector3: ...
    @property
    def uul(self) -> U32Vector3: ...
    @property
    def uvu(self) -> U32Vector3: ...
    @property
    def uvv(self) -> U32Vector3: ...
    @property
    def uvo(self) -> U32Vector3: ...
    @property
    def uvl(self) -> U32Vector3: ...
    @property
    def uou(self) -> U32Vector3: ...
    @property
    def uov(self) -> U32Vector3: ...
    @property
    def uoo(self) -> U32Vector3: ...
    @property
    def uol(self) -> U32Vector3: ...
    @property
    def ulu(self) -> U32Vector3: ...
    @property
    def ulv(self) -> U32Vector3: ...
    @property
    def ulo(self) -> U32Vector3: ...
    @property
    def ull(self) -> U32Vector3: ...
    @property
    def vuu(self) -> U32Vector3: ...
    @property
    def vuv(self) -> U32Vector3: ...
    @property
    def vuo(self) -> U32Vector3: ...
    @property
    def vul(self) -> U32Vector3: ...
    @property
    def vvu(self) -> U32Vector3: ...
    @property
    def vvv(self) -> U32Vector3: ...
    @property
    def vvo(self) -> U32Vector3: ...
    @property
    def vvl(self) -> U32Vector3: ...
    @property
    def vou(self) -> U32Vector3: ...
    @property
    def vov(self) -> U32Vector3: ...
    @property
    def voo(self) -> U32Vector3: ...
    @property
    def vol(self) -> U32Vector3: ...
    @property
    def vlu(self) -> U32Vector3: ...
    @property
    def vlv(self) -> U32Vector3: ...
    @property
    def vlo(self) -> U32Vector3: ...
    @property
    def vll(self) -> U32Vector3: ...
    @property
    def ouu(self) -> U32Vector3: ...
    @property
    def ouv(self) -> U32Vector3: ...
    @property
    def ouo(self) -> U32Vector3: ...
    @property
    def oul(self) -> U32Vector3: ...
    @property
    def ovu(self) -> U32Vector3: ...
    @property
    def ovv(self) -> U32Vector3: ...
    @property
    def ovo(self) -> U32Vector3: ...
    @property
    def ovl(self) -> U32Vector3: ...
    @property
    def oou(self) -> U32Vector3: ...
    @property
    def oov(self) -> U32Vector3: ...
    @property
    def olu(self) -> U32Vector3: ...
    @property
    def olv(self) -> U32Vector3: ...
    @property
    def luu(self) -> U32Vector3: ...
    @property
    def luv(self) -> U32Vector3: ...
    @property
    def luo(self) -> U32Vector3: ...
    @property
    def lul(self) -> U32Vector3: ...
    @property
    def lvu(self) -> U32Vector3: ...
    @property
    def lvv(self) -> U32Vector3: ...
    @property
    def lvo(self) -> U32Vector3: ...
    @property
    def lvl(self) -> U32Vector3: ...
    @property
    def lou(self) -> U32Vector3: ...
    @property
    def lov(self) -> U32Vector3: ...
    @property
    def llu(self) -> U32Vector3: ...
    @property
    def llv(self) -> U32Vector3: ...
    @property
    def uuuu(self) -> U32Vector4: ...
    @property
    def uuuv(self) -> U32Vector4: ...
    @property
    def uuuo(self) -> U32Vector4: ...
    @property
    def uuul(self) -> U32Vector4: ...
    @property
    def uuvu(self) -> U32Vector4: ...
    @property
    def uuvv(self) -> U32Vector4: ...
    @property
    def uuvo(self) -> U32Vector4: ...
    @property
    def uuvl(self) -> U32Vector4: ...
    @property
    def uuou(self) -> U32Vector4: ...
    @property
    def uuov(self) -> U32Vector4: ...
    @property
    def uuoo(self) -> U32Vector4: ...
    @property
    def uuol(self) -> U32Vector4: ...
    @property
    def uulu(self) -> U32Vector4: ...
    @property
    def uulv(self) -> U32Vector4: ...
    @property
    def uulo(self) -> U32Vector4: ...
    @property
    def uull(self) -> U32Vector4: ...
    @property
    def uvuu(self) -> U32Vector4: ...
    @property
    def uvuv(self) -> U32Vector4: ...
    @property
    def uvuo(self) -> U32Vector4: ...
    @property
    def uvul(self) -> U32Vector4: ...
    @property
    def uvvu(self) -> U32Vector4: ...
    @property
    def uvvv(self) -> U32Vector4: ...
    @property
    def uvvo(self) -> U32Vector4: ...
    @property
    def uvvl(self) -> U32Vector4: ...
    @property
    def uvou(self) -> U32Vector4: ...
    @property
    def uvov(self) -> U32Vector4: ...
    @property
    def uvoo(self) -> U32Vector4: ...
    @property
    def uvol(self) -> U32Vector4: ...
    @property
    def uvlu(self) -> U32Vector4: ...
    @property
    def uvlv(self) -> U32Vector4: ...
    @property
    def uvlo(self) -> U32Vector4: ...
    @property
    def uvll(self) -> U32Vector4: ...
    @property
    def uouu(self) -> U32Vector4: ...
    @property
    def uouv(self) -> U32Vector4: ...
    @property
    def uouo(self) -> U32Vector4: ...
    @property
    def uoul(self) -> U32Vector4: ...
    @property
    def uovu(self) -> U32Vector4: ...
    @property
    def uovv(self) -> U32Vector4: ...
    @property
    def uovo(self) -> U32Vector4: ...
    @property
    def uovl(self) -> U32Vector4: ...
    @property
    def uoou(self) -> U32Vector4: ...
    @property
    def uoov(self) -> U32Vector4: ...
    @property
    def uooo(self) -> U32Vector4: ...
    @property
    def uool(self) -> U32Vector4: ...
    @property
    def uolu(self) -> U32Vector4: ...
    @property
    def uolv(self) -> U32Vector4: ...
    @property
    def uolo(self) -> U32Vector4: ...
    @property
    def uoll(self) -> U32Vector4: ...
    @property
    def uluu(self) -> U32Vector4: ...
    @property
    def uluv(self) -> U32Vector4: ...
    @property
    def uluo(self) -> U32Vector4: ...
    @property
    def ulul(self) -> U32Vector4: ...
    @property
    def ulvu(self) -> U32Vector4: ...
    @property
    def ulvv(self) -> U32Vector4: ...
    @property
    def ulvo(self) -> U32Vector4: ...
    @property
    def ulvl(self) -> U32Vector4: ...
    @property
    def ulou(self) -> U32Vector4: ...
    @property
    def ulov(self) -> U32Vector4: ...
    @property
    def uloo(self) -> U32Vector4: ...
    @property
    def ulol(self) -> U32Vector4: ...
    @property
    def ullu(self) -> U32Vector4: ...
    @property
    def ullv(self) -> U32Vector4: ...
    @property
    def ullo(self) -> U32Vector4: ...
    @property
    def ulll(self) -> U32Vector4: ...
    @property
    def vuuu(self) -> U32Vector4: ...
    @property
    def vuuv(self) -> U32Vector4: ...
    @property
    def vuuo(self) -> U32Vector4: ...
    @property
    def vuul(self) -> U32Vector4: ...
    @property
    def vuvu(self) -> U32Vector4: ...
    @property
    def vuvv(self) -> U32Vector4: ...
    @property
    def vuvo(self) -> U32Vector4: ...
    @property
    def vuvl(self) -> U32Vector4: ...
    @property
    def vuou(self) -> U32Vector4: ...
    @property
    def vuov(self) -> U32Vector4: ...
    @property
    def vuoo(self) -> U32Vector4: ...
    @property
    def vuol(self) -> U32Vector4: ...
    @property
    def vulu(self) -> U32Vector4: ...
    @property
    def vulv(self) -> U32Vector4: ...
    @property
    def vulo(self) -> U32Vector4: ...
    @property
    def vull(self) -> U32Vector4: ...
    @property
    def vvuu(self) -> U32Vector4: ...
    @property
    def vvuv(self) -> U32Vector4: ...
    @property
    def vvuo(self) -> U32Vector4: ...
    @property
    def vvul(self) -> U32Vector4: ...
    @property
    def vvvu(self) -> U32Vector4: ...
    @property
    def vvvv(self) -> U32Vector4: ...
    @property
    def vvvo(self) -> U32Vector4: ...
    @property
    def vvvl(self) -> U32Vector4: ...
    @property
    def vvou(self) -> U32Vector4: ...
    @property
    def vvov(self) -> U32Vector4: ...
    @property
    def vvoo(self) -> U32Vector4: ...
    @property
    def vvol(self) -> U32Vector4: ...
    @property
    def vvlu(self) -> U32Vector4: ...
    @property
    def vvlv(self) -> U32Vector4: ...
    @property
    def vvlo(self) -> U32Vector4: ...
    @property
    def vvll(self) -> U32Vector4: ...
    @property
    def vouu(self) -> U32Vector4: ...
    @property
    def vouv(self) -> U32Vector4: ...
    @property
    def vouo(self) -> U32Vector4: ...
    @property
    def voul(self) -> U32Vector4: ...
    @property
    def vovu(self) -> U32Vector4: ...
    @property
    def vovv(self) -> U32Vector4: ...
    @property
    def vovo(self) -> U32Vector4: ...
    @property
    def vovl(self) -> U32Vector4: ...
    @property
    def voou(self) -> U32Vector4: ...
    @property
    def voov(self) -> U32Vector4: ...
    @property
    def vooo(self) -> U32Vector4: ...
    @property
    def vool(self) -> U32Vector4: ...
    @property
    def volu(self) -> U32Vector4: ...
    @property
    def volv(self) -> U32Vector4: ...
    @property
    def volo(self) -> U32Vector4: ...
    @property
    def voll(self) -> U32Vector4: ...
    @property
    def vluu(self) -> U32Vector4: ...
    @property
    def vluv(self) -> U32Vector4: ...
    @property
    def vluo(self) -> U32Vector4: ...
    @property
    def vlul(self) -> U32Vector4: ...
    @property
    def vlvu(self) -> U32Vector4: ...
    @property
    def vlvv(self) -> U32Vector4: ...
    @property
    def vlvo(self) -> U32Vector4: ...
    @property
    def vlvl(self) -> U32Vector4: ...
    @property
    def vlou(self) -> U32Vector4: ...
    @property
    def vlov(self) -> U32Vector4: ...
    @property
    def vloo(self) -> U32Vector4: ...
    @property
    def vlol(self) -> U32Vector4: ...
    @property
    def vllu(self) -> U32Vector4: ...
    @property
    def vllv(self) -> U32Vector4: ...
    @property
    def vllo(self) -> U32Vector4: ...
    @property
    def vlll(self) -> U32Vector4: ...
    @property
    def ouuu(self) -> U32Vector4: ...
    @property
    def ouuv(self) -> U32Vector4: ...
    @property
    def ouuo(self) -> U32Vector4: ...
    @property
    def ouul(self) -> U32Vector4: ...
    @property
    def ouvu(self) -> U32Vector4: ...
    @property
    def ouvv(self) -> U32Vector4: ...
    @property
    def ouvo(self) -> U32Vector4: ...
    @property
    def ouvl(self) -> U32Vector4: ...
    @property
    def ouou(self) -> U32Vector4: ...
    @property
    def ouov(self) -> U32Vector4: ...
    @property
    def ouoo(self) -> U32Vector4: ...
    @property
    def ouol(self) -> U32Vector4: ...
    @property
    def oulu(self) -> U32Vector4: ...
    @property
    def oulv(self) -> U32Vector4: ...
    @property
    def oulo(self) -> U32Vector4: ...
    @property
    def oull(self) -> U32Vector4: ...
    @property
    def ovuu(self) -> U32Vector4: ...
    @property
    def ovuv(self) -> U32Vector4: ...
    @property
    def ovuo(self) -> U32Vector4: ...
    @property
    def ovul(self) -> U32Vector4: ...
    @property
    def ovvu(self) -> U32Vector4: ...
    @property
    def ovvv(self) -> U32Vector4: ...
    @property
    def ovvo(self) -> U32Vector4: ...
    @property
    def ovvl(self) -> U32Vector4: ...
    @property
    def ovou(self) -> U32Vector4: ...
    @property
    def ovov(self) -> U32Vector4: ...
    @property
    def ovoo(self) -> U32Vector4: ...
    @property
    def ovol(self) -> U32Vector4: ...
    @property
    def ovlu(self) -> U32Vector4: ...
    @property
    def ovlv(self) -> U32Vector4: ...
    @property
    def ovlo(self) -> U32Vector4: ...
    @property
    def ovll(self) -> U32Vector4: ...
    @property
    def oouu(self) -> U32Vector4: ...
    @property
    def oouv(self) -> U32Vector4: ...
    @property
    def oouo(self) -> U32Vector4: ...
    @property
    def ooul(self) -> U32Vector4: ...
    @property
    def oovu(self) -> U32Vector4: ...
    @property
    def oovv(self) -> U32Vector4: ...
    @property
    def oovo(self) -> U32Vector4: ...
    @property
    def oovl(self) -> U32Vector4: ...
    @property
    def ooou(self) -> U32Vector4: ...
    @property
    def ooov(self) -> U32Vector4: ...
    @property
    def oolu(self) -> U32Vector4: ...
    @property
    def oolv(self) -> U32Vector4: ...
    @property
    def oluu(self) -> U32Vector4: ...
    @property
    def oluv(self) -> U32Vector4: ...
    @property
    def oluo(self) -> U32Vector4: ...
    @property
    def olul(self) -> U32Vector4: ...
    @property
    def olvu(self) -> U32Vector4: ...
    @property
    def olvv(self) -> U32Vector4: ...
    @property
    def olvo(self) -> U32Vector4: ...
    @property
    def olvl(self) -> U32Vector4: ...
    @property
    def olou(self) -> U32Vector4: ...
    @property
    def olov(self) -> U32Vector4: ...
    @property
    def ollu(self) -> U32Vector4: ...
    @property
    def ollv(self) -> U32Vector4: ...
    @property
    def luuu(self) -> U32Vector4: ...
    @property
    def luuv(self) -> U32Vector4: ...
    @property
    def luuo(self) -> U32Vector4: ...
    @property
    def luul(self) -> U32Vector4: ...
    @property
    def luvu(self) -> U32Vector4: ...
    @property
    def luvv(self) -> U32Vector4: ...
    @property
    def luvo(self) -> U32Vector4: ...
    @property
    def luvl(self) -> U32Vector4: ...
    @property
    def luou(self) -> U32Vector4: ...
    @property
    def luov(self) -> U32Vector4: ...
    @property
    def luoo(self) -> U32Vector4: ...
    @property
    def luol(self) -> U32Vector4: ...
    @property
    def lulu(self) -> U32Vector4: ...
    @property
    def lulv(self) -> U32Vector4: ...
    @property
    def lulo(self) -> U32Vector4: ...
    @property
    def lull(self) -> U32Vector4: ...
    @property
    def lvuu(self) -> U32Vector4: ...
    @property
    def lvuv(self) -> U32Vector4: ...
    @property
    def lvuo(self) -> U32Vector4: ...
    @property
    def lvul(self) -> U32Vector4: ...
    @property
    def lvvu(self) -> U32Vector4: ...
    @property
    def lvvv(self) -> U32Vector4: ...
    @property
    def lvvo(self) -> U32Vector4: ...
    @property
    def lvvl(self) -> U32Vector4: ...
    @property
    def lvou(self) -> U32Vector4: ...
    @property
    def lvov(self) -> U32Vector4: ...
    @property
    def lvoo(self) -> U32Vector4: ...
    @property
    def lvol(self) -> U32Vector4: ...
    @property
    def lvlu(self) -> U32Vector4: ...
    @property
    def lvlv(self) -> U32Vector4: ...
    @property
    def lvlo(self) -> U32Vector4: ...
    @property
    def lvll(self) -> U32Vector4: ...
    @property
    def louu(self) -> U32Vector4: ...
    @property
    def louv(self) -> U32Vector4: ...
    @property
    def louo(self) -> U32Vector4: ...
    @property
    def loul(self) -> U32Vector4: ...
    @property
    def lovu(self) -> U32Vector4: ...
    @property
    def lovv(self) -> U32Vector4: ...
    @property
    def lovo(self) -> U32Vector4: ...
    @property
    def lovl(self) -> U32Vector4: ...
    @property
    def loou(self) -> U32Vector4: ...
    @property
    def loov(self) -> U32Vector4: ...
    @property
    def lolu(self) -> U32Vector4: ...
    @property
    def lolv(self) -> U32Vector4: ...
    @property
    def lluu(self) -> U32Vector4: ...
    @property
    def lluv(self) -> U32Vector4: ...
    @property
    def lluo(self) -> U32Vector4: ...
    @property
    def llul(self) -> U32Vector4: ...
    @property
    def llvu(self) -> U32Vector4: ...
    @property
    def llvv(self) -> U32Vector4: ...
    @property
    def llvo(self) -> U32Vector4: ...
    @property
    def llvl(self) -> U32Vector4: ...
    @property
    def llou(self) -> U32Vector4: ...
    @property
    def llov(self) -> U32Vector4: ...
    @property
    def lllu(self) -> U32Vector4: ...
    @property
    def lllv(self) -> U32Vector4: ...
    @property
    def oo(self) -> U32Vector2: ...
    @property
    def ol(self) -> U32Vector2: ...
    @property
    def lo(self) -> U32Vector2: ...
    @property
    def ll(self) -> U32Vector2: ...
    @property
    def ooo(self) -> U32Vector3: ...
    @property
    def ool(self) -> U32Vector3: ...
    @property
    def olo(self) -> U32Vector3: ...
    @property
    def oll(self) -> U32Vector3: ...
    @property
    def loo(self) -> U32Vector3: ...
    @property
    def lol(self) -> U32Vector3: ...
    @property
    def llo(self) -> U32Vector3: ...
    @property
    def lll(self) -> U32Vector3: ...
    @property
    def oooo(self) -> U32Vector4: ...
    @property
    def oool(self) -> U32Vector4: ...
    @property
    def oolo(self) -> U32Vector4: ...
    @property
    def ooll(self) -> U32Vector4: ...
    @property
    def oloo(self) -> U32Vector4: ...
    @property
    def olol(self) -> U32Vector4: ...
    @property
    def ollo(self) -> U32Vector4: ...
    @property
    def olll(self) -> U32Vector4: ...
    @property
    def looo(self) -> U32Vector4: ...
    @property
    def lool(self) -> U32Vector4: ...
    @property
    def lolo(self) -> U32Vector4: ...
    @property
    def loll(self) -> U32Vector4: ...
    @property
    def lloo(self) -> U32Vector4: ...
    @property
    def llol(self) -> U32Vector4: ...
    @property
    def lllo(self) -> U32Vector4: ...
    @property
    def llll(self) -> U32Vector4: ...
    def min(self, n: Number, /) -> U32Vector3: ...
    def max(self, n: Number, /) -> U32Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> U32Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U32Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector3: ...

@final
class U32Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U32Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U32Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> U32Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> U32Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[U32Vector3]: ...

@final
class IVector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: IVector3) -> IVector3: ...
    @overload
    def __add__(self, other: Number) -> IVector3: ...
    @overload
    def __radd__(self, other: IVector3) -> IVector3: ...
    @overload
    def __radd__(self, other: Number) -> IVector3: ...
    @overload
    def __sub__(self, other: IVector3) -> IVector3: ...
    @overload
    def __sub__(self, other: Number) -> IVector3: ...
    @overload
    def __rsub__(self, other: IVector3) -> IVector3: ...
    @overload
    def __rsub__(self, other: Number) -> IVector3: ...
    @overload
    def __mul__(self, other: IVector3) -> IVector3: ...
    @overload
    def __mul__(self, other: Number) -> IVector3: ...
    @overload
    def __rmul__(self, other: IVector3) -> IVector3: ...
    @overload
    def __rmul__(self, other: Number) -> IVector3: ...
    @overload
    def __truediv__(self, other: IVector3) -> IVector3: ...
    @overload
    def __truediv__(self, other: Number) -> IVector3: ...
    @overload
    def __rtruediv__(self, other: IVector3) -> IVector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> IVector3: ...
    def __neg__(self) -> IVector3: ...
    def __abs__(self) -> IVector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> IVector2: ...
    @property
    def xy(self) -> IVector2: ...
    @property
    def xz(self) -> IVector2: ...
    @property
    def xo(self) -> IVector2: ...
    @property
    def xl(self) -> IVector2: ...
    @property
    def yx(self) -> IVector2: ...
    @property
    def yy(self) -> IVector2: ...
    @property
    def yz(self) -> IVector2: ...
    @property
    def yo(self) -> IVector2: ...
    @property
    def yl(self) -> IVector2: ...
    @property
    def zx(self) -> IVector2: ...
    @property
    def zy(self) -> IVector2: ...
    @property
    def zz(self) -> IVector2: ...
    @property
    def zo(self) -> IVector2: ...
    @property
    def zl(self) -> IVector2: ...
    @property
    def ox(self) -> IVector2: ...
    @property
    def oy(self) -> IVector2: ...
    @property
    def oz(self) -> IVector2: ...
    @property
    def lx(self) -> IVector2: ...
    @property
    def ly(self) -> IVector2: ...
    @property
    def lz(self) -> IVector2: ...
    @property
    def xxx(self) -> IVector3: ...
    @property
    def xxy(self) -> IVector3: ...
    @property
    def xxz(self) -> IVector3: ...
    @property
    def xxo(self) -> IVector3: ...
    @property
    def xxl(self) -> IVector3: ...
    @property
    def xyx(self) -> IVector3: ...
    @property
    def xyy(self) -> IVector3: ...
    @property
    def xyz(self) -> IVector3: ...
    @property
    def xyo(self) -> IVector3: ...
    @property
    def xyl(self) -> IVector3: ...
    @property
    def xzx(self) -> IVector3: ...
    @property
    def xzy(self) -> IVector3: ...
    @property
    def xzz(self) -> IVector3: ...
    @property
    def xzo(self) -> IVector3: ...
    @property
    def xzl(self) -> IVector3: ...
    @property
    def xox(self) -> IVector3: ...
    @property
    def xoy(self) -> IVector3: ...
    @property
    def xoz(self) -> IVector3: ...
    @property
    def xoo(self) -> IVector3: ...
    @property
    def xol(self) -> IVector3: ...
    @property
    def xlx(self) -> IVector3: ...
    @property
    def xly(self) -> IVector3: ...
    @property
    def xlz(self) -> IVector3: ...
    @property
    def xlo(self) -> IVector3: ...
    @property
    def xll(self) -> IVector3: ...
    @property
    def yxx(self) -> IVector3: ...
    @property
    def yxy(self) -> IVector3: ...
    @property
    def yxz(self) -> IVector3: ...
    @property
    def yxo(self) -> IVector3: ...
    @property
    def yxl(self) -> IVector3: ...
    @property
    def yyx(self) -> IVector3: ...
    @property
    def yyy(self) -> IVector3: ...
    @property
    def yyz(self) -> IVector3: ...
    @property
    def yyo(self) -> IVector3: ...
    @property
    def yyl(self) -> IVector3: ...
    @property
    def yzx(self) -> IVector3: ...
    @property
    def yzy(self) -> IVector3: ...
    @property
    def yzz(self) -> IVector3: ...
    @property
    def yzo(self) -> IVector3: ...
    @property
    def yzl(self) -> IVector3: ...
    @property
    def yox(self) -> IVector3: ...
    @property
    def yoy(self) -> IVector3: ...
    @property
    def yoz(self) -> IVector3: ...
    @property
    def yoo(self) -> IVector3: ...
    @property
    def yol(self) -> IVector3: ...
    @property
    def ylx(self) -> IVector3: ...
    @property
    def yly(self) -> IVector3: ...
    @property
    def ylz(self) -> IVector3: ...
    @property
    def ylo(self) -> IVector3: ...
    @property
    def yll(self) -> IVector3: ...
    @property
    def zxx(self) -> IVector3: ...
    @property
    def zxy(self) -> IVector3: ...
    @property
    def zxz(self) -> IVector3: ...
    @property
    def zxo(self) -> IVector3: ...
    @property
    def zxl(self) -> IVector3: ...
    @property
    def zyx(self) -> IVector3: ...
    @property
    def zyy(self) -> IVector3: ...
    @property
    def zyz(self) -> IVector3: ...
    @property
    def zyo(self) -> IVector3: ...
    @property
    def zyl(self) -> IVector3: ...
    @property
    def zzx(self) -> IVector3: ...
    @property
    def zzy(self) -> IVector3: ...
    @property
    def zzz(self) -> IVector3: ...
    @property
    def zzo(self) -> IVector3: ...
    @property
    def zzl(self) -> IVector3: ...
    @property
    def zox(self) -> IVector3: ...
    @property
    def zoy(self) -> IVector3: ...
    @property
    def zoz(self) -> IVector3: ...
    @property
    def zoo(self) -> IVector3: ...
    @property
    def zol(self) -> IVector3: ...
    @property
    def zlx(self) -> IVector3: ...
    @property
    def zly(self) -> IVector3: ...
    @property
    def zlz(self) -> IVector3: ...
    @property
    def zlo(self) -> IVector3: ...
    @property
    def zll(self) -> IVector3: ...
    @property
    def oxx(self) -> IVector3: ...
    @property
    def oxy(self) -> IVector3: ...
    @property
    def oxz(self) -> IVector3: ...
    @property
    def oxo(self) -> IVector3: ...
    @property
    def oxl(self) -> IVector3: ...
    @property
    def oyx(self) -> IVector3: ...
    @property
    def oyy(self) -> IVector3: ...
    @property
    def oyz(self) -> IVector3: ...
    @property
    def oyo(self) -> IVector3: ...
    @property
    def oyl(self) -> IVector3: ...
    @property
    def ozx(self) -> IVector3: ...
    @property
    def ozy(self) -> IVector3: ...
    @property
    def ozz(self) -> IVector3: ...
    @property
    def ozo(self) -> IVector3: ...
    @property
    def ozl(self) -> IVector3: ...
    @property
    def oox(self) -> IVector3: ...
    @property
    def ooy(self) -> IVector3: ...
    @property
    def ooz(self) -> IVector3: ...
    @property
    def olx(self) -> IVector3: ...
    @property
    def oly(self) -> IVector3: ...
    @property
    def olz(self) -> IVector3: ...
    @property
    def lxx(self) -> IVector3: ...
    @property
    def lxy(self) -> IVector3: ...
    @property
    def lxz(self) -> IVector3: ...
    @property
    def lxo(self) -> IVector3: ...
    @property
    def lxl(self) -> IVector3: ...
    @property
    def lyx(self) -> IVector3: ...
    @property
    def lyy(self) -> IVector3: ...
    @property
    def lyz(self) -> IVector3: ...
    @property
    def lyo(self) -> IVector3: ...
    @property
    def lyl(self) -> IVector3: ...
    @property
    def lzx(self) -> IVector3: ...
    @property
    def lzy(self) -> IVector3: ...
    @property
    def lzz(self) -> IVector3: ...
    @property
    def lzo(self) -> IVector3: ...
    @property
    def lzl(self) -> IVector3: ...
    @property
    def lox(self) -> IVector3: ...
    @property
    def loy(self) -> IVector3: ...
    @property
    def loz(self) -> IVector3: ...
    @property
    def llx(self) -> IVector3: ...
    @property
    def lly(self) -> IVector3: ...
    @property
    def llz(self) -> IVector3: ...
    @property
    def xxxx(self) -> IVector4: ...
    @property
    def xxxy(self) -> IVector4: ...
    @property
    def xxxz(self) -> IVector4: ...
    @property
    def xxxo(self) -> IVector4: ...
    @property
    def xxxl(self) -> IVector4: ...
    @property
    def xxyx(self) -> IVector4: ...
    @property
    def xxyy(self) -> IVector4: ...
    @property
    def xxyz(self) -> IVector4: ...
    @property
    def xxyo(self) -> IVector4: ...
    @property
    def xxyl(self) -> IVector4: ...
    @property
    def xxzx(self) -> IVector4: ...
    @property
    def xxzy(self) -> IVector4: ...
    @property
    def xxzz(self) -> IVector4: ...
    @property
    def xxzo(self) -> IVector4: ...
    @property
    def xxzl(self) -> IVector4: ...
    @property
    def xxox(self) -> IVector4: ...
    @property
    def xxoy(self) -> IVector4: ...
    @property
    def xxoz(self) -> IVector4: ...
    @property
    def xxoo(self) -> IVector4: ...
    @property
    def xxol(self) -> IVector4: ...
    @property
    def xxlx(self) -> IVector4: ...
    @property
    def xxly(self) -> IVector4: ...
    @property
    def xxlz(self) -> IVector4: ...
    @property
    def xxlo(self) -> IVector4: ...
    @property
    def xxll(self) -> IVector4: ...
    @property
    def xyxx(self) -> IVector4: ...
    @property
    def xyxy(self) -> IVector4: ...
    @property
    def xyxz(self) -> IVector4: ...
    @property
    def xyxo(self) -> IVector4: ...
    @property
    def xyxl(self) -> IVector4: ...
    @property
    def xyyx(self) -> IVector4: ...
    @property
    def xyyy(self) -> IVector4: ...
    @property
    def xyyz(self) -> IVector4: ...
    @property
    def xyyo(self) -> IVector4: ...
    @property
    def xyyl(self) -> IVector4: ...
    @property
    def xyzx(self) -> IVector4: ...
    @property
    def xyzy(self) -> IVector4: ...
    @property
    def xyzz(self) -> IVector4: ...
    @property
    def xyzo(self) -> IVector4: ...
    @property
    def xyzl(self) -> IVector4: ...
    @property
    def xyox(self) -> IVector4: ...
    @property
    def xyoy(self) -> IVector4: ...
    @property
    def xyoz(self) -> IVector4: ...
    @property
    def xyoo(self) -> IVector4: ...
    @property
    def xyol(self) -> IVector4: ...
    @property
    def xylx(self) -> IVector4: ...
    @property
    def xyly(self) -> IVector4: ...
    @property
    def xylz(self) -> IVector4: ...
    @property
    def xylo(self) -> IVector4: ...
    @property
    def xyll(self) -> IVector4: ...
    @property
    def xzxx(self) -> IVector4: ...
    @property
    def xzxy(self) -> IVector4: ...
    @property
    def xzxz(self) -> IVector4: ...
    @property
    def xzxo(self) -> IVector4: ...
    @property
    def xzxl(self) -> IVector4: ...
    @property
    def xzyx(self) -> IVector4: ...
    @property
    def xzyy(self) -> IVector4: ...
    @property
    def xzyz(self) -> IVector4: ...
    @property
    def xzyo(self) -> IVector4: ...
    @property
    def xzyl(self) -> IVector4: ...
    @property
    def xzzx(self) -> IVector4: ...
    @property
    def xzzy(self) -> IVector4: ...
    @property
    def xzzz(self) -> IVector4: ...
    @property
    def xzzo(self) -> IVector4: ...
    @property
    def xzzl(self) -> IVector4: ...
    @property
    def xzox(self) -> IVector4: ...
    @property
    def xzoy(self) -> IVector4: ...
    @property
    def xzoz(self) -> IVector4: ...
    @property
    def xzoo(self) -> IVector4: ...
    @property
    def xzol(self) -> IVector4: ...
    @property
    def xzlx(self) -> IVector4: ...
    @property
    def xzly(self) -> IVector4: ...
    @property
    def xzlz(self) -> IVector4: ...
    @property
    def xzlo(self) -> IVector4: ...
    @property
    def xzll(self) -> IVector4: ...
    @property
    def xoxx(self) -> IVector4: ...
    @property
    def xoxy(self) -> IVector4: ...
    @property
    def xoxz(self) -> IVector4: ...
    @property
    def xoxo(self) -> IVector4: ...
    @property
    def xoxl(self) -> IVector4: ...
    @property
    def xoyx(self) -> IVector4: ...
    @property
    def xoyy(self) -> IVector4: ...
    @property
    def xoyz(self) -> IVector4: ...
    @property
    def xoyo(self) -> IVector4: ...
    @property
    def xoyl(self) -> IVector4: ...
    @property
    def xozx(self) -> IVector4: ...
    @property
    def xozy(self) -> IVector4: ...
    @property
    def xozz(self) -> IVector4: ...
    @property
    def xozo(self) -> IVector4: ...
    @property
    def xozl(self) -> IVector4: ...
    @property
    def xoox(self) -> IVector4: ...
    @property
    def xooy(self) -> IVector4: ...
    @property
    def xooz(self) -> IVector4: ...
    @property
    def xooo(self) -> IVector4: ...
    @property
    def xool(self) -> IVector4: ...
    @property
    def xolx(self) -> IVector4: ...
    @property
    def xoly(self) -> IVector4: ...
    @property
    def xolz(self) -> IVector4: ...
    @property
    def xolo(self) -> IVector4: ...
    @property
    def xoll(self) -> IVector4: ...
    @property
    def xlxx(self) -> IVector4: ...
    @property
    def xlxy(self) -> IVector4: ...
    @property
    def xlxz(self) -> IVector4: ...
    @property
    def xlxo(self) -> IVector4: ...
    @property
    def xlxl(self) -> IVector4: ...
    @property
    def xlyx(self) -> IVector4: ...
    @property
    def xlyy(self) -> IVector4: ...
    @property
    def xlyz(self) -> IVector4: ...
    @property
    def xlyo(self) -> IVector4: ...
    @property
    def xlyl(self) -> IVector4: ...
    @property
    def xlzx(self) -> IVector4: ...
    @property
    def xlzy(self) -> IVector4: ...
    @property
    def xlzz(self) -> IVector4: ...
    @property
    def xlzo(self) -> IVector4: ...
    @property
    def xlzl(self) -> IVector4: ...
    @property
    def xlox(self) -> IVector4: ...
    @property
    def xloy(self) -> IVector4: ...
    @property
    def xloz(self) -> IVector4: ...
    @property
    def xloo(self) -> IVector4: ...
    @property
    def xlol(self) -> IVector4: ...
    @property
    def xllx(self) -> IVector4: ...
    @property
    def xlly(self) -> IVector4: ...
    @property
    def xllz(self) -> IVector4: ...
    @property
    def xllo(self) -> IVector4: ...
    @property
    def xlll(self) -> IVector4: ...
    @property
    def yxxx(self) -> IVector4: ...
    @property
    def yxxy(self) -> IVector4: ...
    @property
    def yxxz(self) -> IVector4: ...
    @property
    def yxxo(self) -> IVector4: ...
    @property
    def yxxl(self) -> IVector4: ...
    @property
    def yxyx(self) -> IVector4: ...
    @property
    def yxyy(self) -> IVector4: ...
    @property
    def yxyz(self) -> IVector4: ...
    @property
    def yxyo(self) -> IVector4: ...
    @property
    def yxyl(self) -> IVector4: ...
    @property
    def yxzx(self) -> IVector4: ...
    @property
    def yxzy(self) -> IVector4: ...
    @property
    def yxzz(self) -> IVector4: ...
    @property
    def yxzo(self) -> IVector4: ...
    @property
    def yxzl(self) -> IVector4: ...
    @property
    def yxox(self) -> IVector4: ...
    @property
    def yxoy(self) -> IVector4: ...
    @property
    def yxoz(self) -> IVector4: ...
    @property
    def yxoo(self) -> IVector4: ...
    @property
    def yxol(self) -> IVector4: ...
    @property
    def yxlx(self) -> IVector4: ...
    @property
    def yxly(self) -> IVector4: ...
    @property
    def yxlz(self) -> IVector4: ...
    @property
    def yxlo(self) -> IVector4: ...
    @property
    def yxll(self) -> IVector4: ...
    @property
    def yyxx(self) -> IVector4: ...
    @property
    def yyxy(self) -> IVector4: ...
    @property
    def yyxz(self) -> IVector4: ...
    @property
    def yyxo(self) -> IVector4: ...
    @property
    def yyxl(self) -> IVector4: ...
    @property
    def yyyx(self) -> IVector4: ...
    @property
    def yyyy(self) -> IVector4: ...
    @property
    def yyyz(self) -> IVector4: ...
    @property
    def yyyo(self) -> IVector4: ...
    @property
    def yyyl(self) -> IVector4: ...
    @property
    def yyzx(self) -> IVector4: ...
    @property
    def yyzy(self) -> IVector4: ...
    @property
    def yyzz(self) -> IVector4: ...
    @property
    def yyzo(self) -> IVector4: ...
    @property
    def yyzl(self) -> IVector4: ...
    @property
    def yyox(self) -> IVector4: ...
    @property
    def yyoy(self) -> IVector4: ...
    @property
    def yyoz(self) -> IVector4: ...
    @property
    def yyoo(self) -> IVector4: ...
    @property
    def yyol(self) -> IVector4: ...
    @property
    def yylx(self) -> IVector4: ...
    @property
    def yyly(self) -> IVector4: ...
    @property
    def yylz(self) -> IVector4: ...
    @property
    def yylo(self) -> IVector4: ...
    @property
    def yyll(self) -> IVector4: ...
    @property
    def yzxx(self) -> IVector4: ...
    @property
    def yzxy(self) -> IVector4: ...
    @property
    def yzxz(self) -> IVector4: ...
    @property
    def yzxo(self) -> IVector4: ...
    @property
    def yzxl(self) -> IVector4: ...
    @property
    def yzyx(self) -> IVector4: ...
    @property
    def yzyy(self) -> IVector4: ...
    @property
    def yzyz(self) -> IVector4: ...
    @property
    def yzyo(self) -> IVector4: ...
    @property
    def yzyl(self) -> IVector4: ...
    @property
    def yzzx(self) -> IVector4: ...
    @property
    def yzzy(self) -> IVector4: ...
    @property
    def yzzz(self) -> IVector4: ...
    @property
    def yzzo(self) -> IVector4: ...
    @property
    def yzzl(self) -> IVector4: ...
    @property
    def yzox(self) -> IVector4: ...
    @property
    def yzoy(self) -> IVector4: ...
    @property
    def yzoz(self) -> IVector4: ...
    @property
    def yzoo(self) -> IVector4: ...
    @property
    def yzol(self) -> IVector4: ...
    @property
    def yzlx(self) -> IVector4: ...
    @property
    def yzly(self) -> IVector4: ...
    @property
    def yzlz(self) -> IVector4: ...
    @property
    def yzlo(self) -> IVector4: ...
    @property
    def yzll(self) -> IVector4: ...
    @property
    def yoxx(self) -> IVector4: ...
    @property
    def yoxy(self) -> IVector4: ...
    @property
    def yoxz(self) -> IVector4: ...
    @property
    def yoxo(self) -> IVector4: ...
    @property
    def yoxl(self) -> IVector4: ...
    @property
    def yoyx(self) -> IVector4: ...
    @property
    def yoyy(self) -> IVector4: ...
    @property
    def yoyz(self) -> IVector4: ...
    @property
    def yoyo(self) -> IVector4: ...
    @property
    def yoyl(self) -> IVector4: ...
    @property
    def yozx(self) -> IVector4: ...
    @property
    def yozy(self) -> IVector4: ...
    @property
    def yozz(self) -> IVector4: ...
    @property
    def yozo(self) -> IVector4: ...
    @property
    def yozl(self) -> IVector4: ...
    @property
    def yoox(self) -> IVector4: ...
    @property
    def yooy(self) -> IVector4: ...
    @property
    def yooz(self) -> IVector4: ...
    @property
    def yooo(self) -> IVector4: ...
    @property
    def yool(self) -> IVector4: ...
    @property
    def yolx(self) -> IVector4: ...
    @property
    def yoly(self) -> IVector4: ...
    @property
    def yolz(self) -> IVector4: ...
    @property
    def yolo(self) -> IVector4: ...
    @property
    def yoll(self) -> IVector4: ...
    @property
    def ylxx(self) -> IVector4: ...
    @property
    def ylxy(self) -> IVector4: ...
    @property
    def ylxz(self) -> IVector4: ...
    @property
    def ylxo(self) -> IVector4: ...
    @property
    def ylxl(self) -> IVector4: ...
    @property
    def ylyx(self) -> IVector4: ...
    @property
    def ylyy(self) -> IVector4: ...
    @property
    def ylyz(self) -> IVector4: ...
    @property
    def ylyo(self) -> IVector4: ...
    @property
    def ylyl(self) -> IVector4: ...
    @property
    def ylzx(self) -> IVector4: ...
    @property
    def ylzy(self) -> IVector4: ...
    @property
    def ylzz(self) -> IVector4: ...
    @property
    def ylzo(self) -> IVector4: ...
    @property
    def ylzl(self) -> IVector4: ...
    @property
    def ylox(self) -> IVector4: ...
    @property
    def yloy(self) -> IVector4: ...
    @property
    def yloz(self) -> IVector4: ...
    @property
    def yloo(self) -> IVector4: ...
    @property
    def ylol(self) -> IVector4: ...
    @property
    def yllx(self) -> IVector4: ...
    @property
    def ylly(self) -> IVector4: ...
    @property
    def yllz(self) -> IVector4: ...
    @property
    def yllo(self) -> IVector4: ...
    @property
    def ylll(self) -> IVector4: ...
    @property
    def zxxx(self) -> IVector4: ...
    @property
    def zxxy(self) -> IVector4: ...
    @property
    def zxxz(self) -> IVector4: ...
    @property
    def zxxo(self) -> IVector4: ...
    @property
    def zxxl(self) -> IVector4: ...
    @property
    def zxyx(self) -> IVector4: ...
    @property
    def zxyy(self) -> IVector4: ...
    @property
    def zxyz(self) -> IVector4: ...
    @property
    def zxyo(self) -> IVector4: ...
    @property
    def zxyl(self) -> IVector4: ...
    @property
    def zxzx(self) -> IVector4: ...
    @property
    def zxzy(self) -> IVector4: ...
    @property
    def zxzz(self) -> IVector4: ...
    @property
    def zxzo(self) -> IVector4: ...
    @property
    def zxzl(self) -> IVector4: ...
    @property
    def zxox(self) -> IVector4: ...
    @property
    def zxoy(self) -> IVector4: ...
    @property
    def zxoz(self) -> IVector4: ...
    @property
    def zxoo(self) -> IVector4: ...
    @property
    def zxol(self) -> IVector4: ...
    @property
    def zxlx(self) -> IVector4: ...
    @property
    def zxly(self) -> IVector4: ...
    @property
    def zxlz(self) -> IVector4: ...
    @property
    def zxlo(self) -> IVector4: ...
    @property
    def zxll(self) -> IVector4: ...
    @property
    def zyxx(self) -> IVector4: ...
    @property
    def zyxy(self) -> IVector4: ...
    @property
    def zyxz(self) -> IVector4: ...
    @property
    def zyxo(self) -> IVector4: ...
    @property
    def zyxl(self) -> IVector4: ...
    @property
    def zyyx(self) -> IVector4: ...
    @property
    def zyyy(self) -> IVector4: ...
    @property
    def zyyz(self) -> IVector4: ...
    @property
    def zyyo(self) -> IVector4: ...
    @property
    def zyyl(self) -> IVector4: ...
    @property
    def zyzx(self) -> IVector4: ...
    @property
    def zyzy(self) -> IVector4: ...
    @property
    def zyzz(self) -> IVector4: ...
    @property
    def zyzo(self) -> IVector4: ...
    @property
    def zyzl(self) -> IVector4: ...
    @property
    def zyox(self) -> IVector4: ...
    @property
    def zyoy(self) -> IVector4: ...
    @property
    def zyoz(self) -> IVector4: ...
    @property
    def zyoo(self) -> IVector4: ...
    @property
    def zyol(self) -> IVector4: ...
    @property
    def zylx(self) -> IVector4: ...
    @property
    def zyly(self) -> IVector4: ...
    @property
    def zylz(self) -> IVector4: ...
    @property
    def zylo(self) -> IVector4: ...
    @property
    def zyll(self) -> IVector4: ...
    @property
    def zzxx(self) -> IVector4: ...
    @property
    def zzxy(self) -> IVector4: ...
    @property
    def zzxz(self) -> IVector4: ...
    @property
    def zzxo(self) -> IVector4: ...
    @property
    def zzxl(self) -> IVector4: ...
    @property
    def zzyx(self) -> IVector4: ...
    @property
    def zzyy(self) -> IVector4: ...
    @property
    def zzyz(self) -> IVector4: ...
    @property
    def zzyo(self) -> IVector4: ...
    @property
    def zzyl(self) -> IVector4: ...
    @property
    def zzzx(self) -> IVector4: ...
    @property
    def zzzy(self) -> IVector4: ...
    @property
    def zzzz(self) -> IVector4: ...
    @property
    def zzzo(self) -> IVector4: ...
    @property
    def zzzl(self) -> IVector4: ...
    @property
    def zzox(self) -> IVector4: ...
    @property
    def zzoy(self) -> IVector4: ...
    @property
    def zzoz(self) -> IVector4: ...
    @property
    def zzoo(self) -> IVector4: ...
    @property
    def zzol(self) -> IVector4: ...
    @property
    def zzlx(self) -> IVector4: ...
    @property
    def zzly(self) -> IVector4: ...
    @property
    def zzlz(self) -> IVector4: ...
    @property
    def zzlo(self) -> IVector4: ...
    @property
    def zzll(self) -> IVector4: ...
    @property
    def zoxx(self) -> IVector4: ...
    @property
    def zoxy(self) -> IVector4: ...
    @property
    def zoxz(self) -> IVector4: ...
    @property
    def zoxo(self) -> IVector4: ...
    @property
    def zoxl(self) -> IVector4: ...
    @property
    def zoyx(self) -> IVector4: ...
    @property
    def zoyy(self) -> IVector4: ...
    @property
    def zoyz(self) -> IVector4: ...
    @property
    def zoyo(self) -> IVector4: ...
    @property
    def zoyl(self) -> IVector4: ...
    @property
    def zozx(self) -> IVector4: ...
    @property
    def zozy(self) -> IVector4: ...
    @property
    def zozz(self) -> IVector4: ...
    @property
    def zozo(self) -> IVector4: ...
    @property
    def zozl(self) -> IVector4: ...
    @property
    def zoox(self) -> IVector4: ...
    @property
    def zooy(self) -> IVector4: ...
    @property
    def zooz(self) -> IVector4: ...
    @property
    def zooo(self) -> IVector4: ...
    @property
    def zool(self) -> IVector4: ...
    @property
    def zolx(self) -> IVector4: ...
    @property
    def zoly(self) -> IVector4: ...
    @property
    def zolz(self) -> IVector4: ...
    @property
    def zolo(self) -> IVector4: ...
    @property
    def zoll(self) -> IVector4: ...
    @property
    def zlxx(self) -> IVector4: ...
    @property
    def zlxy(self) -> IVector4: ...
    @property
    def zlxz(self) -> IVector4: ...
    @property
    def zlxo(self) -> IVector4: ...
    @property
    def zlxl(self) -> IVector4: ...
    @property
    def zlyx(self) -> IVector4: ...
    @property
    def zlyy(self) -> IVector4: ...
    @property
    def zlyz(self) -> IVector4: ...
    @property
    def zlyo(self) -> IVector4: ...
    @property
    def zlyl(self) -> IVector4: ...
    @property
    def zlzx(self) -> IVector4: ...
    @property
    def zlzy(self) -> IVector4: ...
    @property
    def zlzz(self) -> IVector4: ...
    @property
    def zlzo(self) -> IVector4: ...
    @property
    def zlzl(self) -> IVector4: ...
    @property
    def zlox(self) -> IVector4: ...
    @property
    def zloy(self) -> IVector4: ...
    @property
    def zloz(self) -> IVector4: ...
    @property
    def zloo(self) -> IVector4: ...
    @property
    def zlol(self) -> IVector4: ...
    @property
    def zllx(self) -> IVector4: ...
    @property
    def zlly(self) -> IVector4: ...
    @property
    def zllz(self) -> IVector4: ...
    @property
    def zllo(self) -> IVector4: ...
    @property
    def zlll(self) -> IVector4: ...
    @property
    def oxxx(self) -> IVector4: ...
    @property
    def oxxy(self) -> IVector4: ...
    @property
    def oxxz(self) -> IVector4: ...
    @property
    def oxxo(self) -> IVector4: ...
    @property
    def oxxl(self) -> IVector4: ...
    @property
    def oxyx(self) -> IVector4: ...
    @property
    def oxyy(self) -> IVector4: ...
    @property
    def oxyz(self) -> IVector4: ...
    @property
    def oxyo(self) -> IVector4: ...
    @property
    def oxyl(self) -> IVector4: ...
    @property
    def oxzx(self) -> IVector4: ...
    @property
    def oxzy(self) -> IVector4: ...
    @property
    def oxzz(self) -> IVector4: ...
    @property
    def oxzo(self) -> IVector4: ...
    @property
    def oxzl(self) -> IVector4: ...
    @property
    def oxox(self) -> IVector4: ...
    @property
    def oxoy(self) -> IVector4: ...
    @property
    def oxoz(self) -> IVector4: ...
    @property
    def oxoo(self) -> IVector4: ...
    @property
    def oxol(self) -> IVector4: ...
    @property
    def oxlx(self) -> IVector4: ...
    @property
    def oxly(self) -> IVector4: ...
    @property
    def oxlz(self) -> IVector4: ...
    @property
    def oxlo(self) -> IVector4: ...
    @property
    def oxll(self) -> IVector4: ...
    @property
    def oyxx(self) -> IVector4: ...
    @property
    def oyxy(self) -> IVector4: ...
    @property
    def oyxz(self) -> IVector4: ...
    @property
    def oyxo(self) -> IVector4: ...
    @property
    def oyxl(self) -> IVector4: ...
    @property
    def oyyx(self) -> IVector4: ...
    @property
    def oyyy(self) -> IVector4: ...
    @property
    def oyyz(self) -> IVector4: ...
    @property
    def oyyo(self) -> IVector4: ...
    @property
    def oyyl(self) -> IVector4: ...
    @property
    def oyzx(self) -> IVector4: ...
    @property
    def oyzy(self) -> IVector4: ...
    @property
    def oyzz(self) -> IVector4: ...
    @property
    def oyzo(self) -> IVector4: ...
    @property
    def oyzl(self) -> IVector4: ...
    @property
    def oyox(self) -> IVector4: ...
    @property
    def oyoy(self) -> IVector4: ...
    @property
    def oyoz(self) -> IVector4: ...
    @property
    def oyoo(self) -> IVector4: ...
    @property
    def oyol(self) -> IVector4: ...
    @property
    def oylx(self) -> IVector4: ...
    @property
    def oyly(self) -> IVector4: ...
    @property
    def oylz(self) -> IVector4: ...
    @property
    def oylo(self) -> IVector4: ...
    @property
    def oyll(self) -> IVector4: ...
    @property
    def ozxx(self) -> IVector4: ...
    @property
    def ozxy(self) -> IVector4: ...
    @property
    def ozxz(self) -> IVector4: ...
    @property
    def ozxo(self) -> IVector4: ...
    @property
    def ozxl(self) -> IVector4: ...
    @property
    def ozyx(self) -> IVector4: ...
    @property
    def ozyy(self) -> IVector4: ...
    @property
    def ozyz(self) -> IVector4: ...
    @property
    def ozyo(self) -> IVector4: ...
    @property
    def ozyl(self) -> IVector4: ...
    @property
    def ozzx(self) -> IVector4: ...
    @property
    def ozzy(self) -> IVector4: ...
    @property
    def ozzz(self) -> IVector4: ...
    @property
    def ozzo(self) -> IVector4: ...
    @property
    def ozzl(self) -> IVector4: ...
    @property
    def ozox(self) -> IVector4: ...
    @property
    def ozoy(self) -> IVector4: ...
    @property
    def ozoz(self) -> IVector4: ...
    @property
    def ozoo(self) -> IVector4: ...
    @property
    def ozol(self) -> IVector4: ...
    @property
    def ozlx(self) -> IVector4: ...
    @property
    def ozly(self) -> IVector4: ...
    @property
    def ozlz(self) -> IVector4: ...
    @property
    def ozlo(self) -> IVector4: ...
    @property
    def ozll(self) -> IVector4: ...
    @property
    def ooxx(self) -> IVector4: ...
    @property
    def ooxy(self) -> IVector4: ...
    @property
    def ooxz(self) -> IVector4: ...
    @property
    def ooxo(self) -> IVector4: ...
    @property
    def ooxl(self) -> IVector4: ...
    @property
    def ooyx(self) -> IVector4: ...
    @property
    def ooyy(self) -> IVector4: ...
    @property
    def ooyz(self) -> IVector4: ...
    @property
    def ooyo(self) -> IVector4: ...
    @property
    def ooyl(self) -> IVector4: ...
    @property
    def oozx(self) -> IVector4: ...
    @property
    def oozy(self) -> IVector4: ...
    @property
    def oozz(self) -> IVector4: ...
    @property
    def oozo(self) -> IVector4: ...
    @property
    def oozl(self) -> IVector4: ...
    @property
    def ooox(self) -> IVector4: ...
    @property
    def oooy(self) -> IVector4: ...
    @property
    def oooz(self) -> IVector4: ...
    @property
    def oolx(self) -> IVector4: ...
    @property
    def ooly(self) -> IVector4: ...
    @property
    def oolz(self) -> IVector4: ...
    @property
    def olxx(self) -> IVector4: ...
    @property
    def olxy(self) -> IVector4: ...
    @property
    def olxz(self) -> IVector4: ...
    @property
    def olxo(self) -> IVector4: ...
    @property
    def olxl(self) -> IVector4: ...
    @property
    def olyx(self) -> IVector4: ...
    @property
    def olyy(self) -> IVector4: ...
    @property
    def olyz(self) -> IVector4: ...
    @property
    def olyo(self) -> IVector4: ...
    @property
    def olyl(self) -> IVector4: ...
    @property
    def olzx(self) -> IVector4: ...
    @property
    def olzy(self) -> IVector4: ...
    @property
    def olzz(self) -> IVector4: ...
    @property
    def olzo(self) -> IVector4: ...
    @property
    def olzl(self) -> IVector4: ...
    @property
    def olox(self) -> IVector4: ...
    @property
    def oloy(self) -> IVector4: ...
    @property
    def oloz(self) -> IVector4: ...
    @property
    def ollx(self) -> IVector4: ...
    @property
    def olly(self) -> IVector4: ...
    @property
    def ollz(self) -> IVector4: ...
    @property
    def lxxx(self) -> IVector4: ...
    @property
    def lxxy(self) -> IVector4: ...
    @property
    def lxxz(self) -> IVector4: ...
    @property
    def lxxo(self) -> IVector4: ...
    @property
    def lxxl(self) -> IVector4: ...
    @property
    def lxyx(self) -> IVector4: ...
    @property
    def lxyy(self) -> IVector4: ...
    @property
    def lxyz(self) -> IVector4: ...
    @property
    def lxyo(self) -> IVector4: ...
    @property
    def lxyl(self) -> IVector4: ...
    @property
    def lxzx(self) -> IVector4: ...
    @property
    def lxzy(self) -> IVector4: ...
    @property
    def lxzz(self) -> IVector4: ...
    @property
    def lxzo(self) -> IVector4: ...
    @property
    def lxzl(self) -> IVector4: ...
    @property
    def lxox(self) -> IVector4: ...
    @property
    def lxoy(self) -> IVector4: ...
    @property
    def lxoz(self) -> IVector4: ...
    @property
    def lxoo(self) -> IVector4: ...
    @property
    def lxol(self) -> IVector4: ...
    @property
    def lxlx(self) -> IVector4: ...
    @property
    def lxly(self) -> IVector4: ...
    @property
    def lxlz(self) -> IVector4: ...
    @property
    def lxlo(self) -> IVector4: ...
    @property
    def lxll(self) -> IVector4: ...
    @property
    def lyxx(self) -> IVector4: ...
    @property
    def lyxy(self) -> IVector4: ...
    @property
    def lyxz(self) -> IVector4: ...
    @property
    def lyxo(self) -> IVector4: ...
    @property
    def lyxl(self) -> IVector4: ...
    @property
    def lyyx(self) -> IVector4: ...
    @property
    def lyyy(self) -> IVector4: ...
    @property
    def lyyz(self) -> IVector4: ...
    @property
    def lyyo(self) -> IVector4: ...
    @property
    def lyyl(self) -> IVector4: ...
    @property
    def lyzx(self) -> IVector4: ...
    @property
    def lyzy(self) -> IVector4: ...
    @property
    def lyzz(self) -> IVector4: ...
    @property
    def lyzo(self) -> IVector4: ...
    @property
    def lyzl(self) -> IVector4: ...
    @property
    def lyox(self) -> IVector4: ...
    @property
    def lyoy(self) -> IVector4: ...
    @property
    def lyoz(self) -> IVector4: ...
    @property
    def lyoo(self) -> IVector4: ...
    @property
    def lyol(self) -> IVector4: ...
    @property
    def lylx(self) -> IVector4: ...
    @property
    def lyly(self) -> IVector4: ...
    @property
    def lylz(self) -> IVector4: ...
    @property
    def lylo(self) -> IVector4: ...
    @property
    def lyll(self) -> IVector4: ...
    @property
    def lzxx(self) -> IVector4: ...
    @property
    def lzxy(self) -> IVector4: ...
    @property
    def lzxz(self) -> IVector4: ...
    @property
    def lzxo(self) -> IVector4: ...
    @property
    def lzxl(self) -> IVector4: ...
    @property
    def lzyx(self) -> IVector4: ...
    @property
    def lzyy(self) -> IVector4: ...
    @property
    def lzyz(self) -> IVector4: ...
    @property
    def lzyo(self) -> IVector4: ...
    @property
    def lzyl(self) -> IVector4: ...
    @property
    def lzzx(self) -> IVector4: ...
    @property
    def lzzy(self) -> IVector4: ...
    @property
    def lzzz(self) -> IVector4: ...
    @property
    def lzzo(self) -> IVector4: ...
    @property
    def lzzl(self) -> IVector4: ...
    @property
    def lzox(self) -> IVector4: ...
    @property
    def lzoy(self) -> IVector4: ...
    @property
    def lzoz(self) -> IVector4: ...
    @property
    def lzoo(self) -> IVector4: ...
    @property
    def lzol(self) -> IVector4: ...
    @property
    def lzlx(self) -> IVector4: ...
    @property
    def lzly(self) -> IVector4: ...
    @property
    def lzlz(self) -> IVector4: ...
    @property
    def lzlo(self) -> IVector4: ...
    @property
    def lzll(self) -> IVector4: ...
    @property
    def loxx(self) -> IVector4: ...
    @property
    def loxy(self) -> IVector4: ...
    @property
    def loxz(self) -> IVector4: ...
    @property
    def loxo(self) -> IVector4: ...
    @property
    def loxl(self) -> IVector4: ...
    @property
    def loyx(self) -> IVector4: ...
    @property
    def loyy(self) -> IVector4: ...
    @property
    def loyz(self) -> IVector4: ...
    @property
    def loyo(self) -> IVector4: ...
    @property
    def loyl(self) -> IVector4: ...
    @property
    def lozx(self) -> IVector4: ...
    @property
    def lozy(self) -> IVector4: ...
    @property
    def lozz(self) -> IVector4: ...
    @property
    def lozo(self) -> IVector4: ...
    @property
    def lozl(self) -> IVector4: ...
    @property
    def loox(self) -> IVector4: ...
    @property
    def looy(self) -> IVector4: ...
    @property
    def looz(self) -> IVector4: ...
    @property
    def lolx(self) -> IVector4: ...
    @property
    def loly(self) -> IVector4: ...
    @property
    def lolz(self) -> IVector4: ...
    @property
    def llxx(self) -> IVector4: ...
    @property
    def llxy(self) -> IVector4: ...
    @property
    def llxz(self) -> IVector4: ...
    @property
    def llxo(self) -> IVector4: ...
    @property
    def llxl(self) -> IVector4: ...
    @property
    def llyx(self) -> IVector4: ...
    @property
    def llyy(self) -> IVector4: ...
    @property
    def llyz(self) -> IVector4: ...
    @property
    def llyo(self) -> IVector4: ...
    @property
    def llyl(self) -> IVector4: ...
    @property
    def llzx(self) -> IVector4: ...
    @property
    def llzy(self) -> IVector4: ...
    @property
    def llzz(self) -> IVector4: ...
    @property
    def llzo(self) -> IVector4: ...
    @property
    def llzl(self) -> IVector4: ...
    @property
    def llox(self) -> IVector4: ...
    @property
    def lloy(self) -> IVector4: ...
    @property
    def lloz(self) -> IVector4: ...
    @property
    def lllx(self) -> IVector4: ...
    @property
    def llly(self) -> IVector4: ...
    @property
    def lllz(self) -> IVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> IVector2: ...
    @property
    def rg(self) -> IVector2: ...
    @property
    def rb(self) -> IVector2: ...
    @property
    def ro(self) -> IVector2: ...
    @property
    def rl(self) -> IVector2: ...
    @property
    def gr(self) -> IVector2: ...
    @property
    def gg(self) -> IVector2: ...
    @property
    def gb(self) -> IVector2: ...
    @property
    def go(self) -> IVector2: ...
    @property
    def gl(self) -> IVector2: ...
    @property
    def br(self) -> IVector2: ...
    @property
    def bg(self) -> IVector2: ...
    @property
    def bb(self) -> IVector2: ...
    @property
    def bo(self) -> IVector2: ...
    @property
    def bl(self) -> IVector2: ...
    @property
    def og(self) -> IVector2: ...
    @property
    def ob(self) -> IVector2: ...
    @property
    def lr(self) -> IVector2: ...
    @property
    def lg(self) -> IVector2: ...
    @property
    def lb(self) -> IVector2: ...
    @property
    def rrr(self) -> IVector3: ...
    @property
    def rrg(self) -> IVector3: ...
    @property
    def rrb(self) -> IVector3: ...
    @property
    def rro(self) -> IVector3: ...
    @property
    def rrl(self) -> IVector3: ...
    @property
    def rgr(self) -> IVector3: ...
    @property
    def rgg(self) -> IVector3: ...
    @property
    def rgb(self) -> IVector3: ...
    @property
    def rgo(self) -> IVector3: ...
    @property
    def rgl(self) -> IVector3: ...
    @property
    def rbr(self) -> IVector3: ...
    @property
    def rbg(self) -> IVector3: ...
    @property
    def rbb(self) -> IVector3: ...
    @property
    def rbo(self) -> IVector3: ...
    @property
    def rbl(self) -> IVector3: ...
    @property
    def ror(self) -> IVector3: ...
    @property
    def rog(self) -> IVector3: ...
    @property
    def rob(self) -> IVector3: ...
    @property
    def roo(self) -> IVector3: ...
    @property
    def rol(self) -> IVector3: ...
    @property
    def rlr(self) -> IVector3: ...
    @property
    def rlg(self) -> IVector3: ...
    @property
    def rlb(self) -> IVector3: ...
    @property
    def rlo(self) -> IVector3: ...
    @property
    def rll(self) -> IVector3: ...
    @property
    def grr(self) -> IVector3: ...
    @property
    def grg(self) -> IVector3: ...
    @property
    def grb(self) -> IVector3: ...
    @property
    def gro(self) -> IVector3: ...
    @property
    def grl(self) -> IVector3: ...
    @property
    def ggr(self) -> IVector3: ...
    @property
    def ggg(self) -> IVector3: ...
    @property
    def ggb(self) -> IVector3: ...
    @property
    def ggo(self) -> IVector3: ...
    @property
    def ggl(self) -> IVector3: ...
    @property
    def gbr(self) -> IVector3: ...
    @property
    def gbg(self) -> IVector3: ...
    @property
    def gbb(self) -> IVector3: ...
    @property
    def gbo(self) -> IVector3: ...
    @property
    def gbl(self) -> IVector3: ...
    @property
    def gor(self) -> IVector3: ...
    @property
    def gog(self) -> IVector3: ...
    @property
    def gob(self) -> IVector3: ...
    @property
    def goo(self) -> IVector3: ...
    @property
    def gol(self) -> IVector3: ...
    @property
    def glr(self) -> IVector3: ...
    @property
    def glg(self) -> IVector3: ...
    @property
    def glb(self) -> IVector3: ...
    @property
    def glo(self) -> IVector3: ...
    @property
    def gll(self) -> IVector3: ...
    @property
    def brr(self) -> IVector3: ...
    @property
    def brg(self) -> IVector3: ...
    @property
    def brb(self) -> IVector3: ...
    @property
    def bro(self) -> IVector3: ...
    @property
    def brl(self) -> IVector3: ...
    @property
    def bgr(self) -> IVector3: ...
    @property
    def bgg(self) -> IVector3: ...
    @property
    def bgb(self) -> IVector3: ...
    @property
    def bgo(self) -> IVector3: ...
    @property
    def bgl(self) -> IVector3: ...
    @property
    def bbr(self) -> IVector3: ...
    @property
    def bbg(self) -> IVector3: ...
    @property
    def bbb(self) -> IVector3: ...
    @property
    def bbo(self) -> IVector3: ...
    @property
    def bbl(self) -> IVector3: ...
    @property
    def bor(self) -> IVector3: ...
    @property
    def bog(self) -> IVector3: ...
    @property
    def bob(self) -> IVector3: ...
    @property
    def boo(self) -> IVector3: ...
    @property
    def bol(self) -> IVector3: ...
    @property
    def blr(self) -> IVector3: ...
    @property
    def blg(self) -> IVector3: ...
    @property
    def blb(self) -> IVector3: ...
    @property
    def blo(self) -> IVector3: ...
    @property
    def bll(self) -> IVector3: ...
    @property
    def orr(self) -> IVector3: ...
    @property
    def org(self) -> IVector3: ...
    @property
    def orb(self) -> IVector3: ...
    @property
    def oro(self) -> IVector3: ...
    @property
    def orl(self) -> IVector3: ...
    @property
    def ogr(self) -> IVector3: ...
    @property
    def ogg(self) -> IVector3: ...
    @property
    def ogb(self) -> IVector3: ...
    @property
    def ogo(self) -> IVector3: ...
    @property
    def ogl(self) -> IVector3: ...
    @property
    def obr(self) -> IVector3: ...
    @property
    def obg(self) -> IVector3: ...
    @property
    def obb(self) -> IVector3: ...
    @property
    def obo(self) -> IVector3: ...
    @property
    def obl(self) -> IVector3: ...
    @property
    def oor(self) -> IVector3: ...
    @property
    def oog(self) -> IVector3: ...
    @property
    def oob(self) -> IVector3: ...
    @property
    def olr(self) -> IVector3: ...
    @property
    def olg(self) -> IVector3: ...
    @property
    def olb(self) -> IVector3: ...
    @property
    def lrr(self) -> IVector3: ...
    @property
    def lrg(self) -> IVector3: ...
    @property
    def lrb(self) -> IVector3: ...
    @property
    def lro(self) -> IVector3: ...
    @property
    def lrl(self) -> IVector3: ...
    @property
    def lgr(self) -> IVector3: ...
    @property
    def lgg(self) -> IVector3: ...
    @property
    def lgb(self) -> IVector3: ...
    @property
    def lgo(self) -> IVector3: ...
    @property
    def lgl(self) -> IVector3: ...
    @property
    def lbr(self) -> IVector3: ...
    @property
    def lbg(self) -> IVector3: ...
    @property
    def lbb(self) -> IVector3: ...
    @property
    def lbo(self) -> IVector3: ...
    @property
    def lbl(self) -> IVector3: ...
    @property
    def lor(self) -> IVector3: ...
    @property
    def log(self) -> IVector3: ...
    @property
    def lob(self) -> IVector3: ...
    @property
    def llr(self) -> IVector3: ...
    @property
    def llg(self) -> IVector3: ...
    @property
    def llb(self) -> IVector3: ...
    @property
    def rrrr(self) -> IVector4: ...
    @property
    def rrrg(self) -> IVector4: ...
    @property
    def rrrb(self) -> IVector4: ...
    @property
    def rrro(self) -> IVector4: ...
    @property
    def rrrl(self) -> IVector4: ...
    @property
    def rrgr(self) -> IVector4: ...
    @property
    def rrgg(self) -> IVector4: ...
    @property
    def rrgb(self) -> IVector4: ...
    @property
    def rrgo(self) -> IVector4: ...
    @property
    def rrgl(self) -> IVector4: ...
    @property
    def rrbr(self) -> IVector4: ...
    @property
    def rrbg(self) -> IVector4: ...
    @property
    def rrbb(self) -> IVector4: ...
    @property
    def rrbo(self) -> IVector4: ...
    @property
    def rrbl(self) -> IVector4: ...
    @property
    def rror(self) -> IVector4: ...
    @property
    def rrog(self) -> IVector4: ...
    @property
    def rrob(self) -> IVector4: ...
    @property
    def rroo(self) -> IVector4: ...
    @property
    def rrol(self) -> IVector4: ...
    @property
    def rrlr(self) -> IVector4: ...
    @property
    def rrlg(self) -> IVector4: ...
    @property
    def rrlb(self) -> IVector4: ...
    @property
    def rrlo(self) -> IVector4: ...
    @property
    def rrll(self) -> IVector4: ...
    @property
    def rgrr(self) -> IVector4: ...
    @property
    def rgrg(self) -> IVector4: ...
    @property
    def rgrb(self) -> IVector4: ...
    @property
    def rgro(self) -> IVector4: ...
    @property
    def rgrl(self) -> IVector4: ...
    @property
    def rggr(self) -> IVector4: ...
    @property
    def rggg(self) -> IVector4: ...
    @property
    def rggb(self) -> IVector4: ...
    @property
    def rggo(self) -> IVector4: ...
    @property
    def rggl(self) -> IVector4: ...
    @property
    def rgbr(self) -> IVector4: ...
    @property
    def rgbg(self) -> IVector4: ...
    @property
    def rgbb(self) -> IVector4: ...
    @property
    def rgbo(self) -> IVector4: ...
    @property
    def rgbl(self) -> IVector4: ...
    @property
    def rgor(self) -> IVector4: ...
    @property
    def rgog(self) -> IVector4: ...
    @property
    def rgob(self) -> IVector4: ...
    @property
    def rgoo(self) -> IVector4: ...
    @property
    def rgol(self) -> IVector4: ...
    @property
    def rglr(self) -> IVector4: ...
    @property
    def rglg(self) -> IVector4: ...
    @property
    def rglb(self) -> IVector4: ...
    @property
    def rglo(self) -> IVector4: ...
    @property
    def rgll(self) -> IVector4: ...
    @property
    def rbrr(self) -> IVector4: ...
    @property
    def rbrg(self) -> IVector4: ...
    @property
    def rbrb(self) -> IVector4: ...
    @property
    def rbro(self) -> IVector4: ...
    @property
    def rbrl(self) -> IVector4: ...
    @property
    def rbgr(self) -> IVector4: ...
    @property
    def rbgg(self) -> IVector4: ...
    @property
    def rbgb(self) -> IVector4: ...
    @property
    def rbgo(self) -> IVector4: ...
    @property
    def rbgl(self) -> IVector4: ...
    @property
    def rbbr(self) -> IVector4: ...
    @property
    def rbbg(self) -> IVector4: ...
    @property
    def rbbb(self) -> IVector4: ...
    @property
    def rbbo(self) -> IVector4: ...
    @property
    def rbbl(self) -> IVector4: ...
    @property
    def rbor(self) -> IVector4: ...
    @property
    def rbog(self) -> IVector4: ...
    @property
    def rbob(self) -> IVector4: ...
    @property
    def rboo(self) -> IVector4: ...
    @property
    def rbol(self) -> IVector4: ...
    @property
    def rblr(self) -> IVector4: ...
    @property
    def rblg(self) -> IVector4: ...
    @property
    def rblb(self) -> IVector4: ...
    @property
    def rblo(self) -> IVector4: ...
    @property
    def rbll(self) -> IVector4: ...
    @property
    def rorr(self) -> IVector4: ...
    @property
    def rorg(self) -> IVector4: ...
    @property
    def rorb(self) -> IVector4: ...
    @property
    def roro(self) -> IVector4: ...
    @property
    def rorl(self) -> IVector4: ...
    @property
    def rogr(self) -> IVector4: ...
    @property
    def rogg(self) -> IVector4: ...
    @property
    def rogb(self) -> IVector4: ...
    @property
    def rogo(self) -> IVector4: ...
    @property
    def rogl(self) -> IVector4: ...
    @property
    def robr(self) -> IVector4: ...
    @property
    def robg(self) -> IVector4: ...
    @property
    def robb(self) -> IVector4: ...
    @property
    def robo(self) -> IVector4: ...
    @property
    def robl(self) -> IVector4: ...
    @property
    def roor(self) -> IVector4: ...
    @property
    def roog(self) -> IVector4: ...
    @property
    def roob(self) -> IVector4: ...
    @property
    def rooo(self) -> IVector4: ...
    @property
    def rool(self) -> IVector4: ...
    @property
    def rolr(self) -> IVector4: ...
    @property
    def rolg(self) -> IVector4: ...
    @property
    def rolb(self) -> IVector4: ...
    @property
    def rolo(self) -> IVector4: ...
    @property
    def roll(self) -> IVector4: ...
    @property
    def rlrr(self) -> IVector4: ...
    @property
    def rlrg(self) -> IVector4: ...
    @property
    def rlrb(self) -> IVector4: ...
    @property
    def rlro(self) -> IVector4: ...
    @property
    def rlrl(self) -> IVector4: ...
    @property
    def rlgr(self) -> IVector4: ...
    @property
    def rlgg(self) -> IVector4: ...
    @property
    def rlgb(self) -> IVector4: ...
    @property
    def rlgo(self) -> IVector4: ...
    @property
    def rlgl(self) -> IVector4: ...
    @property
    def rlbr(self) -> IVector4: ...
    @property
    def rlbg(self) -> IVector4: ...
    @property
    def rlbb(self) -> IVector4: ...
    @property
    def rlbo(self) -> IVector4: ...
    @property
    def rlbl(self) -> IVector4: ...
    @property
    def rlor(self) -> IVector4: ...
    @property
    def rlog(self) -> IVector4: ...
    @property
    def rlob(self) -> IVector4: ...
    @property
    def rloo(self) -> IVector4: ...
    @property
    def rlol(self) -> IVector4: ...
    @property
    def rllr(self) -> IVector4: ...
    @property
    def rllg(self) -> IVector4: ...
    @property
    def rllb(self) -> IVector4: ...
    @property
    def rllo(self) -> IVector4: ...
    @property
    def rlll(self) -> IVector4: ...
    @property
    def grrr(self) -> IVector4: ...
    @property
    def grrg(self) -> IVector4: ...
    @property
    def grrb(self) -> IVector4: ...
    @property
    def grro(self) -> IVector4: ...
    @property
    def grrl(self) -> IVector4: ...
    @property
    def grgr(self) -> IVector4: ...
    @property
    def grgg(self) -> IVector4: ...
    @property
    def grgb(self) -> IVector4: ...
    @property
    def grgo(self) -> IVector4: ...
    @property
    def grgl(self) -> IVector4: ...
    @property
    def grbr(self) -> IVector4: ...
    @property
    def grbg(self) -> IVector4: ...
    @property
    def grbb(self) -> IVector4: ...
    @property
    def grbo(self) -> IVector4: ...
    @property
    def grbl(self) -> IVector4: ...
    @property
    def gror(self) -> IVector4: ...
    @property
    def grog(self) -> IVector4: ...
    @property
    def grob(self) -> IVector4: ...
    @property
    def groo(self) -> IVector4: ...
    @property
    def grol(self) -> IVector4: ...
    @property
    def grlr(self) -> IVector4: ...
    @property
    def grlg(self) -> IVector4: ...
    @property
    def grlb(self) -> IVector4: ...
    @property
    def grlo(self) -> IVector4: ...
    @property
    def grll(self) -> IVector4: ...
    @property
    def ggrr(self) -> IVector4: ...
    @property
    def ggrg(self) -> IVector4: ...
    @property
    def ggrb(self) -> IVector4: ...
    @property
    def ggro(self) -> IVector4: ...
    @property
    def ggrl(self) -> IVector4: ...
    @property
    def gggr(self) -> IVector4: ...
    @property
    def gggg(self) -> IVector4: ...
    @property
    def gggb(self) -> IVector4: ...
    @property
    def gggo(self) -> IVector4: ...
    @property
    def gggl(self) -> IVector4: ...
    @property
    def ggbr(self) -> IVector4: ...
    @property
    def ggbg(self) -> IVector4: ...
    @property
    def ggbb(self) -> IVector4: ...
    @property
    def ggbo(self) -> IVector4: ...
    @property
    def ggbl(self) -> IVector4: ...
    @property
    def ggor(self) -> IVector4: ...
    @property
    def ggog(self) -> IVector4: ...
    @property
    def ggob(self) -> IVector4: ...
    @property
    def ggoo(self) -> IVector4: ...
    @property
    def ggol(self) -> IVector4: ...
    @property
    def gglr(self) -> IVector4: ...
    @property
    def gglg(self) -> IVector4: ...
    @property
    def gglb(self) -> IVector4: ...
    @property
    def gglo(self) -> IVector4: ...
    @property
    def ggll(self) -> IVector4: ...
    @property
    def gbrr(self) -> IVector4: ...
    @property
    def gbrg(self) -> IVector4: ...
    @property
    def gbrb(self) -> IVector4: ...
    @property
    def gbro(self) -> IVector4: ...
    @property
    def gbrl(self) -> IVector4: ...
    @property
    def gbgr(self) -> IVector4: ...
    @property
    def gbgg(self) -> IVector4: ...
    @property
    def gbgb(self) -> IVector4: ...
    @property
    def gbgo(self) -> IVector4: ...
    @property
    def gbgl(self) -> IVector4: ...
    @property
    def gbbr(self) -> IVector4: ...
    @property
    def gbbg(self) -> IVector4: ...
    @property
    def gbbb(self) -> IVector4: ...
    @property
    def gbbo(self) -> IVector4: ...
    @property
    def gbbl(self) -> IVector4: ...
    @property
    def gbor(self) -> IVector4: ...
    @property
    def gbog(self) -> IVector4: ...
    @property
    def gbob(self) -> IVector4: ...
    @property
    def gboo(self) -> IVector4: ...
    @property
    def gbol(self) -> IVector4: ...
    @property
    def gblr(self) -> IVector4: ...
    @property
    def gblg(self) -> IVector4: ...
    @property
    def gblb(self) -> IVector4: ...
    @property
    def gblo(self) -> IVector4: ...
    @property
    def gbll(self) -> IVector4: ...
    @property
    def gorr(self) -> IVector4: ...
    @property
    def gorg(self) -> IVector4: ...
    @property
    def gorb(self) -> IVector4: ...
    @property
    def goro(self) -> IVector4: ...
    @property
    def gorl(self) -> IVector4: ...
    @property
    def gogr(self) -> IVector4: ...
    @property
    def gogg(self) -> IVector4: ...
    @property
    def gogb(self) -> IVector4: ...
    @property
    def gogo(self) -> IVector4: ...
    @property
    def gogl(self) -> IVector4: ...
    @property
    def gobr(self) -> IVector4: ...
    @property
    def gobg(self) -> IVector4: ...
    @property
    def gobb(self) -> IVector4: ...
    @property
    def gobo(self) -> IVector4: ...
    @property
    def gobl(self) -> IVector4: ...
    @property
    def goor(self) -> IVector4: ...
    @property
    def goog(self) -> IVector4: ...
    @property
    def goob(self) -> IVector4: ...
    @property
    def gooo(self) -> IVector4: ...
    @property
    def gool(self) -> IVector4: ...
    @property
    def golr(self) -> IVector4: ...
    @property
    def golg(self) -> IVector4: ...
    @property
    def golb(self) -> IVector4: ...
    @property
    def golo(self) -> IVector4: ...
    @property
    def goll(self) -> IVector4: ...
    @property
    def glrr(self) -> IVector4: ...
    @property
    def glrg(self) -> IVector4: ...
    @property
    def glrb(self) -> IVector4: ...
    @property
    def glro(self) -> IVector4: ...
    @property
    def glrl(self) -> IVector4: ...
    @property
    def glgr(self) -> IVector4: ...
    @property
    def glgg(self) -> IVector4: ...
    @property
    def glgb(self) -> IVector4: ...
    @property
    def glgo(self) -> IVector4: ...
    @property
    def glgl(self) -> IVector4: ...
    @property
    def glbr(self) -> IVector4: ...
    @property
    def glbg(self) -> IVector4: ...
    @property
    def glbb(self) -> IVector4: ...
    @property
    def glbo(self) -> IVector4: ...
    @property
    def glbl(self) -> IVector4: ...
    @property
    def glor(self) -> IVector4: ...
    @property
    def glog(self) -> IVector4: ...
    @property
    def glob(self) -> IVector4: ...
    @property
    def gloo(self) -> IVector4: ...
    @property
    def glol(self) -> IVector4: ...
    @property
    def gllr(self) -> IVector4: ...
    @property
    def gllg(self) -> IVector4: ...
    @property
    def gllb(self) -> IVector4: ...
    @property
    def gllo(self) -> IVector4: ...
    @property
    def glll(self) -> IVector4: ...
    @property
    def brrr(self) -> IVector4: ...
    @property
    def brrg(self) -> IVector4: ...
    @property
    def brrb(self) -> IVector4: ...
    @property
    def brro(self) -> IVector4: ...
    @property
    def brrl(self) -> IVector4: ...
    @property
    def brgr(self) -> IVector4: ...
    @property
    def brgg(self) -> IVector4: ...
    @property
    def brgb(self) -> IVector4: ...
    @property
    def brgo(self) -> IVector4: ...
    @property
    def brgl(self) -> IVector4: ...
    @property
    def brbr(self) -> IVector4: ...
    @property
    def brbg(self) -> IVector4: ...
    @property
    def brbb(self) -> IVector4: ...
    @property
    def brbo(self) -> IVector4: ...
    @property
    def brbl(self) -> IVector4: ...
    @property
    def bror(self) -> IVector4: ...
    @property
    def brog(self) -> IVector4: ...
    @property
    def brob(self) -> IVector4: ...
    @property
    def broo(self) -> IVector4: ...
    @property
    def brol(self) -> IVector4: ...
    @property
    def brlr(self) -> IVector4: ...
    @property
    def brlg(self) -> IVector4: ...
    @property
    def brlb(self) -> IVector4: ...
    @property
    def brlo(self) -> IVector4: ...
    @property
    def brll(self) -> IVector4: ...
    @property
    def bgrr(self) -> IVector4: ...
    @property
    def bgrg(self) -> IVector4: ...
    @property
    def bgrb(self) -> IVector4: ...
    @property
    def bgro(self) -> IVector4: ...
    @property
    def bgrl(self) -> IVector4: ...
    @property
    def bggr(self) -> IVector4: ...
    @property
    def bggg(self) -> IVector4: ...
    @property
    def bggb(self) -> IVector4: ...
    @property
    def bggo(self) -> IVector4: ...
    @property
    def bggl(self) -> IVector4: ...
    @property
    def bgbr(self) -> IVector4: ...
    @property
    def bgbg(self) -> IVector4: ...
    @property
    def bgbb(self) -> IVector4: ...
    @property
    def bgbo(self) -> IVector4: ...
    @property
    def bgbl(self) -> IVector4: ...
    @property
    def bgor(self) -> IVector4: ...
    @property
    def bgog(self) -> IVector4: ...
    @property
    def bgob(self) -> IVector4: ...
    @property
    def bgoo(self) -> IVector4: ...
    @property
    def bgol(self) -> IVector4: ...
    @property
    def bglr(self) -> IVector4: ...
    @property
    def bglg(self) -> IVector4: ...
    @property
    def bglb(self) -> IVector4: ...
    @property
    def bglo(self) -> IVector4: ...
    @property
    def bgll(self) -> IVector4: ...
    @property
    def bbrr(self) -> IVector4: ...
    @property
    def bbrg(self) -> IVector4: ...
    @property
    def bbrb(self) -> IVector4: ...
    @property
    def bbro(self) -> IVector4: ...
    @property
    def bbrl(self) -> IVector4: ...
    @property
    def bbgr(self) -> IVector4: ...
    @property
    def bbgg(self) -> IVector4: ...
    @property
    def bbgb(self) -> IVector4: ...
    @property
    def bbgo(self) -> IVector4: ...
    @property
    def bbgl(self) -> IVector4: ...
    @property
    def bbbr(self) -> IVector4: ...
    @property
    def bbbg(self) -> IVector4: ...
    @property
    def bbbb(self) -> IVector4: ...
    @property
    def bbbo(self) -> IVector4: ...
    @property
    def bbbl(self) -> IVector4: ...
    @property
    def bbor(self) -> IVector4: ...
    @property
    def bbog(self) -> IVector4: ...
    @property
    def bbob(self) -> IVector4: ...
    @property
    def bboo(self) -> IVector4: ...
    @property
    def bbol(self) -> IVector4: ...
    @property
    def bblr(self) -> IVector4: ...
    @property
    def bblg(self) -> IVector4: ...
    @property
    def bblb(self) -> IVector4: ...
    @property
    def bblo(self) -> IVector4: ...
    @property
    def bbll(self) -> IVector4: ...
    @property
    def borr(self) -> IVector4: ...
    @property
    def borg(self) -> IVector4: ...
    @property
    def borb(self) -> IVector4: ...
    @property
    def boro(self) -> IVector4: ...
    @property
    def borl(self) -> IVector4: ...
    @property
    def bogr(self) -> IVector4: ...
    @property
    def bogg(self) -> IVector4: ...
    @property
    def bogb(self) -> IVector4: ...
    @property
    def bogo(self) -> IVector4: ...
    @property
    def bogl(self) -> IVector4: ...
    @property
    def bobr(self) -> IVector4: ...
    @property
    def bobg(self) -> IVector4: ...
    @property
    def bobb(self) -> IVector4: ...
    @property
    def bobo(self) -> IVector4: ...
    @property
    def bobl(self) -> IVector4: ...
    @property
    def boor(self) -> IVector4: ...
    @property
    def boog(self) -> IVector4: ...
    @property
    def boob(self) -> IVector4: ...
    @property
    def booo(self) -> IVector4: ...
    @property
    def bool(self) -> IVector4: ...
    @property
    def bolr(self) -> IVector4: ...
    @property
    def bolg(self) -> IVector4: ...
    @property
    def bolb(self) -> IVector4: ...
    @property
    def bolo(self) -> IVector4: ...
    @property
    def boll(self) -> IVector4: ...
    @property
    def blrr(self) -> IVector4: ...
    @property
    def blrg(self) -> IVector4: ...
    @property
    def blrb(self) -> IVector4: ...
    @property
    def blro(self) -> IVector4: ...
    @property
    def blrl(self) -> IVector4: ...
    @property
    def blgr(self) -> IVector4: ...
    @property
    def blgg(self) -> IVector4: ...
    @property
    def blgb(self) -> IVector4: ...
    @property
    def blgo(self) -> IVector4: ...
    @property
    def blgl(self) -> IVector4: ...
    @property
    def blbr(self) -> IVector4: ...
    @property
    def blbg(self) -> IVector4: ...
    @property
    def blbb(self) -> IVector4: ...
    @property
    def blbo(self) -> IVector4: ...
    @property
    def blbl(self) -> IVector4: ...
    @property
    def blor(self) -> IVector4: ...
    @property
    def blog(self) -> IVector4: ...
    @property
    def blob(self) -> IVector4: ...
    @property
    def bloo(self) -> IVector4: ...
    @property
    def blol(self) -> IVector4: ...
    @property
    def bllr(self) -> IVector4: ...
    @property
    def bllg(self) -> IVector4: ...
    @property
    def bllb(self) -> IVector4: ...
    @property
    def bllo(self) -> IVector4: ...
    @property
    def blll(self) -> IVector4: ...
    @property
    def orrr(self) -> IVector4: ...
    @property
    def orrg(self) -> IVector4: ...
    @property
    def orrb(self) -> IVector4: ...
    @property
    def orro(self) -> IVector4: ...
    @property
    def orrl(self) -> IVector4: ...
    @property
    def orgr(self) -> IVector4: ...
    @property
    def orgg(self) -> IVector4: ...
    @property
    def orgb(self) -> IVector4: ...
    @property
    def orgo(self) -> IVector4: ...
    @property
    def orgl(self) -> IVector4: ...
    @property
    def orbr(self) -> IVector4: ...
    @property
    def orbg(self) -> IVector4: ...
    @property
    def orbb(self) -> IVector4: ...
    @property
    def orbo(self) -> IVector4: ...
    @property
    def orbl(self) -> IVector4: ...
    @property
    def oror(self) -> IVector4: ...
    @property
    def orog(self) -> IVector4: ...
    @property
    def orob(self) -> IVector4: ...
    @property
    def oroo(self) -> IVector4: ...
    @property
    def orol(self) -> IVector4: ...
    @property
    def orlr(self) -> IVector4: ...
    @property
    def orlg(self) -> IVector4: ...
    @property
    def orlb(self) -> IVector4: ...
    @property
    def orlo(self) -> IVector4: ...
    @property
    def orll(self) -> IVector4: ...
    @property
    def ogrr(self) -> IVector4: ...
    @property
    def ogrg(self) -> IVector4: ...
    @property
    def ogrb(self) -> IVector4: ...
    @property
    def ogro(self) -> IVector4: ...
    @property
    def ogrl(self) -> IVector4: ...
    @property
    def oggr(self) -> IVector4: ...
    @property
    def oggg(self) -> IVector4: ...
    @property
    def oggb(self) -> IVector4: ...
    @property
    def oggo(self) -> IVector4: ...
    @property
    def oggl(self) -> IVector4: ...
    @property
    def ogbr(self) -> IVector4: ...
    @property
    def ogbg(self) -> IVector4: ...
    @property
    def ogbb(self) -> IVector4: ...
    @property
    def ogbo(self) -> IVector4: ...
    @property
    def ogbl(self) -> IVector4: ...
    @property
    def ogor(self) -> IVector4: ...
    @property
    def ogog(self) -> IVector4: ...
    @property
    def ogob(self) -> IVector4: ...
    @property
    def ogoo(self) -> IVector4: ...
    @property
    def ogol(self) -> IVector4: ...
    @property
    def oglr(self) -> IVector4: ...
    @property
    def oglg(self) -> IVector4: ...
    @property
    def oglb(self) -> IVector4: ...
    @property
    def oglo(self) -> IVector4: ...
    @property
    def ogll(self) -> IVector4: ...
    @property
    def obrr(self) -> IVector4: ...
    @property
    def obrg(self) -> IVector4: ...
    @property
    def obrb(self) -> IVector4: ...
    @property
    def obro(self) -> IVector4: ...
    @property
    def obrl(self) -> IVector4: ...
    @property
    def obgr(self) -> IVector4: ...
    @property
    def obgg(self) -> IVector4: ...
    @property
    def obgb(self) -> IVector4: ...
    @property
    def obgo(self) -> IVector4: ...
    @property
    def obgl(self) -> IVector4: ...
    @property
    def obbr(self) -> IVector4: ...
    @property
    def obbg(self) -> IVector4: ...
    @property
    def obbb(self) -> IVector4: ...
    @property
    def obbo(self) -> IVector4: ...
    @property
    def obbl(self) -> IVector4: ...
    @property
    def obor(self) -> IVector4: ...
    @property
    def obog(self) -> IVector4: ...
    @property
    def obob(self) -> IVector4: ...
    @property
    def oboo(self) -> IVector4: ...
    @property
    def obol(self) -> IVector4: ...
    @property
    def oblr(self) -> IVector4: ...
    @property
    def oblg(self) -> IVector4: ...
    @property
    def oblb(self) -> IVector4: ...
    @property
    def oblo(self) -> IVector4: ...
    @property
    def obll(self) -> IVector4: ...
    @property
    def oorr(self) -> IVector4: ...
    @property
    def oorg(self) -> IVector4: ...
    @property
    def oorb(self) -> IVector4: ...
    @property
    def ooro(self) -> IVector4: ...
    @property
    def oorl(self) -> IVector4: ...
    @property
    def oogr(self) -> IVector4: ...
    @property
    def oogg(self) -> IVector4: ...
    @property
    def oogb(self) -> IVector4: ...
    @property
    def oogo(self) -> IVector4: ...
    @property
    def oogl(self) -> IVector4: ...
    @property
    def oobr(self) -> IVector4: ...
    @property
    def oobg(self) -> IVector4: ...
    @property
    def oobb(self) -> IVector4: ...
    @property
    def oobo(self) -> IVector4: ...
    @property
    def oobl(self) -> IVector4: ...
    @property
    def ooor(self) -> IVector4: ...
    @property
    def ooog(self) -> IVector4: ...
    @property
    def ooob(self) -> IVector4: ...
    @property
    def oolr(self) -> IVector4: ...
    @property
    def oolg(self) -> IVector4: ...
    @property
    def oolb(self) -> IVector4: ...
    @property
    def olrr(self) -> IVector4: ...
    @property
    def olrg(self) -> IVector4: ...
    @property
    def olrb(self) -> IVector4: ...
    @property
    def olro(self) -> IVector4: ...
    @property
    def olrl(self) -> IVector4: ...
    @property
    def olgr(self) -> IVector4: ...
    @property
    def olgg(self) -> IVector4: ...
    @property
    def olgb(self) -> IVector4: ...
    @property
    def olgo(self) -> IVector4: ...
    @property
    def olgl(self) -> IVector4: ...
    @property
    def olbr(self) -> IVector4: ...
    @property
    def olbg(self) -> IVector4: ...
    @property
    def olbb(self) -> IVector4: ...
    @property
    def olbo(self) -> IVector4: ...
    @property
    def olbl(self) -> IVector4: ...
    @property
    def olor(self) -> IVector4: ...
    @property
    def olog(self) -> IVector4: ...
    @property
    def olob(self) -> IVector4: ...
    @property
    def ollr(self) -> IVector4: ...
    @property
    def ollg(self) -> IVector4: ...
    @property
    def ollb(self) -> IVector4: ...
    @property
    def lrrr(self) -> IVector4: ...
    @property
    def lrrg(self) -> IVector4: ...
    @property
    def lrrb(self) -> IVector4: ...
    @property
    def lrro(self) -> IVector4: ...
    @property
    def lrrl(self) -> IVector4: ...
    @property
    def lrgr(self) -> IVector4: ...
    @property
    def lrgg(self) -> IVector4: ...
    @property
    def lrgb(self) -> IVector4: ...
    @property
    def lrgo(self) -> IVector4: ...
    @property
    def lrgl(self) -> IVector4: ...
    @property
    def lrbr(self) -> IVector4: ...
    @property
    def lrbg(self) -> IVector4: ...
    @property
    def lrbb(self) -> IVector4: ...
    @property
    def lrbo(self) -> IVector4: ...
    @property
    def lrbl(self) -> IVector4: ...
    @property
    def lror(self) -> IVector4: ...
    @property
    def lrog(self) -> IVector4: ...
    @property
    def lrob(self) -> IVector4: ...
    @property
    def lroo(self) -> IVector4: ...
    @property
    def lrol(self) -> IVector4: ...
    @property
    def lrlr(self) -> IVector4: ...
    @property
    def lrlg(self) -> IVector4: ...
    @property
    def lrlb(self) -> IVector4: ...
    @property
    def lrlo(self) -> IVector4: ...
    @property
    def lrll(self) -> IVector4: ...
    @property
    def lgrr(self) -> IVector4: ...
    @property
    def lgrg(self) -> IVector4: ...
    @property
    def lgrb(self) -> IVector4: ...
    @property
    def lgro(self) -> IVector4: ...
    @property
    def lgrl(self) -> IVector4: ...
    @property
    def lggr(self) -> IVector4: ...
    @property
    def lggg(self) -> IVector4: ...
    @property
    def lggb(self) -> IVector4: ...
    @property
    def lggo(self) -> IVector4: ...
    @property
    def lggl(self) -> IVector4: ...
    @property
    def lgbr(self) -> IVector4: ...
    @property
    def lgbg(self) -> IVector4: ...
    @property
    def lgbb(self) -> IVector4: ...
    @property
    def lgbo(self) -> IVector4: ...
    @property
    def lgbl(self) -> IVector4: ...
    @property
    def lgor(self) -> IVector4: ...
    @property
    def lgog(self) -> IVector4: ...
    @property
    def lgob(self) -> IVector4: ...
    @property
    def lgoo(self) -> IVector4: ...
    @property
    def lgol(self) -> IVector4: ...
    @property
    def lglr(self) -> IVector4: ...
    @property
    def lglg(self) -> IVector4: ...
    @property
    def lglb(self) -> IVector4: ...
    @property
    def lglo(self) -> IVector4: ...
    @property
    def lgll(self) -> IVector4: ...
    @property
    def lbrr(self) -> IVector4: ...
    @property
    def lbrg(self) -> IVector4: ...
    @property
    def lbrb(self) -> IVector4: ...
    @property
    def lbro(self) -> IVector4: ...
    @property
    def lbrl(self) -> IVector4: ...
    @property
    def lbgr(self) -> IVector4: ...
    @property
    def lbgg(self) -> IVector4: ...
    @property
    def lbgb(self) -> IVector4: ...
    @property
    def lbgo(self) -> IVector4: ...
    @property
    def lbgl(self) -> IVector4: ...
    @property
    def lbbr(self) -> IVector4: ...
    @property
    def lbbg(self) -> IVector4: ...
    @property
    def lbbb(self) -> IVector4: ...
    @property
    def lbbo(self) -> IVector4: ...
    @property
    def lbbl(self) -> IVector4: ...
    @property
    def lbor(self) -> IVector4: ...
    @property
    def lbog(self) -> IVector4: ...
    @property
    def lbob(self) -> IVector4: ...
    @property
    def lboo(self) -> IVector4: ...
    @property
    def lbol(self) -> IVector4: ...
    @property
    def lblr(self) -> IVector4: ...
    @property
    def lblg(self) -> IVector4: ...
    @property
    def lblb(self) -> IVector4: ...
    @property
    def lblo(self) -> IVector4: ...
    @property
    def lbll(self) -> IVector4: ...
    @property
    def lorr(self) -> IVector4: ...
    @property
    def lorg(self) -> IVector4: ...
    @property
    def lorb(self) -> IVector4: ...
    @property
    def loro(self) -> IVector4: ...
    @property
    def lorl(self) -> IVector4: ...
    @property
    def logr(self) -> IVector4: ...
    @property
    def logg(self) -> IVector4: ...
    @property
    def logb(self) -> IVector4: ...
    @property
    def logo(self) -> IVector4: ...
    @property
    def logl(self) -> IVector4: ...
    @property
    def lobr(self) -> IVector4: ...
    @property
    def lobg(self) -> IVector4: ...
    @property
    def lobb(self) -> IVector4: ...
    @property
    def lobo(self) -> IVector4: ...
    @property
    def lobl(self) -> IVector4: ...
    @property
    def loor(self) -> IVector4: ...
    @property
    def loog(self) -> IVector4: ...
    @property
    def loob(self) -> IVector4: ...
    @property
    def lolr(self) -> IVector4: ...
    @property
    def lolg(self) -> IVector4: ...
    @property
    def lolb(self) -> IVector4: ...
    @property
    def llrr(self) -> IVector4: ...
    @property
    def llrg(self) -> IVector4: ...
    @property
    def llrb(self) -> IVector4: ...
    @property
    def llro(self) -> IVector4: ...
    @property
    def llrl(self) -> IVector4: ...
    @property
    def llgr(self) -> IVector4: ...
    @property
    def llgg(self) -> IVector4: ...
    @property
    def llgb(self) -> IVector4: ...
    @property
    def llgo(self) -> IVector4: ...
    @property
    def llgl(self) -> IVector4: ...
    @property
    def llbr(self) -> IVector4: ...
    @property
    def llbg(self) -> IVector4: ...
    @property
    def llbb(self) -> IVector4: ...
    @property
    def llbo(self) -> IVector4: ...
    @property
    def llbl(self) -> IVector4: ...
    @property
    def llor(self) -> IVector4: ...
    @property
    def llog(self) -> IVector4: ...
    @property
    def llob(self) -> IVector4: ...
    @property
    def lllr(self) -> IVector4: ...
    @property
    def lllg(self) -> IVector4: ...
    @property
    def lllb(self) -> IVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> IVector2: ...
    @property
    def st(self) -> IVector2: ...
    @property
    def sq(self) -> IVector2: ...
    @property
    def so(self) -> IVector2: ...
    @property
    def sl(self) -> IVector2: ...
    @property
    def ts(self) -> IVector2: ...
    @property
    def tt(self) -> IVector2: ...
    @property
    def tq(self) -> IVector2: ...
    @property
    def to(self) -> IVector2: ...
    @property
    def tl(self) -> IVector2: ...
    @property
    def qs(self) -> IVector2: ...
    @property
    def qt(self) -> IVector2: ...
    @property
    def qq(self) -> IVector2: ...
    @property
    def qo(self) -> IVector2: ...
    @property
    def ql(self) -> IVector2: ...
    @property
    def os(self) -> IVector2: ...
    @property
    def ot(self) -> IVector2: ...
    @property
    def oq(self) -> IVector2: ...
    @property
    def ls(self) -> IVector2: ...
    @property
    def lt(self) -> IVector2: ...
    @property
    def lq(self) -> IVector2: ...
    @property
    def sss(self) -> IVector3: ...
    @property
    def sst(self) -> IVector3: ...
    @property
    def ssq(self) -> IVector3: ...
    @property
    def sso(self) -> IVector3: ...
    @property
    def ssl(self) -> IVector3: ...
    @property
    def sts(self) -> IVector3: ...
    @property
    def stt(self) -> IVector3: ...
    @property
    def stq(self) -> IVector3: ...
    @property
    def sto(self) -> IVector3: ...
    @property
    def stl(self) -> IVector3: ...
    @property
    def sqs(self) -> IVector3: ...
    @property
    def sqt(self) -> IVector3: ...
    @property
    def sqq(self) -> IVector3: ...
    @property
    def sqo(self) -> IVector3: ...
    @property
    def sql(self) -> IVector3: ...
    @property
    def sos(self) -> IVector3: ...
    @property
    def sot(self) -> IVector3: ...
    @property
    def soq(self) -> IVector3: ...
    @property
    def soo(self) -> IVector3: ...
    @property
    def sol(self) -> IVector3: ...
    @property
    def sls(self) -> IVector3: ...
    @property
    def slt(self) -> IVector3: ...
    @property
    def slq(self) -> IVector3: ...
    @property
    def slo(self) -> IVector3: ...
    @property
    def sll(self) -> IVector3: ...
    @property
    def tss(self) -> IVector3: ...
    @property
    def tst(self) -> IVector3: ...
    @property
    def tsq(self) -> IVector3: ...
    @property
    def tso(self) -> IVector3: ...
    @property
    def tsl(self) -> IVector3: ...
    @property
    def tts(self) -> IVector3: ...
    @property
    def ttt(self) -> IVector3: ...
    @property
    def ttq(self) -> IVector3: ...
    @property
    def tto(self) -> IVector3: ...
    @property
    def ttl(self) -> IVector3: ...
    @property
    def tqs(self) -> IVector3: ...
    @property
    def tqt(self) -> IVector3: ...
    @property
    def tqq(self) -> IVector3: ...
    @property
    def tqo(self) -> IVector3: ...
    @property
    def tql(self) -> IVector3: ...
    @property
    def tos(self) -> IVector3: ...
    @property
    def tot(self) -> IVector3: ...
    @property
    def toq(self) -> IVector3: ...
    @property
    def too(self) -> IVector3: ...
    @property
    def tol(self) -> IVector3: ...
    @property
    def tls(self) -> IVector3: ...
    @property
    def tlt(self) -> IVector3: ...
    @property
    def tlq(self) -> IVector3: ...
    @property
    def tlo(self) -> IVector3: ...
    @property
    def tll(self) -> IVector3: ...
    @property
    def qss(self) -> IVector3: ...
    @property
    def qst(self) -> IVector3: ...
    @property
    def qsq(self) -> IVector3: ...
    @property
    def qso(self) -> IVector3: ...
    @property
    def qsl(self) -> IVector3: ...
    @property
    def qts(self) -> IVector3: ...
    @property
    def qtt(self) -> IVector3: ...
    @property
    def qtq(self) -> IVector3: ...
    @property
    def qto(self) -> IVector3: ...
    @property
    def qtl(self) -> IVector3: ...
    @property
    def qqs(self) -> IVector3: ...
    @property
    def qqt(self) -> IVector3: ...
    @property
    def qqq(self) -> IVector3: ...
    @property
    def qqo(self) -> IVector3: ...
    @property
    def qql(self) -> IVector3: ...
    @property
    def qos(self) -> IVector3: ...
    @property
    def qot(self) -> IVector3: ...
    @property
    def qoq(self) -> IVector3: ...
    @property
    def qoo(self) -> IVector3: ...
    @property
    def qol(self) -> IVector3: ...
    @property
    def qls(self) -> IVector3: ...
    @property
    def qlt(self) -> IVector3: ...
    @property
    def qlq(self) -> IVector3: ...
    @property
    def qlo(self) -> IVector3: ...
    @property
    def qll(self) -> IVector3: ...
    @property
    def oss(self) -> IVector3: ...
    @property
    def ost(self) -> IVector3: ...
    @property
    def osq(self) -> IVector3: ...
    @property
    def oso(self) -> IVector3: ...
    @property
    def osl(self) -> IVector3: ...
    @property
    def ots(self) -> IVector3: ...
    @property
    def ott(self) -> IVector3: ...
    @property
    def otq(self) -> IVector3: ...
    @property
    def oto(self) -> IVector3: ...
    @property
    def otl(self) -> IVector3: ...
    @property
    def oqs(self) -> IVector3: ...
    @property
    def oqt(self) -> IVector3: ...
    @property
    def oqq(self) -> IVector3: ...
    @property
    def oqo(self) -> IVector3: ...
    @property
    def oql(self) -> IVector3: ...
    @property
    def oos(self) -> IVector3: ...
    @property
    def oot(self) -> IVector3: ...
    @property
    def ooq(self) -> IVector3: ...
    @property
    def ols(self) -> IVector3: ...
    @property
    def olt(self) -> IVector3: ...
    @property
    def olq(self) -> IVector3: ...
    @property
    def lss(self) -> IVector3: ...
    @property
    def lst(self) -> IVector3: ...
    @property
    def lsq(self) -> IVector3: ...
    @property
    def lso(self) -> IVector3: ...
    @property
    def lsl(self) -> IVector3: ...
    @property
    def lts(self) -> IVector3: ...
    @property
    def ltt(self) -> IVector3: ...
    @property
    def ltq(self) -> IVector3: ...
    @property
    def lto(self) -> IVector3: ...
    @property
    def ltl(self) -> IVector3: ...
    @property
    def lqs(self) -> IVector3: ...
    @property
    def lqt(self) -> IVector3: ...
    @property
    def lqq(self) -> IVector3: ...
    @property
    def lqo(self) -> IVector3: ...
    @property
    def lql(self) -> IVector3: ...
    @property
    def los(self) -> IVector3: ...
    @property
    def lot(self) -> IVector3: ...
    @property
    def loq(self) -> IVector3: ...
    @property
    def lls(self) -> IVector3: ...
    @property
    def llt(self) -> IVector3: ...
    @property
    def llq(self) -> IVector3: ...
    @property
    def ssss(self) -> IVector4: ...
    @property
    def ssst(self) -> IVector4: ...
    @property
    def sssq(self) -> IVector4: ...
    @property
    def ssso(self) -> IVector4: ...
    @property
    def sssl(self) -> IVector4: ...
    @property
    def ssts(self) -> IVector4: ...
    @property
    def sstt(self) -> IVector4: ...
    @property
    def sstq(self) -> IVector4: ...
    @property
    def ssto(self) -> IVector4: ...
    @property
    def sstl(self) -> IVector4: ...
    @property
    def ssqs(self) -> IVector4: ...
    @property
    def ssqt(self) -> IVector4: ...
    @property
    def ssqq(self) -> IVector4: ...
    @property
    def ssqo(self) -> IVector4: ...
    @property
    def ssql(self) -> IVector4: ...
    @property
    def ssos(self) -> IVector4: ...
    @property
    def ssot(self) -> IVector4: ...
    @property
    def ssoq(self) -> IVector4: ...
    @property
    def ssoo(self) -> IVector4: ...
    @property
    def ssol(self) -> IVector4: ...
    @property
    def ssls(self) -> IVector4: ...
    @property
    def sslt(self) -> IVector4: ...
    @property
    def sslq(self) -> IVector4: ...
    @property
    def sslo(self) -> IVector4: ...
    @property
    def ssll(self) -> IVector4: ...
    @property
    def stss(self) -> IVector4: ...
    @property
    def stst(self) -> IVector4: ...
    @property
    def stsq(self) -> IVector4: ...
    @property
    def stso(self) -> IVector4: ...
    @property
    def stsl(self) -> IVector4: ...
    @property
    def stts(self) -> IVector4: ...
    @property
    def sttt(self) -> IVector4: ...
    @property
    def sttq(self) -> IVector4: ...
    @property
    def stto(self) -> IVector4: ...
    @property
    def sttl(self) -> IVector4: ...
    @property
    def stqs(self) -> IVector4: ...
    @property
    def stqt(self) -> IVector4: ...
    @property
    def stqq(self) -> IVector4: ...
    @property
    def stqo(self) -> IVector4: ...
    @property
    def stql(self) -> IVector4: ...
    @property
    def stos(self) -> IVector4: ...
    @property
    def stot(self) -> IVector4: ...
    @property
    def stoq(self) -> IVector4: ...
    @property
    def stoo(self) -> IVector4: ...
    @property
    def stol(self) -> IVector4: ...
    @property
    def stls(self) -> IVector4: ...
    @property
    def stlt(self) -> IVector4: ...
    @property
    def stlq(self) -> IVector4: ...
    @property
    def stlo(self) -> IVector4: ...
    @property
    def stll(self) -> IVector4: ...
    @property
    def sqss(self) -> IVector4: ...
    @property
    def sqst(self) -> IVector4: ...
    @property
    def sqsq(self) -> IVector4: ...
    @property
    def sqso(self) -> IVector4: ...
    @property
    def sqsl(self) -> IVector4: ...
    @property
    def sqts(self) -> IVector4: ...
    @property
    def sqtt(self) -> IVector4: ...
    @property
    def sqtq(self) -> IVector4: ...
    @property
    def sqto(self) -> IVector4: ...
    @property
    def sqtl(self) -> IVector4: ...
    @property
    def sqqs(self) -> IVector4: ...
    @property
    def sqqt(self) -> IVector4: ...
    @property
    def sqqq(self) -> IVector4: ...
    @property
    def sqqo(self) -> IVector4: ...
    @property
    def sqql(self) -> IVector4: ...
    @property
    def sqos(self) -> IVector4: ...
    @property
    def sqot(self) -> IVector4: ...
    @property
    def sqoq(self) -> IVector4: ...
    @property
    def sqoo(self) -> IVector4: ...
    @property
    def sqol(self) -> IVector4: ...
    @property
    def sqls(self) -> IVector4: ...
    @property
    def sqlt(self) -> IVector4: ...
    @property
    def sqlq(self) -> IVector4: ...
    @property
    def sqlo(self) -> IVector4: ...
    @property
    def sqll(self) -> IVector4: ...
    @property
    def soss(self) -> IVector4: ...
    @property
    def sost(self) -> IVector4: ...
    @property
    def sosq(self) -> IVector4: ...
    @property
    def soso(self) -> IVector4: ...
    @property
    def sosl(self) -> IVector4: ...
    @property
    def sots(self) -> IVector4: ...
    @property
    def sott(self) -> IVector4: ...
    @property
    def sotq(self) -> IVector4: ...
    @property
    def soto(self) -> IVector4: ...
    @property
    def sotl(self) -> IVector4: ...
    @property
    def soqs(self) -> IVector4: ...
    @property
    def soqt(self) -> IVector4: ...
    @property
    def soqq(self) -> IVector4: ...
    @property
    def soqo(self) -> IVector4: ...
    @property
    def soql(self) -> IVector4: ...
    @property
    def soos(self) -> IVector4: ...
    @property
    def soot(self) -> IVector4: ...
    @property
    def sooq(self) -> IVector4: ...
    @property
    def sooo(self) -> IVector4: ...
    @property
    def sool(self) -> IVector4: ...
    @property
    def sols(self) -> IVector4: ...
    @property
    def solt(self) -> IVector4: ...
    @property
    def solq(self) -> IVector4: ...
    @property
    def solo(self) -> IVector4: ...
    @property
    def soll(self) -> IVector4: ...
    @property
    def slss(self) -> IVector4: ...
    @property
    def slst(self) -> IVector4: ...
    @property
    def slsq(self) -> IVector4: ...
    @property
    def slso(self) -> IVector4: ...
    @property
    def slsl(self) -> IVector4: ...
    @property
    def slts(self) -> IVector4: ...
    @property
    def sltt(self) -> IVector4: ...
    @property
    def sltq(self) -> IVector4: ...
    @property
    def slto(self) -> IVector4: ...
    @property
    def sltl(self) -> IVector4: ...
    @property
    def slqs(self) -> IVector4: ...
    @property
    def slqt(self) -> IVector4: ...
    @property
    def slqq(self) -> IVector4: ...
    @property
    def slqo(self) -> IVector4: ...
    @property
    def slql(self) -> IVector4: ...
    @property
    def slos(self) -> IVector4: ...
    @property
    def slot(self) -> IVector4: ...
    @property
    def sloq(self) -> IVector4: ...
    @property
    def sloo(self) -> IVector4: ...
    @property
    def slol(self) -> IVector4: ...
    @property
    def slls(self) -> IVector4: ...
    @property
    def sllt(self) -> IVector4: ...
    @property
    def sllq(self) -> IVector4: ...
    @property
    def sllo(self) -> IVector4: ...
    @property
    def slll(self) -> IVector4: ...
    @property
    def tsss(self) -> IVector4: ...
    @property
    def tsst(self) -> IVector4: ...
    @property
    def tssq(self) -> IVector4: ...
    @property
    def tsso(self) -> IVector4: ...
    @property
    def tssl(self) -> IVector4: ...
    @property
    def tsts(self) -> IVector4: ...
    @property
    def tstt(self) -> IVector4: ...
    @property
    def tstq(self) -> IVector4: ...
    @property
    def tsto(self) -> IVector4: ...
    @property
    def tstl(self) -> IVector4: ...
    @property
    def tsqs(self) -> IVector4: ...
    @property
    def tsqt(self) -> IVector4: ...
    @property
    def tsqq(self) -> IVector4: ...
    @property
    def tsqo(self) -> IVector4: ...
    @property
    def tsql(self) -> IVector4: ...
    @property
    def tsos(self) -> IVector4: ...
    @property
    def tsot(self) -> IVector4: ...
    @property
    def tsoq(self) -> IVector4: ...
    @property
    def tsoo(self) -> IVector4: ...
    @property
    def tsol(self) -> IVector4: ...
    @property
    def tsls(self) -> IVector4: ...
    @property
    def tslt(self) -> IVector4: ...
    @property
    def tslq(self) -> IVector4: ...
    @property
    def tslo(self) -> IVector4: ...
    @property
    def tsll(self) -> IVector4: ...
    @property
    def ttss(self) -> IVector4: ...
    @property
    def ttst(self) -> IVector4: ...
    @property
    def ttsq(self) -> IVector4: ...
    @property
    def ttso(self) -> IVector4: ...
    @property
    def ttsl(self) -> IVector4: ...
    @property
    def ttts(self) -> IVector4: ...
    @property
    def tttt(self) -> IVector4: ...
    @property
    def tttq(self) -> IVector4: ...
    @property
    def ttto(self) -> IVector4: ...
    @property
    def tttl(self) -> IVector4: ...
    @property
    def ttqs(self) -> IVector4: ...
    @property
    def ttqt(self) -> IVector4: ...
    @property
    def ttqq(self) -> IVector4: ...
    @property
    def ttqo(self) -> IVector4: ...
    @property
    def ttql(self) -> IVector4: ...
    @property
    def ttos(self) -> IVector4: ...
    @property
    def ttot(self) -> IVector4: ...
    @property
    def ttoq(self) -> IVector4: ...
    @property
    def ttoo(self) -> IVector4: ...
    @property
    def ttol(self) -> IVector4: ...
    @property
    def ttls(self) -> IVector4: ...
    @property
    def ttlt(self) -> IVector4: ...
    @property
    def ttlq(self) -> IVector4: ...
    @property
    def ttlo(self) -> IVector4: ...
    @property
    def ttll(self) -> IVector4: ...
    @property
    def tqss(self) -> IVector4: ...
    @property
    def tqst(self) -> IVector4: ...
    @property
    def tqsq(self) -> IVector4: ...
    @property
    def tqso(self) -> IVector4: ...
    @property
    def tqsl(self) -> IVector4: ...
    @property
    def tqts(self) -> IVector4: ...
    @property
    def tqtt(self) -> IVector4: ...
    @property
    def tqtq(self) -> IVector4: ...
    @property
    def tqto(self) -> IVector4: ...
    @property
    def tqtl(self) -> IVector4: ...
    @property
    def tqqs(self) -> IVector4: ...
    @property
    def tqqt(self) -> IVector4: ...
    @property
    def tqqq(self) -> IVector4: ...
    @property
    def tqqo(self) -> IVector4: ...
    @property
    def tqql(self) -> IVector4: ...
    @property
    def tqos(self) -> IVector4: ...
    @property
    def tqot(self) -> IVector4: ...
    @property
    def tqoq(self) -> IVector4: ...
    @property
    def tqoo(self) -> IVector4: ...
    @property
    def tqol(self) -> IVector4: ...
    @property
    def tqls(self) -> IVector4: ...
    @property
    def tqlt(self) -> IVector4: ...
    @property
    def tqlq(self) -> IVector4: ...
    @property
    def tqlo(self) -> IVector4: ...
    @property
    def tqll(self) -> IVector4: ...
    @property
    def toss(self) -> IVector4: ...
    @property
    def tost(self) -> IVector4: ...
    @property
    def tosq(self) -> IVector4: ...
    @property
    def toso(self) -> IVector4: ...
    @property
    def tosl(self) -> IVector4: ...
    @property
    def tots(self) -> IVector4: ...
    @property
    def tott(self) -> IVector4: ...
    @property
    def totq(self) -> IVector4: ...
    @property
    def toto(self) -> IVector4: ...
    @property
    def totl(self) -> IVector4: ...
    @property
    def toqs(self) -> IVector4: ...
    @property
    def toqt(self) -> IVector4: ...
    @property
    def toqq(self) -> IVector4: ...
    @property
    def toqo(self) -> IVector4: ...
    @property
    def toql(self) -> IVector4: ...
    @property
    def toos(self) -> IVector4: ...
    @property
    def toot(self) -> IVector4: ...
    @property
    def tooq(self) -> IVector4: ...
    @property
    def tooo(self) -> IVector4: ...
    @property
    def tool(self) -> IVector4: ...
    @property
    def tols(self) -> IVector4: ...
    @property
    def tolt(self) -> IVector4: ...
    @property
    def tolq(self) -> IVector4: ...
    @property
    def tolo(self) -> IVector4: ...
    @property
    def toll(self) -> IVector4: ...
    @property
    def tlss(self) -> IVector4: ...
    @property
    def tlst(self) -> IVector4: ...
    @property
    def tlsq(self) -> IVector4: ...
    @property
    def tlso(self) -> IVector4: ...
    @property
    def tlsl(self) -> IVector4: ...
    @property
    def tlts(self) -> IVector4: ...
    @property
    def tltt(self) -> IVector4: ...
    @property
    def tltq(self) -> IVector4: ...
    @property
    def tlto(self) -> IVector4: ...
    @property
    def tltl(self) -> IVector4: ...
    @property
    def tlqs(self) -> IVector4: ...
    @property
    def tlqt(self) -> IVector4: ...
    @property
    def tlqq(self) -> IVector4: ...
    @property
    def tlqo(self) -> IVector4: ...
    @property
    def tlql(self) -> IVector4: ...
    @property
    def tlos(self) -> IVector4: ...
    @property
    def tlot(self) -> IVector4: ...
    @property
    def tloq(self) -> IVector4: ...
    @property
    def tloo(self) -> IVector4: ...
    @property
    def tlol(self) -> IVector4: ...
    @property
    def tlls(self) -> IVector4: ...
    @property
    def tllt(self) -> IVector4: ...
    @property
    def tllq(self) -> IVector4: ...
    @property
    def tllo(self) -> IVector4: ...
    @property
    def tlll(self) -> IVector4: ...
    @property
    def qsss(self) -> IVector4: ...
    @property
    def qsst(self) -> IVector4: ...
    @property
    def qssq(self) -> IVector4: ...
    @property
    def qsso(self) -> IVector4: ...
    @property
    def qssl(self) -> IVector4: ...
    @property
    def qsts(self) -> IVector4: ...
    @property
    def qstt(self) -> IVector4: ...
    @property
    def qstq(self) -> IVector4: ...
    @property
    def qsto(self) -> IVector4: ...
    @property
    def qstl(self) -> IVector4: ...
    @property
    def qsqs(self) -> IVector4: ...
    @property
    def qsqt(self) -> IVector4: ...
    @property
    def qsqq(self) -> IVector4: ...
    @property
    def qsqo(self) -> IVector4: ...
    @property
    def qsql(self) -> IVector4: ...
    @property
    def qsos(self) -> IVector4: ...
    @property
    def qsot(self) -> IVector4: ...
    @property
    def qsoq(self) -> IVector4: ...
    @property
    def qsoo(self) -> IVector4: ...
    @property
    def qsol(self) -> IVector4: ...
    @property
    def qsls(self) -> IVector4: ...
    @property
    def qslt(self) -> IVector4: ...
    @property
    def qslq(self) -> IVector4: ...
    @property
    def qslo(self) -> IVector4: ...
    @property
    def qsll(self) -> IVector4: ...
    @property
    def qtss(self) -> IVector4: ...
    @property
    def qtst(self) -> IVector4: ...
    @property
    def qtsq(self) -> IVector4: ...
    @property
    def qtso(self) -> IVector4: ...
    @property
    def qtsl(self) -> IVector4: ...
    @property
    def qtts(self) -> IVector4: ...
    @property
    def qttt(self) -> IVector4: ...
    @property
    def qttq(self) -> IVector4: ...
    @property
    def qtto(self) -> IVector4: ...
    @property
    def qttl(self) -> IVector4: ...
    @property
    def qtqs(self) -> IVector4: ...
    @property
    def qtqt(self) -> IVector4: ...
    @property
    def qtqq(self) -> IVector4: ...
    @property
    def qtqo(self) -> IVector4: ...
    @property
    def qtql(self) -> IVector4: ...
    @property
    def qtos(self) -> IVector4: ...
    @property
    def qtot(self) -> IVector4: ...
    @property
    def qtoq(self) -> IVector4: ...
    @property
    def qtoo(self) -> IVector4: ...
    @property
    def qtol(self) -> IVector4: ...
    @property
    def qtls(self) -> IVector4: ...
    @property
    def qtlt(self) -> IVector4: ...
    @property
    def qtlq(self) -> IVector4: ...
    @property
    def qtlo(self) -> IVector4: ...
    @property
    def qtll(self) -> IVector4: ...
    @property
    def qqss(self) -> IVector4: ...
    @property
    def qqst(self) -> IVector4: ...
    @property
    def qqsq(self) -> IVector4: ...
    @property
    def qqso(self) -> IVector4: ...
    @property
    def qqsl(self) -> IVector4: ...
    @property
    def qqts(self) -> IVector4: ...
    @property
    def qqtt(self) -> IVector4: ...
    @property
    def qqtq(self) -> IVector4: ...
    @property
    def qqto(self) -> IVector4: ...
    @property
    def qqtl(self) -> IVector4: ...
    @property
    def qqqs(self) -> IVector4: ...
    @property
    def qqqt(self) -> IVector4: ...
    @property
    def qqqq(self) -> IVector4: ...
    @property
    def qqqo(self) -> IVector4: ...
    @property
    def qqql(self) -> IVector4: ...
    @property
    def qqos(self) -> IVector4: ...
    @property
    def qqot(self) -> IVector4: ...
    @property
    def qqoq(self) -> IVector4: ...
    @property
    def qqoo(self) -> IVector4: ...
    @property
    def qqol(self) -> IVector4: ...
    @property
    def qqls(self) -> IVector4: ...
    @property
    def qqlt(self) -> IVector4: ...
    @property
    def qqlq(self) -> IVector4: ...
    @property
    def qqlo(self) -> IVector4: ...
    @property
    def qqll(self) -> IVector4: ...
    @property
    def qoss(self) -> IVector4: ...
    @property
    def qost(self) -> IVector4: ...
    @property
    def qosq(self) -> IVector4: ...
    @property
    def qoso(self) -> IVector4: ...
    @property
    def qosl(self) -> IVector4: ...
    @property
    def qots(self) -> IVector4: ...
    @property
    def qott(self) -> IVector4: ...
    @property
    def qotq(self) -> IVector4: ...
    @property
    def qoto(self) -> IVector4: ...
    @property
    def qotl(self) -> IVector4: ...
    @property
    def qoqs(self) -> IVector4: ...
    @property
    def qoqt(self) -> IVector4: ...
    @property
    def qoqq(self) -> IVector4: ...
    @property
    def qoqo(self) -> IVector4: ...
    @property
    def qoql(self) -> IVector4: ...
    @property
    def qoos(self) -> IVector4: ...
    @property
    def qoot(self) -> IVector4: ...
    @property
    def qooq(self) -> IVector4: ...
    @property
    def qooo(self) -> IVector4: ...
    @property
    def qool(self) -> IVector4: ...
    @property
    def qols(self) -> IVector4: ...
    @property
    def qolt(self) -> IVector4: ...
    @property
    def qolq(self) -> IVector4: ...
    @property
    def qolo(self) -> IVector4: ...
    @property
    def qoll(self) -> IVector4: ...
    @property
    def qlss(self) -> IVector4: ...
    @property
    def qlst(self) -> IVector4: ...
    @property
    def qlsq(self) -> IVector4: ...
    @property
    def qlso(self) -> IVector4: ...
    @property
    def qlsl(self) -> IVector4: ...
    @property
    def qlts(self) -> IVector4: ...
    @property
    def qltt(self) -> IVector4: ...
    @property
    def qltq(self) -> IVector4: ...
    @property
    def qlto(self) -> IVector4: ...
    @property
    def qltl(self) -> IVector4: ...
    @property
    def qlqs(self) -> IVector4: ...
    @property
    def qlqt(self) -> IVector4: ...
    @property
    def qlqq(self) -> IVector4: ...
    @property
    def qlqo(self) -> IVector4: ...
    @property
    def qlql(self) -> IVector4: ...
    @property
    def qlos(self) -> IVector4: ...
    @property
    def qlot(self) -> IVector4: ...
    @property
    def qloq(self) -> IVector4: ...
    @property
    def qloo(self) -> IVector4: ...
    @property
    def qlol(self) -> IVector4: ...
    @property
    def qlls(self) -> IVector4: ...
    @property
    def qllt(self) -> IVector4: ...
    @property
    def qllq(self) -> IVector4: ...
    @property
    def qllo(self) -> IVector4: ...
    @property
    def qlll(self) -> IVector4: ...
    @property
    def osss(self) -> IVector4: ...
    @property
    def osst(self) -> IVector4: ...
    @property
    def ossq(self) -> IVector4: ...
    @property
    def osso(self) -> IVector4: ...
    @property
    def ossl(self) -> IVector4: ...
    @property
    def osts(self) -> IVector4: ...
    @property
    def ostt(self) -> IVector4: ...
    @property
    def ostq(self) -> IVector4: ...
    @property
    def osto(self) -> IVector4: ...
    @property
    def ostl(self) -> IVector4: ...
    @property
    def osqs(self) -> IVector4: ...
    @property
    def osqt(self) -> IVector4: ...
    @property
    def osqq(self) -> IVector4: ...
    @property
    def osqo(self) -> IVector4: ...
    @property
    def osql(self) -> IVector4: ...
    @property
    def osos(self) -> IVector4: ...
    @property
    def osot(self) -> IVector4: ...
    @property
    def osoq(self) -> IVector4: ...
    @property
    def osoo(self) -> IVector4: ...
    @property
    def osol(self) -> IVector4: ...
    @property
    def osls(self) -> IVector4: ...
    @property
    def oslt(self) -> IVector4: ...
    @property
    def oslq(self) -> IVector4: ...
    @property
    def oslo(self) -> IVector4: ...
    @property
    def osll(self) -> IVector4: ...
    @property
    def otss(self) -> IVector4: ...
    @property
    def otst(self) -> IVector4: ...
    @property
    def otsq(self) -> IVector4: ...
    @property
    def otso(self) -> IVector4: ...
    @property
    def otsl(self) -> IVector4: ...
    @property
    def otts(self) -> IVector4: ...
    @property
    def ottt(self) -> IVector4: ...
    @property
    def ottq(self) -> IVector4: ...
    @property
    def otto(self) -> IVector4: ...
    @property
    def ottl(self) -> IVector4: ...
    @property
    def otqs(self) -> IVector4: ...
    @property
    def otqt(self) -> IVector4: ...
    @property
    def otqq(self) -> IVector4: ...
    @property
    def otqo(self) -> IVector4: ...
    @property
    def otql(self) -> IVector4: ...
    @property
    def otos(self) -> IVector4: ...
    @property
    def otot(self) -> IVector4: ...
    @property
    def otoq(self) -> IVector4: ...
    @property
    def otoo(self) -> IVector4: ...
    @property
    def otol(self) -> IVector4: ...
    @property
    def otls(self) -> IVector4: ...
    @property
    def otlt(self) -> IVector4: ...
    @property
    def otlq(self) -> IVector4: ...
    @property
    def otlo(self) -> IVector4: ...
    @property
    def otll(self) -> IVector4: ...
    @property
    def oqss(self) -> IVector4: ...
    @property
    def oqst(self) -> IVector4: ...
    @property
    def oqsq(self) -> IVector4: ...
    @property
    def oqso(self) -> IVector4: ...
    @property
    def oqsl(self) -> IVector4: ...
    @property
    def oqts(self) -> IVector4: ...
    @property
    def oqtt(self) -> IVector4: ...
    @property
    def oqtq(self) -> IVector4: ...
    @property
    def oqto(self) -> IVector4: ...
    @property
    def oqtl(self) -> IVector4: ...
    @property
    def oqqs(self) -> IVector4: ...
    @property
    def oqqt(self) -> IVector4: ...
    @property
    def oqqq(self) -> IVector4: ...
    @property
    def oqqo(self) -> IVector4: ...
    @property
    def oqql(self) -> IVector4: ...
    @property
    def oqos(self) -> IVector4: ...
    @property
    def oqot(self) -> IVector4: ...
    @property
    def oqoq(self) -> IVector4: ...
    @property
    def oqoo(self) -> IVector4: ...
    @property
    def oqol(self) -> IVector4: ...
    @property
    def oqls(self) -> IVector4: ...
    @property
    def oqlt(self) -> IVector4: ...
    @property
    def oqlq(self) -> IVector4: ...
    @property
    def oqlo(self) -> IVector4: ...
    @property
    def oqll(self) -> IVector4: ...
    @property
    def ooss(self) -> IVector4: ...
    @property
    def oost(self) -> IVector4: ...
    @property
    def oosq(self) -> IVector4: ...
    @property
    def ooso(self) -> IVector4: ...
    @property
    def oosl(self) -> IVector4: ...
    @property
    def oots(self) -> IVector4: ...
    @property
    def oott(self) -> IVector4: ...
    @property
    def ootq(self) -> IVector4: ...
    @property
    def ooto(self) -> IVector4: ...
    @property
    def ootl(self) -> IVector4: ...
    @property
    def ooqs(self) -> IVector4: ...
    @property
    def ooqt(self) -> IVector4: ...
    @property
    def ooqq(self) -> IVector4: ...
    @property
    def ooqo(self) -> IVector4: ...
    @property
    def ooql(self) -> IVector4: ...
    @property
    def ooos(self) -> IVector4: ...
    @property
    def ooot(self) -> IVector4: ...
    @property
    def oooq(self) -> IVector4: ...
    @property
    def ools(self) -> IVector4: ...
    @property
    def oolt(self) -> IVector4: ...
    @property
    def oolq(self) -> IVector4: ...
    @property
    def olss(self) -> IVector4: ...
    @property
    def olst(self) -> IVector4: ...
    @property
    def olsq(self) -> IVector4: ...
    @property
    def olso(self) -> IVector4: ...
    @property
    def olsl(self) -> IVector4: ...
    @property
    def olts(self) -> IVector4: ...
    @property
    def oltt(self) -> IVector4: ...
    @property
    def oltq(self) -> IVector4: ...
    @property
    def olto(self) -> IVector4: ...
    @property
    def oltl(self) -> IVector4: ...
    @property
    def olqs(self) -> IVector4: ...
    @property
    def olqt(self) -> IVector4: ...
    @property
    def olqq(self) -> IVector4: ...
    @property
    def olqo(self) -> IVector4: ...
    @property
    def olql(self) -> IVector4: ...
    @property
    def olos(self) -> IVector4: ...
    @property
    def olot(self) -> IVector4: ...
    @property
    def oloq(self) -> IVector4: ...
    @property
    def olls(self) -> IVector4: ...
    @property
    def ollt(self) -> IVector4: ...
    @property
    def ollq(self) -> IVector4: ...
    @property
    def lsss(self) -> IVector4: ...
    @property
    def lsst(self) -> IVector4: ...
    @property
    def lssq(self) -> IVector4: ...
    @property
    def lsso(self) -> IVector4: ...
    @property
    def lssl(self) -> IVector4: ...
    @property
    def lsts(self) -> IVector4: ...
    @property
    def lstt(self) -> IVector4: ...
    @property
    def lstq(self) -> IVector4: ...
    @property
    def lsto(self) -> IVector4: ...
    @property
    def lstl(self) -> IVector4: ...
    @property
    def lsqs(self) -> IVector4: ...
    @property
    def lsqt(self) -> IVector4: ...
    @property
    def lsqq(self) -> IVector4: ...
    @property
    def lsqo(self) -> IVector4: ...
    @property
    def lsql(self) -> IVector4: ...
    @property
    def lsos(self) -> IVector4: ...
    @property
    def lsot(self) -> IVector4: ...
    @property
    def lsoq(self) -> IVector4: ...
    @property
    def lsoo(self) -> IVector4: ...
    @property
    def lsol(self) -> IVector4: ...
    @property
    def lsls(self) -> IVector4: ...
    @property
    def lslt(self) -> IVector4: ...
    @property
    def lslq(self) -> IVector4: ...
    @property
    def lslo(self) -> IVector4: ...
    @property
    def lsll(self) -> IVector4: ...
    @property
    def ltss(self) -> IVector4: ...
    @property
    def ltst(self) -> IVector4: ...
    @property
    def ltsq(self) -> IVector4: ...
    @property
    def ltso(self) -> IVector4: ...
    @property
    def ltsl(self) -> IVector4: ...
    @property
    def ltts(self) -> IVector4: ...
    @property
    def lttt(self) -> IVector4: ...
    @property
    def lttq(self) -> IVector4: ...
    @property
    def ltto(self) -> IVector4: ...
    @property
    def lttl(self) -> IVector4: ...
    @property
    def ltqs(self) -> IVector4: ...
    @property
    def ltqt(self) -> IVector4: ...
    @property
    def ltqq(self) -> IVector4: ...
    @property
    def ltqo(self) -> IVector4: ...
    @property
    def ltql(self) -> IVector4: ...
    @property
    def ltos(self) -> IVector4: ...
    @property
    def ltot(self) -> IVector4: ...
    @property
    def ltoq(self) -> IVector4: ...
    @property
    def ltoo(self) -> IVector4: ...
    @property
    def ltol(self) -> IVector4: ...
    @property
    def ltls(self) -> IVector4: ...
    @property
    def ltlt(self) -> IVector4: ...
    @property
    def ltlq(self) -> IVector4: ...
    @property
    def ltlo(self) -> IVector4: ...
    @property
    def ltll(self) -> IVector4: ...
    @property
    def lqss(self) -> IVector4: ...
    @property
    def lqst(self) -> IVector4: ...
    @property
    def lqsq(self) -> IVector4: ...
    @property
    def lqso(self) -> IVector4: ...
    @property
    def lqsl(self) -> IVector4: ...
    @property
    def lqts(self) -> IVector4: ...
    @property
    def lqtt(self) -> IVector4: ...
    @property
    def lqtq(self) -> IVector4: ...
    @property
    def lqto(self) -> IVector4: ...
    @property
    def lqtl(self) -> IVector4: ...
    @property
    def lqqs(self) -> IVector4: ...
    @property
    def lqqt(self) -> IVector4: ...
    @property
    def lqqq(self) -> IVector4: ...
    @property
    def lqqo(self) -> IVector4: ...
    @property
    def lqql(self) -> IVector4: ...
    @property
    def lqos(self) -> IVector4: ...
    @property
    def lqot(self) -> IVector4: ...
    @property
    def lqoq(self) -> IVector4: ...
    @property
    def lqoo(self) -> IVector4: ...
    @property
    def lqol(self) -> IVector4: ...
    @property
    def lqls(self) -> IVector4: ...
    @property
    def lqlt(self) -> IVector4: ...
    @property
    def lqlq(self) -> IVector4: ...
    @property
    def lqlo(self) -> IVector4: ...
    @property
    def lqll(self) -> IVector4: ...
    @property
    def loss(self) -> IVector4: ...
    @property
    def lost(self) -> IVector4: ...
    @property
    def losq(self) -> IVector4: ...
    @property
    def loso(self) -> IVector4: ...
    @property
    def losl(self) -> IVector4: ...
    @property
    def lots(self) -> IVector4: ...
    @property
    def lott(self) -> IVector4: ...
    @property
    def lotq(self) -> IVector4: ...
    @property
    def loto(self) -> IVector4: ...
    @property
    def lotl(self) -> IVector4: ...
    @property
    def loqs(self) -> IVector4: ...
    @property
    def loqt(self) -> IVector4: ...
    @property
    def loqq(self) -> IVector4: ...
    @property
    def loqo(self) -> IVector4: ...
    @property
    def loql(self) -> IVector4: ...
    @property
    def loos(self) -> IVector4: ...
    @property
    def loot(self) -> IVector4: ...
    @property
    def looq(self) -> IVector4: ...
    @property
    def lols(self) -> IVector4: ...
    @property
    def lolt(self) -> IVector4: ...
    @property
    def lolq(self) -> IVector4: ...
    @property
    def llss(self) -> IVector4: ...
    @property
    def llst(self) -> IVector4: ...
    @property
    def llsq(self) -> IVector4: ...
    @property
    def llso(self) -> IVector4: ...
    @property
    def llsl(self) -> IVector4: ...
    @property
    def llts(self) -> IVector4: ...
    @property
    def lltt(self) -> IVector4: ...
    @property
    def lltq(self) -> IVector4: ...
    @property
    def llto(self) -> IVector4: ...
    @property
    def lltl(self) -> IVector4: ...
    @property
    def llqs(self) -> IVector4: ...
    @property
    def llqt(self) -> IVector4: ...
    @property
    def llqq(self) -> IVector4: ...
    @property
    def llqo(self) -> IVector4: ...
    @property
    def llql(self) -> IVector4: ...
    @property
    def llos(self) -> IVector4: ...
    @property
    def llot(self) -> IVector4: ...
    @property
    def lloq(self) -> IVector4: ...
    @property
    def llls(self) -> IVector4: ...
    @property
    def lllt(self) -> IVector4: ...
    @property
    def lllq(self) -> IVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> IVector2: ...
    @property
    def uv(self) -> IVector2: ...
    @property
    def uo(self) -> IVector2: ...
    @property
    def ul(self) -> IVector2: ...
    @property
    def vu(self) -> IVector2: ...
    @property
    def vv(self) -> IVector2: ...
    @property
    def vo(self) -> IVector2: ...
    @property
    def vl(self) -> IVector2: ...
    @property
    def ou(self) -> IVector2: ...
    @property
    def ov(self) -> IVector2: ...
    @property
    def lu(self) -> IVector2: ...
    @property
    def lv(self) -> IVector2: ...
    @property
    def uuu(self) -> IVector3: ...
    @property
    def uuv(self) -> IVector3: ...
    @property
    def uuo(self) -> IVector3: ...
    @property
    def uul(self) -> IVector3: ...
    @property
    def uvu(self) -> IVector3: ...
    @property
    def uvv(self) -> IVector3: ...
    @property
    def uvo(self) -> IVector3: ...
    @property
    def uvl(self) -> IVector3: ...
    @property
    def uou(self) -> IVector3: ...
    @property
    def uov(self) -> IVector3: ...
    @property
    def uoo(self) -> IVector3: ...
    @property
    def uol(self) -> IVector3: ...
    @property
    def ulu(self) -> IVector3: ...
    @property
    def ulv(self) -> IVector3: ...
    @property
    def ulo(self) -> IVector3: ...
    @property
    def ull(self) -> IVector3: ...
    @property
    def vuu(self) -> IVector3: ...
    @property
    def vuv(self) -> IVector3: ...
    @property
    def vuo(self) -> IVector3: ...
    @property
    def vul(self) -> IVector3: ...
    @property
    def vvu(self) -> IVector3: ...
    @property
    def vvv(self) -> IVector3: ...
    @property
    def vvo(self) -> IVector3: ...
    @property
    def vvl(self) -> IVector3: ...
    @property
    def vou(self) -> IVector3: ...
    @property
    def vov(self) -> IVector3: ...
    @property
    def voo(self) -> IVector3: ...
    @property
    def vol(self) -> IVector3: ...
    @property
    def vlu(self) -> IVector3: ...
    @property
    def vlv(self) -> IVector3: ...
    @property
    def vlo(self) -> IVector3: ...
    @property
    def vll(self) -> IVector3: ...
    @property
    def ouu(self) -> IVector3: ...
    @property
    def ouv(self) -> IVector3: ...
    @property
    def ouo(self) -> IVector3: ...
    @property
    def oul(self) -> IVector3: ...
    @property
    def ovu(self) -> IVector3: ...
    @property
    def ovv(self) -> IVector3: ...
    @property
    def ovo(self) -> IVector3: ...
    @property
    def ovl(self) -> IVector3: ...
    @property
    def oou(self) -> IVector3: ...
    @property
    def oov(self) -> IVector3: ...
    @property
    def olu(self) -> IVector3: ...
    @property
    def olv(self) -> IVector3: ...
    @property
    def luu(self) -> IVector3: ...
    @property
    def luv(self) -> IVector3: ...
    @property
    def luo(self) -> IVector3: ...
    @property
    def lul(self) -> IVector3: ...
    @property
    def lvu(self) -> IVector3: ...
    @property
    def lvv(self) -> IVector3: ...
    @property
    def lvo(self) -> IVector3: ...
    @property
    def lvl(self) -> IVector3: ...
    @property
    def lou(self) -> IVector3: ...
    @property
    def lov(self) -> IVector3: ...
    @property
    def llu(self) -> IVector3: ...
    @property
    def llv(self) -> IVector3: ...
    @property
    def uuuu(self) -> IVector4: ...
    @property
    def uuuv(self) -> IVector4: ...
    @property
    def uuuo(self) -> IVector4: ...
    @property
    def uuul(self) -> IVector4: ...
    @property
    def uuvu(self) -> IVector4: ...
    @property
    def uuvv(self) -> IVector4: ...
    @property
    def uuvo(self) -> IVector4: ...
    @property
    def uuvl(self) -> IVector4: ...
    @property
    def uuou(self) -> IVector4: ...
    @property
    def uuov(self) -> IVector4: ...
    @property
    def uuoo(self) -> IVector4: ...
    @property
    def uuol(self) -> IVector4: ...
    @property
    def uulu(self) -> IVector4: ...
    @property
    def uulv(self) -> IVector4: ...
    @property
    def uulo(self) -> IVector4: ...
    @property
    def uull(self) -> IVector4: ...
    @property
    def uvuu(self) -> IVector4: ...
    @property
    def uvuv(self) -> IVector4: ...
    @property
    def uvuo(self) -> IVector4: ...
    @property
    def uvul(self) -> IVector4: ...
    @property
    def uvvu(self) -> IVector4: ...
    @property
    def uvvv(self) -> IVector4: ...
    @property
    def uvvo(self) -> IVector4: ...
    @property
    def uvvl(self) -> IVector4: ...
    @property
    def uvou(self) -> IVector4: ...
    @property
    def uvov(self) -> IVector4: ...
    @property
    def uvoo(self) -> IVector4: ...
    @property
    def uvol(self) -> IVector4: ...
    @property
    def uvlu(self) -> IVector4: ...
    @property
    def uvlv(self) -> IVector4: ...
    @property
    def uvlo(self) -> IVector4: ...
    @property
    def uvll(self) -> IVector4: ...
    @property
    def uouu(self) -> IVector4: ...
    @property
    def uouv(self) -> IVector4: ...
    @property
    def uouo(self) -> IVector4: ...
    @property
    def uoul(self) -> IVector4: ...
    @property
    def uovu(self) -> IVector4: ...
    @property
    def uovv(self) -> IVector4: ...
    @property
    def uovo(self) -> IVector4: ...
    @property
    def uovl(self) -> IVector4: ...
    @property
    def uoou(self) -> IVector4: ...
    @property
    def uoov(self) -> IVector4: ...
    @property
    def uooo(self) -> IVector4: ...
    @property
    def uool(self) -> IVector4: ...
    @property
    def uolu(self) -> IVector4: ...
    @property
    def uolv(self) -> IVector4: ...
    @property
    def uolo(self) -> IVector4: ...
    @property
    def uoll(self) -> IVector4: ...
    @property
    def uluu(self) -> IVector4: ...
    @property
    def uluv(self) -> IVector4: ...
    @property
    def uluo(self) -> IVector4: ...
    @property
    def ulul(self) -> IVector4: ...
    @property
    def ulvu(self) -> IVector4: ...
    @property
    def ulvv(self) -> IVector4: ...
    @property
    def ulvo(self) -> IVector4: ...
    @property
    def ulvl(self) -> IVector4: ...
    @property
    def ulou(self) -> IVector4: ...
    @property
    def ulov(self) -> IVector4: ...
    @property
    def uloo(self) -> IVector4: ...
    @property
    def ulol(self) -> IVector4: ...
    @property
    def ullu(self) -> IVector4: ...
    @property
    def ullv(self) -> IVector4: ...
    @property
    def ullo(self) -> IVector4: ...
    @property
    def ulll(self) -> IVector4: ...
    @property
    def vuuu(self) -> IVector4: ...
    @property
    def vuuv(self) -> IVector4: ...
    @property
    def vuuo(self) -> IVector4: ...
    @property
    def vuul(self) -> IVector4: ...
    @property
    def vuvu(self) -> IVector4: ...
    @property
    def vuvv(self) -> IVector4: ...
    @property
    def vuvo(self) -> IVector4: ...
    @property
    def vuvl(self) -> IVector4: ...
    @property
    def vuou(self) -> IVector4: ...
    @property
    def vuov(self) -> IVector4: ...
    @property
    def vuoo(self) -> IVector4: ...
    @property
    def vuol(self) -> IVector4: ...
    @property
    def vulu(self) -> IVector4: ...
    @property
    def vulv(self) -> IVector4: ...
    @property
    def vulo(self) -> IVector4: ...
    @property
    def vull(self) -> IVector4: ...
    @property
    def vvuu(self) -> IVector4: ...
    @property
    def vvuv(self) -> IVector4: ...
    @property
    def vvuo(self) -> IVector4: ...
    @property
    def vvul(self) -> IVector4: ...
    @property
    def vvvu(self) -> IVector4: ...
    @property
    def vvvv(self) -> IVector4: ...
    @property
    def vvvo(self) -> IVector4: ...
    @property
    def vvvl(self) -> IVector4: ...
    @property
    def vvou(self) -> IVector4: ...
    @property
    def vvov(self) -> IVector4: ...
    @property
    def vvoo(self) -> IVector4: ...
    @property
    def vvol(self) -> IVector4: ...
    @property
    def vvlu(self) -> IVector4: ...
    @property
    def vvlv(self) -> IVector4: ...
    @property
    def vvlo(self) -> IVector4: ...
    @property
    def vvll(self) -> IVector4: ...
    @property
    def vouu(self) -> IVector4: ...
    @property
    def vouv(self) -> IVector4: ...
    @property
    def vouo(self) -> IVector4: ...
    @property
    def voul(self) -> IVector4: ...
    @property
    def vovu(self) -> IVector4: ...
    @property
    def vovv(self) -> IVector4: ...
    @property
    def vovo(self) -> IVector4: ...
    @property
    def vovl(self) -> IVector4: ...
    @property
    def voou(self) -> IVector4: ...
    @property
    def voov(self) -> IVector4: ...
    @property
    def vooo(self) -> IVector4: ...
    @property
    def vool(self) -> IVector4: ...
    @property
    def volu(self) -> IVector4: ...
    @property
    def volv(self) -> IVector4: ...
    @property
    def volo(self) -> IVector4: ...
    @property
    def voll(self) -> IVector4: ...
    @property
    def vluu(self) -> IVector4: ...
    @property
    def vluv(self) -> IVector4: ...
    @property
    def vluo(self) -> IVector4: ...
    @property
    def vlul(self) -> IVector4: ...
    @property
    def vlvu(self) -> IVector4: ...
    @property
    def vlvv(self) -> IVector4: ...
    @property
    def vlvo(self) -> IVector4: ...
    @property
    def vlvl(self) -> IVector4: ...
    @property
    def vlou(self) -> IVector4: ...
    @property
    def vlov(self) -> IVector4: ...
    @property
    def vloo(self) -> IVector4: ...
    @property
    def vlol(self) -> IVector4: ...
    @property
    def vllu(self) -> IVector4: ...
    @property
    def vllv(self) -> IVector4: ...
    @property
    def vllo(self) -> IVector4: ...
    @property
    def vlll(self) -> IVector4: ...
    @property
    def ouuu(self) -> IVector4: ...
    @property
    def ouuv(self) -> IVector4: ...
    @property
    def ouuo(self) -> IVector4: ...
    @property
    def ouul(self) -> IVector4: ...
    @property
    def ouvu(self) -> IVector4: ...
    @property
    def ouvv(self) -> IVector4: ...
    @property
    def ouvo(self) -> IVector4: ...
    @property
    def ouvl(self) -> IVector4: ...
    @property
    def ouou(self) -> IVector4: ...
    @property
    def ouov(self) -> IVector4: ...
    @property
    def ouoo(self) -> IVector4: ...
    @property
    def ouol(self) -> IVector4: ...
    @property
    def oulu(self) -> IVector4: ...
    @property
    def oulv(self) -> IVector4: ...
    @property
    def oulo(self) -> IVector4: ...
    @property
    def oull(self) -> IVector4: ...
    @property
    def ovuu(self) -> IVector4: ...
    @property
    def ovuv(self) -> IVector4: ...
    @property
    def ovuo(self) -> IVector4: ...
    @property
    def ovul(self) -> IVector4: ...
    @property
    def ovvu(self) -> IVector4: ...
    @property
    def ovvv(self) -> IVector4: ...
    @property
    def ovvo(self) -> IVector4: ...
    @property
    def ovvl(self) -> IVector4: ...
    @property
    def ovou(self) -> IVector4: ...
    @property
    def ovov(self) -> IVector4: ...
    @property
    def ovoo(self) -> IVector4: ...
    @property
    def ovol(self) -> IVector4: ...
    @property
    def ovlu(self) -> IVector4: ...
    @property
    def ovlv(self) -> IVector4: ...
    @property
    def ovlo(self) -> IVector4: ...
    @property
    def ovll(self) -> IVector4: ...
    @property
    def oouu(self) -> IVector4: ...
    @property
    def oouv(self) -> IVector4: ...
    @property
    def oouo(self) -> IVector4: ...
    @property
    def ooul(self) -> IVector4: ...
    @property
    def oovu(self) -> IVector4: ...
    @property
    def oovv(self) -> IVector4: ...
    @property
    def oovo(self) -> IVector4: ...
    @property
    def oovl(self) -> IVector4: ...
    @property
    def ooou(self) -> IVector4: ...
    @property
    def ooov(self) -> IVector4: ...
    @property
    def oolu(self) -> IVector4: ...
    @property
    def oolv(self) -> IVector4: ...
    @property
    def oluu(self) -> IVector4: ...
    @property
    def oluv(self) -> IVector4: ...
    @property
    def oluo(self) -> IVector4: ...
    @property
    def olul(self) -> IVector4: ...
    @property
    def olvu(self) -> IVector4: ...
    @property
    def olvv(self) -> IVector4: ...
    @property
    def olvo(self) -> IVector4: ...
    @property
    def olvl(self) -> IVector4: ...
    @property
    def olou(self) -> IVector4: ...
    @property
    def olov(self) -> IVector4: ...
    @property
    def ollu(self) -> IVector4: ...
    @property
    def ollv(self) -> IVector4: ...
    @property
    def luuu(self) -> IVector4: ...
    @property
    def luuv(self) -> IVector4: ...
    @property
    def luuo(self) -> IVector4: ...
    @property
    def luul(self) -> IVector4: ...
    @property
    def luvu(self) -> IVector4: ...
    @property
    def luvv(self) -> IVector4: ...
    @property
    def luvo(self) -> IVector4: ...
    @property
    def luvl(self) -> IVector4: ...
    @property
    def luou(self) -> IVector4: ...
    @property
    def luov(self) -> IVector4: ...
    @property
    def luoo(self) -> IVector4: ...
    @property
    def luol(self) -> IVector4: ...
    @property
    def lulu(self) -> IVector4: ...
    @property
    def lulv(self) -> IVector4: ...
    @property
    def lulo(self) -> IVector4: ...
    @property
    def lull(self) -> IVector4: ...
    @property
    def lvuu(self) -> IVector4: ...
    @property
    def lvuv(self) -> IVector4: ...
    @property
    def lvuo(self) -> IVector4: ...
    @property
    def lvul(self) -> IVector4: ...
    @property
    def lvvu(self) -> IVector4: ...
    @property
    def lvvv(self) -> IVector4: ...
    @property
    def lvvo(self) -> IVector4: ...
    @property
    def lvvl(self) -> IVector4: ...
    @property
    def lvou(self) -> IVector4: ...
    @property
    def lvov(self) -> IVector4: ...
    @property
    def lvoo(self) -> IVector4: ...
    @property
    def lvol(self) -> IVector4: ...
    @property
    def lvlu(self) -> IVector4: ...
    @property
    def lvlv(self) -> IVector4: ...
    @property
    def lvlo(self) -> IVector4: ...
    @property
    def lvll(self) -> IVector4: ...
    @property
    def louu(self) -> IVector4: ...
    @property
    def louv(self) -> IVector4: ...
    @property
    def louo(self) -> IVector4: ...
    @property
    def loul(self) -> IVector4: ...
    @property
    def lovu(self) -> IVector4: ...
    @property
    def lovv(self) -> IVector4: ...
    @property
    def lovo(self) -> IVector4: ...
    @property
    def lovl(self) -> IVector4: ...
    @property
    def loou(self) -> IVector4: ...
    @property
    def loov(self) -> IVector4: ...
    @property
    def lolu(self) -> IVector4: ...
    @property
    def lolv(self) -> IVector4: ...
    @property
    def lluu(self) -> IVector4: ...
    @property
    def lluv(self) -> IVector4: ...
    @property
    def lluo(self) -> IVector4: ...
    @property
    def llul(self) -> IVector4: ...
    @property
    def llvu(self) -> IVector4: ...
    @property
    def llvv(self) -> IVector4: ...
    @property
    def llvo(self) -> IVector4: ...
    @property
    def llvl(self) -> IVector4: ...
    @property
    def llou(self) -> IVector4: ...
    @property
    def llov(self) -> IVector4: ...
    @property
    def lllu(self) -> IVector4: ...
    @property
    def lllv(self) -> IVector4: ...
    @property
    def oo(self) -> IVector2: ...
    @property
    def ol(self) -> IVector2: ...
    @property
    def lo(self) -> IVector2: ...
    @property
    def ll(self) -> IVector2: ...
    @property
    def ooo(self) -> IVector3: ...
    @property
    def ool(self) -> IVector3: ...
    @property
    def olo(self) -> IVector3: ...
    @property
    def oll(self) -> IVector3: ...
    @property
    def loo(self) -> IVector3: ...
    @property
    def lol(self) -> IVector3: ...
    @property
    def llo(self) -> IVector3: ...
    @property
    def lll(self) -> IVector3: ...
    @property
    def oooo(self) -> IVector4: ...
    @property
    def oool(self) -> IVector4: ...
    @property
    def oolo(self) -> IVector4: ...
    @property
    def ooll(self) -> IVector4: ...
    @property
    def oloo(self) -> IVector4: ...
    @property
    def olol(self) -> IVector4: ...
    @property
    def ollo(self) -> IVector4: ...
    @property
    def olll(self) -> IVector4: ...
    @property
    def looo(self) -> IVector4: ...
    @property
    def lool(self) -> IVector4: ...
    @property
    def lolo(self) -> IVector4: ...
    @property
    def loll(self) -> IVector4: ...
    @property
    def lloo(self) -> IVector4: ...
    @property
    def llol(self) -> IVector4: ...
    @property
    def lllo(self) -> IVector4: ...
    @property
    def llll(self) -> IVector4: ...
    def min(self, n: Number, /) -> IVector3: ...
    def max(self, n: Number, /) -> IVector3: ...
    def clamp(self, min: Number, max: Number, /) -> IVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[IVector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector3: ...

@final
class IVector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: IVector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[IVector3]: ...
    @overload
    def __getitem__(self, index: int) -> IVector3: ...
    @overload
    def __getitem__(self, index: slice) -> IVector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[IVector3]: ...

@final
class UVector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: UVector3) -> UVector3: ...
    @overload
    def __add__(self, other: Number) -> UVector3: ...
    @overload
    def __radd__(self, other: UVector3) -> UVector3: ...
    @overload
    def __radd__(self, other: Number) -> UVector3: ...
    @overload
    def __sub__(self, other: UVector3) -> UVector3: ...
    @overload
    def __sub__(self, other: Number) -> UVector3: ...
    @overload
    def __rsub__(self, other: UVector3) -> UVector3: ...
    @overload
    def __rsub__(self, other: Number) -> UVector3: ...
    @overload
    def __mul__(self, other: UVector3) -> UVector3: ...
    @overload
    def __mul__(self, other: Number) -> UVector3: ...
    @overload
    def __rmul__(self, other: UVector3) -> UVector3: ...
    @overload
    def __rmul__(self, other: Number) -> UVector3: ...
    @overload
    def __truediv__(self, other: UVector3) -> UVector3: ...
    @overload
    def __truediv__(self, other: Number) -> UVector3: ...
    @overload
    def __rtruediv__(self, other: UVector3) -> UVector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> UVector3: ...
    def __abs__(self) -> UVector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> UVector2: ...
    @property
    def xy(self) -> UVector2: ...
    @property
    def xz(self) -> UVector2: ...
    @property
    def xo(self) -> UVector2: ...
    @property
    def xl(self) -> UVector2: ...
    @property
    def yx(self) -> UVector2: ...
    @property
    def yy(self) -> UVector2: ...
    @property
    def yz(self) -> UVector2: ...
    @property
    def yo(self) -> UVector2: ...
    @property
    def yl(self) -> UVector2: ...
    @property
    def zx(self) -> UVector2: ...
    @property
    def zy(self) -> UVector2: ...
    @property
    def zz(self) -> UVector2: ...
    @property
    def zo(self) -> UVector2: ...
    @property
    def zl(self) -> UVector2: ...
    @property
    def ox(self) -> UVector2: ...
    @property
    def oy(self) -> UVector2: ...
    @property
    def oz(self) -> UVector2: ...
    @property
    def lx(self) -> UVector2: ...
    @property
    def ly(self) -> UVector2: ...
    @property
    def lz(self) -> UVector2: ...
    @property
    def xxx(self) -> UVector3: ...
    @property
    def xxy(self) -> UVector3: ...
    @property
    def xxz(self) -> UVector3: ...
    @property
    def xxo(self) -> UVector3: ...
    @property
    def xxl(self) -> UVector3: ...
    @property
    def xyx(self) -> UVector3: ...
    @property
    def xyy(self) -> UVector3: ...
    @property
    def xyz(self) -> UVector3: ...
    @property
    def xyo(self) -> UVector3: ...
    @property
    def xyl(self) -> UVector3: ...
    @property
    def xzx(self) -> UVector3: ...
    @property
    def xzy(self) -> UVector3: ...
    @property
    def xzz(self) -> UVector3: ...
    @property
    def xzo(self) -> UVector3: ...
    @property
    def xzl(self) -> UVector3: ...
    @property
    def xox(self) -> UVector3: ...
    @property
    def xoy(self) -> UVector3: ...
    @property
    def xoz(self) -> UVector3: ...
    @property
    def xoo(self) -> UVector3: ...
    @property
    def xol(self) -> UVector3: ...
    @property
    def xlx(self) -> UVector3: ...
    @property
    def xly(self) -> UVector3: ...
    @property
    def xlz(self) -> UVector3: ...
    @property
    def xlo(self) -> UVector3: ...
    @property
    def xll(self) -> UVector3: ...
    @property
    def yxx(self) -> UVector3: ...
    @property
    def yxy(self) -> UVector3: ...
    @property
    def yxz(self) -> UVector3: ...
    @property
    def yxo(self) -> UVector3: ...
    @property
    def yxl(self) -> UVector3: ...
    @property
    def yyx(self) -> UVector3: ...
    @property
    def yyy(self) -> UVector3: ...
    @property
    def yyz(self) -> UVector3: ...
    @property
    def yyo(self) -> UVector3: ...
    @property
    def yyl(self) -> UVector3: ...
    @property
    def yzx(self) -> UVector3: ...
    @property
    def yzy(self) -> UVector3: ...
    @property
    def yzz(self) -> UVector3: ...
    @property
    def yzo(self) -> UVector3: ...
    @property
    def yzl(self) -> UVector3: ...
    @property
    def yox(self) -> UVector3: ...
    @property
    def yoy(self) -> UVector3: ...
    @property
    def yoz(self) -> UVector3: ...
    @property
    def yoo(self) -> UVector3: ...
    @property
    def yol(self) -> UVector3: ...
    @property
    def ylx(self) -> UVector3: ...
    @property
    def yly(self) -> UVector3: ...
    @property
    def ylz(self) -> UVector3: ...
    @property
    def ylo(self) -> UVector3: ...
    @property
    def yll(self) -> UVector3: ...
    @property
    def zxx(self) -> UVector3: ...
    @property
    def zxy(self) -> UVector3: ...
    @property
    def zxz(self) -> UVector3: ...
    @property
    def zxo(self) -> UVector3: ...
    @property
    def zxl(self) -> UVector3: ...
    @property
    def zyx(self) -> UVector3: ...
    @property
    def zyy(self) -> UVector3: ...
    @property
    def zyz(self) -> UVector3: ...
    @property
    def zyo(self) -> UVector3: ...
    @property
    def zyl(self) -> UVector3: ...
    @property
    def zzx(self) -> UVector3: ...
    @property
    def zzy(self) -> UVector3: ...
    @property
    def zzz(self) -> UVector3: ...
    @property
    def zzo(self) -> UVector3: ...
    @property
    def zzl(self) -> UVector3: ...
    @property
    def zox(self) -> UVector3: ...
    @property
    def zoy(self) -> UVector3: ...
    @property
    def zoz(self) -> UVector3: ...
    @property
    def zoo(self) -> UVector3: ...
    @property
    def zol(self) -> UVector3: ...
    @property
    def zlx(self) -> UVector3: ...
    @property
    def zly(self) -> UVector3: ...
    @property
    def zlz(self) -> UVector3: ...
    @property
    def zlo(self) -> UVector3: ...
    @property
    def zll(self) -> UVector3: ...
    @property
    def oxx(self) -> UVector3: ...
    @property
    def oxy(self) -> UVector3: ...
    @property
    def oxz(self) -> UVector3: ...
    @property
    def oxo(self) -> UVector3: ...
    @property
    def oxl(self) -> UVector3: ...
    @property
    def oyx(self) -> UVector3: ...
    @property
    def oyy(self) -> UVector3: ...
    @property
    def oyz(self) -> UVector3: ...
    @property
    def oyo(self) -> UVector3: ...
    @property
    def oyl(self) -> UVector3: ...
    @property
    def ozx(self) -> UVector3: ...
    @property
    def ozy(self) -> UVector3: ...
    @property
    def ozz(self) -> UVector3: ...
    @property
    def ozo(self) -> UVector3: ...
    @property
    def ozl(self) -> UVector3: ...
    @property
    def oox(self) -> UVector3: ...
    @property
    def ooy(self) -> UVector3: ...
    @property
    def ooz(self) -> UVector3: ...
    @property
    def olx(self) -> UVector3: ...
    @property
    def oly(self) -> UVector3: ...
    @property
    def olz(self) -> UVector3: ...
    @property
    def lxx(self) -> UVector3: ...
    @property
    def lxy(self) -> UVector3: ...
    @property
    def lxz(self) -> UVector3: ...
    @property
    def lxo(self) -> UVector3: ...
    @property
    def lxl(self) -> UVector3: ...
    @property
    def lyx(self) -> UVector3: ...
    @property
    def lyy(self) -> UVector3: ...
    @property
    def lyz(self) -> UVector3: ...
    @property
    def lyo(self) -> UVector3: ...
    @property
    def lyl(self) -> UVector3: ...
    @property
    def lzx(self) -> UVector3: ...
    @property
    def lzy(self) -> UVector3: ...
    @property
    def lzz(self) -> UVector3: ...
    @property
    def lzo(self) -> UVector3: ...
    @property
    def lzl(self) -> UVector3: ...
    @property
    def lox(self) -> UVector3: ...
    @property
    def loy(self) -> UVector3: ...
    @property
    def loz(self) -> UVector3: ...
    @property
    def llx(self) -> UVector3: ...
    @property
    def lly(self) -> UVector3: ...
    @property
    def llz(self) -> UVector3: ...
    @property
    def xxxx(self) -> UVector4: ...
    @property
    def xxxy(self) -> UVector4: ...
    @property
    def xxxz(self) -> UVector4: ...
    @property
    def xxxo(self) -> UVector4: ...
    @property
    def xxxl(self) -> UVector4: ...
    @property
    def xxyx(self) -> UVector4: ...
    @property
    def xxyy(self) -> UVector4: ...
    @property
    def xxyz(self) -> UVector4: ...
    @property
    def xxyo(self) -> UVector4: ...
    @property
    def xxyl(self) -> UVector4: ...
    @property
    def xxzx(self) -> UVector4: ...
    @property
    def xxzy(self) -> UVector4: ...
    @property
    def xxzz(self) -> UVector4: ...
    @property
    def xxzo(self) -> UVector4: ...
    @property
    def xxzl(self) -> UVector4: ...
    @property
    def xxox(self) -> UVector4: ...
    @property
    def xxoy(self) -> UVector4: ...
    @property
    def xxoz(self) -> UVector4: ...
    @property
    def xxoo(self) -> UVector4: ...
    @property
    def xxol(self) -> UVector4: ...
    @property
    def xxlx(self) -> UVector4: ...
    @property
    def xxly(self) -> UVector4: ...
    @property
    def xxlz(self) -> UVector4: ...
    @property
    def xxlo(self) -> UVector4: ...
    @property
    def xxll(self) -> UVector4: ...
    @property
    def xyxx(self) -> UVector4: ...
    @property
    def xyxy(self) -> UVector4: ...
    @property
    def xyxz(self) -> UVector4: ...
    @property
    def xyxo(self) -> UVector4: ...
    @property
    def xyxl(self) -> UVector4: ...
    @property
    def xyyx(self) -> UVector4: ...
    @property
    def xyyy(self) -> UVector4: ...
    @property
    def xyyz(self) -> UVector4: ...
    @property
    def xyyo(self) -> UVector4: ...
    @property
    def xyyl(self) -> UVector4: ...
    @property
    def xyzx(self) -> UVector4: ...
    @property
    def xyzy(self) -> UVector4: ...
    @property
    def xyzz(self) -> UVector4: ...
    @property
    def xyzo(self) -> UVector4: ...
    @property
    def xyzl(self) -> UVector4: ...
    @property
    def xyox(self) -> UVector4: ...
    @property
    def xyoy(self) -> UVector4: ...
    @property
    def xyoz(self) -> UVector4: ...
    @property
    def xyoo(self) -> UVector4: ...
    @property
    def xyol(self) -> UVector4: ...
    @property
    def xylx(self) -> UVector4: ...
    @property
    def xyly(self) -> UVector4: ...
    @property
    def xylz(self) -> UVector4: ...
    @property
    def xylo(self) -> UVector4: ...
    @property
    def xyll(self) -> UVector4: ...
    @property
    def xzxx(self) -> UVector4: ...
    @property
    def xzxy(self) -> UVector4: ...
    @property
    def xzxz(self) -> UVector4: ...
    @property
    def xzxo(self) -> UVector4: ...
    @property
    def xzxl(self) -> UVector4: ...
    @property
    def xzyx(self) -> UVector4: ...
    @property
    def xzyy(self) -> UVector4: ...
    @property
    def xzyz(self) -> UVector4: ...
    @property
    def xzyo(self) -> UVector4: ...
    @property
    def xzyl(self) -> UVector4: ...
    @property
    def xzzx(self) -> UVector4: ...
    @property
    def xzzy(self) -> UVector4: ...
    @property
    def xzzz(self) -> UVector4: ...
    @property
    def xzzo(self) -> UVector4: ...
    @property
    def xzzl(self) -> UVector4: ...
    @property
    def xzox(self) -> UVector4: ...
    @property
    def xzoy(self) -> UVector4: ...
    @property
    def xzoz(self) -> UVector4: ...
    @property
    def xzoo(self) -> UVector4: ...
    @property
    def xzol(self) -> UVector4: ...
    @property
    def xzlx(self) -> UVector4: ...
    @property
    def xzly(self) -> UVector4: ...
    @property
    def xzlz(self) -> UVector4: ...
    @property
    def xzlo(self) -> UVector4: ...
    @property
    def xzll(self) -> UVector4: ...
    @property
    def xoxx(self) -> UVector4: ...
    @property
    def xoxy(self) -> UVector4: ...
    @property
    def xoxz(self) -> UVector4: ...
    @property
    def xoxo(self) -> UVector4: ...
    @property
    def xoxl(self) -> UVector4: ...
    @property
    def xoyx(self) -> UVector4: ...
    @property
    def xoyy(self) -> UVector4: ...
    @property
    def xoyz(self) -> UVector4: ...
    @property
    def xoyo(self) -> UVector4: ...
    @property
    def xoyl(self) -> UVector4: ...
    @property
    def xozx(self) -> UVector4: ...
    @property
    def xozy(self) -> UVector4: ...
    @property
    def xozz(self) -> UVector4: ...
    @property
    def xozo(self) -> UVector4: ...
    @property
    def xozl(self) -> UVector4: ...
    @property
    def xoox(self) -> UVector4: ...
    @property
    def xooy(self) -> UVector4: ...
    @property
    def xooz(self) -> UVector4: ...
    @property
    def xooo(self) -> UVector4: ...
    @property
    def xool(self) -> UVector4: ...
    @property
    def xolx(self) -> UVector4: ...
    @property
    def xoly(self) -> UVector4: ...
    @property
    def xolz(self) -> UVector4: ...
    @property
    def xolo(self) -> UVector4: ...
    @property
    def xoll(self) -> UVector4: ...
    @property
    def xlxx(self) -> UVector4: ...
    @property
    def xlxy(self) -> UVector4: ...
    @property
    def xlxz(self) -> UVector4: ...
    @property
    def xlxo(self) -> UVector4: ...
    @property
    def xlxl(self) -> UVector4: ...
    @property
    def xlyx(self) -> UVector4: ...
    @property
    def xlyy(self) -> UVector4: ...
    @property
    def xlyz(self) -> UVector4: ...
    @property
    def xlyo(self) -> UVector4: ...
    @property
    def xlyl(self) -> UVector4: ...
    @property
    def xlzx(self) -> UVector4: ...
    @property
    def xlzy(self) -> UVector4: ...
    @property
    def xlzz(self) -> UVector4: ...
    @property
    def xlzo(self) -> UVector4: ...
    @property
    def xlzl(self) -> UVector4: ...
    @property
    def xlox(self) -> UVector4: ...
    @property
    def xloy(self) -> UVector4: ...
    @property
    def xloz(self) -> UVector4: ...
    @property
    def xloo(self) -> UVector4: ...
    @property
    def xlol(self) -> UVector4: ...
    @property
    def xllx(self) -> UVector4: ...
    @property
    def xlly(self) -> UVector4: ...
    @property
    def xllz(self) -> UVector4: ...
    @property
    def xllo(self) -> UVector4: ...
    @property
    def xlll(self) -> UVector4: ...
    @property
    def yxxx(self) -> UVector4: ...
    @property
    def yxxy(self) -> UVector4: ...
    @property
    def yxxz(self) -> UVector4: ...
    @property
    def yxxo(self) -> UVector4: ...
    @property
    def yxxl(self) -> UVector4: ...
    @property
    def yxyx(self) -> UVector4: ...
    @property
    def yxyy(self) -> UVector4: ...
    @property
    def yxyz(self) -> UVector4: ...
    @property
    def yxyo(self) -> UVector4: ...
    @property
    def yxyl(self) -> UVector4: ...
    @property
    def yxzx(self) -> UVector4: ...
    @property
    def yxzy(self) -> UVector4: ...
    @property
    def yxzz(self) -> UVector4: ...
    @property
    def yxzo(self) -> UVector4: ...
    @property
    def yxzl(self) -> UVector4: ...
    @property
    def yxox(self) -> UVector4: ...
    @property
    def yxoy(self) -> UVector4: ...
    @property
    def yxoz(self) -> UVector4: ...
    @property
    def yxoo(self) -> UVector4: ...
    @property
    def yxol(self) -> UVector4: ...
    @property
    def yxlx(self) -> UVector4: ...
    @property
    def yxly(self) -> UVector4: ...
    @property
    def yxlz(self) -> UVector4: ...
    @property
    def yxlo(self) -> UVector4: ...
    @property
    def yxll(self) -> UVector4: ...
    @property
    def yyxx(self) -> UVector4: ...
    @property
    def yyxy(self) -> UVector4: ...
    @property
    def yyxz(self) -> UVector4: ...
    @property
    def yyxo(self) -> UVector4: ...
    @property
    def yyxl(self) -> UVector4: ...
    @property
    def yyyx(self) -> UVector4: ...
    @property
    def yyyy(self) -> UVector4: ...
    @property
    def yyyz(self) -> UVector4: ...
    @property
    def yyyo(self) -> UVector4: ...
    @property
    def yyyl(self) -> UVector4: ...
    @property
    def yyzx(self) -> UVector4: ...
    @property
    def yyzy(self) -> UVector4: ...
    @property
    def yyzz(self) -> UVector4: ...
    @property
    def yyzo(self) -> UVector4: ...
    @property
    def yyzl(self) -> UVector4: ...
    @property
    def yyox(self) -> UVector4: ...
    @property
    def yyoy(self) -> UVector4: ...
    @property
    def yyoz(self) -> UVector4: ...
    @property
    def yyoo(self) -> UVector4: ...
    @property
    def yyol(self) -> UVector4: ...
    @property
    def yylx(self) -> UVector4: ...
    @property
    def yyly(self) -> UVector4: ...
    @property
    def yylz(self) -> UVector4: ...
    @property
    def yylo(self) -> UVector4: ...
    @property
    def yyll(self) -> UVector4: ...
    @property
    def yzxx(self) -> UVector4: ...
    @property
    def yzxy(self) -> UVector4: ...
    @property
    def yzxz(self) -> UVector4: ...
    @property
    def yzxo(self) -> UVector4: ...
    @property
    def yzxl(self) -> UVector4: ...
    @property
    def yzyx(self) -> UVector4: ...
    @property
    def yzyy(self) -> UVector4: ...
    @property
    def yzyz(self) -> UVector4: ...
    @property
    def yzyo(self) -> UVector4: ...
    @property
    def yzyl(self) -> UVector4: ...
    @property
    def yzzx(self) -> UVector4: ...
    @property
    def yzzy(self) -> UVector4: ...
    @property
    def yzzz(self) -> UVector4: ...
    @property
    def yzzo(self) -> UVector4: ...
    @property
    def yzzl(self) -> UVector4: ...
    @property
    def yzox(self) -> UVector4: ...
    @property
    def yzoy(self) -> UVector4: ...
    @property
    def yzoz(self) -> UVector4: ...
    @property
    def yzoo(self) -> UVector4: ...
    @property
    def yzol(self) -> UVector4: ...
    @property
    def yzlx(self) -> UVector4: ...
    @property
    def yzly(self) -> UVector4: ...
    @property
    def yzlz(self) -> UVector4: ...
    @property
    def yzlo(self) -> UVector4: ...
    @property
    def yzll(self) -> UVector4: ...
    @property
    def yoxx(self) -> UVector4: ...
    @property
    def yoxy(self) -> UVector4: ...
    @property
    def yoxz(self) -> UVector4: ...
    @property
    def yoxo(self) -> UVector4: ...
    @property
    def yoxl(self) -> UVector4: ...
    @property
    def yoyx(self) -> UVector4: ...
    @property
    def yoyy(self) -> UVector4: ...
    @property
    def yoyz(self) -> UVector4: ...
    @property
    def yoyo(self) -> UVector4: ...
    @property
    def yoyl(self) -> UVector4: ...
    @property
    def yozx(self) -> UVector4: ...
    @property
    def yozy(self) -> UVector4: ...
    @property
    def yozz(self) -> UVector4: ...
    @property
    def yozo(self) -> UVector4: ...
    @property
    def yozl(self) -> UVector4: ...
    @property
    def yoox(self) -> UVector4: ...
    @property
    def yooy(self) -> UVector4: ...
    @property
    def yooz(self) -> UVector4: ...
    @property
    def yooo(self) -> UVector4: ...
    @property
    def yool(self) -> UVector4: ...
    @property
    def yolx(self) -> UVector4: ...
    @property
    def yoly(self) -> UVector4: ...
    @property
    def yolz(self) -> UVector4: ...
    @property
    def yolo(self) -> UVector4: ...
    @property
    def yoll(self) -> UVector4: ...
    @property
    def ylxx(self) -> UVector4: ...
    @property
    def ylxy(self) -> UVector4: ...
    @property
    def ylxz(self) -> UVector4: ...
    @property
    def ylxo(self) -> UVector4: ...
    @property
    def ylxl(self) -> UVector4: ...
    @property
    def ylyx(self) -> UVector4: ...
    @property
    def ylyy(self) -> UVector4: ...
    @property
    def ylyz(self) -> UVector4: ...
    @property
    def ylyo(self) -> UVector4: ...
    @property
    def ylyl(self) -> UVector4: ...
    @property
    def ylzx(self) -> UVector4: ...
    @property
    def ylzy(self) -> UVector4: ...
    @property
    def ylzz(self) -> UVector4: ...
    @property
    def ylzo(self) -> UVector4: ...
    @property
    def ylzl(self) -> UVector4: ...
    @property
    def ylox(self) -> UVector4: ...
    @property
    def yloy(self) -> UVector4: ...
    @property
    def yloz(self) -> UVector4: ...
    @property
    def yloo(self) -> UVector4: ...
    @property
    def ylol(self) -> UVector4: ...
    @property
    def yllx(self) -> UVector4: ...
    @property
    def ylly(self) -> UVector4: ...
    @property
    def yllz(self) -> UVector4: ...
    @property
    def yllo(self) -> UVector4: ...
    @property
    def ylll(self) -> UVector4: ...
    @property
    def zxxx(self) -> UVector4: ...
    @property
    def zxxy(self) -> UVector4: ...
    @property
    def zxxz(self) -> UVector4: ...
    @property
    def zxxo(self) -> UVector4: ...
    @property
    def zxxl(self) -> UVector4: ...
    @property
    def zxyx(self) -> UVector4: ...
    @property
    def zxyy(self) -> UVector4: ...
    @property
    def zxyz(self) -> UVector4: ...
    @property
    def zxyo(self) -> UVector4: ...
    @property
    def zxyl(self) -> UVector4: ...
    @property
    def zxzx(self) -> UVector4: ...
    @property
    def zxzy(self) -> UVector4: ...
    @property
    def zxzz(self) -> UVector4: ...
    @property
    def zxzo(self) -> UVector4: ...
    @property
    def zxzl(self) -> UVector4: ...
    @property
    def zxox(self) -> UVector4: ...
    @property
    def zxoy(self) -> UVector4: ...
    @property
    def zxoz(self) -> UVector4: ...
    @property
    def zxoo(self) -> UVector4: ...
    @property
    def zxol(self) -> UVector4: ...
    @property
    def zxlx(self) -> UVector4: ...
    @property
    def zxly(self) -> UVector4: ...
    @property
    def zxlz(self) -> UVector4: ...
    @property
    def zxlo(self) -> UVector4: ...
    @property
    def zxll(self) -> UVector4: ...
    @property
    def zyxx(self) -> UVector4: ...
    @property
    def zyxy(self) -> UVector4: ...
    @property
    def zyxz(self) -> UVector4: ...
    @property
    def zyxo(self) -> UVector4: ...
    @property
    def zyxl(self) -> UVector4: ...
    @property
    def zyyx(self) -> UVector4: ...
    @property
    def zyyy(self) -> UVector4: ...
    @property
    def zyyz(self) -> UVector4: ...
    @property
    def zyyo(self) -> UVector4: ...
    @property
    def zyyl(self) -> UVector4: ...
    @property
    def zyzx(self) -> UVector4: ...
    @property
    def zyzy(self) -> UVector4: ...
    @property
    def zyzz(self) -> UVector4: ...
    @property
    def zyzo(self) -> UVector4: ...
    @property
    def zyzl(self) -> UVector4: ...
    @property
    def zyox(self) -> UVector4: ...
    @property
    def zyoy(self) -> UVector4: ...
    @property
    def zyoz(self) -> UVector4: ...
    @property
    def zyoo(self) -> UVector4: ...
    @property
    def zyol(self) -> UVector4: ...
    @property
    def zylx(self) -> UVector4: ...
    @property
    def zyly(self) -> UVector4: ...
    @property
    def zylz(self) -> UVector4: ...
    @property
    def zylo(self) -> UVector4: ...
    @property
    def zyll(self) -> UVector4: ...
    @property
    def zzxx(self) -> UVector4: ...
    @property
    def zzxy(self) -> UVector4: ...
    @property
    def zzxz(self) -> UVector4: ...
    @property
    def zzxo(self) -> UVector4: ...
    @property
    def zzxl(self) -> UVector4: ...
    @property
    def zzyx(self) -> UVector4: ...
    @property
    def zzyy(self) -> UVector4: ...
    @property
    def zzyz(self) -> UVector4: ...
    @property
    def zzyo(self) -> UVector4: ...
    @property
    def zzyl(self) -> UVector4: ...
    @property
    def zzzx(self) -> UVector4: ...
    @property
    def zzzy(self) -> UVector4: ...
    @property
    def zzzz(self) -> UVector4: ...
    @property
    def zzzo(self) -> UVector4: ...
    @property
    def zzzl(self) -> UVector4: ...
    @property
    def zzox(self) -> UVector4: ...
    @property
    def zzoy(self) -> UVector4: ...
    @property
    def zzoz(self) -> UVector4: ...
    @property
    def zzoo(self) -> UVector4: ...
    @property
    def zzol(self) -> UVector4: ...
    @property
    def zzlx(self) -> UVector4: ...
    @property
    def zzly(self) -> UVector4: ...
    @property
    def zzlz(self) -> UVector4: ...
    @property
    def zzlo(self) -> UVector4: ...
    @property
    def zzll(self) -> UVector4: ...
    @property
    def zoxx(self) -> UVector4: ...
    @property
    def zoxy(self) -> UVector4: ...
    @property
    def zoxz(self) -> UVector4: ...
    @property
    def zoxo(self) -> UVector4: ...
    @property
    def zoxl(self) -> UVector4: ...
    @property
    def zoyx(self) -> UVector4: ...
    @property
    def zoyy(self) -> UVector4: ...
    @property
    def zoyz(self) -> UVector4: ...
    @property
    def zoyo(self) -> UVector4: ...
    @property
    def zoyl(self) -> UVector4: ...
    @property
    def zozx(self) -> UVector4: ...
    @property
    def zozy(self) -> UVector4: ...
    @property
    def zozz(self) -> UVector4: ...
    @property
    def zozo(self) -> UVector4: ...
    @property
    def zozl(self) -> UVector4: ...
    @property
    def zoox(self) -> UVector4: ...
    @property
    def zooy(self) -> UVector4: ...
    @property
    def zooz(self) -> UVector4: ...
    @property
    def zooo(self) -> UVector4: ...
    @property
    def zool(self) -> UVector4: ...
    @property
    def zolx(self) -> UVector4: ...
    @property
    def zoly(self) -> UVector4: ...
    @property
    def zolz(self) -> UVector4: ...
    @property
    def zolo(self) -> UVector4: ...
    @property
    def zoll(self) -> UVector4: ...
    @property
    def zlxx(self) -> UVector4: ...
    @property
    def zlxy(self) -> UVector4: ...
    @property
    def zlxz(self) -> UVector4: ...
    @property
    def zlxo(self) -> UVector4: ...
    @property
    def zlxl(self) -> UVector4: ...
    @property
    def zlyx(self) -> UVector4: ...
    @property
    def zlyy(self) -> UVector4: ...
    @property
    def zlyz(self) -> UVector4: ...
    @property
    def zlyo(self) -> UVector4: ...
    @property
    def zlyl(self) -> UVector4: ...
    @property
    def zlzx(self) -> UVector4: ...
    @property
    def zlzy(self) -> UVector4: ...
    @property
    def zlzz(self) -> UVector4: ...
    @property
    def zlzo(self) -> UVector4: ...
    @property
    def zlzl(self) -> UVector4: ...
    @property
    def zlox(self) -> UVector4: ...
    @property
    def zloy(self) -> UVector4: ...
    @property
    def zloz(self) -> UVector4: ...
    @property
    def zloo(self) -> UVector4: ...
    @property
    def zlol(self) -> UVector4: ...
    @property
    def zllx(self) -> UVector4: ...
    @property
    def zlly(self) -> UVector4: ...
    @property
    def zllz(self) -> UVector4: ...
    @property
    def zllo(self) -> UVector4: ...
    @property
    def zlll(self) -> UVector4: ...
    @property
    def oxxx(self) -> UVector4: ...
    @property
    def oxxy(self) -> UVector4: ...
    @property
    def oxxz(self) -> UVector4: ...
    @property
    def oxxo(self) -> UVector4: ...
    @property
    def oxxl(self) -> UVector4: ...
    @property
    def oxyx(self) -> UVector4: ...
    @property
    def oxyy(self) -> UVector4: ...
    @property
    def oxyz(self) -> UVector4: ...
    @property
    def oxyo(self) -> UVector4: ...
    @property
    def oxyl(self) -> UVector4: ...
    @property
    def oxzx(self) -> UVector4: ...
    @property
    def oxzy(self) -> UVector4: ...
    @property
    def oxzz(self) -> UVector4: ...
    @property
    def oxzo(self) -> UVector4: ...
    @property
    def oxzl(self) -> UVector4: ...
    @property
    def oxox(self) -> UVector4: ...
    @property
    def oxoy(self) -> UVector4: ...
    @property
    def oxoz(self) -> UVector4: ...
    @property
    def oxoo(self) -> UVector4: ...
    @property
    def oxol(self) -> UVector4: ...
    @property
    def oxlx(self) -> UVector4: ...
    @property
    def oxly(self) -> UVector4: ...
    @property
    def oxlz(self) -> UVector4: ...
    @property
    def oxlo(self) -> UVector4: ...
    @property
    def oxll(self) -> UVector4: ...
    @property
    def oyxx(self) -> UVector4: ...
    @property
    def oyxy(self) -> UVector4: ...
    @property
    def oyxz(self) -> UVector4: ...
    @property
    def oyxo(self) -> UVector4: ...
    @property
    def oyxl(self) -> UVector4: ...
    @property
    def oyyx(self) -> UVector4: ...
    @property
    def oyyy(self) -> UVector4: ...
    @property
    def oyyz(self) -> UVector4: ...
    @property
    def oyyo(self) -> UVector4: ...
    @property
    def oyyl(self) -> UVector4: ...
    @property
    def oyzx(self) -> UVector4: ...
    @property
    def oyzy(self) -> UVector4: ...
    @property
    def oyzz(self) -> UVector4: ...
    @property
    def oyzo(self) -> UVector4: ...
    @property
    def oyzl(self) -> UVector4: ...
    @property
    def oyox(self) -> UVector4: ...
    @property
    def oyoy(self) -> UVector4: ...
    @property
    def oyoz(self) -> UVector4: ...
    @property
    def oyoo(self) -> UVector4: ...
    @property
    def oyol(self) -> UVector4: ...
    @property
    def oylx(self) -> UVector4: ...
    @property
    def oyly(self) -> UVector4: ...
    @property
    def oylz(self) -> UVector4: ...
    @property
    def oylo(self) -> UVector4: ...
    @property
    def oyll(self) -> UVector4: ...
    @property
    def ozxx(self) -> UVector4: ...
    @property
    def ozxy(self) -> UVector4: ...
    @property
    def ozxz(self) -> UVector4: ...
    @property
    def ozxo(self) -> UVector4: ...
    @property
    def ozxl(self) -> UVector4: ...
    @property
    def ozyx(self) -> UVector4: ...
    @property
    def ozyy(self) -> UVector4: ...
    @property
    def ozyz(self) -> UVector4: ...
    @property
    def ozyo(self) -> UVector4: ...
    @property
    def ozyl(self) -> UVector4: ...
    @property
    def ozzx(self) -> UVector4: ...
    @property
    def ozzy(self) -> UVector4: ...
    @property
    def ozzz(self) -> UVector4: ...
    @property
    def ozzo(self) -> UVector4: ...
    @property
    def ozzl(self) -> UVector4: ...
    @property
    def ozox(self) -> UVector4: ...
    @property
    def ozoy(self) -> UVector4: ...
    @property
    def ozoz(self) -> UVector4: ...
    @property
    def ozoo(self) -> UVector4: ...
    @property
    def ozol(self) -> UVector4: ...
    @property
    def ozlx(self) -> UVector4: ...
    @property
    def ozly(self) -> UVector4: ...
    @property
    def ozlz(self) -> UVector4: ...
    @property
    def ozlo(self) -> UVector4: ...
    @property
    def ozll(self) -> UVector4: ...
    @property
    def ooxx(self) -> UVector4: ...
    @property
    def ooxy(self) -> UVector4: ...
    @property
    def ooxz(self) -> UVector4: ...
    @property
    def ooxo(self) -> UVector4: ...
    @property
    def ooxl(self) -> UVector4: ...
    @property
    def ooyx(self) -> UVector4: ...
    @property
    def ooyy(self) -> UVector4: ...
    @property
    def ooyz(self) -> UVector4: ...
    @property
    def ooyo(self) -> UVector4: ...
    @property
    def ooyl(self) -> UVector4: ...
    @property
    def oozx(self) -> UVector4: ...
    @property
    def oozy(self) -> UVector4: ...
    @property
    def oozz(self) -> UVector4: ...
    @property
    def oozo(self) -> UVector4: ...
    @property
    def oozl(self) -> UVector4: ...
    @property
    def ooox(self) -> UVector4: ...
    @property
    def oooy(self) -> UVector4: ...
    @property
    def oooz(self) -> UVector4: ...
    @property
    def oolx(self) -> UVector4: ...
    @property
    def ooly(self) -> UVector4: ...
    @property
    def oolz(self) -> UVector4: ...
    @property
    def olxx(self) -> UVector4: ...
    @property
    def olxy(self) -> UVector4: ...
    @property
    def olxz(self) -> UVector4: ...
    @property
    def olxo(self) -> UVector4: ...
    @property
    def olxl(self) -> UVector4: ...
    @property
    def olyx(self) -> UVector4: ...
    @property
    def olyy(self) -> UVector4: ...
    @property
    def olyz(self) -> UVector4: ...
    @property
    def olyo(self) -> UVector4: ...
    @property
    def olyl(self) -> UVector4: ...
    @property
    def olzx(self) -> UVector4: ...
    @property
    def olzy(self) -> UVector4: ...
    @property
    def olzz(self) -> UVector4: ...
    @property
    def olzo(self) -> UVector4: ...
    @property
    def olzl(self) -> UVector4: ...
    @property
    def olox(self) -> UVector4: ...
    @property
    def oloy(self) -> UVector4: ...
    @property
    def oloz(self) -> UVector4: ...
    @property
    def ollx(self) -> UVector4: ...
    @property
    def olly(self) -> UVector4: ...
    @property
    def ollz(self) -> UVector4: ...
    @property
    def lxxx(self) -> UVector4: ...
    @property
    def lxxy(self) -> UVector4: ...
    @property
    def lxxz(self) -> UVector4: ...
    @property
    def lxxo(self) -> UVector4: ...
    @property
    def lxxl(self) -> UVector4: ...
    @property
    def lxyx(self) -> UVector4: ...
    @property
    def lxyy(self) -> UVector4: ...
    @property
    def lxyz(self) -> UVector4: ...
    @property
    def lxyo(self) -> UVector4: ...
    @property
    def lxyl(self) -> UVector4: ...
    @property
    def lxzx(self) -> UVector4: ...
    @property
    def lxzy(self) -> UVector4: ...
    @property
    def lxzz(self) -> UVector4: ...
    @property
    def lxzo(self) -> UVector4: ...
    @property
    def lxzl(self) -> UVector4: ...
    @property
    def lxox(self) -> UVector4: ...
    @property
    def lxoy(self) -> UVector4: ...
    @property
    def lxoz(self) -> UVector4: ...
    @property
    def lxoo(self) -> UVector4: ...
    @property
    def lxol(self) -> UVector4: ...
    @property
    def lxlx(self) -> UVector4: ...
    @property
    def lxly(self) -> UVector4: ...
    @property
    def lxlz(self) -> UVector4: ...
    @property
    def lxlo(self) -> UVector4: ...
    @property
    def lxll(self) -> UVector4: ...
    @property
    def lyxx(self) -> UVector4: ...
    @property
    def lyxy(self) -> UVector4: ...
    @property
    def lyxz(self) -> UVector4: ...
    @property
    def lyxo(self) -> UVector4: ...
    @property
    def lyxl(self) -> UVector4: ...
    @property
    def lyyx(self) -> UVector4: ...
    @property
    def lyyy(self) -> UVector4: ...
    @property
    def lyyz(self) -> UVector4: ...
    @property
    def lyyo(self) -> UVector4: ...
    @property
    def lyyl(self) -> UVector4: ...
    @property
    def lyzx(self) -> UVector4: ...
    @property
    def lyzy(self) -> UVector4: ...
    @property
    def lyzz(self) -> UVector4: ...
    @property
    def lyzo(self) -> UVector4: ...
    @property
    def lyzl(self) -> UVector4: ...
    @property
    def lyox(self) -> UVector4: ...
    @property
    def lyoy(self) -> UVector4: ...
    @property
    def lyoz(self) -> UVector4: ...
    @property
    def lyoo(self) -> UVector4: ...
    @property
    def lyol(self) -> UVector4: ...
    @property
    def lylx(self) -> UVector4: ...
    @property
    def lyly(self) -> UVector4: ...
    @property
    def lylz(self) -> UVector4: ...
    @property
    def lylo(self) -> UVector4: ...
    @property
    def lyll(self) -> UVector4: ...
    @property
    def lzxx(self) -> UVector4: ...
    @property
    def lzxy(self) -> UVector4: ...
    @property
    def lzxz(self) -> UVector4: ...
    @property
    def lzxo(self) -> UVector4: ...
    @property
    def lzxl(self) -> UVector4: ...
    @property
    def lzyx(self) -> UVector4: ...
    @property
    def lzyy(self) -> UVector4: ...
    @property
    def lzyz(self) -> UVector4: ...
    @property
    def lzyo(self) -> UVector4: ...
    @property
    def lzyl(self) -> UVector4: ...
    @property
    def lzzx(self) -> UVector4: ...
    @property
    def lzzy(self) -> UVector4: ...
    @property
    def lzzz(self) -> UVector4: ...
    @property
    def lzzo(self) -> UVector4: ...
    @property
    def lzzl(self) -> UVector4: ...
    @property
    def lzox(self) -> UVector4: ...
    @property
    def lzoy(self) -> UVector4: ...
    @property
    def lzoz(self) -> UVector4: ...
    @property
    def lzoo(self) -> UVector4: ...
    @property
    def lzol(self) -> UVector4: ...
    @property
    def lzlx(self) -> UVector4: ...
    @property
    def lzly(self) -> UVector4: ...
    @property
    def lzlz(self) -> UVector4: ...
    @property
    def lzlo(self) -> UVector4: ...
    @property
    def lzll(self) -> UVector4: ...
    @property
    def loxx(self) -> UVector4: ...
    @property
    def loxy(self) -> UVector4: ...
    @property
    def loxz(self) -> UVector4: ...
    @property
    def loxo(self) -> UVector4: ...
    @property
    def loxl(self) -> UVector4: ...
    @property
    def loyx(self) -> UVector4: ...
    @property
    def loyy(self) -> UVector4: ...
    @property
    def loyz(self) -> UVector4: ...
    @property
    def loyo(self) -> UVector4: ...
    @property
    def loyl(self) -> UVector4: ...
    @property
    def lozx(self) -> UVector4: ...
    @property
    def lozy(self) -> UVector4: ...
    @property
    def lozz(self) -> UVector4: ...
    @property
    def lozo(self) -> UVector4: ...
    @property
    def lozl(self) -> UVector4: ...
    @property
    def loox(self) -> UVector4: ...
    @property
    def looy(self) -> UVector4: ...
    @property
    def looz(self) -> UVector4: ...
    @property
    def lolx(self) -> UVector4: ...
    @property
    def loly(self) -> UVector4: ...
    @property
    def lolz(self) -> UVector4: ...
    @property
    def llxx(self) -> UVector4: ...
    @property
    def llxy(self) -> UVector4: ...
    @property
    def llxz(self) -> UVector4: ...
    @property
    def llxo(self) -> UVector4: ...
    @property
    def llxl(self) -> UVector4: ...
    @property
    def llyx(self) -> UVector4: ...
    @property
    def llyy(self) -> UVector4: ...
    @property
    def llyz(self) -> UVector4: ...
    @property
    def llyo(self) -> UVector4: ...
    @property
    def llyl(self) -> UVector4: ...
    @property
    def llzx(self) -> UVector4: ...
    @property
    def llzy(self) -> UVector4: ...
    @property
    def llzz(self) -> UVector4: ...
    @property
    def llzo(self) -> UVector4: ...
    @property
    def llzl(self) -> UVector4: ...
    @property
    def llox(self) -> UVector4: ...
    @property
    def lloy(self) -> UVector4: ...
    @property
    def lloz(self) -> UVector4: ...
    @property
    def lllx(self) -> UVector4: ...
    @property
    def llly(self) -> UVector4: ...
    @property
    def lllz(self) -> UVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> UVector2: ...
    @property
    def rg(self) -> UVector2: ...
    @property
    def rb(self) -> UVector2: ...
    @property
    def ro(self) -> UVector2: ...
    @property
    def rl(self) -> UVector2: ...
    @property
    def gr(self) -> UVector2: ...
    @property
    def gg(self) -> UVector2: ...
    @property
    def gb(self) -> UVector2: ...
    @property
    def go(self) -> UVector2: ...
    @property
    def gl(self) -> UVector2: ...
    @property
    def br(self) -> UVector2: ...
    @property
    def bg(self) -> UVector2: ...
    @property
    def bb(self) -> UVector2: ...
    @property
    def bo(self) -> UVector2: ...
    @property
    def bl(self) -> UVector2: ...
    @property
    def og(self) -> UVector2: ...
    @property
    def ob(self) -> UVector2: ...
    @property
    def lr(self) -> UVector2: ...
    @property
    def lg(self) -> UVector2: ...
    @property
    def lb(self) -> UVector2: ...
    @property
    def rrr(self) -> UVector3: ...
    @property
    def rrg(self) -> UVector3: ...
    @property
    def rrb(self) -> UVector3: ...
    @property
    def rro(self) -> UVector3: ...
    @property
    def rrl(self) -> UVector3: ...
    @property
    def rgr(self) -> UVector3: ...
    @property
    def rgg(self) -> UVector3: ...
    @property
    def rgb(self) -> UVector3: ...
    @property
    def rgo(self) -> UVector3: ...
    @property
    def rgl(self) -> UVector3: ...
    @property
    def rbr(self) -> UVector3: ...
    @property
    def rbg(self) -> UVector3: ...
    @property
    def rbb(self) -> UVector3: ...
    @property
    def rbo(self) -> UVector3: ...
    @property
    def rbl(self) -> UVector3: ...
    @property
    def ror(self) -> UVector3: ...
    @property
    def rog(self) -> UVector3: ...
    @property
    def rob(self) -> UVector3: ...
    @property
    def roo(self) -> UVector3: ...
    @property
    def rol(self) -> UVector3: ...
    @property
    def rlr(self) -> UVector3: ...
    @property
    def rlg(self) -> UVector3: ...
    @property
    def rlb(self) -> UVector3: ...
    @property
    def rlo(self) -> UVector3: ...
    @property
    def rll(self) -> UVector3: ...
    @property
    def grr(self) -> UVector3: ...
    @property
    def grg(self) -> UVector3: ...
    @property
    def grb(self) -> UVector3: ...
    @property
    def gro(self) -> UVector3: ...
    @property
    def grl(self) -> UVector3: ...
    @property
    def ggr(self) -> UVector3: ...
    @property
    def ggg(self) -> UVector3: ...
    @property
    def ggb(self) -> UVector3: ...
    @property
    def ggo(self) -> UVector3: ...
    @property
    def ggl(self) -> UVector3: ...
    @property
    def gbr(self) -> UVector3: ...
    @property
    def gbg(self) -> UVector3: ...
    @property
    def gbb(self) -> UVector3: ...
    @property
    def gbo(self) -> UVector3: ...
    @property
    def gbl(self) -> UVector3: ...
    @property
    def gor(self) -> UVector3: ...
    @property
    def gog(self) -> UVector3: ...
    @property
    def gob(self) -> UVector3: ...
    @property
    def goo(self) -> UVector3: ...
    @property
    def gol(self) -> UVector3: ...
    @property
    def glr(self) -> UVector3: ...
    @property
    def glg(self) -> UVector3: ...
    @property
    def glb(self) -> UVector3: ...
    @property
    def glo(self) -> UVector3: ...
    @property
    def gll(self) -> UVector3: ...
    @property
    def brr(self) -> UVector3: ...
    @property
    def brg(self) -> UVector3: ...
    @property
    def brb(self) -> UVector3: ...
    @property
    def bro(self) -> UVector3: ...
    @property
    def brl(self) -> UVector3: ...
    @property
    def bgr(self) -> UVector3: ...
    @property
    def bgg(self) -> UVector3: ...
    @property
    def bgb(self) -> UVector3: ...
    @property
    def bgo(self) -> UVector3: ...
    @property
    def bgl(self) -> UVector3: ...
    @property
    def bbr(self) -> UVector3: ...
    @property
    def bbg(self) -> UVector3: ...
    @property
    def bbb(self) -> UVector3: ...
    @property
    def bbo(self) -> UVector3: ...
    @property
    def bbl(self) -> UVector3: ...
    @property
    def bor(self) -> UVector3: ...
    @property
    def bog(self) -> UVector3: ...
    @property
    def bob(self) -> UVector3: ...
    @property
    def boo(self) -> UVector3: ...
    @property
    def bol(self) -> UVector3: ...
    @property
    def blr(self) -> UVector3: ...
    @property
    def blg(self) -> UVector3: ...
    @property
    def blb(self) -> UVector3: ...
    @property
    def blo(self) -> UVector3: ...
    @property
    def bll(self) -> UVector3: ...
    @property
    def orr(self) -> UVector3: ...
    @property
    def org(self) -> UVector3: ...
    @property
    def orb(self) -> UVector3: ...
    @property
    def oro(self) -> UVector3: ...
    @property
    def orl(self) -> UVector3: ...
    @property
    def ogr(self) -> UVector3: ...
    @property
    def ogg(self) -> UVector3: ...
    @property
    def ogb(self) -> UVector3: ...
    @property
    def ogo(self) -> UVector3: ...
    @property
    def ogl(self) -> UVector3: ...
    @property
    def obr(self) -> UVector3: ...
    @property
    def obg(self) -> UVector3: ...
    @property
    def obb(self) -> UVector3: ...
    @property
    def obo(self) -> UVector3: ...
    @property
    def obl(self) -> UVector3: ...
    @property
    def oor(self) -> UVector3: ...
    @property
    def oog(self) -> UVector3: ...
    @property
    def oob(self) -> UVector3: ...
    @property
    def olr(self) -> UVector3: ...
    @property
    def olg(self) -> UVector3: ...
    @property
    def olb(self) -> UVector3: ...
    @property
    def lrr(self) -> UVector3: ...
    @property
    def lrg(self) -> UVector3: ...
    @property
    def lrb(self) -> UVector3: ...
    @property
    def lro(self) -> UVector3: ...
    @property
    def lrl(self) -> UVector3: ...
    @property
    def lgr(self) -> UVector3: ...
    @property
    def lgg(self) -> UVector3: ...
    @property
    def lgb(self) -> UVector3: ...
    @property
    def lgo(self) -> UVector3: ...
    @property
    def lgl(self) -> UVector3: ...
    @property
    def lbr(self) -> UVector3: ...
    @property
    def lbg(self) -> UVector3: ...
    @property
    def lbb(self) -> UVector3: ...
    @property
    def lbo(self) -> UVector3: ...
    @property
    def lbl(self) -> UVector3: ...
    @property
    def lor(self) -> UVector3: ...
    @property
    def log(self) -> UVector3: ...
    @property
    def lob(self) -> UVector3: ...
    @property
    def llr(self) -> UVector3: ...
    @property
    def llg(self) -> UVector3: ...
    @property
    def llb(self) -> UVector3: ...
    @property
    def rrrr(self) -> UVector4: ...
    @property
    def rrrg(self) -> UVector4: ...
    @property
    def rrrb(self) -> UVector4: ...
    @property
    def rrro(self) -> UVector4: ...
    @property
    def rrrl(self) -> UVector4: ...
    @property
    def rrgr(self) -> UVector4: ...
    @property
    def rrgg(self) -> UVector4: ...
    @property
    def rrgb(self) -> UVector4: ...
    @property
    def rrgo(self) -> UVector4: ...
    @property
    def rrgl(self) -> UVector4: ...
    @property
    def rrbr(self) -> UVector4: ...
    @property
    def rrbg(self) -> UVector4: ...
    @property
    def rrbb(self) -> UVector4: ...
    @property
    def rrbo(self) -> UVector4: ...
    @property
    def rrbl(self) -> UVector4: ...
    @property
    def rror(self) -> UVector4: ...
    @property
    def rrog(self) -> UVector4: ...
    @property
    def rrob(self) -> UVector4: ...
    @property
    def rroo(self) -> UVector4: ...
    @property
    def rrol(self) -> UVector4: ...
    @property
    def rrlr(self) -> UVector4: ...
    @property
    def rrlg(self) -> UVector4: ...
    @property
    def rrlb(self) -> UVector4: ...
    @property
    def rrlo(self) -> UVector4: ...
    @property
    def rrll(self) -> UVector4: ...
    @property
    def rgrr(self) -> UVector4: ...
    @property
    def rgrg(self) -> UVector4: ...
    @property
    def rgrb(self) -> UVector4: ...
    @property
    def rgro(self) -> UVector4: ...
    @property
    def rgrl(self) -> UVector4: ...
    @property
    def rggr(self) -> UVector4: ...
    @property
    def rggg(self) -> UVector4: ...
    @property
    def rggb(self) -> UVector4: ...
    @property
    def rggo(self) -> UVector4: ...
    @property
    def rggl(self) -> UVector4: ...
    @property
    def rgbr(self) -> UVector4: ...
    @property
    def rgbg(self) -> UVector4: ...
    @property
    def rgbb(self) -> UVector4: ...
    @property
    def rgbo(self) -> UVector4: ...
    @property
    def rgbl(self) -> UVector4: ...
    @property
    def rgor(self) -> UVector4: ...
    @property
    def rgog(self) -> UVector4: ...
    @property
    def rgob(self) -> UVector4: ...
    @property
    def rgoo(self) -> UVector4: ...
    @property
    def rgol(self) -> UVector4: ...
    @property
    def rglr(self) -> UVector4: ...
    @property
    def rglg(self) -> UVector4: ...
    @property
    def rglb(self) -> UVector4: ...
    @property
    def rglo(self) -> UVector4: ...
    @property
    def rgll(self) -> UVector4: ...
    @property
    def rbrr(self) -> UVector4: ...
    @property
    def rbrg(self) -> UVector4: ...
    @property
    def rbrb(self) -> UVector4: ...
    @property
    def rbro(self) -> UVector4: ...
    @property
    def rbrl(self) -> UVector4: ...
    @property
    def rbgr(self) -> UVector4: ...
    @property
    def rbgg(self) -> UVector4: ...
    @property
    def rbgb(self) -> UVector4: ...
    @property
    def rbgo(self) -> UVector4: ...
    @property
    def rbgl(self) -> UVector4: ...
    @property
    def rbbr(self) -> UVector4: ...
    @property
    def rbbg(self) -> UVector4: ...
    @property
    def rbbb(self) -> UVector4: ...
    @property
    def rbbo(self) -> UVector4: ...
    @property
    def rbbl(self) -> UVector4: ...
    @property
    def rbor(self) -> UVector4: ...
    @property
    def rbog(self) -> UVector4: ...
    @property
    def rbob(self) -> UVector4: ...
    @property
    def rboo(self) -> UVector4: ...
    @property
    def rbol(self) -> UVector4: ...
    @property
    def rblr(self) -> UVector4: ...
    @property
    def rblg(self) -> UVector4: ...
    @property
    def rblb(self) -> UVector4: ...
    @property
    def rblo(self) -> UVector4: ...
    @property
    def rbll(self) -> UVector4: ...
    @property
    def rorr(self) -> UVector4: ...
    @property
    def rorg(self) -> UVector4: ...
    @property
    def rorb(self) -> UVector4: ...
    @property
    def roro(self) -> UVector4: ...
    @property
    def rorl(self) -> UVector4: ...
    @property
    def rogr(self) -> UVector4: ...
    @property
    def rogg(self) -> UVector4: ...
    @property
    def rogb(self) -> UVector4: ...
    @property
    def rogo(self) -> UVector4: ...
    @property
    def rogl(self) -> UVector4: ...
    @property
    def robr(self) -> UVector4: ...
    @property
    def robg(self) -> UVector4: ...
    @property
    def robb(self) -> UVector4: ...
    @property
    def robo(self) -> UVector4: ...
    @property
    def robl(self) -> UVector4: ...
    @property
    def roor(self) -> UVector4: ...
    @property
    def roog(self) -> UVector4: ...
    @property
    def roob(self) -> UVector4: ...
    @property
    def rooo(self) -> UVector4: ...
    @property
    def rool(self) -> UVector4: ...
    @property
    def rolr(self) -> UVector4: ...
    @property
    def rolg(self) -> UVector4: ...
    @property
    def rolb(self) -> UVector4: ...
    @property
    def rolo(self) -> UVector4: ...
    @property
    def roll(self) -> UVector4: ...
    @property
    def rlrr(self) -> UVector4: ...
    @property
    def rlrg(self) -> UVector4: ...
    @property
    def rlrb(self) -> UVector4: ...
    @property
    def rlro(self) -> UVector4: ...
    @property
    def rlrl(self) -> UVector4: ...
    @property
    def rlgr(self) -> UVector4: ...
    @property
    def rlgg(self) -> UVector4: ...
    @property
    def rlgb(self) -> UVector4: ...
    @property
    def rlgo(self) -> UVector4: ...
    @property
    def rlgl(self) -> UVector4: ...
    @property
    def rlbr(self) -> UVector4: ...
    @property
    def rlbg(self) -> UVector4: ...
    @property
    def rlbb(self) -> UVector4: ...
    @property
    def rlbo(self) -> UVector4: ...
    @property
    def rlbl(self) -> UVector4: ...
    @property
    def rlor(self) -> UVector4: ...
    @property
    def rlog(self) -> UVector4: ...
    @property
    def rlob(self) -> UVector4: ...
    @property
    def rloo(self) -> UVector4: ...
    @property
    def rlol(self) -> UVector4: ...
    @property
    def rllr(self) -> UVector4: ...
    @property
    def rllg(self) -> UVector4: ...
    @property
    def rllb(self) -> UVector4: ...
    @property
    def rllo(self) -> UVector4: ...
    @property
    def rlll(self) -> UVector4: ...
    @property
    def grrr(self) -> UVector4: ...
    @property
    def grrg(self) -> UVector4: ...
    @property
    def grrb(self) -> UVector4: ...
    @property
    def grro(self) -> UVector4: ...
    @property
    def grrl(self) -> UVector4: ...
    @property
    def grgr(self) -> UVector4: ...
    @property
    def grgg(self) -> UVector4: ...
    @property
    def grgb(self) -> UVector4: ...
    @property
    def grgo(self) -> UVector4: ...
    @property
    def grgl(self) -> UVector4: ...
    @property
    def grbr(self) -> UVector4: ...
    @property
    def grbg(self) -> UVector4: ...
    @property
    def grbb(self) -> UVector4: ...
    @property
    def grbo(self) -> UVector4: ...
    @property
    def grbl(self) -> UVector4: ...
    @property
    def gror(self) -> UVector4: ...
    @property
    def grog(self) -> UVector4: ...
    @property
    def grob(self) -> UVector4: ...
    @property
    def groo(self) -> UVector4: ...
    @property
    def grol(self) -> UVector4: ...
    @property
    def grlr(self) -> UVector4: ...
    @property
    def grlg(self) -> UVector4: ...
    @property
    def grlb(self) -> UVector4: ...
    @property
    def grlo(self) -> UVector4: ...
    @property
    def grll(self) -> UVector4: ...
    @property
    def ggrr(self) -> UVector4: ...
    @property
    def ggrg(self) -> UVector4: ...
    @property
    def ggrb(self) -> UVector4: ...
    @property
    def ggro(self) -> UVector4: ...
    @property
    def ggrl(self) -> UVector4: ...
    @property
    def gggr(self) -> UVector4: ...
    @property
    def gggg(self) -> UVector4: ...
    @property
    def gggb(self) -> UVector4: ...
    @property
    def gggo(self) -> UVector4: ...
    @property
    def gggl(self) -> UVector4: ...
    @property
    def ggbr(self) -> UVector4: ...
    @property
    def ggbg(self) -> UVector4: ...
    @property
    def ggbb(self) -> UVector4: ...
    @property
    def ggbo(self) -> UVector4: ...
    @property
    def ggbl(self) -> UVector4: ...
    @property
    def ggor(self) -> UVector4: ...
    @property
    def ggog(self) -> UVector4: ...
    @property
    def ggob(self) -> UVector4: ...
    @property
    def ggoo(self) -> UVector4: ...
    @property
    def ggol(self) -> UVector4: ...
    @property
    def gglr(self) -> UVector4: ...
    @property
    def gglg(self) -> UVector4: ...
    @property
    def gglb(self) -> UVector4: ...
    @property
    def gglo(self) -> UVector4: ...
    @property
    def ggll(self) -> UVector4: ...
    @property
    def gbrr(self) -> UVector4: ...
    @property
    def gbrg(self) -> UVector4: ...
    @property
    def gbrb(self) -> UVector4: ...
    @property
    def gbro(self) -> UVector4: ...
    @property
    def gbrl(self) -> UVector4: ...
    @property
    def gbgr(self) -> UVector4: ...
    @property
    def gbgg(self) -> UVector4: ...
    @property
    def gbgb(self) -> UVector4: ...
    @property
    def gbgo(self) -> UVector4: ...
    @property
    def gbgl(self) -> UVector4: ...
    @property
    def gbbr(self) -> UVector4: ...
    @property
    def gbbg(self) -> UVector4: ...
    @property
    def gbbb(self) -> UVector4: ...
    @property
    def gbbo(self) -> UVector4: ...
    @property
    def gbbl(self) -> UVector4: ...
    @property
    def gbor(self) -> UVector4: ...
    @property
    def gbog(self) -> UVector4: ...
    @property
    def gbob(self) -> UVector4: ...
    @property
    def gboo(self) -> UVector4: ...
    @property
    def gbol(self) -> UVector4: ...
    @property
    def gblr(self) -> UVector4: ...
    @property
    def gblg(self) -> UVector4: ...
    @property
    def gblb(self) -> UVector4: ...
    @property
    def gblo(self) -> UVector4: ...
    @property
    def gbll(self) -> UVector4: ...
    @property
    def gorr(self) -> UVector4: ...
    @property
    def gorg(self) -> UVector4: ...
    @property
    def gorb(self) -> UVector4: ...
    @property
    def goro(self) -> UVector4: ...
    @property
    def gorl(self) -> UVector4: ...
    @property
    def gogr(self) -> UVector4: ...
    @property
    def gogg(self) -> UVector4: ...
    @property
    def gogb(self) -> UVector4: ...
    @property
    def gogo(self) -> UVector4: ...
    @property
    def gogl(self) -> UVector4: ...
    @property
    def gobr(self) -> UVector4: ...
    @property
    def gobg(self) -> UVector4: ...
    @property
    def gobb(self) -> UVector4: ...
    @property
    def gobo(self) -> UVector4: ...
    @property
    def gobl(self) -> UVector4: ...
    @property
    def goor(self) -> UVector4: ...
    @property
    def goog(self) -> UVector4: ...
    @property
    def goob(self) -> UVector4: ...
    @property
    def gooo(self) -> UVector4: ...
    @property
    def gool(self) -> UVector4: ...
    @property
    def golr(self) -> UVector4: ...
    @property
    def golg(self) -> UVector4: ...
    @property
    def golb(self) -> UVector4: ...
    @property
    def golo(self) -> UVector4: ...
    @property
    def goll(self) -> UVector4: ...
    @property
    def glrr(self) -> UVector4: ...
    @property
    def glrg(self) -> UVector4: ...
    @property
    def glrb(self) -> UVector4: ...
    @property
    def glro(self) -> UVector4: ...
    @property
    def glrl(self) -> UVector4: ...
    @property
    def glgr(self) -> UVector4: ...
    @property
    def glgg(self) -> UVector4: ...
    @property
    def glgb(self) -> UVector4: ...
    @property
    def glgo(self) -> UVector4: ...
    @property
    def glgl(self) -> UVector4: ...
    @property
    def glbr(self) -> UVector4: ...
    @property
    def glbg(self) -> UVector4: ...
    @property
    def glbb(self) -> UVector4: ...
    @property
    def glbo(self) -> UVector4: ...
    @property
    def glbl(self) -> UVector4: ...
    @property
    def glor(self) -> UVector4: ...
    @property
    def glog(self) -> UVector4: ...
    @property
    def glob(self) -> UVector4: ...
    @property
    def gloo(self) -> UVector4: ...
    @property
    def glol(self) -> UVector4: ...
    @property
    def gllr(self) -> UVector4: ...
    @property
    def gllg(self) -> UVector4: ...
    @property
    def gllb(self) -> UVector4: ...
    @property
    def gllo(self) -> UVector4: ...
    @property
    def glll(self) -> UVector4: ...
    @property
    def brrr(self) -> UVector4: ...
    @property
    def brrg(self) -> UVector4: ...
    @property
    def brrb(self) -> UVector4: ...
    @property
    def brro(self) -> UVector4: ...
    @property
    def brrl(self) -> UVector4: ...
    @property
    def brgr(self) -> UVector4: ...
    @property
    def brgg(self) -> UVector4: ...
    @property
    def brgb(self) -> UVector4: ...
    @property
    def brgo(self) -> UVector4: ...
    @property
    def brgl(self) -> UVector4: ...
    @property
    def brbr(self) -> UVector4: ...
    @property
    def brbg(self) -> UVector4: ...
    @property
    def brbb(self) -> UVector4: ...
    @property
    def brbo(self) -> UVector4: ...
    @property
    def brbl(self) -> UVector4: ...
    @property
    def bror(self) -> UVector4: ...
    @property
    def brog(self) -> UVector4: ...
    @property
    def brob(self) -> UVector4: ...
    @property
    def broo(self) -> UVector4: ...
    @property
    def brol(self) -> UVector4: ...
    @property
    def brlr(self) -> UVector4: ...
    @property
    def brlg(self) -> UVector4: ...
    @property
    def brlb(self) -> UVector4: ...
    @property
    def brlo(self) -> UVector4: ...
    @property
    def brll(self) -> UVector4: ...
    @property
    def bgrr(self) -> UVector4: ...
    @property
    def bgrg(self) -> UVector4: ...
    @property
    def bgrb(self) -> UVector4: ...
    @property
    def bgro(self) -> UVector4: ...
    @property
    def bgrl(self) -> UVector4: ...
    @property
    def bggr(self) -> UVector4: ...
    @property
    def bggg(self) -> UVector4: ...
    @property
    def bggb(self) -> UVector4: ...
    @property
    def bggo(self) -> UVector4: ...
    @property
    def bggl(self) -> UVector4: ...
    @property
    def bgbr(self) -> UVector4: ...
    @property
    def bgbg(self) -> UVector4: ...
    @property
    def bgbb(self) -> UVector4: ...
    @property
    def bgbo(self) -> UVector4: ...
    @property
    def bgbl(self) -> UVector4: ...
    @property
    def bgor(self) -> UVector4: ...
    @property
    def bgog(self) -> UVector4: ...
    @property
    def bgob(self) -> UVector4: ...
    @property
    def bgoo(self) -> UVector4: ...
    @property
    def bgol(self) -> UVector4: ...
    @property
    def bglr(self) -> UVector4: ...
    @property
    def bglg(self) -> UVector4: ...
    @property
    def bglb(self) -> UVector4: ...
    @property
    def bglo(self) -> UVector4: ...
    @property
    def bgll(self) -> UVector4: ...
    @property
    def bbrr(self) -> UVector4: ...
    @property
    def bbrg(self) -> UVector4: ...
    @property
    def bbrb(self) -> UVector4: ...
    @property
    def bbro(self) -> UVector4: ...
    @property
    def bbrl(self) -> UVector4: ...
    @property
    def bbgr(self) -> UVector4: ...
    @property
    def bbgg(self) -> UVector4: ...
    @property
    def bbgb(self) -> UVector4: ...
    @property
    def bbgo(self) -> UVector4: ...
    @property
    def bbgl(self) -> UVector4: ...
    @property
    def bbbr(self) -> UVector4: ...
    @property
    def bbbg(self) -> UVector4: ...
    @property
    def bbbb(self) -> UVector4: ...
    @property
    def bbbo(self) -> UVector4: ...
    @property
    def bbbl(self) -> UVector4: ...
    @property
    def bbor(self) -> UVector4: ...
    @property
    def bbog(self) -> UVector4: ...
    @property
    def bbob(self) -> UVector4: ...
    @property
    def bboo(self) -> UVector4: ...
    @property
    def bbol(self) -> UVector4: ...
    @property
    def bblr(self) -> UVector4: ...
    @property
    def bblg(self) -> UVector4: ...
    @property
    def bblb(self) -> UVector4: ...
    @property
    def bblo(self) -> UVector4: ...
    @property
    def bbll(self) -> UVector4: ...
    @property
    def borr(self) -> UVector4: ...
    @property
    def borg(self) -> UVector4: ...
    @property
    def borb(self) -> UVector4: ...
    @property
    def boro(self) -> UVector4: ...
    @property
    def borl(self) -> UVector4: ...
    @property
    def bogr(self) -> UVector4: ...
    @property
    def bogg(self) -> UVector4: ...
    @property
    def bogb(self) -> UVector4: ...
    @property
    def bogo(self) -> UVector4: ...
    @property
    def bogl(self) -> UVector4: ...
    @property
    def bobr(self) -> UVector4: ...
    @property
    def bobg(self) -> UVector4: ...
    @property
    def bobb(self) -> UVector4: ...
    @property
    def bobo(self) -> UVector4: ...
    @property
    def bobl(self) -> UVector4: ...
    @property
    def boor(self) -> UVector4: ...
    @property
    def boog(self) -> UVector4: ...
    @property
    def boob(self) -> UVector4: ...
    @property
    def booo(self) -> UVector4: ...
    @property
    def bool(self) -> UVector4: ...
    @property
    def bolr(self) -> UVector4: ...
    @property
    def bolg(self) -> UVector4: ...
    @property
    def bolb(self) -> UVector4: ...
    @property
    def bolo(self) -> UVector4: ...
    @property
    def boll(self) -> UVector4: ...
    @property
    def blrr(self) -> UVector4: ...
    @property
    def blrg(self) -> UVector4: ...
    @property
    def blrb(self) -> UVector4: ...
    @property
    def blro(self) -> UVector4: ...
    @property
    def blrl(self) -> UVector4: ...
    @property
    def blgr(self) -> UVector4: ...
    @property
    def blgg(self) -> UVector4: ...
    @property
    def blgb(self) -> UVector4: ...
    @property
    def blgo(self) -> UVector4: ...
    @property
    def blgl(self) -> UVector4: ...
    @property
    def blbr(self) -> UVector4: ...
    @property
    def blbg(self) -> UVector4: ...
    @property
    def blbb(self) -> UVector4: ...
    @property
    def blbo(self) -> UVector4: ...
    @property
    def blbl(self) -> UVector4: ...
    @property
    def blor(self) -> UVector4: ...
    @property
    def blog(self) -> UVector4: ...
    @property
    def blob(self) -> UVector4: ...
    @property
    def bloo(self) -> UVector4: ...
    @property
    def blol(self) -> UVector4: ...
    @property
    def bllr(self) -> UVector4: ...
    @property
    def bllg(self) -> UVector4: ...
    @property
    def bllb(self) -> UVector4: ...
    @property
    def bllo(self) -> UVector4: ...
    @property
    def blll(self) -> UVector4: ...
    @property
    def orrr(self) -> UVector4: ...
    @property
    def orrg(self) -> UVector4: ...
    @property
    def orrb(self) -> UVector4: ...
    @property
    def orro(self) -> UVector4: ...
    @property
    def orrl(self) -> UVector4: ...
    @property
    def orgr(self) -> UVector4: ...
    @property
    def orgg(self) -> UVector4: ...
    @property
    def orgb(self) -> UVector4: ...
    @property
    def orgo(self) -> UVector4: ...
    @property
    def orgl(self) -> UVector4: ...
    @property
    def orbr(self) -> UVector4: ...
    @property
    def orbg(self) -> UVector4: ...
    @property
    def orbb(self) -> UVector4: ...
    @property
    def orbo(self) -> UVector4: ...
    @property
    def orbl(self) -> UVector4: ...
    @property
    def oror(self) -> UVector4: ...
    @property
    def orog(self) -> UVector4: ...
    @property
    def orob(self) -> UVector4: ...
    @property
    def oroo(self) -> UVector4: ...
    @property
    def orol(self) -> UVector4: ...
    @property
    def orlr(self) -> UVector4: ...
    @property
    def orlg(self) -> UVector4: ...
    @property
    def orlb(self) -> UVector4: ...
    @property
    def orlo(self) -> UVector4: ...
    @property
    def orll(self) -> UVector4: ...
    @property
    def ogrr(self) -> UVector4: ...
    @property
    def ogrg(self) -> UVector4: ...
    @property
    def ogrb(self) -> UVector4: ...
    @property
    def ogro(self) -> UVector4: ...
    @property
    def ogrl(self) -> UVector4: ...
    @property
    def oggr(self) -> UVector4: ...
    @property
    def oggg(self) -> UVector4: ...
    @property
    def oggb(self) -> UVector4: ...
    @property
    def oggo(self) -> UVector4: ...
    @property
    def oggl(self) -> UVector4: ...
    @property
    def ogbr(self) -> UVector4: ...
    @property
    def ogbg(self) -> UVector4: ...
    @property
    def ogbb(self) -> UVector4: ...
    @property
    def ogbo(self) -> UVector4: ...
    @property
    def ogbl(self) -> UVector4: ...
    @property
    def ogor(self) -> UVector4: ...
    @property
    def ogog(self) -> UVector4: ...
    @property
    def ogob(self) -> UVector4: ...
    @property
    def ogoo(self) -> UVector4: ...
    @property
    def ogol(self) -> UVector4: ...
    @property
    def oglr(self) -> UVector4: ...
    @property
    def oglg(self) -> UVector4: ...
    @property
    def oglb(self) -> UVector4: ...
    @property
    def oglo(self) -> UVector4: ...
    @property
    def ogll(self) -> UVector4: ...
    @property
    def obrr(self) -> UVector4: ...
    @property
    def obrg(self) -> UVector4: ...
    @property
    def obrb(self) -> UVector4: ...
    @property
    def obro(self) -> UVector4: ...
    @property
    def obrl(self) -> UVector4: ...
    @property
    def obgr(self) -> UVector4: ...
    @property
    def obgg(self) -> UVector4: ...
    @property
    def obgb(self) -> UVector4: ...
    @property
    def obgo(self) -> UVector4: ...
    @property
    def obgl(self) -> UVector4: ...
    @property
    def obbr(self) -> UVector4: ...
    @property
    def obbg(self) -> UVector4: ...
    @property
    def obbb(self) -> UVector4: ...
    @property
    def obbo(self) -> UVector4: ...
    @property
    def obbl(self) -> UVector4: ...
    @property
    def obor(self) -> UVector4: ...
    @property
    def obog(self) -> UVector4: ...
    @property
    def obob(self) -> UVector4: ...
    @property
    def oboo(self) -> UVector4: ...
    @property
    def obol(self) -> UVector4: ...
    @property
    def oblr(self) -> UVector4: ...
    @property
    def oblg(self) -> UVector4: ...
    @property
    def oblb(self) -> UVector4: ...
    @property
    def oblo(self) -> UVector4: ...
    @property
    def obll(self) -> UVector4: ...
    @property
    def oorr(self) -> UVector4: ...
    @property
    def oorg(self) -> UVector4: ...
    @property
    def oorb(self) -> UVector4: ...
    @property
    def ooro(self) -> UVector4: ...
    @property
    def oorl(self) -> UVector4: ...
    @property
    def oogr(self) -> UVector4: ...
    @property
    def oogg(self) -> UVector4: ...
    @property
    def oogb(self) -> UVector4: ...
    @property
    def oogo(self) -> UVector4: ...
    @property
    def oogl(self) -> UVector4: ...
    @property
    def oobr(self) -> UVector4: ...
    @property
    def oobg(self) -> UVector4: ...
    @property
    def oobb(self) -> UVector4: ...
    @property
    def oobo(self) -> UVector4: ...
    @property
    def oobl(self) -> UVector4: ...
    @property
    def ooor(self) -> UVector4: ...
    @property
    def ooog(self) -> UVector4: ...
    @property
    def ooob(self) -> UVector4: ...
    @property
    def oolr(self) -> UVector4: ...
    @property
    def oolg(self) -> UVector4: ...
    @property
    def oolb(self) -> UVector4: ...
    @property
    def olrr(self) -> UVector4: ...
    @property
    def olrg(self) -> UVector4: ...
    @property
    def olrb(self) -> UVector4: ...
    @property
    def olro(self) -> UVector4: ...
    @property
    def olrl(self) -> UVector4: ...
    @property
    def olgr(self) -> UVector4: ...
    @property
    def olgg(self) -> UVector4: ...
    @property
    def olgb(self) -> UVector4: ...
    @property
    def olgo(self) -> UVector4: ...
    @property
    def olgl(self) -> UVector4: ...
    @property
    def olbr(self) -> UVector4: ...
    @property
    def olbg(self) -> UVector4: ...
    @property
    def olbb(self) -> UVector4: ...
    @property
    def olbo(self) -> UVector4: ...
    @property
    def olbl(self) -> UVector4: ...
    @property
    def olor(self) -> UVector4: ...
    @property
    def olog(self) -> UVector4: ...
    @property
    def olob(self) -> UVector4: ...
    @property
    def ollr(self) -> UVector4: ...
    @property
    def ollg(self) -> UVector4: ...
    @property
    def ollb(self) -> UVector4: ...
    @property
    def lrrr(self) -> UVector4: ...
    @property
    def lrrg(self) -> UVector4: ...
    @property
    def lrrb(self) -> UVector4: ...
    @property
    def lrro(self) -> UVector4: ...
    @property
    def lrrl(self) -> UVector4: ...
    @property
    def lrgr(self) -> UVector4: ...
    @property
    def lrgg(self) -> UVector4: ...
    @property
    def lrgb(self) -> UVector4: ...
    @property
    def lrgo(self) -> UVector4: ...
    @property
    def lrgl(self) -> UVector4: ...
    @property
    def lrbr(self) -> UVector4: ...
    @property
    def lrbg(self) -> UVector4: ...
    @property
    def lrbb(self) -> UVector4: ...
    @property
    def lrbo(self) -> UVector4: ...
    @property
    def lrbl(self) -> UVector4: ...
    @property
    def lror(self) -> UVector4: ...
    @property
    def lrog(self) -> UVector4: ...
    @property
    def lrob(self) -> UVector4: ...
    @property
    def lroo(self) -> UVector4: ...
    @property
    def lrol(self) -> UVector4: ...
    @property
    def lrlr(self) -> UVector4: ...
    @property
    def lrlg(self) -> UVector4: ...
    @property
    def lrlb(self) -> UVector4: ...
    @property
    def lrlo(self) -> UVector4: ...
    @property
    def lrll(self) -> UVector4: ...
    @property
    def lgrr(self) -> UVector4: ...
    @property
    def lgrg(self) -> UVector4: ...
    @property
    def lgrb(self) -> UVector4: ...
    @property
    def lgro(self) -> UVector4: ...
    @property
    def lgrl(self) -> UVector4: ...
    @property
    def lggr(self) -> UVector4: ...
    @property
    def lggg(self) -> UVector4: ...
    @property
    def lggb(self) -> UVector4: ...
    @property
    def lggo(self) -> UVector4: ...
    @property
    def lggl(self) -> UVector4: ...
    @property
    def lgbr(self) -> UVector4: ...
    @property
    def lgbg(self) -> UVector4: ...
    @property
    def lgbb(self) -> UVector4: ...
    @property
    def lgbo(self) -> UVector4: ...
    @property
    def lgbl(self) -> UVector4: ...
    @property
    def lgor(self) -> UVector4: ...
    @property
    def lgog(self) -> UVector4: ...
    @property
    def lgob(self) -> UVector4: ...
    @property
    def lgoo(self) -> UVector4: ...
    @property
    def lgol(self) -> UVector4: ...
    @property
    def lglr(self) -> UVector4: ...
    @property
    def lglg(self) -> UVector4: ...
    @property
    def lglb(self) -> UVector4: ...
    @property
    def lglo(self) -> UVector4: ...
    @property
    def lgll(self) -> UVector4: ...
    @property
    def lbrr(self) -> UVector4: ...
    @property
    def lbrg(self) -> UVector4: ...
    @property
    def lbrb(self) -> UVector4: ...
    @property
    def lbro(self) -> UVector4: ...
    @property
    def lbrl(self) -> UVector4: ...
    @property
    def lbgr(self) -> UVector4: ...
    @property
    def lbgg(self) -> UVector4: ...
    @property
    def lbgb(self) -> UVector4: ...
    @property
    def lbgo(self) -> UVector4: ...
    @property
    def lbgl(self) -> UVector4: ...
    @property
    def lbbr(self) -> UVector4: ...
    @property
    def lbbg(self) -> UVector4: ...
    @property
    def lbbb(self) -> UVector4: ...
    @property
    def lbbo(self) -> UVector4: ...
    @property
    def lbbl(self) -> UVector4: ...
    @property
    def lbor(self) -> UVector4: ...
    @property
    def lbog(self) -> UVector4: ...
    @property
    def lbob(self) -> UVector4: ...
    @property
    def lboo(self) -> UVector4: ...
    @property
    def lbol(self) -> UVector4: ...
    @property
    def lblr(self) -> UVector4: ...
    @property
    def lblg(self) -> UVector4: ...
    @property
    def lblb(self) -> UVector4: ...
    @property
    def lblo(self) -> UVector4: ...
    @property
    def lbll(self) -> UVector4: ...
    @property
    def lorr(self) -> UVector4: ...
    @property
    def lorg(self) -> UVector4: ...
    @property
    def lorb(self) -> UVector4: ...
    @property
    def loro(self) -> UVector4: ...
    @property
    def lorl(self) -> UVector4: ...
    @property
    def logr(self) -> UVector4: ...
    @property
    def logg(self) -> UVector4: ...
    @property
    def logb(self) -> UVector4: ...
    @property
    def logo(self) -> UVector4: ...
    @property
    def logl(self) -> UVector4: ...
    @property
    def lobr(self) -> UVector4: ...
    @property
    def lobg(self) -> UVector4: ...
    @property
    def lobb(self) -> UVector4: ...
    @property
    def lobo(self) -> UVector4: ...
    @property
    def lobl(self) -> UVector4: ...
    @property
    def loor(self) -> UVector4: ...
    @property
    def loog(self) -> UVector4: ...
    @property
    def loob(self) -> UVector4: ...
    @property
    def lolr(self) -> UVector4: ...
    @property
    def lolg(self) -> UVector4: ...
    @property
    def lolb(self) -> UVector4: ...
    @property
    def llrr(self) -> UVector4: ...
    @property
    def llrg(self) -> UVector4: ...
    @property
    def llrb(self) -> UVector4: ...
    @property
    def llro(self) -> UVector4: ...
    @property
    def llrl(self) -> UVector4: ...
    @property
    def llgr(self) -> UVector4: ...
    @property
    def llgg(self) -> UVector4: ...
    @property
    def llgb(self) -> UVector4: ...
    @property
    def llgo(self) -> UVector4: ...
    @property
    def llgl(self) -> UVector4: ...
    @property
    def llbr(self) -> UVector4: ...
    @property
    def llbg(self) -> UVector4: ...
    @property
    def llbb(self) -> UVector4: ...
    @property
    def llbo(self) -> UVector4: ...
    @property
    def llbl(self) -> UVector4: ...
    @property
    def llor(self) -> UVector4: ...
    @property
    def llog(self) -> UVector4: ...
    @property
    def llob(self) -> UVector4: ...
    @property
    def lllr(self) -> UVector4: ...
    @property
    def lllg(self) -> UVector4: ...
    @property
    def lllb(self) -> UVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> UVector2: ...
    @property
    def st(self) -> UVector2: ...
    @property
    def sq(self) -> UVector2: ...
    @property
    def so(self) -> UVector2: ...
    @property
    def sl(self) -> UVector2: ...
    @property
    def ts(self) -> UVector2: ...
    @property
    def tt(self) -> UVector2: ...
    @property
    def tq(self) -> UVector2: ...
    @property
    def to(self) -> UVector2: ...
    @property
    def tl(self) -> UVector2: ...
    @property
    def qs(self) -> UVector2: ...
    @property
    def qt(self) -> UVector2: ...
    @property
    def qq(self) -> UVector2: ...
    @property
    def qo(self) -> UVector2: ...
    @property
    def ql(self) -> UVector2: ...
    @property
    def os(self) -> UVector2: ...
    @property
    def ot(self) -> UVector2: ...
    @property
    def oq(self) -> UVector2: ...
    @property
    def ls(self) -> UVector2: ...
    @property
    def lt(self) -> UVector2: ...
    @property
    def lq(self) -> UVector2: ...
    @property
    def sss(self) -> UVector3: ...
    @property
    def sst(self) -> UVector3: ...
    @property
    def ssq(self) -> UVector3: ...
    @property
    def sso(self) -> UVector3: ...
    @property
    def ssl(self) -> UVector3: ...
    @property
    def sts(self) -> UVector3: ...
    @property
    def stt(self) -> UVector3: ...
    @property
    def stq(self) -> UVector3: ...
    @property
    def sto(self) -> UVector3: ...
    @property
    def stl(self) -> UVector3: ...
    @property
    def sqs(self) -> UVector3: ...
    @property
    def sqt(self) -> UVector3: ...
    @property
    def sqq(self) -> UVector3: ...
    @property
    def sqo(self) -> UVector3: ...
    @property
    def sql(self) -> UVector3: ...
    @property
    def sos(self) -> UVector3: ...
    @property
    def sot(self) -> UVector3: ...
    @property
    def soq(self) -> UVector3: ...
    @property
    def soo(self) -> UVector3: ...
    @property
    def sol(self) -> UVector3: ...
    @property
    def sls(self) -> UVector3: ...
    @property
    def slt(self) -> UVector3: ...
    @property
    def slq(self) -> UVector3: ...
    @property
    def slo(self) -> UVector3: ...
    @property
    def sll(self) -> UVector3: ...
    @property
    def tss(self) -> UVector3: ...
    @property
    def tst(self) -> UVector3: ...
    @property
    def tsq(self) -> UVector3: ...
    @property
    def tso(self) -> UVector3: ...
    @property
    def tsl(self) -> UVector3: ...
    @property
    def tts(self) -> UVector3: ...
    @property
    def ttt(self) -> UVector3: ...
    @property
    def ttq(self) -> UVector3: ...
    @property
    def tto(self) -> UVector3: ...
    @property
    def ttl(self) -> UVector3: ...
    @property
    def tqs(self) -> UVector3: ...
    @property
    def tqt(self) -> UVector3: ...
    @property
    def tqq(self) -> UVector3: ...
    @property
    def tqo(self) -> UVector3: ...
    @property
    def tql(self) -> UVector3: ...
    @property
    def tos(self) -> UVector3: ...
    @property
    def tot(self) -> UVector3: ...
    @property
    def toq(self) -> UVector3: ...
    @property
    def too(self) -> UVector3: ...
    @property
    def tol(self) -> UVector3: ...
    @property
    def tls(self) -> UVector3: ...
    @property
    def tlt(self) -> UVector3: ...
    @property
    def tlq(self) -> UVector3: ...
    @property
    def tlo(self) -> UVector3: ...
    @property
    def tll(self) -> UVector3: ...
    @property
    def qss(self) -> UVector3: ...
    @property
    def qst(self) -> UVector3: ...
    @property
    def qsq(self) -> UVector3: ...
    @property
    def qso(self) -> UVector3: ...
    @property
    def qsl(self) -> UVector3: ...
    @property
    def qts(self) -> UVector3: ...
    @property
    def qtt(self) -> UVector3: ...
    @property
    def qtq(self) -> UVector3: ...
    @property
    def qto(self) -> UVector3: ...
    @property
    def qtl(self) -> UVector3: ...
    @property
    def qqs(self) -> UVector3: ...
    @property
    def qqt(self) -> UVector3: ...
    @property
    def qqq(self) -> UVector3: ...
    @property
    def qqo(self) -> UVector3: ...
    @property
    def qql(self) -> UVector3: ...
    @property
    def qos(self) -> UVector3: ...
    @property
    def qot(self) -> UVector3: ...
    @property
    def qoq(self) -> UVector3: ...
    @property
    def qoo(self) -> UVector3: ...
    @property
    def qol(self) -> UVector3: ...
    @property
    def qls(self) -> UVector3: ...
    @property
    def qlt(self) -> UVector3: ...
    @property
    def qlq(self) -> UVector3: ...
    @property
    def qlo(self) -> UVector3: ...
    @property
    def qll(self) -> UVector3: ...
    @property
    def oss(self) -> UVector3: ...
    @property
    def ost(self) -> UVector3: ...
    @property
    def osq(self) -> UVector3: ...
    @property
    def oso(self) -> UVector3: ...
    @property
    def osl(self) -> UVector3: ...
    @property
    def ots(self) -> UVector3: ...
    @property
    def ott(self) -> UVector3: ...
    @property
    def otq(self) -> UVector3: ...
    @property
    def oto(self) -> UVector3: ...
    @property
    def otl(self) -> UVector3: ...
    @property
    def oqs(self) -> UVector3: ...
    @property
    def oqt(self) -> UVector3: ...
    @property
    def oqq(self) -> UVector3: ...
    @property
    def oqo(self) -> UVector3: ...
    @property
    def oql(self) -> UVector3: ...
    @property
    def oos(self) -> UVector3: ...
    @property
    def oot(self) -> UVector3: ...
    @property
    def ooq(self) -> UVector3: ...
    @property
    def ols(self) -> UVector3: ...
    @property
    def olt(self) -> UVector3: ...
    @property
    def olq(self) -> UVector3: ...
    @property
    def lss(self) -> UVector3: ...
    @property
    def lst(self) -> UVector3: ...
    @property
    def lsq(self) -> UVector3: ...
    @property
    def lso(self) -> UVector3: ...
    @property
    def lsl(self) -> UVector3: ...
    @property
    def lts(self) -> UVector3: ...
    @property
    def ltt(self) -> UVector3: ...
    @property
    def ltq(self) -> UVector3: ...
    @property
    def lto(self) -> UVector3: ...
    @property
    def ltl(self) -> UVector3: ...
    @property
    def lqs(self) -> UVector3: ...
    @property
    def lqt(self) -> UVector3: ...
    @property
    def lqq(self) -> UVector3: ...
    @property
    def lqo(self) -> UVector3: ...
    @property
    def lql(self) -> UVector3: ...
    @property
    def los(self) -> UVector3: ...
    @property
    def lot(self) -> UVector3: ...
    @property
    def loq(self) -> UVector3: ...
    @property
    def lls(self) -> UVector3: ...
    @property
    def llt(self) -> UVector3: ...
    @property
    def llq(self) -> UVector3: ...
    @property
    def ssss(self) -> UVector4: ...
    @property
    def ssst(self) -> UVector4: ...
    @property
    def sssq(self) -> UVector4: ...
    @property
    def ssso(self) -> UVector4: ...
    @property
    def sssl(self) -> UVector4: ...
    @property
    def ssts(self) -> UVector4: ...
    @property
    def sstt(self) -> UVector4: ...
    @property
    def sstq(self) -> UVector4: ...
    @property
    def ssto(self) -> UVector4: ...
    @property
    def sstl(self) -> UVector4: ...
    @property
    def ssqs(self) -> UVector4: ...
    @property
    def ssqt(self) -> UVector4: ...
    @property
    def ssqq(self) -> UVector4: ...
    @property
    def ssqo(self) -> UVector4: ...
    @property
    def ssql(self) -> UVector4: ...
    @property
    def ssos(self) -> UVector4: ...
    @property
    def ssot(self) -> UVector4: ...
    @property
    def ssoq(self) -> UVector4: ...
    @property
    def ssoo(self) -> UVector4: ...
    @property
    def ssol(self) -> UVector4: ...
    @property
    def ssls(self) -> UVector4: ...
    @property
    def sslt(self) -> UVector4: ...
    @property
    def sslq(self) -> UVector4: ...
    @property
    def sslo(self) -> UVector4: ...
    @property
    def ssll(self) -> UVector4: ...
    @property
    def stss(self) -> UVector4: ...
    @property
    def stst(self) -> UVector4: ...
    @property
    def stsq(self) -> UVector4: ...
    @property
    def stso(self) -> UVector4: ...
    @property
    def stsl(self) -> UVector4: ...
    @property
    def stts(self) -> UVector4: ...
    @property
    def sttt(self) -> UVector4: ...
    @property
    def sttq(self) -> UVector4: ...
    @property
    def stto(self) -> UVector4: ...
    @property
    def sttl(self) -> UVector4: ...
    @property
    def stqs(self) -> UVector4: ...
    @property
    def stqt(self) -> UVector4: ...
    @property
    def stqq(self) -> UVector4: ...
    @property
    def stqo(self) -> UVector4: ...
    @property
    def stql(self) -> UVector4: ...
    @property
    def stos(self) -> UVector4: ...
    @property
    def stot(self) -> UVector4: ...
    @property
    def stoq(self) -> UVector4: ...
    @property
    def stoo(self) -> UVector4: ...
    @property
    def stol(self) -> UVector4: ...
    @property
    def stls(self) -> UVector4: ...
    @property
    def stlt(self) -> UVector4: ...
    @property
    def stlq(self) -> UVector4: ...
    @property
    def stlo(self) -> UVector4: ...
    @property
    def stll(self) -> UVector4: ...
    @property
    def sqss(self) -> UVector4: ...
    @property
    def sqst(self) -> UVector4: ...
    @property
    def sqsq(self) -> UVector4: ...
    @property
    def sqso(self) -> UVector4: ...
    @property
    def sqsl(self) -> UVector4: ...
    @property
    def sqts(self) -> UVector4: ...
    @property
    def sqtt(self) -> UVector4: ...
    @property
    def sqtq(self) -> UVector4: ...
    @property
    def sqto(self) -> UVector4: ...
    @property
    def sqtl(self) -> UVector4: ...
    @property
    def sqqs(self) -> UVector4: ...
    @property
    def sqqt(self) -> UVector4: ...
    @property
    def sqqq(self) -> UVector4: ...
    @property
    def sqqo(self) -> UVector4: ...
    @property
    def sqql(self) -> UVector4: ...
    @property
    def sqos(self) -> UVector4: ...
    @property
    def sqot(self) -> UVector4: ...
    @property
    def sqoq(self) -> UVector4: ...
    @property
    def sqoo(self) -> UVector4: ...
    @property
    def sqol(self) -> UVector4: ...
    @property
    def sqls(self) -> UVector4: ...
    @property
    def sqlt(self) -> UVector4: ...
    @property
    def sqlq(self) -> UVector4: ...
    @property
    def sqlo(self) -> UVector4: ...
    @property
    def sqll(self) -> UVector4: ...
    @property
    def soss(self) -> UVector4: ...
    @property
    def sost(self) -> UVector4: ...
    @property
    def sosq(self) -> UVector4: ...
    @property
    def soso(self) -> UVector4: ...
    @property
    def sosl(self) -> UVector4: ...
    @property
    def sots(self) -> UVector4: ...
    @property
    def sott(self) -> UVector4: ...
    @property
    def sotq(self) -> UVector4: ...
    @property
    def soto(self) -> UVector4: ...
    @property
    def sotl(self) -> UVector4: ...
    @property
    def soqs(self) -> UVector4: ...
    @property
    def soqt(self) -> UVector4: ...
    @property
    def soqq(self) -> UVector4: ...
    @property
    def soqo(self) -> UVector4: ...
    @property
    def soql(self) -> UVector4: ...
    @property
    def soos(self) -> UVector4: ...
    @property
    def soot(self) -> UVector4: ...
    @property
    def sooq(self) -> UVector4: ...
    @property
    def sooo(self) -> UVector4: ...
    @property
    def sool(self) -> UVector4: ...
    @property
    def sols(self) -> UVector4: ...
    @property
    def solt(self) -> UVector4: ...
    @property
    def solq(self) -> UVector4: ...
    @property
    def solo(self) -> UVector4: ...
    @property
    def soll(self) -> UVector4: ...
    @property
    def slss(self) -> UVector4: ...
    @property
    def slst(self) -> UVector4: ...
    @property
    def slsq(self) -> UVector4: ...
    @property
    def slso(self) -> UVector4: ...
    @property
    def slsl(self) -> UVector4: ...
    @property
    def slts(self) -> UVector4: ...
    @property
    def sltt(self) -> UVector4: ...
    @property
    def sltq(self) -> UVector4: ...
    @property
    def slto(self) -> UVector4: ...
    @property
    def sltl(self) -> UVector4: ...
    @property
    def slqs(self) -> UVector4: ...
    @property
    def slqt(self) -> UVector4: ...
    @property
    def slqq(self) -> UVector4: ...
    @property
    def slqo(self) -> UVector4: ...
    @property
    def slql(self) -> UVector4: ...
    @property
    def slos(self) -> UVector4: ...
    @property
    def slot(self) -> UVector4: ...
    @property
    def sloq(self) -> UVector4: ...
    @property
    def sloo(self) -> UVector4: ...
    @property
    def slol(self) -> UVector4: ...
    @property
    def slls(self) -> UVector4: ...
    @property
    def sllt(self) -> UVector4: ...
    @property
    def sllq(self) -> UVector4: ...
    @property
    def sllo(self) -> UVector4: ...
    @property
    def slll(self) -> UVector4: ...
    @property
    def tsss(self) -> UVector4: ...
    @property
    def tsst(self) -> UVector4: ...
    @property
    def tssq(self) -> UVector4: ...
    @property
    def tsso(self) -> UVector4: ...
    @property
    def tssl(self) -> UVector4: ...
    @property
    def tsts(self) -> UVector4: ...
    @property
    def tstt(self) -> UVector4: ...
    @property
    def tstq(self) -> UVector4: ...
    @property
    def tsto(self) -> UVector4: ...
    @property
    def tstl(self) -> UVector4: ...
    @property
    def tsqs(self) -> UVector4: ...
    @property
    def tsqt(self) -> UVector4: ...
    @property
    def tsqq(self) -> UVector4: ...
    @property
    def tsqo(self) -> UVector4: ...
    @property
    def tsql(self) -> UVector4: ...
    @property
    def tsos(self) -> UVector4: ...
    @property
    def tsot(self) -> UVector4: ...
    @property
    def tsoq(self) -> UVector4: ...
    @property
    def tsoo(self) -> UVector4: ...
    @property
    def tsol(self) -> UVector4: ...
    @property
    def tsls(self) -> UVector4: ...
    @property
    def tslt(self) -> UVector4: ...
    @property
    def tslq(self) -> UVector4: ...
    @property
    def tslo(self) -> UVector4: ...
    @property
    def tsll(self) -> UVector4: ...
    @property
    def ttss(self) -> UVector4: ...
    @property
    def ttst(self) -> UVector4: ...
    @property
    def ttsq(self) -> UVector4: ...
    @property
    def ttso(self) -> UVector4: ...
    @property
    def ttsl(self) -> UVector4: ...
    @property
    def ttts(self) -> UVector4: ...
    @property
    def tttt(self) -> UVector4: ...
    @property
    def tttq(self) -> UVector4: ...
    @property
    def ttto(self) -> UVector4: ...
    @property
    def tttl(self) -> UVector4: ...
    @property
    def ttqs(self) -> UVector4: ...
    @property
    def ttqt(self) -> UVector4: ...
    @property
    def ttqq(self) -> UVector4: ...
    @property
    def ttqo(self) -> UVector4: ...
    @property
    def ttql(self) -> UVector4: ...
    @property
    def ttos(self) -> UVector4: ...
    @property
    def ttot(self) -> UVector4: ...
    @property
    def ttoq(self) -> UVector4: ...
    @property
    def ttoo(self) -> UVector4: ...
    @property
    def ttol(self) -> UVector4: ...
    @property
    def ttls(self) -> UVector4: ...
    @property
    def ttlt(self) -> UVector4: ...
    @property
    def ttlq(self) -> UVector4: ...
    @property
    def ttlo(self) -> UVector4: ...
    @property
    def ttll(self) -> UVector4: ...
    @property
    def tqss(self) -> UVector4: ...
    @property
    def tqst(self) -> UVector4: ...
    @property
    def tqsq(self) -> UVector4: ...
    @property
    def tqso(self) -> UVector4: ...
    @property
    def tqsl(self) -> UVector4: ...
    @property
    def tqts(self) -> UVector4: ...
    @property
    def tqtt(self) -> UVector4: ...
    @property
    def tqtq(self) -> UVector4: ...
    @property
    def tqto(self) -> UVector4: ...
    @property
    def tqtl(self) -> UVector4: ...
    @property
    def tqqs(self) -> UVector4: ...
    @property
    def tqqt(self) -> UVector4: ...
    @property
    def tqqq(self) -> UVector4: ...
    @property
    def tqqo(self) -> UVector4: ...
    @property
    def tqql(self) -> UVector4: ...
    @property
    def tqos(self) -> UVector4: ...
    @property
    def tqot(self) -> UVector4: ...
    @property
    def tqoq(self) -> UVector4: ...
    @property
    def tqoo(self) -> UVector4: ...
    @property
    def tqol(self) -> UVector4: ...
    @property
    def tqls(self) -> UVector4: ...
    @property
    def tqlt(self) -> UVector4: ...
    @property
    def tqlq(self) -> UVector4: ...
    @property
    def tqlo(self) -> UVector4: ...
    @property
    def tqll(self) -> UVector4: ...
    @property
    def toss(self) -> UVector4: ...
    @property
    def tost(self) -> UVector4: ...
    @property
    def tosq(self) -> UVector4: ...
    @property
    def toso(self) -> UVector4: ...
    @property
    def tosl(self) -> UVector4: ...
    @property
    def tots(self) -> UVector4: ...
    @property
    def tott(self) -> UVector4: ...
    @property
    def totq(self) -> UVector4: ...
    @property
    def toto(self) -> UVector4: ...
    @property
    def totl(self) -> UVector4: ...
    @property
    def toqs(self) -> UVector4: ...
    @property
    def toqt(self) -> UVector4: ...
    @property
    def toqq(self) -> UVector4: ...
    @property
    def toqo(self) -> UVector4: ...
    @property
    def toql(self) -> UVector4: ...
    @property
    def toos(self) -> UVector4: ...
    @property
    def toot(self) -> UVector4: ...
    @property
    def tooq(self) -> UVector4: ...
    @property
    def tooo(self) -> UVector4: ...
    @property
    def tool(self) -> UVector4: ...
    @property
    def tols(self) -> UVector4: ...
    @property
    def tolt(self) -> UVector4: ...
    @property
    def tolq(self) -> UVector4: ...
    @property
    def tolo(self) -> UVector4: ...
    @property
    def toll(self) -> UVector4: ...
    @property
    def tlss(self) -> UVector4: ...
    @property
    def tlst(self) -> UVector4: ...
    @property
    def tlsq(self) -> UVector4: ...
    @property
    def tlso(self) -> UVector4: ...
    @property
    def tlsl(self) -> UVector4: ...
    @property
    def tlts(self) -> UVector4: ...
    @property
    def tltt(self) -> UVector4: ...
    @property
    def tltq(self) -> UVector4: ...
    @property
    def tlto(self) -> UVector4: ...
    @property
    def tltl(self) -> UVector4: ...
    @property
    def tlqs(self) -> UVector4: ...
    @property
    def tlqt(self) -> UVector4: ...
    @property
    def tlqq(self) -> UVector4: ...
    @property
    def tlqo(self) -> UVector4: ...
    @property
    def tlql(self) -> UVector4: ...
    @property
    def tlos(self) -> UVector4: ...
    @property
    def tlot(self) -> UVector4: ...
    @property
    def tloq(self) -> UVector4: ...
    @property
    def tloo(self) -> UVector4: ...
    @property
    def tlol(self) -> UVector4: ...
    @property
    def tlls(self) -> UVector4: ...
    @property
    def tllt(self) -> UVector4: ...
    @property
    def tllq(self) -> UVector4: ...
    @property
    def tllo(self) -> UVector4: ...
    @property
    def tlll(self) -> UVector4: ...
    @property
    def qsss(self) -> UVector4: ...
    @property
    def qsst(self) -> UVector4: ...
    @property
    def qssq(self) -> UVector4: ...
    @property
    def qsso(self) -> UVector4: ...
    @property
    def qssl(self) -> UVector4: ...
    @property
    def qsts(self) -> UVector4: ...
    @property
    def qstt(self) -> UVector4: ...
    @property
    def qstq(self) -> UVector4: ...
    @property
    def qsto(self) -> UVector4: ...
    @property
    def qstl(self) -> UVector4: ...
    @property
    def qsqs(self) -> UVector4: ...
    @property
    def qsqt(self) -> UVector4: ...
    @property
    def qsqq(self) -> UVector4: ...
    @property
    def qsqo(self) -> UVector4: ...
    @property
    def qsql(self) -> UVector4: ...
    @property
    def qsos(self) -> UVector4: ...
    @property
    def qsot(self) -> UVector4: ...
    @property
    def qsoq(self) -> UVector4: ...
    @property
    def qsoo(self) -> UVector4: ...
    @property
    def qsol(self) -> UVector4: ...
    @property
    def qsls(self) -> UVector4: ...
    @property
    def qslt(self) -> UVector4: ...
    @property
    def qslq(self) -> UVector4: ...
    @property
    def qslo(self) -> UVector4: ...
    @property
    def qsll(self) -> UVector4: ...
    @property
    def qtss(self) -> UVector4: ...
    @property
    def qtst(self) -> UVector4: ...
    @property
    def qtsq(self) -> UVector4: ...
    @property
    def qtso(self) -> UVector4: ...
    @property
    def qtsl(self) -> UVector4: ...
    @property
    def qtts(self) -> UVector4: ...
    @property
    def qttt(self) -> UVector4: ...
    @property
    def qttq(self) -> UVector4: ...
    @property
    def qtto(self) -> UVector4: ...
    @property
    def qttl(self) -> UVector4: ...
    @property
    def qtqs(self) -> UVector4: ...
    @property
    def qtqt(self) -> UVector4: ...
    @property
    def qtqq(self) -> UVector4: ...
    @property
    def qtqo(self) -> UVector4: ...
    @property
    def qtql(self) -> UVector4: ...
    @property
    def qtos(self) -> UVector4: ...
    @property
    def qtot(self) -> UVector4: ...
    @property
    def qtoq(self) -> UVector4: ...
    @property
    def qtoo(self) -> UVector4: ...
    @property
    def qtol(self) -> UVector4: ...
    @property
    def qtls(self) -> UVector4: ...
    @property
    def qtlt(self) -> UVector4: ...
    @property
    def qtlq(self) -> UVector4: ...
    @property
    def qtlo(self) -> UVector4: ...
    @property
    def qtll(self) -> UVector4: ...
    @property
    def qqss(self) -> UVector4: ...
    @property
    def qqst(self) -> UVector4: ...
    @property
    def qqsq(self) -> UVector4: ...
    @property
    def qqso(self) -> UVector4: ...
    @property
    def qqsl(self) -> UVector4: ...
    @property
    def qqts(self) -> UVector4: ...
    @property
    def qqtt(self) -> UVector4: ...
    @property
    def qqtq(self) -> UVector4: ...
    @property
    def qqto(self) -> UVector4: ...
    @property
    def qqtl(self) -> UVector4: ...
    @property
    def qqqs(self) -> UVector4: ...
    @property
    def qqqt(self) -> UVector4: ...
    @property
    def qqqq(self) -> UVector4: ...
    @property
    def qqqo(self) -> UVector4: ...
    @property
    def qqql(self) -> UVector4: ...
    @property
    def qqos(self) -> UVector4: ...
    @property
    def qqot(self) -> UVector4: ...
    @property
    def qqoq(self) -> UVector4: ...
    @property
    def qqoo(self) -> UVector4: ...
    @property
    def qqol(self) -> UVector4: ...
    @property
    def qqls(self) -> UVector4: ...
    @property
    def qqlt(self) -> UVector4: ...
    @property
    def qqlq(self) -> UVector4: ...
    @property
    def qqlo(self) -> UVector4: ...
    @property
    def qqll(self) -> UVector4: ...
    @property
    def qoss(self) -> UVector4: ...
    @property
    def qost(self) -> UVector4: ...
    @property
    def qosq(self) -> UVector4: ...
    @property
    def qoso(self) -> UVector4: ...
    @property
    def qosl(self) -> UVector4: ...
    @property
    def qots(self) -> UVector4: ...
    @property
    def qott(self) -> UVector4: ...
    @property
    def qotq(self) -> UVector4: ...
    @property
    def qoto(self) -> UVector4: ...
    @property
    def qotl(self) -> UVector4: ...
    @property
    def qoqs(self) -> UVector4: ...
    @property
    def qoqt(self) -> UVector4: ...
    @property
    def qoqq(self) -> UVector4: ...
    @property
    def qoqo(self) -> UVector4: ...
    @property
    def qoql(self) -> UVector4: ...
    @property
    def qoos(self) -> UVector4: ...
    @property
    def qoot(self) -> UVector4: ...
    @property
    def qooq(self) -> UVector4: ...
    @property
    def qooo(self) -> UVector4: ...
    @property
    def qool(self) -> UVector4: ...
    @property
    def qols(self) -> UVector4: ...
    @property
    def qolt(self) -> UVector4: ...
    @property
    def qolq(self) -> UVector4: ...
    @property
    def qolo(self) -> UVector4: ...
    @property
    def qoll(self) -> UVector4: ...
    @property
    def qlss(self) -> UVector4: ...
    @property
    def qlst(self) -> UVector4: ...
    @property
    def qlsq(self) -> UVector4: ...
    @property
    def qlso(self) -> UVector4: ...
    @property
    def qlsl(self) -> UVector4: ...
    @property
    def qlts(self) -> UVector4: ...
    @property
    def qltt(self) -> UVector4: ...
    @property
    def qltq(self) -> UVector4: ...
    @property
    def qlto(self) -> UVector4: ...
    @property
    def qltl(self) -> UVector4: ...
    @property
    def qlqs(self) -> UVector4: ...
    @property
    def qlqt(self) -> UVector4: ...
    @property
    def qlqq(self) -> UVector4: ...
    @property
    def qlqo(self) -> UVector4: ...
    @property
    def qlql(self) -> UVector4: ...
    @property
    def qlos(self) -> UVector4: ...
    @property
    def qlot(self) -> UVector4: ...
    @property
    def qloq(self) -> UVector4: ...
    @property
    def qloo(self) -> UVector4: ...
    @property
    def qlol(self) -> UVector4: ...
    @property
    def qlls(self) -> UVector4: ...
    @property
    def qllt(self) -> UVector4: ...
    @property
    def qllq(self) -> UVector4: ...
    @property
    def qllo(self) -> UVector4: ...
    @property
    def qlll(self) -> UVector4: ...
    @property
    def osss(self) -> UVector4: ...
    @property
    def osst(self) -> UVector4: ...
    @property
    def ossq(self) -> UVector4: ...
    @property
    def osso(self) -> UVector4: ...
    @property
    def ossl(self) -> UVector4: ...
    @property
    def osts(self) -> UVector4: ...
    @property
    def ostt(self) -> UVector4: ...
    @property
    def ostq(self) -> UVector4: ...
    @property
    def osto(self) -> UVector4: ...
    @property
    def ostl(self) -> UVector4: ...
    @property
    def osqs(self) -> UVector4: ...
    @property
    def osqt(self) -> UVector4: ...
    @property
    def osqq(self) -> UVector4: ...
    @property
    def osqo(self) -> UVector4: ...
    @property
    def osql(self) -> UVector4: ...
    @property
    def osos(self) -> UVector4: ...
    @property
    def osot(self) -> UVector4: ...
    @property
    def osoq(self) -> UVector4: ...
    @property
    def osoo(self) -> UVector4: ...
    @property
    def osol(self) -> UVector4: ...
    @property
    def osls(self) -> UVector4: ...
    @property
    def oslt(self) -> UVector4: ...
    @property
    def oslq(self) -> UVector4: ...
    @property
    def oslo(self) -> UVector4: ...
    @property
    def osll(self) -> UVector4: ...
    @property
    def otss(self) -> UVector4: ...
    @property
    def otst(self) -> UVector4: ...
    @property
    def otsq(self) -> UVector4: ...
    @property
    def otso(self) -> UVector4: ...
    @property
    def otsl(self) -> UVector4: ...
    @property
    def otts(self) -> UVector4: ...
    @property
    def ottt(self) -> UVector4: ...
    @property
    def ottq(self) -> UVector4: ...
    @property
    def otto(self) -> UVector4: ...
    @property
    def ottl(self) -> UVector4: ...
    @property
    def otqs(self) -> UVector4: ...
    @property
    def otqt(self) -> UVector4: ...
    @property
    def otqq(self) -> UVector4: ...
    @property
    def otqo(self) -> UVector4: ...
    @property
    def otql(self) -> UVector4: ...
    @property
    def otos(self) -> UVector4: ...
    @property
    def otot(self) -> UVector4: ...
    @property
    def otoq(self) -> UVector4: ...
    @property
    def otoo(self) -> UVector4: ...
    @property
    def otol(self) -> UVector4: ...
    @property
    def otls(self) -> UVector4: ...
    @property
    def otlt(self) -> UVector4: ...
    @property
    def otlq(self) -> UVector4: ...
    @property
    def otlo(self) -> UVector4: ...
    @property
    def otll(self) -> UVector4: ...
    @property
    def oqss(self) -> UVector4: ...
    @property
    def oqst(self) -> UVector4: ...
    @property
    def oqsq(self) -> UVector4: ...
    @property
    def oqso(self) -> UVector4: ...
    @property
    def oqsl(self) -> UVector4: ...
    @property
    def oqts(self) -> UVector4: ...
    @property
    def oqtt(self) -> UVector4: ...
    @property
    def oqtq(self) -> UVector4: ...
    @property
    def oqto(self) -> UVector4: ...
    @property
    def oqtl(self) -> UVector4: ...
    @property
    def oqqs(self) -> UVector4: ...
    @property
    def oqqt(self) -> UVector4: ...
    @property
    def oqqq(self) -> UVector4: ...
    @property
    def oqqo(self) -> UVector4: ...
    @property
    def oqql(self) -> UVector4: ...
    @property
    def oqos(self) -> UVector4: ...
    @property
    def oqot(self) -> UVector4: ...
    @property
    def oqoq(self) -> UVector4: ...
    @property
    def oqoo(self) -> UVector4: ...
    @property
    def oqol(self) -> UVector4: ...
    @property
    def oqls(self) -> UVector4: ...
    @property
    def oqlt(self) -> UVector4: ...
    @property
    def oqlq(self) -> UVector4: ...
    @property
    def oqlo(self) -> UVector4: ...
    @property
    def oqll(self) -> UVector4: ...
    @property
    def ooss(self) -> UVector4: ...
    @property
    def oost(self) -> UVector4: ...
    @property
    def oosq(self) -> UVector4: ...
    @property
    def ooso(self) -> UVector4: ...
    @property
    def oosl(self) -> UVector4: ...
    @property
    def oots(self) -> UVector4: ...
    @property
    def oott(self) -> UVector4: ...
    @property
    def ootq(self) -> UVector4: ...
    @property
    def ooto(self) -> UVector4: ...
    @property
    def ootl(self) -> UVector4: ...
    @property
    def ooqs(self) -> UVector4: ...
    @property
    def ooqt(self) -> UVector4: ...
    @property
    def ooqq(self) -> UVector4: ...
    @property
    def ooqo(self) -> UVector4: ...
    @property
    def ooql(self) -> UVector4: ...
    @property
    def ooos(self) -> UVector4: ...
    @property
    def ooot(self) -> UVector4: ...
    @property
    def oooq(self) -> UVector4: ...
    @property
    def ools(self) -> UVector4: ...
    @property
    def oolt(self) -> UVector4: ...
    @property
    def oolq(self) -> UVector4: ...
    @property
    def olss(self) -> UVector4: ...
    @property
    def olst(self) -> UVector4: ...
    @property
    def olsq(self) -> UVector4: ...
    @property
    def olso(self) -> UVector4: ...
    @property
    def olsl(self) -> UVector4: ...
    @property
    def olts(self) -> UVector4: ...
    @property
    def oltt(self) -> UVector4: ...
    @property
    def oltq(self) -> UVector4: ...
    @property
    def olto(self) -> UVector4: ...
    @property
    def oltl(self) -> UVector4: ...
    @property
    def olqs(self) -> UVector4: ...
    @property
    def olqt(self) -> UVector4: ...
    @property
    def olqq(self) -> UVector4: ...
    @property
    def olqo(self) -> UVector4: ...
    @property
    def olql(self) -> UVector4: ...
    @property
    def olos(self) -> UVector4: ...
    @property
    def olot(self) -> UVector4: ...
    @property
    def oloq(self) -> UVector4: ...
    @property
    def olls(self) -> UVector4: ...
    @property
    def ollt(self) -> UVector4: ...
    @property
    def ollq(self) -> UVector4: ...
    @property
    def lsss(self) -> UVector4: ...
    @property
    def lsst(self) -> UVector4: ...
    @property
    def lssq(self) -> UVector4: ...
    @property
    def lsso(self) -> UVector4: ...
    @property
    def lssl(self) -> UVector4: ...
    @property
    def lsts(self) -> UVector4: ...
    @property
    def lstt(self) -> UVector4: ...
    @property
    def lstq(self) -> UVector4: ...
    @property
    def lsto(self) -> UVector4: ...
    @property
    def lstl(self) -> UVector4: ...
    @property
    def lsqs(self) -> UVector4: ...
    @property
    def lsqt(self) -> UVector4: ...
    @property
    def lsqq(self) -> UVector4: ...
    @property
    def lsqo(self) -> UVector4: ...
    @property
    def lsql(self) -> UVector4: ...
    @property
    def lsos(self) -> UVector4: ...
    @property
    def lsot(self) -> UVector4: ...
    @property
    def lsoq(self) -> UVector4: ...
    @property
    def lsoo(self) -> UVector4: ...
    @property
    def lsol(self) -> UVector4: ...
    @property
    def lsls(self) -> UVector4: ...
    @property
    def lslt(self) -> UVector4: ...
    @property
    def lslq(self) -> UVector4: ...
    @property
    def lslo(self) -> UVector4: ...
    @property
    def lsll(self) -> UVector4: ...
    @property
    def ltss(self) -> UVector4: ...
    @property
    def ltst(self) -> UVector4: ...
    @property
    def ltsq(self) -> UVector4: ...
    @property
    def ltso(self) -> UVector4: ...
    @property
    def ltsl(self) -> UVector4: ...
    @property
    def ltts(self) -> UVector4: ...
    @property
    def lttt(self) -> UVector4: ...
    @property
    def lttq(self) -> UVector4: ...
    @property
    def ltto(self) -> UVector4: ...
    @property
    def lttl(self) -> UVector4: ...
    @property
    def ltqs(self) -> UVector4: ...
    @property
    def ltqt(self) -> UVector4: ...
    @property
    def ltqq(self) -> UVector4: ...
    @property
    def ltqo(self) -> UVector4: ...
    @property
    def ltql(self) -> UVector4: ...
    @property
    def ltos(self) -> UVector4: ...
    @property
    def ltot(self) -> UVector4: ...
    @property
    def ltoq(self) -> UVector4: ...
    @property
    def ltoo(self) -> UVector4: ...
    @property
    def ltol(self) -> UVector4: ...
    @property
    def ltls(self) -> UVector4: ...
    @property
    def ltlt(self) -> UVector4: ...
    @property
    def ltlq(self) -> UVector4: ...
    @property
    def ltlo(self) -> UVector4: ...
    @property
    def ltll(self) -> UVector4: ...
    @property
    def lqss(self) -> UVector4: ...
    @property
    def lqst(self) -> UVector4: ...
    @property
    def lqsq(self) -> UVector4: ...
    @property
    def lqso(self) -> UVector4: ...
    @property
    def lqsl(self) -> UVector4: ...
    @property
    def lqts(self) -> UVector4: ...
    @property
    def lqtt(self) -> UVector4: ...
    @property
    def lqtq(self) -> UVector4: ...
    @property
    def lqto(self) -> UVector4: ...
    @property
    def lqtl(self) -> UVector4: ...
    @property
    def lqqs(self) -> UVector4: ...
    @property
    def lqqt(self) -> UVector4: ...
    @property
    def lqqq(self) -> UVector4: ...
    @property
    def lqqo(self) -> UVector4: ...
    @property
    def lqql(self) -> UVector4: ...
    @property
    def lqos(self) -> UVector4: ...
    @property
    def lqot(self) -> UVector4: ...
    @property
    def lqoq(self) -> UVector4: ...
    @property
    def lqoo(self) -> UVector4: ...
    @property
    def lqol(self) -> UVector4: ...
    @property
    def lqls(self) -> UVector4: ...
    @property
    def lqlt(self) -> UVector4: ...
    @property
    def lqlq(self) -> UVector4: ...
    @property
    def lqlo(self) -> UVector4: ...
    @property
    def lqll(self) -> UVector4: ...
    @property
    def loss(self) -> UVector4: ...
    @property
    def lost(self) -> UVector4: ...
    @property
    def losq(self) -> UVector4: ...
    @property
    def loso(self) -> UVector4: ...
    @property
    def losl(self) -> UVector4: ...
    @property
    def lots(self) -> UVector4: ...
    @property
    def lott(self) -> UVector4: ...
    @property
    def lotq(self) -> UVector4: ...
    @property
    def loto(self) -> UVector4: ...
    @property
    def lotl(self) -> UVector4: ...
    @property
    def loqs(self) -> UVector4: ...
    @property
    def loqt(self) -> UVector4: ...
    @property
    def loqq(self) -> UVector4: ...
    @property
    def loqo(self) -> UVector4: ...
    @property
    def loql(self) -> UVector4: ...
    @property
    def loos(self) -> UVector4: ...
    @property
    def loot(self) -> UVector4: ...
    @property
    def looq(self) -> UVector4: ...
    @property
    def lols(self) -> UVector4: ...
    @property
    def lolt(self) -> UVector4: ...
    @property
    def lolq(self) -> UVector4: ...
    @property
    def llss(self) -> UVector4: ...
    @property
    def llst(self) -> UVector4: ...
    @property
    def llsq(self) -> UVector4: ...
    @property
    def llso(self) -> UVector4: ...
    @property
    def llsl(self) -> UVector4: ...
    @property
    def llts(self) -> UVector4: ...
    @property
    def lltt(self) -> UVector4: ...
    @property
    def lltq(self) -> UVector4: ...
    @property
    def llto(self) -> UVector4: ...
    @property
    def lltl(self) -> UVector4: ...
    @property
    def llqs(self) -> UVector4: ...
    @property
    def llqt(self) -> UVector4: ...
    @property
    def llqq(self) -> UVector4: ...
    @property
    def llqo(self) -> UVector4: ...
    @property
    def llql(self) -> UVector4: ...
    @property
    def llos(self) -> UVector4: ...
    @property
    def llot(self) -> UVector4: ...
    @property
    def lloq(self) -> UVector4: ...
    @property
    def llls(self) -> UVector4: ...
    @property
    def lllt(self) -> UVector4: ...
    @property
    def lllq(self) -> UVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> UVector2: ...
    @property
    def uv(self) -> UVector2: ...
    @property
    def uo(self) -> UVector2: ...
    @property
    def ul(self) -> UVector2: ...
    @property
    def vu(self) -> UVector2: ...
    @property
    def vv(self) -> UVector2: ...
    @property
    def vo(self) -> UVector2: ...
    @property
    def vl(self) -> UVector2: ...
    @property
    def ou(self) -> UVector2: ...
    @property
    def ov(self) -> UVector2: ...
    @property
    def lu(self) -> UVector2: ...
    @property
    def lv(self) -> UVector2: ...
    @property
    def uuu(self) -> UVector3: ...
    @property
    def uuv(self) -> UVector3: ...
    @property
    def uuo(self) -> UVector3: ...
    @property
    def uul(self) -> UVector3: ...
    @property
    def uvu(self) -> UVector3: ...
    @property
    def uvv(self) -> UVector3: ...
    @property
    def uvo(self) -> UVector3: ...
    @property
    def uvl(self) -> UVector3: ...
    @property
    def uou(self) -> UVector3: ...
    @property
    def uov(self) -> UVector3: ...
    @property
    def uoo(self) -> UVector3: ...
    @property
    def uol(self) -> UVector3: ...
    @property
    def ulu(self) -> UVector3: ...
    @property
    def ulv(self) -> UVector3: ...
    @property
    def ulo(self) -> UVector3: ...
    @property
    def ull(self) -> UVector3: ...
    @property
    def vuu(self) -> UVector3: ...
    @property
    def vuv(self) -> UVector3: ...
    @property
    def vuo(self) -> UVector3: ...
    @property
    def vul(self) -> UVector3: ...
    @property
    def vvu(self) -> UVector3: ...
    @property
    def vvv(self) -> UVector3: ...
    @property
    def vvo(self) -> UVector3: ...
    @property
    def vvl(self) -> UVector3: ...
    @property
    def vou(self) -> UVector3: ...
    @property
    def vov(self) -> UVector3: ...
    @property
    def voo(self) -> UVector3: ...
    @property
    def vol(self) -> UVector3: ...
    @property
    def vlu(self) -> UVector3: ...
    @property
    def vlv(self) -> UVector3: ...
    @property
    def vlo(self) -> UVector3: ...
    @property
    def vll(self) -> UVector3: ...
    @property
    def ouu(self) -> UVector3: ...
    @property
    def ouv(self) -> UVector3: ...
    @property
    def ouo(self) -> UVector3: ...
    @property
    def oul(self) -> UVector3: ...
    @property
    def ovu(self) -> UVector3: ...
    @property
    def ovv(self) -> UVector3: ...
    @property
    def ovo(self) -> UVector3: ...
    @property
    def ovl(self) -> UVector3: ...
    @property
    def oou(self) -> UVector3: ...
    @property
    def oov(self) -> UVector3: ...
    @property
    def olu(self) -> UVector3: ...
    @property
    def olv(self) -> UVector3: ...
    @property
    def luu(self) -> UVector3: ...
    @property
    def luv(self) -> UVector3: ...
    @property
    def luo(self) -> UVector3: ...
    @property
    def lul(self) -> UVector3: ...
    @property
    def lvu(self) -> UVector3: ...
    @property
    def lvv(self) -> UVector3: ...
    @property
    def lvo(self) -> UVector3: ...
    @property
    def lvl(self) -> UVector3: ...
    @property
    def lou(self) -> UVector3: ...
    @property
    def lov(self) -> UVector3: ...
    @property
    def llu(self) -> UVector3: ...
    @property
    def llv(self) -> UVector3: ...
    @property
    def uuuu(self) -> UVector4: ...
    @property
    def uuuv(self) -> UVector4: ...
    @property
    def uuuo(self) -> UVector4: ...
    @property
    def uuul(self) -> UVector4: ...
    @property
    def uuvu(self) -> UVector4: ...
    @property
    def uuvv(self) -> UVector4: ...
    @property
    def uuvo(self) -> UVector4: ...
    @property
    def uuvl(self) -> UVector4: ...
    @property
    def uuou(self) -> UVector4: ...
    @property
    def uuov(self) -> UVector4: ...
    @property
    def uuoo(self) -> UVector4: ...
    @property
    def uuol(self) -> UVector4: ...
    @property
    def uulu(self) -> UVector4: ...
    @property
    def uulv(self) -> UVector4: ...
    @property
    def uulo(self) -> UVector4: ...
    @property
    def uull(self) -> UVector4: ...
    @property
    def uvuu(self) -> UVector4: ...
    @property
    def uvuv(self) -> UVector4: ...
    @property
    def uvuo(self) -> UVector4: ...
    @property
    def uvul(self) -> UVector4: ...
    @property
    def uvvu(self) -> UVector4: ...
    @property
    def uvvv(self) -> UVector4: ...
    @property
    def uvvo(self) -> UVector4: ...
    @property
    def uvvl(self) -> UVector4: ...
    @property
    def uvou(self) -> UVector4: ...
    @property
    def uvov(self) -> UVector4: ...
    @property
    def uvoo(self) -> UVector4: ...
    @property
    def uvol(self) -> UVector4: ...
    @property
    def uvlu(self) -> UVector4: ...
    @property
    def uvlv(self) -> UVector4: ...
    @property
    def uvlo(self) -> UVector4: ...
    @property
    def uvll(self) -> UVector4: ...
    @property
    def uouu(self) -> UVector4: ...
    @property
    def uouv(self) -> UVector4: ...
    @property
    def uouo(self) -> UVector4: ...
    @property
    def uoul(self) -> UVector4: ...
    @property
    def uovu(self) -> UVector4: ...
    @property
    def uovv(self) -> UVector4: ...
    @property
    def uovo(self) -> UVector4: ...
    @property
    def uovl(self) -> UVector4: ...
    @property
    def uoou(self) -> UVector4: ...
    @property
    def uoov(self) -> UVector4: ...
    @property
    def uooo(self) -> UVector4: ...
    @property
    def uool(self) -> UVector4: ...
    @property
    def uolu(self) -> UVector4: ...
    @property
    def uolv(self) -> UVector4: ...
    @property
    def uolo(self) -> UVector4: ...
    @property
    def uoll(self) -> UVector4: ...
    @property
    def uluu(self) -> UVector4: ...
    @property
    def uluv(self) -> UVector4: ...
    @property
    def uluo(self) -> UVector4: ...
    @property
    def ulul(self) -> UVector4: ...
    @property
    def ulvu(self) -> UVector4: ...
    @property
    def ulvv(self) -> UVector4: ...
    @property
    def ulvo(self) -> UVector4: ...
    @property
    def ulvl(self) -> UVector4: ...
    @property
    def ulou(self) -> UVector4: ...
    @property
    def ulov(self) -> UVector4: ...
    @property
    def uloo(self) -> UVector4: ...
    @property
    def ulol(self) -> UVector4: ...
    @property
    def ullu(self) -> UVector4: ...
    @property
    def ullv(self) -> UVector4: ...
    @property
    def ullo(self) -> UVector4: ...
    @property
    def ulll(self) -> UVector4: ...
    @property
    def vuuu(self) -> UVector4: ...
    @property
    def vuuv(self) -> UVector4: ...
    @property
    def vuuo(self) -> UVector4: ...
    @property
    def vuul(self) -> UVector4: ...
    @property
    def vuvu(self) -> UVector4: ...
    @property
    def vuvv(self) -> UVector4: ...
    @property
    def vuvo(self) -> UVector4: ...
    @property
    def vuvl(self) -> UVector4: ...
    @property
    def vuou(self) -> UVector4: ...
    @property
    def vuov(self) -> UVector4: ...
    @property
    def vuoo(self) -> UVector4: ...
    @property
    def vuol(self) -> UVector4: ...
    @property
    def vulu(self) -> UVector4: ...
    @property
    def vulv(self) -> UVector4: ...
    @property
    def vulo(self) -> UVector4: ...
    @property
    def vull(self) -> UVector4: ...
    @property
    def vvuu(self) -> UVector4: ...
    @property
    def vvuv(self) -> UVector4: ...
    @property
    def vvuo(self) -> UVector4: ...
    @property
    def vvul(self) -> UVector4: ...
    @property
    def vvvu(self) -> UVector4: ...
    @property
    def vvvv(self) -> UVector4: ...
    @property
    def vvvo(self) -> UVector4: ...
    @property
    def vvvl(self) -> UVector4: ...
    @property
    def vvou(self) -> UVector4: ...
    @property
    def vvov(self) -> UVector4: ...
    @property
    def vvoo(self) -> UVector4: ...
    @property
    def vvol(self) -> UVector4: ...
    @property
    def vvlu(self) -> UVector4: ...
    @property
    def vvlv(self) -> UVector4: ...
    @property
    def vvlo(self) -> UVector4: ...
    @property
    def vvll(self) -> UVector4: ...
    @property
    def vouu(self) -> UVector4: ...
    @property
    def vouv(self) -> UVector4: ...
    @property
    def vouo(self) -> UVector4: ...
    @property
    def voul(self) -> UVector4: ...
    @property
    def vovu(self) -> UVector4: ...
    @property
    def vovv(self) -> UVector4: ...
    @property
    def vovo(self) -> UVector4: ...
    @property
    def vovl(self) -> UVector4: ...
    @property
    def voou(self) -> UVector4: ...
    @property
    def voov(self) -> UVector4: ...
    @property
    def vooo(self) -> UVector4: ...
    @property
    def vool(self) -> UVector4: ...
    @property
    def volu(self) -> UVector4: ...
    @property
    def volv(self) -> UVector4: ...
    @property
    def volo(self) -> UVector4: ...
    @property
    def voll(self) -> UVector4: ...
    @property
    def vluu(self) -> UVector4: ...
    @property
    def vluv(self) -> UVector4: ...
    @property
    def vluo(self) -> UVector4: ...
    @property
    def vlul(self) -> UVector4: ...
    @property
    def vlvu(self) -> UVector4: ...
    @property
    def vlvv(self) -> UVector4: ...
    @property
    def vlvo(self) -> UVector4: ...
    @property
    def vlvl(self) -> UVector4: ...
    @property
    def vlou(self) -> UVector4: ...
    @property
    def vlov(self) -> UVector4: ...
    @property
    def vloo(self) -> UVector4: ...
    @property
    def vlol(self) -> UVector4: ...
    @property
    def vllu(self) -> UVector4: ...
    @property
    def vllv(self) -> UVector4: ...
    @property
    def vllo(self) -> UVector4: ...
    @property
    def vlll(self) -> UVector4: ...
    @property
    def ouuu(self) -> UVector4: ...
    @property
    def ouuv(self) -> UVector4: ...
    @property
    def ouuo(self) -> UVector4: ...
    @property
    def ouul(self) -> UVector4: ...
    @property
    def ouvu(self) -> UVector4: ...
    @property
    def ouvv(self) -> UVector4: ...
    @property
    def ouvo(self) -> UVector4: ...
    @property
    def ouvl(self) -> UVector4: ...
    @property
    def ouou(self) -> UVector4: ...
    @property
    def ouov(self) -> UVector4: ...
    @property
    def ouoo(self) -> UVector4: ...
    @property
    def ouol(self) -> UVector4: ...
    @property
    def oulu(self) -> UVector4: ...
    @property
    def oulv(self) -> UVector4: ...
    @property
    def oulo(self) -> UVector4: ...
    @property
    def oull(self) -> UVector4: ...
    @property
    def ovuu(self) -> UVector4: ...
    @property
    def ovuv(self) -> UVector4: ...
    @property
    def ovuo(self) -> UVector4: ...
    @property
    def ovul(self) -> UVector4: ...
    @property
    def ovvu(self) -> UVector4: ...
    @property
    def ovvv(self) -> UVector4: ...
    @property
    def ovvo(self) -> UVector4: ...
    @property
    def ovvl(self) -> UVector4: ...
    @property
    def ovou(self) -> UVector4: ...
    @property
    def ovov(self) -> UVector4: ...
    @property
    def ovoo(self) -> UVector4: ...
    @property
    def ovol(self) -> UVector4: ...
    @property
    def ovlu(self) -> UVector4: ...
    @property
    def ovlv(self) -> UVector4: ...
    @property
    def ovlo(self) -> UVector4: ...
    @property
    def ovll(self) -> UVector4: ...
    @property
    def oouu(self) -> UVector4: ...
    @property
    def oouv(self) -> UVector4: ...
    @property
    def oouo(self) -> UVector4: ...
    @property
    def ooul(self) -> UVector4: ...
    @property
    def oovu(self) -> UVector4: ...
    @property
    def oovv(self) -> UVector4: ...
    @property
    def oovo(self) -> UVector4: ...
    @property
    def oovl(self) -> UVector4: ...
    @property
    def ooou(self) -> UVector4: ...
    @property
    def ooov(self) -> UVector4: ...
    @property
    def oolu(self) -> UVector4: ...
    @property
    def oolv(self) -> UVector4: ...
    @property
    def oluu(self) -> UVector4: ...
    @property
    def oluv(self) -> UVector4: ...
    @property
    def oluo(self) -> UVector4: ...
    @property
    def olul(self) -> UVector4: ...
    @property
    def olvu(self) -> UVector4: ...
    @property
    def olvv(self) -> UVector4: ...
    @property
    def olvo(self) -> UVector4: ...
    @property
    def olvl(self) -> UVector4: ...
    @property
    def olou(self) -> UVector4: ...
    @property
    def olov(self) -> UVector4: ...
    @property
    def ollu(self) -> UVector4: ...
    @property
    def ollv(self) -> UVector4: ...
    @property
    def luuu(self) -> UVector4: ...
    @property
    def luuv(self) -> UVector4: ...
    @property
    def luuo(self) -> UVector4: ...
    @property
    def luul(self) -> UVector4: ...
    @property
    def luvu(self) -> UVector4: ...
    @property
    def luvv(self) -> UVector4: ...
    @property
    def luvo(self) -> UVector4: ...
    @property
    def luvl(self) -> UVector4: ...
    @property
    def luou(self) -> UVector4: ...
    @property
    def luov(self) -> UVector4: ...
    @property
    def luoo(self) -> UVector4: ...
    @property
    def luol(self) -> UVector4: ...
    @property
    def lulu(self) -> UVector4: ...
    @property
    def lulv(self) -> UVector4: ...
    @property
    def lulo(self) -> UVector4: ...
    @property
    def lull(self) -> UVector4: ...
    @property
    def lvuu(self) -> UVector4: ...
    @property
    def lvuv(self) -> UVector4: ...
    @property
    def lvuo(self) -> UVector4: ...
    @property
    def lvul(self) -> UVector4: ...
    @property
    def lvvu(self) -> UVector4: ...
    @property
    def lvvv(self) -> UVector4: ...
    @property
    def lvvo(self) -> UVector4: ...
    @property
    def lvvl(self) -> UVector4: ...
    @property
    def lvou(self) -> UVector4: ...
    @property
    def lvov(self) -> UVector4: ...
    @property
    def lvoo(self) -> UVector4: ...
    @property
    def lvol(self) -> UVector4: ...
    @property
    def lvlu(self) -> UVector4: ...
    @property
    def lvlv(self) -> UVector4: ...
    @property
    def lvlo(self) -> UVector4: ...
    @property
    def lvll(self) -> UVector4: ...
    @property
    def louu(self) -> UVector4: ...
    @property
    def louv(self) -> UVector4: ...
    @property
    def louo(self) -> UVector4: ...
    @property
    def loul(self) -> UVector4: ...
    @property
    def lovu(self) -> UVector4: ...
    @property
    def lovv(self) -> UVector4: ...
    @property
    def lovo(self) -> UVector4: ...
    @property
    def lovl(self) -> UVector4: ...
    @property
    def loou(self) -> UVector4: ...
    @property
    def loov(self) -> UVector4: ...
    @property
    def lolu(self) -> UVector4: ...
    @property
    def lolv(self) -> UVector4: ...
    @property
    def lluu(self) -> UVector4: ...
    @property
    def lluv(self) -> UVector4: ...
    @property
    def lluo(self) -> UVector4: ...
    @property
    def llul(self) -> UVector4: ...
    @property
    def llvu(self) -> UVector4: ...
    @property
    def llvv(self) -> UVector4: ...
    @property
    def llvo(self) -> UVector4: ...
    @property
    def llvl(self) -> UVector4: ...
    @property
    def llou(self) -> UVector4: ...
    @property
    def llov(self) -> UVector4: ...
    @property
    def lllu(self) -> UVector4: ...
    @property
    def lllv(self) -> UVector4: ...
    @property
    def oo(self) -> UVector2: ...
    @property
    def ol(self) -> UVector2: ...
    @property
    def lo(self) -> UVector2: ...
    @property
    def ll(self) -> UVector2: ...
    @property
    def ooo(self) -> UVector3: ...
    @property
    def ool(self) -> UVector3: ...
    @property
    def olo(self) -> UVector3: ...
    @property
    def oll(self) -> UVector3: ...
    @property
    def loo(self) -> UVector3: ...
    @property
    def lol(self) -> UVector3: ...
    @property
    def llo(self) -> UVector3: ...
    @property
    def lll(self) -> UVector3: ...
    @property
    def oooo(self) -> UVector4: ...
    @property
    def oool(self) -> UVector4: ...
    @property
    def oolo(self) -> UVector4: ...
    @property
    def ooll(self) -> UVector4: ...
    @property
    def oloo(self) -> UVector4: ...
    @property
    def olol(self) -> UVector4: ...
    @property
    def ollo(self) -> UVector4: ...
    @property
    def olll(self) -> UVector4: ...
    @property
    def looo(self) -> UVector4: ...
    @property
    def lool(self) -> UVector4: ...
    @property
    def lolo(self) -> UVector4: ...
    @property
    def loll(self) -> UVector4: ...
    @property
    def lloo(self) -> UVector4: ...
    @property
    def llol(self) -> UVector4: ...
    @property
    def lllo(self) -> UVector4: ...
    @property
    def llll(self) -> UVector4: ...
    def min(self, n: Number, /) -> UVector3: ...
    def max(self, n: Number, /) -> UVector3: ...
    def clamp(self, min: Number, max: Number, /) -> UVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[UVector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector3: ...

@final
class UVector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: UVector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[UVector3]: ...
    @overload
    def __getitem__(self, index: int) -> UVector3: ...
    @overload
    def __getitem__(self, index: slice) -> UVector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[UVector3]: ...

@final
class I64Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __add__(self, other: Number) -> I64Vector3: ...
    @overload
    def __radd__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __radd__(self, other: Number) -> I64Vector3: ...
    @overload
    def __sub__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __sub__(self, other: Number) -> I64Vector3: ...
    @overload
    def __rsub__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> I64Vector3: ...
    @overload
    def __mul__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __mul__(self, other: Number) -> I64Vector3: ...
    @overload
    def __rmul__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> I64Vector3: ...
    @overload
    def __truediv__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> I64Vector3: ...
    @overload
    def __rtruediv__(self, other: I64Vector3) -> I64Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> I64Vector3: ...
    def __neg__(self) -> I64Vector3: ...
    def __abs__(self) -> I64Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> I64Vector2: ...
    @property
    def xy(self) -> I64Vector2: ...
    @property
    def xz(self) -> I64Vector2: ...
    @property
    def xo(self) -> I64Vector2: ...
    @property
    def xl(self) -> I64Vector2: ...
    @property
    def yx(self) -> I64Vector2: ...
    @property
    def yy(self) -> I64Vector2: ...
    @property
    def yz(self) -> I64Vector2: ...
    @property
    def yo(self) -> I64Vector2: ...
    @property
    def yl(self) -> I64Vector2: ...
    @property
    def zx(self) -> I64Vector2: ...
    @property
    def zy(self) -> I64Vector2: ...
    @property
    def zz(self) -> I64Vector2: ...
    @property
    def zo(self) -> I64Vector2: ...
    @property
    def zl(self) -> I64Vector2: ...
    @property
    def ox(self) -> I64Vector2: ...
    @property
    def oy(self) -> I64Vector2: ...
    @property
    def oz(self) -> I64Vector2: ...
    @property
    def lx(self) -> I64Vector2: ...
    @property
    def ly(self) -> I64Vector2: ...
    @property
    def lz(self) -> I64Vector2: ...
    @property
    def xxx(self) -> I64Vector3: ...
    @property
    def xxy(self) -> I64Vector3: ...
    @property
    def xxz(self) -> I64Vector3: ...
    @property
    def xxo(self) -> I64Vector3: ...
    @property
    def xxl(self) -> I64Vector3: ...
    @property
    def xyx(self) -> I64Vector3: ...
    @property
    def xyy(self) -> I64Vector3: ...
    @property
    def xyz(self) -> I64Vector3: ...
    @property
    def xyo(self) -> I64Vector3: ...
    @property
    def xyl(self) -> I64Vector3: ...
    @property
    def xzx(self) -> I64Vector3: ...
    @property
    def xzy(self) -> I64Vector3: ...
    @property
    def xzz(self) -> I64Vector3: ...
    @property
    def xzo(self) -> I64Vector3: ...
    @property
    def xzl(self) -> I64Vector3: ...
    @property
    def xox(self) -> I64Vector3: ...
    @property
    def xoy(self) -> I64Vector3: ...
    @property
    def xoz(self) -> I64Vector3: ...
    @property
    def xoo(self) -> I64Vector3: ...
    @property
    def xol(self) -> I64Vector3: ...
    @property
    def xlx(self) -> I64Vector3: ...
    @property
    def xly(self) -> I64Vector3: ...
    @property
    def xlz(self) -> I64Vector3: ...
    @property
    def xlo(self) -> I64Vector3: ...
    @property
    def xll(self) -> I64Vector3: ...
    @property
    def yxx(self) -> I64Vector3: ...
    @property
    def yxy(self) -> I64Vector3: ...
    @property
    def yxz(self) -> I64Vector3: ...
    @property
    def yxo(self) -> I64Vector3: ...
    @property
    def yxl(self) -> I64Vector3: ...
    @property
    def yyx(self) -> I64Vector3: ...
    @property
    def yyy(self) -> I64Vector3: ...
    @property
    def yyz(self) -> I64Vector3: ...
    @property
    def yyo(self) -> I64Vector3: ...
    @property
    def yyl(self) -> I64Vector3: ...
    @property
    def yzx(self) -> I64Vector3: ...
    @property
    def yzy(self) -> I64Vector3: ...
    @property
    def yzz(self) -> I64Vector3: ...
    @property
    def yzo(self) -> I64Vector3: ...
    @property
    def yzl(self) -> I64Vector3: ...
    @property
    def yox(self) -> I64Vector3: ...
    @property
    def yoy(self) -> I64Vector3: ...
    @property
    def yoz(self) -> I64Vector3: ...
    @property
    def yoo(self) -> I64Vector3: ...
    @property
    def yol(self) -> I64Vector3: ...
    @property
    def ylx(self) -> I64Vector3: ...
    @property
    def yly(self) -> I64Vector3: ...
    @property
    def ylz(self) -> I64Vector3: ...
    @property
    def ylo(self) -> I64Vector3: ...
    @property
    def yll(self) -> I64Vector3: ...
    @property
    def zxx(self) -> I64Vector3: ...
    @property
    def zxy(self) -> I64Vector3: ...
    @property
    def zxz(self) -> I64Vector3: ...
    @property
    def zxo(self) -> I64Vector3: ...
    @property
    def zxl(self) -> I64Vector3: ...
    @property
    def zyx(self) -> I64Vector3: ...
    @property
    def zyy(self) -> I64Vector3: ...
    @property
    def zyz(self) -> I64Vector3: ...
    @property
    def zyo(self) -> I64Vector3: ...
    @property
    def zyl(self) -> I64Vector3: ...
    @property
    def zzx(self) -> I64Vector3: ...
    @property
    def zzy(self) -> I64Vector3: ...
    @property
    def zzz(self) -> I64Vector3: ...
    @property
    def zzo(self) -> I64Vector3: ...
    @property
    def zzl(self) -> I64Vector3: ...
    @property
    def zox(self) -> I64Vector3: ...
    @property
    def zoy(self) -> I64Vector3: ...
    @property
    def zoz(self) -> I64Vector3: ...
    @property
    def zoo(self) -> I64Vector3: ...
    @property
    def zol(self) -> I64Vector3: ...
    @property
    def zlx(self) -> I64Vector3: ...
    @property
    def zly(self) -> I64Vector3: ...
    @property
    def zlz(self) -> I64Vector3: ...
    @property
    def zlo(self) -> I64Vector3: ...
    @property
    def zll(self) -> I64Vector3: ...
    @property
    def oxx(self) -> I64Vector3: ...
    @property
    def oxy(self) -> I64Vector3: ...
    @property
    def oxz(self) -> I64Vector3: ...
    @property
    def oxo(self) -> I64Vector3: ...
    @property
    def oxl(self) -> I64Vector3: ...
    @property
    def oyx(self) -> I64Vector3: ...
    @property
    def oyy(self) -> I64Vector3: ...
    @property
    def oyz(self) -> I64Vector3: ...
    @property
    def oyo(self) -> I64Vector3: ...
    @property
    def oyl(self) -> I64Vector3: ...
    @property
    def ozx(self) -> I64Vector3: ...
    @property
    def ozy(self) -> I64Vector3: ...
    @property
    def ozz(self) -> I64Vector3: ...
    @property
    def ozo(self) -> I64Vector3: ...
    @property
    def ozl(self) -> I64Vector3: ...
    @property
    def oox(self) -> I64Vector3: ...
    @property
    def ooy(self) -> I64Vector3: ...
    @property
    def ooz(self) -> I64Vector3: ...
    @property
    def olx(self) -> I64Vector3: ...
    @property
    def oly(self) -> I64Vector3: ...
    @property
    def olz(self) -> I64Vector3: ...
    @property
    def lxx(self) -> I64Vector3: ...
    @property
    def lxy(self) -> I64Vector3: ...
    @property
    def lxz(self) -> I64Vector3: ...
    @property
    def lxo(self) -> I64Vector3: ...
    @property
    def lxl(self) -> I64Vector3: ...
    @property
    def lyx(self) -> I64Vector3: ...
    @property
    def lyy(self) -> I64Vector3: ...
    @property
    def lyz(self) -> I64Vector3: ...
    @property
    def lyo(self) -> I64Vector3: ...
    @property
    def lyl(self) -> I64Vector3: ...
    @property
    def lzx(self) -> I64Vector3: ...
    @property
    def lzy(self) -> I64Vector3: ...
    @property
    def lzz(self) -> I64Vector3: ...
    @property
    def lzo(self) -> I64Vector3: ...
    @property
    def lzl(self) -> I64Vector3: ...
    @property
    def lox(self) -> I64Vector3: ...
    @property
    def loy(self) -> I64Vector3: ...
    @property
    def loz(self) -> I64Vector3: ...
    @property
    def llx(self) -> I64Vector3: ...
    @property
    def lly(self) -> I64Vector3: ...
    @property
    def llz(self) -> I64Vector3: ...
    @property
    def xxxx(self) -> I64Vector4: ...
    @property
    def xxxy(self) -> I64Vector4: ...
    @property
    def xxxz(self) -> I64Vector4: ...
    @property
    def xxxo(self) -> I64Vector4: ...
    @property
    def xxxl(self) -> I64Vector4: ...
    @property
    def xxyx(self) -> I64Vector4: ...
    @property
    def xxyy(self) -> I64Vector4: ...
    @property
    def xxyz(self) -> I64Vector4: ...
    @property
    def xxyo(self) -> I64Vector4: ...
    @property
    def xxyl(self) -> I64Vector4: ...
    @property
    def xxzx(self) -> I64Vector4: ...
    @property
    def xxzy(self) -> I64Vector4: ...
    @property
    def xxzz(self) -> I64Vector4: ...
    @property
    def xxzo(self) -> I64Vector4: ...
    @property
    def xxzl(self) -> I64Vector4: ...
    @property
    def xxox(self) -> I64Vector4: ...
    @property
    def xxoy(self) -> I64Vector4: ...
    @property
    def xxoz(self) -> I64Vector4: ...
    @property
    def xxoo(self) -> I64Vector4: ...
    @property
    def xxol(self) -> I64Vector4: ...
    @property
    def xxlx(self) -> I64Vector4: ...
    @property
    def xxly(self) -> I64Vector4: ...
    @property
    def xxlz(self) -> I64Vector4: ...
    @property
    def xxlo(self) -> I64Vector4: ...
    @property
    def xxll(self) -> I64Vector4: ...
    @property
    def xyxx(self) -> I64Vector4: ...
    @property
    def xyxy(self) -> I64Vector4: ...
    @property
    def xyxz(self) -> I64Vector4: ...
    @property
    def xyxo(self) -> I64Vector4: ...
    @property
    def xyxl(self) -> I64Vector4: ...
    @property
    def xyyx(self) -> I64Vector4: ...
    @property
    def xyyy(self) -> I64Vector4: ...
    @property
    def xyyz(self) -> I64Vector4: ...
    @property
    def xyyo(self) -> I64Vector4: ...
    @property
    def xyyl(self) -> I64Vector4: ...
    @property
    def xyzx(self) -> I64Vector4: ...
    @property
    def xyzy(self) -> I64Vector4: ...
    @property
    def xyzz(self) -> I64Vector4: ...
    @property
    def xyzo(self) -> I64Vector4: ...
    @property
    def xyzl(self) -> I64Vector4: ...
    @property
    def xyox(self) -> I64Vector4: ...
    @property
    def xyoy(self) -> I64Vector4: ...
    @property
    def xyoz(self) -> I64Vector4: ...
    @property
    def xyoo(self) -> I64Vector4: ...
    @property
    def xyol(self) -> I64Vector4: ...
    @property
    def xylx(self) -> I64Vector4: ...
    @property
    def xyly(self) -> I64Vector4: ...
    @property
    def xylz(self) -> I64Vector4: ...
    @property
    def xylo(self) -> I64Vector4: ...
    @property
    def xyll(self) -> I64Vector4: ...
    @property
    def xzxx(self) -> I64Vector4: ...
    @property
    def xzxy(self) -> I64Vector4: ...
    @property
    def xzxz(self) -> I64Vector4: ...
    @property
    def xzxo(self) -> I64Vector4: ...
    @property
    def xzxl(self) -> I64Vector4: ...
    @property
    def xzyx(self) -> I64Vector4: ...
    @property
    def xzyy(self) -> I64Vector4: ...
    @property
    def xzyz(self) -> I64Vector4: ...
    @property
    def xzyo(self) -> I64Vector4: ...
    @property
    def xzyl(self) -> I64Vector4: ...
    @property
    def xzzx(self) -> I64Vector4: ...
    @property
    def xzzy(self) -> I64Vector4: ...
    @property
    def xzzz(self) -> I64Vector4: ...
    @property
    def xzzo(self) -> I64Vector4: ...
    @property
    def xzzl(self) -> I64Vector4: ...
    @property
    def xzox(self) -> I64Vector4: ...
    @property
    def xzoy(self) -> I64Vector4: ...
    @property
    def xzoz(self) -> I64Vector4: ...
    @property
    def xzoo(self) -> I64Vector4: ...
    @property
    def xzol(self) -> I64Vector4: ...
    @property
    def xzlx(self) -> I64Vector4: ...
    @property
    def xzly(self) -> I64Vector4: ...
    @property
    def xzlz(self) -> I64Vector4: ...
    @property
    def xzlo(self) -> I64Vector4: ...
    @property
    def xzll(self) -> I64Vector4: ...
    @property
    def xoxx(self) -> I64Vector4: ...
    @property
    def xoxy(self) -> I64Vector4: ...
    @property
    def xoxz(self) -> I64Vector4: ...
    @property
    def xoxo(self) -> I64Vector4: ...
    @property
    def xoxl(self) -> I64Vector4: ...
    @property
    def xoyx(self) -> I64Vector4: ...
    @property
    def xoyy(self) -> I64Vector4: ...
    @property
    def xoyz(self) -> I64Vector4: ...
    @property
    def xoyo(self) -> I64Vector4: ...
    @property
    def xoyl(self) -> I64Vector4: ...
    @property
    def xozx(self) -> I64Vector4: ...
    @property
    def xozy(self) -> I64Vector4: ...
    @property
    def xozz(self) -> I64Vector4: ...
    @property
    def xozo(self) -> I64Vector4: ...
    @property
    def xozl(self) -> I64Vector4: ...
    @property
    def xoox(self) -> I64Vector4: ...
    @property
    def xooy(self) -> I64Vector4: ...
    @property
    def xooz(self) -> I64Vector4: ...
    @property
    def xooo(self) -> I64Vector4: ...
    @property
    def xool(self) -> I64Vector4: ...
    @property
    def xolx(self) -> I64Vector4: ...
    @property
    def xoly(self) -> I64Vector4: ...
    @property
    def xolz(self) -> I64Vector4: ...
    @property
    def xolo(self) -> I64Vector4: ...
    @property
    def xoll(self) -> I64Vector4: ...
    @property
    def xlxx(self) -> I64Vector4: ...
    @property
    def xlxy(self) -> I64Vector4: ...
    @property
    def xlxz(self) -> I64Vector4: ...
    @property
    def xlxo(self) -> I64Vector4: ...
    @property
    def xlxl(self) -> I64Vector4: ...
    @property
    def xlyx(self) -> I64Vector4: ...
    @property
    def xlyy(self) -> I64Vector4: ...
    @property
    def xlyz(self) -> I64Vector4: ...
    @property
    def xlyo(self) -> I64Vector4: ...
    @property
    def xlyl(self) -> I64Vector4: ...
    @property
    def xlzx(self) -> I64Vector4: ...
    @property
    def xlzy(self) -> I64Vector4: ...
    @property
    def xlzz(self) -> I64Vector4: ...
    @property
    def xlzo(self) -> I64Vector4: ...
    @property
    def xlzl(self) -> I64Vector4: ...
    @property
    def xlox(self) -> I64Vector4: ...
    @property
    def xloy(self) -> I64Vector4: ...
    @property
    def xloz(self) -> I64Vector4: ...
    @property
    def xloo(self) -> I64Vector4: ...
    @property
    def xlol(self) -> I64Vector4: ...
    @property
    def xllx(self) -> I64Vector4: ...
    @property
    def xlly(self) -> I64Vector4: ...
    @property
    def xllz(self) -> I64Vector4: ...
    @property
    def xllo(self) -> I64Vector4: ...
    @property
    def xlll(self) -> I64Vector4: ...
    @property
    def yxxx(self) -> I64Vector4: ...
    @property
    def yxxy(self) -> I64Vector4: ...
    @property
    def yxxz(self) -> I64Vector4: ...
    @property
    def yxxo(self) -> I64Vector4: ...
    @property
    def yxxl(self) -> I64Vector4: ...
    @property
    def yxyx(self) -> I64Vector4: ...
    @property
    def yxyy(self) -> I64Vector4: ...
    @property
    def yxyz(self) -> I64Vector4: ...
    @property
    def yxyo(self) -> I64Vector4: ...
    @property
    def yxyl(self) -> I64Vector4: ...
    @property
    def yxzx(self) -> I64Vector4: ...
    @property
    def yxzy(self) -> I64Vector4: ...
    @property
    def yxzz(self) -> I64Vector4: ...
    @property
    def yxzo(self) -> I64Vector4: ...
    @property
    def yxzl(self) -> I64Vector4: ...
    @property
    def yxox(self) -> I64Vector4: ...
    @property
    def yxoy(self) -> I64Vector4: ...
    @property
    def yxoz(self) -> I64Vector4: ...
    @property
    def yxoo(self) -> I64Vector4: ...
    @property
    def yxol(self) -> I64Vector4: ...
    @property
    def yxlx(self) -> I64Vector4: ...
    @property
    def yxly(self) -> I64Vector4: ...
    @property
    def yxlz(self) -> I64Vector4: ...
    @property
    def yxlo(self) -> I64Vector4: ...
    @property
    def yxll(self) -> I64Vector4: ...
    @property
    def yyxx(self) -> I64Vector4: ...
    @property
    def yyxy(self) -> I64Vector4: ...
    @property
    def yyxz(self) -> I64Vector4: ...
    @property
    def yyxo(self) -> I64Vector4: ...
    @property
    def yyxl(self) -> I64Vector4: ...
    @property
    def yyyx(self) -> I64Vector4: ...
    @property
    def yyyy(self) -> I64Vector4: ...
    @property
    def yyyz(self) -> I64Vector4: ...
    @property
    def yyyo(self) -> I64Vector4: ...
    @property
    def yyyl(self) -> I64Vector4: ...
    @property
    def yyzx(self) -> I64Vector4: ...
    @property
    def yyzy(self) -> I64Vector4: ...
    @property
    def yyzz(self) -> I64Vector4: ...
    @property
    def yyzo(self) -> I64Vector4: ...
    @property
    def yyzl(self) -> I64Vector4: ...
    @property
    def yyox(self) -> I64Vector4: ...
    @property
    def yyoy(self) -> I64Vector4: ...
    @property
    def yyoz(self) -> I64Vector4: ...
    @property
    def yyoo(self) -> I64Vector4: ...
    @property
    def yyol(self) -> I64Vector4: ...
    @property
    def yylx(self) -> I64Vector4: ...
    @property
    def yyly(self) -> I64Vector4: ...
    @property
    def yylz(self) -> I64Vector4: ...
    @property
    def yylo(self) -> I64Vector4: ...
    @property
    def yyll(self) -> I64Vector4: ...
    @property
    def yzxx(self) -> I64Vector4: ...
    @property
    def yzxy(self) -> I64Vector4: ...
    @property
    def yzxz(self) -> I64Vector4: ...
    @property
    def yzxo(self) -> I64Vector4: ...
    @property
    def yzxl(self) -> I64Vector4: ...
    @property
    def yzyx(self) -> I64Vector4: ...
    @property
    def yzyy(self) -> I64Vector4: ...
    @property
    def yzyz(self) -> I64Vector4: ...
    @property
    def yzyo(self) -> I64Vector4: ...
    @property
    def yzyl(self) -> I64Vector4: ...
    @property
    def yzzx(self) -> I64Vector4: ...
    @property
    def yzzy(self) -> I64Vector4: ...
    @property
    def yzzz(self) -> I64Vector4: ...
    @property
    def yzzo(self) -> I64Vector4: ...
    @property
    def yzzl(self) -> I64Vector4: ...
    @property
    def yzox(self) -> I64Vector4: ...
    @property
    def yzoy(self) -> I64Vector4: ...
    @property
    def yzoz(self) -> I64Vector4: ...
    @property
    def yzoo(self) -> I64Vector4: ...
    @property
    def yzol(self) -> I64Vector4: ...
    @property
    def yzlx(self) -> I64Vector4: ...
    @property
    def yzly(self) -> I64Vector4: ...
    @property
    def yzlz(self) -> I64Vector4: ...
    @property
    def yzlo(self) -> I64Vector4: ...
    @property
    def yzll(self) -> I64Vector4: ...
    @property
    def yoxx(self) -> I64Vector4: ...
    @property
    def yoxy(self) -> I64Vector4: ...
    @property
    def yoxz(self) -> I64Vector4: ...
    @property
    def yoxo(self) -> I64Vector4: ...
    @property
    def yoxl(self) -> I64Vector4: ...
    @property
    def yoyx(self) -> I64Vector4: ...
    @property
    def yoyy(self) -> I64Vector4: ...
    @property
    def yoyz(self) -> I64Vector4: ...
    @property
    def yoyo(self) -> I64Vector4: ...
    @property
    def yoyl(self) -> I64Vector4: ...
    @property
    def yozx(self) -> I64Vector4: ...
    @property
    def yozy(self) -> I64Vector4: ...
    @property
    def yozz(self) -> I64Vector4: ...
    @property
    def yozo(self) -> I64Vector4: ...
    @property
    def yozl(self) -> I64Vector4: ...
    @property
    def yoox(self) -> I64Vector4: ...
    @property
    def yooy(self) -> I64Vector4: ...
    @property
    def yooz(self) -> I64Vector4: ...
    @property
    def yooo(self) -> I64Vector4: ...
    @property
    def yool(self) -> I64Vector4: ...
    @property
    def yolx(self) -> I64Vector4: ...
    @property
    def yoly(self) -> I64Vector4: ...
    @property
    def yolz(self) -> I64Vector4: ...
    @property
    def yolo(self) -> I64Vector4: ...
    @property
    def yoll(self) -> I64Vector4: ...
    @property
    def ylxx(self) -> I64Vector4: ...
    @property
    def ylxy(self) -> I64Vector4: ...
    @property
    def ylxz(self) -> I64Vector4: ...
    @property
    def ylxo(self) -> I64Vector4: ...
    @property
    def ylxl(self) -> I64Vector4: ...
    @property
    def ylyx(self) -> I64Vector4: ...
    @property
    def ylyy(self) -> I64Vector4: ...
    @property
    def ylyz(self) -> I64Vector4: ...
    @property
    def ylyo(self) -> I64Vector4: ...
    @property
    def ylyl(self) -> I64Vector4: ...
    @property
    def ylzx(self) -> I64Vector4: ...
    @property
    def ylzy(self) -> I64Vector4: ...
    @property
    def ylzz(self) -> I64Vector4: ...
    @property
    def ylzo(self) -> I64Vector4: ...
    @property
    def ylzl(self) -> I64Vector4: ...
    @property
    def ylox(self) -> I64Vector4: ...
    @property
    def yloy(self) -> I64Vector4: ...
    @property
    def yloz(self) -> I64Vector4: ...
    @property
    def yloo(self) -> I64Vector4: ...
    @property
    def ylol(self) -> I64Vector4: ...
    @property
    def yllx(self) -> I64Vector4: ...
    @property
    def ylly(self) -> I64Vector4: ...
    @property
    def yllz(self) -> I64Vector4: ...
    @property
    def yllo(self) -> I64Vector4: ...
    @property
    def ylll(self) -> I64Vector4: ...
    @property
    def zxxx(self) -> I64Vector4: ...
    @property
    def zxxy(self) -> I64Vector4: ...
    @property
    def zxxz(self) -> I64Vector4: ...
    @property
    def zxxo(self) -> I64Vector4: ...
    @property
    def zxxl(self) -> I64Vector4: ...
    @property
    def zxyx(self) -> I64Vector4: ...
    @property
    def zxyy(self) -> I64Vector4: ...
    @property
    def zxyz(self) -> I64Vector4: ...
    @property
    def zxyo(self) -> I64Vector4: ...
    @property
    def zxyl(self) -> I64Vector4: ...
    @property
    def zxzx(self) -> I64Vector4: ...
    @property
    def zxzy(self) -> I64Vector4: ...
    @property
    def zxzz(self) -> I64Vector4: ...
    @property
    def zxzo(self) -> I64Vector4: ...
    @property
    def zxzl(self) -> I64Vector4: ...
    @property
    def zxox(self) -> I64Vector4: ...
    @property
    def zxoy(self) -> I64Vector4: ...
    @property
    def zxoz(self) -> I64Vector4: ...
    @property
    def zxoo(self) -> I64Vector4: ...
    @property
    def zxol(self) -> I64Vector4: ...
    @property
    def zxlx(self) -> I64Vector4: ...
    @property
    def zxly(self) -> I64Vector4: ...
    @property
    def zxlz(self) -> I64Vector4: ...
    @property
    def zxlo(self) -> I64Vector4: ...
    @property
    def zxll(self) -> I64Vector4: ...
    @property
    def zyxx(self) -> I64Vector4: ...
    @property
    def zyxy(self) -> I64Vector4: ...
    @property
    def zyxz(self) -> I64Vector4: ...
    @property
    def zyxo(self) -> I64Vector4: ...
    @property
    def zyxl(self) -> I64Vector4: ...
    @property
    def zyyx(self) -> I64Vector4: ...
    @property
    def zyyy(self) -> I64Vector4: ...
    @property
    def zyyz(self) -> I64Vector4: ...
    @property
    def zyyo(self) -> I64Vector4: ...
    @property
    def zyyl(self) -> I64Vector4: ...
    @property
    def zyzx(self) -> I64Vector4: ...
    @property
    def zyzy(self) -> I64Vector4: ...
    @property
    def zyzz(self) -> I64Vector4: ...
    @property
    def zyzo(self) -> I64Vector4: ...
    @property
    def zyzl(self) -> I64Vector4: ...
    @property
    def zyox(self) -> I64Vector4: ...
    @property
    def zyoy(self) -> I64Vector4: ...
    @property
    def zyoz(self) -> I64Vector4: ...
    @property
    def zyoo(self) -> I64Vector4: ...
    @property
    def zyol(self) -> I64Vector4: ...
    @property
    def zylx(self) -> I64Vector4: ...
    @property
    def zyly(self) -> I64Vector4: ...
    @property
    def zylz(self) -> I64Vector4: ...
    @property
    def zylo(self) -> I64Vector4: ...
    @property
    def zyll(self) -> I64Vector4: ...
    @property
    def zzxx(self) -> I64Vector4: ...
    @property
    def zzxy(self) -> I64Vector4: ...
    @property
    def zzxz(self) -> I64Vector4: ...
    @property
    def zzxo(self) -> I64Vector4: ...
    @property
    def zzxl(self) -> I64Vector4: ...
    @property
    def zzyx(self) -> I64Vector4: ...
    @property
    def zzyy(self) -> I64Vector4: ...
    @property
    def zzyz(self) -> I64Vector4: ...
    @property
    def zzyo(self) -> I64Vector4: ...
    @property
    def zzyl(self) -> I64Vector4: ...
    @property
    def zzzx(self) -> I64Vector4: ...
    @property
    def zzzy(self) -> I64Vector4: ...
    @property
    def zzzz(self) -> I64Vector4: ...
    @property
    def zzzo(self) -> I64Vector4: ...
    @property
    def zzzl(self) -> I64Vector4: ...
    @property
    def zzox(self) -> I64Vector4: ...
    @property
    def zzoy(self) -> I64Vector4: ...
    @property
    def zzoz(self) -> I64Vector4: ...
    @property
    def zzoo(self) -> I64Vector4: ...
    @property
    def zzol(self) -> I64Vector4: ...
    @property
    def zzlx(self) -> I64Vector4: ...
    @property
    def zzly(self) -> I64Vector4: ...
    @property
    def zzlz(self) -> I64Vector4: ...
    @property
    def zzlo(self) -> I64Vector4: ...
    @property
    def zzll(self) -> I64Vector4: ...
    @property
    def zoxx(self) -> I64Vector4: ...
    @property
    def zoxy(self) -> I64Vector4: ...
    @property
    def zoxz(self) -> I64Vector4: ...
    @property
    def zoxo(self) -> I64Vector4: ...
    @property
    def zoxl(self) -> I64Vector4: ...
    @property
    def zoyx(self) -> I64Vector4: ...
    @property
    def zoyy(self) -> I64Vector4: ...
    @property
    def zoyz(self) -> I64Vector4: ...
    @property
    def zoyo(self) -> I64Vector4: ...
    @property
    def zoyl(self) -> I64Vector4: ...
    @property
    def zozx(self) -> I64Vector4: ...
    @property
    def zozy(self) -> I64Vector4: ...
    @property
    def zozz(self) -> I64Vector4: ...
    @property
    def zozo(self) -> I64Vector4: ...
    @property
    def zozl(self) -> I64Vector4: ...
    @property
    def zoox(self) -> I64Vector4: ...
    @property
    def zooy(self) -> I64Vector4: ...
    @property
    def zooz(self) -> I64Vector4: ...
    @property
    def zooo(self) -> I64Vector4: ...
    @property
    def zool(self) -> I64Vector4: ...
    @property
    def zolx(self) -> I64Vector4: ...
    @property
    def zoly(self) -> I64Vector4: ...
    @property
    def zolz(self) -> I64Vector4: ...
    @property
    def zolo(self) -> I64Vector4: ...
    @property
    def zoll(self) -> I64Vector4: ...
    @property
    def zlxx(self) -> I64Vector4: ...
    @property
    def zlxy(self) -> I64Vector4: ...
    @property
    def zlxz(self) -> I64Vector4: ...
    @property
    def zlxo(self) -> I64Vector4: ...
    @property
    def zlxl(self) -> I64Vector4: ...
    @property
    def zlyx(self) -> I64Vector4: ...
    @property
    def zlyy(self) -> I64Vector4: ...
    @property
    def zlyz(self) -> I64Vector4: ...
    @property
    def zlyo(self) -> I64Vector4: ...
    @property
    def zlyl(self) -> I64Vector4: ...
    @property
    def zlzx(self) -> I64Vector4: ...
    @property
    def zlzy(self) -> I64Vector4: ...
    @property
    def zlzz(self) -> I64Vector4: ...
    @property
    def zlzo(self) -> I64Vector4: ...
    @property
    def zlzl(self) -> I64Vector4: ...
    @property
    def zlox(self) -> I64Vector4: ...
    @property
    def zloy(self) -> I64Vector4: ...
    @property
    def zloz(self) -> I64Vector4: ...
    @property
    def zloo(self) -> I64Vector4: ...
    @property
    def zlol(self) -> I64Vector4: ...
    @property
    def zllx(self) -> I64Vector4: ...
    @property
    def zlly(self) -> I64Vector4: ...
    @property
    def zllz(self) -> I64Vector4: ...
    @property
    def zllo(self) -> I64Vector4: ...
    @property
    def zlll(self) -> I64Vector4: ...
    @property
    def oxxx(self) -> I64Vector4: ...
    @property
    def oxxy(self) -> I64Vector4: ...
    @property
    def oxxz(self) -> I64Vector4: ...
    @property
    def oxxo(self) -> I64Vector4: ...
    @property
    def oxxl(self) -> I64Vector4: ...
    @property
    def oxyx(self) -> I64Vector4: ...
    @property
    def oxyy(self) -> I64Vector4: ...
    @property
    def oxyz(self) -> I64Vector4: ...
    @property
    def oxyo(self) -> I64Vector4: ...
    @property
    def oxyl(self) -> I64Vector4: ...
    @property
    def oxzx(self) -> I64Vector4: ...
    @property
    def oxzy(self) -> I64Vector4: ...
    @property
    def oxzz(self) -> I64Vector4: ...
    @property
    def oxzo(self) -> I64Vector4: ...
    @property
    def oxzl(self) -> I64Vector4: ...
    @property
    def oxox(self) -> I64Vector4: ...
    @property
    def oxoy(self) -> I64Vector4: ...
    @property
    def oxoz(self) -> I64Vector4: ...
    @property
    def oxoo(self) -> I64Vector4: ...
    @property
    def oxol(self) -> I64Vector4: ...
    @property
    def oxlx(self) -> I64Vector4: ...
    @property
    def oxly(self) -> I64Vector4: ...
    @property
    def oxlz(self) -> I64Vector4: ...
    @property
    def oxlo(self) -> I64Vector4: ...
    @property
    def oxll(self) -> I64Vector4: ...
    @property
    def oyxx(self) -> I64Vector4: ...
    @property
    def oyxy(self) -> I64Vector4: ...
    @property
    def oyxz(self) -> I64Vector4: ...
    @property
    def oyxo(self) -> I64Vector4: ...
    @property
    def oyxl(self) -> I64Vector4: ...
    @property
    def oyyx(self) -> I64Vector4: ...
    @property
    def oyyy(self) -> I64Vector4: ...
    @property
    def oyyz(self) -> I64Vector4: ...
    @property
    def oyyo(self) -> I64Vector4: ...
    @property
    def oyyl(self) -> I64Vector4: ...
    @property
    def oyzx(self) -> I64Vector4: ...
    @property
    def oyzy(self) -> I64Vector4: ...
    @property
    def oyzz(self) -> I64Vector4: ...
    @property
    def oyzo(self) -> I64Vector4: ...
    @property
    def oyzl(self) -> I64Vector4: ...
    @property
    def oyox(self) -> I64Vector4: ...
    @property
    def oyoy(self) -> I64Vector4: ...
    @property
    def oyoz(self) -> I64Vector4: ...
    @property
    def oyoo(self) -> I64Vector4: ...
    @property
    def oyol(self) -> I64Vector4: ...
    @property
    def oylx(self) -> I64Vector4: ...
    @property
    def oyly(self) -> I64Vector4: ...
    @property
    def oylz(self) -> I64Vector4: ...
    @property
    def oylo(self) -> I64Vector4: ...
    @property
    def oyll(self) -> I64Vector4: ...
    @property
    def ozxx(self) -> I64Vector4: ...
    @property
    def ozxy(self) -> I64Vector4: ...
    @property
    def ozxz(self) -> I64Vector4: ...
    @property
    def ozxo(self) -> I64Vector4: ...
    @property
    def ozxl(self) -> I64Vector4: ...
    @property
    def ozyx(self) -> I64Vector4: ...
    @property
    def ozyy(self) -> I64Vector4: ...
    @property
    def ozyz(self) -> I64Vector4: ...
    @property
    def ozyo(self) -> I64Vector4: ...
    @property
    def ozyl(self) -> I64Vector4: ...
    @property
    def ozzx(self) -> I64Vector4: ...
    @property
    def ozzy(self) -> I64Vector4: ...
    @property
    def ozzz(self) -> I64Vector4: ...
    @property
    def ozzo(self) -> I64Vector4: ...
    @property
    def ozzl(self) -> I64Vector4: ...
    @property
    def ozox(self) -> I64Vector4: ...
    @property
    def ozoy(self) -> I64Vector4: ...
    @property
    def ozoz(self) -> I64Vector4: ...
    @property
    def ozoo(self) -> I64Vector4: ...
    @property
    def ozol(self) -> I64Vector4: ...
    @property
    def ozlx(self) -> I64Vector4: ...
    @property
    def ozly(self) -> I64Vector4: ...
    @property
    def ozlz(self) -> I64Vector4: ...
    @property
    def ozlo(self) -> I64Vector4: ...
    @property
    def ozll(self) -> I64Vector4: ...
    @property
    def ooxx(self) -> I64Vector4: ...
    @property
    def ooxy(self) -> I64Vector4: ...
    @property
    def ooxz(self) -> I64Vector4: ...
    @property
    def ooxo(self) -> I64Vector4: ...
    @property
    def ooxl(self) -> I64Vector4: ...
    @property
    def ooyx(self) -> I64Vector4: ...
    @property
    def ooyy(self) -> I64Vector4: ...
    @property
    def ooyz(self) -> I64Vector4: ...
    @property
    def ooyo(self) -> I64Vector4: ...
    @property
    def ooyl(self) -> I64Vector4: ...
    @property
    def oozx(self) -> I64Vector4: ...
    @property
    def oozy(self) -> I64Vector4: ...
    @property
    def oozz(self) -> I64Vector4: ...
    @property
    def oozo(self) -> I64Vector4: ...
    @property
    def oozl(self) -> I64Vector4: ...
    @property
    def ooox(self) -> I64Vector4: ...
    @property
    def oooy(self) -> I64Vector4: ...
    @property
    def oooz(self) -> I64Vector4: ...
    @property
    def oolx(self) -> I64Vector4: ...
    @property
    def ooly(self) -> I64Vector4: ...
    @property
    def oolz(self) -> I64Vector4: ...
    @property
    def olxx(self) -> I64Vector4: ...
    @property
    def olxy(self) -> I64Vector4: ...
    @property
    def olxz(self) -> I64Vector4: ...
    @property
    def olxo(self) -> I64Vector4: ...
    @property
    def olxl(self) -> I64Vector4: ...
    @property
    def olyx(self) -> I64Vector4: ...
    @property
    def olyy(self) -> I64Vector4: ...
    @property
    def olyz(self) -> I64Vector4: ...
    @property
    def olyo(self) -> I64Vector4: ...
    @property
    def olyl(self) -> I64Vector4: ...
    @property
    def olzx(self) -> I64Vector4: ...
    @property
    def olzy(self) -> I64Vector4: ...
    @property
    def olzz(self) -> I64Vector4: ...
    @property
    def olzo(self) -> I64Vector4: ...
    @property
    def olzl(self) -> I64Vector4: ...
    @property
    def olox(self) -> I64Vector4: ...
    @property
    def oloy(self) -> I64Vector4: ...
    @property
    def oloz(self) -> I64Vector4: ...
    @property
    def ollx(self) -> I64Vector4: ...
    @property
    def olly(self) -> I64Vector4: ...
    @property
    def ollz(self) -> I64Vector4: ...
    @property
    def lxxx(self) -> I64Vector4: ...
    @property
    def lxxy(self) -> I64Vector4: ...
    @property
    def lxxz(self) -> I64Vector4: ...
    @property
    def lxxo(self) -> I64Vector4: ...
    @property
    def lxxl(self) -> I64Vector4: ...
    @property
    def lxyx(self) -> I64Vector4: ...
    @property
    def lxyy(self) -> I64Vector4: ...
    @property
    def lxyz(self) -> I64Vector4: ...
    @property
    def lxyo(self) -> I64Vector4: ...
    @property
    def lxyl(self) -> I64Vector4: ...
    @property
    def lxzx(self) -> I64Vector4: ...
    @property
    def lxzy(self) -> I64Vector4: ...
    @property
    def lxzz(self) -> I64Vector4: ...
    @property
    def lxzo(self) -> I64Vector4: ...
    @property
    def lxzl(self) -> I64Vector4: ...
    @property
    def lxox(self) -> I64Vector4: ...
    @property
    def lxoy(self) -> I64Vector4: ...
    @property
    def lxoz(self) -> I64Vector4: ...
    @property
    def lxoo(self) -> I64Vector4: ...
    @property
    def lxol(self) -> I64Vector4: ...
    @property
    def lxlx(self) -> I64Vector4: ...
    @property
    def lxly(self) -> I64Vector4: ...
    @property
    def lxlz(self) -> I64Vector4: ...
    @property
    def lxlo(self) -> I64Vector4: ...
    @property
    def lxll(self) -> I64Vector4: ...
    @property
    def lyxx(self) -> I64Vector4: ...
    @property
    def lyxy(self) -> I64Vector4: ...
    @property
    def lyxz(self) -> I64Vector4: ...
    @property
    def lyxo(self) -> I64Vector4: ...
    @property
    def lyxl(self) -> I64Vector4: ...
    @property
    def lyyx(self) -> I64Vector4: ...
    @property
    def lyyy(self) -> I64Vector4: ...
    @property
    def lyyz(self) -> I64Vector4: ...
    @property
    def lyyo(self) -> I64Vector4: ...
    @property
    def lyyl(self) -> I64Vector4: ...
    @property
    def lyzx(self) -> I64Vector4: ...
    @property
    def lyzy(self) -> I64Vector4: ...
    @property
    def lyzz(self) -> I64Vector4: ...
    @property
    def lyzo(self) -> I64Vector4: ...
    @property
    def lyzl(self) -> I64Vector4: ...
    @property
    def lyox(self) -> I64Vector4: ...
    @property
    def lyoy(self) -> I64Vector4: ...
    @property
    def lyoz(self) -> I64Vector4: ...
    @property
    def lyoo(self) -> I64Vector4: ...
    @property
    def lyol(self) -> I64Vector4: ...
    @property
    def lylx(self) -> I64Vector4: ...
    @property
    def lyly(self) -> I64Vector4: ...
    @property
    def lylz(self) -> I64Vector4: ...
    @property
    def lylo(self) -> I64Vector4: ...
    @property
    def lyll(self) -> I64Vector4: ...
    @property
    def lzxx(self) -> I64Vector4: ...
    @property
    def lzxy(self) -> I64Vector4: ...
    @property
    def lzxz(self) -> I64Vector4: ...
    @property
    def lzxo(self) -> I64Vector4: ...
    @property
    def lzxl(self) -> I64Vector4: ...
    @property
    def lzyx(self) -> I64Vector4: ...
    @property
    def lzyy(self) -> I64Vector4: ...
    @property
    def lzyz(self) -> I64Vector4: ...
    @property
    def lzyo(self) -> I64Vector4: ...
    @property
    def lzyl(self) -> I64Vector4: ...
    @property
    def lzzx(self) -> I64Vector4: ...
    @property
    def lzzy(self) -> I64Vector4: ...
    @property
    def lzzz(self) -> I64Vector4: ...
    @property
    def lzzo(self) -> I64Vector4: ...
    @property
    def lzzl(self) -> I64Vector4: ...
    @property
    def lzox(self) -> I64Vector4: ...
    @property
    def lzoy(self) -> I64Vector4: ...
    @property
    def lzoz(self) -> I64Vector4: ...
    @property
    def lzoo(self) -> I64Vector4: ...
    @property
    def lzol(self) -> I64Vector4: ...
    @property
    def lzlx(self) -> I64Vector4: ...
    @property
    def lzly(self) -> I64Vector4: ...
    @property
    def lzlz(self) -> I64Vector4: ...
    @property
    def lzlo(self) -> I64Vector4: ...
    @property
    def lzll(self) -> I64Vector4: ...
    @property
    def loxx(self) -> I64Vector4: ...
    @property
    def loxy(self) -> I64Vector4: ...
    @property
    def loxz(self) -> I64Vector4: ...
    @property
    def loxo(self) -> I64Vector4: ...
    @property
    def loxl(self) -> I64Vector4: ...
    @property
    def loyx(self) -> I64Vector4: ...
    @property
    def loyy(self) -> I64Vector4: ...
    @property
    def loyz(self) -> I64Vector4: ...
    @property
    def loyo(self) -> I64Vector4: ...
    @property
    def loyl(self) -> I64Vector4: ...
    @property
    def lozx(self) -> I64Vector4: ...
    @property
    def lozy(self) -> I64Vector4: ...
    @property
    def lozz(self) -> I64Vector4: ...
    @property
    def lozo(self) -> I64Vector4: ...
    @property
    def lozl(self) -> I64Vector4: ...
    @property
    def loox(self) -> I64Vector4: ...
    @property
    def looy(self) -> I64Vector4: ...
    @property
    def looz(self) -> I64Vector4: ...
    @property
    def lolx(self) -> I64Vector4: ...
    @property
    def loly(self) -> I64Vector4: ...
    @property
    def lolz(self) -> I64Vector4: ...
    @property
    def llxx(self) -> I64Vector4: ...
    @property
    def llxy(self) -> I64Vector4: ...
    @property
    def llxz(self) -> I64Vector4: ...
    @property
    def llxo(self) -> I64Vector4: ...
    @property
    def llxl(self) -> I64Vector4: ...
    @property
    def llyx(self) -> I64Vector4: ...
    @property
    def llyy(self) -> I64Vector4: ...
    @property
    def llyz(self) -> I64Vector4: ...
    @property
    def llyo(self) -> I64Vector4: ...
    @property
    def llyl(self) -> I64Vector4: ...
    @property
    def llzx(self) -> I64Vector4: ...
    @property
    def llzy(self) -> I64Vector4: ...
    @property
    def llzz(self) -> I64Vector4: ...
    @property
    def llzo(self) -> I64Vector4: ...
    @property
    def llzl(self) -> I64Vector4: ...
    @property
    def llox(self) -> I64Vector4: ...
    @property
    def lloy(self) -> I64Vector4: ...
    @property
    def lloz(self) -> I64Vector4: ...
    @property
    def lllx(self) -> I64Vector4: ...
    @property
    def llly(self) -> I64Vector4: ...
    @property
    def lllz(self) -> I64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> I64Vector2: ...
    @property
    def rg(self) -> I64Vector2: ...
    @property
    def rb(self) -> I64Vector2: ...
    @property
    def ro(self) -> I64Vector2: ...
    @property
    def rl(self) -> I64Vector2: ...
    @property
    def gr(self) -> I64Vector2: ...
    @property
    def gg(self) -> I64Vector2: ...
    @property
    def gb(self) -> I64Vector2: ...
    @property
    def go(self) -> I64Vector2: ...
    @property
    def gl(self) -> I64Vector2: ...
    @property
    def br(self) -> I64Vector2: ...
    @property
    def bg(self) -> I64Vector2: ...
    @property
    def bb(self) -> I64Vector2: ...
    @property
    def bo(self) -> I64Vector2: ...
    @property
    def bl(self) -> I64Vector2: ...
    @property
    def og(self) -> I64Vector2: ...
    @property
    def ob(self) -> I64Vector2: ...
    @property
    def lr(self) -> I64Vector2: ...
    @property
    def lg(self) -> I64Vector2: ...
    @property
    def lb(self) -> I64Vector2: ...
    @property
    def rrr(self) -> I64Vector3: ...
    @property
    def rrg(self) -> I64Vector3: ...
    @property
    def rrb(self) -> I64Vector3: ...
    @property
    def rro(self) -> I64Vector3: ...
    @property
    def rrl(self) -> I64Vector3: ...
    @property
    def rgr(self) -> I64Vector3: ...
    @property
    def rgg(self) -> I64Vector3: ...
    @property
    def rgb(self) -> I64Vector3: ...
    @property
    def rgo(self) -> I64Vector3: ...
    @property
    def rgl(self) -> I64Vector3: ...
    @property
    def rbr(self) -> I64Vector3: ...
    @property
    def rbg(self) -> I64Vector3: ...
    @property
    def rbb(self) -> I64Vector3: ...
    @property
    def rbo(self) -> I64Vector3: ...
    @property
    def rbl(self) -> I64Vector3: ...
    @property
    def ror(self) -> I64Vector3: ...
    @property
    def rog(self) -> I64Vector3: ...
    @property
    def rob(self) -> I64Vector3: ...
    @property
    def roo(self) -> I64Vector3: ...
    @property
    def rol(self) -> I64Vector3: ...
    @property
    def rlr(self) -> I64Vector3: ...
    @property
    def rlg(self) -> I64Vector3: ...
    @property
    def rlb(self) -> I64Vector3: ...
    @property
    def rlo(self) -> I64Vector3: ...
    @property
    def rll(self) -> I64Vector3: ...
    @property
    def grr(self) -> I64Vector3: ...
    @property
    def grg(self) -> I64Vector3: ...
    @property
    def grb(self) -> I64Vector3: ...
    @property
    def gro(self) -> I64Vector3: ...
    @property
    def grl(self) -> I64Vector3: ...
    @property
    def ggr(self) -> I64Vector3: ...
    @property
    def ggg(self) -> I64Vector3: ...
    @property
    def ggb(self) -> I64Vector3: ...
    @property
    def ggo(self) -> I64Vector3: ...
    @property
    def ggl(self) -> I64Vector3: ...
    @property
    def gbr(self) -> I64Vector3: ...
    @property
    def gbg(self) -> I64Vector3: ...
    @property
    def gbb(self) -> I64Vector3: ...
    @property
    def gbo(self) -> I64Vector3: ...
    @property
    def gbl(self) -> I64Vector3: ...
    @property
    def gor(self) -> I64Vector3: ...
    @property
    def gog(self) -> I64Vector3: ...
    @property
    def gob(self) -> I64Vector3: ...
    @property
    def goo(self) -> I64Vector3: ...
    @property
    def gol(self) -> I64Vector3: ...
    @property
    def glr(self) -> I64Vector3: ...
    @property
    def glg(self) -> I64Vector3: ...
    @property
    def glb(self) -> I64Vector3: ...
    @property
    def glo(self) -> I64Vector3: ...
    @property
    def gll(self) -> I64Vector3: ...
    @property
    def brr(self) -> I64Vector3: ...
    @property
    def brg(self) -> I64Vector3: ...
    @property
    def brb(self) -> I64Vector3: ...
    @property
    def bro(self) -> I64Vector3: ...
    @property
    def brl(self) -> I64Vector3: ...
    @property
    def bgr(self) -> I64Vector3: ...
    @property
    def bgg(self) -> I64Vector3: ...
    @property
    def bgb(self) -> I64Vector3: ...
    @property
    def bgo(self) -> I64Vector3: ...
    @property
    def bgl(self) -> I64Vector3: ...
    @property
    def bbr(self) -> I64Vector3: ...
    @property
    def bbg(self) -> I64Vector3: ...
    @property
    def bbb(self) -> I64Vector3: ...
    @property
    def bbo(self) -> I64Vector3: ...
    @property
    def bbl(self) -> I64Vector3: ...
    @property
    def bor(self) -> I64Vector3: ...
    @property
    def bog(self) -> I64Vector3: ...
    @property
    def bob(self) -> I64Vector3: ...
    @property
    def boo(self) -> I64Vector3: ...
    @property
    def bol(self) -> I64Vector3: ...
    @property
    def blr(self) -> I64Vector3: ...
    @property
    def blg(self) -> I64Vector3: ...
    @property
    def blb(self) -> I64Vector3: ...
    @property
    def blo(self) -> I64Vector3: ...
    @property
    def bll(self) -> I64Vector3: ...
    @property
    def orr(self) -> I64Vector3: ...
    @property
    def org(self) -> I64Vector3: ...
    @property
    def orb(self) -> I64Vector3: ...
    @property
    def oro(self) -> I64Vector3: ...
    @property
    def orl(self) -> I64Vector3: ...
    @property
    def ogr(self) -> I64Vector3: ...
    @property
    def ogg(self) -> I64Vector3: ...
    @property
    def ogb(self) -> I64Vector3: ...
    @property
    def ogo(self) -> I64Vector3: ...
    @property
    def ogl(self) -> I64Vector3: ...
    @property
    def obr(self) -> I64Vector3: ...
    @property
    def obg(self) -> I64Vector3: ...
    @property
    def obb(self) -> I64Vector3: ...
    @property
    def obo(self) -> I64Vector3: ...
    @property
    def obl(self) -> I64Vector3: ...
    @property
    def oor(self) -> I64Vector3: ...
    @property
    def oog(self) -> I64Vector3: ...
    @property
    def oob(self) -> I64Vector3: ...
    @property
    def olr(self) -> I64Vector3: ...
    @property
    def olg(self) -> I64Vector3: ...
    @property
    def olb(self) -> I64Vector3: ...
    @property
    def lrr(self) -> I64Vector3: ...
    @property
    def lrg(self) -> I64Vector3: ...
    @property
    def lrb(self) -> I64Vector3: ...
    @property
    def lro(self) -> I64Vector3: ...
    @property
    def lrl(self) -> I64Vector3: ...
    @property
    def lgr(self) -> I64Vector3: ...
    @property
    def lgg(self) -> I64Vector3: ...
    @property
    def lgb(self) -> I64Vector3: ...
    @property
    def lgo(self) -> I64Vector3: ...
    @property
    def lgl(self) -> I64Vector3: ...
    @property
    def lbr(self) -> I64Vector3: ...
    @property
    def lbg(self) -> I64Vector3: ...
    @property
    def lbb(self) -> I64Vector3: ...
    @property
    def lbo(self) -> I64Vector3: ...
    @property
    def lbl(self) -> I64Vector3: ...
    @property
    def lor(self) -> I64Vector3: ...
    @property
    def log(self) -> I64Vector3: ...
    @property
    def lob(self) -> I64Vector3: ...
    @property
    def llr(self) -> I64Vector3: ...
    @property
    def llg(self) -> I64Vector3: ...
    @property
    def llb(self) -> I64Vector3: ...
    @property
    def rrrr(self) -> I64Vector4: ...
    @property
    def rrrg(self) -> I64Vector4: ...
    @property
    def rrrb(self) -> I64Vector4: ...
    @property
    def rrro(self) -> I64Vector4: ...
    @property
    def rrrl(self) -> I64Vector4: ...
    @property
    def rrgr(self) -> I64Vector4: ...
    @property
    def rrgg(self) -> I64Vector4: ...
    @property
    def rrgb(self) -> I64Vector4: ...
    @property
    def rrgo(self) -> I64Vector4: ...
    @property
    def rrgl(self) -> I64Vector4: ...
    @property
    def rrbr(self) -> I64Vector4: ...
    @property
    def rrbg(self) -> I64Vector4: ...
    @property
    def rrbb(self) -> I64Vector4: ...
    @property
    def rrbo(self) -> I64Vector4: ...
    @property
    def rrbl(self) -> I64Vector4: ...
    @property
    def rror(self) -> I64Vector4: ...
    @property
    def rrog(self) -> I64Vector4: ...
    @property
    def rrob(self) -> I64Vector4: ...
    @property
    def rroo(self) -> I64Vector4: ...
    @property
    def rrol(self) -> I64Vector4: ...
    @property
    def rrlr(self) -> I64Vector4: ...
    @property
    def rrlg(self) -> I64Vector4: ...
    @property
    def rrlb(self) -> I64Vector4: ...
    @property
    def rrlo(self) -> I64Vector4: ...
    @property
    def rrll(self) -> I64Vector4: ...
    @property
    def rgrr(self) -> I64Vector4: ...
    @property
    def rgrg(self) -> I64Vector4: ...
    @property
    def rgrb(self) -> I64Vector4: ...
    @property
    def rgro(self) -> I64Vector4: ...
    @property
    def rgrl(self) -> I64Vector4: ...
    @property
    def rggr(self) -> I64Vector4: ...
    @property
    def rggg(self) -> I64Vector4: ...
    @property
    def rggb(self) -> I64Vector4: ...
    @property
    def rggo(self) -> I64Vector4: ...
    @property
    def rggl(self) -> I64Vector4: ...
    @property
    def rgbr(self) -> I64Vector4: ...
    @property
    def rgbg(self) -> I64Vector4: ...
    @property
    def rgbb(self) -> I64Vector4: ...
    @property
    def rgbo(self) -> I64Vector4: ...
    @property
    def rgbl(self) -> I64Vector4: ...
    @property
    def rgor(self) -> I64Vector4: ...
    @property
    def rgog(self) -> I64Vector4: ...
    @property
    def rgob(self) -> I64Vector4: ...
    @property
    def rgoo(self) -> I64Vector4: ...
    @property
    def rgol(self) -> I64Vector4: ...
    @property
    def rglr(self) -> I64Vector4: ...
    @property
    def rglg(self) -> I64Vector4: ...
    @property
    def rglb(self) -> I64Vector4: ...
    @property
    def rglo(self) -> I64Vector4: ...
    @property
    def rgll(self) -> I64Vector4: ...
    @property
    def rbrr(self) -> I64Vector4: ...
    @property
    def rbrg(self) -> I64Vector4: ...
    @property
    def rbrb(self) -> I64Vector4: ...
    @property
    def rbro(self) -> I64Vector4: ...
    @property
    def rbrl(self) -> I64Vector4: ...
    @property
    def rbgr(self) -> I64Vector4: ...
    @property
    def rbgg(self) -> I64Vector4: ...
    @property
    def rbgb(self) -> I64Vector4: ...
    @property
    def rbgo(self) -> I64Vector4: ...
    @property
    def rbgl(self) -> I64Vector4: ...
    @property
    def rbbr(self) -> I64Vector4: ...
    @property
    def rbbg(self) -> I64Vector4: ...
    @property
    def rbbb(self) -> I64Vector4: ...
    @property
    def rbbo(self) -> I64Vector4: ...
    @property
    def rbbl(self) -> I64Vector4: ...
    @property
    def rbor(self) -> I64Vector4: ...
    @property
    def rbog(self) -> I64Vector4: ...
    @property
    def rbob(self) -> I64Vector4: ...
    @property
    def rboo(self) -> I64Vector4: ...
    @property
    def rbol(self) -> I64Vector4: ...
    @property
    def rblr(self) -> I64Vector4: ...
    @property
    def rblg(self) -> I64Vector4: ...
    @property
    def rblb(self) -> I64Vector4: ...
    @property
    def rblo(self) -> I64Vector4: ...
    @property
    def rbll(self) -> I64Vector4: ...
    @property
    def rorr(self) -> I64Vector4: ...
    @property
    def rorg(self) -> I64Vector4: ...
    @property
    def rorb(self) -> I64Vector4: ...
    @property
    def roro(self) -> I64Vector4: ...
    @property
    def rorl(self) -> I64Vector4: ...
    @property
    def rogr(self) -> I64Vector4: ...
    @property
    def rogg(self) -> I64Vector4: ...
    @property
    def rogb(self) -> I64Vector4: ...
    @property
    def rogo(self) -> I64Vector4: ...
    @property
    def rogl(self) -> I64Vector4: ...
    @property
    def robr(self) -> I64Vector4: ...
    @property
    def robg(self) -> I64Vector4: ...
    @property
    def robb(self) -> I64Vector4: ...
    @property
    def robo(self) -> I64Vector4: ...
    @property
    def robl(self) -> I64Vector4: ...
    @property
    def roor(self) -> I64Vector4: ...
    @property
    def roog(self) -> I64Vector4: ...
    @property
    def roob(self) -> I64Vector4: ...
    @property
    def rooo(self) -> I64Vector4: ...
    @property
    def rool(self) -> I64Vector4: ...
    @property
    def rolr(self) -> I64Vector4: ...
    @property
    def rolg(self) -> I64Vector4: ...
    @property
    def rolb(self) -> I64Vector4: ...
    @property
    def rolo(self) -> I64Vector4: ...
    @property
    def roll(self) -> I64Vector4: ...
    @property
    def rlrr(self) -> I64Vector4: ...
    @property
    def rlrg(self) -> I64Vector4: ...
    @property
    def rlrb(self) -> I64Vector4: ...
    @property
    def rlro(self) -> I64Vector4: ...
    @property
    def rlrl(self) -> I64Vector4: ...
    @property
    def rlgr(self) -> I64Vector4: ...
    @property
    def rlgg(self) -> I64Vector4: ...
    @property
    def rlgb(self) -> I64Vector4: ...
    @property
    def rlgo(self) -> I64Vector4: ...
    @property
    def rlgl(self) -> I64Vector4: ...
    @property
    def rlbr(self) -> I64Vector4: ...
    @property
    def rlbg(self) -> I64Vector4: ...
    @property
    def rlbb(self) -> I64Vector4: ...
    @property
    def rlbo(self) -> I64Vector4: ...
    @property
    def rlbl(self) -> I64Vector4: ...
    @property
    def rlor(self) -> I64Vector4: ...
    @property
    def rlog(self) -> I64Vector4: ...
    @property
    def rlob(self) -> I64Vector4: ...
    @property
    def rloo(self) -> I64Vector4: ...
    @property
    def rlol(self) -> I64Vector4: ...
    @property
    def rllr(self) -> I64Vector4: ...
    @property
    def rllg(self) -> I64Vector4: ...
    @property
    def rllb(self) -> I64Vector4: ...
    @property
    def rllo(self) -> I64Vector4: ...
    @property
    def rlll(self) -> I64Vector4: ...
    @property
    def grrr(self) -> I64Vector4: ...
    @property
    def grrg(self) -> I64Vector4: ...
    @property
    def grrb(self) -> I64Vector4: ...
    @property
    def grro(self) -> I64Vector4: ...
    @property
    def grrl(self) -> I64Vector4: ...
    @property
    def grgr(self) -> I64Vector4: ...
    @property
    def grgg(self) -> I64Vector4: ...
    @property
    def grgb(self) -> I64Vector4: ...
    @property
    def grgo(self) -> I64Vector4: ...
    @property
    def grgl(self) -> I64Vector4: ...
    @property
    def grbr(self) -> I64Vector4: ...
    @property
    def grbg(self) -> I64Vector4: ...
    @property
    def grbb(self) -> I64Vector4: ...
    @property
    def grbo(self) -> I64Vector4: ...
    @property
    def grbl(self) -> I64Vector4: ...
    @property
    def gror(self) -> I64Vector4: ...
    @property
    def grog(self) -> I64Vector4: ...
    @property
    def grob(self) -> I64Vector4: ...
    @property
    def groo(self) -> I64Vector4: ...
    @property
    def grol(self) -> I64Vector4: ...
    @property
    def grlr(self) -> I64Vector4: ...
    @property
    def grlg(self) -> I64Vector4: ...
    @property
    def grlb(self) -> I64Vector4: ...
    @property
    def grlo(self) -> I64Vector4: ...
    @property
    def grll(self) -> I64Vector4: ...
    @property
    def ggrr(self) -> I64Vector4: ...
    @property
    def ggrg(self) -> I64Vector4: ...
    @property
    def ggrb(self) -> I64Vector4: ...
    @property
    def ggro(self) -> I64Vector4: ...
    @property
    def ggrl(self) -> I64Vector4: ...
    @property
    def gggr(self) -> I64Vector4: ...
    @property
    def gggg(self) -> I64Vector4: ...
    @property
    def gggb(self) -> I64Vector4: ...
    @property
    def gggo(self) -> I64Vector4: ...
    @property
    def gggl(self) -> I64Vector4: ...
    @property
    def ggbr(self) -> I64Vector4: ...
    @property
    def ggbg(self) -> I64Vector4: ...
    @property
    def ggbb(self) -> I64Vector4: ...
    @property
    def ggbo(self) -> I64Vector4: ...
    @property
    def ggbl(self) -> I64Vector4: ...
    @property
    def ggor(self) -> I64Vector4: ...
    @property
    def ggog(self) -> I64Vector4: ...
    @property
    def ggob(self) -> I64Vector4: ...
    @property
    def ggoo(self) -> I64Vector4: ...
    @property
    def ggol(self) -> I64Vector4: ...
    @property
    def gglr(self) -> I64Vector4: ...
    @property
    def gglg(self) -> I64Vector4: ...
    @property
    def gglb(self) -> I64Vector4: ...
    @property
    def gglo(self) -> I64Vector4: ...
    @property
    def ggll(self) -> I64Vector4: ...
    @property
    def gbrr(self) -> I64Vector4: ...
    @property
    def gbrg(self) -> I64Vector4: ...
    @property
    def gbrb(self) -> I64Vector4: ...
    @property
    def gbro(self) -> I64Vector4: ...
    @property
    def gbrl(self) -> I64Vector4: ...
    @property
    def gbgr(self) -> I64Vector4: ...
    @property
    def gbgg(self) -> I64Vector4: ...
    @property
    def gbgb(self) -> I64Vector4: ...
    @property
    def gbgo(self) -> I64Vector4: ...
    @property
    def gbgl(self) -> I64Vector4: ...
    @property
    def gbbr(self) -> I64Vector4: ...
    @property
    def gbbg(self) -> I64Vector4: ...
    @property
    def gbbb(self) -> I64Vector4: ...
    @property
    def gbbo(self) -> I64Vector4: ...
    @property
    def gbbl(self) -> I64Vector4: ...
    @property
    def gbor(self) -> I64Vector4: ...
    @property
    def gbog(self) -> I64Vector4: ...
    @property
    def gbob(self) -> I64Vector4: ...
    @property
    def gboo(self) -> I64Vector4: ...
    @property
    def gbol(self) -> I64Vector4: ...
    @property
    def gblr(self) -> I64Vector4: ...
    @property
    def gblg(self) -> I64Vector4: ...
    @property
    def gblb(self) -> I64Vector4: ...
    @property
    def gblo(self) -> I64Vector4: ...
    @property
    def gbll(self) -> I64Vector4: ...
    @property
    def gorr(self) -> I64Vector4: ...
    @property
    def gorg(self) -> I64Vector4: ...
    @property
    def gorb(self) -> I64Vector4: ...
    @property
    def goro(self) -> I64Vector4: ...
    @property
    def gorl(self) -> I64Vector4: ...
    @property
    def gogr(self) -> I64Vector4: ...
    @property
    def gogg(self) -> I64Vector4: ...
    @property
    def gogb(self) -> I64Vector4: ...
    @property
    def gogo(self) -> I64Vector4: ...
    @property
    def gogl(self) -> I64Vector4: ...
    @property
    def gobr(self) -> I64Vector4: ...
    @property
    def gobg(self) -> I64Vector4: ...
    @property
    def gobb(self) -> I64Vector4: ...
    @property
    def gobo(self) -> I64Vector4: ...
    @property
    def gobl(self) -> I64Vector4: ...
    @property
    def goor(self) -> I64Vector4: ...
    @property
    def goog(self) -> I64Vector4: ...
    @property
    def goob(self) -> I64Vector4: ...
    @property
    def gooo(self) -> I64Vector4: ...
    @property
    def gool(self) -> I64Vector4: ...
    @property
    def golr(self) -> I64Vector4: ...
    @property
    def golg(self) -> I64Vector4: ...
    @property
    def golb(self) -> I64Vector4: ...
    @property
    def golo(self) -> I64Vector4: ...
    @property
    def goll(self) -> I64Vector4: ...
    @property
    def glrr(self) -> I64Vector4: ...
    @property
    def glrg(self) -> I64Vector4: ...
    @property
    def glrb(self) -> I64Vector4: ...
    @property
    def glro(self) -> I64Vector4: ...
    @property
    def glrl(self) -> I64Vector4: ...
    @property
    def glgr(self) -> I64Vector4: ...
    @property
    def glgg(self) -> I64Vector4: ...
    @property
    def glgb(self) -> I64Vector4: ...
    @property
    def glgo(self) -> I64Vector4: ...
    @property
    def glgl(self) -> I64Vector4: ...
    @property
    def glbr(self) -> I64Vector4: ...
    @property
    def glbg(self) -> I64Vector4: ...
    @property
    def glbb(self) -> I64Vector4: ...
    @property
    def glbo(self) -> I64Vector4: ...
    @property
    def glbl(self) -> I64Vector4: ...
    @property
    def glor(self) -> I64Vector4: ...
    @property
    def glog(self) -> I64Vector4: ...
    @property
    def glob(self) -> I64Vector4: ...
    @property
    def gloo(self) -> I64Vector4: ...
    @property
    def glol(self) -> I64Vector4: ...
    @property
    def gllr(self) -> I64Vector4: ...
    @property
    def gllg(self) -> I64Vector4: ...
    @property
    def gllb(self) -> I64Vector4: ...
    @property
    def gllo(self) -> I64Vector4: ...
    @property
    def glll(self) -> I64Vector4: ...
    @property
    def brrr(self) -> I64Vector4: ...
    @property
    def brrg(self) -> I64Vector4: ...
    @property
    def brrb(self) -> I64Vector4: ...
    @property
    def brro(self) -> I64Vector4: ...
    @property
    def brrl(self) -> I64Vector4: ...
    @property
    def brgr(self) -> I64Vector4: ...
    @property
    def brgg(self) -> I64Vector4: ...
    @property
    def brgb(self) -> I64Vector4: ...
    @property
    def brgo(self) -> I64Vector4: ...
    @property
    def brgl(self) -> I64Vector4: ...
    @property
    def brbr(self) -> I64Vector4: ...
    @property
    def brbg(self) -> I64Vector4: ...
    @property
    def brbb(self) -> I64Vector4: ...
    @property
    def brbo(self) -> I64Vector4: ...
    @property
    def brbl(self) -> I64Vector4: ...
    @property
    def bror(self) -> I64Vector4: ...
    @property
    def brog(self) -> I64Vector4: ...
    @property
    def brob(self) -> I64Vector4: ...
    @property
    def broo(self) -> I64Vector4: ...
    @property
    def brol(self) -> I64Vector4: ...
    @property
    def brlr(self) -> I64Vector4: ...
    @property
    def brlg(self) -> I64Vector4: ...
    @property
    def brlb(self) -> I64Vector4: ...
    @property
    def brlo(self) -> I64Vector4: ...
    @property
    def brll(self) -> I64Vector4: ...
    @property
    def bgrr(self) -> I64Vector4: ...
    @property
    def bgrg(self) -> I64Vector4: ...
    @property
    def bgrb(self) -> I64Vector4: ...
    @property
    def bgro(self) -> I64Vector4: ...
    @property
    def bgrl(self) -> I64Vector4: ...
    @property
    def bggr(self) -> I64Vector4: ...
    @property
    def bggg(self) -> I64Vector4: ...
    @property
    def bggb(self) -> I64Vector4: ...
    @property
    def bggo(self) -> I64Vector4: ...
    @property
    def bggl(self) -> I64Vector4: ...
    @property
    def bgbr(self) -> I64Vector4: ...
    @property
    def bgbg(self) -> I64Vector4: ...
    @property
    def bgbb(self) -> I64Vector4: ...
    @property
    def bgbo(self) -> I64Vector4: ...
    @property
    def bgbl(self) -> I64Vector4: ...
    @property
    def bgor(self) -> I64Vector4: ...
    @property
    def bgog(self) -> I64Vector4: ...
    @property
    def bgob(self) -> I64Vector4: ...
    @property
    def bgoo(self) -> I64Vector4: ...
    @property
    def bgol(self) -> I64Vector4: ...
    @property
    def bglr(self) -> I64Vector4: ...
    @property
    def bglg(self) -> I64Vector4: ...
    @property
    def bglb(self) -> I64Vector4: ...
    @property
    def bglo(self) -> I64Vector4: ...
    @property
    def bgll(self) -> I64Vector4: ...
    @property
    def bbrr(self) -> I64Vector4: ...
    @property
    def bbrg(self) -> I64Vector4: ...
    @property
    def bbrb(self) -> I64Vector4: ...
    @property
    def bbro(self) -> I64Vector4: ...
    @property
    def bbrl(self) -> I64Vector4: ...
    @property
    def bbgr(self) -> I64Vector4: ...
    @property
    def bbgg(self) -> I64Vector4: ...
    @property
    def bbgb(self) -> I64Vector4: ...
    @property
    def bbgo(self) -> I64Vector4: ...
    @property
    def bbgl(self) -> I64Vector4: ...
    @property
    def bbbr(self) -> I64Vector4: ...
    @property
    def bbbg(self) -> I64Vector4: ...
    @property
    def bbbb(self) -> I64Vector4: ...
    @property
    def bbbo(self) -> I64Vector4: ...
    @property
    def bbbl(self) -> I64Vector4: ...
    @property
    def bbor(self) -> I64Vector4: ...
    @property
    def bbog(self) -> I64Vector4: ...
    @property
    def bbob(self) -> I64Vector4: ...
    @property
    def bboo(self) -> I64Vector4: ...
    @property
    def bbol(self) -> I64Vector4: ...
    @property
    def bblr(self) -> I64Vector4: ...
    @property
    def bblg(self) -> I64Vector4: ...
    @property
    def bblb(self) -> I64Vector4: ...
    @property
    def bblo(self) -> I64Vector4: ...
    @property
    def bbll(self) -> I64Vector4: ...
    @property
    def borr(self) -> I64Vector4: ...
    @property
    def borg(self) -> I64Vector4: ...
    @property
    def borb(self) -> I64Vector4: ...
    @property
    def boro(self) -> I64Vector4: ...
    @property
    def borl(self) -> I64Vector4: ...
    @property
    def bogr(self) -> I64Vector4: ...
    @property
    def bogg(self) -> I64Vector4: ...
    @property
    def bogb(self) -> I64Vector4: ...
    @property
    def bogo(self) -> I64Vector4: ...
    @property
    def bogl(self) -> I64Vector4: ...
    @property
    def bobr(self) -> I64Vector4: ...
    @property
    def bobg(self) -> I64Vector4: ...
    @property
    def bobb(self) -> I64Vector4: ...
    @property
    def bobo(self) -> I64Vector4: ...
    @property
    def bobl(self) -> I64Vector4: ...
    @property
    def boor(self) -> I64Vector4: ...
    @property
    def boog(self) -> I64Vector4: ...
    @property
    def boob(self) -> I64Vector4: ...
    @property
    def booo(self) -> I64Vector4: ...
    @property
    def bool(self) -> I64Vector4: ...
    @property
    def bolr(self) -> I64Vector4: ...
    @property
    def bolg(self) -> I64Vector4: ...
    @property
    def bolb(self) -> I64Vector4: ...
    @property
    def bolo(self) -> I64Vector4: ...
    @property
    def boll(self) -> I64Vector4: ...
    @property
    def blrr(self) -> I64Vector4: ...
    @property
    def blrg(self) -> I64Vector4: ...
    @property
    def blrb(self) -> I64Vector4: ...
    @property
    def blro(self) -> I64Vector4: ...
    @property
    def blrl(self) -> I64Vector4: ...
    @property
    def blgr(self) -> I64Vector4: ...
    @property
    def blgg(self) -> I64Vector4: ...
    @property
    def blgb(self) -> I64Vector4: ...
    @property
    def blgo(self) -> I64Vector4: ...
    @property
    def blgl(self) -> I64Vector4: ...
    @property
    def blbr(self) -> I64Vector4: ...
    @property
    def blbg(self) -> I64Vector4: ...
    @property
    def blbb(self) -> I64Vector4: ...
    @property
    def blbo(self) -> I64Vector4: ...
    @property
    def blbl(self) -> I64Vector4: ...
    @property
    def blor(self) -> I64Vector4: ...
    @property
    def blog(self) -> I64Vector4: ...
    @property
    def blob(self) -> I64Vector4: ...
    @property
    def bloo(self) -> I64Vector4: ...
    @property
    def blol(self) -> I64Vector4: ...
    @property
    def bllr(self) -> I64Vector4: ...
    @property
    def bllg(self) -> I64Vector4: ...
    @property
    def bllb(self) -> I64Vector4: ...
    @property
    def bllo(self) -> I64Vector4: ...
    @property
    def blll(self) -> I64Vector4: ...
    @property
    def orrr(self) -> I64Vector4: ...
    @property
    def orrg(self) -> I64Vector4: ...
    @property
    def orrb(self) -> I64Vector4: ...
    @property
    def orro(self) -> I64Vector4: ...
    @property
    def orrl(self) -> I64Vector4: ...
    @property
    def orgr(self) -> I64Vector4: ...
    @property
    def orgg(self) -> I64Vector4: ...
    @property
    def orgb(self) -> I64Vector4: ...
    @property
    def orgo(self) -> I64Vector4: ...
    @property
    def orgl(self) -> I64Vector4: ...
    @property
    def orbr(self) -> I64Vector4: ...
    @property
    def orbg(self) -> I64Vector4: ...
    @property
    def orbb(self) -> I64Vector4: ...
    @property
    def orbo(self) -> I64Vector4: ...
    @property
    def orbl(self) -> I64Vector4: ...
    @property
    def oror(self) -> I64Vector4: ...
    @property
    def orog(self) -> I64Vector4: ...
    @property
    def orob(self) -> I64Vector4: ...
    @property
    def oroo(self) -> I64Vector4: ...
    @property
    def orol(self) -> I64Vector4: ...
    @property
    def orlr(self) -> I64Vector4: ...
    @property
    def orlg(self) -> I64Vector4: ...
    @property
    def orlb(self) -> I64Vector4: ...
    @property
    def orlo(self) -> I64Vector4: ...
    @property
    def orll(self) -> I64Vector4: ...
    @property
    def ogrr(self) -> I64Vector4: ...
    @property
    def ogrg(self) -> I64Vector4: ...
    @property
    def ogrb(self) -> I64Vector4: ...
    @property
    def ogro(self) -> I64Vector4: ...
    @property
    def ogrl(self) -> I64Vector4: ...
    @property
    def oggr(self) -> I64Vector4: ...
    @property
    def oggg(self) -> I64Vector4: ...
    @property
    def oggb(self) -> I64Vector4: ...
    @property
    def oggo(self) -> I64Vector4: ...
    @property
    def oggl(self) -> I64Vector4: ...
    @property
    def ogbr(self) -> I64Vector4: ...
    @property
    def ogbg(self) -> I64Vector4: ...
    @property
    def ogbb(self) -> I64Vector4: ...
    @property
    def ogbo(self) -> I64Vector4: ...
    @property
    def ogbl(self) -> I64Vector4: ...
    @property
    def ogor(self) -> I64Vector4: ...
    @property
    def ogog(self) -> I64Vector4: ...
    @property
    def ogob(self) -> I64Vector4: ...
    @property
    def ogoo(self) -> I64Vector4: ...
    @property
    def ogol(self) -> I64Vector4: ...
    @property
    def oglr(self) -> I64Vector4: ...
    @property
    def oglg(self) -> I64Vector4: ...
    @property
    def oglb(self) -> I64Vector4: ...
    @property
    def oglo(self) -> I64Vector4: ...
    @property
    def ogll(self) -> I64Vector4: ...
    @property
    def obrr(self) -> I64Vector4: ...
    @property
    def obrg(self) -> I64Vector4: ...
    @property
    def obrb(self) -> I64Vector4: ...
    @property
    def obro(self) -> I64Vector4: ...
    @property
    def obrl(self) -> I64Vector4: ...
    @property
    def obgr(self) -> I64Vector4: ...
    @property
    def obgg(self) -> I64Vector4: ...
    @property
    def obgb(self) -> I64Vector4: ...
    @property
    def obgo(self) -> I64Vector4: ...
    @property
    def obgl(self) -> I64Vector4: ...
    @property
    def obbr(self) -> I64Vector4: ...
    @property
    def obbg(self) -> I64Vector4: ...
    @property
    def obbb(self) -> I64Vector4: ...
    @property
    def obbo(self) -> I64Vector4: ...
    @property
    def obbl(self) -> I64Vector4: ...
    @property
    def obor(self) -> I64Vector4: ...
    @property
    def obog(self) -> I64Vector4: ...
    @property
    def obob(self) -> I64Vector4: ...
    @property
    def oboo(self) -> I64Vector4: ...
    @property
    def obol(self) -> I64Vector4: ...
    @property
    def oblr(self) -> I64Vector4: ...
    @property
    def oblg(self) -> I64Vector4: ...
    @property
    def oblb(self) -> I64Vector4: ...
    @property
    def oblo(self) -> I64Vector4: ...
    @property
    def obll(self) -> I64Vector4: ...
    @property
    def oorr(self) -> I64Vector4: ...
    @property
    def oorg(self) -> I64Vector4: ...
    @property
    def oorb(self) -> I64Vector4: ...
    @property
    def ooro(self) -> I64Vector4: ...
    @property
    def oorl(self) -> I64Vector4: ...
    @property
    def oogr(self) -> I64Vector4: ...
    @property
    def oogg(self) -> I64Vector4: ...
    @property
    def oogb(self) -> I64Vector4: ...
    @property
    def oogo(self) -> I64Vector4: ...
    @property
    def oogl(self) -> I64Vector4: ...
    @property
    def oobr(self) -> I64Vector4: ...
    @property
    def oobg(self) -> I64Vector4: ...
    @property
    def oobb(self) -> I64Vector4: ...
    @property
    def oobo(self) -> I64Vector4: ...
    @property
    def oobl(self) -> I64Vector4: ...
    @property
    def ooor(self) -> I64Vector4: ...
    @property
    def ooog(self) -> I64Vector4: ...
    @property
    def ooob(self) -> I64Vector4: ...
    @property
    def oolr(self) -> I64Vector4: ...
    @property
    def oolg(self) -> I64Vector4: ...
    @property
    def oolb(self) -> I64Vector4: ...
    @property
    def olrr(self) -> I64Vector4: ...
    @property
    def olrg(self) -> I64Vector4: ...
    @property
    def olrb(self) -> I64Vector4: ...
    @property
    def olro(self) -> I64Vector4: ...
    @property
    def olrl(self) -> I64Vector4: ...
    @property
    def olgr(self) -> I64Vector4: ...
    @property
    def olgg(self) -> I64Vector4: ...
    @property
    def olgb(self) -> I64Vector4: ...
    @property
    def olgo(self) -> I64Vector4: ...
    @property
    def olgl(self) -> I64Vector4: ...
    @property
    def olbr(self) -> I64Vector4: ...
    @property
    def olbg(self) -> I64Vector4: ...
    @property
    def olbb(self) -> I64Vector4: ...
    @property
    def olbo(self) -> I64Vector4: ...
    @property
    def olbl(self) -> I64Vector4: ...
    @property
    def olor(self) -> I64Vector4: ...
    @property
    def olog(self) -> I64Vector4: ...
    @property
    def olob(self) -> I64Vector4: ...
    @property
    def ollr(self) -> I64Vector4: ...
    @property
    def ollg(self) -> I64Vector4: ...
    @property
    def ollb(self) -> I64Vector4: ...
    @property
    def lrrr(self) -> I64Vector4: ...
    @property
    def lrrg(self) -> I64Vector4: ...
    @property
    def lrrb(self) -> I64Vector4: ...
    @property
    def lrro(self) -> I64Vector4: ...
    @property
    def lrrl(self) -> I64Vector4: ...
    @property
    def lrgr(self) -> I64Vector4: ...
    @property
    def lrgg(self) -> I64Vector4: ...
    @property
    def lrgb(self) -> I64Vector4: ...
    @property
    def lrgo(self) -> I64Vector4: ...
    @property
    def lrgl(self) -> I64Vector4: ...
    @property
    def lrbr(self) -> I64Vector4: ...
    @property
    def lrbg(self) -> I64Vector4: ...
    @property
    def lrbb(self) -> I64Vector4: ...
    @property
    def lrbo(self) -> I64Vector4: ...
    @property
    def lrbl(self) -> I64Vector4: ...
    @property
    def lror(self) -> I64Vector4: ...
    @property
    def lrog(self) -> I64Vector4: ...
    @property
    def lrob(self) -> I64Vector4: ...
    @property
    def lroo(self) -> I64Vector4: ...
    @property
    def lrol(self) -> I64Vector4: ...
    @property
    def lrlr(self) -> I64Vector4: ...
    @property
    def lrlg(self) -> I64Vector4: ...
    @property
    def lrlb(self) -> I64Vector4: ...
    @property
    def lrlo(self) -> I64Vector4: ...
    @property
    def lrll(self) -> I64Vector4: ...
    @property
    def lgrr(self) -> I64Vector4: ...
    @property
    def lgrg(self) -> I64Vector4: ...
    @property
    def lgrb(self) -> I64Vector4: ...
    @property
    def lgro(self) -> I64Vector4: ...
    @property
    def lgrl(self) -> I64Vector4: ...
    @property
    def lggr(self) -> I64Vector4: ...
    @property
    def lggg(self) -> I64Vector4: ...
    @property
    def lggb(self) -> I64Vector4: ...
    @property
    def lggo(self) -> I64Vector4: ...
    @property
    def lggl(self) -> I64Vector4: ...
    @property
    def lgbr(self) -> I64Vector4: ...
    @property
    def lgbg(self) -> I64Vector4: ...
    @property
    def lgbb(self) -> I64Vector4: ...
    @property
    def lgbo(self) -> I64Vector4: ...
    @property
    def lgbl(self) -> I64Vector4: ...
    @property
    def lgor(self) -> I64Vector4: ...
    @property
    def lgog(self) -> I64Vector4: ...
    @property
    def lgob(self) -> I64Vector4: ...
    @property
    def lgoo(self) -> I64Vector4: ...
    @property
    def lgol(self) -> I64Vector4: ...
    @property
    def lglr(self) -> I64Vector4: ...
    @property
    def lglg(self) -> I64Vector4: ...
    @property
    def lglb(self) -> I64Vector4: ...
    @property
    def lglo(self) -> I64Vector4: ...
    @property
    def lgll(self) -> I64Vector4: ...
    @property
    def lbrr(self) -> I64Vector4: ...
    @property
    def lbrg(self) -> I64Vector4: ...
    @property
    def lbrb(self) -> I64Vector4: ...
    @property
    def lbro(self) -> I64Vector4: ...
    @property
    def lbrl(self) -> I64Vector4: ...
    @property
    def lbgr(self) -> I64Vector4: ...
    @property
    def lbgg(self) -> I64Vector4: ...
    @property
    def lbgb(self) -> I64Vector4: ...
    @property
    def lbgo(self) -> I64Vector4: ...
    @property
    def lbgl(self) -> I64Vector4: ...
    @property
    def lbbr(self) -> I64Vector4: ...
    @property
    def lbbg(self) -> I64Vector4: ...
    @property
    def lbbb(self) -> I64Vector4: ...
    @property
    def lbbo(self) -> I64Vector4: ...
    @property
    def lbbl(self) -> I64Vector4: ...
    @property
    def lbor(self) -> I64Vector4: ...
    @property
    def lbog(self) -> I64Vector4: ...
    @property
    def lbob(self) -> I64Vector4: ...
    @property
    def lboo(self) -> I64Vector4: ...
    @property
    def lbol(self) -> I64Vector4: ...
    @property
    def lblr(self) -> I64Vector4: ...
    @property
    def lblg(self) -> I64Vector4: ...
    @property
    def lblb(self) -> I64Vector4: ...
    @property
    def lblo(self) -> I64Vector4: ...
    @property
    def lbll(self) -> I64Vector4: ...
    @property
    def lorr(self) -> I64Vector4: ...
    @property
    def lorg(self) -> I64Vector4: ...
    @property
    def lorb(self) -> I64Vector4: ...
    @property
    def loro(self) -> I64Vector4: ...
    @property
    def lorl(self) -> I64Vector4: ...
    @property
    def logr(self) -> I64Vector4: ...
    @property
    def logg(self) -> I64Vector4: ...
    @property
    def logb(self) -> I64Vector4: ...
    @property
    def logo(self) -> I64Vector4: ...
    @property
    def logl(self) -> I64Vector4: ...
    @property
    def lobr(self) -> I64Vector4: ...
    @property
    def lobg(self) -> I64Vector4: ...
    @property
    def lobb(self) -> I64Vector4: ...
    @property
    def lobo(self) -> I64Vector4: ...
    @property
    def lobl(self) -> I64Vector4: ...
    @property
    def loor(self) -> I64Vector4: ...
    @property
    def loog(self) -> I64Vector4: ...
    @property
    def loob(self) -> I64Vector4: ...
    @property
    def lolr(self) -> I64Vector4: ...
    @property
    def lolg(self) -> I64Vector4: ...
    @property
    def lolb(self) -> I64Vector4: ...
    @property
    def llrr(self) -> I64Vector4: ...
    @property
    def llrg(self) -> I64Vector4: ...
    @property
    def llrb(self) -> I64Vector4: ...
    @property
    def llro(self) -> I64Vector4: ...
    @property
    def llrl(self) -> I64Vector4: ...
    @property
    def llgr(self) -> I64Vector4: ...
    @property
    def llgg(self) -> I64Vector4: ...
    @property
    def llgb(self) -> I64Vector4: ...
    @property
    def llgo(self) -> I64Vector4: ...
    @property
    def llgl(self) -> I64Vector4: ...
    @property
    def llbr(self) -> I64Vector4: ...
    @property
    def llbg(self) -> I64Vector4: ...
    @property
    def llbb(self) -> I64Vector4: ...
    @property
    def llbo(self) -> I64Vector4: ...
    @property
    def llbl(self) -> I64Vector4: ...
    @property
    def llor(self) -> I64Vector4: ...
    @property
    def llog(self) -> I64Vector4: ...
    @property
    def llob(self) -> I64Vector4: ...
    @property
    def lllr(self) -> I64Vector4: ...
    @property
    def lllg(self) -> I64Vector4: ...
    @property
    def lllb(self) -> I64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> I64Vector2: ...
    @property
    def st(self) -> I64Vector2: ...
    @property
    def sq(self) -> I64Vector2: ...
    @property
    def so(self) -> I64Vector2: ...
    @property
    def sl(self) -> I64Vector2: ...
    @property
    def ts(self) -> I64Vector2: ...
    @property
    def tt(self) -> I64Vector2: ...
    @property
    def tq(self) -> I64Vector2: ...
    @property
    def to(self) -> I64Vector2: ...
    @property
    def tl(self) -> I64Vector2: ...
    @property
    def qs(self) -> I64Vector2: ...
    @property
    def qt(self) -> I64Vector2: ...
    @property
    def qq(self) -> I64Vector2: ...
    @property
    def qo(self) -> I64Vector2: ...
    @property
    def ql(self) -> I64Vector2: ...
    @property
    def os(self) -> I64Vector2: ...
    @property
    def ot(self) -> I64Vector2: ...
    @property
    def oq(self) -> I64Vector2: ...
    @property
    def ls(self) -> I64Vector2: ...
    @property
    def lt(self) -> I64Vector2: ...
    @property
    def lq(self) -> I64Vector2: ...
    @property
    def sss(self) -> I64Vector3: ...
    @property
    def sst(self) -> I64Vector3: ...
    @property
    def ssq(self) -> I64Vector3: ...
    @property
    def sso(self) -> I64Vector3: ...
    @property
    def ssl(self) -> I64Vector3: ...
    @property
    def sts(self) -> I64Vector3: ...
    @property
    def stt(self) -> I64Vector3: ...
    @property
    def stq(self) -> I64Vector3: ...
    @property
    def sto(self) -> I64Vector3: ...
    @property
    def stl(self) -> I64Vector3: ...
    @property
    def sqs(self) -> I64Vector3: ...
    @property
    def sqt(self) -> I64Vector3: ...
    @property
    def sqq(self) -> I64Vector3: ...
    @property
    def sqo(self) -> I64Vector3: ...
    @property
    def sql(self) -> I64Vector3: ...
    @property
    def sos(self) -> I64Vector3: ...
    @property
    def sot(self) -> I64Vector3: ...
    @property
    def soq(self) -> I64Vector3: ...
    @property
    def soo(self) -> I64Vector3: ...
    @property
    def sol(self) -> I64Vector3: ...
    @property
    def sls(self) -> I64Vector3: ...
    @property
    def slt(self) -> I64Vector3: ...
    @property
    def slq(self) -> I64Vector3: ...
    @property
    def slo(self) -> I64Vector3: ...
    @property
    def sll(self) -> I64Vector3: ...
    @property
    def tss(self) -> I64Vector3: ...
    @property
    def tst(self) -> I64Vector3: ...
    @property
    def tsq(self) -> I64Vector3: ...
    @property
    def tso(self) -> I64Vector3: ...
    @property
    def tsl(self) -> I64Vector3: ...
    @property
    def tts(self) -> I64Vector3: ...
    @property
    def ttt(self) -> I64Vector3: ...
    @property
    def ttq(self) -> I64Vector3: ...
    @property
    def tto(self) -> I64Vector3: ...
    @property
    def ttl(self) -> I64Vector3: ...
    @property
    def tqs(self) -> I64Vector3: ...
    @property
    def tqt(self) -> I64Vector3: ...
    @property
    def tqq(self) -> I64Vector3: ...
    @property
    def tqo(self) -> I64Vector3: ...
    @property
    def tql(self) -> I64Vector3: ...
    @property
    def tos(self) -> I64Vector3: ...
    @property
    def tot(self) -> I64Vector3: ...
    @property
    def toq(self) -> I64Vector3: ...
    @property
    def too(self) -> I64Vector3: ...
    @property
    def tol(self) -> I64Vector3: ...
    @property
    def tls(self) -> I64Vector3: ...
    @property
    def tlt(self) -> I64Vector3: ...
    @property
    def tlq(self) -> I64Vector3: ...
    @property
    def tlo(self) -> I64Vector3: ...
    @property
    def tll(self) -> I64Vector3: ...
    @property
    def qss(self) -> I64Vector3: ...
    @property
    def qst(self) -> I64Vector3: ...
    @property
    def qsq(self) -> I64Vector3: ...
    @property
    def qso(self) -> I64Vector3: ...
    @property
    def qsl(self) -> I64Vector3: ...
    @property
    def qts(self) -> I64Vector3: ...
    @property
    def qtt(self) -> I64Vector3: ...
    @property
    def qtq(self) -> I64Vector3: ...
    @property
    def qto(self) -> I64Vector3: ...
    @property
    def qtl(self) -> I64Vector3: ...
    @property
    def qqs(self) -> I64Vector3: ...
    @property
    def qqt(self) -> I64Vector3: ...
    @property
    def qqq(self) -> I64Vector3: ...
    @property
    def qqo(self) -> I64Vector3: ...
    @property
    def qql(self) -> I64Vector3: ...
    @property
    def qos(self) -> I64Vector3: ...
    @property
    def qot(self) -> I64Vector3: ...
    @property
    def qoq(self) -> I64Vector3: ...
    @property
    def qoo(self) -> I64Vector3: ...
    @property
    def qol(self) -> I64Vector3: ...
    @property
    def qls(self) -> I64Vector3: ...
    @property
    def qlt(self) -> I64Vector3: ...
    @property
    def qlq(self) -> I64Vector3: ...
    @property
    def qlo(self) -> I64Vector3: ...
    @property
    def qll(self) -> I64Vector3: ...
    @property
    def oss(self) -> I64Vector3: ...
    @property
    def ost(self) -> I64Vector3: ...
    @property
    def osq(self) -> I64Vector3: ...
    @property
    def oso(self) -> I64Vector3: ...
    @property
    def osl(self) -> I64Vector3: ...
    @property
    def ots(self) -> I64Vector3: ...
    @property
    def ott(self) -> I64Vector3: ...
    @property
    def otq(self) -> I64Vector3: ...
    @property
    def oto(self) -> I64Vector3: ...
    @property
    def otl(self) -> I64Vector3: ...
    @property
    def oqs(self) -> I64Vector3: ...
    @property
    def oqt(self) -> I64Vector3: ...
    @property
    def oqq(self) -> I64Vector3: ...
    @property
    def oqo(self) -> I64Vector3: ...
    @property
    def oql(self) -> I64Vector3: ...
    @property
    def oos(self) -> I64Vector3: ...
    @property
    def oot(self) -> I64Vector3: ...
    @property
    def ooq(self) -> I64Vector3: ...
    @property
    def ols(self) -> I64Vector3: ...
    @property
    def olt(self) -> I64Vector3: ...
    @property
    def olq(self) -> I64Vector3: ...
    @property
    def lss(self) -> I64Vector3: ...
    @property
    def lst(self) -> I64Vector3: ...
    @property
    def lsq(self) -> I64Vector3: ...
    @property
    def lso(self) -> I64Vector3: ...
    @property
    def lsl(self) -> I64Vector3: ...
    @property
    def lts(self) -> I64Vector3: ...
    @property
    def ltt(self) -> I64Vector3: ...
    @property
    def ltq(self) -> I64Vector3: ...
    @property
    def lto(self) -> I64Vector3: ...
    @property
    def ltl(self) -> I64Vector3: ...
    @property
    def lqs(self) -> I64Vector3: ...
    @property
    def lqt(self) -> I64Vector3: ...
    @property
    def lqq(self) -> I64Vector3: ...
    @property
    def lqo(self) -> I64Vector3: ...
    @property
    def lql(self) -> I64Vector3: ...
    @property
    def los(self) -> I64Vector3: ...
    @property
    def lot(self) -> I64Vector3: ...
    @property
    def loq(self) -> I64Vector3: ...
    @property
    def lls(self) -> I64Vector3: ...
    @property
    def llt(self) -> I64Vector3: ...
    @property
    def llq(self) -> I64Vector3: ...
    @property
    def ssss(self) -> I64Vector4: ...
    @property
    def ssst(self) -> I64Vector4: ...
    @property
    def sssq(self) -> I64Vector4: ...
    @property
    def ssso(self) -> I64Vector4: ...
    @property
    def sssl(self) -> I64Vector4: ...
    @property
    def ssts(self) -> I64Vector4: ...
    @property
    def sstt(self) -> I64Vector4: ...
    @property
    def sstq(self) -> I64Vector4: ...
    @property
    def ssto(self) -> I64Vector4: ...
    @property
    def sstl(self) -> I64Vector4: ...
    @property
    def ssqs(self) -> I64Vector4: ...
    @property
    def ssqt(self) -> I64Vector4: ...
    @property
    def ssqq(self) -> I64Vector4: ...
    @property
    def ssqo(self) -> I64Vector4: ...
    @property
    def ssql(self) -> I64Vector4: ...
    @property
    def ssos(self) -> I64Vector4: ...
    @property
    def ssot(self) -> I64Vector4: ...
    @property
    def ssoq(self) -> I64Vector4: ...
    @property
    def ssoo(self) -> I64Vector4: ...
    @property
    def ssol(self) -> I64Vector4: ...
    @property
    def ssls(self) -> I64Vector4: ...
    @property
    def sslt(self) -> I64Vector4: ...
    @property
    def sslq(self) -> I64Vector4: ...
    @property
    def sslo(self) -> I64Vector4: ...
    @property
    def ssll(self) -> I64Vector4: ...
    @property
    def stss(self) -> I64Vector4: ...
    @property
    def stst(self) -> I64Vector4: ...
    @property
    def stsq(self) -> I64Vector4: ...
    @property
    def stso(self) -> I64Vector4: ...
    @property
    def stsl(self) -> I64Vector4: ...
    @property
    def stts(self) -> I64Vector4: ...
    @property
    def sttt(self) -> I64Vector4: ...
    @property
    def sttq(self) -> I64Vector4: ...
    @property
    def stto(self) -> I64Vector4: ...
    @property
    def sttl(self) -> I64Vector4: ...
    @property
    def stqs(self) -> I64Vector4: ...
    @property
    def stqt(self) -> I64Vector4: ...
    @property
    def stqq(self) -> I64Vector4: ...
    @property
    def stqo(self) -> I64Vector4: ...
    @property
    def stql(self) -> I64Vector4: ...
    @property
    def stos(self) -> I64Vector4: ...
    @property
    def stot(self) -> I64Vector4: ...
    @property
    def stoq(self) -> I64Vector4: ...
    @property
    def stoo(self) -> I64Vector4: ...
    @property
    def stol(self) -> I64Vector4: ...
    @property
    def stls(self) -> I64Vector4: ...
    @property
    def stlt(self) -> I64Vector4: ...
    @property
    def stlq(self) -> I64Vector4: ...
    @property
    def stlo(self) -> I64Vector4: ...
    @property
    def stll(self) -> I64Vector4: ...
    @property
    def sqss(self) -> I64Vector4: ...
    @property
    def sqst(self) -> I64Vector4: ...
    @property
    def sqsq(self) -> I64Vector4: ...
    @property
    def sqso(self) -> I64Vector4: ...
    @property
    def sqsl(self) -> I64Vector4: ...
    @property
    def sqts(self) -> I64Vector4: ...
    @property
    def sqtt(self) -> I64Vector4: ...
    @property
    def sqtq(self) -> I64Vector4: ...
    @property
    def sqto(self) -> I64Vector4: ...
    @property
    def sqtl(self) -> I64Vector4: ...
    @property
    def sqqs(self) -> I64Vector4: ...
    @property
    def sqqt(self) -> I64Vector4: ...
    @property
    def sqqq(self) -> I64Vector4: ...
    @property
    def sqqo(self) -> I64Vector4: ...
    @property
    def sqql(self) -> I64Vector4: ...
    @property
    def sqos(self) -> I64Vector4: ...
    @property
    def sqot(self) -> I64Vector4: ...
    @property
    def sqoq(self) -> I64Vector4: ...
    @property
    def sqoo(self) -> I64Vector4: ...
    @property
    def sqol(self) -> I64Vector4: ...
    @property
    def sqls(self) -> I64Vector4: ...
    @property
    def sqlt(self) -> I64Vector4: ...
    @property
    def sqlq(self) -> I64Vector4: ...
    @property
    def sqlo(self) -> I64Vector4: ...
    @property
    def sqll(self) -> I64Vector4: ...
    @property
    def soss(self) -> I64Vector4: ...
    @property
    def sost(self) -> I64Vector4: ...
    @property
    def sosq(self) -> I64Vector4: ...
    @property
    def soso(self) -> I64Vector4: ...
    @property
    def sosl(self) -> I64Vector4: ...
    @property
    def sots(self) -> I64Vector4: ...
    @property
    def sott(self) -> I64Vector4: ...
    @property
    def sotq(self) -> I64Vector4: ...
    @property
    def soto(self) -> I64Vector4: ...
    @property
    def sotl(self) -> I64Vector4: ...
    @property
    def soqs(self) -> I64Vector4: ...
    @property
    def soqt(self) -> I64Vector4: ...
    @property
    def soqq(self) -> I64Vector4: ...
    @property
    def soqo(self) -> I64Vector4: ...
    @property
    def soql(self) -> I64Vector4: ...
    @property
    def soos(self) -> I64Vector4: ...
    @property
    def soot(self) -> I64Vector4: ...
    @property
    def sooq(self) -> I64Vector4: ...
    @property
    def sooo(self) -> I64Vector4: ...
    @property
    def sool(self) -> I64Vector4: ...
    @property
    def sols(self) -> I64Vector4: ...
    @property
    def solt(self) -> I64Vector4: ...
    @property
    def solq(self) -> I64Vector4: ...
    @property
    def solo(self) -> I64Vector4: ...
    @property
    def soll(self) -> I64Vector4: ...
    @property
    def slss(self) -> I64Vector4: ...
    @property
    def slst(self) -> I64Vector4: ...
    @property
    def slsq(self) -> I64Vector4: ...
    @property
    def slso(self) -> I64Vector4: ...
    @property
    def slsl(self) -> I64Vector4: ...
    @property
    def slts(self) -> I64Vector4: ...
    @property
    def sltt(self) -> I64Vector4: ...
    @property
    def sltq(self) -> I64Vector4: ...
    @property
    def slto(self) -> I64Vector4: ...
    @property
    def sltl(self) -> I64Vector4: ...
    @property
    def slqs(self) -> I64Vector4: ...
    @property
    def slqt(self) -> I64Vector4: ...
    @property
    def slqq(self) -> I64Vector4: ...
    @property
    def slqo(self) -> I64Vector4: ...
    @property
    def slql(self) -> I64Vector4: ...
    @property
    def slos(self) -> I64Vector4: ...
    @property
    def slot(self) -> I64Vector4: ...
    @property
    def sloq(self) -> I64Vector4: ...
    @property
    def sloo(self) -> I64Vector4: ...
    @property
    def slol(self) -> I64Vector4: ...
    @property
    def slls(self) -> I64Vector4: ...
    @property
    def sllt(self) -> I64Vector4: ...
    @property
    def sllq(self) -> I64Vector4: ...
    @property
    def sllo(self) -> I64Vector4: ...
    @property
    def slll(self) -> I64Vector4: ...
    @property
    def tsss(self) -> I64Vector4: ...
    @property
    def tsst(self) -> I64Vector4: ...
    @property
    def tssq(self) -> I64Vector4: ...
    @property
    def tsso(self) -> I64Vector4: ...
    @property
    def tssl(self) -> I64Vector4: ...
    @property
    def tsts(self) -> I64Vector4: ...
    @property
    def tstt(self) -> I64Vector4: ...
    @property
    def tstq(self) -> I64Vector4: ...
    @property
    def tsto(self) -> I64Vector4: ...
    @property
    def tstl(self) -> I64Vector4: ...
    @property
    def tsqs(self) -> I64Vector4: ...
    @property
    def tsqt(self) -> I64Vector4: ...
    @property
    def tsqq(self) -> I64Vector4: ...
    @property
    def tsqo(self) -> I64Vector4: ...
    @property
    def tsql(self) -> I64Vector4: ...
    @property
    def tsos(self) -> I64Vector4: ...
    @property
    def tsot(self) -> I64Vector4: ...
    @property
    def tsoq(self) -> I64Vector4: ...
    @property
    def tsoo(self) -> I64Vector4: ...
    @property
    def tsol(self) -> I64Vector4: ...
    @property
    def tsls(self) -> I64Vector4: ...
    @property
    def tslt(self) -> I64Vector4: ...
    @property
    def tslq(self) -> I64Vector4: ...
    @property
    def tslo(self) -> I64Vector4: ...
    @property
    def tsll(self) -> I64Vector4: ...
    @property
    def ttss(self) -> I64Vector4: ...
    @property
    def ttst(self) -> I64Vector4: ...
    @property
    def ttsq(self) -> I64Vector4: ...
    @property
    def ttso(self) -> I64Vector4: ...
    @property
    def ttsl(self) -> I64Vector4: ...
    @property
    def ttts(self) -> I64Vector4: ...
    @property
    def tttt(self) -> I64Vector4: ...
    @property
    def tttq(self) -> I64Vector4: ...
    @property
    def ttto(self) -> I64Vector4: ...
    @property
    def tttl(self) -> I64Vector4: ...
    @property
    def ttqs(self) -> I64Vector4: ...
    @property
    def ttqt(self) -> I64Vector4: ...
    @property
    def ttqq(self) -> I64Vector4: ...
    @property
    def ttqo(self) -> I64Vector4: ...
    @property
    def ttql(self) -> I64Vector4: ...
    @property
    def ttos(self) -> I64Vector4: ...
    @property
    def ttot(self) -> I64Vector4: ...
    @property
    def ttoq(self) -> I64Vector4: ...
    @property
    def ttoo(self) -> I64Vector4: ...
    @property
    def ttol(self) -> I64Vector4: ...
    @property
    def ttls(self) -> I64Vector4: ...
    @property
    def ttlt(self) -> I64Vector4: ...
    @property
    def ttlq(self) -> I64Vector4: ...
    @property
    def ttlo(self) -> I64Vector4: ...
    @property
    def ttll(self) -> I64Vector4: ...
    @property
    def tqss(self) -> I64Vector4: ...
    @property
    def tqst(self) -> I64Vector4: ...
    @property
    def tqsq(self) -> I64Vector4: ...
    @property
    def tqso(self) -> I64Vector4: ...
    @property
    def tqsl(self) -> I64Vector4: ...
    @property
    def tqts(self) -> I64Vector4: ...
    @property
    def tqtt(self) -> I64Vector4: ...
    @property
    def tqtq(self) -> I64Vector4: ...
    @property
    def tqto(self) -> I64Vector4: ...
    @property
    def tqtl(self) -> I64Vector4: ...
    @property
    def tqqs(self) -> I64Vector4: ...
    @property
    def tqqt(self) -> I64Vector4: ...
    @property
    def tqqq(self) -> I64Vector4: ...
    @property
    def tqqo(self) -> I64Vector4: ...
    @property
    def tqql(self) -> I64Vector4: ...
    @property
    def tqos(self) -> I64Vector4: ...
    @property
    def tqot(self) -> I64Vector4: ...
    @property
    def tqoq(self) -> I64Vector4: ...
    @property
    def tqoo(self) -> I64Vector4: ...
    @property
    def tqol(self) -> I64Vector4: ...
    @property
    def tqls(self) -> I64Vector4: ...
    @property
    def tqlt(self) -> I64Vector4: ...
    @property
    def tqlq(self) -> I64Vector4: ...
    @property
    def tqlo(self) -> I64Vector4: ...
    @property
    def tqll(self) -> I64Vector4: ...
    @property
    def toss(self) -> I64Vector4: ...
    @property
    def tost(self) -> I64Vector4: ...
    @property
    def tosq(self) -> I64Vector4: ...
    @property
    def toso(self) -> I64Vector4: ...
    @property
    def tosl(self) -> I64Vector4: ...
    @property
    def tots(self) -> I64Vector4: ...
    @property
    def tott(self) -> I64Vector4: ...
    @property
    def totq(self) -> I64Vector4: ...
    @property
    def toto(self) -> I64Vector4: ...
    @property
    def totl(self) -> I64Vector4: ...
    @property
    def toqs(self) -> I64Vector4: ...
    @property
    def toqt(self) -> I64Vector4: ...
    @property
    def toqq(self) -> I64Vector4: ...
    @property
    def toqo(self) -> I64Vector4: ...
    @property
    def toql(self) -> I64Vector4: ...
    @property
    def toos(self) -> I64Vector4: ...
    @property
    def toot(self) -> I64Vector4: ...
    @property
    def tooq(self) -> I64Vector4: ...
    @property
    def tooo(self) -> I64Vector4: ...
    @property
    def tool(self) -> I64Vector4: ...
    @property
    def tols(self) -> I64Vector4: ...
    @property
    def tolt(self) -> I64Vector4: ...
    @property
    def tolq(self) -> I64Vector4: ...
    @property
    def tolo(self) -> I64Vector4: ...
    @property
    def toll(self) -> I64Vector4: ...
    @property
    def tlss(self) -> I64Vector4: ...
    @property
    def tlst(self) -> I64Vector4: ...
    @property
    def tlsq(self) -> I64Vector4: ...
    @property
    def tlso(self) -> I64Vector4: ...
    @property
    def tlsl(self) -> I64Vector4: ...
    @property
    def tlts(self) -> I64Vector4: ...
    @property
    def tltt(self) -> I64Vector4: ...
    @property
    def tltq(self) -> I64Vector4: ...
    @property
    def tlto(self) -> I64Vector4: ...
    @property
    def tltl(self) -> I64Vector4: ...
    @property
    def tlqs(self) -> I64Vector4: ...
    @property
    def tlqt(self) -> I64Vector4: ...
    @property
    def tlqq(self) -> I64Vector4: ...
    @property
    def tlqo(self) -> I64Vector4: ...
    @property
    def tlql(self) -> I64Vector4: ...
    @property
    def tlos(self) -> I64Vector4: ...
    @property
    def tlot(self) -> I64Vector4: ...
    @property
    def tloq(self) -> I64Vector4: ...
    @property
    def tloo(self) -> I64Vector4: ...
    @property
    def tlol(self) -> I64Vector4: ...
    @property
    def tlls(self) -> I64Vector4: ...
    @property
    def tllt(self) -> I64Vector4: ...
    @property
    def tllq(self) -> I64Vector4: ...
    @property
    def tllo(self) -> I64Vector4: ...
    @property
    def tlll(self) -> I64Vector4: ...
    @property
    def qsss(self) -> I64Vector4: ...
    @property
    def qsst(self) -> I64Vector4: ...
    @property
    def qssq(self) -> I64Vector4: ...
    @property
    def qsso(self) -> I64Vector4: ...
    @property
    def qssl(self) -> I64Vector4: ...
    @property
    def qsts(self) -> I64Vector4: ...
    @property
    def qstt(self) -> I64Vector4: ...
    @property
    def qstq(self) -> I64Vector4: ...
    @property
    def qsto(self) -> I64Vector4: ...
    @property
    def qstl(self) -> I64Vector4: ...
    @property
    def qsqs(self) -> I64Vector4: ...
    @property
    def qsqt(self) -> I64Vector4: ...
    @property
    def qsqq(self) -> I64Vector4: ...
    @property
    def qsqo(self) -> I64Vector4: ...
    @property
    def qsql(self) -> I64Vector4: ...
    @property
    def qsos(self) -> I64Vector4: ...
    @property
    def qsot(self) -> I64Vector4: ...
    @property
    def qsoq(self) -> I64Vector4: ...
    @property
    def qsoo(self) -> I64Vector4: ...
    @property
    def qsol(self) -> I64Vector4: ...
    @property
    def qsls(self) -> I64Vector4: ...
    @property
    def qslt(self) -> I64Vector4: ...
    @property
    def qslq(self) -> I64Vector4: ...
    @property
    def qslo(self) -> I64Vector4: ...
    @property
    def qsll(self) -> I64Vector4: ...
    @property
    def qtss(self) -> I64Vector4: ...
    @property
    def qtst(self) -> I64Vector4: ...
    @property
    def qtsq(self) -> I64Vector4: ...
    @property
    def qtso(self) -> I64Vector4: ...
    @property
    def qtsl(self) -> I64Vector4: ...
    @property
    def qtts(self) -> I64Vector4: ...
    @property
    def qttt(self) -> I64Vector4: ...
    @property
    def qttq(self) -> I64Vector4: ...
    @property
    def qtto(self) -> I64Vector4: ...
    @property
    def qttl(self) -> I64Vector4: ...
    @property
    def qtqs(self) -> I64Vector4: ...
    @property
    def qtqt(self) -> I64Vector4: ...
    @property
    def qtqq(self) -> I64Vector4: ...
    @property
    def qtqo(self) -> I64Vector4: ...
    @property
    def qtql(self) -> I64Vector4: ...
    @property
    def qtos(self) -> I64Vector4: ...
    @property
    def qtot(self) -> I64Vector4: ...
    @property
    def qtoq(self) -> I64Vector4: ...
    @property
    def qtoo(self) -> I64Vector4: ...
    @property
    def qtol(self) -> I64Vector4: ...
    @property
    def qtls(self) -> I64Vector4: ...
    @property
    def qtlt(self) -> I64Vector4: ...
    @property
    def qtlq(self) -> I64Vector4: ...
    @property
    def qtlo(self) -> I64Vector4: ...
    @property
    def qtll(self) -> I64Vector4: ...
    @property
    def qqss(self) -> I64Vector4: ...
    @property
    def qqst(self) -> I64Vector4: ...
    @property
    def qqsq(self) -> I64Vector4: ...
    @property
    def qqso(self) -> I64Vector4: ...
    @property
    def qqsl(self) -> I64Vector4: ...
    @property
    def qqts(self) -> I64Vector4: ...
    @property
    def qqtt(self) -> I64Vector4: ...
    @property
    def qqtq(self) -> I64Vector4: ...
    @property
    def qqto(self) -> I64Vector4: ...
    @property
    def qqtl(self) -> I64Vector4: ...
    @property
    def qqqs(self) -> I64Vector4: ...
    @property
    def qqqt(self) -> I64Vector4: ...
    @property
    def qqqq(self) -> I64Vector4: ...
    @property
    def qqqo(self) -> I64Vector4: ...
    @property
    def qqql(self) -> I64Vector4: ...
    @property
    def qqos(self) -> I64Vector4: ...
    @property
    def qqot(self) -> I64Vector4: ...
    @property
    def qqoq(self) -> I64Vector4: ...
    @property
    def qqoo(self) -> I64Vector4: ...
    @property
    def qqol(self) -> I64Vector4: ...
    @property
    def qqls(self) -> I64Vector4: ...
    @property
    def qqlt(self) -> I64Vector4: ...
    @property
    def qqlq(self) -> I64Vector4: ...
    @property
    def qqlo(self) -> I64Vector4: ...
    @property
    def qqll(self) -> I64Vector4: ...
    @property
    def qoss(self) -> I64Vector4: ...
    @property
    def qost(self) -> I64Vector4: ...
    @property
    def qosq(self) -> I64Vector4: ...
    @property
    def qoso(self) -> I64Vector4: ...
    @property
    def qosl(self) -> I64Vector4: ...
    @property
    def qots(self) -> I64Vector4: ...
    @property
    def qott(self) -> I64Vector4: ...
    @property
    def qotq(self) -> I64Vector4: ...
    @property
    def qoto(self) -> I64Vector4: ...
    @property
    def qotl(self) -> I64Vector4: ...
    @property
    def qoqs(self) -> I64Vector4: ...
    @property
    def qoqt(self) -> I64Vector4: ...
    @property
    def qoqq(self) -> I64Vector4: ...
    @property
    def qoqo(self) -> I64Vector4: ...
    @property
    def qoql(self) -> I64Vector4: ...
    @property
    def qoos(self) -> I64Vector4: ...
    @property
    def qoot(self) -> I64Vector4: ...
    @property
    def qooq(self) -> I64Vector4: ...
    @property
    def qooo(self) -> I64Vector4: ...
    @property
    def qool(self) -> I64Vector4: ...
    @property
    def qols(self) -> I64Vector4: ...
    @property
    def qolt(self) -> I64Vector4: ...
    @property
    def qolq(self) -> I64Vector4: ...
    @property
    def qolo(self) -> I64Vector4: ...
    @property
    def qoll(self) -> I64Vector4: ...
    @property
    def qlss(self) -> I64Vector4: ...
    @property
    def qlst(self) -> I64Vector4: ...
    @property
    def qlsq(self) -> I64Vector4: ...
    @property
    def qlso(self) -> I64Vector4: ...
    @property
    def qlsl(self) -> I64Vector4: ...
    @property
    def qlts(self) -> I64Vector4: ...
    @property
    def qltt(self) -> I64Vector4: ...
    @property
    def qltq(self) -> I64Vector4: ...
    @property
    def qlto(self) -> I64Vector4: ...
    @property
    def qltl(self) -> I64Vector4: ...
    @property
    def qlqs(self) -> I64Vector4: ...
    @property
    def qlqt(self) -> I64Vector4: ...
    @property
    def qlqq(self) -> I64Vector4: ...
    @property
    def qlqo(self) -> I64Vector4: ...
    @property
    def qlql(self) -> I64Vector4: ...
    @property
    def qlos(self) -> I64Vector4: ...
    @property
    def qlot(self) -> I64Vector4: ...
    @property
    def qloq(self) -> I64Vector4: ...
    @property
    def qloo(self) -> I64Vector4: ...
    @property
    def qlol(self) -> I64Vector4: ...
    @property
    def qlls(self) -> I64Vector4: ...
    @property
    def qllt(self) -> I64Vector4: ...
    @property
    def qllq(self) -> I64Vector4: ...
    @property
    def qllo(self) -> I64Vector4: ...
    @property
    def qlll(self) -> I64Vector4: ...
    @property
    def osss(self) -> I64Vector4: ...
    @property
    def osst(self) -> I64Vector4: ...
    @property
    def ossq(self) -> I64Vector4: ...
    @property
    def osso(self) -> I64Vector4: ...
    @property
    def ossl(self) -> I64Vector4: ...
    @property
    def osts(self) -> I64Vector4: ...
    @property
    def ostt(self) -> I64Vector4: ...
    @property
    def ostq(self) -> I64Vector4: ...
    @property
    def osto(self) -> I64Vector4: ...
    @property
    def ostl(self) -> I64Vector4: ...
    @property
    def osqs(self) -> I64Vector4: ...
    @property
    def osqt(self) -> I64Vector4: ...
    @property
    def osqq(self) -> I64Vector4: ...
    @property
    def osqo(self) -> I64Vector4: ...
    @property
    def osql(self) -> I64Vector4: ...
    @property
    def osos(self) -> I64Vector4: ...
    @property
    def osot(self) -> I64Vector4: ...
    @property
    def osoq(self) -> I64Vector4: ...
    @property
    def osoo(self) -> I64Vector4: ...
    @property
    def osol(self) -> I64Vector4: ...
    @property
    def osls(self) -> I64Vector4: ...
    @property
    def oslt(self) -> I64Vector4: ...
    @property
    def oslq(self) -> I64Vector4: ...
    @property
    def oslo(self) -> I64Vector4: ...
    @property
    def osll(self) -> I64Vector4: ...
    @property
    def otss(self) -> I64Vector4: ...
    @property
    def otst(self) -> I64Vector4: ...
    @property
    def otsq(self) -> I64Vector4: ...
    @property
    def otso(self) -> I64Vector4: ...
    @property
    def otsl(self) -> I64Vector4: ...
    @property
    def otts(self) -> I64Vector4: ...
    @property
    def ottt(self) -> I64Vector4: ...
    @property
    def ottq(self) -> I64Vector4: ...
    @property
    def otto(self) -> I64Vector4: ...
    @property
    def ottl(self) -> I64Vector4: ...
    @property
    def otqs(self) -> I64Vector4: ...
    @property
    def otqt(self) -> I64Vector4: ...
    @property
    def otqq(self) -> I64Vector4: ...
    @property
    def otqo(self) -> I64Vector4: ...
    @property
    def otql(self) -> I64Vector4: ...
    @property
    def otos(self) -> I64Vector4: ...
    @property
    def otot(self) -> I64Vector4: ...
    @property
    def otoq(self) -> I64Vector4: ...
    @property
    def otoo(self) -> I64Vector4: ...
    @property
    def otol(self) -> I64Vector4: ...
    @property
    def otls(self) -> I64Vector4: ...
    @property
    def otlt(self) -> I64Vector4: ...
    @property
    def otlq(self) -> I64Vector4: ...
    @property
    def otlo(self) -> I64Vector4: ...
    @property
    def otll(self) -> I64Vector4: ...
    @property
    def oqss(self) -> I64Vector4: ...
    @property
    def oqst(self) -> I64Vector4: ...
    @property
    def oqsq(self) -> I64Vector4: ...
    @property
    def oqso(self) -> I64Vector4: ...
    @property
    def oqsl(self) -> I64Vector4: ...
    @property
    def oqts(self) -> I64Vector4: ...
    @property
    def oqtt(self) -> I64Vector4: ...
    @property
    def oqtq(self) -> I64Vector4: ...
    @property
    def oqto(self) -> I64Vector4: ...
    @property
    def oqtl(self) -> I64Vector4: ...
    @property
    def oqqs(self) -> I64Vector4: ...
    @property
    def oqqt(self) -> I64Vector4: ...
    @property
    def oqqq(self) -> I64Vector4: ...
    @property
    def oqqo(self) -> I64Vector4: ...
    @property
    def oqql(self) -> I64Vector4: ...
    @property
    def oqos(self) -> I64Vector4: ...
    @property
    def oqot(self) -> I64Vector4: ...
    @property
    def oqoq(self) -> I64Vector4: ...
    @property
    def oqoo(self) -> I64Vector4: ...
    @property
    def oqol(self) -> I64Vector4: ...
    @property
    def oqls(self) -> I64Vector4: ...
    @property
    def oqlt(self) -> I64Vector4: ...
    @property
    def oqlq(self) -> I64Vector4: ...
    @property
    def oqlo(self) -> I64Vector4: ...
    @property
    def oqll(self) -> I64Vector4: ...
    @property
    def ooss(self) -> I64Vector4: ...
    @property
    def oost(self) -> I64Vector4: ...
    @property
    def oosq(self) -> I64Vector4: ...
    @property
    def ooso(self) -> I64Vector4: ...
    @property
    def oosl(self) -> I64Vector4: ...
    @property
    def oots(self) -> I64Vector4: ...
    @property
    def oott(self) -> I64Vector4: ...
    @property
    def ootq(self) -> I64Vector4: ...
    @property
    def ooto(self) -> I64Vector4: ...
    @property
    def ootl(self) -> I64Vector4: ...
    @property
    def ooqs(self) -> I64Vector4: ...
    @property
    def ooqt(self) -> I64Vector4: ...
    @property
    def ooqq(self) -> I64Vector4: ...
    @property
    def ooqo(self) -> I64Vector4: ...
    @property
    def ooql(self) -> I64Vector4: ...
    @property
    def ooos(self) -> I64Vector4: ...
    @property
    def ooot(self) -> I64Vector4: ...
    @property
    def oooq(self) -> I64Vector4: ...
    @property
    def ools(self) -> I64Vector4: ...
    @property
    def oolt(self) -> I64Vector4: ...
    @property
    def oolq(self) -> I64Vector4: ...
    @property
    def olss(self) -> I64Vector4: ...
    @property
    def olst(self) -> I64Vector4: ...
    @property
    def olsq(self) -> I64Vector4: ...
    @property
    def olso(self) -> I64Vector4: ...
    @property
    def olsl(self) -> I64Vector4: ...
    @property
    def olts(self) -> I64Vector4: ...
    @property
    def oltt(self) -> I64Vector4: ...
    @property
    def oltq(self) -> I64Vector4: ...
    @property
    def olto(self) -> I64Vector4: ...
    @property
    def oltl(self) -> I64Vector4: ...
    @property
    def olqs(self) -> I64Vector4: ...
    @property
    def olqt(self) -> I64Vector4: ...
    @property
    def olqq(self) -> I64Vector4: ...
    @property
    def olqo(self) -> I64Vector4: ...
    @property
    def olql(self) -> I64Vector4: ...
    @property
    def olos(self) -> I64Vector4: ...
    @property
    def olot(self) -> I64Vector4: ...
    @property
    def oloq(self) -> I64Vector4: ...
    @property
    def olls(self) -> I64Vector4: ...
    @property
    def ollt(self) -> I64Vector4: ...
    @property
    def ollq(self) -> I64Vector4: ...
    @property
    def lsss(self) -> I64Vector4: ...
    @property
    def lsst(self) -> I64Vector4: ...
    @property
    def lssq(self) -> I64Vector4: ...
    @property
    def lsso(self) -> I64Vector4: ...
    @property
    def lssl(self) -> I64Vector4: ...
    @property
    def lsts(self) -> I64Vector4: ...
    @property
    def lstt(self) -> I64Vector4: ...
    @property
    def lstq(self) -> I64Vector4: ...
    @property
    def lsto(self) -> I64Vector4: ...
    @property
    def lstl(self) -> I64Vector4: ...
    @property
    def lsqs(self) -> I64Vector4: ...
    @property
    def lsqt(self) -> I64Vector4: ...
    @property
    def lsqq(self) -> I64Vector4: ...
    @property
    def lsqo(self) -> I64Vector4: ...
    @property
    def lsql(self) -> I64Vector4: ...
    @property
    def lsos(self) -> I64Vector4: ...
    @property
    def lsot(self) -> I64Vector4: ...
    @property
    def lsoq(self) -> I64Vector4: ...
    @property
    def lsoo(self) -> I64Vector4: ...
    @property
    def lsol(self) -> I64Vector4: ...
    @property
    def lsls(self) -> I64Vector4: ...
    @property
    def lslt(self) -> I64Vector4: ...
    @property
    def lslq(self) -> I64Vector4: ...
    @property
    def lslo(self) -> I64Vector4: ...
    @property
    def lsll(self) -> I64Vector4: ...
    @property
    def ltss(self) -> I64Vector4: ...
    @property
    def ltst(self) -> I64Vector4: ...
    @property
    def ltsq(self) -> I64Vector4: ...
    @property
    def ltso(self) -> I64Vector4: ...
    @property
    def ltsl(self) -> I64Vector4: ...
    @property
    def ltts(self) -> I64Vector4: ...
    @property
    def lttt(self) -> I64Vector4: ...
    @property
    def lttq(self) -> I64Vector4: ...
    @property
    def ltto(self) -> I64Vector4: ...
    @property
    def lttl(self) -> I64Vector4: ...
    @property
    def ltqs(self) -> I64Vector4: ...
    @property
    def ltqt(self) -> I64Vector4: ...
    @property
    def ltqq(self) -> I64Vector4: ...
    @property
    def ltqo(self) -> I64Vector4: ...
    @property
    def ltql(self) -> I64Vector4: ...
    @property
    def ltos(self) -> I64Vector4: ...
    @property
    def ltot(self) -> I64Vector4: ...
    @property
    def ltoq(self) -> I64Vector4: ...
    @property
    def ltoo(self) -> I64Vector4: ...
    @property
    def ltol(self) -> I64Vector4: ...
    @property
    def ltls(self) -> I64Vector4: ...
    @property
    def ltlt(self) -> I64Vector4: ...
    @property
    def ltlq(self) -> I64Vector4: ...
    @property
    def ltlo(self) -> I64Vector4: ...
    @property
    def ltll(self) -> I64Vector4: ...
    @property
    def lqss(self) -> I64Vector4: ...
    @property
    def lqst(self) -> I64Vector4: ...
    @property
    def lqsq(self) -> I64Vector4: ...
    @property
    def lqso(self) -> I64Vector4: ...
    @property
    def lqsl(self) -> I64Vector4: ...
    @property
    def lqts(self) -> I64Vector4: ...
    @property
    def lqtt(self) -> I64Vector4: ...
    @property
    def lqtq(self) -> I64Vector4: ...
    @property
    def lqto(self) -> I64Vector4: ...
    @property
    def lqtl(self) -> I64Vector4: ...
    @property
    def lqqs(self) -> I64Vector4: ...
    @property
    def lqqt(self) -> I64Vector4: ...
    @property
    def lqqq(self) -> I64Vector4: ...
    @property
    def lqqo(self) -> I64Vector4: ...
    @property
    def lqql(self) -> I64Vector4: ...
    @property
    def lqos(self) -> I64Vector4: ...
    @property
    def lqot(self) -> I64Vector4: ...
    @property
    def lqoq(self) -> I64Vector4: ...
    @property
    def lqoo(self) -> I64Vector4: ...
    @property
    def lqol(self) -> I64Vector4: ...
    @property
    def lqls(self) -> I64Vector4: ...
    @property
    def lqlt(self) -> I64Vector4: ...
    @property
    def lqlq(self) -> I64Vector4: ...
    @property
    def lqlo(self) -> I64Vector4: ...
    @property
    def lqll(self) -> I64Vector4: ...
    @property
    def loss(self) -> I64Vector4: ...
    @property
    def lost(self) -> I64Vector4: ...
    @property
    def losq(self) -> I64Vector4: ...
    @property
    def loso(self) -> I64Vector4: ...
    @property
    def losl(self) -> I64Vector4: ...
    @property
    def lots(self) -> I64Vector4: ...
    @property
    def lott(self) -> I64Vector4: ...
    @property
    def lotq(self) -> I64Vector4: ...
    @property
    def loto(self) -> I64Vector4: ...
    @property
    def lotl(self) -> I64Vector4: ...
    @property
    def loqs(self) -> I64Vector4: ...
    @property
    def loqt(self) -> I64Vector4: ...
    @property
    def loqq(self) -> I64Vector4: ...
    @property
    def loqo(self) -> I64Vector4: ...
    @property
    def loql(self) -> I64Vector4: ...
    @property
    def loos(self) -> I64Vector4: ...
    @property
    def loot(self) -> I64Vector4: ...
    @property
    def looq(self) -> I64Vector4: ...
    @property
    def lols(self) -> I64Vector4: ...
    @property
    def lolt(self) -> I64Vector4: ...
    @property
    def lolq(self) -> I64Vector4: ...
    @property
    def llss(self) -> I64Vector4: ...
    @property
    def llst(self) -> I64Vector4: ...
    @property
    def llsq(self) -> I64Vector4: ...
    @property
    def llso(self) -> I64Vector4: ...
    @property
    def llsl(self) -> I64Vector4: ...
    @property
    def llts(self) -> I64Vector4: ...
    @property
    def lltt(self) -> I64Vector4: ...
    @property
    def lltq(self) -> I64Vector4: ...
    @property
    def llto(self) -> I64Vector4: ...
    @property
    def lltl(self) -> I64Vector4: ...
    @property
    def llqs(self) -> I64Vector4: ...
    @property
    def llqt(self) -> I64Vector4: ...
    @property
    def llqq(self) -> I64Vector4: ...
    @property
    def llqo(self) -> I64Vector4: ...
    @property
    def llql(self) -> I64Vector4: ...
    @property
    def llos(self) -> I64Vector4: ...
    @property
    def llot(self) -> I64Vector4: ...
    @property
    def lloq(self) -> I64Vector4: ...
    @property
    def llls(self) -> I64Vector4: ...
    @property
    def lllt(self) -> I64Vector4: ...
    @property
    def lllq(self) -> I64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I64Vector2: ...
    @property
    def uv(self) -> I64Vector2: ...
    @property
    def uo(self) -> I64Vector2: ...
    @property
    def ul(self) -> I64Vector2: ...
    @property
    def vu(self) -> I64Vector2: ...
    @property
    def vv(self) -> I64Vector2: ...
    @property
    def vo(self) -> I64Vector2: ...
    @property
    def vl(self) -> I64Vector2: ...
    @property
    def ou(self) -> I64Vector2: ...
    @property
    def ov(self) -> I64Vector2: ...
    @property
    def lu(self) -> I64Vector2: ...
    @property
    def lv(self) -> I64Vector2: ...
    @property
    def uuu(self) -> I64Vector3: ...
    @property
    def uuv(self) -> I64Vector3: ...
    @property
    def uuo(self) -> I64Vector3: ...
    @property
    def uul(self) -> I64Vector3: ...
    @property
    def uvu(self) -> I64Vector3: ...
    @property
    def uvv(self) -> I64Vector3: ...
    @property
    def uvo(self) -> I64Vector3: ...
    @property
    def uvl(self) -> I64Vector3: ...
    @property
    def uou(self) -> I64Vector3: ...
    @property
    def uov(self) -> I64Vector3: ...
    @property
    def uoo(self) -> I64Vector3: ...
    @property
    def uol(self) -> I64Vector3: ...
    @property
    def ulu(self) -> I64Vector3: ...
    @property
    def ulv(self) -> I64Vector3: ...
    @property
    def ulo(self) -> I64Vector3: ...
    @property
    def ull(self) -> I64Vector3: ...
    @property
    def vuu(self) -> I64Vector3: ...
    @property
    def vuv(self) -> I64Vector3: ...
    @property
    def vuo(self) -> I64Vector3: ...
    @property
    def vul(self) -> I64Vector3: ...
    @property
    def vvu(self) -> I64Vector3: ...
    @property
    def vvv(self) -> I64Vector3: ...
    @property
    def vvo(self) -> I64Vector3: ...
    @property
    def vvl(self) -> I64Vector3: ...
    @property
    def vou(self) -> I64Vector3: ...
    @property
    def vov(self) -> I64Vector3: ...
    @property
    def voo(self) -> I64Vector3: ...
    @property
    def vol(self) -> I64Vector3: ...
    @property
    def vlu(self) -> I64Vector3: ...
    @property
    def vlv(self) -> I64Vector3: ...
    @property
    def vlo(self) -> I64Vector3: ...
    @property
    def vll(self) -> I64Vector3: ...
    @property
    def ouu(self) -> I64Vector3: ...
    @property
    def ouv(self) -> I64Vector3: ...
    @property
    def ouo(self) -> I64Vector3: ...
    @property
    def oul(self) -> I64Vector3: ...
    @property
    def ovu(self) -> I64Vector3: ...
    @property
    def ovv(self) -> I64Vector3: ...
    @property
    def ovo(self) -> I64Vector3: ...
    @property
    def ovl(self) -> I64Vector3: ...
    @property
    def oou(self) -> I64Vector3: ...
    @property
    def oov(self) -> I64Vector3: ...
    @property
    def olu(self) -> I64Vector3: ...
    @property
    def olv(self) -> I64Vector3: ...
    @property
    def luu(self) -> I64Vector3: ...
    @property
    def luv(self) -> I64Vector3: ...
    @property
    def luo(self) -> I64Vector3: ...
    @property
    def lul(self) -> I64Vector3: ...
    @property
    def lvu(self) -> I64Vector3: ...
    @property
    def lvv(self) -> I64Vector3: ...
    @property
    def lvo(self) -> I64Vector3: ...
    @property
    def lvl(self) -> I64Vector3: ...
    @property
    def lou(self) -> I64Vector3: ...
    @property
    def lov(self) -> I64Vector3: ...
    @property
    def llu(self) -> I64Vector3: ...
    @property
    def llv(self) -> I64Vector3: ...
    @property
    def uuuu(self) -> I64Vector4: ...
    @property
    def uuuv(self) -> I64Vector4: ...
    @property
    def uuuo(self) -> I64Vector4: ...
    @property
    def uuul(self) -> I64Vector4: ...
    @property
    def uuvu(self) -> I64Vector4: ...
    @property
    def uuvv(self) -> I64Vector4: ...
    @property
    def uuvo(self) -> I64Vector4: ...
    @property
    def uuvl(self) -> I64Vector4: ...
    @property
    def uuou(self) -> I64Vector4: ...
    @property
    def uuov(self) -> I64Vector4: ...
    @property
    def uuoo(self) -> I64Vector4: ...
    @property
    def uuol(self) -> I64Vector4: ...
    @property
    def uulu(self) -> I64Vector4: ...
    @property
    def uulv(self) -> I64Vector4: ...
    @property
    def uulo(self) -> I64Vector4: ...
    @property
    def uull(self) -> I64Vector4: ...
    @property
    def uvuu(self) -> I64Vector4: ...
    @property
    def uvuv(self) -> I64Vector4: ...
    @property
    def uvuo(self) -> I64Vector4: ...
    @property
    def uvul(self) -> I64Vector4: ...
    @property
    def uvvu(self) -> I64Vector4: ...
    @property
    def uvvv(self) -> I64Vector4: ...
    @property
    def uvvo(self) -> I64Vector4: ...
    @property
    def uvvl(self) -> I64Vector4: ...
    @property
    def uvou(self) -> I64Vector4: ...
    @property
    def uvov(self) -> I64Vector4: ...
    @property
    def uvoo(self) -> I64Vector4: ...
    @property
    def uvol(self) -> I64Vector4: ...
    @property
    def uvlu(self) -> I64Vector4: ...
    @property
    def uvlv(self) -> I64Vector4: ...
    @property
    def uvlo(self) -> I64Vector4: ...
    @property
    def uvll(self) -> I64Vector4: ...
    @property
    def uouu(self) -> I64Vector4: ...
    @property
    def uouv(self) -> I64Vector4: ...
    @property
    def uouo(self) -> I64Vector4: ...
    @property
    def uoul(self) -> I64Vector4: ...
    @property
    def uovu(self) -> I64Vector4: ...
    @property
    def uovv(self) -> I64Vector4: ...
    @property
    def uovo(self) -> I64Vector4: ...
    @property
    def uovl(self) -> I64Vector4: ...
    @property
    def uoou(self) -> I64Vector4: ...
    @property
    def uoov(self) -> I64Vector4: ...
    @property
    def uooo(self) -> I64Vector4: ...
    @property
    def uool(self) -> I64Vector4: ...
    @property
    def uolu(self) -> I64Vector4: ...
    @property
    def uolv(self) -> I64Vector4: ...
    @property
    def uolo(self) -> I64Vector4: ...
    @property
    def uoll(self) -> I64Vector4: ...
    @property
    def uluu(self) -> I64Vector4: ...
    @property
    def uluv(self) -> I64Vector4: ...
    @property
    def uluo(self) -> I64Vector4: ...
    @property
    def ulul(self) -> I64Vector4: ...
    @property
    def ulvu(self) -> I64Vector4: ...
    @property
    def ulvv(self) -> I64Vector4: ...
    @property
    def ulvo(self) -> I64Vector4: ...
    @property
    def ulvl(self) -> I64Vector4: ...
    @property
    def ulou(self) -> I64Vector4: ...
    @property
    def ulov(self) -> I64Vector4: ...
    @property
    def uloo(self) -> I64Vector4: ...
    @property
    def ulol(self) -> I64Vector4: ...
    @property
    def ullu(self) -> I64Vector4: ...
    @property
    def ullv(self) -> I64Vector4: ...
    @property
    def ullo(self) -> I64Vector4: ...
    @property
    def ulll(self) -> I64Vector4: ...
    @property
    def vuuu(self) -> I64Vector4: ...
    @property
    def vuuv(self) -> I64Vector4: ...
    @property
    def vuuo(self) -> I64Vector4: ...
    @property
    def vuul(self) -> I64Vector4: ...
    @property
    def vuvu(self) -> I64Vector4: ...
    @property
    def vuvv(self) -> I64Vector4: ...
    @property
    def vuvo(self) -> I64Vector4: ...
    @property
    def vuvl(self) -> I64Vector4: ...
    @property
    def vuou(self) -> I64Vector4: ...
    @property
    def vuov(self) -> I64Vector4: ...
    @property
    def vuoo(self) -> I64Vector4: ...
    @property
    def vuol(self) -> I64Vector4: ...
    @property
    def vulu(self) -> I64Vector4: ...
    @property
    def vulv(self) -> I64Vector4: ...
    @property
    def vulo(self) -> I64Vector4: ...
    @property
    def vull(self) -> I64Vector4: ...
    @property
    def vvuu(self) -> I64Vector4: ...
    @property
    def vvuv(self) -> I64Vector4: ...
    @property
    def vvuo(self) -> I64Vector4: ...
    @property
    def vvul(self) -> I64Vector4: ...
    @property
    def vvvu(self) -> I64Vector4: ...
    @property
    def vvvv(self) -> I64Vector4: ...
    @property
    def vvvo(self) -> I64Vector4: ...
    @property
    def vvvl(self) -> I64Vector4: ...
    @property
    def vvou(self) -> I64Vector4: ...
    @property
    def vvov(self) -> I64Vector4: ...
    @property
    def vvoo(self) -> I64Vector4: ...
    @property
    def vvol(self) -> I64Vector4: ...
    @property
    def vvlu(self) -> I64Vector4: ...
    @property
    def vvlv(self) -> I64Vector4: ...
    @property
    def vvlo(self) -> I64Vector4: ...
    @property
    def vvll(self) -> I64Vector4: ...
    @property
    def vouu(self) -> I64Vector4: ...
    @property
    def vouv(self) -> I64Vector4: ...
    @property
    def vouo(self) -> I64Vector4: ...
    @property
    def voul(self) -> I64Vector4: ...
    @property
    def vovu(self) -> I64Vector4: ...
    @property
    def vovv(self) -> I64Vector4: ...
    @property
    def vovo(self) -> I64Vector4: ...
    @property
    def vovl(self) -> I64Vector4: ...
    @property
    def voou(self) -> I64Vector4: ...
    @property
    def voov(self) -> I64Vector4: ...
    @property
    def vooo(self) -> I64Vector4: ...
    @property
    def vool(self) -> I64Vector4: ...
    @property
    def volu(self) -> I64Vector4: ...
    @property
    def volv(self) -> I64Vector4: ...
    @property
    def volo(self) -> I64Vector4: ...
    @property
    def voll(self) -> I64Vector4: ...
    @property
    def vluu(self) -> I64Vector4: ...
    @property
    def vluv(self) -> I64Vector4: ...
    @property
    def vluo(self) -> I64Vector4: ...
    @property
    def vlul(self) -> I64Vector4: ...
    @property
    def vlvu(self) -> I64Vector4: ...
    @property
    def vlvv(self) -> I64Vector4: ...
    @property
    def vlvo(self) -> I64Vector4: ...
    @property
    def vlvl(self) -> I64Vector4: ...
    @property
    def vlou(self) -> I64Vector4: ...
    @property
    def vlov(self) -> I64Vector4: ...
    @property
    def vloo(self) -> I64Vector4: ...
    @property
    def vlol(self) -> I64Vector4: ...
    @property
    def vllu(self) -> I64Vector4: ...
    @property
    def vllv(self) -> I64Vector4: ...
    @property
    def vllo(self) -> I64Vector4: ...
    @property
    def vlll(self) -> I64Vector4: ...
    @property
    def ouuu(self) -> I64Vector4: ...
    @property
    def ouuv(self) -> I64Vector4: ...
    @property
    def ouuo(self) -> I64Vector4: ...
    @property
    def ouul(self) -> I64Vector4: ...
    @property
    def ouvu(self) -> I64Vector4: ...
    @property
    def ouvv(self) -> I64Vector4: ...
    @property
    def ouvo(self) -> I64Vector4: ...
    @property
    def ouvl(self) -> I64Vector4: ...
    @property
    def ouou(self) -> I64Vector4: ...
    @property
    def ouov(self) -> I64Vector4: ...
    @property
    def ouoo(self) -> I64Vector4: ...
    @property
    def ouol(self) -> I64Vector4: ...
    @property
    def oulu(self) -> I64Vector4: ...
    @property
    def oulv(self) -> I64Vector4: ...
    @property
    def oulo(self) -> I64Vector4: ...
    @property
    def oull(self) -> I64Vector4: ...
    @property
    def ovuu(self) -> I64Vector4: ...
    @property
    def ovuv(self) -> I64Vector4: ...
    @property
    def ovuo(self) -> I64Vector4: ...
    @property
    def ovul(self) -> I64Vector4: ...
    @property
    def ovvu(self) -> I64Vector4: ...
    @property
    def ovvv(self) -> I64Vector4: ...
    @property
    def ovvo(self) -> I64Vector4: ...
    @property
    def ovvl(self) -> I64Vector4: ...
    @property
    def ovou(self) -> I64Vector4: ...
    @property
    def ovov(self) -> I64Vector4: ...
    @property
    def ovoo(self) -> I64Vector4: ...
    @property
    def ovol(self) -> I64Vector4: ...
    @property
    def ovlu(self) -> I64Vector4: ...
    @property
    def ovlv(self) -> I64Vector4: ...
    @property
    def ovlo(self) -> I64Vector4: ...
    @property
    def ovll(self) -> I64Vector4: ...
    @property
    def oouu(self) -> I64Vector4: ...
    @property
    def oouv(self) -> I64Vector4: ...
    @property
    def oouo(self) -> I64Vector4: ...
    @property
    def ooul(self) -> I64Vector4: ...
    @property
    def oovu(self) -> I64Vector4: ...
    @property
    def oovv(self) -> I64Vector4: ...
    @property
    def oovo(self) -> I64Vector4: ...
    @property
    def oovl(self) -> I64Vector4: ...
    @property
    def ooou(self) -> I64Vector4: ...
    @property
    def ooov(self) -> I64Vector4: ...
    @property
    def oolu(self) -> I64Vector4: ...
    @property
    def oolv(self) -> I64Vector4: ...
    @property
    def oluu(self) -> I64Vector4: ...
    @property
    def oluv(self) -> I64Vector4: ...
    @property
    def oluo(self) -> I64Vector4: ...
    @property
    def olul(self) -> I64Vector4: ...
    @property
    def olvu(self) -> I64Vector4: ...
    @property
    def olvv(self) -> I64Vector4: ...
    @property
    def olvo(self) -> I64Vector4: ...
    @property
    def olvl(self) -> I64Vector4: ...
    @property
    def olou(self) -> I64Vector4: ...
    @property
    def olov(self) -> I64Vector4: ...
    @property
    def ollu(self) -> I64Vector4: ...
    @property
    def ollv(self) -> I64Vector4: ...
    @property
    def luuu(self) -> I64Vector4: ...
    @property
    def luuv(self) -> I64Vector4: ...
    @property
    def luuo(self) -> I64Vector4: ...
    @property
    def luul(self) -> I64Vector4: ...
    @property
    def luvu(self) -> I64Vector4: ...
    @property
    def luvv(self) -> I64Vector4: ...
    @property
    def luvo(self) -> I64Vector4: ...
    @property
    def luvl(self) -> I64Vector4: ...
    @property
    def luou(self) -> I64Vector4: ...
    @property
    def luov(self) -> I64Vector4: ...
    @property
    def luoo(self) -> I64Vector4: ...
    @property
    def luol(self) -> I64Vector4: ...
    @property
    def lulu(self) -> I64Vector4: ...
    @property
    def lulv(self) -> I64Vector4: ...
    @property
    def lulo(self) -> I64Vector4: ...
    @property
    def lull(self) -> I64Vector4: ...
    @property
    def lvuu(self) -> I64Vector4: ...
    @property
    def lvuv(self) -> I64Vector4: ...
    @property
    def lvuo(self) -> I64Vector4: ...
    @property
    def lvul(self) -> I64Vector4: ...
    @property
    def lvvu(self) -> I64Vector4: ...
    @property
    def lvvv(self) -> I64Vector4: ...
    @property
    def lvvo(self) -> I64Vector4: ...
    @property
    def lvvl(self) -> I64Vector4: ...
    @property
    def lvou(self) -> I64Vector4: ...
    @property
    def lvov(self) -> I64Vector4: ...
    @property
    def lvoo(self) -> I64Vector4: ...
    @property
    def lvol(self) -> I64Vector4: ...
    @property
    def lvlu(self) -> I64Vector4: ...
    @property
    def lvlv(self) -> I64Vector4: ...
    @property
    def lvlo(self) -> I64Vector4: ...
    @property
    def lvll(self) -> I64Vector4: ...
    @property
    def louu(self) -> I64Vector4: ...
    @property
    def louv(self) -> I64Vector4: ...
    @property
    def louo(self) -> I64Vector4: ...
    @property
    def loul(self) -> I64Vector4: ...
    @property
    def lovu(self) -> I64Vector4: ...
    @property
    def lovv(self) -> I64Vector4: ...
    @property
    def lovo(self) -> I64Vector4: ...
    @property
    def lovl(self) -> I64Vector4: ...
    @property
    def loou(self) -> I64Vector4: ...
    @property
    def loov(self) -> I64Vector4: ...
    @property
    def lolu(self) -> I64Vector4: ...
    @property
    def lolv(self) -> I64Vector4: ...
    @property
    def lluu(self) -> I64Vector4: ...
    @property
    def lluv(self) -> I64Vector4: ...
    @property
    def lluo(self) -> I64Vector4: ...
    @property
    def llul(self) -> I64Vector4: ...
    @property
    def llvu(self) -> I64Vector4: ...
    @property
    def llvv(self) -> I64Vector4: ...
    @property
    def llvo(self) -> I64Vector4: ...
    @property
    def llvl(self) -> I64Vector4: ...
    @property
    def llou(self) -> I64Vector4: ...
    @property
    def llov(self) -> I64Vector4: ...
    @property
    def lllu(self) -> I64Vector4: ...
    @property
    def lllv(self) -> I64Vector4: ...
    @property
    def oo(self) -> I64Vector2: ...
    @property
    def ol(self) -> I64Vector2: ...
    @property
    def lo(self) -> I64Vector2: ...
    @property
    def ll(self) -> I64Vector2: ...
    @property
    def ooo(self) -> I64Vector3: ...
    @property
    def ool(self) -> I64Vector3: ...
    @property
    def olo(self) -> I64Vector3: ...
    @property
    def oll(self) -> I64Vector3: ...
    @property
    def loo(self) -> I64Vector3: ...
    @property
    def lol(self) -> I64Vector3: ...
    @property
    def llo(self) -> I64Vector3: ...
    @property
    def lll(self) -> I64Vector3: ...
    @property
    def oooo(self) -> I64Vector4: ...
    @property
    def oool(self) -> I64Vector4: ...
    @property
    def oolo(self) -> I64Vector4: ...
    @property
    def ooll(self) -> I64Vector4: ...
    @property
    def oloo(self) -> I64Vector4: ...
    @property
    def olol(self) -> I64Vector4: ...
    @property
    def ollo(self) -> I64Vector4: ...
    @property
    def olll(self) -> I64Vector4: ...
    @property
    def looo(self) -> I64Vector4: ...
    @property
    def lool(self) -> I64Vector4: ...
    @property
    def lolo(self) -> I64Vector4: ...
    @property
    def loll(self) -> I64Vector4: ...
    @property
    def lloo(self) -> I64Vector4: ...
    @property
    def llol(self) -> I64Vector4: ...
    @property
    def lllo(self) -> I64Vector4: ...
    @property
    def llll(self) -> I64Vector4: ...
    def min(self, n: Number, /) -> I64Vector3: ...
    def max(self, n: Number, /) -> I64Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> I64Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I64Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector3: ...

@final
class I64Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I64Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I64Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> I64Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> I64Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[I64Vector3]: ...

@final
class U64Vector3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __add__(self, other: Number) -> U64Vector3: ...
    @overload
    def __radd__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __radd__(self, other: Number) -> U64Vector3: ...
    @overload
    def __sub__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __sub__(self, other: Number) -> U64Vector3: ...
    @overload
    def __rsub__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __rsub__(self, other: Number) -> U64Vector3: ...
    @overload
    def __mul__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __mul__(self, other: Number) -> U64Vector3: ...
    @overload
    def __rmul__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __rmul__(self, other: Number) -> U64Vector3: ...
    @overload
    def __truediv__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __truediv__(self, other: Number) -> U64Vector3: ...
    @overload
    def __rtruediv__(self, other: U64Vector3) -> U64Vector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> U64Vector3: ...
    def __abs__(self) -> U64Vector3: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def xx(self) -> U64Vector2: ...
    @property
    def xy(self) -> U64Vector2: ...
    @property
    def xz(self) -> U64Vector2: ...
    @property
    def xo(self) -> U64Vector2: ...
    @property
    def xl(self) -> U64Vector2: ...
    @property
    def yx(self) -> U64Vector2: ...
    @property
    def yy(self) -> U64Vector2: ...
    @property
    def yz(self) -> U64Vector2: ...
    @property
    def yo(self) -> U64Vector2: ...
    @property
    def yl(self) -> U64Vector2: ...
    @property
    def zx(self) -> U64Vector2: ...
    @property
    def zy(self) -> U64Vector2: ...
    @property
    def zz(self) -> U64Vector2: ...
    @property
    def zo(self) -> U64Vector2: ...
    @property
    def zl(self) -> U64Vector2: ...
    @property
    def ox(self) -> U64Vector2: ...
    @property
    def oy(self) -> U64Vector2: ...
    @property
    def oz(self) -> U64Vector2: ...
    @property
    def lx(self) -> U64Vector2: ...
    @property
    def ly(self) -> U64Vector2: ...
    @property
    def lz(self) -> U64Vector2: ...
    @property
    def xxx(self) -> U64Vector3: ...
    @property
    def xxy(self) -> U64Vector3: ...
    @property
    def xxz(self) -> U64Vector3: ...
    @property
    def xxo(self) -> U64Vector3: ...
    @property
    def xxl(self) -> U64Vector3: ...
    @property
    def xyx(self) -> U64Vector3: ...
    @property
    def xyy(self) -> U64Vector3: ...
    @property
    def xyz(self) -> U64Vector3: ...
    @property
    def xyo(self) -> U64Vector3: ...
    @property
    def xyl(self) -> U64Vector3: ...
    @property
    def xzx(self) -> U64Vector3: ...
    @property
    def xzy(self) -> U64Vector3: ...
    @property
    def xzz(self) -> U64Vector3: ...
    @property
    def xzo(self) -> U64Vector3: ...
    @property
    def xzl(self) -> U64Vector3: ...
    @property
    def xox(self) -> U64Vector3: ...
    @property
    def xoy(self) -> U64Vector3: ...
    @property
    def xoz(self) -> U64Vector3: ...
    @property
    def xoo(self) -> U64Vector3: ...
    @property
    def xol(self) -> U64Vector3: ...
    @property
    def xlx(self) -> U64Vector3: ...
    @property
    def xly(self) -> U64Vector3: ...
    @property
    def xlz(self) -> U64Vector3: ...
    @property
    def xlo(self) -> U64Vector3: ...
    @property
    def xll(self) -> U64Vector3: ...
    @property
    def yxx(self) -> U64Vector3: ...
    @property
    def yxy(self) -> U64Vector3: ...
    @property
    def yxz(self) -> U64Vector3: ...
    @property
    def yxo(self) -> U64Vector3: ...
    @property
    def yxl(self) -> U64Vector3: ...
    @property
    def yyx(self) -> U64Vector3: ...
    @property
    def yyy(self) -> U64Vector3: ...
    @property
    def yyz(self) -> U64Vector3: ...
    @property
    def yyo(self) -> U64Vector3: ...
    @property
    def yyl(self) -> U64Vector3: ...
    @property
    def yzx(self) -> U64Vector3: ...
    @property
    def yzy(self) -> U64Vector3: ...
    @property
    def yzz(self) -> U64Vector3: ...
    @property
    def yzo(self) -> U64Vector3: ...
    @property
    def yzl(self) -> U64Vector3: ...
    @property
    def yox(self) -> U64Vector3: ...
    @property
    def yoy(self) -> U64Vector3: ...
    @property
    def yoz(self) -> U64Vector3: ...
    @property
    def yoo(self) -> U64Vector3: ...
    @property
    def yol(self) -> U64Vector3: ...
    @property
    def ylx(self) -> U64Vector3: ...
    @property
    def yly(self) -> U64Vector3: ...
    @property
    def ylz(self) -> U64Vector3: ...
    @property
    def ylo(self) -> U64Vector3: ...
    @property
    def yll(self) -> U64Vector3: ...
    @property
    def zxx(self) -> U64Vector3: ...
    @property
    def zxy(self) -> U64Vector3: ...
    @property
    def zxz(self) -> U64Vector3: ...
    @property
    def zxo(self) -> U64Vector3: ...
    @property
    def zxl(self) -> U64Vector3: ...
    @property
    def zyx(self) -> U64Vector3: ...
    @property
    def zyy(self) -> U64Vector3: ...
    @property
    def zyz(self) -> U64Vector3: ...
    @property
    def zyo(self) -> U64Vector3: ...
    @property
    def zyl(self) -> U64Vector3: ...
    @property
    def zzx(self) -> U64Vector3: ...
    @property
    def zzy(self) -> U64Vector3: ...
    @property
    def zzz(self) -> U64Vector3: ...
    @property
    def zzo(self) -> U64Vector3: ...
    @property
    def zzl(self) -> U64Vector3: ...
    @property
    def zox(self) -> U64Vector3: ...
    @property
    def zoy(self) -> U64Vector3: ...
    @property
    def zoz(self) -> U64Vector3: ...
    @property
    def zoo(self) -> U64Vector3: ...
    @property
    def zol(self) -> U64Vector3: ...
    @property
    def zlx(self) -> U64Vector3: ...
    @property
    def zly(self) -> U64Vector3: ...
    @property
    def zlz(self) -> U64Vector3: ...
    @property
    def zlo(self) -> U64Vector3: ...
    @property
    def zll(self) -> U64Vector3: ...
    @property
    def oxx(self) -> U64Vector3: ...
    @property
    def oxy(self) -> U64Vector3: ...
    @property
    def oxz(self) -> U64Vector3: ...
    @property
    def oxo(self) -> U64Vector3: ...
    @property
    def oxl(self) -> U64Vector3: ...
    @property
    def oyx(self) -> U64Vector3: ...
    @property
    def oyy(self) -> U64Vector3: ...
    @property
    def oyz(self) -> U64Vector3: ...
    @property
    def oyo(self) -> U64Vector3: ...
    @property
    def oyl(self) -> U64Vector3: ...
    @property
    def ozx(self) -> U64Vector3: ...
    @property
    def ozy(self) -> U64Vector3: ...
    @property
    def ozz(self) -> U64Vector3: ...
    @property
    def ozo(self) -> U64Vector3: ...
    @property
    def ozl(self) -> U64Vector3: ...
    @property
    def oox(self) -> U64Vector3: ...
    @property
    def ooy(self) -> U64Vector3: ...
    @property
    def ooz(self) -> U64Vector3: ...
    @property
    def olx(self) -> U64Vector3: ...
    @property
    def oly(self) -> U64Vector3: ...
    @property
    def olz(self) -> U64Vector3: ...
    @property
    def lxx(self) -> U64Vector3: ...
    @property
    def lxy(self) -> U64Vector3: ...
    @property
    def lxz(self) -> U64Vector3: ...
    @property
    def lxo(self) -> U64Vector3: ...
    @property
    def lxl(self) -> U64Vector3: ...
    @property
    def lyx(self) -> U64Vector3: ...
    @property
    def lyy(self) -> U64Vector3: ...
    @property
    def lyz(self) -> U64Vector3: ...
    @property
    def lyo(self) -> U64Vector3: ...
    @property
    def lyl(self) -> U64Vector3: ...
    @property
    def lzx(self) -> U64Vector3: ...
    @property
    def lzy(self) -> U64Vector3: ...
    @property
    def lzz(self) -> U64Vector3: ...
    @property
    def lzo(self) -> U64Vector3: ...
    @property
    def lzl(self) -> U64Vector3: ...
    @property
    def lox(self) -> U64Vector3: ...
    @property
    def loy(self) -> U64Vector3: ...
    @property
    def loz(self) -> U64Vector3: ...
    @property
    def llx(self) -> U64Vector3: ...
    @property
    def lly(self) -> U64Vector3: ...
    @property
    def llz(self) -> U64Vector3: ...
    @property
    def xxxx(self) -> U64Vector4: ...
    @property
    def xxxy(self) -> U64Vector4: ...
    @property
    def xxxz(self) -> U64Vector4: ...
    @property
    def xxxo(self) -> U64Vector4: ...
    @property
    def xxxl(self) -> U64Vector4: ...
    @property
    def xxyx(self) -> U64Vector4: ...
    @property
    def xxyy(self) -> U64Vector4: ...
    @property
    def xxyz(self) -> U64Vector4: ...
    @property
    def xxyo(self) -> U64Vector4: ...
    @property
    def xxyl(self) -> U64Vector4: ...
    @property
    def xxzx(self) -> U64Vector4: ...
    @property
    def xxzy(self) -> U64Vector4: ...
    @property
    def xxzz(self) -> U64Vector4: ...
    @property
    def xxzo(self) -> U64Vector4: ...
    @property
    def xxzl(self) -> U64Vector4: ...
    @property
    def xxox(self) -> U64Vector4: ...
    @property
    def xxoy(self) -> U64Vector4: ...
    @property
    def xxoz(self) -> U64Vector4: ...
    @property
    def xxoo(self) -> U64Vector4: ...
    @property
    def xxol(self) -> U64Vector4: ...
    @property
    def xxlx(self) -> U64Vector4: ...
    @property
    def xxly(self) -> U64Vector4: ...
    @property
    def xxlz(self) -> U64Vector4: ...
    @property
    def xxlo(self) -> U64Vector4: ...
    @property
    def xxll(self) -> U64Vector4: ...
    @property
    def xyxx(self) -> U64Vector4: ...
    @property
    def xyxy(self) -> U64Vector4: ...
    @property
    def xyxz(self) -> U64Vector4: ...
    @property
    def xyxo(self) -> U64Vector4: ...
    @property
    def xyxl(self) -> U64Vector4: ...
    @property
    def xyyx(self) -> U64Vector4: ...
    @property
    def xyyy(self) -> U64Vector4: ...
    @property
    def xyyz(self) -> U64Vector4: ...
    @property
    def xyyo(self) -> U64Vector4: ...
    @property
    def xyyl(self) -> U64Vector4: ...
    @property
    def xyzx(self) -> U64Vector4: ...
    @property
    def xyzy(self) -> U64Vector4: ...
    @property
    def xyzz(self) -> U64Vector4: ...
    @property
    def xyzo(self) -> U64Vector4: ...
    @property
    def xyzl(self) -> U64Vector4: ...
    @property
    def xyox(self) -> U64Vector4: ...
    @property
    def xyoy(self) -> U64Vector4: ...
    @property
    def xyoz(self) -> U64Vector4: ...
    @property
    def xyoo(self) -> U64Vector4: ...
    @property
    def xyol(self) -> U64Vector4: ...
    @property
    def xylx(self) -> U64Vector4: ...
    @property
    def xyly(self) -> U64Vector4: ...
    @property
    def xylz(self) -> U64Vector4: ...
    @property
    def xylo(self) -> U64Vector4: ...
    @property
    def xyll(self) -> U64Vector4: ...
    @property
    def xzxx(self) -> U64Vector4: ...
    @property
    def xzxy(self) -> U64Vector4: ...
    @property
    def xzxz(self) -> U64Vector4: ...
    @property
    def xzxo(self) -> U64Vector4: ...
    @property
    def xzxl(self) -> U64Vector4: ...
    @property
    def xzyx(self) -> U64Vector4: ...
    @property
    def xzyy(self) -> U64Vector4: ...
    @property
    def xzyz(self) -> U64Vector4: ...
    @property
    def xzyo(self) -> U64Vector4: ...
    @property
    def xzyl(self) -> U64Vector4: ...
    @property
    def xzzx(self) -> U64Vector4: ...
    @property
    def xzzy(self) -> U64Vector4: ...
    @property
    def xzzz(self) -> U64Vector4: ...
    @property
    def xzzo(self) -> U64Vector4: ...
    @property
    def xzzl(self) -> U64Vector4: ...
    @property
    def xzox(self) -> U64Vector4: ...
    @property
    def xzoy(self) -> U64Vector4: ...
    @property
    def xzoz(self) -> U64Vector4: ...
    @property
    def xzoo(self) -> U64Vector4: ...
    @property
    def xzol(self) -> U64Vector4: ...
    @property
    def xzlx(self) -> U64Vector4: ...
    @property
    def xzly(self) -> U64Vector4: ...
    @property
    def xzlz(self) -> U64Vector4: ...
    @property
    def xzlo(self) -> U64Vector4: ...
    @property
    def xzll(self) -> U64Vector4: ...
    @property
    def xoxx(self) -> U64Vector4: ...
    @property
    def xoxy(self) -> U64Vector4: ...
    @property
    def xoxz(self) -> U64Vector4: ...
    @property
    def xoxo(self) -> U64Vector4: ...
    @property
    def xoxl(self) -> U64Vector4: ...
    @property
    def xoyx(self) -> U64Vector4: ...
    @property
    def xoyy(self) -> U64Vector4: ...
    @property
    def xoyz(self) -> U64Vector4: ...
    @property
    def xoyo(self) -> U64Vector4: ...
    @property
    def xoyl(self) -> U64Vector4: ...
    @property
    def xozx(self) -> U64Vector4: ...
    @property
    def xozy(self) -> U64Vector4: ...
    @property
    def xozz(self) -> U64Vector4: ...
    @property
    def xozo(self) -> U64Vector4: ...
    @property
    def xozl(self) -> U64Vector4: ...
    @property
    def xoox(self) -> U64Vector4: ...
    @property
    def xooy(self) -> U64Vector4: ...
    @property
    def xooz(self) -> U64Vector4: ...
    @property
    def xooo(self) -> U64Vector4: ...
    @property
    def xool(self) -> U64Vector4: ...
    @property
    def xolx(self) -> U64Vector4: ...
    @property
    def xoly(self) -> U64Vector4: ...
    @property
    def xolz(self) -> U64Vector4: ...
    @property
    def xolo(self) -> U64Vector4: ...
    @property
    def xoll(self) -> U64Vector4: ...
    @property
    def xlxx(self) -> U64Vector4: ...
    @property
    def xlxy(self) -> U64Vector4: ...
    @property
    def xlxz(self) -> U64Vector4: ...
    @property
    def xlxo(self) -> U64Vector4: ...
    @property
    def xlxl(self) -> U64Vector4: ...
    @property
    def xlyx(self) -> U64Vector4: ...
    @property
    def xlyy(self) -> U64Vector4: ...
    @property
    def xlyz(self) -> U64Vector4: ...
    @property
    def xlyo(self) -> U64Vector4: ...
    @property
    def xlyl(self) -> U64Vector4: ...
    @property
    def xlzx(self) -> U64Vector4: ...
    @property
    def xlzy(self) -> U64Vector4: ...
    @property
    def xlzz(self) -> U64Vector4: ...
    @property
    def xlzo(self) -> U64Vector4: ...
    @property
    def xlzl(self) -> U64Vector4: ...
    @property
    def xlox(self) -> U64Vector4: ...
    @property
    def xloy(self) -> U64Vector4: ...
    @property
    def xloz(self) -> U64Vector4: ...
    @property
    def xloo(self) -> U64Vector4: ...
    @property
    def xlol(self) -> U64Vector4: ...
    @property
    def xllx(self) -> U64Vector4: ...
    @property
    def xlly(self) -> U64Vector4: ...
    @property
    def xllz(self) -> U64Vector4: ...
    @property
    def xllo(self) -> U64Vector4: ...
    @property
    def xlll(self) -> U64Vector4: ...
    @property
    def yxxx(self) -> U64Vector4: ...
    @property
    def yxxy(self) -> U64Vector4: ...
    @property
    def yxxz(self) -> U64Vector4: ...
    @property
    def yxxo(self) -> U64Vector4: ...
    @property
    def yxxl(self) -> U64Vector4: ...
    @property
    def yxyx(self) -> U64Vector4: ...
    @property
    def yxyy(self) -> U64Vector4: ...
    @property
    def yxyz(self) -> U64Vector4: ...
    @property
    def yxyo(self) -> U64Vector4: ...
    @property
    def yxyl(self) -> U64Vector4: ...
    @property
    def yxzx(self) -> U64Vector4: ...
    @property
    def yxzy(self) -> U64Vector4: ...
    @property
    def yxzz(self) -> U64Vector4: ...
    @property
    def yxzo(self) -> U64Vector4: ...
    @property
    def yxzl(self) -> U64Vector4: ...
    @property
    def yxox(self) -> U64Vector4: ...
    @property
    def yxoy(self) -> U64Vector4: ...
    @property
    def yxoz(self) -> U64Vector4: ...
    @property
    def yxoo(self) -> U64Vector4: ...
    @property
    def yxol(self) -> U64Vector4: ...
    @property
    def yxlx(self) -> U64Vector4: ...
    @property
    def yxly(self) -> U64Vector4: ...
    @property
    def yxlz(self) -> U64Vector4: ...
    @property
    def yxlo(self) -> U64Vector4: ...
    @property
    def yxll(self) -> U64Vector4: ...
    @property
    def yyxx(self) -> U64Vector4: ...
    @property
    def yyxy(self) -> U64Vector4: ...
    @property
    def yyxz(self) -> U64Vector4: ...
    @property
    def yyxo(self) -> U64Vector4: ...
    @property
    def yyxl(self) -> U64Vector4: ...
    @property
    def yyyx(self) -> U64Vector4: ...
    @property
    def yyyy(self) -> U64Vector4: ...
    @property
    def yyyz(self) -> U64Vector4: ...
    @property
    def yyyo(self) -> U64Vector4: ...
    @property
    def yyyl(self) -> U64Vector4: ...
    @property
    def yyzx(self) -> U64Vector4: ...
    @property
    def yyzy(self) -> U64Vector4: ...
    @property
    def yyzz(self) -> U64Vector4: ...
    @property
    def yyzo(self) -> U64Vector4: ...
    @property
    def yyzl(self) -> U64Vector4: ...
    @property
    def yyox(self) -> U64Vector4: ...
    @property
    def yyoy(self) -> U64Vector4: ...
    @property
    def yyoz(self) -> U64Vector4: ...
    @property
    def yyoo(self) -> U64Vector4: ...
    @property
    def yyol(self) -> U64Vector4: ...
    @property
    def yylx(self) -> U64Vector4: ...
    @property
    def yyly(self) -> U64Vector4: ...
    @property
    def yylz(self) -> U64Vector4: ...
    @property
    def yylo(self) -> U64Vector4: ...
    @property
    def yyll(self) -> U64Vector4: ...
    @property
    def yzxx(self) -> U64Vector4: ...
    @property
    def yzxy(self) -> U64Vector4: ...
    @property
    def yzxz(self) -> U64Vector4: ...
    @property
    def yzxo(self) -> U64Vector4: ...
    @property
    def yzxl(self) -> U64Vector4: ...
    @property
    def yzyx(self) -> U64Vector4: ...
    @property
    def yzyy(self) -> U64Vector4: ...
    @property
    def yzyz(self) -> U64Vector4: ...
    @property
    def yzyo(self) -> U64Vector4: ...
    @property
    def yzyl(self) -> U64Vector4: ...
    @property
    def yzzx(self) -> U64Vector4: ...
    @property
    def yzzy(self) -> U64Vector4: ...
    @property
    def yzzz(self) -> U64Vector4: ...
    @property
    def yzzo(self) -> U64Vector4: ...
    @property
    def yzzl(self) -> U64Vector4: ...
    @property
    def yzox(self) -> U64Vector4: ...
    @property
    def yzoy(self) -> U64Vector4: ...
    @property
    def yzoz(self) -> U64Vector4: ...
    @property
    def yzoo(self) -> U64Vector4: ...
    @property
    def yzol(self) -> U64Vector4: ...
    @property
    def yzlx(self) -> U64Vector4: ...
    @property
    def yzly(self) -> U64Vector4: ...
    @property
    def yzlz(self) -> U64Vector4: ...
    @property
    def yzlo(self) -> U64Vector4: ...
    @property
    def yzll(self) -> U64Vector4: ...
    @property
    def yoxx(self) -> U64Vector4: ...
    @property
    def yoxy(self) -> U64Vector4: ...
    @property
    def yoxz(self) -> U64Vector4: ...
    @property
    def yoxo(self) -> U64Vector4: ...
    @property
    def yoxl(self) -> U64Vector4: ...
    @property
    def yoyx(self) -> U64Vector4: ...
    @property
    def yoyy(self) -> U64Vector4: ...
    @property
    def yoyz(self) -> U64Vector4: ...
    @property
    def yoyo(self) -> U64Vector4: ...
    @property
    def yoyl(self) -> U64Vector4: ...
    @property
    def yozx(self) -> U64Vector4: ...
    @property
    def yozy(self) -> U64Vector4: ...
    @property
    def yozz(self) -> U64Vector4: ...
    @property
    def yozo(self) -> U64Vector4: ...
    @property
    def yozl(self) -> U64Vector4: ...
    @property
    def yoox(self) -> U64Vector4: ...
    @property
    def yooy(self) -> U64Vector4: ...
    @property
    def yooz(self) -> U64Vector4: ...
    @property
    def yooo(self) -> U64Vector4: ...
    @property
    def yool(self) -> U64Vector4: ...
    @property
    def yolx(self) -> U64Vector4: ...
    @property
    def yoly(self) -> U64Vector4: ...
    @property
    def yolz(self) -> U64Vector4: ...
    @property
    def yolo(self) -> U64Vector4: ...
    @property
    def yoll(self) -> U64Vector4: ...
    @property
    def ylxx(self) -> U64Vector4: ...
    @property
    def ylxy(self) -> U64Vector4: ...
    @property
    def ylxz(self) -> U64Vector4: ...
    @property
    def ylxo(self) -> U64Vector4: ...
    @property
    def ylxl(self) -> U64Vector4: ...
    @property
    def ylyx(self) -> U64Vector4: ...
    @property
    def ylyy(self) -> U64Vector4: ...
    @property
    def ylyz(self) -> U64Vector4: ...
    @property
    def ylyo(self) -> U64Vector4: ...
    @property
    def ylyl(self) -> U64Vector4: ...
    @property
    def ylzx(self) -> U64Vector4: ...
    @property
    def ylzy(self) -> U64Vector4: ...
    @property
    def ylzz(self) -> U64Vector4: ...
    @property
    def ylzo(self) -> U64Vector4: ...
    @property
    def ylzl(self) -> U64Vector4: ...
    @property
    def ylox(self) -> U64Vector4: ...
    @property
    def yloy(self) -> U64Vector4: ...
    @property
    def yloz(self) -> U64Vector4: ...
    @property
    def yloo(self) -> U64Vector4: ...
    @property
    def ylol(self) -> U64Vector4: ...
    @property
    def yllx(self) -> U64Vector4: ...
    @property
    def ylly(self) -> U64Vector4: ...
    @property
    def yllz(self) -> U64Vector4: ...
    @property
    def yllo(self) -> U64Vector4: ...
    @property
    def ylll(self) -> U64Vector4: ...
    @property
    def zxxx(self) -> U64Vector4: ...
    @property
    def zxxy(self) -> U64Vector4: ...
    @property
    def zxxz(self) -> U64Vector4: ...
    @property
    def zxxo(self) -> U64Vector4: ...
    @property
    def zxxl(self) -> U64Vector4: ...
    @property
    def zxyx(self) -> U64Vector4: ...
    @property
    def zxyy(self) -> U64Vector4: ...
    @property
    def zxyz(self) -> U64Vector4: ...
    @property
    def zxyo(self) -> U64Vector4: ...
    @property
    def zxyl(self) -> U64Vector4: ...
    @property
    def zxzx(self) -> U64Vector4: ...
    @property
    def zxzy(self) -> U64Vector4: ...
    @property
    def zxzz(self) -> U64Vector4: ...
    @property
    def zxzo(self) -> U64Vector4: ...
    @property
    def zxzl(self) -> U64Vector4: ...
    @property
    def zxox(self) -> U64Vector4: ...
    @property
    def zxoy(self) -> U64Vector4: ...
    @property
    def zxoz(self) -> U64Vector4: ...
    @property
    def zxoo(self) -> U64Vector4: ...
    @property
    def zxol(self) -> U64Vector4: ...
    @property
    def zxlx(self) -> U64Vector4: ...
    @property
    def zxly(self) -> U64Vector4: ...
    @property
    def zxlz(self) -> U64Vector4: ...
    @property
    def zxlo(self) -> U64Vector4: ...
    @property
    def zxll(self) -> U64Vector4: ...
    @property
    def zyxx(self) -> U64Vector4: ...
    @property
    def zyxy(self) -> U64Vector4: ...
    @property
    def zyxz(self) -> U64Vector4: ...
    @property
    def zyxo(self) -> U64Vector4: ...
    @property
    def zyxl(self) -> U64Vector4: ...
    @property
    def zyyx(self) -> U64Vector4: ...
    @property
    def zyyy(self) -> U64Vector4: ...
    @property
    def zyyz(self) -> U64Vector4: ...
    @property
    def zyyo(self) -> U64Vector4: ...
    @property
    def zyyl(self) -> U64Vector4: ...
    @property
    def zyzx(self) -> U64Vector4: ...
    @property
    def zyzy(self) -> U64Vector4: ...
    @property
    def zyzz(self) -> U64Vector4: ...
    @property
    def zyzo(self) -> U64Vector4: ...
    @property
    def zyzl(self) -> U64Vector4: ...
    @property
    def zyox(self) -> U64Vector4: ...
    @property
    def zyoy(self) -> U64Vector4: ...
    @property
    def zyoz(self) -> U64Vector4: ...
    @property
    def zyoo(self) -> U64Vector4: ...
    @property
    def zyol(self) -> U64Vector4: ...
    @property
    def zylx(self) -> U64Vector4: ...
    @property
    def zyly(self) -> U64Vector4: ...
    @property
    def zylz(self) -> U64Vector4: ...
    @property
    def zylo(self) -> U64Vector4: ...
    @property
    def zyll(self) -> U64Vector4: ...
    @property
    def zzxx(self) -> U64Vector4: ...
    @property
    def zzxy(self) -> U64Vector4: ...
    @property
    def zzxz(self) -> U64Vector4: ...
    @property
    def zzxo(self) -> U64Vector4: ...
    @property
    def zzxl(self) -> U64Vector4: ...
    @property
    def zzyx(self) -> U64Vector4: ...
    @property
    def zzyy(self) -> U64Vector4: ...
    @property
    def zzyz(self) -> U64Vector4: ...
    @property
    def zzyo(self) -> U64Vector4: ...
    @property
    def zzyl(self) -> U64Vector4: ...
    @property
    def zzzx(self) -> U64Vector4: ...
    @property
    def zzzy(self) -> U64Vector4: ...
    @property
    def zzzz(self) -> U64Vector4: ...
    @property
    def zzzo(self) -> U64Vector4: ...
    @property
    def zzzl(self) -> U64Vector4: ...
    @property
    def zzox(self) -> U64Vector4: ...
    @property
    def zzoy(self) -> U64Vector4: ...
    @property
    def zzoz(self) -> U64Vector4: ...
    @property
    def zzoo(self) -> U64Vector4: ...
    @property
    def zzol(self) -> U64Vector4: ...
    @property
    def zzlx(self) -> U64Vector4: ...
    @property
    def zzly(self) -> U64Vector4: ...
    @property
    def zzlz(self) -> U64Vector4: ...
    @property
    def zzlo(self) -> U64Vector4: ...
    @property
    def zzll(self) -> U64Vector4: ...
    @property
    def zoxx(self) -> U64Vector4: ...
    @property
    def zoxy(self) -> U64Vector4: ...
    @property
    def zoxz(self) -> U64Vector4: ...
    @property
    def zoxo(self) -> U64Vector4: ...
    @property
    def zoxl(self) -> U64Vector4: ...
    @property
    def zoyx(self) -> U64Vector4: ...
    @property
    def zoyy(self) -> U64Vector4: ...
    @property
    def zoyz(self) -> U64Vector4: ...
    @property
    def zoyo(self) -> U64Vector4: ...
    @property
    def zoyl(self) -> U64Vector4: ...
    @property
    def zozx(self) -> U64Vector4: ...
    @property
    def zozy(self) -> U64Vector4: ...
    @property
    def zozz(self) -> U64Vector4: ...
    @property
    def zozo(self) -> U64Vector4: ...
    @property
    def zozl(self) -> U64Vector4: ...
    @property
    def zoox(self) -> U64Vector4: ...
    @property
    def zooy(self) -> U64Vector4: ...
    @property
    def zooz(self) -> U64Vector4: ...
    @property
    def zooo(self) -> U64Vector4: ...
    @property
    def zool(self) -> U64Vector4: ...
    @property
    def zolx(self) -> U64Vector4: ...
    @property
    def zoly(self) -> U64Vector4: ...
    @property
    def zolz(self) -> U64Vector4: ...
    @property
    def zolo(self) -> U64Vector4: ...
    @property
    def zoll(self) -> U64Vector4: ...
    @property
    def zlxx(self) -> U64Vector4: ...
    @property
    def zlxy(self) -> U64Vector4: ...
    @property
    def zlxz(self) -> U64Vector4: ...
    @property
    def zlxo(self) -> U64Vector4: ...
    @property
    def zlxl(self) -> U64Vector4: ...
    @property
    def zlyx(self) -> U64Vector4: ...
    @property
    def zlyy(self) -> U64Vector4: ...
    @property
    def zlyz(self) -> U64Vector4: ...
    @property
    def zlyo(self) -> U64Vector4: ...
    @property
    def zlyl(self) -> U64Vector4: ...
    @property
    def zlzx(self) -> U64Vector4: ...
    @property
    def zlzy(self) -> U64Vector4: ...
    @property
    def zlzz(self) -> U64Vector4: ...
    @property
    def zlzo(self) -> U64Vector4: ...
    @property
    def zlzl(self) -> U64Vector4: ...
    @property
    def zlox(self) -> U64Vector4: ...
    @property
    def zloy(self) -> U64Vector4: ...
    @property
    def zloz(self) -> U64Vector4: ...
    @property
    def zloo(self) -> U64Vector4: ...
    @property
    def zlol(self) -> U64Vector4: ...
    @property
    def zllx(self) -> U64Vector4: ...
    @property
    def zlly(self) -> U64Vector4: ...
    @property
    def zllz(self) -> U64Vector4: ...
    @property
    def zllo(self) -> U64Vector4: ...
    @property
    def zlll(self) -> U64Vector4: ...
    @property
    def oxxx(self) -> U64Vector4: ...
    @property
    def oxxy(self) -> U64Vector4: ...
    @property
    def oxxz(self) -> U64Vector4: ...
    @property
    def oxxo(self) -> U64Vector4: ...
    @property
    def oxxl(self) -> U64Vector4: ...
    @property
    def oxyx(self) -> U64Vector4: ...
    @property
    def oxyy(self) -> U64Vector4: ...
    @property
    def oxyz(self) -> U64Vector4: ...
    @property
    def oxyo(self) -> U64Vector4: ...
    @property
    def oxyl(self) -> U64Vector4: ...
    @property
    def oxzx(self) -> U64Vector4: ...
    @property
    def oxzy(self) -> U64Vector4: ...
    @property
    def oxzz(self) -> U64Vector4: ...
    @property
    def oxzo(self) -> U64Vector4: ...
    @property
    def oxzl(self) -> U64Vector4: ...
    @property
    def oxox(self) -> U64Vector4: ...
    @property
    def oxoy(self) -> U64Vector4: ...
    @property
    def oxoz(self) -> U64Vector4: ...
    @property
    def oxoo(self) -> U64Vector4: ...
    @property
    def oxol(self) -> U64Vector4: ...
    @property
    def oxlx(self) -> U64Vector4: ...
    @property
    def oxly(self) -> U64Vector4: ...
    @property
    def oxlz(self) -> U64Vector4: ...
    @property
    def oxlo(self) -> U64Vector4: ...
    @property
    def oxll(self) -> U64Vector4: ...
    @property
    def oyxx(self) -> U64Vector4: ...
    @property
    def oyxy(self) -> U64Vector4: ...
    @property
    def oyxz(self) -> U64Vector4: ...
    @property
    def oyxo(self) -> U64Vector4: ...
    @property
    def oyxl(self) -> U64Vector4: ...
    @property
    def oyyx(self) -> U64Vector4: ...
    @property
    def oyyy(self) -> U64Vector4: ...
    @property
    def oyyz(self) -> U64Vector4: ...
    @property
    def oyyo(self) -> U64Vector4: ...
    @property
    def oyyl(self) -> U64Vector4: ...
    @property
    def oyzx(self) -> U64Vector4: ...
    @property
    def oyzy(self) -> U64Vector4: ...
    @property
    def oyzz(self) -> U64Vector4: ...
    @property
    def oyzo(self) -> U64Vector4: ...
    @property
    def oyzl(self) -> U64Vector4: ...
    @property
    def oyox(self) -> U64Vector4: ...
    @property
    def oyoy(self) -> U64Vector4: ...
    @property
    def oyoz(self) -> U64Vector4: ...
    @property
    def oyoo(self) -> U64Vector4: ...
    @property
    def oyol(self) -> U64Vector4: ...
    @property
    def oylx(self) -> U64Vector4: ...
    @property
    def oyly(self) -> U64Vector4: ...
    @property
    def oylz(self) -> U64Vector4: ...
    @property
    def oylo(self) -> U64Vector4: ...
    @property
    def oyll(self) -> U64Vector4: ...
    @property
    def ozxx(self) -> U64Vector4: ...
    @property
    def ozxy(self) -> U64Vector4: ...
    @property
    def ozxz(self) -> U64Vector4: ...
    @property
    def ozxo(self) -> U64Vector4: ...
    @property
    def ozxl(self) -> U64Vector4: ...
    @property
    def ozyx(self) -> U64Vector4: ...
    @property
    def ozyy(self) -> U64Vector4: ...
    @property
    def ozyz(self) -> U64Vector4: ...
    @property
    def ozyo(self) -> U64Vector4: ...
    @property
    def ozyl(self) -> U64Vector4: ...
    @property
    def ozzx(self) -> U64Vector4: ...
    @property
    def ozzy(self) -> U64Vector4: ...
    @property
    def ozzz(self) -> U64Vector4: ...
    @property
    def ozzo(self) -> U64Vector4: ...
    @property
    def ozzl(self) -> U64Vector4: ...
    @property
    def ozox(self) -> U64Vector4: ...
    @property
    def ozoy(self) -> U64Vector4: ...
    @property
    def ozoz(self) -> U64Vector4: ...
    @property
    def ozoo(self) -> U64Vector4: ...
    @property
    def ozol(self) -> U64Vector4: ...
    @property
    def ozlx(self) -> U64Vector4: ...
    @property
    def ozly(self) -> U64Vector4: ...
    @property
    def ozlz(self) -> U64Vector4: ...
    @property
    def ozlo(self) -> U64Vector4: ...
    @property
    def ozll(self) -> U64Vector4: ...
    @property
    def ooxx(self) -> U64Vector4: ...
    @property
    def ooxy(self) -> U64Vector4: ...
    @property
    def ooxz(self) -> U64Vector4: ...
    @property
    def ooxo(self) -> U64Vector4: ...
    @property
    def ooxl(self) -> U64Vector4: ...
    @property
    def ooyx(self) -> U64Vector4: ...
    @property
    def ooyy(self) -> U64Vector4: ...
    @property
    def ooyz(self) -> U64Vector4: ...
    @property
    def ooyo(self) -> U64Vector4: ...
    @property
    def ooyl(self) -> U64Vector4: ...
    @property
    def oozx(self) -> U64Vector4: ...
    @property
    def oozy(self) -> U64Vector4: ...
    @property
    def oozz(self) -> U64Vector4: ...
    @property
    def oozo(self) -> U64Vector4: ...
    @property
    def oozl(self) -> U64Vector4: ...
    @property
    def ooox(self) -> U64Vector4: ...
    @property
    def oooy(self) -> U64Vector4: ...
    @property
    def oooz(self) -> U64Vector4: ...
    @property
    def oolx(self) -> U64Vector4: ...
    @property
    def ooly(self) -> U64Vector4: ...
    @property
    def oolz(self) -> U64Vector4: ...
    @property
    def olxx(self) -> U64Vector4: ...
    @property
    def olxy(self) -> U64Vector4: ...
    @property
    def olxz(self) -> U64Vector4: ...
    @property
    def olxo(self) -> U64Vector4: ...
    @property
    def olxl(self) -> U64Vector4: ...
    @property
    def olyx(self) -> U64Vector4: ...
    @property
    def olyy(self) -> U64Vector4: ...
    @property
    def olyz(self) -> U64Vector4: ...
    @property
    def olyo(self) -> U64Vector4: ...
    @property
    def olyl(self) -> U64Vector4: ...
    @property
    def olzx(self) -> U64Vector4: ...
    @property
    def olzy(self) -> U64Vector4: ...
    @property
    def olzz(self) -> U64Vector4: ...
    @property
    def olzo(self) -> U64Vector4: ...
    @property
    def olzl(self) -> U64Vector4: ...
    @property
    def olox(self) -> U64Vector4: ...
    @property
    def oloy(self) -> U64Vector4: ...
    @property
    def oloz(self) -> U64Vector4: ...
    @property
    def ollx(self) -> U64Vector4: ...
    @property
    def olly(self) -> U64Vector4: ...
    @property
    def ollz(self) -> U64Vector4: ...
    @property
    def lxxx(self) -> U64Vector4: ...
    @property
    def lxxy(self) -> U64Vector4: ...
    @property
    def lxxz(self) -> U64Vector4: ...
    @property
    def lxxo(self) -> U64Vector4: ...
    @property
    def lxxl(self) -> U64Vector4: ...
    @property
    def lxyx(self) -> U64Vector4: ...
    @property
    def lxyy(self) -> U64Vector4: ...
    @property
    def lxyz(self) -> U64Vector4: ...
    @property
    def lxyo(self) -> U64Vector4: ...
    @property
    def lxyl(self) -> U64Vector4: ...
    @property
    def lxzx(self) -> U64Vector4: ...
    @property
    def lxzy(self) -> U64Vector4: ...
    @property
    def lxzz(self) -> U64Vector4: ...
    @property
    def lxzo(self) -> U64Vector4: ...
    @property
    def lxzl(self) -> U64Vector4: ...
    @property
    def lxox(self) -> U64Vector4: ...
    @property
    def lxoy(self) -> U64Vector4: ...
    @property
    def lxoz(self) -> U64Vector4: ...
    @property
    def lxoo(self) -> U64Vector4: ...
    @property
    def lxol(self) -> U64Vector4: ...
    @property
    def lxlx(self) -> U64Vector4: ...
    @property
    def lxly(self) -> U64Vector4: ...
    @property
    def lxlz(self) -> U64Vector4: ...
    @property
    def lxlo(self) -> U64Vector4: ...
    @property
    def lxll(self) -> U64Vector4: ...
    @property
    def lyxx(self) -> U64Vector4: ...
    @property
    def lyxy(self) -> U64Vector4: ...
    @property
    def lyxz(self) -> U64Vector4: ...
    @property
    def lyxo(self) -> U64Vector4: ...
    @property
    def lyxl(self) -> U64Vector4: ...
    @property
    def lyyx(self) -> U64Vector4: ...
    @property
    def lyyy(self) -> U64Vector4: ...
    @property
    def lyyz(self) -> U64Vector4: ...
    @property
    def lyyo(self) -> U64Vector4: ...
    @property
    def lyyl(self) -> U64Vector4: ...
    @property
    def lyzx(self) -> U64Vector4: ...
    @property
    def lyzy(self) -> U64Vector4: ...
    @property
    def lyzz(self) -> U64Vector4: ...
    @property
    def lyzo(self) -> U64Vector4: ...
    @property
    def lyzl(self) -> U64Vector4: ...
    @property
    def lyox(self) -> U64Vector4: ...
    @property
    def lyoy(self) -> U64Vector4: ...
    @property
    def lyoz(self) -> U64Vector4: ...
    @property
    def lyoo(self) -> U64Vector4: ...
    @property
    def lyol(self) -> U64Vector4: ...
    @property
    def lylx(self) -> U64Vector4: ...
    @property
    def lyly(self) -> U64Vector4: ...
    @property
    def lylz(self) -> U64Vector4: ...
    @property
    def lylo(self) -> U64Vector4: ...
    @property
    def lyll(self) -> U64Vector4: ...
    @property
    def lzxx(self) -> U64Vector4: ...
    @property
    def lzxy(self) -> U64Vector4: ...
    @property
    def lzxz(self) -> U64Vector4: ...
    @property
    def lzxo(self) -> U64Vector4: ...
    @property
    def lzxl(self) -> U64Vector4: ...
    @property
    def lzyx(self) -> U64Vector4: ...
    @property
    def lzyy(self) -> U64Vector4: ...
    @property
    def lzyz(self) -> U64Vector4: ...
    @property
    def lzyo(self) -> U64Vector4: ...
    @property
    def lzyl(self) -> U64Vector4: ...
    @property
    def lzzx(self) -> U64Vector4: ...
    @property
    def lzzy(self) -> U64Vector4: ...
    @property
    def lzzz(self) -> U64Vector4: ...
    @property
    def lzzo(self) -> U64Vector4: ...
    @property
    def lzzl(self) -> U64Vector4: ...
    @property
    def lzox(self) -> U64Vector4: ...
    @property
    def lzoy(self) -> U64Vector4: ...
    @property
    def lzoz(self) -> U64Vector4: ...
    @property
    def lzoo(self) -> U64Vector4: ...
    @property
    def lzol(self) -> U64Vector4: ...
    @property
    def lzlx(self) -> U64Vector4: ...
    @property
    def lzly(self) -> U64Vector4: ...
    @property
    def lzlz(self) -> U64Vector4: ...
    @property
    def lzlo(self) -> U64Vector4: ...
    @property
    def lzll(self) -> U64Vector4: ...
    @property
    def loxx(self) -> U64Vector4: ...
    @property
    def loxy(self) -> U64Vector4: ...
    @property
    def loxz(self) -> U64Vector4: ...
    @property
    def loxo(self) -> U64Vector4: ...
    @property
    def loxl(self) -> U64Vector4: ...
    @property
    def loyx(self) -> U64Vector4: ...
    @property
    def loyy(self) -> U64Vector4: ...
    @property
    def loyz(self) -> U64Vector4: ...
    @property
    def loyo(self) -> U64Vector4: ...
    @property
    def loyl(self) -> U64Vector4: ...
    @property
    def lozx(self) -> U64Vector4: ...
    @property
    def lozy(self) -> U64Vector4: ...
    @property
    def lozz(self) -> U64Vector4: ...
    @property
    def lozo(self) -> U64Vector4: ...
    @property
    def lozl(self) -> U64Vector4: ...
    @property
    def loox(self) -> U64Vector4: ...
    @property
    def looy(self) -> U64Vector4: ...
    @property
    def looz(self) -> U64Vector4: ...
    @property
    def lolx(self) -> U64Vector4: ...
    @property
    def loly(self) -> U64Vector4: ...
    @property
    def lolz(self) -> U64Vector4: ...
    @property
    def llxx(self) -> U64Vector4: ...
    @property
    def llxy(self) -> U64Vector4: ...
    @property
    def llxz(self) -> U64Vector4: ...
    @property
    def llxo(self) -> U64Vector4: ...
    @property
    def llxl(self) -> U64Vector4: ...
    @property
    def llyx(self) -> U64Vector4: ...
    @property
    def llyy(self) -> U64Vector4: ...
    @property
    def llyz(self) -> U64Vector4: ...
    @property
    def llyo(self) -> U64Vector4: ...
    @property
    def llyl(self) -> U64Vector4: ...
    @property
    def llzx(self) -> U64Vector4: ...
    @property
    def llzy(self) -> U64Vector4: ...
    @property
    def llzz(self) -> U64Vector4: ...
    @property
    def llzo(self) -> U64Vector4: ...
    @property
    def llzl(self) -> U64Vector4: ...
    @property
    def llox(self) -> U64Vector4: ...
    @property
    def lloy(self) -> U64Vector4: ...
    @property
    def lloz(self) -> U64Vector4: ...
    @property
    def lllx(self) -> U64Vector4: ...
    @property
    def llly(self) -> U64Vector4: ...
    @property
    def lllz(self) -> U64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def rr(self) -> U64Vector2: ...
    @property
    def rg(self) -> U64Vector2: ...
    @property
    def rb(self) -> U64Vector2: ...
    @property
    def ro(self) -> U64Vector2: ...
    @property
    def rl(self) -> U64Vector2: ...
    @property
    def gr(self) -> U64Vector2: ...
    @property
    def gg(self) -> U64Vector2: ...
    @property
    def gb(self) -> U64Vector2: ...
    @property
    def go(self) -> U64Vector2: ...
    @property
    def gl(self) -> U64Vector2: ...
    @property
    def br(self) -> U64Vector2: ...
    @property
    def bg(self) -> U64Vector2: ...
    @property
    def bb(self) -> U64Vector2: ...
    @property
    def bo(self) -> U64Vector2: ...
    @property
    def bl(self) -> U64Vector2: ...
    @property
    def og(self) -> U64Vector2: ...
    @property
    def ob(self) -> U64Vector2: ...
    @property
    def lr(self) -> U64Vector2: ...
    @property
    def lg(self) -> U64Vector2: ...
    @property
    def lb(self) -> U64Vector2: ...
    @property
    def rrr(self) -> U64Vector3: ...
    @property
    def rrg(self) -> U64Vector3: ...
    @property
    def rrb(self) -> U64Vector3: ...
    @property
    def rro(self) -> U64Vector3: ...
    @property
    def rrl(self) -> U64Vector3: ...
    @property
    def rgr(self) -> U64Vector3: ...
    @property
    def rgg(self) -> U64Vector3: ...
    @property
    def rgb(self) -> U64Vector3: ...
    @property
    def rgo(self) -> U64Vector3: ...
    @property
    def rgl(self) -> U64Vector3: ...
    @property
    def rbr(self) -> U64Vector3: ...
    @property
    def rbg(self) -> U64Vector3: ...
    @property
    def rbb(self) -> U64Vector3: ...
    @property
    def rbo(self) -> U64Vector3: ...
    @property
    def rbl(self) -> U64Vector3: ...
    @property
    def ror(self) -> U64Vector3: ...
    @property
    def rog(self) -> U64Vector3: ...
    @property
    def rob(self) -> U64Vector3: ...
    @property
    def roo(self) -> U64Vector3: ...
    @property
    def rol(self) -> U64Vector3: ...
    @property
    def rlr(self) -> U64Vector3: ...
    @property
    def rlg(self) -> U64Vector3: ...
    @property
    def rlb(self) -> U64Vector3: ...
    @property
    def rlo(self) -> U64Vector3: ...
    @property
    def rll(self) -> U64Vector3: ...
    @property
    def grr(self) -> U64Vector3: ...
    @property
    def grg(self) -> U64Vector3: ...
    @property
    def grb(self) -> U64Vector3: ...
    @property
    def gro(self) -> U64Vector3: ...
    @property
    def grl(self) -> U64Vector3: ...
    @property
    def ggr(self) -> U64Vector3: ...
    @property
    def ggg(self) -> U64Vector3: ...
    @property
    def ggb(self) -> U64Vector3: ...
    @property
    def ggo(self) -> U64Vector3: ...
    @property
    def ggl(self) -> U64Vector3: ...
    @property
    def gbr(self) -> U64Vector3: ...
    @property
    def gbg(self) -> U64Vector3: ...
    @property
    def gbb(self) -> U64Vector3: ...
    @property
    def gbo(self) -> U64Vector3: ...
    @property
    def gbl(self) -> U64Vector3: ...
    @property
    def gor(self) -> U64Vector3: ...
    @property
    def gog(self) -> U64Vector3: ...
    @property
    def gob(self) -> U64Vector3: ...
    @property
    def goo(self) -> U64Vector3: ...
    @property
    def gol(self) -> U64Vector3: ...
    @property
    def glr(self) -> U64Vector3: ...
    @property
    def glg(self) -> U64Vector3: ...
    @property
    def glb(self) -> U64Vector3: ...
    @property
    def glo(self) -> U64Vector3: ...
    @property
    def gll(self) -> U64Vector3: ...
    @property
    def brr(self) -> U64Vector3: ...
    @property
    def brg(self) -> U64Vector3: ...
    @property
    def brb(self) -> U64Vector3: ...
    @property
    def bro(self) -> U64Vector3: ...
    @property
    def brl(self) -> U64Vector3: ...
    @property
    def bgr(self) -> U64Vector3: ...
    @property
    def bgg(self) -> U64Vector3: ...
    @property
    def bgb(self) -> U64Vector3: ...
    @property
    def bgo(self) -> U64Vector3: ...
    @property
    def bgl(self) -> U64Vector3: ...
    @property
    def bbr(self) -> U64Vector3: ...
    @property
    def bbg(self) -> U64Vector3: ...
    @property
    def bbb(self) -> U64Vector3: ...
    @property
    def bbo(self) -> U64Vector3: ...
    @property
    def bbl(self) -> U64Vector3: ...
    @property
    def bor(self) -> U64Vector3: ...
    @property
    def bog(self) -> U64Vector3: ...
    @property
    def bob(self) -> U64Vector3: ...
    @property
    def boo(self) -> U64Vector3: ...
    @property
    def bol(self) -> U64Vector3: ...
    @property
    def blr(self) -> U64Vector3: ...
    @property
    def blg(self) -> U64Vector3: ...
    @property
    def blb(self) -> U64Vector3: ...
    @property
    def blo(self) -> U64Vector3: ...
    @property
    def bll(self) -> U64Vector3: ...
    @property
    def orr(self) -> U64Vector3: ...
    @property
    def org(self) -> U64Vector3: ...
    @property
    def orb(self) -> U64Vector3: ...
    @property
    def oro(self) -> U64Vector3: ...
    @property
    def orl(self) -> U64Vector3: ...
    @property
    def ogr(self) -> U64Vector3: ...
    @property
    def ogg(self) -> U64Vector3: ...
    @property
    def ogb(self) -> U64Vector3: ...
    @property
    def ogo(self) -> U64Vector3: ...
    @property
    def ogl(self) -> U64Vector3: ...
    @property
    def obr(self) -> U64Vector3: ...
    @property
    def obg(self) -> U64Vector3: ...
    @property
    def obb(self) -> U64Vector3: ...
    @property
    def obo(self) -> U64Vector3: ...
    @property
    def obl(self) -> U64Vector3: ...
    @property
    def oor(self) -> U64Vector3: ...
    @property
    def oog(self) -> U64Vector3: ...
    @property
    def oob(self) -> U64Vector3: ...
    @property
    def olr(self) -> U64Vector3: ...
    @property
    def olg(self) -> U64Vector3: ...
    @property
    def olb(self) -> U64Vector3: ...
    @property
    def lrr(self) -> U64Vector3: ...
    @property
    def lrg(self) -> U64Vector3: ...
    @property
    def lrb(self) -> U64Vector3: ...
    @property
    def lro(self) -> U64Vector3: ...
    @property
    def lrl(self) -> U64Vector3: ...
    @property
    def lgr(self) -> U64Vector3: ...
    @property
    def lgg(self) -> U64Vector3: ...
    @property
    def lgb(self) -> U64Vector3: ...
    @property
    def lgo(self) -> U64Vector3: ...
    @property
    def lgl(self) -> U64Vector3: ...
    @property
    def lbr(self) -> U64Vector3: ...
    @property
    def lbg(self) -> U64Vector3: ...
    @property
    def lbb(self) -> U64Vector3: ...
    @property
    def lbo(self) -> U64Vector3: ...
    @property
    def lbl(self) -> U64Vector3: ...
    @property
    def lor(self) -> U64Vector3: ...
    @property
    def log(self) -> U64Vector3: ...
    @property
    def lob(self) -> U64Vector3: ...
    @property
    def llr(self) -> U64Vector3: ...
    @property
    def llg(self) -> U64Vector3: ...
    @property
    def llb(self) -> U64Vector3: ...
    @property
    def rrrr(self) -> U64Vector4: ...
    @property
    def rrrg(self) -> U64Vector4: ...
    @property
    def rrrb(self) -> U64Vector4: ...
    @property
    def rrro(self) -> U64Vector4: ...
    @property
    def rrrl(self) -> U64Vector4: ...
    @property
    def rrgr(self) -> U64Vector4: ...
    @property
    def rrgg(self) -> U64Vector4: ...
    @property
    def rrgb(self) -> U64Vector4: ...
    @property
    def rrgo(self) -> U64Vector4: ...
    @property
    def rrgl(self) -> U64Vector4: ...
    @property
    def rrbr(self) -> U64Vector4: ...
    @property
    def rrbg(self) -> U64Vector4: ...
    @property
    def rrbb(self) -> U64Vector4: ...
    @property
    def rrbo(self) -> U64Vector4: ...
    @property
    def rrbl(self) -> U64Vector4: ...
    @property
    def rror(self) -> U64Vector4: ...
    @property
    def rrog(self) -> U64Vector4: ...
    @property
    def rrob(self) -> U64Vector4: ...
    @property
    def rroo(self) -> U64Vector4: ...
    @property
    def rrol(self) -> U64Vector4: ...
    @property
    def rrlr(self) -> U64Vector4: ...
    @property
    def rrlg(self) -> U64Vector4: ...
    @property
    def rrlb(self) -> U64Vector4: ...
    @property
    def rrlo(self) -> U64Vector4: ...
    @property
    def rrll(self) -> U64Vector4: ...
    @property
    def rgrr(self) -> U64Vector4: ...
    @property
    def rgrg(self) -> U64Vector4: ...
    @property
    def rgrb(self) -> U64Vector4: ...
    @property
    def rgro(self) -> U64Vector4: ...
    @property
    def rgrl(self) -> U64Vector4: ...
    @property
    def rggr(self) -> U64Vector4: ...
    @property
    def rggg(self) -> U64Vector4: ...
    @property
    def rggb(self) -> U64Vector4: ...
    @property
    def rggo(self) -> U64Vector4: ...
    @property
    def rggl(self) -> U64Vector4: ...
    @property
    def rgbr(self) -> U64Vector4: ...
    @property
    def rgbg(self) -> U64Vector4: ...
    @property
    def rgbb(self) -> U64Vector4: ...
    @property
    def rgbo(self) -> U64Vector4: ...
    @property
    def rgbl(self) -> U64Vector4: ...
    @property
    def rgor(self) -> U64Vector4: ...
    @property
    def rgog(self) -> U64Vector4: ...
    @property
    def rgob(self) -> U64Vector4: ...
    @property
    def rgoo(self) -> U64Vector4: ...
    @property
    def rgol(self) -> U64Vector4: ...
    @property
    def rglr(self) -> U64Vector4: ...
    @property
    def rglg(self) -> U64Vector4: ...
    @property
    def rglb(self) -> U64Vector4: ...
    @property
    def rglo(self) -> U64Vector4: ...
    @property
    def rgll(self) -> U64Vector4: ...
    @property
    def rbrr(self) -> U64Vector4: ...
    @property
    def rbrg(self) -> U64Vector4: ...
    @property
    def rbrb(self) -> U64Vector4: ...
    @property
    def rbro(self) -> U64Vector4: ...
    @property
    def rbrl(self) -> U64Vector4: ...
    @property
    def rbgr(self) -> U64Vector4: ...
    @property
    def rbgg(self) -> U64Vector4: ...
    @property
    def rbgb(self) -> U64Vector4: ...
    @property
    def rbgo(self) -> U64Vector4: ...
    @property
    def rbgl(self) -> U64Vector4: ...
    @property
    def rbbr(self) -> U64Vector4: ...
    @property
    def rbbg(self) -> U64Vector4: ...
    @property
    def rbbb(self) -> U64Vector4: ...
    @property
    def rbbo(self) -> U64Vector4: ...
    @property
    def rbbl(self) -> U64Vector4: ...
    @property
    def rbor(self) -> U64Vector4: ...
    @property
    def rbog(self) -> U64Vector4: ...
    @property
    def rbob(self) -> U64Vector4: ...
    @property
    def rboo(self) -> U64Vector4: ...
    @property
    def rbol(self) -> U64Vector4: ...
    @property
    def rblr(self) -> U64Vector4: ...
    @property
    def rblg(self) -> U64Vector4: ...
    @property
    def rblb(self) -> U64Vector4: ...
    @property
    def rblo(self) -> U64Vector4: ...
    @property
    def rbll(self) -> U64Vector4: ...
    @property
    def rorr(self) -> U64Vector4: ...
    @property
    def rorg(self) -> U64Vector4: ...
    @property
    def rorb(self) -> U64Vector4: ...
    @property
    def roro(self) -> U64Vector4: ...
    @property
    def rorl(self) -> U64Vector4: ...
    @property
    def rogr(self) -> U64Vector4: ...
    @property
    def rogg(self) -> U64Vector4: ...
    @property
    def rogb(self) -> U64Vector4: ...
    @property
    def rogo(self) -> U64Vector4: ...
    @property
    def rogl(self) -> U64Vector4: ...
    @property
    def robr(self) -> U64Vector4: ...
    @property
    def robg(self) -> U64Vector4: ...
    @property
    def robb(self) -> U64Vector4: ...
    @property
    def robo(self) -> U64Vector4: ...
    @property
    def robl(self) -> U64Vector4: ...
    @property
    def roor(self) -> U64Vector4: ...
    @property
    def roog(self) -> U64Vector4: ...
    @property
    def roob(self) -> U64Vector4: ...
    @property
    def rooo(self) -> U64Vector4: ...
    @property
    def rool(self) -> U64Vector4: ...
    @property
    def rolr(self) -> U64Vector4: ...
    @property
    def rolg(self) -> U64Vector4: ...
    @property
    def rolb(self) -> U64Vector4: ...
    @property
    def rolo(self) -> U64Vector4: ...
    @property
    def roll(self) -> U64Vector4: ...
    @property
    def rlrr(self) -> U64Vector4: ...
    @property
    def rlrg(self) -> U64Vector4: ...
    @property
    def rlrb(self) -> U64Vector4: ...
    @property
    def rlro(self) -> U64Vector4: ...
    @property
    def rlrl(self) -> U64Vector4: ...
    @property
    def rlgr(self) -> U64Vector4: ...
    @property
    def rlgg(self) -> U64Vector4: ...
    @property
    def rlgb(self) -> U64Vector4: ...
    @property
    def rlgo(self) -> U64Vector4: ...
    @property
    def rlgl(self) -> U64Vector4: ...
    @property
    def rlbr(self) -> U64Vector4: ...
    @property
    def rlbg(self) -> U64Vector4: ...
    @property
    def rlbb(self) -> U64Vector4: ...
    @property
    def rlbo(self) -> U64Vector4: ...
    @property
    def rlbl(self) -> U64Vector4: ...
    @property
    def rlor(self) -> U64Vector4: ...
    @property
    def rlog(self) -> U64Vector4: ...
    @property
    def rlob(self) -> U64Vector4: ...
    @property
    def rloo(self) -> U64Vector4: ...
    @property
    def rlol(self) -> U64Vector4: ...
    @property
    def rllr(self) -> U64Vector4: ...
    @property
    def rllg(self) -> U64Vector4: ...
    @property
    def rllb(self) -> U64Vector4: ...
    @property
    def rllo(self) -> U64Vector4: ...
    @property
    def rlll(self) -> U64Vector4: ...
    @property
    def grrr(self) -> U64Vector4: ...
    @property
    def grrg(self) -> U64Vector4: ...
    @property
    def grrb(self) -> U64Vector4: ...
    @property
    def grro(self) -> U64Vector4: ...
    @property
    def grrl(self) -> U64Vector4: ...
    @property
    def grgr(self) -> U64Vector4: ...
    @property
    def grgg(self) -> U64Vector4: ...
    @property
    def grgb(self) -> U64Vector4: ...
    @property
    def grgo(self) -> U64Vector4: ...
    @property
    def grgl(self) -> U64Vector4: ...
    @property
    def grbr(self) -> U64Vector4: ...
    @property
    def grbg(self) -> U64Vector4: ...
    @property
    def grbb(self) -> U64Vector4: ...
    @property
    def grbo(self) -> U64Vector4: ...
    @property
    def grbl(self) -> U64Vector4: ...
    @property
    def gror(self) -> U64Vector4: ...
    @property
    def grog(self) -> U64Vector4: ...
    @property
    def grob(self) -> U64Vector4: ...
    @property
    def groo(self) -> U64Vector4: ...
    @property
    def grol(self) -> U64Vector4: ...
    @property
    def grlr(self) -> U64Vector4: ...
    @property
    def grlg(self) -> U64Vector4: ...
    @property
    def grlb(self) -> U64Vector4: ...
    @property
    def grlo(self) -> U64Vector4: ...
    @property
    def grll(self) -> U64Vector4: ...
    @property
    def ggrr(self) -> U64Vector4: ...
    @property
    def ggrg(self) -> U64Vector4: ...
    @property
    def ggrb(self) -> U64Vector4: ...
    @property
    def ggro(self) -> U64Vector4: ...
    @property
    def ggrl(self) -> U64Vector4: ...
    @property
    def gggr(self) -> U64Vector4: ...
    @property
    def gggg(self) -> U64Vector4: ...
    @property
    def gggb(self) -> U64Vector4: ...
    @property
    def gggo(self) -> U64Vector4: ...
    @property
    def gggl(self) -> U64Vector4: ...
    @property
    def ggbr(self) -> U64Vector4: ...
    @property
    def ggbg(self) -> U64Vector4: ...
    @property
    def ggbb(self) -> U64Vector4: ...
    @property
    def ggbo(self) -> U64Vector4: ...
    @property
    def ggbl(self) -> U64Vector4: ...
    @property
    def ggor(self) -> U64Vector4: ...
    @property
    def ggog(self) -> U64Vector4: ...
    @property
    def ggob(self) -> U64Vector4: ...
    @property
    def ggoo(self) -> U64Vector4: ...
    @property
    def ggol(self) -> U64Vector4: ...
    @property
    def gglr(self) -> U64Vector4: ...
    @property
    def gglg(self) -> U64Vector4: ...
    @property
    def gglb(self) -> U64Vector4: ...
    @property
    def gglo(self) -> U64Vector4: ...
    @property
    def ggll(self) -> U64Vector4: ...
    @property
    def gbrr(self) -> U64Vector4: ...
    @property
    def gbrg(self) -> U64Vector4: ...
    @property
    def gbrb(self) -> U64Vector4: ...
    @property
    def gbro(self) -> U64Vector4: ...
    @property
    def gbrl(self) -> U64Vector4: ...
    @property
    def gbgr(self) -> U64Vector4: ...
    @property
    def gbgg(self) -> U64Vector4: ...
    @property
    def gbgb(self) -> U64Vector4: ...
    @property
    def gbgo(self) -> U64Vector4: ...
    @property
    def gbgl(self) -> U64Vector4: ...
    @property
    def gbbr(self) -> U64Vector4: ...
    @property
    def gbbg(self) -> U64Vector4: ...
    @property
    def gbbb(self) -> U64Vector4: ...
    @property
    def gbbo(self) -> U64Vector4: ...
    @property
    def gbbl(self) -> U64Vector4: ...
    @property
    def gbor(self) -> U64Vector4: ...
    @property
    def gbog(self) -> U64Vector4: ...
    @property
    def gbob(self) -> U64Vector4: ...
    @property
    def gboo(self) -> U64Vector4: ...
    @property
    def gbol(self) -> U64Vector4: ...
    @property
    def gblr(self) -> U64Vector4: ...
    @property
    def gblg(self) -> U64Vector4: ...
    @property
    def gblb(self) -> U64Vector4: ...
    @property
    def gblo(self) -> U64Vector4: ...
    @property
    def gbll(self) -> U64Vector4: ...
    @property
    def gorr(self) -> U64Vector4: ...
    @property
    def gorg(self) -> U64Vector4: ...
    @property
    def gorb(self) -> U64Vector4: ...
    @property
    def goro(self) -> U64Vector4: ...
    @property
    def gorl(self) -> U64Vector4: ...
    @property
    def gogr(self) -> U64Vector4: ...
    @property
    def gogg(self) -> U64Vector4: ...
    @property
    def gogb(self) -> U64Vector4: ...
    @property
    def gogo(self) -> U64Vector4: ...
    @property
    def gogl(self) -> U64Vector4: ...
    @property
    def gobr(self) -> U64Vector4: ...
    @property
    def gobg(self) -> U64Vector4: ...
    @property
    def gobb(self) -> U64Vector4: ...
    @property
    def gobo(self) -> U64Vector4: ...
    @property
    def gobl(self) -> U64Vector4: ...
    @property
    def goor(self) -> U64Vector4: ...
    @property
    def goog(self) -> U64Vector4: ...
    @property
    def goob(self) -> U64Vector4: ...
    @property
    def gooo(self) -> U64Vector4: ...
    @property
    def gool(self) -> U64Vector4: ...
    @property
    def golr(self) -> U64Vector4: ...
    @property
    def golg(self) -> U64Vector4: ...
    @property
    def golb(self) -> U64Vector4: ...
    @property
    def golo(self) -> U64Vector4: ...
    @property
    def goll(self) -> U64Vector4: ...
    @property
    def glrr(self) -> U64Vector4: ...
    @property
    def glrg(self) -> U64Vector4: ...
    @property
    def glrb(self) -> U64Vector4: ...
    @property
    def glro(self) -> U64Vector4: ...
    @property
    def glrl(self) -> U64Vector4: ...
    @property
    def glgr(self) -> U64Vector4: ...
    @property
    def glgg(self) -> U64Vector4: ...
    @property
    def glgb(self) -> U64Vector4: ...
    @property
    def glgo(self) -> U64Vector4: ...
    @property
    def glgl(self) -> U64Vector4: ...
    @property
    def glbr(self) -> U64Vector4: ...
    @property
    def glbg(self) -> U64Vector4: ...
    @property
    def glbb(self) -> U64Vector4: ...
    @property
    def glbo(self) -> U64Vector4: ...
    @property
    def glbl(self) -> U64Vector4: ...
    @property
    def glor(self) -> U64Vector4: ...
    @property
    def glog(self) -> U64Vector4: ...
    @property
    def glob(self) -> U64Vector4: ...
    @property
    def gloo(self) -> U64Vector4: ...
    @property
    def glol(self) -> U64Vector4: ...
    @property
    def gllr(self) -> U64Vector4: ...
    @property
    def gllg(self) -> U64Vector4: ...
    @property
    def gllb(self) -> U64Vector4: ...
    @property
    def gllo(self) -> U64Vector4: ...
    @property
    def glll(self) -> U64Vector4: ...
    @property
    def brrr(self) -> U64Vector4: ...
    @property
    def brrg(self) -> U64Vector4: ...
    @property
    def brrb(self) -> U64Vector4: ...
    @property
    def brro(self) -> U64Vector4: ...
    @property
    def brrl(self) -> U64Vector4: ...
    @property
    def brgr(self) -> U64Vector4: ...
    @property
    def brgg(self) -> U64Vector4: ...
    @property
    def brgb(self) -> U64Vector4: ...
    @property
    def brgo(self) -> U64Vector4: ...
    @property
    def brgl(self) -> U64Vector4: ...
    @property
    def brbr(self) -> U64Vector4: ...
    @property
    def brbg(self) -> U64Vector4: ...
    @property
    def brbb(self) -> U64Vector4: ...
    @property
    def brbo(self) -> U64Vector4: ...
    @property
    def brbl(self) -> U64Vector4: ...
    @property
    def bror(self) -> U64Vector4: ...
    @property
    def brog(self) -> U64Vector4: ...
    @property
    def brob(self) -> U64Vector4: ...
    @property
    def broo(self) -> U64Vector4: ...
    @property
    def brol(self) -> U64Vector4: ...
    @property
    def brlr(self) -> U64Vector4: ...
    @property
    def brlg(self) -> U64Vector4: ...
    @property
    def brlb(self) -> U64Vector4: ...
    @property
    def brlo(self) -> U64Vector4: ...
    @property
    def brll(self) -> U64Vector4: ...
    @property
    def bgrr(self) -> U64Vector4: ...
    @property
    def bgrg(self) -> U64Vector4: ...
    @property
    def bgrb(self) -> U64Vector4: ...
    @property
    def bgro(self) -> U64Vector4: ...
    @property
    def bgrl(self) -> U64Vector4: ...
    @property
    def bggr(self) -> U64Vector4: ...
    @property
    def bggg(self) -> U64Vector4: ...
    @property
    def bggb(self) -> U64Vector4: ...
    @property
    def bggo(self) -> U64Vector4: ...
    @property
    def bggl(self) -> U64Vector4: ...
    @property
    def bgbr(self) -> U64Vector4: ...
    @property
    def bgbg(self) -> U64Vector4: ...
    @property
    def bgbb(self) -> U64Vector4: ...
    @property
    def bgbo(self) -> U64Vector4: ...
    @property
    def bgbl(self) -> U64Vector4: ...
    @property
    def bgor(self) -> U64Vector4: ...
    @property
    def bgog(self) -> U64Vector4: ...
    @property
    def bgob(self) -> U64Vector4: ...
    @property
    def bgoo(self) -> U64Vector4: ...
    @property
    def bgol(self) -> U64Vector4: ...
    @property
    def bglr(self) -> U64Vector4: ...
    @property
    def bglg(self) -> U64Vector4: ...
    @property
    def bglb(self) -> U64Vector4: ...
    @property
    def bglo(self) -> U64Vector4: ...
    @property
    def bgll(self) -> U64Vector4: ...
    @property
    def bbrr(self) -> U64Vector4: ...
    @property
    def bbrg(self) -> U64Vector4: ...
    @property
    def bbrb(self) -> U64Vector4: ...
    @property
    def bbro(self) -> U64Vector4: ...
    @property
    def bbrl(self) -> U64Vector4: ...
    @property
    def bbgr(self) -> U64Vector4: ...
    @property
    def bbgg(self) -> U64Vector4: ...
    @property
    def bbgb(self) -> U64Vector4: ...
    @property
    def bbgo(self) -> U64Vector4: ...
    @property
    def bbgl(self) -> U64Vector4: ...
    @property
    def bbbr(self) -> U64Vector4: ...
    @property
    def bbbg(self) -> U64Vector4: ...
    @property
    def bbbb(self) -> U64Vector4: ...
    @property
    def bbbo(self) -> U64Vector4: ...
    @property
    def bbbl(self) -> U64Vector4: ...
    @property
    def bbor(self) -> U64Vector4: ...
    @property
    def bbog(self) -> U64Vector4: ...
    @property
    def bbob(self) -> U64Vector4: ...
    @property
    def bboo(self) -> U64Vector4: ...
    @property
    def bbol(self) -> U64Vector4: ...
    @property
    def bblr(self) -> U64Vector4: ...
    @property
    def bblg(self) -> U64Vector4: ...
    @property
    def bblb(self) -> U64Vector4: ...
    @property
    def bblo(self) -> U64Vector4: ...
    @property
    def bbll(self) -> U64Vector4: ...
    @property
    def borr(self) -> U64Vector4: ...
    @property
    def borg(self) -> U64Vector4: ...
    @property
    def borb(self) -> U64Vector4: ...
    @property
    def boro(self) -> U64Vector4: ...
    @property
    def borl(self) -> U64Vector4: ...
    @property
    def bogr(self) -> U64Vector4: ...
    @property
    def bogg(self) -> U64Vector4: ...
    @property
    def bogb(self) -> U64Vector4: ...
    @property
    def bogo(self) -> U64Vector4: ...
    @property
    def bogl(self) -> U64Vector4: ...
    @property
    def bobr(self) -> U64Vector4: ...
    @property
    def bobg(self) -> U64Vector4: ...
    @property
    def bobb(self) -> U64Vector4: ...
    @property
    def bobo(self) -> U64Vector4: ...
    @property
    def bobl(self) -> U64Vector4: ...
    @property
    def boor(self) -> U64Vector4: ...
    @property
    def boog(self) -> U64Vector4: ...
    @property
    def boob(self) -> U64Vector4: ...
    @property
    def booo(self) -> U64Vector4: ...
    @property
    def bool(self) -> U64Vector4: ...
    @property
    def bolr(self) -> U64Vector4: ...
    @property
    def bolg(self) -> U64Vector4: ...
    @property
    def bolb(self) -> U64Vector4: ...
    @property
    def bolo(self) -> U64Vector4: ...
    @property
    def boll(self) -> U64Vector4: ...
    @property
    def blrr(self) -> U64Vector4: ...
    @property
    def blrg(self) -> U64Vector4: ...
    @property
    def blrb(self) -> U64Vector4: ...
    @property
    def blro(self) -> U64Vector4: ...
    @property
    def blrl(self) -> U64Vector4: ...
    @property
    def blgr(self) -> U64Vector4: ...
    @property
    def blgg(self) -> U64Vector4: ...
    @property
    def blgb(self) -> U64Vector4: ...
    @property
    def blgo(self) -> U64Vector4: ...
    @property
    def blgl(self) -> U64Vector4: ...
    @property
    def blbr(self) -> U64Vector4: ...
    @property
    def blbg(self) -> U64Vector4: ...
    @property
    def blbb(self) -> U64Vector4: ...
    @property
    def blbo(self) -> U64Vector4: ...
    @property
    def blbl(self) -> U64Vector4: ...
    @property
    def blor(self) -> U64Vector4: ...
    @property
    def blog(self) -> U64Vector4: ...
    @property
    def blob(self) -> U64Vector4: ...
    @property
    def bloo(self) -> U64Vector4: ...
    @property
    def blol(self) -> U64Vector4: ...
    @property
    def bllr(self) -> U64Vector4: ...
    @property
    def bllg(self) -> U64Vector4: ...
    @property
    def bllb(self) -> U64Vector4: ...
    @property
    def bllo(self) -> U64Vector4: ...
    @property
    def blll(self) -> U64Vector4: ...
    @property
    def orrr(self) -> U64Vector4: ...
    @property
    def orrg(self) -> U64Vector4: ...
    @property
    def orrb(self) -> U64Vector4: ...
    @property
    def orro(self) -> U64Vector4: ...
    @property
    def orrl(self) -> U64Vector4: ...
    @property
    def orgr(self) -> U64Vector4: ...
    @property
    def orgg(self) -> U64Vector4: ...
    @property
    def orgb(self) -> U64Vector4: ...
    @property
    def orgo(self) -> U64Vector4: ...
    @property
    def orgl(self) -> U64Vector4: ...
    @property
    def orbr(self) -> U64Vector4: ...
    @property
    def orbg(self) -> U64Vector4: ...
    @property
    def orbb(self) -> U64Vector4: ...
    @property
    def orbo(self) -> U64Vector4: ...
    @property
    def orbl(self) -> U64Vector4: ...
    @property
    def oror(self) -> U64Vector4: ...
    @property
    def orog(self) -> U64Vector4: ...
    @property
    def orob(self) -> U64Vector4: ...
    @property
    def oroo(self) -> U64Vector4: ...
    @property
    def orol(self) -> U64Vector4: ...
    @property
    def orlr(self) -> U64Vector4: ...
    @property
    def orlg(self) -> U64Vector4: ...
    @property
    def orlb(self) -> U64Vector4: ...
    @property
    def orlo(self) -> U64Vector4: ...
    @property
    def orll(self) -> U64Vector4: ...
    @property
    def ogrr(self) -> U64Vector4: ...
    @property
    def ogrg(self) -> U64Vector4: ...
    @property
    def ogrb(self) -> U64Vector4: ...
    @property
    def ogro(self) -> U64Vector4: ...
    @property
    def ogrl(self) -> U64Vector4: ...
    @property
    def oggr(self) -> U64Vector4: ...
    @property
    def oggg(self) -> U64Vector4: ...
    @property
    def oggb(self) -> U64Vector4: ...
    @property
    def oggo(self) -> U64Vector4: ...
    @property
    def oggl(self) -> U64Vector4: ...
    @property
    def ogbr(self) -> U64Vector4: ...
    @property
    def ogbg(self) -> U64Vector4: ...
    @property
    def ogbb(self) -> U64Vector4: ...
    @property
    def ogbo(self) -> U64Vector4: ...
    @property
    def ogbl(self) -> U64Vector4: ...
    @property
    def ogor(self) -> U64Vector4: ...
    @property
    def ogog(self) -> U64Vector4: ...
    @property
    def ogob(self) -> U64Vector4: ...
    @property
    def ogoo(self) -> U64Vector4: ...
    @property
    def ogol(self) -> U64Vector4: ...
    @property
    def oglr(self) -> U64Vector4: ...
    @property
    def oglg(self) -> U64Vector4: ...
    @property
    def oglb(self) -> U64Vector4: ...
    @property
    def oglo(self) -> U64Vector4: ...
    @property
    def ogll(self) -> U64Vector4: ...
    @property
    def obrr(self) -> U64Vector4: ...
    @property
    def obrg(self) -> U64Vector4: ...
    @property
    def obrb(self) -> U64Vector4: ...
    @property
    def obro(self) -> U64Vector4: ...
    @property
    def obrl(self) -> U64Vector4: ...
    @property
    def obgr(self) -> U64Vector4: ...
    @property
    def obgg(self) -> U64Vector4: ...
    @property
    def obgb(self) -> U64Vector4: ...
    @property
    def obgo(self) -> U64Vector4: ...
    @property
    def obgl(self) -> U64Vector4: ...
    @property
    def obbr(self) -> U64Vector4: ...
    @property
    def obbg(self) -> U64Vector4: ...
    @property
    def obbb(self) -> U64Vector4: ...
    @property
    def obbo(self) -> U64Vector4: ...
    @property
    def obbl(self) -> U64Vector4: ...
    @property
    def obor(self) -> U64Vector4: ...
    @property
    def obog(self) -> U64Vector4: ...
    @property
    def obob(self) -> U64Vector4: ...
    @property
    def oboo(self) -> U64Vector4: ...
    @property
    def obol(self) -> U64Vector4: ...
    @property
    def oblr(self) -> U64Vector4: ...
    @property
    def oblg(self) -> U64Vector4: ...
    @property
    def oblb(self) -> U64Vector4: ...
    @property
    def oblo(self) -> U64Vector4: ...
    @property
    def obll(self) -> U64Vector4: ...
    @property
    def oorr(self) -> U64Vector4: ...
    @property
    def oorg(self) -> U64Vector4: ...
    @property
    def oorb(self) -> U64Vector4: ...
    @property
    def ooro(self) -> U64Vector4: ...
    @property
    def oorl(self) -> U64Vector4: ...
    @property
    def oogr(self) -> U64Vector4: ...
    @property
    def oogg(self) -> U64Vector4: ...
    @property
    def oogb(self) -> U64Vector4: ...
    @property
    def oogo(self) -> U64Vector4: ...
    @property
    def oogl(self) -> U64Vector4: ...
    @property
    def oobr(self) -> U64Vector4: ...
    @property
    def oobg(self) -> U64Vector4: ...
    @property
    def oobb(self) -> U64Vector4: ...
    @property
    def oobo(self) -> U64Vector4: ...
    @property
    def oobl(self) -> U64Vector4: ...
    @property
    def ooor(self) -> U64Vector4: ...
    @property
    def ooog(self) -> U64Vector4: ...
    @property
    def ooob(self) -> U64Vector4: ...
    @property
    def oolr(self) -> U64Vector4: ...
    @property
    def oolg(self) -> U64Vector4: ...
    @property
    def oolb(self) -> U64Vector4: ...
    @property
    def olrr(self) -> U64Vector4: ...
    @property
    def olrg(self) -> U64Vector4: ...
    @property
    def olrb(self) -> U64Vector4: ...
    @property
    def olro(self) -> U64Vector4: ...
    @property
    def olrl(self) -> U64Vector4: ...
    @property
    def olgr(self) -> U64Vector4: ...
    @property
    def olgg(self) -> U64Vector4: ...
    @property
    def olgb(self) -> U64Vector4: ...
    @property
    def olgo(self) -> U64Vector4: ...
    @property
    def olgl(self) -> U64Vector4: ...
    @property
    def olbr(self) -> U64Vector4: ...
    @property
    def olbg(self) -> U64Vector4: ...
    @property
    def olbb(self) -> U64Vector4: ...
    @property
    def olbo(self) -> U64Vector4: ...
    @property
    def olbl(self) -> U64Vector4: ...
    @property
    def olor(self) -> U64Vector4: ...
    @property
    def olog(self) -> U64Vector4: ...
    @property
    def olob(self) -> U64Vector4: ...
    @property
    def ollr(self) -> U64Vector4: ...
    @property
    def ollg(self) -> U64Vector4: ...
    @property
    def ollb(self) -> U64Vector4: ...
    @property
    def lrrr(self) -> U64Vector4: ...
    @property
    def lrrg(self) -> U64Vector4: ...
    @property
    def lrrb(self) -> U64Vector4: ...
    @property
    def lrro(self) -> U64Vector4: ...
    @property
    def lrrl(self) -> U64Vector4: ...
    @property
    def lrgr(self) -> U64Vector4: ...
    @property
    def lrgg(self) -> U64Vector4: ...
    @property
    def lrgb(self) -> U64Vector4: ...
    @property
    def lrgo(self) -> U64Vector4: ...
    @property
    def lrgl(self) -> U64Vector4: ...
    @property
    def lrbr(self) -> U64Vector4: ...
    @property
    def lrbg(self) -> U64Vector4: ...
    @property
    def lrbb(self) -> U64Vector4: ...
    @property
    def lrbo(self) -> U64Vector4: ...
    @property
    def lrbl(self) -> U64Vector4: ...
    @property
    def lror(self) -> U64Vector4: ...
    @property
    def lrog(self) -> U64Vector4: ...
    @property
    def lrob(self) -> U64Vector4: ...
    @property
    def lroo(self) -> U64Vector4: ...
    @property
    def lrol(self) -> U64Vector4: ...
    @property
    def lrlr(self) -> U64Vector4: ...
    @property
    def lrlg(self) -> U64Vector4: ...
    @property
    def lrlb(self) -> U64Vector4: ...
    @property
    def lrlo(self) -> U64Vector4: ...
    @property
    def lrll(self) -> U64Vector4: ...
    @property
    def lgrr(self) -> U64Vector4: ...
    @property
    def lgrg(self) -> U64Vector4: ...
    @property
    def lgrb(self) -> U64Vector4: ...
    @property
    def lgro(self) -> U64Vector4: ...
    @property
    def lgrl(self) -> U64Vector4: ...
    @property
    def lggr(self) -> U64Vector4: ...
    @property
    def lggg(self) -> U64Vector4: ...
    @property
    def lggb(self) -> U64Vector4: ...
    @property
    def lggo(self) -> U64Vector4: ...
    @property
    def lggl(self) -> U64Vector4: ...
    @property
    def lgbr(self) -> U64Vector4: ...
    @property
    def lgbg(self) -> U64Vector4: ...
    @property
    def lgbb(self) -> U64Vector4: ...
    @property
    def lgbo(self) -> U64Vector4: ...
    @property
    def lgbl(self) -> U64Vector4: ...
    @property
    def lgor(self) -> U64Vector4: ...
    @property
    def lgog(self) -> U64Vector4: ...
    @property
    def lgob(self) -> U64Vector4: ...
    @property
    def lgoo(self) -> U64Vector4: ...
    @property
    def lgol(self) -> U64Vector4: ...
    @property
    def lglr(self) -> U64Vector4: ...
    @property
    def lglg(self) -> U64Vector4: ...
    @property
    def lglb(self) -> U64Vector4: ...
    @property
    def lglo(self) -> U64Vector4: ...
    @property
    def lgll(self) -> U64Vector4: ...
    @property
    def lbrr(self) -> U64Vector4: ...
    @property
    def lbrg(self) -> U64Vector4: ...
    @property
    def lbrb(self) -> U64Vector4: ...
    @property
    def lbro(self) -> U64Vector4: ...
    @property
    def lbrl(self) -> U64Vector4: ...
    @property
    def lbgr(self) -> U64Vector4: ...
    @property
    def lbgg(self) -> U64Vector4: ...
    @property
    def lbgb(self) -> U64Vector4: ...
    @property
    def lbgo(self) -> U64Vector4: ...
    @property
    def lbgl(self) -> U64Vector4: ...
    @property
    def lbbr(self) -> U64Vector4: ...
    @property
    def lbbg(self) -> U64Vector4: ...
    @property
    def lbbb(self) -> U64Vector4: ...
    @property
    def lbbo(self) -> U64Vector4: ...
    @property
    def lbbl(self) -> U64Vector4: ...
    @property
    def lbor(self) -> U64Vector4: ...
    @property
    def lbog(self) -> U64Vector4: ...
    @property
    def lbob(self) -> U64Vector4: ...
    @property
    def lboo(self) -> U64Vector4: ...
    @property
    def lbol(self) -> U64Vector4: ...
    @property
    def lblr(self) -> U64Vector4: ...
    @property
    def lblg(self) -> U64Vector4: ...
    @property
    def lblb(self) -> U64Vector4: ...
    @property
    def lblo(self) -> U64Vector4: ...
    @property
    def lbll(self) -> U64Vector4: ...
    @property
    def lorr(self) -> U64Vector4: ...
    @property
    def lorg(self) -> U64Vector4: ...
    @property
    def lorb(self) -> U64Vector4: ...
    @property
    def loro(self) -> U64Vector4: ...
    @property
    def lorl(self) -> U64Vector4: ...
    @property
    def logr(self) -> U64Vector4: ...
    @property
    def logg(self) -> U64Vector4: ...
    @property
    def logb(self) -> U64Vector4: ...
    @property
    def logo(self) -> U64Vector4: ...
    @property
    def logl(self) -> U64Vector4: ...
    @property
    def lobr(self) -> U64Vector4: ...
    @property
    def lobg(self) -> U64Vector4: ...
    @property
    def lobb(self) -> U64Vector4: ...
    @property
    def lobo(self) -> U64Vector4: ...
    @property
    def lobl(self) -> U64Vector4: ...
    @property
    def loor(self) -> U64Vector4: ...
    @property
    def loog(self) -> U64Vector4: ...
    @property
    def loob(self) -> U64Vector4: ...
    @property
    def lolr(self) -> U64Vector4: ...
    @property
    def lolg(self) -> U64Vector4: ...
    @property
    def lolb(self) -> U64Vector4: ...
    @property
    def llrr(self) -> U64Vector4: ...
    @property
    def llrg(self) -> U64Vector4: ...
    @property
    def llrb(self) -> U64Vector4: ...
    @property
    def llro(self) -> U64Vector4: ...
    @property
    def llrl(self) -> U64Vector4: ...
    @property
    def llgr(self) -> U64Vector4: ...
    @property
    def llgg(self) -> U64Vector4: ...
    @property
    def llgb(self) -> U64Vector4: ...
    @property
    def llgo(self) -> U64Vector4: ...
    @property
    def llgl(self) -> U64Vector4: ...
    @property
    def llbr(self) -> U64Vector4: ...
    @property
    def llbg(self) -> U64Vector4: ...
    @property
    def llbb(self) -> U64Vector4: ...
    @property
    def llbo(self) -> U64Vector4: ...
    @property
    def llbl(self) -> U64Vector4: ...
    @property
    def llor(self) -> U64Vector4: ...
    @property
    def llog(self) -> U64Vector4: ...
    @property
    def llob(self) -> U64Vector4: ...
    @property
    def lllr(self) -> U64Vector4: ...
    @property
    def lllg(self) -> U64Vector4: ...
    @property
    def lllb(self) -> U64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def ss(self) -> U64Vector2: ...
    @property
    def st(self) -> U64Vector2: ...
    @property
    def sq(self) -> U64Vector2: ...
    @property
    def so(self) -> U64Vector2: ...
    @property
    def sl(self) -> U64Vector2: ...
    @property
    def ts(self) -> U64Vector2: ...
    @property
    def tt(self) -> U64Vector2: ...
    @property
    def tq(self) -> U64Vector2: ...
    @property
    def to(self) -> U64Vector2: ...
    @property
    def tl(self) -> U64Vector2: ...
    @property
    def qs(self) -> U64Vector2: ...
    @property
    def qt(self) -> U64Vector2: ...
    @property
    def qq(self) -> U64Vector2: ...
    @property
    def qo(self) -> U64Vector2: ...
    @property
    def ql(self) -> U64Vector2: ...
    @property
    def os(self) -> U64Vector2: ...
    @property
    def ot(self) -> U64Vector2: ...
    @property
    def oq(self) -> U64Vector2: ...
    @property
    def ls(self) -> U64Vector2: ...
    @property
    def lt(self) -> U64Vector2: ...
    @property
    def lq(self) -> U64Vector2: ...
    @property
    def sss(self) -> U64Vector3: ...
    @property
    def sst(self) -> U64Vector3: ...
    @property
    def ssq(self) -> U64Vector3: ...
    @property
    def sso(self) -> U64Vector3: ...
    @property
    def ssl(self) -> U64Vector3: ...
    @property
    def sts(self) -> U64Vector3: ...
    @property
    def stt(self) -> U64Vector3: ...
    @property
    def stq(self) -> U64Vector3: ...
    @property
    def sto(self) -> U64Vector3: ...
    @property
    def stl(self) -> U64Vector3: ...
    @property
    def sqs(self) -> U64Vector3: ...
    @property
    def sqt(self) -> U64Vector3: ...
    @property
    def sqq(self) -> U64Vector3: ...
    @property
    def sqo(self) -> U64Vector3: ...
    @property
    def sql(self) -> U64Vector3: ...
    @property
    def sos(self) -> U64Vector3: ...
    @property
    def sot(self) -> U64Vector3: ...
    @property
    def soq(self) -> U64Vector3: ...
    @property
    def soo(self) -> U64Vector3: ...
    @property
    def sol(self) -> U64Vector3: ...
    @property
    def sls(self) -> U64Vector3: ...
    @property
    def slt(self) -> U64Vector3: ...
    @property
    def slq(self) -> U64Vector3: ...
    @property
    def slo(self) -> U64Vector3: ...
    @property
    def sll(self) -> U64Vector3: ...
    @property
    def tss(self) -> U64Vector3: ...
    @property
    def tst(self) -> U64Vector3: ...
    @property
    def tsq(self) -> U64Vector3: ...
    @property
    def tso(self) -> U64Vector3: ...
    @property
    def tsl(self) -> U64Vector3: ...
    @property
    def tts(self) -> U64Vector3: ...
    @property
    def ttt(self) -> U64Vector3: ...
    @property
    def ttq(self) -> U64Vector3: ...
    @property
    def tto(self) -> U64Vector3: ...
    @property
    def ttl(self) -> U64Vector3: ...
    @property
    def tqs(self) -> U64Vector3: ...
    @property
    def tqt(self) -> U64Vector3: ...
    @property
    def tqq(self) -> U64Vector3: ...
    @property
    def tqo(self) -> U64Vector3: ...
    @property
    def tql(self) -> U64Vector3: ...
    @property
    def tos(self) -> U64Vector3: ...
    @property
    def tot(self) -> U64Vector3: ...
    @property
    def toq(self) -> U64Vector3: ...
    @property
    def too(self) -> U64Vector3: ...
    @property
    def tol(self) -> U64Vector3: ...
    @property
    def tls(self) -> U64Vector3: ...
    @property
    def tlt(self) -> U64Vector3: ...
    @property
    def tlq(self) -> U64Vector3: ...
    @property
    def tlo(self) -> U64Vector3: ...
    @property
    def tll(self) -> U64Vector3: ...
    @property
    def qss(self) -> U64Vector3: ...
    @property
    def qst(self) -> U64Vector3: ...
    @property
    def qsq(self) -> U64Vector3: ...
    @property
    def qso(self) -> U64Vector3: ...
    @property
    def qsl(self) -> U64Vector3: ...
    @property
    def qts(self) -> U64Vector3: ...
    @property
    def qtt(self) -> U64Vector3: ...
    @property
    def qtq(self) -> U64Vector3: ...
    @property
    def qto(self) -> U64Vector3: ...
    @property
    def qtl(self) -> U64Vector3: ...
    @property
    def qqs(self) -> U64Vector3: ...
    @property
    def qqt(self) -> U64Vector3: ...
    @property
    def qqq(self) -> U64Vector3: ...
    @property
    def qqo(self) -> U64Vector3: ...
    @property
    def qql(self) -> U64Vector3: ...
    @property
    def qos(self) -> U64Vector3: ...
    @property
    def qot(self) -> U64Vector3: ...
    @property
    def qoq(self) -> U64Vector3: ...
    @property
    def qoo(self) -> U64Vector3: ...
    @property
    def qol(self) -> U64Vector3: ...
    @property
    def qls(self) -> U64Vector3: ...
    @property
    def qlt(self) -> U64Vector3: ...
    @property
    def qlq(self) -> U64Vector3: ...
    @property
    def qlo(self) -> U64Vector3: ...
    @property
    def qll(self) -> U64Vector3: ...
    @property
    def oss(self) -> U64Vector3: ...
    @property
    def ost(self) -> U64Vector3: ...
    @property
    def osq(self) -> U64Vector3: ...
    @property
    def oso(self) -> U64Vector3: ...
    @property
    def osl(self) -> U64Vector3: ...
    @property
    def ots(self) -> U64Vector3: ...
    @property
    def ott(self) -> U64Vector3: ...
    @property
    def otq(self) -> U64Vector3: ...
    @property
    def oto(self) -> U64Vector3: ...
    @property
    def otl(self) -> U64Vector3: ...
    @property
    def oqs(self) -> U64Vector3: ...
    @property
    def oqt(self) -> U64Vector3: ...
    @property
    def oqq(self) -> U64Vector3: ...
    @property
    def oqo(self) -> U64Vector3: ...
    @property
    def oql(self) -> U64Vector3: ...
    @property
    def oos(self) -> U64Vector3: ...
    @property
    def oot(self) -> U64Vector3: ...
    @property
    def ooq(self) -> U64Vector3: ...
    @property
    def ols(self) -> U64Vector3: ...
    @property
    def olt(self) -> U64Vector3: ...
    @property
    def olq(self) -> U64Vector3: ...
    @property
    def lss(self) -> U64Vector3: ...
    @property
    def lst(self) -> U64Vector3: ...
    @property
    def lsq(self) -> U64Vector3: ...
    @property
    def lso(self) -> U64Vector3: ...
    @property
    def lsl(self) -> U64Vector3: ...
    @property
    def lts(self) -> U64Vector3: ...
    @property
    def ltt(self) -> U64Vector3: ...
    @property
    def ltq(self) -> U64Vector3: ...
    @property
    def lto(self) -> U64Vector3: ...
    @property
    def ltl(self) -> U64Vector3: ...
    @property
    def lqs(self) -> U64Vector3: ...
    @property
    def lqt(self) -> U64Vector3: ...
    @property
    def lqq(self) -> U64Vector3: ...
    @property
    def lqo(self) -> U64Vector3: ...
    @property
    def lql(self) -> U64Vector3: ...
    @property
    def los(self) -> U64Vector3: ...
    @property
    def lot(self) -> U64Vector3: ...
    @property
    def loq(self) -> U64Vector3: ...
    @property
    def lls(self) -> U64Vector3: ...
    @property
    def llt(self) -> U64Vector3: ...
    @property
    def llq(self) -> U64Vector3: ...
    @property
    def ssss(self) -> U64Vector4: ...
    @property
    def ssst(self) -> U64Vector4: ...
    @property
    def sssq(self) -> U64Vector4: ...
    @property
    def ssso(self) -> U64Vector4: ...
    @property
    def sssl(self) -> U64Vector4: ...
    @property
    def ssts(self) -> U64Vector4: ...
    @property
    def sstt(self) -> U64Vector4: ...
    @property
    def sstq(self) -> U64Vector4: ...
    @property
    def ssto(self) -> U64Vector4: ...
    @property
    def sstl(self) -> U64Vector4: ...
    @property
    def ssqs(self) -> U64Vector4: ...
    @property
    def ssqt(self) -> U64Vector4: ...
    @property
    def ssqq(self) -> U64Vector4: ...
    @property
    def ssqo(self) -> U64Vector4: ...
    @property
    def ssql(self) -> U64Vector4: ...
    @property
    def ssos(self) -> U64Vector4: ...
    @property
    def ssot(self) -> U64Vector4: ...
    @property
    def ssoq(self) -> U64Vector4: ...
    @property
    def ssoo(self) -> U64Vector4: ...
    @property
    def ssol(self) -> U64Vector4: ...
    @property
    def ssls(self) -> U64Vector4: ...
    @property
    def sslt(self) -> U64Vector4: ...
    @property
    def sslq(self) -> U64Vector4: ...
    @property
    def sslo(self) -> U64Vector4: ...
    @property
    def ssll(self) -> U64Vector4: ...
    @property
    def stss(self) -> U64Vector4: ...
    @property
    def stst(self) -> U64Vector4: ...
    @property
    def stsq(self) -> U64Vector4: ...
    @property
    def stso(self) -> U64Vector4: ...
    @property
    def stsl(self) -> U64Vector4: ...
    @property
    def stts(self) -> U64Vector4: ...
    @property
    def sttt(self) -> U64Vector4: ...
    @property
    def sttq(self) -> U64Vector4: ...
    @property
    def stto(self) -> U64Vector4: ...
    @property
    def sttl(self) -> U64Vector4: ...
    @property
    def stqs(self) -> U64Vector4: ...
    @property
    def stqt(self) -> U64Vector4: ...
    @property
    def stqq(self) -> U64Vector4: ...
    @property
    def stqo(self) -> U64Vector4: ...
    @property
    def stql(self) -> U64Vector4: ...
    @property
    def stos(self) -> U64Vector4: ...
    @property
    def stot(self) -> U64Vector4: ...
    @property
    def stoq(self) -> U64Vector4: ...
    @property
    def stoo(self) -> U64Vector4: ...
    @property
    def stol(self) -> U64Vector4: ...
    @property
    def stls(self) -> U64Vector4: ...
    @property
    def stlt(self) -> U64Vector4: ...
    @property
    def stlq(self) -> U64Vector4: ...
    @property
    def stlo(self) -> U64Vector4: ...
    @property
    def stll(self) -> U64Vector4: ...
    @property
    def sqss(self) -> U64Vector4: ...
    @property
    def sqst(self) -> U64Vector4: ...
    @property
    def sqsq(self) -> U64Vector4: ...
    @property
    def sqso(self) -> U64Vector4: ...
    @property
    def sqsl(self) -> U64Vector4: ...
    @property
    def sqts(self) -> U64Vector4: ...
    @property
    def sqtt(self) -> U64Vector4: ...
    @property
    def sqtq(self) -> U64Vector4: ...
    @property
    def sqto(self) -> U64Vector4: ...
    @property
    def sqtl(self) -> U64Vector4: ...
    @property
    def sqqs(self) -> U64Vector4: ...
    @property
    def sqqt(self) -> U64Vector4: ...
    @property
    def sqqq(self) -> U64Vector4: ...
    @property
    def sqqo(self) -> U64Vector4: ...
    @property
    def sqql(self) -> U64Vector4: ...
    @property
    def sqos(self) -> U64Vector4: ...
    @property
    def sqot(self) -> U64Vector4: ...
    @property
    def sqoq(self) -> U64Vector4: ...
    @property
    def sqoo(self) -> U64Vector4: ...
    @property
    def sqol(self) -> U64Vector4: ...
    @property
    def sqls(self) -> U64Vector4: ...
    @property
    def sqlt(self) -> U64Vector4: ...
    @property
    def sqlq(self) -> U64Vector4: ...
    @property
    def sqlo(self) -> U64Vector4: ...
    @property
    def sqll(self) -> U64Vector4: ...
    @property
    def soss(self) -> U64Vector4: ...
    @property
    def sost(self) -> U64Vector4: ...
    @property
    def sosq(self) -> U64Vector4: ...
    @property
    def soso(self) -> U64Vector4: ...
    @property
    def sosl(self) -> U64Vector4: ...
    @property
    def sots(self) -> U64Vector4: ...
    @property
    def sott(self) -> U64Vector4: ...
    @property
    def sotq(self) -> U64Vector4: ...
    @property
    def soto(self) -> U64Vector4: ...
    @property
    def sotl(self) -> U64Vector4: ...
    @property
    def soqs(self) -> U64Vector4: ...
    @property
    def soqt(self) -> U64Vector4: ...
    @property
    def soqq(self) -> U64Vector4: ...
    @property
    def soqo(self) -> U64Vector4: ...
    @property
    def soql(self) -> U64Vector4: ...
    @property
    def soos(self) -> U64Vector4: ...
    @property
    def soot(self) -> U64Vector4: ...
    @property
    def sooq(self) -> U64Vector4: ...
    @property
    def sooo(self) -> U64Vector4: ...
    @property
    def sool(self) -> U64Vector4: ...
    @property
    def sols(self) -> U64Vector4: ...
    @property
    def solt(self) -> U64Vector4: ...
    @property
    def solq(self) -> U64Vector4: ...
    @property
    def solo(self) -> U64Vector4: ...
    @property
    def soll(self) -> U64Vector4: ...
    @property
    def slss(self) -> U64Vector4: ...
    @property
    def slst(self) -> U64Vector4: ...
    @property
    def slsq(self) -> U64Vector4: ...
    @property
    def slso(self) -> U64Vector4: ...
    @property
    def slsl(self) -> U64Vector4: ...
    @property
    def slts(self) -> U64Vector4: ...
    @property
    def sltt(self) -> U64Vector4: ...
    @property
    def sltq(self) -> U64Vector4: ...
    @property
    def slto(self) -> U64Vector4: ...
    @property
    def sltl(self) -> U64Vector4: ...
    @property
    def slqs(self) -> U64Vector4: ...
    @property
    def slqt(self) -> U64Vector4: ...
    @property
    def slqq(self) -> U64Vector4: ...
    @property
    def slqo(self) -> U64Vector4: ...
    @property
    def slql(self) -> U64Vector4: ...
    @property
    def slos(self) -> U64Vector4: ...
    @property
    def slot(self) -> U64Vector4: ...
    @property
    def sloq(self) -> U64Vector4: ...
    @property
    def sloo(self) -> U64Vector4: ...
    @property
    def slol(self) -> U64Vector4: ...
    @property
    def slls(self) -> U64Vector4: ...
    @property
    def sllt(self) -> U64Vector4: ...
    @property
    def sllq(self) -> U64Vector4: ...
    @property
    def sllo(self) -> U64Vector4: ...
    @property
    def slll(self) -> U64Vector4: ...
    @property
    def tsss(self) -> U64Vector4: ...
    @property
    def tsst(self) -> U64Vector4: ...
    @property
    def tssq(self) -> U64Vector4: ...
    @property
    def tsso(self) -> U64Vector4: ...
    @property
    def tssl(self) -> U64Vector4: ...
    @property
    def tsts(self) -> U64Vector4: ...
    @property
    def tstt(self) -> U64Vector4: ...
    @property
    def tstq(self) -> U64Vector4: ...
    @property
    def tsto(self) -> U64Vector4: ...
    @property
    def tstl(self) -> U64Vector4: ...
    @property
    def tsqs(self) -> U64Vector4: ...
    @property
    def tsqt(self) -> U64Vector4: ...
    @property
    def tsqq(self) -> U64Vector4: ...
    @property
    def tsqo(self) -> U64Vector4: ...
    @property
    def tsql(self) -> U64Vector4: ...
    @property
    def tsos(self) -> U64Vector4: ...
    @property
    def tsot(self) -> U64Vector4: ...
    @property
    def tsoq(self) -> U64Vector4: ...
    @property
    def tsoo(self) -> U64Vector4: ...
    @property
    def tsol(self) -> U64Vector4: ...
    @property
    def tsls(self) -> U64Vector4: ...
    @property
    def tslt(self) -> U64Vector4: ...
    @property
    def tslq(self) -> U64Vector4: ...
    @property
    def tslo(self) -> U64Vector4: ...
    @property
    def tsll(self) -> U64Vector4: ...
    @property
    def ttss(self) -> U64Vector4: ...
    @property
    def ttst(self) -> U64Vector4: ...
    @property
    def ttsq(self) -> U64Vector4: ...
    @property
    def ttso(self) -> U64Vector4: ...
    @property
    def ttsl(self) -> U64Vector4: ...
    @property
    def ttts(self) -> U64Vector4: ...
    @property
    def tttt(self) -> U64Vector4: ...
    @property
    def tttq(self) -> U64Vector4: ...
    @property
    def ttto(self) -> U64Vector4: ...
    @property
    def tttl(self) -> U64Vector4: ...
    @property
    def ttqs(self) -> U64Vector4: ...
    @property
    def ttqt(self) -> U64Vector4: ...
    @property
    def ttqq(self) -> U64Vector4: ...
    @property
    def ttqo(self) -> U64Vector4: ...
    @property
    def ttql(self) -> U64Vector4: ...
    @property
    def ttos(self) -> U64Vector4: ...
    @property
    def ttot(self) -> U64Vector4: ...
    @property
    def ttoq(self) -> U64Vector4: ...
    @property
    def ttoo(self) -> U64Vector4: ...
    @property
    def ttol(self) -> U64Vector4: ...
    @property
    def ttls(self) -> U64Vector4: ...
    @property
    def ttlt(self) -> U64Vector4: ...
    @property
    def ttlq(self) -> U64Vector4: ...
    @property
    def ttlo(self) -> U64Vector4: ...
    @property
    def ttll(self) -> U64Vector4: ...
    @property
    def tqss(self) -> U64Vector4: ...
    @property
    def tqst(self) -> U64Vector4: ...
    @property
    def tqsq(self) -> U64Vector4: ...
    @property
    def tqso(self) -> U64Vector4: ...
    @property
    def tqsl(self) -> U64Vector4: ...
    @property
    def tqts(self) -> U64Vector4: ...
    @property
    def tqtt(self) -> U64Vector4: ...
    @property
    def tqtq(self) -> U64Vector4: ...
    @property
    def tqto(self) -> U64Vector4: ...
    @property
    def tqtl(self) -> U64Vector4: ...
    @property
    def tqqs(self) -> U64Vector4: ...
    @property
    def tqqt(self) -> U64Vector4: ...
    @property
    def tqqq(self) -> U64Vector4: ...
    @property
    def tqqo(self) -> U64Vector4: ...
    @property
    def tqql(self) -> U64Vector4: ...
    @property
    def tqos(self) -> U64Vector4: ...
    @property
    def tqot(self) -> U64Vector4: ...
    @property
    def tqoq(self) -> U64Vector4: ...
    @property
    def tqoo(self) -> U64Vector4: ...
    @property
    def tqol(self) -> U64Vector4: ...
    @property
    def tqls(self) -> U64Vector4: ...
    @property
    def tqlt(self) -> U64Vector4: ...
    @property
    def tqlq(self) -> U64Vector4: ...
    @property
    def tqlo(self) -> U64Vector4: ...
    @property
    def tqll(self) -> U64Vector4: ...
    @property
    def toss(self) -> U64Vector4: ...
    @property
    def tost(self) -> U64Vector4: ...
    @property
    def tosq(self) -> U64Vector4: ...
    @property
    def toso(self) -> U64Vector4: ...
    @property
    def tosl(self) -> U64Vector4: ...
    @property
    def tots(self) -> U64Vector4: ...
    @property
    def tott(self) -> U64Vector4: ...
    @property
    def totq(self) -> U64Vector4: ...
    @property
    def toto(self) -> U64Vector4: ...
    @property
    def totl(self) -> U64Vector4: ...
    @property
    def toqs(self) -> U64Vector4: ...
    @property
    def toqt(self) -> U64Vector4: ...
    @property
    def toqq(self) -> U64Vector4: ...
    @property
    def toqo(self) -> U64Vector4: ...
    @property
    def toql(self) -> U64Vector4: ...
    @property
    def toos(self) -> U64Vector4: ...
    @property
    def toot(self) -> U64Vector4: ...
    @property
    def tooq(self) -> U64Vector4: ...
    @property
    def tooo(self) -> U64Vector4: ...
    @property
    def tool(self) -> U64Vector4: ...
    @property
    def tols(self) -> U64Vector4: ...
    @property
    def tolt(self) -> U64Vector4: ...
    @property
    def tolq(self) -> U64Vector4: ...
    @property
    def tolo(self) -> U64Vector4: ...
    @property
    def toll(self) -> U64Vector4: ...
    @property
    def tlss(self) -> U64Vector4: ...
    @property
    def tlst(self) -> U64Vector4: ...
    @property
    def tlsq(self) -> U64Vector4: ...
    @property
    def tlso(self) -> U64Vector4: ...
    @property
    def tlsl(self) -> U64Vector4: ...
    @property
    def tlts(self) -> U64Vector4: ...
    @property
    def tltt(self) -> U64Vector4: ...
    @property
    def tltq(self) -> U64Vector4: ...
    @property
    def tlto(self) -> U64Vector4: ...
    @property
    def tltl(self) -> U64Vector4: ...
    @property
    def tlqs(self) -> U64Vector4: ...
    @property
    def tlqt(self) -> U64Vector4: ...
    @property
    def tlqq(self) -> U64Vector4: ...
    @property
    def tlqo(self) -> U64Vector4: ...
    @property
    def tlql(self) -> U64Vector4: ...
    @property
    def tlos(self) -> U64Vector4: ...
    @property
    def tlot(self) -> U64Vector4: ...
    @property
    def tloq(self) -> U64Vector4: ...
    @property
    def tloo(self) -> U64Vector4: ...
    @property
    def tlol(self) -> U64Vector4: ...
    @property
    def tlls(self) -> U64Vector4: ...
    @property
    def tllt(self) -> U64Vector4: ...
    @property
    def tllq(self) -> U64Vector4: ...
    @property
    def tllo(self) -> U64Vector4: ...
    @property
    def tlll(self) -> U64Vector4: ...
    @property
    def qsss(self) -> U64Vector4: ...
    @property
    def qsst(self) -> U64Vector4: ...
    @property
    def qssq(self) -> U64Vector4: ...
    @property
    def qsso(self) -> U64Vector4: ...
    @property
    def qssl(self) -> U64Vector4: ...
    @property
    def qsts(self) -> U64Vector4: ...
    @property
    def qstt(self) -> U64Vector4: ...
    @property
    def qstq(self) -> U64Vector4: ...
    @property
    def qsto(self) -> U64Vector4: ...
    @property
    def qstl(self) -> U64Vector4: ...
    @property
    def qsqs(self) -> U64Vector4: ...
    @property
    def qsqt(self) -> U64Vector4: ...
    @property
    def qsqq(self) -> U64Vector4: ...
    @property
    def qsqo(self) -> U64Vector4: ...
    @property
    def qsql(self) -> U64Vector4: ...
    @property
    def qsos(self) -> U64Vector4: ...
    @property
    def qsot(self) -> U64Vector4: ...
    @property
    def qsoq(self) -> U64Vector4: ...
    @property
    def qsoo(self) -> U64Vector4: ...
    @property
    def qsol(self) -> U64Vector4: ...
    @property
    def qsls(self) -> U64Vector4: ...
    @property
    def qslt(self) -> U64Vector4: ...
    @property
    def qslq(self) -> U64Vector4: ...
    @property
    def qslo(self) -> U64Vector4: ...
    @property
    def qsll(self) -> U64Vector4: ...
    @property
    def qtss(self) -> U64Vector4: ...
    @property
    def qtst(self) -> U64Vector4: ...
    @property
    def qtsq(self) -> U64Vector4: ...
    @property
    def qtso(self) -> U64Vector4: ...
    @property
    def qtsl(self) -> U64Vector4: ...
    @property
    def qtts(self) -> U64Vector4: ...
    @property
    def qttt(self) -> U64Vector4: ...
    @property
    def qttq(self) -> U64Vector4: ...
    @property
    def qtto(self) -> U64Vector4: ...
    @property
    def qttl(self) -> U64Vector4: ...
    @property
    def qtqs(self) -> U64Vector4: ...
    @property
    def qtqt(self) -> U64Vector4: ...
    @property
    def qtqq(self) -> U64Vector4: ...
    @property
    def qtqo(self) -> U64Vector4: ...
    @property
    def qtql(self) -> U64Vector4: ...
    @property
    def qtos(self) -> U64Vector4: ...
    @property
    def qtot(self) -> U64Vector4: ...
    @property
    def qtoq(self) -> U64Vector4: ...
    @property
    def qtoo(self) -> U64Vector4: ...
    @property
    def qtol(self) -> U64Vector4: ...
    @property
    def qtls(self) -> U64Vector4: ...
    @property
    def qtlt(self) -> U64Vector4: ...
    @property
    def qtlq(self) -> U64Vector4: ...
    @property
    def qtlo(self) -> U64Vector4: ...
    @property
    def qtll(self) -> U64Vector4: ...
    @property
    def qqss(self) -> U64Vector4: ...
    @property
    def qqst(self) -> U64Vector4: ...
    @property
    def qqsq(self) -> U64Vector4: ...
    @property
    def qqso(self) -> U64Vector4: ...
    @property
    def qqsl(self) -> U64Vector4: ...
    @property
    def qqts(self) -> U64Vector4: ...
    @property
    def qqtt(self) -> U64Vector4: ...
    @property
    def qqtq(self) -> U64Vector4: ...
    @property
    def qqto(self) -> U64Vector4: ...
    @property
    def qqtl(self) -> U64Vector4: ...
    @property
    def qqqs(self) -> U64Vector4: ...
    @property
    def qqqt(self) -> U64Vector4: ...
    @property
    def qqqq(self) -> U64Vector4: ...
    @property
    def qqqo(self) -> U64Vector4: ...
    @property
    def qqql(self) -> U64Vector4: ...
    @property
    def qqos(self) -> U64Vector4: ...
    @property
    def qqot(self) -> U64Vector4: ...
    @property
    def qqoq(self) -> U64Vector4: ...
    @property
    def qqoo(self) -> U64Vector4: ...
    @property
    def qqol(self) -> U64Vector4: ...
    @property
    def qqls(self) -> U64Vector4: ...
    @property
    def qqlt(self) -> U64Vector4: ...
    @property
    def qqlq(self) -> U64Vector4: ...
    @property
    def qqlo(self) -> U64Vector4: ...
    @property
    def qqll(self) -> U64Vector4: ...
    @property
    def qoss(self) -> U64Vector4: ...
    @property
    def qost(self) -> U64Vector4: ...
    @property
    def qosq(self) -> U64Vector4: ...
    @property
    def qoso(self) -> U64Vector4: ...
    @property
    def qosl(self) -> U64Vector4: ...
    @property
    def qots(self) -> U64Vector4: ...
    @property
    def qott(self) -> U64Vector4: ...
    @property
    def qotq(self) -> U64Vector4: ...
    @property
    def qoto(self) -> U64Vector4: ...
    @property
    def qotl(self) -> U64Vector4: ...
    @property
    def qoqs(self) -> U64Vector4: ...
    @property
    def qoqt(self) -> U64Vector4: ...
    @property
    def qoqq(self) -> U64Vector4: ...
    @property
    def qoqo(self) -> U64Vector4: ...
    @property
    def qoql(self) -> U64Vector4: ...
    @property
    def qoos(self) -> U64Vector4: ...
    @property
    def qoot(self) -> U64Vector4: ...
    @property
    def qooq(self) -> U64Vector4: ...
    @property
    def qooo(self) -> U64Vector4: ...
    @property
    def qool(self) -> U64Vector4: ...
    @property
    def qols(self) -> U64Vector4: ...
    @property
    def qolt(self) -> U64Vector4: ...
    @property
    def qolq(self) -> U64Vector4: ...
    @property
    def qolo(self) -> U64Vector4: ...
    @property
    def qoll(self) -> U64Vector4: ...
    @property
    def qlss(self) -> U64Vector4: ...
    @property
    def qlst(self) -> U64Vector4: ...
    @property
    def qlsq(self) -> U64Vector4: ...
    @property
    def qlso(self) -> U64Vector4: ...
    @property
    def qlsl(self) -> U64Vector4: ...
    @property
    def qlts(self) -> U64Vector4: ...
    @property
    def qltt(self) -> U64Vector4: ...
    @property
    def qltq(self) -> U64Vector4: ...
    @property
    def qlto(self) -> U64Vector4: ...
    @property
    def qltl(self) -> U64Vector4: ...
    @property
    def qlqs(self) -> U64Vector4: ...
    @property
    def qlqt(self) -> U64Vector4: ...
    @property
    def qlqq(self) -> U64Vector4: ...
    @property
    def qlqo(self) -> U64Vector4: ...
    @property
    def qlql(self) -> U64Vector4: ...
    @property
    def qlos(self) -> U64Vector4: ...
    @property
    def qlot(self) -> U64Vector4: ...
    @property
    def qloq(self) -> U64Vector4: ...
    @property
    def qloo(self) -> U64Vector4: ...
    @property
    def qlol(self) -> U64Vector4: ...
    @property
    def qlls(self) -> U64Vector4: ...
    @property
    def qllt(self) -> U64Vector4: ...
    @property
    def qllq(self) -> U64Vector4: ...
    @property
    def qllo(self) -> U64Vector4: ...
    @property
    def qlll(self) -> U64Vector4: ...
    @property
    def osss(self) -> U64Vector4: ...
    @property
    def osst(self) -> U64Vector4: ...
    @property
    def ossq(self) -> U64Vector4: ...
    @property
    def osso(self) -> U64Vector4: ...
    @property
    def ossl(self) -> U64Vector4: ...
    @property
    def osts(self) -> U64Vector4: ...
    @property
    def ostt(self) -> U64Vector4: ...
    @property
    def ostq(self) -> U64Vector4: ...
    @property
    def osto(self) -> U64Vector4: ...
    @property
    def ostl(self) -> U64Vector4: ...
    @property
    def osqs(self) -> U64Vector4: ...
    @property
    def osqt(self) -> U64Vector4: ...
    @property
    def osqq(self) -> U64Vector4: ...
    @property
    def osqo(self) -> U64Vector4: ...
    @property
    def osql(self) -> U64Vector4: ...
    @property
    def osos(self) -> U64Vector4: ...
    @property
    def osot(self) -> U64Vector4: ...
    @property
    def osoq(self) -> U64Vector4: ...
    @property
    def osoo(self) -> U64Vector4: ...
    @property
    def osol(self) -> U64Vector4: ...
    @property
    def osls(self) -> U64Vector4: ...
    @property
    def oslt(self) -> U64Vector4: ...
    @property
    def oslq(self) -> U64Vector4: ...
    @property
    def oslo(self) -> U64Vector4: ...
    @property
    def osll(self) -> U64Vector4: ...
    @property
    def otss(self) -> U64Vector4: ...
    @property
    def otst(self) -> U64Vector4: ...
    @property
    def otsq(self) -> U64Vector4: ...
    @property
    def otso(self) -> U64Vector4: ...
    @property
    def otsl(self) -> U64Vector4: ...
    @property
    def otts(self) -> U64Vector4: ...
    @property
    def ottt(self) -> U64Vector4: ...
    @property
    def ottq(self) -> U64Vector4: ...
    @property
    def otto(self) -> U64Vector4: ...
    @property
    def ottl(self) -> U64Vector4: ...
    @property
    def otqs(self) -> U64Vector4: ...
    @property
    def otqt(self) -> U64Vector4: ...
    @property
    def otqq(self) -> U64Vector4: ...
    @property
    def otqo(self) -> U64Vector4: ...
    @property
    def otql(self) -> U64Vector4: ...
    @property
    def otos(self) -> U64Vector4: ...
    @property
    def otot(self) -> U64Vector4: ...
    @property
    def otoq(self) -> U64Vector4: ...
    @property
    def otoo(self) -> U64Vector4: ...
    @property
    def otol(self) -> U64Vector4: ...
    @property
    def otls(self) -> U64Vector4: ...
    @property
    def otlt(self) -> U64Vector4: ...
    @property
    def otlq(self) -> U64Vector4: ...
    @property
    def otlo(self) -> U64Vector4: ...
    @property
    def otll(self) -> U64Vector4: ...
    @property
    def oqss(self) -> U64Vector4: ...
    @property
    def oqst(self) -> U64Vector4: ...
    @property
    def oqsq(self) -> U64Vector4: ...
    @property
    def oqso(self) -> U64Vector4: ...
    @property
    def oqsl(self) -> U64Vector4: ...
    @property
    def oqts(self) -> U64Vector4: ...
    @property
    def oqtt(self) -> U64Vector4: ...
    @property
    def oqtq(self) -> U64Vector4: ...
    @property
    def oqto(self) -> U64Vector4: ...
    @property
    def oqtl(self) -> U64Vector4: ...
    @property
    def oqqs(self) -> U64Vector4: ...
    @property
    def oqqt(self) -> U64Vector4: ...
    @property
    def oqqq(self) -> U64Vector4: ...
    @property
    def oqqo(self) -> U64Vector4: ...
    @property
    def oqql(self) -> U64Vector4: ...
    @property
    def oqos(self) -> U64Vector4: ...
    @property
    def oqot(self) -> U64Vector4: ...
    @property
    def oqoq(self) -> U64Vector4: ...
    @property
    def oqoo(self) -> U64Vector4: ...
    @property
    def oqol(self) -> U64Vector4: ...
    @property
    def oqls(self) -> U64Vector4: ...
    @property
    def oqlt(self) -> U64Vector4: ...
    @property
    def oqlq(self) -> U64Vector4: ...
    @property
    def oqlo(self) -> U64Vector4: ...
    @property
    def oqll(self) -> U64Vector4: ...
    @property
    def ooss(self) -> U64Vector4: ...
    @property
    def oost(self) -> U64Vector4: ...
    @property
    def oosq(self) -> U64Vector4: ...
    @property
    def ooso(self) -> U64Vector4: ...
    @property
    def oosl(self) -> U64Vector4: ...
    @property
    def oots(self) -> U64Vector4: ...
    @property
    def oott(self) -> U64Vector4: ...
    @property
    def ootq(self) -> U64Vector4: ...
    @property
    def ooto(self) -> U64Vector4: ...
    @property
    def ootl(self) -> U64Vector4: ...
    @property
    def ooqs(self) -> U64Vector4: ...
    @property
    def ooqt(self) -> U64Vector4: ...
    @property
    def ooqq(self) -> U64Vector4: ...
    @property
    def ooqo(self) -> U64Vector4: ...
    @property
    def ooql(self) -> U64Vector4: ...
    @property
    def ooos(self) -> U64Vector4: ...
    @property
    def ooot(self) -> U64Vector4: ...
    @property
    def oooq(self) -> U64Vector4: ...
    @property
    def ools(self) -> U64Vector4: ...
    @property
    def oolt(self) -> U64Vector4: ...
    @property
    def oolq(self) -> U64Vector4: ...
    @property
    def olss(self) -> U64Vector4: ...
    @property
    def olst(self) -> U64Vector4: ...
    @property
    def olsq(self) -> U64Vector4: ...
    @property
    def olso(self) -> U64Vector4: ...
    @property
    def olsl(self) -> U64Vector4: ...
    @property
    def olts(self) -> U64Vector4: ...
    @property
    def oltt(self) -> U64Vector4: ...
    @property
    def oltq(self) -> U64Vector4: ...
    @property
    def olto(self) -> U64Vector4: ...
    @property
    def oltl(self) -> U64Vector4: ...
    @property
    def olqs(self) -> U64Vector4: ...
    @property
    def olqt(self) -> U64Vector4: ...
    @property
    def olqq(self) -> U64Vector4: ...
    @property
    def olqo(self) -> U64Vector4: ...
    @property
    def olql(self) -> U64Vector4: ...
    @property
    def olos(self) -> U64Vector4: ...
    @property
    def olot(self) -> U64Vector4: ...
    @property
    def oloq(self) -> U64Vector4: ...
    @property
    def olls(self) -> U64Vector4: ...
    @property
    def ollt(self) -> U64Vector4: ...
    @property
    def ollq(self) -> U64Vector4: ...
    @property
    def lsss(self) -> U64Vector4: ...
    @property
    def lsst(self) -> U64Vector4: ...
    @property
    def lssq(self) -> U64Vector4: ...
    @property
    def lsso(self) -> U64Vector4: ...
    @property
    def lssl(self) -> U64Vector4: ...
    @property
    def lsts(self) -> U64Vector4: ...
    @property
    def lstt(self) -> U64Vector4: ...
    @property
    def lstq(self) -> U64Vector4: ...
    @property
    def lsto(self) -> U64Vector4: ...
    @property
    def lstl(self) -> U64Vector4: ...
    @property
    def lsqs(self) -> U64Vector4: ...
    @property
    def lsqt(self) -> U64Vector4: ...
    @property
    def lsqq(self) -> U64Vector4: ...
    @property
    def lsqo(self) -> U64Vector4: ...
    @property
    def lsql(self) -> U64Vector4: ...
    @property
    def lsos(self) -> U64Vector4: ...
    @property
    def lsot(self) -> U64Vector4: ...
    @property
    def lsoq(self) -> U64Vector4: ...
    @property
    def lsoo(self) -> U64Vector4: ...
    @property
    def lsol(self) -> U64Vector4: ...
    @property
    def lsls(self) -> U64Vector4: ...
    @property
    def lslt(self) -> U64Vector4: ...
    @property
    def lslq(self) -> U64Vector4: ...
    @property
    def lslo(self) -> U64Vector4: ...
    @property
    def lsll(self) -> U64Vector4: ...
    @property
    def ltss(self) -> U64Vector4: ...
    @property
    def ltst(self) -> U64Vector4: ...
    @property
    def ltsq(self) -> U64Vector4: ...
    @property
    def ltso(self) -> U64Vector4: ...
    @property
    def ltsl(self) -> U64Vector4: ...
    @property
    def ltts(self) -> U64Vector4: ...
    @property
    def lttt(self) -> U64Vector4: ...
    @property
    def lttq(self) -> U64Vector4: ...
    @property
    def ltto(self) -> U64Vector4: ...
    @property
    def lttl(self) -> U64Vector4: ...
    @property
    def ltqs(self) -> U64Vector4: ...
    @property
    def ltqt(self) -> U64Vector4: ...
    @property
    def ltqq(self) -> U64Vector4: ...
    @property
    def ltqo(self) -> U64Vector4: ...
    @property
    def ltql(self) -> U64Vector4: ...
    @property
    def ltos(self) -> U64Vector4: ...
    @property
    def ltot(self) -> U64Vector4: ...
    @property
    def ltoq(self) -> U64Vector4: ...
    @property
    def ltoo(self) -> U64Vector4: ...
    @property
    def ltol(self) -> U64Vector4: ...
    @property
    def ltls(self) -> U64Vector4: ...
    @property
    def ltlt(self) -> U64Vector4: ...
    @property
    def ltlq(self) -> U64Vector4: ...
    @property
    def ltlo(self) -> U64Vector4: ...
    @property
    def ltll(self) -> U64Vector4: ...
    @property
    def lqss(self) -> U64Vector4: ...
    @property
    def lqst(self) -> U64Vector4: ...
    @property
    def lqsq(self) -> U64Vector4: ...
    @property
    def lqso(self) -> U64Vector4: ...
    @property
    def lqsl(self) -> U64Vector4: ...
    @property
    def lqts(self) -> U64Vector4: ...
    @property
    def lqtt(self) -> U64Vector4: ...
    @property
    def lqtq(self) -> U64Vector4: ...
    @property
    def lqto(self) -> U64Vector4: ...
    @property
    def lqtl(self) -> U64Vector4: ...
    @property
    def lqqs(self) -> U64Vector4: ...
    @property
    def lqqt(self) -> U64Vector4: ...
    @property
    def lqqq(self) -> U64Vector4: ...
    @property
    def lqqo(self) -> U64Vector4: ...
    @property
    def lqql(self) -> U64Vector4: ...
    @property
    def lqos(self) -> U64Vector4: ...
    @property
    def lqot(self) -> U64Vector4: ...
    @property
    def lqoq(self) -> U64Vector4: ...
    @property
    def lqoo(self) -> U64Vector4: ...
    @property
    def lqol(self) -> U64Vector4: ...
    @property
    def lqls(self) -> U64Vector4: ...
    @property
    def lqlt(self) -> U64Vector4: ...
    @property
    def lqlq(self) -> U64Vector4: ...
    @property
    def lqlo(self) -> U64Vector4: ...
    @property
    def lqll(self) -> U64Vector4: ...
    @property
    def loss(self) -> U64Vector4: ...
    @property
    def lost(self) -> U64Vector4: ...
    @property
    def losq(self) -> U64Vector4: ...
    @property
    def loso(self) -> U64Vector4: ...
    @property
    def losl(self) -> U64Vector4: ...
    @property
    def lots(self) -> U64Vector4: ...
    @property
    def lott(self) -> U64Vector4: ...
    @property
    def lotq(self) -> U64Vector4: ...
    @property
    def loto(self) -> U64Vector4: ...
    @property
    def lotl(self) -> U64Vector4: ...
    @property
    def loqs(self) -> U64Vector4: ...
    @property
    def loqt(self) -> U64Vector4: ...
    @property
    def loqq(self) -> U64Vector4: ...
    @property
    def loqo(self) -> U64Vector4: ...
    @property
    def loql(self) -> U64Vector4: ...
    @property
    def loos(self) -> U64Vector4: ...
    @property
    def loot(self) -> U64Vector4: ...
    @property
    def looq(self) -> U64Vector4: ...
    @property
    def lols(self) -> U64Vector4: ...
    @property
    def lolt(self) -> U64Vector4: ...
    @property
    def lolq(self) -> U64Vector4: ...
    @property
    def llss(self) -> U64Vector4: ...
    @property
    def llst(self) -> U64Vector4: ...
    @property
    def llsq(self) -> U64Vector4: ...
    @property
    def llso(self) -> U64Vector4: ...
    @property
    def llsl(self) -> U64Vector4: ...
    @property
    def llts(self) -> U64Vector4: ...
    @property
    def lltt(self) -> U64Vector4: ...
    @property
    def lltq(self) -> U64Vector4: ...
    @property
    def llto(self) -> U64Vector4: ...
    @property
    def lltl(self) -> U64Vector4: ...
    @property
    def llqs(self) -> U64Vector4: ...
    @property
    def llqt(self) -> U64Vector4: ...
    @property
    def llqq(self) -> U64Vector4: ...
    @property
    def llqo(self) -> U64Vector4: ...
    @property
    def llql(self) -> U64Vector4: ...
    @property
    def llos(self) -> U64Vector4: ...
    @property
    def llot(self) -> U64Vector4: ...
    @property
    def lloq(self) -> U64Vector4: ...
    @property
    def llls(self) -> U64Vector4: ...
    @property
    def lllt(self) -> U64Vector4: ...
    @property
    def lllq(self) -> U64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U64Vector2: ...
    @property
    def uv(self) -> U64Vector2: ...
    @property
    def uo(self) -> U64Vector2: ...
    @property
    def ul(self) -> U64Vector2: ...
    @property
    def vu(self) -> U64Vector2: ...
    @property
    def vv(self) -> U64Vector2: ...
    @property
    def vo(self) -> U64Vector2: ...
    @property
    def vl(self) -> U64Vector2: ...
    @property
    def ou(self) -> U64Vector2: ...
    @property
    def ov(self) -> U64Vector2: ...
    @property
    def lu(self) -> U64Vector2: ...
    @property
    def lv(self) -> U64Vector2: ...
    @property
    def uuu(self) -> U64Vector3: ...
    @property
    def uuv(self) -> U64Vector3: ...
    @property
    def uuo(self) -> U64Vector3: ...
    @property
    def uul(self) -> U64Vector3: ...
    @property
    def uvu(self) -> U64Vector3: ...
    @property
    def uvv(self) -> U64Vector3: ...
    @property
    def uvo(self) -> U64Vector3: ...
    @property
    def uvl(self) -> U64Vector3: ...
    @property
    def uou(self) -> U64Vector3: ...
    @property
    def uov(self) -> U64Vector3: ...
    @property
    def uoo(self) -> U64Vector3: ...
    @property
    def uol(self) -> U64Vector3: ...
    @property
    def ulu(self) -> U64Vector3: ...
    @property
    def ulv(self) -> U64Vector3: ...
    @property
    def ulo(self) -> U64Vector3: ...
    @property
    def ull(self) -> U64Vector3: ...
    @property
    def vuu(self) -> U64Vector3: ...
    @property
    def vuv(self) -> U64Vector3: ...
    @property
    def vuo(self) -> U64Vector3: ...
    @property
    def vul(self) -> U64Vector3: ...
    @property
    def vvu(self) -> U64Vector3: ...
    @property
    def vvv(self) -> U64Vector3: ...
    @property
    def vvo(self) -> U64Vector3: ...
    @property
    def vvl(self) -> U64Vector3: ...
    @property
    def vou(self) -> U64Vector3: ...
    @property
    def vov(self) -> U64Vector3: ...
    @property
    def voo(self) -> U64Vector3: ...
    @property
    def vol(self) -> U64Vector3: ...
    @property
    def vlu(self) -> U64Vector3: ...
    @property
    def vlv(self) -> U64Vector3: ...
    @property
    def vlo(self) -> U64Vector3: ...
    @property
    def vll(self) -> U64Vector3: ...
    @property
    def ouu(self) -> U64Vector3: ...
    @property
    def ouv(self) -> U64Vector3: ...
    @property
    def ouo(self) -> U64Vector3: ...
    @property
    def oul(self) -> U64Vector3: ...
    @property
    def ovu(self) -> U64Vector3: ...
    @property
    def ovv(self) -> U64Vector3: ...
    @property
    def ovo(self) -> U64Vector3: ...
    @property
    def ovl(self) -> U64Vector3: ...
    @property
    def oou(self) -> U64Vector3: ...
    @property
    def oov(self) -> U64Vector3: ...
    @property
    def olu(self) -> U64Vector3: ...
    @property
    def olv(self) -> U64Vector3: ...
    @property
    def luu(self) -> U64Vector3: ...
    @property
    def luv(self) -> U64Vector3: ...
    @property
    def luo(self) -> U64Vector3: ...
    @property
    def lul(self) -> U64Vector3: ...
    @property
    def lvu(self) -> U64Vector3: ...
    @property
    def lvv(self) -> U64Vector3: ...
    @property
    def lvo(self) -> U64Vector3: ...
    @property
    def lvl(self) -> U64Vector3: ...
    @property
    def lou(self) -> U64Vector3: ...
    @property
    def lov(self) -> U64Vector3: ...
    @property
    def llu(self) -> U64Vector3: ...
    @property
    def llv(self) -> U64Vector3: ...
    @property
    def uuuu(self) -> U64Vector4: ...
    @property
    def uuuv(self) -> U64Vector4: ...
    @property
    def uuuo(self) -> U64Vector4: ...
    @property
    def uuul(self) -> U64Vector4: ...
    @property
    def uuvu(self) -> U64Vector4: ...
    @property
    def uuvv(self) -> U64Vector4: ...
    @property
    def uuvo(self) -> U64Vector4: ...
    @property
    def uuvl(self) -> U64Vector4: ...
    @property
    def uuou(self) -> U64Vector4: ...
    @property
    def uuov(self) -> U64Vector4: ...
    @property
    def uuoo(self) -> U64Vector4: ...
    @property
    def uuol(self) -> U64Vector4: ...
    @property
    def uulu(self) -> U64Vector4: ...
    @property
    def uulv(self) -> U64Vector4: ...
    @property
    def uulo(self) -> U64Vector4: ...
    @property
    def uull(self) -> U64Vector4: ...
    @property
    def uvuu(self) -> U64Vector4: ...
    @property
    def uvuv(self) -> U64Vector4: ...
    @property
    def uvuo(self) -> U64Vector4: ...
    @property
    def uvul(self) -> U64Vector4: ...
    @property
    def uvvu(self) -> U64Vector4: ...
    @property
    def uvvv(self) -> U64Vector4: ...
    @property
    def uvvo(self) -> U64Vector4: ...
    @property
    def uvvl(self) -> U64Vector4: ...
    @property
    def uvou(self) -> U64Vector4: ...
    @property
    def uvov(self) -> U64Vector4: ...
    @property
    def uvoo(self) -> U64Vector4: ...
    @property
    def uvol(self) -> U64Vector4: ...
    @property
    def uvlu(self) -> U64Vector4: ...
    @property
    def uvlv(self) -> U64Vector4: ...
    @property
    def uvlo(self) -> U64Vector4: ...
    @property
    def uvll(self) -> U64Vector4: ...
    @property
    def uouu(self) -> U64Vector4: ...
    @property
    def uouv(self) -> U64Vector4: ...
    @property
    def uouo(self) -> U64Vector4: ...
    @property
    def uoul(self) -> U64Vector4: ...
    @property
    def uovu(self) -> U64Vector4: ...
    @property
    def uovv(self) -> U64Vector4: ...
    @property
    def uovo(self) -> U64Vector4: ...
    @property
    def uovl(self) -> U64Vector4: ...
    @property
    def uoou(self) -> U64Vector4: ...
    @property
    def uoov(self) -> U64Vector4: ...
    @property
    def uooo(self) -> U64Vector4: ...
    @property
    def uool(self) -> U64Vector4: ...
    @property
    def uolu(self) -> U64Vector4: ...
    @property
    def uolv(self) -> U64Vector4: ...
    @property
    def uolo(self) -> U64Vector4: ...
    @property
    def uoll(self) -> U64Vector4: ...
    @property
    def uluu(self) -> U64Vector4: ...
    @property
    def uluv(self) -> U64Vector4: ...
    @property
    def uluo(self) -> U64Vector4: ...
    @property
    def ulul(self) -> U64Vector4: ...
    @property
    def ulvu(self) -> U64Vector4: ...
    @property
    def ulvv(self) -> U64Vector4: ...
    @property
    def ulvo(self) -> U64Vector4: ...
    @property
    def ulvl(self) -> U64Vector4: ...
    @property
    def ulou(self) -> U64Vector4: ...
    @property
    def ulov(self) -> U64Vector4: ...
    @property
    def uloo(self) -> U64Vector4: ...
    @property
    def ulol(self) -> U64Vector4: ...
    @property
    def ullu(self) -> U64Vector4: ...
    @property
    def ullv(self) -> U64Vector4: ...
    @property
    def ullo(self) -> U64Vector4: ...
    @property
    def ulll(self) -> U64Vector4: ...
    @property
    def vuuu(self) -> U64Vector4: ...
    @property
    def vuuv(self) -> U64Vector4: ...
    @property
    def vuuo(self) -> U64Vector4: ...
    @property
    def vuul(self) -> U64Vector4: ...
    @property
    def vuvu(self) -> U64Vector4: ...
    @property
    def vuvv(self) -> U64Vector4: ...
    @property
    def vuvo(self) -> U64Vector4: ...
    @property
    def vuvl(self) -> U64Vector4: ...
    @property
    def vuou(self) -> U64Vector4: ...
    @property
    def vuov(self) -> U64Vector4: ...
    @property
    def vuoo(self) -> U64Vector4: ...
    @property
    def vuol(self) -> U64Vector4: ...
    @property
    def vulu(self) -> U64Vector4: ...
    @property
    def vulv(self) -> U64Vector4: ...
    @property
    def vulo(self) -> U64Vector4: ...
    @property
    def vull(self) -> U64Vector4: ...
    @property
    def vvuu(self) -> U64Vector4: ...
    @property
    def vvuv(self) -> U64Vector4: ...
    @property
    def vvuo(self) -> U64Vector4: ...
    @property
    def vvul(self) -> U64Vector4: ...
    @property
    def vvvu(self) -> U64Vector4: ...
    @property
    def vvvv(self) -> U64Vector4: ...
    @property
    def vvvo(self) -> U64Vector4: ...
    @property
    def vvvl(self) -> U64Vector4: ...
    @property
    def vvou(self) -> U64Vector4: ...
    @property
    def vvov(self) -> U64Vector4: ...
    @property
    def vvoo(self) -> U64Vector4: ...
    @property
    def vvol(self) -> U64Vector4: ...
    @property
    def vvlu(self) -> U64Vector4: ...
    @property
    def vvlv(self) -> U64Vector4: ...
    @property
    def vvlo(self) -> U64Vector4: ...
    @property
    def vvll(self) -> U64Vector4: ...
    @property
    def vouu(self) -> U64Vector4: ...
    @property
    def vouv(self) -> U64Vector4: ...
    @property
    def vouo(self) -> U64Vector4: ...
    @property
    def voul(self) -> U64Vector4: ...
    @property
    def vovu(self) -> U64Vector4: ...
    @property
    def vovv(self) -> U64Vector4: ...
    @property
    def vovo(self) -> U64Vector4: ...
    @property
    def vovl(self) -> U64Vector4: ...
    @property
    def voou(self) -> U64Vector4: ...
    @property
    def voov(self) -> U64Vector4: ...
    @property
    def vooo(self) -> U64Vector4: ...
    @property
    def vool(self) -> U64Vector4: ...
    @property
    def volu(self) -> U64Vector4: ...
    @property
    def volv(self) -> U64Vector4: ...
    @property
    def volo(self) -> U64Vector4: ...
    @property
    def voll(self) -> U64Vector4: ...
    @property
    def vluu(self) -> U64Vector4: ...
    @property
    def vluv(self) -> U64Vector4: ...
    @property
    def vluo(self) -> U64Vector4: ...
    @property
    def vlul(self) -> U64Vector4: ...
    @property
    def vlvu(self) -> U64Vector4: ...
    @property
    def vlvv(self) -> U64Vector4: ...
    @property
    def vlvo(self) -> U64Vector4: ...
    @property
    def vlvl(self) -> U64Vector4: ...
    @property
    def vlou(self) -> U64Vector4: ...
    @property
    def vlov(self) -> U64Vector4: ...
    @property
    def vloo(self) -> U64Vector4: ...
    @property
    def vlol(self) -> U64Vector4: ...
    @property
    def vllu(self) -> U64Vector4: ...
    @property
    def vllv(self) -> U64Vector4: ...
    @property
    def vllo(self) -> U64Vector4: ...
    @property
    def vlll(self) -> U64Vector4: ...
    @property
    def ouuu(self) -> U64Vector4: ...
    @property
    def ouuv(self) -> U64Vector4: ...
    @property
    def ouuo(self) -> U64Vector4: ...
    @property
    def ouul(self) -> U64Vector4: ...
    @property
    def ouvu(self) -> U64Vector4: ...
    @property
    def ouvv(self) -> U64Vector4: ...
    @property
    def ouvo(self) -> U64Vector4: ...
    @property
    def ouvl(self) -> U64Vector4: ...
    @property
    def ouou(self) -> U64Vector4: ...
    @property
    def ouov(self) -> U64Vector4: ...
    @property
    def ouoo(self) -> U64Vector4: ...
    @property
    def ouol(self) -> U64Vector4: ...
    @property
    def oulu(self) -> U64Vector4: ...
    @property
    def oulv(self) -> U64Vector4: ...
    @property
    def oulo(self) -> U64Vector4: ...
    @property
    def oull(self) -> U64Vector4: ...
    @property
    def ovuu(self) -> U64Vector4: ...
    @property
    def ovuv(self) -> U64Vector4: ...
    @property
    def ovuo(self) -> U64Vector4: ...
    @property
    def ovul(self) -> U64Vector4: ...
    @property
    def ovvu(self) -> U64Vector4: ...
    @property
    def ovvv(self) -> U64Vector4: ...
    @property
    def ovvo(self) -> U64Vector4: ...
    @property
    def ovvl(self) -> U64Vector4: ...
    @property
    def ovou(self) -> U64Vector4: ...
    @property
    def ovov(self) -> U64Vector4: ...
    @property
    def ovoo(self) -> U64Vector4: ...
    @property
    def ovol(self) -> U64Vector4: ...
    @property
    def ovlu(self) -> U64Vector4: ...
    @property
    def ovlv(self) -> U64Vector4: ...
    @property
    def ovlo(self) -> U64Vector4: ...
    @property
    def ovll(self) -> U64Vector4: ...
    @property
    def oouu(self) -> U64Vector4: ...
    @property
    def oouv(self) -> U64Vector4: ...
    @property
    def oouo(self) -> U64Vector4: ...
    @property
    def ooul(self) -> U64Vector4: ...
    @property
    def oovu(self) -> U64Vector4: ...
    @property
    def oovv(self) -> U64Vector4: ...
    @property
    def oovo(self) -> U64Vector4: ...
    @property
    def oovl(self) -> U64Vector4: ...
    @property
    def ooou(self) -> U64Vector4: ...
    @property
    def ooov(self) -> U64Vector4: ...
    @property
    def oolu(self) -> U64Vector4: ...
    @property
    def oolv(self) -> U64Vector4: ...
    @property
    def oluu(self) -> U64Vector4: ...
    @property
    def oluv(self) -> U64Vector4: ...
    @property
    def oluo(self) -> U64Vector4: ...
    @property
    def olul(self) -> U64Vector4: ...
    @property
    def olvu(self) -> U64Vector4: ...
    @property
    def olvv(self) -> U64Vector4: ...
    @property
    def olvo(self) -> U64Vector4: ...
    @property
    def olvl(self) -> U64Vector4: ...
    @property
    def olou(self) -> U64Vector4: ...
    @property
    def olov(self) -> U64Vector4: ...
    @property
    def ollu(self) -> U64Vector4: ...
    @property
    def ollv(self) -> U64Vector4: ...
    @property
    def luuu(self) -> U64Vector4: ...
    @property
    def luuv(self) -> U64Vector4: ...
    @property
    def luuo(self) -> U64Vector4: ...
    @property
    def luul(self) -> U64Vector4: ...
    @property
    def luvu(self) -> U64Vector4: ...
    @property
    def luvv(self) -> U64Vector4: ...
    @property
    def luvo(self) -> U64Vector4: ...
    @property
    def luvl(self) -> U64Vector4: ...
    @property
    def luou(self) -> U64Vector4: ...
    @property
    def luov(self) -> U64Vector4: ...
    @property
    def luoo(self) -> U64Vector4: ...
    @property
    def luol(self) -> U64Vector4: ...
    @property
    def lulu(self) -> U64Vector4: ...
    @property
    def lulv(self) -> U64Vector4: ...
    @property
    def lulo(self) -> U64Vector4: ...
    @property
    def lull(self) -> U64Vector4: ...
    @property
    def lvuu(self) -> U64Vector4: ...
    @property
    def lvuv(self) -> U64Vector4: ...
    @property
    def lvuo(self) -> U64Vector4: ...
    @property
    def lvul(self) -> U64Vector4: ...
    @property
    def lvvu(self) -> U64Vector4: ...
    @property
    def lvvv(self) -> U64Vector4: ...
    @property
    def lvvo(self) -> U64Vector4: ...
    @property
    def lvvl(self) -> U64Vector4: ...
    @property
    def lvou(self) -> U64Vector4: ...
    @property
    def lvov(self) -> U64Vector4: ...
    @property
    def lvoo(self) -> U64Vector4: ...
    @property
    def lvol(self) -> U64Vector4: ...
    @property
    def lvlu(self) -> U64Vector4: ...
    @property
    def lvlv(self) -> U64Vector4: ...
    @property
    def lvlo(self) -> U64Vector4: ...
    @property
    def lvll(self) -> U64Vector4: ...
    @property
    def louu(self) -> U64Vector4: ...
    @property
    def louv(self) -> U64Vector4: ...
    @property
    def louo(self) -> U64Vector4: ...
    @property
    def loul(self) -> U64Vector4: ...
    @property
    def lovu(self) -> U64Vector4: ...
    @property
    def lovv(self) -> U64Vector4: ...
    @property
    def lovo(self) -> U64Vector4: ...
    @property
    def lovl(self) -> U64Vector4: ...
    @property
    def loou(self) -> U64Vector4: ...
    @property
    def loov(self) -> U64Vector4: ...
    @property
    def lolu(self) -> U64Vector4: ...
    @property
    def lolv(self) -> U64Vector4: ...
    @property
    def lluu(self) -> U64Vector4: ...
    @property
    def lluv(self) -> U64Vector4: ...
    @property
    def lluo(self) -> U64Vector4: ...
    @property
    def llul(self) -> U64Vector4: ...
    @property
    def llvu(self) -> U64Vector4: ...
    @property
    def llvv(self) -> U64Vector4: ...
    @property
    def llvo(self) -> U64Vector4: ...
    @property
    def llvl(self) -> U64Vector4: ...
    @property
    def llou(self) -> U64Vector4: ...
    @property
    def llov(self) -> U64Vector4: ...
    @property
    def lllu(self) -> U64Vector4: ...
    @property
    def lllv(self) -> U64Vector4: ...
    @property
    def oo(self) -> U64Vector2: ...
    @property
    def ol(self) -> U64Vector2: ...
    @property
    def lo(self) -> U64Vector2: ...
    @property
    def ll(self) -> U64Vector2: ...
    @property
    def ooo(self) -> U64Vector3: ...
    @property
    def ool(self) -> U64Vector3: ...
    @property
    def olo(self) -> U64Vector3: ...
    @property
    def oll(self) -> U64Vector3: ...
    @property
    def loo(self) -> U64Vector3: ...
    @property
    def lol(self) -> U64Vector3: ...
    @property
    def llo(self) -> U64Vector3: ...
    @property
    def lll(self) -> U64Vector3: ...
    @property
    def oooo(self) -> U64Vector4: ...
    @property
    def oool(self) -> U64Vector4: ...
    @property
    def oolo(self) -> U64Vector4: ...
    @property
    def ooll(self) -> U64Vector4: ...
    @property
    def oloo(self) -> U64Vector4: ...
    @property
    def olol(self) -> U64Vector4: ...
    @property
    def ollo(self) -> U64Vector4: ...
    @property
    def olll(self) -> U64Vector4: ...
    @property
    def looo(self) -> U64Vector4: ...
    @property
    def lool(self) -> U64Vector4: ...
    @property
    def lolo(self) -> U64Vector4: ...
    @property
    def loll(self) -> U64Vector4: ...
    @property
    def lloo(self) -> U64Vector4: ...
    @property
    def llol(self) -> U64Vector4: ...
    @property
    def lllo(self) -> U64Vector4: ...
    @property
    def llll(self) -> U64Vector4: ...
    def min(self, n: Number, /) -> U64Vector3: ...
    def max(self, n: Number, /) -> U64Vector3: ...
    def clamp(self, min: Number, max: Number, /) -> U64Vector3: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U64Vector3Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector3: ...

@final
class U64Vector3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U64Vector3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U64Vector3]: ...
    @overload
    def __getitem__(self, index: int) -> U64Vector3: ...
    @overload
    def __getitem__(self, index: slice) -> U64Vector3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector3Array: ...
    @classmethod
    def get_component_type(cls) -> type[U64Vector3]: ...

@final
class BVector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[_bool]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> _bool: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: BVector4) -> BVector4: ...
    @overload
    def __add__(self, other: Number) -> BVector4: ...
    @overload
    def __radd__(self, other: BVector4) -> BVector4: ...
    @overload
    def __radd__(self, other: Number) -> BVector4: ...
    @overload
    def __sub__(self, other: BVector4) -> BVector4: ...
    @overload
    def __sub__(self, other: Number) -> BVector4: ...
    @overload
    def __rsub__(self, other: BVector4) -> BVector4: ...
    @overload
    def __rsub__(self, other: Number) -> BVector4: ...
    @overload
    def __mul__(self, other: BVector4) -> BVector4: ...
    @overload
    def __mul__(self, other: Number) -> BVector4: ...
    @overload
    def __rmul__(self, other: BVector4) -> BVector4: ...
    @overload
    def __rmul__(self, other: Number) -> BVector4: ...
    def __neg__(self) -> BVector4: ...
    def __abs__(self) -> BVector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> _bool: ...
    @property
    def y(self) -> _bool: ...
    @property
    def z(self) -> _bool: ...
    @property
    def w(self) -> _bool: ...
    @property
    def xx(self) -> BVector2: ...
    @property
    def xy(self) -> BVector2: ...
    @property
    def xz(self) -> BVector2: ...
    @property
    def xw(self) -> BVector2: ...
    @property
    def xo(self) -> BVector2: ...
    @property
    def xl(self) -> BVector2: ...
    @property
    def yx(self) -> BVector2: ...
    @property
    def yy(self) -> BVector2: ...
    @property
    def yz(self) -> BVector2: ...
    @property
    def yw(self) -> BVector2: ...
    @property
    def yo(self) -> BVector2: ...
    @property
    def yl(self) -> BVector2: ...
    @property
    def zx(self) -> BVector2: ...
    @property
    def zy(self) -> BVector2: ...
    @property
    def zz(self) -> BVector2: ...
    @property
    def zw(self) -> BVector2: ...
    @property
    def zo(self) -> BVector2: ...
    @property
    def zl(self) -> BVector2: ...
    @property
    def wx(self) -> BVector2: ...
    @property
    def wy(self) -> BVector2: ...
    @property
    def wz(self) -> BVector2: ...
    @property
    def ww(self) -> BVector2: ...
    @property
    def wo(self) -> BVector2: ...
    @property
    def wl(self) -> BVector2: ...
    @property
    def ox(self) -> BVector2: ...
    @property
    def oy(self) -> BVector2: ...
    @property
    def oz(self) -> BVector2: ...
    @property
    def ow(self) -> BVector2: ...
    @property
    def lx(self) -> BVector2: ...
    @property
    def ly(self) -> BVector2: ...
    @property
    def lz(self) -> BVector2: ...
    @property
    def lw(self) -> BVector2: ...
    @property
    def xxx(self) -> BVector3: ...
    @property
    def xxy(self) -> BVector3: ...
    @property
    def xxz(self) -> BVector3: ...
    @property
    def xxw(self) -> BVector3: ...
    @property
    def xxo(self) -> BVector3: ...
    @property
    def xxl(self) -> BVector3: ...
    @property
    def xyx(self) -> BVector3: ...
    @property
    def xyy(self) -> BVector3: ...
    @property
    def xyz(self) -> BVector3: ...
    @property
    def xyw(self) -> BVector3: ...
    @property
    def xyo(self) -> BVector3: ...
    @property
    def xyl(self) -> BVector3: ...
    @property
    def xzx(self) -> BVector3: ...
    @property
    def xzy(self) -> BVector3: ...
    @property
    def xzz(self) -> BVector3: ...
    @property
    def xzw(self) -> BVector3: ...
    @property
    def xzo(self) -> BVector3: ...
    @property
    def xzl(self) -> BVector3: ...
    @property
    def xwx(self) -> BVector3: ...
    @property
    def xwy(self) -> BVector3: ...
    @property
    def xwz(self) -> BVector3: ...
    @property
    def xww(self) -> BVector3: ...
    @property
    def xwo(self) -> BVector3: ...
    @property
    def xwl(self) -> BVector3: ...
    @property
    def xox(self) -> BVector3: ...
    @property
    def xoy(self) -> BVector3: ...
    @property
    def xoz(self) -> BVector3: ...
    @property
    def xow(self) -> BVector3: ...
    @property
    def xoo(self) -> BVector3: ...
    @property
    def xol(self) -> BVector3: ...
    @property
    def xlx(self) -> BVector3: ...
    @property
    def xly(self) -> BVector3: ...
    @property
    def xlz(self) -> BVector3: ...
    @property
    def xlw(self) -> BVector3: ...
    @property
    def xlo(self) -> BVector3: ...
    @property
    def xll(self) -> BVector3: ...
    @property
    def yxx(self) -> BVector3: ...
    @property
    def yxy(self) -> BVector3: ...
    @property
    def yxz(self) -> BVector3: ...
    @property
    def yxw(self) -> BVector3: ...
    @property
    def yxo(self) -> BVector3: ...
    @property
    def yxl(self) -> BVector3: ...
    @property
    def yyx(self) -> BVector3: ...
    @property
    def yyy(self) -> BVector3: ...
    @property
    def yyz(self) -> BVector3: ...
    @property
    def yyw(self) -> BVector3: ...
    @property
    def yyo(self) -> BVector3: ...
    @property
    def yyl(self) -> BVector3: ...
    @property
    def yzx(self) -> BVector3: ...
    @property
    def yzy(self) -> BVector3: ...
    @property
    def yzz(self) -> BVector3: ...
    @property
    def yzw(self) -> BVector3: ...
    @property
    def yzo(self) -> BVector3: ...
    @property
    def yzl(self) -> BVector3: ...
    @property
    def ywx(self) -> BVector3: ...
    @property
    def ywy(self) -> BVector3: ...
    @property
    def ywz(self) -> BVector3: ...
    @property
    def yww(self) -> BVector3: ...
    @property
    def ywo(self) -> BVector3: ...
    @property
    def ywl(self) -> BVector3: ...
    @property
    def yox(self) -> BVector3: ...
    @property
    def yoy(self) -> BVector3: ...
    @property
    def yoz(self) -> BVector3: ...
    @property
    def yow(self) -> BVector3: ...
    @property
    def yoo(self) -> BVector3: ...
    @property
    def yol(self) -> BVector3: ...
    @property
    def ylx(self) -> BVector3: ...
    @property
    def yly(self) -> BVector3: ...
    @property
    def ylz(self) -> BVector3: ...
    @property
    def ylw(self) -> BVector3: ...
    @property
    def ylo(self) -> BVector3: ...
    @property
    def yll(self) -> BVector3: ...
    @property
    def zxx(self) -> BVector3: ...
    @property
    def zxy(self) -> BVector3: ...
    @property
    def zxz(self) -> BVector3: ...
    @property
    def zxw(self) -> BVector3: ...
    @property
    def zxo(self) -> BVector3: ...
    @property
    def zxl(self) -> BVector3: ...
    @property
    def zyx(self) -> BVector3: ...
    @property
    def zyy(self) -> BVector3: ...
    @property
    def zyz(self) -> BVector3: ...
    @property
    def zyw(self) -> BVector3: ...
    @property
    def zyo(self) -> BVector3: ...
    @property
    def zyl(self) -> BVector3: ...
    @property
    def zzx(self) -> BVector3: ...
    @property
    def zzy(self) -> BVector3: ...
    @property
    def zzz(self) -> BVector3: ...
    @property
    def zzw(self) -> BVector3: ...
    @property
    def zzo(self) -> BVector3: ...
    @property
    def zzl(self) -> BVector3: ...
    @property
    def zwx(self) -> BVector3: ...
    @property
    def zwy(self) -> BVector3: ...
    @property
    def zwz(self) -> BVector3: ...
    @property
    def zww(self) -> BVector3: ...
    @property
    def zwo(self) -> BVector3: ...
    @property
    def zwl(self) -> BVector3: ...
    @property
    def zox(self) -> BVector3: ...
    @property
    def zoy(self) -> BVector3: ...
    @property
    def zoz(self) -> BVector3: ...
    @property
    def zow(self) -> BVector3: ...
    @property
    def zoo(self) -> BVector3: ...
    @property
    def zol(self) -> BVector3: ...
    @property
    def zlx(self) -> BVector3: ...
    @property
    def zly(self) -> BVector3: ...
    @property
    def zlz(self) -> BVector3: ...
    @property
    def zlw(self) -> BVector3: ...
    @property
    def zlo(self) -> BVector3: ...
    @property
    def zll(self) -> BVector3: ...
    @property
    def wxx(self) -> BVector3: ...
    @property
    def wxy(self) -> BVector3: ...
    @property
    def wxz(self) -> BVector3: ...
    @property
    def wxw(self) -> BVector3: ...
    @property
    def wxo(self) -> BVector3: ...
    @property
    def wxl(self) -> BVector3: ...
    @property
    def wyx(self) -> BVector3: ...
    @property
    def wyy(self) -> BVector3: ...
    @property
    def wyz(self) -> BVector3: ...
    @property
    def wyw(self) -> BVector3: ...
    @property
    def wyo(self) -> BVector3: ...
    @property
    def wyl(self) -> BVector3: ...
    @property
    def wzx(self) -> BVector3: ...
    @property
    def wzy(self) -> BVector3: ...
    @property
    def wzz(self) -> BVector3: ...
    @property
    def wzw(self) -> BVector3: ...
    @property
    def wzo(self) -> BVector3: ...
    @property
    def wzl(self) -> BVector3: ...
    @property
    def wwx(self) -> BVector3: ...
    @property
    def wwy(self) -> BVector3: ...
    @property
    def wwz(self) -> BVector3: ...
    @property
    def www(self) -> BVector3: ...
    @property
    def wwo(self) -> BVector3: ...
    @property
    def wwl(self) -> BVector3: ...
    @property
    def wox(self) -> BVector3: ...
    @property
    def woy(self) -> BVector3: ...
    @property
    def woz(self) -> BVector3: ...
    @property
    def wow(self) -> BVector3: ...
    @property
    def woo(self) -> BVector3: ...
    @property
    def wol(self) -> BVector3: ...
    @property
    def wlx(self) -> BVector3: ...
    @property
    def wly(self) -> BVector3: ...
    @property
    def wlz(self) -> BVector3: ...
    @property
    def wlw(self) -> BVector3: ...
    @property
    def wlo(self) -> BVector3: ...
    @property
    def wll(self) -> BVector3: ...
    @property
    def oxx(self) -> BVector3: ...
    @property
    def oxy(self) -> BVector3: ...
    @property
    def oxz(self) -> BVector3: ...
    @property
    def oxw(self) -> BVector3: ...
    @property
    def oxo(self) -> BVector3: ...
    @property
    def oxl(self) -> BVector3: ...
    @property
    def oyx(self) -> BVector3: ...
    @property
    def oyy(self) -> BVector3: ...
    @property
    def oyz(self) -> BVector3: ...
    @property
    def oyw(self) -> BVector3: ...
    @property
    def oyo(self) -> BVector3: ...
    @property
    def oyl(self) -> BVector3: ...
    @property
    def ozx(self) -> BVector3: ...
    @property
    def ozy(self) -> BVector3: ...
    @property
    def ozz(self) -> BVector3: ...
    @property
    def ozw(self) -> BVector3: ...
    @property
    def ozo(self) -> BVector3: ...
    @property
    def ozl(self) -> BVector3: ...
    @property
    def owx(self) -> BVector3: ...
    @property
    def owy(self) -> BVector3: ...
    @property
    def owz(self) -> BVector3: ...
    @property
    def oww(self) -> BVector3: ...
    @property
    def owo(self) -> BVector3: ...
    @property
    def owl(self) -> BVector3: ...
    @property
    def oox(self) -> BVector3: ...
    @property
    def ooy(self) -> BVector3: ...
    @property
    def ooz(self) -> BVector3: ...
    @property
    def oow(self) -> BVector3: ...
    @property
    def olx(self) -> BVector3: ...
    @property
    def oly(self) -> BVector3: ...
    @property
    def olz(self) -> BVector3: ...
    @property
    def olw(self) -> BVector3: ...
    @property
    def lxx(self) -> BVector3: ...
    @property
    def lxy(self) -> BVector3: ...
    @property
    def lxz(self) -> BVector3: ...
    @property
    def lxw(self) -> BVector3: ...
    @property
    def lxo(self) -> BVector3: ...
    @property
    def lxl(self) -> BVector3: ...
    @property
    def lyx(self) -> BVector3: ...
    @property
    def lyy(self) -> BVector3: ...
    @property
    def lyz(self) -> BVector3: ...
    @property
    def lyw(self) -> BVector3: ...
    @property
    def lyo(self) -> BVector3: ...
    @property
    def lyl(self) -> BVector3: ...
    @property
    def lzx(self) -> BVector3: ...
    @property
    def lzy(self) -> BVector3: ...
    @property
    def lzz(self) -> BVector3: ...
    @property
    def lzw(self) -> BVector3: ...
    @property
    def lzo(self) -> BVector3: ...
    @property
    def lzl(self) -> BVector3: ...
    @property
    def lwx(self) -> BVector3: ...
    @property
    def lwy(self) -> BVector3: ...
    @property
    def lwz(self) -> BVector3: ...
    @property
    def lww(self) -> BVector3: ...
    @property
    def lwo(self) -> BVector3: ...
    @property
    def lwl(self) -> BVector3: ...
    @property
    def lox(self) -> BVector3: ...
    @property
    def loy(self) -> BVector3: ...
    @property
    def loz(self) -> BVector3: ...
    @property
    def low(self) -> BVector3: ...
    @property
    def llx(self) -> BVector3: ...
    @property
    def lly(self) -> BVector3: ...
    @property
    def llz(self) -> BVector3: ...
    @property
    def llw(self) -> BVector3: ...
    @property
    def xxxx(self) -> BVector4: ...
    @property
    def xxxy(self) -> BVector4: ...
    @property
    def xxxz(self) -> BVector4: ...
    @property
    def xxxw(self) -> BVector4: ...
    @property
    def xxxo(self) -> BVector4: ...
    @property
    def xxxl(self) -> BVector4: ...
    @property
    def xxyx(self) -> BVector4: ...
    @property
    def xxyy(self) -> BVector4: ...
    @property
    def xxyz(self) -> BVector4: ...
    @property
    def xxyw(self) -> BVector4: ...
    @property
    def xxyo(self) -> BVector4: ...
    @property
    def xxyl(self) -> BVector4: ...
    @property
    def xxzx(self) -> BVector4: ...
    @property
    def xxzy(self) -> BVector4: ...
    @property
    def xxzz(self) -> BVector4: ...
    @property
    def xxzw(self) -> BVector4: ...
    @property
    def xxzo(self) -> BVector4: ...
    @property
    def xxzl(self) -> BVector4: ...
    @property
    def xxwx(self) -> BVector4: ...
    @property
    def xxwy(self) -> BVector4: ...
    @property
    def xxwz(self) -> BVector4: ...
    @property
    def xxww(self) -> BVector4: ...
    @property
    def xxwo(self) -> BVector4: ...
    @property
    def xxwl(self) -> BVector4: ...
    @property
    def xxox(self) -> BVector4: ...
    @property
    def xxoy(self) -> BVector4: ...
    @property
    def xxoz(self) -> BVector4: ...
    @property
    def xxow(self) -> BVector4: ...
    @property
    def xxoo(self) -> BVector4: ...
    @property
    def xxol(self) -> BVector4: ...
    @property
    def xxlx(self) -> BVector4: ...
    @property
    def xxly(self) -> BVector4: ...
    @property
    def xxlz(self) -> BVector4: ...
    @property
    def xxlw(self) -> BVector4: ...
    @property
    def xxlo(self) -> BVector4: ...
    @property
    def xxll(self) -> BVector4: ...
    @property
    def xyxx(self) -> BVector4: ...
    @property
    def xyxy(self) -> BVector4: ...
    @property
    def xyxz(self) -> BVector4: ...
    @property
    def xyxw(self) -> BVector4: ...
    @property
    def xyxo(self) -> BVector4: ...
    @property
    def xyxl(self) -> BVector4: ...
    @property
    def xyyx(self) -> BVector4: ...
    @property
    def xyyy(self) -> BVector4: ...
    @property
    def xyyz(self) -> BVector4: ...
    @property
    def xyyw(self) -> BVector4: ...
    @property
    def xyyo(self) -> BVector4: ...
    @property
    def xyyl(self) -> BVector4: ...
    @property
    def xyzx(self) -> BVector4: ...
    @property
    def xyzy(self) -> BVector4: ...
    @property
    def xyzz(self) -> BVector4: ...
    @property
    def xyzw(self) -> BVector4: ...
    @property
    def xyzo(self) -> BVector4: ...
    @property
    def xyzl(self) -> BVector4: ...
    @property
    def xywx(self) -> BVector4: ...
    @property
    def xywy(self) -> BVector4: ...
    @property
    def xywz(self) -> BVector4: ...
    @property
    def xyww(self) -> BVector4: ...
    @property
    def xywo(self) -> BVector4: ...
    @property
    def xywl(self) -> BVector4: ...
    @property
    def xyox(self) -> BVector4: ...
    @property
    def xyoy(self) -> BVector4: ...
    @property
    def xyoz(self) -> BVector4: ...
    @property
    def xyow(self) -> BVector4: ...
    @property
    def xyoo(self) -> BVector4: ...
    @property
    def xyol(self) -> BVector4: ...
    @property
    def xylx(self) -> BVector4: ...
    @property
    def xyly(self) -> BVector4: ...
    @property
    def xylz(self) -> BVector4: ...
    @property
    def xylw(self) -> BVector4: ...
    @property
    def xylo(self) -> BVector4: ...
    @property
    def xyll(self) -> BVector4: ...
    @property
    def xzxx(self) -> BVector4: ...
    @property
    def xzxy(self) -> BVector4: ...
    @property
    def xzxz(self) -> BVector4: ...
    @property
    def xzxw(self) -> BVector4: ...
    @property
    def xzxo(self) -> BVector4: ...
    @property
    def xzxl(self) -> BVector4: ...
    @property
    def xzyx(self) -> BVector4: ...
    @property
    def xzyy(self) -> BVector4: ...
    @property
    def xzyz(self) -> BVector4: ...
    @property
    def xzyw(self) -> BVector4: ...
    @property
    def xzyo(self) -> BVector4: ...
    @property
    def xzyl(self) -> BVector4: ...
    @property
    def xzzx(self) -> BVector4: ...
    @property
    def xzzy(self) -> BVector4: ...
    @property
    def xzzz(self) -> BVector4: ...
    @property
    def xzzw(self) -> BVector4: ...
    @property
    def xzzo(self) -> BVector4: ...
    @property
    def xzzl(self) -> BVector4: ...
    @property
    def xzwx(self) -> BVector4: ...
    @property
    def xzwy(self) -> BVector4: ...
    @property
    def xzwz(self) -> BVector4: ...
    @property
    def xzww(self) -> BVector4: ...
    @property
    def xzwo(self) -> BVector4: ...
    @property
    def xzwl(self) -> BVector4: ...
    @property
    def xzox(self) -> BVector4: ...
    @property
    def xzoy(self) -> BVector4: ...
    @property
    def xzoz(self) -> BVector4: ...
    @property
    def xzow(self) -> BVector4: ...
    @property
    def xzoo(self) -> BVector4: ...
    @property
    def xzol(self) -> BVector4: ...
    @property
    def xzlx(self) -> BVector4: ...
    @property
    def xzly(self) -> BVector4: ...
    @property
    def xzlz(self) -> BVector4: ...
    @property
    def xzlw(self) -> BVector4: ...
    @property
    def xzlo(self) -> BVector4: ...
    @property
    def xzll(self) -> BVector4: ...
    @property
    def xwxx(self) -> BVector4: ...
    @property
    def xwxy(self) -> BVector4: ...
    @property
    def xwxz(self) -> BVector4: ...
    @property
    def xwxw(self) -> BVector4: ...
    @property
    def xwxo(self) -> BVector4: ...
    @property
    def xwxl(self) -> BVector4: ...
    @property
    def xwyx(self) -> BVector4: ...
    @property
    def xwyy(self) -> BVector4: ...
    @property
    def xwyz(self) -> BVector4: ...
    @property
    def xwyw(self) -> BVector4: ...
    @property
    def xwyo(self) -> BVector4: ...
    @property
    def xwyl(self) -> BVector4: ...
    @property
    def xwzx(self) -> BVector4: ...
    @property
    def xwzy(self) -> BVector4: ...
    @property
    def xwzz(self) -> BVector4: ...
    @property
    def xwzw(self) -> BVector4: ...
    @property
    def xwzo(self) -> BVector4: ...
    @property
    def xwzl(self) -> BVector4: ...
    @property
    def xwwx(self) -> BVector4: ...
    @property
    def xwwy(self) -> BVector4: ...
    @property
    def xwwz(self) -> BVector4: ...
    @property
    def xwww(self) -> BVector4: ...
    @property
    def xwwo(self) -> BVector4: ...
    @property
    def xwwl(self) -> BVector4: ...
    @property
    def xwox(self) -> BVector4: ...
    @property
    def xwoy(self) -> BVector4: ...
    @property
    def xwoz(self) -> BVector4: ...
    @property
    def xwow(self) -> BVector4: ...
    @property
    def xwoo(self) -> BVector4: ...
    @property
    def xwol(self) -> BVector4: ...
    @property
    def xwlx(self) -> BVector4: ...
    @property
    def xwly(self) -> BVector4: ...
    @property
    def xwlz(self) -> BVector4: ...
    @property
    def xwlw(self) -> BVector4: ...
    @property
    def xwlo(self) -> BVector4: ...
    @property
    def xwll(self) -> BVector4: ...
    @property
    def xoxx(self) -> BVector4: ...
    @property
    def xoxy(self) -> BVector4: ...
    @property
    def xoxz(self) -> BVector4: ...
    @property
    def xoxw(self) -> BVector4: ...
    @property
    def xoxo(self) -> BVector4: ...
    @property
    def xoxl(self) -> BVector4: ...
    @property
    def xoyx(self) -> BVector4: ...
    @property
    def xoyy(self) -> BVector4: ...
    @property
    def xoyz(self) -> BVector4: ...
    @property
    def xoyw(self) -> BVector4: ...
    @property
    def xoyo(self) -> BVector4: ...
    @property
    def xoyl(self) -> BVector4: ...
    @property
    def xozx(self) -> BVector4: ...
    @property
    def xozy(self) -> BVector4: ...
    @property
    def xozz(self) -> BVector4: ...
    @property
    def xozw(self) -> BVector4: ...
    @property
    def xozo(self) -> BVector4: ...
    @property
    def xozl(self) -> BVector4: ...
    @property
    def xowx(self) -> BVector4: ...
    @property
    def xowy(self) -> BVector4: ...
    @property
    def xowz(self) -> BVector4: ...
    @property
    def xoww(self) -> BVector4: ...
    @property
    def xowo(self) -> BVector4: ...
    @property
    def xowl(self) -> BVector4: ...
    @property
    def xoox(self) -> BVector4: ...
    @property
    def xooy(self) -> BVector4: ...
    @property
    def xooz(self) -> BVector4: ...
    @property
    def xoow(self) -> BVector4: ...
    @property
    def xooo(self) -> BVector4: ...
    @property
    def xool(self) -> BVector4: ...
    @property
    def xolx(self) -> BVector4: ...
    @property
    def xoly(self) -> BVector4: ...
    @property
    def xolz(self) -> BVector4: ...
    @property
    def xolw(self) -> BVector4: ...
    @property
    def xolo(self) -> BVector4: ...
    @property
    def xoll(self) -> BVector4: ...
    @property
    def xlxx(self) -> BVector4: ...
    @property
    def xlxy(self) -> BVector4: ...
    @property
    def xlxz(self) -> BVector4: ...
    @property
    def xlxw(self) -> BVector4: ...
    @property
    def xlxo(self) -> BVector4: ...
    @property
    def xlxl(self) -> BVector4: ...
    @property
    def xlyx(self) -> BVector4: ...
    @property
    def xlyy(self) -> BVector4: ...
    @property
    def xlyz(self) -> BVector4: ...
    @property
    def xlyw(self) -> BVector4: ...
    @property
    def xlyo(self) -> BVector4: ...
    @property
    def xlyl(self) -> BVector4: ...
    @property
    def xlzx(self) -> BVector4: ...
    @property
    def xlzy(self) -> BVector4: ...
    @property
    def xlzz(self) -> BVector4: ...
    @property
    def xlzw(self) -> BVector4: ...
    @property
    def xlzo(self) -> BVector4: ...
    @property
    def xlzl(self) -> BVector4: ...
    @property
    def xlwx(self) -> BVector4: ...
    @property
    def xlwy(self) -> BVector4: ...
    @property
    def xlwz(self) -> BVector4: ...
    @property
    def xlww(self) -> BVector4: ...
    @property
    def xlwo(self) -> BVector4: ...
    @property
    def xlwl(self) -> BVector4: ...
    @property
    def xlox(self) -> BVector4: ...
    @property
    def xloy(self) -> BVector4: ...
    @property
    def xloz(self) -> BVector4: ...
    @property
    def xlow(self) -> BVector4: ...
    @property
    def xloo(self) -> BVector4: ...
    @property
    def xlol(self) -> BVector4: ...
    @property
    def xllx(self) -> BVector4: ...
    @property
    def xlly(self) -> BVector4: ...
    @property
    def xllz(self) -> BVector4: ...
    @property
    def xllw(self) -> BVector4: ...
    @property
    def xllo(self) -> BVector4: ...
    @property
    def xlll(self) -> BVector4: ...
    @property
    def yxxx(self) -> BVector4: ...
    @property
    def yxxy(self) -> BVector4: ...
    @property
    def yxxz(self) -> BVector4: ...
    @property
    def yxxw(self) -> BVector4: ...
    @property
    def yxxo(self) -> BVector4: ...
    @property
    def yxxl(self) -> BVector4: ...
    @property
    def yxyx(self) -> BVector4: ...
    @property
    def yxyy(self) -> BVector4: ...
    @property
    def yxyz(self) -> BVector4: ...
    @property
    def yxyw(self) -> BVector4: ...
    @property
    def yxyo(self) -> BVector4: ...
    @property
    def yxyl(self) -> BVector4: ...
    @property
    def yxzx(self) -> BVector4: ...
    @property
    def yxzy(self) -> BVector4: ...
    @property
    def yxzz(self) -> BVector4: ...
    @property
    def yxzw(self) -> BVector4: ...
    @property
    def yxzo(self) -> BVector4: ...
    @property
    def yxzl(self) -> BVector4: ...
    @property
    def yxwx(self) -> BVector4: ...
    @property
    def yxwy(self) -> BVector4: ...
    @property
    def yxwz(self) -> BVector4: ...
    @property
    def yxww(self) -> BVector4: ...
    @property
    def yxwo(self) -> BVector4: ...
    @property
    def yxwl(self) -> BVector4: ...
    @property
    def yxox(self) -> BVector4: ...
    @property
    def yxoy(self) -> BVector4: ...
    @property
    def yxoz(self) -> BVector4: ...
    @property
    def yxow(self) -> BVector4: ...
    @property
    def yxoo(self) -> BVector4: ...
    @property
    def yxol(self) -> BVector4: ...
    @property
    def yxlx(self) -> BVector4: ...
    @property
    def yxly(self) -> BVector4: ...
    @property
    def yxlz(self) -> BVector4: ...
    @property
    def yxlw(self) -> BVector4: ...
    @property
    def yxlo(self) -> BVector4: ...
    @property
    def yxll(self) -> BVector4: ...
    @property
    def yyxx(self) -> BVector4: ...
    @property
    def yyxy(self) -> BVector4: ...
    @property
    def yyxz(self) -> BVector4: ...
    @property
    def yyxw(self) -> BVector4: ...
    @property
    def yyxo(self) -> BVector4: ...
    @property
    def yyxl(self) -> BVector4: ...
    @property
    def yyyx(self) -> BVector4: ...
    @property
    def yyyy(self) -> BVector4: ...
    @property
    def yyyz(self) -> BVector4: ...
    @property
    def yyyw(self) -> BVector4: ...
    @property
    def yyyo(self) -> BVector4: ...
    @property
    def yyyl(self) -> BVector4: ...
    @property
    def yyzx(self) -> BVector4: ...
    @property
    def yyzy(self) -> BVector4: ...
    @property
    def yyzz(self) -> BVector4: ...
    @property
    def yyzw(self) -> BVector4: ...
    @property
    def yyzo(self) -> BVector4: ...
    @property
    def yyzl(self) -> BVector4: ...
    @property
    def yywx(self) -> BVector4: ...
    @property
    def yywy(self) -> BVector4: ...
    @property
    def yywz(self) -> BVector4: ...
    @property
    def yyww(self) -> BVector4: ...
    @property
    def yywo(self) -> BVector4: ...
    @property
    def yywl(self) -> BVector4: ...
    @property
    def yyox(self) -> BVector4: ...
    @property
    def yyoy(self) -> BVector4: ...
    @property
    def yyoz(self) -> BVector4: ...
    @property
    def yyow(self) -> BVector4: ...
    @property
    def yyoo(self) -> BVector4: ...
    @property
    def yyol(self) -> BVector4: ...
    @property
    def yylx(self) -> BVector4: ...
    @property
    def yyly(self) -> BVector4: ...
    @property
    def yylz(self) -> BVector4: ...
    @property
    def yylw(self) -> BVector4: ...
    @property
    def yylo(self) -> BVector4: ...
    @property
    def yyll(self) -> BVector4: ...
    @property
    def yzxx(self) -> BVector4: ...
    @property
    def yzxy(self) -> BVector4: ...
    @property
    def yzxz(self) -> BVector4: ...
    @property
    def yzxw(self) -> BVector4: ...
    @property
    def yzxo(self) -> BVector4: ...
    @property
    def yzxl(self) -> BVector4: ...
    @property
    def yzyx(self) -> BVector4: ...
    @property
    def yzyy(self) -> BVector4: ...
    @property
    def yzyz(self) -> BVector4: ...
    @property
    def yzyw(self) -> BVector4: ...
    @property
    def yzyo(self) -> BVector4: ...
    @property
    def yzyl(self) -> BVector4: ...
    @property
    def yzzx(self) -> BVector4: ...
    @property
    def yzzy(self) -> BVector4: ...
    @property
    def yzzz(self) -> BVector4: ...
    @property
    def yzzw(self) -> BVector4: ...
    @property
    def yzzo(self) -> BVector4: ...
    @property
    def yzzl(self) -> BVector4: ...
    @property
    def yzwx(self) -> BVector4: ...
    @property
    def yzwy(self) -> BVector4: ...
    @property
    def yzwz(self) -> BVector4: ...
    @property
    def yzww(self) -> BVector4: ...
    @property
    def yzwo(self) -> BVector4: ...
    @property
    def yzwl(self) -> BVector4: ...
    @property
    def yzox(self) -> BVector4: ...
    @property
    def yzoy(self) -> BVector4: ...
    @property
    def yzoz(self) -> BVector4: ...
    @property
    def yzow(self) -> BVector4: ...
    @property
    def yzoo(self) -> BVector4: ...
    @property
    def yzol(self) -> BVector4: ...
    @property
    def yzlx(self) -> BVector4: ...
    @property
    def yzly(self) -> BVector4: ...
    @property
    def yzlz(self) -> BVector4: ...
    @property
    def yzlw(self) -> BVector4: ...
    @property
    def yzlo(self) -> BVector4: ...
    @property
    def yzll(self) -> BVector4: ...
    @property
    def ywxx(self) -> BVector4: ...
    @property
    def ywxy(self) -> BVector4: ...
    @property
    def ywxz(self) -> BVector4: ...
    @property
    def ywxw(self) -> BVector4: ...
    @property
    def ywxo(self) -> BVector4: ...
    @property
    def ywxl(self) -> BVector4: ...
    @property
    def ywyx(self) -> BVector4: ...
    @property
    def ywyy(self) -> BVector4: ...
    @property
    def ywyz(self) -> BVector4: ...
    @property
    def ywyw(self) -> BVector4: ...
    @property
    def ywyo(self) -> BVector4: ...
    @property
    def ywyl(self) -> BVector4: ...
    @property
    def ywzx(self) -> BVector4: ...
    @property
    def ywzy(self) -> BVector4: ...
    @property
    def ywzz(self) -> BVector4: ...
    @property
    def ywzw(self) -> BVector4: ...
    @property
    def ywzo(self) -> BVector4: ...
    @property
    def ywzl(self) -> BVector4: ...
    @property
    def ywwx(self) -> BVector4: ...
    @property
    def ywwy(self) -> BVector4: ...
    @property
    def ywwz(self) -> BVector4: ...
    @property
    def ywww(self) -> BVector4: ...
    @property
    def ywwo(self) -> BVector4: ...
    @property
    def ywwl(self) -> BVector4: ...
    @property
    def ywox(self) -> BVector4: ...
    @property
    def ywoy(self) -> BVector4: ...
    @property
    def ywoz(self) -> BVector4: ...
    @property
    def ywow(self) -> BVector4: ...
    @property
    def ywoo(self) -> BVector4: ...
    @property
    def ywol(self) -> BVector4: ...
    @property
    def ywlx(self) -> BVector4: ...
    @property
    def ywly(self) -> BVector4: ...
    @property
    def ywlz(self) -> BVector4: ...
    @property
    def ywlw(self) -> BVector4: ...
    @property
    def ywlo(self) -> BVector4: ...
    @property
    def ywll(self) -> BVector4: ...
    @property
    def yoxx(self) -> BVector4: ...
    @property
    def yoxy(self) -> BVector4: ...
    @property
    def yoxz(self) -> BVector4: ...
    @property
    def yoxw(self) -> BVector4: ...
    @property
    def yoxo(self) -> BVector4: ...
    @property
    def yoxl(self) -> BVector4: ...
    @property
    def yoyx(self) -> BVector4: ...
    @property
    def yoyy(self) -> BVector4: ...
    @property
    def yoyz(self) -> BVector4: ...
    @property
    def yoyw(self) -> BVector4: ...
    @property
    def yoyo(self) -> BVector4: ...
    @property
    def yoyl(self) -> BVector4: ...
    @property
    def yozx(self) -> BVector4: ...
    @property
    def yozy(self) -> BVector4: ...
    @property
    def yozz(self) -> BVector4: ...
    @property
    def yozw(self) -> BVector4: ...
    @property
    def yozo(self) -> BVector4: ...
    @property
    def yozl(self) -> BVector4: ...
    @property
    def yowx(self) -> BVector4: ...
    @property
    def yowy(self) -> BVector4: ...
    @property
    def yowz(self) -> BVector4: ...
    @property
    def yoww(self) -> BVector4: ...
    @property
    def yowo(self) -> BVector4: ...
    @property
    def yowl(self) -> BVector4: ...
    @property
    def yoox(self) -> BVector4: ...
    @property
    def yooy(self) -> BVector4: ...
    @property
    def yooz(self) -> BVector4: ...
    @property
    def yoow(self) -> BVector4: ...
    @property
    def yooo(self) -> BVector4: ...
    @property
    def yool(self) -> BVector4: ...
    @property
    def yolx(self) -> BVector4: ...
    @property
    def yoly(self) -> BVector4: ...
    @property
    def yolz(self) -> BVector4: ...
    @property
    def yolw(self) -> BVector4: ...
    @property
    def yolo(self) -> BVector4: ...
    @property
    def yoll(self) -> BVector4: ...
    @property
    def ylxx(self) -> BVector4: ...
    @property
    def ylxy(self) -> BVector4: ...
    @property
    def ylxz(self) -> BVector4: ...
    @property
    def ylxw(self) -> BVector4: ...
    @property
    def ylxo(self) -> BVector4: ...
    @property
    def ylxl(self) -> BVector4: ...
    @property
    def ylyx(self) -> BVector4: ...
    @property
    def ylyy(self) -> BVector4: ...
    @property
    def ylyz(self) -> BVector4: ...
    @property
    def ylyw(self) -> BVector4: ...
    @property
    def ylyo(self) -> BVector4: ...
    @property
    def ylyl(self) -> BVector4: ...
    @property
    def ylzx(self) -> BVector4: ...
    @property
    def ylzy(self) -> BVector4: ...
    @property
    def ylzz(self) -> BVector4: ...
    @property
    def ylzw(self) -> BVector4: ...
    @property
    def ylzo(self) -> BVector4: ...
    @property
    def ylzl(self) -> BVector4: ...
    @property
    def ylwx(self) -> BVector4: ...
    @property
    def ylwy(self) -> BVector4: ...
    @property
    def ylwz(self) -> BVector4: ...
    @property
    def ylww(self) -> BVector4: ...
    @property
    def ylwo(self) -> BVector4: ...
    @property
    def ylwl(self) -> BVector4: ...
    @property
    def ylox(self) -> BVector4: ...
    @property
    def yloy(self) -> BVector4: ...
    @property
    def yloz(self) -> BVector4: ...
    @property
    def ylow(self) -> BVector4: ...
    @property
    def yloo(self) -> BVector4: ...
    @property
    def ylol(self) -> BVector4: ...
    @property
    def yllx(self) -> BVector4: ...
    @property
    def ylly(self) -> BVector4: ...
    @property
    def yllz(self) -> BVector4: ...
    @property
    def yllw(self) -> BVector4: ...
    @property
    def yllo(self) -> BVector4: ...
    @property
    def ylll(self) -> BVector4: ...
    @property
    def zxxx(self) -> BVector4: ...
    @property
    def zxxy(self) -> BVector4: ...
    @property
    def zxxz(self) -> BVector4: ...
    @property
    def zxxw(self) -> BVector4: ...
    @property
    def zxxo(self) -> BVector4: ...
    @property
    def zxxl(self) -> BVector4: ...
    @property
    def zxyx(self) -> BVector4: ...
    @property
    def zxyy(self) -> BVector4: ...
    @property
    def zxyz(self) -> BVector4: ...
    @property
    def zxyw(self) -> BVector4: ...
    @property
    def zxyo(self) -> BVector4: ...
    @property
    def zxyl(self) -> BVector4: ...
    @property
    def zxzx(self) -> BVector4: ...
    @property
    def zxzy(self) -> BVector4: ...
    @property
    def zxzz(self) -> BVector4: ...
    @property
    def zxzw(self) -> BVector4: ...
    @property
    def zxzo(self) -> BVector4: ...
    @property
    def zxzl(self) -> BVector4: ...
    @property
    def zxwx(self) -> BVector4: ...
    @property
    def zxwy(self) -> BVector4: ...
    @property
    def zxwz(self) -> BVector4: ...
    @property
    def zxww(self) -> BVector4: ...
    @property
    def zxwo(self) -> BVector4: ...
    @property
    def zxwl(self) -> BVector4: ...
    @property
    def zxox(self) -> BVector4: ...
    @property
    def zxoy(self) -> BVector4: ...
    @property
    def zxoz(self) -> BVector4: ...
    @property
    def zxow(self) -> BVector4: ...
    @property
    def zxoo(self) -> BVector4: ...
    @property
    def zxol(self) -> BVector4: ...
    @property
    def zxlx(self) -> BVector4: ...
    @property
    def zxly(self) -> BVector4: ...
    @property
    def zxlz(self) -> BVector4: ...
    @property
    def zxlw(self) -> BVector4: ...
    @property
    def zxlo(self) -> BVector4: ...
    @property
    def zxll(self) -> BVector4: ...
    @property
    def zyxx(self) -> BVector4: ...
    @property
    def zyxy(self) -> BVector4: ...
    @property
    def zyxz(self) -> BVector4: ...
    @property
    def zyxw(self) -> BVector4: ...
    @property
    def zyxo(self) -> BVector4: ...
    @property
    def zyxl(self) -> BVector4: ...
    @property
    def zyyx(self) -> BVector4: ...
    @property
    def zyyy(self) -> BVector4: ...
    @property
    def zyyz(self) -> BVector4: ...
    @property
    def zyyw(self) -> BVector4: ...
    @property
    def zyyo(self) -> BVector4: ...
    @property
    def zyyl(self) -> BVector4: ...
    @property
    def zyzx(self) -> BVector4: ...
    @property
    def zyzy(self) -> BVector4: ...
    @property
    def zyzz(self) -> BVector4: ...
    @property
    def zyzw(self) -> BVector4: ...
    @property
    def zyzo(self) -> BVector4: ...
    @property
    def zyzl(self) -> BVector4: ...
    @property
    def zywx(self) -> BVector4: ...
    @property
    def zywy(self) -> BVector4: ...
    @property
    def zywz(self) -> BVector4: ...
    @property
    def zyww(self) -> BVector4: ...
    @property
    def zywo(self) -> BVector4: ...
    @property
    def zywl(self) -> BVector4: ...
    @property
    def zyox(self) -> BVector4: ...
    @property
    def zyoy(self) -> BVector4: ...
    @property
    def zyoz(self) -> BVector4: ...
    @property
    def zyow(self) -> BVector4: ...
    @property
    def zyoo(self) -> BVector4: ...
    @property
    def zyol(self) -> BVector4: ...
    @property
    def zylx(self) -> BVector4: ...
    @property
    def zyly(self) -> BVector4: ...
    @property
    def zylz(self) -> BVector4: ...
    @property
    def zylw(self) -> BVector4: ...
    @property
    def zylo(self) -> BVector4: ...
    @property
    def zyll(self) -> BVector4: ...
    @property
    def zzxx(self) -> BVector4: ...
    @property
    def zzxy(self) -> BVector4: ...
    @property
    def zzxz(self) -> BVector4: ...
    @property
    def zzxw(self) -> BVector4: ...
    @property
    def zzxo(self) -> BVector4: ...
    @property
    def zzxl(self) -> BVector4: ...
    @property
    def zzyx(self) -> BVector4: ...
    @property
    def zzyy(self) -> BVector4: ...
    @property
    def zzyz(self) -> BVector4: ...
    @property
    def zzyw(self) -> BVector4: ...
    @property
    def zzyo(self) -> BVector4: ...
    @property
    def zzyl(self) -> BVector4: ...
    @property
    def zzzx(self) -> BVector4: ...
    @property
    def zzzy(self) -> BVector4: ...
    @property
    def zzzz(self) -> BVector4: ...
    @property
    def zzzw(self) -> BVector4: ...
    @property
    def zzzo(self) -> BVector4: ...
    @property
    def zzzl(self) -> BVector4: ...
    @property
    def zzwx(self) -> BVector4: ...
    @property
    def zzwy(self) -> BVector4: ...
    @property
    def zzwz(self) -> BVector4: ...
    @property
    def zzww(self) -> BVector4: ...
    @property
    def zzwo(self) -> BVector4: ...
    @property
    def zzwl(self) -> BVector4: ...
    @property
    def zzox(self) -> BVector4: ...
    @property
    def zzoy(self) -> BVector4: ...
    @property
    def zzoz(self) -> BVector4: ...
    @property
    def zzow(self) -> BVector4: ...
    @property
    def zzoo(self) -> BVector4: ...
    @property
    def zzol(self) -> BVector4: ...
    @property
    def zzlx(self) -> BVector4: ...
    @property
    def zzly(self) -> BVector4: ...
    @property
    def zzlz(self) -> BVector4: ...
    @property
    def zzlw(self) -> BVector4: ...
    @property
    def zzlo(self) -> BVector4: ...
    @property
    def zzll(self) -> BVector4: ...
    @property
    def zwxx(self) -> BVector4: ...
    @property
    def zwxy(self) -> BVector4: ...
    @property
    def zwxz(self) -> BVector4: ...
    @property
    def zwxw(self) -> BVector4: ...
    @property
    def zwxo(self) -> BVector4: ...
    @property
    def zwxl(self) -> BVector4: ...
    @property
    def zwyx(self) -> BVector4: ...
    @property
    def zwyy(self) -> BVector4: ...
    @property
    def zwyz(self) -> BVector4: ...
    @property
    def zwyw(self) -> BVector4: ...
    @property
    def zwyo(self) -> BVector4: ...
    @property
    def zwyl(self) -> BVector4: ...
    @property
    def zwzx(self) -> BVector4: ...
    @property
    def zwzy(self) -> BVector4: ...
    @property
    def zwzz(self) -> BVector4: ...
    @property
    def zwzw(self) -> BVector4: ...
    @property
    def zwzo(self) -> BVector4: ...
    @property
    def zwzl(self) -> BVector4: ...
    @property
    def zwwx(self) -> BVector4: ...
    @property
    def zwwy(self) -> BVector4: ...
    @property
    def zwwz(self) -> BVector4: ...
    @property
    def zwww(self) -> BVector4: ...
    @property
    def zwwo(self) -> BVector4: ...
    @property
    def zwwl(self) -> BVector4: ...
    @property
    def zwox(self) -> BVector4: ...
    @property
    def zwoy(self) -> BVector4: ...
    @property
    def zwoz(self) -> BVector4: ...
    @property
    def zwow(self) -> BVector4: ...
    @property
    def zwoo(self) -> BVector4: ...
    @property
    def zwol(self) -> BVector4: ...
    @property
    def zwlx(self) -> BVector4: ...
    @property
    def zwly(self) -> BVector4: ...
    @property
    def zwlz(self) -> BVector4: ...
    @property
    def zwlw(self) -> BVector4: ...
    @property
    def zwlo(self) -> BVector4: ...
    @property
    def zwll(self) -> BVector4: ...
    @property
    def zoxx(self) -> BVector4: ...
    @property
    def zoxy(self) -> BVector4: ...
    @property
    def zoxz(self) -> BVector4: ...
    @property
    def zoxw(self) -> BVector4: ...
    @property
    def zoxo(self) -> BVector4: ...
    @property
    def zoxl(self) -> BVector4: ...
    @property
    def zoyx(self) -> BVector4: ...
    @property
    def zoyy(self) -> BVector4: ...
    @property
    def zoyz(self) -> BVector4: ...
    @property
    def zoyw(self) -> BVector4: ...
    @property
    def zoyo(self) -> BVector4: ...
    @property
    def zoyl(self) -> BVector4: ...
    @property
    def zozx(self) -> BVector4: ...
    @property
    def zozy(self) -> BVector4: ...
    @property
    def zozz(self) -> BVector4: ...
    @property
    def zozw(self) -> BVector4: ...
    @property
    def zozo(self) -> BVector4: ...
    @property
    def zozl(self) -> BVector4: ...
    @property
    def zowx(self) -> BVector4: ...
    @property
    def zowy(self) -> BVector4: ...
    @property
    def zowz(self) -> BVector4: ...
    @property
    def zoww(self) -> BVector4: ...
    @property
    def zowo(self) -> BVector4: ...
    @property
    def zowl(self) -> BVector4: ...
    @property
    def zoox(self) -> BVector4: ...
    @property
    def zooy(self) -> BVector4: ...
    @property
    def zooz(self) -> BVector4: ...
    @property
    def zoow(self) -> BVector4: ...
    @property
    def zooo(self) -> BVector4: ...
    @property
    def zool(self) -> BVector4: ...
    @property
    def zolx(self) -> BVector4: ...
    @property
    def zoly(self) -> BVector4: ...
    @property
    def zolz(self) -> BVector4: ...
    @property
    def zolw(self) -> BVector4: ...
    @property
    def zolo(self) -> BVector4: ...
    @property
    def zoll(self) -> BVector4: ...
    @property
    def zlxx(self) -> BVector4: ...
    @property
    def zlxy(self) -> BVector4: ...
    @property
    def zlxz(self) -> BVector4: ...
    @property
    def zlxw(self) -> BVector4: ...
    @property
    def zlxo(self) -> BVector4: ...
    @property
    def zlxl(self) -> BVector4: ...
    @property
    def zlyx(self) -> BVector4: ...
    @property
    def zlyy(self) -> BVector4: ...
    @property
    def zlyz(self) -> BVector4: ...
    @property
    def zlyw(self) -> BVector4: ...
    @property
    def zlyo(self) -> BVector4: ...
    @property
    def zlyl(self) -> BVector4: ...
    @property
    def zlzx(self) -> BVector4: ...
    @property
    def zlzy(self) -> BVector4: ...
    @property
    def zlzz(self) -> BVector4: ...
    @property
    def zlzw(self) -> BVector4: ...
    @property
    def zlzo(self) -> BVector4: ...
    @property
    def zlzl(self) -> BVector4: ...
    @property
    def zlwx(self) -> BVector4: ...
    @property
    def zlwy(self) -> BVector4: ...
    @property
    def zlwz(self) -> BVector4: ...
    @property
    def zlww(self) -> BVector4: ...
    @property
    def zlwo(self) -> BVector4: ...
    @property
    def zlwl(self) -> BVector4: ...
    @property
    def zlox(self) -> BVector4: ...
    @property
    def zloy(self) -> BVector4: ...
    @property
    def zloz(self) -> BVector4: ...
    @property
    def zlow(self) -> BVector4: ...
    @property
    def zloo(self) -> BVector4: ...
    @property
    def zlol(self) -> BVector4: ...
    @property
    def zllx(self) -> BVector4: ...
    @property
    def zlly(self) -> BVector4: ...
    @property
    def zllz(self) -> BVector4: ...
    @property
    def zllw(self) -> BVector4: ...
    @property
    def zllo(self) -> BVector4: ...
    @property
    def zlll(self) -> BVector4: ...
    @property
    def wxxx(self) -> BVector4: ...
    @property
    def wxxy(self) -> BVector4: ...
    @property
    def wxxz(self) -> BVector4: ...
    @property
    def wxxw(self) -> BVector4: ...
    @property
    def wxxo(self) -> BVector4: ...
    @property
    def wxxl(self) -> BVector4: ...
    @property
    def wxyx(self) -> BVector4: ...
    @property
    def wxyy(self) -> BVector4: ...
    @property
    def wxyz(self) -> BVector4: ...
    @property
    def wxyw(self) -> BVector4: ...
    @property
    def wxyo(self) -> BVector4: ...
    @property
    def wxyl(self) -> BVector4: ...
    @property
    def wxzx(self) -> BVector4: ...
    @property
    def wxzy(self) -> BVector4: ...
    @property
    def wxzz(self) -> BVector4: ...
    @property
    def wxzw(self) -> BVector4: ...
    @property
    def wxzo(self) -> BVector4: ...
    @property
    def wxzl(self) -> BVector4: ...
    @property
    def wxwx(self) -> BVector4: ...
    @property
    def wxwy(self) -> BVector4: ...
    @property
    def wxwz(self) -> BVector4: ...
    @property
    def wxww(self) -> BVector4: ...
    @property
    def wxwo(self) -> BVector4: ...
    @property
    def wxwl(self) -> BVector4: ...
    @property
    def wxox(self) -> BVector4: ...
    @property
    def wxoy(self) -> BVector4: ...
    @property
    def wxoz(self) -> BVector4: ...
    @property
    def wxow(self) -> BVector4: ...
    @property
    def wxoo(self) -> BVector4: ...
    @property
    def wxol(self) -> BVector4: ...
    @property
    def wxlx(self) -> BVector4: ...
    @property
    def wxly(self) -> BVector4: ...
    @property
    def wxlz(self) -> BVector4: ...
    @property
    def wxlw(self) -> BVector4: ...
    @property
    def wxlo(self) -> BVector4: ...
    @property
    def wxll(self) -> BVector4: ...
    @property
    def wyxx(self) -> BVector4: ...
    @property
    def wyxy(self) -> BVector4: ...
    @property
    def wyxz(self) -> BVector4: ...
    @property
    def wyxw(self) -> BVector4: ...
    @property
    def wyxo(self) -> BVector4: ...
    @property
    def wyxl(self) -> BVector4: ...
    @property
    def wyyx(self) -> BVector4: ...
    @property
    def wyyy(self) -> BVector4: ...
    @property
    def wyyz(self) -> BVector4: ...
    @property
    def wyyw(self) -> BVector4: ...
    @property
    def wyyo(self) -> BVector4: ...
    @property
    def wyyl(self) -> BVector4: ...
    @property
    def wyzx(self) -> BVector4: ...
    @property
    def wyzy(self) -> BVector4: ...
    @property
    def wyzz(self) -> BVector4: ...
    @property
    def wyzw(self) -> BVector4: ...
    @property
    def wyzo(self) -> BVector4: ...
    @property
    def wyzl(self) -> BVector4: ...
    @property
    def wywx(self) -> BVector4: ...
    @property
    def wywy(self) -> BVector4: ...
    @property
    def wywz(self) -> BVector4: ...
    @property
    def wyww(self) -> BVector4: ...
    @property
    def wywo(self) -> BVector4: ...
    @property
    def wywl(self) -> BVector4: ...
    @property
    def wyox(self) -> BVector4: ...
    @property
    def wyoy(self) -> BVector4: ...
    @property
    def wyoz(self) -> BVector4: ...
    @property
    def wyow(self) -> BVector4: ...
    @property
    def wyoo(self) -> BVector4: ...
    @property
    def wyol(self) -> BVector4: ...
    @property
    def wylx(self) -> BVector4: ...
    @property
    def wyly(self) -> BVector4: ...
    @property
    def wylz(self) -> BVector4: ...
    @property
    def wylw(self) -> BVector4: ...
    @property
    def wylo(self) -> BVector4: ...
    @property
    def wyll(self) -> BVector4: ...
    @property
    def wzxx(self) -> BVector4: ...
    @property
    def wzxy(self) -> BVector4: ...
    @property
    def wzxz(self) -> BVector4: ...
    @property
    def wzxw(self) -> BVector4: ...
    @property
    def wzxo(self) -> BVector4: ...
    @property
    def wzxl(self) -> BVector4: ...
    @property
    def wzyx(self) -> BVector4: ...
    @property
    def wzyy(self) -> BVector4: ...
    @property
    def wzyz(self) -> BVector4: ...
    @property
    def wzyw(self) -> BVector4: ...
    @property
    def wzyo(self) -> BVector4: ...
    @property
    def wzyl(self) -> BVector4: ...
    @property
    def wzzx(self) -> BVector4: ...
    @property
    def wzzy(self) -> BVector4: ...
    @property
    def wzzz(self) -> BVector4: ...
    @property
    def wzzw(self) -> BVector4: ...
    @property
    def wzzo(self) -> BVector4: ...
    @property
    def wzzl(self) -> BVector4: ...
    @property
    def wzwx(self) -> BVector4: ...
    @property
    def wzwy(self) -> BVector4: ...
    @property
    def wzwz(self) -> BVector4: ...
    @property
    def wzww(self) -> BVector4: ...
    @property
    def wzwo(self) -> BVector4: ...
    @property
    def wzwl(self) -> BVector4: ...
    @property
    def wzox(self) -> BVector4: ...
    @property
    def wzoy(self) -> BVector4: ...
    @property
    def wzoz(self) -> BVector4: ...
    @property
    def wzow(self) -> BVector4: ...
    @property
    def wzoo(self) -> BVector4: ...
    @property
    def wzol(self) -> BVector4: ...
    @property
    def wzlx(self) -> BVector4: ...
    @property
    def wzly(self) -> BVector4: ...
    @property
    def wzlz(self) -> BVector4: ...
    @property
    def wzlw(self) -> BVector4: ...
    @property
    def wzlo(self) -> BVector4: ...
    @property
    def wzll(self) -> BVector4: ...
    @property
    def wwxx(self) -> BVector4: ...
    @property
    def wwxy(self) -> BVector4: ...
    @property
    def wwxz(self) -> BVector4: ...
    @property
    def wwxw(self) -> BVector4: ...
    @property
    def wwxo(self) -> BVector4: ...
    @property
    def wwxl(self) -> BVector4: ...
    @property
    def wwyx(self) -> BVector4: ...
    @property
    def wwyy(self) -> BVector4: ...
    @property
    def wwyz(self) -> BVector4: ...
    @property
    def wwyw(self) -> BVector4: ...
    @property
    def wwyo(self) -> BVector4: ...
    @property
    def wwyl(self) -> BVector4: ...
    @property
    def wwzx(self) -> BVector4: ...
    @property
    def wwzy(self) -> BVector4: ...
    @property
    def wwzz(self) -> BVector4: ...
    @property
    def wwzw(self) -> BVector4: ...
    @property
    def wwzo(self) -> BVector4: ...
    @property
    def wwzl(self) -> BVector4: ...
    @property
    def wwwx(self) -> BVector4: ...
    @property
    def wwwy(self) -> BVector4: ...
    @property
    def wwwz(self) -> BVector4: ...
    @property
    def wwww(self) -> BVector4: ...
    @property
    def wwwo(self) -> BVector4: ...
    @property
    def wwwl(self) -> BVector4: ...
    @property
    def wwox(self) -> BVector4: ...
    @property
    def wwoy(self) -> BVector4: ...
    @property
    def wwoz(self) -> BVector4: ...
    @property
    def wwow(self) -> BVector4: ...
    @property
    def wwoo(self) -> BVector4: ...
    @property
    def wwol(self) -> BVector4: ...
    @property
    def wwlx(self) -> BVector4: ...
    @property
    def wwly(self) -> BVector4: ...
    @property
    def wwlz(self) -> BVector4: ...
    @property
    def wwlw(self) -> BVector4: ...
    @property
    def wwlo(self) -> BVector4: ...
    @property
    def wwll(self) -> BVector4: ...
    @property
    def woxx(self) -> BVector4: ...
    @property
    def woxy(self) -> BVector4: ...
    @property
    def woxz(self) -> BVector4: ...
    @property
    def woxw(self) -> BVector4: ...
    @property
    def woxo(self) -> BVector4: ...
    @property
    def woxl(self) -> BVector4: ...
    @property
    def woyx(self) -> BVector4: ...
    @property
    def woyy(self) -> BVector4: ...
    @property
    def woyz(self) -> BVector4: ...
    @property
    def woyw(self) -> BVector4: ...
    @property
    def woyo(self) -> BVector4: ...
    @property
    def woyl(self) -> BVector4: ...
    @property
    def wozx(self) -> BVector4: ...
    @property
    def wozy(self) -> BVector4: ...
    @property
    def wozz(self) -> BVector4: ...
    @property
    def wozw(self) -> BVector4: ...
    @property
    def wozo(self) -> BVector4: ...
    @property
    def wozl(self) -> BVector4: ...
    @property
    def wowx(self) -> BVector4: ...
    @property
    def wowy(self) -> BVector4: ...
    @property
    def wowz(self) -> BVector4: ...
    @property
    def woww(self) -> BVector4: ...
    @property
    def wowo(self) -> BVector4: ...
    @property
    def wowl(self) -> BVector4: ...
    @property
    def woox(self) -> BVector4: ...
    @property
    def wooy(self) -> BVector4: ...
    @property
    def wooz(self) -> BVector4: ...
    @property
    def woow(self) -> BVector4: ...
    @property
    def wooo(self) -> BVector4: ...
    @property
    def wool(self) -> BVector4: ...
    @property
    def wolx(self) -> BVector4: ...
    @property
    def woly(self) -> BVector4: ...
    @property
    def wolz(self) -> BVector4: ...
    @property
    def wolw(self) -> BVector4: ...
    @property
    def wolo(self) -> BVector4: ...
    @property
    def woll(self) -> BVector4: ...
    @property
    def wlxx(self) -> BVector4: ...
    @property
    def wlxy(self) -> BVector4: ...
    @property
    def wlxz(self) -> BVector4: ...
    @property
    def wlxw(self) -> BVector4: ...
    @property
    def wlxo(self) -> BVector4: ...
    @property
    def wlxl(self) -> BVector4: ...
    @property
    def wlyx(self) -> BVector4: ...
    @property
    def wlyy(self) -> BVector4: ...
    @property
    def wlyz(self) -> BVector4: ...
    @property
    def wlyw(self) -> BVector4: ...
    @property
    def wlyo(self) -> BVector4: ...
    @property
    def wlyl(self) -> BVector4: ...
    @property
    def wlzx(self) -> BVector4: ...
    @property
    def wlzy(self) -> BVector4: ...
    @property
    def wlzz(self) -> BVector4: ...
    @property
    def wlzw(self) -> BVector4: ...
    @property
    def wlzo(self) -> BVector4: ...
    @property
    def wlzl(self) -> BVector4: ...
    @property
    def wlwx(self) -> BVector4: ...
    @property
    def wlwy(self) -> BVector4: ...
    @property
    def wlwz(self) -> BVector4: ...
    @property
    def wlww(self) -> BVector4: ...
    @property
    def wlwo(self) -> BVector4: ...
    @property
    def wlwl(self) -> BVector4: ...
    @property
    def wlox(self) -> BVector4: ...
    @property
    def wloy(self) -> BVector4: ...
    @property
    def wloz(self) -> BVector4: ...
    @property
    def wlow(self) -> BVector4: ...
    @property
    def wloo(self) -> BVector4: ...
    @property
    def wlol(self) -> BVector4: ...
    @property
    def wllx(self) -> BVector4: ...
    @property
    def wlly(self) -> BVector4: ...
    @property
    def wllz(self) -> BVector4: ...
    @property
    def wllw(self) -> BVector4: ...
    @property
    def wllo(self) -> BVector4: ...
    @property
    def wlll(self) -> BVector4: ...
    @property
    def oxxx(self) -> BVector4: ...
    @property
    def oxxy(self) -> BVector4: ...
    @property
    def oxxz(self) -> BVector4: ...
    @property
    def oxxw(self) -> BVector4: ...
    @property
    def oxxo(self) -> BVector4: ...
    @property
    def oxxl(self) -> BVector4: ...
    @property
    def oxyx(self) -> BVector4: ...
    @property
    def oxyy(self) -> BVector4: ...
    @property
    def oxyz(self) -> BVector4: ...
    @property
    def oxyw(self) -> BVector4: ...
    @property
    def oxyo(self) -> BVector4: ...
    @property
    def oxyl(self) -> BVector4: ...
    @property
    def oxzx(self) -> BVector4: ...
    @property
    def oxzy(self) -> BVector4: ...
    @property
    def oxzz(self) -> BVector4: ...
    @property
    def oxzw(self) -> BVector4: ...
    @property
    def oxzo(self) -> BVector4: ...
    @property
    def oxzl(self) -> BVector4: ...
    @property
    def oxwx(self) -> BVector4: ...
    @property
    def oxwy(self) -> BVector4: ...
    @property
    def oxwz(self) -> BVector4: ...
    @property
    def oxww(self) -> BVector4: ...
    @property
    def oxwo(self) -> BVector4: ...
    @property
    def oxwl(self) -> BVector4: ...
    @property
    def oxox(self) -> BVector4: ...
    @property
    def oxoy(self) -> BVector4: ...
    @property
    def oxoz(self) -> BVector4: ...
    @property
    def oxow(self) -> BVector4: ...
    @property
    def oxoo(self) -> BVector4: ...
    @property
    def oxol(self) -> BVector4: ...
    @property
    def oxlx(self) -> BVector4: ...
    @property
    def oxly(self) -> BVector4: ...
    @property
    def oxlz(self) -> BVector4: ...
    @property
    def oxlw(self) -> BVector4: ...
    @property
    def oxlo(self) -> BVector4: ...
    @property
    def oxll(self) -> BVector4: ...
    @property
    def oyxx(self) -> BVector4: ...
    @property
    def oyxy(self) -> BVector4: ...
    @property
    def oyxz(self) -> BVector4: ...
    @property
    def oyxw(self) -> BVector4: ...
    @property
    def oyxo(self) -> BVector4: ...
    @property
    def oyxl(self) -> BVector4: ...
    @property
    def oyyx(self) -> BVector4: ...
    @property
    def oyyy(self) -> BVector4: ...
    @property
    def oyyz(self) -> BVector4: ...
    @property
    def oyyw(self) -> BVector4: ...
    @property
    def oyyo(self) -> BVector4: ...
    @property
    def oyyl(self) -> BVector4: ...
    @property
    def oyzx(self) -> BVector4: ...
    @property
    def oyzy(self) -> BVector4: ...
    @property
    def oyzz(self) -> BVector4: ...
    @property
    def oyzw(self) -> BVector4: ...
    @property
    def oyzo(self) -> BVector4: ...
    @property
    def oyzl(self) -> BVector4: ...
    @property
    def oywx(self) -> BVector4: ...
    @property
    def oywy(self) -> BVector4: ...
    @property
    def oywz(self) -> BVector4: ...
    @property
    def oyww(self) -> BVector4: ...
    @property
    def oywo(self) -> BVector4: ...
    @property
    def oywl(self) -> BVector4: ...
    @property
    def oyox(self) -> BVector4: ...
    @property
    def oyoy(self) -> BVector4: ...
    @property
    def oyoz(self) -> BVector4: ...
    @property
    def oyow(self) -> BVector4: ...
    @property
    def oyoo(self) -> BVector4: ...
    @property
    def oyol(self) -> BVector4: ...
    @property
    def oylx(self) -> BVector4: ...
    @property
    def oyly(self) -> BVector4: ...
    @property
    def oylz(self) -> BVector4: ...
    @property
    def oylw(self) -> BVector4: ...
    @property
    def oylo(self) -> BVector4: ...
    @property
    def oyll(self) -> BVector4: ...
    @property
    def ozxx(self) -> BVector4: ...
    @property
    def ozxy(self) -> BVector4: ...
    @property
    def ozxz(self) -> BVector4: ...
    @property
    def ozxw(self) -> BVector4: ...
    @property
    def ozxo(self) -> BVector4: ...
    @property
    def ozxl(self) -> BVector4: ...
    @property
    def ozyx(self) -> BVector4: ...
    @property
    def ozyy(self) -> BVector4: ...
    @property
    def ozyz(self) -> BVector4: ...
    @property
    def ozyw(self) -> BVector4: ...
    @property
    def ozyo(self) -> BVector4: ...
    @property
    def ozyl(self) -> BVector4: ...
    @property
    def ozzx(self) -> BVector4: ...
    @property
    def ozzy(self) -> BVector4: ...
    @property
    def ozzz(self) -> BVector4: ...
    @property
    def ozzw(self) -> BVector4: ...
    @property
    def ozzo(self) -> BVector4: ...
    @property
    def ozzl(self) -> BVector4: ...
    @property
    def ozwx(self) -> BVector4: ...
    @property
    def ozwy(self) -> BVector4: ...
    @property
    def ozwz(self) -> BVector4: ...
    @property
    def ozww(self) -> BVector4: ...
    @property
    def ozwo(self) -> BVector4: ...
    @property
    def ozwl(self) -> BVector4: ...
    @property
    def ozox(self) -> BVector4: ...
    @property
    def ozoy(self) -> BVector4: ...
    @property
    def ozoz(self) -> BVector4: ...
    @property
    def ozow(self) -> BVector4: ...
    @property
    def ozoo(self) -> BVector4: ...
    @property
    def ozol(self) -> BVector4: ...
    @property
    def ozlx(self) -> BVector4: ...
    @property
    def ozly(self) -> BVector4: ...
    @property
    def ozlz(self) -> BVector4: ...
    @property
    def ozlw(self) -> BVector4: ...
    @property
    def ozlo(self) -> BVector4: ...
    @property
    def ozll(self) -> BVector4: ...
    @property
    def owxx(self) -> BVector4: ...
    @property
    def owxy(self) -> BVector4: ...
    @property
    def owxz(self) -> BVector4: ...
    @property
    def owxw(self) -> BVector4: ...
    @property
    def owxo(self) -> BVector4: ...
    @property
    def owxl(self) -> BVector4: ...
    @property
    def owyx(self) -> BVector4: ...
    @property
    def owyy(self) -> BVector4: ...
    @property
    def owyz(self) -> BVector4: ...
    @property
    def owyw(self) -> BVector4: ...
    @property
    def owyo(self) -> BVector4: ...
    @property
    def owyl(self) -> BVector4: ...
    @property
    def owzx(self) -> BVector4: ...
    @property
    def owzy(self) -> BVector4: ...
    @property
    def owzz(self) -> BVector4: ...
    @property
    def owzw(self) -> BVector4: ...
    @property
    def owzo(self) -> BVector4: ...
    @property
    def owzl(self) -> BVector4: ...
    @property
    def owwx(self) -> BVector4: ...
    @property
    def owwy(self) -> BVector4: ...
    @property
    def owwz(self) -> BVector4: ...
    @property
    def owww(self) -> BVector4: ...
    @property
    def owwo(self) -> BVector4: ...
    @property
    def owwl(self) -> BVector4: ...
    @property
    def owox(self) -> BVector4: ...
    @property
    def owoy(self) -> BVector4: ...
    @property
    def owoz(self) -> BVector4: ...
    @property
    def owow(self) -> BVector4: ...
    @property
    def owoo(self) -> BVector4: ...
    @property
    def owol(self) -> BVector4: ...
    @property
    def owlx(self) -> BVector4: ...
    @property
    def owly(self) -> BVector4: ...
    @property
    def owlz(self) -> BVector4: ...
    @property
    def owlw(self) -> BVector4: ...
    @property
    def owlo(self) -> BVector4: ...
    @property
    def owll(self) -> BVector4: ...
    @property
    def ooxx(self) -> BVector4: ...
    @property
    def ooxy(self) -> BVector4: ...
    @property
    def ooxz(self) -> BVector4: ...
    @property
    def ooxw(self) -> BVector4: ...
    @property
    def ooxo(self) -> BVector4: ...
    @property
    def ooxl(self) -> BVector4: ...
    @property
    def ooyx(self) -> BVector4: ...
    @property
    def ooyy(self) -> BVector4: ...
    @property
    def ooyz(self) -> BVector4: ...
    @property
    def ooyw(self) -> BVector4: ...
    @property
    def ooyo(self) -> BVector4: ...
    @property
    def ooyl(self) -> BVector4: ...
    @property
    def oozx(self) -> BVector4: ...
    @property
    def oozy(self) -> BVector4: ...
    @property
    def oozz(self) -> BVector4: ...
    @property
    def oozw(self) -> BVector4: ...
    @property
    def oozo(self) -> BVector4: ...
    @property
    def oozl(self) -> BVector4: ...
    @property
    def oowx(self) -> BVector4: ...
    @property
    def oowy(self) -> BVector4: ...
    @property
    def oowz(self) -> BVector4: ...
    @property
    def ooww(self) -> BVector4: ...
    @property
    def oowo(self) -> BVector4: ...
    @property
    def oowl(self) -> BVector4: ...
    @property
    def ooox(self) -> BVector4: ...
    @property
    def oooy(self) -> BVector4: ...
    @property
    def oooz(self) -> BVector4: ...
    @property
    def ooow(self) -> BVector4: ...
    @property
    def oolx(self) -> BVector4: ...
    @property
    def ooly(self) -> BVector4: ...
    @property
    def oolz(self) -> BVector4: ...
    @property
    def oolw(self) -> BVector4: ...
    @property
    def olxx(self) -> BVector4: ...
    @property
    def olxy(self) -> BVector4: ...
    @property
    def olxz(self) -> BVector4: ...
    @property
    def olxw(self) -> BVector4: ...
    @property
    def olxo(self) -> BVector4: ...
    @property
    def olxl(self) -> BVector4: ...
    @property
    def olyx(self) -> BVector4: ...
    @property
    def olyy(self) -> BVector4: ...
    @property
    def olyz(self) -> BVector4: ...
    @property
    def olyw(self) -> BVector4: ...
    @property
    def olyo(self) -> BVector4: ...
    @property
    def olyl(self) -> BVector4: ...
    @property
    def olzx(self) -> BVector4: ...
    @property
    def olzy(self) -> BVector4: ...
    @property
    def olzz(self) -> BVector4: ...
    @property
    def olzw(self) -> BVector4: ...
    @property
    def olzo(self) -> BVector4: ...
    @property
    def olzl(self) -> BVector4: ...
    @property
    def olwx(self) -> BVector4: ...
    @property
    def olwy(self) -> BVector4: ...
    @property
    def olwz(self) -> BVector4: ...
    @property
    def olww(self) -> BVector4: ...
    @property
    def olwo(self) -> BVector4: ...
    @property
    def olwl(self) -> BVector4: ...
    @property
    def olox(self) -> BVector4: ...
    @property
    def oloy(self) -> BVector4: ...
    @property
    def oloz(self) -> BVector4: ...
    @property
    def olow(self) -> BVector4: ...
    @property
    def ollx(self) -> BVector4: ...
    @property
    def olly(self) -> BVector4: ...
    @property
    def ollz(self) -> BVector4: ...
    @property
    def ollw(self) -> BVector4: ...
    @property
    def lxxx(self) -> BVector4: ...
    @property
    def lxxy(self) -> BVector4: ...
    @property
    def lxxz(self) -> BVector4: ...
    @property
    def lxxw(self) -> BVector4: ...
    @property
    def lxxo(self) -> BVector4: ...
    @property
    def lxxl(self) -> BVector4: ...
    @property
    def lxyx(self) -> BVector4: ...
    @property
    def lxyy(self) -> BVector4: ...
    @property
    def lxyz(self) -> BVector4: ...
    @property
    def lxyw(self) -> BVector4: ...
    @property
    def lxyo(self) -> BVector4: ...
    @property
    def lxyl(self) -> BVector4: ...
    @property
    def lxzx(self) -> BVector4: ...
    @property
    def lxzy(self) -> BVector4: ...
    @property
    def lxzz(self) -> BVector4: ...
    @property
    def lxzw(self) -> BVector4: ...
    @property
    def lxzo(self) -> BVector4: ...
    @property
    def lxzl(self) -> BVector4: ...
    @property
    def lxwx(self) -> BVector4: ...
    @property
    def lxwy(self) -> BVector4: ...
    @property
    def lxwz(self) -> BVector4: ...
    @property
    def lxww(self) -> BVector4: ...
    @property
    def lxwo(self) -> BVector4: ...
    @property
    def lxwl(self) -> BVector4: ...
    @property
    def lxox(self) -> BVector4: ...
    @property
    def lxoy(self) -> BVector4: ...
    @property
    def lxoz(self) -> BVector4: ...
    @property
    def lxow(self) -> BVector4: ...
    @property
    def lxoo(self) -> BVector4: ...
    @property
    def lxol(self) -> BVector4: ...
    @property
    def lxlx(self) -> BVector4: ...
    @property
    def lxly(self) -> BVector4: ...
    @property
    def lxlz(self) -> BVector4: ...
    @property
    def lxlw(self) -> BVector4: ...
    @property
    def lxlo(self) -> BVector4: ...
    @property
    def lxll(self) -> BVector4: ...
    @property
    def lyxx(self) -> BVector4: ...
    @property
    def lyxy(self) -> BVector4: ...
    @property
    def lyxz(self) -> BVector4: ...
    @property
    def lyxw(self) -> BVector4: ...
    @property
    def lyxo(self) -> BVector4: ...
    @property
    def lyxl(self) -> BVector4: ...
    @property
    def lyyx(self) -> BVector4: ...
    @property
    def lyyy(self) -> BVector4: ...
    @property
    def lyyz(self) -> BVector4: ...
    @property
    def lyyw(self) -> BVector4: ...
    @property
    def lyyo(self) -> BVector4: ...
    @property
    def lyyl(self) -> BVector4: ...
    @property
    def lyzx(self) -> BVector4: ...
    @property
    def lyzy(self) -> BVector4: ...
    @property
    def lyzz(self) -> BVector4: ...
    @property
    def lyzw(self) -> BVector4: ...
    @property
    def lyzo(self) -> BVector4: ...
    @property
    def lyzl(self) -> BVector4: ...
    @property
    def lywx(self) -> BVector4: ...
    @property
    def lywy(self) -> BVector4: ...
    @property
    def lywz(self) -> BVector4: ...
    @property
    def lyww(self) -> BVector4: ...
    @property
    def lywo(self) -> BVector4: ...
    @property
    def lywl(self) -> BVector4: ...
    @property
    def lyox(self) -> BVector4: ...
    @property
    def lyoy(self) -> BVector4: ...
    @property
    def lyoz(self) -> BVector4: ...
    @property
    def lyow(self) -> BVector4: ...
    @property
    def lyoo(self) -> BVector4: ...
    @property
    def lyol(self) -> BVector4: ...
    @property
    def lylx(self) -> BVector4: ...
    @property
    def lyly(self) -> BVector4: ...
    @property
    def lylz(self) -> BVector4: ...
    @property
    def lylw(self) -> BVector4: ...
    @property
    def lylo(self) -> BVector4: ...
    @property
    def lyll(self) -> BVector4: ...
    @property
    def lzxx(self) -> BVector4: ...
    @property
    def lzxy(self) -> BVector4: ...
    @property
    def lzxz(self) -> BVector4: ...
    @property
    def lzxw(self) -> BVector4: ...
    @property
    def lzxo(self) -> BVector4: ...
    @property
    def lzxl(self) -> BVector4: ...
    @property
    def lzyx(self) -> BVector4: ...
    @property
    def lzyy(self) -> BVector4: ...
    @property
    def lzyz(self) -> BVector4: ...
    @property
    def lzyw(self) -> BVector4: ...
    @property
    def lzyo(self) -> BVector4: ...
    @property
    def lzyl(self) -> BVector4: ...
    @property
    def lzzx(self) -> BVector4: ...
    @property
    def lzzy(self) -> BVector4: ...
    @property
    def lzzz(self) -> BVector4: ...
    @property
    def lzzw(self) -> BVector4: ...
    @property
    def lzzo(self) -> BVector4: ...
    @property
    def lzzl(self) -> BVector4: ...
    @property
    def lzwx(self) -> BVector4: ...
    @property
    def lzwy(self) -> BVector4: ...
    @property
    def lzwz(self) -> BVector4: ...
    @property
    def lzww(self) -> BVector4: ...
    @property
    def lzwo(self) -> BVector4: ...
    @property
    def lzwl(self) -> BVector4: ...
    @property
    def lzox(self) -> BVector4: ...
    @property
    def lzoy(self) -> BVector4: ...
    @property
    def lzoz(self) -> BVector4: ...
    @property
    def lzow(self) -> BVector4: ...
    @property
    def lzoo(self) -> BVector4: ...
    @property
    def lzol(self) -> BVector4: ...
    @property
    def lzlx(self) -> BVector4: ...
    @property
    def lzly(self) -> BVector4: ...
    @property
    def lzlz(self) -> BVector4: ...
    @property
    def lzlw(self) -> BVector4: ...
    @property
    def lzlo(self) -> BVector4: ...
    @property
    def lzll(self) -> BVector4: ...
    @property
    def lwxx(self) -> BVector4: ...
    @property
    def lwxy(self) -> BVector4: ...
    @property
    def lwxz(self) -> BVector4: ...
    @property
    def lwxw(self) -> BVector4: ...
    @property
    def lwxo(self) -> BVector4: ...
    @property
    def lwxl(self) -> BVector4: ...
    @property
    def lwyx(self) -> BVector4: ...
    @property
    def lwyy(self) -> BVector4: ...
    @property
    def lwyz(self) -> BVector4: ...
    @property
    def lwyw(self) -> BVector4: ...
    @property
    def lwyo(self) -> BVector4: ...
    @property
    def lwyl(self) -> BVector4: ...
    @property
    def lwzx(self) -> BVector4: ...
    @property
    def lwzy(self) -> BVector4: ...
    @property
    def lwzz(self) -> BVector4: ...
    @property
    def lwzw(self) -> BVector4: ...
    @property
    def lwzo(self) -> BVector4: ...
    @property
    def lwzl(self) -> BVector4: ...
    @property
    def lwwx(self) -> BVector4: ...
    @property
    def lwwy(self) -> BVector4: ...
    @property
    def lwwz(self) -> BVector4: ...
    @property
    def lwww(self) -> BVector4: ...
    @property
    def lwwo(self) -> BVector4: ...
    @property
    def lwwl(self) -> BVector4: ...
    @property
    def lwox(self) -> BVector4: ...
    @property
    def lwoy(self) -> BVector4: ...
    @property
    def lwoz(self) -> BVector4: ...
    @property
    def lwow(self) -> BVector4: ...
    @property
    def lwoo(self) -> BVector4: ...
    @property
    def lwol(self) -> BVector4: ...
    @property
    def lwlx(self) -> BVector4: ...
    @property
    def lwly(self) -> BVector4: ...
    @property
    def lwlz(self) -> BVector4: ...
    @property
    def lwlw(self) -> BVector4: ...
    @property
    def lwlo(self) -> BVector4: ...
    @property
    def lwll(self) -> BVector4: ...
    @property
    def loxx(self) -> BVector4: ...
    @property
    def loxy(self) -> BVector4: ...
    @property
    def loxz(self) -> BVector4: ...
    @property
    def loxw(self) -> BVector4: ...
    @property
    def loxo(self) -> BVector4: ...
    @property
    def loxl(self) -> BVector4: ...
    @property
    def loyx(self) -> BVector4: ...
    @property
    def loyy(self) -> BVector4: ...
    @property
    def loyz(self) -> BVector4: ...
    @property
    def loyw(self) -> BVector4: ...
    @property
    def loyo(self) -> BVector4: ...
    @property
    def loyl(self) -> BVector4: ...
    @property
    def lozx(self) -> BVector4: ...
    @property
    def lozy(self) -> BVector4: ...
    @property
    def lozz(self) -> BVector4: ...
    @property
    def lozw(self) -> BVector4: ...
    @property
    def lozo(self) -> BVector4: ...
    @property
    def lozl(self) -> BVector4: ...
    @property
    def lowx(self) -> BVector4: ...
    @property
    def lowy(self) -> BVector4: ...
    @property
    def lowz(self) -> BVector4: ...
    @property
    def loww(self) -> BVector4: ...
    @property
    def lowo(self) -> BVector4: ...
    @property
    def lowl(self) -> BVector4: ...
    @property
    def loox(self) -> BVector4: ...
    @property
    def looy(self) -> BVector4: ...
    @property
    def looz(self) -> BVector4: ...
    @property
    def loow(self) -> BVector4: ...
    @property
    def lolx(self) -> BVector4: ...
    @property
    def loly(self) -> BVector4: ...
    @property
    def lolz(self) -> BVector4: ...
    @property
    def lolw(self) -> BVector4: ...
    @property
    def llxx(self) -> BVector4: ...
    @property
    def llxy(self) -> BVector4: ...
    @property
    def llxz(self) -> BVector4: ...
    @property
    def llxw(self) -> BVector4: ...
    @property
    def llxo(self) -> BVector4: ...
    @property
    def llxl(self) -> BVector4: ...
    @property
    def llyx(self) -> BVector4: ...
    @property
    def llyy(self) -> BVector4: ...
    @property
    def llyz(self) -> BVector4: ...
    @property
    def llyw(self) -> BVector4: ...
    @property
    def llyo(self) -> BVector4: ...
    @property
    def llyl(self) -> BVector4: ...
    @property
    def llzx(self) -> BVector4: ...
    @property
    def llzy(self) -> BVector4: ...
    @property
    def llzz(self) -> BVector4: ...
    @property
    def llzw(self) -> BVector4: ...
    @property
    def llzo(self) -> BVector4: ...
    @property
    def llzl(self) -> BVector4: ...
    @property
    def llwx(self) -> BVector4: ...
    @property
    def llwy(self) -> BVector4: ...
    @property
    def llwz(self) -> BVector4: ...
    @property
    def llww(self) -> BVector4: ...
    @property
    def llwo(self) -> BVector4: ...
    @property
    def llwl(self) -> BVector4: ...
    @property
    def llox(self) -> BVector4: ...
    @property
    def lloy(self) -> BVector4: ...
    @property
    def lloz(self) -> BVector4: ...
    @property
    def llow(self) -> BVector4: ...
    @property
    def lllx(self) -> BVector4: ...
    @property
    def llly(self) -> BVector4: ...
    @property
    def lllz(self) -> BVector4: ...
    @property
    def lllw(self) -> BVector4: ...
    @property
    def r(self) -> _bool: ...
    @property
    def g(self) -> _bool: ...
    @property
    def b(self) -> _bool: ...
    @property
    def a(self) -> _bool: ...
    @property
    def rr(self) -> BVector2: ...
    @property
    def rg(self) -> BVector2: ...
    @property
    def rb(self) -> BVector2: ...
    @property
    def ra(self) -> BVector2: ...
    @property
    def ro(self) -> BVector2: ...
    @property
    def rl(self) -> BVector2: ...
    @property
    def gr(self) -> BVector2: ...
    @property
    def gg(self) -> BVector2: ...
    @property
    def gb(self) -> BVector2: ...
    @property
    def ga(self) -> BVector2: ...
    @property
    def go(self) -> BVector2: ...
    @property
    def gl(self) -> BVector2: ...
    @property
    def br(self) -> BVector2: ...
    @property
    def bg(self) -> BVector2: ...
    @property
    def bb(self) -> BVector2: ...
    @property
    def ba(self) -> BVector2: ...
    @property
    def bo(self) -> BVector2: ...
    @property
    def bl(self) -> BVector2: ...
    @property
    def ar(self) -> BVector2: ...
    @property
    def ag(self) -> BVector2: ...
    @property
    def ab(self) -> BVector2: ...
    @property
    def aa(self) -> BVector2: ...
    @property
    def ao(self) -> BVector2: ...
    @property
    def al(self) -> BVector2: ...
    @property
    def og(self) -> BVector2: ...
    @property
    def ob(self) -> BVector2: ...
    @property
    def oa(self) -> BVector2: ...
    @property
    def lr(self) -> BVector2: ...
    @property
    def lg(self) -> BVector2: ...
    @property
    def lb(self) -> BVector2: ...
    @property
    def la(self) -> BVector2: ...
    @property
    def rrr(self) -> BVector3: ...
    @property
    def rrg(self) -> BVector3: ...
    @property
    def rrb(self) -> BVector3: ...
    @property
    def rra(self) -> BVector3: ...
    @property
    def rro(self) -> BVector3: ...
    @property
    def rrl(self) -> BVector3: ...
    @property
    def rgr(self) -> BVector3: ...
    @property
    def rgg(self) -> BVector3: ...
    @property
    def rgb(self) -> BVector3: ...
    @property
    def rga(self) -> BVector3: ...
    @property
    def rgo(self) -> BVector3: ...
    @property
    def rgl(self) -> BVector3: ...
    @property
    def rbr(self) -> BVector3: ...
    @property
    def rbg(self) -> BVector3: ...
    @property
    def rbb(self) -> BVector3: ...
    @property
    def rba(self) -> BVector3: ...
    @property
    def rbo(self) -> BVector3: ...
    @property
    def rbl(self) -> BVector3: ...
    @property
    def rar(self) -> BVector3: ...
    @property
    def rag(self) -> BVector3: ...
    @property
    def rab(self) -> BVector3: ...
    @property
    def raa(self) -> BVector3: ...
    @property
    def rao(self) -> BVector3: ...
    @property
    def ral(self) -> BVector3: ...
    @property
    def ror(self) -> BVector3: ...
    @property
    def rog(self) -> BVector3: ...
    @property
    def rob(self) -> BVector3: ...
    @property
    def roa(self) -> BVector3: ...
    @property
    def roo(self) -> BVector3: ...
    @property
    def rol(self) -> BVector3: ...
    @property
    def rlr(self) -> BVector3: ...
    @property
    def rlg(self) -> BVector3: ...
    @property
    def rlb(self) -> BVector3: ...
    @property
    def rla(self) -> BVector3: ...
    @property
    def rlo(self) -> BVector3: ...
    @property
    def rll(self) -> BVector3: ...
    @property
    def grr(self) -> BVector3: ...
    @property
    def grg(self) -> BVector3: ...
    @property
    def grb(self) -> BVector3: ...
    @property
    def gra(self) -> BVector3: ...
    @property
    def gro(self) -> BVector3: ...
    @property
    def grl(self) -> BVector3: ...
    @property
    def ggr(self) -> BVector3: ...
    @property
    def ggg(self) -> BVector3: ...
    @property
    def ggb(self) -> BVector3: ...
    @property
    def gga(self) -> BVector3: ...
    @property
    def ggo(self) -> BVector3: ...
    @property
    def ggl(self) -> BVector3: ...
    @property
    def gbr(self) -> BVector3: ...
    @property
    def gbg(self) -> BVector3: ...
    @property
    def gbb(self) -> BVector3: ...
    @property
    def gba(self) -> BVector3: ...
    @property
    def gbo(self) -> BVector3: ...
    @property
    def gbl(self) -> BVector3: ...
    @property
    def gar(self) -> BVector3: ...
    @property
    def gag(self) -> BVector3: ...
    @property
    def gab(self) -> BVector3: ...
    @property
    def gaa(self) -> BVector3: ...
    @property
    def gao(self) -> BVector3: ...
    @property
    def gal(self) -> BVector3: ...
    @property
    def gor(self) -> BVector3: ...
    @property
    def gog(self) -> BVector3: ...
    @property
    def gob(self) -> BVector3: ...
    @property
    def goa(self) -> BVector3: ...
    @property
    def goo(self) -> BVector3: ...
    @property
    def gol(self) -> BVector3: ...
    @property
    def glr(self) -> BVector3: ...
    @property
    def glg(self) -> BVector3: ...
    @property
    def glb(self) -> BVector3: ...
    @property
    def gla(self) -> BVector3: ...
    @property
    def glo(self) -> BVector3: ...
    @property
    def gll(self) -> BVector3: ...
    @property
    def brr(self) -> BVector3: ...
    @property
    def brg(self) -> BVector3: ...
    @property
    def brb(self) -> BVector3: ...
    @property
    def bra(self) -> BVector3: ...
    @property
    def bro(self) -> BVector3: ...
    @property
    def brl(self) -> BVector3: ...
    @property
    def bgr(self) -> BVector3: ...
    @property
    def bgg(self) -> BVector3: ...
    @property
    def bgb(self) -> BVector3: ...
    @property
    def bga(self) -> BVector3: ...
    @property
    def bgo(self) -> BVector3: ...
    @property
    def bgl(self) -> BVector3: ...
    @property
    def bbr(self) -> BVector3: ...
    @property
    def bbg(self) -> BVector3: ...
    @property
    def bbb(self) -> BVector3: ...
    @property
    def bba(self) -> BVector3: ...
    @property
    def bbo(self) -> BVector3: ...
    @property
    def bbl(self) -> BVector3: ...
    @property
    def bar(self) -> BVector3: ...
    @property
    def bag(self) -> BVector3: ...
    @property
    def bab(self) -> BVector3: ...
    @property
    def baa(self) -> BVector3: ...
    @property
    def bao(self) -> BVector3: ...
    @property
    def bal(self) -> BVector3: ...
    @property
    def bor(self) -> BVector3: ...
    @property
    def bog(self) -> BVector3: ...
    @property
    def bob(self) -> BVector3: ...
    @property
    def boa(self) -> BVector3: ...
    @property
    def boo(self) -> BVector3: ...
    @property
    def bol(self) -> BVector3: ...
    @property
    def blr(self) -> BVector3: ...
    @property
    def blg(self) -> BVector3: ...
    @property
    def blb(self) -> BVector3: ...
    @property
    def bla(self) -> BVector3: ...
    @property
    def blo(self) -> BVector3: ...
    @property
    def bll(self) -> BVector3: ...
    @property
    def arr(self) -> BVector3: ...
    @property
    def arg(self) -> BVector3: ...
    @property
    def arb(self) -> BVector3: ...
    @property
    def ara(self) -> BVector3: ...
    @property
    def aro(self) -> BVector3: ...
    @property
    def arl(self) -> BVector3: ...
    @property
    def agr(self) -> BVector3: ...
    @property
    def agg(self) -> BVector3: ...
    @property
    def agb(self) -> BVector3: ...
    @property
    def aga(self) -> BVector3: ...
    @property
    def ago(self) -> BVector3: ...
    @property
    def agl(self) -> BVector3: ...
    @property
    def abr(self) -> BVector3: ...
    @property
    def abg(self) -> BVector3: ...
    @property
    def abb(self) -> BVector3: ...
    @property
    def aba(self) -> BVector3: ...
    @property
    def abo(self) -> BVector3: ...
    @property
    def abl(self) -> BVector3: ...
    @property
    def aar(self) -> BVector3: ...
    @property
    def aag(self) -> BVector3: ...
    @property
    def aab(self) -> BVector3: ...
    @property
    def aaa(self) -> BVector3: ...
    @property
    def aao(self) -> BVector3: ...
    @property
    def aal(self) -> BVector3: ...
    @property
    def aor(self) -> BVector3: ...
    @property
    def aog(self) -> BVector3: ...
    @property
    def aob(self) -> BVector3: ...
    @property
    def aoa(self) -> BVector3: ...
    @property
    def aoo(self) -> BVector3: ...
    @property
    def aol(self) -> BVector3: ...
    @property
    def alr(self) -> BVector3: ...
    @property
    def alg(self) -> BVector3: ...
    @property
    def alb(self) -> BVector3: ...
    @property
    def ala(self) -> BVector3: ...
    @property
    def alo(self) -> BVector3: ...
    @property
    def all(self) -> BVector3: ...
    @property
    def orr(self) -> BVector3: ...
    @property
    def org(self) -> BVector3: ...
    @property
    def orb(self) -> BVector3: ...
    @property
    def ora(self) -> BVector3: ...
    @property
    def oro(self) -> BVector3: ...
    @property
    def orl(self) -> BVector3: ...
    @property
    def ogr(self) -> BVector3: ...
    @property
    def ogg(self) -> BVector3: ...
    @property
    def ogb(self) -> BVector3: ...
    @property
    def oga(self) -> BVector3: ...
    @property
    def ogo(self) -> BVector3: ...
    @property
    def ogl(self) -> BVector3: ...
    @property
    def obr(self) -> BVector3: ...
    @property
    def obg(self) -> BVector3: ...
    @property
    def obb(self) -> BVector3: ...
    @property
    def oba(self) -> BVector3: ...
    @property
    def obo(self) -> BVector3: ...
    @property
    def obl(self) -> BVector3: ...
    @property
    def oar(self) -> BVector3: ...
    @property
    def oag(self) -> BVector3: ...
    @property
    def oab(self) -> BVector3: ...
    @property
    def oaa(self) -> BVector3: ...
    @property
    def oao(self) -> BVector3: ...
    @property
    def oal(self) -> BVector3: ...
    @property
    def oor(self) -> BVector3: ...
    @property
    def oog(self) -> BVector3: ...
    @property
    def oob(self) -> BVector3: ...
    @property
    def ooa(self) -> BVector3: ...
    @property
    def olr(self) -> BVector3: ...
    @property
    def olg(self) -> BVector3: ...
    @property
    def olb(self) -> BVector3: ...
    @property
    def ola(self) -> BVector3: ...
    @property
    def lrr(self) -> BVector3: ...
    @property
    def lrg(self) -> BVector3: ...
    @property
    def lrb(self) -> BVector3: ...
    @property
    def lra(self) -> BVector3: ...
    @property
    def lro(self) -> BVector3: ...
    @property
    def lrl(self) -> BVector3: ...
    @property
    def lgr(self) -> BVector3: ...
    @property
    def lgg(self) -> BVector3: ...
    @property
    def lgb(self) -> BVector3: ...
    @property
    def lga(self) -> BVector3: ...
    @property
    def lgo(self) -> BVector3: ...
    @property
    def lgl(self) -> BVector3: ...
    @property
    def lbr(self) -> BVector3: ...
    @property
    def lbg(self) -> BVector3: ...
    @property
    def lbb(self) -> BVector3: ...
    @property
    def lba(self) -> BVector3: ...
    @property
    def lbo(self) -> BVector3: ...
    @property
    def lbl(self) -> BVector3: ...
    @property
    def lar(self) -> BVector3: ...
    @property
    def lag(self) -> BVector3: ...
    @property
    def lab(self) -> BVector3: ...
    @property
    def laa(self) -> BVector3: ...
    @property
    def lao(self) -> BVector3: ...
    @property
    def lal(self) -> BVector3: ...
    @property
    def lor(self) -> BVector3: ...
    @property
    def log(self) -> BVector3: ...
    @property
    def lob(self) -> BVector3: ...
    @property
    def loa(self) -> BVector3: ...
    @property
    def llr(self) -> BVector3: ...
    @property
    def llg(self) -> BVector3: ...
    @property
    def llb(self) -> BVector3: ...
    @property
    def lla(self) -> BVector3: ...
    @property
    def rrrr(self) -> BVector4: ...
    @property
    def rrrg(self) -> BVector4: ...
    @property
    def rrrb(self) -> BVector4: ...
    @property
    def rrra(self) -> BVector4: ...
    @property
    def rrro(self) -> BVector4: ...
    @property
    def rrrl(self) -> BVector4: ...
    @property
    def rrgr(self) -> BVector4: ...
    @property
    def rrgg(self) -> BVector4: ...
    @property
    def rrgb(self) -> BVector4: ...
    @property
    def rrga(self) -> BVector4: ...
    @property
    def rrgo(self) -> BVector4: ...
    @property
    def rrgl(self) -> BVector4: ...
    @property
    def rrbr(self) -> BVector4: ...
    @property
    def rrbg(self) -> BVector4: ...
    @property
    def rrbb(self) -> BVector4: ...
    @property
    def rrba(self) -> BVector4: ...
    @property
    def rrbo(self) -> BVector4: ...
    @property
    def rrbl(self) -> BVector4: ...
    @property
    def rrar(self) -> BVector4: ...
    @property
    def rrag(self) -> BVector4: ...
    @property
    def rrab(self) -> BVector4: ...
    @property
    def rraa(self) -> BVector4: ...
    @property
    def rrao(self) -> BVector4: ...
    @property
    def rral(self) -> BVector4: ...
    @property
    def rror(self) -> BVector4: ...
    @property
    def rrog(self) -> BVector4: ...
    @property
    def rrob(self) -> BVector4: ...
    @property
    def rroa(self) -> BVector4: ...
    @property
    def rroo(self) -> BVector4: ...
    @property
    def rrol(self) -> BVector4: ...
    @property
    def rrlr(self) -> BVector4: ...
    @property
    def rrlg(self) -> BVector4: ...
    @property
    def rrlb(self) -> BVector4: ...
    @property
    def rrla(self) -> BVector4: ...
    @property
    def rrlo(self) -> BVector4: ...
    @property
    def rrll(self) -> BVector4: ...
    @property
    def rgrr(self) -> BVector4: ...
    @property
    def rgrg(self) -> BVector4: ...
    @property
    def rgrb(self) -> BVector4: ...
    @property
    def rgra(self) -> BVector4: ...
    @property
    def rgro(self) -> BVector4: ...
    @property
    def rgrl(self) -> BVector4: ...
    @property
    def rggr(self) -> BVector4: ...
    @property
    def rggg(self) -> BVector4: ...
    @property
    def rggb(self) -> BVector4: ...
    @property
    def rgga(self) -> BVector4: ...
    @property
    def rggo(self) -> BVector4: ...
    @property
    def rggl(self) -> BVector4: ...
    @property
    def rgbr(self) -> BVector4: ...
    @property
    def rgbg(self) -> BVector4: ...
    @property
    def rgbb(self) -> BVector4: ...
    @property
    def rgba(self) -> BVector4: ...
    @property
    def rgbo(self) -> BVector4: ...
    @property
    def rgbl(self) -> BVector4: ...
    @property
    def rgar(self) -> BVector4: ...
    @property
    def rgag(self) -> BVector4: ...
    @property
    def rgab(self) -> BVector4: ...
    @property
    def rgaa(self) -> BVector4: ...
    @property
    def rgao(self) -> BVector4: ...
    @property
    def rgal(self) -> BVector4: ...
    @property
    def rgor(self) -> BVector4: ...
    @property
    def rgog(self) -> BVector4: ...
    @property
    def rgob(self) -> BVector4: ...
    @property
    def rgoa(self) -> BVector4: ...
    @property
    def rgoo(self) -> BVector4: ...
    @property
    def rgol(self) -> BVector4: ...
    @property
    def rglr(self) -> BVector4: ...
    @property
    def rglg(self) -> BVector4: ...
    @property
    def rglb(self) -> BVector4: ...
    @property
    def rgla(self) -> BVector4: ...
    @property
    def rglo(self) -> BVector4: ...
    @property
    def rgll(self) -> BVector4: ...
    @property
    def rbrr(self) -> BVector4: ...
    @property
    def rbrg(self) -> BVector4: ...
    @property
    def rbrb(self) -> BVector4: ...
    @property
    def rbra(self) -> BVector4: ...
    @property
    def rbro(self) -> BVector4: ...
    @property
    def rbrl(self) -> BVector4: ...
    @property
    def rbgr(self) -> BVector4: ...
    @property
    def rbgg(self) -> BVector4: ...
    @property
    def rbgb(self) -> BVector4: ...
    @property
    def rbga(self) -> BVector4: ...
    @property
    def rbgo(self) -> BVector4: ...
    @property
    def rbgl(self) -> BVector4: ...
    @property
    def rbbr(self) -> BVector4: ...
    @property
    def rbbg(self) -> BVector4: ...
    @property
    def rbbb(self) -> BVector4: ...
    @property
    def rbba(self) -> BVector4: ...
    @property
    def rbbo(self) -> BVector4: ...
    @property
    def rbbl(self) -> BVector4: ...
    @property
    def rbar(self) -> BVector4: ...
    @property
    def rbag(self) -> BVector4: ...
    @property
    def rbab(self) -> BVector4: ...
    @property
    def rbaa(self) -> BVector4: ...
    @property
    def rbao(self) -> BVector4: ...
    @property
    def rbal(self) -> BVector4: ...
    @property
    def rbor(self) -> BVector4: ...
    @property
    def rbog(self) -> BVector4: ...
    @property
    def rbob(self) -> BVector4: ...
    @property
    def rboa(self) -> BVector4: ...
    @property
    def rboo(self) -> BVector4: ...
    @property
    def rbol(self) -> BVector4: ...
    @property
    def rblr(self) -> BVector4: ...
    @property
    def rblg(self) -> BVector4: ...
    @property
    def rblb(self) -> BVector4: ...
    @property
    def rbla(self) -> BVector4: ...
    @property
    def rblo(self) -> BVector4: ...
    @property
    def rbll(self) -> BVector4: ...
    @property
    def rarr(self) -> BVector4: ...
    @property
    def rarg(self) -> BVector4: ...
    @property
    def rarb(self) -> BVector4: ...
    @property
    def rara(self) -> BVector4: ...
    @property
    def raro(self) -> BVector4: ...
    @property
    def rarl(self) -> BVector4: ...
    @property
    def ragr(self) -> BVector4: ...
    @property
    def ragg(self) -> BVector4: ...
    @property
    def ragb(self) -> BVector4: ...
    @property
    def raga(self) -> BVector4: ...
    @property
    def rago(self) -> BVector4: ...
    @property
    def ragl(self) -> BVector4: ...
    @property
    def rabr(self) -> BVector4: ...
    @property
    def rabg(self) -> BVector4: ...
    @property
    def rabb(self) -> BVector4: ...
    @property
    def raba(self) -> BVector4: ...
    @property
    def rabo(self) -> BVector4: ...
    @property
    def rabl(self) -> BVector4: ...
    @property
    def raar(self) -> BVector4: ...
    @property
    def raag(self) -> BVector4: ...
    @property
    def raab(self) -> BVector4: ...
    @property
    def raaa(self) -> BVector4: ...
    @property
    def raao(self) -> BVector4: ...
    @property
    def raal(self) -> BVector4: ...
    @property
    def raor(self) -> BVector4: ...
    @property
    def raog(self) -> BVector4: ...
    @property
    def raob(self) -> BVector4: ...
    @property
    def raoa(self) -> BVector4: ...
    @property
    def raoo(self) -> BVector4: ...
    @property
    def raol(self) -> BVector4: ...
    @property
    def ralr(self) -> BVector4: ...
    @property
    def ralg(self) -> BVector4: ...
    @property
    def ralb(self) -> BVector4: ...
    @property
    def rala(self) -> BVector4: ...
    @property
    def ralo(self) -> BVector4: ...
    @property
    def rall(self) -> BVector4: ...
    @property
    def rorr(self) -> BVector4: ...
    @property
    def rorg(self) -> BVector4: ...
    @property
    def rorb(self) -> BVector4: ...
    @property
    def rora(self) -> BVector4: ...
    @property
    def roro(self) -> BVector4: ...
    @property
    def rorl(self) -> BVector4: ...
    @property
    def rogr(self) -> BVector4: ...
    @property
    def rogg(self) -> BVector4: ...
    @property
    def rogb(self) -> BVector4: ...
    @property
    def roga(self) -> BVector4: ...
    @property
    def rogo(self) -> BVector4: ...
    @property
    def rogl(self) -> BVector4: ...
    @property
    def robr(self) -> BVector4: ...
    @property
    def robg(self) -> BVector4: ...
    @property
    def robb(self) -> BVector4: ...
    @property
    def roba(self) -> BVector4: ...
    @property
    def robo(self) -> BVector4: ...
    @property
    def robl(self) -> BVector4: ...
    @property
    def roar(self) -> BVector4: ...
    @property
    def roag(self) -> BVector4: ...
    @property
    def roab(self) -> BVector4: ...
    @property
    def roaa(self) -> BVector4: ...
    @property
    def roao(self) -> BVector4: ...
    @property
    def roal(self) -> BVector4: ...
    @property
    def roor(self) -> BVector4: ...
    @property
    def roog(self) -> BVector4: ...
    @property
    def roob(self) -> BVector4: ...
    @property
    def rooa(self) -> BVector4: ...
    @property
    def rooo(self) -> BVector4: ...
    @property
    def rool(self) -> BVector4: ...
    @property
    def rolr(self) -> BVector4: ...
    @property
    def rolg(self) -> BVector4: ...
    @property
    def rolb(self) -> BVector4: ...
    @property
    def rola(self) -> BVector4: ...
    @property
    def rolo(self) -> BVector4: ...
    @property
    def roll(self) -> BVector4: ...
    @property
    def rlrr(self) -> BVector4: ...
    @property
    def rlrg(self) -> BVector4: ...
    @property
    def rlrb(self) -> BVector4: ...
    @property
    def rlra(self) -> BVector4: ...
    @property
    def rlro(self) -> BVector4: ...
    @property
    def rlrl(self) -> BVector4: ...
    @property
    def rlgr(self) -> BVector4: ...
    @property
    def rlgg(self) -> BVector4: ...
    @property
    def rlgb(self) -> BVector4: ...
    @property
    def rlga(self) -> BVector4: ...
    @property
    def rlgo(self) -> BVector4: ...
    @property
    def rlgl(self) -> BVector4: ...
    @property
    def rlbr(self) -> BVector4: ...
    @property
    def rlbg(self) -> BVector4: ...
    @property
    def rlbb(self) -> BVector4: ...
    @property
    def rlba(self) -> BVector4: ...
    @property
    def rlbo(self) -> BVector4: ...
    @property
    def rlbl(self) -> BVector4: ...
    @property
    def rlar(self) -> BVector4: ...
    @property
    def rlag(self) -> BVector4: ...
    @property
    def rlab(self) -> BVector4: ...
    @property
    def rlaa(self) -> BVector4: ...
    @property
    def rlao(self) -> BVector4: ...
    @property
    def rlal(self) -> BVector4: ...
    @property
    def rlor(self) -> BVector4: ...
    @property
    def rlog(self) -> BVector4: ...
    @property
    def rlob(self) -> BVector4: ...
    @property
    def rloa(self) -> BVector4: ...
    @property
    def rloo(self) -> BVector4: ...
    @property
    def rlol(self) -> BVector4: ...
    @property
    def rllr(self) -> BVector4: ...
    @property
    def rllg(self) -> BVector4: ...
    @property
    def rllb(self) -> BVector4: ...
    @property
    def rlla(self) -> BVector4: ...
    @property
    def rllo(self) -> BVector4: ...
    @property
    def rlll(self) -> BVector4: ...
    @property
    def grrr(self) -> BVector4: ...
    @property
    def grrg(self) -> BVector4: ...
    @property
    def grrb(self) -> BVector4: ...
    @property
    def grra(self) -> BVector4: ...
    @property
    def grro(self) -> BVector4: ...
    @property
    def grrl(self) -> BVector4: ...
    @property
    def grgr(self) -> BVector4: ...
    @property
    def grgg(self) -> BVector4: ...
    @property
    def grgb(self) -> BVector4: ...
    @property
    def grga(self) -> BVector4: ...
    @property
    def grgo(self) -> BVector4: ...
    @property
    def grgl(self) -> BVector4: ...
    @property
    def grbr(self) -> BVector4: ...
    @property
    def grbg(self) -> BVector4: ...
    @property
    def grbb(self) -> BVector4: ...
    @property
    def grba(self) -> BVector4: ...
    @property
    def grbo(self) -> BVector4: ...
    @property
    def grbl(self) -> BVector4: ...
    @property
    def grar(self) -> BVector4: ...
    @property
    def grag(self) -> BVector4: ...
    @property
    def grab(self) -> BVector4: ...
    @property
    def graa(self) -> BVector4: ...
    @property
    def grao(self) -> BVector4: ...
    @property
    def gral(self) -> BVector4: ...
    @property
    def gror(self) -> BVector4: ...
    @property
    def grog(self) -> BVector4: ...
    @property
    def grob(self) -> BVector4: ...
    @property
    def groa(self) -> BVector4: ...
    @property
    def groo(self) -> BVector4: ...
    @property
    def grol(self) -> BVector4: ...
    @property
    def grlr(self) -> BVector4: ...
    @property
    def grlg(self) -> BVector4: ...
    @property
    def grlb(self) -> BVector4: ...
    @property
    def grla(self) -> BVector4: ...
    @property
    def grlo(self) -> BVector4: ...
    @property
    def grll(self) -> BVector4: ...
    @property
    def ggrr(self) -> BVector4: ...
    @property
    def ggrg(self) -> BVector4: ...
    @property
    def ggrb(self) -> BVector4: ...
    @property
    def ggra(self) -> BVector4: ...
    @property
    def ggro(self) -> BVector4: ...
    @property
    def ggrl(self) -> BVector4: ...
    @property
    def gggr(self) -> BVector4: ...
    @property
    def gggg(self) -> BVector4: ...
    @property
    def gggb(self) -> BVector4: ...
    @property
    def ggga(self) -> BVector4: ...
    @property
    def gggo(self) -> BVector4: ...
    @property
    def gggl(self) -> BVector4: ...
    @property
    def ggbr(self) -> BVector4: ...
    @property
    def ggbg(self) -> BVector4: ...
    @property
    def ggbb(self) -> BVector4: ...
    @property
    def ggba(self) -> BVector4: ...
    @property
    def ggbo(self) -> BVector4: ...
    @property
    def ggbl(self) -> BVector4: ...
    @property
    def ggar(self) -> BVector4: ...
    @property
    def ggag(self) -> BVector4: ...
    @property
    def ggab(self) -> BVector4: ...
    @property
    def ggaa(self) -> BVector4: ...
    @property
    def ggao(self) -> BVector4: ...
    @property
    def ggal(self) -> BVector4: ...
    @property
    def ggor(self) -> BVector4: ...
    @property
    def ggog(self) -> BVector4: ...
    @property
    def ggob(self) -> BVector4: ...
    @property
    def ggoa(self) -> BVector4: ...
    @property
    def ggoo(self) -> BVector4: ...
    @property
    def ggol(self) -> BVector4: ...
    @property
    def gglr(self) -> BVector4: ...
    @property
    def gglg(self) -> BVector4: ...
    @property
    def gglb(self) -> BVector4: ...
    @property
    def ggla(self) -> BVector4: ...
    @property
    def gglo(self) -> BVector4: ...
    @property
    def ggll(self) -> BVector4: ...
    @property
    def gbrr(self) -> BVector4: ...
    @property
    def gbrg(self) -> BVector4: ...
    @property
    def gbrb(self) -> BVector4: ...
    @property
    def gbra(self) -> BVector4: ...
    @property
    def gbro(self) -> BVector4: ...
    @property
    def gbrl(self) -> BVector4: ...
    @property
    def gbgr(self) -> BVector4: ...
    @property
    def gbgg(self) -> BVector4: ...
    @property
    def gbgb(self) -> BVector4: ...
    @property
    def gbga(self) -> BVector4: ...
    @property
    def gbgo(self) -> BVector4: ...
    @property
    def gbgl(self) -> BVector4: ...
    @property
    def gbbr(self) -> BVector4: ...
    @property
    def gbbg(self) -> BVector4: ...
    @property
    def gbbb(self) -> BVector4: ...
    @property
    def gbba(self) -> BVector4: ...
    @property
    def gbbo(self) -> BVector4: ...
    @property
    def gbbl(self) -> BVector4: ...
    @property
    def gbar(self) -> BVector4: ...
    @property
    def gbag(self) -> BVector4: ...
    @property
    def gbab(self) -> BVector4: ...
    @property
    def gbaa(self) -> BVector4: ...
    @property
    def gbao(self) -> BVector4: ...
    @property
    def gbal(self) -> BVector4: ...
    @property
    def gbor(self) -> BVector4: ...
    @property
    def gbog(self) -> BVector4: ...
    @property
    def gbob(self) -> BVector4: ...
    @property
    def gboa(self) -> BVector4: ...
    @property
    def gboo(self) -> BVector4: ...
    @property
    def gbol(self) -> BVector4: ...
    @property
    def gblr(self) -> BVector4: ...
    @property
    def gblg(self) -> BVector4: ...
    @property
    def gblb(self) -> BVector4: ...
    @property
    def gbla(self) -> BVector4: ...
    @property
    def gblo(self) -> BVector4: ...
    @property
    def gbll(self) -> BVector4: ...
    @property
    def garr(self) -> BVector4: ...
    @property
    def garg(self) -> BVector4: ...
    @property
    def garb(self) -> BVector4: ...
    @property
    def gara(self) -> BVector4: ...
    @property
    def garo(self) -> BVector4: ...
    @property
    def garl(self) -> BVector4: ...
    @property
    def gagr(self) -> BVector4: ...
    @property
    def gagg(self) -> BVector4: ...
    @property
    def gagb(self) -> BVector4: ...
    @property
    def gaga(self) -> BVector4: ...
    @property
    def gago(self) -> BVector4: ...
    @property
    def gagl(self) -> BVector4: ...
    @property
    def gabr(self) -> BVector4: ...
    @property
    def gabg(self) -> BVector4: ...
    @property
    def gabb(self) -> BVector4: ...
    @property
    def gaba(self) -> BVector4: ...
    @property
    def gabo(self) -> BVector4: ...
    @property
    def gabl(self) -> BVector4: ...
    @property
    def gaar(self) -> BVector4: ...
    @property
    def gaag(self) -> BVector4: ...
    @property
    def gaab(self) -> BVector4: ...
    @property
    def gaaa(self) -> BVector4: ...
    @property
    def gaao(self) -> BVector4: ...
    @property
    def gaal(self) -> BVector4: ...
    @property
    def gaor(self) -> BVector4: ...
    @property
    def gaog(self) -> BVector4: ...
    @property
    def gaob(self) -> BVector4: ...
    @property
    def gaoa(self) -> BVector4: ...
    @property
    def gaoo(self) -> BVector4: ...
    @property
    def gaol(self) -> BVector4: ...
    @property
    def galr(self) -> BVector4: ...
    @property
    def galg(self) -> BVector4: ...
    @property
    def galb(self) -> BVector4: ...
    @property
    def gala(self) -> BVector4: ...
    @property
    def galo(self) -> BVector4: ...
    @property
    def gall(self) -> BVector4: ...
    @property
    def gorr(self) -> BVector4: ...
    @property
    def gorg(self) -> BVector4: ...
    @property
    def gorb(self) -> BVector4: ...
    @property
    def gora(self) -> BVector4: ...
    @property
    def goro(self) -> BVector4: ...
    @property
    def gorl(self) -> BVector4: ...
    @property
    def gogr(self) -> BVector4: ...
    @property
    def gogg(self) -> BVector4: ...
    @property
    def gogb(self) -> BVector4: ...
    @property
    def goga(self) -> BVector4: ...
    @property
    def gogo(self) -> BVector4: ...
    @property
    def gogl(self) -> BVector4: ...
    @property
    def gobr(self) -> BVector4: ...
    @property
    def gobg(self) -> BVector4: ...
    @property
    def gobb(self) -> BVector4: ...
    @property
    def goba(self) -> BVector4: ...
    @property
    def gobo(self) -> BVector4: ...
    @property
    def gobl(self) -> BVector4: ...
    @property
    def goar(self) -> BVector4: ...
    @property
    def goag(self) -> BVector4: ...
    @property
    def goab(self) -> BVector4: ...
    @property
    def goaa(self) -> BVector4: ...
    @property
    def goao(self) -> BVector4: ...
    @property
    def goal(self) -> BVector4: ...
    @property
    def goor(self) -> BVector4: ...
    @property
    def goog(self) -> BVector4: ...
    @property
    def goob(self) -> BVector4: ...
    @property
    def gooa(self) -> BVector4: ...
    @property
    def gooo(self) -> BVector4: ...
    @property
    def gool(self) -> BVector4: ...
    @property
    def golr(self) -> BVector4: ...
    @property
    def golg(self) -> BVector4: ...
    @property
    def golb(self) -> BVector4: ...
    @property
    def gola(self) -> BVector4: ...
    @property
    def golo(self) -> BVector4: ...
    @property
    def goll(self) -> BVector4: ...
    @property
    def glrr(self) -> BVector4: ...
    @property
    def glrg(self) -> BVector4: ...
    @property
    def glrb(self) -> BVector4: ...
    @property
    def glra(self) -> BVector4: ...
    @property
    def glro(self) -> BVector4: ...
    @property
    def glrl(self) -> BVector4: ...
    @property
    def glgr(self) -> BVector4: ...
    @property
    def glgg(self) -> BVector4: ...
    @property
    def glgb(self) -> BVector4: ...
    @property
    def glga(self) -> BVector4: ...
    @property
    def glgo(self) -> BVector4: ...
    @property
    def glgl(self) -> BVector4: ...
    @property
    def glbr(self) -> BVector4: ...
    @property
    def glbg(self) -> BVector4: ...
    @property
    def glbb(self) -> BVector4: ...
    @property
    def glba(self) -> BVector4: ...
    @property
    def glbo(self) -> BVector4: ...
    @property
    def glbl(self) -> BVector4: ...
    @property
    def glar(self) -> BVector4: ...
    @property
    def glag(self) -> BVector4: ...
    @property
    def glab(self) -> BVector4: ...
    @property
    def glaa(self) -> BVector4: ...
    @property
    def glao(self) -> BVector4: ...
    @property
    def glal(self) -> BVector4: ...
    @property
    def glor(self) -> BVector4: ...
    @property
    def glog(self) -> BVector4: ...
    @property
    def glob(self) -> BVector4: ...
    @property
    def gloa(self) -> BVector4: ...
    @property
    def gloo(self) -> BVector4: ...
    @property
    def glol(self) -> BVector4: ...
    @property
    def gllr(self) -> BVector4: ...
    @property
    def gllg(self) -> BVector4: ...
    @property
    def gllb(self) -> BVector4: ...
    @property
    def glla(self) -> BVector4: ...
    @property
    def gllo(self) -> BVector4: ...
    @property
    def glll(self) -> BVector4: ...
    @property
    def brrr(self) -> BVector4: ...
    @property
    def brrg(self) -> BVector4: ...
    @property
    def brrb(self) -> BVector4: ...
    @property
    def brra(self) -> BVector4: ...
    @property
    def brro(self) -> BVector4: ...
    @property
    def brrl(self) -> BVector4: ...
    @property
    def brgr(self) -> BVector4: ...
    @property
    def brgg(self) -> BVector4: ...
    @property
    def brgb(self) -> BVector4: ...
    @property
    def brga(self) -> BVector4: ...
    @property
    def brgo(self) -> BVector4: ...
    @property
    def brgl(self) -> BVector4: ...
    @property
    def brbr(self) -> BVector4: ...
    @property
    def brbg(self) -> BVector4: ...
    @property
    def brbb(self) -> BVector4: ...
    @property
    def brba(self) -> BVector4: ...
    @property
    def brbo(self) -> BVector4: ...
    @property
    def brbl(self) -> BVector4: ...
    @property
    def brar(self) -> BVector4: ...
    @property
    def brag(self) -> BVector4: ...
    @property
    def brab(self) -> BVector4: ...
    @property
    def braa(self) -> BVector4: ...
    @property
    def brao(self) -> BVector4: ...
    @property
    def bral(self) -> BVector4: ...
    @property
    def bror(self) -> BVector4: ...
    @property
    def brog(self) -> BVector4: ...
    @property
    def brob(self) -> BVector4: ...
    @property
    def broa(self) -> BVector4: ...
    @property
    def broo(self) -> BVector4: ...
    @property
    def brol(self) -> BVector4: ...
    @property
    def brlr(self) -> BVector4: ...
    @property
    def brlg(self) -> BVector4: ...
    @property
    def brlb(self) -> BVector4: ...
    @property
    def brla(self) -> BVector4: ...
    @property
    def brlo(self) -> BVector4: ...
    @property
    def brll(self) -> BVector4: ...
    @property
    def bgrr(self) -> BVector4: ...
    @property
    def bgrg(self) -> BVector4: ...
    @property
    def bgrb(self) -> BVector4: ...
    @property
    def bgra(self) -> BVector4: ...
    @property
    def bgro(self) -> BVector4: ...
    @property
    def bgrl(self) -> BVector4: ...
    @property
    def bggr(self) -> BVector4: ...
    @property
    def bggg(self) -> BVector4: ...
    @property
    def bggb(self) -> BVector4: ...
    @property
    def bgga(self) -> BVector4: ...
    @property
    def bggo(self) -> BVector4: ...
    @property
    def bggl(self) -> BVector4: ...
    @property
    def bgbr(self) -> BVector4: ...
    @property
    def bgbg(self) -> BVector4: ...
    @property
    def bgbb(self) -> BVector4: ...
    @property
    def bgba(self) -> BVector4: ...
    @property
    def bgbo(self) -> BVector4: ...
    @property
    def bgbl(self) -> BVector4: ...
    @property
    def bgar(self) -> BVector4: ...
    @property
    def bgag(self) -> BVector4: ...
    @property
    def bgab(self) -> BVector4: ...
    @property
    def bgaa(self) -> BVector4: ...
    @property
    def bgao(self) -> BVector4: ...
    @property
    def bgal(self) -> BVector4: ...
    @property
    def bgor(self) -> BVector4: ...
    @property
    def bgog(self) -> BVector4: ...
    @property
    def bgob(self) -> BVector4: ...
    @property
    def bgoa(self) -> BVector4: ...
    @property
    def bgoo(self) -> BVector4: ...
    @property
    def bgol(self) -> BVector4: ...
    @property
    def bglr(self) -> BVector4: ...
    @property
    def bglg(self) -> BVector4: ...
    @property
    def bglb(self) -> BVector4: ...
    @property
    def bgla(self) -> BVector4: ...
    @property
    def bglo(self) -> BVector4: ...
    @property
    def bgll(self) -> BVector4: ...
    @property
    def bbrr(self) -> BVector4: ...
    @property
    def bbrg(self) -> BVector4: ...
    @property
    def bbrb(self) -> BVector4: ...
    @property
    def bbra(self) -> BVector4: ...
    @property
    def bbro(self) -> BVector4: ...
    @property
    def bbrl(self) -> BVector4: ...
    @property
    def bbgr(self) -> BVector4: ...
    @property
    def bbgg(self) -> BVector4: ...
    @property
    def bbgb(self) -> BVector4: ...
    @property
    def bbga(self) -> BVector4: ...
    @property
    def bbgo(self) -> BVector4: ...
    @property
    def bbgl(self) -> BVector4: ...
    @property
    def bbbr(self) -> BVector4: ...
    @property
    def bbbg(self) -> BVector4: ...
    @property
    def bbbb(self) -> BVector4: ...
    @property
    def bbba(self) -> BVector4: ...
    @property
    def bbbo(self) -> BVector4: ...
    @property
    def bbbl(self) -> BVector4: ...
    @property
    def bbar(self) -> BVector4: ...
    @property
    def bbag(self) -> BVector4: ...
    @property
    def bbab(self) -> BVector4: ...
    @property
    def bbaa(self) -> BVector4: ...
    @property
    def bbao(self) -> BVector4: ...
    @property
    def bbal(self) -> BVector4: ...
    @property
    def bbor(self) -> BVector4: ...
    @property
    def bbog(self) -> BVector4: ...
    @property
    def bbob(self) -> BVector4: ...
    @property
    def bboa(self) -> BVector4: ...
    @property
    def bboo(self) -> BVector4: ...
    @property
    def bbol(self) -> BVector4: ...
    @property
    def bblr(self) -> BVector4: ...
    @property
    def bblg(self) -> BVector4: ...
    @property
    def bblb(self) -> BVector4: ...
    @property
    def bbla(self) -> BVector4: ...
    @property
    def bblo(self) -> BVector4: ...
    @property
    def bbll(self) -> BVector4: ...
    @property
    def barr(self) -> BVector4: ...
    @property
    def barg(self) -> BVector4: ...
    @property
    def barb(self) -> BVector4: ...
    @property
    def bara(self) -> BVector4: ...
    @property
    def baro(self) -> BVector4: ...
    @property
    def barl(self) -> BVector4: ...
    @property
    def bagr(self) -> BVector4: ...
    @property
    def bagg(self) -> BVector4: ...
    @property
    def bagb(self) -> BVector4: ...
    @property
    def baga(self) -> BVector4: ...
    @property
    def bago(self) -> BVector4: ...
    @property
    def bagl(self) -> BVector4: ...
    @property
    def babr(self) -> BVector4: ...
    @property
    def babg(self) -> BVector4: ...
    @property
    def babb(self) -> BVector4: ...
    @property
    def baba(self) -> BVector4: ...
    @property
    def babo(self) -> BVector4: ...
    @property
    def babl(self) -> BVector4: ...
    @property
    def baar(self) -> BVector4: ...
    @property
    def baag(self) -> BVector4: ...
    @property
    def baab(self) -> BVector4: ...
    @property
    def baaa(self) -> BVector4: ...
    @property
    def baao(self) -> BVector4: ...
    @property
    def baal(self) -> BVector4: ...
    @property
    def baor(self) -> BVector4: ...
    @property
    def baog(self) -> BVector4: ...
    @property
    def baob(self) -> BVector4: ...
    @property
    def baoa(self) -> BVector4: ...
    @property
    def baoo(self) -> BVector4: ...
    @property
    def baol(self) -> BVector4: ...
    @property
    def balr(self) -> BVector4: ...
    @property
    def balg(self) -> BVector4: ...
    @property
    def balb(self) -> BVector4: ...
    @property
    def bala(self) -> BVector4: ...
    @property
    def balo(self) -> BVector4: ...
    @property
    def ball(self) -> BVector4: ...
    @property
    def borr(self) -> BVector4: ...
    @property
    def borg(self) -> BVector4: ...
    @property
    def borb(self) -> BVector4: ...
    @property
    def bora(self) -> BVector4: ...
    @property
    def boro(self) -> BVector4: ...
    @property
    def borl(self) -> BVector4: ...
    @property
    def bogr(self) -> BVector4: ...
    @property
    def bogg(self) -> BVector4: ...
    @property
    def bogb(self) -> BVector4: ...
    @property
    def boga(self) -> BVector4: ...
    @property
    def bogo(self) -> BVector4: ...
    @property
    def bogl(self) -> BVector4: ...
    @property
    def bobr(self) -> BVector4: ...
    @property
    def bobg(self) -> BVector4: ...
    @property
    def bobb(self) -> BVector4: ...
    @property
    def boba(self) -> BVector4: ...
    @property
    def bobo(self) -> BVector4: ...
    @property
    def bobl(self) -> BVector4: ...
    @property
    def boar(self) -> BVector4: ...
    @property
    def boag(self) -> BVector4: ...
    @property
    def boab(self) -> BVector4: ...
    @property
    def boaa(self) -> BVector4: ...
    @property
    def boao(self) -> BVector4: ...
    @property
    def boal(self) -> BVector4: ...
    @property
    def boor(self) -> BVector4: ...
    @property
    def boog(self) -> BVector4: ...
    @property
    def boob(self) -> BVector4: ...
    @property
    def booa(self) -> BVector4: ...
    @property
    def booo(self) -> BVector4: ...
    @property
    def bool(self) -> BVector4: ...
    @property
    def bolr(self) -> BVector4: ...
    @property
    def bolg(self) -> BVector4: ...
    @property
    def bolb(self) -> BVector4: ...
    @property
    def bola(self) -> BVector4: ...
    @property
    def bolo(self) -> BVector4: ...
    @property
    def boll(self) -> BVector4: ...
    @property
    def blrr(self) -> BVector4: ...
    @property
    def blrg(self) -> BVector4: ...
    @property
    def blrb(self) -> BVector4: ...
    @property
    def blra(self) -> BVector4: ...
    @property
    def blro(self) -> BVector4: ...
    @property
    def blrl(self) -> BVector4: ...
    @property
    def blgr(self) -> BVector4: ...
    @property
    def blgg(self) -> BVector4: ...
    @property
    def blgb(self) -> BVector4: ...
    @property
    def blga(self) -> BVector4: ...
    @property
    def blgo(self) -> BVector4: ...
    @property
    def blgl(self) -> BVector4: ...
    @property
    def blbr(self) -> BVector4: ...
    @property
    def blbg(self) -> BVector4: ...
    @property
    def blbb(self) -> BVector4: ...
    @property
    def blba(self) -> BVector4: ...
    @property
    def blbo(self) -> BVector4: ...
    @property
    def blbl(self) -> BVector4: ...
    @property
    def blar(self) -> BVector4: ...
    @property
    def blag(self) -> BVector4: ...
    @property
    def blab(self) -> BVector4: ...
    @property
    def blaa(self) -> BVector4: ...
    @property
    def blao(self) -> BVector4: ...
    @property
    def blal(self) -> BVector4: ...
    @property
    def blor(self) -> BVector4: ...
    @property
    def blog(self) -> BVector4: ...
    @property
    def blob(self) -> BVector4: ...
    @property
    def bloa(self) -> BVector4: ...
    @property
    def bloo(self) -> BVector4: ...
    @property
    def blol(self) -> BVector4: ...
    @property
    def bllr(self) -> BVector4: ...
    @property
    def bllg(self) -> BVector4: ...
    @property
    def bllb(self) -> BVector4: ...
    @property
    def blla(self) -> BVector4: ...
    @property
    def bllo(self) -> BVector4: ...
    @property
    def blll(self) -> BVector4: ...
    @property
    def arrr(self) -> BVector4: ...
    @property
    def arrg(self) -> BVector4: ...
    @property
    def arrb(self) -> BVector4: ...
    @property
    def arra(self) -> BVector4: ...
    @property
    def arro(self) -> BVector4: ...
    @property
    def arrl(self) -> BVector4: ...
    @property
    def argr(self) -> BVector4: ...
    @property
    def argg(self) -> BVector4: ...
    @property
    def argb(self) -> BVector4: ...
    @property
    def arga(self) -> BVector4: ...
    @property
    def argo(self) -> BVector4: ...
    @property
    def argl(self) -> BVector4: ...
    @property
    def arbr(self) -> BVector4: ...
    @property
    def arbg(self) -> BVector4: ...
    @property
    def arbb(self) -> BVector4: ...
    @property
    def arba(self) -> BVector4: ...
    @property
    def arbo(self) -> BVector4: ...
    @property
    def arbl(self) -> BVector4: ...
    @property
    def arar(self) -> BVector4: ...
    @property
    def arag(self) -> BVector4: ...
    @property
    def arab(self) -> BVector4: ...
    @property
    def araa(self) -> BVector4: ...
    @property
    def arao(self) -> BVector4: ...
    @property
    def aral(self) -> BVector4: ...
    @property
    def aror(self) -> BVector4: ...
    @property
    def arog(self) -> BVector4: ...
    @property
    def arob(self) -> BVector4: ...
    @property
    def aroa(self) -> BVector4: ...
    @property
    def aroo(self) -> BVector4: ...
    @property
    def arol(self) -> BVector4: ...
    @property
    def arlr(self) -> BVector4: ...
    @property
    def arlg(self) -> BVector4: ...
    @property
    def arlb(self) -> BVector4: ...
    @property
    def arla(self) -> BVector4: ...
    @property
    def arlo(self) -> BVector4: ...
    @property
    def arll(self) -> BVector4: ...
    @property
    def agrr(self) -> BVector4: ...
    @property
    def agrg(self) -> BVector4: ...
    @property
    def agrb(self) -> BVector4: ...
    @property
    def agra(self) -> BVector4: ...
    @property
    def agro(self) -> BVector4: ...
    @property
    def agrl(self) -> BVector4: ...
    @property
    def aggr(self) -> BVector4: ...
    @property
    def aggg(self) -> BVector4: ...
    @property
    def aggb(self) -> BVector4: ...
    @property
    def agga(self) -> BVector4: ...
    @property
    def aggo(self) -> BVector4: ...
    @property
    def aggl(self) -> BVector4: ...
    @property
    def agbr(self) -> BVector4: ...
    @property
    def agbg(self) -> BVector4: ...
    @property
    def agbb(self) -> BVector4: ...
    @property
    def agba(self) -> BVector4: ...
    @property
    def agbo(self) -> BVector4: ...
    @property
    def agbl(self) -> BVector4: ...
    @property
    def agar(self) -> BVector4: ...
    @property
    def agag(self) -> BVector4: ...
    @property
    def agab(self) -> BVector4: ...
    @property
    def agaa(self) -> BVector4: ...
    @property
    def agao(self) -> BVector4: ...
    @property
    def agal(self) -> BVector4: ...
    @property
    def agor(self) -> BVector4: ...
    @property
    def agog(self) -> BVector4: ...
    @property
    def agob(self) -> BVector4: ...
    @property
    def agoa(self) -> BVector4: ...
    @property
    def agoo(self) -> BVector4: ...
    @property
    def agol(self) -> BVector4: ...
    @property
    def aglr(self) -> BVector4: ...
    @property
    def aglg(self) -> BVector4: ...
    @property
    def aglb(self) -> BVector4: ...
    @property
    def agla(self) -> BVector4: ...
    @property
    def aglo(self) -> BVector4: ...
    @property
    def agll(self) -> BVector4: ...
    @property
    def abrr(self) -> BVector4: ...
    @property
    def abrg(self) -> BVector4: ...
    @property
    def abrb(self) -> BVector4: ...
    @property
    def abra(self) -> BVector4: ...
    @property
    def abro(self) -> BVector4: ...
    @property
    def abrl(self) -> BVector4: ...
    @property
    def abgr(self) -> BVector4: ...
    @property
    def abgg(self) -> BVector4: ...
    @property
    def abgb(self) -> BVector4: ...
    @property
    def abga(self) -> BVector4: ...
    @property
    def abgo(self) -> BVector4: ...
    @property
    def abgl(self) -> BVector4: ...
    @property
    def abbr(self) -> BVector4: ...
    @property
    def abbg(self) -> BVector4: ...
    @property
    def abbb(self) -> BVector4: ...
    @property
    def abba(self) -> BVector4: ...
    @property
    def abbo(self) -> BVector4: ...
    @property
    def abbl(self) -> BVector4: ...
    @property
    def abar(self) -> BVector4: ...
    @property
    def abag(self) -> BVector4: ...
    @property
    def abab(self) -> BVector4: ...
    @property
    def abaa(self) -> BVector4: ...
    @property
    def abao(self) -> BVector4: ...
    @property
    def abal(self) -> BVector4: ...
    @property
    def abor(self) -> BVector4: ...
    @property
    def abog(self) -> BVector4: ...
    @property
    def abob(self) -> BVector4: ...
    @property
    def aboa(self) -> BVector4: ...
    @property
    def aboo(self) -> BVector4: ...
    @property
    def abol(self) -> BVector4: ...
    @property
    def ablr(self) -> BVector4: ...
    @property
    def ablg(self) -> BVector4: ...
    @property
    def ablb(self) -> BVector4: ...
    @property
    def abla(self) -> BVector4: ...
    @property
    def ablo(self) -> BVector4: ...
    @property
    def abll(self) -> BVector4: ...
    @property
    def aarr(self) -> BVector4: ...
    @property
    def aarg(self) -> BVector4: ...
    @property
    def aarb(self) -> BVector4: ...
    @property
    def aara(self) -> BVector4: ...
    @property
    def aaro(self) -> BVector4: ...
    @property
    def aarl(self) -> BVector4: ...
    @property
    def aagr(self) -> BVector4: ...
    @property
    def aagg(self) -> BVector4: ...
    @property
    def aagb(self) -> BVector4: ...
    @property
    def aaga(self) -> BVector4: ...
    @property
    def aago(self) -> BVector4: ...
    @property
    def aagl(self) -> BVector4: ...
    @property
    def aabr(self) -> BVector4: ...
    @property
    def aabg(self) -> BVector4: ...
    @property
    def aabb(self) -> BVector4: ...
    @property
    def aaba(self) -> BVector4: ...
    @property
    def aabo(self) -> BVector4: ...
    @property
    def aabl(self) -> BVector4: ...
    @property
    def aaar(self) -> BVector4: ...
    @property
    def aaag(self) -> BVector4: ...
    @property
    def aaab(self) -> BVector4: ...
    @property
    def aaaa(self) -> BVector4: ...
    @property
    def aaao(self) -> BVector4: ...
    @property
    def aaal(self) -> BVector4: ...
    @property
    def aaor(self) -> BVector4: ...
    @property
    def aaog(self) -> BVector4: ...
    @property
    def aaob(self) -> BVector4: ...
    @property
    def aaoa(self) -> BVector4: ...
    @property
    def aaoo(self) -> BVector4: ...
    @property
    def aaol(self) -> BVector4: ...
    @property
    def aalr(self) -> BVector4: ...
    @property
    def aalg(self) -> BVector4: ...
    @property
    def aalb(self) -> BVector4: ...
    @property
    def aala(self) -> BVector4: ...
    @property
    def aalo(self) -> BVector4: ...
    @property
    def aall(self) -> BVector4: ...
    @property
    def aorr(self) -> BVector4: ...
    @property
    def aorg(self) -> BVector4: ...
    @property
    def aorb(self) -> BVector4: ...
    @property
    def aora(self) -> BVector4: ...
    @property
    def aoro(self) -> BVector4: ...
    @property
    def aorl(self) -> BVector4: ...
    @property
    def aogr(self) -> BVector4: ...
    @property
    def aogg(self) -> BVector4: ...
    @property
    def aogb(self) -> BVector4: ...
    @property
    def aoga(self) -> BVector4: ...
    @property
    def aogo(self) -> BVector4: ...
    @property
    def aogl(self) -> BVector4: ...
    @property
    def aobr(self) -> BVector4: ...
    @property
    def aobg(self) -> BVector4: ...
    @property
    def aobb(self) -> BVector4: ...
    @property
    def aoba(self) -> BVector4: ...
    @property
    def aobo(self) -> BVector4: ...
    @property
    def aobl(self) -> BVector4: ...
    @property
    def aoar(self) -> BVector4: ...
    @property
    def aoag(self) -> BVector4: ...
    @property
    def aoab(self) -> BVector4: ...
    @property
    def aoaa(self) -> BVector4: ...
    @property
    def aoao(self) -> BVector4: ...
    @property
    def aoal(self) -> BVector4: ...
    @property
    def aoor(self) -> BVector4: ...
    @property
    def aoog(self) -> BVector4: ...
    @property
    def aoob(self) -> BVector4: ...
    @property
    def aooa(self) -> BVector4: ...
    @property
    def aooo(self) -> BVector4: ...
    @property
    def aool(self) -> BVector4: ...
    @property
    def aolr(self) -> BVector4: ...
    @property
    def aolg(self) -> BVector4: ...
    @property
    def aolb(self) -> BVector4: ...
    @property
    def aola(self) -> BVector4: ...
    @property
    def aolo(self) -> BVector4: ...
    @property
    def aoll(self) -> BVector4: ...
    @property
    def alrr(self) -> BVector4: ...
    @property
    def alrg(self) -> BVector4: ...
    @property
    def alrb(self) -> BVector4: ...
    @property
    def alra(self) -> BVector4: ...
    @property
    def alro(self) -> BVector4: ...
    @property
    def alrl(self) -> BVector4: ...
    @property
    def algr(self) -> BVector4: ...
    @property
    def algg(self) -> BVector4: ...
    @property
    def algb(self) -> BVector4: ...
    @property
    def alga(self) -> BVector4: ...
    @property
    def algo(self) -> BVector4: ...
    @property
    def algl(self) -> BVector4: ...
    @property
    def albr(self) -> BVector4: ...
    @property
    def albg(self) -> BVector4: ...
    @property
    def albb(self) -> BVector4: ...
    @property
    def alba(self) -> BVector4: ...
    @property
    def albo(self) -> BVector4: ...
    @property
    def albl(self) -> BVector4: ...
    @property
    def alar(self) -> BVector4: ...
    @property
    def alag(self) -> BVector4: ...
    @property
    def alab(self) -> BVector4: ...
    @property
    def alaa(self) -> BVector4: ...
    @property
    def alao(self) -> BVector4: ...
    @property
    def alal(self) -> BVector4: ...
    @property
    def alor(self) -> BVector4: ...
    @property
    def alog(self) -> BVector4: ...
    @property
    def alob(self) -> BVector4: ...
    @property
    def aloa(self) -> BVector4: ...
    @property
    def aloo(self) -> BVector4: ...
    @property
    def alol(self) -> BVector4: ...
    @property
    def allr(self) -> BVector4: ...
    @property
    def allg(self) -> BVector4: ...
    @property
    def allb(self) -> BVector4: ...
    @property
    def alla(self) -> BVector4: ...
    @property
    def allo(self) -> BVector4: ...
    @property
    def alll(self) -> BVector4: ...
    @property
    def orrr(self) -> BVector4: ...
    @property
    def orrg(self) -> BVector4: ...
    @property
    def orrb(self) -> BVector4: ...
    @property
    def orra(self) -> BVector4: ...
    @property
    def orro(self) -> BVector4: ...
    @property
    def orrl(self) -> BVector4: ...
    @property
    def orgr(self) -> BVector4: ...
    @property
    def orgg(self) -> BVector4: ...
    @property
    def orgb(self) -> BVector4: ...
    @property
    def orga(self) -> BVector4: ...
    @property
    def orgo(self) -> BVector4: ...
    @property
    def orgl(self) -> BVector4: ...
    @property
    def orbr(self) -> BVector4: ...
    @property
    def orbg(self) -> BVector4: ...
    @property
    def orbb(self) -> BVector4: ...
    @property
    def orba(self) -> BVector4: ...
    @property
    def orbo(self) -> BVector4: ...
    @property
    def orbl(self) -> BVector4: ...
    @property
    def orar(self) -> BVector4: ...
    @property
    def orag(self) -> BVector4: ...
    @property
    def orab(self) -> BVector4: ...
    @property
    def oraa(self) -> BVector4: ...
    @property
    def orao(self) -> BVector4: ...
    @property
    def oral(self) -> BVector4: ...
    @property
    def oror(self) -> BVector4: ...
    @property
    def orog(self) -> BVector4: ...
    @property
    def orob(self) -> BVector4: ...
    @property
    def oroa(self) -> BVector4: ...
    @property
    def oroo(self) -> BVector4: ...
    @property
    def orol(self) -> BVector4: ...
    @property
    def orlr(self) -> BVector4: ...
    @property
    def orlg(self) -> BVector4: ...
    @property
    def orlb(self) -> BVector4: ...
    @property
    def orla(self) -> BVector4: ...
    @property
    def orlo(self) -> BVector4: ...
    @property
    def orll(self) -> BVector4: ...
    @property
    def ogrr(self) -> BVector4: ...
    @property
    def ogrg(self) -> BVector4: ...
    @property
    def ogrb(self) -> BVector4: ...
    @property
    def ogra(self) -> BVector4: ...
    @property
    def ogro(self) -> BVector4: ...
    @property
    def ogrl(self) -> BVector4: ...
    @property
    def oggr(self) -> BVector4: ...
    @property
    def oggg(self) -> BVector4: ...
    @property
    def oggb(self) -> BVector4: ...
    @property
    def ogga(self) -> BVector4: ...
    @property
    def oggo(self) -> BVector4: ...
    @property
    def oggl(self) -> BVector4: ...
    @property
    def ogbr(self) -> BVector4: ...
    @property
    def ogbg(self) -> BVector4: ...
    @property
    def ogbb(self) -> BVector4: ...
    @property
    def ogba(self) -> BVector4: ...
    @property
    def ogbo(self) -> BVector4: ...
    @property
    def ogbl(self) -> BVector4: ...
    @property
    def ogar(self) -> BVector4: ...
    @property
    def ogag(self) -> BVector4: ...
    @property
    def ogab(self) -> BVector4: ...
    @property
    def ogaa(self) -> BVector4: ...
    @property
    def ogao(self) -> BVector4: ...
    @property
    def ogal(self) -> BVector4: ...
    @property
    def ogor(self) -> BVector4: ...
    @property
    def ogog(self) -> BVector4: ...
    @property
    def ogob(self) -> BVector4: ...
    @property
    def ogoa(self) -> BVector4: ...
    @property
    def ogoo(self) -> BVector4: ...
    @property
    def ogol(self) -> BVector4: ...
    @property
    def oglr(self) -> BVector4: ...
    @property
    def oglg(self) -> BVector4: ...
    @property
    def oglb(self) -> BVector4: ...
    @property
    def ogla(self) -> BVector4: ...
    @property
    def oglo(self) -> BVector4: ...
    @property
    def ogll(self) -> BVector4: ...
    @property
    def obrr(self) -> BVector4: ...
    @property
    def obrg(self) -> BVector4: ...
    @property
    def obrb(self) -> BVector4: ...
    @property
    def obra(self) -> BVector4: ...
    @property
    def obro(self) -> BVector4: ...
    @property
    def obrl(self) -> BVector4: ...
    @property
    def obgr(self) -> BVector4: ...
    @property
    def obgg(self) -> BVector4: ...
    @property
    def obgb(self) -> BVector4: ...
    @property
    def obga(self) -> BVector4: ...
    @property
    def obgo(self) -> BVector4: ...
    @property
    def obgl(self) -> BVector4: ...
    @property
    def obbr(self) -> BVector4: ...
    @property
    def obbg(self) -> BVector4: ...
    @property
    def obbb(self) -> BVector4: ...
    @property
    def obba(self) -> BVector4: ...
    @property
    def obbo(self) -> BVector4: ...
    @property
    def obbl(self) -> BVector4: ...
    @property
    def obar(self) -> BVector4: ...
    @property
    def obag(self) -> BVector4: ...
    @property
    def obab(self) -> BVector4: ...
    @property
    def obaa(self) -> BVector4: ...
    @property
    def obao(self) -> BVector4: ...
    @property
    def obal(self) -> BVector4: ...
    @property
    def obor(self) -> BVector4: ...
    @property
    def obog(self) -> BVector4: ...
    @property
    def obob(self) -> BVector4: ...
    @property
    def oboa(self) -> BVector4: ...
    @property
    def oboo(self) -> BVector4: ...
    @property
    def obol(self) -> BVector4: ...
    @property
    def oblr(self) -> BVector4: ...
    @property
    def oblg(self) -> BVector4: ...
    @property
    def oblb(self) -> BVector4: ...
    @property
    def obla(self) -> BVector4: ...
    @property
    def oblo(self) -> BVector4: ...
    @property
    def obll(self) -> BVector4: ...
    @property
    def oarr(self) -> BVector4: ...
    @property
    def oarg(self) -> BVector4: ...
    @property
    def oarb(self) -> BVector4: ...
    @property
    def oara(self) -> BVector4: ...
    @property
    def oaro(self) -> BVector4: ...
    @property
    def oarl(self) -> BVector4: ...
    @property
    def oagr(self) -> BVector4: ...
    @property
    def oagg(self) -> BVector4: ...
    @property
    def oagb(self) -> BVector4: ...
    @property
    def oaga(self) -> BVector4: ...
    @property
    def oago(self) -> BVector4: ...
    @property
    def oagl(self) -> BVector4: ...
    @property
    def oabr(self) -> BVector4: ...
    @property
    def oabg(self) -> BVector4: ...
    @property
    def oabb(self) -> BVector4: ...
    @property
    def oaba(self) -> BVector4: ...
    @property
    def oabo(self) -> BVector4: ...
    @property
    def oabl(self) -> BVector4: ...
    @property
    def oaar(self) -> BVector4: ...
    @property
    def oaag(self) -> BVector4: ...
    @property
    def oaab(self) -> BVector4: ...
    @property
    def oaaa(self) -> BVector4: ...
    @property
    def oaao(self) -> BVector4: ...
    @property
    def oaal(self) -> BVector4: ...
    @property
    def oaor(self) -> BVector4: ...
    @property
    def oaog(self) -> BVector4: ...
    @property
    def oaob(self) -> BVector4: ...
    @property
    def oaoa(self) -> BVector4: ...
    @property
    def oaoo(self) -> BVector4: ...
    @property
    def oaol(self) -> BVector4: ...
    @property
    def oalr(self) -> BVector4: ...
    @property
    def oalg(self) -> BVector4: ...
    @property
    def oalb(self) -> BVector4: ...
    @property
    def oala(self) -> BVector4: ...
    @property
    def oalo(self) -> BVector4: ...
    @property
    def oall(self) -> BVector4: ...
    @property
    def oorr(self) -> BVector4: ...
    @property
    def oorg(self) -> BVector4: ...
    @property
    def oorb(self) -> BVector4: ...
    @property
    def oora(self) -> BVector4: ...
    @property
    def ooro(self) -> BVector4: ...
    @property
    def oorl(self) -> BVector4: ...
    @property
    def oogr(self) -> BVector4: ...
    @property
    def oogg(self) -> BVector4: ...
    @property
    def oogb(self) -> BVector4: ...
    @property
    def ooga(self) -> BVector4: ...
    @property
    def oogo(self) -> BVector4: ...
    @property
    def oogl(self) -> BVector4: ...
    @property
    def oobr(self) -> BVector4: ...
    @property
    def oobg(self) -> BVector4: ...
    @property
    def oobb(self) -> BVector4: ...
    @property
    def ooba(self) -> BVector4: ...
    @property
    def oobo(self) -> BVector4: ...
    @property
    def oobl(self) -> BVector4: ...
    @property
    def ooar(self) -> BVector4: ...
    @property
    def ooag(self) -> BVector4: ...
    @property
    def ooab(self) -> BVector4: ...
    @property
    def ooaa(self) -> BVector4: ...
    @property
    def ooao(self) -> BVector4: ...
    @property
    def ooal(self) -> BVector4: ...
    @property
    def ooor(self) -> BVector4: ...
    @property
    def ooog(self) -> BVector4: ...
    @property
    def ooob(self) -> BVector4: ...
    @property
    def oooa(self) -> BVector4: ...
    @property
    def oolr(self) -> BVector4: ...
    @property
    def oolg(self) -> BVector4: ...
    @property
    def oolb(self) -> BVector4: ...
    @property
    def oola(self) -> BVector4: ...
    @property
    def olrr(self) -> BVector4: ...
    @property
    def olrg(self) -> BVector4: ...
    @property
    def olrb(self) -> BVector4: ...
    @property
    def olra(self) -> BVector4: ...
    @property
    def olro(self) -> BVector4: ...
    @property
    def olrl(self) -> BVector4: ...
    @property
    def olgr(self) -> BVector4: ...
    @property
    def olgg(self) -> BVector4: ...
    @property
    def olgb(self) -> BVector4: ...
    @property
    def olga(self) -> BVector4: ...
    @property
    def olgo(self) -> BVector4: ...
    @property
    def olgl(self) -> BVector4: ...
    @property
    def olbr(self) -> BVector4: ...
    @property
    def olbg(self) -> BVector4: ...
    @property
    def olbb(self) -> BVector4: ...
    @property
    def olba(self) -> BVector4: ...
    @property
    def olbo(self) -> BVector4: ...
    @property
    def olbl(self) -> BVector4: ...
    @property
    def olar(self) -> BVector4: ...
    @property
    def olag(self) -> BVector4: ...
    @property
    def olab(self) -> BVector4: ...
    @property
    def olaa(self) -> BVector4: ...
    @property
    def olao(self) -> BVector4: ...
    @property
    def olal(self) -> BVector4: ...
    @property
    def olor(self) -> BVector4: ...
    @property
    def olog(self) -> BVector4: ...
    @property
    def olob(self) -> BVector4: ...
    @property
    def oloa(self) -> BVector4: ...
    @property
    def ollr(self) -> BVector4: ...
    @property
    def ollg(self) -> BVector4: ...
    @property
    def ollb(self) -> BVector4: ...
    @property
    def olla(self) -> BVector4: ...
    @property
    def lrrr(self) -> BVector4: ...
    @property
    def lrrg(self) -> BVector4: ...
    @property
    def lrrb(self) -> BVector4: ...
    @property
    def lrra(self) -> BVector4: ...
    @property
    def lrro(self) -> BVector4: ...
    @property
    def lrrl(self) -> BVector4: ...
    @property
    def lrgr(self) -> BVector4: ...
    @property
    def lrgg(self) -> BVector4: ...
    @property
    def lrgb(self) -> BVector4: ...
    @property
    def lrga(self) -> BVector4: ...
    @property
    def lrgo(self) -> BVector4: ...
    @property
    def lrgl(self) -> BVector4: ...
    @property
    def lrbr(self) -> BVector4: ...
    @property
    def lrbg(self) -> BVector4: ...
    @property
    def lrbb(self) -> BVector4: ...
    @property
    def lrba(self) -> BVector4: ...
    @property
    def lrbo(self) -> BVector4: ...
    @property
    def lrbl(self) -> BVector4: ...
    @property
    def lrar(self) -> BVector4: ...
    @property
    def lrag(self) -> BVector4: ...
    @property
    def lrab(self) -> BVector4: ...
    @property
    def lraa(self) -> BVector4: ...
    @property
    def lrao(self) -> BVector4: ...
    @property
    def lral(self) -> BVector4: ...
    @property
    def lror(self) -> BVector4: ...
    @property
    def lrog(self) -> BVector4: ...
    @property
    def lrob(self) -> BVector4: ...
    @property
    def lroa(self) -> BVector4: ...
    @property
    def lroo(self) -> BVector4: ...
    @property
    def lrol(self) -> BVector4: ...
    @property
    def lrlr(self) -> BVector4: ...
    @property
    def lrlg(self) -> BVector4: ...
    @property
    def lrlb(self) -> BVector4: ...
    @property
    def lrla(self) -> BVector4: ...
    @property
    def lrlo(self) -> BVector4: ...
    @property
    def lrll(self) -> BVector4: ...
    @property
    def lgrr(self) -> BVector4: ...
    @property
    def lgrg(self) -> BVector4: ...
    @property
    def lgrb(self) -> BVector4: ...
    @property
    def lgra(self) -> BVector4: ...
    @property
    def lgro(self) -> BVector4: ...
    @property
    def lgrl(self) -> BVector4: ...
    @property
    def lggr(self) -> BVector4: ...
    @property
    def lggg(self) -> BVector4: ...
    @property
    def lggb(self) -> BVector4: ...
    @property
    def lgga(self) -> BVector4: ...
    @property
    def lggo(self) -> BVector4: ...
    @property
    def lggl(self) -> BVector4: ...
    @property
    def lgbr(self) -> BVector4: ...
    @property
    def lgbg(self) -> BVector4: ...
    @property
    def lgbb(self) -> BVector4: ...
    @property
    def lgba(self) -> BVector4: ...
    @property
    def lgbo(self) -> BVector4: ...
    @property
    def lgbl(self) -> BVector4: ...
    @property
    def lgar(self) -> BVector4: ...
    @property
    def lgag(self) -> BVector4: ...
    @property
    def lgab(self) -> BVector4: ...
    @property
    def lgaa(self) -> BVector4: ...
    @property
    def lgao(self) -> BVector4: ...
    @property
    def lgal(self) -> BVector4: ...
    @property
    def lgor(self) -> BVector4: ...
    @property
    def lgog(self) -> BVector4: ...
    @property
    def lgob(self) -> BVector4: ...
    @property
    def lgoa(self) -> BVector4: ...
    @property
    def lgoo(self) -> BVector4: ...
    @property
    def lgol(self) -> BVector4: ...
    @property
    def lglr(self) -> BVector4: ...
    @property
    def lglg(self) -> BVector4: ...
    @property
    def lglb(self) -> BVector4: ...
    @property
    def lgla(self) -> BVector4: ...
    @property
    def lglo(self) -> BVector4: ...
    @property
    def lgll(self) -> BVector4: ...
    @property
    def lbrr(self) -> BVector4: ...
    @property
    def lbrg(self) -> BVector4: ...
    @property
    def lbrb(self) -> BVector4: ...
    @property
    def lbra(self) -> BVector4: ...
    @property
    def lbro(self) -> BVector4: ...
    @property
    def lbrl(self) -> BVector4: ...
    @property
    def lbgr(self) -> BVector4: ...
    @property
    def lbgg(self) -> BVector4: ...
    @property
    def lbgb(self) -> BVector4: ...
    @property
    def lbga(self) -> BVector4: ...
    @property
    def lbgo(self) -> BVector4: ...
    @property
    def lbgl(self) -> BVector4: ...
    @property
    def lbbr(self) -> BVector4: ...
    @property
    def lbbg(self) -> BVector4: ...
    @property
    def lbbb(self) -> BVector4: ...
    @property
    def lbba(self) -> BVector4: ...
    @property
    def lbbo(self) -> BVector4: ...
    @property
    def lbbl(self) -> BVector4: ...
    @property
    def lbar(self) -> BVector4: ...
    @property
    def lbag(self) -> BVector4: ...
    @property
    def lbab(self) -> BVector4: ...
    @property
    def lbaa(self) -> BVector4: ...
    @property
    def lbao(self) -> BVector4: ...
    @property
    def lbal(self) -> BVector4: ...
    @property
    def lbor(self) -> BVector4: ...
    @property
    def lbog(self) -> BVector4: ...
    @property
    def lbob(self) -> BVector4: ...
    @property
    def lboa(self) -> BVector4: ...
    @property
    def lboo(self) -> BVector4: ...
    @property
    def lbol(self) -> BVector4: ...
    @property
    def lblr(self) -> BVector4: ...
    @property
    def lblg(self) -> BVector4: ...
    @property
    def lblb(self) -> BVector4: ...
    @property
    def lbla(self) -> BVector4: ...
    @property
    def lblo(self) -> BVector4: ...
    @property
    def lbll(self) -> BVector4: ...
    @property
    def larr(self) -> BVector4: ...
    @property
    def larg(self) -> BVector4: ...
    @property
    def larb(self) -> BVector4: ...
    @property
    def lara(self) -> BVector4: ...
    @property
    def laro(self) -> BVector4: ...
    @property
    def larl(self) -> BVector4: ...
    @property
    def lagr(self) -> BVector4: ...
    @property
    def lagg(self) -> BVector4: ...
    @property
    def lagb(self) -> BVector4: ...
    @property
    def laga(self) -> BVector4: ...
    @property
    def lago(self) -> BVector4: ...
    @property
    def lagl(self) -> BVector4: ...
    @property
    def labr(self) -> BVector4: ...
    @property
    def labg(self) -> BVector4: ...
    @property
    def labb(self) -> BVector4: ...
    @property
    def laba(self) -> BVector4: ...
    @property
    def labo(self) -> BVector4: ...
    @property
    def labl(self) -> BVector4: ...
    @property
    def laar(self) -> BVector4: ...
    @property
    def laag(self) -> BVector4: ...
    @property
    def laab(self) -> BVector4: ...
    @property
    def laaa(self) -> BVector4: ...
    @property
    def laao(self) -> BVector4: ...
    @property
    def laal(self) -> BVector4: ...
    @property
    def laor(self) -> BVector4: ...
    @property
    def laog(self) -> BVector4: ...
    @property
    def laob(self) -> BVector4: ...
    @property
    def laoa(self) -> BVector4: ...
    @property
    def laoo(self) -> BVector4: ...
    @property
    def laol(self) -> BVector4: ...
    @property
    def lalr(self) -> BVector4: ...
    @property
    def lalg(self) -> BVector4: ...
    @property
    def lalb(self) -> BVector4: ...
    @property
    def lala(self) -> BVector4: ...
    @property
    def lalo(self) -> BVector4: ...
    @property
    def lall(self) -> BVector4: ...
    @property
    def lorr(self) -> BVector4: ...
    @property
    def lorg(self) -> BVector4: ...
    @property
    def lorb(self) -> BVector4: ...
    @property
    def lora(self) -> BVector4: ...
    @property
    def loro(self) -> BVector4: ...
    @property
    def lorl(self) -> BVector4: ...
    @property
    def logr(self) -> BVector4: ...
    @property
    def logg(self) -> BVector4: ...
    @property
    def logb(self) -> BVector4: ...
    @property
    def loga(self) -> BVector4: ...
    @property
    def logo(self) -> BVector4: ...
    @property
    def logl(self) -> BVector4: ...
    @property
    def lobr(self) -> BVector4: ...
    @property
    def lobg(self) -> BVector4: ...
    @property
    def lobb(self) -> BVector4: ...
    @property
    def loba(self) -> BVector4: ...
    @property
    def lobo(self) -> BVector4: ...
    @property
    def lobl(self) -> BVector4: ...
    @property
    def loar(self) -> BVector4: ...
    @property
    def loag(self) -> BVector4: ...
    @property
    def loab(self) -> BVector4: ...
    @property
    def loaa(self) -> BVector4: ...
    @property
    def loao(self) -> BVector4: ...
    @property
    def loal(self) -> BVector4: ...
    @property
    def loor(self) -> BVector4: ...
    @property
    def loog(self) -> BVector4: ...
    @property
    def loob(self) -> BVector4: ...
    @property
    def looa(self) -> BVector4: ...
    @property
    def lolr(self) -> BVector4: ...
    @property
    def lolg(self) -> BVector4: ...
    @property
    def lolb(self) -> BVector4: ...
    @property
    def lola(self) -> BVector4: ...
    @property
    def llrr(self) -> BVector4: ...
    @property
    def llrg(self) -> BVector4: ...
    @property
    def llrb(self) -> BVector4: ...
    @property
    def llra(self) -> BVector4: ...
    @property
    def llro(self) -> BVector4: ...
    @property
    def llrl(self) -> BVector4: ...
    @property
    def llgr(self) -> BVector4: ...
    @property
    def llgg(self) -> BVector4: ...
    @property
    def llgb(self) -> BVector4: ...
    @property
    def llga(self) -> BVector4: ...
    @property
    def llgo(self) -> BVector4: ...
    @property
    def llgl(self) -> BVector4: ...
    @property
    def llbr(self) -> BVector4: ...
    @property
    def llbg(self) -> BVector4: ...
    @property
    def llbb(self) -> BVector4: ...
    @property
    def llba(self) -> BVector4: ...
    @property
    def llbo(self) -> BVector4: ...
    @property
    def llbl(self) -> BVector4: ...
    @property
    def llar(self) -> BVector4: ...
    @property
    def llag(self) -> BVector4: ...
    @property
    def llab(self) -> BVector4: ...
    @property
    def llaa(self) -> BVector4: ...
    @property
    def llao(self) -> BVector4: ...
    @property
    def llal(self) -> BVector4: ...
    @property
    def llor(self) -> BVector4: ...
    @property
    def llog(self) -> BVector4: ...
    @property
    def llob(self) -> BVector4: ...
    @property
    def lloa(self) -> BVector4: ...
    @property
    def lllr(self) -> BVector4: ...
    @property
    def lllg(self) -> BVector4: ...
    @property
    def lllb(self) -> BVector4: ...
    @property
    def llla(self) -> BVector4: ...
    @property
    def s(self) -> _bool: ...
    @property
    def t(self) -> _bool: ...
    @property
    def q(self) -> _bool: ...
    @property
    def p(self) -> _bool: ...
    @property
    def ss(self) -> BVector2: ...
    @property
    def st(self) -> BVector2: ...
    @property
    def sq(self) -> BVector2: ...
    @property
    def sp(self) -> BVector2: ...
    @property
    def so(self) -> BVector2: ...
    @property
    def sl(self) -> BVector2: ...
    @property
    def ts(self) -> BVector2: ...
    @property
    def tt(self) -> BVector2: ...
    @property
    def tq(self) -> BVector2: ...
    @property
    def tp(self) -> BVector2: ...
    @property
    def to(self) -> BVector2: ...
    @property
    def tl(self) -> BVector2: ...
    @property
    def qs(self) -> BVector2: ...
    @property
    def qt(self) -> BVector2: ...
    @property
    def qq(self) -> BVector2: ...
    @property
    def qp(self) -> BVector2: ...
    @property
    def qo(self) -> BVector2: ...
    @property
    def ql(self) -> BVector2: ...
    @property
    def ps(self) -> BVector2: ...
    @property
    def pt(self) -> BVector2: ...
    @property
    def pq(self) -> BVector2: ...
    @property
    def pp(self) -> BVector2: ...
    @property
    def po(self) -> BVector2: ...
    @property
    def pl(self) -> BVector2: ...
    @property
    def os(self) -> BVector2: ...
    @property
    def ot(self) -> BVector2: ...
    @property
    def oq(self) -> BVector2: ...
    @property
    def op(self) -> BVector2: ...
    @property
    def ls(self) -> BVector2: ...
    @property
    def lt(self) -> BVector2: ...
    @property
    def lq(self) -> BVector2: ...
    @property
    def lp(self) -> BVector2: ...
    @property
    def sss(self) -> BVector3: ...
    @property
    def sst(self) -> BVector3: ...
    @property
    def ssq(self) -> BVector3: ...
    @property
    def ssp(self) -> BVector3: ...
    @property
    def sso(self) -> BVector3: ...
    @property
    def ssl(self) -> BVector3: ...
    @property
    def sts(self) -> BVector3: ...
    @property
    def stt(self) -> BVector3: ...
    @property
    def stq(self) -> BVector3: ...
    @property
    def stp(self) -> BVector3: ...
    @property
    def sto(self) -> BVector3: ...
    @property
    def stl(self) -> BVector3: ...
    @property
    def sqs(self) -> BVector3: ...
    @property
    def sqt(self) -> BVector3: ...
    @property
    def sqq(self) -> BVector3: ...
    @property
    def sqp(self) -> BVector3: ...
    @property
    def sqo(self) -> BVector3: ...
    @property
    def sql(self) -> BVector3: ...
    @property
    def sps(self) -> BVector3: ...
    @property
    def spt(self) -> BVector3: ...
    @property
    def spq(self) -> BVector3: ...
    @property
    def spp(self) -> BVector3: ...
    @property
    def spo(self) -> BVector3: ...
    @property
    def spl(self) -> BVector3: ...
    @property
    def sos(self) -> BVector3: ...
    @property
    def sot(self) -> BVector3: ...
    @property
    def soq(self) -> BVector3: ...
    @property
    def sop(self) -> BVector3: ...
    @property
    def soo(self) -> BVector3: ...
    @property
    def sol(self) -> BVector3: ...
    @property
    def sls(self) -> BVector3: ...
    @property
    def slt(self) -> BVector3: ...
    @property
    def slq(self) -> BVector3: ...
    @property
    def slp(self) -> BVector3: ...
    @property
    def slo(self) -> BVector3: ...
    @property
    def sll(self) -> BVector3: ...
    @property
    def tss(self) -> BVector3: ...
    @property
    def tst(self) -> BVector3: ...
    @property
    def tsq(self) -> BVector3: ...
    @property
    def tsp(self) -> BVector3: ...
    @property
    def tso(self) -> BVector3: ...
    @property
    def tsl(self) -> BVector3: ...
    @property
    def tts(self) -> BVector3: ...
    @property
    def ttt(self) -> BVector3: ...
    @property
    def ttq(self) -> BVector3: ...
    @property
    def ttp(self) -> BVector3: ...
    @property
    def tto(self) -> BVector3: ...
    @property
    def ttl(self) -> BVector3: ...
    @property
    def tqs(self) -> BVector3: ...
    @property
    def tqt(self) -> BVector3: ...
    @property
    def tqq(self) -> BVector3: ...
    @property
    def tqp(self) -> BVector3: ...
    @property
    def tqo(self) -> BVector3: ...
    @property
    def tql(self) -> BVector3: ...
    @property
    def tps(self) -> BVector3: ...
    @property
    def tpt(self) -> BVector3: ...
    @property
    def tpq(self) -> BVector3: ...
    @property
    def tpp(self) -> BVector3: ...
    @property
    def tpo(self) -> BVector3: ...
    @property
    def tpl(self) -> BVector3: ...
    @property
    def tos(self) -> BVector3: ...
    @property
    def tot(self) -> BVector3: ...
    @property
    def toq(self) -> BVector3: ...
    @property
    def top(self) -> BVector3: ...
    @property
    def too(self) -> BVector3: ...
    @property
    def tol(self) -> BVector3: ...
    @property
    def tls(self) -> BVector3: ...
    @property
    def tlt(self) -> BVector3: ...
    @property
    def tlq(self) -> BVector3: ...
    @property
    def tlp(self) -> BVector3: ...
    @property
    def tlo(self) -> BVector3: ...
    @property
    def tll(self) -> BVector3: ...
    @property
    def qss(self) -> BVector3: ...
    @property
    def qst(self) -> BVector3: ...
    @property
    def qsq(self) -> BVector3: ...
    @property
    def qsp(self) -> BVector3: ...
    @property
    def qso(self) -> BVector3: ...
    @property
    def qsl(self) -> BVector3: ...
    @property
    def qts(self) -> BVector3: ...
    @property
    def qtt(self) -> BVector3: ...
    @property
    def qtq(self) -> BVector3: ...
    @property
    def qtp(self) -> BVector3: ...
    @property
    def qto(self) -> BVector3: ...
    @property
    def qtl(self) -> BVector3: ...
    @property
    def qqs(self) -> BVector3: ...
    @property
    def qqt(self) -> BVector3: ...
    @property
    def qqq(self) -> BVector3: ...
    @property
    def qqp(self) -> BVector3: ...
    @property
    def qqo(self) -> BVector3: ...
    @property
    def qql(self) -> BVector3: ...
    @property
    def qps(self) -> BVector3: ...
    @property
    def qpt(self) -> BVector3: ...
    @property
    def qpq(self) -> BVector3: ...
    @property
    def qpp(self) -> BVector3: ...
    @property
    def qpo(self) -> BVector3: ...
    @property
    def qpl(self) -> BVector3: ...
    @property
    def qos(self) -> BVector3: ...
    @property
    def qot(self) -> BVector3: ...
    @property
    def qoq(self) -> BVector3: ...
    @property
    def qop(self) -> BVector3: ...
    @property
    def qoo(self) -> BVector3: ...
    @property
    def qol(self) -> BVector3: ...
    @property
    def qls(self) -> BVector3: ...
    @property
    def qlt(self) -> BVector3: ...
    @property
    def qlq(self) -> BVector3: ...
    @property
    def qlp(self) -> BVector3: ...
    @property
    def qlo(self) -> BVector3: ...
    @property
    def qll(self) -> BVector3: ...
    @property
    def pss(self) -> BVector3: ...
    @property
    def pst(self) -> BVector3: ...
    @property
    def psq(self) -> BVector3: ...
    @property
    def psp(self) -> BVector3: ...
    @property
    def pso(self) -> BVector3: ...
    @property
    def psl(self) -> BVector3: ...
    @property
    def pts(self) -> BVector3: ...
    @property
    def ptt(self) -> BVector3: ...
    @property
    def ptq(self) -> BVector3: ...
    @property
    def ptp(self) -> BVector3: ...
    @property
    def pto(self) -> BVector3: ...
    @property
    def ptl(self) -> BVector3: ...
    @property
    def pqs(self) -> BVector3: ...
    @property
    def pqt(self) -> BVector3: ...
    @property
    def pqq(self) -> BVector3: ...
    @property
    def pqp(self) -> BVector3: ...
    @property
    def pqo(self) -> BVector3: ...
    @property
    def pql(self) -> BVector3: ...
    @property
    def pps(self) -> BVector3: ...
    @property
    def ppt(self) -> BVector3: ...
    @property
    def ppq(self) -> BVector3: ...
    @property
    def ppp(self) -> BVector3: ...
    @property
    def ppo(self) -> BVector3: ...
    @property
    def ppl(self) -> BVector3: ...
    @property
    def pos(self) -> BVector3: ...
    @property
    def pot(self) -> BVector3: ...
    @property
    def poq(self) -> BVector3: ...
    @property
    def pop(self) -> BVector3: ...
    @property
    def poo(self) -> BVector3: ...
    @property
    def pol(self) -> BVector3: ...
    @property
    def pls(self) -> BVector3: ...
    @property
    def plt(self) -> BVector3: ...
    @property
    def plq(self) -> BVector3: ...
    @property
    def plp(self) -> BVector3: ...
    @property
    def plo(self) -> BVector3: ...
    @property
    def pll(self) -> BVector3: ...
    @property
    def oss(self) -> BVector3: ...
    @property
    def ost(self) -> BVector3: ...
    @property
    def osq(self) -> BVector3: ...
    @property
    def osp(self) -> BVector3: ...
    @property
    def oso(self) -> BVector3: ...
    @property
    def osl(self) -> BVector3: ...
    @property
    def ots(self) -> BVector3: ...
    @property
    def ott(self) -> BVector3: ...
    @property
    def otq(self) -> BVector3: ...
    @property
    def otp(self) -> BVector3: ...
    @property
    def oto(self) -> BVector3: ...
    @property
    def otl(self) -> BVector3: ...
    @property
    def oqs(self) -> BVector3: ...
    @property
    def oqt(self) -> BVector3: ...
    @property
    def oqq(self) -> BVector3: ...
    @property
    def oqp(self) -> BVector3: ...
    @property
    def oqo(self) -> BVector3: ...
    @property
    def oql(self) -> BVector3: ...
    @property
    def ops(self) -> BVector3: ...
    @property
    def opt(self) -> BVector3: ...
    @property
    def opq(self) -> BVector3: ...
    @property
    def opp(self) -> BVector3: ...
    @property
    def opo(self) -> BVector3: ...
    @property
    def opl(self) -> BVector3: ...
    @property
    def oos(self) -> BVector3: ...
    @property
    def oot(self) -> BVector3: ...
    @property
    def ooq(self) -> BVector3: ...
    @property
    def oop(self) -> BVector3: ...
    @property
    def ols(self) -> BVector3: ...
    @property
    def olt(self) -> BVector3: ...
    @property
    def olq(self) -> BVector3: ...
    @property
    def olp(self) -> BVector3: ...
    @property
    def lss(self) -> BVector3: ...
    @property
    def lst(self) -> BVector3: ...
    @property
    def lsq(self) -> BVector3: ...
    @property
    def lsp(self) -> BVector3: ...
    @property
    def lso(self) -> BVector3: ...
    @property
    def lsl(self) -> BVector3: ...
    @property
    def lts(self) -> BVector3: ...
    @property
    def ltt(self) -> BVector3: ...
    @property
    def ltq(self) -> BVector3: ...
    @property
    def ltp(self) -> BVector3: ...
    @property
    def lto(self) -> BVector3: ...
    @property
    def ltl(self) -> BVector3: ...
    @property
    def lqs(self) -> BVector3: ...
    @property
    def lqt(self) -> BVector3: ...
    @property
    def lqq(self) -> BVector3: ...
    @property
    def lqp(self) -> BVector3: ...
    @property
    def lqo(self) -> BVector3: ...
    @property
    def lql(self) -> BVector3: ...
    @property
    def lps(self) -> BVector3: ...
    @property
    def lpt(self) -> BVector3: ...
    @property
    def lpq(self) -> BVector3: ...
    @property
    def lpp(self) -> BVector3: ...
    @property
    def lpo(self) -> BVector3: ...
    @property
    def lpl(self) -> BVector3: ...
    @property
    def los(self) -> BVector3: ...
    @property
    def lot(self) -> BVector3: ...
    @property
    def loq(self) -> BVector3: ...
    @property
    def lop(self) -> BVector3: ...
    @property
    def lls(self) -> BVector3: ...
    @property
    def llt(self) -> BVector3: ...
    @property
    def llq(self) -> BVector3: ...
    @property
    def llp(self) -> BVector3: ...
    @property
    def ssss(self) -> BVector4: ...
    @property
    def ssst(self) -> BVector4: ...
    @property
    def sssq(self) -> BVector4: ...
    @property
    def sssp(self) -> BVector4: ...
    @property
    def ssso(self) -> BVector4: ...
    @property
    def sssl(self) -> BVector4: ...
    @property
    def ssts(self) -> BVector4: ...
    @property
    def sstt(self) -> BVector4: ...
    @property
    def sstq(self) -> BVector4: ...
    @property
    def sstp(self) -> BVector4: ...
    @property
    def ssto(self) -> BVector4: ...
    @property
    def sstl(self) -> BVector4: ...
    @property
    def ssqs(self) -> BVector4: ...
    @property
    def ssqt(self) -> BVector4: ...
    @property
    def ssqq(self) -> BVector4: ...
    @property
    def ssqp(self) -> BVector4: ...
    @property
    def ssqo(self) -> BVector4: ...
    @property
    def ssql(self) -> BVector4: ...
    @property
    def ssps(self) -> BVector4: ...
    @property
    def sspt(self) -> BVector4: ...
    @property
    def sspq(self) -> BVector4: ...
    @property
    def sspp(self) -> BVector4: ...
    @property
    def sspo(self) -> BVector4: ...
    @property
    def sspl(self) -> BVector4: ...
    @property
    def ssos(self) -> BVector4: ...
    @property
    def ssot(self) -> BVector4: ...
    @property
    def ssoq(self) -> BVector4: ...
    @property
    def ssop(self) -> BVector4: ...
    @property
    def ssoo(self) -> BVector4: ...
    @property
    def ssol(self) -> BVector4: ...
    @property
    def ssls(self) -> BVector4: ...
    @property
    def sslt(self) -> BVector4: ...
    @property
    def sslq(self) -> BVector4: ...
    @property
    def sslp(self) -> BVector4: ...
    @property
    def sslo(self) -> BVector4: ...
    @property
    def ssll(self) -> BVector4: ...
    @property
    def stss(self) -> BVector4: ...
    @property
    def stst(self) -> BVector4: ...
    @property
    def stsq(self) -> BVector4: ...
    @property
    def stsp(self) -> BVector4: ...
    @property
    def stso(self) -> BVector4: ...
    @property
    def stsl(self) -> BVector4: ...
    @property
    def stts(self) -> BVector4: ...
    @property
    def sttt(self) -> BVector4: ...
    @property
    def sttq(self) -> BVector4: ...
    @property
    def sttp(self) -> BVector4: ...
    @property
    def stto(self) -> BVector4: ...
    @property
    def sttl(self) -> BVector4: ...
    @property
    def stqs(self) -> BVector4: ...
    @property
    def stqt(self) -> BVector4: ...
    @property
    def stqq(self) -> BVector4: ...
    @property
    def stqp(self) -> BVector4: ...
    @property
    def stqo(self) -> BVector4: ...
    @property
    def stql(self) -> BVector4: ...
    @property
    def stps(self) -> BVector4: ...
    @property
    def stpt(self) -> BVector4: ...
    @property
    def stpq(self) -> BVector4: ...
    @property
    def stpp(self) -> BVector4: ...
    @property
    def stpo(self) -> BVector4: ...
    @property
    def stpl(self) -> BVector4: ...
    @property
    def stos(self) -> BVector4: ...
    @property
    def stot(self) -> BVector4: ...
    @property
    def stoq(self) -> BVector4: ...
    @property
    def stop(self) -> BVector4: ...
    @property
    def stoo(self) -> BVector4: ...
    @property
    def stol(self) -> BVector4: ...
    @property
    def stls(self) -> BVector4: ...
    @property
    def stlt(self) -> BVector4: ...
    @property
    def stlq(self) -> BVector4: ...
    @property
    def stlp(self) -> BVector4: ...
    @property
    def stlo(self) -> BVector4: ...
    @property
    def stll(self) -> BVector4: ...
    @property
    def sqss(self) -> BVector4: ...
    @property
    def sqst(self) -> BVector4: ...
    @property
    def sqsq(self) -> BVector4: ...
    @property
    def sqsp(self) -> BVector4: ...
    @property
    def sqso(self) -> BVector4: ...
    @property
    def sqsl(self) -> BVector4: ...
    @property
    def sqts(self) -> BVector4: ...
    @property
    def sqtt(self) -> BVector4: ...
    @property
    def sqtq(self) -> BVector4: ...
    @property
    def sqtp(self) -> BVector4: ...
    @property
    def sqto(self) -> BVector4: ...
    @property
    def sqtl(self) -> BVector4: ...
    @property
    def sqqs(self) -> BVector4: ...
    @property
    def sqqt(self) -> BVector4: ...
    @property
    def sqqq(self) -> BVector4: ...
    @property
    def sqqp(self) -> BVector4: ...
    @property
    def sqqo(self) -> BVector4: ...
    @property
    def sqql(self) -> BVector4: ...
    @property
    def sqps(self) -> BVector4: ...
    @property
    def sqpt(self) -> BVector4: ...
    @property
    def sqpq(self) -> BVector4: ...
    @property
    def sqpp(self) -> BVector4: ...
    @property
    def sqpo(self) -> BVector4: ...
    @property
    def sqpl(self) -> BVector4: ...
    @property
    def sqos(self) -> BVector4: ...
    @property
    def sqot(self) -> BVector4: ...
    @property
    def sqoq(self) -> BVector4: ...
    @property
    def sqop(self) -> BVector4: ...
    @property
    def sqoo(self) -> BVector4: ...
    @property
    def sqol(self) -> BVector4: ...
    @property
    def sqls(self) -> BVector4: ...
    @property
    def sqlt(self) -> BVector4: ...
    @property
    def sqlq(self) -> BVector4: ...
    @property
    def sqlp(self) -> BVector4: ...
    @property
    def sqlo(self) -> BVector4: ...
    @property
    def sqll(self) -> BVector4: ...
    @property
    def spss(self) -> BVector4: ...
    @property
    def spst(self) -> BVector4: ...
    @property
    def spsq(self) -> BVector4: ...
    @property
    def spsp(self) -> BVector4: ...
    @property
    def spso(self) -> BVector4: ...
    @property
    def spsl(self) -> BVector4: ...
    @property
    def spts(self) -> BVector4: ...
    @property
    def sptt(self) -> BVector4: ...
    @property
    def sptq(self) -> BVector4: ...
    @property
    def sptp(self) -> BVector4: ...
    @property
    def spto(self) -> BVector4: ...
    @property
    def sptl(self) -> BVector4: ...
    @property
    def spqs(self) -> BVector4: ...
    @property
    def spqt(self) -> BVector4: ...
    @property
    def spqq(self) -> BVector4: ...
    @property
    def spqp(self) -> BVector4: ...
    @property
    def spqo(self) -> BVector4: ...
    @property
    def spql(self) -> BVector4: ...
    @property
    def spps(self) -> BVector4: ...
    @property
    def sppt(self) -> BVector4: ...
    @property
    def sppq(self) -> BVector4: ...
    @property
    def sppp(self) -> BVector4: ...
    @property
    def sppo(self) -> BVector4: ...
    @property
    def sppl(self) -> BVector4: ...
    @property
    def spos(self) -> BVector4: ...
    @property
    def spot(self) -> BVector4: ...
    @property
    def spoq(self) -> BVector4: ...
    @property
    def spop(self) -> BVector4: ...
    @property
    def spoo(self) -> BVector4: ...
    @property
    def spol(self) -> BVector4: ...
    @property
    def spls(self) -> BVector4: ...
    @property
    def splt(self) -> BVector4: ...
    @property
    def splq(self) -> BVector4: ...
    @property
    def splp(self) -> BVector4: ...
    @property
    def splo(self) -> BVector4: ...
    @property
    def spll(self) -> BVector4: ...
    @property
    def soss(self) -> BVector4: ...
    @property
    def sost(self) -> BVector4: ...
    @property
    def sosq(self) -> BVector4: ...
    @property
    def sosp(self) -> BVector4: ...
    @property
    def soso(self) -> BVector4: ...
    @property
    def sosl(self) -> BVector4: ...
    @property
    def sots(self) -> BVector4: ...
    @property
    def sott(self) -> BVector4: ...
    @property
    def sotq(self) -> BVector4: ...
    @property
    def sotp(self) -> BVector4: ...
    @property
    def soto(self) -> BVector4: ...
    @property
    def sotl(self) -> BVector4: ...
    @property
    def soqs(self) -> BVector4: ...
    @property
    def soqt(self) -> BVector4: ...
    @property
    def soqq(self) -> BVector4: ...
    @property
    def soqp(self) -> BVector4: ...
    @property
    def soqo(self) -> BVector4: ...
    @property
    def soql(self) -> BVector4: ...
    @property
    def sops(self) -> BVector4: ...
    @property
    def sopt(self) -> BVector4: ...
    @property
    def sopq(self) -> BVector4: ...
    @property
    def sopp(self) -> BVector4: ...
    @property
    def sopo(self) -> BVector4: ...
    @property
    def sopl(self) -> BVector4: ...
    @property
    def soos(self) -> BVector4: ...
    @property
    def soot(self) -> BVector4: ...
    @property
    def sooq(self) -> BVector4: ...
    @property
    def soop(self) -> BVector4: ...
    @property
    def sooo(self) -> BVector4: ...
    @property
    def sool(self) -> BVector4: ...
    @property
    def sols(self) -> BVector4: ...
    @property
    def solt(self) -> BVector4: ...
    @property
    def solq(self) -> BVector4: ...
    @property
    def solp(self) -> BVector4: ...
    @property
    def solo(self) -> BVector4: ...
    @property
    def soll(self) -> BVector4: ...
    @property
    def slss(self) -> BVector4: ...
    @property
    def slst(self) -> BVector4: ...
    @property
    def slsq(self) -> BVector4: ...
    @property
    def slsp(self) -> BVector4: ...
    @property
    def slso(self) -> BVector4: ...
    @property
    def slsl(self) -> BVector4: ...
    @property
    def slts(self) -> BVector4: ...
    @property
    def sltt(self) -> BVector4: ...
    @property
    def sltq(self) -> BVector4: ...
    @property
    def sltp(self) -> BVector4: ...
    @property
    def slto(self) -> BVector4: ...
    @property
    def sltl(self) -> BVector4: ...
    @property
    def slqs(self) -> BVector4: ...
    @property
    def slqt(self) -> BVector4: ...
    @property
    def slqq(self) -> BVector4: ...
    @property
    def slqp(self) -> BVector4: ...
    @property
    def slqo(self) -> BVector4: ...
    @property
    def slql(self) -> BVector4: ...
    @property
    def slps(self) -> BVector4: ...
    @property
    def slpt(self) -> BVector4: ...
    @property
    def slpq(self) -> BVector4: ...
    @property
    def slpp(self) -> BVector4: ...
    @property
    def slpo(self) -> BVector4: ...
    @property
    def slpl(self) -> BVector4: ...
    @property
    def slos(self) -> BVector4: ...
    @property
    def slot(self) -> BVector4: ...
    @property
    def sloq(self) -> BVector4: ...
    @property
    def slop(self) -> BVector4: ...
    @property
    def sloo(self) -> BVector4: ...
    @property
    def slol(self) -> BVector4: ...
    @property
    def slls(self) -> BVector4: ...
    @property
    def sllt(self) -> BVector4: ...
    @property
    def sllq(self) -> BVector4: ...
    @property
    def sllp(self) -> BVector4: ...
    @property
    def sllo(self) -> BVector4: ...
    @property
    def slll(self) -> BVector4: ...
    @property
    def tsss(self) -> BVector4: ...
    @property
    def tsst(self) -> BVector4: ...
    @property
    def tssq(self) -> BVector4: ...
    @property
    def tssp(self) -> BVector4: ...
    @property
    def tsso(self) -> BVector4: ...
    @property
    def tssl(self) -> BVector4: ...
    @property
    def tsts(self) -> BVector4: ...
    @property
    def tstt(self) -> BVector4: ...
    @property
    def tstq(self) -> BVector4: ...
    @property
    def tstp(self) -> BVector4: ...
    @property
    def tsto(self) -> BVector4: ...
    @property
    def tstl(self) -> BVector4: ...
    @property
    def tsqs(self) -> BVector4: ...
    @property
    def tsqt(self) -> BVector4: ...
    @property
    def tsqq(self) -> BVector4: ...
    @property
    def tsqp(self) -> BVector4: ...
    @property
    def tsqo(self) -> BVector4: ...
    @property
    def tsql(self) -> BVector4: ...
    @property
    def tsps(self) -> BVector4: ...
    @property
    def tspt(self) -> BVector4: ...
    @property
    def tspq(self) -> BVector4: ...
    @property
    def tspp(self) -> BVector4: ...
    @property
    def tspo(self) -> BVector4: ...
    @property
    def tspl(self) -> BVector4: ...
    @property
    def tsos(self) -> BVector4: ...
    @property
    def tsot(self) -> BVector4: ...
    @property
    def tsoq(self) -> BVector4: ...
    @property
    def tsop(self) -> BVector4: ...
    @property
    def tsoo(self) -> BVector4: ...
    @property
    def tsol(self) -> BVector4: ...
    @property
    def tsls(self) -> BVector4: ...
    @property
    def tslt(self) -> BVector4: ...
    @property
    def tslq(self) -> BVector4: ...
    @property
    def tslp(self) -> BVector4: ...
    @property
    def tslo(self) -> BVector4: ...
    @property
    def tsll(self) -> BVector4: ...
    @property
    def ttss(self) -> BVector4: ...
    @property
    def ttst(self) -> BVector4: ...
    @property
    def ttsq(self) -> BVector4: ...
    @property
    def ttsp(self) -> BVector4: ...
    @property
    def ttso(self) -> BVector4: ...
    @property
    def ttsl(self) -> BVector4: ...
    @property
    def ttts(self) -> BVector4: ...
    @property
    def tttt(self) -> BVector4: ...
    @property
    def tttq(self) -> BVector4: ...
    @property
    def tttp(self) -> BVector4: ...
    @property
    def ttto(self) -> BVector4: ...
    @property
    def tttl(self) -> BVector4: ...
    @property
    def ttqs(self) -> BVector4: ...
    @property
    def ttqt(self) -> BVector4: ...
    @property
    def ttqq(self) -> BVector4: ...
    @property
    def ttqp(self) -> BVector4: ...
    @property
    def ttqo(self) -> BVector4: ...
    @property
    def ttql(self) -> BVector4: ...
    @property
    def ttps(self) -> BVector4: ...
    @property
    def ttpt(self) -> BVector4: ...
    @property
    def ttpq(self) -> BVector4: ...
    @property
    def ttpp(self) -> BVector4: ...
    @property
    def ttpo(self) -> BVector4: ...
    @property
    def ttpl(self) -> BVector4: ...
    @property
    def ttos(self) -> BVector4: ...
    @property
    def ttot(self) -> BVector4: ...
    @property
    def ttoq(self) -> BVector4: ...
    @property
    def ttop(self) -> BVector4: ...
    @property
    def ttoo(self) -> BVector4: ...
    @property
    def ttol(self) -> BVector4: ...
    @property
    def ttls(self) -> BVector4: ...
    @property
    def ttlt(self) -> BVector4: ...
    @property
    def ttlq(self) -> BVector4: ...
    @property
    def ttlp(self) -> BVector4: ...
    @property
    def ttlo(self) -> BVector4: ...
    @property
    def ttll(self) -> BVector4: ...
    @property
    def tqss(self) -> BVector4: ...
    @property
    def tqst(self) -> BVector4: ...
    @property
    def tqsq(self) -> BVector4: ...
    @property
    def tqsp(self) -> BVector4: ...
    @property
    def tqso(self) -> BVector4: ...
    @property
    def tqsl(self) -> BVector4: ...
    @property
    def tqts(self) -> BVector4: ...
    @property
    def tqtt(self) -> BVector4: ...
    @property
    def tqtq(self) -> BVector4: ...
    @property
    def tqtp(self) -> BVector4: ...
    @property
    def tqto(self) -> BVector4: ...
    @property
    def tqtl(self) -> BVector4: ...
    @property
    def tqqs(self) -> BVector4: ...
    @property
    def tqqt(self) -> BVector4: ...
    @property
    def tqqq(self) -> BVector4: ...
    @property
    def tqqp(self) -> BVector4: ...
    @property
    def tqqo(self) -> BVector4: ...
    @property
    def tqql(self) -> BVector4: ...
    @property
    def tqps(self) -> BVector4: ...
    @property
    def tqpt(self) -> BVector4: ...
    @property
    def tqpq(self) -> BVector4: ...
    @property
    def tqpp(self) -> BVector4: ...
    @property
    def tqpo(self) -> BVector4: ...
    @property
    def tqpl(self) -> BVector4: ...
    @property
    def tqos(self) -> BVector4: ...
    @property
    def tqot(self) -> BVector4: ...
    @property
    def tqoq(self) -> BVector4: ...
    @property
    def tqop(self) -> BVector4: ...
    @property
    def tqoo(self) -> BVector4: ...
    @property
    def tqol(self) -> BVector4: ...
    @property
    def tqls(self) -> BVector4: ...
    @property
    def tqlt(self) -> BVector4: ...
    @property
    def tqlq(self) -> BVector4: ...
    @property
    def tqlp(self) -> BVector4: ...
    @property
    def tqlo(self) -> BVector4: ...
    @property
    def tqll(self) -> BVector4: ...
    @property
    def tpss(self) -> BVector4: ...
    @property
    def tpst(self) -> BVector4: ...
    @property
    def tpsq(self) -> BVector4: ...
    @property
    def tpsp(self) -> BVector4: ...
    @property
    def tpso(self) -> BVector4: ...
    @property
    def tpsl(self) -> BVector4: ...
    @property
    def tpts(self) -> BVector4: ...
    @property
    def tptt(self) -> BVector4: ...
    @property
    def tptq(self) -> BVector4: ...
    @property
    def tptp(self) -> BVector4: ...
    @property
    def tpto(self) -> BVector4: ...
    @property
    def tptl(self) -> BVector4: ...
    @property
    def tpqs(self) -> BVector4: ...
    @property
    def tpqt(self) -> BVector4: ...
    @property
    def tpqq(self) -> BVector4: ...
    @property
    def tpqp(self) -> BVector4: ...
    @property
    def tpqo(self) -> BVector4: ...
    @property
    def tpql(self) -> BVector4: ...
    @property
    def tpps(self) -> BVector4: ...
    @property
    def tppt(self) -> BVector4: ...
    @property
    def tppq(self) -> BVector4: ...
    @property
    def tppp(self) -> BVector4: ...
    @property
    def tppo(self) -> BVector4: ...
    @property
    def tppl(self) -> BVector4: ...
    @property
    def tpos(self) -> BVector4: ...
    @property
    def tpot(self) -> BVector4: ...
    @property
    def tpoq(self) -> BVector4: ...
    @property
    def tpop(self) -> BVector4: ...
    @property
    def tpoo(self) -> BVector4: ...
    @property
    def tpol(self) -> BVector4: ...
    @property
    def tpls(self) -> BVector4: ...
    @property
    def tplt(self) -> BVector4: ...
    @property
    def tplq(self) -> BVector4: ...
    @property
    def tplp(self) -> BVector4: ...
    @property
    def tplo(self) -> BVector4: ...
    @property
    def tpll(self) -> BVector4: ...
    @property
    def toss(self) -> BVector4: ...
    @property
    def tost(self) -> BVector4: ...
    @property
    def tosq(self) -> BVector4: ...
    @property
    def tosp(self) -> BVector4: ...
    @property
    def toso(self) -> BVector4: ...
    @property
    def tosl(self) -> BVector4: ...
    @property
    def tots(self) -> BVector4: ...
    @property
    def tott(self) -> BVector4: ...
    @property
    def totq(self) -> BVector4: ...
    @property
    def totp(self) -> BVector4: ...
    @property
    def toto(self) -> BVector4: ...
    @property
    def totl(self) -> BVector4: ...
    @property
    def toqs(self) -> BVector4: ...
    @property
    def toqt(self) -> BVector4: ...
    @property
    def toqq(self) -> BVector4: ...
    @property
    def toqp(self) -> BVector4: ...
    @property
    def toqo(self) -> BVector4: ...
    @property
    def toql(self) -> BVector4: ...
    @property
    def tops(self) -> BVector4: ...
    @property
    def topt(self) -> BVector4: ...
    @property
    def topq(self) -> BVector4: ...
    @property
    def topp(self) -> BVector4: ...
    @property
    def topo(self) -> BVector4: ...
    @property
    def topl(self) -> BVector4: ...
    @property
    def toos(self) -> BVector4: ...
    @property
    def toot(self) -> BVector4: ...
    @property
    def tooq(self) -> BVector4: ...
    @property
    def toop(self) -> BVector4: ...
    @property
    def tooo(self) -> BVector4: ...
    @property
    def tool(self) -> BVector4: ...
    @property
    def tols(self) -> BVector4: ...
    @property
    def tolt(self) -> BVector4: ...
    @property
    def tolq(self) -> BVector4: ...
    @property
    def tolp(self) -> BVector4: ...
    @property
    def tolo(self) -> BVector4: ...
    @property
    def toll(self) -> BVector4: ...
    @property
    def tlss(self) -> BVector4: ...
    @property
    def tlst(self) -> BVector4: ...
    @property
    def tlsq(self) -> BVector4: ...
    @property
    def tlsp(self) -> BVector4: ...
    @property
    def tlso(self) -> BVector4: ...
    @property
    def tlsl(self) -> BVector4: ...
    @property
    def tlts(self) -> BVector4: ...
    @property
    def tltt(self) -> BVector4: ...
    @property
    def tltq(self) -> BVector4: ...
    @property
    def tltp(self) -> BVector4: ...
    @property
    def tlto(self) -> BVector4: ...
    @property
    def tltl(self) -> BVector4: ...
    @property
    def tlqs(self) -> BVector4: ...
    @property
    def tlqt(self) -> BVector4: ...
    @property
    def tlqq(self) -> BVector4: ...
    @property
    def tlqp(self) -> BVector4: ...
    @property
    def tlqo(self) -> BVector4: ...
    @property
    def tlql(self) -> BVector4: ...
    @property
    def tlps(self) -> BVector4: ...
    @property
    def tlpt(self) -> BVector4: ...
    @property
    def tlpq(self) -> BVector4: ...
    @property
    def tlpp(self) -> BVector4: ...
    @property
    def tlpo(self) -> BVector4: ...
    @property
    def tlpl(self) -> BVector4: ...
    @property
    def tlos(self) -> BVector4: ...
    @property
    def tlot(self) -> BVector4: ...
    @property
    def tloq(self) -> BVector4: ...
    @property
    def tlop(self) -> BVector4: ...
    @property
    def tloo(self) -> BVector4: ...
    @property
    def tlol(self) -> BVector4: ...
    @property
    def tlls(self) -> BVector4: ...
    @property
    def tllt(self) -> BVector4: ...
    @property
    def tllq(self) -> BVector4: ...
    @property
    def tllp(self) -> BVector4: ...
    @property
    def tllo(self) -> BVector4: ...
    @property
    def tlll(self) -> BVector4: ...
    @property
    def qsss(self) -> BVector4: ...
    @property
    def qsst(self) -> BVector4: ...
    @property
    def qssq(self) -> BVector4: ...
    @property
    def qssp(self) -> BVector4: ...
    @property
    def qsso(self) -> BVector4: ...
    @property
    def qssl(self) -> BVector4: ...
    @property
    def qsts(self) -> BVector4: ...
    @property
    def qstt(self) -> BVector4: ...
    @property
    def qstq(self) -> BVector4: ...
    @property
    def qstp(self) -> BVector4: ...
    @property
    def qsto(self) -> BVector4: ...
    @property
    def qstl(self) -> BVector4: ...
    @property
    def qsqs(self) -> BVector4: ...
    @property
    def qsqt(self) -> BVector4: ...
    @property
    def qsqq(self) -> BVector4: ...
    @property
    def qsqp(self) -> BVector4: ...
    @property
    def qsqo(self) -> BVector4: ...
    @property
    def qsql(self) -> BVector4: ...
    @property
    def qsps(self) -> BVector4: ...
    @property
    def qspt(self) -> BVector4: ...
    @property
    def qspq(self) -> BVector4: ...
    @property
    def qspp(self) -> BVector4: ...
    @property
    def qspo(self) -> BVector4: ...
    @property
    def qspl(self) -> BVector4: ...
    @property
    def qsos(self) -> BVector4: ...
    @property
    def qsot(self) -> BVector4: ...
    @property
    def qsoq(self) -> BVector4: ...
    @property
    def qsop(self) -> BVector4: ...
    @property
    def qsoo(self) -> BVector4: ...
    @property
    def qsol(self) -> BVector4: ...
    @property
    def qsls(self) -> BVector4: ...
    @property
    def qslt(self) -> BVector4: ...
    @property
    def qslq(self) -> BVector4: ...
    @property
    def qslp(self) -> BVector4: ...
    @property
    def qslo(self) -> BVector4: ...
    @property
    def qsll(self) -> BVector4: ...
    @property
    def qtss(self) -> BVector4: ...
    @property
    def qtst(self) -> BVector4: ...
    @property
    def qtsq(self) -> BVector4: ...
    @property
    def qtsp(self) -> BVector4: ...
    @property
    def qtso(self) -> BVector4: ...
    @property
    def qtsl(self) -> BVector4: ...
    @property
    def qtts(self) -> BVector4: ...
    @property
    def qttt(self) -> BVector4: ...
    @property
    def qttq(self) -> BVector4: ...
    @property
    def qttp(self) -> BVector4: ...
    @property
    def qtto(self) -> BVector4: ...
    @property
    def qttl(self) -> BVector4: ...
    @property
    def qtqs(self) -> BVector4: ...
    @property
    def qtqt(self) -> BVector4: ...
    @property
    def qtqq(self) -> BVector4: ...
    @property
    def qtqp(self) -> BVector4: ...
    @property
    def qtqo(self) -> BVector4: ...
    @property
    def qtql(self) -> BVector4: ...
    @property
    def qtps(self) -> BVector4: ...
    @property
    def qtpt(self) -> BVector4: ...
    @property
    def qtpq(self) -> BVector4: ...
    @property
    def qtpp(self) -> BVector4: ...
    @property
    def qtpo(self) -> BVector4: ...
    @property
    def qtpl(self) -> BVector4: ...
    @property
    def qtos(self) -> BVector4: ...
    @property
    def qtot(self) -> BVector4: ...
    @property
    def qtoq(self) -> BVector4: ...
    @property
    def qtop(self) -> BVector4: ...
    @property
    def qtoo(self) -> BVector4: ...
    @property
    def qtol(self) -> BVector4: ...
    @property
    def qtls(self) -> BVector4: ...
    @property
    def qtlt(self) -> BVector4: ...
    @property
    def qtlq(self) -> BVector4: ...
    @property
    def qtlp(self) -> BVector4: ...
    @property
    def qtlo(self) -> BVector4: ...
    @property
    def qtll(self) -> BVector4: ...
    @property
    def qqss(self) -> BVector4: ...
    @property
    def qqst(self) -> BVector4: ...
    @property
    def qqsq(self) -> BVector4: ...
    @property
    def qqsp(self) -> BVector4: ...
    @property
    def qqso(self) -> BVector4: ...
    @property
    def qqsl(self) -> BVector4: ...
    @property
    def qqts(self) -> BVector4: ...
    @property
    def qqtt(self) -> BVector4: ...
    @property
    def qqtq(self) -> BVector4: ...
    @property
    def qqtp(self) -> BVector4: ...
    @property
    def qqto(self) -> BVector4: ...
    @property
    def qqtl(self) -> BVector4: ...
    @property
    def qqqs(self) -> BVector4: ...
    @property
    def qqqt(self) -> BVector4: ...
    @property
    def qqqq(self) -> BVector4: ...
    @property
    def qqqp(self) -> BVector4: ...
    @property
    def qqqo(self) -> BVector4: ...
    @property
    def qqql(self) -> BVector4: ...
    @property
    def qqps(self) -> BVector4: ...
    @property
    def qqpt(self) -> BVector4: ...
    @property
    def qqpq(self) -> BVector4: ...
    @property
    def qqpp(self) -> BVector4: ...
    @property
    def qqpo(self) -> BVector4: ...
    @property
    def qqpl(self) -> BVector4: ...
    @property
    def qqos(self) -> BVector4: ...
    @property
    def qqot(self) -> BVector4: ...
    @property
    def qqoq(self) -> BVector4: ...
    @property
    def qqop(self) -> BVector4: ...
    @property
    def qqoo(self) -> BVector4: ...
    @property
    def qqol(self) -> BVector4: ...
    @property
    def qqls(self) -> BVector4: ...
    @property
    def qqlt(self) -> BVector4: ...
    @property
    def qqlq(self) -> BVector4: ...
    @property
    def qqlp(self) -> BVector4: ...
    @property
    def qqlo(self) -> BVector4: ...
    @property
    def qqll(self) -> BVector4: ...
    @property
    def qpss(self) -> BVector4: ...
    @property
    def qpst(self) -> BVector4: ...
    @property
    def qpsq(self) -> BVector4: ...
    @property
    def qpsp(self) -> BVector4: ...
    @property
    def qpso(self) -> BVector4: ...
    @property
    def qpsl(self) -> BVector4: ...
    @property
    def qpts(self) -> BVector4: ...
    @property
    def qptt(self) -> BVector4: ...
    @property
    def qptq(self) -> BVector4: ...
    @property
    def qptp(self) -> BVector4: ...
    @property
    def qpto(self) -> BVector4: ...
    @property
    def qptl(self) -> BVector4: ...
    @property
    def qpqs(self) -> BVector4: ...
    @property
    def qpqt(self) -> BVector4: ...
    @property
    def qpqq(self) -> BVector4: ...
    @property
    def qpqp(self) -> BVector4: ...
    @property
    def qpqo(self) -> BVector4: ...
    @property
    def qpql(self) -> BVector4: ...
    @property
    def qpps(self) -> BVector4: ...
    @property
    def qppt(self) -> BVector4: ...
    @property
    def qppq(self) -> BVector4: ...
    @property
    def qppp(self) -> BVector4: ...
    @property
    def qppo(self) -> BVector4: ...
    @property
    def qppl(self) -> BVector4: ...
    @property
    def qpos(self) -> BVector4: ...
    @property
    def qpot(self) -> BVector4: ...
    @property
    def qpoq(self) -> BVector4: ...
    @property
    def qpop(self) -> BVector4: ...
    @property
    def qpoo(self) -> BVector4: ...
    @property
    def qpol(self) -> BVector4: ...
    @property
    def qpls(self) -> BVector4: ...
    @property
    def qplt(self) -> BVector4: ...
    @property
    def qplq(self) -> BVector4: ...
    @property
    def qplp(self) -> BVector4: ...
    @property
    def qplo(self) -> BVector4: ...
    @property
    def qpll(self) -> BVector4: ...
    @property
    def qoss(self) -> BVector4: ...
    @property
    def qost(self) -> BVector4: ...
    @property
    def qosq(self) -> BVector4: ...
    @property
    def qosp(self) -> BVector4: ...
    @property
    def qoso(self) -> BVector4: ...
    @property
    def qosl(self) -> BVector4: ...
    @property
    def qots(self) -> BVector4: ...
    @property
    def qott(self) -> BVector4: ...
    @property
    def qotq(self) -> BVector4: ...
    @property
    def qotp(self) -> BVector4: ...
    @property
    def qoto(self) -> BVector4: ...
    @property
    def qotl(self) -> BVector4: ...
    @property
    def qoqs(self) -> BVector4: ...
    @property
    def qoqt(self) -> BVector4: ...
    @property
    def qoqq(self) -> BVector4: ...
    @property
    def qoqp(self) -> BVector4: ...
    @property
    def qoqo(self) -> BVector4: ...
    @property
    def qoql(self) -> BVector4: ...
    @property
    def qops(self) -> BVector4: ...
    @property
    def qopt(self) -> BVector4: ...
    @property
    def qopq(self) -> BVector4: ...
    @property
    def qopp(self) -> BVector4: ...
    @property
    def qopo(self) -> BVector4: ...
    @property
    def qopl(self) -> BVector4: ...
    @property
    def qoos(self) -> BVector4: ...
    @property
    def qoot(self) -> BVector4: ...
    @property
    def qooq(self) -> BVector4: ...
    @property
    def qoop(self) -> BVector4: ...
    @property
    def qooo(self) -> BVector4: ...
    @property
    def qool(self) -> BVector4: ...
    @property
    def qols(self) -> BVector4: ...
    @property
    def qolt(self) -> BVector4: ...
    @property
    def qolq(self) -> BVector4: ...
    @property
    def qolp(self) -> BVector4: ...
    @property
    def qolo(self) -> BVector4: ...
    @property
    def qoll(self) -> BVector4: ...
    @property
    def qlss(self) -> BVector4: ...
    @property
    def qlst(self) -> BVector4: ...
    @property
    def qlsq(self) -> BVector4: ...
    @property
    def qlsp(self) -> BVector4: ...
    @property
    def qlso(self) -> BVector4: ...
    @property
    def qlsl(self) -> BVector4: ...
    @property
    def qlts(self) -> BVector4: ...
    @property
    def qltt(self) -> BVector4: ...
    @property
    def qltq(self) -> BVector4: ...
    @property
    def qltp(self) -> BVector4: ...
    @property
    def qlto(self) -> BVector4: ...
    @property
    def qltl(self) -> BVector4: ...
    @property
    def qlqs(self) -> BVector4: ...
    @property
    def qlqt(self) -> BVector4: ...
    @property
    def qlqq(self) -> BVector4: ...
    @property
    def qlqp(self) -> BVector4: ...
    @property
    def qlqo(self) -> BVector4: ...
    @property
    def qlql(self) -> BVector4: ...
    @property
    def qlps(self) -> BVector4: ...
    @property
    def qlpt(self) -> BVector4: ...
    @property
    def qlpq(self) -> BVector4: ...
    @property
    def qlpp(self) -> BVector4: ...
    @property
    def qlpo(self) -> BVector4: ...
    @property
    def qlpl(self) -> BVector4: ...
    @property
    def qlos(self) -> BVector4: ...
    @property
    def qlot(self) -> BVector4: ...
    @property
    def qloq(self) -> BVector4: ...
    @property
    def qlop(self) -> BVector4: ...
    @property
    def qloo(self) -> BVector4: ...
    @property
    def qlol(self) -> BVector4: ...
    @property
    def qlls(self) -> BVector4: ...
    @property
    def qllt(self) -> BVector4: ...
    @property
    def qllq(self) -> BVector4: ...
    @property
    def qllp(self) -> BVector4: ...
    @property
    def qllo(self) -> BVector4: ...
    @property
    def qlll(self) -> BVector4: ...
    @property
    def psss(self) -> BVector4: ...
    @property
    def psst(self) -> BVector4: ...
    @property
    def pssq(self) -> BVector4: ...
    @property
    def pssp(self) -> BVector4: ...
    @property
    def psso(self) -> BVector4: ...
    @property
    def pssl(self) -> BVector4: ...
    @property
    def psts(self) -> BVector4: ...
    @property
    def pstt(self) -> BVector4: ...
    @property
    def pstq(self) -> BVector4: ...
    @property
    def pstp(self) -> BVector4: ...
    @property
    def psto(self) -> BVector4: ...
    @property
    def pstl(self) -> BVector4: ...
    @property
    def psqs(self) -> BVector4: ...
    @property
    def psqt(self) -> BVector4: ...
    @property
    def psqq(self) -> BVector4: ...
    @property
    def psqp(self) -> BVector4: ...
    @property
    def psqo(self) -> BVector4: ...
    @property
    def psql(self) -> BVector4: ...
    @property
    def psps(self) -> BVector4: ...
    @property
    def pspt(self) -> BVector4: ...
    @property
    def pspq(self) -> BVector4: ...
    @property
    def pspp(self) -> BVector4: ...
    @property
    def pspo(self) -> BVector4: ...
    @property
    def pspl(self) -> BVector4: ...
    @property
    def psos(self) -> BVector4: ...
    @property
    def psot(self) -> BVector4: ...
    @property
    def psoq(self) -> BVector4: ...
    @property
    def psop(self) -> BVector4: ...
    @property
    def psoo(self) -> BVector4: ...
    @property
    def psol(self) -> BVector4: ...
    @property
    def psls(self) -> BVector4: ...
    @property
    def pslt(self) -> BVector4: ...
    @property
    def pslq(self) -> BVector4: ...
    @property
    def pslp(self) -> BVector4: ...
    @property
    def pslo(self) -> BVector4: ...
    @property
    def psll(self) -> BVector4: ...
    @property
    def ptss(self) -> BVector4: ...
    @property
    def ptst(self) -> BVector4: ...
    @property
    def ptsq(self) -> BVector4: ...
    @property
    def ptsp(self) -> BVector4: ...
    @property
    def ptso(self) -> BVector4: ...
    @property
    def ptsl(self) -> BVector4: ...
    @property
    def ptts(self) -> BVector4: ...
    @property
    def pttt(self) -> BVector4: ...
    @property
    def pttq(self) -> BVector4: ...
    @property
    def pttp(self) -> BVector4: ...
    @property
    def ptto(self) -> BVector4: ...
    @property
    def pttl(self) -> BVector4: ...
    @property
    def ptqs(self) -> BVector4: ...
    @property
    def ptqt(self) -> BVector4: ...
    @property
    def ptqq(self) -> BVector4: ...
    @property
    def ptqp(self) -> BVector4: ...
    @property
    def ptqo(self) -> BVector4: ...
    @property
    def ptql(self) -> BVector4: ...
    @property
    def ptps(self) -> BVector4: ...
    @property
    def ptpt(self) -> BVector4: ...
    @property
    def ptpq(self) -> BVector4: ...
    @property
    def ptpp(self) -> BVector4: ...
    @property
    def ptpo(self) -> BVector4: ...
    @property
    def ptpl(self) -> BVector4: ...
    @property
    def ptos(self) -> BVector4: ...
    @property
    def ptot(self) -> BVector4: ...
    @property
    def ptoq(self) -> BVector4: ...
    @property
    def ptop(self) -> BVector4: ...
    @property
    def ptoo(self) -> BVector4: ...
    @property
    def ptol(self) -> BVector4: ...
    @property
    def ptls(self) -> BVector4: ...
    @property
    def ptlt(self) -> BVector4: ...
    @property
    def ptlq(self) -> BVector4: ...
    @property
    def ptlp(self) -> BVector4: ...
    @property
    def ptlo(self) -> BVector4: ...
    @property
    def ptll(self) -> BVector4: ...
    @property
    def pqss(self) -> BVector4: ...
    @property
    def pqst(self) -> BVector4: ...
    @property
    def pqsq(self) -> BVector4: ...
    @property
    def pqsp(self) -> BVector4: ...
    @property
    def pqso(self) -> BVector4: ...
    @property
    def pqsl(self) -> BVector4: ...
    @property
    def pqts(self) -> BVector4: ...
    @property
    def pqtt(self) -> BVector4: ...
    @property
    def pqtq(self) -> BVector4: ...
    @property
    def pqtp(self) -> BVector4: ...
    @property
    def pqto(self) -> BVector4: ...
    @property
    def pqtl(self) -> BVector4: ...
    @property
    def pqqs(self) -> BVector4: ...
    @property
    def pqqt(self) -> BVector4: ...
    @property
    def pqqq(self) -> BVector4: ...
    @property
    def pqqp(self) -> BVector4: ...
    @property
    def pqqo(self) -> BVector4: ...
    @property
    def pqql(self) -> BVector4: ...
    @property
    def pqps(self) -> BVector4: ...
    @property
    def pqpt(self) -> BVector4: ...
    @property
    def pqpq(self) -> BVector4: ...
    @property
    def pqpp(self) -> BVector4: ...
    @property
    def pqpo(self) -> BVector4: ...
    @property
    def pqpl(self) -> BVector4: ...
    @property
    def pqos(self) -> BVector4: ...
    @property
    def pqot(self) -> BVector4: ...
    @property
    def pqoq(self) -> BVector4: ...
    @property
    def pqop(self) -> BVector4: ...
    @property
    def pqoo(self) -> BVector4: ...
    @property
    def pqol(self) -> BVector4: ...
    @property
    def pqls(self) -> BVector4: ...
    @property
    def pqlt(self) -> BVector4: ...
    @property
    def pqlq(self) -> BVector4: ...
    @property
    def pqlp(self) -> BVector4: ...
    @property
    def pqlo(self) -> BVector4: ...
    @property
    def pqll(self) -> BVector4: ...
    @property
    def ppss(self) -> BVector4: ...
    @property
    def ppst(self) -> BVector4: ...
    @property
    def ppsq(self) -> BVector4: ...
    @property
    def ppsp(self) -> BVector4: ...
    @property
    def ppso(self) -> BVector4: ...
    @property
    def ppsl(self) -> BVector4: ...
    @property
    def ppts(self) -> BVector4: ...
    @property
    def pptt(self) -> BVector4: ...
    @property
    def pptq(self) -> BVector4: ...
    @property
    def pptp(self) -> BVector4: ...
    @property
    def ppto(self) -> BVector4: ...
    @property
    def pptl(self) -> BVector4: ...
    @property
    def ppqs(self) -> BVector4: ...
    @property
    def ppqt(self) -> BVector4: ...
    @property
    def ppqq(self) -> BVector4: ...
    @property
    def ppqp(self) -> BVector4: ...
    @property
    def ppqo(self) -> BVector4: ...
    @property
    def ppql(self) -> BVector4: ...
    @property
    def ppps(self) -> BVector4: ...
    @property
    def pppt(self) -> BVector4: ...
    @property
    def pppq(self) -> BVector4: ...
    @property
    def pppp(self) -> BVector4: ...
    @property
    def pppo(self) -> BVector4: ...
    @property
    def pppl(self) -> BVector4: ...
    @property
    def ppos(self) -> BVector4: ...
    @property
    def ppot(self) -> BVector4: ...
    @property
    def ppoq(self) -> BVector4: ...
    @property
    def ppop(self) -> BVector4: ...
    @property
    def ppoo(self) -> BVector4: ...
    @property
    def ppol(self) -> BVector4: ...
    @property
    def ppls(self) -> BVector4: ...
    @property
    def pplt(self) -> BVector4: ...
    @property
    def pplq(self) -> BVector4: ...
    @property
    def pplp(self) -> BVector4: ...
    @property
    def pplo(self) -> BVector4: ...
    @property
    def ppll(self) -> BVector4: ...
    @property
    def poss(self) -> BVector4: ...
    @property
    def post(self) -> BVector4: ...
    @property
    def posq(self) -> BVector4: ...
    @property
    def posp(self) -> BVector4: ...
    @property
    def poso(self) -> BVector4: ...
    @property
    def posl(self) -> BVector4: ...
    @property
    def pots(self) -> BVector4: ...
    @property
    def pott(self) -> BVector4: ...
    @property
    def potq(self) -> BVector4: ...
    @property
    def potp(self) -> BVector4: ...
    @property
    def poto(self) -> BVector4: ...
    @property
    def potl(self) -> BVector4: ...
    @property
    def poqs(self) -> BVector4: ...
    @property
    def poqt(self) -> BVector4: ...
    @property
    def poqq(self) -> BVector4: ...
    @property
    def poqp(self) -> BVector4: ...
    @property
    def poqo(self) -> BVector4: ...
    @property
    def poql(self) -> BVector4: ...
    @property
    def pops(self) -> BVector4: ...
    @property
    def popt(self) -> BVector4: ...
    @property
    def popq(self) -> BVector4: ...
    @property
    def popp(self) -> BVector4: ...
    @property
    def popo(self) -> BVector4: ...
    @property
    def popl(self) -> BVector4: ...
    @property
    def poos(self) -> BVector4: ...
    @property
    def poot(self) -> BVector4: ...
    @property
    def pooq(self) -> BVector4: ...
    @property
    def poop(self) -> BVector4: ...
    @property
    def pooo(self) -> BVector4: ...
    @property
    def pool(self) -> BVector4: ...
    @property
    def pols(self) -> BVector4: ...
    @property
    def polt(self) -> BVector4: ...
    @property
    def polq(self) -> BVector4: ...
    @property
    def polp(self) -> BVector4: ...
    @property
    def polo(self) -> BVector4: ...
    @property
    def poll(self) -> BVector4: ...
    @property
    def plss(self) -> BVector4: ...
    @property
    def plst(self) -> BVector4: ...
    @property
    def plsq(self) -> BVector4: ...
    @property
    def plsp(self) -> BVector4: ...
    @property
    def plso(self) -> BVector4: ...
    @property
    def plsl(self) -> BVector4: ...
    @property
    def plts(self) -> BVector4: ...
    @property
    def pltt(self) -> BVector4: ...
    @property
    def pltq(self) -> BVector4: ...
    @property
    def pltp(self) -> BVector4: ...
    @property
    def plto(self) -> BVector4: ...
    @property
    def pltl(self) -> BVector4: ...
    @property
    def plqs(self) -> BVector4: ...
    @property
    def plqt(self) -> BVector4: ...
    @property
    def plqq(self) -> BVector4: ...
    @property
    def plqp(self) -> BVector4: ...
    @property
    def plqo(self) -> BVector4: ...
    @property
    def plql(self) -> BVector4: ...
    @property
    def plps(self) -> BVector4: ...
    @property
    def plpt(self) -> BVector4: ...
    @property
    def plpq(self) -> BVector4: ...
    @property
    def plpp(self) -> BVector4: ...
    @property
    def plpo(self) -> BVector4: ...
    @property
    def plpl(self) -> BVector4: ...
    @property
    def plos(self) -> BVector4: ...
    @property
    def plot(self) -> BVector4: ...
    @property
    def ploq(self) -> BVector4: ...
    @property
    def plop(self) -> BVector4: ...
    @property
    def ploo(self) -> BVector4: ...
    @property
    def plol(self) -> BVector4: ...
    @property
    def plls(self) -> BVector4: ...
    @property
    def pllt(self) -> BVector4: ...
    @property
    def pllq(self) -> BVector4: ...
    @property
    def pllp(self) -> BVector4: ...
    @property
    def pllo(self) -> BVector4: ...
    @property
    def plll(self) -> BVector4: ...
    @property
    def osss(self) -> BVector4: ...
    @property
    def osst(self) -> BVector4: ...
    @property
    def ossq(self) -> BVector4: ...
    @property
    def ossp(self) -> BVector4: ...
    @property
    def osso(self) -> BVector4: ...
    @property
    def ossl(self) -> BVector4: ...
    @property
    def osts(self) -> BVector4: ...
    @property
    def ostt(self) -> BVector4: ...
    @property
    def ostq(self) -> BVector4: ...
    @property
    def ostp(self) -> BVector4: ...
    @property
    def osto(self) -> BVector4: ...
    @property
    def ostl(self) -> BVector4: ...
    @property
    def osqs(self) -> BVector4: ...
    @property
    def osqt(self) -> BVector4: ...
    @property
    def osqq(self) -> BVector4: ...
    @property
    def osqp(self) -> BVector4: ...
    @property
    def osqo(self) -> BVector4: ...
    @property
    def osql(self) -> BVector4: ...
    @property
    def osps(self) -> BVector4: ...
    @property
    def ospt(self) -> BVector4: ...
    @property
    def ospq(self) -> BVector4: ...
    @property
    def ospp(self) -> BVector4: ...
    @property
    def ospo(self) -> BVector4: ...
    @property
    def ospl(self) -> BVector4: ...
    @property
    def osos(self) -> BVector4: ...
    @property
    def osot(self) -> BVector4: ...
    @property
    def osoq(self) -> BVector4: ...
    @property
    def osop(self) -> BVector4: ...
    @property
    def osoo(self) -> BVector4: ...
    @property
    def osol(self) -> BVector4: ...
    @property
    def osls(self) -> BVector4: ...
    @property
    def oslt(self) -> BVector4: ...
    @property
    def oslq(self) -> BVector4: ...
    @property
    def oslp(self) -> BVector4: ...
    @property
    def oslo(self) -> BVector4: ...
    @property
    def osll(self) -> BVector4: ...
    @property
    def otss(self) -> BVector4: ...
    @property
    def otst(self) -> BVector4: ...
    @property
    def otsq(self) -> BVector4: ...
    @property
    def otsp(self) -> BVector4: ...
    @property
    def otso(self) -> BVector4: ...
    @property
    def otsl(self) -> BVector4: ...
    @property
    def otts(self) -> BVector4: ...
    @property
    def ottt(self) -> BVector4: ...
    @property
    def ottq(self) -> BVector4: ...
    @property
    def ottp(self) -> BVector4: ...
    @property
    def otto(self) -> BVector4: ...
    @property
    def ottl(self) -> BVector4: ...
    @property
    def otqs(self) -> BVector4: ...
    @property
    def otqt(self) -> BVector4: ...
    @property
    def otqq(self) -> BVector4: ...
    @property
    def otqp(self) -> BVector4: ...
    @property
    def otqo(self) -> BVector4: ...
    @property
    def otql(self) -> BVector4: ...
    @property
    def otps(self) -> BVector4: ...
    @property
    def otpt(self) -> BVector4: ...
    @property
    def otpq(self) -> BVector4: ...
    @property
    def otpp(self) -> BVector4: ...
    @property
    def otpo(self) -> BVector4: ...
    @property
    def otpl(self) -> BVector4: ...
    @property
    def otos(self) -> BVector4: ...
    @property
    def otot(self) -> BVector4: ...
    @property
    def otoq(self) -> BVector4: ...
    @property
    def otop(self) -> BVector4: ...
    @property
    def otoo(self) -> BVector4: ...
    @property
    def otol(self) -> BVector4: ...
    @property
    def otls(self) -> BVector4: ...
    @property
    def otlt(self) -> BVector4: ...
    @property
    def otlq(self) -> BVector4: ...
    @property
    def otlp(self) -> BVector4: ...
    @property
    def otlo(self) -> BVector4: ...
    @property
    def otll(self) -> BVector4: ...
    @property
    def oqss(self) -> BVector4: ...
    @property
    def oqst(self) -> BVector4: ...
    @property
    def oqsq(self) -> BVector4: ...
    @property
    def oqsp(self) -> BVector4: ...
    @property
    def oqso(self) -> BVector4: ...
    @property
    def oqsl(self) -> BVector4: ...
    @property
    def oqts(self) -> BVector4: ...
    @property
    def oqtt(self) -> BVector4: ...
    @property
    def oqtq(self) -> BVector4: ...
    @property
    def oqtp(self) -> BVector4: ...
    @property
    def oqto(self) -> BVector4: ...
    @property
    def oqtl(self) -> BVector4: ...
    @property
    def oqqs(self) -> BVector4: ...
    @property
    def oqqt(self) -> BVector4: ...
    @property
    def oqqq(self) -> BVector4: ...
    @property
    def oqqp(self) -> BVector4: ...
    @property
    def oqqo(self) -> BVector4: ...
    @property
    def oqql(self) -> BVector4: ...
    @property
    def oqps(self) -> BVector4: ...
    @property
    def oqpt(self) -> BVector4: ...
    @property
    def oqpq(self) -> BVector4: ...
    @property
    def oqpp(self) -> BVector4: ...
    @property
    def oqpo(self) -> BVector4: ...
    @property
    def oqpl(self) -> BVector4: ...
    @property
    def oqos(self) -> BVector4: ...
    @property
    def oqot(self) -> BVector4: ...
    @property
    def oqoq(self) -> BVector4: ...
    @property
    def oqop(self) -> BVector4: ...
    @property
    def oqoo(self) -> BVector4: ...
    @property
    def oqol(self) -> BVector4: ...
    @property
    def oqls(self) -> BVector4: ...
    @property
    def oqlt(self) -> BVector4: ...
    @property
    def oqlq(self) -> BVector4: ...
    @property
    def oqlp(self) -> BVector4: ...
    @property
    def oqlo(self) -> BVector4: ...
    @property
    def oqll(self) -> BVector4: ...
    @property
    def opss(self) -> BVector4: ...
    @property
    def opst(self) -> BVector4: ...
    @property
    def opsq(self) -> BVector4: ...
    @property
    def opsp(self) -> BVector4: ...
    @property
    def opso(self) -> BVector4: ...
    @property
    def opsl(self) -> BVector4: ...
    @property
    def opts(self) -> BVector4: ...
    @property
    def optt(self) -> BVector4: ...
    @property
    def optq(self) -> BVector4: ...
    @property
    def optp(self) -> BVector4: ...
    @property
    def opto(self) -> BVector4: ...
    @property
    def optl(self) -> BVector4: ...
    @property
    def opqs(self) -> BVector4: ...
    @property
    def opqt(self) -> BVector4: ...
    @property
    def opqq(self) -> BVector4: ...
    @property
    def opqp(self) -> BVector4: ...
    @property
    def opqo(self) -> BVector4: ...
    @property
    def opql(self) -> BVector4: ...
    @property
    def opps(self) -> BVector4: ...
    @property
    def oppt(self) -> BVector4: ...
    @property
    def oppq(self) -> BVector4: ...
    @property
    def oppp(self) -> BVector4: ...
    @property
    def oppo(self) -> BVector4: ...
    @property
    def oppl(self) -> BVector4: ...
    @property
    def opos(self) -> BVector4: ...
    @property
    def opot(self) -> BVector4: ...
    @property
    def opoq(self) -> BVector4: ...
    @property
    def opop(self) -> BVector4: ...
    @property
    def opoo(self) -> BVector4: ...
    @property
    def opol(self) -> BVector4: ...
    @property
    def opls(self) -> BVector4: ...
    @property
    def oplt(self) -> BVector4: ...
    @property
    def oplq(self) -> BVector4: ...
    @property
    def oplp(self) -> BVector4: ...
    @property
    def oplo(self) -> BVector4: ...
    @property
    def opll(self) -> BVector4: ...
    @property
    def ooss(self) -> BVector4: ...
    @property
    def oost(self) -> BVector4: ...
    @property
    def oosq(self) -> BVector4: ...
    @property
    def oosp(self) -> BVector4: ...
    @property
    def ooso(self) -> BVector4: ...
    @property
    def oosl(self) -> BVector4: ...
    @property
    def oots(self) -> BVector4: ...
    @property
    def oott(self) -> BVector4: ...
    @property
    def ootq(self) -> BVector4: ...
    @property
    def ootp(self) -> BVector4: ...
    @property
    def ooto(self) -> BVector4: ...
    @property
    def ootl(self) -> BVector4: ...
    @property
    def ooqs(self) -> BVector4: ...
    @property
    def ooqt(self) -> BVector4: ...
    @property
    def ooqq(self) -> BVector4: ...
    @property
    def ooqp(self) -> BVector4: ...
    @property
    def ooqo(self) -> BVector4: ...
    @property
    def ooql(self) -> BVector4: ...
    @property
    def oops(self) -> BVector4: ...
    @property
    def oopt(self) -> BVector4: ...
    @property
    def oopq(self) -> BVector4: ...
    @property
    def oopp(self) -> BVector4: ...
    @property
    def oopo(self) -> BVector4: ...
    @property
    def oopl(self) -> BVector4: ...
    @property
    def ooos(self) -> BVector4: ...
    @property
    def ooot(self) -> BVector4: ...
    @property
    def oooq(self) -> BVector4: ...
    @property
    def ooop(self) -> BVector4: ...
    @property
    def ools(self) -> BVector4: ...
    @property
    def oolt(self) -> BVector4: ...
    @property
    def oolq(self) -> BVector4: ...
    @property
    def oolp(self) -> BVector4: ...
    @property
    def olss(self) -> BVector4: ...
    @property
    def olst(self) -> BVector4: ...
    @property
    def olsq(self) -> BVector4: ...
    @property
    def olsp(self) -> BVector4: ...
    @property
    def olso(self) -> BVector4: ...
    @property
    def olsl(self) -> BVector4: ...
    @property
    def olts(self) -> BVector4: ...
    @property
    def oltt(self) -> BVector4: ...
    @property
    def oltq(self) -> BVector4: ...
    @property
    def oltp(self) -> BVector4: ...
    @property
    def olto(self) -> BVector4: ...
    @property
    def oltl(self) -> BVector4: ...
    @property
    def olqs(self) -> BVector4: ...
    @property
    def olqt(self) -> BVector4: ...
    @property
    def olqq(self) -> BVector4: ...
    @property
    def olqp(self) -> BVector4: ...
    @property
    def olqo(self) -> BVector4: ...
    @property
    def olql(self) -> BVector4: ...
    @property
    def olps(self) -> BVector4: ...
    @property
    def olpt(self) -> BVector4: ...
    @property
    def olpq(self) -> BVector4: ...
    @property
    def olpp(self) -> BVector4: ...
    @property
    def olpo(self) -> BVector4: ...
    @property
    def olpl(self) -> BVector4: ...
    @property
    def olos(self) -> BVector4: ...
    @property
    def olot(self) -> BVector4: ...
    @property
    def oloq(self) -> BVector4: ...
    @property
    def olop(self) -> BVector4: ...
    @property
    def olls(self) -> BVector4: ...
    @property
    def ollt(self) -> BVector4: ...
    @property
    def ollq(self) -> BVector4: ...
    @property
    def ollp(self) -> BVector4: ...
    @property
    def lsss(self) -> BVector4: ...
    @property
    def lsst(self) -> BVector4: ...
    @property
    def lssq(self) -> BVector4: ...
    @property
    def lssp(self) -> BVector4: ...
    @property
    def lsso(self) -> BVector4: ...
    @property
    def lssl(self) -> BVector4: ...
    @property
    def lsts(self) -> BVector4: ...
    @property
    def lstt(self) -> BVector4: ...
    @property
    def lstq(self) -> BVector4: ...
    @property
    def lstp(self) -> BVector4: ...
    @property
    def lsto(self) -> BVector4: ...
    @property
    def lstl(self) -> BVector4: ...
    @property
    def lsqs(self) -> BVector4: ...
    @property
    def lsqt(self) -> BVector4: ...
    @property
    def lsqq(self) -> BVector4: ...
    @property
    def lsqp(self) -> BVector4: ...
    @property
    def lsqo(self) -> BVector4: ...
    @property
    def lsql(self) -> BVector4: ...
    @property
    def lsps(self) -> BVector4: ...
    @property
    def lspt(self) -> BVector4: ...
    @property
    def lspq(self) -> BVector4: ...
    @property
    def lspp(self) -> BVector4: ...
    @property
    def lspo(self) -> BVector4: ...
    @property
    def lspl(self) -> BVector4: ...
    @property
    def lsos(self) -> BVector4: ...
    @property
    def lsot(self) -> BVector4: ...
    @property
    def lsoq(self) -> BVector4: ...
    @property
    def lsop(self) -> BVector4: ...
    @property
    def lsoo(self) -> BVector4: ...
    @property
    def lsol(self) -> BVector4: ...
    @property
    def lsls(self) -> BVector4: ...
    @property
    def lslt(self) -> BVector4: ...
    @property
    def lslq(self) -> BVector4: ...
    @property
    def lslp(self) -> BVector4: ...
    @property
    def lslo(self) -> BVector4: ...
    @property
    def lsll(self) -> BVector4: ...
    @property
    def ltss(self) -> BVector4: ...
    @property
    def ltst(self) -> BVector4: ...
    @property
    def ltsq(self) -> BVector4: ...
    @property
    def ltsp(self) -> BVector4: ...
    @property
    def ltso(self) -> BVector4: ...
    @property
    def ltsl(self) -> BVector4: ...
    @property
    def ltts(self) -> BVector4: ...
    @property
    def lttt(self) -> BVector4: ...
    @property
    def lttq(self) -> BVector4: ...
    @property
    def lttp(self) -> BVector4: ...
    @property
    def ltto(self) -> BVector4: ...
    @property
    def lttl(self) -> BVector4: ...
    @property
    def ltqs(self) -> BVector4: ...
    @property
    def ltqt(self) -> BVector4: ...
    @property
    def ltqq(self) -> BVector4: ...
    @property
    def ltqp(self) -> BVector4: ...
    @property
    def ltqo(self) -> BVector4: ...
    @property
    def ltql(self) -> BVector4: ...
    @property
    def ltps(self) -> BVector4: ...
    @property
    def ltpt(self) -> BVector4: ...
    @property
    def ltpq(self) -> BVector4: ...
    @property
    def ltpp(self) -> BVector4: ...
    @property
    def ltpo(self) -> BVector4: ...
    @property
    def ltpl(self) -> BVector4: ...
    @property
    def ltos(self) -> BVector4: ...
    @property
    def ltot(self) -> BVector4: ...
    @property
    def ltoq(self) -> BVector4: ...
    @property
    def ltop(self) -> BVector4: ...
    @property
    def ltoo(self) -> BVector4: ...
    @property
    def ltol(self) -> BVector4: ...
    @property
    def ltls(self) -> BVector4: ...
    @property
    def ltlt(self) -> BVector4: ...
    @property
    def ltlq(self) -> BVector4: ...
    @property
    def ltlp(self) -> BVector4: ...
    @property
    def ltlo(self) -> BVector4: ...
    @property
    def ltll(self) -> BVector4: ...
    @property
    def lqss(self) -> BVector4: ...
    @property
    def lqst(self) -> BVector4: ...
    @property
    def lqsq(self) -> BVector4: ...
    @property
    def lqsp(self) -> BVector4: ...
    @property
    def lqso(self) -> BVector4: ...
    @property
    def lqsl(self) -> BVector4: ...
    @property
    def lqts(self) -> BVector4: ...
    @property
    def lqtt(self) -> BVector4: ...
    @property
    def lqtq(self) -> BVector4: ...
    @property
    def lqtp(self) -> BVector4: ...
    @property
    def lqto(self) -> BVector4: ...
    @property
    def lqtl(self) -> BVector4: ...
    @property
    def lqqs(self) -> BVector4: ...
    @property
    def lqqt(self) -> BVector4: ...
    @property
    def lqqq(self) -> BVector4: ...
    @property
    def lqqp(self) -> BVector4: ...
    @property
    def lqqo(self) -> BVector4: ...
    @property
    def lqql(self) -> BVector4: ...
    @property
    def lqps(self) -> BVector4: ...
    @property
    def lqpt(self) -> BVector4: ...
    @property
    def lqpq(self) -> BVector4: ...
    @property
    def lqpp(self) -> BVector4: ...
    @property
    def lqpo(self) -> BVector4: ...
    @property
    def lqpl(self) -> BVector4: ...
    @property
    def lqos(self) -> BVector4: ...
    @property
    def lqot(self) -> BVector4: ...
    @property
    def lqoq(self) -> BVector4: ...
    @property
    def lqop(self) -> BVector4: ...
    @property
    def lqoo(self) -> BVector4: ...
    @property
    def lqol(self) -> BVector4: ...
    @property
    def lqls(self) -> BVector4: ...
    @property
    def lqlt(self) -> BVector4: ...
    @property
    def lqlq(self) -> BVector4: ...
    @property
    def lqlp(self) -> BVector4: ...
    @property
    def lqlo(self) -> BVector4: ...
    @property
    def lqll(self) -> BVector4: ...
    @property
    def lpss(self) -> BVector4: ...
    @property
    def lpst(self) -> BVector4: ...
    @property
    def lpsq(self) -> BVector4: ...
    @property
    def lpsp(self) -> BVector4: ...
    @property
    def lpso(self) -> BVector4: ...
    @property
    def lpsl(self) -> BVector4: ...
    @property
    def lpts(self) -> BVector4: ...
    @property
    def lptt(self) -> BVector4: ...
    @property
    def lptq(self) -> BVector4: ...
    @property
    def lptp(self) -> BVector4: ...
    @property
    def lpto(self) -> BVector4: ...
    @property
    def lptl(self) -> BVector4: ...
    @property
    def lpqs(self) -> BVector4: ...
    @property
    def lpqt(self) -> BVector4: ...
    @property
    def lpqq(self) -> BVector4: ...
    @property
    def lpqp(self) -> BVector4: ...
    @property
    def lpqo(self) -> BVector4: ...
    @property
    def lpql(self) -> BVector4: ...
    @property
    def lpps(self) -> BVector4: ...
    @property
    def lppt(self) -> BVector4: ...
    @property
    def lppq(self) -> BVector4: ...
    @property
    def lppp(self) -> BVector4: ...
    @property
    def lppo(self) -> BVector4: ...
    @property
    def lppl(self) -> BVector4: ...
    @property
    def lpos(self) -> BVector4: ...
    @property
    def lpot(self) -> BVector4: ...
    @property
    def lpoq(self) -> BVector4: ...
    @property
    def lpop(self) -> BVector4: ...
    @property
    def lpoo(self) -> BVector4: ...
    @property
    def lpol(self) -> BVector4: ...
    @property
    def lpls(self) -> BVector4: ...
    @property
    def lplt(self) -> BVector4: ...
    @property
    def lplq(self) -> BVector4: ...
    @property
    def lplp(self) -> BVector4: ...
    @property
    def lplo(self) -> BVector4: ...
    @property
    def lpll(self) -> BVector4: ...
    @property
    def loss(self) -> BVector4: ...
    @property
    def lost(self) -> BVector4: ...
    @property
    def losq(self) -> BVector4: ...
    @property
    def losp(self) -> BVector4: ...
    @property
    def loso(self) -> BVector4: ...
    @property
    def losl(self) -> BVector4: ...
    @property
    def lots(self) -> BVector4: ...
    @property
    def lott(self) -> BVector4: ...
    @property
    def lotq(self) -> BVector4: ...
    @property
    def lotp(self) -> BVector4: ...
    @property
    def loto(self) -> BVector4: ...
    @property
    def lotl(self) -> BVector4: ...
    @property
    def loqs(self) -> BVector4: ...
    @property
    def loqt(self) -> BVector4: ...
    @property
    def loqq(self) -> BVector4: ...
    @property
    def loqp(self) -> BVector4: ...
    @property
    def loqo(self) -> BVector4: ...
    @property
    def loql(self) -> BVector4: ...
    @property
    def lops(self) -> BVector4: ...
    @property
    def lopt(self) -> BVector4: ...
    @property
    def lopq(self) -> BVector4: ...
    @property
    def lopp(self) -> BVector4: ...
    @property
    def lopo(self) -> BVector4: ...
    @property
    def lopl(self) -> BVector4: ...
    @property
    def loos(self) -> BVector4: ...
    @property
    def loot(self) -> BVector4: ...
    @property
    def looq(self) -> BVector4: ...
    @property
    def loop(self) -> BVector4: ...
    @property
    def lols(self) -> BVector4: ...
    @property
    def lolt(self) -> BVector4: ...
    @property
    def lolq(self) -> BVector4: ...
    @property
    def lolp(self) -> BVector4: ...
    @property
    def llss(self) -> BVector4: ...
    @property
    def llst(self) -> BVector4: ...
    @property
    def llsq(self) -> BVector4: ...
    @property
    def llsp(self) -> BVector4: ...
    @property
    def llso(self) -> BVector4: ...
    @property
    def llsl(self) -> BVector4: ...
    @property
    def llts(self) -> BVector4: ...
    @property
    def lltt(self) -> BVector4: ...
    @property
    def lltq(self) -> BVector4: ...
    @property
    def lltp(self) -> BVector4: ...
    @property
    def llto(self) -> BVector4: ...
    @property
    def lltl(self) -> BVector4: ...
    @property
    def llqs(self) -> BVector4: ...
    @property
    def llqt(self) -> BVector4: ...
    @property
    def llqq(self) -> BVector4: ...
    @property
    def llqp(self) -> BVector4: ...
    @property
    def llqo(self) -> BVector4: ...
    @property
    def llql(self) -> BVector4: ...
    @property
    def llps(self) -> BVector4: ...
    @property
    def llpt(self) -> BVector4: ...
    @property
    def llpq(self) -> BVector4: ...
    @property
    def llpp(self) -> BVector4: ...
    @property
    def llpo(self) -> BVector4: ...
    @property
    def llpl(self) -> BVector4: ...
    @property
    def llos(self) -> BVector4: ...
    @property
    def llot(self) -> BVector4: ...
    @property
    def lloq(self) -> BVector4: ...
    @property
    def llop(self) -> BVector4: ...
    @property
    def llls(self) -> BVector4: ...
    @property
    def lllt(self) -> BVector4: ...
    @property
    def lllq(self) -> BVector4: ...
    @property
    def lllp(self) -> BVector4: ...
    @property
    def u(self) -> _bool: ...
    @property
    def v(self) -> _bool: ...
    @property
    def uu(self) -> BVector2: ...
    @property
    def uv(self) -> BVector2: ...
    @property
    def uo(self) -> BVector2: ...
    @property
    def ul(self) -> BVector2: ...
    @property
    def vu(self) -> BVector2: ...
    @property
    def vv(self) -> BVector2: ...
    @property
    def vo(self) -> BVector2: ...
    @property
    def vl(self) -> BVector2: ...
    @property
    def ou(self) -> BVector2: ...
    @property
    def ov(self) -> BVector2: ...
    @property
    def lu(self) -> BVector2: ...
    @property
    def lv(self) -> BVector2: ...
    @property
    def uuu(self) -> BVector3: ...
    @property
    def uuv(self) -> BVector3: ...
    @property
    def uuo(self) -> BVector3: ...
    @property
    def uul(self) -> BVector3: ...
    @property
    def uvu(self) -> BVector3: ...
    @property
    def uvv(self) -> BVector3: ...
    @property
    def uvo(self) -> BVector3: ...
    @property
    def uvl(self) -> BVector3: ...
    @property
    def uou(self) -> BVector3: ...
    @property
    def uov(self) -> BVector3: ...
    @property
    def uoo(self) -> BVector3: ...
    @property
    def uol(self) -> BVector3: ...
    @property
    def ulu(self) -> BVector3: ...
    @property
    def ulv(self) -> BVector3: ...
    @property
    def ulo(self) -> BVector3: ...
    @property
    def ull(self) -> BVector3: ...
    @property
    def vuu(self) -> BVector3: ...
    @property
    def vuv(self) -> BVector3: ...
    @property
    def vuo(self) -> BVector3: ...
    @property
    def vul(self) -> BVector3: ...
    @property
    def vvu(self) -> BVector3: ...
    @property
    def vvv(self) -> BVector3: ...
    @property
    def vvo(self) -> BVector3: ...
    @property
    def vvl(self) -> BVector3: ...
    @property
    def vou(self) -> BVector3: ...
    @property
    def vov(self) -> BVector3: ...
    @property
    def voo(self) -> BVector3: ...
    @property
    def vol(self) -> BVector3: ...
    @property
    def vlu(self) -> BVector3: ...
    @property
    def vlv(self) -> BVector3: ...
    @property
    def vlo(self) -> BVector3: ...
    @property
    def vll(self) -> BVector3: ...
    @property
    def ouu(self) -> BVector3: ...
    @property
    def ouv(self) -> BVector3: ...
    @property
    def ouo(self) -> BVector3: ...
    @property
    def oul(self) -> BVector3: ...
    @property
    def ovu(self) -> BVector3: ...
    @property
    def ovv(self) -> BVector3: ...
    @property
    def ovo(self) -> BVector3: ...
    @property
    def ovl(self) -> BVector3: ...
    @property
    def oou(self) -> BVector3: ...
    @property
    def oov(self) -> BVector3: ...
    @property
    def olu(self) -> BVector3: ...
    @property
    def olv(self) -> BVector3: ...
    @property
    def luu(self) -> BVector3: ...
    @property
    def luv(self) -> BVector3: ...
    @property
    def luo(self) -> BVector3: ...
    @property
    def lul(self) -> BVector3: ...
    @property
    def lvu(self) -> BVector3: ...
    @property
    def lvv(self) -> BVector3: ...
    @property
    def lvo(self) -> BVector3: ...
    @property
    def lvl(self) -> BVector3: ...
    @property
    def lou(self) -> BVector3: ...
    @property
    def lov(self) -> BVector3: ...
    @property
    def llu(self) -> BVector3: ...
    @property
    def llv(self) -> BVector3: ...
    @property
    def uuuu(self) -> BVector4: ...
    @property
    def uuuv(self) -> BVector4: ...
    @property
    def uuuo(self) -> BVector4: ...
    @property
    def uuul(self) -> BVector4: ...
    @property
    def uuvu(self) -> BVector4: ...
    @property
    def uuvv(self) -> BVector4: ...
    @property
    def uuvo(self) -> BVector4: ...
    @property
    def uuvl(self) -> BVector4: ...
    @property
    def uuou(self) -> BVector4: ...
    @property
    def uuov(self) -> BVector4: ...
    @property
    def uuoo(self) -> BVector4: ...
    @property
    def uuol(self) -> BVector4: ...
    @property
    def uulu(self) -> BVector4: ...
    @property
    def uulv(self) -> BVector4: ...
    @property
    def uulo(self) -> BVector4: ...
    @property
    def uull(self) -> BVector4: ...
    @property
    def uvuu(self) -> BVector4: ...
    @property
    def uvuv(self) -> BVector4: ...
    @property
    def uvuo(self) -> BVector4: ...
    @property
    def uvul(self) -> BVector4: ...
    @property
    def uvvu(self) -> BVector4: ...
    @property
    def uvvv(self) -> BVector4: ...
    @property
    def uvvo(self) -> BVector4: ...
    @property
    def uvvl(self) -> BVector4: ...
    @property
    def uvou(self) -> BVector4: ...
    @property
    def uvov(self) -> BVector4: ...
    @property
    def uvoo(self) -> BVector4: ...
    @property
    def uvol(self) -> BVector4: ...
    @property
    def uvlu(self) -> BVector4: ...
    @property
    def uvlv(self) -> BVector4: ...
    @property
    def uvlo(self) -> BVector4: ...
    @property
    def uvll(self) -> BVector4: ...
    @property
    def uouu(self) -> BVector4: ...
    @property
    def uouv(self) -> BVector4: ...
    @property
    def uouo(self) -> BVector4: ...
    @property
    def uoul(self) -> BVector4: ...
    @property
    def uovu(self) -> BVector4: ...
    @property
    def uovv(self) -> BVector4: ...
    @property
    def uovo(self) -> BVector4: ...
    @property
    def uovl(self) -> BVector4: ...
    @property
    def uoou(self) -> BVector4: ...
    @property
    def uoov(self) -> BVector4: ...
    @property
    def uooo(self) -> BVector4: ...
    @property
    def uool(self) -> BVector4: ...
    @property
    def uolu(self) -> BVector4: ...
    @property
    def uolv(self) -> BVector4: ...
    @property
    def uolo(self) -> BVector4: ...
    @property
    def uoll(self) -> BVector4: ...
    @property
    def uluu(self) -> BVector4: ...
    @property
    def uluv(self) -> BVector4: ...
    @property
    def uluo(self) -> BVector4: ...
    @property
    def ulul(self) -> BVector4: ...
    @property
    def ulvu(self) -> BVector4: ...
    @property
    def ulvv(self) -> BVector4: ...
    @property
    def ulvo(self) -> BVector4: ...
    @property
    def ulvl(self) -> BVector4: ...
    @property
    def ulou(self) -> BVector4: ...
    @property
    def ulov(self) -> BVector4: ...
    @property
    def uloo(self) -> BVector4: ...
    @property
    def ulol(self) -> BVector4: ...
    @property
    def ullu(self) -> BVector4: ...
    @property
    def ullv(self) -> BVector4: ...
    @property
    def ullo(self) -> BVector4: ...
    @property
    def ulll(self) -> BVector4: ...
    @property
    def vuuu(self) -> BVector4: ...
    @property
    def vuuv(self) -> BVector4: ...
    @property
    def vuuo(self) -> BVector4: ...
    @property
    def vuul(self) -> BVector4: ...
    @property
    def vuvu(self) -> BVector4: ...
    @property
    def vuvv(self) -> BVector4: ...
    @property
    def vuvo(self) -> BVector4: ...
    @property
    def vuvl(self) -> BVector4: ...
    @property
    def vuou(self) -> BVector4: ...
    @property
    def vuov(self) -> BVector4: ...
    @property
    def vuoo(self) -> BVector4: ...
    @property
    def vuol(self) -> BVector4: ...
    @property
    def vulu(self) -> BVector4: ...
    @property
    def vulv(self) -> BVector4: ...
    @property
    def vulo(self) -> BVector4: ...
    @property
    def vull(self) -> BVector4: ...
    @property
    def vvuu(self) -> BVector4: ...
    @property
    def vvuv(self) -> BVector4: ...
    @property
    def vvuo(self) -> BVector4: ...
    @property
    def vvul(self) -> BVector4: ...
    @property
    def vvvu(self) -> BVector4: ...
    @property
    def vvvv(self) -> BVector4: ...
    @property
    def vvvo(self) -> BVector4: ...
    @property
    def vvvl(self) -> BVector4: ...
    @property
    def vvou(self) -> BVector4: ...
    @property
    def vvov(self) -> BVector4: ...
    @property
    def vvoo(self) -> BVector4: ...
    @property
    def vvol(self) -> BVector4: ...
    @property
    def vvlu(self) -> BVector4: ...
    @property
    def vvlv(self) -> BVector4: ...
    @property
    def vvlo(self) -> BVector4: ...
    @property
    def vvll(self) -> BVector4: ...
    @property
    def vouu(self) -> BVector4: ...
    @property
    def vouv(self) -> BVector4: ...
    @property
    def vouo(self) -> BVector4: ...
    @property
    def voul(self) -> BVector4: ...
    @property
    def vovu(self) -> BVector4: ...
    @property
    def vovv(self) -> BVector4: ...
    @property
    def vovo(self) -> BVector4: ...
    @property
    def vovl(self) -> BVector4: ...
    @property
    def voou(self) -> BVector4: ...
    @property
    def voov(self) -> BVector4: ...
    @property
    def vooo(self) -> BVector4: ...
    @property
    def vool(self) -> BVector4: ...
    @property
    def volu(self) -> BVector4: ...
    @property
    def volv(self) -> BVector4: ...
    @property
    def volo(self) -> BVector4: ...
    @property
    def voll(self) -> BVector4: ...
    @property
    def vluu(self) -> BVector4: ...
    @property
    def vluv(self) -> BVector4: ...
    @property
    def vluo(self) -> BVector4: ...
    @property
    def vlul(self) -> BVector4: ...
    @property
    def vlvu(self) -> BVector4: ...
    @property
    def vlvv(self) -> BVector4: ...
    @property
    def vlvo(self) -> BVector4: ...
    @property
    def vlvl(self) -> BVector4: ...
    @property
    def vlou(self) -> BVector4: ...
    @property
    def vlov(self) -> BVector4: ...
    @property
    def vloo(self) -> BVector4: ...
    @property
    def vlol(self) -> BVector4: ...
    @property
    def vllu(self) -> BVector4: ...
    @property
    def vllv(self) -> BVector4: ...
    @property
    def vllo(self) -> BVector4: ...
    @property
    def vlll(self) -> BVector4: ...
    @property
    def ouuu(self) -> BVector4: ...
    @property
    def ouuv(self) -> BVector4: ...
    @property
    def ouuo(self) -> BVector4: ...
    @property
    def ouul(self) -> BVector4: ...
    @property
    def ouvu(self) -> BVector4: ...
    @property
    def ouvv(self) -> BVector4: ...
    @property
    def ouvo(self) -> BVector4: ...
    @property
    def ouvl(self) -> BVector4: ...
    @property
    def ouou(self) -> BVector4: ...
    @property
    def ouov(self) -> BVector4: ...
    @property
    def ouoo(self) -> BVector4: ...
    @property
    def ouol(self) -> BVector4: ...
    @property
    def oulu(self) -> BVector4: ...
    @property
    def oulv(self) -> BVector4: ...
    @property
    def oulo(self) -> BVector4: ...
    @property
    def oull(self) -> BVector4: ...
    @property
    def ovuu(self) -> BVector4: ...
    @property
    def ovuv(self) -> BVector4: ...
    @property
    def ovuo(self) -> BVector4: ...
    @property
    def ovul(self) -> BVector4: ...
    @property
    def ovvu(self) -> BVector4: ...
    @property
    def ovvv(self) -> BVector4: ...
    @property
    def ovvo(self) -> BVector4: ...
    @property
    def ovvl(self) -> BVector4: ...
    @property
    def ovou(self) -> BVector4: ...
    @property
    def ovov(self) -> BVector4: ...
    @property
    def ovoo(self) -> BVector4: ...
    @property
    def ovol(self) -> BVector4: ...
    @property
    def ovlu(self) -> BVector4: ...
    @property
    def ovlv(self) -> BVector4: ...
    @property
    def ovlo(self) -> BVector4: ...
    @property
    def ovll(self) -> BVector4: ...
    @property
    def oouu(self) -> BVector4: ...
    @property
    def oouv(self) -> BVector4: ...
    @property
    def oouo(self) -> BVector4: ...
    @property
    def ooul(self) -> BVector4: ...
    @property
    def oovu(self) -> BVector4: ...
    @property
    def oovv(self) -> BVector4: ...
    @property
    def oovo(self) -> BVector4: ...
    @property
    def oovl(self) -> BVector4: ...
    @property
    def ooou(self) -> BVector4: ...
    @property
    def ooov(self) -> BVector4: ...
    @property
    def oolu(self) -> BVector4: ...
    @property
    def oolv(self) -> BVector4: ...
    @property
    def oluu(self) -> BVector4: ...
    @property
    def oluv(self) -> BVector4: ...
    @property
    def oluo(self) -> BVector4: ...
    @property
    def olul(self) -> BVector4: ...
    @property
    def olvu(self) -> BVector4: ...
    @property
    def olvv(self) -> BVector4: ...
    @property
    def olvo(self) -> BVector4: ...
    @property
    def olvl(self) -> BVector4: ...
    @property
    def olou(self) -> BVector4: ...
    @property
    def olov(self) -> BVector4: ...
    @property
    def ollu(self) -> BVector4: ...
    @property
    def ollv(self) -> BVector4: ...
    @property
    def luuu(self) -> BVector4: ...
    @property
    def luuv(self) -> BVector4: ...
    @property
    def luuo(self) -> BVector4: ...
    @property
    def luul(self) -> BVector4: ...
    @property
    def luvu(self) -> BVector4: ...
    @property
    def luvv(self) -> BVector4: ...
    @property
    def luvo(self) -> BVector4: ...
    @property
    def luvl(self) -> BVector4: ...
    @property
    def luou(self) -> BVector4: ...
    @property
    def luov(self) -> BVector4: ...
    @property
    def luoo(self) -> BVector4: ...
    @property
    def luol(self) -> BVector4: ...
    @property
    def lulu(self) -> BVector4: ...
    @property
    def lulv(self) -> BVector4: ...
    @property
    def lulo(self) -> BVector4: ...
    @property
    def lull(self) -> BVector4: ...
    @property
    def lvuu(self) -> BVector4: ...
    @property
    def lvuv(self) -> BVector4: ...
    @property
    def lvuo(self) -> BVector4: ...
    @property
    def lvul(self) -> BVector4: ...
    @property
    def lvvu(self) -> BVector4: ...
    @property
    def lvvv(self) -> BVector4: ...
    @property
    def lvvo(self) -> BVector4: ...
    @property
    def lvvl(self) -> BVector4: ...
    @property
    def lvou(self) -> BVector4: ...
    @property
    def lvov(self) -> BVector4: ...
    @property
    def lvoo(self) -> BVector4: ...
    @property
    def lvol(self) -> BVector4: ...
    @property
    def lvlu(self) -> BVector4: ...
    @property
    def lvlv(self) -> BVector4: ...
    @property
    def lvlo(self) -> BVector4: ...
    @property
    def lvll(self) -> BVector4: ...
    @property
    def louu(self) -> BVector4: ...
    @property
    def louv(self) -> BVector4: ...
    @property
    def louo(self) -> BVector4: ...
    @property
    def loul(self) -> BVector4: ...
    @property
    def lovu(self) -> BVector4: ...
    @property
    def lovv(self) -> BVector4: ...
    @property
    def lovo(self) -> BVector4: ...
    @property
    def lovl(self) -> BVector4: ...
    @property
    def loou(self) -> BVector4: ...
    @property
    def loov(self) -> BVector4: ...
    @property
    def lolu(self) -> BVector4: ...
    @property
    def lolv(self) -> BVector4: ...
    @property
    def lluu(self) -> BVector4: ...
    @property
    def lluv(self) -> BVector4: ...
    @property
    def lluo(self) -> BVector4: ...
    @property
    def llul(self) -> BVector4: ...
    @property
    def llvu(self) -> BVector4: ...
    @property
    def llvv(self) -> BVector4: ...
    @property
    def llvo(self) -> BVector4: ...
    @property
    def llvl(self) -> BVector4: ...
    @property
    def llou(self) -> BVector4: ...
    @property
    def llov(self) -> BVector4: ...
    @property
    def lllu(self) -> BVector4: ...
    @property
    def lllv(self) -> BVector4: ...
    @property
    def oo(self) -> BVector2: ...
    @property
    def ol(self) -> BVector2: ...
    @property
    def lo(self) -> BVector2: ...
    @property
    def ll(self) -> BVector2: ...
    @property
    def ooo(self) -> BVector3: ...
    @property
    def ool(self) -> BVector3: ...
    @property
    def olo(self) -> BVector3: ...
    @property
    def oll(self) -> BVector3: ...
    @property
    def loo(self) -> BVector3: ...
    @property
    def lol(self) -> BVector3: ...
    @property
    def llo(self) -> BVector3: ...
    @property
    def lll(self) -> BVector3: ...
    @property
    def oooo(self) -> BVector4: ...
    @property
    def oool(self) -> BVector4: ...
    @property
    def oolo(self) -> BVector4: ...
    @property
    def ooll(self) -> BVector4: ...
    @property
    def oloo(self) -> BVector4: ...
    @property
    def olol(self) -> BVector4: ...
    @property
    def ollo(self) -> BVector4: ...
    @property
    def olll(self) -> BVector4: ...
    @property
    def looo(self) -> BVector4: ...
    @property
    def lool(self) -> BVector4: ...
    @property
    def lolo(self) -> BVector4: ...
    @property
    def loll(self) -> BVector4: ...
    @property
    def lloo(self) -> BVector4: ...
    @property
    def llol(self) -> BVector4: ...
    @property
    def lllo(self) -> BVector4: ...
    @property
    def llll(self) -> BVector4: ...
    def min(self, n: Number, /) -> BVector4: ...
    def max(self, n: Number, /) -> BVector4: ...
    def clamp(self, min: Number, max: Number, /) -> BVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[_bool, _bool]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[BVector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector4: ...

@final
class BVector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: BVector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[BVector4]: ...
    @overload
    def __getitem__(self, index: int) -> BVector4: ...
    @overload
    def __getitem__(self, index: slice) -> BVector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> BVector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[BVector4]: ...

@final
class DVector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DVector4) -> DVector4: ...
    @overload
    def __add__(self, other: Number) -> DVector4: ...
    @overload
    def __radd__(self, other: DVector4) -> DVector4: ...
    @overload
    def __radd__(self, other: Number) -> DVector4: ...
    @overload
    def __sub__(self, other: DVector4) -> DVector4: ...
    @overload
    def __sub__(self, other: Number) -> DVector4: ...
    @overload
    def __rsub__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rsub__(self, other: Number) -> DVector4: ...
    @overload
    def __mul__(self, other: DVector4) -> DVector4: ...
    @overload
    def __mul__(self, other: Number) -> DVector4: ...
    @overload
    def __rmul__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rmul__(self, other: Number) -> DVector4: ...
    def __matmul__(self, other: DVector4) -> float: ...
    def __rmatmul__(self, other: DVector4) -> float: ...
    @overload
    def __mod__(self, other: DVector4) -> DVector4: ...
    @overload
    def __mod__(self, other: Number) -> DVector4: ...
    @overload
    def __rmod__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rmod__(self, other: Number) -> DVector4: ...
    @overload
    def __pow__(self, other: DVector4) -> DVector4: ...
    @overload
    def __pow__(self, other: Number) -> DVector4: ...
    @overload
    def __rpow__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rpow__(self, other: Number) -> DVector4: ...
    @overload
    def __truediv__(self, other: DVector4) -> DVector4: ...
    @overload
    def __truediv__(self, other: Number) -> DVector4: ...
    @overload
    def __rtruediv__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> DVector4: ...
    def __neg__(self) -> DVector4: ...
    def __abs__(self) -> DVector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float: ...
    @property
    def w(self) -> float: ...
    @property
    def xx(self) -> DVector2: ...
    @property
    def xy(self) -> DVector2: ...
    @property
    def xz(self) -> DVector2: ...
    @property
    def xw(self) -> DVector2: ...
    @property
    def xo(self) -> DVector2: ...
    @property
    def xl(self) -> DVector2: ...
    @property
    def yx(self) -> DVector2: ...
    @property
    def yy(self) -> DVector2: ...
    @property
    def yz(self) -> DVector2: ...
    @property
    def yw(self) -> DVector2: ...
    @property
    def yo(self) -> DVector2: ...
    @property
    def yl(self) -> DVector2: ...
    @property
    def zx(self) -> DVector2: ...
    @property
    def zy(self) -> DVector2: ...
    @property
    def zz(self) -> DVector2: ...
    @property
    def zw(self) -> DVector2: ...
    @property
    def zo(self) -> DVector2: ...
    @property
    def zl(self) -> DVector2: ...
    @property
    def wx(self) -> DVector2: ...
    @property
    def wy(self) -> DVector2: ...
    @property
    def wz(self) -> DVector2: ...
    @property
    def ww(self) -> DVector2: ...
    @property
    def wo(self) -> DVector2: ...
    @property
    def wl(self) -> DVector2: ...
    @property
    def ox(self) -> DVector2: ...
    @property
    def oy(self) -> DVector2: ...
    @property
    def oz(self) -> DVector2: ...
    @property
    def ow(self) -> DVector2: ...
    @property
    def lx(self) -> DVector2: ...
    @property
    def ly(self) -> DVector2: ...
    @property
    def lz(self) -> DVector2: ...
    @property
    def lw(self) -> DVector2: ...
    @property
    def xxx(self) -> DVector3: ...
    @property
    def xxy(self) -> DVector3: ...
    @property
    def xxz(self) -> DVector3: ...
    @property
    def xxw(self) -> DVector3: ...
    @property
    def xxo(self) -> DVector3: ...
    @property
    def xxl(self) -> DVector3: ...
    @property
    def xyx(self) -> DVector3: ...
    @property
    def xyy(self) -> DVector3: ...
    @property
    def xyz(self) -> DVector3: ...
    @property
    def xyw(self) -> DVector3: ...
    @property
    def xyo(self) -> DVector3: ...
    @property
    def xyl(self) -> DVector3: ...
    @property
    def xzx(self) -> DVector3: ...
    @property
    def xzy(self) -> DVector3: ...
    @property
    def xzz(self) -> DVector3: ...
    @property
    def xzw(self) -> DVector3: ...
    @property
    def xzo(self) -> DVector3: ...
    @property
    def xzl(self) -> DVector3: ...
    @property
    def xwx(self) -> DVector3: ...
    @property
    def xwy(self) -> DVector3: ...
    @property
    def xwz(self) -> DVector3: ...
    @property
    def xww(self) -> DVector3: ...
    @property
    def xwo(self) -> DVector3: ...
    @property
    def xwl(self) -> DVector3: ...
    @property
    def xox(self) -> DVector3: ...
    @property
    def xoy(self) -> DVector3: ...
    @property
    def xoz(self) -> DVector3: ...
    @property
    def xow(self) -> DVector3: ...
    @property
    def xoo(self) -> DVector3: ...
    @property
    def xol(self) -> DVector3: ...
    @property
    def xlx(self) -> DVector3: ...
    @property
    def xly(self) -> DVector3: ...
    @property
    def xlz(self) -> DVector3: ...
    @property
    def xlw(self) -> DVector3: ...
    @property
    def xlo(self) -> DVector3: ...
    @property
    def xll(self) -> DVector3: ...
    @property
    def yxx(self) -> DVector3: ...
    @property
    def yxy(self) -> DVector3: ...
    @property
    def yxz(self) -> DVector3: ...
    @property
    def yxw(self) -> DVector3: ...
    @property
    def yxo(self) -> DVector3: ...
    @property
    def yxl(self) -> DVector3: ...
    @property
    def yyx(self) -> DVector3: ...
    @property
    def yyy(self) -> DVector3: ...
    @property
    def yyz(self) -> DVector3: ...
    @property
    def yyw(self) -> DVector3: ...
    @property
    def yyo(self) -> DVector3: ...
    @property
    def yyl(self) -> DVector3: ...
    @property
    def yzx(self) -> DVector3: ...
    @property
    def yzy(self) -> DVector3: ...
    @property
    def yzz(self) -> DVector3: ...
    @property
    def yzw(self) -> DVector3: ...
    @property
    def yzo(self) -> DVector3: ...
    @property
    def yzl(self) -> DVector3: ...
    @property
    def ywx(self) -> DVector3: ...
    @property
    def ywy(self) -> DVector3: ...
    @property
    def ywz(self) -> DVector3: ...
    @property
    def yww(self) -> DVector3: ...
    @property
    def ywo(self) -> DVector3: ...
    @property
    def ywl(self) -> DVector3: ...
    @property
    def yox(self) -> DVector3: ...
    @property
    def yoy(self) -> DVector3: ...
    @property
    def yoz(self) -> DVector3: ...
    @property
    def yow(self) -> DVector3: ...
    @property
    def yoo(self) -> DVector3: ...
    @property
    def yol(self) -> DVector3: ...
    @property
    def ylx(self) -> DVector3: ...
    @property
    def yly(self) -> DVector3: ...
    @property
    def ylz(self) -> DVector3: ...
    @property
    def ylw(self) -> DVector3: ...
    @property
    def ylo(self) -> DVector3: ...
    @property
    def yll(self) -> DVector3: ...
    @property
    def zxx(self) -> DVector3: ...
    @property
    def zxy(self) -> DVector3: ...
    @property
    def zxz(self) -> DVector3: ...
    @property
    def zxw(self) -> DVector3: ...
    @property
    def zxo(self) -> DVector3: ...
    @property
    def zxl(self) -> DVector3: ...
    @property
    def zyx(self) -> DVector3: ...
    @property
    def zyy(self) -> DVector3: ...
    @property
    def zyz(self) -> DVector3: ...
    @property
    def zyw(self) -> DVector3: ...
    @property
    def zyo(self) -> DVector3: ...
    @property
    def zyl(self) -> DVector3: ...
    @property
    def zzx(self) -> DVector3: ...
    @property
    def zzy(self) -> DVector3: ...
    @property
    def zzz(self) -> DVector3: ...
    @property
    def zzw(self) -> DVector3: ...
    @property
    def zzo(self) -> DVector3: ...
    @property
    def zzl(self) -> DVector3: ...
    @property
    def zwx(self) -> DVector3: ...
    @property
    def zwy(self) -> DVector3: ...
    @property
    def zwz(self) -> DVector3: ...
    @property
    def zww(self) -> DVector3: ...
    @property
    def zwo(self) -> DVector3: ...
    @property
    def zwl(self) -> DVector3: ...
    @property
    def zox(self) -> DVector3: ...
    @property
    def zoy(self) -> DVector3: ...
    @property
    def zoz(self) -> DVector3: ...
    @property
    def zow(self) -> DVector3: ...
    @property
    def zoo(self) -> DVector3: ...
    @property
    def zol(self) -> DVector3: ...
    @property
    def zlx(self) -> DVector3: ...
    @property
    def zly(self) -> DVector3: ...
    @property
    def zlz(self) -> DVector3: ...
    @property
    def zlw(self) -> DVector3: ...
    @property
    def zlo(self) -> DVector3: ...
    @property
    def zll(self) -> DVector3: ...
    @property
    def wxx(self) -> DVector3: ...
    @property
    def wxy(self) -> DVector3: ...
    @property
    def wxz(self) -> DVector3: ...
    @property
    def wxw(self) -> DVector3: ...
    @property
    def wxo(self) -> DVector3: ...
    @property
    def wxl(self) -> DVector3: ...
    @property
    def wyx(self) -> DVector3: ...
    @property
    def wyy(self) -> DVector3: ...
    @property
    def wyz(self) -> DVector3: ...
    @property
    def wyw(self) -> DVector3: ...
    @property
    def wyo(self) -> DVector3: ...
    @property
    def wyl(self) -> DVector3: ...
    @property
    def wzx(self) -> DVector3: ...
    @property
    def wzy(self) -> DVector3: ...
    @property
    def wzz(self) -> DVector3: ...
    @property
    def wzw(self) -> DVector3: ...
    @property
    def wzo(self) -> DVector3: ...
    @property
    def wzl(self) -> DVector3: ...
    @property
    def wwx(self) -> DVector3: ...
    @property
    def wwy(self) -> DVector3: ...
    @property
    def wwz(self) -> DVector3: ...
    @property
    def www(self) -> DVector3: ...
    @property
    def wwo(self) -> DVector3: ...
    @property
    def wwl(self) -> DVector3: ...
    @property
    def wox(self) -> DVector3: ...
    @property
    def woy(self) -> DVector3: ...
    @property
    def woz(self) -> DVector3: ...
    @property
    def wow(self) -> DVector3: ...
    @property
    def woo(self) -> DVector3: ...
    @property
    def wol(self) -> DVector3: ...
    @property
    def wlx(self) -> DVector3: ...
    @property
    def wly(self) -> DVector3: ...
    @property
    def wlz(self) -> DVector3: ...
    @property
    def wlw(self) -> DVector3: ...
    @property
    def wlo(self) -> DVector3: ...
    @property
    def wll(self) -> DVector3: ...
    @property
    def oxx(self) -> DVector3: ...
    @property
    def oxy(self) -> DVector3: ...
    @property
    def oxz(self) -> DVector3: ...
    @property
    def oxw(self) -> DVector3: ...
    @property
    def oxo(self) -> DVector3: ...
    @property
    def oxl(self) -> DVector3: ...
    @property
    def oyx(self) -> DVector3: ...
    @property
    def oyy(self) -> DVector3: ...
    @property
    def oyz(self) -> DVector3: ...
    @property
    def oyw(self) -> DVector3: ...
    @property
    def oyo(self) -> DVector3: ...
    @property
    def oyl(self) -> DVector3: ...
    @property
    def ozx(self) -> DVector3: ...
    @property
    def ozy(self) -> DVector3: ...
    @property
    def ozz(self) -> DVector3: ...
    @property
    def ozw(self) -> DVector3: ...
    @property
    def ozo(self) -> DVector3: ...
    @property
    def ozl(self) -> DVector3: ...
    @property
    def owx(self) -> DVector3: ...
    @property
    def owy(self) -> DVector3: ...
    @property
    def owz(self) -> DVector3: ...
    @property
    def oww(self) -> DVector3: ...
    @property
    def owo(self) -> DVector3: ...
    @property
    def owl(self) -> DVector3: ...
    @property
    def oox(self) -> DVector3: ...
    @property
    def ooy(self) -> DVector3: ...
    @property
    def ooz(self) -> DVector3: ...
    @property
    def oow(self) -> DVector3: ...
    @property
    def olx(self) -> DVector3: ...
    @property
    def oly(self) -> DVector3: ...
    @property
    def olz(self) -> DVector3: ...
    @property
    def olw(self) -> DVector3: ...
    @property
    def lxx(self) -> DVector3: ...
    @property
    def lxy(self) -> DVector3: ...
    @property
    def lxz(self) -> DVector3: ...
    @property
    def lxw(self) -> DVector3: ...
    @property
    def lxo(self) -> DVector3: ...
    @property
    def lxl(self) -> DVector3: ...
    @property
    def lyx(self) -> DVector3: ...
    @property
    def lyy(self) -> DVector3: ...
    @property
    def lyz(self) -> DVector3: ...
    @property
    def lyw(self) -> DVector3: ...
    @property
    def lyo(self) -> DVector3: ...
    @property
    def lyl(self) -> DVector3: ...
    @property
    def lzx(self) -> DVector3: ...
    @property
    def lzy(self) -> DVector3: ...
    @property
    def lzz(self) -> DVector3: ...
    @property
    def lzw(self) -> DVector3: ...
    @property
    def lzo(self) -> DVector3: ...
    @property
    def lzl(self) -> DVector3: ...
    @property
    def lwx(self) -> DVector3: ...
    @property
    def lwy(self) -> DVector3: ...
    @property
    def lwz(self) -> DVector3: ...
    @property
    def lww(self) -> DVector3: ...
    @property
    def lwo(self) -> DVector3: ...
    @property
    def lwl(self) -> DVector3: ...
    @property
    def lox(self) -> DVector3: ...
    @property
    def loy(self) -> DVector3: ...
    @property
    def loz(self) -> DVector3: ...
    @property
    def low(self) -> DVector3: ...
    @property
    def llx(self) -> DVector3: ...
    @property
    def lly(self) -> DVector3: ...
    @property
    def llz(self) -> DVector3: ...
    @property
    def llw(self) -> DVector3: ...
    @property
    def xxxx(self) -> DVector4: ...
    @property
    def xxxy(self) -> DVector4: ...
    @property
    def xxxz(self) -> DVector4: ...
    @property
    def xxxw(self) -> DVector4: ...
    @property
    def xxxo(self) -> DVector4: ...
    @property
    def xxxl(self) -> DVector4: ...
    @property
    def xxyx(self) -> DVector4: ...
    @property
    def xxyy(self) -> DVector4: ...
    @property
    def xxyz(self) -> DVector4: ...
    @property
    def xxyw(self) -> DVector4: ...
    @property
    def xxyo(self) -> DVector4: ...
    @property
    def xxyl(self) -> DVector4: ...
    @property
    def xxzx(self) -> DVector4: ...
    @property
    def xxzy(self) -> DVector4: ...
    @property
    def xxzz(self) -> DVector4: ...
    @property
    def xxzw(self) -> DVector4: ...
    @property
    def xxzo(self) -> DVector4: ...
    @property
    def xxzl(self) -> DVector4: ...
    @property
    def xxwx(self) -> DVector4: ...
    @property
    def xxwy(self) -> DVector4: ...
    @property
    def xxwz(self) -> DVector4: ...
    @property
    def xxww(self) -> DVector4: ...
    @property
    def xxwo(self) -> DVector4: ...
    @property
    def xxwl(self) -> DVector4: ...
    @property
    def xxox(self) -> DVector4: ...
    @property
    def xxoy(self) -> DVector4: ...
    @property
    def xxoz(self) -> DVector4: ...
    @property
    def xxow(self) -> DVector4: ...
    @property
    def xxoo(self) -> DVector4: ...
    @property
    def xxol(self) -> DVector4: ...
    @property
    def xxlx(self) -> DVector4: ...
    @property
    def xxly(self) -> DVector4: ...
    @property
    def xxlz(self) -> DVector4: ...
    @property
    def xxlw(self) -> DVector4: ...
    @property
    def xxlo(self) -> DVector4: ...
    @property
    def xxll(self) -> DVector4: ...
    @property
    def xyxx(self) -> DVector4: ...
    @property
    def xyxy(self) -> DVector4: ...
    @property
    def xyxz(self) -> DVector4: ...
    @property
    def xyxw(self) -> DVector4: ...
    @property
    def xyxo(self) -> DVector4: ...
    @property
    def xyxl(self) -> DVector4: ...
    @property
    def xyyx(self) -> DVector4: ...
    @property
    def xyyy(self) -> DVector4: ...
    @property
    def xyyz(self) -> DVector4: ...
    @property
    def xyyw(self) -> DVector4: ...
    @property
    def xyyo(self) -> DVector4: ...
    @property
    def xyyl(self) -> DVector4: ...
    @property
    def xyzx(self) -> DVector4: ...
    @property
    def xyzy(self) -> DVector4: ...
    @property
    def xyzz(self) -> DVector4: ...
    @property
    def xyzw(self) -> DVector4: ...
    @property
    def xyzo(self) -> DVector4: ...
    @property
    def xyzl(self) -> DVector4: ...
    @property
    def xywx(self) -> DVector4: ...
    @property
    def xywy(self) -> DVector4: ...
    @property
    def xywz(self) -> DVector4: ...
    @property
    def xyww(self) -> DVector4: ...
    @property
    def xywo(self) -> DVector4: ...
    @property
    def xywl(self) -> DVector4: ...
    @property
    def xyox(self) -> DVector4: ...
    @property
    def xyoy(self) -> DVector4: ...
    @property
    def xyoz(self) -> DVector4: ...
    @property
    def xyow(self) -> DVector4: ...
    @property
    def xyoo(self) -> DVector4: ...
    @property
    def xyol(self) -> DVector4: ...
    @property
    def xylx(self) -> DVector4: ...
    @property
    def xyly(self) -> DVector4: ...
    @property
    def xylz(self) -> DVector4: ...
    @property
    def xylw(self) -> DVector4: ...
    @property
    def xylo(self) -> DVector4: ...
    @property
    def xyll(self) -> DVector4: ...
    @property
    def xzxx(self) -> DVector4: ...
    @property
    def xzxy(self) -> DVector4: ...
    @property
    def xzxz(self) -> DVector4: ...
    @property
    def xzxw(self) -> DVector4: ...
    @property
    def xzxo(self) -> DVector4: ...
    @property
    def xzxl(self) -> DVector4: ...
    @property
    def xzyx(self) -> DVector4: ...
    @property
    def xzyy(self) -> DVector4: ...
    @property
    def xzyz(self) -> DVector4: ...
    @property
    def xzyw(self) -> DVector4: ...
    @property
    def xzyo(self) -> DVector4: ...
    @property
    def xzyl(self) -> DVector4: ...
    @property
    def xzzx(self) -> DVector4: ...
    @property
    def xzzy(self) -> DVector4: ...
    @property
    def xzzz(self) -> DVector4: ...
    @property
    def xzzw(self) -> DVector4: ...
    @property
    def xzzo(self) -> DVector4: ...
    @property
    def xzzl(self) -> DVector4: ...
    @property
    def xzwx(self) -> DVector4: ...
    @property
    def xzwy(self) -> DVector4: ...
    @property
    def xzwz(self) -> DVector4: ...
    @property
    def xzww(self) -> DVector4: ...
    @property
    def xzwo(self) -> DVector4: ...
    @property
    def xzwl(self) -> DVector4: ...
    @property
    def xzox(self) -> DVector4: ...
    @property
    def xzoy(self) -> DVector4: ...
    @property
    def xzoz(self) -> DVector4: ...
    @property
    def xzow(self) -> DVector4: ...
    @property
    def xzoo(self) -> DVector4: ...
    @property
    def xzol(self) -> DVector4: ...
    @property
    def xzlx(self) -> DVector4: ...
    @property
    def xzly(self) -> DVector4: ...
    @property
    def xzlz(self) -> DVector4: ...
    @property
    def xzlw(self) -> DVector4: ...
    @property
    def xzlo(self) -> DVector4: ...
    @property
    def xzll(self) -> DVector4: ...
    @property
    def xwxx(self) -> DVector4: ...
    @property
    def xwxy(self) -> DVector4: ...
    @property
    def xwxz(self) -> DVector4: ...
    @property
    def xwxw(self) -> DVector4: ...
    @property
    def xwxo(self) -> DVector4: ...
    @property
    def xwxl(self) -> DVector4: ...
    @property
    def xwyx(self) -> DVector4: ...
    @property
    def xwyy(self) -> DVector4: ...
    @property
    def xwyz(self) -> DVector4: ...
    @property
    def xwyw(self) -> DVector4: ...
    @property
    def xwyo(self) -> DVector4: ...
    @property
    def xwyl(self) -> DVector4: ...
    @property
    def xwzx(self) -> DVector4: ...
    @property
    def xwzy(self) -> DVector4: ...
    @property
    def xwzz(self) -> DVector4: ...
    @property
    def xwzw(self) -> DVector4: ...
    @property
    def xwzo(self) -> DVector4: ...
    @property
    def xwzl(self) -> DVector4: ...
    @property
    def xwwx(self) -> DVector4: ...
    @property
    def xwwy(self) -> DVector4: ...
    @property
    def xwwz(self) -> DVector4: ...
    @property
    def xwww(self) -> DVector4: ...
    @property
    def xwwo(self) -> DVector4: ...
    @property
    def xwwl(self) -> DVector4: ...
    @property
    def xwox(self) -> DVector4: ...
    @property
    def xwoy(self) -> DVector4: ...
    @property
    def xwoz(self) -> DVector4: ...
    @property
    def xwow(self) -> DVector4: ...
    @property
    def xwoo(self) -> DVector4: ...
    @property
    def xwol(self) -> DVector4: ...
    @property
    def xwlx(self) -> DVector4: ...
    @property
    def xwly(self) -> DVector4: ...
    @property
    def xwlz(self) -> DVector4: ...
    @property
    def xwlw(self) -> DVector4: ...
    @property
    def xwlo(self) -> DVector4: ...
    @property
    def xwll(self) -> DVector4: ...
    @property
    def xoxx(self) -> DVector4: ...
    @property
    def xoxy(self) -> DVector4: ...
    @property
    def xoxz(self) -> DVector4: ...
    @property
    def xoxw(self) -> DVector4: ...
    @property
    def xoxo(self) -> DVector4: ...
    @property
    def xoxl(self) -> DVector4: ...
    @property
    def xoyx(self) -> DVector4: ...
    @property
    def xoyy(self) -> DVector4: ...
    @property
    def xoyz(self) -> DVector4: ...
    @property
    def xoyw(self) -> DVector4: ...
    @property
    def xoyo(self) -> DVector4: ...
    @property
    def xoyl(self) -> DVector4: ...
    @property
    def xozx(self) -> DVector4: ...
    @property
    def xozy(self) -> DVector4: ...
    @property
    def xozz(self) -> DVector4: ...
    @property
    def xozw(self) -> DVector4: ...
    @property
    def xozo(self) -> DVector4: ...
    @property
    def xozl(self) -> DVector4: ...
    @property
    def xowx(self) -> DVector4: ...
    @property
    def xowy(self) -> DVector4: ...
    @property
    def xowz(self) -> DVector4: ...
    @property
    def xoww(self) -> DVector4: ...
    @property
    def xowo(self) -> DVector4: ...
    @property
    def xowl(self) -> DVector4: ...
    @property
    def xoox(self) -> DVector4: ...
    @property
    def xooy(self) -> DVector4: ...
    @property
    def xooz(self) -> DVector4: ...
    @property
    def xoow(self) -> DVector4: ...
    @property
    def xooo(self) -> DVector4: ...
    @property
    def xool(self) -> DVector4: ...
    @property
    def xolx(self) -> DVector4: ...
    @property
    def xoly(self) -> DVector4: ...
    @property
    def xolz(self) -> DVector4: ...
    @property
    def xolw(self) -> DVector4: ...
    @property
    def xolo(self) -> DVector4: ...
    @property
    def xoll(self) -> DVector4: ...
    @property
    def xlxx(self) -> DVector4: ...
    @property
    def xlxy(self) -> DVector4: ...
    @property
    def xlxz(self) -> DVector4: ...
    @property
    def xlxw(self) -> DVector4: ...
    @property
    def xlxo(self) -> DVector4: ...
    @property
    def xlxl(self) -> DVector4: ...
    @property
    def xlyx(self) -> DVector4: ...
    @property
    def xlyy(self) -> DVector4: ...
    @property
    def xlyz(self) -> DVector4: ...
    @property
    def xlyw(self) -> DVector4: ...
    @property
    def xlyo(self) -> DVector4: ...
    @property
    def xlyl(self) -> DVector4: ...
    @property
    def xlzx(self) -> DVector4: ...
    @property
    def xlzy(self) -> DVector4: ...
    @property
    def xlzz(self) -> DVector4: ...
    @property
    def xlzw(self) -> DVector4: ...
    @property
    def xlzo(self) -> DVector4: ...
    @property
    def xlzl(self) -> DVector4: ...
    @property
    def xlwx(self) -> DVector4: ...
    @property
    def xlwy(self) -> DVector4: ...
    @property
    def xlwz(self) -> DVector4: ...
    @property
    def xlww(self) -> DVector4: ...
    @property
    def xlwo(self) -> DVector4: ...
    @property
    def xlwl(self) -> DVector4: ...
    @property
    def xlox(self) -> DVector4: ...
    @property
    def xloy(self) -> DVector4: ...
    @property
    def xloz(self) -> DVector4: ...
    @property
    def xlow(self) -> DVector4: ...
    @property
    def xloo(self) -> DVector4: ...
    @property
    def xlol(self) -> DVector4: ...
    @property
    def xllx(self) -> DVector4: ...
    @property
    def xlly(self) -> DVector4: ...
    @property
    def xllz(self) -> DVector4: ...
    @property
    def xllw(self) -> DVector4: ...
    @property
    def xllo(self) -> DVector4: ...
    @property
    def xlll(self) -> DVector4: ...
    @property
    def yxxx(self) -> DVector4: ...
    @property
    def yxxy(self) -> DVector4: ...
    @property
    def yxxz(self) -> DVector4: ...
    @property
    def yxxw(self) -> DVector4: ...
    @property
    def yxxo(self) -> DVector4: ...
    @property
    def yxxl(self) -> DVector4: ...
    @property
    def yxyx(self) -> DVector4: ...
    @property
    def yxyy(self) -> DVector4: ...
    @property
    def yxyz(self) -> DVector4: ...
    @property
    def yxyw(self) -> DVector4: ...
    @property
    def yxyo(self) -> DVector4: ...
    @property
    def yxyl(self) -> DVector4: ...
    @property
    def yxzx(self) -> DVector4: ...
    @property
    def yxzy(self) -> DVector4: ...
    @property
    def yxzz(self) -> DVector4: ...
    @property
    def yxzw(self) -> DVector4: ...
    @property
    def yxzo(self) -> DVector4: ...
    @property
    def yxzl(self) -> DVector4: ...
    @property
    def yxwx(self) -> DVector4: ...
    @property
    def yxwy(self) -> DVector4: ...
    @property
    def yxwz(self) -> DVector4: ...
    @property
    def yxww(self) -> DVector4: ...
    @property
    def yxwo(self) -> DVector4: ...
    @property
    def yxwl(self) -> DVector4: ...
    @property
    def yxox(self) -> DVector4: ...
    @property
    def yxoy(self) -> DVector4: ...
    @property
    def yxoz(self) -> DVector4: ...
    @property
    def yxow(self) -> DVector4: ...
    @property
    def yxoo(self) -> DVector4: ...
    @property
    def yxol(self) -> DVector4: ...
    @property
    def yxlx(self) -> DVector4: ...
    @property
    def yxly(self) -> DVector4: ...
    @property
    def yxlz(self) -> DVector4: ...
    @property
    def yxlw(self) -> DVector4: ...
    @property
    def yxlo(self) -> DVector4: ...
    @property
    def yxll(self) -> DVector4: ...
    @property
    def yyxx(self) -> DVector4: ...
    @property
    def yyxy(self) -> DVector4: ...
    @property
    def yyxz(self) -> DVector4: ...
    @property
    def yyxw(self) -> DVector4: ...
    @property
    def yyxo(self) -> DVector4: ...
    @property
    def yyxl(self) -> DVector4: ...
    @property
    def yyyx(self) -> DVector4: ...
    @property
    def yyyy(self) -> DVector4: ...
    @property
    def yyyz(self) -> DVector4: ...
    @property
    def yyyw(self) -> DVector4: ...
    @property
    def yyyo(self) -> DVector4: ...
    @property
    def yyyl(self) -> DVector4: ...
    @property
    def yyzx(self) -> DVector4: ...
    @property
    def yyzy(self) -> DVector4: ...
    @property
    def yyzz(self) -> DVector4: ...
    @property
    def yyzw(self) -> DVector4: ...
    @property
    def yyzo(self) -> DVector4: ...
    @property
    def yyzl(self) -> DVector4: ...
    @property
    def yywx(self) -> DVector4: ...
    @property
    def yywy(self) -> DVector4: ...
    @property
    def yywz(self) -> DVector4: ...
    @property
    def yyww(self) -> DVector4: ...
    @property
    def yywo(self) -> DVector4: ...
    @property
    def yywl(self) -> DVector4: ...
    @property
    def yyox(self) -> DVector4: ...
    @property
    def yyoy(self) -> DVector4: ...
    @property
    def yyoz(self) -> DVector4: ...
    @property
    def yyow(self) -> DVector4: ...
    @property
    def yyoo(self) -> DVector4: ...
    @property
    def yyol(self) -> DVector4: ...
    @property
    def yylx(self) -> DVector4: ...
    @property
    def yyly(self) -> DVector4: ...
    @property
    def yylz(self) -> DVector4: ...
    @property
    def yylw(self) -> DVector4: ...
    @property
    def yylo(self) -> DVector4: ...
    @property
    def yyll(self) -> DVector4: ...
    @property
    def yzxx(self) -> DVector4: ...
    @property
    def yzxy(self) -> DVector4: ...
    @property
    def yzxz(self) -> DVector4: ...
    @property
    def yzxw(self) -> DVector4: ...
    @property
    def yzxo(self) -> DVector4: ...
    @property
    def yzxl(self) -> DVector4: ...
    @property
    def yzyx(self) -> DVector4: ...
    @property
    def yzyy(self) -> DVector4: ...
    @property
    def yzyz(self) -> DVector4: ...
    @property
    def yzyw(self) -> DVector4: ...
    @property
    def yzyo(self) -> DVector4: ...
    @property
    def yzyl(self) -> DVector4: ...
    @property
    def yzzx(self) -> DVector4: ...
    @property
    def yzzy(self) -> DVector4: ...
    @property
    def yzzz(self) -> DVector4: ...
    @property
    def yzzw(self) -> DVector4: ...
    @property
    def yzzo(self) -> DVector4: ...
    @property
    def yzzl(self) -> DVector4: ...
    @property
    def yzwx(self) -> DVector4: ...
    @property
    def yzwy(self) -> DVector4: ...
    @property
    def yzwz(self) -> DVector4: ...
    @property
    def yzww(self) -> DVector4: ...
    @property
    def yzwo(self) -> DVector4: ...
    @property
    def yzwl(self) -> DVector4: ...
    @property
    def yzox(self) -> DVector4: ...
    @property
    def yzoy(self) -> DVector4: ...
    @property
    def yzoz(self) -> DVector4: ...
    @property
    def yzow(self) -> DVector4: ...
    @property
    def yzoo(self) -> DVector4: ...
    @property
    def yzol(self) -> DVector4: ...
    @property
    def yzlx(self) -> DVector4: ...
    @property
    def yzly(self) -> DVector4: ...
    @property
    def yzlz(self) -> DVector4: ...
    @property
    def yzlw(self) -> DVector4: ...
    @property
    def yzlo(self) -> DVector4: ...
    @property
    def yzll(self) -> DVector4: ...
    @property
    def ywxx(self) -> DVector4: ...
    @property
    def ywxy(self) -> DVector4: ...
    @property
    def ywxz(self) -> DVector4: ...
    @property
    def ywxw(self) -> DVector4: ...
    @property
    def ywxo(self) -> DVector4: ...
    @property
    def ywxl(self) -> DVector4: ...
    @property
    def ywyx(self) -> DVector4: ...
    @property
    def ywyy(self) -> DVector4: ...
    @property
    def ywyz(self) -> DVector4: ...
    @property
    def ywyw(self) -> DVector4: ...
    @property
    def ywyo(self) -> DVector4: ...
    @property
    def ywyl(self) -> DVector4: ...
    @property
    def ywzx(self) -> DVector4: ...
    @property
    def ywzy(self) -> DVector4: ...
    @property
    def ywzz(self) -> DVector4: ...
    @property
    def ywzw(self) -> DVector4: ...
    @property
    def ywzo(self) -> DVector4: ...
    @property
    def ywzl(self) -> DVector4: ...
    @property
    def ywwx(self) -> DVector4: ...
    @property
    def ywwy(self) -> DVector4: ...
    @property
    def ywwz(self) -> DVector4: ...
    @property
    def ywww(self) -> DVector4: ...
    @property
    def ywwo(self) -> DVector4: ...
    @property
    def ywwl(self) -> DVector4: ...
    @property
    def ywox(self) -> DVector4: ...
    @property
    def ywoy(self) -> DVector4: ...
    @property
    def ywoz(self) -> DVector4: ...
    @property
    def ywow(self) -> DVector4: ...
    @property
    def ywoo(self) -> DVector4: ...
    @property
    def ywol(self) -> DVector4: ...
    @property
    def ywlx(self) -> DVector4: ...
    @property
    def ywly(self) -> DVector4: ...
    @property
    def ywlz(self) -> DVector4: ...
    @property
    def ywlw(self) -> DVector4: ...
    @property
    def ywlo(self) -> DVector4: ...
    @property
    def ywll(self) -> DVector4: ...
    @property
    def yoxx(self) -> DVector4: ...
    @property
    def yoxy(self) -> DVector4: ...
    @property
    def yoxz(self) -> DVector4: ...
    @property
    def yoxw(self) -> DVector4: ...
    @property
    def yoxo(self) -> DVector4: ...
    @property
    def yoxl(self) -> DVector4: ...
    @property
    def yoyx(self) -> DVector4: ...
    @property
    def yoyy(self) -> DVector4: ...
    @property
    def yoyz(self) -> DVector4: ...
    @property
    def yoyw(self) -> DVector4: ...
    @property
    def yoyo(self) -> DVector4: ...
    @property
    def yoyl(self) -> DVector4: ...
    @property
    def yozx(self) -> DVector4: ...
    @property
    def yozy(self) -> DVector4: ...
    @property
    def yozz(self) -> DVector4: ...
    @property
    def yozw(self) -> DVector4: ...
    @property
    def yozo(self) -> DVector4: ...
    @property
    def yozl(self) -> DVector4: ...
    @property
    def yowx(self) -> DVector4: ...
    @property
    def yowy(self) -> DVector4: ...
    @property
    def yowz(self) -> DVector4: ...
    @property
    def yoww(self) -> DVector4: ...
    @property
    def yowo(self) -> DVector4: ...
    @property
    def yowl(self) -> DVector4: ...
    @property
    def yoox(self) -> DVector4: ...
    @property
    def yooy(self) -> DVector4: ...
    @property
    def yooz(self) -> DVector4: ...
    @property
    def yoow(self) -> DVector4: ...
    @property
    def yooo(self) -> DVector4: ...
    @property
    def yool(self) -> DVector4: ...
    @property
    def yolx(self) -> DVector4: ...
    @property
    def yoly(self) -> DVector4: ...
    @property
    def yolz(self) -> DVector4: ...
    @property
    def yolw(self) -> DVector4: ...
    @property
    def yolo(self) -> DVector4: ...
    @property
    def yoll(self) -> DVector4: ...
    @property
    def ylxx(self) -> DVector4: ...
    @property
    def ylxy(self) -> DVector4: ...
    @property
    def ylxz(self) -> DVector4: ...
    @property
    def ylxw(self) -> DVector4: ...
    @property
    def ylxo(self) -> DVector4: ...
    @property
    def ylxl(self) -> DVector4: ...
    @property
    def ylyx(self) -> DVector4: ...
    @property
    def ylyy(self) -> DVector4: ...
    @property
    def ylyz(self) -> DVector4: ...
    @property
    def ylyw(self) -> DVector4: ...
    @property
    def ylyo(self) -> DVector4: ...
    @property
    def ylyl(self) -> DVector4: ...
    @property
    def ylzx(self) -> DVector4: ...
    @property
    def ylzy(self) -> DVector4: ...
    @property
    def ylzz(self) -> DVector4: ...
    @property
    def ylzw(self) -> DVector4: ...
    @property
    def ylzo(self) -> DVector4: ...
    @property
    def ylzl(self) -> DVector4: ...
    @property
    def ylwx(self) -> DVector4: ...
    @property
    def ylwy(self) -> DVector4: ...
    @property
    def ylwz(self) -> DVector4: ...
    @property
    def ylww(self) -> DVector4: ...
    @property
    def ylwo(self) -> DVector4: ...
    @property
    def ylwl(self) -> DVector4: ...
    @property
    def ylox(self) -> DVector4: ...
    @property
    def yloy(self) -> DVector4: ...
    @property
    def yloz(self) -> DVector4: ...
    @property
    def ylow(self) -> DVector4: ...
    @property
    def yloo(self) -> DVector4: ...
    @property
    def ylol(self) -> DVector4: ...
    @property
    def yllx(self) -> DVector4: ...
    @property
    def ylly(self) -> DVector4: ...
    @property
    def yllz(self) -> DVector4: ...
    @property
    def yllw(self) -> DVector4: ...
    @property
    def yllo(self) -> DVector4: ...
    @property
    def ylll(self) -> DVector4: ...
    @property
    def zxxx(self) -> DVector4: ...
    @property
    def zxxy(self) -> DVector4: ...
    @property
    def zxxz(self) -> DVector4: ...
    @property
    def zxxw(self) -> DVector4: ...
    @property
    def zxxo(self) -> DVector4: ...
    @property
    def zxxl(self) -> DVector4: ...
    @property
    def zxyx(self) -> DVector4: ...
    @property
    def zxyy(self) -> DVector4: ...
    @property
    def zxyz(self) -> DVector4: ...
    @property
    def zxyw(self) -> DVector4: ...
    @property
    def zxyo(self) -> DVector4: ...
    @property
    def zxyl(self) -> DVector4: ...
    @property
    def zxzx(self) -> DVector4: ...
    @property
    def zxzy(self) -> DVector4: ...
    @property
    def zxzz(self) -> DVector4: ...
    @property
    def zxzw(self) -> DVector4: ...
    @property
    def zxzo(self) -> DVector4: ...
    @property
    def zxzl(self) -> DVector4: ...
    @property
    def zxwx(self) -> DVector4: ...
    @property
    def zxwy(self) -> DVector4: ...
    @property
    def zxwz(self) -> DVector4: ...
    @property
    def zxww(self) -> DVector4: ...
    @property
    def zxwo(self) -> DVector4: ...
    @property
    def zxwl(self) -> DVector4: ...
    @property
    def zxox(self) -> DVector4: ...
    @property
    def zxoy(self) -> DVector4: ...
    @property
    def zxoz(self) -> DVector4: ...
    @property
    def zxow(self) -> DVector4: ...
    @property
    def zxoo(self) -> DVector4: ...
    @property
    def zxol(self) -> DVector4: ...
    @property
    def zxlx(self) -> DVector4: ...
    @property
    def zxly(self) -> DVector4: ...
    @property
    def zxlz(self) -> DVector4: ...
    @property
    def zxlw(self) -> DVector4: ...
    @property
    def zxlo(self) -> DVector4: ...
    @property
    def zxll(self) -> DVector4: ...
    @property
    def zyxx(self) -> DVector4: ...
    @property
    def zyxy(self) -> DVector4: ...
    @property
    def zyxz(self) -> DVector4: ...
    @property
    def zyxw(self) -> DVector4: ...
    @property
    def zyxo(self) -> DVector4: ...
    @property
    def zyxl(self) -> DVector4: ...
    @property
    def zyyx(self) -> DVector4: ...
    @property
    def zyyy(self) -> DVector4: ...
    @property
    def zyyz(self) -> DVector4: ...
    @property
    def zyyw(self) -> DVector4: ...
    @property
    def zyyo(self) -> DVector4: ...
    @property
    def zyyl(self) -> DVector4: ...
    @property
    def zyzx(self) -> DVector4: ...
    @property
    def zyzy(self) -> DVector4: ...
    @property
    def zyzz(self) -> DVector4: ...
    @property
    def zyzw(self) -> DVector4: ...
    @property
    def zyzo(self) -> DVector4: ...
    @property
    def zyzl(self) -> DVector4: ...
    @property
    def zywx(self) -> DVector4: ...
    @property
    def zywy(self) -> DVector4: ...
    @property
    def zywz(self) -> DVector4: ...
    @property
    def zyww(self) -> DVector4: ...
    @property
    def zywo(self) -> DVector4: ...
    @property
    def zywl(self) -> DVector4: ...
    @property
    def zyox(self) -> DVector4: ...
    @property
    def zyoy(self) -> DVector4: ...
    @property
    def zyoz(self) -> DVector4: ...
    @property
    def zyow(self) -> DVector4: ...
    @property
    def zyoo(self) -> DVector4: ...
    @property
    def zyol(self) -> DVector4: ...
    @property
    def zylx(self) -> DVector4: ...
    @property
    def zyly(self) -> DVector4: ...
    @property
    def zylz(self) -> DVector4: ...
    @property
    def zylw(self) -> DVector4: ...
    @property
    def zylo(self) -> DVector4: ...
    @property
    def zyll(self) -> DVector4: ...
    @property
    def zzxx(self) -> DVector4: ...
    @property
    def zzxy(self) -> DVector4: ...
    @property
    def zzxz(self) -> DVector4: ...
    @property
    def zzxw(self) -> DVector4: ...
    @property
    def zzxo(self) -> DVector4: ...
    @property
    def zzxl(self) -> DVector4: ...
    @property
    def zzyx(self) -> DVector4: ...
    @property
    def zzyy(self) -> DVector4: ...
    @property
    def zzyz(self) -> DVector4: ...
    @property
    def zzyw(self) -> DVector4: ...
    @property
    def zzyo(self) -> DVector4: ...
    @property
    def zzyl(self) -> DVector4: ...
    @property
    def zzzx(self) -> DVector4: ...
    @property
    def zzzy(self) -> DVector4: ...
    @property
    def zzzz(self) -> DVector4: ...
    @property
    def zzzw(self) -> DVector4: ...
    @property
    def zzzo(self) -> DVector4: ...
    @property
    def zzzl(self) -> DVector4: ...
    @property
    def zzwx(self) -> DVector4: ...
    @property
    def zzwy(self) -> DVector4: ...
    @property
    def zzwz(self) -> DVector4: ...
    @property
    def zzww(self) -> DVector4: ...
    @property
    def zzwo(self) -> DVector4: ...
    @property
    def zzwl(self) -> DVector4: ...
    @property
    def zzox(self) -> DVector4: ...
    @property
    def zzoy(self) -> DVector4: ...
    @property
    def zzoz(self) -> DVector4: ...
    @property
    def zzow(self) -> DVector4: ...
    @property
    def zzoo(self) -> DVector4: ...
    @property
    def zzol(self) -> DVector4: ...
    @property
    def zzlx(self) -> DVector4: ...
    @property
    def zzly(self) -> DVector4: ...
    @property
    def zzlz(self) -> DVector4: ...
    @property
    def zzlw(self) -> DVector4: ...
    @property
    def zzlo(self) -> DVector4: ...
    @property
    def zzll(self) -> DVector4: ...
    @property
    def zwxx(self) -> DVector4: ...
    @property
    def zwxy(self) -> DVector4: ...
    @property
    def zwxz(self) -> DVector4: ...
    @property
    def zwxw(self) -> DVector4: ...
    @property
    def zwxo(self) -> DVector4: ...
    @property
    def zwxl(self) -> DVector4: ...
    @property
    def zwyx(self) -> DVector4: ...
    @property
    def zwyy(self) -> DVector4: ...
    @property
    def zwyz(self) -> DVector4: ...
    @property
    def zwyw(self) -> DVector4: ...
    @property
    def zwyo(self) -> DVector4: ...
    @property
    def zwyl(self) -> DVector4: ...
    @property
    def zwzx(self) -> DVector4: ...
    @property
    def zwzy(self) -> DVector4: ...
    @property
    def zwzz(self) -> DVector4: ...
    @property
    def zwzw(self) -> DVector4: ...
    @property
    def zwzo(self) -> DVector4: ...
    @property
    def zwzl(self) -> DVector4: ...
    @property
    def zwwx(self) -> DVector4: ...
    @property
    def zwwy(self) -> DVector4: ...
    @property
    def zwwz(self) -> DVector4: ...
    @property
    def zwww(self) -> DVector4: ...
    @property
    def zwwo(self) -> DVector4: ...
    @property
    def zwwl(self) -> DVector4: ...
    @property
    def zwox(self) -> DVector4: ...
    @property
    def zwoy(self) -> DVector4: ...
    @property
    def zwoz(self) -> DVector4: ...
    @property
    def zwow(self) -> DVector4: ...
    @property
    def zwoo(self) -> DVector4: ...
    @property
    def zwol(self) -> DVector4: ...
    @property
    def zwlx(self) -> DVector4: ...
    @property
    def zwly(self) -> DVector4: ...
    @property
    def zwlz(self) -> DVector4: ...
    @property
    def zwlw(self) -> DVector4: ...
    @property
    def zwlo(self) -> DVector4: ...
    @property
    def zwll(self) -> DVector4: ...
    @property
    def zoxx(self) -> DVector4: ...
    @property
    def zoxy(self) -> DVector4: ...
    @property
    def zoxz(self) -> DVector4: ...
    @property
    def zoxw(self) -> DVector4: ...
    @property
    def zoxo(self) -> DVector4: ...
    @property
    def zoxl(self) -> DVector4: ...
    @property
    def zoyx(self) -> DVector4: ...
    @property
    def zoyy(self) -> DVector4: ...
    @property
    def zoyz(self) -> DVector4: ...
    @property
    def zoyw(self) -> DVector4: ...
    @property
    def zoyo(self) -> DVector4: ...
    @property
    def zoyl(self) -> DVector4: ...
    @property
    def zozx(self) -> DVector4: ...
    @property
    def zozy(self) -> DVector4: ...
    @property
    def zozz(self) -> DVector4: ...
    @property
    def zozw(self) -> DVector4: ...
    @property
    def zozo(self) -> DVector4: ...
    @property
    def zozl(self) -> DVector4: ...
    @property
    def zowx(self) -> DVector4: ...
    @property
    def zowy(self) -> DVector4: ...
    @property
    def zowz(self) -> DVector4: ...
    @property
    def zoww(self) -> DVector4: ...
    @property
    def zowo(self) -> DVector4: ...
    @property
    def zowl(self) -> DVector4: ...
    @property
    def zoox(self) -> DVector4: ...
    @property
    def zooy(self) -> DVector4: ...
    @property
    def zooz(self) -> DVector4: ...
    @property
    def zoow(self) -> DVector4: ...
    @property
    def zooo(self) -> DVector4: ...
    @property
    def zool(self) -> DVector4: ...
    @property
    def zolx(self) -> DVector4: ...
    @property
    def zoly(self) -> DVector4: ...
    @property
    def zolz(self) -> DVector4: ...
    @property
    def zolw(self) -> DVector4: ...
    @property
    def zolo(self) -> DVector4: ...
    @property
    def zoll(self) -> DVector4: ...
    @property
    def zlxx(self) -> DVector4: ...
    @property
    def zlxy(self) -> DVector4: ...
    @property
    def zlxz(self) -> DVector4: ...
    @property
    def zlxw(self) -> DVector4: ...
    @property
    def zlxo(self) -> DVector4: ...
    @property
    def zlxl(self) -> DVector4: ...
    @property
    def zlyx(self) -> DVector4: ...
    @property
    def zlyy(self) -> DVector4: ...
    @property
    def zlyz(self) -> DVector4: ...
    @property
    def zlyw(self) -> DVector4: ...
    @property
    def zlyo(self) -> DVector4: ...
    @property
    def zlyl(self) -> DVector4: ...
    @property
    def zlzx(self) -> DVector4: ...
    @property
    def zlzy(self) -> DVector4: ...
    @property
    def zlzz(self) -> DVector4: ...
    @property
    def zlzw(self) -> DVector4: ...
    @property
    def zlzo(self) -> DVector4: ...
    @property
    def zlzl(self) -> DVector4: ...
    @property
    def zlwx(self) -> DVector4: ...
    @property
    def zlwy(self) -> DVector4: ...
    @property
    def zlwz(self) -> DVector4: ...
    @property
    def zlww(self) -> DVector4: ...
    @property
    def zlwo(self) -> DVector4: ...
    @property
    def zlwl(self) -> DVector4: ...
    @property
    def zlox(self) -> DVector4: ...
    @property
    def zloy(self) -> DVector4: ...
    @property
    def zloz(self) -> DVector4: ...
    @property
    def zlow(self) -> DVector4: ...
    @property
    def zloo(self) -> DVector4: ...
    @property
    def zlol(self) -> DVector4: ...
    @property
    def zllx(self) -> DVector4: ...
    @property
    def zlly(self) -> DVector4: ...
    @property
    def zllz(self) -> DVector4: ...
    @property
    def zllw(self) -> DVector4: ...
    @property
    def zllo(self) -> DVector4: ...
    @property
    def zlll(self) -> DVector4: ...
    @property
    def wxxx(self) -> DVector4: ...
    @property
    def wxxy(self) -> DVector4: ...
    @property
    def wxxz(self) -> DVector4: ...
    @property
    def wxxw(self) -> DVector4: ...
    @property
    def wxxo(self) -> DVector4: ...
    @property
    def wxxl(self) -> DVector4: ...
    @property
    def wxyx(self) -> DVector4: ...
    @property
    def wxyy(self) -> DVector4: ...
    @property
    def wxyz(self) -> DVector4: ...
    @property
    def wxyw(self) -> DVector4: ...
    @property
    def wxyo(self) -> DVector4: ...
    @property
    def wxyl(self) -> DVector4: ...
    @property
    def wxzx(self) -> DVector4: ...
    @property
    def wxzy(self) -> DVector4: ...
    @property
    def wxzz(self) -> DVector4: ...
    @property
    def wxzw(self) -> DVector4: ...
    @property
    def wxzo(self) -> DVector4: ...
    @property
    def wxzl(self) -> DVector4: ...
    @property
    def wxwx(self) -> DVector4: ...
    @property
    def wxwy(self) -> DVector4: ...
    @property
    def wxwz(self) -> DVector4: ...
    @property
    def wxww(self) -> DVector4: ...
    @property
    def wxwo(self) -> DVector4: ...
    @property
    def wxwl(self) -> DVector4: ...
    @property
    def wxox(self) -> DVector4: ...
    @property
    def wxoy(self) -> DVector4: ...
    @property
    def wxoz(self) -> DVector4: ...
    @property
    def wxow(self) -> DVector4: ...
    @property
    def wxoo(self) -> DVector4: ...
    @property
    def wxol(self) -> DVector4: ...
    @property
    def wxlx(self) -> DVector4: ...
    @property
    def wxly(self) -> DVector4: ...
    @property
    def wxlz(self) -> DVector4: ...
    @property
    def wxlw(self) -> DVector4: ...
    @property
    def wxlo(self) -> DVector4: ...
    @property
    def wxll(self) -> DVector4: ...
    @property
    def wyxx(self) -> DVector4: ...
    @property
    def wyxy(self) -> DVector4: ...
    @property
    def wyxz(self) -> DVector4: ...
    @property
    def wyxw(self) -> DVector4: ...
    @property
    def wyxo(self) -> DVector4: ...
    @property
    def wyxl(self) -> DVector4: ...
    @property
    def wyyx(self) -> DVector4: ...
    @property
    def wyyy(self) -> DVector4: ...
    @property
    def wyyz(self) -> DVector4: ...
    @property
    def wyyw(self) -> DVector4: ...
    @property
    def wyyo(self) -> DVector4: ...
    @property
    def wyyl(self) -> DVector4: ...
    @property
    def wyzx(self) -> DVector4: ...
    @property
    def wyzy(self) -> DVector4: ...
    @property
    def wyzz(self) -> DVector4: ...
    @property
    def wyzw(self) -> DVector4: ...
    @property
    def wyzo(self) -> DVector4: ...
    @property
    def wyzl(self) -> DVector4: ...
    @property
    def wywx(self) -> DVector4: ...
    @property
    def wywy(self) -> DVector4: ...
    @property
    def wywz(self) -> DVector4: ...
    @property
    def wyww(self) -> DVector4: ...
    @property
    def wywo(self) -> DVector4: ...
    @property
    def wywl(self) -> DVector4: ...
    @property
    def wyox(self) -> DVector4: ...
    @property
    def wyoy(self) -> DVector4: ...
    @property
    def wyoz(self) -> DVector4: ...
    @property
    def wyow(self) -> DVector4: ...
    @property
    def wyoo(self) -> DVector4: ...
    @property
    def wyol(self) -> DVector4: ...
    @property
    def wylx(self) -> DVector4: ...
    @property
    def wyly(self) -> DVector4: ...
    @property
    def wylz(self) -> DVector4: ...
    @property
    def wylw(self) -> DVector4: ...
    @property
    def wylo(self) -> DVector4: ...
    @property
    def wyll(self) -> DVector4: ...
    @property
    def wzxx(self) -> DVector4: ...
    @property
    def wzxy(self) -> DVector4: ...
    @property
    def wzxz(self) -> DVector4: ...
    @property
    def wzxw(self) -> DVector4: ...
    @property
    def wzxo(self) -> DVector4: ...
    @property
    def wzxl(self) -> DVector4: ...
    @property
    def wzyx(self) -> DVector4: ...
    @property
    def wzyy(self) -> DVector4: ...
    @property
    def wzyz(self) -> DVector4: ...
    @property
    def wzyw(self) -> DVector4: ...
    @property
    def wzyo(self) -> DVector4: ...
    @property
    def wzyl(self) -> DVector4: ...
    @property
    def wzzx(self) -> DVector4: ...
    @property
    def wzzy(self) -> DVector4: ...
    @property
    def wzzz(self) -> DVector4: ...
    @property
    def wzzw(self) -> DVector4: ...
    @property
    def wzzo(self) -> DVector4: ...
    @property
    def wzzl(self) -> DVector4: ...
    @property
    def wzwx(self) -> DVector4: ...
    @property
    def wzwy(self) -> DVector4: ...
    @property
    def wzwz(self) -> DVector4: ...
    @property
    def wzww(self) -> DVector4: ...
    @property
    def wzwo(self) -> DVector4: ...
    @property
    def wzwl(self) -> DVector4: ...
    @property
    def wzox(self) -> DVector4: ...
    @property
    def wzoy(self) -> DVector4: ...
    @property
    def wzoz(self) -> DVector4: ...
    @property
    def wzow(self) -> DVector4: ...
    @property
    def wzoo(self) -> DVector4: ...
    @property
    def wzol(self) -> DVector4: ...
    @property
    def wzlx(self) -> DVector4: ...
    @property
    def wzly(self) -> DVector4: ...
    @property
    def wzlz(self) -> DVector4: ...
    @property
    def wzlw(self) -> DVector4: ...
    @property
    def wzlo(self) -> DVector4: ...
    @property
    def wzll(self) -> DVector4: ...
    @property
    def wwxx(self) -> DVector4: ...
    @property
    def wwxy(self) -> DVector4: ...
    @property
    def wwxz(self) -> DVector4: ...
    @property
    def wwxw(self) -> DVector4: ...
    @property
    def wwxo(self) -> DVector4: ...
    @property
    def wwxl(self) -> DVector4: ...
    @property
    def wwyx(self) -> DVector4: ...
    @property
    def wwyy(self) -> DVector4: ...
    @property
    def wwyz(self) -> DVector4: ...
    @property
    def wwyw(self) -> DVector4: ...
    @property
    def wwyo(self) -> DVector4: ...
    @property
    def wwyl(self) -> DVector4: ...
    @property
    def wwzx(self) -> DVector4: ...
    @property
    def wwzy(self) -> DVector4: ...
    @property
    def wwzz(self) -> DVector4: ...
    @property
    def wwzw(self) -> DVector4: ...
    @property
    def wwzo(self) -> DVector4: ...
    @property
    def wwzl(self) -> DVector4: ...
    @property
    def wwwx(self) -> DVector4: ...
    @property
    def wwwy(self) -> DVector4: ...
    @property
    def wwwz(self) -> DVector4: ...
    @property
    def wwww(self) -> DVector4: ...
    @property
    def wwwo(self) -> DVector4: ...
    @property
    def wwwl(self) -> DVector4: ...
    @property
    def wwox(self) -> DVector4: ...
    @property
    def wwoy(self) -> DVector4: ...
    @property
    def wwoz(self) -> DVector4: ...
    @property
    def wwow(self) -> DVector4: ...
    @property
    def wwoo(self) -> DVector4: ...
    @property
    def wwol(self) -> DVector4: ...
    @property
    def wwlx(self) -> DVector4: ...
    @property
    def wwly(self) -> DVector4: ...
    @property
    def wwlz(self) -> DVector4: ...
    @property
    def wwlw(self) -> DVector4: ...
    @property
    def wwlo(self) -> DVector4: ...
    @property
    def wwll(self) -> DVector4: ...
    @property
    def woxx(self) -> DVector4: ...
    @property
    def woxy(self) -> DVector4: ...
    @property
    def woxz(self) -> DVector4: ...
    @property
    def woxw(self) -> DVector4: ...
    @property
    def woxo(self) -> DVector4: ...
    @property
    def woxl(self) -> DVector4: ...
    @property
    def woyx(self) -> DVector4: ...
    @property
    def woyy(self) -> DVector4: ...
    @property
    def woyz(self) -> DVector4: ...
    @property
    def woyw(self) -> DVector4: ...
    @property
    def woyo(self) -> DVector4: ...
    @property
    def woyl(self) -> DVector4: ...
    @property
    def wozx(self) -> DVector4: ...
    @property
    def wozy(self) -> DVector4: ...
    @property
    def wozz(self) -> DVector4: ...
    @property
    def wozw(self) -> DVector4: ...
    @property
    def wozo(self) -> DVector4: ...
    @property
    def wozl(self) -> DVector4: ...
    @property
    def wowx(self) -> DVector4: ...
    @property
    def wowy(self) -> DVector4: ...
    @property
    def wowz(self) -> DVector4: ...
    @property
    def woww(self) -> DVector4: ...
    @property
    def wowo(self) -> DVector4: ...
    @property
    def wowl(self) -> DVector4: ...
    @property
    def woox(self) -> DVector4: ...
    @property
    def wooy(self) -> DVector4: ...
    @property
    def wooz(self) -> DVector4: ...
    @property
    def woow(self) -> DVector4: ...
    @property
    def wooo(self) -> DVector4: ...
    @property
    def wool(self) -> DVector4: ...
    @property
    def wolx(self) -> DVector4: ...
    @property
    def woly(self) -> DVector4: ...
    @property
    def wolz(self) -> DVector4: ...
    @property
    def wolw(self) -> DVector4: ...
    @property
    def wolo(self) -> DVector4: ...
    @property
    def woll(self) -> DVector4: ...
    @property
    def wlxx(self) -> DVector4: ...
    @property
    def wlxy(self) -> DVector4: ...
    @property
    def wlxz(self) -> DVector4: ...
    @property
    def wlxw(self) -> DVector4: ...
    @property
    def wlxo(self) -> DVector4: ...
    @property
    def wlxl(self) -> DVector4: ...
    @property
    def wlyx(self) -> DVector4: ...
    @property
    def wlyy(self) -> DVector4: ...
    @property
    def wlyz(self) -> DVector4: ...
    @property
    def wlyw(self) -> DVector4: ...
    @property
    def wlyo(self) -> DVector4: ...
    @property
    def wlyl(self) -> DVector4: ...
    @property
    def wlzx(self) -> DVector4: ...
    @property
    def wlzy(self) -> DVector4: ...
    @property
    def wlzz(self) -> DVector4: ...
    @property
    def wlzw(self) -> DVector4: ...
    @property
    def wlzo(self) -> DVector4: ...
    @property
    def wlzl(self) -> DVector4: ...
    @property
    def wlwx(self) -> DVector4: ...
    @property
    def wlwy(self) -> DVector4: ...
    @property
    def wlwz(self) -> DVector4: ...
    @property
    def wlww(self) -> DVector4: ...
    @property
    def wlwo(self) -> DVector4: ...
    @property
    def wlwl(self) -> DVector4: ...
    @property
    def wlox(self) -> DVector4: ...
    @property
    def wloy(self) -> DVector4: ...
    @property
    def wloz(self) -> DVector4: ...
    @property
    def wlow(self) -> DVector4: ...
    @property
    def wloo(self) -> DVector4: ...
    @property
    def wlol(self) -> DVector4: ...
    @property
    def wllx(self) -> DVector4: ...
    @property
    def wlly(self) -> DVector4: ...
    @property
    def wllz(self) -> DVector4: ...
    @property
    def wllw(self) -> DVector4: ...
    @property
    def wllo(self) -> DVector4: ...
    @property
    def wlll(self) -> DVector4: ...
    @property
    def oxxx(self) -> DVector4: ...
    @property
    def oxxy(self) -> DVector4: ...
    @property
    def oxxz(self) -> DVector4: ...
    @property
    def oxxw(self) -> DVector4: ...
    @property
    def oxxo(self) -> DVector4: ...
    @property
    def oxxl(self) -> DVector4: ...
    @property
    def oxyx(self) -> DVector4: ...
    @property
    def oxyy(self) -> DVector4: ...
    @property
    def oxyz(self) -> DVector4: ...
    @property
    def oxyw(self) -> DVector4: ...
    @property
    def oxyo(self) -> DVector4: ...
    @property
    def oxyl(self) -> DVector4: ...
    @property
    def oxzx(self) -> DVector4: ...
    @property
    def oxzy(self) -> DVector4: ...
    @property
    def oxzz(self) -> DVector4: ...
    @property
    def oxzw(self) -> DVector4: ...
    @property
    def oxzo(self) -> DVector4: ...
    @property
    def oxzl(self) -> DVector4: ...
    @property
    def oxwx(self) -> DVector4: ...
    @property
    def oxwy(self) -> DVector4: ...
    @property
    def oxwz(self) -> DVector4: ...
    @property
    def oxww(self) -> DVector4: ...
    @property
    def oxwo(self) -> DVector4: ...
    @property
    def oxwl(self) -> DVector4: ...
    @property
    def oxox(self) -> DVector4: ...
    @property
    def oxoy(self) -> DVector4: ...
    @property
    def oxoz(self) -> DVector4: ...
    @property
    def oxow(self) -> DVector4: ...
    @property
    def oxoo(self) -> DVector4: ...
    @property
    def oxol(self) -> DVector4: ...
    @property
    def oxlx(self) -> DVector4: ...
    @property
    def oxly(self) -> DVector4: ...
    @property
    def oxlz(self) -> DVector4: ...
    @property
    def oxlw(self) -> DVector4: ...
    @property
    def oxlo(self) -> DVector4: ...
    @property
    def oxll(self) -> DVector4: ...
    @property
    def oyxx(self) -> DVector4: ...
    @property
    def oyxy(self) -> DVector4: ...
    @property
    def oyxz(self) -> DVector4: ...
    @property
    def oyxw(self) -> DVector4: ...
    @property
    def oyxo(self) -> DVector4: ...
    @property
    def oyxl(self) -> DVector4: ...
    @property
    def oyyx(self) -> DVector4: ...
    @property
    def oyyy(self) -> DVector4: ...
    @property
    def oyyz(self) -> DVector4: ...
    @property
    def oyyw(self) -> DVector4: ...
    @property
    def oyyo(self) -> DVector4: ...
    @property
    def oyyl(self) -> DVector4: ...
    @property
    def oyzx(self) -> DVector4: ...
    @property
    def oyzy(self) -> DVector4: ...
    @property
    def oyzz(self) -> DVector4: ...
    @property
    def oyzw(self) -> DVector4: ...
    @property
    def oyzo(self) -> DVector4: ...
    @property
    def oyzl(self) -> DVector4: ...
    @property
    def oywx(self) -> DVector4: ...
    @property
    def oywy(self) -> DVector4: ...
    @property
    def oywz(self) -> DVector4: ...
    @property
    def oyww(self) -> DVector4: ...
    @property
    def oywo(self) -> DVector4: ...
    @property
    def oywl(self) -> DVector4: ...
    @property
    def oyox(self) -> DVector4: ...
    @property
    def oyoy(self) -> DVector4: ...
    @property
    def oyoz(self) -> DVector4: ...
    @property
    def oyow(self) -> DVector4: ...
    @property
    def oyoo(self) -> DVector4: ...
    @property
    def oyol(self) -> DVector4: ...
    @property
    def oylx(self) -> DVector4: ...
    @property
    def oyly(self) -> DVector4: ...
    @property
    def oylz(self) -> DVector4: ...
    @property
    def oylw(self) -> DVector4: ...
    @property
    def oylo(self) -> DVector4: ...
    @property
    def oyll(self) -> DVector4: ...
    @property
    def ozxx(self) -> DVector4: ...
    @property
    def ozxy(self) -> DVector4: ...
    @property
    def ozxz(self) -> DVector4: ...
    @property
    def ozxw(self) -> DVector4: ...
    @property
    def ozxo(self) -> DVector4: ...
    @property
    def ozxl(self) -> DVector4: ...
    @property
    def ozyx(self) -> DVector4: ...
    @property
    def ozyy(self) -> DVector4: ...
    @property
    def ozyz(self) -> DVector4: ...
    @property
    def ozyw(self) -> DVector4: ...
    @property
    def ozyo(self) -> DVector4: ...
    @property
    def ozyl(self) -> DVector4: ...
    @property
    def ozzx(self) -> DVector4: ...
    @property
    def ozzy(self) -> DVector4: ...
    @property
    def ozzz(self) -> DVector4: ...
    @property
    def ozzw(self) -> DVector4: ...
    @property
    def ozzo(self) -> DVector4: ...
    @property
    def ozzl(self) -> DVector4: ...
    @property
    def ozwx(self) -> DVector4: ...
    @property
    def ozwy(self) -> DVector4: ...
    @property
    def ozwz(self) -> DVector4: ...
    @property
    def ozww(self) -> DVector4: ...
    @property
    def ozwo(self) -> DVector4: ...
    @property
    def ozwl(self) -> DVector4: ...
    @property
    def ozox(self) -> DVector4: ...
    @property
    def ozoy(self) -> DVector4: ...
    @property
    def ozoz(self) -> DVector4: ...
    @property
    def ozow(self) -> DVector4: ...
    @property
    def ozoo(self) -> DVector4: ...
    @property
    def ozol(self) -> DVector4: ...
    @property
    def ozlx(self) -> DVector4: ...
    @property
    def ozly(self) -> DVector4: ...
    @property
    def ozlz(self) -> DVector4: ...
    @property
    def ozlw(self) -> DVector4: ...
    @property
    def ozlo(self) -> DVector4: ...
    @property
    def ozll(self) -> DVector4: ...
    @property
    def owxx(self) -> DVector4: ...
    @property
    def owxy(self) -> DVector4: ...
    @property
    def owxz(self) -> DVector4: ...
    @property
    def owxw(self) -> DVector4: ...
    @property
    def owxo(self) -> DVector4: ...
    @property
    def owxl(self) -> DVector4: ...
    @property
    def owyx(self) -> DVector4: ...
    @property
    def owyy(self) -> DVector4: ...
    @property
    def owyz(self) -> DVector4: ...
    @property
    def owyw(self) -> DVector4: ...
    @property
    def owyo(self) -> DVector4: ...
    @property
    def owyl(self) -> DVector4: ...
    @property
    def owzx(self) -> DVector4: ...
    @property
    def owzy(self) -> DVector4: ...
    @property
    def owzz(self) -> DVector4: ...
    @property
    def owzw(self) -> DVector4: ...
    @property
    def owzo(self) -> DVector4: ...
    @property
    def owzl(self) -> DVector4: ...
    @property
    def owwx(self) -> DVector4: ...
    @property
    def owwy(self) -> DVector4: ...
    @property
    def owwz(self) -> DVector4: ...
    @property
    def owww(self) -> DVector4: ...
    @property
    def owwo(self) -> DVector4: ...
    @property
    def owwl(self) -> DVector4: ...
    @property
    def owox(self) -> DVector4: ...
    @property
    def owoy(self) -> DVector4: ...
    @property
    def owoz(self) -> DVector4: ...
    @property
    def owow(self) -> DVector4: ...
    @property
    def owoo(self) -> DVector4: ...
    @property
    def owol(self) -> DVector4: ...
    @property
    def owlx(self) -> DVector4: ...
    @property
    def owly(self) -> DVector4: ...
    @property
    def owlz(self) -> DVector4: ...
    @property
    def owlw(self) -> DVector4: ...
    @property
    def owlo(self) -> DVector4: ...
    @property
    def owll(self) -> DVector4: ...
    @property
    def ooxx(self) -> DVector4: ...
    @property
    def ooxy(self) -> DVector4: ...
    @property
    def ooxz(self) -> DVector4: ...
    @property
    def ooxw(self) -> DVector4: ...
    @property
    def ooxo(self) -> DVector4: ...
    @property
    def ooxl(self) -> DVector4: ...
    @property
    def ooyx(self) -> DVector4: ...
    @property
    def ooyy(self) -> DVector4: ...
    @property
    def ooyz(self) -> DVector4: ...
    @property
    def ooyw(self) -> DVector4: ...
    @property
    def ooyo(self) -> DVector4: ...
    @property
    def ooyl(self) -> DVector4: ...
    @property
    def oozx(self) -> DVector4: ...
    @property
    def oozy(self) -> DVector4: ...
    @property
    def oozz(self) -> DVector4: ...
    @property
    def oozw(self) -> DVector4: ...
    @property
    def oozo(self) -> DVector4: ...
    @property
    def oozl(self) -> DVector4: ...
    @property
    def oowx(self) -> DVector4: ...
    @property
    def oowy(self) -> DVector4: ...
    @property
    def oowz(self) -> DVector4: ...
    @property
    def ooww(self) -> DVector4: ...
    @property
    def oowo(self) -> DVector4: ...
    @property
    def oowl(self) -> DVector4: ...
    @property
    def ooox(self) -> DVector4: ...
    @property
    def oooy(self) -> DVector4: ...
    @property
    def oooz(self) -> DVector4: ...
    @property
    def ooow(self) -> DVector4: ...
    @property
    def oolx(self) -> DVector4: ...
    @property
    def ooly(self) -> DVector4: ...
    @property
    def oolz(self) -> DVector4: ...
    @property
    def oolw(self) -> DVector4: ...
    @property
    def olxx(self) -> DVector4: ...
    @property
    def olxy(self) -> DVector4: ...
    @property
    def olxz(self) -> DVector4: ...
    @property
    def olxw(self) -> DVector4: ...
    @property
    def olxo(self) -> DVector4: ...
    @property
    def olxl(self) -> DVector4: ...
    @property
    def olyx(self) -> DVector4: ...
    @property
    def olyy(self) -> DVector4: ...
    @property
    def olyz(self) -> DVector4: ...
    @property
    def olyw(self) -> DVector4: ...
    @property
    def olyo(self) -> DVector4: ...
    @property
    def olyl(self) -> DVector4: ...
    @property
    def olzx(self) -> DVector4: ...
    @property
    def olzy(self) -> DVector4: ...
    @property
    def olzz(self) -> DVector4: ...
    @property
    def olzw(self) -> DVector4: ...
    @property
    def olzo(self) -> DVector4: ...
    @property
    def olzl(self) -> DVector4: ...
    @property
    def olwx(self) -> DVector4: ...
    @property
    def olwy(self) -> DVector4: ...
    @property
    def olwz(self) -> DVector4: ...
    @property
    def olww(self) -> DVector4: ...
    @property
    def olwo(self) -> DVector4: ...
    @property
    def olwl(self) -> DVector4: ...
    @property
    def olox(self) -> DVector4: ...
    @property
    def oloy(self) -> DVector4: ...
    @property
    def oloz(self) -> DVector4: ...
    @property
    def olow(self) -> DVector4: ...
    @property
    def ollx(self) -> DVector4: ...
    @property
    def olly(self) -> DVector4: ...
    @property
    def ollz(self) -> DVector4: ...
    @property
    def ollw(self) -> DVector4: ...
    @property
    def lxxx(self) -> DVector4: ...
    @property
    def lxxy(self) -> DVector4: ...
    @property
    def lxxz(self) -> DVector4: ...
    @property
    def lxxw(self) -> DVector4: ...
    @property
    def lxxo(self) -> DVector4: ...
    @property
    def lxxl(self) -> DVector4: ...
    @property
    def lxyx(self) -> DVector4: ...
    @property
    def lxyy(self) -> DVector4: ...
    @property
    def lxyz(self) -> DVector4: ...
    @property
    def lxyw(self) -> DVector4: ...
    @property
    def lxyo(self) -> DVector4: ...
    @property
    def lxyl(self) -> DVector4: ...
    @property
    def lxzx(self) -> DVector4: ...
    @property
    def lxzy(self) -> DVector4: ...
    @property
    def lxzz(self) -> DVector4: ...
    @property
    def lxzw(self) -> DVector4: ...
    @property
    def lxzo(self) -> DVector4: ...
    @property
    def lxzl(self) -> DVector4: ...
    @property
    def lxwx(self) -> DVector4: ...
    @property
    def lxwy(self) -> DVector4: ...
    @property
    def lxwz(self) -> DVector4: ...
    @property
    def lxww(self) -> DVector4: ...
    @property
    def lxwo(self) -> DVector4: ...
    @property
    def lxwl(self) -> DVector4: ...
    @property
    def lxox(self) -> DVector4: ...
    @property
    def lxoy(self) -> DVector4: ...
    @property
    def lxoz(self) -> DVector4: ...
    @property
    def lxow(self) -> DVector4: ...
    @property
    def lxoo(self) -> DVector4: ...
    @property
    def lxol(self) -> DVector4: ...
    @property
    def lxlx(self) -> DVector4: ...
    @property
    def lxly(self) -> DVector4: ...
    @property
    def lxlz(self) -> DVector4: ...
    @property
    def lxlw(self) -> DVector4: ...
    @property
    def lxlo(self) -> DVector4: ...
    @property
    def lxll(self) -> DVector4: ...
    @property
    def lyxx(self) -> DVector4: ...
    @property
    def lyxy(self) -> DVector4: ...
    @property
    def lyxz(self) -> DVector4: ...
    @property
    def lyxw(self) -> DVector4: ...
    @property
    def lyxo(self) -> DVector4: ...
    @property
    def lyxl(self) -> DVector4: ...
    @property
    def lyyx(self) -> DVector4: ...
    @property
    def lyyy(self) -> DVector4: ...
    @property
    def lyyz(self) -> DVector4: ...
    @property
    def lyyw(self) -> DVector4: ...
    @property
    def lyyo(self) -> DVector4: ...
    @property
    def lyyl(self) -> DVector4: ...
    @property
    def lyzx(self) -> DVector4: ...
    @property
    def lyzy(self) -> DVector4: ...
    @property
    def lyzz(self) -> DVector4: ...
    @property
    def lyzw(self) -> DVector4: ...
    @property
    def lyzo(self) -> DVector4: ...
    @property
    def lyzl(self) -> DVector4: ...
    @property
    def lywx(self) -> DVector4: ...
    @property
    def lywy(self) -> DVector4: ...
    @property
    def lywz(self) -> DVector4: ...
    @property
    def lyww(self) -> DVector4: ...
    @property
    def lywo(self) -> DVector4: ...
    @property
    def lywl(self) -> DVector4: ...
    @property
    def lyox(self) -> DVector4: ...
    @property
    def lyoy(self) -> DVector4: ...
    @property
    def lyoz(self) -> DVector4: ...
    @property
    def lyow(self) -> DVector4: ...
    @property
    def lyoo(self) -> DVector4: ...
    @property
    def lyol(self) -> DVector4: ...
    @property
    def lylx(self) -> DVector4: ...
    @property
    def lyly(self) -> DVector4: ...
    @property
    def lylz(self) -> DVector4: ...
    @property
    def lylw(self) -> DVector4: ...
    @property
    def lylo(self) -> DVector4: ...
    @property
    def lyll(self) -> DVector4: ...
    @property
    def lzxx(self) -> DVector4: ...
    @property
    def lzxy(self) -> DVector4: ...
    @property
    def lzxz(self) -> DVector4: ...
    @property
    def lzxw(self) -> DVector4: ...
    @property
    def lzxo(self) -> DVector4: ...
    @property
    def lzxl(self) -> DVector4: ...
    @property
    def lzyx(self) -> DVector4: ...
    @property
    def lzyy(self) -> DVector4: ...
    @property
    def lzyz(self) -> DVector4: ...
    @property
    def lzyw(self) -> DVector4: ...
    @property
    def lzyo(self) -> DVector4: ...
    @property
    def lzyl(self) -> DVector4: ...
    @property
    def lzzx(self) -> DVector4: ...
    @property
    def lzzy(self) -> DVector4: ...
    @property
    def lzzz(self) -> DVector4: ...
    @property
    def lzzw(self) -> DVector4: ...
    @property
    def lzzo(self) -> DVector4: ...
    @property
    def lzzl(self) -> DVector4: ...
    @property
    def lzwx(self) -> DVector4: ...
    @property
    def lzwy(self) -> DVector4: ...
    @property
    def lzwz(self) -> DVector4: ...
    @property
    def lzww(self) -> DVector4: ...
    @property
    def lzwo(self) -> DVector4: ...
    @property
    def lzwl(self) -> DVector4: ...
    @property
    def lzox(self) -> DVector4: ...
    @property
    def lzoy(self) -> DVector4: ...
    @property
    def lzoz(self) -> DVector4: ...
    @property
    def lzow(self) -> DVector4: ...
    @property
    def lzoo(self) -> DVector4: ...
    @property
    def lzol(self) -> DVector4: ...
    @property
    def lzlx(self) -> DVector4: ...
    @property
    def lzly(self) -> DVector4: ...
    @property
    def lzlz(self) -> DVector4: ...
    @property
    def lzlw(self) -> DVector4: ...
    @property
    def lzlo(self) -> DVector4: ...
    @property
    def lzll(self) -> DVector4: ...
    @property
    def lwxx(self) -> DVector4: ...
    @property
    def lwxy(self) -> DVector4: ...
    @property
    def lwxz(self) -> DVector4: ...
    @property
    def lwxw(self) -> DVector4: ...
    @property
    def lwxo(self) -> DVector4: ...
    @property
    def lwxl(self) -> DVector4: ...
    @property
    def lwyx(self) -> DVector4: ...
    @property
    def lwyy(self) -> DVector4: ...
    @property
    def lwyz(self) -> DVector4: ...
    @property
    def lwyw(self) -> DVector4: ...
    @property
    def lwyo(self) -> DVector4: ...
    @property
    def lwyl(self) -> DVector4: ...
    @property
    def lwzx(self) -> DVector4: ...
    @property
    def lwzy(self) -> DVector4: ...
    @property
    def lwzz(self) -> DVector4: ...
    @property
    def lwzw(self) -> DVector4: ...
    @property
    def lwzo(self) -> DVector4: ...
    @property
    def lwzl(self) -> DVector4: ...
    @property
    def lwwx(self) -> DVector4: ...
    @property
    def lwwy(self) -> DVector4: ...
    @property
    def lwwz(self) -> DVector4: ...
    @property
    def lwww(self) -> DVector4: ...
    @property
    def lwwo(self) -> DVector4: ...
    @property
    def lwwl(self) -> DVector4: ...
    @property
    def lwox(self) -> DVector4: ...
    @property
    def lwoy(self) -> DVector4: ...
    @property
    def lwoz(self) -> DVector4: ...
    @property
    def lwow(self) -> DVector4: ...
    @property
    def lwoo(self) -> DVector4: ...
    @property
    def lwol(self) -> DVector4: ...
    @property
    def lwlx(self) -> DVector4: ...
    @property
    def lwly(self) -> DVector4: ...
    @property
    def lwlz(self) -> DVector4: ...
    @property
    def lwlw(self) -> DVector4: ...
    @property
    def lwlo(self) -> DVector4: ...
    @property
    def lwll(self) -> DVector4: ...
    @property
    def loxx(self) -> DVector4: ...
    @property
    def loxy(self) -> DVector4: ...
    @property
    def loxz(self) -> DVector4: ...
    @property
    def loxw(self) -> DVector4: ...
    @property
    def loxo(self) -> DVector4: ...
    @property
    def loxl(self) -> DVector4: ...
    @property
    def loyx(self) -> DVector4: ...
    @property
    def loyy(self) -> DVector4: ...
    @property
    def loyz(self) -> DVector4: ...
    @property
    def loyw(self) -> DVector4: ...
    @property
    def loyo(self) -> DVector4: ...
    @property
    def loyl(self) -> DVector4: ...
    @property
    def lozx(self) -> DVector4: ...
    @property
    def lozy(self) -> DVector4: ...
    @property
    def lozz(self) -> DVector4: ...
    @property
    def lozw(self) -> DVector4: ...
    @property
    def lozo(self) -> DVector4: ...
    @property
    def lozl(self) -> DVector4: ...
    @property
    def lowx(self) -> DVector4: ...
    @property
    def lowy(self) -> DVector4: ...
    @property
    def lowz(self) -> DVector4: ...
    @property
    def loww(self) -> DVector4: ...
    @property
    def lowo(self) -> DVector4: ...
    @property
    def lowl(self) -> DVector4: ...
    @property
    def loox(self) -> DVector4: ...
    @property
    def looy(self) -> DVector4: ...
    @property
    def looz(self) -> DVector4: ...
    @property
    def loow(self) -> DVector4: ...
    @property
    def lolx(self) -> DVector4: ...
    @property
    def loly(self) -> DVector4: ...
    @property
    def lolz(self) -> DVector4: ...
    @property
    def lolw(self) -> DVector4: ...
    @property
    def llxx(self) -> DVector4: ...
    @property
    def llxy(self) -> DVector4: ...
    @property
    def llxz(self) -> DVector4: ...
    @property
    def llxw(self) -> DVector4: ...
    @property
    def llxo(self) -> DVector4: ...
    @property
    def llxl(self) -> DVector4: ...
    @property
    def llyx(self) -> DVector4: ...
    @property
    def llyy(self) -> DVector4: ...
    @property
    def llyz(self) -> DVector4: ...
    @property
    def llyw(self) -> DVector4: ...
    @property
    def llyo(self) -> DVector4: ...
    @property
    def llyl(self) -> DVector4: ...
    @property
    def llzx(self) -> DVector4: ...
    @property
    def llzy(self) -> DVector4: ...
    @property
    def llzz(self) -> DVector4: ...
    @property
    def llzw(self) -> DVector4: ...
    @property
    def llzo(self) -> DVector4: ...
    @property
    def llzl(self) -> DVector4: ...
    @property
    def llwx(self) -> DVector4: ...
    @property
    def llwy(self) -> DVector4: ...
    @property
    def llwz(self) -> DVector4: ...
    @property
    def llww(self) -> DVector4: ...
    @property
    def llwo(self) -> DVector4: ...
    @property
    def llwl(self) -> DVector4: ...
    @property
    def llox(self) -> DVector4: ...
    @property
    def lloy(self) -> DVector4: ...
    @property
    def lloz(self) -> DVector4: ...
    @property
    def llow(self) -> DVector4: ...
    @property
    def lllx(self) -> DVector4: ...
    @property
    def llly(self) -> DVector4: ...
    @property
    def lllz(self) -> DVector4: ...
    @property
    def lllw(self) -> DVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def g(self) -> float: ...
    @property
    def b(self) -> float: ...
    @property
    def a(self) -> float: ...
    @property
    def rr(self) -> DVector2: ...
    @property
    def rg(self) -> DVector2: ...
    @property
    def rb(self) -> DVector2: ...
    @property
    def ra(self) -> DVector2: ...
    @property
    def ro(self) -> DVector2: ...
    @property
    def rl(self) -> DVector2: ...
    @property
    def gr(self) -> DVector2: ...
    @property
    def gg(self) -> DVector2: ...
    @property
    def gb(self) -> DVector2: ...
    @property
    def ga(self) -> DVector2: ...
    @property
    def go(self) -> DVector2: ...
    @property
    def gl(self) -> DVector2: ...
    @property
    def br(self) -> DVector2: ...
    @property
    def bg(self) -> DVector2: ...
    @property
    def bb(self) -> DVector2: ...
    @property
    def ba(self) -> DVector2: ...
    @property
    def bo(self) -> DVector2: ...
    @property
    def bl(self) -> DVector2: ...
    @property
    def ar(self) -> DVector2: ...
    @property
    def ag(self) -> DVector2: ...
    @property
    def ab(self) -> DVector2: ...
    @property
    def aa(self) -> DVector2: ...
    @property
    def ao(self) -> DVector2: ...
    @property
    def al(self) -> DVector2: ...
    @property
    def og(self) -> DVector2: ...
    @property
    def ob(self) -> DVector2: ...
    @property
    def oa(self) -> DVector2: ...
    @property
    def lr(self) -> DVector2: ...
    @property
    def lg(self) -> DVector2: ...
    @property
    def lb(self) -> DVector2: ...
    @property
    def la(self) -> DVector2: ...
    @property
    def rrr(self) -> DVector3: ...
    @property
    def rrg(self) -> DVector3: ...
    @property
    def rrb(self) -> DVector3: ...
    @property
    def rra(self) -> DVector3: ...
    @property
    def rro(self) -> DVector3: ...
    @property
    def rrl(self) -> DVector3: ...
    @property
    def rgr(self) -> DVector3: ...
    @property
    def rgg(self) -> DVector3: ...
    @property
    def rgb(self) -> DVector3: ...
    @property
    def rga(self) -> DVector3: ...
    @property
    def rgo(self) -> DVector3: ...
    @property
    def rgl(self) -> DVector3: ...
    @property
    def rbr(self) -> DVector3: ...
    @property
    def rbg(self) -> DVector3: ...
    @property
    def rbb(self) -> DVector3: ...
    @property
    def rba(self) -> DVector3: ...
    @property
    def rbo(self) -> DVector3: ...
    @property
    def rbl(self) -> DVector3: ...
    @property
    def rar(self) -> DVector3: ...
    @property
    def rag(self) -> DVector3: ...
    @property
    def rab(self) -> DVector3: ...
    @property
    def raa(self) -> DVector3: ...
    @property
    def rao(self) -> DVector3: ...
    @property
    def ral(self) -> DVector3: ...
    @property
    def ror(self) -> DVector3: ...
    @property
    def rog(self) -> DVector3: ...
    @property
    def rob(self) -> DVector3: ...
    @property
    def roa(self) -> DVector3: ...
    @property
    def roo(self) -> DVector3: ...
    @property
    def rol(self) -> DVector3: ...
    @property
    def rlr(self) -> DVector3: ...
    @property
    def rlg(self) -> DVector3: ...
    @property
    def rlb(self) -> DVector3: ...
    @property
    def rla(self) -> DVector3: ...
    @property
    def rlo(self) -> DVector3: ...
    @property
    def rll(self) -> DVector3: ...
    @property
    def grr(self) -> DVector3: ...
    @property
    def grg(self) -> DVector3: ...
    @property
    def grb(self) -> DVector3: ...
    @property
    def gra(self) -> DVector3: ...
    @property
    def gro(self) -> DVector3: ...
    @property
    def grl(self) -> DVector3: ...
    @property
    def ggr(self) -> DVector3: ...
    @property
    def ggg(self) -> DVector3: ...
    @property
    def ggb(self) -> DVector3: ...
    @property
    def gga(self) -> DVector3: ...
    @property
    def ggo(self) -> DVector3: ...
    @property
    def ggl(self) -> DVector3: ...
    @property
    def gbr(self) -> DVector3: ...
    @property
    def gbg(self) -> DVector3: ...
    @property
    def gbb(self) -> DVector3: ...
    @property
    def gba(self) -> DVector3: ...
    @property
    def gbo(self) -> DVector3: ...
    @property
    def gbl(self) -> DVector3: ...
    @property
    def gar(self) -> DVector3: ...
    @property
    def gag(self) -> DVector3: ...
    @property
    def gab(self) -> DVector3: ...
    @property
    def gaa(self) -> DVector3: ...
    @property
    def gao(self) -> DVector3: ...
    @property
    def gal(self) -> DVector3: ...
    @property
    def gor(self) -> DVector3: ...
    @property
    def gog(self) -> DVector3: ...
    @property
    def gob(self) -> DVector3: ...
    @property
    def goa(self) -> DVector3: ...
    @property
    def goo(self) -> DVector3: ...
    @property
    def gol(self) -> DVector3: ...
    @property
    def glr(self) -> DVector3: ...
    @property
    def glg(self) -> DVector3: ...
    @property
    def glb(self) -> DVector3: ...
    @property
    def gla(self) -> DVector3: ...
    @property
    def glo(self) -> DVector3: ...
    @property
    def gll(self) -> DVector3: ...
    @property
    def brr(self) -> DVector3: ...
    @property
    def brg(self) -> DVector3: ...
    @property
    def brb(self) -> DVector3: ...
    @property
    def bra(self) -> DVector3: ...
    @property
    def bro(self) -> DVector3: ...
    @property
    def brl(self) -> DVector3: ...
    @property
    def bgr(self) -> DVector3: ...
    @property
    def bgg(self) -> DVector3: ...
    @property
    def bgb(self) -> DVector3: ...
    @property
    def bga(self) -> DVector3: ...
    @property
    def bgo(self) -> DVector3: ...
    @property
    def bgl(self) -> DVector3: ...
    @property
    def bbr(self) -> DVector3: ...
    @property
    def bbg(self) -> DVector3: ...
    @property
    def bbb(self) -> DVector3: ...
    @property
    def bba(self) -> DVector3: ...
    @property
    def bbo(self) -> DVector3: ...
    @property
    def bbl(self) -> DVector3: ...
    @property
    def bar(self) -> DVector3: ...
    @property
    def bag(self) -> DVector3: ...
    @property
    def bab(self) -> DVector3: ...
    @property
    def baa(self) -> DVector3: ...
    @property
    def bao(self) -> DVector3: ...
    @property
    def bal(self) -> DVector3: ...
    @property
    def bor(self) -> DVector3: ...
    @property
    def bog(self) -> DVector3: ...
    @property
    def bob(self) -> DVector3: ...
    @property
    def boa(self) -> DVector3: ...
    @property
    def boo(self) -> DVector3: ...
    @property
    def bol(self) -> DVector3: ...
    @property
    def blr(self) -> DVector3: ...
    @property
    def blg(self) -> DVector3: ...
    @property
    def blb(self) -> DVector3: ...
    @property
    def bla(self) -> DVector3: ...
    @property
    def blo(self) -> DVector3: ...
    @property
    def bll(self) -> DVector3: ...
    @property
    def arr(self) -> DVector3: ...
    @property
    def arg(self) -> DVector3: ...
    @property
    def arb(self) -> DVector3: ...
    @property
    def ara(self) -> DVector3: ...
    @property
    def aro(self) -> DVector3: ...
    @property
    def arl(self) -> DVector3: ...
    @property
    def agr(self) -> DVector3: ...
    @property
    def agg(self) -> DVector3: ...
    @property
    def agb(self) -> DVector3: ...
    @property
    def aga(self) -> DVector3: ...
    @property
    def ago(self) -> DVector3: ...
    @property
    def agl(self) -> DVector3: ...
    @property
    def abr(self) -> DVector3: ...
    @property
    def abg(self) -> DVector3: ...
    @property
    def abb(self) -> DVector3: ...
    @property
    def aba(self) -> DVector3: ...
    @property
    def abo(self) -> DVector3: ...
    @property
    def abl(self) -> DVector3: ...
    @property
    def aar(self) -> DVector3: ...
    @property
    def aag(self) -> DVector3: ...
    @property
    def aab(self) -> DVector3: ...
    @property
    def aaa(self) -> DVector3: ...
    @property
    def aao(self) -> DVector3: ...
    @property
    def aal(self) -> DVector3: ...
    @property
    def aor(self) -> DVector3: ...
    @property
    def aog(self) -> DVector3: ...
    @property
    def aob(self) -> DVector3: ...
    @property
    def aoa(self) -> DVector3: ...
    @property
    def aoo(self) -> DVector3: ...
    @property
    def aol(self) -> DVector3: ...
    @property
    def alr(self) -> DVector3: ...
    @property
    def alg(self) -> DVector3: ...
    @property
    def alb(self) -> DVector3: ...
    @property
    def ala(self) -> DVector3: ...
    @property
    def alo(self) -> DVector3: ...
    @property
    def all(self) -> DVector3: ...
    @property
    def orr(self) -> DVector3: ...
    @property
    def org(self) -> DVector3: ...
    @property
    def orb(self) -> DVector3: ...
    @property
    def ora(self) -> DVector3: ...
    @property
    def oro(self) -> DVector3: ...
    @property
    def orl(self) -> DVector3: ...
    @property
    def ogr(self) -> DVector3: ...
    @property
    def ogg(self) -> DVector3: ...
    @property
    def ogb(self) -> DVector3: ...
    @property
    def oga(self) -> DVector3: ...
    @property
    def ogo(self) -> DVector3: ...
    @property
    def ogl(self) -> DVector3: ...
    @property
    def obr(self) -> DVector3: ...
    @property
    def obg(self) -> DVector3: ...
    @property
    def obb(self) -> DVector3: ...
    @property
    def oba(self) -> DVector3: ...
    @property
    def obo(self) -> DVector3: ...
    @property
    def obl(self) -> DVector3: ...
    @property
    def oar(self) -> DVector3: ...
    @property
    def oag(self) -> DVector3: ...
    @property
    def oab(self) -> DVector3: ...
    @property
    def oaa(self) -> DVector3: ...
    @property
    def oao(self) -> DVector3: ...
    @property
    def oal(self) -> DVector3: ...
    @property
    def oor(self) -> DVector3: ...
    @property
    def oog(self) -> DVector3: ...
    @property
    def oob(self) -> DVector3: ...
    @property
    def ooa(self) -> DVector3: ...
    @property
    def olr(self) -> DVector3: ...
    @property
    def olg(self) -> DVector3: ...
    @property
    def olb(self) -> DVector3: ...
    @property
    def ola(self) -> DVector3: ...
    @property
    def lrr(self) -> DVector3: ...
    @property
    def lrg(self) -> DVector3: ...
    @property
    def lrb(self) -> DVector3: ...
    @property
    def lra(self) -> DVector3: ...
    @property
    def lro(self) -> DVector3: ...
    @property
    def lrl(self) -> DVector3: ...
    @property
    def lgr(self) -> DVector3: ...
    @property
    def lgg(self) -> DVector3: ...
    @property
    def lgb(self) -> DVector3: ...
    @property
    def lga(self) -> DVector3: ...
    @property
    def lgo(self) -> DVector3: ...
    @property
    def lgl(self) -> DVector3: ...
    @property
    def lbr(self) -> DVector3: ...
    @property
    def lbg(self) -> DVector3: ...
    @property
    def lbb(self) -> DVector3: ...
    @property
    def lba(self) -> DVector3: ...
    @property
    def lbo(self) -> DVector3: ...
    @property
    def lbl(self) -> DVector3: ...
    @property
    def lar(self) -> DVector3: ...
    @property
    def lag(self) -> DVector3: ...
    @property
    def lab(self) -> DVector3: ...
    @property
    def laa(self) -> DVector3: ...
    @property
    def lao(self) -> DVector3: ...
    @property
    def lal(self) -> DVector3: ...
    @property
    def lor(self) -> DVector3: ...
    @property
    def log(self) -> DVector3: ...
    @property
    def lob(self) -> DVector3: ...
    @property
    def loa(self) -> DVector3: ...
    @property
    def llr(self) -> DVector3: ...
    @property
    def llg(self) -> DVector3: ...
    @property
    def llb(self) -> DVector3: ...
    @property
    def lla(self) -> DVector3: ...
    @property
    def rrrr(self) -> DVector4: ...
    @property
    def rrrg(self) -> DVector4: ...
    @property
    def rrrb(self) -> DVector4: ...
    @property
    def rrra(self) -> DVector4: ...
    @property
    def rrro(self) -> DVector4: ...
    @property
    def rrrl(self) -> DVector4: ...
    @property
    def rrgr(self) -> DVector4: ...
    @property
    def rrgg(self) -> DVector4: ...
    @property
    def rrgb(self) -> DVector4: ...
    @property
    def rrga(self) -> DVector4: ...
    @property
    def rrgo(self) -> DVector4: ...
    @property
    def rrgl(self) -> DVector4: ...
    @property
    def rrbr(self) -> DVector4: ...
    @property
    def rrbg(self) -> DVector4: ...
    @property
    def rrbb(self) -> DVector4: ...
    @property
    def rrba(self) -> DVector4: ...
    @property
    def rrbo(self) -> DVector4: ...
    @property
    def rrbl(self) -> DVector4: ...
    @property
    def rrar(self) -> DVector4: ...
    @property
    def rrag(self) -> DVector4: ...
    @property
    def rrab(self) -> DVector4: ...
    @property
    def rraa(self) -> DVector4: ...
    @property
    def rrao(self) -> DVector4: ...
    @property
    def rral(self) -> DVector4: ...
    @property
    def rror(self) -> DVector4: ...
    @property
    def rrog(self) -> DVector4: ...
    @property
    def rrob(self) -> DVector4: ...
    @property
    def rroa(self) -> DVector4: ...
    @property
    def rroo(self) -> DVector4: ...
    @property
    def rrol(self) -> DVector4: ...
    @property
    def rrlr(self) -> DVector4: ...
    @property
    def rrlg(self) -> DVector4: ...
    @property
    def rrlb(self) -> DVector4: ...
    @property
    def rrla(self) -> DVector4: ...
    @property
    def rrlo(self) -> DVector4: ...
    @property
    def rrll(self) -> DVector4: ...
    @property
    def rgrr(self) -> DVector4: ...
    @property
    def rgrg(self) -> DVector4: ...
    @property
    def rgrb(self) -> DVector4: ...
    @property
    def rgra(self) -> DVector4: ...
    @property
    def rgro(self) -> DVector4: ...
    @property
    def rgrl(self) -> DVector4: ...
    @property
    def rggr(self) -> DVector4: ...
    @property
    def rggg(self) -> DVector4: ...
    @property
    def rggb(self) -> DVector4: ...
    @property
    def rgga(self) -> DVector4: ...
    @property
    def rggo(self) -> DVector4: ...
    @property
    def rggl(self) -> DVector4: ...
    @property
    def rgbr(self) -> DVector4: ...
    @property
    def rgbg(self) -> DVector4: ...
    @property
    def rgbb(self) -> DVector4: ...
    @property
    def rgba(self) -> DVector4: ...
    @property
    def rgbo(self) -> DVector4: ...
    @property
    def rgbl(self) -> DVector4: ...
    @property
    def rgar(self) -> DVector4: ...
    @property
    def rgag(self) -> DVector4: ...
    @property
    def rgab(self) -> DVector4: ...
    @property
    def rgaa(self) -> DVector4: ...
    @property
    def rgao(self) -> DVector4: ...
    @property
    def rgal(self) -> DVector4: ...
    @property
    def rgor(self) -> DVector4: ...
    @property
    def rgog(self) -> DVector4: ...
    @property
    def rgob(self) -> DVector4: ...
    @property
    def rgoa(self) -> DVector4: ...
    @property
    def rgoo(self) -> DVector4: ...
    @property
    def rgol(self) -> DVector4: ...
    @property
    def rglr(self) -> DVector4: ...
    @property
    def rglg(self) -> DVector4: ...
    @property
    def rglb(self) -> DVector4: ...
    @property
    def rgla(self) -> DVector4: ...
    @property
    def rglo(self) -> DVector4: ...
    @property
    def rgll(self) -> DVector4: ...
    @property
    def rbrr(self) -> DVector4: ...
    @property
    def rbrg(self) -> DVector4: ...
    @property
    def rbrb(self) -> DVector4: ...
    @property
    def rbra(self) -> DVector4: ...
    @property
    def rbro(self) -> DVector4: ...
    @property
    def rbrl(self) -> DVector4: ...
    @property
    def rbgr(self) -> DVector4: ...
    @property
    def rbgg(self) -> DVector4: ...
    @property
    def rbgb(self) -> DVector4: ...
    @property
    def rbga(self) -> DVector4: ...
    @property
    def rbgo(self) -> DVector4: ...
    @property
    def rbgl(self) -> DVector4: ...
    @property
    def rbbr(self) -> DVector4: ...
    @property
    def rbbg(self) -> DVector4: ...
    @property
    def rbbb(self) -> DVector4: ...
    @property
    def rbba(self) -> DVector4: ...
    @property
    def rbbo(self) -> DVector4: ...
    @property
    def rbbl(self) -> DVector4: ...
    @property
    def rbar(self) -> DVector4: ...
    @property
    def rbag(self) -> DVector4: ...
    @property
    def rbab(self) -> DVector4: ...
    @property
    def rbaa(self) -> DVector4: ...
    @property
    def rbao(self) -> DVector4: ...
    @property
    def rbal(self) -> DVector4: ...
    @property
    def rbor(self) -> DVector4: ...
    @property
    def rbog(self) -> DVector4: ...
    @property
    def rbob(self) -> DVector4: ...
    @property
    def rboa(self) -> DVector4: ...
    @property
    def rboo(self) -> DVector4: ...
    @property
    def rbol(self) -> DVector4: ...
    @property
    def rblr(self) -> DVector4: ...
    @property
    def rblg(self) -> DVector4: ...
    @property
    def rblb(self) -> DVector4: ...
    @property
    def rbla(self) -> DVector4: ...
    @property
    def rblo(self) -> DVector4: ...
    @property
    def rbll(self) -> DVector4: ...
    @property
    def rarr(self) -> DVector4: ...
    @property
    def rarg(self) -> DVector4: ...
    @property
    def rarb(self) -> DVector4: ...
    @property
    def rara(self) -> DVector4: ...
    @property
    def raro(self) -> DVector4: ...
    @property
    def rarl(self) -> DVector4: ...
    @property
    def ragr(self) -> DVector4: ...
    @property
    def ragg(self) -> DVector4: ...
    @property
    def ragb(self) -> DVector4: ...
    @property
    def raga(self) -> DVector4: ...
    @property
    def rago(self) -> DVector4: ...
    @property
    def ragl(self) -> DVector4: ...
    @property
    def rabr(self) -> DVector4: ...
    @property
    def rabg(self) -> DVector4: ...
    @property
    def rabb(self) -> DVector4: ...
    @property
    def raba(self) -> DVector4: ...
    @property
    def rabo(self) -> DVector4: ...
    @property
    def rabl(self) -> DVector4: ...
    @property
    def raar(self) -> DVector4: ...
    @property
    def raag(self) -> DVector4: ...
    @property
    def raab(self) -> DVector4: ...
    @property
    def raaa(self) -> DVector4: ...
    @property
    def raao(self) -> DVector4: ...
    @property
    def raal(self) -> DVector4: ...
    @property
    def raor(self) -> DVector4: ...
    @property
    def raog(self) -> DVector4: ...
    @property
    def raob(self) -> DVector4: ...
    @property
    def raoa(self) -> DVector4: ...
    @property
    def raoo(self) -> DVector4: ...
    @property
    def raol(self) -> DVector4: ...
    @property
    def ralr(self) -> DVector4: ...
    @property
    def ralg(self) -> DVector4: ...
    @property
    def ralb(self) -> DVector4: ...
    @property
    def rala(self) -> DVector4: ...
    @property
    def ralo(self) -> DVector4: ...
    @property
    def rall(self) -> DVector4: ...
    @property
    def rorr(self) -> DVector4: ...
    @property
    def rorg(self) -> DVector4: ...
    @property
    def rorb(self) -> DVector4: ...
    @property
    def rora(self) -> DVector4: ...
    @property
    def roro(self) -> DVector4: ...
    @property
    def rorl(self) -> DVector4: ...
    @property
    def rogr(self) -> DVector4: ...
    @property
    def rogg(self) -> DVector4: ...
    @property
    def rogb(self) -> DVector4: ...
    @property
    def roga(self) -> DVector4: ...
    @property
    def rogo(self) -> DVector4: ...
    @property
    def rogl(self) -> DVector4: ...
    @property
    def robr(self) -> DVector4: ...
    @property
    def robg(self) -> DVector4: ...
    @property
    def robb(self) -> DVector4: ...
    @property
    def roba(self) -> DVector4: ...
    @property
    def robo(self) -> DVector4: ...
    @property
    def robl(self) -> DVector4: ...
    @property
    def roar(self) -> DVector4: ...
    @property
    def roag(self) -> DVector4: ...
    @property
    def roab(self) -> DVector4: ...
    @property
    def roaa(self) -> DVector4: ...
    @property
    def roao(self) -> DVector4: ...
    @property
    def roal(self) -> DVector4: ...
    @property
    def roor(self) -> DVector4: ...
    @property
    def roog(self) -> DVector4: ...
    @property
    def roob(self) -> DVector4: ...
    @property
    def rooa(self) -> DVector4: ...
    @property
    def rooo(self) -> DVector4: ...
    @property
    def rool(self) -> DVector4: ...
    @property
    def rolr(self) -> DVector4: ...
    @property
    def rolg(self) -> DVector4: ...
    @property
    def rolb(self) -> DVector4: ...
    @property
    def rola(self) -> DVector4: ...
    @property
    def rolo(self) -> DVector4: ...
    @property
    def roll(self) -> DVector4: ...
    @property
    def rlrr(self) -> DVector4: ...
    @property
    def rlrg(self) -> DVector4: ...
    @property
    def rlrb(self) -> DVector4: ...
    @property
    def rlra(self) -> DVector4: ...
    @property
    def rlro(self) -> DVector4: ...
    @property
    def rlrl(self) -> DVector4: ...
    @property
    def rlgr(self) -> DVector4: ...
    @property
    def rlgg(self) -> DVector4: ...
    @property
    def rlgb(self) -> DVector4: ...
    @property
    def rlga(self) -> DVector4: ...
    @property
    def rlgo(self) -> DVector4: ...
    @property
    def rlgl(self) -> DVector4: ...
    @property
    def rlbr(self) -> DVector4: ...
    @property
    def rlbg(self) -> DVector4: ...
    @property
    def rlbb(self) -> DVector4: ...
    @property
    def rlba(self) -> DVector4: ...
    @property
    def rlbo(self) -> DVector4: ...
    @property
    def rlbl(self) -> DVector4: ...
    @property
    def rlar(self) -> DVector4: ...
    @property
    def rlag(self) -> DVector4: ...
    @property
    def rlab(self) -> DVector4: ...
    @property
    def rlaa(self) -> DVector4: ...
    @property
    def rlao(self) -> DVector4: ...
    @property
    def rlal(self) -> DVector4: ...
    @property
    def rlor(self) -> DVector4: ...
    @property
    def rlog(self) -> DVector4: ...
    @property
    def rlob(self) -> DVector4: ...
    @property
    def rloa(self) -> DVector4: ...
    @property
    def rloo(self) -> DVector4: ...
    @property
    def rlol(self) -> DVector4: ...
    @property
    def rllr(self) -> DVector4: ...
    @property
    def rllg(self) -> DVector4: ...
    @property
    def rllb(self) -> DVector4: ...
    @property
    def rlla(self) -> DVector4: ...
    @property
    def rllo(self) -> DVector4: ...
    @property
    def rlll(self) -> DVector4: ...
    @property
    def grrr(self) -> DVector4: ...
    @property
    def grrg(self) -> DVector4: ...
    @property
    def grrb(self) -> DVector4: ...
    @property
    def grra(self) -> DVector4: ...
    @property
    def grro(self) -> DVector4: ...
    @property
    def grrl(self) -> DVector4: ...
    @property
    def grgr(self) -> DVector4: ...
    @property
    def grgg(self) -> DVector4: ...
    @property
    def grgb(self) -> DVector4: ...
    @property
    def grga(self) -> DVector4: ...
    @property
    def grgo(self) -> DVector4: ...
    @property
    def grgl(self) -> DVector4: ...
    @property
    def grbr(self) -> DVector4: ...
    @property
    def grbg(self) -> DVector4: ...
    @property
    def grbb(self) -> DVector4: ...
    @property
    def grba(self) -> DVector4: ...
    @property
    def grbo(self) -> DVector4: ...
    @property
    def grbl(self) -> DVector4: ...
    @property
    def grar(self) -> DVector4: ...
    @property
    def grag(self) -> DVector4: ...
    @property
    def grab(self) -> DVector4: ...
    @property
    def graa(self) -> DVector4: ...
    @property
    def grao(self) -> DVector4: ...
    @property
    def gral(self) -> DVector4: ...
    @property
    def gror(self) -> DVector4: ...
    @property
    def grog(self) -> DVector4: ...
    @property
    def grob(self) -> DVector4: ...
    @property
    def groa(self) -> DVector4: ...
    @property
    def groo(self) -> DVector4: ...
    @property
    def grol(self) -> DVector4: ...
    @property
    def grlr(self) -> DVector4: ...
    @property
    def grlg(self) -> DVector4: ...
    @property
    def grlb(self) -> DVector4: ...
    @property
    def grla(self) -> DVector4: ...
    @property
    def grlo(self) -> DVector4: ...
    @property
    def grll(self) -> DVector4: ...
    @property
    def ggrr(self) -> DVector4: ...
    @property
    def ggrg(self) -> DVector4: ...
    @property
    def ggrb(self) -> DVector4: ...
    @property
    def ggra(self) -> DVector4: ...
    @property
    def ggro(self) -> DVector4: ...
    @property
    def ggrl(self) -> DVector4: ...
    @property
    def gggr(self) -> DVector4: ...
    @property
    def gggg(self) -> DVector4: ...
    @property
    def gggb(self) -> DVector4: ...
    @property
    def ggga(self) -> DVector4: ...
    @property
    def gggo(self) -> DVector4: ...
    @property
    def gggl(self) -> DVector4: ...
    @property
    def ggbr(self) -> DVector4: ...
    @property
    def ggbg(self) -> DVector4: ...
    @property
    def ggbb(self) -> DVector4: ...
    @property
    def ggba(self) -> DVector4: ...
    @property
    def ggbo(self) -> DVector4: ...
    @property
    def ggbl(self) -> DVector4: ...
    @property
    def ggar(self) -> DVector4: ...
    @property
    def ggag(self) -> DVector4: ...
    @property
    def ggab(self) -> DVector4: ...
    @property
    def ggaa(self) -> DVector4: ...
    @property
    def ggao(self) -> DVector4: ...
    @property
    def ggal(self) -> DVector4: ...
    @property
    def ggor(self) -> DVector4: ...
    @property
    def ggog(self) -> DVector4: ...
    @property
    def ggob(self) -> DVector4: ...
    @property
    def ggoa(self) -> DVector4: ...
    @property
    def ggoo(self) -> DVector4: ...
    @property
    def ggol(self) -> DVector4: ...
    @property
    def gglr(self) -> DVector4: ...
    @property
    def gglg(self) -> DVector4: ...
    @property
    def gglb(self) -> DVector4: ...
    @property
    def ggla(self) -> DVector4: ...
    @property
    def gglo(self) -> DVector4: ...
    @property
    def ggll(self) -> DVector4: ...
    @property
    def gbrr(self) -> DVector4: ...
    @property
    def gbrg(self) -> DVector4: ...
    @property
    def gbrb(self) -> DVector4: ...
    @property
    def gbra(self) -> DVector4: ...
    @property
    def gbro(self) -> DVector4: ...
    @property
    def gbrl(self) -> DVector4: ...
    @property
    def gbgr(self) -> DVector4: ...
    @property
    def gbgg(self) -> DVector4: ...
    @property
    def gbgb(self) -> DVector4: ...
    @property
    def gbga(self) -> DVector4: ...
    @property
    def gbgo(self) -> DVector4: ...
    @property
    def gbgl(self) -> DVector4: ...
    @property
    def gbbr(self) -> DVector4: ...
    @property
    def gbbg(self) -> DVector4: ...
    @property
    def gbbb(self) -> DVector4: ...
    @property
    def gbba(self) -> DVector4: ...
    @property
    def gbbo(self) -> DVector4: ...
    @property
    def gbbl(self) -> DVector4: ...
    @property
    def gbar(self) -> DVector4: ...
    @property
    def gbag(self) -> DVector4: ...
    @property
    def gbab(self) -> DVector4: ...
    @property
    def gbaa(self) -> DVector4: ...
    @property
    def gbao(self) -> DVector4: ...
    @property
    def gbal(self) -> DVector4: ...
    @property
    def gbor(self) -> DVector4: ...
    @property
    def gbog(self) -> DVector4: ...
    @property
    def gbob(self) -> DVector4: ...
    @property
    def gboa(self) -> DVector4: ...
    @property
    def gboo(self) -> DVector4: ...
    @property
    def gbol(self) -> DVector4: ...
    @property
    def gblr(self) -> DVector4: ...
    @property
    def gblg(self) -> DVector4: ...
    @property
    def gblb(self) -> DVector4: ...
    @property
    def gbla(self) -> DVector4: ...
    @property
    def gblo(self) -> DVector4: ...
    @property
    def gbll(self) -> DVector4: ...
    @property
    def garr(self) -> DVector4: ...
    @property
    def garg(self) -> DVector4: ...
    @property
    def garb(self) -> DVector4: ...
    @property
    def gara(self) -> DVector4: ...
    @property
    def garo(self) -> DVector4: ...
    @property
    def garl(self) -> DVector4: ...
    @property
    def gagr(self) -> DVector4: ...
    @property
    def gagg(self) -> DVector4: ...
    @property
    def gagb(self) -> DVector4: ...
    @property
    def gaga(self) -> DVector4: ...
    @property
    def gago(self) -> DVector4: ...
    @property
    def gagl(self) -> DVector4: ...
    @property
    def gabr(self) -> DVector4: ...
    @property
    def gabg(self) -> DVector4: ...
    @property
    def gabb(self) -> DVector4: ...
    @property
    def gaba(self) -> DVector4: ...
    @property
    def gabo(self) -> DVector4: ...
    @property
    def gabl(self) -> DVector4: ...
    @property
    def gaar(self) -> DVector4: ...
    @property
    def gaag(self) -> DVector4: ...
    @property
    def gaab(self) -> DVector4: ...
    @property
    def gaaa(self) -> DVector4: ...
    @property
    def gaao(self) -> DVector4: ...
    @property
    def gaal(self) -> DVector4: ...
    @property
    def gaor(self) -> DVector4: ...
    @property
    def gaog(self) -> DVector4: ...
    @property
    def gaob(self) -> DVector4: ...
    @property
    def gaoa(self) -> DVector4: ...
    @property
    def gaoo(self) -> DVector4: ...
    @property
    def gaol(self) -> DVector4: ...
    @property
    def galr(self) -> DVector4: ...
    @property
    def galg(self) -> DVector4: ...
    @property
    def galb(self) -> DVector4: ...
    @property
    def gala(self) -> DVector4: ...
    @property
    def galo(self) -> DVector4: ...
    @property
    def gall(self) -> DVector4: ...
    @property
    def gorr(self) -> DVector4: ...
    @property
    def gorg(self) -> DVector4: ...
    @property
    def gorb(self) -> DVector4: ...
    @property
    def gora(self) -> DVector4: ...
    @property
    def goro(self) -> DVector4: ...
    @property
    def gorl(self) -> DVector4: ...
    @property
    def gogr(self) -> DVector4: ...
    @property
    def gogg(self) -> DVector4: ...
    @property
    def gogb(self) -> DVector4: ...
    @property
    def goga(self) -> DVector4: ...
    @property
    def gogo(self) -> DVector4: ...
    @property
    def gogl(self) -> DVector4: ...
    @property
    def gobr(self) -> DVector4: ...
    @property
    def gobg(self) -> DVector4: ...
    @property
    def gobb(self) -> DVector4: ...
    @property
    def goba(self) -> DVector4: ...
    @property
    def gobo(self) -> DVector4: ...
    @property
    def gobl(self) -> DVector4: ...
    @property
    def goar(self) -> DVector4: ...
    @property
    def goag(self) -> DVector4: ...
    @property
    def goab(self) -> DVector4: ...
    @property
    def goaa(self) -> DVector4: ...
    @property
    def goao(self) -> DVector4: ...
    @property
    def goal(self) -> DVector4: ...
    @property
    def goor(self) -> DVector4: ...
    @property
    def goog(self) -> DVector4: ...
    @property
    def goob(self) -> DVector4: ...
    @property
    def gooa(self) -> DVector4: ...
    @property
    def gooo(self) -> DVector4: ...
    @property
    def gool(self) -> DVector4: ...
    @property
    def golr(self) -> DVector4: ...
    @property
    def golg(self) -> DVector4: ...
    @property
    def golb(self) -> DVector4: ...
    @property
    def gola(self) -> DVector4: ...
    @property
    def golo(self) -> DVector4: ...
    @property
    def goll(self) -> DVector4: ...
    @property
    def glrr(self) -> DVector4: ...
    @property
    def glrg(self) -> DVector4: ...
    @property
    def glrb(self) -> DVector4: ...
    @property
    def glra(self) -> DVector4: ...
    @property
    def glro(self) -> DVector4: ...
    @property
    def glrl(self) -> DVector4: ...
    @property
    def glgr(self) -> DVector4: ...
    @property
    def glgg(self) -> DVector4: ...
    @property
    def glgb(self) -> DVector4: ...
    @property
    def glga(self) -> DVector4: ...
    @property
    def glgo(self) -> DVector4: ...
    @property
    def glgl(self) -> DVector4: ...
    @property
    def glbr(self) -> DVector4: ...
    @property
    def glbg(self) -> DVector4: ...
    @property
    def glbb(self) -> DVector4: ...
    @property
    def glba(self) -> DVector4: ...
    @property
    def glbo(self) -> DVector4: ...
    @property
    def glbl(self) -> DVector4: ...
    @property
    def glar(self) -> DVector4: ...
    @property
    def glag(self) -> DVector4: ...
    @property
    def glab(self) -> DVector4: ...
    @property
    def glaa(self) -> DVector4: ...
    @property
    def glao(self) -> DVector4: ...
    @property
    def glal(self) -> DVector4: ...
    @property
    def glor(self) -> DVector4: ...
    @property
    def glog(self) -> DVector4: ...
    @property
    def glob(self) -> DVector4: ...
    @property
    def gloa(self) -> DVector4: ...
    @property
    def gloo(self) -> DVector4: ...
    @property
    def glol(self) -> DVector4: ...
    @property
    def gllr(self) -> DVector4: ...
    @property
    def gllg(self) -> DVector4: ...
    @property
    def gllb(self) -> DVector4: ...
    @property
    def glla(self) -> DVector4: ...
    @property
    def gllo(self) -> DVector4: ...
    @property
    def glll(self) -> DVector4: ...
    @property
    def brrr(self) -> DVector4: ...
    @property
    def brrg(self) -> DVector4: ...
    @property
    def brrb(self) -> DVector4: ...
    @property
    def brra(self) -> DVector4: ...
    @property
    def brro(self) -> DVector4: ...
    @property
    def brrl(self) -> DVector4: ...
    @property
    def brgr(self) -> DVector4: ...
    @property
    def brgg(self) -> DVector4: ...
    @property
    def brgb(self) -> DVector4: ...
    @property
    def brga(self) -> DVector4: ...
    @property
    def brgo(self) -> DVector4: ...
    @property
    def brgl(self) -> DVector4: ...
    @property
    def brbr(self) -> DVector4: ...
    @property
    def brbg(self) -> DVector4: ...
    @property
    def brbb(self) -> DVector4: ...
    @property
    def brba(self) -> DVector4: ...
    @property
    def brbo(self) -> DVector4: ...
    @property
    def brbl(self) -> DVector4: ...
    @property
    def brar(self) -> DVector4: ...
    @property
    def brag(self) -> DVector4: ...
    @property
    def brab(self) -> DVector4: ...
    @property
    def braa(self) -> DVector4: ...
    @property
    def brao(self) -> DVector4: ...
    @property
    def bral(self) -> DVector4: ...
    @property
    def bror(self) -> DVector4: ...
    @property
    def brog(self) -> DVector4: ...
    @property
    def brob(self) -> DVector4: ...
    @property
    def broa(self) -> DVector4: ...
    @property
    def broo(self) -> DVector4: ...
    @property
    def brol(self) -> DVector4: ...
    @property
    def brlr(self) -> DVector4: ...
    @property
    def brlg(self) -> DVector4: ...
    @property
    def brlb(self) -> DVector4: ...
    @property
    def brla(self) -> DVector4: ...
    @property
    def brlo(self) -> DVector4: ...
    @property
    def brll(self) -> DVector4: ...
    @property
    def bgrr(self) -> DVector4: ...
    @property
    def bgrg(self) -> DVector4: ...
    @property
    def bgrb(self) -> DVector4: ...
    @property
    def bgra(self) -> DVector4: ...
    @property
    def bgro(self) -> DVector4: ...
    @property
    def bgrl(self) -> DVector4: ...
    @property
    def bggr(self) -> DVector4: ...
    @property
    def bggg(self) -> DVector4: ...
    @property
    def bggb(self) -> DVector4: ...
    @property
    def bgga(self) -> DVector4: ...
    @property
    def bggo(self) -> DVector4: ...
    @property
    def bggl(self) -> DVector4: ...
    @property
    def bgbr(self) -> DVector4: ...
    @property
    def bgbg(self) -> DVector4: ...
    @property
    def bgbb(self) -> DVector4: ...
    @property
    def bgba(self) -> DVector4: ...
    @property
    def bgbo(self) -> DVector4: ...
    @property
    def bgbl(self) -> DVector4: ...
    @property
    def bgar(self) -> DVector4: ...
    @property
    def bgag(self) -> DVector4: ...
    @property
    def bgab(self) -> DVector4: ...
    @property
    def bgaa(self) -> DVector4: ...
    @property
    def bgao(self) -> DVector4: ...
    @property
    def bgal(self) -> DVector4: ...
    @property
    def bgor(self) -> DVector4: ...
    @property
    def bgog(self) -> DVector4: ...
    @property
    def bgob(self) -> DVector4: ...
    @property
    def bgoa(self) -> DVector4: ...
    @property
    def bgoo(self) -> DVector4: ...
    @property
    def bgol(self) -> DVector4: ...
    @property
    def bglr(self) -> DVector4: ...
    @property
    def bglg(self) -> DVector4: ...
    @property
    def bglb(self) -> DVector4: ...
    @property
    def bgla(self) -> DVector4: ...
    @property
    def bglo(self) -> DVector4: ...
    @property
    def bgll(self) -> DVector4: ...
    @property
    def bbrr(self) -> DVector4: ...
    @property
    def bbrg(self) -> DVector4: ...
    @property
    def bbrb(self) -> DVector4: ...
    @property
    def bbra(self) -> DVector4: ...
    @property
    def bbro(self) -> DVector4: ...
    @property
    def bbrl(self) -> DVector4: ...
    @property
    def bbgr(self) -> DVector4: ...
    @property
    def bbgg(self) -> DVector4: ...
    @property
    def bbgb(self) -> DVector4: ...
    @property
    def bbga(self) -> DVector4: ...
    @property
    def bbgo(self) -> DVector4: ...
    @property
    def bbgl(self) -> DVector4: ...
    @property
    def bbbr(self) -> DVector4: ...
    @property
    def bbbg(self) -> DVector4: ...
    @property
    def bbbb(self) -> DVector4: ...
    @property
    def bbba(self) -> DVector4: ...
    @property
    def bbbo(self) -> DVector4: ...
    @property
    def bbbl(self) -> DVector4: ...
    @property
    def bbar(self) -> DVector4: ...
    @property
    def bbag(self) -> DVector4: ...
    @property
    def bbab(self) -> DVector4: ...
    @property
    def bbaa(self) -> DVector4: ...
    @property
    def bbao(self) -> DVector4: ...
    @property
    def bbal(self) -> DVector4: ...
    @property
    def bbor(self) -> DVector4: ...
    @property
    def bbog(self) -> DVector4: ...
    @property
    def bbob(self) -> DVector4: ...
    @property
    def bboa(self) -> DVector4: ...
    @property
    def bboo(self) -> DVector4: ...
    @property
    def bbol(self) -> DVector4: ...
    @property
    def bblr(self) -> DVector4: ...
    @property
    def bblg(self) -> DVector4: ...
    @property
    def bblb(self) -> DVector4: ...
    @property
    def bbla(self) -> DVector4: ...
    @property
    def bblo(self) -> DVector4: ...
    @property
    def bbll(self) -> DVector4: ...
    @property
    def barr(self) -> DVector4: ...
    @property
    def barg(self) -> DVector4: ...
    @property
    def barb(self) -> DVector4: ...
    @property
    def bara(self) -> DVector4: ...
    @property
    def baro(self) -> DVector4: ...
    @property
    def barl(self) -> DVector4: ...
    @property
    def bagr(self) -> DVector4: ...
    @property
    def bagg(self) -> DVector4: ...
    @property
    def bagb(self) -> DVector4: ...
    @property
    def baga(self) -> DVector4: ...
    @property
    def bago(self) -> DVector4: ...
    @property
    def bagl(self) -> DVector4: ...
    @property
    def babr(self) -> DVector4: ...
    @property
    def babg(self) -> DVector4: ...
    @property
    def babb(self) -> DVector4: ...
    @property
    def baba(self) -> DVector4: ...
    @property
    def babo(self) -> DVector4: ...
    @property
    def babl(self) -> DVector4: ...
    @property
    def baar(self) -> DVector4: ...
    @property
    def baag(self) -> DVector4: ...
    @property
    def baab(self) -> DVector4: ...
    @property
    def baaa(self) -> DVector4: ...
    @property
    def baao(self) -> DVector4: ...
    @property
    def baal(self) -> DVector4: ...
    @property
    def baor(self) -> DVector4: ...
    @property
    def baog(self) -> DVector4: ...
    @property
    def baob(self) -> DVector4: ...
    @property
    def baoa(self) -> DVector4: ...
    @property
    def baoo(self) -> DVector4: ...
    @property
    def baol(self) -> DVector4: ...
    @property
    def balr(self) -> DVector4: ...
    @property
    def balg(self) -> DVector4: ...
    @property
    def balb(self) -> DVector4: ...
    @property
    def bala(self) -> DVector4: ...
    @property
    def balo(self) -> DVector4: ...
    @property
    def ball(self) -> DVector4: ...
    @property
    def borr(self) -> DVector4: ...
    @property
    def borg(self) -> DVector4: ...
    @property
    def borb(self) -> DVector4: ...
    @property
    def bora(self) -> DVector4: ...
    @property
    def boro(self) -> DVector4: ...
    @property
    def borl(self) -> DVector4: ...
    @property
    def bogr(self) -> DVector4: ...
    @property
    def bogg(self) -> DVector4: ...
    @property
    def bogb(self) -> DVector4: ...
    @property
    def boga(self) -> DVector4: ...
    @property
    def bogo(self) -> DVector4: ...
    @property
    def bogl(self) -> DVector4: ...
    @property
    def bobr(self) -> DVector4: ...
    @property
    def bobg(self) -> DVector4: ...
    @property
    def bobb(self) -> DVector4: ...
    @property
    def boba(self) -> DVector4: ...
    @property
    def bobo(self) -> DVector4: ...
    @property
    def bobl(self) -> DVector4: ...
    @property
    def boar(self) -> DVector4: ...
    @property
    def boag(self) -> DVector4: ...
    @property
    def boab(self) -> DVector4: ...
    @property
    def boaa(self) -> DVector4: ...
    @property
    def boao(self) -> DVector4: ...
    @property
    def boal(self) -> DVector4: ...
    @property
    def boor(self) -> DVector4: ...
    @property
    def boog(self) -> DVector4: ...
    @property
    def boob(self) -> DVector4: ...
    @property
    def booa(self) -> DVector4: ...
    @property
    def booo(self) -> DVector4: ...
    @property
    def bool(self) -> DVector4: ...
    @property
    def bolr(self) -> DVector4: ...
    @property
    def bolg(self) -> DVector4: ...
    @property
    def bolb(self) -> DVector4: ...
    @property
    def bola(self) -> DVector4: ...
    @property
    def bolo(self) -> DVector4: ...
    @property
    def boll(self) -> DVector4: ...
    @property
    def blrr(self) -> DVector4: ...
    @property
    def blrg(self) -> DVector4: ...
    @property
    def blrb(self) -> DVector4: ...
    @property
    def blra(self) -> DVector4: ...
    @property
    def blro(self) -> DVector4: ...
    @property
    def blrl(self) -> DVector4: ...
    @property
    def blgr(self) -> DVector4: ...
    @property
    def blgg(self) -> DVector4: ...
    @property
    def blgb(self) -> DVector4: ...
    @property
    def blga(self) -> DVector4: ...
    @property
    def blgo(self) -> DVector4: ...
    @property
    def blgl(self) -> DVector4: ...
    @property
    def blbr(self) -> DVector4: ...
    @property
    def blbg(self) -> DVector4: ...
    @property
    def blbb(self) -> DVector4: ...
    @property
    def blba(self) -> DVector4: ...
    @property
    def blbo(self) -> DVector4: ...
    @property
    def blbl(self) -> DVector4: ...
    @property
    def blar(self) -> DVector4: ...
    @property
    def blag(self) -> DVector4: ...
    @property
    def blab(self) -> DVector4: ...
    @property
    def blaa(self) -> DVector4: ...
    @property
    def blao(self) -> DVector4: ...
    @property
    def blal(self) -> DVector4: ...
    @property
    def blor(self) -> DVector4: ...
    @property
    def blog(self) -> DVector4: ...
    @property
    def blob(self) -> DVector4: ...
    @property
    def bloa(self) -> DVector4: ...
    @property
    def bloo(self) -> DVector4: ...
    @property
    def blol(self) -> DVector4: ...
    @property
    def bllr(self) -> DVector4: ...
    @property
    def bllg(self) -> DVector4: ...
    @property
    def bllb(self) -> DVector4: ...
    @property
    def blla(self) -> DVector4: ...
    @property
    def bllo(self) -> DVector4: ...
    @property
    def blll(self) -> DVector4: ...
    @property
    def arrr(self) -> DVector4: ...
    @property
    def arrg(self) -> DVector4: ...
    @property
    def arrb(self) -> DVector4: ...
    @property
    def arra(self) -> DVector4: ...
    @property
    def arro(self) -> DVector4: ...
    @property
    def arrl(self) -> DVector4: ...
    @property
    def argr(self) -> DVector4: ...
    @property
    def argg(self) -> DVector4: ...
    @property
    def argb(self) -> DVector4: ...
    @property
    def arga(self) -> DVector4: ...
    @property
    def argo(self) -> DVector4: ...
    @property
    def argl(self) -> DVector4: ...
    @property
    def arbr(self) -> DVector4: ...
    @property
    def arbg(self) -> DVector4: ...
    @property
    def arbb(self) -> DVector4: ...
    @property
    def arba(self) -> DVector4: ...
    @property
    def arbo(self) -> DVector4: ...
    @property
    def arbl(self) -> DVector4: ...
    @property
    def arar(self) -> DVector4: ...
    @property
    def arag(self) -> DVector4: ...
    @property
    def arab(self) -> DVector4: ...
    @property
    def araa(self) -> DVector4: ...
    @property
    def arao(self) -> DVector4: ...
    @property
    def aral(self) -> DVector4: ...
    @property
    def aror(self) -> DVector4: ...
    @property
    def arog(self) -> DVector4: ...
    @property
    def arob(self) -> DVector4: ...
    @property
    def aroa(self) -> DVector4: ...
    @property
    def aroo(self) -> DVector4: ...
    @property
    def arol(self) -> DVector4: ...
    @property
    def arlr(self) -> DVector4: ...
    @property
    def arlg(self) -> DVector4: ...
    @property
    def arlb(self) -> DVector4: ...
    @property
    def arla(self) -> DVector4: ...
    @property
    def arlo(self) -> DVector4: ...
    @property
    def arll(self) -> DVector4: ...
    @property
    def agrr(self) -> DVector4: ...
    @property
    def agrg(self) -> DVector4: ...
    @property
    def agrb(self) -> DVector4: ...
    @property
    def agra(self) -> DVector4: ...
    @property
    def agro(self) -> DVector4: ...
    @property
    def agrl(self) -> DVector4: ...
    @property
    def aggr(self) -> DVector4: ...
    @property
    def aggg(self) -> DVector4: ...
    @property
    def aggb(self) -> DVector4: ...
    @property
    def agga(self) -> DVector4: ...
    @property
    def aggo(self) -> DVector4: ...
    @property
    def aggl(self) -> DVector4: ...
    @property
    def agbr(self) -> DVector4: ...
    @property
    def agbg(self) -> DVector4: ...
    @property
    def agbb(self) -> DVector4: ...
    @property
    def agba(self) -> DVector4: ...
    @property
    def agbo(self) -> DVector4: ...
    @property
    def agbl(self) -> DVector4: ...
    @property
    def agar(self) -> DVector4: ...
    @property
    def agag(self) -> DVector4: ...
    @property
    def agab(self) -> DVector4: ...
    @property
    def agaa(self) -> DVector4: ...
    @property
    def agao(self) -> DVector4: ...
    @property
    def agal(self) -> DVector4: ...
    @property
    def agor(self) -> DVector4: ...
    @property
    def agog(self) -> DVector4: ...
    @property
    def agob(self) -> DVector4: ...
    @property
    def agoa(self) -> DVector4: ...
    @property
    def agoo(self) -> DVector4: ...
    @property
    def agol(self) -> DVector4: ...
    @property
    def aglr(self) -> DVector4: ...
    @property
    def aglg(self) -> DVector4: ...
    @property
    def aglb(self) -> DVector4: ...
    @property
    def agla(self) -> DVector4: ...
    @property
    def aglo(self) -> DVector4: ...
    @property
    def agll(self) -> DVector4: ...
    @property
    def abrr(self) -> DVector4: ...
    @property
    def abrg(self) -> DVector4: ...
    @property
    def abrb(self) -> DVector4: ...
    @property
    def abra(self) -> DVector4: ...
    @property
    def abro(self) -> DVector4: ...
    @property
    def abrl(self) -> DVector4: ...
    @property
    def abgr(self) -> DVector4: ...
    @property
    def abgg(self) -> DVector4: ...
    @property
    def abgb(self) -> DVector4: ...
    @property
    def abga(self) -> DVector4: ...
    @property
    def abgo(self) -> DVector4: ...
    @property
    def abgl(self) -> DVector4: ...
    @property
    def abbr(self) -> DVector4: ...
    @property
    def abbg(self) -> DVector4: ...
    @property
    def abbb(self) -> DVector4: ...
    @property
    def abba(self) -> DVector4: ...
    @property
    def abbo(self) -> DVector4: ...
    @property
    def abbl(self) -> DVector4: ...
    @property
    def abar(self) -> DVector4: ...
    @property
    def abag(self) -> DVector4: ...
    @property
    def abab(self) -> DVector4: ...
    @property
    def abaa(self) -> DVector4: ...
    @property
    def abao(self) -> DVector4: ...
    @property
    def abal(self) -> DVector4: ...
    @property
    def abor(self) -> DVector4: ...
    @property
    def abog(self) -> DVector4: ...
    @property
    def abob(self) -> DVector4: ...
    @property
    def aboa(self) -> DVector4: ...
    @property
    def aboo(self) -> DVector4: ...
    @property
    def abol(self) -> DVector4: ...
    @property
    def ablr(self) -> DVector4: ...
    @property
    def ablg(self) -> DVector4: ...
    @property
    def ablb(self) -> DVector4: ...
    @property
    def abla(self) -> DVector4: ...
    @property
    def ablo(self) -> DVector4: ...
    @property
    def abll(self) -> DVector4: ...
    @property
    def aarr(self) -> DVector4: ...
    @property
    def aarg(self) -> DVector4: ...
    @property
    def aarb(self) -> DVector4: ...
    @property
    def aara(self) -> DVector4: ...
    @property
    def aaro(self) -> DVector4: ...
    @property
    def aarl(self) -> DVector4: ...
    @property
    def aagr(self) -> DVector4: ...
    @property
    def aagg(self) -> DVector4: ...
    @property
    def aagb(self) -> DVector4: ...
    @property
    def aaga(self) -> DVector4: ...
    @property
    def aago(self) -> DVector4: ...
    @property
    def aagl(self) -> DVector4: ...
    @property
    def aabr(self) -> DVector4: ...
    @property
    def aabg(self) -> DVector4: ...
    @property
    def aabb(self) -> DVector4: ...
    @property
    def aaba(self) -> DVector4: ...
    @property
    def aabo(self) -> DVector4: ...
    @property
    def aabl(self) -> DVector4: ...
    @property
    def aaar(self) -> DVector4: ...
    @property
    def aaag(self) -> DVector4: ...
    @property
    def aaab(self) -> DVector4: ...
    @property
    def aaaa(self) -> DVector4: ...
    @property
    def aaao(self) -> DVector4: ...
    @property
    def aaal(self) -> DVector4: ...
    @property
    def aaor(self) -> DVector4: ...
    @property
    def aaog(self) -> DVector4: ...
    @property
    def aaob(self) -> DVector4: ...
    @property
    def aaoa(self) -> DVector4: ...
    @property
    def aaoo(self) -> DVector4: ...
    @property
    def aaol(self) -> DVector4: ...
    @property
    def aalr(self) -> DVector4: ...
    @property
    def aalg(self) -> DVector4: ...
    @property
    def aalb(self) -> DVector4: ...
    @property
    def aala(self) -> DVector4: ...
    @property
    def aalo(self) -> DVector4: ...
    @property
    def aall(self) -> DVector4: ...
    @property
    def aorr(self) -> DVector4: ...
    @property
    def aorg(self) -> DVector4: ...
    @property
    def aorb(self) -> DVector4: ...
    @property
    def aora(self) -> DVector4: ...
    @property
    def aoro(self) -> DVector4: ...
    @property
    def aorl(self) -> DVector4: ...
    @property
    def aogr(self) -> DVector4: ...
    @property
    def aogg(self) -> DVector4: ...
    @property
    def aogb(self) -> DVector4: ...
    @property
    def aoga(self) -> DVector4: ...
    @property
    def aogo(self) -> DVector4: ...
    @property
    def aogl(self) -> DVector4: ...
    @property
    def aobr(self) -> DVector4: ...
    @property
    def aobg(self) -> DVector4: ...
    @property
    def aobb(self) -> DVector4: ...
    @property
    def aoba(self) -> DVector4: ...
    @property
    def aobo(self) -> DVector4: ...
    @property
    def aobl(self) -> DVector4: ...
    @property
    def aoar(self) -> DVector4: ...
    @property
    def aoag(self) -> DVector4: ...
    @property
    def aoab(self) -> DVector4: ...
    @property
    def aoaa(self) -> DVector4: ...
    @property
    def aoao(self) -> DVector4: ...
    @property
    def aoal(self) -> DVector4: ...
    @property
    def aoor(self) -> DVector4: ...
    @property
    def aoog(self) -> DVector4: ...
    @property
    def aoob(self) -> DVector4: ...
    @property
    def aooa(self) -> DVector4: ...
    @property
    def aooo(self) -> DVector4: ...
    @property
    def aool(self) -> DVector4: ...
    @property
    def aolr(self) -> DVector4: ...
    @property
    def aolg(self) -> DVector4: ...
    @property
    def aolb(self) -> DVector4: ...
    @property
    def aola(self) -> DVector4: ...
    @property
    def aolo(self) -> DVector4: ...
    @property
    def aoll(self) -> DVector4: ...
    @property
    def alrr(self) -> DVector4: ...
    @property
    def alrg(self) -> DVector4: ...
    @property
    def alrb(self) -> DVector4: ...
    @property
    def alra(self) -> DVector4: ...
    @property
    def alro(self) -> DVector4: ...
    @property
    def alrl(self) -> DVector4: ...
    @property
    def algr(self) -> DVector4: ...
    @property
    def algg(self) -> DVector4: ...
    @property
    def algb(self) -> DVector4: ...
    @property
    def alga(self) -> DVector4: ...
    @property
    def algo(self) -> DVector4: ...
    @property
    def algl(self) -> DVector4: ...
    @property
    def albr(self) -> DVector4: ...
    @property
    def albg(self) -> DVector4: ...
    @property
    def albb(self) -> DVector4: ...
    @property
    def alba(self) -> DVector4: ...
    @property
    def albo(self) -> DVector4: ...
    @property
    def albl(self) -> DVector4: ...
    @property
    def alar(self) -> DVector4: ...
    @property
    def alag(self) -> DVector4: ...
    @property
    def alab(self) -> DVector4: ...
    @property
    def alaa(self) -> DVector4: ...
    @property
    def alao(self) -> DVector4: ...
    @property
    def alal(self) -> DVector4: ...
    @property
    def alor(self) -> DVector4: ...
    @property
    def alog(self) -> DVector4: ...
    @property
    def alob(self) -> DVector4: ...
    @property
    def aloa(self) -> DVector4: ...
    @property
    def aloo(self) -> DVector4: ...
    @property
    def alol(self) -> DVector4: ...
    @property
    def allr(self) -> DVector4: ...
    @property
    def allg(self) -> DVector4: ...
    @property
    def allb(self) -> DVector4: ...
    @property
    def alla(self) -> DVector4: ...
    @property
    def allo(self) -> DVector4: ...
    @property
    def alll(self) -> DVector4: ...
    @property
    def orrr(self) -> DVector4: ...
    @property
    def orrg(self) -> DVector4: ...
    @property
    def orrb(self) -> DVector4: ...
    @property
    def orra(self) -> DVector4: ...
    @property
    def orro(self) -> DVector4: ...
    @property
    def orrl(self) -> DVector4: ...
    @property
    def orgr(self) -> DVector4: ...
    @property
    def orgg(self) -> DVector4: ...
    @property
    def orgb(self) -> DVector4: ...
    @property
    def orga(self) -> DVector4: ...
    @property
    def orgo(self) -> DVector4: ...
    @property
    def orgl(self) -> DVector4: ...
    @property
    def orbr(self) -> DVector4: ...
    @property
    def orbg(self) -> DVector4: ...
    @property
    def orbb(self) -> DVector4: ...
    @property
    def orba(self) -> DVector4: ...
    @property
    def orbo(self) -> DVector4: ...
    @property
    def orbl(self) -> DVector4: ...
    @property
    def orar(self) -> DVector4: ...
    @property
    def orag(self) -> DVector4: ...
    @property
    def orab(self) -> DVector4: ...
    @property
    def oraa(self) -> DVector4: ...
    @property
    def orao(self) -> DVector4: ...
    @property
    def oral(self) -> DVector4: ...
    @property
    def oror(self) -> DVector4: ...
    @property
    def orog(self) -> DVector4: ...
    @property
    def orob(self) -> DVector4: ...
    @property
    def oroa(self) -> DVector4: ...
    @property
    def oroo(self) -> DVector4: ...
    @property
    def orol(self) -> DVector4: ...
    @property
    def orlr(self) -> DVector4: ...
    @property
    def orlg(self) -> DVector4: ...
    @property
    def orlb(self) -> DVector4: ...
    @property
    def orla(self) -> DVector4: ...
    @property
    def orlo(self) -> DVector4: ...
    @property
    def orll(self) -> DVector4: ...
    @property
    def ogrr(self) -> DVector4: ...
    @property
    def ogrg(self) -> DVector4: ...
    @property
    def ogrb(self) -> DVector4: ...
    @property
    def ogra(self) -> DVector4: ...
    @property
    def ogro(self) -> DVector4: ...
    @property
    def ogrl(self) -> DVector4: ...
    @property
    def oggr(self) -> DVector4: ...
    @property
    def oggg(self) -> DVector4: ...
    @property
    def oggb(self) -> DVector4: ...
    @property
    def ogga(self) -> DVector4: ...
    @property
    def oggo(self) -> DVector4: ...
    @property
    def oggl(self) -> DVector4: ...
    @property
    def ogbr(self) -> DVector4: ...
    @property
    def ogbg(self) -> DVector4: ...
    @property
    def ogbb(self) -> DVector4: ...
    @property
    def ogba(self) -> DVector4: ...
    @property
    def ogbo(self) -> DVector4: ...
    @property
    def ogbl(self) -> DVector4: ...
    @property
    def ogar(self) -> DVector4: ...
    @property
    def ogag(self) -> DVector4: ...
    @property
    def ogab(self) -> DVector4: ...
    @property
    def ogaa(self) -> DVector4: ...
    @property
    def ogao(self) -> DVector4: ...
    @property
    def ogal(self) -> DVector4: ...
    @property
    def ogor(self) -> DVector4: ...
    @property
    def ogog(self) -> DVector4: ...
    @property
    def ogob(self) -> DVector4: ...
    @property
    def ogoa(self) -> DVector4: ...
    @property
    def ogoo(self) -> DVector4: ...
    @property
    def ogol(self) -> DVector4: ...
    @property
    def oglr(self) -> DVector4: ...
    @property
    def oglg(self) -> DVector4: ...
    @property
    def oglb(self) -> DVector4: ...
    @property
    def ogla(self) -> DVector4: ...
    @property
    def oglo(self) -> DVector4: ...
    @property
    def ogll(self) -> DVector4: ...
    @property
    def obrr(self) -> DVector4: ...
    @property
    def obrg(self) -> DVector4: ...
    @property
    def obrb(self) -> DVector4: ...
    @property
    def obra(self) -> DVector4: ...
    @property
    def obro(self) -> DVector4: ...
    @property
    def obrl(self) -> DVector4: ...
    @property
    def obgr(self) -> DVector4: ...
    @property
    def obgg(self) -> DVector4: ...
    @property
    def obgb(self) -> DVector4: ...
    @property
    def obga(self) -> DVector4: ...
    @property
    def obgo(self) -> DVector4: ...
    @property
    def obgl(self) -> DVector4: ...
    @property
    def obbr(self) -> DVector4: ...
    @property
    def obbg(self) -> DVector4: ...
    @property
    def obbb(self) -> DVector4: ...
    @property
    def obba(self) -> DVector4: ...
    @property
    def obbo(self) -> DVector4: ...
    @property
    def obbl(self) -> DVector4: ...
    @property
    def obar(self) -> DVector4: ...
    @property
    def obag(self) -> DVector4: ...
    @property
    def obab(self) -> DVector4: ...
    @property
    def obaa(self) -> DVector4: ...
    @property
    def obao(self) -> DVector4: ...
    @property
    def obal(self) -> DVector4: ...
    @property
    def obor(self) -> DVector4: ...
    @property
    def obog(self) -> DVector4: ...
    @property
    def obob(self) -> DVector4: ...
    @property
    def oboa(self) -> DVector4: ...
    @property
    def oboo(self) -> DVector4: ...
    @property
    def obol(self) -> DVector4: ...
    @property
    def oblr(self) -> DVector4: ...
    @property
    def oblg(self) -> DVector4: ...
    @property
    def oblb(self) -> DVector4: ...
    @property
    def obla(self) -> DVector4: ...
    @property
    def oblo(self) -> DVector4: ...
    @property
    def obll(self) -> DVector4: ...
    @property
    def oarr(self) -> DVector4: ...
    @property
    def oarg(self) -> DVector4: ...
    @property
    def oarb(self) -> DVector4: ...
    @property
    def oara(self) -> DVector4: ...
    @property
    def oaro(self) -> DVector4: ...
    @property
    def oarl(self) -> DVector4: ...
    @property
    def oagr(self) -> DVector4: ...
    @property
    def oagg(self) -> DVector4: ...
    @property
    def oagb(self) -> DVector4: ...
    @property
    def oaga(self) -> DVector4: ...
    @property
    def oago(self) -> DVector4: ...
    @property
    def oagl(self) -> DVector4: ...
    @property
    def oabr(self) -> DVector4: ...
    @property
    def oabg(self) -> DVector4: ...
    @property
    def oabb(self) -> DVector4: ...
    @property
    def oaba(self) -> DVector4: ...
    @property
    def oabo(self) -> DVector4: ...
    @property
    def oabl(self) -> DVector4: ...
    @property
    def oaar(self) -> DVector4: ...
    @property
    def oaag(self) -> DVector4: ...
    @property
    def oaab(self) -> DVector4: ...
    @property
    def oaaa(self) -> DVector4: ...
    @property
    def oaao(self) -> DVector4: ...
    @property
    def oaal(self) -> DVector4: ...
    @property
    def oaor(self) -> DVector4: ...
    @property
    def oaog(self) -> DVector4: ...
    @property
    def oaob(self) -> DVector4: ...
    @property
    def oaoa(self) -> DVector4: ...
    @property
    def oaoo(self) -> DVector4: ...
    @property
    def oaol(self) -> DVector4: ...
    @property
    def oalr(self) -> DVector4: ...
    @property
    def oalg(self) -> DVector4: ...
    @property
    def oalb(self) -> DVector4: ...
    @property
    def oala(self) -> DVector4: ...
    @property
    def oalo(self) -> DVector4: ...
    @property
    def oall(self) -> DVector4: ...
    @property
    def oorr(self) -> DVector4: ...
    @property
    def oorg(self) -> DVector4: ...
    @property
    def oorb(self) -> DVector4: ...
    @property
    def oora(self) -> DVector4: ...
    @property
    def ooro(self) -> DVector4: ...
    @property
    def oorl(self) -> DVector4: ...
    @property
    def oogr(self) -> DVector4: ...
    @property
    def oogg(self) -> DVector4: ...
    @property
    def oogb(self) -> DVector4: ...
    @property
    def ooga(self) -> DVector4: ...
    @property
    def oogo(self) -> DVector4: ...
    @property
    def oogl(self) -> DVector4: ...
    @property
    def oobr(self) -> DVector4: ...
    @property
    def oobg(self) -> DVector4: ...
    @property
    def oobb(self) -> DVector4: ...
    @property
    def ooba(self) -> DVector4: ...
    @property
    def oobo(self) -> DVector4: ...
    @property
    def oobl(self) -> DVector4: ...
    @property
    def ooar(self) -> DVector4: ...
    @property
    def ooag(self) -> DVector4: ...
    @property
    def ooab(self) -> DVector4: ...
    @property
    def ooaa(self) -> DVector4: ...
    @property
    def ooao(self) -> DVector4: ...
    @property
    def ooal(self) -> DVector4: ...
    @property
    def ooor(self) -> DVector4: ...
    @property
    def ooog(self) -> DVector4: ...
    @property
    def ooob(self) -> DVector4: ...
    @property
    def oooa(self) -> DVector4: ...
    @property
    def oolr(self) -> DVector4: ...
    @property
    def oolg(self) -> DVector4: ...
    @property
    def oolb(self) -> DVector4: ...
    @property
    def oola(self) -> DVector4: ...
    @property
    def olrr(self) -> DVector4: ...
    @property
    def olrg(self) -> DVector4: ...
    @property
    def olrb(self) -> DVector4: ...
    @property
    def olra(self) -> DVector4: ...
    @property
    def olro(self) -> DVector4: ...
    @property
    def olrl(self) -> DVector4: ...
    @property
    def olgr(self) -> DVector4: ...
    @property
    def olgg(self) -> DVector4: ...
    @property
    def olgb(self) -> DVector4: ...
    @property
    def olga(self) -> DVector4: ...
    @property
    def olgo(self) -> DVector4: ...
    @property
    def olgl(self) -> DVector4: ...
    @property
    def olbr(self) -> DVector4: ...
    @property
    def olbg(self) -> DVector4: ...
    @property
    def olbb(self) -> DVector4: ...
    @property
    def olba(self) -> DVector4: ...
    @property
    def olbo(self) -> DVector4: ...
    @property
    def olbl(self) -> DVector4: ...
    @property
    def olar(self) -> DVector4: ...
    @property
    def olag(self) -> DVector4: ...
    @property
    def olab(self) -> DVector4: ...
    @property
    def olaa(self) -> DVector4: ...
    @property
    def olao(self) -> DVector4: ...
    @property
    def olal(self) -> DVector4: ...
    @property
    def olor(self) -> DVector4: ...
    @property
    def olog(self) -> DVector4: ...
    @property
    def olob(self) -> DVector4: ...
    @property
    def oloa(self) -> DVector4: ...
    @property
    def ollr(self) -> DVector4: ...
    @property
    def ollg(self) -> DVector4: ...
    @property
    def ollb(self) -> DVector4: ...
    @property
    def olla(self) -> DVector4: ...
    @property
    def lrrr(self) -> DVector4: ...
    @property
    def lrrg(self) -> DVector4: ...
    @property
    def lrrb(self) -> DVector4: ...
    @property
    def lrra(self) -> DVector4: ...
    @property
    def lrro(self) -> DVector4: ...
    @property
    def lrrl(self) -> DVector4: ...
    @property
    def lrgr(self) -> DVector4: ...
    @property
    def lrgg(self) -> DVector4: ...
    @property
    def lrgb(self) -> DVector4: ...
    @property
    def lrga(self) -> DVector4: ...
    @property
    def lrgo(self) -> DVector4: ...
    @property
    def lrgl(self) -> DVector4: ...
    @property
    def lrbr(self) -> DVector4: ...
    @property
    def lrbg(self) -> DVector4: ...
    @property
    def lrbb(self) -> DVector4: ...
    @property
    def lrba(self) -> DVector4: ...
    @property
    def lrbo(self) -> DVector4: ...
    @property
    def lrbl(self) -> DVector4: ...
    @property
    def lrar(self) -> DVector4: ...
    @property
    def lrag(self) -> DVector4: ...
    @property
    def lrab(self) -> DVector4: ...
    @property
    def lraa(self) -> DVector4: ...
    @property
    def lrao(self) -> DVector4: ...
    @property
    def lral(self) -> DVector4: ...
    @property
    def lror(self) -> DVector4: ...
    @property
    def lrog(self) -> DVector4: ...
    @property
    def lrob(self) -> DVector4: ...
    @property
    def lroa(self) -> DVector4: ...
    @property
    def lroo(self) -> DVector4: ...
    @property
    def lrol(self) -> DVector4: ...
    @property
    def lrlr(self) -> DVector4: ...
    @property
    def lrlg(self) -> DVector4: ...
    @property
    def lrlb(self) -> DVector4: ...
    @property
    def lrla(self) -> DVector4: ...
    @property
    def lrlo(self) -> DVector4: ...
    @property
    def lrll(self) -> DVector4: ...
    @property
    def lgrr(self) -> DVector4: ...
    @property
    def lgrg(self) -> DVector4: ...
    @property
    def lgrb(self) -> DVector4: ...
    @property
    def lgra(self) -> DVector4: ...
    @property
    def lgro(self) -> DVector4: ...
    @property
    def lgrl(self) -> DVector4: ...
    @property
    def lggr(self) -> DVector4: ...
    @property
    def lggg(self) -> DVector4: ...
    @property
    def lggb(self) -> DVector4: ...
    @property
    def lgga(self) -> DVector4: ...
    @property
    def lggo(self) -> DVector4: ...
    @property
    def lggl(self) -> DVector4: ...
    @property
    def lgbr(self) -> DVector4: ...
    @property
    def lgbg(self) -> DVector4: ...
    @property
    def lgbb(self) -> DVector4: ...
    @property
    def lgba(self) -> DVector4: ...
    @property
    def lgbo(self) -> DVector4: ...
    @property
    def lgbl(self) -> DVector4: ...
    @property
    def lgar(self) -> DVector4: ...
    @property
    def lgag(self) -> DVector4: ...
    @property
    def lgab(self) -> DVector4: ...
    @property
    def lgaa(self) -> DVector4: ...
    @property
    def lgao(self) -> DVector4: ...
    @property
    def lgal(self) -> DVector4: ...
    @property
    def lgor(self) -> DVector4: ...
    @property
    def lgog(self) -> DVector4: ...
    @property
    def lgob(self) -> DVector4: ...
    @property
    def lgoa(self) -> DVector4: ...
    @property
    def lgoo(self) -> DVector4: ...
    @property
    def lgol(self) -> DVector4: ...
    @property
    def lglr(self) -> DVector4: ...
    @property
    def lglg(self) -> DVector4: ...
    @property
    def lglb(self) -> DVector4: ...
    @property
    def lgla(self) -> DVector4: ...
    @property
    def lglo(self) -> DVector4: ...
    @property
    def lgll(self) -> DVector4: ...
    @property
    def lbrr(self) -> DVector4: ...
    @property
    def lbrg(self) -> DVector4: ...
    @property
    def lbrb(self) -> DVector4: ...
    @property
    def lbra(self) -> DVector4: ...
    @property
    def lbro(self) -> DVector4: ...
    @property
    def lbrl(self) -> DVector4: ...
    @property
    def lbgr(self) -> DVector4: ...
    @property
    def lbgg(self) -> DVector4: ...
    @property
    def lbgb(self) -> DVector4: ...
    @property
    def lbga(self) -> DVector4: ...
    @property
    def lbgo(self) -> DVector4: ...
    @property
    def lbgl(self) -> DVector4: ...
    @property
    def lbbr(self) -> DVector4: ...
    @property
    def lbbg(self) -> DVector4: ...
    @property
    def lbbb(self) -> DVector4: ...
    @property
    def lbba(self) -> DVector4: ...
    @property
    def lbbo(self) -> DVector4: ...
    @property
    def lbbl(self) -> DVector4: ...
    @property
    def lbar(self) -> DVector4: ...
    @property
    def lbag(self) -> DVector4: ...
    @property
    def lbab(self) -> DVector4: ...
    @property
    def lbaa(self) -> DVector4: ...
    @property
    def lbao(self) -> DVector4: ...
    @property
    def lbal(self) -> DVector4: ...
    @property
    def lbor(self) -> DVector4: ...
    @property
    def lbog(self) -> DVector4: ...
    @property
    def lbob(self) -> DVector4: ...
    @property
    def lboa(self) -> DVector4: ...
    @property
    def lboo(self) -> DVector4: ...
    @property
    def lbol(self) -> DVector4: ...
    @property
    def lblr(self) -> DVector4: ...
    @property
    def lblg(self) -> DVector4: ...
    @property
    def lblb(self) -> DVector4: ...
    @property
    def lbla(self) -> DVector4: ...
    @property
    def lblo(self) -> DVector4: ...
    @property
    def lbll(self) -> DVector4: ...
    @property
    def larr(self) -> DVector4: ...
    @property
    def larg(self) -> DVector4: ...
    @property
    def larb(self) -> DVector4: ...
    @property
    def lara(self) -> DVector4: ...
    @property
    def laro(self) -> DVector4: ...
    @property
    def larl(self) -> DVector4: ...
    @property
    def lagr(self) -> DVector4: ...
    @property
    def lagg(self) -> DVector4: ...
    @property
    def lagb(self) -> DVector4: ...
    @property
    def laga(self) -> DVector4: ...
    @property
    def lago(self) -> DVector4: ...
    @property
    def lagl(self) -> DVector4: ...
    @property
    def labr(self) -> DVector4: ...
    @property
    def labg(self) -> DVector4: ...
    @property
    def labb(self) -> DVector4: ...
    @property
    def laba(self) -> DVector4: ...
    @property
    def labo(self) -> DVector4: ...
    @property
    def labl(self) -> DVector4: ...
    @property
    def laar(self) -> DVector4: ...
    @property
    def laag(self) -> DVector4: ...
    @property
    def laab(self) -> DVector4: ...
    @property
    def laaa(self) -> DVector4: ...
    @property
    def laao(self) -> DVector4: ...
    @property
    def laal(self) -> DVector4: ...
    @property
    def laor(self) -> DVector4: ...
    @property
    def laog(self) -> DVector4: ...
    @property
    def laob(self) -> DVector4: ...
    @property
    def laoa(self) -> DVector4: ...
    @property
    def laoo(self) -> DVector4: ...
    @property
    def laol(self) -> DVector4: ...
    @property
    def lalr(self) -> DVector4: ...
    @property
    def lalg(self) -> DVector4: ...
    @property
    def lalb(self) -> DVector4: ...
    @property
    def lala(self) -> DVector4: ...
    @property
    def lalo(self) -> DVector4: ...
    @property
    def lall(self) -> DVector4: ...
    @property
    def lorr(self) -> DVector4: ...
    @property
    def lorg(self) -> DVector4: ...
    @property
    def lorb(self) -> DVector4: ...
    @property
    def lora(self) -> DVector4: ...
    @property
    def loro(self) -> DVector4: ...
    @property
    def lorl(self) -> DVector4: ...
    @property
    def logr(self) -> DVector4: ...
    @property
    def logg(self) -> DVector4: ...
    @property
    def logb(self) -> DVector4: ...
    @property
    def loga(self) -> DVector4: ...
    @property
    def logo(self) -> DVector4: ...
    @property
    def logl(self) -> DVector4: ...
    @property
    def lobr(self) -> DVector4: ...
    @property
    def lobg(self) -> DVector4: ...
    @property
    def lobb(self) -> DVector4: ...
    @property
    def loba(self) -> DVector4: ...
    @property
    def lobo(self) -> DVector4: ...
    @property
    def lobl(self) -> DVector4: ...
    @property
    def loar(self) -> DVector4: ...
    @property
    def loag(self) -> DVector4: ...
    @property
    def loab(self) -> DVector4: ...
    @property
    def loaa(self) -> DVector4: ...
    @property
    def loao(self) -> DVector4: ...
    @property
    def loal(self) -> DVector4: ...
    @property
    def loor(self) -> DVector4: ...
    @property
    def loog(self) -> DVector4: ...
    @property
    def loob(self) -> DVector4: ...
    @property
    def looa(self) -> DVector4: ...
    @property
    def lolr(self) -> DVector4: ...
    @property
    def lolg(self) -> DVector4: ...
    @property
    def lolb(self) -> DVector4: ...
    @property
    def lola(self) -> DVector4: ...
    @property
    def llrr(self) -> DVector4: ...
    @property
    def llrg(self) -> DVector4: ...
    @property
    def llrb(self) -> DVector4: ...
    @property
    def llra(self) -> DVector4: ...
    @property
    def llro(self) -> DVector4: ...
    @property
    def llrl(self) -> DVector4: ...
    @property
    def llgr(self) -> DVector4: ...
    @property
    def llgg(self) -> DVector4: ...
    @property
    def llgb(self) -> DVector4: ...
    @property
    def llga(self) -> DVector4: ...
    @property
    def llgo(self) -> DVector4: ...
    @property
    def llgl(self) -> DVector4: ...
    @property
    def llbr(self) -> DVector4: ...
    @property
    def llbg(self) -> DVector4: ...
    @property
    def llbb(self) -> DVector4: ...
    @property
    def llba(self) -> DVector4: ...
    @property
    def llbo(self) -> DVector4: ...
    @property
    def llbl(self) -> DVector4: ...
    @property
    def llar(self) -> DVector4: ...
    @property
    def llag(self) -> DVector4: ...
    @property
    def llab(self) -> DVector4: ...
    @property
    def llaa(self) -> DVector4: ...
    @property
    def llao(self) -> DVector4: ...
    @property
    def llal(self) -> DVector4: ...
    @property
    def llor(self) -> DVector4: ...
    @property
    def llog(self) -> DVector4: ...
    @property
    def llob(self) -> DVector4: ...
    @property
    def lloa(self) -> DVector4: ...
    @property
    def lllr(self) -> DVector4: ...
    @property
    def lllg(self) -> DVector4: ...
    @property
    def lllb(self) -> DVector4: ...
    @property
    def llla(self) -> DVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def t(self) -> float: ...
    @property
    def q(self) -> float: ...
    @property
    def p(self) -> float: ...
    @property
    def ss(self) -> DVector2: ...
    @property
    def st(self) -> DVector2: ...
    @property
    def sq(self) -> DVector2: ...
    @property
    def sp(self) -> DVector2: ...
    @property
    def so(self) -> DVector2: ...
    @property
    def sl(self) -> DVector2: ...
    @property
    def ts(self) -> DVector2: ...
    @property
    def tt(self) -> DVector2: ...
    @property
    def tq(self) -> DVector2: ...
    @property
    def tp(self) -> DVector2: ...
    @property
    def to(self) -> DVector2: ...
    @property
    def tl(self) -> DVector2: ...
    @property
    def qs(self) -> DVector2: ...
    @property
    def qt(self) -> DVector2: ...
    @property
    def qq(self) -> DVector2: ...
    @property
    def qp(self) -> DVector2: ...
    @property
    def qo(self) -> DVector2: ...
    @property
    def ql(self) -> DVector2: ...
    @property
    def ps(self) -> DVector2: ...
    @property
    def pt(self) -> DVector2: ...
    @property
    def pq(self) -> DVector2: ...
    @property
    def pp(self) -> DVector2: ...
    @property
    def po(self) -> DVector2: ...
    @property
    def pl(self) -> DVector2: ...
    @property
    def os(self) -> DVector2: ...
    @property
    def ot(self) -> DVector2: ...
    @property
    def oq(self) -> DVector2: ...
    @property
    def op(self) -> DVector2: ...
    @property
    def ls(self) -> DVector2: ...
    @property
    def lt(self) -> DVector2: ...
    @property
    def lq(self) -> DVector2: ...
    @property
    def lp(self) -> DVector2: ...
    @property
    def sss(self) -> DVector3: ...
    @property
    def sst(self) -> DVector3: ...
    @property
    def ssq(self) -> DVector3: ...
    @property
    def ssp(self) -> DVector3: ...
    @property
    def sso(self) -> DVector3: ...
    @property
    def ssl(self) -> DVector3: ...
    @property
    def sts(self) -> DVector3: ...
    @property
    def stt(self) -> DVector3: ...
    @property
    def stq(self) -> DVector3: ...
    @property
    def stp(self) -> DVector3: ...
    @property
    def sto(self) -> DVector3: ...
    @property
    def stl(self) -> DVector3: ...
    @property
    def sqs(self) -> DVector3: ...
    @property
    def sqt(self) -> DVector3: ...
    @property
    def sqq(self) -> DVector3: ...
    @property
    def sqp(self) -> DVector3: ...
    @property
    def sqo(self) -> DVector3: ...
    @property
    def sql(self) -> DVector3: ...
    @property
    def sps(self) -> DVector3: ...
    @property
    def spt(self) -> DVector3: ...
    @property
    def spq(self) -> DVector3: ...
    @property
    def spp(self) -> DVector3: ...
    @property
    def spo(self) -> DVector3: ...
    @property
    def spl(self) -> DVector3: ...
    @property
    def sos(self) -> DVector3: ...
    @property
    def sot(self) -> DVector3: ...
    @property
    def soq(self) -> DVector3: ...
    @property
    def sop(self) -> DVector3: ...
    @property
    def soo(self) -> DVector3: ...
    @property
    def sol(self) -> DVector3: ...
    @property
    def sls(self) -> DVector3: ...
    @property
    def slt(self) -> DVector3: ...
    @property
    def slq(self) -> DVector3: ...
    @property
    def slp(self) -> DVector3: ...
    @property
    def slo(self) -> DVector3: ...
    @property
    def sll(self) -> DVector3: ...
    @property
    def tss(self) -> DVector3: ...
    @property
    def tst(self) -> DVector3: ...
    @property
    def tsq(self) -> DVector3: ...
    @property
    def tsp(self) -> DVector3: ...
    @property
    def tso(self) -> DVector3: ...
    @property
    def tsl(self) -> DVector3: ...
    @property
    def tts(self) -> DVector3: ...
    @property
    def ttt(self) -> DVector3: ...
    @property
    def ttq(self) -> DVector3: ...
    @property
    def ttp(self) -> DVector3: ...
    @property
    def tto(self) -> DVector3: ...
    @property
    def ttl(self) -> DVector3: ...
    @property
    def tqs(self) -> DVector3: ...
    @property
    def tqt(self) -> DVector3: ...
    @property
    def tqq(self) -> DVector3: ...
    @property
    def tqp(self) -> DVector3: ...
    @property
    def tqo(self) -> DVector3: ...
    @property
    def tql(self) -> DVector3: ...
    @property
    def tps(self) -> DVector3: ...
    @property
    def tpt(self) -> DVector3: ...
    @property
    def tpq(self) -> DVector3: ...
    @property
    def tpp(self) -> DVector3: ...
    @property
    def tpo(self) -> DVector3: ...
    @property
    def tpl(self) -> DVector3: ...
    @property
    def tos(self) -> DVector3: ...
    @property
    def tot(self) -> DVector3: ...
    @property
    def toq(self) -> DVector3: ...
    @property
    def top(self) -> DVector3: ...
    @property
    def too(self) -> DVector3: ...
    @property
    def tol(self) -> DVector3: ...
    @property
    def tls(self) -> DVector3: ...
    @property
    def tlt(self) -> DVector3: ...
    @property
    def tlq(self) -> DVector3: ...
    @property
    def tlp(self) -> DVector3: ...
    @property
    def tlo(self) -> DVector3: ...
    @property
    def tll(self) -> DVector3: ...
    @property
    def qss(self) -> DVector3: ...
    @property
    def qst(self) -> DVector3: ...
    @property
    def qsq(self) -> DVector3: ...
    @property
    def qsp(self) -> DVector3: ...
    @property
    def qso(self) -> DVector3: ...
    @property
    def qsl(self) -> DVector3: ...
    @property
    def qts(self) -> DVector3: ...
    @property
    def qtt(self) -> DVector3: ...
    @property
    def qtq(self) -> DVector3: ...
    @property
    def qtp(self) -> DVector3: ...
    @property
    def qto(self) -> DVector3: ...
    @property
    def qtl(self) -> DVector3: ...
    @property
    def qqs(self) -> DVector3: ...
    @property
    def qqt(self) -> DVector3: ...
    @property
    def qqq(self) -> DVector3: ...
    @property
    def qqp(self) -> DVector3: ...
    @property
    def qqo(self) -> DVector3: ...
    @property
    def qql(self) -> DVector3: ...
    @property
    def qps(self) -> DVector3: ...
    @property
    def qpt(self) -> DVector3: ...
    @property
    def qpq(self) -> DVector3: ...
    @property
    def qpp(self) -> DVector3: ...
    @property
    def qpo(self) -> DVector3: ...
    @property
    def qpl(self) -> DVector3: ...
    @property
    def qos(self) -> DVector3: ...
    @property
    def qot(self) -> DVector3: ...
    @property
    def qoq(self) -> DVector3: ...
    @property
    def qop(self) -> DVector3: ...
    @property
    def qoo(self) -> DVector3: ...
    @property
    def qol(self) -> DVector3: ...
    @property
    def qls(self) -> DVector3: ...
    @property
    def qlt(self) -> DVector3: ...
    @property
    def qlq(self) -> DVector3: ...
    @property
    def qlp(self) -> DVector3: ...
    @property
    def qlo(self) -> DVector3: ...
    @property
    def qll(self) -> DVector3: ...
    @property
    def pss(self) -> DVector3: ...
    @property
    def pst(self) -> DVector3: ...
    @property
    def psq(self) -> DVector3: ...
    @property
    def psp(self) -> DVector3: ...
    @property
    def pso(self) -> DVector3: ...
    @property
    def psl(self) -> DVector3: ...
    @property
    def pts(self) -> DVector3: ...
    @property
    def ptt(self) -> DVector3: ...
    @property
    def ptq(self) -> DVector3: ...
    @property
    def ptp(self) -> DVector3: ...
    @property
    def pto(self) -> DVector3: ...
    @property
    def ptl(self) -> DVector3: ...
    @property
    def pqs(self) -> DVector3: ...
    @property
    def pqt(self) -> DVector3: ...
    @property
    def pqq(self) -> DVector3: ...
    @property
    def pqp(self) -> DVector3: ...
    @property
    def pqo(self) -> DVector3: ...
    @property
    def pql(self) -> DVector3: ...
    @property
    def pps(self) -> DVector3: ...
    @property
    def ppt(self) -> DVector3: ...
    @property
    def ppq(self) -> DVector3: ...
    @property
    def ppp(self) -> DVector3: ...
    @property
    def ppo(self) -> DVector3: ...
    @property
    def ppl(self) -> DVector3: ...
    @property
    def pos(self) -> DVector3: ...
    @property
    def pot(self) -> DVector3: ...
    @property
    def poq(self) -> DVector3: ...
    @property
    def pop(self) -> DVector3: ...
    @property
    def poo(self) -> DVector3: ...
    @property
    def pol(self) -> DVector3: ...
    @property
    def pls(self) -> DVector3: ...
    @property
    def plt(self) -> DVector3: ...
    @property
    def plq(self) -> DVector3: ...
    @property
    def plp(self) -> DVector3: ...
    @property
    def plo(self) -> DVector3: ...
    @property
    def pll(self) -> DVector3: ...
    @property
    def oss(self) -> DVector3: ...
    @property
    def ost(self) -> DVector3: ...
    @property
    def osq(self) -> DVector3: ...
    @property
    def osp(self) -> DVector3: ...
    @property
    def oso(self) -> DVector3: ...
    @property
    def osl(self) -> DVector3: ...
    @property
    def ots(self) -> DVector3: ...
    @property
    def ott(self) -> DVector3: ...
    @property
    def otq(self) -> DVector3: ...
    @property
    def otp(self) -> DVector3: ...
    @property
    def oto(self) -> DVector3: ...
    @property
    def otl(self) -> DVector3: ...
    @property
    def oqs(self) -> DVector3: ...
    @property
    def oqt(self) -> DVector3: ...
    @property
    def oqq(self) -> DVector3: ...
    @property
    def oqp(self) -> DVector3: ...
    @property
    def oqo(self) -> DVector3: ...
    @property
    def oql(self) -> DVector3: ...
    @property
    def ops(self) -> DVector3: ...
    @property
    def opt(self) -> DVector3: ...
    @property
    def opq(self) -> DVector3: ...
    @property
    def opp(self) -> DVector3: ...
    @property
    def opo(self) -> DVector3: ...
    @property
    def opl(self) -> DVector3: ...
    @property
    def oos(self) -> DVector3: ...
    @property
    def oot(self) -> DVector3: ...
    @property
    def ooq(self) -> DVector3: ...
    @property
    def oop(self) -> DVector3: ...
    @property
    def ols(self) -> DVector3: ...
    @property
    def olt(self) -> DVector3: ...
    @property
    def olq(self) -> DVector3: ...
    @property
    def olp(self) -> DVector3: ...
    @property
    def lss(self) -> DVector3: ...
    @property
    def lst(self) -> DVector3: ...
    @property
    def lsq(self) -> DVector3: ...
    @property
    def lsp(self) -> DVector3: ...
    @property
    def lso(self) -> DVector3: ...
    @property
    def lsl(self) -> DVector3: ...
    @property
    def lts(self) -> DVector3: ...
    @property
    def ltt(self) -> DVector3: ...
    @property
    def ltq(self) -> DVector3: ...
    @property
    def ltp(self) -> DVector3: ...
    @property
    def lto(self) -> DVector3: ...
    @property
    def ltl(self) -> DVector3: ...
    @property
    def lqs(self) -> DVector3: ...
    @property
    def lqt(self) -> DVector3: ...
    @property
    def lqq(self) -> DVector3: ...
    @property
    def lqp(self) -> DVector3: ...
    @property
    def lqo(self) -> DVector3: ...
    @property
    def lql(self) -> DVector3: ...
    @property
    def lps(self) -> DVector3: ...
    @property
    def lpt(self) -> DVector3: ...
    @property
    def lpq(self) -> DVector3: ...
    @property
    def lpp(self) -> DVector3: ...
    @property
    def lpo(self) -> DVector3: ...
    @property
    def lpl(self) -> DVector3: ...
    @property
    def los(self) -> DVector3: ...
    @property
    def lot(self) -> DVector3: ...
    @property
    def loq(self) -> DVector3: ...
    @property
    def lop(self) -> DVector3: ...
    @property
    def lls(self) -> DVector3: ...
    @property
    def llt(self) -> DVector3: ...
    @property
    def llq(self) -> DVector3: ...
    @property
    def llp(self) -> DVector3: ...
    @property
    def ssss(self) -> DVector4: ...
    @property
    def ssst(self) -> DVector4: ...
    @property
    def sssq(self) -> DVector4: ...
    @property
    def sssp(self) -> DVector4: ...
    @property
    def ssso(self) -> DVector4: ...
    @property
    def sssl(self) -> DVector4: ...
    @property
    def ssts(self) -> DVector4: ...
    @property
    def sstt(self) -> DVector4: ...
    @property
    def sstq(self) -> DVector4: ...
    @property
    def sstp(self) -> DVector4: ...
    @property
    def ssto(self) -> DVector4: ...
    @property
    def sstl(self) -> DVector4: ...
    @property
    def ssqs(self) -> DVector4: ...
    @property
    def ssqt(self) -> DVector4: ...
    @property
    def ssqq(self) -> DVector4: ...
    @property
    def ssqp(self) -> DVector4: ...
    @property
    def ssqo(self) -> DVector4: ...
    @property
    def ssql(self) -> DVector4: ...
    @property
    def ssps(self) -> DVector4: ...
    @property
    def sspt(self) -> DVector4: ...
    @property
    def sspq(self) -> DVector4: ...
    @property
    def sspp(self) -> DVector4: ...
    @property
    def sspo(self) -> DVector4: ...
    @property
    def sspl(self) -> DVector4: ...
    @property
    def ssos(self) -> DVector4: ...
    @property
    def ssot(self) -> DVector4: ...
    @property
    def ssoq(self) -> DVector4: ...
    @property
    def ssop(self) -> DVector4: ...
    @property
    def ssoo(self) -> DVector4: ...
    @property
    def ssol(self) -> DVector4: ...
    @property
    def ssls(self) -> DVector4: ...
    @property
    def sslt(self) -> DVector4: ...
    @property
    def sslq(self) -> DVector4: ...
    @property
    def sslp(self) -> DVector4: ...
    @property
    def sslo(self) -> DVector4: ...
    @property
    def ssll(self) -> DVector4: ...
    @property
    def stss(self) -> DVector4: ...
    @property
    def stst(self) -> DVector4: ...
    @property
    def stsq(self) -> DVector4: ...
    @property
    def stsp(self) -> DVector4: ...
    @property
    def stso(self) -> DVector4: ...
    @property
    def stsl(self) -> DVector4: ...
    @property
    def stts(self) -> DVector4: ...
    @property
    def sttt(self) -> DVector4: ...
    @property
    def sttq(self) -> DVector4: ...
    @property
    def sttp(self) -> DVector4: ...
    @property
    def stto(self) -> DVector4: ...
    @property
    def sttl(self) -> DVector4: ...
    @property
    def stqs(self) -> DVector4: ...
    @property
    def stqt(self) -> DVector4: ...
    @property
    def stqq(self) -> DVector4: ...
    @property
    def stqp(self) -> DVector4: ...
    @property
    def stqo(self) -> DVector4: ...
    @property
    def stql(self) -> DVector4: ...
    @property
    def stps(self) -> DVector4: ...
    @property
    def stpt(self) -> DVector4: ...
    @property
    def stpq(self) -> DVector4: ...
    @property
    def stpp(self) -> DVector4: ...
    @property
    def stpo(self) -> DVector4: ...
    @property
    def stpl(self) -> DVector4: ...
    @property
    def stos(self) -> DVector4: ...
    @property
    def stot(self) -> DVector4: ...
    @property
    def stoq(self) -> DVector4: ...
    @property
    def stop(self) -> DVector4: ...
    @property
    def stoo(self) -> DVector4: ...
    @property
    def stol(self) -> DVector4: ...
    @property
    def stls(self) -> DVector4: ...
    @property
    def stlt(self) -> DVector4: ...
    @property
    def stlq(self) -> DVector4: ...
    @property
    def stlp(self) -> DVector4: ...
    @property
    def stlo(self) -> DVector4: ...
    @property
    def stll(self) -> DVector4: ...
    @property
    def sqss(self) -> DVector4: ...
    @property
    def sqst(self) -> DVector4: ...
    @property
    def sqsq(self) -> DVector4: ...
    @property
    def sqsp(self) -> DVector4: ...
    @property
    def sqso(self) -> DVector4: ...
    @property
    def sqsl(self) -> DVector4: ...
    @property
    def sqts(self) -> DVector4: ...
    @property
    def sqtt(self) -> DVector4: ...
    @property
    def sqtq(self) -> DVector4: ...
    @property
    def sqtp(self) -> DVector4: ...
    @property
    def sqto(self) -> DVector4: ...
    @property
    def sqtl(self) -> DVector4: ...
    @property
    def sqqs(self) -> DVector4: ...
    @property
    def sqqt(self) -> DVector4: ...
    @property
    def sqqq(self) -> DVector4: ...
    @property
    def sqqp(self) -> DVector4: ...
    @property
    def sqqo(self) -> DVector4: ...
    @property
    def sqql(self) -> DVector4: ...
    @property
    def sqps(self) -> DVector4: ...
    @property
    def sqpt(self) -> DVector4: ...
    @property
    def sqpq(self) -> DVector4: ...
    @property
    def sqpp(self) -> DVector4: ...
    @property
    def sqpo(self) -> DVector4: ...
    @property
    def sqpl(self) -> DVector4: ...
    @property
    def sqos(self) -> DVector4: ...
    @property
    def sqot(self) -> DVector4: ...
    @property
    def sqoq(self) -> DVector4: ...
    @property
    def sqop(self) -> DVector4: ...
    @property
    def sqoo(self) -> DVector4: ...
    @property
    def sqol(self) -> DVector4: ...
    @property
    def sqls(self) -> DVector4: ...
    @property
    def sqlt(self) -> DVector4: ...
    @property
    def sqlq(self) -> DVector4: ...
    @property
    def sqlp(self) -> DVector4: ...
    @property
    def sqlo(self) -> DVector4: ...
    @property
    def sqll(self) -> DVector4: ...
    @property
    def spss(self) -> DVector4: ...
    @property
    def spst(self) -> DVector4: ...
    @property
    def spsq(self) -> DVector4: ...
    @property
    def spsp(self) -> DVector4: ...
    @property
    def spso(self) -> DVector4: ...
    @property
    def spsl(self) -> DVector4: ...
    @property
    def spts(self) -> DVector4: ...
    @property
    def sptt(self) -> DVector4: ...
    @property
    def sptq(self) -> DVector4: ...
    @property
    def sptp(self) -> DVector4: ...
    @property
    def spto(self) -> DVector4: ...
    @property
    def sptl(self) -> DVector4: ...
    @property
    def spqs(self) -> DVector4: ...
    @property
    def spqt(self) -> DVector4: ...
    @property
    def spqq(self) -> DVector4: ...
    @property
    def spqp(self) -> DVector4: ...
    @property
    def spqo(self) -> DVector4: ...
    @property
    def spql(self) -> DVector4: ...
    @property
    def spps(self) -> DVector4: ...
    @property
    def sppt(self) -> DVector4: ...
    @property
    def sppq(self) -> DVector4: ...
    @property
    def sppp(self) -> DVector4: ...
    @property
    def sppo(self) -> DVector4: ...
    @property
    def sppl(self) -> DVector4: ...
    @property
    def spos(self) -> DVector4: ...
    @property
    def spot(self) -> DVector4: ...
    @property
    def spoq(self) -> DVector4: ...
    @property
    def spop(self) -> DVector4: ...
    @property
    def spoo(self) -> DVector4: ...
    @property
    def spol(self) -> DVector4: ...
    @property
    def spls(self) -> DVector4: ...
    @property
    def splt(self) -> DVector4: ...
    @property
    def splq(self) -> DVector4: ...
    @property
    def splp(self) -> DVector4: ...
    @property
    def splo(self) -> DVector4: ...
    @property
    def spll(self) -> DVector4: ...
    @property
    def soss(self) -> DVector4: ...
    @property
    def sost(self) -> DVector4: ...
    @property
    def sosq(self) -> DVector4: ...
    @property
    def sosp(self) -> DVector4: ...
    @property
    def soso(self) -> DVector4: ...
    @property
    def sosl(self) -> DVector4: ...
    @property
    def sots(self) -> DVector4: ...
    @property
    def sott(self) -> DVector4: ...
    @property
    def sotq(self) -> DVector4: ...
    @property
    def sotp(self) -> DVector4: ...
    @property
    def soto(self) -> DVector4: ...
    @property
    def sotl(self) -> DVector4: ...
    @property
    def soqs(self) -> DVector4: ...
    @property
    def soqt(self) -> DVector4: ...
    @property
    def soqq(self) -> DVector4: ...
    @property
    def soqp(self) -> DVector4: ...
    @property
    def soqo(self) -> DVector4: ...
    @property
    def soql(self) -> DVector4: ...
    @property
    def sops(self) -> DVector4: ...
    @property
    def sopt(self) -> DVector4: ...
    @property
    def sopq(self) -> DVector4: ...
    @property
    def sopp(self) -> DVector4: ...
    @property
    def sopo(self) -> DVector4: ...
    @property
    def sopl(self) -> DVector4: ...
    @property
    def soos(self) -> DVector4: ...
    @property
    def soot(self) -> DVector4: ...
    @property
    def sooq(self) -> DVector4: ...
    @property
    def soop(self) -> DVector4: ...
    @property
    def sooo(self) -> DVector4: ...
    @property
    def sool(self) -> DVector4: ...
    @property
    def sols(self) -> DVector4: ...
    @property
    def solt(self) -> DVector4: ...
    @property
    def solq(self) -> DVector4: ...
    @property
    def solp(self) -> DVector4: ...
    @property
    def solo(self) -> DVector4: ...
    @property
    def soll(self) -> DVector4: ...
    @property
    def slss(self) -> DVector4: ...
    @property
    def slst(self) -> DVector4: ...
    @property
    def slsq(self) -> DVector4: ...
    @property
    def slsp(self) -> DVector4: ...
    @property
    def slso(self) -> DVector4: ...
    @property
    def slsl(self) -> DVector4: ...
    @property
    def slts(self) -> DVector4: ...
    @property
    def sltt(self) -> DVector4: ...
    @property
    def sltq(self) -> DVector4: ...
    @property
    def sltp(self) -> DVector4: ...
    @property
    def slto(self) -> DVector4: ...
    @property
    def sltl(self) -> DVector4: ...
    @property
    def slqs(self) -> DVector4: ...
    @property
    def slqt(self) -> DVector4: ...
    @property
    def slqq(self) -> DVector4: ...
    @property
    def slqp(self) -> DVector4: ...
    @property
    def slqo(self) -> DVector4: ...
    @property
    def slql(self) -> DVector4: ...
    @property
    def slps(self) -> DVector4: ...
    @property
    def slpt(self) -> DVector4: ...
    @property
    def slpq(self) -> DVector4: ...
    @property
    def slpp(self) -> DVector4: ...
    @property
    def slpo(self) -> DVector4: ...
    @property
    def slpl(self) -> DVector4: ...
    @property
    def slos(self) -> DVector4: ...
    @property
    def slot(self) -> DVector4: ...
    @property
    def sloq(self) -> DVector4: ...
    @property
    def slop(self) -> DVector4: ...
    @property
    def sloo(self) -> DVector4: ...
    @property
    def slol(self) -> DVector4: ...
    @property
    def slls(self) -> DVector4: ...
    @property
    def sllt(self) -> DVector4: ...
    @property
    def sllq(self) -> DVector4: ...
    @property
    def sllp(self) -> DVector4: ...
    @property
    def sllo(self) -> DVector4: ...
    @property
    def slll(self) -> DVector4: ...
    @property
    def tsss(self) -> DVector4: ...
    @property
    def tsst(self) -> DVector4: ...
    @property
    def tssq(self) -> DVector4: ...
    @property
    def tssp(self) -> DVector4: ...
    @property
    def tsso(self) -> DVector4: ...
    @property
    def tssl(self) -> DVector4: ...
    @property
    def tsts(self) -> DVector4: ...
    @property
    def tstt(self) -> DVector4: ...
    @property
    def tstq(self) -> DVector4: ...
    @property
    def tstp(self) -> DVector4: ...
    @property
    def tsto(self) -> DVector4: ...
    @property
    def tstl(self) -> DVector4: ...
    @property
    def tsqs(self) -> DVector4: ...
    @property
    def tsqt(self) -> DVector4: ...
    @property
    def tsqq(self) -> DVector4: ...
    @property
    def tsqp(self) -> DVector4: ...
    @property
    def tsqo(self) -> DVector4: ...
    @property
    def tsql(self) -> DVector4: ...
    @property
    def tsps(self) -> DVector4: ...
    @property
    def tspt(self) -> DVector4: ...
    @property
    def tspq(self) -> DVector4: ...
    @property
    def tspp(self) -> DVector4: ...
    @property
    def tspo(self) -> DVector4: ...
    @property
    def tspl(self) -> DVector4: ...
    @property
    def tsos(self) -> DVector4: ...
    @property
    def tsot(self) -> DVector4: ...
    @property
    def tsoq(self) -> DVector4: ...
    @property
    def tsop(self) -> DVector4: ...
    @property
    def tsoo(self) -> DVector4: ...
    @property
    def tsol(self) -> DVector4: ...
    @property
    def tsls(self) -> DVector4: ...
    @property
    def tslt(self) -> DVector4: ...
    @property
    def tslq(self) -> DVector4: ...
    @property
    def tslp(self) -> DVector4: ...
    @property
    def tslo(self) -> DVector4: ...
    @property
    def tsll(self) -> DVector4: ...
    @property
    def ttss(self) -> DVector4: ...
    @property
    def ttst(self) -> DVector4: ...
    @property
    def ttsq(self) -> DVector4: ...
    @property
    def ttsp(self) -> DVector4: ...
    @property
    def ttso(self) -> DVector4: ...
    @property
    def ttsl(self) -> DVector4: ...
    @property
    def ttts(self) -> DVector4: ...
    @property
    def tttt(self) -> DVector4: ...
    @property
    def tttq(self) -> DVector4: ...
    @property
    def tttp(self) -> DVector4: ...
    @property
    def ttto(self) -> DVector4: ...
    @property
    def tttl(self) -> DVector4: ...
    @property
    def ttqs(self) -> DVector4: ...
    @property
    def ttqt(self) -> DVector4: ...
    @property
    def ttqq(self) -> DVector4: ...
    @property
    def ttqp(self) -> DVector4: ...
    @property
    def ttqo(self) -> DVector4: ...
    @property
    def ttql(self) -> DVector4: ...
    @property
    def ttps(self) -> DVector4: ...
    @property
    def ttpt(self) -> DVector4: ...
    @property
    def ttpq(self) -> DVector4: ...
    @property
    def ttpp(self) -> DVector4: ...
    @property
    def ttpo(self) -> DVector4: ...
    @property
    def ttpl(self) -> DVector4: ...
    @property
    def ttos(self) -> DVector4: ...
    @property
    def ttot(self) -> DVector4: ...
    @property
    def ttoq(self) -> DVector4: ...
    @property
    def ttop(self) -> DVector4: ...
    @property
    def ttoo(self) -> DVector4: ...
    @property
    def ttol(self) -> DVector4: ...
    @property
    def ttls(self) -> DVector4: ...
    @property
    def ttlt(self) -> DVector4: ...
    @property
    def ttlq(self) -> DVector4: ...
    @property
    def ttlp(self) -> DVector4: ...
    @property
    def ttlo(self) -> DVector4: ...
    @property
    def ttll(self) -> DVector4: ...
    @property
    def tqss(self) -> DVector4: ...
    @property
    def tqst(self) -> DVector4: ...
    @property
    def tqsq(self) -> DVector4: ...
    @property
    def tqsp(self) -> DVector4: ...
    @property
    def tqso(self) -> DVector4: ...
    @property
    def tqsl(self) -> DVector4: ...
    @property
    def tqts(self) -> DVector4: ...
    @property
    def tqtt(self) -> DVector4: ...
    @property
    def tqtq(self) -> DVector4: ...
    @property
    def tqtp(self) -> DVector4: ...
    @property
    def tqto(self) -> DVector4: ...
    @property
    def tqtl(self) -> DVector4: ...
    @property
    def tqqs(self) -> DVector4: ...
    @property
    def tqqt(self) -> DVector4: ...
    @property
    def tqqq(self) -> DVector4: ...
    @property
    def tqqp(self) -> DVector4: ...
    @property
    def tqqo(self) -> DVector4: ...
    @property
    def tqql(self) -> DVector4: ...
    @property
    def tqps(self) -> DVector4: ...
    @property
    def tqpt(self) -> DVector4: ...
    @property
    def tqpq(self) -> DVector4: ...
    @property
    def tqpp(self) -> DVector4: ...
    @property
    def tqpo(self) -> DVector4: ...
    @property
    def tqpl(self) -> DVector4: ...
    @property
    def tqos(self) -> DVector4: ...
    @property
    def tqot(self) -> DVector4: ...
    @property
    def tqoq(self) -> DVector4: ...
    @property
    def tqop(self) -> DVector4: ...
    @property
    def tqoo(self) -> DVector4: ...
    @property
    def tqol(self) -> DVector4: ...
    @property
    def tqls(self) -> DVector4: ...
    @property
    def tqlt(self) -> DVector4: ...
    @property
    def tqlq(self) -> DVector4: ...
    @property
    def tqlp(self) -> DVector4: ...
    @property
    def tqlo(self) -> DVector4: ...
    @property
    def tqll(self) -> DVector4: ...
    @property
    def tpss(self) -> DVector4: ...
    @property
    def tpst(self) -> DVector4: ...
    @property
    def tpsq(self) -> DVector4: ...
    @property
    def tpsp(self) -> DVector4: ...
    @property
    def tpso(self) -> DVector4: ...
    @property
    def tpsl(self) -> DVector4: ...
    @property
    def tpts(self) -> DVector4: ...
    @property
    def tptt(self) -> DVector4: ...
    @property
    def tptq(self) -> DVector4: ...
    @property
    def tptp(self) -> DVector4: ...
    @property
    def tpto(self) -> DVector4: ...
    @property
    def tptl(self) -> DVector4: ...
    @property
    def tpqs(self) -> DVector4: ...
    @property
    def tpqt(self) -> DVector4: ...
    @property
    def tpqq(self) -> DVector4: ...
    @property
    def tpqp(self) -> DVector4: ...
    @property
    def tpqo(self) -> DVector4: ...
    @property
    def tpql(self) -> DVector4: ...
    @property
    def tpps(self) -> DVector4: ...
    @property
    def tppt(self) -> DVector4: ...
    @property
    def tppq(self) -> DVector4: ...
    @property
    def tppp(self) -> DVector4: ...
    @property
    def tppo(self) -> DVector4: ...
    @property
    def tppl(self) -> DVector4: ...
    @property
    def tpos(self) -> DVector4: ...
    @property
    def tpot(self) -> DVector4: ...
    @property
    def tpoq(self) -> DVector4: ...
    @property
    def tpop(self) -> DVector4: ...
    @property
    def tpoo(self) -> DVector4: ...
    @property
    def tpol(self) -> DVector4: ...
    @property
    def tpls(self) -> DVector4: ...
    @property
    def tplt(self) -> DVector4: ...
    @property
    def tplq(self) -> DVector4: ...
    @property
    def tplp(self) -> DVector4: ...
    @property
    def tplo(self) -> DVector4: ...
    @property
    def tpll(self) -> DVector4: ...
    @property
    def toss(self) -> DVector4: ...
    @property
    def tost(self) -> DVector4: ...
    @property
    def tosq(self) -> DVector4: ...
    @property
    def tosp(self) -> DVector4: ...
    @property
    def toso(self) -> DVector4: ...
    @property
    def tosl(self) -> DVector4: ...
    @property
    def tots(self) -> DVector4: ...
    @property
    def tott(self) -> DVector4: ...
    @property
    def totq(self) -> DVector4: ...
    @property
    def totp(self) -> DVector4: ...
    @property
    def toto(self) -> DVector4: ...
    @property
    def totl(self) -> DVector4: ...
    @property
    def toqs(self) -> DVector4: ...
    @property
    def toqt(self) -> DVector4: ...
    @property
    def toqq(self) -> DVector4: ...
    @property
    def toqp(self) -> DVector4: ...
    @property
    def toqo(self) -> DVector4: ...
    @property
    def toql(self) -> DVector4: ...
    @property
    def tops(self) -> DVector4: ...
    @property
    def topt(self) -> DVector4: ...
    @property
    def topq(self) -> DVector4: ...
    @property
    def topp(self) -> DVector4: ...
    @property
    def topo(self) -> DVector4: ...
    @property
    def topl(self) -> DVector4: ...
    @property
    def toos(self) -> DVector4: ...
    @property
    def toot(self) -> DVector4: ...
    @property
    def tooq(self) -> DVector4: ...
    @property
    def toop(self) -> DVector4: ...
    @property
    def tooo(self) -> DVector4: ...
    @property
    def tool(self) -> DVector4: ...
    @property
    def tols(self) -> DVector4: ...
    @property
    def tolt(self) -> DVector4: ...
    @property
    def tolq(self) -> DVector4: ...
    @property
    def tolp(self) -> DVector4: ...
    @property
    def tolo(self) -> DVector4: ...
    @property
    def toll(self) -> DVector4: ...
    @property
    def tlss(self) -> DVector4: ...
    @property
    def tlst(self) -> DVector4: ...
    @property
    def tlsq(self) -> DVector4: ...
    @property
    def tlsp(self) -> DVector4: ...
    @property
    def tlso(self) -> DVector4: ...
    @property
    def tlsl(self) -> DVector4: ...
    @property
    def tlts(self) -> DVector4: ...
    @property
    def tltt(self) -> DVector4: ...
    @property
    def tltq(self) -> DVector4: ...
    @property
    def tltp(self) -> DVector4: ...
    @property
    def tlto(self) -> DVector4: ...
    @property
    def tltl(self) -> DVector4: ...
    @property
    def tlqs(self) -> DVector4: ...
    @property
    def tlqt(self) -> DVector4: ...
    @property
    def tlqq(self) -> DVector4: ...
    @property
    def tlqp(self) -> DVector4: ...
    @property
    def tlqo(self) -> DVector4: ...
    @property
    def tlql(self) -> DVector4: ...
    @property
    def tlps(self) -> DVector4: ...
    @property
    def tlpt(self) -> DVector4: ...
    @property
    def tlpq(self) -> DVector4: ...
    @property
    def tlpp(self) -> DVector4: ...
    @property
    def tlpo(self) -> DVector4: ...
    @property
    def tlpl(self) -> DVector4: ...
    @property
    def tlos(self) -> DVector4: ...
    @property
    def tlot(self) -> DVector4: ...
    @property
    def tloq(self) -> DVector4: ...
    @property
    def tlop(self) -> DVector4: ...
    @property
    def tloo(self) -> DVector4: ...
    @property
    def tlol(self) -> DVector4: ...
    @property
    def tlls(self) -> DVector4: ...
    @property
    def tllt(self) -> DVector4: ...
    @property
    def tllq(self) -> DVector4: ...
    @property
    def tllp(self) -> DVector4: ...
    @property
    def tllo(self) -> DVector4: ...
    @property
    def tlll(self) -> DVector4: ...
    @property
    def qsss(self) -> DVector4: ...
    @property
    def qsst(self) -> DVector4: ...
    @property
    def qssq(self) -> DVector4: ...
    @property
    def qssp(self) -> DVector4: ...
    @property
    def qsso(self) -> DVector4: ...
    @property
    def qssl(self) -> DVector4: ...
    @property
    def qsts(self) -> DVector4: ...
    @property
    def qstt(self) -> DVector4: ...
    @property
    def qstq(self) -> DVector4: ...
    @property
    def qstp(self) -> DVector4: ...
    @property
    def qsto(self) -> DVector4: ...
    @property
    def qstl(self) -> DVector4: ...
    @property
    def qsqs(self) -> DVector4: ...
    @property
    def qsqt(self) -> DVector4: ...
    @property
    def qsqq(self) -> DVector4: ...
    @property
    def qsqp(self) -> DVector4: ...
    @property
    def qsqo(self) -> DVector4: ...
    @property
    def qsql(self) -> DVector4: ...
    @property
    def qsps(self) -> DVector4: ...
    @property
    def qspt(self) -> DVector4: ...
    @property
    def qspq(self) -> DVector4: ...
    @property
    def qspp(self) -> DVector4: ...
    @property
    def qspo(self) -> DVector4: ...
    @property
    def qspl(self) -> DVector4: ...
    @property
    def qsos(self) -> DVector4: ...
    @property
    def qsot(self) -> DVector4: ...
    @property
    def qsoq(self) -> DVector4: ...
    @property
    def qsop(self) -> DVector4: ...
    @property
    def qsoo(self) -> DVector4: ...
    @property
    def qsol(self) -> DVector4: ...
    @property
    def qsls(self) -> DVector4: ...
    @property
    def qslt(self) -> DVector4: ...
    @property
    def qslq(self) -> DVector4: ...
    @property
    def qslp(self) -> DVector4: ...
    @property
    def qslo(self) -> DVector4: ...
    @property
    def qsll(self) -> DVector4: ...
    @property
    def qtss(self) -> DVector4: ...
    @property
    def qtst(self) -> DVector4: ...
    @property
    def qtsq(self) -> DVector4: ...
    @property
    def qtsp(self) -> DVector4: ...
    @property
    def qtso(self) -> DVector4: ...
    @property
    def qtsl(self) -> DVector4: ...
    @property
    def qtts(self) -> DVector4: ...
    @property
    def qttt(self) -> DVector4: ...
    @property
    def qttq(self) -> DVector4: ...
    @property
    def qttp(self) -> DVector4: ...
    @property
    def qtto(self) -> DVector4: ...
    @property
    def qttl(self) -> DVector4: ...
    @property
    def qtqs(self) -> DVector4: ...
    @property
    def qtqt(self) -> DVector4: ...
    @property
    def qtqq(self) -> DVector4: ...
    @property
    def qtqp(self) -> DVector4: ...
    @property
    def qtqo(self) -> DVector4: ...
    @property
    def qtql(self) -> DVector4: ...
    @property
    def qtps(self) -> DVector4: ...
    @property
    def qtpt(self) -> DVector4: ...
    @property
    def qtpq(self) -> DVector4: ...
    @property
    def qtpp(self) -> DVector4: ...
    @property
    def qtpo(self) -> DVector4: ...
    @property
    def qtpl(self) -> DVector4: ...
    @property
    def qtos(self) -> DVector4: ...
    @property
    def qtot(self) -> DVector4: ...
    @property
    def qtoq(self) -> DVector4: ...
    @property
    def qtop(self) -> DVector4: ...
    @property
    def qtoo(self) -> DVector4: ...
    @property
    def qtol(self) -> DVector4: ...
    @property
    def qtls(self) -> DVector4: ...
    @property
    def qtlt(self) -> DVector4: ...
    @property
    def qtlq(self) -> DVector4: ...
    @property
    def qtlp(self) -> DVector4: ...
    @property
    def qtlo(self) -> DVector4: ...
    @property
    def qtll(self) -> DVector4: ...
    @property
    def qqss(self) -> DVector4: ...
    @property
    def qqst(self) -> DVector4: ...
    @property
    def qqsq(self) -> DVector4: ...
    @property
    def qqsp(self) -> DVector4: ...
    @property
    def qqso(self) -> DVector4: ...
    @property
    def qqsl(self) -> DVector4: ...
    @property
    def qqts(self) -> DVector4: ...
    @property
    def qqtt(self) -> DVector4: ...
    @property
    def qqtq(self) -> DVector4: ...
    @property
    def qqtp(self) -> DVector4: ...
    @property
    def qqto(self) -> DVector4: ...
    @property
    def qqtl(self) -> DVector4: ...
    @property
    def qqqs(self) -> DVector4: ...
    @property
    def qqqt(self) -> DVector4: ...
    @property
    def qqqq(self) -> DVector4: ...
    @property
    def qqqp(self) -> DVector4: ...
    @property
    def qqqo(self) -> DVector4: ...
    @property
    def qqql(self) -> DVector4: ...
    @property
    def qqps(self) -> DVector4: ...
    @property
    def qqpt(self) -> DVector4: ...
    @property
    def qqpq(self) -> DVector4: ...
    @property
    def qqpp(self) -> DVector4: ...
    @property
    def qqpo(self) -> DVector4: ...
    @property
    def qqpl(self) -> DVector4: ...
    @property
    def qqos(self) -> DVector4: ...
    @property
    def qqot(self) -> DVector4: ...
    @property
    def qqoq(self) -> DVector4: ...
    @property
    def qqop(self) -> DVector4: ...
    @property
    def qqoo(self) -> DVector4: ...
    @property
    def qqol(self) -> DVector4: ...
    @property
    def qqls(self) -> DVector4: ...
    @property
    def qqlt(self) -> DVector4: ...
    @property
    def qqlq(self) -> DVector4: ...
    @property
    def qqlp(self) -> DVector4: ...
    @property
    def qqlo(self) -> DVector4: ...
    @property
    def qqll(self) -> DVector4: ...
    @property
    def qpss(self) -> DVector4: ...
    @property
    def qpst(self) -> DVector4: ...
    @property
    def qpsq(self) -> DVector4: ...
    @property
    def qpsp(self) -> DVector4: ...
    @property
    def qpso(self) -> DVector4: ...
    @property
    def qpsl(self) -> DVector4: ...
    @property
    def qpts(self) -> DVector4: ...
    @property
    def qptt(self) -> DVector4: ...
    @property
    def qptq(self) -> DVector4: ...
    @property
    def qptp(self) -> DVector4: ...
    @property
    def qpto(self) -> DVector4: ...
    @property
    def qptl(self) -> DVector4: ...
    @property
    def qpqs(self) -> DVector4: ...
    @property
    def qpqt(self) -> DVector4: ...
    @property
    def qpqq(self) -> DVector4: ...
    @property
    def qpqp(self) -> DVector4: ...
    @property
    def qpqo(self) -> DVector4: ...
    @property
    def qpql(self) -> DVector4: ...
    @property
    def qpps(self) -> DVector4: ...
    @property
    def qppt(self) -> DVector4: ...
    @property
    def qppq(self) -> DVector4: ...
    @property
    def qppp(self) -> DVector4: ...
    @property
    def qppo(self) -> DVector4: ...
    @property
    def qppl(self) -> DVector4: ...
    @property
    def qpos(self) -> DVector4: ...
    @property
    def qpot(self) -> DVector4: ...
    @property
    def qpoq(self) -> DVector4: ...
    @property
    def qpop(self) -> DVector4: ...
    @property
    def qpoo(self) -> DVector4: ...
    @property
    def qpol(self) -> DVector4: ...
    @property
    def qpls(self) -> DVector4: ...
    @property
    def qplt(self) -> DVector4: ...
    @property
    def qplq(self) -> DVector4: ...
    @property
    def qplp(self) -> DVector4: ...
    @property
    def qplo(self) -> DVector4: ...
    @property
    def qpll(self) -> DVector4: ...
    @property
    def qoss(self) -> DVector4: ...
    @property
    def qost(self) -> DVector4: ...
    @property
    def qosq(self) -> DVector4: ...
    @property
    def qosp(self) -> DVector4: ...
    @property
    def qoso(self) -> DVector4: ...
    @property
    def qosl(self) -> DVector4: ...
    @property
    def qots(self) -> DVector4: ...
    @property
    def qott(self) -> DVector4: ...
    @property
    def qotq(self) -> DVector4: ...
    @property
    def qotp(self) -> DVector4: ...
    @property
    def qoto(self) -> DVector4: ...
    @property
    def qotl(self) -> DVector4: ...
    @property
    def qoqs(self) -> DVector4: ...
    @property
    def qoqt(self) -> DVector4: ...
    @property
    def qoqq(self) -> DVector4: ...
    @property
    def qoqp(self) -> DVector4: ...
    @property
    def qoqo(self) -> DVector4: ...
    @property
    def qoql(self) -> DVector4: ...
    @property
    def qops(self) -> DVector4: ...
    @property
    def qopt(self) -> DVector4: ...
    @property
    def qopq(self) -> DVector4: ...
    @property
    def qopp(self) -> DVector4: ...
    @property
    def qopo(self) -> DVector4: ...
    @property
    def qopl(self) -> DVector4: ...
    @property
    def qoos(self) -> DVector4: ...
    @property
    def qoot(self) -> DVector4: ...
    @property
    def qooq(self) -> DVector4: ...
    @property
    def qoop(self) -> DVector4: ...
    @property
    def qooo(self) -> DVector4: ...
    @property
    def qool(self) -> DVector4: ...
    @property
    def qols(self) -> DVector4: ...
    @property
    def qolt(self) -> DVector4: ...
    @property
    def qolq(self) -> DVector4: ...
    @property
    def qolp(self) -> DVector4: ...
    @property
    def qolo(self) -> DVector4: ...
    @property
    def qoll(self) -> DVector4: ...
    @property
    def qlss(self) -> DVector4: ...
    @property
    def qlst(self) -> DVector4: ...
    @property
    def qlsq(self) -> DVector4: ...
    @property
    def qlsp(self) -> DVector4: ...
    @property
    def qlso(self) -> DVector4: ...
    @property
    def qlsl(self) -> DVector4: ...
    @property
    def qlts(self) -> DVector4: ...
    @property
    def qltt(self) -> DVector4: ...
    @property
    def qltq(self) -> DVector4: ...
    @property
    def qltp(self) -> DVector4: ...
    @property
    def qlto(self) -> DVector4: ...
    @property
    def qltl(self) -> DVector4: ...
    @property
    def qlqs(self) -> DVector4: ...
    @property
    def qlqt(self) -> DVector4: ...
    @property
    def qlqq(self) -> DVector4: ...
    @property
    def qlqp(self) -> DVector4: ...
    @property
    def qlqo(self) -> DVector4: ...
    @property
    def qlql(self) -> DVector4: ...
    @property
    def qlps(self) -> DVector4: ...
    @property
    def qlpt(self) -> DVector4: ...
    @property
    def qlpq(self) -> DVector4: ...
    @property
    def qlpp(self) -> DVector4: ...
    @property
    def qlpo(self) -> DVector4: ...
    @property
    def qlpl(self) -> DVector4: ...
    @property
    def qlos(self) -> DVector4: ...
    @property
    def qlot(self) -> DVector4: ...
    @property
    def qloq(self) -> DVector4: ...
    @property
    def qlop(self) -> DVector4: ...
    @property
    def qloo(self) -> DVector4: ...
    @property
    def qlol(self) -> DVector4: ...
    @property
    def qlls(self) -> DVector4: ...
    @property
    def qllt(self) -> DVector4: ...
    @property
    def qllq(self) -> DVector4: ...
    @property
    def qllp(self) -> DVector4: ...
    @property
    def qllo(self) -> DVector4: ...
    @property
    def qlll(self) -> DVector4: ...
    @property
    def psss(self) -> DVector4: ...
    @property
    def psst(self) -> DVector4: ...
    @property
    def pssq(self) -> DVector4: ...
    @property
    def pssp(self) -> DVector4: ...
    @property
    def psso(self) -> DVector4: ...
    @property
    def pssl(self) -> DVector4: ...
    @property
    def psts(self) -> DVector4: ...
    @property
    def pstt(self) -> DVector4: ...
    @property
    def pstq(self) -> DVector4: ...
    @property
    def pstp(self) -> DVector4: ...
    @property
    def psto(self) -> DVector4: ...
    @property
    def pstl(self) -> DVector4: ...
    @property
    def psqs(self) -> DVector4: ...
    @property
    def psqt(self) -> DVector4: ...
    @property
    def psqq(self) -> DVector4: ...
    @property
    def psqp(self) -> DVector4: ...
    @property
    def psqo(self) -> DVector4: ...
    @property
    def psql(self) -> DVector4: ...
    @property
    def psps(self) -> DVector4: ...
    @property
    def pspt(self) -> DVector4: ...
    @property
    def pspq(self) -> DVector4: ...
    @property
    def pspp(self) -> DVector4: ...
    @property
    def pspo(self) -> DVector4: ...
    @property
    def pspl(self) -> DVector4: ...
    @property
    def psos(self) -> DVector4: ...
    @property
    def psot(self) -> DVector4: ...
    @property
    def psoq(self) -> DVector4: ...
    @property
    def psop(self) -> DVector4: ...
    @property
    def psoo(self) -> DVector4: ...
    @property
    def psol(self) -> DVector4: ...
    @property
    def psls(self) -> DVector4: ...
    @property
    def pslt(self) -> DVector4: ...
    @property
    def pslq(self) -> DVector4: ...
    @property
    def pslp(self) -> DVector4: ...
    @property
    def pslo(self) -> DVector4: ...
    @property
    def psll(self) -> DVector4: ...
    @property
    def ptss(self) -> DVector4: ...
    @property
    def ptst(self) -> DVector4: ...
    @property
    def ptsq(self) -> DVector4: ...
    @property
    def ptsp(self) -> DVector4: ...
    @property
    def ptso(self) -> DVector4: ...
    @property
    def ptsl(self) -> DVector4: ...
    @property
    def ptts(self) -> DVector4: ...
    @property
    def pttt(self) -> DVector4: ...
    @property
    def pttq(self) -> DVector4: ...
    @property
    def pttp(self) -> DVector4: ...
    @property
    def ptto(self) -> DVector4: ...
    @property
    def pttl(self) -> DVector4: ...
    @property
    def ptqs(self) -> DVector4: ...
    @property
    def ptqt(self) -> DVector4: ...
    @property
    def ptqq(self) -> DVector4: ...
    @property
    def ptqp(self) -> DVector4: ...
    @property
    def ptqo(self) -> DVector4: ...
    @property
    def ptql(self) -> DVector4: ...
    @property
    def ptps(self) -> DVector4: ...
    @property
    def ptpt(self) -> DVector4: ...
    @property
    def ptpq(self) -> DVector4: ...
    @property
    def ptpp(self) -> DVector4: ...
    @property
    def ptpo(self) -> DVector4: ...
    @property
    def ptpl(self) -> DVector4: ...
    @property
    def ptos(self) -> DVector4: ...
    @property
    def ptot(self) -> DVector4: ...
    @property
    def ptoq(self) -> DVector4: ...
    @property
    def ptop(self) -> DVector4: ...
    @property
    def ptoo(self) -> DVector4: ...
    @property
    def ptol(self) -> DVector4: ...
    @property
    def ptls(self) -> DVector4: ...
    @property
    def ptlt(self) -> DVector4: ...
    @property
    def ptlq(self) -> DVector4: ...
    @property
    def ptlp(self) -> DVector4: ...
    @property
    def ptlo(self) -> DVector4: ...
    @property
    def ptll(self) -> DVector4: ...
    @property
    def pqss(self) -> DVector4: ...
    @property
    def pqst(self) -> DVector4: ...
    @property
    def pqsq(self) -> DVector4: ...
    @property
    def pqsp(self) -> DVector4: ...
    @property
    def pqso(self) -> DVector4: ...
    @property
    def pqsl(self) -> DVector4: ...
    @property
    def pqts(self) -> DVector4: ...
    @property
    def pqtt(self) -> DVector4: ...
    @property
    def pqtq(self) -> DVector4: ...
    @property
    def pqtp(self) -> DVector4: ...
    @property
    def pqto(self) -> DVector4: ...
    @property
    def pqtl(self) -> DVector4: ...
    @property
    def pqqs(self) -> DVector4: ...
    @property
    def pqqt(self) -> DVector4: ...
    @property
    def pqqq(self) -> DVector4: ...
    @property
    def pqqp(self) -> DVector4: ...
    @property
    def pqqo(self) -> DVector4: ...
    @property
    def pqql(self) -> DVector4: ...
    @property
    def pqps(self) -> DVector4: ...
    @property
    def pqpt(self) -> DVector4: ...
    @property
    def pqpq(self) -> DVector4: ...
    @property
    def pqpp(self) -> DVector4: ...
    @property
    def pqpo(self) -> DVector4: ...
    @property
    def pqpl(self) -> DVector4: ...
    @property
    def pqos(self) -> DVector4: ...
    @property
    def pqot(self) -> DVector4: ...
    @property
    def pqoq(self) -> DVector4: ...
    @property
    def pqop(self) -> DVector4: ...
    @property
    def pqoo(self) -> DVector4: ...
    @property
    def pqol(self) -> DVector4: ...
    @property
    def pqls(self) -> DVector4: ...
    @property
    def pqlt(self) -> DVector4: ...
    @property
    def pqlq(self) -> DVector4: ...
    @property
    def pqlp(self) -> DVector4: ...
    @property
    def pqlo(self) -> DVector4: ...
    @property
    def pqll(self) -> DVector4: ...
    @property
    def ppss(self) -> DVector4: ...
    @property
    def ppst(self) -> DVector4: ...
    @property
    def ppsq(self) -> DVector4: ...
    @property
    def ppsp(self) -> DVector4: ...
    @property
    def ppso(self) -> DVector4: ...
    @property
    def ppsl(self) -> DVector4: ...
    @property
    def ppts(self) -> DVector4: ...
    @property
    def pptt(self) -> DVector4: ...
    @property
    def pptq(self) -> DVector4: ...
    @property
    def pptp(self) -> DVector4: ...
    @property
    def ppto(self) -> DVector4: ...
    @property
    def pptl(self) -> DVector4: ...
    @property
    def ppqs(self) -> DVector4: ...
    @property
    def ppqt(self) -> DVector4: ...
    @property
    def ppqq(self) -> DVector4: ...
    @property
    def ppqp(self) -> DVector4: ...
    @property
    def ppqo(self) -> DVector4: ...
    @property
    def ppql(self) -> DVector4: ...
    @property
    def ppps(self) -> DVector4: ...
    @property
    def pppt(self) -> DVector4: ...
    @property
    def pppq(self) -> DVector4: ...
    @property
    def pppp(self) -> DVector4: ...
    @property
    def pppo(self) -> DVector4: ...
    @property
    def pppl(self) -> DVector4: ...
    @property
    def ppos(self) -> DVector4: ...
    @property
    def ppot(self) -> DVector4: ...
    @property
    def ppoq(self) -> DVector4: ...
    @property
    def ppop(self) -> DVector4: ...
    @property
    def ppoo(self) -> DVector4: ...
    @property
    def ppol(self) -> DVector4: ...
    @property
    def ppls(self) -> DVector4: ...
    @property
    def pplt(self) -> DVector4: ...
    @property
    def pplq(self) -> DVector4: ...
    @property
    def pplp(self) -> DVector4: ...
    @property
    def pplo(self) -> DVector4: ...
    @property
    def ppll(self) -> DVector4: ...
    @property
    def poss(self) -> DVector4: ...
    @property
    def post(self) -> DVector4: ...
    @property
    def posq(self) -> DVector4: ...
    @property
    def posp(self) -> DVector4: ...
    @property
    def poso(self) -> DVector4: ...
    @property
    def posl(self) -> DVector4: ...
    @property
    def pots(self) -> DVector4: ...
    @property
    def pott(self) -> DVector4: ...
    @property
    def potq(self) -> DVector4: ...
    @property
    def potp(self) -> DVector4: ...
    @property
    def poto(self) -> DVector4: ...
    @property
    def potl(self) -> DVector4: ...
    @property
    def poqs(self) -> DVector4: ...
    @property
    def poqt(self) -> DVector4: ...
    @property
    def poqq(self) -> DVector4: ...
    @property
    def poqp(self) -> DVector4: ...
    @property
    def poqo(self) -> DVector4: ...
    @property
    def poql(self) -> DVector4: ...
    @property
    def pops(self) -> DVector4: ...
    @property
    def popt(self) -> DVector4: ...
    @property
    def popq(self) -> DVector4: ...
    @property
    def popp(self) -> DVector4: ...
    @property
    def popo(self) -> DVector4: ...
    @property
    def popl(self) -> DVector4: ...
    @property
    def poos(self) -> DVector4: ...
    @property
    def poot(self) -> DVector4: ...
    @property
    def pooq(self) -> DVector4: ...
    @property
    def poop(self) -> DVector4: ...
    @property
    def pooo(self) -> DVector4: ...
    @property
    def pool(self) -> DVector4: ...
    @property
    def pols(self) -> DVector4: ...
    @property
    def polt(self) -> DVector4: ...
    @property
    def polq(self) -> DVector4: ...
    @property
    def polp(self) -> DVector4: ...
    @property
    def polo(self) -> DVector4: ...
    @property
    def poll(self) -> DVector4: ...
    @property
    def plss(self) -> DVector4: ...
    @property
    def plst(self) -> DVector4: ...
    @property
    def plsq(self) -> DVector4: ...
    @property
    def plsp(self) -> DVector4: ...
    @property
    def plso(self) -> DVector4: ...
    @property
    def plsl(self) -> DVector4: ...
    @property
    def plts(self) -> DVector4: ...
    @property
    def pltt(self) -> DVector4: ...
    @property
    def pltq(self) -> DVector4: ...
    @property
    def pltp(self) -> DVector4: ...
    @property
    def plto(self) -> DVector4: ...
    @property
    def pltl(self) -> DVector4: ...
    @property
    def plqs(self) -> DVector4: ...
    @property
    def plqt(self) -> DVector4: ...
    @property
    def plqq(self) -> DVector4: ...
    @property
    def plqp(self) -> DVector4: ...
    @property
    def plqo(self) -> DVector4: ...
    @property
    def plql(self) -> DVector4: ...
    @property
    def plps(self) -> DVector4: ...
    @property
    def plpt(self) -> DVector4: ...
    @property
    def plpq(self) -> DVector4: ...
    @property
    def plpp(self) -> DVector4: ...
    @property
    def plpo(self) -> DVector4: ...
    @property
    def plpl(self) -> DVector4: ...
    @property
    def plos(self) -> DVector4: ...
    @property
    def plot(self) -> DVector4: ...
    @property
    def ploq(self) -> DVector4: ...
    @property
    def plop(self) -> DVector4: ...
    @property
    def ploo(self) -> DVector4: ...
    @property
    def plol(self) -> DVector4: ...
    @property
    def plls(self) -> DVector4: ...
    @property
    def pllt(self) -> DVector4: ...
    @property
    def pllq(self) -> DVector4: ...
    @property
    def pllp(self) -> DVector4: ...
    @property
    def pllo(self) -> DVector4: ...
    @property
    def plll(self) -> DVector4: ...
    @property
    def osss(self) -> DVector4: ...
    @property
    def osst(self) -> DVector4: ...
    @property
    def ossq(self) -> DVector4: ...
    @property
    def ossp(self) -> DVector4: ...
    @property
    def osso(self) -> DVector4: ...
    @property
    def ossl(self) -> DVector4: ...
    @property
    def osts(self) -> DVector4: ...
    @property
    def ostt(self) -> DVector4: ...
    @property
    def ostq(self) -> DVector4: ...
    @property
    def ostp(self) -> DVector4: ...
    @property
    def osto(self) -> DVector4: ...
    @property
    def ostl(self) -> DVector4: ...
    @property
    def osqs(self) -> DVector4: ...
    @property
    def osqt(self) -> DVector4: ...
    @property
    def osqq(self) -> DVector4: ...
    @property
    def osqp(self) -> DVector4: ...
    @property
    def osqo(self) -> DVector4: ...
    @property
    def osql(self) -> DVector4: ...
    @property
    def osps(self) -> DVector4: ...
    @property
    def ospt(self) -> DVector4: ...
    @property
    def ospq(self) -> DVector4: ...
    @property
    def ospp(self) -> DVector4: ...
    @property
    def ospo(self) -> DVector4: ...
    @property
    def ospl(self) -> DVector4: ...
    @property
    def osos(self) -> DVector4: ...
    @property
    def osot(self) -> DVector4: ...
    @property
    def osoq(self) -> DVector4: ...
    @property
    def osop(self) -> DVector4: ...
    @property
    def osoo(self) -> DVector4: ...
    @property
    def osol(self) -> DVector4: ...
    @property
    def osls(self) -> DVector4: ...
    @property
    def oslt(self) -> DVector4: ...
    @property
    def oslq(self) -> DVector4: ...
    @property
    def oslp(self) -> DVector4: ...
    @property
    def oslo(self) -> DVector4: ...
    @property
    def osll(self) -> DVector4: ...
    @property
    def otss(self) -> DVector4: ...
    @property
    def otst(self) -> DVector4: ...
    @property
    def otsq(self) -> DVector4: ...
    @property
    def otsp(self) -> DVector4: ...
    @property
    def otso(self) -> DVector4: ...
    @property
    def otsl(self) -> DVector4: ...
    @property
    def otts(self) -> DVector4: ...
    @property
    def ottt(self) -> DVector4: ...
    @property
    def ottq(self) -> DVector4: ...
    @property
    def ottp(self) -> DVector4: ...
    @property
    def otto(self) -> DVector4: ...
    @property
    def ottl(self) -> DVector4: ...
    @property
    def otqs(self) -> DVector4: ...
    @property
    def otqt(self) -> DVector4: ...
    @property
    def otqq(self) -> DVector4: ...
    @property
    def otqp(self) -> DVector4: ...
    @property
    def otqo(self) -> DVector4: ...
    @property
    def otql(self) -> DVector4: ...
    @property
    def otps(self) -> DVector4: ...
    @property
    def otpt(self) -> DVector4: ...
    @property
    def otpq(self) -> DVector4: ...
    @property
    def otpp(self) -> DVector4: ...
    @property
    def otpo(self) -> DVector4: ...
    @property
    def otpl(self) -> DVector4: ...
    @property
    def otos(self) -> DVector4: ...
    @property
    def otot(self) -> DVector4: ...
    @property
    def otoq(self) -> DVector4: ...
    @property
    def otop(self) -> DVector4: ...
    @property
    def otoo(self) -> DVector4: ...
    @property
    def otol(self) -> DVector4: ...
    @property
    def otls(self) -> DVector4: ...
    @property
    def otlt(self) -> DVector4: ...
    @property
    def otlq(self) -> DVector4: ...
    @property
    def otlp(self) -> DVector4: ...
    @property
    def otlo(self) -> DVector4: ...
    @property
    def otll(self) -> DVector4: ...
    @property
    def oqss(self) -> DVector4: ...
    @property
    def oqst(self) -> DVector4: ...
    @property
    def oqsq(self) -> DVector4: ...
    @property
    def oqsp(self) -> DVector4: ...
    @property
    def oqso(self) -> DVector4: ...
    @property
    def oqsl(self) -> DVector4: ...
    @property
    def oqts(self) -> DVector4: ...
    @property
    def oqtt(self) -> DVector4: ...
    @property
    def oqtq(self) -> DVector4: ...
    @property
    def oqtp(self) -> DVector4: ...
    @property
    def oqto(self) -> DVector4: ...
    @property
    def oqtl(self) -> DVector4: ...
    @property
    def oqqs(self) -> DVector4: ...
    @property
    def oqqt(self) -> DVector4: ...
    @property
    def oqqq(self) -> DVector4: ...
    @property
    def oqqp(self) -> DVector4: ...
    @property
    def oqqo(self) -> DVector4: ...
    @property
    def oqql(self) -> DVector4: ...
    @property
    def oqps(self) -> DVector4: ...
    @property
    def oqpt(self) -> DVector4: ...
    @property
    def oqpq(self) -> DVector4: ...
    @property
    def oqpp(self) -> DVector4: ...
    @property
    def oqpo(self) -> DVector4: ...
    @property
    def oqpl(self) -> DVector4: ...
    @property
    def oqos(self) -> DVector4: ...
    @property
    def oqot(self) -> DVector4: ...
    @property
    def oqoq(self) -> DVector4: ...
    @property
    def oqop(self) -> DVector4: ...
    @property
    def oqoo(self) -> DVector4: ...
    @property
    def oqol(self) -> DVector4: ...
    @property
    def oqls(self) -> DVector4: ...
    @property
    def oqlt(self) -> DVector4: ...
    @property
    def oqlq(self) -> DVector4: ...
    @property
    def oqlp(self) -> DVector4: ...
    @property
    def oqlo(self) -> DVector4: ...
    @property
    def oqll(self) -> DVector4: ...
    @property
    def opss(self) -> DVector4: ...
    @property
    def opst(self) -> DVector4: ...
    @property
    def opsq(self) -> DVector4: ...
    @property
    def opsp(self) -> DVector4: ...
    @property
    def opso(self) -> DVector4: ...
    @property
    def opsl(self) -> DVector4: ...
    @property
    def opts(self) -> DVector4: ...
    @property
    def optt(self) -> DVector4: ...
    @property
    def optq(self) -> DVector4: ...
    @property
    def optp(self) -> DVector4: ...
    @property
    def opto(self) -> DVector4: ...
    @property
    def optl(self) -> DVector4: ...
    @property
    def opqs(self) -> DVector4: ...
    @property
    def opqt(self) -> DVector4: ...
    @property
    def opqq(self) -> DVector4: ...
    @property
    def opqp(self) -> DVector4: ...
    @property
    def opqo(self) -> DVector4: ...
    @property
    def opql(self) -> DVector4: ...
    @property
    def opps(self) -> DVector4: ...
    @property
    def oppt(self) -> DVector4: ...
    @property
    def oppq(self) -> DVector4: ...
    @property
    def oppp(self) -> DVector4: ...
    @property
    def oppo(self) -> DVector4: ...
    @property
    def oppl(self) -> DVector4: ...
    @property
    def opos(self) -> DVector4: ...
    @property
    def opot(self) -> DVector4: ...
    @property
    def opoq(self) -> DVector4: ...
    @property
    def opop(self) -> DVector4: ...
    @property
    def opoo(self) -> DVector4: ...
    @property
    def opol(self) -> DVector4: ...
    @property
    def opls(self) -> DVector4: ...
    @property
    def oplt(self) -> DVector4: ...
    @property
    def oplq(self) -> DVector4: ...
    @property
    def oplp(self) -> DVector4: ...
    @property
    def oplo(self) -> DVector4: ...
    @property
    def opll(self) -> DVector4: ...
    @property
    def ooss(self) -> DVector4: ...
    @property
    def oost(self) -> DVector4: ...
    @property
    def oosq(self) -> DVector4: ...
    @property
    def oosp(self) -> DVector4: ...
    @property
    def ooso(self) -> DVector4: ...
    @property
    def oosl(self) -> DVector4: ...
    @property
    def oots(self) -> DVector4: ...
    @property
    def oott(self) -> DVector4: ...
    @property
    def ootq(self) -> DVector4: ...
    @property
    def ootp(self) -> DVector4: ...
    @property
    def ooto(self) -> DVector4: ...
    @property
    def ootl(self) -> DVector4: ...
    @property
    def ooqs(self) -> DVector4: ...
    @property
    def ooqt(self) -> DVector4: ...
    @property
    def ooqq(self) -> DVector4: ...
    @property
    def ooqp(self) -> DVector4: ...
    @property
    def ooqo(self) -> DVector4: ...
    @property
    def ooql(self) -> DVector4: ...
    @property
    def oops(self) -> DVector4: ...
    @property
    def oopt(self) -> DVector4: ...
    @property
    def oopq(self) -> DVector4: ...
    @property
    def oopp(self) -> DVector4: ...
    @property
    def oopo(self) -> DVector4: ...
    @property
    def oopl(self) -> DVector4: ...
    @property
    def ooos(self) -> DVector4: ...
    @property
    def ooot(self) -> DVector4: ...
    @property
    def oooq(self) -> DVector4: ...
    @property
    def ooop(self) -> DVector4: ...
    @property
    def ools(self) -> DVector4: ...
    @property
    def oolt(self) -> DVector4: ...
    @property
    def oolq(self) -> DVector4: ...
    @property
    def oolp(self) -> DVector4: ...
    @property
    def olss(self) -> DVector4: ...
    @property
    def olst(self) -> DVector4: ...
    @property
    def olsq(self) -> DVector4: ...
    @property
    def olsp(self) -> DVector4: ...
    @property
    def olso(self) -> DVector4: ...
    @property
    def olsl(self) -> DVector4: ...
    @property
    def olts(self) -> DVector4: ...
    @property
    def oltt(self) -> DVector4: ...
    @property
    def oltq(self) -> DVector4: ...
    @property
    def oltp(self) -> DVector4: ...
    @property
    def olto(self) -> DVector4: ...
    @property
    def oltl(self) -> DVector4: ...
    @property
    def olqs(self) -> DVector4: ...
    @property
    def olqt(self) -> DVector4: ...
    @property
    def olqq(self) -> DVector4: ...
    @property
    def olqp(self) -> DVector4: ...
    @property
    def olqo(self) -> DVector4: ...
    @property
    def olql(self) -> DVector4: ...
    @property
    def olps(self) -> DVector4: ...
    @property
    def olpt(self) -> DVector4: ...
    @property
    def olpq(self) -> DVector4: ...
    @property
    def olpp(self) -> DVector4: ...
    @property
    def olpo(self) -> DVector4: ...
    @property
    def olpl(self) -> DVector4: ...
    @property
    def olos(self) -> DVector4: ...
    @property
    def olot(self) -> DVector4: ...
    @property
    def oloq(self) -> DVector4: ...
    @property
    def olop(self) -> DVector4: ...
    @property
    def olls(self) -> DVector4: ...
    @property
    def ollt(self) -> DVector4: ...
    @property
    def ollq(self) -> DVector4: ...
    @property
    def ollp(self) -> DVector4: ...
    @property
    def lsss(self) -> DVector4: ...
    @property
    def lsst(self) -> DVector4: ...
    @property
    def lssq(self) -> DVector4: ...
    @property
    def lssp(self) -> DVector4: ...
    @property
    def lsso(self) -> DVector4: ...
    @property
    def lssl(self) -> DVector4: ...
    @property
    def lsts(self) -> DVector4: ...
    @property
    def lstt(self) -> DVector4: ...
    @property
    def lstq(self) -> DVector4: ...
    @property
    def lstp(self) -> DVector4: ...
    @property
    def lsto(self) -> DVector4: ...
    @property
    def lstl(self) -> DVector4: ...
    @property
    def lsqs(self) -> DVector4: ...
    @property
    def lsqt(self) -> DVector4: ...
    @property
    def lsqq(self) -> DVector4: ...
    @property
    def lsqp(self) -> DVector4: ...
    @property
    def lsqo(self) -> DVector4: ...
    @property
    def lsql(self) -> DVector4: ...
    @property
    def lsps(self) -> DVector4: ...
    @property
    def lspt(self) -> DVector4: ...
    @property
    def lspq(self) -> DVector4: ...
    @property
    def lspp(self) -> DVector4: ...
    @property
    def lspo(self) -> DVector4: ...
    @property
    def lspl(self) -> DVector4: ...
    @property
    def lsos(self) -> DVector4: ...
    @property
    def lsot(self) -> DVector4: ...
    @property
    def lsoq(self) -> DVector4: ...
    @property
    def lsop(self) -> DVector4: ...
    @property
    def lsoo(self) -> DVector4: ...
    @property
    def lsol(self) -> DVector4: ...
    @property
    def lsls(self) -> DVector4: ...
    @property
    def lslt(self) -> DVector4: ...
    @property
    def lslq(self) -> DVector4: ...
    @property
    def lslp(self) -> DVector4: ...
    @property
    def lslo(self) -> DVector4: ...
    @property
    def lsll(self) -> DVector4: ...
    @property
    def ltss(self) -> DVector4: ...
    @property
    def ltst(self) -> DVector4: ...
    @property
    def ltsq(self) -> DVector4: ...
    @property
    def ltsp(self) -> DVector4: ...
    @property
    def ltso(self) -> DVector4: ...
    @property
    def ltsl(self) -> DVector4: ...
    @property
    def ltts(self) -> DVector4: ...
    @property
    def lttt(self) -> DVector4: ...
    @property
    def lttq(self) -> DVector4: ...
    @property
    def lttp(self) -> DVector4: ...
    @property
    def ltto(self) -> DVector4: ...
    @property
    def lttl(self) -> DVector4: ...
    @property
    def ltqs(self) -> DVector4: ...
    @property
    def ltqt(self) -> DVector4: ...
    @property
    def ltqq(self) -> DVector4: ...
    @property
    def ltqp(self) -> DVector4: ...
    @property
    def ltqo(self) -> DVector4: ...
    @property
    def ltql(self) -> DVector4: ...
    @property
    def ltps(self) -> DVector4: ...
    @property
    def ltpt(self) -> DVector4: ...
    @property
    def ltpq(self) -> DVector4: ...
    @property
    def ltpp(self) -> DVector4: ...
    @property
    def ltpo(self) -> DVector4: ...
    @property
    def ltpl(self) -> DVector4: ...
    @property
    def ltos(self) -> DVector4: ...
    @property
    def ltot(self) -> DVector4: ...
    @property
    def ltoq(self) -> DVector4: ...
    @property
    def ltop(self) -> DVector4: ...
    @property
    def ltoo(self) -> DVector4: ...
    @property
    def ltol(self) -> DVector4: ...
    @property
    def ltls(self) -> DVector4: ...
    @property
    def ltlt(self) -> DVector4: ...
    @property
    def ltlq(self) -> DVector4: ...
    @property
    def ltlp(self) -> DVector4: ...
    @property
    def ltlo(self) -> DVector4: ...
    @property
    def ltll(self) -> DVector4: ...
    @property
    def lqss(self) -> DVector4: ...
    @property
    def lqst(self) -> DVector4: ...
    @property
    def lqsq(self) -> DVector4: ...
    @property
    def lqsp(self) -> DVector4: ...
    @property
    def lqso(self) -> DVector4: ...
    @property
    def lqsl(self) -> DVector4: ...
    @property
    def lqts(self) -> DVector4: ...
    @property
    def lqtt(self) -> DVector4: ...
    @property
    def lqtq(self) -> DVector4: ...
    @property
    def lqtp(self) -> DVector4: ...
    @property
    def lqto(self) -> DVector4: ...
    @property
    def lqtl(self) -> DVector4: ...
    @property
    def lqqs(self) -> DVector4: ...
    @property
    def lqqt(self) -> DVector4: ...
    @property
    def lqqq(self) -> DVector4: ...
    @property
    def lqqp(self) -> DVector4: ...
    @property
    def lqqo(self) -> DVector4: ...
    @property
    def lqql(self) -> DVector4: ...
    @property
    def lqps(self) -> DVector4: ...
    @property
    def lqpt(self) -> DVector4: ...
    @property
    def lqpq(self) -> DVector4: ...
    @property
    def lqpp(self) -> DVector4: ...
    @property
    def lqpo(self) -> DVector4: ...
    @property
    def lqpl(self) -> DVector4: ...
    @property
    def lqos(self) -> DVector4: ...
    @property
    def lqot(self) -> DVector4: ...
    @property
    def lqoq(self) -> DVector4: ...
    @property
    def lqop(self) -> DVector4: ...
    @property
    def lqoo(self) -> DVector4: ...
    @property
    def lqol(self) -> DVector4: ...
    @property
    def lqls(self) -> DVector4: ...
    @property
    def lqlt(self) -> DVector4: ...
    @property
    def lqlq(self) -> DVector4: ...
    @property
    def lqlp(self) -> DVector4: ...
    @property
    def lqlo(self) -> DVector4: ...
    @property
    def lqll(self) -> DVector4: ...
    @property
    def lpss(self) -> DVector4: ...
    @property
    def lpst(self) -> DVector4: ...
    @property
    def lpsq(self) -> DVector4: ...
    @property
    def lpsp(self) -> DVector4: ...
    @property
    def lpso(self) -> DVector4: ...
    @property
    def lpsl(self) -> DVector4: ...
    @property
    def lpts(self) -> DVector4: ...
    @property
    def lptt(self) -> DVector4: ...
    @property
    def lptq(self) -> DVector4: ...
    @property
    def lptp(self) -> DVector4: ...
    @property
    def lpto(self) -> DVector4: ...
    @property
    def lptl(self) -> DVector4: ...
    @property
    def lpqs(self) -> DVector4: ...
    @property
    def lpqt(self) -> DVector4: ...
    @property
    def lpqq(self) -> DVector4: ...
    @property
    def lpqp(self) -> DVector4: ...
    @property
    def lpqo(self) -> DVector4: ...
    @property
    def lpql(self) -> DVector4: ...
    @property
    def lpps(self) -> DVector4: ...
    @property
    def lppt(self) -> DVector4: ...
    @property
    def lppq(self) -> DVector4: ...
    @property
    def lppp(self) -> DVector4: ...
    @property
    def lppo(self) -> DVector4: ...
    @property
    def lppl(self) -> DVector4: ...
    @property
    def lpos(self) -> DVector4: ...
    @property
    def lpot(self) -> DVector4: ...
    @property
    def lpoq(self) -> DVector4: ...
    @property
    def lpop(self) -> DVector4: ...
    @property
    def lpoo(self) -> DVector4: ...
    @property
    def lpol(self) -> DVector4: ...
    @property
    def lpls(self) -> DVector4: ...
    @property
    def lplt(self) -> DVector4: ...
    @property
    def lplq(self) -> DVector4: ...
    @property
    def lplp(self) -> DVector4: ...
    @property
    def lplo(self) -> DVector4: ...
    @property
    def lpll(self) -> DVector4: ...
    @property
    def loss(self) -> DVector4: ...
    @property
    def lost(self) -> DVector4: ...
    @property
    def losq(self) -> DVector4: ...
    @property
    def losp(self) -> DVector4: ...
    @property
    def loso(self) -> DVector4: ...
    @property
    def losl(self) -> DVector4: ...
    @property
    def lots(self) -> DVector4: ...
    @property
    def lott(self) -> DVector4: ...
    @property
    def lotq(self) -> DVector4: ...
    @property
    def lotp(self) -> DVector4: ...
    @property
    def loto(self) -> DVector4: ...
    @property
    def lotl(self) -> DVector4: ...
    @property
    def loqs(self) -> DVector4: ...
    @property
    def loqt(self) -> DVector4: ...
    @property
    def loqq(self) -> DVector4: ...
    @property
    def loqp(self) -> DVector4: ...
    @property
    def loqo(self) -> DVector4: ...
    @property
    def loql(self) -> DVector4: ...
    @property
    def lops(self) -> DVector4: ...
    @property
    def lopt(self) -> DVector4: ...
    @property
    def lopq(self) -> DVector4: ...
    @property
    def lopp(self) -> DVector4: ...
    @property
    def lopo(self) -> DVector4: ...
    @property
    def lopl(self) -> DVector4: ...
    @property
    def loos(self) -> DVector4: ...
    @property
    def loot(self) -> DVector4: ...
    @property
    def looq(self) -> DVector4: ...
    @property
    def loop(self) -> DVector4: ...
    @property
    def lols(self) -> DVector4: ...
    @property
    def lolt(self) -> DVector4: ...
    @property
    def lolq(self) -> DVector4: ...
    @property
    def lolp(self) -> DVector4: ...
    @property
    def llss(self) -> DVector4: ...
    @property
    def llst(self) -> DVector4: ...
    @property
    def llsq(self) -> DVector4: ...
    @property
    def llsp(self) -> DVector4: ...
    @property
    def llso(self) -> DVector4: ...
    @property
    def llsl(self) -> DVector4: ...
    @property
    def llts(self) -> DVector4: ...
    @property
    def lltt(self) -> DVector4: ...
    @property
    def lltq(self) -> DVector4: ...
    @property
    def lltp(self) -> DVector4: ...
    @property
    def llto(self) -> DVector4: ...
    @property
    def lltl(self) -> DVector4: ...
    @property
    def llqs(self) -> DVector4: ...
    @property
    def llqt(self) -> DVector4: ...
    @property
    def llqq(self) -> DVector4: ...
    @property
    def llqp(self) -> DVector4: ...
    @property
    def llqo(self) -> DVector4: ...
    @property
    def llql(self) -> DVector4: ...
    @property
    def llps(self) -> DVector4: ...
    @property
    def llpt(self) -> DVector4: ...
    @property
    def llpq(self) -> DVector4: ...
    @property
    def llpp(self) -> DVector4: ...
    @property
    def llpo(self) -> DVector4: ...
    @property
    def llpl(self) -> DVector4: ...
    @property
    def llos(self) -> DVector4: ...
    @property
    def llot(self) -> DVector4: ...
    @property
    def lloq(self) -> DVector4: ...
    @property
    def llop(self) -> DVector4: ...
    @property
    def llls(self) -> DVector4: ...
    @property
    def lllt(self) -> DVector4: ...
    @property
    def lllq(self) -> DVector4: ...
    @property
    def lllp(self) -> DVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def v(self) -> float: ...
    @property
    def uu(self) -> DVector2: ...
    @property
    def uv(self) -> DVector2: ...
    @property
    def uo(self) -> DVector2: ...
    @property
    def ul(self) -> DVector2: ...
    @property
    def vu(self) -> DVector2: ...
    @property
    def vv(self) -> DVector2: ...
    @property
    def vo(self) -> DVector2: ...
    @property
    def vl(self) -> DVector2: ...
    @property
    def ou(self) -> DVector2: ...
    @property
    def ov(self) -> DVector2: ...
    @property
    def lu(self) -> DVector2: ...
    @property
    def lv(self) -> DVector2: ...
    @property
    def uuu(self) -> DVector3: ...
    @property
    def uuv(self) -> DVector3: ...
    @property
    def uuo(self) -> DVector3: ...
    @property
    def uul(self) -> DVector3: ...
    @property
    def uvu(self) -> DVector3: ...
    @property
    def uvv(self) -> DVector3: ...
    @property
    def uvo(self) -> DVector3: ...
    @property
    def uvl(self) -> DVector3: ...
    @property
    def uou(self) -> DVector3: ...
    @property
    def uov(self) -> DVector3: ...
    @property
    def uoo(self) -> DVector3: ...
    @property
    def uol(self) -> DVector3: ...
    @property
    def ulu(self) -> DVector3: ...
    @property
    def ulv(self) -> DVector3: ...
    @property
    def ulo(self) -> DVector3: ...
    @property
    def ull(self) -> DVector3: ...
    @property
    def vuu(self) -> DVector3: ...
    @property
    def vuv(self) -> DVector3: ...
    @property
    def vuo(self) -> DVector3: ...
    @property
    def vul(self) -> DVector3: ...
    @property
    def vvu(self) -> DVector3: ...
    @property
    def vvv(self) -> DVector3: ...
    @property
    def vvo(self) -> DVector3: ...
    @property
    def vvl(self) -> DVector3: ...
    @property
    def vou(self) -> DVector3: ...
    @property
    def vov(self) -> DVector3: ...
    @property
    def voo(self) -> DVector3: ...
    @property
    def vol(self) -> DVector3: ...
    @property
    def vlu(self) -> DVector3: ...
    @property
    def vlv(self) -> DVector3: ...
    @property
    def vlo(self) -> DVector3: ...
    @property
    def vll(self) -> DVector3: ...
    @property
    def ouu(self) -> DVector3: ...
    @property
    def ouv(self) -> DVector3: ...
    @property
    def ouo(self) -> DVector3: ...
    @property
    def oul(self) -> DVector3: ...
    @property
    def ovu(self) -> DVector3: ...
    @property
    def ovv(self) -> DVector3: ...
    @property
    def ovo(self) -> DVector3: ...
    @property
    def ovl(self) -> DVector3: ...
    @property
    def oou(self) -> DVector3: ...
    @property
    def oov(self) -> DVector3: ...
    @property
    def olu(self) -> DVector3: ...
    @property
    def olv(self) -> DVector3: ...
    @property
    def luu(self) -> DVector3: ...
    @property
    def luv(self) -> DVector3: ...
    @property
    def luo(self) -> DVector3: ...
    @property
    def lul(self) -> DVector3: ...
    @property
    def lvu(self) -> DVector3: ...
    @property
    def lvv(self) -> DVector3: ...
    @property
    def lvo(self) -> DVector3: ...
    @property
    def lvl(self) -> DVector3: ...
    @property
    def lou(self) -> DVector3: ...
    @property
    def lov(self) -> DVector3: ...
    @property
    def llu(self) -> DVector3: ...
    @property
    def llv(self) -> DVector3: ...
    @property
    def uuuu(self) -> DVector4: ...
    @property
    def uuuv(self) -> DVector4: ...
    @property
    def uuuo(self) -> DVector4: ...
    @property
    def uuul(self) -> DVector4: ...
    @property
    def uuvu(self) -> DVector4: ...
    @property
    def uuvv(self) -> DVector4: ...
    @property
    def uuvo(self) -> DVector4: ...
    @property
    def uuvl(self) -> DVector4: ...
    @property
    def uuou(self) -> DVector4: ...
    @property
    def uuov(self) -> DVector4: ...
    @property
    def uuoo(self) -> DVector4: ...
    @property
    def uuol(self) -> DVector4: ...
    @property
    def uulu(self) -> DVector4: ...
    @property
    def uulv(self) -> DVector4: ...
    @property
    def uulo(self) -> DVector4: ...
    @property
    def uull(self) -> DVector4: ...
    @property
    def uvuu(self) -> DVector4: ...
    @property
    def uvuv(self) -> DVector4: ...
    @property
    def uvuo(self) -> DVector4: ...
    @property
    def uvul(self) -> DVector4: ...
    @property
    def uvvu(self) -> DVector4: ...
    @property
    def uvvv(self) -> DVector4: ...
    @property
    def uvvo(self) -> DVector4: ...
    @property
    def uvvl(self) -> DVector4: ...
    @property
    def uvou(self) -> DVector4: ...
    @property
    def uvov(self) -> DVector4: ...
    @property
    def uvoo(self) -> DVector4: ...
    @property
    def uvol(self) -> DVector4: ...
    @property
    def uvlu(self) -> DVector4: ...
    @property
    def uvlv(self) -> DVector4: ...
    @property
    def uvlo(self) -> DVector4: ...
    @property
    def uvll(self) -> DVector4: ...
    @property
    def uouu(self) -> DVector4: ...
    @property
    def uouv(self) -> DVector4: ...
    @property
    def uouo(self) -> DVector4: ...
    @property
    def uoul(self) -> DVector4: ...
    @property
    def uovu(self) -> DVector4: ...
    @property
    def uovv(self) -> DVector4: ...
    @property
    def uovo(self) -> DVector4: ...
    @property
    def uovl(self) -> DVector4: ...
    @property
    def uoou(self) -> DVector4: ...
    @property
    def uoov(self) -> DVector4: ...
    @property
    def uooo(self) -> DVector4: ...
    @property
    def uool(self) -> DVector4: ...
    @property
    def uolu(self) -> DVector4: ...
    @property
    def uolv(self) -> DVector4: ...
    @property
    def uolo(self) -> DVector4: ...
    @property
    def uoll(self) -> DVector4: ...
    @property
    def uluu(self) -> DVector4: ...
    @property
    def uluv(self) -> DVector4: ...
    @property
    def uluo(self) -> DVector4: ...
    @property
    def ulul(self) -> DVector4: ...
    @property
    def ulvu(self) -> DVector4: ...
    @property
    def ulvv(self) -> DVector4: ...
    @property
    def ulvo(self) -> DVector4: ...
    @property
    def ulvl(self) -> DVector4: ...
    @property
    def ulou(self) -> DVector4: ...
    @property
    def ulov(self) -> DVector4: ...
    @property
    def uloo(self) -> DVector4: ...
    @property
    def ulol(self) -> DVector4: ...
    @property
    def ullu(self) -> DVector4: ...
    @property
    def ullv(self) -> DVector4: ...
    @property
    def ullo(self) -> DVector4: ...
    @property
    def ulll(self) -> DVector4: ...
    @property
    def vuuu(self) -> DVector4: ...
    @property
    def vuuv(self) -> DVector4: ...
    @property
    def vuuo(self) -> DVector4: ...
    @property
    def vuul(self) -> DVector4: ...
    @property
    def vuvu(self) -> DVector4: ...
    @property
    def vuvv(self) -> DVector4: ...
    @property
    def vuvo(self) -> DVector4: ...
    @property
    def vuvl(self) -> DVector4: ...
    @property
    def vuou(self) -> DVector4: ...
    @property
    def vuov(self) -> DVector4: ...
    @property
    def vuoo(self) -> DVector4: ...
    @property
    def vuol(self) -> DVector4: ...
    @property
    def vulu(self) -> DVector4: ...
    @property
    def vulv(self) -> DVector4: ...
    @property
    def vulo(self) -> DVector4: ...
    @property
    def vull(self) -> DVector4: ...
    @property
    def vvuu(self) -> DVector4: ...
    @property
    def vvuv(self) -> DVector4: ...
    @property
    def vvuo(self) -> DVector4: ...
    @property
    def vvul(self) -> DVector4: ...
    @property
    def vvvu(self) -> DVector4: ...
    @property
    def vvvv(self) -> DVector4: ...
    @property
    def vvvo(self) -> DVector4: ...
    @property
    def vvvl(self) -> DVector4: ...
    @property
    def vvou(self) -> DVector4: ...
    @property
    def vvov(self) -> DVector4: ...
    @property
    def vvoo(self) -> DVector4: ...
    @property
    def vvol(self) -> DVector4: ...
    @property
    def vvlu(self) -> DVector4: ...
    @property
    def vvlv(self) -> DVector4: ...
    @property
    def vvlo(self) -> DVector4: ...
    @property
    def vvll(self) -> DVector4: ...
    @property
    def vouu(self) -> DVector4: ...
    @property
    def vouv(self) -> DVector4: ...
    @property
    def vouo(self) -> DVector4: ...
    @property
    def voul(self) -> DVector4: ...
    @property
    def vovu(self) -> DVector4: ...
    @property
    def vovv(self) -> DVector4: ...
    @property
    def vovo(self) -> DVector4: ...
    @property
    def vovl(self) -> DVector4: ...
    @property
    def voou(self) -> DVector4: ...
    @property
    def voov(self) -> DVector4: ...
    @property
    def vooo(self) -> DVector4: ...
    @property
    def vool(self) -> DVector4: ...
    @property
    def volu(self) -> DVector4: ...
    @property
    def volv(self) -> DVector4: ...
    @property
    def volo(self) -> DVector4: ...
    @property
    def voll(self) -> DVector4: ...
    @property
    def vluu(self) -> DVector4: ...
    @property
    def vluv(self) -> DVector4: ...
    @property
    def vluo(self) -> DVector4: ...
    @property
    def vlul(self) -> DVector4: ...
    @property
    def vlvu(self) -> DVector4: ...
    @property
    def vlvv(self) -> DVector4: ...
    @property
    def vlvo(self) -> DVector4: ...
    @property
    def vlvl(self) -> DVector4: ...
    @property
    def vlou(self) -> DVector4: ...
    @property
    def vlov(self) -> DVector4: ...
    @property
    def vloo(self) -> DVector4: ...
    @property
    def vlol(self) -> DVector4: ...
    @property
    def vllu(self) -> DVector4: ...
    @property
    def vllv(self) -> DVector4: ...
    @property
    def vllo(self) -> DVector4: ...
    @property
    def vlll(self) -> DVector4: ...
    @property
    def ouuu(self) -> DVector4: ...
    @property
    def ouuv(self) -> DVector4: ...
    @property
    def ouuo(self) -> DVector4: ...
    @property
    def ouul(self) -> DVector4: ...
    @property
    def ouvu(self) -> DVector4: ...
    @property
    def ouvv(self) -> DVector4: ...
    @property
    def ouvo(self) -> DVector4: ...
    @property
    def ouvl(self) -> DVector4: ...
    @property
    def ouou(self) -> DVector4: ...
    @property
    def ouov(self) -> DVector4: ...
    @property
    def ouoo(self) -> DVector4: ...
    @property
    def ouol(self) -> DVector4: ...
    @property
    def oulu(self) -> DVector4: ...
    @property
    def oulv(self) -> DVector4: ...
    @property
    def oulo(self) -> DVector4: ...
    @property
    def oull(self) -> DVector4: ...
    @property
    def ovuu(self) -> DVector4: ...
    @property
    def ovuv(self) -> DVector4: ...
    @property
    def ovuo(self) -> DVector4: ...
    @property
    def ovul(self) -> DVector4: ...
    @property
    def ovvu(self) -> DVector4: ...
    @property
    def ovvv(self) -> DVector4: ...
    @property
    def ovvo(self) -> DVector4: ...
    @property
    def ovvl(self) -> DVector4: ...
    @property
    def ovou(self) -> DVector4: ...
    @property
    def ovov(self) -> DVector4: ...
    @property
    def ovoo(self) -> DVector4: ...
    @property
    def ovol(self) -> DVector4: ...
    @property
    def ovlu(self) -> DVector4: ...
    @property
    def ovlv(self) -> DVector4: ...
    @property
    def ovlo(self) -> DVector4: ...
    @property
    def ovll(self) -> DVector4: ...
    @property
    def oouu(self) -> DVector4: ...
    @property
    def oouv(self) -> DVector4: ...
    @property
    def oouo(self) -> DVector4: ...
    @property
    def ooul(self) -> DVector4: ...
    @property
    def oovu(self) -> DVector4: ...
    @property
    def oovv(self) -> DVector4: ...
    @property
    def oovo(self) -> DVector4: ...
    @property
    def oovl(self) -> DVector4: ...
    @property
    def ooou(self) -> DVector4: ...
    @property
    def ooov(self) -> DVector4: ...
    @property
    def oolu(self) -> DVector4: ...
    @property
    def oolv(self) -> DVector4: ...
    @property
    def oluu(self) -> DVector4: ...
    @property
    def oluv(self) -> DVector4: ...
    @property
    def oluo(self) -> DVector4: ...
    @property
    def olul(self) -> DVector4: ...
    @property
    def olvu(self) -> DVector4: ...
    @property
    def olvv(self) -> DVector4: ...
    @property
    def olvo(self) -> DVector4: ...
    @property
    def olvl(self) -> DVector4: ...
    @property
    def olou(self) -> DVector4: ...
    @property
    def olov(self) -> DVector4: ...
    @property
    def ollu(self) -> DVector4: ...
    @property
    def ollv(self) -> DVector4: ...
    @property
    def luuu(self) -> DVector4: ...
    @property
    def luuv(self) -> DVector4: ...
    @property
    def luuo(self) -> DVector4: ...
    @property
    def luul(self) -> DVector4: ...
    @property
    def luvu(self) -> DVector4: ...
    @property
    def luvv(self) -> DVector4: ...
    @property
    def luvo(self) -> DVector4: ...
    @property
    def luvl(self) -> DVector4: ...
    @property
    def luou(self) -> DVector4: ...
    @property
    def luov(self) -> DVector4: ...
    @property
    def luoo(self) -> DVector4: ...
    @property
    def luol(self) -> DVector4: ...
    @property
    def lulu(self) -> DVector4: ...
    @property
    def lulv(self) -> DVector4: ...
    @property
    def lulo(self) -> DVector4: ...
    @property
    def lull(self) -> DVector4: ...
    @property
    def lvuu(self) -> DVector4: ...
    @property
    def lvuv(self) -> DVector4: ...
    @property
    def lvuo(self) -> DVector4: ...
    @property
    def lvul(self) -> DVector4: ...
    @property
    def lvvu(self) -> DVector4: ...
    @property
    def lvvv(self) -> DVector4: ...
    @property
    def lvvo(self) -> DVector4: ...
    @property
    def lvvl(self) -> DVector4: ...
    @property
    def lvou(self) -> DVector4: ...
    @property
    def lvov(self) -> DVector4: ...
    @property
    def lvoo(self) -> DVector4: ...
    @property
    def lvol(self) -> DVector4: ...
    @property
    def lvlu(self) -> DVector4: ...
    @property
    def lvlv(self) -> DVector4: ...
    @property
    def lvlo(self) -> DVector4: ...
    @property
    def lvll(self) -> DVector4: ...
    @property
    def louu(self) -> DVector4: ...
    @property
    def louv(self) -> DVector4: ...
    @property
    def louo(self) -> DVector4: ...
    @property
    def loul(self) -> DVector4: ...
    @property
    def lovu(self) -> DVector4: ...
    @property
    def lovv(self) -> DVector4: ...
    @property
    def lovo(self) -> DVector4: ...
    @property
    def lovl(self) -> DVector4: ...
    @property
    def loou(self) -> DVector4: ...
    @property
    def loov(self) -> DVector4: ...
    @property
    def lolu(self) -> DVector4: ...
    @property
    def lolv(self) -> DVector4: ...
    @property
    def lluu(self) -> DVector4: ...
    @property
    def lluv(self) -> DVector4: ...
    @property
    def lluo(self) -> DVector4: ...
    @property
    def llul(self) -> DVector4: ...
    @property
    def llvu(self) -> DVector4: ...
    @property
    def llvv(self) -> DVector4: ...
    @property
    def llvo(self) -> DVector4: ...
    @property
    def llvl(self) -> DVector4: ...
    @property
    def llou(self) -> DVector4: ...
    @property
    def llov(self) -> DVector4: ...
    @property
    def lllu(self) -> DVector4: ...
    @property
    def lllv(self) -> DVector4: ...
    @property
    def oo(self) -> DVector2: ...
    @property
    def ol(self) -> DVector2: ...
    @property
    def lo(self) -> DVector2: ...
    @property
    def ll(self) -> DVector2: ...
    @property
    def ooo(self) -> DVector3: ...
    @property
    def ool(self) -> DVector3: ...
    @property
    def olo(self) -> DVector3: ...
    @property
    def oll(self) -> DVector3: ...
    @property
    def loo(self) -> DVector3: ...
    @property
    def lol(self) -> DVector3: ...
    @property
    def llo(self) -> DVector3: ...
    @property
    def lll(self) -> DVector3: ...
    @property
    def oooo(self) -> DVector4: ...
    @property
    def oool(self) -> DVector4: ...
    @property
    def oolo(self) -> DVector4: ...
    @property
    def ooll(self) -> DVector4: ...
    @property
    def oloo(self) -> DVector4: ...
    @property
    def olol(self) -> DVector4: ...
    @property
    def ollo(self) -> DVector4: ...
    @property
    def olll(self) -> DVector4: ...
    @property
    def looo(self) -> DVector4: ...
    @property
    def lool(self) -> DVector4: ...
    @property
    def lolo(self) -> DVector4: ...
    @property
    def loll(self) -> DVector4: ...
    @property
    def lloo(self) -> DVector4: ...
    @property
    def llol(self) -> DVector4: ...
    @property
    def lllo(self) -> DVector4: ...
    @property
    def llll(self) -> DVector4: ...
    @property
    def magnitude(self) -> float: ...
    def normalize(self) -> DVector4: ...
    def distance(self, other: DVector4, /) -> float: ...
    def lerp(self, other: DVector4, x: float, /) -> DVector4: ...
    def min(self, n: Number, /) -> DVector4: ...
    def max(self, n: Number, /) -> DVector4: ...
    def clamp(self, min: Number, max: Number, /) -> DVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[DVector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector4: ...

@final
class DVector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: DVector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector4]: ...
    @overload
    def __getitem__(self, index: int) -> DVector4: ...
    @overload
    def __getitem__(self, index: slice) -> DVector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> DVector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[DVector4]: ...

@final
class FVector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[float]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FVector4) -> FVector4: ...
    @overload
    def __add__(self, other: Number) -> FVector4: ...
    @overload
    def __radd__(self, other: FVector4) -> FVector4: ...
    @overload
    def __radd__(self, other: Number) -> FVector4: ...
    @overload
    def __sub__(self, other: FVector4) -> FVector4: ...
    @overload
    def __sub__(self, other: Number) -> FVector4: ...
    @overload
    def __rsub__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rsub__(self, other: Number) -> FVector4: ...
    @overload
    def __mul__(self, other: FVector4) -> FVector4: ...
    @overload
    def __mul__(self, other: Number) -> FVector4: ...
    @overload
    def __rmul__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rmul__(self, other: Number) -> FVector4: ...
    def __matmul__(self, other: FVector4) -> float: ...
    def __rmatmul__(self, other: FVector4) -> float: ...
    @overload
    def __mod__(self, other: FVector4) -> FVector4: ...
    @overload
    def __mod__(self, other: Number) -> FVector4: ...
    @overload
    def __rmod__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rmod__(self, other: Number) -> FVector4: ...
    @overload
    def __pow__(self, other: FVector4) -> FVector4: ...
    @overload
    def __pow__(self, other: Number) -> FVector4: ...
    @overload
    def __rpow__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rpow__(self, other: Number) -> FVector4: ...
    @overload
    def __truediv__(self, other: FVector4) -> FVector4: ...
    @overload
    def __truediv__(self, other: Number) -> FVector4: ...
    @overload
    def __rtruediv__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> FVector4: ...
    def __neg__(self) -> FVector4: ...
    def __abs__(self) -> FVector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float: ...
    @property
    def w(self) -> float: ...
    @property
    def xx(self) -> FVector2: ...
    @property
    def xy(self) -> FVector2: ...
    @property
    def xz(self) -> FVector2: ...
    @property
    def xw(self) -> FVector2: ...
    @property
    def xo(self) -> FVector2: ...
    @property
    def xl(self) -> FVector2: ...
    @property
    def yx(self) -> FVector2: ...
    @property
    def yy(self) -> FVector2: ...
    @property
    def yz(self) -> FVector2: ...
    @property
    def yw(self) -> FVector2: ...
    @property
    def yo(self) -> FVector2: ...
    @property
    def yl(self) -> FVector2: ...
    @property
    def zx(self) -> FVector2: ...
    @property
    def zy(self) -> FVector2: ...
    @property
    def zz(self) -> FVector2: ...
    @property
    def zw(self) -> FVector2: ...
    @property
    def zo(self) -> FVector2: ...
    @property
    def zl(self) -> FVector2: ...
    @property
    def wx(self) -> FVector2: ...
    @property
    def wy(self) -> FVector2: ...
    @property
    def wz(self) -> FVector2: ...
    @property
    def ww(self) -> FVector2: ...
    @property
    def wo(self) -> FVector2: ...
    @property
    def wl(self) -> FVector2: ...
    @property
    def ox(self) -> FVector2: ...
    @property
    def oy(self) -> FVector2: ...
    @property
    def oz(self) -> FVector2: ...
    @property
    def ow(self) -> FVector2: ...
    @property
    def lx(self) -> FVector2: ...
    @property
    def ly(self) -> FVector2: ...
    @property
    def lz(self) -> FVector2: ...
    @property
    def lw(self) -> FVector2: ...
    @property
    def xxx(self) -> FVector3: ...
    @property
    def xxy(self) -> FVector3: ...
    @property
    def xxz(self) -> FVector3: ...
    @property
    def xxw(self) -> FVector3: ...
    @property
    def xxo(self) -> FVector3: ...
    @property
    def xxl(self) -> FVector3: ...
    @property
    def xyx(self) -> FVector3: ...
    @property
    def xyy(self) -> FVector3: ...
    @property
    def xyz(self) -> FVector3: ...
    @property
    def xyw(self) -> FVector3: ...
    @property
    def xyo(self) -> FVector3: ...
    @property
    def xyl(self) -> FVector3: ...
    @property
    def xzx(self) -> FVector3: ...
    @property
    def xzy(self) -> FVector3: ...
    @property
    def xzz(self) -> FVector3: ...
    @property
    def xzw(self) -> FVector3: ...
    @property
    def xzo(self) -> FVector3: ...
    @property
    def xzl(self) -> FVector3: ...
    @property
    def xwx(self) -> FVector3: ...
    @property
    def xwy(self) -> FVector3: ...
    @property
    def xwz(self) -> FVector3: ...
    @property
    def xww(self) -> FVector3: ...
    @property
    def xwo(self) -> FVector3: ...
    @property
    def xwl(self) -> FVector3: ...
    @property
    def xox(self) -> FVector3: ...
    @property
    def xoy(self) -> FVector3: ...
    @property
    def xoz(self) -> FVector3: ...
    @property
    def xow(self) -> FVector3: ...
    @property
    def xoo(self) -> FVector3: ...
    @property
    def xol(self) -> FVector3: ...
    @property
    def xlx(self) -> FVector3: ...
    @property
    def xly(self) -> FVector3: ...
    @property
    def xlz(self) -> FVector3: ...
    @property
    def xlw(self) -> FVector3: ...
    @property
    def xlo(self) -> FVector3: ...
    @property
    def xll(self) -> FVector3: ...
    @property
    def yxx(self) -> FVector3: ...
    @property
    def yxy(self) -> FVector3: ...
    @property
    def yxz(self) -> FVector3: ...
    @property
    def yxw(self) -> FVector3: ...
    @property
    def yxo(self) -> FVector3: ...
    @property
    def yxl(self) -> FVector3: ...
    @property
    def yyx(self) -> FVector3: ...
    @property
    def yyy(self) -> FVector3: ...
    @property
    def yyz(self) -> FVector3: ...
    @property
    def yyw(self) -> FVector3: ...
    @property
    def yyo(self) -> FVector3: ...
    @property
    def yyl(self) -> FVector3: ...
    @property
    def yzx(self) -> FVector3: ...
    @property
    def yzy(self) -> FVector3: ...
    @property
    def yzz(self) -> FVector3: ...
    @property
    def yzw(self) -> FVector3: ...
    @property
    def yzo(self) -> FVector3: ...
    @property
    def yzl(self) -> FVector3: ...
    @property
    def ywx(self) -> FVector3: ...
    @property
    def ywy(self) -> FVector3: ...
    @property
    def ywz(self) -> FVector3: ...
    @property
    def yww(self) -> FVector3: ...
    @property
    def ywo(self) -> FVector3: ...
    @property
    def ywl(self) -> FVector3: ...
    @property
    def yox(self) -> FVector3: ...
    @property
    def yoy(self) -> FVector3: ...
    @property
    def yoz(self) -> FVector3: ...
    @property
    def yow(self) -> FVector3: ...
    @property
    def yoo(self) -> FVector3: ...
    @property
    def yol(self) -> FVector3: ...
    @property
    def ylx(self) -> FVector3: ...
    @property
    def yly(self) -> FVector3: ...
    @property
    def ylz(self) -> FVector3: ...
    @property
    def ylw(self) -> FVector3: ...
    @property
    def ylo(self) -> FVector3: ...
    @property
    def yll(self) -> FVector3: ...
    @property
    def zxx(self) -> FVector3: ...
    @property
    def zxy(self) -> FVector3: ...
    @property
    def zxz(self) -> FVector3: ...
    @property
    def zxw(self) -> FVector3: ...
    @property
    def zxo(self) -> FVector3: ...
    @property
    def zxl(self) -> FVector3: ...
    @property
    def zyx(self) -> FVector3: ...
    @property
    def zyy(self) -> FVector3: ...
    @property
    def zyz(self) -> FVector3: ...
    @property
    def zyw(self) -> FVector3: ...
    @property
    def zyo(self) -> FVector3: ...
    @property
    def zyl(self) -> FVector3: ...
    @property
    def zzx(self) -> FVector3: ...
    @property
    def zzy(self) -> FVector3: ...
    @property
    def zzz(self) -> FVector3: ...
    @property
    def zzw(self) -> FVector3: ...
    @property
    def zzo(self) -> FVector3: ...
    @property
    def zzl(self) -> FVector3: ...
    @property
    def zwx(self) -> FVector3: ...
    @property
    def zwy(self) -> FVector3: ...
    @property
    def zwz(self) -> FVector3: ...
    @property
    def zww(self) -> FVector3: ...
    @property
    def zwo(self) -> FVector3: ...
    @property
    def zwl(self) -> FVector3: ...
    @property
    def zox(self) -> FVector3: ...
    @property
    def zoy(self) -> FVector3: ...
    @property
    def zoz(self) -> FVector3: ...
    @property
    def zow(self) -> FVector3: ...
    @property
    def zoo(self) -> FVector3: ...
    @property
    def zol(self) -> FVector3: ...
    @property
    def zlx(self) -> FVector3: ...
    @property
    def zly(self) -> FVector3: ...
    @property
    def zlz(self) -> FVector3: ...
    @property
    def zlw(self) -> FVector3: ...
    @property
    def zlo(self) -> FVector3: ...
    @property
    def zll(self) -> FVector3: ...
    @property
    def wxx(self) -> FVector3: ...
    @property
    def wxy(self) -> FVector3: ...
    @property
    def wxz(self) -> FVector3: ...
    @property
    def wxw(self) -> FVector3: ...
    @property
    def wxo(self) -> FVector3: ...
    @property
    def wxl(self) -> FVector3: ...
    @property
    def wyx(self) -> FVector3: ...
    @property
    def wyy(self) -> FVector3: ...
    @property
    def wyz(self) -> FVector3: ...
    @property
    def wyw(self) -> FVector3: ...
    @property
    def wyo(self) -> FVector3: ...
    @property
    def wyl(self) -> FVector3: ...
    @property
    def wzx(self) -> FVector3: ...
    @property
    def wzy(self) -> FVector3: ...
    @property
    def wzz(self) -> FVector3: ...
    @property
    def wzw(self) -> FVector3: ...
    @property
    def wzo(self) -> FVector3: ...
    @property
    def wzl(self) -> FVector3: ...
    @property
    def wwx(self) -> FVector3: ...
    @property
    def wwy(self) -> FVector3: ...
    @property
    def wwz(self) -> FVector3: ...
    @property
    def www(self) -> FVector3: ...
    @property
    def wwo(self) -> FVector3: ...
    @property
    def wwl(self) -> FVector3: ...
    @property
    def wox(self) -> FVector3: ...
    @property
    def woy(self) -> FVector3: ...
    @property
    def woz(self) -> FVector3: ...
    @property
    def wow(self) -> FVector3: ...
    @property
    def woo(self) -> FVector3: ...
    @property
    def wol(self) -> FVector3: ...
    @property
    def wlx(self) -> FVector3: ...
    @property
    def wly(self) -> FVector3: ...
    @property
    def wlz(self) -> FVector3: ...
    @property
    def wlw(self) -> FVector3: ...
    @property
    def wlo(self) -> FVector3: ...
    @property
    def wll(self) -> FVector3: ...
    @property
    def oxx(self) -> FVector3: ...
    @property
    def oxy(self) -> FVector3: ...
    @property
    def oxz(self) -> FVector3: ...
    @property
    def oxw(self) -> FVector3: ...
    @property
    def oxo(self) -> FVector3: ...
    @property
    def oxl(self) -> FVector3: ...
    @property
    def oyx(self) -> FVector3: ...
    @property
    def oyy(self) -> FVector3: ...
    @property
    def oyz(self) -> FVector3: ...
    @property
    def oyw(self) -> FVector3: ...
    @property
    def oyo(self) -> FVector3: ...
    @property
    def oyl(self) -> FVector3: ...
    @property
    def ozx(self) -> FVector3: ...
    @property
    def ozy(self) -> FVector3: ...
    @property
    def ozz(self) -> FVector3: ...
    @property
    def ozw(self) -> FVector3: ...
    @property
    def ozo(self) -> FVector3: ...
    @property
    def ozl(self) -> FVector3: ...
    @property
    def owx(self) -> FVector3: ...
    @property
    def owy(self) -> FVector3: ...
    @property
    def owz(self) -> FVector3: ...
    @property
    def oww(self) -> FVector3: ...
    @property
    def owo(self) -> FVector3: ...
    @property
    def owl(self) -> FVector3: ...
    @property
    def oox(self) -> FVector3: ...
    @property
    def ooy(self) -> FVector3: ...
    @property
    def ooz(self) -> FVector3: ...
    @property
    def oow(self) -> FVector3: ...
    @property
    def olx(self) -> FVector3: ...
    @property
    def oly(self) -> FVector3: ...
    @property
    def olz(self) -> FVector3: ...
    @property
    def olw(self) -> FVector3: ...
    @property
    def lxx(self) -> FVector3: ...
    @property
    def lxy(self) -> FVector3: ...
    @property
    def lxz(self) -> FVector3: ...
    @property
    def lxw(self) -> FVector3: ...
    @property
    def lxo(self) -> FVector3: ...
    @property
    def lxl(self) -> FVector3: ...
    @property
    def lyx(self) -> FVector3: ...
    @property
    def lyy(self) -> FVector3: ...
    @property
    def lyz(self) -> FVector3: ...
    @property
    def lyw(self) -> FVector3: ...
    @property
    def lyo(self) -> FVector3: ...
    @property
    def lyl(self) -> FVector3: ...
    @property
    def lzx(self) -> FVector3: ...
    @property
    def lzy(self) -> FVector3: ...
    @property
    def lzz(self) -> FVector3: ...
    @property
    def lzw(self) -> FVector3: ...
    @property
    def lzo(self) -> FVector3: ...
    @property
    def lzl(self) -> FVector3: ...
    @property
    def lwx(self) -> FVector3: ...
    @property
    def lwy(self) -> FVector3: ...
    @property
    def lwz(self) -> FVector3: ...
    @property
    def lww(self) -> FVector3: ...
    @property
    def lwo(self) -> FVector3: ...
    @property
    def lwl(self) -> FVector3: ...
    @property
    def lox(self) -> FVector3: ...
    @property
    def loy(self) -> FVector3: ...
    @property
    def loz(self) -> FVector3: ...
    @property
    def low(self) -> FVector3: ...
    @property
    def llx(self) -> FVector3: ...
    @property
    def lly(self) -> FVector3: ...
    @property
    def llz(self) -> FVector3: ...
    @property
    def llw(self) -> FVector3: ...
    @property
    def xxxx(self) -> FVector4: ...
    @property
    def xxxy(self) -> FVector4: ...
    @property
    def xxxz(self) -> FVector4: ...
    @property
    def xxxw(self) -> FVector4: ...
    @property
    def xxxo(self) -> FVector4: ...
    @property
    def xxxl(self) -> FVector4: ...
    @property
    def xxyx(self) -> FVector4: ...
    @property
    def xxyy(self) -> FVector4: ...
    @property
    def xxyz(self) -> FVector4: ...
    @property
    def xxyw(self) -> FVector4: ...
    @property
    def xxyo(self) -> FVector4: ...
    @property
    def xxyl(self) -> FVector4: ...
    @property
    def xxzx(self) -> FVector4: ...
    @property
    def xxzy(self) -> FVector4: ...
    @property
    def xxzz(self) -> FVector4: ...
    @property
    def xxzw(self) -> FVector4: ...
    @property
    def xxzo(self) -> FVector4: ...
    @property
    def xxzl(self) -> FVector4: ...
    @property
    def xxwx(self) -> FVector4: ...
    @property
    def xxwy(self) -> FVector4: ...
    @property
    def xxwz(self) -> FVector4: ...
    @property
    def xxww(self) -> FVector4: ...
    @property
    def xxwo(self) -> FVector4: ...
    @property
    def xxwl(self) -> FVector4: ...
    @property
    def xxox(self) -> FVector4: ...
    @property
    def xxoy(self) -> FVector4: ...
    @property
    def xxoz(self) -> FVector4: ...
    @property
    def xxow(self) -> FVector4: ...
    @property
    def xxoo(self) -> FVector4: ...
    @property
    def xxol(self) -> FVector4: ...
    @property
    def xxlx(self) -> FVector4: ...
    @property
    def xxly(self) -> FVector4: ...
    @property
    def xxlz(self) -> FVector4: ...
    @property
    def xxlw(self) -> FVector4: ...
    @property
    def xxlo(self) -> FVector4: ...
    @property
    def xxll(self) -> FVector4: ...
    @property
    def xyxx(self) -> FVector4: ...
    @property
    def xyxy(self) -> FVector4: ...
    @property
    def xyxz(self) -> FVector4: ...
    @property
    def xyxw(self) -> FVector4: ...
    @property
    def xyxo(self) -> FVector4: ...
    @property
    def xyxl(self) -> FVector4: ...
    @property
    def xyyx(self) -> FVector4: ...
    @property
    def xyyy(self) -> FVector4: ...
    @property
    def xyyz(self) -> FVector4: ...
    @property
    def xyyw(self) -> FVector4: ...
    @property
    def xyyo(self) -> FVector4: ...
    @property
    def xyyl(self) -> FVector4: ...
    @property
    def xyzx(self) -> FVector4: ...
    @property
    def xyzy(self) -> FVector4: ...
    @property
    def xyzz(self) -> FVector4: ...
    @property
    def xyzw(self) -> FVector4: ...
    @property
    def xyzo(self) -> FVector4: ...
    @property
    def xyzl(self) -> FVector4: ...
    @property
    def xywx(self) -> FVector4: ...
    @property
    def xywy(self) -> FVector4: ...
    @property
    def xywz(self) -> FVector4: ...
    @property
    def xyww(self) -> FVector4: ...
    @property
    def xywo(self) -> FVector4: ...
    @property
    def xywl(self) -> FVector4: ...
    @property
    def xyox(self) -> FVector4: ...
    @property
    def xyoy(self) -> FVector4: ...
    @property
    def xyoz(self) -> FVector4: ...
    @property
    def xyow(self) -> FVector4: ...
    @property
    def xyoo(self) -> FVector4: ...
    @property
    def xyol(self) -> FVector4: ...
    @property
    def xylx(self) -> FVector4: ...
    @property
    def xyly(self) -> FVector4: ...
    @property
    def xylz(self) -> FVector4: ...
    @property
    def xylw(self) -> FVector4: ...
    @property
    def xylo(self) -> FVector4: ...
    @property
    def xyll(self) -> FVector4: ...
    @property
    def xzxx(self) -> FVector4: ...
    @property
    def xzxy(self) -> FVector4: ...
    @property
    def xzxz(self) -> FVector4: ...
    @property
    def xzxw(self) -> FVector4: ...
    @property
    def xzxo(self) -> FVector4: ...
    @property
    def xzxl(self) -> FVector4: ...
    @property
    def xzyx(self) -> FVector4: ...
    @property
    def xzyy(self) -> FVector4: ...
    @property
    def xzyz(self) -> FVector4: ...
    @property
    def xzyw(self) -> FVector4: ...
    @property
    def xzyo(self) -> FVector4: ...
    @property
    def xzyl(self) -> FVector4: ...
    @property
    def xzzx(self) -> FVector4: ...
    @property
    def xzzy(self) -> FVector4: ...
    @property
    def xzzz(self) -> FVector4: ...
    @property
    def xzzw(self) -> FVector4: ...
    @property
    def xzzo(self) -> FVector4: ...
    @property
    def xzzl(self) -> FVector4: ...
    @property
    def xzwx(self) -> FVector4: ...
    @property
    def xzwy(self) -> FVector4: ...
    @property
    def xzwz(self) -> FVector4: ...
    @property
    def xzww(self) -> FVector4: ...
    @property
    def xzwo(self) -> FVector4: ...
    @property
    def xzwl(self) -> FVector4: ...
    @property
    def xzox(self) -> FVector4: ...
    @property
    def xzoy(self) -> FVector4: ...
    @property
    def xzoz(self) -> FVector4: ...
    @property
    def xzow(self) -> FVector4: ...
    @property
    def xzoo(self) -> FVector4: ...
    @property
    def xzol(self) -> FVector4: ...
    @property
    def xzlx(self) -> FVector4: ...
    @property
    def xzly(self) -> FVector4: ...
    @property
    def xzlz(self) -> FVector4: ...
    @property
    def xzlw(self) -> FVector4: ...
    @property
    def xzlo(self) -> FVector4: ...
    @property
    def xzll(self) -> FVector4: ...
    @property
    def xwxx(self) -> FVector4: ...
    @property
    def xwxy(self) -> FVector4: ...
    @property
    def xwxz(self) -> FVector4: ...
    @property
    def xwxw(self) -> FVector4: ...
    @property
    def xwxo(self) -> FVector4: ...
    @property
    def xwxl(self) -> FVector4: ...
    @property
    def xwyx(self) -> FVector4: ...
    @property
    def xwyy(self) -> FVector4: ...
    @property
    def xwyz(self) -> FVector4: ...
    @property
    def xwyw(self) -> FVector4: ...
    @property
    def xwyo(self) -> FVector4: ...
    @property
    def xwyl(self) -> FVector4: ...
    @property
    def xwzx(self) -> FVector4: ...
    @property
    def xwzy(self) -> FVector4: ...
    @property
    def xwzz(self) -> FVector4: ...
    @property
    def xwzw(self) -> FVector4: ...
    @property
    def xwzo(self) -> FVector4: ...
    @property
    def xwzl(self) -> FVector4: ...
    @property
    def xwwx(self) -> FVector4: ...
    @property
    def xwwy(self) -> FVector4: ...
    @property
    def xwwz(self) -> FVector4: ...
    @property
    def xwww(self) -> FVector4: ...
    @property
    def xwwo(self) -> FVector4: ...
    @property
    def xwwl(self) -> FVector4: ...
    @property
    def xwox(self) -> FVector4: ...
    @property
    def xwoy(self) -> FVector4: ...
    @property
    def xwoz(self) -> FVector4: ...
    @property
    def xwow(self) -> FVector4: ...
    @property
    def xwoo(self) -> FVector4: ...
    @property
    def xwol(self) -> FVector4: ...
    @property
    def xwlx(self) -> FVector4: ...
    @property
    def xwly(self) -> FVector4: ...
    @property
    def xwlz(self) -> FVector4: ...
    @property
    def xwlw(self) -> FVector4: ...
    @property
    def xwlo(self) -> FVector4: ...
    @property
    def xwll(self) -> FVector4: ...
    @property
    def xoxx(self) -> FVector4: ...
    @property
    def xoxy(self) -> FVector4: ...
    @property
    def xoxz(self) -> FVector4: ...
    @property
    def xoxw(self) -> FVector4: ...
    @property
    def xoxo(self) -> FVector4: ...
    @property
    def xoxl(self) -> FVector4: ...
    @property
    def xoyx(self) -> FVector4: ...
    @property
    def xoyy(self) -> FVector4: ...
    @property
    def xoyz(self) -> FVector4: ...
    @property
    def xoyw(self) -> FVector4: ...
    @property
    def xoyo(self) -> FVector4: ...
    @property
    def xoyl(self) -> FVector4: ...
    @property
    def xozx(self) -> FVector4: ...
    @property
    def xozy(self) -> FVector4: ...
    @property
    def xozz(self) -> FVector4: ...
    @property
    def xozw(self) -> FVector4: ...
    @property
    def xozo(self) -> FVector4: ...
    @property
    def xozl(self) -> FVector4: ...
    @property
    def xowx(self) -> FVector4: ...
    @property
    def xowy(self) -> FVector4: ...
    @property
    def xowz(self) -> FVector4: ...
    @property
    def xoww(self) -> FVector4: ...
    @property
    def xowo(self) -> FVector4: ...
    @property
    def xowl(self) -> FVector4: ...
    @property
    def xoox(self) -> FVector4: ...
    @property
    def xooy(self) -> FVector4: ...
    @property
    def xooz(self) -> FVector4: ...
    @property
    def xoow(self) -> FVector4: ...
    @property
    def xooo(self) -> FVector4: ...
    @property
    def xool(self) -> FVector4: ...
    @property
    def xolx(self) -> FVector4: ...
    @property
    def xoly(self) -> FVector4: ...
    @property
    def xolz(self) -> FVector4: ...
    @property
    def xolw(self) -> FVector4: ...
    @property
    def xolo(self) -> FVector4: ...
    @property
    def xoll(self) -> FVector4: ...
    @property
    def xlxx(self) -> FVector4: ...
    @property
    def xlxy(self) -> FVector4: ...
    @property
    def xlxz(self) -> FVector4: ...
    @property
    def xlxw(self) -> FVector4: ...
    @property
    def xlxo(self) -> FVector4: ...
    @property
    def xlxl(self) -> FVector4: ...
    @property
    def xlyx(self) -> FVector4: ...
    @property
    def xlyy(self) -> FVector4: ...
    @property
    def xlyz(self) -> FVector4: ...
    @property
    def xlyw(self) -> FVector4: ...
    @property
    def xlyo(self) -> FVector4: ...
    @property
    def xlyl(self) -> FVector4: ...
    @property
    def xlzx(self) -> FVector4: ...
    @property
    def xlzy(self) -> FVector4: ...
    @property
    def xlzz(self) -> FVector4: ...
    @property
    def xlzw(self) -> FVector4: ...
    @property
    def xlzo(self) -> FVector4: ...
    @property
    def xlzl(self) -> FVector4: ...
    @property
    def xlwx(self) -> FVector4: ...
    @property
    def xlwy(self) -> FVector4: ...
    @property
    def xlwz(self) -> FVector4: ...
    @property
    def xlww(self) -> FVector4: ...
    @property
    def xlwo(self) -> FVector4: ...
    @property
    def xlwl(self) -> FVector4: ...
    @property
    def xlox(self) -> FVector4: ...
    @property
    def xloy(self) -> FVector4: ...
    @property
    def xloz(self) -> FVector4: ...
    @property
    def xlow(self) -> FVector4: ...
    @property
    def xloo(self) -> FVector4: ...
    @property
    def xlol(self) -> FVector4: ...
    @property
    def xllx(self) -> FVector4: ...
    @property
    def xlly(self) -> FVector4: ...
    @property
    def xllz(self) -> FVector4: ...
    @property
    def xllw(self) -> FVector4: ...
    @property
    def xllo(self) -> FVector4: ...
    @property
    def xlll(self) -> FVector4: ...
    @property
    def yxxx(self) -> FVector4: ...
    @property
    def yxxy(self) -> FVector4: ...
    @property
    def yxxz(self) -> FVector4: ...
    @property
    def yxxw(self) -> FVector4: ...
    @property
    def yxxo(self) -> FVector4: ...
    @property
    def yxxl(self) -> FVector4: ...
    @property
    def yxyx(self) -> FVector4: ...
    @property
    def yxyy(self) -> FVector4: ...
    @property
    def yxyz(self) -> FVector4: ...
    @property
    def yxyw(self) -> FVector4: ...
    @property
    def yxyo(self) -> FVector4: ...
    @property
    def yxyl(self) -> FVector4: ...
    @property
    def yxzx(self) -> FVector4: ...
    @property
    def yxzy(self) -> FVector4: ...
    @property
    def yxzz(self) -> FVector4: ...
    @property
    def yxzw(self) -> FVector4: ...
    @property
    def yxzo(self) -> FVector4: ...
    @property
    def yxzl(self) -> FVector4: ...
    @property
    def yxwx(self) -> FVector4: ...
    @property
    def yxwy(self) -> FVector4: ...
    @property
    def yxwz(self) -> FVector4: ...
    @property
    def yxww(self) -> FVector4: ...
    @property
    def yxwo(self) -> FVector4: ...
    @property
    def yxwl(self) -> FVector4: ...
    @property
    def yxox(self) -> FVector4: ...
    @property
    def yxoy(self) -> FVector4: ...
    @property
    def yxoz(self) -> FVector4: ...
    @property
    def yxow(self) -> FVector4: ...
    @property
    def yxoo(self) -> FVector4: ...
    @property
    def yxol(self) -> FVector4: ...
    @property
    def yxlx(self) -> FVector4: ...
    @property
    def yxly(self) -> FVector4: ...
    @property
    def yxlz(self) -> FVector4: ...
    @property
    def yxlw(self) -> FVector4: ...
    @property
    def yxlo(self) -> FVector4: ...
    @property
    def yxll(self) -> FVector4: ...
    @property
    def yyxx(self) -> FVector4: ...
    @property
    def yyxy(self) -> FVector4: ...
    @property
    def yyxz(self) -> FVector4: ...
    @property
    def yyxw(self) -> FVector4: ...
    @property
    def yyxo(self) -> FVector4: ...
    @property
    def yyxl(self) -> FVector4: ...
    @property
    def yyyx(self) -> FVector4: ...
    @property
    def yyyy(self) -> FVector4: ...
    @property
    def yyyz(self) -> FVector4: ...
    @property
    def yyyw(self) -> FVector4: ...
    @property
    def yyyo(self) -> FVector4: ...
    @property
    def yyyl(self) -> FVector4: ...
    @property
    def yyzx(self) -> FVector4: ...
    @property
    def yyzy(self) -> FVector4: ...
    @property
    def yyzz(self) -> FVector4: ...
    @property
    def yyzw(self) -> FVector4: ...
    @property
    def yyzo(self) -> FVector4: ...
    @property
    def yyzl(self) -> FVector4: ...
    @property
    def yywx(self) -> FVector4: ...
    @property
    def yywy(self) -> FVector4: ...
    @property
    def yywz(self) -> FVector4: ...
    @property
    def yyww(self) -> FVector4: ...
    @property
    def yywo(self) -> FVector4: ...
    @property
    def yywl(self) -> FVector4: ...
    @property
    def yyox(self) -> FVector4: ...
    @property
    def yyoy(self) -> FVector4: ...
    @property
    def yyoz(self) -> FVector4: ...
    @property
    def yyow(self) -> FVector4: ...
    @property
    def yyoo(self) -> FVector4: ...
    @property
    def yyol(self) -> FVector4: ...
    @property
    def yylx(self) -> FVector4: ...
    @property
    def yyly(self) -> FVector4: ...
    @property
    def yylz(self) -> FVector4: ...
    @property
    def yylw(self) -> FVector4: ...
    @property
    def yylo(self) -> FVector4: ...
    @property
    def yyll(self) -> FVector4: ...
    @property
    def yzxx(self) -> FVector4: ...
    @property
    def yzxy(self) -> FVector4: ...
    @property
    def yzxz(self) -> FVector4: ...
    @property
    def yzxw(self) -> FVector4: ...
    @property
    def yzxo(self) -> FVector4: ...
    @property
    def yzxl(self) -> FVector4: ...
    @property
    def yzyx(self) -> FVector4: ...
    @property
    def yzyy(self) -> FVector4: ...
    @property
    def yzyz(self) -> FVector4: ...
    @property
    def yzyw(self) -> FVector4: ...
    @property
    def yzyo(self) -> FVector4: ...
    @property
    def yzyl(self) -> FVector4: ...
    @property
    def yzzx(self) -> FVector4: ...
    @property
    def yzzy(self) -> FVector4: ...
    @property
    def yzzz(self) -> FVector4: ...
    @property
    def yzzw(self) -> FVector4: ...
    @property
    def yzzo(self) -> FVector4: ...
    @property
    def yzzl(self) -> FVector4: ...
    @property
    def yzwx(self) -> FVector4: ...
    @property
    def yzwy(self) -> FVector4: ...
    @property
    def yzwz(self) -> FVector4: ...
    @property
    def yzww(self) -> FVector4: ...
    @property
    def yzwo(self) -> FVector4: ...
    @property
    def yzwl(self) -> FVector4: ...
    @property
    def yzox(self) -> FVector4: ...
    @property
    def yzoy(self) -> FVector4: ...
    @property
    def yzoz(self) -> FVector4: ...
    @property
    def yzow(self) -> FVector4: ...
    @property
    def yzoo(self) -> FVector4: ...
    @property
    def yzol(self) -> FVector4: ...
    @property
    def yzlx(self) -> FVector4: ...
    @property
    def yzly(self) -> FVector4: ...
    @property
    def yzlz(self) -> FVector4: ...
    @property
    def yzlw(self) -> FVector4: ...
    @property
    def yzlo(self) -> FVector4: ...
    @property
    def yzll(self) -> FVector4: ...
    @property
    def ywxx(self) -> FVector4: ...
    @property
    def ywxy(self) -> FVector4: ...
    @property
    def ywxz(self) -> FVector4: ...
    @property
    def ywxw(self) -> FVector4: ...
    @property
    def ywxo(self) -> FVector4: ...
    @property
    def ywxl(self) -> FVector4: ...
    @property
    def ywyx(self) -> FVector4: ...
    @property
    def ywyy(self) -> FVector4: ...
    @property
    def ywyz(self) -> FVector4: ...
    @property
    def ywyw(self) -> FVector4: ...
    @property
    def ywyo(self) -> FVector4: ...
    @property
    def ywyl(self) -> FVector4: ...
    @property
    def ywzx(self) -> FVector4: ...
    @property
    def ywzy(self) -> FVector4: ...
    @property
    def ywzz(self) -> FVector4: ...
    @property
    def ywzw(self) -> FVector4: ...
    @property
    def ywzo(self) -> FVector4: ...
    @property
    def ywzl(self) -> FVector4: ...
    @property
    def ywwx(self) -> FVector4: ...
    @property
    def ywwy(self) -> FVector4: ...
    @property
    def ywwz(self) -> FVector4: ...
    @property
    def ywww(self) -> FVector4: ...
    @property
    def ywwo(self) -> FVector4: ...
    @property
    def ywwl(self) -> FVector4: ...
    @property
    def ywox(self) -> FVector4: ...
    @property
    def ywoy(self) -> FVector4: ...
    @property
    def ywoz(self) -> FVector4: ...
    @property
    def ywow(self) -> FVector4: ...
    @property
    def ywoo(self) -> FVector4: ...
    @property
    def ywol(self) -> FVector4: ...
    @property
    def ywlx(self) -> FVector4: ...
    @property
    def ywly(self) -> FVector4: ...
    @property
    def ywlz(self) -> FVector4: ...
    @property
    def ywlw(self) -> FVector4: ...
    @property
    def ywlo(self) -> FVector4: ...
    @property
    def ywll(self) -> FVector4: ...
    @property
    def yoxx(self) -> FVector4: ...
    @property
    def yoxy(self) -> FVector4: ...
    @property
    def yoxz(self) -> FVector4: ...
    @property
    def yoxw(self) -> FVector4: ...
    @property
    def yoxo(self) -> FVector4: ...
    @property
    def yoxl(self) -> FVector4: ...
    @property
    def yoyx(self) -> FVector4: ...
    @property
    def yoyy(self) -> FVector4: ...
    @property
    def yoyz(self) -> FVector4: ...
    @property
    def yoyw(self) -> FVector4: ...
    @property
    def yoyo(self) -> FVector4: ...
    @property
    def yoyl(self) -> FVector4: ...
    @property
    def yozx(self) -> FVector4: ...
    @property
    def yozy(self) -> FVector4: ...
    @property
    def yozz(self) -> FVector4: ...
    @property
    def yozw(self) -> FVector4: ...
    @property
    def yozo(self) -> FVector4: ...
    @property
    def yozl(self) -> FVector4: ...
    @property
    def yowx(self) -> FVector4: ...
    @property
    def yowy(self) -> FVector4: ...
    @property
    def yowz(self) -> FVector4: ...
    @property
    def yoww(self) -> FVector4: ...
    @property
    def yowo(self) -> FVector4: ...
    @property
    def yowl(self) -> FVector4: ...
    @property
    def yoox(self) -> FVector4: ...
    @property
    def yooy(self) -> FVector4: ...
    @property
    def yooz(self) -> FVector4: ...
    @property
    def yoow(self) -> FVector4: ...
    @property
    def yooo(self) -> FVector4: ...
    @property
    def yool(self) -> FVector4: ...
    @property
    def yolx(self) -> FVector4: ...
    @property
    def yoly(self) -> FVector4: ...
    @property
    def yolz(self) -> FVector4: ...
    @property
    def yolw(self) -> FVector4: ...
    @property
    def yolo(self) -> FVector4: ...
    @property
    def yoll(self) -> FVector4: ...
    @property
    def ylxx(self) -> FVector4: ...
    @property
    def ylxy(self) -> FVector4: ...
    @property
    def ylxz(self) -> FVector4: ...
    @property
    def ylxw(self) -> FVector4: ...
    @property
    def ylxo(self) -> FVector4: ...
    @property
    def ylxl(self) -> FVector4: ...
    @property
    def ylyx(self) -> FVector4: ...
    @property
    def ylyy(self) -> FVector4: ...
    @property
    def ylyz(self) -> FVector4: ...
    @property
    def ylyw(self) -> FVector4: ...
    @property
    def ylyo(self) -> FVector4: ...
    @property
    def ylyl(self) -> FVector4: ...
    @property
    def ylzx(self) -> FVector4: ...
    @property
    def ylzy(self) -> FVector4: ...
    @property
    def ylzz(self) -> FVector4: ...
    @property
    def ylzw(self) -> FVector4: ...
    @property
    def ylzo(self) -> FVector4: ...
    @property
    def ylzl(self) -> FVector4: ...
    @property
    def ylwx(self) -> FVector4: ...
    @property
    def ylwy(self) -> FVector4: ...
    @property
    def ylwz(self) -> FVector4: ...
    @property
    def ylww(self) -> FVector4: ...
    @property
    def ylwo(self) -> FVector4: ...
    @property
    def ylwl(self) -> FVector4: ...
    @property
    def ylox(self) -> FVector4: ...
    @property
    def yloy(self) -> FVector4: ...
    @property
    def yloz(self) -> FVector4: ...
    @property
    def ylow(self) -> FVector4: ...
    @property
    def yloo(self) -> FVector4: ...
    @property
    def ylol(self) -> FVector4: ...
    @property
    def yllx(self) -> FVector4: ...
    @property
    def ylly(self) -> FVector4: ...
    @property
    def yllz(self) -> FVector4: ...
    @property
    def yllw(self) -> FVector4: ...
    @property
    def yllo(self) -> FVector4: ...
    @property
    def ylll(self) -> FVector4: ...
    @property
    def zxxx(self) -> FVector4: ...
    @property
    def zxxy(self) -> FVector4: ...
    @property
    def zxxz(self) -> FVector4: ...
    @property
    def zxxw(self) -> FVector4: ...
    @property
    def zxxo(self) -> FVector4: ...
    @property
    def zxxl(self) -> FVector4: ...
    @property
    def zxyx(self) -> FVector4: ...
    @property
    def zxyy(self) -> FVector4: ...
    @property
    def zxyz(self) -> FVector4: ...
    @property
    def zxyw(self) -> FVector4: ...
    @property
    def zxyo(self) -> FVector4: ...
    @property
    def zxyl(self) -> FVector4: ...
    @property
    def zxzx(self) -> FVector4: ...
    @property
    def zxzy(self) -> FVector4: ...
    @property
    def zxzz(self) -> FVector4: ...
    @property
    def zxzw(self) -> FVector4: ...
    @property
    def zxzo(self) -> FVector4: ...
    @property
    def zxzl(self) -> FVector4: ...
    @property
    def zxwx(self) -> FVector4: ...
    @property
    def zxwy(self) -> FVector4: ...
    @property
    def zxwz(self) -> FVector4: ...
    @property
    def zxww(self) -> FVector4: ...
    @property
    def zxwo(self) -> FVector4: ...
    @property
    def zxwl(self) -> FVector4: ...
    @property
    def zxox(self) -> FVector4: ...
    @property
    def zxoy(self) -> FVector4: ...
    @property
    def zxoz(self) -> FVector4: ...
    @property
    def zxow(self) -> FVector4: ...
    @property
    def zxoo(self) -> FVector4: ...
    @property
    def zxol(self) -> FVector4: ...
    @property
    def zxlx(self) -> FVector4: ...
    @property
    def zxly(self) -> FVector4: ...
    @property
    def zxlz(self) -> FVector4: ...
    @property
    def zxlw(self) -> FVector4: ...
    @property
    def zxlo(self) -> FVector4: ...
    @property
    def zxll(self) -> FVector4: ...
    @property
    def zyxx(self) -> FVector4: ...
    @property
    def zyxy(self) -> FVector4: ...
    @property
    def zyxz(self) -> FVector4: ...
    @property
    def zyxw(self) -> FVector4: ...
    @property
    def zyxo(self) -> FVector4: ...
    @property
    def zyxl(self) -> FVector4: ...
    @property
    def zyyx(self) -> FVector4: ...
    @property
    def zyyy(self) -> FVector4: ...
    @property
    def zyyz(self) -> FVector4: ...
    @property
    def zyyw(self) -> FVector4: ...
    @property
    def zyyo(self) -> FVector4: ...
    @property
    def zyyl(self) -> FVector4: ...
    @property
    def zyzx(self) -> FVector4: ...
    @property
    def zyzy(self) -> FVector4: ...
    @property
    def zyzz(self) -> FVector4: ...
    @property
    def zyzw(self) -> FVector4: ...
    @property
    def zyzo(self) -> FVector4: ...
    @property
    def zyzl(self) -> FVector4: ...
    @property
    def zywx(self) -> FVector4: ...
    @property
    def zywy(self) -> FVector4: ...
    @property
    def zywz(self) -> FVector4: ...
    @property
    def zyww(self) -> FVector4: ...
    @property
    def zywo(self) -> FVector4: ...
    @property
    def zywl(self) -> FVector4: ...
    @property
    def zyox(self) -> FVector4: ...
    @property
    def zyoy(self) -> FVector4: ...
    @property
    def zyoz(self) -> FVector4: ...
    @property
    def zyow(self) -> FVector4: ...
    @property
    def zyoo(self) -> FVector4: ...
    @property
    def zyol(self) -> FVector4: ...
    @property
    def zylx(self) -> FVector4: ...
    @property
    def zyly(self) -> FVector4: ...
    @property
    def zylz(self) -> FVector4: ...
    @property
    def zylw(self) -> FVector4: ...
    @property
    def zylo(self) -> FVector4: ...
    @property
    def zyll(self) -> FVector4: ...
    @property
    def zzxx(self) -> FVector4: ...
    @property
    def zzxy(self) -> FVector4: ...
    @property
    def zzxz(self) -> FVector4: ...
    @property
    def zzxw(self) -> FVector4: ...
    @property
    def zzxo(self) -> FVector4: ...
    @property
    def zzxl(self) -> FVector4: ...
    @property
    def zzyx(self) -> FVector4: ...
    @property
    def zzyy(self) -> FVector4: ...
    @property
    def zzyz(self) -> FVector4: ...
    @property
    def zzyw(self) -> FVector4: ...
    @property
    def zzyo(self) -> FVector4: ...
    @property
    def zzyl(self) -> FVector4: ...
    @property
    def zzzx(self) -> FVector4: ...
    @property
    def zzzy(self) -> FVector4: ...
    @property
    def zzzz(self) -> FVector4: ...
    @property
    def zzzw(self) -> FVector4: ...
    @property
    def zzzo(self) -> FVector4: ...
    @property
    def zzzl(self) -> FVector4: ...
    @property
    def zzwx(self) -> FVector4: ...
    @property
    def zzwy(self) -> FVector4: ...
    @property
    def zzwz(self) -> FVector4: ...
    @property
    def zzww(self) -> FVector4: ...
    @property
    def zzwo(self) -> FVector4: ...
    @property
    def zzwl(self) -> FVector4: ...
    @property
    def zzox(self) -> FVector4: ...
    @property
    def zzoy(self) -> FVector4: ...
    @property
    def zzoz(self) -> FVector4: ...
    @property
    def zzow(self) -> FVector4: ...
    @property
    def zzoo(self) -> FVector4: ...
    @property
    def zzol(self) -> FVector4: ...
    @property
    def zzlx(self) -> FVector4: ...
    @property
    def zzly(self) -> FVector4: ...
    @property
    def zzlz(self) -> FVector4: ...
    @property
    def zzlw(self) -> FVector4: ...
    @property
    def zzlo(self) -> FVector4: ...
    @property
    def zzll(self) -> FVector4: ...
    @property
    def zwxx(self) -> FVector4: ...
    @property
    def zwxy(self) -> FVector4: ...
    @property
    def zwxz(self) -> FVector4: ...
    @property
    def zwxw(self) -> FVector4: ...
    @property
    def zwxo(self) -> FVector4: ...
    @property
    def zwxl(self) -> FVector4: ...
    @property
    def zwyx(self) -> FVector4: ...
    @property
    def zwyy(self) -> FVector4: ...
    @property
    def zwyz(self) -> FVector4: ...
    @property
    def zwyw(self) -> FVector4: ...
    @property
    def zwyo(self) -> FVector4: ...
    @property
    def zwyl(self) -> FVector4: ...
    @property
    def zwzx(self) -> FVector4: ...
    @property
    def zwzy(self) -> FVector4: ...
    @property
    def zwzz(self) -> FVector4: ...
    @property
    def zwzw(self) -> FVector4: ...
    @property
    def zwzo(self) -> FVector4: ...
    @property
    def zwzl(self) -> FVector4: ...
    @property
    def zwwx(self) -> FVector4: ...
    @property
    def zwwy(self) -> FVector4: ...
    @property
    def zwwz(self) -> FVector4: ...
    @property
    def zwww(self) -> FVector4: ...
    @property
    def zwwo(self) -> FVector4: ...
    @property
    def zwwl(self) -> FVector4: ...
    @property
    def zwox(self) -> FVector4: ...
    @property
    def zwoy(self) -> FVector4: ...
    @property
    def zwoz(self) -> FVector4: ...
    @property
    def zwow(self) -> FVector4: ...
    @property
    def zwoo(self) -> FVector4: ...
    @property
    def zwol(self) -> FVector4: ...
    @property
    def zwlx(self) -> FVector4: ...
    @property
    def zwly(self) -> FVector4: ...
    @property
    def zwlz(self) -> FVector4: ...
    @property
    def zwlw(self) -> FVector4: ...
    @property
    def zwlo(self) -> FVector4: ...
    @property
    def zwll(self) -> FVector4: ...
    @property
    def zoxx(self) -> FVector4: ...
    @property
    def zoxy(self) -> FVector4: ...
    @property
    def zoxz(self) -> FVector4: ...
    @property
    def zoxw(self) -> FVector4: ...
    @property
    def zoxo(self) -> FVector4: ...
    @property
    def zoxl(self) -> FVector4: ...
    @property
    def zoyx(self) -> FVector4: ...
    @property
    def zoyy(self) -> FVector4: ...
    @property
    def zoyz(self) -> FVector4: ...
    @property
    def zoyw(self) -> FVector4: ...
    @property
    def zoyo(self) -> FVector4: ...
    @property
    def zoyl(self) -> FVector4: ...
    @property
    def zozx(self) -> FVector4: ...
    @property
    def zozy(self) -> FVector4: ...
    @property
    def zozz(self) -> FVector4: ...
    @property
    def zozw(self) -> FVector4: ...
    @property
    def zozo(self) -> FVector4: ...
    @property
    def zozl(self) -> FVector4: ...
    @property
    def zowx(self) -> FVector4: ...
    @property
    def zowy(self) -> FVector4: ...
    @property
    def zowz(self) -> FVector4: ...
    @property
    def zoww(self) -> FVector4: ...
    @property
    def zowo(self) -> FVector4: ...
    @property
    def zowl(self) -> FVector4: ...
    @property
    def zoox(self) -> FVector4: ...
    @property
    def zooy(self) -> FVector4: ...
    @property
    def zooz(self) -> FVector4: ...
    @property
    def zoow(self) -> FVector4: ...
    @property
    def zooo(self) -> FVector4: ...
    @property
    def zool(self) -> FVector4: ...
    @property
    def zolx(self) -> FVector4: ...
    @property
    def zoly(self) -> FVector4: ...
    @property
    def zolz(self) -> FVector4: ...
    @property
    def zolw(self) -> FVector4: ...
    @property
    def zolo(self) -> FVector4: ...
    @property
    def zoll(self) -> FVector4: ...
    @property
    def zlxx(self) -> FVector4: ...
    @property
    def zlxy(self) -> FVector4: ...
    @property
    def zlxz(self) -> FVector4: ...
    @property
    def zlxw(self) -> FVector4: ...
    @property
    def zlxo(self) -> FVector4: ...
    @property
    def zlxl(self) -> FVector4: ...
    @property
    def zlyx(self) -> FVector4: ...
    @property
    def zlyy(self) -> FVector4: ...
    @property
    def zlyz(self) -> FVector4: ...
    @property
    def zlyw(self) -> FVector4: ...
    @property
    def zlyo(self) -> FVector4: ...
    @property
    def zlyl(self) -> FVector4: ...
    @property
    def zlzx(self) -> FVector4: ...
    @property
    def zlzy(self) -> FVector4: ...
    @property
    def zlzz(self) -> FVector4: ...
    @property
    def zlzw(self) -> FVector4: ...
    @property
    def zlzo(self) -> FVector4: ...
    @property
    def zlzl(self) -> FVector4: ...
    @property
    def zlwx(self) -> FVector4: ...
    @property
    def zlwy(self) -> FVector4: ...
    @property
    def zlwz(self) -> FVector4: ...
    @property
    def zlww(self) -> FVector4: ...
    @property
    def zlwo(self) -> FVector4: ...
    @property
    def zlwl(self) -> FVector4: ...
    @property
    def zlox(self) -> FVector4: ...
    @property
    def zloy(self) -> FVector4: ...
    @property
    def zloz(self) -> FVector4: ...
    @property
    def zlow(self) -> FVector4: ...
    @property
    def zloo(self) -> FVector4: ...
    @property
    def zlol(self) -> FVector4: ...
    @property
    def zllx(self) -> FVector4: ...
    @property
    def zlly(self) -> FVector4: ...
    @property
    def zllz(self) -> FVector4: ...
    @property
    def zllw(self) -> FVector4: ...
    @property
    def zllo(self) -> FVector4: ...
    @property
    def zlll(self) -> FVector4: ...
    @property
    def wxxx(self) -> FVector4: ...
    @property
    def wxxy(self) -> FVector4: ...
    @property
    def wxxz(self) -> FVector4: ...
    @property
    def wxxw(self) -> FVector4: ...
    @property
    def wxxo(self) -> FVector4: ...
    @property
    def wxxl(self) -> FVector4: ...
    @property
    def wxyx(self) -> FVector4: ...
    @property
    def wxyy(self) -> FVector4: ...
    @property
    def wxyz(self) -> FVector4: ...
    @property
    def wxyw(self) -> FVector4: ...
    @property
    def wxyo(self) -> FVector4: ...
    @property
    def wxyl(self) -> FVector4: ...
    @property
    def wxzx(self) -> FVector4: ...
    @property
    def wxzy(self) -> FVector4: ...
    @property
    def wxzz(self) -> FVector4: ...
    @property
    def wxzw(self) -> FVector4: ...
    @property
    def wxzo(self) -> FVector4: ...
    @property
    def wxzl(self) -> FVector4: ...
    @property
    def wxwx(self) -> FVector4: ...
    @property
    def wxwy(self) -> FVector4: ...
    @property
    def wxwz(self) -> FVector4: ...
    @property
    def wxww(self) -> FVector4: ...
    @property
    def wxwo(self) -> FVector4: ...
    @property
    def wxwl(self) -> FVector4: ...
    @property
    def wxox(self) -> FVector4: ...
    @property
    def wxoy(self) -> FVector4: ...
    @property
    def wxoz(self) -> FVector4: ...
    @property
    def wxow(self) -> FVector4: ...
    @property
    def wxoo(self) -> FVector4: ...
    @property
    def wxol(self) -> FVector4: ...
    @property
    def wxlx(self) -> FVector4: ...
    @property
    def wxly(self) -> FVector4: ...
    @property
    def wxlz(self) -> FVector4: ...
    @property
    def wxlw(self) -> FVector4: ...
    @property
    def wxlo(self) -> FVector4: ...
    @property
    def wxll(self) -> FVector4: ...
    @property
    def wyxx(self) -> FVector4: ...
    @property
    def wyxy(self) -> FVector4: ...
    @property
    def wyxz(self) -> FVector4: ...
    @property
    def wyxw(self) -> FVector4: ...
    @property
    def wyxo(self) -> FVector4: ...
    @property
    def wyxl(self) -> FVector4: ...
    @property
    def wyyx(self) -> FVector4: ...
    @property
    def wyyy(self) -> FVector4: ...
    @property
    def wyyz(self) -> FVector4: ...
    @property
    def wyyw(self) -> FVector4: ...
    @property
    def wyyo(self) -> FVector4: ...
    @property
    def wyyl(self) -> FVector4: ...
    @property
    def wyzx(self) -> FVector4: ...
    @property
    def wyzy(self) -> FVector4: ...
    @property
    def wyzz(self) -> FVector4: ...
    @property
    def wyzw(self) -> FVector4: ...
    @property
    def wyzo(self) -> FVector4: ...
    @property
    def wyzl(self) -> FVector4: ...
    @property
    def wywx(self) -> FVector4: ...
    @property
    def wywy(self) -> FVector4: ...
    @property
    def wywz(self) -> FVector4: ...
    @property
    def wyww(self) -> FVector4: ...
    @property
    def wywo(self) -> FVector4: ...
    @property
    def wywl(self) -> FVector4: ...
    @property
    def wyox(self) -> FVector4: ...
    @property
    def wyoy(self) -> FVector4: ...
    @property
    def wyoz(self) -> FVector4: ...
    @property
    def wyow(self) -> FVector4: ...
    @property
    def wyoo(self) -> FVector4: ...
    @property
    def wyol(self) -> FVector4: ...
    @property
    def wylx(self) -> FVector4: ...
    @property
    def wyly(self) -> FVector4: ...
    @property
    def wylz(self) -> FVector4: ...
    @property
    def wylw(self) -> FVector4: ...
    @property
    def wylo(self) -> FVector4: ...
    @property
    def wyll(self) -> FVector4: ...
    @property
    def wzxx(self) -> FVector4: ...
    @property
    def wzxy(self) -> FVector4: ...
    @property
    def wzxz(self) -> FVector4: ...
    @property
    def wzxw(self) -> FVector4: ...
    @property
    def wzxo(self) -> FVector4: ...
    @property
    def wzxl(self) -> FVector4: ...
    @property
    def wzyx(self) -> FVector4: ...
    @property
    def wzyy(self) -> FVector4: ...
    @property
    def wzyz(self) -> FVector4: ...
    @property
    def wzyw(self) -> FVector4: ...
    @property
    def wzyo(self) -> FVector4: ...
    @property
    def wzyl(self) -> FVector4: ...
    @property
    def wzzx(self) -> FVector4: ...
    @property
    def wzzy(self) -> FVector4: ...
    @property
    def wzzz(self) -> FVector4: ...
    @property
    def wzzw(self) -> FVector4: ...
    @property
    def wzzo(self) -> FVector4: ...
    @property
    def wzzl(self) -> FVector4: ...
    @property
    def wzwx(self) -> FVector4: ...
    @property
    def wzwy(self) -> FVector4: ...
    @property
    def wzwz(self) -> FVector4: ...
    @property
    def wzww(self) -> FVector4: ...
    @property
    def wzwo(self) -> FVector4: ...
    @property
    def wzwl(self) -> FVector4: ...
    @property
    def wzox(self) -> FVector4: ...
    @property
    def wzoy(self) -> FVector4: ...
    @property
    def wzoz(self) -> FVector4: ...
    @property
    def wzow(self) -> FVector4: ...
    @property
    def wzoo(self) -> FVector4: ...
    @property
    def wzol(self) -> FVector4: ...
    @property
    def wzlx(self) -> FVector4: ...
    @property
    def wzly(self) -> FVector4: ...
    @property
    def wzlz(self) -> FVector4: ...
    @property
    def wzlw(self) -> FVector4: ...
    @property
    def wzlo(self) -> FVector4: ...
    @property
    def wzll(self) -> FVector4: ...
    @property
    def wwxx(self) -> FVector4: ...
    @property
    def wwxy(self) -> FVector4: ...
    @property
    def wwxz(self) -> FVector4: ...
    @property
    def wwxw(self) -> FVector4: ...
    @property
    def wwxo(self) -> FVector4: ...
    @property
    def wwxl(self) -> FVector4: ...
    @property
    def wwyx(self) -> FVector4: ...
    @property
    def wwyy(self) -> FVector4: ...
    @property
    def wwyz(self) -> FVector4: ...
    @property
    def wwyw(self) -> FVector4: ...
    @property
    def wwyo(self) -> FVector4: ...
    @property
    def wwyl(self) -> FVector4: ...
    @property
    def wwzx(self) -> FVector4: ...
    @property
    def wwzy(self) -> FVector4: ...
    @property
    def wwzz(self) -> FVector4: ...
    @property
    def wwzw(self) -> FVector4: ...
    @property
    def wwzo(self) -> FVector4: ...
    @property
    def wwzl(self) -> FVector4: ...
    @property
    def wwwx(self) -> FVector4: ...
    @property
    def wwwy(self) -> FVector4: ...
    @property
    def wwwz(self) -> FVector4: ...
    @property
    def wwww(self) -> FVector4: ...
    @property
    def wwwo(self) -> FVector4: ...
    @property
    def wwwl(self) -> FVector4: ...
    @property
    def wwox(self) -> FVector4: ...
    @property
    def wwoy(self) -> FVector4: ...
    @property
    def wwoz(self) -> FVector4: ...
    @property
    def wwow(self) -> FVector4: ...
    @property
    def wwoo(self) -> FVector4: ...
    @property
    def wwol(self) -> FVector4: ...
    @property
    def wwlx(self) -> FVector4: ...
    @property
    def wwly(self) -> FVector4: ...
    @property
    def wwlz(self) -> FVector4: ...
    @property
    def wwlw(self) -> FVector4: ...
    @property
    def wwlo(self) -> FVector4: ...
    @property
    def wwll(self) -> FVector4: ...
    @property
    def woxx(self) -> FVector4: ...
    @property
    def woxy(self) -> FVector4: ...
    @property
    def woxz(self) -> FVector4: ...
    @property
    def woxw(self) -> FVector4: ...
    @property
    def woxo(self) -> FVector4: ...
    @property
    def woxl(self) -> FVector4: ...
    @property
    def woyx(self) -> FVector4: ...
    @property
    def woyy(self) -> FVector4: ...
    @property
    def woyz(self) -> FVector4: ...
    @property
    def woyw(self) -> FVector4: ...
    @property
    def woyo(self) -> FVector4: ...
    @property
    def woyl(self) -> FVector4: ...
    @property
    def wozx(self) -> FVector4: ...
    @property
    def wozy(self) -> FVector4: ...
    @property
    def wozz(self) -> FVector4: ...
    @property
    def wozw(self) -> FVector4: ...
    @property
    def wozo(self) -> FVector4: ...
    @property
    def wozl(self) -> FVector4: ...
    @property
    def wowx(self) -> FVector4: ...
    @property
    def wowy(self) -> FVector4: ...
    @property
    def wowz(self) -> FVector4: ...
    @property
    def woww(self) -> FVector4: ...
    @property
    def wowo(self) -> FVector4: ...
    @property
    def wowl(self) -> FVector4: ...
    @property
    def woox(self) -> FVector4: ...
    @property
    def wooy(self) -> FVector4: ...
    @property
    def wooz(self) -> FVector4: ...
    @property
    def woow(self) -> FVector4: ...
    @property
    def wooo(self) -> FVector4: ...
    @property
    def wool(self) -> FVector4: ...
    @property
    def wolx(self) -> FVector4: ...
    @property
    def woly(self) -> FVector4: ...
    @property
    def wolz(self) -> FVector4: ...
    @property
    def wolw(self) -> FVector4: ...
    @property
    def wolo(self) -> FVector4: ...
    @property
    def woll(self) -> FVector4: ...
    @property
    def wlxx(self) -> FVector4: ...
    @property
    def wlxy(self) -> FVector4: ...
    @property
    def wlxz(self) -> FVector4: ...
    @property
    def wlxw(self) -> FVector4: ...
    @property
    def wlxo(self) -> FVector4: ...
    @property
    def wlxl(self) -> FVector4: ...
    @property
    def wlyx(self) -> FVector4: ...
    @property
    def wlyy(self) -> FVector4: ...
    @property
    def wlyz(self) -> FVector4: ...
    @property
    def wlyw(self) -> FVector4: ...
    @property
    def wlyo(self) -> FVector4: ...
    @property
    def wlyl(self) -> FVector4: ...
    @property
    def wlzx(self) -> FVector4: ...
    @property
    def wlzy(self) -> FVector4: ...
    @property
    def wlzz(self) -> FVector4: ...
    @property
    def wlzw(self) -> FVector4: ...
    @property
    def wlzo(self) -> FVector4: ...
    @property
    def wlzl(self) -> FVector4: ...
    @property
    def wlwx(self) -> FVector4: ...
    @property
    def wlwy(self) -> FVector4: ...
    @property
    def wlwz(self) -> FVector4: ...
    @property
    def wlww(self) -> FVector4: ...
    @property
    def wlwo(self) -> FVector4: ...
    @property
    def wlwl(self) -> FVector4: ...
    @property
    def wlox(self) -> FVector4: ...
    @property
    def wloy(self) -> FVector4: ...
    @property
    def wloz(self) -> FVector4: ...
    @property
    def wlow(self) -> FVector4: ...
    @property
    def wloo(self) -> FVector4: ...
    @property
    def wlol(self) -> FVector4: ...
    @property
    def wllx(self) -> FVector4: ...
    @property
    def wlly(self) -> FVector4: ...
    @property
    def wllz(self) -> FVector4: ...
    @property
    def wllw(self) -> FVector4: ...
    @property
    def wllo(self) -> FVector4: ...
    @property
    def wlll(self) -> FVector4: ...
    @property
    def oxxx(self) -> FVector4: ...
    @property
    def oxxy(self) -> FVector4: ...
    @property
    def oxxz(self) -> FVector4: ...
    @property
    def oxxw(self) -> FVector4: ...
    @property
    def oxxo(self) -> FVector4: ...
    @property
    def oxxl(self) -> FVector4: ...
    @property
    def oxyx(self) -> FVector4: ...
    @property
    def oxyy(self) -> FVector4: ...
    @property
    def oxyz(self) -> FVector4: ...
    @property
    def oxyw(self) -> FVector4: ...
    @property
    def oxyo(self) -> FVector4: ...
    @property
    def oxyl(self) -> FVector4: ...
    @property
    def oxzx(self) -> FVector4: ...
    @property
    def oxzy(self) -> FVector4: ...
    @property
    def oxzz(self) -> FVector4: ...
    @property
    def oxzw(self) -> FVector4: ...
    @property
    def oxzo(self) -> FVector4: ...
    @property
    def oxzl(self) -> FVector4: ...
    @property
    def oxwx(self) -> FVector4: ...
    @property
    def oxwy(self) -> FVector4: ...
    @property
    def oxwz(self) -> FVector4: ...
    @property
    def oxww(self) -> FVector4: ...
    @property
    def oxwo(self) -> FVector4: ...
    @property
    def oxwl(self) -> FVector4: ...
    @property
    def oxox(self) -> FVector4: ...
    @property
    def oxoy(self) -> FVector4: ...
    @property
    def oxoz(self) -> FVector4: ...
    @property
    def oxow(self) -> FVector4: ...
    @property
    def oxoo(self) -> FVector4: ...
    @property
    def oxol(self) -> FVector4: ...
    @property
    def oxlx(self) -> FVector4: ...
    @property
    def oxly(self) -> FVector4: ...
    @property
    def oxlz(self) -> FVector4: ...
    @property
    def oxlw(self) -> FVector4: ...
    @property
    def oxlo(self) -> FVector4: ...
    @property
    def oxll(self) -> FVector4: ...
    @property
    def oyxx(self) -> FVector4: ...
    @property
    def oyxy(self) -> FVector4: ...
    @property
    def oyxz(self) -> FVector4: ...
    @property
    def oyxw(self) -> FVector4: ...
    @property
    def oyxo(self) -> FVector4: ...
    @property
    def oyxl(self) -> FVector4: ...
    @property
    def oyyx(self) -> FVector4: ...
    @property
    def oyyy(self) -> FVector4: ...
    @property
    def oyyz(self) -> FVector4: ...
    @property
    def oyyw(self) -> FVector4: ...
    @property
    def oyyo(self) -> FVector4: ...
    @property
    def oyyl(self) -> FVector4: ...
    @property
    def oyzx(self) -> FVector4: ...
    @property
    def oyzy(self) -> FVector4: ...
    @property
    def oyzz(self) -> FVector4: ...
    @property
    def oyzw(self) -> FVector4: ...
    @property
    def oyzo(self) -> FVector4: ...
    @property
    def oyzl(self) -> FVector4: ...
    @property
    def oywx(self) -> FVector4: ...
    @property
    def oywy(self) -> FVector4: ...
    @property
    def oywz(self) -> FVector4: ...
    @property
    def oyww(self) -> FVector4: ...
    @property
    def oywo(self) -> FVector4: ...
    @property
    def oywl(self) -> FVector4: ...
    @property
    def oyox(self) -> FVector4: ...
    @property
    def oyoy(self) -> FVector4: ...
    @property
    def oyoz(self) -> FVector4: ...
    @property
    def oyow(self) -> FVector4: ...
    @property
    def oyoo(self) -> FVector4: ...
    @property
    def oyol(self) -> FVector4: ...
    @property
    def oylx(self) -> FVector4: ...
    @property
    def oyly(self) -> FVector4: ...
    @property
    def oylz(self) -> FVector4: ...
    @property
    def oylw(self) -> FVector4: ...
    @property
    def oylo(self) -> FVector4: ...
    @property
    def oyll(self) -> FVector4: ...
    @property
    def ozxx(self) -> FVector4: ...
    @property
    def ozxy(self) -> FVector4: ...
    @property
    def ozxz(self) -> FVector4: ...
    @property
    def ozxw(self) -> FVector4: ...
    @property
    def ozxo(self) -> FVector4: ...
    @property
    def ozxl(self) -> FVector4: ...
    @property
    def ozyx(self) -> FVector4: ...
    @property
    def ozyy(self) -> FVector4: ...
    @property
    def ozyz(self) -> FVector4: ...
    @property
    def ozyw(self) -> FVector4: ...
    @property
    def ozyo(self) -> FVector4: ...
    @property
    def ozyl(self) -> FVector4: ...
    @property
    def ozzx(self) -> FVector4: ...
    @property
    def ozzy(self) -> FVector4: ...
    @property
    def ozzz(self) -> FVector4: ...
    @property
    def ozzw(self) -> FVector4: ...
    @property
    def ozzo(self) -> FVector4: ...
    @property
    def ozzl(self) -> FVector4: ...
    @property
    def ozwx(self) -> FVector4: ...
    @property
    def ozwy(self) -> FVector4: ...
    @property
    def ozwz(self) -> FVector4: ...
    @property
    def ozww(self) -> FVector4: ...
    @property
    def ozwo(self) -> FVector4: ...
    @property
    def ozwl(self) -> FVector4: ...
    @property
    def ozox(self) -> FVector4: ...
    @property
    def ozoy(self) -> FVector4: ...
    @property
    def ozoz(self) -> FVector4: ...
    @property
    def ozow(self) -> FVector4: ...
    @property
    def ozoo(self) -> FVector4: ...
    @property
    def ozol(self) -> FVector4: ...
    @property
    def ozlx(self) -> FVector4: ...
    @property
    def ozly(self) -> FVector4: ...
    @property
    def ozlz(self) -> FVector4: ...
    @property
    def ozlw(self) -> FVector4: ...
    @property
    def ozlo(self) -> FVector4: ...
    @property
    def ozll(self) -> FVector4: ...
    @property
    def owxx(self) -> FVector4: ...
    @property
    def owxy(self) -> FVector4: ...
    @property
    def owxz(self) -> FVector4: ...
    @property
    def owxw(self) -> FVector4: ...
    @property
    def owxo(self) -> FVector4: ...
    @property
    def owxl(self) -> FVector4: ...
    @property
    def owyx(self) -> FVector4: ...
    @property
    def owyy(self) -> FVector4: ...
    @property
    def owyz(self) -> FVector4: ...
    @property
    def owyw(self) -> FVector4: ...
    @property
    def owyo(self) -> FVector4: ...
    @property
    def owyl(self) -> FVector4: ...
    @property
    def owzx(self) -> FVector4: ...
    @property
    def owzy(self) -> FVector4: ...
    @property
    def owzz(self) -> FVector4: ...
    @property
    def owzw(self) -> FVector4: ...
    @property
    def owzo(self) -> FVector4: ...
    @property
    def owzl(self) -> FVector4: ...
    @property
    def owwx(self) -> FVector4: ...
    @property
    def owwy(self) -> FVector4: ...
    @property
    def owwz(self) -> FVector4: ...
    @property
    def owww(self) -> FVector4: ...
    @property
    def owwo(self) -> FVector4: ...
    @property
    def owwl(self) -> FVector4: ...
    @property
    def owox(self) -> FVector4: ...
    @property
    def owoy(self) -> FVector4: ...
    @property
    def owoz(self) -> FVector4: ...
    @property
    def owow(self) -> FVector4: ...
    @property
    def owoo(self) -> FVector4: ...
    @property
    def owol(self) -> FVector4: ...
    @property
    def owlx(self) -> FVector4: ...
    @property
    def owly(self) -> FVector4: ...
    @property
    def owlz(self) -> FVector4: ...
    @property
    def owlw(self) -> FVector4: ...
    @property
    def owlo(self) -> FVector4: ...
    @property
    def owll(self) -> FVector4: ...
    @property
    def ooxx(self) -> FVector4: ...
    @property
    def ooxy(self) -> FVector4: ...
    @property
    def ooxz(self) -> FVector4: ...
    @property
    def ooxw(self) -> FVector4: ...
    @property
    def ooxo(self) -> FVector4: ...
    @property
    def ooxl(self) -> FVector4: ...
    @property
    def ooyx(self) -> FVector4: ...
    @property
    def ooyy(self) -> FVector4: ...
    @property
    def ooyz(self) -> FVector4: ...
    @property
    def ooyw(self) -> FVector4: ...
    @property
    def ooyo(self) -> FVector4: ...
    @property
    def ooyl(self) -> FVector4: ...
    @property
    def oozx(self) -> FVector4: ...
    @property
    def oozy(self) -> FVector4: ...
    @property
    def oozz(self) -> FVector4: ...
    @property
    def oozw(self) -> FVector4: ...
    @property
    def oozo(self) -> FVector4: ...
    @property
    def oozl(self) -> FVector4: ...
    @property
    def oowx(self) -> FVector4: ...
    @property
    def oowy(self) -> FVector4: ...
    @property
    def oowz(self) -> FVector4: ...
    @property
    def ooww(self) -> FVector4: ...
    @property
    def oowo(self) -> FVector4: ...
    @property
    def oowl(self) -> FVector4: ...
    @property
    def ooox(self) -> FVector4: ...
    @property
    def oooy(self) -> FVector4: ...
    @property
    def oooz(self) -> FVector4: ...
    @property
    def ooow(self) -> FVector4: ...
    @property
    def oolx(self) -> FVector4: ...
    @property
    def ooly(self) -> FVector4: ...
    @property
    def oolz(self) -> FVector4: ...
    @property
    def oolw(self) -> FVector4: ...
    @property
    def olxx(self) -> FVector4: ...
    @property
    def olxy(self) -> FVector4: ...
    @property
    def olxz(self) -> FVector4: ...
    @property
    def olxw(self) -> FVector4: ...
    @property
    def olxo(self) -> FVector4: ...
    @property
    def olxl(self) -> FVector4: ...
    @property
    def olyx(self) -> FVector4: ...
    @property
    def olyy(self) -> FVector4: ...
    @property
    def olyz(self) -> FVector4: ...
    @property
    def olyw(self) -> FVector4: ...
    @property
    def olyo(self) -> FVector4: ...
    @property
    def olyl(self) -> FVector4: ...
    @property
    def olzx(self) -> FVector4: ...
    @property
    def olzy(self) -> FVector4: ...
    @property
    def olzz(self) -> FVector4: ...
    @property
    def olzw(self) -> FVector4: ...
    @property
    def olzo(self) -> FVector4: ...
    @property
    def olzl(self) -> FVector4: ...
    @property
    def olwx(self) -> FVector4: ...
    @property
    def olwy(self) -> FVector4: ...
    @property
    def olwz(self) -> FVector4: ...
    @property
    def olww(self) -> FVector4: ...
    @property
    def olwo(self) -> FVector4: ...
    @property
    def olwl(self) -> FVector4: ...
    @property
    def olox(self) -> FVector4: ...
    @property
    def oloy(self) -> FVector4: ...
    @property
    def oloz(self) -> FVector4: ...
    @property
    def olow(self) -> FVector4: ...
    @property
    def ollx(self) -> FVector4: ...
    @property
    def olly(self) -> FVector4: ...
    @property
    def ollz(self) -> FVector4: ...
    @property
    def ollw(self) -> FVector4: ...
    @property
    def lxxx(self) -> FVector4: ...
    @property
    def lxxy(self) -> FVector4: ...
    @property
    def lxxz(self) -> FVector4: ...
    @property
    def lxxw(self) -> FVector4: ...
    @property
    def lxxo(self) -> FVector4: ...
    @property
    def lxxl(self) -> FVector4: ...
    @property
    def lxyx(self) -> FVector4: ...
    @property
    def lxyy(self) -> FVector4: ...
    @property
    def lxyz(self) -> FVector4: ...
    @property
    def lxyw(self) -> FVector4: ...
    @property
    def lxyo(self) -> FVector4: ...
    @property
    def lxyl(self) -> FVector4: ...
    @property
    def lxzx(self) -> FVector4: ...
    @property
    def lxzy(self) -> FVector4: ...
    @property
    def lxzz(self) -> FVector4: ...
    @property
    def lxzw(self) -> FVector4: ...
    @property
    def lxzo(self) -> FVector4: ...
    @property
    def lxzl(self) -> FVector4: ...
    @property
    def lxwx(self) -> FVector4: ...
    @property
    def lxwy(self) -> FVector4: ...
    @property
    def lxwz(self) -> FVector4: ...
    @property
    def lxww(self) -> FVector4: ...
    @property
    def lxwo(self) -> FVector4: ...
    @property
    def lxwl(self) -> FVector4: ...
    @property
    def lxox(self) -> FVector4: ...
    @property
    def lxoy(self) -> FVector4: ...
    @property
    def lxoz(self) -> FVector4: ...
    @property
    def lxow(self) -> FVector4: ...
    @property
    def lxoo(self) -> FVector4: ...
    @property
    def lxol(self) -> FVector4: ...
    @property
    def lxlx(self) -> FVector4: ...
    @property
    def lxly(self) -> FVector4: ...
    @property
    def lxlz(self) -> FVector4: ...
    @property
    def lxlw(self) -> FVector4: ...
    @property
    def lxlo(self) -> FVector4: ...
    @property
    def lxll(self) -> FVector4: ...
    @property
    def lyxx(self) -> FVector4: ...
    @property
    def lyxy(self) -> FVector4: ...
    @property
    def lyxz(self) -> FVector4: ...
    @property
    def lyxw(self) -> FVector4: ...
    @property
    def lyxo(self) -> FVector4: ...
    @property
    def lyxl(self) -> FVector4: ...
    @property
    def lyyx(self) -> FVector4: ...
    @property
    def lyyy(self) -> FVector4: ...
    @property
    def lyyz(self) -> FVector4: ...
    @property
    def lyyw(self) -> FVector4: ...
    @property
    def lyyo(self) -> FVector4: ...
    @property
    def lyyl(self) -> FVector4: ...
    @property
    def lyzx(self) -> FVector4: ...
    @property
    def lyzy(self) -> FVector4: ...
    @property
    def lyzz(self) -> FVector4: ...
    @property
    def lyzw(self) -> FVector4: ...
    @property
    def lyzo(self) -> FVector4: ...
    @property
    def lyzl(self) -> FVector4: ...
    @property
    def lywx(self) -> FVector4: ...
    @property
    def lywy(self) -> FVector4: ...
    @property
    def lywz(self) -> FVector4: ...
    @property
    def lyww(self) -> FVector4: ...
    @property
    def lywo(self) -> FVector4: ...
    @property
    def lywl(self) -> FVector4: ...
    @property
    def lyox(self) -> FVector4: ...
    @property
    def lyoy(self) -> FVector4: ...
    @property
    def lyoz(self) -> FVector4: ...
    @property
    def lyow(self) -> FVector4: ...
    @property
    def lyoo(self) -> FVector4: ...
    @property
    def lyol(self) -> FVector4: ...
    @property
    def lylx(self) -> FVector4: ...
    @property
    def lyly(self) -> FVector4: ...
    @property
    def lylz(self) -> FVector4: ...
    @property
    def lylw(self) -> FVector4: ...
    @property
    def lylo(self) -> FVector4: ...
    @property
    def lyll(self) -> FVector4: ...
    @property
    def lzxx(self) -> FVector4: ...
    @property
    def lzxy(self) -> FVector4: ...
    @property
    def lzxz(self) -> FVector4: ...
    @property
    def lzxw(self) -> FVector4: ...
    @property
    def lzxo(self) -> FVector4: ...
    @property
    def lzxl(self) -> FVector4: ...
    @property
    def lzyx(self) -> FVector4: ...
    @property
    def lzyy(self) -> FVector4: ...
    @property
    def lzyz(self) -> FVector4: ...
    @property
    def lzyw(self) -> FVector4: ...
    @property
    def lzyo(self) -> FVector4: ...
    @property
    def lzyl(self) -> FVector4: ...
    @property
    def lzzx(self) -> FVector4: ...
    @property
    def lzzy(self) -> FVector4: ...
    @property
    def lzzz(self) -> FVector4: ...
    @property
    def lzzw(self) -> FVector4: ...
    @property
    def lzzo(self) -> FVector4: ...
    @property
    def lzzl(self) -> FVector4: ...
    @property
    def lzwx(self) -> FVector4: ...
    @property
    def lzwy(self) -> FVector4: ...
    @property
    def lzwz(self) -> FVector4: ...
    @property
    def lzww(self) -> FVector4: ...
    @property
    def lzwo(self) -> FVector4: ...
    @property
    def lzwl(self) -> FVector4: ...
    @property
    def lzox(self) -> FVector4: ...
    @property
    def lzoy(self) -> FVector4: ...
    @property
    def lzoz(self) -> FVector4: ...
    @property
    def lzow(self) -> FVector4: ...
    @property
    def lzoo(self) -> FVector4: ...
    @property
    def lzol(self) -> FVector4: ...
    @property
    def lzlx(self) -> FVector4: ...
    @property
    def lzly(self) -> FVector4: ...
    @property
    def lzlz(self) -> FVector4: ...
    @property
    def lzlw(self) -> FVector4: ...
    @property
    def lzlo(self) -> FVector4: ...
    @property
    def lzll(self) -> FVector4: ...
    @property
    def lwxx(self) -> FVector4: ...
    @property
    def lwxy(self) -> FVector4: ...
    @property
    def lwxz(self) -> FVector4: ...
    @property
    def lwxw(self) -> FVector4: ...
    @property
    def lwxo(self) -> FVector4: ...
    @property
    def lwxl(self) -> FVector4: ...
    @property
    def lwyx(self) -> FVector4: ...
    @property
    def lwyy(self) -> FVector4: ...
    @property
    def lwyz(self) -> FVector4: ...
    @property
    def lwyw(self) -> FVector4: ...
    @property
    def lwyo(self) -> FVector4: ...
    @property
    def lwyl(self) -> FVector4: ...
    @property
    def lwzx(self) -> FVector4: ...
    @property
    def lwzy(self) -> FVector4: ...
    @property
    def lwzz(self) -> FVector4: ...
    @property
    def lwzw(self) -> FVector4: ...
    @property
    def lwzo(self) -> FVector4: ...
    @property
    def lwzl(self) -> FVector4: ...
    @property
    def lwwx(self) -> FVector4: ...
    @property
    def lwwy(self) -> FVector4: ...
    @property
    def lwwz(self) -> FVector4: ...
    @property
    def lwww(self) -> FVector4: ...
    @property
    def lwwo(self) -> FVector4: ...
    @property
    def lwwl(self) -> FVector4: ...
    @property
    def lwox(self) -> FVector4: ...
    @property
    def lwoy(self) -> FVector4: ...
    @property
    def lwoz(self) -> FVector4: ...
    @property
    def lwow(self) -> FVector4: ...
    @property
    def lwoo(self) -> FVector4: ...
    @property
    def lwol(self) -> FVector4: ...
    @property
    def lwlx(self) -> FVector4: ...
    @property
    def lwly(self) -> FVector4: ...
    @property
    def lwlz(self) -> FVector4: ...
    @property
    def lwlw(self) -> FVector4: ...
    @property
    def lwlo(self) -> FVector4: ...
    @property
    def lwll(self) -> FVector4: ...
    @property
    def loxx(self) -> FVector4: ...
    @property
    def loxy(self) -> FVector4: ...
    @property
    def loxz(self) -> FVector4: ...
    @property
    def loxw(self) -> FVector4: ...
    @property
    def loxo(self) -> FVector4: ...
    @property
    def loxl(self) -> FVector4: ...
    @property
    def loyx(self) -> FVector4: ...
    @property
    def loyy(self) -> FVector4: ...
    @property
    def loyz(self) -> FVector4: ...
    @property
    def loyw(self) -> FVector4: ...
    @property
    def loyo(self) -> FVector4: ...
    @property
    def loyl(self) -> FVector4: ...
    @property
    def lozx(self) -> FVector4: ...
    @property
    def lozy(self) -> FVector4: ...
    @property
    def lozz(self) -> FVector4: ...
    @property
    def lozw(self) -> FVector4: ...
    @property
    def lozo(self) -> FVector4: ...
    @property
    def lozl(self) -> FVector4: ...
    @property
    def lowx(self) -> FVector4: ...
    @property
    def lowy(self) -> FVector4: ...
    @property
    def lowz(self) -> FVector4: ...
    @property
    def loww(self) -> FVector4: ...
    @property
    def lowo(self) -> FVector4: ...
    @property
    def lowl(self) -> FVector4: ...
    @property
    def loox(self) -> FVector4: ...
    @property
    def looy(self) -> FVector4: ...
    @property
    def looz(self) -> FVector4: ...
    @property
    def loow(self) -> FVector4: ...
    @property
    def lolx(self) -> FVector4: ...
    @property
    def loly(self) -> FVector4: ...
    @property
    def lolz(self) -> FVector4: ...
    @property
    def lolw(self) -> FVector4: ...
    @property
    def llxx(self) -> FVector4: ...
    @property
    def llxy(self) -> FVector4: ...
    @property
    def llxz(self) -> FVector4: ...
    @property
    def llxw(self) -> FVector4: ...
    @property
    def llxo(self) -> FVector4: ...
    @property
    def llxl(self) -> FVector4: ...
    @property
    def llyx(self) -> FVector4: ...
    @property
    def llyy(self) -> FVector4: ...
    @property
    def llyz(self) -> FVector4: ...
    @property
    def llyw(self) -> FVector4: ...
    @property
    def llyo(self) -> FVector4: ...
    @property
    def llyl(self) -> FVector4: ...
    @property
    def llzx(self) -> FVector4: ...
    @property
    def llzy(self) -> FVector4: ...
    @property
    def llzz(self) -> FVector4: ...
    @property
    def llzw(self) -> FVector4: ...
    @property
    def llzo(self) -> FVector4: ...
    @property
    def llzl(self) -> FVector4: ...
    @property
    def llwx(self) -> FVector4: ...
    @property
    def llwy(self) -> FVector4: ...
    @property
    def llwz(self) -> FVector4: ...
    @property
    def llww(self) -> FVector4: ...
    @property
    def llwo(self) -> FVector4: ...
    @property
    def llwl(self) -> FVector4: ...
    @property
    def llox(self) -> FVector4: ...
    @property
    def lloy(self) -> FVector4: ...
    @property
    def lloz(self) -> FVector4: ...
    @property
    def llow(self) -> FVector4: ...
    @property
    def lllx(self) -> FVector4: ...
    @property
    def llly(self) -> FVector4: ...
    @property
    def lllz(self) -> FVector4: ...
    @property
    def lllw(self) -> FVector4: ...
    @property
    def r(self) -> float: ...
    @property
    def g(self) -> float: ...
    @property
    def b(self) -> float: ...
    @property
    def a(self) -> float: ...
    @property
    def rr(self) -> FVector2: ...
    @property
    def rg(self) -> FVector2: ...
    @property
    def rb(self) -> FVector2: ...
    @property
    def ra(self) -> FVector2: ...
    @property
    def ro(self) -> FVector2: ...
    @property
    def rl(self) -> FVector2: ...
    @property
    def gr(self) -> FVector2: ...
    @property
    def gg(self) -> FVector2: ...
    @property
    def gb(self) -> FVector2: ...
    @property
    def ga(self) -> FVector2: ...
    @property
    def go(self) -> FVector2: ...
    @property
    def gl(self) -> FVector2: ...
    @property
    def br(self) -> FVector2: ...
    @property
    def bg(self) -> FVector2: ...
    @property
    def bb(self) -> FVector2: ...
    @property
    def ba(self) -> FVector2: ...
    @property
    def bo(self) -> FVector2: ...
    @property
    def bl(self) -> FVector2: ...
    @property
    def ar(self) -> FVector2: ...
    @property
    def ag(self) -> FVector2: ...
    @property
    def ab(self) -> FVector2: ...
    @property
    def aa(self) -> FVector2: ...
    @property
    def ao(self) -> FVector2: ...
    @property
    def al(self) -> FVector2: ...
    @property
    def og(self) -> FVector2: ...
    @property
    def ob(self) -> FVector2: ...
    @property
    def oa(self) -> FVector2: ...
    @property
    def lr(self) -> FVector2: ...
    @property
    def lg(self) -> FVector2: ...
    @property
    def lb(self) -> FVector2: ...
    @property
    def la(self) -> FVector2: ...
    @property
    def rrr(self) -> FVector3: ...
    @property
    def rrg(self) -> FVector3: ...
    @property
    def rrb(self) -> FVector3: ...
    @property
    def rra(self) -> FVector3: ...
    @property
    def rro(self) -> FVector3: ...
    @property
    def rrl(self) -> FVector3: ...
    @property
    def rgr(self) -> FVector3: ...
    @property
    def rgg(self) -> FVector3: ...
    @property
    def rgb(self) -> FVector3: ...
    @property
    def rga(self) -> FVector3: ...
    @property
    def rgo(self) -> FVector3: ...
    @property
    def rgl(self) -> FVector3: ...
    @property
    def rbr(self) -> FVector3: ...
    @property
    def rbg(self) -> FVector3: ...
    @property
    def rbb(self) -> FVector3: ...
    @property
    def rba(self) -> FVector3: ...
    @property
    def rbo(self) -> FVector3: ...
    @property
    def rbl(self) -> FVector3: ...
    @property
    def rar(self) -> FVector3: ...
    @property
    def rag(self) -> FVector3: ...
    @property
    def rab(self) -> FVector3: ...
    @property
    def raa(self) -> FVector3: ...
    @property
    def rao(self) -> FVector3: ...
    @property
    def ral(self) -> FVector3: ...
    @property
    def ror(self) -> FVector3: ...
    @property
    def rog(self) -> FVector3: ...
    @property
    def rob(self) -> FVector3: ...
    @property
    def roa(self) -> FVector3: ...
    @property
    def roo(self) -> FVector3: ...
    @property
    def rol(self) -> FVector3: ...
    @property
    def rlr(self) -> FVector3: ...
    @property
    def rlg(self) -> FVector3: ...
    @property
    def rlb(self) -> FVector3: ...
    @property
    def rla(self) -> FVector3: ...
    @property
    def rlo(self) -> FVector3: ...
    @property
    def rll(self) -> FVector3: ...
    @property
    def grr(self) -> FVector3: ...
    @property
    def grg(self) -> FVector3: ...
    @property
    def grb(self) -> FVector3: ...
    @property
    def gra(self) -> FVector3: ...
    @property
    def gro(self) -> FVector3: ...
    @property
    def grl(self) -> FVector3: ...
    @property
    def ggr(self) -> FVector3: ...
    @property
    def ggg(self) -> FVector3: ...
    @property
    def ggb(self) -> FVector3: ...
    @property
    def gga(self) -> FVector3: ...
    @property
    def ggo(self) -> FVector3: ...
    @property
    def ggl(self) -> FVector3: ...
    @property
    def gbr(self) -> FVector3: ...
    @property
    def gbg(self) -> FVector3: ...
    @property
    def gbb(self) -> FVector3: ...
    @property
    def gba(self) -> FVector3: ...
    @property
    def gbo(self) -> FVector3: ...
    @property
    def gbl(self) -> FVector3: ...
    @property
    def gar(self) -> FVector3: ...
    @property
    def gag(self) -> FVector3: ...
    @property
    def gab(self) -> FVector3: ...
    @property
    def gaa(self) -> FVector3: ...
    @property
    def gao(self) -> FVector3: ...
    @property
    def gal(self) -> FVector3: ...
    @property
    def gor(self) -> FVector3: ...
    @property
    def gog(self) -> FVector3: ...
    @property
    def gob(self) -> FVector3: ...
    @property
    def goa(self) -> FVector3: ...
    @property
    def goo(self) -> FVector3: ...
    @property
    def gol(self) -> FVector3: ...
    @property
    def glr(self) -> FVector3: ...
    @property
    def glg(self) -> FVector3: ...
    @property
    def glb(self) -> FVector3: ...
    @property
    def gla(self) -> FVector3: ...
    @property
    def glo(self) -> FVector3: ...
    @property
    def gll(self) -> FVector3: ...
    @property
    def brr(self) -> FVector3: ...
    @property
    def brg(self) -> FVector3: ...
    @property
    def brb(self) -> FVector3: ...
    @property
    def bra(self) -> FVector3: ...
    @property
    def bro(self) -> FVector3: ...
    @property
    def brl(self) -> FVector3: ...
    @property
    def bgr(self) -> FVector3: ...
    @property
    def bgg(self) -> FVector3: ...
    @property
    def bgb(self) -> FVector3: ...
    @property
    def bga(self) -> FVector3: ...
    @property
    def bgo(self) -> FVector3: ...
    @property
    def bgl(self) -> FVector3: ...
    @property
    def bbr(self) -> FVector3: ...
    @property
    def bbg(self) -> FVector3: ...
    @property
    def bbb(self) -> FVector3: ...
    @property
    def bba(self) -> FVector3: ...
    @property
    def bbo(self) -> FVector3: ...
    @property
    def bbl(self) -> FVector3: ...
    @property
    def bar(self) -> FVector3: ...
    @property
    def bag(self) -> FVector3: ...
    @property
    def bab(self) -> FVector3: ...
    @property
    def baa(self) -> FVector3: ...
    @property
    def bao(self) -> FVector3: ...
    @property
    def bal(self) -> FVector3: ...
    @property
    def bor(self) -> FVector3: ...
    @property
    def bog(self) -> FVector3: ...
    @property
    def bob(self) -> FVector3: ...
    @property
    def boa(self) -> FVector3: ...
    @property
    def boo(self) -> FVector3: ...
    @property
    def bol(self) -> FVector3: ...
    @property
    def blr(self) -> FVector3: ...
    @property
    def blg(self) -> FVector3: ...
    @property
    def blb(self) -> FVector3: ...
    @property
    def bla(self) -> FVector3: ...
    @property
    def blo(self) -> FVector3: ...
    @property
    def bll(self) -> FVector3: ...
    @property
    def arr(self) -> FVector3: ...
    @property
    def arg(self) -> FVector3: ...
    @property
    def arb(self) -> FVector3: ...
    @property
    def ara(self) -> FVector3: ...
    @property
    def aro(self) -> FVector3: ...
    @property
    def arl(self) -> FVector3: ...
    @property
    def agr(self) -> FVector3: ...
    @property
    def agg(self) -> FVector3: ...
    @property
    def agb(self) -> FVector3: ...
    @property
    def aga(self) -> FVector3: ...
    @property
    def ago(self) -> FVector3: ...
    @property
    def agl(self) -> FVector3: ...
    @property
    def abr(self) -> FVector3: ...
    @property
    def abg(self) -> FVector3: ...
    @property
    def abb(self) -> FVector3: ...
    @property
    def aba(self) -> FVector3: ...
    @property
    def abo(self) -> FVector3: ...
    @property
    def abl(self) -> FVector3: ...
    @property
    def aar(self) -> FVector3: ...
    @property
    def aag(self) -> FVector3: ...
    @property
    def aab(self) -> FVector3: ...
    @property
    def aaa(self) -> FVector3: ...
    @property
    def aao(self) -> FVector3: ...
    @property
    def aal(self) -> FVector3: ...
    @property
    def aor(self) -> FVector3: ...
    @property
    def aog(self) -> FVector3: ...
    @property
    def aob(self) -> FVector3: ...
    @property
    def aoa(self) -> FVector3: ...
    @property
    def aoo(self) -> FVector3: ...
    @property
    def aol(self) -> FVector3: ...
    @property
    def alr(self) -> FVector3: ...
    @property
    def alg(self) -> FVector3: ...
    @property
    def alb(self) -> FVector3: ...
    @property
    def ala(self) -> FVector3: ...
    @property
    def alo(self) -> FVector3: ...
    @property
    def all(self) -> FVector3: ...
    @property
    def orr(self) -> FVector3: ...
    @property
    def org(self) -> FVector3: ...
    @property
    def orb(self) -> FVector3: ...
    @property
    def ora(self) -> FVector3: ...
    @property
    def oro(self) -> FVector3: ...
    @property
    def orl(self) -> FVector3: ...
    @property
    def ogr(self) -> FVector3: ...
    @property
    def ogg(self) -> FVector3: ...
    @property
    def ogb(self) -> FVector3: ...
    @property
    def oga(self) -> FVector3: ...
    @property
    def ogo(self) -> FVector3: ...
    @property
    def ogl(self) -> FVector3: ...
    @property
    def obr(self) -> FVector3: ...
    @property
    def obg(self) -> FVector3: ...
    @property
    def obb(self) -> FVector3: ...
    @property
    def oba(self) -> FVector3: ...
    @property
    def obo(self) -> FVector3: ...
    @property
    def obl(self) -> FVector3: ...
    @property
    def oar(self) -> FVector3: ...
    @property
    def oag(self) -> FVector3: ...
    @property
    def oab(self) -> FVector3: ...
    @property
    def oaa(self) -> FVector3: ...
    @property
    def oao(self) -> FVector3: ...
    @property
    def oal(self) -> FVector3: ...
    @property
    def oor(self) -> FVector3: ...
    @property
    def oog(self) -> FVector3: ...
    @property
    def oob(self) -> FVector3: ...
    @property
    def ooa(self) -> FVector3: ...
    @property
    def olr(self) -> FVector3: ...
    @property
    def olg(self) -> FVector3: ...
    @property
    def olb(self) -> FVector3: ...
    @property
    def ola(self) -> FVector3: ...
    @property
    def lrr(self) -> FVector3: ...
    @property
    def lrg(self) -> FVector3: ...
    @property
    def lrb(self) -> FVector3: ...
    @property
    def lra(self) -> FVector3: ...
    @property
    def lro(self) -> FVector3: ...
    @property
    def lrl(self) -> FVector3: ...
    @property
    def lgr(self) -> FVector3: ...
    @property
    def lgg(self) -> FVector3: ...
    @property
    def lgb(self) -> FVector3: ...
    @property
    def lga(self) -> FVector3: ...
    @property
    def lgo(self) -> FVector3: ...
    @property
    def lgl(self) -> FVector3: ...
    @property
    def lbr(self) -> FVector3: ...
    @property
    def lbg(self) -> FVector3: ...
    @property
    def lbb(self) -> FVector3: ...
    @property
    def lba(self) -> FVector3: ...
    @property
    def lbo(self) -> FVector3: ...
    @property
    def lbl(self) -> FVector3: ...
    @property
    def lar(self) -> FVector3: ...
    @property
    def lag(self) -> FVector3: ...
    @property
    def lab(self) -> FVector3: ...
    @property
    def laa(self) -> FVector3: ...
    @property
    def lao(self) -> FVector3: ...
    @property
    def lal(self) -> FVector3: ...
    @property
    def lor(self) -> FVector3: ...
    @property
    def log(self) -> FVector3: ...
    @property
    def lob(self) -> FVector3: ...
    @property
    def loa(self) -> FVector3: ...
    @property
    def llr(self) -> FVector3: ...
    @property
    def llg(self) -> FVector3: ...
    @property
    def llb(self) -> FVector3: ...
    @property
    def lla(self) -> FVector3: ...
    @property
    def rrrr(self) -> FVector4: ...
    @property
    def rrrg(self) -> FVector4: ...
    @property
    def rrrb(self) -> FVector4: ...
    @property
    def rrra(self) -> FVector4: ...
    @property
    def rrro(self) -> FVector4: ...
    @property
    def rrrl(self) -> FVector4: ...
    @property
    def rrgr(self) -> FVector4: ...
    @property
    def rrgg(self) -> FVector4: ...
    @property
    def rrgb(self) -> FVector4: ...
    @property
    def rrga(self) -> FVector4: ...
    @property
    def rrgo(self) -> FVector4: ...
    @property
    def rrgl(self) -> FVector4: ...
    @property
    def rrbr(self) -> FVector4: ...
    @property
    def rrbg(self) -> FVector4: ...
    @property
    def rrbb(self) -> FVector4: ...
    @property
    def rrba(self) -> FVector4: ...
    @property
    def rrbo(self) -> FVector4: ...
    @property
    def rrbl(self) -> FVector4: ...
    @property
    def rrar(self) -> FVector4: ...
    @property
    def rrag(self) -> FVector4: ...
    @property
    def rrab(self) -> FVector4: ...
    @property
    def rraa(self) -> FVector4: ...
    @property
    def rrao(self) -> FVector4: ...
    @property
    def rral(self) -> FVector4: ...
    @property
    def rror(self) -> FVector4: ...
    @property
    def rrog(self) -> FVector4: ...
    @property
    def rrob(self) -> FVector4: ...
    @property
    def rroa(self) -> FVector4: ...
    @property
    def rroo(self) -> FVector4: ...
    @property
    def rrol(self) -> FVector4: ...
    @property
    def rrlr(self) -> FVector4: ...
    @property
    def rrlg(self) -> FVector4: ...
    @property
    def rrlb(self) -> FVector4: ...
    @property
    def rrla(self) -> FVector4: ...
    @property
    def rrlo(self) -> FVector4: ...
    @property
    def rrll(self) -> FVector4: ...
    @property
    def rgrr(self) -> FVector4: ...
    @property
    def rgrg(self) -> FVector4: ...
    @property
    def rgrb(self) -> FVector4: ...
    @property
    def rgra(self) -> FVector4: ...
    @property
    def rgro(self) -> FVector4: ...
    @property
    def rgrl(self) -> FVector4: ...
    @property
    def rggr(self) -> FVector4: ...
    @property
    def rggg(self) -> FVector4: ...
    @property
    def rggb(self) -> FVector4: ...
    @property
    def rgga(self) -> FVector4: ...
    @property
    def rggo(self) -> FVector4: ...
    @property
    def rggl(self) -> FVector4: ...
    @property
    def rgbr(self) -> FVector4: ...
    @property
    def rgbg(self) -> FVector4: ...
    @property
    def rgbb(self) -> FVector4: ...
    @property
    def rgba(self) -> FVector4: ...
    @property
    def rgbo(self) -> FVector4: ...
    @property
    def rgbl(self) -> FVector4: ...
    @property
    def rgar(self) -> FVector4: ...
    @property
    def rgag(self) -> FVector4: ...
    @property
    def rgab(self) -> FVector4: ...
    @property
    def rgaa(self) -> FVector4: ...
    @property
    def rgao(self) -> FVector4: ...
    @property
    def rgal(self) -> FVector4: ...
    @property
    def rgor(self) -> FVector4: ...
    @property
    def rgog(self) -> FVector4: ...
    @property
    def rgob(self) -> FVector4: ...
    @property
    def rgoa(self) -> FVector4: ...
    @property
    def rgoo(self) -> FVector4: ...
    @property
    def rgol(self) -> FVector4: ...
    @property
    def rglr(self) -> FVector4: ...
    @property
    def rglg(self) -> FVector4: ...
    @property
    def rglb(self) -> FVector4: ...
    @property
    def rgla(self) -> FVector4: ...
    @property
    def rglo(self) -> FVector4: ...
    @property
    def rgll(self) -> FVector4: ...
    @property
    def rbrr(self) -> FVector4: ...
    @property
    def rbrg(self) -> FVector4: ...
    @property
    def rbrb(self) -> FVector4: ...
    @property
    def rbra(self) -> FVector4: ...
    @property
    def rbro(self) -> FVector4: ...
    @property
    def rbrl(self) -> FVector4: ...
    @property
    def rbgr(self) -> FVector4: ...
    @property
    def rbgg(self) -> FVector4: ...
    @property
    def rbgb(self) -> FVector4: ...
    @property
    def rbga(self) -> FVector4: ...
    @property
    def rbgo(self) -> FVector4: ...
    @property
    def rbgl(self) -> FVector4: ...
    @property
    def rbbr(self) -> FVector4: ...
    @property
    def rbbg(self) -> FVector4: ...
    @property
    def rbbb(self) -> FVector4: ...
    @property
    def rbba(self) -> FVector4: ...
    @property
    def rbbo(self) -> FVector4: ...
    @property
    def rbbl(self) -> FVector4: ...
    @property
    def rbar(self) -> FVector4: ...
    @property
    def rbag(self) -> FVector4: ...
    @property
    def rbab(self) -> FVector4: ...
    @property
    def rbaa(self) -> FVector4: ...
    @property
    def rbao(self) -> FVector4: ...
    @property
    def rbal(self) -> FVector4: ...
    @property
    def rbor(self) -> FVector4: ...
    @property
    def rbog(self) -> FVector4: ...
    @property
    def rbob(self) -> FVector4: ...
    @property
    def rboa(self) -> FVector4: ...
    @property
    def rboo(self) -> FVector4: ...
    @property
    def rbol(self) -> FVector4: ...
    @property
    def rblr(self) -> FVector4: ...
    @property
    def rblg(self) -> FVector4: ...
    @property
    def rblb(self) -> FVector4: ...
    @property
    def rbla(self) -> FVector4: ...
    @property
    def rblo(self) -> FVector4: ...
    @property
    def rbll(self) -> FVector4: ...
    @property
    def rarr(self) -> FVector4: ...
    @property
    def rarg(self) -> FVector4: ...
    @property
    def rarb(self) -> FVector4: ...
    @property
    def rara(self) -> FVector4: ...
    @property
    def raro(self) -> FVector4: ...
    @property
    def rarl(self) -> FVector4: ...
    @property
    def ragr(self) -> FVector4: ...
    @property
    def ragg(self) -> FVector4: ...
    @property
    def ragb(self) -> FVector4: ...
    @property
    def raga(self) -> FVector4: ...
    @property
    def rago(self) -> FVector4: ...
    @property
    def ragl(self) -> FVector4: ...
    @property
    def rabr(self) -> FVector4: ...
    @property
    def rabg(self) -> FVector4: ...
    @property
    def rabb(self) -> FVector4: ...
    @property
    def raba(self) -> FVector4: ...
    @property
    def rabo(self) -> FVector4: ...
    @property
    def rabl(self) -> FVector4: ...
    @property
    def raar(self) -> FVector4: ...
    @property
    def raag(self) -> FVector4: ...
    @property
    def raab(self) -> FVector4: ...
    @property
    def raaa(self) -> FVector4: ...
    @property
    def raao(self) -> FVector4: ...
    @property
    def raal(self) -> FVector4: ...
    @property
    def raor(self) -> FVector4: ...
    @property
    def raog(self) -> FVector4: ...
    @property
    def raob(self) -> FVector4: ...
    @property
    def raoa(self) -> FVector4: ...
    @property
    def raoo(self) -> FVector4: ...
    @property
    def raol(self) -> FVector4: ...
    @property
    def ralr(self) -> FVector4: ...
    @property
    def ralg(self) -> FVector4: ...
    @property
    def ralb(self) -> FVector4: ...
    @property
    def rala(self) -> FVector4: ...
    @property
    def ralo(self) -> FVector4: ...
    @property
    def rall(self) -> FVector4: ...
    @property
    def rorr(self) -> FVector4: ...
    @property
    def rorg(self) -> FVector4: ...
    @property
    def rorb(self) -> FVector4: ...
    @property
    def rora(self) -> FVector4: ...
    @property
    def roro(self) -> FVector4: ...
    @property
    def rorl(self) -> FVector4: ...
    @property
    def rogr(self) -> FVector4: ...
    @property
    def rogg(self) -> FVector4: ...
    @property
    def rogb(self) -> FVector4: ...
    @property
    def roga(self) -> FVector4: ...
    @property
    def rogo(self) -> FVector4: ...
    @property
    def rogl(self) -> FVector4: ...
    @property
    def robr(self) -> FVector4: ...
    @property
    def robg(self) -> FVector4: ...
    @property
    def robb(self) -> FVector4: ...
    @property
    def roba(self) -> FVector4: ...
    @property
    def robo(self) -> FVector4: ...
    @property
    def robl(self) -> FVector4: ...
    @property
    def roar(self) -> FVector4: ...
    @property
    def roag(self) -> FVector4: ...
    @property
    def roab(self) -> FVector4: ...
    @property
    def roaa(self) -> FVector4: ...
    @property
    def roao(self) -> FVector4: ...
    @property
    def roal(self) -> FVector4: ...
    @property
    def roor(self) -> FVector4: ...
    @property
    def roog(self) -> FVector4: ...
    @property
    def roob(self) -> FVector4: ...
    @property
    def rooa(self) -> FVector4: ...
    @property
    def rooo(self) -> FVector4: ...
    @property
    def rool(self) -> FVector4: ...
    @property
    def rolr(self) -> FVector4: ...
    @property
    def rolg(self) -> FVector4: ...
    @property
    def rolb(self) -> FVector4: ...
    @property
    def rola(self) -> FVector4: ...
    @property
    def rolo(self) -> FVector4: ...
    @property
    def roll(self) -> FVector4: ...
    @property
    def rlrr(self) -> FVector4: ...
    @property
    def rlrg(self) -> FVector4: ...
    @property
    def rlrb(self) -> FVector4: ...
    @property
    def rlra(self) -> FVector4: ...
    @property
    def rlro(self) -> FVector4: ...
    @property
    def rlrl(self) -> FVector4: ...
    @property
    def rlgr(self) -> FVector4: ...
    @property
    def rlgg(self) -> FVector4: ...
    @property
    def rlgb(self) -> FVector4: ...
    @property
    def rlga(self) -> FVector4: ...
    @property
    def rlgo(self) -> FVector4: ...
    @property
    def rlgl(self) -> FVector4: ...
    @property
    def rlbr(self) -> FVector4: ...
    @property
    def rlbg(self) -> FVector4: ...
    @property
    def rlbb(self) -> FVector4: ...
    @property
    def rlba(self) -> FVector4: ...
    @property
    def rlbo(self) -> FVector4: ...
    @property
    def rlbl(self) -> FVector4: ...
    @property
    def rlar(self) -> FVector4: ...
    @property
    def rlag(self) -> FVector4: ...
    @property
    def rlab(self) -> FVector4: ...
    @property
    def rlaa(self) -> FVector4: ...
    @property
    def rlao(self) -> FVector4: ...
    @property
    def rlal(self) -> FVector4: ...
    @property
    def rlor(self) -> FVector4: ...
    @property
    def rlog(self) -> FVector4: ...
    @property
    def rlob(self) -> FVector4: ...
    @property
    def rloa(self) -> FVector4: ...
    @property
    def rloo(self) -> FVector4: ...
    @property
    def rlol(self) -> FVector4: ...
    @property
    def rllr(self) -> FVector4: ...
    @property
    def rllg(self) -> FVector4: ...
    @property
    def rllb(self) -> FVector4: ...
    @property
    def rlla(self) -> FVector4: ...
    @property
    def rllo(self) -> FVector4: ...
    @property
    def rlll(self) -> FVector4: ...
    @property
    def grrr(self) -> FVector4: ...
    @property
    def grrg(self) -> FVector4: ...
    @property
    def grrb(self) -> FVector4: ...
    @property
    def grra(self) -> FVector4: ...
    @property
    def grro(self) -> FVector4: ...
    @property
    def grrl(self) -> FVector4: ...
    @property
    def grgr(self) -> FVector4: ...
    @property
    def grgg(self) -> FVector4: ...
    @property
    def grgb(self) -> FVector4: ...
    @property
    def grga(self) -> FVector4: ...
    @property
    def grgo(self) -> FVector4: ...
    @property
    def grgl(self) -> FVector4: ...
    @property
    def grbr(self) -> FVector4: ...
    @property
    def grbg(self) -> FVector4: ...
    @property
    def grbb(self) -> FVector4: ...
    @property
    def grba(self) -> FVector4: ...
    @property
    def grbo(self) -> FVector4: ...
    @property
    def grbl(self) -> FVector4: ...
    @property
    def grar(self) -> FVector4: ...
    @property
    def grag(self) -> FVector4: ...
    @property
    def grab(self) -> FVector4: ...
    @property
    def graa(self) -> FVector4: ...
    @property
    def grao(self) -> FVector4: ...
    @property
    def gral(self) -> FVector4: ...
    @property
    def gror(self) -> FVector4: ...
    @property
    def grog(self) -> FVector4: ...
    @property
    def grob(self) -> FVector4: ...
    @property
    def groa(self) -> FVector4: ...
    @property
    def groo(self) -> FVector4: ...
    @property
    def grol(self) -> FVector4: ...
    @property
    def grlr(self) -> FVector4: ...
    @property
    def grlg(self) -> FVector4: ...
    @property
    def grlb(self) -> FVector4: ...
    @property
    def grla(self) -> FVector4: ...
    @property
    def grlo(self) -> FVector4: ...
    @property
    def grll(self) -> FVector4: ...
    @property
    def ggrr(self) -> FVector4: ...
    @property
    def ggrg(self) -> FVector4: ...
    @property
    def ggrb(self) -> FVector4: ...
    @property
    def ggra(self) -> FVector4: ...
    @property
    def ggro(self) -> FVector4: ...
    @property
    def ggrl(self) -> FVector4: ...
    @property
    def gggr(self) -> FVector4: ...
    @property
    def gggg(self) -> FVector4: ...
    @property
    def gggb(self) -> FVector4: ...
    @property
    def ggga(self) -> FVector4: ...
    @property
    def gggo(self) -> FVector4: ...
    @property
    def gggl(self) -> FVector4: ...
    @property
    def ggbr(self) -> FVector4: ...
    @property
    def ggbg(self) -> FVector4: ...
    @property
    def ggbb(self) -> FVector4: ...
    @property
    def ggba(self) -> FVector4: ...
    @property
    def ggbo(self) -> FVector4: ...
    @property
    def ggbl(self) -> FVector4: ...
    @property
    def ggar(self) -> FVector4: ...
    @property
    def ggag(self) -> FVector4: ...
    @property
    def ggab(self) -> FVector4: ...
    @property
    def ggaa(self) -> FVector4: ...
    @property
    def ggao(self) -> FVector4: ...
    @property
    def ggal(self) -> FVector4: ...
    @property
    def ggor(self) -> FVector4: ...
    @property
    def ggog(self) -> FVector4: ...
    @property
    def ggob(self) -> FVector4: ...
    @property
    def ggoa(self) -> FVector4: ...
    @property
    def ggoo(self) -> FVector4: ...
    @property
    def ggol(self) -> FVector4: ...
    @property
    def gglr(self) -> FVector4: ...
    @property
    def gglg(self) -> FVector4: ...
    @property
    def gglb(self) -> FVector4: ...
    @property
    def ggla(self) -> FVector4: ...
    @property
    def gglo(self) -> FVector4: ...
    @property
    def ggll(self) -> FVector4: ...
    @property
    def gbrr(self) -> FVector4: ...
    @property
    def gbrg(self) -> FVector4: ...
    @property
    def gbrb(self) -> FVector4: ...
    @property
    def gbra(self) -> FVector4: ...
    @property
    def gbro(self) -> FVector4: ...
    @property
    def gbrl(self) -> FVector4: ...
    @property
    def gbgr(self) -> FVector4: ...
    @property
    def gbgg(self) -> FVector4: ...
    @property
    def gbgb(self) -> FVector4: ...
    @property
    def gbga(self) -> FVector4: ...
    @property
    def gbgo(self) -> FVector4: ...
    @property
    def gbgl(self) -> FVector4: ...
    @property
    def gbbr(self) -> FVector4: ...
    @property
    def gbbg(self) -> FVector4: ...
    @property
    def gbbb(self) -> FVector4: ...
    @property
    def gbba(self) -> FVector4: ...
    @property
    def gbbo(self) -> FVector4: ...
    @property
    def gbbl(self) -> FVector4: ...
    @property
    def gbar(self) -> FVector4: ...
    @property
    def gbag(self) -> FVector4: ...
    @property
    def gbab(self) -> FVector4: ...
    @property
    def gbaa(self) -> FVector4: ...
    @property
    def gbao(self) -> FVector4: ...
    @property
    def gbal(self) -> FVector4: ...
    @property
    def gbor(self) -> FVector4: ...
    @property
    def gbog(self) -> FVector4: ...
    @property
    def gbob(self) -> FVector4: ...
    @property
    def gboa(self) -> FVector4: ...
    @property
    def gboo(self) -> FVector4: ...
    @property
    def gbol(self) -> FVector4: ...
    @property
    def gblr(self) -> FVector4: ...
    @property
    def gblg(self) -> FVector4: ...
    @property
    def gblb(self) -> FVector4: ...
    @property
    def gbla(self) -> FVector4: ...
    @property
    def gblo(self) -> FVector4: ...
    @property
    def gbll(self) -> FVector4: ...
    @property
    def garr(self) -> FVector4: ...
    @property
    def garg(self) -> FVector4: ...
    @property
    def garb(self) -> FVector4: ...
    @property
    def gara(self) -> FVector4: ...
    @property
    def garo(self) -> FVector4: ...
    @property
    def garl(self) -> FVector4: ...
    @property
    def gagr(self) -> FVector4: ...
    @property
    def gagg(self) -> FVector4: ...
    @property
    def gagb(self) -> FVector4: ...
    @property
    def gaga(self) -> FVector4: ...
    @property
    def gago(self) -> FVector4: ...
    @property
    def gagl(self) -> FVector4: ...
    @property
    def gabr(self) -> FVector4: ...
    @property
    def gabg(self) -> FVector4: ...
    @property
    def gabb(self) -> FVector4: ...
    @property
    def gaba(self) -> FVector4: ...
    @property
    def gabo(self) -> FVector4: ...
    @property
    def gabl(self) -> FVector4: ...
    @property
    def gaar(self) -> FVector4: ...
    @property
    def gaag(self) -> FVector4: ...
    @property
    def gaab(self) -> FVector4: ...
    @property
    def gaaa(self) -> FVector4: ...
    @property
    def gaao(self) -> FVector4: ...
    @property
    def gaal(self) -> FVector4: ...
    @property
    def gaor(self) -> FVector4: ...
    @property
    def gaog(self) -> FVector4: ...
    @property
    def gaob(self) -> FVector4: ...
    @property
    def gaoa(self) -> FVector4: ...
    @property
    def gaoo(self) -> FVector4: ...
    @property
    def gaol(self) -> FVector4: ...
    @property
    def galr(self) -> FVector4: ...
    @property
    def galg(self) -> FVector4: ...
    @property
    def galb(self) -> FVector4: ...
    @property
    def gala(self) -> FVector4: ...
    @property
    def galo(self) -> FVector4: ...
    @property
    def gall(self) -> FVector4: ...
    @property
    def gorr(self) -> FVector4: ...
    @property
    def gorg(self) -> FVector4: ...
    @property
    def gorb(self) -> FVector4: ...
    @property
    def gora(self) -> FVector4: ...
    @property
    def goro(self) -> FVector4: ...
    @property
    def gorl(self) -> FVector4: ...
    @property
    def gogr(self) -> FVector4: ...
    @property
    def gogg(self) -> FVector4: ...
    @property
    def gogb(self) -> FVector4: ...
    @property
    def goga(self) -> FVector4: ...
    @property
    def gogo(self) -> FVector4: ...
    @property
    def gogl(self) -> FVector4: ...
    @property
    def gobr(self) -> FVector4: ...
    @property
    def gobg(self) -> FVector4: ...
    @property
    def gobb(self) -> FVector4: ...
    @property
    def goba(self) -> FVector4: ...
    @property
    def gobo(self) -> FVector4: ...
    @property
    def gobl(self) -> FVector4: ...
    @property
    def goar(self) -> FVector4: ...
    @property
    def goag(self) -> FVector4: ...
    @property
    def goab(self) -> FVector4: ...
    @property
    def goaa(self) -> FVector4: ...
    @property
    def goao(self) -> FVector4: ...
    @property
    def goal(self) -> FVector4: ...
    @property
    def goor(self) -> FVector4: ...
    @property
    def goog(self) -> FVector4: ...
    @property
    def goob(self) -> FVector4: ...
    @property
    def gooa(self) -> FVector4: ...
    @property
    def gooo(self) -> FVector4: ...
    @property
    def gool(self) -> FVector4: ...
    @property
    def golr(self) -> FVector4: ...
    @property
    def golg(self) -> FVector4: ...
    @property
    def golb(self) -> FVector4: ...
    @property
    def gola(self) -> FVector4: ...
    @property
    def golo(self) -> FVector4: ...
    @property
    def goll(self) -> FVector4: ...
    @property
    def glrr(self) -> FVector4: ...
    @property
    def glrg(self) -> FVector4: ...
    @property
    def glrb(self) -> FVector4: ...
    @property
    def glra(self) -> FVector4: ...
    @property
    def glro(self) -> FVector4: ...
    @property
    def glrl(self) -> FVector4: ...
    @property
    def glgr(self) -> FVector4: ...
    @property
    def glgg(self) -> FVector4: ...
    @property
    def glgb(self) -> FVector4: ...
    @property
    def glga(self) -> FVector4: ...
    @property
    def glgo(self) -> FVector4: ...
    @property
    def glgl(self) -> FVector4: ...
    @property
    def glbr(self) -> FVector4: ...
    @property
    def glbg(self) -> FVector4: ...
    @property
    def glbb(self) -> FVector4: ...
    @property
    def glba(self) -> FVector4: ...
    @property
    def glbo(self) -> FVector4: ...
    @property
    def glbl(self) -> FVector4: ...
    @property
    def glar(self) -> FVector4: ...
    @property
    def glag(self) -> FVector4: ...
    @property
    def glab(self) -> FVector4: ...
    @property
    def glaa(self) -> FVector4: ...
    @property
    def glao(self) -> FVector4: ...
    @property
    def glal(self) -> FVector4: ...
    @property
    def glor(self) -> FVector4: ...
    @property
    def glog(self) -> FVector4: ...
    @property
    def glob(self) -> FVector4: ...
    @property
    def gloa(self) -> FVector4: ...
    @property
    def gloo(self) -> FVector4: ...
    @property
    def glol(self) -> FVector4: ...
    @property
    def gllr(self) -> FVector4: ...
    @property
    def gllg(self) -> FVector4: ...
    @property
    def gllb(self) -> FVector4: ...
    @property
    def glla(self) -> FVector4: ...
    @property
    def gllo(self) -> FVector4: ...
    @property
    def glll(self) -> FVector4: ...
    @property
    def brrr(self) -> FVector4: ...
    @property
    def brrg(self) -> FVector4: ...
    @property
    def brrb(self) -> FVector4: ...
    @property
    def brra(self) -> FVector4: ...
    @property
    def brro(self) -> FVector4: ...
    @property
    def brrl(self) -> FVector4: ...
    @property
    def brgr(self) -> FVector4: ...
    @property
    def brgg(self) -> FVector4: ...
    @property
    def brgb(self) -> FVector4: ...
    @property
    def brga(self) -> FVector4: ...
    @property
    def brgo(self) -> FVector4: ...
    @property
    def brgl(self) -> FVector4: ...
    @property
    def brbr(self) -> FVector4: ...
    @property
    def brbg(self) -> FVector4: ...
    @property
    def brbb(self) -> FVector4: ...
    @property
    def brba(self) -> FVector4: ...
    @property
    def brbo(self) -> FVector4: ...
    @property
    def brbl(self) -> FVector4: ...
    @property
    def brar(self) -> FVector4: ...
    @property
    def brag(self) -> FVector4: ...
    @property
    def brab(self) -> FVector4: ...
    @property
    def braa(self) -> FVector4: ...
    @property
    def brao(self) -> FVector4: ...
    @property
    def bral(self) -> FVector4: ...
    @property
    def bror(self) -> FVector4: ...
    @property
    def brog(self) -> FVector4: ...
    @property
    def brob(self) -> FVector4: ...
    @property
    def broa(self) -> FVector4: ...
    @property
    def broo(self) -> FVector4: ...
    @property
    def brol(self) -> FVector4: ...
    @property
    def brlr(self) -> FVector4: ...
    @property
    def brlg(self) -> FVector4: ...
    @property
    def brlb(self) -> FVector4: ...
    @property
    def brla(self) -> FVector4: ...
    @property
    def brlo(self) -> FVector4: ...
    @property
    def brll(self) -> FVector4: ...
    @property
    def bgrr(self) -> FVector4: ...
    @property
    def bgrg(self) -> FVector4: ...
    @property
    def bgrb(self) -> FVector4: ...
    @property
    def bgra(self) -> FVector4: ...
    @property
    def bgro(self) -> FVector4: ...
    @property
    def bgrl(self) -> FVector4: ...
    @property
    def bggr(self) -> FVector4: ...
    @property
    def bggg(self) -> FVector4: ...
    @property
    def bggb(self) -> FVector4: ...
    @property
    def bgga(self) -> FVector4: ...
    @property
    def bggo(self) -> FVector4: ...
    @property
    def bggl(self) -> FVector4: ...
    @property
    def bgbr(self) -> FVector4: ...
    @property
    def bgbg(self) -> FVector4: ...
    @property
    def bgbb(self) -> FVector4: ...
    @property
    def bgba(self) -> FVector4: ...
    @property
    def bgbo(self) -> FVector4: ...
    @property
    def bgbl(self) -> FVector4: ...
    @property
    def bgar(self) -> FVector4: ...
    @property
    def bgag(self) -> FVector4: ...
    @property
    def bgab(self) -> FVector4: ...
    @property
    def bgaa(self) -> FVector4: ...
    @property
    def bgao(self) -> FVector4: ...
    @property
    def bgal(self) -> FVector4: ...
    @property
    def bgor(self) -> FVector4: ...
    @property
    def bgog(self) -> FVector4: ...
    @property
    def bgob(self) -> FVector4: ...
    @property
    def bgoa(self) -> FVector4: ...
    @property
    def bgoo(self) -> FVector4: ...
    @property
    def bgol(self) -> FVector4: ...
    @property
    def bglr(self) -> FVector4: ...
    @property
    def bglg(self) -> FVector4: ...
    @property
    def bglb(self) -> FVector4: ...
    @property
    def bgla(self) -> FVector4: ...
    @property
    def bglo(self) -> FVector4: ...
    @property
    def bgll(self) -> FVector4: ...
    @property
    def bbrr(self) -> FVector4: ...
    @property
    def bbrg(self) -> FVector4: ...
    @property
    def bbrb(self) -> FVector4: ...
    @property
    def bbra(self) -> FVector4: ...
    @property
    def bbro(self) -> FVector4: ...
    @property
    def bbrl(self) -> FVector4: ...
    @property
    def bbgr(self) -> FVector4: ...
    @property
    def bbgg(self) -> FVector4: ...
    @property
    def bbgb(self) -> FVector4: ...
    @property
    def bbga(self) -> FVector4: ...
    @property
    def bbgo(self) -> FVector4: ...
    @property
    def bbgl(self) -> FVector4: ...
    @property
    def bbbr(self) -> FVector4: ...
    @property
    def bbbg(self) -> FVector4: ...
    @property
    def bbbb(self) -> FVector4: ...
    @property
    def bbba(self) -> FVector4: ...
    @property
    def bbbo(self) -> FVector4: ...
    @property
    def bbbl(self) -> FVector4: ...
    @property
    def bbar(self) -> FVector4: ...
    @property
    def bbag(self) -> FVector4: ...
    @property
    def bbab(self) -> FVector4: ...
    @property
    def bbaa(self) -> FVector4: ...
    @property
    def bbao(self) -> FVector4: ...
    @property
    def bbal(self) -> FVector4: ...
    @property
    def bbor(self) -> FVector4: ...
    @property
    def bbog(self) -> FVector4: ...
    @property
    def bbob(self) -> FVector4: ...
    @property
    def bboa(self) -> FVector4: ...
    @property
    def bboo(self) -> FVector4: ...
    @property
    def bbol(self) -> FVector4: ...
    @property
    def bblr(self) -> FVector4: ...
    @property
    def bblg(self) -> FVector4: ...
    @property
    def bblb(self) -> FVector4: ...
    @property
    def bbla(self) -> FVector4: ...
    @property
    def bblo(self) -> FVector4: ...
    @property
    def bbll(self) -> FVector4: ...
    @property
    def barr(self) -> FVector4: ...
    @property
    def barg(self) -> FVector4: ...
    @property
    def barb(self) -> FVector4: ...
    @property
    def bara(self) -> FVector4: ...
    @property
    def baro(self) -> FVector4: ...
    @property
    def barl(self) -> FVector4: ...
    @property
    def bagr(self) -> FVector4: ...
    @property
    def bagg(self) -> FVector4: ...
    @property
    def bagb(self) -> FVector4: ...
    @property
    def baga(self) -> FVector4: ...
    @property
    def bago(self) -> FVector4: ...
    @property
    def bagl(self) -> FVector4: ...
    @property
    def babr(self) -> FVector4: ...
    @property
    def babg(self) -> FVector4: ...
    @property
    def babb(self) -> FVector4: ...
    @property
    def baba(self) -> FVector4: ...
    @property
    def babo(self) -> FVector4: ...
    @property
    def babl(self) -> FVector4: ...
    @property
    def baar(self) -> FVector4: ...
    @property
    def baag(self) -> FVector4: ...
    @property
    def baab(self) -> FVector4: ...
    @property
    def baaa(self) -> FVector4: ...
    @property
    def baao(self) -> FVector4: ...
    @property
    def baal(self) -> FVector4: ...
    @property
    def baor(self) -> FVector4: ...
    @property
    def baog(self) -> FVector4: ...
    @property
    def baob(self) -> FVector4: ...
    @property
    def baoa(self) -> FVector4: ...
    @property
    def baoo(self) -> FVector4: ...
    @property
    def baol(self) -> FVector4: ...
    @property
    def balr(self) -> FVector4: ...
    @property
    def balg(self) -> FVector4: ...
    @property
    def balb(self) -> FVector4: ...
    @property
    def bala(self) -> FVector4: ...
    @property
    def balo(self) -> FVector4: ...
    @property
    def ball(self) -> FVector4: ...
    @property
    def borr(self) -> FVector4: ...
    @property
    def borg(self) -> FVector4: ...
    @property
    def borb(self) -> FVector4: ...
    @property
    def bora(self) -> FVector4: ...
    @property
    def boro(self) -> FVector4: ...
    @property
    def borl(self) -> FVector4: ...
    @property
    def bogr(self) -> FVector4: ...
    @property
    def bogg(self) -> FVector4: ...
    @property
    def bogb(self) -> FVector4: ...
    @property
    def boga(self) -> FVector4: ...
    @property
    def bogo(self) -> FVector4: ...
    @property
    def bogl(self) -> FVector4: ...
    @property
    def bobr(self) -> FVector4: ...
    @property
    def bobg(self) -> FVector4: ...
    @property
    def bobb(self) -> FVector4: ...
    @property
    def boba(self) -> FVector4: ...
    @property
    def bobo(self) -> FVector4: ...
    @property
    def bobl(self) -> FVector4: ...
    @property
    def boar(self) -> FVector4: ...
    @property
    def boag(self) -> FVector4: ...
    @property
    def boab(self) -> FVector4: ...
    @property
    def boaa(self) -> FVector4: ...
    @property
    def boao(self) -> FVector4: ...
    @property
    def boal(self) -> FVector4: ...
    @property
    def boor(self) -> FVector4: ...
    @property
    def boog(self) -> FVector4: ...
    @property
    def boob(self) -> FVector4: ...
    @property
    def booa(self) -> FVector4: ...
    @property
    def booo(self) -> FVector4: ...
    @property
    def bool(self) -> FVector4: ...
    @property
    def bolr(self) -> FVector4: ...
    @property
    def bolg(self) -> FVector4: ...
    @property
    def bolb(self) -> FVector4: ...
    @property
    def bola(self) -> FVector4: ...
    @property
    def bolo(self) -> FVector4: ...
    @property
    def boll(self) -> FVector4: ...
    @property
    def blrr(self) -> FVector4: ...
    @property
    def blrg(self) -> FVector4: ...
    @property
    def blrb(self) -> FVector4: ...
    @property
    def blra(self) -> FVector4: ...
    @property
    def blro(self) -> FVector4: ...
    @property
    def blrl(self) -> FVector4: ...
    @property
    def blgr(self) -> FVector4: ...
    @property
    def blgg(self) -> FVector4: ...
    @property
    def blgb(self) -> FVector4: ...
    @property
    def blga(self) -> FVector4: ...
    @property
    def blgo(self) -> FVector4: ...
    @property
    def blgl(self) -> FVector4: ...
    @property
    def blbr(self) -> FVector4: ...
    @property
    def blbg(self) -> FVector4: ...
    @property
    def blbb(self) -> FVector4: ...
    @property
    def blba(self) -> FVector4: ...
    @property
    def blbo(self) -> FVector4: ...
    @property
    def blbl(self) -> FVector4: ...
    @property
    def blar(self) -> FVector4: ...
    @property
    def blag(self) -> FVector4: ...
    @property
    def blab(self) -> FVector4: ...
    @property
    def blaa(self) -> FVector4: ...
    @property
    def blao(self) -> FVector4: ...
    @property
    def blal(self) -> FVector4: ...
    @property
    def blor(self) -> FVector4: ...
    @property
    def blog(self) -> FVector4: ...
    @property
    def blob(self) -> FVector4: ...
    @property
    def bloa(self) -> FVector4: ...
    @property
    def bloo(self) -> FVector4: ...
    @property
    def blol(self) -> FVector4: ...
    @property
    def bllr(self) -> FVector4: ...
    @property
    def bllg(self) -> FVector4: ...
    @property
    def bllb(self) -> FVector4: ...
    @property
    def blla(self) -> FVector4: ...
    @property
    def bllo(self) -> FVector4: ...
    @property
    def blll(self) -> FVector4: ...
    @property
    def arrr(self) -> FVector4: ...
    @property
    def arrg(self) -> FVector4: ...
    @property
    def arrb(self) -> FVector4: ...
    @property
    def arra(self) -> FVector4: ...
    @property
    def arro(self) -> FVector4: ...
    @property
    def arrl(self) -> FVector4: ...
    @property
    def argr(self) -> FVector4: ...
    @property
    def argg(self) -> FVector4: ...
    @property
    def argb(self) -> FVector4: ...
    @property
    def arga(self) -> FVector4: ...
    @property
    def argo(self) -> FVector4: ...
    @property
    def argl(self) -> FVector4: ...
    @property
    def arbr(self) -> FVector4: ...
    @property
    def arbg(self) -> FVector4: ...
    @property
    def arbb(self) -> FVector4: ...
    @property
    def arba(self) -> FVector4: ...
    @property
    def arbo(self) -> FVector4: ...
    @property
    def arbl(self) -> FVector4: ...
    @property
    def arar(self) -> FVector4: ...
    @property
    def arag(self) -> FVector4: ...
    @property
    def arab(self) -> FVector4: ...
    @property
    def araa(self) -> FVector4: ...
    @property
    def arao(self) -> FVector4: ...
    @property
    def aral(self) -> FVector4: ...
    @property
    def aror(self) -> FVector4: ...
    @property
    def arog(self) -> FVector4: ...
    @property
    def arob(self) -> FVector4: ...
    @property
    def aroa(self) -> FVector4: ...
    @property
    def aroo(self) -> FVector4: ...
    @property
    def arol(self) -> FVector4: ...
    @property
    def arlr(self) -> FVector4: ...
    @property
    def arlg(self) -> FVector4: ...
    @property
    def arlb(self) -> FVector4: ...
    @property
    def arla(self) -> FVector4: ...
    @property
    def arlo(self) -> FVector4: ...
    @property
    def arll(self) -> FVector4: ...
    @property
    def agrr(self) -> FVector4: ...
    @property
    def agrg(self) -> FVector4: ...
    @property
    def agrb(self) -> FVector4: ...
    @property
    def agra(self) -> FVector4: ...
    @property
    def agro(self) -> FVector4: ...
    @property
    def agrl(self) -> FVector4: ...
    @property
    def aggr(self) -> FVector4: ...
    @property
    def aggg(self) -> FVector4: ...
    @property
    def aggb(self) -> FVector4: ...
    @property
    def agga(self) -> FVector4: ...
    @property
    def aggo(self) -> FVector4: ...
    @property
    def aggl(self) -> FVector4: ...
    @property
    def agbr(self) -> FVector4: ...
    @property
    def agbg(self) -> FVector4: ...
    @property
    def agbb(self) -> FVector4: ...
    @property
    def agba(self) -> FVector4: ...
    @property
    def agbo(self) -> FVector4: ...
    @property
    def agbl(self) -> FVector4: ...
    @property
    def agar(self) -> FVector4: ...
    @property
    def agag(self) -> FVector4: ...
    @property
    def agab(self) -> FVector4: ...
    @property
    def agaa(self) -> FVector4: ...
    @property
    def agao(self) -> FVector4: ...
    @property
    def agal(self) -> FVector4: ...
    @property
    def agor(self) -> FVector4: ...
    @property
    def agog(self) -> FVector4: ...
    @property
    def agob(self) -> FVector4: ...
    @property
    def agoa(self) -> FVector4: ...
    @property
    def agoo(self) -> FVector4: ...
    @property
    def agol(self) -> FVector4: ...
    @property
    def aglr(self) -> FVector4: ...
    @property
    def aglg(self) -> FVector4: ...
    @property
    def aglb(self) -> FVector4: ...
    @property
    def agla(self) -> FVector4: ...
    @property
    def aglo(self) -> FVector4: ...
    @property
    def agll(self) -> FVector4: ...
    @property
    def abrr(self) -> FVector4: ...
    @property
    def abrg(self) -> FVector4: ...
    @property
    def abrb(self) -> FVector4: ...
    @property
    def abra(self) -> FVector4: ...
    @property
    def abro(self) -> FVector4: ...
    @property
    def abrl(self) -> FVector4: ...
    @property
    def abgr(self) -> FVector4: ...
    @property
    def abgg(self) -> FVector4: ...
    @property
    def abgb(self) -> FVector4: ...
    @property
    def abga(self) -> FVector4: ...
    @property
    def abgo(self) -> FVector4: ...
    @property
    def abgl(self) -> FVector4: ...
    @property
    def abbr(self) -> FVector4: ...
    @property
    def abbg(self) -> FVector4: ...
    @property
    def abbb(self) -> FVector4: ...
    @property
    def abba(self) -> FVector4: ...
    @property
    def abbo(self) -> FVector4: ...
    @property
    def abbl(self) -> FVector4: ...
    @property
    def abar(self) -> FVector4: ...
    @property
    def abag(self) -> FVector4: ...
    @property
    def abab(self) -> FVector4: ...
    @property
    def abaa(self) -> FVector4: ...
    @property
    def abao(self) -> FVector4: ...
    @property
    def abal(self) -> FVector4: ...
    @property
    def abor(self) -> FVector4: ...
    @property
    def abog(self) -> FVector4: ...
    @property
    def abob(self) -> FVector4: ...
    @property
    def aboa(self) -> FVector4: ...
    @property
    def aboo(self) -> FVector4: ...
    @property
    def abol(self) -> FVector4: ...
    @property
    def ablr(self) -> FVector4: ...
    @property
    def ablg(self) -> FVector4: ...
    @property
    def ablb(self) -> FVector4: ...
    @property
    def abla(self) -> FVector4: ...
    @property
    def ablo(self) -> FVector4: ...
    @property
    def abll(self) -> FVector4: ...
    @property
    def aarr(self) -> FVector4: ...
    @property
    def aarg(self) -> FVector4: ...
    @property
    def aarb(self) -> FVector4: ...
    @property
    def aara(self) -> FVector4: ...
    @property
    def aaro(self) -> FVector4: ...
    @property
    def aarl(self) -> FVector4: ...
    @property
    def aagr(self) -> FVector4: ...
    @property
    def aagg(self) -> FVector4: ...
    @property
    def aagb(self) -> FVector4: ...
    @property
    def aaga(self) -> FVector4: ...
    @property
    def aago(self) -> FVector4: ...
    @property
    def aagl(self) -> FVector4: ...
    @property
    def aabr(self) -> FVector4: ...
    @property
    def aabg(self) -> FVector4: ...
    @property
    def aabb(self) -> FVector4: ...
    @property
    def aaba(self) -> FVector4: ...
    @property
    def aabo(self) -> FVector4: ...
    @property
    def aabl(self) -> FVector4: ...
    @property
    def aaar(self) -> FVector4: ...
    @property
    def aaag(self) -> FVector4: ...
    @property
    def aaab(self) -> FVector4: ...
    @property
    def aaaa(self) -> FVector4: ...
    @property
    def aaao(self) -> FVector4: ...
    @property
    def aaal(self) -> FVector4: ...
    @property
    def aaor(self) -> FVector4: ...
    @property
    def aaog(self) -> FVector4: ...
    @property
    def aaob(self) -> FVector4: ...
    @property
    def aaoa(self) -> FVector4: ...
    @property
    def aaoo(self) -> FVector4: ...
    @property
    def aaol(self) -> FVector4: ...
    @property
    def aalr(self) -> FVector4: ...
    @property
    def aalg(self) -> FVector4: ...
    @property
    def aalb(self) -> FVector4: ...
    @property
    def aala(self) -> FVector4: ...
    @property
    def aalo(self) -> FVector4: ...
    @property
    def aall(self) -> FVector4: ...
    @property
    def aorr(self) -> FVector4: ...
    @property
    def aorg(self) -> FVector4: ...
    @property
    def aorb(self) -> FVector4: ...
    @property
    def aora(self) -> FVector4: ...
    @property
    def aoro(self) -> FVector4: ...
    @property
    def aorl(self) -> FVector4: ...
    @property
    def aogr(self) -> FVector4: ...
    @property
    def aogg(self) -> FVector4: ...
    @property
    def aogb(self) -> FVector4: ...
    @property
    def aoga(self) -> FVector4: ...
    @property
    def aogo(self) -> FVector4: ...
    @property
    def aogl(self) -> FVector4: ...
    @property
    def aobr(self) -> FVector4: ...
    @property
    def aobg(self) -> FVector4: ...
    @property
    def aobb(self) -> FVector4: ...
    @property
    def aoba(self) -> FVector4: ...
    @property
    def aobo(self) -> FVector4: ...
    @property
    def aobl(self) -> FVector4: ...
    @property
    def aoar(self) -> FVector4: ...
    @property
    def aoag(self) -> FVector4: ...
    @property
    def aoab(self) -> FVector4: ...
    @property
    def aoaa(self) -> FVector4: ...
    @property
    def aoao(self) -> FVector4: ...
    @property
    def aoal(self) -> FVector4: ...
    @property
    def aoor(self) -> FVector4: ...
    @property
    def aoog(self) -> FVector4: ...
    @property
    def aoob(self) -> FVector4: ...
    @property
    def aooa(self) -> FVector4: ...
    @property
    def aooo(self) -> FVector4: ...
    @property
    def aool(self) -> FVector4: ...
    @property
    def aolr(self) -> FVector4: ...
    @property
    def aolg(self) -> FVector4: ...
    @property
    def aolb(self) -> FVector4: ...
    @property
    def aola(self) -> FVector4: ...
    @property
    def aolo(self) -> FVector4: ...
    @property
    def aoll(self) -> FVector4: ...
    @property
    def alrr(self) -> FVector4: ...
    @property
    def alrg(self) -> FVector4: ...
    @property
    def alrb(self) -> FVector4: ...
    @property
    def alra(self) -> FVector4: ...
    @property
    def alro(self) -> FVector4: ...
    @property
    def alrl(self) -> FVector4: ...
    @property
    def algr(self) -> FVector4: ...
    @property
    def algg(self) -> FVector4: ...
    @property
    def algb(self) -> FVector4: ...
    @property
    def alga(self) -> FVector4: ...
    @property
    def algo(self) -> FVector4: ...
    @property
    def algl(self) -> FVector4: ...
    @property
    def albr(self) -> FVector4: ...
    @property
    def albg(self) -> FVector4: ...
    @property
    def albb(self) -> FVector4: ...
    @property
    def alba(self) -> FVector4: ...
    @property
    def albo(self) -> FVector4: ...
    @property
    def albl(self) -> FVector4: ...
    @property
    def alar(self) -> FVector4: ...
    @property
    def alag(self) -> FVector4: ...
    @property
    def alab(self) -> FVector4: ...
    @property
    def alaa(self) -> FVector4: ...
    @property
    def alao(self) -> FVector4: ...
    @property
    def alal(self) -> FVector4: ...
    @property
    def alor(self) -> FVector4: ...
    @property
    def alog(self) -> FVector4: ...
    @property
    def alob(self) -> FVector4: ...
    @property
    def aloa(self) -> FVector4: ...
    @property
    def aloo(self) -> FVector4: ...
    @property
    def alol(self) -> FVector4: ...
    @property
    def allr(self) -> FVector4: ...
    @property
    def allg(self) -> FVector4: ...
    @property
    def allb(self) -> FVector4: ...
    @property
    def alla(self) -> FVector4: ...
    @property
    def allo(self) -> FVector4: ...
    @property
    def alll(self) -> FVector4: ...
    @property
    def orrr(self) -> FVector4: ...
    @property
    def orrg(self) -> FVector4: ...
    @property
    def orrb(self) -> FVector4: ...
    @property
    def orra(self) -> FVector4: ...
    @property
    def orro(self) -> FVector4: ...
    @property
    def orrl(self) -> FVector4: ...
    @property
    def orgr(self) -> FVector4: ...
    @property
    def orgg(self) -> FVector4: ...
    @property
    def orgb(self) -> FVector4: ...
    @property
    def orga(self) -> FVector4: ...
    @property
    def orgo(self) -> FVector4: ...
    @property
    def orgl(self) -> FVector4: ...
    @property
    def orbr(self) -> FVector4: ...
    @property
    def orbg(self) -> FVector4: ...
    @property
    def orbb(self) -> FVector4: ...
    @property
    def orba(self) -> FVector4: ...
    @property
    def orbo(self) -> FVector4: ...
    @property
    def orbl(self) -> FVector4: ...
    @property
    def orar(self) -> FVector4: ...
    @property
    def orag(self) -> FVector4: ...
    @property
    def orab(self) -> FVector4: ...
    @property
    def oraa(self) -> FVector4: ...
    @property
    def orao(self) -> FVector4: ...
    @property
    def oral(self) -> FVector4: ...
    @property
    def oror(self) -> FVector4: ...
    @property
    def orog(self) -> FVector4: ...
    @property
    def orob(self) -> FVector4: ...
    @property
    def oroa(self) -> FVector4: ...
    @property
    def oroo(self) -> FVector4: ...
    @property
    def orol(self) -> FVector4: ...
    @property
    def orlr(self) -> FVector4: ...
    @property
    def orlg(self) -> FVector4: ...
    @property
    def orlb(self) -> FVector4: ...
    @property
    def orla(self) -> FVector4: ...
    @property
    def orlo(self) -> FVector4: ...
    @property
    def orll(self) -> FVector4: ...
    @property
    def ogrr(self) -> FVector4: ...
    @property
    def ogrg(self) -> FVector4: ...
    @property
    def ogrb(self) -> FVector4: ...
    @property
    def ogra(self) -> FVector4: ...
    @property
    def ogro(self) -> FVector4: ...
    @property
    def ogrl(self) -> FVector4: ...
    @property
    def oggr(self) -> FVector4: ...
    @property
    def oggg(self) -> FVector4: ...
    @property
    def oggb(self) -> FVector4: ...
    @property
    def ogga(self) -> FVector4: ...
    @property
    def oggo(self) -> FVector4: ...
    @property
    def oggl(self) -> FVector4: ...
    @property
    def ogbr(self) -> FVector4: ...
    @property
    def ogbg(self) -> FVector4: ...
    @property
    def ogbb(self) -> FVector4: ...
    @property
    def ogba(self) -> FVector4: ...
    @property
    def ogbo(self) -> FVector4: ...
    @property
    def ogbl(self) -> FVector4: ...
    @property
    def ogar(self) -> FVector4: ...
    @property
    def ogag(self) -> FVector4: ...
    @property
    def ogab(self) -> FVector4: ...
    @property
    def ogaa(self) -> FVector4: ...
    @property
    def ogao(self) -> FVector4: ...
    @property
    def ogal(self) -> FVector4: ...
    @property
    def ogor(self) -> FVector4: ...
    @property
    def ogog(self) -> FVector4: ...
    @property
    def ogob(self) -> FVector4: ...
    @property
    def ogoa(self) -> FVector4: ...
    @property
    def ogoo(self) -> FVector4: ...
    @property
    def ogol(self) -> FVector4: ...
    @property
    def oglr(self) -> FVector4: ...
    @property
    def oglg(self) -> FVector4: ...
    @property
    def oglb(self) -> FVector4: ...
    @property
    def ogla(self) -> FVector4: ...
    @property
    def oglo(self) -> FVector4: ...
    @property
    def ogll(self) -> FVector4: ...
    @property
    def obrr(self) -> FVector4: ...
    @property
    def obrg(self) -> FVector4: ...
    @property
    def obrb(self) -> FVector4: ...
    @property
    def obra(self) -> FVector4: ...
    @property
    def obro(self) -> FVector4: ...
    @property
    def obrl(self) -> FVector4: ...
    @property
    def obgr(self) -> FVector4: ...
    @property
    def obgg(self) -> FVector4: ...
    @property
    def obgb(self) -> FVector4: ...
    @property
    def obga(self) -> FVector4: ...
    @property
    def obgo(self) -> FVector4: ...
    @property
    def obgl(self) -> FVector4: ...
    @property
    def obbr(self) -> FVector4: ...
    @property
    def obbg(self) -> FVector4: ...
    @property
    def obbb(self) -> FVector4: ...
    @property
    def obba(self) -> FVector4: ...
    @property
    def obbo(self) -> FVector4: ...
    @property
    def obbl(self) -> FVector4: ...
    @property
    def obar(self) -> FVector4: ...
    @property
    def obag(self) -> FVector4: ...
    @property
    def obab(self) -> FVector4: ...
    @property
    def obaa(self) -> FVector4: ...
    @property
    def obao(self) -> FVector4: ...
    @property
    def obal(self) -> FVector4: ...
    @property
    def obor(self) -> FVector4: ...
    @property
    def obog(self) -> FVector4: ...
    @property
    def obob(self) -> FVector4: ...
    @property
    def oboa(self) -> FVector4: ...
    @property
    def oboo(self) -> FVector4: ...
    @property
    def obol(self) -> FVector4: ...
    @property
    def oblr(self) -> FVector4: ...
    @property
    def oblg(self) -> FVector4: ...
    @property
    def oblb(self) -> FVector4: ...
    @property
    def obla(self) -> FVector4: ...
    @property
    def oblo(self) -> FVector4: ...
    @property
    def obll(self) -> FVector4: ...
    @property
    def oarr(self) -> FVector4: ...
    @property
    def oarg(self) -> FVector4: ...
    @property
    def oarb(self) -> FVector4: ...
    @property
    def oara(self) -> FVector4: ...
    @property
    def oaro(self) -> FVector4: ...
    @property
    def oarl(self) -> FVector4: ...
    @property
    def oagr(self) -> FVector4: ...
    @property
    def oagg(self) -> FVector4: ...
    @property
    def oagb(self) -> FVector4: ...
    @property
    def oaga(self) -> FVector4: ...
    @property
    def oago(self) -> FVector4: ...
    @property
    def oagl(self) -> FVector4: ...
    @property
    def oabr(self) -> FVector4: ...
    @property
    def oabg(self) -> FVector4: ...
    @property
    def oabb(self) -> FVector4: ...
    @property
    def oaba(self) -> FVector4: ...
    @property
    def oabo(self) -> FVector4: ...
    @property
    def oabl(self) -> FVector4: ...
    @property
    def oaar(self) -> FVector4: ...
    @property
    def oaag(self) -> FVector4: ...
    @property
    def oaab(self) -> FVector4: ...
    @property
    def oaaa(self) -> FVector4: ...
    @property
    def oaao(self) -> FVector4: ...
    @property
    def oaal(self) -> FVector4: ...
    @property
    def oaor(self) -> FVector4: ...
    @property
    def oaog(self) -> FVector4: ...
    @property
    def oaob(self) -> FVector4: ...
    @property
    def oaoa(self) -> FVector4: ...
    @property
    def oaoo(self) -> FVector4: ...
    @property
    def oaol(self) -> FVector4: ...
    @property
    def oalr(self) -> FVector4: ...
    @property
    def oalg(self) -> FVector4: ...
    @property
    def oalb(self) -> FVector4: ...
    @property
    def oala(self) -> FVector4: ...
    @property
    def oalo(self) -> FVector4: ...
    @property
    def oall(self) -> FVector4: ...
    @property
    def oorr(self) -> FVector4: ...
    @property
    def oorg(self) -> FVector4: ...
    @property
    def oorb(self) -> FVector4: ...
    @property
    def oora(self) -> FVector4: ...
    @property
    def ooro(self) -> FVector4: ...
    @property
    def oorl(self) -> FVector4: ...
    @property
    def oogr(self) -> FVector4: ...
    @property
    def oogg(self) -> FVector4: ...
    @property
    def oogb(self) -> FVector4: ...
    @property
    def ooga(self) -> FVector4: ...
    @property
    def oogo(self) -> FVector4: ...
    @property
    def oogl(self) -> FVector4: ...
    @property
    def oobr(self) -> FVector4: ...
    @property
    def oobg(self) -> FVector4: ...
    @property
    def oobb(self) -> FVector4: ...
    @property
    def ooba(self) -> FVector4: ...
    @property
    def oobo(self) -> FVector4: ...
    @property
    def oobl(self) -> FVector4: ...
    @property
    def ooar(self) -> FVector4: ...
    @property
    def ooag(self) -> FVector4: ...
    @property
    def ooab(self) -> FVector4: ...
    @property
    def ooaa(self) -> FVector4: ...
    @property
    def ooao(self) -> FVector4: ...
    @property
    def ooal(self) -> FVector4: ...
    @property
    def ooor(self) -> FVector4: ...
    @property
    def ooog(self) -> FVector4: ...
    @property
    def ooob(self) -> FVector4: ...
    @property
    def oooa(self) -> FVector4: ...
    @property
    def oolr(self) -> FVector4: ...
    @property
    def oolg(self) -> FVector4: ...
    @property
    def oolb(self) -> FVector4: ...
    @property
    def oola(self) -> FVector4: ...
    @property
    def olrr(self) -> FVector4: ...
    @property
    def olrg(self) -> FVector4: ...
    @property
    def olrb(self) -> FVector4: ...
    @property
    def olra(self) -> FVector4: ...
    @property
    def olro(self) -> FVector4: ...
    @property
    def olrl(self) -> FVector4: ...
    @property
    def olgr(self) -> FVector4: ...
    @property
    def olgg(self) -> FVector4: ...
    @property
    def olgb(self) -> FVector4: ...
    @property
    def olga(self) -> FVector4: ...
    @property
    def olgo(self) -> FVector4: ...
    @property
    def olgl(self) -> FVector4: ...
    @property
    def olbr(self) -> FVector4: ...
    @property
    def olbg(self) -> FVector4: ...
    @property
    def olbb(self) -> FVector4: ...
    @property
    def olba(self) -> FVector4: ...
    @property
    def olbo(self) -> FVector4: ...
    @property
    def olbl(self) -> FVector4: ...
    @property
    def olar(self) -> FVector4: ...
    @property
    def olag(self) -> FVector4: ...
    @property
    def olab(self) -> FVector4: ...
    @property
    def olaa(self) -> FVector4: ...
    @property
    def olao(self) -> FVector4: ...
    @property
    def olal(self) -> FVector4: ...
    @property
    def olor(self) -> FVector4: ...
    @property
    def olog(self) -> FVector4: ...
    @property
    def olob(self) -> FVector4: ...
    @property
    def oloa(self) -> FVector4: ...
    @property
    def ollr(self) -> FVector4: ...
    @property
    def ollg(self) -> FVector4: ...
    @property
    def ollb(self) -> FVector4: ...
    @property
    def olla(self) -> FVector4: ...
    @property
    def lrrr(self) -> FVector4: ...
    @property
    def lrrg(self) -> FVector4: ...
    @property
    def lrrb(self) -> FVector4: ...
    @property
    def lrra(self) -> FVector4: ...
    @property
    def lrro(self) -> FVector4: ...
    @property
    def lrrl(self) -> FVector4: ...
    @property
    def lrgr(self) -> FVector4: ...
    @property
    def lrgg(self) -> FVector4: ...
    @property
    def lrgb(self) -> FVector4: ...
    @property
    def lrga(self) -> FVector4: ...
    @property
    def lrgo(self) -> FVector4: ...
    @property
    def lrgl(self) -> FVector4: ...
    @property
    def lrbr(self) -> FVector4: ...
    @property
    def lrbg(self) -> FVector4: ...
    @property
    def lrbb(self) -> FVector4: ...
    @property
    def lrba(self) -> FVector4: ...
    @property
    def lrbo(self) -> FVector4: ...
    @property
    def lrbl(self) -> FVector4: ...
    @property
    def lrar(self) -> FVector4: ...
    @property
    def lrag(self) -> FVector4: ...
    @property
    def lrab(self) -> FVector4: ...
    @property
    def lraa(self) -> FVector4: ...
    @property
    def lrao(self) -> FVector4: ...
    @property
    def lral(self) -> FVector4: ...
    @property
    def lror(self) -> FVector4: ...
    @property
    def lrog(self) -> FVector4: ...
    @property
    def lrob(self) -> FVector4: ...
    @property
    def lroa(self) -> FVector4: ...
    @property
    def lroo(self) -> FVector4: ...
    @property
    def lrol(self) -> FVector4: ...
    @property
    def lrlr(self) -> FVector4: ...
    @property
    def lrlg(self) -> FVector4: ...
    @property
    def lrlb(self) -> FVector4: ...
    @property
    def lrla(self) -> FVector4: ...
    @property
    def lrlo(self) -> FVector4: ...
    @property
    def lrll(self) -> FVector4: ...
    @property
    def lgrr(self) -> FVector4: ...
    @property
    def lgrg(self) -> FVector4: ...
    @property
    def lgrb(self) -> FVector4: ...
    @property
    def lgra(self) -> FVector4: ...
    @property
    def lgro(self) -> FVector4: ...
    @property
    def lgrl(self) -> FVector4: ...
    @property
    def lggr(self) -> FVector4: ...
    @property
    def lggg(self) -> FVector4: ...
    @property
    def lggb(self) -> FVector4: ...
    @property
    def lgga(self) -> FVector4: ...
    @property
    def lggo(self) -> FVector4: ...
    @property
    def lggl(self) -> FVector4: ...
    @property
    def lgbr(self) -> FVector4: ...
    @property
    def lgbg(self) -> FVector4: ...
    @property
    def lgbb(self) -> FVector4: ...
    @property
    def lgba(self) -> FVector4: ...
    @property
    def lgbo(self) -> FVector4: ...
    @property
    def lgbl(self) -> FVector4: ...
    @property
    def lgar(self) -> FVector4: ...
    @property
    def lgag(self) -> FVector4: ...
    @property
    def lgab(self) -> FVector4: ...
    @property
    def lgaa(self) -> FVector4: ...
    @property
    def lgao(self) -> FVector4: ...
    @property
    def lgal(self) -> FVector4: ...
    @property
    def lgor(self) -> FVector4: ...
    @property
    def lgog(self) -> FVector4: ...
    @property
    def lgob(self) -> FVector4: ...
    @property
    def lgoa(self) -> FVector4: ...
    @property
    def lgoo(self) -> FVector4: ...
    @property
    def lgol(self) -> FVector4: ...
    @property
    def lglr(self) -> FVector4: ...
    @property
    def lglg(self) -> FVector4: ...
    @property
    def lglb(self) -> FVector4: ...
    @property
    def lgla(self) -> FVector4: ...
    @property
    def lglo(self) -> FVector4: ...
    @property
    def lgll(self) -> FVector4: ...
    @property
    def lbrr(self) -> FVector4: ...
    @property
    def lbrg(self) -> FVector4: ...
    @property
    def lbrb(self) -> FVector4: ...
    @property
    def lbra(self) -> FVector4: ...
    @property
    def lbro(self) -> FVector4: ...
    @property
    def lbrl(self) -> FVector4: ...
    @property
    def lbgr(self) -> FVector4: ...
    @property
    def lbgg(self) -> FVector4: ...
    @property
    def lbgb(self) -> FVector4: ...
    @property
    def lbga(self) -> FVector4: ...
    @property
    def lbgo(self) -> FVector4: ...
    @property
    def lbgl(self) -> FVector4: ...
    @property
    def lbbr(self) -> FVector4: ...
    @property
    def lbbg(self) -> FVector4: ...
    @property
    def lbbb(self) -> FVector4: ...
    @property
    def lbba(self) -> FVector4: ...
    @property
    def lbbo(self) -> FVector4: ...
    @property
    def lbbl(self) -> FVector4: ...
    @property
    def lbar(self) -> FVector4: ...
    @property
    def lbag(self) -> FVector4: ...
    @property
    def lbab(self) -> FVector4: ...
    @property
    def lbaa(self) -> FVector4: ...
    @property
    def lbao(self) -> FVector4: ...
    @property
    def lbal(self) -> FVector4: ...
    @property
    def lbor(self) -> FVector4: ...
    @property
    def lbog(self) -> FVector4: ...
    @property
    def lbob(self) -> FVector4: ...
    @property
    def lboa(self) -> FVector4: ...
    @property
    def lboo(self) -> FVector4: ...
    @property
    def lbol(self) -> FVector4: ...
    @property
    def lblr(self) -> FVector4: ...
    @property
    def lblg(self) -> FVector4: ...
    @property
    def lblb(self) -> FVector4: ...
    @property
    def lbla(self) -> FVector4: ...
    @property
    def lblo(self) -> FVector4: ...
    @property
    def lbll(self) -> FVector4: ...
    @property
    def larr(self) -> FVector4: ...
    @property
    def larg(self) -> FVector4: ...
    @property
    def larb(self) -> FVector4: ...
    @property
    def lara(self) -> FVector4: ...
    @property
    def laro(self) -> FVector4: ...
    @property
    def larl(self) -> FVector4: ...
    @property
    def lagr(self) -> FVector4: ...
    @property
    def lagg(self) -> FVector4: ...
    @property
    def lagb(self) -> FVector4: ...
    @property
    def laga(self) -> FVector4: ...
    @property
    def lago(self) -> FVector4: ...
    @property
    def lagl(self) -> FVector4: ...
    @property
    def labr(self) -> FVector4: ...
    @property
    def labg(self) -> FVector4: ...
    @property
    def labb(self) -> FVector4: ...
    @property
    def laba(self) -> FVector4: ...
    @property
    def labo(self) -> FVector4: ...
    @property
    def labl(self) -> FVector4: ...
    @property
    def laar(self) -> FVector4: ...
    @property
    def laag(self) -> FVector4: ...
    @property
    def laab(self) -> FVector4: ...
    @property
    def laaa(self) -> FVector4: ...
    @property
    def laao(self) -> FVector4: ...
    @property
    def laal(self) -> FVector4: ...
    @property
    def laor(self) -> FVector4: ...
    @property
    def laog(self) -> FVector4: ...
    @property
    def laob(self) -> FVector4: ...
    @property
    def laoa(self) -> FVector4: ...
    @property
    def laoo(self) -> FVector4: ...
    @property
    def laol(self) -> FVector4: ...
    @property
    def lalr(self) -> FVector4: ...
    @property
    def lalg(self) -> FVector4: ...
    @property
    def lalb(self) -> FVector4: ...
    @property
    def lala(self) -> FVector4: ...
    @property
    def lalo(self) -> FVector4: ...
    @property
    def lall(self) -> FVector4: ...
    @property
    def lorr(self) -> FVector4: ...
    @property
    def lorg(self) -> FVector4: ...
    @property
    def lorb(self) -> FVector4: ...
    @property
    def lora(self) -> FVector4: ...
    @property
    def loro(self) -> FVector4: ...
    @property
    def lorl(self) -> FVector4: ...
    @property
    def logr(self) -> FVector4: ...
    @property
    def logg(self) -> FVector4: ...
    @property
    def logb(self) -> FVector4: ...
    @property
    def loga(self) -> FVector4: ...
    @property
    def logo(self) -> FVector4: ...
    @property
    def logl(self) -> FVector4: ...
    @property
    def lobr(self) -> FVector4: ...
    @property
    def lobg(self) -> FVector4: ...
    @property
    def lobb(self) -> FVector4: ...
    @property
    def loba(self) -> FVector4: ...
    @property
    def lobo(self) -> FVector4: ...
    @property
    def lobl(self) -> FVector4: ...
    @property
    def loar(self) -> FVector4: ...
    @property
    def loag(self) -> FVector4: ...
    @property
    def loab(self) -> FVector4: ...
    @property
    def loaa(self) -> FVector4: ...
    @property
    def loao(self) -> FVector4: ...
    @property
    def loal(self) -> FVector4: ...
    @property
    def loor(self) -> FVector4: ...
    @property
    def loog(self) -> FVector4: ...
    @property
    def loob(self) -> FVector4: ...
    @property
    def looa(self) -> FVector4: ...
    @property
    def lolr(self) -> FVector4: ...
    @property
    def lolg(self) -> FVector4: ...
    @property
    def lolb(self) -> FVector4: ...
    @property
    def lola(self) -> FVector4: ...
    @property
    def llrr(self) -> FVector4: ...
    @property
    def llrg(self) -> FVector4: ...
    @property
    def llrb(self) -> FVector4: ...
    @property
    def llra(self) -> FVector4: ...
    @property
    def llro(self) -> FVector4: ...
    @property
    def llrl(self) -> FVector4: ...
    @property
    def llgr(self) -> FVector4: ...
    @property
    def llgg(self) -> FVector4: ...
    @property
    def llgb(self) -> FVector4: ...
    @property
    def llga(self) -> FVector4: ...
    @property
    def llgo(self) -> FVector4: ...
    @property
    def llgl(self) -> FVector4: ...
    @property
    def llbr(self) -> FVector4: ...
    @property
    def llbg(self) -> FVector4: ...
    @property
    def llbb(self) -> FVector4: ...
    @property
    def llba(self) -> FVector4: ...
    @property
    def llbo(self) -> FVector4: ...
    @property
    def llbl(self) -> FVector4: ...
    @property
    def llar(self) -> FVector4: ...
    @property
    def llag(self) -> FVector4: ...
    @property
    def llab(self) -> FVector4: ...
    @property
    def llaa(self) -> FVector4: ...
    @property
    def llao(self) -> FVector4: ...
    @property
    def llal(self) -> FVector4: ...
    @property
    def llor(self) -> FVector4: ...
    @property
    def llog(self) -> FVector4: ...
    @property
    def llob(self) -> FVector4: ...
    @property
    def lloa(self) -> FVector4: ...
    @property
    def lllr(self) -> FVector4: ...
    @property
    def lllg(self) -> FVector4: ...
    @property
    def lllb(self) -> FVector4: ...
    @property
    def llla(self) -> FVector4: ...
    @property
    def s(self) -> float: ...
    @property
    def t(self) -> float: ...
    @property
    def q(self) -> float: ...
    @property
    def p(self) -> float: ...
    @property
    def ss(self) -> FVector2: ...
    @property
    def st(self) -> FVector2: ...
    @property
    def sq(self) -> FVector2: ...
    @property
    def sp(self) -> FVector2: ...
    @property
    def so(self) -> FVector2: ...
    @property
    def sl(self) -> FVector2: ...
    @property
    def ts(self) -> FVector2: ...
    @property
    def tt(self) -> FVector2: ...
    @property
    def tq(self) -> FVector2: ...
    @property
    def tp(self) -> FVector2: ...
    @property
    def to(self) -> FVector2: ...
    @property
    def tl(self) -> FVector2: ...
    @property
    def qs(self) -> FVector2: ...
    @property
    def qt(self) -> FVector2: ...
    @property
    def qq(self) -> FVector2: ...
    @property
    def qp(self) -> FVector2: ...
    @property
    def qo(self) -> FVector2: ...
    @property
    def ql(self) -> FVector2: ...
    @property
    def ps(self) -> FVector2: ...
    @property
    def pt(self) -> FVector2: ...
    @property
    def pq(self) -> FVector2: ...
    @property
    def pp(self) -> FVector2: ...
    @property
    def po(self) -> FVector2: ...
    @property
    def pl(self) -> FVector2: ...
    @property
    def os(self) -> FVector2: ...
    @property
    def ot(self) -> FVector2: ...
    @property
    def oq(self) -> FVector2: ...
    @property
    def op(self) -> FVector2: ...
    @property
    def ls(self) -> FVector2: ...
    @property
    def lt(self) -> FVector2: ...
    @property
    def lq(self) -> FVector2: ...
    @property
    def lp(self) -> FVector2: ...
    @property
    def sss(self) -> FVector3: ...
    @property
    def sst(self) -> FVector3: ...
    @property
    def ssq(self) -> FVector3: ...
    @property
    def ssp(self) -> FVector3: ...
    @property
    def sso(self) -> FVector3: ...
    @property
    def ssl(self) -> FVector3: ...
    @property
    def sts(self) -> FVector3: ...
    @property
    def stt(self) -> FVector3: ...
    @property
    def stq(self) -> FVector3: ...
    @property
    def stp(self) -> FVector3: ...
    @property
    def sto(self) -> FVector3: ...
    @property
    def stl(self) -> FVector3: ...
    @property
    def sqs(self) -> FVector3: ...
    @property
    def sqt(self) -> FVector3: ...
    @property
    def sqq(self) -> FVector3: ...
    @property
    def sqp(self) -> FVector3: ...
    @property
    def sqo(self) -> FVector3: ...
    @property
    def sql(self) -> FVector3: ...
    @property
    def sps(self) -> FVector3: ...
    @property
    def spt(self) -> FVector3: ...
    @property
    def spq(self) -> FVector3: ...
    @property
    def spp(self) -> FVector3: ...
    @property
    def spo(self) -> FVector3: ...
    @property
    def spl(self) -> FVector3: ...
    @property
    def sos(self) -> FVector3: ...
    @property
    def sot(self) -> FVector3: ...
    @property
    def soq(self) -> FVector3: ...
    @property
    def sop(self) -> FVector3: ...
    @property
    def soo(self) -> FVector3: ...
    @property
    def sol(self) -> FVector3: ...
    @property
    def sls(self) -> FVector3: ...
    @property
    def slt(self) -> FVector3: ...
    @property
    def slq(self) -> FVector3: ...
    @property
    def slp(self) -> FVector3: ...
    @property
    def slo(self) -> FVector3: ...
    @property
    def sll(self) -> FVector3: ...
    @property
    def tss(self) -> FVector3: ...
    @property
    def tst(self) -> FVector3: ...
    @property
    def tsq(self) -> FVector3: ...
    @property
    def tsp(self) -> FVector3: ...
    @property
    def tso(self) -> FVector3: ...
    @property
    def tsl(self) -> FVector3: ...
    @property
    def tts(self) -> FVector3: ...
    @property
    def ttt(self) -> FVector3: ...
    @property
    def ttq(self) -> FVector3: ...
    @property
    def ttp(self) -> FVector3: ...
    @property
    def tto(self) -> FVector3: ...
    @property
    def ttl(self) -> FVector3: ...
    @property
    def tqs(self) -> FVector3: ...
    @property
    def tqt(self) -> FVector3: ...
    @property
    def tqq(self) -> FVector3: ...
    @property
    def tqp(self) -> FVector3: ...
    @property
    def tqo(self) -> FVector3: ...
    @property
    def tql(self) -> FVector3: ...
    @property
    def tps(self) -> FVector3: ...
    @property
    def tpt(self) -> FVector3: ...
    @property
    def tpq(self) -> FVector3: ...
    @property
    def tpp(self) -> FVector3: ...
    @property
    def tpo(self) -> FVector3: ...
    @property
    def tpl(self) -> FVector3: ...
    @property
    def tos(self) -> FVector3: ...
    @property
    def tot(self) -> FVector3: ...
    @property
    def toq(self) -> FVector3: ...
    @property
    def top(self) -> FVector3: ...
    @property
    def too(self) -> FVector3: ...
    @property
    def tol(self) -> FVector3: ...
    @property
    def tls(self) -> FVector3: ...
    @property
    def tlt(self) -> FVector3: ...
    @property
    def tlq(self) -> FVector3: ...
    @property
    def tlp(self) -> FVector3: ...
    @property
    def tlo(self) -> FVector3: ...
    @property
    def tll(self) -> FVector3: ...
    @property
    def qss(self) -> FVector3: ...
    @property
    def qst(self) -> FVector3: ...
    @property
    def qsq(self) -> FVector3: ...
    @property
    def qsp(self) -> FVector3: ...
    @property
    def qso(self) -> FVector3: ...
    @property
    def qsl(self) -> FVector3: ...
    @property
    def qts(self) -> FVector3: ...
    @property
    def qtt(self) -> FVector3: ...
    @property
    def qtq(self) -> FVector3: ...
    @property
    def qtp(self) -> FVector3: ...
    @property
    def qto(self) -> FVector3: ...
    @property
    def qtl(self) -> FVector3: ...
    @property
    def qqs(self) -> FVector3: ...
    @property
    def qqt(self) -> FVector3: ...
    @property
    def qqq(self) -> FVector3: ...
    @property
    def qqp(self) -> FVector3: ...
    @property
    def qqo(self) -> FVector3: ...
    @property
    def qql(self) -> FVector3: ...
    @property
    def qps(self) -> FVector3: ...
    @property
    def qpt(self) -> FVector3: ...
    @property
    def qpq(self) -> FVector3: ...
    @property
    def qpp(self) -> FVector3: ...
    @property
    def qpo(self) -> FVector3: ...
    @property
    def qpl(self) -> FVector3: ...
    @property
    def qos(self) -> FVector3: ...
    @property
    def qot(self) -> FVector3: ...
    @property
    def qoq(self) -> FVector3: ...
    @property
    def qop(self) -> FVector3: ...
    @property
    def qoo(self) -> FVector3: ...
    @property
    def qol(self) -> FVector3: ...
    @property
    def qls(self) -> FVector3: ...
    @property
    def qlt(self) -> FVector3: ...
    @property
    def qlq(self) -> FVector3: ...
    @property
    def qlp(self) -> FVector3: ...
    @property
    def qlo(self) -> FVector3: ...
    @property
    def qll(self) -> FVector3: ...
    @property
    def pss(self) -> FVector3: ...
    @property
    def pst(self) -> FVector3: ...
    @property
    def psq(self) -> FVector3: ...
    @property
    def psp(self) -> FVector3: ...
    @property
    def pso(self) -> FVector3: ...
    @property
    def psl(self) -> FVector3: ...
    @property
    def pts(self) -> FVector3: ...
    @property
    def ptt(self) -> FVector3: ...
    @property
    def ptq(self) -> FVector3: ...
    @property
    def ptp(self) -> FVector3: ...
    @property
    def pto(self) -> FVector3: ...
    @property
    def ptl(self) -> FVector3: ...
    @property
    def pqs(self) -> FVector3: ...
    @property
    def pqt(self) -> FVector3: ...
    @property
    def pqq(self) -> FVector3: ...
    @property
    def pqp(self) -> FVector3: ...
    @property
    def pqo(self) -> FVector3: ...
    @property
    def pql(self) -> FVector3: ...
    @property
    def pps(self) -> FVector3: ...
    @property
    def ppt(self) -> FVector3: ...
    @property
    def ppq(self) -> FVector3: ...
    @property
    def ppp(self) -> FVector3: ...
    @property
    def ppo(self) -> FVector3: ...
    @property
    def ppl(self) -> FVector3: ...
    @property
    def pos(self) -> FVector3: ...
    @property
    def pot(self) -> FVector3: ...
    @property
    def poq(self) -> FVector3: ...
    @property
    def pop(self) -> FVector3: ...
    @property
    def poo(self) -> FVector3: ...
    @property
    def pol(self) -> FVector3: ...
    @property
    def pls(self) -> FVector3: ...
    @property
    def plt(self) -> FVector3: ...
    @property
    def plq(self) -> FVector3: ...
    @property
    def plp(self) -> FVector3: ...
    @property
    def plo(self) -> FVector3: ...
    @property
    def pll(self) -> FVector3: ...
    @property
    def oss(self) -> FVector3: ...
    @property
    def ost(self) -> FVector3: ...
    @property
    def osq(self) -> FVector3: ...
    @property
    def osp(self) -> FVector3: ...
    @property
    def oso(self) -> FVector3: ...
    @property
    def osl(self) -> FVector3: ...
    @property
    def ots(self) -> FVector3: ...
    @property
    def ott(self) -> FVector3: ...
    @property
    def otq(self) -> FVector3: ...
    @property
    def otp(self) -> FVector3: ...
    @property
    def oto(self) -> FVector3: ...
    @property
    def otl(self) -> FVector3: ...
    @property
    def oqs(self) -> FVector3: ...
    @property
    def oqt(self) -> FVector3: ...
    @property
    def oqq(self) -> FVector3: ...
    @property
    def oqp(self) -> FVector3: ...
    @property
    def oqo(self) -> FVector3: ...
    @property
    def oql(self) -> FVector3: ...
    @property
    def ops(self) -> FVector3: ...
    @property
    def opt(self) -> FVector3: ...
    @property
    def opq(self) -> FVector3: ...
    @property
    def opp(self) -> FVector3: ...
    @property
    def opo(self) -> FVector3: ...
    @property
    def opl(self) -> FVector3: ...
    @property
    def oos(self) -> FVector3: ...
    @property
    def oot(self) -> FVector3: ...
    @property
    def ooq(self) -> FVector3: ...
    @property
    def oop(self) -> FVector3: ...
    @property
    def ols(self) -> FVector3: ...
    @property
    def olt(self) -> FVector3: ...
    @property
    def olq(self) -> FVector3: ...
    @property
    def olp(self) -> FVector3: ...
    @property
    def lss(self) -> FVector3: ...
    @property
    def lst(self) -> FVector3: ...
    @property
    def lsq(self) -> FVector3: ...
    @property
    def lsp(self) -> FVector3: ...
    @property
    def lso(self) -> FVector3: ...
    @property
    def lsl(self) -> FVector3: ...
    @property
    def lts(self) -> FVector3: ...
    @property
    def ltt(self) -> FVector3: ...
    @property
    def ltq(self) -> FVector3: ...
    @property
    def ltp(self) -> FVector3: ...
    @property
    def lto(self) -> FVector3: ...
    @property
    def ltl(self) -> FVector3: ...
    @property
    def lqs(self) -> FVector3: ...
    @property
    def lqt(self) -> FVector3: ...
    @property
    def lqq(self) -> FVector3: ...
    @property
    def lqp(self) -> FVector3: ...
    @property
    def lqo(self) -> FVector3: ...
    @property
    def lql(self) -> FVector3: ...
    @property
    def lps(self) -> FVector3: ...
    @property
    def lpt(self) -> FVector3: ...
    @property
    def lpq(self) -> FVector3: ...
    @property
    def lpp(self) -> FVector3: ...
    @property
    def lpo(self) -> FVector3: ...
    @property
    def lpl(self) -> FVector3: ...
    @property
    def los(self) -> FVector3: ...
    @property
    def lot(self) -> FVector3: ...
    @property
    def loq(self) -> FVector3: ...
    @property
    def lop(self) -> FVector3: ...
    @property
    def lls(self) -> FVector3: ...
    @property
    def llt(self) -> FVector3: ...
    @property
    def llq(self) -> FVector3: ...
    @property
    def llp(self) -> FVector3: ...
    @property
    def ssss(self) -> FVector4: ...
    @property
    def ssst(self) -> FVector4: ...
    @property
    def sssq(self) -> FVector4: ...
    @property
    def sssp(self) -> FVector4: ...
    @property
    def ssso(self) -> FVector4: ...
    @property
    def sssl(self) -> FVector4: ...
    @property
    def ssts(self) -> FVector4: ...
    @property
    def sstt(self) -> FVector4: ...
    @property
    def sstq(self) -> FVector4: ...
    @property
    def sstp(self) -> FVector4: ...
    @property
    def ssto(self) -> FVector4: ...
    @property
    def sstl(self) -> FVector4: ...
    @property
    def ssqs(self) -> FVector4: ...
    @property
    def ssqt(self) -> FVector4: ...
    @property
    def ssqq(self) -> FVector4: ...
    @property
    def ssqp(self) -> FVector4: ...
    @property
    def ssqo(self) -> FVector4: ...
    @property
    def ssql(self) -> FVector4: ...
    @property
    def ssps(self) -> FVector4: ...
    @property
    def sspt(self) -> FVector4: ...
    @property
    def sspq(self) -> FVector4: ...
    @property
    def sspp(self) -> FVector4: ...
    @property
    def sspo(self) -> FVector4: ...
    @property
    def sspl(self) -> FVector4: ...
    @property
    def ssos(self) -> FVector4: ...
    @property
    def ssot(self) -> FVector4: ...
    @property
    def ssoq(self) -> FVector4: ...
    @property
    def ssop(self) -> FVector4: ...
    @property
    def ssoo(self) -> FVector4: ...
    @property
    def ssol(self) -> FVector4: ...
    @property
    def ssls(self) -> FVector4: ...
    @property
    def sslt(self) -> FVector4: ...
    @property
    def sslq(self) -> FVector4: ...
    @property
    def sslp(self) -> FVector4: ...
    @property
    def sslo(self) -> FVector4: ...
    @property
    def ssll(self) -> FVector4: ...
    @property
    def stss(self) -> FVector4: ...
    @property
    def stst(self) -> FVector4: ...
    @property
    def stsq(self) -> FVector4: ...
    @property
    def stsp(self) -> FVector4: ...
    @property
    def stso(self) -> FVector4: ...
    @property
    def stsl(self) -> FVector4: ...
    @property
    def stts(self) -> FVector4: ...
    @property
    def sttt(self) -> FVector4: ...
    @property
    def sttq(self) -> FVector4: ...
    @property
    def sttp(self) -> FVector4: ...
    @property
    def stto(self) -> FVector4: ...
    @property
    def sttl(self) -> FVector4: ...
    @property
    def stqs(self) -> FVector4: ...
    @property
    def stqt(self) -> FVector4: ...
    @property
    def stqq(self) -> FVector4: ...
    @property
    def stqp(self) -> FVector4: ...
    @property
    def stqo(self) -> FVector4: ...
    @property
    def stql(self) -> FVector4: ...
    @property
    def stps(self) -> FVector4: ...
    @property
    def stpt(self) -> FVector4: ...
    @property
    def stpq(self) -> FVector4: ...
    @property
    def stpp(self) -> FVector4: ...
    @property
    def stpo(self) -> FVector4: ...
    @property
    def stpl(self) -> FVector4: ...
    @property
    def stos(self) -> FVector4: ...
    @property
    def stot(self) -> FVector4: ...
    @property
    def stoq(self) -> FVector4: ...
    @property
    def stop(self) -> FVector4: ...
    @property
    def stoo(self) -> FVector4: ...
    @property
    def stol(self) -> FVector4: ...
    @property
    def stls(self) -> FVector4: ...
    @property
    def stlt(self) -> FVector4: ...
    @property
    def stlq(self) -> FVector4: ...
    @property
    def stlp(self) -> FVector4: ...
    @property
    def stlo(self) -> FVector4: ...
    @property
    def stll(self) -> FVector4: ...
    @property
    def sqss(self) -> FVector4: ...
    @property
    def sqst(self) -> FVector4: ...
    @property
    def sqsq(self) -> FVector4: ...
    @property
    def sqsp(self) -> FVector4: ...
    @property
    def sqso(self) -> FVector4: ...
    @property
    def sqsl(self) -> FVector4: ...
    @property
    def sqts(self) -> FVector4: ...
    @property
    def sqtt(self) -> FVector4: ...
    @property
    def sqtq(self) -> FVector4: ...
    @property
    def sqtp(self) -> FVector4: ...
    @property
    def sqto(self) -> FVector4: ...
    @property
    def sqtl(self) -> FVector4: ...
    @property
    def sqqs(self) -> FVector4: ...
    @property
    def sqqt(self) -> FVector4: ...
    @property
    def sqqq(self) -> FVector4: ...
    @property
    def sqqp(self) -> FVector4: ...
    @property
    def sqqo(self) -> FVector4: ...
    @property
    def sqql(self) -> FVector4: ...
    @property
    def sqps(self) -> FVector4: ...
    @property
    def sqpt(self) -> FVector4: ...
    @property
    def sqpq(self) -> FVector4: ...
    @property
    def sqpp(self) -> FVector4: ...
    @property
    def sqpo(self) -> FVector4: ...
    @property
    def sqpl(self) -> FVector4: ...
    @property
    def sqos(self) -> FVector4: ...
    @property
    def sqot(self) -> FVector4: ...
    @property
    def sqoq(self) -> FVector4: ...
    @property
    def sqop(self) -> FVector4: ...
    @property
    def sqoo(self) -> FVector4: ...
    @property
    def sqol(self) -> FVector4: ...
    @property
    def sqls(self) -> FVector4: ...
    @property
    def sqlt(self) -> FVector4: ...
    @property
    def sqlq(self) -> FVector4: ...
    @property
    def sqlp(self) -> FVector4: ...
    @property
    def sqlo(self) -> FVector4: ...
    @property
    def sqll(self) -> FVector4: ...
    @property
    def spss(self) -> FVector4: ...
    @property
    def spst(self) -> FVector4: ...
    @property
    def spsq(self) -> FVector4: ...
    @property
    def spsp(self) -> FVector4: ...
    @property
    def spso(self) -> FVector4: ...
    @property
    def spsl(self) -> FVector4: ...
    @property
    def spts(self) -> FVector4: ...
    @property
    def sptt(self) -> FVector4: ...
    @property
    def sptq(self) -> FVector4: ...
    @property
    def sptp(self) -> FVector4: ...
    @property
    def spto(self) -> FVector4: ...
    @property
    def sptl(self) -> FVector4: ...
    @property
    def spqs(self) -> FVector4: ...
    @property
    def spqt(self) -> FVector4: ...
    @property
    def spqq(self) -> FVector4: ...
    @property
    def spqp(self) -> FVector4: ...
    @property
    def spqo(self) -> FVector4: ...
    @property
    def spql(self) -> FVector4: ...
    @property
    def spps(self) -> FVector4: ...
    @property
    def sppt(self) -> FVector4: ...
    @property
    def sppq(self) -> FVector4: ...
    @property
    def sppp(self) -> FVector4: ...
    @property
    def sppo(self) -> FVector4: ...
    @property
    def sppl(self) -> FVector4: ...
    @property
    def spos(self) -> FVector4: ...
    @property
    def spot(self) -> FVector4: ...
    @property
    def spoq(self) -> FVector4: ...
    @property
    def spop(self) -> FVector4: ...
    @property
    def spoo(self) -> FVector4: ...
    @property
    def spol(self) -> FVector4: ...
    @property
    def spls(self) -> FVector4: ...
    @property
    def splt(self) -> FVector4: ...
    @property
    def splq(self) -> FVector4: ...
    @property
    def splp(self) -> FVector4: ...
    @property
    def splo(self) -> FVector4: ...
    @property
    def spll(self) -> FVector4: ...
    @property
    def soss(self) -> FVector4: ...
    @property
    def sost(self) -> FVector4: ...
    @property
    def sosq(self) -> FVector4: ...
    @property
    def sosp(self) -> FVector4: ...
    @property
    def soso(self) -> FVector4: ...
    @property
    def sosl(self) -> FVector4: ...
    @property
    def sots(self) -> FVector4: ...
    @property
    def sott(self) -> FVector4: ...
    @property
    def sotq(self) -> FVector4: ...
    @property
    def sotp(self) -> FVector4: ...
    @property
    def soto(self) -> FVector4: ...
    @property
    def sotl(self) -> FVector4: ...
    @property
    def soqs(self) -> FVector4: ...
    @property
    def soqt(self) -> FVector4: ...
    @property
    def soqq(self) -> FVector4: ...
    @property
    def soqp(self) -> FVector4: ...
    @property
    def soqo(self) -> FVector4: ...
    @property
    def soql(self) -> FVector4: ...
    @property
    def sops(self) -> FVector4: ...
    @property
    def sopt(self) -> FVector4: ...
    @property
    def sopq(self) -> FVector4: ...
    @property
    def sopp(self) -> FVector4: ...
    @property
    def sopo(self) -> FVector4: ...
    @property
    def sopl(self) -> FVector4: ...
    @property
    def soos(self) -> FVector4: ...
    @property
    def soot(self) -> FVector4: ...
    @property
    def sooq(self) -> FVector4: ...
    @property
    def soop(self) -> FVector4: ...
    @property
    def sooo(self) -> FVector4: ...
    @property
    def sool(self) -> FVector4: ...
    @property
    def sols(self) -> FVector4: ...
    @property
    def solt(self) -> FVector4: ...
    @property
    def solq(self) -> FVector4: ...
    @property
    def solp(self) -> FVector4: ...
    @property
    def solo(self) -> FVector4: ...
    @property
    def soll(self) -> FVector4: ...
    @property
    def slss(self) -> FVector4: ...
    @property
    def slst(self) -> FVector4: ...
    @property
    def slsq(self) -> FVector4: ...
    @property
    def slsp(self) -> FVector4: ...
    @property
    def slso(self) -> FVector4: ...
    @property
    def slsl(self) -> FVector4: ...
    @property
    def slts(self) -> FVector4: ...
    @property
    def sltt(self) -> FVector4: ...
    @property
    def sltq(self) -> FVector4: ...
    @property
    def sltp(self) -> FVector4: ...
    @property
    def slto(self) -> FVector4: ...
    @property
    def sltl(self) -> FVector4: ...
    @property
    def slqs(self) -> FVector4: ...
    @property
    def slqt(self) -> FVector4: ...
    @property
    def slqq(self) -> FVector4: ...
    @property
    def slqp(self) -> FVector4: ...
    @property
    def slqo(self) -> FVector4: ...
    @property
    def slql(self) -> FVector4: ...
    @property
    def slps(self) -> FVector4: ...
    @property
    def slpt(self) -> FVector4: ...
    @property
    def slpq(self) -> FVector4: ...
    @property
    def slpp(self) -> FVector4: ...
    @property
    def slpo(self) -> FVector4: ...
    @property
    def slpl(self) -> FVector4: ...
    @property
    def slos(self) -> FVector4: ...
    @property
    def slot(self) -> FVector4: ...
    @property
    def sloq(self) -> FVector4: ...
    @property
    def slop(self) -> FVector4: ...
    @property
    def sloo(self) -> FVector4: ...
    @property
    def slol(self) -> FVector4: ...
    @property
    def slls(self) -> FVector4: ...
    @property
    def sllt(self) -> FVector4: ...
    @property
    def sllq(self) -> FVector4: ...
    @property
    def sllp(self) -> FVector4: ...
    @property
    def sllo(self) -> FVector4: ...
    @property
    def slll(self) -> FVector4: ...
    @property
    def tsss(self) -> FVector4: ...
    @property
    def tsst(self) -> FVector4: ...
    @property
    def tssq(self) -> FVector4: ...
    @property
    def tssp(self) -> FVector4: ...
    @property
    def tsso(self) -> FVector4: ...
    @property
    def tssl(self) -> FVector4: ...
    @property
    def tsts(self) -> FVector4: ...
    @property
    def tstt(self) -> FVector4: ...
    @property
    def tstq(self) -> FVector4: ...
    @property
    def tstp(self) -> FVector4: ...
    @property
    def tsto(self) -> FVector4: ...
    @property
    def tstl(self) -> FVector4: ...
    @property
    def tsqs(self) -> FVector4: ...
    @property
    def tsqt(self) -> FVector4: ...
    @property
    def tsqq(self) -> FVector4: ...
    @property
    def tsqp(self) -> FVector4: ...
    @property
    def tsqo(self) -> FVector4: ...
    @property
    def tsql(self) -> FVector4: ...
    @property
    def tsps(self) -> FVector4: ...
    @property
    def tspt(self) -> FVector4: ...
    @property
    def tspq(self) -> FVector4: ...
    @property
    def tspp(self) -> FVector4: ...
    @property
    def tspo(self) -> FVector4: ...
    @property
    def tspl(self) -> FVector4: ...
    @property
    def tsos(self) -> FVector4: ...
    @property
    def tsot(self) -> FVector4: ...
    @property
    def tsoq(self) -> FVector4: ...
    @property
    def tsop(self) -> FVector4: ...
    @property
    def tsoo(self) -> FVector4: ...
    @property
    def tsol(self) -> FVector4: ...
    @property
    def tsls(self) -> FVector4: ...
    @property
    def tslt(self) -> FVector4: ...
    @property
    def tslq(self) -> FVector4: ...
    @property
    def tslp(self) -> FVector4: ...
    @property
    def tslo(self) -> FVector4: ...
    @property
    def tsll(self) -> FVector4: ...
    @property
    def ttss(self) -> FVector4: ...
    @property
    def ttst(self) -> FVector4: ...
    @property
    def ttsq(self) -> FVector4: ...
    @property
    def ttsp(self) -> FVector4: ...
    @property
    def ttso(self) -> FVector4: ...
    @property
    def ttsl(self) -> FVector4: ...
    @property
    def ttts(self) -> FVector4: ...
    @property
    def tttt(self) -> FVector4: ...
    @property
    def tttq(self) -> FVector4: ...
    @property
    def tttp(self) -> FVector4: ...
    @property
    def ttto(self) -> FVector4: ...
    @property
    def tttl(self) -> FVector4: ...
    @property
    def ttqs(self) -> FVector4: ...
    @property
    def ttqt(self) -> FVector4: ...
    @property
    def ttqq(self) -> FVector4: ...
    @property
    def ttqp(self) -> FVector4: ...
    @property
    def ttqo(self) -> FVector4: ...
    @property
    def ttql(self) -> FVector4: ...
    @property
    def ttps(self) -> FVector4: ...
    @property
    def ttpt(self) -> FVector4: ...
    @property
    def ttpq(self) -> FVector4: ...
    @property
    def ttpp(self) -> FVector4: ...
    @property
    def ttpo(self) -> FVector4: ...
    @property
    def ttpl(self) -> FVector4: ...
    @property
    def ttos(self) -> FVector4: ...
    @property
    def ttot(self) -> FVector4: ...
    @property
    def ttoq(self) -> FVector4: ...
    @property
    def ttop(self) -> FVector4: ...
    @property
    def ttoo(self) -> FVector4: ...
    @property
    def ttol(self) -> FVector4: ...
    @property
    def ttls(self) -> FVector4: ...
    @property
    def ttlt(self) -> FVector4: ...
    @property
    def ttlq(self) -> FVector4: ...
    @property
    def ttlp(self) -> FVector4: ...
    @property
    def ttlo(self) -> FVector4: ...
    @property
    def ttll(self) -> FVector4: ...
    @property
    def tqss(self) -> FVector4: ...
    @property
    def tqst(self) -> FVector4: ...
    @property
    def tqsq(self) -> FVector4: ...
    @property
    def tqsp(self) -> FVector4: ...
    @property
    def tqso(self) -> FVector4: ...
    @property
    def tqsl(self) -> FVector4: ...
    @property
    def tqts(self) -> FVector4: ...
    @property
    def tqtt(self) -> FVector4: ...
    @property
    def tqtq(self) -> FVector4: ...
    @property
    def tqtp(self) -> FVector4: ...
    @property
    def tqto(self) -> FVector4: ...
    @property
    def tqtl(self) -> FVector4: ...
    @property
    def tqqs(self) -> FVector4: ...
    @property
    def tqqt(self) -> FVector4: ...
    @property
    def tqqq(self) -> FVector4: ...
    @property
    def tqqp(self) -> FVector4: ...
    @property
    def tqqo(self) -> FVector4: ...
    @property
    def tqql(self) -> FVector4: ...
    @property
    def tqps(self) -> FVector4: ...
    @property
    def tqpt(self) -> FVector4: ...
    @property
    def tqpq(self) -> FVector4: ...
    @property
    def tqpp(self) -> FVector4: ...
    @property
    def tqpo(self) -> FVector4: ...
    @property
    def tqpl(self) -> FVector4: ...
    @property
    def tqos(self) -> FVector4: ...
    @property
    def tqot(self) -> FVector4: ...
    @property
    def tqoq(self) -> FVector4: ...
    @property
    def tqop(self) -> FVector4: ...
    @property
    def tqoo(self) -> FVector4: ...
    @property
    def tqol(self) -> FVector4: ...
    @property
    def tqls(self) -> FVector4: ...
    @property
    def tqlt(self) -> FVector4: ...
    @property
    def tqlq(self) -> FVector4: ...
    @property
    def tqlp(self) -> FVector4: ...
    @property
    def tqlo(self) -> FVector4: ...
    @property
    def tqll(self) -> FVector4: ...
    @property
    def tpss(self) -> FVector4: ...
    @property
    def tpst(self) -> FVector4: ...
    @property
    def tpsq(self) -> FVector4: ...
    @property
    def tpsp(self) -> FVector4: ...
    @property
    def tpso(self) -> FVector4: ...
    @property
    def tpsl(self) -> FVector4: ...
    @property
    def tpts(self) -> FVector4: ...
    @property
    def tptt(self) -> FVector4: ...
    @property
    def tptq(self) -> FVector4: ...
    @property
    def tptp(self) -> FVector4: ...
    @property
    def tpto(self) -> FVector4: ...
    @property
    def tptl(self) -> FVector4: ...
    @property
    def tpqs(self) -> FVector4: ...
    @property
    def tpqt(self) -> FVector4: ...
    @property
    def tpqq(self) -> FVector4: ...
    @property
    def tpqp(self) -> FVector4: ...
    @property
    def tpqo(self) -> FVector4: ...
    @property
    def tpql(self) -> FVector4: ...
    @property
    def tpps(self) -> FVector4: ...
    @property
    def tppt(self) -> FVector4: ...
    @property
    def tppq(self) -> FVector4: ...
    @property
    def tppp(self) -> FVector4: ...
    @property
    def tppo(self) -> FVector4: ...
    @property
    def tppl(self) -> FVector4: ...
    @property
    def tpos(self) -> FVector4: ...
    @property
    def tpot(self) -> FVector4: ...
    @property
    def tpoq(self) -> FVector4: ...
    @property
    def tpop(self) -> FVector4: ...
    @property
    def tpoo(self) -> FVector4: ...
    @property
    def tpol(self) -> FVector4: ...
    @property
    def tpls(self) -> FVector4: ...
    @property
    def tplt(self) -> FVector4: ...
    @property
    def tplq(self) -> FVector4: ...
    @property
    def tplp(self) -> FVector4: ...
    @property
    def tplo(self) -> FVector4: ...
    @property
    def tpll(self) -> FVector4: ...
    @property
    def toss(self) -> FVector4: ...
    @property
    def tost(self) -> FVector4: ...
    @property
    def tosq(self) -> FVector4: ...
    @property
    def tosp(self) -> FVector4: ...
    @property
    def toso(self) -> FVector4: ...
    @property
    def tosl(self) -> FVector4: ...
    @property
    def tots(self) -> FVector4: ...
    @property
    def tott(self) -> FVector4: ...
    @property
    def totq(self) -> FVector4: ...
    @property
    def totp(self) -> FVector4: ...
    @property
    def toto(self) -> FVector4: ...
    @property
    def totl(self) -> FVector4: ...
    @property
    def toqs(self) -> FVector4: ...
    @property
    def toqt(self) -> FVector4: ...
    @property
    def toqq(self) -> FVector4: ...
    @property
    def toqp(self) -> FVector4: ...
    @property
    def toqo(self) -> FVector4: ...
    @property
    def toql(self) -> FVector4: ...
    @property
    def tops(self) -> FVector4: ...
    @property
    def topt(self) -> FVector4: ...
    @property
    def topq(self) -> FVector4: ...
    @property
    def topp(self) -> FVector4: ...
    @property
    def topo(self) -> FVector4: ...
    @property
    def topl(self) -> FVector4: ...
    @property
    def toos(self) -> FVector4: ...
    @property
    def toot(self) -> FVector4: ...
    @property
    def tooq(self) -> FVector4: ...
    @property
    def toop(self) -> FVector4: ...
    @property
    def tooo(self) -> FVector4: ...
    @property
    def tool(self) -> FVector4: ...
    @property
    def tols(self) -> FVector4: ...
    @property
    def tolt(self) -> FVector4: ...
    @property
    def tolq(self) -> FVector4: ...
    @property
    def tolp(self) -> FVector4: ...
    @property
    def tolo(self) -> FVector4: ...
    @property
    def toll(self) -> FVector4: ...
    @property
    def tlss(self) -> FVector4: ...
    @property
    def tlst(self) -> FVector4: ...
    @property
    def tlsq(self) -> FVector4: ...
    @property
    def tlsp(self) -> FVector4: ...
    @property
    def tlso(self) -> FVector4: ...
    @property
    def tlsl(self) -> FVector4: ...
    @property
    def tlts(self) -> FVector4: ...
    @property
    def tltt(self) -> FVector4: ...
    @property
    def tltq(self) -> FVector4: ...
    @property
    def tltp(self) -> FVector4: ...
    @property
    def tlto(self) -> FVector4: ...
    @property
    def tltl(self) -> FVector4: ...
    @property
    def tlqs(self) -> FVector4: ...
    @property
    def tlqt(self) -> FVector4: ...
    @property
    def tlqq(self) -> FVector4: ...
    @property
    def tlqp(self) -> FVector4: ...
    @property
    def tlqo(self) -> FVector4: ...
    @property
    def tlql(self) -> FVector4: ...
    @property
    def tlps(self) -> FVector4: ...
    @property
    def tlpt(self) -> FVector4: ...
    @property
    def tlpq(self) -> FVector4: ...
    @property
    def tlpp(self) -> FVector4: ...
    @property
    def tlpo(self) -> FVector4: ...
    @property
    def tlpl(self) -> FVector4: ...
    @property
    def tlos(self) -> FVector4: ...
    @property
    def tlot(self) -> FVector4: ...
    @property
    def tloq(self) -> FVector4: ...
    @property
    def tlop(self) -> FVector4: ...
    @property
    def tloo(self) -> FVector4: ...
    @property
    def tlol(self) -> FVector4: ...
    @property
    def tlls(self) -> FVector4: ...
    @property
    def tllt(self) -> FVector4: ...
    @property
    def tllq(self) -> FVector4: ...
    @property
    def tllp(self) -> FVector4: ...
    @property
    def tllo(self) -> FVector4: ...
    @property
    def tlll(self) -> FVector4: ...
    @property
    def qsss(self) -> FVector4: ...
    @property
    def qsst(self) -> FVector4: ...
    @property
    def qssq(self) -> FVector4: ...
    @property
    def qssp(self) -> FVector4: ...
    @property
    def qsso(self) -> FVector4: ...
    @property
    def qssl(self) -> FVector4: ...
    @property
    def qsts(self) -> FVector4: ...
    @property
    def qstt(self) -> FVector4: ...
    @property
    def qstq(self) -> FVector4: ...
    @property
    def qstp(self) -> FVector4: ...
    @property
    def qsto(self) -> FVector4: ...
    @property
    def qstl(self) -> FVector4: ...
    @property
    def qsqs(self) -> FVector4: ...
    @property
    def qsqt(self) -> FVector4: ...
    @property
    def qsqq(self) -> FVector4: ...
    @property
    def qsqp(self) -> FVector4: ...
    @property
    def qsqo(self) -> FVector4: ...
    @property
    def qsql(self) -> FVector4: ...
    @property
    def qsps(self) -> FVector4: ...
    @property
    def qspt(self) -> FVector4: ...
    @property
    def qspq(self) -> FVector4: ...
    @property
    def qspp(self) -> FVector4: ...
    @property
    def qspo(self) -> FVector4: ...
    @property
    def qspl(self) -> FVector4: ...
    @property
    def qsos(self) -> FVector4: ...
    @property
    def qsot(self) -> FVector4: ...
    @property
    def qsoq(self) -> FVector4: ...
    @property
    def qsop(self) -> FVector4: ...
    @property
    def qsoo(self) -> FVector4: ...
    @property
    def qsol(self) -> FVector4: ...
    @property
    def qsls(self) -> FVector4: ...
    @property
    def qslt(self) -> FVector4: ...
    @property
    def qslq(self) -> FVector4: ...
    @property
    def qslp(self) -> FVector4: ...
    @property
    def qslo(self) -> FVector4: ...
    @property
    def qsll(self) -> FVector4: ...
    @property
    def qtss(self) -> FVector4: ...
    @property
    def qtst(self) -> FVector4: ...
    @property
    def qtsq(self) -> FVector4: ...
    @property
    def qtsp(self) -> FVector4: ...
    @property
    def qtso(self) -> FVector4: ...
    @property
    def qtsl(self) -> FVector4: ...
    @property
    def qtts(self) -> FVector4: ...
    @property
    def qttt(self) -> FVector4: ...
    @property
    def qttq(self) -> FVector4: ...
    @property
    def qttp(self) -> FVector4: ...
    @property
    def qtto(self) -> FVector4: ...
    @property
    def qttl(self) -> FVector4: ...
    @property
    def qtqs(self) -> FVector4: ...
    @property
    def qtqt(self) -> FVector4: ...
    @property
    def qtqq(self) -> FVector4: ...
    @property
    def qtqp(self) -> FVector4: ...
    @property
    def qtqo(self) -> FVector4: ...
    @property
    def qtql(self) -> FVector4: ...
    @property
    def qtps(self) -> FVector4: ...
    @property
    def qtpt(self) -> FVector4: ...
    @property
    def qtpq(self) -> FVector4: ...
    @property
    def qtpp(self) -> FVector4: ...
    @property
    def qtpo(self) -> FVector4: ...
    @property
    def qtpl(self) -> FVector4: ...
    @property
    def qtos(self) -> FVector4: ...
    @property
    def qtot(self) -> FVector4: ...
    @property
    def qtoq(self) -> FVector4: ...
    @property
    def qtop(self) -> FVector4: ...
    @property
    def qtoo(self) -> FVector4: ...
    @property
    def qtol(self) -> FVector4: ...
    @property
    def qtls(self) -> FVector4: ...
    @property
    def qtlt(self) -> FVector4: ...
    @property
    def qtlq(self) -> FVector4: ...
    @property
    def qtlp(self) -> FVector4: ...
    @property
    def qtlo(self) -> FVector4: ...
    @property
    def qtll(self) -> FVector4: ...
    @property
    def qqss(self) -> FVector4: ...
    @property
    def qqst(self) -> FVector4: ...
    @property
    def qqsq(self) -> FVector4: ...
    @property
    def qqsp(self) -> FVector4: ...
    @property
    def qqso(self) -> FVector4: ...
    @property
    def qqsl(self) -> FVector4: ...
    @property
    def qqts(self) -> FVector4: ...
    @property
    def qqtt(self) -> FVector4: ...
    @property
    def qqtq(self) -> FVector4: ...
    @property
    def qqtp(self) -> FVector4: ...
    @property
    def qqto(self) -> FVector4: ...
    @property
    def qqtl(self) -> FVector4: ...
    @property
    def qqqs(self) -> FVector4: ...
    @property
    def qqqt(self) -> FVector4: ...
    @property
    def qqqq(self) -> FVector4: ...
    @property
    def qqqp(self) -> FVector4: ...
    @property
    def qqqo(self) -> FVector4: ...
    @property
    def qqql(self) -> FVector4: ...
    @property
    def qqps(self) -> FVector4: ...
    @property
    def qqpt(self) -> FVector4: ...
    @property
    def qqpq(self) -> FVector4: ...
    @property
    def qqpp(self) -> FVector4: ...
    @property
    def qqpo(self) -> FVector4: ...
    @property
    def qqpl(self) -> FVector4: ...
    @property
    def qqos(self) -> FVector4: ...
    @property
    def qqot(self) -> FVector4: ...
    @property
    def qqoq(self) -> FVector4: ...
    @property
    def qqop(self) -> FVector4: ...
    @property
    def qqoo(self) -> FVector4: ...
    @property
    def qqol(self) -> FVector4: ...
    @property
    def qqls(self) -> FVector4: ...
    @property
    def qqlt(self) -> FVector4: ...
    @property
    def qqlq(self) -> FVector4: ...
    @property
    def qqlp(self) -> FVector4: ...
    @property
    def qqlo(self) -> FVector4: ...
    @property
    def qqll(self) -> FVector4: ...
    @property
    def qpss(self) -> FVector4: ...
    @property
    def qpst(self) -> FVector4: ...
    @property
    def qpsq(self) -> FVector4: ...
    @property
    def qpsp(self) -> FVector4: ...
    @property
    def qpso(self) -> FVector4: ...
    @property
    def qpsl(self) -> FVector4: ...
    @property
    def qpts(self) -> FVector4: ...
    @property
    def qptt(self) -> FVector4: ...
    @property
    def qptq(self) -> FVector4: ...
    @property
    def qptp(self) -> FVector4: ...
    @property
    def qpto(self) -> FVector4: ...
    @property
    def qptl(self) -> FVector4: ...
    @property
    def qpqs(self) -> FVector4: ...
    @property
    def qpqt(self) -> FVector4: ...
    @property
    def qpqq(self) -> FVector4: ...
    @property
    def qpqp(self) -> FVector4: ...
    @property
    def qpqo(self) -> FVector4: ...
    @property
    def qpql(self) -> FVector4: ...
    @property
    def qpps(self) -> FVector4: ...
    @property
    def qppt(self) -> FVector4: ...
    @property
    def qppq(self) -> FVector4: ...
    @property
    def qppp(self) -> FVector4: ...
    @property
    def qppo(self) -> FVector4: ...
    @property
    def qppl(self) -> FVector4: ...
    @property
    def qpos(self) -> FVector4: ...
    @property
    def qpot(self) -> FVector4: ...
    @property
    def qpoq(self) -> FVector4: ...
    @property
    def qpop(self) -> FVector4: ...
    @property
    def qpoo(self) -> FVector4: ...
    @property
    def qpol(self) -> FVector4: ...
    @property
    def qpls(self) -> FVector4: ...
    @property
    def qplt(self) -> FVector4: ...
    @property
    def qplq(self) -> FVector4: ...
    @property
    def qplp(self) -> FVector4: ...
    @property
    def qplo(self) -> FVector4: ...
    @property
    def qpll(self) -> FVector4: ...
    @property
    def qoss(self) -> FVector4: ...
    @property
    def qost(self) -> FVector4: ...
    @property
    def qosq(self) -> FVector4: ...
    @property
    def qosp(self) -> FVector4: ...
    @property
    def qoso(self) -> FVector4: ...
    @property
    def qosl(self) -> FVector4: ...
    @property
    def qots(self) -> FVector4: ...
    @property
    def qott(self) -> FVector4: ...
    @property
    def qotq(self) -> FVector4: ...
    @property
    def qotp(self) -> FVector4: ...
    @property
    def qoto(self) -> FVector4: ...
    @property
    def qotl(self) -> FVector4: ...
    @property
    def qoqs(self) -> FVector4: ...
    @property
    def qoqt(self) -> FVector4: ...
    @property
    def qoqq(self) -> FVector4: ...
    @property
    def qoqp(self) -> FVector4: ...
    @property
    def qoqo(self) -> FVector4: ...
    @property
    def qoql(self) -> FVector4: ...
    @property
    def qops(self) -> FVector4: ...
    @property
    def qopt(self) -> FVector4: ...
    @property
    def qopq(self) -> FVector4: ...
    @property
    def qopp(self) -> FVector4: ...
    @property
    def qopo(self) -> FVector4: ...
    @property
    def qopl(self) -> FVector4: ...
    @property
    def qoos(self) -> FVector4: ...
    @property
    def qoot(self) -> FVector4: ...
    @property
    def qooq(self) -> FVector4: ...
    @property
    def qoop(self) -> FVector4: ...
    @property
    def qooo(self) -> FVector4: ...
    @property
    def qool(self) -> FVector4: ...
    @property
    def qols(self) -> FVector4: ...
    @property
    def qolt(self) -> FVector4: ...
    @property
    def qolq(self) -> FVector4: ...
    @property
    def qolp(self) -> FVector4: ...
    @property
    def qolo(self) -> FVector4: ...
    @property
    def qoll(self) -> FVector4: ...
    @property
    def qlss(self) -> FVector4: ...
    @property
    def qlst(self) -> FVector4: ...
    @property
    def qlsq(self) -> FVector4: ...
    @property
    def qlsp(self) -> FVector4: ...
    @property
    def qlso(self) -> FVector4: ...
    @property
    def qlsl(self) -> FVector4: ...
    @property
    def qlts(self) -> FVector4: ...
    @property
    def qltt(self) -> FVector4: ...
    @property
    def qltq(self) -> FVector4: ...
    @property
    def qltp(self) -> FVector4: ...
    @property
    def qlto(self) -> FVector4: ...
    @property
    def qltl(self) -> FVector4: ...
    @property
    def qlqs(self) -> FVector4: ...
    @property
    def qlqt(self) -> FVector4: ...
    @property
    def qlqq(self) -> FVector4: ...
    @property
    def qlqp(self) -> FVector4: ...
    @property
    def qlqo(self) -> FVector4: ...
    @property
    def qlql(self) -> FVector4: ...
    @property
    def qlps(self) -> FVector4: ...
    @property
    def qlpt(self) -> FVector4: ...
    @property
    def qlpq(self) -> FVector4: ...
    @property
    def qlpp(self) -> FVector4: ...
    @property
    def qlpo(self) -> FVector4: ...
    @property
    def qlpl(self) -> FVector4: ...
    @property
    def qlos(self) -> FVector4: ...
    @property
    def qlot(self) -> FVector4: ...
    @property
    def qloq(self) -> FVector4: ...
    @property
    def qlop(self) -> FVector4: ...
    @property
    def qloo(self) -> FVector4: ...
    @property
    def qlol(self) -> FVector4: ...
    @property
    def qlls(self) -> FVector4: ...
    @property
    def qllt(self) -> FVector4: ...
    @property
    def qllq(self) -> FVector4: ...
    @property
    def qllp(self) -> FVector4: ...
    @property
    def qllo(self) -> FVector4: ...
    @property
    def qlll(self) -> FVector4: ...
    @property
    def psss(self) -> FVector4: ...
    @property
    def psst(self) -> FVector4: ...
    @property
    def pssq(self) -> FVector4: ...
    @property
    def pssp(self) -> FVector4: ...
    @property
    def psso(self) -> FVector4: ...
    @property
    def pssl(self) -> FVector4: ...
    @property
    def psts(self) -> FVector4: ...
    @property
    def pstt(self) -> FVector4: ...
    @property
    def pstq(self) -> FVector4: ...
    @property
    def pstp(self) -> FVector4: ...
    @property
    def psto(self) -> FVector4: ...
    @property
    def pstl(self) -> FVector4: ...
    @property
    def psqs(self) -> FVector4: ...
    @property
    def psqt(self) -> FVector4: ...
    @property
    def psqq(self) -> FVector4: ...
    @property
    def psqp(self) -> FVector4: ...
    @property
    def psqo(self) -> FVector4: ...
    @property
    def psql(self) -> FVector4: ...
    @property
    def psps(self) -> FVector4: ...
    @property
    def pspt(self) -> FVector4: ...
    @property
    def pspq(self) -> FVector4: ...
    @property
    def pspp(self) -> FVector4: ...
    @property
    def pspo(self) -> FVector4: ...
    @property
    def pspl(self) -> FVector4: ...
    @property
    def psos(self) -> FVector4: ...
    @property
    def psot(self) -> FVector4: ...
    @property
    def psoq(self) -> FVector4: ...
    @property
    def psop(self) -> FVector4: ...
    @property
    def psoo(self) -> FVector4: ...
    @property
    def psol(self) -> FVector4: ...
    @property
    def psls(self) -> FVector4: ...
    @property
    def pslt(self) -> FVector4: ...
    @property
    def pslq(self) -> FVector4: ...
    @property
    def pslp(self) -> FVector4: ...
    @property
    def pslo(self) -> FVector4: ...
    @property
    def psll(self) -> FVector4: ...
    @property
    def ptss(self) -> FVector4: ...
    @property
    def ptst(self) -> FVector4: ...
    @property
    def ptsq(self) -> FVector4: ...
    @property
    def ptsp(self) -> FVector4: ...
    @property
    def ptso(self) -> FVector4: ...
    @property
    def ptsl(self) -> FVector4: ...
    @property
    def ptts(self) -> FVector4: ...
    @property
    def pttt(self) -> FVector4: ...
    @property
    def pttq(self) -> FVector4: ...
    @property
    def pttp(self) -> FVector4: ...
    @property
    def ptto(self) -> FVector4: ...
    @property
    def pttl(self) -> FVector4: ...
    @property
    def ptqs(self) -> FVector4: ...
    @property
    def ptqt(self) -> FVector4: ...
    @property
    def ptqq(self) -> FVector4: ...
    @property
    def ptqp(self) -> FVector4: ...
    @property
    def ptqo(self) -> FVector4: ...
    @property
    def ptql(self) -> FVector4: ...
    @property
    def ptps(self) -> FVector4: ...
    @property
    def ptpt(self) -> FVector4: ...
    @property
    def ptpq(self) -> FVector4: ...
    @property
    def ptpp(self) -> FVector4: ...
    @property
    def ptpo(self) -> FVector4: ...
    @property
    def ptpl(self) -> FVector4: ...
    @property
    def ptos(self) -> FVector4: ...
    @property
    def ptot(self) -> FVector4: ...
    @property
    def ptoq(self) -> FVector4: ...
    @property
    def ptop(self) -> FVector4: ...
    @property
    def ptoo(self) -> FVector4: ...
    @property
    def ptol(self) -> FVector4: ...
    @property
    def ptls(self) -> FVector4: ...
    @property
    def ptlt(self) -> FVector4: ...
    @property
    def ptlq(self) -> FVector4: ...
    @property
    def ptlp(self) -> FVector4: ...
    @property
    def ptlo(self) -> FVector4: ...
    @property
    def ptll(self) -> FVector4: ...
    @property
    def pqss(self) -> FVector4: ...
    @property
    def pqst(self) -> FVector4: ...
    @property
    def pqsq(self) -> FVector4: ...
    @property
    def pqsp(self) -> FVector4: ...
    @property
    def pqso(self) -> FVector4: ...
    @property
    def pqsl(self) -> FVector4: ...
    @property
    def pqts(self) -> FVector4: ...
    @property
    def pqtt(self) -> FVector4: ...
    @property
    def pqtq(self) -> FVector4: ...
    @property
    def pqtp(self) -> FVector4: ...
    @property
    def pqto(self) -> FVector4: ...
    @property
    def pqtl(self) -> FVector4: ...
    @property
    def pqqs(self) -> FVector4: ...
    @property
    def pqqt(self) -> FVector4: ...
    @property
    def pqqq(self) -> FVector4: ...
    @property
    def pqqp(self) -> FVector4: ...
    @property
    def pqqo(self) -> FVector4: ...
    @property
    def pqql(self) -> FVector4: ...
    @property
    def pqps(self) -> FVector4: ...
    @property
    def pqpt(self) -> FVector4: ...
    @property
    def pqpq(self) -> FVector4: ...
    @property
    def pqpp(self) -> FVector4: ...
    @property
    def pqpo(self) -> FVector4: ...
    @property
    def pqpl(self) -> FVector4: ...
    @property
    def pqos(self) -> FVector4: ...
    @property
    def pqot(self) -> FVector4: ...
    @property
    def pqoq(self) -> FVector4: ...
    @property
    def pqop(self) -> FVector4: ...
    @property
    def pqoo(self) -> FVector4: ...
    @property
    def pqol(self) -> FVector4: ...
    @property
    def pqls(self) -> FVector4: ...
    @property
    def pqlt(self) -> FVector4: ...
    @property
    def pqlq(self) -> FVector4: ...
    @property
    def pqlp(self) -> FVector4: ...
    @property
    def pqlo(self) -> FVector4: ...
    @property
    def pqll(self) -> FVector4: ...
    @property
    def ppss(self) -> FVector4: ...
    @property
    def ppst(self) -> FVector4: ...
    @property
    def ppsq(self) -> FVector4: ...
    @property
    def ppsp(self) -> FVector4: ...
    @property
    def ppso(self) -> FVector4: ...
    @property
    def ppsl(self) -> FVector4: ...
    @property
    def ppts(self) -> FVector4: ...
    @property
    def pptt(self) -> FVector4: ...
    @property
    def pptq(self) -> FVector4: ...
    @property
    def pptp(self) -> FVector4: ...
    @property
    def ppto(self) -> FVector4: ...
    @property
    def pptl(self) -> FVector4: ...
    @property
    def ppqs(self) -> FVector4: ...
    @property
    def ppqt(self) -> FVector4: ...
    @property
    def ppqq(self) -> FVector4: ...
    @property
    def ppqp(self) -> FVector4: ...
    @property
    def ppqo(self) -> FVector4: ...
    @property
    def ppql(self) -> FVector4: ...
    @property
    def ppps(self) -> FVector4: ...
    @property
    def pppt(self) -> FVector4: ...
    @property
    def pppq(self) -> FVector4: ...
    @property
    def pppp(self) -> FVector4: ...
    @property
    def pppo(self) -> FVector4: ...
    @property
    def pppl(self) -> FVector4: ...
    @property
    def ppos(self) -> FVector4: ...
    @property
    def ppot(self) -> FVector4: ...
    @property
    def ppoq(self) -> FVector4: ...
    @property
    def ppop(self) -> FVector4: ...
    @property
    def ppoo(self) -> FVector4: ...
    @property
    def ppol(self) -> FVector4: ...
    @property
    def ppls(self) -> FVector4: ...
    @property
    def pplt(self) -> FVector4: ...
    @property
    def pplq(self) -> FVector4: ...
    @property
    def pplp(self) -> FVector4: ...
    @property
    def pplo(self) -> FVector4: ...
    @property
    def ppll(self) -> FVector4: ...
    @property
    def poss(self) -> FVector4: ...
    @property
    def post(self) -> FVector4: ...
    @property
    def posq(self) -> FVector4: ...
    @property
    def posp(self) -> FVector4: ...
    @property
    def poso(self) -> FVector4: ...
    @property
    def posl(self) -> FVector4: ...
    @property
    def pots(self) -> FVector4: ...
    @property
    def pott(self) -> FVector4: ...
    @property
    def potq(self) -> FVector4: ...
    @property
    def potp(self) -> FVector4: ...
    @property
    def poto(self) -> FVector4: ...
    @property
    def potl(self) -> FVector4: ...
    @property
    def poqs(self) -> FVector4: ...
    @property
    def poqt(self) -> FVector4: ...
    @property
    def poqq(self) -> FVector4: ...
    @property
    def poqp(self) -> FVector4: ...
    @property
    def poqo(self) -> FVector4: ...
    @property
    def poql(self) -> FVector4: ...
    @property
    def pops(self) -> FVector4: ...
    @property
    def popt(self) -> FVector4: ...
    @property
    def popq(self) -> FVector4: ...
    @property
    def popp(self) -> FVector4: ...
    @property
    def popo(self) -> FVector4: ...
    @property
    def popl(self) -> FVector4: ...
    @property
    def poos(self) -> FVector4: ...
    @property
    def poot(self) -> FVector4: ...
    @property
    def pooq(self) -> FVector4: ...
    @property
    def poop(self) -> FVector4: ...
    @property
    def pooo(self) -> FVector4: ...
    @property
    def pool(self) -> FVector4: ...
    @property
    def pols(self) -> FVector4: ...
    @property
    def polt(self) -> FVector4: ...
    @property
    def polq(self) -> FVector4: ...
    @property
    def polp(self) -> FVector4: ...
    @property
    def polo(self) -> FVector4: ...
    @property
    def poll(self) -> FVector4: ...
    @property
    def plss(self) -> FVector4: ...
    @property
    def plst(self) -> FVector4: ...
    @property
    def plsq(self) -> FVector4: ...
    @property
    def plsp(self) -> FVector4: ...
    @property
    def plso(self) -> FVector4: ...
    @property
    def plsl(self) -> FVector4: ...
    @property
    def plts(self) -> FVector4: ...
    @property
    def pltt(self) -> FVector4: ...
    @property
    def pltq(self) -> FVector4: ...
    @property
    def pltp(self) -> FVector4: ...
    @property
    def plto(self) -> FVector4: ...
    @property
    def pltl(self) -> FVector4: ...
    @property
    def plqs(self) -> FVector4: ...
    @property
    def plqt(self) -> FVector4: ...
    @property
    def plqq(self) -> FVector4: ...
    @property
    def plqp(self) -> FVector4: ...
    @property
    def plqo(self) -> FVector4: ...
    @property
    def plql(self) -> FVector4: ...
    @property
    def plps(self) -> FVector4: ...
    @property
    def plpt(self) -> FVector4: ...
    @property
    def plpq(self) -> FVector4: ...
    @property
    def plpp(self) -> FVector4: ...
    @property
    def plpo(self) -> FVector4: ...
    @property
    def plpl(self) -> FVector4: ...
    @property
    def plos(self) -> FVector4: ...
    @property
    def plot(self) -> FVector4: ...
    @property
    def ploq(self) -> FVector4: ...
    @property
    def plop(self) -> FVector4: ...
    @property
    def ploo(self) -> FVector4: ...
    @property
    def plol(self) -> FVector4: ...
    @property
    def plls(self) -> FVector4: ...
    @property
    def pllt(self) -> FVector4: ...
    @property
    def pllq(self) -> FVector4: ...
    @property
    def pllp(self) -> FVector4: ...
    @property
    def pllo(self) -> FVector4: ...
    @property
    def plll(self) -> FVector4: ...
    @property
    def osss(self) -> FVector4: ...
    @property
    def osst(self) -> FVector4: ...
    @property
    def ossq(self) -> FVector4: ...
    @property
    def ossp(self) -> FVector4: ...
    @property
    def osso(self) -> FVector4: ...
    @property
    def ossl(self) -> FVector4: ...
    @property
    def osts(self) -> FVector4: ...
    @property
    def ostt(self) -> FVector4: ...
    @property
    def ostq(self) -> FVector4: ...
    @property
    def ostp(self) -> FVector4: ...
    @property
    def osto(self) -> FVector4: ...
    @property
    def ostl(self) -> FVector4: ...
    @property
    def osqs(self) -> FVector4: ...
    @property
    def osqt(self) -> FVector4: ...
    @property
    def osqq(self) -> FVector4: ...
    @property
    def osqp(self) -> FVector4: ...
    @property
    def osqo(self) -> FVector4: ...
    @property
    def osql(self) -> FVector4: ...
    @property
    def osps(self) -> FVector4: ...
    @property
    def ospt(self) -> FVector4: ...
    @property
    def ospq(self) -> FVector4: ...
    @property
    def ospp(self) -> FVector4: ...
    @property
    def ospo(self) -> FVector4: ...
    @property
    def ospl(self) -> FVector4: ...
    @property
    def osos(self) -> FVector4: ...
    @property
    def osot(self) -> FVector4: ...
    @property
    def osoq(self) -> FVector4: ...
    @property
    def osop(self) -> FVector4: ...
    @property
    def osoo(self) -> FVector4: ...
    @property
    def osol(self) -> FVector4: ...
    @property
    def osls(self) -> FVector4: ...
    @property
    def oslt(self) -> FVector4: ...
    @property
    def oslq(self) -> FVector4: ...
    @property
    def oslp(self) -> FVector4: ...
    @property
    def oslo(self) -> FVector4: ...
    @property
    def osll(self) -> FVector4: ...
    @property
    def otss(self) -> FVector4: ...
    @property
    def otst(self) -> FVector4: ...
    @property
    def otsq(self) -> FVector4: ...
    @property
    def otsp(self) -> FVector4: ...
    @property
    def otso(self) -> FVector4: ...
    @property
    def otsl(self) -> FVector4: ...
    @property
    def otts(self) -> FVector4: ...
    @property
    def ottt(self) -> FVector4: ...
    @property
    def ottq(self) -> FVector4: ...
    @property
    def ottp(self) -> FVector4: ...
    @property
    def otto(self) -> FVector4: ...
    @property
    def ottl(self) -> FVector4: ...
    @property
    def otqs(self) -> FVector4: ...
    @property
    def otqt(self) -> FVector4: ...
    @property
    def otqq(self) -> FVector4: ...
    @property
    def otqp(self) -> FVector4: ...
    @property
    def otqo(self) -> FVector4: ...
    @property
    def otql(self) -> FVector4: ...
    @property
    def otps(self) -> FVector4: ...
    @property
    def otpt(self) -> FVector4: ...
    @property
    def otpq(self) -> FVector4: ...
    @property
    def otpp(self) -> FVector4: ...
    @property
    def otpo(self) -> FVector4: ...
    @property
    def otpl(self) -> FVector4: ...
    @property
    def otos(self) -> FVector4: ...
    @property
    def otot(self) -> FVector4: ...
    @property
    def otoq(self) -> FVector4: ...
    @property
    def otop(self) -> FVector4: ...
    @property
    def otoo(self) -> FVector4: ...
    @property
    def otol(self) -> FVector4: ...
    @property
    def otls(self) -> FVector4: ...
    @property
    def otlt(self) -> FVector4: ...
    @property
    def otlq(self) -> FVector4: ...
    @property
    def otlp(self) -> FVector4: ...
    @property
    def otlo(self) -> FVector4: ...
    @property
    def otll(self) -> FVector4: ...
    @property
    def oqss(self) -> FVector4: ...
    @property
    def oqst(self) -> FVector4: ...
    @property
    def oqsq(self) -> FVector4: ...
    @property
    def oqsp(self) -> FVector4: ...
    @property
    def oqso(self) -> FVector4: ...
    @property
    def oqsl(self) -> FVector4: ...
    @property
    def oqts(self) -> FVector4: ...
    @property
    def oqtt(self) -> FVector4: ...
    @property
    def oqtq(self) -> FVector4: ...
    @property
    def oqtp(self) -> FVector4: ...
    @property
    def oqto(self) -> FVector4: ...
    @property
    def oqtl(self) -> FVector4: ...
    @property
    def oqqs(self) -> FVector4: ...
    @property
    def oqqt(self) -> FVector4: ...
    @property
    def oqqq(self) -> FVector4: ...
    @property
    def oqqp(self) -> FVector4: ...
    @property
    def oqqo(self) -> FVector4: ...
    @property
    def oqql(self) -> FVector4: ...
    @property
    def oqps(self) -> FVector4: ...
    @property
    def oqpt(self) -> FVector4: ...
    @property
    def oqpq(self) -> FVector4: ...
    @property
    def oqpp(self) -> FVector4: ...
    @property
    def oqpo(self) -> FVector4: ...
    @property
    def oqpl(self) -> FVector4: ...
    @property
    def oqos(self) -> FVector4: ...
    @property
    def oqot(self) -> FVector4: ...
    @property
    def oqoq(self) -> FVector4: ...
    @property
    def oqop(self) -> FVector4: ...
    @property
    def oqoo(self) -> FVector4: ...
    @property
    def oqol(self) -> FVector4: ...
    @property
    def oqls(self) -> FVector4: ...
    @property
    def oqlt(self) -> FVector4: ...
    @property
    def oqlq(self) -> FVector4: ...
    @property
    def oqlp(self) -> FVector4: ...
    @property
    def oqlo(self) -> FVector4: ...
    @property
    def oqll(self) -> FVector4: ...
    @property
    def opss(self) -> FVector4: ...
    @property
    def opst(self) -> FVector4: ...
    @property
    def opsq(self) -> FVector4: ...
    @property
    def opsp(self) -> FVector4: ...
    @property
    def opso(self) -> FVector4: ...
    @property
    def opsl(self) -> FVector4: ...
    @property
    def opts(self) -> FVector4: ...
    @property
    def optt(self) -> FVector4: ...
    @property
    def optq(self) -> FVector4: ...
    @property
    def optp(self) -> FVector4: ...
    @property
    def opto(self) -> FVector4: ...
    @property
    def optl(self) -> FVector4: ...
    @property
    def opqs(self) -> FVector4: ...
    @property
    def opqt(self) -> FVector4: ...
    @property
    def opqq(self) -> FVector4: ...
    @property
    def opqp(self) -> FVector4: ...
    @property
    def opqo(self) -> FVector4: ...
    @property
    def opql(self) -> FVector4: ...
    @property
    def opps(self) -> FVector4: ...
    @property
    def oppt(self) -> FVector4: ...
    @property
    def oppq(self) -> FVector4: ...
    @property
    def oppp(self) -> FVector4: ...
    @property
    def oppo(self) -> FVector4: ...
    @property
    def oppl(self) -> FVector4: ...
    @property
    def opos(self) -> FVector4: ...
    @property
    def opot(self) -> FVector4: ...
    @property
    def opoq(self) -> FVector4: ...
    @property
    def opop(self) -> FVector4: ...
    @property
    def opoo(self) -> FVector4: ...
    @property
    def opol(self) -> FVector4: ...
    @property
    def opls(self) -> FVector4: ...
    @property
    def oplt(self) -> FVector4: ...
    @property
    def oplq(self) -> FVector4: ...
    @property
    def oplp(self) -> FVector4: ...
    @property
    def oplo(self) -> FVector4: ...
    @property
    def opll(self) -> FVector4: ...
    @property
    def ooss(self) -> FVector4: ...
    @property
    def oost(self) -> FVector4: ...
    @property
    def oosq(self) -> FVector4: ...
    @property
    def oosp(self) -> FVector4: ...
    @property
    def ooso(self) -> FVector4: ...
    @property
    def oosl(self) -> FVector4: ...
    @property
    def oots(self) -> FVector4: ...
    @property
    def oott(self) -> FVector4: ...
    @property
    def ootq(self) -> FVector4: ...
    @property
    def ootp(self) -> FVector4: ...
    @property
    def ooto(self) -> FVector4: ...
    @property
    def ootl(self) -> FVector4: ...
    @property
    def ooqs(self) -> FVector4: ...
    @property
    def ooqt(self) -> FVector4: ...
    @property
    def ooqq(self) -> FVector4: ...
    @property
    def ooqp(self) -> FVector4: ...
    @property
    def ooqo(self) -> FVector4: ...
    @property
    def ooql(self) -> FVector4: ...
    @property
    def oops(self) -> FVector4: ...
    @property
    def oopt(self) -> FVector4: ...
    @property
    def oopq(self) -> FVector4: ...
    @property
    def oopp(self) -> FVector4: ...
    @property
    def oopo(self) -> FVector4: ...
    @property
    def oopl(self) -> FVector4: ...
    @property
    def ooos(self) -> FVector4: ...
    @property
    def ooot(self) -> FVector4: ...
    @property
    def oooq(self) -> FVector4: ...
    @property
    def ooop(self) -> FVector4: ...
    @property
    def ools(self) -> FVector4: ...
    @property
    def oolt(self) -> FVector4: ...
    @property
    def oolq(self) -> FVector4: ...
    @property
    def oolp(self) -> FVector4: ...
    @property
    def olss(self) -> FVector4: ...
    @property
    def olst(self) -> FVector4: ...
    @property
    def olsq(self) -> FVector4: ...
    @property
    def olsp(self) -> FVector4: ...
    @property
    def olso(self) -> FVector4: ...
    @property
    def olsl(self) -> FVector4: ...
    @property
    def olts(self) -> FVector4: ...
    @property
    def oltt(self) -> FVector4: ...
    @property
    def oltq(self) -> FVector4: ...
    @property
    def oltp(self) -> FVector4: ...
    @property
    def olto(self) -> FVector4: ...
    @property
    def oltl(self) -> FVector4: ...
    @property
    def olqs(self) -> FVector4: ...
    @property
    def olqt(self) -> FVector4: ...
    @property
    def olqq(self) -> FVector4: ...
    @property
    def olqp(self) -> FVector4: ...
    @property
    def olqo(self) -> FVector4: ...
    @property
    def olql(self) -> FVector4: ...
    @property
    def olps(self) -> FVector4: ...
    @property
    def olpt(self) -> FVector4: ...
    @property
    def olpq(self) -> FVector4: ...
    @property
    def olpp(self) -> FVector4: ...
    @property
    def olpo(self) -> FVector4: ...
    @property
    def olpl(self) -> FVector4: ...
    @property
    def olos(self) -> FVector4: ...
    @property
    def olot(self) -> FVector4: ...
    @property
    def oloq(self) -> FVector4: ...
    @property
    def olop(self) -> FVector4: ...
    @property
    def olls(self) -> FVector4: ...
    @property
    def ollt(self) -> FVector4: ...
    @property
    def ollq(self) -> FVector4: ...
    @property
    def ollp(self) -> FVector4: ...
    @property
    def lsss(self) -> FVector4: ...
    @property
    def lsst(self) -> FVector4: ...
    @property
    def lssq(self) -> FVector4: ...
    @property
    def lssp(self) -> FVector4: ...
    @property
    def lsso(self) -> FVector4: ...
    @property
    def lssl(self) -> FVector4: ...
    @property
    def lsts(self) -> FVector4: ...
    @property
    def lstt(self) -> FVector4: ...
    @property
    def lstq(self) -> FVector4: ...
    @property
    def lstp(self) -> FVector4: ...
    @property
    def lsto(self) -> FVector4: ...
    @property
    def lstl(self) -> FVector4: ...
    @property
    def lsqs(self) -> FVector4: ...
    @property
    def lsqt(self) -> FVector4: ...
    @property
    def lsqq(self) -> FVector4: ...
    @property
    def lsqp(self) -> FVector4: ...
    @property
    def lsqo(self) -> FVector4: ...
    @property
    def lsql(self) -> FVector4: ...
    @property
    def lsps(self) -> FVector4: ...
    @property
    def lspt(self) -> FVector4: ...
    @property
    def lspq(self) -> FVector4: ...
    @property
    def lspp(self) -> FVector4: ...
    @property
    def lspo(self) -> FVector4: ...
    @property
    def lspl(self) -> FVector4: ...
    @property
    def lsos(self) -> FVector4: ...
    @property
    def lsot(self) -> FVector4: ...
    @property
    def lsoq(self) -> FVector4: ...
    @property
    def lsop(self) -> FVector4: ...
    @property
    def lsoo(self) -> FVector4: ...
    @property
    def lsol(self) -> FVector4: ...
    @property
    def lsls(self) -> FVector4: ...
    @property
    def lslt(self) -> FVector4: ...
    @property
    def lslq(self) -> FVector4: ...
    @property
    def lslp(self) -> FVector4: ...
    @property
    def lslo(self) -> FVector4: ...
    @property
    def lsll(self) -> FVector4: ...
    @property
    def ltss(self) -> FVector4: ...
    @property
    def ltst(self) -> FVector4: ...
    @property
    def ltsq(self) -> FVector4: ...
    @property
    def ltsp(self) -> FVector4: ...
    @property
    def ltso(self) -> FVector4: ...
    @property
    def ltsl(self) -> FVector4: ...
    @property
    def ltts(self) -> FVector4: ...
    @property
    def lttt(self) -> FVector4: ...
    @property
    def lttq(self) -> FVector4: ...
    @property
    def lttp(self) -> FVector4: ...
    @property
    def ltto(self) -> FVector4: ...
    @property
    def lttl(self) -> FVector4: ...
    @property
    def ltqs(self) -> FVector4: ...
    @property
    def ltqt(self) -> FVector4: ...
    @property
    def ltqq(self) -> FVector4: ...
    @property
    def ltqp(self) -> FVector4: ...
    @property
    def ltqo(self) -> FVector4: ...
    @property
    def ltql(self) -> FVector4: ...
    @property
    def ltps(self) -> FVector4: ...
    @property
    def ltpt(self) -> FVector4: ...
    @property
    def ltpq(self) -> FVector4: ...
    @property
    def ltpp(self) -> FVector4: ...
    @property
    def ltpo(self) -> FVector4: ...
    @property
    def ltpl(self) -> FVector4: ...
    @property
    def ltos(self) -> FVector4: ...
    @property
    def ltot(self) -> FVector4: ...
    @property
    def ltoq(self) -> FVector4: ...
    @property
    def ltop(self) -> FVector4: ...
    @property
    def ltoo(self) -> FVector4: ...
    @property
    def ltol(self) -> FVector4: ...
    @property
    def ltls(self) -> FVector4: ...
    @property
    def ltlt(self) -> FVector4: ...
    @property
    def ltlq(self) -> FVector4: ...
    @property
    def ltlp(self) -> FVector4: ...
    @property
    def ltlo(self) -> FVector4: ...
    @property
    def ltll(self) -> FVector4: ...
    @property
    def lqss(self) -> FVector4: ...
    @property
    def lqst(self) -> FVector4: ...
    @property
    def lqsq(self) -> FVector4: ...
    @property
    def lqsp(self) -> FVector4: ...
    @property
    def lqso(self) -> FVector4: ...
    @property
    def lqsl(self) -> FVector4: ...
    @property
    def lqts(self) -> FVector4: ...
    @property
    def lqtt(self) -> FVector4: ...
    @property
    def lqtq(self) -> FVector4: ...
    @property
    def lqtp(self) -> FVector4: ...
    @property
    def lqto(self) -> FVector4: ...
    @property
    def lqtl(self) -> FVector4: ...
    @property
    def lqqs(self) -> FVector4: ...
    @property
    def lqqt(self) -> FVector4: ...
    @property
    def lqqq(self) -> FVector4: ...
    @property
    def lqqp(self) -> FVector4: ...
    @property
    def lqqo(self) -> FVector4: ...
    @property
    def lqql(self) -> FVector4: ...
    @property
    def lqps(self) -> FVector4: ...
    @property
    def lqpt(self) -> FVector4: ...
    @property
    def lqpq(self) -> FVector4: ...
    @property
    def lqpp(self) -> FVector4: ...
    @property
    def lqpo(self) -> FVector4: ...
    @property
    def lqpl(self) -> FVector4: ...
    @property
    def lqos(self) -> FVector4: ...
    @property
    def lqot(self) -> FVector4: ...
    @property
    def lqoq(self) -> FVector4: ...
    @property
    def lqop(self) -> FVector4: ...
    @property
    def lqoo(self) -> FVector4: ...
    @property
    def lqol(self) -> FVector4: ...
    @property
    def lqls(self) -> FVector4: ...
    @property
    def lqlt(self) -> FVector4: ...
    @property
    def lqlq(self) -> FVector4: ...
    @property
    def lqlp(self) -> FVector4: ...
    @property
    def lqlo(self) -> FVector4: ...
    @property
    def lqll(self) -> FVector4: ...
    @property
    def lpss(self) -> FVector4: ...
    @property
    def lpst(self) -> FVector4: ...
    @property
    def lpsq(self) -> FVector4: ...
    @property
    def lpsp(self) -> FVector4: ...
    @property
    def lpso(self) -> FVector4: ...
    @property
    def lpsl(self) -> FVector4: ...
    @property
    def lpts(self) -> FVector4: ...
    @property
    def lptt(self) -> FVector4: ...
    @property
    def lptq(self) -> FVector4: ...
    @property
    def lptp(self) -> FVector4: ...
    @property
    def lpto(self) -> FVector4: ...
    @property
    def lptl(self) -> FVector4: ...
    @property
    def lpqs(self) -> FVector4: ...
    @property
    def lpqt(self) -> FVector4: ...
    @property
    def lpqq(self) -> FVector4: ...
    @property
    def lpqp(self) -> FVector4: ...
    @property
    def lpqo(self) -> FVector4: ...
    @property
    def lpql(self) -> FVector4: ...
    @property
    def lpps(self) -> FVector4: ...
    @property
    def lppt(self) -> FVector4: ...
    @property
    def lppq(self) -> FVector4: ...
    @property
    def lppp(self) -> FVector4: ...
    @property
    def lppo(self) -> FVector4: ...
    @property
    def lppl(self) -> FVector4: ...
    @property
    def lpos(self) -> FVector4: ...
    @property
    def lpot(self) -> FVector4: ...
    @property
    def lpoq(self) -> FVector4: ...
    @property
    def lpop(self) -> FVector4: ...
    @property
    def lpoo(self) -> FVector4: ...
    @property
    def lpol(self) -> FVector4: ...
    @property
    def lpls(self) -> FVector4: ...
    @property
    def lplt(self) -> FVector4: ...
    @property
    def lplq(self) -> FVector4: ...
    @property
    def lplp(self) -> FVector4: ...
    @property
    def lplo(self) -> FVector4: ...
    @property
    def lpll(self) -> FVector4: ...
    @property
    def loss(self) -> FVector4: ...
    @property
    def lost(self) -> FVector4: ...
    @property
    def losq(self) -> FVector4: ...
    @property
    def losp(self) -> FVector4: ...
    @property
    def loso(self) -> FVector4: ...
    @property
    def losl(self) -> FVector4: ...
    @property
    def lots(self) -> FVector4: ...
    @property
    def lott(self) -> FVector4: ...
    @property
    def lotq(self) -> FVector4: ...
    @property
    def lotp(self) -> FVector4: ...
    @property
    def loto(self) -> FVector4: ...
    @property
    def lotl(self) -> FVector4: ...
    @property
    def loqs(self) -> FVector4: ...
    @property
    def loqt(self) -> FVector4: ...
    @property
    def loqq(self) -> FVector4: ...
    @property
    def loqp(self) -> FVector4: ...
    @property
    def loqo(self) -> FVector4: ...
    @property
    def loql(self) -> FVector4: ...
    @property
    def lops(self) -> FVector4: ...
    @property
    def lopt(self) -> FVector4: ...
    @property
    def lopq(self) -> FVector4: ...
    @property
    def lopp(self) -> FVector4: ...
    @property
    def lopo(self) -> FVector4: ...
    @property
    def lopl(self) -> FVector4: ...
    @property
    def loos(self) -> FVector4: ...
    @property
    def loot(self) -> FVector4: ...
    @property
    def looq(self) -> FVector4: ...
    @property
    def loop(self) -> FVector4: ...
    @property
    def lols(self) -> FVector4: ...
    @property
    def lolt(self) -> FVector4: ...
    @property
    def lolq(self) -> FVector4: ...
    @property
    def lolp(self) -> FVector4: ...
    @property
    def llss(self) -> FVector4: ...
    @property
    def llst(self) -> FVector4: ...
    @property
    def llsq(self) -> FVector4: ...
    @property
    def llsp(self) -> FVector4: ...
    @property
    def llso(self) -> FVector4: ...
    @property
    def llsl(self) -> FVector4: ...
    @property
    def llts(self) -> FVector4: ...
    @property
    def lltt(self) -> FVector4: ...
    @property
    def lltq(self) -> FVector4: ...
    @property
    def lltp(self) -> FVector4: ...
    @property
    def llto(self) -> FVector4: ...
    @property
    def lltl(self) -> FVector4: ...
    @property
    def llqs(self) -> FVector4: ...
    @property
    def llqt(self) -> FVector4: ...
    @property
    def llqq(self) -> FVector4: ...
    @property
    def llqp(self) -> FVector4: ...
    @property
    def llqo(self) -> FVector4: ...
    @property
    def llql(self) -> FVector4: ...
    @property
    def llps(self) -> FVector4: ...
    @property
    def llpt(self) -> FVector4: ...
    @property
    def llpq(self) -> FVector4: ...
    @property
    def llpp(self) -> FVector4: ...
    @property
    def llpo(self) -> FVector4: ...
    @property
    def llpl(self) -> FVector4: ...
    @property
    def llos(self) -> FVector4: ...
    @property
    def llot(self) -> FVector4: ...
    @property
    def lloq(self) -> FVector4: ...
    @property
    def llop(self) -> FVector4: ...
    @property
    def llls(self) -> FVector4: ...
    @property
    def lllt(self) -> FVector4: ...
    @property
    def lllq(self) -> FVector4: ...
    @property
    def lllp(self) -> FVector4: ...
    @property
    def u(self) -> float: ...
    @property
    def v(self) -> float: ...
    @property
    def uu(self) -> FVector2: ...
    @property
    def uv(self) -> FVector2: ...
    @property
    def uo(self) -> FVector2: ...
    @property
    def ul(self) -> FVector2: ...
    @property
    def vu(self) -> FVector2: ...
    @property
    def vv(self) -> FVector2: ...
    @property
    def vo(self) -> FVector2: ...
    @property
    def vl(self) -> FVector2: ...
    @property
    def ou(self) -> FVector2: ...
    @property
    def ov(self) -> FVector2: ...
    @property
    def lu(self) -> FVector2: ...
    @property
    def lv(self) -> FVector2: ...
    @property
    def uuu(self) -> FVector3: ...
    @property
    def uuv(self) -> FVector3: ...
    @property
    def uuo(self) -> FVector3: ...
    @property
    def uul(self) -> FVector3: ...
    @property
    def uvu(self) -> FVector3: ...
    @property
    def uvv(self) -> FVector3: ...
    @property
    def uvo(self) -> FVector3: ...
    @property
    def uvl(self) -> FVector3: ...
    @property
    def uou(self) -> FVector3: ...
    @property
    def uov(self) -> FVector3: ...
    @property
    def uoo(self) -> FVector3: ...
    @property
    def uol(self) -> FVector3: ...
    @property
    def ulu(self) -> FVector3: ...
    @property
    def ulv(self) -> FVector3: ...
    @property
    def ulo(self) -> FVector3: ...
    @property
    def ull(self) -> FVector3: ...
    @property
    def vuu(self) -> FVector3: ...
    @property
    def vuv(self) -> FVector3: ...
    @property
    def vuo(self) -> FVector3: ...
    @property
    def vul(self) -> FVector3: ...
    @property
    def vvu(self) -> FVector3: ...
    @property
    def vvv(self) -> FVector3: ...
    @property
    def vvo(self) -> FVector3: ...
    @property
    def vvl(self) -> FVector3: ...
    @property
    def vou(self) -> FVector3: ...
    @property
    def vov(self) -> FVector3: ...
    @property
    def voo(self) -> FVector3: ...
    @property
    def vol(self) -> FVector3: ...
    @property
    def vlu(self) -> FVector3: ...
    @property
    def vlv(self) -> FVector3: ...
    @property
    def vlo(self) -> FVector3: ...
    @property
    def vll(self) -> FVector3: ...
    @property
    def ouu(self) -> FVector3: ...
    @property
    def ouv(self) -> FVector3: ...
    @property
    def ouo(self) -> FVector3: ...
    @property
    def oul(self) -> FVector3: ...
    @property
    def ovu(self) -> FVector3: ...
    @property
    def ovv(self) -> FVector3: ...
    @property
    def ovo(self) -> FVector3: ...
    @property
    def ovl(self) -> FVector3: ...
    @property
    def oou(self) -> FVector3: ...
    @property
    def oov(self) -> FVector3: ...
    @property
    def olu(self) -> FVector3: ...
    @property
    def olv(self) -> FVector3: ...
    @property
    def luu(self) -> FVector3: ...
    @property
    def luv(self) -> FVector3: ...
    @property
    def luo(self) -> FVector3: ...
    @property
    def lul(self) -> FVector3: ...
    @property
    def lvu(self) -> FVector3: ...
    @property
    def lvv(self) -> FVector3: ...
    @property
    def lvo(self) -> FVector3: ...
    @property
    def lvl(self) -> FVector3: ...
    @property
    def lou(self) -> FVector3: ...
    @property
    def lov(self) -> FVector3: ...
    @property
    def llu(self) -> FVector3: ...
    @property
    def llv(self) -> FVector3: ...
    @property
    def uuuu(self) -> FVector4: ...
    @property
    def uuuv(self) -> FVector4: ...
    @property
    def uuuo(self) -> FVector4: ...
    @property
    def uuul(self) -> FVector4: ...
    @property
    def uuvu(self) -> FVector4: ...
    @property
    def uuvv(self) -> FVector4: ...
    @property
    def uuvo(self) -> FVector4: ...
    @property
    def uuvl(self) -> FVector4: ...
    @property
    def uuou(self) -> FVector4: ...
    @property
    def uuov(self) -> FVector4: ...
    @property
    def uuoo(self) -> FVector4: ...
    @property
    def uuol(self) -> FVector4: ...
    @property
    def uulu(self) -> FVector4: ...
    @property
    def uulv(self) -> FVector4: ...
    @property
    def uulo(self) -> FVector4: ...
    @property
    def uull(self) -> FVector4: ...
    @property
    def uvuu(self) -> FVector4: ...
    @property
    def uvuv(self) -> FVector4: ...
    @property
    def uvuo(self) -> FVector4: ...
    @property
    def uvul(self) -> FVector4: ...
    @property
    def uvvu(self) -> FVector4: ...
    @property
    def uvvv(self) -> FVector4: ...
    @property
    def uvvo(self) -> FVector4: ...
    @property
    def uvvl(self) -> FVector4: ...
    @property
    def uvou(self) -> FVector4: ...
    @property
    def uvov(self) -> FVector4: ...
    @property
    def uvoo(self) -> FVector4: ...
    @property
    def uvol(self) -> FVector4: ...
    @property
    def uvlu(self) -> FVector4: ...
    @property
    def uvlv(self) -> FVector4: ...
    @property
    def uvlo(self) -> FVector4: ...
    @property
    def uvll(self) -> FVector4: ...
    @property
    def uouu(self) -> FVector4: ...
    @property
    def uouv(self) -> FVector4: ...
    @property
    def uouo(self) -> FVector4: ...
    @property
    def uoul(self) -> FVector4: ...
    @property
    def uovu(self) -> FVector4: ...
    @property
    def uovv(self) -> FVector4: ...
    @property
    def uovo(self) -> FVector4: ...
    @property
    def uovl(self) -> FVector4: ...
    @property
    def uoou(self) -> FVector4: ...
    @property
    def uoov(self) -> FVector4: ...
    @property
    def uooo(self) -> FVector4: ...
    @property
    def uool(self) -> FVector4: ...
    @property
    def uolu(self) -> FVector4: ...
    @property
    def uolv(self) -> FVector4: ...
    @property
    def uolo(self) -> FVector4: ...
    @property
    def uoll(self) -> FVector4: ...
    @property
    def uluu(self) -> FVector4: ...
    @property
    def uluv(self) -> FVector4: ...
    @property
    def uluo(self) -> FVector4: ...
    @property
    def ulul(self) -> FVector4: ...
    @property
    def ulvu(self) -> FVector4: ...
    @property
    def ulvv(self) -> FVector4: ...
    @property
    def ulvo(self) -> FVector4: ...
    @property
    def ulvl(self) -> FVector4: ...
    @property
    def ulou(self) -> FVector4: ...
    @property
    def ulov(self) -> FVector4: ...
    @property
    def uloo(self) -> FVector4: ...
    @property
    def ulol(self) -> FVector4: ...
    @property
    def ullu(self) -> FVector4: ...
    @property
    def ullv(self) -> FVector4: ...
    @property
    def ullo(self) -> FVector4: ...
    @property
    def ulll(self) -> FVector4: ...
    @property
    def vuuu(self) -> FVector4: ...
    @property
    def vuuv(self) -> FVector4: ...
    @property
    def vuuo(self) -> FVector4: ...
    @property
    def vuul(self) -> FVector4: ...
    @property
    def vuvu(self) -> FVector4: ...
    @property
    def vuvv(self) -> FVector4: ...
    @property
    def vuvo(self) -> FVector4: ...
    @property
    def vuvl(self) -> FVector4: ...
    @property
    def vuou(self) -> FVector4: ...
    @property
    def vuov(self) -> FVector4: ...
    @property
    def vuoo(self) -> FVector4: ...
    @property
    def vuol(self) -> FVector4: ...
    @property
    def vulu(self) -> FVector4: ...
    @property
    def vulv(self) -> FVector4: ...
    @property
    def vulo(self) -> FVector4: ...
    @property
    def vull(self) -> FVector4: ...
    @property
    def vvuu(self) -> FVector4: ...
    @property
    def vvuv(self) -> FVector4: ...
    @property
    def vvuo(self) -> FVector4: ...
    @property
    def vvul(self) -> FVector4: ...
    @property
    def vvvu(self) -> FVector4: ...
    @property
    def vvvv(self) -> FVector4: ...
    @property
    def vvvo(self) -> FVector4: ...
    @property
    def vvvl(self) -> FVector4: ...
    @property
    def vvou(self) -> FVector4: ...
    @property
    def vvov(self) -> FVector4: ...
    @property
    def vvoo(self) -> FVector4: ...
    @property
    def vvol(self) -> FVector4: ...
    @property
    def vvlu(self) -> FVector4: ...
    @property
    def vvlv(self) -> FVector4: ...
    @property
    def vvlo(self) -> FVector4: ...
    @property
    def vvll(self) -> FVector4: ...
    @property
    def vouu(self) -> FVector4: ...
    @property
    def vouv(self) -> FVector4: ...
    @property
    def vouo(self) -> FVector4: ...
    @property
    def voul(self) -> FVector4: ...
    @property
    def vovu(self) -> FVector4: ...
    @property
    def vovv(self) -> FVector4: ...
    @property
    def vovo(self) -> FVector4: ...
    @property
    def vovl(self) -> FVector4: ...
    @property
    def voou(self) -> FVector4: ...
    @property
    def voov(self) -> FVector4: ...
    @property
    def vooo(self) -> FVector4: ...
    @property
    def vool(self) -> FVector4: ...
    @property
    def volu(self) -> FVector4: ...
    @property
    def volv(self) -> FVector4: ...
    @property
    def volo(self) -> FVector4: ...
    @property
    def voll(self) -> FVector4: ...
    @property
    def vluu(self) -> FVector4: ...
    @property
    def vluv(self) -> FVector4: ...
    @property
    def vluo(self) -> FVector4: ...
    @property
    def vlul(self) -> FVector4: ...
    @property
    def vlvu(self) -> FVector4: ...
    @property
    def vlvv(self) -> FVector4: ...
    @property
    def vlvo(self) -> FVector4: ...
    @property
    def vlvl(self) -> FVector4: ...
    @property
    def vlou(self) -> FVector4: ...
    @property
    def vlov(self) -> FVector4: ...
    @property
    def vloo(self) -> FVector4: ...
    @property
    def vlol(self) -> FVector4: ...
    @property
    def vllu(self) -> FVector4: ...
    @property
    def vllv(self) -> FVector4: ...
    @property
    def vllo(self) -> FVector4: ...
    @property
    def vlll(self) -> FVector4: ...
    @property
    def ouuu(self) -> FVector4: ...
    @property
    def ouuv(self) -> FVector4: ...
    @property
    def ouuo(self) -> FVector4: ...
    @property
    def ouul(self) -> FVector4: ...
    @property
    def ouvu(self) -> FVector4: ...
    @property
    def ouvv(self) -> FVector4: ...
    @property
    def ouvo(self) -> FVector4: ...
    @property
    def ouvl(self) -> FVector4: ...
    @property
    def ouou(self) -> FVector4: ...
    @property
    def ouov(self) -> FVector4: ...
    @property
    def ouoo(self) -> FVector4: ...
    @property
    def ouol(self) -> FVector4: ...
    @property
    def oulu(self) -> FVector4: ...
    @property
    def oulv(self) -> FVector4: ...
    @property
    def oulo(self) -> FVector4: ...
    @property
    def oull(self) -> FVector4: ...
    @property
    def ovuu(self) -> FVector4: ...
    @property
    def ovuv(self) -> FVector4: ...
    @property
    def ovuo(self) -> FVector4: ...
    @property
    def ovul(self) -> FVector4: ...
    @property
    def ovvu(self) -> FVector4: ...
    @property
    def ovvv(self) -> FVector4: ...
    @property
    def ovvo(self) -> FVector4: ...
    @property
    def ovvl(self) -> FVector4: ...
    @property
    def ovou(self) -> FVector4: ...
    @property
    def ovov(self) -> FVector4: ...
    @property
    def ovoo(self) -> FVector4: ...
    @property
    def ovol(self) -> FVector4: ...
    @property
    def ovlu(self) -> FVector4: ...
    @property
    def ovlv(self) -> FVector4: ...
    @property
    def ovlo(self) -> FVector4: ...
    @property
    def ovll(self) -> FVector4: ...
    @property
    def oouu(self) -> FVector4: ...
    @property
    def oouv(self) -> FVector4: ...
    @property
    def oouo(self) -> FVector4: ...
    @property
    def ooul(self) -> FVector4: ...
    @property
    def oovu(self) -> FVector4: ...
    @property
    def oovv(self) -> FVector4: ...
    @property
    def oovo(self) -> FVector4: ...
    @property
    def oovl(self) -> FVector4: ...
    @property
    def ooou(self) -> FVector4: ...
    @property
    def ooov(self) -> FVector4: ...
    @property
    def oolu(self) -> FVector4: ...
    @property
    def oolv(self) -> FVector4: ...
    @property
    def oluu(self) -> FVector4: ...
    @property
    def oluv(self) -> FVector4: ...
    @property
    def oluo(self) -> FVector4: ...
    @property
    def olul(self) -> FVector4: ...
    @property
    def olvu(self) -> FVector4: ...
    @property
    def olvv(self) -> FVector4: ...
    @property
    def olvo(self) -> FVector4: ...
    @property
    def olvl(self) -> FVector4: ...
    @property
    def olou(self) -> FVector4: ...
    @property
    def olov(self) -> FVector4: ...
    @property
    def ollu(self) -> FVector4: ...
    @property
    def ollv(self) -> FVector4: ...
    @property
    def luuu(self) -> FVector4: ...
    @property
    def luuv(self) -> FVector4: ...
    @property
    def luuo(self) -> FVector4: ...
    @property
    def luul(self) -> FVector4: ...
    @property
    def luvu(self) -> FVector4: ...
    @property
    def luvv(self) -> FVector4: ...
    @property
    def luvo(self) -> FVector4: ...
    @property
    def luvl(self) -> FVector4: ...
    @property
    def luou(self) -> FVector4: ...
    @property
    def luov(self) -> FVector4: ...
    @property
    def luoo(self) -> FVector4: ...
    @property
    def luol(self) -> FVector4: ...
    @property
    def lulu(self) -> FVector4: ...
    @property
    def lulv(self) -> FVector4: ...
    @property
    def lulo(self) -> FVector4: ...
    @property
    def lull(self) -> FVector4: ...
    @property
    def lvuu(self) -> FVector4: ...
    @property
    def lvuv(self) -> FVector4: ...
    @property
    def lvuo(self) -> FVector4: ...
    @property
    def lvul(self) -> FVector4: ...
    @property
    def lvvu(self) -> FVector4: ...
    @property
    def lvvv(self) -> FVector4: ...
    @property
    def lvvo(self) -> FVector4: ...
    @property
    def lvvl(self) -> FVector4: ...
    @property
    def lvou(self) -> FVector4: ...
    @property
    def lvov(self) -> FVector4: ...
    @property
    def lvoo(self) -> FVector4: ...
    @property
    def lvol(self) -> FVector4: ...
    @property
    def lvlu(self) -> FVector4: ...
    @property
    def lvlv(self) -> FVector4: ...
    @property
    def lvlo(self) -> FVector4: ...
    @property
    def lvll(self) -> FVector4: ...
    @property
    def louu(self) -> FVector4: ...
    @property
    def louv(self) -> FVector4: ...
    @property
    def louo(self) -> FVector4: ...
    @property
    def loul(self) -> FVector4: ...
    @property
    def lovu(self) -> FVector4: ...
    @property
    def lovv(self) -> FVector4: ...
    @property
    def lovo(self) -> FVector4: ...
    @property
    def lovl(self) -> FVector4: ...
    @property
    def loou(self) -> FVector4: ...
    @property
    def loov(self) -> FVector4: ...
    @property
    def lolu(self) -> FVector4: ...
    @property
    def lolv(self) -> FVector4: ...
    @property
    def lluu(self) -> FVector4: ...
    @property
    def lluv(self) -> FVector4: ...
    @property
    def lluo(self) -> FVector4: ...
    @property
    def llul(self) -> FVector4: ...
    @property
    def llvu(self) -> FVector4: ...
    @property
    def llvv(self) -> FVector4: ...
    @property
    def llvo(self) -> FVector4: ...
    @property
    def llvl(self) -> FVector4: ...
    @property
    def llou(self) -> FVector4: ...
    @property
    def llov(self) -> FVector4: ...
    @property
    def lllu(self) -> FVector4: ...
    @property
    def lllv(self) -> FVector4: ...
    @property
    def oo(self) -> FVector2: ...
    @property
    def ol(self) -> FVector2: ...
    @property
    def lo(self) -> FVector2: ...
    @property
    def ll(self) -> FVector2: ...
    @property
    def ooo(self) -> FVector3: ...
    @property
    def ool(self) -> FVector3: ...
    @property
    def olo(self) -> FVector3: ...
    @property
    def oll(self) -> FVector3: ...
    @property
    def loo(self) -> FVector3: ...
    @property
    def lol(self) -> FVector3: ...
    @property
    def llo(self) -> FVector3: ...
    @property
    def lll(self) -> FVector3: ...
    @property
    def oooo(self) -> FVector4: ...
    @property
    def oool(self) -> FVector4: ...
    @property
    def oolo(self) -> FVector4: ...
    @property
    def ooll(self) -> FVector4: ...
    @property
    def oloo(self) -> FVector4: ...
    @property
    def olol(self) -> FVector4: ...
    @property
    def ollo(self) -> FVector4: ...
    @property
    def olll(self) -> FVector4: ...
    @property
    def looo(self) -> FVector4: ...
    @property
    def lool(self) -> FVector4: ...
    @property
    def lolo(self) -> FVector4: ...
    @property
    def loll(self) -> FVector4: ...
    @property
    def lloo(self) -> FVector4: ...
    @property
    def llol(self) -> FVector4: ...
    @property
    def lllo(self) -> FVector4: ...
    @property
    def llll(self) -> FVector4: ...
    @property
    def magnitude(self) -> float: ...
    def normalize(self) -> FVector4: ...
    def distance(self, other: FVector4, /) -> float: ...
    def lerp(self, other: FVector4, x: float, /) -> FVector4: ...
    def min(self, n: Number, /) -> FVector4: ...
    def max(self, n: Number, /) -> FVector4: ...
    def clamp(self, min: Number, max: Number, /) -> FVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[FVector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector4: ...

@final
class FVector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: FVector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector4]: ...
    @overload
    def __getitem__(self, index: int) -> FVector4: ...
    @overload
    def __getitem__(self, index: slice) -> FVector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> FVector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[FVector4]: ...

@final
class I8Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __add__(self, other: Number) -> I8Vector4: ...
    @overload
    def __radd__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __radd__(self, other: Number) -> I8Vector4: ...
    @overload
    def __sub__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __sub__(self, other: Number) -> I8Vector4: ...
    @overload
    def __rsub__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> I8Vector4: ...
    @overload
    def __mul__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __mul__(self, other: Number) -> I8Vector4: ...
    @overload
    def __rmul__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> I8Vector4: ...
    @overload
    def __truediv__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> I8Vector4: ...
    @overload
    def __rtruediv__(self, other: I8Vector4) -> I8Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> I8Vector4: ...
    def __neg__(self) -> I8Vector4: ...
    def __abs__(self) -> I8Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> I8Vector2: ...
    @property
    def xy(self) -> I8Vector2: ...
    @property
    def xz(self) -> I8Vector2: ...
    @property
    def xw(self) -> I8Vector2: ...
    @property
    def xo(self) -> I8Vector2: ...
    @property
    def xl(self) -> I8Vector2: ...
    @property
    def yx(self) -> I8Vector2: ...
    @property
    def yy(self) -> I8Vector2: ...
    @property
    def yz(self) -> I8Vector2: ...
    @property
    def yw(self) -> I8Vector2: ...
    @property
    def yo(self) -> I8Vector2: ...
    @property
    def yl(self) -> I8Vector2: ...
    @property
    def zx(self) -> I8Vector2: ...
    @property
    def zy(self) -> I8Vector2: ...
    @property
    def zz(self) -> I8Vector2: ...
    @property
    def zw(self) -> I8Vector2: ...
    @property
    def zo(self) -> I8Vector2: ...
    @property
    def zl(self) -> I8Vector2: ...
    @property
    def wx(self) -> I8Vector2: ...
    @property
    def wy(self) -> I8Vector2: ...
    @property
    def wz(self) -> I8Vector2: ...
    @property
    def ww(self) -> I8Vector2: ...
    @property
    def wo(self) -> I8Vector2: ...
    @property
    def wl(self) -> I8Vector2: ...
    @property
    def ox(self) -> I8Vector2: ...
    @property
    def oy(self) -> I8Vector2: ...
    @property
    def oz(self) -> I8Vector2: ...
    @property
    def ow(self) -> I8Vector2: ...
    @property
    def lx(self) -> I8Vector2: ...
    @property
    def ly(self) -> I8Vector2: ...
    @property
    def lz(self) -> I8Vector2: ...
    @property
    def lw(self) -> I8Vector2: ...
    @property
    def xxx(self) -> I8Vector3: ...
    @property
    def xxy(self) -> I8Vector3: ...
    @property
    def xxz(self) -> I8Vector3: ...
    @property
    def xxw(self) -> I8Vector3: ...
    @property
    def xxo(self) -> I8Vector3: ...
    @property
    def xxl(self) -> I8Vector3: ...
    @property
    def xyx(self) -> I8Vector3: ...
    @property
    def xyy(self) -> I8Vector3: ...
    @property
    def xyz(self) -> I8Vector3: ...
    @property
    def xyw(self) -> I8Vector3: ...
    @property
    def xyo(self) -> I8Vector3: ...
    @property
    def xyl(self) -> I8Vector3: ...
    @property
    def xzx(self) -> I8Vector3: ...
    @property
    def xzy(self) -> I8Vector3: ...
    @property
    def xzz(self) -> I8Vector3: ...
    @property
    def xzw(self) -> I8Vector3: ...
    @property
    def xzo(self) -> I8Vector3: ...
    @property
    def xzl(self) -> I8Vector3: ...
    @property
    def xwx(self) -> I8Vector3: ...
    @property
    def xwy(self) -> I8Vector3: ...
    @property
    def xwz(self) -> I8Vector3: ...
    @property
    def xww(self) -> I8Vector3: ...
    @property
    def xwo(self) -> I8Vector3: ...
    @property
    def xwl(self) -> I8Vector3: ...
    @property
    def xox(self) -> I8Vector3: ...
    @property
    def xoy(self) -> I8Vector3: ...
    @property
    def xoz(self) -> I8Vector3: ...
    @property
    def xow(self) -> I8Vector3: ...
    @property
    def xoo(self) -> I8Vector3: ...
    @property
    def xol(self) -> I8Vector3: ...
    @property
    def xlx(self) -> I8Vector3: ...
    @property
    def xly(self) -> I8Vector3: ...
    @property
    def xlz(self) -> I8Vector3: ...
    @property
    def xlw(self) -> I8Vector3: ...
    @property
    def xlo(self) -> I8Vector3: ...
    @property
    def xll(self) -> I8Vector3: ...
    @property
    def yxx(self) -> I8Vector3: ...
    @property
    def yxy(self) -> I8Vector3: ...
    @property
    def yxz(self) -> I8Vector3: ...
    @property
    def yxw(self) -> I8Vector3: ...
    @property
    def yxo(self) -> I8Vector3: ...
    @property
    def yxl(self) -> I8Vector3: ...
    @property
    def yyx(self) -> I8Vector3: ...
    @property
    def yyy(self) -> I8Vector3: ...
    @property
    def yyz(self) -> I8Vector3: ...
    @property
    def yyw(self) -> I8Vector3: ...
    @property
    def yyo(self) -> I8Vector3: ...
    @property
    def yyl(self) -> I8Vector3: ...
    @property
    def yzx(self) -> I8Vector3: ...
    @property
    def yzy(self) -> I8Vector3: ...
    @property
    def yzz(self) -> I8Vector3: ...
    @property
    def yzw(self) -> I8Vector3: ...
    @property
    def yzo(self) -> I8Vector3: ...
    @property
    def yzl(self) -> I8Vector3: ...
    @property
    def ywx(self) -> I8Vector3: ...
    @property
    def ywy(self) -> I8Vector3: ...
    @property
    def ywz(self) -> I8Vector3: ...
    @property
    def yww(self) -> I8Vector3: ...
    @property
    def ywo(self) -> I8Vector3: ...
    @property
    def ywl(self) -> I8Vector3: ...
    @property
    def yox(self) -> I8Vector3: ...
    @property
    def yoy(self) -> I8Vector3: ...
    @property
    def yoz(self) -> I8Vector3: ...
    @property
    def yow(self) -> I8Vector3: ...
    @property
    def yoo(self) -> I8Vector3: ...
    @property
    def yol(self) -> I8Vector3: ...
    @property
    def ylx(self) -> I8Vector3: ...
    @property
    def yly(self) -> I8Vector3: ...
    @property
    def ylz(self) -> I8Vector3: ...
    @property
    def ylw(self) -> I8Vector3: ...
    @property
    def ylo(self) -> I8Vector3: ...
    @property
    def yll(self) -> I8Vector3: ...
    @property
    def zxx(self) -> I8Vector3: ...
    @property
    def zxy(self) -> I8Vector3: ...
    @property
    def zxz(self) -> I8Vector3: ...
    @property
    def zxw(self) -> I8Vector3: ...
    @property
    def zxo(self) -> I8Vector3: ...
    @property
    def zxl(self) -> I8Vector3: ...
    @property
    def zyx(self) -> I8Vector3: ...
    @property
    def zyy(self) -> I8Vector3: ...
    @property
    def zyz(self) -> I8Vector3: ...
    @property
    def zyw(self) -> I8Vector3: ...
    @property
    def zyo(self) -> I8Vector3: ...
    @property
    def zyl(self) -> I8Vector3: ...
    @property
    def zzx(self) -> I8Vector3: ...
    @property
    def zzy(self) -> I8Vector3: ...
    @property
    def zzz(self) -> I8Vector3: ...
    @property
    def zzw(self) -> I8Vector3: ...
    @property
    def zzo(self) -> I8Vector3: ...
    @property
    def zzl(self) -> I8Vector3: ...
    @property
    def zwx(self) -> I8Vector3: ...
    @property
    def zwy(self) -> I8Vector3: ...
    @property
    def zwz(self) -> I8Vector3: ...
    @property
    def zww(self) -> I8Vector3: ...
    @property
    def zwo(self) -> I8Vector3: ...
    @property
    def zwl(self) -> I8Vector3: ...
    @property
    def zox(self) -> I8Vector3: ...
    @property
    def zoy(self) -> I8Vector3: ...
    @property
    def zoz(self) -> I8Vector3: ...
    @property
    def zow(self) -> I8Vector3: ...
    @property
    def zoo(self) -> I8Vector3: ...
    @property
    def zol(self) -> I8Vector3: ...
    @property
    def zlx(self) -> I8Vector3: ...
    @property
    def zly(self) -> I8Vector3: ...
    @property
    def zlz(self) -> I8Vector3: ...
    @property
    def zlw(self) -> I8Vector3: ...
    @property
    def zlo(self) -> I8Vector3: ...
    @property
    def zll(self) -> I8Vector3: ...
    @property
    def wxx(self) -> I8Vector3: ...
    @property
    def wxy(self) -> I8Vector3: ...
    @property
    def wxz(self) -> I8Vector3: ...
    @property
    def wxw(self) -> I8Vector3: ...
    @property
    def wxo(self) -> I8Vector3: ...
    @property
    def wxl(self) -> I8Vector3: ...
    @property
    def wyx(self) -> I8Vector3: ...
    @property
    def wyy(self) -> I8Vector3: ...
    @property
    def wyz(self) -> I8Vector3: ...
    @property
    def wyw(self) -> I8Vector3: ...
    @property
    def wyo(self) -> I8Vector3: ...
    @property
    def wyl(self) -> I8Vector3: ...
    @property
    def wzx(self) -> I8Vector3: ...
    @property
    def wzy(self) -> I8Vector3: ...
    @property
    def wzz(self) -> I8Vector3: ...
    @property
    def wzw(self) -> I8Vector3: ...
    @property
    def wzo(self) -> I8Vector3: ...
    @property
    def wzl(self) -> I8Vector3: ...
    @property
    def wwx(self) -> I8Vector3: ...
    @property
    def wwy(self) -> I8Vector3: ...
    @property
    def wwz(self) -> I8Vector3: ...
    @property
    def www(self) -> I8Vector3: ...
    @property
    def wwo(self) -> I8Vector3: ...
    @property
    def wwl(self) -> I8Vector3: ...
    @property
    def wox(self) -> I8Vector3: ...
    @property
    def woy(self) -> I8Vector3: ...
    @property
    def woz(self) -> I8Vector3: ...
    @property
    def wow(self) -> I8Vector3: ...
    @property
    def woo(self) -> I8Vector3: ...
    @property
    def wol(self) -> I8Vector3: ...
    @property
    def wlx(self) -> I8Vector3: ...
    @property
    def wly(self) -> I8Vector3: ...
    @property
    def wlz(self) -> I8Vector3: ...
    @property
    def wlw(self) -> I8Vector3: ...
    @property
    def wlo(self) -> I8Vector3: ...
    @property
    def wll(self) -> I8Vector3: ...
    @property
    def oxx(self) -> I8Vector3: ...
    @property
    def oxy(self) -> I8Vector3: ...
    @property
    def oxz(self) -> I8Vector3: ...
    @property
    def oxw(self) -> I8Vector3: ...
    @property
    def oxo(self) -> I8Vector3: ...
    @property
    def oxl(self) -> I8Vector3: ...
    @property
    def oyx(self) -> I8Vector3: ...
    @property
    def oyy(self) -> I8Vector3: ...
    @property
    def oyz(self) -> I8Vector3: ...
    @property
    def oyw(self) -> I8Vector3: ...
    @property
    def oyo(self) -> I8Vector3: ...
    @property
    def oyl(self) -> I8Vector3: ...
    @property
    def ozx(self) -> I8Vector3: ...
    @property
    def ozy(self) -> I8Vector3: ...
    @property
    def ozz(self) -> I8Vector3: ...
    @property
    def ozw(self) -> I8Vector3: ...
    @property
    def ozo(self) -> I8Vector3: ...
    @property
    def ozl(self) -> I8Vector3: ...
    @property
    def owx(self) -> I8Vector3: ...
    @property
    def owy(self) -> I8Vector3: ...
    @property
    def owz(self) -> I8Vector3: ...
    @property
    def oww(self) -> I8Vector3: ...
    @property
    def owo(self) -> I8Vector3: ...
    @property
    def owl(self) -> I8Vector3: ...
    @property
    def oox(self) -> I8Vector3: ...
    @property
    def ooy(self) -> I8Vector3: ...
    @property
    def ooz(self) -> I8Vector3: ...
    @property
    def oow(self) -> I8Vector3: ...
    @property
    def olx(self) -> I8Vector3: ...
    @property
    def oly(self) -> I8Vector3: ...
    @property
    def olz(self) -> I8Vector3: ...
    @property
    def olw(self) -> I8Vector3: ...
    @property
    def lxx(self) -> I8Vector3: ...
    @property
    def lxy(self) -> I8Vector3: ...
    @property
    def lxz(self) -> I8Vector3: ...
    @property
    def lxw(self) -> I8Vector3: ...
    @property
    def lxo(self) -> I8Vector3: ...
    @property
    def lxl(self) -> I8Vector3: ...
    @property
    def lyx(self) -> I8Vector3: ...
    @property
    def lyy(self) -> I8Vector3: ...
    @property
    def lyz(self) -> I8Vector3: ...
    @property
    def lyw(self) -> I8Vector3: ...
    @property
    def lyo(self) -> I8Vector3: ...
    @property
    def lyl(self) -> I8Vector3: ...
    @property
    def lzx(self) -> I8Vector3: ...
    @property
    def lzy(self) -> I8Vector3: ...
    @property
    def lzz(self) -> I8Vector3: ...
    @property
    def lzw(self) -> I8Vector3: ...
    @property
    def lzo(self) -> I8Vector3: ...
    @property
    def lzl(self) -> I8Vector3: ...
    @property
    def lwx(self) -> I8Vector3: ...
    @property
    def lwy(self) -> I8Vector3: ...
    @property
    def lwz(self) -> I8Vector3: ...
    @property
    def lww(self) -> I8Vector3: ...
    @property
    def lwo(self) -> I8Vector3: ...
    @property
    def lwl(self) -> I8Vector3: ...
    @property
    def lox(self) -> I8Vector3: ...
    @property
    def loy(self) -> I8Vector3: ...
    @property
    def loz(self) -> I8Vector3: ...
    @property
    def low(self) -> I8Vector3: ...
    @property
    def llx(self) -> I8Vector3: ...
    @property
    def lly(self) -> I8Vector3: ...
    @property
    def llz(self) -> I8Vector3: ...
    @property
    def llw(self) -> I8Vector3: ...
    @property
    def xxxx(self) -> I8Vector4: ...
    @property
    def xxxy(self) -> I8Vector4: ...
    @property
    def xxxz(self) -> I8Vector4: ...
    @property
    def xxxw(self) -> I8Vector4: ...
    @property
    def xxxo(self) -> I8Vector4: ...
    @property
    def xxxl(self) -> I8Vector4: ...
    @property
    def xxyx(self) -> I8Vector4: ...
    @property
    def xxyy(self) -> I8Vector4: ...
    @property
    def xxyz(self) -> I8Vector4: ...
    @property
    def xxyw(self) -> I8Vector4: ...
    @property
    def xxyo(self) -> I8Vector4: ...
    @property
    def xxyl(self) -> I8Vector4: ...
    @property
    def xxzx(self) -> I8Vector4: ...
    @property
    def xxzy(self) -> I8Vector4: ...
    @property
    def xxzz(self) -> I8Vector4: ...
    @property
    def xxzw(self) -> I8Vector4: ...
    @property
    def xxzo(self) -> I8Vector4: ...
    @property
    def xxzl(self) -> I8Vector4: ...
    @property
    def xxwx(self) -> I8Vector4: ...
    @property
    def xxwy(self) -> I8Vector4: ...
    @property
    def xxwz(self) -> I8Vector4: ...
    @property
    def xxww(self) -> I8Vector4: ...
    @property
    def xxwo(self) -> I8Vector4: ...
    @property
    def xxwl(self) -> I8Vector4: ...
    @property
    def xxox(self) -> I8Vector4: ...
    @property
    def xxoy(self) -> I8Vector4: ...
    @property
    def xxoz(self) -> I8Vector4: ...
    @property
    def xxow(self) -> I8Vector4: ...
    @property
    def xxoo(self) -> I8Vector4: ...
    @property
    def xxol(self) -> I8Vector4: ...
    @property
    def xxlx(self) -> I8Vector4: ...
    @property
    def xxly(self) -> I8Vector4: ...
    @property
    def xxlz(self) -> I8Vector4: ...
    @property
    def xxlw(self) -> I8Vector4: ...
    @property
    def xxlo(self) -> I8Vector4: ...
    @property
    def xxll(self) -> I8Vector4: ...
    @property
    def xyxx(self) -> I8Vector4: ...
    @property
    def xyxy(self) -> I8Vector4: ...
    @property
    def xyxz(self) -> I8Vector4: ...
    @property
    def xyxw(self) -> I8Vector4: ...
    @property
    def xyxo(self) -> I8Vector4: ...
    @property
    def xyxl(self) -> I8Vector4: ...
    @property
    def xyyx(self) -> I8Vector4: ...
    @property
    def xyyy(self) -> I8Vector4: ...
    @property
    def xyyz(self) -> I8Vector4: ...
    @property
    def xyyw(self) -> I8Vector4: ...
    @property
    def xyyo(self) -> I8Vector4: ...
    @property
    def xyyl(self) -> I8Vector4: ...
    @property
    def xyzx(self) -> I8Vector4: ...
    @property
    def xyzy(self) -> I8Vector4: ...
    @property
    def xyzz(self) -> I8Vector4: ...
    @property
    def xyzw(self) -> I8Vector4: ...
    @property
    def xyzo(self) -> I8Vector4: ...
    @property
    def xyzl(self) -> I8Vector4: ...
    @property
    def xywx(self) -> I8Vector4: ...
    @property
    def xywy(self) -> I8Vector4: ...
    @property
    def xywz(self) -> I8Vector4: ...
    @property
    def xyww(self) -> I8Vector4: ...
    @property
    def xywo(self) -> I8Vector4: ...
    @property
    def xywl(self) -> I8Vector4: ...
    @property
    def xyox(self) -> I8Vector4: ...
    @property
    def xyoy(self) -> I8Vector4: ...
    @property
    def xyoz(self) -> I8Vector4: ...
    @property
    def xyow(self) -> I8Vector4: ...
    @property
    def xyoo(self) -> I8Vector4: ...
    @property
    def xyol(self) -> I8Vector4: ...
    @property
    def xylx(self) -> I8Vector4: ...
    @property
    def xyly(self) -> I8Vector4: ...
    @property
    def xylz(self) -> I8Vector4: ...
    @property
    def xylw(self) -> I8Vector4: ...
    @property
    def xylo(self) -> I8Vector4: ...
    @property
    def xyll(self) -> I8Vector4: ...
    @property
    def xzxx(self) -> I8Vector4: ...
    @property
    def xzxy(self) -> I8Vector4: ...
    @property
    def xzxz(self) -> I8Vector4: ...
    @property
    def xzxw(self) -> I8Vector4: ...
    @property
    def xzxo(self) -> I8Vector4: ...
    @property
    def xzxl(self) -> I8Vector4: ...
    @property
    def xzyx(self) -> I8Vector4: ...
    @property
    def xzyy(self) -> I8Vector4: ...
    @property
    def xzyz(self) -> I8Vector4: ...
    @property
    def xzyw(self) -> I8Vector4: ...
    @property
    def xzyo(self) -> I8Vector4: ...
    @property
    def xzyl(self) -> I8Vector4: ...
    @property
    def xzzx(self) -> I8Vector4: ...
    @property
    def xzzy(self) -> I8Vector4: ...
    @property
    def xzzz(self) -> I8Vector4: ...
    @property
    def xzzw(self) -> I8Vector4: ...
    @property
    def xzzo(self) -> I8Vector4: ...
    @property
    def xzzl(self) -> I8Vector4: ...
    @property
    def xzwx(self) -> I8Vector4: ...
    @property
    def xzwy(self) -> I8Vector4: ...
    @property
    def xzwz(self) -> I8Vector4: ...
    @property
    def xzww(self) -> I8Vector4: ...
    @property
    def xzwo(self) -> I8Vector4: ...
    @property
    def xzwl(self) -> I8Vector4: ...
    @property
    def xzox(self) -> I8Vector4: ...
    @property
    def xzoy(self) -> I8Vector4: ...
    @property
    def xzoz(self) -> I8Vector4: ...
    @property
    def xzow(self) -> I8Vector4: ...
    @property
    def xzoo(self) -> I8Vector4: ...
    @property
    def xzol(self) -> I8Vector4: ...
    @property
    def xzlx(self) -> I8Vector4: ...
    @property
    def xzly(self) -> I8Vector4: ...
    @property
    def xzlz(self) -> I8Vector4: ...
    @property
    def xzlw(self) -> I8Vector4: ...
    @property
    def xzlo(self) -> I8Vector4: ...
    @property
    def xzll(self) -> I8Vector4: ...
    @property
    def xwxx(self) -> I8Vector4: ...
    @property
    def xwxy(self) -> I8Vector4: ...
    @property
    def xwxz(self) -> I8Vector4: ...
    @property
    def xwxw(self) -> I8Vector4: ...
    @property
    def xwxo(self) -> I8Vector4: ...
    @property
    def xwxl(self) -> I8Vector4: ...
    @property
    def xwyx(self) -> I8Vector4: ...
    @property
    def xwyy(self) -> I8Vector4: ...
    @property
    def xwyz(self) -> I8Vector4: ...
    @property
    def xwyw(self) -> I8Vector4: ...
    @property
    def xwyo(self) -> I8Vector4: ...
    @property
    def xwyl(self) -> I8Vector4: ...
    @property
    def xwzx(self) -> I8Vector4: ...
    @property
    def xwzy(self) -> I8Vector4: ...
    @property
    def xwzz(self) -> I8Vector4: ...
    @property
    def xwzw(self) -> I8Vector4: ...
    @property
    def xwzo(self) -> I8Vector4: ...
    @property
    def xwzl(self) -> I8Vector4: ...
    @property
    def xwwx(self) -> I8Vector4: ...
    @property
    def xwwy(self) -> I8Vector4: ...
    @property
    def xwwz(self) -> I8Vector4: ...
    @property
    def xwww(self) -> I8Vector4: ...
    @property
    def xwwo(self) -> I8Vector4: ...
    @property
    def xwwl(self) -> I8Vector4: ...
    @property
    def xwox(self) -> I8Vector4: ...
    @property
    def xwoy(self) -> I8Vector4: ...
    @property
    def xwoz(self) -> I8Vector4: ...
    @property
    def xwow(self) -> I8Vector4: ...
    @property
    def xwoo(self) -> I8Vector4: ...
    @property
    def xwol(self) -> I8Vector4: ...
    @property
    def xwlx(self) -> I8Vector4: ...
    @property
    def xwly(self) -> I8Vector4: ...
    @property
    def xwlz(self) -> I8Vector4: ...
    @property
    def xwlw(self) -> I8Vector4: ...
    @property
    def xwlo(self) -> I8Vector4: ...
    @property
    def xwll(self) -> I8Vector4: ...
    @property
    def xoxx(self) -> I8Vector4: ...
    @property
    def xoxy(self) -> I8Vector4: ...
    @property
    def xoxz(self) -> I8Vector4: ...
    @property
    def xoxw(self) -> I8Vector4: ...
    @property
    def xoxo(self) -> I8Vector4: ...
    @property
    def xoxl(self) -> I8Vector4: ...
    @property
    def xoyx(self) -> I8Vector4: ...
    @property
    def xoyy(self) -> I8Vector4: ...
    @property
    def xoyz(self) -> I8Vector4: ...
    @property
    def xoyw(self) -> I8Vector4: ...
    @property
    def xoyo(self) -> I8Vector4: ...
    @property
    def xoyl(self) -> I8Vector4: ...
    @property
    def xozx(self) -> I8Vector4: ...
    @property
    def xozy(self) -> I8Vector4: ...
    @property
    def xozz(self) -> I8Vector4: ...
    @property
    def xozw(self) -> I8Vector4: ...
    @property
    def xozo(self) -> I8Vector4: ...
    @property
    def xozl(self) -> I8Vector4: ...
    @property
    def xowx(self) -> I8Vector4: ...
    @property
    def xowy(self) -> I8Vector4: ...
    @property
    def xowz(self) -> I8Vector4: ...
    @property
    def xoww(self) -> I8Vector4: ...
    @property
    def xowo(self) -> I8Vector4: ...
    @property
    def xowl(self) -> I8Vector4: ...
    @property
    def xoox(self) -> I8Vector4: ...
    @property
    def xooy(self) -> I8Vector4: ...
    @property
    def xooz(self) -> I8Vector4: ...
    @property
    def xoow(self) -> I8Vector4: ...
    @property
    def xooo(self) -> I8Vector4: ...
    @property
    def xool(self) -> I8Vector4: ...
    @property
    def xolx(self) -> I8Vector4: ...
    @property
    def xoly(self) -> I8Vector4: ...
    @property
    def xolz(self) -> I8Vector4: ...
    @property
    def xolw(self) -> I8Vector4: ...
    @property
    def xolo(self) -> I8Vector4: ...
    @property
    def xoll(self) -> I8Vector4: ...
    @property
    def xlxx(self) -> I8Vector4: ...
    @property
    def xlxy(self) -> I8Vector4: ...
    @property
    def xlxz(self) -> I8Vector4: ...
    @property
    def xlxw(self) -> I8Vector4: ...
    @property
    def xlxo(self) -> I8Vector4: ...
    @property
    def xlxl(self) -> I8Vector4: ...
    @property
    def xlyx(self) -> I8Vector4: ...
    @property
    def xlyy(self) -> I8Vector4: ...
    @property
    def xlyz(self) -> I8Vector4: ...
    @property
    def xlyw(self) -> I8Vector4: ...
    @property
    def xlyo(self) -> I8Vector4: ...
    @property
    def xlyl(self) -> I8Vector4: ...
    @property
    def xlzx(self) -> I8Vector4: ...
    @property
    def xlzy(self) -> I8Vector4: ...
    @property
    def xlzz(self) -> I8Vector4: ...
    @property
    def xlzw(self) -> I8Vector4: ...
    @property
    def xlzo(self) -> I8Vector4: ...
    @property
    def xlzl(self) -> I8Vector4: ...
    @property
    def xlwx(self) -> I8Vector4: ...
    @property
    def xlwy(self) -> I8Vector4: ...
    @property
    def xlwz(self) -> I8Vector4: ...
    @property
    def xlww(self) -> I8Vector4: ...
    @property
    def xlwo(self) -> I8Vector4: ...
    @property
    def xlwl(self) -> I8Vector4: ...
    @property
    def xlox(self) -> I8Vector4: ...
    @property
    def xloy(self) -> I8Vector4: ...
    @property
    def xloz(self) -> I8Vector4: ...
    @property
    def xlow(self) -> I8Vector4: ...
    @property
    def xloo(self) -> I8Vector4: ...
    @property
    def xlol(self) -> I8Vector4: ...
    @property
    def xllx(self) -> I8Vector4: ...
    @property
    def xlly(self) -> I8Vector4: ...
    @property
    def xllz(self) -> I8Vector4: ...
    @property
    def xllw(self) -> I8Vector4: ...
    @property
    def xllo(self) -> I8Vector4: ...
    @property
    def xlll(self) -> I8Vector4: ...
    @property
    def yxxx(self) -> I8Vector4: ...
    @property
    def yxxy(self) -> I8Vector4: ...
    @property
    def yxxz(self) -> I8Vector4: ...
    @property
    def yxxw(self) -> I8Vector4: ...
    @property
    def yxxo(self) -> I8Vector4: ...
    @property
    def yxxl(self) -> I8Vector4: ...
    @property
    def yxyx(self) -> I8Vector4: ...
    @property
    def yxyy(self) -> I8Vector4: ...
    @property
    def yxyz(self) -> I8Vector4: ...
    @property
    def yxyw(self) -> I8Vector4: ...
    @property
    def yxyo(self) -> I8Vector4: ...
    @property
    def yxyl(self) -> I8Vector4: ...
    @property
    def yxzx(self) -> I8Vector4: ...
    @property
    def yxzy(self) -> I8Vector4: ...
    @property
    def yxzz(self) -> I8Vector4: ...
    @property
    def yxzw(self) -> I8Vector4: ...
    @property
    def yxzo(self) -> I8Vector4: ...
    @property
    def yxzl(self) -> I8Vector4: ...
    @property
    def yxwx(self) -> I8Vector4: ...
    @property
    def yxwy(self) -> I8Vector4: ...
    @property
    def yxwz(self) -> I8Vector4: ...
    @property
    def yxww(self) -> I8Vector4: ...
    @property
    def yxwo(self) -> I8Vector4: ...
    @property
    def yxwl(self) -> I8Vector4: ...
    @property
    def yxox(self) -> I8Vector4: ...
    @property
    def yxoy(self) -> I8Vector4: ...
    @property
    def yxoz(self) -> I8Vector4: ...
    @property
    def yxow(self) -> I8Vector4: ...
    @property
    def yxoo(self) -> I8Vector4: ...
    @property
    def yxol(self) -> I8Vector4: ...
    @property
    def yxlx(self) -> I8Vector4: ...
    @property
    def yxly(self) -> I8Vector4: ...
    @property
    def yxlz(self) -> I8Vector4: ...
    @property
    def yxlw(self) -> I8Vector4: ...
    @property
    def yxlo(self) -> I8Vector4: ...
    @property
    def yxll(self) -> I8Vector4: ...
    @property
    def yyxx(self) -> I8Vector4: ...
    @property
    def yyxy(self) -> I8Vector4: ...
    @property
    def yyxz(self) -> I8Vector4: ...
    @property
    def yyxw(self) -> I8Vector4: ...
    @property
    def yyxo(self) -> I8Vector4: ...
    @property
    def yyxl(self) -> I8Vector4: ...
    @property
    def yyyx(self) -> I8Vector4: ...
    @property
    def yyyy(self) -> I8Vector4: ...
    @property
    def yyyz(self) -> I8Vector4: ...
    @property
    def yyyw(self) -> I8Vector4: ...
    @property
    def yyyo(self) -> I8Vector4: ...
    @property
    def yyyl(self) -> I8Vector4: ...
    @property
    def yyzx(self) -> I8Vector4: ...
    @property
    def yyzy(self) -> I8Vector4: ...
    @property
    def yyzz(self) -> I8Vector4: ...
    @property
    def yyzw(self) -> I8Vector4: ...
    @property
    def yyzo(self) -> I8Vector4: ...
    @property
    def yyzl(self) -> I8Vector4: ...
    @property
    def yywx(self) -> I8Vector4: ...
    @property
    def yywy(self) -> I8Vector4: ...
    @property
    def yywz(self) -> I8Vector4: ...
    @property
    def yyww(self) -> I8Vector4: ...
    @property
    def yywo(self) -> I8Vector4: ...
    @property
    def yywl(self) -> I8Vector4: ...
    @property
    def yyox(self) -> I8Vector4: ...
    @property
    def yyoy(self) -> I8Vector4: ...
    @property
    def yyoz(self) -> I8Vector4: ...
    @property
    def yyow(self) -> I8Vector4: ...
    @property
    def yyoo(self) -> I8Vector4: ...
    @property
    def yyol(self) -> I8Vector4: ...
    @property
    def yylx(self) -> I8Vector4: ...
    @property
    def yyly(self) -> I8Vector4: ...
    @property
    def yylz(self) -> I8Vector4: ...
    @property
    def yylw(self) -> I8Vector4: ...
    @property
    def yylo(self) -> I8Vector4: ...
    @property
    def yyll(self) -> I8Vector4: ...
    @property
    def yzxx(self) -> I8Vector4: ...
    @property
    def yzxy(self) -> I8Vector4: ...
    @property
    def yzxz(self) -> I8Vector4: ...
    @property
    def yzxw(self) -> I8Vector4: ...
    @property
    def yzxo(self) -> I8Vector4: ...
    @property
    def yzxl(self) -> I8Vector4: ...
    @property
    def yzyx(self) -> I8Vector4: ...
    @property
    def yzyy(self) -> I8Vector4: ...
    @property
    def yzyz(self) -> I8Vector4: ...
    @property
    def yzyw(self) -> I8Vector4: ...
    @property
    def yzyo(self) -> I8Vector4: ...
    @property
    def yzyl(self) -> I8Vector4: ...
    @property
    def yzzx(self) -> I8Vector4: ...
    @property
    def yzzy(self) -> I8Vector4: ...
    @property
    def yzzz(self) -> I8Vector4: ...
    @property
    def yzzw(self) -> I8Vector4: ...
    @property
    def yzzo(self) -> I8Vector4: ...
    @property
    def yzzl(self) -> I8Vector4: ...
    @property
    def yzwx(self) -> I8Vector4: ...
    @property
    def yzwy(self) -> I8Vector4: ...
    @property
    def yzwz(self) -> I8Vector4: ...
    @property
    def yzww(self) -> I8Vector4: ...
    @property
    def yzwo(self) -> I8Vector4: ...
    @property
    def yzwl(self) -> I8Vector4: ...
    @property
    def yzox(self) -> I8Vector4: ...
    @property
    def yzoy(self) -> I8Vector4: ...
    @property
    def yzoz(self) -> I8Vector4: ...
    @property
    def yzow(self) -> I8Vector4: ...
    @property
    def yzoo(self) -> I8Vector4: ...
    @property
    def yzol(self) -> I8Vector4: ...
    @property
    def yzlx(self) -> I8Vector4: ...
    @property
    def yzly(self) -> I8Vector4: ...
    @property
    def yzlz(self) -> I8Vector4: ...
    @property
    def yzlw(self) -> I8Vector4: ...
    @property
    def yzlo(self) -> I8Vector4: ...
    @property
    def yzll(self) -> I8Vector4: ...
    @property
    def ywxx(self) -> I8Vector4: ...
    @property
    def ywxy(self) -> I8Vector4: ...
    @property
    def ywxz(self) -> I8Vector4: ...
    @property
    def ywxw(self) -> I8Vector4: ...
    @property
    def ywxo(self) -> I8Vector4: ...
    @property
    def ywxl(self) -> I8Vector4: ...
    @property
    def ywyx(self) -> I8Vector4: ...
    @property
    def ywyy(self) -> I8Vector4: ...
    @property
    def ywyz(self) -> I8Vector4: ...
    @property
    def ywyw(self) -> I8Vector4: ...
    @property
    def ywyo(self) -> I8Vector4: ...
    @property
    def ywyl(self) -> I8Vector4: ...
    @property
    def ywzx(self) -> I8Vector4: ...
    @property
    def ywzy(self) -> I8Vector4: ...
    @property
    def ywzz(self) -> I8Vector4: ...
    @property
    def ywzw(self) -> I8Vector4: ...
    @property
    def ywzo(self) -> I8Vector4: ...
    @property
    def ywzl(self) -> I8Vector4: ...
    @property
    def ywwx(self) -> I8Vector4: ...
    @property
    def ywwy(self) -> I8Vector4: ...
    @property
    def ywwz(self) -> I8Vector4: ...
    @property
    def ywww(self) -> I8Vector4: ...
    @property
    def ywwo(self) -> I8Vector4: ...
    @property
    def ywwl(self) -> I8Vector4: ...
    @property
    def ywox(self) -> I8Vector4: ...
    @property
    def ywoy(self) -> I8Vector4: ...
    @property
    def ywoz(self) -> I8Vector4: ...
    @property
    def ywow(self) -> I8Vector4: ...
    @property
    def ywoo(self) -> I8Vector4: ...
    @property
    def ywol(self) -> I8Vector4: ...
    @property
    def ywlx(self) -> I8Vector4: ...
    @property
    def ywly(self) -> I8Vector4: ...
    @property
    def ywlz(self) -> I8Vector4: ...
    @property
    def ywlw(self) -> I8Vector4: ...
    @property
    def ywlo(self) -> I8Vector4: ...
    @property
    def ywll(self) -> I8Vector4: ...
    @property
    def yoxx(self) -> I8Vector4: ...
    @property
    def yoxy(self) -> I8Vector4: ...
    @property
    def yoxz(self) -> I8Vector4: ...
    @property
    def yoxw(self) -> I8Vector4: ...
    @property
    def yoxo(self) -> I8Vector4: ...
    @property
    def yoxl(self) -> I8Vector4: ...
    @property
    def yoyx(self) -> I8Vector4: ...
    @property
    def yoyy(self) -> I8Vector4: ...
    @property
    def yoyz(self) -> I8Vector4: ...
    @property
    def yoyw(self) -> I8Vector4: ...
    @property
    def yoyo(self) -> I8Vector4: ...
    @property
    def yoyl(self) -> I8Vector4: ...
    @property
    def yozx(self) -> I8Vector4: ...
    @property
    def yozy(self) -> I8Vector4: ...
    @property
    def yozz(self) -> I8Vector4: ...
    @property
    def yozw(self) -> I8Vector4: ...
    @property
    def yozo(self) -> I8Vector4: ...
    @property
    def yozl(self) -> I8Vector4: ...
    @property
    def yowx(self) -> I8Vector4: ...
    @property
    def yowy(self) -> I8Vector4: ...
    @property
    def yowz(self) -> I8Vector4: ...
    @property
    def yoww(self) -> I8Vector4: ...
    @property
    def yowo(self) -> I8Vector4: ...
    @property
    def yowl(self) -> I8Vector4: ...
    @property
    def yoox(self) -> I8Vector4: ...
    @property
    def yooy(self) -> I8Vector4: ...
    @property
    def yooz(self) -> I8Vector4: ...
    @property
    def yoow(self) -> I8Vector4: ...
    @property
    def yooo(self) -> I8Vector4: ...
    @property
    def yool(self) -> I8Vector4: ...
    @property
    def yolx(self) -> I8Vector4: ...
    @property
    def yoly(self) -> I8Vector4: ...
    @property
    def yolz(self) -> I8Vector4: ...
    @property
    def yolw(self) -> I8Vector4: ...
    @property
    def yolo(self) -> I8Vector4: ...
    @property
    def yoll(self) -> I8Vector4: ...
    @property
    def ylxx(self) -> I8Vector4: ...
    @property
    def ylxy(self) -> I8Vector4: ...
    @property
    def ylxz(self) -> I8Vector4: ...
    @property
    def ylxw(self) -> I8Vector4: ...
    @property
    def ylxo(self) -> I8Vector4: ...
    @property
    def ylxl(self) -> I8Vector4: ...
    @property
    def ylyx(self) -> I8Vector4: ...
    @property
    def ylyy(self) -> I8Vector4: ...
    @property
    def ylyz(self) -> I8Vector4: ...
    @property
    def ylyw(self) -> I8Vector4: ...
    @property
    def ylyo(self) -> I8Vector4: ...
    @property
    def ylyl(self) -> I8Vector4: ...
    @property
    def ylzx(self) -> I8Vector4: ...
    @property
    def ylzy(self) -> I8Vector4: ...
    @property
    def ylzz(self) -> I8Vector4: ...
    @property
    def ylzw(self) -> I8Vector4: ...
    @property
    def ylzo(self) -> I8Vector4: ...
    @property
    def ylzl(self) -> I8Vector4: ...
    @property
    def ylwx(self) -> I8Vector4: ...
    @property
    def ylwy(self) -> I8Vector4: ...
    @property
    def ylwz(self) -> I8Vector4: ...
    @property
    def ylww(self) -> I8Vector4: ...
    @property
    def ylwo(self) -> I8Vector4: ...
    @property
    def ylwl(self) -> I8Vector4: ...
    @property
    def ylox(self) -> I8Vector4: ...
    @property
    def yloy(self) -> I8Vector4: ...
    @property
    def yloz(self) -> I8Vector4: ...
    @property
    def ylow(self) -> I8Vector4: ...
    @property
    def yloo(self) -> I8Vector4: ...
    @property
    def ylol(self) -> I8Vector4: ...
    @property
    def yllx(self) -> I8Vector4: ...
    @property
    def ylly(self) -> I8Vector4: ...
    @property
    def yllz(self) -> I8Vector4: ...
    @property
    def yllw(self) -> I8Vector4: ...
    @property
    def yllo(self) -> I8Vector4: ...
    @property
    def ylll(self) -> I8Vector4: ...
    @property
    def zxxx(self) -> I8Vector4: ...
    @property
    def zxxy(self) -> I8Vector4: ...
    @property
    def zxxz(self) -> I8Vector4: ...
    @property
    def zxxw(self) -> I8Vector4: ...
    @property
    def zxxo(self) -> I8Vector4: ...
    @property
    def zxxl(self) -> I8Vector4: ...
    @property
    def zxyx(self) -> I8Vector4: ...
    @property
    def zxyy(self) -> I8Vector4: ...
    @property
    def zxyz(self) -> I8Vector4: ...
    @property
    def zxyw(self) -> I8Vector4: ...
    @property
    def zxyo(self) -> I8Vector4: ...
    @property
    def zxyl(self) -> I8Vector4: ...
    @property
    def zxzx(self) -> I8Vector4: ...
    @property
    def zxzy(self) -> I8Vector4: ...
    @property
    def zxzz(self) -> I8Vector4: ...
    @property
    def zxzw(self) -> I8Vector4: ...
    @property
    def zxzo(self) -> I8Vector4: ...
    @property
    def zxzl(self) -> I8Vector4: ...
    @property
    def zxwx(self) -> I8Vector4: ...
    @property
    def zxwy(self) -> I8Vector4: ...
    @property
    def zxwz(self) -> I8Vector4: ...
    @property
    def zxww(self) -> I8Vector4: ...
    @property
    def zxwo(self) -> I8Vector4: ...
    @property
    def zxwl(self) -> I8Vector4: ...
    @property
    def zxox(self) -> I8Vector4: ...
    @property
    def zxoy(self) -> I8Vector4: ...
    @property
    def zxoz(self) -> I8Vector4: ...
    @property
    def zxow(self) -> I8Vector4: ...
    @property
    def zxoo(self) -> I8Vector4: ...
    @property
    def zxol(self) -> I8Vector4: ...
    @property
    def zxlx(self) -> I8Vector4: ...
    @property
    def zxly(self) -> I8Vector4: ...
    @property
    def zxlz(self) -> I8Vector4: ...
    @property
    def zxlw(self) -> I8Vector4: ...
    @property
    def zxlo(self) -> I8Vector4: ...
    @property
    def zxll(self) -> I8Vector4: ...
    @property
    def zyxx(self) -> I8Vector4: ...
    @property
    def zyxy(self) -> I8Vector4: ...
    @property
    def zyxz(self) -> I8Vector4: ...
    @property
    def zyxw(self) -> I8Vector4: ...
    @property
    def zyxo(self) -> I8Vector4: ...
    @property
    def zyxl(self) -> I8Vector4: ...
    @property
    def zyyx(self) -> I8Vector4: ...
    @property
    def zyyy(self) -> I8Vector4: ...
    @property
    def zyyz(self) -> I8Vector4: ...
    @property
    def zyyw(self) -> I8Vector4: ...
    @property
    def zyyo(self) -> I8Vector4: ...
    @property
    def zyyl(self) -> I8Vector4: ...
    @property
    def zyzx(self) -> I8Vector4: ...
    @property
    def zyzy(self) -> I8Vector4: ...
    @property
    def zyzz(self) -> I8Vector4: ...
    @property
    def zyzw(self) -> I8Vector4: ...
    @property
    def zyzo(self) -> I8Vector4: ...
    @property
    def zyzl(self) -> I8Vector4: ...
    @property
    def zywx(self) -> I8Vector4: ...
    @property
    def zywy(self) -> I8Vector4: ...
    @property
    def zywz(self) -> I8Vector4: ...
    @property
    def zyww(self) -> I8Vector4: ...
    @property
    def zywo(self) -> I8Vector4: ...
    @property
    def zywl(self) -> I8Vector4: ...
    @property
    def zyox(self) -> I8Vector4: ...
    @property
    def zyoy(self) -> I8Vector4: ...
    @property
    def zyoz(self) -> I8Vector4: ...
    @property
    def zyow(self) -> I8Vector4: ...
    @property
    def zyoo(self) -> I8Vector4: ...
    @property
    def zyol(self) -> I8Vector4: ...
    @property
    def zylx(self) -> I8Vector4: ...
    @property
    def zyly(self) -> I8Vector4: ...
    @property
    def zylz(self) -> I8Vector4: ...
    @property
    def zylw(self) -> I8Vector4: ...
    @property
    def zylo(self) -> I8Vector4: ...
    @property
    def zyll(self) -> I8Vector4: ...
    @property
    def zzxx(self) -> I8Vector4: ...
    @property
    def zzxy(self) -> I8Vector4: ...
    @property
    def zzxz(self) -> I8Vector4: ...
    @property
    def zzxw(self) -> I8Vector4: ...
    @property
    def zzxo(self) -> I8Vector4: ...
    @property
    def zzxl(self) -> I8Vector4: ...
    @property
    def zzyx(self) -> I8Vector4: ...
    @property
    def zzyy(self) -> I8Vector4: ...
    @property
    def zzyz(self) -> I8Vector4: ...
    @property
    def zzyw(self) -> I8Vector4: ...
    @property
    def zzyo(self) -> I8Vector4: ...
    @property
    def zzyl(self) -> I8Vector4: ...
    @property
    def zzzx(self) -> I8Vector4: ...
    @property
    def zzzy(self) -> I8Vector4: ...
    @property
    def zzzz(self) -> I8Vector4: ...
    @property
    def zzzw(self) -> I8Vector4: ...
    @property
    def zzzo(self) -> I8Vector4: ...
    @property
    def zzzl(self) -> I8Vector4: ...
    @property
    def zzwx(self) -> I8Vector4: ...
    @property
    def zzwy(self) -> I8Vector4: ...
    @property
    def zzwz(self) -> I8Vector4: ...
    @property
    def zzww(self) -> I8Vector4: ...
    @property
    def zzwo(self) -> I8Vector4: ...
    @property
    def zzwl(self) -> I8Vector4: ...
    @property
    def zzox(self) -> I8Vector4: ...
    @property
    def zzoy(self) -> I8Vector4: ...
    @property
    def zzoz(self) -> I8Vector4: ...
    @property
    def zzow(self) -> I8Vector4: ...
    @property
    def zzoo(self) -> I8Vector4: ...
    @property
    def zzol(self) -> I8Vector4: ...
    @property
    def zzlx(self) -> I8Vector4: ...
    @property
    def zzly(self) -> I8Vector4: ...
    @property
    def zzlz(self) -> I8Vector4: ...
    @property
    def zzlw(self) -> I8Vector4: ...
    @property
    def zzlo(self) -> I8Vector4: ...
    @property
    def zzll(self) -> I8Vector4: ...
    @property
    def zwxx(self) -> I8Vector4: ...
    @property
    def zwxy(self) -> I8Vector4: ...
    @property
    def zwxz(self) -> I8Vector4: ...
    @property
    def zwxw(self) -> I8Vector4: ...
    @property
    def zwxo(self) -> I8Vector4: ...
    @property
    def zwxl(self) -> I8Vector4: ...
    @property
    def zwyx(self) -> I8Vector4: ...
    @property
    def zwyy(self) -> I8Vector4: ...
    @property
    def zwyz(self) -> I8Vector4: ...
    @property
    def zwyw(self) -> I8Vector4: ...
    @property
    def zwyo(self) -> I8Vector4: ...
    @property
    def zwyl(self) -> I8Vector4: ...
    @property
    def zwzx(self) -> I8Vector4: ...
    @property
    def zwzy(self) -> I8Vector4: ...
    @property
    def zwzz(self) -> I8Vector4: ...
    @property
    def zwzw(self) -> I8Vector4: ...
    @property
    def zwzo(self) -> I8Vector4: ...
    @property
    def zwzl(self) -> I8Vector4: ...
    @property
    def zwwx(self) -> I8Vector4: ...
    @property
    def zwwy(self) -> I8Vector4: ...
    @property
    def zwwz(self) -> I8Vector4: ...
    @property
    def zwww(self) -> I8Vector4: ...
    @property
    def zwwo(self) -> I8Vector4: ...
    @property
    def zwwl(self) -> I8Vector4: ...
    @property
    def zwox(self) -> I8Vector4: ...
    @property
    def zwoy(self) -> I8Vector4: ...
    @property
    def zwoz(self) -> I8Vector4: ...
    @property
    def zwow(self) -> I8Vector4: ...
    @property
    def zwoo(self) -> I8Vector4: ...
    @property
    def zwol(self) -> I8Vector4: ...
    @property
    def zwlx(self) -> I8Vector4: ...
    @property
    def zwly(self) -> I8Vector4: ...
    @property
    def zwlz(self) -> I8Vector4: ...
    @property
    def zwlw(self) -> I8Vector4: ...
    @property
    def zwlo(self) -> I8Vector4: ...
    @property
    def zwll(self) -> I8Vector4: ...
    @property
    def zoxx(self) -> I8Vector4: ...
    @property
    def zoxy(self) -> I8Vector4: ...
    @property
    def zoxz(self) -> I8Vector4: ...
    @property
    def zoxw(self) -> I8Vector4: ...
    @property
    def zoxo(self) -> I8Vector4: ...
    @property
    def zoxl(self) -> I8Vector4: ...
    @property
    def zoyx(self) -> I8Vector4: ...
    @property
    def zoyy(self) -> I8Vector4: ...
    @property
    def zoyz(self) -> I8Vector4: ...
    @property
    def zoyw(self) -> I8Vector4: ...
    @property
    def zoyo(self) -> I8Vector4: ...
    @property
    def zoyl(self) -> I8Vector4: ...
    @property
    def zozx(self) -> I8Vector4: ...
    @property
    def zozy(self) -> I8Vector4: ...
    @property
    def zozz(self) -> I8Vector4: ...
    @property
    def zozw(self) -> I8Vector4: ...
    @property
    def zozo(self) -> I8Vector4: ...
    @property
    def zozl(self) -> I8Vector4: ...
    @property
    def zowx(self) -> I8Vector4: ...
    @property
    def zowy(self) -> I8Vector4: ...
    @property
    def zowz(self) -> I8Vector4: ...
    @property
    def zoww(self) -> I8Vector4: ...
    @property
    def zowo(self) -> I8Vector4: ...
    @property
    def zowl(self) -> I8Vector4: ...
    @property
    def zoox(self) -> I8Vector4: ...
    @property
    def zooy(self) -> I8Vector4: ...
    @property
    def zooz(self) -> I8Vector4: ...
    @property
    def zoow(self) -> I8Vector4: ...
    @property
    def zooo(self) -> I8Vector4: ...
    @property
    def zool(self) -> I8Vector4: ...
    @property
    def zolx(self) -> I8Vector4: ...
    @property
    def zoly(self) -> I8Vector4: ...
    @property
    def zolz(self) -> I8Vector4: ...
    @property
    def zolw(self) -> I8Vector4: ...
    @property
    def zolo(self) -> I8Vector4: ...
    @property
    def zoll(self) -> I8Vector4: ...
    @property
    def zlxx(self) -> I8Vector4: ...
    @property
    def zlxy(self) -> I8Vector4: ...
    @property
    def zlxz(self) -> I8Vector4: ...
    @property
    def zlxw(self) -> I8Vector4: ...
    @property
    def zlxo(self) -> I8Vector4: ...
    @property
    def zlxl(self) -> I8Vector4: ...
    @property
    def zlyx(self) -> I8Vector4: ...
    @property
    def zlyy(self) -> I8Vector4: ...
    @property
    def zlyz(self) -> I8Vector4: ...
    @property
    def zlyw(self) -> I8Vector4: ...
    @property
    def zlyo(self) -> I8Vector4: ...
    @property
    def zlyl(self) -> I8Vector4: ...
    @property
    def zlzx(self) -> I8Vector4: ...
    @property
    def zlzy(self) -> I8Vector4: ...
    @property
    def zlzz(self) -> I8Vector4: ...
    @property
    def zlzw(self) -> I8Vector4: ...
    @property
    def zlzo(self) -> I8Vector4: ...
    @property
    def zlzl(self) -> I8Vector4: ...
    @property
    def zlwx(self) -> I8Vector4: ...
    @property
    def zlwy(self) -> I8Vector4: ...
    @property
    def zlwz(self) -> I8Vector4: ...
    @property
    def zlww(self) -> I8Vector4: ...
    @property
    def zlwo(self) -> I8Vector4: ...
    @property
    def zlwl(self) -> I8Vector4: ...
    @property
    def zlox(self) -> I8Vector4: ...
    @property
    def zloy(self) -> I8Vector4: ...
    @property
    def zloz(self) -> I8Vector4: ...
    @property
    def zlow(self) -> I8Vector4: ...
    @property
    def zloo(self) -> I8Vector4: ...
    @property
    def zlol(self) -> I8Vector4: ...
    @property
    def zllx(self) -> I8Vector4: ...
    @property
    def zlly(self) -> I8Vector4: ...
    @property
    def zllz(self) -> I8Vector4: ...
    @property
    def zllw(self) -> I8Vector4: ...
    @property
    def zllo(self) -> I8Vector4: ...
    @property
    def zlll(self) -> I8Vector4: ...
    @property
    def wxxx(self) -> I8Vector4: ...
    @property
    def wxxy(self) -> I8Vector4: ...
    @property
    def wxxz(self) -> I8Vector4: ...
    @property
    def wxxw(self) -> I8Vector4: ...
    @property
    def wxxo(self) -> I8Vector4: ...
    @property
    def wxxl(self) -> I8Vector4: ...
    @property
    def wxyx(self) -> I8Vector4: ...
    @property
    def wxyy(self) -> I8Vector4: ...
    @property
    def wxyz(self) -> I8Vector4: ...
    @property
    def wxyw(self) -> I8Vector4: ...
    @property
    def wxyo(self) -> I8Vector4: ...
    @property
    def wxyl(self) -> I8Vector4: ...
    @property
    def wxzx(self) -> I8Vector4: ...
    @property
    def wxzy(self) -> I8Vector4: ...
    @property
    def wxzz(self) -> I8Vector4: ...
    @property
    def wxzw(self) -> I8Vector4: ...
    @property
    def wxzo(self) -> I8Vector4: ...
    @property
    def wxzl(self) -> I8Vector4: ...
    @property
    def wxwx(self) -> I8Vector4: ...
    @property
    def wxwy(self) -> I8Vector4: ...
    @property
    def wxwz(self) -> I8Vector4: ...
    @property
    def wxww(self) -> I8Vector4: ...
    @property
    def wxwo(self) -> I8Vector4: ...
    @property
    def wxwl(self) -> I8Vector4: ...
    @property
    def wxox(self) -> I8Vector4: ...
    @property
    def wxoy(self) -> I8Vector4: ...
    @property
    def wxoz(self) -> I8Vector4: ...
    @property
    def wxow(self) -> I8Vector4: ...
    @property
    def wxoo(self) -> I8Vector4: ...
    @property
    def wxol(self) -> I8Vector4: ...
    @property
    def wxlx(self) -> I8Vector4: ...
    @property
    def wxly(self) -> I8Vector4: ...
    @property
    def wxlz(self) -> I8Vector4: ...
    @property
    def wxlw(self) -> I8Vector4: ...
    @property
    def wxlo(self) -> I8Vector4: ...
    @property
    def wxll(self) -> I8Vector4: ...
    @property
    def wyxx(self) -> I8Vector4: ...
    @property
    def wyxy(self) -> I8Vector4: ...
    @property
    def wyxz(self) -> I8Vector4: ...
    @property
    def wyxw(self) -> I8Vector4: ...
    @property
    def wyxo(self) -> I8Vector4: ...
    @property
    def wyxl(self) -> I8Vector4: ...
    @property
    def wyyx(self) -> I8Vector4: ...
    @property
    def wyyy(self) -> I8Vector4: ...
    @property
    def wyyz(self) -> I8Vector4: ...
    @property
    def wyyw(self) -> I8Vector4: ...
    @property
    def wyyo(self) -> I8Vector4: ...
    @property
    def wyyl(self) -> I8Vector4: ...
    @property
    def wyzx(self) -> I8Vector4: ...
    @property
    def wyzy(self) -> I8Vector4: ...
    @property
    def wyzz(self) -> I8Vector4: ...
    @property
    def wyzw(self) -> I8Vector4: ...
    @property
    def wyzo(self) -> I8Vector4: ...
    @property
    def wyzl(self) -> I8Vector4: ...
    @property
    def wywx(self) -> I8Vector4: ...
    @property
    def wywy(self) -> I8Vector4: ...
    @property
    def wywz(self) -> I8Vector4: ...
    @property
    def wyww(self) -> I8Vector4: ...
    @property
    def wywo(self) -> I8Vector4: ...
    @property
    def wywl(self) -> I8Vector4: ...
    @property
    def wyox(self) -> I8Vector4: ...
    @property
    def wyoy(self) -> I8Vector4: ...
    @property
    def wyoz(self) -> I8Vector4: ...
    @property
    def wyow(self) -> I8Vector4: ...
    @property
    def wyoo(self) -> I8Vector4: ...
    @property
    def wyol(self) -> I8Vector4: ...
    @property
    def wylx(self) -> I8Vector4: ...
    @property
    def wyly(self) -> I8Vector4: ...
    @property
    def wylz(self) -> I8Vector4: ...
    @property
    def wylw(self) -> I8Vector4: ...
    @property
    def wylo(self) -> I8Vector4: ...
    @property
    def wyll(self) -> I8Vector4: ...
    @property
    def wzxx(self) -> I8Vector4: ...
    @property
    def wzxy(self) -> I8Vector4: ...
    @property
    def wzxz(self) -> I8Vector4: ...
    @property
    def wzxw(self) -> I8Vector4: ...
    @property
    def wzxo(self) -> I8Vector4: ...
    @property
    def wzxl(self) -> I8Vector4: ...
    @property
    def wzyx(self) -> I8Vector4: ...
    @property
    def wzyy(self) -> I8Vector4: ...
    @property
    def wzyz(self) -> I8Vector4: ...
    @property
    def wzyw(self) -> I8Vector4: ...
    @property
    def wzyo(self) -> I8Vector4: ...
    @property
    def wzyl(self) -> I8Vector4: ...
    @property
    def wzzx(self) -> I8Vector4: ...
    @property
    def wzzy(self) -> I8Vector4: ...
    @property
    def wzzz(self) -> I8Vector4: ...
    @property
    def wzzw(self) -> I8Vector4: ...
    @property
    def wzzo(self) -> I8Vector4: ...
    @property
    def wzzl(self) -> I8Vector4: ...
    @property
    def wzwx(self) -> I8Vector4: ...
    @property
    def wzwy(self) -> I8Vector4: ...
    @property
    def wzwz(self) -> I8Vector4: ...
    @property
    def wzww(self) -> I8Vector4: ...
    @property
    def wzwo(self) -> I8Vector4: ...
    @property
    def wzwl(self) -> I8Vector4: ...
    @property
    def wzox(self) -> I8Vector4: ...
    @property
    def wzoy(self) -> I8Vector4: ...
    @property
    def wzoz(self) -> I8Vector4: ...
    @property
    def wzow(self) -> I8Vector4: ...
    @property
    def wzoo(self) -> I8Vector4: ...
    @property
    def wzol(self) -> I8Vector4: ...
    @property
    def wzlx(self) -> I8Vector4: ...
    @property
    def wzly(self) -> I8Vector4: ...
    @property
    def wzlz(self) -> I8Vector4: ...
    @property
    def wzlw(self) -> I8Vector4: ...
    @property
    def wzlo(self) -> I8Vector4: ...
    @property
    def wzll(self) -> I8Vector4: ...
    @property
    def wwxx(self) -> I8Vector4: ...
    @property
    def wwxy(self) -> I8Vector4: ...
    @property
    def wwxz(self) -> I8Vector4: ...
    @property
    def wwxw(self) -> I8Vector4: ...
    @property
    def wwxo(self) -> I8Vector4: ...
    @property
    def wwxl(self) -> I8Vector4: ...
    @property
    def wwyx(self) -> I8Vector4: ...
    @property
    def wwyy(self) -> I8Vector4: ...
    @property
    def wwyz(self) -> I8Vector4: ...
    @property
    def wwyw(self) -> I8Vector4: ...
    @property
    def wwyo(self) -> I8Vector4: ...
    @property
    def wwyl(self) -> I8Vector4: ...
    @property
    def wwzx(self) -> I8Vector4: ...
    @property
    def wwzy(self) -> I8Vector4: ...
    @property
    def wwzz(self) -> I8Vector4: ...
    @property
    def wwzw(self) -> I8Vector4: ...
    @property
    def wwzo(self) -> I8Vector4: ...
    @property
    def wwzl(self) -> I8Vector4: ...
    @property
    def wwwx(self) -> I8Vector4: ...
    @property
    def wwwy(self) -> I8Vector4: ...
    @property
    def wwwz(self) -> I8Vector4: ...
    @property
    def wwww(self) -> I8Vector4: ...
    @property
    def wwwo(self) -> I8Vector4: ...
    @property
    def wwwl(self) -> I8Vector4: ...
    @property
    def wwox(self) -> I8Vector4: ...
    @property
    def wwoy(self) -> I8Vector4: ...
    @property
    def wwoz(self) -> I8Vector4: ...
    @property
    def wwow(self) -> I8Vector4: ...
    @property
    def wwoo(self) -> I8Vector4: ...
    @property
    def wwol(self) -> I8Vector4: ...
    @property
    def wwlx(self) -> I8Vector4: ...
    @property
    def wwly(self) -> I8Vector4: ...
    @property
    def wwlz(self) -> I8Vector4: ...
    @property
    def wwlw(self) -> I8Vector4: ...
    @property
    def wwlo(self) -> I8Vector4: ...
    @property
    def wwll(self) -> I8Vector4: ...
    @property
    def woxx(self) -> I8Vector4: ...
    @property
    def woxy(self) -> I8Vector4: ...
    @property
    def woxz(self) -> I8Vector4: ...
    @property
    def woxw(self) -> I8Vector4: ...
    @property
    def woxo(self) -> I8Vector4: ...
    @property
    def woxl(self) -> I8Vector4: ...
    @property
    def woyx(self) -> I8Vector4: ...
    @property
    def woyy(self) -> I8Vector4: ...
    @property
    def woyz(self) -> I8Vector4: ...
    @property
    def woyw(self) -> I8Vector4: ...
    @property
    def woyo(self) -> I8Vector4: ...
    @property
    def woyl(self) -> I8Vector4: ...
    @property
    def wozx(self) -> I8Vector4: ...
    @property
    def wozy(self) -> I8Vector4: ...
    @property
    def wozz(self) -> I8Vector4: ...
    @property
    def wozw(self) -> I8Vector4: ...
    @property
    def wozo(self) -> I8Vector4: ...
    @property
    def wozl(self) -> I8Vector4: ...
    @property
    def wowx(self) -> I8Vector4: ...
    @property
    def wowy(self) -> I8Vector4: ...
    @property
    def wowz(self) -> I8Vector4: ...
    @property
    def woww(self) -> I8Vector4: ...
    @property
    def wowo(self) -> I8Vector4: ...
    @property
    def wowl(self) -> I8Vector4: ...
    @property
    def woox(self) -> I8Vector4: ...
    @property
    def wooy(self) -> I8Vector4: ...
    @property
    def wooz(self) -> I8Vector4: ...
    @property
    def woow(self) -> I8Vector4: ...
    @property
    def wooo(self) -> I8Vector4: ...
    @property
    def wool(self) -> I8Vector4: ...
    @property
    def wolx(self) -> I8Vector4: ...
    @property
    def woly(self) -> I8Vector4: ...
    @property
    def wolz(self) -> I8Vector4: ...
    @property
    def wolw(self) -> I8Vector4: ...
    @property
    def wolo(self) -> I8Vector4: ...
    @property
    def woll(self) -> I8Vector4: ...
    @property
    def wlxx(self) -> I8Vector4: ...
    @property
    def wlxy(self) -> I8Vector4: ...
    @property
    def wlxz(self) -> I8Vector4: ...
    @property
    def wlxw(self) -> I8Vector4: ...
    @property
    def wlxo(self) -> I8Vector4: ...
    @property
    def wlxl(self) -> I8Vector4: ...
    @property
    def wlyx(self) -> I8Vector4: ...
    @property
    def wlyy(self) -> I8Vector4: ...
    @property
    def wlyz(self) -> I8Vector4: ...
    @property
    def wlyw(self) -> I8Vector4: ...
    @property
    def wlyo(self) -> I8Vector4: ...
    @property
    def wlyl(self) -> I8Vector4: ...
    @property
    def wlzx(self) -> I8Vector4: ...
    @property
    def wlzy(self) -> I8Vector4: ...
    @property
    def wlzz(self) -> I8Vector4: ...
    @property
    def wlzw(self) -> I8Vector4: ...
    @property
    def wlzo(self) -> I8Vector4: ...
    @property
    def wlzl(self) -> I8Vector4: ...
    @property
    def wlwx(self) -> I8Vector4: ...
    @property
    def wlwy(self) -> I8Vector4: ...
    @property
    def wlwz(self) -> I8Vector4: ...
    @property
    def wlww(self) -> I8Vector4: ...
    @property
    def wlwo(self) -> I8Vector4: ...
    @property
    def wlwl(self) -> I8Vector4: ...
    @property
    def wlox(self) -> I8Vector4: ...
    @property
    def wloy(self) -> I8Vector4: ...
    @property
    def wloz(self) -> I8Vector4: ...
    @property
    def wlow(self) -> I8Vector4: ...
    @property
    def wloo(self) -> I8Vector4: ...
    @property
    def wlol(self) -> I8Vector4: ...
    @property
    def wllx(self) -> I8Vector4: ...
    @property
    def wlly(self) -> I8Vector4: ...
    @property
    def wllz(self) -> I8Vector4: ...
    @property
    def wllw(self) -> I8Vector4: ...
    @property
    def wllo(self) -> I8Vector4: ...
    @property
    def wlll(self) -> I8Vector4: ...
    @property
    def oxxx(self) -> I8Vector4: ...
    @property
    def oxxy(self) -> I8Vector4: ...
    @property
    def oxxz(self) -> I8Vector4: ...
    @property
    def oxxw(self) -> I8Vector4: ...
    @property
    def oxxo(self) -> I8Vector4: ...
    @property
    def oxxl(self) -> I8Vector4: ...
    @property
    def oxyx(self) -> I8Vector4: ...
    @property
    def oxyy(self) -> I8Vector4: ...
    @property
    def oxyz(self) -> I8Vector4: ...
    @property
    def oxyw(self) -> I8Vector4: ...
    @property
    def oxyo(self) -> I8Vector4: ...
    @property
    def oxyl(self) -> I8Vector4: ...
    @property
    def oxzx(self) -> I8Vector4: ...
    @property
    def oxzy(self) -> I8Vector4: ...
    @property
    def oxzz(self) -> I8Vector4: ...
    @property
    def oxzw(self) -> I8Vector4: ...
    @property
    def oxzo(self) -> I8Vector4: ...
    @property
    def oxzl(self) -> I8Vector4: ...
    @property
    def oxwx(self) -> I8Vector4: ...
    @property
    def oxwy(self) -> I8Vector4: ...
    @property
    def oxwz(self) -> I8Vector4: ...
    @property
    def oxww(self) -> I8Vector4: ...
    @property
    def oxwo(self) -> I8Vector4: ...
    @property
    def oxwl(self) -> I8Vector4: ...
    @property
    def oxox(self) -> I8Vector4: ...
    @property
    def oxoy(self) -> I8Vector4: ...
    @property
    def oxoz(self) -> I8Vector4: ...
    @property
    def oxow(self) -> I8Vector4: ...
    @property
    def oxoo(self) -> I8Vector4: ...
    @property
    def oxol(self) -> I8Vector4: ...
    @property
    def oxlx(self) -> I8Vector4: ...
    @property
    def oxly(self) -> I8Vector4: ...
    @property
    def oxlz(self) -> I8Vector4: ...
    @property
    def oxlw(self) -> I8Vector4: ...
    @property
    def oxlo(self) -> I8Vector4: ...
    @property
    def oxll(self) -> I8Vector4: ...
    @property
    def oyxx(self) -> I8Vector4: ...
    @property
    def oyxy(self) -> I8Vector4: ...
    @property
    def oyxz(self) -> I8Vector4: ...
    @property
    def oyxw(self) -> I8Vector4: ...
    @property
    def oyxo(self) -> I8Vector4: ...
    @property
    def oyxl(self) -> I8Vector4: ...
    @property
    def oyyx(self) -> I8Vector4: ...
    @property
    def oyyy(self) -> I8Vector4: ...
    @property
    def oyyz(self) -> I8Vector4: ...
    @property
    def oyyw(self) -> I8Vector4: ...
    @property
    def oyyo(self) -> I8Vector4: ...
    @property
    def oyyl(self) -> I8Vector4: ...
    @property
    def oyzx(self) -> I8Vector4: ...
    @property
    def oyzy(self) -> I8Vector4: ...
    @property
    def oyzz(self) -> I8Vector4: ...
    @property
    def oyzw(self) -> I8Vector4: ...
    @property
    def oyzo(self) -> I8Vector4: ...
    @property
    def oyzl(self) -> I8Vector4: ...
    @property
    def oywx(self) -> I8Vector4: ...
    @property
    def oywy(self) -> I8Vector4: ...
    @property
    def oywz(self) -> I8Vector4: ...
    @property
    def oyww(self) -> I8Vector4: ...
    @property
    def oywo(self) -> I8Vector4: ...
    @property
    def oywl(self) -> I8Vector4: ...
    @property
    def oyox(self) -> I8Vector4: ...
    @property
    def oyoy(self) -> I8Vector4: ...
    @property
    def oyoz(self) -> I8Vector4: ...
    @property
    def oyow(self) -> I8Vector4: ...
    @property
    def oyoo(self) -> I8Vector4: ...
    @property
    def oyol(self) -> I8Vector4: ...
    @property
    def oylx(self) -> I8Vector4: ...
    @property
    def oyly(self) -> I8Vector4: ...
    @property
    def oylz(self) -> I8Vector4: ...
    @property
    def oylw(self) -> I8Vector4: ...
    @property
    def oylo(self) -> I8Vector4: ...
    @property
    def oyll(self) -> I8Vector4: ...
    @property
    def ozxx(self) -> I8Vector4: ...
    @property
    def ozxy(self) -> I8Vector4: ...
    @property
    def ozxz(self) -> I8Vector4: ...
    @property
    def ozxw(self) -> I8Vector4: ...
    @property
    def ozxo(self) -> I8Vector4: ...
    @property
    def ozxl(self) -> I8Vector4: ...
    @property
    def ozyx(self) -> I8Vector4: ...
    @property
    def ozyy(self) -> I8Vector4: ...
    @property
    def ozyz(self) -> I8Vector4: ...
    @property
    def ozyw(self) -> I8Vector4: ...
    @property
    def ozyo(self) -> I8Vector4: ...
    @property
    def ozyl(self) -> I8Vector4: ...
    @property
    def ozzx(self) -> I8Vector4: ...
    @property
    def ozzy(self) -> I8Vector4: ...
    @property
    def ozzz(self) -> I8Vector4: ...
    @property
    def ozzw(self) -> I8Vector4: ...
    @property
    def ozzo(self) -> I8Vector4: ...
    @property
    def ozzl(self) -> I8Vector4: ...
    @property
    def ozwx(self) -> I8Vector4: ...
    @property
    def ozwy(self) -> I8Vector4: ...
    @property
    def ozwz(self) -> I8Vector4: ...
    @property
    def ozww(self) -> I8Vector4: ...
    @property
    def ozwo(self) -> I8Vector4: ...
    @property
    def ozwl(self) -> I8Vector4: ...
    @property
    def ozox(self) -> I8Vector4: ...
    @property
    def ozoy(self) -> I8Vector4: ...
    @property
    def ozoz(self) -> I8Vector4: ...
    @property
    def ozow(self) -> I8Vector4: ...
    @property
    def ozoo(self) -> I8Vector4: ...
    @property
    def ozol(self) -> I8Vector4: ...
    @property
    def ozlx(self) -> I8Vector4: ...
    @property
    def ozly(self) -> I8Vector4: ...
    @property
    def ozlz(self) -> I8Vector4: ...
    @property
    def ozlw(self) -> I8Vector4: ...
    @property
    def ozlo(self) -> I8Vector4: ...
    @property
    def ozll(self) -> I8Vector4: ...
    @property
    def owxx(self) -> I8Vector4: ...
    @property
    def owxy(self) -> I8Vector4: ...
    @property
    def owxz(self) -> I8Vector4: ...
    @property
    def owxw(self) -> I8Vector4: ...
    @property
    def owxo(self) -> I8Vector4: ...
    @property
    def owxl(self) -> I8Vector4: ...
    @property
    def owyx(self) -> I8Vector4: ...
    @property
    def owyy(self) -> I8Vector4: ...
    @property
    def owyz(self) -> I8Vector4: ...
    @property
    def owyw(self) -> I8Vector4: ...
    @property
    def owyo(self) -> I8Vector4: ...
    @property
    def owyl(self) -> I8Vector4: ...
    @property
    def owzx(self) -> I8Vector4: ...
    @property
    def owzy(self) -> I8Vector4: ...
    @property
    def owzz(self) -> I8Vector4: ...
    @property
    def owzw(self) -> I8Vector4: ...
    @property
    def owzo(self) -> I8Vector4: ...
    @property
    def owzl(self) -> I8Vector4: ...
    @property
    def owwx(self) -> I8Vector4: ...
    @property
    def owwy(self) -> I8Vector4: ...
    @property
    def owwz(self) -> I8Vector4: ...
    @property
    def owww(self) -> I8Vector4: ...
    @property
    def owwo(self) -> I8Vector4: ...
    @property
    def owwl(self) -> I8Vector4: ...
    @property
    def owox(self) -> I8Vector4: ...
    @property
    def owoy(self) -> I8Vector4: ...
    @property
    def owoz(self) -> I8Vector4: ...
    @property
    def owow(self) -> I8Vector4: ...
    @property
    def owoo(self) -> I8Vector4: ...
    @property
    def owol(self) -> I8Vector4: ...
    @property
    def owlx(self) -> I8Vector4: ...
    @property
    def owly(self) -> I8Vector4: ...
    @property
    def owlz(self) -> I8Vector4: ...
    @property
    def owlw(self) -> I8Vector4: ...
    @property
    def owlo(self) -> I8Vector4: ...
    @property
    def owll(self) -> I8Vector4: ...
    @property
    def ooxx(self) -> I8Vector4: ...
    @property
    def ooxy(self) -> I8Vector4: ...
    @property
    def ooxz(self) -> I8Vector4: ...
    @property
    def ooxw(self) -> I8Vector4: ...
    @property
    def ooxo(self) -> I8Vector4: ...
    @property
    def ooxl(self) -> I8Vector4: ...
    @property
    def ooyx(self) -> I8Vector4: ...
    @property
    def ooyy(self) -> I8Vector4: ...
    @property
    def ooyz(self) -> I8Vector4: ...
    @property
    def ooyw(self) -> I8Vector4: ...
    @property
    def ooyo(self) -> I8Vector4: ...
    @property
    def ooyl(self) -> I8Vector4: ...
    @property
    def oozx(self) -> I8Vector4: ...
    @property
    def oozy(self) -> I8Vector4: ...
    @property
    def oozz(self) -> I8Vector4: ...
    @property
    def oozw(self) -> I8Vector4: ...
    @property
    def oozo(self) -> I8Vector4: ...
    @property
    def oozl(self) -> I8Vector4: ...
    @property
    def oowx(self) -> I8Vector4: ...
    @property
    def oowy(self) -> I8Vector4: ...
    @property
    def oowz(self) -> I8Vector4: ...
    @property
    def ooww(self) -> I8Vector4: ...
    @property
    def oowo(self) -> I8Vector4: ...
    @property
    def oowl(self) -> I8Vector4: ...
    @property
    def ooox(self) -> I8Vector4: ...
    @property
    def oooy(self) -> I8Vector4: ...
    @property
    def oooz(self) -> I8Vector4: ...
    @property
    def ooow(self) -> I8Vector4: ...
    @property
    def oolx(self) -> I8Vector4: ...
    @property
    def ooly(self) -> I8Vector4: ...
    @property
    def oolz(self) -> I8Vector4: ...
    @property
    def oolw(self) -> I8Vector4: ...
    @property
    def olxx(self) -> I8Vector4: ...
    @property
    def olxy(self) -> I8Vector4: ...
    @property
    def olxz(self) -> I8Vector4: ...
    @property
    def olxw(self) -> I8Vector4: ...
    @property
    def olxo(self) -> I8Vector4: ...
    @property
    def olxl(self) -> I8Vector4: ...
    @property
    def olyx(self) -> I8Vector4: ...
    @property
    def olyy(self) -> I8Vector4: ...
    @property
    def olyz(self) -> I8Vector4: ...
    @property
    def olyw(self) -> I8Vector4: ...
    @property
    def olyo(self) -> I8Vector4: ...
    @property
    def olyl(self) -> I8Vector4: ...
    @property
    def olzx(self) -> I8Vector4: ...
    @property
    def olzy(self) -> I8Vector4: ...
    @property
    def olzz(self) -> I8Vector4: ...
    @property
    def olzw(self) -> I8Vector4: ...
    @property
    def olzo(self) -> I8Vector4: ...
    @property
    def olzl(self) -> I8Vector4: ...
    @property
    def olwx(self) -> I8Vector4: ...
    @property
    def olwy(self) -> I8Vector4: ...
    @property
    def olwz(self) -> I8Vector4: ...
    @property
    def olww(self) -> I8Vector4: ...
    @property
    def olwo(self) -> I8Vector4: ...
    @property
    def olwl(self) -> I8Vector4: ...
    @property
    def olox(self) -> I8Vector4: ...
    @property
    def oloy(self) -> I8Vector4: ...
    @property
    def oloz(self) -> I8Vector4: ...
    @property
    def olow(self) -> I8Vector4: ...
    @property
    def ollx(self) -> I8Vector4: ...
    @property
    def olly(self) -> I8Vector4: ...
    @property
    def ollz(self) -> I8Vector4: ...
    @property
    def ollw(self) -> I8Vector4: ...
    @property
    def lxxx(self) -> I8Vector4: ...
    @property
    def lxxy(self) -> I8Vector4: ...
    @property
    def lxxz(self) -> I8Vector4: ...
    @property
    def lxxw(self) -> I8Vector4: ...
    @property
    def lxxo(self) -> I8Vector4: ...
    @property
    def lxxl(self) -> I8Vector4: ...
    @property
    def lxyx(self) -> I8Vector4: ...
    @property
    def lxyy(self) -> I8Vector4: ...
    @property
    def lxyz(self) -> I8Vector4: ...
    @property
    def lxyw(self) -> I8Vector4: ...
    @property
    def lxyo(self) -> I8Vector4: ...
    @property
    def lxyl(self) -> I8Vector4: ...
    @property
    def lxzx(self) -> I8Vector4: ...
    @property
    def lxzy(self) -> I8Vector4: ...
    @property
    def lxzz(self) -> I8Vector4: ...
    @property
    def lxzw(self) -> I8Vector4: ...
    @property
    def lxzo(self) -> I8Vector4: ...
    @property
    def lxzl(self) -> I8Vector4: ...
    @property
    def lxwx(self) -> I8Vector4: ...
    @property
    def lxwy(self) -> I8Vector4: ...
    @property
    def lxwz(self) -> I8Vector4: ...
    @property
    def lxww(self) -> I8Vector4: ...
    @property
    def lxwo(self) -> I8Vector4: ...
    @property
    def lxwl(self) -> I8Vector4: ...
    @property
    def lxox(self) -> I8Vector4: ...
    @property
    def lxoy(self) -> I8Vector4: ...
    @property
    def lxoz(self) -> I8Vector4: ...
    @property
    def lxow(self) -> I8Vector4: ...
    @property
    def lxoo(self) -> I8Vector4: ...
    @property
    def lxol(self) -> I8Vector4: ...
    @property
    def lxlx(self) -> I8Vector4: ...
    @property
    def lxly(self) -> I8Vector4: ...
    @property
    def lxlz(self) -> I8Vector4: ...
    @property
    def lxlw(self) -> I8Vector4: ...
    @property
    def lxlo(self) -> I8Vector4: ...
    @property
    def lxll(self) -> I8Vector4: ...
    @property
    def lyxx(self) -> I8Vector4: ...
    @property
    def lyxy(self) -> I8Vector4: ...
    @property
    def lyxz(self) -> I8Vector4: ...
    @property
    def lyxw(self) -> I8Vector4: ...
    @property
    def lyxo(self) -> I8Vector4: ...
    @property
    def lyxl(self) -> I8Vector4: ...
    @property
    def lyyx(self) -> I8Vector4: ...
    @property
    def lyyy(self) -> I8Vector4: ...
    @property
    def lyyz(self) -> I8Vector4: ...
    @property
    def lyyw(self) -> I8Vector4: ...
    @property
    def lyyo(self) -> I8Vector4: ...
    @property
    def lyyl(self) -> I8Vector4: ...
    @property
    def lyzx(self) -> I8Vector4: ...
    @property
    def lyzy(self) -> I8Vector4: ...
    @property
    def lyzz(self) -> I8Vector4: ...
    @property
    def lyzw(self) -> I8Vector4: ...
    @property
    def lyzo(self) -> I8Vector4: ...
    @property
    def lyzl(self) -> I8Vector4: ...
    @property
    def lywx(self) -> I8Vector4: ...
    @property
    def lywy(self) -> I8Vector4: ...
    @property
    def lywz(self) -> I8Vector4: ...
    @property
    def lyww(self) -> I8Vector4: ...
    @property
    def lywo(self) -> I8Vector4: ...
    @property
    def lywl(self) -> I8Vector4: ...
    @property
    def lyox(self) -> I8Vector4: ...
    @property
    def lyoy(self) -> I8Vector4: ...
    @property
    def lyoz(self) -> I8Vector4: ...
    @property
    def lyow(self) -> I8Vector4: ...
    @property
    def lyoo(self) -> I8Vector4: ...
    @property
    def lyol(self) -> I8Vector4: ...
    @property
    def lylx(self) -> I8Vector4: ...
    @property
    def lyly(self) -> I8Vector4: ...
    @property
    def lylz(self) -> I8Vector4: ...
    @property
    def lylw(self) -> I8Vector4: ...
    @property
    def lylo(self) -> I8Vector4: ...
    @property
    def lyll(self) -> I8Vector4: ...
    @property
    def lzxx(self) -> I8Vector4: ...
    @property
    def lzxy(self) -> I8Vector4: ...
    @property
    def lzxz(self) -> I8Vector4: ...
    @property
    def lzxw(self) -> I8Vector4: ...
    @property
    def lzxo(self) -> I8Vector4: ...
    @property
    def lzxl(self) -> I8Vector4: ...
    @property
    def lzyx(self) -> I8Vector4: ...
    @property
    def lzyy(self) -> I8Vector4: ...
    @property
    def lzyz(self) -> I8Vector4: ...
    @property
    def lzyw(self) -> I8Vector4: ...
    @property
    def lzyo(self) -> I8Vector4: ...
    @property
    def lzyl(self) -> I8Vector4: ...
    @property
    def lzzx(self) -> I8Vector4: ...
    @property
    def lzzy(self) -> I8Vector4: ...
    @property
    def lzzz(self) -> I8Vector4: ...
    @property
    def lzzw(self) -> I8Vector4: ...
    @property
    def lzzo(self) -> I8Vector4: ...
    @property
    def lzzl(self) -> I8Vector4: ...
    @property
    def lzwx(self) -> I8Vector4: ...
    @property
    def lzwy(self) -> I8Vector4: ...
    @property
    def lzwz(self) -> I8Vector4: ...
    @property
    def lzww(self) -> I8Vector4: ...
    @property
    def lzwo(self) -> I8Vector4: ...
    @property
    def lzwl(self) -> I8Vector4: ...
    @property
    def lzox(self) -> I8Vector4: ...
    @property
    def lzoy(self) -> I8Vector4: ...
    @property
    def lzoz(self) -> I8Vector4: ...
    @property
    def lzow(self) -> I8Vector4: ...
    @property
    def lzoo(self) -> I8Vector4: ...
    @property
    def lzol(self) -> I8Vector4: ...
    @property
    def lzlx(self) -> I8Vector4: ...
    @property
    def lzly(self) -> I8Vector4: ...
    @property
    def lzlz(self) -> I8Vector4: ...
    @property
    def lzlw(self) -> I8Vector4: ...
    @property
    def lzlo(self) -> I8Vector4: ...
    @property
    def lzll(self) -> I8Vector4: ...
    @property
    def lwxx(self) -> I8Vector4: ...
    @property
    def lwxy(self) -> I8Vector4: ...
    @property
    def lwxz(self) -> I8Vector4: ...
    @property
    def lwxw(self) -> I8Vector4: ...
    @property
    def lwxo(self) -> I8Vector4: ...
    @property
    def lwxl(self) -> I8Vector4: ...
    @property
    def lwyx(self) -> I8Vector4: ...
    @property
    def lwyy(self) -> I8Vector4: ...
    @property
    def lwyz(self) -> I8Vector4: ...
    @property
    def lwyw(self) -> I8Vector4: ...
    @property
    def lwyo(self) -> I8Vector4: ...
    @property
    def lwyl(self) -> I8Vector4: ...
    @property
    def lwzx(self) -> I8Vector4: ...
    @property
    def lwzy(self) -> I8Vector4: ...
    @property
    def lwzz(self) -> I8Vector4: ...
    @property
    def lwzw(self) -> I8Vector4: ...
    @property
    def lwzo(self) -> I8Vector4: ...
    @property
    def lwzl(self) -> I8Vector4: ...
    @property
    def lwwx(self) -> I8Vector4: ...
    @property
    def lwwy(self) -> I8Vector4: ...
    @property
    def lwwz(self) -> I8Vector4: ...
    @property
    def lwww(self) -> I8Vector4: ...
    @property
    def lwwo(self) -> I8Vector4: ...
    @property
    def lwwl(self) -> I8Vector4: ...
    @property
    def lwox(self) -> I8Vector4: ...
    @property
    def lwoy(self) -> I8Vector4: ...
    @property
    def lwoz(self) -> I8Vector4: ...
    @property
    def lwow(self) -> I8Vector4: ...
    @property
    def lwoo(self) -> I8Vector4: ...
    @property
    def lwol(self) -> I8Vector4: ...
    @property
    def lwlx(self) -> I8Vector4: ...
    @property
    def lwly(self) -> I8Vector4: ...
    @property
    def lwlz(self) -> I8Vector4: ...
    @property
    def lwlw(self) -> I8Vector4: ...
    @property
    def lwlo(self) -> I8Vector4: ...
    @property
    def lwll(self) -> I8Vector4: ...
    @property
    def loxx(self) -> I8Vector4: ...
    @property
    def loxy(self) -> I8Vector4: ...
    @property
    def loxz(self) -> I8Vector4: ...
    @property
    def loxw(self) -> I8Vector4: ...
    @property
    def loxo(self) -> I8Vector4: ...
    @property
    def loxl(self) -> I8Vector4: ...
    @property
    def loyx(self) -> I8Vector4: ...
    @property
    def loyy(self) -> I8Vector4: ...
    @property
    def loyz(self) -> I8Vector4: ...
    @property
    def loyw(self) -> I8Vector4: ...
    @property
    def loyo(self) -> I8Vector4: ...
    @property
    def loyl(self) -> I8Vector4: ...
    @property
    def lozx(self) -> I8Vector4: ...
    @property
    def lozy(self) -> I8Vector4: ...
    @property
    def lozz(self) -> I8Vector4: ...
    @property
    def lozw(self) -> I8Vector4: ...
    @property
    def lozo(self) -> I8Vector4: ...
    @property
    def lozl(self) -> I8Vector4: ...
    @property
    def lowx(self) -> I8Vector4: ...
    @property
    def lowy(self) -> I8Vector4: ...
    @property
    def lowz(self) -> I8Vector4: ...
    @property
    def loww(self) -> I8Vector4: ...
    @property
    def lowo(self) -> I8Vector4: ...
    @property
    def lowl(self) -> I8Vector4: ...
    @property
    def loox(self) -> I8Vector4: ...
    @property
    def looy(self) -> I8Vector4: ...
    @property
    def looz(self) -> I8Vector4: ...
    @property
    def loow(self) -> I8Vector4: ...
    @property
    def lolx(self) -> I8Vector4: ...
    @property
    def loly(self) -> I8Vector4: ...
    @property
    def lolz(self) -> I8Vector4: ...
    @property
    def lolw(self) -> I8Vector4: ...
    @property
    def llxx(self) -> I8Vector4: ...
    @property
    def llxy(self) -> I8Vector4: ...
    @property
    def llxz(self) -> I8Vector4: ...
    @property
    def llxw(self) -> I8Vector4: ...
    @property
    def llxo(self) -> I8Vector4: ...
    @property
    def llxl(self) -> I8Vector4: ...
    @property
    def llyx(self) -> I8Vector4: ...
    @property
    def llyy(self) -> I8Vector4: ...
    @property
    def llyz(self) -> I8Vector4: ...
    @property
    def llyw(self) -> I8Vector4: ...
    @property
    def llyo(self) -> I8Vector4: ...
    @property
    def llyl(self) -> I8Vector4: ...
    @property
    def llzx(self) -> I8Vector4: ...
    @property
    def llzy(self) -> I8Vector4: ...
    @property
    def llzz(self) -> I8Vector4: ...
    @property
    def llzw(self) -> I8Vector4: ...
    @property
    def llzo(self) -> I8Vector4: ...
    @property
    def llzl(self) -> I8Vector4: ...
    @property
    def llwx(self) -> I8Vector4: ...
    @property
    def llwy(self) -> I8Vector4: ...
    @property
    def llwz(self) -> I8Vector4: ...
    @property
    def llww(self) -> I8Vector4: ...
    @property
    def llwo(self) -> I8Vector4: ...
    @property
    def llwl(self) -> I8Vector4: ...
    @property
    def llox(self) -> I8Vector4: ...
    @property
    def lloy(self) -> I8Vector4: ...
    @property
    def lloz(self) -> I8Vector4: ...
    @property
    def llow(self) -> I8Vector4: ...
    @property
    def lllx(self) -> I8Vector4: ...
    @property
    def llly(self) -> I8Vector4: ...
    @property
    def lllz(self) -> I8Vector4: ...
    @property
    def lllw(self) -> I8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> I8Vector2: ...
    @property
    def rg(self) -> I8Vector2: ...
    @property
    def rb(self) -> I8Vector2: ...
    @property
    def ra(self) -> I8Vector2: ...
    @property
    def ro(self) -> I8Vector2: ...
    @property
    def rl(self) -> I8Vector2: ...
    @property
    def gr(self) -> I8Vector2: ...
    @property
    def gg(self) -> I8Vector2: ...
    @property
    def gb(self) -> I8Vector2: ...
    @property
    def ga(self) -> I8Vector2: ...
    @property
    def go(self) -> I8Vector2: ...
    @property
    def gl(self) -> I8Vector2: ...
    @property
    def br(self) -> I8Vector2: ...
    @property
    def bg(self) -> I8Vector2: ...
    @property
    def bb(self) -> I8Vector2: ...
    @property
    def ba(self) -> I8Vector2: ...
    @property
    def bo(self) -> I8Vector2: ...
    @property
    def bl(self) -> I8Vector2: ...
    @property
    def ar(self) -> I8Vector2: ...
    @property
    def ag(self) -> I8Vector2: ...
    @property
    def ab(self) -> I8Vector2: ...
    @property
    def aa(self) -> I8Vector2: ...
    @property
    def ao(self) -> I8Vector2: ...
    @property
    def al(self) -> I8Vector2: ...
    @property
    def og(self) -> I8Vector2: ...
    @property
    def ob(self) -> I8Vector2: ...
    @property
    def oa(self) -> I8Vector2: ...
    @property
    def lr(self) -> I8Vector2: ...
    @property
    def lg(self) -> I8Vector2: ...
    @property
    def lb(self) -> I8Vector2: ...
    @property
    def la(self) -> I8Vector2: ...
    @property
    def rrr(self) -> I8Vector3: ...
    @property
    def rrg(self) -> I8Vector3: ...
    @property
    def rrb(self) -> I8Vector3: ...
    @property
    def rra(self) -> I8Vector3: ...
    @property
    def rro(self) -> I8Vector3: ...
    @property
    def rrl(self) -> I8Vector3: ...
    @property
    def rgr(self) -> I8Vector3: ...
    @property
    def rgg(self) -> I8Vector3: ...
    @property
    def rgb(self) -> I8Vector3: ...
    @property
    def rga(self) -> I8Vector3: ...
    @property
    def rgo(self) -> I8Vector3: ...
    @property
    def rgl(self) -> I8Vector3: ...
    @property
    def rbr(self) -> I8Vector3: ...
    @property
    def rbg(self) -> I8Vector3: ...
    @property
    def rbb(self) -> I8Vector3: ...
    @property
    def rba(self) -> I8Vector3: ...
    @property
    def rbo(self) -> I8Vector3: ...
    @property
    def rbl(self) -> I8Vector3: ...
    @property
    def rar(self) -> I8Vector3: ...
    @property
    def rag(self) -> I8Vector3: ...
    @property
    def rab(self) -> I8Vector3: ...
    @property
    def raa(self) -> I8Vector3: ...
    @property
    def rao(self) -> I8Vector3: ...
    @property
    def ral(self) -> I8Vector3: ...
    @property
    def ror(self) -> I8Vector3: ...
    @property
    def rog(self) -> I8Vector3: ...
    @property
    def rob(self) -> I8Vector3: ...
    @property
    def roa(self) -> I8Vector3: ...
    @property
    def roo(self) -> I8Vector3: ...
    @property
    def rol(self) -> I8Vector3: ...
    @property
    def rlr(self) -> I8Vector3: ...
    @property
    def rlg(self) -> I8Vector3: ...
    @property
    def rlb(self) -> I8Vector3: ...
    @property
    def rla(self) -> I8Vector3: ...
    @property
    def rlo(self) -> I8Vector3: ...
    @property
    def rll(self) -> I8Vector3: ...
    @property
    def grr(self) -> I8Vector3: ...
    @property
    def grg(self) -> I8Vector3: ...
    @property
    def grb(self) -> I8Vector3: ...
    @property
    def gra(self) -> I8Vector3: ...
    @property
    def gro(self) -> I8Vector3: ...
    @property
    def grl(self) -> I8Vector3: ...
    @property
    def ggr(self) -> I8Vector3: ...
    @property
    def ggg(self) -> I8Vector3: ...
    @property
    def ggb(self) -> I8Vector3: ...
    @property
    def gga(self) -> I8Vector3: ...
    @property
    def ggo(self) -> I8Vector3: ...
    @property
    def ggl(self) -> I8Vector3: ...
    @property
    def gbr(self) -> I8Vector3: ...
    @property
    def gbg(self) -> I8Vector3: ...
    @property
    def gbb(self) -> I8Vector3: ...
    @property
    def gba(self) -> I8Vector3: ...
    @property
    def gbo(self) -> I8Vector3: ...
    @property
    def gbl(self) -> I8Vector3: ...
    @property
    def gar(self) -> I8Vector3: ...
    @property
    def gag(self) -> I8Vector3: ...
    @property
    def gab(self) -> I8Vector3: ...
    @property
    def gaa(self) -> I8Vector3: ...
    @property
    def gao(self) -> I8Vector3: ...
    @property
    def gal(self) -> I8Vector3: ...
    @property
    def gor(self) -> I8Vector3: ...
    @property
    def gog(self) -> I8Vector3: ...
    @property
    def gob(self) -> I8Vector3: ...
    @property
    def goa(self) -> I8Vector3: ...
    @property
    def goo(self) -> I8Vector3: ...
    @property
    def gol(self) -> I8Vector3: ...
    @property
    def glr(self) -> I8Vector3: ...
    @property
    def glg(self) -> I8Vector3: ...
    @property
    def glb(self) -> I8Vector3: ...
    @property
    def gla(self) -> I8Vector3: ...
    @property
    def glo(self) -> I8Vector3: ...
    @property
    def gll(self) -> I8Vector3: ...
    @property
    def brr(self) -> I8Vector3: ...
    @property
    def brg(self) -> I8Vector3: ...
    @property
    def brb(self) -> I8Vector3: ...
    @property
    def bra(self) -> I8Vector3: ...
    @property
    def bro(self) -> I8Vector3: ...
    @property
    def brl(self) -> I8Vector3: ...
    @property
    def bgr(self) -> I8Vector3: ...
    @property
    def bgg(self) -> I8Vector3: ...
    @property
    def bgb(self) -> I8Vector3: ...
    @property
    def bga(self) -> I8Vector3: ...
    @property
    def bgo(self) -> I8Vector3: ...
    @property
    def bgl(self) -> I8Vector3: ...
    @property
    def bbr(self) -> I8Vector3: ...
    @property
    def bbg(self) -> I8Vector3: ...
    @property
    def bbb(self) -> I8Vector3: ...
    @property
    def bba(self) -> I8Vector3: ...
    @property
    def bbo(self) -> I8Vector3: ...
    @property
    def bbl(self) -> I8Vector3: ...
    @property
    def bar(self) -> I8Vector3: ...
    @property
    def bag(self) -> I8Vector3: ...
    @property
    def bab(self) -> I8Vector3: ...
    @property
    def baa(self) -> I8Vector3: ...
    @property
    def bao(self) -> I8Vector3: ...
    @property
    def bal(self) -> I8Vector3: ...
    @property
    def bor(self) -> I8Vector3: ...
    @property
    def bog(self) -> I8Vector3: ...
    @property
    def bob(self) -> I8Vector3: ...
    @property
    def boa(self) -> I8Vector3: ...
    @property
    def boo(self) -> I8Vector3: ...
    @property
    def bol(self) -> I8Vector3: ...
    @property
    def blr(self) -> I8Vector3: ...
    @property
    def blg(self) -> I8Vector3: ...
    @property
    def blb(self) -> I8Vector3: ...
    @property
    def bla(self) -> I8Vector3: ...
    @property
    def blo(self) -> I8Vector3: ...
    @property
    def bll(self) -> I8Vector3: ...
    @property
    def arr(self) -> I8Vector3: ...
    @property
    def arg(self) -> I8Vector3: ...
    @property
    def arb(self) -> I8Vector3: ...
    @property
    def ara(self) -> I8Vector3: ...
    @property
    def aro(self) -> I8Vector3: ...
    @property
    def arl(self) -> I8Vector3: ...
    @property
    def agr(self) -> I8Vector3: ...
    @property
    def agg(self) -> I8Vector3: ...
    @property
    def agb(self) -> I8Vector3: ...
    @property
    def aga(self) -> I8Vector3: ...
    @property
    def ago(self) -> I8Vector3: ...
    @property
    def agl(self) -> I8Vector3: ...
    @property
    def abr(self) -> I8Vector3: ...
    @property
    def abg(self) -> I8Vector3: ...
    @property
    def abb(self) -> I8Vector3: ...
    @property
    def aba(self) -> I8Vector3: ...
    @property
    def abo(self) -> I8Vector3: ...
    @property
    def abl(self) -> I8Vector3: ...
    @property
    def aar(self) -> I8Vector3: ...
    @property
    def aag(self) -> I8Vector3: ...
    @property
    def aab(self) -> I8Vector3: ...
    @property
    def aaa(self) -> I8Vector3: ...
    @property
    def aao(self) -> I8Vector3: ...
    @property
    def aal(self) -> I8Vector3: ...
    @property
    def aor(self) -> I8Vector3: ...
    @property
    def aog(self) -> I8Vector3: ...
    @property
    def aob(self) -> I8Vector3: ...
    @property
    def aoa(self) -> I8Vector3: ...
    @property
    def aoo(self) -> I8Vector3: ...
    @property
    def aol(self) -> I8Vector3: ...
    @property
    def alr(self) -> I8Vector3: ...
    @property
    def alg(self) -> I8Vector3: ...
    @property
    def alb(self) -> I8Vector3: ...
    @property
    def ala(self) -> I8Vector3: ...
    @property
    def alo(self) -> I8Vector3: ...
    @property
    def all(self) -> I8Vector3: ...
    @property
    def orr(self) -> I8Vector3: ...
    @property
    def org(self) -> I8Vector3: ...
    @property
    def orb(self) -> I8Vector3: ...
    @property
    def ora(self) -> I8Vector3: ...
    @property
    def oro(self) -> I8Vector3: ...
    @property
    def orl(self) -> I8Vector3: ...
    @property
    def ogr(self) -> I8Vector3: ...
    @property
    def ogg(self) -> I8Vector3: ...
    @property
    def ogb(self) -> I8Vector3: ...
    @property
    def oga(self) -> I8Vector3: ...
    @property
    def ogo(self) -> I8Vector3: ...
    @property
    def ogl(self) -> I8Vector3: ...
    @property
    def obr(self) -> I8Vector3: ...
    @property
    def obg(self) -> I8Vector3: ...
    @property
    def obb(self) -> I8Vector3: ...
    @property
    def oba(self) -> I8Vector3: ...
    @property
    def obo(self) -> I8Vector3: ...
    @property
    def obl(self) -> I8Vector3: ...
    @property
    def oar(self) -> I8Vector3: ...
    @property
    def oag(self) -> I8Vector3: ...
    @property
    def oab(self) -> I8Vector3: ...
    @property
    def oaa(self) -> I8Vector3: ...
    @property
    def oao(self) -> I8Vector3: ...
    @property
    def oal(self) -> I8Vector3: ...
    @property
    def oor(self) -> I8Vector3: ...
    @property
    def oog(self) -> I8Vector3: ...
    @property
    def oob(self) -> I8Vector3: ...
    @property
    def ooa(self) -> I8Vector3: ...
    @property
    def olr(self) -> I8Vector3: ...
    @property
    def olg(self) -> I8Vector3: ...
    @property
    def olb(self) -> I8Vector3: ...
    @property
    def ola(self) -> I8Vector3: ...
    @property
    def lrr(self) -> I8Vector3: ...
    @property
    def lrg(self) -> I8Vector3: ...
    @property
    def lrb(self) -> I8Vector3: ...
    @property
    def lra(self) -> I8Vector3: ...
    @property
    def lro(self) -> I8Vector3: ...
    @property
    def lrl(self) -> I8Vector3: ...
    @property
    def lgr(self) -> I8Vector3: ...
    @property
    def lgg(self) -> I8Vector3: ...
    @property
    def lgb(self) -> I8Vector3: ...
    @property
    def lga(self) -> I8Vector3: ...
    @property
    def lgo(self) -> I8Vector3: ...
    @property
    def lgl(self) -> I8Vector3: ...
    @property
    def lbr(self) -> I8Vector3: ...
    @property
    def lbg(self) -> I8Vector3: ...
    @property
    def lbb(self) -> I8Vector3: ...
    @property
    def lba(self) -> I8Vector3: ...
    @property
    def lbo(self) -> I8Vector3: ...
    @property
    def lbl(self) -> I8Vector3: ...
    @property
    def lar(self) -> I8Vector3: ...
    @property
    def lag(self) -> I8Vector3: ...
    @property
    def lab(self) -> I8Vector3: ...
    @property
    def laa(self) -> I8Vector3: ...
    @property
    def lao(self) -> I8Vector3: ...
    @property
    def lal(self) -> I8Vector3: ...
    @property
    def lor(self) -> I8Vector3: ...
    @property
    def log(self) -> I8Vector3: ...
    @property
    def lob(self) -> I8Vector3: ...
    @property
    def loa(self) -> I8Vector3: ...
    @property
    def llr(self) -> I8Vector3: ...
    @property
    def llg(self) -> I8Vector3: ...
    @property
    def llb(self) -> I8Vector3: ...
    @property
    def lla(self) -> I8Vector3: ...
    @property
    def rrrr(self) -> I8Vector4: ...
    @property
    def rrrg(self) -> I8Vector4: ...
    @property
    def rrrb(self) -> I8Vector4: ...
    @property
    def rrra(self) -> I8Vector4: ...
    @property
    def rrro(self) -> I8Vector4: ...
    @property
    def rrrl(self) -> I8Vector4: ...
    @property
    def rrgr(self) -> I8Vector4: ...
    @property
    def rrgg(self) -> I8Vector4: ...
    @property
    def rrgb(self) -> I8Vector4: ...
    @property
    def rrga(self) -> I8Vector4: ...
    @property
    def rrgo(self) -> I8Vector4: ...
    @property
    def rrgl(self) -> I8Vector4: ...
    @property
    def rrbr(self) -> I8Vector4: ...
    @property
    def rrbg(self) -> I8Vector4: ...
    @property
    def rrbb(self) -> I8Vector4: ...
    @property
    def rrba(self) -> I8Vector4: ...
    @property
    def rrbo(self) -> I8Vector4: ...
    @property
    def rrbl(self) -> I8Vector4: ...
    @property
    def rrar(self) -> I8Vector4: ...
    @property
    def rrag(self) -> I8Vector4: ...
    @property
    def rrab(self) -> I8Vector4: ...
    @property
    def rraa(self) -> I8Vector4: ...
    @property
    def rrao(self) -> I8Vector4: ...
    @property
    def rral(self) -> I8Vector4: ...
    @property
    def rror(self) -> I8Vector4: ...
    @property
    def rrog(self) -> I8Vector4: ...
    @property
    def rrob(self) -> I8Vector4: ...
    @property
    def rroa(self) -> I8Vector4: ...
    @property
    def rroo(self) -> I8Vector4: ...
    @property
    def rrol(self) -> I8Vector4: ...
    @property
    def rrlr(self) -> I8Vector4: ...
    @property
    def rrlg(self) -> I8Vector4: ...
    @property
    def rrlb(self) -> I8Vector4: ...
    @property
    def rrla(self) -> I8Vector4: ...
    @property
    def rrlo(self) -> I8Vector4: ...
    @property
    def rrll(self) -> I8Vector4: ...
    @property
    def rgrr(self) -> I8Vector4: ...
    @property
    def rgrg(self) -> I8Vector4: ...
    @property
    def rgrb(self) -> I8Vector4: ...
    @property
    def rgra(self) -> I8Vector4: ...
    @property
    def rgro(self) -> I8Vector4: ...
    @property
    def rgrl(self) -> I8Vector4: ...
    @property
    def rggr(self) -> I8Vector4: ...
    @property
    def rggg(self) -> I8Vector4: ...
    @property
    def rggb(self) -> I8Vector4: ...
    @property
    def rgga(self) -> I8Vector4: ...
    @property
    def rggo(self) -> I8Vector4: ...
    @property
    def rggl(self) -> I8Vector4: ...
    @property
    def rgbr(self) -> I8Vector4: ...
    @property
    def rgbg(self) -> I8Vector4: ...
    @property
    def rgbb(self) -> I8Vector4: ...
    @property
    def rgba(self) -> I8Vector4: ...
    @property
    def rgbo(self) -> I8Vector4: ...
    @property
    def rgbl(self) -> I8Vector4: ...
    @property
    def rgar(self) -> I8Vector4: ...
    @property
    def rgag(self) -> I8Vector4: ...
    @property
    def rgab(self) -> I8Vector4: ...
    @property
    def rgaa(self) -> I8Vector4: ...
    @property
    def rgao(self) -> I8Vector4: ...
    @property
    def rgal(self) -> I8Vector4: ...
    @property
    def rgor(self) -> I8Vector4: ...
    @property
    def rgog(self) -> I8Vector4: ...
    @property
    def rgob(self) -> I8Vector4: ...
    @property
    def rgoa(self) -> I8Vector4: ...
    @property
    def rgoo(self) -> I8Vector4: ...
    @property
    def rgol(self) -> I8Vector4: ...
    @property
    def rglr(self) -> I8Vector4: ...
    @property
    def rglg(self) -> I8Vector4: ...
    @property
    def rglb(self) -> I8Vector4: ...
    @property
    def rgla(self) -> I8Vector4: ...
    @property
    def rglo(self) -> I8Vector4: ...
    @property
    def rgll(self) -> I8Vector4: ...
    @property
    def rbrr(self) -> I8Vector4: ...
    @property
    def rbrg(self) -> I8Vector4: ...
    @property
    def rbrb(self) -> I8Vector4: ...
    @property
    def rbra(self) -> I8Vector4: ...
    @property
    def rbro(self) -> I8Vector4: ...
    @property
    def rbrl(self) -> I8Vector4: ...
    @property
    def rbgr(self) -> I8Vector4: ...
    @property
    def rbgg(self) -> I8Vector4: ...
    @property
    def rbgb(self) -> I8Vector4: ...
    @property
    def rbga(self) -> I8Vector4: ...
    @property
    def rbgo(self) -> I8Vector4: ...
    @property
    def rbgl(self) -> I8Vector4: ...
    @property
    def rbbr(self) -> I8Vector4: ...
    @property
    def rbbg(self) -> I8Vector4: ...
    @property
    def rbbb(self) -> I8Vector4: ...
    @property
    def rbba(self) -> I8Vector4: ...
    @property
    def rbbo(self) -> I8Vector4: ...
    @property
    def rbbl(self) -> I8Vector4: ...
    @property
    def rbar(self) -> I8Vector4: ...
    @property
    def rbag(self) -> I8Vector4: ...
    @property
    def rbab(self) -> I8Vector4: ...
    @property
    def rbaa(self) -> I8Vector4: ...
    @property
    def rbao(self) -> I8Vector4: ...
    @property
    def rbal(self) -> I8Vector4: ...
    @property
    def rbor(self) -> I8Vector4: ...
    @property
    def rbog(self) -> I8Vector4: ...
    @property
    def rbob(self) -> I8Vector4: ...
    @property
    def rboa(self) -> I8Vector4: ...
    @property
    def rboo(self) -> I8Vector4: ...
    @property
    def rbol(self) -> I8Vector4: ...
    @property
    def rblr(self) -> I8Vector4: ...
    @property
    def rblg(self) -> I8Vector4: ...
    @property
    def rblb(self) -> I8Vector4: ...
    @property
    def rbla(self) -> I8Vector4: ...
    @property
    def rblo(self) -> I8Vector4: ...
    @property
    def rbll(self) -> I8Vector4: ...
    @property
    def rarr(self) -> I8Vector4: ...
    @property
    def rarg(self) -> I8Vector4: ...
    @property
    def rarb(self) -> I8Vector4: ...
    @property
    def rara(self) -> I8Vector4: ...
    @property
    def raro(self) -> I8Vector4: ...
    @property
    def rarl(self) -> I8Vector4: ...
    @property
    def ragr(self) -> I8Vector4: ...
    @property
    def ragg(self) -> I8Vector4: ...
    @property
    def ragb(self) -> I8Vector4: ...
    @property
    def raga(self) -> I8Vector4: ...
    @property
    def rago(self) -> I8Vector4: ...
    @property
    def ragl(self) -> I8Vector4: ...
    @property
    def rabr(self) -> I8Vector4: ...
    @property
    def rabg(self) -> I8Vector4: ...
    @property
    def rabb(self) -> I8Vector4: ...
    @property
    def raba(self) -> I8Vector4: ...
    @property
    def rabo(self) -> I8Vector4: ...
    @property
    def rabl(self) -> I8Vector4: ...
    @property
    def raar(self) -> I8Vector4: ...
    @property
    def raag(self) -> I8Vector4: ...
    @property
    def raab(self) -> I8Vector4: ...
    @property
    def raaa(self) -> I8Vector4: ...
    @property
    def raao(self) -> I8Vector4: ...
    @property
    def raal(self) -> I8Vector4: ...
    @property
    def raor(self) -> I8Vector4: ...
    @property
    def raog(self) -> I8Vector4: ...
    @property
    def raob(self) -> I8Vector4: ...
    @property
    def raoa(self) -> I8Vector4: ...
    @property
    def raoo(self) -> I8Vector4: ...
    @property
    def raol(self) -> I8Vector4: ...
    @property
    def ralr(self) -> I8Vector4: ...
    @property
    def ralg(self) -> I8Vector4: ...
    @property
    def ralb(self) -> I8Vector4: ...
    @property
    def rala(self) -> I8Vector4: ...
    @property
    def ralo(self) -> I8Vector4: ...
    @property
    def rall(self) -> I8Vector4: ...
    @property
    def rorr(self) -> I8Vector4: ...
    @property
    def rorg(self) -> I8Vector4: ...
    @property
    def rorb(self) -> I8Vector4: ...
    @property
    def rora(self) -> I8Vector4: ...
    @property
    def roro(self) -> I8Vector4: ...
    @property
    def rorl(self) -> I8Vector4: ...
    @property
    def rogr(self) -> I8Vector4: ...
    @property
    def rogg(self) -> I8Vector4: ...
    @property
    def rogb(self) -> I8Vector4: ...
    @property
    def roga(self) -> I8Vector4: ...
    @property
    def rogo(self) -> I8Vector4: ...
    @property
    def rogl(self) -> I8Vector4: ...
    @property
    def robr(self) -> I8Vector4: ...
    @property
    def robg(self) -> I8Vector4: ...
    @property
    def robb(self) -> I8Vector4: ...
    @property
    def roba(self) -> I8Vector4: ...
    @property
    def robo(self) -> I8Vector4: ...
    @property
    def robl(self) -> I8Vector4: ...
    @property
    def roar(self) -> I8Vector4: ...
    @property
    def roag(self) -> I8Vector4: ...
    @property
    def roab(self) -> I8Vector4: ...
    @property
    def roaa(self) -> I8Vector4: ...
    @property
    def roao(self) -> I8Vector4: ...
    @property
    def roal(self) -> I8Vector4: ...
    @property
    def roor(self) -> I8Vector4: ...
    @property
    def roog(self) -> I8Vector4: ...
    @property
    def roob(self) -> I8Vector4: ...
    @property
    def rooa(self) -> I8Vector4: ...
    @property
    def rooo(self) -> I8Vector4: ...
    @property
    def rool(self) -> I8Vector4: ...
    @property
    def rolr(self) -> I8Vector4: ...
    @property
    def rolg(self) -> I8Vector4: ...
    @property
    def rolb(self) -> I8Vector4: ...
    @property
    def rola(self) -> I8Vector4: ...
    @property
    def rolo(self) -> I8Vector4: ...
    @property
    def roll(self) -> I8Vector4: ...
    @property
    def rlrr(self) -> I8Vector4: ...
    @property
    def rlrg(self) -> I8Vector4: ...
    @property
    def rlrb(self) -> I8Vector4: ...
    @property
    def rlra(self) -> I8Vector4: ...
    @property
    def rlro(self) -> I8Vector4: ...
    @property
    def rlrl(self) -> I8Vector4: ...
    @property
    def rlgr(self) -> I8Vector4: ...
    @property
    def rlgg(self) -> I8Vector4: ...
    @property
    def rlgb(self) -> I8Vector4: ...
    @property
    def rlga(self) -> I8Vector4: ...
    @property
    def rlgo(self) -> I8Vector4: ...
    @property
    def rlgl(self) -> I8Vector4: ...
    @property
    def rlbr(self) -> I8Vector4: ...
    @property
    def rlbg(self) -> I8Vector4: ...
    @property
    def rlbb(self) -> I8Vector4: ...
    @property
    def rlba(self) -> I8Vector4: ...
    @property
    def rlbo(self) -> I8Vector4: ...
    @property
    def rlbl(self) -> I8Vector4: ...
    @property
    def rlar(self) -> I8Vector4: ...
    @property
    def rlag(self) -> I8Vector4: ...
    @property
    def rlab(self) -> I8Vector4: ...
    @property
    def rlaa(self) -> I8Vector4: ...
    @property
    def rlao(self) -> I8Vector4: ...
    @property
    def rlal(self) -> I8Vector4: ...
    @property
    def rlor(self) -> I8Vector4: ...
    @property
    def rlog(self) -> I8Vector4: ...
    @property
    def rlob(self) -> I8Vector4: ...
    @property
    def rloa(self) -> I8Vector4: ...
    @property
    def rloo(self) -> I8Vector4: ...
    @property
    def rlol(self) -> I8Vector4: ...
    @property
    def rllr(self) -> I8Vector4: ...
    @property
    def rllg(self) -> I8Vector4: ...
    @property
    def rllb(self) -> I8Vector4: ...
    @property
    def rlla(self) -> I8Vector4: ...
    @property
    def rllo(self) -> I8Vector4: ...
    @property
    def rlll(self) -> I8Vector4: ...
    @property
    def grrr(self) -> I8Vector4: ...
    @property
    def grrg(self) -> I8Vector4: ...
    @property
    def grrb(self) -> I8Vector4: ...
    @property
    def grra(self) -> I8Vector4: ...
    @property
    def grro(self) -> I8Vector4: ...
    @property
    def grrl(self) -> I8Vector4: ...
    @property
    def grgr(self) -> I8Vector4: ...
    @property
    def grgg(self) -> I8Vector4: ...
    @property
    def grgb(self) -> I8Vector4: ...
    @property
    def grga(self) -> I8Vector4: ...
    @property
    def grgo(self) -> I8Vector4: ...
    @property
    def grgl(self) -> I8Vector4: ...
    @property
    def grbr(self) -> I8Vector4: ...
    @property
    def grbg(self) -> I8Vector4: ...
    @property
    def grbb(self) -> I8Vector4: ...
    @property
    def grba(self) -> I8Vector4: ...
    @property
    def grbo(self) -> I8Vector4: ...
    @property
    def grbl(self) -> I8Vector4: ...
    @property
    def grar(self) -> I8Vector4: ...
    @property
    def grag(self) -> I8Vector4: ...
    @property
    def grab(self) -> I8Vector4: ...
    @property
    def graa(self) -> I8Vector4: ...
    @property
    def grao(self) -> I8Vector4: ...
    @property
    def gral(self) -> I8Vector4: ...
    @property
    def gror(self) -> I8Vector4: ...
    @property
    def grog(self) -> I8Vector4: ...
    @property
    def grob(self) -> I8Vector4: ...
    @property
    def groa(self) -> I8Vector4: ...
    @property
    def groo(self) -> I8Vector4: ...
    @property
    def grol(self) -> I8Vector4: ...
    @property
    def grlr(self) -> I8Vector4: ...
    @property
    def grlg(self) -> I8Vector4: ...
    @property
    def grlb(self) -> I8Vector4: ...
    @property
    def grla(self) -> I8Vector4: ...
    @property
    def grlo(self) -> I8Vector4: ...
    @property
    def grll(self) -> I8Vector4: ...
    @property
    def ggrr(self) -> I8Vector4: ...
    @property
    def ggrg(self) -> I8Vector4: ...
    @property
    def ggrb(self) -> I8Vector4: ...
    @property
    def ggra(self) -> I8Vector4: ...
    @property
    def ggro(self) -> I8Vector4: ...
    @property
    def ggrl(self) -> I8Vector4: ...
    @property
    def gggr(self) -> I8Vector4: ...
    @property
    def gggg(self) -> I8Vector4: ...
    @property
    def gggb(self) -> I8Vector4: ...
    @property
    def ggga(self) -> I8Vector4: ...
    @property
    def gggo(self) -> I8Vector4: ...
    @property
    def gggl(self) -> I8Vector4: ...
    @property
    def ggbr(self) -> I8Vector4: ...
    @property
    def ggbg(self) -> I8Vector4: ...
    @property
    def ggbb(self) -> I8Vector4: ...
    @property
    def ggba(self) -> I8Vector4: ...
    @property
    def ggbo(self) -> I8Vector4: ...
    @property
    def ggbl(self) -> I8Vector4: ...
    @property
    def ggar(self) -> I8Vector4: ...
    @property
    def ggag(self) -> I8Vector4: ...
    @property
    def ggab(self) -> I8Vector4: ...
    @property
    def ggaa(self) -> I8Vector4: ...
    @property
    def ggao(self) -> I8Vector4: ...
    @property
    def ggal(self) -> I8Vector4: ...
    @property
    def ggor(self) -> I8Vector4: ...
    @property
    def ggog(self) -> I8Vector4: ...
    @property
    def ggob(self) -> I8Vector4: ...
    @property
    def ggoa(self) -> I8Vector4: ...
    @property
    def ggoo(self) -> I8Vector4: ...
    @property
    def ggol(self) -> I8Vector4: ...
    @property
    def gglr(self) -> I8Vector4: ...
    @property
    def gglg(self) -> I8Vector4: ...
    @property
    def gglb(self) -> I8Vector4: ...
    @property
    def ggla(self) -> I8Vector4: ...
    @property
    def gglo(self) -> I8Vector4: ...
    @property
    def ggll(self) -> I8Vector4: ...
    @property
    def gbrr(self) -> I8Vector4: ...
    @property
    def gbrg(self) -> I8Vector4: ...
    @property
    def gbrb(self) -> I8Vector4: ...
    @property
    def gbra(self) -> I8Vector4: ...
    @property
    def gbro(self) -> I8Vector4: ...
    @property
    def gbrl(self) -> I8Vector4: ...
    @property
    def gbgr(self) -> I8Vector4: ...
    @property
    def gbgg(self) -> I8Vector4: ...
    @property
    def gbgb(self) -> I8Vector4: ...
    @property
    def gbga(self) -> I8Vector4: ...
    @property
    def gbgo(self) -> I8Vector4: ...
    @property
    def gbgl(self) -> I8Vector4: ...
    @property
    def gbbr(self) -> I8Vector4: ...
    @property
    def gbbg(self) -> I8Vector4: ...
    @property
    def gbbb(self) -> I8Vector4: ...
    @property
    def gbba(self) -> I8Vector4: ...
    @property
    def gbbo(self) -> I8Vector4: ...
    @property
    def gbbl(self) -> I8Vector4: ...
    @property
    def gbar(self) -> I8Vector4: ...
    @property
    def gbag(self) -> I8Vector4: ...
    @property
    def gbab(self) -> I8Vector4: ...
    @property
    def gbaa(self) -> I8Vector4: ...
    @property
    def gbao(self) -> I8Vector4: ...
    @property
    def gbal(self) -> I8Vector4: ...
    @property
    def gbor(self) -> I8Vector4: ...
    @property
    def gbog(self) -> I8Vector4: ...
    @property
    def gbob(self) -> I8Vector4: ...
    @property
    def gboa(self) -> I8Vector4: ...
    @property
    def gboo(self) -> I8Vector4: ...
    @property
    def gbol(self) -> I8Vector4: ...
    @property
    def gblr(self) -> I8Vector4: ...
    @property
    def gblg(self) -> I8Vector4: ...
    @property
    def gblb(self) -> I8Vector4: ...
    @property
    def gbla(self) -> I8Vector4: ...
    @property
    def gblo(self) -> I8Vector4: ...
    @property
    def gbll(self) -> I8Vector4: ...
    @property
    def garr(self) -> I8Vector4: ...
    @property
    def garg(self) -> I8Vector4: ...
    @property
    def garb(self) -> I8Vector4: ...
    @property
    def gara(self) -> I8Vector4: ...
    @property
    def garo(self) -> I8Vector4: ...
    @property
    def garl(self) -> I8Vector4: ...
    @property
    def gagr(self) -> I8Vector4: ...
    @property
    def gagg(self) -> I8Vector4: ...
    @property
    def gagb(self) -> I8Vector4: ...
    @property
    def gaga(self) -> I8Vector4: ...
    @property
    def gago(self) -> I8Vector4: ...
    @property
    def gagl(self) -> I8Vector4: ...
    @property
    def gabr(self) -> I8Vector4: ...
    @property
    def gabg(self) -> I8Vector4: ...
    @property
    def gabb(self) -> I8Vector4: ...
    @property
    def gaba(self) -> I8Vector4: ...
    @property
    def gabo(self) -> I8Vector4: ...
    @property
    def gabl(self) -> I8Vector4: ...
    @property
    def gaar(self) -> I8Vector4: ...
    @property
    def gaag(self) -> I8Vector4: ...
    @property
    def gaab(self) -> I8Vector4: ...
    @property
    def gaaa(self) -> I8Vector4: ...
    @property
    def gaao(self) -> I8Vector4: ...
    @property
    def gaal(self) -> I8Vector4: ...
    @property
    def gaor(self) -> I8Vector4: ...
    @property
    def gaog(self) -> I8Vector4: ...
    @property
    def gaob(self) -> I8Vector4: ...
    @property
    def gaoa(self) -> I8Vector4: ...
    @property
    def gaoo(self) -> I8Vector4: ...
    @property
    def gaol(self) -> I8Vector4: ...
    @property
    def galr(self) -> I8Vector4: ...
    @property
    def galg(self) -> I8Vector4: ...
    @property
    def galb(self) -> I8Vector4: ...
    @property
    def gala(self) -> I8Vector4: ...
    @property
    def galo(self) -> I8Vector4: ...
    @property
    def gall(self) -> I8Vector4: ...
    @property
    def gorr(self) -> I8Vector4: ...
    @property
    def gorg(self) -> I8Vector4: ...
    @property
    def gorb(self) -> I8Vector4: ...
    @property
    def gora(self) -> I8Vector4: ...
    @property
    def goro(self) -> I8Vector4: ...
    @property
    def gorl(self) -> I8Vector4: ...
    @property
    def gogr(self) -> I8Vector4: ...
    @property
    def gogg(self) -> I8Vector4: ...
    @property
    def gogb(self) -> I8Vector4: ...
    @property
    def goga(self) -> I8Vector4: ...
    @property
    def gogo(self) -> I8Vector4: ...
    @property
    def gogl(self) -> I8Vector4: ...
    @property
    def gobr(self) -> I8Vector4: ...
    @property
    def gobg(self) -> I8Vector4: ...
    @property
    def gobb(self) -> I8Vector4: ...
    @property
    def goba(self) -> I8Vector4: ...
    @property
    def gobo(self) -> I8Vector4: ...
    @property
    def gobl(self) -> I8Vector4: ...
    @property
    def goar(self) -> I8Vector4: ...
    @property
    def goag(self) -> I8Vector4: ...
    @property
    def goab(self) -> I8Vector4: ...
    @property
    def goaa(self) -> I8Vector4: ...
    @property
    def goao(self) -> I8Vector4: ...
    @property
    def goal(self) -> I8Vector4: ...
    @property
    def goor(self) -> I8Vector4: ...
    @property
    def goog(self) -> I8Vector4: ...
    @property
    def goob(self) -> I8Vector4: ...
    @property
    def gooa(self) -> I8Vector4: ...
    @property
    def gooo(self) -> I8Vector4: ...
    @property
    def gool(self) -> I8Vector4: ...
    @property
    def golr(self) -> I8Vector4: ...
    @property
    def golg(self) -> I8Vector4: ...
    @property
    def golb(self) -> I8Vector4: ...
    @property
    def gola(self) -> I8Vector4: ...
    @property
    def golo(self) -> I8Vector4: ...
    @property
    def goll(self) -> I8Vector4: ...
    @property
    def glrr(self) -> I8Vector4: ...
    @property
    def glrg(self) -> I8Vector4: ...
    @property
    def glrb(self) -> I8Vector4: ...
    @property
    def glra(self) -> I8Vector4: ...
    @property
    def glro(self) -> I8Vector4: ...
    @property
    def glrl(self) -> I8Vector4: ...
    @property
    def glgr(self) -> I8Vector4: ...
    @property
    def glgg(self) -> I8Vector4: ...
    @property
    def glgb(self) -> I8Vector4: ...
    @property
    def glga(self) -> I8Vector4: ...
    @property
    def glgo(self) -> I8Vector4: ...
    @property
    def glgl(self) -> I8Vector4: ...
    @property
    def glbr(self) -> I8Vector4: ...
    @property
    def glbg(self) -> I8Vector4: ...
    @property
    def glbb(self) -> I8Vector4: ...
    @property
    def glba(self) -> I8Vector4: ...
    @property
    def glbo(self) -> I8Vector4: ...
    @property
    def glbl(self) -> I8Vector4: ...
    @property
    def glar(self) -> I8Vector4: ...
    @property
    def glag(self) -> I8Vector4: ...
    @property
    def glab(self) -> I8Vector4: ...
    @property
    def glaa(self) -> I8Vector4: ...
    @property
    def glao(self) -> I8Vector4: ...
    @property
    def glal(self) -> I8Vector4: ...
    @property
    def glor(self) -> I8Vector4: ...
    @property
    def glog(self) -> I8Vector4: ...
    @property
    def glob(self) -> I8Vector4: ...
    @property
    def gloa(self) -> I8Vector4: ...
    @property
    def gloo(self) -> I8Vector4: ...
    @property
    def glol(self) -> I8Vector4: ...
    @property
    def gllr(self) -> I8Vector4: ...
    @property
    def gllg(self) -> I8Vector4: ...
    @property
    def gllb(self) -> I8Vector4: ...
    @property
    def glla(self) -> I8Vector4: ...
    @property
    def gllo(self) -> I8Vector4: ...
    @property
    def glll(self) -> I8Vector4: ...
    @property
    def brrr(self) -> I8Vector4: ...
    @property
    def brrg(self) -> I8Vector4: ...
    @property
    def brrb(self) -> I8Vector4: ...
    @property
    def brra(self) -> I8Vector4: ...
    @property
    def brro(self) -> I8Vector4: ...
    @property
    def brrl(self) -> I8Vector4: ...
    @property
    def brgr(self) -> I8Vector4: ...
    @property
    def brgg(self) -> I8Vector4: ...
    @property
    def brgb(self) -> I8Vector4: ...
    @property
    def brga(self) -> I8Vector4: ...
    @property
    def brgo(self) -> I8Vector4: ...
    @property
    def brgl(self) -> I8Vector4: ...
    @property
    def brbr(self) -> I8Vector4: ...
    @property
    def brbg(self) -> I8Vector4: ...
    @property
    def brbb(self) -> I8Vector4: ...
    @property
    def brba(self) -> I8Vector4: ...
    @property
    def brbo(self) -> I8Vector4: ...
    @property
    def brbl(self) -> I8Vector4: ...
    @property
    def brar(self) -> I8Vector4: ...
    @property
    def brag(self) -> I8Vector4: ...
    @property
    def brab(self) -> I8Vector4: ...
    @property
    def braa(self) -> I8Vector4: ...
    @property
    def brao(self) -> I8Vector4: ...
    @property
    def bral(self) -> I8Vector4: ...
    @property
    def bror(self) -> I8Vector4: ...
    @property
    def brog(self) -> I8Vector4: ...
    @property
    def brob(self) -> I8Vector4: ...
    @property
    def broa(self) -> I8Vector4: ...
    @property
    def broo(self) -> I8Vector4: ...
    @property
    def brol(self) -> I8Vector4: ...
    @property
    def brlr(self) -> I8Vector4: ...
    @property
    def brlg(self) -> I8Vector4: ...
    @property
    def brlb(self) -> I8Vector4: ...
    @property
    def brla(self) -> I8Vector4: ...
    @property
    def brlo(self) -> I8Vector4: ...
    @property
    def brll(self) -> I8Vector4: ...
    @property
    def bgrr(self) -> I8Vector4: ...
    @property
    def bgrg(self) -> I8Vector4: ...
    @property
    def bgrb(self) -> I8Vector4: ...
    @property
    def bgra(self) -> I8Vector4: ...
    @property
    def bgro(self) -> I8Vector4: ...
    @property
    def bgrl(self) -> I8Vector4: ...
    @property
    def bggr(self) -> I8Vector4: ...
    @property
    def bggg(self) -> I8Vector4: ...
    @property
    def bggb(self) -> I8Vector4: ...
    @property
    def bgga(self) -> I8Vector4: ...
    @property
    def bggo(self) -> I8Vector4: ...
    @property
    def bggl(self) -> I8Vector4: ...
    @property
    def bgbr(self) -> I8Vector4: ...
    @property
    def bgbg(self) -> I8Vector4: ...
    @property
    def bgbb(self) -> I8Vector4: ...
    @property
    def bgba(self) -> I8Vector4: ...
    @property
    def bgbo(self) -> I8Vector4: ...
    @property
    def bgbl(self) -> I8Vector4: ...
    @property
    def bgar(self) -> I8Vector4: ...
    @property
    def bgag(self) -> I8Vector4: ...
    @property
    def bgab(self) -> I8Vector4: ...
    @property
    def bgaa(self) -> I8Vector4: ...
    @property
    def bgao(self) -> I8Vector4: ...
    @property
    def bgal(self) -> I8Vector4: ...
    @property
    def bgor(self) -> I8Vector4: ...
    @property
    def bgog(self) -> I8Vector4: ...
    @property
    def bgob(self) -> I8Vector4: ...
    @property
    def bgoa(self) -> I8Vector4: ...
    @property
    def bgoo(self) -> I8Vector4: ...
    @property
    def bgol(self) -> I8Vector4: ...
    @property
    def bglr(self) -> I8Vector4: ...
    @property
    def bglg(self) -> I8Vector4: ...
    @property
    def bglb(self) -> I8Vector4: ...
    @property
    def bgla(self) -> I8Vector4: ...
    @property
    def bglo(self) -> I8Vector4: ...
    @property
    def bgll(self) -> I8Vector4: ...
    @property
    def bbrr(self) -> I8Vector4: ...
    @property
    def bbrg(self) -> I8Vector4: ...
    @property
    def bbrb(self) -> I8Vector4: ...
    @property
    def bbra(self) -> I8Vector4: ...
    @property
    def bbro(self) -> I8Vector4: ...
    @property
    def bbrl(self) -> I8Vector4: ...
    @property
    def bbgr(self) -> I8Vector4: ...
    @property
    def bbgg(self) -> I8Vector4: ...
    @property
    def bbgb(self) -> I8Vector4: ...
    @property
    def bbga(self) -> I8Vector4: ...
    @property
    def bbgo(self) -> I8Vector4: ...
    @property
    def bbgl(self) -> I8Vector4: ...
    @property
    def bbbr(self) -> I8Vector4: ...
    @property
    def bbbg(self) -> I8Vector4: ...
    @property
    def bbbb(self) -> I8Vector4: ...
    @property
    def bbba(self) -> I8Vector4: ...
    @property
    def bbbo(self) -> I8Vector4: ...
    @property
    def bbbl(self) -> I8Vector4: ...
    @property
    def bbar(self) -> I8Vector4: ...
    @property
    def bbag(self) -> I8Vector4: ...
    @property
    def bbab(self) -> I8Vector4: ...
    @property
    def bbaa(self) -> I8Vector4: ...
    @property
    def bbao(self) -> I8Vector4: ...
    @property
    def bbal(self) -> I8Vector4: ...
    @property
    def bbor(self) -> I8Vector4: ...
    @property
    def bbog(self) -> I8Vector4: ...
    @property
    def bbob(self) -> I8Vector4: ...
    @property
    def bboa(self) -> I8Vector4: ...
    @property
    def bboo(self) -> I8Vector4: ...
    @property
    def bbol(self) -> I8Vector4: ...
    @property
    def bblr(self) -> I8Vector4: ...
    @property
    def bblg(self) -> I8Vector4: ...
    @property
    def bblb(self) -> I8Vector4: ...
    @property
    def bbla(self) -> I8Vector4: ...
    @property
    def bblo(self) -> I8Vector4: ...
    @property
    def bbll(self) -> I8Vector4: ...
    @property
    def barr(self) -> I8Vector4: ...
    @property
    def barg(self) -> I8Vector4: ...
    @property
    def barb(self) -> I8Vector4: ...
    @property
    def bara(self) -> I8Vector4: ...
    @property
    def baro(self) -> I8Vector4: ...
    @property
    def barl(self) -> I8Vector4: ...
    @property
    def bagr(self) -> I8Vector4: ...
    @property
    def bagg(self) -> I8Vector4: ...
    @property
    def bagb(self) -> I8Vector4: ...
    @property
    def baga(self) -> I8Vector4: ...
    @property
    def bago(self) -> I8Vector4: ...
    @property
    def bagl(self) -> I8Vector4: ...
    @property
    def babr(self) -> I8Vector4: ...
    @property
    def babg(self) -> I8Vector4: ...
    @property
    def babb(self) -> I8Vector4: ...
    @property
    def baba(self) -> I8Vector4: ...
    @property
    def babo(self) -> I8Vector4: ...
    @property
    def babl(self) -> I8Vector4: ...
    @property
    def baar(self) -> I8Vector4: ...
    @property
    def baag(self) -> I8Vector4: ...
    @property
    def baab(self) -> I8Vector4: ...
    @property
    def baaa(self) -> I8Vector4: ...
    @property
    def baao(self) -> I8Vector4: ...
    @property
    def baal(self) -> I8Vector4: ...
    @property
    def baor(self) -> I8Vector4: ...
    @property
    def baog(self) -> I8Vector4: ...
    @property
    def baob(self) -> I8Vector4: ...
    @property
    def baoa(self) -> I8Vector4: ...
    @property
    def baoo(self) -> I8Vector4: ...
    @property
    def baol(self) -> I8Vector4: ...
    @property
    def balr(self) -> I8Vector4: ...
    @property
    def balg(self) -> I8Vector4: ...
    @property
    def balb(self) -> I8Vector4: ...
    @property
    def bala(self) -> I8Vector4: ...
    @property
    def balo(self) -> I8Vector4: ...
    @property
    def ball(self) -> I8Vector4: ...
    @property
    def borr(self) -> I8Vector4: ...
    @property
    def borg(self) -> I8Vector4: ...
    @property
    def borb(self) -> I8Vector4: ...
    @property
    def bora(self) -> I8Vector4: ...
    @property
    def boro(self) -> I8Vector4: ...
    @property
    def borl(self) -> I8Vector4: ...
    @property
    def bogr(self) -> I8Vector4: ...
    @property
    def bogg(self) -> I8Vector4: ...
    @property
    def bogb(self) -> I8Vector4: ...
    @property
    def boga(self) -> I8Vector4: ...
    @property
    def bogo(self) -> I8Vector4: ...
    @property
    def bogl(self) -> I8Vector4: ...
    @property
    def bobr(self) -> I8Vector4: ...
    @property
    def bobg(self) -> I8Vector4: ...
    @property
    def bobb(self) -> I8Vector4: ...
    @property
    def boba(self) -> I8Vector4: ...
    @property
    def bobo(self) -> I8Vector4: ...
    @property
    def bobl(self) -> I8Vector4: ...
    @property
    def boar(self) -> I8Vector4: ...
    @property
    def boag(self) -> I8Vector4: ...
    @property
    def boab(self) -> I8Vector4: ...
    @property
    def boaa(self) -> I8Vector4: ...
    @property
    def boao(self) -> I8Vector4: ...
    @property
    def boal(self) -> I8Vector4: ...
    @property
    def boor(self) -> I8Vector4: ...
    @property
    def boog(self) -> I8Vector4: ...
    @property
    def boob(self) -> I8Vector4: ...
    @property
    def booa(self) -> I8Vector4: ...
    @property
    def booo(self) -> I8Vector4: ...
    @property
    def bool(self) -> I8Vector4: ...
    @property
    def bolr(self) -> I8Vector4: ...
    @property
    def bolg(self) -> I8Vector4: ...
    @property
    def bolb(self) -> I8Vector4: ...
    @property
    def bola(self) -> I8Vector4: ...
    @property
    def bolo(self) -> I8Vector4: ...
    @property
    def boll(self) -> I8Vector4: ...
    @property
    def blrr(self) -> I8Vector4: ...
    @property
    def blrg(self) -> I8Vector4: ...
    @property
    def blrb(self) -> I8Vector4: ...
    @property
    def blra(self) -> I8Vector4: ...
    @property
    def blro(self) -> I8Vector4: ...
    @property
    def blrl(self) -> I8Vector4: ...
    @property
    def blgr(self) -> I8Vector4: ...
    @property
    def blgg(self) -> I8Vector4: ...
    @property
    def blgb(self) -> I8Vector4: ...
    @property
    def blga(self) -> I8Vector4: ...
    @property
    def blgo(self) -> I8Vector4: ...
    @property
    def blgl(self) -> I8Vector4: ...
    @property
    def blbr(self) -> I8Vector4: ...
    @property
    def blbg(self) -> I8Vector4: ...
    @property
    def blbb(self) -> I8Vector4: ...
    @property
    def blba(self) -> I8Vector4: ...
    @property
    def blbo(self) -> I8Vector4: ...
    @property
    def blbl(self) -> I8Vector4: ...
    @property
    def blar(self) -> I8Vector4: ...
    @property
    def blag(self) -> I8Vector4: ...
    @property
    def blab(self) -> I8Vector4: ...
    @property
    def blaa(self) -> I8Vector4: ...
    @property
    def blao(self) -> I8Vector4: ...
    @property
    def blal(self) -> I8Vector4: ...
    @property
    def blor(self) -> I8Vector4: ...
    @property
    def blog(self) -> I8Vector4: ...
    @property
    def blob(self) -> I8Vector4: ...
    @property
    def bloa(self) -> I8Vector4: ...
    @property
    def bloo(self) -> I8Vector4: ...
    @property
    def blol(self) -> I8Vector4: ...
    @property
    def bllr(self) -> I8Vector4: ...
    @property
    def bllg(self) -> I8Vector4: ...
    @property
    def bllb(self) -> I8Vector4: ...
    @property
    def blla(self) -> I8Vector4: ...
    @property
    def bllo(self) -> I8Vector4: ...
    @property
    def blll(self) -> I8Vector4: ...
    @property
    def arrr(self) -> I8Vector4: ...
    @property
    def arrg(self) -> I8Vector4: ...
    @property
    def arrb(self) -> I8Vector4: ...
    @property
    def arra(self) -> I8Vector4: ...
    @property
    def arro(self) -> I8Vector4: ...
    @property
    def arrl(self) -> I8Vector4: ...
    @property
    def argr(self) -> I8Vector4: ...
    @property
    def argg(self) -> I8Vector4: ...
    @property
    def argb(self) -> I8Vector4: ...
    @property
    def arga(self) -> I8Vector4: ...
    @property
    def argo(self) -> I8Vector4: ...
    @property
    def argl(self) -> I8Vector4: ...
    @property
    def arbr(self) -> I8Vector4: ...
    @property
    def arbg(self) -> I8Vector4: ...
    @property
    def arbb(self) -> I8Vector4: ...
    @property
    def arba(self) -> I8Vector4: ...
    @property
    def arbo(self) -> I8Vector4: ...
    @property
    def arbl(self) -> I8Vector4: ...
    @property
    def arar(self) -> I8Vector4: ...
    @property
    def arag(self) -> I8Vector4: ...
    @property
    def arab(self) -> I8Vector4: ...
    @property
    def araa(self) -> I8Vector4: ...
    @property
    def arao(self) -> I8Vector4: ...
    @property
    def aral(self) -> I8Vector4: ...
    @property
    def aror(self) -> I8Vector4: ...
    @property
    def arog(self) -> I8Vector4: ...
    @property
    def arob(self) -> I8Vector4: ...
    @property
    def aroa(self) -> I8Vector4: ...
    @property
    def aroo(self) -> I8Vector4: ...
    @property
    def arol(self) -> I8Vector4: ...
    @property
    def arlr(self) -> I8Vector4: ...
    @property
    def arlg(self) -> I8Vector4: ...
    @property
    def arlb(self) -> I8Vector4: ...
    @property
    def arla(self) -> I8Vector4: ...
    @property
    def arlo(self) -> I8Vector4: ...
    @property
    def arll(self) -> I8Vector4: ...
    @property
    def agrr(self) -> I8Vector4: ...
    @property
    def agrg(self) -> I8Vector4: ...
    @property
    def agrb(self) -> I8Vector4: ...
    @property
    def agra(self) -> I8Vector4: ...
    @property
    def agro(self) -> I8Vector4: ...
    @property
    def agrl(self) -> I8Vector4: ...
    @property
    def aggr(self) -> I8Vector4: ...
    @property
    def aggg(self) -> I8Vector4: ...
    @property
    def aggb(self) -> I8Vector4: ...
    @property
    def agga(self) -> I8Vector4: ...
    @property
    def aggo(self) -> I8Vector4: ...
    @property
    def aggl(self) -> I8Vector4: ...
    @property
    def agbr(self) -> I8Vector4: ...
    @property
    def agbg(self) -> I8Vector4: ...
    @property
    def agbb(self) -> I8Vector4: ...
    @property
    def agba(self) -> I8Vector4: ...
    @property
    def agbo(self) -> I8Vector4: ...
    @property
    def agbl(self) -> I8Vector4: ...
    @property
    def agar(self) -> I8Vector4: ...
    @property
    def agag(self) -> I8Vector4: ...
    @property
    def agab(self) -> I8Vector4: ...
    @property
    def agaa(self) -> I8Vector4: ...
    @property
    def agao(self) -> I8Vector4: ...
    @property
    def agal(self) -> I8Vector4: ...
    @property
    def agor(self) -> I8Vector4: ...
    @property
    def agog(self) -> I8Vector4: ...
    @property
    def agob(self) -> I8Vector4: ...
    @property
    def agoa(self) -> I8Vector4: ...
    @property
    def agoo(self) -> I8Vector4: ...
    @property
    def agol(self) -> I8Vector4: ...
    @property
    def aglr(self) -> I8Vector4: ...
    @property
    def aglg(self) -> I8Vector4: ...
    @property
    def aglb(self) -> I8Vector4: ...
    @property
    def agla(self) -> I8Vector4: ...
    @property
    def aglo(self) -> I8Vector4: ...
    @property
    def agll(self) -> I8Vector4: ...
    @property
    def abrr(self) -> I8Vector4: ...
    @property
    def abrg(self) -> I8Vector4: ...
    @property
    def abrb(self) -> I8Vector4: ...
    @property
    def abra(self) -> I8Vector4: ...
    @property
    def abro(self) -> I8Vector4: ...
    @property
    def abrl(self) -> I8Vector4: ...
    @property
    def abgr(self) -> I8Vector4: ...
    @property
    def abgg(self) -> I8Vector4: ...
    @property
    def abgb(self) -> I8Vector4: ...
    @property
    def abga(self) -> I8Vector4: ...
    @property
    def abgo(self) -> I8Vector4: ...
    @property
    def abgl(self) -> I8Vector4: ...
    @property
    def abbr(self) -> I8Vector4: ...
    @property
    def abbg(self) -> I8Vector4: ...
    @property
    def abbb(self) -> I8Vector4: ...
    @property
    def abba(self) -> I8Vector4: ...
    @property
    def abbo(self) -> I8Vector4: ...
    @property
    def abbl(self) -> I8Vector4: ...
    @property
    def abar(self) -> I8Vector4: ...
    @property
    def abag(self) -> I8Vector4: ...
    @property
    def abab(self) -> I8Vector4: ...
    @property
    def abaa(self) -> I8Vector4: ...
    @property
    def abao(self) -> I8Vector4: ...
    @property
    def abal(self) -> I8Vector4: ...
    @property
    def abor(self) -> I8Vector4: ...
    @property
    def abog(self) -> I8Vector4: ...
    @property
    def abob(self) -> I8Vector4: ...
    @property
    def aboa(self) -> I8Vector4: ...
    @property
    def aboo(self) -> I8Vector4: ...
    @property
    def abol(self) -> I8Vector4: ...
    @property
    def ablr(self) -> I8Vector4: ...
    @property
    def ablg(self) -> I8Vector4: ...
    @property
    def ablb(self) -> I8Vector4: ...
    @property
    def abla(self) -> I8Vector4: ...
    @property
    def ablo(self) -> I8Vector4: ...
    @property
    def abll(self) -> I8Vector4: ...
    @property
    def aarr(self) -> I8Vector4: ...
    @property
    def aarg(self) -> I8Vector4: ...
    @property
    def aarb(self) -> I8Vector4: ...
    @property
    def aara(self) -> I8Vector4: ...
    @property
    def aaro(self) -> I8Vector4: ...
    @property
    def aarl(self) -> I8Vector4: ...
    @property
    def aagr(self) -> I8Vector4: ...
    @property
    def aagg(self) -> I8Vector4: ...
    @property
    def aagb(self) -> I8Vector4: ...
    @property
    def aaga(self) -> I8Vector4: ...
    @property
    def aago(self) -> I8Vector4: ...
    @property
    def aagl(self) -> I8Vector4: ...
    @property
    def aabr(self) -> I8Vector4: ...
    @property
    def aabg(self) -> I8Vector4: ...
    @property
    def aabb(self) -> I8Vector4: ...
    @property
    def aaba(self) -> I8Vector4: ...
    @property
    def aabo(self) -> I8Vector4: ...
    @property
    def aabl(self) -> I8Vector4: ...
    @property
    def aaar(self) -> I8Vector4: ...
    @property
    def aaag(self) -> I8Vector4: ...
    @property
    def aaab(self) -> I8Vector4: ...
    @property
    def aaaa(self) -> I8Vector4: ...
    @property
    def aaao(self) -> I8Vector4: ...
    @property
    def aaal(self) -> I8Vector4: ...
    @property
    def aaor(self) -> I8Vector4: ...
    @property
    def aaog(self) -> I8Vector4: ...
    @property
    def aaob(self) -> I8Vector4: ...
    @property
    def aaoa(self) -> I8Vector4: ...
    @property
    def aaoo(self) -> I8Vector4: ...
    @property
    def aaol(self) -> I8Vector4: ...
    @property
    def aalr(self) -> I8Vector4: ...
    @property
    def aalg(self) -> I8Vector4: ...
    @property
    def aalb(self) -> I8Vector4: ...
    @property
    def aala(self) -> I8Vector4: ...
    @property
    def aalo(self) -> I8Vector4: ...
    @property
    def aall(self) -> I8Vector4: ...
    @property
    def aorr(self) -> I8Vector4: ...
    @property
    def aorg(self) -> I8Vector4: ...
    @property
    def aorb(self) -> I8Vector4: ...
    @property
    def aora(self) -> I8Vector4: ...
    @property
    def aoro(self) -> I8Vector4: ...
    @property
    def aorl(self) -> I8Vector4: ...
    @property
    def aogr(self) -> I8Vector4: ...
    @property
    def aogg(self) -> I8Vector4: ...
    @property
    def aogb(self) -> I8Vector4: ...
    @property
    def aoga(self) -> I8Vector4: ...
    @property
    def aogo(self) -> I8Vector4: ...
    @property
    def aogl(self) -> I8Vector4: ...
    @property
    def aobr(self) -> I8Vector4: ...
    @property
    def aobg(self) -> I8Vector4: ...
    @property
    def aobb(self) -> I8Vector4: ...
    @property
    def aoba(self) -> I8Vector4: ...
    @property
    def aobo(self) -> I8Vector4: ...
    @property
    def aobl(self) -> I8Vector4: ...
    @property
    def aoar(self) -> I8Vector4: ...
    @property
    def aoag(self) -> I8Vector4: ...
    @property
    def aoab(self) -> I8Vector4: ...
    @property
    def aoaa(self) -> I8Vector4: ...
    @property
    def aoao(self) -> I8Vector4: ...
    @property
    def aoal(self) -> I8Vector4: ...
    @property
    def aoor(self) -> I8Vector4: ...
    @property
    def aoog(self) -> I8Vector4: ...
    @property
    def aoob(self) -> I8Vector4: ...
    @property
    def aooa(self) -> I8Vector4: ...
    @property
    def aooo(self) -> I8Vector4: ...
    @property
    def aool(self) -> I8Vector4: ...
    @property
    def aolr(self) -> I8Vector4: ...
    @property
    def aolg(self) -> I8Vector4: ...
    @property
    def aolb(self) -> I8Vector4: ...
    @property
    def aola(self) -> I8Vector4: ...
    @property
    def aolo(self) -> I8Vector4: ...
    @property
    def aoll(self) -> I8Vector4: ...
    @property
    def alrr(self) -> I8Vector4: ...
    @property
    def alrg(self) -> I8Vector4: ...
    @property
    def alrb(self) -> I8Vector4: ...
    @property
    def alra(self) -> I8Vector4: ...
    @property
    def alro(self) -> I8Vector4: ...
    @property
    def alrl(self) -> I8Vector4: ...
    @property
    def algr(self) -> I8Vector4: ...
    @property
    def algg(self) -> I8Vector4: ...
    @property
    def algb(self) -> I8Vector4: ...
    @property
    def alga(self) -> I8Vector4: ...
    @property
    def algo(self) -> I8Vector4: ...
    @property
    def algl(self) -> I8Vector4: ...
    @property
    def albr(self) -> I8Vector4: ...
    @property
    def albg(self) -> I8Vector4: ...
    @property
    def albb(self) -> I8Vector4: ...
    @property
    def alba(self) -> I8Vector4: ...
    @property
    def albo(self) -> I8Vector4: ...
    @property
    def albl(self) -> I8Vector4: ...
    @property
    def alar(self) -> I8Vector4: ...
    @property
    def alag(self) -> I8Vector4: ...
    @property
    def alab(self) -> I8Vector4: ...
    @property
    def alaa(self) -> I8Vector4: ...
    @property
    def alao(self) -> I8Vector4: ...
    @property
    def alal(self) -> I8Vector4: ...
    @property
    def alor(self) -> I8Vector4: ...
    @property
    def alog(self) -> I8Vector4: ...
    @property
    def alob(self) -> I8Vector4: ...
    @property
    def aloa(self) -> I8Vector4: ...
    @property
    def aloo(self) -> I8Vector4: ...
    @property
    def alol(self) -> I8Vector4: ...
    @property
    def allr(self) -> I8Vector4: ...
    @property
    def allg(self) -> I8Vector4: ...
    @property
    def allb(self) -> I8Vector4: ...
    @property
    def alla(self) -> I8Vector4: ...
    @property
    def allo(self) -> I8Vector4: ...
    @property
    def alll(self) -> I8Vector4: ...
    @property
    def orrr(self) -> I8Vector4: ...
    @property
    def orrg(self) -> I8Vector4: ...
    @property
    def orrb(self) -> I8Vector4: ...
    @property
    def orra(self) -> I8Vector4: ...
    @property
    def orro(self) -> I8Vector4: ...
    @property
    def orrl(self) -> I8Vector4: ...
    @property
    def orgr(self) -> I8Vector4: ...
    @property
    def orgg(self) -> I8Vector4: ...
    @property
    def orgb(self) -> I8Vector4: ...
    @property
    def orga(self) -> I8Vector4: ...
    @property
    def orgo(self) -> I8Vector4: ...
    @property
    def orgl(self) -> I8Vector4: ...
    @property
    def orbr(self) -> I8Vector4: ...
    @property
    def orbg(self) -> I8Vector4: ...
    @property
    def orbb(self) -> I8Vector4: ...
    @property
    def orba(self) -> I8Vector4: ...
    @property
    def orbo(self) -> I8Vector4: ...
    @property
    def orbl(self) -> I8Vector4: ...
    @property
    def orar(self) -> I8Vector4: ...
    @property
    def orag(self) -> I8Vector4: ...
    @property
    def orab(self) -> I8Vector4: ...
    @property
    def oraa(self) -> I8Vector4: ...
    @property
    def orao(self) -> I8Vector4: ...
    @property
    def oral(self) -> I8Vector4: ...
    @property
    def oror(self) -> I8Vector4: ...
    @property
    def orog(self) -> I8Vector4: ...
    @property
    def orob(self) -> I8Vector4: ...
    @property
    def oroa(self) -> I8Vector4: ...
    @property
    def oroo(self) -> I8Vector4: ...
    @property
    def orol(self) -> I8Vector4: ...
    @property
    def orlr(self) -> I8Vector4: ...
    @property
    def orlg(self) -> I8Vector4: ...
    @property
    def orlb(self) -> I8Vector4: ...
    @property
    def orla(self) -> I8Vector4: ...
    @property
    def orlo(self) -> I8Vector4: ...
    @property
    def orll(self) -> I8Vector4: ...
    @property
    def ogrr(self) -> I8Vector4: ...
    @property
    def ogrg(self) -> I8Vector4: ...
    @property
    def ogrb(self) -> I8Vector4: ...
    @property
    def ogra(self) -> I8Vector4: ...
    @property
    def ogro(self) -> I8Vector4: ...
    @property
    def ogrl(self) -> I8Vector4: ...
    @property
    def oggr(self) -> I8Vector4: ...
    @property
    def oggg(self) -> I8Vector4: ...
    @property
    def oggb(self) -> I8Vector4: ...
    @property
    def ogga(self) -> I8Vector4: ...
    @property
    def oggo(self) -> I8Vector4: ...
    @property
    def oggl(self) -> I8Vector4: ...
    @property
    def ogbr(self) -> I8Vector4: ...
    @property
    def ogbg(self) -> I8Vector4: ...
    @property
    def ogbb(self) -> I8Vector4: ...
    @property
    def ogba(self) -> I8Vector4: ...
    @property
    def ogbo(self) -> I8Vector4: ...
    @property
    def ogbl(self) -> I8Vector4: ...
    @property
    def ogar(self) -> I8Vector4: ...
    @property
    def ogag(self) -> I8Vector4: ...
    @property
    def ogab(self) -> I8Vector4: ...
    @property
    def ogaa(self) -> I8Vector4: ...
    @property
    def ogao(self) -> I8Vector4: ...
    @property
    def ogal(self) -> I8Vector4: ...
    @property
    def ogor(self) -> I8Vector4: ...
    @property
    def ogog(self) -> I8Vector4: ...
    @property
    def ogob(self) -> I8Vector4: ...
    @property
    def ogoa(self) -> I8Vector4: ...
    @property
    def ogoo(self) -> I8Vector4: ...
    @property
    def ogol(self) -> I8Vector4: ...
    @property
    def oglr(self) -> I8Vector4: ...
    @property
    def oglg(self) -> I8Vector4: ...
    @property
    def oglb(self) -> I8Vector4: ...
    @property
    def ogla(self) -> I8Vector4: ...
    @property
    def oglo(self) -> I8Vector4: ...
    @property
    def ogll(self) -> I8Vector4: ...
    @property
    def obrr(self) -> I8Vector4: ...
    @property
    def obrg(self) -> I8Vector4: ...
    @property
    def obrb(self) -> I8Vector4: ...
    @property
    def obra(self) -> I8Vector4: ...
    @property
    def obro(self) -> I8Vector4: ...
    @property
    def obrl(self) -> I8Vector4: ...
    @property
    def obgr(self) -> I8Vector4: ...
    @property
    def obgg(self) -> I8Vector4: ...
    @property
    def obgb(self) -> I8Vector4: ...
    @property
    def obga(self) -> I8Vector4: ...
    @property
    def obgo(self) -> I8Vector4: ...
    @property
    def obgl(self) -> I8Vector4: ...
    @property
    def obbr(self) -> I8Vector4: ...
    @property
    def obbg(self) -> I8Vector4: ...
    @property
    def obbb(self) -> I8Vector4: ...
    @property
    def obba(self) -> I8Vector4: ...
    @property
    def obbo(self) -> I8Vector4: ...
    @property
    def obbl(self) -> I8Vector4: ...
    @property
    def obar(self) -> I8Vector4: ...
    @property
    def obag(self) -> I8Vector4: ...
    @property
    def obab(self) -> I8Vector4: ...
    @property
    def obaa(self) -> I8Vector4: ...
    @property
    def obao(self) -> I8Vector4: ...
    @property
    def obal(self) -> I8Vector4: ...
    @property
    def obor(self) -> I8Vector4: ...
    @property
    def obog(self) -> I8Vector4: ...
    @property
    def obob(self) -> I8Vector4: ...
    @property
    def oboa(self) -> I8Vector4: ...
    @property
    def oboo(self) -> I8Vector4: ...
    @property
    def obol(self) -> I8Vector4: ...
    @property
    def oblr(self) -> I8Vector4: ...
    @property
    def oblg(self) -> I8Vector4: ...
    @property
    def oblb(self) -> I8Vector4: ...
    @property
    def obla(self) -> I8Vector4: ...
    @property
    def oblo(self) -> I8Vector4: ...
    @property
    def obll(self) -> I8Vector4: ...
    @property
    def oarr(self) -> I8Vector4: ...
    @property
    def oarg(self) -> I8Vector4: ...
    @property
    def oarb(self) -> I8Vector4: ...
    @property
    def oara(self) -> I8Vector4: ...
    @property
    def oaro(self) -> I8Vector4: ...
    @property
    def oarl(self) -> I8Vector4: ...
    @property
    def oagr(self) -> I8Vector4: ...
    @property
    def oagg(self) -> I8Vector4: ...
    @property
    def oagb(self) -> I8Vector4: ...
    @property
    def oaga(self) -> I8Vector4: ...
    @property
    def oago(self) -> I8Vector4: ...
    @property
    def oagl(self) -> I8Vector4: ...
    @property
    def oabr(self) -> I8Vector4: ...
    @property
    def oabg(self) -> I8Vector4: ...
    @property
    def oabb(self) -> I8Vector4: ...
    @property
    def oaba(self) -> I8Vector4: ...
    @property
    def oabo(self) -> I8Vector4: ...
    @property
    def oabl(self) -> I8Vector4: ...
    @property
    def oaar(self) -> I8Vector4: ...
    @property
    def oaag(self) -> I8Vector4: ...
    @property
    def oaab(self) -> I8Vector4: ...
    @property
    def oaaa(self) -> I8Vector4: ...
    @property
    def oaao(self) -> I8Vector4: ...
    @property
    def oaal(self) -> I8Vector4: ...
    @property
    def oaor(self) -> I8Vector4: ...
    @property
    def oaog(self) -> I8Vector4: ...
    @property
    def oaob(self) -> I8Vector4: ...
    @property
    def oaoa(self) -> I8Vector4: ...
    @property
    def oaoo(self) -> I8Vector4: ...
    @property
    def oaol(self) -> I8Vector4: ...
    @property
    def oalr(self) -> I8Vector4: ...
    @property
    def oalg(self) -> I8Vector4: ...
    @property
    def oalb(self) -> I8Vector4: ...
    @property
    def oala(self) -> I8Vector4: ...
    @property
    def oalo(self) -> I8Vector4: ...
    @property
    def oall(self) -> I8Vector4: ...
    @property
    def oorr(self) -> I8Vector4: ...
    @property
    def oorg(self) -> I8Vector4: ...
    @property
    def oorb(self) -> I8Vector4: ...
    @property
    def oora(self) -> I8Vector4: ...
    @property
    def ooro(self) -> I8Vector4: ...
    @property
    def oorl(self) -> I8Vector4: ...
    @property
    def oogr(self) -> I8Vector4: ...
    @property
    def oogg(self) -> I8Vector4: ...
    @property
    def oogb(self) -> I8Vector4: ...
    @property
    def ooga(self) -> I8Vector4: ...
    @property
    def oogo(self) -> I8Vector4: ...
    @property
    def oogl(self) -> I8Vector4: ...
    @property
    def oobr(self) -> I8Vector4: ...
    @property
    def oobg(self) -> I8Vector4: ...
    @property
    def oobb(self) -> I8Vector4: ...
    @property
    def ooba(self) -> I8Vector4: ...
    @property
    def oobo(self) -> I8Vector4: ...
    @property
    def oobl(self) -> I8Vector4: ...
    @property
    def ooar(self) -> I8Vector4: ...
    @property
    def ooag(self) -> I8Vector4: ...
    @property
    def ooab(self) -> I8Vector4: ...
    @property
    def ooaa(self) -> I8Vector4: ...
    @property
    def ooao(self) -> I8Vector4: ...
    @property
    def ooal(self) -> I8Vector4: ...
    @property
    def ooor(self) -> I8Vector4: ...
    @property
    def ooog(self) -> I8Vector4: ...
    @property
    def ooob(self) -> I8Vector4: ...
    @property
    def oooa(self) -> I8Vector4: ...
    @property
    def oolr(self) -> I8Vector4: ...
    @property
    def oolg(self) -> I8Vector4: ...
    @property
    def oolb(self) -> I8Vector4: ...
    @property
    def oola(self) -> I8Vector4: ...
    @property
    def olrr(self) -> I8Vector4: ...
    @property
    def olrg(self) -> I8Vector4: ...
    @property
    def olrb(self) -> I8Vector4: ...
    @property
    def olra(self) -> I8Vector4: ...
    @property
    def olro(self) -> I8Vector4: ...
    @property
    def olrl(self) -> I8Vector4: ...
    @property
    def olgr(self) -> I8Vector4: ...
    @property
    def olgg(self) -> I8Vector4: ...
    @property
    def olgb(self) -> I8Vector4: ...
    @property
    def olga(self) -> I8Vector4: ...
    @property
    def olgo(self) -> I8Vector4: ...
    @property
    def olgl(self) -> I8Vector4: ...
    @property
    def olbr(self) -> I8Vector4: ...
    @property
    def olbg(self) -> I8Vector4: ...
    @property
    def olbb(self) -> I8Vector4: ...
    @property
    def olba(self) -> I8Vector4: ...
    @property
    def olbo(self) -> I8Vector4: ...
    @property
    def olbl(self) -> I8Vector4: ...
    @property
    def olar(self) -> I8Vector4: ...
    @property
    def olag(self) -> I8Vector4: ...
    @property
    def olab(self) -> I8Vector4: ...
    @property
    def olaa(self) -> I8Vector4: ...
    @property
    def olao(self) -> I8Vector4: ...
    @property
    def olal(self) -> I8Vector4: ...
    @property
    def olor(self) -> I8Vector4: ...
    @property
    def olog(self) -> I8Vector4: ...
    @property
    def olob(self) -> I8Vector4: ...
    @property
    def oloa(self) -> I8Vector4: ...
    @property
    def ollr(self) -> I8Vector4: ...
    @property
    def ollg(self) -> I8Vector4: ...
    @property
    def ollb(self) -> I8Vector4: ...
    @property
    def olla(self) -> I8Vector4: ...
    @property
    def lrrr(self) -> I8Vector4: ...
    @property
    def lrrg(self) -> I8Vector4: ...
    @property
    def lrrb(self) -> I8Vector4: ...
    @property
    def lrra(self) -> I8Vector4: ...
    @property
    def lrro(self) -> I8Vector4: ...
    @property
    def lrrl(self) -> I8Vector4: ...
    @property
    def lrgr(self) -> I8Vector4: ...
    @property
    def lrgg(self) -> I8Vector4: ...
    @property
    def lrgb(self) -> I8Vector4: ...
    @property
    def lrga(self) -> I8Vector4: ...
    @property
    def lrgo(self) -> I8Vector4: ...
    @property
    def lrgl(self) -> I8Vector4: ...
    @property
    def lrbr(self) -> I8Vector4: ...
    @property
    def lrbg(self) -> I8Vector4: ...
    @property
    def lrbb(self) -> I8Vector4: ...
    @property
    def lrba(self) -> I8Vector4: ...
    @property
    def lrbo(self) -> I8Vector4: ...
    @property
    def lrbl(self) -> I8Vector4: ...
    @property
    def lrar(self) -> I8Vector4: ...
    @property
    def lrag(self) -> I8Vector4: ...
    @property
    def lrab(self) -> I8Vector4: ...
    @property
    def lraa(self) -> I8Vector4: ...
    @property
    def lrao(self) -> I8Vector4: ...
    @property
    def lral(self) -> I8Vector4: ...
    @property
    def lror(self) -> I8Vector4: ...
    @property
    def lrog(self) -> I8Vector4: ...
    @property
    def lrob(self) -> I8Vector4: ...
    @property
    def lroa(self) -> I8Vector4: ...
    @property
    def lroo(self) -> I8Vector4: ...
    @property
    def lrol(self) -> I8Vector4: ...
    @property
    def lrlr(self) -> I8Vector4: ...
    @property
    def lrlg(self) -> I8Vector4: ...
    @property
    def lrlb(self) -> I8Vector4: ...
    @property
    def lrla(self) -> I8Vector4: ...
    @property
    def lrlo(self) -> I8Vector4: ...
    @property
    def lrll(self) -> I8Vector4: ...
    @property
    def lgrr(self) -> I8Vector4: ...
    @property
    def lgrg(self) -> I8Vector4: ...
    @property
    def lgrb(self) -> I8Vector4: ...
    @property
    def lgra(self) -> I8Vector4: ...
    @property
    def lgro(self) -> I8Vector4: ...
    @property
    def lgrl(self) -> I8Vector4: ...
    @property
    def lggr(self) -> I8Vector4: ...
    @property
    def lggg(self) -> I8Vector4: ...
    @property
    def lggb(self) -> I8Vector4: ...
    @property
    def lgga(self) -> I8Vector4: ...
    @property
    def lggo(self) -> I8Vector4: ...
    @property
    def lggl(self) -> I8Vector4: ...
    @property
    def lgbr(self) -> I8Vector4: ...
    @property
    def lgbg(self) -> I8Vector4: ...
    @property
    def lgbb(self) -> I8Vector4: ...
    @property
    def lgba(self) -> I8Vector4: ...
    @property
    def lgbo(self) -> I8Vector4: ...
    @property
    def lgbl(self) -> I8Vector4: ...
    @property
    def lgar(self) -> I8Vector4: ...
    @property
    def lgag(self) -> I8Vector4: ...
    @property
    def lgab(self) -> I8Vector4: ...
    @property
    def lgaa(self) -> I8Vector4: ...
    @property
    def lgao(self) -> I8Vector4: ...
    @property
    def lgal(self) -> I8Vector4: ...
    @property
    def lgor(self) -> I8Vector4: ...
    @property
    def lgog(self) -> I8Vector4: ...
    @property
    def lgob(self) -> I8Vector4: ...
    @property
    def lgoa(self) -> I8Vector4: ...
    @property
    def lgoo(self) -> I8Vector4: ...
    @property
    def lgol(self) -> I8Vector4: ...
    @property
    def lglr(self) -> I8Vector4: ...
    @property
    def lglg(self) -> I8Vector4: ...
    @property
    def lglb(self) -> I8Vector4: ...
    @property
    def lgla(self) -> I8Vector4: ...
    @property
    def lglo(self) -> I8Vector4: ...
    @property
    def lgll(self) -> I8Vector4: ...
    @property
    def lbrr(self) -> I8Vector4: ...
    @property
    def lbrg(self) -> I8Vector4: ...
    @property
    def lbrb(self) -> I8Vector4: ...
    @property
    def lbra(self) -> I8Vector4: ...
    @property
    def lbro(self) -> I8Vector4: ...
    @property
    def lbrl(self) -> I8Vector4: ...
    @property
    def lbgr(self) -> I8Vector4: ...
    @property
    def lbgg(self) -> I8Vector4: ...
    @property
    def lbgb(self) -> I8Vector4: ...
    @property
    def lbga(self) -> I8Vector4: ...
    @property
    def lbgo(self) -> I8Vector4: ...
    @property
    def lbgl(self) -> I8Vector4: ...
    @property
    def lbbr(self) -> I8Vector4: ...
    @property
    def lbbg(self) -> I8Vector4: ...
    @property
    def lbbb(self) -> I8Vector4: ...
    @property
    def lbba(self) -> I8Vector4: ...
    @property
    def lbbo(self) -> I8Vector4: ...
    @property
    def lbbl(self) -> I8Vector4: ...
    @property
    def lbar(self) -> I8Vector4: ...
    @property
    def lbag(self) -> I8Vector4: ...
    @property
    def lbab(self) -> I8Vector4: ...
    @property
    def lbaa(self) -> I8Vector4: ...
    @property
    def lbao(self) -> I8Vector4: ...
    @property
    def lbal(self) -> I8Vector4: ...
    @property
    def lbor(self) -> I8Vector4: ...
    @property
    def lbog(self) -> I8Vector4: ...
    @property
    def lbob(self) -> I8Vector4: ...
    @property
    def lboa(self) -> I8Vector4: ...
    @property
    def lboo(self) -> I8Vector4: ...
    @property
    def lbol(self) -> I8Vector4: ...
    @property
    def lblr(self) -> I8Vector4: ...
    @property
    def lblg(self) -> I8Vector4: ...
    @property
    def lblb(self) -> I8Vector4: ...
    @property
    def lbla(self) -> I8Vector4: ...
    @property
    def lblo(self) -> I8Vector4: ...
    @property
    def lbll(self) -> I8Vector4: ...
    @property
    def larr(self) -> I8Vector4: ...
    @property
    def larg(self) -> I8Vector4: ...
    @property
    def larb(self) -> I8Vector4: ...
    @property
    def lara(self) -> I8Vector4: ...
    @property
    def laro(self) -> I8Vector4: ...
    @property
    def larl(self) -> I8Vector4: ...
    @property
    def lagr(self) -> I8Vector4: ...
    @property
    def lagg(self) -> I8Vector4: ...
    @property
    def lagb(self) -> I8Vector4: ...
    @property
    def laga(self) -> I8Vector4: ...
    @property
    def lago(self) -> I8Vector4: ...
    @property
    def lagl(self) -> I8Vector4: ...
    @property
    def labr(self) -> I8Vector4: ...
    @property
    def labg(self) -> I8Vector4: ...
    @property
    def labb(self) -> I8Vector4: ...
    @property
    def laba(self) -> I8Vector4: ...
    @property
    def labo(self) -> I8Vector4: ...
    @property
    def labl(self) -> I8Vector4: ...
    @property
    def laar(self) -> I8Vector4: ...
    @property
    def laag(self) -> I8Vector4: ...
    @property
    def laab(self) -> I8Vector4: ...
    @property
    def laaa(self) -> I8Vector4: ...
    @property
    def laao(self) -> I8Vector4: ...
    @property
    def laal(self) -> I8Vector4: ...
    @property
    def laor(self) -> I8Vector4: ...
    @property
    def laog(self) -> I8Vector4: ...
    @property
    def laob(self) -> I8Vector4: ...
    @property
    def laoa(self) -> I8Vector4: ...
    @property
    def laoo(self) -> I8Vector4: ...
    @property
    def laol(self) -> I8Vector4: ...
    @property
    def lalr(self) -> I8Vector4: ...
    @property
    def lalg(self) -> I8Vector4: ...
    @property
    def lalb(self) -> I8Vector4: ...
    @property
    def lala(self) -> I8Vector4: ...
    @property
    def lalo(self) -> I8Vector4: ...
    @property
    def lall(self) -> I8Vector4: ...
    @property
    def lorr(self) -> I8Vector4: ...
    @property
    def lorg(self) -> I8Vector4: ...
    @property
    def lorb(self) -> I8Vector4: ...
    @property
    def lora(self) -> I8Vector4: ...
    @property
    def loro(self) -> I8Vector4: ...
    @property
    def lorl(self) -> I8Vector4: ...
    @property
    def logr(self) -> I8Vector4: ...
    @property
    def logg(self) -> I8Vector4: ...
    @property
    def logb(self) -> I8Vector4: ...
    @property
    def loga(self) -> I8Vector4: ...
    @property
    def logo(self) -> I8Vector4: ...
    @property
    def logl(self) -> I8Vector4: ...
    @property
    def lobr(self) -> I8Vector4: ...
    @property
    def lobg(self) -> I8Vector4: ...
    @property
    def lobb(self) -> I8Vector4: ...
    @property
    def loba(self) -> I8Vector4: ...
    @property
    def lobo(self) -> I8Vector4: ...
    @property
    def lobl(self) -> I8Vector4: ...
    @property
    def loar(self) -> I8Vector4: ...
    @property
    def loag(self) -> I8Vector4: ...
    @property
    def loab(self) -> I8Vector4: ...
    @property
    def loaa(self) -> I8Vector4: ...
    @property
    def loao(self) -> I8Vector4: ...
    @property
    def loal(self) -> I8Vector4: ...
    @property
    def loor(self) -> I8Vector4: ...
    @property
    def loog(self) -> I8Vector4: ...
    @property
    def loob(self) -> I8Vector4: ...
    @property
    def looa(self) -> I8Vector4: ...
    @property
    def lolr(self) -> I8Vector4: ...
    @property
    def lolg(self) -> I8Vector4: ...
    @property
    def lolb(self) -> I8Vector4: ...
    @property
    def lola(self) -> I8Vector4: ...
    @property
    def llrr(self) -> I8Vector4: ...
    @property
    def llrg(self) -> I8Vector4: ...
    @property
    def llrb(self) -> I8Vector4: ...
    @property
    def llra(self) -> I8Vector4: ...
    @property
    def llro(self) -> I8Vector4: ...
    @property
    def llrl(self) -> I8Vector4: ...
    @property
    def llgr(self) -> I8Vector4: ...
    @property
    def llgg(self) -> I8Vector4: ...
    @property
    def llgb(self) -> I8Vector4: ...
    @property
    def llga(self) -> I8Vector4: ...
    @property
    def llgo(self) -> I8Vector4: ...
    @property
    def llgl(self) -> I8Vector4: ...
    @property
    def llbr(self) -> I8Vector4: ...
    @property
    def llbg(self) -> I8Vector4: ...
    @property
    def llbb(self) -> I8Vector4: ...
    @property
    def llba(self) -> I8Vector4: ...
    @property
    def llbo(self) -> I8Vector4: ...
    @property
    def llbl(self) -> I8Vector4: ...
    @property
    def llar(self) -> I8Vector4: ...
    @property
    def llag(self) -> I8Vector4: ...
    @property
    def llab(self) -> I8Vector4: ...
    @property
    def llaa(self) -> I8Vector4: ...
    @property
    def llao(self) -> I8Vector4: ...
    @property
    def llal(self) -> I8Vector4: ...
    @property
    def llor(self) -> I8Vector4: ...
    @property
    def llog(self) -> I8Vector4: ...
    @property
    def llob(self) -> I8Vector4: ...
    @property
    def lloa(self) -> I8Vector4: ...
    @property
    def lllr(self) -> I8Vector4: ...
    @property
    def lllg(self) -> I8Vector4: ...
    @property
    def lllb(self) -> I8Vector4: ...
    @property
    def llla(self) -> I8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> I8Vector2: ...
    @property
    def st(self) -> I8Vector2: ...
    @property
    def sq(self) -> I8Vector2: ...
    @property
    def sp(self) -> I8Vector2: ...
    @property
    def so(self) -> I8Vector2: ...
    @property
    def sl(self) -> I8Vector2: ...
    @property
    def ts(self) -> I8Vector2: ...
    @property
    def tt(self) -> I8Vector2: ...
    @property
    def tq(self) -> I8Vector2: ...
    @property
    def tp(self) -> I8Vector2: ...
    @property
    def to(self) -> I8Vector2: ...
    @property
    def tl(self) -> I8Vector2: ...
    @property
    def qs(self) -> I8Vector2: ...
    @property
    def qt(self) -> I8Vector2: ...
    @property
    def qq(self) -> I8Vector2: ...
    @property
    def qp(self) -> I8Vector2: ...
    @property
    def qo(self) -> I8Vector2: ...
    @property
    def ql(self) -> I8Vector2: ...
    @property
    def ps(self) -> I8Vector2: ...
    @property
    def pt(self) -> I8Vector2: ...
    @property
    def pq(self) -> I8Vector2: ...
    @property
    def pp(self) -> I8Vector2: ...
    @property
    def po(self) -> I8Vector2: ...
    @property
    def pl(self) -> I8Vector2: ...
    @property
    def os(self) -> I8Vector2: ...
    @property
    def ot(self) -> I8Vector2: ...
    @property
    def oq(self) -> I8Vector2: ...
    @property
    def op(self) -> I8Vector2: ...
    @property
    def ls(self) -> I8Vector2: ...
    @property
    def lt(self) -> I8Vector2: ...
    @property
    def lq(self) -> I8Vector2: ...
    @property
    def lp(self) -> I8Vector2: ...
    @property
    def sss(self) -> I8Vector3: ...
    @property
    def sst(self) -> I8Vector3: ...
    @property
    def ssq(self) -> I8Vector3: ...
    @property
    def ssp(self) -> I8Vector3: ...
    @property
    def sso(self) -> I8Vector3: ...
    @property
    def ssl(self) -> I8Vector3: ...
    @property
    def sts(self) -> I8Vector3: ...
    @property
    def stt(self) -> I8Vector3: ...
    @property
    def stq(self) -> I8Vector3: ...
    @property
    def stp(self) -> I8Vector3: ...
    @property
    def sto(self) -> I8Vector3: ...
    @property
    def stl(self) -> I8Vector3: ...
    @property
    def sqs(self) -> I8Vector3: ...
    @property
    def sqt(self) -> I8Vector3: ...
    @property
    def sqq(self) -> I8Vector3: ...
    @property
    def sqp(self) -> I8Vector3: ...
    @property
    def sqo(self) -> I8Vector3: ...
    @property
    def sql(self) -> I8Vector3: ...
    @property
    def sps(self) -> I8Vector3: ...
    @property
    def spt(self) -> I8Vector3: ...
    @property
    def spq(self) -> I8Vector3: ...
    @property
    def spp(self) -> I8Vector3: ...
    @property
    def spo(self) -> I8Vector3: ...
    @property
    def spl(self) -> I8Vector3: ...
    @property
    def sos(self) -> I8Vector3: ...
    @property
    def sot(self) -> I8Vector3: ...
    @property
    def soq(self) -> I8Vector3: ...
    @property
    def sop(self) -> I8Vector3: ...
    @property
    def soo(self) -> I8Vector3: ...
    @property
    def sol(self) -> I8Vector3: ...
    @property
    def sls(self) -> I8Vector3: ...
    @property
    def slt(self) -> I8Vector3: ...
    @property
    def slq(self) -> I8Vector3: ...
    @property
    def slp(self) -> I8Vector3: ...
    @property
    def slo(self) -> I8Vector3: ...
    @property
    def sll(self) -> I8Vector3: ...
    @property
    def tss(self) -> I8Vector3: ...
    @property
    def tst(self) -> I8Vector3: ...
    @property
    def tsq(self) -> I8Vector3: ...
    @property
    def tsp(self) -> I8Vector3: ...
    @property
    def tso(self) -> I8Vector3: ...
    @property
    def tsl(self) -> I8Vector3: ...
    @property
    def tts(self) -> I8Vector3: ...
    @property
    def ttt(self) -> I8Vector3: ...
    @property
    def ttq(self) -> I8Vector3: ...
    @property
    def ttp(self) -> I8Vector3: ...
    @property
    def tto(self) -> I8Vector3: ...
    @property
    def ttl(self) -> I8Vector3: ...
    @property
    def tqs(self) -> I8Vector3: ...
    @property
    def tqt(self) -> I8Vector3: ...
    @property
    def tqq(self) -> I8Vector3: ...
    @property
    def tqp(self) -> I8Vector3: ...
    @property
    def tqo(self) -> I8Vector3: ...
    @property
    def tql(self) -> I8Vector3: ...
    @property
    def tps(self) -> I8Vector3: ...
    @property
    def tpt(self) -> I8Vector3: ...
    @property
    def tpq(self) -> I8Vector3: ...
    @property
    def tpp(self) -> I8Vector3: ...
    @property
    def tpo(self) -> I8Vector3: ...
    @property
    def tpl(self) -> I8Vector3: ...
    @property
    def tos(self) -> I8Vector3: ...
    @property
    def tot(self) -> I8Vector3: ...
    @property
    def toq(self) -> I8Vector3: ...
    @property
    def top(self) -> I8Vector3: ...
    @property
    def too(self) -> I8Vector3: ...
    @property
    def tol(self) -> I8Vector3: ...
    @property
    def tls(self) -> I8Vector3: ...
    @property
    def tlt(self) -> I8Vector3: ...
    @property
    def tlq(self) -> I8Vector3: ...
    @property
    def tlp(self) -> I8Vector3: ...
    @property
    def tlo(self) -> I8Vector3: ...
    @property
    def tll(self) -> I8Vector3: ...
    @property
    def qss(self) -> I8Vector3: ...
    @property
    def qst(self) -> I8Vector3: ...
    @property
    def qsq(self) -> I8Vector3: ...
    @property
    def qsp(self) -> I8Vector3: ...
    @property
    def qso(self) -> I8Vector3: ...
    @property
    def qsl(self) -> I8Vector3: ...
    @property
    def qts(self) -> I8Vector3: ...
    @property
    def qtt(self) -> I8Vector3: ...
    @property
    def qtq(self) -> I8Vector3: ...
    @property
    def qtp(self) -> I8Vector3: ...
    @property
    def qto(self) -> I8Vector3: ...
    @property
    def qtl(self) -> I8Vector3: ...
    @property
    def qqs(self) -> I8Vector3: ...
    @property
    def qqt(self) -> I8Vector3: ...
    @property
    def qqq(self) -> I8Vector3: ...
    @property
    def qqp(self) -> I8Vector3: ...
    @property
    def qqo(self) -> I8Vector3: ...
    @property
    def qql(self) -> I8Vector3: ...
    @property
    def qps(self) -> I8Vector3: ...
    @property
    def qpt(self) -> I8Vector3: ...
    @property
    def qpq(self) -> I8Vector3: ...
    @property
    def qpp(self) -> I8Vector3: ...
    @property
    def qpo(self) -> I8Vector3: ...
    @property
    def qpl(self) -> I8Vector3: ...
    @property
    def qos(self) -> I8Vector3: ...
    @property
    def qot(self) -> I8Vector3: ...
    @property
    def qoq(self) -> I8Vector3: ...
    @property
    def qop(self) -> I8Vector3: ...
    @property
    def qoo(self) -> I8Vector3: ...
    @property
    def qol(self) -> I8Vector3: ...
    @property
    def qls(self) -> I8Vector3: ...
    @property
    def qlt(self) -> I8Vector3: ...
    @property
    def qlq(self) -> I8Vector3: ...
    @property
    def qlp(self) -> I8Vector3: ...
    @property
    def qlo(self) -> I8Vector3: ...
    @property
    def qll(self) -> I8Vector3: ...
    @property
    def pss(self) -> I8Vector3: ...
    @property
    def pst(self) -> I8Vector3: ...
    @property
    def psq(self) -> I8Vector3: ...
    @property
    def psp(self) -> I8Vector3: ...
    @property
    def pso(self) -> I8Vector3: ...
    @property
    def psl(self) -> I8Vector3: ...
    @property
    def pts(self) -> I8Vector3: ...
    @property
    def ptt(self) -> I8Vector3: ...
    @property
    def ptq(self) -> I8Vector3: ...
    @property
    def ptp(self) -> I8Vector3: ...
    @property
    def pto(self) -> I8Vector3: ...
    @property
    def ptl(self) -> I8Vector3: ...
    @property
    def pqs(self) -> I8Vector3: ...
    @property
    def pqt(self) -> I8Vector3: ...
    @property
    def pqq(self) -> I8Vector3: ...
    @property
    def pqp(self) -> I8Vector3: ...
    @property
    def pqo(self) -> I8Vector3: ...
    @property
    def pql(self) -> I8Vector3: ...
    @property
    def pps(self) -> I8Vector3: ...
    @property
    def ppt(self) -> I8Vector3: ...
    @property
    def ppq(self) -> I8Vector3: ...
    @property
    def ppp(self) -> I8Vector3: ...
    @property
    def ppo(self) -> I8Vector3: ...
    @property
    def ppl(self) -> I8Vector3: ...
    @property
    def pos(self) -> I8Vector3: ...
    @property
    def pot(self) -> I8Vector3: ...
    @property
    def poq(self) -> I8Vector3: ...
    @property
    def pop(self) -> I8Vector3: ...
    @property
    def poo(self) -> I8Vector3: ...
    @property
    def pol(self) -> I8Vector3: ...
    @property
    def pls(self) -> I8Vector3: ...
    @property
    def plt(self) -> I8Vector3: ...
    @property
    def plq(self) -> I8Vector3: ...
    @property
    def plp(self) -> I8Vector3: ...
    @property
    def plo(self) -> I8Vector3: ...
    @property
    def pll(self) -> I8Vector3: ...
    @property
    def oss(self) -> I8Vector3: ...
    @property
    def ost(self) -> I8Vector3: ...
    @property
    def osq(self) -> I8Vector3: ...
    @property
    def osp(self) -> I8Vector3: ...
    @property
    def oso(self) -> I8Vector3: ...
    @property
    def osl(self) -> I8Vector3: ...
    @property
    def ots(self) -> I8Vector3: ...
    @property
    def ott(self) -> I8Vector3: ...
    @property
    def otq(self) -> I8Vector3: ...
    @property
    def otp(self) -> I8Vector3: ...
    @property
    def oto(self) -> I8Vector3: ...
    @property
    def otl(self) -> I8Vector3: ...
    @property
    def oqs(self) -> I8Vector3: ...
    @property
    def oqt(self) -> I8Vector3: ...
    @property
    def oqq(self) -> I8Vector3: ...
    @property
    def oqp(self) -> I8Vector3: ...
    @property
    def oqo(self) -> I8Vector3: ...
    @property
    def oql(self) -> I8Vector3: ...
    @property
    def ops(self) -> I8Vector3: ...
    @property
    def opt(self) -> I8Vector3: ...
    @property
    def opq(self) -> I8Vector3: ...
    @property
    def opp(self) -> I8Vector3: ...
    @property
    def opo(self) -> I8Vector3: ...
    @property
    def opl(self) -> I8Vector3: ...
    @property
    def oos(self) -> I8Vector3: ...
    @property
    def oot(self) -> I8Vector3: ...
    @property
    def ooq(self) -> I8Vector3: ...
    @property
    def oop(self) -> I8Vector3: ...
    @property
    def ols(self) -> I8Vector3: ...
    @property
    def olt(self) -> I8Vector3: ...
    @property
    def olq(self) -> I8Vector3: ...
    @property
    def olp(self) -> I8Vector3: ...
    @property
    def lss(self) -> I8Vector3: ...
    @property
    def lst(self) -> I8Vector3: ...
    @property
    def lsq(self) -> I8Vector3: ...
    @property
    def lsp(self) -> I8Vector3: ...
    @property
    def lso(self) -> I8Vector3: ...
    @property
    def lsl(self) -> I8Vector3: ...
    @property
    def lts(self) -> I8Vector3: ...
    @property
    def ltt(self) -> I8Vector3: ...
    @property
    def ltq(self) -> I8Vector3: ...
    @property
    def ltp(self) -> I8Vector3: ...
    @property
    def lto(self) -> I8Vector3: ...
    @property
    def ltl(self) -> I8Vector3: ...
    @property
    def lqs(self) -> I8Vector3: ...
    @property
    def lqt(self) -> I8Vector3: ...
    @property
    def lqq(self) -> I8Vector3: ...
    @property
    def lqp(self) -> I8Vector3: ...
    @property
    def lqo(self) -> I8Vector3: ...
    @property
    def lql(self) -> I8Vector3: ...
    @property
    def lps(self) -> I8Vector3: ...
    @property
    def lpt(self) -> I8Vector3: ...
    @property
    def lpq(self) -> I8Vector3: ...
    @property
    def lpp(self) -> I8Vector3: ...
    @property
    def lpo(self) -> I8Vector3: ...
    @property
    def lpl(self) -> I8Vector3: ...
    @property
    def los(self) -> I8Vector3: ...
    @property
    def lot(self) -> I8Vector3: ...
    @property
    def loq(self) -> I8Vector3: ...
    @property
    def lop(self) -> I8Vector3: ...
    @property
    def lls(self) -> I8Vector3: ...
    @property
    def llt(self) -> I8Vector3: ...
    @property
    def llq(self) -> I8Vector3: ...
    @property
    def llp(self) -> I8Vector3: ...
    @property
    def ssss(self) -> I8Vector4: ...
    @property
    def ssst(self) -> I8Vector4: ...
    @property
    def sssq(self) -> I8Vector4: ...
    @property
    def sssp(self) -> I8Vector4: ...
    @property
    def ssso(self) -> I8Vector4: ...
    @property
    def sssl(self) -> I8Vector4: ...
    @property
    def ssts(self) -> I8Vector4: ...
    @property
    def sstt(self) -> I8Vector4: ...
    @property
    def sstq(self) -> I8Vector4: ...
    @property
    def sstp(self) -> I8Vector4: ...
    @property
    def ssto(self) -> I8Vector4: ...
    @property
    def sstl(self) -> I8Vector4: ...
    @property
    def ssqs(self) -> I8Vector4: ...
    @property
    def ssqt(self) -> I8Vector4: ...
    @property
    def ssqq(self) -> I8Vector4: ...
    @property
    def ssqp(self) -> I8Vector4: ...
    @property
    def ssqo(self) -> I8Vector4: ...
    @property
    def ssql(self) -> I8Vector4: ...
    @property
    def ssps(self) -> I8Vector4: ...
    @property
    def sspt(self) -> I8Vector4: ...
    @property
    def sspq(self) -> I8Vector4: ...
    @property
    def sspp(self) -> I8Vector4: ...
    @property
    def sspo(self) -> I8Vector4: ...
    @property
    def sspl(self) -> I8Vector4: ...
    @property
    def ssos(self) -> I8Vector4: ...
    @property
    def ssot(self) -> I8Vector4: ...
    @property
    def ssoq(self) -> I8Vector4: ...
    @property
    def ssop(self) -> I8Vector4: ...
    @property
    def ssoo(self) -> I8Vector4: ...
    @property
    def ssol(self) -> I8Vector4: ...
    @property
    def ssls(self) -> I8Vector4: ...
    @property
    def sslt(self) -> I8Vector4: ...
    @property
    def sslq(self) -> I8Vector4: ...
    @property
    def sslp(self) -> I8Vector4: ...
    @property
    def sslo(self) -> I8Vector4: ...
    @property
    def ssll(self) -> I8Vector4: ...
    @property
    def stss(self) -> I8Vector4: ...
    @property
    def stst(self) -> I8Vector4: ...
    @property
    def stsq(self) -> I8Vector4: ...
    @property
    def stsp(self) -> I8Vector4: ...
    @property
    def stso(self) -> I8Vector4: ...
    @property
    def stsl(self) -> I8Vector4: ...
    @property
    def stts(self) -> I8Vector4: ...
    @property
    def sttt(self) -> I8Vector4: ...
    @property
    def sttq(self) -> I8Vector4: ...
    @property
    def sttp(self) -> I8Vector4: ...
    @property
    def stto(self) -> I8Vector4: ...
    @property
    def sttl(self) -> I8Vector4: ...
    @property
    def stqs(self) -> I8Vector4: ...
    @property
    def stqt(self) -> I8Vector4: ...
    @property
    def stqq(self) -> I8Vector4: ...
    @property
    def stqp(self) -> I8Vector4: ...
    @property
    def stqo(self) -> I8Vector4: ...
    @property
    def stql(self) -> I8Vector4: ...
    @property
    def stps(self) -> I8Vector4: ...
    @property
    def stpt(self) -> I8Vector4: ...
    @property
    def stpq(self) -> I8Vector4: ...
    @property
    def stpp(self) -> I8Vector4: ...
    @property
    def stpo(self) -> I8Vector4: ...
    @property
    def stpl(self) -> I8Vector4: ...
    @property
    def stos(self) -> I8Vector4: ...
    @property
    def stot(self) -> I8Vector4: ...
    @property
    def stoq(self) -> I8Vector4: ...
    @property
    def stop(self) -> I8Vector4: ...
    @property
    def stoo(self) -> I8Vector4: ...
    @property
    def stol(self) -> I8Vector4: ...
    @property
    def stls(self) -> I8Vector4: ...
    @property
    def stlt(self) -> I8Vector4: ...
    @property
    def stlq(self) -> I8Vector4: ...
    @property
    def stlp(self) -> I8Vector4: ...
    @property
    def stlo(self) -> I8Vector4: ...
    @property
    def stll(self) -> I8Vector4: ...
    @property
    def sqss(self) -> I8Vector4: ...
    @property
    def sqst(self) -> I8Vector4: ...
    @property
    def sqsq(self) -> I8Vector4: ...
    @property
    def sqsp(self) -> I8Vector4: ...
    @property
    def sqso(self) -> I8Vector4: ...
    @property
    def sqsl(self) -> I8Vector4: ...
    @property
    def sqts(self) -> I8Vector4: ...
    @property
    def sqtt(self) -> I8Vector4: ...
    @property
    def sqtq(self) -> I8Vector4: ...
    @property
    def sqtp(self) -> I8Vector4: ...
    @property
    def sqto(self) -> I8Vector4: ...
    @property
    def sqtl(self) -> I8Vector4: ...
    @property
    def sqqs(self) -> I8Vector4: ...
    @property
    def sqqt(self) -> I8Vector4: ...
    @property
    def sqqq(self) -> I8Vector4: ...
    @property
    def sqqp(self) -> I8Vector4: ...
    @property
    def sqqo(self) -> I8Vector4: ...
    @property
    def sqql(self) -> I8Vector4: ...
    @property
    def sqps(self) -> I8Vector4: ...
    @property
    def sqpt(self) -> I8Vector4: ...
    @property
    def sqpq(self) -> I8Vector4: ...
    @property
    def sqpp(self) -> I8Vector4: ...
    @property
    def sqpo(self) -> I8Vector4: ...
    @property
    def sqpl(self) -> I8Vector4: ...
    @property
    def sqos(self) -> I8Vector4: ...
    @property
    def sqot(self) -> I8Vector4: ...
    @property
    def sqoq(self) -> I8Vector4: ...
    @property
    def sqop(self) -> I8Vector4: ...
    @property
    def sqoo(self) -> I8Vector4: ...
    @property
    def sqol(self) -> I8Vector4: ...
    @property
    def sqls(self) -> I8Vector4: ...
    @property
    def sqlt(self) -> I8Vector4: ...
    @property
    def sqlq(self) -> I8Vector4: ...
    @property
    def sqlp(self) -> I8Vector4: ...
    @property
    def sqlo(self) -> I8Vector4: ...
    @property
    def sqll(self) -> I8Vector4: ...
    @property
    def spss(self) -> I8Vector4: ...
    @property
    def spst(self) -> I8Vector4: ...
    @property
    def spsq(self) -> I8Vector4: ...
    @property
    def spsp(self) -> I8Vector4: ...
    @property
    def spso(self) -> I8Vector4: ...
    @property
    def spsl(self) -> I8Vector4: ...
    @property
    def spts(self) -> I8Vector4: ...
    @property
    def sptt(self) -> I8Vector4: ...
    @property
    def sptq(self) -> I8Vector4: ...
    @property
    def sptp(self) -> I8Vector4: ...
    @property
    def spto(self) -> I8Vector4: ...
    @property
    def sptl(self) -> I8Vector4: ...
    @property
    def spqs(self) -> I8Vector4: ...
    @property
    def spqt(self) -> I8Vector4: ...
    @property
    def spqq(self) -> I8Vector4: ...
    @property
    def spqp(self) -> I8Vector4: ...
    @property
    def spqo(self) -> I8Vector4: ...
    @property
    def spql(self) -> I8Vector4: ...
    @property
    def spps(self) -> I8Vector4: ...
    @property
    def sppt(self) -> I8Vector4: ...
    @property
    def sppq(self) -> I8Vector4: ...
    @property
    def sppp(self) -> I8Vector4: ...
    @property
    def sppo(self) -> I8Vector4: ...
    @property
    def sppl(self) -> I8Vector4: ...
    @property
    def spos(self) -> I8Vector4: ...
    @property
    def spot(self) -> I8Vector4: ...
    @property
    def spoq(self) -> I8Vector4: ...
    @property
    def spop(self) -> I8Vector4: ...
    @property
    def spoo(self) -> I8Vector4: ...
    @property
    def spol(self) -> I8Vector4: ...
    @property
    def spls(self) -> I8Vector4: ...
    @property
    def splt(self) -> I8Vector4: ...
    @property
    def splq(self) -> I8Vector4: ...
    @property
    def splp(self) -> I8Vector4: ...
    @property
    def splo(self) -> I8Vector4: ...
    @property
    def spll(self) -> I8Vector4: ...
    @property
    def soss(self) -> I8Vector4: ...
    @property
    def sost(self) -> I8Vector4: ...
    @property
    def sosq(self) -> I8Vector4: ...
    @property
    def sosp(self) -> I8Vector4: ...
    @property
    def soso(self) -> I8Vector4: ...
    @property
    def sosl(self) -> I8Vector4: ...
    @property
    def sots(self) -> I8Vector4: ...
    @property
    def sott(self) -> I8Vector4: ...
    @property
    def sotq(self) -> I8Vector4: ...
    @property
    def sotp(self) -> I8Vector4: ...
    @property
    def soto(self) -> I8Vector4: ...
    @property
    def sotl(self) -> I8Vector4: ...
    @property
    def soqs(self) -> I8Vector4: ...
    @property
    def soqt(self) -> I8Vector4: ...
    @property
    def soqq(self) -> I8Vector4: ...
    @property
    def soqp(self) -> I8Vector4: ...
    @property
    def soqo(self) -> I8Vector4: ...
    @property
    def soql(self) -> I8Vector4: ...
    @property
    def sops(self) -> I8Vector4: ...
    @property
    def sopt(self) -> I8Vector4: ...
    @property
    def sopq(self) -> I8Vector4: ...
    @property
    def sopp(self) -> I8Vector4: ...
    @property
    def sopo(self) -> I8Vector4: ...
    @property
    def sopl(self) -> I8Vector4: ...
    @property
    def soos(self) -> I8Vector4: ...
    @property
    def soot(self) -> I8Vector4: ...
    @property
    def sooq(self) -> I8Vector4: ...
    @property
    def soop(self) -> I8Vector4: ...
    @property
    def sooo(self) -> I8Vector4: ...
    @property
    def sool(self) -> I8Vector4: ...
    @property
    def sols(self) -> I8Vector4: ...
    @property
    def solt(self) -> I8Vector4: ...
    @property
    def solq(self) -> I8Vector4: ...
    @property
    def solp(self) -> I8Vector4: ...
    @property
    def solo(self) -> I8Vector4: ...
    @property
    def soll(self) -> I8Vector4: ...
    @property
    def slss(self) -> I8Vector4: ...
    @property
    def slst(self) -> I8Vector4: ...
    @property
    def slsq(self) -> I8Vector4: ...
    @property
    def slsp(self) -> I8Vector4: ...
    @property
    def slso(self) -> I8Vector4: ...
    @property
    def slsl(self) -> I8Vector4: ...
    @property
    def slts(self) -> I8Vector4: ...
    @property
    def sltt(self) -> I8Vector4: ...
    @property
    def sltq(self) -> I8Vector4: ...
    @property
    def sltp(self) -> I8Vector4: ...
    @property
    def slto(self) -> I8Vector4: ...
    @property
    def sltl(self) -> I8Vector4: ...
    @property
    def slqs(self) -> I8Vector4: ...
    @property
    def slqt(self) -> I8Vector4: ...
    @property
    def slqq(self) -> I8Vector4: ...
    @property
    def slqp(self) -> I8Vector4: ...
    @property
    def slqo(self) -> I8Vector4: ...
    @property
    def slql(self) -> I8Vector4: ...
    @property
    def slps(self) -> I8Vector4: ...
    @property
    def slpt(self) -> I8Vector4: ...
    @property
    def slpq(self) -> I8Vector4: ...
    @property
    def slpp(self) -> I8Vector4: ...
    @property
    def slpo(self) -> I8Vector4: ...
    @property
    def slpl(self) -> I8Vector4: ...
    @property
    def slos(self) -> I8Vector4: ...
    @property
    def slot(self) -> I8Vector4: ...
    @property
    def sloq(self) -> I8Vector4: ...
    @property
    def slop(self) -> I8Vector4: ...
    @property
    def sloo(self) -> I8Vector4: ...
    @property
    def slol(self) -> I8Vector4: ...
    @property
    def slls(self) -> I8Vector4: ...
    @property
    def sllt(self) -> I8Vector4: ...
    @property
    def sllq(self) -> I8Vector4: ...
    @property
    def sllp(self) -> I8Vector4: ...
    @property
    def sllo(self) -> I8Vector4: ...
    @property
    def slll(self) -> I8Vector4: ...
    @property
    def tsss(self) -> I8Vector4: ...
    @property
    def tsst(self) -> I8Vector4: ...
    @property
    def tssq(self) -> I8Vector4: ...
    @property
    def tssp(self) -> I8Vector4: ...
    @property
    def tsso(self) -> I8Vector4: ...
    @property
    def tssl(self) -> I8Vector4: ...
    @property
    def tsts(self) -> I8Vector4: ...
    @property
    def tstt(self) -> I8Vector4: ...
    @property
    def tstq(self) -> I8Vector4: ...
    @property
    def tstp(self) -> I8Vector4: ...
    @property
    def tsto(self) -> I8Vector4: ...
    @property
    def tstl(self) -> I8Vector4: ...
    @property
    def tsqs(self) -> I8Vector4: ...
    @property
    def tsqt(self) -> I8Vector4: ...
    @property
    def tsqq(self) -> I8Vector4: ...
    @property
    def tsqp(self) -> I8Vector4: ...
    @property
    def tsqo(self) -> I8Vector4: ...
    @property
    def tsql(self) -> I8Vector4: ...
    @property
    def tsps(self) -> I8Vector4: ...
    @property
    def tspt(self) -> I8Vector4: ...
    @property
    def tspq(self) -> I8Vector4: ...
    @property
    def tspp(self) -> I8Vector4: ...
    @property
    def tspo(self) -> I8Vector4: ...
    @property
    def tspl(self) -> I8Vector4: ...
    @property
    def tsos(self) -> I8Vector4: ...
    @property
    def tsot(self) -> I8Vector4: ...
    @property
    def tsoq(self) -> I8Vector4: ...
    @property
    def tsop(self) -> I8Vector4: ...
    @property
    def tsoo(self) -> I8Vector4: ...
    @property
    def tsol(self) -> I8Vector4: ...
    @property
    def tsls(self) -> I8Vector4: ...
    @property
    def tslt(self) -> I8Vector4: ...
    @property
    def tslq(self) -> I8Vector4: ...
    @property
    def tslp(self) -> I8Vector4: ...
    @property
    def tslo(self) -> I8Vector4: ...
    @property
    def tsll(self) -> I8Vector4: ...
    @property
    def ttss(self) -> I8Vector4: ...
    @property
    def ttst(self) -> I8Vector4: ...
    @property
    def ttsq(self) -> I8Vector4: ...
    @property
    def ttsp(self) -> I8Vector4: ...
    @property
    def ttso(self) -> I8Vector4: ...
    @property
    def ttsl(self) -> I8Vector4: ...
    @property
    def ttts(self) -> I8Vector4: ...
    @property
    def tttt(self) -> I8Vector4: ...
    @property
    def tttq(self) -> I8Vector4: ...
    @property
    def tttp(self) -> I8Vector4: ...
    @property
    def ttto(self) -> I8Vector4: ...
    @property
    def tttl(self) -> I8Vector4: ...
    @property
    def ttqs(self) -> I8Vector4: ...
    @property
    def ttqt(self) -> I8Vector4: ...
    @property
    def ttqq(self) -> I8Vector4: ...
    @property
    def ttqp(self) -> I8Vector4: ...
    @property
    def ttqo(self) -> I8Vector4: ...
    @property
    def ttql(self) -> I8Vector4: ...
    @property
    def ttps(self) -> I8Vector4: ...
    @property
    def ttpt(self) -> I8Vector4: ...
    @property
    def ttpq(self) -> I8Vector4: ...
    @property
    def ttpp(self) -> I8Vector4: ...
    @property
    def ttpo(self) -> I8Vector4: ...
    @property
    def ttpl(self) -> I8Vector4: ...
    @property
    def ttos(self) -> I8Vector4: ...
    @property
    def ttot(self) -> I8Vector4: ...
    @property
    def ttoq(self) -> I8Vector4: ...
    @property
    def ttop(self) -> I8Vector4: ...
    @property
    def ttoo(self) -> I8Vector4: ...
    @property
    def ttol(self) -> I8Vector4: ...
    @property
    def ttls(self) -> I8Vector4: ...
    @property
    def ttlt(self) -> I8Vector4: ...
    @property
    def ttlq(self) -> I8Vector4: ...
    @property
    def ttlp(self) -> I8Vector4: ...
    @property
    def ttlo(self) -> I8Vector4: ...
    @property
    def ttll(self) -> I8Vector4: ...
    @property
    def tqss(self) -> I8Vector4: ...
    @property
    def tqst(self) -> I8Vector4: ...
    @property
    def tqsq(self) -> I8Vector4: ...
    @property
    def tqsp(self) -> I8Vector4: ...
    @property
    def tqso(self) -> I8Vector4: ...
    @property
    def tqsl(self) -> I8Vector4: ...
    @property
    def tqts(self) -> I8Vector4: ...
    @property
    def tqtt(self) -> I8Vector4: ...
    @property
    def tqtq(self) -> I8Vector4: ...
    @property
    def tqtp(self) -> I8Vector4: ...
    @property
    def tqto(self) -> I8Vector4: ...
    @property
    def tqtl(self) -> I8Vector4: ...
    @property
    def tqqs(self) -> I8Vector4: ...
    @property
    def tqqt(self) -> I8Vector4: ...
    @property
    def tqqq(self) -> I8Vector4: ...
    @property
    def tqqp(self) -> I8Vector4: ...
    @property
    def tqqo(self) -> I8Vector4: ...
    @property
    def tqql(self) -> I8Vector4: ...
    @property
    def tqps(self) -> I8Vector4: ...
    @property
    def tqpt(self) -> I8Vector4: ...
    @property
    def tqpq(self) -> I8Vector4: ...
    @property
    def tqpp(self) -> I8Vector4: ...
    @property
    def tqpo(self) -> I8Vector4: ...
    @property
    def tqpl(self) -> I8Vector4: ...
    @property
    def tqos(self) -> I8Vector4: ...
    @property
    def tqot(self) -> I8Vector4: ...
    @property
    def tqoq(self) -> I8Vector4: ...
    @property
    def tqop(self) -> I8Vector4: ...
    @property
    def tqoo(self) -> I8Vector4: ...
    @property
    def tqol(self) -> I8Vector4: ...
    @property
    def tqls(self) -> I8Vector4: ...
    @property
    def tqlt(self) -> I8Vector4: ...
    @property
    def tqlq(self) -> I8Vector4: ...
    @property
    def tqlp(self) -> I8Vector4: ...
    @property
    def tqlo(self) -> I8Vector4: ...
    @property
    def tqll(self) -> I8Vector4: ...
    @property
    def tpss(self) -> I8Vector4: ...
    @property
    def tpst(self) -> I8Vector4: ...
    @property
    def tpsq(self) -> I8Vector4: ...
    @property
    def tpsp(self) -> I8Vector4: ...
    @property
    def tpso(self) -> I8Vector4: ...
    @property
    def tpsl(self) -> I8Vector4: ...
    @property
    def tpts(self) -> I8Vector4: ...
    @property
    def tptt(self) -> I8Vector4: ...
    @property
    def tptq(self) -> I8Vector4: ...
    @property
    def tptp(self) -> I8Vector4: ...
    @property
    def tpto(self) -> I8Vector4: ...
    @property
    def tptl(self) -> I8Vector4: ...
    @property
    def tpqs(self) -> I8Vector4: ...
    @property
    def tpqt(self) -> I8Vector4: ...
    @property
    def tpqq(self) -> I8Vector4: ...
    @property
    def tpqp(self) -> I8Vector4: ...
    @property
    def tpqo(self) -> I8Vector4: ...
    @property
    def tpql(self) -> I8Vector4: ...
    @property
    def tpps(self) -> I8Vector4: ...
    @property
    def tppt(self) -> I8Vector4: ...
    @property
    def tppq(self) -> I8Vector4: ...
    @property
    def tppp(self) -> I8Vector4: ...
    @property
    def tppo(self) -> I8Vector4: ...
    @property
    def tppl(self) -> I8Vector4: ...
    @property
    def tpos(self) -> I8Vector4: ...
    @property
    def tpot(self) -> I8Vector4: ...
    @property
    def tpoq(self) -> I8Vector4: ...
    @property
    def tpop(self) -> I8Vector4: ...
    @property
    def tpoo(self) -> I8Vector4: ...
    @property
    def tpol(self) -> I8Vector4: ...
    @property
    def tpls(self) -> I8Vector4: ...
    @property
    def tplt(self) -> I8Vector4: ...
    @property
    def tplq(self) -> I8Vector4: ...
    @property
    def tplp(self) -> I8Vector4: ...
    @property
    def tplo(self) -> I8Vector4: ...
    @property
    def tpll(self) -> I8Vector4: ...
    @property
    def toss(self) -> I8Vector4: ...
    @property
    def tost(self) -> I8Vector4: ...
    @property
    def tosq(self) -> I8Vector4: ...
    @property
    def tosp(self) -> I8Vector4: ...
    @property
    def toso(self) -> I8Vector4: ...
    @property
    def tosl(self) -> I8Vector4: ...
    @property
    def tots(self) -> I8Vector4: ...
    @property
    def tott(self) -> I8Vector4: ...
    @property
    def totq(self) -> I8Vector4: ...
    @property
    def totp(self) -> I8Vector4: ...
    @property
    def toto(self) -> I8Vector4: ...
    @property
    def totl(self) -> I8Vector4: ...
    @property
    def toqs(self) -> I8Vector4: ...
    @property
    def toqt(self) -> I8Vector4: ...
    @property
    def toqq(self) -> I8Vector4: ...
    @property
    def toqp(self) -> I8Vector4: ...
    @property
    def toqo(self) -> I8Vector4: ...
    @property
    def toql(self) -> I8Vector4: ...
    @property
    def tops(self) -> I8Vector4: ...
    @property
    def topt(self) -> I8Vector4: ...
    @property
    def topq(self) -> I8Vector4: ...
    @property
    def topp(self) -> I8Vector4: ...
    @property
    def topo(self) -> I8Vector4: ...
    @property
    def topl(self) -> I8Vector4: ...
    @property
    def toos(self) -> I8Vector4: ...
    @property
    def toot(self) -> I8Vector4: ...
    @property
    def tooq(self) -> I8Vector4: ...
    @property
    def toop(self) -> I8Vector4: ...
    @property
    def tooo(self) -> I8Vector4: ...
    @property
    def tool(self) -> I8Vector4: ...
    @property
    def tols(self) -> I8Vector4: ...
    @property
    def tolt(self) -> I8Vector4: ...
    @property
    def tolq(self) -> I8Vector4: ...
    @property
    def tolp(self) -> I8Vector4: ...
    @property
    def tolo(self) -> I8Vector4: ...
    @property
    def toll(self) -> I8Vector4: ...
    @property
    def tlss(self) -> I8Vector4: ...
    @property
    def tlst(self) -> I8Vector4: ...
    @property
    def tlsq(self) -> I8Vector4: ...
    @property
    def tlsp(self) -> I8Vector4: ...
    @property
    def tlso(self) -> I8Vector4: ...
    @property
    def tlsl(self) -> I8Vector4: ...
    @property
    def tlts(self) -> I8Vector4: ...
    @property
    def tltt(self) -> I8Vector4: ...
    @property
    def tltq(self) -> I8Vector4: ...
    @property
    def tltp(self) -> I8Vector4: ...
    @property
    def tlto(self) -> I8Vector4: ...
    @property
    def tltl(self) -> I8Vector4: ...
    @property
    def tlqs(self) -> I8Vector4: ...
    @property
    def tlqt(self) -> I8Vector4: ...
    @property
    def tlqq(self) -> I8Vector4: ...
    @property
    def tlqp(self) -> I8Vector4: ...
    @property
    def tlqo(self) -> I8Vector4: ...
    @property
    def tlql(self) -> I8Vector4: ...
    @property
    def tlps(self) -> I8Vector4: ...
    @property
    def tlpt(self) -> I8Vector4: ...
    @property
    def tlpq(self) -> I8Vector4: ...
    @property
    def tlpp(self) -> I8Vector4: ...
    @property
    def tlpo(self) -> I8Vector4: ...
    @property
    def tlpl(self) -> I8Vector4: ...
    @property
    def tlos(self) -> I8Vector4: ...
    @property
    def tlot(self) -> I8Vector4: ...
    @property
    def tloq(self) -> I8Vector4: ...
    @property
    def tlop(self) -> I8Vector4: ...
    @property
    def tloo(self) -> I8Vector4: ...
    @property
    def tlol(self) -> I8Vector4: ...
    @property
    def tlls(self) -> I8Vector4: ...
    @property
    def tllt(self) -> I8Vector4: ...
    @property
    def tllq(self) -> I8Vector4: ...
    @property
    def tllp(self) -> I8Vector4: ...
    @property
    def tllo(self) -> I8Vector4: ...
    @property
    def tlll(self) -> I8Vector4: ...
    @property
    def qsss(self) -> I8Vector4: ...
    @property
    def qsst(self) -> I8Vector4: ...
    @property
    def qssq(self) -> I8Vector4: ...
    @property
    def qssp(self) -> I8Vector4: ...
    @property
    def qsso(self) -> I8Vector4: ...
    @property
    def qssl(self) -> I8Vector4: ...
    @property
    def qsts(self) -> I8Vector4: ...
    @property
    def qstt(self) -> I8Vector4: ...
    @property
    def qstq(self) -> I8Vector4: ...
    @property
    def qstp(self) -> I8Vector4: ...
    @property
    def qsto(self) -> I8Vector4: ...
    @property
    def qstl(self) -> I8Vector4: ...
    @property
    def qsqs(self) -> I8Vector4: ...
    @property
    def qsqt(self) -> I8Vector4: ...
    @property
    def qsqq(self) -> I8Vector4: ...
    @property
    def qsqp(self) -> I8Vector4: ...
    @property
    def qsqo(self) -> I8Vector4: ...
    @property
    def qsql(self) -> I8Vector4: ...
    @property
    def qsps(self) -> I8Vector4: ...
    @property
    def qspt(self) -> I8Vector4: ...
    @property
    def qspq(self) -> I8Vector4: ...
    @property
    def qspp(self) -> I8Vector4: ...
    @property
    def qspo(self) -> I8Vector4: ...
    @property
    def qspl(self) -> I8Vector4: ...
    @property
    def qsos(self) -> I8Vector4: ...
    @property
    def qsot(self) -> I8Vector4: ...
    @property
    def qsoq(self) -> I8Vector4: ...
    @property
    def qsop(self) -> I8Vector4: ...
    @property
    def qsoo(self) -> I8Vector4: ...
    @property
    def qsol(self) -> I8Vector4: ...
    @property
    def qsls(self) -> I8Vector4: ...
    @property
    def qslt(self) -> I8Vector4: ...
    @property
    def qslq(self) -> I8Vector4: ...
    @property
    def qslp(self) -> I8Vector4: ...
    @property
    def qslo(self) -> I8Vector4: ...
    @property
    def qsll(self) -> I8Vector4: ...
    @property
    def qtss(self) -> I8Vector4: ...
    @property
    def qtst(self) -> I8Vector4: ...
    @property
    def qtsq(self) -> I8Vector4: ...
    @property
    def qtsp(self) -> I8Vector4: ...
    @property
    def qtso(self) -> I8Vector4: ...
    @property
    def qtsl(self) -> I8Vector4: ...
    @property
    def qtts(self) -> I8Vector4: ...
    @property
    def qttt(self) -> I8Vector4: ...
    @property
    def qttq(self) -> I8Vector4: ...
    @property
    def qttp(self) -> I8Vector4: ...
    @property
    def qtto(self) -> I8Vector4: ...
    @property
    def qttl(self) -> I8Vector4: ...
    @property
    def qtqs(self) -> I8Vector4: ...
    @property
    def qtqt(self) -> I8Vector4: ...
    @property
    def qtqq(self) -> I8Vector4: ...
    @property
    def qtqp(self) -> I8Vector4: ...
    @property
    def qtqo(self) -> I8Vector4: ...
    @property
    def qtql(self) -> I8Vector4: ...
    @property
    def qtps(self) -> I8Vector4: ...
    @property
    def qtpt(self) -> I8Vector4: ...
    @property
    def qtpq(self) -> I8Vector4: ...
    @property
    def qtpp(self) -> I8Vector4: ...
    @property
    def qtpo(self) -> I8Vector4: ...
    @property
    def qtpl(self) -> I8Vector4: ...
    @property
    def qtos(self) -> I8Vector4: ...
    @property
    def qtot(self) -> I8Vector4: ...
    @property
    def qtoq(self) -> I8Vector4: ...
    @property
    def qtop(self) -> I8Vector4: ...
    @property
    def qtoo(self) -> I8Vector4: ...
    @property
    def qtol(self) -> I8Vector4: ...
    @property
    def qtls(self) -> I8Vector4: ...
    @property
    def qtlt(self) -> I8Vector4: ...
    @property
    def qtlq(self) -> I8Vector4: ...
    @property
    def qtlp(self) -> I8Vector4: ...
    @property
    def qtlo(self) -> I8Vector4: ...
    @property
    def qtll(self) -> I8Vector4: ...
    @property
    def qqss(self) -> I8Vector4: ...
    @property
    def qqst(self) -> I8Vector4: ...
    @property
    def qqsq(self) -> I8Vector4: ...
    @property
    def qqsp(self) -> I8Vector4: ...
    @property
    def qqso(self) -> I8Vector4: ...
    @property
    def qqsl(self) -> I8Vector4: ...
    @property
    def qqts(self) -> I8Vector4: ...
    @property
    def qqtt(self) -> I8Vector4: ...
    @property
    def qqtq(self) -> I8Vector4: ...
    @property
    def qqtp(self) -> I8Vector4: ...
    @property
    def qqto(self) -> I8Vector4: ...
    @property
    def qqtl(self) -> I8Vector4: ...
    @property
    def qqqs(self) -> I8Vector4: ...
    @property
    def qqqt(self) -> I8Vector4: ...
    @property
    def qqqq(self) -> I8Vector4: ...
    @property
    def qqqp(self) -> I8Vector4: ...
    @property
    def qqqo(self) -> I8Vector4: ...
    @property
    def qqql(self) -> I8Vector4: ...
    @property
    def qqps(self) -> I8Vector4: ...
    @property
    def qqpt(self) -> I8Vector4: ...
    @property
    def qqpq(self) -> I8Vector4: ...
    @property
    def qqpp(self) -> I8Vector4: ...
    @property
    def qqpo(self) -> I8Vector4: ...
    @property
    def qqpl(self) -> I8Vector4: ...
    @property
    def qqos(self) -> I8Vector4: ...
    @property
    def qqot(self) -> I8Vector4: ...
    @property
    def qqoq(self) -> I8Vector4: ...
    @property
    def qqop(self) -> I8Vector4: ...
    @property
    def qqoo(self) -> I8Vector4: ...
    @property
    def qqol(self) -> I8Vector4: ...
    @property
    def qqls(self) -> I8Vector4: ...
    @property
    def qqlt(self) -> I8Vector4: ...
    @property
    def qqlq(self) -> I8Vector4: ...
    @property
    def qqlp(self) -> I8Vector4: ...
    @property
    def qqlo(self) -> I8Vector4: ...
    @property
    def qqll(self) -> I8Vector4: ...
    @property
    def qpss(self) -> I8Vector4: ...
    @property
    def qpst(self) -> I8Vector4: ...
    @property
    def qpsq(self) -> I8Vector4: ...
    @property
    def qpsp(self) -> I8Vector4: ...
    @property
    def qpso(self) -> I8Vector4: ...
    @property
    def qpsl(self) -> I8Vector4: ...
    @property
    def qpts(self) -> I8Vector4: ...
    @property
    def qptt(self) -> I8Vector4: ...
    @property
    def qptq(self) -> I8Vector4: ...
    @property
    def qptp(self) -> I8Vector4: ...
    @property
    def qpto(self) -> I8Vector4: ...
    @property
    def qptl(self) -> I8Vector4: ...
    @property
    def qpqs(self) -> I8Vector4: ...
    @property
    def qpqt(self) -> I8Vector4: ...
    @property
    def qpqq(self) -> I8Vector4: ...
    @property
    def qpqp(self) -> I8Vector4: ...
    @property
    def qpqo(self) -> I8Vector4: ...
    @property
    def qpql(self) -> I8Vector4: ...
    @property
    def qpps(self) -> I8Vector4: ...
    @property
    def qppt(self) -> I8Vector4: ...
    @property
    def qppq(self) -> I8Vector4: ...
    @property
    def qppp(self) -> I8Vector4: ...
    @property
    def qppo(self) -> I8Vector4: ...
    @property
    def qppl(self) -> I8Vector4: ...
    @property
    def qpos(self) -> I8Vector4: ...
    @property
    def qpot(self) -> I8Vector4: ...
    @property
    def qpoq(self) -> I8Vector4: ...
    @property
    def qpop(self) -> I8Vector4: ...
    @property
    def qpoo(self) -> I8Vector4: ...
    @property
    def qpol(self) -> I8Vector4: ...
    @property
    def qpls(self) -> I8Vector4: ...
    @property
    def qplt(self) -> I8Vector4: ...
    @property
    def qplq(self) -> I8Vector4: ...
    @property
    def qplp(self) -> I8Vector4: ...
    @property
    def qplo(self) -> I8Vector4: ...
    @property
    def qpll(self) -> I8Vector4: ...
    @property
    def qoss(self) -> I8Vector4: ...
    @property
    def qost(self) -> I8Vector4: ...
    @property
    def qosq(self) -> I8Vector4: ...
    @property
    def qosp(self) -> I8Vector4: ...
    @property
    def qoso(self) -> I8Vector4: ...
    @property
    def qosl(self) -> I8Vector4: ...
    @property
    def qots(self) -> I8Vector4: ...
    @property
    def qott(self) -> I8Vector4: ...
    @property
    def qotq(self) -> I8Vector4: ...
    @property
    def qotp(self) -> I8Vector4: ...
    @property
    def qoto(self) -> I8Vector4: ...
    @property
    def qotl(self) -> I8Vector4: ...
    @property
    def qoqs(self) -> I8Vector4: ...
    @property
    def qoqt(self) -> I8Vector4: ...
    @property
    def qoqq(self) -> I8Vector4: ...
    @property
    def qoqp(self) -> I8Vector4: ...
    @property
    def qoqo(self) -> I8Vector4: ...
    @property
    def qoql(self) -> I8Vector4: ...
    @property
    def qops(self) -> I8Vector4: ...
    @property
    def qopt(self) -> I8Vector4: ...
    @property
    def qopq(self) -> I8Vector4: ...
    @property
    def qopp(self) -> I8Vector4: ...
    @property
    def qopo(self) -> I8Vector4: ...
    @property
    def qopl(self) -> I8Vector4: ...
    @property
    def qoos(self) -> I8Vector4: ...
    @property
    def qoot(self) -> I8Vector4: ...
    @property
    def qooq(self) -> I8Vector4: ...
    @property
    def qoop(self) -> I8Vector4: ...
    @property
    def qooo(self) -> I8Vector4: ...
    @property
    def qool(self) -> I8Vector4: ...
    @property
    def qols(self) -> I8Vector4: ...
    @property
    def qolt(self) -> I8Vector4: ...
    @property
    def qolq(self) -> I8Vector4: ...
    @property
    def qolp(self) -> I8Vector4: ...
    @property
    def qolo(self) -> I8Vector4: ...
    @property
    def qoll(self) -> I8Vector4: ...
    @property
    def qlss(self) -> I8Vector4: ...
    @property
    def qlst(self) -> I8Vector4: ...
    @property
    def qlsq(self) -> I8Vector4: ...
    @property
    def qlsp(self) -> I8Vector4: ...
    @property
    def qlso(self) -> I8Vector4: ...
    @property
    def qlsl(self) -> I8Vector4: ...
    @property
    def qlts(self) -> I8Vector4: ...
    @property
    def qltt(self) -> I8Vector4: ...
    @property
    def qltq(self) -> I8Vector4: ...
    @property
    def qltp(self) -> I8Vector4: ...
    @property
    def qlto(self) -> I8Vector4: ...
    @property
    def qltl(self) -> I8Vector4: ...
    @property
    def qlqs(self) -> I8Vector4: ...
    @property
    def qlqt(self) -> I8Vector4: ...
    @property
    def qlqq(self) -> I8Vector4: ...
    @property
    def qlqp(self) -> I8Vector4: ...
    @property
    def qlqo(self) -> I8Vector4: ...
    @property
    def qlql(self) -> I8Vector4: ...
    @property
    def qlps(self) -> I8Vector4: ...
    @property
    def qlpt(self) -> I8Vector4: ...
    @property
    def qlpq(self) -> I8Vector4: ...
    @property
    def qlpp(self) -> I8Vector4: ...
    @property
    def qlpo(self) -> I8Vector4: ...
    @property
    def qlpl(self) -> I8Vector4: ...
    @property
    def qlos(self) -> I8Vector4: ...
    @property
    def qlot(self) -> I8Vector4: ...
    @property
    def qloq(self) -> I8Vector4: ...
    @property
    def qlop(self) -> I8Vector4: ...
    @property
    def qloo(self) -> I8Vector4: ...
    @property
    def qlol(self) -> I8Vector4: ...
    @property
    def qlls(self) -> I8Vector4: ...
    @property
    def qllt(self) -> I8Vector4: ...
    @property
    def qllq(self) -> I8Vector4: ...
    @property
    def qllp(self) -> I8Vector4: ...
    @property
    def qllo(self) -> I8Vector4: ...
    @property
    def qlll(self) -> I8Vector4: ...
    @property
    def psss(self) -> I8Vector4: ...
    @property
    def psst(self) -> I8Vector4: ...
    @property
    def pssq(self) -> I8Vector4: ...
    @property
    def pssp(self) -> I8Vector4: ...
    @property
    def psso(self) -> I8Vector4: ...
    @property
    def pssl(self) -> I8Vector4: ...
    @property
    def psts(self) -> I8Vector4: ...
    @property
    def pstt(self) -> I8Vector4: ...
    @property
    def pstq(self) -> I8Vector4: ...
    @property
    def pstp(self) -> I8Vector4: ...
    @property
    def psto(self) -> I8Vector4: ...
    @property
    def pstl(self) -> I8Vector4: ...
    @property
    def psqs(self) -> I8Vector4: ...
    @property
    def psqt(self) -> I8Vector4: ...
    @property
    def psqq(self) -> I8Vector4: ...
    @property
    def psqp(self) -> I8Vector4: ...
    @property
    def psqo(self) -> I8Vector4: ...
    @property
    def psql(self) -> I8Vector4: ...
    @property
    def psps(self) -> I8Vector4: ...
    @property
    def pspt(self) -> I8Vector4: ...
    @property
    def pspq(self) -> I8Vector4: ...
    @property
    def pspp(self) -> I8Vector4: ...
    @property
    def pspo(self) -> I8Vector4: ...
    @property
    def pspl(self) -> I8Vector4: ...
    @property
    def psos(self) -> I8Vector4: ...
    @property
    def psot(self) -> I8Vector4: ...
    @property
    def psoq(self) -> I8Vector4: ...
    @property
    def psop(self) -> I8Vector4: ...
    @property
    def psoo(self) -> I8Vector4: ...
    @property
    def psol(self) -> I8Vector4: ...
    @property
    def psls(self) -> I8Vector4: ...
    @property
    def pslt(self) -> I8Vector4: ...
    @property
    def pslq(self) -> I8Vector4: ...
    @property
    def pslp(self) -> I8Vector4: ...
    @property
    def pslo(self) -> I8Vector4: ...
    @property
    def psll(self) -> I8Vector4: ...
    @property
    def ptss(self) -> I8Vector4: ...
    @property
    def ptst(self) -> I8Vector4: ...
    @property
    def ptsq(self) -> I8Vector4: ...
    @property
    def ptsp(self) -> I8Vector4: ...
    @property
    def ptso(self) -> I8Vector4: ...
    @property
    def ptsl(self) -> I8Vector4: ...
    @property
    def ptts(self) -> I8Vector4: ...
    @property
    def pttt(self) -> I8Vector4: ...
    @property
    def pttq(self) -> I8Vector4: ...
    @property
    def pttp(self) -> I8Vector4: ...
    @property
    def ptto(self) -> I8Vector4: ...
    @property
    def pttl(self) -> I8Vector4: ...
    @property
    def ptqs(self) -> I8Vector4: ...
    @property
    def ptqt(self) -> I8Vector4: ...
    @property
    def ptqq(self) -> I8Vector4: ...
    @property
    def ptqp(self) -> I8Vector4: ...
    @property
    def ptqo(self) -> I8Vector4: ...
    @property
    def ptql(self) -> I8Vector4: ...
    @property
    def ptps(self) -> I8Vector4: ...
    @property
    def ptpt(self) -> I8Vector4: ...
    @property
    def ptpq(self) -> I8Vector4: ...
    @property
    def ptpp(self) -> I8Vector4: ...
    @property
    def ptpo(self) -> I8Vector4: ...
    @property
    def ptpl(self) -> I8Vector4: ...
    @property
    def ptos(self) -> I8Vector4: ...
    @property
    def ptot(self) -> I8Vector4: ...
    @property
    def ptoq(self) -> I8Vector4: ...
    @property
    def ptop(self) -> I8Vector4: ...
    @property
    def ptoo(self) -> I8Vector4: ...
    @property
    def ptol(self) -> I8Vector4: ...
    @property
    def ptls(self) -> I8Vector4: ...
    @property
    def ptlt(self) -> I8Vector4: ...
    @property
    def ptlq(self) -> I8Vector4: ...
    @property
    def ptlp(self) -> I8Vector4: ...
    @property
    def ptlo(self) -> I8Vector4: ...
    @property
    def ptll(self) -> I8Vector4: ...
    @property
    def pqss(self) -> I8Vector4: ...
    @property
    def pqst(self) -> I8Vector4: ...
    @property
    def pqsq(self) -> I8Vector4: ...
    @property
    def pqsp(self) -> I8Vector4: ...
    @property
    def pqso(self) -> I8Vector4: ...
    @property
    def pqsl(self) -> I8Vector4: ...
    @property
    def pqts(self) -> I8Vector4: ...
    @property
    def pqtt(self) -> I8Vector4: ...
    @property
    def pqtq(self) -> I8Vector4: ...
    @property
    def pqtp(self) -> I8Vector4: ...
    @property
    def pqto(self) -> I8Vector4: ...
    @property
    def pqtl(self) -> I8Vector4: ...
    @property
    def pqqs(self) -> I8Vector4: ...
    @property
    def pqqt(self) -> I8Vector4: ...
    @property
    def pqqq(self) -> I8Vector4: ...
    @property
    def pqqp(self) -> I8Vector4: ...
    @property
    def pqqo(self) -> I8Vector4: ...
    @property
    def pqql(self) -> I8Vector4: ...
    @property
    def pqps(self) -> I8Vector4: ...
    @property
    def pqpt(self) -> I8Vector4: ...
    @property
    def pqpq(self) -> I8Vector4: ...
    @property
    def pqpp(self) -> I8Vector4: ...
    @property
    def pqpo(self) -> I8Vector4: ...
    @property
    def pqpl(self) -> I8Vector4: ...
    @property
    def pqos(self) -> I8Vector4: ...
    @property
    def pqot(self) -> I8Vector4: ...
    @property
    def pqoq(self) -> I8Vector4: ...
    @property
    def pqop(self) -> I8Vector4: ...
    @property
    def pqoo(self) -> I8Vector4: ...
    @property
    def pqol(self) -> I8Vector4: ...
    @property
    def pqls(self) -> I8Vector4: ...
    @property
    def pqlt(self) -> I8Vector4: ...
    @property
    def pqlq(self) -> I8Vector4: ...
    @property
    def pqlp(self) -> I8Vector4: ...
    @property
    def pqlo(self) -> I8Vector4: ...
    @property
    def pqll(self) -> I8Vector4: ...
    @property
    def ppss(self) -> I8Vector4: ...
    @property
    def ppst(self) -> I8Vector4: ...
    @property
    def ppsq(self) -> I8Vector4: ...
    @property
    def ppsp(self) -> I8Vector4: ...
    @property
    def ppso(self) -> I8Vector4: ...
    @property
    def ppsl(self) -> I8Vector4: ...
    @property
    def ppts(self) -> I8Vector4: ...
    @property
    def pptt(self) -> I8Vector4: ...
    @property
    def pptq(self) -> I8Vector4: ...
    @property
    def pptp(self) -> I8Vector4: ...
    @property
    def ppto(self) -> I8Vector4: ...
    @property
    def pptl(self) -> I8Vector4: ...
    @property
    def ppqs(self) -> I8Vector4: ...
    @property
    def ppqt(self) -> I8Vector4: ...
    @property
    def ppqq(self) -> I8Vector4: ...
    @property
    def ppqp(self) -> I8Vector4: ...
    @property
    def ppqo(self) -> I8Vector4: ...
    @property
    def ppql(self) -> I8Vector4: ...
    @property
    def ppps(self) -> I8Vector4: ...
    @property
    def pppt(self) -> I8Vector4: ...
    @property
    def pppq(self) -> I8Vector4: ...
    @property
    def pppp(self) -> I8Vector4: ...
    @property
    def pppo(self) -> I8Vector4: ...
    @property
    def pppl(self) -> I8Vector4: ...
    @property
    def ppos(self) -> I8Vector4: ...
    @property
    def ppot(self) -> I8Vector4: ...
    @property
    def ppoq(self) -> I8Vector4: ...
    @property
    def ppop(self) -> I8Vector4: ...
    @property
    def ppoo(self) -> I8Vector4: ...
    @property
    def ppol(self) -> I8Vector4: ...
    @property
    def ppls(self) -> I8Vector4: ...
    @property
    def pplt(self) -> I8Vector4: ...
    @property
    def pplq(self) -> I8Vector4: ...
    @property
    def pplp(self) -> I8Vector4: ...
    @property
    def pplo(self) -> I8Vector4: ...
    @property
    def ppll(self) -> I8Vector4: ...
    @property
    def poss(self) -> I8Vector4: ...
    @property
    def post(self) -> I8Vector4: ...
    @property
    def posq(self) -> I8Vector4: ...
    @property
    def posp(self) -> I8Vector4: ...
    @property
    def poso(self) -> I8Vector4: ...
    @property
    def posl(self) -> I8Vector4: ...
    @property
    def pots(self) -> I8Vector4: ...
    @property
    def pott(self) -> I8Vector4: ...
    @property
    def potq(self) -> I8Vector4: ...
    @property
    def potp(self) -> I8Vector4: ...
    @property
    def poto(self) -> I8Vector4: ...
    @property
    def potl(self) -> I8Vector4: ...
    @property
    def poqs(self) -> I8Vector4: ...
    @property
    def poqt(self) -> I8Vector4: ...
    @property
    def poqq(self) -> I8Vector4: ...
    @property
    def poqp(self) -> I8Vector4: ...
    @property
    def poqo(self) -> I8Vector4: ...
    @property
    def poql(self) -> I8Vector4: ...
    @property
    def pops(self) -> I8Vector4: ...
    @property
    def popt(self) -> I8Vector4: ...
    @property
    def popq(self) -> I8Vector4: ...
    @property
    def popp(self) -> I8Vector4: ...
    @property
    def popo(self) -> I8Vector4: ...
    @property
    def popl(self) -> I8Vector4: ...
    @property
    def poos(self) -> I8Vector4: ...
    @property
    def poot(self) -> I8Vector4: ...
    @property
    def pooq(self) -> I8Vector4: ...
    @property
    def poop(self) -> I8Vector4: ...
    @property
    def pooo(self) -> I8Vector4: ...
    @property
    def pool(self) -> I8Vector4: ...
    @property
    def pols(self) -> I8Vector4: ...
    @property
    def polt(self) -> I8Vector4: ...
    @property
    def polq(self) -> I8Vector4: ...
    @property
    def polp(self) -> I8Vector4: ...
    @property
    def polo(self) -> I8Vector4: ...
    @property
    def poll(self) -> I8Vector4: ...
    @property
    def plss(self) -> I8Vector4: ...
    @property
    def plst(self) -> I8Vector4: ...
    @property
    def plsq(self) -> I8Vector4: ...
    @property
    def plsp(self) -> I8Vector4: ...
    @property
    def plso(self) -> I8Vector4: ...
    @property
    def plsl(self) -> I8Vector4: ...
    @property
    def plts(self) -> I8Vector4: ...
    @property
    def pltt(self) -> I8Vector4: ...
    @property
    def pltq(self) -> I8Vector4: ...
    @property
    def pltp(self) -> I8Vector4: ...
    @property
    def plto(self) -> I8Vector4: ...
    @property
    def pltl(self) -> I8Vector4: ...
    @property
    def plqs(self) -> I8Vector4: ...
    @property
    def plqt(self) -> I8Vector4: ...
    @property
    def plqq(self) -> I8Vector4: ...
    @property
    def plqp(self) -> I8Vector4: ...
    @property
    def plqo(self) -> I8Vector4: ...
    @property
    def plql(self) -> I8Vector4: ...
    @property
    def plps(self) -> I8Vector4: ...
    @property
    def plpt(self) -> I8Vector4: ...
    @property
    def plpq(self) -> I8Vector4: ...
    @property
    def plpp(self) -> I8Vector4: ...
    @property
    def plpo(self) -> I8Vector4: ...
    @property
    def plpl(self) -> I8Vector4: ...
    @property
    def plos(self) -> I8Vector4: ...
    @property
    def plot(self) -> I8Vector4: ...
    @property
    def ploq(self) -> I8Vector4: ...
    @property
    def plop(self) -> I8Vector4: ...
    @property
    def ploo(self) -> I8Vector4: ...
    @property
    def plol(self) -> I8Vector4: ...
    @property
    def plls(self) -> I8Vector4: ...
    @property
    def pllt(self) -> I8Vector4: ...
    @property
    def pllq(self) -> I8Vector4: ...
    @property
    def pllp(self) -> I8Vector4: ...
    @property
    def pllo(self) -> I8Vector4: ...
    @property
    def plll(self) -> I8Vector4: ...
    @property
    def osss(self) -> I8Vector4: ...
    @property
    def osst(self) -> I8Vector4: ...
    @property
    def ossq(self) -> I8Vector4: ...
    @property
    def ossp(self) -> I8Vector4: ...
    @property
    def osso(self) -> I8Vector4: ...
    @property
    def ossl(self) -> I8Vector4: ...
    @property
    def osts(self) -> I8Vector4: ...
    @property
    def ostt(self) -> I8Vector4: ...
    @property
    def ostq(self) -> I8Vector4: ...
    @property
    def ostp(self) -> I8Vector4: ...
    @property
    def osto(self) -> I8Vector4: ...
    @property
    def ostl(self) -> I8Vector4: ...
    @property
    def osqs(self) -> I8Vector4: ...
    @property
    def osqt(self) -> I8Vector4: ...
    @property
    def osqq(self) -> I8Vector4: ...
    @property
    def osqp(self) -> I8Vector4: ...
    @property
    def osqo(self) -> I8Vector4: ...
    @property
    def osql(self) -> I8Vector4: ...
    @property
    def osps(self) -> I8Vector4: ...
    @property
    def ospt(self) -> I8Vector4: ...
    @property
    def ospq(self) -> I8Vector4: ...
    @property
    def ospp(self) -> I8Vector4: ...
    @property
    def ospo(self) -> I8Vector4: ...
    @property
    def ospl(self) -> I8Vector4: ...
    @property
    def osos(self) -> I8Vector4: ...
    @property
    def osot(self) -> I8Vector4: ...
    @property
    def osoq(self) -> I8Vector4: ...
    @property
    def osop(self) -> I8Vector4: ...
    @property
    def osoo(self) -> I8Vector4: ...
    @property
    def osol(self) -> I8Vector4: ...
    @property
    def osls(self) -> I8Vector4: ...
    @property
    def oslt(self) -> I8Vector4: ...
    @property
    def oslq(self) -> I8Vector4: ...
    @property
    def oslp(self) -> I8Vector4: ...
    @property
    def oslo(self) -> I8Vector4: ...
    @property
    def osll(self) -> I8Vector4: ...
    @property
    def otss(self) -> I8Vector4: ...
    @property
    def otst(self) -> I8Vector4: ...
    @property
    def otsq(self) -> I8Vector4: ...
    @property
    def otsp(self) -> I8Vector4: ...
    @property
    def otso(self) -> I8Vector4: ...
    @property
    def otsl(self) -> I8Vector4: ...
    @property
    def otts(self) -> I8Vector4: ...
    @property
    def ottt(self) -> I8Vector4: ...
    @property
    def ottq(self) -> I8Vector4: ...
    @property
    def ottp(self) -> I8Vector4: ...
    @property
    def otto(self) -> I8Vector4: ...
    @property
    def ottl(self) -> I8Vector4: ...
    @property
    def otqs(self) -> I8Vector4: ...
    @property
    def otqt(self) -> I8Vector4: ...
    @property
    def otqq(self) -> I8Vector4: ...
    @property
    def otqp(self) -> I8Vector4: ...
    @property
    def otqo(self) -> I8Vector4: ...
    @property
    def otql(self) -> I8Vector4: ...
    @property
    def otps(self) -> I8Vector4: ...
    @property
    def otpt(self) -> I8Vector4: ...
    @property
    def otpq(self) -> I8Vector4: ...
    @property
    def otpp(self) -> I8Vector4: ...
    @property
    def otpo(self) -> I8Vector4: ...
    @property
    def otpl(self) -> I8Vector4: ...
    @property
    def otos(self) -> I8Vector4: ...
    @property
    def otot(self) -> I8Vector4: ...
    @property
    def otoq(self) -> I8Vector4: ...
    @property
    def otop(self) -> I8Vector4: ...
    @property
    def otoo(self) -> I8Vector4: ...
    @property
    def otol(self) -> I8Vector4: ...
    @property
    def otls(self) -> I8Vector4: ...
    @property
    def otlt(self) -> I8Vector4: ...
    @property
    def otlq(self) -> I8Vector4: ...
    @property
    def otlp(self) -> I8Vector4: ...
    @property
    def otlo(self) -> I8Vector4: ...
    @property
    def otll(self) -> I8Vector4: ...
    @property
    def oqss(self) -> I8Vector4: ...
    @property
    def oqst(self) -> I8Vector4: ...
    @property
    def oqsq(self) -> I8Vector4: ...
    @property
    def oqsp(self) -> I8Vector4: ...
    @property
    def oqso(self) -> I8Vector4: ...
    @property
    def oqsl(self) -> I8Vector4: ...
    @property
    def oqts(self) -> I8Vector4: ...
    @property
    def oqtt(self) -> I8Vector4: ...
    @property
    def oqtq(self) -> I8Vector4: ...
    @property
    def oqtp(self) -> I8Vector4: ...
    @property
    def oqto(self) -> I8Vector4: ...
    @property
    def oqtl(self) -> I8Vector4: ...
    @property
    def oqqs(self) -> I8Vector4: ...
    @property
    def oqqt(self) -> I8Vector4: ...
    @property
    def oqqq(self) -> I8Vector4: ...
    @property
    def oqqp(self) -> I8Vector4: ...
    @property
    def oqqo(self) -> I8Vector4: ...
    @property
    def oqql(self) -> I8Vector4: ...
    @property
    def oqps(self) -> I8Vector4: ...
    @property
    def oqpt(self) -> I8Vector4: ...
    @property
    def oqpq(self) -> I8Vector4: ...
    @property
    def oqpp(self) -> I8Vector4: ...
    @property
    def oqpo(self) -> I8Vector4: ...
    @property
    def oqpl(self) -> I8Vector4: ...
    @property
    def oqos(self) -> I8Vector4: ...
    @property
    def oqot(self) -> I8Vector4: ...
    @property
    def oqoq(self) -> I8Vector4: ...
    @property
    def oqop(self) -> I8Vector4: ...
    @property
    def oqoo(self) -> I8Vector4: ...
    @property
    def oqol(self) -> I8Vector4: ...
    @property
    def oqls(self) -> I8Vector4: ...
    @property
    def oqlt(self) -> I8Vector4: ...
    @property
    def oqlq(self) -> I8Vector4: ...
    @property
    def oqlp(self) -> I8Vector4: ...
    @property
    def oqlo(self) -> I8Vector4: ...
    @property
    def oqll(self) -> I8Vector4: ...
    @property
    def opss(self) -> I8Vector4: ...
    @property
    def opst(self) -> I8Vector4: ...
    @property
    def opsq(self) -> I8Vector4: ...
    @property
    def opsp(self) -> I8Vector4: ...
    @property
    def opso(self) -> I8Vector4: ...
    @property
    def opsl(self) -> I8Vector4: ...
    @property
    def opts(self) -> I8Vector4: ...
    @property
    def optt(self) -> I8Vector4: ...
    @property
    def optq(self) -> I8Vector4: ...
    @property
    def optp(self) -> I8Vector4: ...
    @property
    def opto(self) -> I8Vector4: ...
    @property
    def optl(self) -> I8Vector4: ...
    @property
    def opqs(self) -> I8Vector4: ...
    @property
    def opqt(self) -> I8Vector4: ...
    @property
    def opqq(self) -> I8Vector4: ...
    @property
    def opqp(self) -> I8Vector4: ...
    @property
    def opqo(self) -> I8Vector4: ...
    @property
    def opql(self) -> I8Vector4: ...
    @property
    def opps(self) -> I8Vector4: ...
    @property
    def oppt(self) -> I8Vector4: ...
    @property
    def oppq(self) -> I8Vector4: ...
    @property
    def oppp(self) -> I8Vector4: ...
    @property
    def oppo(self) -> I8Vector4: ...
    @property
    def oppl(self) -> I8Vector4: ...
    @property
    def opos(self) -> I8Vector4: ...
    @property
    def opot(self) -> I8Vector4: ...
    @property
    def opoq(self) -> I8Vector4: ...
    @property
    def opop(self) -> I8Vector4: ...
    @property
    def opoo(self) -> I8Vector4: ...
    @property
    def opol(self) -> I8Vector4: ...
    @property
    def opls(self) -> I8Vector4: ...
    @property
    def oplt(self) -> I8Vector4: ...
    @property
    def oplq(self) -> I8Vector4: ...
    @property
    def oplp(self) -> I8Vector4: ...
    @property
    def oplo(self) -> I8Vector4: ...
    @property
    def opll(self) -> I8Vector4: ...
    @property
    def ooss(self) -> I8Vector4: ...
    @property
    def oost(self) -> I8Vector4: ...
    @property
    def oosq(self) -> I8Vector4: ...
    @property
    def oosp(self) -> I8Vector4: ...
    @property
    def ooso(self) -> I8Vector4: ...
    @property
    def oosl(self) -> I8Vector4: ...
    @property
    def oots(self) -> I8Vector4: ...
    @property
    def oott(self) -> I8Vector4: ...
    @property
    def ootq(self) -> I8Vector4: ...
    @property
    def ootp(self) -> I8Vector4: ...
    @property
    def ooto(self) -> I8Vector4: ...
    @property
    def ootl(self) -> I8Vector4: ...
    @property
    def ooqs(self) -> I8Vector4: ...
    @property
    def ooqt(self) -> I8Vector4: ...
    @property
    def ooqq(self) -> I8Vector4: ...
    @property
    def ooqp(self) -> I8Vector4: ...
    @property
    def ooqo(self) -> I8Vector4: ...
    @property
    def ooql(self) -> I8Vector4: ...
    @property
    def oops(self) -> I8Vector4: ...
    @property
    def oopt(self) -> I8Vector4: ...
    @property
    def oopq(self) -> I8Vector4: ...
    @property
    def oopp(self) -> I8Vector4: ...
    @property
    def oopo(self) -> I8Vector4: ...
    @property
    def oopl(self) -> I8Vector4: ...
    @property
    def ooos(self) -> I8Vector4: ...
    @property
    def ooot(self) -> I8Vector4: ...
    @property
    def oooq(self) -> I8Vector4: ...
    @property
    def ooop(self) -> I8Vector4: ...
    @property
    def ools(self) -> I8Vector4: ...
    @property
    def oolt(self) -> I8Vector4: ...
    @property
    def oolq(self) -> I8Vector4: ...
    @property
    def oolp(self) -> I8Vector4: ...
    @property
    def olss(self) -> I8Vector4: ...
    @property
    def olst(self) -> I8Vector4: ...
    @property
    def olsq(self) -> I8Vector4: ...
    @property
    def olsp(self) -> I8Vector4: ...
    @property
    def olso(self) -> I8Vector4: ...
    @property
    def olsl(self) -> I8Vector4: ...
    @property
    def olts(self) -> I8Vector4: ...
    @property
    def oltt(self) -> I8Vector4: ...
    @property
    def oltq(self) -> I8Vector4: ...
    @property
    def oltp(self) -> I8Vector4: ...
    @property
    def olto(self) -> I8Vector4: ...
    @property
    def oltl(self) -> I8Vector4: ...
    @property
    def olqs(self) -> I8Vector4: ...
    @property
    def olqt(self) -> I8Vector4: ...
    @property
    def olqq(self) -> I8Vector4: ...
    @property
    def olqp(self) -> I8Vector4: ...
    @property
    def olqo(self) -> I8Vector4: ...
    @property
    def olql(self) -> I8Vector4: ...
    @property
    def olps(self) -> I8Vector4: ...
    @property
    def olpt(self) -> I8Vector4: ...
    @property
    def olpq(self) -> I8Vector4: ...
    @property
    def olpp(self) -> I8Vector4: ...
    @property
    def olpo(self) -> I8Vector4: ...
    @property
    def olpl(self) -> I8Vector4: ...
    @property
    def olos(self) -> I8Vector4: ...
    @property
    def olot(self) -> I8Vector4: ...
    @property
    def oloq(self) -> I8Vector4: ...
    @property
    def olop(self) -> I8Vector4: ...
    @property
    def olls(self) -> I8Vector4: ...
    @property
    def ollt(self) -> I8Vector4: ...
    @property
    def ollq(self) -> I8Vector4: ...
    @property
    def ollp(self) -> I8Vector4: ...
    @property
    def lsss(self) -> I8Vector4: ...
    @property
    def lsst(self) -> I8Vector4: ...
    @property
    def lssq(self) -> I8Vector4: ...
    @property
    def lssp(self) -> I8Vector4: ...
    @property
    def lsso(self) -> I8Vector4: ...
    @property
    def lssl(self) -> I8Vector4: ...
    @property
    def lsts(self) -> I8Vector4: ...
    @property
    def lstt(self) -> I8Vector4: ...
    @property
    def lstq(self) -> I8Vector4: ...
    @property
    def lstp(self) -> I8Vector4: ...
    @property
    def lsto(self) -> I8Vector4: ...
    @property
    def lstl(self) -> I8Vector4: ...
    @property
    def lsqs(self) -> I8Vector4: ...
    @property
    def lsqt(self) -> I8Vector4: ...
    @property
    def lsqq(self) -> I8Vector4: ...
    @property
    def lsqp(self) -> I8Vector4: ...
    @property
    def lsqo(self) -> I8Vector4: ...
    @property
    def lsql(self) -> I8Vector4: ...
    @property
    def lsps(self) -> I8Vector4: ...
    @property
    def lspt(self) -> I8Vector4: ...
    @property
    def lspq(self) -> I8Vector4: ...
    @property
    def lspp(self) -> I8Vector4: ...
    @property
    def lspo(self) -> I8Vector4: ...
    @property
    def lspl(self) -> I8Vector4: ...
    @property
    def lsos(self) -> I8Vector4: ...
    @property
    def lsot(self) -> I8Vector4: ...
    @property
    def lsoq(self) -> I8Vector4: ...
    @property
    def lsop(self) -> I8Vector4: ...
    @property
    def lsoo(self) -> I8Vector4: ...
    @property
    def lsol(self) -> I8Vector4: ...
    @property
    def lsls(self) -> I8Vector4: ...
    @property
    def lslt(self) -> I8Vector4: ...
    @property
    def lslq(self) -> I8Vector4: ...
    @property
    def lslp(self) -> I8Vector4: ...
    @property
    def lslo(self) -> I8Vector4: ...
    @property
    def lsll(self) -> I8Vector4: ...
    @property
    def ltss(self) -> I8Vector4: ...
    @property
    def ltst(self) -> I8Vector4: ...
    @property
    def ltsq(self) -> I8Vector4: ...
    @property
    def ltsp(self) -> I8Vector4: ...
    @property
    def ltso(self) -> I8Vector4: ...
    @property
    def ltsl(self) -> I8Vector4: ...
    @property
    def ltts(self) -> I8Vector4: ...
    @property
    def lttt(self) -> I8Vector4: ...
    @property
    def lttq(self) -> I8Vector4: ...
    @property
    def lttp(self) -> I8Vector4: ...
    @property
    def ltto(self) -> I8Vector4: ...
    @property
    def lttl(self) -> I8Vector4: ...
    @property
    def ltqs(self) -> I8Vector4: ...
    @property
    def ltqt(self) -> I8Vector4: ...
    @property
    def ltqq(self) -> I8Vector4: ...
    @property
    def ltqp(self) -> I8Vector4: ...
    @property
    def ltqo(self) -> I8Vector4: ...
    @property
    def ltql(self) -> I8Vector4: ...
    @property
    def ltps(self) -> I8Vector4: ...
    @property
    def ltpt(self) -> I8Vector4: ...
    @property
    def ltpq(self) -> I8Vector4: ...
    @property
    def ltpp(self) -> I8Vector4: ...
    @property
    def ltpo(self) -> I8Vector4: ...
    @property
    def ltpl(self) -> I8Vector4: ...
    @property
    def ltos(self) -> I8Vector4: ...
    @property
    def ltot(self) -> I8Vector4: ...
    @property
    def ltoq(self) -> I8Vector4: ...
    @property
    def ltop(self) -> I8Vector4: ...
    @property
    def ltoo(self) -> I8Vector4: ...
    @property
    def ltol(self) -> I8Vector4: ...
    @property
    def ltls(self) -> I8Vector4: ...
    @property
    def ltlt(self) -> I8Vector4: ...
    @property
    def ltlq(self) -> I8Vector4: ...
    @property
    def ltlp(self) -> I8Vector4: ...
    @property
    def ltlo(self) -> I8Vector4: ...
    @property
    def ltll(self) -> I8Vector4: ...
    @property
    def lqss(self) -> I8Vector4: ...
    @property
    def lqst(self) -> I8Vector4: ...
    @property
    def lqsq(self) -> I8Vector4: ...
    @property
    def lqsp(self) -> I8Vector4: ...
    @property
    def lqso(self) -> I8Vector4: ...
    @property
    def lqsl(self) -> I8Vector4: ...
    @property
    def lqts(self) -> I8Vector4: ...
    @property
    def lqtt(self) -> I8Vector4: ...
    @property
    def lqtq(self) -> I8Vector4: ...
    @property
    def lqtp(self) -> I8Vector4: ...
    @property
    def lqto(self) -> I8Vector4: ...
    @property
    def lqtl(self) -> I8Vector4: ...
    @property
    def lqqs(self) -> I8Vector4: ...
    @property
    def lqqt(self) -> I8Vector4: ...
    @property
    def lqqq(self) -> I8Vector4: ...
    @property
    def lqqp(self) -> I8Vector4: ...
    @property
    def lqqo(self) -> I8Vector4: ...
    @property
    def lqql(self) -> I8Vector4: ...
    @property
    def lqps(self) -> I8Vector4: ...
    @property
    def lqpt(self) -> I8Vector4: ...
    @property
    def lqpq(self) -> I8Vector4: ...
    @property
    def lqpp(self) -> I8Vector4: ...
    @property
    def lqpo(self) -> I8Vector4: ...
    @property
    def lqpl(self) -> I8Vector4: ...
    @property
    def lqos(self) -> I8Vector4: ...
    @property
    def lqot(self) -> I8Vector4: ...
    @property
    def lqoq(self) -> I8Vector4: ...
    @property
    def lqop(self) -> I8Vector4: ...
    @property
    def lqoo(self) -> I8Vector4: ...
    @property
    def lqol(self) -> I8Vector4: ...
    @property
    def lqls(self) -> I8Vector4: ...
    @property
    def lqlt(self) -> I8Vector4: ...
    @property
    def lqlq(self) -> I8Vector4: ...
    @property
    def lqlp(self) -> I8Vector4: ...
    @property
    def lqlo(self) -> I8Vector4: ...
    @property
    def lqll(self) -> I8Vector4: ...
    @property
    def lpss(self) -> I8Vector4: ...
    @property
    def lpst(self) -> I8Vector4: ...
    @property
    def lpsq(self) -> I8Vector4: ...
    @property
    def lpsp(self) -> I8Vector4: ...
    @property
    def lpso(self) -> I8Vector4: ...
    @property
    def lpsl(self) -> I8Vector4: ...
    @property
    def lpts(self) -> I8Vector4: ...
    @property
    def lptt(self) -> I8Vector4: ...
    @property
    def lptq(self) -> I8Vector4: ...
    @property
    def lptp(self) -> I8Vector4: ...
    @property
    def lpto(self) -> I8Vector4: ...
    @property
    def lptl(self) -> I8Vector4: ...
    @property
    def lpqs(self) -> I8Vector4: ...
    @property
    def lpqt(self) -> I8Vector4: ...
    @property
    def lpqq(self) -> I8Vector4: ...
    @property
    def lpqp(self) -> I8Vector4: ...
    @property
    def lpqo(self) -> I8Vector4: ...
    @property
    def lpql(self) -> I8Vector4: ...
    @property
    def lpps(self) -> I8Vector4: ...
    @property
    def lppt(self) -> I8Vector4: ...
    @property
    def lppq(self) -> I8Vector4: ...
    @property
    def lppp(self) -> I8Vector4: ...
    @property
    def lppo(self) -> I8Vector4: ...
    @property
    def lppl(self) -> I8Vector4: ...
    @property
    def lpos(self) -> I8Vector4: ...
    @property
    def lpot(self) -> I8Vector4: ...
    @property
    def lpoq(self) -> I8Vector4: ...
    @property
    def lpop(self) -> I8Vector4: ...
    @property
    def lpoo(self) -> I8Vector4: ...
    @property
    def lpol(self) -> I8Vector4: ...
    @property
    def lpls(self) -> I8Vector4: ...
    @property
    def lplt(self) -> I8Vector4: ...
    @property
    def lplq(self) -> I8Vector4: ...
    @property
    def lplp(self) -> I8Vector4: ...
    @property
    def lplo(self) -> I8Vector4: ...
    @property
    def lpll(self) -> I8Vector4: ...
    @property
    def loss(self) -> I8Vector4: ...
    @property
    def lost(self) -> I8Vector4: ...
    @property
    def losq(self) -> I8Vector4: ...
    @property
    def losp(self) -> I8Vector4: ...
    @property
    def loso(self) -> I8Vector4: ...
    @property
    def losl(self) -> I8Vector4: ...
    @property
    def lots(self) -> I8Vector4: ...
    @property
    def lott(self) -> I8Vector4: ...
    @property
    def lotq(self) -> I8Vector4: ...
    @property
    def lotp(self) -> I8Vector4: ...
    @property
    def loto(self) -> I8Vector4: ...
    @property
    def lotl(self) -> I8Vector4: ...
    @property
    def loqs(self) -> I8Vector4: ...
    @property
    def loqt(self) -> I8Vector4: ...
    @property
    def loqq(self) -> I8Vector4: ...
    @property
    def loqp(self) -> I8Vector4: ...
    @property
    def loqo(self) -> I8Vector4: ...
    @property
    def loql(self) -> I8Vector4: ...
    @property
    def lops(self) -> I8Vector4: ...
    @property
    def lopt(self) -> I8Vector4: ...
    @property
    def lopq(self) -> I8Vector4: ...
    @property
    def lopp(self) -> I8Vector4: ...
    @property
    def lopo(self) -> I8Vector4: ...
    @property
    def lopl(self) -> I8Vector4: ...
    @property
    def loos(self) -> I8Vector4: ...
    @property
    def loot(self) -> I8Vector4: ...
    @property
    def looq(self) -> I8Vector4: ...
    @property
    def loop(self) -> I8Vector4: ...
    @property
    def lols(self) -> I8Vector4: ...
    @property
    def lolt(self) -> I8Vector4: ...
    @property
    def lolq(self) -> I8Vector4: ...
    @property
    def lolp(self) -> I8Vector4: ...
    @property
    def llss(self) -> I8Vector4: ...
    @property
    def llst(self) -> I8Vector4: ...
    @property
    def llsq(self) -> I8Vector4: ...
    @property
    def llsp(self) -> I8Vector4: ...
    @property
    def llso(self) -> I8Vector4: ...
    @property
    def llsl(self) -> I8Vector4: ...
    @property
    def llts(self) -> I8Vector4: ...
    @property
    def lltt(self) -> I8Vector4: ...
    @property
    def lltq(self) -> I8Vector4: ...
    @property
    def lltp(self) -> I8Vector4: ...
    @property
    def llto(self) -> I8Vector4: ...
    @property
    def lltl(self) -> I8Vector4: ...
    @property
    def llqs(self) -> I8Vector4: ...
    @property
    def llqt(self) -> I8Vector4: ...
    @property
    def llqq(self) -> I8Vector4: ...
    @property
    def llqp(self) -> I8Vector4: ...
    @property
    def llqo(self) -> I8Vector4: ...
    @property
    def llql(self) -> I8Vector4: ...
    @property
    def llps(self) -> I8Vector4: ...
    @property
    def llpt(self) -> I8Vector4: ...
    @property
    def llpq(self) -> I8Vector4: ...
    @property
    def llpp(self) -> I8Vector4: ...
    @property
    def llpo(self) -> I8Vector4: ...
    @property
    def llpl(self) -> I8Vector4: ...
    @property
    def llos(self) -> I8Vector4: ...
    @property
    def llot(self) -> I8Vector4: ...
    @property
    def lloq(self) -> I8Vector4: ...
    @property
    def llop(self) -> I8Vector4: ...
    @property
    def llls(self) -> I8Vector4: ...
    @property
    def lllt(self) -> I8Vector4: ...
    @property
    def lllq(self) -> I8Vector4: ...
    @property
    def lllp(self) -> I8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I8Vector2: ...
    @property
    def uv(self) -> I8Vector2: ...
    @property
    def uo(self) -> I8Vector2: ...
    @property
    def ul(self) -> I8Vector2: ...
    @property
    def vu(self) -> I8Vector2: ...
    @property
    def vv(self) -> I8Vector2: ...
    @property
    def vo(self) -> I8Vector2: ...
    @property
    def vl(self) -> I8Vector2: ...
    @property
    def ou(self) -> I8Vector2: ...
    @property
    def ov(self) -> I8Vector2: ...
    @property
    def lu(self) -> I8Vector2: ...
    @property
    def lv(self) -> I8Vector2: ...
    @property
    def uuu(self) -> I8Vector3: ...
    @property
    def uuv(self) -> I8Vector3: ...
    @property
    def uuo(self) -> I8Vector3: ...
    @property
    def uul(self) -> I8Vector3: ...
    @property
    def uvu(self) -> I8Vector3: ...
    @property
    def uvv(self) -> I8Vector3: ...
    @property
    def uvo(self) -> I8Vector3: ...
    @property
    def uvl(self) -> I8Vector3: ...
    @property
    def uou(self) -> I8Vector3: ...
    @property
    def uov(self) -> I8Vector3: ...
    @property
    def uoo(self) -> I8Vector3: ...
    @property
    def uol(self) -> I8Vector3: ...
    @property
    def ulu(self) -> I8Vector3: ...
    @property
    def ulv(self) -> I8Vector3: ...
    @property
    def ulo(self) -> I8Vector3: ...
    @property
    def ull(self) -> I8Vector3: ...
    @property
    def vuu(self) -> I8Vector3: ...
    @property
    def vuv(self) -> I8Vector3: ...
    @property
    def vuo(self) -> I8Vector3: ...
    @property
    def vul(self) -> I8Vector3: ...
    @property
    def vvu(self) -> I8Vector3: ...
    @property
    def vvv(self) -> I8Vector3: ...
    @property
    def vvo(self) -> I8Vector3: ...
    @property
    def vvl(self) -> I8Vector3: ...
    @property
    def vou(self) -> I8Vector3: ...
    @property
    def vov(self) -> I8Vector3: ...
    @property
    def voo(self) -> I8Vector3: ...
    @property
    def vol(self) -> I8Vector3: ...
    @property
    def vlu(self) -> I8Vector3: ...
    @property
    def vlv(self) -> I8Vector3: ...
    @property
    def vlo(self) -> I8Vector3: ...
    @property
    def vll(self) -> I8Vector3: ...
    @property
    def ouu(self) -> I8Vector3: ...
    @property
    def ouv(self) -> I8Vector3: ...
    @property
    def ouo(self) -> I8Vector3: ...
    @property
    def oul(self) -> I8Vector3: ...
    @property
    def ovu(self) -> I8Vector3: ...
    @property
    def ovv(self) -> I8Vector3: ...
    @property
    def ovo(self) -> I8Vector3: ...
    @property
    def ovl(self) -> I8Vector3: ...
    @property
    def oou(self) -> I8Vector3: ...
    @property
    def oov(self) -> I8Vector3: ...
    @property
    def olu(self) -> I8Vector3: ...
    @property
    def olv(self) -> I8Vector3: ...
    @property
    def luu(self) -> I8Vector3: ...
    @property
    def luv(self) -> I8Vector3: ...
    @property
    def luo(self) -> I8Vector3: ...
    @property
    def lul(self) -> I8Vector3: ...
    @property
    def lvu(self) -> I8Vector3: ...
    @property
    def lvv(self) -> I8Vector3: ...
    @property
    def lvo(self) -> I8Vector3: ...
    @property
    def lvl(self) -> I8Vector3: ...
    @property
    def lou(self) -> I8Vector3: ...
    @property
    def lov(self) -> I8Vector3: ...
    @property
    def llu(self) -> I8Vector3: ...
    @property
    def llv(self) -> I8Vector3: ...
    @property
    def uuuu(self) -> I8Vector4: ...
    @property
    def uuuv(self) -> I8Vector4: ...
    @property
    def uuuo(self) -> I8Vector4: ...
    @property
    def uuul(self) -> I8Vector4: ...
    @property
    def uuvu(self) -> I8Vector4: ...
    @property
    def uuvv(self) -> I8Vector4: ...
    @property
    def uuvo(self) -> I8Vector4: ...
    @property
    def uuvl(self) -> I8Vector4: ...
    @property
    def uuou(self) -> I8Vector4: ...
    @property
    def uuov(self) -> I8Vector4: ...
    @property
    def uuoo(self) -> I8Vector4: ...
    @property
    def uuol(self) -> I8Vector4: ...
    @property
    def uulu(self) -> I8Vector4: ...
    @property
    def uulv(self) -> I8Vector4: ...
    @property
    def uulo(self) -> I8Vector4: ...
    @property
    def uull(self) -> I8Vector4: ...
    @property
    def uvuu(self) -> I8Vector4: ...
    @property
    def uvuv(self) -> I8Vector4: ...
    @property
    def uvuo(self) -> I8Vector4: ...
    @property
    def uvul(self) -> I8Vector4: ...
    @property
    def uvvu(self) -> I8Vector4: ...
    @property
    def uvvv(self) -> I8Vector4: ...
    @property
    def uvvo(self) -> I8Vector4: ...
    @property
    def uvvl(self) -> I8Vector4: ...
    @property
    def uvou(self) -> I8Vector4: ...
    @property
    def uvov(self) -> I8Vector4: ...
    @property
    def uvoo(self) -> I8Vector4: ...
    @property
    def uvol(self) -> I8Vector4: ...
    @property
    def uvlu(self) -> I8Vector4: ...
    @property
    def uvlv(self) -> I8Vector4: ...
    @property
    def uvlo(self) -> I8Vector4: ...
    @property
    def uvll(self) -> I8Vector4: ...
    @property
    def uouu(self) -> I8Vector4: ...
    @property
    def uouv(self) -> I8Vector4: ...
    @property
    def uouo(self) -> I8Vector4: ...
    @property
    def uoul(self) -> I8Vector4: ...
    @property
    def uovu(self) -> I8Vector4: ...
    @property
    def uovv(self) -> I8Vector4: ...
    @property
    def uovo(self) -> I8Vector4: ...
    @property
    def uovl(self) -> I8Vector4: ...
    @property
    def uoou(self) -> I8Vector4: ...
    @property
    def uoov(self) -> I8Vector4: ...
    @property
    def uooo(self) -> I8Vector4: ...
    @property
    def uool(self) -> I8Vector4: ...
    @property
    def uolu(self) -> I8Vector4: ...
    @property
    def uolv(self) -> I8Vector4: ...
    @property
    def uolo(self) -> I8Vector4: ...
    @property
    def uoll(self) -> I8Vector4: ...
    @property
    def uluu(self) -> I8Vector4: ...
    @property
    def uluv(self) -> I8Vector4: ...
    @property
    def uluo(self) -> I8Vector4: ...
    @property
    def ulul(self) -> I8Vector4: ...
    @property
    def ulvu(self) -> I8Vector4: ...
    @property
    def ulvv(self) -> I8Vector4: ...
    @property
    def ulvo(self) -> I8Vector4: ...
    @property
    def ulvl(self) -> I8Vector4: ...
    @property
    def ulou(self) -> I8Vector4: ...
    @property
    def ulov(self) -> I8Vector4: ...
    @property
    def uloo(self) -> I8Vector4: ...
    @property
    def ulol(self) -> I8Vector4: ...
    @property
    def ullu(self) -> I8Vector4: ...
    @property
    def ullv(self) -> I8Vector4: ...
    @property
    def ullo(self) -> I8Vector4: ...
    @property
    def ulll(self) -> I8Vector4: ...
    @property
    def vuuu(self) -> I8Vector4: ...
    @property
    def vuuv(self) -> I8Vector4: ...
    @property
    def vuuo(self) -> I8Vector4: ...
    @property
    def vuul(self) -> I8Vector4: ...
    @property
    def vuvu(self) -> I8Vector4: ...
    @property
    def vuvv(self) -> I8Vector4: ...
    @property
    def vuvo(self) -> I8Vector4: ...
    @property
    def vuvl(self) -> I8Vector4: ...
    @property
    def vuou(self) -> I8Vector4: ...
    @property
    def vuov(self) -> I8Vector4: ...
    @property
    def vuoo(self) -> I8Vector4: ...
    @property
    def vuol(self) -> I8Vector4: ...
    @property
    def vulu(self) -> I8Vector4: ...
    @property
    def vulv(self) -> I8Vector4: ...
    @property
    def vulo(self) -> I8Vector4: ...
    @property
    def vull(self) -> I8Vector4: ...
    @property
    def vvuu(self) -> I8Vector4: ...
    @property
    def vvuv(self) -> I8Vector4: ...
    @property
    def vvuo(self) -> I8Vector4: ...
    @property
    def vvul(self) -> I8Vector4: ...
    @property
    def vvvu(self) -> I8Vector4: ...
    @property
    def vvvv(self) -> I8Vector4: ...
    @property
    def vvvo(self) -> I8Vector4: ...
    @property
    def vvvl(self) -> I8Vector4: ...
    @property
    def vvou(self) -> I8Vector4: ...
    @property
    def vvov(self) -> I8Vector4: ...
    @property
    def vvoo(self) -> I8Vector4: ...
    @property
    def vvol(self) -> I8Vector4: ...
    @property
    def vvlu(self) -> I8Vector4: ...
    @property
    def vvlv(self) -> I8Vector4: ...
    @property
    def vvlo(self) -> I8Vector4: ...
    @property
    def vvll(self) -> I8Vector4: ...
    @property
    def vouu(self) -> I8Vector4: ...
    @property
    def vouv(self) -> I8Vector4: ...
    @property
    def vouo(self) -> I8Vector4: ...
    @property
    def voul(self) -> I8Vector4: ...
    @property
    def vovu(self) -> I8Vector4: ...
    @property
    def vovv(self) -> I8Vector4: ...
    @property
    def vovo(self) -> I8Vector4: ...
    @property
    def vovl(self) -> I8Vector4: ...
    @property
    def voou(self) -> I8Vector4: ...
    @property
    def voov(self) -> I8Vector4: ...
    @property
    def vooo(self) -> I8Vector4: ...
    @property
    def vool(self) -> I8Vector4: ...
    @property
    def volu(self) -> I8Vector4: ...
    @property
    def volv(self) -> I8Vector4: ...
    @property
    def volo(self) -> I8Vector4: ...
    @property
    def voll(self) -> I8Vector4: ...
    @property
    def vluu(self) -> I8Vector4: ...
    @property
    def vluv(self) -> I8Vector4: ...
    @property
    def vluo(self) -> I8Vector4: ...
    @property
    def vlul(self) -> I8Vector4: ...
    @property
    def vlvu(self) -> I8Vector4: ...
    @property
    def vlvv(self) -> I8Vector4: ...
    @property
    def vlvo(self) -> I8Vector4: ...
    @property
    def vlvl(self) -> I8Vector4: ...
    @property
    def vlou(self) -> I8Vector4: ...
    @property
    def vlov(self) -> I8Vector4: ...
    @property
    def vloo(self) -> I8Vector4: ...
    @property
    def vlol(self) -> I8Vector4: ...
    @property
    def vllu(self) -> I8Vector4: ...
    @property
    def vllv(self) -> I8Vector4: ...
    @property
    def vllo(self) -> I8Vector4: ...
    @property
    def vlll(self) -> I8Vector4: ...
    @property
    def ouuu(self) -> I8Vector4: ...
    @property
    def ouuv(self) -> I8Vector4: ...
    @property
    def ouuo(self) -> I8Vector4: ...
    @property
    def ouul(self) -> I8Vector4: ...
    @property
    def ouvu(self) -> I8Vector4: ...
    @property
    def ouvv(self) -> I8Vector4: ...
    @property
    def ouvo(self) -> I8Vector4: ...
    @property
    def ouvl(self) -> I8Vector4: ...
    @property
    def ouou(self) -> I8Vector4: ...
    @property
    def ouov(self) -> I8Vector4: ...
    @property
    def ouoo(self) -> I8Vector4: ...
    @property
    def ouol(self) -> I8Vector4: ...
    @property
    def oulu(self) -> I8Vector4: ...
    @property
    def oulv(self) -> I8Vector4: ...
    @property
    def oulo(self) -> I8Vector4: ...
    @property
    def oull(self) -> I8Vector4: ...
    @property
    def ovuu(self) -> I8Vector4: ...
    @property
    def ovuv(self) -> I8Vector4: ...
    @property
    def ovuo(self) -> I8Vector4: ...
    @property
    def ovul(self) -> I8Vector4: ...
    @property
    def ovvu(self) -> I8Vector4: ...
    @property
    def ovvv(self) -> I8Vector4: ...
    @property
    def ovvo(self) -> I8Vector4: ...
    @property
    def ovvl(self) -> I8Vector4: ...
    @property
    def ovou(self) -> I8Vector4: ...
    @property
    def ovov(self) -> I8Vector4: ...
    @property
    def ovoo(self) -> I8Vector4: ...
    @property
    def ovol(self) -> I8Vector4: ...
    @property
    def ovlu(self) -> I8Vector4: ...
    @property
    def ovlv(self) -> I8Vector4: ...
    @property
    def ovlo(self) -> I8Vector4: ...
    @property
    def ovll(self) -> I8Vector4: ...
    @property
    def oouu(self) -> I8Vector4: ...
    @property
    def oouv(self) -> I8Vector4: ...
    @property
    def oouo(self) -> I8Vector4: ...
    @property
    def ooul(self) -> I8Vector4: ...
    @property
    def oovu(self) -> I8Vector4: ...
    @property
    def oovv(self) -> I8Vector4: ...
    @property
    def oovo(self) -> I8Vector4: ...
    @property
    def oovl(self) -> I8Vector4: ...
    @property
    def ooou(self) -> I8Vector4: ...
    @property
    def ooov(self) -> I8Vector4: ...
    @property
    def oolu(self) -> I8Vector4: ...
    @property
    def oolv(self) -> I8Vector4: ...
    @property
    def oluu(self) -> I8Vector4: ...
    @property
    def oluv(self) -> I8Vector4: ...
    @property
    def oluo(self) -> I8Vector4: ...
    @property
    def olul(self) -> I8Vector4: ...
    @property
    def olvu(self) -> I8Vector4: ...
    @property
    def olvv(self) -> I8Vector4: ...
    @property
    def olvo(self) -> I8Vector4: ...
    @property
    def olvl(self) -> I8Vector4: ...
    @property
    def olou(self) -> I8Vector4: ...
    @property
    def olov(self) -> I8Vector4: ...
    @property
    def ollu(self) -> I8Vector4: ...
    @property
    def ollv(self) -> I8Vector4: ...
    @property
    def luuu(self) -> I8Vector4: ...
    @property
    def luuv(self) -> I8Vector4: ...
    @property
    def luuo(self) -> I8Vector4: ...
    @property
    def luul(self) -> I8Vector4: ...
    @property
    def luvu(self) -> I8Vector4: ...
    @property
    def luvv(self) -> I8Vector4: ...
    @property
    def luvo(self) -> I8Vector4: ...
    @property
    def luvl(self) -> I8Vector4: ...
    @property
    def luou(self) -> I8Vector4: ...
    @property
    def luov(self) -> I8Vector4: ...
    @property
    def luoo(self) -> I8Vector4: ...
    @property
    def luol(self) -> I8Vector4: ...
    @property
    def lulu(self) -> I8Vector4: ...
    @property
    def lulv(self) -> I8Vector4: ...
    @property
    def lulo(self) -> I8Vector4: ...
    @property
    def lull(self) -> I8Vector4: ...
    @property
    def lvuu(self) -> I8Vector4: ...
    @property
    def lvuv(self) -> I8Vector4: ...
    @property
    def lvuo(self) -> I8Vector4: ...
    @property
    def lvul(self) -> I8Vector4: ...
    @property
    def lvvu(self) -> I8Vector4: ...
    @property
    def lvvv(self) -> I8Vector4: ...
    @property
    def lvvo(self) -> I8Vector4: ...
    @property
    def lvvl(self) -> I8Vector4: ...
    @property
    def lvou(self) -> I8Vector4: ...
    @property
    def lvov(self) -> I8Vector4: ...
    @property
    def lvoo(self) -> I8Vector4: ...
    @property
    def lvol(self) -> I8Vector4: ...
    @property
    def lvlu(self) -> I8Vector4: ...
    @property
    def lvlv(self) -> I8Vector4: ...
    @property
    def lvlo(self) -> I8Vector4: ...
    @property
    def lvll(self) -> I8Vector4: ...
    @property
    def louu(self) -> I8Vector4: ...
    @property
    def louv(self) -> I8Vector4: ...
    @property
    def louo(self) -> I8Vector4: ...
    @property
    def loul(self) -> I8Vector4: ...
    @property
    def lovu(self) -> I8Vector4: ...
    @property
    def lovv(self) -> I8Vector4: ...
    @property
    def lovo(self) -> I8Vector4: ...
    @property
    def lovl(self) -> I8Vector4: ...
    @property
    def loou(self) -> I8Vector4: ...
    @property
    def loov(self) -> I8Vector4: ...
    @property
    def lolu(self) -> I8Vector4: ...
    @property
    def lolv(self) -> I8Vector4: ...
    @property
    def lluu(self) -> I8Vector4: ...
    @property
    def lluv(self) -> I8Vector4: ...
    @property
    def lluo(self) -> I8Vector4: ...
    @property
    def llul(self) -> I8Vector4: ...
    @property
    def llvu(self) -> I8Vector4: ...
    @property
    def llvv(self) -> I8Vector4: ...
    @property
    def llvo(self) -> I8Vector4: ...
    @property
    def llvl(self) -> I8Vector4: ...
    @property
    def llou(self) -> I8Vector4: ...
    @property
    def llov(self) -> I8Vector4: ...
    @property
    def lllu(self) -> I8Vector4: ...
    @property
    def lllv(self) -> I8Vector4: ...
    @property
    def oo(self) -> I8Vector2: ...
    @property
    def ol(self) -> I8Vector2: ...
    @property
    def lo(self) -> I8Vector2: ...
    @property
    def ll(self) -> I8Vector2: ...
    @property
    def ooo(self) -> I8Vector3: ...
    @property
    def ool(self) -> I8Vector3: ...
    @property
    def olo(self) -> I8Vector3: ...
    @property
    def oll(self) -> I8Vector3: ...
    @property
    def loo(self) -> I8Vector3: ...
    @property
    def lol(self) -> I8Vector3: ...
    @property
    def llo(self) -> I8Vector3: ...
    @property
    def lll(self) -> I8Vector3: ...
    @property
    def oooo(self) -> I8Vector4: ...
    @property
    def oool(self) -> I8Vector4: ...
    @property
    def oolo(self) -> I8Vector4: ...
    @property
    def ooll(self) -> I8Vector4: ...
    @property
    def oloo(self) -> I8Vector4: ...
    @property
    def olol(self) -> I8Vector4: ...
    @property
    def ollo(self) -> I8Vector4: ...
    @property
    def olll(self) -> I8Vector4: ...
    @property
    def looo(self) -> I8Vector4: ...
    @property
    def lool(self) -> I8Vector4: ...
    @property
    def lolo(self) -> I8Vector4: ...
    @property
    def loll(self) -> I8Vector4: ...
    @property
    def lloo(self) -> I8Vector4: ...
    @property
    def llol(self) -> I8Vector4: ...
    @property
    def lllo(self) -> I8Vector4: ...
    @property
    def llll(self) -> I8Vector4: ...
    def min(self, n: Number, /) -> I8Vector4: ...
    def max(self, n: Number, /) -> I8Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> I8Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I8Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector4: ...

@final
class I8Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I8Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I8Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> I8Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> I8Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I8Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[I8Vector4]: ...

@final
class U8Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __add__(self, other: Number) -> U8Vector4: ...
    @overload
    def __radd__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __radd__(self, other: Number) -> U8Vector4: ...
    @overload
    def __sub__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __sub__(self, other: Number) -> U8Vector4: ...
    @overload
    def __rsub__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> U8Vector4: ...
    @overload
    def __mul__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __mul__(self, other: Number) -> U8Vector4: ...
    @overload
    def __rmul__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> U8Vector4: ...
    @overload
    def __truediv__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> U8Vector4: ...
    @overload
    def __rtruediv__(self, other: U8Vector4) -> U8Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> U8Vector4: ...
    def __abs__(self) -> U8Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> U8Vector2: ...
    @property
    def xy(self) -> U8Vector2: ...
    @property
    def xz(self) -> U8Vector2: ...
    @property
    def xw(self) -> U8Vector2: ...
    @property
    def xo(self) -> U8Vector2: ...
    @property
    def xl(self) -> U8Vector2: ...
    @property
    def yx(self) -> U8Vector2: ...
    @property
    def yy(self) -> U8Vector2: ...
    @property
    def yz(self) -> U8Vector2: ...
    @property
    def yw(self) -> U8Vector2: ...
    @property
    def yo(self) -> U8Vector2: ...
    @property
    def yl(self) -> U8Vector2: ...
    @property
    def zx(self) -> U8Vector2: ...
    @property
    def zy(self) -> U8Vector2: ...
    @property
    def zz(self) -> U8Vector2: ...
    @property
    def zw(self) -> U8Vector2: ...
    @property
    def zo(self) -> U8Vector2: ...
    @property
    def zl(self) -> U8Vector2: ...
    @property
    def wx(self) -> U8Vector2: ...
    @property
    def wy(self) -> U8Vector2: ...
    @property
    def wz(self) -> U8Vector2: ...
    @property
    def ww(self) -> U8Vector2: ...
    @property
    def wo(self) -> U8Vector2: ...
    @property
    def wl(self) -> U8Vector2: ...
    @property
    def ox(self) -> U8Vector2: ...
    @property
    def oy(self) -> U8Vector2: ...
    @property
    def oz(self) -> U8Vector2: ...
    @property
    def ow(self) -> U8Vector2: ...
    @property
    def lx(self) -> U8Vector2: ...
    @property
    def ly(self) -> U8Vector2: ...
    @property
    def lz(self) -> U8Vector2: ...
    @property
    def lw(self) -> U8Vector2: ...
    @property
    def xxx(self) -> U8Vector3: ...
    @property
    def xxy(self) -> U8Vector3: ...
    @property
    def xxz(self) -> U8Vector3: ...
    @property
    def xxw(self) -> U8Vector3: ...
    @property
    def xxo(self) -> U8Vector3: ...
    @property
    def xxl(self) -> U8Vector3: ...
    @property
    def xyx(self) -> U8Vector3: ...
    @property
    def xyy(self) -> U8Vector3: ...
    @property
    def xyz(self) -> U8Vector3: ...
    @property
    def xyw(self) -> U8Vector3: ...
    @property
    def xyo(self) -> U8Vector3: ...
    @property
    def xyl(self) -> U8Vector3: ...
    @property
    def xzx(self) -> U8Vector3: ...
    @property
    def xzy(self) -> U8Vector3: ...
    @property
    def xzz(self) -> U8Vector3: ...
    @property
    def xzw(self) -> U8Vector3: ...
    @property
    def xzo(self) -> U8Vector3: ...
    @property
    def xzl(self) -> U8Vector3: ...
    @property
    def xwx(self) -> U8Vector3: ...
    @property
    def xwy(self) -> U8Vector3: ...
    @property
    def xwz(self) -> U8Vector3: ...
    @property
    def xww(self) -> U8Vector3: ...
    @property
    def xwo(self) -> U8Vector3: ...
    @property
    def xwl(self) -> U8Vector3: ...
    @property
    def xox(self) -> U8Vector3: ...
    @property
    def xoy(self) -> U8Vector3: ...
    @property
    def xoz(self) -> U8Vector3: ...
    @property
    def xow(self) -> U8Vector3: ...
    @property
    def xoo(self) -> U8Vector3: ...
    @property
    def xol(self) -> U8Vector3: ...
    @property
    def xlx(self) -> U8Vector3: ...
    @property
    def xly(self) -> U8Vector3: ...
    @property
    def xlz(self) -> U8Vector3: ...
    @property
    def xlw(self) -> U8Vector3: ...
    @property
    def xlo(self) -> U8Vector3: ...
    @property
    def xll(self) -> U8Vector3: ...
    @property
    def yxx(self) -> U8Vector3: ...
    @property
    def yxy(self) -> U8Vector3: ...
    @property
    def yxz(self) -> U8Vector3: ...
    @property
    def yxw(self) -> U8Vector3: ...
    @property
    def yxo(self) -> U8Vector3: ...
    @property
    def yxl(self) -> U8Vector3: ...
    @property
    def yyx(self) -> U8Vector3: ...
    @property
    def yyy(self) -> U8Vector3: ...
    @property
    def yyz(self) -> U8Vector3: ...
    @property
    def yyw(self) -> U8Vector3: ...
    @property
    def yyo(self) -> U8Vector3: ...
    @property
    def yyl(self) -> U8Vector3: ...
    @property
    def yzx(self) -> U8Vector3: ...
    @property
    def yzy(self) -> U8Vector3: ...
    @property
    def yzz(self) -> U8Vector3: ...
    @property
    def yzw(self) -> U8Vector3: ...
    @property
    def yzo(self) -> U8Vector3: ...
    @property
    def yzl(self) -> U8Vector3: ...
    @property
    def ywx(self) -> U8Vector3: ...
    @property
    def ywy(self) -> U8Vector3: ...
    @property
    def ywz(self) -> U8Vector3: ...
    @property
    def yww(self) -> U8Vector3: ...
    @property
    def ywo(self) -> U8Vector3: ...
    @property
    def ywl(self) -> U8Vector3: ...
    @property
    def yox(self) -> U8Vector3: ...
    @property
    def yoy(self) -> U8Vector3: ...
    @property
    def yoz(self) -> U8Vector3: ...
    @property
    def yow(self) -> U8Vector3: ...
    @property
    def yoo(self) -> U8Vector3: ...
    @property
    def yol(self) -> U8Vector3: ...
    @property
    def ylx(self) -> U8Vector3: ...
    @property
    def yly(self) -> U8Vector3: ...
    @property
    def ylz(self) -> U8Vector3: ...
    @property
    def ylw(self) -> U8Vector3: ...
    @property
    def ylo(self) -> U8Vector3: ...
    @property
    def yll(self) -> U8Vector3: ...
    @property
    def zxx(self) -> U8Vector3: ...
    @property
    def zxy(self) -> U8Vector3: ...
    @property
    def zxz(self) -> U8Vector3: ...
    @property
    def zxw(self) -> U8Vector3: ...
    @property
    def zxo(self) -> U8Vector3: ...
    @property
    def zxl(self) -> U8Vector3: ...
    @property
    def zyx(self) -> U8Vector3: ...
    @property
    def zyy(self) -> U8Vector3: ...
    @property
    def zyz(self) -> U8Vector3: ...
    @property
    def zyw(self) -> U8Vector3: ...
    @property
    def zyo(self) -> U8Vector3: ...
    @property
    def zyl(self) -> U8Vector3: ...
    @property
    def zzx(self) -> U8Vector3: ...
    @property
    def zzy(self) -> U8Vector3: ...
    @property
    def zzz(self) -> U8Vector3: ...
    @property
    def zzw(self) -> U8Vector3: ...
    @property
    def zzo(self) -> U8Vector3: ...
    @property
    def zzl(self) -> U8Vector3: ...
    @property
    def zwx(self) -> U8Vector3: ...
    @property
    def zwy(self) -> U8Vector3: ...
    @property
    def zwz(self) -> U8Vector3: ...
    @property
    def zww(self) -> U8Vector3: ...
    @property
    def zwo(self) -> U8Vector3: ...
    @property
    def zwl(self) -> U8Vector3: ...
    @property
    def zox(self) -> U8Vector3: ...
    @property
    def zoy(self) -> U8Vector3: ...
    @property
    def zoz(self) -> U8Vector3: ...
    @property
    def zow(self) -> U8Vector3: ...
    @property
    def zoo(self) -> U8Vector3: ...
    @property
    def zol(self) -> U8Vector3: ...
    @property
    def zlx(self) -> U8Vector3: ...
    @property
    def zly(self) -> U8Vector3: ...
    @property
    def zlz(self) -> U8Vector3: ...
    @property
    def zlw(self) -> U8Vector3: ...
    @property
    def zlo(self) -> U8Vector3: ...
    @property
    def zll(self) -> U8Vector3: ...
    @property
    def wxx(self) -> U8Vector3: ...
    @property
    def wxy(self) -> U8Vector3: ...
    @property
    def wxz(self) -> U8Vector3: ...
    @property
    def wxw(self) -> U8Vector3: ...
    @property
    def wxo(self) -> U8Vector3: ...
    @property
    def wxl(self) -> U8Vector3: ...
    @property
    def wyx(self) -> U8Vector3: ...
    @property
    def wyy(self) -> U8Vector3: ...
    @property
    def wyz(self) -> U8Vector3: ...
    @property
    def wyw(self) -> U8Vector3: ...
    @property
    def wyo(self) -> U8Vector3: ...
    @property
    def wyl(self) -> U8Vector3: ...
    @property
    def wzx(self) -> U8Vector3: ...
    @property
    def wzy(self) -> U8Vector3: ...
    @property
    def wzz(self) -> U8Vector3: ...
    @property
    def wzw(self) -> U8Vector3: ...
    @property
    def wzo(self) -> U8Vector3: ...
    @property
    def wzl(self) -> U8Vector3: ...
    @property
    def wwx(self) -> U8Vector3: ...
    @property
    def wwy(self) -> U8Vector3: ...
    @property
    def wwz(self) -> U8Vector3: ...
    @property
    def www(self) -> U8Vector3: ...
    @property
    def wwo(self) -> U8Vector3: ...
    @property
    def wwl(self) -> U8Vector3: ...
    @property
    def wox(self) -> U8Vector3: ...
    @property
    def woy(self) -> U8Vector3: ...
    @property
    def woz(self) -> U8Vector3: ...
    @property
    def wow(self) -> U8Vector3: ...
    @property
    def woo(self) -> U8Vector3: ...
    @property
    def wol(self) -> U8Vector3: ...
    @property
    def wlx(self) -> U8Vector3: ...
    @property
    def wly(self) -> U8Vector3: ...
    @property
    def wlz(self) -> U8Vector3: ...
    @property
    def wlw(self) -> U8Vector3: ...
    @property
    def wlo(self) -> U8Vector3: ...
    @property
    def wll(self) -> U8Vector3: ...
    @property
    def oxx(self) -> U8Vector3: ...
    @property
    def oxy(self) -> U8Vector3: ...
    @property
    def oxz(self) -> U8Vector3: ...
    @property
    def oxw(self) -> U8Vector3: ...
    @property
    def oxo(self) -> U8Vector3: ...
    @property
    def oxl(self) -> U8Vector3: ...
    @property
    def oyx(self) -> U8Vector3: ...
    @property
    def oyy(self) -> U8Vector3: ...
    @property
    def oyz(self) -> U8Vector3: ...
    @property
    def oyw(self) -> U8Vector3: ...
    @property
    def oyo(self) -> U8Vector3: ...
    @property
    def oyl(self) -> U8Vector3: ...
    @property
    def ozx(self) -> U8Vector3: ...
    @property
    def ozy(self) -> U8Vector3: ...
    @property
    def ozz(self) -> U8Vector3: ...
    @property
    def ozw(self) -> U8Vector3: ...
    @property
    def ozo(self) -> U8Vector3: ...
    @property
    def ozl(self) -> U8Vector3: ...
    @property
    def owx(self) -> U8Vector3: ...
    @property
    def owy(self) -> U8Vector3: ...
    @property
    def owz(self) -> U8Vector3: ...
    @property
    def oww(self) -> U8Vector3: ...
    @property
    def owo(self) -> U8Vector3: ...
    @property
    def owl(self) -> U8Vector3: ...
    @property
    def oox(self) -> U8Vector3: ...
    @property
    def ooy(self) -> U8Vector3: ...
    @property
    def ooz(self) -> U8Vector3: ...
    @property
    def oow(self) -> U8Vector3: ...
    @property
    def olx(self) -> U8Vector3: ...
    @property
    def oly(self) -> U8Vector3: ...
    @property
    def olz(self) -> U8Vector3: ...
    @property
    def olw(self) -> U8Vector3: ...
    @property
    def lxx(self) -> U8Vector3: ...
    @property
    def lxy(self) -> U8Vector3: ...
    @property
    def lxz(self) -> U8Vector3: ...
    @property
    def lxw(self) -> U8Vector3: ...
    @property
    def lxo(self) -> U8Vector3: ...
    @property
    def lxl(self) -> U8Vector3: ...
    @property
    def lyx(self) -> U8Vector3: ...
    @property
    def lyy(self) -> U8Vector3: ...
    @property
    def lyz(self) -> U8Vector3: ...
    @property
    def lyw(self) -> U8Vector3: ...
    @property
    def lyo(self) -> U8Vector3: ...
    @property
    def lyl(self) -> U8Vector3: ...
    @property
    def lzx(self) -> U8Vector3: ...
    @property
    def lzy(self) -> U8Vector3: ...
    @property
    def lzz(self) -> U8Vector3: ...
    @property
    def lzw(self) -> U8Vector3: ...
    @property
    def lzo(self) -> U8Vector3: ...
    @property
    def lzl(self) -> U8Vector3: ...
    @property
    def lwx(self) -> U8Vector3: ...
    @property
    def lwy(self) -> U8Vector3: ...
    @property
    def lwz(self) -> U8Vector3: ...
    @property
    def lww(self) -> U8Vector3: ...
    @property
    def lwo(self) -> U8Vector3: ...
    @property
    def lwl(self) -> U8Vector3: ...
    @property
    def lox(self) -> U8Vector3: ...
    @property
    def loy(self) -> U8Vector3: ...
    @property
    def loz(self) -> U8Vector3: ...
    @property
    def low(self) -> U8Vector3: ...
    @property
    def llx(self) -> U8Vector3: ...
    @property
    def lly(self) -> U8Vector3: ...
    @property
    def llz(self) -> U8Vector3: ...
    @property
    def llw(self) -> U8Vector3: ...
    @property
    def xxxx(self) -> U8Vector4: ...
    @property
    def xxxy(self) -> U8Vector4: ...
    @property
    def xxxz(self) -> U8Vector4: ...
    @property
    def xxxw(self) -> U8Vector4: ...
    @property
    def xxxo(self) -> U8Vector4: ...
    @property
    def xxxl(self) -> U8Vector4: ...
    @property
    def xxyx(self) -> U8Vector4: ...
    @property
    def xxyy(self) -> U8Vector4: ...
    @property
    def xxyz(self) -> U8Vector4: ...
    @property
    def xxyw(self) -> U8Vector4: ...
    @property
    def xxyo(self) -> U8Vector4: ...
    @property
    def xxyl(self) -> U8Vector4: ...
    @property
    def xxzx(self) -> U8Vector4: ...
    @property
    def xxzy(self) -> U8Vector4: ...
    @property
    def xxzz(self) -> U8Vector4: ...
    @property
    def xxzw(self) -> U8Vector4: ...
    @property
    def xxzo(self) -> U8Vector4: ...
    @property
    def xxzl(self) -> U8Vector4: ...
    @property
    def xxwx(self) -> U8Vector4: ...
    @property
    def xxwy(self) -> U8Vector4: ...
    @property
    def xxwz(self) -> U8Vector4: ...
    @property
    def xxww(self) -> U8Vector4: ...
    @property
    def xxwo(self) -> U8Vector4: ...
    @property
    def xxwl(self) -> U8Vector4: ...
    @property
    def xxox(self) -> U8Vector4: ...
    @property
    def xxoy(self) -> U8Vector4: ...
    @property
    def xxoz(self) -> U8Vector4: ...
    @property
    def xxow(self) -> U8Vector4: ...
    @property
    def xxoo(self) -> U8Vector4: ...
    @property
    def xxol(self) -> U8Vector4: ...
    @property
    def xxlx(self) -> U8Vector4: ...
    @property
    def xxly(self) -> U8Vector4: ...
    @property
    def xxlz(self) -> U8Vector4: ...
    @property
    def xxlw(self) -> U8Vector4: ...
    @property
    def xxlo(self) -> U8Vector4: ...
    @property
    def xxll(self) -> U8Vector4: ...
    @property
    def xyxx(self) -> U8Vector4: ...
    @property
    def xyxy(self) -> U8Vector4: ...
    @property
    def xyxz(self) -> U8Vector4: ...
    @property
    def xyxw(self) -> U8Vector4: ...
    @property
    def xyxo(self) -> U8Vector4: ...
    @property
    def xyxl(self) -> U8Vector4: ...
    @property
    def xyyx(self) -> U8Vector4: ...
    @property
    def xyyy(self) -> U8Vector4: ...
    @property
    def xyyz(self) -> U8Vector4: ...
    @property
    def xyyw(self) -> U8Vector4: ...
    @property
    def xyyo(self) -> U8Vector4: ...
    @property
    def xyyl(self) -> U8Vector4: ...
    @property
    def xyzx(self) -> U8Vector4: ...
    @property
    def xyzy(self) -> U8Vector4: ...
    @property
    def xyzz(self) -> U8Vector4: ...
    @property
    def xyzw(self) -> U8Vector4: ...
    @property
    def xyzo(self) -> U8Vector4: ...
    @property
    def xyzl(self) -> U8Vector4: ...
    @property
    def xywx(self) -> U8Vector4: ...
    @property
    def xywy(self) -> U8Vector4: ...
    @property
    def xywz(self) -> U8Vector4: ...
    @property
    def xyww(self) -> U8Vector4: ...
    @property
    def xywo(self) -> U8Vector4: ...
    @property
    def xywl(self) -> U8Vector4: ...
    @property
    def xyox(self) -> U8Vector4: ...
    @property
    def xyoy(self) -> U8Vector4: ...
    @property
    def xyoz(self) -> U8Vector4: ...
    @property
    def xyow(self) -> U8Vector4: ...
    @property
    def xyoo(self) -> U8Vector4: ...
    @property
    def xyol(self) -> U8Vector4: ...
    @property
    def xylx(self) -> U8Vector4: ...
    @property
    def xyly(self) -> U8Vector4: ...
    @property
    def xylz(self) -> U8Vector4: ...
    @property
    def xylw(self) -> U8Vector4: ...
    @property
    def xylo(self) -> U8Vector4: ...
    @property
    def xyll(self) -> U8Vector4: ...
    @property
    def xzxx(self) -> U8Vector4: ...
    @property
    def xzxy(self) -> U8Vector4: ...
    @property
    def xzxz(self) -> U8Vector4: ...
    @property
    def xzxw(self) -> U8Vector4: ...
    @property
    def xzxo(self) -> U8Vector4: ...
    @property
    def xzxl(self) -> U8Vector4: ...
    @property
    def xzyx(self) -> U8Vector4: ...
    @property
    def xzyy(self) -> U8Vector4: ...
    @property
    def xzyz(self) -> U8Vector4: ...
    @property
    def xzyw(self) -> U8Vector4: ...
    @property
    def xzyo(self) -> U8Vector4: ...
    @property
    def xzyl(self) -> U8Vector4: ...
    @property
    def xzzx(self) -> U8Vector4: ...
    @property
    def xzzy(self) -> U8Vector4: ...
    @property
    def xzzz(self) -> U8Vector4: ...
    @property
    def xzzw(self) -> U8Vector4: ...
    @property
    def xzzo(self) -> U8Vector4: ...
    @property
    def xzzl(self) -> U8Vector4: ...
    @property
    def xzwx(self) -> U8Vector4: ...
    @property
    def xzwy(self) -> U8Vector4: ...
    @property
    def xzwz(self) -> U8Vector4: ...
    @property
    def xzww(self) -> U8Vector4: ...
    @property
    def xzwo(self) -> U8Vector4: ...
    @property
    def xzwl(self) -> U8Vector4: ...
    @property
    def xzox(self) -> U8Vector4: ...
    @property
    def xzoy(self) -> U8Vector4: ...
    @property
    def xzoz(self) -> U8Vector4: ...
    @property
    def xzow(self) -> U8Vector4: ...
    @property
    def xzoo(self) -> U8Vector4: ...
    @property
    def xzol(self) -> U8Vector4: ...
    @property
    def xzlx(self) -> U8Vector4: ...
    @property
    def xzly(self) -> U8Vector4: ...
    @property
    def xzlz(self) -> U8Vector4: ...
    @property
    def xzlw(self) -> U8Vector4: ...
    @property
    def xzlo(self) -> U8Vector4: ...
    @property
    def xzll(self) -> U8Vector4: ...
    @property
    def xwxx(self) -> U8Vector4: ...
    @property
    def xwxy(self) -> U8Vector4: ...
    @property
    def xwxz(self) -> U8Vector4: ...
    @property
    def xwxw(self) -> U8Vector4: ...
    @property
    def xwxo(self) -> U8Vector4: ...
    @property
    def xwxl(self) -> U8Vector4: ...
    @property
    def xwyx(self) -> U8Vector4: ...
    @property
    def xwyy(self) -> U8Vector4: ...
    @property
    def xwyz(self) -> U8Vector4: ...
    @property
    def xwyw(self) -> U8Vector4: ...
    @property
    def xwyo(self) -> U8Vector4: ...
    @property
    def xwyl(self) -> U8Vector4: ...
    @property
    def xwzx(self) -> U8Vector4: ...
    @property
    def xwzy(self) -> U8Vector4: ...
    @property
    def xwzz(self) -> U8Vector4: ...
    @property
    def xwzw(self) -> U8Vector4: ...
    @property
    def xwzo(self) -> U8Vector4: ...
    @property
    def xwzl(self) -> U8Vector4: ...
    @property
    def xwwx(self) -> U8Vector4: ...
    @property
    def xwwy(self) -> U8Vector4: ...
    @property
    def xwwz(self) -> U8Vector4: ...
    @property
    def xwww(self) -> U8Vector4: ...
    @property
    def xwwo(self) -> U8Vector4: ...
    @property
    def xwwl(self) -> U8Vector4: ...
    @property
    def xwox(self) -> U8Vector4: ...
    @property
    def xwoy(self) -> U8Vector4: ...
    @property
    def xwoz(self) -> U8Vector4: ...
    @property
    def xwow(self) -> U8Vector4: ...
    @property
    def xwoo(self) -> U8Vector4: ...
    @property
    def xwol(self) -> U8Vector4: ...
    @property
    def xwlx(self) -> U8Vector4: ...
    @property
    def xwly(self) -> U8Vector4: ...
    @property
    def xwlz(self) -> U8Vector4: ...
    @property
    def xwlw(self) -> U8Vector4: ...
    @property
    def xwlo(self) -> U8Vector4: ...
    @property
    def xwll(self) -> U8Vector4: ...
    @property
    def xoxx(self) -> U8Vector4: ...
    @property
    def xoxy(self) -> U8Vector4: ...
    @property
    def xoxz(self) -> U8Vector4: ...
    @property
    def xoxw(self) -> U8Vector4: ...
    @property
    def xoxo(self) -> U8Vector4: ...
    @property
    def xoxl(self) -> U8Vector4: ...
    @property
    def xoyx(self) -> U8Vector4: ...
    @property
    def xoyy(self) -> U8Vector4: ...
    @property
    def xoyz(self) -> U8Vector4: ...
    @property
    def xoyw(self) -> U8Vector4: ...
    @property
    def xoyo(self) -> U8Vector4: ...
    @property
    def xoyl(self) -> U8Vector4: ...
    @property
    def xozx(self) -> U8Vector4: ...
    @property
    def xozy(self) -> U8Vector4: ...
    @property
    def xozz(self) -> U8Vector4: ...
    @property
    def xozw(self) -> U8Vector4: ...
    @property
    def xozo(self) -> U8Vector4: ...
    @property
    def xozl(self) -> U8Vector4: ...
    @property
    def xowx(self) -> U8Vector4: ...
    @property
    def xowy(self) -> U8Vector4: ...
    @property
    def xowz(self) -> U8Vector4: ...
    @property
    def xoww(self) -> U8Vector4: ...
    @property
    def xowo(self) -> U8Vector4: ...
    @property
    def xowl(self) -> U8Vector4: ...
    @property
    def xoox(self) -> U8Vector4: ...
    @property
    def xooy(self) -> U8Vector4: ...
    @property
    def xooz(self) -> U8Vector4: ...
    @property
    def xoow(self) -> U8Vector4: ...
    @property
    def xooo(self) -> U8Vector4: ...
    @property
    def xool(self) -> U8Vector4: ...
    @property
    def xolx(self) -> U8Vector4: ...
    @property
    def xoly(self) -> U8Vector4: ...
    @property
    def xolz(self) -> U8Vector4: ...
    @property
    def xolw(self) -> U8Vector4: ...
    @property
    def xolo(self) -> U8Vector4: ...
    @property
    def xoll(self) -> U8Vector4: ...
    @property
    def xlxx(self) -> U8Vector4: ...
    @property
    def xlxy(self) -> U8Vector4: ...
    @property
    def xlxz(self) -> U8Vector4: ...
    @property
    def xlxw(self) -> U8Vector4: ...
    @property
    def xlxo(self) -> U8Vector4: ...
    @property
    def xlxl(self) -> U8Vector4: ...
    @property
    def xlyx(self) -> U8Vector4: ...
    @property
    def xlyy(self) -> U8Vector4: ...
    @property
    def xlyz(self) -> U8Vector4: ...
    @property
    def xlyw(self) -> U8Vector4: ...
    @property
    def xlyo(self) -> U8Vector4: ...
    @property
    def xlyl(self) -> U8Vector4: ...
    @property
    def xlzx(self) -> U8Vector4: ...
    @property
    def xlzy(self) -> U8Vector4: ...
    @property
    def xlzz(self) -> U8Vector4: ...
    @property
    def xlzw(self) -> U8Vector4: ...
    @property
    def xlzo(self) -> U8Vector4: ...
    @property
    def xlzl(self) -> U8Vector4: ...
    @property
    def xlwx(self) -> U8Vector4: ...
    @property
    def xlwy(self) -> U8Vector4: ...
    @property
    def xlwz(self) -> U8Vector4: ...
    @property
    def xlww(self) -> U8Vector4: ...
    @property
    def xlwo(self) -> U8Vector4: ...
    @property
    def xlwl(self) -> U8Vector4: ...
    @property
    def xlox(self) -> U8Vector4: ...
    @property
    def xloy(self) -> U8Vector4: ...
    @property
    def xloz(self) -> U8Vector4: ...
    @property
    def xlow(self) -> U8Vector4: ...
    @property
    def xloo(self) -> U8Vector4: ...
    @property
    def xlol(self) -> U8Vector4: ...
    @property
    def xllx(self) -> U8Vector4: ...
    @property
    def xlly(self) -> U8Vector4: ...
    @property
    def xllz(self) -> U8Vector4: ...
    @property
    def xllw(self) -> U8Vector4: ...
    @property
    def xllo(self) -> U8Vector4: ...
    @property
    def xlll(self) -> U8Vector4: ...
    @property
    def yxxx(self) -> U8Vector4: ...
    @property
    def yxxy(self) -> U8Vector4: ...
    @property
    def yxxz(self) -> U8Vector4: ...
    @property
    def yxxw(self) -> U8Vector4: ...
    @property
    def yxxo(self) -> U8Vector4: ...
    @property
    def yxxl(self) -> U8Vector4: ...
    @property
    def yxyx(self) -> U8Vector4: ...
    @property
    def yxyy(self) -> U8Vector4: ...
    @property
    def yxyz(self) -> U8Vector4: ...
    @property
    def yxyw(self) -> U8Vector4: ...
    @property
    def yxyo(self) -> U8Vector4: ...
    @property
    def yxyl(self) -> U8Vector4: ...
    @property
    def yxzx(self) -> U8Vector4: ...
    @property
    def yxzy(self) -> U8Vector4: ...
    @property
    def yxzz(self) -> U8Vector4: ...
    @property
    def yxzw(self) -> U8Vector4: ...
    @property
    def yxzo(self) -> U8Vector4: ...
    @property
    def yxzl(self) -> U8Vector4: ...
    @property
    def yxwx(self) -> U8Vector4: ...
    @property
    def yxwy(self) -> U8Vector4: ...
    @property
    def yxwz(self) -> U8Vector4: ...
    @property
    def yxww(self) -> U8Vector4: ...
    @property
    def yxwo(self) -> U8Vector4: ...
    @property
    def yxwl(self) -> U8Vector4: ...
    @property
    def yxox(self) -> U8Vector4: ...
    @property
    def yxoy(self) -> U8Vector4: ...
    @property
    def yxoz(self) -> U8Vector4: ...
    @property
    def yxow(self) -> U8Vector4: ...
    @property
    def yxoo(self) -> U8Vector4: ...
    @property
    def yxol(self) -> U8Vector4: ...
    @property
    def yxlx(self) -> U8Vector4: ...
    @property
    def yxly(self) -> U8Vector4: ...
    @property
    def yxlz(self) -> U8Vector4: ...
    @property
    def yxlw(self) -> U8Vector4: ...
    @property
    def yxlo(self) -> U8Vector4: ...
    @property
    def yxll(self) -> U8Vector4: ...
    @property
    def yyxx(self) -> U8Vector4: ...
    @property
    def yyxy(self) -> U8Vector4: ...
    @property
    def yyxz(self) -> U8Vector4: ...
    @property
    def yyxw(self) -> U8Vector4: ...
    @property
    def yyxo(self) -> U8Vector4: ...
    @property
    def yyxl(self) -> U8Vector4: ...
    @property
    def yyyx(self) -> U8Vector4: ...
    @property
    def yyyy(self) -> U8Vector4: ...
    @property
    def yyyz(self) -> U8Vector4: ...
    @property
    def yyyw(self) -> U8Vector4: ...
    @property
    def yyyo(self) -> U8Vector4: ...
    @property
    def yyyl(self) -> U8Vector4: ...
    @property
    def yyzx(self) -> U8Vector4: ...
    @property
    def yyzy(self) -> U8Vector4: ...
    @property
    def yyzz(self) -> U8Vector4: ...
    @property
    def yyzw(self) -> U8Vector4: ...
    @property
    def yyzo(self) -> U8Vector4: ...
    @property
    def yyzl(self) -> U8Vector4: ...
    @property
    def yywx(self) -> U8Vector4: ...
    @property
    def yywy(self) -> U8Vector4: ...
    @property
    def yywz(self) -> U8Vector4: ...
    @property
    def yyww(self) -> U8Vector4: ...
    @property
    def yywo(self) -> U8Vector4: ...
    @property
    def yywl(self) -> U8Vector4: ...
    @property
    def yyox(self) -> U8Vector4: ...
    @property
    def yyoy(self) -> U8Vector4: ...
    @property
    def yyoz(self) -> U8Vector4: ...
    @property
    def yyow(self) -> U8Vector4: ...
    @property
    def yyoo(self) -> U8Vector4: ...
    @property
    def yyol(self) -> U8Vector4: ...
    @property
    def yylx(self) -> U8Vector4: ...
    @property
    def yyly(self) -> U8Vector4: ...
    @property
    def yylz(self) -> U8Vector4: ...
    @property
    def yylw(self) -> U8Vector4: ...
    @property
    def yylo(self) -> U8Vector4: ...
    @property
    def yyll(self) -> U8Vector4: ...
    @property
    def yzxx(self) -> U8Vector4: ...
    @property
    def yzxy(self) -> U8Vector4: ...
    @property
    def yzxz(self) -> U8Vector4: ...
    @property
    def yzxw(self) -> U8Vector4: ...
    @property
    def yzxo(self) -> U8Vector4: ...
    @property
    def yzxl(self) -> U8Vector4: ...
    @property
    def yzyx(self) -> U8Vector4: ...
    @property
    def yzyy(self) -> U8Vector4: ...
    @property
    def yzyz(self) -> U8Vector4: ...
    @property
    def yzyw(self) -> U8Vector4: ...
    @property
    def yzyo(self) -> U8Vector4: ...
    @property
    def yzyl(self) -> U8Vector4: ...
    @property
    def yzzx(self) -> U8Vector4: ...
    @property
    def yzzy(self) -> U8Vector4: ...
    @property
    def yzzz(self) -> U8Vector4: ...
    @property
    def yzzw(self) -> U8Vector4: ...
    @property
    def yzzo(self) -> U8Vector4: ...
    @property
    def yzzl(self) -> U8Vector4: ...
    @property
    def yzwx(self) -> U8Vector4: ...
    @property
    def yzwy(self) -> U8Vector4: ...
    @property
    def yzwz(self) -> U8Vector4: ...
    @property
    def yzww(self) -> U8Vector4: ...
    @property
    def yzwo(self) -> U8Vector4: ...
    @property
    def yzwl(self) -> U8Vector4: ...
    @property
    def yzox(self) -> U8Vector4: ...
    @property
    def yzoy(self) -> U8Vector4: ...
    @property
    def yzoz(self) -> U8Vector4: ...
    @property
    def yzow(self) -> U8Vector4: ...
    @property
    def yzoo(self) -> U8Vector4: ...
    @property
    def yzol(self) -> U8Vector4: ...
    @property
    def yzlx(self) -> U8Vector4: ...
    @property
    def yzly(self) -> U8Vector4: ...
    @property
    def yzlz(self) -> U8Vector4: ...
    @property
    def yzlw(self) -> U8Vector4: ...
    @property
    def yzlo(self) -> U8Vector4: ...
    @property
    def yzll(self) -> U8Vector4: ...
    @property
    def ywxx(self) -> U8Vector4: ...
    @property
    def ywxy(self) -> U8Vector4: ...
    @property
    def ywxz(self) -> U8Vector4: ...
    @property
    def ywxw(self) -> U8Vector4: ...
    @property
    def ywxo(self) -> U8Vector4: ...
    @property
    def ywxl(self) -> U8Vector4: ...
    @property
    def ywyx(self) -> U8Vector4: ...
    @property
    def ywyy(self) -> U8Vector4: ...
    @property
    def ywyz(self) -> U8Vector4: ...
    @property
    def ywyw(self) -> U8Vector4: ...
    @property
    def ywyo(self) -> U8Vector4: ...
    @property
    def ywyl(self) -> U8Vector4: ...
    @property
    def ywzx(self) -> U8Vector4: ...
    @property
    def ywzy(self) -> U8Vector4: ...
    @property
    def ywzz(self) -> U8Vector4: ...
    @property
    def ywzw(self) -> U8Vector4: ...
    @property
    def ywzo(self) -> U8Vector4: ...
    @property
    def ywzl(self) -> U8Vector4: ...
    @property
    def ywwx(self) -> U8Vector4: ...
    @property
    def ywwy(self) -> U8Vector4: ...
    @property
    def ywwz(self) -> U8Vector4: ...
    @property
    def ywww(self) -> U8Vector4: ...
    @property
    def ywwo(self) -> U8Vector4: ...
    @property
    def ywwl(self) -> U8Vector4: ...
    @property
    def ywox(self) -> U8Vector4: ...
    @property
    def ywoy(self) -> U8Vector4: ...
    @property
    def ywoz(self) -> U8Vector4: ...
    @property
    def ywow(self) -> U8Vector4: ...
    @property
    def ywoo(self) -> U8Vector4: ...
    @property
    def ywol(self) -> U8Vector4: ...
    @property
    def ywlx(self) -> U8Vector4: ...
    @property
    def ywly(self) -> U8Vector4: ...
    @property
    def ywlz(self) -> U8Vector4: ...
    @property
    def ywlw(self) -> U8Vector4: ...
    @property
    def ywlo(self) -> U8Vector4: ...
    @property
    def ywll(self) -> U8Vector4: ...
    @property
    def yoxx(self) -> U8Vector4: ...
    @property
    def yoxy(self) -> U8Vector4: ...
    @property
    def yoxz(self) -> U8Vector4: ...
    @property
    def yoxw(self) -> U8Vector4: ...
    @property
    def yoxo(self) -> U8Vector4: ...
    @property
    def yoxl(self) -> U8Vector4: ...
    @property
    def yoyx(self) -> U8Vector4: ...
    @property
    def yoyy(self) -> U8Vector4: ...
    @property
    def yoyz(self) -> U8Vector4: ...
    @property
    def yoyw(self) -> U8Vector4: ...
    @property
    def yoyo(self) -> U8Vector4: ...
    @property
    def yoyl(self) -> U8Vector4: ...
    @property
    def yozx(self) -> U8Vector4: ...
    @property
    def yozy(self) -> U8Vector4: ...
    @property
    def yozz(self) -> U8Vector4: ...
    @property
    def yozw(self) -> U8Vector4: ...
    @property
    def yozo(self) -> U8Vector4: ...
    @property
    def yozl(self) -> U8Vector4: ...
    @property
    def yowx(self) -> U8Vector4: ...
    @property
    def yowy(self) -> U8Vector4: ...
    @property
    def yowz(self) -> U8Vector4: ...
    @property
    def yoww(self) -> U8Vector4: ...
    @property
    def yowo(self) -> U8Vector4: ...
    @property
    def yowl(self) -> U8Vector4: ...
    @property
    def yoox(self) -> U8Vector4: ...
    @property
    def yooy(self) -> U8Vector4: ...
    @property
    def yooz(self) -> U8Vector4: ...
    @property
    def yoow(self) -> U8Vector4: ...
    @property
    def yooo(self) -> U8Vector4: ...
    @property
    def yool(self) -> U8Vector4: ...
    @property
    def yolx(self) -> U8Vector4: ...
    @property
    def yoly(self) -> U8Vector4: ...
    @property
    def yolz(self) -> U8Vector4: ...
    @property
    def yolw(self) -> U8Vector4: ...
    @property
    def yolo(self) -> U8Vector4: ...
    @property
    def yoll(self) -> U8Vector4: ...
    @property
    def ylxx(self) -> U8Vector4: ...
    @property
    def ylxy(self) -> U8Vector4: ...
    @property
    def ylxz(self) -> U8Vector4: ...
    @property
    def ylxw(self) -> U8Vector4: ...
    @property
    def ylxo(self) -> U8Vector4: ...
    @property
    def ylxl(self) -> U8Vector4: ...
    @property
    def ylyx(self) -> U8Vector4: ...
    @property
    def ylyy(self) -> U8Vector4: ...
    @property
    def ylyz(self) -> U8Vector4: ...
    @property
    def ylyw(self) -> U8Vector4: ...
    @property
    def ylyo(self) -> U8Vector4: ...
    @property
    def ylyl(self) -> U8Vector4: ...
    @property
    def ylzx(self) -> U8Vector4: ...
    @property
    def ylzy(self) -> U8Vector4: ...
    @property
    def ylzz(self) -> U8Vector4: ...
    @property
    def ylzw(self) -> U8Vector4: ...
    @property
    def ylzo(self) -> U8Vector4: ...
    @property
    def ylzl(self) -> U8Vector4: ...
    @property
    def ylwx(self) -> U8Vector4: ...
    @property
    def ylwy(self) -> U8Vector4: ...
    @property
    def ylwz(self) -> U8Vector4: ...
    @property
    def ylww(self) -> U8Vector4: ...
    @property
    def ylwo(self) -> U8Vector4: ...
    @property
    def ylwl(self) -> U8Vector4: ...
    @property
    def ylox(self) -> U8Vector4: ...
    @property
    def yloy(self) -> U8Vector4: ...
    @property
    def yloz(self) -> U8Vector4: ...
    @property
    def ylow(self) -> U8Vector4: ...
    @property
    def yloo(self) -> U8Vector4: ...
    @property
    def ylol(self) -> U8Vector4: ...
    @property
    def yllx(self) -> U8Vector4: ...
    @property
    def ylly(self) -> U8Vector4: ...
    @property
    def yllz(self) -> U8Vector4: ...
    @property
    def yllw(self) -> U8Vector4: ...
    @property
    def yllo(self) -> U8Vector4: ...
    @property
    def ylll(self) -> U8Vector4: ...
    @property
    def zxxx(self) -> U8Vector4: ...
    @property
    def zxxy(self) -> U8Vector4: ...
    @property
    def zxxz(self) -> U8Vector4: ...
    @property
    def zxxw(self) -> U8Vector4: ...
    @property
    def zxxo(self) -> U8Vector4: ...
    @property
    def zxxl(self) -> U8Vector4: ...
    @property
    def zxyx(self) -> U8Vector4: ...
    @property
    def zxyy(self) -> U8Vector4: ...
    @property
    def zxyz(self) -> U8Vector4: ...
    @property
    def zxyw(self) -> U8Vector4: ...
    @property
    def zxyo(self) -> U8Vector4: ...
    @property
    def zxyl(self) -> U8Vector4: ...
    @property
    def zxzx(self) -> U8Vector4: ...
    @property
    def zxzy(self) -> U8Vector4: ...
    @property
    def zxzz(self) -> U8Vector4: ...
    @property
    def zxzw(self) -> U8Vector4: ...
    @property
    def zxzo(self) -> U8Vector4: ...
    @property
    def zxzl(self) -> U8Vector4: ...
    @property
    def zxwx(self) -> U8Vector4: ...
    @property
    def zxwy(self) -> U8Vector4: ...
    @property
    def zxwz(self) -> U8Vector4: ...
    @property
    def zxww(self) -> U8Vector4: ...
    @property
    def zxwo(self) -> U8Vector4: ...
    @property
    def zxwl(self) -> U8Vector4: ...
    @property
    def zxox(self) -> U8Vector4: ...
    @property
    def zxoy(self) -> U8Vector4: ...
    @property
    def zxoz(self) -> U8Vector4: ...
    @property
    def zxow(self) -> U8Vector4: ...
    @property
    def zxoo(self) -> U8Vector4: ...
    @property
    def zxol(self) -> U8Vector4: ...
    @property
    def zxlx(self) -> U8Vector4: ...
    @property
    def zxly(self) -> U8Vector4: ...
    @property
    def zxlz(self) -> U8Vector4: ...
    @property
    def zxlw(self) -> U8Vector4: ...
    @property
    def zxlo(self) -> U8Vector4: ...
    @property
    def zxll(self) -> U8Vector4: ...
    @property
    def zyxx(self) -> U8Vector4: ...
    @property
    def zyxy(self) -> U8Vector4: ...
    @property
    def zyxz(self) -> U8Vector4: ...
    @property
    def zyxw(self) -> U8Vector4: ...
    @property
    def zyxo(self) -> U8Vector4: ...
    @property
    def zyxl(self) -> U8Vector4: ...
    @property
    def zyyx(self) -> U8Vector4: ...
    @property
    def zyyy(self) -> U8Vector4: ...
    @property
    def zyyz(self) -> U8Vector4: ...
    @property
    def zyyw(self) -> U8Vector4: ...
    @property
    def zyyo(self) -> U8Vector4: ...
    @property
    def zyyl(self) -> U8Vector4: ...
    @property
    def zyzx(self) -> U8Vector4: ...
    @property
    def zyzy(self) -> U8Vector4: ...
    @property
    def zyzz(self) -> U8Vector4: ...
    @property
    def zyzw(self) -> U8Vector4: ...
    @property
    def zyzo(self) -> U8Vector4: ...
    @property
    def zyzl(self) -> U8Vector4: ...
    @property
    def zywx(self) -> U8Vector4: ...
    @property
    def zywy(self) -> U8Vector4: ...
    @property
    def zywz(self) -> U8Vector4: ...
    @property
    def zyww(self) -> U8Vector4: ...
    @property
    def zywo(self) -> U8Vector4: ...
    @property
    def zywl(self) -> U8Vector4: ...
    @property
    def zyox(self) -> U8Vector4: ...
    @property
    def zyoy(self) -> U8Vector4: ...
    @property
    def zyoz(self) -> U8Vector4: ...
    @property
    def zyow(self) -> U8Vector4: ...
    @property
    def zyoo(self) -> U8Vector4: ...
    @property
    def zyol(self) -> U8Vector4: ...
    @property
    def zylx(self) -> U8Vector4: ...
    @property
    def zyly(self) -> U8Vector4: ...
    @property
    def zylz(self) -> U8Vector4: ...
    @property
    def zylw(self) -> U8Vector4: ...
    @property
    def zylo(self) -> U8Vector4: ...
    @property
    def zyll(self) -> U8Vector4: ...
    @property
    def zzxx(self) -> U8Vector4: ...
    @property
    def zzxy(self) -> U8Vector4: ...
    @property
    def zzxz(self) -> U8Vector4: ...
    @property
    def zzxw(self) -> U8Vector4: ...
    @property
    def zzxo(self) -> U8Vector4: ...
    @property
    def zzxl(self) -> U8Vector4: ...
    @property
    def zzyx(self) -> U8Vector4: ...
    @property
    def zzyy(self) -> U8Vector4: ...
    @property
    def zzyz(self) -> U8Vector4: ...
    @property
    def zzyw(self) -> U8Vector4: ...
    @property
    def zzyo(self) -> U8Vector4: ...
    @property
    def zzyl(self) -> U8Vector4: ...
    @property
    def zzzx(self) -> U8Vector4: ...
    @property
    def zzzy(self) -> U8Vector4: ...
    @property
    def zzzz(self) -> U8Vector4: ...
    @property
    def zzzw(self) -> U8Vector4: ...
    @property
    def zzzo(self) -> U8Vector4: ...
    @property
    def zzzl(self) -> U8Vector4: ...
    @property
    def zzwx(self) -> U8Vector4: ...
    @property
    def zzwy(self) -> U8Vector4: ...
    @property
    def zzwz(self) -> U8Vector4: ...
    @property
    def zzww(self) -> U8Vector4: ...
    @property
    def zzwo(self) -> U8Vector4: ...
    @property
    def zzwl(self) -> U8Vector4: ...
    @property
    def zzox(self) -> U8Vector4: ...
    @property
    def zzoy(self) -> U8Vector4: ...
    @property
    def zzoz(self) -> U8Vector4: ...
    @property
    def zzow(self) -> U8Vector4: ...
    @property
    def zzoo(self) -> U8Vector4: ...
    @property
    def zzol(self) -> U8Vector4: ...
    @property
    def zzlx(self) -> U8Vector4: ...
    @property
    def zzly(self) -> U8Vector4: ...
    @property
    def zzlz(self) -> U8Vector4: ...
    @property
    def zzlw(self) -> U8Vector4: ...
    @property
    def zzlo(self) -> U8Vector4: ...
    @property
    def zzll(self) -> U8Vector4: ...
    @property
    def zwxx(self) -> U8Vector4: ...
    @property
    def zwxy(self) -> U8Vector4: ...
    @property
    def zwxz(self) -> U8Vector4: ...
    @property
    def zwxw(self) -> U8Vector4: ...
    @property
    def zwxo(self) -> U8Vector4: ...
    @property
    def zwxl(self) -> U8Vector4: ...
    @property
    def zwyx(self) -> U8Vector4: ...
    @property
    def zwyy(self) -> U8Vector4: ...
    @property
    def zwyz(self) -> U8Vector4: ...
    @property
    def zwyw(self) -> U8Vector4: ...
    @property
    def zwyo(self) -> U8Vector4: ...
    @property
    def zwyl(self) -> U8Vector4: ...
    @property
    def zwzx(self) -> U8Vector4: ...
    @property
    def zwzy(self) -> U8Vector4: ...
    @property
    def zwzz(self) -> U8Vector4: ...
    @property
    def zwzw(self) -> U8Vector4: ...
    @property
    def zwzo(self) -> U8Vector4: ...
    @property
    def zwzl(self) -> U8Vector4: ...
    @property
    def zwwx(self) -> U8Vector4: ...
    @property
    def zwwy(self) -> U8Vector4: ...
    @property
    def zwwz(self) -> U8Vector4: ...
    @property
    def zwww(self) -> U8Vector4: ...
    @property
    def zwwo(self) -> U8Vector4: ...
    @property
    def zwwl(self) -> U8Vector4: ...
    @property
    def zwox(self) -> U8Vector4: ...
    @property
    def zwoy(self) -> U8Vector4: ...
    @property
    def zwoz(self) -> U8Vector4: ...
    @property
    def zwow(self) -> U8Vector4: ...
    @property
    def zwoo(self) -> U8Vector4: ...
    @property
    def zwol(self) -> U8Vector4: ...
    @property
    def zwlx(self) -> U8Vector4: ...
    @property
    def zwly(self) -> U8Vector4: ...
    @property
    def zwlz(self) -> U8Vector4: ...
    @property
    def zwlw(self) -> U8Vector4: ...
    @property
    def zwlo(self) -> U8Vector4: ...
    @property
    def zwll(self) -> U8Vector4: ...
    @property
    def zoxx(self) -> U8Vector4: ...
    @property
    def zoxy(self) -> U8Vector4: ...
    @property
    def zoxz(self) -> U8Vector4: ...
    @property
    def zoxw(self) -> U8Vector4: ...
    @property
    def zoxo(self) -> U8Vector4: ...
    @property
    def zoxl(self) -> U8Vector4: ...
    @property
    def zoyx(self) -> U8Vector4: ...
    @property
    def zoyy(self) -> U8Vector4: ...
    @property
    def zoyz(self) -> U8Vector4: ...
    @property
    def zoyw(self) -> U8Vector4: ...
    @property
    def zoyo(self) -> U8Vector4: ...
    @property
    def zoyl(self) -> U8Vector4: ...
    @property
    def zozx(self) -> U8Vector4: ...
    @property
    def zozy(self) -> U8Vector4: ...
    @property
    def zozz(self) -> U8Vector4: ...
    @property
    def zozw(self) -> U8Vector4: ...
    @property
    def zozo(self) -> U8Vector4: ...
    @property
    def zozl(self) -> U8Vector4: ...
    @property
    def zowx(self) -> U8Vector4: ...
    @property
    def zowy(self) -> U8Vector4: ...
    @property
    def zowz(self) -> U8Vector4: ...
    @property
    def zoww(self) -> U8Vector4: ...
    @property
    def zowo(self) -> U8Vector4: ...
    @property
    def zowl(self) -> U8Vector4: ...
    @property
    def zoox(self) -> U8Vector4: ...
    @property
    def zooy(self) -> U8Vector4: ...
    @property
    def zooz(self) -> U8Vector4: ...
    @property
    def zoow(self) -> U8Vector4: ...
    @property
    def zooo(self) -> U8Vector4: ...
    @property
    def zool(self) -> U8Vector4: ...
    @property
    def zolx(self) -> U8Vector4: ...
    @property
    def zoly(self) -> U8Vector4: ...
    @property
    def zolz(self) -> U8Vector4: ...
    @property
    def zolw(self) -> U8Vector4: ...
    @property
    def zolo(self) -> U8Vector4: ...
    @property
    def zoll(self) -> U8Vector4: ...
    @property
    def zlxx(self) -> U8Vector4: ...
    @property
    def zlxy(self) -> U8Vector4: ...
    @property
    def zlxz(self) -> U8Vector4: ...
    @property
    def zlxw(self) -> U8Vector4: ...
    @property
    def zlxo(self) -> U8Vector4: ...
    @property
    def zlxl(self) -> U8Vector4: ...
    @property
    def zlyx(self) -> U8Vector4: ...
    @property
    def zlyy(self) -> U8Vector4: ...
    @property
    def zlyz(self) -> U8Vector4: ...
    @property
    def zlyw(self) -> U8Vector4: ...
    @property
    def zlyo(self) -> U8Vector4: ...
    @property
    def zlyl(self) -> U8Vector4: ...
    @property
    def zlzx(self) -> U8Vector4: ...
    @property
    def zlzy(self) -> U8Vector4: ...
    @property
    def zlzz(self) -> U8Vector4: ...
    @property
    def zlzw(self) -> U8Vector4: ...
    @property
    def zlzo(self) -> U8Vector4: ...
    @property
    def zlzl(self) -> U8Vector4: ...
    @property
    def zlwx(self) -> U8Vector4: ...
    @property
    def zlwy(self) -> U8Vector4: ...
    @property
    def zlwz(self) -> U8Vector4: ...
    @property
    def zlww(self) -> U8Vector4: ...
    @property
    def zlwo(self) -> U8Vector4: ...
    @property
    def zlwl(self) -> U8Vector4: ...
    @property
    def zlox(self) -> U8Vector4: ...
    @property
    def zloy(self) -> U8Vector4: ...
    @property
    def zloz(self) -> U8Vector4: ...
    @property
    def zlow(self) -> U8Vector4: ...
    @property
    def zloo(self) -> U8Vector4: ...
    @property
    def zlol(self) -> U8Vector4: ...
    @property
    def zllx(self) -> U8Vector4: ...
    @property
    def zlly(self) -> U8Vector4: ...
    @property
    def zllz(self) -> U8Vector4: ...
    @property
    def zllw(self) -> U8Vector4: ...
    @property
    def zllo(self) -> U8Vector4: ...
    @property
    def zlll(self) -> U8Vector4: ...
    @property
    def wxxx(self) -> U8Vector4: ...
    @property
    def wxxy(self) -> U8Vector4: ...
    @property
    def wxxz(self) -> U8Vector4: ...
    @property
    def wxxw(self) -> U8Vector4: ...
    @property
    def wxxo(self) -> U8Vector4: ...
    @property
    def wxxl(self) -> U8Vector4: ...
    @property
    def wxyx(self) -> U8Vector4: ...
    @property
    def wxyy(self) -> U8Vector4: ...
    @property
    def wxyz(self) -> U8Vector4: ...
    @property
    def wxyw(self) -> U8Vector4: ...
    @property
    def wxyo(self) -> U8Vector4: ...
    @property
    def wxyl(self) -> U8Vector4: ...
    @property
    def wxzx(self) -> U8Vector4: ...
    @property
    def wxzy(self) -> U8Vector4: ...
    @property
    def wxzz(self) -> U8Vector4: ...
    @property
    def wxzw(self) -> U8Vector4: ...
    @property
    def wxzo(self) -> U8Vector4: ...
    @property
    def wxzl(self) -> U8Vector4: ...
    @property
    def wxwx(self) -> U8Vector4: ...
    @property
    def wxwy(self) -> U8Vector4: ...
    @property
    def wxwz(self) -> U8Vector4: ...
    @property
    def wxww(self) -> U8Vector4: ...
    @property
    def wxwo(self) -> U8Vector4: ...
    @property
    def wxwl(self) -> U8Vector4: ...
    @property
    def wxox(self) -> U8Vector4: ...
    @property
    def wxoy(self) -> U8Vector4: ...
    @property
    def wxoz(self) -> U8Vector4: ...
    @property
    def wxow(self) -> U8Vector4: ...
    @property
    def wxoo(self) -> U8Vector4: ...
    @property
    def wxol(self) -> U8Vector4: ...
    @property
    def wxlx(self) -> U8Vector4: ...
    @property
    def wxly(self) -> U8Vector4: ...
    @property
    def wxlz(self) -> U8Vector4: ...
    @property
    def wxlw(self) -> U8Vector4: ...
    @property
    def wxlo(self) -> U8Vector4: ...
    @property
    def wxll(self) -> U8Vector4: ...
    @property
    def wyxx(self) -> U8Vector4: ...
    @property
    def wyxy(self) -> U8Vector4: ...
    @property
    def wyxz(self) -> U8Vector4: ...
    @property
    def wyxw(self) -> U8Vector4: ...
    @property
    def wyxo(self) -> U8Vector4: ...
    @property
    def wyxl(self) -> U8Vector4: ...
    @property
    def wyyx(self) -> U8Vector4: ...
    @property
    def wyyy(self) -> U8Vector4: ...
    @property
    def wyyz(self) -> U8Vector4: ...
    @property
    def wyyw(self) -> U8Vector4: ...
    @property
    def wyyo(self) -> U8Vector4: ...
    @property
    def wyyl(self) -> U8Vector4: ...
    @property
    def wyzx(self) -> U8Vector4: ...
    @property
    def wyzy(self) -> U8Vector4: ...
    @property
    def wyzz(self) -> U8Vector4: ...
    @property
    def wyzw(self) -> U8Vector4: ...
    @property
    def wyzo(self) -> U8Vector4: ...
    @property
    def wyzl(self) -> U8Vector4: ...
    @property
    def wywx(self) -> U8Vector4: ...
    @property
    def wywy(self) -> U8Vector4: ...
    @property
    def wywz(self) -> U8Vector4: ...
    @property
    def wyww(self) -> U8Vector4: ...
    @property
    def wywo(self) -> U8Vector4: ...
    @property
    def wywl(self) -> U8Vector4: ...
    @property
    def wyox(self) -> U8Vector4: ...
    @property
    def wyoy(self) -> U8Vector4: ...
    @property
    def wyoz(self) -> U8Vector4: ...
    @property
    def wyow(self) -> U8Vector4: ...
    @property
    def wyoo(self) -> U8Vector4: ...
    @property
    def wyol(self) -> U8Vector4: ...
    @property
    def wylx(self) -> U8Vector4: ...
    @property
    def wyly(self) -> U8Vector4: ...
    @property
    def wylz(self) -> U8Vector4: ...
    @property
    def wylw(self) -> U8Vector4: ...
    @property
    def wylo(self) -> U8Vector4: ...
    @property
    def wyll(self) -> U8Vector4: ...
    @property
    def wzxx(self) -> U8Vector4: ...
    @property
    def wzxy(self) -> U8Vector4: ...
    @property
    def wzxz(self) -> U8Vector4: ...
    @property
    def wzxw(self) -> U8Vector4: ...
    @property
    def wzxo(self) -> U8Vector4: ...
    @property
    def wzxl(self) -> U8Vector4: ...
    @property
    def wzyx(self) -> U8Vector4: ...
    @property
    def wzyy(self) -> U8Vector4: ...
    @property
    def wzyz(self) -> U8Vector4: ...
    @property
    def wzyw(self) -> U8Vector4: ...
    @property
    def wzyo(self) -> U8Vector4: ...
    @property
    def wzyl(self) -> U8Vector4: ...
    @property
    def wzzx(self) -> U8Vector4: ...
    @property
    def wzzy(self) -> U8Vector4: ...
    @property
    def wzzz(self) -> U8Vector4: ...
    @property
    def wzzw(self) -> U8Vector4: ...
    @property
    def wzzo(self) -> U8Vector4: ...
    @property
    def wzzl(self) -> U8Vector4: ...
    @property
    def wzwx(self) -> U8Vector4: ...
    @property
    def wzwy(self) -> U8Vector4: ...
    @property
    def wzwz(self) -> U8Vector4: ...
    @property
    def wzww(self) -> U8Vector4: ...
    @property
    def wzwo(self) -> U8Vector4: ...
    @property
    def wzwl(self) -> U8Vector4: ...
    @property
    def wzox(self) -> U8Vector4: ...
    @property
    def wzoy(self) -> U8Vector4: ...
    @property
    def wzoz(self) -> U8Vector4: ...
    @property
    def wzow(self) -> U8Vector4: ...
    @property
    def wzoo(self) -> U8Vector4: ...
    @property
    def wzol(self) -> U8Vector4: ...
    @property
    def wzlx(self) -> U8Vector4: ...
    @property
    def wzly(self) -> U8Vector4: ...
    @property
    def wzlz(self) -> U8Vector4: ...
    @property
    def wzlw(self) -> U8Vector4: ...
    @property
    def wzlo(self) -> U8Vector4: ...
    @property
    def wzll(self) -> U8Vector4: ...
    @property
    def wwxx(self) -> U8Vector4: ...
    @property
    def wwxy(self) -> U8Vector4: ...
    @property
    def wwxz(self) -> U8Vector4: ...
    @property
    def wwxw(self) -> U8Vector4: ...
    @property
    def wwxo(self) -> U8Vector4: ...
    @property
    def wwxl(self) -> U8Vector4: ...
    @property
    def wwyx(self) -> U8Vector4: ...
    @property
    def wwyy(self) -> U8Vector4: ...
    @property
    def wwyz(self) -> U8Vector4: ...
    @property
    def wwyw(self) -> U8Vector4: ...
    @property
    def wwyo(self) -> U8Vector4: ...
    @property
    def wwyl(self) -> U8Vector4: ...
    @property
    def wwzx(self) -> U8Vector4: ...
    @property
    def wwzy(self) -> U8Vector4: ...
    @property
    def wwzz(self) -> U8Vector4: ...
    @property
    def wwzw(self) -> U8Vector4: ...
    @property
    def wwzo(self) -> U8Vector4: ...
    @property
    def wwzl(self) -> U8Vector4: ...
    @property
    def wwwx(self) -> U8Vector4: ...
    @property
    def wwwy(self) -> U8Vector4: ...
    @property
    def wwwz(self) -> U8Vector4: ...
    @property
    def wwww(self) -> U8Vector4: ...
    @property
    def wwwo(self) -> U8Vector4: ...
    @property
    def wwwl(self) -> U8Vector4: ...
    @property
    def wwox(self) -> U8Vector4: ...
    @property
    def wwoy(self) -> U8Vector4: ...
    @property
    def wwoz(self) -> U8Vector4: ...
    @property
    def wwow(self) -> U8Vector4: ...
    @property
    def wwoo(self) -> U8Vector4: ...
    @property
    def wwol(self) -> U8Vector4: ...
    @property
    def wwlx(self) -> U8Vector4: ...
    @property
    def wwly(self) -> U8Vector4: ...
    @property
    def wwlz(self) -> U8Vector4: ...
    @property
    def wwlw(self) -> U8Vector4: ...
    @property
    def wwlo(self) -> U8Vector4: ...
    @property
    def wwll(self) -> U8Vector4: ...
    @property
    def woxx(self) -> U8Vector4: ...
    @property
    def woxy(self) -> U8Vector4: ...
    @property
    def woxz(self) -> U8Vector4: ...
    @property
    def woxw(self) -> U8Vector4: ...
    @property
    def woxo(self) -> U8Vector4: ...
    @property
    def woxl(self) -> U8Vector4: ...
    @property
    def woyx(self) -> U8Vector4: ...
    @property
    def woyy(self) -> U8Vector4: ...
    @property
    def woyz(self) -> U8Vector4: ...
    @property
    def woyw(self) -> U8Vector4: ...
    @property
    def woyo(self) -> U8Vector4: ...
    @property
    def woyl(self) -> U8Vector4: ...
    @property
    def wozx(self) -> U8Vector4: ...
    @property
    def wozy(self) -> U8Vector4: ...
    @property
    def wozz(self) -> U8Vector4: ...
    @property
    def wozw(self) -> U8Vector4: ...
    @property
    def wozo(self) -> U8Vector4: ...
    @property
    def wozl(self) -> U8Vector4: ...
    @property
    def wowx(self) -> U8Vector4: ...
    @property
    def wowy(self) -> U8Vector4: ...
    @property
    def wowz(self) -> U8Vector4: ...
    @property
    def woww(self) -> U8Vector4: ...
    @property
    def wowo(self) -> U8Vector4: ...
    @property
    def wowl(self) -> U8Vector4: ...
    @property
    def woox(self) -> U8Vector4: ...
    @property
    def wooy(self) -> U8Vector4: ...
    @property
    def wooz(self) -> U8Vector4: ...
    @property
    def woow(self) -> U8Vector4: ...
    @property
    def wooo(self) -> U8Vector4: ...
    @property
    def wool(self) -> U8Vector4: ...
    @property
    def wolx(self) -> U8Vector4: ...
    @property
    def woly(self) -> U8Vector4: ...
    @property
    def wolz(self) -> U8Vector4: ...
    @property
    def wolw(self) -> U8Vector4: ...
    @property
    def wolo(self) -> U8Vector4: ...
    @property
    def woll(self) -> U8Vector4: ...
    @property
    def wlxx(self) -> U8Vector4: ...
    @property
    def wlxy(self) -> U8Vector4: ...
    @property
    def wlxz(self) -> U8Vector4: ...
    @property
    def wlxw(self) -> U8Vector4: ...
    @property
    def wlxo(self) -> U8Vector4: ...
    @property
    def wlxl(self) -> U8Vector4: ...
    @property
    def wlyx(self) -> U8Vector4: ...
    @property
    def wlyy(self) -> U8Vector4: ...
    @property
    def wlyz(self) -> U8Vector4: ...
    @property
    def wlyw(self) -> U8Vector4: ...
    @property
    def wlyo(self) -> U8Vector4: ...
    @property
    def wlyl(self) -> U8Vector4: ...
    @property
    def wlzx(self) -> U8Vector4: ...
    @property
    def wlzy(self) -> U8Vector4: ...
    @property
    def wlzz(self) -> U8Vector4: ...
    @property
    def wlzw(self) -> U8Vector4: ...
    @property
    def wlzo(self) -> U8Vector4: ...
    @property
    def wlzl(self) -> U8Vector4: ...
    @property
    def wlwx(self) -> U8Vector4: ...
    @property
    def wlwy(self) -> U8Vector4: ...
    @property
    def wlwz(self) -> U8Vector4: ...
    @property
    def wlww(self) -> U8Vector4: ...
    @property
    def wlwo(self) -> U8Vector4: ...
    @property
    def wlwl(self) -> U8Vector4: ...
    @property
    def wlox(self) -> U8Vector4: ...
    @property
    def wloy(self) -> U8Vector4: ...
    @property
    def wloz(self) -> U8Vector4: ...
    @property
    def wlow(self) -> U8Vector4: ...
    @property
    def wloo(self) -> U8Vector4: ...
    @property
    def wlol(self) -> U8Vector4: ...
    @property
    def wllx(self) -> U8Vector4: ...
    @property
    def wlly(self) -> U8Vector4: ...
    @property
    def wllz(self) -> U8Vector4: ...
    @property
    def wllw(self) -> U8Vector4: ...
    @property
    def wllo(self) -> U8Vector4: ...
    @property
    def wlll(self) -> U8Vector4: ...
    @property
    def oxxx(self) -> U8Vector4: ...
    @property
    def oxxy(self) -> U8Vector4: ...
    @property
    def oxxz(self) -> U8Vector4: ...
    @property
    def oxxw(self) -> U8Vector4: ...
    @property
    def oxxo(self) -> U8Vector4: ...
    @property
    def oxxl(self) -> U8Vector4: ...
    @property
    def oxyx(self) -> U8Vector4: ...
    @property
    def oxyy(self) -> U8Vector4: ...
    @property
    def oxyz(self) -> U8Vector4: ...
    @property
    def oxyw(self) -> U8Vector4: ...
    @property
    def oxyo(self) -> U8Vector4: ...
    @property
    def oxyl(self) -> U8Vector4: ...
    @property
    def oxzx(self) -> U8Vector4: ...
    @property
    def oxzy(self) -> U8Vector4: ...
    @property
    def oxzz(self) -> U8Vector4: ...
    @property
    def oxzw(self) -> U8Vector4: ...
    @property
    def oxzo(self) -> U8Vector4: ...
    @property
    def oxzl(self) -> U8Vector4: ...
    @property
    def oxwx(self) -> U8Vector4: ...
    @property
    def oxwy(self) -> U8Vector4: ...
    @property
    def oxwz(self) -> U8Vector4: ...
    @property
    def oxww(self) -> U8Vector4: ...
    @property
    def oxwo(self) -> U8Vector4: ...
    @property
    def oxwl(self) -> U8Vector4: ...
    @property
    def oxox(self) -> U8Vector4: ...
    @property
    def oxoy(self) -> U8Vector4: ...
    @property
    def oxoz(self) -> U8Vector4: ...
    @property
    def oxow(self) -> U8Vector4: ...
    @property
    def oxoo(self) -> U8Vector4: ...
    @property
    def oxol(self) -> U8Vector4: ...
    @property
    def oxlx(self) -> U8Vector4: ...
    @property
    def oxly(self) -> U8Vector4: ...
    @property
    def oxlz(self) -> U8Vector4: ...
    @property
    def oxlw(self) -> U8Vector4: ...
    @property
    def oxlo(self) -> U8Vector4: ...
    @property
    def oxll(self) -> U8Vector4: ...
    @property
    def oyxx(self) -> U8Vector4: ...
    @property
    def oyxy(self) -> U8Vector4: ...
    @property
    def oyxz(self) -> U8Vector4: ...
    @property
    def oyxw(self) -> U8Vector4: ...
    @property
    def oyxo(self) -> U8Vector4: ...
    @property
    def oyxl(self) -> U8Vector4: ...
    @property
    def oyyx(self) -> U8Vector4: ...
    @property
    def oyyy(self) -> U8Vector4: ...
    @property
    def oyyz(self) -> U8Vector4: ...
    @property
    def oyyw(self) -> U8Vector4: ...
    @property
    def oyyo(self) -> U8Vector4: ...
    @property
    def oyyl(self) -> U8Vector4: ...
    @property
    def oyzx(self) -> U8Vector4: ...
    @property
    def oyzy(self) -> U8Vector4: ...
    @property
    def oyzz(self) -> U8Vector4: ...
    @property
    def oyzw(self) -> U8Vector4: ...
    @property
    def oyzo(self) -> U8Vector4: ...
    @property
    def oyzl(self) -> U8Vector4: ...
    @property
    def oywx(self) -> U8Vector4: ...
    @property
    def oywy(self) -> U8Vector4: ...
    @property
    def oywz(self) -> U8Vector4: ...
    @property
    def oyww(self) -> U8Vector4: ...
    @property
    def oywo(self) -> U8Vector4: ...
    @property
    def oywl(self) -> U8Vector4: ...
    @property
    def oyox(self) -> U8Vector4: ...
    @property
    def oyoy(self) -> U8Vector4: ...
    @property
    def oyoz(self) -> U8Vector4: ...
    @property
    def oyow(self) -> U8Vector4: ...
    @property
    def oyoo(self) -> U8Vector4: ...
    @property
    def oyol(self) -> U8Vector4: ...
    @property
    def oylx(self) -> U8Vector4: ...
    @property
    def oyly(self) -> U8Vector4: ...
    @property
    def oylz(self) -> U8Vector4: ...
    @property
    def oylw(self) -> U8Vector4: ...
    @property
    def oylo(self) -> U8Vector4: ...
    @property
    def oyll(self) -> U8Vector4: ...
    @property
    def ozxx(self) -> U8Vector4: ...
    @property
    def ozxy(self) -> U8Vector4: ...
    @property
    def ozxz(self) -> U8Vector4: ...
    @property
    def ozxw(self) -> U8Vector4: ...
    @property
    def ozxo(self) -> U8Vector4: ...
    @property
    def ozxl(self) -> U8Vector4: ...
    @property
    def ozyx(self) -> U8Vector4: ...
    @property
    def ozyy(self) -> U8Vector4: ...
    @property
    def ozyz(self) -> U8Vector4: ...
    @property
    def ozyw(self) -> U8Vector4: ...
    @property
    def ozyo(self) -> U8Vector4: ...
    @property
    def ozyl(self) -> U8Vector4: ...
    @property
    def ozzx(self) -> U8Vector4: ...
    @property
    def ozzy(self) -> U8Vector4: ...
    @property
    def ozzz(self) -> U8Vector4: ...
    @property
    def ozzw(self) -> U8Vector4: ...
    @property
    def ozzo(self) -> U8Vector4: ...
    @property
    def ozzl(self) -> U8Vector4: ...
    @property
    def ozwx(self) -> U8Vector4: ...
    @property
    def ozwy(self) -> U8Vector4: ...
    @property
    def ozwz(self) -> U8Vector4: ...
    @property
    def ozww(self) -> U8Vector4: ...
    @property
    def ozwo(self) -> U8Vector4: ...
    @property
    def ozwl(self) -> U8Vector4: ...
    @property
    def ozox(self) -> U8Vector4: ...
    @property
    def ozoy(self) -> U8Vector4: ...
    @property
    def ozoz(self) -> U8Vector4: ...
    @property
    def ozow(self) -> U8Vector4: ...
    @property
    def ozoo(self) -> U8Vector4: ...
    @property
    def ozol(self) -> U8Vector4: ...
    @property
    def ozlx(self) -> U8Vector4: ...
    @property
    def ozly(self) -> U8Vector4: ...
    @property
    def ozlz(self) -> U8Vector4: ...
    @property
    def ozlw(self) -> U8Vector4: ...
    @property
    def ozlo(self) -> U8Vector4: ...
    @property
    def ozll(self) -> U8Vector4: ...
    @property
    def owxx(self) -> U8Vector4: ...
    @property
    def owxy(self) -> U8Vector4: ...
    @property
    def owxz(self) -> U8Vector4: ...
    @property
    def owxw(self) -> U8Vector4: ...
    @property
    def owxo(self) -> U8Vector4: ...
    @property
    def owxl(self) -> U8Vector4: ...
    @property
    def owyx(self) -> U8Vector4: ...
    @property
    def owyy(self) -> U8Vector4: ...
    @property
    def owyz(self) -> U8Vector4: ...
    @property
    def owyw(self) -> U8Vector4: ...
    @property
    def owyo(self) -> U8Vector4: ...
    @property
    def owyl(self) -> U8Vector4: ...
    @property
    def owzx(self) -> U8Vector4: ...
    @property
    def owzy(self) -> U8Vector4: ...
    @property
    def owzz(self) -> U8Vector4: ...
    @property
    def owzw(self) -> U8Vector4: ...
    @property
    def owzo(self) -> U8Vector4: ...
    @property
    def owzl(self) -> U8Vector4: ...
    @property
    def owwx(self) -> U8Vector4: ...
    @property
    def owwy(self) -> U8Vector4: ...
    @property
    def owwz(self) -> U8Vector4: ...
    @property
    def owww(self) -> U8Vector4: ...
    @property
    def owwo(self) -> U8Vector4: ...
    @property
    def owwl(self) -> U8Vector4: ...
    @property
    def owox(self) -> U8Vector4: ...
    @property
    def owoy(self) -> U8Vector4: ...
    @property
    def owoz(self) -> U8Vector4: ...
    @property
    def owow(self) -> U8Vector4: ...
    @property
    def owoo(self) -> U8Vector4: ...
    @property
    def owol(self) -> U8Vector4: ...
    @property
    def owlx(self) -> U8Vector4: ...
    @property
    def owly(self) -> U8Vector4: ...
    @property
    def owlz(self) -> U8Vector4: ...
    @property
    def owlw(self) -> U8Vector4: ...
    @property
    def owlo(self) -> U8Vector4: ...
    @property
    def owll(self) -> U8Vector4: ...
    @property
    def ooxx(self) -> U8Vector4: ...
    @property
    def ooxy(self) -> U8Vector4: ...
    @property
    def ooxz(self) -> U8Vector4: ...
    @property
    def ooxw(self) -> U8Vector4: ...
    @property
    def ooxo(self) -> U8Vector4: ...
    @property
    def ooxl(self) -> U8Vector4: ...
    @property
    def ooyx(self) -> U8Vector4: ...
    @property
    def ooyy(self) -> U8Vector4: ...
    @property
    def ooyz(self) -> U8Vector4: ...
    @property
    def ooyw(self) -> U8Vector4: ...
    @property
    def ooyo(self) -> U8Vector4: ...
    @property
    def ooyl(self) -> U8Vector4: ...
    @property
    def oozx(self) -> U8Vector4: ...
    @property
    def oozy(self) -> U8Vector4: ...
    @property
    def oozz(self) -> U8Vector4: ...
    @property
    def oozw(self) -> U8Vector4: ...
    @property
    def oozo(self) -> U8Vector4: ...
    @property
    def oozl(self) -> U8Vector4: ...
    @property
    def oowx(self) -> U8Vector4: ...
    @property
    def oowy(self) -> U8Vector4: ...
    @property
    def oowz(self) -> U8Vector4: ...
    @property
    def ooww(self) -> U8Vector4: ...
    @property
    def oowo(self) -> U8Vector4: ...
    @property
    def oowl(self) -> U8Vector4: ...
    @property
    def ooox(self) -> U8Vector4: ...
    @property
    def oooy(self) -> U8Vector4: ...
    @property
    def oooz(self) -> U8Vector4: ...
    @property
    def ooow(self) -> U8Vector4: ...
    @property
    def oolx(self) -> U8Vector4: ...
    @property
    def ooly(self) -> U8Vector4: ...
    @property
    def oolz(self) -> U8Vector4: ...
    @property
    def oolw(self) -> U8Vector4: ...
    @property
    def olxx(self) -> U8Vector4: ...
    @property
    def olxy(self) -> U8Vector4: ...
    @property
    def olxz(self) -> U8Vector4: ...
    @property
    def olxw(self) -> U8Vector4: ...
    @property
    def olxo(self) -> U8Vector4: ...
    @property
    def olxl(self) -> U8Vector4: ...
    @property
    def olyx(self) -> U8Vector4: ...
    @property
    def olyy(self) -> U8Vector4: ...
    @property
    def olyz(self) -> U8Vector4: ...
    @property
    def olyw(self) -> U8Vector4: ...
    @property
    def olyo(self) -> U8Vector4: ...
    @property
    def olyl(self) -> U8Vector4: ...
    @property
    def olzx(self) -> U8Vector4: ...
    @property
    def olzy(self) -> U8Vector4: ...
    @property
    def olzz(self) -> U8Vector4: ...
    @property
    def olzw(self) -> U8Vector4: ...
    @property
    def olzo(self) -> U8Vector4: ...
    @property
    def olzl(self) -> U8Vector4: ...
    @property
    def olwx(self) -> U8Vector4: ...
    @property
    def olwy(self) -> U8Vector4: ...
    @property
    def olwz(self) -> U8Vector4: ...
    @property
    def olww(self) -> U8Vector4: ...
    @property
    def olwo(self) -> U8Vector4: ...
    @property
    def olwl(self) -> U8Vector4: ...
    @property
    def olox(self) -> U8Vector4: ...
    @property
    def oloy(self) -> U8Vector4: ...
    @property
    def oloz(self) -> U8Vector4: ...
    @property
    def olow(self) -> U8Vector4: ...
    @property
    def ollx(self) -> U8Vector4: ...
    @property
    def olly(self) -> U8Vector4: ...
    @property
    def ollz(self) -> U8Vector4: ...
    @property
    def ollw(self) -> U8Vector4: ...
    @property
    def lxxx(self) -> U8Vector4: ...
    @property
    def lxxy(self) -> U8Vector4: ...
    @property
    def lxxz(self) -> U8Vector4: ...
    @property
    def lxxw(self) -> U8Vector4: ...
    @property
    def lxxo(self) -> U8Vector4: ...
    @property
    def lxxl(self) -> U8Vector4: ...
    @property
    def lxyx(self) -> U8Vector4: ...
    @property
    def lxyy(self) -> U8Vector4: ...
    @property
    def lxyz(self) -> U8Vector4: ...
    @property
    def lxyw(self) -> U8Vector4: ...
    @property
    def lxyo(self) -> U8Vector4: ...
    @property
    def lxyl(self) -> U8Vector4: ...
    @property
    def lxzx(self) -> U8Vector4: ...
    @property
    def lxzy(self) -> U8Vector4: ...
    @property
    def lxzz(self) -> U8Vector4: ...
    @property
    def lxzw(self) -> U8Vector4: ...
    @property
    def lxzo(self) -> U8Vector4: ...
    @property
    def lxzl(self) -> U8Vector4: ...
    @property
    def lxwx(self) -> U8Vector4: ...
    @property
    def lxwy(self) -> U8Vector4: ...
    @property
    def lxwz(self) -> U8Vector4: ...
    @property
    def lxww(self) -> U8Vector4: ...
    @property
    def lxwo(self) -> U8Vector4: ...
    @property
    def lxwl(self) -> U8Vector4: ...
    @property
    def lxox(self) -> U8Vector4: ...
    @property
    def lxoy(self) -> U8Vector4: ...
    @property
    def lxoz(self) -> U8Vector4: ...
    @property
    def lxow(self) -> U8Vector4: ...
    @property
    def lxoo(self) -> U8Vector4: ...
    @property
    def lxol(self) -> U8Vector4: ...
    @property
    def lxlx(self) -> U8Vector4: ...
    @property
    def lxly(self) -> U8Vector4: ...
    @property
    def lxlz(self) -> U8Vector4: ...
    @property
    def lxlw(self) -> U8Vector4: ...
    @property
    def lxlo(self) -> U8Vector4: ...
    @property
    def lxll(self) -> U8Vector4: ...
    @property
    def lyxx(self) -> U8Vector4: ...
    @property
    def lyxy(self) -> U8Vector4: ...
    @property
    def lyxz(self) -> U8Vector4: ...
    @property
    def lyxw(self) -> U8Vector4: ...
    @property
    def lyxo(self) -> U8Vector4: ...
    @property
    def lyxl(self) -> U8Vector4: ...
    @property
    def lyyx(self) -> U8Vector4: ...
    @property
    def lyyy(self) -> U8Vector4: ...
    @property
    def lyyz(self) -> U8Vector4: ...
    @property
    def lyyw(self) -> U8Vector4: ...
    @property
    def lyyo(self) -> U8Vector4: ...
    @property
    def lyyl(self) -> U8Vector4: ...
    @property
    def lyzx(self) -> U8Vector4: ...
    @property
    def lyzy(self) -> U8Vector4: ...
    @property
    def lyzz(self) -> U8Vector4: ...
    @property
    def lyzw(self) -> U8Vector4: ...
    @property
    def lyzo(self) -> U8Vector4: ...
    @property
    def lyzl(self) -> U8Vector4: ...
    @property
    def lywx(self) -> U8Vector4: ...
    @property
    def lywy(self) -> U8Vector4: ...
    @property
    def lywz(self) -> U8Vector4: ...
    @property
    def lyww(self) -> U8Vector4: ...
    @property
    def lywo(self) -> U8Vector4: ...
    @property
    def lywl(self) -> U8Vector4: ...
    @property
    def lyox(self) -> U8Vector4: ...
    @property
    def lyoy(self) -> U8Vector4: ...
    @property
    def lyoz(self) -> U8Vector4: ...
    @property
    def lyow(self) -> U8Vector4: ...
    @property
    def lyoo(self) -> U8Vector4: ...
    @property
    def lyol(self) -> U8Vector4: ...
    @property
    def lylx(self) -> U8Vector4: ...
    @property
    def lyly(self) -> U8Vector4: ...
    @property
    def lylz(self) -> U8Vector4: ...
    @property
    def lylw(self) -> U8Vector4: ...
    @property
    def lylo(self) -> U8Vector4: ...
    @property
    def lyll(self) -> U8Vector4: ...
    @property
    def lzxx(self) -> U8Vector4: ...
    @property
    def lzxy(self) -> U8Vector4: ...
    @property
    def lzxz(self) -> U8Vector4: ...
    @property
    def lzxw(self) -> U8Vector4: ...
    @property
    def lzxo(self) -> U8Vector4: ...
    @property
    def lzxl(self) -> U8Vector4: ...
    @property
    def lzyx(self) -> U8Vector4: ...
    @property
    def lzyy(self) -> U8Vector4: ...
    @property
    def lzyz(self) -> U8Vector4: ...
    @property
    def lzyw(self) -> U8Vector4: ...
    @property
    def lzyo(self) -> U8Vector4: ...
    @property
    def lzyl(self) -> U8Vector4: ...
    @property
    def lzzx(self) -> U8Vector4: ...
    @property
    def lzzy(self) -> U8Vector4: ...
    @property
    def lzzz(self) -> U8Vector4: ...
    @property
    def lzzw(self) -> U8Vector4: ...
    @property
    def lzzo(self) -> U8Vector4: ...
    @property
    def lzzl(self) -> U8Vector4: ...
    @property
    def lzwx(self) -> U8Vector4: ...
    @property
    def lzwy(self) -> U8Vector4: ...
    @property
    def lzwz(self) -> U8Vector4: ...
    @property
    def lzww(self) -> U8Vector4: ...
    @property
    def lzwo(self) -> U8Vector4: ...
    @property
    def lzwl(self) -> U8Vector4: ...
    @property
    def lzox(self) -> U8Vector4: ...
    @property
    def lzoy(self) -> U8Vector4: ...
    @property
    def lzoz(self) -> U8Vector4: ...
    @property
    def lzow(self) -> U8Vector4: ...
    @property
    def lzoo(self) -> U8Vector4: ...
    @property
    def lzol(self) -> U8Vector4: ...
    @property
    def lzlx(self) -> U8Vector4: ...
    @property
    def lzly(self) -> U8Vector4: ...
    @property
    def lzlz(self) -> U8Vector4: ...
    @property
    def lzlw(self) -> U8Vector4: ...
    @property
    def lzlo(self) -> U8Vector4: ...
    @property
    def lzll(self) -> U8Vector4: ...
    @property
    def lwxx(self) -> U8Vector4: ...
    @property
    def lwxy(self) -> U8Vector4: ...
    @property
    def lwxz(self) -> U8Vector4: ...
    @property
    def lwxw(self) -> U8Vector4: ...
    @property
    def lwxo(self) -> U8Vector4: ...
    @property
    def lwxl(self) -> U8Vector4: ...
    @property
    def lwyx(self) -> U8Vector4: ...
    @property
    def lwyy(self) -> U8Vector4: ...
    @property
    def lwyz(self) -> U8Vector4: ...
    @property
    def lwyw(self) -> U8Vector4: ...
    @property
    def lwyo(self) -> U8Vector4: ...
    @property
    def lwyl(self) -> U8Vector4: ...
    @property
    def lwzx(self) -> U8Vector4: ...
    @property
    def lwzy(self) -> U8Vector4: ...
    @property
    def lwzz(self) -> U8Vector4: ...
    @property
    def lwzw(self) -> U8Vector4: ...
    @property
    def lwzo(self) -> U8Vector4: ...
    @property
    def lwzl(self) -> U8Vector4: ...
    @property
    def lwwx(self) -> U8Vector4: ...
    @property
    def lwwy(self) -> U8Vector4: ...
    @property
    def lwwz(self) -> U8Vector4: ...
    @property
    def lwww(self) -> U8Vector4: ...
    @property
    def lwwo(self) -> U8Vector4: ...
    @property
    def lwwl(self) -> U8Vector4: ...
    @property
    def lwox(self) -> U8Vector4: ...
    @property
    def lwoy(self) -> U8Vector4: ...
    @property
    def lwoz(self) -> U8Vector4: ...
    @property
    def lwow(self) -> U8Vector4: ...
    @property
    def lwoo(self) -> U8Vector4: ...
    @property
    def lwol(self) -> U8Vector4: ...
    @property
    def lwlx(self) -> U8Vector4: ...
    @property
    def lwly(self) -> U8Vector4: ...
    @property
    def lwlz(self) -> U8Vector4: ...
    @property
    def lwlw(self) -> U8Vector4: ...
    @property
    def lwlo(self) -> U8Vector4: ...
    @property
    def lwll(self) -> U8Vector4: ...
    @property
    def loxx(self) -> U8Vector4: ...
    @property
    def loxy(self) -> U8Vector4: ...
    @property
    def loxz(self) -> U8Vector4: ...
    @property
    def loxw(self) -> U8Vector4: ...
    @property
    def loxo(self) -> U8Vector4: ...
    @property
    def loxl(self) -> U8Vector4: ...
    @property
    def loyx(self) -> U8Vector4: ...
    @property
    def loyy(self) -> U8Vector4: ...
    @property
    def loyz(self) -> U8Vector4: ...
    @property
    def loyw(self) -> U8Vector4: ...
    @property
    def loyo(self) -> U8Vector4: ...
    @property
    def loyl(self) -> U8Vector4: ...
    @property
    def lozx(self) -> U8Vector4: ...
    @property
    def lozy(self) -> U8Vector4: ...
    @property
    def lozz(self) -> U8Vector4: ...
    @property
    def lozw(self) -> U8Vector4: ...
    @property
    def lozo(self) -> U8Vector4: ...
    @property
    def lozl(self) -> U8Vector4: ...
    @property
    def lowx(self) -> U8Vector4: ...
    @property
    def lowy(self) -> U8Vector4: ...
    @property
    def lowz(self) -> U8Vector4: ...
    @property
    def loww(self) -> U8Vector4: ...
    @property
    def lowo(self) -> U8Vector4: ...
    @property
    def lowl(self) -> U8Vector4: ...
    @property
    def loox(self) -> U8Vector4: ...
    @property
    def looy(self) -> U8Vector4: ...
    @property
    def looz(self) -> U8Vector4: ...
    @property
    def loow(self) -> U8Vector4: ...
    @property
    def lolx(self) -> U8Vector4: ...
    @property
    def loly(self) -> U8Vector4: ...
    @property
    def lolz(self) -> U8Vector4: ...
    @property
    def lolw(self) -> U8Vector4: ...
    @property
    def llxx(self) -> U8Vector4: ...
    @property
    def llxy(self) -> U8Vector4: ...
    @property
    def llxz(self) -> U8Vector4: ...
    @property
    def llxw(self) -> U8Vector4: ...
    @property
    def llxo(self) -> U8Vector4: ...
    @property
    def llxl(self) -> U8Vector4: ...
    @property
    def llyx(self) -> U8Vector4: ...
    @property
    def llyy(self) -> U8Vector4: ...
    @property
    def llyz(self) -> U8Vector4: ...
    @property
    def llyw(self) -> U8Vector4: ...
    @property
    def llyo(self) -> U8Vector4: ...
    @property
    def llyl(self) -> U8Vector4: ...
    @property
    def llzx(self) -> U8Vector4: ...
    @property
    def llzy(self) -> U8Vector4: ...
    @property
    def llzz(self) -> U8Vector4: ...
    @property
    def llzw(self) -> U8Vector4: ...
    @property
    def llzo(self) -> U8Vector4: ...
    @property
    def llzl(self) -> U8Vector4: ...
    @property
    def llwx(self) -> U8Vector4: ...
    @property
    def llwy(self) -> U8Vector4: ...
    @property
    def llwz(self) -> U8Vector4: ...
    @property
    def llww(self) -> U8Vector4: ...
    @property
    def llwo(self) -> U8Vector4: ...
    @property
    def llwl(self) -> U8Vector4: ...
    @property
    def llox(self) -> U8Vector4: ...
    @property
    def lloy(self) -> U8Vector4: ...
    @property
    def lloz(self) -> U8Vector4: ...
    @property
    def llow(self) -> U8Vector4: ...
    @property
    def lllx(self) -> U8Vector4: ...
    @property
    def llly(self) -> U8Vector4: ...
    @property
    def lllz(self) -> U8Vector4: ...
    @property
    def lllw(self) -> U8Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> U8Vector2: ...
    @property
    def rg(self) -> U8Vector2: ...
    @property
    def rb(self) -> U8Vector2: ...
    @property
    def ra(self) -> U8Vector2: ...
    @property
    def ro(self) -> U8Vector2: ...
    @property
    def rl(self) -> U8Vector2: ...
    @property
    def gr(self) -> U8Vector2: ...
    @property
    def gg(self) -> U8Vector2: ...
    @property
    def gb(self) -> U8Vector2: ...
    @property
    def ga(self) -> U8Vector2: ...
    @property
    def go(self) -> U8Vector2: ...
    @property
    def gl(self) -> U8Vector2: ...
    @property
    def br(self) -> U8Vector2: ...
    @property
    def bg(self) -> U8Vector2: ...
    @property
    def bb(self) -> U8Vector2: ...
    @property
    def ba(self) -> U8Vector2: ...
    @property
    def bo(self) -> U8Vector2: ...
    @property
    def bl(self) -> U8Vector2: ...
    @property
    def ar(self) -> U8Vector2: ...
    @property
    def ag(self) -> U8Vector2: ...
    @property
    def ab(self) -> U8Vector2: ...
    @property
    def aa(self) -> U8Vector2: ...
    @property
    def ao(self) -> U8Vector2: ...
    @property
    def al(self) -> U8Vector2: ...
    @property
    def og(self) -> U8Vector2: ...
    @property
    def ob(self) -> U8Vector2: ...
    @property
    def oa(self) -> U8Vector2: ...
    @property
    def lr(self) -> U8Vector2: ...
    @property
    def lg(self) -> U8Vector2: ...
    @property
    def lb(self) -> U8Vector2: ...
    @property
    def la(self) -> U8Vector2: ...
    @property
    def rrr(self) -> U8Vector3: ...
    @property
    def rrg(self) -> U8Vector3: ...
    @property
    def rrb(self) -> U8Vector3: ...
    @property
    def rra(self) -> U8Vector3: ...
    @property
    def rro(self) -> U8Vector3: ...
    @property
    def rrl(self) -> U8Vector3: ...
    @property
    def rgr(self) -> U8Vector3: ...
    @property
    def rgg(self) -> U8Vector3: ...
    @property
    def rgb(self) -> U8Vector3: ...
    @property
    def rga(self) -> U8Vector3: ...
    @property
    def rgo(self) -> U8Vector3: ...
    @property
    def rgl(self) -> U8Vector3: ...
    @property
    def rbr(self) -> U8Vector3: ...
    @property
    def rbg(self) -> U8Vector3: ...
    @property
    def rbb(self) -> U8Vector3: ...
    @property
    def rba(self) -> U8Vector3: ...
    @property
    def rbo(self) -> U8Vector3: ...
    @property
    def rbl(self) -> U8Vector3: ...
    @property
    def rar(self) -> U8Vector3: ...
    @property
    def rag(self) -> U8Vector3: ...
    @property
    def rab(self) -> U8Vector3: ...
    @property
    def raa(self) -> U8Vector3: ...
    @property
    def rao(self) -> U8Vector3: ...
    @property
    def ral(self) -> U8Vector3: ...
    @property
    def ror(self) -> U8Vector3: ...
    @property
    def rog(self) -> U8Vector3: ...
    @property
    def rob(self) -> U8Vector3: ...
    @property
    def roa(self) -> U8Vector3: ...
    @property
    def roo(self) -> U8Vector3: ...
    @property
    def rol(self) -> U8Vector3: ...
    @property
    def rlr(self) -> U8Vector3: ...
    @property
    def rlg(self) -> U8Vector3: ...
    @property
    def rlb(self) -> U8Vector3: ...
    @property
    def rla(self) -> U8Vector3: ...
    @property
    def rlo(self) -> U8Vector3: ...
    @property
    def rll(self) -> U8Vector3: ...
    @property
    def grr(self) -> U8Vector3: ...
    @property
    def grg(self) -> U8Vector3: ...
    @property
    def grb(self) -> U8Vector3: ...
    @property
    def gra(self) -> U8Vector3: ...
    @property
    def gro(self) -> U8Vector3: ...
    @property
    def grl(self) -> U8Vector3: ...
    @property
    def ggr(self) -> U8Vector3: ...
    @property
    def ggg(self) -> U8Vector3: ...
    @property
    def ggb(self) -> U8Vector3: ...
    @property
    def gga(self) -> U8Vector3: ...
    @property
    def ggo(self) -> U8Vector3: ...
    @property
    def ggl(self) -> U8Vector3: ...
    @property
    def gbr(self) -> U8Vector3: ...
    @property
    def gbg(self) -> U8Vector3: ...
    @property
    def gbb(self) -> U8Vector3: ...
    @property
    def gba(self) -> U8Vector3: ...
    @property
    def gbo(self) -> U8Vector3: ...
    @property
    def gbl(self) -> U8Vector3: ...
    @property
    def gar(self) -> U8Vector3: ...
    @property
    def gag(self) -> U8Vector3: ...
    @property
    def gab(self) -> U8Vector3: ...
    @property
    def gaa(self) -> U8Vector3: ...
    @property
    def gao(self) -> U8Vector3: ...
    @property
    def gal(self) -> U8Vector3: ...
    @property
    def gor(self) -> U8Vector3: ...
    @property
    def gog(self) -> U8Vector3: ...
    @property
    def gob(self) -> U8Vector3: ...
    @property
    def goa(self) -> U8Vector3: ...
    @property
    def goo(self) -> U8Vector3: ...
    @property
    def gol(self) -> U8Vector3: ...
    @property
    def glr(self) -> U8Vector3: ...
    @property
    def glg(self) -> U8Vector3: ...
    @property
    def glb(self) -> U8Vector3: ...
    @property
    def gla(self) -> U8Vector3: ...
    @property
    def glo(self) -> U8Vector3: ...
    @property
    def gll(self) -> U8Vector3: ...
    @property
    def brr(self) -> U8Vector3: ...
    @property
    def brg(self) -> U8Vector3: ...
    @property
    def brb(self) -> U8Vector3: ...
    @property
    def bra(self) -> U8Vector3: ...
    @property
    def bro(self) -> U8Vector3: ...
    @property
    def brl(self) -> U8Vector3: ...
    @property
    def bgr(self) -> U8Vector3: ...
    @property
    def bgg(self) -> U8Vector3: ...
    @property
    def bgb(self) -> U8Vector3: ...
    @property
    def bga(self) -> U8Vector3: ...
    @property
    def bgo(self) -> U8Vector3: ...
    @property
    def bgl(self) -> U8Vector3: ...
    @property
    def bbr(self) -> U8Vector3: ...
    @property
    def bbg(self) -> U8Vector3: ...
    @property
    def bbb(self) -> U8Vector3: ...
    @property
    def bba(self) -> U8Vector3: ...
    @property
    def bbo(self) -> U8Vector3: ...
    @property
    def bbl(self) -> U8Vector3: ...
    @property
    def bar(self) -> U8Vector3: ...
    @property
    def bag(self) -> U8Vector3: ...
    @property
    def bab(self) -> U8Vector3: ...
    @property
    def baa(self) -> U8Vector3: ...
    @property
    def bao(self) -> U8Vector3: ...
    @property
    def bal(self) -> U8Vector3: ...
    @property
    def bor(self) -> U8Vector3: ...
    @property
    def bog(self) -> U8Vector3: ...
    @property
    def bob(self) -> U8Vector3: ...
    @property
    def boa(self) -> U8Vector3: ...
    @property
    def boo(self) -> U8Vector3: ...
    @property
    def bol(self) -> U8Vector3: ...
    @property
    def blr(self) -> U8Vector3: ...
    @property
    def blg(self) -> U8Vector3: ...
    @property
    def blb(self) -> U8Vector3: ...
    @property
    def bla(self) -> U8Vector3: ...
    @property
    def blo(self) -> U8Vector3: ...
    @property
    def bll(self) -> U8Vector3: ...
    @property
    def arr(self) -> U8Vector3: ...
    @property
    def arg(self) -> U8Vector3: ...
    @property
    def arb(self) -> U8Vector3: ...
    @property
    def ara(self) -> U8Vector3: ...
    @property
    def aro(self) -> U8Vector3: ...
    @property
    def arl(self) -> U8Vector3: ...
    @property
    def agr(self) -> U8Vector3: ...
    @property
    def agg(self) -> U8Vector3: ...
    @property
    def agb(self) -> U8Vector3: ...
    @property
    def aga(self) -> U8Vector3: ...
    @property
    def ago(self) -> U8Vector3: ...
    @property
    def agl(self) -> U8Vector3: ...
    @property
    def abr(self) -> U8Vector3: ...
    @property
    def abg(self) -> U8Vector3: ...
    @property
    def abb(self) -> U8Vector3: ...
    @property
    def aba(self) -> U8Vector3: ...
    @property
    def abo(self) -> U8Vector3: ...
    @property
    def abl(self) -> U8Vector3: ...
    @property
    def aar(self) -> U8Vector3: ...
    @property
    def aag(self) -> U8Vector3: ...
    @property
    def aab(self) -> U8Vector3: ...
    @property
    def aaa(self) -> U8Vector3: ...
    @property
    def aao(self) -> U8Vector3: ...
    @property
    def aal(self) -> U8Vector3: ...
    @property
    def aor(self) -> U8Vector3: ...
    @property
    def aog(self) -> U8Vector3: ...
    @property
    def aob(self) -> U8Vector3: ...
    @property
    def aoa(self) -> U8Vector3: ...
    @property
    def aoo(self) -> U8Vector3: ...
    @property
    def aol(self) -> U8Vector3: ...
    @property
    def alr(self) -> U8Vector3: ...
    @property
    def alg(self) -> U8Vector3: ...
    @property
    def alb(self) -> U8Vector3: ...
    @property
    def ala(self) -> U8Vector3: ...
    @property
    def alo(self) -> U8Vector3: ...
    @property
    def all(self) -> U8Vector3: ...
    @property
    def orr(self) -> U8Vector3: ...
    @property
    def org(self) -> U8Vector3: ...
    @property
    def orb(self) -> U8Vector3: ...
    @property
    def ora(self) -> U8Vector3: ...
    @property
    def oro(self) -> U8Vector3: ...
    @property
    def orl(self) -> U8Vector3: ...
    @property
    def ogr(self) -> U8Vector3: ...
    @property
    def ogg(self) -> U8Vector3: ...
    @property
    def ogb(self) -> U8Vector3: ...
    @property
    def oga(self) -> U8Vector3: ...
    @property
    def ogo(self) -> U8Vector3: ...
    @property
    def ogl(self) -> U8Vector3: ...
    @property
    def obr(self) -> U8Vector3: ...
    @property
    def obg(self) -> U8Vector3: ...
    @property
    def obb(self) -> U8Vector3: ...
    @property
    def oba(self) -> U8Vector3: ...
    @property
    def obo(self) -> U8Vector3: ...
    @property
    def obl(self) -> U8Vector3: ...
    @property
    def oar(self) -> U8Vector3: ...
    @property
    def oag(self) -> U8Vector3: ...
    @property
    def oab(self) -> U8Vector3: ...
    @property
    def oaa(self) -> U8Vector3: ...
    @property
    def oao(self) -> U8Vector3: ...
    @property
    def oal(self) -> U8Vector3: ...
    @property
    def oor(self) -> U8Vector3: ...
    @property
    def oog(self) -> U8Vector3: ...
    @property
    def oob(self) -> U8Vector3: ...
    @property
    def ooa(self) -> U8Vector3: ...
    @property
    def olr(self) -> U8Vector3: ...
    @property
    def olg(self) -> U8Vector3: ...
    @property
    def olb(self) -> U8Vector3: ...
    @property
    def ola(self) -> U8Vector3: ...
    @property
    def lrr(self) -> U8Vector3: ...
    @property
    def lrg(self) -> U8Vector3: ...
    @property
    def lrb(self) -> U8Vector3: ...
    @property
    def lra(self) -> U8Vector3: ...
    @property
    def lro(self) -> U8Vector3: ...
    @property
    def lrl(self) -> U8Vector3: ...
    @property
    def lgr(self) -> U8Vector3: ...
    @property
    def lgg(self) -> U8Vector3: ...
    @property
    def lgb(self) -> U8Vector3: ...
    @property
    def lga(self) -> U8Vector3: ...
    @property
    def lgo(self) -> U8Vector3: ...
    @property
    def lgl(self) -> U8Vector3: ...
    @property
    def lbr(self) -> U8Vector3: ...
    @property
    def lbg(self) -> U8Vector3: ...
    @property
    def lbb(self) -> U8Vector3: ...
    @property
    def lba(self) -> U8Vector3: ...
    @property
    def lbo(self) -> U8Vector3: ...
    @property
    def lbl(self) -> U8Vector3: ...
    @property
    def lar(self) -> U8Vector3: ...
    @property
    def lag(self) -> U8Vector3: ...
    @property
    def lab(self) -> U8Vector3: ...
    @property
    def laa(self) -> U8Vector3: ...
    @property
    def lao(self) -> U8Vector3: ...
    @property
    def lal(self) -> U8Vector3: ...
    @property
    def lor(self) -> U8Vector3: ...
    @property
    def log(self) -> U8Vector3: ...
    @property
    def lob(self) -> U8Vector3: ...
    @property
    def loa(self) -> U8Vector3: ...
    @property
    def llr(self) -> U8Vector3: ...
    @property
    def llg(self) -> U8Vector3: ...
    @property
    def llb(self) -> U8Vector3: ...
    @property
    def lla(self) -> U8Vector3: ...
    @property
    def rrrr(self) -> U8Vector4: ...
    @property
    def rrrg(self) -> U8Vector4: ...
    @property
    def rrrb(self) -> U8Vector4: ...
    @property
    def rrra(self) -> U8Vector4: ...
    @property
    def rrro(self) -> U8Vector4: ...
    @property
    def rrrl(self) -> U8Vector4: ...
    @property
    def rrgr(self) -> U8Vector4: ...
    @property
    def rrgg(self) -> U8Vector4: ...
    @property
    def rrgb(self) -> U8Vector4: ...
    @property
    def rrga(self) -> U8Vector4: ...
    @property
    def rrgo(self) -> U8Vector4: ...
    @property
    def rrgl(self) -> U8Vector4: ...
    @property
    def rrbr(self) -> U8Vector4: ...
    @property
    def rrbg(self) -> U8Vector4: ...
    @property
    def rrbb(self) -> U8Vector4: ...
    @property
    def rrba(self) -> U8Vector4: ...
    @property
    def rrbo(self) -> U8Vector4: ...
    @property
    def rrbl(self) -> U8Vector4: ...
    @property
    def rrar(self) -> U8Vector4: ...
    @property
    def rrag(self) -> U8Vector4: ...
    @property
    def rrab(self) -> U8Vector4: ...
    @property
    def rraa(self) -> U8Vector4: ...
    @property
    def rrao(self) -> U8Vector4: ...
    @property
    def rral(self) -> U8Vector4: ...
    @property
    def rror(self) -> U8Vector4: ...
    @property
    def rrog(self) -> U8Vector4: ...
    @property
    def rrob(self) -> U8Vector4: ...
    @property
    def rroa(self) -> U8Vector4: ...
    @property
    def rroo(self) -> U8Vector4: ...
    @property
    def rrol(self) -> U8Vector4: ...
    @property
    def rrlr(self) -> U8Vector4: ...
    @property
    def rrlg(self) -> U8Vector4: ...
    @property
    def rrlb(self) -> U8Vector4: ...
    @property
    def rrla(self) -> U8Vector4: ...
    @property
    def rrlo(self) -> U8Vector4: ...
    @property
    def rrll(self) -> U8Vector4: ...
    @property
    def rgrr(self) -> U8Vector4: ...
    @property
    def rgrg(self) -> U8Vector4: ...
    @property
    def rgrb(self) -> U8Vector4: ...
    @property
    def rgra(self) -> U8Vector4: ...
    @property
    def rgro(self) -> U8Vector4: ...
    @property
    def rgrl(self) -> U8Vector4: ...
    @property
    def rggr(self) -> U8Vector4: ...
    @property
    def rggg(self) -> U8Vector4: ...
    @property
    def rggb(self) -> U8Vector4: ...
    @property
    def rgga(self) -> U8Vector4: ...
    @property
    def rggo(self) -> U8Vector4: ...
    @property
    def rggl(self) -> U8Vector4: ...
    @property
    def rgbr(self) -> U8Vector4: ...
    @property
    def rgbg(self) -> U8Vector4: ...
    @property
    def rgbb(self) -> U8Vector4: ...
    @property
    def rgba(self) -> U8Vector4: ...
    @property
    def rgbo(self) -> U8Vector4: ...
    @property
    def rgbl(self) -> U8Vector4: ...
    @property
    def rgar(self) -> U8Vector4: ...
    @property
    def rgag(self) -> U8Vector4: ...
    @property
    def rgab(self) -> U8Vector4: ...
    @property
    def rgaa(self) -> U8Vector4: ...
    @property
    def rgao(self) -> U8Vector4: ...
    @property
    def rgal(self) -> U8Vector4: ...
    @property
    def rgor(self) -> U8Vector4: ...
    @property
    def rgog(self) -> U8Vector4: ...
    @property
    def rgob(self) -> U8Vector4: ...
    @property
    def rgoa(self) -> U8Vector4: ...
    @property
    def rgoo(self) -> U8Vector4: ...
    @property
    def rgol(self) -> U8Vector4: ...
    @property
    def rglr(self) -> U8Vector4: ...
    @property
    def rglg(self) -> U8Vector4: ...
    @property
    def rglb(self) -> U8Vector4: ...
    @property
    def rgla(self) -> U8Vector4: ...
    @property
    def rglo(self) -> U8Vector4: ...
    @property
    def rgll(self) -> U8Vector4: ...
    @property
    def rbrr(self) -> U8Vector4: ...
    @property
    def rbrg(self) -> U8Vector4: ...
    @property
    def rbrb(self) -> U8Vector4: ...
    @property
    def rbra(self) -> U8Vector4: ...
    @property
    def rbro(self) -> U8Vector4: ...
    @property
    def rbrl(self) -> U8Vector4: ...
    @property
    def rbgr(self) -> U8Vector4: ...
    @property
    def rbgg(self) -> U8Vector4: ...
    @property
    def rbgb(self) -> U8Vector4: ...
    @property
    def rbga(self) -> U8Vector4: ...
    @property
    def rbgo(self) -> U8Vector4: ...
    @property
    def rbgl(self) -> U8Vector4: ...
    @property
    def rbbr(self) -> U8Vector4: ...
    @property
    def rbbg(self) -> U8Vector4: ...
    @property
    def rbbb(self) -> U8Vector4: ...
    @property
    def rbba(self) -> U8Vector4: ...
    @property
    def rbbo(self) -> U8Vector4: ...
    @property
    def rbbl(self) -> U8Vector4: ...
    @property
    def rbar(self) -> U8Vector4: ...
    @property
    def rbag(self) -> U8Vector4: ...
    @property
    def rbab(self) -> U8Vector4: ...
    @property
    def rbaa(self) -> U8Vector4: ...
    @property
    def rbao(self) -> U8Vector4: ...
    @property
    def rbal(self) -> U8Vector4: ...
    @property
    def rbor(self) -> U8Vector4: ...
    @property
    def rbog(self) -> U8Vector4: ...
    @property
    def rbob(self) -> U8Vector4: ...
    @property
    def rboa(self) -> U8Vector4: ...
    @property
    def rboo(self) -> U8Vector4: ...
    @property
    def rbol(self) -> U8Vector4: ...
    @property
    def rblr(self) -> U8Vector4: ...
    @property
    def rblg(self) -> U8Vector4: ...
    @property
    def rblb(self) -> U8Vector4: ...
    @property
    def rbla(self) -> U8Vector4: ...
    @property
    def rblo(self) -> U8Vector4: ...
    @property
    def rbll(self) -> U8Vector4: ...
    @property
    def rarr(self) -> U8Vector4: ...
    @property
    def rarg(self) -> U8Vector4: ...
    @property
    def rarb(self) -> U8Vector4: ...
    @property
    def rara(self) -> U8Vector4: ...
    @property
    def raro(self) -> U8Vector4: ...
    @property
    def rarl(self) -> U8Vector4: ...
    @property
    def ragr(self) -> U8Vector4: ...
    @property
    def ragg(self) -> U8Vector4: ...
    @property
    def ragb(self) -> U8Vector4: ...
    @property
    def raga(self) -> U8Vector4: ...
    @property
    def rago(self) -> U8Vector4: ...
    @property
    def ragl(self) -> U8Vector4: ...
    @property
    def rabr(self) -> U8Vector4: ...
    @property
    def rabg(self) -> U8Vector4: ...
    @property
    def rabb(self) -> U8Vector4: ...
    @property
    def raba(self) -> U8Vector4: ...
    @property
    def rabo(self) -> U8Vector4: ...
    @property
    def rabl(self) -> U8Vector4: ...
    @property
    def raar(self) -> U8Vector4: ...
    @property
    def raag(self) -> U8Vector4: ...
    @property
    def raab(self) -> U8Vector4: ...
    @property
    def raaa(self) -> U8Vector4: ...
    @property
    def raao(self) -> U8Vector4: ...
    @property
    def raal(self) -> U8Vector4: ...
    @property
    def raor(self) -> U8Vector4: ...
    @property
    def raog(self) -> U8Vector4: ...
    @property
    def raob(self) -> U8Vector4: ...
    @property
    def raoa(self) -> U8Vector4: ...
    @property
    def raoo(self) -> U8Vector4: ...
    @property
    def raol(self) -> U8Vector4: ...
    @property
    def ralr(self) -> U8Vector4: ...
    @property
    def ralg(self) -> U8Vector4: ...
    @property
    def ralb(self) -> U8Vector4: ...
    @property
    def rala(self) -> U8Vector4: ...
    @property
    def ralo(self) -> U8Vector4: ...
    @property
    def rall(self) -> U8Vector4: ...
    @property
    def rorr(self) -> U8Vector4: ...
    @property
    def rorg(self) -> U8Vector4: ...
    @property
    def rorb(self) -> U8Vector4: ...
    @property
    def rora(self) -> U8Vector4: ...
    @property
    def roro(self) -> U8Vector4: ...
    @property
    def rorl(self) -> U8Vector4: ...
    @property
    def rogr(self) -> U8Vector4: ...
    @property
    def rogg(self) -> U8Vector4: ...
    @property
    def rogb(self) -> U8Vector4: ...
    @property
    def roga(self) -> U8Vector4: ...
    @property
    def rogo(self) -> U8Vector4: ...
    @property
    def rogl(self) -> U8Vector4: ...
    @property
    def robr(self) -> U8Vector4: ...
    @property
    def robg(self) -> U8Vector4: ...
    @property
    def robb(self) -> U8Vector4: ...
    @property
    def roba(self) -> U8Vector4: ...
    @property
    def robo(self) -> U8Vector4: ...
    @property
    def robl(self) -> U8Vector4: ...
    @property
    def roar(self) -> U8Vector4: ...
    @property
    def roag(self) -> U8Vector4: ...
    @property
    def roab(self) -> U8Vector4: ...
    @property
    def roaa(self) -> U8Vector4: ...
    @property
    def roao(self) -> U8Vector4: ...
    @property
    def roal(self) -> U8Vector4: ...
    @property
    def roor(self) -> U8Vector4: ...
    @property
    def roog(self) -> U8Vector4: ...
    @property
    def roob(self) -> U8Vector4: ...
    @property
    def rooa(self) -> U8Vector4: ...
    @property
    def rooo(self) -> U8Vector4: ...
    @property
    def rool(self) -> U8Vector4: ...
    @property
    def rolr(self) -> U8Vector4: ...
    @property
    def rolg(self) -> U8Vector4: ...
    @property
    def rolb(self) -> U8Vector4: ...
    @property
    def rola(self) -> U8Vector4: ...
    @property
    def rolo(self) -> U8Vector4: ...
    @property
    def roll(self) -> U8Vector4: ...
    @property
    def rlrr(self) -> U8Vector4: ...
    @property
    def rlrg(self) -> U8Vector4: ...
    @property
    def rlrb(self) -> U8Vector4: ...
    @property
    def rlra(self) -> U8Vector4: ...
    @property
    def rlro(self) -> U8Vector4: ...
    @property
    def rlrl(self) -> U8Vector4: ...
    @property
    def rlgr(self) -> U8Vector4: ...
    @property
    def rlgg(self) -> U8Vector4: ...
    @property
    def rlgb(self) -> U8Vector4: ...
    @property
    def rlga(self) -> U8Vector4: ...
    @property
    def rlgo(self) -> U8Vector4: ...
    @property
    def rlgl(self) -> U8Vector4: ...
    @property
    def rlbr(self) -> U8Vector4: ...
    @property
    def rlbg(self) -> U8Vector4: ...
    @property
    def rlbb(self) -> U8Vector4: ...
    @property
    def rlba(self) -> U8Vector4: ...
    @property
    def rlbo(self) -> U8Vector4: ...
    @property
    def rlbl(self) -> U8Vector4: ...
    @property
    def rlar(self) -> U8Vector4: ...
    @property
    def rlag(self) -> U8Vector4: ...
    @property
    def rlab(self) -> U8Vector4: ...
    @property
    def rlaa(self) -> U8Vector4: ...
    @property
    def rlao(self) -> U8Vector4: ...
    @property
    def rlal(self) -> U8Vector4: ...
    @property
    def rlor(self) -> U8Vector4: ...
    @property
    def rlog(self) -> U8Vector4: ...
    @property
    def rlob(self) -> U8Vector4: ...
    @property
    def rloa(self) -> U8Vector4: ...
    @property
    def rloo(self) -> U8Vector4: ...
    @property
    def rlol(self) -> U8Vector4: ...
    @property
    def rllr(self) -> U8Vector4: ...
    @property
    def rllg(self) -> U8Vector4: ...
    @property
    def rllb(self) -> U8Vector4: ...
    @property
    def rlla(self) -> U8Vector4: ...
    @property
    def rllo(self) -> U8Vector4: ...
    @property
    def rlll(self) -> U8Vector4: ...
    @property
    def grrr(self) -> U8Vector4: ...
    @property
    def grrg(self) -> U8Vector4: ...
    @property
    def grrb(self) -> U8Vector4: ...
    @property
    def grra(self) -> U8Vector4: ...
    @property
    def grro(self) -> U8Vector4: ...
    @property
    def grrl(self) -> U8Vector4: ...
    @property
    def grgr(self) -> U8Vector4: ...
    @property
    def grgg(self) -> U8Vector4: ...
    @property
    def grgb(self) -> U8Vector4: ...
    @property
    def grga(self) -> U8Vector4: ...
    @property
    def grgo(self) -> U8Vector4: ...
    @property
    def grgl(self) -> U8Vector4: ...
    @property
    def grbr(self) -> U8Vector4: ...
    @property
    def grbg(self) -> U8Vector4: ...
    @property
    def grbb(self) -> U8Vector4: ...
    @property
    def grba(self) -> U8Vector4: ...
    @property
    def grbo(self) -> U8Vector4: ...
    @property
    def grbl(self) -> U8Vector4: ...
    @property
    def grar(self) -> U8Vector4: ...
    @property
    def grag(self) -> U8Vector4: ...
    @property
    def grab(self) -> U8Vector4: ...
    @property
    def graa(self) -> U8Vector4: ...
    @property
    def grao(self) -> U8Vector4: ...
    @property
    def gral(self) -> U8Vector4: ...
    @property
    def gror(self) -> U8Vector4: ...
    @property
    def grog(self) -> U8Vector4: ...
    @property
    def grob(self) -> U8Vector4: ...
    @property
    def groa(self) -> U8Vector4: ...
    @property
    def groo(self) -> U8Vector4: ...
    @property
    def grol(self) -> U8Vector4: ...
    @property
    def grlr(self) -> U8Vector4: ...
    @property
    def grlg(self) -> U8Vector4: ...
    @property
    def grlb(self) -> U8Vector4: ...
    @property
    def grla(self) -> U8Vector4: ...
    @property
    def grlo(self) -> U8Vector4: ...
    @property
    def grll(self) -> U8Vector4: ...
    @property
    def ggrr(self) -> U8Vector4: ...
    @property
    def ggrg(self) -> U8Vector4: ...
    @property
    def ggrb(self) -> U8Vector4: ...
    @property
    def ggra(self) -> U8Vector4: ...
    @property
    def ggro(self) -> U8Vector4: ...
    @property
    def ggrl(self) -> U8Vector4: ...
    @property
    def gggr(self) -> U8Vector4: ...
    @property
    def gggg(self) -> U8Vector4: ...
    @property
    def gggb(self) -> U8Vector4: ...
    @property
    def ggga(self) -> U8Vector4: ...
    @property
    def gggo(self) -> U8Vector4: ...
    @property
    def gggl(self) -> U8Vector4: ...
    @property
    def ggbr(self) -> U8Vector4: ...
    @property
    def ggbg(self) -> U8Vector4: ...
    @property
    def ggbb(self) -> U8Vector4: ...
    @property
    def ggba(self) -> U8Vector4: ...
    @property
    def ggbo(self) -> U8Vector4: ...
    @property
    def ggbl(self) -> U8Vector4: ...
    @property
    def ggar(self) -> U8Vector4: ...
    @property
    def ggag(self) -> U8Vector4: ...
    @property
    def ggab(self) -> U8Vector4: ...
    @property
    def ggaa(self) -> U8Vector4: ...
    @property
    def ggao(self) -> U8Vector4: ...
    @property
    def ggal(self) -> U8Vector4: ...
    @property
    def ggor(self) -> U8Vector4: ...
    @property
    def ggog(self) -> U8Vector4: ...
    @property
    def ggob(self) -> U8Vector4: ...
    @property
    def ggoa(self) -> U8Vector4: ...
    @property
    def ggoo(self) -> U8Vector4: ...
    @property
    def ggol(self) -> U8Vector4: ...
    @property
    def gglr(self) -> U8Vector4: ...
    @property
    def gglg(self) -> U8Vector4: ...
    @property
    def gglb(self) -> U8Vector4: ...
    @property
    def ggla(self) -> U8Vector4: ...
    @property
    def gglo(self) -> U8Vector4: ...
    @property
    def ggll(self) -> U8Vector4: ...
    @property
    def gbrr(self) -> U8Vector4: ...
    @property
    def gbrg(self) -> U8Vector4: ...
    @property
    def gbrb(self) -> U8Vector4: ...
    @property
    def gbra(self) -> U8Vector4: ...
    @property
    def gbro(self) -> U8Vector4: ...
    @property
    def gbrl(self) -> U8Vector4: ...
    @property
    def gbgr(self) -> U8Vector4: ...
    @property
    def gbgg(self) -> U8Vector4: ...
    @property
    def gbgb(self) -> U8Vector4: ...
    @property
    def gbga(self) -> U8Vector4: ...
    @property
    def gbgo(self) -> U8Vector4: ...
    @property
    def gbgl(self) -> U8Vector4: ...
    @property
    def gbbr(self) -> U8Vector4: ...
    @property
    def gbbg(self) -> U8Vector4: ...
    @property
    def gbbb(self) -> U8Vector4: ...
    @property
    def gbba(self) -> U8Vector4: ...
    @property
    def gbbo(self) -> U8Vector4: ...
    @property
    def gbbl(self) -> U8Vector4: ...
    @property
    def gbar(self) -> U8Vector4: ...
    @property
    def gbag(self) -> U8Vector4: ...
    @property
    def gbab(self) -> U8Vector4: ...
    @property
    def gbaa(self) -> U8Vector4: ...
    @property
    def gbao(self) -> U8Vector4: ...
    @property
    def gbal(self) -> U8Vector4: ...
    @property
    def gbor(self) -> U8Vector4: ...
    @property
    def gbog(self) -> U8Vector4: ...
    @property
    def gbob(self) -> U8Vector4: ...
    @property
    def gboa(self) -> U8Vector4: ...
    @property
    def gboo(self) -> U8Vector4: ...
    @property
    def gbol(self) -> U8Vector4: ...
    @property
    def gblr(self) -> U8Vector4: ...
    @property
    def gblg(self) -> U8Vector4: ...
    @property
    def gblb(self) -> U8Vector4: ...
    @property
    def gbla(self) -> U8Vector4: ...
    @property
    def gblo(self) -> U8Vector4: ...
    @property
    def gbll(self) -> U8Vector4: ...
    @property
    def garr(self) -> U8Vector4: ...
    @property
    def garg(self) -> U8Vector4: ...
    @property
    def garb(self) -> U8Vector4: ...
    @property
    def gara(self) -> U8Vector4: ...
    @property
    def garo(self) -> U8Vector4: ...
    @property
    def garl(self) -> U8Vector4: ...
    @property
    def gagr(self) -> U8Vector4: ...
    @property
    def gagg(self) -> U8Vector4: ...
    @property
    def gagb(self) -> U8Vector4: ...
    @property
    def gaga(self) -> U8Vector4: ...
    @property
    def gago(self) -> U8Vector4: ...
    @property
    def gagl(self) -> U8Vector4: ...
    @property
    def gabr(self) -> U8Vector4: ...
    @property
    def gabg(self) -> U8Vector4: ...
    @property
    def gabb(self) -> U8Vector4: ...
    @property
    def gaba(self) -> U8Vector4: ...
    @property
    def gabo(self) -> U8Vector4: ...
    @property
    def gabl(self) -> U8Vector4: ...
    @property
    def gaar(self) -> U8Vector4: ...
    @property
    def gaag(self) -> U8Vector4: ...
    @property
    def gaab(self) -> U8Vector4: ...
    @property
    def gaaa(self) -> U8Vector4: ...
    @property
    def gaao(self) -> U8Vector4: ...
    @property
    def gaal(self) -> U8Vector4: ...
    @property
    def gaor(self) -> U8Vector4: ...
    @property
    def gaog(self) -> U8Vector4: ...
    @property
    def gaob(self) -> U8Vector4: ...
    @property
    def gaoa(self) -> U8Vector4: ...
    @property
    def gaoo(self) -> U8Vector4: ...
    @property
    def gaol(self) -> U8Vector4: ...
    @property
    def galr(self) -> U8Vector4: ...
    @property
    def galg(self) -> U8Vector4: ...
    @property
    def galb(self) -> U8Vector4: ...
    @property
    def gala(self) -> U8Vector4: ...
    @property
    def galo(self) -> U8Vector4: ...
    @property
    def gall(self) -> U8Vector4: ...
    @property
    def gorr(self) -> U8Vector4: ...
    @property
    def gorg(self) -> U8Vector4: ...
    @property
    def gorb(self) -> U8Vector4: ...
    @property
    def gora(self) -> U8Vector4: ...
    @property
    def goro(self) -> U8Vector4: ...
    @property
    def gorl(self) -> U8Vector4: ...
    @property
    def gogr(self) -> U8Vector4: ...
    @property
    def gogg(self) -> U8Vector4: ...
    @property
    def gogb(self) -> U8Vector4: ...
    @property
    def goga(self) -> U8Vector4: ...
    @property
    def gogo(self) -> U8Vector4: ...
    @property
    def gogl(self) -> U8Vector4: ...
    @property
    def gobr(self) -> U8Vector4: ...
    @property
    def gobg(self) -> U8Vector4: ...
    @property
    def gobb(self) -> U8Vector4: ...
    @property
    def goba(self) -> U8Vector4: ...
    @property
    def gobo(self) -> U8Vector4: ...
    @property
    def gobl(self) -> U8Vector4: ...
    @property
    def goar(self) -> U8Vector4: ...
    @property
    def goag(self) -> U8Vector4: ...
    @property
    def goab(self) -> U8Vector4: ...
    @property
    def goaa(self) -> U8Vector4: ...
    @property
    def goao(self) -> U8Vector4: ...
    @property
    def goal(self) -> U8Vector4: ...
    @property
    def goor(self) -> U8Vector4: ...
    @property
    def goog(self) -> U8Vector4: ...
    @property
    def goob(self) -> U8Vector4: ...
    @property
    def gooa(self) -> U8Vector4: ...
    @property
    def gooo(self) -> U8Vector4: ...
    @property
    def gool(self) -> U8Vector4: ...
    @property
    def golr(self) -> U8Vector4: ...
    @property
    def golg(self) -> U8Vector4: ...
    @property
    def golb(self) -> U8Vector4: ...
    @property
    def gola(self) -> U8Vector4: ...
    @property
    def golo(self) -> U8Vector4: ...
    @property
    def goll(self) -> U8Vector4: ...
    @property
    def glrr(self) -> U8Vector4: ...
    @property
    def glrg(self) -> U8Vector4: ...
    @property
    def glrb(self) -> U8Vector4: ...
    @property
    def glra(self) -> U8Vector4: ...
    @property
    def glro(self) -> U8Vector4: ...
    @property
    def glrl(self) -> U8Vector4: ...
    @property
    def glgr(self) -> U8Vector4: ...
    @property
    def glgg(self) -> U8Vector4: ...
    @property
    def glgb(self) -> U8Vector4: ...
    @property
    def glga(self) -> U8Vector4: ...
    @property
    def glgo(self) -> U8Vector4: ...
    @property
    def glgl(self) -> U8Vector4: ...
    @property
    def glbr(self) -> U8Vector4: ...
    @property
    def glbg(self) -> U8Vector4: ...
    @property
    def glbb(self) -> U8Vector4: ...
    @property
    def glba(self) -> U8Vector4: ...
    @property
    def glbo(self) -> U8Vector4: ...
    @property
    def glbl(self) -> U8Vector4: ...
    @property
    def glar(self) -> U8Vector4: ...
    @property
    def glag(self) -> U8Vector4: ...
    @property
    def glab(self) -> U8Vector4: ...
    @property
    def glaa(self) -> U8Vector4: ...
    @property
    def glao(self) -> U8Vector4: ...
    @property
    def glal(self) -> U8Vector4: ...
    @property
    def glor(self) -> U8Vector4: ...
    @property
    def glog(self) -> U8Vector4: ...
    @property
    def glob(self) -> U8Vector4: ...
    @property
    def gloa(self) -> U8Vector4: ...
    @property
    def gloo(self) -> U8Vector4: ...
    @property
    def glol(self) -> U8Vector4: ...
    @property
    def gllr(self) -> U8Vector4: ...
    @property
    def gllg(self) -> U8Vector4: ...
    @property
    def gllb(self) -> U8Vector4: ...
    @property
    def glla(self) -> U8Vector4: ...
    @property
    def gllo(self) -> U8Vector4: ...
    @property
    def glll(self) -> U8Vector4: ...
    @property
    def brrr(self) -> U8Vector4: ...
    @property
    def brrg(self) -> U8Vector4: ...
    @property
    def brrb(self) -> U8Vector4: ...
    @property
    def brra(self) -> U8Vector4: ...
    @property
    def brro(self) -> U8Vector4: ...
    @property
    def brrl(self) -> U8Vector4: ...
    @property
    def brgr(self) -> U8Vector4: ...
    @property
    def brgg(self) -> U8Vector4: ...
    @property
    def brgb(self) -> U8Vector4: ...
    @property
    def brga(self) -> U8Vector4: ...
    @property
    def brgo(self) -> U8Vector4: ...
    @property
    def brgl(self) -> U8Vector4: ...
    @property
    def brbr(self) -> U8Vector4: ...
    @property
    def brbg(self) -> U8Vector4: ...
    @property
    def brbb(self) -> U8Vector4: ...
    @property
    def brba(self) -> U8Vector4: ...
    @property
    def brbo(self) -> U8Vector4: ...
    @property
    def brbl(self) -> U8Vector4: ...
    @property
    def brar(self) -> U8Vector4: ...
    @property
    def brag(self) -> U8Vector4: ...
    @property
    def brab(self) -> U8Vector4: ...
    @property
    def braa(self) -> U8Vector4: ...
    @property
    def brao(self) -> U8Vector4: ...
    @property
    def bral(self) -> U8Vector4: ...
    @property
    def bror(self) -> U8Vector4: ...
    @property
    def brog(self) -> U8Vector4: ...
    @property
    def brob(self) -> U8Vector4: ...
    @property
    def broa(self) -> U8Vector4: ...
    @property
    def broo(self) -> U8Vector4: ...
    @property
    def brol(self) -> U8Vector4: ...
    @property
    def brlr(self) -> U8Vector4: ...
    @property
    def brlg(self) -> U8Vector4: ...
    @property
    def brlb(self) -> U8Vector4: ...
    @property
    def brla(self) -> U8Vector4: ...
    @property
    def brlo(self) -> U8Vector4: ...
    @property
    def brll(self) -> U8Vector4: ...
    @property
    def bgrr(self) -> U8Vector4: ...
    @property
    def bgrg(self) -> U8Vector4: ...
    @property
    def bgrb(self) -> U8Vector4: ...
    @property
    def bgra(self) -> U8Vector4: ...
    @property
    def bgro(self) -> U8Vector4: ...
    @property
    def bgrl(self) -> U8Vector4: ...
    @property
    def bggr(self) -> U8Vector4: ...
    @property
    def bggg(self) -> U8Vector4: ...
    @property
    def bggb(self) -> U8Vector4: ...
    @property
    def bgga(self) -> U8Vector4: ...
    @property
    def bggo(self) -> U8Vector4: ...
    @property
    def bggl(self) -> U8Vector4: ...
    @property
    def bgbr(self) -> U8Vector4: ...
    @property
    def bgbg(self) -> U8Vector4: ...
    @property
    def bgbb(self) -> U8Vector4: ...
    @property
    def bgba(self) -> U8Vector4: ...
    @property
    def bgbo(self) -> U8Vector4: ...
    @property
    def bgbl(self) -> U8Vector4: ...
    @property
    def bgar(self) -> U8Vector4: ...
    @property
    def bgag(self) -> U8Vector4: ...
    @property
    def bgab(self) -> U8Vector4: ...
    @property
    def bgaa(self) -> U8Vector4: ...
    @property
    def bgao(self) -> U8Vector4: ...
    @property
    def bgal(self) -> U8Vector4: ...
    @property
    def bgor(self) -> U8Vector4: ...
    @property
    def bgog(self) -> U8Vector4: ...
    @property
    def bgob(self) -> U8Vector4: ...
    @property
    def bgoa(self) -> U8Vector4: ...
    @property
    def bgoo(self) -> U8Vector4: ...
    @property
    def bgol(self) -> U8Vector4: ...
    @property
    def bglr(self) -> U8Vector4: ...
    @property
    def bglg(self) -> U8Vector4: ...
    @property
    def bglb(self) -> U8Vector4: ...
    @property
    def bgla(self) -> U8Vector4: ...
    @property
    def bglo(self) -> U8Vector4: ...
    @property
    def bgll(self) -> U8Vector4: ...
    @property
    def bbrr(self) -> U8Vector4: ...
    @property
    def bbrg(self) -> U8Vector4: ...
    @property
    def bbrb(self) -> U8Vector4: ...
    @property
    def bbra(self) -> U8Vector4: ...
    @property
    def bbro(self) -> U8Vector4: ...
    @property
    def bbrl(self) -> U8Vector4: ...
    @property
    def bbgr(self) -> U8Vector4: ...
    @property
    def bbgg(self) -> U8Vector4: ...
    @property
    def bbgb(self) -> U8Vector4: ...
    @property
    def bbga(self) -> U8Vector4: ...
    @property
    def bbgo(self) -> U8Vector4: ...
    @property
    def bbgl(self) -> U8Vector4: ...
    @property
    def bbbr(self) -> U8Vector4: ...
    @property
    def bbbg(self) -> U8Vector4: ...
    @property
    def bbbb(self) -> U8Vector4: ...
    @property
    def bbba(self) -> U8Vector4: ...
    @property
    def bbbo(self) -> U8Vector4: ...
    @property
    def bbbl(self) -> U8Vector4: ...
    @property
    def bbar(self) -> U8Vector4: ...
    @property
    def bbag(self) -> U8Vector4: ...
    @property
    def bbab(self) -> U8Vector4: ...
    @property
    def bbaa(self) -> U8Vector4: ...
    @property
    def bbao(self) -> U8Vector4: ...
    @property
    def bbal(self) -> U8Vector4: ...
    @property
    def bbor(self) -> U8Vector4: ...
    @property
    def bbog(self) -> U8Vector4: ...
    @property
    def bbob(self) -> U8Vector4: ...
    @property
    def bboa(self) -> U8Vector4: ...
    @property
    def bboo(self) -> U8Vector4: ...
    @property
    def bbol(self) -> U8Vector4: ...
    @property
    def bblr(self) -> U8Vector4: ...
    @property
    def bblg(self) -> U8Vector4: ...
    @property
    def bblb(self) -> U8Vector4: ...
    @property
    def bbla(self) -> U8Vector4: ...
    @property
    def bblo(self) -> U8Vector4: ...
    @property
    def bbll(self) -> U8Vector4: ...
    @property
    def barr(self) -> U8Vector4: ...
    @property
    def barg(self) -> U8Vector4: ...
    @property
    def barb(self) -> U8Vector4: ...
    @property
    def bara(self) -> U8Vector4: ...
    @property
    def baro(self) -> U8Vector4: ...
    @property
    def barl(self) -> U8Vector4: ...
    @property
    def bagr(self) -> U8Vector4: ...
    @property
    def bagg(self) -> U8Vector4: ...
    @property
    def bagb(self) -> U8Vector4: ...
    @property
    def baga(self) -> U8Vector4: ...
    @property
    def bago(self) -> U8Vector4: ...
    @property
    def bagl(self) -> U8Vector4: ...
    @property
    def babr(self) -> U8Vector4: ...
    @property
    def babg(self) -> U8Vector4: ...
    @property
    def babb(self) -> U8Vector4: ...
    @property
    def baba(self) -> U8Vector4: ...
    @property
    def babo(self) -> U8Vector4: ...
    @property
    def babl(self) -> U8Vector4: ...
    @property
    def baar(self) -> U8Vector4: ...
    @property
    def baag(self) -> U8Vector4: ...
    @property
    def baab(self) -> U8Vector4: ...
    @property
    def baaa(self) -> U8Vector4: ...
    @property
    def baao(self) -> U8Vector4: ...
    @property
    def baal(self) -> U8Vector4: ...
    @property
    def baor(self) -> U8Vector4: ...
    @property
    def baog(self) -> U8Vector4: ...
    @property
    def baob(self) -> U8Vector4: ...
    @property
    def baoa(self) -> U8Vector4: ...
    @property
    def baoo(self) -> U8Vector4: ...
    @property
    def baol(self) -> U8Vector4: ...
    @property
    def balr(self) -> U8Vector4: ...
    @property
    def balg(self) -> U8Vector4: ...
    @property
    def balb(self) -> U8Vector4: ...
    @property
    def bala(self) -> U8Vector4: ...
    @property
    def balo(self) -> U8Vector4: ...
    @property
    def ball(self) -> U8Vector4: ...
    @property
    def borr(self) -> U8Vector4: ...
    @property
    def borg(self) -> U8Vector4: ...
    @property
    def borb(self) -> U8Vector4: ...
    @property
    def bora(self) -> U8Vector4: ...
    @property
    def boro(self) -> U8Vector4: ...
    @property
    def borl(self) -> U8Vector4: ...
    @property
    def bogr(self) -> U8Vector4: ...
    @property
    def bogg(self) -> U8Vector4: ...
    @property
    def bogb(self) -> U8Vector4: ...
    @property
    def boga(self) -> U8Vector4: ...
    @property
    def bogo(self) -> U8Vector4: ...
    @property
    def bogl(self) -> U8Vector4: ...
    @property
    def bobr(self) -> U8Vector4: ...
    @property
    def bobg(self) -> U8Vector4: ...
    @property
    def bobb(self) -> U8Vector4: ...
    @property
    def boba(self) -> U8Vector4: ...
    @property
    def bobo(self) -> U8Vector4: ...
    @property
    def bobl(self) -> U8Vector4: ...
    @property
    def boar(self) -> U8Vector4: ...
    @property
    def boag(self) -> U8Vector4: ...
    @property
    def boab(self) -> U8Vector4: ...
    @property
    def boaa(self) -> U8Vector4: ...
    @property
    def boao(self) -> U8Vector4: ...
    @property
    def boal(self) -> U8Vector4: ...
    @property
    def boor(self) -> U8Vector4: ...
    @property
    def boog(self) -> U8Vector4: ...
    @property
    def boob(self) -> U8Vector4: ...
    @property
    def booa(self) -> U8Vector4: ...
    @property
    def booo(self) -> U8Vector4: ...
    @property
    def bool(self) -> U8Vector4: ...
    @property
    def bolr(self) -> U8Vector4: ...
    @property
    def bolg(self) -> U8Vector4: ...
    @property
    def bolb(self) -> U8Vector4: ...
    @property
    def bola(self) -> U8Vector4: ...
    @property
    def bolo(self) -> U8Vector4: ...
    @property
    def boll(self) -> U8Vector4: ...
    @property
    def blrr(self) -> U8Vector4: ...
    @property
    def blrg(self) -> U8Vector4: ...
    @property
    def blrb(self) -> U8Vector4: ...
    @property
    def blra(self) -> U8Vector4: ...
    @property
    def blro(self) -> U8Vector4: ...
    @property
    def blrl(self) -> U8Vector4: ...
    @property
    def blgr(self) -> U8Vector4: ...
    @property
    def blgg(self) -> U8Vector4: ...
    @property
    def blgb(self) -> U8Vector4: ...
    @property
    def blga(self) -> U8Vector4: ...
    @property
    def blgo(self) -> U8Vector4: ...
    @property
    def blgl(self) -> U8Vector4: ...
    @property
    def blbr(self) -> U8Vector4: ...
    @property
    def blbg(self) -> U8Vector4: ...
    @property
    def blbb(self) -> U8Vector4: ...
    @property
    def blba(self) -> U8Vector4: ...
    @property
    def blbo(self) -> U8Vector4: ...
    @property
    def blbl(self) -> U8Vector4: ...
    @property
    def blar(self) -> U8Vector4: ...
    @property
    def blag(self) -> U8Vector4: ...
    @property
    def blab(self) -> U8Vector4: ...
    @property
    def blaa(self) -> U8Vector4: ...
    @property
    def blao(self) -> U8Vector4: ...
    @property
    def blal(self) -> U8Vector4: ...
    @property
    def blor(self) -> U8Vector4: ...
    @property
    def blog(self) -> U8Vector4: ...
    @property
    def blob(self) -> U8Vector4: ...
    @property
    def bloa(self) -> U8Vector4: ...
    @property
    def bloo(self) -> U8Vector4: ...
    @property
    def blol(self) -> U8Vector4: ...
    @property
    def bllr(self) -> U8Vector4: ...
    @property
    def bllg(self) -> U8Vector4: ...
    @property
    def bllb(self) -> U8Vector4: ...
    @property
    def blla(self) -> U8Vector4: ...
    @property
    def bllo(self) -> U8Vector4: ...
    @property
    def blll(self) -> U8Vector4: ...
    @property
    def arrr(self) -> U8Vector4: ...
    @property
    def arrg(self) -> U8Vector4: ...
    @property
    def arrb(self) -> U8Vector4: ...
    @property
    def arra(self) -> U8Vector4: ...
    @property
    def arro(self) -> U8Vector4: ...
    @property
    def arrl(self) -> U8Vector4: ...
    @property
    def argr(self) -> U8Vector4: ...
    @property
    def argg(self) -> U8Vector4: ...
    @property
    def argb(self) -> U8Vector4: ...
    @property
    def arga(self) -> U8Vector4: ...
    @property
    def argo(self) -> U8Vector4: ...
    @property
    def argl(self) -> U8Vector4: ...
    @property
    def arbr(self) -> U8Vector4: ...
    @property
    def arbg(self) -> U8Vector4: ...
    @property
    def arbb(self) -> U8Vector4: ...
    @property
    def arba(self) -> U8Vector4: ...
    @property
    def arbo(self) -> U8Vector4: ...
    @property
    def arbl(self) -> U8Vector4: ...
    @property
    def arar(self) -> U8Vector4: ...
    @property
    def arag(self) -> U8Vector4: ...
    @property
    def arab(self) -> U8Vector4: ...
    @property
    def araa(self) -> U8Vector4: ...
    @property
    def arao(self) -> U8Vector4: ...
    @property
    def aral(self) -> U8Vector4: ...
    @property
    def aror(self) -> U8Vector4: ...
    @property
    def arog(self) -> U8Vector4: ...
    @property
    def arob(self) -> U8Vector4: ...
    @property
    def aroa(self) -> U8Vector4: ...
    @property
    def aroo(self) -> U8Vector4: ...
    @property
    def arol(self) -> U8Vector4: ...
    @property
    def arlr(self) -> U8Vector4: ...
    @property
    def arlg(self) -> U8Vector4: ...
    @property
    def arlb(self) -> U8Vector4: ...
    @property
    def arla(self) -> U8Vector4: ...
    @property
    def arlo(self) -> U8Vector4: ...
    @property
    def arll(self) -> U8Vector4: ...
    @property
    def agrr(self) -> U8Vector4: ...
    @property
    def agrg(self) -> U8Vector4: ...
    @property
    def agrb(self) -> U8Vector4: ...
    @property
    def agra(self) -> U8Vector4: ...
    @property
    def agro(self) -> U8Vector4: ...
    @property
    def agrl(self) -> U8Vector4: ...
    @property
    def aggr(self) -> U8Vector4: ...
    @property
    def aggg(self) -> U8Vector4: ...
    @property
    def aggb(self) -> U8Vector4: ...
    @property
    def agga(self) -> U8Vector4: ...
    @property
    def aggo(self) -> U8Vector4: ...
    @property
    def aggl(self) -> U8Vector4: ...
    @property
    def agbr(self) -> U8Vector4: ...
    @property
    def agbg(self) -> U8Vector4: ...
    @property
    def agbb(self) -> U8Vector4: ...
    @property
    def agba(self) -> U8Vector4: ...
    @property
    def agbo(self) -> U8Vector4: ...
    @property
    def agbl(self) -> U8Vector4: ...
    @property
    def agar(self) -> U8Vector4: ...
    @property
    def agag(self) -> U8Vector4: ...
    @property
    def agab(self) -> U8Vector4: ...
    @property
    def agaa(self) -> U8Vector4: ...
    @property
    def agao(self) -> U8Vector4: ...
    @property
    def agal(self) -> U8Vector4: ...
    @property
    def agor(self) -> U8Vector4: ...
    @property
    def agog(self) -> U8Vector4: ...
    @property
    def agob(self) -> U8Vector4: ...
    @property
    def agoa(self) -> U8Vector4: ...
    @property
    def agoo(self) -> U8Vector4: ...
    @property
    def agol(self) -> U8Vector4: ...
    @property
    def aglr(self) -> U8Vector4: ...
    @property
    def aglg(self) -> U8Vector4: ...
    @property
    def aglb(self) -> U8Vector4: ...
    @property
    def agla(self) -> U8Vector4: ...
    @property
    def aglo(self) -> U8Vector4: ...
    @property
    def agll(self) -> U8Vector4: ...
    @property
    def abrr(self) -> U8Vector4: ...
    @property
    def abrg(self) -> U8Vector4: ...
    @property
    def abrb(self) -> U8Vector4: ...
    @property
    def abra(self) -> U8Vector4: ...
    @property
    def abro(self) -> U8Vector4: ...
    @property
    def abrl(self) -> U8Vector4: ...
    @property
    def abgr(self) -> U8Vector4: ...
    @property
    def abgg(self) -> U8Vector4: ...
    @property
    def abgb(self) -> U8Vector4: ...
    @property
    def abga(self) -> U8Vector4: ...
    @property
    def abgo(self) -> U8Vector4: ...
    @property
    def abgl(self) -> U8Vector4: ...
    @property
    def abbr(self) -> U8Vector4: ...
    @property
    def abbg(self) -> U8Vector4: ...
    @property
    def abbb(self) -> U8Vector4: ...
    @property
    def abba(self) -> U8Vector4: ...
    @property
    def abbo(self) -> U8Vector4: ...
    @property
    def abbl(self) -> U8Vector4: ...
    @property
    def abar(self) -> U8Vector4: ...
    @property
    def abag(self) -> U8Vector4: ...
    @property
    def abab(self) -> U8Vector4: ...
    @property
    def abaa(self) -> U8Vector4: ...
    @property
    def abao(self) -> U8Vector4: ...
    @property
    def abal(self) -> U8Vector4: ...
    @property
    def abor(self) -> U8Vector4: ...
    @property
    def abog(self) -> U8Vector4: ...
    @property
    def abob(self) -> U8Vector4: ...
    @property
    def aboa(self) -> U8Vector4: ...
    @property
    def aboo(self) -> U8Vector4: ...
    @property
    def abol(self) -> U8Vector4: ...
    @property
    def ablr(self) -> U8Vector4: ...
    @property
    def ablg(self) -> U8Vector4: ...
    @property
    def ablb(self) -> U8Vector4: ...
    @property
    def abla(self) -> U8Vector4: ...
    @property
    def ablo(self) -> U8Vector4: ...
    @property
    def abll(self) -> U8Vector4: ...
    @property
    def aarr(self) -> U8Vector4: ...
    @property
    def aarg(self) -> U8Vector4: ...
    @property
    def aarb(self) -> U8Vector4: ...
    @property
    def aara(self) -> U8Vector4: ...
    @property
    def aaro(self) -> U8Vector4: ...
    @property
    def aarl(self) -> U8Vector4: ...
    @property
    def aagr(self) -> U8Vector4: ...
    @property
    def aagg(self) -> U8Vector4: ...
    @property
    def aagb(self) -> U8Vector4: ...
    @property
    def aaga(self) -> U8Vector4: ...
    @property
    def aago(self) -> U8Vector4: ...
    @property
    def aagl(self) -> U8Vector4: ...
    @property
    def aabr(self) -> U8Vector4: ...
    @property
    def aabg(self) -> U8Vector4: ...
    @property
    def aabb(self) -> U8Vector4: ...
    @property
    def aaba(self) -> U8Vector4: ...
    @property
    def aabo(self) -> U8Vector4: ...
    @property
    def aabl(self) -> U8Vector4: ...
    @property
    def aaar(self) -> U8Vector4: ...
    @property
    def aaag(self) -> U8Vector4: ...
    @property
    def aaab(self) -> U8Vector4: ...
    @property
    def aaaa(self) -> U8Vector4: ...
    @property
    def aaao(self) -> U8Vector4: ...
    @property
    def aaal(self) -> U8Vector4: ...
    @property
    def aaor(self) -> U8Vector4: ...
    @property
    def aaog(self) -> U8Vector4: ...
    @property
    def aaob(self) -> U8Vector4: ...
    @property
    def aaoa(self) -> U8Vector4: ...
    @property
    def aaoo(self) -> U8Vector4: ...
    @property
    def aaol(self) -> U8Vector4: ...
    @property
    def aalr(self) -> U8Vector4: ...
    @property
    def aalg(self) -> U8Vector4: ...
    @property
    def aalb(self) -> U8Vector4: ...
    @property
    def aala(self) -> U8Vector4: ...
    @property
    def aalo(self) -> U8Vector4: ...
    @property
    def aall(self) -> U8Vector4: ...
    @property
    def aorr(self) -> U8Vector4: ...
    @property
    def aorg(self) -> U8Vector4: ...
    @property
    def aorb(self) -> U8Vector4: ...
    @property
    def aora(self) -> U8Vector4: ...
    @property
    def aoro(self) -> U8Vector4: ...
    @property
    def aorl(self) -> U8Vector4: ...
    @property
    def aogr(self) -> U8Vector4: ...
    @property
    def aogg(self) -> U8Vector4: ...
    @property
    def aogb(self) -> U8Vector4: ...
    @property
    def aoga(self) -> U8Vector4: ...
    @property
    def aogo(self) -> U8Vector4: ...
    @property
    def aogl(self) -> U8Vector4: ...
    @property
    def aobr(self) -> U8Vector4: ...
    @property
    def aobg(self) -> U8Vector4: ...
    @property
    def aobb(self) -> U8Vector4: ...
    @property
    def aoba(self) -> U8Vector4: ...
    @property
    def aobo(self) -> U8Vector4: ...
    @property
    def aobl(self) -> U8Vector4: ...
    @property
    def aoar(self) -> U8Vector4: ...
    @property
    def aoag(self) -> U8Vector4: ...
    @property
    def aoab(self) -> U8Vector4: ...
    @property
    def aoaa(self) -> U8Vector4: ...
    @property
    def aoao(self) -> U8Vector4: ...
    @property
    def aoal(self) -> U8Vector4: ...
    @property
    def aoor(self) -> U8Vector4: ...
    @property
    def aoog(self) -> U8Vector4: ...
    @property
    def aoob(self) -> U8Vector4: ...
    @property
    def aooa(self) -> U8Vector4: ...
    @property
    def aooo(self) -> U8Vector4: ...
    @property
    def aool(self) -> U8Vector4: ...
    @property
    def aolr(self) -> U8Vector4: ...
    @property
    def aolg(self) -> U8Vector4: ...
    @property
    def aolb(self) -> U8Vector4: ...
    @property
    def aola(self) -> U8Vector4: ...
    @property
    def aolo(self) -> U8Vector4: ...
    @property
    def aoll(self) -> U8Vector4: ...
    @property
    def alrr(self) -> U8Vector4: ...
    @property
    def alrg(self) -> U8Vector4: ...
    @property
    def alrb(self) -> U8Vector4: ...
    @property
    def alra(self) -> U8Vector4: ...
    @property
    def alro(self) -> U8Vector4: ...
    @property
    def alrl(self) -> U8Vector4: ...
    @property
    def algr(self) -> U8Vector4: ...
    @property
    def algg(self) -> U8Vector4: ...
    @property
    def algb(self) -> U8Vector4: ...
    @property
    def alga(self) -> U8Vector4: ...
    @property
    def algo(self) -> U8Vector4: ...
    @property
    def algl(self) -> U8Vector4: ...
    @property
    def albr(self) -> U8Vector4: ...
    @property
    def albg(self) -> U8Vector4: ...
    @property
    def albb(self) -> U8Vector4: ...
    @property
    def alba(self) -> U8Vector4: ...
    @property
    def albo(self) -> U8Vector4: ...
    @property
    def albl(self) -> U8Vector4: ...
    @property
    def alar(self) -> U8Vector4: ...
    @property
    def alag(self) -> U8Vector4: ...
    @property
    def alab(self) -> U8Vector4: ...
    @property
    def alaa(self) -> U8Vector4: ...
    @property
    def alao(self) -> U8Vector4: ...
    @property
    def alal(self) -> U8Vector4: ...
    @property
    def alor(self) -> U8Vector4: ...
    @property
    def alog(self) -> U8Vector4: ...
    @property
    def alob(self) -> U8Vector4: ...
    @property
    def aloa(self) -> U8Vector4: ...
    @property
    def aloo(self) -> U8Vector4: ...
    @property
    def alol(self) -> U8Vector4: ...
    @property
    def allr(self) -> U8Vector4: ...
    @property
    def allg(self) -> U8Vector4: ...
    @property
    def allb(self) -> U8Vector4: ...
    @property
    def alla(self) -> U8Vector4: ...
    @property
    def allo(self) -> U8Vector4: ...
    @property
    def alll(self) -> U8Vector4: ...
    @property
    def orrr(self) -> U8Vector4: ...
    @property
    def orrg(self) -> U8Vector4: ...
    @property
    def orrb(self) -> U8Vector4: ...
    @property
    def orra(self) -> U8Vector4: ...
    @property
    def orro(self) -> U8Vector4: ...
    @property
    def orrl(self) -> U8Vector4: ...
    @property
    def orgr(self) -> U8Vector4: ...
    @property
    def orgg(self) -> U8Vector4: ...
    @property
    def orgb(self) -> U8Vector4: ...
    @property
    def orga(self) -> U8Vector4: ...
    @property
    def orgo(self) -> U8Vector4: ...
    @property
    def orgl(self) -> U8Vector4: ...
    @property
    def orbr(self) -> U8Vector4: ...
    @property
    def orbg(self) -> U8Vector4: ...
    @property
    def orbb(self) -> U8Vector4: ...
    @property
    def orba(self) -> U8Vector4: ...
    @property
    def orbo(self) -> U8Vector4: ...
    @property
    def orbl(self) -> U8Vector4: ...
    @property
    def orar(self) -> U8Vector4: ...
    @property
    def orag(self) -> U8Vector4: ...
    @property
    def orab(self) -> U8Vector4: ...
    @property
    def oraa(self) -> U8Vector4: ...
    @property
    def orao(self) -> U8Vector4: ...
    @property
    def oral(self) -> U8Vector4: ...
    @property
    def oror(self) -> U8Vector4: ...
    @property
    def orog(self) -> U8Vector4: ...
    @property
    def orob(self) -> U8Vector4: ...
    @property
    def oroa(self) -> U8Vector4: ...
    @property
    def oroo(self) -> U8Vector4: ...
    @property
    def orol(self) -> U8Vector4: ...
    @property
    def orlr(self) -> U8Vector4: ...
    @property
    def orlg(self) -> U8Vector4: ...
    @property
    def orlb(self) -> U8Vector4: ...
    @property
    def orla(self) -> U8Vector4: ...
    @property
    def orlo(self) -> U8Vector4: ...
    @property
    def orll(self) -> U8Vector4: ...
    @property
    def ogrr(self) -> U8Vector4: ...
    @property
    def ogrg(self) -> U8Vector4: ...
    @property
    def ogrb(self) -> U8Vector4: ...
    @property
    def ogra(self) -> U8Vector4: ...
    @property
    def ogro(self) -> U8Vector4: ...
    @property
    def ogrl(self) -> U8Vector4: ...
    @property
    def oggr(self) -> U8Vector4: ...
    @property
    def oggg(self) -> U8Vector4: ...
    @property
    def oggb(self) -> U8Vector4: ...
    @property
    def ogga(self) -> U8Vector4: ...
    @property
    def oggo(self) -> U8Vector4: ...
    @property
    def oggl(self) -> U8Vector4: ...
    @property
    def ogbr(self) -> U8Vector4: ...
    @property
    def ogbg(self) -> U8Vector4: ...
    @property
    def ogbb(self) -> U8Vector4: ...
    @property
    def ogba(self) -> U8Vector4: ...
    @property
    def ogbo(self) -> U8Vector4: ...
    @property
    def ogbl(self) -> U8Vector4: ...
    @property
    def ogar(self) -> U8Vector4: ...
    @property
    def ogag(self) -> U8Vector4: ...
    @property
    def ogab(self) -> U8Vector4: ...
    @property
    def ogaa(self) -> U8Vector4: ...
    @property
    def ogao(self) -> U8Vector4: ...
    @property
    def ogal(self) -> U8Vector4: ...
    @property
    def ogor(self) -> U8Vector4: ...
    @property
    def ogog(self) -> U8Vector4: ...
    @property
    def ogob(self) -> U8Vector4: ...
    @property
    def ogoa(self) -> U8Vector4: ...
    @property
    def ogoo(self) -> U8Vector4: ...
    @property
    def ogol(self) -> U8Vector4: ...
    @property
    def oglr(self) -> U8Vector4: ...
    @property
    def oglg(self) -> U8Vector4: ...
    @property
    def oglb(self) -> U8Vector4: ...
    @property
    def ogla(self) -> U8Vector4: ...
    @property
    def oglo(self) -> U8Vector4: ...
    @property
    def ogll(self) -> U8Vector4: ...
    @property
    def obrr(self) -> U8Vector4: ...
    @property
    def obrg(self) -> U8Vector4: ...
    @property
    def obrb(self) -> U8Vector4: ...
    @property
    def obra(self) -> U8Vector4: ...
    @property
    def obro(self) -> U8Vector4: ...
    @property
    def obrl(self) -> U8Vector4: ...
    @property
    def obgr(self) -> U8Vector4: ...
    @property
    def obgg(self) -> U8Vector4: ...
    @property
    def obgb(self) -> U8Vector4: ...
    @property
    def obga(self) -> U8Vector4: ...
    @property
    def obgo(self) -> U8Vector4: ...
    @property
    def obgl(self) -> U8Vector4: ...
    @property
    def obbr(self) -> U8Vector4: ...
    @property
    def obbg(self) -> U8Vector4: ...
    @property
    def obbb(self) -> U8Vector4: ...
    @property
    def obba(self) -> U8Vector4: ...
    @property
    def obbo(self) -> U8Vector4: ...
    @property
    def obbl(self) -> U8Vector4: ...
    @property
    def obar(self) -> U8Vector4: ...
    @property
    def obag(self) -> U8Vector4: ...
    @property
    def obab(self) -> U8Vector4: ...
    @property
    def obaa(self) -> U8Vector4: ...
    @property
    def obao(self) -> U8Vector4: ...
    @property
    def obal(self) -> U8Vector4: ...
    @property
    def obor(self) -> U8Vector4: ...
    @property
    def obog(self) -> U8Vector4: ...
    @property
    def obob(self) -> U8Vector4: ...
    @property
    def oboa(self) -> U8Vector4: ...
    @property
    def oboo(self) -> U8Vector4: ...
    @property
    def obol(self) -> U8Vector4: ...
    @property
    def oblr(self) -> U8Vector4: ...
    @property
    def oblg(self) -> U8Vector4: ...
    @property
    def oblb(self) -> U8Vector4: ...
    @property
    def obla(self) -> U8Vector4: ...
    @property
    def oblo(self) -> U8Vector4: ...
    @property
    def obll(self) -> U8Vector4: ...
    @property
    def oarr(self) -> U8Vector4: ...
    @property
    def oarg(self) -> U8Vector4: ...
    @property
    def oarb(self) -> U8Vector4: ...
    @property
    def oara(self) -> U8Vector4: ...
    @property
    def oaro(self) -> U8Vector4: ...
    @property
    def oarl(self) -> U8Vector4: ...
    @property
    def oagr(self) -> U8Vector4: ...
    @property
    def oagg(self) -> U8Vector4: ...
    @property
    def oagb(self) -> U8Vector4: ...
    @property
    def oaga(self) -> U8Vector4: ...
    @property
    def oago(self) -> U8Vector4: ...
    @property
    def oagl(self) -> U8Vector4: ...
    @property
    def oabr(self) -> U8Vector4: ...
    @property
    def oabg(self) -> U8Vector4: ...
    @property
    def oabb(self) -> U8Vector4: ...
    @property
    def oaba(self) -> U8Vector4: ...
    @property
    def oabo(self) -> U8Vector4: ...
    @property
    def oabl(self) -> U8Vector4: ...
    @property
    def oaar(self) -> U8Vector4: ...
    @property
    def oaag(self) -> U8Vector4: ...
    @property
    def oaab(self) -> U8Vector4: ...
    @property
    def oaaa(self) -> U8Vector4: ...
    @property
    def oaao(self) -> U8Vector4: ...
    @property
    def oaal(self) -> U8Vector4: ...
    @property
    def oaor(self) -> U8Vector4: ...
    @property
    def oaog(self) -> U8Vector4: ...
    @property
    def oaob(self) -> U8Vector4: ...
    @property
    def oaoa(self) -> U8Vector4: ...
    @property
    def oaoo(self) -> U8Vector4: ...
    @property
    def oaol(self) -> U8Vector4: ...
    @property
    def oalr(self) -> U8Vector4: ...
    @property
    def oalg(self) -> U8Vector4: ...
    @property
    def oalb(self) -> U8Vector4: ...
    @property
    def oala(self) -> U8Vector4: ...
    @property
    def oalo(self) -> U8Vector4: ...
    @property
    def oall(self) -> U8Vector4: ...
    @property
    def oorr(self) -> U8Vector4: ...
    @property
    def oorg(self) -> U8Vector4: ...
    @property
    def oorb(self) -> U8Vector4: ...
    @property
    def oora(self) -> U8Vector4: ...
    @property
    def ooro(self) -> U8Vector4: ...
    @property
    def oorl(self) -> U8Vector4: ...
    @property
    def oogr(self) -> U8Vector4: ...
    @property
    def oogg(self) -> U8Vector4: ...
    @property
    def oogb(self) -> U8Vector4: ...
    @property
    def ooga(self) -> U8Vector4: ...
    @property
    def oogo(self) -> U8Vector4: ...
    @property
    def oogl(self) -> U8Vector4: ...
    @property
    def oobr(self) -> U8Vector4: ...
    @property
    def oobg(self) -> U8Vector4: ...
    @property
    def oobb(self) -> U8Vector4: ...
    @property
    def ooba(self) -> U8Vector4: ...
    @property
    def oobo(self) -> U8Vector4: ...
    @property
    def oobl(self) -> U8Vector4: ...
    @property
    def ooar(self) -> U8Vector4: ...
    @property
    def ooag(self) -> U8Vector4: ...
    @property
    def ooab(self) -> U8Vector4: ...
    @property
    def ooaa(self) -> U8Vector4: ...
    @property
    def ooao(self) -> U8Vector4: ...
    @property
    def ooal(self) -> U8Vector4: ...
    @property
    def ooor(self) -> U8Vector4: ...
    @property
    def ooog(self) -> U8Vector4: ...
    @property
    def ooob(self) -> U8Vector4: ...
    @property
    def oooa(self) -> U8Vector4: ...
    @property
    def oolr(self) -> U8Vector4: ...
    @property
    def oolg(self) -> U8Vector4: ...
    @property
    def oolb(self) -> U8Vector4: ...
    @property
    def oola(self) -> U8Vector4: ...
    @property
    def olrr(self) -> U8Vector4: ...
    @property
    def olrg(self) -> U8Vector4: ...
    @property
    def olrb(self) -> U8Vector4: ...
    @property
    def olra(self) -> U8Vector4: ...
    @property
    def olro(self) -> U8Vector4: ...
    @property
    def olrl(self) -> U8Vector4: ...
    @property
    def olgr(self) -> U8Vector4: ...
    @property
    def olgg(self) -> U8Vector4: ...
    @property
    def olgb(self) -> U8Vector4: ...
    @property
    def olga(self) -> U8Vector4: ...
    @property
    def olgo(self) -> U8Vector4: ...
    @property
    def olgl(self) -> U8Vector4: ...
    @property
    def olbr(self) -> U8Vector4: ...
    @property
    def olbg(self) -> U8Vector4: ...
    @property
    def olbb(self) -> U8Vector4: ...
    @property
    def olba(self) -> U8Vector4: ...
    @property
    def olbo(self) -> U8Vector4: ...
    @property
    def olbl(self) -> U8Vector4: ...
    @property
    def olar(self) -> U8Vector4: ...
    @property
    def olag(self) -> U8Vector4: ...
    @property
    def olab(self) -> U8Vector4: ...
    @property
    def olaa(self) -> U8Vector4: ...
    @property
    def olao(self) -> U8Vector4: ...
    @property
    def olal(self) -> U8Vector4: ...
    @property
    def olor(self) -> U8Vector4: ...
    @property
    def olog(self) -> U8Vector4: ...
    @property
    def olob(self) -> U8Vector4: ...
    @property
    def oloa(self) -> U8Vector4: ...
    @property
    def ollr(self) -> U8Vector4: ...
    @property
    def ollg(self) -> U8Vector4: ...
    @property
    def ollb(self) -> U8Vector4: ...
    @property
    def olla(self) -> U8Vector4: ...
    @property
    def lrrr(self) -> U8Vector4: ...
    @property
    def lrrg(self) -> U8Vector4: ...
    @property
    def lrrb(self) -> U8Vector4: ...
    @property
    def lrra(self) -> U8Vector4: ...
    @property
    def lrro(self) -> U8Vector4: ...
    @property
    def lrrl(self) -> U8Vector4: ...
    @property
    def lrgr(self) -> U8Vector4: ...
    @property
    def lrgg(self) -> U8Vector4: ...
    @property
    def lrgb(self) -> U8Vector4: ...
    @property
    def lrga(self) -> U8Vector4: ...
    @property
    def lrgo(self) -> U8Vector4: ...
    @property
    def lrgl(self) -> U8Vector4: ...
    @property
    def lrbr(self) -> U8Vector4: ...
    @property
    def lrbg(self) -> U8Vector4: ...
    @property
    def lrbb(self) -> U8Vector4: ...
    @property
    def lrba(self) -> U8Vector4: ...
    @property
    def lrbo(self) -> U8Vector4: ...
    @property
    def lrbl(self) -> U8Vector4: ...
    @property
    def lrar(self) -> U8Vector4: ...
    @property
    def lrag(self) -> U8Vector4: ...
    @property
    def lrab(self) -> U8Vector4: ...
    @property
    def lraa(self) -> U8Vector4: ...
    @property
    def lrao(self) -> U8Vector4: ...
    @property
    def lral(self) -> U8Vector4: ...
    @property
    def lror(self) -> U8Vector4: ...
    @property
    def lrog(self) -> U8Vector4: ...
    @property
    def lrob(self) -> U8Vector4: ...
    @property
    def lroa(self) -> U8Vector4: ...
    @property
    def lroo(self) -> U8Vector4: ...
    @property
    def lrol(self) -> U8Vector4: ...
    @property
    def lrlr(self) -> U8Vector4: ...
    @property
    def lrlg(self) -> U8Vector4: ...
    @property
    def lrlb(self) -> U8Vector4: ...
    @property
    def lrla(self) -> U8Vector4: ...
    @property
    def lrlo(self) -> U8Vector4: ...
    @property
    def lrll(self) -> U8Vector4: ...
    @property
    def lgrr(self) -> U8Vector4: ...
    @property
    def lgrg(self) -> U8Vector4: ...
    @property
    def lgrb(self) -> U8Vector4: ...
    @property
    def lgra(self) -> U8Vector4: ...
    @property
    def lgro(self) -> U8Vector4: ...
    @property
    def lgrl(self) -> U8Vector4: ...
    @property
    def lggr(self) -> U8Vector4: ...
    @property
    def lggg(self) -> U8Vector4: ...
    @property
    def lggb(self) -> U8Vector4: ...
    @property
    def lgga(self) -> U8Vector4: ...
    @property
    def lggo(self) -> U8Vector4: ...
    @property
    def lggl(self) -> U8Vector4: ...
    @property
    def lgbr(self) -> U8Vector4: ...
    @property
    def lgbg(self) -> U8Vector4: ...
    @property
    def lgbb(self) -> U8Vector4: ...
    @property
    def lgba(self) -> U8Vector4: ...
    @property
    def lgbo(self) -> U8Vector4: ...
    @property
    def lgbl(self) -> U8Vector4: ...
    @property
    def lgar(self) -> U8Vector4: ...
    @property
    def lgag(self) -> U8Vector4: ...
    @property
    def lgab(self) -> U8Vector4: ...
    @property
    def lgaa(self) -> U8Vector4: ...
    @property
    def lgao(self) -> U8Vector4: ...
    @property
    def lgal(self) -> U8Vector4: ...
    @property
    def lgor(self) -> U8Vector4: ...
    @property
    def lgog(self) -> U8Vector4: ...
    @property
    def lgob(self) -> U8Vector4: ...
    @property
    def lgoa(self) -> U8Vector4: ...
    @property
    def lgoo(self) -> U8Vector4: ...
    @property
    def lgol(self) -> U8Vector4: ...
    @property
    def lglr(self) -> U8Vector4: ...
    @property
    def lglg(self) -> U8Vector4: ...
    @property
    def lglb(self) -> U8Vector4: ...
    @property
    def lgla(self) -> U8Vector4: ...
    @property
    def lglo(self) -> U8Vector4: ...
    @property
    def lgll(self) -> U8Vector4: ...
    @property
    def lbrr(self) -> U8Vector4: ...
    @property
    def lbrg(self) -> U8Vector4: ...
    @property
    def lbrb(self) -> U8Vector4: ...
    @property
    def lbra(self) -> U8Vector4: ...
    @property
    def lbro(self) -> U8Vector4: ...
    @property
    def lbrl(self) -> U8Vector4: ...
    @property
    def lbgr(self) -> U8Vector4: ...
    @property
    def lbgg(self) -> U8Vector4: ...
    @property
    def lbgb(self) -> U8Vector4: ...
    @property
    def lbga(self) -> U8Vector4: ...
    @property
    def lbgo(self) -> U8Vector4: ...
    @property
    def lbgl(self) -> U8Vector4: ...
    @property
    def lbbr(self) -> U8Vector4: ...
    @property
    def lbbg(self) -> U8Vector4: ...
    @property
    def lbbb(self) -> U8Vector4: ...
    @property
    def lbba(self) -> U8Vector4: ...
    @property
    def lbbo(self) -> U8Vector4: ...
    @property
    def lbbl(self) -> U8Vector4: ...
    @property
    def lbar(self) -> U8Vector4: ...
    @property
    def lbag(self) -> U8Vector4: ...
    @property
    def lbab(self) -> U8Vector4: ...
    @property
    def lbaa(self) -> U8Vector4: ...
    @property
    def lbao(self) -> U8Vector4: ...
    @property
    def lbal(self) -> U8Vector4: ...
    @property
    def lbor(self) -> U8Vector4: ...
    @property
    def lbog(self) -> U8Vector4: ...
    @property
    def lbob(self) -> U8Vector4: ...
    @property
    def lboa(self) -> U8Vector4: ...
    @property
    def lboo(self) -> U8Vector4: ...
    @property
    def lbol(self) -> U8Vector4: ...
    @property
    def lblr(self) -> U8Vector4: ...
    @property
    def lblg(self) -> U8Vector4: ...
    @property
    def lblb(self) -> U8Vector4: ...
    @property
    def lbla(self) -> U8Vector4: ...
    @property
    def lblo(self) -> U8Vector4: ...
    @property
    def lbll(self) -> U8Vector4: ...
    @property
    def larr(self) -> U8Vector4: ...
    @property
    def larg(self) -> U8Vector4: ...
    @property
    def larb(self) -> U8Vector4: ...
    @property
    def lara(self) -> U8Vector4: ...
    @property
    def laro(self) -> U8Vector4: ...
    @property
    def larl(self) -> U8Vector4: ...
    @property
    def lagr(self) -> U8Vector4: ...
    @property
    def lagg(self) -> U8Vector4: ...
    @property
    def lagb(self) -> U8Vector4: ...
    @property
    def laga(self) -> U8Vector4: ...
    @property
    def lago(self) -> U8Vector4: ...
    @property
    def lagl(self) -> U8Vector4: ...
    @property
    def labr(self) -> U8Vector4: ...
    @property
    def labg(self) -> U8Vector4: ...
    @property
    def labb(self) -> U8Vector4: ...
    @property
    def laba(self) -> U8Vector4: ...
    @property
    def labo(self) -> U8Vector4: ...
    @property
    def labl(self) -> U8Vector4: ...
    @property
    def laar(self) -> U8Vector4: ...
    @property
    def laag(self) -> U8Vector4: ...
    @property
    def laab(self) -> U8Vector4: ...
    @property
    def laaa(self) -> U8Vector4: ...
    @property
    def laao(self) -> U8Vector4: ...
    @property
    def laal(self) -> U8Vector4: ...
    @property
    def laor(self) -> U8Vector4: ...
    @property
    def laog(self) -> U8Vector4: ...
    @property
    def laob(self) -> U8Vector4: ...
    @property
    def laoa(self) -> U8Vector4: ...
    @property
    def laoo(self) -> U8Vector4: ...
    @property
    def laol(self) -> U8Vector4: ...
    @property
    def lalr(self) -> U8Vector4: ...
    @property
    def lalg(self) -> U8Vector4: ...
    @property
    def lalb(self) -> U8Vector4: ...
    @property
    def lala(self) -> U8Vector4: ...
    @property
    def lalo(self) -> U8Vector4: ...
    @property
    def lall(self) -> U8Vector4: ...
    @property
    def lorr(self) -> U8Vector4: ...
    @property
    def lorg(self) -> U8Vector4: ...
    @property
    def lorb(self) -> U8Vector4: ...
    @property
    def lora(self) -> U8Vector4: ...
    @property
    def loro(self) -> U8Vector4: ...
    @property
    def lorl(self) -> U8Vector4: ...
    @property
    def logr(self) -> U8Vector4: ...
    @property
    def logg(self) -> U8Vector4: ...
    @property
    def logb(self) -> U8Vector4: ...
    @property
    def loga(self) -> U8Vector4: ...
    @property
    def logo(self) -> U8Vector4: ...
    @property
    def logl(self) -> U8Vector4: ...
    @property
    def lobr(self) -> U8Vector4: ...
    @property
    def lobg(self) -> U8Vector4: ...
    @property
    def lobb(self) -> U8Vector4: ...
    @property
    def loba(self) -> U8Vector4: ...
    @property
    def lobo(self) -> U8Vector4: ...
    @property
    def lobl(self) -> U8Vector4: ...
    @property
    def loar(self) -> U8Vector4: ...
    @property
    def loag(self) -> U8Vector4: ...
    @property
    def loab(self) -> U8Vector4: ...
    @property
    def loaa(self) -> U8Vector4: ...
    @property
    def loao(self) -> U8Vector4: ...
    @property
    def loal(self) -> U8Vector4: ...
    @property
    def loor(self) -> U8Vector4: ...
    @property
    def loog(self) -> U8Vector4: ...
    @property
    def loob(self) -> U8Vector4: ...
    @property
    def looa(self) -> U8Vector4: ...
    @property
    def lolr(self) -> U8Vector4: ...
    @property
    def lolg(self) -> U8Vector4: ...
    @property
    def lolb(self) -> U8Vector4: ...
    @property
    def lola(self) -> U8Vector4: ...
    @property
    def llrr(self) -> U8Vector4: ...
    @property
    def llrg(self) -> U8Vector4: ...
    @property
    def llrb(self) -> U8Vector4: ...
    @property
    def llra(self) -> U8Vector4: ...
    @property
    def llro(self) -> U8Vector4: ...
    @property
    def llrl(self) -> U8Vector4: ...
    @property
    def llgr(self) -> U8Vector4: ...
    @property
    def llgg(self) -> U8Vector4: ...
    @property
    def llgb(self) -> U8Vector4: ...
    @property
    def llga(self) -> U8Vector4: ...
    @property
    def llgo(self) -> U8Vector4: ...
    @property
    def llgl(self) -> U8Vector4: ...
    @property
    def llbr(self) -> U8Vector4: ...
    @property
    def llbg(self) -> U8Vector4: ...
    @property
    def llbb(self) -> U8Vector4: ...
    @property
    def llba(self) -> U8Vector4: ...
    @property
    def llbo(self) -> U8Vector4: ...
    @property
    def llbl(self) -> U8Vector4: ...
    @property
    def llar(self) -> U8Vector4: ...
    @property
    def llag(self) -> U8Vector4: ...
    @property
    def llab(self) -> U8Vector4: ...
    @property
    def llaa(self) -> U8Vector4: ...
    @property
    def llao(self) -> U8Vector4: ...
    @property
    def llal(self) -> U8Vector4: ...
    @property
    def llor(self) -> U8Vector4: ...
    @property
    def llog(self) -> U8Vector4: ...
    @property
    def llob(self) -> U8Vector4: ...
    @property
    def lloa(self) -> U8Vector4: ...
    @property
    def lllr(self) -> U8Vector4: ...
    @property
    def lllg(self) -> U8Vector4: ...
    @property
    def lllb(self) -> U8Vector4: ...
    @property
    def llla(self) -> U8Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> U8Vector2: ...
    @property
    def st(self) -> U8Vector2: ...
    @property
    def sq(self) -> U8Vector2: ...
    @property
    def sp(self) -> U8Vector2: ...
    @property
    def so(self) -> U8Vector2: ...
    @property
    def sl(self) -> U8Vector2: ...
    @property
    def ts(self) -> U8Vector2: ...
    @property
    def tt(self) -> U8Vector2: ...
    @property
    def tq(self) -> U8Vector2: ...
    @property
    def tp(self) -> U8Vector2: ...
    @property
    def to(self) -> U8Vector2: ...
    @property
    def tl(self) -> U8Vector2: ...
    @property
    def qs(self) -> U8Vector2: ...
    @property
    def qt(self) -> U8Vector2: ...
    @property
    def qq(self) -> U8Vector2: ...
    @property
    def qp(self) -> U8Vector2: ...
    @property
    def qo(self) -> U8Vector2: ...
    @property
    def ql(self) -> U8Vector2: ...
    @property
    def ps(self) -> U8Vector2: ...
    @property
    def pt(self) -> U8Vector2: ...
    @property
    def pq(self) -> U8Vector2: ...
    @property
    def pp(self) -> U8Vector2: ...
    @property
    def po(self) -> U8Vector2: ...
    @property
    def pl(self) -> U8Vector2: ...
    @property
    def os(self) -> U8Vector2: ...
    @property
    def ot(self) -> U8Vector2: ...
    @property
    def oq(self) -> U8Vector2: ...
    @property
    def op(self) -> U8Vector2: ...
    @property
    def ls(self) -> U8Vector2: ...
    @property
    def lt(self) -> U8Vector2: ...
    @property
    def lq(self) -> U8Vector2: ...
    @property
    def lp(self) -> U8Vector2: ...
    @property
    def sss(self) -> U8Vector3: ...
    @property
    def sst(self) -> U8Vector3: ...
    @property
    def ssq(self) -> U8Vector3: ...
    @property
    def ssp(self) -> U8Vector3: ...
    @property
    def sso(self) -> U8Vector3: ...
    @property
    def ssl(self) -> U8Vector3: ...
    @property
    def sts(self) -> U8Vector3: ...
    @property
    def stt(self) -> U8Vector3: ...
    @property
    def stq(self) -> U8Vector3: ...
    @property
    def stp(self) -> U8Vector3: ...
    @property
    def sto(self) -> U8Vector3: ...
    @property
    def stl(self) -> U8Vector3: ...
    @property
    def sqs(self) -> U8Vector3: ...
    @property
    def sqt(self) -> U8Vector3: ...
    @property
    def sqq(self) -> U8Vector3: ...
    @property
    def sqp(self) -> U8Vector3: ...
    @property
    def sqo(self) -> U8Vector3: ...
    @property
    def sql(self) -> U8Vector3: ...
    @property
    def sps(self) -> U8Vector3: ...
    @property
    def spt(self) -> U8Vector3: ...
    @property
    def spq(self) -> U8Vector3: ...
    @property
    def spp(self) -> U8Vector3: ...
    @property
    def spo(self) -> U8Vector3: ...
    @property
    def spl(self) -> U8Vector3: ...
    @property
    def sos(self) -> U8Vector3: ...
    @property
    def sot(self) -> U8Vector3: ...
    @property
    def soq(self) -> U8Vector3: ...
    @property
    def sop(self) -> U8Vector3: ...
    @property
    def soo(self) -> U8Vector3: ...
    @property
    def sol(self) -> U8Vector3: ...
    @property
    def sls(self) -> U8Vector3: ...
    @property
    def slt(self) -> U8Vector3: ...
    @property
    def slq(self) -> U8Vector3: ...
    @property
    def slp(self) -> U8Vector3: ...
    @property
    def slo(self) -> U8Vector3: ...
    @property
    def sll(self) -> U8Vector3: ...
    @property
    def tss(self) -> U8Vector3: ...
    @property
    def tst(self) -> U8Vector3: ...
    @property
    def tsq(self) -> U8Vector3: ...
    @property
    def tsp(self) -> U8Vector3: ...
    @property
    def tso(self) -> U8Vector3: ...
    @property
    def tsl(self) -> U8Vector3: ...
    @property
    def tts(self) -> U8Vector3: ...
    @property
    def ttt(self) -> U8Vector3: ...
    @property
    def ttq(self) -> U8Vector3: ...
    @property
    def ttp(self) -> U8Vector3: ...
    @property
    def tto(self) -> U8Vector3: ...
    @property
    def ttl(self) -> U8Vector3: ...
    @property
    def tqs(self) -> U8Vector3: ...
    @property
    def tqt(self) -> U8Vector3: ...
    @property
    def tqq(self) -> U8Vector3: ...
    @property
    def tqp(self) -> U8Vector3: ...
    @property
    def tqo(self) -> U8Vector3: ...
    @property
    def tql(self) -> U8Vector3: ...
    @property
    def tps(self) -> U8Vector3: ...
    @property
    def tpt(self) -> U8Vector3: ...
    @property
    def tpq(self) -> U8Vector3: ...
    @property
    def tpp(self) -> U8Vector3: ...
    @property
    def tpo(self) -> U8Vector3: ...
    @property
    def tpl(self) -> U8Vector3: ...
    @property
    def tos(self) -> U8Vector3: ...
    @property
    def tot(self) -> U8Vector3: ...
    @property
    def toq(self) -> U8Vector3: ...
    @property
    def top(self) -> U8Vector3: ...
    @property
    def too(self) -> U8Vector3: ...
    @property
    def tol(self) -> U8Vector3: ...
    @property
    def tls(self) -> U8Vector3: ...
    @property
    def tlt(self) -> U8Vector3: ...
    @property
    def tlq(self) -> U8Vector3: ...
    @property
    def tlp(self) -> U8Vector3: ...
    @property
    def tlo(self) -> U8Vector3: ...
    @property
    def tll(self) -> U8Vector3: ...
    @property
    def qss(self) -> U8Vector3: ...
    @property
    def qst(self) -> U8Vector3: ...
    @property
    def qsq(self) -> U8Vector3: ...
    @property
    def qsp(self) -> U8Vector3: ...
    @property
    def qso(self) -> U8Vector3: ...
    @property
    def qsl(self) -> U8Vector3: ...
    @property
    def qts(self) -> U8Vector3: ...
    @property
    def qtt(self) -> U8Vector3: ...
    @property
    def qtq(self) -> U8Vector3: ...
    @property
    def qtp(self) -> U8Vector3: ...
    @property
    def qto(self) -> U8Vector3: ...
    @property
    def qtl(self) -> U8Vector3: ...
    @property
    def qqs(self) -> U8Vector3: ...
    @property
    def qqt(self) -> U8Vector3: ...
    @property
    def qqq(self) -> U8Vector3: ...
    @property
    def qqp(self) -> U8Vector3: ...
    @property
    def qqo(self) -> U8Vector3: ...
    @property
    def qql(self) -> U8Vector3: ...
    @property
    def qps(self) -> U8Vector3: ...
    @property
    def qpt(self) -> U8Vector3: ...
    @property
    def qpq(self) -> U8Vector3: ...
    @property
    def qpp(self) -> U8Vector3: ...
    @property
    def qpo(self) -> U8Vector3: ...
    @property
    def qpl(self) -> U8Vector3: ...
    @property
    def qos(self) -> U8Vector3: ...
    @property
    def qot(self) -> U8Vector3: ...
    @property
    def qoq(self) -> U8Vector3: ...
    @property
    def qop(self) -> U8Vector3: ...
    @property
    def qoo(self) -> U8Vector3: ...
    @property
    def qol(self) -> U8Vector3: ...
    @property
    def qls(self) -> U8Vector3: ...
    @property
    def qlt(self) -> U8Vector3: ...
    @property
    def qlq(self) -> U8Vector3: ...
    @property
    def qlp(self) -> U8Vector3: ...
    @property
    def qlo(self) -> U8Vector3: ...
    @property
    def qll(self) -> U8Vector3: ...
    @property
    def pss(self) -> U8Vector3: ...
    @property
    def pst(self) -> U8Vector3: ...
    @property
    def psq(self) -> U8Vector3: ...
    @property
    def psp(self) -> U8Vector3: ...
    @property
    def pso(self) -> U8Vector3: ...
    @property
    def psl(self) -> U8Vector3: ...
    @property
    def pts(self) -> U8Vector3: ...
    @property
    def ptt(self) -> U8Vector3: ...
    @property
    def ptq(self) -> U8Vector3: ...
    @property
    def ptp(self) -> U8Vector3: ...
    @property
    def pto(self) -> U8Vector3: ...
    @property
    def ptl(self) -> U8Vector3: ...
    @property
    def pqs(self) -> U8Vector3: ...
    @property
    def pqt(self) -> U8Vector3: ...
    @property
    def pqq(self) -> U8Vector3: ...
    @property
    def pqp(self) -> U8Vector3: ...
    @property
    def pqo(self) -> U8Vector3: ...
    @property
    def pql(self) -> U8Vector3: ...
    @property
    def pps(self) -> U8Vector3: ...
    @property
    def ppt(self) -> U8Vector3: ...
    @property
    def ppq(self) -> U8Vector3: ...
    @property
    def ppp(self) -> U8Vector3: ...
    @property
    def ppo(self) -> U8Vector3: ...
    @property
    def ppl(self) -> U8Vector3: ...
    @property
    def pos(self) -> U8Vector3: ...
    @property
    def pot(self) -> U8Vector3: ...
    @property
    def poq(self) -> U8Vector3: ...
    @property
    def pop(self) -> U8Vector3: ...
    @property
    def poo(self) -> U8Vector3: ...
    @property
    def pol(self) -> U8Vector3: ...
    @property
    def pls(self) -> U8Vector3: ...
    @property
    def plt(self) -> U8Vector3: ...
    @property
    def plq(self) -> U8Vector3: ...
    @property
    def plp(self) -> U8Vector3: ...
    @property
    def plo(self) -> U8Vector3: ...
    @property
    def pll(self) -> U8Vector3: ...
    @property
    def oss(self) -> U8Vector3: ...
    @property
    def ost(self) -> U8Vector3: ...
    @property
    def osq(self) -> U8Vector3: ...
    @property
    def osp(self) -> U8Vector3: ...
    @property
    def oso(self) -> U8Vector3: ...
    @property
    def osl(self) -> U8Vector3: ...
    @property
    def ots(self) -> U8Vector3: ...
    @property
    def ott(self) -> U8Vector3: ...
    @property
    def otq(self) -> U8Vector3: ...
    @property
    def otp(self) -> U8Vector3: ...
    @property
    def oto(self) -> U8Vector3: ...
    @property
    def otl(self) -> U8Vector3: ...
    @property
    def oqs(self) -> U8Vector3: ...
    @property
    def oqt(self) -> U8Vector3: ...
    @property
    def oqq(self) -> U8Vector3: ...
    @property
    def oqp(self) -> U8Vector3: ...
    @property
    def oqo(self) -> U8Vector3: ...
    @property
    def oql(self) -> U8Vector3: ...
    @property
    def ops(self) -> U8Vector3: ...
    @property
    def opt(self) -> U8Vector3: ...
    @property
    def opq(self) -> U8Vector3: ...
    @property
    def opp(self) -> U8Vector3: ...
    @property
    def opo(self) -> U8Vector3: ...
    @property
    def opl(self) -> U8Vector3: ...
    @property
    def oos(self) -> U8Vector3: ...
    @property
    def oot(self) -> U8Vector3: ...
    @property
    def ooq(self) -> U8Vector3: ...
    @property
    def oop(self) -> U8Vector3: ...
    @property
    def ols(self) -> U8Vector3: ...
    @property
    def olt(self) -> U8Vector3: ...
    @property
    def olq(self) -> U8Vector3: ...
    @property
    def olp(self) -> U8Vector3: ...
    @property
    def lss(self) -> U8Vector3: ...
    @property
    def lst(self) -> U8Vector3: ...
    @property
    def lsq(self) -> U8Vector3: ...
    @property
    def lsp(self) -> U8Vector3: ...
    @property
    def lso(self) -> U8Vector3: ...
    @property
    def lsl(self) -> U8Vector3: ...
    @property
    def lts(self) -> U8Vector3: ...
    @property
    def ltt(self) -> U8Vector3: ...
    @property
    def ltq(self) -> U8Vector3: ...
    @property
    def ltp(self) -> U8Vector3: ...
    @property
    def lto(self) -> U8Vector3: ...
    @property
    def ltl(self) -> U8Vector3: ...
    @property
    def lqs(self) -> U8Vector3: ...
    @property
    def lqt(self) -> U8Vector3: ...
    @property
    def lqq(self) -> U8Vector3: ...
    @property
    def lqp(self) -> U8Vector3: ...
    @property
    def lqo(self) -> U8Vector3: ...
    @property
    def lql(self) -> U8Vector3: ...
    @property
    def lps(self) -> U8Vector3: ...
    @property
    def lpt(self) -> U8Vector3: ...
    @property
    def lpq(self) -> U8Vector3: ...
    @property
    def lpp(self) -> U8Vector3: ...
    @property
    def lpo(self) -> U8Vector3: ...
    @property
    def lpl(self) -> U8Vector3: ...
    @property
    def los(self) -> U8Vector3: ...
    @property
    def lot(self) -> U8Vector3: ...
    @property
    def loq(self) -> U8Vector3: ...
    @property
    def lop(self) -> U8Vector3: ...
    @property
    def lls(self) -> U8Vector3: ...
    @property
    def llt(self) -> U8Vector3: ...
    @property
    def llq(self) -> U8Vector3: ...
    @property
    def llp(self) -> U8Vector3: ...
    @property
    def ssss(self) -> U8Vector4: ...
    @property
    def ssst(self) -> U8Vector4: ...
    @property
    def sssq(self) -> U8Vector4: ...
    @property
    def sssp(self) -> U8Vector4: ...
    @property
    def ssso(self) -> U8Vector4: ...
    @property
    def sssl(self) -> U8Vector4: ...
    @property
    def ssts(self) -> U8Vector4: ...
    @property
    def sstt(self) -> U8Vector4: ...
    @property
    def sstq(self) -> U8Vector4: ...
    @property
    def sstp(self) -> U8Vector4: ...
    @property
    def ssto(self) -> U8Vector4: ...
    @property
    def sstl(self) -> U8Vector4: ...
    @property
    def ssqs(self) -> U8Vector4: ...
    @property
    def ssqt(self) -> U8Vector4: ...
    @property
    def ssqq(self) -> U8Vector4: ...
    @property
    def ssqp(self) -> U8Vector4: ...
    @property
    def ssqo(self) -> U8Vector4: ...
    @property
    def ssql(self) -> U8Vector4: ...
    @property
    def ssps(self) -> U8Vector4: ...
    @property
    def sspt(self) -> U8Vector4: ...
    @property
    def sspq(self) -> U8Vector4: ...
    @property
    def sspp(self) -> U8Vector4: ...
    @property
    def sspo(self) -> U8Vector4: ...
    @property
    def sspl(self) -> U8Vector4: ...
    @property
    def ssos(self) -> U8Vector4: ...
    @property
    def ssot(self) -> U8Vector4: ...
    @property
    def ssoq(self) -> U8Vector4: ...
    @property
    def ssop(self) -> U8Vector4: ...
    @property
    def ssoo(self) -> U8Vector4: ...
    @property
    def ssol(self) -> U8Vector4: ...
    @property
    def ssls(self) -> U8Vector4: ...
    @property
    def sslt(self) -> U8Vector4: ...
    @property
    def sslq(self) -> U8Vector4: ...
    @property
    def sslp(self) -> U8Vector4: ...
    @property
    def sslo(self) -> U8Vector4: ...
    @property
    def ssll(self) -> U8Vector4: ...
    @property
    def stss(self) -> U8Vector4: ...
    @property
    def stst(self) -> U8Vector4: ...
    @property
    def stsq(self) -> U8Vector4: ...
    @property
    def stsp(self) -> U8Vector4: ...
    @property
    def stso(self) -> U8Vector4: ...
    @property
    def stsl(self) -> U8Vector4: ...
    @property
    def stts(self) -> U8Vector4: ...
    @property
    def sttt(self) -> U8Vector4: ...
    @property
    def sttq(self) -> U8Vector4: ...
    @property
    def sttp(self) -> U8Vector4: ...
    @property
    def stto(self) -> U8Vector4: ...
    @property
    def sttl(self) -> U8Vector4: ...
    @property
    def stqs(self) -> U8Vector4: ...
    @property
    def stqt(self) -> U8Vector4: ...
    @property
    def stqq(self) -> U8Vector4: ...
    @property
    def stqp(self) -> U8Vector4: ...
    @property
    def stqo(self) -> U8Vector4: ...
    @property
    def stql(self) -> U8Vector4: ...
    @property
    def stps(self) -> U8Vector4: ...
    @property
    def stpt(self) -> U8Vector4: ...
    @property
    def stpq(self) -> U8Vector4: ...
    @property
    def stpp(self) -> U8Vector4: ...
    @property
    def stpo(self) -> U8Vector4: ...
    @property
    def stpl(self) -> U8Vector4: ...
    @property
    def stos(self) -> U8Vector4: ...
    @property
    def stot(self) -> U8Vector4: ...
    @property
    def stoq(self) -> U8Vector4: ...
    @property
    def stop(self) -> U8Vector4: ...
    @property
    def stoo(self) -> U8Vector4: ...
    @property
    def stol(self) -> U8Vector4: ...
    @property
    def stls(self) -> U8Vector4: ...
    @property
    def stlt(self) -> U8Vector4: ...
    @property
    def stlq(self) -> U8Vector4: ...
    @property
    def stlp(self) -> U8Vector4: ...
    @property
    def stlo(self) -> U8Vector4: ...
    @property
    def stll(self) -> U8Vector4: ...
    @property
    def sqss(self) -> U8Vector4: ...
    @property
    def sqst(self) -> U8Vector4: ...
    @property
    def sqsq(self) -> U8Vector4: ...
    @property
    def sqsp(self) -> U8Vector4: ...
    @property
    def sqso(self) -> U8Vector4: ...
    @property
    def sqsl(self) -> U8Vector4: ...
    @property
    def sqts(self) -> U8Vector4: ...
    @property
    def sqtt(self) -> U8Vector4: ...
    @property
    def sqtq(self) -> U8Vector4: ...
    @property
    def sqtp(self) -> U8Vector4: ...
    @property
    def sqto(self) -> U8Vector4: ...
    @property
    def sqtl(self) -> U8Vector4: ...
    @property
    def sqqs(self) -> U8Vector4: ...
    @property
    def sqqt(self) -> U8Vector4: ...
    @property
    def sqqq(self) -> U8Vector4: ...
    @property
    def sqqp(self) -> U8Vector4: ...
    @property
    def sqqo(self) -> U8Vector4: ...
    @property
    def sqql(self) -> U8Vector4: ...
    @property
    def sqps(self) -> U8Vector4: ...
    @property
    def sqpt(self) -> U8Vector4: ...
    @property
    def sqpq(self) -> U8Vector4: ...
    @property
    def sqpp(self) -> U8Vector4: ...
    @property
    def sqpo(self) -> U8Vector4: ...
    @property
    def sqpl(self) -> U8Vector4: ...
    @property
    def sqos(self) -> U8Vector4: ...
    @property
    def sqot(self) -> U8Vector4: ...
    @property
    def sqoq(self) -> U8Vector4: ...
    @property
    def sqop(self) -> U8Vector4: ...
    @property
    def sqoo(self) -> U8Vector4: ...
    @property
    def sqol(self) -> U8Vector4: ...
    @property
    def sqls(self) -> U8Vector4: ...
    @property
    def sqlt(self) -> U8Vector4: ...
    @property
    def sqlq(self) -> U8Vector4: ...
    @property
    def sqlp(self) -> U8Vector4: ...
    @property
    def sqlo(self) -> U8Vector4: ...
    @property
    def sqll(self) -> U8Vector4: ...
    @property
    def spss(self) -> U8Vector4: ...
    @property
    def spst(self) -> U8Vector4: ...
    @property
    def spsq(self) -> U8Vector4: ...
    @property
    def spsp(self) -> U8Vector4: ...
    @property
    def spso(self) -> U8Vector4: ...
    @property
    def spsl(self) -> U8Vector4: ...
    @property
    def spts(self) -> U8Vector4: ...
    @property
    def sptt(self) -> U8Vector4: ...
    @property
    def sptq(self) -> U8Vector4: ...
    @property
    def sptp(self) -> U8Vector4: ...
    @property
    def spto(self) -> U8Vector4: ...
    @property
    def sptl(self) -> U8Vector4: ...
    @property
    def spqs(self) -> U8Vector4: ...
    @property
    def spqt(self) -> U8Vector4: ...
    @property
    def spqq(self) -> U8Vector4: ...
    @property
    def spqp(self) -> U8Vector4: ...
    @property
    def spqo(self) -> U8Vector4: ...
    @property
    def spql(self) -> U8Vector4: ...
    @property
    def spps(self) -> U8Vector4: ...
    @property
    def sppt(self) -> U8Vector4: ...
    @property
    def sppq(self) -> U8Vector4: ...
    @property
    def sppp(self) -> U8Vector4: ...
    @property
    def sppo(self) -> U8Vector4: ...
    @property
    def sppl(self) -> U8Vector4: ...
    @property
    def spos(self) -> U8Vector4: ...
    @property
    def spot(self) -> U8Vector4: ...
    @property
    def spoq(self) -> U8Vector4: ...
    @property
    def spop(self) -> U8Vector4: ...
    @property
    def spoo(self) -> U8Vector4: ...
    @property
    def spol(self) -> U8Vector4: ...
    @property
    def spls(self) -> U8Vector4: ...
    @property
    def splt(self) -> U8Vector4: ...
    @property
    def splq(self) -> U8Vector4: ...
    @property
    def splp(self) -> U8Vector4: ...
    @property
    def splo(self) -> U8Vector4: ...
    @property
    def spll(self) -> U8Vector4: ...
    @property
    def soss(self) -> U8Vector4: ...
    @property
    def sost(self) -> U8Vector4: ...
    @property
    def sosq(self) -> U8Vector4: ...
    @property
    def sosp(self) -> U8Vector4: ...
    @property
    def soso(self) -> U8Vector4: ...
    @property
    def sosl(self) -> U8Vector4: ...
    @property
    def sots(self) -> U8Vector4: ...
    @property
    def sott(self) -> U8Vector4: ...
    @property
    def sotq(self) -> U8Vector4: ...
    @property
    def sotp(self) -> U8Vector4: ...
    @property
    def soto(self) -> U8Vector4: ...
    @property
    def sotl(self) -> U8Vector4: ...
    @property
    def soqs(self) -> U8Vector4: ...
    @property
    def soqt(self) -> U8Vector4: ...
    @property
    def soqq(self) -> U8Vector4: ...
    @property
    def soqp(self) -> U8Vector4: ...
    @property
    def soqo(self) -> U8Vector4: ...
    @property
    def soql(self) -> U8Vector4: ...
    @property
    def sops(self) -> U8Vector4: ...
    @property
    def sopt(self) -> U8Vector4: ...
    @property
    def sopq(self) -> U8Vector4: ...
    @property
    def sopp(self) -> U8Vector4: ...
    @property
    def sopo(self) -> U8Vector4: ...
    @property
    def sopl(self) -> U8Vector4: ...
    @property
    def soos(self) -> U8Vector4: ...
    @property
    def soot(self) -> U8Vector4: ...
    @property
    def sooq(self) -> U8Vector4: ...
    @property
    def soop(self) -> U8Vector4: ...
    @property
    def sooo(self) -> U8Vector4: ...
    @property
    def sool(self) -> U8Vector4: ...
    @property
    def sols(self) -> U8Vector4: ...
    @property
    def solt(self) -> U8Vector4: ...
    @property
    def solq(self) -> U8Vector4: ...
    @property
    def solp(self) -> U8Vector4: ...
    @property
    def solo(self) -> U8Vector4: ...
    @property
    def soll(self) -> U8Vector4: ...
    @property
    def slss(self) -> U8Vector4: ...
    @property
    def slst(self) -> U8Vector4: ...
    @property
    def slsq(self) -> U8Vector4: ...
    @property
    def slsp(self) -> U8Vector4: ...
    @property
    def slso(self) -> U8Vector4: ...
    @property
    def slsl(self) -> U8Vector4: ...
    @property
    def slts(self) -> U8Vector4: ...
    @property
    def sltt(self) -> U8Vector4: ...
    @property
    def sltq(self) -> U8Vector4: ...
    @property
    def sltp(self) -> U8Vector4: ...
    @property
    def slto(self) -> U8Vector4: ...
    @property
    def sltl(self) -> U8Vector4: ...
    @property
    def slqs(self) -> U8Vector4: ...
    @property
    def slqt(self) -> U8Vector4: ...
    @property
    def slqq(self) -> U8Vector4: ...
    @property
    def slqp(self) -> U8Vector4: ...
    @property
    def slqo(self) -> U8Vector4: ...
    @property
    def slql(self) -> U8Vector4: ...
    @property
    def slps(self) -> U8Vector4: ...
    @property
    def slpt(self) -> U8Vector4: ...
    @property
    def slpq(self) -> U8Vector4: ...
    @property
    def slpp(self) -> U8Vector4: ...
    @property
    def slpo(self) -> U8Vector4: ...
    @property
    def slpl(self) -> U8Vector4: ...
    @property
    def slos(self) -> U8Vector4: ...
    @property
    def slot(self) -> U8Vector4: ...
    @property
    def sloq(self) -> U8Vector4: ...
    @property
    def slop(self) -> U8Vector4: ...
    @property
    def sloo(self) -> U8Vector4: ...
    @property
    def slol(self) -> U8Vector4: ...
    @property
    def slls(self) -> U8Vector4: ...
    @property
    def sllt(self) -> U8Vector4: ...
    @property
    def sllq(self) -> U8Vector4: ...
    @property
    def sllp(self) -> U8Vector4: ...
    @property
    def sllo(self) -> U8Vector4: ...
    @property
    def slll(self) -> U8Vector4: ...
    @property
    def tsss(self) -> U8Vector4: ...
    @property
    def tsst(self) -> U8Vector4: ...
    @property
    def tssq(self) -> U8Vector4: ...
    @property
    def tssp(self) -> U8Vector4: ...
    @property
    def tsso(self) -> U8Vector4: ...
    @property
    def tssl(self) -> U8Vector4: ...
    @property
    def tsts(self) -> U8Vector4: ...
    @property
    def tstt(self) -> U8Vector4: ...
    @property
    def tstq(self) -> U8Vector4: ...
    @property
    def tstp(self) -> U8Vector4: ...
    @property
    def tsto(self) -> U8Vector4: ...
    @property
    def tstl(self) -> U8Vector4: ...
    @property
    def tsqs(self) -> U8Vector4: ...
    @property
    def tsqt(self) -> U8Vector4: ...
    @property
    def tsqq(self) -> U8Vector4: ...
    @property
    def tsqp(self) -> U8Vector4: ...
    @property
    def tsqo(self) -> U8Vector4: ...
    @property
    def tsql(self) -> U8Vector4: ...
    @property
    def tsps(self) -> U8Vector4: ...
    @property
    def tspt(self) -> U8Vector4: ...
    @property
    def tspq(self) -> U8Vector4: ...
    @property
    def tspp(self) -> U8Vector4: ...
    @property
    def tspo(self) -> U8Vector4: ...
    @property
    def tspl(self) -> U8Vector4: ...
    @property
    def tsos(self) -> U8Vector4: ...
    @property
    def tsot(self) -> U8Vector4: ...
    @property
    def tsoq(self) -> U8Vector4: ...
    @property
    def tsop(self) -> U8Vector4: ...
    @property
    def tsoo(self) -> U8Vector4: ...
    @property
    def tsol(self) -> U8Vector4: ...
    @property
    def tsls(self) -> U8Vector4: ...
    @property
    def tslt(self) -> U8Vector4: ...
    @property
    def tslq(self) -> U8Vector4: ...
    @property
    def tslp(self) -> U8Vector4: ...
    @property
    def tslo(self) -> U8Vector4: ...
    @property
    def tsll(self) -> U8Vector4: ...
    @property
    def ttss(self) -> U8Vector4: ...
    @property
    def ttst(self) -> U8Vector4: ...
    @property
    def ttsq(self) -> U8Vector4: ...
    @property
    def ttsp(self) -> U8Vector4: ...
    @property
    def ttso(self) -> U8Vector4: ...
    @property
    def ttsl(self) -> U8Vector4: ...
    @property
    def ttts(self) -> U8Vector4: ...
    @property
    def tttt(self) -> U8Vector4: ...
    @property
    def tttq(self) -> U8Vector4: ...
    @property
    def tttp(self) -> U8Vector4: ...
    @property
    def ttto(self) -> U8Vector4: ...
    @property
    def tttl(self) -> U8Vector4: ...
    @property
    def ttqs(self) -> U8Vector4: ...
    @property
    def ttqt(self) -> U8Vector4: ...
    @property
    def ttqq(self) -> U8Vector4: ...
    @property
    def ttqp(self) -> U8Vector4: ...
    @property
    def ttqo(self) -> U8Vector4: ...
    @property
    def ttql(self) -> U8Vector4: ...
    @property
    def ttps(self) -> U8Vector4: ...
    @property
    def ttpt(self) -> U8Vector4: ...
    @property
    def ttpq(self) -> U8Vector4: ...
    @property
    def ttpp(self) -> U8Vector4: ...
    @property
    def ttpo(self) -> U8Vector4: ...
    @property
    def ttpl(self) -> U8Vector4: ...
    @property
    def ttos(self) -> U8Vector4: ...
    @property
    def ttot(self) -> U8Vector4: ...
    @property
    def ttoq(self) -> U8Vector4: ...
    @property
    def ttop(self) -> U8Vector4: ...
    @property
    def ttoo(self) -> U8Vector4: ...
    @property
    def ttol(self) -> U8Vector4: ...
    @property
    def ttls(self) -> U8Vector4: ...
    @property
    def ttlt(self) -> U8Vector4: ...
    @property
    def ttlq(self) -> U8Vector4: ...
    @property
    def ttlp(self) -> U8Vector4: ...
    @property
    def ttlo(self) -> U8Vector4: ...
    @property
    def ttll(self) -> U8Vector4: ...
    @property
    def tqss(self) -> U8Vector4: ...
    @property
    def tqst(self) -> U8Vector4: ...
    @property
    def tqsq(self) -> U8Vector4: ...
    @property
    def tqsp(self) -> U8Vector4: ...
    @property
    def tqso(self) -> U8Vector4: ...
    @property
    def tqsl(self) -> U8Vector4: ...
    @property
    def tqts(self) -> U8Vector4: ...
    @property
    def tqtt(self) -> U8Vector4: ...
    @property
    def tqtq(self) -> U8Vector4: ...
    @property
    def tqtp(self) -> U8Vector4: ...
    @property
    def tqto(self) -> U8Vector4: ...
    @property
    def tqtl(self) -> U8Vector4: ...
    @property
    def tqqs(self) -> U8Vector4: ...
    @property
    def tqqt(self) -> U8Vector4: ...
    @property
    def tqqq(self) -> U8Vector4: ...
    @property
    def tqqp(self) -> U8Vector4: ...
    @property
    def tqqo(self) -> U8Vector4: ...
    @property
    def tqql(self) -> U8Vector4: ...
    @property
    def tqps(self) -> U8Vector4: ...
    @property
    def tqpt(self) -> U8Vector4: ...
    @property
    def tqpq(self) -> U8Vector4: ...
    @property
    def tqpp(self) -> U8Vector4: ...
    @property
    def tqpo(self) -> U8Vector4: ...
    @property
    def tqpl(self) -> U8Vector4: ...
    @property
    def tqos(self) -> U8Vector4: ...
    @property
    def tqot(self) -> U8Vector4: ...
    @property
    def tqoq(self) -> U8Vector4: ...
    @property
    def tqop(self) -> U8Vector4: ...
    @property
    def tqoo(self) -> U8Vector4: ...
    @property
    def tqol(self) -> U8Vector4: ...
    @property
    def tqls(self) -> U8Vector4: ...
    @property
    def tqlt(self) -> U8Vector4: ...
    @property
    def tqlq(self) -> U8Vector4: ...
    @property
    def tqlp(self) -> U8Vector4: ...
    @property
    def tqlo(self) -> U8Vector4: ...
    @property
    def tqll(self) -> U8Vector4: ...
    @property
    def tpss(self) -> U8Vector4: ...
    @property
    def tpst(self) -> U8Vector4: ...
    @property
    def tpsq(self) -> U8Vector4: ...
    @property
    def tpsp(self) -> U8Vector4: ...
    @property
    def tpso(self) -> U8Vector4: ...
    @property
    def tpsl(self) -> U8Vector4: ...
    @property
    def tpts(self) -> U8Vector4: ...
    @property
    def tptt(self) -> U8Vector4: ...
    @property
    def tptq(self) -> U8Vector4: ...
    @property
    def tptp(self) -> U8Vector4: ...
    @property
    def tpto(self) -> U8Vector4: ...
    @property
    def tptl(self) -> U8Vector4: ...
    @property
    def tpqs(self) -> U8Vector4: ...
    @property
    def tpqt(self) -> U8Vector4: ...
    @property
    def tpqq(self) -> U8Vector4: ...
    @property
    def tpqp(self) -> U8Vector4: ...
    @property
    def tpqo(self) -> U8Vector4: ...
    @property
    def tpql(self) -> U8Vector4: ...
    @property
    def tpps(self) -> U8Vector4: ...
    @property
    def tppt(self) -> U8Vector4: ...
    @property
    def tppq(self) -> U8Vector4: ...
    @property
    def tppp(self) -> U8Vector4: ...
    @property
    def tppo(self) -> U8Vector4: ...
    @property
    def tppl(self) -> U8Vector4: ...
    @property
    def tpos(self) -> U8Vector4: ...
    @property
    def tpot(self) -> U8Vector4: ...
    @property
    def tpoq(self) -> U8Vector4: ...
    @property
    def tpop(self) -> U8Vector4: ...
    @property
    def tpoo(self) -> U8Vector4: ...
    @property
    def tpol(self) -> U8Vector4: ...
    @property
    def tpls(self) -> U8Vector4: ...
    @property
    def tplt(self) -> U8Vector4: ...
    @property
    def tplq(self) -> U8Vector4: ...
    @property
    def tplp(self) -> U8Vector4: ...
    @property
    def tplo(self) -> U8Vector4: ...
    @property
    def tpll(self) -> U8Vector4: ...
    @property
    def toss(self) -> U8Vector4: ...
    @property
    def tost(self) -> U8Vector4: ...
    @property
    def tosq(self) -> U8Vector4: ...
    @property
    def tosp(self) -> U8Vector4: ...
    @property
    def toso(self) -> U8Vector4: ...
    @property
    def tosl(self) -> U8Vector4: ...
    @property
    def tots(self) -> U8Vector4: ...
    @property
    def tott(self) -> U8Vector4: ...
    @property
    def totq(self) -> U8Vector4: ...
    @property
    def totp(self) -> U8Vector4: ...
    @property
    def toto(self) -> U8Vector4: ...
    @property
    def totl(self) -> U8Vector4: ...
    @property
    def toqs(self) -> U8Vector4: ...
    @property
    def toqt(self) -> U8Vector4: ...
    @property
    def toqq(self) -> U8Vector4: ...
    @property
    def toqp(self) -> U8Vector4: ...
    @property
    def toqo(self) -> U8Vector4: ...
    @property
    def toql(self) -> U8Vector4: ...
    @property
    def tops(self) -> U8Vector4: ...
    @property
    def topt(self) -> U8Vector4: ...
    @property
    def topq(self) -> U8Vector4: ...
    @property
    def topp(self) -> U8Vector4: ...
    @property
    def topo(self) -> U8Vector4: ...
    @property
    def topl(self) -> U8Vector4: ...
    @property
    def toos(self) -> U8Vector4: ...
    @property
    def toot(self) -> U8Vector4: ...
    @property
    def tooq(self) -> U8Vector4: ...
    @property
    def toop(self) -> U8Vector4: ...
    @property
    def tooo(self) -> U8Vector4: ...
    @property
    def tool(self) -> U8Vector4: ...
    @property
    def tols(self) -> U8Vector4: ...
    @property
    def tolt(self) -> U8Vector4: ...
    @property
    def tolq(self) -> U8Vector4: ...
    @property
    def tolp(self) -> U8Vector4: ...
    @property
    def tolo(self) -> U8Vector4: ...
    @property
    def toll(self) -> U8Vector4: ...
    @property
    def tlss(self) -> U8Vector4: ...
    @property
    def tlst(self) -> U8Vector4: ...
    @property
    def tlsq(self) -> U8Vector4: ...
    @property
    def tlsp(self) -> U8Vector4: ...
    @property
    def tlso(self) -> U8Vector4: ...
    @property
    def tlsl(self) -> U8Vector4: ...
    @property
    def tlts(self) -> U8Vector4: ...
    @property
    def tltt(self) -> U8Vector4: ...
    @property
    def tltq(self) -> U8Vector4: ...
    @property
    def tltp(self) -> U8Vector4: ...
    @property
    def tlto(self) -> U8Vector4: ...
    @property
    def tltl(self) -> U8Vector4: ...
    @property
    def tlqs(self) -> U8Vector4: ...
    @property
    def tlqt(self) -> U8Vector4: ...
    @property
    def tlqq(self) -> U8Vector4: ...
    @property
    def tlqp(self) -> U8Vector4: ...
    @property
    def tlqo(self) -> U8Vector4: ...
    @property
    def tlql(self) -> U8Vector4: ...
    @property
    def tlps(self) -> U8Vector4: ...
    @property
    def tlpt(self) -> U8Vector4: ...
    @property
    def tlpq(self) -> U8Vector4: ...
    @property
    def tlpp(self) -> U8Vector4: ...
    @property
    def tlpo(self) -> U8Vector4: ...
    @property
    def tlpl(self) -> U8Vector4: ...
    @property
    def tlos(self) -> U8Vector4: ...
    @property
    def tlot(self) -> U8Vector4: ...
    @property
    def tloq(self) -> U8Vector4: ...
    @property
    def tlop(self) -> U8Vector4: ...
    @property
    def tloo(self) -> U8Vector4: ...
    @property
    def tlol(self) -> U8Vector4: ...
    @property
    def tlls(self) -> U8Vector4: ...
    @property
    def tllt(self) -> U8Vector4: ...
    @property
    def tllq(self) -> U8Vector4: ...
    @property
    def tllp(self) -> U8Vector4: ...
    @property
    def tllo(self) -> U8Vector4: ...
    @property
    def tlll(self) -> U8Vector4: ...
    @property
    def qsss(self) -> U8Vector4: ...
    @property
    def qsst(self) -> U8Vector4: ...
    @property
    def qssq(self) -> U8Vector4: ...
    @property
    def qssp(self) -> U8Vector4: ...
    @property
    def qsso(self) -> U8Vector4: ...
    @property
    def qssl(self) -> U8Vector4: ...
    @property
    def qsts(self) -> U8Vector4: ...
    @property
    def qstt(self) -> U8Vector4: ...
    @property
    def qstq(self) -> U8Vector4: ...
    @property
    def qstp(self) -> U8Vector4: ...
    @property
    def qsto(self) -> U8Vector4: ...
    @property
    def qstl(self) -> U8Vector4: ...
    @property
    def qsqs(self) -> U8Vector4: ...
    @property
    def qsqt(self) -> U8Vector4: ...
    @property
    def qsqq(self) -> U8Vector4: ...
    @property
    def qsqp(self) -> U8Vector4: ...
    @property
    def qsqo(self) -> U8Vector4: ...
    @property
    def qsql(self) -> U8Vector4: ...
    @property
    def qsps(self) -> U8Vector4: ...
    @property
    def qspt(self) -> U8Vector4: ...
    @property
    def qspq(self) -> U8Vector4: ...
    @property
    def qspp(self) -> U8Vector4: ...
    @property
    def qspo(self) -> U8Vector4: ...
    @property
    def qspl(self) -> U8Vector4: ...
    @property
    def qsos(self) -> U8Vector4: ...
    @property
    def qsot(self) -> U8Vector4: ...
    @property
    def qsoq(self) -> U8Vector4: ...
    @property
    def qsop(self) -> U8Vector4: ...
    @property
    def qsoo(self) -> U8Vector4: ...
    @property
    def qsol(self) -> U8Vector4: ...
    @property
    def qsls(self) -> U8Vector4: ...
    @property
    def qslt(self) -> U8Vector4: ...
    @property
    def qslq(self) -> U8Vector4: ...
    @property
    def qslp(self) -> U8Vector4: ...
    @property
    def qslo(self) -> U8Vector4: ...
    @property
    def qsll(self) -> U8Vector4: ...
    @property
    def qtss(self) -> U8Vector4: ...
    @property
    def qtst(self) -> U8Vector4: ...
    @property
    def qtsq(self) -> U8Vector4: ...
    @property
    def qtsp(self) -> U8Vector4: ...
    @property
    def qtso(self) -> U8Vector4: ...
    @property
    def qtsl(self) -> U8Vector4: ...
    @property
    def qtts(self) -> U8Vector4: ...
    @property
    def qttt(self) -> U8Vector4: ...
    @property
    def qttq(self) -> U8Vector4: ...
    @property
    def qttp(self) -> U8Vector4: ...
    @property
    def qtto(self) -> U8Vector4: ...
    @property
    def qttl(self) -> U8Vector4: ...
    @property
    def qtqs(self) -> U8Vector4: ...
    @property
    def qtqt(self) -> U8Vector4: ...
    @property
    def qtqq(self) -> U8Vector4: ...
    @property
    def qtqp(self) -> U8Vector4: ...
    @property
    def qtqo(self) -> U8Vector4: ...
    @property
    def qtql(self) -> U8Vector4: ...
    @property
    def qtps(self) -> U8Vector4: ...
    @property
    def qtpt(self) -> U8Vector4: ...
    @property
    def qtpq(self) -> U8Vector4: ...
    @property
    def qtpp(self) -> U8Vector4: ...
    @property
    def qtpo(self) -> U8Vector4: ...
    @property
    def qtpl(self) -> U8Vector4: ...
    @property
    def qtos(self) -> U8Vector4: ...
    @property
    def qtot(self) -> U8Vector4: ...
    @property
    def qtoq(self) -> U8Vector4: ...
    @property
    def qtop(self) -> U8Vector4: ...
    @property
    def qtoo(self) -> U8Vector4: ...
    @property
    def qtol(self) -> U8Vector4: ...
    @property
    def qtls(self) -> U8Vector4: ...
    @property
    def qtlt(self) -> U8Vector4: ...
    @property
    def qtlq(self) -> U8Vector4: ...
    @property
    def qtlp(self) -> U8Vector4: ...
    @property
    def qtlo(self) -> U8Vector4: ...
    @property
    def qtll(self) -> U8Vector4: ...
    @property
    def qqss(self) -> U8Vector4: ...
    @property
    def qqst(self) -> U8Vector4: ...
    @property
    def qqsq(self) -> U8Vector4: ...
    @property
    def qqsp(self) -> U8Vector4: ...
    @property
    def qqso(self) -> U8Vector4: ...
    @property
    def qqsl(self) -> U8Vector4: ...
    @property
    def qqts(self) -> U8Vector4: ...
    @property
    def qqtt(self) -> U8Vector4: ...
    @property
    def qqtq(self) -> U8Vector4: ...
    @property
    def qqtp(self) -> U8Vector4: ...
    @property
    def qqto(self) -> U8Vector4: ...
    @property
    def qqtl(self) -> U8Vector4: ...
    @property
    def qqqs(self) -> U8Vector4: ...
    @property
    def qqqt(self) -> U8Vector4: ...
    @property
    def qqqq(self) -> U8Vector4: ...
    @property
    def qqqp(self) -> U8Vector4: ...
    @property
    def qqqo(self) -> U8Vector4: ...
    @property
    def qqql(self) -> U8Vector4: ...
    @property
    def qqps(self) -> U8Vector4: ...
    @property
    def qqpt(self) -> U8Vector4: ...
    @property
    def qqpq(self) -> U8Vector4: ...
    @property
    def qqpp(self) -> U8Vector4: ...
    @property
    def qqpo(self) -> U8Vector4: ...
    @property
    def qqpl(self) -> U8Vector4: ...
    @property
    def qqos(self) -> U8Vector4: ...
    @property
    def qqot(self) -> U8Vector4: ...
    @property
    def qqoq(self) -> U8Vector4: ...
    @property
    def qqop(self) -> U8Vector4: ...
    @property
    def qqoo(self) -> U8Vector4: ...
    @property
    def qqol(self) -> U8Vector4: ...
    @property
    def qqls(self) -> U8Vector4: ...
    @property
    def qqlt(self) -> U8Vector4: ...
    @property
    def qqlq(self) -> U8Vector4: ...
    @property
    def qqlp(self) -> U8Vector4: ...
    @property
    def qqlo(self) -> U8Vector4: ...
    @property
    def qqll(self) -> U8Vector4: ...
    @property
    def qpss(self) -> U8Vector4: ...
    @property
    def qpst(self) -> U8Vector4: ...
    @property
    def qpsq(self) -> U8Vector4: ...
    @property
    def qpsp(self) -> U8Vector4: ...
    @property
    def qpso(self) -> U8Vector4: ...
    @property
    def qpsl(self) -> U8Vector4: ...
    @property
    def qpts(self) -> U8Vector4: ...
    @property
    def qptt(self) -> U8Vector4: ...
    @property
    def qptq(self) -> U8Vector4: ...
    @property
    def qptp(self) -> U8Vector4: ...
    @property
    def qpto(self) -> U8Vector4: ...
    @property
    def qptl(self) -> U8Vector4: ...
    @property
    def qpqs(self) -> U8Vector4: ...
    @property
    def qpqt(self) -> U8Vector4: ...
    @property
    def qpqq(self) -> U8Vector4: ...
    @property
    def qpqp(self) -> U8Vector4: ...
    @property
    def qpqo(self) -> U8Vector4: ...
    @property
    def qpql(self) -> U8Vector4: ...
    @property
    def qpps(self) -> U8Vector4: ...
    @property
    def qppt(self) -> U8Vector4: ...
    @property
    def qppq(self) -> U8Vector4: ...
    @property
    def qppp(self) -> U8Vector4: ...
    @property
    def qppo(self) -> U8Vector4: ...
    @property
    def qppl(self) -> U8Vector4: ...
    @property
    def qpos(self) -> U8Vector4: ...
    @property
    def qpot(self) -> U8Vector4: ...
    @property
    def qpoq(self) -> U8Vector4: ...
    @property
    def qpop(self) -> U8Vector4: ...
    @property
    def qpoo(self) -> U8Vector4: ...
    @property
    def qpol(self) -> U8Vector4: ...
    @property
    def qpls(self) -> U8Vector4: ...
    @property
    def qplt(self) -> U8Vector4: ...
    @property
    def qplq(self) -> U8Vector4: ...
    @property
    def qplp(self) -> U8Vector4: ...
    @property
    def qplo(self) -> U8Vector4: ...
    @property
    def qpll(self) -> U8Vector4: ...
    @property
    def qoss(self) -> U8Vector4: ...
    @property
    def qost(self) -> U8Vector4: ...
    @property
    def qosq(self) -> U8Vector4: ...
    @property
    def qosp(self) -> U8Vector4: ...
    @property
    def qoso(self) -> U8Vector4: ...
    @property
    def qosl(self) -> U8Vector4: ...
    @property
    def qots(self) -> U8Vector4: ...
    @property
    def qott(self) -> U8Vector4: ...
    @property
    def qotq(self) -> U8Vector4: ...
    @property
    def qotp(self) -> U8Vector4: ...
    @property
    def qoto(self) -> U8Vector4: ...
    @property
    def qotl(self) -> U8Vector4: ...
    @property
    def qoqs(self) -> U8Vector4: ...
    @property
    def qoqt(self) -> U8Vector4: ...
    @property
    def qoqq(self) -> U8Vector4: ...
    @property
    def qoqp(self) -> U8Vector4: ...
    @property
    def qoqo(self) -> U8Vector4: ...
    @property
    def qoql(self) -> U8Vector4: ...
    @property
    def qops(self) -> U8Vector4: ...
    @property
    def qopt(self) -> U8Vector4: ...
    @property
    def qopq(self) -> U8Vector4: ...
    @property
    def qopp(self) -> U8Vector4: ...
    @property
    def qopo(self) -> U8Vector4: ...
    @property
    def qopl(self) -> U8Vector4: ...
    @property
    def qoos(self) -> U8Vector4: ...
    @property
    def qoot(self) -> U8Vector4: ...
    @property
    def qooq(self) -> U8Vector4: ...
    @property
    def qoop(self) -> U8Vector4: ...
    @property
    def qooo(self) -> U8Vector4: ...
    @property
    def qool(self) -> U8Vector4: ...
    @property
    def qols(self) -> U8Vector4: ...
    @property
    def qolt(self) -> U8Vector4: ...
    @property
    def qolq(self) -> U8Vector4: ...
    @property
    def qolp(self) -> U8Vector4: ...
    @property
    def qolo(self) -> U8Vector4: ...
    @property
    def qoll(self) -> U8Vector4: ...
    @property
    def qlss(self) -> U8Vector4: ...
    @property
    def qlst(self) -> U8Vector4: ...
    @property
    def qlsq(self) -> U8Vector4: ...
    @property
    def qlsp(self) -> U8Vector4: ...
    @property
    def qlso(self) -> U8Vector4: ...
    @property
    def qlsl(self) -> U8Vector4: ...
    @property
    def qlts(self) -> U8Vector4: ...
    @property
    def qltt(self) -> U8Vector4: ...
    @property
    def qltq(self) -> U8Vector4: ...
    @property
    def qltp(self) -> U8Vector4: ...
    @property
    def qlto(self) -> U8Vector4: ...
    @property
    def qltl(self) -> U8Vector4: ...
    @property
    def qlqs(self) -> U8Vector4: ...
    @property
    def qlqt(self) -> U8Vector4: ...
    @property
    def qlqq(self) -> U8Vector4: ...
    @property
    def qlqp(self) -> U8Vector4: ...
    @property
    def qlqo(self) -> U8Vector4: ...
    @property
    def qlql(self) -> U8Vector4: ...
    @property
    def qlps(self) -> U8Vector4: ...
    @property
    def qlpt(self) -> U8Vector4: ...
    @property
    def qlpq(self) -> U8Vector4: ...
    @property
    def qlpp(self) -> U8Vector4: ...
    @property
    def qlpo(self) -> U8Vector4: ...
    @property
    def qlpl(self) -> U8Vector4: ...
    @property
    def qlos(self) -> U8Vector4: ...
    @property
    def qlot(self) -> U8Vector4: ...
    @property
    def qloq(self) -> U8Vector4: ...
    @property
    def qlop(self) -> U8Vector4: ...
    @property
    def qloo(self) -> U8Vector4: ...
    @property
    def qlol(self) -> U8Vector4: ...
    @property
    def qlls(self) -> U8Vector4: ...
    @property
    def qllt(self) -> U8Vector4: ...
    @property
    def qllq(self) -> U8Vector4: ...
    @property
    def qllp(self) -> U8Vector4: ...
    @property
    def qllo(self) -> U8Vector4: ...
    @property
    def qlll(self) -> U8Vector4: ...
    @property
    def psss(self) -> U8Vector4: ...
    @property
    def psst(self) -> U8Vector4: ...
    @property
    def pssq(self) -> U8Vector4: ...
    @property
    def pssp(self) -> U8Vector4: ...
    @property
    def psso(self) -> U8Vector4: ...
    @property
    def pssl(self) -> U8Vector4: ...
    @property
    def psts(self) -> U8Vector4: ...
    @property
    def pstt(self) -> U8Vector4: ...
    @property
    def pstq(self) -> U8Vector4: ...
    @property
    def pstp(self) -> U8Vector4: ...
    @property
    def psto(self) -> U8Vector4: ...
    @property
    def pstl(self) -> U8Vector4: ...
    @property
    def psqs(self) -> U8Vector4: ...
    @property
    def psqt(self) -> U8Vector4: ...
    @property
    def psqq(self) -> U8Vector4: ...
    @property
    def psqp(self) -> U8Vector4: ...
    @property
    def psqo(self) -> U8Vector4: ...
    @property
    def psql(self) -> U8Vector4: ...
    @property
    def psps(self) -> U8Vector4: ...
    @property
    def pspt(self) -> U8Vector4: ...
    @property
    def pspq(self) -> U8Vector4: ...
    @property
    def pspp(self) -> U8Vector4: ...
    @property
    def pspo(self) -> U8Vector4: ...
    @property
    def pspl(self) -> U8Vector4: ...
    @property
    def psos(self) -> U8Vector4: ...
    @property
    def psot(self) -> U8Vector4: ...
    @property
    def psoq(self) -> U8Vector4: ...
    @property
    def psop(self) -> U8Vector4: ...
    @property
    def psoo(self) -> U8Vector4: ...
    @property
    def psol(self) -> U8Vector4: ...
    @property
    def psls(self) -> U8Vector4: ...
    @property
    def pslt(self) -> U8Vector4: ...
    @property
    def pslq(self) -> U8Vector4: ...
    @property
    def pslp(self) -> U8Vector4: ...
    @property
    def pslo(self) -> U8Vector4: ...
    @property
    def psll(self) -> U8Vector4: ...
    @property
    def ptss(self) -> U8Vector4: ...
    @property
    def ptst(self) -> U8Vector4: ...
    @property
    def ptsq(self) -> U8Vector4: ...
    @property
    def ptsp(self) -> U8Vector4: ...
    @property
    def ptso(self) -> U8Vector4: ...
    @property
    def ptsl(self) -> U8Vector4: ...
    @property
    def ptts(self) -> U8Vector4: ...
    @property
    def pttt(self) -> U8Vector4: ...
    @property
    def pttq(self) -> U8Vector4: ...
    @property
    def pttp(self) -> U8Vector4: ...
    @property
    def ptto(self) -> U8Vector4: ...
    @property
    def pttl(self) -> U8Vector4: ...
    @property
    def ptqs(self) -> U8Vector4: ...
    @property
    def ptqt(self) -> U8Vector4: ...
    @property
    def ptqq(self) -> U8Vector4: ...
    @property
    def ptqp(self) -> U8Vector4: ...
    @property
    def ptqo(self) -> U8Vector4: ...
    @property
    def ptql(self) -> U8Vector4: ...
    @property
    def ptps(self) -> U8Vector4: ...
    @property
    def ptpt(self) -> U8Vector4: ...
    @property
    def ptpq(self) -> U8Vector4: ...
    @property
    def ptpp(self) -> U8Vector4: ...
    @property
    def ptpo(self) -> U8Vector4: ...
    @property
    def ptpl(self) -> U8Vector4: ...
    @property
    def ptos(self) -> U8Vector4: ...
    @property
    def ptot(self) -> U8Vector4: ...
    @property
    def ptoq(self) -> U8Vector4: ...
    @property
    def ptop(self) -> U8Vector4: ...
    @property
    def ptoo(self) -> U8Vector4: ...
    @property
    def ptol(self) -> U8Vector4: ...
    @property
    def ptls(self) -> U8Vector4: ...
    @property
    def ptlt(self) -> U8Vector4: ...
    @property
    def ptlq(self) -> U8Vector4: ...
    @property
    def ptlp(self) -> U8Vector4: ...
    @property
    def ptlo(self) -> U8Vector4: ...
    @property
    def ptll(self) -> U8Vector4: ...
    @property
    def pqss(self) -> U8Vector4: ...
    @property
    def pqst(self) -> U8Vector4: ...
    @property
    def pqsq(self) -> U8Vector4: ...
    @property
    def pqsp(self) -> U8Vector4: ...
    @property
    def pqso(self) -> U8Vector4: ...
    @property
    def pqsl(self) -> U8Vector4: ...
    @property
    def pqts(self) -> U8Vector4: ...
    @property
    def pqtt(self) -> U8Vector4: ...
    @property
    def pqtq(self) -> U8Vector4: ...
    @property
    def pqtp(self) -> U8Vector4: ...
    @property
    def pqto(self) -> U8Vector4: ...
    @property
    def pqtl(self) -> U8Vector4: ...
    @property
    def pqqs(self) -> U8Vector4: ...
    @property
    def pqqt(self) -> U8Vector4: ...
    @property
    def pqqq(self) -> U8Vector4: ...
    @property
    def pqqp(self) -> U8Vector4: ...
    @property
    def pqqo(self) -> U8Vector4: ...
    @property
    def pqql(self) -> U8Vector4: ...
    @property
    def pqps(self) -> U8Vector4: ...
    @property
    def pqpt(self) -> U8Vector4: ...
    @property
    def pqpq(self) -> U8Vector4: ...
    @property
    def pqpp(self) -> U8Vector4: ...
    @property
    def pqpo(self) -> U8Vector4: ...
    @property
    def pqpl(self) -> U8Vector4: ...
    @property
    def pqos(self) -> U8Vector4: ...
    @property
    def pqot(self) -> U8Vector4: ...
    @property
    def pqoq(self) -> U8Vector4: ...
    @property
    def pqop(self) -> U8Vector4: ...
    @property
    def pqoo(self) -> U8Vector4: ...
    @property
    def pqol(self) -> U8Vector4: ...
    @property
    def pqls(self) -> U8Vector4: ...
    @property
    def pqlt(self) -> U8Vector4: ...
    @property
    def pqlq(self) -> U8Vector4: ...
    @property
    def pqlp(self) -> U8Vector4: ...
    @property
    def pqlo(self) -> U8Vector4: ...
    @property
    def pqll(self) -> U8Vector4: ...
    @property
    def ppss(self) -> U8Vector4: ...
    @property
    def ppst(self) -> U8Vector4: ...
    @property
    def ppsq(self) -> U8Vector4: ...
    @property
    def ppsp(self) -> U8Vector4: ...
    @property
    def ppso(self) -> U8Vector4: ...
    @property
    def ppsl(self) -> U8Vector4: ...
    @property
    def ppts(self) -> U8Vector4: ...
    @property
    def pptt(self) -> U8Vector4: ...
    @property
    def pptq(self) -> U8Vector4: ...
    @property
    def pptp(self) -> U8Vector4: ...
    @property
    def ppto(self) -> U8Vector4: ...
    @property
    def pptl(self) -> U8Vector4: ...
    @property
    def ppqs(self) -> U8Vector4: ...
    @property
    def ppqt(self) -> U8Vector4: ...
    @property
    def ppqq(self) -> U8Vector4: ...
    @property
    def ppqp(self) -> U8Vector4: ...
    @property
    def ppqo(self) -> U8Vector4: ...
    @property
    def ppql(self) -> U8Vector4: ...
    @property
    def ppps(self) -> U8Vector4: ...
    @property
    def pppt(self) -> U8Vector4: ...
    @property
    def pppq(self) -> U8Vector4: ...
    @property
    def pppp(self) -> U8Vector4: ...
    @property
    def pppo(self) -> U8Vector4: ...
    @property
    def pppl(self) -> U8Vector4: ...
    @property
    def ppos(self) -> U8Vector4: ...
    @property
    def ppot(self) -> U8Vector4: ...
    @property
    def ppoq(self) -> U8Vector4: ...
    @property
    def ppop(self) -> U8Vector4: ...
    @property
    def ppoo(self) -> U8Vector4: ...
    @property
    def ppol(self) -> U8Vector4: ...
    @property
    def ppls(self) -> U8Vector4: ...
    @property
    def pplt(self) -> U8Vector4: ...
    @property
    def pplq(self) -> U8Vector4: ...
    @property
    def pplp(self) -> U8Vector4: ...
    @property
    def pplo(self) -> U8Vector4: ...
    @property
    def ppll(self) -> U8Vector4: ...
    @property
    def poss(self) -> U8Vector4: ...
    @property
    def post(self) -> U8Vector4: ...
    @property
    def posq(self) -> U8Vector4: ...
    @property
    def posp(self) -> U8Vector4: ...
    @property
    def poso(self) -> U8Vector4: ...
    @property
    def posl(self) -> U8Vector4: ...
    @property
    def pots(self) -> U8Vector4: ...
    @property
    def pott(self) -> U8Vector4: ...
    @property
    def potq(self) -> U8Vector4: ...
    @property
    def potp(self) -> U8Vector4: ...
    @property
    def poto(self) -> U8Vector4: ...
    @property
    def potl(self) -> U8Vector4: ...
    @property
    def poqs(self) -> U8Vector4: ...
    @property
    def poqt(self) -> U8Vector4: ...
    @property
    def poqq(self) -> U8Vector4: ...
    @property
    def poqp(self) -> U8Vector4: ...
    @property
    def poqo(self) -> U8Vector4: ...
    @property
    def poql(self) -> U8Vector4: ...
    @property
    def pops(self) -> U8Vector4: ...
    @property
    def popt(self) -> U8Vector4: ...
    @property
    def popq(self) -> U8Vector4: ...
    @property
    def popp(self) -> U8Vector4: ...
    @property
    def popo(self) -> U8Vector4: ...
    @property
    def popl(self) -> U8Vector4: ...
    @property
    def poos(self) -> U8Vector4: ...
    @property
    def poot(self) -> U8Vector4: ...
    @property
    def pooq(self) -> U8Vector4: ...
    @property
    def poop(self) -> U8Vector4: ...
    @property
    def pooo(self) -> U8Vector4: ...
    @property
    def pool(self) -> U8Vector4: ...
    @property
    def pols(self) -> U8Vector4: ...
    @property
    def polt(self) -> U8Vector4: ...
    @property
    def polq(self) -> U8Vector4: ...
    @property
    def polp(self) -> U8Vector4: ...
    @property
    def polo(self) -> U8Vector4: ...
    @property
    def poll(self) -> U8Vector4: ...
    @property
    def plss(self) -> U8Vector4: ...
    @property
    def plst(self) -> U8Vector4: ...
    @property
    def plsq(self) -> U8Vector4: ...
    @property
    def plsp(self) -> U8Vector4: ...
    @property
    def plso(self) -> U8Vector4: ...
    @property
    def plsl(self) -> U8Vector4: ...
    @property
    def plts(self) -> U8Vector4: ...
    @property
    def pltt(self) -> U8Vector4: ...
    @property
    def pltq(self) -> U8Vector4: ...
    @property
    def pltp(self) -> U8Vector4: ...
    @property
    def plto(self) -> U8Vector4: ...
    @property
    def pltl(self) -> U8Vector4: ...
    @property
    def plqs(self) -> U8Vector4: ...
    @property
    def plqt(self) -> U8Vector4: ...
    @property
    def plqq(self) -> U8Vector4: ...
    @property
    def plqp(self) -> U8Vector4: ...
    @property
    def plqo(self) -> U8Vector4: ...
    @property
    def plql(self) -> U8Vector4: ...
    @property
    def plps(self) -> U8Vector4: ...
    @property
    def plpt(self) -> U8Vector4: ...
    @property
    def plpq(self) -> U8Vector4: ...
    @property
    def plpp(self) -> U8Vector4: ...
    @property
    def plpo(self) -> U8Vector4: ...
    @property
    def plpl(self) -> U8Vector4: ...
    @property
    def plos(self) -> U8Vector4: ...
    @property
    def plot(self) -> U8Vector4: ...
    @property
    def ploq(self) -> U8Vector4: ...
    @property
    def plop(self) -> U8Vector4: ...
    @property
    def ploo(self) -> U8Vector4: ...
    @property
    def plol(self) -> U8Vector4: ...
    @property
    def plls(self) -> U8Vector4: ...
    @property
    def pllt(self) -> U8Vector4: ...
    @property
    def pllq(self) -> U8Vector4: ...
    @property
    def pllp(self) -> U8Vector4: ...
    @property
    def pllo(self) -> U8Vector4: ...
    @property
    def plll(self) -> U8Vector4: ...
    @property
    def osss(self) -> U8Vector4: ...
    @property
    def osst(self) -> U8Vector4: ...
    @property
    def ossq(self) -> U8Vector4: ...
    @property
    def ossp(self) -> U8Vector4: ...
    @property
    def osso(self) -> U8Vector4: ...
    @property
    def ossl(self) -> U8Vector4: ...
    @property
    def osts(self) -> U8Vector4: ...
    @property
    def ostt(self) -> U8Vector4: ...
    @property
    def ostq(self) -> U8Vector4: ...
    @property
    def ostp(self) -> U8Vector4: ...
    @property
    def osto(self) -> U8Vector4: ...
    @property
    def ostl(self) -> U8Vector4: ...
    @property
    def osqs(self) -> U8Vector4: ...
    @property
    def osqt(self) -> U8Vector4: ...
    @property
    def osqq(self) -> U8Vector4: ...
    @property
    def osqp(self) -> U8Vector4: ...
    @property
    def osqo(self) -> U8Vector4: ...
    @property
    def osql(self) -> U8Vector4: ...
    @property
    def osps(self) -> U8Vector4: ...
    @property
    def ospt(self) -> U8Vector4: ...
    @property
    def ospq(self) -> U8Vector4: ...
    @property
    def ospp(self) -> U8Vector4: ...
    @property
    def ospo(self) -> U8Vector4: ...
    @property
    def ospl(self) -> U8Vector4: ...
    @property
    def osos(self) -> U8Vector4: ...
    @property
    def osot(self) -> U8Vector4: ...
    @property
    def osoq(self) -> U8Vector4: ...
    @property
    def osop(self) -> U8Vector4: ...
    @property
    def osoo(self) -> U8Vector4: ...
    @property
    def osol(self) -> U8Vector4: ...
    @property
    def osls(self) -> U8Vector4: ...
    @property
    def oslt(self) -> U8Vector4: ...
    @property
    def oslq(self) -> U8Vector4: ...
    @property
    def oslp(self) -> U8Vector4: ...
    @property
    def oslo(self) -> U8Vector4: ...
    @property
    def osll(self) -> U8Vector4: ...
    @property
    def otss(self) -> U8Vector4: ...
    @property
    def otst(self) -> U8Vector4: ...
    @property
    def otsq(self) -> U8Vector4: ...
    @property
    def otsp(self) -> U8Vector4: ...
    @property
    def otso(self) -> U8Vector4: ...
    @property
    def otsl(self) -> U8Vector4: ...
    @property
    def otts(self) -> U8Vector4: ...
    @property
    def ottt(self) -> U8Vector4: ...
    @property
    def ottq(self) -> U8Vector4: ...
    @property
    def ottp(self) -> U8Vector4: ...
    @property
    def otto(self) -> U8Vector4: ...
    @property
    def ottl(self) -> U8Vector4: ...
    @property
    def otqs(self) -> U8Vector4: ...
    @property
    def otqt(self) -> U8Vector4: ...
    @property
    def otqq(self) -> U8Vector4: ...
    @property
    def otqp(self) -> U8Vector4: ...
    @property
    def otqo(self) -> U8Vector4: ...
    @property
    def otql(self) -> U8Vector4: ...
    @property
    def otps(self) -> U8Vector4: ...
    @property
    def otpt(self) -> U8Vector4: ...
    @property
    def otpq(self) -> U8Vector4: ...
    @property
    def otpp(self) -> U8Vector4: ...
    @property
    def otpo(self) -> U8Vector4: ...
    @property
    def otpl(self) -> U8Vector4: ...
    @property
    def otos(self) -> U8Vector4: ...
    @property
    def otot(self) -> U8Vector4: ...
    @property
    def otoq(self) -> U8Vector4: ...
    @property
    def otop(self) -> U8Vector4: ...
    @property
    def otoo(self) -> U8Vector4: ...
    @property
    def otol(self) -> U8Vector4: ...
    @property
    def otls(self) -> U8Vector4: ...
    @property
    def otlt(self) -> U8Vector4: ...
    @property
    def otlq(self) -> U8Vector4: ...
    @property
    def otlp(self) -> U8Vector4: ...
    @property
    def otlo(self) -> U8Vector4: ...
    @property
    def otll(self) -> U8Vector4: ...
    @property
    def oqss(self) -> U8Vector4: ...
    @property
    def oqst(self) -> U8Vector4: ...
    @property
    def oqsq(self) -> U8Vector4: ...
    @property
    def oqsp(self) -> U8Vector4: ...
    @property
    def oqso(self) -> U8Vector4: ...
    @property
    def oqsl(self) -> U8Vector4: ...
    @property
    def oqts(self) -> U8Vector4: ...
    @property
    def oqtt(self) -> U8Vector4: ...
    @property
    def oqtq(self) -> U8Vector4: ...
    @property
    def oqtp(self) -> U8Vector4: ...
    @property
    def oqto(self) -> U8Vector4: ...
    @property
    def oqtl(self) -> U8Vector4: ...
    @property
    def oqqs(self) -> U8Vector4: ...
    @property
    def oqqt(self) -> U8Vector4: ...
    @property
    def oqqq(self) -> U8Vector4: ...
    @property
    def oqqp(self) -> U8Vector4: ...
    @property
    def oqqo(self) -> U8Vector4: ...
    @property
    def oqql(self) -> U8Vector4: ...
    @property
    def oqps(self) -> U8Vector4: ...
    @property
    def oqpt(self) -> U8Vector4: ...
    @property
    def oqpq(self) -> U8Vector4: ...
    @property
    def oqpp(self) -> U8Vector4: ...
    @property
    def oqpo(self) -> U8Vector4: ...
    @property
    def oqpl(self) -> U8Vector4: ...
    @property
    def oqos(self) -> U8Vector4: ...
    @property
    def oqot(self) -> U8Vector4: ...
    @property
    def oqoq(self) -> U8Vector4: ...
    @property
    def oqop(self) -> U8Vector4: ...
    @property
    def oqoo(self) -> U8Vector4: ...
    @property
    def oqol(self) -> U8Vector4: ...
    @property
    def oqls(self) -> U8Vector4: ...
    @property
    def oqlt(self) -> U8Vector4: ...
    @property
    def oqlq(self) -> U8Vector4: ...
    @property
    def oqlp(self) -> U8Vector4: ...
    @property
    def oqlo(self) -> U8Vector4: ...
    @property
    def oqll(self) -> U8Vector4: ...
    @property
    def opss(self) -> U8Vector4: ...
    @property
    def opst(self) -> U8Vector4: ...
    @property
    def opsq(self) -> U8Vector4: ...
    @property
    def opsp(self) -> U8Vector4: ...
    @property
    def opso(self) -> U8Vector4: ...
    @property
    def opsl(self) -> U8Vector4: ...
    @property
    def opts(self) -> U8Vector4: ...
    @property
    def optt(self) -> U8Vector4: ...
    @property
    def optq(self) -> U8Vector4: ...
    @property
    def optp(self) -> U8Vector4: ...
    @property
    def opto(self) -> U8Vector4: ...
    @property
    def optl(self) -> U8Vector4: ...
    @property
    def opqs(self) -> U8Vector4: ...
    @property
    def opqt(self) -> U8Vector4: ...
    @property
    def opqq(self) -> U8Vector4: ...
    @property
    def opqp(self) -> U8Vector4: ...
    @property
    def opqo(self) -> U8Vector4: ...
    @property
    def opql(self) -> U8Vector4: ...
    @property
    def opps(self) -> U8Vector4: ...
    @property
    def oppt(self) -> U8Vector4: ...
    @property
    def oppq(self) -> U8Vector4: ...
    @property
    def oppp(self) -> U8Vector4: ...
    @property
    def oppo(self) -> U8Vector4: ...
    @property
    def oppl(self) -> U8Vector4: ...
    @property
    def opos(self) -> U8Vector4: ...
    @property
    def opot(self) -> U8Vector4: ...
    @property
    def opoq(self) -> U8Vector4: ...
    @property
    def opop(self) -> U8Vector4: ...
    @property
    def opoo(self) -> U8Vector4: ...
    @property
    def opol(self) -> U8Vector4: ...
    @property
    def opls(self) -> U8Vector4: ...
    @property
    def oplt(self) -> U8Vector4: ...
    @property
    def oplq(self) -> U8Vector4: ...
    @property
    def oplp(self) -> U8Vector4: ...
    @property
    def oplo(self) -> U8Vector4: ...
    @property
    def opll(self) -> U8Vector4: ...
    @property
    def ooss(self) -> U8Vector4: ...
    @property
    def oost(self) -> U8Vector4: ...
    @property
    def oosq(self) -> U8Vector4: ...
    @property
    def oosp(self) -> U8Vector4: ...
    @property
    def ooso(self) -> U8Vector4: ...
    @property
    def oosl(self) -> U8Vector4: ...
    @property
    def oots(self) -> U8Vector4: ...
    @property
    def oott(self) -> U8Vector4: ...
    @property
    def ootq(self) -> U8Vector4: ...
    @property
    def ootp(self) -> U8Vector4: ...
    @property
    def ooto(self) -> U8Vector4: ...
    @property
    def ootl(self) -> U8Vector4: ...
    @property
    def ooqs(self) -> U8Vector4: ...
    @property
    def ooqt(self) -> U8Vector4: ...
    @property
    def ooqq(self) -> U8Vector4: ...
    @property
    def ooqp(self) -> U8Vector4: ...
    @property
    def ooqo(self) -> U8Vector4: ...
    @property
    def ooql(self) -> U8Vector4: ...
    @property
    def oops(self) -> U8Vector4: ...
    @property
    def oopt(self) -> U8Vector4: ...
    @property
    def oopq(self) -> U8Vector4: ...
    @property
    def oopp(self) -> U8Vector4: ...
    @property
    def oopo(self) -> U8Vector4: ...
    @property
    def oopl(self) -> U8Vector4: ...
    @property
    def ooos(self) -> U8Vector4: ...
    @property
    def ooot(self) -> U8Vector4: ...
    @property
    def oooq(self) -> U8Vector4: ...
    @property
    def ooop(self) -> U8Vector4: ...
    @property
    def ools(self) -> U8Vector4: ...
    @property
    def oolt(self) -> U8Vector4: ...
    @property
    def oolq(self) -> U8Vector4: ...
    @property
    def oolp(self) -> U8Vector4: ...
    @property
    def olss(self) -> U8Vector4: ...
    @property
    def olst(self) -> U8Vector4: ...
    @property
    def olsq(self) -> U8Vector4: ...
    @property
    def olsp(self) -> U8Vector4: ...
    @property
    def olso(self) -> U8Vector4: ...
    @property
    def olsl(self) -> U8Vector4: ...
    @property
    def olts(self) -> U8Vector4: ...
    @property
    def oltt(self) -> U8Vector4: ...
    @property
    def oltq(self) -> U8Vector4: ...
    @property
    def oltp(self) -> U8Vector4: ...
    @property
    def olto(self) -> U8Vector4: ...
    @property
    def oltl(self) -> U8Vector4: ...
    @property
    def olqs(self) -> U8Vector4: ...
    @property
    def olqt(self) -> U8Vector4: ...
    @property
    def olqq(self) -> U8Vector4: ...
    @property
    def olqp(self) -> U8Vector4: ...
    @property
    def olqo(self) -> U8Vector4: ...
    @property
    def olql(self) -> U8Vector4: ...
    @property
    def olps(self) -> U8Vector4: ...
    @property
    def olpt(self) -> U8Vector4: ...
    @property
    def olpq(self) -> U8Vector4: ...
    @property
    def olpp(self) -> U8Vector4: ...
    @property
    def olpo(self) -> U8Vector4: ...
    @property
    def olpl(self) -> U8Vector4: ...
    @property
    def olos(self) -> U8Vector4: ...
    @property
    def olot(self) -> U8Vector4: ...
    @property
    def oloq(self) -> U8Vector4: ...
    @property
    def olop(self) -> U8Vector4: ...
    @property
    def olls(self) -> U8Vector4: ...
    @property
    def ollt(self) -> U8Vector4: ...
    @property
    def ollq(self) -> U8Vector4: ...
    @property
    def ollp(self) -> U8Vector4: ...
    @property
    def lsss(self) -> U8Vector4: ...
    @property
    def lsst(self) -> U8Vector4: ...
    @property
    def lssq(self) -> U8Vector4: ...
    @property
    def lssp(self) -> U8Vector4: ...
    @property
    def lsso(self) -> U8Vector4: ...
    @property
    def lssl(self) -> U8Vector4: ...
    @property
    def lsts(self) -> U8Vector4: ...
    @property
    def lstt(self) -> U8Vector4: ...
    @property
    def lstq(self) -> U8Vector4: ...
    @property
    def lstp(self) -> U8Vector4: ...
    @property
    def lsto(self) -> U8Vector4: ...
    @property
    def lstl(self) -> U8Vector4: ...
    @property
    def lsqs(self) -> U8Vector4: ...
    @property
    def lsqt(self) -> U8Vector4: ...
    @property
    def lsqq(self) -> U8Vector4: ...
    @property
    def lsqp(self) -> U8Vector4: ...
    @property
    def lsqo(self) -> U8Vector4: ...
    @property
    def lsql(self) -> U8Vector4: ...
    @property
    def lsps(self) -> U8Vector4: ...
    @property
    def lspt(self) -> U8Vector4: ...
    @property
    def lspq(self) -> U8Vector4: ...
    @property
    def lspp(self) -> U8Vector4: ...
    @property
    def lspo(self) -> U8Vector4: ...
    @property
    def lspl(self) -> U8Vector4: ...
    @property
    def lsos(self) -> U8Vector4: ...
    @property
    def lsot(self) -> U8Vector4: ...
    @property
    def lsoq(self) -> U8Vector4: ...
    @property
    def lsop(self) -> U8Vector4: ...
    @property
    def lsoo(self) -> U8Vector4: ...
    @property
    def lsol(self) -> U8Vector4: ...
    @property
    def lsls(self) -> U8Vector4: ...
    @property
    def lslt(self) -> U8Vector4: ...
    @property
    def lslq(self) -> U8Vector4: ...
    @property
    def lslp(self) -> U8Vector4: ...
    @property
    def lslo(self) -> U8Vector4: ...
    @property
    def lsll(self) -> U8Vector4: ...
    @property
    def ltss(self) -> U8Vector4: ...
    @property
    def ltst(self) -> U8Vector4: ...
    @property
    def ltsq(self) -> U8Vector4: ...
    @property
    def ltsp(self) -> U8Vector4: ...
    @property
    def ltso(self) -> U8Vector4: ...
    @property
    def ltsl(self) -> U8Vector4: ...
    @property
    def ltts(self) -> U8Vector4: ...
    @property
    def lttt(self) -> U8Vector4: ...
    @property
    def lttq(self) -> U8Vector4: ...
    @property
    def lttp(self) -> U8Vector4: ...
    @property
    def ltto(self) -> U8Vector4: ...
    @property
    def lttl(self) -> U8Vector4: ...
    @property
    def ltqs(self) -> U8Vector4: ...
    @property
    def ltqt(self) -> U8Vector4: ...
    @property
    def ltqq(self) -> U8Vector4: ...
    @property
    def ltqp(self) -> U8Vector4: ...
    @property
    def ltqo(self) -> U8Vector4: ...
    @property
    def ltql(self) -> U8Vector4: ...
    @property
    def ltps(self) -> U8Vector4: ...
    @property
    def ltpt(self) -> U8Vector4: ...
    @property
    def ltpq(self) -> U8Vector4: ...
    @property
    def ltpp(self) -> U8Vector4: ...
    @property
    def ltpo(self) -> U8Vector4: ...
    @property
    def ltpl(self) -> U8Vector4: ...
    @property
    def ltos(self) -> U8Vector4: ...
    @property
    def ltot(self) -> U8Vector4: ...
    @property
    def ltoq(self) -> U8Vector4: ...
    @property
    def ltop(self) -> U8Vector4: ...
    @property
    def ltoo(self) -> U8Vector4: ...
    @property
    def ltol(self) -> U8Vector4: ...
    @property
    def ltls(self) -> U8Vector4: ...
    @property
    def ltlt(self) -> U8Vector4: ...
    @property
    def ltlq(self) -> U8Vector4: ...
    @property
    def ltlp(self) -> U8Vector4: ...
    @property
    def ltlo(self) -> U8Vector4: ...
    @property
    def ltll(self) -> U8Vector4: ...
    @property
    def lqss(self) -> U8Vector4: ...
    @property
    def lqst(self) -> U8Vector4: ...
    @property
    def lqsq(self) -> U8Vector4: ...
    @property
    def lqsp(self) -> U8Vector4: ...
    @property
    def lqso(self) -> U8Vector4: ...
    @property
    def lqsl(self) -> U8Vector4: ...
    @property
    def lqts(self) -> U8Vector4: ...
    @property
    def lqtt(self) -> U8Vector4: ...
    @property
    def lqtq(self) -> U8Vector4: ...
    @property
    def lqtp(self) -> U8Vector4: ...
    @property
    def lqto(self) -> U8Vector4: ...
    @property
    def lqtl(self) -> U8Vector4: ...
    @property
    def lqqs(self) -> U8Vector4: ...
    @property
    def lqqt(self) -> U8Vector4: ...
    @property
    def lqqq(self) -> U8Vector4: ...
    @property
    def lqqp(self) -> U8Vector4: ...
    @property
    def lqqo(self) -> U8Vector4: ...
    @property
    def lqql(self) -> U8Vector4: ...
    @property
    def lqps(self) -> U8Vector4: ...
    @property
    def lqpt(self) -> U8Vector4: ...
    @property
    def lqpq(self) -> U8Vector4: ...
    @property
    def lqpp(self) -> U8Vector4: ...
    @property
    def lqpo(self) -> U8Vector4: ...
    @property
    def lqpl(self) -> U8Vector4: ...
    @property
    def lqos(self) -> U8Vector4: ...
    @property
    def lqot(self) -> U8Vector4: ...
    @property
    def lqoq(self) -> U8Vector4: ...
    @property
    def lqop(self) -> U8Vector4: ...
    @property
    def lqoo(self) -> U8Vector4: ...
    @property
    def lqol(self) -> U8Vector4: ...
    @property
    def lqls(self) -> U8Vector4: ...
    @property
    def lqlt(self) -> U8Vector4: ...
    @property
    def lqlq(self) -> U8Vector4: ...
    @property
    def lqlp(self) -> U8Vector4: ...
    @property
    def lqlo(self) -> U8Vector4: ...
    @property
    def lqll(self) -> U8Vector4: ...
    @property
    def lpss(self) -> U8Vector4: ...
    @property
    def lpst(self) -> U8Vector4: ...
    @property
    def lpsq(self) -> U8Vector4: ...
    @property
    def lpsp(self) -> U8Vector4: ...
    @property
    def lpso(self) -> U8Vector4: ...
    @property
    def lpsl(self) -> U8Vector4: ...
    @property
    def lpts(self) -> U8Vector4: ...
    @property
    def lptt(self) -> U8Vector4: ...
    @property
    def lptq(self) -> U8Vector4: ...
    @property
    def lptp(self) -> U8Vector4: ...
    @property
    def lpto(self) -> U8Vector4: ...
    @property
    def lptl(self) -> U8Vector4: ...
    @property
    def lpqs(self) -> U8Vector4: ...
    @property
    def lpqt(self) -> U8Vector4: ...
    @property
    def lpqq(self) -> U8Vector4: ...
    @property
    def lpqp(self) -> U8Vector4: ...
    @property
    def lpqo(self) -> U8Vector4: ...
    @property
    def lpql(self) -> U8Vector4: ...
    @property
    def lpps(self) -> U8Vector4: ...
    @property
    def lppt(self) -> U8Vector4: ...
    @property
    def lppq(self) -> U8Vector4: ...
    @property
    def lppp(self) -> U8Vector4: ...
    @property
    def lppo(self) -> U8Vector4: ...
    @property
    def lppl(self) -> U8Vector4: ...
    @property
    def lpos(self) -> U8Vector4: ...
    @property
    def lpot(self) -> U8Vector4: ...
    @property
    def lpoq(self) -> U8Vector4: ...
    @property
    def lpop(self) -> U8Vector4: ...
    @property
    def lpoo(self) -> U8Vector4: ...
    @property
    def lpol(self) -> U8Vector4: ...
    @property
    def lpls(self) -> U8Vector4: ...
    @property
    def lplt(self) -> U8Vector4: ...
    @property
    def lplq(self) -> U8Vector4: ...
    @property
    def lplp(self) -> U8Vector4: ...
    @property
    def lplo(self) -> U8Vector4: ...
    @property
    def lpll(self) -> U8Vector4: ...
    @property
    def loss(self) -> U8Vector4: ...
    @property
    def lost(self) -> U8Vector4: ...
    @property
    def losq(self) -> U8Vector4: ...
    @property
    def losp(self) -> U8Vector4: ...
    @property
    def loso(self) -> U8Vector4: ...
    @property
    def losl(self) -> U8Vector4: ...
    @property
    def lots(self) -> U8Vector4: ...
    @property
    def lott(self) -> U8Vector4: ...
    @property
    def lotq(self) -> U8Vector4: ...
    @property
    def lotp(self) -> U8Vector4: ...
    @property
    def loto(self) -> U8Vector4: ...
    @property
    def lotl(self) -> U8Vector4: ...
    @property
    def loqs(self) -> U8Vector4: ...
    @property
    def loqt(self) -> U8Vector4: ...
    @property
    def loqq(self) -> U8Vector4: ...
    @property
    def loqp(self) -> U8Vector4: ...
    @property
    def loqo(self) -> U8Vector4: ...
    @property
    def loql(self) -> U8Vector4: ...
    @property
    def lops(self) -> U8Vector4: ...
    @property
    def lopt(self) -> U8Vector4: ...
    @property
    def lopq(self) -> U8Vector4: ...
    @property
    def lopp(self) -> U8Vector4: ...
    @property
    def lopo(self) -> U8Vector4: ...
    @property
    def lopl(self) -> U8Vector4: ...
    @property
    def loos(self) -> U8Vector4: ...
    @property
    def loot(self) -> U8Vector4: ...
    @property
    def looq(self) -> U8Vector4: ...
    @property
    def loop(self) -> U8Vector4: ...
    @property
    def lols(self) -> U8Vector4: ...
    @property
    def lolt(self) -> U8Vector4: ...
    @property
    def lolq(self) -> U8Vector4: ...
    @property
    def lolp(self) -> U8Vector4: ...
    @property
    def llss(self) -> U8Vector4: ...
    @property
    def llst(self) -> U8Vector4: ...
    @property
    def llsq(self) -> U8Vector4: ...
    @property
    def llsp(self) -> U8Vector4: ...
    @property
    def llso(self) -> U8Vector4: ...
    @property
    def llsl(self) -> U8Vector4: ...
    @property
    def llts(self) -> U8Vector4: ...
    @property
    def lltt(self) -> U8Vector4: ...
    @property
    def lltq(self) -> U8Vector4: ...
    @property
    def lltp(self) -> U8Vector4: ...
    @property
    def llto(self) -> U8Vector4: ...
    @property
    def lltl(self) -> U8Vector4: ...
    @property
    def llqs(self) -> U8Vector4: ...
    @property
    def llqt(self) -> U8Vector4: ...
    @property
    def llqq(self) -> U8Vector4: ...
    @property
    def llqp(self) -> U8Vector4: ...
    @property
    def llqo(self) -> U8Vector4: ...
    @property
    def llql(self) -> U8Vector4: ...
    @property
    def llps(self) -> U8Vector4: ...
    @property
    def llpt(self) -> U8Vector4: ...
    @property
    def llpq(self) -> U8Vector4: ...
    @property
    def llpp(self) -> U8Vector4: ...
    @property
    def llpo(self) -> U8Vector4: ...
    @property
    def llpl(self) -> U8Vector4: ...
    @property
    def llos(self) -> U8Vector4: ...
    @property
    def llot(self) -> U8Vector4: ...
    @property
    def lloq(self) -> U8Vector4: ...
    @property
    def llop(self) -> U8Vector4: ...
    @property
    def llls(self) -> U8Vector4: ...
    @property
    def lllt(self) -> U8Vector4: ...
    @property
    def lllq(self) -> U8Vector4: ...
    @property
    def lllp(self) -> U8Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U8Vector2: ...
    @property
    def uv(self) -> U8Vector2: ...
    @property
    def uo(self) -> U8Vector2: ...
    @property
    def ul(self) -> U8Vector2: ...
    @property
    def vu(self) -> U8Vector2: ...
    @property
    def vv(self) -> U8Vector2: ...
    @property
    def vo(self) -> U8Vector2: ...
    @property
    def vl(self) -> U8Vector2: ...
    @property
    def ou(self) -> U8Vector2: ...
    @property
    def ov(self) -> U8Vector2: ...
    @property
    def lu(self) -> U8Vector2: ...
    @property
    def lv(self) -> U8Vector2: ...
    @property
    def uuu(self) -> U8Vector3: ...
    @property
    def uuv(self) -> U8Vector3: ...
    @property
    def uuo(self) -> U8Vector3: ...
    @property
    def uul(self) -> U8Vector3: ...
    @property
    def uvu(self) -> U8Vector3: ...
    @property
    def uvv(self) -> U8Vector3: ...
    @property
    def uvo(self) -> U8Vector3: ...
    @property
    def uvl(self) -> U8Vector3: ...
    @property
    def uou(self) -> U8Vector3: ...
    @property
    def uov(self) -> U8Vector3: ...
    @property
    def uoo(self) -> U8Vector3: ...
    @property
    def uol(self) -> U8Vector3: ...
    @property
    def ulu(self) -> U8Vector3: ...
    @property
    def ulv(self) -> U8Vector3: ...
    @property
    def ulo(self) -> U8Vector3: ...
    @property
    def ull(self) -> U8Vector3: ...
    @property
    def vuu(self) -> U8Vector3: ...
    @property
    def vuv(self) -> U8Vector3: ...
    @property
    def vuo(self) -> U8Vector3: ...
    @property
    def vul(self) -> U8Vector3: ...
    @property
    def vvu(self) -> U8Vector3: ...
    @property
    def vvv(self) -> U8Vector3: ...
    @property
    def vvo(self) -> U8Vector3: ...
    @property
    def vvl(self) -> U8Vector3: ...
    @property
    def vou(self) -> U8Vector3: ...
    @property
    def vov(self) -> U8Vector3: ...
    @property
    def voo(self) -> U8Vector3: ...
    @property
    def vol(self) -> U8Vector3: ...
    @property
    def vlu(self) -> U8Vector3: ...
    @property
    def vlv(self) -> U8Vector3: ...
    @property
    def vlo(self) -> U8Vector3: ...
    @property
    def vll(self) -> U8Vector3: ...
    @property
    def ouu(self) -> U8Vector3: ...
    @property
    def ouv(self) -> U8Vector3: ...
    @property
    def ouo(self) -> U8Vector3: ...
    @property
    def oul(self) -> U8Vector3: ...
    @property
    def ovu(self) -> U8Vector3: ...
    @property
    def ovv(self) -> U8Vector3: ...
    @property
    def ovo(self) -> U8Vector3: ...
    @property
    def ovl(self) -> U8Vector3: ...
    @property
    def oou(self) -> U8Vector3: ...
    @property
    def oov(self) -> U8Vector3: ...
    @property
    def olu(self) -> U8Vector3: ...
    @property
    def olv(self) -> U8Vector3: ...
    @property
    def luu(self) -> U8Vector3: ...
    @property
    def luv(self) -> U8Vector3: ...
    @property
    def luo(self) -> U8Vector3: ...
    @property
    def lul(self) -> U8Vector3: ...
    @property
    def lvu(self) -> U8Vector3: ...
    @property
    def lvv(self) -> U8Vector3: ...
    @property
    def lvo(self) -> U8Vector3: ...
    @property
    def lvl(self) -> U8Vector3: ...
    @property
    def lou(self) -> U8Vector3: ...
    @property
    def lov(self) -> U8Vector3: ...
    @property
    def llu(self) -> U8Vector3: ...
    @property
    def llv(self) -> U8Vector3: ...
    @property
    def uuuu(self) -> U8Vector4: ...
    @property
    def uuuv(self) -> U8Vector4: ...
    @property
    def uuuo(self) -> U8Vector4: ...
    @property
    def uuul(self) -> U8Vector4: ...
    @property
    def uuvu(self) -> U8Vector4: ...
    @property
    def uuvv(self) -> U8Vector4: ...
    @property
    def uuvo(self) -> U8Vector4: ...
    @property
    def uuvl(self) -> U8Vector4: ...
    @property
    def uuou(self) -> U8Vector4: ...
    @property
    def uuov(self) -> U8Vector4: ...
    @property
    def uuoo(self) -> U8Vector4: ...
    @property
    def uuol(self) -> U8Vector4: ...
    @property
    def uulu(self) -> U8Vector4: ...
    @property
    def uulv(self) -> U8Vector4: ...
    @property
    def uulo(self) -> U8Vector4: ...
    @property
    def uull(self) -> U8Vector4: ...
    @property
    def uvuu(self) -> U8Vector4: ...
    @property
    def uvuv(self) -> U8Vector4: ...
    @property
    def uvuo(self) -> U8Vector4: ...
    @property
    def uvul(self) -> U8Vector4: ...
    @property
    def uvvu(self) -> U8Vector4: ...
    @property
    def uvvv(self) -> U8Vector4: ...
    @property
    def uvvo(self) -> U8Vector4: ...
    @property
    def uvvl(self) -> U8Vector4: ...
    @property
    def uvou(self) -> U8Vector4: ...
    @property
    def uvov(self) -> U8Vector4: ...
    @property
    def uvoo(self) -> U8Vector4: ...
    @property
    def uvol(self) -> U8Vector4: ...
    @property
    def uvlu(self) -> U8Vector4: ...
    @property
    def uvlv(self) -> U8Vector4: ...
    @property
    def uvlo(self) -> U8Vector4: ...
    @property
    def uvll(self) -> U8Vector4: ...
    @property
    def uouu(self) -> U8Vector4: ...
    @property
    def uouv(self) -> U8Vector4: ...
    @property
    def uouo(self) -> U8Vector4: ...
    @property
    def uoul(self) -> U8Vector4: ...
    @property
    def uovu(self) -> U8Vector4: ...
    @property
    def uovv(self) -> U8Vector4: ...
    @property
    def uovo(self) -> U8Vector4: ...
    @property
    def uovl(self) -> U8Vector4: ...
    @property
    def uoou(self) -> U8Vector4: ...
    @property
    def uoov(self) -> U8Vector4: ...
    @property
    def uooo(self) -> U8Vector4: ...
    @property
    def uool(self) -> U8Vector4: ...
    @property
    def uolu(self) -> U8Vector4: ...
    @property
    def uolv(self) -> U8Vector4: ...
    @property
    def uolo(self) -> U8Vector4: ...
    @property
    def uoll(self) -> U8Vector4: ...
    @property
    def uluu(self) -> U8Vector4: ...
    @property
    def uluv(self) -> U8Vector4: ...
    @property
    def uluo(self) -> U8Vector4: ...
    @property
    def ulul(self) -> U8Vector4: ...
    @property
    def ulvu(self) -> U8Vector4: ...
    @property
    def ulvv(self) -> U8Vector4: ...
    @property
    def ulvo(self) -> U8Vector4: ...
    @property
    def ulvl(self) -> U8Vector4: ...
    @property
    def ulou(self) -> U8Vector4: ...
    @property
    def ulov(self) -> U8Vector4: ...
    @property
    def uloo(self) -> U8Vector4: ...
    @property
    def ulol(self) -> U8Vector4: ...
    @property
    def ullu(self) -> U8Vector4: ...
    @property
    def ullv(self) -> U8Vector4: ...
    @property
    def ullo(self) -> U8Vector4: ...
    @property
    def ulll(self) -> U8Vector4: ...
    @property
    def vuuu(self) -> U8Vector4: ...
    @property
    def vuuv(self) -> U8Vector4: ...
    @property
    def vuuo(self) -> U8Vector4: ...
    @property
    def vuul(self) -> U8Vector4: ...
    @property
    def vuvu(self) -> U8Vector4: ...
    @property
    def vuvv(self) -> U8Vector4: ...
    @property
    def vuvo(self) -> U8Vector4: ...
    @property
    def vuvl(self) -> U8Vector4: ...
    @property
    def vuou(self) -> U8Vector4: ...
    @property
    def vuov(self) -> U8Vector4: ...
    @property
    def vuoo(self) -> U8Vector4: ...
    @property
    def vuol(self) -> U8Vector4: ...
    @property
    def vulu(self) -> U8Vector4: ...
    @property
    def vulv(self) -> U8Vector4: ...
    @property
    def vulo(self) -> U8Vector4: ...
    @property
    def vull(self) -> U8Vector4: ...
    @property
    def vvuu(self) -> U8Vector4: ...
    @property
    def vvuv(self) -> U8Vector4: ...
    @property
    def vvuo(self) -> U8Vector4: ...
    @property
    def vvul(self) -> U8Vector4: ...
    @property
    def vvvu(self) -> U8Vector4: ...
    @property
    def vvvv(self) -> U8Vector4: ...
    @property
    def vvvo(self) -> U8Vector4: ...
    @property
    def vvvl(self) -> U8Vector4: ...
    @property
    def vvou(self) -> U8Vector4: ...
    @property
    def vvov(self) -> U8Vector4: ...
    @property
    def vvoo(self) -> U8Vector4: ...
    @property
    def vvol(self) -> U8Vector4: ...
    @property
    def vvlu(self) -> U8Vector4: ...
    @property
    def vvlv(self) -> U8Vector4: ...
    @property
    def vvlo(self) -> U8Vector4: ...
    @property
    def vvll(self) -> U8Vector4: ...
    @property
    def vouu(self) -> U8Vector4: ...
    @property
    def vouv(self) -> U8Vector4: ...
    @property
    def vouo(self) -> U8Vector4: ...
    @property
    def voul(self) -> U8Vector4: ...
    @property
    def vovu(self) -> U8Vector4: ...
    @property
    def vovv(self) -> U8Vector4: ...
    @property
    def vovo(self) -> U8Vector4: ...
    @property
    def vovl(self) -> U8Vector4: ...
    @property
    def voou(self) -> U8Vector4: ...
    @property
    def voov(self) -> U8Vector4: ...
    @property
    def vooo(self) -> U8Vector4: ...
    @property
    def vool(self) -> U8Vector4: ...
    @property
    def volu(self) -> U8Vector4: ...
    @property
    def volv(self) -> U8Vector4: ...
    @property
    def volo(self) -> U8Vector4: ...
    @property
    def voll(self) -> U8Vector4: ...
    @property
    def vluu(self) -> U8Vector4: ...
    @property
    def vluv(self) -> U8Vector4: ...
    @property
    def vluo(self) -> U8Vector4: ...
    @property
    def vlul(self) -> U8Vector4: ...
    @property
    def vlvu(self) -> U8Vector4: ...
    @property
    def vlvv(self) -> U8Vector4: ...
    @property
    def vlvo(self) -> U8Vector4: ...
    @property
    def vlvl(self) -> U8Vector4: ...
    @property
    def vlou(self) -> U8Vector4: ...
    @property
    def vlov(self) -> U8Vector4: ...
    @property
    def vloo(self) -> U8Vector4: ...
    @property
    def vlol(self) -> U8Vector4: ...
    @property
    def vllu(self) -> U8Vector4: ...
    @property
    def vllv(self) -> U8Vector4: ...
    @property
    def vllo(self) -> U8Vector4: ...
    @property
    def vlll(self) -> U8Vector4: ...
    @property
    def ouuu(self) -> U8Vector4: ...
    @property
    def ouuv(self) -> U8Vector4: ...
    @property
    def ouuo(self) -> U8Vector4: ...
    @property
    def ouul(self) -> U8Vector4: ...
    @property
    def ouvu(self) -> U8Vector4: ...
    @property
    def ouvv(self) -> U8Vector4: ...
    @property
    def ouvo(self) -> U8Vector4: ...
    @property
    def ouvl(self) -> U8Vector4: ...
    @property
    def ouou(self) -> U8Vector4: ...
    @property
    def ouov(self) -> U8Vector4: ...
    @property
    def ouoo(self) -> U8Vector4: ...
    @property
    def ouol(self) -> U8Vector4: ...
    @property
    def oulu(self) -> U8Vector4: ...
    @property
    def oulv(self) -> U8Vector4: ...
    @property
    def oulo(self) -> U8Vector4: ...
    @property
    def oull(self) -> U8Vector4: ...
    @property
    def ovuu(self) -> U8Vector4: ...
    @property
    def ovuv(self) -> U8Vector4: ...
    @property
    def ovuo(self) -> U8Vector4: ...
    @property
    def ovul(self) -> U8Vector4: ...
    @property
    def ovvu(self) -> U8Vector4: ...
    @property
    def ovvv(self) -> U8Vector4: ...
    @property
    def ovvo(self) -> U8Vector4: ...
    @property
    def ovvl(self) -> U8Vector4: ...
    @property
    def ovou(self) -> U8Vector4: ...
    @property
    def ovov(self) -> U8Vector4: ...
    @property
    def ovoo(self) -> U8Vector4: ...
    @property
    def ovol(self) -> U8Vector4: ...
    @property
    def ovlu(self) -> U8Vector4: ...
    @property
    def ovlv(self) -> U8Vector4: ...
    @property
    def ovlo(self) -> U8Vector4: ...
    @property
    def ovll(self) -> U8Vector4: ...
    @property
    def oouu(self) -> U8Vector4: ...
    @property
    def oouv(self) -> U8Vector4: ...
    @property
    def oouo(self) -> U8Vector4: ...
    @property
    def ooul(self) -> U8Vector4: ...
    @property
    def oovu(self) -> U8Vector4: ...
    @property
    def oovv(self) -> U8Vector4: ...
    @property
    def oovo(self) -> U8Vector4: ...
    @property
    def oovl(self) -> U8Vector4: ...
    @property
    def ooou(self) -> U8Vector4: ...
    @property
    def ooov(self) -> U8Vector4: ...
    @property
    def oolu(self) -> U8Vector4: ...
    @property
    def oolv(self) -> U8Vector4: ...
    @property
    def oluu(self) -> U8Vector4: ...
    @property
    def oluv(self) -> U8Vector4: ...
    @property
    def oluo(self) -> U8Vector4: ...
    @property
    def olul(self) -> U8Vector4: ...
    @property
    def olvu(self) -> U8Vector4: ...
    @property
    def olvv(self) -> U8Vector4: ...
    @property
    def olvo(self) -> U8Vector4: ...
    @property
    def olvl(self) -> U8Vector4: ...
    @property
    def olou(self) -> U8Vector4: ...
    @property
    def olov(self) -> U8Vector4: ...
    @property
    def ollu(self) -> U8Vector4: ...
    @property
    def ollv(self) -> U8Vector4: ...
    @property
    def luuu(self) -> U8Vector4: ...
    @property
    def luuv(self) -> U8Vector4: ...
    @property
    def luuo(self) -> U8Vector4: ...
    @property
    def luul(self) -> U8Vector4: ...
    @property
    def luvu(self) -> U8Vector4: ...
    @property
    def luvv(self) -> U8Vector4: ...
    @property
    def luvo(self) -> U8Vector4: ...
    @property
    def luvl(self) -> U8Vector4: ...
    @property
    def luou(self) -> U8Vector4: ...
    @property
    def luov(self) -> U8Vector4: ...
    @property
    def luoo(self) -> U8Vector4: ...
    @property
    def luol(self) -> U8Vector4: ...
    @property
    def lulu(self) -> U8Vector4: ...
    @property
    def lulv(self) -> U8Vector4: ...
    @property
    def lulo(self) -> U8Vector4: ...
    @property
    def lull(self) -> U8Vector4: ...
    @property
    def lvuu(self) -> U8Vector4: ...
    @property
    def lvuv(self) -> U8Vector4: ...
    @property
    def lvuo(self) -> U8Vector4: ...
    @property
    def lvul(self) -> U8Vector4: ...
    @property
    def lvvu(self) -> U8Vector4: ...
    @property
    def lvvv(self) -> U8Vector4: ...
    @property
    def lvvo(self) -> U8Vector4: ...
    @property
    def lvvl(self) -> U8Vector4: ...
    @property
    def lvou(self) -> U8Vector4: ...
    @property
    def lvov(self) -> U8Vector4: ...
    @property
    def lvoo(self) -> U8Vector4: ...
    @property
    def lvol(self) -> U8Vector4: ...
    @property
    def lvlu(self) -> U8Vector4: ...
    @property
    def lvlv(self) -> U8Vector4: ...
    @property
    def lvlo(self) -> U8Vector4: ...
    @property
    def lvll(self) -> U8Vector4: ...
    @property
    def louu(self) -> U8Vector4: ...
    @property
    def louv(self) -> U8Vector4: ...
    @property
    def louo(self) -> U8Vector4: ...
    @property
    def loul(self) -> U8Vector4: ...
    @property
    def lovu(self) -> U8Vector4: ...
    @property
    def lovv(self) -> U8Vector4: ...
    @property
    def lovo(self) -> U8Vector4: ...
    @property
    def lovl(self) -> U8Vector4: ...
    @property
    def loou(self) -> U8Vector4: ...
    @property
    def loov(self) -> U8Vector4: ...
    @property
    def lolu(self) -> U8Vector4: ...
    @property
    def lolv(self) -> U8Vector4: ...
    @property
    def lluu(self) -> U8Vector4: ...
    @property
    def lluv(self) -> U8Vector4: ...
    @property
    def lluo(self) -> U8Vector4: ...
    @property
    def llul(self) -> U8Vector4: ...
    @property
    def llvu(self) -> U8Vector4: ...
    @property
    def llvv(self) -> U8Vector4: ...
    @property
    def llvo(self) -> U8Vector4: ...
    @property
    def llvl(self) -> U8Vector4: ...
    @property
    def llou(self) -> U8Vector4: ...
    @property
    def llov(self) -> U8Vector4: ...
    @property
    def lllu(self) -> U8Vector4: ...
    @property
    def lllv(self) -> U8Vector4: ...
    @property
    def oo(self) -> U8Vector2: ...
    @property
    def ol(self) -> U8Vector2: ...
    @property
    def lo(self) -> U8Vector2: ...
    @property
    def ll(self) -> U8Vector2: ...
    @property
    def ooo(self) -> U8Vector3: ...
    @property
    def ool(self) -> U8Vector3: ...
    @property
    def olo(self) -> U8Vector3: ...
    @property
    def oll(self) -> U8Vector3: ...
    @property
    def loo(self) -> U8Vector3: ...
    @property
    def lol(self) -> U8Vector3: ...
    @property
    def llo(self) -> U8Vector3: ...
    @property
    def lll(self) -> U8Vector3: ...
    @property
    def oooo(self) -> U8Vector4: ...
    @property
    def oool(self) -> U8Vector4: ...
    @property
    def oolo(self) -> U8Vector4: ...
    @property
    def ooll(self) -> U8Vector4: ...
    @property
    def oloo(self) -> U8Vector4: ...
    @property
    def olol(self) -> U8Vector4: ...
    @property
    def ollo(self) -> U8Vector4: ...
    @property
    def olll(self) -> U8Vector4: ...
    @property
    def looo(self) -> U8Vector4: ...
    @property
    def lool(self) -> U8Vector4: ...
    @property
    def lolo(self) -> U8Vector4: ...
    @property
    def loll(self) -> U8Vector4: ...
    @property
    def lloo(self) -> U8Vector4: ...
    @property
    def llol(self) -> U8Vector4: ...
    @property
    def lllo(self) -> U8Vector4: ...
    @property
    def llll(self) -> U8Vector4: ...
    def min(self, n: Number, /) -> U8Vector4: ...
    def max(self, n: Number, /) -> U8Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> U8Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U8Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector4: ...

@final
class U8Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U8Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U8Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> U8Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> U8Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U8Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[U8Vector4]: ...

@final
class I16Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __add__(self, other: Number) -> I16Vector4: ...
    @overload
    def __radd__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __radd__(self, other: Number) -> I16Vector4: ...
    @overload
    def __sub__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __sub__(self, other: Number) -> I16Vector4: ...
    @overload
    def __rsub__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> I16Vector4: ...
    @overload
    def __mul__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __mul__(self, other: Number) -> I16Vector4: ...
    @overload
    def __rmul__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> I16Vector4: ...
    @overload
    def __truediv__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> I16Vector4: ...
    @overload
    def __rtruediv__(self, other: I16Vector4) -> I16Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> I16Vector4: ...
    def __neg__(self) -> I16Vector4: ...
    def __abs__(self) -> I16Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> I16Vector2: ...
    @property
    def xy(self) -> I16Vector2: ...
    @property
    def xz(self) -> I16Vector2: ...
    @property
    def xw(self) -> I16Vector2: ...
    @property
    def xo(self) -> I16Vector2: ...
    @property
    def xl(self) -> I16Vector2: ...
    @property
    def yx(self) -> I16Vector2: ...
    @property
    def yy(self) -> I16Vector2: ...
    @property
    def yz(self) -> I16Vector2: ...
    @property
    def yw(self) -> I16Vector2: ...
    @property
    def yo(self) -> I16Vector2: ...
    @property
    def yl(self) -> I16Vector2: ...
    @property
    def zx(self) -> I16Vector2: ...
    @property
    def zy(self) -> I16Vector2: ...
    @property
    def zz(self) -> I16Vector2: ...
    @property
    def zw(self) -> I16Vector2: ...
    @property
    def zo(self) -> I16Vector2: ...
    @property
    def zl(self) -> I16Vector2: ...
    @property
    def wx(self) -> I16Vector2: ...
    @property
    def wy(self) -> I16Vector2: ...
    @property
    def wz(self) -> I16Vector2: ...
    @property
    def ww(self) -> I16Vector2: ...
    @property
    def wo(self) -> I16Vector2: ...
    @property
    def wl(self) -> I16Vector2: ...
    @property
    def ox(self) -> I16Vector2: ...
    @property
    def oy(self) -> I16Vector2: ...
    @property
    def oz(self) -> I16Vector2: ...
    @property
    def ow(self) -> I16Vector2: ...
    @property
    def lx(self) -> I16Vector2: ...
    @property
    def ly(self) -> I16Vector2: ...
    @property
    def lz(self) -> I16Vector2: ...
    @property
    def lw(self) -> I16Vector2: ...
    @property
    def xxx(self) -> I16Vector3: ...
    @property
    def xxy(self) -> I16Vector3: ...
    @property
    def xxz(self) -> I16Vector3: ...
    @property
    def xxw(self) -> I16Vector3: ...
    @property
    def xxo(self) -> I16Vector3: ...
    @property
    def xxl(self) -> I16Vector3: ...
    @property
    def xyx(self) -> I16Vector3: ...
    @property
    def xyy(self) -> I16Vector3: ...
    @property
    def xyz(self) -> I16Vector3: ...
    @property
    def xyw(self) -> I16Vector3: ...
    @property
    def xyo(self) -> I16Vector3: ...
    @property
    def xyl(self) -> I16Vector3: ...
    @property
    def xzx(self) -> I16Vector3: ...
    @property
    def xzy(self) -> I16Vector3: ...
    @property
    def xzz(self) -> I16Vector3: ...
    @property
    def xzw(self) -> I16Vector3: ...
    @property
    def xzo(self) -> I16Vector3: ...
    @property
    def xzl(self) -> I16Vector3: ...
    @property
    def xwx(self) -> I16Vector3: ...
    @property
    def xwy(self) -> I16Vector3: ...
    @property
    def xwz(self) -> I16Vector3: ...
    @property
    def xww(self) -> I16Vector3: ...
    @property
    def xwo(self) -> I16Vector3: ...
    @property
    def xwl(self) -> I16Vector3: ...
    @property
    def xox(self) -> I16Vector3: ...
    @property
    def xoy(self) -> I16Vector3: ...
    @property
    def xoz(self) -> I16Vector3: ...
    @property
    def xow(self) -> I16Vector3: ...
    @property
    def xoo(self) -> I16Vector3: ...
    @property
    def xol(self) -> I16Vector3: ...
    @property
    def xlx(self) -> I16Vector3: ...
    @property
    def xly(self) -> I16Vector3: ...
    @property
    def xlz(self) -> I16Vector3: ...
    @property
    def xlw(self) -> I16Vector3: ...
    @property
    def xlo(self) -> I16Vector3: ...
    @property
    def xll(self) -> I16Vector3: ...
    @property
    def yxx(self) -> I16Vector3: ...
    @property
    def yxy(self) -> I16Vector3: ...
    @property
    def yxz(self) -> I16Vector3: ...
    @property
    def yxw(self) -> I16Vector3: ...
    @property
    def yxo(self) -> I16Vector3: ...
    @property
    def yxl(self) -> I16Vector3: ...
    @property
    def yyx(self) -> I16Vector3: ...
    @property
    def yyy(self) -> I16Vector3: ...
    @property
    def yyz(self) -> I16Vector3: ...
    @property
    def yyw(self) -> I16Vector3: ...
    @property
    def yyo(self) -> I16Vector3: ...
    @property
    def yyl(self) -> I16Vector3: ...
    @property
    def yzx(self) -> I16Vector3: ...
    @property
    def yzy(self) -> I16Vector3: ...
    @property
    def yzz(self) -> I16Vector3: ...
    @property
    def yzw(self) -> I16Vector3: ...
    @property
    def yzo(self) -> I16Vector3: ...
    @property
    def yzl(self) -> I16Vector3: ...
    @property
    def ywx(self) -> I16Vector3: ...
    @property
    def ywy(self) -> I16Vector3: ...
    @property
    def ywz(self) -> I16Vector3: ...
    @property
    def yww(self) -> I16Vector3: ...
    @property
    def ywo(self) -> I16Vector3: ...
    @property
    def ywl(self) -> I16Vector3: ...
    @property
    def yox(self) -> I16Vector3: ...
    @property
    def yoy(self) -> I16Vector3: ...
    @property
    def yoz(self) -> I16Vector3: ...
    @property
    def yow(self) -> I16Vector3: ...
    @property
    def yoo(self) -> I16Vector3: ...
    @property
    def yol(self) -> I16Vector3: ...
    @property
    def ylx(self) -> I16Vector3: ...
    @property
    def yly(self) -> I16Vector3: ...
    @property
    def ylz(self) -> I16Vector3: ...
    @property
    def ylw(self) -> I16Vector3: ...
    @property
    def ylo(self) -> I16Vector3: ...
    @property
    def yll(self) -> I16Vector3: ...
    @property
    def zxx(self) -> I16Vector3: ...
    @property
    def zxy(self) -> I16Vector3: ...
    @property
    def zxz(self) -> I16Vector3: ...
    @property
    def zxw(self) -> I16Vector3: ...
    @property
    def zxo(self) -> I16Vector3: ...
    @property
    def zxl(self) -> I16Vector3: ...
    @property
    def zyx(self) -> I16Vector3: ...
    @property
    def zyy(self) -> I16Vector3: ...
    @property
    def zyz(self) -> I16Vector3: ...
    @property
    def zyw(self) -> I16Vector3: ...
    @property
    def zyo(self) -> I16Vector3: ...
    @property
    def zyl(self) -> I16Vector3: ...
    @property
    def zzx(self) -> I16Vector3: ...
    @property
    def zzy(self) -> I16Vector3: ...
    @property
    def zzz(self) -> I16Vector3: ...
    @property
    def zzw(self) -> I16Vector3: ...
    @property
    def zzo(self) -> I16Vector3: ...
    @property
    def zzl(self) -> I16Vector3: ...
    @property
    def zwx(self) -> I16Vector3: ...
    @property
    def zwy(self) -> I16Vector3: ...
    @property
    def zwz(self) -> I16Vector3: ...
    @property
    def zww(self) -> I16Vector3: ...
    @property
    def zwo(self) -> I16Vector3: ...
    @property
    def zwl(self) -> I16Vector3: ...
    @property
    def zox(self) -> I16Vector3: ...
    @property
    def zoy(self) -> I16Vector3: ...
    @property
    def zoz(self) -> I16Vector3: ...
    @property
    def zow(self) -> I16Vector3: ...
    @property
    def zoo(self) -> I16Vector3: ...
    @property
    def zol(self) -> I16Vector3: ...
    @property
    def zlx(self) -> I16Vector3: ...
    @property
    def zly(self) -> I16Vector3: ...
    @property
    def zlz(self) -> I16Vector3: ...
    @property
    def zlw(self) -> I16Vector3: ...
    @property
    def zlo(self) -> I16Vector3: ...
    @property
    def zll(self) -> I16Vector3: ...
    @property
    def wxx(self) -> I16Vector3: ...
    @property
    def wxy(self) -> I16Vector3: ...
    @property
    def wxz(self) -> I16Vector3: ...
    @property
    def wxw(self) -> I16Vector3: ...
    @property
    def wxo(self) -> I16Vector3: ...
    @property
    def wxl(self) -> I16Vector3: ...
    @property
    def wyx(self) -> I16Vector3: ...
    @property
    def wyy(self) -> I16Vector3: ...
    @property
    def wyz(self) -> I16Vector3: ...
    @property
    def wyw(self) -> I16Vector3: ...
    @property
    def wyo(self) -> I16Vector3: ...
    @property
    def wyl(self) -> I16Vector3: ...
    @property
    def wzx(self) -> I16Vector3: ...
    @property
    def wzy(self) -> I16Vector3: ...
    @property
    def wzz(self) -> I16Vector3: ...
    @property
    def wzw(self) -> I16Vector3: ...
    @property
    def wzo(self) -> I16Vector3: ...
    @property
    def wzl(self) -> I16Vector3: ...
    @property
    def wwx(self) -> I16Vector3: ...
    @property
    def wwy(self) -> I16Vector3: ...
    @property
    def wwz(self) -> I16Vector3: ...
    @property
    def www(self) -> I16Vector3: ...
    @property
    def wwo(self) -> I16Vector3: ...
    @property
    def wwl(self) -> I16Vector3: ...
    @property
    def wox(self) -> I16Vector3: ...
    @property
    def woy(self) -> I16Vector3: ...
    @property
    def woz(self) -> I16Vector3: ...
    @property
    def wow(self) -> I16Vector3: ...
    @property
    def woo(self) -> I16Vector3: ...
    @property
    def wol(self) -> I16Vector3: ...
    @property
    def wlx(self) -> I16Vector3: ...
    @property
    def wly(self) -> I16Vector3: ...
    @property
    def wlz(self) -> I16Vector3: ...
    @property
    def wlw(self) -> I16Vector3: ...
    @property
    def wlo(self) -> I16Vector3: ...
    @property
    def wll(self) -> I16Vector3: ...
    @property
    def oxx(self) -> I16Vector3: ...
    @property
    def oxy(self) -> I16Vector3: ...
    @property
    def oxz(self) -> I16Vector3: ...
    @property
    def oxw(self) -> I16Vector3: ...
    @property
    def oxo(self) -> I16Vector3: ...
    @property
    def oxl(self) -> I16Vector3: ...
    @property
    def oyx(self) -> I16Vector3: ...
    @property
    def oyy(self) -> I16Vector3: ...
    @property
    def oyz(self) -> I16Vector3: ...
    @property
    def oyw(self) -> I16Vector3: ...
    @property
    def oyo(self) -> I16Vector3: ...
    @property
    def oyl(self) -> I16Vector3: ...
    @property
    def ozx(self) -> I16Vector3: ...
    @property
    def ozy(self) -> I16Vector3: ...
    @property
    def ozz(self) -> I16Vector3: ...
    @property
    def ozw(self) -> I16Vector3: ...
    @property
    def ozo(self) -> I16Vector3: ...
    @property
    def ozl(self) -> I16Vector3: ...
    @property
    def owx(self) -> I16Vector3: ...
    @property
    def owy(self) -> I16Vector3: ...
    @property
    def owz(self) -> I16Vector3: ...
    @property
    def oww(self) -> I16Vector3: ...
    @property
    def owo(self) -> I16Vector3: ...
    @property
    def owl(self) -> I16Vector3: ...
    @property
    def oox(self) -> I16Vector3: ...
    @property
    def ooy(self) -> I16Vector3: ...
    @property
    def ooz(self) -> I16Vector3: ...
    @property
    def oow(self) -> I16Vector3: ...
    @property
    def olx(self) -> I16Vector3: ...
    @property
    def oly(self) -> I16Vector3: ...
    @property
    def olz(self) -> I16Vector3: ...
    @property
    def olw(self) -> I16Vector3: ...
    @property
    def lxx(self) -> I16Vector3: ...
    @property
    def lxy(self) -> I16Vector3: ...
    @property
    def lxz(self) -> I16Vector3: ...
    @property
    def lxw(self) -> I16Vector3: ...
    @property
    def lxo(self) -> I16Vector3: ...
    @property
    def lxl(self) -> I16Vector3: ...
    @property
    def lyx(self) -> I16Vector3: ...
    @property
    def lyy(self) -> I16Vector3: ...
    @property
    def lyz(self) -> I16Vector3: ...
    @property
    def lyw(self) -> I16Vector3: ...
    @property
    def lyo(self) -> I16Vector3: ...
    @property
    def lyl(self) -> I16Vector3: ...
    @property
    def lzx(self) -> I16Vector3: ...
    @property
    def lzy(self) -> I16Vector3: ...
    @property
    def lzz(self) -> I16Vector3: ...
    @property
    def lzw(self) -> I16Vector3: ...
    @property
    def lzo(self) -> I16Vector3: ...
    @property
    def lzl(self) -> I16Vector3: ...
    @property
    def lwx(self) -> I16Vector3: ...
    @property
    def lwy(self) -> I16Vector3: ...
    @property
    def lwz(self) -> I16Vector3: ...
    @property
    def lww(self) -> I16Vector3: ...
    @property
    def lwo(self) -> I16Vector3: ...
    @property
    def lwl(self) -> I16Vector3: ...
    @property
    def lox(self) -> I16Vector3: ...
    @property
    def loy(self) -> I16Vector3: ...
    @property
    def loz(self) -> I16Vector3: ...
    @property
    def low(self) -> I16Vector3: ...
    @property
    def llx(self) -> I16Vector3: ...
    @property
    def lly(self) -> I16Vector3: ...
    @property
    def llz(self) -> I16Vector3: ...
    @property
    def llw(self) -> I16Vector3: ...
    @property
    def xxxx(self) -> I16Vector4: ...
    @property
    def xxxy(self) -> I16Vector4: ...
    @property
    def xxxz(self) -> I16Vector4: ...
    @property
    def xxxw(self) -> I16Vector4: ...
    @property
    def xxxo(self) -> I16Vector4: ...
    @property
    def xxxl(self) -> I16Vector4: ...
    @property
    def xxyx(self) -> I16Vector4: ...
    @property
    def xxyy(self) -> I16Vector4: ...
    @property
    def xxyz(self) -> I16Vector4: ...
    @property
    def xxyw(self) -> I16Vector4: ...
    @property
    def xxyo(self) -> I16Vector4: ...
    @property
    def xxyl(self) -> I16Vector4: ...
    @property
    def xxzx(self) -> I16Vector4: ...
    @property
    def xxzy(self) -> I16Vector4: ...
    @property
    def xxzz(self) -> I16Vector4: ...
    @property
    def xxzw(self) -> I16Vector4: ...
    @property
    def xxzo(self) -> I16Vector4: ...
    @property
    def xxzl(self) -> I16Vector4: ...
    @property
    def xxwx(self) -> I16Vector4: ...
    @property
    def xxwy(self) -> I16Vector4: ...
    @property
    def xxwz(self) -> I16Vector4: ...
    @property
    def xxww(self) -> I16Vector4: ...
    @property
    def xxwo(self) -> I16Vector4: ...
    @property
    def xxwl(self) -> I16Vector4: ...
    @property
    def xxox(self) -> I16Vector4: ...
    @property
    def xxoy(self) -> I16Vector4: ...
    @property
    def xxoz(self) -> I16Vector4: ...
    @property
    def xxow(self) -> I16Vector4: ...
    @property
    def xxoo(self) -> I16Vector4: ...
    @property
    def xxol(self) -> I16Vector4: ...
    @property
    def xxlx(self) -> I16Vector4: ...
    @property
    def xxly(self) -> I16Vector4: ...
    @property
    def xxlz(self) -> I16Vector4: ...
    @property
    def xxlw(self) -> I16Vector4: ...
    @property
    def xxlo(self) -> I16Vector4: ...
    @property
    def xxll(self) -> I16Vector4: ...
    @property
    def xyxx(self) -> I16Vector4: ...
    @property
    def xyxy(self) -> I16Vector4: ...
    @property
    def xyxz(self) -> I16Vector4: ...
    @property
    def xyxw(self) -> I16Vector4: ...
    @property
    def xyxo(self) -> I16Vector4: ...
    @property
    def xyxl(self) -> I16Vector4: ...
    @property
    def xyyx(self) -> I16Vector4: ...
    @property
    def xyyy(self) -> I16Vector4: ...
    @property
    def xyyz(self) -> I16Vector4: ...
    @property
    def xyyw(self) -> I16Vector4: ...
    @property
    def xyyo(self) -> I16Vector4: ...
    @property
    def xyyl(self) -> I16Vector4: ...
    @property
    def xyzx(self) -> I16Vector4: ...
    @property
    def xyzy(self) -> I16Vector4: ...
    @property
    def xyzz(self) -> I16Vector4: ...
    @property
    def xyzw(self) -> I16Vector4: ...
    @property
    def xyzo(self) -> I16Vector4: ...
    @property
    def xyzl(self) -> I16Vector4: ...
    @property
    def xywx(self) -> I16Vector4: ...
    @property
    def xywy(self) -> I16Vector4: ...
    @property
    def xywz(self) -> I16Vector4: ...
    @property
    def xyww(self) -> I16Vector4: ...
    @property
    def xywo(self) -> I16Vector4: ...
    @property
    def xywl(self) -> I16Vector4: ...
    @property
    def xyox(self) -> I16Vector4: ...
    @property
    def xyoy(self) -> I16Vector4: ...
    @property
    def xyoz(self) -> I16Vector4: ...
    @property
    def xyow(self) -> I16Vector4: ...
    @property
    def xyoo(self) -> I16Vector4: ...
    @property
    def xyol(self) -> I16Vector4: ...
    @property
    def xylx(self) -> I16Vector4: ...
    @property
    def xyly(self) -> I16Vector4: ...
    @property
    def xylz(self) -> I16Vector4: ...
    @property
    def xylw(self) -> I16Vector4: ...
    @property
    def xylo(self) -> I16Vector4: ...
    @property
    def xyll(self) -> I16Vector4: ...
    @property
    def xzxx(self) -> I16Vector4: ...
    @property
    def xzxy(self) -> I16Vector4: ...
    @property
    def xzxz(self) -> I16Vector4: ...
    @property
    def xzxw(self) -> I16Vector4: ...
    @property
    def xzxo(self) -> I16Vector4: ...
    @property
    def xzxl(self) -> I16Vector4: ...
    @property
    def xzyx(self) -> I16Vector4: ...
    @property
    def xzyy(self) -> I16Vector4: ...
    @property
    def xzyz(self) -> I16Vector4: ...
    @property
    def xzyw(self) -> I16Vector4: ...
    @property
    def xzyo(self) -> I16Vector4: ...
    @property
    def xzyl(self) -> I16Vector4: ...
    @property
    def xzzx(self) -> I16Vector4: ...
    @property
    def xzzy(self) -> I16Vector4: ...
    @property
    def xzzz(self) -> I16Vector4: ...
    @property
    def xzzw(self) -> I16Vector4: ...
    @property
    def xzzo(self) -> I16Vector4: ...
    @property
    def xzzl(self) -> I16Vector4: ...
    @property
    def xzwx(self) -> I16Vector4: ...
    @property
    def xzwy(self) -> I16Vector4: ...
    @property
    def xzwz(self) -> I16Vector4: ...
    @property
    def xzww(self) -> I16Vector4: ...
    @property
    def xzwo(self) -> I16Vector4: ...
    @property
    def xzwl(self) -> I16Vector4: ...
    @property
    def xzox(self) -> I16Vector4: ...
    @property
    def xzoy(self) -> I16Vector4: ...
    @property
    def xzoz(self) -> I16Vector4: ...
    @property
    def xzow(self) -> I16Vector4: ...
    @property
    def xzoo(self) -> I16Vector4: ...
    @property
    def xzol(self) -> I16Vector4: ...
    @property
    def xzlx(self) -> I16Vector4: ...
    @property
    def xzly(self) -> I16Vector4: ...
    @property
    def xzlz(self) -> I16Vector4: ...
    @property
    def xzlw(self) -> I16Vector4: ...
    @property
    def xzlo(self) -> I16Vector4: ...
    @property
    def xzll(self) -> I16Vector4: ...
    @property
    def xwxx(self) -> I16Vector4: ...
    @property
    def xwxy(self) -> I16Vector4: ...
    @property
    def xwxz(self) -> I16Vector4: ...
    @property
    def xwxw(self) -> I16Vector4: ...
    @property
    def xwxo(self) -> I16Vector4: ...
    @property
    def xwxl(self) -> I16Vector4: ...
    @property
    def xwyx(self) -> I16Vector4: ...
    @property
    def xwyy(self) -> I16Vector4: ...
    @property
    def xwyz(self) -> I16Vector4: ...
    @property
    def xwyw(self) -> I16Vector4: ...
    @property
    def xwyo(self) -> I16Vector4: ...
    @property
    def xwyl(self) -> I16Vector4: ...
    @property
    def xwzx(self) -> I16Vector4: ...
    @property
    def xwzy(self) -> I16Vector4: ...
    @property
    def xwzz(self) -> I16Vector4: ...
    @property
    def xwzw(self) -> I16Vector4: ...
    @property
    def xwzo(self) -> I16Vector4: ...
    @property
    def xwzl(self) -> I16Vector4: ...
    @property
    def xwwx(self) -> I16Vector4: ...
    @property
    def xwwy(self) -> I16Vector4: ...
    @property
    def xwwz(self) -> I16Vector4: ...
    @property
    def xwww(self) -> I16Vector4: ...
    @property
    def xwwo(self) -> I16Vector4: ...
    @property
    def xwwl(self) -> I16Vector4: ...
    @property
    def xwox(self) -> I16Vector4: ...
    @property
    def xwoy(self) -> I16Vector4: ...
    @property
    def xwoz(self) -> I16Vector4: ...
    @property
    def xwow(self) -> I16Vector4: ...
    @property
    def xwoo(self) -> I16Vector4: ...
    @property
    def xwol(self) -> I16Vector4: ...
    @property
    def xwlx(self) -> I16Vector4: ...
    @property
    def xwly(self) -> I16Vector4: ...
    @property
    def xwlz(self) -> I16Vector4: ...
    @property
    def xwlw(self) -> I16Vector4: ...
    @property
    def xwlo(self) -> I16Vector4: ...
    @property
    def xwll(self) -> I16Vector4: ...
    @property
    def xoxx(self) -> I16Vector4: ...
    @property
    def xoxy(self) -> I16Vector4: ...
    @property
    def xoxz(self) -> I16Vector4: ...
    @property
    def xoxw(self) -> I16Vector4: ...
    @property
    def xoxo(self) -> I16Vector4: ...
    @property
    def xoxl(self) -> I16Vector4: ...
    @property
    def xoyx(self) -> I16Vector4: ...
    @property
    def xoyy(self) -> I16Vector4: ...
    @property
    def xoyz(self) -> I16Vector4: ...
    @property
    def xoyw(self) -> I16Vector4: ...
    @property
    def xoyo(self) -> I16Vector4: ...
    @property
    def xoyl(self) -> I16Vector4: ...
    @property
    def xozx(self) -> I16Vector4: ...
    @property
    def xozy(self) -> I16Vector4: ...
    @property
    def xozz(self) -> I16Vector4: ...
    @property
    def xozw(self) -> I16Vector4: ...
    @property
    def xozo(self) -> I16Vector4: ...
    @property
    def xozl(self) -> I16Vector4: ...
    @property
    def xowx(self) -> I16Vector4: ...
    @property
    def xowy(self) -> I16Vector4: ...
    @property
    def xowz(self) -> I16Vector4: ...
    @property
    def xoww(self) -> I16Vector4: ...
    @property
    def xowo(self) -> I16Vector4: ...
    @property
    def xowl(self) -> I16Vector4: ...
    @property
    def xoox(self) -> I16Vector4: ...
    @property
    def xooy(self) -> I16Vector4: ...
    @property
    def xooz(self) -> I16Vector4: ...
    @property
    def xoow(self) -> I16Vector4: ...
    @property
    def xooo(self) -> I16Vector4: ...
    @property
    def xool(self) -> I16Vector4: ...
    @property
    def xolx(self) -> I16Vector4: ...
    @property
    def xoly(self) -> I16Vector4: ...
    @property
    def xolz(self) -> I16Vector4: ...
    @property
    def xolw(self) -> I16Vector4: ...
    @property
    def xolo(self) -> I16Vector4: ...
    @property
    def xoll(self) -> I16Vector4: ...
    @property
    def xlxx(self) -> I16Vector4: ...
    @property
    def xlxy(self) -> I16Vector4: ...
    @property
    def xlxz(self) -> I16Vector4: ...
    @property
    def xlxw(self) -> I16Vector4: ...
    @property
    def xlxo(self) -> I16Vector4: ...
    @property
    def xlxl(self) -> I16Vector4: ...
    @property
    def xlyx(self) -> I16Vector4: ...
    @property
    def xlyy(self) -> I16Vector4: ...
    @property
    def xlyz(self) -> I16Vector4: ...
    @property
    def xlyw(self) -> I16Vector4: ...
    @property
    def xlyo(self) -> I16Vector4: ...
    @property
    def xlyl(self) -> I16Vector4: ...
    @property
    def xlzx(self) -> I16Vector4: ...
    @property
    def xlzy(self) -> I16Vector4: ...
    @property
    def xlzz(self) -> I16Vector4: ...
    @property
    def xlzw(self) -> I16Vector4: ...
    @property
    def xlzo(self) -> I16Vector4: ...
    @property
    def xlzl(self) -> I16Vector4: ...
    @property
    def xlwx(self) -> I16Vector4: ...
    @property
    def xlwy(self) -> I16Vector4: ...
    @property
    def xlwz(self) -> I16Vector4: ...
    @property
    def xlww(self) -> I16Vector4: ...
    @property
    def xlwo(self) -> I16Vector4: ...
    @property
    def xlwl(self) -> I16Vector4: ...
    @property
    def xlox(self) -> I16Vector4: ...
    @property
    def xloy(self) -> I16Vector4: ...
    @property
    def xloz(self) -> I16Vector4: ...
    @property
    def xlow(self) -> I16Vector4: ...
    @property
    def xloo(self) -> I16Vector4: ...
    @property
    def xlol(self) -> I16Vector4: ...
    @property
    def xllx(self) -> I16Vector4: ...
    @property
    def xlly(self) -> I16Vector4: ...
    @property
    def xllz(self) -> I16Vector4: ...
    @property
    def xllw(self) -> I16Vector4: ...
    @property
    def xllo(self) -> I16Vector4: ...
    @property
    def xlll(self) -> I16Vector4: ...
    @property
    def yxxx(self) -> I16Vector4: ...
    @property
    def yxxy(self) -> I16Vector4: ...
    @property
    def yxxz(self) -> I16Vector4: ...
    @property
    def yxxw(self) -> I16Vector4: ...
    @property
    def yxxo(self) -> I16Vector4: ...
    @property
    def yxxl(self) -> I16Vector4: ...
    @property
    def yxyx(self) -> I16Vector4: ...
    @property
    def yxyy(self) -> I16Vector4: ...
    @property
    def yxyz(self) -> I16Vector4: ...
    @property
    def yxyw(self) -> I16Vector4: ...
    @property
    def yxyo(self) -> I16Vector4: ...
    @property
    def yxyl(self) -> I16Vector4: ...
    @property
    def yxzx(self) -> I16Vector4: ...
    @property
    def yxzy(self) -> I16Vector4: ...
    @property
    def yxzz(self) -> I16Vector4: ...
    @property
    def yxzw(self) -> I16Vector4: ...
    @property
    def yxzo(self) -> I16Vector4: ...
    @property
    def yxzl(self) -> I16Vector4: ...
    @property
    def yxwx(self) -> I16Vector4: ...
    @property
    def yxwy(self) -> I16Vector4: ...
    @property
    def yxwz(self) -> I16Vector4: ...
    @property
    def yxww(self) -> I16Vector4: ...
    @property
    def yxwo(self) -> I16Vector4: ...
    @property
    def yxwl(self) -> I16Vector4: ...
    @property
    def yxox(self) -> I16Vector4: ...
    @property
    def yxoy(self) -> I16Vector4: ...
    @property
    def yxoz(self) -> I16Vector4: ...
    @property
    def yxow(self) -> I16Vector4: ...
    @property
    def yxoo(self) -> I16Vector4: ...
    @property
    def yxol(self) -> I16Vector4: ...
    @property
    def yxlx(self) -> I16Vector4: ...
    @property
    def yxly(self) -> I16Vector4: ...
    @property
    def yxlz(self) -> I16Vector4: ...
    @property
    def yxlw(self) -> I16Vector4: ...
    @property
    def yxlo(self) -> I16Vector4: ...
    @property
    def yxll(self) -> I16Vector4: ...
    @property
    def yyxx(self) -> I16Vector4: ...
    @property
    def yyxy(self) -> I16Vector4: ...
    @property
    def yyxz(self) -> I16Vector4: ...
    @property
    def yyxw(self) -> I16Vector4: ...
    @property
    def yyxo(self) -> I16Vector4: ...
    @property
    def yyxl(self) -> I16Vector4: ...
    @property
    def yyyx(self) -> I16Vector4: ...
    @property
    def yyyy(self) -> I16Vector4: ...
    @property
    def yyyz(self) -> I16Vector4: ...
    @property
    def yyyw(self) -> I16Vector4: ...
    @property
    def yyyo(self) -> I16Vector4: ...
    @property
    def yyyl(self) -> I16Vector4: ...
    @property
    def yyzx(self) -> I16Vector4: ...
    @property
    def yyzy(self) -> I16Vector4: ...
    @property
    def yyzz(self) -> I16Vector4: ...
    @property
    def yyzw(self) -> I16Vector4: ...
    @property
    def yyzo(self) -> I16Vector4: ...
    @property
    def yyzl(self) -> I16Vector4: ...
    @property
    def yywx(self) -> I16Vector4: ...
    @property
    def yywy(self) -> I16Vector4: ...
    @property
    def yywz(self) -> I16Vector4: ...
    @property
    def yyww(self) -> I16Vector4: ...
    @property
    def yywo(self) -> I16Vector4: ...
    @property
    def yywl(self) -> I16Vector4: ...
    @property
    def yyox(self) -> I16Vector4: ...
    @property
    def yyoy(self) -> I16Vector4: ...
    @property
    def yyoz(self) -> I16Vector4: ...
    @property
    def yyow(self) -> I16Vector4: ...
    @property
    def yyoo(self) -> I16Vector4: ...
    @property
    def yyol(self) -> I16Vector4: ...
    @property
    def yylx(self) -> I16Vector4: ...
    @property
    def yyly(self) -> I16Vector4: ...
    @property
    def yylz(self) -> I16Vector4: ...
    @property
    def yylw(self) -> I16Vector4: ...
    @property
    def yylo(self) -> I16Vector4: ...
    @property
    def yyll(self) -> I16Vector4: ...
    @property
    def yzxx(self) -> I16Vector4: ...
    @property
    def yzxy(self) -> I16Vector4: ...
    @property
    def yzxz(self) -> I16Vector4: ...
    @property
    def yzxw(self) -> I16Vector4: ...
    @property
    def yzxo(self) -> I16Vector4: ...
    @property
    def yzxl(self) -> I16Vector4: ...
    @property
    def yzyx(self) -> I16Vector4: ...
    @property
    def yzyy(self) -> I16Vector4: ...
    @property
    def yzyz(self) -> I16Vector4: ...
    @property
    def yzyw(self) -> I16Vector4: ...
    @property
    def yzyo(self) -> I16Vector4: ...
    @property
    def yzyl(self) -> I16Vector4: ...
    @property
    def yzzx(self) -> I16Vector4: ...
    @property
    def yzzy(self) -> I16Vector4: ...
    @property
    def yzzz(self) -> I16Vector4: ...
    @property
    def yzzw(self) -> I16Vector4: ...
    @property
    def yzzo(self) -> I16Vector4: ...
    @property
    def yzzl(self) -> I16Vector4: ...
    @property
    def yzwx(self) -> I16Vector4: ...
    @property
    def yzwy(self) -> I16Vector4: ...
    @property
    def yzwz(self) -> I16Vector4: ...
    @property
    def yzww(self) -> I16Vector4: ...
    @property
    def yzwo(self) -> I16Vector4: ...
    @property
    def yzwl(self) -> I16Vector4: ...
    @property
    def yzox(self) -> I16Vector4: ...
    @property
    def yzoy(self) -> I16Vector4: ...
    @property
    def yzoz(self) -> I16Vector4: ...
    @property
    def yzow(self) -> I16Vector4: ...
    @property
    def yzoo(self) -> I16Vector4: ...
    @property
    def yzol(self) -> I16Vector4: ...
    @property
    def yzlx(self) -> I16Vector4: ...
    @property
    def yzly(self) -> I16Vector4: ...
    @property
    def yzlz(self) -> I16Vector4: ...
    @property
    def yzlw(self) -> I16Vector4: ...
    @property
    def yzlo(self) -> I16Vector4: ...
    @property
    def yzll(self) -> I16Vector4: ...
    @property
    def ywxx(self) -> I16Vector4: ...
    @property
    def ywxy(self) -> I16Vector4: ...
    @property
    def ywxz(self) -> I16Vector4: ...
    @property
    def ywxw(self) -> I16Vector4: ...
    @property
    def ywxo(self) -> I16Vector4: ...
    @property
    def ywxl(self) -> I16Vector4: ...
    @property
    def ywyx(self) -> I16Vector4: ...
    @property
    def ywyy(self) -> I16Vector4: ...
    @property
    def ywyz(self) -> I16Vector4: ...
    @property
    def ywyw(self) -> I16Vector4: ...
    @property
    def ywyo(self) -> I16Vector4: ...
    @property
    def ywyl(self) -> I16Vector4: ...
    @property
    def ywzx(self) -> I16Vector4: ...
    @property
    def ywzy(self) -> I16Vector4: ...
    @property
    def ywzz(self) -> I16Vector4: ...
    @property
    def ywzw(self) -> I16Vector4: ...
    @property
    def ywzo(self) -> I16Vector4: ...
    @property
    def ywzl(self) -> I16Vector4: ...
    @property
    def ywwx(self) -> I16Vector4: ...
    @property
    def ywwy(self) -> I16Vector4: ...
    @property
    def ywwz(self) -> I16Vector4: ...
    @property
    def ywww(self) -> I16Vector4: ...
    @property
    def ywwo(self) -> I16Vector4: ...
    @property
    def ywwl(self) -> I16Vector4: ...
    @property
    def ywox(self) -> I16Vector4: ...
    @property
    def ywoy(self) -> I16Vector4: ...
    @property
    def ywoz(self) -> I16Vector4: ...
    @property
    def ywow(self) -> I16Vector4: ...
    @property
    def ywoo(self) -> I16Vector4: ...
    @property
    def ywol(self) -> I16Vector4: ...
    @property
    def ywlx(self) -> I16Vector4: ...
    @property
    def ywly(self) -> I16Vector4: ...
    @property
    def ywlz(self) -> I16Vector4: ...
    @property
    def ywlw(self) -> I16Vector4: ...
    @property
    def ywlo(self) -> I16Vector4: ...
    @property
    def ywll(self) -> I16Vector4: ...
    @property
    def yoxx(self) -> I16Vector4: ...
    @property
    def yoxy(self) -> I16Vector4: ...
    @property
    def yoxz(self) -> I16Vector4: ...
    @property
    def yoxw(self) -> I16Vector4: ...
    @property
    def yoxo(self) -> I16Vector4: ...
    @property
    def yoxl(self) -> I16Vector4: ...
    @property
    def yoyx(self) -> I16Vector4: ...
    @property
    def yoyy(self) -> I16Vector4: ...
    @property
    def yoyz(self) -> I16Vector4: ...
    @property
    def yoyw(self) -> I16Vector4: ...
    @property
    def yoyo(self) -> I16Vector4: ...
    @property
    def yoyl(self) -> I16Vector4: ...
    @property
    def yozx(self) -> I16Vector4: ...
    @property
    def yozy(self) -> I16Vector4: ...
    @property
    def yozz(self) -> I16Vector4: ...
    @property
    def yozw(self) -> I16Vector4: ...
    @property
    def yozo(self) -> I16Vector4: ...
    @property
    def yozl(self) -> I16Vector4: ...
    @property
    def yowx(self) -> I16Vector4: ...
    @property
    def yowy(self) -> I16Vector4: ...
    @property
    def yowz(self) -> I16Vector4: ...
    @property
    def yoww(self) -> I16Vector4: ...
    @property
    def yowo(self) -> I16Vector4: ...
    @property
    def yowl(self) -> I16Vector4: ...
    @property
    def yoox(self) -> I16Vector4: ...
    @property
    def yooy(self) -> I16Vector4: ...
    @property
    def yooz(self) -> I16Vector4: ...
    @property
    def yoow(self) -> I16Vector4: ...
    @property
    def yooo(self) -> I16Vector4: ...
    @property
    def yool(self) -> I16Vector4: ...
    @property
    def yolx(self) -> I16Vector4: ...
    @property
    def yoly(self) -> I16Vector4: ...
    @property
    def yolz(self) -> I16Vector4: ...
    @property
    def yolw(self) -> I16Vector4: ...
    @property
    def yolo(self) -> I16Vector4: ...
    @property
    def yoll(self) -> I16Vector4: ...
    @property
    def ylxx(self) -> I16Vector4: ...
    @property
    def ylxy(self) -> I16Vector4: ...
    @property
    def ylxz(self) -> I16Vector4: ...
    @property
    def ylxw(self) -> I16Vector4: ...
    @property
    def ylxo(self) -> I16Vector4: ...
    @property
    def ylxl(self) -> I16Vector4: ...
    @property
    def ylyx(self) -> I16Vector4: ...
    @property
    def ylyy(self) -> I16Vector4: ...
    @property
    def ylyz(self) -> I16Vector4: ...
    @property
    def ylyw(self) -> I16Vector4: ...
    @property
    def ylyo(self) -> I16Vector4: ...
    @property
    def ylyl(self) -> I16Vector4: ...
    @property
    def ylzx(self) -> I16Vector4: ...
    @property
    def ylzy(self) -> I16Vector4: ...
    @property
    def ylzz(self) -> I16Vector4: ...
    @property
    def ylzw(self) -> I16Vector4: ...
    @property
    def ylzo(self) -> I16Vector4: ...
    @property
    def ylzl(self) -> I16Vector4: ...
    @property
    def ylwx(self) -> I16Vector4: ...
    @property
    def ylwy(self) -> I16Vector4: ...
    @property
    def ylwz(self) -> I16Vector4: ...
    @property
    def ylww(self) -> I16Vector4: ...
    @property
    def ylwo(self) -> I16Vector4: ...
    @property
    def ylwl(self) -> I16Vector4: ...
    @property
    def ylox(self) -> I16Vector4: ...
    @property
    def yloy(self) -> I16Vector4: ...
    @property
    def yloz(self) -> I16Vector4: ...
    @property
    def ylow(self) -> I16Vector4: ...
    @property
    def yloo(self) -> I16Vector4: ...
    @property
    def ylol(self) -> I16Vector4: ...
    @property
    def yllx(self) -> I16Vector4: ...
    @property
    def ylly(self) -> I16Vector4: ...
    @property
    def yllz(self) -> I16Vector4: ...
    @property
    def yllw(self) -> I16Vector4: ...
    @property
    def yllo(self) -> I16Vector4: ...
    @property
    def ylll(self) -> I16Vector4: ...
    @property
    def zxxx(self) -> I16Vector4: ...
    @property
    def zxxy(self) -> I16Vector4: ...
    @property
    def zxxz(self) -> I16Vector4: ...
    @property
    def zxxw(self) -> I16Vector4: ...
    @property
    def zxxo(self) -> I16Vector4: ...
    @property
    def zxxl(self) -> I16Vector4: ...
    @property
    def zxyx(self) -> I16Vector4: ...
    @property
    def zxyy(self) -> I16Vector4: ...
    @property
    def zxyz(self) -> I16Vector4: ...
    @property
    def zxyw(self) -> I16Vector4: ...
    @property
    def zxyo(self) -> I16Vector4: ...
    @property
    def zxyl(self) -> I16Vector4: ...
    @property
    def zxzx(self) -> I16Vector4: ...
    @property
    def zxzy(self) -> I16Vector4: ...
    @property
    def zxzz(self) -> I16Vector4: ...
    @property
    def zxzw(self) -> I16Vector4: ...
    @property
    def zxzo(self) -> I16Vector4: ...
    @property
    def zxzl(self) -> I16Vector4: ...
    @property
    def zxwx(self) -> I16Vector4: ...
    @property
    def zxwy(self) -> I16Vector4: ...
    @property
    def zxwz(self) -> I16Vector4: ...
    @property
    def zxww(self) -> I16Vector4: ...
    @property
    def zxwo(self) -> I16Vector4: ...
    @property
    def zxwl(self) -> I16Vector4: ...
    @property
    def zxox(self) -> I16Vector4: ...
    @property
    def zxoy(self) -> I16Vector4: ...
    @property
    def zxoz(self) -> I16Vector4: ...
    @property
    def zxow(self) -> I16Vector4: ...
    @property
    def zxoo(self) -> I16Vector4: ...
    @property
    def zxol(self) -> I16Vector4: ...
    @property
    def zxlx(self) -> I16Vector4: ...
    @property
    def zxly(self) -> I16Vector4: ...
    @property
    def zxlz(self) -> I16Vector4: ...
    @property
    def zxlw(self) -> I16Vector4: ...
    @property
    def zxlo(self) -> I16Vector4: ...
    @property
    def zxll(self) -> I16Vector4: ...
    @property
    def zyxx(self) -> I16Vector4: ...
    @property
    def zyxy(self) -> I16Vector4: ...
    @property
    def zyxz(self) -> I16Vector4: ...
    @property
    def zyxw(self) -> I16Vector4: ...
    @property
    def zyxo(self) -> I16Vector4: ...
    @property
    def zyxl(self) -> I16Vector4: ...
    @property
    def zyyx(self) -> I16Vector4: ...
    @property
    def zyyy(self) -> I16Vector4: ...
    @property
    def zyyz(self) -> I16Vector4: ...
    @property
    def zyyw(self) -> I16Vector4: ...
    @property
    def zyyo(self) -> I16Vector4: ...
    @property
    def zyyl(self) -> I16Vector4: ...
    @property
    def zyzx(self) -> I16Vector4: ...
    @property
    def zyzy(self) -> I16Vector4: ...
    @property
    def zyzz(self) -> I16Vector4: ...
    @property
    def zyzw(self) -> I16Vector4: ...
    @property
    def zyzo(self) -> I16Vector4: ...
    @property
    def zyzl(self) -> I16Vector4: ...
    @property
    def zywx(self) -> I16Vector4: ...
    @property
    def zywy(self) -> I16Vector4: ...
    @property
    def zywz(self) -> I16Vector4: ...
    @property
    def zyww(self) -> I16Vector4: ...
    @property
    def zywo(self) -> I16Vector4: ...
    @property
    def zywl(self) -> I16Vector4: ...
    @property
    def zyox(self) -> I16Vector4: ...
    @property
    def zyoy(self) -> I16Vector4: ...
    @property
    def zyoz(self) -> I16Vector4: ...
    @property
    def zyow(self) -> I16Vector4: ...
    @property
    def zyoo(self) -> I16Vector4: ...
    @property
    def zyol(self) -> I16Vector4: ...
    @property
    def zylx(self) -> I16Vector4: ...
    @property
    def zyly(self) -> I16Vector4: ...
    @property
    def zylz(self) -> I16Vector4: ...
    @property
    def zylw(self) -> I16Vector4: ...
    @property
    def zylo(self) -> I16Vector4: ...
    @property
    def zyll(self) -> I16Vector4: ...
    @property
    def zzxx(self) -> I16Vector4: ...
    @property
    def zzxy(self) -> I16Vector4: ...
    @property
    def zzxz(self) -> I16Vector4: ...
    @property
    def zzxw(self) -> I16Vector4: ...
    @property
    def zzxo(self) -> I16Vector4: ...
    @property
    def zzxl(self) -> I16Vector4: ...
    @property
    def zzyx(self) -> I16Vector4: ...
    @property
    def zzyy(self) -> I16Vector4: ...
    @property
    def zzyz(self) -> I16Vector4: ...
    @property
    def zzyw(self) -> I16Vector4: ...
    @property
    def zzyo(self) -> I16Vector4: ...
    @property
    def zzyl(self) -> I16Vector4: ...
    @property
    def zzzx(self) -> I16Vector4: ...
    @property
    def zzzy(self) -> I16Vector4: ...
    @property
    def zzzz(self) -> I16Vector4: ...
    @property
    def zzzw(self) -> I16Vector4: ...
    @property
    def zzzo(self) -> I16Vector4: ...
    @property
    def zzzl(self) -> I16Vector4: ...
    @property
    def zzwx(self) -> I16Vector4: ...
    @property
    def zzwy(self) -> I16Vector4: ...
    @property
    def zzwz(self) -> I16Vector4: ...
    @property
    def zzww(self) -> I16Vector4: ...
    @property
    def zzwo(self) -> I16Vector4: ...
    @property
    def zzwl(self) -> I16Vector4: ...
    @property
    def zzox(self) -> I16Vector4: ...
    @property
    def zzoy(self) -> I16Vector4: ...
    @property
    def zzoz(self) -> I16Vector4: ...
    @property
    def zzow(self) -> I16Vector4: ...
    @property
    def zzoo(self) -> I16Vector4: ...
    @property
    def zzol(self) -> I16Vector4: ...
    @property
    def zzlx(self) -> I16Vector4: ...
    @property
    def zzly(self) -> I16Vector4: ...
    @property
    def zzlz(self) -> I16Vector4: ...
    @property
    def zzlw(self) -> I16Vector4: ...
    @property
    def zzlo(self) -> I16Vector4: ...
    @property
    def zzll(self) -> I16Vector4: ...
    @property
    def zwxx(self) -> I16Vector4: ...
    @property
    def zwxy(self) -> I16Vector4: ...
    @property
    def zwxz(self) -> I16Vector4: ...
    @property
    def zwxw(self) -> I16Vector4: ...
    @property
    def zwxo(self) -> I16Vector4: ...
    @property
    def zwxl(self) -> I16Vector4: ...
    @property
    def zwyx(self) -> I16Vector4: ...
    @property
    def zwyy(self) -> I16Vector4: ...
    @property
    def zwyz(self) -> I16Vector4: ...
    @property
    def zwyw(self) -> I16Vector4: ...
    @property
    def zwyo(self) -> I16Vector4: ...
    @property
    def zwyl(self) -> I16Vector4: ...
    @property
    def zwzx(self) -> I16Vector4: ...
    @property
    def zwzy(self) -> I16Vector4: ...
    @property
    def zwzz(self) -> I16Vector4: ...
    @property
    def zwzw(self) -> I16Vector4: ...
    @property
    def zwzo(self) -> I16Vector4: ...
    @property
    def zwzl(self) -> I16Vector4: ...
    @property
    def zwwx(self) -> I16Vector4: ...
    @property
    def zwwy(self) -> I16Vector4: ...
    @property
    def zwwz(self) -> I16Vector4: ...
    @property
    def zwww(self) -> I16Vector4: ...
    @property
    def zwwo(self) -> I16Vector4: ...
    @property
    def zwwl(self) -> I16Vector4: ...
    @property
    def zwox(self) -> I16Vector4: ...
    @property
    def zwoy(self) -> I16Vector4: ...
    @property
    def zwoz(self) -> I16Vector4: ...
    @property
    def zwow(self) -> I16Vector4: ...
    @property
    def zwoo(self) -> I16Vector4: ...
    @property
    def zwol(self) -> I16Vector4: ...
    @property
    def zwlx(self) -> I16Vector4: ...
    @property
    def zwly(self) -> I16Vector4: ...
    @property
    def zwlz(self) -> I16Vector4: ...
    @property
    def zwlw(self) -> I16Vector4: ...
    @property
    def zwlo(self) -> I16Vector4: ...
    @property
    def zwll(self) -> I16Vector4: ...
    @property
    def zoxx(self) -> I16Vector4: ...
    @property
    def zoxy(self) -> I16Vector4: ...
    @property
    def zoxz(self) -> I16Vector4: ...
    @property
    def zoxw(self) -> I16Vector4: ...
    @property
    def zoxo(self) -> I16Vector4: ...
    @property
    def zoxl(self) -> I16Vector4: ...
    @property
    def zoyx(self) -> I16Vector4: ...
    @property
    def zoyy(self) -> I16Vector4: ...
    @property
    def zoyz(self) -> I16Vector4: ...
    @property
    def zoyw(self) -> I16Vector4: ...
    @property
    def zoyo(self) -> I16Vector4: ...
    @property
    def zoyl(self) -> I16Vector4: ...
    @property
    def zozx(self) -> I16Vector4: ...
    @property
    def zozy(self) -> I16Vector4: ...
    @property
    def zozz(self) -> I16Vector4: ...
    @property
    def zozw(self) -> I16Vector4: ...
    @property
    def zozo(self) -> I16Vector4: ...
    @property
    def zozl(self) -> I16Vector4: ...
    @property
    def zowx(self) -> I16Vector4: ...
    @property
    def zowy(self) -> I16Vector4: ...
    @property
    def zowz(self) -> I16Vector4: ...
    @property
    def zoww(self) -> I16Vector4: ...
    @property
    def zowo(self) -> I16Vector4: ...
    @property
    def zowl(self) -> I16Vector4: ...
    @property
    def zoox(self) -> I16Vector4: ...
    @property
    def zooy(self) -> I16Vector4: ...
    @property
    def zooz(self) -> I16Vector4: ...
    @property
    def zoow(self) -> I16Vector4: ...
    @property
    def zooo(self) -> I16Vector4: ...
    @property
    def zool(self) -> I16Vector4: ...
    @property
    def zolx(self) -> I16Vector4: ...
    @property
    def zoly(self) -> I16Vector4: ...
    @property
    def zolz(self) -> I16Vector4: ...
    @property
    def zolw(self) -> I16Vector4: ...
    @property
    def zolo(self) -> I16Vector4: ...
    @property
    def zoll(self) -> I16Vector4: ...
    @property
    def zlxx(self) -> I16Vector4: ...
    @property
    def zlxy(self) -> I16Vector4: ...
    @property
    def zlxz(self) -> I16Vector4: ...
    @property
    def zlxw(self) -> I16Vector4: ...
    @property
    def zlxo(self) -> I16Vector4: ...
    @property
    def zlxl(self) -> I16Vector4: ...
    @property
    def zlyx(self) -> I16Vector4: ...
    @property
    def zlyy(self) -> I16Vector4: ...
    @property
    def zlyz(self) -> I16Vector4: ...
    @property
    def zlyw(self) -> I16Vector4: ...
    @property
    def zlyo(self) -> I16Vector4: ...
    @property
    def zlyl(self) -> I16Vector4: ...
    @property
    def zlzx(self) -> I16Vector4: ...
    @property
    def zlzy(self) -> I16Vector4: ...
    @property
    def zlzz(self) -> I16Vector4: ...
    @property
    def zlzw(self) -> I16Vector4: ...
    @property
    def zlzo(self) -> I16Vector4: ...
    @property
    def zlzl(self) -> I16Vector4: ...
    @property
    def zlwx(self) -> I16Vector4: ...
    @property
    def zlwy(self) -> I16Vector4: ...
    @property
    def zlwz(self) -> I16Vector4: ...
    @property
    def zlww(self) -> I16Vector4: ...
    @property
    def zlwo(self) -> I16Vector4: ...
    @property
    def zlwl(self) -> I16Vector4: ...
    @property
    def zlox(self) -> I16Vector4: ...
    @property
    def zloy(self) -> I16Vector4: ...
    @property
    def zloz(self) -> I16Vector4: ...
    @property
    def zlow(self) -> I16Vector4: ...
    @property
    def zloo(self) -> I16Vector4: ...
    @property
    def zlol(self) -> I16Vector4: ...
    @property
    def zllx(self) -> I16Vector4: ...
    @property
    def zlly(self) -> I16Vector4: ...
    @property
    def zllz(self) -> I16Vector4: ...
    @property
    def zllw(self) -> I16Vector4: ...
    @property
    def zllo(self) -> I16Vector4: ...
    @property
    def zlll(self) -> I16Vector4: ...
    @property
    def wxxx(self) -> I16Vector4: ...
    @property
    def wxxy(self) -> I16Vector4: ...
    @property
    def wxxz(self) -> I16Vector4: ...
    @property
    def wxxw(self) -> I16Vector4: ...
    @property
    def wxxo(self) -> I16Vector4: ...
    @property
    def wxxl(self) -> I16Vector4: ...
    @property
    def wxyx(self) -> I16Vector4: ...
    @property
    def wxyy(self) -> I16Vector4: ...
    @property
    def wxyz(self) -> I16Vector4: ...
    @property
    def wxyw(self) -> I16Vector4: ...
    @property
    def wxyo(self) -> I16Vector4: ...
    @property
    def wxyl(self) -> I16Vector4: ...
    @property
    def wxzx(self) -> I16Vector4: ...
    @property
    def wxzy(self) -> I16Vector4: ...
    @property
    def wxzz(self) -> I16Vector4: ...
    @property
    def wxzw(self) -> I16Vector4: ...
    @property
    def wxzo(self) -> I16Vector4: ...
    @property
    def wxzl(self) -> I16Vector4: ...
    @property
    def wxwx(self) -> I16Vector4: ...
    @property
    def wxwy(self) -> I16Vector4: ...
    @property
    def wxwz(self) -> I16Vector4: ...
    @property
    def wxww(self) -> I16Vector4: ...
    @property
    def wxwo(self) -> I16Vector4: ...
    @property
    def wxwl(self) -> I16Vector4: ...
    @property
    def wxox(self) -> I16Vector4: ...
    @property
    def wxoy(self) -> I16Vector4: ...
    @property
    def wxoz(self) -> I16Vector4: ...
    @property
    def wxow(self) -> I16Vector4: ...
    @property
    def wxoo(self) -> I16Vector4: ...
    @property
    def wxol(self) -> I16Vector4: ...
    @property
    def wxlx(self) -> I16Vector4: ...
    @property
    def wxly(self) -> I16Vector4: ...
    @property
    def wxlz(self) -> I16Vector4: ...
    @property
    def wxlw(self) -> I16Vector4: ...
    @property
    def wxlo(self) -> I16Vector4: ...
    @property
    def wxll(self) -> I16Vector4: ...
    @property
    def wyxx(self) -> I16Vector4: ...
    @property
    def wyxy(self) -> I16Vector4: ...
    @property
    def wyxz(self) -> I16Vector4: ...
    @property
    def wyxw(self) -> I16Vector4: ...
    @property
    def wyxo(self) -> I16Vector4: ...
    @property
    def wyxl(self) -> I16Vector4: ...
    @property
    def wyyx(self) -> I16Vector4: ...
    @property
    def wyyy(self) -> I16Vector4: ...
    @property
    def wyyz(self) -> I16Vector4: ...
    @property
    def wyyw(self) -> I16Vector4: ...
    @property
    def wyyo(self) -> I16Vector4: ...
    @property
    def wyyl(self) -> I16Vector4: ...
    @property
    def wyzx(self) -> I16Vector4: ...
    @property
    def wyzy(self) -> I16Vector4: ...
    @property
    def wyzz(self) -> I16Vector4: ...
    @property
    def wyzw(self) -> I16Vector4: ...
    @property
    def wyzo(self) -> I16Vector4: ...
    @property
    def wyzl(self) -> I16Vector4: ...
    @property
    def wywx(self) -> I16Vector4: ...
    @property
    def wywy(self) -> I16Vector4: ...
    @property
    def wywz(self) -> I16Vector4: ...
    @property
    def wyww(self) -> I16Vector4: ...
    @property
    def wywo(self) -> I16Vector4: ...
    @property
    def wywl(self) -> I16Vector4: ...
    @property
    def wyox(self) -> I16Vector4: ...
    @property
    def wyoy(self) -> I16Vector4: ...
    @property
    def wyoz(self) -> I16Vector4: ...
    @property
    def wyow(self) -> I16Vector4: ...
    @property
    def wyoo(self) -> I16Vector4: ...
    @property
    def wyol(self) -> I16Vector4: ...
    @property
    def wylx(self) -> I16Vector4: ...
    @property
    def wyly(self) -> I16Vector4: ...
    @property
    def wylz(self) -> I16Vector4: ...
    @property
    def wylw(self) -> I16Vector4: ...
    @property
    def wylo(self) -> I16Vector4: ...
    @property
    def wyll(self) -> I16Vector4: ...
    @property
    def wzxx(self) -> I16Vector4: ...
    @property
    def wzxy(self) -> I16Vector4: ...
    @property
    def wzxz(self) -> I16Vector4: ...
    @property
    def wzxw(self) -> I16Vector4: ...
    @property
    def wzxo(self) -> I16Vector4: ...
    @property
    def wzxl(self) -> I16Vector4: ...
    @property
    def wzyx(self) -> I16Vector4: ...
    @property
    def wzyy(self) -> I16Vector4: ...
    @property
    def wzyz(self) -> I16Vector4: ...
    @property
    def wzyw(self) -> I16Vector4: ...
    @property
    def wzyo(self) -> I16Vector4: ...
    @property
    def wzyl(self) -> I16Vector4: ...
    @property
    def wzzx(self) -> I16Vector4: ...
    @property
    def wzzy(self) -> I16Vector4: ...
    @property
    def wzzz(self) -> I16Vector4: ...
    @property
    def wzzw(self) -> I16Vector4: ...
    @property
    def wzzo(self) -> I16Vector4: ...
    @property
    def wzzl(self) -> I16Vector4: ...
    @property
    def wzwx(self) -> I16Vector4: ...
    @property
    def wzwy(self) -> I16Vector4: ...
    @property
    def wzwz(self) -> I16Vector4: ...
    @property
    def wzww(self) -> I16Vector4: ...
    @property
    def wzwo(self) -> I16Vector4: ...
    @property
    def wzwl(self) -> I16Vector4: ...
    @property
    def wzox(self) -> I16Vector4: ...
    @property
    def wzoy(self) -> I16Vector4: ...
    @property
    def wzoz(self) -> I16Vector4: ...
    @property
    def wzow(self) -> I16Vector4: ...
    @property
    def wzoo(self) -> I16Vector4: ...
    @property
    def wzol(self) -> I16Vector4: ...
    @property
    def wzlx(self) -> I16Vector4: ...
    @property
    def wzly(self) -> I16Vector4: ...
    @property
    def wzlz(self) -> I16Vector4: ...
    @property
    def wzlw(self) -> I16Vector4: ...
    @property
    def wzlo(self) -> I16Vector4: ...
    @property
    def wzll(self) -> I16Vector4: ...
    @property
    def wwxx(self) -> I16Vector4: ...
    @property
    def wwxy(self) -> I16Vector4: ...
    @property
    def wwxz(self) -> I16Vector4: ...
    @property
    def wwxw(self) -> I16Vector4: ...
    @property
    def wwxo(self) -> I16Vector4: ...
    @property
    def wwxl(self) -> I16Vector4: ...
    @property
    def wwyx(self) -> I16Vector4: ...
    @property
    def wwyy(self) -> I16Vector4: ...
    @property
    def wwyz(self) -> I16Vector4: ...
    @property
    def wwyw(self) -> I16Vector4: ...
    @property
    def wwyo(self) -> I16Vector4: ...
    @property
    def wwyl(self) -> I16Vector4: ...
    @property
    def wwzx(self) -> I16Vector4: ...
    @property
    def wwzy(self) -> I16Vector4: ...
    @property
    def wwzz(self) -> I16Vector4: ...
    @property
    def wwzw(self) -> I16Vector4: ...
    @property
    def wwzo(self) -> I16Vector4: ...
    @property
    def wwzl(self) -> I16Vector4: ...
    @property
    def wwwx(self) -> I16Vector4: ...
    @property
    def wwwy(self) -> I16Vector4: ...
    @property
    def wwwz(self) -> I16Vector4: ...
    @property
    def wwww(self) -> I16Vector4: ...
    @property
    def wwwo(self) -> I16Vector4: ...
    @property
    def wwwl(self) -> I16Vector4: ...
    @property
    def wwox(self) -> I16Vector4: ...
    @property
    def wwoy(self) -> I16Vector4: ...
    @property
    def wwoz(self) -> I16Vector4: ...
    @property
    def wwow(self) -> I16Vector4: ...
    @property
    def wwoo(self) -> I16Vector4: ...
    @property
    def wwol(self) -> I16Vector4: ...
    @property
    def wwlx(self) -> I16Vector4: ...
    @property
    def wwly(self) -> I16Vector4: ...
    @property
    def wwlz(self) -> I16Vector4: ...
    @property
    def wwlw(self) -> I16Vector4: ...
    @property
    def wwlo(self) -> I16Vector4: ...
    @property
    def wwll(self) -> I16Vector4: ...
    @property
    def woxx(self) -> I16Vector4: ...
    @property
    def woxy(self) -> I16Vector4: ...
    @property
    def woxz(self) -> I16Vector4: ...
    @property
    def woxw(self) -> I16Vector4: ...
    @property
    def woxo(self) -> I16Vector4: ...
    @property
    def woxl(self) -> I16Vector4: ...
    @property
    def woyx(self) -> I16Vector4: ...
    @property
    def woyy(self) -> I16Vector4: ...
    @property
    def woyz(self) -> I16Vector4: ...
    @property
    def woyw(self) -> I16Vector4: ...
    @property
    def woyo(self) -> I16Vector4: ...
    @property
    def woyl(self) -> I16Vector4: ...
    @property
    def wozx(self) -> I16Vector4: ...
    @property
    def wozy(self) -> I16Vector4: ...
    @property
    def wozz(self) -> I16Vector4: ...
    @property
    def wozw(self) -> I16Vector4: ...
    @property
    def wozo(self) -> I16Vector4: ...
    @property
    def wozl(self) -> I16Vector4: ...
    @property
    def wowx(self) -> I16Vector4: ...
    @property
    def wowy(self) -> I16Vector4: ...
    @property
    def wowz(self) -> I16Vector4: ...
    @property
    def woww(self) -> I16Vector4: ...
    @property
    def wowo(self) -> I16Vector4: ...
    @property
    def wowl(self) -> I16Vector4: ...
    @property
    def woox(self) -> I16Vector4: ...
    @property
    def wooy(self) -> I16Vector4: ...
    @property
    def wooz(self) -> I16Vector4: ...
    @property
    def woow(self) -> I16Vector4: ...
    @property
    def wooo(self) -> I16Vector4: ...
    @property
    def wool(self) -> I16Vector4: ...
    @property
    def wolx(self) -> I16Vector4: ...
    @property
    def woly(self) -> I16Vector4: ...
    @property
    def wolz(self) -> I16Vector4: ...
    @property
    def wolw(self) -> I16Vector4: ...
    @property
    def wolo(self) -> I16Vector4: ...
    @property
    def woll(self) -> I16Vector4: ...
    @property
    def wlxx(self) -> I16Vector4: ...
    @property
    def wlxy(self) -> I16Vector4: ...
    @property
    def wlxz(self) -> I16Vector4: ...
    @property
    def wlxw(self) -> I16Vector4: ...
    @property
    def wlxo(self) -> I16Vector4: ...
    @property
    def wlxl(self) -> I16Vector4: ...
    @property
    def wlyx(self) -> I16Vector4: ...
    @property
    def wlyy(self) -> I16Vector4: ...
    @property
    def wlyz(self) -> I16Vector4: ...
    @property
    def wlyw(self) -> I16Vector4: ...
    @property
    def wlyo(self) -> I16Vector4: ...
    @property
    def wlyl(self) -> I16Vector4: ...
    @property
    def wlzx(self) -> I16Vector4: ...
    @property
    def wlzy(self) -> I16Vector4: ...
    @property
    def wlzz(self) -> I16Vector4: ...
    @property
    def wlzw(self) -> I16Vector4: ...
    @property
    def wlzo(self) -> I16Vector4: ...
    @property
    def wlzl(self) -> I16Vector4: ...
    @property
    def wlwx(self) -> I16Vector4: ...
    @property
    def wlwy(self) -> I16Vector4: ...
    @property
    def wlwz(self) -> I16Vector4: ...
    @property
    def wlww(self) -> I16Vector4: ...
    @property
    def wlwo(self) -> I16Vector4: ...
    @property
    def wlwl(self) -> I16Vector4: ...
    @property
    def wlox(self) -> I16Vector4: ...
    @property
    def wloy(self) -> I16Vector4: ...
    @property
    def wloz(self) -> I16Vector4: ...
    @property
    def wlow(self) -> I16Vector4: ...
    @property
    def wloo(self) -> I16Vector4: ...
    @property
    def wlol(self) -> I16Vector4: ...
    @property
    def wllx(self) -> I16Vector4: ...
    @property
    def wlly(self) -> I16Vector4: ...
    @property
    def wllz(self) -> I16Vector4: ...
    @property
    def wllw(self) -> I16Vector4: ...
    @property
    def wllo(self) -> I16Vector4: ...
    @property
    def wlll(self) -> I16Vector4: ...
    @property
    def oxxx(self) -> I16Vector4: ...
    @property
    def oxxy(self) -> I16Vector4: ...
    @property
    def oxxz(self) -> I16Vector4: ...
    @property
    def oxxw(self) -> I16Vector4: ...
    @property
    def oxxo(self) -> I16Vector4: ...
    @property
    def oxxl(self) -> I16Vector4: ...
    @property
    def oxyx(self) -> I16Vector4: ...
    @property
    def oxyy(self) -> I16Vector4: ...
    @property
    def oxyz(self) -> I16Vector4: ...
    @property
    def oxyw(self) -> I16Vector4: ...
    @property
    def oxyo(self) -> I16Vector4: ...
    @property
    def oxyl(self) -> I16Vector4: ...
    @property
    def oxzx(self) -> I16Vector4: ...
    @property
    def oxzy(self) -> I16Vector4: ...
    @property
    def oxzz(self) -> I16Vector4: ...
    @property
    def oxzw(self) -> I16Vector4: ...
    @property
    def oxzo(self) -> I16Vector4: ...
    @property
    def oxzl(self) -> I16Vector4: ...
    @property
    def oxwx(self) -> I16Vector4: ...
    @property
    def oxwy(self) -> I16Vector4: ...
    @property
    def oxwz(self) -> I16Vector4: ...
    @property
    def oxww(self) -> I16Vector4: ...
    @property
    def oxwo(self) -> I16Vector4: ...
    @property
    def oxwl(self) -> I16Vector4: ...
    @property
    def oxox(self) -> I16Vector4: ...
    @property
    def oxoy(self) -> I16Vector4: ...
    @property
    def oxoz(self) -> I16Vector4: ...
    @property
    def oxow(self) -> I16Vector4: ...
    @property
    def oxoo(self) -> I16Vector4: ...
    @property
    def oxol(self) -> I16Vector4: ...
    @property
    def oxlx(self) -> I16Vector4: ...
    @property
    def oxly(self) -> I16Vector4: ...
    @property
    def oxlz(self) -> I16Vector4: ...
    @property
    def oxlw(self) -> I16Vector4: ...
    @property
    def oxlo(self) -> I16Vector4: ...
    @property
    def oxll(self) -> I16Vector4: ...
    @property
    def oyxx(self) -> I16Vector4: ...
    @property
    def oyxy(self) -> I16Vector4: ...
    @property
    def oyxz(self) -> I16Vector4: ...
    @property
    def oyxw(self) -> I16Vector4: ...
    @property
    def oyxo(self) -> I16Vector4: ...
    @property
    def oyxl(self) -> I16Vector4: ...
    @property
    def oyyx(self) -> I16Vector4: ...
    @property
    def oyyy(self) -> I16Vector4: ...
    @property
    def oyyz(self) -> I16Vector4: ...
    @property
    def oyyw(self) -> I16Vector4: ...
    @property
    def oyyo(self) -> I16Vector4: ...
    @property
    def oyyl(self) -> I16Vector4: ...
    @property
    def oyzx(self) -> I16Vector4: ...
    @property
    def oyzy(self) -> I16Vector4: ...
    @property
    def oyzz(self) -> I16Vector4: ...
    @property
    def oyzw(self) -> I16Vector4: ...
    @property
    def oyzo(self) -> I16Vector4: ...
    @property
    def oyzl(self) -> I16Vector4: ...
    @property
    def oywx(self) -> I16Vector4: ...
    @property
    def oywy(self) -> I16Vector4: ...
    @property
    def oywz(self) -> I16Vector4: ...
    @property
    def oyww(self) -> I16Vector4: ...
    @property
    def oywo(self) -> I16Vector4: ...
    @property
    def oywl(self) -> I16Vector4: ...
    @property
    def oyox(self) -> I16Vector4: ...
    @property
    def oyoy(self) -> I16Vector4: ...
    @property
    def oyoz(self) -> I16Vector4: ...
    @property
    def oyow(self) -> I16Vector4: ...
    @property
    def oyoo(self) -> I16Vector4: ...
    @property
    def oyol(self) -> I16Vector4: ...
    @property
    def oylx(self) -> I16Vector4: ...
    @property
    def oyly(self) -> I16Vector4: ...
    @property
    def oylz(self) -> I16Vector4: ...
    @property
    def oylw(self) -> I16Vector4: ...
    @property
    def oylo(self) -> I16Vector4: ...
    @property
    def oyll(self) -> I16Vector4: ...
    @property
    def ozxx(self) -> I16Vector4: ...
    @property
    def ozxy(self) -> I16Vector4: ...
    @property
    def ozxz(self) -> I16Vector4: ...
    @property
    def ozxw(self) -> I16Vector4: ...
    @property
    def ozxo(self) -> I16Vector4: ...
    @property
    def ozxl(self) -> I16Vector4: ...
    @property
    def ozyx(self) -> I16Vector4: ...
    @property
    def ozyy(self) -> I16Vector4: ...
    @property
    def ozyz(self) -> I16Vector4: ...
    @property
    def ozyw(self) -> I16Vector4: ...
    @property
    def ozyo(self) -> I16Vector4: ...
    @property
    def ozyl(self) -> I16Vector4: ...
    @property
    def ozzx(self) -> I16Vector4: ...
    @property
    def ozzy(self) -> I16Vector4: ...
    @property
    def ozzz(self) -> I16Vector4: ...
    @property
    def ozzw(self) -> I16Vector4: ...
    @property
    def ozzo(self) -> I16Vector4: ...
    @property
    def ozzl(self) -> I16Vector4: ...
    @property
    def ozwx(self) -> I16Vector4: ...
    @property
    def ozwy(self) -> I16Vector4: ...
    @property
    def ozwz(self) -> I16Vector4: ...
    @property
    def ozww(self) -> I16Vector4: ...
    @property
    def ozwo(self) -> I16Vector4: ...
    @property
    def ozwl(self) -> I16Vector4: ...
    @property
    def ozox(self) -> I16Vector4: ...
    @property
    def ozoy(self) -> I16Vector4: ...
    @property
    def ozoz(self) -> I16Vector4: ...
    @property
    def ozow(self) -> I16Vector4: ...
    @property
    def ozoo(self) -> I16Vector4: ...
    @property
    def ozol(self) -> I16Vector4: ...
    @property
    def ozlx(self) -> I16Vector4: ...
    @property
    def ozly(self) -> I16Vector4: ...
    @property
    def ozlz(self) -> I16Vector4: ...
    @property
    def ozlw(self) -> I16Vector4: ...
    @property
    def ozlo(self) -> I16Vector4: ...
    @property
    def ozll(self) -> I16Vector4: ...
    @property
    def owxx(self) -> I16Vector4: ...
    @property
    def owxy(self) -> I16Vector4: ...
    @property
    def owxz(self) -> I16Vector4: ...
    @property
    def owxw(self) -> I16Vector4: ...
    @property
    def owxo(self) -> I16Vector4: ...
    @property
    def owxl(self) -> I16Vector4: ...
    @property
    def owyx(self) -> I16Vector4: ...
    @property
    def owyy(self) -> I16Vector4: ...
    @property
    def owyz(self) -> I16Vector4: ...
    @property
    def owyw(self) -> I16Vector4: ...
    @property
    def owyo(self) -> I16Vector4: ...
    @property
    def owyl(self) -> I16Vector4: ...
    @property
    def owzx(self) -> I16Vector4: ...
    @property
    def owzy(self) -> I16Vector4: ...
    @property
    def owzz(self) -> I16Vector4: ...
    @property
    def owzw(self) -> I16Vector4: ...
    @property
    def owzo(self) -> I16Vector4: ...
    @property
    def owzl(self) -> I16Vector4: ...
    @property
    def owwx(self) -> I16Vector4: ...
    @property
    def owwy(self) -> I16Vector4: ...
    @property
    def owwz(self) -> I16Vector4: ...
    @property
    def owww(self) -> I16Vector4: ...
    @property
    def owwo(self) -> I16Vector4: ...
    @property
    def owwl(self) -> I16Vector4: ...
    @property
    def owox(self) -> I16Vector4: ...
    @property
    def owoy(self) -> I16Vector4: ...
    @property
    def owoz(self) -> I16Vector4: ...
    @property
    def owow(self) -> I16Vector4: ...
    @property
    def owoo(self) -> I16Vector4: ...
    @property
    def owol(self) -> I16Vector4: ...
    @property
    def owlx(self) -> I16Vector4: ...
    @property
    def owly(self) -> I16Vector4: ...
    @property
    def owlz(self) -> I16Vector4: ...
    @property
    def owlw(self) -> I16Vector4: ...
    @property
    def owlo(self) -> I16Vector4: ...
    @property
    def owll(self) -> I16Vector4: ...
    @property
    def ooxx(self) -> I16Vector4: ...
    @property
    def ooxy(self) -> I16Vector4: ...
    @property
    def ooxz(self) -> I16Vector4: ...
    @property
    def ooxw(self) -> I16Vector4: ...
    @property
    def ooxo(self) -> I16Vector4: ...
    @property
    def ooxl(self) -> I16Vector4: ...
    @property
    def ooyx(self) -> I16Vector4: ...
    @property
    def ooyy(self) -> I16Vector4: ...
    @property
    def ooyz(self) -> I16Vector4: ...
    @property
    def ooyw(self) -> I16Vector4: ...
    @property
    def ooyo(self) -> I16Vector4: ...
    @property
    def ooyl(self) -> I16Vector4: ...
    @property
    def oozx(self) -> I16Vector4: ...
    @property
    def oozy(self) -> I16Vector4: ...
    @property
    def oozz(self) -> I16Vector4: ...
    @property
    def oozw(self) -> I16Vector4: ...
    @property
    def oozo(self) -> I16Vector4: ...
    @property
    def oozl(self) -> I16Vector4: ...
    @property
    def oowx(self) -> I16Vector4: ...
    @property
    def oowy(self) -> I16Vector4: ...
    @property
    def oowz(self) -> I16Vector4: ...
    @property
    def ooww(self) -> I16Vector4: ...
    @property
    def oowo(self) -> I16Vector4: ...
    @property
    def oowl(self) -> I16Vector4: ...
    @property
    def ooox(self) -> I16Vector4: ...
    @property
    def oooy(self) -> I16Vector4: ...
    @property
    def oooz(self) -> I16Vector4: ...
    @property
    def ooow(self) -> I16Vector4: ...
    @property
    def oolx(self) -> I16Vector4: ...
    @property
    def ooly(self) -> I16Vector4: ...
    @property
    def oolz(self) -> I16Vector4: ...
    @property
    def oolw(self) -> I16Vector4: ...
    @property
    def olxx(self) -> I16Vector4: ...
    @property
    def olxy(self) -> I16Vector4: ...
    @property
    def olxz(self) -> I16Vector4: ...
    @property
    def olxw(self) -> I16Vector4: ...
    @property
    def olxo(self) -> I16Vector4: ...
    @property
    def olxl(self) -> I16Vector4: ...
    @property
    def olyx(self) -> I16Vector4: ...
    @property
    def olyy(self) -> I16Vector4: ...
    @property
    def olyz(self) -> I16Vector4: ...
    @property
    def olyw(self) -> I16Vector4: ...
    @property
    def olyo(self) -> I16Vector4: ...
    @property
    def olyl(self) -> I16Vector4: ...
    @property
    def olzx(self) -> I16Vector4: ...
    @property
    def olzy(self) -> I16Vector4: ...
    @property
    def olzz(self) -> I16Vector4: ...
    @property
    def olzw(self) -> I16Vector4: ...
    @property
    def olzo(self) -> I16Vector4: ...
    @property
    def olzl(self) -> I16Vector4: ...
    @property
    def olwx(self) -> I16Vector4: ...
    @property
    def olwy(self) -> I16Vector4: ...
    @property
    def olwz(self) -> I16Vector4: ...
    @property
    def olww(self) -> I16Vector4: ...
    @property
    def olwo(self) -> I16Vector4: ...
    @property
    def olwl(self) -> I16Vector4: ...
    @property
    def olox(self) -> I16Vector4: ...
    @property
    def oloy(self) -> I16Vector4: ...
    @property
    def oloz(self) -> I16Vector4: ...
    @property
    def olow(self) -> I16Vector4: ...
    @property
    def ollx(self) -> I16Vector4: ...
    @property
    def olly(self) -> I16Vector4: ...
    @property
    def ollz(self) -> I16Vector4: ...
    @property
    def ollw(self) -> I16Vector4: ...
    @property
    def lxxx(self) -> I16Vector4: ...
    @property
    def lxxy(self) -> I16Vector4: ...
    @property
    def lxxz(self) -> I16Vector4: ...
    @property
    def lxxw(self) -> I16Vector4: ...
    @property
    def lxxo(self) -> I16Vector4: ...
    @property
    def lxxl(self) -> I16Vector4: ...
    @property
    def lxyx(self) -> I16Vector4: ...
    @property
    def lxyy(self) -> I16Vector4: ...
    @property
    def lxyz(self) -> I16Vector4: ...
    @property
    def lxyw(self) -> I16Vector4: ...
    @property
    def lxyo(self) -> I16Vector4: ...
    @property
    def lxyl(self) -> I16Vector4: ...
    @property
    def lxzx(self) -> I16Vector4: ...
    @property
    def lxzy(self) -> I16Vector4: ...
    @property
    def lxzz(self) -> I16Vector4: ...
    @property
    def lxzw(self) -> I16Vector4: ...
    @property
    def lxzo(self) -> I16Vector4: ...
    @property
    def lxzl(self) -> I16Vector4: ...
    @property
    def lxwx(self) -> I16Vector4: ...
    @property
    def lxwy(self) -> I16Vector4: ...
    @property
    def lxwz(self) -> I16Vector4: ...
    @property
    def lxww(self) -> I16Vector4: ...
    @property
    def lxwo(self) -> I16Vector4: ...
    @property
    def lxwl(self) -> I16Vector4: ...
    @property
    def lxox(self) -> I16Vector4: ...
    @property
    def lxoy(self) -> I16Vector4: ...
    @property
    def lxoz(self) -> I16Vector4: ...
    @property
    def lxow(self) -> I16Vector4: ...
    @property
    def lxoo(self) -> I16Vector4: ...
    @property
    def lxol(self) -> I16Vector4: ...
    @property
    def lxlx(self) -> I16Vector4: ...
    @property
    def lxly(self) -> I16Vector4: ...
    @property
    def lxlz(self) -> I16Vector4: ...
    @property
    def lxlw(self) -> I16Vector4: ...
    @property
    def lxlo(self) -> I16Vector4: ...
    @property
    def lxll(self) -> I16Vector4: ...
    @property
    def lyxx(self) -> I16Vector4: ...
    @property
    def lyxy(self) -> I16Vector4: ...
    @property
    def lyxz(self) -> I16Vector4: ...
    @property
    def lyxw(self) -> I16Vector4: ...
    @property
    def lyxo(self) -> I16Vector4: ...
    @property
    def lyxl(self) -> I16Vector4: ...
    @property
    def lyyx(self) -> I16Vector4: ...
    @property
    def lyyy(self) -> I16Vector4: ...
    @property
    def lyyz(self) -> I16Vector4: ...
    @property
    def lyyw(self) -> I16Vector4: ...
    @property
    def lyyo(self) -> I16Vector4: ...
    @property
    def lyyl(self) -> I16Vector4: ...
    @property
    def lyzx(self) -> I16Vector4: ...
    @property
    def lyzy(self) -> I16Vector4: ...
    @property
    def lyzz(self) -> I16Vector4: ...
    @property
    def lyzw(self) -> I16Vector4: ...
    @property
    def lyzo(self) -> I16Vector4: ...
    @property
    def lyzl(self) -> I16Vector4: ...
    @property
    def lywx(self) -> I16Vector4: ...
    @property
    def lywy(self) -> I16Vector4: ...
    @property
    def lywz(self) -> I16Vector4: ...
    @property
    def lyww(self) -> I16Vector4: ...
    @property
    def lywo(self) -> I16Vector4: ...
    @property
    def lywl(self) -> I16Vector4: ...
    @property
    def lyox(self) -> I16Vector4: ...
    @property
    def lyoy(self) -> I16Vector4: ...
    @property
    def lyoz(self) -> I16Vector4: ...
    @property
    def lyow(self) -> I16Vector4: ...
    @property
    def lyoo(self) -> I16Vector4: ...
    @property
    def lyol(self) -> I16Vector4: ...
    @property
    def lylx(self) -> I16Vector4: ...
    @property
    def lyly(self) -> I16Vector4: ...
    @property
    def lylz(self) -> I16Vector4: ...
    @property
    def lylw(self) -> I16Vector4: ...
    @property
    def lylo(self) -> I16Vector4: ...
    @property
    def lyll(self) -> I16Vector4: ...
    @property
    def lzxx(self) -> I16Vector4: ...
    @property
    def lzxy(self) -> I16Vector4: ...
    @property
    def lzxz(self) -> I16Vector4: ...
    @property
    def lzxw(self) -> I16Vector4: ...
    @property
    def lzxo(self) -> I16Vector4: ...
    @property
    def lzxl(self) -> I16Vector4: ...
    @property
    def lzyx(self) -> I16Vector4: ...
    @property
    def lzyy(self) -> I16Vector4: ...
    @property
    def lzyz(self) -> I16Vector4: ...
    @property
    def lzyw(self) -> I16Vector4: ...
    @property
    def lzyo(self) -> I16Vector4: ...
    @property
    def lzyl(self) -> I16Vector4: ...
    @property
    def lzzx(self) -> I16Vector4: ...
    @property
    def lzzy(self) -> I16Vector4: ...
    @property
    def lzzz(self) -> I16Vector4: ...
    @property
    def lzzw(self) -> I16Vector4: ...
    @property
    def lzzo(self) -> I16Vector4: ...
    @property
    def lzzl(self) -> I16Vector4: ...
    @property
    def lzwx(self) -> I16Vector4: ...
    @property
    def lzwy(self) -> I16Vector4: ...
    @property
    def lzwz(self) -> I16Vector4: ...
    @property
    def lzww(self) -> I16Vector4: ...
    @property
    def lzwo(self) -> I16Vector4: ...
    @property
    def lzwl(self) -> I16Vector4: ...
    @property
    def lzox(self) -> I16Vector4: ...
    @property
    def lzoy(self) -> I16Vector4: ...
    @property
    def lzoz(self) -> I16Vector4: ...
    @property
    def lzow(self) -> I16Vector4: ...
    @property
    def lzoo(self) -> I16Vector4: ...
    @property
    def lzol(self) -> I16Vector4: ...
    @property
    def lzlx(self) -> I16Vector4: ...
    @property
    def lzly(self) -> I16Vector4: ...
    @property
    def lzlz(self) -> I16Vector4: ...
    @property
    def lzlw(self) -> I16Vector4: ...
    @property
    def lzlo(self) -> I16Vector4: ...
    @property
    def lzll(self) -> I16Vector4: ...
    @property
    def lwxx(self) -> I16Vector4: ...
    @property
    def lwxy(self) -> I16Vector4: ...
    @property
    def lwxz(self) -> I16Vector4: ...
    @property
    def lwxw(self) -> I16Vector4: ...
    @property
    def lwxo(self) -> I16Vector4: ...
    @property
    def lwxl(self) -> I16Vector4: ...
    @property
    def lwyx(self) -> I16Vector4: ...
    @property
    def lwyy(self) -> I16Vector4: ...
    @property
    def lwyz(self) -> I16Vector4: ...
    @property
    def lwyw(self) -> I16Vector4: ...
    @property
    def lwyo(self) -> I16Vector4: ...
    @property
    def lwyl(self) -> I16Vector4: ...
    @property
    def lwzx(self) -> I16Vector4: ...
    @property
    def lwzy(self) -> I16Vector4: ...
    @property
    def lwzz(self) -> I16Vector4: ...
    @property
    def lwzw(self) -> I16Vector4: ...
    @property
    def lwzo(self) -> I16Vector4: ...
    @property
    def lwzl(self) -> I16Vector4: ...
    @property
    def lwwx(self) -> I16Vector4: ...
    @property
    def lwwy(self) -> I16Vector4: ...
    @property
    def lwwz(self) -> I16Vector4: ...
    @property
    def lwww(self) -> I16Vector4: ...
    @property
    def lwwo(self) -> I16Vector4: ...
    @property
    def lwwl(self) -> I16Vector4: ...
    @property
    def lwox(self) -> I16Vector4: ...
    @property
    def lwoy(self) -> I16Vector4: ...
    @property
    def lwoz(self) -> I16Vector4: ...
    @property
    def lwow(self) -> I16Vector4: ...
    @property
    def lwoo(self) -> I16Vector4: ...
    @property
    def lwol(self) -> I16Vector4: ...
    @property
    def lwlx(self) -> I16Vector4: ...
    @property
    def lwly(self) -> I16Vector4: ...
    @property
    def lwlz(self) -> I16Vector4: ...
    @property
    def lwlw(self) -> I16Vector4: ...
    @property
    def lwlo(self) -> I16Vector4: ...
    @property
    def lwll(self) -> I16Vector4: ...
    @property
    def loxx(self) -> I16Vector4: ...
    @property
    def loxy(self) -> I16Vector4: ...
    @property
    def loxz(self) -> I16Vector4: ...
    @property
    def loxw(self) -> I16Vector4: ...
    @property
    def loxo(self) -> I16Vector4: ...
    @property
    def loxl(self) -> I16Vector4: ...
    @property
    def loyx(self) -> I16Vector4: ...
    @property
    def loyy(self) -> I16Vector4: ...
    @property
    def loyz(self) -> I16Vector4: ...
    @property
    def loyw(self) -> I16Vector4: ...
    @property
    def loyo(self) -> I16Vector4: ...
    @property
    def loyl(self) -> I16Vector4: ...
    @property
    def lozx(self) -> I16Vector4: ...
    @property
    def lozy(self) -> I16Vector4: ...
    @property
    def lozz(self) -> I16Vector4: ...
    @property
    def lozw(self) -> I16Vector4: ...
    @property
    def lozo(self) -> I16Vector4: ...
    @property
    def lozl(self) -> I16Vector4: ...
    @property
    def lowx(self) -> I16Vector4: ...
    @property
    def lowy(self) -> I16Vector4: ...
    @property
    def lowz(self) -> I16Vector4: ...
    @property
    def loww(self) -> I16Vector4: ...
    @property
    def lowo(self) -> I16Vector4: ...
    @property
    def lowl(self) -> I16Vector4: ...
    @property
    def loox(self) -> I16Vector4: ...
    @property
    def looy(self) -> I16Vector4: ...
    @property
    def looz(self) -> I16Vector4: ...
    @property
    def loow(self) -> I16Vector4: ...
    @property
    def lolx(self) -> I16Vector4: ...
    @property
    def loly(self) -> I16Vector4: ...
    @property
    def lolz(self) -> I16Vector4: ...
    @property
    def lolw(self) -> I16Vector4: ...
    @property
    def llxx(self) -> I16Vector4: ...
    @property
    def llxy(self) -> I16Vector4: ...
    @property
    def llxz(self) -> I16Vector4: ...
    @property
    def llxw(self) -> I16Vector4: ...
    @property
    def llxo(self) -> I16Vector4: ...
    @property
    def llxl(self) -> I16Vector4: ...
    @property
    def llyx(self) -> I16Vector4: ...
    @property
    def llyy(self) -> I16Vector4: ...
    @property
    def llyz(self) -> I16Vector4: ...
    @property
    def llyw(self) -> I16Vector4: ...
    @property
    def llyo(self) -> I16Vector4: ...
    @property
    def llyl(self) -> I16Vector4: ...
    @property
    def llzx(self) -> I16Vector4: ...
    @property
    def llzy(self) -> I16Vector4: ...
    @property
    def llzz(self) -> I16Vector4: ...
    @property
    def llzw(self) -> I16Vector4: ...
    @property
    def llzo(self) -> I16Vector4: ...
    @property
    def llzl(self) -> I16Vector4: ...
    @property
    def llwx(self) -> I16Vector4: ...
    @property
    def llwy(self) -> I16Vector4: ...
    @property
    def llwz(self) -> I16Vector4: ...
    @property
    def llww(self) -> I16Vector4: ...
    @property
    def llwo(self) -> I16Vector4: ...
    @property
    def llwl(self) -> I16Vector4: ...
    @property
    def llox(self) -> I16Vector4: ...
    @property
    def lloy(self) -> I16Vector4: ...
    @property
    def lloz(self) -> I16Vector4: ...
    @property
    def llow(self) -> I16Vector4: ...
    @property
    def lllx(self) -> I16Vector4: ...
    @property
    def llly(self) -> I16Vector4: ...
    @property
    def lllz(self) -> I16Vector4: ...
    @property
    def lllw(self) -> I16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> I16Vector2: ...
    @property
    def rg(self) -> I16Vector2: ...
    @property
    def rb(self) -> I16Vector2: ...
    @property
    def ra(self) -> I16Vector2: ...
    @property
    def ro(self) -> I16Vector2: ...
    @property
    def rl(self) -> I16Vector2: ...
    @property
    def gr(self) -> I16Vector2: ...
    @property
    def gg(self) -> I16Vector2: ...
    @property
    def gb(self) -> I16Vector2: ...
    @property
    def ga(self) -> I16Vector2: ...
    @property
    def go(self) -> I16Vector2: ...
    @property
    def gl(self) -> I16Vector2: ...
    @property
    def br(self) -> I16Vector2: ...
    @property
    def bg(self) -> I16Vector2: ...
    @property
    def bb(self) -> I16Vector2: ...
    @property
    def ba(self) -> I16Vector2: ...
    @property
    def bo(self) -> I16Vector2: ...
    @property
    def bl(self) -> I16Vector2: ...
    @property
    def ar(self) -> I16Vector2: ...
    @property
    def ag(self) -> I16Vector2: ...
    @property
    def ab(self) -> I16Vector2: ...
    @property
    def aa(self) -> I16Vector2: ...
    @property
    def ao(self) -> I16Vector2: ...
    @property
    def al(self) -> I16Vector2: ...
    @property
    def og(self) -> I16Vector2: ...
    @property
    def ob(self) -> I16Vector2: ...
    @property
    def oa(self) -> I16Vector2: ...
    @property
    def lr(self) -> I16Vector2: ...
    @property
    def lg(self) -> I16Vector2: ...
    @property
    def lb(self) -> I16Vector2: ...
    @property
    def la(self) -> I16Vector2: ...
    @property
    def rrr(self) -> I16Vector3: ...
    @property
    def rrg(self) -> I16Vector3: ...
    @property
    def rrb(self) -> I16Vector3: ...
    @property
    def rra(self) -> I16Vector3: ...
    @property
    def rro(self) -> I16Vector3: ...
    @property
    def rrl(self) -> I16Vector3: ...
    @property
    def rgr(self) -> I16Vector3: ...
    @property
    def rgg(self) -> I16Vector3: ...
    @property
    def rgb(self) -> I16Vector3: ...
    @property
    def rga(self) -> I16Vector3: ...
    @property
    def rgo(self) -> I16Vector3: ...
    @property
    def rgl(self) -> I16Vector3: ...
    @property
    def rbr(self) -> I16Vector3: ...
    @property
    def rbg(self) -> I16Vector3: ...
    @property
    def rbb(self) -> I16Vector3: ...
    @property
    def rba(self) -> I16Vector3: ...
    @property
    def rbo(self) -> I16Vector3: ...
    @property
    def rbl(self) -> I16Vector3: ...
    @property
    def rar(self) -> I16Vector3: ...
    @property
    def rag(self) -> I16Vector3: ...
    @property
    def rab(self) -> I16Vector3: ...
    @property
    def raa(self) -> I16Vector3: ...
    @property
    def rao(self) -> I16Vector3: ...
    @property
    def ral(self) -> I16Vector3: ...
    @property
    def ror(self) -> I16Vector3: ...
    @property
    def rog(self) -> I16Vector3: ...
    @property
    def rob(self) -> I16Vector3: ...
    @property
    def roa(self) -> I16Vector3: ...
    @property
    def roo(self) -> I16Vector3: ...
    @property
    def rol(self) -> I16Vector3: ...
    @property
    def rlr(self) -> I16Vector3: ...
    @property
    def rlg(self) -> I16Vector3: ...
    @property
    def rlb(self) -> I16Vector3: ...
    @property
    def rla(self) -> I16Vector3: ...
    @property
    def rlo(self) -> I16Vector3: ...
    @property
    def rll(self) -> I16Vector3: ...
    @property
    def grr(self) -> I16Vector3: ...
    @property
    def grg(self) -> I16Vector3: ...
    @property
    def grb(self) -> I16Vector3: ...
    @property
    def gra(self) -> I16Vector3: ...
    @property
    def gro(self) -> I16Vector3: ...
    @property
    def grl(self) -> I16Vector3: ...
    @property
    def ggr(self) -> I16Vector3: ...
    @property
    def ggg(self) -> I16Vector3: ...
    @property
    def ggb(self) -> I16Vector3: ...
    @property
    def gga(self) -> I16Vector3: ...
    @property
    def ggo(self) -> I16Vector3: ...
    @property
    def ggl(self) -> I16Vector3: ...
    @property
    def gbr(self) -> I16Vector3: ...
    @property
    def gbg(self) -> I16Vector3: ...
    @property
    def gbb(self) -> I16Vector3: ...
    @property
    def gba(self) -> I16Vector3: ...
    @property
    def gbo(self) -> I16Vector3: ...
    @property
    def gbl(self) -> I16Vector3: ...
    @property
    def gar(self) -> I16Vector3: ...
    @property
    def gag(self) -> I16Vector3: ...
    @property
    def gab(self) -> I16Vector3: ...
    @property
    def gaa(self) -> I16Vector3: ...
    @property
    def gao(self) -> I16Vector3: ...
    @property
    def gal(self) -> I16Vector3: ...
    @property
    def gor(self) -> I16Vector3: ...
    @property
    def gog(self) -> I16Vector3: ...
    @property
    def gob(self) -> I16Vector3: ...
    @property
    def goa(self) -> I16Vector3: ...
    @property
    def goo(self) -> I16Vector3: ...
    @property
    def gol(self) -> I16Vector3: ...
    @property
    def glr(self) -> I16Vector3: ...
    @property
    def glg(self) -> I16Vector3: ...
    @property
    def glb(self) -> I16Vector3: ...
    @property
    def gla(self) -> I16Vector3: ...
    @property
    def glo(self) -> I16Vector3: ...
    @property
    def gll(self) -> I16Vector3: ...
    @property
    def brr(self) -> I16Vector3: ...
    @property
    def brg(self) -> I16Vector3: ...
    @property
    def brb(self) -> I16Vector3: ...
    @property
    def bra(self) -> I16Vector3: ...
    @property
    def bro(self) -> I16Vector3: ...
    @property
    def brl(self) -> I16Vector3: ...
    @property
    def bgr(self) -> I16Vector3: ...
    @property
    def bgg(self) -> I16Vector3: ...
    @property
    def bgb(self) -> I16Vector3: ...
    @property
    def bga(self) -> I16Vector3: ...
    @property
    def bgo(self) -> I16Vector3: ...
    @property
    def bgl(self) -> I16Vector3: ...
    @property
    def bbr(self) -> I16Vector3: ...
    @property
    def bbg(self) -> I16Vector3: ...
    @property
    def bbb(self) -> I16Vector3: ...
    @property
    def bba(self) -> I16Vector3: ...
    @property
    def bbo(self) -> I16Vector3: ...
    @property
    def bbl(self) -> I16Vector3: ...
    @property
    def bar(self) -> I16Vector3: ...
    @property
    def bag(self) -> I16Vector3: ...
    @property
    def bab(self) -> I16Vector3: ...
    @property
    def baa(self) -> I16Vector3: ...
    @property
    def bao(self) -> I16Vector3: ...
    @property
    def bal(self) -> I16Vector3: ...
    @property
    def bor(self) -> I16Vector3: ...
    @property
    def bog(self) -> I16Vector3: ...
    @property
    def bob(self) -> I16Vector3: ...
    @property
    def boa(self) -> I16Vector3: ...
    @property
    def boo(self) -> I16Vector3: ...
    @property
    def bol(self) -> I16Vector3: ...
    @property
    def blr(self) -> I16Vector3: ...
    @property
    def blg(self) -> I16Vector3: ...
    @property
    def blb(self) -> I16Vector3: ...
    @property
    def bla(self) -> I16Vector3: ...
    @property
    def blo(self) -> I16Vector3: ...
    @property
    def bll(self) -> I16Vector3: ...
    @property
    def arr(self) -> I16Vector3: ...
    @property
    def arg(self) -> I16Vector3: ...
    @property
    def arb(self) -> I16Vector3: ...
    @property
    def ara(self) -> I16Vector3: ...
    @property
    def aro(self) -> I16Vector3: ...
    @property
    def arl(self) -> I16Vector3: ...
    @property
    def agr(self) -> I16Vector3: ...
    @property
    def agg(self) -> I16Vector3: ...
    @property
    def agb(self) -> I16Vector3: ...
    @property
    def aga(self) -> I16Vector3: ...
    @property
    def ago(self) -> I16Vector3: ...
    @property
    def agl(self) -> I16Vector3: ...
    @property
    def abr(self) -> I16Vector3: ...
    @property
    def abg(self) -> I16Vector3: ...
    @property
    def abb(self) -> I16Vector3: ...
    @property
    def aba(self) -> I16Vector3: ...
    @property
    def abo(self) -> I16Vector3: ...
    @property
    def abl(self) -> I16Vector3: ...
    @property
    def aar(self) -> I16Vector3: ...
    @property
    def aag(self) -> I16Vector3: ...
    @property
    def aab(self) -> I16Vector3: ...
    @property
    def aaa(self) -> I16Vector3: ...
    @property
    def aao(self) -> I16Vector3: ...
    @property
    def aal(self) -> I16Vector3: ...
    @property
    def aor(self) -> I16Vector3: ...
    @property
    def aog(self) -> I16Vector3: ...
    @property
    def aob(self) -> I16Vector3: ...
    @property
    def aoa(self) -> I16Vector3: ...
    @property
    def aoo(self) -> I16Vector3: ...
    @property
    def aol(self) -> I16Vector3: ...
    @property
    def alr(self) -> I16Vector3: ...
    @property
    def alg(self) -> I16Vector3: ...
    @property
    def alb(self) -> I16Vector3: ...
    @property
    def ala(self) -> I16Vector3: ...
    @property
    def alo(self) -> I16Vector3: ...
    @property
    def all(self) -> I16Vector3: ...
    @property
    def orr(self) -> I16Vector3: ...
    @property
    def org(self) -> I16Vector3: ...
    @property
    def orb(self) -> I16Vector3: ...
    @property
    def ora(self) -> I16Vector3: ...
    @property
    def oro(self) -> I16Vector3: ...
    @property
    def orl(self) -> I16Vector3: ...
    @property
    def ogr(self) -> I16Vector3: ...
    @property
    def ogg(self) -> I16Vector3: ...
    @property
    def ogb(self) -> I16Vector3: ...
    @property
    def oga(self) -> I16Vector3: ...
    @property
    def ogo(self) -> I16Vector3: ...
    @property
    def ogl(self) -> I16Vector3: ...
    @property
    def obr(self) -> I16Vector3: ...
    @property
    def obg(self) -> I16Vector3: ...
    @property
    def obb(self) -> I16Vector3: ...
    @property
    def oba(self) -> I16Vector3: ...
    @property
    def obo(self) -> I16Vector3: ...
    @property
    def obl(self) -> I16Vector3: ...
    @property
    def oar(self) -> I16Vector3: ...
    @property
    def oag(self) -> I16Vector3: ...
    @property
    def oab(self) -> I16Vector3: ...
    @property
    def oaa(self) -> I16Vector3: ...
    @property
    def oao(self) -> I16Vector3: ...
    @property
    def oal(self) -> I16Vector3: ...
    @property
    def oor(self) -> I16Vector3: ...
    @property
    def oog(self) -> I16Vector3: ...
    @property
    def oob(self) -> I16Vector3: ...
    @property
    def ooa(self) -> I16Vector3: ...
    @property
    def olr(self) -> I16Vector3: ...
    @property
    def olg(self) -> I16Vector3: ...
    @property
    def olb(self) -> I16Vector3: ...
    @property
    def ola(self) -> I16Vector3: ...
    @property
    def lrr(self) -> I16Vector3: ...
    @property
    def lrg(self) -> I16Vector3: ...
    @property
    def lrb(self) -> I16Vector3: ...
    @property
    def lra(self) -> I16Vector3: ...
    @property
    def lro(self) -> I16Vector3: ...
    @property
    def lrl(self) -> I16Vector3: ...
    @property
    def lgr(self) -> I16Vector3: ...
    @property
    def lgg(self) -> I16Vector3: ...
    @property
    def lgb(self) -> I16Vector3: ...
    @property
    def lga(self) -> I16Vector3: ...
    @property
    def lgo(self) -> I16Vector3: ...
    @property
    def lgl(self) -> I16Vector3: ...
    @property
    def lbr(self) -> I16Vector3: ...
    @property
    def lbg(self) -> I16Vector3: ...
    @property
    def lbb(self) -> I16Vector3: ...
    @property
    def lba(self) -> I16Vector3: ...
    @property
    def lbo(self) -> I16Vector3: ...
    @property
    def lbl(self) -> I16Vector3: ...
    @property
    def lar(self) -> I16Vector3: ...
    @property
    def lag(self) -> I16Vector3: ...
    @property
    def lab(self) -> I16Vector3: ...
    @property
    def laa(self) -> I16Vector3: ...
    @property
    def lao(self) -> I16Vector3: ...
    @property
    def lal(self) -> I16Vector3: ...
    @property
    def lor(self) -> I16Vector3: ...
    @property
    def log(self) -> I16Vector3: ...
    @property
    def lob(self) -> I16Vector3: ...
    @property
    def loa(self) -> I16Vector3: ...
    @property
    def llr(self) -> I16Vector3: ...
    @property
    def llg(self) -> I16Vector3: ...
    @property
    def llb(self) -> I16Vector3: ...
    @property
    def lla(self) -> I16Vector3: ...
    @property
    def rrrr(self) -> I16Vector4: ...
    @property
    def rrrg(self) -> I16Vector4: ...
    @property
    def rrrb(self) -> I16Vector4: ...
    @property
    def rrra(self) -> I16Vector4: ...
    @property
    def rrro(self) -> I16Vector4: ...
    @property
    def rrrl(self) -> I16Vector4: ...
    @property
    def rrgr(self) -> I16Vector4: ...
    @property
    def rrgg(self) -> I16Vector4: ...
    @property
    def rrgb(self) -> I16Vector4: ...
    @property
    def rrga(self) -> I16Vector4: ...
    @property
    def rrgo(self) -> I16Vector4: ...
    @property
    def rrgl(self) -> I16Vector4: ...
    @property
    def rrbr(self) -> I16Vector4: ...
    @property
    def rrbg(self) -> I16Vector4: ...
    @property
    def rrbb(self) -> I16Vector4: ...
    @property
    def rrba(self) -> I16Vector4: ...
    @property
    def rrbo(self) -> I16Vector4: ...
    @property
    def rrbl(self) -> I16Vector4: ...
    @property
    def rrar(self) -> I16Vector4: ...
    @property
    def rrag(self) -> I16Vector4: ...
    @property
    def rrab(self) -> I16Vector4: ...
    @property
    def rraa(self) -> I16Vector4: ...
    @property
    def rrao(self) -> I16Vector4: ...
    @property
    def rral(self) -> I16Vector4: ...
    @property
    def rror(self) -> I16Vector4: ...
    @property
    def rrog(self) -> I16Vector4: ...
    @property
    def rrob(self) -> I16Vector4: ...
    @property
    def rroa(self) -> I16Vector4: ...
    @property
    def rroo(self) -> I16Vector4: ...
    @property
    def rrol(self) -> I16Vector4: ...
    @property
    def rrlr(self) -> I16Vector4: ...
    @property
    def rrlg(self) -> I16Vector4: ...
    @property
    def rrlb(self) -> I16Vector4: ...
    @property
    def rrla(self) -> I16Vector4: ...
    @property
    def rrlo(self) -> I16Vector4: ...
    @property
    def rrll(self) -> I16Vector4: ...
    @property
    def rgrr(self) -> I16Vector4: ...
    @property
    def rgrg(self) -> I16Vector4: ...
    @property
    def rgrb(self) -> I16Vector4: ...
    @property
    def rgra(self) -> I16Vector4: ...
    @property
    def rgro(self) -> I16Vector4: ...
    @property
    def rgrl(self) -> I16Vector4: ...
    @property
    def rggr(self) -> I16Vector4: ...
    @property
    def rggg(self) -> I16Vector4: ...
    @property
    def rggb(self) -> I16Vector4: ...
    @property
    def rgga(self) -> I16Vector4: ...
    @property
    def rggo(self) -> I16Vector4: ...
    @property
    def rggl(self) -> I16Vector4: ...
    @property
    def rgbr(self) -> I16Vector4: ...
    @property
    def rgbg(self) -> I16Vector4: ...
    @property
    def rgbb(self) -> I16Vector4: ...
    @property
    def rgba(self) -> I16Vector4: ...
    @property
    def rgbo(self) -> I16Vector4: ...
    @property
    def rgbl(self) -> I16Vector4: ...
    @property
    def rgar(self) -> I16Vector4: ...
    @property
    def rgag(self) -> I16Vector4: ...
    @property
    def rgab(self) -> I16Vector4: ...
    @property
    def rgaa(self) -> I16Vector4: ...
    @property
    def rgao(self) -> I16Vector4: ...
    @property
    def rgal(self) -> I16Vector4: ...
    @property
    def rgor(self) -> I16Vector4: ...
    @property
    def rgog(self) -> I16Vector4: ...
    @property
    def rgob(self) -> I16Vector4: ...
    @property
    def rgoa(self) -> I16Vector4: ...
    @property
    def rgoo(self) -> I16Vector4: ...
    @property
    def rgol(self) -> I16Vector4: ...
    @property
    def rglr(self) -> I16Vector4: ...
    @property
    def rglg(self) -> I16Vector4: ...
    @property
    def rglb(self) -> I16Vector4: ...
    @property
    def rgla(self) -> I16Vector4: ...
    @property
    def rglo(self) -> I16Vector4: ...
    @property
    def rgll(self) -> I16Vector4: ...
    @property
    def rbrr(self) -> I16Vector4: ...
    @property
    def rbrg(self) -> I16Vector4: ...
    @property
    def rbrb(self) -> I16Vector4: ...
    @property
    def rbra(self) -> I16Vector4: ...
    @property
    def rbro(self) -> I16Vector4: ...
    @property
    def rbrl(self) -> I16Vector4: ...
    @property
    def rbgr(self) -> I16Vector4: ...
    @property
    def rbgg(self) -> I16Vector4: ...
    @property
    def rbgb(self) -> I16Vector4: ...
    @property
    def rbga(self) -> I16Vector4: ...
    @property
    def rbgo(self) -> I16Vector4: ...
    @property
    def rbgl(self) -> I16Vector4: ...
    @property
    def rbbr(self) -> I16Vector4: ...
    @property
    def rbbg(self) -> I16Vector4: ...
    @property
    def rbbb(self) -> I16Vector4: ...
    @property
    def rbba(self) -> I16Vector4: ...
    @property
    def rbbo(self) -> I16Vector4: ...
    @property
    def rbbl(self) -> I16Vector4: ...
    @property
    def rbar(self) -> I16Vector4: ...
    @property
    def rbag(self) -> I16Vector4: ...
    @property
    def rbab(self) -> I16Vector4: ...
    @property
    def rbaa(self) -> I16Vector4: ...
    @property
    def rbao(self) -> I16Vector4: ...
    @property
    def rbal(self) -> I16Vector4: ...
    @property
    def rbor(self) -> I16Vector4: ...
    @property
    def rbog(self) -> I16Vector4: ...
    @property
    def rbob(self) -> I16Vector4: ...
    @property
    def rboa(self) -> I16Vector4: ...
    @property
    def rboo(self) -> I16Vector4: ...
    @property
    def rbol(self) -> I16Vector4: ...
    @property
    def rblr(self) -> I16Vector4: ...
    @property
    def rblg(self) -> I16Vector4: ...
    @property
    def rblb(self) -> I16Vector4: ...
    @property
    def rbla(self) -> I16Vector4: ...
    @property
    def rblo(self) -> I16Vector4: ...
    @property
    def rbll(self) -> I16Vector4: ...
    @property
    def rarr(self) -> I16Vector4: ...
    @property
    def rarg(self) -> I16Vector4: ...
    @property
    def rarb(self) -> I16Vector4: ...
    @property
    def rara(self) -> I16Vector4: ...
    @property
    def raro(self) -> I16Vector4: ...
    @property
    def rarl(self) -> I16Vector4: ...
    @property
    def ragr(self) -> I16Vector4: ...
    @property
    def ragg(self) -> I16Vector4: ...
    @property
    def ragb(self) -> I16Vector4: ...
    @property
    def raga(self) -> I16Vector4: ...
    @property
    def rago(self) -> I16Vector4: ...
    @property
    def ragl(self) -> I16Vector4: ...
    @property
    def rabr(self) -> I16Vector4: ...
    @property
    def rabg(self) -> I16Vector4: ...
    @property
    def rabb(self) -> I16Vector4: ...
    @property
    def raba(self) -> I16Vector4: ...
    @property
    def rabo(self) -> I16Vector4: ...
    @property
    def rabl(self) -> I16Vector4: ...
    @property
    def raar(self) -> I16Vector4: ...
    @property
    def raag(self) -> I16Vector4: ...
    @property
    def raab(self) -> I16Vector4: ...
    @property
    def raaa(self) -> I16Vector4: ...
    @property
    def raao(self) -> I16Vector4: ...
    @property
    def raal(self) -> I16Vector4: ...
    @property
    def raor(self) -> I16Vector4: ...
    @property
    def raog(self) -> I16Vector4: ...
    @property
    def raob(self) -> I16Vector4: ...
    @property
    def raoa(self) -> I16Vector4: ...
    @property
    def raoo(self) -> I16Vector4: ...
    @property
    def raol(self) -> I16Vector4: ...
    @property
    def ralr(self) -> I16Vector4: ...
    @property
    def ralg(self) -> I16Vector4: ...
    @property
    def ralb(self) -> I16Vector4: ...
    @property
    def rala(self) -> I16Vector4: ...
    @property
    def ralo(self) -> I16Vector4: ...
    @property
    def rall(self) -> I16Vector4: ...
    @property
    def rorr(self) -> I16Vector4: ...
    @property
    def rorg(self) -> I16Vector4: ...
    @property
    def rorb(self) -> I16Vector4: ...
    @property
    def rora(self) -> I16Vector4: ...
    @property
    def roro(self) -> I16Vector4: ...
    @property
    def rorl(self) -> I16Vector4: ...
    @property
    def rogr(self) -> I16Vector4: ...
    @property
    def rogg(self) -> I16Vector4: ...
    @property
    def rogb(self) -> I16Vector4: ...
    @property
    def roga(self) -> I16Vector4: ...
    @property
    def rogo(self) -> I16Vector4: ...
    @property
    def rogl(self) -> I16Vector4: ...
    @property
    def robr(self) -> I16Vector4: ...
    @property
    def robg(self) -> I16Vector4: ...
    @property
    def robb(self) -> I16Vector4: ...
    @property
    def roba(self) -> I16Vector4: ...
    @property
    def robo(self) -> I16Vector4: ...
    @property
    def robl(self) -> I16Vector4: ...
    @property
    def roar(self) -> I16Vector4: ...
    @property
    def roag(self) -> I16Vector4: ...
    @property
    def roab(self) -> I16Vector4: ...
    @property
    def roaa(self) -> I16Vector4: ...
    @property
    def roao(self) -> I16Vector4: ...
    @property
    def roal(self) -> I16Vector4: ...
    @property
    def roor(self) -> I16Vector4: ...
    @property
    def roog(self) -> I16Vector4: ...
    @property
    def roob(self) -> I16Vector4: ...
    @property
    def rooa(self) -> I16Vector4: ...
    @property
    def rooo(self) -> I16Vector4: ...
    @property
    def rool(self) -> I16Vector4: ...
    @property
    def rolr(self) -> I16Vector4: ...
    @property
    def rolg(self) -> I16Vector4: ...
    @property
    def rolb(self) -> I16Vector4: ...
    @property
    def rola(self) -> I16Vector4: ...
    @property
    def rolo(self) -> I16Vector4: ...
    @property
    def roll(self) -> I16Vector4: ...
    @property
    def rlrr(self) -> I16Vector4: ...
    @property
    def rlrg(self) -> I16Vector4: ...
    @property
    def rlrb(self) -> I16Vector4: ...
    @property
    def rlra(self) -> I16Vector4: ...
    @property
    def rlro(self) -> I16Vector4: ...
    @property
    def rlrl(self) -> I16Vector4: ...
    @property
    def rlgr(self) -> I16Vector4: ...
    @property
    def rlgg(self) -> I16Vector4: ...
    @property
    def rlgb(self) -> I16Vector4: ...
    @property
    def rlga(self) -> I16Vector4: ...
    @property
    def rlgo(self) -> I16Vector4: ...
    @property
    def rlgl(self) -> I16Vector4: ...
    @property
    def rlbr(self) -> I16Vector4: ...
    @property
    def rlbg(self) -> I16Vector4: ...
    @property
    def rlbb(self) -> I16Vector4: ...
    @property
    def rlba(self) -> I16Vector4: ...
    @property
    def rlbo(self) -> I16Vector4: ...
    @property
    def rlbl(self) -> I16Vector4: ...
    @property
    def rlar(self) -> I16Vector4: ...
    @property
    def rlag(self) -> I16Vector4: ...
    @property
    def rlab(self) -> I16Vector4: ...
    @property
    def rlaa(self) -> I16Vector4: ...
    @property
    def rlao(self) -> I16Vector4: ...
    @property
    def rlal(self) -> I16Vector4: ...
    @property
    def rlor(self) -> I16Vector4: ...
    @property
    def rlog(self) -> I16Vector4: ...
    @property
    def rlob(self) -> I16Vector4: ...
    @property
    def rloa(self) -> I16Vector4: ...
    @property
    def rloo(self) -> I16Vector4: ...
    @property
    def rlol(self) -> I16Vector4: ...
    @property
    def rllr(self) -> I16Vector4: ...
    @property
    def rllg(self) -> I16Vector4: ...
    @property
    def rllb(self) -> I16Vector4: ...
    @property
    def rlla(self) -> I16Vector4: ...
    @property
    def rllo(self) -> I16Vector4: ...
    @property
    def rlll(self) -> I16Vector4: ...
    @property
    def grrr(self) -> I16Vector4: ...
    @property
    def grrg(self) -> I16Vector4: ...
    @property
    def grrb(self) -> I16Vector4: ...
    @property
    def grra(self) -> I16Vector4: ...
    @property
    def grro(self) -> I16Vector4: ...
    @property
    def grrl(self) -> I16Vector4: ...
    @property
    def grgr(self) -> I16Vector4: ...
    @property
    def grgg(self) -> I16Vector4: ...
    @property
    def grgb(self) -> I16Vector4: ...
    @property
    def grga(self) -> I16Vector4: ...
    @property
    def grgo(self) -> I16Vector4: ...
    @property
    def grgl(self) -> I16Vector4: ...
    @property
    def grbr(self) -> I16Vector4: ...
    @property
    def grbg(self) -> I16Vector4: ...
    @property
    def grbb(self) -> I16Vector4: ...
    @property
    def grba(self) -> I16Vector4: ...
    @property
    def grbo(self) -> I16Vector4: ...
    @property
    def grbl(self) -> I16Vector4: ...
    @property
    def grar(self) -> I16Vector4: ...
    @property
    def grag(self) -> I16Vector4: ...
    @property
    def grab(self) -> I16Vector4: ...
    @property
    def graa(self) -> I16Vector4: ...
    @property
    def grao(self) -> I16Vector4: ...
    @property
    def gral(self) -> I16Vector4: ...
    @property
    def gror(self) -> I16Vector4: ...
    @property
    def grog(self) -> I16Vector4: ...
    @property
    def grob(self) -> I16Vector4: ...
    @property
    def groa(self) -> I16Vector4: ...
    @property
    def groo(self) -> I16Vector4: ...
    @property
    def grol(self) -> I16Vector4: ...
    @property
    def grlr(self) -> I16Vector4: ...
    @property
    def grlg(self) -> I16Vector4: ...
    @property
    def grlb(self) -> I16Vector4: ...
    @property
    def grla(self) -> I16Vector4: ...
    @property
    def grlo(self) -> I16Vector4: ...
    @property
    def grll(self) -> I16Vector4: ...
    @property
    def ggrr(self) -> I16Vector4: ...
    @property
    def ggrg(self) -> I16Vector4: ...
    @property
    def ggrb(self) -> I16Vector4: ...
    @property
    def ggra(self) -> I16Vector4: ...
    @property
    def ggro(self) -> I16Vector4: ...
    @property
    def ggrl(self) -> I16Vector4: ...
    @property
    def gggr(self) -> I16Vector4: ...
    @property
    def gggg(self) -> I16Vector4: ...
    @property
    def gggb(self) -> I16Vector4: ...
    @property
    def ggga(self) -> I16Vector4: ...
    @property
    def gggo(self) -> I16Vector4: ...
    @property
    def gggl(self) -> I16Vector4: ...
    @property
    def ggbr(self) -> I16Vector4: ...
    @property
    def ggbg(self) -> I16Vector4: ...
    @property
    def ggbb(self) -> I16Vector4: ...
    @property
    def ggba(self) -> I16Vector4: ...
    @property
    def ggbo(self) -> I16Vector4: ...
    @property
    def ggbl(self) -> I16Vector4: ...
    @property
    def ggar(self) -> I16Vector4: ...
    @property
    def ggag(self) -> I16Vector4: ...
    @property
    def ggab(self) -> I16Vector4: ...
    @property
    def ggaa(self) -> I16Vector4: ...
    @property
    def ggao(self) -> I16Vector4: ...
    @property
    def ggal(self) -> I16Vector4: ...
    @property
    def ggor(self) -> I16Vector4: ...
    @property
    def ggog(self) -> I16Vector4: ...
    @property
    def ggob(self) -> I16Vector4: ...
    @property
    def ggoa(self) -> I16Vector4: ...
    @property
    def ggoo(self) -> I16Vector4: ...
    @property
    def ggol(self) -> I16Vector4: ...
    @property
    def gglr(self) -> I16Vector4: ...
    @property
    def gglg(self) -> I16Vector4: ...
    @property
    def gglb(self) -> I16Vector4: ...
    @property
    def ggla(self) -> I16Vector4: ...
    @property
    def gglo(self) -> I16Vector4: ...
    @property
    def ggll(self) -> I16Vector4: ...
    @property
    def gbrr(self) -> I16Vector4: ...
    @property
    def gbrg(self) -> I16Vector4: ...
    @property
    def gbrb(self) -> I16Vector4: ...
    @property
    def gbra(self) -> I16Vector4: ...
    @property
    def gbro(self) -> I16Vector4: ...
    @property
    def gbrl(self) -> I16Vector4: ...
    @property
    def gbgr(self) -> I16Vector4: ...
    @property
    def gbgg(self) -> I16Vector4: ...
    @property
    def gbgb(self) -> I16Vector4: ...
    @property
    def gbga(self) -> I16Vector4: ...
    @property
    def gbgo(self) -> I16Vector4: ...
    @property
    def gbgl(self) -> I16Vector4: ...
    @property
    def gbbr(self) -> I16Vector4: ...
    @property
    def gbbg(self) -> I16Vector4: ...
    @property
    def gbbb(self) -> I16Vector4: ...
    @property
    def gbba(self) -> I16Vector4: ...
    @property
    def gbbo(self) -> I16Vector4: ...
    @property
    def gbbl(self) -> I16Vector4: ...
    @property
    def gbar(self) -> I16Vector4: ...
    @property
    def gbag(self) -> I16Vector4: ...
    @property
    def gbab(self) -> I16Vector4: ...
    @property
    def gbaa(self) -> I16Vector4: ...
    @property
    def gbao(self) -> I16Vector4: ...
    @property
    def gbal(self) -> I16Vector4: ...
    @property
    def gbor(self) -> I16Vector4: ...
    @property
    def gbog(self) -> I16Vector4: ...
    @property
    def gbob(self) -> I16Vector4: ...
    @property
    def gboa(self) -> I16Vector4: ...
    @property
    def gboo(self) -> I16Vector4: ...
    @property
    def gbol(self) -> I16Vector4: ...
    @property
    def gblr(self) -> I16Vector4: ...
    @property
    def gblg(self) -> I16Vector4: ...
    @property
    def gblb(self) -> I16Vector4: ...
    @property
    def gbla(self) -> I16Vector4: ...
    @property
    def gblo(self) -> I16Vector4: ...
    @property
    def gbll(self) -> I16Vector4: ...
    @property
    def garr(self) -> I16Vector4: ...
    @property
    def garg(self) -> I16Vector4: ...
    @property
    def garb(self) -> I16Vector4: ...
    @property
    def gara(self) -> I16Vector4: ...
    @property
    def garo(self) -> I16Vector4: ...
    @property
    def garl(self) -> I16Vector4: ...
    @property
    def gagr(self) -> I16Vector4: ...
    @property
    def gagg(self) -> I16Vector4: ...
    @property
    def gagb(self) -> I16Vector4: ...
    @property
    def gaga(self) -> I16Vector4: ...
    @property
    def gago(self) -> I16Vector4: ...
    @property
    def gagl(self) -> I16Vector4: ...
    @property
    def gabr(self) -> I16Vector4: ...
    @property
    def gabg(self) -> I16Vector4: ...
    @property
    def gabb(self) -> I16Vector4: ...
    @property
    def gaba(self) -> I16Vector4: ...
    @property
    def gabo(self) -> I16Vector4: ...
    @property
    def gabl(self) -> I16Vector4: ...
    @property
    def gaar(self) -> I16Vector4: ...
    @property
    def gaag(self) -> I16Vector4: ...
    @property
    def gaab(self) -> I16Vector4: ...
    @property
    def gaaa(self) -> I16Vector4: ...
    @property
    def gaao(self) -> I16Vector4: ...
    @property
    def gaal(self) -> I16Vector4: ...
    @property
    def gaor(self) -> I16Vector4: ...
    @property
    def gaog(self) -> I16Vector4: ...
    @property
    def gaob(self) -> I16Vector4: ...
    @property
    def gaoa(self) -> I16Vector4: ...
    @property
    def gaoo(self) -> I16Vector4: ...
    @property
    def gaol(self) -> I16Vector4: ...
    @property
    def galr(self) -> I16Vector4: ...
    @property
    def galg(self) -> I16Vector4: ...
    @property
    def galb(self) -> I16Vector4: ...
    @property
    def gala(self) -> I16Vector4: ...
    @property
    def galo(self) -> I16Vector4: ...
    @property
    def gall(self) -> I16Vector4: ...
    @property
    def gorr(self) -> I16Vector4: ...
    @property
    def gorg(self) -> I16Vector4: ...
    @property
    def gorb(self) -> I16Vector4: ...
    @property
    def gora(self) -> I16Vector4: ...
    @property
    def goro(self) -> I16Vector4: ...
    @property
    def gorl(self) -> I16Vector4: ...
    @property
    def gogr(self) -> I16Vector4: ...
    @property
    def gogg(self) -> I16Vector4: ...
    @property
    def gogb(self) -> I16Vector4: ...
    @property
    def goga(self) -> I16Vector4: ...
    @property
    def gogo(self) -> I16Vector4: ...
    @property
    def gogl(self) -> I16Vector4: ...
    @property
    def gobr(self) -> I16Vector4: ...
    @property
    def gobg(self) -> I16Vector4: ...
    @property
    def gobb(self) -> I16Vector4: ...
    @property
    def goba(self) -> I16Vector4: ...
    @property
    def gobo(self) -> I16Vector4: ...
    @property
    def gobl(self) -> I16Vector4: ...
    @property
    def goar(self) -> I16Vector4: ...
    @property
    def goag(self) -> I16Vector4: ...
    @property
    def goab(self) -> I16Vector4: ...
    @property
    def goaa(self) -> I16Vector4: ...
    @property
    def goao(self) -> I16Vector4: ...
    @property
    def goal(self) -> I16Vector4: ...
    @property
    def goor(self) -> I16Vector4: ...
    @property
    def goog(self) -> I16Vector4: ...
    @property
    def goob(self) -> I16Vector4: ...
    @property
    def gooa(self) -> I16Vector4: ...
    @property
    def gooo(self) -> I16Vector4: ...
    @property
    def gool(self) -> I16Vector4: ...
    @property
    def golr(self) -> I16Vector4: ...
    @property
    def golg(self) -> I16Vector4: ...
    @property
    def golb(self) -> I16Vector4: ...
    @property
    def gola(self) -> I16Vector4: ...
    @property
    def golo(self) -> I16Vector4: ...
    @property
    def goll(self) -> I16Vector4: ...
    @property
    def glrr(self) -> I16Vector4: ...
    @property
    def glrg(self) -> I16Vector4: ...
    @property
    def glrb(self) -> I16Vector4: ...
    @property
    def glra(self) -> I16Vector4: ...
    @property
    def glro(self) -> I16Vector4: ...
    @property
    def glrl(self) -> I16Vector4: ...
    @property
    def glgr(self) -> I16Vector4: ...
    @property
    def glgg(self) -> I16Vector4: ...
    @property
    def glgb(self) -> I16Vector4: ...
    @property
    def glga(self) -> I16Vector4: ...
    @property
    def glgo(self) -> I16Vector4: ...
    @property
    def glgl(self) -> I16Vector4: ...
    @property
    def glbr(self) -> I16Vector4: ...
    @property
    def glbg(self) -> I16Vector4: ...
    @property
    def glbb(self) -> I16Vector4: ...
    @property
    def glba(self) -> I16Vector4: ...
    @property
    def glbo(self) -> I16Vector4: ...
    @property
    def glbl(self) -> I16Vector4: ...
    @property
    def glar(self) -> I16Vector4: ...
    @property
    def glag(self) -> I16Vector4: ...
    @property
    def glab(self) -> I16Vector4: ...
    @property
    def glaa(self) -> I16Vector4: ...
    @property
    def glao(self) -> I16Vector4: ...
    @property
    def glal(self) -> I16Vector4: ...
    @property
    def glor(self) -> I16Vector4: ...
    @property
    def glog(self) -> I16Vector4: ...
    @property
    def glob(self) -> I16Vector4: ...
    @property
    def gloa(self) -> I16Vector4: ...
    @property
    def gloo(self) -> I16Vector4: ...
    @property
    def glol(self) -> I16Vector4: ...
    @property
    def gllr(self) -> I16Vector4: ...
    @property
    def gllg(self) -> I16Vector4: ...
    @property
    def gllb(self) -> I16Vector4: ...
    @property
    def glla(self) -> I16Vector4: ...
    @property
    def gllo(self) -> I16Vector4: ...
    @property
    def glll(self) -> I16Vector4: ...
    @property
    def brrr(self) -> I16Vector4: ...
    @property
    def brrg(self) -> I16Vector4: ...
    @property
    def brrb(self) -> I16Vector4: ...
    @property
    def brra(self) -> I16Vector4: ...
    @property
    def brro(self) -> I16Vector4: ...
    @property
    def brrl(self) -> I16Vector4: ...
    @property
    def brgr(self) -> I16Vector4: ...
    @property
    def brgg(self) -> I16Vector4: ...
    @property
    def brgb(self) -> I16Vector4: ...
    @property
    def brga(self) -> I16Vector4: ...
    @property
    def brgo(self) -> I16Vector4: ...
    @property
    def brgl(self) -> I16Vector4: ...
    @property
    def brbr(self) -> I16Vector4: ...
    @property
    def brbg(self) -> I16Vector4: ...
    @property
    def brbb(self) -> I16Vector4: ...
    @property
    def brba(self) -> I16Vector4: ...
    @property
    def brbo(self) -> I16Vector4: ...
    @property
    def brbl(self) -> I16Vector4: ...
    @property
    def brar(self) -> I16Vector4: ...
    @property
    def brag(self) -> I16Vector4: ...
    @property
    def brab(self) -> I16Vector4: ...
    @property
    def braa(self) -> I16Vector4: ...
    @property
    def brao(self) -> I16Vector4: ...
    @property
    def bral(self) -> I16Vector4: ...
    @property
    def bror(self) -> I16Vector4: ...
    @property
    def brog(self) -> I16Vector4: ...
    @property
    def brob(self) -> I16Vector4: ...
    @property
    def broa(self) -> I16Vector4: ...
    @property
    def broo(self) -> I16Vector4: ...
    @property
    def brol(self) -> I16Vector4: ...
    @property
    def brlr(self) -> I16Vector4: ...
    @property
    def brlg(self) -> I16Vector4: ...
    @property
    def brlb(self) -> I16Vector4: ...
    @property
    def brla(self) -> I16Vector4: ...
    @property
    def brlo(self) -> I16Vector4: ...
    @property
    def brll(self) -> I16Vector4: ...
    @property
    def bgrr(self) -> I16Vector4: ...
    @property
    def bgrg(self) -> I16Vector4: ...
    @property
    def bgrb(self) -> I16Vector4: ...
    @property
    def bgra(self) -> I16Vector4: ...
    @property
    def bgro(self) -> I16Vector4: ...
    @property
    def bgrl(self) -> I16Vector4: ...
    @property
    def bggr(self) -> I16Vector4: ...
    @property
    def bggg(self) -> I16Vector4: ...
    @property
    def bggb(self) -> I16Vector4: ...
    @property
    def bgga(self) -> I16Vector4: ...
    @property
    def bggo(self) -> I16Vector4: ...
    @property
    def bggl(self) -> I16Vector4: ...
    @property
    def bgbr(self) -> I16Vector4: ...
    @property
    def bgbg(self) -> I16Vector4: ...
    @property
    def bgbb(self) -> I16Vector4: ...
    @property
    def bgba(self) -> I16Vector4: ...
    @property
    def bgbo(self) -> I16Vector4: ...
    @property
    def bgbl(self) -> I16Vector4: ...
    @property
    def bgar(self) -> I16Vector4: ...
    @property
    def bgag(self) -> I16Vector4: ...
    @property
    def bgab(self) -> I16Vector4: ...
    @property
    def bgaa(self) -> I16Vector4: ...
    @property
    def bgao(self) -> I16Vector4: ...
    @property
    def bgal(self) -> I16Vector4: ...
    @property
    def bgor(self) -> I16Vector4: ...
    @property
    def bgog(self) -> I16Vector4: ...
    @property
    def bgob(self) -> I16Vector4: ...
    @property
    def bgoa(self) -> I16Vector4: ...
    @property
    def bgoo(self) -> I16Vector4: ...
    @property
    def bgol(self) -> I16Vector4: ...
    @property
    def bglr(self) -> I16Vector4: ...
    @property
    def bglg(self) -> I16Vector4: ...
    @property
    def bglb(self) -> I16Vector4: ...
    @property
    def bgla(self) -> I16Vector4: ...
    @property
    def bglo(self) -> I16Vector4: ...
    @property
    def bgll(self) -> I16Vector4: ...
    @property
    def bbrr(self) -> I16Vector4: ...
    @property
    def bbrg(self) -> I16Vector4: ...
    @property
    def bbrb(self) -> I16Vector4: ...
    @property
    def bbra(self) -> I16Vector4: ...
    @property
    def bbro(self) -> I16Vector4: ...
    @property
    def bbrl(self) -> I16Vector4: ...
    @property
    def bbgr(self) -> I16Vector4: ...
    @property
    def bbgg(self) -> I16Vector4: ...
    @property
    def bbgb(self) -> I16Vector4: ...
    @property
    def bbga(self) -> I16Vector4: ...
    @property
    def bbgo(self) -> I16Vector4: ...
    @property
    def bbgl(self) -> I16Vector4: ...
    @property
    def bbbr(self) -> I16Vector4: ...
    @property
    def bbbg(self) -> I16Vector4: ...
    @property
    def bbbb(self) -> I16Vector4: ...
    @property
    def bbba(self) -> I16Vector4: ...
    @property
    def bbbo(self) -> I16Vector4: ...
    @property
    def bbbl(self) -> I16Vector4: ...
    @property
    def bbar(self) -> I16Vector4: ...
    @property
    def bbag(self) -> I16Vector4: ...
    @property
    def bbab(self) -> I16Vector4: ...
    @property
    def bbaa(self) -> I16Vector4: ...
    @property
    def bbao(self) -> I16Vector4: ...
    @property
    def bbal(self) -> I16Vector4: ...
    @property
    def bbor(self) -> I16Vector4: ...
    @property
    def bbog(self) -> I16Vector4: ...
    @property
    def bbob(self) -> I16Vector4: ...
    @property
    def bboa(self) -> I16Vector4: ...
    @property
    def bboo(self) -> I16Vector4: ...
    @property
    def bbol(self) -> I16Vector4: ...
    @property
    def bblr(self) -> I16Vector4: ...
    @property
    def bblg(self) -> I16Vector4: ...
    @property
    def bblb(self) -> I16Vector4: ...
    @property
    def bbla(self) -> I16Vector4: ...
    @property
    def bblo(self) -> I16Vector4: ...
    @property
    def bbll(self) -> I16Vector4: ...
    @property
    def barr(self) -> I16Vector4: ...
    @property
    def barg(self) -> I16Vector4: ...
    @property
    def barb(self) -> I16Vector4: ...
    @property
    def bara(self) -> I16Vector4: ...
    @property
    def baro(self) -> I16Vector4: ...
    @property
    def barl(self) -> I16Vector4: ...
    @property
    def bagr(self) -> I16Vector4: ...
    @property
    def bagg(self) -> I16Vector4: ...
    @property
    def bagb(self) -> I16Vector4: ...
    @property
    def baga(self) -> I16Vector4: ...
    @property
    def bago(self) -> I16Vector4: ...
    @property
    def bagl(self) -> I16Vector4: ...
    @property
    def babr(self) -> I16Vector4: ...
    @property
    def babg(self) -> I16Vector4: ...
    @property
    def babb(self) -> I16Vector4: ...
    @property
    def baba(self) -> I16Vector4: ...
    @property
    def babo(self) -> I16Vector4: ...
    @property
    def babl(self) -> I16Vector4: ...
    @property
    def baar(self) -> I16Vector4: ...
    @property
    def baag(self) -> I16Vector4: ...
    @property
    def baab(self) -> I16Vector4: ...
    @property
    def baaa(self) -> I16Vector4: ...
    @property
    def baao(self) -> I16Vector4: ...
    @property
    def baal(self) -> I16Vector4: ...
    @property
    def baor(self) -> I16Vector4: ...
    @property
    def baog(self) -> I16Vector4: ...
    @property
    def baob(self) -> I16Vector4: ...
    @property
    def baoa(self) -> I16Vector4: ...
    @property
    def baoo(self) -> I16Vector4: ...
    @property
    def baol(self) -> I16Vector4: ...
    @property
    def balr(self) -> I16Vector4: ...
    @property
    def balg(self) -> I16Vector4: ...
    @property
    def balb(self) -> I16Vector4: ...
    @property
    def bala(self) -> I16Vector4: ...
    @property
    def balo(self) -> I16Vector4: ...
    @property
    def ball(self) -> I16Vector4: ...
    @property
    def borr(self) -> I16Vector4: ...
    @property
    def borg(self) -> I16Vector4: ...
    @property
    def borb(self) -> I16Vector4: ...
    @property
    def bora(self) -> I16Vector4: ...
    @property
    def boro(self) -> I16Vector4: ...
    @property
    def borl(self) -> I16Vector4: ...
    @property
    def bogr(self) -> I16Vector4: ...
    @property
    def bogg(self) -> I16Vector4: ...
    @property
    def bogb(self) -> I16Vector4: ...
    @property
    def boga(self) -> I16Vector4: ...
    @property
    def bogo(self) -> I16Vector4: ...
    @property
    def bogl(self) -> I16Vector4: ...
    @property
    def bobr(self) -> I16Vector4: ...
    @property
    def bobg(self) -> I16Vector4: ...
    @property
    def bobb(self) -> I16Vector4: ...
    @property
    def boba(self) -> I16Vector4: ...
    @property
    def bobo(self) -> I16Vector4: ...
    @property
    def bobl(self) -> I16Vector4: ...
    @property
    def boar(self) -> I16Vector4: ...
    @property
    def boag(self) -> I16Vector4: ...
    @property
    def boab(self) -> I16Vector4: ...
    @property
    def boaa(self) -> I16Vector4: ...
    @property
    def boao(self) -> I16Vector4: ...
    @property
    def boal(self) -> I16Vector4: ...
    @property
    def boor(self) -> I16Vector4: ...
    @property
    def boog(self) -> I16Vector4: ...
    @property
    def boob(self) -> I16Vector4: ...
    @property
    def booa(self) -> I16Vector4: ...
    @property
    def booo(self) -> I16Vector4: ...
    @property
    def bool(self) -> I16Vector4: ...
    @property
    def bolr(self) -> I16Vector4: ...
    @property
    def bolg(self) -> I16Vector4: ...
    @property
    def bolb(self) -> I16Vector4: ...
    @property
    def bola(self) -> I16Vector4: ...
    @property
    def bolo(self) -> I16Vector4: ...
    @property
    def boll(self) -> I16Vector4: ...
    @property
    def blrr(self) -> I16Vector4: ...
    @property
    def blrg(self) -> I16Vector4: ...
    @property
    def blrb(self) -> I16Vector4: ...
    @property
    def blra(self) -> I16Vector4: ...
    @property
    def blro(self) -> I16Vector4: ...
    @property
    def blrl(self) -> I16Vector4: ...
    @property
    def blgr(self) -> I16Vector4: ...
    @property
    def blgg(self) -> I16Vector4: ...
    @property
    def blgb(self) -> I16Vector4: ...
    @property
    def blga(self) -> I16Vector4: ...
    @property
    def blgo(self) -> I16Vector4: ...
    @property
    def blgl(self) -> I16Vector4: ...
    @property
    def blbr(self) -> I16Vector4: ...
    @property
    def blbg(self) -> I16Vector4: ...
    @property
    def blbb(self) -> I16Vector4: ...
    @property
    def blba(self) -> I16Vector4: ...
    @property
    def blbo(self) -> I16Vector4: ...
    @property
    def blbl(self) -> I16Vector4: ...
    @property
    def blar(self) -> I16Vector4: ...
    @property
    def blag(self) -> I16Vector4: ...
    @property
    def blab(self) -> I16Vector4: ...
    @property
    def blaa(self) -> I16Vector4: ...
    @property
    def blao(self) -> I16Vector4: ...
    @property
    def blal(self) -> I16Vector4: ...
    @property
    def blor(self) -> I16Vector4: ...
    @property
    def blog(self) -> I16Vector4: ...
    @property
    def blob(self) -> I16Vector4: ...
    @property
    def bloa(self) -> I16Vector4: ...
    @property
    def bloo(self) -> I16Vector4: ...
    @property
    def blol(self) -> I16Vector4: ...
    @property
    def bllr(self) -> I16Vector4: ...
    @property
    def bllg(self) -> I16Vector4: ...
    @property
    def bllb(self) -> I16Vector4: ...
    @property
    def blla(self) -> I16Vector4: ...
    @property
    def bllo(self) -> I16Vector4: ...
    @property
    def blll(self) -> I16Vector4: ...
    @property
    def arrr(self) -> I16Vector4: ...
    @property
    def arrg(self) -> I16Vector4: ...
    @property
    def arrb(self) -> I16Vector4: ...
    @property
    def arra(self) -> I16Vector4: ...
    @property
    def arro(self) -> I16Vector4: ...
    @property
    def arrl(self) -> I16Vector4: ...
    @property
    def argr(self) -> I16Vector4: ...
    @property
    def argg(self) -> I16Vector4: ...
    @property
    def argb(self) -> I16Vector4: ...
    @property
    def arga(self) -> I16Vector4: ...
    @property
    def argo(self) -> I16Vector4: ...
    @property
    def argl(self) -> I16Vector4: ...
    @property
    def arbr(self) -> I16Vector4: ...
    @property
    def arbg(self) -> I16Vector4: ...
    @property
    def arbb(self) -> I16Vector4: ...
    @property
    def arba(self) -> I16Vector4: ...
    @property
    def arbo(self) -> I16Vector4: ...
    @property
    def arbl(self) -> I16Vector4: ...
    @property
    def arar(self) -> I16Vector4: ...
    @property
    def arag(self) -> I16Vector4: ...
    @property
    def arab(self) -> I16Vector4: ...
    @property
    def araa(self) -> I16Vector4: ...
    @property
    def arao(self) -> I16Vector4: ...
    @property
    def aral(self) -> I16Vector4: ...
    @property
    def aror(self) -> I16Vector4: ...
    @property
    def arog(self) -> I16Vector4: ...
    @property
    def arob(self) -> I16Vector4: ...
    @property
    def aroa(self) -> I16Vector4: ...
    @property
    def aroo(self) -> I16Vector4: ...
    @property
    def arol(self) -> I16Vector4: ...
    @property
    def arlr(self) -> I16Vector4: ...
    @property
    def arlg(self) -> I16Vector4: ...
    @property
    def arlb(self) -> I16Vector4: ...
    @property
    def arla(self) -> I16Vector4: ...
    @property
    def arlo(self) -> I16Vector4: ...
    @property
    def arll(self) -> I16Vector4: ...
    @property
    def agrr(self) -> I16Vector4: ...
    @property
    def agrg(self) -> I16Vector4: ...
    @property
    def agrb(self) -> I16Vector4: ...
    @property
    def agra(self) -> I16Vector4: ...
    @property
    def agro(self) -> I16Vector4: ...
    @property
    def agrl(self) -> I16Vector4: ...
    @property
    def aggr(self) -> I16Vector4: ...
    @property
    def aggg(self) -> I16Vector4: ...
    @property
    def aggb(self) -> I16Vector4: ...
    @property
    def agga(self) -> I16Vector4: ...
    @property
    def aggo(self) -> I16Vector4: ...
    @property
    def aggl(self) -> I16Vector4: ...
    @property
    def agbr(self) -> I16Vector4: ...
    @property
    def agbg(self) -> I16Vector4: ...
    @property
    def agbb(self) -> I16Vector4: ...
    @property
    def agba(self) -> I16Vector4: ...
    @property
    def agbo(self) -> I16Vector4: ...
    @property
    def agbl(self) -> I16Vector4: ...
    @property
    def agar(self) -> I16Vector4: ...
    @property
    def agag(self) -> I16Vector4: ...
    @property
    def agab(self) -> I16Vector4: ...
    @property
    def agaa(self) -> I16Vector4: ...
    @property
    def agao(self) -> I16Vector4: ...
    @property
    def agal(self) -> I16Vector4: ...
    @property
    def agor(self) -> I16Vector4: ...
    @property
    def agog(self) -> I16Vector4: ...
    @property
    def agob(self) -> I16Vector4: ...
    @property
    def agoa(self) -> I16Vector4: ...
    @property
    def agoo(self) -> I16Vector4: ...
    @property
    def agol(self) -> I16Vector4: ...
    @property
    def aglr(self) -> I16Vector4: ...
    @property
    def aglg(self) -> I16Vector4: ...
    @property
    def aglb(self) -> I16Vector4: ...
    @property
    def agla(self) -> I16Vector4: ...
    @property
    def aglo(self) -> I16Vector4: ...
    @property
    def agll(self) -> I16Vector4: ...
    @property
    def abrr(self) -> I16Vector4: ...
    @property
    def abrg(self) -> I16Vector4: ...
    @property
    def abrb(self) -> I16Vector4: ...
    @property
    def abra(self) -> I16Vector4: ...
    @property
    def abro(self) -> I16Vector4: ...
    @property
    def abrl(self) -> I16Vector4: ...
    @property
    def abgr(self) -> I16Vector4: ...
    @property
    def abgg(self) -> I16Vector4: ...
    @property
    def abgb(self) -> I16Vector4: ...
    @property
    def abga(self) -> I16Vector4: ...
    @property
    def abgo(self) -> I16Vector4: ...
    @property
    def abgl(self) -> I16Vector4: ...
    @property
    def abbr(self) -> I16Vector4: ...
    @property
    def abbg(self) -> I16Vector4: ...
    @property
    def abbb(self) -> I16Vector4: ...
    @property
    def abba(self) -> I16Vector4: ...
    @property
    def abbo(self) -> I16Vector4: ...
    @property
    def abbl(self) -> I16Vector4: ...
    @property
    def abar(self) -> I16Vector4: ...
    @property
    def abag(self) -> I16Vector4: ...
    @property
    def abab(self) -> I16Vector4: ...
    @property
    def abaa(self) -> I16Vector4: ...
    @property
    def abao(self) -> I16Vector4: ...
    @property
    def abal(self) -> I16Vector4: ...
    @property
    def abor(self) -> I16Vector4: ...
    @property
    def abog(self) -> I16Vector4: ...
    @property
    def abob(self) -> I16Vector4: ...
    @property
    def aboa(self) -> I16Vector4: ...
    @property
    def aboo(self) -> I16Vector4: ...
    @property
    def abol(self) -> I16Vector4: ...
    @property
    def ablr(self) -> I16Vector4: ...
    @property
    def ablg(self) -> I16Vector4: ...
    @property
    def ablb(self) -> I16Vector4: ...
    @property
    def abla(self) -> I16Vector4: ...
    @property
    def ablo(self) -> I16Vector4: ...
    @property
    def abll(self) -> I16Vector4: ...
    @property
    def aarr(self) -> I16Vector4: ...
    @property
    def aarg(self) -> I16Vector4: ...
    @property
    def aarb(self) -> I16Vector4: ...
    @property
    def aara(self) -> I16Vector4: ...
    @property
    def aaro(self) -> I16Vector4: ...
    @property
    def aarl(self) -> I16Vector4: ...
    @property
    def aagr(self) -> I16Vector4: ...
    @property
    def aagg(self) -> I16Vector4: ...
    @property
    def aagb(self) -> I16Vector4: ...
    @property
    def aaga(self) -> I16Vector4: ...
    @property
    def aago(self) -> I16Vector4: ...
    @property
    def aagl(self) -> I16Vector4: ...
    @property
    def aabr(self) -> I16Vector4: ...
    @property
    def aabg(self) -> I16Vector4: ...
    @property
    def aabb(self) -> I16Vector4: ...
    @property
    def aaba(self) -> I16Vector4: ...
    @property
    def aabo(self) -> I16Vector4: ...
    @property
    def aabl(self) -> I16Vector4: ...
    @property
    def aaar(self) -> I16Vector4: ...
    @property
    def aaag(self) -> I16Vector4: ...
    @property
    def aaab(self) -> I16Vector4: ...
    @property
    def aaaa(self) -> I16Vector4: ...
    @property
    def aaao(self) -> I16Vector4: ...
    @property
    def aaal(self) -> I16Vector4: ...
    @property
    def aaor(self) -> I16Vector4: ...
    @property
    def aaog(self) -> I16Vector4: ...
    @property
    def aaob(self) -> I16Vector4: ...
    @property
    def aaoa(self) -> I16Vector4: ...
    @property
    def aaoo(self) -> I16Vector4: ...
    @property
    def aaol(self) -> I16Vector4: ...
    @property
    def aalr(self) -> I16Vector4: ...
    @property
    def aalg(self) -> I16Vector4: ...
    @property
    def aalb(self) -> I16Vector4: ...
    @property
    def aala(self) -> I16Vector4: ...
    @property
    def aalo(self) -> I16Vector4: ...
    @property
    def aall(self) -> I16Vector4: ...
    @property
    def aorr(self) -> I16Vector4: ...
    @property
    def aorg(self) -> I16Vector4: ...
    @property
    def aorb(self) -> I16Vector4: ...
    @property
    def aora(self) -> I16Vector4: ...
    @property
    def aoro(self) -> I16Vector4: ...
    @property
    def aorl(self) -> I16Vector4: ...
    @property
    def aogr(self) -> I16Vector4: ...
    @property
    def aogg(self) -> I16Vector4: ...
    @property
    def aogb(self) -> I16Vector4: ...
    @property
    def aoga(self) -> I16Vector4: ...
    @property
    def aogo(self) -> I16Vector4: ...
    @property
    def aogl(self) -> I16Vector4: ...
    @property
    def aobr(self) -> I16Vector4: ...
    @property
    def aobg(self) -> I16Vector4: ...
    @property
    def aobb(self) -> I16Vector4: ...
    @property
    def aoba(self) -> I16Vector4: ...
    @property
    def aobo(self) -> I16Vector4: ...
    @property
    def aobl(self) -> I16Vector4: ...
    @property
    def aoar(self) -> I16Vector4: ...
    @property
    def aoag(self) -> I16Vector4: ...
    @property
    def aoab(self) -> I16Vector4: ...
    @property
    def aoaa(self) -> I16Vector4: ...
    @property
    def aoao(self) -> I16Vector4: ...
    @property
    def aoal(self) -> I16Vector4: ...
    @property
    def aoor(self) -> I16Vector4: ...
    @property
    def aoog(self) -> I16Vector4: ...
    @property
    def aoob(self) -> I16Vector4: ...
    @property
    def aooa(self) -> I16Vector4: ...
    @property
    def aooo(self) -> I16Vector4: ...
    @property
    def aool(self) -> I16Vector4: ...
    @property
    def aolr(self) -> I16Vector4: ...
    @property
    def aolg(self) -> I16Vector4: ...
    @property
    def aolb(self) -> I16Vector4: ...
    @property
    def aola(self) -> I16Vector4: ...
    @property
    def aolo(self) -> I16Vector4: ...
    @property
    def aoll(self) -> I16Vector4: ...
    @property
    def alrr(self) -> I16Vector4: ...
    @property
    def alrg(self) -> I16Vector4: ...
    @property
    def alrb(self) -> I16Vector4: ...
    @property
    def alra(self) -> I16Vector4: ...
    @property
    def alro(self) -> I16Vector4: ...
    @property
    def alrl(self) -> I16Vector4: ...
    @property
    def algr(self) -> I16Vector4: ...
    @property
    def algg(self) -> I16Vector4: ...
    @property
    def algb(self) -> I16Vector4: ...
    @property
    def alga(self) -> I16Vector4: ...
    @property
    def algo(self) -> I16Vector4: ...
    @property
    def algl(self) -> I16Vector4: ...
    @property
    def albr(self) -> I16Vector4: ...
    @property
    def albg(self) -> I16Vector4: ...
    @property
    def albb(self) -> I16Vector4: ...
    @property
    def alba(self) -> I16Vector4: ...
    @property
    def albo(self) -> I16Vector4: ...
    @property
    def albl(self) -> I16Vector4: ...
    @property
    def alar(self) -> I16Vector4: ...
    @property
    def alag(self) -> I16Vector4: ...
    @property
    def alab(self) -> I16Vector4: ...
    @property
    def alaa(self) -> I16Vector4: ...
    @property
    def alao(self) -> I16Vector4: ...
    @property
    def alal(self) -> I16Vector4: ...
    @property
    def alor(self) -> I16Vector4: ...
    @property
    def alog(self) -> I16Vector4: ...
    @property
    def alob(self) -> I16Vector4: ...
    @property
    def aloa(self) -> I16Vector4: ...
    @property
    def aloo(self) -> I16Vector4: ...
    @property
    def alol(self) -> I16Vector4: ...
    @property
    def allr(self) -> I16Vector4: ...
    @property
    def allg(self) -> I16Vector4: ...
    @property
    def allb(self) -> I16Vector4: ...
    @property
    def alla(self) -> I16Vector4: ...
    @property
    def allo(self) -> I16Vector4: ...
    @property
    def alll(self) -> I16Vector4: ...
    @property
    def orrr(self) -> I16Vector4: ...
    @property
    def orrg(self) -> I16Vector4: ...
    @property
    def orrb(self) -> I16Vector4: ...
    @property
    def orra(self) -> I16Vector4: ...
    @property
    def orro(self) -> I16Vector4: ...
    @property
    def orrl(self) -> I16Vector4: ...
    @property
    def orgr(self) -> I16Vector4: ...
    @property
    def orgg(self) -> I16Vector4: ...
    @property
    def orgb(self) -> I16Vector4: ...
    @property
    def orga(self) -> I16Vector4: ...
    @property
    def orgo(self) -> I16Vector4: ...
    @property
    def orgl(self) -> I16Vector4: ...
    @property
    def orbr(self) -> I16Vector4: ...
    @property
    def orbg(self) -> I16Vector4: ...
    @property
    def orbb(self) -> I16Vector4: ...
    @property
    def orba(self) -> I16Vector4: ...
    @property
    def orbo(self) -> I16Vector4: ...
    @property
    def orbl(self) -> I16Vector4: ...
    @property
    def orar(self) -> I16Vector4: ...
    @property
    def orag(self) -> I16Vector4: ...
    @property
    def orab(self) -> I16Vector4: ...
    @property
    def oraa(self) -> I16Vector4: ...
    @property
    def orao(self) -> I16Vector4: ...
    @property
    def oral(self) -> I16Vector4: ...
    @property
    def oror(self) -> I16Vector4: ...
    @property
    def orog(self) -> I16Vector4: ...
    @property
    def orob(self) -> I16Vector4: ...
    @property
    def oroa(self) -> I16Vector4: ...
    @property
    def oroo(self) -> I16Vector4: ...
    @property
    def orol(self) -> I16Vector4: ...
    @property
    def orlr(self) -> I16Vector4: ...
    @property
    def orlg(self) -> I16Vector4: ...
    @property
    def orlb(self) -> I16Vector4: ...
    @property
    def orla(self) -> I16Vector4: ...
    @property
    def orlo(self) -> I16Vector4: ...
    @property
    def orll(self) -> I16Vector4: ...
    @property
    def ogrr(self) -> I16Vector4: ...
    @property
    def ogrg(self) -> I16Vector4: ...
    @property
    def ogrb(self) -> I16Vector4: ...
    @property
    def ogra(self) -> I16Vector4: ...
    @property
    def ogro(self) -> I16Vector4: ...
    @property
    def ogrl(self) -> I16Vector4: ...
    @property
    def oggr(self) -> I16Vector4: ...
    @property
    def oggg(self) -> I16Vector4: ...
    @property
    def oggb(self) -> I16Vector4: ...
    @property
    def ogga(self) -> I16Vector4: ...
    @property
    def oggo(self) -> I16Vector4: ...
    @property
    def oggl(self) -> I16Vector4: ...
    @property
    def ogbr(self) -> I16Vector4: ...
    @property
    def ogbg(self) -> I16Vector4: ...
    @property
    def ogbb(self) -> I16Vector4: ...
    @property
    def ogba(self) -> I16Vector4: ...
    @property
    def ogbo(self) -> I16Vector4: ...
    @property
    def ogbl(self) -> I16Vector4: ...
    @property
    def ogar(self) -> I16Vector4: ...
    @property
    def ogag(self) -> I16Vector4: ...
    @property
    def ogab(self) -> I16Vector4: ...
    @property
    def ogaa(self) -> I16Vector4: ...
    @property
    def ogao(self) -> I16Vector4: ...
    @property
    def ogal(self) -> I16Vector4: ...
    @property
    def ogor(self) -> I16Vector4: ...
    @property
    def ogog(self) -> I16Vector4: ...
    @property
    def ogob(self) -> I16Vector4: ...
    @property
    def ogoa(self) -> I16Vector4: ...
    @property
    def ogoo(self) -> I16Vector4: ...
    @property
    def ogol(self) -> I16Vector4: ...
    @property
    def oglr(self) -> I16Vector4: ...
    @property
    def oglg(self) -> I16Vector4: ...
    @property
    def oglb(self) -> I16Vector4: ...
    @property
    def ogla(self) -> I16Vector4: ...
    @property
    def oglo(self) -> I16Vector4: ...
    @property
    def ogll(self) -> I16Vector4: ...
    @property
    def obrr(self) -> I16Vector4: ...
    @property
    def obrg(self) -> I16Vector4: ...
    @property
    def obrb(self) -> I16Vector4: ...
    @property
    def obra(self) -> I16Vector4: ...
    @property
    def obro(self) -> I16Vector4: ...
    @property
    def obrl(self) -> I16Vector4: ...
    @property
    def obgr(self) -> I16Vector4: ...
    @property
    def obgg(self) -> I16Vector4: ...
    @property
    def obgb(self) -> I16Vector4: ...
    @property
    def obga(self) -> I16Vector4: ...
    @property
    def obgo(self) -> I16Vector4: ...
    @property
    def obgl(self) -> I16Vector4: ...
    @property
    def obbr(self) -> I16Vector4: ...
    @property
    def obbg(self) -> I16Vector4: ...
    @property
    def obbb(self) -> I16Vector4: ...
    @property
    def obba(self) -> I16Vector4: ...
    @property
    def obbo(self) -> I16Vector4: ...
    @property
    def obbl(self) -> I16Vector4: ...
    @property
    def obar(self) -> I16Vector4: ...
    @property
    def obag(self) -> I16Vector4: ...
    @property
    def obab(self) -> I16Vector4: ...
    @property
    def obaa(self) -> I16Vector4: ...
    @property
    def obao(self) -> I16Vector4: ...
    @property
    def obal(self) -> I16Vector4: ...
    @property
    def obor(self) -> I16Vector4: ...
    @property
    def obog(self) -> I16Vector4: ...
    @property
    def obob(self) -> I16Vector4: ...
    @property
    def oboa(self) -> I16Vector4: ...
    @property
    def oboo(self) -> I16Vector4: ...
    @property
    def obol(self) -> I16Vector4: ...
    @property
    def oblr(self) -> I16Vector4: ...
    @property
    def oblg(self) -> I16Vector4: ...
    @property
    def oblb(self) -> I16Vector4: ...
    @property
    def obla(self) -> I16Vector4: ...
    @property
    def oblo(self) -> I16Vector4: ...
    @property
    def obll(self) -> I16Vector4: ...
    @property
    def oarr(self) -> I16Vector4: ...
    @property
    def oarg(self) -> I16Vector4: ...
    @property
    def oarb(self) -> I16Vector4: ...
    @property
    def oara(self) -> I16Vector4: ...
    @property
    def oaro(self) -> I16Vector4: ...
    @property
    def oarl(self) -> I16Vector4: ...
    @property
    def oagr(self) -> I16Vector4: ...
    @property
    def oagg(self) -> I16Vector4: ...
    @property
    def oagb(self) -> I16Vector4: ...
    @property
    def oaga(self) -> I16Vector4: ...
    @property
    def oago(self) -> I16Vector4: ...
    @property
    def oagl(self) -> I16Vector4: ...
    @property
    def oabr(self) -> I16Vector4: ...
    @property
    def oabg(self) -> I16Vector4: ...
    @property
    def oabb(self) -> I16Vector4: ...
    @property
    def oaba(self) -> I16Vector4: ...
    @property
    def oabo(self) -> I16Vector4: ...
    @property
    def oabl(self) -> I16Vector4: ...
    @property
    def oaar(self) -> I16Vector4: ...
    @property
    def oaag(self) -> I16Vector4: ...
    @property
    def oaab(self) -> I16Vector4: ...
    @property
    def oaaa(self) -> I16Vector4: ...
    @property
    def oaao(self) -> I16Vector4: ...
    @property
    def oaal(self) -> I16Vector4: ...
    @property
    def oaor(self) -> I16Vector4: ...
    @property
    def oaog(self) -> I16Vector4: ...
    @property
    def oaob(self) -> I16Vector4: ...
    @property
    def oaoa(self) -> I16Vector4: ...
    @property
    def oaoo(self) -> I16Vector4: ...
    @property
    def oaol(self) -> I16Vector4: ...
    @property
    def oalr(self) -> I16Vector4: ...
    @property
    def oalg(self) -> I16Vector4: ...
    @property
    def oalb(self) -> I16Vector4: ...
    @property
    def oala(self) -> I16Vector4: ...
    @property
    def oalo(self) -> I16Vector4: ...
    @property
    def oall(self) -> I16Vector4: ...
    @property
    def oorr(self) -> I16Vector4: ...
    @property
    def oorg(self) -> I16Vector4: ...
    @property
    def oorb(self) -> I16Vector4: ...
    @property
    def oora(self) -> I16Vector4: ...
    @property
    def ooro(self) -> I16Vector4: ...
    @property
    def oorl(self) -> I16Vector4: ...
    @property
    def oogr(self) -> I16Vector4: ...
    @property
    def oogg(self) -> I16Vector4: ...
    @property
    def oogb(self) -> I16Vector4: ...
    @property
    def ooga(self) -> I16Vector4: ...
    @property
    def oogo(self) -> I16Vector4: ...
    @property
    def oogl(self) -> I16Vector4: ...
    @property
    def oobr(self) -> I16Vector4: ...
    @property
    def oobg(self) -> I16Vector4: ...
    @property
    def oobb(self) -> I16Vector4: ...
    @property
    def ooba(self) -> I16Vector4: ...
    @property
    def oobo(self) -> I16Vector4: ...
    @property
    def oobl(self) -> I16Vector4: ...
    @property
    def ooar(self) -> I16Vector4: ...
    @property
    def ooag(self) -> I16Vector4: ...
    @property
    def ooab(self) -> I16Vector4: ...
    @property
    def ooaa(self) -> I16Vector4: ...
    @property
    def ooao(self) -> I16Vector4: ...
    @property
    def ooal(self) -> I16Vector4: ...
    @property
    def ooor(self) -> I16Vector4: ...
    @property
    def ooog(self) -> I16Vector4: ...
    @property
    def ooob(self) -> I16Vector4: ...
    @property
    def oooa(self) -> I16Vector4: ...
    @property
    def oolr(self) -> I16Vector4: ...
    @property
    def oolg(self) -> I16Vector4: ...
    @property
    def oolb(self) -> I16Vector4: ...
    @property
    def oola(self) -> I16Vector4: ...
    @property
    def olrr(self) -> I16Vector4: ...
    @property
    def olrg(self) -> I16Vector4: ...
    @property
    def olrb(self) -> I16Vector4: ...
    @property
    def olra(self) -> I16Vector4: ...
    @property
    def olro(self) -> I16Vector4: ...
    @property
    def olrl(self) -> I16Vector4: ...
    @property
    def olgr(self) -> I16Vector4: ...
    @property
    def olgg(self) -> I16Vector4: ...
    @property
    def olgb(self) -> I16Vector4: ...
    @property
    def olga(self) -> I16Vector4: ...
    @property
    def olgo(self) -> I16Vector4: ...
    @property
    def olgl(self) -> I16Vector4: ...
    @property
    def olbr(self) -> I16Vector4: ...
    @property
    def olbg(self) -> I16Vector4: ...
    @property
    def olbb(self) -> I16Vector4: ...
    @property
    def olba(self) -> I16Vector4: ...
    @property
    def olbo(self) -> I16Vector4: ...
    @property
    def olbl(self) -> I16Vector4: ...
    @property
    def olar(self) -> I16Vector4: ...
    @property
    def olag(self) -> I16Vector4: ...
    @property
    def olab(self) -> I16Vector4: ...
    @property
    def olaa(self) -> I16Vector4: ...
    @property
    def olao(self) -> I16Vector4: ...
    @property
    def olal(self) -> I16Vector4: ...
    @property
    def olor(self) -> I16Vector4: ...
    @property
    def olog(self) -> I16Vector4: ...
    @property
    def olob(self) -> I16Vector4: ...
    @property
    def oloa(self) -> I16Vector4: ...
    @property
    def ollr(self) -> I16Vector4: ...
    @property
    def ollg(self) -> I16Vector4: ...
    @property
    def ollb(self) -> I16Vector4: ...
    @property
    def olla(self) -> I16Vector4: ...
    @property
    def lrrr(self) -> I16Vector4: ...
    @property
    def lrrg(self) -> I16Vector4: ...
    @property
    def lrrb(self) -> I16Vector4: ...
    @property
    def lrra(self) -> I16Vector4: ...
    @property
    def lrro(self) -> I16Vector4: ...
    @property
    def lrrl(self) -> I16Vector4: ...
    @property
    def lrgr(self) -> I16Vector4: ...
    @property
    def lrgg(self) -> I16Vector4: ...
    @property
    def lrgb(self) -> I16Vector4: ...
    @property
    def lrga(self) -> I16Vector4: ...
    @property
    def lrgo(self) -> I16Vector4: ...
    @property
    def lrgl(self) -> I16Vector4: ...
    @property
    def lrbr(self) -> I16Vector4: ...
    @property
    def lrbg(self) -> I16Vector4: ...
    @property
    def lrbb(self) -> I16Vector4: ...
    @property
    def lrba(self) -> I16Vector4: ...
    @property
    def lrbo(self) -> I16Vector4: ...
    @property
    def lrbl(self) -> I16Vector4: ...
    @property
    def lrar(self) -> I16Vector4: ...
    @property
    def lrag(self) -> I16Vector4: ...
    @property
    def lrab(self) -> I16Vector4: ...
    @property
    def lraa(self) -> I16Vector4: ...
    @property
    def lrao(self) -> I16Vector4: ...
    @property
    def lral(self) -> I16Vector4: ...
    @property
    def lror(self) -> I16Vector4: ...
    @property
    def lrog(self) -> I16Vector4: ...
    @property
    def lrob(self) -> I16Vector4: ...
    @property
    def lroa(self) -> I16Vector4: ...
    @property
    def lroo(self) -> I16Vector4: ...
    @property
    def lrol(self) -> I16Vector4: ...
    @property
    def lrlr(self) -> I16Vector4: ...
    @property
    def lrlg(self) -> I16Vector4: ...
    @property
    def lrlb(self) -> I16Vector4: ...
    @property
    def lrla(self) -> I16Vector4: ...
    @property
    def lrlo(self) -> I16Vector4: ...
    @property
    def lrll(self) -> I16Vector4: ...
    @property
    def lgrr(self) -> I16Vector4: ...
    @property
    def lgrg(self) -> I16Vector4: ...
    @property
    def lgrb(self) -> I16Vector4: ...
    @property
    def lgra(self) -> I16Vector4: ...
    @property
    def lgro(self) -> I16Vector4: ...
    @property
    def lgrl(self) -> I16Vector4: ...
    @property
    def lggr(self) -> I16Vector4: ...
    @property
    def lggg(self) -> I16Vector4: ...
    @property
    def lggb(self) -> I16Vector4: ...
    @property
    def lgga(self) -> I16Vector4: ...
    @property
    def lggo(self) -> I16Vector4: ...
    @property
    def lggl(self) -> I16Vector4: ...
    @property
    def lgbr(self) -> I16Vector4: ...
    @property
    def lgbg(self) -> I16Vector4: ...
    @property
    def lgbb(self) -> I16Vector4: ...
    @property
    def lgba(self) -> I16Vector4: ...
    @property
    def lgbo(self) -> I16Vector4: ...
    @property
    def lgbl(self) -> I16Vector4: ...
    @property
    def lgar(self) -> I16Vector4: ...
    @property
    def lgag(self) -> I16Vector4: ...
    @property
    def lgab(self) -> I16Vector4: ...
    @property
    def lgaa(self) -> I16Vector4: ...
    @property
    def lgao(self) -> I16Vector4: ...
    @property
    def lgal(self) -> I16Vector4: ...
    @property
    def lgor(self) -> I16Vector4: ...
    @property
    def lgog(self) -> I16Vector4: ...
    @property
    def lgob(self) -> I16Vector4: ...
    @property
    def lgoa(self) -> I16Vector4: ...
    @property
    def lgoo(self) -> I16Vector4: ...
    @property
    def lgol(self) -> I16Vector4: ...
    @property
    def lglr(self) -> I16Vector4: ...
    @property
    def lglg(self) -> I16Vector4: ...
    @property
    def lglb(self) -> I16Vector4: ...
    @property
    def lgla(self) -> I16Vector4: ...
    @property
    def lglo(self) -> I16Vector4: ...
    @property
    def lgll(self) -> I16Vector4: ...
    @property
    def lbrr(self) -> I16Vector4: ...
    @property
    def lbrg(self) -> I16Vector4: ...
    @property
    def lbrb(self) -> I16Vector4: ...
    @property
    def lbra(self) -> I16Vector4: ...
    @property
    def lbro(self) -> I16Vector4: ...
    @property
    def lbrl(self) -> I16Vector4: ...
    @property
    def lbgr(self) -> I16Vector4: ...
    @property
    def lbgg(self) -> I16Vector4: ...
    @property
    def lbgb(self) -> I16Vector4: ...
    @property
    def lbga(self) -> I16Vector4: ...
    @property
    def lbgo(self) -> I16Vector4: ...
    @property
    def lbgl(self) -> I16Vector4: ...
    @property
    def lbbr(self) -> I16Vector4: ...
    @property
    def lbbg(self) -> I16Vector4: ...
    @property
    def lbbb(self) -> I16Vector4: ...
    @property
    def lbba(self) -> I16Vector4: ...
    @property
    def lbbo(self) -> I16Vector4: ...
    @property
    def lbbl(self) -> I16Vector4: ...
    @property
    def lbar(self) -> I16Vector4: ...
    @property
    def lbag(self) -> I16Vector4: ...
    @property
    def lbab(self) -> I16Vector4: ...
    @property
    def lbaa(self) -> I16Vector4: ...
    @property
    def lbao(self) -> I16Vector4: ...
    @property
    def lbal(self) -> I16Vector4: ...
    @property
    def lbor(self) -> I16Vector4: ...
    @property
    def lbog(self) -> I16Vector4: ...
    @property
    def lbob(self) -> I16Vector4: ...
    @property
    def lboa(self) -> I16Vector4: ...
    @property
    def lboo(self) -> I16Vector4: ...
    @property
    def lbol(self) -> I16Vector4: ...
    @property
    def lblr(self) -> I16Vector4: ...
    @property
    def lblg(self) -> I16Vector4: ...
    @property
    def lblb(self) -> I16Vector4: ...
    @property
    def lbla(self) -> I16Vector4: ...
    @property
    def lblo(self) -> I16Vector4: ...
    @property
    def lbll(self) -> I16Vector4: ...
    @property
    def larr(self) -> I16Vector4: ...
    @property
    def larg(self) -> I16Vector4: ...
    @property
    def larb(self) -> I16Vector4: ...
    @property
    def lara(self) -> I16Vector4: ...
    @property
    def laro(self) -> I16Vector4: ...
    @property
    def larl(self) -> I16Vector4: ...
    @property
    def lagr(self) -> I16Vector4: ...
    @property
    def lagg(self) -> I16Vector4: ...
    @property
    def lagb(self) -> I16Vector4: ...
    @property
    def laga(self) -> I16Vector4: ...
    @property
    def lago(self) -> I16Vector4: ...
    @property
    def lagl(self) -> I16Vector4: ...
    @property
    def labr(self) -> I16Vector4: ...
    @property
    def labg(self) -> I16Vector4: ...
    @property
    def labb(self) -> I16Vector4: ...
    @property
    def laba(self) -> I16Vector4: ...
    @property
    def labo(self) -> I16Vector4: ...
    @property
    def labl(self) -> I16Vector4: ...
    @property
    def laar(self) -> I16Vector4: ...
    @property
    def laag(self) -> I16Vector4: ...
    @property
    def laab(self) -> I16Vector4: ...
    @property
    def laaa(self) -> I16Vector4: ...
    @property
    def laao(self) -> I16Vector4: ...
    @property
    def laal(self) -> I16Vector4: ...
    @property
    def laor(self) -> I16Vector4: ...
    @property
    def laog(self) -> I16Vector4: ...
    @property
    def laob(self) -> I16Vector4: ...
    @property
    def laoa(self) -> I16Vector4: ...
    @property
    def laoo(self) -> I16Vector4: ...
    @property
    def laol(self) -> I16Vector4: ...
    @property
    def lalr(self) -> I16Vector4: ...
    @property
    def lalg(self) -> I16Vector4: ...
    @property
    def lalb(self) -> I16Vector4: ...
    @property
    def lala(self) -> I16Vector4: ...
    @property
    def lalo(self) -> I16Vector4: ...
    @property
    def lall(self) -> I16Vector4: ...
    @property
    def lorr(self) -> I16Vector4: ...
    @property
    def lorg(self) -> I16Vector4: ...
    @property
    def lorb(self) -> I16Vector4: ...
    @property
    def lora(self) -> I16Vector4: ...
    @property
    def loro(self) -> I16Vector4: ...
    @property
    def lorl(self) -> I16Vector4: ...
    @property
    def logr(self) -> I16Vector4: ...
    @property
    def logg(self) -> I16Vector4: ...
    @property
    def logb(self) -> I16Vector4: ...
    @property
    def loga(self) -> I16Vector4: ...
    @property
    def logo(self) -> I16Vector4: ...
    @property
    def logl(self) -> I16Vector4: ...
    @property
    def lobr(self) -> I16Vector4: ...
    @property
    def lobg(self) -> I16Vector4: ...
    @property
    def lobb(self) -> I16Vector4: ...
    @property
    def loba(self) -> I16Vector4: ...
    @property
    def lobo(self) -> I16Vector4: ...
    @property
    def lobl(self) -> I16Vector4: ...
    @property
    def loar(self) -> I16Vector4: ...
    @property
    def loag(self) -> I16Vector4: ...
    @property
    def loab(self) -> I16Vector4: ...
    @property
    def loaa(self) -> I16Vector4: ...
    @property
    def loao(self) -> I16Vector4: ...
    @property
    def loal(self) -> I16Vector4: ...
    @property
    def loor(self) -> I16Vector4: ...
    @property
    def loog(self) -> I16Vector4: ...
    @property
    def loob(self) -> I16Vector4: ...
    @property
    def looa(self) -> I16Vector4: ...
    @property
    def lolr(self) -> I16Vector4: ...
    @property
    def lolg(self) -> I16Vector4: ...
    @property
    def lolb(self) -> I16Vector4: ...
    @property
    def lola(self) -> I16Vector4: ...
    @property
    def llrr(self) -> I16Vector4: ...
    @property
    def llrg(self) -> I16Vector4: ...
    @property
    def llrb(self) -> I16Vector4: ...
    @property
    def llra(self) -> I16Vector4: ...
    @property
    def llro(self) -> I16Vector4: ...
    @property
    def llrl(self) -> I16Vector4: ...
    @property
    def llgr(self) -> I16Vector4: ...
    @property
    def llgg(self) -> I16Vector4: ...
    @property
    def llgb(self) -> I16Vector4: ...
    @property
    def llga(self) -> I16Vector4: ...
    @property
    def llgo(self) -> I16Vector4: ...
    @property
    def llgl(self) -> I16Vector4: ...
    @property
    def llbr(self) -> I16Vector4: ...
    @property
    def llbg(self) -> I16Vector4: ...
    @property
    def llbb(self) -> I16Vector4: ...
    @property
    def llba(self) -> I16Vector4: ...
    @property
    def llbo(self) -> I16Vector4: ...
    @property
    def llbl(self) -> I16Vector4: ...
    @property
    def llar(self) -> I16Vector4: ...
    @property
    def llag(self) -> I16Vector4: ...
    @property
    def llab(self) -> I16Vector4: ...
    @property
    def llaa(self) -> I16Vector4: ...
    @property
    def llao(self) -> I16Vector4: ...
    @property
    def llal(self) -> I16Vector4: ...
    @property
    def llor(self) -> I16Vector4: ...
    @property
    def llog(self) -> I16Vector4: ...
    @property
    def llob(self) -> I16Vector4: ...
    @property
    def lloa(self) -> I16Vector4: ...
    @property
    def lllr(self) -> I16Vector4: ...
    @property
    def lllg(self) -> I16Vector4: ...
    @property
    def lllb(self) -> I16Vector4: ...
    @property
    def llla(self) -> I16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> I16Vector2: ...
    @property
    def st(self) -> I16Vector2: ...
    @property
    def sq(self) -> I16Vector2: ...
    @property
    def sp(self) -> I16Vector2: ...
    @property
    def so(self) -> I16Vector2: ...
    @property
    def sl(self) -> I16Vector2: ...
    @property
    def ts(self) -> I16Vector2: ...
    @property
    def tt(self) -> I16Vector2: ...
    @property
    def tq(self) -> I16Vector2: ...
    @property
    def tp(self) -> I16Vector2: ...
    @property
    def to(self) -> I16Vector2: ...
    @property
    def tl(self) -> I16Vector2: ...
    @property
    def qs(self) -> I16Vector2: ...
    @property
    def qt(self) -> I16Vector2: ...
    @property
    def qq(self) -> I16Vector2: ...
    @property
    def qp(self) -> I16Vector2: ...
    @property
    def qo(self) -> I16Vector2: ...
    @property
    def ql(self) -> I16Vector2: ...
    @property
    def ps(self) -> I16Vector2: ...
    @property
    def pt(self) -> I16Vector2: ...
    @property
    def pq(self) -> I16Vector2: ...
    @property
    def pp(self) -> I16Vector2: ...
    @property
    def po(self) -> I16Vector2: ...
    @property
    def pl(self) -> I16Vector2: ...
    @property
    def os(self) -> I16Vector2: ...
    @property
    def ot(self) -> I16Vector2: ...
    @property
    def oq(self) -> I16Vector2: ...
    @property
    def op(self) -> I16Vector2: ...
    @property
    def ls(self) -> I16Vector2: ...
    @property
    def lt(self) -> I16Vector2: ...
    @property
    def lq(self) -> I16Vector2: ...
    @property
    def lp(self) -> I16Vector2: ...
    @property
    def sss(self) -> I16Vector3: ...
    @property
    def sst(self) -> I16Vector3: ...
    @property
    def ssq(self) -> I16Vector3: ...
    @property
    def ssp(self) -> I16Vector3: ...
    @property
    def sso(self) -> I16Vector3: ...
    @property
    def ssl(self) -> I16Vector3: ...
    @property
    def sts(self) -> I16Vector3: ...
    @property
    def stt(self) -> I16Vector3: ...
    @property
    def stq(self) -> I16Vector3: ...
    @property
    def stp(self) -> I16Vector3: ...
    @property
    def sto(self) -> I16Vector3: ...
    @property
    def stl(self) -> I16Vector3: ...
    @property
    def sqs(self) -> I16Vector3: ...
    @property
    def sqt(self) -> I16Vector3: ...
    @property
    def sqq(self) -> I16Vector3: ...
    @property
    def sqp(self) -> I16Vector3: ...
    @property
    def sqo(self) -> I16Vector3: ...
    @property
    def sql(self) -> I16Vector3: ...
    @property
    def sps(self) -> I16Vector3: ...
    @property
    def spt(self) -> I16Vector3: ...
    @property
    def spq(self) -> I16Vector3: ...
    @property
    def spp(self) -> I16Vector3: ...
    @property
    def spo(self) -> I16Vector3: ...
    @property
    def spl(self) -> I16Vector3: ...
    @property
    def sos(self) -> I16Vector3: ...
    @property
    def sot(self) -> I16Vector3: ...
    @property
    def soq(self) -> I16Vector3: ...
    @property
    def sop(self) -> I16Vector3: ...
    @property
    def soo(self) -> I16Vector3: ...
    @property
    def sol(self) -> I16Vector3: ...
    @property
    def sls(self) -> I16Vector3: ...
    @property
    def slt(self) -> I16Vector3: ...
    @property
    def slq(self) -> I16Vector3: ...
    @property
    def slp(self) -> I16Vector3: ...
    @property
    def slo(self) -> I16Vector3: ...
    @property
    def sll(self) -> I16Vector3: ...
    @property
    def tss(self) -> I16Vector3: ...
    @property
    def tst(self) -> I16Vector3: ...
    @property
    def tsq(self) -> I16Vector3: ...
    @property
    def tsp(self) -> I16Vector3: ...
    @property
    def tso(self) -> I16Vector3: ...
    @property
    def tsl(self) -> I16Vector3: ...
    @property
    def tts(self) -> I16Vector3: ...
    @property
    def ttt(self) -> I16Vector3: ...
    @property
    def ttq(self) -> I16Vector3: ...
    @property
    def ttp(self) -> I16Vector3: ...
    @property
    def tto(self) -> I16Vector3: ...
    @property
    def ttl(self) -> I16Vector3: ...
    @property
    def tqs(self) -> I16Vector3: ...
    @property
    def tqt(self) -> I16Vector3: ...
    @property
    def tqq(self) -> I16Vector3: ...
    @property
    def tqp(self) -> I16Vector3: ...
    @property
    def tqo(self) -> I16Vector3: ...
    @property
    def tql(self) -> I16Vector3: ...
    @property
    def tps(self) -> I16Vector3: ...
    @property
    def tpt(self) -> I16Vector3: ...
    @property
    def tpq(self) -> I16Vector3: ...
    @property
    def tpp(self) -> I16Vector3: ...
    @property
    def tpo(self) -> I16Vector3: ...
    @property
    def tpl(self) -> I16Vector3: ...
    @property
    def tos(self) -> I16Vector3: ...
    @property
    def tot(self) -> I16Vector3: ...
    @property
    def toq(self) -> I16Vector3: ...
    @property
    def top(self) -> I16Vector3: ...
    @property
    def too(self) -> I16Vector3: ...
    @property
    def tol(self) -> I16Vector3: ...
    @property
    def tls(self) -> I16Vector3: ...
    @property
    def tlt(self) -> I16Vector3: ...
    @property
    def tlq(self) -> I16Vector3: ...
    @property
    def tlp(self) -> I16Vector3: ...
    @property
    def tlo(self) -> I16Vector3: ...
    @property
    def tll(self) -> I16Vector3: ...
    @property
    def qss(self) -> I16Vector3: ...
    @property
    def qst(self) -> I16Vector3: ...
    @property
    def qsq(self) -> I16Vector3: ...
    @property
    def qsp(self) -> I16Vector3: ...
    @property
    def qso(self) -> I16Vector3: ...
    @property
    def qsl(self) -> I16Vector3: ...
    @property
    def qts(self) -> I16Vector3: ...
    @property
    def qtt(self) -> I16Vector3: ...
    @property
    def qtq(self) -> I16Vector3: ...
    @property
    def qtp(self) -> I16Vector3: ...
    @property
    def qto(self) -> I16Vector3: ...
    @property
    def qtl(self) -> I16Vector3: ...
    @property
    def qqs(self) -> I16Vector3: ...
    @property
    def qqt(self) -> I16Vector3: ...
    @property
    def qqq(self) -> I16Vector3: ...
    @property
    def qqp(self) -> I16Vector3: ...
    @property
    def qqo(self) -> I16Vector3: ...
    @property
    def qql(self) -> I16Vector3: ...
    @property
    def qps(self) -> I16Vector3: ...
    @property
    def qpt(self) -> I16Vector3: ...
    @property
    def qpq(self) -> I16Vector3: ...
    @property
    def qpp(self) -> I16Vector3: ...
    @property
    def qpo(self) -> I16Vector3: ...
    @property
    def qpl(self) -> I16Vector3: ...
    @property
    def qos(self) -> I16Vector3: ...
    @property
    def qot(self) -> I16Vector3: ...
    @property
    def qoq(self) -> I16Vector3: ...
    @property
    def qop(self) -> I16Vector3: ...
    @property
    def qoo(self) -> I16Vector3: ...
    @property
    def qol(self) -> I16Vector3: ...
    @property
    def qls(self) -> I16Vector3: ...
    @property
    def qlt(self) -> I16Vector3: ...
    @property
    def qlq(self) -> I16Vector3: ...
    @property
    def qlp(self) -> I16Vector3: ...
    @property
    def qlo(self) -> I16Vector3: ...
    @property
    def qll(self) -> I16Vector3: ...
    @property
    def pss(self) -> I16Vector3: ...
    @property
    def pst(self) -> I16Vector3: ...
    @property
    def psq(self) -> I16Vector3: ...
    @property
    def psp(self) -> I16Vector3: ...
    @property
    def pso(self) -> I16Vector3: ...
    @property
    def psl(self) -> I16Vector3: ...
    @property
    def pts(self) -> I16Vector3: ...
    @property
    def ptt(self) -> I16Vector3: ...
    @property
    def ptq(self) -> I16Vector3: ...
    @property
    def ptp(self) -> I16Vector3: ...
    @property
    def pto(self) -> I16Vector3: ...
    @property
    def ptl(self) -> I16Vector3: ...
    @property
    def pqs(self) -> I16Vector3: ...
    @property
    def pqt(self) -> I16Vector3: ...
    @property
    def pqq(self) -> I16Vector3: ...
    @property
    def pqp(self) -> I16Vector3: ...
    @property
    def pqo(self) -> I16Vector3: ...
    @property
    def pql(self) -> I16Vector3: ...
    @property
    def pps(self) -> I16Vector3: ...
    @property
    def ppt(self) -> I16Vector3: ...
    @property
    def ppq(self) -> I16Vector3: ...
    @property
    def ppp(self) -> I16Vector3: ...
    @property
    def ppo(self) -> I16Vector3: ...
    @property
    def ppl(self) -> I16Vector3: ...
    @property
    def pos(self) -> I16Vector3: ...
    @property
    def pot(self) -> I16Vector3: ...
    @property
    def poq(self) -> I16Vector3: ...
    @property
    def pop(self) -> I16Vector3: ...
    @property
    def poo(self) -> I16Vector3: ...
    @property
    def pol(self) -> I16Vector3: ...
    @property
    def pls(self) -> I16Vector3: ...
    @property
    def plt(self) -> I16Vector3: ...
    @property
    def plq(self) -> I16Vector3: ...
    @property
    def plp(self) -> I16Vector3: ...
    @property
    def plo(self) -> I16Vector3: ...
    @property
    def pll(self) -> I16Vector3: ...
    @property
    def oss(self) -> I16Vector3: ...
    @property
    def ost(self) -> I16Vector3: ...
    @property
    def osq(self) -> I16Vector3: ...
    @property
    def osp(self) -> I16Vector3: ...
    @property
    def oso(self) -> I16Vector3: ...
    @property
    def osl(self) -> I16Vector3: ...
    @property
    def ots(self) -> I16Vector3: ...
    @property
    def ott(self) -> I16Vector3: ...
    @property
    def otq(self) -> I16Vector3: ...
    @property
    def otp(self) -> I16Vector3: ...
    @property
    def oto(self) -> I16Vector3: ...
    @property
    def otl(self) -> I16Vector3: ...
    @property
    def oqs(self) -> I16Vector3: ...
    @property
    def oqt(self) -> I16Vector3: ...
    @property
    def oqq(self) -> I16Vector3: ...
    @property
    def oqp(self) -> I16Vector3: ...
    @property
    def oqo(self) -> I16Vector3: ...
    @property
    def oql(self) -> I16Vector3: ...
    @property
    def ops(self) -> I16Vector3: ...
    @property
    def opt(self) -> I16Vector3: ...
    @property
    def opq(self) -> I16Vector3: ...
    @property
    def opp(self) -> I16Vector3: ...
    @property
    def opo(self) -> I16Vector3: ...
    @property
    def opl(self) -> I16Vector3: ...
    @property
    def oos(self) -> I16Vector3: ...
    @property
    def oot(self) -> I16Vector3: ...
    @property
    def ooq(self) -> I16Vector3: ...
    @property
    def oop(self) -> I16Vector3: ...
    @property
    def ols(self) -> I16Vector3: ...
    @property
    def olt(self) -> I16Vector3: ...
    @property
    def olq(self) -> I16Vector3: ...
    @property
    def olp(self) -> I16Vector3: ...
    @property
    def lss(self) -> I16Vector3: ...
    @property
    def lst(self) -> I16Vector3: ...
    @property
    def lsq(self) -> I16Vector3: ...
    @property
    def lsp(self) -> I16Vector3: ...
    @property
    def lso(self) -> I16Vector3: ...
    @property
    def lsl(self) -> I16Vector3: ...
    @property
    def lts(self) -> I16Vector3: ...
    @property
    def ltt(self) -> I16Vector3: ...
    @property
    def ltq(self) -> I16Vector3: ...
    @property
    def ltp(self) -> I16Vector3: ...
    @property
    def lto(self) -> I16Vector3: ...
    @property
    def ltl(self) -> I16Vector3: ...
    @property
    def lqs(self) -> I16Vector3: ...
    @property
    def lqt(self) -> I16Vector3: ...
    @property
    def lqq(self) -> I16Vector3: ...
    @property
    def lqp(self) -> I16Vector3: ...
    @property
    def lqo(self) -> I16Vector3: ...
    @property
    def lql(self) -> I16Vector3: ...
    @property
    def lps(self) -> I16Vector3: ...
    @property
    def lpt(self) -> I16Vector3: ...
    @property
    def lpq(self) -> I16Vector3: ...
    @property
    def lpp(self) -> I16Vector3: ...
    @property
    def lpo(self) -> I16Vector3: ...
    @property
    def lpl(self) -> I16Vector3: ...
    @property
    def los(self) -> I16Vector3: ...
    @property
    def lot(self) -> I16Vector3: ...
    @property
    def loq(self) -> I16Vector3: ...
    @property
    def lop(self) -> I16Vector3: ...
    @property
    def lls(self) -> I16Vector3: ...
    @property
    def llt(self) -> I16Vector3: ...
    @property
    def llq(self) -> I16Vector3: ...
    @property
    def llp(self) -> I16Vector3: ...
    @property
    def ssss(self) -> I16Vector4: ...
    @property
    def ssst(self) -> I16Vector4: ...
    @property
    def sssq(self) -> I16Vector4: ...
    @property
    def sssp(self) -> I16Vector4: ...
    @property
    def ssso(self) -> I16Vector4: ...
    @property
    def sssl(self) -> I16Vector4: ...
    @property
    def ssts(self) -> I16Vector4: ...
    @property
    def sstt(self) -> I16Vector4: ...
    @property
    def sstq(self) -> I16Vector4: ...
    @property
    def sstp(self) -> I16Vector4: ...
    @property
    def ssto(self) -> I16Vector4: ...
    @property
    def sstl(self) -> I16Vector4: ...
    @property
    def ssqs(self) -> I16Vector4: ...
    @property
    def ssqt(self) -> I16Vector4: ...
    @property
    def ssqq(self) -> I16Vector4: ...
    @property
    def ssqp(self) -> I16Vector4: ...
    @property
    def ssqo(self) -> I16Vector4: ...
    @property
    def ssql(self) -> I16Vector4: ...
    @property
    def ssps(self) -> I16Vector4: ...
    @property
    def sspt(self) -> I16Vector4: ...
    @property
    def sspq(self) -> I16Vector4: ...
    @property
    def sspp(self) -> I16Vector4: ...
    @property
    def sspo(self) -> I16Vector4: ...
    @property
    def sspl(self) -> I16Vector4: ...
    @property
    def ssos(self) -> I16Vector4: ...
    @property
    def ssot(self) -> I16Vector4: ...
    @property
    def ssoq(self) -> I16Vector4: ...
    @property
    def ssop(self) -> I16Vector4: ...
    @property
    def ssoo(self) -> I16Vector4: ...
    @property
    def ssol(self) -> I16Vector4: ...
    @property
    def ssls(self) -> I16Vector4: ...
    @property
    def sslt(self) -> I16Vector4: ...
    @property
    def sslq(self) -> I16Vector4: ...
    @property
    def sslp(self) -> I16Vector4: ...
    @property
    def sslo(self) -> I16Vector4: ...
    @property
    def ssll(self) -> I16Vector4: ...
    @property
    def stss(self) -> I16Vector4: ...
    @property
    def stst(self) -> I16Vector4: ...
    @property
    def stsq(self) -> I16Vector4: ...
    @property
    def stsp(self) -> I16Vector4: ...
    @property
    def stso(self) -> I16Vector4: ...
    @property
    def stsl(self) -> I16Vector4: ...
    @property
    def stts(self) -> I16Vector4: ...
    @property
    def sttt(self) -> I16Vector4: ...
    @property
    def sttq(self) -> I16Vector4: ...
    @property
    def sttp(self) -> I16Vector4: ...
    @property
    def stto(self) -> I16Vector4: ...
    @property
    def sttl(self) -> I16Vector4: ...
    @property
    def stqs(self) -> I16Vector4: ...
    @property
    def stqt(self) -> I16Vector4: ...
    @property
    def stqq(self) -> I16Vector4: ...
    @property
    def stqp(self) -> I16Vector4: ...
    @property
    def stqo(self) -> I16Vector4: ...
    @property
    def stql(self) -> I16Vector4: ...
    @property
    def stps(self) -> I16Vector4: ...
    @property
    def stpt(self) -> I16Vector4: ...
    @property
    def stpq(self) -> I16Vector4: ...
    @property
    def stpp(self) -> I16Vector4: ...
    @property
    def stpo(self) -> I16Vector4: ...
    @property
    def stpl(self) -> I16Vector4: ...
    @property
    def stos(self) -> I16Vector4: ...
    @property
    def stot(self) -> I16Vector4: ...
    @property
    def stoq(self) -> I16Vector4: ...
    @property
    def stop(self) -> I16Vector4: ...
    @property
    def stoo(self) -> I16Vector4: ...
    @property
    def stol(self) -> I16Vector4: ...
    @property
    def stls(self) -> I16Vector4: ...
    @property
    def stlt(self) -> I16Vector4: ...
    @property
    def stlq(self) -> I16Vector4: ...
    @property
    def stlp(self) -> I16Vector4: ...
    @property
    def stlo(self) -> I16Vector4: ...
    @property
    def stll(self) -> I16Vector4: ...
    @property
    def sqss(self) -> I16Vector4: ...
    @property
    def sqst(self) -> I16Vector4: ...
    @property
    def sqsq(self) -> I16Vector4: ...
    @property
    def sqsp(self) -> I16Vector4: ...
    @property
    def sqso(self) -> I16Vector4: ...
    @property
    def sqsl(self) -> I16Vector4: ...
    @property
    def sqts(self) -> I16Vector4: ...
    @property
    def sqtt(self) -> I16Vector4: ...
    @property
    def sqtq(self) -> I16Vector4: ...
    @property
    def sqtp(self) -> I16Vector4: ...
    @property
    def sqto(self) -> I16Vector4: ...
    @property
    def sqtl(self) -> I16Vector4: ...
    @property
    def sqqs(self) -> I16Vector4: ...
    @property
    def sqqt(self) -> I16Vector4: ...
    @property
    def sqqq(self) -> I16Vector4: ...
    @property
    def sqqp(self) -> I16Vector4: ...
    @property
    def sqqo(self) -> I16Vector4: ...
    @property
    def sqql(self) -> I16Vector4: ...
    @property
    def sqps(self) -> I16Vector4: ...
    @property
    def sqpt(self) -> I16Vector4: ...
    @property
    def sqpq(self) -> I16Vector4: ...
    @property
    def sqpp(self) -> I16Vector4: ...
    @property
    def sqpo(self) -> I16Vector4: ...
    @property
    def sqpl(self) -> I16Vector4: ...
    @property
    def sqos(self) -> I16Vector4: ...
    @property
    def sqot(self) -> I16Vector4: ...
    @property
    def sqoq(self) -> I16Vector4: ...
    @property
    def sqop(self) -> I16Vector4: ...
    @property
    def sqoo(self) -> I16Vector4: ...
    @property
    def sqol(self) -> I16Vector4: ...
    @property
    def sqls(self) -> I16Vector4: ...
    @property
    def sqlt(self) -> I16Vector4: ...
    @property
    def sqlq(self) -> I16Vector4: ...
    @property
    def sqlp(self) -> I16Vector4: ...
    @property
    def sqlo(self) -> I16Vector4: ...
    @property
    def sqll(self) -> I16Vector4: ...
    @property
    def spss(self) -> I16Vector4: ...
    @property
    def spst(self) -> I16Vector4: ...
    @property
    def spsq(self) -> I16Vector4: ...
    @property
    def spsp(self) -> I16Vector4: ...
    @property
    def spso(self) -> I16Vector4: ...
    @property
    def spsl(self) -> I16Vector4: ...
    @property
    def spts(self) -> I16Vector4: ...
    @property
    def sptt(self) -> I16Vector4: ...
    @property
    def sptq(self) -> I16Vector4: ...
    @property
    def sptp(self) -> I16Vector4: ...
    @property
    def spto(self) -> I16Vector4: ...
    @property
    def sptl(self) -> I16Vector4: ...
    @property
    def spqs(self) -> I16Vector4: ...
    @property
    def spqt(self) -> I16Vector4: ...
    @property
    def spqq(self) -> I16Vector4: ...
    @property
    def spqp(self) -> I16Vector4: ...
    @property
    def spqo(self) -> I16Vector4: ...
    @property
    def spql(self) -> I16Vector4: ...
    @property
    def spps(self) -> I16Vector4: ...
    @property
    def sppt(self) -> I16Vector4: ...
    @property
    def sppq(self) -> I16Vector4: ...
    @property
    def sppp(self) -> I16Vector4: ...
    @property
    def sppo(self) -> I16Vector4: ...
    @property
    def sppl(self) -> I16Vector4: ...
    @property
    def spos(self) -> I16Vector4: ...
    @property
    def spot(self) -> I16Vector4: ...
    @property
    def spoq(self) -> I16Vector4: ...
    @property
    def spop(self) -> I16Vector4: ...
    @property
    def spoo(self) -> I16Vector4: ...
    @property
    def spol(self) -> I16Vector4: ...
    @property
    def spls(self) -> I16Vector4: ...
    @property
    def splt(self) -> I16Vector4: ...
    @property
    def splq(self) -> I16Vector4: ...
    @property
    def splp(self) -> I16Vector4: ...
    @property
    def splo(self) -> I16Vector4: ...
    @property
    def spll(self) -> I16Vector4: ...
    @property
    def soss(self) -> I16Vector4: ...
    @property
    def sost(self) -> I16Vector4: ...
    @property
    def sosq(self) -> I16Vector4: ...
    @property
    def sosp(self) -> I16Vector4: ...
    @property
    def soso(self) -> I16Vector4: ...
    @property
    def sosl(self) -> I16Vector4: ...
    @property
    def sots(self) -> I16Vector4: ...
    @property
    def sott(self) -> I16Vector4: ...
    @property
    def sotq(self) -> I16Vector4: ...
    @property
    def sotp(self) -> I16Vector4: ...
    @property
    def soto(self) -> I16Vector4: ...
    @property
    def sotl(self) -> I16Vector4: ...
    @property
    def soqs(self) -> I16Vector4: ...
    @property
    def soqt(self) -> I16Vector4: ...
    @property
    def soqq(self) -> I16Vector4: ...
    @property
    def soqp(self) -> I16Vector4: ...
    @property
    def soqo(self) -> I16Vector4: ...
    @property
    def soql(self) -> I16Vector4: ...
    @property
    def sops(self) -> I16Vector4: ...
    @property
    def sopt(self) -> I16Vector4: ...
    @property
    def sopq(self) -> I16Vector4: ...
    @property
    def sopp(self) -> I16Vector4: ...
    @property
    def sopo(self) -> I16Vector4: ...
    @property
    def sopl(self) -> I16Vector4: ...
    @property
    def soos(self) -> I16Vector4: ...
    @property
    def soot(self) -> I16Vector4: ...
    @property
    def sooq(self) -> I16Vector4: ...
    @property
    def soop(self) -> I16Vector4: ...
    @property
    def sooo(self) -> I16Vector4: ...
    @property
    def sool(self) -> I16Vector4: ...
    @property
    def sols(self) -> I16Vector4: ...
    @property
    def solt(self) -> I16Vector4: ...
    @property
    def solq(self) -> I16Vector4: ...
    @property
    def solp(self) -> I16Vector4: ...
    @property
    def solo(self) -> I16Vector4: ...
    @property
    def soll(self) -> I16Vector4: ...
    @property
    def slss(self) -> I16Vector4: ...
    @property
    def slst(self) -> I16Vector4: ...
    @property
    def slsq(self) -> I16Vector4: ...
    @property
    def slsp(self) -> I16Vector4: ...
    @property
    def slso(self) -> I16Vector4: ...
    @property
    def slsl(self) -> I16Vector4: ...
    @property
    def slts(self) -> I16Vector4: ...
    @property
    def sltt(self) -> I16Vector4: ...
    @property
    def sltq(self) -> I16Vector4: ...
    @property
    def sltp(self) -> I16Vector4: ...
    @property
    def slto(self) -> I16Vector4: ...
    @property
    def sltl(self) -> I16Vector4: ...
    @property
    def slqs(self) -> I16Vector4: ...
    @property
    def slqt(self) -> I16Vector4: ...
    @property
    def slqq(self) -> I16Vector4: ...
    @property
    def slqp(self) -> I16Vector4: ...
    @property
    def slqo(self) -> I16Vector4: ...
    @property
    def slql(self) -> I16Vector4: ...
    @property
    def slps(self) -> I16Vector4: ...
    @property
    def slpt(self) -> I16Vector4: ...
    @property
    def slpq(self) -> I16Vector4: ...
    @property
    def slpp(self) -> I16Vector4: ...
    @property
    def slpo(self) -> I16Vector4: ...
    @property
    def slpl(self) -> I16Vector4: ...
    @property
    def slos(self) -> I16Vector4: ...
    @property
    def slot(self) -> I16Vector4: ...
    @property
    def sloq(self) -> I16Vector4: ...
    @property
    def slop(self) -> I16Vector4: ...
    @property
    def sloo(self) -> I16Vector4: ...
    @property
    def slol(self) -> I16Vector4: ...
    @property
    def slls(self) -> I16Vector4: ...
    @property
    def sllt(self) -> I16Vector4: ...
    @property
    def sllq(self) -> I16Vector4: ...
    @property
    def sllp(self) -> I16Vector4: ...
    @property
    def sllo(self) -> I16Vector4: ...
    @property
    def slll(self) -> I16Vector4: ...
    @property
    def tsss(self) -> I16Vector4: ...
    @property
    def tsst(self) -> I16Vector4: ...
    @property
    def tssq(self) -> I16Vector4: ...
    @property
    def tssp(self) -> I16Vector4: ...
    @property
    def tsso(self) -> I16Vector4: ...
    @property
    def tssl(self) -> I16Vector4: ...
    @property
    def tsts(self) -> I16Vector4: ...
    @property
    def tstt(self) -> I16Vector4: ...
    @property
    def tstq(self) -> I16Vector4: ...
    @property
    def tstp(self) -> I16Vector4: ...
    @property
    def tsto(self) -> I16Vector4: ...
    @property
    def tstl(self) -> I16Vector4: ...
    @property
    def tsqs(self) -> I16Vector4: ...
    @property
    def tsqt(self) -> I16Vector4: ...
    @property
    def tsqq(self) -> I16Vector4: ...
    @property
    def tsqp(self) -> I16Vector4: ...
    @property
    def tsqo(self) -> I16Vector4: ...
    @property
    def tsql(self) -> I16Vector4: ...
    @property
    def tsps(self) -> I16Vector4: ...
    @property
    def tspt(self) -> I16Vector4: ...
    @property
    def tspq(self) -> I16Vector4: ...
    @property
    def tspp(self) -> I16Vector4: ...
    @property
    def tspo(self) -> I16Vector4: ...
    @property
    def tspl(self) -> I16Vector4: ...
    @property
    def tsos(self) -> I16Vector4: ...
    @property
    def tsot(self) -> I16Vector4: ...
    @property
    def tsoq(self) -> I16Vector4: ...
    @property
    def tsop(self) -> I16Vector4: ...
    @property
    def tsoo(self) -> I16Vector4: ...
    @property
    def tsol(self) -> I16Vector4: ...
    @property
    def tsls(self) -> I16Vector4: ...
    @property
    def tslt(self) -> I16Vector4: ...
    @property
    def tslq(self) -> I16Vector4: ...
    @property
    def tslp(self) -> I16Vector4: ...
    @property
    def tslo(self) -> I16Vector4: ...
    @property
    def tsll(self) -> I16Vector4: ...
    @property
    def ttss(self) -> I16Vector4: ...
    @property
    def ttst(self) -> I16Vector4: ...
    @property
    def ttsq(self) -> I16Vector4: ...
    @property
    def ttsp(self) -> I16Vector4: ...
    @property
    def ttso(self) -> I16Vector4: ...
    @property
    def ttsl(self) -> I16Vector4: ...
    @property
    def ttts(self) -> I16Vector4: ...
    @property
    def tttt(self) -> I16Vector4: ...
    @property
    def tttq(self) -> I16Vector4: ...
    @property
    def tttp(self) -> I16Vector4: ...
    @property
    def ttto(self) -> I16Vector4: ...
    @property
    def tttl(self) -> I16Vector4: ...
    @property
    def ttqs(self) -> I16Vector4: ...
    @property
    def ttqt(self) -> I16Vector4: ...
    @property
    def ttqq(self) -> I16Vector4: ...
    @property
    def ttqp(self) -> I16Vector4: ...
    @property
    def ttqo(self) -> I16Vector4: ...
    @property
    def ttql(self) -> I16Vector4: ...
    @property
    def ttps(self) -> I16Vector4: ...
    @property
    def ttpt(self) -> I16Vector4: ...
    @property
    def ttpq(self) -> I16Vector4: ...
    @property
    def ttpp(self) -> I16Vector4: ...
    @property
    def ttpo(self) -> I16Vector4: ...
    @property
    def ttpl(self) -> I16Vector4: ...
    @property
    def ttos(self) -> I16Vector4: ...
    @property
    def ttot(self) -> I16Vector4: ...
    @property
    def ttoq(self) -> I16Vector4: ...
    @property
    def ttop(self) -> I16Vector4: ...
    @property
    def ttoo(self) -> I16Vector4: ...
    @property
    def ttol(self) -> I16Vector4: ...
    @property
    def ttls(self) -> I16Vector4: ...
    @property
    def ttlt(self) -> I16Vector4: ...
    @property
    def ttlq(self) -> I16Vector4: ...
    @property
    def ttlp(self) -> I16Vector4: ...
    @property
    def ttlo(self) -> I16Vector4: ...
    @property
    def ttll(self) -> I16Vector4: ...
    @property
    def tqss(self) -> I16Vector4: ...
    @property
    def tqst(self) -> I16Vector4: ...
    @property
    def tqsq(self) -> I16Vector4: ...
    @property
    def tqsp(self) -> I16Vector4: ...
    @property
    def tqso(self) -> I16Vector4: ...
    @property
    def tqsl(self) -> I16Vector4: ...
    @property
    def tqts(self) -> I16Vector4: ...
    @property
    def tqtt(self) -> I16Vector4: ...
    @property
    def tqtq(self) -> I16Vector4: ...
    @property
    def tqtp(self) -> I16Vector4: ...
    @property
    def tqto(self) -> I16Vector4: ...
    @property
    def tqtl(self) -> I16Vector4: ...
    @property
    def tqqs(self) -> I16Vector4: ...
    @property
    def tqqt(self) -> I16Vector4: ...
    @property
    def tqqq(self) -> I16Vector4: ...
    @property
    def tqqp(self) -> I16Vector4: ...
    @property
    def tqqo(self) -> I16Vector4: ...
    @property
    def tqql(self) -> I16Vector4: ...
    @property
    def tqps(self) -> I16Vector4: ...
    @property
    def tqpt(self) -> I16Vector4: ...
    @property
    def tqpq(self) -> I16Vector4: ...
    @property
    def tqpp(self) -> I16Vector4: ...
    @property
    def tqpo(self) -> I16Vector4: ...
    @property
    def tqpl(self) -> I16Vector4: ...
    @property
    def tqos(self) -> I16Vector4: ...
    @property
    def tqot(self) -> I16Vector4: ...
    @property
    def tqoq(self) -> I16Vector4: ...
    @property
    def tqop(self) -> I16Vector4: ...
    @property
    def tqoo(self) -> I16Vector4: ...
    @property
    def tqol(self) -> I16Vector4: ...
    @property
    def tqls(self) -> I16Vector4: ...
    @property
    def tqlt(self) -> I16Vector4: ...
    @property
    def tqlq(self) -> I16Vector4: ...
    @property
    def tqlp(self) -> I16Vector4: ...
    @property
    def tqlo(self) -> I16Vector4: ...
    @property
    def tqll(self) -> I16Vector4: ...
    @property
    def tpss(self) -> I16Vector4: ...
    @property
    def tpst(self) -> I16Vector4: ...
    @property
    def tpsq(self) -> I16Vector4: ...
    @property
    def tpsp(self) -> I16Vector4: ...
    @property
    def tpso(self) -> I16Vector4: ...
    @property
    def tpsl(self) -> I16Vector4: ...
    @property
    def tpts(self) -> I16Vector4: ...
    @property
    def tptt(self) -> I16Vector4: ...
    @property
    def tptq(self) -> I16Vector4: ...
    @property
    def tptp(self) -> I16Vector4: ...
    @property
    def tpto(self) -> I16Vector4: ...
    @property
    def tptl(self) -> I16Vector4: ...
    @property
    def tpqs(self) -> I16Vector4: ...
    @property
    def tpqt(self) -> I16Vector4: ...
    @property
    def tpqq(self) -> I16Vector4: ...
    @property
    def tpqp(self) -> I16Vector4: ...
    @property
    def tpqo(self) -> I16Vector4: ...
    @property
    def tpql(self) -> I16Vector4: ...
    @property
    def tpps(self) -> I16Vector4: ...
    @property
    def tppt(self) -> I16Vector4: ...
    @property
    def tppq(self) -> I16Vector4: ...
    @property
    def tppp(self) -> I16Vector4: ...
    @property
    def tppo(self) -> I16Vector4: ...
    @property
    def tppl(self) -> I16Vector4: ...
    @property
    def tpos(self) -> I16Vector4: ...
    @property
    def tpot(self) -> I16Vector4: ...
    @property
    def tpoq(self) -> I16Vector4: ...
    @property
    def tpop(self) -> I16Vector4: ...
    @property
    def tpoo(self) -> I16Vector4: ...
    @property
    def tpol(self) -> I16Vector4: ...
    @property
    def tpls(self) -> I16Vector4: ...
    @property
    def tplt(self) -> I16Vector4: ...
    @property
    def tplq(self) -> I16Vector4: ...
    @property
    def tplp(self) -> I16Vector4: ...
    @property
    def tplo(self) -> I16Vector4: ...
    @property
    def tpll(self) -> I16Vector4: ...
    @property
    def toss(self) -> I16Vector4: ...
    @property
    def tost(self) -> I16Vector4: ...
    @property
    def tosq(self) -> I16Vector4: ...
    @property
    def tosp(self) -> I16Vector4: ...
    @property
    def toso(self) -> I16Vector4: ...
    @property
    def tosl(self) -> I16Vector4: ...
    @property
    def tots(self) -> I16Vector4: ...
    @property
    def tott(self) -> I16Vector4: ...
    @property
    def totq(self) -> I16Vector4: ...
    @property
    def totp(self) -> I16Vector4: ...
    @property
    def toto(self) -> I16Vector4: ...
    @property
    def totl(self) -> I16Vector4: ...
    @property
    def toqs(self) -> I16Vector4: ...
    @property
    def toqt(self) -> I16Vector4: ...
    @property
    def toqq(self) -> I16Vector4: ...
    @property
    def toqp(self) -> I16Vector4: ...
    @property
    def toqo(self) -> I16Vector4: ...
    @property
    def toql(self) -> I16Vector4: ...
    @property
    def tops(self) -> I16Vector4: ...
    @property
    def topt(self) -> I16Vector4: ...
    @property
    def topq(self) -> I16Vector4: ...
    @property
    def topp(self) -> I16Vector4: ...
    @property
    def topo(self) -> I16Vector4: ...
    @property
    def topl(self) -> I16Vector4: ...
    @property
    def toos(self) -> I16Vector4: ...
    @property
    def toot(self) -> I16Vector4: ...
    @property
    def tooq(self) -> I16Vector4: ...
    @property
    def toop(self) -> I16Vector4: ...
    @property
    def tooo(self) -> I16Vector4: ...
    @property
    def tool(self) -> I16Vector4: ...
    @property
    def tols(self) -> I16Vector4: ...
    @property
    def tolt(self) -> I16Vector4: ...
    @property
    def tolq(self) -> I16Vector4: ...
    @property
    def tolp(self) -> I16Vector4: ...
    @property
    def tolo(self) -> I16Vector4: ...
    @property
    def toll(self) -> I16Vector4: ...
    @property
    def tlss(self) -> I16Vector4: ...
    @property
    def tlst(self) -> I16Vector4: ...
    @property
    def tlsq(self) -> I16Vector4: ...
    @property
    def tlsp(self) -> I16Vector4: ...
    @property
    def tlso(self) -> I16Vector4: ...
    @property
    def tlsl(self) -> I16Vector4: ...
    @property
    def tlts(self) -> I16Vector4: ...
    @property
    def tltt(self) -> I16Vector4: ...
    @property
    def tltq(self) -> I16Vector4: ...
    @property
    def tltp(self) -> I16Vector4: ...
    @property
    def tlto(self) -> I16Vector4: ...
    @property
    def tltl(self) -> I16Vector4: ...
    @property
    def tlqs(self) -> I16Vector4: ...
    @property
    def tlqt(self) -> I16Vector4: ...
    @property
    def tlqq(self) -> I16Vector4: ...
    @property
    def tlqp(self) -> I16Vector4: ...
    @property
    def tlqo(self) -> I16Vector4: ...
    @property
    def tlql(self) -> I16Vector4: ...
    @property
    def tlps(self) -> I16Vector4: ...
    @property
    def tlpt(self) -> I16Vector4: ...
    @property
    def tlpq(self) -> I16Vector4: ...
    @property
    def tlpp(self) -> I16Vector4: ...
    @property
    def tlpo(self) -> I16Vector4: ...
    @property
    def tlpl(self) -> I16Vector4: ...
    @property
    def tlos(self) -> I16Vector4: ...
    @property
    def tlot(self) -> I16Vector4: ...
    @property
    def tloq(self) -> I16Vector4: ...
    @property
    def tlop(self) -> I16Vector4: ...
    @property
    def tloo(self) -> I16Vector4: ...
    @property
    def tlol(self) -> I16Vector4: ...
    @property
    def tlls(self) -> I16Vector4: ...
    @property
    def tllt(self) -> I16Vector4: ...
    @property
    def tllq(self) -> I16Vector4: ...
    @property
    def tllp(self) -> I16Vector4: ...
    @property
    def tllo(self) -> I16Vector4: ...
    @property
    def tlll(self) -> I16Vector4: ...
    @property
    def qsss(self) -> I16Vector4: ...
    @property
    def qsst(self) -> I16Vector4: ...
    @property
    def qssq(self) -> I16Vector4: ...
    @property
    def qssp(self) -> I16Vector4: ...
    @property
    def qsso(self) -> I16Vector4: ...
    @property
    def qssl(self) -> I16Vector4: ...
    @property
    def qsts(self) -> I16Vector4: ...
    @property
    def qstt(self) -> I16Vector4: ...
    @property
    def qstq(self) -> I16Vector4: ...
    @property
    def qstp(self) -> I16Vector4: ...
    @property
    def qsto(self) -> I16Vector4: ...
    @property
    def qstl(self) -> I16Vector4: ...
    @property
    def qsqs(self) -> I16Vector4: ...
    @property
    def qsqt(self) -> I16Vector4: ...
    @property
    def qsqq(self) -> I16Vector4: ...
    @property
    def qsqp(self) -> I16Vector4: ...
    @property
    def qsqo(self) -> I16Vector4: ...
    @property
    def qsql(self) -> I16Vector4: ...
    @property
    def qsps(self) -> I16Vector4: ...
    @property
    def qspt(self) -> I16Vector4: ...
    @property
    def qspq(self) -> I16Vector4: ...
    @property
    def qspp(self) -> I16Vector4: ...
    @property
    def qspo(self) -> I16Vector4: ...
    @property
    def qspl(self) -> I16Vector4: ...
    @property
    def qsos(self) -> I16Vector4: ...
    @property
    def qsot(self) -> I16Vector4: ...
    @property
    def qsoq(self) -> I16Vector4: ...
    @property
    def qsop(self) -> I16Vector4: ...
    @property
    def qsoo(self) -> I16Vector4: ...
    @property
    def qsol(self) -> I16Vector4: ...
    @property
    def qsls(self) -> I16Vector4: ...
    @property
    def qslt(self) -> I16Vector4: ...
    @property
    def qslq(self) -> I16Vector4: ...
    @property
    def qslp(self) -> I16Vector4: ...
    @property
    def qslo(self) -> I16Vector4: ...
    @property
    def qsll(self) -> I16Vector4: ...
    @property
    def qtss(self) -> I16Vector4: ...
    @property
    def qtst(self) -> I16Vector4: ...
    @property
    def qtsq(self) -> I16Vector4: ...
    @property
    def qtsp(self) -> I16Vector4: ...
    @property
    def qtso(self) -> I16Vector4: ...
    @property
    def qtsl(self) -> I16Vector4: ...
    @property
    def qtts(self) -> I16Vector4: ...
    @property
    def qttt(self) -> I16Vector4: ...
    @property
    def qttq(self) -> I16Vector4: ...
    @property
    def qttp(self) -> I16Vector4: ...
    @property
    def qtto(self) -> I16Vector4: ...
    @property
    def qttl(self) -> I16Vector4: ...
    @property
    def qtqs(self) -> I16Vector4: ...
    @property
    def qtqt(self) -> I16Vector4: ...
    @property
    def qtqq(self) -> I16Vector4: ...
    @property
    def qtqp(self) -> I16Vector4: ...
    @property
    def qtqo(self) -> I16Vector4: ...
    @property
    def qtql(self) -> I16Vector4: ...
    @property
    def qtps(self) -> I16Vector4: ...
    @property
    def qtpt(self) -> I16Vector4: ...
    @property
    def qtpq(self) -> I16Vector4: ...
    @property
    def qtpp(self) -> I16Vector4: ...
    @property
    def qtpo(self) -> I16Vector4: ...
    @property
    def qtpl(self) -> I16Vector4: ...
    @property
    def qtos(self) -> I16Vector4: ...
    @property
    def qtot(self) -> I16Vector4: ...
    @property
    def qtoq(self) -> I16Vector4: ...
    @property
    def qtop(self) -> I16Vector4: ...
    @property
    def qtoo(self) -> I16Vector4: ...
    @property
    def qtol(self) -> I16Vector4: ...
    @property
    def qtls(self) -> I16Vector4: ...
    @property
    def qtlt(self) -> I16Vector4: ...
    @property
    def qtlq(self) -> I16Vector4: ...
    @property
    def qtlp(self) -> I16Vector4: ...
    @property
    def qtlo(self) -> I16Vector4: ...
    @property
    def qtll(self) -> I16Vector4: ...
    @property
    def qqss(self) -> I16Vector4: ...
    @property
    def qqst(self) -> I16Vector4: ...
    @property
    def qqsq(self) -> I16Vector4: ...
    @property
    def qqsp(self) -> I16Vector4: ...
    @property
    def qqso(self) -> I16Vector4: ...
    @property
    def qqsl(self) -> I16Vector4: ...
    @property
    def qqts(self) -> I16Vector4: ...
    @property
    def qqtt(self) -> I16Vector4: ...
    @property
    def qqtq(self) -> I16Vector4: ...
    @property
    def qqtp(self) -> I16Vector4: ...
    @property
    def qqto(self) -> I16Vector4: ...
    @property
    def qqtl(self) -> I16Vector4: ...
    @property
    def qqqs(self) -> I16Vector4: ...
    @property
    def qqqt(self) -> I16Vector4: ...
    @property
    def qqqq(self) -> I16Vector4: ...
    @property
    def qqqp(self) -> I16Vector4: ...
    @property
    def qqqo(self) -> I16Vector4: ...
    @property
    def qqql(self) -> I16Vector4: ...
    @property
    def qqps(self) -> I16Vector4: ...
    @property
    def qqpt(self) -> I16Vector4: ...
    @property
    def qqpq(self) -> I16Vector4: ...
    @property
    def qqpp(self) -> I16Vector4: ...
    @property
    def qqpo(self) -> I16Vector4: ...
    @property
    def qqpl(self) -> I16Vector4: ...
    @property
    def qqos(self) -> I16Vector4: ...
    @property
    def qqot(self) -> I16Vector4: ...
    @property
    def qqoq(self) -> I16Vector4: ...
    @property
    def qqop(self) -> I16Vector4: ...
    @property
    def qqoo(self) -> I16Vector4: ...
    @property
    def qqol(self) -> I16Vector4: ...
    @property
    def qqls(self) -> I16Vector4: ...
    @property
    def qqlt(self) -> I16Vector4: ...
    @property
    def qqlq(self) -> I16Vector4: ...
    @property
    def qqlp(self) -> I16Vector4: ...
    @property
    def qqlo(self) -> I16Vector4: ...
    @property
    def qqll(self) -> I16Vector4: ...
    @property
    def qpss(self) -> I16Vector4: ...
    @property
    def qpst(self) -> I16Vector4: ...
    @property
    def qpsq(self) -> I16Vector4: ...
    @property
    def qpsp(self) -> I16Vector4: ...
    @property
    def qpso(self) -> I16Vector4: ...
    @property
    def qpsl(self) -> I16Vector4: ...
    @property
    def qpts(self) -> I16Vector4: ...
    @property
    def qptt(self) -> I16Vector4: ...
    @property
    def qptq(self) -> I16Vector4: ...
    @property
    def qptp(self) -> I16Vector4: ...
    @property
    def qpto(self) -> I16Vector4: ...
    @property
    def qptl(self) -> I16Vector4: ...
    @property
    def qpqs(self) -> I16Vector4: ...
    @property
    def qpqt(self) -> I16Vector4: ...
    @property
    def qpqq(self) -> I16Vector4: ...
    @property
    def qpqp(self) -> I16Vector4: ...
    @property
    def qpqo(self) -> I16Vector4: ...
    @property
    def qpql(self) -> I16Vector4: ...
    @property
    def qpps(self) -> I16Vector4: ...
    @property
    def qppt(self) -> I16Vector4: ...
    @property
    def qppq(self) -> I16Vector4: ...
    @property
    def qppp(self) -> I16Vector4: ...
    @property
    def qppo(self) -> I16Vector4: ...
    @property
    def qppl(self) -> I16Vector4: ...
    @property
    def qpos(self) -> I16Vector4: ...
    @property
    def qpot(self) -> I16Vector4: ...
    @property
    def qpoq(self) -> I16Vector4: ...
    @property
    def qpop(self) -> I16Vector4: ...
    @property
    def qpoo(self) -> I16Vector4: ...
    @property
    def qpol(self) -> I16Vector4: ...
    @property
    def qpls(self) -> I16Vector4: ...
    @property
    def qplt(self) -> I16Vector4: ...
    @property
    def qplq(self) -> I16Vector4: ...
    @property
    def qplp(self) -> I16Vector4: ...
    @property
    def qplo(self) -> I16Vector4: ...
    @property
    def qpll(self) -> I16Vector4: ...
    @property
    def qoss(self) -> I16Vector4: ...
    @property
    def qost(self) -> I16Vector4: ...
    @property
    def qosq(self) -> I16Vector4: ...
    @property
    def qosp(self) -> I16Vector4: ...
    @property
    def qoso(self) -> I16Vector4: ...
    @property
    def qosl(self) -> I16Vector4: ...
    @property
    def qots(self) -> I16Vector4: ...
    @property
    def qott(self) -> I16Vector4: ...
    @property
    def qotq(self) -> I16Vector4: ...
    @property
    def qotp(self) -> I16Vector4: ...
    @property
    def qoto(self) -> I16Vector4: ...
    @property
    def qotl(self) -> I16Vector4: ...
    @property
    def qoqs(self) -> I16Vector4: ...
    @property
    def qoqt(self) -> I16Vector4: ...
    @property
    def qoqq(self) -> I16Vector4: ...
    @property
    def qoqp(self) -> I16Vector4: ...
    @property
    def qoqo(self) -> I16Vector4: ...
    @property
    def qoql(self) -> I16Vector4: ...
    @property
    def qops(self) -> I16Vector4: ...
    @property
    def qopt(self) -> I16Vector4: ...
    @property
    def qopq(self) -> I16Vector4: ...
    @property
    def qopp(self) -> I16Vector4: ...
    @property
    def qopo(self) -> I16Vector4: ...
    @property
    def qopl(self) -> I16Vector4: ...
    @property
    def qoos(self) -> I16Vector4: ...
    @property
    def qoot(self) -> I16Vector4: ...
    @property
    def qooq(self) -> I16Vector4: ...
    @property
    def qoop(self) -> I16Vector4: ...
    @property
    def qooo(self) -> I16Vector4: ...
    @property
    def qool(self) -> I16Vector4: ...
    @property
    def qols(self) -> I16Vector4: ...
    @property
    def qolt(self) -> I16Vector4: ...
    @property
    def qolq(self) -> I16Vector4: ...
    @property
    def qolp(self) -> I16Vector4: ...
    @property
    def qolo(self) -> I16Vector4: ...
    @property
    def qoll(self) -> I16Vector4: ...
    @property
    def qlss(self) -> I16Vector4: ...
    @property
    def qlst(self) -> I16Vector4: ...
    @property
    def qlsq(self) -> I16Vector4: ...
    @property
    def qlsp(self) -> I16Vector4: ...
    @property
    def qlso(self) -> I16Vector4: ...
    @property
    def qlsl(self) -> I16Vector4: ...
    @property
    def qlts(self) -> I16Vector4: ...
    @property
    def qltt(self) -> I16Vector4: ...
    @property
    def qltq(self) -> I16Vector4: ...
    @property
    def qltp(self) -> I16Vector4: ...
    @property
    def qlto(self) -> I16Vector4: ...
    @property
    def qltl(self) -> I16Vector4: ...
    @property
    def qlqs(self) -> I16Vector4: ...
    @property
    def qlqt(self) -> I16Vector4: ...
    @property
    def qlqq(self) -> I16Vector4: ...
    @property
    def qlqp(self) -> I16Vector4: ...
    @property
    def qlqo(self) -> I16Vector4: ...
    @property
    def qlql(self) -> I16Vector4: ...
    @property
    def qlps(self) -> I16Vector4: ...
    @property
    def qlpt(self) -> I16Vector4: ...
    @property
    def qlpq(self) -> I16Vector4: ...
    @property
    def qlpp(self) -> I16Vector4: ...
    @property
    def qlpo(self) -> I16Vector4: ...
    @property
    def qlpl(self) -> I16Vector4: ...
    @property
    def qlos(self) -> I16Vector4: ...
    @property
    def qlot(self) -> I16Vector4: ...
    @property
    def qloq(self) -> I16Vector4: ...
    @property
    def qlop(self) -> I16Vector4: ...
    @property
    def qloo(self) -> I16Vector4: ...
    @property
    def qlol(self) -> I16Vector4: ...
    @property
    def qlls(self) -> I16Vector4: ...
    @property
    def qllt(self) -> I16Vector4: ...
    @property
    def qllq(self) -> I16Vector4: ...
    @property
    def qllp(self) -> I16Vector4: ...
    @property
    def qllo(self) -> I16Vector4: ...
    @property
    def qlll(self) -> I16Vector4: ...
    @property
    def psss(self) -> I16Vector4: ...
    @property
    def psst(self) -> I16Vector4: ...
    @property
    def pssq(self) -> I16Vector4: ...
    @property
    def pssp(self) -> I16Vector4: ...
    @property
    def psso(self) -> I16Vector4: ...
    @property
    def pssl(self) -> I16Vector4: ...
    @property
    def psts(self) -> I16Vector4: ...
    @property
    def pstt(self) -> I16Vector4: ...
    @property
    def pstq(self) -> I16Vector4: ...
    @property
    def pstp(self) -> I16Vector4: ...
    @property
    def psto(self) -> I16Vector4: ...
    @property
    def pstl(self) -> I16Vector4: ...
    @property
    def psqs(self) -> I16Vector4: ...
    @property
    def psqt(self) -> I16Vector4: ...
    @property
    def psqq(self) -> I16Vector4: ...
    @property
    def psqp(self) -> I16Vector4: ...
    @property
    def psqo(self) -> I16Vector4: ...
    @property
    def psql(self) -> I16Vector4: ...
    @property
    def psps(self) -> I16Vector4: ...
    @property
    def pspt(self) -> I16Vector4: ...
    @property
    def pspq(self) -> I16Vector4: ...
    @property
    def pspp(self) -> I16Vector4: ...
    @property
    def pspo(self) -> I16Vector4: ...
    @property
    def pspl(self) -> I16Vector4: ...
    @property
    def psos(self) -> I16Vector4: ...
    @property
    def psot(self) -> I16Vector4: ...
    @property
    def psoq(self) -> I16Vector4: ...
    @property
    def psop(self) -> I16Vector4: ...
    @property
    def psoo(self) -> I16Vector4: ...
    @property
    def psol(self) -> I16Vector4: ...
    @property
    def psls(self) -> I16Vector4: ...
    @property
    def pslt(self) -> I16Vector4: ...
    @property
    def pslq(self) -> I16Vector4: ...
    @property
    def pslp(self) -> I16Vector4: ...
    @property
    def pslo(self) -> I16Vector4: ...
    @property
    def psll(self) -> I16Vector4: ...
    @property
    def ptss(self) -> I16Vector4: ...
    @property
    def ptst(self) -> I16Vector4: ...
    @property
    def ptsq(self) -> I16Vector4: ...
    @property
    def ptsp(self) -> I16Vector4: ...
    @property
    def ptso(self) -> I16Vector4: ...
    @property
    def ptsl(self) -> I16Vector4: ...
    @property
    def ptts(self) -> I16Vector4: ...
    @property
    def pttt(self) -> I16Vector4: ...
    @property
    def pttq(self) -> I16Vector4: ...
    @property
    def pttp(self) -> I16Vector4: ...
    @property
    def ptto(self) -> I16Vector4: ...
    @property
    def pttl(self) -> I16Vector4: ...
    @property
    def ptqs(self) -> I16Vector4: ...
    @property
    def ptqt(self) -> I16Vector4: ...
    @property
    def ptqq(self) -> I16Vector4: ...
    @property
    def ptqp(self) -> I16Vector4: ...
    @property
    def ptqo(self) -> I16Vector4: ...
    @property
    def ptql(self) -> I16Vector4: ...
    @property
    def ptps(self) -> I16Vector4: ...
    @property
    def ptpt(self) -> I16Vector4: ...
    @property
    def ptpq(self) -> I16Vector4: ...
    @property
    def ptpp(self) -> I16Vector4: ...
    @property
    def ptpo(self) -> I16Vector4: ...
    @property
    def ptpl(self) -> I16Vector4: ...
    @property
    def ptos(self) -> I16Vector4: ...
    @property
    def ptot(self) -> I16Vector4: ...
    @property
    def ptoq(self) -> I16Vector4: ...
    @property
    def ptop(self) -> I16Vector4: ...
    @property
    def ptoo(self) -> I16Vector4: ...
    @property
    def ptol(self) -> I16Vector4: ...
    @property
    def ptls(self) -> I16Vector4: ...
    @property
    def ptlt(self) -> I16Vector4: ...
    @property
    def ptlq(self) -> I16Vector4: ...
    @property
    def ptlp(self) -> I16Vector4: ...
    @property
    def ptlo(self) -> I16Vector4: ...
    @property
    def ptll(self) -> I16Vector4: ...
    @property
    def pqss(self) -> I16Vector4: ...
    @property
    def pqst(self) -> I16Vector4: ...
    @property
    def pqsq(self) -> I16Vector4: ...
    @property
    def pqsp(self) -> I16Vector4: ...
    @property
    def pqso(self) -> I16Vector4: ...
    @property
    def pqsl(self) -> I16Vector4: ...
    @property
    def pqts(self) -> I16Vector4: ...
    @property
    def pqtt(self) -> I16Vector4: ...
    @property
    def pqtq(self) -> I16Vector4: ...
    @property
    def pqtp(self) -> I16Vector4: ...
    @property
    def pqto(self) -> I16Vector4: ...
    @property
    def pqtl(self) -> I16Vector4: ...
    @property
    def pqqs(self) -> I16Vector4: ...
    @property
    def pqqt(self) -> I16Vector4: ...
    @property
    def pqqq(self) -> I16Vector4: ...
    @property
    def pqqp(self) -> I16Vector4: ...
    @property
    def pqqo(self) -> I16Vector4: ...
    @property
    def pqql(self) -> I16Vector4: ...
    @property
    def pqps(self) -> I16Vector4: ...
    @property
    def pqpt(self) -> I16Vector4: ...
    @property
    def pqpq(self) -> I16Vector4: ...
    @property
    def pqpp(self) -> I16Vector4: ...
    @property
    def pqpo(self) -> I16Vector4: ...
    @property
    def pqpl(self) -> I16Vector4: ...
    @property
    def pqos(self) -> I16Vector4: ...
    @property
    def pqot(self) -> I16Vector4: ...
    @property
    def pqoq(self) -> I16Vector4: ...
    @property
    def pqop(self) -> I16Vector4: ...
    @property
    def pqoo(self) -> I16Vector4: ...
    @property
    def pqol(self) -> I16Vector4: ...
    @property
    def pqls(self) -> I16Vector4: ...
    @property
    def pqlt(self) -> I16Vector4: ...
    @property
    def pqlq(self) -> I16Vector4: ...
    @property
    def pqlp(self) -> I16Vector4: ...
    @property
    def pqlo(self) -> I16Vector4: ...
    @property
    def pqll(self) -> I16Vector4: ...
    @property
    def ppss(self) -> I16Vector4: ...
    @property
    def ppst(self) -> I16Vector4: ...
    @property
    def ppsq(self) -> I16Vector4: ...
    @property
    def ppsp(self) -> I16Vector4: ...
    @property
    def ppso(self) -> I16Vector4: ...
    @property
    def ppsl(self) -> I16Vector4: ...
    @property
    def ppts(self) -> I16Vector4: ...
    @property
    def pptt(self) -> I16Vector4: ...
    @property
    def pptq(self) -> I16Vector4: ...
    @property
    def pptp(self) -> I16Vector4: ...
    @property
    def ppto(self) -> I16Vector4: ...
    @property
    def pptl(self) -> I16Vector4: ...
    @property
    def ppqs(self) -> I16Vector4: ...
    @property
    def ppqt(self) -> I16Vector4: ...
    @property
    def ppqq(self) -> I16Vector4: ...
    @property
    def ppqp(self) -> I16Vector4: ...
    @property
    def ppqo(self) -> I16Vector4: ...
    @property
    def ppql(self) -> I16Vector4: ...
    @property
    def ppps(self) -> I16Vector4: ...
    @property
    def pppt(self) -> I16Vector4: ...
    @property
    def pppq(self) -> I16Vector4: ...
    @property
    def pppp(self) -> I16Vector4: ...
    @property
    def pppo(self) -> I16Vector4: ...
    @property
    def pppl(self) -> I16Vector4: ...
    @property
    def ppos(self) -> I16Vector4: ...
    @property
    def ppot(self) -> I16Vector4: ...
    @property
    def ppoq(self) -> I16Vector4: ...
    @property
    def ppop(self) -> I16Vector4: ...
    @property
    def ppoo(self) -> I16Vector4: ...
    @property
    def ppol(self) -> I16Vector4: ...
    @property
    def ppls(self) -> I16Vector4: ...
    @property
    def pplt(self) -> I16Vector4: ...
    @property
    def pplq(self) -> I16Vector4: ...
    @property
    def pplp(self) -> I16Vector4: ...
    @property
    def pplo(self) -> I16Vector4: ...
    @property
    def ppll(self) -> I16Vector4: ...
    @property
    def poss(self) -> I16Vector4: ...
    @property
    def post(self) -> I16Vector4: ...
    @property
    def posq(self) -> I16Vector4: ...
    @property
    def posp(self) -> I16Vector4: ...
    @property
    def poso(self) -> I16Vector4: ...
    @property
    def posl(self) -> I16Vector4: ...
    @property
    def pots(self) -> I16Vector4: ...
    @property
    def pott(self) -> I16Vector4: ...
    @property
    def potq(self) -> I16Vector4: ...
    @property
    def potp(self) -> I16Vector4: ...
    @property
    def poto(self) -> I16Vector4: ...
    @property
    def potl(self) -> I16Vector4: ...
    @property
    def poqs(self) -> I16Vector4: ...
    @property
    def poqt(self) -> I16Vector4: ...
    @property
    def poqq(self) -> I16Vector4: ...
    @property
    def poqp(self) -> I16Vector4: ...
    @property
    def poqo(self) -> I16Vector4: ...
    @property
    def poql(self) -> I16Vector4: ...
    @property
    def pops(self) -> I16Vector4: ...
    @property
    def popt(self) -> I16Vector4: ...
    @property
    def popq(self) -> I16Vector4: ...
    @property
    def popp(self) -> I16Vector4: ...
    @property
    def popo(self) -> I16Vector4: ...
    @property
    def popl(self) -> I16Vector4: ...
    @property
    def poos(self) -> I16Vector4: ...
    @property
    def poot(self) -> I16Vector4: ...
    @property
    def pooq(self) -> I16Vector4: ...
    @property
    def poop(self) -> I16Vector4: ...
    @property
    def pooo(self) -> I16Vector4: ...
    @property
    def pool(self) -> I16Vector4: ...
    @property
    def pols(self) -> I16Vector4: ...
    @property
    def polt(self) -> I16Vector4: ...
    @property
    def polq(self) -> I16Vector4: ...
    @property
    def polp(self) -> I16Vector4: ...
    @property
    def polo(self) -> I16Vector4: ...
    @property
    def poll(self) -> I16Vector4: ...
    @property
    def plss(self) -> I16Vector4: ...
    @property
    def plst(self) -> I16Vector4: ...
    @property
    def plsq(self) -> I16Vector4: ...
    @property
    def plsp(self) -> I16Vector4: ...
    @property
    def plso(self) -> I16Vector4: ...
    @property
    def plsl(self) -> I16Vector4: ...
    @property
    def plts(self) -> I16Vector4: ...
    @property
    def pltt(self) -> I16Vector4: ...
    @property
    def pltq(self) -> I16Vector4: ...
    @property
    def pltp(self) -> I16Vector4: ...
    @property
    def plto(self) -> I16Vector4: ...
    @property
    def pltl(self) -> I16Vector4: ...
    @property
    def plqs(self) -> I16Vector4: ...
    @property
    def plqt(self) -> I16Vector4: ...
    @property
    def plqq(self) -> I16Vector4: ...
    @property
    def plqp(self) -> I16Vector4: ...
    @property
    def plqo(self) -> I16Vector4: ...
    @property
    def plql(self) -> I16Vector4: ...
    @property
    def plps(self) -> I16Vector4: ...
    @property
    def plpt(self) -> I16Vector4: ...
    @property
    def plpq(self) -> I16Vector4: ...
    @property
    def plpp(self) -> I16Vector4: ...
    @property
    def plpo(self) -> I16Vector4: ...
    @property
    def plpl(self) -> I16Vector4: ...
    @property
    def plos(self) -> I16Vector4: ...
    @property
    def plot(self) -> I16Vector4: ...
    @property
    def ploq(self) -> I16Vector4: ...
    @property
    def plop(self) -> I16Vector4: ...
    @property
    def ploo(self) -> I16Vector4: ...
    @property
    def plol(self) -> I16Vector4: ...
    @property
    def plls(self) -> I16Vector4: ...
    @property
    def pllt(self) -> I16Vector4: ...
    @property
    def pllq(self) -> I16Vector4: ...
    @property
    def pllp(self) -> I16Vector4: ...
    @property
    def pllo(self) -> I16Vector4: ...
    @property
    def plll(self) -> I16Vector4: ...
    @property
    def osss(self) -> I16Vector4: ...
    @property
    def osst(self) -> I16Vector4: ...
    @property
    def ossq(self) -> I16Vector4: ...
    @property
    def ossp(self) -> I16Vector4: ...
    @property
    def osso(self) -> I16Vector4: ...
    @property
    def ossl(self) -> I16Vector4: ...
    @property
    def osts(self) -> I16Vector4: ...
    @property
    def ostt(self) -> I16Vector4: ...
    @property
    def ostq(self) -> I16Vector4: ...
    @property
    def ostp(self) -> I16Vector4: ...
    @property
    def osto(self) -> I16Vector4: ...
    @property
    def ostl(self) -> I16Vector4: ...
    @property
    def osqs(self) -> I16Vector4: ...
    @property
    def osqt(self) -> I16Vector4: ...
    @property
    def osqq(self) -> I16Vector4: ...
    @property
    def osqp(self) -> I16Vector4: ...
    @property
    def osqo(self) -> I16Vector4: ...
    @property
    def osql(self) -> I16Vector4: ...
    @property
    def osps(self) -> I16Vector4: ...
    @property
    def ospt(self) -> I16Vector4: ...
    @property
    def ospq(self) -> I16Vector4: ...
    @property
    def ospp(self) -> I16Vector4: ...
    @property
    def ospo(self) -> I16Vector4: ...
    @property
    def ospl(self) -> I16Vector4: ...
    @property
    def osos(self) -> I16Vector4: ...
    @property
    def osot(self) -> I16Vector4: ...
    @property
    def osoq(self) -> I16Vector4: ...
    @property
    def osop(self) -> I16Vector4: ...
    @property
    def osoo(self) -> I16Vector4: ...
    @property
    def osol(self) -> I16Vector4: ...
    @property
    def osls(self) -> I16Vector4: ...
    @property
    def oslt(self) -> I16Vector4: ...
    @property
    def oslq(self) -> I16Vector4: ...
    @property
    def oslp(self) -> I16Vector4: ...
    @property
    def oslo(self) -> I16Vector4: ...
    @property
    def osll(self) -> I16Vector4: ...
    @property
    def otss(self) -> I16Vector4: ...
    @property
    def otst(self) -> I16Vector4: ...
    @property
    def otsq(self) -> I16Vector4: ...
    @property
    def otsp(self) -> I16Vector4: ...
    @property
    def otso(self) -> I16Vector4: ...
    @property
    def otsl(self) -> I16Vector4: ...
    @property
    def otts(self) -> I16Vector4: ...
    @property
    def ottt(self) -> I16Vector4: ...
    @property
    def ottq(self) -> I16Vector4: ...
    @property
    def ottp(self) -> I16Vector4: ...
    @property
    def otto(self) -> I16Vector4: ...
    @property
    def ottl(self) -> I16Vector4: ...
    @property
    def otqs(self) -> I16Vector4: ...
    @property
    def otqt(self) -> I16Vector4: ...
    @property
    def otqq(self) -> I16Vector4: ...
    @property
    def otqp(self) -> I16Vector4: ...
    @property
    def otqo(self) -> I16Vector4: ...
    @property
    def otql(self) -> I16Vector4: ...
    @property
    def otps(self) -> I16Vector4: ...
    @property
    def otpt(self) -> I16Vector4: ...
    @property
    def otpq(self) -> I16Vector4: ...
    @property
    def otpp(self) -> I16Vector4: ...
    @property
    def otpo(self) -> I16Vector4: ...
    @property
    def otpl(self) -> I16Vector4: ...
    @property
    def otos(self) -> I16Vector4: ...
    @property
    def otot(self) -> I16Vector4: ...
    @property
    def otoq(self) -> I16Vector4: ...
    @property
    def otop(self) -> I16Vector4: ...
    @property
    def otoo(self) -> I16Vector4: ...
    @property
    def otol(self) -> I16Vector4: ...
    @property
    def otls(self) -> I16Vector4: ...
    @property
    def otlt(self) -> I16Vector4: ...
    @property
    def otlq(self) -> I16Vector4: ...
    @property
    def otlp(self) -> I16Vector4: ...
    @property
    def otlo(self) -> I16Vector4: ...
    @property
    def otll(self) -> I16Vector4: ...
    @property
    def oqss(self) -> I16Vector4: ...
    @property
    def oqst(self) -> I16Vector4: ...
    @property
    def oqsq(self) -> I16Vector4: ...
    @property
    def oqsp(self) -> I16Vector4: ...
    @property
    def oqso(self) -> I16Vector4: ...
    @property
    def oqsl(self) -> I16Vector4: ...
    @property
    def oqts(self) -> I16Vector4: ...
    @property
    def oqtt(self) -> I16Vector4: ...
    @property
    def oqtq(self) -> I16Vector4: ...
    @property
    def oqtp(self) -> I16Vector4: ...
    @property
    def oqto(self) -> I16Vector4: ...
    @property
    def oqtl(self) -> I16Vector4: ...
    @property
    def oqqs(self) -> I16Vector4: ...
    @property
    def oqqt(self) -> I16Vector4: ...
    @property
    def oqqq(self) -> I16Vector4: ...
    @property
    def oqqp(self) -> I16Vector4: ...
    @property
    def oqqo(self) -> I16Vector4: ...
    @property
    def oqql(self) -> I16Vector4: ...
    @property
    def oqps(self) -> I16Vector4: ...
    @property
    def oqpt(self) -> I16Vector4: ...
    @property
    def oqpq(self) -> I16Vector4: ...
    @property
    def oqpp(self) -> I16Vector4: ...
    @property
    def oqpo(self) -> I16Vector4: ...
    @property
    def oqpl(self) -> I16Vector4: ...
    @property
    def oqos(self) -> I16Vector4: ...
    @property
    def oqot(self) -> I16Vector4: ...
    @property
    def oqoq(self) -> I16Vector4: ...
    @property
    def oqop(self) -> I16Vector4: ...
    @property
    def oqoo(self) -> I16Vector4: ...
    @property
    def oqol(self) -> I16Vector4: ...
    @property
    def oqls(self) -> I16Vector4: ...
    @property
    def oqlt(self) -> I16Vector4: ...
    @property
    def oqlq(self) -> I16Vector4: ...
    @property
    def oqlp(self) -> I16Vector4: ...
    @property
    def oqlo(self) -> I16Vector4: ...
    @property
    def oqll(self) -> I16Vector4: ...
    @property
    def opss(self) -> I16Vector4: ...
    @property
    def opst(self) -> I16Vector4: ...
    @property
    def opsq(self) -> I16Vector4: ...
    @property
    def opsp(self) -> I16Vector4: ...
    @property
    def opso(self) -> I16Vector4: ...
    @property
    def opsl(self) -> I16Vector4: ...
    @property
    def opts(self) -> I16Vector4: ...
    @property
    def optt(self) -> I16Vector4: ...
    @property
    def optq(self) -> I16Vector4: ...
    @property
    def optp(self) -> I16Vector4: ...
    @property
    def opto(self) -> I16Vector4: ...
    @property
    def optl(self) -> I16Vector4: ...
    @property
    def opqs(self) -> I16Vector4: ...
    @property
    def opqt(self) -> I16Vector4: ...
    @property
    def opqq(self) -> I16Vector4: ...
    @property
    def opqp(self) -> I16Vector4: ...
    @property
    def opqo(self) -> I16Vector4: ...
    @property
    def opql(self) -> I16Vector4: ...
    @property
    def opps(self) -> I16Vector4: ...
    @property
    def oppt(self) -> I16Vector4: ...
    @property
    def oppq(self) -> I16Vector4: ...
    @property
    def oppp(self) -> I16Vector4: ...
    @property
    def oppo(self) -> I16Vector4: ...
    @property
    def oppl(self) -> I16Vector4: ...
    @property
    def opos(self) -> I16Vector4: ...
    @property
    def opot(self) -> I16Vector4: ...
    @property
    def opoq(self) -> I16Vector4: ...
    @property
    def opop(self) -> I16Vector4: ...
    @property
    def opoo(self) -> I16Vector4: ...
    @property
    def opol(self) -> I16Vector4: ...
    @property
    def opls(self) -> I16Vector4: ...
    @property
    def oplt(self) -> I16Vector4: ...
    @property
    def oplq(self) -> I16Vector4: ...
    @property
    def oplp(self) -> I16Vector4: ...
    @property
    def oplo(self) -> I16Vector4: ...
    @property
    def opll(self) -> I16Vector4: ...
    @property
    def ooss(self) -> I16Vector4: ...
    @property
    def oost(self) -> I16Vector4: ...
    @property
    def oosq(self) -> I16Vector4: ...
    @property
    def oosp(self) -> I16Vector4: ...
    @property
    def ooso(self) -> I16Vector4: ...
    @property
    def oosl(self) -> I16Vector4: ...
    @property
    def oots(self) -> I16Vector4: ...
    @property
    def oott(self) -> I16Vector4: ...
    @property
    def ootq(self) -> I16Vector4: ...
    @property
    def ootp(self) -> I16Vector4: ...
    @property
    def ooto(self) -> I16Vector4: ...
    @property
    def ootl(self) -> I16Vector4: ...
    @property
    def ooqs(self) -> I16Vector4: ...
    @property
    def ooqt(self) -> I16Vector4: ...
    @property
    def ooqq(self) -> I16Vector4: ...
    @property
    def ooqp(self) -> I16Vector4: ...
    @property
    def ooqo(self) -> I16Vector4: ...
    @property
    def ooql(self) -> I16Vector4: ...
    @property
    def oops(self) -> I16Vector4: ...
    @property
    def oopt(self) -> I16Vector4: ...
    @property
    def oopq(self) -> I16Vector4: ...
    @property
    def oopp(self) -> I16Vector4: ...
    @property
    def oopo(self) -> I16Vector4: ...
    @property
    def oopl(self) -> I16Vector4: ...
    @property
    def ooos(self) -> I16Vector4: ...
    @property
    def ooot(self) -> I16Vector4: ...
    @property
    def oooq(self) -> I16Vector4: ...
    @property
    def ooop(self) -> I16Vector4: ...
    @property
    def ools(self) -> I16Vector4: ...
    @property
    def oolt(self) -> I16Vector4: ...
    @property
    def oolq(self) -> I16Vector4: ...
    @property
    def oolp(self) -> I16Vector4: ...
    @property
    def olss(self) -> I16Vector4: ...
    @property
    def olst(self) -> I16Vector4: ...
    @property
    def olsq(self) -> I16Vector4: ...
    @property
    def olsp(self) -> I16Vector4: ...
    @property
    def olso(self) -> I16Vector4: ...
    @property
    def olsl(self) -> I16Vector4: ...
    @property
    def olts(self) -> I16Vector4: ...
    @property
    def oltt(self) -> I16Vector4: ...
    @property
    def oltq(self) -> I16Vector4: ...
    @property
    def oltp(self) -> I16Vector4: ...
    @property
    def olto(self) -> I16Vector4: ...
    @property
    def oltl(self) -> I16Vector4: ...
    @property
    def olqs(self) -> I16Vector4: ...
    @property
    def olqt(self) -> I16Vector4: ...
    @property
    def olqq(self) -> I16Vector4: ...
    @property
    def olqp(self) -> I16Vector4: ...
    @property
    def olqo(self) -> I16Vector4: ...
    @property
    def olql(self) -> I16Vector4: ...
    @property
    def olps(self) -> I16Vector4: ...
    @property
    def olpt(self) -> I16Vector4: ...
    @property
    def olpq(self) -> I16Vector4: ...
    @property
    def olpp(self) -> I16Vector4: ...
    @property
    def olpo(self) -> I16Vector4: ...
    @property
    def olpl(self) -> I16Vector4: ...
    @property
    def olos(self) -> I16Vector4: ...
    @property
    def olot(self) -> I16Vector4: ...
    @property
    def oloq(self) -> I16Vector4: ...
    @property
    def olop(self) -> I16Vector4: ...
    @property
    def olls(self) -> I16Vector4: ...
    @property
    def ollt(self) -> I16Vector4: ...
    @property
    def ollq(self) -> I16Vector4: ...
    @property
    def ollp(self) -> I16Vector4: ...
    @property
    def lsss(self) -> I16Vector4: ...
    @property
    def lsst(self) -> I16Vector4: ...
    @property
    def lssq(self) -> I16Vector4: ...
    @property
    def lssp(self) -> I16Vector4: ...
    @property
    def lsso(self) -> I16Vector4: ...
    @property
    def lssl(self) -> I16Vector4: ...
    @property
    def lsts(self) -> I16Vector4: ...
    @property
    def lstt(self) -> I16Vector4: ...
    @property
    def lstq(self) -> I16Vector4: ...
    @property
    def lstp(self) -> I16Vector4: ...
    @property
    def lsto(self) -> I16Vector4: ...
    @property
    def lstl(self) -> I16Vector4: ...
    @property
    def lsqs(self) -> I16Vector4: ...
    @property
    def lsqt(self) -> I16Vector4: ...
    @property
    def lsqq(self) -> I16Vector4: ...
    @property
    def lsqp(self) -> I16Vector4: ...
    @property
    def lsqo(self) -> I16Vector4: ...
    @property
    def lsql(self) -> I16Vector4: ...
    @property
    def lsps(self) -> I16Vector4: ...
    @property
    def lspt(self) -> I16Vector4: ...
    @property
    def lspq(self) -> I16Vector4: ...
    @property
    def lspp(self) -> I16Vector4: ...
    @property
    def lspo(self) -> I16Vector4: ...
    @property
    def lspl(self) -> I16Vector4: ...
    @property
    def lsos(self) -> I16Vector4: ...
    @property
    def lsot(self) -> I16Vector4: ...
    @property
    def lsoq(self) -> I16Vector4: ...
    @property
    def lsop(self) -> I16Vector4: ...
    @property
    def lsoo(self) -> I16Vector4: ...
    @property
    def lsol(self) -> I16Vector4: ...
    @property
    def lsls(self) -> I16Vector4: ...
    @property
    def lslt(self) -> I16Vector4: ...
    @property
    def lslq(self) -> I16Vector4: ...
    @property
    def lslp(self) -> I16Vector4: ...
    @property
    def lslo(self) -> I16Vector4: ...
    @property
    def lsll(self) -> I16Vector4: ...
    @property
    def ltss(self) -> I16Vector4: ...
    @property
    def ltst(self) -> I16Vector4: ...
    @property
    def ltsq(self) -> I16Vector4: ...
    @property
    def ltsp(self) -> I16Vector4: ...
    @property
    def ltso(self) -> I16Vector4: ...
    @property
    def ltsl(self) -> I16Vector4: ...
    @property
    def ltts(self) -> I16Vector4: ...
    @property
    def lttt(self) -> I16Vector4: ...
    @property
    def lttq(self) -> I16Vector4: ...
    @property
    def lttp(self) -> I16Vector4: ...
    @property
    def ltto(self) -> I16Vector4: ...
    @property
    def lttl(self) -> I16Vector4: ...
    @property
    def ltqs(self) -> I16Vector4: ...
    @property
    def ltqt(self) -> I16Vector4: ...
    @property
    def ltqq(self) -> I16Vector4: ...
    @property
    def ltqp(self) -> I16Vector4: ...
    @property
    def ltqo(self) -> I16Vector4: ...
    @property
    def ltql(self) -> I16Vector4: ...
    @property
    def ltps(self) -> I16Vector4: ...
    @property
    def ltpt(self) -> I16Vector4: ...
    @property
    def ltpq(self) -> I16Vector4: ...
    @property
    def ltpp(self) -> I16Vector4: ...
    @property
    def ltpo(self) -> I16Vector4: ...
    @property
    def ltpl(self) -> I16Vector4: ...
    @property
    def ltos(self) -> I16Vector4: ...
    @property
    def ltot(self) -> I16Vector4: ...
    @property
    def ltoq(self) -> I16Vector4: ...
    @property
    def ltop(self) -> I16Vector4: ...
    @property
    def ltoo(self) -> I16Vector4: ...
    @property
    def ltol(self) -> I16Vector4: ...
    @property
    def ltls(self) -> I16Vector4: ...
    @property
    def ltlt(self) -> I16Vector4: ...
    @property
    def ltlq(self) -> I16Vector4: ...
    @property
    def ltlp(self) -> I16Vector4: ...
    @property
    def ltlo(self) -> I16Vector4: ...
    @property
    def ltll(self) -> I16Vector4: ...
    @property
    def lqss(self) -> I16Vector4: ...
    @property
    def lqst(self) -> I16Vector4: ...
    @property
    def lqsq(self) -> I16Vector4: ...
    @property
    def lqsp(self) -> I16Vector4: ...
    @property
    def lqso(self) -> I16Vector4: ...
    @property
    def lqsl(self) -> I16Vector4: ...
    @property
    def lqts(self) -> I16Vector4: ...
    @property
    def lqtt(self) -> I16Vector4: ...
    @property
    def lqtq(self) -> I16Vector4: ...
    @property
    def lqtp(self) -> I16Vector4: ...
    @property
    def lqto(self) -> I16Vector4: ...
    @property
    def lqtl(self) -> I16Vector4: ...
    @property
    def lqqs(self) -> I16Vector4: ...
    @property
    def lqqt(self) -> I16Vector4: ...
    @property
    def lqqq(self) -> I16Vector4: ...
    @property
    def lqqp(self) -> I16Vector4: ...
    @property
    def lqqo(self) -> I16Vector4: ...
    @property
    def lqql(self) -> I16Vector4: ...
    @property
    def lqps(self) -> I16Vector4: ...
    @property
    def lqpt(self) -> I16Vector4: ...
    @property
    def lqpq(self) -> I16Vector4: ...
    @property
    def lqpp(self) -> I16Vector4: ...
    @property
    def lqpo(self) -> I16Vector4: ...
    @property
    def lqpl(self) -> I16Vector4: ...
    @property
    def lqos(self) -> I16Vector4: ...
    @property
    def lqot(self) -> I16Vector4: ...
    @property
    def lqoq(self) -> I16Vector4: ...
    @property
    def lqop(self) -> I16Vector4: ...
    @property
    def lqoo(self) -> I16Vector4: ...
    @property
    def lqol(self) -> I16Vector4: ...
    @property
    def lqls(self) -> I16Vector4: ...
    @property
    def lqlt(self) -> I16Vector4: ...
    @property
    def lqlq(self) -> I16Vector4: ...
    @property
    def lqlp(self) -> I16Vector4: ...
    @property
    def lqlo(self) -> I16Vector4: ...
    @property
    def lqll(self) -> I16Vector4: ...
    @property
    def lpss(self) -> I16Vector4: ...
    @property
    def lpst(self) -> I16Vector4: ...
    @property
    def lpsq(self) -> I16Vector4: ...
    @property
    def lpsp(self) -> I16Vector4: ...
    @property
    def lpso(self) -> I16Vector4: ...
    @property
    def lpsl(self) -> I16Vector4: ...
    @property
    def lpts(self) -> I16Vector4: ...
    @property
    def lptt(self) -> I16Vector4: ...
    @property
    def lptq(self) -> I16Vector4: ...
    @property
    def lptp(self) -> I16Vector4: ...
    @property
    def lpto(self) -> I16Vector4: ...
    @property
    def lptl(self) -> I16Vector4: ...
    @property
    def lpqs(self) -> I16Vector4: ...
    @property
    def lpqt(self) -> I16Vector4: ...
    @property
    def lpqq(self) -> I16Vector4: ...
    @property
    def lpqp(self) -> I16Vector4: ...
    @property
    def lpqo(self) -> I16Vector4: ...
    @property
    def lpql(self) -> I16Vector4: ...
    @property
    def lpps(self) -> I16Vector4: ...
    @property
    def lppt(self) -> I16Vector4: ...
    @property
    def lppq(self) -> I16Vector4: ...
    @property
    def lppp(self) -> I16Vector4: ...
    @property
    def lppo(self) -> I16Vector4: ...
    @property
    def lppl(self) -> I16Vector4: ...
    @property
    def lpos(self) -> I16Vector4: ...
    @property
    def lpot(self) -> I16Vector4: ...
    @property
    def lpoq(self) -> I16Vector4: ...
    @property
    def lpop(self) -> I16Vector4: ...
    @property
    def lpoo(self) -> I16Vector4: ...
    @property
    def lpol(self) -> I16Vector4: ...
    @property
    def lpls(self) -> I16Vector4: ...
    @property
    def lplt(self) -> I16Vector4: ...
    @property
    def lplq(self) -> I16Vector4: ...
    @property
    def lplp(self) -> I16Vector4: ...
    @property
    def lplo(self) -> I16Vector4: ...
    @property
    def lpll(self) -> I16Vector4: ...
    @property
    def loss(self) -> I16Vector4: ...
    @property
    def lost(self) -> I16Vector4: ...
    @property
    def losq(self) -> I16Vector4: ...
    @property
    def losp(self) -> I16Vector4: ...
    @property
    def loso(self) -> I16Vector4: ...
    @property
    def losl(self) -> I16Vector4: ...
    @property
    def lots(self) -> I16Vector4: ...
    @property
    def lott(self) -> I16Vector4: ...
    @property
    def lotq(self) -> I16Vector4: ...
    @property
    def lotp(self) -> I16Vector4: ...
    @property
    def loto(self) -> I16Vector4: ...
    @property
    def lotl(self) -> I16Vector4: ...
    @property
    def loqs(self) -> I16Vector4: ...
    @property
    def loqt(self) -> I16Vector4: ...
    @property
    def loqq(self) -> I16Vector4: ...
    @property
    def loqp(self) -> I16Vector4: ...
    @property
    def loqo(self) -> I16Vector4: ...
    @property
    def loql(self) -> I16Vector4: ...
    @property
    def lops(self) -> I16Vector4: ...
    @property
    def lopt(self) -> I16Vector4: ...
    @property
    def lopq(self) -> I16Vector4: ...
    @property
    def lopp(self) -> I16Vector4: ...
    @property
    def lopo(self) -> I16Vector4: ...
    @property
    def lopl(self) -> I16Vector4: ...
    @property
    def loos(self) -> I16Vector4: ...
    @property
    def loot(self) -> I16Vector4: ...
    @property
    def looq(self) -> I16Vector4: ...
    @property
    def loop(self) -> I16Vector4: ...
    @property
    def lols(self) -> I16Vector4: ...
    @property
    def lolt(self) -> I16Vector4: ...
    @property
    def lolq(self) -> I16Vector4: ...
    @property
    def lolp(self) -> I16Vector4: ...
    @property
    def llss(self) -> I16Vector4: ...
    @property
    def llst(self) -> I16Vector4: ...
    @property
    def llsq(self) -> I16Vector4: ...
    @property
    def llsp(self) -> I16Vector4: ...
    @property
    def llso(self) -> I16Vector4: ...
    @property
    def llsl(self) -> I16Vector4: ...
    @property
    def llts(self) -> I16Vector4: ...
    @property
    def lltt(self) -> I16Vector4: ...
    @property
    def lltq(self) -> I16Vector4: ...
    @property
    def lltp(self) -> I16Vector4: ...
    @property
    def llto(self) -> I16Vector4: ...
    @property
    def lltl(self) -> I16Vector4: ...
    @property
    def llqs(self) -> I16Vector4: ...
    @property
    def llqt(self) -> I16Vector4: ...
    @property
    def llqq(self) -> I16Vector4: ...
    @property
    def llqp(self) -> I16Vector4: ...
    @property
    def llqo(self) -> I16Vector4: ...
    @property
    def llql(self) -> I16Vector4: ...
    @property
    def llps(self) -> I16Vector4: ...
    @property
    def llpt(self) -> I16Vector4: ...
    @property
    def llpq(self) -> I16Vector4: ...
    @property
    def llpp(self) -> I16Vector4: ...
    @property
    def llpo(self) -> I16Vector4: ...
    @property
    def llpl(self) -> I16Vector4: ...
    @property
    def llos(self) -> I16Vector4: ...
    @property
    def llot(self) -> I16Vector4: ...
    @property
    def lloq(self) -> I16Vector4: ...
    @property
    def llop(self) -> I16Vector4: ...
    @property
    def llls(self) -> I16Vector4: ...
    @property
    def lllt(self) -> I16Vector4: ...
    @property
    def lllq(self) -> I16Vector4: ...
    @property
    def lllp(self) -> I16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I16Vector2: ...
    @property
    def uv(self) -> I16Vector2: ...
    @property
    def uo(self) -> I16Vector2: ...
    @property
    def ul(self) -> I16Vector2: ...
    @property
    def vu(self) -> I16Vector2: ...
    @property
    def vv(self) -> I16Vector2: ...
    @property
    def vo(self) -> I16Vector2: ...
    @property
    def vl(self) -> I16Vector2: ...
    @property
    def ou(self) -> I16Vector2: ...
    @property
    def ov(self) -> I16Vector2: ...
    @property
    def lu(self) -> I16Vector2: ...
    @property
    def lv(self) -> I16Vector2: ...
    @property
    def uuu(self) -> I16Vector3: ...
    @property
    def uuv(self) -> I16Vector3: ...
    @property
    def uuo(self) -> I16Vector3: ...
    @property
    def uul(self) -> I16Vector3: ...
    @property
    def uvu(self) -> I16Vector3: ...
    @property
    def uvv(self) -> I16Vector3: ...
    @property
    def uvo(self) -> I16Vector3: ...
    @property
    def uvl(self) -> I16Vector3: ...
    @property
    def uou(self) -> I16Vector3: ...
    @property
    def uov(self) -> I16Vector3: ...
    @property
    def uoo(self) -> I16Vector3: ...
    @property
    def uol(self) -> I16Vector3: ...
    @property
    def ulu(self) -> I16Vector3: ...
    @property
    def ulv(self) -> I16Vector3: ...
    @property
    def ulo(self) -> I16Vector3: ...
    @property
    def ull(self) -> I16Vector3: ...
    @property
    def vuu(self) -> I16Vector3: ...
    @property
    def vuv(self) -> I16Vector3: ...
    @property
    def vuo(self) -> I16Vector3: ...
    @property
    def vul(self) -> I16Vector3: ...
    @property
    def vvu(self) -> I16Vector3: ...
    @property
    def vvv(self) -> I16Vector3: ...
    @property
    def vvo(self) -> I16Vector3: ...
    @property
    def vvl(self) -> I16Vector3: ...
    @property
    def vou(self) -> I16Vector3: ...
    @property
    def vov(self) -> I16Vector3: ...
    @property
    def voo(self) -> I16Vector3: ...
    @property
    def vol(self) -> I16Vector3: ...
    @property
    def vlu(self) -> I16Vector3: ...
    @property
    def vlv(self) -> I16Vector3: ...
    @property
    def vlo(self) -> I16Vector3: ...
    @property
    def vll(self) -> I16Vector3: ...
    @property
    def ouu(self) -> I16Vector3: ...
    @property
    def ouv(self) -> I16Vector3: ...
    @property
    def ouo(self) -> I16Vector3: ...
    @property
    def oul(self) -> I16Vector3: ...
    @property
    def ovu(self) -> I16Vector3: ...
    @property
    def ovv(self) -> I16Vector3: ...
    @property
    def ovo(self) -> I16Vector3: ...
    @property
    def ovl(self) -> I16Vector3: ...
    @property
    def oou(self) -> I16Vector3: ...
    @property
    def oov(self) -> I16Vector3: ...
    @property
    def olu(self) -> I16Vector3: ...
    @property
    def olv(self) -> I16Vector3: ...
    @property
    def luu(self) -> I16Vector3: ...
    @property
    def luv(self) -> I16Vector3: ...
    @property
    def luo(self) -> I16Vector3: ...
    @property
    def lul(self) -> I16Vector3: ...
    @property
    def lvu(self) -> I16Vector3: ...
    @property
    def lvv(self) -> I16Vector3: ...
    @property
    def lvo(self) -> I16Vector3: ...
    @property
    def lvl(self) -> I16Vector3: ...
    @property
    def lou(self) -> I16Vector3: ...
    @property
    def lov(self) -> I16Vector3: ...
    @property
    def llu(self) -> I16Vector3: ...
    @property
    def llv(self) -> I16Vector3: ...
    @property
    def uuuu(self) -> I16Vector4: ...
    @property
    def uuuv(self) -> I16Vector4: ...
    @property
    def uuuo(self) -> I16Vector4: ...
    @property
    def uuul(self) -> I16Vector4: ...
    @property
    def uuvu(self) -> I16Vector4: ...
    @property
    def uuvv(self) -> I16Vector4: ...
    @property
    def uuvo(self) -> I16Vector4: ...
    @property
    def uuvl(self) -> I16Vector4: ...
    @property
    def uuou(self) -> I16Vector4: ...
    @property
    def uuov(self) -> I16Vector4: ...
    @property
    def uuoo(self) -> I16Vector4: ...
    @property
    def uuol(self) -> I16Vector4: ...
    @property
    def uulu(self) -> I16Vector4: ...
    @property
    def uulv(self) -> I16Vector4: ...
    @property
    def uulo(self) -> I16Vector4: ...
    @property
    def uull(self) -> I16Vector4: ...
    @property
    def uvuu(self) -> I16Vector4: ...
    @property
    def uvuv(self) -> I16Vector4: ...
    @property
    def uvuo(self) -> I16Vector4: ...
    @property
    def uvul(self) -> I16Vector4: ...
    @property
    def uvvu(self) -> I16Vector4: ...
    @property
    def uvvv(self) -> I16Vector4: ...
    @property
    def uvvo(self) -> I16Vector4: ...
    @property
    def uvvl(self) -> I16Vector4: ...
    @property
    def uvou(self) -> I16Vector4: ...
    @property
    def uvov(self) -> I16Vector4: ...
    @property
    def uvoo(self) -> I16Vector4: ...
    @property
    def uvol(self) -> I16Vector4: ...
    @property
    def uvlu(self) -> I16Vector4: ...
    @property
    def uvlv(self) -> I16Vector4: ...
    @property
    def uvlo(self) -> I16Vector4: ...
    @property
    def uvll(self) -> I16Vector4: ...
    @property
    def uouu(self) -> I16Vector4: ...
    @property
    def uouv(self) -> I16Vector4: ...
    @property
    def uouo(self) -> I16Vector4: ...
    @property
    def uoul(self) -> I16Vector4: ...
    @property
    def uovu(self) -> I16Vector4: ...
    @property
    def uovv(self) -> I16Vector4: ...
    @property
    def uovo(self) -> I16Vector4: ...
    @property
    def uovl(self) -> I16Vector4: ...
    @property
    def uoou(self) -> I16Vector4: ...
    @property
    def uoov(self) -> I16Vector4: ...
    @property
    def uooo(self) -> I16Vector4: ...
    @property
    def uool(self) -> I16Vector4: ...
    @property
    def uolu(self) -> I16Vector4: ...
    @property
    def uolv(self) -> I16Vector4: ...
    @property
    def uolo(self) -> I16Vector4: ...
    @property
    def uoll(self) -> I16Vector4: ...
    @property
    def uluu(self) -> I16Vector4: ...
    @property
    def uluv(self) -> I16Vector4: ...
    @property
    def uluo(self) -> I16Vector4: ...
    @property
    def ulul(self) -> I16Vector4: ...
    @property
    def ulvu(self) -> I16Vector4: ...
    @property
    def ulvv(self) -> I16Vector4: ...
    @property
    def ulvo(self) -> I16Vector4: ...
    @property
    def ulvl(self) -> I16Vector4: ...
    @property
    def ulou(self) -> I16Vector4: ...
    @property
    def ulov(self) -> I16Vector4: ...
    @property
    def uloo(self) -> I16Vector4: ...
    @property
    def ulol(self) -> I16Vector4: ...
    @property
    def ullu(self) -> I16Vector4: ...
    @property
    def ullv(self) -> I16Vector4: ...
    @property
    def ullo(self) -> I16Vector4: ...
    @property
    def ulll(self) -> I16Vector4: ...
    @property
    def vuuu(self) -> I16Vector4: ...
    @property
    def vuuv(self) -> I16Vector4: ...
    @property
    def vuuo(self) -> I16Vector4: ...
    @property
    def vuul(self) -> I16Vector4: ...
    @property
    def vuvu(self) -> I16Vector4: ...
    @property
    def vuvv(self) -> I16Vector4: ...
    @property
    def vuvo(self) -> I16Vector4: ...
    @property
    def vuvl(self) -> I16Vector4: ...
    @property
    def vuou(self) -> I16Vector4: ...
    @property
    def vuov(self) -> I16Vector4: ...
    @property
    def vuoo(self) -> I16Vector4: ...
    @property
    def vuol(self) -> I16Vector4: ...
    @property
    def vulu(self) -> I16Vector4: ...
    @property
    def vulv(self) -> I16Vector4: ...
    @property
    def vulo(self) -> I16Vector4: ...
    @property
    def vull(self) -> I16Vector4: ...
    @property
    def vvuu(self) -> I16Vector4: ...
    @property
    def vvuv(self) -> I16Vector4: ...
    @property
    def vvuo(self) -> I16Vector4: ...
    @property
    def vvul(self) -> I16Vector4: ...
    @property
    def vvvu(self) -> I16Vector4: ...
    @property
    def vvvv(self) -> I16Vector4: ...
    @property
    def vvvo(self) -> I16Vector4: ...
    @property
    def vvvl(self) -> I16Vector4: ...
    @property
    def vvou(self) -> I16Vector4: ...
    @property
    def vvov(self) -> I16Vector4: ...
    @property
    def vvoo(self) -> I16Vector4: ...
    @property
    def vvol(self) -> I16Vector4: ...
    @property
    def vvlu(self) -> I16Vector4: ...
    @property
    def vvlv(self) -> I16Vector4: ...
    @property
    def vvlo(self) -> I16Vector4: ...
    @property
    def vvll(self) -> I16Vector4: ...
    @property
    def vouu(self) -> I16Vector4: ...
    @property
    def vouv(self) -> I16Vector4: ...
    @property
    def vouo(self) -> I16Vector4: ...
    @property
    def voul(self) -> I16Vector4: ...
    @property
    def vovu(self) -> I16Vector4: ...
    @property
    def vovv(self) -> I16Vector4: ...
    @property
    def vovo(self) -> I16Vector4: ...
    @property
    def vovl(self) -> I16Vector4: ...
    @property
    def voou(self) -> I16Vector4: ...
    @property
    def voov(self) -> I16Vector4: ...
    @property
    def vooo(self) -> I16Vector4: ...
    @property
    def vool(self) -> I16Vector4: ...
    @property
    def volu(self) -> I16Vector4: ...
    @property
    def volv(self) -> I16Vector4: ...
    @property
    def volo(self) -> I16Vector4: ...
    @property
    def voll(self) -> I16Vector4: ...
    @property
    def vluu(self) -> I16Vector4: ...
    @property
    def vluv(self) -> I16Vector4: ...
    @property
    def vluo(self) -> I16Vector4: ...
    @property
    def vlul(self) -> I16Vector4: ...
    @property
    def vlvu(self) -> I16Vector4: ...
    @property
    def vlvv(self) -> I16Vector4: ...
    @property
    def vlvo(self) -> I16Vector4: ...
    @property
    def vlvl(self) -> I16Vector4: ...
    @property
    def vlou(self) -> I16Vector4: ...
    @property
    def vlov(self) -> I16Vector4: ...
    @property
    def vloo(self) -> I16Vector4: ...
    @property
    def vlol(self) -> I16Vector4: ...
    @property
    def vllu(self) -> I16Vector4: ...
    @property
    def vllv(self) -> I16Vector4: ...
    @property
    def vllo(self) -> I16Vector4: ...
    @property
    def vlll(self) -> I16Vector4: ...
    @property
    def ouuu(self) -> I16Vector4: ...
    @property
    def ouuv(self) -> I16Vector4: ...
    @property
    def ouuo(self) -> I16Vector4: ...
    @property
    def ouul(self) -> I16Vector4: ...
    @property
    def ouvu(self) -> I16Vector4: ...
    @property
    def ouvv(self) -> I16Vector4: ...
    @property
    def ouvo(self) -> I16Vector4: ...
    @property
    def ouvl(self) -> I16Vector4: ...
    @property
    def ouou(self) -> I16Vector4: ...
    @property
    def ouov(self) -> I16Vector4: ...
    @property
    def ouoo(self) -> I16Vector4: ...
    @property
    def ouol(self) -> I16Vector4: ...
    @property
    def oulu(self) -> I16Vector4: ...
    @property
    def oulv(self) -> I16Vector4: ...
    @property
    def oulo(self) -> I16Vector4: ...
    @property
    def oull(self) -> I16Vector4: ...
    @property
    def ovuu(self) -> I16Vector4: ...
    @property
    def ovuv(self) -> I16Vector4: ...
    @property
    def ovuo(self) -> I16Vector4: ...
    @property
    def ovul(self) -> I16Vector4: ...
    @property
    def ovvu(self) -> I16Vector4: ...
    @property
    def ovvv(self) -> I16Vector4: ...
    @property
    def ovvo(self) -> I16Vector4: ...
    @property
    def ovvl(self) -> I16Vector4: ...
    @property
    def ovou(self) -> I16Vector4: ...
    @property
    def ovov(self) -> I16Vector4: ...
    @property
    def ovoo(self) -> I16Vector4: ...
    @property
    def ovol(self) -> I16Vector4: ...
    @property
    def ovlu(self) -> I16Vector4: ...
    @property
    def ovlv(self) -> I16Vector4: ...
    @property
    def ovlo(self) -> I16Vector4: ...
    @property
    def ovll(self) -> I16Vector4: ...
    @property
    def oouu(self) -> I16Vector4: ...
    @property
    def oouv(self) -> I16Vector4: ...
    @property
    def oouo(self) -> I16Vector4: ...
    @property
    def ooul(self) -> I16Vector4: ...
    @property
    def oovu(self) -> I16Vector4: ...
    @property
    def oovv(self) -> I16Vector4: ...
    @property
    def oovo(self) -> I16Vector4: ...
    @property
    def oovl(self) -> I16Vector4: ...
    @property
    def ooou(self) -> I16Vector4: ...
    @property
    def ooov(self) -> I16Vector4: ...
    @property
    def oolu(self) -> I16Vector4: ...
    @property
    def oolv(self) -> I16Vector4: ...
    @property
    def oluu(self) -> I16Vector4: ...
    @property
    def oluv(self) -> I16Vector4: ...
    @property
    def oluo(self) -> I16Vector4: ...
    @property
    def olul(self) -> I16Vector4: ...
    @property
    def olvu(self) -> I16Vector4: ...
    @property
    def olvv(self) -> I16Vector4: ...
    @property
    def olvo(self) -> I16Vector4: ...
    @property
    def olvl(self) -> I16Vector4: ...
    @property
    def olou(self) -> I16Vector4: ...
    @property
    def olov(self) -> I16Vector4: ...
    @property
    def ollu(self) -> I16Vector4: ...
    @property
    def ollv(self) -> I16Vector4: ...
    @property
    def luuu(self) -> I16Vector4: ...
    @property
    def luuv(self) -> I16Vector4: ...
    @property
    def luuo(self) -> I16Vector4: ...
    @property
    def luul(self) -> I16Vector4: ...
    @property
    def luvu(self) -> I16Vector4: ...
    @property
    def luvv(self) -> I16Vector4: ...
    @property
    def luvo(self) -> I16Vector4: ...
    @property
    def luvl(self) -> I16Vector4: ...
    @property
    def luou(self) -> I16Vector4: ...
    @property
    def luov(self) -> I16Vector4: ...
    @property
    def luoo(self) -> I16Vector4: ...
    @property
    def luol(self) -> I16Vector4: ...
    @property
    def lulu(self) -> I16Vector4: ...
    @property
    def lulv(self) -> I16Vector4: ...
    @property
    def lulo(self) -> I16Vector4: ...
    @property
    def lull(self) -> I16Vector4: ...
    @property
    def lvuu(self) -> I16Vector4: ...
    @property
    def lvuv(self) -> I16Vector4: ...
    @property
    def lvuo(self) -> I16Vector4: ...
    @property
    def lvul(self) -> I16Vector4: ...
    @property
    def lvvu(self) -> I16Vector4: ...
    @property
    def lvvv(self) -> I16Vector4: ...
    @property
    def lvvo(self) -> I16Vector4: ...
    @property
    def lvvl(self) -> I16Vector4: ...
    @property
    def lvou(self) -> I16Vector4: ...
    @property
    def lvov(self) -> I16Vector4: ...
    @property
    def lvoo(self) -> I16Vector4: ...
    @property
    def lvol(self) -> I16Vector4: ...
    @property
    def lvlu(self) -> I16Vector4: ...
    @property
    def lvlv(self) -> I16Vector4: ...
    @property
    def lvlo(self) -> I16Vector4: ...
    @property
    def lvll(self) -> I16Vector4: ...
    @property
    def louu(self) -> I16Vector4: ...
    @property
    def louv(self) -> I16Vector4: ...
    @property
    def louo(self) -> I16Vector4: ...
    @property
    def loul(self) -> I16Vector4: ...
    @property
    def lovu(self) -> I16Vector4: ...
    @property
    def lovv(self) -> I16Vector4: ...
    @property
    def lovo(self) -> I16Vector4: ...
    @property
    def lovl(self) -> I16Vector4: ...
    @property
    def loou(self) -> I16Vector4: ...
    @property
    def loov(self) -> I16Vector4: ...
    @property
    def lolu(self) -> I16Vector4: ...
    @property
    def lolv(self) -> I16Vector4: ...
    @property
    def lluu(self) -> I16Vector4: ...
    @property
    def lluv(self) -> I16Vector4: ...
    @property
    def lluo(self) -> I16Vector4: ...
    @property
    def llul(self) -> I16Vector4: ...
    @property
    def llvu(self) -> I16Vector4: ...
    @property
    def llvv(self) -> I16Vector4: ...
    @property
    def llvo(self) -> I16Vector4: ...
    @property
    def llvl(self) -> I16Vector4: ...
    @property
    def llou(self) -> I16Vector4: ...
    @property
    def llov(self) -> I16Vector4: ...
    @property
    def lllu(self) -> I16Vector4: ...
    @property
    def lllv(self) -> I16Vector4: ...
    @property
    def oo(self) -> I16Vector2: ...
    @property
    def ol(self) -> I16Vector2: ...
    @property
    def lo(self) -> I16Vector2: ...
    @property
    def ll(self) -> I16Vector2: ...
    @property
    def ooo(self) -> I16Vector3: ...
    @property
    def ool(self) -> I16Vector3: ...
    @property
    def olo(self) -> I16Vector3: ...
    @property
    def oll(self) -> I16Vector3: ...
    @property
    def loo(self) -> I16Vector3: ...
    @property
    def lol(self) -> I16Vector3: ...
    @property
    def llo(self) -> I16Vector3: ...
    @property
    def lll(self) -> I16Vector3: ...
    @property
    def oooo(self) -> I16Vector4: ...
    @property
    def oool(self) -> I16Vector4: ...
    @property
    def oolo(self) -> I16Vector4: ...
    @property
    def ooll(self) -> I16Vector4: ...
    @property
    def oloo(self) -> I16Vector4: ...
    @property
    def olol(self) -> I16Vector4: ...
    @property
    def ollo(self) -> I16Vector4: ...
    @property
    def olll(self) -> I16Vector4: ...
    @property
    def looo(self) -> I16Vector4: ...
    @property
    def lool(self) -> I16Vector4: ...
    @property
    def lolo(self) -> I16Vector4: ...
    @property
    def loll(self) -> I16Vector4: ...
    @property
    def lloo(self) -> I16Vector4: ...
    @property
    def llol(self) -> I16Vector4: ...
    @property
    def lllo(self) -> I16Vector4: ...
    @property
    def llll(self) -> I16Vector4: ...
    def min(self, n: Number, /) -> I16Vector4: ...
    def max(self, n: Number, /) -> I16Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> I16Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I16Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector4: ...

@final
class I16Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I16Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I16Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> I16Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> I16Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I16Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[I16Vector4]: ...

@final
class U16Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __add__(self, other: Number) -> U16Vector4: ...
    @overload
    def __radd__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __radd__(self, other: Number) -> U16Vector4: ...
    @overload
    def __sub__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __sub__(self, other: Number) -> U16Vector4: ...
    @overload
    def __rsub__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> U16Vector4: ...
    @overload
    def __mul__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __mul__(self, other: Number) -> U16Vector4: ...
    @overload
    def __rmul__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> U16Vector4: ...
    @overload
    def __truediv__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> U16Vector4: ...
    @overload
    def __rtruediv__(self, other: U16Vector4) -> U16Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> U16Vector4: ...
    def __abs__(self) -> U16Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> U16Vector2: ...
    @property
    def xy(self) -> U16Vector2: ...
    @property
    def xz(self) -> U16Vector2: ...
    @property
    def xw(self) -> U16Vector2: ...
    @property
    def xo(self) -> U16Vector2: ...
    @property
    def xl(self) -> U16Vector2: ...
    @property
    def yx(self) -> U16Vector2: ...
    @property
    def yy(self) -> U16Vector2: ...
    @property
    def yz(self) -> U16Vector2: ...
    @property
    def yw(self) -> U16Vector2: ...
    @property
    def yo(self) -> U16Vector2: ...
    @property
    def yl(self) -> U16Vector2: ...
    @property
    def zx(self) -> U16Vector2: ...
    @property
    def zy(self) -> U16Vector2: ...
    @property
    def zz(self) -> U16Vector2: ...
    @property
    def zw(self) -> U16Vector2: ...
    @property
    def zo(self) -> U16Vector2: ...
    @property
    def zl(self) -> U16Vector2: ...
    @property
    def wx(self) -> U16Vector2: ...
    @property
    def wy(self) -> U16Vector2: ...
    @property
    def wz(self) -> U16Vector2: ...
    @property
    def ww(self) -> U16Vector2: ...
    @property
    def wo(self) -> U16Vector2: ...
    @property
    def wl(self) -> U16Vector2: ...
    @property
    def ox(self) -> U16Vector2: ...
    @property
    def oy(self) -> U16Vector2: ...
    @property
    def oz(self) -> U16Vector2: ...
    @property
    def ow(self) -> U16Vector2: ...
    @property
    def lx(self) -> U16Vector2: ...
    @property
    def ly(self) -> U16Vector2: ...
    @property
    def lz(self) -> U16Vector2: ...
    @property
    def lw(self) -> U16Vector2: ...
    @property
    def xxx(self) -> U16Vector3: ...
    @property
    def xxy(self) -> U16Vector3: ...
    @property
    def xxz(self) -> U16Vector3: ...
    @property
    def xxw(self) -> U16Vector3: ...
    @property
    def xxo(self) -> U16Vector3: ...
    @property
    def xxl(self) -> U16Vector3: ...
    @property
    def xyx(self) -> U16Vector3: ...
    @property
    def xyy(self) -> U16Vector3: ...
    @property
    def xyz(self) -> U16Vector3: ...
    @property
    def xyw(self) -> U16Vector3: ...
    @property
    def xyo(self) -> U16Vector3: ...
    @property
    def xyl(self) -> U16Vector3: ...
    @property
    def xzx(self) -> U16Vector3: ...
    @property
    def xzy(self) -> U16Vector3: ...
    @property
    def xzz(self) -> U16Vector3: ...
    @property
    def xzw(self) -> U16Vector3: ...
    @property
    def xzo(self) -> U16Vector3: ...
    @property
    def xzl(self) -> U16Vector3: ...
    @property
    def xwx(self) -> U16Vector3: ...
    @property
    def xwy(self) -> U16Vector3: ...
    @property
    def xwz(self) -> U16Vector3: ...
    @property
    def xww(self) -> U16Vector3: ...
    @property
    def xwo(self) -> U16Vector3: ...
    @property
    def xwl(self) -> U16Vector3: ...
    @property
    def xox(self) -> U16Vector3: ...
    @property
    def xoy(self) -> U16Vector3: ...
    @property
    def xoz(self) -> U16Vector3: ...
    @property
    def xow(self) -> U16Vector3: ...
    @property
    def xoo(self) -> U16Vector3: ...
    @property
    def xol(self) -> U16Vector3: ...
    @property
    def xlx(self) -> U16Vector3: ...
    @property
    def xly(self) -> U16Vector3: ...
    @property
    def xlz(self) -> U16Vector3: ...
    @property
    def xlw(self) -> U16Vector3: ...
    @property
    def xlo(self) -> U16Vector3: ...
    @property
    def xll(self) -> U16Vector3: ...
    @property
    def yxx(self) -> U16Vector3: ...
    @property
    def yxy(self) -> U16Vector3: ...
    @property
    def yxz(self) -> U16Vector3: ...
    @property
    def yxw(self) -> U16Vector3: ...
    @property
    def yxo(self) -> U16Vector3: ...
    @property
    def yxl(self) -> U16Vector3: ...
    @property
    def yyx(self) -> U16Vector3: ...
    @property
    def yyy(self) -> U16Vector3: ...
    @property
    def yyz(self) -> U16Vector3: ...
    @property
    def yyw(self) -> U16Vector3: ...
    @property
    def yyo(self) -> U16Vector3: ...
    @property
    def yyl(self) -> U16Vector3: ...
    @property
    def yzx(self) -> U16Vector3: ...
    @property
    def yzy(self) -> U16Vector3: ...
    @property
    def yzz(self) -> U16Vector3: ...
    @property
    def yzw(self) -> U16Vector3: ...
    @property
    def yzo(self) -> U16Vector3: ...
    @property
    def yzl(self) -> U16Vector3: ...
    @property
    def ywx(self) -> U16Vector3: ...
    @property
    def ywy(self) -> U16Vector3: ...
    @property
    def ywz(self) -> U16Vector3: ...
    @property
    def yww(self) -> U16Vector3: ...
    @property
    def ywo(self) -> U16Vector3: ...
    @property
    def ywl(self) -> U16Vector3: ...
    @property
    def yox(self) -> U16Vector3: ...
    @property
    def yoy(self) -> U16Vector3: ...
    @property
    def yoz(self) -> U16Vector3: ...
    @property
    def yow(self) -> U16Vector3: ...
    @property
    def yoo(self) -> U16Vector3: ...
    @property
    def yol(self) -> U16Vector3: ...
    @property
    def ylx(self) -> U16Vector3: ...
    @property
    def yly(self) -> U16Vector3: ...
    @property
    def ylz(self) -> U16Vector3: ...
    @property
    def ylw(self) -> U16Vector3: ...
    @property
    def ylo(self) -> U16Vector3: ...
    @property
    def yll(self) -> U16Vector3: ...
    @property
    def zxx(self) -> U16Vector3: ...
    @property
    def zxy(self) -> U16Vector3: ...
    @property
    def zxz(self) -> U16Vector3: ...
    @property
    def zxw(self) -> U16Vector3: ...
    @property
    def zxo(self) -> U16Vector3: ...
    @property
    def zxl(self) -> U16Vector3: ...
    @property
    def zyx(self) -> U16Vector3: ...
    @property
    def zyy(self) -> U16Vector3: ...
    @property
    def zyz(self) -> U16Vector3: ...
    @property
    def zyw(self) -> U16Vector3: ...
    @property
    def zyo(self) -> U16Vector3: ...
    @property
    def zyl(self) -> U16Vector3: ...
    @property
    def zzx(self) -> U16Vector3: ...
    @property
    def zzy(self) -> U16Vector3: ...
    @property
    def zzz(self) -> U16Vector3: ...
    @property
    def zzw(self) -> U16Vector3: ...
    @property
    def zzo(self) -> U16Vector3: ...
    @property
    def zzl(self) -> U16Vector3: ...
    @property
    def zwx(self) -> U16Vector3: ...
    @property
    def zwy(self) -> U16Vector3: ...
    @property
    def zwz(self) -> U16Vector3: ...
    @property
    def zww(self) -> U16Vector3: ...
    @property
    def zwo(self) -> U16Vector3: ...
    @property
    def zwl(self) -> U16Vector3: ...
    @property
    def zox(self) -> U16Vector3: ...
    @property
    def zoy(self) -> U16Vector3: ...
    @property
    def zoz(self) -> U16Vector3: ...
    @property
    def zow(self) -> U16Vector3: ...
    @property
    def zoo(self) -> U16Vector3: ...
    @property
    def zol(self) -> U16Vector3: ...
    @property
    def zlx(self) -> U16Vector3: ...
    @property
    def zly(self) -> U16Vector3: ...
    @property
    def zlz(self) -> U16Vector3: ...
    @property
    def zlw(self) -> U16Vector3: ...
    @property
    def zlo(self) -> U16Vector3: ...
    @property
    def zll(self) -> U16Vector3: ...
    @property
    def wxx(self) -> U16Vector3: ...
    @property
    def wxy(self) -> U16Vector3: ...
    @property
    def wxz(self) -> U16Vector3: ...
    @property
    def wxw(self) -> U16Vector3: ...
    @property
    def wxo(self) -> U16Vector3: ...
    @property
    def wxl(self) -> U16Vector3: ...
    @property
    def wyx(self) -> U16Vector3: ...
    @property
    def wyy(self) -> U16Vector3: ...
    @property
    def wyz(self) -> U16Vector3: ...
    @property
    def wyw(self) -> U16Vector3: ...
    @property
    def wyo(self) -> U16Vector3: ...
    @property
    def wyl(self) -> U16Vector3: ...
    @property
    def wzx(self) -> U16Vector3: ...
    @property
    def wzy(self) -> U16Vector3: ...
    @property
    def wzz(self) -> U16Vector3: ...
    @property
    def wzw(self) -> U16Vector3: ...
    @property
    def wzo(self) -> U16Vector3: ...
    @property
    def wzl(self) -> U16Vector3: ...
    @property
    def wwx(self) -> U16Vector3: ...
    @property
    def wwy(self) -> U16Vector3: ...
    @property
    def wwz(self) -> U16Vector3: ...
    @property
    def www(self) -> U16Vector3: ...
    @property
    def wwo(self) -> U16Vector3: ...
    @property
    def wwl(self) -> U16Vector3: ...
    @property
    def wox(self) -> U16Vector3: ...
    @property
    def woy(self) -> U16Vector3: ...
    @property
    def woz(self) -> U16Vector3: ...
    @property
    def wow(self) -> U16Vector3: ...
    @property
    def woo(self) -> U16Vector3: ...
    @property
    def wol(self) -> U16Vector3: ...
    @property
    def wlx(self) -> U16Vector3: ...
    @property
    def wly(self) -> U16Vector3: ...
    @property
    def wlz(self) -> U16Vector3: ...
    @property
    def wlw(self) -> U16Vector3: ...
    @property
    def wlo(self) -> U16Vector3: ...
    @property
    def wll(self) -> U16Vector3: ...
    @property
    def oxx(self) -> U16Vector3: ...
    @property
    def oxy(self) -> U16Vector3: ...
    @property
    def oxz(self) -> U16Vector3: ...
    @property
    def oxw(self) -> U16Vector3: ...
    @property
    def oxo(self) -> U16Vector3: ...
    @property
    def oxl(self) -> U16Vector3: ...
    @property
    def oyx(self) -> U16Vector3: ...
    @property
    def oyy(self) -> U16Vector3: ...
    @property
    def oyz(self) -> U16Vector3: ...
    @property
    def oyw(self) -> U16Vector3: ...
    @property
    def oyo(self) -> U16Vector3: ...
    @property
    def oyl(self) -> U16Vector3: ...
    @property
    def ozx(self) -> U16Vector3: ...
    @property
    def ozy(self) -> U16Vector3: ...
    @property
    def ozz(self) -> U16Vector3: ...
    @property
    def ozw(self) -> U16Vector3: ...
    @property
    def ozo(self) -> U16Vector3: ...
    @property
    def ozl(self) -> U16Vector3: ...
    @property
    def owx(self) -> U16Vector3: ...
    @property
    def owy(self) -> U16Vector3: ...
    @property
    def owz(self) -> U16Vector3: ...
    @property
    def oww(self) -> U16Vector3: ...
    @property
    def owo(self) -> U16Vector3: ...
    @property
    def owl(self) -> U16Vector3: ...
    @property
    def oox(self) -> U16Vector3: ...
    @property
    def ooy(self) -> U16Vector3: ...
    @property
    def ooz(self) -> U16Vector3: ...
    @property
    def oow(self) -> U16Vector3: ...
    @property
    def olx(self) -> U16Vector3: ...
    @property
    def oly(self) -> U16Vector3: ...
    @property
    def olz(self) -> U16Vector3: ...
    @property
    def olw(self) -> U16Vector3: ...
    @property
    def lxx(self) -> U16Vector3: ...
    @property
    def lxy(self) -> U16Vector3: ...
    @property
    def lxz(self) -> U16Vector3: ...
    @property
    def lxw(self) -> U16Vector3: ...
    @property
    def lxo(self) -> U16Vector3: ...
    @property
    def lxl(self) -> U16Vector3: ...
    @property
    def lyx(self) -> U16Vector3: ...
    @property
    def lyy(self) -> U16Vector3: ...
    @property
    def lyz(self) -> U16Vector3: ...
    @property
    def lyw(self) -> U16Vector3: ...
    @property
    def lyo(self) -> U16Vector3: ...
    @property
    def lyl(self) -> U16Vector3: ...
    @property
    def lzx(self) -> U16Vector3: ...
    @property
    def lzy(self) -> U16Vector3: ...
    @property
    def lzz(self) -> U16Vector3: ...
    @property
    def lzw(self) -> U16Vector3: ...
    @property
    def lzo(self) -> U16Vector3: ...
    @property
    def lzl(self) -> U16Vector3: ...
    @property
    def lwx(self) -> U16Vector3: ...
    @property
    def lwy(self) -> U16Vector3: ...
    @property
    def lwz(self) -> U16Vector3: ...
    @property
    def lww(self) -> U16Vector3: ...
    @property
    def lwo(self) -> U16Vector3: ...
    @property
    def lwl(self) -> U16Vector3: ...
    @property
    def lox(self) -> U16Vector3: ...
    @property
    def loy(self) -> U16Vector3: ...
    @property
    def loz(self) -> U16Vector3: ...
    @property
    def low(self) -> U16Vector3: ...
    @property
    def llx(self) -> U16Vector3: ...
    @property
    def lly(self) -> U16Vector3: ...
    @property
    def llz(self) -> U16Vector3: ...
    @property
    def llw(self) -> U16Vector3: ...
    @property
    def xxxx(self) -> U16Vector4: ...
    @property
    def xxxy(self) -> U16Vector4: ...
    @property
    def xxxz(self) -> U16Vector4: ...
    @property
    def xxxw(self) -> U16Vector4: ...
    @property
    def xxxo(self) -> U16Vector4: ...
    @property
    def xxxl(self) -> U16Vector4: ...
    @property
    def xxyx(self) -> U16Vector4: ...
    @property
    def xxyy(self) -> U16Vector4: ...
    @property
    def xxyz(self) -> U16Vector4: ...
    @property
    def xxyw(self) -> U16Vector4: ...
    @property
    def xxyo(self) -> U16Vector4: ...
    @property
    def xxyl(self) -> U16Vector4: ...
    @property
    def xxzx(self) -> U16Vector4: ...
    @property
    def xxzy(self) -> U16Vector4: ...
    @property
    def xxzz(self) -> U16Vector4: ...
    @property
    def xxzw(self) -> U16Vector4: ...
    @property
    def xxzo(self) -> U16Vector4: ...
    @property
    def xxzl(self) -> U16Vector4: ...
    @property
    def xxwx(self) -> U16Vector4: ...
    @property
    def xxwy(self) -> U16Vector4: ...
    @property
    def xxwz(self) -> U16Vector4: ...
    @property
    def xxww(self) -> U16Vector4: ...
    @property
    def xxwo(self) -> U16Vector4: ...
    @property
    def xxwl(self) -> U16Vector4: ...
    @property
    def xxox(self) -> U16Vector4: ...
    @property
    def xxoy(self) -> U16Vector4: ...
    @property
    def xxoz(self) -> U16Vector4: ...
    @property
    def xxow(self) -> U16Vector4: ...
    @property
    def xxoo(self) -> U16Vector4: ...
    @property
    def xxol(self) -> U16Vector4: ...
    @property
    def xxlx(self) -> U16Vector4: ...
    @property
    def xxly(self) -> U16Vector4: ...
    @property
    def xxlz(self) -> U16Vector4: ...
    @property
    def xxlw(self) -> U16Vector4: ...
    @property
    def xxlo(self) -> U16Vector4: ...
    @property
    def xxll(self) -> U16Vector4: ...
    @property
    def xyxx(self) -> U16Vector4: ...
    @property
    def xyxy(self) -> U16Vector4: ...
    @property
    def xyxz(self) -> U16Vector4: ...
    @property
    def xyxw(self) -> U16Vector4: ...
    @property
    def xyxo(self) -> U16Vector4: ...
    @property
    def xyxl(self) -> U16Vector4: ...
    @property
    def xyyx(self) -> U16Vector4: ...
    @property
    def xyyy(self) -> U16Vector4: ...
    @property
    def xyyz(self) -> U16Vector4: ...
    @property
    def xyyw(self) -> U16Vector4: ...
    @property
    def xyyo(self) -> U16Vector4: ...
    @property
    def xyyl(self) -> U16Vector4: ...
    @property
    def xyzx(self) -> U16Vector4: ...
    @property
    def xyzy(self) -> U16Vector4: ...
    @property
    def xyzz(self) -> U16Vector4: ...
    @property
    def xyzw(self) -> U16Vector4: ...
    @property
    def xyzo(self) -> U16Vector4: ...
    @property
    def xyzl(self) -> U16Vector4: ...
    @property
    def xywx(self) -> U16Vector4: ...
    @property
    def xywy(self) -> U16Vector4: ...
    @property
    def xywz(self) -> U16Vector4: ...
    @property
    def xyww(self) -> U16Vector4: ...
    @property
    def xywo(self) -> U16Vector4: ...
    @property
    def xywl(self) -> U16Vector4: ...
    @property
    def xyox(self) -> U16Vector4: ...
    @property
    def xyoy(self) -> U16Vector4: ...
    @property
    def xyoz(self) -> U16Vector4: ...
    @property
    def xyow(self) -> U16Vector4: ...
    @property
    def xyoo(self) -> U16Vector4: ...
    @property
    def xyol(self) -> U16Vector4: ...
    @property
    def xylx(self) -> U16Vector4: ...
    @property
    def xyly(self) -> U16Vector4: ...
    @property
    def xylz(self) -> U16Vector4: ...
    @property
    def xylw(self) -> U16Vector4: ...
    @property
    def xylo(self) -> U16Vector4: ...
    @property
    def xyll(self) -> U16Vector4: ...
    @property
    def xzxx(self) -> U16Vector4: ...
    @property
    def xzxy(self) -> U16Vector4: ...
    @property
    def xzxz(self) -> U16Vector4: ...
    @property
    def xzxw(self) -> U16Vector4: ...
    @property
    def xzxo(self) -> U16Vector4: ...
    @property
    def xzxl(self) -> U16Vector4: ...
    @property
    def xzyx(self) -> U16Vector4: ...
    @property
    def xzyy(self) -> U16Vector4: ...
    @property
    def xzyz(self) -> U16Vector4: ...
    @property
    def xzyw(self) -> U16Vector4: ...
    @property
    def xzyo(self) -> U16Vector4: ...
    @property
    def xzyl(self) -> U16Vector4: ...
    @property
    def xzzx(self) -> U16Vector4: ...
    @property
    def xzzy(self) -> U16Vector4: ...
    @property
    def xzzz(self) -> U16Vector4: ...
    @property
    def xzzw(self) -> U16Vector4: ...
    @property
    def xzzo(self) -> U16Vector4: ...
    @property
    def xzzl(self) -> U16Vector4: ...
    @property
    def xzwx(self) -> U16Vector4: ...
    @property
    def xzwy(self) -> U16Vector4: ...
    @property
    def xzwz(self) -> U16Vector4: ...
    @property
    def xzww(self) -> U16Vector4: ...
    @property
    def xzwo(self) -> U16Vector4: ...
    @property
    def xzwl(self) -> U16Vector4: ...
    @property
    def xzox(self) -> U16Vector4: ...
    @property
    def xzoy(self) -> U16Vector4: ...
    @property
    def xzoz(self) -> U16Vector4: ...
    @property
    def xzow(self) -> U16Vector4: ...
    @property
    def xzoo(self) -> U16Vector4: ...
    @property
    def xzol(self) -> U16Vector4: ...
    @property
    def xzlx(self) -> U16Vector4: ...
    @property
    def xzly(self) -> U16Vector4: ...
    @property
    def xzlz(self) -> U16Vector4: ...
    @property
    def xzlw(self) -> U16Vector4: ...
    @property
    def xzlo(self) -> U16Vector4: ...
    @property
    def xzll(self) -> U16Vector4: ...
    @property
    def xwxx(self) -> U16Vector4: ...
    @property
    def xwxy(self) -> U16Vector4: ...
    @property
    def xwxz(self) -> U16Vector4: ...
    @property
    def xwxw(self) -> U16Vector4: ...
    @property
    def xwxo(self) -> U16Vector4: ...
    @property
    def xwxl(self) -> U16Vector4: ...
    @property
    def xwyx(self) -> U16Vector4: ...
    @property
    def xwyy(self) -> U16Vector4: ...
    @property
    def xwyz(self) -> U16Vector4: ...
    @property
    def xwyw(self) -> U16Vector4: ...
    @property
    def xwyo(self) -> U16Vector4: ...
    @property
    def xwyl(self) -> U16Vector4: ...
    @property
    def xwzx(self) -> U16Vector4: ...
    @property
    def xwzy(self) -> U16Vector4: ...
    @property
    def xwzz(self) -> U16Vector4: ...
    @property
    def xwzw(self) -> U16Vector4: ...
    @property
    def xwzo(self) -> U16Vector4: ...
    @property
    def xwzl(self) -> U16Vector4: ...
    @property
    def xwwx(self) -> U16Vector4: ...
    @property
    def xwwy(self) -> U16Vector4: ...
    @property
    def xwwz(self) -> U16Vector4: ...
    @property
    def xwww(self) -> U16Vector4: ...
    @property
    def xwwo(self) -> U16Vector4: ...
    @property
    def xwwl(self) -> U16Vector4: ...
    @property
    def xwox(self) -> U16Vector4: ...
    @property
    def xwoy(self) -> U16Vector4: ...
    @property
    def xwoz(self) -> U16Vector4: ...
    @property
    def xwow(self) -> U16Vector4: ...
    @property
    def xwoo(self) -> U16Vector4: ...
    @property
    def xwol(self) -> U16Vector4: ...
    @property
    def xwlx(self) -> U16Vector4: ...
    @property
    def xwly(self) -> U16Vector4: ...
    @property
    def xwlz(self) -> U16Vector4: ...
    @property
    def xwlw(self) -> U16Vector4: ...
    @property
    def xwlo(self) -> U16Vector4: ...
    @property
    def xwll(self) -> U16Vector4: ...
    @property
    def xoxx(self) -> U16Vector4: ...
    @property
    def xoxy(self) -> U16Vector4: ...
    @property
    def xoxz(self) -> U16Vector4: ...
    @property
    def xoxw(self) -> U16Vector4: ...
    @property
    def xoxo(self) -> U16Vector4: ...
    @property
    def xoxl(self) -> U16Vector4: ...
    @property
    def xoyx(self) -> U16Vector4: ...
    @property
    def xoyy(self) -> U16Vector4: ...
    @property
    def xoyz(self) -> U16Vector4: ...
    @property
    def xoyw(self) -> U16Vector4: ...
    @property
    def xoyo(self) -> U16Vector4: ...
    @property
    def xoyl(self) -> U16Vector4: ...
    @property
    def xozx(self) -> U16Vector4: ...
    @property
    def xozy(self) -> U16Vector4: ...
    @property
    def xozz(self) -> U16Vector4: ...
    @property
    def xozw(self) -> U16Vector4: ...
    @property
    def xozo(self) -> U16Vector4: ...
    @property
    def xozl(self) -> U16Vector4: ...
    @property
    def xowx(self) -> U16Vector4: ...
    @property
    def xowy(self) -> U16Vector4: ...
    @property
    def xowz(self) -> U16Vector4: ...
    @property
    def xoww(self) -> U16Vector4: ...
    @property
    def xowo(self) -> U16Vector4: ...
    @property
    def xowl(self) -> U16Vector4: ...
    @property
    def xoox(self) -> U16Vector4: ...
    @property
    def xooy(self) -> U16Vector4: ...
    @property
    def xooz(self) -> U16Vector4: ...
    @property
    def xoow(self) -> U16Vector4: ...
    @property
    def xooo(self) -> U16Vector4: ...
    @property
    def xool(self) -> U16Vector4: ...
    @property
    def xolx(self) -> U16Vector4: ...
    @property
    def xoly(self) -> U16Vector4: ...
    @property
    def xolz(self) -> U16Vector4: ...
    @property
    def xolw(self) -> U16Vector4: ...
    @property
    def xolo(self) -> U16Vector4: ...
    @property
    def xoll(self) -> U16Vector4: ...
    @property
    def xlxx(self) -> U16Vector4: ...
    @property
    def xlxy(self) -> U16Vector4: ...
    @property
    def xlxz(self) -> U16Vector4: ...
    @property
    def xlxw(self) -> U16Vector4: ...
    @property
    def xlxo(self) -> U16Vector4: ...
    @property
    def xlxl(self) -> U16Vector4: ...
    @property
    def xlyx(self) -> U16Vector4: ...
    @property
    def xlyy(self) -> U16Vector4: ...
    @property
    def xlyz(self) -> U16Vector4: ...
    @property
    def xlyw(self) -> U16Vector4: ...
    @property
    def xlyo(self) -> U16Vector4: ...
    @property
    def xlyl(self) -> U16Vector4: ...
    @property
    def xlzx(self) -> U16Vector4: ...
    @property
    def xlzy(self) -> U16Vector4: ...
    @property
    def xlzz(self) -> U16Vector4: ...
    @property
    def xlzw(self) -> U16Vector4: ...
    @property
    def xlzo(self) -> U16Vector4: ...
    @property
    def xlzl(self) -> U16Vector4: ...
    @property
    def xlwx(self) -> U16Vector4: ...
    @property
    def xlwy(self) -> U16Vector4: ...
    @property
    def xlwz(self) -> U16Vector4: ...
    @property
    def xlww(self) -> U16Vector4: ...
    @property
    def xlwo(self) -> U16Vector4: ...
    @property
    def xlwl(self) -> U16Vector4: ...
    @property
    def xlox(self) -> U16Vector4: ...
    @property
    def xloy(self) -> U16Vector4: ...
    @property
    def xloz(self) -> U16Vector4: ...
    @property
    def xlow(self) -> U16Vector4: ...
    @property
    def xloo(self) -> U16Vector4: ...
    @property
    def xlol(self) -> U16Vector4: ...
    @property
    def xllx(self) -> U16Vector4: ...
    @property
    def xlly(self) -> U16Vector4: ...
    @property
    def xllz(self) -> U16Vector4: ...
    @property
    def xllw(self) -> U16Vector4: ...
    @property
    def xllo(self) -> U16Vector4: ...
    @property
    def xlll(self) -> U16Vector4: ...
    @property
    def yxxx(self) -> U16Vector4: ...
    @property
    def yxxy(self) -> U16Vector4: ...
    @property
    def yxxz(self) -> U16Vector4: ...
    @property
    def yxxw(self) -> U16Vector4: ...
    @property
    def yxxo(self) -> U16Vector4: ...
    @property
    def yxxl(self) -> U16Vector4: ...
    @property
    def yxyx(self) -> U16Vector4: ...
    @property
    def yxyy(self) -> U16Vector4: ...
    @property
    def yxyz(self) -> U16Vector4: ...
    @property
    def yxyw(self) -> U16Vector4: ...
    @property
    def yxyo(self) -> U16Vector4: ...
    @property
    def yxyl(self) -> U16Vector4: ...
    @property
    def yxzx(self) -> U16Vector4: ...
    @property
    def yxzy(self) -> U16Vector4: ...
    @property
    def yxzz(self) -> U16Vector4: ...
    @property
    def yxzw(self) -> U16Vector4: ...
    @property
    def yxzo(self) -> U16Vector4: ...
    @property
    def yxzl(self) -> U16Vector4: ...
    @property
    def yxwx(self) -> U16Vector4: ...
    @property
    def yxwy(self) -> U16Vector4: ...
    @property
    def yxwz(self) -> U16Vector4: ...
    @property
    def yxww(self) -> U16Vector4: ...
    @property
    def yxwo(self) -> U16Vector4: ...
    @property
    def yxwl(self) -> U16Vector4: ...
    @property
    def yxox(self) -> U16Vector4: ...
    @property
    def yxoy(self) -> U16Vector4: ...
    @property
    def yxoz(self) -> U16Vector4: ...
    @property
    def yxow(self) -> U16Vector4: ...
    @property
    def yxoo(self) -> U16Vector4: ...
    @property
    def yxol(self) -> U16Vector4: ...
    @property
    def yxlx(self) -> U16Vector4: ...
    @property
    def yxly(self) -> U16Vector4: ...
    @property
    def yxlz(self) -> U16Vector4: ...
    @property
    def yxlw(self) -> U16Vector4: ...
    @property
    def yxlo(self) -> U16Vector4: ...
    @property
    def yxll(self) -> U16Vector4: ...
    @property
    def yyxx(self) -> U16Vector4: ...
    @property
    def yyxy(self) -> U16Vector4: ...
    @property
    def yyxz(self) -> U16Vector4: ...
    @property
    def yyxw(self) -> U16Vector4: ...
    @property
    def yyxo(self) -> U16Vector4: ...
    @property
    def yyxl(self) -> U16Vector4: ...
    @property
    def yyyx(self) -> U16Vector4: ...
    @property
    def yyyy(self) -> U16Vector4: ...
    @property
    def yyyz(self) -> U16Vector4: ...
    @property
    def yyyw(self) -> U16Vector4: ...
    @property
    def yyyo(self) -> U16Vector4: ...
    @property
    def yyyl(self) -> U16Vector4: ...
    @property
    def yyzx(self) -> U16Vector4: ...
    @property
    def yyzy(self) -> U16Vector4: ...
    @property
    def yyzz(self) -> U16Vector4: ...
    @property
    def yyzw(self) -> U16Vector4: ...
    @property
    def yyzo(self) -> U16Vector4: ...
    @property
    def yyzl(self) -> U16Vector4: ...
    @property
    def yywx(self) -> U16Vector4: ...
    @property
    def yywy(self) -> U16Vector4: ...
    @property
    def yywz(self) -> U16Vector4: ...
    @property
    def yyww(self) -> U16Vector4: ...
    @property
    def yywo(self) -> U16Vector4: ...
    @property
    def yywl(self) -> U16Vector4: ...
    @property
    def yyox(self) -> U16Vector4: ...
    @property
    def yyoy(self) -> U16Vector4: ...
    @property
    def yyoz(self) -> U16Vector4: ...
    @property
    def yyow(self) -> U16Vector4: ...
    @property
    def yyoo(self) -> U16Vector4: ...
    @property
    def yyol(self) -> U16Vector4: ...
    @property
    def yylx(self) -> U16Vector4: ...
    @property
    def yyly(self) -> U16Vector4: ...
    @property
    def yylz(self) -> U16Vector4: ...
    @property
    def yylw(self) -> U16Vector4: ...
    @property
    def yylo(self) -> U16Vector4: ...
    @property
    def yyll(self) -> U16Vector4: ...
    @property
    def yzxx(self) -> U16Vector4: ...
    @property
    def yzxy(self) -> U16Vector4: ...
    @property
    def yzxz(self) -> U16Vector4: ...
    @property
    def yzxw(self) -> U16Vector4: ...
    @property
    def yzxo(self) -> U16Vector4: ...
    @property
    def yzxl(self) -> U16Vector4: ...
    @property
    def yzyx(self) -> U16Vector4: ...
    @property
    def yzyy(self) -> U16Vector4: ...
    @property
    def yzyz(self) -> U16Vector4: ...
    @property
    def yzyw(self) -> U16Vector4: ...
    @property
    def yzyo(self) -> U16Vector4: ...
    @property
    def yzyl(self) -> U16Vector4: ...
    @property
    def yzzx(self) -> U16Vector4: ...
    @property
    def yzzy(self) -> U16Vector4: ...
    @property
    def yzzz(self) -> U16Vector4: ...
    @property
    def yzzw(self) -> U16Vector4: ...
    @property
    def yzzo(self) -> U16Vector4: ...
    @property
    def yzzl(self) -> U16Vector4: ...
    @property
    def yzwx(self) -> U16Vector4: ...
    @property
    def yzwy(self) -> U16Vector4: ...
    @property
    def yzwz(self) -> U16Vector4: ...
    @property
    def yzww(self) -> U16Vector4: ...
    @property
    def yzwo(self) -> U16Vector4: ...
    @property
    def yzwl(self) -> U16Vector4: ...
    @property
    def yzox(self) -> U16Vector4: ...
    @property
    def yzoy(self) -> U16Vector4: ...
    @property
    def yzoz(self) -> U16Vector4: ...
    @property
    def yzow(self) -> U16Vector4: ...
    @property
    def yzoo(self) -> U16Vector4: ...
    @property
    def yzol(self) -> U16Vector4: ...
    @property
    def yzlx(self) -> U16Vector4: ...
    @property
    def yzly(self) -> U16Vector4: ...
    @property
    def yzlz(self) -> U16Vector4: ...
    @property
    def yzlw(self) -> U16Vector4: ...
    @property
    def yzlo(self) -> U16Vector4: ...
    @property
    def yzll(self) -> U16Vector4: ...
    @property
    def ywxx(self) -> U16Vector4: ...
    @property
    def ywxy(self) -> U16Vector4: ...
    @property
    def ywxz(self) -> U16Vector4: ...
    @property
    def ywxw(self) -> U16Vector4: ...
    @property
    def ywxo(self) -> U16Vector4: ...
    @property
    def ywxl(self) -> U16Vector4: ...
    @property
    def ywyx(self) -> U16Vector4: ...
    @property
    def ywyy(self) -> U16Vector4: ...
    @property
    def ywyz(self) -> U16Vector4: ...
    @property
    def ywyw(self) -> U16Vector4: ...
    @property
    def ywyo(self) -> U16Vector4: ...
    @property
    def ywyl(self) -> U16Vector4: ...
    @property
    def ywzx(self) -> U16Vector4: ...
    @property
    def ywzy(self) -> U16Vector4: ...
    @property
    def ywzz(self) -> U16Vector4: ...
    @property
    def ywzw(self) -> U16Vector4: ...
    @property
    def ywzo(self) -> U16Vector4: ...
    @property
    def ywzl(self) -> U16Vector4: ...
    @property
    def ywwx(self) -> U16Vector4: ...
    @property
    def ywwy(self) -> U16Vector4: ...
    @property
    def ywwz(self) -> U16Vector4: ...
    @property
    def ywww(self) -> U16Vector4: ...
    @property
    def ywwo(self) -> U16Vector4: ...
    @property
    def ywwl(self) -> U16Vector4: ...
    @property
    def ywox(self) -> U16Vector4: ...
    @property
    def ywoy(self) -> U16Vector4: ...
    @property
    def ywoz(self) -> U16Vector4: ...
    @property
    def ywow(self) -> U16Vector4: ...
    @property
    def ywoo(self) -> U16Vector4: ...
    @property
    def ywol(self) -> U16Vector4: ...
    @property
    def ywlx(self) -> U16Vector4: ...
    @property
    def ywly(self) -> U16Vector4: ...
    @property
    def ywlz(self) -> U16Vector4: ...
    @property
    def ywlw(self) -> U16Vector4: ...
    @property
    def ywlo(self) -> U16Vector4: ...
    @property
    def ywll(self) -> U16Vector4: ...
    @property
    def yoxx(self) -> U16Vector4: ...
    @property
    def yoxy(self) -> U16Vector4: ...
    @property
    def yoxz(self) -> U16Vector4: ...
    @property
    def yoxw(self) -> U16Vector4: ...
    @property
    def yoxo(self) -> U16Vector4: ...
    @property
    def yoxl(self) -> U16Vector4: ...
    @property
    def yoyx(self) -> U16Vector4: ...
    @property
    def yoyy(self) -> U16Vector4: ...
    @property
    def yoyz(self) -> U16Vector4: ...
    @property
    def yoyw(self) -> U16Vector4: ...
    @property
    def yoyo(self) -> U16Vector4: ...
    @property
    def yoyl(self) -> U16Vector4: ...
    @property
    def yozx(self) -> U16Vector4: ...
    @property
    def yozy(self) -> U16Vector4: ...
    @property
    def yozz(self) -> U16Vector4: ...
    @property
    def yozw(self) -> U16Vector4: ...
    @property
    def yozo(self) -> U16Vector4: ...
    @property
    def yozl(self) -> U16Vector4: ...
    @property
    def yowx(self) -> U16Vector4: ...
    @property
    def yowy(self) -> U16Vector4: ...
    @property
    def yowz(self) -> U16Vector4: ...
    @property
    def yoww(self) -> U16Vector4: ...
    @property
    def yowo(self) -> U16Vector4: ...
    @property
    def yowl(self) -> U16Vector4: ...
    @property
    def yoox(self) -> U16Vector4: ...
    @property
    def yooy(self) -> U16Vector4: ...
    @property
    def yooz(self) -> U16Vector4: ...
    @property
    def yoow(self) -> U16Vector4: ...
    @property
    def yooo(self) -> U16Vector4: ...
    @property
    def yool(self) -> U16Vector4: ...
    @property
    def yolx(self) -> U16Vector4: ...
    @property
    def yoly(self) -> U16Vector4: ...
    @property
    def yolz(self) -> U16Vector4: ...
    @property
    def yolw(self) -> U16Vector4: ...
    @property
    def yolo(self) -> U16Vector4: ...
    @property
    def yoll(self) -> U16Vector4: ...
    @property
    def ylxx(self) -> U16Vector4: ...
    @property
    def ylxy(self) -> U16Vector4: ...
    @property
    def ylxz(self) -> U16Vector4: ...
    @property
    def ylxw(self) -> U16Vector4: ...
    @property
    def ylxo(self) -> U16Vector4: ...
    @property
    def ylxl(self) -> U16Vector4: ...
    @property
    def ylyx(self) -> U16Vector4: ...
    @property
    def ylyy(self) -> U16Vector4: ...
    @property
    def ylyz(self) -> U16Vector4: ...
    @property
    def ylyw(self) -> U16Vector4: ...
    @property
    def ylyo(self) -> U16Vector4: ...
    @property
    def ylyl(self) -> U16Vector4: ...
    @property
    def ylzx(self) -> U16Vector4: ...
    @property
    def ylzy(self) -> U16Vector4: ...
    @property
    def ylzz(self) -> U16Vector4: ...
    @property
    def ylzw(self) -> U16Vector4: ...
    @property
    def ylzo(self) -> U16Vector4: ...
    @property
    def ylzl(self) -> U16Vector4: ...
    @property
    def ylwx(self) -> U16Vector4: ...
    @property
    def ylwy(self) -> U16Vector4: ...
    @property
    def ylwz(self) -> U16Vector4: ...
    @property
    def ylww(self) -> U16Vector4: ...
    @property
    def ylwo(self) -> U16Vector4: ...
    @property
    def ylwl(self) -> U16Vector4: ...
    @property
    def ylox(self) -> U16Vector4: ...
    @property
    def yloy(self) -> U16Vector4: ...
    @property
    def yloz(self) -> U16Vector4: ...
    @property
    def ylow(self) -> U16Vector4: ...
    @property
    def yloo(self) -> U16Vector4: ...
    @property
    def ylol(self) -> U16Vector4: ...
    @property
    def yllx(self) -> U16Vector4: ...
    @property
    def ylly(self) -> U16Vector4: ...
    @property
    def yllz(self) -> U16Vector4: ...
    @property
    def yllw(self) -> U16Vector4: ...
    @property
    def yllo(self) -> U16Vector4: ...
    @property
    def ylll(self) -> U16Vector4: ...
    @property
    def zxxx(self) -> U16Vector4: ...
    @property
    def zxxy(self) -> U16Vector4: ...
    @property
    def zxxz(self) -> U16Vector4: ...
    @property
    def zxxw(self) -> U16Vector4: ...
    @property
    def zxxo(self) -> U16Vector4: ...
    @property
    def zxxl(self) -> U16Vector4: ...
    @property
    def zxyx(self) -> U16Vector4: ...
    @property
    def zxyy(self) -> U16Vector4: ...
    @property
    def zxyz(self) -> U16Vector4: ...
    @property
    def zxyw(self) -> U16Vector4: ...
    @property
    def zxyo(self) -> U16Vector4: ...
    @property
    def zxyl(self) -> U16Vector4: ...
    @property
    def zxzx(self) -> U16Vector4: ...
    @property
    def zxzy(self) -> U16Vector4: ...
    @property
    def zxzz(self) -> U16Vector4: ...
    @property
    def zxzw(self) -> U16Vector4: ...
    @property
    def zxzo(self) -> U16Vector4: ...
    @property
    def zxzl(self) -> U16Vector4: ...
    @property
    def zxwx(self) -> U16Vector4: ...
    @property
    def zxwy(self) -> U16Vector4: ...
    @property
    def zxwz(self) -> U16Vector4: ...
    @property
    def zxww(self) -> U16Vector4: ...
    @property
    def zxwo(self) -> U16Vector4: ...
    @property
    def zxwl(self) -> U16Vector4: ...
    @property
    def zxox(self) -> U16Vector4: ...
    @property
    def zxoy(self) -> U16Vector4: ...
    @property
    def zxoz(self) -> U16Vector4: ...
    @property
    def zxow(self) -> U16Vector4: ...
    @property
    def zxoo(self) -> U16Vector4: ...
    @property
    def zxol(self) -> U16Vector4: ...
    @property
    def zxlx(self) -> U16Vector4: ...
    @property
    def zxly(self) -> U16Vector4: ...
    @property
    def zxlz(self) -> U16Vector4: ...
    @property
    def zxlw(self) -> U16Vector4: ...
    @property
    def zxlo(self) -> U16Vector4: ...
    @property
    def zxll(self) -> U16Vector4: ...
    @property
    def zyxx(self) -> U16Vector4: ...
    @property
    def zyxy(self) -> U16Vector4: ...
    @property
    def zyxz(self) -> U16Vector4: ...
    @property
    def zyxw(self) -> U16Vector4: ...
    @property
    def zyxo(self) -> U16Vector4: ...
    @property
    def zyxl(self) -> U16Vector4: ...
    @property
    def zyyx(self) -> U16Vector4: ...
    @property
    def zyyy(self) -> U16Vector4: ...
    @property
    def zyyz(self) -> U16Vector4: ...
    @property
    def zyyw(self) -> U16Vector4: ...
    @property
    def zyyo(self) -> U16Vector4: ...
    @property
    def zyyl(self) -> U16Vector4: ...
    @property
    def zyzx(self) -> U16Vector4: ...
    @property
    def zyzy(self) -> U16Vector4: ...
    @property
    def zyzz(self) -> U16Vector4: ...
    @property
    def zyzw(self) -> U16Vector4: ...
    @property
    def zyzo(self) -> U16Vector4: ...
    @property
    def zyzl(self) -> U16Vector4: ...
    @property
    def zywx(self) -> U16Vector4: ...
    @property
    def zywy(self) -> U16Vector4: ...
    @property
    def zywz(self) -> U16Vector4: ...
    @property
    def zyww(self) -> U16Vector4: ...
    @property
    def zywo(self) -> U16Vector4: ...
    @property
    def zywl(self) -> U16Vector4: ...
    @property
    def zyox(self) -> U16Vector4: ...
    @property
    def zyoy(self) -> U16Vector4: ...
    @property
    def zyoz(self) -> U16Vector4: ...
    @property
    def zyow(self) -> U16Vector4: ...
    @property
    def zyoo(self) -> U16Vector4: ...
    @property
    def zyol(self) -> U16Vector4: ...
    @property
    def zylx(self) -> U16Vector4: ...
    @property
    def zyly(self) -> U16Vector4: ...
    @property
    def zylz(self) -> U16Vector4: ...
    @property
    def zylw(self) -> U16Vector4: ...
    @property
    def zylo(self) -> U16Vector4: ...
    @property
    def zyll(self) -> U16Vector4: ...
    @property
    def zzxx(self) -> U16Vector4: ...
    @property
    def zzxy(self) -> U16Vector4: ...
    @property
    def zzxz(self) -> U16Vector4: ...
    @property
    def zzxw(self) -> U16Vector4: ...
    @property
    def zzxo(self) -> U16Vector4: ...
    @property
    def zzxl(self) -> U16Vector4: ...
    @property
    def zzyx(self) -> U16Vector4: ...
    @property
    def zzyy(self) -> U16Vector4: ...
    @property
    def zzyz(self) -> U16Vector4: ...
    @property
    def zzyw(self) -> U16Vector4: ...
    @property
    def zzyo(self) -> U16Vector4: ...
    @property
    def zzyl(self) -> U16Vector4: ...
    @property
    def zzzx(self) -> U16Vector4: ...
    @property
    def zzzy(self) -> U16Vector4: ...
    @property
    def zzzz(self) -> U16Vector4: ...
    @property
    def zzzw(self) -> U16Vector4: ...
    @property
    def zzzo(self) -> U16Vector4: ...
    @property
    def zzzl(self) -> U16Vector4: ...
    @property
    def zzwx(self) -> U16Vector4: ...
    @property
    def zzwy(self) -> U16Vector4: ...
    @property
    def zzwz(self) -> U16Vector4: ...
    @property
    def zzww(self) -> U16Vector4: ...
    @property
    def zzwo(self) -> U16Vector4: ...
    @property
    def zzwl(self) -> U16Vector4: ...
    @property
    def zzox(self) -> U16Vector4: ...
    @property
    def zzoy(self) -> U16Vector4: ...
    @property
    def zzoz(self) -> U16Vector4: ...
    @property
    def zzow(self) -> U16Vector4: ...
    @property
    def zzoo(self) -> U16Vector4: ...
    @property
    def zzol(self) -> U16Vector4: ...
    @property
    def zzlx(self) -> U16Vector4: ...
    @property
    def zzly(self) -> U16Vector4: ...
    @property
    def zzlz(self) -> U16Vector4: ...
    @property
    def zzlw(self) -> U16Vector4: ...
    @property
    def zzlo(self) -> U16Vector4: ...
    @property
    def zzll(self) -> U16Vector4: ...
    @property
    def zwxx(self) -> U16Vector4: ...
    @property
    def zwxy(self) -> U16Vector4: ...
    @property
    def zwxz(self) -> U16Vector4: ...
    @property
    def zwxw(self) -> U16Vector4: ...
    @property
    def zwxo(self) -> U16Vector4: ...
    @property
    def zwxl(self) -> U16Vector4: ...
    @property
    def zwyx(self) -> U16Vector4: ...
    @property
    def zwyy(self) -> U16Vector4: ...
    @property
    def zwyz(self) -> U16Vector4: ...
    @property
    def zwyw(self) -> U16Vector4: ...
    @property
    def zwyo(self) -> U16Vector4: ...
    @property
    def zwyl(self) -> U16Vector4: ...
    @property
    def zwzx(self) -> U16Vector4: ...
    @property
    def zwzy(self) -> U16Vector4: ...
    @property
    def zwzz(self) -> U16Vector4: ...
    @property
    def zwzw(self) -> U16Vector4: ...
    @property
    def zwzo(self) -> U16Vector4: ...
    @property
    def zwzl(self) -> U16Vector4: ...
    @property
    def zwwx(self) -> U16Vector4: ...
    @property
    def zwwy(self) -> U16Vector4: ...
    @property
    def zwwz(self) -> U16Vector4: ...
    @property
    def zwww(self) -> U16Vector4: ...
    @property
    def zwwo(self) -> U16Vector4: ...
    @property
    def zwwl(self) -> U16Vector4: ...
    @property
    def zwox(self) -> U16Vector4: ...
    @property
    def zwoy(self) -> U16Vector4: ...
    @property
    def zwoz(self) -> U16Vector4: ...
    @property
    def zwow(self) -> U16Vector4: ...
    @property
    def zwoo(self) -> U16Vector4: ...
    @property
    def zwol(self) -> U16Vector4: ...
    @property
    def zwlx(self) -> U16Vector4: ...
    @property
    def zwly(self) -> U16Vector4: ...
    @property
    def zwlz(self) -> U16Vector4: ...
    @property
    def zwlw(self) -> U16Vector4: ...
    @property
    def zwlo(self) -> U16Vector4: ...
    @property
    def zwll(self) -> U16Vector4: ...
    @property
    def zoxx(self) -> U16Vector4: ...
    @property
    def zoxy(self) -> U16Vector4: ...
    @property
    def zoxz(self) -> U16Vector4: ...
    @property
    def zoxw(self) -> U16Vector4: ...
    @property
    def zoxo(self) -> U16Vector4: ...
    @property
    def zoxl(self) -> U16Vector4: ...
    @property
    def zoyx(self) -> U16Vector4: ...
    @property
    def zoyy(self) -> U16Vector4: ...
    @property
    def zoyz(self) -> U16Vector4: ...
    @property
    def zoyw(self) -> U16Vector4: ...
    @property
    def zoyo(self) -> U16Vector4: ...
    @property
    def zoyl(self) -> U16Vector4: ...
    @property
    def zozx(self) -> U16Vector4: ...
    @property
    def zozy(self) -> U16Vector4: ...
    @property
    def zozz(self) -> U16Vector4: ...
    @property
    def zozw(self) -> U16Vector4: ...
    @property
    def zozo(self) -> U16Vector4: ...
    @property
    def zozl(self) -> U16Vector4: ...
    @property
    def zowx(self) -> U16Vector4: ...
    @property
    def zowy(self) -> U16Vector4: ...
    @property
    def zowz(self) -> U16Vector4: ...
    @property
    def zoww(self) -> U16Vector4: ...
    @property
    def zowo(self) -> U16Vector4: ...
    @property
    def zowl(self) -> U16Vector4: ...
    @property
    def zoox(self) -> U16Vector4: ...
    @property
    def zooy(self) -> U16Vector4: ...
    @property
    def zooz(self) -> U16Vector4: ...
    @property
    def zoow(self) -> U16Vector4: ...
    @property
    def zooo(self) -> U16Vector4: ...
    @property
    def zool(self) -> U16Vector4: ...
    @property
    def zolx(self) -> U16Vector4: ...
    @property
    def zoly(self) -> U16Vector4: ...
    @property
    def zolz(self) -> U16Vector4: ...
    @property
    def zolw(self) -> U16Vector4: ...
    @property
    def zolo(self) -> U16Vector4: ...
    @property
    def zoll(self) -> U16Vector4: ...
    @property
    def zlxx(self) -> U16Vector4: ...
    @property
    def zlxy(self) -> U16Vector4: ...
    @property
    def zlxz(self) -> U16Vector4: ...
    @property
    def zlxw(self) -> U16Vector4: ...
    @property
    def zlxo(self) -> U16Vector4: ...
    @property
    def zlxl(self) -> U16Vector4: ...
    @property
    def zlyx(self) -> U16Vector4: ...
    @property
    def zlyy(self) -> U16Vector4: ...
    @property
    def zlyz(self) -> U16Vector4: ...
    @property
    def zlyw(self) -> U16Vector4: ...
    @property
    def zlyo(self) -> U16Vector4: ...
    @property
    def zlyl(self) -> U16Vector4: ...
    @property
    def zlzx(self) -> U16Vector4: ...
    @property
    def zlzy(self) -> U16Vector4: ...
    @property
    def zlzz(self) -> U16Vector4: ...
    @property
    def zlzw(self) -> U16Vector4: ...
    @property
    def zlzo(self) -> U16Vector4: ...
    @property
    def zlzl(self) -> U16Vector4: ...
    @property
    def zlwx(self) -> U16Vector4: ...
    @property
    def zlwy(self) -> U16Vector4: ...
    @property
    def zlwz(self) -> U16Vector4: ...
    @property
    def zlww(self) -> U16Vector4: ...
    @property
    def zlwo(self) -> U16Vector4: ...
    @property
    def zlwl(self) -> U16Vector4: ...
    @property
    def zlox(self) -> U16Vector4: ...
    @property
    def zloy(self) -> U16Vector4: ...
    @property
    def zloz(self) -> U16Vector4: ...
    @property
    def zlow(self) -> U16Vector4: ...
    @property
    def zloo(self) -> U16Vector4: ...
    @property
    def zlol(self) -> U16Vector4: ...
    @property
    def zllx(self) -> U16Vector4: ...
    @property
    def zlly(self) -> U16Vector4: ...
    @property
    def zllz(self) -> U16Vector4: ...
    @property
    def zllw(self) -> U16Vector4: ...
    @property
    def zllo(self) -> U16Vector4: ...
    @property
    def zlll(self) -> U16Vector4: ...
    @property
    def wxxx(self) -> U16Vector4: ...
    @property
    def wxxy(self) -> U16Vector4: ...
    @property
    def wxxz(self) -> U16Vector4: ...
    @property
    def wxxw(self) -> U16Vector4: ...
    @property
    def wxxo(self) -> U16Vector4: ...
    @property
    def wxxl(self) -> U16Vector4: ...
    @property
    def wxyx(self) -> U16Vector4: ...
    @property
    def wxyy(self) -> U16Vector4: ...
    @property
    def wxyz(self) -> U16Vector4: ...
    @property
    def wxyw(self) -> U16Vector4: ...
    @property
    def wxyo(self) -> U16Vector4: ...
    @property
    def wxyl(self) -> U16Vector4: ...
    @property
    def wxzx(self) -> U16Vector4: ...
    @property
    def wxzy(self) -> U16Vector4: ...
    @property
    def wxzz(self) -> U16Vector4: ...
    @property
    def wxzw(self) -> U16Vector4: ...
    @property
    def wxzo(self) -> U16Vector4: ...
    @property
    def wxzl(self) -> U16Vector4: ...
    @property
    def wxwx(self) -> U16Vector4: ...
    @property
    def wxwy(self) -> U16Vector4: ...
    @property
    def wxwz(self) -> U16Vector4: ...
    @property
    def wxww(self) -> U16Vector4: ...
    @property
    def wxwo(self) -> U16Vector4: ...
    @property
    def wxwl(self) -> U16Vector4: ...
    @property
    def wxox(self) -> U16Vector4: ...
    @property
    def wxoy(self) -> U16Vector4: ...
    @property
    def wxoz(self) -> U16Vector4: ...
    @property
    def wxow(self) -> U16Vector4: ...
    @property
    def wxoo(self) -> U16Vector4: ...
    @property
    def wxol(self) -> U16Vector4: ...
    @property
    def wxlx(self) -> U16Vector4: ...
    @property
    def wxly(self) -> U16Vector4: ...
    @property
    def wxlz(self) -> U16Vector4: ...
    @property
    def wxlw(self) -> U16Vector4: ...
    @property
    def wxlo(self) -> U16Vector4: ...
    @property
    def wxll(self) -> U16Vector4: ...
    @property
    def wyxx(self) -> U16Vector4: ...
    @property
    def wyxy(self) -> U16Vector4: ...
    @property
    def wyxz(self) -> U16Vector4: ...
    @property
    def wyxw(self) -> U16Vector4: ...
    @property
    def wyxo(self) -> U16Vector4: ...
    @property
    def wyxl(self) -> U16Vector4: ...
    @property
    def wyyx(self) -> U16Vector4: ...
    @property
    def wyyy(self) -> U16Vector4: ...
    @property
    def wyyz(self) -> U16Vector4: ...
    @property
    def wyyw(self) -> U16Vector4: ...
    @property
    def wyyo(self) -> U16Vector4: ...
    @property
    def wyyl(self) -> U16Vector4: ...
    @property
    def wyzx(self) -> U16Vector4: ...
    @property
    def wyzy(self) -> U16Vector4: ...
    @property
    def wyzz(self) -> U16Vector4: ...
    @property
    def wyzw(self) -> U16Vector4: ...
    @property
    def wyzo(self) -> U16Vector4: ...
    @property
    def wyzl(self) -> U16Vector4: ...
    @property
    def wywx(self) -> U16Vector4: ...
    @property
    def wywy(self) -> U16Vector4: ...
    @property
    def wywz(self) -> U16Vector4: ...
    @property
    def wyww(self) -> U16Vector4: ...
    @property
    def wywo(self) -> U16Vector4: ...
    @property
    def wywl(self) -> U16Vector4: ...
    @property
    def wyox(self) -> U16Vector4: ...
    @property
    def wyoy(self) -> U16Vector4: ...
    @property
    def wyoz(self) -> U16Vector4: ...
    @property
    def wyow(self) -> U16Vector4: ...
    @property
    def wyoo(self) -> U16Vector4: ...
    @property
    def wyol(self) -> U16Vector4: ...
    @property
    def wylx(self) -> U16Vector4: ...
    @property
    def wyly(self) -> U16Vector4: ...
    @property
    def wylz(self) -> U16Vector4: ...
    @property
    def wylw(self) -> U16Vector4: ...
    @property
    def wylo(self) -> U16Vector4: ...
    @property
    def wyll(self) -> U16Vector4: ...
    @property
    def wzxx(self) -> U16Vector4: ...
    @property
    def wzxy(self) -> U16Vector4: ...
    @property
    def wzxz(self) -> U16Vector4: ...
    @property
    def wzxw(self) -> U16Vector4: ...
    @property
    def wzxo(self) -> U16Vector4: ...
    @property
    def wzxl(self) -> U16Vector4: ...
    @property
    def wzyx(self) -> U16Vector4: ...
    @property
    def wzyy(self) -> U16Vector4: ...
    @property
    def wzyz(self) -> U16Vector4: ...
    @property
    def wzyw(self) -> U16Vector4: ...
    @property
    def wzyo(self) -> U16Vector4: ...
    @property
    def wzyl(self) -> U16Vector4: ...
    @property
    def wzzx(self) -> U16Vector4: ...
    @property
    def wzzy(self) -> U16Vector4: ...
    @property
    def wzzz(self) -> U16Vector4: ...
    @property
    def wzzw(self) -> U16Vector4: ...
    @property
    def wzzo(self) -> U16Vector4: ...
    @property
    def wzzl(self) -> U16Vector4: ...
    @property
    def wzwx(self) -> U16Vector4: ...
    @property
    def wzwy(self) -> U16Vector4: ...
    @property
    def wzwz(self) -> U16Vector4: ...
    @property
    def wzww(self) -> U16Vector4: ...
    @property
    def wzwo(self) -> U16Vector4: ...
    @property
    def wzwl(self) -> U16Vector4: ...
    @property
    def wzox(self) -> U16Vector4: ...
    @property
    def wzoy(self) -> U16Vector4: ...
    @property
    def wzoz(self) -> U16Vector4: ...
    @property
    def wzow(self) -> U16Vector4: ...
    @property
    def wzoo(self) -> U16Vector4: ...
    @property
    def wzol(self) -> U16Vector4: ...
    @property
    def wzlx(self) -> U16Vector4: ...
    @property
    def wzly(self) -> U16Vector4: ...
    @property
    def wzlz(self) -> U16Vector4: ...
    @property
    def wzlw(self) -> U16Vector4: ...
    @property
    def wzlo(self) -> U16Vector4: ...
    @property
    def wzll(self) -> U16Vector4: ...
    @property
    def wwxx(self) -> U16Vector4: ...
    @property
    def wwxy(self) -> U16Vector4: ...
    @property
    def wwxz(self) -> U16Vector4: ...
    @property
    def wwxw(self) -> U16Vector4: ...
    @property
    def wwxo(self) -> U16Vector4: ...
    @property
    def wwxl(self) -> U16Vector4: ...
    @property
    def wwyx(self) -> U16Vector4: ...
    @property
    def wwyy(self) -> U16Vector4: ...
    @property
    def wwyz(self) -> U16Vector4: ...
    @property
    def wwyw(self) -> U16Vector4: ...
    @property
    def wwyo(self) -> U16Vector4: ...
    @property
    def wwyl(self) -> U16Vector4: ...
    @property
    def wwzx(self) -> U16Vector4: ...
    @property
    def wwzy(self) -> U16Vector4: ...
    @property
    def wwzz(self) -> U16Vector4: ...
    @property
    def wwzw(self) -> U16Vector4: ...
    @property
    def wwzo(self) -> U16Vector4: ...
    @property
    def wwzl(self) -> U16Vector4: ...
    @property
    def wwwx(self) -> U16Vector4: ...
    @property
    def wwwy(self) -> U16Vector4: ...
    @property
    def wwwz(self) -> U16Vector4: ...
    @property
    def wwww(self) -> U16Vector4: ...
    @property
    def wwwo(self) -> U16Vector4: ...
    @property
    def wwwl(self) -> U16Vector4: ...
    @property
    def wwox(self) -> U16Vector4: ...
    @property
    def wwoy(self) -> U16Vector4: ...
    @property
    def wwoz(self) -> U16Vector4: ...
    @property
    def wwow(self) -> U16Vector4: ...
    @property
    def wwoo(self) -> U16Vector4: ...
    @property
    def wwol(self) -> U16Vector4: ...
    @property
    def wwlx(self) -> U16Vector4: ...
    @property
    def wwly(self) -> U16Vector4: ...
    @property
    def wwlz(self) -> U16Vector4: ...
    @property
    def wwlw(self) -> U16Vector4: ...
    @property
    def wwlo(self) -> U16Vector4: ...
    @property
    def wwll(self) -> U16Vector4: ...
    @property
    def woxx(self) -> U16Vector4: ...
    @property
    def woxy(self) -> U16Vector4: ...
    @property
    def woxz(self) -> U16Vector4: ...
    @property
    def woxw(self) -> U16Vector4: ...
    @property
    def woxo(self) -> U16Vector4: ...
    @property
    def woxl(self) -> U16Vector4: ...
    @property
    def woyx(self) -> U16Vector4: ...
    @property
    def woyy(self) -> U16Vector4: ...
    @property
    def woyz(self) -> U16Vector4: ...
    @property
    def woyw(self) -> U16Vector4: ...
    @property
    def woyo(self) -> U16Vector4: ...
    @property
    def woyl(self) -> U16Vector4: ...
    @property
    def wozx(self) -> U16Vector4: ...
    @property
    def wozy(self) -> U16Vector4: ...
    @property
    def wozz(self) -> U16Vector4: ...
    @property
    def wozw(self) -> U16Vector4: ...
    @property
    def wozo(self) -> U16Vector4: ...
    @property
    def wozl(self) -> U16Vector4: ...
    @property
    def wowx(self) -> U16Vector4: ...
    @property
    def wowy(self) -> U16Vector4: ...
    @property
    def wowz(self) -> U16Vector4: ...
    @property
    def woww(self) -> U16Vector4: ...
    @property
    def wowo(self) -> U16Vector4: ...
    @property
    def wowl(self) -> U16Vector4: ...
    @property
    def woox(self) -> U16Vector4: ...
    @property
    def wooy(self) -> U16Vector4: ...
    @property
    def wooz(self) -> U16Vector4: ...
    @property
    def woow(self) -> U16Vector4: ...
    @property
    def wooo(self) -> U16Vector4: ...
    @property
    def wool(self) -> U16Vector4: ...
    @property
    def wolx(self) -> U16Vector4: ...
    @property
    def woly(self) -> U16Vector4: ...
    @property
    def wolz(self) -> U16Vector4: ...
    @property
    def wolw(self) -> U16Vector4: ...
    @property
    def wolo(self) -> U16Vector4: ...
    @property
    def woll(self) -> U16Vector4: ...
    @property
    def wlxx(self) -> U16Vector4: ...
    @property
    def wlxy(self) -> U16Vector4: ...
    @property
    def wlxz(self) -> U16Vector4: ...
    @property
    def wlxw(self) -> U16Vector4: ...
    @property
    def wlxo(self) -> U16Vector4: ...
    @property
    def wlxl(self) -> U16Vector4: ...
    @property
    def wlyx(self) -> U16Vector4: ...
    @property
    def wlyy(self) -> U16Vector4: ...
    @property
    def wlyz(self) -> U16Vector4: ...
    @property
    def wlyw(self) -> U16Vector4: ...
    @property
    def wlyo(self) -> U16Vector4: ...
    @property
    def wlyl(self) -> U16Vector4: ...
    @property
    def wlzx(self) -> U16Vector4: ...
    @property
    def wlzy(self) -> U16Vector4: ...
    @property
    def wlzz(self) -> U16Vector4: ...
    @property
    def wlzw(self) -> U16Vector4: ...
    @property
    def wlzo(self) -> U16Vector4: ...
    @property
    def wlzl(self) -> U16Vector4: ...
    @property
    def wlwx(self) -> U16Vector4: ...
    @property
    def wlwy(self) -> U16Vector4: ...
    @property
    def wlwz(self) -> U16Vector4: ...
    @property
    def wlww(self) -> U16Vector4: ...
    @property
    def wlwo(self) -> U16Vector4: ...
    @property
    def wlwl(self) -> U16Vector4: ...
    @property
    def wlox(self) -> U16Vector4: ...
    @property
    def wloy(self) -> U16Vector4: ...
    @property
    def wloz(self) -> U16Vector4: ...
    @property
    def wlow(self) -> U16Vector4: ...
    @property
    def wloo(self) -> U16Vector4: ...
    @property
    def wlol(self) -> U16Vector4: ...
    @property
    def wllx(self) -> U16Vector4: ...
    @property
    def wlly(self) -> U16Vector4: ...
    @property
    def wllz(self) -> U16Vector4: ...
    @property
    def wllw(self) -> U16Vector4: ...
    @property
    def wllo(self) -> U16Vector4: ...
    @property
    def wlll(self) -> U16Vector4: ...
    @property
    def oxxx(self) -> U16Vector4: ...
    @property
    def oxxy(self) -> U16Vector4: ...
    @property
    def oxxz(self) -> U16Vector4: ...
    @property
    def oxxw(self) -> U16Vector4: ...
    @property
    def oxxo(self) -> U16Vector4: ...
    @property
    def oxxl(self) -> U16Vector4: ...
    @property
    def oxyx(self) -> U16Vector4: ...
    @property
    def oxyy(self) -> U16Vector4: ...
    @property
    def oxyz(self) -> U16Vector4: ...
    @property
    def oxyw(self) -> U16Vector4: ...
    @property
    def oxyo(self) -> U16Vector4: ...
    @property
    def oxyl(self) -> U16Vector4: ...
    @property
    def oxzx(self) -> U16Vector4: ...
    @property
    def oxzy(self) -> U16Vector4: ...
    @property
    def oxzz(self) -> U16Vector4: ...
    @property
    def oxzw(self) -> U16Vector4: ...
    @property
    def oxzo(self) -> U16Vector4: ...
    @property
    def oxzl(self) -> U16Vector4: ...
    @property
    def oxwx(self) -> U16Vector4: ...
    @property
    def oxwy(self) -> U16Vector4: ...
    @property
    def oxwz(self) -> U16Vector4: ...
    @property
    def oxww(self) -> U16Vector4: ...
    @property
    def oxwo(self) -> U16Vector4: ...
    @property
    def oxwl(self) -> U16Vector4: ...
    @property
    def oxox(self) -> U16Vector4: ...
    @property
    def oxoy(self) -> U16Vector4: ...
    @property
    def oxoz(self) -> U16Vector4: ...
    @property
    def oxow(self) -> U16Vector4: ...
    @property
    def oxoo(self) -> U16Vector4: ...
    @property
    def oxol(self) -> U16Vector4: ...
    @property
    def oxlx(self) -> U16Vector4: ...
    @property
    def oxly(self) -> U16Vector4: ...
    @property
    def oxlz(self) -> U16Vector4: ...
    @property
    def oxlw(self) -> U16Vector4: ...
    @property
    def oxlo(self) -> U16Vector4: ...
    @property
    def oxll(self) -> U16Vector4: ...
    @property
    def oyxx(self) -> U16Vector4: ...
    @property
    def oyxy(self) -> U16Vector4: ...
    @property
    def oyxz(self) -> U16Vector4: ...
    @property
    def oyxw(self) -> U16Vector4: ...
    @property
    def oyxo(self) -> U16Vector4: ...
    @property
    def oyxl(self) -> U16Vector4: ...
    @property
    def oyyx(self) -> U16Vector4: ...
    @property
    def oyyy(self) -> U16Vector4: ...
    @property
    def oyyz(self) -> U16Vector4: ...
    @property
    def oyyw(self) -> U16Vector4: ...
    @property
    def oyyo(self) -> U16Vector4: ...
    @property
    def oyyl(self) -> U16Vector4: ...
    @property
    def oyzx(self) -> U16Vector4: ...
    @property
    def oyzy(self) -> U16Vector4: ...
    @property
    def oyzz(self) -> U16Vector4: ...
    @property
    def oyzw(self) -> U16Vector4: ...
    @property
    def oyzo(self) -> U16Vector4: ...
    @property
    def oyzl(self) -> U16Vector4: ...
    @property
    def oywx(self) -> U16Vector4: ...
    @property
    def oywy(self) -> U16Vector4: ...
    @property
    def oywz(self) -> U16Vector4: ...
    @property
    def oyww(self) -> U16Vector4: ...
    @property
    def oywo(self) -> U16Vector4: ...
    @property
    def oywl(self) -> U16Vector4: ...
    @property
    def oyox(self) -> U16Vector4: ...
    @property
    def oyoy(self) -> U16Vector4: ...
    @property
    def oyoz(self) -> U16Vector4: ...
    @property
    def oyow(self) -> U16Vector4: ...
    @property
    def oyoo(self) -> U16Vector4: ...
    @property
    def oyol(self) -> U16Vector4: ...
    @property
    def oylx(self) -> U16Vector4: ...
    @property
    def oyly(self) -> U16Vector4: ...
    @property
    def oylz(self) -> U16Vector4: ...
    @property
    def oylw(self) -> U16Vector4: ...
    @property
    def oylo(self) -> U16Vector4: ...
    @property
    def oyll(self) -> U16Vector4: ...
    @property
    def ozxx(self) -> U16Vector4: ...
    @property
    def ozxy(self) -> U16Vector4: ...
    @property
    def ozxz(self) -> U16Vector4: ...
    @property
    def ozxw(self) -> U16Vector4: ...
    @property
    def ozxo(self) -> U16Vector4: ...
    @property
    def ozxl(self) -> U16Vector4: ...
    @property
    def ozyx(self) -> U16Vector4: ...
    @property
    def ozyy(self) -> U16Vector4: ...
    @property
    def ozyz(self) -> U16Vector4: ...
    @property
    def ozyw(self) -> U16Vector4: ...
    @property
    def ozyo(self) -> U16Vector4: ...
    @property
    def ozyl(self) -> U16Vector4: ...
    @property
    def ozzx(self) -> U16Vector4: ...
    @property
    def ozzy(self) -> U16Vector4: ...
    @property
    def ozzz(self) -> U16Vector4: ...
    @property
    def ozzw(self) -> U16Vector4: ...
    @property
    def ozzo(self) -> U16Vector4: ...
    @property
    def ozzl(self) -> U16Vector4: ...
    @property
    def ozwx(self) -> U16Vector4: ...
    @property
    def ozwy(self) -> U16Vector4: ...
    @property
    def ozwz(self) -> U16Vector4: ...
    @property
    def ozww(self) -> U16Vector4: ...
    @property
    def ozwo(self) -> U16Vector4: ...
    @property
    def ozwl(self) -> U16Vector4: ...
    @property
    def ozox(self) -> U16Vector4: ...
    @property
    def ozoy(self) -> U16Vector4: ...
    @property
    def ozoz(self) -> U16Vector4: ...
    @property
    def ozow(self) -> U16Vector4: ...
    @property
    def ozoo(self) -> U16Vector4: ...
    @property
    def ozol(self) -> U16Vector4: ...
    @property
    def ozlx(self) -> U16Vector4: ...
    @property
    def ozly(self) -> U16Vector4: ...
    @property
    def ozlz(self) -> U16Vector4: ...
    @property
    def ozlw(self) -> U16Vector4: ...
    @property
    def ozlo(self) -> U16Vector4: ...
    @property
    def ozll(self) -> U16Vector4: ...
    @property
    def owxx(self) -> U16Vector4: ...
    @property
    def owxy(self) -> U16Vector4: ...
    @property
    def owxz(self) -> U16Vector4: ...
    @property
    def owxw(self) -> U16Vector4: ...
    @property
    def owxo(self) -> U16Vector4: ...
    @property
    def owxl(self) -> U16Vector4: ...
    @property
    def owyx(self) -> U16Vector4: ...
    @property
    def owyy(self) -> U16Vector4: ...
    @property
    def owyz(self) -> U16Vector4: ...
    @property
    def owyw(self) -> U16Vector4: ...
    @property
    def owyo(self) -> U16Vector4: ...
    @property
    def owyl(self) -> U16Vector4: ...
    @property
    def owzx(self) -> U16Vector4: ...
    @property
    def owzy(self) -> U16Vector4: ...
    @property
    def owzz(self) -> U16Vector4: ...
    @property
    def owzw(self) -> U16Vector4: ...
    @property
    def owzo(self) -> U16Vector4: ...
    @property
    def owzl(self) -> U16Vector4: ...
    @property
    def owwx(self) -> U16Vector4: ...
    @property
    def owwy(self) -> U16Vector4: ...
    @property
    def owwz(self) -> U16Vector4: ...
    @property
    def owww(self) -> U16Vector4: ...
    @property
    def owwo(self) -> U16Vector4: ...
    @property
    def owwl(self) -> U16Vector4: ...
    @property
    def owox(self) -> U16Vector4: ...
    @property
    def owoy(self) -> U16Vector4: ...
    @property
    def owoz(self) -> U16Vector4: ...
    @property
    def owow(self) -> U16Vector4: ...
    @property
    def owoo(self) -> U16Vector4: ...
    @property
    def owol(self) -> U16Vector4: ...
    @property
    def owlx(self) -> U16Vector4: ...
    @property
    def owly(self) -> U16Vector4: ...
    @property
    def owlz(self) -> U16Vector4: ...
    @property
    def owlw(self) -> U16Vector4: ...
    @property
    def owlo(self) -> U16Vector4: ...
    @property
    def owll(self) -> U16Vector4: ...
    @property
    def ooxx(self) -> U16Vector4: ...
    @property
    def ooxy(self) -> U16Vector4: ...
    @property
    def ooxz(self) -> U16Vector4: ...
    @property
    def ooxw(self) -> U16Vector4: ...
    @property
    def ooxo(self) -> U16Vector4: ...
    @property
    def ooxl(self) -> U16Vector4: ...
    @property
    def ooyx(self) -> U16Vector4: ...
    @property
    def ooyy(self) -> U16Vector4: ...
    @property
    def ooyz(self) -> U16Vector4: ...
    @property
    def ooyw(self) -> U16Vector4: ...
    @property
    def ooyo(self) -> U16Vector4: ...
    @property
    def ooyl(self) -> U16Vector4: ...
    @property
    def oozx(self) -> U16Vector4: ...
    @property
    def oozy(self) -> U16Vector4: ...
    @property
    def oozz(self) -> U16Vector4: ...
    @property
    def oozw(self) -> U16Vector4: ...
    @property
    def oozo(self) -> U16Vector4: ...
    @property
    def oozl(self) -> U16Vector4: ...
    @property
    def oowx(self) -> U16Vector4: ...
    @property
    def oowy(self) -> U16Vector4: ...
    @property
    def oowz(self) -> U16Vector4: ...
    @property
    def ooww(self) -> U16Vector4: ...
    @property
    def oowo(self) -> U16Vector4: ...
    @property
    def oowl(self) -> U16Vector4: ...
    @property
    def ooox(self) -> U16Vector4: ...
    @property
    def oooy(self) -> U16Vector4: ...
    @property
    def oooz(self) -> U16Vector4: ...
    @property
    def ooow(self) -> U16Vector4: ...
    @property
    def oolx(self) -> U16Vector4: ...
    @property
    def ooly(self) -> U16Vector4: ...
    @property
    def oolz(self) -> U16Vector4: ...
    @property
    def oolw(self) -> U16Vector4: ...
    @property
    def olxx(self) -> U16Vector4: ...
    @property
    def olxy(self) -> U16Vector4: ...
    @property
    def olxz(self) -> U16Vector4: ...
    @property
    def olxw(self) -> U16Vector4: ...
    @property
    def olxo(self) -> U16Vector4: ...
    @property
    def olxl(self) -> U16Vector4: ...
    @property
    def olyx(self) -> U16Vector4: ...
    @property
    def olyy(self) -> U16Vector4: ...
    @property
    def olyz(self) -> U16Vector4: ...
    @property
    def olyw(self) -> U16Vector4: ...
    @property
    def olyo(self) -> U16Vector4: ...
    @property
    def olyl(self) -> U16Vector4: ...
    @property
    def olzx(self) -> U16Vector4: ...
    @property
    def olzy(self) -> U16Vector4: ...
    @property
    def olzz(self) -> U16Vector4: ...
    @property
    def olzw(self) -> U16Vector4: ...
    @property
    def olzo(self) -> U16Vector4: ...
    @property
    def olzl(self) -> U16Vector4: ...
    @property
    def olwx(self) -> U16Vector4: ...
    @property
    def olwy(self) -> U16Vector4: ...
    @property
    def olwz(self) -> U16Vector4: ...
    @property
    def olww(self) -> U16Vector4: ...
    @property
    def olwo(self) -> U16Vector4: ...
    @property
    def olwl(self) -> U16Vector4: ...
    @property
    def olox(self) -> U16Vector4: ...
    @property
    def oloy(self) -> U16Vector4: ...
    @property
    def oloz(self) -> U16Vector4: ...
    @property
    def olow(self) -> U16Vector4: ...
    @property
    def ollx(self) -> U16Vector4: ...
    @property
    def olly(self) -> U16Vector4: ...
    @property
    def ollz(self) -> U16Vector4: ...
    @property
    def ollw(self) -> U16Vector4: ...
    @property
    def lxxx(self) -> U16Vector4: ...
    @property
    def lxxy(self) -> U16Vector4: ...
    @property
    def lxxz(self) -> U16Vector4: ...
    @property
    def lxxw(self) -> U16Vector4: ...
    @property
    def lxxo(self) -> U16Vector4: ...
    @property
    def lxxl(self) -> U16Vector4: ...
    @property
    def lxyx(self) -> U16Vector4: ...
    @property
    def lxyy(self) -> U16Vector4: ...
    @property
    def lxyz(self) -> U16Vector4: ...
    @property
    def lxyw(self) -> U16Vector4: ...
    @property
    def lxyo(self) -> U16Vector4: ...
    @property
    def lxyl(self) -> U16Vector4: ...
    @property
    def lxzx(self) -> U16Vector4: ...
    @property
    def lxzy(self) -> U16Vector4: ...
    @property
    def lxzz(self) -> U16Vector4: ...
    @property
    def lxzw(self) -> U16Vector4: ...
    @property
    def lxzo(self) -> U16Vector4: ...
    @property
    def lxzl(self) -> U16Vector4: ...
    @property
    def lxwx(self) -> U16Vector4: ...
    @property
    def lxwy(self) -> U16Vector4: ...
    @property
    def lxwz(self) -> U16Vector4: ...
    @property
    def lxww(self) -> U16Vector4: ...
    @property
    def lxwo(self) -> U16Vector4: ...
    @property
    def lxwl(self) -> U16Vector4: ...
    @property
    def lxox(self) -> U16Vector4: ...
    @property
    def lxoy(self) -> U16Vector4: ...
    @property
    def lxoz(self) -> U16Vector4: ...
    @property
    def lxow(self) -> U16Vector4: ...
    @property
    def lxoo(self) -> U16Vector4: ...
    @property
    def lxol(self) -> U16Vector4: ...
    @property
    def lxlx(self) -> U16Vector4: ...
    @property
    def lxly(self) -> U16Vector4: ...
    @property
    def lxlz(self) -> U16Vector4: ...
    @property
    def lxlw(self) -> U16Vector4: ...
    @property
    def lxlo(self) -> U16Vector4: ...
    @property
    def lxll(self) -> U16Vector4: ...
    @property
    def lyxx(self) -> U16Vector4: ...
    @property
    def lyxy(self) -> U16Vector4: ...
    @property
    def lyxz(self) -> U16Vector4: ...
    @property
    def lyxw(self) -> U16Vector4: ...
    @property
    def lyxo(self) -> U16Vector4: ...
    @property
    def lyxl(self) -> U16Vector4: ...
    @property
    def lyyx(self) -> U16Vector4: ...
    @property
    def lyyy(self) -> U16Vector4: ...
    @property
    def lyyz(self) -> U16Vector4: ...
    @property
    def lyyw(self) -> U16Vector4: ...
    @property
    def lyyo(self) -> U16Vector4: ...
    @property
    def lyyl(self) -> U16Vector4: ...
    @property
    def lyzx(self) -> U16Vector4: ...
    @property
    def lyzy(self) -> U16Vector4: ...
    @property
    def lyzz(self) -> U16Vector4: ...
    @property
    def lyzw(self) -> U16Vector4: ...
    @property
    def lyzo(self) -> U16Vector4: ...
    @property
    def lyzl(self) -> U16Vector4: ...
    @property
    def lywx(self) -> U16Vector4: ...
    @property
    def lywy(self) -> U16Vector4: ...
    @property
    def lywz(self) -> U16Vector4: ...
    @property
    def lyww(self) -> U16Vector4: ...
    @property
    def lywo(self) -> U16Vector4: ...
    @property
    def lywl(self) -> U16Vector4: ...
    @property
    def lyox(self) -> U16Vector4: ...
    @property
    def lyoy(self) -> U16Vector4: ...
    @property
    def lyoz(self) -> U16Vector4: ...
    @property
    def lyow(self) -> U16Vector4: ...
    @property
    def lyoo(self) -> U16Vector4: ...
    @property
    def lyol(self) -> U16Vector4: ...
    @property
    def lylx(self) -> U16Vector4: ...
    @property
    def lyly(self) -> U16Vector4: ...
    @property
    def lylz(self) -> U16Vector4: ...
    @property
    def lylw(self) -> U16Vector4: ...
    @property
    def lylo(self) -> U16Vector4: ...
    @property
    def lyll(self) -> U16Vector4: ...
    @property
    def lzxx(self) -> U16Vector4: ...
    @property
    def lzxy(self) -> U16Vector4: ...
    @property
    def lzxz(self) -> U16Vector4: ...
    @property
    def lzxw(self) -> U16Vector4: ...
    @property
    def lzxo(self) -> U16Vector4: ...
    @property
    def lzxl(self) -> U16Vector4: ...
    @property
    def lzyx(self) -> U16Vector4: ...
    @property
    def lzyy(self) -> U16Vector4: ...
    @property
    def lzyz(self) -> U16Vector4: ...
    @property
    def lzyw(self) -> U16Vector4: ...
    @property
    def lzyo(self) -> U16Vector4: ...
    @property
    def lzyl(self) -> U16Vector4: ...
    @property
    def lzzx(self) -> U16Vector4: ...
    @property
    def lzzy(self) -> U16Vector4: ...
    @property
    def lzzz(self) -> U16Vector4: ...
    @property
    def lzzw(self) -> U16Vector4: ...
    @property
    def lzzo(self) -> U16Vector4: ...
    @property
    def lzzl(self) -> U16Vector4: ...
    @property
    def lzwx(self) -> U16Vector4: ...
    @property
    def lzwy(self) -> U16Vector4: ...
    @property
    def lzwz(self) -> U16Vector4: ...
    @property
    def lzww(self) -> U16Vector4: ...
    @property
    def lzwo(self) -> U16Vector4: ...
    @property
    def lzwl(self) -> U16Vector4: ...
    @property
    def lzox(self) -> U16Vector4: ...
    @property
    def lzoy(self) -> U16Vector4: ...
    @property
    def lzoz(self) -> U16Vector4: ...
    @property
    def lzow(self) -> U16Vector4: ...
    @property
    def lzoo(self) -> U16Vector4: ...
    @property
    def lzol(self) -> U16Vector4: ...
    @property
    def lzlx(self) -> U16Vector4: ...
    @property
    def lzly(self) -> U16Vector4: ...
    @property
    def lzlz(self) -> U16Vector4: ...
    @property
    def lzlw(self) -> U16Vector4: ...
    @property
    def lzlo(self) -> U16Vector4: ...
    @property
    def lzll(self) -> U16Vector4: ...
    @property
    def lwxx(self) -> U16Vector4: ...
    @property
    def lwxy(self) -> U16Vector4: ...
    @property
    def lwxz(self) -> U16Vector4: ...
    @property
    def lwxw(self) -> U16Vector4: ...
    @property
    def lwxo(self) -> U16Vector4: ...
    @property
    def lwxl(self) -> U16Vector4: ...
    @property
    def lwyx(self) -> U16Vector4: ...
    @property
    def lwyy(self) -> U16Vector4: ...
    @property
    def lwyz(self) -> U16Vector4: ...
    @property
    def lwyw(self) -> U16Vector4: ...
    @property
    def lwyo(self) -> U16Vector4: ...
    @property
    def lwyl(self) -> U16Vector4: ...
    @property
    def lwzx(self) -> U16Vector4: ...
    @property
    def lwzy(self) -> U16Vector4: ...
    @property
    def lwzz(self) -> U16Vector4: ...
    @property
    def lwzw(self) -> U16Vector4: ...
    @property
    def lwzo(self) -> U16Vector4: ...
    @property
    def lwzl(self) -> U16Vector4: ...
    @property
    def lwwx(self) -> U16Vector4: ...
    @property
    def lwwy(self) -> U16Vector4: ...
    @property
    def lwwz(self) -> U16Vector4: ...
    @property
    def lwww(self) -> U16Vector4: ...
    @property
    def lwwo(self) -> U16Vector4: ...
    @property
    def lwwl(self) -> U16Vector4: ...
    @property
    def lwox(self) -> U16Vector4: ...
    @property
    def lwoy(self) -> U16Vector4: ...
    @property
    def lwoz(self) -> U16Vector4: ...
    @property
    def lwow(self) -> U16Vector4: ...
    @property
    def lwoo(self) -> U16Vector4: ...
    @property
    def lwol(self) -> U16Vector4: ...
    @property
    def lwlx(self) -> U16Vector4: ...
    @property
    def lwly(self) -> U16Vector4: ...
    @property
    def lwlz(self) -> U16Vector4: ...
    @property
    def lwlw(self) -> U16Vector4: ...
    @property
    def lwlo(self) -> U16Vector4: ...
    @property
    def lwll(self) -> U16Vector4: ...
    @property
    def loxx(self) -> U16Vector4: ...
    @property
    def loxy(self) -> U16Vector4: ...
    @property
    def loxz(self) -> U16Vector4: ...
    @property
    def loxw(self) -> U16Vector4: ...
    @property
    def loxo(self) -> U16Vector4: ...
    @property
    def loxl(self) -> U16Vector4: ...
    @property
    def loyx(self) -> U16Vector4: ...
    @property
    def loyy(self) -> U16Vector4: ...
    @property
    def loyz(self) -> U16Vector4: ...
    @property
    def loyw(self) -> U16Vector4: ...
    @property
    def loyo(self) -> U16Vector4: ...
    @property
    def loyl(self) -> U16Vector4: ...
    @property
    def lozx(self) -> U16Vector4: ...
    @property
    def lozy(self) -> U16Vector4: ...
    @property
    def lozz(self) -> U16Vector4: ...
    @property
    def lozw(self) -> U16Vector4: ...
    @property
    def lozo(self) -> U16Vector4: ...
    @property
    def lozl(self) -> U16Vector4: ...
    @property
    def lowx(self) -> U16Vector4: ...
    @property
    def lowy(self) -> U16Vector4: ...
    @property
    def lowz(self) -> U16Vector4: ...
    @property
    def loww(self) -> U16Vector4: ...
    @property
    def lowo(self) -> U16Vector4: ...
    @property
    def lowl(self) -> U16Vector4: ...
    @property
    def loox(self) -> U16Vector4: ...
    @property
    def looy(self) -> U16Vector4: ...
    @property
    def looz(self) -> U16Vector4: ...
    @property
    def loow(self) -> U16Vector4: ...
    @property
    def lolx(self) -> U16Vector4: ...
    @property
    def loly(self) -> U16Vector4: ...
    @property
    def lolz(self) -> U16Vector4: ...
    @property
    def lolw(self) -> U16Vector4: ...
    @property
    def llxx(self) -> U16Vector4: ...
    @property
    def llxy(self) -> U16Vector4: ...
    @property
    def llxz(self) -> U16Vector4: ...
    @property
    def llxw(self) -> U16Vector4: ...
    @property
    def llxo(self) -> U16Vector4: ...
    @property
    def llxl(self) -> U16Vector4: ...
    @property
    def llyx(self) -> U16Vector4: ...
    @property
    def llyy(self) -> U16Vector4: ...
    @property
    def llyz(self) -> U16Vector4: ...
    @property
    def llyw(self) -> U16Vector4: ...
    @property
    def llyo(self) -> U16Vector4: ...
    @property
    def llyl(self) -> U16Vector4: ...
    @property
    def llzx(self) -> U16Vector4: ...
    @property
    def llzy(self) -> U16Vector4: ...
    @property
    def llzz(self) -> U16Vector4: ...
    @property
    def llzw(self) -> U16Vector4: ...
    @property
    def llzo(self) -> U16Vector4: ...
    @property
    def llzl(self) -> U16Vector4: ...
    @property
    def llwx(self) -> U16Vector4: ...
    @property
    def llwy(self) -> U16Vector4: ...
    @property
    def llwz(self) -> U16Vector4: ...
    @property
    def llww(self) -> U16Vector4: ...
    @property
    def llwo(self) -> U16Vector4: ...
    @property
    def llwl(self) -> U16Vector4: ...
    @property
    def llox(self) -> U16Vector4: ...
    @property
    def lloy(self) -> U16Vector4: ...
    @property
    def lloz(self) -> U16Vector4: ...
    @property
    def llow(self) -> U16Vector4: ...
    @property
    def lllx(self) -> U16Vector4: ...
    @property
    def llly(self) -> U16Vector4: ...
    @property
    def lllz(self) -> U16Vector4: ...
    @property
    def lllw(self) -> U16Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> U16Vector2: ...
    @property
    def rg(self) -> U16Vector2: ...
    @property
    def rb(self) -> U16Vector2: ...
    @property
    def ra(self) -> U16Vector2: ...
    @property
    def ro(self) -> U16Vector2: ...
    @property
    def rl(self) -> U16Vector2: ...
    @property
    def gr(self) -> U16Vector2: ...
    @property
    def gg(self) -> U16Vector2: ...
    @property
    def gb(self) -> U16Vector2: ...
    @property
    def ga(self) -> U16Vector2: ...
    @property
    def go(self) -> U16Vector2: ...
    @property
    def gl(self) -> U16Vector2: ...
    @property
    def br(self) -> U16Vector2: ...
    @property
    def bg(self) -> U16Vector2: ...
    @property
    def bb(self) -> U16Vector2: ...
    @property
    def ba(self) -> U16Vector2: ...
    @property
    def bo(self) -> U16Vector2: ...
    @property
    def bl(self) -> U16Vector2: ...
    @property
    def ar(self) -> U16Vector2: ...
    @property
    def ag(self) -> U16Vector2: ...
    @property
    def ab(self) -> U16Vector2: ...
    @property
    def aa(self) -> U16Vector2: ...
    @property
    def ao(self) -> U16Vector2: ...
    @property
    def al(self) -> U16Vector2: ...
    @property
    def og(self) -> U16Vector2: ...
    @property
    def ob(self) -> U16Vector2: ...
    @property
    def oa(self) -> U16Vector2: ...
    @property
    def lr(self) -> U16Vector2: ...
    @property
    def lg(self) -> U16Vector2: ...
    @property
    def lb(self) -> U16Vector2: ...
    @property
    def la(self) -> U16Vector2: ...
    @property
    def rrr(self) -> U16Vector3: ...
    @property
    def rrg(self) -> U16Vector3: ...
    @property
    def rrb(self) -> U16Vector3: ...
    @property
    def rra(self) -> U16Vector3: ...
    @property
    def rro(self) -> U16Vector3: ...
    @property
    def rrl(self) -> U16Vector3: ...
    @property
    def rgr(self) -> U16Vector3: ...
    @property
    def rgg(self) -> U16Vector3: ...
    @property
    def rgb(self) -> U16Vector3: ...
    @property
    def rga(self) -> U16Vector3: ...
    @property
    def rgo(self) -> U16Vector3: ...
    @property
    def rgl(self) -> U16Vector3: ...
    @property
    def rbr(self) -> U16Vector3: ...
    @property
    def rbg(self) -> U16Vector3: ...
    @property
    def rbb(self) -> U16Vector3: ...
    @property
    def rba(self) -> U16Vector3: ...
    @property
    def rbo(self) -> U16Vector3: ...
    @property
    def rbl(self) -> U16Vector3: ...
    @property
    def rar(self) -> U16Vector3: ...
    @property
    def rag(self) -> U16Vector3: ...
    @property
    def rab(self) -> U16Vector3: ...
    @property
    def raa(self) -> U16Vector3: ...
    @property
    def rao(self) -> U16Vector3: ...
    @property
    def ral(self) -> U16Vector3: ...
    @property
    def ror(self) -> U16Vector3: ...
    @property
    def rog(self) -> U16Vector3: ...
    @property
    def rob(self) -> U16Vector3: ...
    @property
    def roa(self) -> U16Vector3: ...
    @property
    def roo(self) -> U16Vector3: ...
    @property
    def rol(self) -> U16Vector3: ...
    @property
    def rlr(self) -> U16Vector3: ...
    @property
    def rlg(self) -> U16Vector3: ...
    @property
    def rlb(self) -> U16Vector3: ...
    @property
    def rla(self) -> U16Vector3: ...
    @property
    def rlo(self) -> U16Vector3: ...
    @property
    def rll(self) -> U16Vector3: ...
    @property
    def grr(self) -> U16Vector3: ...
    @property
    def grg(self) -> U16Vector3: ...
    @property
    def grb(self) -> U16Vector3: ...
    @property
    def gra(self) -> U16Vector3: ...
    @property
    def gro(self) -> U16Vector3: ...
    @property
    def grl(self) -> U16Vector3: ...
    @property
    def ggr(self) -> U16Vector3: ...
    @property
    def ggg(self) -> U16Vector3: ...
    @property
    def ggb(self) -> U16Vector3: ...
    @property
    def gga(self) -> U16Vector3: ...
    @property
    def ggo(self) -> U16Vector3: ...
    @property
    def ggl(self) -> U16Vector3: ...
    @property
    def gbr(self) -> U16Vector3: ...
    @property
    def gbg(self) -> U16Vector3: ...
    @property
    def gbb(self) -> U16Vector3: ...
    @property
    def gba(self) -> U16Vector3: ...
    @property
    def gbo(self) -> U16Vector3: ...
    @property
    def gbl(self) -> U16Vector3: ...
    @property
    def gar(self) -> U16Vector3: ...
    @property
    def gag(self) -> U16Vector3: ...
    @property
    def gab(self) -> U16Vector3: ...
    @property
    def gaa(self) -> U16Vector3: ...
    @property
    def gao(self) -> U16Vector3: ...
    @property
    def gal(self) -> U16Vector3: ...
    @property
    def gor(self) -> U16Vector3: ...
    @property
    def gog(self) -> U16Vector3: ...
    @property
    def gob(self) -> U16Vector3: ...
    @property
    def goa(self) -> U16Vector3: ...
    @property
    def goo(self) -> U16Vector3: ...
    @property
    def gol(self) -> U16Vector3: ...
    @property
    def glr(self) -> U16Vector3: ...
    @property
    def glg(self) -> U16Vector3: ...
    @property
    def glb(self) -> U16Vector3: ...
    @property
    def gla(self) -> U16Vector3: ...
    @property
    def glo(self) -> U16Vector3: ...
    @property
    def gll(self) -> U16Vector3: ...
    @property
    def brr(self) -> U16Vector3: ...
    @property
    def brg(self) -> U16Vector3: ...
    @property
    def brb(self) -> U16Vector3: ...
    @property
    def bra(self) -> U16Vector3: ...
    @property
    def bro(self) -> U16Vector3: ...
    @property
    def brl(self) -> U16Vector3: ...
    @property
    def bgr(self) -> U16Vector3: ...
    @property
    def bgg(self) -> U16Vector3: ...
    @property
    def bgb(self) -> U16Vector3: ...
    @property
    def bga(self) -> U16Vector3: ...
    @property
    def bgo(self) -> U16Vector3: ...
    @property
    def bgl(self) -> U16Vector3: ...
    @property
    def bbr(self) -> U16Vector3: ...
    @property
    def bbg(self) -> U16Vector3: ...
    @property
    def bbb(self) -> U16Vector3: ...
    @property
    def bba(self) -> U16Vector3: ...
    @property
    def bbo(self) -> U16Vector3: ...
    @property
    def bbl(self) -> U16Vector3: ...
    @property
    def bar(self) -> U16Vector3: ...
    @property
    def bag(self) -> U16Vector3: ...
    @property
    def bab(self) -> U16Vector3: ...
    @property
    def baa(self) -> U16Vector3: ...
    @property
    def bao(self) -> U16Vector3: ...
    @property
    def bal(self) -> U16Vector3: ...
    @property
    def bor(self) -> U16Vector3: ...
    @property
    def bog(self) -> U16Vector3: ...
    @property
    def bob(self) -> U16Vector3: ...
    @property
    def boa(self) -> U16Vector3: ...
    @property
    def boo(self) -> U16Vector3: ...
    @property
    def bol(self) -> U16Vector3: ...
    @property
    def blr(self) -> U16Vector3: ...
    @property
    def blg(self) -> U16Vector3: ...
    @property
    def blb(self) -> U16Vector3: ...
    @property
    def bla(self) -> U16Vector3: ...
    @property
    def blo(self) -> U16Vector3: ...
    @property
    def bll(self) -> U16Vector3: ...
    @property
    def arr(self) -> U16Vector3: ...
    @property
    def arg(self) -> U16Vector3: ...
    @property
    def arb(self) -> U16Vector3: ...
    @property
    def ara(self) -> U16Vector3: ...
    @property
    def aro(self) -> U16Vector3: ...
    @property
    def arl(self) -> U16Vector3: ...
    @property
    def agr(self) -> U16Vector3: ...
    @property
    def agg(self) -> U16Vector3: ...
    @property
    def agb(self) -> U16Vector3: ...
    @property
    def aga(self) -> U16Vector3: ...
    @property
    def ago(self) -> U16Vector3: ...
    @property
    def agl(self) -> U16Vector3: ...
    @property
    def abr(self) -> U16Vector3: ...
    @property
    def abg(self) -> U16Vector3: ...
    @property
    def abb(self) -> U16Vector3: ...
    @property
    def aba(self) -> U16Vector3: ...
    @property
    def abo(self) -> U16Vector3: ...
    @property
    def abl(self) -> U16Vector3: ...
    @property
    def aar(self) -> U16Vector3: ...
    @property
    def aag(self) -> U16Vector3: ...
    @property
    def aab(self) -> U16Vector3: ...
    @property
    def aaa(self) -> U16Vector3: ...
    @property
    def aao(self) -> U16Vector3: ...
    @property
    def aal(self) -> U16Vector3: ...
    @property
    def aor(self) -> U16Vector3: ...
    @property
    def aog(self) -> U16Vector3: ...
    @property
    def aob(self) -> U16Vector3: ...
    @property
    def aoa(self) -> U16Vector3: ...
    @property
    def aoo(self) -> U16Vector3: ...
    @property
    def aol(self) -> U16Vector3: ...
    @property
    def alr(self) -> U16Vector3: ...
    @property
    def alg(self) -> U16Vector3: ...
    @property
    def alb(self) -> U16Vector3: ...
    @property
    def ala(self) -> U16Vector3: ...
    @property
    def alo(self) -> U16Vector3: ...
    @property
    def all(self) -> U16Vector3: ...
    @property
    def orr(self) -> U16Vector3: ...
    @property
    def org(self) -> U16Vector3: ...
    @property
    def orb(self) -> U16Vector3: ...
    @property
    def ora(self) -> U16Vector3: ...
    @property
    def oro(self) -> U16Vector3: ...
    @property
    def orl(self) -> U16Vector3: ...
    @property
    def ogr(self) -> U16Vector3: ...
    @property
    def ogg(self) -> U16Vector3: ...
    @property
    def ogb(self) -> U16Vector3: ...
    @property
    def oga(self) -> U16Vector3: ...
    @property
    def ogo(self) -> U16Vector3: ...
    @property
    def ogl(self) -> U16Vector3: ...
    @property
    def obr(self) -> U16Vector3: ...
    @property
    def obg(self) -> U16Vector3: ...
    @property
    def obb(self) -> U16Vector3: ...
    @property
    def oba(self) -> U16Vector3: ...
    @property
    def obo(self) -> U16Vector3: ...
    @property
    def obl(self) -> U16Vector3: ...
    @property
    def oar(self) -> U16Vector3: ...
    @property
    def oag(self) -> U16Vector3: ...
    @property
    def oab(self) -> U16Vector3: ...
    @property
    def oaa(self) -> U16Vector3: ...
    @property
    def oao(self) -> U16Vector3: ...
    @property
    def oal(self) -> U16Vector3: ...
    @property
    def oor(self) -> U16Vector3: ...
    @property
    def oog(self) -> U16Vector3: ...
    @property
    def oob(self) -> U16Vector3: ...
    @property
    def ooa(self) -> U16Vector3: ...
    @property
    def olr(self) -> U16Vector3: ...
    @property
    def olg(self) -> U16Vector3: ...
    @property
    def olb(self) -> U16Vector3: ...
    @property
    def ola(self) -> U16Vector3: ...
    @property
    def lrr(self) -> U16Vector3: ...
    @property
    def lrg(self) -> U16Vector3: ...
    @property
    def lrb(self) -> U16Vector3: ...
    @property
    def lra(self) -> U16Vector3: ...
    @property
    def lro(self) -> U16Vector3: ...
    @property
    def lrl(self) -> U16Vector3: ...
    @property
    def lgr(self) -> U16Vector3: ...
    @property
    def lgg(self) -> U16Vector3: ...
    @property
    def lgb(self) -> U16Vector3: ...
    @property
    def lga(self) -> U16Vector3: ...
    @property
    def lgo(self) -> U16Vector3: ...
    @property
    def lgl(self) -> U16Vector3: ...
    @property
    def lbr(self) -> U16Vector3: ...
    @property
    def lbg(self) -> U16Vector3: ...
    @property
    def lbb(self) -> U16Vector3: ...
    @property
    def lba(self) -> U16Vector3: ...
    @property
    def lbo(self) -> U16Vector3: ...
    @property
    def lbl(self) -> U16Vector3: ...
    @property
    def lar(self) -> U16Vector3: ...
    @property
    def lag(self) -> U16Vector3: ...
    @property
    def lab(self) -> U16Vector3: ...
    @property
    def laa(self) -> U16Vector3: ...
    @property
    def lao(self) -> U16Vector3: ...
    @property
    def lal(self) -> U16Vector3: ...
    @property
    def lor(self) -> U16Vector3: ...
    @property
    def log(self) -> U16Vector3: ...
    @property
    def lob(self) -> U16Vector3: ...
    @property
    def loa(self) -> U16Vector3: ...
    @property
    def llr(self) -> U16Vector3: ...
    @property
    def llg(self) -> U16Vector3: ...
    @property
    def llb(self) -> U16Vector3: ...
    @property
    def lla(self) -> U16Vector3: ...
    @property
    def rrrr(self) -> U16Vector4: ...
    @property
    def rrrg(self) -> U16Vector4: ...
    @property
    def rrrb(self) -> U16Vector4: ...
    @property
    def rrra(self) -> U16Vector4: ...
    @property
    def rrro(self) -> U16Vector4: ...
    @property
    def rrrl(self) -> U16Vector4: ...
    @property
    def rrgr(self) -> U16Vector4: ...
    @property
    def rrgg(self) -> U16Vector4: ...
    @property
    def rrgb(self) -> U16Vector4: ...
    @property
    def rrga(self) -> U16Vector4: ...
    @property
    def rrgo(self) -> U16Vector4: ...
    @property
    def rrgl(self) -> U16Vector4: ...
    @property
    def rrbr(self) -> U16Vector4: ...
    @property
    def rrbg(self) -> U16Vector4: ...
    @property
    def rrbb(self) -> U16Vector4: ...
    @property
    def rrba(self) -> U16Vector4: ...
    @property
    def rrbo(self) -> U16Vector4: ...
    @property
    def rrbl(self) -> U16Vector4: ...
    @property
    def rrar(self) -> U16Vector4: ...
    @property
    def rrag(self) -> U16Vector4: ...
    @property
    def rrab(self) -> U16Vector4: ...
    @property
    def rraa(self) -> U16Vector4: ...
    @property
    def rrao(self) -> U16Vector4: ...
    @property
    def rral(self) -> U16Vector4: ...
    @property
    def rror(self) -> U16Vector4: ...
    @property
    def rrog(self) -> U16Vector4: ...
    @property
    def rrob(self) -> U16Vector4: ...
    @property
    def rroa(self) -> U16Vector4: ...
    @property
    def rroo(self) -> U16Vector4: ...
    @property
    def rrol(self) -> U16Vector4: ...
    @property
    def rrlr(self) -> U16Vector4: ...
    @property
    def rrlg(self) -> U16Vector4: ...
    @property
    def rrlb(self) -> U16Vector4: ...
    @property
    def rrla(self) -> U16Vector4: ...
    @property
    def rrlo(self) -> U16Vector4: ...
    @property
    def rrll(self) -> U16Vector4: ...
    @property
    def rgrr(self) -> U16Vector4: ...
    @property
    def rgrg(self) -> U16Vector4: ...
    @property
    def rgrb(self) -> U16Vector4: ...
    @property
    def rgra(self) -> U16Vector4: ...
    @property
    def rgro(self) -> U16Vector4: ...
    @property
    def rgrl(self) -> U16Vector4: ...
    @property
    def rggr(self) -> U16Vector4: ...
    @property
    def rggg(self) -> U16Vector4: ...
    @property
    def rggb(self) -> U16Vector4: ...
    @property
    def rgga(self) -> U16Vector4: ...
    @property
    def rggo(self) -> U16Vector4: ...
    @property
    def rggl(self) -> U16Vector4: ...
    @property
    def rgbr(self) -> U16Vector4: ...
    @property
    def rgbg(self) -> U16Vector4: ...
    @property
    def rgbb(self) -> U16Vector4: ...
    @property
    def rgba(self) -> U16Vector4: ...
    @property
    def rgbo(self) -> U16Vector4: ...
    @property
    def rgbl(self) -> U16Vector4: ...
    @property
    def rgar(self) -> U16Vector4: ...
    @property
    def rgag(self) -> U16Vector4: ...
    @property
    def rgab(self) -> U16Vector4: ...
    @property
    def rgaa(self) -> U16Vector4: ...
    @property
    def rgao(self) -> U16Vector4: ...
    @property
    def rgal(self) -> U16Vector4: ...
    @property
    def rgor(self) -> U16Vector4: ...
    @property
    def rgog(self) -> U16Vector4: ...
    @property
    def rgob(self) -> U16Vector4: ...
    @property
    def rgoa(self) -> U16Vector4: ...
    @property
    def rgoo(self) -> U16Vector4: ...
    @property
    def rgol(self) -> U16Vector4: ...
    @property
    def rglr(self) -> U16Vector4: ...
    @property
    def rglg(self) -> U16Vector4: ...
    @property
    def rglb(self) -> U16Vector4: ...
    @property
    def rgla(self) -> U16Vector4: ...
    @property
    def rglo(self) -> U16Vector4: ...
    @property
    def rgll(self) -> U16Vector4: ...
    @property
    def rbrr(self) -> U16Vector4: ...
    @property
    def rbrg(self) -> U16Vector4: ...
    @property
    def rbrb(self) -> U16Vector4: ...
    @property
    def rbra(self) -> U16Vector4: ...
    @property
    def rbro(self) -> U16Vector4: ...
    @property
    def rbrl(self) -> U16Vector4: ...
    @property
    def rbgr(self) -> U16Vector4: ...
    @property
    def rbgg(self) -> U16Vector4: ...
    @property
    def rbgb(self) -> U16Vector4: ...
    @property
    def rbga(self) -> U16Vector4: ...
    @property
    def rbgo(self) -> U16Vector4: ...
    @property
    def rbgl(self) -> U16Vector4: ...
    @property
    def rbbr(self) -> U16Vector4: ...
    @property
    def rbbg(self) -> U16Vector4: ...
    @property
    def rbbb(self) -> U16Vector4: ...
    @property
    def rbba(self) -> U16Vector4: ...
    @property
    def rbbo(self) -> U16Vector4: ...
    @property
    def rbbl(self) -> U16Vector4: ...
    @property
    def rbar(self) -> U16Vector4: ...
    @property
    def rbag(self) -> U16Vector4: ...
    @property
    def rbab(self) -> U16Vector4: ...
    @property
    def rbaa(self) -> U16Vector4: ...
    @property
    def rbao(self) -> U16Vector4: ...
    @property
    def rbal(self) -> U16Vector4: ...
    @property
    def rbor(self) -> U16Vector4: ...
    @property
    def rbog(self) -> U16Vector4: ...
    @property
    def rbob(self) -> U16Vector4: ...
    @property
    def rboa(self) -> U16Vector4: ...
    @property
    def rboo(self) -> U16Vector4: ...
    @property
    def rbol(self) -> U16Vector4: ...
    @property
    def rblr(self) -> U16Vector4: ...
    @property
    def rblg(self) -> U16Vector4: ...
    @property
    def rblb(self) -> U16Vector4: ...
    @property
    def rbla(self) -> U16Vector4: ...
    @property
    def rblo(self) -> U16Vector4: ...
    @property
    def rbll(self) -> U16Vector4: ...
    @property
    def rarr(self) -> U16Vector4: ...
    @property
    def rarg(self) -> U16Vector4: ...
    @property
    def rarb(self) -> U16Vector4: ...
    @property
    def rara(self) -> U16Vector4: ...
    @property
    def raro(self) -> U16Vector4: ...
    @property
    def rarl(self) -> U16Vector4: ...
    @property
    def ragr(self) -> U16Vector4: ...
    @property
    def ragg(self) -> U16Vector4: ...
    @property
    def ragb(self) -> U16Vector4: ...
    @property
    def raga(self) -> U16Vector4: ...
    @property
    def rago(self) -> U16Vector4: ...
    @property
    def ragl(self) -> U16Vector4: ...
    @property
    def rabr(self) -> U16Vector4: ...
    @property
    def rabg(self) -> U16Vector4: ...
    @property
    def rabb(self) -> U16Vector4: ...
    @property
    def raba(self) -> U16Vector4: ...
    @property
    def rabo(self) -> U16Vector4: ...
    @property
    def rabl(self) -> U16Vector4: ...
    @property
    def raar(self) -> U16Vector4: ...
    @property
    def raag(self) -> U16Vector4: ...
    @property
    def raab(self) -> U16Vector4: ...
    @property
    def raaa(self) -> U16Vector4: ...
    @property
    def raao(self) -> U16Vector4: ...
    @property
    def raal(self) -> U16Vector4: ...
    @property
    def raor(self) -> U16Vector4: ...
    @property
    def raog(self) -> U16Vector4: ...
    @property
    def raob(self) -> U16Vector4: ...
    @property
    def raoa(self) -> U16Vector4: ...
    @property
    def raoo(self) -> U16Vector4: ...
    @property
    def raol(self) -> U16Vector4: ...
    @property
    def ralr(self) -> U16Vector4: ...
    @property
    def ralg(self) -> U16Vector4: ...
    @property
    def ralb(self) -> U16Vector4: ...
    @property
    def rala(self) -> U16Vector4: ...
    @property
    def ralo(self) -> U16Vector4: ...
    @property
    def rall(self) -> U16Vector4: ...
    @property
    def rorr(self) -> U16Vector4: ...
    @property
    def rorg(self) -> U16Vector4: ...
    @property
    def rorb(self) -> U16Vector4: ...
    @property
    def rora(self) -> U16Vector4: ...
    @property
    def roro(self) -> U16Vector4: ...
    @property
    def rorl(self) -> U16Vector4: ...
    @property
    def rogr(self) -> U16Vector4: ...
    @property
    def rogg(self) -> U16Vector4: ...
    @property
    def rogb(self) -> U16Vector4: ...
    @property
    def roga(self) -> U16Vector4: ...
    @property
    def rogo(self) -> U16Vector4: ...
    @property
    def rogl(self) -> U16Vector4: ...
    @property
    def robr(self) -> U16Vector4: ...
    @property
    def robg(self) -> U16Vector4: ...
    @property
    def robb(self) -> U16Vector4: ...
    @property
    def roba(self) -> U16Vector4: ...
    @property
    def robo(self) -> U16Vector4: ...
    @property
    def robl(self) -> U16Vector4: ...
    @property
    def roar(self) -> U16Vector4: ...
    @property
    def roag(self) -> U16Vector4: ...
    @property
    def roab(self) -> U16Vector4: ...
    @property
    def roaa(self) -> U16Vector4: ...
    @property
    def roao(self) -> U16Vector4: ...
    @property
    def roal(self) -> U16Vector4: ...
    @property
    def roor(self) -> U16Vector4: ...
    @property
    def roog(self) -> U16Vector4: ...
    @property
    def roob(self) -> U16Vector4: ...
    @property
    def rooa(self) -> U16Vector4: ...
    @property
    def rooo(self) -> U16Vector4: ...
    @property
    def rool(self) -> U16Vector4: ...
    @property
    def rolr(self) -> U16Vector4: ...
    @property
    def rolg(self) -> U16Vector4: ...
    @property
    def rolb(self) -> U16Vector4: ...
    @property
    def rola(self) -> U16Vector4: ...
    @property
    def rolo(self) -> U16Vector4: ...
    @property
    def roll(self) -> U16Vector4: ...
    @property
    def rlrr(self) -> U16Vector4: ...
    @property
    def rlrg(self) -> U16Vector4: ...
    @property
    def rlrb(self) -> U16Vector4: ...
    @property
    def rlra(self) -> U16Vector4: ...
    @property
    def rlro(self) -> U16Vector4: ...
    @property
    def rlrl(self) -> U16Vector4: ...
    @property
    def rlgr(self) -> U16Vector4: ...
    @property
    def rlgg(self) -> U16Vector4: ...
    @property
    def rlgb(self) -> U16Vector4: ...
    @property
    def rlga(self) -> U16Vector4: ...
    @property
    def rlgo(self) -> U16Vector4: ...
    @property
    def rlgl(self) -> U16Vector4: ...
    @property
    def rlbr(self) -> U16Vector4: ...
    @property
    def rlbg(self) -> U16Vector4: ...
    @property
    def rlbb(self) -> U16Vector4: ...
    @property
    def rlba(self) -> U16Vector4: ...
    @property
    def rlbo(self) -> U16Vector4: ...
    @property
    def rlbl(self) -> U16Vector4: ...
    @property
    def rlar(self) -> U16Vector4: ...
    @property
    def rlag(self) -> U16Vector4: ...
    @property
    def rlab(self) -> U16Vector4: ...
    @property
    def rlaa(self) -> U16Vector4: ...
    @property
    def rlao(self) -> U16Vector4: ...
    @property
    def rlal(self) -> U16Vector4: ...
    @property
    def rlor(self) -> U16Vector4: ...
    @property
    def rlog(self) -> U16Vector4: ...
    @property
    def rlob(self) -> U16Vector4: ...
    @property
    def rloa(self) -> U16Vector4: ...
    @property
    def rloo(self) -> U16Vector4: ...
    @property
    def rlol(self) -> U16Vector4: ...
    @property
    def rllr(self) -> U16Vector4: ...
    @property
    def rllg(self) -> U16Vector4: ...
    @property
    def rllb(self) -> U16Vector4: ...
    @property
    def rlla(self) -> U16Vector4: ...
    @property
    def rllo(self) -> U16Vector4: ...
    @property
    def rlll(self) -> U16Vector4: ...
    @property
    def grrr(self) -> U16Vector4: ...
    @property
    def grrg(self) -> U16Vector4: ...
    @property
    def grrb(self) -> U16Vector4: ...
    @property
    def grra(self) -> U16Vector4: ...
    @property
    def grro(self) -> U16Vector4: ...
    @property
    def grrl(self) -> U16Vector4: ...
    @property
    def grgr(self) -> U16Vector4: ...
    @property
    def grgg(self) -> U16Vector4: ...
    @property
    def grgb(self) -> U16Vector4: ...
    @property
    def grga(self) -> U16Vector4: ...
    @property
    def grgo(self) -> U16Vector4: ...
    @property
    def grgl(self) -> U16Vector4: ...
    @property
    def grbr(self) -> U16Vector4: ...
    @property
    def grbg(self) -> U16Vector4: ...
    @property
    def grbb(self) -> U16Vector4: ...
    @property
    def grba(self) -> U16Vector4: ...
    @property
    def grbo(self) -> U16Vector4: ...
    @property
    def grbl(self) -> U16Vector4: ...
    @property
    def grar(self) -> U16Vector4: ...
    @property
    def grag(self) -> U16Vector4: ...
    @property
    def grab(self) -> U16Vector4: ...
    @property
    def graa(self) -> U16Vector4: ...
    @property
    def grao(self) -> U16Vector4: ...
    @property
    def gral(self) -> U16Vector4: ...
    @property
    def gror(self) -> U16Vector4: ...
    @property
    def grog(self) -> U16Vector4: ...
    @property
    def grob(self) -> U16Vector4: ...
    @property
    def groa(self) -> U16Vector4: ...
    @property
    def groo(self) -> U16Vector4: ...
    @property
    def grol(self) -> U16Vector4: ...
    @property
    def grlr(self) -> U16Vector4: ...
    @property
    def grlg(self) -> U16Vector4: ...
    @property
    def grlb(self) -> U16Vector4: ...
    @property
    def grla(self) -> U16Vector4: ...
    @property
    def grlo(self) -> U16Vector4: ...
    @property
    def grll(self) -> U16Vector4: ...
    @property
    def ggrr(self) -> U16Vector4: ...
    @property
    def ggrg(self) -> U16Vector4: ...
    @property
    def ggrb(self) -> U16Vector4: ...
    @property
    def ggra(self) -> U16Vector4: ...
    @property
    def ggro(self) -> U16Vector4: ...
    @property
    def ggrl(self) -> U16Vector4: ...
    @property
    def gggr(self) -> U16Vector4: ...
    @property
    def gggg(self) -> U16Vector4: ...
    @property
    def gggb(self) -> U16Vector4: ...
    @property
    def ggga(self) -> U16Vector4: ...
    @property
    def gggo(self) -> U16Vector4: ...
    @property
    def gggl(self) -> U16Vector4: ...
    @property
    def ggbr(self) -> U16Vector4: ...
    @property
    def ggbg(self) -> U16Vector4: ...
    @property
    def ggbb(self) -> U16Vector4: ...
    @property
    def ggba(self) -> U16Vector4: ...
    @property
    def ggbo(self) -> U16Vector4: ...
    @property
    def ggbl(self) -> U16Vector4: ...
    @property
    def ggar(self) -> U16Vector4: ...
    @property
    def ggag(self) -> U16Vector4: ...
    @property
    def ggab(self) -> U16Vector4: ...
    @property
    def ggaa(self) -> U16Vector4: ...
    @property
    def ggao(self) -> U16Vector4: ...
    @property
    def ggal(self) -> U16Vector4: ...
    @property
    def ggor(self) -> U16Vector4: ...
    @property
    def ggog(self) -> U16Vector4: ...
    @property
    def ggob(self) -> U16Vector4: ...
    @property
    def ggoa(self) -> U16Vector4: ...
    @property
    def ggoo(self) -> U16Vector4: ...
    @property
    def ggol(self) -> U16Vector4: ...
    @property
    def gglr(self) -> U16Vector4: ...
    @property
    def gglg(self) -> U16Vector4: ...
    @property
    def gglb(self) -> U16Vector4: ...
    @property
    def ggla(self) -> U16Vector4: ...
    @property
    def gglo(self) -> U16Vector4: ...
    @property
    def ggll(self) -> U16Vector4: ...
    @property
    def gbrr(self) -> U16Vector4: ...
    @property
    def gbrg(self) -> U16Vector4: ...
    @property
    def gbrb(self) -> U16Vector4: ...
    @property
    def gbra(self) -> U16Vector4: ...
    @property
    def gbro(self) -> U16Vector4: ...
    @property
    def gbrl(self) -> U16Vector4: ...
    @property
    def gbgr(self) -> U16Vector4: ...
    @property
    def gbgg(self) -> U16Vector4: ...
    @property
    def gbgb(self) -> U16Vector4: ...
    @property
    def gbga(self) -> U16Vector4: ...
    @property
    def gbgo(self) -> U16Vector4: ...
    @property
    def gbgl(self) -> U16Vector4: ...
    @property
    def gbbr(self) -> U16Vector4: ...
    @property
    def gbbg(self) -> U16Vector4: ...
    @property
    def gbbb(self) -> U16Vector4: ...
    @property
    def gbba(self) -> U16Vector4: ...
    @property
    def gbbo(self) -> U16Vector4: ...
    @property
    def gbbl(self) -> U16Vector4: ...
    @property
    def gbar(self) -> U16Vector4: ...
    @property
    def gbag(self) -> U16Vector4: ...
    @property
    def gbab(self) -> U16Vector4: ...
    @property
    def gbaa(self) -> U16Vector4: ...
    @property
    def gbao(self) -> U16Vector4: ...
    @property
    def gbal(self) -> U16Vector4: ...
    @property
    def gbor(self) -> U16Vector4: ...
    @property
    def gbog(self) -> U16Vector4: ...
    @property
    def gbob(self) -> U16Vector4: ...
    @property
    def gboa(self) -> U16Vector4: ...
    @property
    def gboo(self) -> U16Vector4: ...
    @property
    def gbol(self) -> U16Vector4: ...
    @property
    def gblr(self) -> U16Vector4: ...
    @property
    def gblg(self) -> U16Vector4: ...
    @property
    def gblb(self) -> U16Vector4: ...
    @property
    def gbla(self) -> U16Vector4: ...
    @property
    def gblo(self) -> U16Vector4: ...
    @property
    def gbll(self) -> U16Vector4: ...
    @property
    def garr(self) -> U16Vector4: ...
    @property
    def garg(self) -> U16Vector4: ...
    @property
    def garb(self) -> U16Vector4: ...
    @property
    def gara(self) -> U16Vector4: ...
    @property
    def garo(self) -> U16Vector4: ...
    @property
    def garl(self) -> U16Vector4: ...
    @property
    def gagr(self) -> U16Vector4: ...
    @property
    def gagg(self) -> U16Vector4: ...
    @property
    def gagb(self) -> U16Vector4: ...
    @property
    def gaga(self) -> U16Vector4: ...
    @property
    def gago(self) -> U16Vector4: ...
    @property
    def gagl(self) -> U16Vector4: ...
    @property
    def gabr(self) -> U16Vector4: ...
    @property
    def gabg(self) -> U16Vector4: ...
    @property
    def gabb(self) -> U16Vector4: ...
    @property
    def gaba(self) -> U16Vector4: ...
    @property
    def gabo(self) -> U16Vector4: ...
    @property
    def gabl(self) -> U16Vector4: ...
    @property
    def gaar(self) -> U16Vector4: ...
    @property
    def gaag(self) -> U16Vector4: ...
    @property
    def gaab(self) -> U16Vector4: ...
    @property
    def gaaa(self) -> U16Vector4: ...
    @property
    def gaao(self) -> U16Vector4: ...
    @property
    def gaal(self) -> U16Vector4: ...
    @property
    def gaor(self) -> U16Vector4: ...
    @property
    def gaog(self) -> U16Vector4: ...
    @property
    def gaob(self) -> U16Vector4: ...
    @property
    def gaoa(self) -> U16Vector4: ...
    @property
    def gaoo(self) -> U16Vector4: ...
    @property
    def gaol(self) -> U16Vector4: ...
    @property
    def galr(self) -> U16Vector4: ...
    @property
    def galg(self) -> U16Vector4: ...
    @property
    def galb(self) -> U16Vector4: ...
    @property
    def gala(self) -> U16Vector4: ...
    @property
    def galo(self) -> U16Vector4: ...
    @property
    def gall(self) -> U16Vector4: ...
    @property
    def gorr(self) -> U16Vector4: ...
    @property
    def gorg(self) -> U16Vector4: ...
    @property
    def gorb(self) -> U16Vector4: ...
    @property
    def gora(self) -> U16Vector4: ...
    @property
    def goro(self) -> U16Vector4: ...
    @property
    def gorl(self) -> U16Vector4: ...
    @property
    def gogr(self) -> U16Vector4: ...
    @property
    def gogg(self) -> U16Vector4: ...
    @property
    def gogb(self) -> U16Vector4: ...
    @property
    def goga(self) -> U16Vector4: ...
    @property
    def gogo(self) -> U16Vector4: ...
    @property
    def gogl(self) -> U16Vector4: ...
    @property
    def gobr(self) -> U16Vector4: ...
    @property
    def gobg(self) -> U16Vector4: ...
    @property
    def gobb(self) -> U16Vector4: ...
    @property
    def goba(self) -> U16Vector4: ...
    @property
    def gobo(self) -> U16Vector4: ...
    @property
    def gobl(self) -> U16Vector4: ...
    @property
    def goar(self) -> U16Vector4: ...
    @property
    def goag(self) -> U16Vector4: ...
    @property
    def goab(self) -> U16Vector4: ...
    @property
    def goaa(self) -> U16Vector4: ...
    @property
    def goao(self) -> U16Vector4: ...
    @property
    def goal(self) -> U16Vector4: ...
    @property
    def goor(self) -> U16Vector4: ...
    @property
    def goog(self) -> U16Vector4: ...
    @property
    def goob(self) -> U16Vector4: ...
    @property
    def gooa(self) -> U16Vector4: ...
    @property
    def gooo(self) -> U16Vector4: ...
    @property
    def gool(self) -> U16Vector4: ...
    @property
    def golr(self) -> U16Vector4: ...
    @property
    def golg(self) -> U16Vector4: ...
    @property
    def golb(self) -> U16Vector4: ...
    @property
    def gola(self) -> U16Vector4: ...
    @property
    def golo(self) -> U16Vector4: ...
    @property
    def goll(self) -> U16Vector4: ...
    @property
    def glrr(self) -> U16Vector4: ...
    @property
    def glrg(self) -> U16Vector4: ...
    @property
    def glrb(self) -> U16Vector4: ...
    @property
    def glra(self) -> U16Vector4: ...
    @property
    def glro(self) -> U16Vector4: ...
    @property
    def glrl(self) -> U16Vector4: ...
    @property
    def glgr(self) -> U16Vector4: ...
    @property
    def glgg(self) -> U16Vector4: ...
    @property
    def glgb(self) -> U16Vector4: ...
    @property
    def glga(self) -> U16Vector4: ...
    @property
    def glgo(self) -> U16Vector4: ...
    @property
    def glgl(self) -> U16Vector4: ...
    @property
    def glbr(self) -> U16Vector4: ...
    @property
    def glbg(self) -> U16Vector4: ...
    @property
    def glbb(self) -> U16Vector4: ...
    @property
    def glba(self) -> U16Vector4: ...
    @property
    def glbo(self) -> U16Vector4: ...
    @property
    def glbl(self) -> U16Vector4: ...
    @property
    def glar(self) -> U16Vector4: ...
    @property
    def glag(self) -> U16Vector4: ...
    @property
    def glab(self) -> U16Vector4: ...
    @property
    def glaa(self) -> U16Vector4: ...
    @property
    def glao(self) -> U16Vector4: ...
    @property
    def glal(self) -> U16Vector4: ...
    @property
    def glor(self) -> U16Vector4: ...
    @property
    def glog(self) -> U16Vector4: ...
    @property
    def glob(self) -> U16Vector4: ...
    @property
    def gloa(self) -> U16Vector4: ...
    @property
    def gloo(self) -> U16Vector4: ...
    @property
    def glol(self) -> U16Vector4: ...
    @property
    def gllr(self) -> U16Vector4: ...
    @property
    def gllg(self) -> U16Vector4: ...
    @property
    def gllb(self) -> U16Vector4: ...
    @property
    def glla(self) -> U16Vector4: ...
    @property
    def gllo(self) -> U16Vector4: ...
    @property
    def glll(self) -> U16Vector4: ...
    @property
    def brrr(self) -> U16Vector4: ...
    @property
    def brrg(self) -> U16Vector4: ...
    @property
    def brrb(self) -> U16Vector4: ...
    @property
    def brra(self) -> U16Vector4: ...
    @property
    def brro(self) -> U16Vector4: ...
    @property
    def brrl(self) -> U16Vector4: ...
    @property
    def brgr(self) -> U16Vector4: ...
    @property
    def brgg(self) -> U16Vector4: ...
    @property
    def brgb(self) -> U16Vector4: ...
    @property
    def brga(self) -> U16Vector4: ...
    @property
    def brgo(self) -> U16Vector4: ...
    @property
    def brgl(self) -> U16Vector4: ...
    @property
    def brbr(self) -> U16Vector4: ...
    @property
    def brbg(self) -> U16Vector4: ...
    @property
    def brbb(self) -> U16Vector4: ...
    @property
    def brba(self) -> U16Vector4: ...
    @property
    def brbo(self) -> U16Vector4: ...
    @property
    def brbl(self) -> U16Vector4: ...
    @property
    def brar(self) -> U16Vector4: ...
    @property
    def brag(self) -> U16Vector4: ...
    @property
    def brab(self) -> U16Vector4: ...
    @property
    def braa(self) -> U16Vector4: ...
    @property
    def brao(self) -> U16Vector4: ...
    @property
    def bral(self) -> U16Vector4: ...
    @property
    def bror(self) -> U16Vector4: ...
    @property
    def brog(self) -> U16Vector4: ...
    @property
    def brob(self) -> U16Vector4: ...
    @property
    def broa(self) -> U16Vector4: ...
    @property
    def broo(self) -> U16Vector4: ...
    @property
    def brol(self) -> U16Vector4: ...
    @property
    def brlr(self) -> U16Vector4: ...
    @property
    def brlg(self) -> U16Vector4: ...
    @property
    def brlb(self) -> U16Vector4: ...
    @property
    def brla(self) -> U16Vector4: ...
    @property
    def brlo(self) -> U16Vector4: ...
    @property
    def brll(self) -> U16Vector4: ...
    @property
    def bgrr(self) -> U16Vector4: ...
    @property
    def bgrg(self) -> U16Vector4: ...
    @property
    def bgrb(self) -> U16Vector4: ...
    @property
    def bgra(self) -> U16Vector4: ...
    @property
    def bgro(self) -> U16Vector4: ...
    @property
    def bgrl(self) -> U16Vector4: ...
    @property
    def bggr(self) -> U16Vector4: ...
    @property
    def bggg(self) -> U16Vector4: ...
    @property
    def bggb(self) -> U16Vector4: ...
    @property
    def bgga(self) -> U16Vector4: ...
    @property
    def bggo(self) -> U16Vector4: ...
    @property
    def bggl(self) -> U16Vector4: ...
    @property
    def bgbr(self) -> U16Vector4: ...
    @property
    def bgbg(self) -> U16Vector4: ...
    @property
    def bgbb(self) -> U16Vector4: ...
    @property
    def bgba(self) -> U16Vector4: ...
    @property
    def bgbo(self) -> U16Vector4: ...
    @property
    def bgbl(self) -> U16Vector4: ...
    @property
    def bgar(self) -> U16Vector4: ...
    @property
    def bgag(self) -> U16Vector4: ...
    @property
    def bgab(self) -> U16Vector4: ...
    @property
    def bgaa(self) -> U16Vector4: ...
    @property
    def bgao(self) -> U16Vector4: ...
    @property
    def bgal(self) -> U16Vector4: ...
    @property
    def bgor(self) -> U16Vector4: ...
    @property
    def bgog(self) -> U16Vector4: ...
    @property
    def bgob(self) -> U16Vector4: ...
    @property
    def bgoa(self) -> U16Vector4: ...
    @property
    def bgoo(self) -> U16Vector4: ...
    @property
    def bgol(self) -> U16Vector4: ...
    @property
    def bglr(self) -> U16Vector4: ...
    @property
    def bglg(self) -> U16Vector4: ...
    @property
    def bglb(self) -> U16Vector4: ...
    @property
    def bgla(self) -> U16Vector4: ...
    @property
    def bglo(self) -> U16Vector4: ...
    @property
    def bgll(self) -> U16Vector4: ...
    @property
    def bbrr(self) -> U16Vector4: ...
    @property
    def bbrg(self) -> U16Vector4: ...
    @property
    def bbrb(self) -> U16Vector4: ...
    @property
    def bbra(self) -> U16Vector4: ...
    @property
    def bbro(self) -> U16Vector4: ...
    @property
    def bbrl(self) -> U16Vector4: ...
    @property
    def bbgr(self) -> U16Vector4: ...
    @property
    def bbgg(self) -> U16Vector4: ...
    @property
    def bbgb(self) -> U16Vector4: ...
    @property
    def bbga(self) -> U16Vector4: ...
    @property
    def bbgo(self) -> U16Vector4: ...
    @property
    def bbgl(self) -> U16Vector4: ...
    @property
    def bbbr(self) -> U16Vector4: ...
    @property
    def bbbg(self) -> U16Vector4: ...
    @property
    def bbbb(self) -> U16Vector4: ...
    @property
    def bbba(self) -> U16Vector4: ...
    @property
    def bbbo(self) -> U16Vector4: ...
    @property
    def bbbl(self) -> U16Vector4: ...
    @property
    def bbar(self) -> U16Vector4: ...
    @property
    def bbag(self) -> U16Vector4: ...
    @property
    def bbab(self) -> U16Vector4: ...
    @property
    def bbaa(self) -> U16Vector4: ...
    @property
    def bbao(self) -> U16Vector4: ...
    @property
    def bbal(self) -> U16Vector4: ...
    @property
    def bbor(self) -> U16Vector4: ...
    @property
    def bbog(self) -> U16Vector4: ...
    @property
    def bbob(self) -> U16Vector4: ...
    @property
    def bboa(self) -> U16Vector4: ...
    @property
    def bboo(self) -> U16Vector4: ...
    @property
    def bbol(self) -> U16Vector4: ...
    @property
    def bblr(self) -> U16Vector4: ...
    @property
    def bblg(self) -> U16Vector4: ...
    @property
    def bblb(self) -> U16Vector4: ...
    @property
    def bbla(self) -> U16Vector4: ...
    @property
    def bblo(self) -> U16Vector4: ...
    @property
    def bbll(self) -> U16Vector4: ...
    @property
    def barr(self) -> U16Vector4: ...
    @property
    def barg(self) -> U16Vector4: ...
    @property
    def barb(self) -> U16Vector4: ...
    @property
    def bara(self) -> U16Vector4: ...
    @property
    def baro(self) -> U16Vector4: ...
    @property
    def barl(self) -> U16Vector4: ...
    @property
    def bagr(self) -> U16Vector4: ...
    @property
    def bagg(self) -> U16Vector4: ...
    @property
    def bagb(self) -> U16Vector4: ...
    @property
    def baga(self) -> U16Vector4: ...
    @property
    def bago(self) -> U16Vector4: ...
    @property
    def bagl(self) -> U16Vector4: ...
    @property
    def babr(self) -> U16Vector4: ...
    @property
    def babg(self) -> U16Vector4: ...
    @property
    def babb(self) -> U16Vector4: ...
    @property
    def baba(self) -> U16Vector4: ...
    @property
    def babo(self) -> U16Vector4: ...
    @property
    def babl(self) -> U16Vector4: ...
    @property
    def baar(self) -> U16Vector4: ...
    @property
    def baag(self) -> U16Vector4: ...
    @property
    def baab(self) -> U16Vector4: ...
    @property
    def baaa(self) -> U16Vector4: ...
    @property
    def baao(self) -> U16Vector4: ...
    @property
    def baal(self) -> U16Vector4: ...
    @property
    def baor(self) -> U16Vector4: ...
    @property
    def baog(self) -> U16Vector4: ...
    @property
    def baob(self) -> U16Vector4: ...
    @property
    def baoa(self) -> U16Vector4: ...
    @property
    def baoo(self) -> U16Vector4: ...
    @property
    def baol(self) -> U16Vector4: ...
    @property
    def balr(self) -> U16Vector4: ...
    @property
    def balg(self) -> U16Vector4: ...
    @property
    def balb(self) -> U16Vector4: ...
    @property
    def bala(self) -> U16Vector4: ...
    @property
    def balo(self) -> U16Vector4: ...
    @property
    def ball(self) -> U16Vector4: ...
    @property
    def borr(self) -> U16Vector4: ...
    @property
    def borg(self) -> U16Vector4: ...
    @property
    def borb(self) -> U16Vector4: ...
    @property
    def bora(self) -> U16Vector4: ...
    @property
    def boro(self) -> U16Vector4: ...
    @property
    def borl(self) -> U16Vector4: ...
    @property
    def bogr(self) -> U16Vector4: ...
    @property
    def bogg(self) -> U16Vector4: ...
    @property
    def bogb(self) -> U16Vector4: ...
    @property
    def boga(self) -> U16Vector4: ...
    @property
    def bogo(self) -> U16Vector4: ...
    @property
    def bogl(self) -> U16Vector4: ...
    @property
    def bobr(self) -> U16Vector4: ...
    @property
    def bobg(self) -> U16Vector4: ...
    @property
    def bobb(self) -> U16Vector4: ...
    @property
    def boba(self) -> U16Vector4: ...
    @property
    def bobo(self) -> U16Vector4: ...
    @property
    def bobl(self) -> U16Vector4: ...
    @property
    def boar(self) -> U16Vector4: ...
    @property
    def boag(self) -> U16Vector4: ...
    @property
    def boab(self) -> U16Vector4: ...
    @property
    def boaa(self) -> U16Vector4: ...
    @property
    def boao(self) -> U16Vector4: ...
    @property
    def boal(self) -> U16Vector4: ...
    @property
    def boor(self) -> U16Vector4: ...
    @property
    def boog(self) -> U16Vector4: ...
    @property
    def boob(self) -> U16Vector4: ...
    @property
    def booa(self) -> U16Vector4: ...
    @property
    def booo(self) -> U16Vector4: ...
    @property
    def bool(self) -> U16Vector4: ...
    @property
    def bolr(self) -> U16Vector4: ...
    @property
    def bolg(self) -> U16Vector4: ...
    @property
    def bolb(self) -> U16Vector4: ...
    @property
    def bola(self) -> U16Vector4: ...
    @property
    def bolo(self) -> U16Vector4: ...
    @property
    def boll(self) -> U16Vector4: ...
    @property
    def blrr(self) -> U16Vector4: ...
    @property
    def blrg(self) -> U16Vector4: ...
    @property
    def blrb(self) -> U16Vector4: ...
    @property
    def blra(self) -> U16Vector4: ...
    @property
    def blro(self) -> U16Vector4: ...
    @property
    def blrl(self) -> U16Vector4: ...
    @property
    def blgr(self) -> U16Vector4: ...
    @property
    def blgg(self) -> U16Vector4: ...
    @property
    def blgb(self) -> U16Vector4: ...
    @property
    def blga(self) -> U16Vector4: ...
    @property
    def blgo(self) -> U16Vector4: ...
    @property
    def blgl(self) -> U16Vector4: ...
    @property
    def blbr(self) -> U16Vector4: ...
    @property
    def blbg(self) -> U16Vector4: ...
    @property
    def blbb(self) -> U16Vector4: ...
    @property
    def blba(self) -> U16Vector4: ...
    @property
    def blbo(self) -> U16Vector4: ...
    @property
    def blbl(self) -> U16Vector4: ...
    @property
    def blar(self) -> U16Vector4: ...
    @property
    def blag(self) -> U16Vector4: ...
    @property
    def blab(self) -> U16Vector4: ...
    @property
    def blaa(self) -> U16Vector4: ...
    @property
    def blao(self) -> U16Vector4: ...
    @property
    def blal(self) -> U16Vector4: ...
    @property
    def blor(self) -> U16Vector4: ...
    @property
    def blog(self) -> U16Vector4: ...
    @property
    def blob(self) -> U16Vector4: ...
    @property
    def bloa(self) -> U16Vector4: ...
    @property
    def bloo(self) -> U16Vector4: ...
    @property
    def blol(self) -> U16Vector4: ...
    @property
    def bllr(self) -> U16Vector4: ...
    @property
    def bllg(self) -> U16Vector4: ...
    @property
    def bllb(self) -> U16Vector4: ...
    @property
    def blla(self) -> U16Vector4: ...
    @property
    def bllo(self) -> U16Vector4: ...
    @property
    def blll(self) -> U16Vector4: ...
    @property
    def arrr(self) -> U16Vector4: ...
    @property
    def arrg(self) -> U16Vector4: ...
    @property
    def arrb(self) -> U16Vector4: ...
    @property
    def arra(self) -> U16Vector4: ...
    @property
    def arro(self) -> U16Vector4: ...
    @property
    def arrl(self) -> U16Vector4: ...
    @property
    def argr(self) -> U16Vector4: ...
    @property
    def argg(self) -> U16Vector4: ...
    @property
    def argb(self) -> U16Vector4: ...
    @property
    def arga(self) -> U16Vector4: ...
    @property
    def argo(self) -> U16Vector4: ...
    @property
    def argl(self) -> U16Vector4: ...
    @property
    def arbr(self) -> U16Vector4: ...
    @property
    def arbg(self) -> U16Vector4: ...
    @property
    def arbb(self) -> U16Vector4: ...
    @property
    def arba(self) -> U16Vector4: ...
    @property
    def arbo(self) -> U16Vector4: ...
    @property
    def arbl(self) -> U16Vector4: ...
    @property
    def arar(self) -> U16Vector4: ...
    @property
    def arag(self) -> U16Vector4: ...
    @property
    def arab(self) -> U16Vector4: ...
    @property
    def araa(self) -> U16Vector4: ...
    @property
    def arao(self) -> U16Vector4: ...
    @property
    def aral(self) -> U16Vector4: ...
    @property
    def aror(self) -> U16Vector4: ...
    @property
    def arog(self) -> U16Vector4: ...
    @property
    def arob(self) -> U16Vector4: ...
    @property
    def aroa(self) -> U16Vector4: ...
    @property
    def aroo(self) -> U16Vector4: ...
    @property
    def arol(self) -> U16Vector4: ...
    @property
    def arlr(self) -> U16Vector4: ...
    @property
    def arlg(self) -> U16Vector4: ...
    @property
    def arlb(self) -> U16Vector4: ...
    @property
    def arla(self) -> U16Vector4: ...
    @property
    def arlo(self) -> U16Vector4: ...
    @property
    def arll(self) -> U16Vector4: ...
    @property
    def agrr(self) -> U16Vector4: ...
    @property
    def agrg(self) -> U16Vector4: ...
    @property
    def agrb(self) -> U16Vector4: ...
    @property
    def agra(self) -> U16Vector4: ...
    @property
    def agro(self) -> U16Vector4: ...
    @property
    def agrl(self) -> U16Vector4: ...
    @property
    def aggr(self) -> U16Vector4: ...
    @property
    def aggg(self) -> U16Vector4: ...
    @property
    def aggb(self) -> U16Vector4: ...
    @property
    def agga(self) -> U16Vector4: ...
    @property
    def aggo(self) -> U16Vector4: ...
    @property
    def aggl(self) -> U16Vector4: ...
    @property
    def agbr(self) -> U16Vector4: ...
    @property
    def agbg(self) -> U16Vector4: ...
    @property
    def agbb(self) -> U16Vector4: ...
    @property
    def agba(self) -> U16Vector4: ...
    @property
    def agbo(self) -> U16Vector4: ...
    @property
    def agbl(self) -> U16Vector4: ...
    @property
    def agar(self) -> U16Vector4: ...
    @property
    def agag(self) -> U16Vector4: ...
    @property
    def agab(self) -> U16Vector4: ...
    @property
    def agaa(self) -> U16Vector4: ...
    @property
    def agao(self) -> U16Vector4: ...
    @property
    def agal(self) -> U16Vector4: ...
    @property
    def agor(self) -> U16Vector4: ...
    @property
    def agog(self) -> U16Vector4: ...
    @property
    def agob(self) -> U16Vector4: ...
    @property
    def agoa(self) -> U16Vector4: ...
    @property
    def agoo(self) -> U16Vector4: ...
    @property
    def agol(self) -> U16Vector4: ...
    @property
    def aglr(self) -> U16Vector4: ...
    @property
    def aglg(self) -> U16Vector4: ...
    @property
    def aglb(self) -> U16Vector4: ...
    @property
    def agla(self) -> U16Vector4: ...
    @property
    def aglo(self) -> U16Vector4: ...
    @property
    def agll(self) -> U16Vector4: ...
    @property
    def abrr(self) -> U16Vector4: ...
    @property
    def abrg(self) -> U16Vector4: ...
    @property
    def abrb(self) -> U16Vector4: ...
    @property
    def abra(self) -> U16Vector4: ...
    @property
    def abro(self) -> U16Vector4: ...
    @property
    def abrl(self) -> U16Vector4: ...
    @property
    def abgr(self) -> U16Vector4: ...
    @property
    def abgg(self) -> U16Vector4: ...
    @property
    def abgb(self) -> U16Vector4: ...
    @property
    def abga(self) -> U16Vector4: ...
    @property
    def abgo(self) -> U16Vector4: ...
    @property
    def abgl(self) -> U16Vector4: ...
    @property
    def abbr(self) -> U16Vector4: ...
    @property
    def abbg(self) -> U16Vector4: ...
    @property
    def abbb(self) -> U16Vector4: ...
    @property
    def abba(self) -> U16Vector4: ...
    @property
    def abbo(self) -> U16Vector4: ...
    @property
    def abbl(self) -> U16Vector4: ...
    @property
    def abar(self) -> U16Vector4: ...
    @property
    def abag(self) -> U16Vector4: ...
    @property
    def abab(self) -> U16Vector4: ...
    @property
    def abaa(self) -> U16Vector4: ...
    @property
    def abao(self) -> U16Vector4: ...
    @property
    def abal(self) -> U16Vector4: ...
    @property
    def abor(self) -> U16Vector4: ...
    @property
    def abog(self) -> U16Vector4: ...
    @property
    def abob(self) -> U16Vector4: ...
    @property
    def aboa(self) -> U16Vector4: ...
    @property
    def aboo(self) -> U16Vector4: ...
    @property
    def abol(self) -> U16Vector4: ...
    @property
    def ablr(self) -> U16Vector4: ...
    @property
    def ablg(self) -> U16Vector4: ...
    @property
    def ablb(self) -> U16Vector4: ...
    @property
    def abla(self) -> U16Vector4: ...
    @property
    def ablo(self) -> U16Vector4: ...
    @property
    def abll(self) -> U16Vector4: ...
    @property
    def aarr(self) -> U16Vector4: ...
    @property
    def aarg(self) -> U16Vector4: ...
    @property
    def aarb(self) -> U16Vector4: ...
    @property
    def aara(self) -> U16Vector4: ...
    @property
    def aaro(self) -> U16Vector4: ...
    @property
    def aarl(self) -> U16Vector4: ...
    @property
    def aagr(self) -> U16Vector4: ...
    @property
    def aagg(self) -> U16Vector4: ...
    @property
    def aagb(self) -> U16Vector4: ...
    @property
    def aaga(self) -> U16Vector4: ...
    @property
    def aago(self) -> U16Vector4: ...
    @property
    def aagl(self) -> U16Vector4: ...
    @property
    def aabr(self) -> U16Vector4: ...
    @property
    def aabg(self) -> U16Vector4: ...
    @property
    def aabb(self) -> U16Vector4: ...
    @property
    def aaba(self) -> U16Vector4: ...
    @property
    def aabo(self) -> U16Vector4: ...
    @property
    def aabl(self) -> U16Vector4: ...
    @property
    def aaar(self) -> U16Vector4: ...
    @property
    def aaag(self) -> U16Vector4: ...
    @property
    def aaab(self) -> U16Vector4: ...
    @property
    def aaaa(self) -> U16Vector4: ...
    @property
    def aaao(self) -> U16Vector4: ...
    @property
    def aaal(self) -> U16Vector4: ...
    @property
    def aaor(self) -> U16Vector4: ...
    @property
    def aaog(self) -> U16Vector4: ...
    @property
    def aaob(self) -> U16Vector4: ...
    @property
    def aaoa(self) -> U16Vector4: ...
    @property
    def aaoo(self) -> U16Vector4: ...
    @property
    def aaol(self) -> U16Vector4: ...
    @property
    def aalr(self) -> U16Vector4: ...
    @property
    def aalg(self) -> U16Vector4: ...
    @property
    def aalb(self) -> U16Vector4: ...
    @property
    def aala(self) -> U16Vector4: ...
    @property
    def aalo(self) -> U16Vector4: ...
    @property
    def aall(self) -> U16Vector4: ...
    @property
    def aorr(self) -> U16Vector4: ...
    @property
    def aorg(self) -> U16Vector4: ...
    @property
    def aorb(self) -> U16Vector4: ...
    @property
    def aora(self) -> U16Vector4: ...
    @property
    def aoro(self) -> U16Vector4: ...
    @property
    def aorl(self) -> U16Vector4: ...
    @property
    def aogr(self) -> U16Vector4: ...
    @property
    def aogg(self) -> U16Vector4: ...
    @property
    def aogb(self) -> U16Vector4: ...
    @property
    def aoga(self) -> U16Vector4: ...
    @property
    def aogo(self) -> U16Vector4: ...
    @property
    def aogl(self) -> U16Vector4: ...
    @property
    def aobr(self) -> U16Vector4: ...
    @property
    def aobg(self) -> U16Vector4: ...
    @property
    def aobb(self) -> U16Vector4: ...
    @property
    def aoba(self) -> U16Vector4: ...
    @property
    def aobo(self) -> U16Vector4: ...
    @property
    def aobl(self) -> U16Vector4: ...
    @property
    def aoar(self) -> U16Vector4: ...
    @property
    def aoag(self) -> U16Vector4: ...
    @property
    def aoab(self) -> U16Vector4: ...
    @property
    def aoaa(self) -> U16Vector4: ...
    @property
    def aoao(self) -> U16Vector4: ...
    @property
    def aoal(self) -> U16Vector4: ...
    @property
    def aoor(self) -> U16Vector4: ...
    @property
    def aoog(self) -> U16Vector4: ...
    @property
    def aoob(self) -> U16Vector4: ...
    @property
    def aooa(self) -> U16Vector4: ...
    @property
    def aooo(self) -> U16Vector4: ...
    @property
    def aool(self) -> U16Vector4: ...
    @property
    def aolr(self) -> U16Vector4: ...
    @property
    def aolg(self) -> U16Vector4: ...
    @property
    def aolb(self) -> U16Vector4: ...
    @property
    def aola(self) -> U16Vector4: ...
    @property
    def aolo(self) -> U16Vector4: ...
    @property
    def aoll(self) -> U16Vector4: ...
    @property
    def alrr(self) -> U16Vector4: ...
    @property
    def alrg(self) -> U16Vector4: ...
    @property
    def alrb(self) -> U16Vector4: ...
    @property
    def alra(self) -> U16Vector4: ...
    @property
    def alro(self) -> U16Vector4: ...
    @property
    def alrl(self) -> U16Vector4: ...
    @property
    def algr(self) -> U16Vector4: ...
    @property
    def algg(self) -> U16Vector4: ...
    @property
    def algb(self) -> U16Vector4: ...
    @property
    def alga(self) -> U16Vector4: ...
    @property
    def algo(self) -> U16Vector4: ...
    @property
    def algl(self) -> U16Vector4: ...
    @property
    def albr(self) -> U16Vector4: ...
    @property
    def albg(self) -> U16Vector4: ...
    @property
    def albb(self) -> U16Vector4: ...
    @property
    def alba(self) -> U16Vector4: ...
    @property
    def albo(self) -> U16Vector4: ...
    @property
    def albl(self) -> U16Vector4: ...
    @property
    def alar(self) -> U16Vector4: ...
    @property
    def alag(self) -> U16Vector4: ...
    @property
    def alab(self) -> U16Vector4: ...
    @property
    def alaa(self) -> U16Vector4: ...
    @property
    def alao(self) -> U16Vector4: ...
    @property
    def alal(self) -> U16Vector4: ...
    @property
    def alor(self) -> U16Vector4: ...
    @property
    def alog(self) -> U16Vector4: ...
    @property
    def alob(self) -> U16Vector4: ...
    @property
    def aloa(self) -> U16Vector4: ...
    @property
    def aloo(self) -> U16Vector4: ...
    @property
    def alol(self) -> U16Vector4: ...
    @property
    def allr(self) -> U16Vector4: ...
    @property
    def allg(self) -> U16Vector4: ...
    @property
    def allb(self) -> U16Vector4: ...
    @property
    def alla(self) -> U16Vector4: ...
    @property
    def allo(self) -> U16Vector4: ...
    @property
    def alll(self) -> U16Vector4: ...
    @property
    def orrr(self) -> U16Vector4: ...
    @property
    def orrg(self) -> U16Vector4: ...
    @property
    def orrb(self) -> U16Vector4: ...
    @property
    def orra(self) -> U16Vector4: ...
    @property
    def orro(self) -> U16Vector4: ...
    @property
    def orrl(self) -> U16Vector4: ...
    @property
    def orgr(self) -> U16Vector4: ...
    @property
    def orgg(self) -> U16Vector4: ...
    @property
    def orgb(self) -> U16Vector4: ...
    @property
    def orga(self) -> U16Vector4: ...
    @property
    def orgo(self) -> U16Vector4: ...
    @property
    def orgl(self) -> U16Vector4: ...
    @property
    def orbr(self) -> U16Vector4: ...
    @property
    def orbg(self) -> U16Vector4: ...
    @property
    def orbb(self) -> U16Vector4: ...
    @property
    def orba(self) -> U16Vector4: ...
    @property
    def orbo(self) -> U16Vector4: ...
    @property
    def orbl(self) -> U16Vector4: ...
    @property
    def orar(self) -> U16Vector4: ...
    @property
    def orag(self) -> U16Vector4: ...
    @property
    def orab(self) -> U16Vector4: ...
    @property
    def oraa(self) -> U16Vector4: ...
    @property
    def orao(self) -> U16Vector4: ...
    @property
    def oral(self) -> U16Vector4: ...
    @property
    def oror(self) -> U16Vector4: ...
    @property
    def orog(self) -> U16Vector4: ...
    @property
    def orob(self) -> U16Vector4: ...
    @property
    def oroa(self) -> U16Vector4: ...
    @property
    def oroo(self) -> U16Vector4: ...
    @property
    def orol(self) -> U16Vector4: ...
    @property
    def orlr(self) -> U16Vector4: ...
    @property
    def orlg(self) -> U16Vector4: ...
    @property
    def orlb(self) -> U16Vector4: ...
    @property
    def orla(self) -> U16Vector4: ...
    @property
    def orlo(self) -> U16Vector4: ...
    @property
    def orll(self) -> U16Vector4: ...
    @property
    def ogrr(self) -> U16Vector4: ...
    @property
    def ogrg(self) -> U16Vector4: ...
    @property
    def ogrb(self) -> U16Vector4: ...
    @property
    def ogra(self) -> U16Vector4: ...
    @property
    def ogro(self) -> U16Vector4: ...
    @property
    def ogrl(self) -> U16Vector4: ...
    @property
    def oggr(self) -> U16Vector4: ...
    @property
    def oggg(self) -> U16Vector4: ...
    @property
    def oggb(self) -> U16Vector4: ...
    @property
    def ogga(self) -> U16Vector4: ...
    @property
    def oggo(self) -> U16Vector4: ...
    @property
    def oggl(self) -> U16Vector4: ...
    @property
    def ogbr(self) -> U16Vector4: ...
    @property
    def ogbg(self) -> U16Vector4: ...
    @property
    def ogbb(self) -> U16Vector4: ...
    @property
    def ogba(self) -> U16Vector4: ...
    @property
    def ogbo(self) -> U16Vector4: ...
    @property
    def ogbl(self) -> U16Vector4: ...
    @property
    def ogar(self) -> U16Vector4: ...
    @property
    def ogag(self) -> U16Vector4: ...
    @property
    def ogab(self) -> U16Vector4: ...
    @property
    def ogaa(self) -> U16Vector4: ...
    @property
    def ogao(self) -> U16Vector4: ...
    @property
    def ogal(self) -> U16Vector4: ...
    @property
    def ogor(self) -> U16Vector4: ...
    @property
    def ogog(self) -> U16Vector4: ...
    @property
    def ogob(self) -> U16Vector4: ...
    @property
    def ogoa(self) -> U16Vector4: ...
    @property
    def ogoo(self) -> U16Vector4: ...
    @property
    def ogol(self) -> U16Vector4: ...
    @property
    def oglr(self) -> U16Vector4: ...
    @property
    def oglg(self) -> U16Vector4: ...
    @property
    def oglb(self) -> U16Vector4: ...
    @property
    def ogla(self) -> U16Vector4: ...
    @property
    def oglo(self) -> U16Vector4: ...
    @property
    def ogll(self) -> U16Vector4: ...
    @property
    def obrr(self) -> U16Vector4: ...
    @property
    def obrg(self) -> U16Vector4: ...
    @property
    def obrb(self) -> U16Vector4: ...
    @property
    def obra(self) -> U16Vector4: ...
    @property
    def obro(self) -> U16Vector4: ...
    @property
    def obrl(self) -> U16Vector4: ...
    @property
    def obgr(self) -> U16Vector4: ...
    @property
    def obgg(self) -> U16Vector4: ...
    @property
    def obgb(self) -> U16Vector4: ...
    @property
    def obga(self) -> U16Vector4: ...
    @property
    def obgo(self) -> U16Vector4: ...
    @property
    def obgl(self) -> U16Vector4: ...
    @property
    def obbr(self) -> U16Vector4: ...
    @property
    def obbg(self) -> U16Vector4: ...
    @property
    def obbb(self) -> U16Vector4: ...
    @property
    def obba(self) -> U16Vector4: ...
    @property
    def obbo(self) -> U16Vector4: ...
    @property
    def obbl(self) -> U16Vector4: ...
    @property
    def obar(self) -> U16Vector4: ...
    @property
    def obag(self) -> U16Vector4: ...
    @property
    def obab(self) -> U16Vector4: ...
    @property
    def obaa(self) -> U16Vector4: ...
    @property
    def obao(self) -> U16Vector4: ...
    @property
    def obal(self) -> U16Vector4: ...
    @property
    def obor(self) -> U16Vector4: ...
    @property
    def obog(self) -> U16Vector4: ...
    @property
    def obob(self) -> U16Vector4: ...
    @property
    def oboa(self) -> U16Vector4: ...
    @property
    def oboo(self) -> U16Vector4: ...
    @property
    def obol(self) -> U16Vector4: ...
    @property
    def oblr(self) -> U16Vector4: ...
    @property
    def oblg(self) -> U16Vector4: ...
    @property
    def oblb(self) -> U16Vector4: ...
    @property
    def obla(self) -> U16Vector4: ...
    @property
    def oblo(self) -> U16Vector4: ...
    @property
    def obll(self) -> U16Vector4: ...
    @property
    def oarr(self) -> U16Vector4: ...
    @property
    def oarg(self) -> U16Vector4: ...
    @property
    def oarb(self) -> U16Vector4: ...
    @property
    def oara(self) -> U16Vector4: ...
    @property
    def oaro(self) -> U16Vector4: ...
    @property
    def oarl(self) -> U16Vector4: ...
    @property
    def oagr(self) -> U16Vector4: ...
    @property
    def oagg(self) -> U16Vector4: ...
    @property
    def oagb(self) -> U16Vector4: ...
    @property
    def oaga(self) -> U16Vector4: ...
    @property
    def oago(self) -> U16Vector4: ...
    @property
    def oagl(self) -> U16Vector4: ...
    @property
    def oabr(self) -> U16Vector4: ...
    @property
    def oabg(self) -> U16Vector4: ...
    @property
    def oabb(self) -> U16Vector4: ...
    @property
    def oaba(self) -> U16Vector4: ...
    @property
    def oabo(self) -> U16Vector4: ...
    @property
    def oabl(self) -> U16Vector4: ...
    @property
    def oaar(self) -> U16Vector4: ...
    @property
    def oaag(self) -> U16Vector4: ...
    @property
    def oaab(self) -> U16Vector4: ...
    @property
    def oaaa(self) -> U16Vector4: ...
    @property
    def oaao(self) -> U16Vector4: ...
    @property
    def oaal(self) -> U16Vector4: ...
    @property
    def oaor(self) -> U16Vector4: ...
    @property
    def oaog(self) -> U16Vector4: ...
    @property
    def oaob(self) -> U16Vector4: ...
    @property
    def oaoa(self) -> U16Vector4: ...
    @property
    def oaoo(self) -> U16Vector4: ...
    @property
    def oaol(self) -> U16Vector4: ...
    @property
    def oalr(self) -> U16Vector4: ...
    @property
    def oalg(self) -> U16Vector4: ...
    @property
    def oalb(self) -> U16Vector4: ...
    @property
    def oala(self) -> U16Vector4: ...
    @property
    def oalo(self) -> U16Vector4: ...
    @property
    def oall(self) -> U16Vector4: ...
    @property
    def oorr(self) -> U16Vector4: ...
    @property
    def oorg(self) -> U16Vector4: ...
    @property
    def oorb(self) -> U16Vector4: ...
    @property
    def oora(self) -> U16Vector4: ...
    @property
    def ooro(self) -> U16Vector4: ...
    @property
    def oorl(self) -> U16Vector4: ...
    @property
    def oogr(self) -> U16Vector4: ...
    @property
    def oogg(self) -> U16Vector4: ...
    @property
    def oogb(self) -> U16Vector4: ...
    @property
    def ooga(self) -> U16Vector4: ...
    @property
    def oogo(self) -> U16Vector4: ...
    @property
    def oogl(self) -> U16Vector4: ...
    @property
    def oobr(self) -> U16Vector4: ...
    @property
    def oobg(self) -> U16Vector4: ...
    @property
    def oobb(self) -> U16Vector4: ...
    @property
    def ooba(self) -> U16Vector4: ...
    @property
    def oobo(self) -> U16Vector4: ...
    @property
    def oobl(self) -> U16Vector4: ...
    @property
    def ooar(self) -> U16Vector4: ...
    @property
    def ooag(self) -> U16Vector4: ...
    @property
    def ooab(self) -> U16Vector4: ...
    @property
    def ooaa(self) -> U16Vector4: ...
    @property
    def ooao(self) -> U16Vector4: ...
    @property
    def ooal(self) -> U16Vector4: ...
    @property
    def ooor(self) -> U16Vector4: ...
    @property
    def ooog(self) -> U16Vector4: ...
    @property
    def ooob(self) -> U16Vector4: ...
    @property
    def oooa(self) -> U16Vector4: ...
    @property
    def oolr(self) -> U16Vector4: ...
    @property
    def oolg(self) -> U16Vector4: ...
    @property
    def oolb(self) -> U16Vector4: ...
    @property
    def oola(self) -> U16Vector4: ...
    @property
    def olrr(self) -> U16Vector4: ...
    @property
    def olrg(self) -> U16Vector4: ...
    @property
    def olrb(self) -> U16Vector4: ...
    @property
    def olra(self) -> U16Vector4: ...
    @property
    def olro(self) -> U16Vector4: ...
    @property
    def olrl(self) -> U16Vector4: ...
    @property
    def olgr(self) -> U16Vector4: ...
    @property
    def olgg(self) -> U16Vector4: ...
    @property
    def olgb(self) -> U16Vector4: ...
    @property
    def olga(self) -> U16Vector4: ...
    @property
    def olgo(self) -> U16Vector4: ...
    @property
    def olgl(self) -> U16Vector4: ...
    @property
    def olbr(self) -> U16Vector4: ...
    @property
    def olbg(self) -> U16Vector4: ...
    @property
    def olbb(self) -> U16Vector4: ...
    @property
    def olba(self) -> U16Vector4: ...
    @property
    def olbo(self) -> U16Vector4: ...
    @property
    def olbl(self) -> U16Vector4: ...
    @property
    def olar(self) -> U16Vector4: ...
    @property
    def olag(self) -> U16Vector4: ...
    @property
    def olab(self) -> U16Vector4: ...
    @property
    def olaa(self) -> U16Vector4: ...
    @property
    def olao(self) -> U16Vector4: ...
    @property
    def olal(self) -> U16Vector4: ...
    @property
    def olor(self) -> U16Vector4: ...
    @property
    def olog(self) -> U16Vector4: ...
    @property
    def olob(self) -> U16Vector4: ...
    @property
    def oloa(self) -> U16Vector4: ...
    @property
    def ollr(self) -> U16Vector4: ...
    @property
    def ollg(self) -> U16Vector4: ...
    @property
    def ollb(self) -> U16Vector4: ...
    @property
    def olla(self) -> U16Vector4: ...
    @property
    def lrrr(self) -> U16Vector4: ...
    @property
    def lrrg(self) -> U16Vector4: ...
    @property
    def lrrb(self) -> U16Vector4: ...
    @property
    def lrra(self) -> U16Vector4: ...
    @property
    def lrro(self) -> U16Vector4: ...
    @property
    def lrrl(self) -> U16Vector4: ...
    @property
    def lrgr(self) -> U16Vector4: ...
    @property
    def lrgg(self) -> U16Vector4: ...
    @property
    def lrgb(self) -> U16Vector4: ...
    @property
    def lrga(self) -> U16Vector4: ...
    @property
    def lrgo(self) -> U16Vector4: ...
    @property
    def lrgl(self) -> U16Vector4: ...
    @property
    def lrbr(self) -> U16Vector4: ...
    @property
    def lrbg(self) -> U16Vector4: ...
    @property
    def lrbb(self) -> U16Vector4: ...
    @property
    def lrba(self) -> U16Vector4: ...
    @property
    def lrbo(self) -> U16Vector4: ...
    @property
    def lrbl(self) -> U16Vector4: ...
    @property
    def lrar(self) -> U16Vector4: ...
    @property
    def lrag(self) -> U16Vector4: ...
    @property
    def lrab(self) -> U16Vector4: ...
    @property
    def lraa(self) -> U16Vector4: ...
    @property
    def lrao(self) -> U16Vector4: ...
    @property
    def lral(self) -> U16Vector4: ...
    @property
    def lror(self) -> U16Vector4: ...
    @property
    def lrog(self) -> U16Vector4: ...
    @property
    def lrob(self) -> U16Vector4: ...
    @property
    def lroa(self) -> U16Vector4: ...
    @property
    def lroo(self) -> U16Vector4: ...
    @property
    def lrol(self) -> U16Vector4: ...
    @property
    def lrlr(self) -> U16Vector4: ...
    @property
    def lrlg(self) -> U16Vector4: ...
    @property
    def lrlb(self) -> U16Vector4: ...
    @property
    def lrla(self) -> U16Vector4: ...
    @property
    def lrlo(self) -> U16Vector4: ...
    @property
    def lrll(self) -> U16Vector4: ...
    @property
    def lgrr(self) -> U16Vector4: ...
    @property
    def lgrg(self) -> U16Vector4: ...
    @property
    def lgrb(self) -> U16Vector4: ...
    @property
    def lgra(self) -> U16Vector4: ...
    @property
    def lgro(self) -> U16Vector4: ...
    @property
    def lgrl(self) -> U16Vector4: ...
    @property
    def lggr(self) -> U16Vector4: ...
    @property
    def lggg(self) -> U16Vector4: ...
    @property
    def lggb(self) -> U16Vector4: ...
    @property
    def lgga(self) -> U16Vector4: ...
    @property
    def lggo(self) -> U16Vector4: ...
    @property
    def lggl(self) -> U16Vector4: ...
    @property
    def lgbr(self) -> U16Vector4: ...
    @property
    def lgbg(self) -> U16Vector4: ...
    @property
    def lgbb(self) -> U16Vector4: ...
    @property
    def lgba(self) -> U16Vector4: ...
    @property
    def lgbo(self) -> U16Vector4: ...
    @property
    def lgbl(self) -> U16Vector4: ...
    @property
    def lgar(self) -> U16Vector4: ...
    @property
    def lgag(self) -> U16Vector4: ...
    @property
    def lgab(self) -> U16Vector4: ...
    @property
    def lgaa(self) -> U16Vector4: ...
    @property
    def lgao(self) -> U16Vector4: ...
    @property
    def lgal(self) -> U16Vector4: ...
    @property
    def lgor(self) -> U16Vector4: ...
    @property
    def lgog(self) -> U16Vector4: ...
    @property
    def lgob(self) -> U16Vector4: ...
    @property
    def lgoa(self) -> U16Vector4: ...
    @property
    def lgoo(self) -> U16Vector4: ...
    @property
    def lgol(self) -> U16Vector4: ...
    @property
    def lglr(self) -> U16Vector4: ...
    @property
    def lglg(self) -> U16Vector4: ...
    @property
    def lglb(self) -> U16Vector4: ...
    @property
    def lgla(self) -> U16Vector4: ...
    @property
    def lglo(self) -> U16Vector4: ...
    @property
    def lgll(self) -> U16Vector4: ...
    @property
    def lbrr(self) -> U16Vector4: ...
    @property
    def lbrg(self) -> U16Vector4: ...
    @property
    def lbrb(self) -> U16Vector4: ...
    @property
    def lbra(self) -> U16Vector4: ...
    @property
    def lbro(self) -> U16Vector4: ...
    @property
    def lbrl(self) -> U16Vector4: ...
    @property
    def lbgr(self) -> U16Vector4: ...
    @property
    def lbgg(self) -> U16Vector4: ...
    @property
    def lbgb(self) -> U16Vector4: ...
    @property
    def lbga(self) -> U16Vector4: ...
    @property
    def lbgo(self) -> U16Vector4: ...
    @property
    def lbgl(self) -> U16Vector4: ...
    @property
    def lbbr(self) -> U16Vector4: ...
    @property
    def lbbg(self) -> U16Vector4: ...
    @property
    def lbbb(self) -> U16Vector4: ...
    @property
    def lbba(self) -> U16Vector4: ...
    @property
    def lbbo(self) -> U16Vector4: ...
    @property
    def lbbl(self) -> U16Vector4: ...
    @property
    def lbar(self) -> U16Vector4: ...
    @property
    def lbag(self) -> U16Vector4: ...
    @property
    def lbab(self) -> U16Vector4: ...
    @property
    def lbaa(self) -> U16Vector4: ...
    @property
    def lbao(self) -> U16Vector4: ...
    @property
    def lbal(self) -> U16Vector4: ...
    @property
    def lbor(self) -> U16Vector4: ...
    @property
    def lbog(self) -> U16Vector4: ...
    @property
    def lbob(self) -> U16Vector4: ...
    @property
    def lboa(self) -> U16Vector4: ...
    @property
    def lboo(self) -> U16Vector4: ...
    @property
    def lbol(self) -> U16Vector4: ...
    @property
    def lblr(self) -> U16Vector4: ...
    @property
    def lblg(self) -> U16Vector4: ...
    @property
    def lblb(self) -> U16Vector4: ...
    @property
    def lbla(self) -> U16Vector4: ...
    @property
    def lblo(self) -> U16Vector4: ...
    @property
    def lbll(self) -> U16Vector4: ...
    @property
    def larr(self) -> U16Vector4: ...
    @property
    def larg(self) -> U16Vector4: ...
    @property
    def larb(self) -> U16Vector4: ...
    @property
    def lara(self) -> U16Vector4: ...
    @property
    def laro(self) -> U16Vector4: ...
    @property
    def larl(self) -> U16Vector4: ...
    @property
    def lagr(self) -> U16Vector4: ...
    @property
    def lagg(self) -> U16Vector4: ...
    @property
    def lagb(self) -> U16Vector4: ...
    @property
    def laga(self) -> U16Vector4: ...
    @property
    def lago(self) -> U16Vector4: ...
    @property
    def lagl(self) -> U16Vector4: ...
    @property
    def labr(self) -> U16Vector4: ...
    @property
    def labg(self) -> U16Vector4: ...
    @property
    def labb(self) -> U16Vector4: ...
    @property
    def laba(self) -> U16Vector4: ...
    @property
    def labo(self) -> U16Vector4: ...
    @property
    def labl(self) -> U16Vector4: ...
    @property
    def laar(self) -> U16Vector4: ...
    @property
    def laag(self) -> U16Vector4: ...
    @property
    def laab(self) -> U16Vector4: ...
    @property
    def laaa(self) -> U16Vector4: ...
    @property
    def laao(self) -> U16Vector4: ...
    @property
    def laal(self) -> U16Vector4: ...
    @property
    def laor(self) -> U16Vector4: ...
    @property
    def laog(self) -> U16Vector4: ...
    @property
    def laob(self) -> U16Vector4: ...
    @property
    def laoa(self) -> U16Vector4: ...
    @property
    def laoo(self) -> U16Vector4: ...
    @property
    def laol(self) -> U16Vector4: ...
    @property
    def lalr(self) -> U16Vector4: ...
    @property
    def lalg(self) -> U16Vector4: ...
    @property
    def lalb(self) -> U16Vector4: ...
    @property
    def lala(self) -> U16Vector4: ...
    @property
    def lalo(self) -> U16Vector4: ...
    @property
    def lall(self) -> U16Vector4: ...
    @property
    def lorr(self) -> U16Vector4: ...
    @property
    def lorg(self) -> U16Vector4: ...
    @property
    def lorb(self) -> U16Vector4: ...
    @property
    def lora(self) -> U16Vector4: ...
    @property
    def loro(self) -> U16Vector4: ...
    @property
    def lorl(self) -> U16Vector4: ...
    @property
    def logr(self) -> U16Vector4: ...
    @property
    def logg(self) -> U16Vector4: ...
    @property
    def logb(self) -> U16Vector4: ...
    @property
    def loga(self) -> U16Vector4: ...
    @property
    def logo(self) -> U16Vector4: ...
    @property
    def logl(self) -> U16Vector4: ...
    @property
    def lobr(self) -> U16Vector4: ...
    @property
    def lobg(self) -> U16Vector4: ...
    @property
    def lobb(self) -> U16Vector4: ...
    @property
    def loba(self) -> U16Vector4: ...
    @property
    def lobo(self) -> U16Vector4: ...
    @property
    def lobl(self) -> U16Vector4: ...
    @property
    def loar(self) -> U16Vector4: ...
    @property
    def loag(self) -> U16Vector4: ...
    @property
    def loab(self) -> U16Vector4: ...
    @property
    def loaa(self) -> U16Vector4: ...
    @property
    def loao(self) -> U16Vector4: ...
    @property
    def loal(self) -> U16Vector4: ...
    @property
    def loor(self) -> U16Vector4: ...
    @property
    def loog(self) -> U16Vector4: ...
    @property
    def loob(self) -> U16Vector4: ...
    @property
    def looa(self) -> U16Vector4: ...
    @property
    def lolr(self) -> U16Vector4: ...
    @property
    def lolg(self) -> U16Vector4: ...
    @property
    def lolb(self) -> U16Vector4: ...
    @property
    def lola(self) -> U16Vector4: ...
    @property
    def llrr(self) -> U16Vector4: ...
    @property
    def llrg(self) -> U16Vector4: ...
    @property
    def llrb(self) -> U16Vector4: ...
    @property
    def llra(self) -> U16Vector4: ...
    @property
    def llro(self) -> U16Vector4: ...
    @property
    def llrl(self) -> U16Vector4: ...
    @property
    def llgr(self) -> U16Vector4: ...
    @property
    def llgg(self) -> U16Vector4: ...
    @property
    def llgb(self) -> U16Vector4: ...
    @property
    def llga(self) -> U16Vector4: ...
    @property
    def llgo(self) -> U16Vector4: ...
    @property
    def llgl(self) -> U16Vector4: ...
    @property
    def llbr(self) -> U16Vector4: ...
    @property
    def llbg(self) -> U16Vector4: ...
    @property
    def llbb(self) -> U16Vector4: ...
    @property
    def llba(self) -> U16Vector4: ...
    @property
    def llbo(self) -> U16Vector4: ...
    @property
    def llbl(self) -> U16Vector4: ...
    @property
    def llar(self) -> U16Vector4: ...
    @property
    def llag(self) -> U16Vector4: ...
    @property
    def llab(self) -> U16Vector4: ...
    @property
    def llaa(self) -> U16Vector4: ...
    @property
    def llao(self) -> U16Vector4: ...
    @property
    def llal(self) -> U16Vector4: ...
    @property
    def llor(self) -> U16Vector4: ...
    @property
    def llog(self) -> U16Vector4: ...
    @property
    def llob(self) -> U16Vector4: ...
    @property
    def lloa(self) -> U16Vector4: ...
    @property
    def lllr(self) -> U16Vector4: ...
    @property
    def lllg(self) -> U16Vector4: ...
    @property
    def lllb(self) -> U16Vector4: ...
    @property
    def llla(self) -> U16Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> U16Vector2: ...
    @property
    def st(self) -> U16Vector2: ...
    @property
    def sq(self) -> U16Vector2: ...
    @property
    def sp(self) -> U16Vector2: ...
    @property
    def so(self) -> U16Vector2: ...
    @property
    def sl(self) -> U16Vector2: ...
    @property
    def ts(self) -> U16Vector2: ...
    @property
    def tt(self) -> U16Vector2: ...
    @property
    def tq(self) -> U16Vector2: ...
    @property
    def tp(self) -> U16Vector2: ...
    @property
    def to(self) -> U16Vector2: ...
    @property
    def tl(self) -> U16Vector2: ...
    @property
    def qs(self) -> U16Vector2: ...
    @property
    def qt(self) -> U16Vector2: ...
    @property
    def qq(self) -> U16Vector2: ...
    @property
    def qp(self) -> U16Vector2: ...
    @property
    def qo(self) -> U16Vector2: ...
    @property
    def ql(self) -> U16Vector2: ...
    @property
    def ps(self) -> U16Vector2: ...
    @property
    def pt(self) -> U16Vector2: ...
    @property
    def pq(self) -> U16Vector2: ...
    @property
    def pp(self) -> U16Vector2: ...
    @property
    def po(self) -> U16Vector2: ...
    @property
    def pl(self) -> U16Vector2: ...
    @property
    def os(self) -> U16Vector2: ...
    @property
    def ot(self) -> U16Vector2: ...
    @property
    def oq(self) -> U16Vector2: ...
    @property
    def op(self) -> U16Vector2: ...
    @property
    def ls(self) -> U16Vector2: ...
    @property
    def lt(self) -> U16Vector2: ...
    @property
    def lq(self) -> U16Vector2: ...
    @property
    def lp(self) -> U16Vector2: ...
    @property
    def sss(self) -> U16Vector3: ...
    @property
    def sst(self) -> U16Vector3: ...
    @property
    def ssq(self) -> U16Vector3: ...
    @property
    def ssp(self) -> U16Vector3: ...
    @property
    def sso(self) -> U16Vector3: ...
    @property
    def ssl(self) -> U16Vector3: ...
    @property
    def sts(self) -> U16Vector3: ...
    @property
    def stt(self) -> U16Vector3: ...
    @property
    def stq(self) -> U16Vector3: ...
    @property
    def stp(self) -> U16Vector3: ...
    @property
    def sto(self) -> U16Vector3: ...
    @property
    def stl(self) -> U16Vector3: ...
    @property
    def sqs(self) -> U16Vector3: ...
    @property
    def sqt(self) -> U16Vector3: ...
    @property
    def sqq(self) -> U16Vector3: ...
    @property
    def sqp(self) -> U16Vector3: ...
    @property
    def sqo(self) -> U16Vector3: ...
    @property
    def sql(self) -> U16Vector3: ...
    @property
    def sps(self) -> U16Vector3: ...
    @property
    def spt(self) -> U16Vector3: ...
    @property
    def spq(self) -> U16Vector3: ...
    @property
    def spp(self) -> U16Vector3: ...
    @property
    def spo(self) -> U16Vector3: ...
    @property
    def spl(self) -> U16Vector3: ...
    @property
    def sos(self) -> U16Vector3: ...
    @property
    def sot(self) -> U16Vector3: ...
    @property
    def soq(self) -> U16Vector3: ...
    @property
    def sop(self) -> U16Vector3: ...
    @property
    def soo(self) -> U16Vector3: ...
    @property
    def sol(self) -> U16Vector3: ...
    @property
    def sls(self) -> U16Vector3: ...
    @property
    def slt(self) -> U16Vector3: ...
    @property
    def slq(self) -> U16Vector3: ...
    @property
    def slp(self) -> U16Vector3: ...
    @property
    def slo(self) -> U16Vector3: ...
    @property
    def sll(self) -> U16Vector3: ...
    @property
    def tss(self) -> U16Vector3: ...
    @property
    def tst(self) -> U16Vector3: ...
    @property
    def tsq(self) -> U16Vector3: ...
    @property
    def tsp(self) -> U16Vector3: ...
    @property
    def tso(self) -> U16Vector3: ...
    @property
    def tsl(self) -> U16Vector3: ...
    @property
    def tts(self) -> U16Vector3: ...
    @property
    def ttt(self) -> U16Vector3: ...
    @property
    def ttq(self) -> U16Vector3: ...
    @property
    def ttp(self) -> U16Vector3: ...
    @property
    def tto(self) -> U16Vector3: ...
    @property
    def ttl(self) -> U16Vector3: ...
    @property
    def tqs(self) -> U16Vector3: ...
    @property
    def tqt(self) -> U16Vector3: ...
    @property
    def tqq(self) -> U16Vector3: ...
    @property
    def tqp(self) -> U16Vector3: ...
    @property
    def tqo(self) -> U16Vector3: ...
    @property
    def tql(self) -> U16Vector3: ...
    @property
    def tps(self) -> U16Vector3: ...
    @property
    def tpt(self) -> U16Vector3: ...
    @property
    def tpq(self) -> U16Vector3: ...
    @property
    def tpp(self) -> U16Vector3: ...
    @property
    def tpo(self) -> U16Vector3: ...
    @property
    def tpl(self) -> U16Vector3: ...
    @property
    def tos(self) -> U16Vector3: ...
    @property
    def tot(self) -> U16Vector3: ...
    @property
    def toq(self) -> U16Vector3: ...
    @property
    def top(self) -> U16Vector3: ...
    @property
    def too(self) -> U16Vector3: ...
    @property
    def tol(self) -> U16Vector3: ...
    @property
    def tls(self) -> U16Vector3: ...
    @property
    def tlt(self) -> U16Vector3: ...
    @property
    def tlq(self) -> U16Vector3: ...
    @property
    def tlp(self) -> U16Vector3: ...
    @property
    def tlo(self) -> U16Vector3: ...
    @property
    def tll(self) -> U16Vector3: ...
    @property
    def qss(self) -> U16Vector3: ...
    @property
    def qst(self) -> U16Vector3: ...
    @property
    def qsq(self) -> U16Vector3: ...
    @property
    def qsp(self) -> U16Vector3: ...
    @property
    def qso(self) -> U16Vector3: ...
    @property
    def qsl(self) -> U16Vector3: ...
    @property
    def qts(self) -> U16Vector3: ...
    @property
    def qtt(self) -> U16Vector3: ...
    @property
    def qtq(self) -> U16Vector3: ...
    @property
    def qtp(self) -> U16Vector3: ...
    @property
    def qto(self) -> U16Vector3: ...
    @property
    def qtl(self) -> U16Vector3: ...
    @property
    def qqs(self) -> U16Vector3: ...
    @property
    def qqt(self) -> U16Vector3: ...
    @property
    def qqq(self) -> U16Vector3: ...
    @property
    def qqp(self) -> U16Vector3: ...
    @property
    def qqo(self) -> U16Vector3: ...
    @property
    def qql(self) -> U16Vector3: ...
    @property
    def qps(self) -> U16Vector3: ...
    @property
    def qpt(self) -> U16Vector3: ...
    @property
    def qpq(self) -> U16Vector3: ...
    @property
    def qpp(self) -> U16Vector3: ...
    @property
    def qpo(self) -> U16Vector3: ...
    @property
    def qpl(self) -> U16Vector3: ...
    @property
    def qos(self) -> U16Vector3: ...
    @property
    def qot(self) -> U16Vector3: ...
    @property
    def qoq(self) -> U16Vector3: ...
    @property
    def qop(self) -> U16Vector3: ...
    @property
    def qoo(self) -> U16Vector3: ...
    @property
    def qol(self) -> U16Vector3: ...
    @property
    def qls(self) -> U16Vector3: ...
    @property
    def qlt(self) -> U16Vector3: ...
    @property
    def qlq(self) -> U16Vector3: ...
    @property
    def qlp(self) -> U16Vector3: ...
    @property
    def qlo(self) -> U16Vector3: ...
    @property
    def qll(self) -> U16Vector3: ...
    @property
    def pss(self) -> U16Vector3: ...
    @property
    def pst(self) -> U16Vector3: ...
    @property
    def psq(self) -> U16Vector3: ...
    @property
    def psp(self) -> U16Vector3: ...
    @property
    def pso(self) -> U16Vector3: ...
    @property
    def psl(self) -> U16Vector3: ...
    @property
    def pts(self) -> U16Vector3: ...
    @property
    def ptt(self) -> U16Vector3: ...
    @property
    def ptq(self) -> U16Vector3: ...
    @property
    def ptp(self) -> U16Vector3: ...
    @property
    def pto(self) -> U16Vector3: ...
    @property
    def ptl(self) -> U16Vector3: ...
    @property
    def pqs(self) -> U16Vector3: ...
    @property
    def pqt(self) -> U16Vector3: ...
    @property
    def pqq(self) -> U16Vector3: ...
    @property
    def pqp(self) -> U16Vector3: ...
    @property
    def pqo(self) -> U16Vector3: ...
    @property
    def pql(self) -> U16Vector3: ...
    @property
    def pps(self) -> U16Vector3: ...
    @property
    def ppt(self) -> U16Vector3: ...
    @property
    def ppq(self) -> U16Vector3: ...
    @property
    def ppp(self) -> U16Vector3: ...
    @property
    def ppo(self) -> U16Vector3: ...
    @property
    def ppl(self) -> U16Vector3: ...
    @property
    def pos(self) -> U16Vector3: ...
    @property
    def pot(self) -> U16Vector3: ...
    @property
    def poq(self) -> U16Vector3: ...
    @property
    def pop(self) -> U16Vector3: ...
    @property
    def poo(self) -> U16Vector3: ...
    @property
    def pol(self) -> U16Vector3: ...
    @property
    def pls(self) -> U16Vector3: ...
    @property
    def plt(self) -> U16Vector3: ...
    @property
    def plq(self) -> U16Vector3: ...
    @property
    def plp(self) -> U16Vector3: ...
    @property
    def plo(self) -> U16Vector3: ...
    @property
    def pll(self) -> U16Vector3: ...
    @property
    def oss(self) -> U16Vector3: ...
    @property
    def ost(self) -> U16Vector3: ...
    @property
    def osq(self) -> U16Vector3: ...
    @property
    def osp(self) -> U16Vector3: ...
    @property
    def oso(self) -> U16Vector3: ...
    @property
    def osl(self) -> U16Vector3: ...
    @property
    def ots(self) -> U16Vector3: ...
    @property
    def ott(self) -> U16Vector3: ...
    @property
    def otq(self) -> U16Vector3: ...
    @property
    def otp(self) -> U16Vector3: ...
    @property
    def oto(self) -> U16Vector3: ...
    @property
    def otl(self) -> U16Vector3: ...
    @property
    def oqs(self) -> U16Vector3: ...
    @property
    def oqt(self) -> U16Vector3: ...
    @property
    def oqq(self) -> U16Vector3: ...
    @property
    def oqp(self) -> U16Vector3: ...
    @property
    def oqo(self) -> U16Vector3: ...
    @property
    def oql(self) -> U16Vector3: ...
    @property
    def ops(self) -> U16Vector3: ...
    @property
    def opt(self) -> U16Vector3: ...
    @property
    def opq(self) -> U16Vector3: ...
    @property
    def opp(self) -> U16Vector3: ...
    @property
    def opo(self) -> U16Vector3: ...
    @property
    def opl(self) -> U16Vector3: ...
    @property
    def oos(self) -> U16Vector3: ...
    @property
    def oot(self) -> U16Vector3: ...
    @property
    def ooq(self) -> U16Vector3: ...
    @property
    def oop(self) -> U16Vector3: ...
    @property
    def ols(self) -> U16Vector3: ...
    @property
    def olt(self) -> U16Vector3: ...
    @property
    def olq(self) -> U16Vector3: ...
    @property
    def olp(self) -> U16Vector3: ...
    @property
    def lss(self) -> U16Vector3: ...
    @property
    def lst(self) -> U16Vector3: ...
    @property
    def lsq(self) -> U16Vector3: ...
    @property
    def lsp(self) -> U16Vector3: ...
    @property
    def lso(self) -> U16Vector3: ...
    @property
    def lsl(self) -> U16Vector3: ...
    @property
    def lts(self) -> U16Vector3: ...
    @property
    def ltt(self) -> U16Vector3: ...
    @property
    def ltq(self) -> U16Vector3: ...
    @property
    def ltp(self) -> U16Vector3: ...
    @property
    def lto(self) -> U16Vector3: ...
    @property
    def ltl(self) -> U16Vector3: ...
    @property
    def lqs(self) -> U16Vector3: ...
    @property
    def lqt(self) -> U16Vector3: ...
    @property
    def lqq(self) -> U16Vector3: ...
    @property
    def lqp(self) -> U16Vector3: ...
    @property
    def lqo(self) -> U16Vector3: ...
    @property
    def lql(self) -> U16Vector3: ...
    @property
    def lps(self) -> U16Vector3: ...
    @property
    def lpt(self) -> U16Vector3: ...
    @property
    def lpq(self) -> U16Vector3: ...
    @property
    def lpp(self) -> U16Vector3: ...
    @property
    def lpo(self) -> U16Vector3: ...
    @property
    def lpl(self) -> U16Vector3: ...
    @property
    def los(self) -> U16Vector3: ...
    @property
    def lot(self) -> U16Vector3: ...
    @property
    def loq(self) -> U16Vector3: ...
    @property
    def lop(self) -> U16Vector3: ...
    @property
    def lls(self) -> U16Vector3: ...
    @property
    def llt(self) -> U16Vector3: ...
    @property
    def llq(self) -> U16Vector3: ...
    @property
    def llp(self) -> U16Vector3: ...
    @property
    def ssss(self) -> U16Vector4: ...
    @property
    def ssst(self) -> U16Vector4: ...
    @property
    def sssq(self) -> U16Vector4: ...
    @property
    def sssp(self) -> U16Vector4: ...
    @property
    def ssso(self) -> U16Vector4: ...
    @property
    def sssl(self) -> U16Vector4: ...
    @property
    def ssts(self) -> U16Vector4: ...
    @property
    def sstt(self) -> U16Vector4: ...
    @property
    def sstq(self) -> U16Vector4: ...
    @property
    def sstp(self) -> U16Vector4: ...
    @property
    def ssto(self) -> U16Vector4: ...
    @property
    def sstl(self) -> U16Vector4: ...
    @property
    def ssqs(self) -> U16Vector4: ...
    @property
    def ssqt(self) -> U16Vector4: ...
    @property
    def ssqq(self) -> U16Vector4: ...
    @property
    def ssqp(self) -> U16Vector4: ...
    @property
    def ssqo(self) -> U16Vector4: ...
    @property
    def ssql(self) -> U16Vector4: ...
    @property
    def ssps(self) -> U16Vector4: ...
    @property
    def sspt(self) -> U16Vector4: ...
    @property
    def sspq(self) -> U16Vector4: ...
    @property
    def sspp(self) -> U16Vector4: ...
    @property
    def sspo(self) -> U16Vector4: ...
    @property
    def sspl(self) -> U16Vector4: ...
    @property
    def ssos(self) -> U16Vector4: ...
    @property
    def ssot(self) -> U16Vector4: ...
    @property
    def ssoq(self) -> U16Vector4: ...
    @property
    def ssop(self) -> U16Vector4: ...
    @property
    def ssoo(self) -> U16Vector4: ...
    @property
    def ssol(self) -> U16Vector4: ...
    @property
    def ssls(self) -> U16Vector4: ...
    @property
    def sslt(self) -> U16Vector4: ...
    @property
    def sslq(self) -> U16Vector4: ...
    @property
    def sslp(self) -> U16Vector4: ...
    @property
    def sslo(self) -> U16Vector4: ...
    @property
    def ssll(self) -> U16Vector4: ...
    @property
    def stss(self) -> U16Vector4: ...
    @property
    def stst(self) -> U16Vector4: ...
    @property
    def stsq(self) -> U16Vector4: ...
    @property
    def stsp(self) -> U16Vector4: ...
    @property
    def stso(self) -> U16Vector4: ...
    @property
    def stsl(self) -> U16Vector4: ...
    @property
    def stts(self) -> U16Vector4: ...
    @property
    def sttt(self) -> U16Vector4: ...
    @property
    def sttq(self) -> U16Vector4: ...
    @property
    def sttp(self) -> U16Vector4: ...
    @property
    def stto(self) -> U16Vector4: ...
    @property
    def sttl(self) -> U16Vector4: ...
    @property
    def stqs(self) -> U16Vector4: ...
    @property
    def stqt(self) -> U16Vector4: ...
    @property
    def stqq(self) -> U16Vector4: ...
    @property
    def stqp(self) -> U16Vector4: ...
    @property
    def stqo(self) -> U16Vector4: ...
    @property
    def stql(self) -> U16Vector4: ...
    @property
    def stps(self) -> U16Vector4: ...
    @property
    def stpt(self) -> U16Vector4: ...
    @property
    def stpq(self) -> U16Vector4: ...
    @property
    def stpp(self) -> U16Vector4: ...
    @property
    def stpo(self) -> U16Vector4: ...
    @property
    def stpl(self) -> U16Vector4: ...
    @property
    def stos(self) -> U16Vector4: ...
    @property
    def stot(self) -> U16Vector4: ...
    @property
    def stoq(self) -> U16Vector4: ...
    @property
    def stop(self) -> U16Vector4: ...
    @property
    def stoo(self) -> U16Vector4: ...
    @property
    def stol(self) -> U16Vector4: ...
    @property
    def stls(self) -> U16Vector4: ...
    @property
    def stlt(self) -> U16Vector4: ...
    @property
    def stlq(self) -> U16Vector4: ...
    @property
    def stlp(self) -> U16Vector4: ...
    @property
    def stlo(self) -> U16Vector4: ...
    @property
    def stll(self) -> U16Vector4: ...
    @property
    def sqss(self) -> U16Vector4: ...
    @property
    def sqst(self) -> U16Vector4: ...
    @property
    def sqsq(self) -> U16Vector4: ...
    @property
    def sqsp(self) -> U16Vector4: ...
    @property
    def sqso(self) -> U16Vector4: ...
    @property
    def sqsl(self) -> U16Vector4: ...
    @property
    def sqts(self) -> U16Vector4: ...
    @property
    def sqtt(self) -> U16Vector4: ...
    @property
    def sqtq(self) -> U16Vector4: ...
    @property
    def sqtp(self) -> U16Vector4: ...
    @property
    def sqto(self) -> U16Vector4: ...
    @property
    def sqtl(self) -> U16Vector4: ...
    @property
    def sqqs(self) -> U16Vector4: ...
    @property
    def sqqt(self) -> U16Vector4: ...
    @property
    def sqqq(self) -> U16Vector4: ...
    @property
    def sqqp(self) -> U16Vector4: ...
    @property
    def sqqo(self) -> U16Vector4: ...
    @property
    def sqql(self) -> U16Vector4: ...
    @property
    def sqps(self) -> U16Vector4: ...
    @property
    def sqpt(self) -> U16Vector4: ...
    @property
    def sqpq(self) -> U16Vector4: ...
    @property
    def sqpp(self) -> U16Vector4: ...
    @property
    def sqpo(self) -> U16Vector4: ...
    @property
    def sqpl(self) -> U16Vector4: ...
    @property
    def sqos(self) -> U16Vector4: ...
    @property
    def sqot(self) -> U16Vector4: ...
    @property
    def sqoq(self) -> U16Vector4: ...
    @property
    def sqop(self) -> U16Vector4: ...
    @property
    def sqoo(self) -> U16Vector4: ...
    @property
    def sqol(self) -> U16Vector4: ...
    @property
    def sqls(self) -> U16Vector4: ...
    @property
    def sqlt(self) -> U16Vector4: ...
    @property
    def sqlq(self) -> U16Vector4: ...
    @property
    def sqlp(self) -> U16Vector4: ...
    @property
    def sqlo(self) -> U16Vector4: ...
    @property
    def sqll(self) -> U16Vector4: ...
    @property
    def spss(self) -> U16Vector4: ...
    @property
    def spst(self) -> U16Vector4: ...
    @property
    def spsq(self) -> U16Vector4: ...
    @property
    def spsp(self) -> U16Vector4: ...
    @property
    def spso(self) -> U16Vector4: ...
    @property
    def spsl(self) -> U16Vector4: ...
    @property
    def spts(self) -> U16Vector4: ...
    @property
    def sptt(self) -> U16Vector4: ...
    @property
    def sptq(self) -> U16Vector4: ...
    @property
    def sptp(self) -> U16Vector4: ...
    @property
    def spto(self) -> U16Vector4: ...
    @property
    def sptl(self) -> U16Vector4: ...
    @property
    def spqs(self) -> U16Vector4: ...
    @property
    def spqt(self) -> U16Vector4: ...
    @property
    def spqq(self) -> U16Vector4: ...
    @property
    def spqp(self) -> U16Vector4: ...
    @property
    def spqo(self) -> U16Vector4: ...
    @property
    def spql(self) -> U16Vector4: ...
    @property
    def spps(self) -> U16Vector4: ...
    @property
    def sppt(self) -> U16Vector4: ...
    @property
    def sppq(self) -> U16Vector4: ...
    @property
    def sppp(self) -> U16Vector4: ...
    @property
    def sppo(self) -> U16Vector4: ...
    @property
    def sppl(self) -> U16Vector4: ...
    @property
    def spos(self) -> U16Vector4: ...
    @property
    def spot(self) -> U16Vector4: ...
    @property
    def spoq(self) -> U16Vector4: ...
    @property
    def spop(self) -> U16Vector4: ...
    @property
    def spoo(self) -> U16Vector4: ...
    @property
    def spol(self) -> U16Vector4: ...
    @property
    def spls(self) -> U16Vector4: ...
    @property
    def splt(self) -> U16Vector4: ...
    @property
    def splq(self) -> U16Vector4: ...
    @property
    def splp(self) -> U16Vector4: ...
    @property
    def splo(self) -> U16Vector4: ...
    @property
    def spll(self) -> U16Vector4: ...
    @property
    def soss(self) -> U16Vector4: ...
    @property
    def sost(self) -> U16Vector4: ...
    @property
    def sosq(self) -> U16Vector4: ...
    @property
    def sosp(self) -> U16Vector4: ...
    @property
    def soso(self) -> U16Vector4: ...
    @property
    def sosl(self) -> U16Vector4: ...
    @property
    def sots(self) -> U16Vector4: ...
    @property
    def sott(self) -> U16Vector4: ...
    @property
    def sotq(self) -> U16Vector4: ...
    @property
    def sotp(self) -> U16Vector4: ...
    @property
    def soto(self) -> U16Vector4: ...
    @property
    def sotl(self) -> U16Vector4: ...
    @property
    def soqs(self) -> U16Vector4: ...
    @property
    def soqt(self) -> U16Vector4: ...
    @property
    def soqq(self) -> U16Vector4: ...
    @property
    def soqp(self) -> U16Vector4: ...
    @property
    def soqo(self) -> U16Vector4: ...
    @property
    def soql(self) -> U16Vector4: ...
    @property
    def sops(self) -> U16Vector4: ...
    @property
    def sopt(self) -> U16Vector4: ...
    @property
    def sopq(self) -> U16Vector4: ...
    @property
    def sopp(self) -> U16Vector4: ...
    @property
    def sopo(self) -> U16Vector4: ...
    @property
    def sopl(self) -> U16Vector4: ...
    @property
    def soos(self) -> U16Vector4: ...
    @property
    def soot(self) -> U16Vector4: ...
    @property
    def sooq(self) -> U16Vector4: ...
    @property
    def soop(self) -> U16Vector4: ...
    @property
    def sooo(self) -> U16Vector4: ...
    @property
    def sool(self) -> U16Vector4: ...
    @property
    def sols(self) -> U16Vector4: ...
    @property
    def solt(self) -> U16Vector4: ...
    @property
    def solq(self) -> U16Vector4: ...
    @property
    def solp(self) -> U16Vector4: ...
    @property
    def solo(self) -> U16Vector4: ...
    @property
    def soll(self) -> U16Vector4: ...
    @property
    def slss(self) -> U16Vector4: ...
    @property
    def slst(self) -> U16Vector4: ...
    @property
    def slsq(self) -> U16Vector4: ...
    @property
    def slsp(self) -> U16Vector4: ...
    @property
    def slso(self) -> U16Vector4: ...
    @property
    def slsl(self) -> U16Vector4: ...
    @property
    def slts(self) -> U16Vector4: ...
    @property
    def sltt(self) -> U16Vector4: ...
    @property
    def sltq(self) -> U16Vector4: ...
    @property
    def sltp(self) -> U16Vector4: ...
    @property
    def slto(self) -> U16Vector4: ...
    @property
    def sltl(self) -> U16Vector4: ...
    @property
    def slqs(self) -> U16Vector4: ...
    @property
    def slqt(self) -> U16Vector4: ...
    @property
    def slqq(self) -> U16Vector4: ...
    @property
    def slqp(self) -> U16Vector4: ...
    @property
    def slqo(self) -> U16Vector4: ...
    @property
    def slql(self) -> U16Vector4: ...
    @property
    def slps(self) -> U16Vector4: ...
    @property
    def slpt(self) -> U16Vector4: ...
    @property
    def slpq(self) -> U16Vector4: ...
    @property
    def slpp(self) -> U16Vector4: ...
    @property
    def slpo(self) -> U16Vector4: ...
    @property
    def slpl(self) -> U16Vector4: ...
    @property
    def slos(self) -> U16Vector4: ...
    @property
    def slot(self) -> U16Vector4: ...
    @property
    def sloq(self) -> U16Vector4: ...
    @property
    def slop(self) -> U16Vector4: ...
    @property
    def sloo(self) -> U16Vector4: ...
    @property
    def slol(self) -> U16Vector4: ...
    @property
    def slls(self) -> U16Vector4: ...
    @property
    def sllt(self) -> U16Vector4: ...
    @property
    def sllq(self) -> U16Vector4: ...
    @property
    def sllp(self) -> U16Vector4: ...
    @property
    def sllo(self) -> U16Vector4: ...
    @property
    def slll(self) -> U16Vector4: ...
    @property
    def tsss(self) -> U16Vector4: ...
    @property
    def tsst(self) -> U16Vector4: ...
    @property
    def tssq(self) -> U16Vector4: ...
    @property
    def tssp(self) -> U16Vector4: ...
    @property
    def tsso(self) -> U16Vector4: ...
    @property
    def tssl(self) -> U16Vector4: ...
    @property
    def tsts(self) -> U16Vector4: ...
    @property
    def tstt(self) -> U16Vector4: ...
    @property
    def tstq(self) -> U16Vector4: ...
    @property
    def tstp(self) -> U16Vector4: ...
    @property
    def tsto(self) -> U16Vector4: ...
    @property
    def tstl(self) -> U16Vector4: ...
    @property
    def tsqs(self) -> U16Vector4: ...
    @property
    def tsqt(self) -> U16Vector4: ...
    @property
    def tsqq(self) -> U16Vector4: ...
    @property
    def tsqp(self) -> U16Vector4: ...
    @property
    def tsqo(self) -> U16Vector4: ...
    @property
    def tsql(self) -> U16Vector4: ...
    @property
    def tsps(self) -> U16Vector4: ...
    @property
    def tspt(self) -> U16Vector4: ...
    @property
    def tspq(self) -> U16Vector4: ...
    @property
    def tspp(self) -> U16Vector4: ...
    @property
    def tspo(self) -> U16Vector4: ...
    @property
    def tspl(self) -> U16Vector4: ...
    @property
    def tsos(self) -> U16Vector4: ...
    @property
    def tsot(self) -> U16Vector4: ...
    @property
    def tsoq(self) -> U16Vector4: ...
    @property
    def tsop(self) -> U16Vector4: ...
    @property
    def tsoo(self) -> U16Vector4: ...
    @property
    def tsol(self) -> U16Vector4: ...
    @property
    def tsls(self) -> U16Vector4: ...
    @property
    def tslt(self) -> U16Vector4: ...
    @property
    def tslq(self) -> U16Vector4: ...
    @property
    def tslp(self) -> U16Vector4: ...
    @property
    def tslo(self) -> U16Vector4: ...
    @property
    def tsll(self) -> U16Vector4: ...
    @property
    def ttss(self) -> U16Vector4: ...
    @property
    def ttst(self) -> U16Vector4: ...
    @property
    def ttsq(self) -> U16Vector4: ...
    @property
    def ttsp(self) -> U16Vector4: ...
    @property
    def ttso(self) -> U16Vector4: ...
    @property
    def ttsl(self) -> U16Vector4: ...
    @property
    def ttts(self) -> U16Vector4: ...
    @property
    def tttt(self) -> U16Vector4: ...
    @property
    def tttq(self) -> U16Vector4: ...
    @property
    def tttp(self) -> U16Vector4: ...
    @property
    def ttto(self) -> U16Vector4: ...
    @property
    def tttl(self) -> U16Vector4: ...
    @property
    def ttqs(self) -> U16Vector4: ...
    @property
    def ttqt(self) -> U16Vector4: ...
    @property
    def ttqq(self) -> U16Vector4: ...
    @property
    def ttqp(self) -> U16Vector4: ...
    @property
    def ttqo(self) -> U16Vector4: ...
    @property
    def ttql(self) -> U16Vector4: ...
    @property
    def ttps(self) -> U16Vector4: ...
    @property
    def ttpt(self) -> U16Vector4: ...
    @property
    def ttpq(self) -> U16Vector4: ...
    @property
    def ttpp(self) -> U16Vector4: ...
    @property
    def ttpo(self) -> U16Vector4: ...
    @property
    def ttpl(self) -> U16Vector4: ...
    @property
    def ttos(self) -> U16Vector4: ...
    @property
    def ttot(self) -> U16Vector4: ...
    @property
    def ttoq(self) -> U16Vector4: ...
    @property
    def ttop(self) -> U16Vector4: ...
    @property
    def ttoo(self) -> U16Vector4: ...
    @property
    def ttol(self) -> U16Vector4: ...
    @property
    def ttls(self) -> U16Vector4: ...
    @property
    def ttlt(self) -> U16Vector4: ...
    @property
    def ttlq(self) -> U16Vector4: ...
    @property
    def ttlp(self) -> U16Vector4: ...
    @property
    def ttlo(self) -> U16Vector4: ...
    @property
    def ttll(self) -> U16Vector4: ...
    @property
    def tqss(self) -> U16Vector4: ...
    @property
    def tqst(self) -> U16Vector4: ...
    @property
    def tqsq(self) -> U16Vector4: ...
    @property
    def tqsp(self) -> U16Vector4: ...
    @property
    def tqso(self) -> U16Vector4: ...
    @property
    def tqsl(self) -> U16Vector4: ...
    @property
    def tqts(self) -> U16Vector4: ...
    @property
    def tqtt(self) -> U16Vector4: ...
    @property
    def tqtq(self) -> U16Vector4: ...
    @property
    def tqtp(self) -> U16Vector4: ...
    @property
    def tqto(self) -> U16Vector4: ...
    @property
    def tqtl(self) -> U16Vector4: ...
    @property
    def tqqs(self) -> U16Vector4: ...
    @property
    def tqqt(self) -> U16Vector4: ...
    @property
    def tqqq(self) -> U16Vector4: ...
    @property
    def tqqp(self) -> U16Vector4: ...
    @property
    def tqqo(self) -> U16Vector4: ...
    @property
    def tqql(self) -> U16Vector4: ...
    @property
    def tqps(self) -> U16Vector4: ...
    @property
    def tqpt(self) -> U16Vector4: ...
    @property
    def tqpq(self) -> U16Vector4: ...
    @property
    def tqpp(self) -> U16Vector4: ...
    @property
    def tqpo(self) -> U16Vector4: ...
    @property
    def tqpl(self) -> U16Vector4: ...
    @property
    def tqos(self) -> U16Vector4: ...
    @property
    def tqot(self) -> U16Vector4: ...
    @property
    def tqoq(self) -> U16Vector4: ...
    @property
    def tqop(self) -> U16Vector4: ...
    @property
    def tqoo(self) -> U16Vector4: ...
    @property
    def tqol(self) -> U16Vector4: ...
    @property
    def tqls(self) -> U16Vector4: ...
    @property
    def tqlt(self) -> U16Vector4: ...
    @property
    def tqlq(self) -> U16Vector4: ...
    @property
    def tqlp(self) -> U16Vector4: ...
    @property
    def tqlo(self) -> U16Vector4: ...
    @property
    def tqll(self) -> U16Vector4: ...
    @property
    def tpss(self) -> U16Vector4: ...
    @property
    def tpst(self) -> U16Vector4: ...
    @property
    def tpsq(self) -> U16Vector4: ...
    @property
    def tpsp(self) -> U16Vector4: ...
    @property
    def tpso(self) -> U16Vector4: ...
    @property
    def tpsl(self) -> U16Vector4: ...
    @property
    def tpts(self) -> U16Vector4: ...
    @property
    def tptt(self) -> U16Vector4: ...
    @property
    def tptq(self) -> U16Vector4: ...
    @property
    def tptp(self) -> U16Vector4: ...
    @property
    def tpto(self) -> U16Vector4: ...
    @property
    def tptl(self) -> U16Vector4: ...
    @property
    def tpqs(self) -> U16Vector4: ...
    @property
    def tpqt(self) -> U16Vector4: ...
    @property
    def tpqq(self) -> U16Vector4: ...
    @property
    def tpqp(self) -> U16Vector4: ...
    @property
    def tpqo(self) -> U16Vector4: ...
    @property
    def tpql(self) -> U16Vector4: ...
    @property
    def tpps(self) -> U16Vector4: ...
    @property
    def tppt(self) -> U16Vector4: ...
    @property
    def tppq(self) -> U16Vector4: ...
    @property
    def tppp(self) -> U16Vector4: ...
    @property
    def tppo(self) -> U16Vector4: ...
    @property
    def tppl(self) -> U16Vector4: ...
    @property
    def tpos(self) -> U16Vector4: ...
    @property
    def tpot(self) -> U16Vector4: ...
    @property
    def tpoq(self) -> U16Vector4: ...
    @property
    def tpop(self) -> U16Vector4: ...
    @property
    def tpoo(self) -> U16Vector4: ...
    @property
    def tpol(self) -> U16Vector4: ...
    @property
    def tpls(self) -> U16Vector4: ...
    @property
    def tplt(self) -> U16Vector4: ...
    @property
    def tplq(self) -> U16Vector4: ...
    @property
    def tplp(self) -> U16Vector4: ...
    @property
    def tplo(self) -> U16Vector4: ...
    @property
    def tpll(self) -> U16Vector4: ...
    @property
    def toss(self) -> U16Vector4: ...
    @property
    def tost(self) -> U16Vector4: ...
    @property
    def tosq(self) -> U16Vector4: ...
    @property
    def tosp(self) -> U16Vector4: ...
    @property
    def toso(self) -> U16Vector4: ...
    @property
    def tosl(self) -> U16Vector4: ...
    @property
    def tots(self) -> U16Vector4: ...
    @property
    def tott(self) -> U16Vector4: ...
    @property
    def totq(self) -> U16Vector4: ...
    @property
    def totp(self) -> U16Vector4: ...
    @property
    def toto(self) -> U16Vector4: ...
    @property
    def totl(self) -> U16Vector4: ...
    @property
    def toqs(self) -> U16Vector4: ...
    @property
    def toqt(self) -> U16Vector4: ...
    @property
    def toqq(self) -> U16Vector4: ...
    @property
    def toqp(self) -> U16Vector4: ...
    @property
    def toqo(self) -> U16Vector4: ...
    @property
    def toql(self) -> U16Vector4: ...
    @property
    def tops(self) -> U16Vector4: ...
    @property
    def topt(self) -> U16Vector4: ...
    @property
    def topq(self) -> U16Vector4: ...
    @property
    def topp(self) -> U16Vector4: ...
    @property
    def topo(self) -> U16Vector4: ...
    @property
    def topl(self) -> U16Vector4: ...
    @property
    def toos(self) -> U16Vector4: ...
    @property
    def toot(self) -> U16Vector4: ...
    @property
    def tooq(self) -> U16Vector4: ...
    @property
    def toop(self) -> U16Vector4: ...
    @property
    def tooo(self) -> U16Vector4: ...
    @property
    def tool(self) -> U16Vector4: ...
    @property
    def tols(self) -> U16Vector4: ...
    @property
    def tolt(self) -> U16Vector4: ...
    @property
    def tolq(self) -> U16Vector4: ...
    @property
    def tolp(self) -> U16Vector4: ...
    @property
    def tolo(self) -> U16Vector4: ...
    @property
    def toll(self) -> U16Vector4: ...
    @property
    def tlss(self) -> U16Vector4: ...
    @property
    def tlst(self) -> U16Vector4: ...
    @property
    def tlsq(self) -> U16Vector4: ...
    @property
    def tlsp(self) -> U16Vector4: ...
    @property
    def tlso(self) -> U16Vector4: ...
    @property
    def tlsl(self) -> U16Vector4: ...
    @property
    def tlts(self) -> U16Vector4: ...
    @property
    def tltt(self) -> U16Vector4: ...
    @property
    def tltq(self) -> U16Vector4: ...
    @property
    def tltp(self) -> U16Vector4: ...
    @property
    def tlto(self) -> U16Vector4: ...
    @property
    def tltl(self) -> U16Vector4: ...
    @property
    def tlqs(self) -> U16Vector4: ...
    @property
    def tlqt(self) -> U16Vector4: ...
    @property
    def tlqq(self) -> U16Vector4: ...
    @property
    def tlqp(self) -> U16Vector4: ...
    @property
    def tlqo(self) -> U16Vector4: ...
    @property
    def tlql(self) -> U16Vector4: ...
    @property
    def tlps(self) -> U16Vector4: ...
    @property
    def tlpt(self) -> U16Vector4: ...
    @property
    def tlpq(self) -> U16Vector4: ...
    @property
    def tlpp(self) -> U16Vector4: ...
    @property
    def tlpo(self) -> U16Vector4: ...
    @property
    def tlpl(self) -> U16Vector4: ...
    @property
    def tlos(self) -> U16Vector4: ...
    @property
    def tlot(self) -> U16Vector4: ...
    @property
    def tloq(self) -> U16Vector4: ...
    @property
    def tlop(self) -> U16Vector4: ...
    @property
    def tloo(self) -> U16Vector4: ...
    @property
    def tlol(self) -> U16Vector4: ...
    @property
    def tlls(self) -> U16Vector4: ...
    @property
    def tllt(self) -> U16Vector4: ...
    @property
    def tllq(self) -> U16Vector4: ...
    @property
    def tllp(self) -> U16Vector4: ...
    @property
    def tllo(self) -> U16Vector4: ...
    @property
    def tlll(self) -> U16Vector4: ...
    @property
    def qsss(self) -> U16Vector4: ...
    @property
    def qsst(self) -> U16Vector4: ...
    @property
    def qssq(self) -> U16Vector4: ...
    @property
    def qssp(self) -> U16Vector4: ...
    @property
    def qsso(self) -> U16Vector4: ...
    @property
    def qssl(self) -> U16Vector4: ...
    @property
    def qsts(self) -> U16Vector4: ...
    @property
    def qstt(self) -> U16Vector4: ...
    @property
    def qstq(self) -> U16Vector4: ...
    @property
    def qstp(self) -> U16Vector4: ...
    @property
    def qsto(self) -> U16Vector4: ...
    @property
    def qstl(self) -> U16Vector4: ...
    @property
    def qsqs(self) -> U16Vector4: ...
    @property
    def qsqt(self) -> U16Vector4: ...
    @property
    def qsqq(self) -> U16Vector4: ...
    @property
    def qsqp(self) -> U16Vector4: ...
    @property
    def qsqo(self) -> U16Vector4: ...
    @property
    def qsql(self) -> U16Vector4: ...
    @property
    def qsps(self) -> U16Vector4: ...
    @property
    def qspt(self) -> U16Vector4: ...
    @property
    def qspq(self) -> U16Vector4: ...
    @property
    def qspp(self) -> U16Vector4: ...
    @property
    def qspo(self) -> U16Vector4: ...
    @property
    def qspl(self) -> U16Vector4: ...
    @property
    def qsos(self) -> U16Vector4: ...
    @property
    def qsot(self) -> U16Vector4: ...
    @property
    def qsoq(self) -> U16Vector4: ...
    @property
    def qsop(self) -> U16Vector4: ...
    @property
    def qsoo(self) -> U16Vector4: ...
    @property
    def qsol(self) -> U16Vector4: ...
    @property
    def qsls(self) -> U16Vector4: ...
    @property
    def qslt(self) -> U16Vector4: ...
    @property
    def qslq(self) -> U16Vector4: ...
    @property
    def qslp(self) -> U16Vector4: ...
    @property
    def qslo(self) -> U16Vector4: ...
    @property
    def qsll(self) -> U16Vector4: ...
    @property
    def qtss(self) -> U16Vector4: ...
    @property
    def qtst(self) -> U16Vector4: ...
    @property
    def qtsq(self) -> U16Vector4: ...
    @property
    def qtsp(self) -> U16Vector4: ...
    @property
    def qtso(self) -> U16Vector4: ...
    @property
    def qtsl(self) -> U16Vector4: ...
    @property
    def qtts(self) -> U16Vector4: ...
    @property
    def qttt(self) -> U16Vector4: ...
    @property
    def qttq(self) -> U16Vector4: ...
    @property
    def qttp(self) -> U16Vector4: ...
    @property
    def qtto(self) -> U16Vector4: ...
    @property
    def qttl(self) -> U16Vector4: ...
    @property
    def qtqs(self) -> U16Vector4: ...
    @property
    def qtqt(self) -> U16Vector4: ...
    @property
    def qtqq(self) -> U16Vector4: ...
    @property
    def qtqp(self) -> U16Vector4: ...
    @property
    def qtqo(self) -> U16Vector4: ...
    @property
    def qtql(self) -> U16Vector4: ...
    @property
    def qtps(self) -> U16Vector4: ...
    @property
    def qtpt(self) -> U16Vector4: ...
    @property
    def qtpq(self) -> U16Vector4: ...
    @property
    def qtpp(self) -> U16Vector4: ...
    @property
    def qtpo(self) -> U16Vector4: ...
    @property
    def qtpl(self) -> U16Vector4: ...
    @property
    def qtos(self) -> U16Vector4: ...
    @property
    def qtot(self) -> U16Vector4: ...
    @property
    def qtoq(self) -> U16Vector4: ...
    @property
    def qtop(self) -> U16Vector4: ...
    @property
    def qtoo(self) -> U16Vector4: ...
    @property
    def qtol(self) -> U16Vector4: ...
    @property
    def qtls(self) -> U16Vector4: ...
    @property
    def qtlt(self) -> U16Vector4: ...
    @property
    def qtlq(self) -> U16Vector4: ...
    @property
    def qtlp(self) -> U16Vector4: ...
    @property
    def qtlo(self) -> U16Vector4: ...
    @property
    def qtll(self) -> U16Vector4: ...
    @property
    def qqss(self) -> U16Vector4: ...
    @property
    def qqst(self) -> U16Vector4: ...
    @property
    def qqsq(self) -> U16Vector4: ...
    @property
    def qqsp(self) -> U16Vector4: ...
    @property
    def qqso(self) -> U16Vector4: ...
    @property
    def qqsl(self) -> U16Vector4: ...
    @property
    def qqts(self) -> U16Vector4: ...
    @property
    def qqtt(self) -> U16Vector4: ...
    @property
    def qqtq(self) -> U16Vector4: ...
    @property
    def qqtp(self) -> U16Vector4: ...
    @property
    def qqto(self) -> U16Vector4: ...
    @property
    def qqtl(self) -> U16Vector4: ...
    @property
    def qqqs(self) -> U16Vector4: ...
    @property
    def qqqt(self) -> U16Vector4: ...
    @property
    def qqqq(self) -> U16Vector4: ...
    @property
    def qqqp(self) -> U16Vector4: ...
    @property
    def qqqo(self) -> U16Vector4: ...
    @property
    def qqql(self) -> U16Vector4: ...
    @property
    def qqps(self) -> U16Vector4: ...
    @property
    def qqpt(self) -> U16Vector4: ...
    @property
    def qqpq(self) -> U16Vector4: ...
    @property
    def qqpp(self) -> U16Vector4: ...
    @property
    def qqpo(self) -> U16Vector4: ...
    @property
    def qqpl(self) -> U16Vector4: ...
    @property
    def qqos(self) -> U16Vector4: ...
    @property
    def qqot(self) -> U16Vector4: ...
    @property
    def qqoq(self) -> U16Vector4: ...
    @property
    def qqop(self) -> U16Vector4: ...
    @property
    def qqoo(self) -> U16Vector4: ...
    @property
    def qqol(self) -> U16Vector4: ...
    @property
    def qqls(self) -> U16Vector4: ...
    @property
    def qqlt(self) -> U16Vector4: ...
    @property
    def qqlq(self) -> U16Vector4: ...
    @property
    def qqlp(self) -> U16Vector4: ...
    @property
    def qqlo(self) -> U16Vector4: ...
    @property
    def qqll(self) -> U16Vector4: ...
    @property
    def qpss(self) -> U16Vector4: ...
    @property
    def qpst(self) -> U16Vector4: ...
    @property
    def qpsq(self) -> U16Vector4: ...
    @property
    def qpsp(self) -> U16Vector4: ...
    @property
    def qpso(self) -> U16Vector4: ...
    @property
    def qpsl(self) -> U16Vector4: ...
    @property
    def qpts(self) -> U16Vector4: ...
    @property
    def qptt(self) -> U16Vector4: ...
    @property
    def qptq(self) -> U16Vector4: ...
    @property
    def qptp(self) -> U16Vector4: ...
    @property
    def qpto(self) -> U16Vector4: ...
    @property
    def qptl(self) -> U16Vector4: ...
    @property
    def qpqs(self) -> U16Vector4: ...
    @property
    def qpqt(self) -> U16Vector4: ...
    @property
    def qpqq(self) -> U16Vector4: ...
    @property
    def qpqp(self) -> U16Vector4: ...
    @property
    def qpqo(self) -> U16Vector4: ...
    @property
    def qpql(self) -> U16Vector4: ...
    @property
    def qpps(self) -> U16Vector4: ...
    @property
    def qppt(self) -> U16Vector4: ...
    @property
    def qppq(self) -> U16Vector4: ...
    @property
    def qppp(self) -> U16Vector4: ...
    @property
    def qppo(self) -> U16Vector4: ...
    @property
    def qppl(self) -> U16Vector4: ...
    @property
    def qpos(self) -> U16Vector4: ...
    @property
    def qpot(self) -> U16Vector4: ...
    @property
    def qpoq(self) -> U16Vector4: ...
    @property
    def qpop(self) -> U16Vector4: ...
    @property
    def qpoo(self) -> U16Vector4: ...
    @property
    def qpol(self) -> U16Vector4: ...
    @property
    def qpls(self) -> U16Vector4: ...
    @property
    def qplt(self) -> U16Vector4: ...
    @property
    def qplq(self) -> U16Vector4: ...
    @property
    def qplp(self) -> U16Vector4: ...
    @property
    def qplo(self) -> U16Vector4: ...
    @property
    def qpll(self) -> U16Vector4: ...
    @property
    def qoss(self) -> U16Vector4: ...
    @property
    def qost(self) -> U16Vector4: ...
    @property
    def qosq(self) -> U16Vector4: ...
    @property
    def qosp(self) -> U16Vector4: ...
    @property
    def qoso(self) -> U16Vector4: ...
    @property
    def qosl(self) -> U16Vector4: ...
    @property
    def qots(self) -> U16Vector4: ...
    @property
    def qott(self) -> U16Vector4: ...
    @property
    def qotq(self) -> U16Vector4: ...
    @property
    def qotp(self) -> U16Vector4: ...
    @property
    def qoto(self) -> U16Vector4: ...
    @property
    def qotl(self) -> U16Vector4: ...
    @property
    def qoqs(self) -> U16Vector4: ...
    @property
    def qoqt(self) -> U16Vector4: ...
    @property
    def qoqq(self) -> U16Vector4: ...
    @property
    def qoqp(self) -> U16Vector4: ...
    @property
    def qoqo(self) -> U16Vector4: ...
    @property
    def qoql(self) -> U16Vector4: ...
    @property
    def qops(self) -> U16Vector4: ...
    @property
    def qopt(self) -> U16Vector4: ...
    @property
    def qopq(self) -> U16Vector4: ...
    @property
    def qopp(self) -> U16Vector4: ...
    @property
    def qopo(self) -> U16Vector4: ...
    @property
    def qopl(self) -> U16Vector4: ...
    @property
    def qoos(self) -> U16Vector4: ...
    @property
    def qoot(self) -> U16Vector4: ...
    @property
    def qooq(self) -> U16Vector4: ...
    @property
    def qoop(self) -> U16Vector4: ...
    @property
    def qooo(self) -> U16Vector4: ...
    @property
    def qool(self) -> U16Vector4: ...
    @property
    def qols(self) -> U16Vector4: ...
    @property
    def qolt(self) -> U16Vector4: ...
    @property
    def qolq(self) -> U16Vector4: ...
    @property
    def qolp(self) -> U16Vector4: ...
    @property
    def qolo(self) -> U16Vector4: ...
    @property
    def qoll(self) -> U16Vector4: ...
    @property
    def qlss(self) -> U16Vector4: ...
    @property
    def qlst(self) -> U16Vector4: ...
    @property
    def qlsq(self) -> U16Vector4: ...
    @property
    def qlsp(self) -> U16Vector4: ...
    @property
    def qlso(self) -> U16Vector4: ...
    @property
    def qlsl(self) -> U16Vector4: ...
    @property
    def qlts(self) -> U16Vector4: ...
    @property
    def qltt(self) -> U16Vector4: ...
    @property
    def qltq(self) -> U16Vector4: ...
    @property
    def qltp(self) -> U16Vector4: ...
    @property
    def qlto(self) -> U16Vector4: ...
    @property
    def qltl(self) -> U16Vector4: ...
    @property
    def qlqs(self) -> U16Vector4: ...
    @property
    def qlqt(self) -> U16Vector4: ...
    @property
    def qlqq(self) -> U16Vector4: ...
    @property
    def qlqp(self) -> U16Vector4: ...
    @property
    def qlqo(self) -> U16Vector4: ...
    @property
    def qlql(self) -> U16Vector4: ...
    @property
    def qlps(self) -> U16Vector4: ...
    @property
    def qlpt(self) -> U16Vector4: ...
    @property
    def qlpq(self) -> U16Vector4: ...
    @property
    def qlpp(self) -> U16Vector4: ...
    @property
    def qlpo(self) -> U16Vector4: ...
    @property
    def qlpl(self) -> U16Vector4: ...
    @property
    def qlos(self) -> U16Vector4: ...
    @property
    def qlot(self) -> U16Vector4: ...
    @property
    def qloq(self) -> U16Vector4: ...
    @property
    def qlop(self) -> U16Vector4: ...
    @property
    def qloo(self) -> U16Vector4: ...
    @property
    def qlol(self) -> U16Vector4: ...
    @property
    def qlls(self) -> U16Vector4: ...
    @property
    def qllt(self) -> U16Vector4: ...
    @property
    def qllq(self) -> U16Vector4: ...
    @property
    def qllp(self) -> U16Vector4: ...
    @property
    def qllo(self) -> U16Vector4: ...
    @property
    def qlll(self) -> U16Vector4: ...
    @property
    def psss(self) -> U16Vector4: ...
    @property
    def psst(self) -> U16Vector4: ...
    @property
    def pssq(self) -> U16Vector4: ...
    @property
    def pssp(self) -> U16Vector4: ...
    @property
    def psso(self) -> U16Vector4: ...
    @property
    def pssl(self) -> U16Vector4: ...
    @property
    def psts(self) -> U16Vector4: ...
    @property
    def pstt(self) -> U16Vector4: ...
    @property
    def pstq(self) -> U16Vector4: ...
    @property
    def pstp(self) -> U16Vector4: ...
    @property
    def psto(self) -> U16Vector4: ...
    @property
    def pstl(self) -> U16Vector4: ...
    @property
    def psqs(self) -> U16Vector4: ...
    @property
    def psqt(self) -> U16Vector4: ...
    @property
    def psqq(self) -> U16Vector4: ...
    @property
    def psqp(self) -> U16Vector4: ...
    @property
    def psqo(self) -> U16Vector4: ...
    @property
    def psql(self) -> U16Vector4: ...
    @property
    def psps(self) -> U16Vector4: ...
    @property
    def pspt(self) -> U16Vector4: ...
    @property
    def pspq(self) -> U16Vector4: ...
    @property
    def pspp(self) -> U16Vector4: ...
    @property
    def pspo(self) -> U16Vector4: ...
    @property
    def pspl(self) -> U16Vector4: ...
    @property
    def psos(self) -> U16Vector4: ...
    @property
    def psot(self) -> U16Vector4: ...
    @property
    def psoq(self) -> U16Vector4: ...
    @property
    def psop(self) -> U16Vector4: ...
    @property
    def psoo(self) -> U16Vector4: ...
    @property
    def psol(self) -> U16Vector4: ...
    @property
    def psls(self) -> U16Vector4: ...
    @property
    def pslt(self) -> U16Vector4: ...
    @property
    def pslq(self) -> U16Vector4: ...
    @property
    def pslp(self) -> U16Vector4: ...
    @property
    def pslo(self) -> U16Vector4: ...
    @property
    def psll(self) -> U16Vector4: ...
    @property
    def ptss(self) -> U16Vector4: ...
    @property
    def ptst(self) -> U16Vector4: ...
    @property
    def ptsq(self) -> U16Vector4: ...
    @property
    def ptsp(self) -> U16Vector4: ...
    @property
    def ptso(self) -> U16Vector4: ...
    @property
    def ptsl(self) -> U16Vector4: ...
    @property
    def ptts(self) -> U16Vector4: ...
    @property
    def pttt(self) -> U16Vector4: ...
    @property
    def pttq(self) -> U16Vector4: ...
    @property
    def pttp(self) -> U16Vector4: ...
    @property
    def ptto(self) -> U16Vector4: ...
    @property
    def pttl(self) -> U16Vector4: ...
    @property
    def ptqs(self) -> U16Vector4: ...
    @property
    def ptqt(self) -> U16Vector4: ...
    @property
    def ptqq(self) -> U16Vector4: ...
    @property
    def ptqp(self) -> U16Vector4: ...
    @property
    def ptqo(self) -> U16Vector4: ...
    @property
    def ptql(self) -> U16Vector4: ...
    @property
    def ptps(self) -> U16Vector4: ...
    @property
    def ptpt(self) -> U16Vector4: ...
    @property
    def ptpq(self) -> U16Vector4: ...
    @property
    def ptpp(self) -> U16Vector4: ...
    @property
    def ptpo(self) -> U16Vector4: ...
    @property
    def ptpl(self) -> U16Vector4: ...
    @property
    def ptos(self) -> U16Vector4: ...
    @property
    def ptot(self) -> U16Vector4: ...
    @property
    def ptoq(self) -> U16Vector4: ...
    @property
    def ptop(self) -> U16Vector4: ...
    @property
    def ptoo(self) -> U16Vector4: ...
    @property
    def ptol(self) -> U16Vector4: ...
    @property
    def ptls(self) -> U16Vector4: ...
    @property
    def ptlt(self) -> U16Vector4: ...
    @property
    def ptlq(self) -> U16Vector4: ...
    @property
    def ptlp(self) -> U16Vector4: ...
    @property
    def ptlo(self) -> U16Vector4: ...
    @property
    def ptll(self) -> U16Vector4: ...
    @property
    def pqss(self) -> U16Vector4: ...
    @property
    def pqst(self) -> U16Vector4: ...
    @property
    def pqsq(self) -> U16Vector4: ...
    @property
    def pqsp(self) -> U16Vector4: ...
    @property
    def pqso(self) -> U16Vector4: ...
    @property
    def pqsl(self) -> U16Vector4: ...
    @property
    def pqts(self) -> U16Vector4: ...
    @property
    def pqtt(self) -> U16Vector4: ...
    @property
    def pqtq(self) -> U16Vector4: ...
    @property
    def pqtp(self) -> U16Vector4: ...
    @property
    def pqto(self) -> U16Vector4: ...
    @property
    def pqtl(self) -> U16Vector4: ...
    @property
    def pqqs(self) -> U16Vector4: ...
    @property
    def pqqt(self) -> U16Vector4: ...
    @property
    def pqqq(self) -> U16Vector4: ...
    @property
    def pqqp(self) -> U16Vector4: ...
    @property
    def pqqo(self) -> U16Vector4: ...
    @property
    def pqql(self) -> U16Vector4: ...
    @property
    def pqps(self) -> U16Vector4: ...
    @property
    def pqpt(self) -> U16Vector4: ...
    @property
    def pqpq(self) -> U16Vector4: ...
    @property
    def pqpp(self) -> U16Vector4: ...
    @property
    def pqpo(self) -> U16Vector4: ...
    @property
    def pqpl(self) -> U16Vector4: ...
    @property
    def pqos(self) -> U16Vector4: ...
    @property
    def pqot(self) -> U16Vector4: ...
    @property
    def pqoq(self) -> U16Vector4: ...
    @property
    def pqop(self) -> U16Vector4: ...
    @property
    def pqoo(self) -> U16Vector4: ...
    @property
    def pqol(self) -> U16Vector4: ...
    @property
    def pqls(self) -> U16Vector4: ...
    @property
    def pqlt(self) -> U16Vector4: ...
    @property
    def pqlq(self) -> U16Vector4: ...
    @property
    def pqlp(self) -> U16Vector4: ...
    @property
    def pqlo(self) -> U16Vector4: ...
    @property
    def pqll(self) -> U16Vector4: ...
    @property
    def ppss(self) -> U16Vector4: ...
    @property
    def ppst(self) -> U16Vector4: ...
    @property
    def ppsq(self) -> U16Vector4: ...
    @property
    def ppsp(self) -> U16Vector4: ...
    @property
    def ppso(self) -> U16Vector4: ...
    @property
    def ppsl(self) -> U16Vector4: ...
    @property
    def ppts(self) -> U16Vector4: ...
    @property
    def pptt(self) -> U16Vector4: ...
    @property
    def pptq(self) -> U16Vector4: ...
    @property
    def pptp(self) -> U16Vector4: ...
    @property
    def ppto(self) -> U16Vector4: ...
    @property
    def pptl(self) -> U16Vector4: ...
    @property
    def ppqs(self) -> U16Vector4: ...
    @property
    def ppqt(self) -> U16Vector4: ...
    @property
    def ppqq(self) -> U16Vector4: ...
    @property
    def ppqp(self) -> U16Vector4: ...
    @property
    def ppqo(self) -> U16Vector4: ...
    @property
    def ppql(self) -> U16Vector4: ...
    @property
    def ppps(self) -> U16Vector4: ...
    @property
    def pppt(self) -> U16Vector4: ...
    @property
    def pppq(self) -> U16Vector4: ...
    @property
    def pppp(self) -> U16Vector4: ...
    @property
    def pppo(self) -> U16Vector4: ...
    @property
    def pppl(self) -> U16Vector4: ...
    @property
    def ppos(self) -> U16Vector4: ...
    @property
    def ppot(self) -> U16Vector4: ...
    @property
    def ppoq(self) -> U16Vector4: ...
    @property
    def ppop(self) -> U16Vector4: ...
    @property
    def ppoo(self) -> U16Vector4: ...
    @property
    def ppol(self) -> U16Vector4: ...
    @property
    def ppls(self) -> U16Vector4: ...
    @property
    def pplt(self) -> U16Vector4: ...
    @property
    def pplq(self) -> U16Vector4: ...
    @property
    def pplp(self) -> U16Vector4: ...
    @property
    def pplo(self) -> U16Vector4: ...
    @property
    def ppll(self) -> U16Vector4: ...
    @property
    def poss(self) -> U16Vector4: ...
    @property
    def post(self) -> U16Vector4: ...
    @property
    def posq(self) -> U16Vector4: ...
    @property
    def posp(self) -> U16Vector4: ...
    @property
    def poso(self) -> U16Vector4: ...
    @property
    def posl(self) -> U16Vector4: ...
    @property
    def pots(self) -> U16Vector4: ...
    @property
    def pott(self) -> U16Vector4: ...
    @property
    def potq(self) -> U16Vector4: ...
    @property
    def potp(self) -> U16Vector4: ...
    @property
    def poto(self) -> U16Vector4: ...
    @property
    def potl(self) -> U16Vector4: ...
    @property
    def poqs(self) -> U16Vector4: ...
    @property
    def poqt(self) -> U16Vector4: ...
    @property
    def poqq(self) -> U16Vector4: ...
    @property
    def poqp(self) -> U16Vector4: ...
    @property
    def poqo(self) -> U16Vector4: ...
    @property
    def poql(self) -> U16Vector4: ...
    @property
    def pops(self) -> U16Vector4: ...
    @property
    def popt(self) -> U16Vector4: ...
    @property
    def popq(self) -> U16Vector4: ...
    @property
    def popp(self) -> U16Vector4: ...
    @property
    def popo(self) -> U16Vector4: ...
    @property
    def popl(self) -> U16Vector4: ...
    @property
    def poos(self) -> U16Vector4: ...
    @property
    def poot(self) -> U16Vector4: ...
    @property
    def pooq(self) -> U16Vector4: ...
    @property
    def poop(self) -> U16Vector4: ...
    @property
    def pooo(self) -> U16Vector4: ...
    @property
    def pool(self) -> U16Vector4: ...
    @property
    def pols(self) -> U16Vector4: ...
    @property
    def polt(self) -> U16Vector4: ...
    @property
    def polq(self) -> U16Vector4: ...
    @property
    def polp(self) -> U16Vector4: ...
    @property
    def polo(self) -> U16Vector4: ...
    @property
    def poll(self) -> U16Vector4: ...
    @property
    def plss(self) -> U16Vector4: ...
    @property
    def plst(self) -> U16Vector4: ...
    @property
    def plsq(self) -> U16Vector4: ...
    @property
    def plsp(self) -> U16Vector4: ...
    @property
    def plso(self) -> U16Vector4: ...
    @property
    def plsl(self) -> U16Vector4: ...
    @property
    def plts(self) -> U16Vector4: ...
    @property
    def pltt(self) -> U16Vector4: ...
    @property
    def pltq(self) -> U16Vector4: ...
    @property
    def pltp(self) -> U16Vector4: ...
    @property
    def plto(self) -> U16Vector4: ...
    @property
    def pltl(self) -> U16Vector4: ...
    @property
    def plqs(self) -> U16Vector4: ...
    @property
    def plqt(self) -> U16Vector4: ...
    @property
    def plqq(self) -> U16Vector4: ...
    @property
    def plqp(self) -> U16Vector4: ...
    @property
    def plqo(self) -> U16Vector4: ...
    @property
    def plql(self) -> U16Vector4: ...
    @property
    def plps(self) -> U16Vector4: ...
    @property
    def plpt(self) -> U16Vector4: ...
    @property
    def plpq(self) -> U16Vector4: ...
    @property
    def plpp(self) -> U16Vector4: ...
    @property
    def plpo(self) -> U16Vector4: ...
    @property
    def plpl(self) -> U16Vector4: ...
    @property
    def plos(self) -> U16Vector4: ...
    @property
    def plot(self) -> U16Vector4: ...
    @property
    def ploq(self) -> U16Vector4: ...
    @property
    def plop(self) -> U16Vector4: ...
    @property
    def ploo(self) -> U16Vector4: ...
    @property
    def plol(self) -> U16Vector4: ...
    @property
    def plls(self) -> U16Vector4: ...
    @property
    def pllt(self) -> U16Vector4: ...
    @property
    def pllq(self) -> U16Vector4: ...
    @property
    def pllp(self) -> U16Vector4: ...
    @property
    def pllo(self) -> U16Vector4: ...
    @property
    def plll(self) -> U16Vector4: ...
    @property
    def osss(self) -> U16Vector4: ...
    @property
    def osst(self) -> U16Vector4: ...
    @property
    def ossq(self) -> U16Vector4: ...
    @property
    def ossp(self) -> U16Vector4: ...
    @property
    def osso(self) -> U16Vector4: ...
    @property
    def ossl(self) -> U16Vector4: ...
    @property
    def osts(self) -> U16Vector4: ...
    @property
    def ostt(self) -> U16Vector4: ...
    @property
    def ostq(self) -> U16Vector4: ...
    @property
    def ostp(self) -> U16Vector4: ...
    @property
    def osto(self) -> U16Vector4: ...
    @property
    def ostl(self) -> U16Vector4: ...
    @property
    def osqs(self) -> U16Vector4: ...
    @property
    def osqt(self) -> U16Vector4: ...
    @property
    def osqq(self) -> U16Vector4: ...
    @property
    def osqp(self) -> U16Vector4: ...
    @property
    def osqo(self) -> U16Vector4: ...
    @property
    def osql(self) -> U16Vector4: ...
    @property
    def osps(self) -> U16Vector4: ...
    @property
    def ospt(self) -> U16Vector4: ...
    @property
    def ospq(self) -> U16Vector4: ...
    @property
    def ospp(self) -> U16Vector4: ...
    @property
    def ospo(self) -> U16Vector4: ...
    @property
    def ospl(self) -> U16Vector4: ...
    @property
    def osos(self) -> U16Vector4: ...
    @property
    def osot(self) -> U16Vector4: ...
    @property
    def osoq(self) -> U16Vector4: ...
    @property
    def osop(self) -> U16Vector4: ...
    @property
    def osoo(self) -> U16Vector4: ...
    @property
    def osol(self) -> U16Vector4: ...
    @property
    def osls(self) -> U16Vector4: ...
    @property
    def oslt(self) -> U16Vector4: ...
    @property
    def oslq(self) -> U16Vector4: ...
    @property
    def oslp(self) -> U16Vector4: ...
    @property
    def oslo(self) -> U16Vector4: ...
    @property
    def osll(self) -> U16Vector4: ...
    @property
    def otss(self) -> U16Vector4: ...
    @property
    def otst(self) -> U16Vector4: ...
    @property
    def otsq(self) -> U16Vector4: ...
    @property
    def otsp(self) -> U16Vector4: ...
    @property
    def otso(self) -> U16Vector4: ...
    @property
    def otsl(self) -> U16Vector4: ...
    @property
    def otts(self) -> U16Vector4: ...
    @property
    def ottt(self) -> U16Vector4: ...
    @property
    def ottq(self) -> U16Vector4: ...
    @property
    def ottp(self) -> U16Vector4: ...
    @property
    def otto(self) -> U16Vector4: ...
    @property
    def ottl(self) -> U16Vector4: ...
    @property
    def otqs(self) -> U16Vector4: ...
    @property
    def otqt(self) -> U16Vector4: ...
    @property
    def otqq(self) -> U16Vector4: ...
    @property
    def otqp(self) -> U16Vector4: ...
    @property
    def otqo(self) -> U16Vector4: ...
    @property
    def otql(self) -> U16Vector4: ...
    @property
    def otps(self) -> U16Vector4: ...
    @property
    def otpt(self) -> U16Vector4: ...
    @property
    def otpq(self) -> U16Vector4: ...
    @property
    def otpp(self) -> U16Vector4: ...
    @property
    def otpo(self) -> U16Vector4: ...
    @property
    def otpl(self) -> U16Vector4: ...
    @property
    def otos(self) -> U16Vector4: ...
    @property
    def otot(self) -> U16Vector4: ...
    @property
    def otoq(self) -> U16Vector4: ...
    @property
    def otop(self) -> U16Vector4: ...
    @property
    def otoo(self) -> U16Vector4: ...
    @property
    def otol(self) -> U16Vector4: ...
    @property
    def otls(self) -> U16Vector4: ...
    @property
    def otlt(self) -> U16Vector4: ...
    @property
    def otlq(self) -> U16Vector4: ...
    @property
    def otlp(self) -> U16Vector4: ...
    @property
    def otlo(self) -> U16Vector4: ...
    @property
    def otll(self) -> U16Vector4: ...
    @property
    def oqss(self) -> U16Vector4: ...
    @property
    def oqst(self) -> U16Vector4: ...
    @property
    def oqsq(self) -> U16Vector4: ...
    @property
    def oqsp(self) -> U16Vector4: ...
    @property
    def oqso(self) -> U16Vector4: ...
    @property
    def oqsl(self) -> U16Vector4: ...
    @property
    def oqts(self) -> U16Vector4: ...
    @property
    def oqtt(self) -> U16Vector4: ...
    @property
    def oqtq(self) -> U16Vector4: ...
    @property
    def oqtp(self) -> U16Vector4: ...
    @property
    def oqto(self) -> U16Vector4: ...
    @property
    def oqtl(self) -> U16Vector4: ...
    @property
    def oqqs(self) -> U16Vector4: ...
    @property
    def oqqt(self) -> U16Vector4: ...
    @property
    def oqqq(self) -> U16Vector4: ...
    @property
    def oqqp(self) -> U16Vector4: ...
    @property
    def oqqo(self) -> U16Vector4: ...
    @property
    def oqql(self) -> U16Vector4: ...
    @property
    def oqps(self) -> U16Vector4: ...
    @property
    def oqpt(self) -> U16Vector4: ...
    @property
    def oqpq(self) -> U16Vector4: ...
    @property
    def oqpp(self) -> U16Vector4: ...
    @property
    def oqpo(self) -> U16Vector4: ...
    @property
    def oqpl(self) -> U16Vector4: ...
    @property
    def oqos(self) -> U16Vector4: ...
    @property
    def oqot(self) -> U16Vector4: ...
    @property
    def oqoq(self) -> U16Vector4: ...
    @property
    def oqop(self) -> U16Vector4: ...
    @property
    def oqoo(self) -> U16Vector4: ...
    @property
    def oqol(self) -> U16Vector4: ...
    @property
    def oqls(self) -> U16Vector4: ...
    @property
    def oqlt(self) -> U16Vector4: ...
    @property
    def oqlq(self) -> U16Vector4: ...
    @property
    def oqlp(self) -> U16Vector4: ...
    @property
    def oqlo(self) -> U16Vector4: ...
    @property
    def oqll(self) -> U16Vector4: ...
    @property
    def opss(self) -> U16Vector4: ...
    @property
    def opst(self) -> U16Vector4: ...
    @property
    def opsq(self) -> U16Vector4: ...
    @property
    def opsp(self) -> U16Vector4: ...
    @property
    def opso(self) -> U16Vector4: ...
    @property
    def opsl(self) -> U16Vector4: ...
    @property
    def opts(self) -> U16Vector4: ...
    @property
    def optt(self) -> U16Vector4: ...
    @property
    def optq(self) -> U16Vector4: ...
    @property
    def optp(self) -> U16Vector4: ...
    @property
    def opto(self) -> U16Vector4: ...
    @property
    def optl(self) -> U16Vector4: ...
    @property
    def opqs(self) -> U16Vector4: ...
    @property
    def opqt(self) -> U16Vector4: ...
    @property
    def opqq(self) -> U16Vector4: ...
    @property
    def opqp(self) -> U16Vector4: ...
    @property
    def opqo(self) -> U16Vector4: ...
    @property
    def opql(self) -> U16Vector4: ...
    @property
    def opps(self) -> U16Vector4: ...
    @property
    def oppt(self) -> U16Vector4: ...
    @property
    def oppq(self) -> U16Vector4: ...
    @property
    def oppp(self) -> U16Vector4: ...
    @property
    def oppo(self) -> U16Vector4: ...
    @property
    def oppl(self) -> U16Vector4: ...
    @property
    def opos(self) -> U16Vector4: ...
    @property
    def opot(self) -> U16Vector4: ...
    @property
    def opoq(self) -> U16Vector4: ...
    @property
    def opop(self) -> U16Vector4: ...
    @property
    def opoo(self) -> U16Vector4: ...
    @property
    def opol(self) -> U16Vector4: ...
    @property
    def opls(self) -> U16Vector4: ...
    @property
    def oplt(self) -> U16Vector4: ...
    @property
    def oplq(self) -> U16Vector4: ...
    @property
    def oplp(self) -> U16Vector4: ...
    @property
    def oplo(self) -> U16Vector4: ...
    @property
    def opll(self) -> U16Vector4: ...
    @property
    def ooss(self) -> U16Vector4: ...
    @property
    def oost(self) -> U16Vector4: ...
    @property
    def oosq(self) -> U16Vector4: ...
    @property
    def oosp(self) -> U16Vector4: ...
    @property
    def ooso(self) -> U16Vector4: ...
    @property
    def oosl(self) -> U16Vector4: ...
    @property
    def oots(self) -> U16Vector4: ...
    @property
    def oott(self) -> U16Vector4: ...
    @property
    def ootq(self) -> U16Vector4: ...
    @property
    def ootp(self) -> U16Vector4: ...
    @property
    def ooto(self) -> U16Vector4: ...
    @property
    def ootl(self) -> U16Vector4: ...
    @property
    def ooqs(self) -> U16Vector4: ...
    @property
    def ooqt(self) -> U16Vector4: ...
    @property
    def ooqq(self) -> U16Vector4: ...
    @property
    def ooqp(self) -> U16Vector4: ...
    @property
    def ooqo(self) -> U16Vector4: ...
    @property
    def ooql(self) -> U16Vector4: ...
    @property
    def oops(self) -> U16Vector4: ...
    @property
    def oopt(self) -> U16Vector4: ...
    @property
    def oopq(self) -> U16Vector4: ...
    @property
    def oopp(self) -> U16Vector4: ...
    @property
    def oopo(self) -> U16Vector4: ...
    @property
    def oopl(self) -> U16Vector4: ...
    @property
    def ooos(self) -> U16Vector4: ...
    @property
    def ooot(self) -> U16Vector4: ...
    @property
    def oooq(self) -> U16Vector4: ...
    @property
    def ooop(self) -> U16Vector4: ...
    @property
    def ools(self) -> U16Vector4: ...
    @property
    def oolt(self) -> U16Vector4: ...
    @property
    def oolq(self) -> U16Vector4: ...
    @property
    def oolp(self) -> U16Vector4: ...
    @property
    def olss(self) -> U16Vector4: ...
    @property
    def olst(self) -> U16Vector4: ...
    @property
    def olsq(self) -> U16Vector4: ...
    @property
    def olsp(self) -> U16Vector4: ...
    @property
    def olso(self) -> U16Vector4: ...
    @property
    def olsl(self) -> U16Vector4: ...
    @property
    def olts(self) -> U16Vector4: ...
    @property
    def oltt(self) -> U16Vector4: ...
    @property
    def oltq(self) -> U16Vector4: ...
    @property
    def oltp(self) -> U16Vector4: ...
    @property
    def olto(self) -> U16Vector4: ...
    @property
    def oltl(self) -> U16Vector4: ...
    @property
    def olqs(self) -> U16Vector4: ...
    @property
    def olqt(self) -> U16Vector4: ...
    @property
    def olqq(self) -> U16Vector4: ...
    @property
    def olqp(self) -> U16Vector4: ...
    @property
    def olqo(self) -> U16Vector4: ...
    @property
    def olql(self) -> U16Vector4: ...
    @property
    def olps(self) -> U16Vector4: ...
    @property
    def olpt(self) -> U16Vector4: ...
    @property
    def olpq(self) -> U16Vector4: ...
    @property
    def olpp(self) -> U16Vector4: ...
    @property
    def olpo(self) -> U16Vector4: ...
    @property
    def olpl(self) -> U16Vector4: ...
    @property
    def olos(self) -> U16Vector4: ...
    @property
    def olot(self) -> U16Vector4: ...
    @property
    def oloq(self) -> U16Vector4: ...
    @property
    def olop(self) -> U16Vector4: ...
    @property
    def olls(self) -> U16Vector4: ...
    @property
    def ollt(self) -> U16Vector4: ...
    @property
    def ollq(self) -> U16Vector4: ...
    @property
    def ollp(self) -> U16Vector4: ...
    @property
    def lsss(self) -> U16Vector4: ...
    @property
    def lsst(self) -> U16Vector4: ...
    @property
    def lssq(self) -> U16Vector4: ...
    @property
    def lssp(self) -> U16Vector4: ...
    @property
    def lsso(self) -> U16Vector4: ...
    @property
    def lssl(self) -> U16Vector4: ...
    @property
    def lsts(self) -> U16Vector4: ...
    @property
    def lstt(self) -> U16Vector4: ...
    @property
    def lstq(self) -> U16Vector4: ...
    @property
    def lstp(self) -> U16Vector4: ...
    @property
    def lsto(self) -> U16Vector4: ...
    @property
    def lstl(self) -> U16Vector4: ...
    @property
    def lsqs(self) -> U16Vector4: ...
    @property
    def lsqt(self) -> U16Vector4: ...
    @property
    def lsqq(self) -> U16Vector4: ...
    @property
    def lsqp(self) -> U16Vector4: ...
    @property
    def lsqo(self) -> U16Vector4: ...
    @property
    def lsql(self) -> U16Vector4: ...
    @property
    def lsps(self) -> U16Vector4: ...
    @property
    def lspt(self) -> U16Vector4: ...
    @property
    def lspq(self) -> U16Vector4: ...
    @property
    def lspp(self) -> U16Vector4: ...
    @property
    def lspo(self) -> U16Vector4: ...
    @property
    def lspl(self) -> U16Vector4: ...
    @property
    def lsos(self) -> U16Vector4: ...
    @property
    def lsot(self) -> U16Vector4: ...
    @property
    def lsoq(self) -> U16Vector4: ...
    @property
    def lsop(self) -> U16Vector4: ...
    @property
    def lsoo(self) -> U16Vector4: ...
    @property
    def lsol(self) -> U16Vector4: ...
    @property
    def lsls(self) -> U16Vector4: ...
    @property
    def lslt(self) -> U16Vector4: ...
    @property
    def lslq(self) -> U16Vector4: ...
    @property
    def lslp(self) -> U16Vector4: ...
    @property
    def lslo(self) -> U16Vector4: ...
    @property
    def lsll(self) -> U16Vector4: ...
    @property
    def ltss(self) -> U16Vector4: ...
    @property
    def ltst(self) -> U16Vector4: ...
    @property
    def ltsq(self) -> U16Vector4: ...
    @property
    def ltsp(self) -> U16Vector4: ...
    @property
    def ltso(self) -> U16Vector4: ...
    @property
    def ltsl(self) -> U16Vector4: ...
    @property
    def ltts(self) -> U16Vector4: ...
    @property
    def lttt(self) -> U16Vector4: ...
    @property
    def lttq(self) -> U16Vector4: ...
    @property
    def lttp(self) -> U16Vector4: ...
    @property
    def ltto(self) -> U16Vector4: ...
    @property
    def lttl(self) -> U16Vector4: ...
    @property
    def ltqs(self) -> U16Vector4: ...
    @property
    def ltqt(self) -> U16Vector4: ...
    @property
    def ltqq(self) -> U16Vector4: ...
    @property
    def ltqp(self) -> U16Vector4: ...
    @property
    def ltqo(self) -> U16Vector4: ...
    @property
    def ltql(self) -> U16Vector4: ...
    @property
    def ltps(self) -> U16Vector4: ...
    @property
    def ltpt(self) -> U16Vector4: ...
    @property
    def ltpq(self) -> U16Vector4: ...
    @property
    def ltpp(self) -> U16Vector4: ...
    @property
    def ltpo(self) -> U16Vector4: ...
    @property
    def ltpl(self) -> U16Vector4: ...
    @property
    def ltos(self) -> U16Vector4: ...
    @property
    def ltot(self) -> U16Vector4: ...
    @property
    def ltoq(self) -> U16Vector4: ...
    @property
    def ltop(self) -> U16Vector4: ...
    @property
    def ltoo(self) -> U16Vector4: ...
    @property
    def ltol(self) -> U16Vector4: ...
    @property
    def ltls(self) -> U16Vector4: ...
    @property
    def ltlt(self) -> U16Vector4: ...
    @property
    def ltlq(self) -> U16Vector4: ...
    @property
    def ltlp(self) -> U16Vector4: ...
    @property
    def ltlo(self) -> U16Vector4: ...
    @property
    def ltll(self) -> U16Vector4: ...
    @property
    def lqss(self) -> U16Vector4: ...
    @property
    def lqst(self) -> U16Vector4: ...
    @property
    def lqsq(self) -> U16Vector4: ...
    @property
    def lqsp(self) -> U16Vector4: ...
    @property
    def lqso(self) -> U16Vector4: ...
    @property
    def lqsl(self) -> U16Vector4: ...
    @property
    def lqts(self) -> U16Vector4: ...
    @property
    def lqtt(self) -> U16Vector4: ...
    @property
    def lqtq(self) -> U16Vector4: ...
    @property
    def lqtp(self) -> U16Vector4: ...
    @property
    def lqto(self) -> U16Vector4: ...
    @property
    def lqtl(self) -> U16Vector4: ...
    @property
    def lqqs(self) -> U16Vector4: ...
    @property
    def lqqt(self) -> U16Vector4: ...
    @property
    def lqqq(self) -> U16Vector4: ...
    @property
    def lqqp(self) -> U16Vector4: ...
    @property
    def lqqo(self) -> U16Vector4: ...
    @property
    def lqql(self) -> U16Vector4: ...
    @property
    def lqps(self) -> U16Vector4: ...
    @property
    def lqpt(self) -> U16Vector4: ...
    @property
    def lqpq(self) -> U16Vector4: ...
    @property
    def lqpp(self) -> U16Vector4: ...
    @property
    def lqpo(self) -> U16Vector4: ...
    @property
    def lqpl(self) -> U16Vector4: ...
    @property
    def lqos(self) -> U16Vector4: ...
    @property
    def lqot(self) -> U16Vector4: ...
    @property
    def lqoq(self) -> U16Vector4: ...
    @property
    def lqop(self) -> U16Vector4: ...
    @property
    def lqoo(self) -> U16Vector4: ...
    @property
    def lqol(self) -> U16Vector4: ...
    @property
    def lqls(self) -> U16Vector4: ...
    @property
    def lqlt(self) -> U16Vector4: ...
    @property
    def lqlq(self) -> U16Vector4: ...
    @property
    def lqlp(self) -> U16Vector4: ...
    @property
    def lqlo(self) -> U16Vector4: ...
    @property
    def lqll(self) -> U16Vector4: ...
    @property
    def lpss(self) -> U16Vector4: ...
    @property
    def lpst(self) -> U16Vector4: ...
    @property
    def lpsq(self) -> U16Vector4: ...
    @property
    def lpsp(self) -> U16Vector4: ...
    @property
    def lpso(self) -> U16Vector4: ...
    @property
    def lpsl(self) -> U16Vector4: ...
    @property
    def lpts(self) -> U16Vector4: ...
    @property
    def lptt(self) -> U16Vector4: ...
    @property
    def lptq(self) -> U16Vector4: ...
    @property
    def lptp(self) -> U16Vector4: ...
    @property
    def lpto(self) -> U16Vector4: ...
    @property
    def lptl(self) -> U16Vector4: ...
    @property
    def lpqs(self) -> U16Vector4: ...
    @property
    def lpqt(self) -> U16Vector4: ...
    @property
    def lpqq(self) -> U16Vector4: ...
    @property
    def lpqp(self) -> U16Vector4: ...
    @property
    def lpqo(self) -> U16Vector4: ...
    @property
    def lpql(self) -> U16Vector4: ...
    @property
    def lpps(self) -> U16Vector4: ...
    @property
    def lppt(self) -> U16Vector4: ...
    @property
    def lppq(self) -> U16Vector4: ...
    @property
    def lppp(self) -> U16Vector4: ...
    @property
    def lppo(self) -> U16Vector4: ...
    @property
    def lppl(self) -> U16Vector4: ...
    @property
    def lpos(self) -> U16Vector4: ...
    @property
    def lpot(self) -> U16Vector4: ...
    @property
    def lpoq(self) -> U16Vector4: ...
    @property
    def lpop(self) -> U16Vector4: ...
    @property
    def lpoo(self) -> U16Vector4: ...
    @property
    def lpol(self) -> U16Vector4: ...
    @property
    def lpls(self) -> U16Vector4: ...
    @property
    def lplt(self) -> U16Vector4: ...
    @property
    def lplq(self) -> U16Vector4: ...
    @property
    def lplp(self) -> U16Vector4: ...
    @property
    def lplo(self) -> U16Vector4: ...
    @property
    def lpll(self) -> U16Vector4: ...
    @property
    def loss(self) -> U16Vector4: ...
    @property
    def lost(self) -> U16Vector4: ...
    @property
    def losq(self) -> U16Vector4: ...
    @property
    def losp(self) -> U16Vector4: ...
    @property
    def loso(self) -> U16Vector4: ...
    @property
    def losl(self) -> U16Vector4: ...
    @property
    def lots(self) -> U16Vector4: ...
    @property
    def lott(self) -> U16Vector4: ...
    @property
    def lotq(self) -> U16Vector4: ...
    @property
    def lotp(self) -> U16Vector4: ...
    @property
    def loto(self) -> U16Vector4: ...
    @property
    def lotl(self) -> U16Vector4: ...
    @property
    def loqs(self) -> U16Vector4: ...
    @property
    def loqt(self) -> U16Vector4: ...
    @property
    def loqq(self) -> U16Vector4: ...
    @property
    def loqp(self) -> U16Vector4: ...
    @property
    def loqo(self) -> U16Vector4: ...
    @property
    def loql(self) -> U16Vector4: ...
    @property
    def lops(self) -> U16Vector4: ...
    @property
    def lopt(self) -> U16Vector4: ...
    @property
    def lopq(self) -> U16Vector4: ...
    @property
    def lopp(self) -> U16Vector4: ...
    @property
    def lopo(self) -> U16Vector4: ...
    @property
    def lopl(self) -> U16Vector4: ...
    @property
    def loos(self) -> U16Vector4: ...
    @property
    def loot(self) -> U16Vector4: ...
    @property
    def looq(self) -> U16Vector4: ...
    @property
    def loop(self) -> U16Vector4: ...
    @property
    def lols(self) -> U16Vector4: ...
    @property
    def lolt(self) -> U16Vector4: ...
    @property
    def lolq(self) -> U16Vector4: ...
    @property
    def lolp(self) -> U16Vector4: ...
    @property
    def llss(self) -> U16Vector4: ...
    @property
    def llst(self) -> U16Vector4: ...
    @property
    def llsq(self) -> U16Vector4: ...
    @property
    def llsp(self) -> U16Vector4: ...
    @property
    def llso(self) -> U16Vector4: ...
    @property
    def llsl(self) -> U16Vector4: ...
    @property
    def llts(self) -> U16Vector4: ...
    @property
    def lltt(self) -> U16Vector4: ...
    @property
    def lltq(self) -> U16Vector4: ...
    @property
    def lltp(self) -> U16Vector4: ...
    @property
    def llto(self) -> U16Vector4: ...
    @property
    def lltl(self) -> U16Vector4: ...
    @property
    def llqs(self) -> U16Vector4: ...
    @property
    def llqt(self) -> U16Vector4: ...
    @property
    def llqq(self) -> U16Vector4: ...
    @property
    def llqp(self) -> U16Vector4: ...
    @property
    def llqo(self) -> U16Vector4: ...
    @property
    def llql(self) -> U16Vector4: ...
    @property
    def llps(self) -> U16Vector4: ...
    @property
    def llpt(self) -> U16Vector4: ...
    @property
    def llpq(self) -> U16Vector4: ...
    @property
    def llpp(self) -> U16Vector4: ...
    @property
    def llpo(self) -> U16Vector4: ...
    @property
    def llpl(self) -> U16Vector4: ...
    @property
    def llos(self) -> U16Vector4: ...
    @property
    def llot(self) -> U16Vector4: ...
    @property
    def lloq(self) -> U16Vector4: ...
    @property
    def llop(self) -> U16Vector4: ...
    @property
    def llls(self) -> U16Vector4: ...
    @property
    def lllt(self) -> U16Vector4: ...
    @property
    def lllq(self) -> U16Vector4: ...
    @property
    def lllp(self) -> U16Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U16Vector2: ...
    @property
    def uv(self) -> U16Vector2: ...
    @property
    def uo(self) -> U16Vector2: ...
    @property
    def ul(self) -> U16Vector2: ...
    @property
    def vu(self) -> U16Vector2: ...
    @property
    def vv(self) -> U16Vector2: ...
    @property
    def vo(self) -> U16Vector2: ...
    @property
    def vl(self) -> U16Vector2: ...
    @property
    def ou(self) -> U16Vector2: ...
    @property
    def ov(self) -> U16Vector2: ...
    @property
    def lu(self) -> U16Vector2: ...
    @property
    def lv(self) -> U16Vector2: ...
    @property
    def uuu(self) -> U16Vector3: ...
    @property
    def uuv(self) -> U16Vector3: ...
    @property
    def uuo(self) -> U16Vector3: ...
    @property
    def uul(self) -> U16Vector3: ...
    @property
    def uvu(self) -> U16Vector3: ...
    @property
    def uvv(self) -> U16Vector3: ...
    @property
    def uvo(self) -> U16Vector3: ...
    @property
    def uvl(self) -> U16Vector3: ...
    @property
    def uou(self) -> U16Vector3: ...
    @property
    def uov(self) -> U16Vector3: ...
    @property
    def uoo(self) -> U16Vector3: ...
    @property
    def uol(self) -> U16Vector3: ...
    @property
    def ulu(self) -> U16Vector3: ...
    @property
    def ulv(self) -> U16Vector3: ...
    @property
    def ulo(self) -> U16Vector3: ...
    @property
    def ull(self) -> U16Vector3: ...
    @property
    def vuu(self) -> U16Vector3: ...
    @property
    def vuv(self) -> U16Vector3: ...
    @property
    def vuo(self) -> U16Vector3: ...
    @property
    def vul(self) -> U16Vector3: ...
    @property
    def vvu(self) -> U16Vector3: ...
    @property
    def vvv(self) -> U16Vector3: ...
    @property
    def vvo(self) -> U16Vector3: ...
    @property
    def vvl(self) -> U16Vector3: ...
    @property
    def vou(self) -> U16Vector3: ...
    @property
    def vov(self) -> U16Vector3: ...
    @property
    def voo(self) -> U16Vector3: ...
    @property
    def vol(self) -> U16Vector3: ...
    @property
    def vlu(self) -> U16Vector3: ...
    @property
    def vlv(self) -> U16Vector3: ...
    @property
    def vlo(self) -> U16Vector3: ...
    @property
    def vll(self) -> U16Vector3: ...
    @property
    def ouu(self) -> U16Vector3: ...
    @property
    def ouv(self) -> U16Vector3: ...
    @property
    def ouo(self) -> U16Vector3: ...
    @property
    def oul(self) -> U16Vector3: ...
    @property
    def ovu(self) -> U16Vector3: ...
    @property
    def ovv(self) -> U16Vector3: ...
    @property
    def ovo(self) -> U16Vector3: ...
    @property
    def ovl(self) -> U16Vector3: ...
    @property
    def oou(self) -> U16Vector3: ...
    @property
    def oov(self) -> U16Vector3: ...
    @property
    def olu(self) -> U16Vector3: ...
    @property
    def olv(self) -> U16Vector3: ...
    @property
    def luu(self) -> U16Vector3: ...
    @property
    def luv(self) -> U16Vector3: ...
    @property
    def luo(self) -> U16Vector3: ...
    @property
    def lul(self) -> U16Vector3: ...
    @property
    def lvu(self) -> U16Vector3: ...
    @property
    def lvv(self) -> U16Vector3: ...
    @property
    def lvo(self) -> U16Vector3: ...
    @property
    def lvl(self) -> U16Vector3: ...
    @property
    def lou(self) -> U16Vector3: ...
    @property
    def lov(self) -> U16Vector3: ...
    @property
    def llu(self) -> U16Vector3: ...
    @property
    def llv(self) -> U16Vector3: ...
    @property
    def uuuu(self) -> U16Vector4: ...
    @property
    def uuuv(self) -> U16Vector4: ...
    @property
    def uuuo(self) -> U16Vector4: ...
    @property
    def uuul(self) -> U16Vector4: ...
    @property
    def uuvu(self) -> U16Vector4: ...
    @property
    def uuvv(self) -> U16Vector4: ...
    @property
    def uuvo(self) -> U16Vector4: ...
    @property
    def uuvl(self) -> U16Vector4: ...
    @property
    def uuou(self) -> U16Vector4: ...
    @property
    def uuov(self) -> U16Vector4: ...
    @property
    def uuoo(self) -> U16Vector4: ...
    @property
    def uuol(self) -> U16Vector4: ...
    @property
    def uulu(self) -> U16Vector4: ...
    @property
    def uulv(self) -> U16Vector4: ...
    @property
    def uulo(self) -> U16Vector4: ...
    @property
    def uull(self) -> U16Vector4: ...
    @property
    def uvuu(self) -> U16Vector4: ...
    @property
    def uvuv(self) -> U16Vector4: ...
    @property
    def uvuo(self) -> U16Vector4: ...
    @property
    def uvul(self) -> U16Vector4: ...
    @property
    def uvvu(self) -> U16Vector4: ...
    @property
    def uvvv(self) -> U16Vector4: ...
    @property
    def uvvo(self) -> U16Vector4: ...
    @property
    def uvvl(self) -> U16Vector4: ...
    @property
    def uvou(self) -> U16Vector4: ...
    @property
    def uvov(self) -> U16Vector4: ...
    @property
    def uvoo(self) -> U16Vector4: ...
    @property
    def uvol(self) -> U16Vector4: ...
    @property
    def uvlu(self) -> U16Vector4: ...
    @property
    def uvlv(self) -> U16Vector4: ...
    @property
    def uvlo(self) -> U16Vector4: ...
    @property
    def uvll(self) -> U16Vector4: ...
    @property
    def uouu(self) -> U16Vector4: ...
    @property
    def uouv(self) -> U16Vector4: ...
    @property
    def uouo(self) -> U16Vector4: ...
    @property
    def uoul(self) -> U16Vector4: ...
    @property
    def uovu(self) -> U16Vector4: ...
    @property
    def uovv(self) -> U16Vector4: ...
    @property
    def uovo(self) -> U16Vector4: ...
    @property
    def uovl(self) -> U16Vector4: ...
    @property
    def uoou(self) -> U16Vector4: ...
    @property
    def uoov(self) -> U16Vector4: ...
    @property
    def uooo(self) -> U16Vector4: ...
    @property
    def uool(self) -> U16Vector4: ...
    @property
    def uolu(self) -> U16Vector4: ...
    @property
    def uolv(self) -> U16Vector4: ...
    @property
    def uolo(self) -> U16Vector4: ...
    @property
    def uoll(self) -> U16Vector4: ...
    @property
    def uluu(self) -> U16Vector4: ...
    @property
    def uluv(self) -> U16Vector4: ...
    @property
    def uluo(self) -> U16Vector4: ...
    @property
    def ulul(self) -> U16Vector4: ...
    @property
    def ulvu(self) -> U16Vector4: ...
    @property
    def ulvv(self) -> U16Vector4: ...
    @property
    def ulvo(self) -> U16Vector4: ...
    @property
    def ulvl(self) -> U16Vector4: ...
    @property
    def ulou(self) -> U16Vector4: ...
    @property
    def ulov(self) -> U16Vector4: ...
    @property
    def uloo(self) -> U16Vector4: ...
    @property
    def ulol(self) -> U16Vector4: ...
    @property
    def ullu(self) -> U16Vector4: ...
    @property
    def ullv(self) -> U16Vector4: ...
    @property
    def ullo(self) -> U16Vector4: ...
    @property
    def ulll(self) -> U16Vector4: ...
    @property
    def vuuu(self) -> U16Vector4: ...
    @property
    def vuuv(self) -> U16Vector4: ...
    @property
    def vuuo(self) -> U16Vector4: ...
    @property
    def vuul(self) -> U16Vector4: ...
    @property
    def vuvu(self) -> U16Vector4: ...
    @property
    def vuvv(self) -> U16Vector4: ...
    @property
    def vuvo(self) -> U16Vector4: ...
    @property
    def vuvl(self) -> U16Vector4: ...
    @property
    def vuou(self) -> U16Vector4: ...
    @property
    def vuov(self) -> U16Vector4: ...
    @property
    def vuoo(self) -> U16Vector4: ...
    @property
    def vuol(self) -> U16Vector4: ...
    @property
    def vulu(self) -> U16Vector4: ...
    @property
    def vulv(self) -> U16Vector4: ...
    @property
    def vulo(self) -> U16Vector4: ...
    @property
    def vull(self) -> U16Vector4: ...
    @property
    def vvuu(self) -> U16Vector4: ...
    @property
    def vvuv(self) -> U16Vector4: ...
    @property
    def vvuo(self) -> U16Vector4: ...
    @property
    def vvul(self) -> U16Vector4: ...
    @property
    def vvvu(self) -> U16Vector4: ...
    @property
    def vvvv(self) -> U16Vector4: ...
    @property
    def vvvo(self) -> U16Vector4: ...
    @property
    def vvvl(self) -> U16Vector4: ...
    @property
    def vvou(self) -> U16Vector4: ...
    @property
    def vvov(self) -> U16Vector4: ...
    @property
    def vvoo(self) -> U16Vector4: ...
    @property
    def vvol(self) -> U16Vector4: ...
    @property
    def vvlu(self) -> U16Vector4: ...
    @property
    def vvlv(self) -> U16Vector4: ...
    @property
    def vvlo(self) -> U16Vector4: ...
    @property
    def vvll(self) -> U16Vector4: ...
    @property
    def vouu(self) -> U16Vector4: ...
    @property
    def vouv(self) -> U16Vector4: ...
    @property
    def vouo(self) -> U16Vector4: ...
    @property
    def voul(self) -> U16Vector4: ...
    @property
    def vovu(self) -> U16Vector4: ...
    @property
    def vovv(self) -> U16Vector4: ...
    @property
    def vovo(self) -> U16Vector4: ...
    @property
    def vovl(self) -> U16Vector4: ...
    @property
    def voou(self) -> U16Vector4: ...
    @property
    def voov(self) -> U16Vector4: ...
    @property
    def vooo(self) -> U16Vector4: ...
    @property
    def vool(self) -> U16Vector4: ...
    @property
    def volu(self) -> U16Vector4: ...
    @property
    def volv(self) -> U16Vector4: ...
    @property
    def volo(self) -> U16Vector4: ...
    @property
    def voll(self) -> U16Vector4: ...
    @property
    def vluu(self) -> U16Vector4: ...
    @property
    def vluv(self) -> U16Vector4: ...
    @property
    def vluo(self) -> U16Vector4: ...
    @property
    def vlul(self) -> U16Vector4: ...
    @property
    def vlvu(self) -> U16Vector4: ...
    @property
    def vlvv(self) -> U16Vector4: ...
    @property
    def vlvo(self) -> U16Vector4: ...
    @property
    def vlvl(self) -> U16Vector4: ...
    @property
    def vlou(self) -> U16Vector4: ...
    @property
    def vlov(self) -> U16Vector4: ...
    @property
    def vloo(self) -> U16Vector4: ...
    @property
    def vlol(self) -> U16Vector4: ...
    @property
    def vllu(self) -> U16Vector4: ...
    @property
    def vllv(self) -> U16Vector4: ...
    @property
    def vllo(self) -> U16Vector4: ...
    @property
    def vlll(self) -> U16Vector4: ...
    @property
    def ouuu(self) -> U16Vector4: ...
    @property
    def ouuv(self) -> U16Vector4: ...
    @property
    def ouuo(self) -> U16Vector4: ...
    @property
    def ouul(self) -> U16Vector4: ...
    @property
    def ouvu(self) -> U16Vector4: ...
    @property
    def ouvv(self) -> U16Vector4: ...
    @property
    def ouvo(self) -> U16Vector4: ...
    @property
    def ouvl(self) -> U16Vector4: ...
    @property
    def ouou(self) -> U16Vector4: ...
    @property
    def ouov(self) -> U16Vector4: ...
    @property
    def ouoo(self) -> U16Vector4: ...
    @property
    def ouol(self) -> U16Vector4: ...
    @property
    def oulu(self) -> U16Vector4: ...
    @property
    def oulv(self) -> U16Vector4: ...
    @property
    def oulo(self) -> U16Vector4: ...
    @property
    def oull(self) -> U16Vector4: ...
    @property
    def ovuu(self) -> U16Vector4: ...
    @property
    def ovuv(self) -> U16Vector4: ...
    @property
    def ovuo(self) -> U16Vector4: ...
    @property
    def ovul(self) -> U16Vector4: ...
    @property
    def ovvu(self) -> U16Vector4: ...
    @property
    def ovvv(self) -> U16Vector4: ...
    @property
    def ovvo(self) -> U16Vector4: ...
    @property
    def ovvl(self) -> U16Vector4: ...
    @property
    def ovou(self) -> U16Vector4: ...
    @property
    def ovov(self) -> U16Vector4: ...
    @property
    def ovoo(self) -> U16Vector4: ...
    @property
    def ovol(self) -> U16Vector4: ...
    @property
    def ovlu(self) -> U16Vector4: ...
    @property
    def ovlv(self) -> U16Vector4: ...
    @property
    def ovlo(self) -> U16Vector4: ...
    @property
    def ovll(self) -> U16Vector4: ...
    @property
    def oouu(self) -> U16Vector4: ...
    @property
    def oouv(self) -> U16Vector4: ...
    @property
    def oouo(self) -> U16Vector4: ...
    @property
    def ooul(self) -> U16Vector4: ...
    @property
    def oovu(self) -> U16Vector4: ...
    @property
    def oovv(self) -> U16Vector4: ...
    @property
    def oovo(self) -> U16Vector4: ...
    @property
    def oovl(self) -> U16Vector4: ...
    @property
    def ooou(self) -> U16Vector4: ...
    @property
    def ooov(self) -> U16Vector4: ...
    @property
    def oolu(self) -> U16Vector4: ...
    @property
    def oolv(self) -> U16Vector4: ...
    @property
    def oluu(self) -> U16Vector4: ...
    @property
    def oluv(self) -> U16Vector4: ...
    @property
    def oluo(self) -> U16Vector4: ...
    @property
    def olul(self) -> U16Vector4: ...
    @property
    def olvu(self) -> U16Vector4: ...
    @property
    def olvv(self) -> U16Vector4: ...
    @property
    def olvo(self) -> U16Vector4: ...
    @property
    def olvl(self) -> U16Vector4: ...
    @property
    def olou(self) -> U16Vector4: ...
    @property
    def olov(self) -> U16Vector4: ...
    @property
    def ollu(self) -> U16Vector4: ...
    @property
    def ollv(self) -> U16Vector4: ...
    @property
    def luuu(self) -> U16Vector4: ...
    @property
    def luuv(self) -> U16Vector4: ...
    @property
    def luuo(self) -> U16Vector4: ...
    @property
    def luul(self) -> U16Vector4: ...
    @property
    def luvu(self) -> U16Vector4: ...
    @property
    def luvv(self) -> U16Vector4: ...
    @property
    def luvo(self) -> U16Vector4: ...
    @property
    def luvl(self) -> U16Vector4: ...
    @property
    def luou(self) -> U16Vector4: ...
    @property
    def luov(self) -> U16Vector4: ...
    @property
    def luoo(self) -> U16Vector4: ...
    @property
    def luol(self) -> U16Vector4: ...
    @property
    def lulu(self) -> U16Vector4: ...
    @property
    def lulv(self) -> U16Vector4: ...
    @property
    def lulo(self) -> U16Vector4: ...
    @property
    def lull(self) -> U16Vector4: ...
    @property
    def lvuu(self) -> U16Vector4: ...
    @property
    def lvuv(self) -> U16Vector4: ...
    @property
    def lvuo(self) -> U16Vector4: ...
    @property
    def lvul(self) -> U16Vector4: ...
    @property
    def lvvu(self) -> U16Vector4: ...
    @property
    def lvvv(self) -> U16Vector4: ...
    @property
    def lvvo(self) -> U16Vector4: ...
    @property
    def lvvl(self) -> U16Vector4: ...
    @property
    def lvou(self) -> U16Vector4: ...
    @property
    def lvov(self) -> U16Vector4: ...
    @property
    def lvoo(self) -> U16Vector4: ...
    @property
    def lvol(self) -> U16Vector4: ...
    @property
    def lvlu(self) -> U16Vector4: ...
    @property
    def lvlv(self) -> U16Vector4: ...
    @property
    def lvlo(self) -> U16Vector4: ...
    @property
    def lvll(self) -> U16Vector4: ...
    @property
    def louu(self) -> U16Vector4: ...
    @property
    def louv(self) -> U16Vector4: ...
    @property
    def louo(self) -> U16Vector4: ...
    @property
    def loul(self) -> U16Vector4: ...
    @property
    def lovu(self) -> U16Vector4: ...
    @property
    def lovv(self) -> U16Vector4: ...
    @property
    def lovo(self) -> U16Vector4: ...
    @property
    def lovl(self) -> U16Vector4: ...
    @property
    def loou(self) -> U16Vector4: ...
    @property
    def loov(self) -> U16Vector4: ...
    @property
    def lolu(self) -> U16Vector4: ...
    @property
    def lolv(self) -> U16Vector4: ...
    @property
    def lluu(self) -> U16Vector4: ...
    @property
    def lluv(self) -> U16Vector4: ...
    @property
    def lluo(self) -> U16Vector4: ...
    @property
    def llul(self) -> U16Vector4: ...
    @property
    def llvu(self) -> U16Vector4: ...
    @property
    def llvv(self) -> U16Vector4: ...
    @property
    def llvo(self) -> U16Vector4: ...
    @property
    def llvl(self) -> U16Vector4: ...
    @property
    def llou(self) -> U16Vector4: ...
    @property
    def llov(self) -> U16Vector4: ...
    @property
    def lllu(self) -> U16Vector4: ...
    @property
    def lllv(self) -> U16Vector4: ...
    @property
    def oo(self) -> U16Vector2: ...
    @property
    def ol(self) -> U16Vector2: ...
    @property
    def lo(self) -> U16Vector2: ...
    @property
    def ll(self) -> U16Vector2: ...
    @property
    def ooo(self) -> U16Vector3: ...
    @property
    def ool(self) -> U16Vector3: ...
    @property
    def olo(self) -> U16Vector3: ...
    @property
    def oll(self) -> U16Vector3: ...
    @property
    def loo(self) -> U16Vector3: ...
    @property
    def lol(self) -> U16Vector3: ...
    @property
    def llo(self) -> U16Vector3: ...
    @property
    def lll(self) -> U16Vector3: ...
    @property
    def oooo(self) -> U16Vector4: ...
    @property
    def oool(self) -> U16Vector4: ...
    @property
    def oolo(self) -> U16Vector4: ...
    @property
    def ooll(self) -> U16Vector4: ...
    @property
    def oloo(self) -> U16Vector4: ...
    @property
    def olol(self) -> U16Vector4: ...
    @property
    def ollo(self) -> U16Vector4: ...
    @property
    def olll(self) -> U16Vector4: ...
    @property
    def looo(self) -> U16Vector4: ...
    @property
    def lool(self) -> U16Vector4: ...
    @property
    def lolo(self) -> U16Vector4: ...
    @property
    def loll(self) -> U16Vector4: ...
    @property
    def lloo(self) -> U16Vector4: ...
    @property
    def llol(self) -> U16Vector4: ...
    @property
    def lllo(self) -> U16Vector4: ...
    @property
    def llll(self) -> U16Vector4: ...
    def min(self, n: Number, /) -> U16Vector4: ...
    def max(self, n: Number, /) -> U16Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> U16Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U16Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector4: ...

@final
class U16Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U16Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U16Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> U16Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> U16Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U16Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[U16Vector4]: ...

@final
class I32Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __add__(self, other: Number) -> I32Vector4: ...
    @overload
    def __radd__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __radd__(self, other: Number) -> I32Vector4: ...
    @overload
    def __sub__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __sub__(self, other: Number) -> I32Vector4: ...
    @overload
    def __rsub__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> I32Vector4: ...
    @overload
    def __mul__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __mul__(self, other: Number) -> I32Vector4: ...
    @overload
    def __rmul__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> I32Vector4: ...
    @overload
    def __truediv__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> I32Vector4: ...
    @overload
    def __rtruediv__(self, other: I32Vector4) -> I32Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> I32Vector4: ...
    def __neg__(self) -> I32Vector4: ...
    def __abs__(self) -> I32Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> I32Vector2: ...
    @property
    def xy(self) -> I32Vector2: ...
    @property
    def xz(self) -> I32Vector2: ...
    @property
    def xw(self) -> I32Vector2: ...
    @property
    def xo(self) -> I32Vector2: ...
    @property
    def xl(self) -> I32Vector2: ...
    @property
    def yx(self) -> I32Vector2: ...
    @property
    def yy(self) -> I32Vector2: ...
    @property
    def yz(self) -> I32Vector2: ...
    @property
    def yw(self) -> I32Vector2: ...
    @property
    def yo(self) -> I32Vector2: ...
    @property
    def yl(self) -> I32Vector2: ...
    @property
    def zx(self) -> I32Vector2: ...
    @property
    def zy(self) -> I32Vector2: ...
    @property
    def zz(self) -> I32Vector2: ...
    @property
    def zw(self) -> I32Vector2: ...
    @property
    def zo(self) -> I32Vector2: ...
    @property
    def zl(self) -> I32Vector2: ...
    @property
    def wx(self) -> I32Vector2: ...
    @property
    def wy(self) -> I32Vector2: ...
    @property
    def wz(self) -> I32Vector2: ...
    @property
    def ww(self) -> I32Vector2: ...
    @property
    def wo(self) -> I32Vector2: ...
    @property
    def wl(self) -> I32Vector2: ...
    @property
    def ox(self) -> I32Vector2: ...
    @property
    def oy(self) -> I32Vector2: ...
    @property
    def oz(self) -> I32Vector2: ...
    @property
    def ow(self) -> I32Vector2: ...
    @property
    def lx(self) -> I32Vector2: ...
    @property
    def ly(self) -> I32Vector2: ...
    @property
    def lz(self) -> I32Vector2: ...
    @property
    def lw(self) -> I32Vector2: ...
    @property
    def xxx(self) -> I32Vector3: ...
    @property
    def xxy(self) -> I32Vector3: ...
    @property
    def xxz(self) -> I32Vector3: ...
    @property
    def xxw(self) -> I32Vector3: ...
    @property
    def xxo(self) -> I32Vector3: ...
    @property
    def xxl(self) -> I32Vector3: ...
    @property
    def xyx(self) -> I32Vector3: ...
    @property
    def xyy(self) -> I32Vector3: ...
    @property
    def xyz(self) -> I32Vector3: ...
    @property
    def xyw(self) -> I32Vector3: ...
    @property
    def xyo(self) -> I32Vector3: ...
    @property
    def xyl(self) -> I32Vector3: ...
    @property
    def xzx(self) -> I32Vector3: ...
    @property
    def xzy(self) -> I32Vector3: ...
    @property
    def xzz(self) -> I32Vector3: ...
    @property
    def xzw(self) -> I32Vector3: ...
    @property
    def xzo(self) -> I32Vector3: ...
    @property
    def xzl(self) -> I32Vector3: ...
    @property
    def xwx(self) -> I32Vector3: ...
    @property
    def xwy(self) -> I32Vector3: ...
    @property
    def xwz(self) -> I32Vector3: ...
    @property
    def xww(self) -> I32Vector3: ...
    @property
    def xwo(self) -> I32Vector3: ...
    @property
    def xwl(self) -> I32Vector3: ...
    @property
    def xox(self) -> I32Vector3: ...
    @property
    def xoy(self) -> I32Vector3: ...
    @property
    def xoz(self) -> I32Vector3: ...
    @property
    def xow(self) -> I32Vector3: ...
    @property
    def xoo(self) -> I32Vector3: ...
    @property
    def xol(self) -> I32Vector3: ...
    @property
    def xlx(self) -> I32Vector3: ...
    @property
    def xly(self) -> I32Vector3: ...
    @property
    def xlz(self) -> I32Vector3: ...
    @property
    def xlw(self) -> I32Vector3: ...
    @property
    def xlo(self) -> I32Vector3: ...
    @property
    def xll(self) -> I32Vector3: ...
    @property
    def yxx(self) -> I32Vector3: ...
    @property
    def yxy(self) -> I32Vector3: ...
    @property
    def yxz(self) -> I32Vector3: ...
    @property
    def yxw(self) -> I32Vector3: ...
    @property
    def yxo(self) -> I32Vector3: ...
    @property
    def yxl(self) -> I32Vector3: ...
    @property
    def yyx(self) -> I32Vector3: ...
    @property
    def yyy(self) -> I32Vector3: ...
    @property
    def yyz(self) -> I32Vector3: ...
    @property
    def yyw(self) -> I32Vector3: ...
    @property
    def yyo(self) -> I32Vector3: ...
    @property
    def yyl(self) -> I32Vector3: ...
    @property
    def yzx(self) -> I32Vector3: ...
    @property
    def yzy(self) -> I32Vector3: ...
    @property
    def yzz(self) -> I32Vector3: ...
    @property
    def yzw(self) -> I32Vector3: ...
    @property
    def yzo(self) -> I32Vector3: ...
    @property
    def yzl(self) -> I32Vector3: ...
    @property
    def ywx(self) -> I32Vector3: ...
    @property
    def ywy(self) -> I32Vector3: ...
    @property
    def ywz(self) -> I32Vector3: ...
    @property
    def yww(self) -> I32Vector3: ...
    @property
    def ywo(self) -> I32Vector3: ...
    @property
    def ywl(self) -> I32Vector3: ...
    @property
    def yox(self) -> I32Vector3: ...
    @property
    def yoy(self) -> I32Vector3: ...
    @property
    def yoz(self) -> I32Vector3: ...
    @property
    def yow(self) -> I32Vector3: ...
    @property
    def yoo(self) -> I32Vector3: ...
    @property
    def yol(self) -> I32Vector3: ...
    @property
    def ylx(self) -> I32Vector3: ...
    @property
    def yly(self) -> I32Vector3: ...
    @property
    def ylz(self) -> I32Vector3: ...
    @property
    def ylw(self) -> I32Vector3: ...
    @property
    def ylo(self) -> I32Vector3: ...
    @property
    def yll(self) -> I32Vector3: ...
    @property
    def zxx(self) -> I32Vector3: ...
    @property
    def zxy(self) -> I32Vector3: ...
    @property
    def zxz(self) -> I32Vector3: ...
    @property
    def zxw(self) -> I32Vector3: ...
    @property
    def zxo(self) -> I32Vector3: ...
    @property
    def zxl(self) -> I32Vector3: ...
    @property
    def zyx(self) -> I32Vector3: ...
    @property
    def zyy(self) -> I32Vector3: ...
    @property
    def zyz(self) -> I32Vector3: ...
    @property
    def zyw(self) -> I32Vector3: ...
    @property
    def zyo(self) -> I32Vector3: ...
    @property
    def zyl(self) -> I32Vector3: ...
    @property
    def zzx(self) -> I32Vector3: ...
    @property
    def zzy(self) -> I32Vector3: ...
    @property
    def zzz(self) -> I32Vector3: ...
    @property
    def zzw(self) -> I32Vector3: ...
    @property
    def zzo(self) -> I32Vector3: ...
    @property
    def zzl(self) -> I32Vector3: ...
    @property
    def zwx(self) -> I32Vector3: ...
    @property
    def zwy(self) -> I32Vector3: ...
    @property
    def zwz(self) -> I32Vector3: ...
    @property
    def zww(self) -> I32Vector3: ...
    @property
    def zwo(self) -> I32Vector3: ...
    @property
    def zwl(self) -> I32Vector3: ...
    @property
    def zox(self) -> I32Vector3: ...
    @property
    def zoy(self) -> I32Vector3: ...
    @property
    def zoz(self) -> I32Vector3: ...
    @property
    def zow(self) -> I32Vector3: ...
    @property
    def zoo(self) -> I32Vector3: ...
    @property
    def zol(self) -> I32Vector3: ...
    @property
    def zlx(self) -> I32Vector3: ...
    @property
    def zly(self) -> I32Vector3: ...
    @property
    def zlz(self) -> I32Vector3: ...
    @property
    def zlw(self) -> I32Vector3: ...
    @property
    def zlo(self) -> I32Vector3: ...
    @property
    def zll(self) -> I32Vector3: ...
    @property
    def wxx(self) -> I32Vector3: ...
    @property
    def wxy(self) -> I32Vector3: ...
    @property
    def wxz(self) -> I32Vector3: ...
    @property
    def wxw(self) -> I32Vector3: ...
    @property
    def wxo(self) -> I32Vector3: ...
    @property
    def wxl(self) -> I32Vector3: ...
    @property
    def wyx(self) -> I32Vector3: ...
    @property
    def wyy(self) -> I32Vector3: ...
    @property
    def wyz(self) -> I32Vector3: ...
    @property
    def wyw(self) -> I32Vector3: ...
    @property
    def wyo(self) -> I32Vector3: ...
    @property
    def wyl(self) -> I32Vector3: ...
    @property
    def wzx(self) -> I32Vector3: ...
    @property
    def wzy(self) -> I32Vector3: ...
    @property
    def wzz(self) -> I32Vector3: ...
    @property
    def wzw(self) -> I32Vector3: ...
    @property
    def wzo(self) -> I32Vector3: ...
    @property
    def wzl(self) -> I32Vector3: ...
    @property
    def wwx(self) -> I32Vector3: ...
    @property
    def wwy(self) -> I32Vector3: ...
    @property
    def wwz(self) -> I32Vector3: ...
    @property
    def www(self) -> I32Vector3: ...
    @property
    def wwo(self) -> I32Vector3: ...
    @property
    def wwl(self) -> I32Vector3: ...
    @property
    def wox(self) -> I32Vector3: ...
    @property
    def woy(self) -> I32Vector3: ...
    @property
    def woz(self) -> I32Vector3: ...
    @property
    def wow(self) -> I32Vector3: ...
    @property
    def woo(self) -> I32Vector3: ...
    @property
    def wol(self) -> I32Vector3: ...
    @property
    def wlx(self) -> I32Vector3: ...
    @property
    def wly(self) -> I32Vector3: ...
    @property
    def wlz(self) -> I32Vector3: ...
    @property
    def wlw(self) -> I32Vector3: ...
    @property
    def wlo(self) -> I32Vector3: ...
    @property
    def wll(self) -> I32Vector3: ...
    @property
    def oxx(self) -> I32Vector3: ...
    @property
    def oxy(self) -> I32Vector3: ...
    @property
    def oxz(self) -> I32Vector3: ...
    @property
    def oxw(self) -> I32Vector3: ...
    @property
    def oxo(self) -> I32Vector3: ...
    @property
    def oxl(self) -> I32Vector3: ...
    @property
    def oyx(self) -> I32Vector3: ...
    @property
    def oyy(self) -> I32Vector3: ...
    @property
    def oyz(self) -> I32Vector3: ...
    @property
    def oyw(self) -> I32Vector3: ...
    @property
    def oyo(self) -> I32Vector3: ...
    @property
    def oyl(self) -> I32Vector3: ...
    @property
    def ozx(self) -> I32Vector3: ...
    @property
    def ozy(self) -> I32Vector3: ...
    @property
    def ozz(self) -> I32Vector3: ...
    @property
    def ozw(self) -> I32Vector3: ...
    @property
    def ozo(self) -> I32Vector3: ...
    @property
    def ozl(self) -> I32Vector3: ...
    @property
    def owx(self) -> I32Vector3: ...
    @property
    def owy(self) -> I32Vector3: ...
    @property
    def owz(self) -> I32Vector3: ...
    @property
    def oww(self) -> I32Vector3: ...
    @property
    def owo(self) -> I32Vector3: ...
    @property
    def owl(self) -> I32Vector3: ...
    @property
    def oox(self) -> I32Vector3: ...
    @property
    def ooy(self) -> I32Vector3: ...
    @property
    def ooz(self) -> I32Vector3: ...
    @property
    def oow(self) -> I32Vector3: ...
    @property
    def olx(self) -> I32Vector3: ...
    @property
    def oly(self) -> I32Vector3: ...
    @property
    def olz(self) -> I32Vector3: ...
    @property
    def olw(self) -> I32Vector3: ...
    @property
    def lxx(self) -> I32Vector3: ...
    @property
    def lxy(self) -> I32Vector3: ...
    @property
    def lxz(self) -> I32Vector3: ...
    @property
    def lxw(self) -> I32Vector3: ...
    @property
    def lxo(self) -> I32Vector3: ...
    @property
    def lxl(self) -> I32Vector3: ...
    @property
    def lyx(self) -> I32Vector3: ...
    @property
    def lyy(self) -> I32Vector3: ...
    @property
    def lyz(self) -> I32Vector3: ...
    @property
    def lyw(self) -> I32Vector3: ...
    @property
    def lyo(self) -> I32Vector3: ...
    @property
    def lyl(self) -> I32Vector3: ...
    @property
    def lzx(self) -> I32Vector3: ...
    @property
    def lzy(self) -> I32Vector3: ...
    @property
    def lzz(self) -> I32Vector3: ...
    @property
    def lzw(self) -> I32Vector3: ...
    @property
    def lzo(self) -> I32Vector3: ...
    @property
    def lzl(self) -> I32Vector3: ...
    @property
    def lwx(self) -> I32Vector3: ...
    @property
    def lwy(self) -> I32Vector3: ...
    @property
    def lwz(self) -> I32Vector3: ...
    @property
    def lww(self) -> I32Vector3: ...
    @property
    def lwo(self) -> I32Vector3: ...
    @property
    def lwl(self) -> I32Vector3: ...
    @property
    def lox(self) -> I32Vector3: ...
    @property
    def loy(self) -> I32Vector3: ...
    @property
    def loz(self) -> I32Vector3: ...
    @property
    def low(self) -> I32Vector3: ...
    @property
    def llx(self) -> I32Vector3: ...
    @property
    def lly(self) -> I32Vector3: ...
    @property
    def llz(self) -> I32Vector3: ...
    @property
    def llw(self) -> I32Vector3: ...
    @property
    def xxxx(self) -> I32Vector4: ...
    @property
    def xxxy(self) -> I32Vector4: ...
    @property
    def xxxz(self) -> I32Vector4: ...
    @property
    def xxxw(self) -> I32Vector4: ...
    @property
    def xxxo(self) -> I32Vector4: ...
    @property
    def xxxl(self) -> I32Vector4: ...
    @property
    def xxyx(self) -> I32Vector4: ...
    @property
    def xxyy(self) -> I32Vector4: ...
    @property
    def xxyz(self) -> I32Vector4: ...
    @property
    def xxyw(self) -> I32Vector4: ...
    @property
    def xxyo(self) -> I32Vector4: ...
    @property
    def xxyl(self) -> I32Vector4: ...
    @property
    def xxzx(self) -> I32Vector4: ...
    @property
    def xxzy(self) -> I32Vector4: ...
    @property
    def xxzz(self) -> I32Vector4: ...
    @property
    def xxzw(self) -> I32Vector4: ...
    @property
    def xxzo(self) -> I32Vector4: ...
    @property
    def xxzl(self) -> I32Vector4: ...
    @property
    def xxwx(self) -> I32Vector4: ...
    @property
    def xxwy(self) -> I32Vector4: ...
    @property
    def xxwz(self) -> I32Vector4: ...
    @property
    def xxww(self) -> I32Vector4: ...
    @property
    def xxwo(self) -> I32Vector4: ...
    @property
    def xxwl(self) -> I32Vector4: ...
    @property
    def xxox(self) -> I32Vector4: ...
    @property
    def xxoy(self) -> I32Vector4: ...
    @property
    def xxoz(self) -> I32Vector4: ...
    @property
    def xxow(self) -> I32Vector4: ...
    @property
    def xxoo(self) -> I32Vector4: ...
    @property
    def xxol(self) -> I32Vector4: ...
    @property
    def xxlx(self) -> I32Vector4: ...
    @property
    def xxly(self) -> I32Vector4: ...
    @property
    def xxlz(self) -> I32Vector4: ...
    @property
    def xxlw(self) -> I32Vector4: ...
    @property
    def xxlo(self) -> I32Vector4: ...
    @property
    def xxll(self) -> I32Vector4: ...
    @property
    def xyxx(self) -> I32Vector4: ...
    @property
    def xyxy(self) -> I32Vector4: ...
    @property
    def xyxz(self) -> I32Vector4: ...
    @property
    def xyxw(self) -> I32Vector4: ...
    @property
    def xyxo(self) -> I32Vector4: ...
    @property
    def xyxl(self) -> I32Vector4: ...
    @property
    def xyyx(self) -> I32Vector4: ...
    @property
    def xyyy(self) -> I32Vector4: ...
    @property
    def xyyz(self) -> I32Vector4: ...
    @property
    def xyyw(self) -> I32Vector4: ...
    @property
    def xyyo(self) -> I32Vector4: ...
    @property
    def xyyl(self) -> I32Vector4: ...
    @property
    def xyzx(self) -> I32Vector4: ...
    @property
    def xyzy(self) -> I32Vector4: ...
    @property
    def xyzz(self) -> I32Vector4: ...
    @property
    def xyzw(self) -> I32Vector4: ...
    @property
    def xyzo(self) -> I32Vector4: ...
    @property
    def xyzl(self) -> I32Vector4: ...
    @property
    def xywx(self) -> I32Vector4: ...
    @property
    def xywy(self) -> I32Vector4: ...
    @property
    def xywz(self) -> I32Vector4: ...
    @property
    def xyww(self) -> I32Vector4: ...
    @property
    def xywo(self) -> I32Vector4: ...
    @property
    def xywl(self) -> I32Vector4: ...
    @property
    def xyox(self) -> I32Vector4: ...
    @property
    def xyoy(self) -> I32Vector4: ...
    @property
    def xyoz(self) -> I32Vector4: ...
    @property
    def xyow(self) -> I32Vector4: ...
    @property
    def xyoo(self) -> I32Vector4: ...
    @property
    def xyol(self) -> I32Vector4: ...
    @property
    def xylx(self) -> I32Vector4: ...
    @property
    def xyly(self) -> I32Vector4: ...
    @property
    def xylz(self) -> I32Vector4: ...
    @property
    def xylw(self) -> I32Vector4: ...
    @property
    def xylo(self) -> I32Vector4: ...
    @property
    def xyll(self) -> I32Vector4: ...
    @property
    def xzxx(self) -> I32Vector4: ...
    @property
    def xzxy(self) -> I32Vector4: ...
    @property
    def xzxz(self) -> I32Vector4: ...
    @property
    def xzxw(self) -> I32Vector4: ...
    @property
    def xzxo(self) -> I32Vector4: ...
    @property
    def xzxl(self) -> I32Vector4: ...
    @property
    def xzyx(self) -> I32Vector4: ...
    @property
    def xzyy(self) -> I32Vector4: ...
    @property
    def xzyz(self) -> I32Vector4: ...
    @property
    def xzyw(self) -> I32Vector4: ...
    @property
    def xzyo(self) -> I32Vector4: ...
    @property
    def xzyl(self) -> I32Vector4: ...
    @property
    def xzzx(self) -> I32Vector4: ...
    @property
    def xzzy(self) -> I32Vector4: ...
    @property
    def xzzz(self) -> I32Vector4: ...
    @property
    def xzzw(self) -> I32Vector4: ...
    @property
    def xzzo(self) -> I32Vector4: ...
    @property
    def xzzl(self) -> I32Vector4: ...
    @property
    def xzwx(self) -> I32Vector4: ...
    @property
    def xzwy(self) -> I32Vector4: ...
    @property
    def xzwz(self) -> I32Vector4: ...
    @property
    def xzww(self) -> I32Vector4: ...
    @property
    def xzwo(self) -> I32Vector4: ...
    @property
    def xzwl(self) -> I32Vector4: ...
    @property
    def xzox(self) -> I32Vector4: ...
    @property
    def xzoy(self) -> I32Vector4: ...
    @property
    def xzoz(self) -> I32Vector4: ...
    @property
    def xzow(self) -> I32Vector4: ...
    @property
    def xzoo(self) -> I32Vector4: ...
    @property
    def xzol(self) -> I32Vector4: ...
    @property
    def xzlx(self) -> I32Vector4: ...
    @property
    def xzly(self) -> I32Vector4: ...
    @property
    def xzlz(self) -> I32Vector4: ...
    @property
    def xzlw(self) -> I32Vector4: ...
    @property
    def xzlo(self) -> I32Vector4: ...
    @property
    def xzll(self) -> I32Vector4: ...
    @property
    def xwxx(self) -> I32Vector4: ...
    @property
    def xwxy(self) -> I32Vector4: ...
    @property
    def xwxz(self) -> I32Vector4: ...
    @property
    def xwxw(self) -> I32Vector4: ...
    @property
    def xwxo(self) -> I32Vector4: ...
    @property
    def xwxl(self) -> I32Vector4: ...
    @property
    def xwyx(self) -> I32Vector4: ...
    @property
    def xwyy(self) -> I32Vector4: ...
    @property
    def xwyz(self) -> I32Vector4: ...
    @property
    def xwyw(self) -> I32Vector4: ...
    @property
    def xwyo(self) -> I32Vector4: ...
    @property
    def xwyl(self) -> I32Vector4: ...
    @property
    def xwzx(self) -> I32Vector4: ...
    @property
    def xwzy(self) -> I32Vector4: ...
    @property
    def xwzz(self) -> I32Vector4: ...
    @property
    def xwzw(self) -> I32Vector4: ...
    @property
    def xwzo(self) -> I32Vector4: ...
    @property
    def xwzl(self) -> I32Vector4: ...
    @property
    def xwwx(self) -> I32Vector4: ...
    @property
    def xwwy(self) -> I32Vector4: ...
    @property
    def xwwz(self) -> I32Vector4: ...
    @property
    def xwww(self) -> I32Vector4: ...
    @property
    def xwwo(self) -> I32Vector4: ...
    @property
    def xwwl(self) -> I32Vector4: ...
    @property
    def xwox(self) -> I32Vector4: ...
    @property
    def xwoy(self) -> I32Vector4: ...
    @property
    def xwoz(self) -> I32Vector4: ...
    @property
    def xwow(self) -> I32Vector4: ...
    @property
    def xwoo(self) -> I32Vector4: ...
    @property
    def xwol(self) -> I32Vector4: ...
    @property
    def xwlx(self) -> I32Vector4: ...
    @property
    def xwly(self) -> I32Vector4: ...
    @property
    def xwlz(self) -> I32Vector4: ...
    @property
    def xwlw(self) -> I32Vector4: ...
    @property
    def xwlo(self) -> I32Vector4: ...
    @property
    def xwll(self) -> I32Vector4: ...
    @property
    def xoxx(self) -> I32Vector4: ...
    @property
    def xoxy(self) -> I32Vector4: ...
    @property
    def xoxz(self) -> I32Vector4: ...
    @property
    def xoxw(self) -> I32Vector4: ...
    @property
    def xoxo(self) -> I32Vector4: ...
    @property
    def xoxl(self) -> I32Vector4: ...
    @property
    def xoyx(self) -> I32Vector4: ...
    @property
    def xoyy(self) -> I32Vector4: ...
    @property
    def xoyz(self) -> I32Vector4: ...
    @property
    def xoyw(self) -> I32Vector4: ...
    @property
    def xoyo(self) -> I32Vector4: ...
    @property
    def xoyl(self) -> I32Vector4: ...
    @property
    def xozx(self) -> I32Vector4: ...
    @property
    def xozy(self) -> I32Vector4: ...
    @property
    def xozz(self) -> I32Vector4: ...
    @property
    def xozw(self) -> I32Vector4: ...
    @property
    def xozo(self) -> I32Vector4: ...
    @property
    def xozl(self) -> I32Vector4: ...
    @property
    def xowx(self) -> I32Vector4: ...
    @property
    def xowy(self) -> I32Vector4: ...
    @property
    def xowz(self) -> I32Vector4: ...
    @property
    def xoww(self) -> I32Vector4: ...
    @property
    def xowo(self) -> I32Vector4: ...
    @property
    def xowl(self) -> I32Vector4: ...
    @property
    def xoox(self) -> I32Vector4: ...
    @property
    def xooy(self) -> I32Vector4: ...
    @property
    def xooz(self) -> I32Vector4: ...
    @property
    def xoow(self) -> I32Vector4: ...
    @property
    def xooo(self) -> I32Vector4: ...
    @property
    def xool(self) -> I32Vector4: ...
    @property
    def xolx(self) -> I32Vector4: ...
    @property
    def xoly(self) -> I32Vector4: ...
    @property
    def xolz(self) -> I32Vector4: ...
    @property
    def xolw(self) -> I32Vector4: ...
    @property
    def xolo(self) -> I32Vector4: ...
    @property
    def xoll(self) -> I32Vector4: ...
    @property
    def xlxx(self) -> I32Vector4: ...
    @property
    def xlxy(self) -> I32Vector4: ...
    @property
    def xlxz(self) -> I32Vector4: ...
    @property
    def xlxw(self) -> I32Vector4: ...
    @property
    def xlxo(self) -> I32Vector4: ...
    @property
    def xlxl(self) -> I32Vector4: ...
    @property
    def xlyx(self) -> I32Vector4: ...
    @property
    def xlyy(self) -> I32Vector4: ...
    @property
    def xlyz(self) -> I32Vector4: ...
    @property
    def xlyw(self) -> I32Vector4: ...
    @property
    def xlyo(self) -> I32Vector4: ...
    @property
    def xlyl(self) -> I32Vector4: ...
    @property
    def xlzx(self) -> I32Vector4: ...
    @property
    def xlzy(self) -> I32Vector4: ...
    @property
    def xlzz(self) -> I32Vector4: ...
    @property
    def xlzw(self) -> I32Vector4: ...
    @property
    def xlzo(self) -> I32Vector4: ...
    @property
    def xlzl(self) -> I32Vector4: ...
    @property
    def xlwx(self) -> I32Vector4: ...
    @property
    def xlwy(self) -> I32Vector4: ...
    @property
    def xlwz(self) -> I32Vector4: ...
    @property
    def xlww(self) -> I32Vector4: ...
    @property
    def xlwo(self) -> I32Vector4: ...
    @property
    def xlwl(self) -> I32Vector4: ...
    @property
    def xlox(self) -> I32Vector4: ...
    @property
    def xloy(self) -> I32Vector4: ...
    @property
    def xloz(self) -> I32Vector4: ...
    @property
    def xlow(self) -> I32Vector4: ...
    @property
    def xloo(self) -> I32Vector4: ...
    @property
    def xlol(self) -> I32Vector4: ...
    @property
    def xllx(self) -> I32Vector4: ...
    @property
    def xlly(self) -> I32Vector4: ...
    @property
    def xllz(self) -> I32Vector4: ...
    @property
    def xllw(self) -> I32Vector4: ...
    @property
    def xllo(self) -> I32Vector4: ...
    @property
    def xlll(self) -> I32Vector4: ...
    @property
    def yxxx(self) -> I32Vector4: ...
    @property
    def yxxy(self) -> I32Vector4: ...
    @property
    def yxxz(self) -> I32Vector4: ...
    @property
    def yxxw(self) -> I32Vector4: ...
    @property
    def yxxo(self) -> I32Vector4: ...
    @property
    def yxxl(self) -> I32Vector4: ...
    @property
    def yxyx(self) -> I32Vector4: ...
    @property
    def yxyy(self) -> I32Vector4: ...
    @property
    def yxyz(self) -> I32Vector4: ...
    @property
    def yxyw(self) -> I32Vector4: ...
    @property
    def yxyo(self) -> I32Vector4: ...
    @property
    def yxyl(self) -> I32Vector4: ...
    @property
    def yxzx(self) -> I32Vector4: ...
    @property
    def yxzy(self) -> I32Vector4: ...
    @property
    def yxzz(self) -> I32Vector4: ...
    @property
    def yxzw(self) -> I32Vector4: ...
    @property
    def yxzo(self) -> I32Vector4: ...
    @property
    def yxzl(self) -> I32Vector4: ...
    @property
    def yxwx(self) -> I32Vector4: ...
    @property
    def yxwy(self) -> I32Vector4: ...
    @property
    def yxwz(self) -> I32Vector4: ...
    @property
    def yxww(self) -> I32Vector4: ...
    @property
    def yxwo(self) -> I32Vector4: ...
    @property
    def yxwl(self) -> I32Vector4: ...
    @property
    def yxox(self) -> I32Vector4: ...
    @property
    def yxoy(self) -> I32Vector4: ...
    @property
    def yxoz(self) -> I32Vector4: ...
    @property
    def yxow(self) -> I32Vector4: ...
    @property
    def yxoo(self) -> I32Vector4: ...
    @property
    def yxol(self) -> I32Vector4: ...
    @property
    def yxlx(self) -> I32Vector4: ...
    @property
    def yxly(self) -> I32Vector4: ...
    @property
    def yxlz(self) -> I32Vector4: ...
    @property
    def yxlw(self) -> I32Vector4: ...
    @property
    def yxlo(self) -> I32Vector4: ...
    @property
    def yxll(self) -> I32Vector4: ...
    @property
    def yyxx(self) -> I32Vector4: ...
    @property
    def yyxy(self) -> I32Vector4: ...
    @property
    def yyxz(self) -> I32Vector4: ...
    @property
    def yyxw(self) -> I32Vector4: ...
    @property
    def yyxo(self) -> I32Vector4: ...
    @property
    def yyxl(self) -> I32Vector4: ...
    @property
    def yyyx(self) -> I32Vector4: ...
    @property
    def yyyy(self) -> I32Vector4: ...
    @property
    def yyyz(self) -> I32Vector4: ...
    @property
    def yyyw(self) -> I32Vector4: ...
    @property
    def yyyo(self) -> I32Vector4: ...
    @property
    def yyyl(self) -> I32Vector4: ...
    @property
    def yyzx(self) -> I32Vector4: ...
    @property
    def yyzy(self) -> I32Vector4: ...
    @property
    def yyzz(self) -> I32Vector4: ...
    @property
    def yyzw(self) -> I32Vector4: ...
    @property
    def yyzo(self) -> I32Vector4: ...
    @property
    def yyzl(self) -> I32Vector4: ...
    @property
    def yywx(self) -> I32Vector4: ...
    @property
    def yywy(self) -> I32Vector4: ...
    @property
    def yywz(self) -> I32Vector4: ...
    @property
    def yyww(self) -> I32Vector4: ...
    @property
    def yywo(self) -> I32Vector4: ...
    @property
    def yywl(self) -> I32Vector4: ...
    @property
    def yyox(self) -> I32Vector4: ...
    @property
    def yyoy(self) -> I32Vector4: ...
    @property
    def yyoz(self) -> I32Vector4: ...
    @property
    def yyow(self) -> I32Vector4: ...
    @property
    def yyoo(self) -> I32Vector4: ...
    @property
    def yyol(self) -> I32Vector4: ...
    @property
    def yylx(self) -> I32Vector4: ...
    @property
    def yyly(self) -> I32Vector4: ...
    @property
    def yylz(self) -> I32Vector4: ...
    @property
    def yylw(self) -> I32Vector4: ...
    @property
    def yylo(self) -> I32Vector4: ...
    @property
    def yyll(self) -> I32Vector4: ...
    @property
    def yzxx(self) -> I32Vector4: ...
    @property
    def yzxy(self) -> I32Vector4: ...
    @property
    def yzxz(self) -> I32Vector4: ...
    @property
    def yzxw(self) -> I32Vector4: ...
    @property
    def yzxo(self) -> I32Vector4: ...
    @property
    def yzxl(self) -> I32Vector4: ...
    @property
    def yzyx(self) -> I32Vector4: ...
    @property
    def yzyy(self) -> I32Vector4: ...
    @property
    def yzyz(self) -> I32Vector4: ...
    @property
    def yzyw(self) -> I32Vector4: ...
    @property
    def yzyo(self) -> I32Vector4: ...
    @property
    def yzyl(self) -> I32Vector4: ...
    @property
    def yzzx(self) -> I32Vector4: ...
    @property
    def yzzy(self) -> I32Vector4: ...
    @property
    def yzzz(self) -> I32Vector4: ...
    @property
    def yzzw(self) -> I32Vector4: ...
    @property
    def yzzo(self) -> I32Vector4: ...
    @property
    def yzzl(self) -> I32Vector4: ...
    @property
    def yzwx(self) -> I32Vector4: ...
    @property
    def yzwy(self) -> I32Vector4: ...
    @property
    def yzwz(self) -> I32Vector4: ...
    @property
    def yzww(self) -> I32Vector4: ...
    @property
    def yzwo(self) -> I32Vector4: ...
    @property
    def yzwl(self) -> I32Vector4: ...
    @property
    def yzox(self) -> I32Vector4: ...
    @property
    def yzoy(self) -> I32Vector4: ...
    @property
    def yzoz(self) -> I32Vector4: ...
    @property
    def yzow(self) -> I32Vector4: ...
    @property
    def yzoo(self) -> I32Vector4: ...
    @property
    def yzol(self) -> I32Vector4: ...
    @property
    def yzlx(self) -> I32Vector4: ...
    @property
    def yzly(self) -> I32Vector4: ...
    @property
    def yzlz(self) -> I32Vector4: ...
    @property
    def yzlw(self) -> I32Vector4: ...
    @property
    def yzlo(self) -> I32Vector4: ...
    @property
    def yzll(self) -> I32Vector4: ...
    @property
    def ywxx(self) -> I32Vector4: ...
    @property
    def ywxy(self) -> I32Vector4: ...
    @property
    def ywxz(self) -> I32Vector4: ...
    @property
    def ywxw(self) -> I32Vector4: ...
    @property
    def ywxo(self) -> I32Vector4: ...
    @property
    def ywxl(self) -> I32Vector4: ...
    @property
    def ywyx(self) -> I32Vector4: ...
    @property
    def ywyy(self) -> I32Vector4: ...
    @property
    def ywyz(self) -> I32Vector4: ...
    @property
    def ywyw(self) -> I32Vector4: ...
    @property
    def ywyo(self) -> I32Vector4: ...
    @property
    def ywyl(self) -> I32Vector4: ...
    @property
    def ywzx(self) -> I32Vector4: ...
    @property
    def ywzy(self) -> I32Vector4: ...
    @property
    def ywzz(self) -> I32Vector4: ...
    @property
    def ywzw(self) -> I32Vector4: ...
    @property
    def ywzo(self) -> I32Vector4: ...
    @property
    def ywzl(self) -> I32Vector4: ...
    @property
    def ywwx(self) -> I32Vector4: ...
    @property
    def ywwy(self) -> I32Vector4: ...
    @property
    def ywwz(self) -> I32Vector4: ...
    @property
    def ywww(self) -> I32Vector4: ...
    @property
    def ywwo(self) -> I32Vector4: ...
    @property
    def ywwl(self) -> I32Vector4: ...
    @property
    def ywox(self) -> I32Vector4: ...
    @property
    def ywoy(self) -> I32Vector4: ...
    @property
    def ywoz(self) -> I32Vector4: ...
    @property
    def ywow(self) -> I32Vector4: ...
    @property
    def ywoo(self) -> I32Vector4: ...
    @property
    def ywol(self) -> I32Vector4: ...
    @property
    def ywlx(self) -> I32Vector4: ...
    @property
    def ywly(self) -> I32Vector4: ...
    @property
    def ywlz(self) -> I32Vector4: ...
    @property
    def ywlw(self) -> I32Vector4: ...
    @property
    def ywlo(self) -> I32Vector4: ...
    @property
    def ywll(self) -> I32Vector4: ...
    @property
    def yoxx(self) -> I32Vector4: ...
    @property
    def yoxy(self) -> I32Vector4: ...
    @property
    def yoxz(self) -> I32Vector4: ...
    @property
    def yoxw(self) -> I32Vector4: ...
    @property
    def yoxo(self) -> I32Vector4: ...
    @property
    def yoxl(self) -> I32Vector4: ...
    @property
    def yoyx(self) -> I32Vector4: ...
    @property
    def yoyy(self) -> I32Vector4: ...
    @property
    def yoyz(self) -> I32Vector4: ...
    @property
    def yoyw(self) -> I32Vector4: ...
    @property
    def yoyo(self) -> I32Vector4: ...
    @property
    def yoyl(self) -> I32Vector4: ...
    @property
    def yozx(self) -> I32Vector4: ...
    @property
    def yozy(self) -> I32Vector4: ...
    @property
    def yozz(self) -> I32Vector4: ...
    @property
    def yozw(self) -> I32Vector4: ...
    @property
    def yozo(self) -> I32Vector4: ...
    @property
    def yozl(self) -> I32Vector4: ...
    @property
    def yowx(self) -> I32Vector4: ...
    @property
    def yowy(self) -> I32Vector4: ...
    @property
    def yowz(self) -> I32Vector4: ...
    @property
    def yoww(self) -> I32Vector4: ...
    @property
    def yowo(self) -> I32Vector4: ...
    @property
    def yowl(self) -> I32Vector4: ...
    @property
    def yoox(self) -> I32Vector4: ...
    @property
    def yooy(self) -> I32Vector4: ...
    @property
    def yooz(self) -> I32Vector4: ...
    @property
    def yoow(self) -> I32Vector4: ...
    @property
    def yooo(self) -> I32Vector4: ...
    @property
    def yool(self) -> I32Vector4: ...
    @property
    def yolx(self) -> I32Vector4: ...
    @property
    def yoly(self) -> I32Vector4: ...
    @property
    def yolz(self) -> I32Vector4: ...
    @property
    def yolw(self) -> I32Vector4: ...
    @property
    def yolo(self) -> I32Vector4: ...
    @property
    def yoll(self) -> I32Vector4: ...
    @property
    def ylxx(self) -> I32Vector4: ...
    @property
    def ylxy(self) -> I32Vector4: ...
    @property
    def ylxz(self) -> I32Vector4: ...
    @property
    def ylxw(self) -> I32Vector4: ...
    @property
    def ylxo(self) -> I32Vector4: ...
    @property
    def ylxl(self) -> I32Vector4: ...
    @property
    def ylyx(self) -> I32Vector4: ...
    @property
    def ylyy(self) -> I32Vector4: ...
    @property
    def ylyz(self) -> I32Vector4: ...
    @property
    def ylyw(self) -> I32Vector4: ...
    @property
    def ylyo(self) -> I32Vector4: ...
    @property
    def ylyl(self) -> I32Vector4: ...
    @property
    def ylzx(self) -> I32Vector4: ...
    @property
    def ylzy(self) -> I32Vector4: ...
    @property
    def ylzz(self) -> I32Vector4: ...
    @property
    def ylzw(self) -> I32Vector4: ...
    @property
    def ylzo(self) -> I32Vector4: ...
    @property
    def ylzl(self) -> I32Vector4: ...
    @property
    def ylwx(self) -> I32Vector4: ...
    @property
    def ylwy(self) -> I32Vector4: ...
    @property
    def ylwz(self) -> I32Vector4: ...
    @property
    def ylww(self) -> I32Vector4: ...
    @property
    def ylwo(self) -> I32Vector4: ...
    @property
    def ylwl(self) -> I32Vector4: ...
    @property
    def ylox(self) -> I32Vector4: ...
    @property
    def yloy(self) -> I32Vector4: ...
    @property
    def yloz(self) -> I32Vector4: ...
    @property
    def ylow(self) -> I32Vector4: ...
    @property
    def yloo(self) -> I32Vector4: ...
    @property
    def ylol(self) -> I32Vector4: ...
    @property
    def yllx(self) -> I32Vector4: ...
    @property
    def ylly(self) -> I32Vector4: ...
    @property
    def yllz(self) -> I32Vector4: ...
    @property
    def yllw(self) -> I32Vector4: ...
    @property
    def yllo(self) -> I32Vector4: ...
    @property
    def ylll(self) -> I32Vector4: ...
    @property
    def zxxx(self) -> I32Vector4: ...
    @property
    def zxxy(self) -> I32Vector4: ...
    @property
    def zxxz(self) -> I32Vector4: ...
    @property
    def zxxw(self) -> I32Vector4: ...
    @property
    def zxxo(self) -> I32Vector4: ...
    @property
    def zxxl(self) -> I32Vector4: ...
    @property
    def zxyx(self) -> I32Vector4: ...
    @property
    def zxyy(self) -> I32Vector4: ...
    @property
    def zxyz(self) -> I32Vector4: ...
    @property
    def zxyw(self) -> I32Vector4: ...
    @property
    def zxyo(self) -> I32Vector4: ...
    @property
    def zxyl(self) -> I32Vector4: ...
    @property
    def zxzx(self) -> I32Vector4: ...
    @property
    def zxzy(self) -> I32Vector4: ...
    @property
    def zxzz(self) -> I32Vector4: ...
    @property
    def zxzw(self) -> I32Vector4: ...
    @property
    def zxzo(self) -> I32Vector4: ...
    @property
    def zxzl(self) -> I32Vector4: ...
    @property
    def zxwx(self) -> I32Vector4: ...
    @property
    def zxwy(self) -> I32Vector4: ...
    @property
    def zxwz(self) -> I32Vector4: ...
    @property
    def zxww(self) -> I32Vector4: ...
    @property
    def zxwo(self) -> I32Vector4: ...
    @property
    def zxwl(self) -> I32Vector4: ...
    @property
    def zxox(self) -> I32Vector4: ...
    @property
    def zxoy(self) -> I32Vector4: ...
    @property
    def zxoz(self) -> I32Vector4: ...
    @property
    def zxow(self) -> I32Vector4: ...
    @property
    def zxoo(self) -> I32Vector4: ...
    @property
    def zxol(self) -> I32Vector4: ...
    @property
    def zxlx(self) -> I32Vector4: ...
    @property
    def zxly(self) -> I32Vector4: ...
    @property
    def zxlz(self) -> I32Vector4: ...
    @property
    def zxlw(self) -> I32Vector4: ...
    @property
    def zxlo(self) -> I32Vector4: ...
    @property
    def zxll(self) -> I32Vector4: ...
    @property
    def zyxx(self) -> I32Vector4: ...
    @property
    def zyxy(self) -> I32Vector4: ...
    @property
    def zyxz(self) -> I32Vector4: ...
    @property
    def zyxw(self) -> I32Vector4: ...
    @property
    def zyxo(self) -> I32Vector4: ...
    @property
    def zyxl(self) -> I32Vector4: ...
    @property
    def zyyx(self) -> I32Vector4: ...
    @property
    def zyyy(self) -> I32Vector4: ...
    @property
    def zyyz(self) -> I32Vector4: ...
    @property
    def zyyw(self) -> I32Vector4: ...
    @property
    def zyyo(self) -> I32Vector4: ...
    @property
    def zyyl(self) -> I32Vector4: ...
    @property
    def zyzx(self) -> I32Vector4: ...
    @property
    def zyzy(self) -> I32Vector4: ...
    @property
    def zyzz(self) -> I32Vector4: ...
    @property
    def zyzw(self) -> I32Vector4: ...
    @property
    def zyzo(self) -> I32Vector4: ...
    @property
    def zyzl(self) -> I32Vector4: ...
    @property
    def zywx(self) -> I32Vector4: ...
    @property
    def zywy(self) -> I32Vector4: ...
    @property
    def zywz(self) -> I32Vector4: ...
    @property
    def zyww(self) -> I32Vector4: ...
    @property
    def zywo(self) -> I32Vector4: ...
    @property
    def zywl(self) -> I32Vector4: ...
    @property
    def zyox(self) -> I32Vector4: ...
    @property
    def zyoy(self) -> I32Vector4: ...
    @property
    def zyoz(self) -> I32Vector4: ...
    @property
    def zyow(self) -> I32Vector4: ...
    @property
    def zyoo(self) -> I32Vector4: ...
    @property
    def zyol(self) -> I32Vector4: ...
    @property
    def zylx(self) -> I32Vector4: ...
    @property
    def zyly(self) -> I32Vector4: ...
    @property
    def zylz(self) -> I32Vector4: ...
    @property
    def zylw(self) -> I32Vector4: ...
    @property
    def zylo(self) -> I32Vector4: ...
    @property
    def zyll(self) -> I32Vector4: ...
    @property
    def zzxx(self) -> I32Vector4: ...
    @property
    def zzxy(self) -> I32Vector4: ...
    @property
    def zzxz(self) -> I32Vector4: ...
    @property
    def zzxw(self) -> I32Vector4: ...
    @property
    def zzxo(self) -> I32Vector4: ...
    @property
    def zzxl(self) -> I32Vector4: ...
    @property
    def zzyx(self) -> I32Vector4: ...
    @property
    def zzyy(self) -> I32Vector4: ...
    @property
    def zzyz(self) -> I32Vector4: ...
    @property
    def zzyw(self) -> I32Vector4: ...
    @property
    def zzyo(self) -> I32Vector4: ...
    @property
    def zzyl(self) -> I32Vector4: ...
    @property
    def zzzx(self) -> I32Vector4: ...
    @property
    def zzzy(self) -> I32Vector4: ...
    @property
    def zzzz(self) -> I32Vector4: ...
    @property
    def zzzw(self) -> I32Vector4: ...
    @property
    def zzzo(self) -> I32Vector4: ...
    @property
    def zzzl(self) -> I32Vector4: ...
    @property
    def zzwx(self) -> I32Vector4: ...
    @property
    def zzwy(self) -> I32Vector4: ...
    @property
    def zzwz(self) -> I32Vector4: ...
    @property
    def zzww(self) -> I32Vector4: ...
    @property
    def zzwo(self) -> I32Vector4: ...
    @property
    def zzwl(self) -> I32Vector4: ...
    @property
    def zzox(self) -> I32Vector4: ...
    @property
    def zzoy(self) -> I32Vector4: ...
    @property
    def zzoz(self) -> I32Vector4: ...
    @property
    def zzow(self) -> I32Vector4: ...
    @property
    def zzoo(self) -> I32Vector4: ...
    @property
    def zzol(self) -> I32Vector4: ...
    @property
    def zzlx(self) -> I32Vector4: ...
    @property
    def zzly(self) -> I32Vector4: ...
    @property
    def zzlz(self) -> I32Vector4: ...
    @property
    def zzlw(self) -> I32Vector4: ...
    @property
    def zzlo(self) -> I32Vector4: ...
    @property
    def zzll(self) -> I32Vector4: ...
    @property
    def zwxx(self) -> I32Vector4: ...
    @property
    def zwxy(self) -> I32Vector4: ...
    @property
    def zwxz(self) -> I32Vector4: ...
    @property
    def zwxw(self) -> I32Vector4: ...
    @property
    def zwxo(self) -> I32Vector4: ...
    @property
    def zwxl(self) -> I32Vector4: ...
    @property
    def zwyx(self) -> I32Vector4: ...
    @property
    def zwyy(self) -> I32Vector4: ...
    @property
    def zwyz(self) -> I32Vector4: ...
    @property
    def zwyw(self) -> I32Vector4: ...
    @property
    def zwyo(self) -> I32Vector4: ...
    @property
    def zwyl(self) -> I32Vector4: ...
    @property
    def zwzx(self) -> I32Vector4: ...
    @property
    def zwzy(self) -> I32Vector4: ...
    @property
    def zwzz(self) -> I32Vector4: ...
    @property
    def zwzw(self) -> I32Vector4: ...
    @property
    def zwzo(self) -> I32Vector4: ...
    @property
    def zwzl(self) -> I32Vector4: ...
    @property
    def zwwx(self) -> I32Vector4: ...
    @property
    def zwwy(self) -> I32Vector4: ...
    @property
    def zwwz(self) -> I32Vector4: ...
    @property
    def zwww(self) -> I32Vector4: ...
    @property
    def zwwo(self) -> I32Vector4: ...
    @property
    def zwwl(self) -> I32Vector4: ...
    @property
    def zwox(self) -> I32Vector4: ...
    @property
    def zwoy(self) -> I32Vector4: ...
    @property
    def zwoz(self) -> I32Vector4: ...
    @property
    def zwow(self) -> I32Vector4: ...
    @property
    def zwoo(self) -> I32Vector4: ...
    @property
    def zwol(self) -> I32Vector4: ...
    @property
    def zwlx(self) -> I32Vector4: ...
    @property
    def zwly(self) -> I32Vector4: ...
    @property
    def zwlz(self) -> I32Vector4: ...
    @property
    def zwlw(self) -> I32Vector4: ...
    @property
    def zwlo(self) -> I32Vector4: ...
    @property
    def zwll(self) -> I32Vector4: ...
    @property
    def zoxx(self) -> I32Vector4: ...
    @property
    def zoxy(self) -> I32Vector4: ...
    @property
    def zoxz(self) -> I32Vector4: ...
    @property
    def zoxw(self) -> I32Vector4: ...
    @property
    def zoxo(self) -> I32Vector4: ...
    @property
    def zoxl(self) -> I32Vector4: ...
    @property
    def zoyx(self) -> I32Vector4: ...
    @property
    def zoyy(self) -> I32Vector4: ...
    @property
    def zoyz(self) -> I32Vector4: ...
    @property
    def zoyw(self) -> I32Vector4: ...
    @property
    def zoyo(self) -> I32Vector4: ...
    @property
    def zoyl(self) -> I32Vector4: ...
    @property
    def zozx(self) -> I32Vector4: ...
    @property
    def zozy(self) -> I32Vector4: ...
    @property
    def zozz(self) -> I32Vector4: ...
    @property
    def zozw(self) -> I32Vector4: ...
    @property
    def zozo(self) -> I32Vector4: ...
    @property
    def zozl(self) -> I32Vector4: ...
    @property
    def zowx(self) -> I32Vector4: ...
    @property
    def zowy(self) -> I32Vector4: ...
    @property
    def zowz(self) -> I32Vector4: ...
    @property
    def zoww(self) -> I32Vector4: ...
    @property
    def zowo(self) -> I32Vector4: ...
    @property
    def zowl(self) -> I32Vector4: ...
    @property
    def zoox(self) -> I32Vector4: ...
    @property
    def zooy(self) -> I32Vector4: ...
    @property
    def zooz(self) -> I32Vector4: ...
    @property
    def zoow(self) -> I32Vector4: ...
    @property
    def zooo(self) -> I32Vector4: ...
    @property
    def zool(self) -> I32Vector4: ...
    @property
    def zolx(self) -> I32Vector4: ...
    @property
    def zoly(self) -> I32Vector4: ...
    @property
    def zolz(self) -> I32Vector4: ...
    @property
    def zolw(self) -> I32Vector4: ...
    @property
    def zolo(self) -> I32Vector4: ...
    @property
    def zoll(self) -> I32Vector4: ...
    @property
    def zlxx(self) -> I32Vector4: ...
    @property
    def zlxy(self) -> I32Vector4: ...
    @property
    def zlxz(self) -> I32Vector4: ...
    @property
    def zlxw(self) -> I32Vector4: ...
    @property
    def zlxo(self) -> I32Vector4: ...
    @property
    def zlxl(self) -> I32Vector4: ...
    @property
    def zlyx(self) -> I32Vector4: ...
    @property
    def zlyy(self) -> I32Vector4: ...
    @property
    def zlyz(self) -> I32Vector4: ...
    @property
    def zlyw(self) -> I32Vector4: ...
    @property
    def zlyo(self) -> I32Vector4: ...
    @property
    def zlyl(self) -> I32Vector4: ...
    @property
    def zlzx(self) -> I32Vector4: ...
    @property
    def zlzy(self) -> I32Vector4: ...
    @property
    def zlzz(self) -> I32Vector4: ...
    @property
    def zlzw(self) -> I32Vector4: ...
    @property
    def zlzo(self) -> I32Vector4: ...
    @property
    def zlzl(self) -> I32Vector4: ...
    @property
    def zlwx(self) -> I32Vector4: ...
    @property
    def zlwy(self) -> I32Vector4: ...
    @property
    def zlwz(self) -> I32Vector4: ...
    @property
    def zlww(self) -> I32Vector4: ...
    @property
    def zlwo(self) -> I32Vector4: ...
    @property
    def zlwl(self) -> I32Vector4: ...
    @property
    def zlox(self) -> I32Vector4: ...
    @property
    def zloy(self) -> I32Vector4: ...
    @property
    def zloz(self) -> I32Vector4: ...
    @property
    def zlow(self) -> I32Vector4: ...
    @property
    def zloo(self) -> I32Vector4: ...
    @property
    def zlol(self) -> I32Vector4: ...
    @property
    def zllx(self) -> I32Vector4: ...
    @property
    def zlly(self) -> I32Vector4: ...
    @property
    def zllz(self) -> I32Vector4: ...
    @property
    def zllw(self) -> I32Vector4: ...
    @property
    def zllo(self) -> I32Vector4: ...
    @property
    def zlll(self) -> I32Vector4: ...
    @property
    def wxxx(self) -> I32Vector4: ...
    @property
    def wxxy(self) -> I32Vector4: ...
    @property
    def wxxz(self) -> I32Vector4: ...
    @property
    def wxxw(self) -> I32Vector4: ...
    @property
    def wxxo(self) -> I32Vector4: ...
    @property
    def wxxl(self) -> I32Vector4: ...
    @property
    def wxyx(self) -> I32Vector4: ...
    @property
    def wxyy(self) -> I32Vector4: ...
    @property
    def wxyz(self) -> I32Vector4: ...
    @property
    def wxyw(self) -> I32Vector4: ...
    @property
    def wxyo(self) -> I32Vector4: ...
    @property
    def wxyl(self) -> I32Vector4: ...
    @property
    def wxzx(self) -> I32Vector4: ...
    @property
    def wxzy(self) -> I32Vector4: ...
    @property
    def wxzz(self) -> I32Vector4: ...
    @property
    def wxzw(self) -> I32Vector4: ...
    @property
    def wxzo(self) -> I32Vector4: ...
    @property
    def wxzl(self) -> I32Vector4: ...
    @property
    def wxwx(self) -> I32Vector4: ...
    @property
    def wxwy(self) -> I32Vector4: ...
    @property
    def wxwz(self) -> I32Vector4: ...
    @property
    def wxww(self) -> I32Vector4: ...
    @property
    def wxwo(self) -> I32Vector4: ...
    @property
    def wxwl(self) -> I32Vector4: ...
    @property
    def wxox(self) -> I32Vector4: ...
    @property
    def wxoy(self) -> I32Vector4: ...
    @property
    def wxoz(self) -> I32Vector4: ...
    @property
    def wxow(self) -> I32Vector4: ...
    @property
    def wxoo(self) -> I32Vector4: ...
    @property
    def wxol(self) -> I32Vector4: ...
    @property
    def wxlx(self) -> I32Vector4: ...
    @property
    def wxly(self) -> I32Vector4: ...
    @property
    def wxlz(self) -> I32Vector4: ...
    @property
    def wxlw(self) -> I32Vector4: ...
    @property
    def wxlo(self) -> I32Vector4: ...
    @property
    def wxll(self) -> I32Vector4: ...
    @property
    def wyxx(self) -> I32Vector4: ...
    @property
    def wyxy(self) -> I32Vector4: ...
    @property
    def wyxz(self) -> I32Vector4: ...
    @property
    def wyxw(self) -> I32Vector4: ...
    @property
    def wyxo(self) -> I32Vector4: ...
    @property
    def wyxl(self) -> I32Vector4: ...
    @property
    def wyyx(self) -> I32Vector4: ...
    @property
    def wyyy(self) -> I32Vector4: ...
    @property
    def wyyz(self) -> I32Vector4: ...
    @property
    def wyyw(self) -> I32Vector4: ...
    @property
    def wyyo(self) -> I32Vector4: ...
    @property
    def wyyl(self) -> I32Vector4: ...
    @property
    def wyzx(self) -> I32Vector4: ...
    @property
    def wyzy(self) -> I32Vector4: ...
    @property
    def wyzz(self) -> I32Vector4: ...
    @property
    def wyzw(self) -> I32Vector4: ...
    @property
    def wyzo(self) -> I32Vector4: ...
    @property
    def wyzl(self) -> I32Vector4: ...
    @property
    def wywx(self) -> I32Vector4: ...
    @property
    def wywy(self) -> I32Vector4: ...
    @property
    def wywz(self) -> I32Vector4: ...
    @property
    def wyww(self) -> I32Vector4: ...
    @property
    def wywo(self) -> I32Vector4: ...
    @property
    def wywl(self) -> I32Vector4: ...
    @property
    def wyox(self) -> I32Vector4: ...
    @property
    def wyoy(self) -> I32Vector4: ...
    @property
    def wyoz(self) -> I32Vector4: ...
    @property
    def wyow(self) -> I32Vector4: ...
    @property
    def wyoo(self) -> I32Vector4: ...
    @property
    def wyol(self) -> I32Vector4: ...
    @property
    def wylx(self) -> I32Vector4: ...
    @property
    def wyly(self) -> I32Vector4: ...
    @property
    def wylz(self) -> I32Vector4: ...
    @property
    def wylw(self) -> I32Vector4: ...
    @property
    def wylo(self) -> I32Vector4: ...
    @property
    def wyll(self) -> I32Vector4: ...
    @property
    def wzxx(self) -> I32Vector4: ...
    @property
    def wzxy(self) -> I32Vector4: ...
    @property
    def wzxz(self) -> I32Vector4: ...
    @property
    def wzxw(self) -> I32Vector4: ...
    @property
    def wzxo(self) -> I32Vector4: ...
    @property
    def wzxl(self) -> I32Vector4: ...
    @property
    def wzyx(self) -> I32Vector4: ...
    @property
    def wzyy(self) -> I32Vector4: ...
    @property
    def wzyz(self) -> I32Vector4: ...
    @property
    def wzyw(self) -> I32Vector4: ...
    @property
    def wzyo(self) -> I32Vector4: ...
    @property
    def wzyl(self) -> I32Vector4: ...
    @property
    def wzzx(self) -> I32Vector4: ...
    @property
    def wzzy(self) -> I32Vector4: ...
    @property
    def wzzz(self) -> I32Vector4: ...
    @property
    def wzzw(self) -> I32Vector4: ...
    @property
    def wzzo(self) -> I32Vector4: ...
    @property
    def wzzl(self) -> I32Vector4: ...
    @property
    def wzwx(self) -> I32Vector4: ...
    @property
    def wzwy(self) -> I32Vector4: ...
    @property
    def wzwz(self) -> I32Vector4: ...
    @property
    def wzww(self) -> I32Vector4: ...
    @property
    def wzwo(self) -> I32Vector4: ...
    @property
    def wzwl(self) -> I32Vector4: ...
    @property
    def wzox(self) -> I32Vector4: ...
    @property
    def wzoy(self) -> I32Vector4: ...
    @property
    def wzoz(self) -> I32Vector4: ...
    @property
    def wzow(self) -> I32Vector4: ...
    @property
    def wzoo(self) -> I32Vector4: ...
    @property
    def wzol(self) -> I32Vector4: ...
    @property
    def wzlx(self) -> I32Vector4: ...
    @property
    def wzly(self) -> I32Vector4: ...
    @property
    def wzlz(self) -> I32Vector4: ...
    @property
    def wzlw(self) -> I32Vector4: ...
    @property
    def wzlo(self) -> I32Vector4: ...
    @property
    def wzll(self) -> I32Vector4: ...
    @property
    def wwxx(self) -> I32Vector4: ...
    @property
    def wwxy(self) -> I32Vector4: ...
    @property
    def wwxz(self) -> I32Vector4: ...
    @property
    def wwxw(self) -> I32Vector4: ...
    @property
    def wwxo(self) -> I32Vector4: ...
    @property
    def wwxl(self) -> I32Vector4: ...
    @property
    def wwyx(self) -> I32Vector4: ...
    @property
    def wwyy(self) -> I32Vector4: ...
    @property
    def wwyz(self) -> I32Vector4: ...
    @property
    def wwyw(self) -> I32Vector4: ...
    @property
    def wwyo(self) -> I32Vector4: ...
    @property
    def wwyl(self) -> I32Vector4: ...
    @property
    def wwzx(self) -> I32Vector4: ...
    @property
    def wwzy(self) -> I32Vector4: ...
    @property
    def wwzz(self) -> I32Vector4: ...
    @property
    def wwzw(self) -> I32Vector4: ...
    @property
    def wwzo(self) -> I32Vector4: ...
    @property
    def wwzl(self) -> I32Vector4: ...
    @property
    def wwwx(self) -> I32Vector4: ...
    @property
    def wwwy(self) -> I32Vector4: ...
    @property
    def wwwz(self) -> I32Vector4: ...
    @property
    def wwww(self) -> I32Vector4: ...
    @property
    def wwwo(self) -> I32Vector4: ...
    @property
    def wwwl(self) -> I32Vector4: ...
    @property
    def wwox(self) -> I32Vector4: ...
    @property
    def wwoy(self) -> I32Vector4: ...
    @property
    def wwoz(self) -> I32Vector4: ...
    @property
    def wwow(self) -> I32Vector4: ...
    @property
    def wwoo(self) -> I32Vector4: ...
    @property
    def wwol(self) -> I32Vector4: ...
    @property
    def wwlx(self) -> I32Vector4: ...
    @property
    def wwly(self) -> I32Vector4: ...
    @property
    def wwlz(self) -> I32Vector4: ...
    @property
    def wwlw(self) -> I32Vector4: ...
    @property
    def wwlo(self) -> I32Vector4: ...
    @property
    def wwll(self) -> I32Vector4: ...
    @property
    def woxx(self) -> I32Vector4: ...
    @property
    def woxy(self) -> I32Vector4: ...
    @property
    def woxz(self) -> I32Vector4: ...
    @property
    def woxw(self) -> I32Vector4: ...
    @property
    def woxo(self) -> I32Vector4: ...
    @property
    def woxl(self) -> I32Vector4: ...
    @property
    def woyx(self) -> I32Vector4: ...
    @property
    def woyy(self) -> I32Vector4: ...
    @property
    def woyz(self) -> I32Vector4: ...
    @property
    def woyw(self) -> I32Vector4: ...
    @property
    def woyo(self) -> I32Vector4: ...
    @property
    def woyl(self) -> I32Vector4: ...
    @property
    def wozx(self) -> I32Vector4: ...
    @property
    def wozy(self) -> I32Vector4: ...
    @property
    def wozz(self) -> I32Vector4: ...
    @property
    def wozw(self) -> I32Vector4: ...
    @property
    def wozo(self) -> I32Vector4: ...
    @property
    def wozl(self) -> I32Vector4: ...
    @property
    def wowx(self) -> I32Vector4: ...
    @property
    def wowy(self) -> I32Vector4: ...
    @property
    def wowz(self) -> I32Vector4: ...
    @property
    def woww(self) -> I32Vector4: ...
    @property
    def wowo(self) -> I32Vector4: ...
    @property
    def wowl(self) -> I32Vector4: ...
    @property
    def woox(self) -> I32Vector4: ...
    @property
    def wooy(self) -> I32Vector4: ...
    @property
    def wooz(self) -> I32Vector4: ...
    @property
    def woow(self) -> I32Vector4: ...
    @property
    def wooo(self) -> I32Vector4: ...
    @property
    def wool(self) -> I32Vector4: ...
    @property
    def wolx(self) -> I32Vector4: ...
    @property
    def woly(self) -> I32Vector4: ...
    @property
    def wolz(self) -> I32Vector4: ...
    @property
    def wolw(self) -> I32Vector4: ...
    @property
    def wolo(self) -> I32Vector4: ...
    @property
    def woll(self) -> I32Vector4: ...
    @property
    def wlxx(self) -> I32Vector4: ...
    @property
    def wlxy(self) -> I32Vector4: ...
    @property
    def wlxz(self) -> I32Vector4: ...
    @property
    def wlxw(self) -> I32Vector4: ...
    @property
    def wlxo(self) -> I32Vector4: ...
    @property
    def wlxl(self) -> I32Vector4: ...
    @property
    def wlyx(self) -> I32Vector4: ...
    @property
    def wlyy(self) -> I32Vector4: ...
    @property
    def wlyz(self) -> I32Vector4: ...
    @property
    def wlyw(self) -> I32Vector4: ...
    @property
    def wlyo(self) -> I32Vector4: ...
    @property
    def wlyl(self) -> I32Vector4: ...
    @property
    def wlzx(self) -> I32Vector4: ...
    @property
    def wlzy(self) -> I32Vector4: ...
    @property
    def wlzz(self) -> I32Vector4: ...
    @property
    def wlzw(self) -> I32Vector4: ...
    @property
    def wlzo(self) -> I32Vector4: ...
    @property
    def wlzl(self) -> I32Vector4: ...
    @property
    def wlwx(self) -> I32Vector4: ...
    @property
    def wlwy(self) -> I32Vector4: ...
    @property
    def wlwz(self) -> I32Vector4: ...
    @property
    def wlww(self) -> I32Vector4: ...
    @property
    def wlwo(self) -> I32Vector4: ...
    @property
    def wlwl(self) -> I32Vector4: ...
    @property
    def wlox(self) -> I32Vector4: ...
    @property
    def wloy(self) -> I32Vector4: ...
    @property
    def wloz(self) -> I32Vector4: ...
    @property
    def wlow(self) -> I32Vector4: ...
    @property
    def wloo(self) -> I32Vector4: ...
    @property
    def wlol(self) -> I32Vector4: ...
    @property
    def wllx(self) -> I32Vector4: ...
    @property
    def wlly(self) -> I32Vector4: ...
    @property
    def wllz(self) -> I32Vector4: ...
    @property
    def wllw(self) -> I32Vector4: ...
    @property
    def wllo(self) -> I32Vector4: ...
    @property
    def wlll(self) -> I32Vector4: ...
    @property
    def oxxx(self) -> I32Vector4: ...
    @property
    def oxxy(self) -> I32Vector4: ...
    @property
    def oxxz(self) -> I32Vector4: ...
    @property
    def oxxw(self) -> I32Vector4: ...
    @property
    def oxxo(self) -> I32Vector4: ...
    @property
    def oxxl(self) -> I32Vector4: ...
    @property
    def oxyx(self) -> I32Vector4: ...
    @property
    def oxyy(self) -> I32Vector4: ...
    @property
    def oxyz(self) -> I32Vector4: ...
    @property
    def oxyw(self) -> I32Vector4: ...
    @property
    def oxyo(self) -> I32Vector4: ...
    @property
    def oxyl(self) -> I32Vector4: ...
    @property
    def oxzx(self) -> I32Vector4: ...
    @property
    def oxzy(self) -> I32Vector4: ...
    @property
    def oxzz(self) -> I32Vector4: ...
    @property
    def oxzw(self) -> I32Vector4: ...
    @property
    def oxzo(self) -> I32Vector4: ...
    @property
    def oxzl(self) -> I32Vector4: ...
    @property
    def oxwx(self) -> I32Vector4: ...
    @property
    def oxwy(self) -> I32Vector4: ...
    @property
    def oxwz(self) -> I32Vector4: ...
    @property
    def oxww(self) -> I32Vector4: ...
    @property
    def oxwo(self) -> I32Vector4: ...
    @property
    def oxwl(self) -> I32Vector4: ...
    @property
    def oxox(self) -> I32Vector4: ...
    @property
    def oxoy(self) -> I32Vector4: ...
    @property
    def oxoz(self) -> I32Vector4: ...
    @property
    def oxow(self) -> I32Vector4: ...
    @property
    def oxoo(self) -> I32Vector4: ...
    @property
    def oxol(self) -> I32Vector4: ...
    @property
    def oxlx(self) -> I32Vector4: ...
    @property
    def oxly(self) -> I32Vector4: ...
    @property
    def oxlz(self) -> I32Vector4: ...
    @property
    def oxlw(self) -> I32Vector4: ...
    @property
    def oxlo(self) -> I32Vector4: ...
    @property
    def oxll(self) -> I32Vector4: ...
    @property
    def oyxx(self) -> I32Vector4: ...
    @property
    def oyxy(self) -> I32Vector4: ...
    @property
    def oyxz(self) -> I32Vector4: ...
    @property
    def oyxw(self) -> I32Vector4: ...
    @property
    def oyxo(self) -> I32Vector4: ...
    @property
    def oyxl(self) -> I32Vector4: ...
    @property
    def oyyx(self) -> I32Vector4: ...
    @property
    def oyyy(self) -> I32Vector4: ...
    @property
    def oyyz(self) -> I32Vector4: ...
    @property
    def oyyw(self) -> I32Vector4: ...
    @property
    def oyyo(self) -> I32Vector4: ...
    @property
    def oyyl(self) -> I32Vector4: ...
    @property
    def oyzx(self) -> I32Vector4: ...
    @property
    def oyzy(self) -> I32Vector4: ...
    @property
    def oyzz(self) -> I32Vector4: ...
    @property
    def oyzw(self) -> I32Vector4: ...
    @property
    def oyzo(self) -> I32Vector4: ...
    @property
    def oyzl(self) -> I32Vector4: ...
    @property
    def oywx(self) -> I32Vector4: ...
    @property
    def oywy(self) -> I32Vector4: ...
    @property
    def oywz(self) -> I32Vector4: ...
    @property
    def oyww(self) -> I32Vector4: ...
    @property
    def oywo(self) -> I32Vector4: ...
    @property
    def oywl(self) -> I32Vector4: ...
    @property
    def oyox(self) -> I32Vector4: ...
    @property
    def oyoy(self) -> I32Vector4: ...
    @property
    def oyoz(self) -> I32Vector4: ...
    @property
    def oyow(self) -> I32Vector4: ...
    @property
    def oyoo(self) -> I32Vector4: ...
    @property
    def oyol(self) -> I32Vector4: ...
    @property
    def oylx(self) -> I32Vector4: ...
    @property
    def oyly(self) -> I32Vector4: ...
    @property
    def oylz(self) -> I32Vector4: ...
    @property
    def oylw(self) -> I32Vector4: ...
    @property
    def oylo(self) -> I32Vector4: ...
    @property
    def oyll(self) -> I32Vector4: ...
    @property
    def ozxx(self) -> I32Vector4: ...
    @property
    def ozxy(self) -> I32Vector4: ...
    @property
    def ozxz(self) -> I32Vector4: ...
    @property
    def ozxw(self) -> I32Vector4: ...
    @property
    def ozxo(self) -> I32Vector4: ...
    @property
    def ozxl(self) -> I32Vector4: ...
    @property
    def ozyx(self) -> I32Vector4: ...
    @property
    def ozyy(self) -> I32Vector4: ...
    @property
    def ozyz(self) -> I32Vector4: ...
    @property
    def ozyw(self) -> I32Vector4: ...
    @property
    def ozyo(self) -> I32Vector4: ...
    @property
    def ozyl(self) -> I32Vector4: ...
    @property
    def ozzx(self) -> I32Vector4: ...
    @property
    def ozzy(self) -> I32Vector4: ...
    @property
    def ozzz(self) -> I32Vector4: ...
    @property
    def ozzw(self) -> I32Vector4: ...
    @property
    def ozzo(self) -> I32Vector4: ...
    @property
    def ozzl(self) -> I32Vector4: ...
    @property
    def ozwx(self) -> I32Vector4: ...
    @property
    def ozwy(self) -> I32Vector4: ...
    @property
    def ozwz(self) -> I32Vector4: ...
    @property
    def ozww(self) -> I32Vector4: ...
    @property
    def ozwo(self) -> I32Vector4: ...
    @property
    def ozwl(self) -> I32Vector4: ...
    @property
    def ozox(self) -> I32Vector4: ...
    @property
    def ozoy(self) -> I32Vector4: ...
    @property
    def ozoz(self) -> I32Vector4: ...
    @property
    def ozow(self) -> I32Vector4: ...
    @property
    def ozoo(self) -> I32Vector4: ...
    @property
    def ozol(self) -> I32Vector4: ...
    @property
    def ozlx(self) -> I32Vector4: ...
    @property
    def ozly(self) -> I32Vector4: ...
    @property
    def ozlz(self) -> I32Vector4: ...
    @property
    def ozlw(self) -> I32Vector4: ...
    @property
    def ozlo(self) -> I32Vector4: ...
    @property
    def ozll(self) -> I32Vector4: ...
    @property
    def owxx(self) -> I32Vector4: ...
    @property
    def owxy(self) -> I32Vector4: ...
    @property
    def owxz(self) -> I32Vector4: ...
    @property
    def owxw(self) -> I32Vector4: ...
    @property
    def owxo(self) -> I32Vector4: ...
    @property
    def owxl(self) -> I32Vector4: ...
    @property
    def owyx(self) -> I32Vector4: ...
    @property
    def owyy(self) -> I32Vector4: ...
    @property
    def owyz(self) -> I32Vector4: ...
    @property
    def owyw(self) -> I32Vector4: ...
    @property
    def owyo(self) -> I32Vector4: ...
    @property
    def owyl(self) -> I32Vector4: ...
    @property
    def owzx(self) -> I32Vector4: ...
    @property
    def owzy(self) -> I32Vector4: ...
    @property
    def owzz(self) -> I32Vector4: ...
    @property
    def owzw(self) -> I32Vector4: ...
    @property
    def owzo(self) -> I32Vector4: ...
    @property
    def owzl(self) -> I32Vector4: ...
    @property
    def owwx(self) -> I32Vector4: ...
    @property
    def owwy(self) -> I32Vector4: ...
    @property
    def owwz(self) -> I32Vector4: ...
    @property
    def owww(self) -> I32Vector4: ...
    @property
    def owwo(self) -> I32Vector4: ...
    @property
    def owwl(self) -> I32Vector4: ...
    @property
    def owox(self) -> I32Vector4: ...
    @property
    def owoy(self) -> I32Vector4: ...
    @property
    def owoz(self) -> I32Vector4: ...
    @property
    def owow(self) -> I32Vector4: ...
    @property
    def owoo(self) -> I32Vector4: ...
    @property
    def owol(self) -> I32Vector4: ...
    @property
    def owlx(self) -> I32Vector4: ...
    @property
    def owly(self) -> I32Vector4: ...
    @property
    def owlz(self) -> I32Vector4: ...
    @property
    def owlw(self) -> I32Vector4: ...
    @property
    def owlo(self) -> I32Vector4: ...
    @property
    def owll(self) -> I32Vector4: ...
    @property
    def ooxx(self) -> I32Vector4: ...
    @property
    def ooxy(self) -> I32Vector4: ...
    @property
    def ooxz(self) -> I32Vector4: ...
    @property
    def ooxw(self) -> I32Vector4: ...
    @property
    def ooxo(self) -> I32Vector4: ...
    @property
    def ooxl(self) -> I32Vector4: ...
    @property
    def ooyx(self) -> I32Vector4: ...
    @property
    def ooyy(self) -> I32Vector4: ...
    @property
    def ooyz(self) -> I32Vector4: ...
    @property
    def ooyw(self) -> I32Vector4: ...
    @property
    def ooyo(self) -> I32Vector4: ...
    @property
    def ooyl(self) -> I32Vector4: ...
    @property
    def oozx(self) -> I32Vector4: ...
    @property
    def oozy(self) -> I32Vector4: ...
    @property
    def oozz(self) -> I32Vector4: ...
    @property
    def oozw(self) -> I32Vector4: ...
    @property
    def oozo(self) -> I32Vector4: ...
    @property
    def oozl(self) -> I32Vector4: ...
    @property
    def oowx(self) -> I32Vector4: ...
    @property
    def oowy(self) -> I32Vector4: ...
    @property
    def oowz(self) -> I32Vector4: ...
    @property
    def ooww(self) -> I32Vector4: ...
    @property
    def oowo(self) -> I32Vector4: ...
    @property
    def oowl(self) -> I32Vector4: ...
    @property
    def ooox(self) -> I32Vector4: ...
    @property
    def oooy(self) -> I32Vector4: ...
    @property
    def oooz(self) -> I32Vector4: ...
    @property
    def ooow(self) -> I32Vector4: ...
    @property
    def oolx(self) -> I32Vector4: ...
    @property
    def ooly(self) -> I32Vector4: ...
    @property
    def oolz(self) -> I32Vector4: ...
    @property
    def oolw(self) -> I32Vector4: ...
    @property
    def olxx(self) -> I32Vector4: ...
    @property
    def olxy(self) -> I32Vector4: ...
    @property
    def olxz(self) -> I32Vector4: ...
    @property
    def olxw(self) -> I32Vector4: ...
    @property
    def olxo(self) -> I32Vector4: ...
    @property
    def olxl(self) -> I32Vector4: ...
    @property
    def olyx(self) -> I32Vector4: ...
    @property
    def olyy(self) -> I32Vector4: ...
    @property
    def olyz(self) -> I32Vector4: ...
    @property
    def olyw(self) -> I32Vector4: ...
    @property
    def olyo(self) -> I32Vector4: ...
    @property
    def olyl(self) -> I32Vector4: ...
    @property
    def olzx(self) -> I32Vector4: ...
    @property
    def olzy(self) -> I32Vector4: ...
    @property
    def olzz(self) -> I32Vector4: ...
    @property
    def olzw(self) -> I32Vector4: ...
    @property
    def olzo(self) -> I32Vector4: ...
    @property
    def olzl(self) -> I32Vector4: ...
    @property
    def olwx(self) -> I32Vector4: ...
    @property
    def olwy(self) -> I32Vector4: ...
    @property
    def olwz(self) -> I32Vector4: ...
    @property
    def olww(self) -> I32Vector4: ...
    @property
    def olwo(self) -> I32Vector4: ...
    @property
    def olwl(self) -> I32Vector4: ...
    @property
    def olox(self) -> I32Vector4: ...
    @property
    def oloy(self) -> I32Vector4: ...
    @property
    def oloz(self) -> I32Vector4: ...
    @property
    def olow(self) -> I32Vector4: ...
    @property
    def ollx(self) -> I32Vector4: ...
    @property
    def olly(self) -> I32Vector4: ...
    @property
    def ollz(self) -> I32Vector4: ...
    @property
    def ollw(self) -> I32Vector4: ...
    @property
    def lxxx(self) -> I32Vector4: ...
    @property
    def lxxy(self) -> I32Vector4: ...
    @property
    def lxxz(self) -> I32Vector4: ...
    @property
    def lxxw(self) -> I32Vector4: ...
    @property
    def lxxo(self) -> I32Vector4: ...
    @property
    def lxxl(self) -> I32Vector4: ...
    @property
    def lxyx(self) -> I32Vector4: ...
    @property
    def lxyy(self) -> I32Vector4: ...
    @property
    def lxyz(self) -> I32Vector4: ...
    @property
    def lxyw(self) -> I32Vector4: ...
    @property
    def lxyo(self) -> I32Vector4: ...
    @property
    def lxyl(self) -> I32Vector4: ...
    @property
    def lxzx(self) -> I32Vector4: ...
    @property
    def lxzy(self) -> I32Vector4: ...
    @property
    def lxzz(self) -> I32Vector4: ...
    @property
    def lxzw(self) -> I32Vector4: ...
    @property
    def lxzo(self) -> I32Vector4: ...
    @property
    def lxzl(self) -> I32Vector4: ...
    @property
    def lxwx(self) -> I32Vector4: ...
    @property
    def lxwy(self) -> I32Vector4: ...
    @property
    def lxwz(self) -> I32Vector4: ...
    @property
    def lxww(self) -> I32Vector4: ...
    @property
    def lxwo(self) -> I32Vector4: ...
    @property
    def lxwl(self) -> I32Vector4: ...
    @property
    def lxox(self) -> I32Vector4: ...
    @property
    def lxoy(self) -> I32Vector4: ...
    @property
    def lxoz(self) -> I32Vector4: ...
    @property
    def lxow(self) -> I32Vector4: ...
    @property
    def lxoo(self) -> I32Vector4: ...
    @property
    def lxol(self) -> I32Vector4: ...
    @property
    def lxlx(self) -> I32Vector4: ...
    @property
    def lxly(self) -> I32Vector4: ...
    @property
    def lxlz(self) -> I32Vector4: ...
    @property
    def lxlw(self) -> I32Vector4: ...
    @property
    def lxlo(self) -> I32Vector4: ...
    @property
    def lxll(self) -> I32Vector4: ...
    @property
    def lyxx(self) -> I32Vector4: ...
    @property
    def lyxy(self) -> I32Vector4: ...
    @property
    def lyxz(self) -> I32Vector4: ...
    @property
    def lyxw(self) -> I32Vector4: ...
    @property
    def lyxo(self) -> I32Vector4: ...
    @property
    def lyxl(self) -> I32Vector4: ...
    @property
    def lyyx(self) -> I32Vector4: ...
    @property
    def lyyy(self) -> I32Vector4: ...
    @property
    def lyyz(self) -> I32Vector4: ...
    @property
    def lyyw(self) -> I32Vector4: ...
    @property
    def lyyo(self) -> I32Vector4: ...
    @property
    def lyyl(self) -> I32Vector4: ...
    @property
    def lyzx(self) -> I32Vector4: ...
    @property
    def lyzy(self) -> I32Vector4: ...
    @property
    def lyzz(self) -> I32Vector4: ...
    @property
    def lyzw(self) -> I32Vector4: ...
    @property
    def lyzo(self) -> I32Vector4: ...
    @property
    def lyzl(self) -> I32Vector4: ...
    @property
    def lywx(self) -> I32Vector4: ...
    @property
    def lywy(self) -> I32Vector4: ...
    @property
    def lywz(self) -> I32Vector4: ...
    @property
    def lyww(self) -> I32Vector4: ...
    @property
    def lywo(self) -> I32Vector4: ...
    @property
    def lywl(self) -> I32Vector4: ...
    @property
    def lyox(self) -> I32Vector4: ...
    @property
    def lyoy(self) -> I32Vector4: ...
    @property
    def lyoz(self) -> I32Vector4: ...
    @property
    def lyow(self) -> I32Vector4: ...
    @property
    def lyoo(self) -> I32Vector4: ...
    @property
    def lyol(self) -> I32Vector4: ...
    @property
    def lylx(self) -> I32Vector4: ...
    @property
    def lyly(self) -> I32Vector4: ...
    @property
    def lylz(self) -> I32Vector4: ...
    @property
    def lylw(self) -> I32Vector4: ...
    @property
    def lylo(self) -> I32Vector4: ...
    @property
    def lyll(self) -> I32Vector4: ...
    @property
    def lzxx(self) -> I32Vector4: ...
    @property
    def lzxy(self) -> I32Vector4: ...
    @property
    def lzxz(self) -> I32Vector4: ...
    @property
    def lzxw(self) -> I32Vector4: ...
    @property
    def lzxo(self) -> I32Vector4: ...
    @property
    def lzxl(self) -> I32Vector4: ...
    @property
    def lzyx(self) -> I32Vector4: ...
    @property
    def lzyy(self) -> I32Vector4: ...
    @property
    def lzyz(self) -> I32Vector4: ...
    @property
    def lzyw(self) -> I32Vector4: ...
    @property
    def lzyo(self) -> I32Vector4: ...
    @property
    def lzyl(self) -> I32Vector4: ...
    @property
    def lzzx(self) -> I32Vector4: ...
    @property
    def lzzy(self) -> I32Vector4: ...
    @property
    def lzzz(self) -> I32Vector4: ...
    @property
    def lzzw(self) -> I32Vector4: ...
    @property
    def lzzo(self) -> I32Vector4: ...
    @property
    def lzzl(self) -> I32Vector4: ...
    @property
    def lzwx(self) -> I32Vector4: ...
    @property
    def lzwy(self) -> I32Vector4: ...
    @property
    def lzwz(self) -> I32Vector4: ...
    @property
    def lzww(self) -> I32Vector4: ...
    @property
    def lzwo(self) -> I32Vector4: ...
    @property
    def lzwl(self) -> I32Vector4: ...
    @property
    def lzox(self) -> I32Vector4: ...
    @property
    def lzoy(self) -> I32Vector4: ...
    @property
    def lzoz(self) -> I32Vector4: ...
    @property
    def lzow(self) -> I32Vector4: ...
    @property
    def lzoo(self) -> I32Vector4: ...
    @property
    def lzol(self) -> I32Vector4: ...
    @property
    def lzlx(self) -> I32Vector4: ...
    @property
    def lzly(self) -> I32Vector4: ...
    @property
    def lzlz(self) -> I32Vector4: ...
    @property
    def lzlw(self) -> I32Vector4: ...
    @property
    def lzlo(self) -> I32Vector4: ...
    @property
    def lzll(self) -> I32Vector4: ...
    @property
    def lwxx(self) -> I32Vector4: ...
    @property
    def lwxy(self) -> I32Vector4: ...
    @property
    def lwxz(self) -> I32Vector4: ...
    @property
    def lwxw(self) -> I32Vector4: ...
    @property
    def lwxo(self) -> I32Vector4: ...
    @property
    def lwxl(self) -> I32Vector4: ...
    @property
    def lwyx(self) -> I32Vector4: ...
    @property
    def lwyy(self) -> I32Vector4: ...
    @property
    def lwyz(self) -> I32Vector4: ...
    @property
    def lwyw(self) -> I32Vector4: ...
    @property
    def lwyo(self) -> I32Vector4: ...
    @property
    def lwyl(self) -> I32Vector4: ...
    @property
    def lwzx(self) -> I32Vector4: ...
    @property
    def lwzy(self) -> I32Vector4: ...
    @property
    def lwzz(self) -> I32Vector4: ...
    @property
    def lwzw(self) -> I32Vector4: ...
    @property
    def lwzo(self) -> I32Vector4: ...
    @property
    def lwzl(self) -> I32Vector4: ...
    @property
    def lwwx(self) -> I32Vector4: ...
    @property
    def lwwy(self) -> I32Vector4: ...
    @property
    def lwwz(self) -> I32Vector4: ...
    @property
    def lwww(self) -> I32Vector4: ...
    @property
    def lwwo(self) -> I32Vector4: ...
    @property
    def lwwl(self) -> I32Vector4: ...
    @property
    def lwox(self) -> I32Vector4: ...
    @property
    def lwoy(self) -> I32Vector4: ...
    @property
    def lwoz(self) -> I32Vector4: ...
    @property
    def lwow(self) -> I32Vector4: ...
    @property
    def lwoo(self) -> I32Vector4: ...
    @property
    def lwol(self) -> I32Vector4: ...
    @property
    def lwlx(self) -> I32Vector4: ...
    @property
    def lwly(self) -> I32Vector4: ...
    @property
    def lwlz(self) -> I32Vector4: ...
    @property
    def lwlw(self) -> I32Vector4: ...
    @property
    def lwlo(self) -> I32Vector4: ...
    @property
    def lwll(self) -> I32Vector4: ...
    @property
    def loxx(self) -> I32Vector4: ...
    @property
    def loxy(self) -> I32Vector4: ...
    @property
    def loxz(self) -> I32Vector4: ...
    @property
    def loxw(self) -> I32Vector4: ...
    @property
    def loxo(self) -> I32Vector4: ...
    @property
    def loxl(self) -> I32Vector4: ...
    @property
    def loyx(self) -> I32Vector4: ...
    @property
    def loyy(self) -> I32Vector4: ...
    @property
    def loyz(self) -> I32Vector4: ...
    @property
    def loyw(self) -> I32Vector4: ...
    @property
    def loyo(self) -> I32Vector4: ...
    @property
    def loyl(self) -> I32Vector4: ...
    @property
    def lozx(self) -> I32Vector4: ...
    @property
    def lozy(self) -> I32Vector4: ...
    @property
    def lozz(self) -> I32Vector4: ...
    @property
    def lozw(self) -> I32Vector4: ...
    @property
    def lozo(self) -> I32Vector4: ...
    @property
    def lozl(self) -> I32Vector4: ...
    @property
    def lowx(self) -> I32Vector4: ...
    @property
    def lowy(self) -> I32Vector4: ...
    @property
    def lowz(self) -> I32Vector4: ...
    @property
    def loww(self) -> I32Vector4: ...
    @property
    def lowo(self) -> I32Vector4: ...
    @property
    def lowl(self) -> I32Vector4: ...
    @property
    def loox(self) -> I32Vector4: ...
    @property
    def looy(self) -> I32Vector4: ...
    @property
    def looz(self) -> I32Vector4: ...
    @property
    def loow(self) -> I32Vector4: ...
    @property
    def lolx(self) -> I32Vector4: ...
    @property
    def loly(self) -> I32Vector4: ...
    @property
    def lolz(self) -> I32Vector4: ...
    @property
    def lolw(self) -> I32Vector4: ...
    @property
    def llxx(self) -> I32Vector4: ...
    @property
    def llxy(self) -> I32Vector4: ...
    @property
    def llxz(self) -> I32Vector4: ...
    @property
    def llxw(self) -> I32Vector4: ...
    @property
    def llxo(self) -> I32Vector4: ...
    @property
    def llxl(self) -> I32Vector4: ...
    @property
    def llyx(self) -> I32Vector4: ...
    @property
    def llyy(self) -> I32Vector4: ...
    @property
    def llyz(self) -> I32Vector4: ...
    @property
    def llyw(self) -> I32Vector4: ...
    @property
    def llyo(self) -> I32Vector4: ...
    @property
    def llyl(self) -> I32Vector4: ...
    @property
    def llzx(self) -> I32Vector4: ...
    @property
    def llzy(self) -> I32Vector4: ...
    @property
    def llzz(self) -> I32Vector4: ...
    @property
    def llzw(self) -> I32Vector4: ...
    @property
    def llzo(self) -> I32Vector4: ...
    @property
    def llzl(self) -> I32Vector4: ...
    @property
    def llwx(self) -> I32Vector4: ...
    @property
    def llwy(self) -> I32Vector4: ...
    @property
    def llwz(self) -> I32Vector4: ...
    @property
    def llww(self) -> I32Vector4: ...
    @property
    def llwo(self) -> I32Vector4: ...
    @property
    def llwl(self) -> I32Vector4: ...
    @property
    def llox(self) -> I32Vector4: ...
    @property
    def lloy(self) -> I32Vector4: ...
    @property
    def lloz(self) -> I32Vector4: ...
    @property
    def llow(self) -> I32Vector4: ...
    @property
    def lllx(self) -> I32Vector4: ...
    @property
    def llly(self) -> I32Vector4: ...
    @property
    def lllz(self) -> I32Vector4: ...
    @property
    def lllw(self) -> I32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> I32Vector2: ...
    @property
    def rg(self) -> I32Vector2: ...
    @property
    def rb(self) -> I32Vector2: ...
    @property
    def ra(self) -> I32Vector2: ...
    @property
    def ro(self) -> I32Vector2: ...
    @property
    def rl(self) -> I32Vector2: ...
    @property
    def gr(self) -> I32Vector2: ...
    @property
    def gg(self) -> I32Vector2: ...
    @property
    def gb(self) -> I32Vector2: ...
    @property
    def ga(self) -> I32Vector2: ...
    @property
    def go(self) -> I32Vector2: ...
    @property
    def gl(self) -> I32Vector2: ...
    @property
    def br(self) -> I32Vector2: ...
    @property
    def bg(self) -> I32Vector2: ...
    @property
    def bb(self) -> I32Vector2: ...
    @property
    def ba(self) -> I32Vector2: ...
    @property
    def bo(self) -> I32Vector2: ...
    @property
    def bl(self) -> I32Vector2: ...
    @property
    def ar(self) -> I32Vector2: ...
    @property
    def ag(self) -> I32Vector2: ...
    @property
    def ab(self) -> I32Vector2: ...
    @property
    def aa(self) -> I32Vector2: ...
    @property
    def ao(self) -> I32Vector2: ...
    @property
    def al(self) -> I32Vector2: ...
    @property
    def og(self) -> I32Vector2: ...
    @property
    def ob(self) -> I32Vector2: ...
    @property
    def oa(self) -> I32Vector2: ...
    @property
    def lr(self) -> I32Vector2: ...
    @property
    def lg(self) -> I32Vector2: ...
    @property
    def lb(self) -> I32Vector2: ...
    @property
    def la(self) -> I32Vector2: ...
    @property
    def rrr(self) -> I32Vector3: ...
    @property
    def rrg(self) -> I32Vector3: ...
    @property
    def rrb(self) -> I32Vector3: ...
    @property
    def rra(self) -> I32Vector3: ...
    @property
    def rro(self) -> I32Vector3: ...
    @property
    def rrl(self) -> I32Vector3: ...
    @property
    def rgr(self) -> I32Vector3: ...
    @property
    def rgg(self) -> I32Vector3: ...
    @property
    def rgb(self) -> I32Vector3: ...
    @property
    def rga(self) -> I32Vector3: ...
    @property
    def rgo(self) -> I32Vector3: ...
    @property
    def rgl(self) -> I32Vector3: ...
    @property
    def rbr(self) -> I32Vector3: ...
    @property
    def rbg(self) -> I32Vector3: ...
    @property
    def rbb(self) -> I32Vector3: ...
    @property
    def rba(self) -> I32Vector3: ...
    @property
    def rbo(self) -> I32Vector3: ...
    @property
    def rbl(self) -> I32Vector3: ...
    @property
    def rar(self) -> I32Vector3: ...
    @property
    def rag(self) -> I32Vector3: ...
    @property
    def rab(self) -> I32Vector3: ...
    @property
    def raa(self) -> I32Vector3: ...
    @property
    def rao(self) -> I32Vector3: ...
    @property
    def ral(self) -> I32Vector3: ...
    @property
    def ror(self) -> I32Vector3: ...
    @property
    def rog(self) -> I32Vector3: ...
    @property
    def rob(self) -> I32Vector3: ...
    @property
    def roa(self) -> I32Vector3: ...
    @property
    def roo(self) -> I32Vector3: ...
    @property
    def rol(self) -> I32Vector3: ...
    @property
    def rlr(self) -> I32Vector3: ...
    @property
    def rlg(self) -> I32Vector3: ...
    @property
    def rlb(self) -> I32Vector3: ...
    @property
    def rla(self) -> I32Vector3: ...
    @property
    def rlo(self) -> I32Vector3: ...
    @property
    def rll(self) -> I32Vector3: ...
    @property
    def grr(self) -> I32Vector3: ...
    @property
    def grg(self) -> I32Vector3: ...
    @property
    def grb(self) -> I32Vector3: ...
    @property
    def gra(self) -> I32Vector3: ...
    @property
    def gro(self) -> I32Vector3: ...
    @property
    def grl(self) -> I32Vector3: ...
    @property
    def ggr(self) -> I32Vector3: ...
    @property
    def ggg(self) -> I32Vector3: ...
    @property
    def ggb(self) -> I32Vector3: ...
    @property
    def gga(self) -> I32Vector3: ...
    @property
    def ggo(self) -> I32Vector3: ...
    @property
    def ggl(self) -> I32Vector3: ...
    @property
    def gbr(self) -> I32Vector3: ...
    @property
    def gbg(self) -> I32Vector3: ...
    @property
    def gbb(self) -> I32Vector3: ...
    @property
    def gba(self) -> I32Vector3: ...
    @property
    def gbo(self) -> I32Vector3: ...
    @property
    def gbl(self) -> I32Vector3: ...
    @property
    def gar(self) -> I32Vector3: ...
    @property
    def gag(self) -> I32Vector3: ...
    @property
    def gab(self) -> I32Vector3: ...
    @property
    def gaa(self) -> I32Vector3: ...
    @property
    def gao(self) -> I32Vector3: ...
    @property
    def gal(self) -> I32Vector3: ...
    @property
    def gor(self) -> I32Vector3: ...
    @property
    def gog(self) -> I32Vector3: ...
    @property
    def gob(self) -> I32Vector3: ...
    @property
    def goa(self) -> I32Vector3: ...
    @property
    def goo(self) -> I32Vector3: ...
    @property
    def gol(self) -> I32Vector3: ...
    @property
    def glr(self) -> I32Vector3: ...
    @property
    def glg(self) -> I32Vector3: ...
    @property
    def glb(self) -> I32Vector3: ...
    @property
    def gla(self) -> I32Vector3: ...
    @property
    def glo(self) -> I32Vector3: ...
    @property
    def gll(self) -> I32Vector3: ...
    @property
    def brr(self) -> I32Vector3: ...
    @property
    def brg(self) -> I32Vector3: ...
    @property
    def brb(self) -> I32Vector3: ...
    @property
    def bra(self) -> I32Vector3: ...
    @property
    def bro(self) -> I32Vector3: ...
    @property
    def brl(self) -> I32Vector3: ...
    @property
    def bgr(self) -> I32Vector3: ...
    @property
    def bgg(self) -> I32Vector3: ...
    @property
    def bgb(self) -> I32Vector3: ...
    @property
    def bga(self) -> I32Vector3: ...
    @property
    def bgo(self) -> I32Vector3: ...
    @property
    def bgl(self) -> I32Vector3: ...
    @property
    def bbr(self) -> I32Vector3: ...
    @property
    def bbg(self) -> I32Vector3: ...
    @property
    def bbb(self) -> I32Vector3: ...
    @property
    def bba(self) -> I32Vector3: ...
    @property
    def bbo(self) -> I32Vector3: ...
    @property
    def bbl(self) -> I32Vector3: ...
    @property
    def bar(self) -> I32Vector3: ...
    @property
    def bag(self) -> I32Vector3: ...
    @property
    def bab(self) -> I32Vector3: ...
    @property
    def baa(self) -> I32Vector3: ...
    @property
    def bao(self) -> I32Vector3: ...
    @property
    def bal(self) -> I32Vector3: ...
    @property
    def bor(self) -> I32Vector3: ...
    @property
    def bog(self) -> I32Vector3: ...
    @property
    def bob(self) -> I32Vector3: ...
    @property
    def boa(self) -> I32Vector3: ...
    @property
    def boo(self) -> I32Vector3: ...
    @property
    def bol(self) -> I32Vector3: ...
    @property
    def blr(self) -> I32Vector3: ...
    @property
    def blg(self) -> I32Vector3: ...
    @property
    def blb(self) -> I32Vector3: ...
    @property
    def bla(self) -> I32Vector3: ...
    @property
    def blo(self) -> I32Vector3: ...
    @property
    def bll(self) -> I32Vector3: ...
    @property
    def arr(self) -> I32Vector3: ...
    @property
    def arg(self) -> I32Vector3: ...
    @property
    def arb(self) -> I32Vector3: ...
    @property
    def ara(self) -> I32Vector3: ...
    @property
    def aro(self) -> I32Vector3: ...
    @property
    def arl(self) -> I32Vector3: ...
    @property
    def agr(self) -> I32Vector3: ...
    @property
    def agg(self) -> I32Vector3: ...
    @property
    def agb(self) -> I32Vector3: ...
    @property
    def aga(self) -> I32Vector3: ...
    @property
    def ago(self) -> I32Vector3: ...
    @property
    def agl(self) -> I32Vector3: ...
    @property
    def abr(self) -> I32Vector3: ...
    @property
    def abg(self) -> I32Vector3: ...
    @property
    def abb(self) -> I32Vector3: ...
    @property
    def aba(self) -> I32Vector3: ...
    @property
    def abo(self) -> I32Vector3: ...
    @property
    def abl(self) -> I32Vector3: ...
    @property
    def aar(self) -> I32Vector3: ...
    @property
    def aag(self) -> I32Vector3: ...
    @property
    def aab(self) -> I32Vector3: ...
    @property
    def aaa(self) -> I32Vector3: ...
    @property
    def aao(self) -> I32Vector3: ...
    @property
    def aal(self) -> I32Vector3: ...
    @property
    def aor(self) -> I32Vector3: ...
    @property
    def aog(self) -> I32Vector3: ...
    @property
    def aob(self) -> I32Vector3: ...
    @property
    def aoa(self) -> I32Vector3: ...
    @property
    def aoo(self) -> I32Vector3: ...
    @property
    def aol(self) -> I32Vector3: ...
    @property
    def alr(self) -> I32Vector3: ...
    @property
    def alg(self) -> I32Vector3: ...
    @property
    def alb(self) -> I32Vector3: ...
    @property
    def ala(self) -> I32Vector3: ...
    @property
    def alo(self) -> I32Vector3: ...
    @property
    def all(self) -> I32Vector3: ...
    @property
    def orr(self) -> I32Vector3: ...
    @property
    def org(self) -> I32Vector3: ...
    @property
    def orb(self) -> I32Vector3: ...
    @property
    def ora(self) -> I32Vector3: ...
    @property
    def oro(self) -> I32Vector3: ...
    @property
    def orl(self) -> I32Vector3: ...
    @property
    def ogr(self) -> I32Vector3: ...
    @property
    def ogg(self) -> I32Vector3: ...
    @property
    def ogb(self) -> I32Vector3: ...
    @property
    def oga(self) -> I32Vector3: ...
    @property
    def ogo(self) -> I32Vector3: ...
    @property
    def ogl(self) -> I32Vector3: ...
    @property
    def obr(self) -> I32Vector3: ...
    @property
    def obg(self) -> I32Vector3: ...
    @property
    def obb(self) -> I32Vector3: ...
    @property
    def oba(self) -> I32Vector3: ...
    @property
    def obo(self) -> I32Vector3: ...
    @property
    def obl(self) -> I32Vector3: ...
    @property
    def oar(self) -> I32Vector3: ...
    @property
    def oag(self) -> I32Vector3: ...
    @property
    def oab(self) -> I32Vector3: ...
    @property
    def oaa(self) -> I32Vector3: ...
    @property
    def oao(self) -> I32Vector3: ...
    @property
    def oal(self) -> I32Vector3: ...
    @property
    def oor(self) -> I32Vector3: ...
    @property
    def oog(self) -> I32Vector3: ...
    @property
    def oob(self) -> I32Vector3: ...
    @property
    def ooa(self) -> I32Vector3: ...
    @property
    def olr(self) -> I32Vector3: ...
    @property
    def olg(self) -> I32Vector3: ...
    @property
    def olb(self) -> I32Vector3: ...
    @property
    def ola(self) -> I32Vector3: ...
    @property
    def lrr(self) -> I32Vector3: ...
    @property
    def lrg(self) -> I32Vector3: ...
    @property
    def lrb(self) -> I32Vector3: ...
    @property
    def lra(self) -> I32Vector3: ...
    @property
    def lro(self) -> I32Vector3: ...
    @property
    def lrl(self) -> I32Vector3: ...
    @property
    def lgr(self) -> I32Vector3: ...
    @property
    def lgg(self) -> I32Vector3: ...
    @property
    def lgb(self) -> I32Vector3: ...
    @property
    def lga(self) -> I32Vector3: ...
    @property
    def lgo(self) -> I32Vector3: ...
    @property
    def lgl(self) -> I32Vector3: ...
    @property
    def lbr(self) -> I32Vector3: ...
    @property
    def lbg(self) -> I32Vector3: ...
    @property
    def lbb(self) -> I32Vector3: ...
    @property
    def lba(self) -> I32Vector3: ...
    @property
    def lbo(self) -> I32Vector3: ...
    @property
    def lbl(self) -> I32Vector3: ...
    @property
    def lar(self) -> I32Vector3: ...
    @property
    def lag(self) -> I32Vector3: ...
    @property
    def lab(self) -> I32Vector3: ...
    @property
    def laa(self) -> I32Vector3: ...
    @property
    def lao(self) -> I32Vector3: ...
    @property
    def lal(self) -> I32Vector3: ...
    @property
    def lor(self) -> I32Vector3: ...
    @property
    def log(self) -> I32Vector3: ...
    @property
    def lob(self) -> I32Vector3: ...
    @property
    def loa(self) -> I32Vector3: ...
    @property
    def llr(self) -> I32Vector3: ...
    @property
    def llg(self) -> I32Vector3: ...
    @property
    def llb(self) -> I32Vector3: ...
    @property
    def lla(self) -> I32Vector3: ...
    @property
    def rrrr(self) -> I32Vector4: ...
    @property
    def rrrg(self) -> I32Vector4: ...
    @property
    def rrrb(self) -> I32Vector4: ...
    @property
    def rrra(self) -> I32Vector4: ...
    @property
    def rrro(self) -> I32Vector4: ...
    @property
    def rrrl(self) -> I32Vector4: ...
    @property
    def rrgr(self) -> I32Vector4: ...
    @property
    def rrgg(self) -> I32Vector4: ...
    @property
    def rrgb(self) -> I32Vector4: ...
    @property
    def rrga(self) -> I32Vector4: ...
    @property
    def rrgo(self) -> I32Vector4: ...
    @property
    def rrgl(self) -> I32Vector4: ...
    @property
    def rrbr(self) -> I32Vector4: ...
    @property
    def rrbg(self) -> I32Vector4: ...
    @property
    def rrbb(self) -> I32Vector4: ...
    @property
    def rrba(self) -> I32Vector4: ...
    @property
    def rrbo(self) -> I32Vector4: ...
    @property
    def rrbl(self) -> I32Vector4: ...
    @property
    def rrar(self) -> I32Vector4: ...
    @property
    def rrag(self) -> I32Vector4: ...
    @property
    def rrab(self) -> I32Vector4: ...
    @property
    def rraa(self) -> I32Vector4: ...
    @property
    def rrao(self) -> I32Vector4: ...
    @property
    def rral(self) -> I32Vector4: ...
    @property
    def rror(self) -> I32Vector4: ...
    @property
    def rrog(self) -> I32Vector4: ...
    @property
    def rrob(self) -> I32Vector4: ...
    @property
    def rroa(self) -> I32Vector4: ...
    @property
    def rroo(self) -> I32Vector4: ...
    @property
    def rrol(self) -> I32Vector4: ...
    @property
    def rrlr(self) -> I32Vector4: ...
    @property
    def rrlg(self) -> I32Vector4: ...
    @property
    def rrlb(self) -> I32Vector4: ...
    @property
    def rrla(self) -> I32Vector4: ...
    @property
    def rrlo(self) -> I32Vector4: ...
    @property
    def rrll(self) -> I32Vector4: ...
    @property
    def rgrr(self) -> I32Vector4: ...
    @property
    def rgrg(self) -> I32Vector4: ...
    @property
    def rgrb(self) -> I32Vector4: ...
    @property
    def rgra(self) -> I32Vector4: ...
    @property
    def rgro(self) -> I32Vector4: ...
    @property
    def rgrl(self) -> I32Vector4: ...
    @property
    def rggr(self) -> I32Vector4: ...
    @property
    def rggg(self) -> I32Vector4: ...
    @property
    def rggb(self) -> I32Vector4: ...
    @property
    def rgga(self) -> I32Vector4: ...
    @property
    def rggo(self) -> I32Vector4: ...
    @property
    def rggl(self) -> I32Vector4: ...
    @property
    def rgbr(self) -> I32Vector4: ...
    @property
    def rgbg(self) -> I32Vector4: ...
    @property
    def rgbb(self) -> I32Vector4: ...
    @property
    def rgba(self) -> I32Vector4: ...
    @property
    def rgbo(self) -> I32Vector4: ...
    @property
    def rgbl(self) -> I32Vector4: ...
    @property
    def rgar(self) -> I32Vector4: ...
    @property
    def rgag(self) -> I32Vector4: ...
    @property
    def rgab(self) -> I32Vector4: ...
    @property
    def rgaa(self) -> I32Vector4: ...
    @property
    def rgao(self) -> I32Vector4: ...
    @property
    def rgal(self) -> I32Vector4: ...
    @property
    def rgor(self) -> I32Vector4: ...
    @property
    def rgog(self) -> I32Vector4: ...
    @property
    def rgob(self) -> I32Vector4: ...
    @property
    def rgoa(self) -> I32Vector4: ...
    @property
    def rgoo(self) -> I32Vector4: ...
    @property
    def rgol(self) -> I32Vector4: ...
    @property
    def rglr(self) -> I32Vector4: ...
    @property
    def rglg(self) -> I32Vector4: ...
    @property
    def rglb(self) -> I32Vector4: ...
    @property
    def rgla(self) -> I32Vector4: ...
    @property
    def rglo(self) -> I32Vector4: ...
    @property
    def rgll(self) -> I32Vector4: ...
    @property
    def rbrr(self) -> I32Vector4: ...
    @property
    def rbrg(self) -> I32Vector4: ...
    @property
    def rbrb(self) -> I32Vector4: ...
    @property
    def rbra(self) -> I32Vector4: ...
    @property
    def rbro(self) -> I32Vector4: ...
    @property
    def rbrl(self) -> I32Vector4: ...
    @property
    def rbgr(self) -> I32Vector4: ...
    @property
    def rbgg(self) -> I32Vector4: ...
    @property
    def rbgb(self) -> I32Vector4: ...
    @property
    def rbga(self) -> I32Vector4: ...
    @property
    def rbgo(self) -> I32Vector4: ...
    @property
    def rbgl(self) -> I32Vector4: ...
    @property
    def rbbr(self) -> I32Vector4: ...
    @property
    def rbbg(self) -> I32Vector4: ...
    @property
    def rbbb(self) -> I32Vector4: ...
    @property
    def rbba(self) -> I32Vector4: ...
    @property
    def rbbo(self) -> I32Vector4: ...
    @property
    def rbbl(self) -> I32Vector4: ...
    @property
    def rbar(self) -> I32Vector4: ...
    @property
    def rbag(self) -> I32Vector4: ...
    @property
    def rbab(self) -> I32Vector4: ...
    @property
    def rbaa(self) -> I32Vector4: ...
    @property
    def rbao(self) -> I32Vector4: ...
    @property
    def rbal(self) -> I32Vector4: ...
    @property
    def rbor(self) -> I32Vector4: ...
    @property
    def rbog(self) -> I32Vector4: ...
    @property
    def rbob(self) -> I32Vector4: ...
    @property
    def rboa(self) -> I32Vector4: ...
    @property
    def rboo(self) -> I32Vector4: ...
    @property
    def rbol(self) -> I32Vector4: ...
    @property
    def rblr(self) -> I32Vector4: ...
    @property
    def rblg(self) -> I32Vector4: ...
    @property
    def rblb(self) -> I32Vector4: ...
    @property
    def rbla(self) -> I32Vector4: ...
    @property
    def rblo(self) -> I32Vector4: ...
    @property
    def rbll(self) -> I32Vector4: ...
    @property
    def rarr(self) -> I32Vector4: ...
    @property
    def rarg(self) -> I32Vector4: ...
    @property
    def rarb(self) -> I32Vector4: ...
    @property
    def rara(self) -> I32Vector4: ...
    @property
    def raro(self) -> I32Vector4: ...
    @property
    def rarl(self) -> I32Vector4: ...
    @property
    def ragr(self) -> I32Vector4: ...
    @property
    def ragg(self) -> I32Vector4: ...
    @property
    def ragb(self) -> I32Vector4: ...
    @property
    def raga(self) -> I32Vector4: ...
    @property
    def rago(self) -> I32Vector4: ...
    @property
    def ragl(self) -> I32Vector4: ...
    @property
    def rabr(self) -> I32Vector4: ...
    @property
    def rabg(self) -> I32Vector4: ...
    @property
    def rabb(self) -> I32Vector4: ...
    @property
    def raba(self) -> I32Vector4: ...
    @property
    def rabo(self) -> I32Vector4: ...
    @property
    def rabl(self) -> I32Vector4: ...
    @property
    def raar(self) -> I32Vector4: ...
    @property
    def raag(self) -> I32Vector4: ...
    @property
    def raab(self) -> I32Vector4: ...
    @property
    def raaa(self) -> I32Vector4: ...
    @property
    def raao(self) -> I32Vector4: ...
    @property
    def raal(self) -> I32Vector4: ...
    @property
    def raor(self) -> I32Vector4: ...
    @property
    def raog(self) -> I32Vector4: ...
    @property
    def raob(self) -> I32Vector4: ...
    @property
    def raoa(self) -> I32Vector4: ...
    @property
    def raoo(self) -> I32Vector4: ...
    @property
    def raol(self) -> I32Vector4: ...
    @property
    def ralr(self) -> I32Vector4: ...
    @property
    def ralg(self) -> I32Vector4: ...
    @property
    def ralb(self) -> I32Vector4: ...
    @property
    def rala(self) -> I32Vector4: ...
    @property
    def ralo(self) -> I32Vector4: ...
    @property
    def rall(self) -> I32Vector4: ...
    @property
    def rorr(self) -> I32Vector4: ...
    @property
    def rorg(self) -> I32Vector4: ...
    @property
    def rorb(self) -> I32Vector4: ...
    @property
    def rora(self) -> I32Vector4: ...
    @property
    def roro(self) -> I32Vector4: ...
    @property
    def rorl(self) -> I32Vector4: ...
    @property
    def rogr(self) -> I32Vector4: ...
    @property
    def rogg(self) -> I32Vector4: ...
    @property
    def rogb(self) -> I32Vector4: ...
    @property
    def roga(self) -> I32Vector4: ...
    @property
    def rogo(self) -> I32Vector4: ...
    @property
    def rogl(self) -> I32Vector4: ...
    @property
    def robr(self) -> I32Vector4: ...
    @property
    def robg(self) -> I32Vector4: ...
    @property
    def robb(self) -> I32Vector4: ...
    @property
    def roba(self) -> I32Vector4: ...
    @property
    def robo(self) -> I32Vector4: ...
    @property
    def robl(self) -> I32Vector4: ...
    @property
    def roar(self) -> I32Vector4: ...
    @property
    def roag(self) -> I32Vector4: ...
    @property
    def roab(self) -> I32Vector4: ...
    @property
    def roaa(self) -> I32Vector4: ...
    @property
    def roao(self) -> I32Vector4: ...
    @property
    def roal(self) -> I32Vector4: ...
    @property
    def roor(self) -> I32Vector4: ...
    @property
    def roog(self) -> I32Vector4: ...
    @property
    def roob(self) -> I32Vector4: ...
    @property
    def rooa(self) -> I32Vector4: ...
    @property
    def rooo(self) -> I32Vector4: ...
    @property
    def rool(self) -> I32Vector4: ...
    @property
    def rolr(self) -> I32Vector4: ...
    @property
    def rolg(self) -> I32Vector4: ...
    @property
    def rolb(self) -> I32Vector4: ...
    @property
    def rola(self) -> I32Vector4: ...
    @property
    def rolo(self) -> I32Vector4: ...
    @property
    def roll(self) -> I32Vector4: ...
    @property
    def rlrr(self) -> I32Vector4: ...
    @property
    def rlrg(self) -> I32Vector4: ...
    @property
    def rlrb(self) -> I32Vector4: ...
    @property
    def rlra(self) -> I32Vector4: ...
    @property
    def rlro(self) -> I32Vector4: ...
    @property
    def rlrl(self) -> I32Vector4: ...
    @property
    def rlgr(self) -> I32Vector4: ...
    @property
    def rlgg(self) -> I32Vector4: ...
    @property
    def rlgb(self) -> I32Vector4: ...
    @property
    def rlga(self) -> I32Vector4: ...
    @property
    def rlgo(self) -> I32Vector4: ...
    @property
    def rlgl(self) -> I32Vector4: ...
    @property
    def rlbr(self) -> I32Vector4: ...
    @property
    def rlbg(self) -> I32Vector4: ...
    @property
    def rlbb(self) -> I32Vector4: ...
    @property
    def rlba(self) -> I32Vector4: ...
    @property
    def rlbo(self) -> I32Vector4: ...
    @property
    def rlbl(self) -> I32Vector4: ...
    @property
    def rlar(self) -> I32Vector4: ...
    @property
    def rlag(self) -> I32Vector4: ...
    @property
    def rlab(self) -> I32Vector4: ...
    @property
    def rlaa(self) -> I32Vector4: ...
    @property
    def rlao(self) -> I32Vector4: ...
    @property
    def rlal(self) -> I32Vector4: ...
    @property
    def rlor(self) -> I32Vector4: ...
    @property
    def rlog(self) -> I32Vector4: ...
    @property
    def rlob(self) -> I32Vector4: ...
    @property
    def rloa(self) -> I32Vector4: ...
    @property
    def rloo(self) -> I32Vector4: ...
    @property
    def rlol(self) -> I32Vector4: ...
    @property
    def rllr(self) -> I32Vector4: ...
    @property
    def rllg(self) -> I32Vector4: ...
    @property
    def rllb(self) -> I32Vector4: ...
    @property
    def rlla(self) -> I32Vector4: ...
    @property
    def rllo(self) -> I32Vector4: ...
    @property
    def rlll(self) -> I32Vector4: ...
    @property
    def grrr(self) -> I32Vector4: ...
    @property
    def grrg(self) -> I32Vector4: ...
    @property
    def grrb(self) -> I32Vector4: ...
    @property
    def grra(self) -> I32Vector4: ...
    @property
    def grro(self) -> I32Vector4: ...
    @property
    def grrl(self) -> I32Vector4: ...
    @property
    def grgr(self) -> I32Vector4: ...
    @property
    def grgg(self) -> I32Vector4: ...
    @property
    def grgb(self) -> I32Vector4: ...
    @property
    def grga(self) -> I32Vector4: ...
    @property
    def grgo(self) -> I32Vector4: ...
    @property
    def grgl(self) -> I32Vector4: ...
    @property
    def grbr(self) -> I32Vector4: ...
    @property
    def grbg(self) -> I32Vector4: ...
    @property
    def grbb(self) -> I32Vector4: ...
    @property
    def grba(self) -> I32Vector4: ...
    @property
    def grbo(self) -> I32Vector4: ...
    @property
    def grbl(self) -> I32Vector4: ...
    @property
    def grar(self) -> I32Vector4: ...
    @property
    def grag(self) -> I32Vector4: ...
    @property
    def grab(self) -> I32Vector4: ...
    @property
    def graa(self) -> I32Vector4: ...
    @property
    def grao(self) -> I32Vector4: ...
    @property
    def gral(self) -> I32Vector4: ...
    @property
    def gror(self) -> I32Vector4: ...
    @property
    def grog(self) -> I32Vector4: ...
    @property
    def grob(self) -> I32Vector4: ...
    @property
    def groa(self) -> I32Vector4: ...
    @property
    def groo(self) -> I32Vector4: ...
    @property
    def grol(self) -> I32Vector4: ...
    @property
    def grlr(self) -> I32Vector4: ...
    @property
    def grlg(self) -> I32Vector4: ...
    @property
    def grlb(self) -> I32Vector4: ...
    @property
    def grla(self) -> I32Vector4: ...
    @property
    def grlo(self) -> I32Vector4: ...
    @property
    def grll(self) -> I32Vector4: ...
    @property
    def ggrr(self) -> I32Vector4: ...
    @property
    def ggrg(self) -> I32Vector4: ...
    @property
    def ggrb(self) -> I32Vector4: ...
    @property
    def ggra(self) -> I32Vector4: ...
    @property
    def ggro(self) -> I32Vector4: ...
    @property
    def ggrl(self) -> I32Vector4: ...
    @property
    def gggr(self) -> I32Vector4: ...
    @property
    def gggg(self) -> I32Vector4: ...
    @property
    def gggb(self) -> I32Vector4: ...
    @property
    def ggga(self) -> I32Vector4: ...
    @property
    def gggo(self) -> I32Vector4: ...
    @property
    def gggl(self) -> I32Vector4: ...
    @property
    def ggbr(self) -> I32Vector4: ...
    @property
    def ggbg(self) -> I32Vector4: ...
    @property
    def ggbb(self) -> I32Vector4: ...
    @property
    def ggba(self) -> I32Vector4: ...
    @property
    def ggbo(self) -> I32Vector4: ...
    @property
    def ggbl(self) -> I32Vector4: ...
    @property
    def ggar(self) -> I32Vector4: ...
    @property
    def ggag(self) -> I32Vector4: ...
    @property
    def ggab(self) -> I32Vector4: ...
    @property
    def ggaa(self) -> I32Vector4: ...
    @property
    def ggao(self) -> I32Vector4: ...
    @property
    def ggal(self) -> I32Vector4: ...
    @property
    def ggor(self) -> I32Vector4: ...
    @property
    def ggog(self) -> I32Vector4: ...
    @property
    def ggob(self) -> I32Vector4: ...
    @property
    def ggoa(self) -> I32Vector4: ...
    @property
    def ggoo(self) -> I32Vector4: ...
    @property
    def ggol(self) -> I32Vector4: ...
    @property
    def gglr(self) -> I32Vector4: ...
    @property
    def gglg(self) -> I32Vector4: ...
    @property
    def gglb(self) -> I32Vector4: ...
    @property
    def ggla(self) -> I32Vector4: ...
    @property
    def gglo(self) -> I32Vector4: ...
    @property
    def ggll(self) -> I32Vector4: ...
    @property
    def gbrr(self) -> I32Vector4: ...
    @property
    def gbrg(self) -> I32Vector4: ...
    @property
    def gbrb(self) -> I32Vector4: ...
    @property
    def gbra(self) -> I32Vector4: ...
    @property
    def gbro(self) -> I32Vector4: ...
    @property
    def gbrl(self) -> I32Vector4: ...
    @property
    def gbgr(self) -> I32Vector4: ...
    @property
    def gbgg(self) -> I32Vector4: ...
    @property
    def gbgb(self) -> I32Vector4: ...
    @property
    def gbga(self) -> I32Vector4: ...
    @property
    def gbgo(self) -> I32Vector4: ...
    @property
    def gbgl(self) -> I32Vector4: ...
    @property
    def gbbr(self) -> I32Vector4: ...
    @property
    def gbbg(self) -> I32Vector4: ...
    @property
    def gbbb(self) -> I32Vector4: ...
    @property
    def gbba(self) -> I32Vector4: ...
    @property
    def gbbo(self) -> I32Vector4: ...
    @property
    def gbbl(self) -> I32Vector4: ...
    @property
    def gbar(self) -> I32Vector4: ...
    @property
    def gbag(self) -> I32Vector4: ...
    @property
    def gbab(self) -> I32Vector4: ...
    @property
    def gbaa(self) -> I32Vector4: ...
    @property
    def gbao(self) -> I32Vector4: ...
    @property
    def gbal(self) -> I32Vector4: ...
    @property
    def gbor(self) -> I32Vector4: ...
    @property
    def gbog(self) -> I32Vector4: ...
    @property
    def gbob(self) -> I32Vector4: ...
    @property
    def gboa(self) -> I32Vector4: ...
    @property
    def gboo(self) -> I32Vector4: ...
    @property
    def gbol(self) -> I32Vector4: ...
    @property
    def gblr(self) -> I32Vector4: ...
    @property
    def gblg(self) -> I32Vector4: ...
    @property
    def gblb(self) -> I32Vector4: ...
    @property
    def gbla(self) -> I32Vector4: ...
    @property
    def gblo(self) -> I32Vector4: ...
    @property
    def gbll(self) -> I32Vector4: ...
    @property
    def garr(self) -> I32Vector4: ...
    @property
    def garg(self) -> I32Vector4: ...
    @property
    def garb(self) -> I32Vector4: ...
    @property
    def gara(self) -> I32Vector4: ...
    @property
    def garo(self) -> I32Vector4: ...
    @property
    def garl(self) -> I32Vector4: ...
    @property
    def gagr(self) -> I32Vector4: ...
    @property
    def gagg(self) -> I32Vector4: ...
    @property
    def gagb(self) -> I32Vector4: ...
    @property
    def gaga(self) -> I32Vector4: ...
    @property
    def gago(self) -> I32Vector4: ...
    @property
    def gagl(self) -> I32Vector4: ...
    @property
    def gabr(self) -> I32Vector4: ...
    @property
    def gabg(self) -> I32Vector4: ...
    @property
    def gabb(self) -> I32Vector4: ...
    @property
    def gaba(self) -> I32Vector4: ...
    @property
    def gabo(self) -> I32Vector4: ...
    @property
    def gabl(self) -> I32Vector4: ...
    @property
    def gaar(self) -> I32Vector4: ...
    @property
    def gaag(self) -> I32Vector4: ...
    @property
    def gaab(self) -> I32Vector4: ...
    @property
    def gaaa(self) -> I32Vector4: ...
    @property
    def gaao(self) -> I32Vector4: ...
    @property
    def gaal(self) -> I32Vector4: ...
    @property
    def gaor(self) -> I32Vector4: ...
    @property
    def gaog(self) -> I32Vector4: ...
    @property
    def gaob(self) -> I32Vector4: ...
    @property
    def gaoa(self) -> I32Vector4: ...
    @property
    def gaoo(self) -> I32Vector4: ...
    @property
    def gaol(self) -> I32Vector4: ...
    @property
    def galr(self) -> I32Vector4: ...
    @property
    def galg(self) -> I32Vector4: ...
    @property
    def galb(self) -> I32Vector4: ...
    @property
    def gala(self) -> I32Vector4: ...
    @property
    def galo(self) -> I32Vector4: ...
    @property
    def gall(self) -> I32Vector4: ...
    @property
    def gorr(self) -> I32Vector4: ...
    @property
    def gorg(self) -> I32Vector4: ...
    @property
    def gorb(self) -> I32Vector4: ...
    @property
    def gora(self) -> I32Vector4: ...
    @property
    def goro(self) -> I32Vector4: ...
    @property
    def gorl(self) -> I32Vector4: ...
    @property
    def gogr(self) -> I32Vector4: ...
    @property
    def gogg(self) -> I32Vector4: ...
    @property
    def gogb(self) -> I32Vector4: ...
    @property
    def goga(self) -> I32Vector4: ...
    @property
    def gogo(self) -> I32Vector4: ...
    @property
    def gogl(self) -> I32Vector4: ...
    @property
    def gobr(self) -> I32Vector4: ...
    @property
    def gobg(self) -> I32Vector4: ...
    @property
    def gobb(self) -> I32Vector4: ...
    @property
    def goba(self) -> I32Vector4: ...
    @property
    def gobo(self) -> I32Vector4: ...
    @property
    def gobl(self) -> I32Vector4: ...
    @property
    def goar(self) -> I32Vector4: ...
    @property
    def goag(self) -> I32Vector4: ...
    @property
    def goab(self) -> I32Vector4: ...
    @property
    def goaa(self) -> I32Vector4: ...
    @property
    def goao(self) -> I32Vector4: ...
    @property
    def goal(self) -> I32Vector4: ...
    @property
    def goor(self) -> I32Vector4: ...
    @property
    def goog(self) -> I32Vector4: ...
    @property
    def goob(self) -> I32Vector4: ...
    @property
    def gooa(self) -> I32Vector4: ...
    @property
    def gooo(self) -> I32Vector4: ...
    @property
    def gool(self) -> I32Vector4: ...
    @property
    def golr(self) -> I32Vector4: ...
    @property
    def golg(self) -> I32Vector4: ...
    @property
    def golb(self) -> I32Vector4: ...
    @property
    def gola(self) -> I32Vector4: ...
    @property
    def golo(self) -> I32Vector4: ...
    @property
    def goll(self) -> I32Vector4: ...
    @property
    def glrr(self) -> I32Vector4: ...
    @property
    def glrg(self) -> I32Vector4: ...
    @property
    def glrb(self) -> I32Vector4: ...
    @property
    def glra(self) -> I32Vector4: ...
    @property
    def glro(self) -> I32Vector4: ...
    @property
    def glrl(self) -> I32Vector4: ...
    @property
    def glgr(self) -> I32Vector4: ...
    @property
    def glgg(self) -> I32Vector4: ...
    @property
    def glgb(self) -> I32Vector4: ...
    @property
    def glga(self) -> I32Vector4: ...
    @property
    def glgo(self) -> I32Vector4: ...
    @property
    def glgl(self) -> I32Vector4: ...
    @property
    def glbr(self) -> I32Vector4: ...
    @property
    def glbg(self) -> I32Vector4: ...
    @property
    def glbb(self) -> I32Vector4: ...
    @property
    def glba(self) -> I32Vector4: ...
    @property
    def glbo(self) -> I32Vector4: ...
    @property
    def glbl(self) -> I32Vector4: ...
    @property
    def glar(self) -> I32Vector4: ...
    @property
    def glag(self) -> I32Vector4: ...
    @property
    def glab(self) -> I32Vector4: ...
    @property
    def glaa(self) -> I32Vector4: ...
    @property
    def glao(self) -> I32Vector4: ...
    @property
    def glal(self) -> I32Vector4: ...
    @property
    def glor(self) -> I32Vector4: ...
    @property
    def glog(self) -> I32Vector4: ...
    @property
    def glob(self) -> I32Vector4: ...
    @property
    def gloa(self) -> I32Vector4: ...
    @property
    def gloo(self) -> I32Vector4: ...
    @property
    def glol(self) -> I32Vector4: ...
    @property
    def gllr(self) -> I32Vector4: ...
    @property
    def gllg(self) -> I32Vector4: ...
    @property
    def gllb(self) -> I32Vector4: ...
    @property
    def glla(self) -> I32Vector4: ...
    @property
    def gllo(self) -> I32Vector4: ...
    @property
    def glll(self) -> I32Vector4: ...
    @property
    def brrr(self) -> I32Vector4: ...
    @property
    def brrg(self) -> I32Vector4: ...
    @property
    def brrb(self) -> I32Vector4: ...
    @property
    def brra(self) -> I32Vector4: ...
    @property
    def brro(self) -> I32Vector4: ...
    @property
    def brrl(self) -> I32Vector4: ...
    @property
    def brgr(self) -> I32Vector4: ...
    @property
    def brgg(self) -> I32Vector4: ...
    @property
    def brgb(self) -> I32Vector4: ...
    @property
    def brga(self) -> I32Vector4: ...
    @property
    def brgo(self) -> I32Vector4: ...
    @property
    def brgl(self) -> I32Vector4: ...
    @property
    def brbr(self) -> I32Vector4: ...
    @property
    def brbg(self) -> I32Vector4: ...
    @property
    def brbb(self) -> I32Vector4: ...
    @property
    def brba(self) -> I32Vector4: ...
    @property
    def brbo(self) -> I32Vector4: ...
    @property
    def brbl(self) -> I32Vector4: ...
    @property
    def brar(self) -> I32Vector4: ...
    @property
    def brag(self) -> I32Vector4: ...
    @property
    def brab(self) -> I32Vector4: ...
    @property
    def braa(self) -> I32Vector4: ...
    @property
    def brao(self) -> I32Vector4: ...
    @property
    def bral(self) -> I32Vector4: ...
    @property
    def bror(self) -> I32Vector4: ...
    @property
    def brog(self) -> I32Vector4: ...
    @property
    def brob(self) -> I32Vector4: ...
    @property
    def broa(self) -> I32Vector4: ...
    @property
    def broo(self) -> I32Vector4: ...
    @property
    def brol(self) -> I32Vector4: ...
    @property
    def brlr(self) -> I32Vector4: ...
    @property
    def brlg(self) -> I32Vector4: ...
    @property
    def brlb(self) -> I32Vector4: ...
    @property
    def brla(self) -> I32Vector4: ...
    @property
    def brlo(self) -> I32Vector4: ...
    @property
    def brll(self) -> I32Vector4: ...
    @property
    def bgrr(self) -> I32Vector4: ...
    @property
    def bgrg(self) -> I32Vector4: ...
    @property
    def bgrb(self) -> I32Vector4: ...
    @property
    def bgra(self) -> I32Vector4: ...
    @property
    def bgro(self) -> I32Vector4: ...
    @property
    def bgrl(self) -> I32Vector4: ...
    @property
    def bggr(self) -> I32Vector4: ...
    @property
    def bggg(self) -> I32Vector4: ...
    @property
    def bggb(self) -> I32Vector4: ...
    @property
    def bgga(self) -> I32Vector4: ...
    @property
    def bggo(self) -> I32Vector4: ...
    @property
    def bggl(self) -> I32Vector4: ...
    @property
    def bgbr(self) -> I32Vector4: ...
    @property
    def bgbg(self) -> I32Vector4: ...
    @property
    def bgbb(self) -> I32Vector4: ...
    @property
    def bgba(self) -> I32Vector4: ...
    @property
    def bgbo(self) -> I32Vector4: ...
    @property
    def bgbl(self) -> I32Vector4: ...
    @property
    def bgar(self) -> I32Vector4: ...
    @property
    def bgag(self) -> I32Vector4: ...
    @property
    def bgab(self) -> I32Vector4: ...
    @property
    def bgaa(self) -> I32Vector4: ...
    @property
    def bgao(self) -> I32Vector4: ...
    @property
    def bgal(self) -> I32Vector4: ...
    @property
    def bgor(self) -> I32Vector4: ...
    @property
    def bgog(self) -> I32Vector4: ...
    @property
    def bgob(self) -> I32Vector4: ...
    @property
    def bgoa(self) -> I32Vector4: ...
    @property
    def bgoo(self) -> I32Vector4: ...
    @property
    def bgol(self) -> I32Vector4: ...
    @property
    def bglr(self) -> I32Vector4: ...
    @property
    def bglg(self) -> I32Vector4: ...
    @property
    def bglb(self) -> I32Vector4: ...
    @property
    def bgla(self) -> I32Vector4: ...
    @property
    def bglo(self) -> I32Vector4: ...
    @property
    def bgll(self) -> I32Vector4: ...
    @property
    def bbrr(self) -> I32Vector4: ...
    @property
    def bbrg(self) -> I32Vector4: ...
    @property
    def bbrb(self) -> I32Vector4: ...
    @property
    def bbra(self) -> I32Vector4: ...
    @property
    def bbro(self) -> I32Vector4: ...
    @property
    def bbrl(self) -> I32Vector4: ...
    @property
    def bbgr(self) -> I32Vector4: ...
    @property
    def bbgg(self) -> I32Vector4: ...
    @property
    def bbgb(self) -> I32Vector4: ...
    @property
    def bbga(self) -> I32Vector4: ...
    @property
    def bbgo(self) -> I32Vector4: ...
    @property
    def bbgl(self) -> I32Vector4: ...
    @property
    def bbbr(self) -> I32Vector4: ...
    @property
    def bbbg(self) -> I32Vector4: ...
    @property
    def bbbb(self) -> I32Vector4: ...
    @property
    def bbba(self) -> I32Vector4: ...
    @property
    def bbbo(self) -> I32Vector4: ...
    @property
    def bbbl(self) -> I32Vector4: ...
    @property
    def bbar(self) -> I32Vector4: ...
    @property
    def bbag(self) -> I32Vector4: ...
    @property
    def bbab(self) -> I32Vector4: ...
    @property
    def bbaa(self) -> I32Vector4: ...
    @property
    def bbao(self) -> I32Vector4: ...
    @property
    def bbal(self) -> I32Vector4: ...
    @property
    def bbor(self) -> I32Vector4: ...
    @property
    def bbog(self) -> I32Vector4: ...
    @property
    def bbob(self) -> I32Vector4: ...
    @property
    def bboa(self) -> I32Vector4: ...
    @property
    def bboo(self) -> I32Vector4: ...
    @property
    def bbol(self) -> I32Vector4: ...
    @property
    def bblr(self) -> I32Vector4: ...
    @property
    def bblg(self) -> I32Vector4: ...
    @property
    def bblb(self) -> I32Vector4: ...
    @property
    def bbla(self) -> I32Vector4: ...
    @property
    def bblo(self) -> I32Vector4: ...
    @property
    def bbll(self) -> I32Vector4: ...
    @property
    def barr(self) -> I32Vector4: ...
    @property
    def barg(self) -> I32Vector4: ...
    @property
    def barb(self) -> I32Vector4: ...
    @property
    def bara(self) -> I32Vector4: ...
    @property
    def baro(self) -> I32Vector4: ...
    @property
    def barl(self) -> I32Vector4: ...
    @property
    def bagr(self) -> I32Vector4: ...
    @property
    def bagg(self) -> I32Vector4: ...
    @property
    def bagb(self) -> I32Vector4: ...
    @property
    def baga(self) -> I32Vector4: ...
    @property
    def bago(self) -> I32Vector4: ...
    @property
    def bagl(self) -> I32Vector4: ...
    @property
    def babr(self) -> I32Vector4: ...
    @property
    def babg(self) -> I32Vector4: ...
    @property
    def babb(self) -> I32Vector4: ...
    @property
    def baba(self) -> I32Vector4: ...
    @property
    def babo(self) -> I32Vector4: ...
    @property
    def babl(self) -> I32Vector4: ...
    @property
    def baar(self) -> I32Vector4: ...
    @property
    def baag(self) -> I32Vector4: ...
    @property
    def baab(self) -> I32Vector4: ...
    @property
    def baaa(self) -> I32Vector4: ...
    @property
    def baao(self) -> I32Vector4: ...
    @property
    def baal(self) -> I32Vector4: ...
    @property
    def baor(self) -> I32Vector4: ...
    @property
    def baog(self) -> I32Vector4: ...
    @property
    def baob(self) -> I32Vector4: ...
    @property
    def baoa(self) -> I32Vector4: ...
    @property
    def baoo(self) -> I32Vector4: ...
    @property
    def baol(self) -> I32Vector4: ...
    @property
    def balr(self) -> I32Vector4: ...
    @property
    def balg(self) -> I32Vector4: ...
    @property
    def balb(self) -> I32Vector4: ...
    @property
    def bala(self) -> I32Vector4: ...
    @property
    def balo(self) -> I32Vector4: ...
    @property
    def ball(self) -> I32Vector4: ...
    @property
    def borr(self) -> I32Vector4: ...
    @property
    def borg(self) -> I32Vector4: ...
    @property
    def borb(self) -> I32Vector4: ...
    @property
    def bora(self) -> I32Vector4: ...
    @property
    def boro(self) -> I32Vector4: ...
    @property
    def borl(self) -> I32Vector4: ...
    @property
    def bogr(self) -> I32Vector4: ...
    @property
    def bogg(self) -> I32Vector4: ...
    @property
    def bogb(self) -> I32Vector4: ...
    @property
    def boga(self) -> I32Vector4: ...
    @property
    def bogo(self) -> I32Vector4: ...
    @property
    def bogl(self) -> I32Vector4: ...
    @property
    def bobr(self) -> I32Vector4: ...
    @property
    def bobg(self) -> I32Vector4: ...
    @property
    def bobb(self) -> I32Vector4: ...
    @property
    def boba(self) -> I32Vector4: ...
    @property
    def bobo(self) -> I32Vector4: ...
    @property
    def bobl(self) -> I32Vector4: ...
    @property
    def boar(self) -> I32Vector4: ...
    @property
    def boag(self) -> I32Vector4: ...
    @property
    def boab(self) -> I32Vector4: ...
    @property
    def boaa(self) -> I32Vector4: ...
    @property
    def boao(self) -> I32Vector4: ...
    @property
    def boal(self) -> I32Vector4: ...
    @property
    def boor(self) -> I32Vector4: ...
    @property
    def boog(self) -> I32Vector4: ...
    @property
    def boob(self) -> I32Vector4: ...
    @property
    def booa(self) -> I32Vector4: ...
    @property
    def booo(self) -> I32Vector4: ...
    @property
    def bool(self) -> I32Vector4: ...
    @property
    def bolr(self) -> I32Vector4: ...
    @property
    def bolg(self) -> I32Vector4: ...
    @property
    def bolb(self) -> I32Vector4: ...
    @property
    def bola(self) -> I32Vector4: ...
    @property
    def bolo(self) -> I32Vector4: ...
    @property
    def boll(self) -> I32Vector4: ...
    @property
    def blrr(self) -> I32Vector4: ...
    @property
    def blrg(self) -> I32Vector4: ...
    @property
    def blrb(self) -> I32Vector4: ...
    @property
    def blra(self) -> I32Vector4: ...
    @property
    def blro(self) -> I32Vector4: ...
    @property
    def blrl(self) -> I32Vector4: ...
    @property
    def blgr(self) -> I32Vector4: ...
    @property
    def blgg(self) -> I32Vector4: ...
    @property
    def blgb(self) -> I32Vector4: ...
    @property
    def blga(self) -> I32Vector4: ...
    @property
    def blgo(self) -> I32Vector4: ...
    @property
    def blgl(self) -> I32Vector4: ...
    @property
    def blbr(self) -> I32Vector4: ...
    @property
    def blbg(self) -> I32Vector4: ...
    @property
    def blbb(self) -> I32Vector4: ...
    @property
    def blba(self) -> I32Vector4: ...
    @property
    def blbo(self) -> I32Vector4: ...
    @property
    def blbl(self) -> I32Vector4: ...
    @property
    def blar(self) -> I32Vector4: ...
    @property
    def blag(self) -> I32Vector4: ...
    @property
    def blab(self) -> I32Vector4: ...
    @property
    def blaa(self) -> I32Vector4: ...
    @property
    def blao(self) -> I32Vector4: ...
    @property
    def blal(self) -> I32Vector4: ...
    @property
    def blor(self) -> I32Vector4: ...
    @property
    def blog(self) -> I32Vector4: ...
    @property
    def blob(self) -> I32Vector4: ...
    @property
    def bloa(self) -> I32Vector4: ...
    @property
    def bloo(self) -> I32Vector4: ...
    @property
    def blol(self) -> I32Vector4: ...
    @property
    def bllr(self) -> I32Vector4: ...
    @property
    def bllg(self) -> I32Vector4: ...
    @property
    def bllb(self) -> I32Vector4: ...
    @property
    def blla(self) -> I32Vector4: ...
    @property
    def bllo(self) -> I32Vector4: ...
    @property
    def blll(self) -> I32Vector4: ...
    @property
    def arrr(self) -> I32Vector4: ...
    @property
    def arrg(self) -> I32Vector4: ...
    @property
    def arrb(self) -> I32Vector4: ...
    @property
    def arra(self) -> I32Vector4: ...
    @property
    def arro(self) -> I32Vector4: ...
    @property
    def arrl(self) -> I32Vector4: ...
    @property
    def argr(self) -> I32Vector4: ...
    @property
    def argg(self) -> I32Vector4: ...
    @property
    def argb(self) -> I32Vector4: ...
    @property
    def arga(self) -> I32Vector4: ...
    @property
    def argo(self) -> I32Vector4: ...
    @property
    def argl(self) -> I32Vector4: ...
    @property
    def arbr(self) -> I32Vector4: ...
    @property
    def arbg(self) -> I32Vector4: ...
    @property
    def arbb(self) -> I32Vector4: ...
    @property
    def arba(self) -> I32Vector4: ...
    @property
    def arbo(self) -> I32Vector4: ...
    @property
    def arbl(self) -> I32Vector4: ...
    @property
    def arar(self) -> I32Vector4: ...
    @property
    def arag(self) -> I32Vector4: ...
    @property
    def arab(self) -> I32Vector4: ...
    @property
    def araa(self) -> I32Vector4: ...
    @property
    def arao(self) -> I32Vector4: ...
    @property
    def aral(self) -> I32Vector4: ...
    @property
    def aror(self) -> I32Vector4: ...
    @property
    def arog(self) -> I32Vector4: ...
    @property
    def arob(self) -> I32Vector4: ...
    @property
    def aroa(self) -> I32Vector4: ...
    @property
    def aroo(self) -> I32Vector4: ...
    @property
    def arol(self) -> I32Vector4: ...
    @property
    def arlr(self) -> I32Vector4: ...
    @property
    def arlg(self) -> I32Vector4: ...
    @property
    def arlb(self) -> I32Vector4: ...
    @property
    def arla(self) -> I32Vector4: ...
    @property
    def arlo(self) -> I32Vector4: ...
    @property
    def arll(self) -> I32Vector4: ...
    @property
    def agrr(self) -> I32Vector4: ...
    @property
    def agrg(self) -> I32Vector4: ...
    @property
    def agrb(self) -> I32Vector4: ...
    @property
    def agra(self) -> I32Vector4: ...
    @property
    def agro(self) -> I32Vector4: ...
    @property
    def agrl(self) -> I32Vector4: ...
    @property
    def aggr(self) -> I32Vector4: ...
    @property
    def aggg(self) -> I32Vector4: ...
    @property
    def aggb(self) -> I32Vector4: ...
    @property
    def agga(self) -> I32Vector4: ...
    @property
    def aggo(self) -> I32Vector4: ...
    @property
    def aggl(self) -> I32Vector4: ...
    @property
    def agbr(self) -> I32Vector4: ...
    @property
    def agbg(self) -> I32Vector4: ...
    @property
    def agbb(self) -> I32Vector4: ...
    @property
    def agba(self) -> I32Vector4: ...
    @property
    def agbo(self) -> I32Vector4: ...
    @property
    def agbl(self) -> I32Vector4: ...
    @property
    def agar(self) -> I32Vector4: ...
    @property
    def agag(self) -> I32Vector4: ...
    @property
    def agab(self) -> I32Vector4: ...
    @property
    def agaa(self) -> I32Vector4: ...
    @property
    def agao(self) -> I32Vector4: ...
    @property
    def agal(self) -> I32Vector4: ...
    @property
    def agor(self) -> I32Vector4: ...
    @property
    def agog(self) -> I32Vector4: ...
    @property
    def agob(self) -> I32Vector4: ...
    @property
    def agoa(self) -> I32Vector4: ...
    @property
    def agoo(self) -> I32Vector4: ...
    @property
    def agol(self) -> I32Vector4: ...
    @property
    def aglr(self) -> I32Vector4: ...
    @property
    def aglg(self) -> I32Vector4: ...
    @property
    def aglb(self) -> I32Vector4: ...
    @property
    def agla(self) -> I32Vector4: ...
    @property
    def aglo(self) -> I32Vector4: ...
    @property
    def agll(self) -> I32Vector4: ...
    @property
    def abrr(self) -> I32Vector4: ...
    @property
    def abrg(self) -> I32Vector4: ...
    @property
    def abrb(self) -> I32Vector4: ...
    @property
    def abra(self) -> I32Vector4: ...
    @property
    def abro(self) -> I32Vector4: ...
    @property
    def abrl(self) -> I32Vector4: ...
    @property
    def abgr(self) -> I32Vector4: ...
    @property
    def abgg(self) -> I32Vector4: ...
    @property
    def abgb(self) -> I32Vector4: ...
    @property
    def abga(self) -> I32Vector4: ...
    @property
    def abgo(self) -> I32Vector4: ...
    @property
    def abgl(self) -> I32Vector4: ...
    @property
    def abbr(self) -> I32Vector4: ...
    @property
    def abbg(self) -> I32Vector4: ...
    @property
    def abbb(self) -> I32Vector4: ...
    @property
    def abba(self) -> I32Vector4: ...
    @property
    def abbo(self) -> I32Vector4: ...
    @property
    def abbl(self) -> I32Vector4: ...
    @property
    def abar(self) -> I32Vector4: ...
    @property
    def abag(self) -> I32Vector4: ...
    @property
    def abab(self) -> I32Vector4: ...
    @property
    def abaa(self) -> I32Vector4: ...
    @property
    def abao(self) -> I32Vector4: ...
    @property
    def abal(self) -> I32Vector4: ...
    @property
    def abor(self) -> I32Vector4: ...
    @property
    def abog(self) -> I32Vector4: ...
    @property
    def abob(self) -> I32Vector4: ...
    @property
    def aboa(self) -> I32Vector4: ...
    @property
    def aboo(self) -> I32Vector4: ...
    @property
    def abol(self) -> I32Vector4: ...
    @property
    def ablr(self) -> I32Vector4: ...
    @property
    def ablg(self) -> I32Vector4: ...
    @property
    def ablb(self) -> I32Vector4: ...
    @property
    def abla(self) -> I32Vector4: ...
    @property
    def ablo(self) -> I32Vector4: ...
    @property
    def abll(self) -> I32Vector4: ...
    @property
    def aarr(self) -> I32Vector4: ...
    @property
    def aarg(self) -> I32Vector4: ...
    @property
    def aarb(self) -> I32Vector4: ...
    @property
    def aara(self) -> I32Vector4: ...
    @property
    def aaro(self) -> I32Vector4: ...
    @property
    def aarl(self) -> I32Vector4: ...
    @property
    def aagr(self) -> I32Vector4: ...
    @property
    def aagg(self) -> I32Vector4: ...
    @property
    def aagb(self) -> I32Vector4: ...
    @property
    def aaga(self) -> I32Vector4: ...
    @property
    def aago(self) -> I32Vector4: ...
    @property
    def aagl(self) -> I32Vector4: ...
    @property
    def aabr(self) -> I32Vector4: ...
    @property
    def aabg(self) -> I32Vector4: ...
    @property
    def aabb(self) -> I32Vector4: ...
    @property
    def aaba(self) -> I32Vector4: ...
    @property
    def aabo(self) -> I32Vector4: ...
    @property
    def aabl(self) -> I32Vector4: ...
    @property
    def aaar(self) -> I32Vector4: ...
    @property
    def aaag(self) -> I32Vector4: ...
    @property
    def aaab(self) -> I32Vector4: ...
    @property
    def aaaa(self) -> I32Vector4: ...
    @property
    def aaao(self) -> I32Vector4: ...
    @property
    def aaal(self) -> I32Vector4: ...
    @property
    def aaor(self) -> I32Vector4: ...
    @property
    def aaog(self) -> I32Vector4: ...
    @property
    def aaob(self) -> I32Vector4: ...
    @property
    def aaoa(self) -> I32Vector4: ...
    @property
    def aaoo(self) -> I32Vector4: ...
    @property
    def aaol(self) -> I32Vector4: ...
    @property
    def aalr(self) -> I32Vector4: ...
    @property
    def aalg(self) -> I32Vector4: ...
    @property
    def aalb(self) -> I32Vector4: ...
    @property
    def aala(self) -> I32Vector4: ...
    @property
    def aalo(self) -> I32Vector4: ...
    @property
    def aall(self) -> I32Vector4: ...
    @property
    def aorr(self) -> I32Vector4: ...
    @property
    def aorg(self) -> I32Vector4: ...
    @property
    def aorb(self) -> I32Vector4: ...
    @property
    def aora(self) -> I32Vector4: ...
    @property
    def aoro(self) -> I32Vector4: ...
    @property
    def aorl(self) -> I32Vector4: ...
    @property
    def aogr(self) -> I32Vector4: ...
    @property
    def aogg(self) -> I32Vector4: ...
    @property
    def aogb(self) -> I32Vector4: ...
    @property
    def aoga(self) -> I32Vector4: ...
    @property
    def aogo(self) -> I32Vector4: ...
    @property
    def aogl(self) -> I32Vector4: ...
    @property
    def aobr(self) -> I32Vector4: ...
    @property
    def aobg(self) -> I32Vector4: ...
    @property
    def aobb(self) -> I32Vector4: ...
    @property
    def aoba(self) -> I32Vector4: ...
    @property
    def aobo(self) -> I32Vector4: ...
    @property
    def aobl(self) -> I32Vector4: ...
    @property
    def aoar(self) -> I32Vector4: ...
    @property
    def aoag(self) -> I32Vector4: ...
    @property
    def aoab(self) -> I32Vector4: ...
    @property
    def aoaa(self) -> I32Vector4: ...
    @property
    def aoao(self) -> I32Vector4: ...
    @property
    def aoal(self) -> I32Vector4: ...
    @property
    def aoor(self) -> I32Vector4: ...
    @property
    def aoog(self) -> I32Vector4: ...
    @property
    def aoob(self) -> I32Vector4: ...
    @property
    def aooa(self) -> I32Vector4: ...
    @property
    def aooo(self) -> I32Vector4: ...
    @property
    def aool(self) -> I32Vector4: ...
    @property
    def aolr(self) -> I32Vector4: ...
    @property
    def aolg(self) -> I32Vector4: ...
    @property
    def aolb(self) -> I32Vector4: ...
    @property
    def aola(self) -> I32Vector4: ...
    @property
    def aolo(self) -> I32Vector4: ...
    @property
    def aoll(self) -> I32Vector4: ...
    @property
    def alrr(self) -> I32Vector4: ...
    @property
    def alrg(self) -> I32Vector4: ...
    @property
    def alrb(self) -> I32Vector4: ...
    @property
    def alra(self) -> I32Vector4: ...
    @property
    def alro(self) -> I32Vector4: ...
    @property
    def alrl(self) -> I32Vector4: ...
    @property
    def algr(self) -> I32Vector4: ...
    @property
    def algg(self) -> I32Vector4: ...
    @property
    def algb(self) -> I32Vector4: ...
    @property
    def alga(self) -> I32Vector4: ...
    @property
    def algo(self) -> I32Vector4: ...
    @property
    def algl(self) -> I32Vector4: ...
    @property
    def albr(self) -> I32Vector4: ...
    @property
    def albg(self) -> I32Vector4: ...
    @property
    def albb(self) -> I32Vector4: ...
    @property
    def alba(self) -> I32Vector4: ...
    @property
    def albo(self) -> I32Vector4: ...
    @property
    def albl(self) -> I32Vector4: ...
    @property
    def alar(self) -> I32Vector4: ...
    @property
    def alag(self) -> I32Vector4: ...
    @property
    def alab(self) -> I32Vector4: ...
    @property
    def alaa(self) -> I32Vector4: ...
    @property
    def alao(self) -> I32Vector4: ...
    @property
    def alal(self) -> I32Vector4: ...
    @property
    def alor(self) -> I32Vector4: ...
    @property
    def alog(self) -> I32Vector4: ...
    @property
    def alob(self) -> I32Vector4: ...
    @property
    def aloa(self) -> I32Vector4: ...
    @property
    def aloo(self) -> I32Vector4: ...
    @property
    def alol(self) -> I32Vector4: ...
    @property
    def allr(self) -> I32Vector4: ...
    @property
    def allg(self) -> I32Vector4: ...
    @property
    def allb(self) -> I32Vector4: ...
    @property
    def alla(self) -> I32Vector4: ...
    @property
    def allo(self) -> I32Vector4: ...
    @property
    def alll(self) -> I32Vector4: ...
    @property
    def orrr(self) -> I32Vector4: ...
    @property
    def orrg(self) -> I32Vector4: ...
    @property
    def orrb(self) -> I32Vector4: ...
    @property
    def orra(self) -> I32Vector4: ...
    @property
    def orro(self) -> I32Vector4: ...
    @property
    def orrl(self) -> I32Vector4: ...
    @property
    def orgr(self) -> I32Vector4: ...
    @property
    def orgg(self) -> I32Vector4: ...
    @property
    def orgb(self) -> I32Vector4: ...
    @property
    def orga(self) -> I32Vector4: ...
    @property
    def orgo(self) -> I32Vector4: ...
    @property
    def orgl(self) -> I32Vector4: ...
    @property
    def orbr(self) -> I32Vector4: ...
    @property
    def orbg(self) -> I32Vector4: ...
    @property
    def orbb(self) -> I32Vector4: ...
    @property
    def orba(self) -> I32Vector4: ...
    @property
    def orbo(self) -> I32Vector4: ...
    @property
    def orbl(self) -> I32Vector4: ...
    @property
    def orar(self) -> I32Vector4: ...
    @property
    def orag(self) -> I32Vector4: ...
    @property
    def orab(self) -> I32Vector4: ...
    @property
    def oraa(self) -> I32Vector4: ...
    @property
    def orao(self) -> I32Vector4: ...
    @property
    def oral(self) -> I32Vector4: ...
    @property
    def oror(self) -> I32Vector4: ...
    @property
    def orog(self) -> I32Vector4: ...
    @property
    def orob(self) -> I32Vector4: ...
    @property
    def oroa(self) -> I32Vector4: ...
    @property
    def oroo(self) -> I32Vector4: ...
    @property
    def orol(self) -> I32Vector4: ...
    @property
    def orlr(self) -> I32Vector4: ...
    @property
    def orlg(self) -> I32Vector4: ...
    @property
    def orlb(self) -> I32Vector4: ...
    @property
    def orla(self) -> I32Vector4: ...
    @property
    def orlo(self) -> I32Vector4: ...
    @property
    def orll(self) -> I32Vector4: ...
    @property
    def ogrr(self) -> I32Vector4: ...
    @property
    def ogrg(self) -> I32Vector4: ...
    @property
    def ogrb(self) -> I32Vector4: ...
    @property
    def ogra(self) -> I32Vector4: ...
    @property
    def ogro(self) -> I32Vector4: ...
    @property
    def ogrl(self) -> I32Vector4: ...
    @property
    def oggr(self) -> I32Vector4: ...
    @property
    def oggg(self) -> I32Vector4: ...
    @property
    def oggb(self) -> I32Vector4: ...
    @property
    def ogga(self) -> I32Vector4: ...
    @property
    def oggo(self) -> I32Vector4: ...
    @property
    def oggl(self) -> I32Vector4: ...
    @property
    def ogbr(self) -> I32Vector4: ...
    @property
    def ogbg(self) -> I32Vector4: ...
    @property
    def ogbb(self) -> I32Vector4: ...
    @property
    def ogba(self) -> I32Vector4: ...
    @property
    def ogbo(self) -> I32Vector4: ...
    @property
    def ogbl(self) -> I32Vector4: ...
    @property
    def ogar(self) -> I32Vector4: ...
    @property
    def ogag(self) -> I32Vector4: ...
    @property
    def ogab(self) -> I32Vector4: ...
    @property
    def ogaa(self) -> I32Vector4: ...
    @property
    def ogao(self) -> I32Vector4: ...
    @property
    def ogal(self) -> I32Vector4: ...
    @property
    def ogor(self) -> I32Vector4: ...
    @property
    def ogog(self) -> I32Vector4: ...
    @property
    def ogob(self) -> I32Vector4: ...
    @property
    def ogoa(self) -> I32Vector4: ...
    @property
    def ogoo(self) -> I32Vector4: ...
    @property
    def ogol(self) -> I32Vector4: ...
    @property
    def oglr(self) -> I32Vector4: ...
    @property
    def oglg(self) -> I32Vector4: ...
    @property
    def oglb(self) -> I32Vector4: ...
    @property
    def ogla(self) -> I32Vector4: ...
    @property
    def oglo(self) -> I32Vector4: ...
    @property
    def ogll(self) -> I32Vector4: ...
    @property
    def obrr(self) -> I32Vector4: ...
    @property
    def obrg(self) -> I32Vector4: ...
    @property
    def obrb(self) -> I32Vector4: ...
    @property
    def obra(self) -> I32Vector4: ...
    @property
    def obro(self) -> I32Vector4: ...
    @property
    def obrl(self) -> I32Vector4: ...
    @property
    def obgr(self) -> I32Vector4: ...
    @property
    def obgg(self) -> I32Vector4: ...
    @property
    def obgb(self) -> I32Vector4: ...
    @property
    def obga(self) -> I32Vector4: ...
    @property
    def obgo(self) -> I32Vector4: ...
    @property
    def obgl(self) -> I32Vector4: ...
    @property
    def obbr(self) -> I32Vector4: ...
    @property
    def obbg(self) -> I32Vector4: ...
    @property
    def obbb(self) -> I32Vector4: ...
    @property
    def obba(self) -> I32Vector4: ...
    @property
    def obbo(self) -> I32Vector4: ...
    @property
    def obbl(self) -> I32Vector4: ...
    @property
    def obar(self) -> I32Vector4: ...
    @property
    def obag(self) -> I32Vector4: ...
    @property
    def obab(self) -> I32Vector4: ...
    @property
    def obaa(self) -> I32Vector4: ...
    @property
    def obao(self) -> I32Vector4: ...
    @property
    def obal(self) -> I32Vector4: ...
    @property
    def obor(self) -> I32Vector4: ...
    @property
    def obog(self) -> I32Vector4: ...
    @property
    def obob(self) -> I32Vector4: ...
    @property
    def oboa(self) -> I32Vector4: ...
    @property
    def oboo(self) -> I32Vector4: ...
    @property
    def obol(self) -> I32Vector4: ...
    @property
    def oblr(self) -> I32Vector4: ...
    @property
    def oblg(self) -> I32Vector4: ...
    @property
    def oblb(self) -> I32Vector4: ...
    @property
    def obla(self) -> I32Vector4: ...
    @property
    def oblo(self) -> I32Vector4: ...
    @property
    def obll(self) -> I32Vector4: ...
    @property
    def oarr(self) -> I32Vector4: ...
    @property
    def oarg(self) -> I32Vector4: ...
    @property
    def oarb(self) -> I32Vector4: ...
    @property
    def oara(self) -> I32Vector4: ...
    @property
    def oaro(self) -> I32Vector4: ...
    @property
    def oarl(self) -> I32Vector4: ...
    @property
    def oagr(self) -> I32Vector4: ...
    @property
    def oagg(self) -> I32Vector4: ...
    @property
    def oagb(self) -> I32Vector4: ...
    @property
    def oaga(self) -> I32Vector4: ...
    @property
    def oago(self) -> I32Vector4: ...
    @property
    def oagl(self) -> I32Vector4: ...
    @property
    def oabr(self) -> I32Vector4: ...
    @property
    def oabg(self) -> I32Vector4: ...
    @property
    def oabb(self) -> I32Vector4: ...
    @property
    def oaba(self) -> I32Vector4: ...
    @property
    def oabo(self) -> I32Vector4: ...
    @property
    def oabl(self) -> I32Vector4: ...
    @property
    def oaar(self) -> I32Vector4: ...
    @property
    def oaag(self) -> I32Vector4: ...
    @property
    def oaab(self) -> I32Vector4: ...
    @property
    def oaaa(self) -> I32Vector4: ...
    @property
    def oaao(self) -> I32Vector4: ...
    @property
    def oaal(self) -> I32Vector4: ...
    @property
    def oaor(self) -> I32Vector4: ...
    @property
    def oaog(self) -> I32Vector4: ...
    @property
    def oaob(self) -> I32Vector4: ...
    @property
    def oaoa(self) -> I32Vector4: ...
    @property
    def oaoo(self) -> I32Vector4: ...
    @property
    def oaol(self) -> I32Vector4: ...
    @property
    def oalr(self) -> I32Vector4: ...
    @property
    def oalg(self) -> I32Vector4: ...
    @property
    def oalb(self) -> I32Vector4: ...
    @property
    def oala(self) -> I32Vector4: ...
    @property
    def oalo(self) -> I32Vector4: ...
    @property
    def oall(self) -> I32Vector4: ...
    @property
    def oorr(self) -> I32Vector4: ...
    @property
    def oorg(self) -> I32Vector4: ...
    @property
    def oorb(self) -> I32Vector4: ...
    @property
    def oora(self) -> I32Vector4: ...
    @property
    def ooro(self) -> I32Vector4: ...
    @property
    def oorl(self) -> I32Vector4: ...
    @property
    def oogr(self) -> I32Vector4: ...
    @property
    def oogg(self) -> I32Vector4: ...
    @property
    def oogb(self) -> I32Vector4: ...
    @property
    def ooga(self) -> I32Vector4: ...
    @property
    def oogo(self) -> I32Vector4: ...
    @property
    def oogl(self) -> I32Vector4: ...
    @property
    def oobr(self) -> I32Vector4: ...
    @property
    def oobg(self) -> I32Vector4: ...
    @property
    def oobb(self) -> I32Vector4: ...
    @property
    def ooba(self) -> I32Vector4: ...
    @property
    def oobo(self) -> I32Vector4: ...
    @property
    def oobl(self) -> I32Vector4: ...
    @property
    def ooar(self) -> I32Vector4: ...
    @property
    def ooag(self) -> I32Vector4: ...
    @property
    def ooab(self) -> I32Vector4: ...
    @property
    def ooaa(self) -> I32Vector4: ...
    @property
    def ooao(self) -> I32Vector4: ...
    @property
    def ooal(self) -> I32Vector4: ...
    @property
    def ooor(self) -> I32Vector4: ...
    @property
    def ooog(self) -> I32Vector4: ...
    @property
    def ooob(self) -> I32Vector4: ...
    @property
    def oooa(self) -> I32Vector4: ...
    @property
    def oolr(self) -> I32Vector4: ...
    @property
    def oolg(self) -> I32Vector4: ...
    @property
    def oolb(self) -> I32Vector4: ...
    @property
    def oola(self) -> I32Vector4: ...
    @property
    def olrr(self) -> I32Vector4: ...
    @property
    def olrg(self) -> I32Vector4: ...
    @property
    def olrb(self) -> I32Vector4: ...
    @property
    def olra(self) -> I32Vector4: ...
    @property
    def olro(self) -> I32Vector4: ...
    @property
    def olrl(self) -> I32Vector4: ...
    @property
    def olgr(self) -> I32Vector4: ...
    @property
    def olgg(self) -> I32Vector4: ...
    @property
    def olgb(self) -> I32Vector4: ...
    @property
    def olga(self) -> I32Vector4: ...
    @property
    def olgo(self) -> I32Vector4: ...
    @property
    def olgl(self) -> I32Vector4: ...
    @property
    def olbr(self) -> I32Vector4: ...
    @property
    def olbg(self) -> I32Vector4: ...
    @property
    def olbb(self) -> I32Vector4: ...
    @property
    def olba(self) -> I32Vector4: ...
    @property
    def olbo(self) -> I32Vector4: ...
    @property
    def olbl(self) -> I32Vector4: ...
    @property
    def olar(self) -> I32Vector4: ...
    @property
    def olag(self) -> I32Vector4: ...
    @property
    def olab(self) -> I32Vector4: ...
    @property
    def olaa(self) -> I32Vector4: ...
    @property
    def olao(self) -> I32Vector4: ...
    @property
    def olal(self) -> I32Vector4: ...
    @property
    def olor(self) -> I32Vector4: ...
    @property
    def olog(self) -> I32Vector4: ...
    @property
    def olob(self) -> I32Vector4: ...
    @property
    def oloa(self) -> I32Vector4: ...
    @property
    def ollr(self) -> I32Vector4: ...
    @property
    def ollg(self) -> I32Vector4: ...
    @property
    def ollb(self) -> I32Vector4: ...
    @property
    def olla(self) -> I32Vector4: ...
    @property
    def lrrr(self) -> I32Vector4: ...
    @property
    def lrrg(self) -> I32Vector4: ...
    @property
    def lrrb(self) -> I32Vector4: ...
    @property
    def lrra(self) -> I32Vector4: ...
    @property
    def lrro(self) -> I32Vector4: ...
    @property
    def lrrl(self) -> I32Vector4: ...
    @property
    def lrgr(self) -> I32Vector4: ...
    @property
    def lrgg(self) -> I32Vector4: ...
    @property
    def lrgb(self) -> I32Vector4: ...
    @property
    def lrga(self) -> I32Vector4: ...
    @property
    def lrgo(self) -> I32Vector4: ...
    @property
    def lrgl(self) -> I32Vector4: ...
    @property
    def lrbr(self) -> I32Vector4: ...
    @property
    def lrbg(self) -> I32Vector4: ...
    @property
    def lrbb(self) -> I32Vector4: ...
    @property
    def lrba(self) -> I32Vector4: ...
    @property
    def lrbo(self) -> I32Vector4: ...
    @property
    def lrbl(self) -> I32Vector4: ...
    @property
    def lrar(self) -> I32Vector4: ...
    @property
    def lrag(self) -> I32Vector4: ...
    @property
    def lrab(self) -> I32Vector4: ...
    @property
    def lraa(self) -> I32Vector4: ...
    @property
    def lrao(self) -> I32Vector4: ...
    @property
    def lral(self) -> I32Vector4: ...
    @property
    def lror(self) -> I32Vector4: ...
    @property
    def lrog(self) -> I32Vector4: ...
    @property
    def lrob(self) -> I32Vector4: ...
    @property
    def lroa(self) -> I32Vector4: ...
    @property
    def lroo(self) -> I32Vector4: ...
    @property
    def lrol(self) -> I32Vector4: ...
    @property
    def lrlr(self) -> I32Vector4: ...
    @property
    def lrlg(self) -> I32Vector4: ...
    @property
    def lrlb(self) -> I32Vector4: ...
    @property
    def lrla(self) -> I32Vector4: ...
    @property
    def lrlo(self) -> I32Vector4: ...
    @property
    def lrll(self) -> I32Vector4: ...
    @property
    def lgrr(self) -> I32Vector4: ...
    @property
    def lgrg(self) -> I32Vector4: ...
    @property
    def lgrb(self) -> I32Vector4: ...
    @property
    def lgra(self) -> I32Vector4: ...
    @property
    def lgro(self) -> I32Vector4: ...
    @property
    def lgrl(self) -> I32Vector4: ...
    @property
    def lggr(self) -> I32Vector4: ...
    @property
    def lggg(self) -> I32Vector4: ...
    @property
    def lggb(self) -> I32Vector4: ...
    @property
    def lgga(self) -> I32Vector4: ...
    @property
    def lggo(self) -> I32Vector4: ...
    @property
    def lggl(self) -> I32Vector4: ...
    @property
    def lgbr(self) -> I32Vector4: ...
    @property
    def lgbg(self) -> I32Vector4: ...
    @property
    def lgbb(self) -> I32Vector4: ...
    @property
    def lgba(self) -> I32Vector4: ...
    @property
    def lgbo(self) -> I32Vector4: ...
    @property
    def lgbl(self) -> I32Vector4: ...
    @property
    def lgar(self) -> I32Vector4: ...
    @property
    def lgag(self) -> I32Vector4: ...
    @property
    def lgab(self) -> I32Vector4: ...
    @property
    def lgaa(self) -> I32Vector4: ...
    @property
    def lgao(self) -> I32Vector4: ...
    @property
    def lgal(self) -> I32Vector4: ...
    @property
    def lgor(self) -> I32Vector4: ...
    @property
    def lgog(self) -> I32Vector4: ...
    @property
    def lgob(self) -> I32Vector4: ...
    @property
    def lgoa(self) -> I32Vector4: ...
    @property
    def lgoo(self) -> I32Vector4: ...
    @property
    def lgol(self) -> I32Vector4: ...
    @property
    def lglr(self) -> I32Vector4: ...
    @property
    def lglg(self) -> I32Vector4: ...
    @property
    def lglb(self) -> I32Vector4: ...
    @property
    def lgla(self) -> I32Vector4: ...
    @property
    def lglo(self) -> I32Vector4: ...
    @property
    def lgll(self) -> I32Vector4: ...
    @property
    def lbrr(self) -> I32Vector4: ...
    @property
    def lbrg(self) -> I32Vector4: ...
    @property
    def lbrb(self) -> I32Vector4: ...
    @property
    def lbra(self) -> I32Vector4: ...
    @property
    def lbro(self) -> I32Vector4: ...
    @property
    def lbrl(self) -> I32Vector4: ...
    @property
    def lbgr(self) -> I32Vector4: ...
    @property
    def lbgg(self) -> I32Vector4: ...
    @property
    def lbgb(self) -> I32Vector4: ...
    @property
    def lbga(self) -> I32Vector4: ...
    @property
    def lbgo(self) -> I32Vector4: ...
    @property
    def lbgl(self) -> I32Vector4: ...
    @property
    def lbbr(self) -> I32Vector4: ...
    @property
    def lbbg(self) -> I32Vector4: ...
    @property
    def lbbb(self) -> I32Vector4: ...
    @property
    def lbba(self) -> I32Vector4: ...
    @property
    def lbbo(self) -> I32Vector4: ...
    @property
    def lbbl(self) -> I32Vector4: ...
    @property
    def lbar(self) -> I32Vector4: ...
    @property
    def lbag(self) -> I32Vector4: ...
    @property
    def lbab(self) -> I32Vector4: ...
    @property
    def lbaa(self) -> I32Vector4: ...
    @property
    def lbao(self) -> I32Vector4: ...
    @property
    def lbal(self) -> I32Vector4: ...
    @property
    def lbor(self) -> I32Vector4: ...
    @property
    def lbog(self) -> I32Vector4: ...
    @property
    def lbob(self) -> I32Vector4: ...
    @property
    def lboa(self) -> I32Vector4: ...
    @property
    def lboo(self) -> I32Vector4: ...
    @property
    def lbol(self) -> I32Vector4: ...
    @property
    def lblr(self) -> I32Vector4: ...
    @property
    def lblg(self) -> I32Vector4: ...
    @property
    def lblb(self) -> I32Vector4: ...
    @property
    def lbla(self) -> I32Vector4: ...
    @property
    def lblo(self) -> I32Vector4: ...
    @property
    def lbll(self) -> I32Vector4: ...
    @property
    def larr(self) -> I32Vector4: ...
    @property
    def larg(self) -> I32Vector4: ...
    @property
    def larb(self) -> I32Vector4: ...
    @property
    def lara(self) -> I32Vector4: ...
    @property
    def laro(self) -> I32Vector4: ...
    @property
    def larl(self) -> I32Vector4: ...
    @property
    def lagr(self) -> I32Vector4: ...
    @property
    def lagg(self) -> I32Vector4: ...
    @property
    def lagb(self) -> I32Vector4: ...
    @property
    def laga(self) -> I32Vector4: ...
    @property
    def lago(self) -> I32Vector4: ...
    @property
    def lagl(self) -> I32Vector4: ...
    @property
    def labr(self) -> I32Vector4: ...
    @property
    def labg(self) -> I32Vector4: ...
    @property
    def labb(self) -> I32Vector4: ...
    @property
    def laba(self) -> I32Vector4: ...
    @property
    def labo(self) -> I32Vector4: ...
    @property
    def labl(self) -> I32Vector4: ...
    @property
    def laar(self) -> I32Vector4: ...
    @property
    def laag(self) -> I32Vector4: ...
    @property
    def laab(self) -> I32Vector4: ...
    @property
    def laaa(self) -> I32Vector4: ...
    @property
    def laao(self) -> I32Vector4: ...
    @property
    def laal(self) -> I32Vector4: ...
    @property
    def laor(self) -> I32Vector4: ...
    @property
    def laog(self) -> I32Vector4: ...
    @property
    def laob(self) -> I32Vector4: ...
    @property
    def laoa(self) -> I32Vector4: ...
    @property
    def laoo(self) -> I32Vector4: ...
    @property
    def laol(self) -> I32Vector4: ...
    @property
    def lalr(self) -> I32Vector4: ...
    @property
    def lalg(self) -> I32Vector4: ...
    @property
    def lalb(self) -> I32Vector4: ...
    @property
    def lala(self) -> I32Vector4: ...
    @property
    def lalo(self) -> I32Vector4: ...
    @property
    def lall(self) -> I32Vector4: ...
    @property
    def lorr(self) -> I32Vector4: ...
    @property
    def lorg(self) -> I32Vector4: ...
    @property
    def lorb(self) -> I32Vector4: ...
    @property
    def lora(self) -> I32Vector4: ...
    @property
    def loro(self) -> I32Vector4: ...
    @property
    def lorl(self) -> I32Vector4: ...
    @property
    def logr(self) -> I32Vector4: ...
    @property
    def logg(self) -> I32Vector4: ...
    @property
    def logb(self) -> I32Vector4: ...
    @property
    def loga(self) -> I32Vector4: ...
    @property
    def logo(self) -> I32Vector4: ...
    @property
    def logl(self) -> I32Vector4: ...
    @property
    def lobr(self) -> I32Vector4: ...
    @property
    def lobg(self) -> I32Vector4: ...
    @property
    def lobb(self) -> I32Vector4: ...
    @property
    def loba(self) -> I32Vector4: ...
    @property
    def lobo(self) -> I32Vector4: ...
    @property
    def lobl(self) -> I32Vector4: ...
    @property
    def loar(self) -> I32Vector4: ...
    @property
    def loag(self) -> I32Vector4: ...
    @property
    def loab(self) -> I32Vector4: ...
    @property
    def loaa(self) -> I32Vector4: ...
    @property
    def loao(self) -> I32Vector4: ...
    @property
    def loal(self) -> I32Vector4: ...
    @property
    def loor(self) -> I32Vector4: ...
    @property
    def loog(self) -> I32Vector4: ...
    @property
    def loob(self) -> I32Vector4: ...
    @property
    def looa(self) -> I32Vector4: ...
    @property
    def lolr(self) -> I32Vector4: ...
    @property
    def lolg(self) -> I32Vector4: ...
    @property
    def lolb(self) -> I32Vector4: ...
    @property
    def lola(self) -> I32Vector4: ...
    @property
    def llrr(self) -> I32Vector4: ...
    @property
    def llrg(self) -> I32Vector4: ...
    @property
    def llrb(self) -> I32Vector4: ...
    @property
    def llra(self) -> I32Vector4: ...
    @property
    def llro(self) -> I32Vector4: ...
    @property
    def llrl(self) -> I32Vector4: ...
    @property
    def llgr(self) -> I32Vector4: ...
    @property
    def llgg(self) -> I32Vector4: ...
    @property
    def llgb(self) -> I32Vector4: ...
    @property
    def llga(self) -> I32Vector4: ...
    @property
    def llgo(self) -> I32Vector4: ...
    @property
    def llgl(self) -> I32Vector4: ...
    @property
    def llbr(self) -> I32Vector4: ...
    @property
    def llbg(self) -> I32Vector4: ...
    @property
    def llbb(self) -> I32Vector4: ...
    @property
    def llba(self) -> I32Vector4: ...
    @property
    def llbo(self) -> I32Vector4: ...
    @property
    def llbl(self) -> I32Vector4: ...
    @property
    def llar(self) -> I32Vector4: ...
    @property
    def llag(self) -> I32Vector4: ...
    @property
    def llab(self) -> I32Vector4: ...
    @property
    def llaa(self) -> I32Vector4: ...
    @property
    def llao(self) -> I32Vector4: ...
    @property
    def llal(self) -> I32Vector4: ...
    @property
    def llor(self) -> I32Vector4: ...
    @property
    def llog(self) -> I32Vector4: ...
    @property
    def llob(self) -> I32Vector4: ...
    @property
    def lloa(self) -> I32Vector4: ...
    @property
    def lllr(self) -> I32Vector4: ...
    @property
    def lllg(self) -> I32Vector4: ...
    @property
    def lllb(self) -> I32Vector4: ...
    @property
    def llla(self) -> I32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> I32Vector2: ...
    @property
    def st(self) -> I32Vector2: ...
    @property
    def sq(self) -> I32Vector2: ...
    @property
    def sp(self) -> I32Vector2: ...
    @property
    def so(self) -> I32Vector2: ...
    @property
    def sl(self) -> I32Vector2: ...
    @property
    def ts(self) -> I32Vector2: ...
    @property
    def tt(self) -> I32Vector2: ...
    @property
    def tq(self) -> I32Vector2: ...
    @property
    def tp(self) -> I32Vector2: ...
    @property
    def to(self) -> I32Vector2: ...
    @property
    def tl(self) -> I32Vector2: ...
    @property
    def qs(self) -> I32Vector2: ...
    @property
    def qt(self) -> I32Vector2: ...
    @property
    def qq(self) -> I32Vector2: ...
    @property
    def qp(self) -> I32Vector2: ...
    @property
    def qo(self) -> I32Vector2: ...
    @property
    def ql(self) -> I32Vector2: ...
    @property
    def ps(self) -> I32Vector2: ...
    @property
    def pt(self) -> I32Vector2: ...
    @property
    def pq(self) -> I32Vector2: ...
    @property
    def pp(self) -> I32Vector2: ...
    @property
    def po(self) -> I32Vector2: ...
    @property
    def pl(self) -> I32Vector2: ...
    @property
    def os(self) -> I32Vector2: ...
    @property
    def ot(self) -> I32Vector2: ...
    @property
    def oq(self) -> I32Vector2: ...
    @property
    def op(self) -> I32Vector2: ...
    @property
    def ls(self) -> I32Vector2: ...
    @property
    def lt(self) -> I32Vector2: ...
    @property
    def lq(self) -> I32Vector2: ...
    @property
    def lp(self) -> I32Vector2: ...
    @property
    def sss(self) -> I32Vector3: ...
    @property
    def sst(self) -> I32Vector3: ...
    @property
    def ssq(self) -> I32Vector3: ...
    @property
    def ssp(self) -> I32Vector3: ...
    @property
    def sso(self) -> I32Vector3: ...
    @property
    def ssl(self) -> I32Vector3: ...
    @property
    def sts(self) -> I32Vector3: ...
    @property
    def stt(self) -> I32Vector3: ...
    @property
    def stq(self) -> I32Vector3: ...
    @property
    def stp(self) -> I32Vector3: ...
    @property
    def sto(self) -> I32Vector3: ...
    @property
    def stl(self) -> I32Vector3: ...
    @property
    def sqs(self) -> I32Vector3: ...
    @property
    def sqt(self) -> I32Vector3: ...
    @property
    def sqq(self) -> I32Vector3: ...
    @property
    def sqp(self) -> I32Vector3: ...
    @property
    def sqo(self) -> I32Vector3: ...
    @property
    def sql(self) -> I32Vector3: ...
    @property
    def sps(self) -> I32Vector3: ...
    @property
    def spt(self) -> I32Vector3: ...
    @property
    def spq(self) -> I32Vector3: ...
    @property
    def spp(self) -> I32Vector3: ...
    @property
    def spo(self) -> I32Vector3: ...
    @property
    def spl(self) -> I32Vector3: ...
    @property
    def sos(self) -> I32Vector3: ...
    @property
    def sot(self) -> I32Vector3: ...
    @property
    def soq(self) -> I32Vector3: ...
    @property
    def sop(self) -> I32Vector3: ...
    @property
    def soo(self) -> I32Vector3: ...
    @property
    def sol(self) -> I32Vector3: ...
    @property
    def sls(self) -> I32Vector3: ...
    @property
    def slt(self) -> I32Vector3: ...
    @property
    def slq(self) -> I32Vector3: ...
    @property
    def slp(self) -> I32Vector3: ...
    @property
    def slo(self) -> I32Vector3: ...
    @property
    def sll(self) -> I32Vector3: ...
    @property
    def tss(self) -> I32Vector3: ...
    @property
    def tst(self) -> I32Vector3: ...
    @property
    def tsq(self) -> I32Vector3: ...
    @property
    def tsp(self) -> I32Vector3: ...
    @property
    def tso(self) -> I32Vector3: ...
    @property
    def tsl(self) -> I32Vector3: ...
    @property
    def tts(self) -> I32Vector3: ...
    @property
    def ttt(self) -> I32Vector3: ...
    @property
    def ttq(self) -> I32Vector3: ...
    @property
    def ttp(self) -> I32Vector3: ...
    @property
    def tto(self) -> I32Vector3: ...
    @property
    def ttl(self) -> I32Vector3: ...
    @property
    def tqs(self) -> I32Vector3: ...
    @property
    def tqt(self) -> I32Vector3: ...
    @property
    def tqq(self) -> I32Vector3: ...
    @property
    def tqp(self) -> I32Vector3: ...
    @property
    def tqo(self) -> I32Vector3: ...
    @property
    def tql(self) -> I32Vector3: ...
    @property
    def tps(self) -> I32Vector3: ...
    @property
    def tpt(self) -> I32Vector3: ...
    @property
    def tpq(self) -> I32Vector3: ...
    @property
    def tpp(self) -> I32Vector3: ...
    @property
    def tpo(self) -> I32Vector3: ...
    @property
    def tpl(self) -> I32Vector3: ...
    @property
    def tos(self) -> I32Vector3: ...
    @property
    def tot(self) -> I32Vector3: ...
    @property
    def toq(self) -> I32Vector3: ...
    @property
    def top(self) -> I32Vector3: ...
    @property
    def too(self) -> I32Vector3: ...
    @property
    def tol(self) -> I32Vector3: ...
    @property
    def tls(self) -> I32Vector3: ...
    @property
    def tlt(self) -> I32Vector3: ...
    @property
    def tlq(self) -> I32Vector3: ...
    @property
    def tlp(self) -> I32Vector3: ...
    @property
    def tlo(self) -> I32Vector3: ...
    @property
    def tll(self) -> I32Vector3: ...
    @property
    def qss(self) -> I32Vector3: ...
    @property
    def qst(self) -> I32Vector3: ...
    @property
    def qsq(self) -> I32Vector3: ...
    @property
    def qsp(self) -> I32Vector3: ...
    @property
    def qso(self) -> I32Vector3: ...
    @property
    def qsl(self) -> I32Vector3: ...
    @property
    def qts(self) -> I32Vector3: ...
    @property
    def qtt(self) -> I32Vector3: ...
    @property
    def qtq(self) -> I32Vector3: ...
    @property
    def qtp(self) -> I32Vector3: ...
    @property
    def qto(self) -> I32Vector3: ...
    @property
    def qtl(self) -> I32Vector3: ...
    @property
    def qqs(self) -> I32Vector3: ...
    @property
    def qqt(self) -> I32Vector3: ...
    @property
    def qqq(self) -> I32Vector3: ...
    @property
    def qqp(self) -> I32Vector3: ...
    @property
    def qqo(self) -> I32Vector3: ...
    @property
    def qql(self) -> I32Vector3: ...
    @property
    def qps(self) -> I32Vector3: ...
    @property
    def qpt(self) -> I32Vector3: ...
    @property
    def qpq(self) -> I32Vector3: ...
    @property
    def qpp(self) -> I32Vector3: ...
    @property
    def qpo(self) -> I32Vector3: ...
    @property
    def qpl(self) -> I32Vector3: ...
    @property
    def qos(self) -> I32Vector3: ...
    @property
    def qot(self) -> I32Vector3: ...
    @property
    def qoq(self) -> I32Vector3: ...
    @property
    def qop(self) -> I32Vector3: ...
    @property
    def qoo(self) -> I32Vector3: ...
    @property
    def qol(self) -> I32Vector3: ...
    @property
    def qls(self) -> I32Vector3: ...
    @property
    def qlt(self) -> I32Vector3: ...
    @property
    def qlq(self) -> I32Vector3: ...
    @property
    def qlp(self) -> I32Vector3: ...
    @property
    def qlo(self) -> I32Vector3: ...
    @property
    def qll(self) -> I32Vector3: ...
    @property
    def pss(self) -> I32Vector3: ...
    @property
    def pst(self) -> I32Vector3: ...
    @property
    def psq(self) -> I32Vector3: ...
    @property
    def psp(self) -> I32Vector3: ...
    @property
    def pso(self) -> I32Vector3: ...
    @property
    def psl(self) -> I32Vector3: ...
    @property
    def pts(self) -> I32Vector3: ...
    @property
    def ptt(self) -> I32Vector3: ...
    @property
    def ptq(self) -> I32Vector3: ...
    @property
    def ptp(self) -> I32Vector3: ...
    @property
    def pto(self) -> I32Vector3: ...
    @property
    def ptl(self) -> I32Vector3: ...
    @property
    def pqs(self) -> I32Vector3: ...
    @property
    def pqt(self) -> I32Vector3: ...
    @property
    def pqq(self) -> I32Vector3: ...
    @property
    def pqp(self) -> I32Vector3: ...
    @property
    def pqo(self) -> I32Vector3: ...
    @property
    def pql(self) -> I32Vector3: ...
    @property
    def pps(self) -> I32Vector3: ...
    @property
    def ppt(self) -> I32Vector3: ...
    @property
    def ppq(self) -> I32Vector3: ...
    @property
    def ppp(self) -> I32Vector3: ...
    @property
    def ppo(self) -> I32Vector3: ...
    @property
    def ppl(self) -> I32Vector3: ...
    @property
    def pos(self) -> I32Vector3: ...
    @property
    def pot(self) -> I32Vector3: ...
    @property
    def poq(self) -> I32Vector3: ...
    @property
    def pop(self) -> I32Vector3: ...
    @property
    def poo(self) -> I32Vector3: ...
    @property
    def pol(self) -> I32Vector3: ...
    @property
    def pls(self) -> I32Vector3: ...
    @property
    def plt(self) -> I32Vector3: ...
    @property
    def plq(self) -> I32Vector3: ...
    @property
    def plp(self) -> I32Vector3: ...
    @property
    def plo(self) -> I32Vector3: ...
    @property
    def pll(self) -> I32Vector3: ...
    @property
    def oss(self) -> I32Vector3: ...
    @property
    def ost(self) -> I32Vector3: ...
    @property
    def osq(self) -> I32Vector3: ...
    @property
    def osp(self) -> I32Vector3: ...
    @property
    def oso(self) -> I32Vector3: ...
    @property
    def osl(self) -> I32Vector3: ...
    @property
    def ots(self) -> I32Vector3: ...
    @property
    def ott(self) -> I32Vector3: ...
    @property
    def otq(self) -> I32Vector3: ...
    @property
    def otp(self) -> I32Vector3: ...
    @property
    def oto(self) -> I32Vector3: ...
    @property
    def otl(self) -> I32Vector3: ...
    @property
    def oqs(self) -> I32Vector3: ...
    @property
    def oqt(self) -> I32Vector3: ...
    @property
    def oqq(self) -> I32Vector3: ...
    @property
    def oqp(self) -> I32Vector3: ...
    @property
    def oqo(self) -> I32Vector3: ...
    @property
    def oql(self) -> I32Vector3: ...
    @property
    def ops(self) -> I32Vector3: ...
    @property
    def opt(self) -> I32Vector3: ...
    @property
    def opq(self) -> I32Vector3: ...
    @property
    def opp(self) -> I32Vector3: ...
    @property
    def opo(self) -> I32Vector3: ...
    @property
    def opl(self) -> I32Vector3: ...
    @property
    def oos(self) -> I32Vector3: ...
    @property
    def oot(self) -> I32Vector3: ...
    @property
    def ooq(self) -> I32Vector3: ...
    @property
    def oop(self) -> I32Vector3: ...
    @property
    def ols(self) -> I32Vector3: ...
    @property
    def olt(self) -> I32Vector3: ...
    @property
    def olq(self) -> I32Vector3: ...
    @property
    def olp(self) -> I32Vector3: ...
    @property
    def lss(self) -> I32Vector3: ...
    @property
    def lst(self) -> I32Vector3: ...
    @property
    def lsq(self) -> I32Vector3: ...
    @property
    def lsp(self) -> I32Vector3: ...
    @property
    def lso(self) -> I32Vector3: ...
    @property
    def lsl(self) -> I32Vector3: ...
    @property
    def lts(self) -> I32Vector3: ...
    @property
    def ltt(self) -> I32Vector3: ...
    @property
    def ltq(self) -> I32Vector3: ...
    @property
    def ltp(self) -> I32Vector3: ...
    @property
    def lto(self) -> I32Vector3: ...
    @property
    def ltl(self) -> I32Vector3: ...
    @property
    def lqs(self) -> I32Vector3: ...
    @property
    def lqt(self) -> I32Vector3: ...
    @property
    def lqq(self) -> I32Vector3: ...
    @property
    def lqp(self) -> I32Vector3: ...
    @property
    def lqo(self) -> I32Vector3: ...
    @property
    def lql(self) -> I32Vector3: ...
    @property
    def lps(self) -> I32Vector3: ...
    @property
    def lpt(self) -> I32Vector3: ...
    @property
    def lpq(self) -> I32Vector3: ...
    @property
    def lpp(self) -> I32Vector3: ...
    @property
    def lpo(self) -> I32Vector3: ...
    @property
    def lpl(self) -> I32Vector3: ...
    @property
    def los(self) -> I32Vector3: ...
    @property
    def lot(self) -> I32Vector3: ...
    @property
    def loq(self) -> I32Vector3: ...
    @property
    def lop(self) -> I32Vector3: ...
    @property
    def lls(self) -> I32Vector3: ...
    @property
    def llt(self) -> I32Vector3: ...
    @property
    def llq(self) -> I32Vector3: ...
    @property
    def llp(self) -> I32Vector3: ...
    @property
    def ssss(self) -> I32Vector4: ...
    @property
    def ssst(self) -> I32Vector4: ...
    @property
    def sssq(self) -> I32Vector4: ...
    @property
    def sssp(self) -> I32Vector4: ...
    @property
    def ssso(self) -> I32Vector4: ...
    @property
    def sssl(self) -> I32Vector4: ...
    @property
    def ssts(self) -> I32Vector4: ...
    @property
    def sstt(self) -> I32Vector4: ...
    @property
    def sstq(self) -> I32Vector4: ...
    @property
    def sstp(self) -> I32Vector4: ...
    @property
    def ssto(self) -> I32Vector4: ...
    @property
    def sstl(self) -> I32Vector4: ...
    @property
    def ssqs(self) -> I32Vector4: ...
    @property
    def ssqt(self) -> I32Vector4: ...
    @property
    def ssqq(self) -> I32Vector4: ...
    @property
    def ssqp(self) -> I32Vector4: ...
    @property
    def ssqo(self) -> I32Vector4: ...
    @property
    def ssql(self) -> I32Vector4: ...
    @property
    def ssps(self) -> I32Vector4: ...
    @property
    def sspt(self) -> I32Vector4: ...
    @property
    def sspq(self) -> I32Vector4: ...
    @property
    def sspp(self) -> I32Vector4: ...
    @property
    def sspo(self) -> I32Vector4: ...
    @property
    def sspl(self) -> I32Vector4: ...
    @property
    def ssos(self) -> I32Vector4: ...
    @property
    def ssot(self) -> I32Vector4: ...
    @property
    def ssoq(self) -> I32Vector4: ...
    @property
    def ssop(self) -> I32Vector4: ...
    @property
    def ssoo(self) -> I32Vector4: ...
    @property
    def ssol(self) -> I32Vector4: ...
    @property
    def ssls(self) -> I32Vector4: ...
    @property
    def sslt(self) -> I32Vector4: ...
    @property
    def sslq(self) -> I32Vector4: ...
    @property
    def sslp(self) -> I32Vector4: ...
    @property
    def sslo(self) -> I32Vector4: ...
    @property
    def ssll(self) -> I32Vector4: ...
    @property
    def stss(self) -> I32Vector4: ...
    @property
    def stst(self) -> I32Vector4: ...
    @property
    def stsq(self) -> I32Vector4: ...
    @property
    def stsp(self) -> I32Vector4: ...
    @property
    def stso(self) -> I32Vector4: ...
    @property
    def stsl(self) -> I32Vector4: ...
    @property
    def stts(self) -> I32Vector4: ...
    @property
    def sttt(self) -> I32Vector4: ...
    @property
    def sttq(self) -> I32Vector4: ...
    @property
    def sttp(self) -> I32Vector4: ...
    @property
    def stto(self) -> I32Vector4: ...
    @property
    def sttl(self) -> I32Vector4: ...
    @property
    def stqs(self) -> I32Vector4: ...
    @property
    def stqt(self) -> I32Vector4: ...
    @property
    def stqq(self) -> I32Vector4: ...
    @property
    def stqp(self) -> I32Vector4: ...
    @property
    def stqo(self) -> I32Vector4: ...
    @property
    def stql(self) -> I32Vector4: ...
    @property
    def stps(self) -> I32Vector4: ...
    @property
    def stpt(self) -> I32Vector4: ...
    @property
    def stpq(self) -> I32Vector4: ...
    @property
    def stpp(self) -> I32Vector4: ...
    @property
    def stpo(self) -> I32Vector4: ...
    @property
    def stpl(self) -> I32Vector4: ...
    @property
    def stos(self) -> I32Vector4: ...
    @property
    def stot(self) -> I32Vector4: ...
    @property
    def stoq(self) -> I32Vector4: ...
    @property
    def stop(self) -> I32Vector4: ...
    @property
    def stoo(self) -> I32Vector4: ...
    @property
    def stol(self) -> I32Vector4: ...
    @property
    def stls(self) -> I32Vector4: ...
    @property
    def stlt(self) -> I32Vector4: ...
    @property
    def stlq(self) -> I32Vector4: ...
    @property
    def stlp(self) -> I32Vector4: ...
    @property
    def stlo(self) -> I32Vector4: ...
    @property
    def stll(self) -> I32Vector4: ...
    @property
    def sqss(self) -> I32Vector4: ...
    @property
    def sqst(self) -> I32Vector4: ...
    @property
    def sqsq(self) -> I32Vector4: ...
    @property
    def sqsp(self) -> I32Vector4: ...
    @property
    def sqso(self) -> I32Vector4: ...
    @property
    def sqsl(self) -> I32Vector4: ...
    @property
    def sqts(self) -> I32Vector4: ...
    @property
    def sqtt(self) -> I32Vector4: ...
    @property
    def sqtq(self) -> I32Vector4: ...
    @property
    def sqtp(self) -> I32Vector4: ...
    @property
    def sqto(self) -> I32Vector4: ...
    @property
    def sqtl(self) -> I32Vector4: ...
    @property
    def sqqs(self) -> I32Vector4: ...
    @property
    def sqqt(self) -> I32Vector4: ...
    @property
    def sqqq(self) -> I32Vector4: ...
    @property
    def sqqp(self) -> I32Vector4: ...
    @property
    def sqqo(self) -> I32Vector4: ...
    @property
    def sqql(self) -> I32Vector4: ...
    @property
    def sqps(self) -> I32Vector4: ...
    @property
    def sqpt(self) -> I32Vector4: ...
    @property
    def sqpq(self) -> I32Vector4: ...
    @property
    def sqpp(self) -> I32Vector4: ...
    @property
    def sqpo(self) -> I32Vector4: ...
    @property
    def sqpl(self) -> I32Vector4: ...
    @property
    def sqos(self) -> I32Vector4: ...
    @property
    def sqot(self) -> I32Vector4: ...
    @property
    def sqoq(self) -> I32Vector4: ...
    @property
    def sqop(self) -> I32Vector4: ...
    @property
    def sqoo(self) -> I32Vector4: ...
    @property
    def sqol(self) -> I32Vector4: ...
    @property
    def sqls(self) -> I32Vector4: ...
    @property
    def sqlt(self) -> I32Vector4: ...
    @property
    def sqlq(self) -> I32Vector4: ...
    @property
    def sqlp(self) -> I32Vector4: ...
    @property
    def sqlo(self) -> I32Vector4: ...
    @property
    def sqll(self) -> I32Vector4: ...
    @property
    def spss(self) -> I32Vector4: ...
    @property
    def spst(self) -> I32Vector4: ...
    @property
    def spsq(self) -> I32Vector4: ...
    @property
    def spsp(self) -> I32Vector4: ...
    @property
    def spso(self) -> I32Vector4: ...
    @property
    def spsl(self) -> I32Vector4: ...
    @property
    def spts(self) -> I32Vector4: ...
    @property
    def sptt(self) -> I32Vector4: ...
    @property
    def sptq(self) -> I32Vector4: ...
    @property
    def sptp(self) -> I32Vector4: ...
    @property
    def spto(self) -> I32Vector4: ...
    @property
    def sptl(self) -> I32Vector4: ...
    @property
    def spqs(self) -> I32Vector4: ...
    @property
    def spqt(self) -> I32Vector4: ...
    @property
    def spqq(self) -> I32Vector4: ...
    @property
    def spqp(self) -> I32Vector4: ...
    @property
    def spqo(self) -> I32Vector4: ...
    @property
    def spql(self) -> I32Vector4: ...
    @property
    def spps(self) -> I32Vector4: ...
    @property
    def sppt(self) -> I32Vector4: ...
    @property
    def sppq(self) -> I32Vector4: ...
    @property
    def sppp(self) -> I32Vector4: ...
    @property
    def sppo(self) -> I32Vector4: ...
    @property
    def sppl(self) -> I32Vector4: ...
    @property
    def spos(self) -> I32Vector4: ...
    @property
    def spot(self) -> I32Vector4: ...
    @property
    def spoq(self) -> I32Vector4: ...
    @property
    def spop(self) -> I32Vector4: ...
    @property
    def spoo(self) -> I32Vector4: ...
    @property
    def spol(self) -> I32Vector4: ...
    @property
    def spls(self) -> I32Vector4: ...
    @property
    def splt(self) -> I32Vector4: ...
    @property
    def splq(self) -> I32Vector4: ...
    @property
    def splp(self) -> I32Vector4: ...
    @property
    def splo(self) -> I32Vector4: ...
    @property
    def spll(self) -> I32Vector4: ...
    @property
    def soss(self) -> I32Vector4: ...
    @property
    def sost(self) -> I32Vector4: ...
    @property
    def sosq(self) -> I32Vector4: ...
    @property
    def sosp(self) -> I32Vector4: ...
    @property
    def soso(self) -> I32Vector4: ...
    @property
    def sosl(self) -> I32Vector4: ...
    @property
    def sots(self) -> I32Vector4: ...
    @property
    def sott(self) -> I32Vector4: ...
    @property
    def sotq(self) -> I32Vector4: ...
    @property
    def sotp(self) -> I32Vector4: ...
    @property
    def soto(self) -> I32Vector4: ...
    @property
    def sotl(self) -> I32Vector4: ...
    @property
    def soqs(self) -> I32Vector4: ...
    @property
    def soqt(self) -> I32Vector4: ...
    @property
    def soqq(self) -> I32Vector4: ...
    @property
    def soqp(self) -> I32Vector4: ...
    @property
    def soqo(self) -> I32Vector4: ...
    @property
    def soql(self) -> I32Vector4: ...
    @property
    def sops(self) -> I32Vector4: ...
    @property
    def sopt(self) -> I32Vector4: ...
    @property
    def sopq(self) -> I32Vector4: ...
    @property
    def sopp(self) -> I32Vector4: ...
    @property
    def sopo(self) -> I32Vector4: ...
    @property
    def sopl(self) -> I32Vector4: ...
    @property
    def soos(self) -> I32Vector4: ...
    @property
    def soot(self) -> I32Vector4: ...
    @property
    def sooq(self) -> I32Vector4: ...
    @property
    def soop(self) -> I32Vector4: ...
    @property
    def sooo(self) -> I32Vector4: ...
    @property
    def sool(self) -> I32Vector4: ...
    @property
    def sols(self) -> I32Vector4: ...
    @property
    def solt(self) -> I32Vector4: ...
    @property
    def solq(self) -> I32Vector4: ...
    @property
    def solp(self) -> I32Vector4: ...
    @property
    def solo(self) -> I32Vector4: ...
    @property
    def soll(self) -> I32Vector4: ...
    @property
    def slss(self) -> I32Vector4: ...
    @property
    def slst(self) -> I32Vector4: ...
    @property
    def slsq(self) -> I32Vector4: ...
    @property
    def slsp(self) -> I32Vector4: ...
    @property
    def slso(self) -> I32Vector4: ...
    @property
    def slsl(self) -> I32Vector4: ...
    @property
    def slts(self) -> I32Vector4: ...
    @property
    def sltt(self) -> I32Vector4: ...
    @property
    def sltq(self) -> I32Vector4: ...
    @property
    def sltp(self) -> I32Vector4: ...
    @property
    def slto(self) -> I32Vector4: ...
    @property
    def sltl(self) -> I32Vector4: ...
    @property
    def slqs(self) -> I32Vector4: ...
    @property
    def slqt(self) -> I32Vector4: ...
    @property
    def slqq(self) -> I32Vector4: ...
    @property
    def slqp(self) -> I32Vector4: ...
    @property
    def slqo(self) -> I32Vector4: ...
    @property
    def slql(self) -> I32Vector4: ...
    @property
    def slps(self) -> I32Vector4: ...
    @property
    def slpt(self) -> I32Vector4: ...
    @property
    def slpq(self) -> I32Vector4: ...
    @property
    def slpp(self) -> I32Vector4: ...
    @property
    def slpo(self) -> I32Vector4: ...
    @property
    def slpl(self) -> I32Vector4: ...
    @property
    def slos(self) -> I32Vector4: ...
    @property
    def slot(self) -> I32Vector4: ...
    @property
    def sloq(self) -> I32Vector4: ...
    @property
    def slop(self) -> I32Vector4: ...
    @property
    def sloo(self) -> I32Vector4: ...
    @property
    def slol(self) -> I32Vector4: ...
    @property
    def slls(self) -> I32Vector4: ...
    @property
    def sllt(self) -> I32Vector4: ...
    @property
    def sllq(self) -> I32Vector4: ...
    @property
    def sllp(self) -> I32Vector4: ...
    @property
    def sllo(self) -> I32Vector4: ...
    @property
    def slll(self) -> I32Vector4: ...
    @property
    def tsss(self) -> I32Vector4: ...
    @property
    def tsst(self) -> I32Vector4: ...
    @property
    def tssq(self) -> I32Vector4: ...
    @property
    def tssp(self) -> I32Vector4: ...
    @property
    def tsso(self) -> I32Vector4: ...
    @property
    def tssl(self) -> I32Vector4: ...
    @property
    def tsts(self) -> I32Vector4: ...
    @property
    def tstt(self) -> I32Vector4: ...
    @property
    def tstq(self) -> I32Vector4: ...
    @property
    def tstp(self) -> I32Vector4: ...
    @property
    def tsto(self) -> I32Vector4: ...
    @property
    def tstl(self) -> I32Vector4: ...
    @property
    def tsqs(self) -> I32Vector4: ...
    @property
    def tsqt(self) -> I32Vector4: ...
    @property
    def tsqq(self) -> I32Vector4: ...
    @property
    def tsqp(self) -> I32Vector4: ...
    @property
    def tsqo(self) -> I32Vector4: ...
    @property
    def tsql(self) -> I32Vector4: ...
    @property
    def tsps(self) -> I32Vector4: ...
    @property
    def tspt(self) -> I32Vector4: ...
    @property
    def tspq(self) -> I32Vector4: ...
    @property
    def tspp(self) -> I32Vector4: ...
    @property
    def tspo(self) -> I32Vector4: ...
    @property
    def tspl(self) -> I32Vector4: ...
    @property
    def tsos(self) -> I32Vector4: ...
    @property
    def tsot(self) -> I32Vector4: ...
    @property
    def tsoq(self) -> I32Vector4: ...
    @property
    def tsop(self) -> I32Vector4: ...
    @property
    def tsoo(self) -> I32Vector4: ...
    @property
    def tsol(self) -> I32Vector4: ...
    @property
    def tsls(self) -> I32Vector4: ...
    @property
    def tslt(self) -> I32Vector4: ...
    @property
    def tslq(self) -> I32Vector4: ...
    @property
    def tslp(self) -> I32Vector4: ...
    @property
    def tslo(self) -> I32Vector4: ...
    @property
    def tsll(self) -> I32Vector4: ...
    @property
    def ttss(self) -> I32Vector4: ...
    @property
    def ttst(self) -> I32Vector4: ...
    @property
    def ttsq(self) -> I32Vector4: ...
    @property
    def ttsp(self) -> I32Vector4: ...
    @property
    def ttso(self) -> I32Vector4: ...
    @property
    def ttsl(self) -> I32Vector4: ...
    @property
    def ttts(self) -> I32Vector4: ...
    @property
    def tttt(self) -> I32Vector4: ...
    @property
    def tttq(self) -> I32Vector4: ...
    @property
    def tttp(self) -> I32Vector4: ...
    @property
    def ttto(self) -> I32Vector4: ...
    @property
    def tttl(self) -> I32Vector4: ...
    @property
    def ttqs(self) -> I32Vector4: ...
    @property
    def ttqt(self) -> I32Vector4: ...
    @property
    def ttqq(self) -> I32Vector4: ...
    @property
    def ttqp(self) -> I32Vector4: ...
    @property
    def ttqo(self) -> I32Vector4: ...
    @property
    def ttql(self) -> I32Vector4: ...
    @property
    def ttps(self) -> I32Vector4: ...
    @property
    def ttpt(self) -> I32Vector4: ...
    @property
    def ttpq(self) -> I32Vector4: ...
    @property
    def ttpp(self) -> I32Vector4: ...
    @property
    def ttpo(self) -> I32Vector4: ...
    @property
    def ttpl(self) -> I32Vector4: ...
    @property
    def ttos(self) -> I32Vector4: ...
    @property
    def ttot(self) -> I32Vector4: ...
    @property
    def ttoq(self) -> I32Vector4: ...
    @property
    def ttop(self) -> I32Vector4: ...
    @property
    def ttoo(self) -> I32Vector4: ...
    @property
    def ttol(self) -> I32Vector4: ...
    @property
    def ttls(self) -> I32Vector4: ...
    @property
    def ttlt(self) -> I32Vector4: ...
    @property
    def ttlq(self) -> I32Vector4: ...
    @property
    def ttlp(self) -> I32Vector4: ...
    @property
    def ttlo(self) -> I32Vector4: ...
    @property
    def ttll(self) -> I32Vector4: ...
    @property
    def tqss(self) -> I32Vector4: ...
    @property
    def tqst(self) -> I32Vector4: ...
    @property
    def tqsq(self) -> I32Vector4: ...
    @property
    def tqsp(self) -> I32Vector4: ...
    @property
    def tqso(self) -> I32Vector4: ...
    @property
    def tqsl(self) -> I32Vector4: ...
    @property
    def tqts(self) -> I32Vector4: ...
    @property
    def tqtt(self) -> I32Vector4: ...
    @property
    def tqtq(self) -> I32Vector4: ...
    @property
    def tqtp(self) -> I32Vector4: ...
    @property
    def tqto(self) -> I32Vector4: ...
    @property
    def tqtl(self) -> I32Vector4: ...
    @property
    def tqqs(self) -> I32Vector4: ...
    @property
    def tqqt(self) -> I32Vector4: ...
    @property
    def tqqq(self) -> I32Vector4: ...
    @property
    def tqqp(self) -> I32Vector4: ...
    @property
    def tqqo(self) -> I32Vector4: ...
    @property
    def tqql(self) -> I32Vector4: ...
    @property
    def tqps(self) -> I32Vector4: ...
    @property
    def tqpt(self) -> I32Vector4: ...
    @property
    def tqpq(self) -> I32Vector4: ...
    @property
    def tqpp(self) -> I32Vector4: ...
    @property
    def tqpo(self) -> I32Vector4: ...
    @property
    def tqpl(self) -> I32Vector4: ...
    @property
    def tqos(self) -> I32Vector4: ...
    @property
    def tqot(self) -> I32Vector4: ...
    @property
    def tqoq(self) -> I32Vector4: ...
    @property
    def tqop(self) -> I32Vector4: ...
    @property
    def tqoo(self) -> I32Vector4: ...
    @property
    def tqol(self) -> I32Vector4: ...
    @property
    def tqls(self) -> I32Vector4: ...
    @property
    def tqlt(self) -> I32Vector4: ...
    @property
    def tqlq(self) -> I32Vector4: ...
    @property
    def tqlp(self) -> I32Vector4: ...
    @property
    def tqlo(self) -> I32Vector4: ...
    @property
    def tqll(self) -> I32Vector4: ...
    @property
    def tpss(self) -> I32Vector4: ...
    @property
    def tpst(self) -> I32Vector4: ...
    @property
    def tpsq(self) -> I32Vector4: ...
    @property
    def tpsp(self) -> I32Vector4: ...
    @property
    def tpso(self) -> I32Vector4: ...
    @property
    def tpsl(self) -> I32Vector4: ...
    @property
    def tpts(self) -> I32Vector4: ...
    @property
    def tptt(self) -> I32Vector4: ...
    @property
    def tptq(self) -> I32Vector4: ...
    @property
    def tptp(self) -> I32Vector4: ...
    @property
    def tpto(self) -> I32Vector4: ...
    @property
    def tptl(self) -> I32Vector4: ...
    @property
    def tpqs(self) -> I32Vector4: ...
    @property
    def tpqt(self) -> I32Vector4: ...
    @property
    def tpqq(self) -> I32Vector4: ...
    @property
    def tpqp(self) -> I32Vector4: ...
    @property
    def tpqo(self) -> I32Vector4: ...
    @property
    def tpql(self) -> I32Vector4: ...
    @property
    def tpps(self) -> I32Vector4: ...
    @property
    def tppt(self) -> I32Vector4: ...
    @property
    def tppq(self) -> I32Vector4: ...
    @property
    def tppp(self) -> I32Vector4: ...
    @property
    def tppo(self) -> I32Vector4: ...
    @property
    def tppl(self) -> I32Vector4: ...
    @property
    def tpos(self) -> I32Vector4: ...
    @property
    def tpot(self) -> I32Vector4: ...
    @property
    def tpoq(self) -> I32Vector4: ...
    @property
    def tpop(self) -> I32Vector4: ...
    @property
    def tpoo(self) -> I32Vector4: ...
    @property
    def tpol(self) -> I32Vector4: ...
    @property
    def tpls(self) -> I32Vector4: ...
    @property
    def tplt(self) -> I32Vector4: ...
    @property
    def tplq(self) -> I32Vector4: ...
    @property
    def tplp(self) -> I32Vector4: ...
    @property
    def tplo(self) -> I32Vector4: ...
    @property
    def tpll(self) -> I32Vector4: ...
    @property
    def toss(self) -> I32Vector4: ...
    @property
    def tost(self) -> I32Vector4: ...
    @property
    def tosq(self) -> I32Vector4: ...
    @property
    def tosp(self) -> I32Vector4: ...
    @property
    def toso(self) -> I32Vector4: ...
    @property
    def tosl(self) -> I32Vector4: ...
    @property
    def tots(self) -> I32Vector4: ...
    @property
    def tott(self) -> I32Vector4: ...
    @property
    def totq(self) -> I32Vector4: ...
    @property
    def totp(self) -> I32Vector4: ...
    @property
    def toto(self) -> I32Vector4: ...
    @property
    def totl(self) -> I32Vector4: ...
    @property
    def toqs(self) -> I32Vector4: ...
    @property
    def toqt(self) -> I32Vector4: ...
    @property
    def toqq(self) -> I32Vector4: ...
    @property
    def toqp(self) -> I32Vector4: ...
    @property
    def toqo(self) -> I32Vector4: ...
    @property
    def toql(self) -> I32Vector4: ...
    @property
    def tops(self) -> I32Vector4: ...
    @property
    def topt(self) -> I32Vector4: ...
    @property
    def topq(self) -> I32Vector4: ...
    @property
    def topp(self) -> I32Vector4: ...
    @property
    def topo(self) -> I32Vector4: ...
    @property
    def topl(self) -> I32Vector4: ...
    @property
    def toos(self) -> I32Vector4: ...
    @property
    def toot(self) -> I32Vector4: ...
    @property
    def tooq(self) -> I32Vector4: ...
    @property
    def toop(self) -> I32Vector4: ...
    @property
    def tooo(self) -> I32Vector4: ...
    @property
    def tool(self) -> I32Vector4: ...
    @property
    def tols(self) -> I32Vector4: ...
    @property
    def tolt(self) -> I32Vector4: ...
    @property
    def tolq(self) -> I32Vector4: ...
    @property
    def tolp(self) -> I32Vector4: ...
    @property
    def tolo(self) -> I32Vector4: ...
    @property
    def toll(self) -> I32Vector4: ...
    @property
    def tlss(self) -> I32Vector4: ...
    @property
    def tlst(self) -> I32Vector4: ...
    @property
    def tlsq(self) -> I32Vector4: ...
    @property
    def tlsp(self) -> I32Vector4: ...
    @property
    def tlso(self) -> I32Vector4: ...
    @property
    def tlsl(self) -> I32Vector4: ...
    @property
    def tlts(self) -> I32Vector4: ...
    @property
    def tltt(self) -> I32Vector4: ...
    @property
    def tltq(self) -> I32Vector4: ...
    @property
    def tltp(self) -> I32Vector4: ...
    @property
    def tlto(self) -> I32Vector4: ...
    @property
    def tltl(self) -> I32Vector4: ...
    @property
    def tlqs(self) -> I32Vector4: ...
    @property
    def tlqt(self) -> I32Vector4: ...
    @property
    def tlqq(self) -> I32Vector4: ...
    @property
    def tlqp(self) -> I32Vector4: ...
    @property
    def tlqo(self) -> I32Vector4: ...
    @property
    def tlql(self) -> I32Vector4: ...
    @property
    def tlps(self) -> I32Vector4: ...
    @property
    def tlpt(self) -> I32Vector4: ...
    @property
    def tlpq(self) -> I32Vector4: ...
    @property
    def tlpp(self) -> I32Vector4: ...
    @property
    def tlpo(self) -> I32Vector4: ...
    @property
    def tlpl(self) -> I32Vector4: ...
    @property
    def tlos(self) -> I32Vector4: ...
    @property
    def tlot(self) -> I32Vector4: ...
    @property
    def tloq(self) -> I32Vector4: ...
    @property
    def tlop(self) -> I32Vector4: ...
    @property
    def tloo(self) -> I32Vector4: ...
    @property
    def tlol(self) -> I32Vector4: ...
    @property
    def tlls(self) -> I32Vector4: ...
    @property
    def tllt(self) -> I32Vector4: ...
    @property
    def tllq(self) -> I32Vector4: ...
    @property
    def tllp(self) -> I32Vector4: ...
    @property
    def tllo(self) -> I32Vector4: ...
    @property
    def tlll(self) -> I32Vector4: ...
    @property
    def qsss(self) -> I32Vector4: ...
    @property
    def qsst(self) -> I32Vector4: ...
    @property
    def qssq(self) -> I32Vector4: ...
    @property
    def qssp(self) -> I32Vector4: ...
    @property
    def qsso(self) -> I32Vector4: ...
    @property
    def qssl(self) -> I32Vector4: ...
    @property
    def qsts(self) -> I32Vector4: ...
    @property
    def qstt(self) -> I32Vector4: ...
    @property
    def qstq(self) -> I32Vector4: ...
    @property
    def qstp(self) -> I32Vector4: ...
    @property
    def qsto(self) -> I32Vector4: ...
    @property
    def qstl(self) -> I32Vector4: ...
    @property
    def qsqs(self) -> I32Vector4: ...
    @property
    def qsqt(self) -> I32Vector4: ...
    @property
    def qsqq(self) -> I32Vector4: ...
    @property
    def qsqp(self) -> I32Vector4: ...
    @property
    def qsqo(self) -> I32Vector4: ...
    @property
    def qsql(self) -> I32Vector4: ...
    @property
    def qsps(self) -> I32Vector4: ...
    @property
    def qspt(self) -> I32Vector4: ...
    @property
    def qspq(self) -> I32Vector4: ...
    @property
    def qspp(self) -> I32Vector4: ...
    @property
    def qspo(self) -> I32Vector4: ...
    @property
    def qspl(self) -> I32Vector4: ...
    @property
    def qsos(self) -> I32Vector4: ...
    @property
    def qsot(self) -> I32Vector4: ...
    @property
    def qsoq(self) -> I32Vector4: ...
    @property
    def qsop(self) -> I32Vector4: ...
    @property
    def qsoo(self) -> I32Vector4: ...
    @property
    def qsol(self) -> I32Vector4: ...
    @property
    def qsls(self) -> I32Vector4: ...
    @property
    def qslt(self) -> I32Vector4: ...
    @property
    def qslq(self) -> I32Vector4: ...
    @property
    def qslp(self) -> I32Vector4: ...
    @property
    def qslo(self) -> I32Vector4: ...
    @property
    def qsll(self) -> I32Vector4: ...
    @property
    def qtss(self) -> I32Vector4: ...
    @property
    def qtst(self) -> I32Vector4: ...
    @property
    def qtsq(self) -> I32Vector4: ...
    @property
    def qtsp(self) -> I32Vector4: ...
    @property
    def qtso(self) -> I32Vector4: ...
    @property
    def qtsl(self) -> I32Vector4: ...
    @property
    def qtts(self) -> I32Vector4: ...
    @property
    def qttt(self) -> I32Vector4: ...
    @property
    def qttq(self) -> I32Vector4: ...
    @property
    def qttp(self) -> I32Vector4: ...
    @property
    def qtto(self) -> I32Vector4: ...
    @property
    def qttl(self) -> I32Vector4: ...
    @property
    def qtqs(self) -> I32Vector4: ...
    @property
    def qtqt(self) -> I32Vector4: ...
    @property
    def qtqq(self) -> I32Vector4: ...
    @property
    def qtqp(self) -> I32Vector4: ...
    @property
    def qtqo(self) -> I32Vector4: ...
    @property
    def qtql(self) -> I32Vector4: ...
    @property
    def qtps(self) -> I32Vector4: ...
    @property
    def qtpt(self) -> I32Vector4: ...
    @property
    def qtpq(self) -> I32Vector4: ...
    @property
    def qtpp(self) -> I32Vector4: ...
    @property
    def qtpo(self) -> I32Vector4: ...
    @property
    def qtpl(self) -> I32Vector4: ...
    @property
    def qtos(self) -> I32Vector4: ...
    @property
    def qtot(self) -> I32Vector4: ...
    @property
    def qtoq(self) -> I32Vector4: ...
    @property
    def qtop(self) -> I32Vector4: ...
    @property
    def qtoo(self) -> I32Vector4: ...
    @property
    def qtol(self) -> I32Vector4: ...
    @property
    def qtls(self) -> I32Vector4: ...
    @property
    def qtlt(self) -> I32Vector4: ...
    @property
    def qtlq(self) -> I32Vector4: ...
    @property
    def qtlp(self) -> I32Vector4: ...
    @property
    def qtlo(self) -> I32Vector4: ...
    @property
    def qtll(self) -> I32Vector4: ...
    @property
    def qqss(self) -> I32Vector4: ...
    @property
    def qqst(self) -> I32Vector4: ...
    @property
    def qqsq(self) -> I32Vector4: ...
    @property
    def qqsp(self) -> I32Vector4: ...
    @property
    def qqso(self) -> I32Vector4: ...
    @property
    def qqsl(self) -> I32Vector4: ...
    @property
    def qqts(self) -> I32Vector4: ...
    @property
    def qqtt(self) -> I32Vector4: ...
    @property
    def qqtq(self) -> I32Vector4: ...
    @property
    def qqtp(self) -> I32Vector4: ...
    @property
    def qqto(self) -> I32Vector4: ...
    @property
    def qqtl(self) -> I32Vector4: ...
    @property
    def qqqs(self) -> I32Vector4: ...
    @property
    def qqqt(self) -> I32Vector4: ...
    @property
    def qqqq(self) -> I32Vector4: ...
    @property
    def qqqp(self) -> I32Vector4: ...
    @property
    def qqqo(self) -> I32Vector4: ...
    @property
    def qqql(self) -> I32Vector4: ...
    @property
    def qqps(self) -> I32Vector4: ...
    @property
    def qqpt(self) -> I32Vector4: ...
    @property
    def qqpq(self) -> I32Vector4: ...
    @property
    def qqpp(self) -> I32Vector4: ...
    @property
    def qqpo(self) -> I32Vector4: ...
    @property
    def qqpl(self) -> I32Vector4: ...
    @property
    def qqos(self) -> I32Vector4: ...
    @property
    def qqot(self) -> I32Vector4: ...
    @property
    def qqoq(self) -> I32Vector4: ...
    @property
    def qqop(self) -> I32Vector4: ...
    @property
    def qqoo(self) -> I32Vector4: ...
    @property
    def qqol(self) -> I32Vector4: ...
    @property
    def qqls(self) -> I32Vector4: ...
    @property
    def qqlt(self) -> I32Vector4: ...
    @property
    def qqlq(self) -> I32Vector4: ...
    @property
    def qqlp(self) -> I32Vector4: ...
    @property
    def qqlo(self) -> I32Vector4: ...
    @property
    def qqll(self) -> I32Vector4: ...
    @property
    def qpss(self) -> I32Vector4: ...
    @property
    def qpst(self) -> I32Vector4: ...
    @property
    def qpsq(self) -> I32Vector4: ...
    @property
    def qpsp(self) -> I32Vector4: ...
    @property
    def qpso(self) -> I32Vector4: ...
    @property
    def qpsl(self) -> I32Vector4: ...
    @property
    def qpts(self) -> I32Vector4: ...
    @property
    def qptt(self) -> I32Vector4: ...
    @property
    def qptq(self) -> I32Vector4: ...
    @property
    def qptp(self) -> I32Vector4: ...
    @property
    def qpto(self) -> I32Vector4: ...
    @property
    def qptl(self) -> I32Vector4: ...
    @property
    def qpqs(self) -> I32Vector4: ...
    @property
    def qpqt(self) -> I32Vector4: ...
    @property
    def qpqq(self) -> I32Vector4: ...
    @property
    def qpqp(self) -> I32Vector4: ...
    @property
    def qpqo(self) -> I32Vector4: ...
    @property
    def qpql(self) -> I32Vector4: ...
    @property
    def qpps(self) -> I32Vector4: ...
    @property
    def qppt(self) -> I32Vector4: ...
    @property
    def qppq(self) -> I32Vector4: ...
    @property
    def qppp(self) -> I32Vector4: ...
    @property
    def qppo(self) -> I32Vector4: ...
    @property
    def qppl(self) -> I32Vector4: ...
    @property
    def qpos(self) -> I32Vector4: ...
    @property
    def qpot(self) -> I32Vector4: ...
    @property
    def qpoq(self) -> I32Vector4: ...
    @property
    def qpop(self) -> I32Vector4: ...
    @property
    def qpoo(self) -> I32Vector4: ...
    @property
    def qpol(self) -> I32Vector4: ...
    @property
    def qpls(self) -> I32Vector4: ...
    @property
    def qplt(self) -> I32Vector4: ...
    @property
    def qplq(self) -> I32Vector4: ...
    @property
    def qplp(self) -> I32Vector4: ...
    @property
    def qplo(self) -> I32Vector4: ...
    @property
    def qpll(self) -> I32Vector4: ...
    @property
    def qoss(self) -> I32Vector4: ...
    @property
    def qost(self) -> I32Vector4: ...
    @property
    def qosq(self) -> I32Vector4: ...
    @property
    def qosp(self) -> I32Vector4: ...
    @property
    def qoso(self) -> I32Vector4: ...
    @property
    def qosl(self) -> I32Vector4: ...
    @property
    def qots(self) -> I32Vector4: ...
    @property
    def qott(self) -> I32Vector4: ...
    @property
    def qotq(self) -> I32Vector4: ...
    @property
    def qotp(self) -> I32Vector4: ...
    @property
    def qoto(self) -> I32Vector4: ...
    @property
    def qotl(self) -> I32Vector4: ...
    @property
    def qoqs(self) -> I32Vector4: ...
    @property
    def qoqt(self) -> I32Vector4: ...
    @property
    def qoqq(self) -> I32Vector4: ...
    @property
    def qoqp(self) -> I32Vector4: ...
    @property
    def qoqo(self) -> I32Vector4: ...
    @property
    def qoql(self) -> I32Vector4: ...
    @property
    def qops(self) -> I32Vector4: ...
    @property
    def qopt(self) -> I32Vector4: ...
    @property
    def qopq(self) -> I32Vector4: ...
    @property
    def qopp(self) -> I32Vector4: ...
    @property
    def qopo(self) -> I32Vector4: ...
    @property
    def qopl(self) -> I32Vector4: ...
    @property
    def qoos(self) -> I32Vector4: ...
    @property
    def qoot(self) -> I32Vector4: ...
    @property
    def qooq(self) -> I32Vector4: ...
    @property
    def qoop(self) -> I32Vector4: ...
    @property
    def qooo(self) -> I32Vector4: ...
    @property
    def qool(self) -> I32Vector4: ...
    @property
    def qols(self) -> I32Vector4: ...
    @property
    def qolt(self) -> I32Vector4: ...
    @property
    def qolq(self) -> I32Vector4: ...
    @property
    def qolp(self) -> I32Vector4: ...
    @property
    def qolo(self) -> I32Vector4: ...
    @property
    def qoll(self) -> I32Vector4: ...
    @property
    def qlss(self) -> I32Vector4: ...
    @property
    def qlst(self) -> I32Vector4: ...
    @property
    def qlsq(self) -> I32Vector4: ...
    @property
    def qlsp(self) -> I32Vector4: ...
    @property
    def qlso(self) -> I32Vector4: ...
    @property
    def qlsl(self) -> I32Vector4: ...
    @property
    def qlts(self) -> I32Vector4: ...
    @property
    def qltt(self) -> I32Vector4: ...
    @property
    def qltq(self) -> I32Vector4: ...
    @property
    def qltp(self) -> I32Vector4: ...
    @property
    def qlto(self) -> I32Vector4: ...
    @property
    def qltl(self) -> I32Vector4: ...
    @property
    def qlqs(self) -> I32Vector4: ...
    @property
    def qlqt(self) -> I32Vector4: ...
    @property
    def qlqq(self) -> I32Vector4: ...
    @property
    def qlqp(self) -> I32Vector4: ...
    @property
    def qlqo(self) -> I32Vector4: ...
    @property
    def qlql(self) -> I32Vector4: ...
    @property
    def qlps(self) -> I32Vector4: ...
    @property
    def qlpt(self) -> I32Vector4: ...
    @property
    def qlpq(self) -> I32Vector4: ...
    @property
    def qlpp(self) -> I32Vector4: ...
    @property
    def qlpo(self) -> I32Vector4: ...
    @property
    def qlpl(self) -> I32Vector4: ...
    @property
    def qlos(self) -> I32Vector4: ...
    @property
    def qlot(self) -> I32Vector4: ...
    @property
    def qloq(self) -> I32Vector4: ...
    @property
    def qlop(self) -> I32Vector4: ...
    @property
    def qloo(self) -> I32Vector4: ...
    @property
    def qlol(self) -> I32Vector4: ...
    @property
    def qlls(self) -> I32Vector4: ...
    @property
    def qllt(self) -> I32Vector4: ...
    @property
    def qllq(self) -> I32Vector4: ...
    @property
    def qllp(self) -> I32Vector4: ...
    @property
    def qllo(self) -> I32Vector4: ...
    @property
    def qlll(self) -> I32Vector4: ...
    @property
    def psss(self) -> I32Vector4: ...
    @property
    def psst(self) -> I32Vector4: ...
    @property
    def pssq(self) -> I32Vector4: ...
    @property
    def pssp(self) -> I32Vector4: ...
    @property
    def psso(self) -> I32Vector4: ...
    @property
    def pssl(self) -> I32Vector4: ...
    @property
    def psts(self) -> I32Vector4: ...
    @property
    def pstt(self) -> I32Vector4: ...
    @property
    def pstq(self) -> I32Vector4: ...
    @property
    def pstp(self) -> I32Vector4: ...
    @property
    def psto(self) -> I32Vector4: ...
    @property
    def pstl(self) -> I32Vector4: ...
    @property
    def psqs(self) -> I32Vector4: ...
    @property
    def psqt(self) -> I32Vector4: ...
    @property
    def psqq(self) -> I32Vector4: ...
    @property
    def psqp(self) -> I32Vector4: ...
    @property
    def psqo(self) -> I32Vector4: ...
    @property
    def psql(self) -> I32Vector4: ...
    @property
    def psps(self) -> I32Vector4: ...
    @property
    def pspt(self) -> I32Vector4: ...
    @property
    def pspq(self) -> I32Vector4: ...
    @property
    def pspp(self) -> I32Vector4: ...
    @property
    def pspo(self) -> I32Vector4: ...
    @property
    def pspl(self) -> I32Vector4: ...
    @property
    def psos(self) -> I32Vector4: ...
    @property
    def psot(self) -> I32Vector4: ...
    @property
    def psoq(self) -> I32Vector4: ...
    @property
    def psop(self) -> I32Vector4: ...
    @property
    def psoo(self) -> I32Vector4: ...
    @property
    def psol(self) -> I32Vector4: ...
    @property
    def psls(self) -> I32Vector4: ...
    @property
    def pslt(self) -> I32Vector4: ...
    @property
    def pslq(self) -> I32Vector4: ...
    @property
    def pslp(self) -> I32Vector4: ...
    @property
    def pslo(self) -> I32Vector4: ...
    @property
    def psll(self) -> I32Vector4: ...
    @property
    def ptss(self) -> I32Vector4: ...
    @property
    def ptst(self) -> I32Vector4: ...
    @property
    def ptsq(self) -> I32Vector4: ...
    @property
    def ptsp(self) -> I32Vector4: ...
    @property
    def ptso(self) -> I32Vector4: ...
    @property
    def ptsl(self) -> I32Vector4: ...
    @property
    def ptts(self) -> I32Vector4: ...
    @property
    def pttt(self) -> I32Vector4: ...
    @property
    def pttq(self) -> I32Vector4: ...
    @property
    def pttp(self) -> I32Vector4: ...
    @property
    def ptto(self) -> I32Vector4: ...
    @property
    def pttl(self) -> I32Vector4: ...
    @property
    def ptqs(self) -> I32Vector4: ...
    @property
    def ptqt(self) -> I32Vector4: ...
    @property
    def ptqq(self) -> I32Vector4: ...
    @property
    def ptqp(self) -> I32Vector4: ...
    @property
    def ptqo(self) -> I32Vector4: ...
    @property
    def ptql(self) -> I32Vector4: ...
    @property
    def ptps(self) -> I32Vector4: ...
    @property
    def ptpt(self) -> I32Vector4: ...
    @property
    def ptpq(self) -> I32Vector4: ...
    @property
    def ptpp(self) -> I32Vector4: ...
    @property
    def ptpo(self) -> I32Vector4: ...
    @property
    def ptpl(self) -> I32Vector4: ...
    @property
    def ptos(self) -> I32Vector4: ...
    @property
    def ptot(self) -> I32Vector4: ...
    @property
    def ptoq(self) -> I32Vector4: ...
    @property
    def ptop(self) -> I32Vector4: ...
    @property
    def ptoo(self) -> I32Vector4: ...
    @property
    def ptol(self) -> I32Vector4: ...
    @property
    def ptls(self) -> I32Vector4: ...
    @property
    def ptlt(self) -> I32Vector4: ...
    @property
    def ptlq(self) -> I32Vector4: ...
    @property
    def ptlp(self) -> I32Vector4: ...
    @property
    def ptlo(self) -> I32Vector4: ...
    @property
    def ptll(self) -> I32Vector4: ...
    @property
    def pqss(self) -> I32Vector4: ...
    @property
    def pqst(self) -> I32Vector4: ...
    @property
    def pqsq(self) -> I32Vector4: ...
    @property
    def pqsp(self) -> I32Vector4: ...
    @property
    def pqso(self) -> I32Vector4: ...
    @property
    def pqsl(self) -> I32Vector4: ...
    @property
    def pqts(self) -> I32Vector4: ...
    @property
    def pqtt(self) -> I32Vector4: ...
    @property
    def pqtq(self) -> I32Vector4: ...
    @property
    def pqtp(self) -> I32Vector4: ...
    @property
    def pqto(self) -> I32Vector4: ...
    @property
    def pqtl(self) -> I32Vector4: ...
    @property
    def pqqs(self) -> I32Vector4: ...
    @property
    def pqqt(self) -> I32Vector4: ...
    @property
    def pqqq(self) -> I32Vector4: ...
    @property
    def pqqp(self) -> I32Vector4: ...
    @property
    def pqqo(self) -> I32Vector4: ...
    @property
    def pqql(self) -> I32Vector4: ...
    @property
    def pqps(self) -> I32Vector4: ...
    @property
    def pqpt(self) -> I32Vector4: ...
    @property
    def pqpq(self) -> I32Vector4: ...
    @property
    def pqpp(self) -> I32Vector4: ...
    @property
    def pqpo(self) -> I32Vector4: ...
    @property
    def pqpl(self) -> I32Vector4: ...
    @property
    def pqos(self) -> I32Vector4: ...
    @property
    def pqot(self) -> I32Vector4: ...
    @property
    def pqoq(self) -> I32Vector4: ...
    @property
    def pqop(self) -> I32Vector4: ...
    @property
    def pqoo(self) -> I32Vector4: ...
    @property
    def pqol(self) -> I32Vector4: ...
    @property
    def pqls(self) -> I32Vector4: ...
    @property
    def pqlt(self) -> I32Vector4: ...
    @property
    def pqlq(self) -> I32Vector4: ...
    @property
    def pqlp(self) -> I32Vector4: ...
    @property
    def pqlo(self) -> I32Vector4: ...
    @property
    def pqll(self) -> I32Vector4: ...
    @property
    def ppss(self) -> I32Vector4: ...
    @property
    def ppst(self) -> I32Vector4: ...
    @property
    def ppsq(self) -> I32Vector4: ...
    @property
    def ppsp(self) -> I32Vector4: ...
    @property
    def ppso(self) -> I32Vector4: ...
    @property
    def ppsl(self) -> I32Vector4: ...
    @property
    def ppts(self) -> I32Vector4: ...
    @property
    def pptt(self) -> I32Vector4: ...
    @property
    def pptq(self) -> I32Vector4: ...
    @property
    def pptp(self) -> I32Vector4: ...
    @property
    def ppto(self) -> I32Vector4: ...
    @property
    def pptl(self) -> I32Vector4: ...
    @property
    def ppqs(self) -> I32Vector4: ...
    @property
    def ppqt(self) -> I32Vector4: ...
    @property
    def ppqq(self) -> I32Vector4: ...
    @property
    def ppqp(self) -> I32Vector4: ...
    @property
    def ppqo(self) -> I32Vector4: ...
    @property
    def ppql(self) -> I32Vector4: ...
    @property
    def ppps(self) -> I32Vector4: ...
    @property
    def pppt(self) -> I32Vector4: ...
    @property
    def pppq(self) -> I32Vector4: ...
    @property
    def pppp(self) -> I32Vector4: ...
    @property
    def pppo(self) -> I32Vector4: ...
    @property
    def pppl(self) -> I32Vector4: ...
    @property
    def ppos(self) -> I32Vector4: ...
    @property
    def ppot(self) -> I32Vector4: ...
    @property
    def ppoq(self) -> I32Vector4: ...
    @property
    def ppop(self) -> I32Vector4: ...
    @property
    def ppoo(self) -> I32Vector4: ...
    @property
    def ppol(self) -> I32Vector4: ...
    @property
    def ppls(self) -> I32Vector4: ...
    @property
    def pplt(self) -> I32Vector4: ...
    @property
    def pplq(self) -> I32Vector4: ...
    @property
    def pplp(self) -> I32Vector4: ...
    @property
    def pplo(self) -> I32Vector4: ...
    @property
    def ppll(self) -> I32Vector4: ...
    @property
    def poss(self) -> I32Vector4: ...
    @property
    def post(self) -> I32Vector4: ...
    @property
    def posq(self) -> I32Vector4: ...
    @property
    def posp(self) -> I32Vector4: ...
    @property
    def poso(self) -> I32Vector4: ...
    @property
    def posl(self) -> I32Vector4: ...
    @property
    def pots(self) -> I32Vector4: ...
    @property
    def pott(self) -> I32Vector4: ...
    @property
    def potq(self) -> I32Vector4: ...
    @property
    def potp(self) -> I32Vector4: ...
    @property
    def poto(self) -> I32Vector4: ...
    @property
    def potl(self) -> I32Vector4: ...
    @property
    def poqs(self) -> I32Vector4: ...
    @property
    def poqt(self) -> I32Vector4: ...
    @property
    def poqq(self) -> I32Vector4: ...
    @property
    def poqp(self) -> I32Vector4: ...
    @property
    def poqo(self) -> I32Vector4: ...
    @property
    def poql(self) -> I32Vector4: ...
    @property
    def pops(self) -> I32Vector4: ...
    @property
    def popt(self) -> I32Vector4: ...
    @property
    def popq(self) -> I32Vector4: ...
    @property
    def popp(self) -> I32Vector4: ...
    @property
    def popo(self) -> I32Vector4: ...
    @property
    def popl(self) -> I32Vector4: ...
    @property
    def poos(self) -> I32Vector4: ...
    @property
    def poot(self) -> I32Vector4: ...
    @property
    def pooq(self) -> I32Vector4: ...
    @property
    def poop(self) -> I32Vector4: ...
    @property
    def pooo(self) -> I32Vector4: ...
    @property
    def pool(self) -> I32Vector4: ...
    @property
    def pols(self) -> I32Vector4: ...
    @property
    def polt(self) -> I32Vector4: ...
    @property
    def polq(self) -> I32Vector4: ...
    @property
    def polp(self) -> I32Vector4: ...
    @property
    def polo(self) -> I32Vector4: ...
    @property
    def poll(self) -> I32Vector4: ...
    @property
    def plss(self) -> I32Vector4: ...
    @property
    def plst(self) -> I32Vector4: ...
    @property
    def plsq(self) -> I32Vector4: ...
    @property
    def plsp(self) -> I32Vector4: ...
    @property
    def plso(self) -> I32Vector4: ...
    @property
    def plsl(self) -> I32Vector4: ...
    @property
    def plts(self) -> I32Vector4: ...
    @property
    def pltt(self) -> I32Vector4: ...
    @property
    def pltq(self) -> I32Vector4: ...
    @property
    def pltp(self) -> I32Vector4: ...
    @property
    def plto(self) -> I32Vector4: ...
    @property
    def pltl(self) -> I32Vector4: ...
    @property
    def plqs(self) -> I32Vector4: ...
    @property
    def plqt(self) -> I32Vector4: ...
    @property
    def plqq(self) -> I32Vector4: ...
    @property
    def plqp(self) -> I32Vector4: ...
    @property
    def plqo(self) -> I32Vector4: ...
    @property
    def plql(self) -> I32Vector4: ...
    @property
    def plps(self) -> I32Vector4: ...
    @property
    def plpt(self) -> I32Vector4: ...
    @property
    def plpq(self) -> I32Vector4: ...
    @property
    def plpp(self) -> I32Vector4: ...
    @property
    def plpo(self) -> I32Vector4: ...
    @property
    def plpl(self) -> I32Vector4: ...
    @property
    def plos(self) -> I32Vector4: ...
    @property
    def plot(self) -> I32Vector4: ...
    @property
    def ploq(self) -> I32Vector4: ...
    @property
    def plop(self) -> I32Vector4: ...
    @property
    def ploo(self) -> I32Vector4: ...
    @property
    def plol(self) -> I32Vector4: ...
    @property
    def plls(self) -> I32Vector4: ...
    @property
    def pllt(self) -> I32Vector4: ...
    @property
    def pllq(self) -> I32Vector4: ...
    @property
    def pllp(self) -> I32Vector4: ...
    @property
    def pllo(self) -> I32Vector4: ...
    @property
    def plll(self) -> I32Vector4: ...
    @property
    def osss(self) -> I32Vector4: ...
    @property
    def osst(self) -> I32Vector4: ...
    @property
    def ossq(self) -> I32Vector4: ...
    @property
    def ossp(self) -> I32Vector4: ...
    @property
    def osso(self) -> I32Vector4: ...
    @property
    def ossl(self) -> I32Vector4: ...
    @property
    def osts(self) -> I32Vector4: ...
    @property
    def ostt(self) -> I32Vector4: ...
    @property
    def ostq(self) -> I32Vector4: ...
    @property
    def ostp(self) -> I32Vector4: ...
    @property
    def osto(self) -> I32Vector4: ...
    @property
    def ostl(self) -> I32Vector4: ...
    @property
    def osqs(self) -> I32Vector4: ...
    @property
    def osqt(self) -> I32Vector4: ...
    @property
    def osqq(self) -> I32Vector4: ...
    @property
    def osqp(self) -> I32Vector4: ...
    @property
    def osqo(self) -> I32Vector4: ...
    @property
    def osql(self) -> I32Vector4: ...
    @property
    def osps(self) -> I32Vector4: ...
    @property
    def ospt(self) -> I32Vector4: ...
    @property
    def ospq(self) -> I32Vector4: ...
    @property
    def ospp(self) -> I32Vector4: ...
    @property
    def ospo(self) -> I32Vector4: ...
    @property
    def ospl(self) -> I32Vector4: ...
    @property
    def osos(self) -> I32Vector4: ...
    @property
    def osot(self) -> I32Vector4: ...
    @property
    def osoq(self) -> I32Vector4: ...
    @property
    def osop(self) -> I32Vector4: ...
    @property
    def osoo(self) -> I32Vector4: ...
    @property
    def osol(self) -> I32Vector4: ...
    @property
    def osls(self) -> I32Vector4: ...
    @property
    def oslt(self) -> I32Vector4: ...
    @property
    def oslq(self) -> I32Vector4: ...
    @property
    def oslp(self) -> I32Vector4: ...
    @property
    def oslo(self) -> I32Vector4: ...
    @property
    def osll(self) -> I32Vector4: ...
    @property
    def otss(self) -> I32Vector4: ...
    @property
    def otst(self) -> I32Vector4: ...
    @property
    def otsq(self) -> I32Vector4: ...
    @property
    def otsp(self) -> I32Vector4: ...
    @property
    def otso(self) -> I32Vector4: ...
    @property
    def otsl(self) -> I32Vector4: ...
    @property
    def otts(self) -> I32Vector4: ...
    @property
    def ottt(self) -> I32Vector4: ...
    @property
    def ottq(self) -> I32Vector4: ...
    @property
    def ottp(self) -> I32Vector4: ...
    @property
    def otto(self) -> I32Vector4: ...
    @property
    def ottl(self) -> I32Vector4: ...
    @property
    def otqs(self) -> I32Vector4: ...
    @property
    def otqt(self) -> I32Vector4: ...
    @property
    def otqq(self) -> I32Vector4: ...
    @property
    def otqp(self) -> I32Vector4: ...
    @property
    def otqo(self) -> I32Vector4: ...
    @property
    def otql(self) -> I32Vector4: ...
    @property
    def otps(self) -> I32Vector4: ...
    @property
    def otpt(self) -> I32Vector4: ...
    @property
    def otpq(self) -> I32Vector4: ...
    @property
    def otpp(self) -> I32Vector4: ...
    @property
    def otpo(self) -> I32Vector4: ...
    @property
    def otpl(self) -> I32Vector4: ...
    @property
    def otos(self) -> I32Vector4: ...
    @property
    def otot(self) -> I32Vector4: ...
    @property
    def otoq(self) -> I32Vector4: ...
    @property
    def otop(self) -> I32Vector4: ...
    @property
    def otoo(self) -> I32Vector4: ...
    @property
    def otol(self) -> I32Vector4: ...
    @property
    def otls(self) -> I32Vector4: ...
    @property
    def otlt(self) -> I32Vector4: ...
    @property
    def otlq(self) -> I32Vector4: ...
    @property
    def otlp(self) -> I32Vector4: ...
    @property
    def otlo(self) -> I32Vector4: ...
    @property
    def otll(self) -> I32Vector4: ...
    @property
    def oqss(self) -> I32Vector4: ...
    @property
    def oqst(self) -> I32Vector4: ...
    @property
    def oqsq(self) -> I32Vector4: ...
    @property
    def oqsp(self) -> I32Vector4: ...
    @property
    def oqso(self) -> I32Vector4: ...
    @property
    def oqsl(self) -> I32Vector4: ...
    @property
    def oqts(self) -> I32Vector4: ...
    @property
    def oqtt(self) -> I32Vector4: ...
    @property
    def oqtq(self) -> I32Vector4: ...
    @property
    def oqtp(self) -> I32Vector4: ...
    @property
    def oqto(self) -> I32Vector4: ...
    @property
    def oqtl(self) -> I32Vector4: ...
    @property
    def oqqs(self) -> I32Vector4: ...
    @property
    def oqqt(self) -> I32Vector4: ...
    @property
    def oqqq(self) -> I32Vector4: ...
    @property
    def oqqp(self) -> I32Vector4: ...
    @property
    def oqqo(self) -> I32Vector4: ...
    @property
    def oqql(self) -> I32Vector4: ...
    @property
    def oqps(self) -> I32Vector4: ...
    @property
    def oqpt(self) -> I32Vector4: ...
    @property
    def oqpq(self) -> I32Vector4: ...
    @property
    def oqpp(self) -> I32Vector4: ...
    @property
    def oqpo(self) -> I32Vector4: ...
    @property
    def oqpl(self) -> I32Vector4: ...
    @property
    def oqos(self) -> I32Vector4: ...
    @property
    def oqot(self) -> I32Vector4: ...
    @property
    def oqoq(self) -> I32Vector4: ...
    @property
    def oqop(self) -> I32Vector4: ...
    @property
    def oqoo(self) -> I32Vector4: ...
    @property
    def oqol(self) -> I32Vector4: ...
    @property
    def oqls(self) -> I32Vector4: ...
    @property
    def oqlt(self) -> I32Vector4: ...
    @property
    def oqlq(self) -> I32Vector4: ...
    @property
    def oqlp(self) -> I32Vector4: ...
    @property
    def oqlo(self) -> I32Vector4: ...
    @property
    def oqll(self) -> I32Vector4: ...
    @property
    def opss(self) -> I32Vector4: ...
    @property
    def opst(self) -> I32Vector4: ...
    @property
    def opsq(self) -> I32Vector4: ...
    @property
    def opsp(self) -> I32Vector4: ...
    @property
    def opso(self) -> I32Vector4: ...
    @property
    def opsl(self) -> I32Vector4: ...
    @property
    def opts(self) -> I32Vector4: ...
    @property
    def optt(self) -> I32Vector4: ...
    @property
    def optq(self) -> I32Vector4: ...
    @property
    def optp(self) -> I32Vector4: ...
    @property
    def opto(self) -> I32Vector4: ...
    @property
    def optl(self) -> I32Vector4: ...
    @property
    def opqs(self) -> I32Vector4: ...
    @property
    def opqt(self) -> I32Vector4: ...
    @property
    def opqq(self) -> I32Vector4: ...
    @property
    def opqp(self) -> I32Vector4: ...
    @property
    def opqo(self) -> I32Vector4: ...
    @property
    def opql(self) -> I32Vector4: ...
    @property
    def opps(self) -> I32Vector4: ...
    @property
    def oppt(self) -> I32Vector4: ...
    @property
    def oppq(self) -> I32Vector4: ...
    @property
    def oppp(self) -> I32Vector4: ...
    @property
    def oppo(self) -> I32Vector4: ...
    @property
    def oppl(self) -> I32Vector4: ...
    @property
    def opos(self) -> I32Vector4: ...
    @property
    def opot(self) -> I32Vector4: ...
    @property
    def opoq(self) -> I32Vector4: ...
    @property
    def opop(self) -> I32Vector4: ...
    @property
    def opoo(self) -> I32Vector4: ...
    @property
    def opol(self) -> I32Vector4: ...
    @property
    def opls(self) -> I32Vector4: ...
    @property
    def oplt(self) -> I32Vector4: ...
    @property
    def oplq(self) -> I32Vector4: ...
    @property
    def oplp(self) -> I32Vector4: ...
    @property
    def oplo(self) -> I32Vector4: ...
    @property
    def opll(self) -> I32Vector4: ...
    @property
    def ooss(self) -> I32Vector4: ...
    @property
    def oost(self) -> I32Vector4: ...
    @property
    def oosq(self) -> I32Vector4: ...
    @property
    def oosp(self) -> I32Vector4: ...
    @property
    def ooso(self) -> I32Vector4: ...
    @property
    def oosl(self) -> I32Vector4: ...
    @property
    def oots(self) -> I32Vector4: ...
    @property
    def oott(self) -> I32Vector4: ...
    @property
    def ootq(self) -> I32Vector4: ...
    @property
    def ootp(self) -> I32Vector4: ...
    @property
    def ooto(self) -> I32Vector4: ...
    @property
    def ootl(self) -> I32Vector4: ...
    @property
    def ooqs(self) -> I32Vector4: ...
    @property
    def ooqt(self) -> I32Vector4: ...
    @property
    def ooqq(self) -> I32Vector4: ...
    @property
    def ooqp(self) -> I32Vector4: ...
    @property
    def ooqo(self) -> I32Vector4: ...
    @property
    def ooql(self) -> I32Vector4: ...
    @property
    def oops(self) -> I32Vector4: ...
    @property
    def oopt(self) -> I32Vector4: ...
    @property
    def oopq(self) -> I32Vector4: ...
    @property
    def oopp(self) -> I32Vector4: ...
    @property
    def oopo(self) -> I32Vector4: ...
    @property
    def oopl(self) -> I32Vector4: ...
    @property
    def ooos(self) -> I32Vector4: ...
    @property
    def ooot(self) -> I32Vector4: ...
    @property
    def oooq(self) -> I32Vector4: ...
    @property
    def ooop(self) -> I32Vector4: ...
    @property
    def ools(self) -> I32Vector4: ...
    @property
    def oolt(self) -> I32Vector4: ...
    @property
    def oolq(self) -> I32Vector4: ...
    @property
    def oolp(self) -> I32Vector4: ...
    @property
    def olss(self) -> I32Vector4: ...
    @property
    def olst(self) -> I32Vector4: ...
    @property
    def olsq(self) -> I32Vector4: ...
    @property
    def olsp(self) -> I32Vector4: ...
    @property
    def olso(self) -> I32Vector4: ...
    @property
    def olsl(self) -> I32Vector4: ...
    @property
    def olts(self) -> I32Vector4: ...
    @property
    def oltt(self) -> I32Vector4: ...
    @property
    def oltq(self) -> I32Vector4: ...
    @property
    def oltp(self) -> I32Vector4: ...
    @property
    def olto(self) -> I32Vector4: ...
    @property
    def oltl(self) -> I32Vector4: ...
    @property
    def olqs(self) -> I32Vector4: ...
    @property
    def olqt(self) -> I32Vector4: ...
    @property
    def olqq(self) -> I32Vector4: ...
    @property
    def olqp(self) -> I32Vector4: ...
    @property
    def olqo(self) -> I32Vector4: ...
    @property
    def olql(self) -> I32Vector4: ...
    @property
    def olps(self) -> I32Vector4: ...
    @property
    def olpt(self) -> I32Vector4: ...
    @property
    def olpq(self) -> I32Vector4: ...
    @property
    def olpp(self) -> I32Vector4: ...
    @property
    def olpo(self) -> I32Vector4: ...
    @property
    def olpl(self) -> I32Vector4: ...
    @property
    def olos(self) -> I32Vector4: ...
    @property
    def olot(self) -> I32Vector4: ...
    @property
    def oloq(self) -> I32Vector4: ...
    @property
    def olop(self) -> I32Vector4: ...
    @property
    def olls(self) -> I32Vector4: ...
    @property
    def ollt(self) -> I32Vector4: ...
    @property
    def ollq(self) -> I32Vector4: ...
    @property
    def ollp(self) -> I32Vector4: ...
    @property
    def lsss(self) -> I32Vector4: ...
    @property
    def lsst(self) -> I32Vector4: ...
    @property
    def lssq(self) -> I32Vector4: ...
    @property
    def lssp(self) -> I32Vector4: ...
    @property
    def lsso(self) -> I32Vector4: ...
    @property
    def lssl(self) -> I32Vector4: ...
    @property
    def lsts(self) -> I32Vector4: ...
    @property
    def lstt(self) -> I32Vector4: ...
    @property
    def lstq(self) -> I32Vector4: ...
    @property
    def lstp(self) -> I32Vector4: ...
    @property
    def lsto(self) -> I32Vector4: ...
    @property
    def lstl(self) -> I32Vector4: ...
    @property
    def lsqs(self) -> I32Vector4: ...
    @property
    def lsqt(self) -> I32Vector4: ...
    @property
    def lsqq(self) -> I32Vector4: ...
    @property
    def lsqp(self) -> I32Vector4: ...
    @property
    def lsqo(self) -> I32Vector4: ...
    @property
    def lsql(self) -> I32Vector4: ...
    @property
    def lsps(self) -> I32Vector4: ...
    @property
    def lspt(self) -> I32Vector4: ...
    @property
    def lspq(self) -> I32Vector4: ...
    @property
    def lspp(self) -> I32Vector4: ...
    @property
    def lspo(self) -> I32Vector4: ...
    @property
    def lspl(self) -> I32Vector4: ...
    @property
    def lsos(self) -> I32Vector4: ...
    @property
    def lsot(self) -> I32Vector4: ...
    @property
    def lsoq(self) -> I32Vector4: ...
    @property
    def lsop(self) -> I32Vector4: ...
    @property
    def lsoo(self) -> I32Vector4: ...
    @property
    def lsol(self) -> I32Vector4: ...
    @property
    def lsls(self) -> I32Vector4: ...
    @property
    def lslt(self) -> I32Vector4: ...
    @property
    def lslq(self) -> I32Vector4: ...
    @property
    def lslp(self) -> I32Vector4: ...
    @property
    def lslo(self) -> I32Vector4: ...
    @property
    def lsll(self) -> I32Vector4: ...
    @property
    def ltss(self) -> I32Vector4: ...
    @property
    def ltst(self) -> I32Vector4: ...
    @property
    def ltsq(self) -> I32Vector4: ...
    @property
    def ltsp(self) -> I32Vector4: ...
    @property
    def ltso(self) -> I32Vector4: ...
    @property
    def ltsl(self) -> I32Vector4: ...
    @property
    def ltts(self) -> I32Vector4: ...
    @property
    def lttt(self) -> I32Vector4: ...
    @property
    def lttq(self) -> I32Vector4: ...
    @property
    def lttp(self) -> I32Vector4: ...
    @property
    def ltto(self) -> I32Vector4: ...
    @property
    def lttl(self) -> I32Vector4: ...
    @property
    def ltqs(self) -> I32Vector4: ...
    @property
    def ltqt(self) -> I32Vector4: ...
    @property
    def ltqq(self) -> I32Vector4: ...
    @property
    def ltqp(self) -> I32Vector4: ...
    @property
    def ltqo(self) -> I32Vector4: ...
    @property
    def ltql(self) -> I32Vector4: ...
    @property
    def ltps(self) -> I32Vector4: ...
    @property
    def ltpt(self) -> I32Vector4: ...
    @property
    def ltpq(self) -> I32Vector4: ...
    @property
    def ltpp(self) -> I32Vector4: ...
    @property
    def ltpo(self) -> I32Vector4: ...
    @property
    def ltpl(self) -> I32Vector4: ...
    @property
    def ltos(self) -> I32Vector4: ...
    @property
    def ltot(self) -> I32Vector4: ...
    @property
    def ltoq(self) -> I32Vector4: ...
    @property
    def ltop(self) -> I32Vector4: ...
    @property
    def ltoo(self) -> I32Vector4: ...
    @property
    def ltol(self) -> I32Vector4: ...
    @property
    def ltls(self) -> I32Vector4: ...
    @property
    def ltlt(self) -> I32Vector4: ...
    @property
    def ltlq(self) -> I32Vector4: ...
    @property
    def ltlp(self) -> I32Vector4: ...
    @property
    def ltlo(self) -> I32Vector4: ...
    @property
    def ltll(self) -> I32Vector4: ...
    @property
    def lqss(self) -> I32Vector4: ...
    @property
    def lqst(self) -> I32Vector4: ...
    @property
    def lqsq(self) -> I32Vector4: ...
    @property
    def lqsp(self) -> I32Vector4: ...
    @property
    def lqso(self) -> I32Vector4: ...
    @property
    def lqsl(self) -> I32Vector4: ...
    @property
    def lqts(self) -> I32Vector4: ...
    @property
    def lqtt(self) -> I32Vector4: ...
    @property
    def lqtq(self) -> I32Vector4: ...
    @property
    def lqtp(self) -> I32Vector4: ...
    @property
    def lqto(self) -> I32Vector4: ...
    @property
    def lqtl(self) -> I32Vector4: ...
    @property
    def lqqs(self) -> I32Vector4: ...
    @property
    def lqqt(self) -> I32Vector4: ...
    @property
    def lqqq(self) -> I32Vector4: ...
    @property
    def lqqp(self) -> I32Vector4: ...
    @property
    def lqqo(self) -> I32Vector4: ...
    @property
    def lqql(self) -> I32Vector4: ...
    @property
    def lqps(self) -> I32Vector4: ...
    @property
    def lqpt(self) -> I32Vector4: ...
    @property
    def lqpq(self) -> I32Vector4: ...
    @property
    def lqpp(self) -> I32Vector4: ...
    @property
    def lqpo(self) -> I32Vector4: ...
    @property
    def lqpl(self) -> I32Vector4: ...
    @property
    def lqos(self) -> I32Vector4: ...
    @property
    def lqot(self) -> I32Vector4: ...
    @property
    def lqoq(self) -> I32Vector4: ...
    @property
    def lqop(self) -> I32Vector4: ...
    @property
    def lqoo(self) -> I32Vector4: ...
    @property
    def lqol(self) -> I32Vector4: ...
    @property
    def lqls(self) -> I32Vector4: ...
    @property
    def lqlt(self) -> I32Vector4: ...
    @property
    def lqlq(self) -> I32Vector4: ...
    @property
    def lqlp(self) -> I32Vector4: ...
    @property
    def lqlo(self) -> I32Vector4: ...
    @property
    def lqll(self) -> I32Vector4: ...
    @property
    def lpss(self) -> I32Vector4: ...
    @property
    def lpst(self) -> I32Vector4: ...
    @property
    def lpsq(self) -> I32Vector4: ...
    @property
    def lpsp(self) -> I32Vector4: ...
    @property
    def lpso(self) -> I32Vector4: ...
    @property
    def lpsl(self) -> I32Vector4: ...
    @property
    def lpts(self) -> I32Vector4: ...
    @property
    def lptt(self) -> I32Vector4: ...
    @property
    def lptq(self) -> I32Vector4: ...
    @property
    def lptp(self) -> I32Vector4: ...
    @property
    def lpto(self) -> I32Vector4: ...
    @property
    def lptl(self) -> I32Vector4: ...
    @property
    def lpqs(self) -> I32Vector4: ...
    @property
    def lpqt(self) -> I32Vector4: ...
    @property
    def lpqq(self) -> I32Vector4: ...
    @property
    def lpqp(self) -> I32Vector4: ...
    @property
    def lpqo(self) -> I32Vector4: ...
    @property
    def lpql(self) -> I32Vector4: ...
    @property
    def lpps(self) -> I32Vector4: ...
    @property
    def lppt(self) -> I32Vector4: ...
    @property
    def lppq(self) -> I32Vector4: ...
    @property
    def lppp(self) -> I32Vector4: ...
    @property
    def lppo(self) -> I32Vector4: ...
    @property
    def lppl(self) -> I32Vector4: ...
    @property
    def lpos(self) -> I32Vector4: ...
    @property
    def lpot(self) -> I32Vector4: ...
    @property
    def lpoq(self) -> I32Vector4: ...
    @property
    def lpop(self) -> I32Vector4: ...
    @property
    def lpoo(self) -> I32Vector4: ...
    @property
    def lpol(self) -> I32Vector4: ...
    @property
    def lpls(self) -> I32Vector4: ...
    @property
    def lplt(self) -> I32Vector4: ...
    @property
    def lplq(self) -> I32Vector4: ...
    @property
    def lplp(self) -> I32Vector4: ...
    @property
    def lplo(self) -> I32Vector4: ...
    @property
    def lpll(self) -> I32Vector4: ...
    @property
    def loss(self) -> I32Vector4: ...
    @property
    def lost(self) -> I32Vector4: ...
    @property
    def losq(self) -> I32Vector4: ...
    @property
    def losp(self) -> I32Vector4: ...
    @property
    def loso(self) -> I32Vector4: ...
    @property
    def losl(self) -> I32Vector4: ...
    @property
    def lots(self) -> I32Vector4: ...
    @property
    def lott(self) -> I32Vector4: ...
    @property
    def lotq(self) -> I32Vector4: ...
    @property
    def lotp(self) -> I32Vector4: ...
    @property
    def loto(self) -> I32Vector4: ...
    @property
    def lotl(self) -> I32Vector4: ...
    @property
    def loqs(self) -> I32Vector4: ...
    @property
    def loqt(self) -> I32Vector4: ...
    @property
    def loqq(self) -> I32Vector4: ...
    @property
    def loqp(self) -> I32Vector4: ...
    @property
    def loqo(self) -> I32Vector4: ...
    @property
    def loql(self) -> I32Vector4: ...
    @property
    def lops(self) -> I32Vector4: ...
    @property
    def lopt(self) -> I32Vector4: ...
    @property
    def lopq(self) -> I32Vector4: ...
    @property
    def lopp(self) -> I32Vector4: ...
    @property
    def lopo(self) -> I32Vector4: ...
    @property
    def lopl(self) -> I32Vector4: ...
    @property
    def loos(self) -> I32Vector4: ...
    @property
    def loot(self) -> I32Vector4: ...
    @property
    def looq(self) -> I32Vector4: ...
    @property
    def loop(self) -> I32Vector4: ...
    @property
    def lols(self) -> I32Vector4: ...
    @property
    def lolt(self) -> I32Vector4: ...
    @property
    def lolq(self) -> I32Vector4: ...
    @property
    def lolp(self) -> I32Vector4: ...
    @property
    def llss(self) -> I32Vector4: ...
    @property
    def llst(self) -> I32Vector4: ...
    @property
    def llsq(self) -> I32Vector4: ...
    @property
    def llsp(self) -> I32Vector4: ...
    @property
    def llso(self) -> I32Vector4: ...
    @property
    def llsl(self) -> I32Vector4: ...
    @property
    def llts(self) -> I32Vector4: ...
    @property
    def lltt(self) -> I32Vector4: ...
    @property
    def lltq(self) -> I32Vector4: ...
    @property
    def lltp(self) -> I32Vector4: ...
    @property
    def llto(self) -> I32Vector4: ...
    @property
    def lltl(self) -> I32Vector4: ...
    @property
    def llqs(self) -> I32Vector4: ...
    @property
    def llqt(self) -> I32Vector4: ...
    @property
    def llqq(self) -> I32Vector4: ...
    @property
    def llqp(self) -> I32Vector4: ...
    @property
    def llqo(self) -> I32Vector4: ...
    @property
    def llql(self) -> I32Vector4: ...
    @property
    def llps(self) -> I32Vector4: ...
    @property
    def llpt(self) -> I32Vector4: ...
    @property
    def llpq(self) -> I32Vector4: ...
    @property
    def llpp(self) -> I32Vector4: ...
    @property
    def llpo(self) -> I32Vector4: ...
    @property
    def llpl(self) -> I32Vector4: ...
    @property
    def llos(self) -> I32Vector4: ...
    @property
    def llot(self) -> I32Vector4: ...
    @property
    def lloq(self) -> I32Vector4: ...
    @property
    def llop(self) -> I32Vector4: ...
    @property
    def llls(self) -> I32Vector4: ...
    @property
    def lllt(self) -> I32Vector4: ...
    @property
    def lllq(self) -> I32Vector4: ...
    @property
    def lllp(self) -> I32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I32Vector2: ...
    @property
    def uv(self) -> I32Vector2: ...
    @property
    def uo(self) -> I32Vector2: ...
    @property
    def ul(self) -> I32Vector2: ...
    @property
    def vu(self) -> I32Vector2: ...
    @property
    def vv(self) -> I32Vector2: ...
    @property
    def vo(self) -> I32Vector2: ...
    @property
    def vl(self) -> I32Vector2: ...
    @property
    def ou(self) -> I32Vector2: ...
    @property
    def ov(self) -> I32Vector2: ...
    @property
    def lu(self) -> I32Vector2: ...
    @property
    def lv(self) -> I32Vector2: ...
    @property
    def uuu(self) -> I32Vector3: ...
    @property
    def uuv(self) -> I32Vector3: ...
    @property
    def uuo(self) -> I32Vector3: ...
    @property
    def uul(self) -> I32Vector3: ...
    @property
    def uvu(self) -> I32Vector3: ...
    @property
    def uvv(self) -> I32Vector3: ...
    @property
    def uvo(self) -> I32Vector3: ...
    @property
    def uvl(self) -> I32Vector3: ...
    @property
    def uou(self) -> I32Vector3: ...
    @property
    def uov(self) -> I32Vector3: ...
    @property
    def uoo(self) -> I32Vector3: ...
    @property
    def uol(self) -> I32Vector3: ...
    @property
    def ulu(self) -> I32Vector3: ...
    @property
    def ulv(self) -> I32Vector3: ...
    @property
    def ulo(self) -> I32Vector3: ...
    @property
    def ull(self) -> I32Vector3: ...
    @property
    def vuu(self) -> I32Vector3: ...
    @property
    def vuv(self) -> I32Vector3: ...
    @property
    def vuo(self) -> I32Vector3: ...
    @property
    def vul(self) -> I32Vector3: ...
    @property
    def vvu(self) -> I32Vector3: ...
    @property
    def vvv(self) -> I32Vector3: ...
    @property
    def vvo(self) -> I32Vector3: ...
    @property
    def vvl(self) -> I32Vector3: ...
    @property
    def vou(self) -> I32Vector3: ...
    @property
    def vov(self) -> I32Vector3: ...
    @property
    def voo(self) -> I32Vector3: ...
    @property
    def vol(self) -> I32Vector3: ...
    @property
    def vlu(self) -> I32Vector3: ...
    @property
    def vlv(self) -> I32Vector3: ...
    @property
    def vlo(self) -> I32Vector3: ...
    @property
    def vll(self) -> I32Vector3: ...
    @property
    def ouu(self) -> I32Vector3: ...
    @property
    def ouv(self) -> I32Vector3: ...
    @property
    def ouo(self) -> I32Vector3: ...
    @property
    def oul(self) -> I32Vector3: ...
    @property
    def ovu(self) -> I32Vector3: ...
    @property
    def ovv(self) -> I32Vector3: ...
    @property
    def ovo(self) -> I32Vector3: ...
    @property
    def ovl(self) -> I32Vector3: ...
    @property
    def oou(self) -> I32Vector3: ...
    @property
    def oov(self) -> I32Vector3: ...
    @property
    def olu(self) -> I32Vector3: ...
    @property
    def olv(self) -> I32Vector3: ...
    @property
    def luu(self) -> I32Vector3: ...
    @property
    def luv(self) -> I32Vector3: ...
    @property
    def luo(self) -> I32Vector3: ...
    @property
    def lul(self) -> I32Vector3: ...
    @property
    def lvu(self) -> I32Vector3: ...
    @property
    def lvv(self) -> I32Vector3: ...
    @property
    def lvo(self) -> I32Vector3: ...
    @property
    def lvl(self) -> I32Vector3: ...
    @property
    def lou(self) -> I32Vector3: ...
    @property
    def lov(self) -> I32Vector3: ...
    @property
    def llu(self) -> I32Vector3: ...
    @property
    def llv(self) -> I32Vector3: ...
    @property
    def uuuu(self) -> I32Vector4: ...
    @property
    def uuuv(self) -> I32Vector4: ...
    @property
    def uuuo(self) -> I32Vector4: ...
    @property
    def uuul(self) -> I32Vector4: ...
    @property
    def uuvu(self) -> I32Vector4: ...
    @property
    def uuvv(self) -> I32Vector4: ...
    @property
    def uuvo(self) -> I32Vector4: ...
    @property
    def uuvl(self) -> I32Vector4: ...
    @property
    def uuou(self) -> I32Vector4: ...
    @property
    def uuov(self) -> I32Vector4: ...
    @property
    def uuoo(self) -> I32Vector4: ...
    @property
    def uuol(self) -> I32Vector4: ...
    @property
    def uulu(self) -> I32Vector4: ...
    @property
    def uulv(self) -> I32Vector4: ...
    @property
    def uulo(self) -> I32Vector4: ...
    @property
    def uull(self) -> I32Vector4: ...
    @property
    def uvuu(self) -> I32Vector4: ...
    @property
    def uvuv(self) -> I32Vector4: ...
    @property
    def uvuo(self) -> I32Vector4: ...
    @property
    def uvul(self) -> I32Vector4: ...
    @property
    def uvvu(self) -> I32Vector4: ...
    @property
    def uvvv(self) -> I32Vector4: ...
    @property
    def uvvo(self) -> I32Vector4: ...
    @property
    def uvvl(self) -> I32Vector4: ...
    @property
    def uvou(self) -> I32Vector4: ...
    @property
    def uvov(self) -> I32Vector4: ...
    @property
    def uvoo(self) -> I32Vector4: ...
    @property
    def uvol(self) -> I32Vector4: ...
    @property
    def uvlu(self) -> I32Vector4: ...
    @property
    def uvlv(self) -> I32Vector4: ...
    @property
    def uvlo(self) -> I32Vector4: ...
    @property
    def uvll(self) -> I32Vector4: ...
    @property
    def uouu(self) -> I32Vector4: ...
    @property
    def uouv(self) -> I32Vector4: ...
    @property
    def uouo(self) -> I32Vector4: ...
    @property
    def uoul(self) -> I32Vector4: ...
    @property
    def uovu(self) -> I32Vector4: ...
    @property
    def uovv(self) -> I32Vector4: ...
    @property
    def uovo(self) -> I32Vector4: ...
    @property
    def uovl(self) -> I32Vector4: ...
    @property
    def uoou(self) -> I32Vector4: ...
    @property
    def uoov(self) -> I32Vector4: ...
    @property
    def uooo(self) -> I32Vector4: ...
    @property
    def uool(self) -> I32Vector4: ...
    @property
    def uolu(self) -> I32Vector4: ...
    @property
    def uolv(self) -> I32Vector4: ...
    @property
    def uolo(self) -> I32Vector4: ...
    @property
    def uoll(self) -> I32Vector4: ...
    @property
    def uluu(self) -> I32Vector4: ...
    @property
    def uluv(self) -> I32Vector4: ...
    @property
    def uluo(self) -> I32Vector4: ...
    @property
    def ulul(self) -> I32Vector4: ...
    @property
    def ulvu(self) -> I32Vector4: ...
    @property
    def ulvv(self) -> I32Vector4: ...
    @property
    def ulvo(self) -> I32Vector4: ...
    @property
    def ulvl(self) -> I32Vector4: ...
    @property
    def ulou(self) -> I32Vector4: ...
    @property
    def ulov(self) -> I32Vector4: ...
    @property
    def uloo(self) -> I32Vector4: ...
    @property
    def ulol(self) -> I32Vector4: ...
    @property
    def ullu(self) -> I32Vector4: ...
    @property
    def ullv(self) -> I32Vector4: ...
    @property
    def ullo(self) -> I32Vector4: ...
    @property
    def ulll(self) -> I32Vector4: ...
    @property
    def vuuu(self) -> I32Vector4: ...
    @property
    def vuuv(self) -> I32Vector4: ...
    @property
    def vuuo(self) -> I32Vector4: ...
    @property
    def vuul(self) -> I32Vector4: ...
    @property
    def vuvu(self) -> I32Vector4: ...
    @property
    def vuvv(self) -> I32Vector4: ...
    @property
    def vuvo(self) -> I32Vector4: ...
    @property
    def vuvl(self) -> I32Vector4: ...
    @property
    def vuou(self) -> I32Vector4: ...
    @property
    def vuov(self) -> I32Vector4: ...
    @property
    def vuoo(self) -> I32Vector4: ...
    @property
    def vuol(self) -> I32Vector4: ...
    @property
    def vulu(self) -> I32Vector4: ...
    @property
    def vulv(self) -> I32Vector4: ...
    @property
    def vulo(self) -> I32Vector4: ...
    @property
    def vull(self) -> I32Vector4: ...
    @property
    def vvuu(self) -> I32Vector4: ...
    @property
    def vvuv(self) -> I32Vector4: ...
    @property
    def vvuo(self) -> I32Vector4: ...
    @property
    def vvul(self) -> I32Vector4: ...
    @property
    def vvvu(self) -> I32Vector4: ...
    @property
    def vvvv(self) -> I32Vector4: ...
    @property
    def vvvo(self) -> I32Vector4: ...
    @property
    def vvvl(self) -> I32Vector4: ...
    @property
    def vvou(self) -> I32Vector4: ...
    @property
    def vvov(self) -> I32Vector4: ...
    @property
    def vvoo(self) -> I32Vector4: ...
    @property
    def vvol(self) -> I32Vector4: ...
    @property
    def vvlu(self) -> I32Vector4: ...
    @property
    def vvlv(self) -> I32Vector4: ...
    @property
    def vvlo(self) -> I32Vector4: ...
    @property
    def vvll(self) -> I32Vector4: ...
    @property
    def vouu(self) -> I32Vector4: ...
    @property
    def vouv(self) -> I32Vector4: ...
    @property
    def vouo(self) -> I32Vector4: ...
    @property
    def voul(self) -> I32Vector4: ...
    @property
    def vovu(self) -> I32Vector4: ...
    @property
    def vovv(self) -> I32Vector4: ...
    @property
    def vovo(self) -> I32Vector4: ...
    @property
    def vovl(self) -> I32Vector4: ...
    @property
    def voou(self) -> I32Vector4: ...
    @property
    def voov(self) -> I32Vector4: ...
    @property
    def vooo(self) -> I32Vector4: ...
    @property
    def vool(self) -> I32Vector4: ...
    @property
    def volu(self) -> I32Vector4: ...
    @property
    def volv(self) -> I32Vector4: ...
    @property
    def volo(self) -> I32Vector4: ...
    @property
    def voll(self) -> I32Vector4: ...
    @property
    def vluu(self) -> I32Vector4: ...
    @property
    def vluv(self) -> I32Vector4: ...
    @property
    def vluo(self) -> I32Vector4: ...
    @property
    def vlul(self) -> I32Vector4: ...
    @property
    def vlvu(self) -> I32Vector4: ...
    @property
    def vlvv(self) -> I32Vector4: ...
    @property
    def vlvo(self) -> I32Vector4: ...
    @property
    def vlvl(self) -> I32Vector4: ...
    @property
    def vlou(self) -> I32Vector4: ...
    @property
    def vlov(self) -> I32Vector4: ...
    @property
    def vloo(self) -> I32Vector4: ...
    @property
    def vlol(self) -> I32Vector4: ...
    @property
    def vllu(self) -> I32Vector4: ...
    @property
    def vllv(self) -> I32Vector4: ...
    @property
    def vllo(self) -> I32Vector4: ...
    @property
    def vlll(self) -> I32Vector4: ...
    @property
    def ouuu(self) -> I32Vector4: ...
    @property
    def ouuv(self) -> I32Vector4: ...
    @property
    def ouuo(self) -> I32Vector4: ...
    @property
    def ouul(self) -> I32Vector4: ...
    @property
    def ouvu(self) -> I32Vector4: ...
    @property
    def ouvv(self) -> I32Vector4: ...
    @property
    def ouvo(self) -> I32Vector4: ...
    @property
    def ouvl(self) -> I32Vector4: ...
    @property
    def ouou(self) -> I32Vector4: ...
    @property
    def ouov(self) -> I32Vector4: ...
    @property
    def ouoo(self) -> I32Vector4: ...
    @property
    def ouol(self) -> I32Vector4: ...
    @property
    def oulu(self) -> I32Vector4: ...
    @property
    def oulv(self) -> I32Vector4: ...
    @property
    def oulo(self) -> I32Vector4: ...
    @property
    def oull(self) -> I32Vector4: ...
    @property
    def ovuu(self) -> I32Vector4: ...
    @property
    def ovuv(self) -> I32Vector4: ...
    @property
    def ovuo(self) -> I32Vector4: ...
    @property
    def ovul(self) -> I32Vector4: ...
    @property
    def ovvu(self) -> I32Vector4: ...
    @property
    def ovvv(self) -> I32Vector4: ...
    @property
    def ovvo(self) -> I32Vector4: ...
    @property
    def ovvl(self) -> I32Vector4: ...
    @property
    def ovou(self) -> I32Vector4: ...
    @property
    def ovov(self) -> I32Vector4: ...
    @property
    def ovoo(self) -> I32Vector4: ...
    @property
    def ovol(self) -> I32Vector4: ...
    @property
    def ovlu(self) -> I32Vector4: ...
    @property
    def ovlv(self) -> I32Vector4: ...
    @property
    def ovlo(self) -> I32Vector4: ...
    @property
    def ovll(self) -> I32Vector4: ...
    @property
    def oouu(self) -> I32Vector4: ...
    @property
    def oouv(self) -> I32Vector4: ...
    @property
    def oouo(self) -> I32Vector4: ...
    @property
    def ooul(self) -> I32Vector4: ...
    @property
    def oovu(self) -> I32Vector4: ...
    @property
    def oovv(self) -> I32Vector4: ...
    @property
    def oovo(self) -> I32Vector4: ...
    @property
    def oovl(self) -> I32Vector4: ...
    @property
    def ooou(self) -> I32Vector4: ...
    @property
    def ooov(self) -> I32Vector4: ...
    @property
    def oolu(self) -> I32Vector4: ...
    @property
    def oolv(self) -> I32Vector4: ...
    @property
    def oluu(self) -> I32Vector4: ...
    @property
    def oluv(self) -> I32Vector4: ...
    @property
    def oluo(self) -> I32Vector4: ...
    @property
    def olul(self) -> I32Vector4: ...
    @property
    def olvu(self) -> I32Vector4: ...
    @property
    def olvv(self) -> I32Vector4: ...
    @property
    def olvo(self) -> I32Vector4: ...
    @property
    def olvl(self) -> I32Vector4: ...
    @property
    def olou(self) -> I32Vector4: ...
    @property
    def olov(self) -> I32Vector4: ...
    @property
    def ollu(self) -> I32Vector4: ...
    @property
    def ollv(self) -> I32Vector4: ...
    @property
    def luuu(self) -> I32Vector4: ...
    @property
    def luuv(self) -> I32Vector4: ...
    @property
    def luuo(self) -> I32Vector4: ...
    @property
    def luul(self) -> I32Vector4: ...
    @property
    def luvu(self) -> I32Vector4: ...
    @property
    def luvv(self) -> I32Vector4: ...
    @property
    def luvo(self) -> I32Vector4: ...
    @property
    def luvl(self) -> I32Vector4: ...
    @property
    def luou(self) -> I32Vector4: ...
    @property
    def luov(self) -> I32Vector4: ...
    @property
    def luoo(self) -> I32Vector4: ...
    @property
    def luol(self) -> I32Vector4: ...
    @property
    def lulu(self) -> I32Vector4: ...
    @property
    def lulv(self) -> I32Vector4: ...
    @property
    def lulo(self) -> I32Vector4: ...
    @property
    def lull(self) -> I32Vector4: ...
    @property
    def lvuu(self) -> I32Vector4: ...
    @property
    def lvuv(self) -> I32Vector4: ...
    @property
    def lvuo(self) -> I32Vector4: ...
    @property
    def lvul(self) -> I32Vector4: ...
    @property
    def lvvu(self) -> I32Vector4: ...
    @property
    def lvvv(self) -> I32Vector4: ...
    @property
    def lvvo(self) -> I32Vector4: ...
    @property
    def lvvl(self) -> I32Vector4: ...
    @property
    def lvou(self) -> I32Vector4: ...
    @property
    def lvov(self) -> I32Vector4: ...
    @property
    def lvoo(self) -> I32Vector4: ...
    @property
    def lvol(self) -> I32Vector4: ...
    @property
    def lvlu(self) -> I32Vector4: ...
    @property
    def lvlv(self) -> I32Vector4: ...
    @property
    def lvlo(self) -> I32Vector4: ...
    @property
    def lvll(self) -> I32Vector4: ...
    @property
    def louu(self) -> I32Vector4: ...
    @property
    def louv(self) -> I32Vector4: ...
    @property
    def louo(self) -> I32Vector4: ...
    @property
    def loul(self) -> I32Vector4: ...
    @property
    def lovu(self) -> I32Vector4: ...
    @property
    def lovv(self) -> I32Vector4: ...
    @property
    def lovo(self) -> I32Vector4: ...
    @property
    def lovl(self) -> I32Vector4: ...
    @property
    def loou(self) -> I32Vector4: ...
    @property
    def loov(self) -> I32Vector4: ...
    @property
    def lolu(self) -> I32Vector4: ...
    @property
    def lolv(self) -> I32Vector4: ...
    @property
    def lluu(self) -> I32Vector4: ...
    @property
    def lluv(self) -> I32Vector4: ...
    @property
    def lluo(self) -> I32Vector4: ...
    @property
    def llul(self) -> I32Vector4: ...
    @property
    def llvu(self) -> I32Vector4: ...
    @property
    def llvv(self) -> I32Vector4: ...
    @property
    def llvo(self) -> I32Vector4: ...
    @property
    def llvl(self) -> I32Vector4: ...
    @property
    def llou(self) -> I32Vector4: ...
    @property
    def llov(self) -> I32Vector4: ...
    @property
    def lllu(self) -> I32Vector4: ...
    @property
    def lllv(self) -> I32Vector4: ...
    @property
    def oo(self) -> I32Vector2: ...
    @property
    def ol(self) -> I32Vector2: ...
    @property
    def lo(self) -> I32Vector2: ...
    @property
    def ll(self) -> I32Vector2: ...
    @property
    def ooo(self) -> I32Vector3: ...
    @property
    def ool(self) -> I32Vector3: ...
    @property
    def olo(self) -> I32Vector3: ...
    @property
    def oll(self) -> I32Vector3: ...
    @property
    def loo(self) -> I32Vector3: ...
    @property
    def lol(self) -> I32Vector3: ...
    @property
    def llo(self) -> I32Vector3: ...
    @property
    def lll(self) -> I32Vector3: ...
    @property
    def oooo(self) -> I32Vector4: ...
    @property
    def oool(self) -> I32Vector4: ...
    @property
    def oolo(self) -> I32Vector4: ...
    @property
    def ooll(self) -> I32Vector4: ...
    @property
    def oloo(self) -> I32Vector4: ...
    @property
    def olol(self) -> I32Vector4: ...
    @property
    def ollo(self) -> I32Vector4: ...
    @property
    def olll(self) -> I32Vector4: ...
    @property
    def looo(self) -> I32Vector4: ...
    @property
    def lool(self) -> I32Vector4: ...
    @property
    def lolo(self) -> I32Vector4: ...
    @property
    def loll(self) -> I32Vector4: ...
    @property
    def lloo(self) -> I32Vector4: ...
    @property
    def llol(self) -> I32Vector4: ...
    @property
    def lllo(self) -> I32Vector4: ...
    @property
    def llll(self) -> I32Vector4: ...
    def min(self, n: Number, /) -> I32Vector4: ...
    def max(self, n: Number, /) -> I32Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> I32Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I32Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector4: ...

@final
class I32Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I32Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I32Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> I32Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> I32Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I32Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[I32Vector4]: ...

@final
class U32Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __add__(self, other: Number) -> U32Vector4: ...
    @overload
    def __radd__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __radd__(self, other: Number) -> U32Vector4: ...
    @overload
    def __sub__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __sub__(self, other: Number) -> U32Vector4: ...
    @overload
    def __rsub__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> U32Vector4: ...
    @overload
    def __mul__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __mul__(self, other: Number) -> U32Vector4: ...
    @overload
    def __rmul__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> U32Vector4: ...
    @overload
    def __truediv__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> U32Vector4: ...
    @overload
    def __rtruediv__(self, other: U32Vector4) -> U32Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> U32Vector4: ...
    def __abs__(self) -> U32Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> U32Vector2: ...
    @property
    def xy(self) -> U32Vector2: ...
    @property
    def xz(self) -> U32Vector2: ...
    @property
    def xw(self) -> U32Vector2: ...
    @property
    def xo(self) -> U32Vector2: ...
    @property
    def xl(self) -> U32Vector2: ...
    @property
    def yx(self) -> U32Vector2: ...
    @property
    def yy(self) -> U32Vector2: ...
    @property
    def yz(self) -> U32Vector2: ...
    @property
    def yw(self) -> U32Vector2: ...
    @property
    def yo(self) -> U32Vector2: ...
    @property
    def yl(self) -> U32Vector2: ...
    @property
    def zx(self) -> U32Vector2: ...
    @property
    def zy(self) -> U32Vector2: ...
    @property
    def zz(self) -> U32Vector2: ...
    @property
    def zw(self) -> U32Vector2: ...
    @property
    def zo(self) -> U32Vector2: ...
    @property
    def zl(self) -> U32Vector2: ...
    @property
    def wx(self) -> U32Vector2: ...
    @property
    def wy(self) -> U32Vector2: ...
    @property
    def wz(self) -> U32Vector2: ...
    @property
    def ww(self) -> U32Vector2: ...
    @property
    def wo(self) -> U32Vector2: ...
    @property
    def wl(self) -> U32Vector2: ...
    @property
    def ox(self) -> U32Vector2: ...
    @property
    def oy(self) -> U32Vector2: ...
    @property
    def oz(self) -> U32Vector2: ...
    @property
    def ow(self) -> U32Vector2: ...
    @property
    def lx(self) -> U32Vector2: ...
    @property
    def ly(self) -> U32Vector2: ...
    @property
    def lz(self) -> U32Vector2: ...
    @property
    def lw(self) -> U32Vector2: ...
    @property
    def xxx(self) -> U32Vector3: ...
    @property
    def xxy(self) -> U32Vector3: ...
    @property
    def xxz(self) -> U32Vector3: ...
    @property
    def xxw(self) -> U32Vector3: ...
    @property
    def xxo(self) -> U32Vector3: ...
    @property
    def xxl(self) -> U32Vector3: ...
    @property
    def xyx(self) -> U32Vector3: ...
    @property
    def xyy(self) -> U32Vector3: ...
    @property
    def xyz(self) -> U32Vector3: ...
    @property
    def xyw(self) -> U32Vector3: ...
    @property
    def xyo(self) -> U32Vector3: ...
    @property
    def xyl(self) -> U32Vector3: ...
    @property
    def xzx(self) -> U32Vector3: ...
    @property
    def xzy(self) -> U32Vector3: ...
    @property
    def xzz(self) -> U32Vector3: ...
    @property
    def xzw(self) -> U32Vector3: ...
    @property
    def xzo(self) -> U32Vector3: ...
    @property
    def xzl(self) -> U32Vector3: ...
    @property
    def xwx(self) -> U32Vector3: ...
    @property
    def xwy(self) -> U32Vector3: ...
    @property
    def xwz(self) -> U32Vector3: ...
    @property
    def xww(self) -> U32Vector3: ...
    @property
    def xwo(self) -> U32Vector3: ...
    @property
    def xwl(self) -> U32Vector3: ...
    @property
    def xox(self) -> U32Vector3: ...
    @property
    def xoy(self) -> U32Vector3: ...
    @property
    def xoz(self) -> U32Vector3: ...
    @property
    def xow(self) -> U32Vector3: ...
    @property
    def xoo(self) -> U32Vector3: ...
    @property
    def xol(self) -> U32Vector3: ...
    @property
    def xlx(self) -> U32Vector3: ...
    @property
    def xly(self) -> U32Vector3: ...
    @property
    def xlz(self) -> U32Vector3: ...
    @property
    def xlw(self) -> U32Vector3: ...
    @property
    def xlo(self) -> U32Vector3: ...
    @property
    def xll(self) -> U32Vector3: ...
    @property
    def yxx(self) -> U32Vector3: ...
    @property
    def yxy(self) -> U32Vector3: ...
    @property
    def yxz(self) -> U32Vector3: ...
    @property
    def yxw(self) -> U32Vector3: ...
    @property
    def yxo(self) -> U32Vector3: ...
    @property
    def yxl(self) -> U32Vector3: ...
    @property
    def yyx(self) -> U32Vector3: ...
    @property
    def yyy(self) -> U32Vector3: ...
    @property
    def yyz(self) -> U32Vector3: ...
    @property
    def yyw(self) -> U32Vector3: ...
    @property
    def yyo(self) -> U32Vector3: ...
    @property
    def yyl(self) -> U32Vector3: ...
    @property
    def yzx(self) -> U32Vector3: ...
    @property
    def yzy(self) -> U32Vector3: ...
    @property
    def yzz(self) -> U32Vector3: ...
    @property
    def yzw(self) -> U32Vector3: ...
    @property
    def yzo(self) -> U32Vector3: ...
    @property
    def yzl(self) -> U32Vector3: ...
    @property
    def ywx(self) -> U32Vector3: ...
    @property
    def ywy(self) -> U32Vector3: ...
    @property
    def ywz(self) -> U32Vector3: ...
    @property
    def yww(self) -> U32Vector3: ...
    @property
    def ywo(self) -> U32Vector3: ...
    @property
    def ywl(self) -> U32Vector3: ...
    @property
    def yox(self) -> U32Vector3: ...
    @property
    def yoy(self) -> U32Vector3: ...
    @property
    def yoz(self) -> U32Vector3: ...
    @property
    def yow(self) -> U32Vector3: ...
    @property
    def yoo(self) -> U32Vector3: ...
    @property
    def yol(self) -> U32Vector3: ...
    @property
    def ylx(self) -> U32Vector3: ...
    @property
    def yly(self) -> U32Vector3: ...
    @property
    def ylz(self) -> U32Vector3: ...
    @property
    def ylw(self) -> U32Vector3: ...
    @property
    def ylo(self) -> U32Vector3: ...
    @property
    def yll(self) -> U32Vector3: ...
    @property
    def zxx(self) -> U32Vector3: ...
    @property
    def zxy(self) -> U32Vector3: ...
    @property
    def zxz(self) -> U32Vector3: ...
    @property
    def zxw(self) -> U32Vector3: ...
    @property
    def zxo(self) -> U32Vector3: ...
    @property
    def zxl(self) -> U32Vector3: ...
    @property
    def zyx(self) -> U32Vector3: ...
    @property
    def zyy(self) -> U32Vector3: ...
    @property
    def zyz(self) -> U32Vector3: ...
    @property
    def zyw(self) -> U32Vector3: ...
    @property
    def zyo(self) -> U32Vector3: ...
    @property
    def zyl(self) -> U32Vector3: ...
    @property
    def zzx(self) -> U32Vector3: ...
    @property
    def zzy(self) -> U32Vector3: ...
    @property
    def zzz(self) -> U32Vector3: ...
    @property
    def zzw(self) -> U32Vector3: ...
    @property
    def zzo(self) -> U32Vector3: ...
    @property
    def zzl(self) -> U32Vector3: ...
    @property
    def zwx(self) -> U32Vector3: ...
    @property
    def zwy(self) -> U32Vector3: ...
    @property
    def zwz(self) -> U32Vector3: ...
    @property
    def zww(self) -> U32Vector3: ...
    @property
    def zwo(self) -> U32Vector3: ...
    @property
    def zwl(self) -> U32Vector3: ...
    @property
    def zox(self) -> U32Vector3: ...
    @property
    def zoy(self) -> U32Vector3: ...
    @property
    def zoz(self) -> U32Vector3: ...
    @property
    def zow(self) -> U32Vector3: ...
    @property
    def zoo(self) -> U32Vector3: ...
    @property
    def zol(self) -> U32Vector3: ...
    @property
    def zlx(self) -> U32Vector3: ...
    @property
    def zly(self) -> U32Vector3: ...
    @property
    def zlz(self) -> U32Vector3: ...
    @property
    def zlw(self) -> U32Vector3: ...
    @property
    def zlo(self) -> U32Vector3: ...
    @property
    def zll(self) -> U32Vector3: ...
    @property
    def wxx(self) -> U32Vector3: ...
    @property
    def wxy(self) -> U32Vector3: ...
    @property
    def wxz(self) -> U32Vector3: ...
    @property
    def wxw(self) -> U32Vector3: ...
    @property
    def wxo(self) -> U32Vector3: ...
    @property
    def wxl(self) -> U32Vector3: ...
    @property
    def wyx(self) -> U32Vector3: ...
    @property
    def wyy(self) -> U32Vector3: ...
    @property
    def wyz(self) -> U32Vector3: ...
    @property
    def wyw(self) -> U32Vector3: ...
    @property
    def wyo(self) -> U32Vector3: ...
    @property
    def wyl(self) -> U32Vector3: ...
    @property
    def wzx(self) -> U32Vector3: ...
    @property
    def wzy(self) -> U32Vector3: ...
    @property
    def wzz(self) -> U32Vector3: ...
    @property
    def wzw(self) -> U32Vector3: ...
    @property
    def wzo(self) -> U32Vector3: ...
    @property
    def wzl(self) -> U32Vector3: ...
    @property
    def wwx(self) -> U32Vector3: ...
    @property
    def wwy(self) -> U32Vector3: ...
    @property
    def wwz(self) -> U32Vector3: ...
    @property
    def www(self) -> U32Vector3: ...
    @property
    def wwo(self) -> U32Vector3: ...
    @property
    def wwl(self) -> U32Vector3: ...
    @property
    def wox(self) -> U32Vector3: ...
    @property
    def woy(self) -> U32Vector3: ...
    @property
    def woz(self) -> U32Vector3: ...
    @property
    def wow(self) -> U32Vector3: ...
    @property
    def woo(self) -> U32Vector3: ...
    @property
    def wol(self) -> U32Vector3: ...
    @property
    def wlx(self) -> U32Vector3: ...
    @property
    def wly(self) -> U32Vector3: ...
    @property
    def wlz(self) -> U32Vector3: ...
    @property
    def wlw(self) -> U32Vector3: ...
    @property
    def wlo(self) -> U32Vector3: ...
    @property
    def wll(self) -> U32Vector3: ...
    @property
    def oxx(self) -> U32Vector3: ...
    @property
    def oxy(self) -> U32Vector3: ...
    @property
    def oxz(self) -> U32Vector3: ...
    @property
    def oxw(self) -> U32Vector3: ...
    @property
    def oxo(self) -> U32Vector3: ...
    @property
    def oxl(self) -> U32Vector3: ...
    @property
    def oyx(self) -> U32Vector3: ...
    @property
    def oyy(self) -> U32Vector3: ...
    @property
    def oyz(self) -> U32Vector3: ...
    @property
    def oyw(self) -> U32Vector3: ...
    @property
    def oyo(self) -> U32Vector3: ...
    @property
    def oyl(self) -> U32Vector3: ...
    @property
    def ozx(self) -> U32Vector3: ...
    @property
    def ozy(self) -> U32Vector3: ...
    @property
    def ozz(self) -> U32Vector3: ...
    @property
    def ozw(self) -> U32Vector3: ...
    @property
    def ozo(self) -> U32Vector3: ...
    @property
    def ozl(self) -> U32Vector3: ...
    @property
    def owx(self) -> U32Vector3: ...
    @property
    def owy(self) -> U32Vector3: ...
    @property
    def owz(self) -> U32Vector3: ...
    @property
    def oww(self) -> U32Vector3: ...
    @property
    def owo(self) -> U32Vector3: ...
    @property
    def owl(self) -> U32Vector3: ...
    @property
    def oox(self) -> U32Vector3: ...
    @property
    def ooy(self) -> U32Vector3: ...
    @property
    def ooz(self) -> U32Vector3: ...
    @property
    def oow(self) -> U32Vector3: ...
    @property
    def olx(self) -> U32Vector3: ...
    @property
    def oly(self) -> U32Vector3: ...
    @property
    def olz(self) -> U32Vector3: ...
    @property
    def olw(self) -> U32Vector3: ...
    @property
    def lxx(self) -> U32Vector3: ...
    @property
    def lxy(self) -> U32Vector3: ...
    @property
    def lxz(self) -> U32Vector3: ...
    @property
    def lxw(self) -> U32Vector3: ...
    @property
    def lxo(self) -> U32Vector3: ...
    @property
    def lxl(self) -> U32Vector3: ...
    @property
    def lyx(self) -> U32Vector3: ...
    @property
    def lyy(self) -> U32Vector3: ...
    @property
    def lyz(self) -> U32Vector3: ...
    @property
    def lyw(self) -> U32Vector3: ...
    @property
    def lyo(self) -> U32Vector3: ...
    @property
    def lyl(self) -> U32Vector3: ...
    @property
    def lzx(self) -> U32Vector3: ...
    @property
    def lzy(self) -> U32Vector3: ...
    @property
    def lzz(self) -> U32Vector3: ...
    @property
    def lzw(self) -> U32Vector3: ...
    @property
    def lzo(self) -> U32Vector3: ...
    @property
    def lzl(self) -> U32Vector3: ...
    @property
    def lwx(self) -> U32Vector3: ...
    @property
    def lwy(self) -> U32Vector3: ...
    @property
    def lwz(self) -> U32Vector3: ...
    @property
    def lww(self) -> U32Vector3: ...
    @property
    def lwo(self) -> U32Vector3: ...
    @property
    def lwl(self) -> U32Vector3: ...
    @property
    def lox(self) -> U32Vector3: ...
    @property
    def loy(self) -> U32Vector3: ...
    @property
    def loz(self) -> U32Vector3: ...
    @property
    def low(self) -> U32Vector3: ...
    @property
    def llx(self) -> U32Vector3: ...
    @property
    def lly(self) -> U32Vector3: ...
    @property
    def llz(self) -> U32Vector3: ...
    @property
    def llw(self) -> U32Vector3: ...
    @property
    def xxxx(self) -> U32Vector4: ...
    @property
    def xxxy(self) -> U32Vector4: ...
    @property
    def xxxz(self) -> U32Vector4: ...
    @property
    def xxxw(self) -> U32Vector4: ...
    @property
    def xxxo(self) -> U32Vector4: ...
    @property
    def xxxl(self) -> U32Vector4: ...
    @property
    def xxyx(self) -> U32Vector4: ...
    @property
    def xxyy(self) -> U32Vector4: ...
    @property
    def xxyz(self) -> U32Vector4: ...
    @property
    def xxyw(self) -> U32Vector4: ...
    @property
    def xxyo(self) -> U32Vector4: ...
    @property
    def xxyl(self) -> U32Vector4: ...
    @property
    def xxzx(self) -> U32Vector4: ...
    @property
    def xxzy(self) -> U32Vector4: ...
    @property
    def xxzz(self) -> U32Vector4: ...
    @property
    def xxzw(self) -> U32Vector4: ...
    @property
    def xxzo(self) -> U32Vector4: ...
    @property
    def xxzl(self) -> U32Vector4: ...
    @property
    def xxwx(self) -> U32Vector4: ...
    @property
    def xxwy(self) -> U32Vector4: ...
    @property
    def xxwz(self) -> U32Vector4: ...
    @property
    def xxww(self) -> U32Vector4: ...
    @property
    def xxwo(self) -> U32Vector4: ...
    @property
    def xxwl(self) -> U32Vector4: ...
    @property
    def xxox(self) -> U32Vector4: ...
    @property
    def xxoy(self) -> U32Vector4: ...
    @property
    def xxoz(self) -> U32Vector4: ...
    @property
    def xxow(self) -> U32Vector4: ...
    @property
    def xxoo(self) -> U32Vector4: ...
    @property
    def xxol(self) -> U32Vector4: ...
    @property
    def xxlx(self) -> U32Vector4: ...
    @property
    def xxly(self) -> U32Vector4: ...
    @property
    def xxlz(self) -> U32Vector4: ...
    @property
    def xxlw(self) -> U32Vector4: ...
    @property
    def xxlo(self) -> U32Vector4: ...
    @property
    def xxll(self) -> U32Vector4: ...
    @property
    def xyxx(self) -> U32Vector4: ...
    @property
    def xyxy(self) -> U32Vector4: ...
    @property
    def xyxz(self) -> U32Vector4: ...
    @property
    def xyxw(self) -> U32Vector4: ...
    @property
    def xyxo(self) -> U32Vector4: ...
    @property
    def xyxl(self) -> U32Vector4: ...
    @property
    def xyyx(self) -> U32Vector4: ...
    @property
    def xyyy(self) -> U32Vector4: ...
    @property
    def xyyz(self) -> U32Vector4: ...
    @property
    def xyyw(self) -> U32Vector4: ...
    @property
    def xyyo(self) -> U32Vector4: ...
    @property
    def xyyl(self) -> U32Vector4: ...
    @property
    def xyzx(self) -> U32Vector4: ...
    @property
    def xyzy(self) -> U32Vector4: ...
    @property
    def xyzz(self) -> U32Vector4: ...
    @property
    def xyzw(self) -> U32Vector4: ...
    @property
    def xyzo(self) -> U32Vector4: ...
    @property
    def xyzl(self) -> U32Vector4: ...
    @property
    def xywx(self) -> U32Vector4: ...
    @property
    def xywy(self) -> U32Vector4: ...
    @property
    def xywz(self) -> U32Vector4: ...
    @property
    def xyww(self) -> U32Vector4: ...
    @property
    def xywo(self) -> U32Vector4: ...
    @property
    def xywl(self) -> U32Vector4: ...
    @property
    def xyox(self) -> U32Vector4: ...
    @property
    def xyoy(self) -> U32Vector4: ...
    @property
    def xyoz(self) -> U32Vector4: ...
    @property
    def xyow(self) -> U32Vector4: ...
    @property
    def xyoo(self) -> U32Vector4: ...
    @property
    def xyol(self) -> U32Vector4: ...
    @property
    def xylx(self) -> U32Vector4: ...
    @property
    def xyly(self) -> U32Vector4: ...
    @property
    def xylz(self) -> U32Vector4: ...
    @property
    def xylw(self) -> U32Vector4: ...
    @property
    def xylo(self) -> U32Vector4: ...
    @property
    def xyll(self) -> U32Vector4: ...
    @property
    def xzxx(self) -> U32Vector4: ...
    @property
    def xzxy(self) -> U32Vector4: ...
    @property
    def xzxz(self) -> U32Vector4: ...
    @property
    def xzxw(self) -> U32Vector4: ...
    @property
    def xzxo(self) -> U32Vector4: ...
    @property
    def xzxl(self) -> U32Vector4: ...
    @property
    def xzyx(self) -> U32Vector4: ...
    @property
    def xzyy(self) -> U32Vector4: ...
    @property
    def xzyz(self) -> U32Vector4: ...
    @property
    def xzyw(self) -> U32Vector4: ...
    @property
    def xzyo(self) -> U32Vector4: ...
    @property
    def xzyl(self) -> U32Vector4: ...
    @property
    def xzzx(self) -> U32Vector4: ...
    @property
    def xzzy(self) -> U32Vector4: ...
    @property
    def xzzz(self) -> U32Vector4: ...
    @property
    def xzzw(self) -> U32Vector4: ...
    @property
    def xzzo(self) -> U32Vector4: ...
    @property
    def xzzl(self) -> U32Vector4: ...
    @property
    def xzwx(self) -> U32Vector4: ...
    @property
    def xzwy(self) -> U32Vector4: ...
    @property
    def xzwz(self) -> U32Vector4: ...
    @property
    def xzww(self) -> U32Vector4: ...
    @property
    def xzwo(self) -> U32Vector4: ...
    @property
    def xzwl(self) -> U32Vector4: ...
    @property
    def xzox(self) -> U32Vector4: ...
    @property
    def xzoy(self) -> U32Vector4: ...
    @property
    def xzoz(self) -> U32Vector4: ...
    @property
    def xzow(self) -> U32Vector4: ...
    @property
    def xzoo(self) -> U32Vector4: ...
    @property
    def xzol(self) -> U32Vector4: ...
    @property
    def xzlx(self) -> U32Vector4: ...
    @property
    def xzly(self) -> U32Vector4: ...
    @property
    def xzlz(self) -> U32Vector4: ...
    @property
    def xzlw(self) -> U32Vector4: ...
    @property
    def xzlo(self) -> U32Vector4: ...
    @property
    def xzll(self) -> U32Vector4: ...
    @property
    def xwxx(self) -> U32Vector4: ...
    @property
    def xwxy(self) -> U32Vector4: ...
    @property
    def xwxz(self) -> U32Vector4: ...
    @property
    def xwxw(self) -> U32Vector4: ...
    @property
    def xwxo(self) -> U32Vector4: ...
    @property
    def xwxl(self) -> U32Vector4: ...
    @property
    def xwyx(self) -> U32Vector4: ...
    @property
    def xwyy(self) -> U32Vector4: ...
    @property
    def xwyz(self) -> U32Vector4: ...
    @property
    def xwyw(self) -> U32Vector4: ...
    @property
    def xwyo(self) -> U32Vector4: ...
    @property
    def xwyl(self) -> U32Vector4: ...
    @property
    def xwzx(self) -> U32Vector4: ...
    @property
    def xwzy(self) -> U32Vector4: ...
    @property
    def xwzz(self) -> U32Vector4: ...
    @property
    def xwzw(self) -> U32Vector4: ...
    @property
    def xwzo(self) -> U32Vector4: ...
    @property
    def xwzl(self) -> U32Vector4: ...
    @property
    def xwwx(self) -> U32Vector4: ...
    @property
    def xwwy(self) -> U32Vector4: ...
    @property
    def xwwz(self) -> U32Vector4: ...
    @property
    def xwww(self) -> U32Vector4: ...
    @property
    def xwwo(self) -> U32Vector4: ...
    @property
    def xwwl(self) -> U32Vector4: ...
    @property
    def xwox(self) -> U32Vector4: ...
    @property
    def xwoy(self) -> U32Vector4: ...
    @property
    def xwoz(self) -> U32Vector4: ...
    @property
    def xwow(self) -> U32Vector4: ...
    @property
    def xwoo(self) -> U32Vector4: ...
    @property
    def xwol(self) -> U32Vector4: ...
    @property
    def xwlx(self) -> U32Vector4: ...
    @property
    def xwly(self) -> U32Vector4: ...
    @property
    def xwlz(self) -> U32Vector4: ...
    @property
    def xwlw(self) -> U32Vector4: ...
    @property
    def xwlo(self) -> U32Vector4: ...
    @property
    def xwll(self) -> U32Vector4: ...
    @property
    def xoxx(self) -> U32Vector4: ...
    @property
    def xoxy(self) -> U32Vector4: ...
    @property
    def xoxz(self) -> U32Vector4: ...
    @property
    def xoxw(self) -> U32Vector4: ...
    @property
    def xoxo(self) -> U32Vector4: ...
    @property
    def xoxl(self) -> U32Vector4: ...
    @property
    def xoyx(self) -> U32Vector4: ...
    @property
    def xoyy(self) -> U32Vector4: ...
    @property
    def xoyz(self) -> U32Vector4: ...
    @property
    def xoyw(self) -> U32Vector4: ...
    @property
    def xoyo(self) -> U32Vector4: ...
    @property
    def xoyl(self) -> U32Vector4: ...
    @property
    def xozx(self) -> U32Vector4: ...
    @property
    def xozy(self) -> U32Vector4: ...
    @property
    def xozz(self) -> U32Vector4: ...
    @property
    def xozw(self) -> U32Vector4: ...
    @property
    def xozo(self) -> U32Vector4: ...
    @property
    def xozl(self) -> U32Vector4: ...
    @property
    def xowx(self) -> U32Vector4: ...
    @property
    def xowy(self) -> U32Vector4: ...
    @property
    def xowz(self) -> U32Vector4: ...
    @property
    def xoww(self) -> U32Vector4: ...
    @property
    def xowo(self) -> U32Vector4: ...
    @property
    def xowl(self) -> U32Vector4: ...
    @property
    def xoox(self) -> U32Vector4: ...
    @property
    def xooy(self) -> U32Vector4: ...
    @property
    def xooz(self) -> U32Vector4: ...
    @property
    def xoow(self) -> U32Vector4: ...
    @property
    def xooo(self) -> U32Vector4: ...
    @property
    def xool(self) -> U32Vector4: ...
    @property
    def xolx(self) -> U32Vector4: ...
    @property
    def xoly(self) -> U32Vector4: ...
    @property
    def xolz(self) -> U32Vector4: ...
    @property
    def xolw(self) -> U32Vector4: ...
    @property
    def xolo(self) -> U32Vector4: ...
    @property
    def xoll(self) -> U32Vector4: ...
    @property
    def xlxx(self) -> U32Vector4: ...
    @property
    def xlxy(self) -> U32Vector4: ...
    @property
    def xlxz(self) -> U32Vector4: ...
    @property
    def xlxw(self) -> U32Vector4: ...
    @property
    def xlxo(self) -> U32Vector4: ...
    @property
    def xlxl(self) -> U32Vector4: ...
    @property
    def xlyx(self) -> U32Vector4: ...
    @property
    def xlyy(self) -> U32Vector4: ...
    @property
    def xlyz(self) -> U32Vector4: ...
    @property
    def xlyw(self) -> U32Vector4: ...
    @property
    def xlyo(self) -> U32Vector4: ...
    @property
    def xlyl(self) -> U32Vector4: ...
    @property
    def xlzx(self) -> U32Vector4: ...
    @property
    def xlzy(self) -> U32Vector4: ...
    @property
    def xlzz(self) -> U32Vector4: ...
    @property
    def xlzw(self) -> U32Vector4: ...
    @property
    def xlzo(self) -> U32Vector4: ...
    @property
    def xlzl(self) -> U32Vector4: ...
    @property
    def xlwx(self) -> U32Vector4: ...
    @property
    def xlwy(self) -> U32Vector4: ...
    @property
    def xlwz(self) -> U32Vector4: ...
    @property
    def xlww(self) -> U32Vector4: ...
    @property
    def xlwo(self) -> U32Vector4: ...
    @property
    def xlwl(self) -> U32Vector4: ...
    @property
    def xlox(self) -> U32Vector4: ...
    @property
    def xloy(self) -> U32Vector4: ...
    @property
    def xloz(self) -> U32Vector4: ...
    @property
    def xlow(self) -> U32Vector4: ...
    @property
    def xloo(self) -> U32Vector4: ...
    @property
    def xlol(self) -> U32Vector4: ...
    @property
    def xllx(self) -> U32Vector4: ...
    @property
    def xlly(self) -> U32Vector4: ...
    @property
    def xllz(self) -> U32Vector4: ...
    @property
    def xllw(self) -> U32Vector4: ...
    @property
    def xllo(self) -> U32Vector4: ...
    @property
    def xlll(self) -> U32Vector4: ...
    @property
    def yxxx(self) -> U32Vector4: ...
    @property
    def yxxy(self) -> U32Vector4: ...
    @property
    def yxxz(self) -> U32Vector4: ...
    @property
    def yxxw(self) -> U32Vector4: ...
    @property
    def yxxo(self) -> U32Vector4: ...
    @property
    def yxxl(self) -> U32Vector4: ...
    @property
    def yxyx(self) -> U32Vector4: ...
    @property
    def yxyy(self) -> U32Vector4: ...
    @property
    def yxyz(self) -> U32Vector4: ...
    @property
    def yxyw(self) -> U32Vector4: ...
    @property
    def yxyo(self) -> U32Vector4: ...
    @property
    def yxyl(self) -> U32Vector4: ...
    @property
    def yxzx(self) -> U32Vector4: ...
    @property
    def yxzy(self) -> U32Vector4: ...
    @property
    def yxzz(self) -> U32Vector4: ...
    @property
    def yxzw(self) -> U32Vector4: ...
    @property
    def yxzo(self) -> U32Vector4: ...
    @property
    def yxzl(self) -> U32Vector4: ...
    @property
    def yxwx(self) -> U32Vector4: ...
    @property
    def yxwy(self) -> U32Vector4: ...
    @property
    def yxwz(self) -> U32Vector4: ...
    @property
    def yxww(self) -> U32Vector4: ...
    @property
    def yxwo(self) -> U32Vector4: ...
    @property
    def yxwl(self) -> U32Vector4: ...
    @property
    def yxox(self) -> U32Vector4: ...
    @property
    def yxoy(self) -> U32Vector4: ...
    @property
    def yxoz(self) -> U32Vector4: ...
    @property
    def yxow(self) -> U32Vector4: ...
    @property
    def yxoo(self) -> U32Vector4: ...
    @property
    def yxol(self) -> U32Vector4: ...
    @property
    def yxlx(self) -> U32Vector4: ...
    @property
    def yxly(self) -> U32Vector4: ...
    @property
    def yxlz(self) -> U32Vector4: ...
    @property
    def yxlw(self) -> U32Vector4: ...
    @property
    def yxlo(self) -> U32Vector4: ...
    @property
    def yxll(self) -> U32Vector4: ...
    @property
    def yyxx(self) -> U32Vector4: ...
    @property
    def yyxy(self) -> U32Vector4: ...
    @property
    def yyxz(self) -> U32Vector4: ...
    @property
    def yyxw(self) -> U32Vector4: ...
    @property
    def yyxo(self) -> U32Vector4: ...
    @property
    def yyxl(self) -> U32Vector4: ...
    @property
    def yyyx(self) -> U32Vector4: ...
    @property
    def yyyy(self) -> U32Vector4: ...
    @property
    def yyyz(self) -> U32Vector4: ...
    @property
    def yyyw(self) -> U32Vector4: ...
    @property
    def yyyo(self) -> U32Vector4: ...
    @property
    def yyyl(self) -> U32Vector4: ...
    @property
    def yyzx(self) -> U32Vector4: ...
    @property
    def yyzy(self) -> U32Vector4: ...
    @property
    def yyzz(self) -> U32Vector4: ...
    @property
    def yyzw(self) -> U32Vector4: ...
    @property
    def yyzo(self) -> U32Vector4: ...
    @property
    def yyzl(self) -> U32Vector4: ...
    @property
    def yywx(self) -> U32Vector4: ...
    @property
    def yywy(self) -> U32Vector4: ...
    @property
    def yywz(self) -> U32Vector4: ...
    @property
    def yyww(self) -> U32Vector4: ...
    @property
    def yywo(self) -> U32Vector4: ...
    @property
    def yywl(self) -> U32Vector4: ...
    @property
    def yyox(self) -> U32Vector4: ...
    @property
    def yyoy(self) -> U32Vector4: ...
    @property
    def yyoz(self) -> U32Vector4: ...
    @property
    def yyow(self) -> U32Vector4: ...
    @property
    def yyoo(self) -> U32Vector4: ...
    @property
    def yyol(self) -> U32Vector4: ...
    @property
    def yylx(self) -> U32Vector4: ...
    @property
    def yyly(self) -> U32Vector4: ...
    @property
    def yylz(self) -> U32Vector4: ...
    @property
    def yylw(self) -> U32Vector4: ...
    @property
    def yylo(self) -> U32Vector4: ...
    @property
    def yyll(self) -> U32Vector4: ...
    @property
    def yzxx(self) -> U32Vector4: ...
    @property
    def yzxy(self) -> U32Vector4: ...
    @property
    def yzxz(self) -> U32Vector4: ...
    @property
    def yzxw(self) -> U32Vector4: ...
    @property
    def yzxo(self) -> U32Vector4: ...
    @property
    def yzxl(self) -> U32Vector4: ...
    @property
    def yzyx(self) -> U32Vector4: ...
    @property
    def yzyy(self) -> U32Vector4: ...
    @property
    def yzyz(self) -> U32Vector4: ...
    @property
    def yzyw(self) -> U32Vector4: ...
    @property
    def yzyo(self) -> U32Vector4: ...
    @property
    def yzyl(self) -> U32Vector4: ...
    @property
    def yzzx(self) -> U32Vector4: ...
    @property
    def yzzy(self) -> U32Vector4: ...
    @property
    def yzzz(self) -> U32Vector4: ...
    @property
    def yzzw(self) -> U32Vector4: ...
    @property
    def yzzo(self) -> U32Vector4: ...
    @property
    def yzzl(self) -> U32Vector4: ...
    @property
    def yzwx(self) -> U32Vector4: ...
    @property
    def yzwy(self) -> U32Vector4: ...
    @property
    def yzwz(self) -> U32Vector4: ...
    @property
    def yzww(self) -> U32Vector4: ...
    @property
    def yzwo(self) -> U32Vector4: ...
    @property
    def yzwl(self) -> U32Vector4: ...
    @property
    def yzox(self) -> U32Vector4: ...
    @property
    def yzoy(self) -> U32Vector4: ...
    @property
    def yzoz(self) -> U32Vector4: ...
    @property
    def yzow(self) -> U32Vector4: ...
    @property
    def yzoo(self) -> U32Vector4: ...
    @property
    def yzol(self) -> U32Vector4: ...
    @property
    def yzlx(self) -> U32Vector4: ...
    @property
    def yzly(self) -> U32Vector4: ...
    @property
    def yzlz(self) -> U32Vector4: ...
    @property
    def yzlw(self) -> U32Vector4: ...
    @property
    def yzlo(self) -> U32Vector4: ...
    @property
    def yzll(self) -> U32Vector4: ...
    @property
    def ywxx(self) -> U32Vector4: ...
    @property
    def ywxy(self) -> U32Vector4: ...
    @property
    def ywxz(self) -> U32Vector4: ...
    @property
    def ywxw(self) -> U32Vector4: ...
    @property
    def ywxo(self) -> U32Vector4: ...
    @property
    def ywxl(self) -> U32Vector4: ...
    @property
    def ywyx(self) -> U32Vector4: ...
    @property
    def ywyy(self) -> U32Vector4: ...
    @property
    def ywyz(self) -> U32Vector4: ...
    @property
    def ywyw(self) -> U32Vector4: ...
    @property
    def ywyo(self) -> U32Vector4: ...
    @property
    def ywyl(self) -> U32Vector4: ...
    @property
    def ywzx(self) -> U32Vector4: ...
    @property
    def ywzy(self) -> U32Vector4: ...
    @property
    def ywzz(self) -> U32Vector4: ...
    @property
    def ywzw(self) -> U32Vector4: ...
    @property
    def ywzo(self) -> U32Vector4: ...
    @property
    def ywzl(self) -> U32Vector4: ...
    @property
    def ywwx(self) -> U32Vector4: ...
    @property
    def ywwy(self) -> U32Vector4: ...
    @property
    def ywwz(self) -> U32Vector4: ...
    @property
    def ywww(self) -> U32Vector4: ...
    @property
    def ywwo(self) -> U32Vector4: ...
    @property
    def ywwl(self) -> U32Vector4: ...
    @property
    def ywox(self) -> U32Vector4: ...
    @property
    def ywoy(self) -> U32Vector4: ...
    @property
    def ywoz(self) -> U32Vector4: ...
    @property
    def ywow(self) -> U32Vector4: ...
    @property
    def ywoo(self) -> U32Vector4: ...
    @property
    def ywol(self) -> U32Vector4: ...
    @property
    def ywlx(self) -> U32Vector4: ...
    @property
    def ywly(self) -> U32Vector4: ...
    @property
    def ywlz(self) -> U32Vector4: ...
    @property
    def ywlw(self) -> U32Vector4: ...
    @property
    def ywlo(self) -> U32Vector4: ...
    @property
    def ywll(self) -> U32Vector4: ...
    @property
    def yoxx(self) -> U32Vector4: ...
    @property
    def yoxy(self) -> U32Vector4: ...
    @property
    def yoxz(self) -> U32Vector4: ...
    @property
    def yoxw(self) -> U32Vector4: ...
    @property
    def yoxo(self) -> U32Vector4: ...
    @property
    def yoxl(self) -> U32Vector4: ...
    @property
    def yoyx(self) -> U32Vector4: ...
    @property
    def yoyy(self) -> U32Vector4: ...
    @property
    def yoyz(self) -> U32Vector4: ...
    @property
    def yoyw(self) -> U32Vector4: ...
    @property
    def yoyo(self) -> U32Vector4: ...
    @property
    def yoyl(self) -> U32Vector4: ...
    @property
    def yozx(self) -> U32Vector4: ...
    @property
    def yozy(self) -> U32Vector4: ...
    @property
    def yozz(self) -> U32Vector4: ...
    @property
    def yozw(self) -> U32Vector4: ...
    @property
    def yozo(self) -> U32Vector4: ...
    @property
    def yozl(self) -> U32Vector4: ...
    @property
    def yowx(self) -> U32Vector4: ...
    @property
    def yowy(self) -> U32Vector4: ...
    @property
    def yowz(self) -> U32Vector4: ...
    @property
    def yoww(self) -> U32Vector4: ...
    @property
    def yowo(self) -> U32Vector4: ...
    @property
    def yowl(self) -> U32Vector4: ...
    @property
    def yoox(self) -> U32Vector4: ...
    @property
    def yooy(self) -> U32Vector4: ...
    @property
    def yooz(self) -> U32Vector4: ...
    @property
    def yoow(self) -> U32Vector4: ...
    @property
    def yooo(self) -> U32Vector4: ...
    @property
    def yool(self) -> U32Vector4: ...
    @property
    def yolx(self) -> U32Vector4: ...
    @property
    def yoly(self) -> U32Vector4: ...
    @property
    def yolz(self) -> U32Vector4: ...
    @property
    def yolw(self) -> U32Vector4: ...
    @property
    def yolo(self) -> U32Vector4: ...
    @property
    def yoll(self) -> U32Vector4: ...
    @property
    def ylxx(self) -> U32Vector4: ...
    @property
    def ylxy(self) -> U32Vector4: ...
    @property
    def ylxz(self) -> U32Vector4: ...
    @property
    def ylxw(self) -> U32Vector4: ...
    @property
    def ylxo(self) -> U32Vector4: ...
    @property
    def ylxl(self) -> U32Vector4: ...
    @property
    def ylyx(self) -> U32Vector4: ...
    @property
    def ylyy(self) -> U32Vector4: ...
    @property
    def ylyz(self) -> U32Vector4: ...
    @property
    def ylyw(self) -> U32Vector4: ...
    @property
    def ylyo(self) -> U32Vector4: ...
    @property
    def ylyl(self) -> U32Vector4: ...
    @property
    def ylzx(self) -> U32Vector4: ...
    @property
    def ylzy(self) -> U32Vector4: ...
    @property
    def ylzz(self) -> U32Vector4: ...
    @property
    def ylzw(self) -> U32Vector4: ...
    @property
    def ylzo(self) -> U32Vector4: ...
    @property
    def ylzl(self) -> U32Vector4: ...
    @property
    def ylwx(self) -> U32Vector4: ...
    @property
    def ylwy(self) -> U32Vector4: ...
    @property
    def ylwz(self) -> U32Vector4: ...
    @property
    def ylww(self) -> U32Vector4: ...
    @property
    def ylwo(self) -> U32Vector4: ...
    @property
    def ylwl(self) -> U32Vector4: ...
    @property
    def ylox(self) -> U32Vector4: ...
    @property
    def yloy(self) -> U32Vector4: ...
    @property
    def yloz(self) -> U32Vector4: ...
    @property
    def ylow(self) -> U32Vector4: ...
    @property
    def yloo(self) -> U32Vector4: ...
    @property
    def ylol(self) -> U32Vector4: ...
    @property
    def yllx(self) -> U32Vector4: ...
    @property
    def ylly(self) -> U32Vector4: ...
    @property
    def yllz(self) -> U32Vector4: ...
    @property
    def yllw(self) -> U32Vector4: ...
    @property
    def yllo(self) -> U32Vector4: ...
    @property
    def ylll(self) -> U32Vector4: ...
    @property
    def zxxx(self) -> U32Vector4: ...
    @property
    def zxxy(self) -> U32Vector4: ...
    @property
    def zxxz(self) -> U32Vector4: ...
    @property
    def zxxw(self) -> U32Vector4: ...
    @property
    def zxxo(self) -> U32Vector4: ...
    @property
    def zxxl(self) -> U32Vector4: ...
    @property
    def zxyx(self) -> U32Vector4: ...
    @property
    def zxyy(self) -> U32Vector4: ...
    @property
    def zxyz(self) -> U32Vector4: ...
    @property
    def zxyw(self) -> U32Vector4: ...
    @property
    def zxyo(self) -> U32Vector4: ...
    @property
    def zxyl(self) -> U32Vector4: ...
    @property
    def zxzx(self) -> U32Vector4: ...
    @property
    def zxzy(self) -> U32Vector4: ...
    @property
    def zxzz(self) -> U32Vector4: ...
    @property
    def zxzw(self) -> U32Vector4: ...
    @property
    def zxzo(self) -> U32Vector4: ...
    @property
    def zxzl(self) -> U32Vector4: ...
    @property
    def zxwx(self) -> U32Vector4: ...
    @property
    def zxwy(self) -> U32Vector4: ...
    @property
    def zxwz(self) -> U32Vector4: ...
    @property
    def zxww(self) -> U32Vector4: ...
    @property
    def zxwo(self) -> U32Vector4: ...
    @property
    def zxwl(self) -> U32Vector4: ...
    @property
    def zxox(self) -> U32Vector4: ...
    @property
    def zxoy(self) -> U32Vector4: ...
    @property
    def zxoz(self) -> U32Vector4: ...
    @property
    def zxow(self) -> U32Vector4: ...
    @property
    def zxoo(self) -> U32Vector4: ...
    @property
    def zxol(self) -> U32Vector4: ...
    @property
    def zxlx(self) -> U32Vector4: ...
    @property
    def zxly(self) -> U32Vector4: ...
    @property
    def zxlz(self) -> U32Vector4: ...
    @property
    def zxlw(self) -> U32Vector4: ...
    @property
    def zxlo(self) -> U32Vector4: ...
    @property
    def zxll(self) -> U32Vector4: ...
    @property
    def zyxx(self) -> U32Vector4: ...
    @property
    def zyxy(self) -> U32Vector4: ...
    @property
    def zyxz(self) -> U32Vector4: ...
    @property
    def zyxw(self) -> U32Vector4: ...
    @property
    def zyxo(self) -> U32Vector4: ...
    @property
    def zyxl(self) -> U32Vector4: ...
    @property
    def zyyx(self) -> U32Vector4: ...
    @property
    def zyyy(self) -> U32Vector4: ...
    @property
    def zyyz(self) -> U32Vector4: ...
    @property
    def zyyw(self) -> U32Vector4: ...
    @property
    def zyyo(self) -> U32Vector4: ...
    @property
    def zyyl(self) -> U32Vector4: ...
    @property
    def zyzx(self) -> U32Vector4: ...
    @property
    def zyzy(self) -> U32Vector4: ...
    @property
    def zyzz(self) -> U32Vector4: ...
    @property
    def zyzw(self) -> U32Vector4: ...
    @property
    def zyzo(self) -> U32Vector4: ...
    @property
    def zyzl(self) -> U32Vector4: ...
    @property
    def zywx(self) -> U32Vector4: ...
    @property
    def zywy(self) -> U32Vector4: ...
    @property
    def zywz(self) -> U32Vector4: ...
    @property
    def zyww(self) -> U32Vector4: ...
    @property
    def zywo(self) -> U32Vector4: ...
    @property
    def zywl(self) -> U32Vector4: ...
    @property
    def zyox(self) -> U32Vector4: ...
    @property
    def zyoy(self) -> U32Vector4: ...
    @property
    def zyoz(self) -> U32Vector4: ...
    @property
    def zyow(self) -> U32Vector4: ...
    @property
    def zyoo(self) -> U32Vector4: ...
    @property
    def zyol(self) -> U32Vector4: ...
    @property
    def zylx(self) -> U32Vector4: ...
    @property
    def zyly(self) -> U32Vector4: ...
    @property
    def zylz(self) -> U32Vector4: ...
    @property
    def zylw(self) -> U32Vector4: ...
    @property
    def zylo(self) -> U32Vector4: ...
    @property
    def zyll(self) -> U32Vector4: ...
    @property
    def zzxx(self) -> U32Vector4: ...
    @property
    def zzxy(self) -> U32Vector4: ...
    @property
    def zzxz(self) -> U32Vector4: ...
    @property
    def zzxw(self) -> U32Vector4: ...
    @property
    def zzxo(self) -> U32Vector4: ...
    @property
    def zzxl(self) -> U32Vector4: ...
    @property
    def zzyx(self) -> U32Vector4: ...
    @property
    def zzyy(self) -> U32Vector4: ...
    @property
    def zzyz(self) -> U32Vector4: ...
    @property
    def zzyw(self) -> U32Vector4: ...
    @property
    def zzyo(self) -> U32Vector4: ...
    @property
    def zzyl(self) -> U32Vector4: ...
    @property
    def zzzx(self) -> U32Vector4: ...
    @property
    def zzzy(self) -> U32Vector4: ...
    @property
    def zzzz(self) -> U32Vector4: ...
    @property
    def zzzw(self) -> U32Vector4: ...
    @property
    def zzzo(self) -> U32Vector4: ...
    @property
    def zzzl(self) -> U32Vector4: ...
    @property
    def zzwx(self) -> U32Vector4: ...
    @property
    def zzwy(self) -> U32Vector4: ...
    @property
    def zzwz(self) -> U32Vector4: ...
    @property
    def zzww(self) -> U32Vector4: ...
    @property
    def zzwo(self) -> U32Vector4: ...
    @property
    def zzwl(self) -> U32Vector4: ...
    @property
    def zzox(self) -> U32Vector4: ...
    @property
    def zzoy(self) -> U32Vector4: ...
    @property
    def zzoz(self) -> U32Vector4: ...
    @property
    def zzow(self) -> U32Vector4: ...
    @property
    def zzoo(self) -> U32Vector4: ...
    @property
    def zzol(self) -> U32Vector4: ...
    @property
    def zzlx(self) -> U32Vector4: ...
    @property
    def zzly(self) -> U32Vector4: ...
    @property
    def zzlz(self) -> U32Vector4: ...
    @property
    def zzlw(self) -> U32Vector4: ...
    @property
    def zzlo(self) -> U32Vector4: ...
    @property
    def zzll(self) -> U32Vector4: ...
    @property
    def zwxx(self) -> U32Vector4: ...
    @property
    def zwxy(self) -> U32Vector4: ...
    @property
    def zwxz(self) -> U32Vector4: ...
    @property
    def zwxw(self) -> U32Vector4: ...
    @property
    def zwxo(self) -> U32Vector4: ...
    @property
    def zwxl(self) -> U32Vector4: ...
    @property
    def zwyx(self) -> U32Vector4: ...
    @property
    def zwyy(self) -> U32Vector4: ...
    @property
    def zwyz(self) -> U32Vector4: ...
    @property
    def zwyw(self) -> U32Vector4: ...
    @property
    def zwyo(self) -> U32Vector4: ...
    @property
    def zwyl(self) -> U32Vector4: ...
    @property
    def zwzx(self) -> U32Vector4: ...
    @property
    def zwzy(self) -> U32Vector4: ...
    @property
    def zwzz(self) -> U32Vector4: ...
    @property
    def zwzw(self) -> U32Vector4: ...
    @property
    def zwzo(self) -> U32Vector4: ...
    @property
    def zwzl(self) -> U32Vector4: ...
    @property
    def zwwx(self) -> U32Vector4: ...
    @property
    def zwwy(self) -> U32Vector4: ...
    @property
    def zwwz(self) -> U32Vector4: ...
    @property
    def zwww(self) -> U32Vector4: ...
    @property
    def zwwo(self) -> U32Vector4: ...
    @property
    def zwwl(self) -> U32Vector4: ...
    @property
    def zwox(self) -> U32Vector4: ...
    @property
    def zwoy(self) -> U32Vector4: ...
    @property
    def zwoz(self) -> U32Vector4: ...
    @property
    def zwow(self) -> U32Vector4: ...
    @property
    def zwoo(self) -> U32Vector4: ...
    @property
    def zwol(self) -> U32Vector4: ...
    @property
    def zwlx(self) -> U32Vector4: ...
    @property
    def zwly(self) -> U32Vector4: ...
    @property
    def zwlz(self) -> U32Vector4: ...
    @property
    def zwlw(self) -> U32Vector4: ...
    @property
    def zwlo(self) -> U32Vector4: ...
    @property
    def zwll(self) -> U32Vector4: ...
    @property
    def zoxx(self) -> U32Vector4: ...
    @property
    def zoxy(self) -> U32Vector4: ...
    @property
    def zoxz(self) -> U32Vector4: ...
    @property
    def zoxw(self) -> U32Vector4: ...
    @property
    def zoxo(self) -> U32Vector4: ...
    @property
    def zoxl(self) -> U32Vector4: ...
    @property
    def zoyx(self) -> U32Vector4: ...
    @property
    def zoyy(self) -> U32Vector4: ...
    @property
    def zoyz(self) -> U32Vector4: ...
    @property
    def zoyw(self) -> U32Vector4: ...
    @property
    def zoyo(self) -> U32Vector4: ...
    @property
    def zoyl(self) -> U32Vector4: ...
    @property
    def zozx(self) -> U32Vector4: ...
    @property
    def zozy(self) -> U32Vector4: ...
    @property
    def zozz(self) -> U32Vector4: ...
    @property
    def zozw(self) -> U32Vector4: ...
    @property
    def zozo(self) -> U32Vector4: ...
    @property
    def zozl(self) -> U32Vector4: ...
    @property
    def zowx(self) -> U32Vector4: ...
    @property
    def zowy(self) -> U32Vector4: ...
    @property
    def zowz(self) -> U32Vector4: ...
    @property
    def zoww(self) -> U32Vector4: ...
    @property
    def zowo(self) -> U32Vector4: ...
    @property
    def zowl(self) -> U32Vector4: ...
    @property
    def zoox(self) -> U32Vector4: ...
    @property
    def zooy(self) -> U32Vector4: ...
    @property
    def zooz(self) -> U32Vector4: ...
    @property
    def zoow(self) -> U32Vector4: ...
    @property
    def zooo(self) -> U32Vector4: ...
    @property
    def zool(self) -> U32Vector4: ...
    @property
    def zolx(self) -> U32Vector4: ...
    @property
    def zoly(self) -> U32Vector4: ...
    @property
    def zolz(self) -> U32Vector4: ...
    @property
    def zolw(self) -> U32Vector4: ...
    @property
    def zolo(self) -> U32Vector4: ...
    @property
    def zoll(self) -> U32Vector4: ...
    @property
    def zlxx(self) -> U32Vector4: ...
    @property
    def zlxy(self) -> U32Vector4: ...
    @property
    def zlxz(self) -> U32Vector4: ...
    @property
    def zlxw(self) -> U32Vector4: ...
    @property
    def zlxo(self) -> U32Vector4: ...
    @property
    def zlxl(self) -> U32Vector4: ...
    @property
    def zlyx(self) -> U32Vector4: ...
    @property
    def zlyy(self) -> U32Vector4: ...
    @property
    def zlyz(self) -> U32Vector4: ...
    @property
    def zlyw(self) -> U32Vector4: ...
    @property
    def zlyo(self) -> U32Vector4: ...
    @property
    def zlyl(self) -> U32Vector4: ...
    @property
    def zlzx(self) -> U32Vector4: ...
    @property
    def zlzy(self) -> U32Vector4: ...
    @property
    def zlzz(self) -> U32Vector4: ...
    @property
    def zlzw(self) -> U32Vector4: ...
    @property
    def zlzo(self) -> U32Vector4: ...
    @property
    def zlzl(self) -> U32Vector4: ...
    @property
    def zlwx(self) -> U32Vector4: ...
    @property
    def zlwy(self) -> U32Vector4: ...
    @property
    def zlwz(self) -> U32Vector4: ...
    @property
    def zlww(self) -> U32Vector4: ...
    @property
    def zlwo(self) -> U32Vector4: ...
    @property
    def zlwl(self) -> U32Vector4: ...
    @property
    def zlox(self) -> U32Vector4: ...
    @property
    def zloy(self) -> U32Vector4: ...
    @property
    def zloz(self) -> U32Vector4: ...
    @property
    def zlow(self) -> U32Vector4: ...
    @property
    def zloo(self) -> U32Vector4: ...
    @property
    def zlol(self) -> U32Vector4: ...
    @property
    def zllx(self) -> U32Vector4: ...
    @property
    def zlly(self) -> U32Vector4: ...
    @property
    def zllz(self) -> U32Vector4: ...
    @property
    def zllw(self) -> U32Vector4: ...
    @property
    def zllo(self) -> U32Vector4: ...
    @property
    def zlll(self) -> U32Vector4: ...
    @property
    def wxxx(self) -> U32Vector4: ...
    @property
    def wxxy(self) -> U32Vector4: ...
    @property
    def wxxz(self) -> U32Vector4: ...
    @property
    def wxxw(self) -> U32Vector4: ...
    @property
    def wxxo(self) -> U32Vector4: ...
    @property
    def wxxl(self) -> U32Vector4: ...
    @property
    def wxyx(self) -> U32Vector4: ...
    @property
    def wxyy(self) -> U32Vector4: ...
    @property
    def wxyz(self) -> U32Vector4: ...
    @property
    def wxyw(self) -> U32Vector4: ...
    @property
    def wxyo(self) -> U32Vector4: ...
    @property
    def wxyl(self) -> U32Vector4: ...
    @property
    def wxzx(self) -> U32Vector4: ...
    @property
    def wxzy(self) -> U32Vector4: ...
    @property
    def wxzz(self) -> U32Vector4: ...
    @property
    def wxzw(self) -> U32Vector4: ...
    @property
    def wxzo(self) -> U32Vector4: ...
    @property
    def wxzl(self) -> U32Vector4: ...
    @property
    def wxwx(self) -> U32Vector4: ...
    @property
    def wxwy(self) -> U32Vector4: ...
    @property
    def wxwz(self) -> U32Vector4: ...
    @property
    def wxww(self) -> U32Vector4: ...
    @property
    def wxwo(self) -> U32Vector4: ...
    @property
    def wxwl(self) -> U32Vector4: ...
    @property
    def wxox(self) -> U32Vector4: ...
    @property
    def wxoy(self) -> U32Vector4: ...
    @property
    def wxoz(self) -> U32Vector4: ...
    @property
    def wxow(self) -> U32Vector4: ...
    @property
    def wxoo(self) -> U32Vector4: ...
    @property
    def wxol(self) -> U32Vector4: ...
    @property
    def wxlx(self) -> U32Vector4: ...
    @property
    def wxly(self) -> U32Vector4: ...
    @property
    def wxlz(self) -> U32Vector4: ...
    @property
    def wxlw(self) -> U32Vector4: ...
    @property
    def wxlo(self) -> U32Vector4: ...
    @property
    def wxll(self) -> U32Vector4: ...
    @property
    def wyxx(self) -> U32Vector4: ...
    @property
    def wyxy(self) -> U32Vector4: ...
    @property
    def wyxz(self) -> U32Vector4: ...
    @property
    def wyxw(self) -> U32Vector4: ...
    @property
    def wyxo(self) -> U32Vector4: ...
    @property
    def wyxl(self) -> U32Vector4: ...
    @property
    def wyyx(self) -> U32Vector4: ...
    @property
    def wyyy(self) -> U32Vector4: ...
    @property
    def wyyz(self) -> U32Vector4: ...
    @property
    def wyyw(self) -> U32Vector4: ...
    @property
    def wyyo(self) -> U32Vector4: ...
    @property
    def wyyl(self) -> U32Vector4: ...
    @property
    def wyzx(self) -> U32Vector4: ...
    @property
    def wyzy(self) -> U32Vector4: ...
    @property
    def wyzz(self) -> U32Vector4: ...
    @property
    def wyzw(self) -> U32Vector4: ...
    @property
    def wyzo(self) -> U32Vector4: ...
    @property
    def wyzl(self) -> U32Vector4: ...
    @property
    def wywx(self) -> U32Vector4: ...
    @property
    def wywy(self) -> U32Vector4: ...
    @property
    def wywz(self) -> U32Vector4: ...
    @property
    def wyww(self) -> U32Vector4: ...
    @property
    def wywo(self) -> U32Vector4: ...
    @property
    def wywl(self) -> U32Vector4: ...
    @property
    def wyox(self) -> U32Vector4: ...
    @property
    def wyoy(self) -> U32Vector4: ...
    @property
    def wyoz(self) -> U32Vector4: ...
    @property
    def wyow(self) -> U32Vector4: ...
    @property
    def wyoo(self) -> U32Vector4: ...
    @property
    def wyol(self) -> U32Vector4: ...
    @property
    def wylx(self) -> U32Vector4: ...
    @property
    def wyly(self) -> U32Vector4: ...
    @property
    def wylz(self) -> U32Vector4: ...
    @property
    def wylw(self) -> U32Vector4: ...
    @property
    def wylo(self) -> U32Vector4: ...
    @property
    def wyll(self) -> U32Vector4: ...
    @property
    def wzxx(self) -> U32Vector4: ...
    @property
    def wzxy(self) -> U32Vector4: ...
    @property
    def wzxz(self) -> U32Vector4: ...
    @property
    def wzxw(self) -> U32Vector4: ...
    @property
    def wzxo(self) -> U32Vector4: ...
    @property
    def wzxl(self) -> U32Vector4: ...
    @property
    def wzyx(self) -> U32Vector4: ...
    @property
    def wzyy(self) -> U32Vector4: ...
    @property
    def wzyz(self) -> U32Vector4: ...
    @property
    def wzyw(self) -> U32Vector4: ...
    @property
    def wzyo(self) -> U32Vector4: ...
    @property
    def wzyl(self) -> U32Vector4: ...
    @property
    def wzzx(self) -> U32Vector4: ...
    @property
    def wzzy(self) -> U32Vector4: ...
    @property
    def wzzz(self) -> U32Vector4: ...
    @property
    def wzzw(self) -> U32Vector4: ...
    @property
    def wzzo(self) -> U32Vector4: ...
    @property
    def wzzl(self) -> U32Vector4: ...
    @property
    def wzwx(self) -> U32Vector4: ...
    @property
    def wzwy(self) -> U32Vector4: ...
    @property
    def wzwz(self) -> U32Vector4: ...
    @property
    def wzww(self) -> U32Vector4: ...
    @property
    def wzwo(self) -> U32Vector4: ...
    @property
    def wzwl(self) -> U32Vector4: ...
    @property
    def wzox(self) -> U32Vector4: ...
    @property
    def wzoy(self) -> U32Vector4: ...
    @property
    def wzoz(self) -> U32Vector4: ...
    @property
    def wzow(self) -> U32Vector4: ...
    @property
    def wzoo(self) -> U32Vector4: ...
    @property
    def wzol(self) -> U32Vector4: ...
    @property
    def wzlx(self) -> U32Vector4: ...
    @property
    def wzly(self) -> U32Vector4: ...
    @property
    def wzlz(self) -> U32Vector4: ...
    @property
    def wzlw(self) -> U32Vector4: ...
    @property
    def wzlo(self) -> U32Vector4: ...
    @property
    def wzll(self) -> U32Vector4: ...
    @property
    def wwxx(self) -> U32Vector4: ...
    @property
    def wwxy(self) -> U32Vector4: ...
    @property
    def wwxz(self) -> U32Vector4: ...
    @property
    def wwxw(self) -> U32Vector4: ...
    @property
    def wwxo(self) -> U32Vector4: ...
    @property
    def wwxl(self) -> U32Vector4: ...
    @property
    def wwyx(self) -> U32Vector4: ...
    @property
    def wwyy(self) -> U32Vector4: ...
    @property
    def wwyz(self) -> U32Vector4: ...
    @property
    def wwyw(self) -> U32Vector4: ...
    @property
    def wwyo(self) -> U32Vector4: ...
    @property
    def wwyl(self) -> U32Vector4: ...
    @property
    def wwzx(self) -> U32Vector4: ...
    @property
    def wwzy(self) -> U32Vector4: ...
    @property
    def wwzz(self) -> U32Vector4: ...
    @property
    def wwzw(self) -> U32Vector4: ...
    @property
    def wwzo(self) -> U32Vector4: ...
    @property
    def wwzl(self) -> U32Vector4: ...
    @property
    def wwwx(self) -> U32Vector4: ...
    @property
    def wwwy(self) -> U32Vector4: ...
    @property
    def wwwz(self) -> U32Vector4: ...
    @property
    def wwww(self) -> U32Vector4: ...
    @property
    def wwwo(self) -> U32Vector4: ...
    @property
    def wwwl(self) -> U32Vector4: ...
    @property
    def wwox(self) -> U32Vector4: ...
    @property
    def wwoy(self) -> U32Vector4: ...
    @property
    def wwoz(self) -> U32Vector4: ...
    @property
    def wwow(self) -> U32Vector4: ...
    @property
    def wwoo(self) -> U32Vector4: ...
    @property
    def wwol(self) -> U32Vector4: ...
    @property
    def wwlx(self) -> U32Vector4: ...
    @property
    def wwly(self) -> U32Vector4: ...
    @property
    def wwlz(self) -> U32Vector4: ...
    @property
    def wwlw(self) -> U32Vector4: ...
    @property
    def wwlo(self) -> U32Vector4: ...
    @property
    def wwll(self) -> U32Vector4: ...
    @property
    def woxx(self) -> U32Vector4: ...
    @property
    def woxy(self) -> U32Vector4: ...
    @property
    def woxz(self) -> U32Vector4: ...
    @property
    def woxw(self) -> U32Vector4: ...
    @property
    def woxo(self) -> U32Vector4: ...
    @property
    def woxl(self) -> U32Vector4: ...
    @property
    def woyx(self) -> U32Vector4: ...
    @property
    def woyy(self) -> U32Vector4: ...
    @property
    def woyz(self) -> U32Vector4: ...
    @property
    def woyw(self) -> U32Vector4: ...
    @property
    def woyo(self) -> U32Vector4: ...
    @property
    def woyl(self) -> U32Vector4: ...
    @property
    def wozx(self) -> U32Vector4: ...
    @property
    def wozy(self) -> U32Vector4: ...
    @property
    def wozz(self) -> U32Vector4: ...
    @property
    def wozw(self) -> U32Vector4: ...
    @property
    def wozo(self) -> U32Vector4: ...
    @property
    def wozl(self) -> U32Vector4: ...
    @property
    def wowx(self) -> U32Vector4: ...
    @property
    def wowy(self) -> U32Vector4: ...
    @property
    def wowz(self) -> U32Vector4: ...
    @property
    def woww(self) -> U32Vector4: ...
    @property
    def wowo(self) -> U32Vector4: ...
    @property
    def wowl(self) -> U32Vector4: ...
    @property
    def woox(self) -> U32Vector4: ...
    @property
    def wooy(self) -> U32Vector4: ...
    @property
    def wooz(self) -> U32Vector4: ...
    @property
    def woow(self) -> U32Vector4: ...
    @property
    def wooo(self) -> U32Vector4: ...
    @property
    def wool(self) -> U32Vector4: ...
    @property
    def wolx(self) -> U32Vector4: ...
    @property
    def woly(self) -> U32Vector4: ...
    @property
    def wolz(self) -> U32Vector4: ...
    @property
    def wolw(self) -> U32Vector4: ...
    @property
    def wolo(self) -> U32Vector4: ...
    @property
    def woll(self) -> U32Vector4: ...
    @property
    def wlxx(self) -> U32Vector4: ...
    @property
    def wlxy(self) -> U32Vector4: ...
    @property
    def wlxz(self) -> U32Vector4: ...
    @property
    def wlxw(self) -> U32Vector4: ...
    @property
    def wlxo(self) -> U32Vector4: ...
    @property
    def wlxl(self) -> U32Vector4: ...
    @property
    def wlyx(self) -> U32Vector4: ...
    @property
    def wlyy(self) -> U32Vector4: ...
    @property
    def wlyz(self) -> U32Vector4: ...
    @property
    def wlyw(self) -> U32Vector4: ...
    @property
    def wlyo(self) -> U32Vector4: ...
    @property
    def wlyl(self) -> U32Vector4: ...
    @property
    def wlzx(self) -> U32Vector4: ...
    @property
    def wlzy(self) -> U32Vector4: ...
    @property
    def wlzz(self) -> U32Vector4: ...
    @property
    def wlzw(self) -> U32Vector4: ...
    @property
    def wlzo(self) -> U32Vector4: ...
    @property
    def wlzl(self) -> U32Vector4: ...
    @property
    def wlwx(self) -> U32Vector4: ...
    @property
    def wlwy(self) -> U32Vector4: ...
    @property
    def wlwz(self) -> U32Vector4: ...
    @property
    def wlww(self) -> U32Vector4: ...
    @property
    def wlwo(self) -> U32Vector4: ...
    @property
    def wlwl(self) -> U32Vector4: ...
    @property
    def wlox(self) -> U32Vector4: ...
    @property
    def wloy(self) -> U32Vector4: ...
    @property
    def wloz(self) -> U32Vector4: ...
    @property
    def wlow(self) -> U32Vector4: ...
    @property
    def wloo(self) -> U32Vector4: ...
    @property
    def wlol(self) -> U32Vector4: ...
    @property
    def wllx(self) -> U32Vector4: ...
    @property
    def wlly(self) -> U32Vector4: ...
    @property
    def wllz(self) -> U32Vector4: ...
    @property
    def wllw(self) -> U32Vector4: ...
    @property
    def wllo(self) -> U32Vector4: ...
    @property
    def wlll(self) -> U32Vector4: ...
    @property
    def oxxx(self) -> U32Vector4: ...
    @property
    def oxxy(self) -> U32Vector4: ...
    @property
    def oxxz(self) -> U32Vector4: ...
    @property
    def oxxw(self) -> U32Vector4: ...
    @property
    def oxxo(self) -> U32Vector4: ...
    @property
    def oxxl(self) -> U32Vector4: ...
    @property
    def oxyx(self) -> U32Vector4: ...
    @property
    def oxyy(self) -> U32Vector4: ...
    @property
    def oxyz(self) -> U32Vector4: ...
    @property
    def oxyw(self) -> U32Vector4: ...
    @property
    def oxyo(self) -> U32Vector4: ...
    @property
    def oxyl(self) -> U32Vector4: ...
    @property
    def oxzx(self) -> U32Vector4: ...
    @property
    def oxzy(self) -> U32Vector4: ...
    @property
    def oxzz(self) -> U32Vector4: ...
    @property
    def oxzw(self) -> U32Vector4: ...
    @property
    def oxzo(self) -> U32Vector4: ...
    @property
    def oxzl(self) -> U32Vector4: ...
    @property
    def oxwx(self) -> U32Vector4: ...
    @property
    def oxwy(self) -> U32Vector4: ...
    @property
    def oxwz(self) -> U32Vector4: ...
    @property
    def oxww(self) -> U32Vector4: ...
    @property
    def oxwo(self) -> U32Vector4: ...
    @property
    def oxwl(self) -> U32Vector4: ...
    @property
    def oxox(self) -> U32Vector4: ...
    @property
    def oxoy(self) -> U32Vector4: ...
    @property
    def oxoz(self) -> U32Vector4: ...
    @property
    def oxow(self) -> U32Vector4: ...
    @property
    def oxoo(self) -> U32Vector4: ...
    @property
    def oxol(self) -> U32Vector4: ...
    @property
    def oxlx(self) -> U32Vector4: ...
    @property
    def oxly(self) -> U32Vector4: ...
    @property
    def oxlz(self) -> U32Vector4: ...
    @property
    def oxlw(self) -> U32Vector4: ...
    @property
    def oxlo(self) -> U32Vector4: ...
    @property
    def oxll(self) -> U32Vector4: ...
    @property
    def oyxx(self) -> U32Vector4: ...
    @property
    def oyxy(self) -> U32Vector4: ...
    @property
    def oyxz(self) -> U32Vector4: ...
    @property
    def oyxw(self) -> U32Vector4: ...
    @property
    def oyxo(self) -> U32Vector4: ...
    @property
    def oyxl(self) -> U32Vector4: ...
    @property
    def oyyx(self) -> U32Vector4: ...
    @property
    def oyyy(self) -> U32Vector4: ...
    @property
    def oyyz(self) -> U32Vector4: ...
    @property
    def oyyw(self) -> U32Vector4: ...
    @property
    def oyyo(self) -> U32Vector4: ...
    @property
    def oyyl(self) -> U32Vector4: ...
    @property
    def oyzx(self) -> U32Vector4: ...
    @property
    def oyzy(self) -> U32Vector4: ...
    @property
    def oyzz(self) -> U32Vector4: ...
    @property
    def oyzw(self) -> U32Vector4: ...
    @property
    def oyzo(self) -> U32Vector4: ...
    @property
    def oyzl(self) -> U32Vector4: ...
    @property
    def oywx(self) -> U32Vector4: ...
    @property
    def oywy(self) -> U32Vector4: ...
    @property
    def oywz(self) -> U32Vector4: ...
    @property
    def oyww(self) -> U32Vector4: ...
    @property
    def oywo(self) -> U32Vector4: ...
    @property
    def oywl(self) -> U32Vector4: ...
    @property
    def oyox(self) -> U32Vector4: ...
    @property
    def oyoy(self) -> U32Vector4: ...
    @property
    def oyoz(self) -> U32Vector4: ...
    @property
    def oyow(self) -> U32Vector4: ...
    @property
    def oyoo(self) -> U32Vector4: ...
    @property
    def oyol(self) -> U32Vector4: ...
    @property
    def oylx(self) -> U32Vector4: ...
    @property
    def oyly(self) -> U32Vector4: ...
    @property
    def oylz(self) -> U32Vector4: ...
    @property
    def oylw(self) -> U32Vector4: ...
    @property
    def oylo(self) -> U32Vector4: ...
    @property
    def oyll(self) -> U32Vector4: ...
    @property
    def ozxx(self) -> U32Vector4: ...
    @property
    def ozxy(self) -> U32Vector4: ...
    @property
    def ozxz(self) -> U32Vector4: ...
    @property
    def ozxw(self) -> U32Vector4: ...
    @property
    def ozxo(self) -> U32Vector4: ...
    @property
    def ozxl(self) -> U32Vector4: ...
    @property
    def ozyx(self) -> U32Vector4: ...
    @property
    def ozyy(self) -> U32Vector4: ...
    @property
    def ozyz(self) -> U32Vector4: ...
    @property
    def ozyw(self) -> U32Vector4: ...
    @property
    def ozyo(self) -> U32Vector4: ...
    @property
    def ozyl(self) -> U32Vector4: ...
    @property
    def ozzx(self) -> U32Vector4: ...
    @property
    def ozzy(self) -> U32Vector4: ...
    @property
    def ozzz(self) -> U32Vector4: ...
    @property
    def ozzw(self) -> U32Vector4: ...
    @property
    def ozzo(self) -> U32Vector4: ...
    @property
    def ozzl(self) -> U32Vector4: ...
    @property
    def ozwx(self) -> U32Vector4: ...
    @property
    def ozwy(self) -> U32Vector4: ...
    @property
    def ozwz(self) -> U32Vector4: ...
    @property
    def ozww(self) -> U32Vector4: ...
    @property
    def ozwo(self) -> U32Vector4: ...
    @property
    def ozwl(self) -> U32Vector4: ...
    @property
    def ozox(self) -> U32Vector4: ...
    @property
    def ozoy(self) -> U32Vector4: ...
    @property
    def ozoz(self) -> U32Vector4: ...
    @property
    def ozow(self) -> U32Vector4: ...
    @property
    def ozoo(self) -> U32Vector4: ...
    @property
    def ozol(self) -> U32Vector4: ...
    @property
    def ozlx(self) -> U32Vector4: ...
    @property
    def ozly(self) -> U32Vector4: ...
    @property
    def ozlz(self) -> U32Vector4: ...
    @property
    def ozlw(self) -> U32Vector4: ...
    @property
    def ozlo(self) -> U32Vector4: ...
    @property
    def ozll(self) -> U32Vector4: ...
    @property
    def owxx(self) -> U32Vector4: ...
    @property
    def owxy(self) -> U32Vector4: ...
    @property
    def owxz(self) -> U32Vector4: ...
    @property
    def owxw(self) -> U32Vector4: ...
    @property
    def owxo(self) -> U32Vector4: ...
    @property
    def owxl(self) -> U32Vector4: ...
    @property
    def owyx(self) -> U32Vector4: ...
    @property
    def owyy(self) -> U32Vector4: ...
    @property
    def owyz(self) -> U32Vector4: ...
    @property
    def owyw(self) -> U32Vector4: ...
    @property
    def owyo(self) -> U32Vector4: ...
    @property
    def owyl(self) -> U32Vector4: ...
    @property
    def owzx(self) -> U32Vector4: ...
    @property
    def owzy(self) -> U32Vector4: ...
    @property
    def owzz(self) -> U32Vector4: ...
    @property
    def owzw(self) -> U32Vector4: ...
    @property
    def owzo(self) -> U32Vector4: ...
    @property
    def owzl(self) -> U32Vector4: ...
    @property
    def owwx(self) -> U32Vector4: ...
    @property
    def owwy(self) -> U32Vector4: ...
    @property
    def owwz(self) -> U32Vector4: ...
    @property
    def owww(self) -> U32Vector4: ...
    @property
    def owwo(self) -> U32Vector4: ...
    @property
    def owwl(self) -> U32Vector4: ...
    @property
    def owox(self) -> U32Vector4: ...
    @property
    def owoy(self) -> U32Vector4: ...
    @property
    def owoz(self) -> U32Vector4: ...
    @property
    def owow(self) -> U32Vector4: ...
    @property
    def owoo(self) -> U32Vector4: ...
    @property
    def owol(self) -> U32Vector4: ...
    @property
    def owlx(self) -> U32Vector4: ...
    @property
    def owly(self) -> U32Vector4: ...
    @property
    def owlz(self) -> U32Vector4: ...
    @property
    def owlw(self) -> U32Vector4: ...
    @property
    def owlo(self) -> U32Vector4: ...
    @property
    def owll(self) -> U32Vector4: ...
    @property
    def ooxx(self) -> U32Vector4: ...
    @property
    def ooxy(self) -> U32Vector4: ...
    @property
    def ooxz(self) -> U32Vector4: ...
    @property
    def ooxw(self) -> U32Vector4: ...
    @property
    def ooxo(self) -> U32Vector4: ...
    @property
    def ooxl(self) -> U32Vector4: ...
    @property
    def ooyx(self) -> U32Vector4: ...
    @property
    def ooyy(self) -> U32Vector4: ...
    @property
    def ooyz(self) -> U32Vector4: ...
    @property
    def ooyw(self) -> U32Vector4: ...
    @property
    def ooyo(self) -> U32Vector4: ...
    @property
    def ooyl(self) -> U32Vector4: ...
    @property
    def oozx(self) -> U32Vector4: ...
    @property
    def oozy(self) -> U32Vector4: ...
    @property
    def oozz(self) -> U32Vector4: ...
    @property
    def oozw(self) -> U32Vector4: ...
    @property
    def oozo(self) -> U32Vector4: ...
    @property
    def oozl(self) -> U32Vector4: ...
    @property
    def oowx(self) -> U32Vector4: ...
    @property
    def oowy(self) -> U32Vector4: ...
    @property
    def oowz(self) -> U32Vector4: ...
    @property
    def ooww(self) -> U32Vector4: ...
    @property
    def oowo(self) -> U32Vector4: ...
    @property
    def oowl(self) -> U32Vector4: ...
    @property
    def ooox(self) -> U32Vector4: ...
    @property
    def oooy(self) -> U32Vector4: ...
    @property
    def oooz(self) -> U32Vector4: ...
    @property
    def ooow(self) -> U32Vector4: ...
    @property
    def oolx(self) -> U32Vector4: ...
    @property
    def ooly(self) -> U32Vector4: ...
    @property
    def oolz(self) -> U32Vector4: ...
    @property
    def oolw(self) -> U32Vector4: ...
    @property
    def olxx(self) -> U32Vector4: ...
    @property
    def olxy(self) -> U32Vector4: ...
    @property
    def olxz(self) -> U32Vector4: ...
    @property
    def olxw(self) -> U32Vector4: ...
    @property
    def olxo(self) -> U32Vector4: ...
    @property
    def olxl(self) -> U32Vector4: ...
    @property
    def olyx(self) -> U32Vector4: ...
    @property
    def olyy(self) -> U32Vector4: ...
    @property
    def olyz(self) -> U32Vector4: ...
    @property
    def olyw(self) -> U32Vector4: ...
    @property
    def olyo(self) -> U32Vector4: ...
    @property
    def olyl(self) -> U32Vector4: ...
    @property
    def olzx(self) -> U32Vector4: ...
    @property
    def olzy(self) -> U32Vector4: ...
    @property
    def olzz(self) -> U32Vector4: ...
    @property
    def olzw(self) -> U32Vector4: ...
    @property
    def olzo(self) -> U32Vector4: ...
    @property
    def olzl(self) -> U32Vector4: ...
    @property
    def olwx(self) -> U32Vector4: ...
    @property
    def olwy(self) -> U32Vector4: ...
    @property
    def olwz(self) -> U32Vector4: ...
    @property
    def olww(self) -> U32Vector4: ...
    @property
    def olwo(self) -> U32Vector4: ...
    @property
    def olwl(self) -> U32Vector4: ...
    @property
    def olox(self) -> U32Vector4: ...
    @property
    def oloy(self) -> U32Vector4: ...
    @property
    def oloz(self) -> U32Vector4: ...
    @property
    def olow(self) -> U32Vector4: ...
    @property
    def ollx(self) -> U32Vector4: ...
    @property
    def olly(self) -> U32Vector4: ...
    @property
    def ollz(self) -> U32Vector4: ...
    @property
    def ollw(self) -> U32Vector4: ...
    @property
    def lxxx(self) -> U32Vector4: ...
    @property
    def lxxy(self) -> U32Vector4: ...
    @property
    def lxxz(self) -> U32Vector4: ...
    @property
    def lxxw(self) -> U32Vector4: ...
    @property
    def lxxo(self) -> U32Vector4: ...
    @property
    def lxxl(self) -> U32Vector4: ...
    @property
    def lxyx(self) -> U32Vector4: ...
    @property
    def lxyy(self) -> U32Vector4: ...
    @property
    def lxyz(self) -> U32Vector4: ...
    @property
    def lxyw(self) -> U32Vector4: ...
    @property
    def lxyo(self) -> U32Vector4: ...
    @property
    def lxyl(self) -> U32Vector4: ...
    @property
    def lxzx(self) -> U32Vector4: ...
    @property
    def lxzy(self) -> U32Vector4: ...
    @property
    def lxzz(self) -> U32Vector4: ...
    @property
    def lxzw(self) -> U32Vector4: ...
    @property
    def lxzo(self) -> U32Vector4: ...
    @property
    def lxzl(self) -> U32Vector4: ...
    @property
    def lxwx(self) -> U32Vector4: ...
    @property
    def lxwy(self) -> U32Vector4: ...
    @property
    def lxwz(self) -> U32Vector4: ...
    @property
    def lxww(self) -> U32Vector4: ...
    @property
    def lxwo(self) -> U32Vector4: ...
    @property
    def lxwl(self) -> U32Vector4: ...
    @property
    def lxox(self) -> U32Vector4: ...
    @property
    def lxoy(self) -> U32Vector4: ...
    @property
    def lxoz(self) -> U32Vector4: ...
    @property
    def lxow(self) -> U32Vector4: ...
    @property
    def lxoo(self) -> U32Vector4: ...
    @property
    def lxol(self) -> U32Vector4: ...
    @property
    def lxlx(self) -> U32Vector4: ...
    @property
    def lxly(self) -> U32Vector4: ...
    @property
    def lxlz(self) -> U32Vector4: ...
    @property
    def lxlw(self) -> U32Vector4: ...
    @property
    def lxlo(self) -> U32Vector4: ...
    @property
    def lxll(self) -> U32Vector4: ...
    @property
    def lyxx(self) -> U32Vector4: ...
    @property
    def lyxy(self) -> U32Vector4: ...
    @property
    def lyxz(self) -> U32Vector4: ...
    @property
    def lyxw(self) -> U32Vector4: ...
    @property
    def lyxo(self) -> U32Vector4: ...
    @property
    def lyxl(self) -> U32Vector4: ...
    @property
    def lyyx(self) -> U32Vector4: ...
    @property
    def lyyy(self) -> U32Vector4: ...
    @property
    def lyyz(self) -> U32Vector4: ...
    @property
    def lyyw(self) -> U32Vector4: ...
    @property
    def lyyo(self) -> U32Vector4: ...
    @property
    def lyyl(self) -> U32Vector4: ...
    @property
    def lyzx(self) -> U32Vector4: ...
    @property
    def lyzy(self) -> U32Vector4: ...
    @property
    def lyzz(self) -> U32Vector4: ...
    @property
    def lyzw(self) -> U32Vector4: ...
    @property
    def lyzo(self) -> U32Vector4: ...
    @property
    def lyzl(self) -> U32Vector4: ...
    @property
    def lywx(self) -> U32Vector4: ...
    @property
    def lywy(self) -> U32Vector4: ...
    @property
    def lywz(self) -> U32Vector4: ...
    @property
    def lyww(self) -> U32Vector4: ...
    @property
    def lywo(self) -> U32Vector4: ...
    @property
    def lywl(self) -> U32Vector4: ...
    @property
    def lyox(self) -> U32Vector4: ...
    @property
    def lyoy(self) -> U32Vector4: ...
    @property
    def lyoz(self) -> U32Vector4: ...
    @property
    def lyow(self) -> U32Vector4: ...
    @property
    def lyoo(self) -> U32Vector4: ...
    @property
    def lyol(self) -> U32Vector4: ...
    @property
    def lylx(self) -> U32Vector4: ...
    @property
    def lyly(self) -> U32Vector4: ...
    @property
    def lylz(self) -> U32Vector4: ...
    @property
    def lylw(self) -> U32Vector4: ...
    @property
    def lylo(self) -> U32Vector4: ...
    @property
    def lyll(self) -> U32Vector4: ...
    @property
    def lzxx(self) -> U32Vector4: ...
    @property
    def lzxy(self) -> U32Vector4: ...
    @property
    def lzxz(self) -> U32Vector4: ...
    @property
    def lzxw(self) -> U32Vector4: ...
    @property
    def lzxo(self) -> U32Vector4: ...
    @property
    def lzxl(self) -> U32Vector4: ...
    @property
    def lzyx(self) -> U32Vector4: ...
    @property
    def lzyy(self) -> U32Vector4: ...
    @property
    def lzyz(self) -> U32Vector4: ...
    @property
    def lzyw(self) -> U32Vector4: ...
    @property
    def lzyo(self) -> U32Vector4: ...
    @property
    def lzyl(self) -> U32Vector4: ...
    @property
    def lzzx(self) -> U32Vector4: ...
    @property
    def lzzy(self) -> U32Vector4: ...
    @property
    def lzzz(self) -> U32Vector4: ...
    @property
    def lzzw(self) -> U32Vector4: ...
    @property
    def lzzo(self) -> U32Vector4: ...
    @property
    def lzzl(self) -> U32Vector4: ...
    @property
    def lzwx(self) -> U32Vector4: ...
    @property
    def lzwy(self) -> U32Vector4: ...
    @property
    def lzwz(self) -> U32Vector4: ...
    @property
    def lzww(self) -> U32Vector4: ...
    @property
    def lzwo(self) -> U32Vector4: ...
    @property
    def lzwl(self) -> U32Vector4: ...
    @property
    def lzox(self) -> U32Vector4: ...
    @property
    def lzoy(self) -> U32Vector4: ...
    @property
    def lzoz(self) -> U32Vector4: ...
    @property
    def lzow(self) -> U32Vector4: ...
    @property
    def lzoo(self) -> U32Vector4: ...
    @property
    def lzol(self) -> U32Vector4: ...
    @property
    def lzlx(self) -> U32Vector4: ...
    @property
    def lzly(self) -> U32Vector4: ...
    @property
    def lzlz(self) -> U32Vector4: ...
    @property
    def lzlw(self) -> U32Vector4: ...
    @property
    def lzlo(self) -> U32Vector4: ...
    @property
    def lzll(self) -> U32Vector4: ...
    @property
    def lwxx(self) -> U32Vector4: ...
    @property
    def lwxy(self) -> U32Vector4: ...
    @property
    def lwxz(self) -> U32Vector4: ...
    @property
    def lwxw(self) -> U32Vector4: ...
    @property
    def lwxo(self) -> U32Vector4: ...
    @property
    def lwxl(self) -> U32Vector4: ...
    @property
    def lwyx(self) -> U32Vector4: ...
    @property
    def lwyy(self) -> U32Vector4: ...
    @property
    def lwyz(self) -> U32Vector4: ...
    @property
    def lwyw(self) -> U32Vector4: ...
    @property
    def lwyo(self) -> U32Vector4: ...
    @property
    def lwyl(self) -> U32Vector4: ...
    @property
    def lwzx(self) -> U32Vector4: ...
    @property
    def lwzy(self) -> U32Vector4: ...
    @property
    def lwzz(self) -> U32Vector4: ...
    @property
    def lwzw(self) -> U32Vector4: ...
    @property
    def lwzo(self) -> U32Vector4: ...
    @property
    def lwzl(self) -> U32Vector4: ...
    @property
    def lwwx(self) -> U32Vector4: ...
    @property
    def lwwy(self) -> U32Vector4: ...
    @property
    def lwwz(self) -> U32Vector4: ...
    @property
    def lwww(self) -> U32Vector4: ...
    @property
    def lwwo(self) -> U32Vector4: ...
    @property
    def lwwl(self) -> U32Vector4: ...
    @property
    def lwox(self) -> U32Vector4: ...
    @property
    def lwoy(self) -> U32Vector4: ...
    @property
    def lwoz(self) -> U32Vector4: ...
    @property
    def lwow(self) -> U32Vector4: ...
    @property
    def lwoo(self) -> U32Vector4: ...
    @property
    def lwol(self) -> U32Vector4: ...
    @property
    def lwlx(self) -> U32Vector4: ...
    @property
    def lwly(self) -> U32Vector4: ...
    @property
    def lwlz(self) -> U32Vector4: ...
    @property
    def lwlw(self) -> U32Vector4: ...
    @property
    def lwlo(self) -> U32Vector4: ...
    @property
    def lwll(self) -> U32Vector4: ...
    @property
    def loxx(self) -> U32Vector4: ...
    @property
    def loxy(self) -> U32Vector4: ...
    @property
    def loxz(self) -> U32Vector4: ...
    @property
    def loxw(self) -> U32Vector4: ...
    @property
    def loxo(self) -> U32Vector4: ...
    @property
    def loxl(self) -> U32Vector4: ...
    @property
    def loyx(self) -> U32Vector4: ...
    @property
    def loyy(self) -> U32Vector4: ...
    @property
    def loyz(self) -> U32Vector4: ...
    @property
    def loyw(self) -> U32Vector4: ...
    @property
    def loyo(self) -> U32Vector4: ...
    @property
    def loyl(self) -> U32Vector4: ...
    @property
    def lozx(self) -> U32Vector4: ...
    @property
    def lozy(self) -> U32Vector4: ...
    @property
    def lozz(self) -> U32Vector4: ...
    @property
    def lozw(self) -> U32Vector4: ...
    @property
    def lozo(self) -> U32Vector4: ...
    @property
    def lozl(self) -> U32Vector4: ...
    @property
    def lowx(self) -> U32Vector4: ...
    @property
    def lowy(self) -> U32Vector4: ...
    @property
    def lowz(self) -> U32Vector4: ...
    @property
    def loww(self) -> U32Vector4: ...
    @property
    def lowo(self) -> U32Vector4: ...
    @property
    def lowl(self) -> U32Vector4: ...
    @property
    def loox(self) -> U32Vector4: ...
    @property
    def looy(self) -> U32Vector4: ...
    @property
    def looz(self) -> U32Vector4: ...
    @property
    def loow(self) -> U32Vector4: ...
    @property
    def lolx(self) -> U32Vector4: ...
    @property
    def loly(self) -> U32Vector4: ...
    @property
    def lolz(self) -> U32Vector4: ...
    @property
    def lolw(self) -> U32Vector4: ...
    @property
    def llxx(self) -> U32Vector4: ...
    @property
    def llxy(self) -> U32Vector4: ...
    @property
    def llxz(self) -> U32Vector4: ...
    @property
    def llxw(self) -> U32Vector4: ...
    @property
    def llxo(self) -> U32Vector4: ...
    @property
    def llxl(self) -> U32Vector4: ...
    @property
    def llyx(self) -> U32Vector4: ...
    @property
    def llyy(self) -> U32Vector4: ...
    @property
    def llyz(self) -> U32Vector4: ...
    @property
    def llyw(self) -> U32Vector4: ...
    @property
    def llyo(self) -> U32Vector4: ...
    @property
    def llyl(self) -> U32Vector4: ...
    @property
    def llzx(self) -> U32Vector4: ...
    @property
    def llzy(self) -> U32Vector4: ...
    @property
    def llzz(self) -> U32Vector4: ...
    @property
    def llzw(self) -> U32Vector4: ...
    @property
    def llzo(self) -> U32Vector4: ...
    @property
    def llzl(self) -> U32Vector4: ...
    @property
    def llwx(self) -> U32Vector4: ...
    @property
    def llwy(self) -> U32Vector4: ...
    @property
    def llwz(self) -> U32Vector4: ...
    @property
    def llww(self) -> U32Vector4: ...
    @property
    def llwo(self) -> U32Vector4: ...
    @property
    def llwl(self) -> U32Vector4: ...
    @property
    def llox(self) -> U32Vector4: ...
    @property
    def lloy(self) -> U32Vector4: ...
    @property
    def lloz(self) -> U32Vector4: ...
    @property
    def llow(self) -> U32Vector4: ...
    @property
    def lllx(self) -> U32Vector4: ...
    @property
    def llly(self) -> U32Vector4: ...
    @property
    def lllz(self) -> U32Vector4: ...
    @property
    def lllw(self) -> U32Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> U32Vector2: ...
    @property
    def rg(self) -> U32Vector2: ...
    @property
    def rb(self) -> U32Vector2: ...
    @property
    def ra(self) -> U32Vector2: ...
    @property
    def ro(self) -> U32Vector2: ...
    @property
    def rl(self) -> U32Vector2: ...
    @property
    def gr(self) -> U32Vector2: ...
    @property
    def gg(self) -> U32Vector2: ...
    @property
    def gb(self) -> U32Vector2: ...
    @property
    def ga(self) -> U32Vector2: ...
    @property
    def go(self) -> U32Vector2: ...
    @property
    def gl(self) -> U32Vector2: ...
    @property
    def br(self) -> U32Vector2: ...
    @property
    def bg(self) -> U32Vector2: ...
    @property
    def bb(self) -> U32Vector2: ...
    @property
    def ba(self) -> U32Vector2: ...
    @property
    def bo(self) -> U32Vector2: ...
    @property
    def bl(self) -> U32Vector2: ...
    @property
    def ar(self) -> U32Vector2: ...
    @property
    def ag(self) -> U32Vector2: ...
    @property
    def ab(self) -> U32Vector2: ...
    @property
    def aa(self) -> U32Vector2: ...
    @property
    def ao(self) -> U32Vector2: ...
    @property
    def al(self) -> U32Vector2: ...
    @property
    def og(self) -> U32Vector2: ...
    @property
    def ob(self) -> U32Vector2: ...
    @property
    def oa(self) -> U32Vector2: ...
    @property
    def lr(self) -> U32Vector2: ...
    @property
    def lg(self) -> U32Vector2: ...
    @property
    def lb(self) -> U32Vector2: ...
    @property
    def la(self) -> U32Vector2: ...
    @property
    def rrr(self) -> U32Vector3: ...
    @property
    def rrg(self) -> U32Vector3: ...
    @property
    def rrb(self) -> U32Vector3: ...
    @property
    def rra(self) -> U32Vector3: ...
    @property
    def rro(self) -> U32Vector3: ...
    @property
    def rrl(self) -> U32Vector3: ...
    @property
    def rgr(self) -> U32Vector3: ...
    @property
    def rgg(self) -> U32Vector3: ...
    @property
    def rgb(self) -> U32Vector3: ...
    @property
    def rga(self) -> U32Vector3: ...
    @property
    def rgo(self) -> U32Vector3: ...
    @property
    def rgl(self) -> U32Vector3: ...
    @property
    def rbr(self) -> U32Vector3: ...
    @property
    def rbg(self) -> U32Vector3: ...
    @property
    def rbb(self) -> U32Vector3: ...
    @property
    def rba(self) -> U32Vector3: ...
    @property
    def rbo(self) -> U32Vector3: ...
    @property
    def rbl(self) -> U32Vector3: ...
    @property
    def rar(self) -> U32Vector3: ...
    @property
    def rag(self) -> U32Vector3: ...
    @property
    def rab(self) -> U32Vector3: ...
    @property
    def raa(self) -> U32Vector3: ...
    @property
    def rao(self) -> U32Vector3: ...
    @property
    def ral(self) -> U32Vector3: ...
    @property
    def ror(self) -> U32Vector3: ...
    @property
    def rog(self) -> U32Vector3: ...
    @property
    def rob(self) -> U32Vector3: ...
    @property
    def roa(self) -> U32Vector3: ...
    @property
    def roo(self) -> U32Vector3: ...
    @property
    def rol(self) -> U32Vector3: ...
    @property
    def rlr(self) -> U32Vector3: ...
    @property
    def rlg(self) -> U32Vector3: ...
    @property
    def rlb(self) -> U32Vector3: ...
    @property
    def rla(self) -> U32Vector3: ...
    @property
    def rlo(self) -> U32Vector3: ...
    @property
    def rll(self) -> U32Vector3: ...
    @property
    def grr(self) -> U32Vector3: ...
    @property
    def grg(self) -> U32Vector3: ...
    @property
    def grb(self) -> U32Vector3: ...
    @property
    def gra(self) -> U32Vector3: ...
    @property
    def gro(self) -> U32Vector3: ...
    @property
    def grl(self) -> U32Vector3: ...
    @property
    def ggr(self) -> U32Vector3: ...
    @property
    def ggg(self) -> U32Vector3: ...
    @property
    def ggb(self) -> U32Vector3: ...
    @property
    def gga(self) -> U32Vector3: ...
    @property
    def ggo(self) -> U32Vector3: ...
    @property
    def ggl(self) -> U32Vector3: ...
    @property
    def gbr(self) -> U32Vector3: ...
    @property
    def gbg(self) -> U32Vector3: ...
    @property
    def gbb(self) -> U32Vector3: ...
    @property
    def gba(self) -> U32Vector3: ...
    @property
    def gbo(self) -> U32Vector3: ...
    @property
    def gbl(self) -> U32Vector3: ...
    @property
    def gar(self) -> U32Vector3: ...
    @property
    def gag(self) -> U32Vector3: ...
    @property
    def gab(self) -> U32Vector3: ...
    @property
    def gaa(self) -> U32Vector3: ...
    @property
    def gao(self) -> U32Vector3: ...
    @property
    def gal(self) -> U32Vector3: ...
    @property
    def gor(self) -> U32Vector3: ...
    @property
    def gog(self) -> U32Vector3: ...
    @property
    def gob(self) -> U32Vector3: ...
    @property
    def goa(self) -> U32Vector3: ...
    @property
    def goo(self) -> U32Vector3: ...
    @property
    def gol(self) -> U32Vector3: ...
    @property
    def glr(self) -> U32Vector3: ...
    @property
    def glg(self) -> U32Vector3: ...
    @property
    def glb(self) -> U32Vector3: ...
    @property
    def gla(self) -> U32Vector3: ...
    @property
    def glo(self) -> U32Vector3: ...
    @property
    def gll(self) -> U32Vector3: ...
    @property
    def brr(self) -> U32Vector3: ...
    @property
    def brg(self) -> U32Vector3: ...
    @property
    def brb(self) -> U32Vector3: ...
    @property
    def bra(self) -> U32Vector3: ...
    @property
    def bro(self) -> U32Vector3: ...
    @property
    def brl(self) -> U32Vector3: ...
    @property
    def bgr(self) -> U32Vector3: ...
    @property
    def bgg(self) -> U32Vector3: ...
    @property
    def bgb(self) -> U32Vector3: ...
    @property
    def bga(self) -> U32Vector3: ...
    @property
    def bgo(self) -> U32Vector3: ...
    @property
    def bgl(self) -> U32Vector3: ...
    @property
    def bbr(self) -> U32Vector3: ...
    @property
    def bbg(self) -> U32Vector3: ...
    @property
    def bbb(self) -> U32Vector3: ...
    @property
    def bba(self) -> U32Vector3: ...
    @property
    def bbo(self) -> U32Vector3: ...
    @property
    def bbl(self) -> U32Vector3: ...
    @property
    def bar(self) -> U32Vector3: ...
    @property
    def bag(self) -> U32Vector3: ...
    @property
    def bab(self) -> U32Vector3: ...
    @property
    def baa(self) -> U32Vector3: ...
    @property
    def bao(self) -> U32Vector3: ...
    @property
    def bal(self) -> U32Vector3: ...
    @property
    def bor(self) -> U32Vector3: ...
    @property
    def bog(self) -> U32Vector3: ...
    @property
    def bob(self) -> U32Vector3: ...
    @property
    def boa(self) -> U32Vector3: ...
    @property
    def boo(self) -> U32Vector3: ...
    @property
    def bol(self) -> U32Vector3: ...
    @property
    def blr(self) -> U32Vector3: ...
    @property
    def blg(self) -> U32Vector3: ...
    @property
    def blb(self) -> U32Vector3: ...
    @property
    def bla(self) -> U32Vector3: ...
    @property
    def blo(self) -> U32Vector3: ...
    @property
    def bll(self) -> U32Vector3: ...
    @property
    def arr(self) -> U32Vector3: ...
    @property
    def arg(self) -> U32Vector3: ...
    @property
    def arb(self) -> U32Vector3: ...
    @property
    def ara(self) -> U32Vector3: ...
    @property
    def aro(self) -> U32Vector3: ...
    @property
    def arl(self) -> U32Vector3: ...
    @property
    def agr(self) -> U32Vector3: ...
    @property
    def agg(self) -> U32Vector3: ...
    @property
    def agb(self) -> U32Vector3: ...
    @property
    def aga(self) -> U32Vector3: ...
    @property
    def ago(self) -> U32Vector3: ...
    @property
    def agl(self) -> U32Vector3: ...
    @property
    def abr(self) -> U32Vector3: ...
    @property
    def abg(self) -> U32Vector3: ...
    @property
    def abb(self) -> U32Vector3: ...
    @property
    def aba(self) -> U32Vector3: ...
    @property
    def abo(self) -> U32Vector3: ...
    @property
    def abl(self) -> U32Vector3: ...
    @property
    def aar(self) -> U32Vector3: ...
    @property
    def aag(self) -> U32Vector3: ...
    @property
    def aab(self) -> U32Vector3: ...
    @property
    def aaa(self) -> U32Vector3: ...
    @property
    def aao(self) -> U32Vector3: ...
    @property
    def aal(self) -> U32Vector3: ...
    @property
    def aor(self) -> U32Vector3: ...
    @property
    def aog(self) -> U32Vector3: ...
    @property
    def aob(self) -> U32Vector3: ...
    @property
    def aoa(self) -> U32Vector3: ...
    @property
    def aoo(self) -> U32Vector3: ...
    @property
    def aol(self) -> U32Vector3: ...
    @property
    def alr(self) -> U32Vector3: ...
    @property
    def alg(self) -> U32Vector3: ...
    @property
    def alb(self) -> U32Vector3: ...
    @property
    def ala(self) -> U32Vector3: ...
    @property
    def alo(self) -> U32Vector3: ...
    @property
    def all(self) -> U32Vector3: ...
    @property
    def orr(self) -> U32Vector3: ...
    @property
    def org(self) -> U32Vector3: ...
    @property
    def orb(self) -> U32Vector3: ...
    @property
    def ora(self) -> U32Vector3: ...
    @property
    def oro(self) -> U32Vector3: ...
    @property
    def orl(self) -> U32Vector3: ...
    @property
    def ogr(self) -> U32Vector3: ...
    @property
    def ogg(self) -> U32Vector3: ...
    @property
    def ogb(self) -> U32Vector3: ...
    @property
    def oga(self) -> U32Vector3: ...
    @property
    def ogo(self) -> U32Vector3: ...
    @property
    def ogl(self) -> U32Vector3: ...
    @property
    def obr(self) -> U32Vector3: ...
    @property
    def obg(self) -> U32Vector3: ...
    @property
    def obb(self) -> U32Vector3: ...
    @property
    def oba(self) -> U32Vector3: ...
    @property
    def obo(self) -> U32Vector3: ...
    @property
    def obl(self) -> U32Vector3: ...
    @property
    def oar(self) -> U32Vector3: ...
    @property
    def oag(self) -> U32Vector3: ...
    @property
    def oab(self) -> U32Vector3: ...
    @property
    def oaa(self) -> U32Vector3: ...
    @property
    def oao(self) -> U32Vector3: ...
    @property
    def oal(self) -> U32Vector3: ...
    @property
    def oor(self) -> U32Vector3: ...
    @property
    def oog(self) -> U32Vector3: ...
    @property
    def oob(self) -> U32Vector3: ...
    @property
    def ooa(self) -> U32Vector3: ...
    @property
    def olr(self) -> U32Vector3: ...
    @property
    def olg(self) -> U32Vector3: ...
    @property
    def olb(self) -> U32Vector3: ...
    @property
    def ola(self) -> U32Vector3: ...
    @property
    def lrr(self) -> U32Vector3: ...
    @property
    def lrg(self) -> U32Vector3: ...
    @property
    def lrb(self) -> U32Vector3: ...
    @property
    def lra(self) -> U32Vector3: ...
    @property
    def lro(self) -> U32Vector3: ...
    @property
    def lrl(self) -> U32Vector3: ...
    @property
    def lgr(self) -> U32Vector3: ...
    @property
    def lgg(self) -> U32Vector3: ...
    @property
    def lgb(self) -> U32Vector3: ...
    @property
    def lga(self) -> U32Vector3: ...
    @property
    def lgo(self) -> U32Vector3: ...
    @property
    def lgl(self) -> U32Vector3: ...
    @property
    def lbr(self) -> U32Vector3: ...
    @property
    def lbg(self) -> U32Vector3: ...
    @property
    def lbb(self) -> U32Vector3: ...
    @property
    def lba(self) -> U32Vector3: ...
    @property
    def lbo(self) -> U32Vector3: ...
    @property
    def lbl(self) -> U32Vector3: ...
    @property
    def lar(self) -> U32Vector3: ...
    @property
    def lag(self) -> U32Vector3: ...
    @property
    def lab(self) -> U32Vector3: ...
    @property
    def laa(self) -> U32Vector3: ...
    @property
    def lao(self) -> U32Vector3: ...
    @property
    def lal(self) -> U32Vector3: ...
    @property
    def lor(self) -> U32Vector3: ...
    @property
    def log(self) -> U32Vector3: ...
    @property
    def lob(self) -> U32Vector3: ...
    @property
    def loa(self) -> U32Vector3: ...
    @property
    def llr(self) -> U32Vector3: ...
    @property
    def llg(self) -> U32Vector3: ...
    @property
    def llb(self) -> U32Vector3: ...
    @property
    def lla(self) -> U32Vector3: ...
    @property
    def rrrr(self) -> U32Vector4: ...
    @property
    def rrrg(self) -> U32Vector4: ...
    @property
    def rrrb(self) -> U32Vector4: ...
    @property
    def rrra(self) -> U32Vector4: ...
    @property
    def rrro(self) -> U32Vector4: ...
    @property
    def rrrl(self) -> U32Vector4: ...
    @property
    def rrgr(self) -> U32Vector4: ...
    @property
    def rrgg(self) -> U32Vector4: ...
    @property
    def rrgb(self) -> U32Vector4: ...
    @property
    def rrga(self) -> U32Vector4: ...
    @property
    def rrgo(self) -> U32Vector4: ...
    @property
    def rrgl(self) -> U32Vector4: ...
    @property
    def rrbr(self) -> U32Vector4: ...
    @property
    def rrbg(self) -> U32Vector4: ...
    @property
    def rrbb(self) -> U32Vector4: ...
    @property
    def rrba(self) -> U32Vector4: ...
    @property
    def rrbo(self) -> U32Vector4: ...
    @property
    def rrbl(self) -> U32Vector4: ...
    @property
    def rrar(self) -> U32Vector4: ...
    @property
    def rrag(self) -> U32Vector4: ...
    @property
    def rrab(self) -> U32Vector4: ...
    @property
    def rraa(self) -> U32Vector4: ...
    @property
    def rrao(self) -> U32Vector4: ...
    @property
    def rral(self) -> U32Vector4: ...
    @property
    def rror(self) -> U32Vector4: ...
    @property
    def rrog(self) -> U32Vector4: ...
    @property
    def rrob(self) -> U32Vector4: ...
    @property
    def rroa(self) -> U32Vector4: ...
    @property
    def rroo(self) -> U32Vector4: ...
    @property
    def rrol(self) -> U32Vector4: ...
    @property
    def rrlr(self) -> U32Vector4: ...
    @property
    def rrlg(self) -> U32Vector4: ...
    @property
    def rrlb(self) -> U32Vector4: ...
    @property
    def rrla(self) -> U32Vector4: ...
    @property
    def rrlo(self) -> U32Vector4: ...
    @property
    def rrll(self) -> U32Vector4: ...
    @property
    def rgrr(self) -> U32Vector4: ...
    @property
    def rgrg(self) -> U32Vector4: ...
    @property
    def rgrb(self) -> U32Vector4: ...
    @property
    def rgra(self) -> U32Vector4: ...
    @property
    def rgro(self) -> U32Vector4: ...
    @property
    def rgrl(self) -> U32Vector4: ...
    @property
    def rggr(self) -> U32Vector4: ...
    @property
    def rggg(self) -> U32Vector4: ...
    @property
    def rggb(self) -> U32Vector4: ...
    @property
    def rgga(self) -> U32Vector4: ...
    @property
    def rggo(self) -> U32Vector4: ...
    @property
    def rggl(self) -> U32Vector4: ...
    @property
    def rgbr(self) -> U32Vector4: ...
    @property
    def rgbg(self) -> U32Vector4: ...
    @property
    def rgbb(self) -> U32Vector4: ...
    @property
    def rgba(self) -> U32Vector4: ...
    @property
    def rgbo(self) -> U32Vector4: ...
    @property
    def rgbl(self) -> U32Vector4: ...
    @property
    def rgar(self) -> U32Vector4: ...
    @property
    def rgag(self) -> U32Vector4: ...
    @property
    def rgab(self) -> U32Vector4: ...
    @property
    def rgaa(self) -> U32Vector4: ...
    @property
    def rgao(self) -> U32Vector4: ...
    @property
    def rgal(self) -> U32Vector4: ...
    @property
    def rgor(self) -> U32Vector4: ...
    @property
    def rgog(self) -> U32Vector4: ...
    @property
    def rgob(self) -> U32Vector4: ...
    @property
    def rgoa(self) -> U32Vector4: ...
    @property
    def rgoo(self) -> U32Vector4: ...
    @property
    def rgol(self) -> U32Vector4: ...
    @property
    def rglr(self) -> U32Vector4: ...
    @property
    def rglg(self) -> U32Vector4: ...
    @property
    def rglb(self) -> U32Vector4: ...
    @property
    def rgla(self) -> U32Vector4: ...
    @property
    def rglo(self) -> U32Vector4: ...
    @property
    def rgll(self) -> U32Vector4: ...
    @property
    def rbrr(self) -> U32Vector4: ...
    @property
    def rbrg(self) -> U32Vector4: ...
    @property
    def rbrb(self) -> U32Vector4: ...
    @property
    def rbra(self) -> U32Vector4: ...
    @property
    def rbro(self) -> U32Vector4: ...
    @property
    def rbrl(self) -> U32Vector4: ...
    @property
    def rbgr(self) -> U32Vector4: ...
    @property
    def rbgg(self) -> U32Vector4: ...
    @property
    def rbgb(self) -> U32Vector4: ...
    @property
    def rbga(self) -> U32Vector4: ...
    @property
    def rbgo(self) -> U32Vector4: ...
    @property
    def rbgl(self) -> U32Vector4: ...
    @property
    def rbbr(self) -> U32Vector4: ...
    @property
    def rbbg(self) -> U32Vector4: ...
    @property
    def rbbb(self) -> U32Vector4: ...
    @property
    def rbba(self) -> U32Vector4: ...
    @property
    def rbbo(self) -> U32Vector4: ...
    @property
    def rbbl(self) -> U32Vector4: ...
    @property
    def rbar(self) -> U32Vector4: ...
    @property
    def rbag(self) -> U32Vector4: ...
    @property
    def rbab(self) -> U32Vector4: ...
    @property
    def rbaa(self) -> U32Vector4: ...
    @property
    def rbao(self) -> U32Vector4: ...
    @property
    def rbal(self) -> U32Vector4: ...
    @property
    def rbor(self) -> U32Vector4: ...
    @property
    def rbog(self) -> U32Vector4: ...
    @property
    def rbob(self) -> U32Vector4: ...
    @property
    def rboa(self) -> U32Vector4: ...
    @property
    def rboo(self) -> U32Vector4: ...
    @property
    def rbol(self) -> U32Vector4: ...
    @property
    def rblr(self) -> U32Vector4: ...
    @property
    def rblg(self) -> U32Vector4: ...
    @property
    def rblb(self) -> U32Vector4: ...
    @property
    def rbla(self) -> U32Vector4: ...
    @property
    def rblo(self) -> U32Vector4: ...
    @property
    def rbll(self) -> U32Vector4: ...
    @property
    def rarr(self) -> U32Vector4: ...
    @property
    def rarg(self) -> U32Vector4: ...
    @property
    def rarb(self) -> U32Vector4: ...
    @property
    def rara(self) -> U32Vector4: ...
    @property
    def raro(self) -> U32Vector4: ...
    @property
    def rarl(self) -> U32Vector4: ...
    @property
    def ragr(self) -> U32Vector4: ...
    @property
    def ragg(self) -> U32Vector4: ...
    @property
    def ragb(self) -> U32Vector4: ...
    @property
    def raga(self) -> U32Vector4: ...
    @property
    def rago(self) -> U32Vector4: ...
    @property
    def ragl(self) -> U32Vector4: ...
    @property
    def rabr(self) -> U32Vector4: ...
    @property
    def rabg(self) -> U32Vector4: ...
    @property
    def rabb(self) -> U32Vector4: ...
    @property
    def raba(self) -> U32Vector4: ...
    @property
    def rabo(self) -> U32Vector4: ...
    @property
    def rabl(self) -> U32Vector4: ...
    @property
    def raar(self) -> U32Vector4: ...
    @property
    def raag(self) -> U32Vector4: ...
    @property
    def raab(self) -> U32Vector4: ...
    @property
    def raaa(self) -> U32Vector4: ...
    @property
    def raao(self) -> U32Vector4: ...
    @property
    def raal(self) -> U32Vector4: ...
    @property
    def raor(self) -> U32Vector4: ...
    @property
    def raog(self) -> U32Vector4: ...
    @property
    def raob(self) -> U32Vector4: ...
    @property
    def raoa(self) -> U32Vector4: ...
    @property
    def raoo(self) -> U32Vector4: ...
    @property
    def raol(self) -> U32Vector4: ...
    @property
    def ralr(self) -> U32Vector4: ...
    @property
    def ralg(self) -> U32Vector4: ...
    @property
    def ralb(self) -> U32Vector4: ...
    @property
    def rala(self) -> U32Vector4: ...
    @property
    def ralo(self) -> U32Vector4: ...
    @property
    def rall(self) -> U32Vector4: ...
    @property
    def rorr(self) -> U32Vector4: ...
    @property
    def rorg(self) -> U32Vector4: ...
    @property
    def rorb(self) -> U32Vector4: ...
    @property
    def rora(self) -> U32Vector4: ...
    @property
    def roro(self) -> U32Vector4: ...
    @property
    def rorl(self) -> U32Vector4: ...
    @property
    def rogr(self) -> U32Vector4: ...
    @property
    def rogg(self) -> U32Vector4: ...
    @property
    def rogb(self) -> U32Vector4: ...
    @property
    def roga(self) -> U32Vector4: ...
    @property
    def rogo(self) -> U32Vector4: ...
    @property
    def rogl(self) -> U32Vector4: ...
    @property
    def robr(self) -> U32Vector4: ...
    @property
    def robg(self) -> U32Vector4: ...
    @property
    def robb(self) -> U32Vector4: ...
    @property
    def roba(self) -> U32Vector4: ...
    @property
    def robo(self) -> U32Vector4: ...
    @property
    def robl(self) -> U32Vector4: ...
    @property
    def roar(self) -> U32Vector4: ...
    @property
    def roag(self) -> U32Vector4: ...
    @property
    def roab(self) -> U32Vector4: ...
    @property
    def roaa(self) -> U32Vector4: ...
    @property
    def roao(self) -> U32Vector4: ...
    @property
    def roal(self) -> U32Vector4: ...
    @property
    def roor(self) -> U32Vector4: ...
    @property
    def roog(self) -> U32Vector4: ...
    @property
    def roob(self) -> U32Vector4: ...
    @property
    def rooa(self) -> U32Vector4: ...
    @property
    def rooo(self) -> U32Vector4: ...
    @property
    def rool(self) -> U32Vector4: ...
    @property
    def rolr(self) -> U32Vector4: ...
    @property
    def rolg(self) -> U32Vector4: ...
    @property
    def rolb(self) -> U32Vector4: ...
    @property
    def rola(self) -> U32Vector4: ...
    @property
    def rolo(self) -> U32Vector4: ...
    @property
    def roll(self) -> U32Vector4: ...
    @property
    def rlrr(self) -> U32Vector4: ...
    @property
    def rlrg(self) -> U32Vector4: ...
    @property
    def rlrb(self) -> U32Vector4: ...
    @property
    def rlra(self) -> U32Vector4: ...
    @property
    def rlro(self) -> U32Vector4: ...
    @property
    def rlrl(self) -> U32Vector4: ...
    @property
    def rlgr(self) -> U32Vector4: ...
    @property
    def rlgg(self) -> U32Vector4: ...
    @property
    def rlgb(self) -> U32Vector4: ...
    @property
    def rlga(self) -> U32Vector4: ...
    @property
    def rlgo(self) -> U32Vector4: ...
    @property
    def rlgl(self) -> U32Vector4: ...
    @property
    def rlbr(self) -> U32Vector4: ...
    @property
    def rlbg(self) -> U32Vector4: ...
    @property
    def rlbb(self) -> U32Vector4: ...
    @property
    def rlba(self) -> U32Vector4: ...
    @property
    def rlbo(self) -> U32Vector4: ...
    @property
    def rlbl(self) -> U32Vector4: ...
    @property
    def rlar(self) -> U32Vector4: ...
    @property
    def rlag(self) -> U32Vector4: ...
    @property
    def rlab(self) -> U32Vector4: ...
    @property
    def rlaa(self) -> U32Vector4: ...
    @property
    def rlao(self) -> U32Vector4: ...
    @property
    def rlal(self) -> U32Vector4: ...
    @property
    def rlor(self) -> U32Vector4: ...
    @property
    def rlog(self) -> U32Vector4: ...
    @property
    def rlob(self) -> U32Vector4: ...
    @property
    def rloa(self) -> U32Vector4: ...
    @property
    def rloo(self) -> U32Vector4: ...
    @property
    def rlol(self) -> U32Vector4: ...
    @property
    def rllr(self) -> U32Vector4: ...
    @property
    def rllg(self) -> U32Vector4: ...
    @property
    def rllb(self) -> U32Vector4: ...
    @property
    def rlla(self) -> U32Vector4: ...
    @property
    def rllo(self) -> U32Vector4: ...
    @property
    def rlll(self) -> U32Vector4: ...
    @property
    def grrr(self) -> U32Vector4: ...
    @property
    def grrg(self) -> U32Vector4: ...
    @property
    def grrb(self) -> U32Vector4: ...
    @property
    def grra(self) -> U32Vector4: ...
    @property
    def grro(self) -> U32Vector4: ...
    @property
    def grrl(self) -> U32Vector4: ...
    @property
    def grgr(self) -> U32Vector4: ...
    @property
    def grgg(self) -> U32Vector4: ...
    @property
    def grgb(self) -> U32Vector4: ...
    @property
    def grga(self) -> U32Vector4: ...
    @property
    def grgo(self) -> U32Vector4: ...
    @property
    def grgl(self) -> U32Vector4: ...
    @property
    def grbr(self) -> U32Vector4: ...
    @property
    def grbg(self) -> U32Vector4: ...
    @property
    def grbb(self) -> U32Vector4: ...
    @property
    def grba(self) -> U32Vector4: ...
    @property
    def grbo(self) -> U32Vector4: ...
    @property
    def grbl(self) -> U32Vector4: ...
    @property
    def grar(self) -> U32Vector4: ...
    @property
    def grag(self) -> U32Vector4: ...
    @property
    def grab(self) -> U32Vector4: ...
    @property
    def graa(self) -> U32Vector4: ...
    @property
    def grao(self) -> U32Vector4: ...
    @property
    def gral(self) -> U32Vector4: ...
    @property
    def gror(self) -> U32Vector4: ...
    @property
    def grog(self) -> U32Vector4: ...
    @property
    def grob(self) -> U32Vector4: ...
    @property
    def groa(self) -> U32Vector4: ...
    @property
    def groo(self) -> U32Vector4: ...
    @property
    def grol(self) -> U32Vector4: ...
    @property
    def grlr(self) -> U32Vector4: ...
    @property
    def grlg(self) -> U32Vector4: ...
    @property
    def grlb(self) -> U32Vector4: ...
    @property
    def grla(self) -> U32Vector4: ...
    @property
    def grlo(self) -> U32Vector4: ...
    @property
    def grll(self) -> U32Vector4: ...
    @property
    def ggrr(self) -> U32Vector4: ...
    @property
    def ggrg(self) -> U32Vector4: ...
    @property
    def ggrb(self) -> U32Vector4: ...
    @property
    def ggra(self) -> U32Vector4: ...
    @property
    def ggro(self) -> U32Vector4: ...
    @property
    def ggrl(self) -> U32Vector4: ...
    @property
    def gggr(self) -> U32Vector4: ...
    @property
    def gggg(self) -> U32Vector4: ...
    @property
    def gggb(self) -> U32Vector4: ...
    @property
    def ggga(self) -> U32Vector4: ...
    @property
    def gggo(self) -> U32Vector4: ...
    @property
    def gggl(self) -> U32Vector4: ...
    @property
    def ggbr(self) -> U32Vector4: ...
    @property
    def ggbg(self) -> U32Vector4: ...
    @property
    def ggbb(self) -> U32Vector4: ...
    @property
    def ggba(self) -> U32Vector4: ...
    @property
    def ggbo(self) -> U32Vector4: ...
    @property
    def ggbl(self) -> U32Vector4: ...
    @property
    def ggar(self) -> U32Vector4: ...
    @property
    def ggag(self) -> U32Vector4: ...
    @property
    def ggab(self) -> U32Vector4: ...
    @property
    def ggaa(self) -> U32Vector4: ...
    @property
    def ggao(self) -> U32Vector4: ...
    @property
    def ggal(self) -> U32Vector4: ...
    @property
    def ggor(self) -> U32Vector4: ...
    @property
    def ggog(self) -> U32Vector4: ...
    @property
    def ggob(self) -> U32Vector4: ...
    @property
    def ggoa(self) -> U32Vector4: ...
    @property
    def ggoo(self) -> U32Vector4: ...
    @property
    def ggol(self) -> U32Vector4: ...
    @property
    def gglr(self) -> U32Vector4: ...
    @property
    def gglg(self) -> U32Vector4: ...
    @property
    def gglb(self) -> U32Vector4: ...
    @property
    def ggla(self) -> U32Vector4: ...
    @property
    def gglo(self) -> U32Vector4: ...
    @property
    def ggll(self) -> U32Vector4: ...
    @property
    def gbrr(self) -> U32Vector4: ...
    @property
    def gbrg(self) -> U32Vector4: ...
    @property
    def gbrb(self) -> U32Vector4: ...
    @property
    def gbra(self) -> U32Vector4: ...
    @property
    def gbro(self) -> U32Vector4: ...
    @property
    def gbrl(self) -> U32Vector4: ...
    @property
    def gbgr(self) -> U32Vector4: ...
    @property
    def gbgg(self) -> U32Vector4: ...
    @property
    def gbgb(self) -> U32Vector4: ...
    @property
    def gbga(self) -> U32Vector4: ...
    @property
    def gbgo(self) -> U32Vector4: ...
    @property
    def gbgl(self) -> U32Vector4: ...
    @property
    def gbbr(self) -> U32Vector4: ...
    @property
    def gbbg(self) -> U32Vector4: ...
    @property
    def gbbb(self) -> U32Vector4: ...
    @property
    def gbba(self) -> U32Vector4: ...
    @property
    def gbbo(self) -> U32Vector4: ...
    @property
    def gbbl(self) -> U32Vector4: ...
    @property
    def gbar(self) -> U32Vector4: ...
    @property
    def gbag(self) -> U32Vector4: ...
    @property
    def gbab(self) -> U32Vector4: ...
    @property
    def gbaa(self) -> U32Vector4: ...
    @property
    def gbao(self) -> U32Vector4: ...
    @property
    def gbal(self) -> U32Vector4: ...
    @property
    def gbor(self) -> U32Vector4: ...
    @property
    def gbog(self) -> U32Vector4: ...
    @property
    def gbob(self) -> U32Vector4: ...
    @property
    def gboa(self) -> U32Vector4: ...
    @property
    def gboo(self) -> U32Vector4: ...
    @property
    def gbol(self) -> U32Vector4: ...
    @property
    def gblr(self) -> U32Vector4: ...
    @property
    def gblg(self) -> U32Vector4: ...
    @property
    def gblb(self) -> U32Vector4: ...
    @property
    def gbla(self) -> U32Vector4: ...
    @property
    def gblo(self) -> U32Vector4: ...
    @property
    def gbll(self) -> U32Vector4: ...
    @property
    def garr(self) -> U32Vector4: ...
    @property
    def garg(self) -> U32Vector4: ...
    @property
    def garb(self) -> U32Vector4: ...
    @property
    def gara(self) -> U32Vector4: ...
    @property
    def garo(self) -> U32Vector4: ...
    @property
    def garl(self) -> U32Vector4: ...
    @property
    def gagr(self) -> U32Vector4: ...
    @property
    def gagg(self) -> U32Vector4: ...
    @property
    def gagb(self) -> U32Vector4: ...
    @property
    def gaga(self) -> U32Vector4: ...
    @property
    def gago(self) -> U32Vector4: ...
    @property
    def gagl(self) -> U32Vector4: ...
    @property
    def gabr(self) -> U32Vector4: ...
    @property
    def gabg(self) -> U32Vector4: ...
    @property
    def gabb(self) -> U32Vector4: ...
    @property
    def gaba(self) -> U32Vector4: ...
    @property
    def gabo(self) -> U32Vector4: ...
    @property
    def gabl(self) -> U32Vector4: ...
    @property
    def gaar(self) -> U32Vector4: ...
    @property
    def gaag(self) -> U32Vector4: ...
    @property
    def gaab(self) -> U32Vector4: ...
    @property
    def gaaa(self) -> U32Vector4: ...
    @property
    def gaao(self) -> U32Vector4: ...
    @property
    def gaal(self) -> U32Vector4: ...
    @property
    def gaor(self) -> U32Vector4: ...
    @property
    def gaog(self) -> U32Vector4: ...
    @property
    def gaob(self) -> U32Vector4: ...
    @property
    def gaoa(self) -> U32Vector4: ...
    @property
    def gaoo(self) -> U32Vector4: ...
    @property
    def gaol(self) -> U32Vector4: ...
    @property
    def galr(self) -> U32Vector4: ...
    @property
    def galg(self) -> U32Vector4: ...
    @property
    def galb(self) -> U32Vector4: ...
    @property
    def gala(self) -> U32Vector4: ...
    @property
    def galo(self) -> U32Vector4: ...
    @property
    def gall(self) -> U32Vector4: ...
    @property
    def gorr(self) -> U32Vector4: ...
    @property
    def gorg(self) -> U32Vector4: ...
    @property
    def gorb(self) -> U32Vector4: ...
    @property
    def gora(self) -> U32Vector4: ...
    @property
    def goro(self) -> U32Vector4: ...
    @property
    def gorl(self) -> U32Vector4: ...
    @property
    def gogr(self) -> U32Vector4: ...
    @property
    def gogg(self) -> U32Vector4: ...
    @property
    def gogb(self) -> U32Vector4: ...
    @property
    def goga(self) -> U32Vector4: ...
    @property
    def gogo(self) -> U32Vector4: ...
    @property
    def gogl(self) -> U32Vector4: ...
    @property
    def gobr(self) -> U32Vector4: ...
    @property
    def gobg(self) -> U32Vector4: ...
    @property
    def gobb(self) -> U32Vector4: ...
    @property
    def goba(self) -> U32Vector4: ...
    @property
    def gobo(self) -> U32Vector4: ...
    @property
    def gobl(self) -> U32Vector4: ...
    @property
    def goar(self) -> U32Vector4: ...
    @property
    def goag(self) -> U32Vector4: ...
    @property
    def goab(self) -> U32Vector4: ...
    @property
    def goaa(self) -> U32Vector4: ...
    @property
    def goao(self) -> U32Vector4: ...
    @property
    def goal(self) -> U32Vector4: ...
    @property
    def goor(self) -> U32Vector4: ...
    @property
    def goog(self) -> U32Vector4: ...
    @property
    def goob(self) -> U32Vector4: ...
    @property
    def gooa(self) -> U32Vector4: ...
    @property
    def gooo(self) -> U32Vector4: ...
    @property
    def gool(self) -> U32Vector4: ...
    @property
    def golr(self) -> U32Vector4: ...
    @property
    def golg(self) -> U32Vector4: ...
    @property
    def golb(self) -> U32Vector4: ...
    @property
    def gola(self) -> U32Vector4: ...
    @property
    def golo(self) -> U32Vector4: ...
    @property
    def goll(self) -> U32Vector4: ...
    @property
    def glrr(self) -> U32Vector4: ...
    @property
    def glrg(self) -> U32Vector4: ...
    @property
    def glrb(self) -> U32Vector4: ...
    @property
    def glra(self) -> U32Vector4: ...
    @property
    def glro(self) -> U32Vector4: ...
    @property
    def glrl(self) -> U32Vector4: ...
    @property
    def glgr(self) -> U32Vector4: ...
    @property
    def glgg(self) -> U32Vector4: ...
    @property
    def glgb(self) -> U32Vector4: ...
    @property
    def glga(self) -> U32Vector4: ...
    @property
    def glgo(self) -> U32Vector4: ...
    @property
    def glgl(self) -> U32Vector4: ...
    @property
    def glbr(self) -> U32Vector4: ...
    @property
    def glbg(self) -> U32Vector4: ...
    @property
    def glbb(self) -> U32Vector4: ...
    @property
    def glba(self) -> U32Vector4: ...
    @property
    def glbo(self) -> U32Vector4: ...
    @property
    def glbl(self) -> U32Vector4: ...
    @property
    def glar(self) -> U32Vector4: ...
    @property
    def glag(self) -> U32Vector4: ...
    @property
    def glab(self) -> U32Vector4: ...
    @property
    def glaa(self) -> U32Vector4: ...
    @property
    def glao(self) -> U32Vector4: ...
    @property
    def glal(self) -> U32Vector4: ...
    @property
    def glor(self) -> U32Vector4: ...
    @property
    def glog(self) -> U32Vector4: ...
    @property
    def glob(self) -> U32Vector4: ...
    @property
    def gloa(self) -> U32Vector4: ...
    @property
    def gloo(self) -> U32Vector4: ...
    @property
    def glol(self) -> U32Vector4: ...
    @property
    def gllr(self) -> U32Vector4: ...
    @property
    def gllg(self) -> U32Vector4: ...
    @property
    def gllb(self) -> U32Vector4: ...
    @property
    def glla(self) -> U32Vector4: ...
    @property
    def gllo(self) -> U32Vector4: ...
    @property
    def glll(self) -> U32Vector4: ...
    @property
    def brrr(self) -> U32Vector4: ...
    @property
    def brrg(self) -> U32Vector4: ...
    @property
    def brrb(self) -> U32Vector4: ...
    @property
    def brra(self) -> U32Vector4: ...
    @property
    def brro(self) -> U32Vector4: ...
    @property
    def brrl(self) -> U32Vector4: ...
    @property
    def brgr(self) -> U32Vector4: ...
    @property
    def brgg(self) -> U32Vector4: ...
    @property
    def brgb(self) -> U32Vector4: ...
    @property
    def brga(self) -> U32Vector4: ...
    @property
    def brgo(self) -> U32Vector4: ...
    @property
    def brgl(self) -> U32Vector4: ...
    @property
    def brbr(self) -> U32Vector4: ...
    @property
    def brbg(self) -> U32Vector4: ...
    @property
    def brbb(self) -> U32Vector4: ...
    @property
    def brba(self) -> U32Vector4: ...
    @property
    def brbo(self) -> U32Vector4: ...
    @property
    def brbl(self) -> U32Vector4: ...
    @property
    def brar(self) -> U32Vector4: ...
    @property
    def brag(self) -> U32Vector4: ...
    @property
    def brab(self) -> U32Vector4: ...
    @property
    def braa(self) -> U32Vector4: ...
    @property
    def brao(self) -> U32Vector4: ...
    @property
    def bral(self) -> U32Vector4: ...
    @property
    def bror(self) -> U32Vector4: ...
    @property
    def brog(self) -> U32Vector4: ...
    @property
    def brob(self) -> U32Vector4: ...
    @property
    def broa(self) -> U32Vector4: ...
    @property
    def broo(self) -> U32Vector4: ...
    @property
    def brol(self) -> U32Vector4: ...
    @property
    def brlr(self) -> U32Vector4: ...
    @property
    def brlg(self) -> U32Vector4: ...
    @property
    def brlb(self) -> U32Vector4: ...
    @property
    def brla(self) -> U32Vector4: ...
    @property
    def brlo(self) -> U32Vector4: ...
    @property
    def brll(self) -> U32Vector4: ...
    @property
    def bgrr(self) -> U32Vector4: ...
    @property
    def bgrg(self) -> U32Vector4: ...
    @property
    def bgrb(self) -> U32Vector4: ...
    @property
    def bgra(self) -> U32Vector4: ...
    @property
    def bgro(self) -> U32Vector4: ...
    @property
    def bgrl(self) -> U32Vector4: ...
    @property
    def bggr(self) -> U32Vector4: ...
    @property
    def bggg(self) -> U32Vector4: ...
    @property
    def bggb(self) -> U32Vector4: ...
    @property
    def bgga(self) -> U32Vector4: ...
    @property
    def bggo(self) -> U32Vector4: ...
    @property
    def bggl(self) -> U32Vector4: ...
    @property
    def bgbr(self) -> U32Vector4: ...
    @property
    def bgbg(self) -> U32Vector4: ...
    @property
    def bgbb(self) -> U32Vector4: ...
    @property
    def bgba(self) -> U32Vector4: ...
    @property
    def bgbo(self) -> U32Vector4: ...
    @property
    def bgbl(self) -> U32Vector4: ...
    @property
    def bgar(self) -> U32Vector4: ...
    @property
    def bgag(self) -> U32Vector4: ...
    @property
    def bgab(self) -> U32Vector4: ...
    @property
    def bgaa(self) -> U32Vector4: ...
    @property
    def bgao(self) -> U32Vector4: ...
    @property
    def bgal(self) -> U32Vector4: ...
    @property
    def bgor(self) -> U32Vector4: ...
    @property
    def bgog(self) -> U32Vector4: ...
    @property
    def bgob(self) -> U32Vector4: ...
    @property
    def bgoa(self) -> U32Vector4: ...
    @property
    def bgoo(self) -> U32Vector4: ...
    @property
    def bgol(self) -> U32Vector4: ...
    @property
    def bglr(self) -> U32Vector4: ...
    @property
    def bglg(self) -> U32Vector4: ...
    @property
    def bglb(self) -> U32Vector4: ...
    @property
    def bgla(self) -> U32Vector4: ...
    @property
    def bglo(self) -> U32Vector4: ...
    @property
    def bgll(self) -> U32Vector4: ...
    @property
    def bbrr(self) -> U32Vector4: ...
    @property
    def bbrg(self) -> U32Vector4: ...
    @property
    def bbrb(self) -> U32Vector4: ...
    @property
    def bbra(self) -> U32Vector4: ...
    @property
    def bbro(self) -> U32Vector4: ...
    @property
    def bbrl(self) -> U32Vector4: ...
    @property
    def bbgr(self) -> U32Vector4: ...
    @property
    def bbgg(self) -> U32Vector4: ...
    @property
    def bbgb(self) -> U32Vector4: ...
    @property
    def bbga(self) -> U32Vector4: ...
    @property
    def bbgo(self) -> U32Vector4: ...
    @property
    def bbgl(self) -> U32Vector4: ...
    @property
    def bbbr(self) -> U32Vector4: ...
    @property
    def bbbg(self) -> U32Vector4: ...
    @property
    def bbbb(self) -> U32Vector4: ...
    @property
    def bbba(self) -> U32Vector4: ...
    @property
    def bbbo(self) -> U32Vector4: ...
    @property
    def bbbl(self) -> U32Vector4: ...
    @property
    def bbar(self) -> U32Vector4: ...
    @property
    def bbag(self) -> U32Vector4: ...
    @property
    def bbab(self) -> U32Vector4: ...
    @property
    def bbaa(self) -> U32Vector4: ...
    @property
    def bbao(self) -> U32Vector4: ...
    @property
    def bbal(self) -> U32Vector4: ...
    @property
    def bbor(self) -> U32Vector4: ...
    @property
    def bbog(self) -> U32Vector4: ...
    @property
    def bbob(self) -> U32Vector4: ...
    @property
    def bboa(self) -> U32Vector4: ...
    @property
    def bboo(self) -> U32Vector4: ...
    @property
    def bbol(self) -> U32Vector4: ...
    @property
    def bblr(self) -> U32Vector4: ...
    @property
    def bblg(self) -> U32Vector4: ...
    @property
    def bblb(self) -> U32Vector4: ...
    @property
    def bbla(self) -> U32Vector4: ...
    @property
    def bblo(self) -> U32Vector4: ...
    @property
    def bbll(self) -> U32Vector4: ...
    @property
    def barr(self) -> U32Vector4: ...
    @property
    def barg(self) -> U32Vector4: ...
    @property
    def barb(self) -> U32Vector4: ...
    @property
    def bara(self) -> U32Vector4: ...
    @property
    def baro(self) -> U32Vector4: ...
    @property
    def barl(self) -> U32Vector4: ...
    @property
    def bagr(self) -> U32Vector4: ...
    @property
    def bagg(self) -> U32Vector4: ...
    @property
    def bagb(self) -> U32Vector4: ...
    @property
    def baga(self) -> U32Vector4: ...
    @property
    def bago(self) -> U32Vector4: ...
    @property
    def bagl(self) -> U32Vector4: ...
    @property
    def babr(self) -> U32Vector4: ...
    @property
    def babg(self) -> U32Vector4: ...
    @property
    def babb(self) -> U32Vector4: ...
    @property
    def baba(self) -> U32Vector4: ...
    @property
    def babo(self) -> U32Vector4: ...
    @property
    def babl(self) -> U32Vector4: ...
    @property
    def baar(self) -> U32Vector4: ...
    @property
    def baag(self) -> U32Vector4: ...
    @property
    def baab(self) -> U32Vector4: ...
    @property
    def baaa(self) -> U32Vector4: ...
    @property
    def baao(self) -> U32Vector4: ...
    @property
    def baal(self) -> U32Vector4: ...
    @property
    def baor(self) -> U32Vector4: ...
    @property
    def baog(self) -> U32Vector4: ...
    @property
    def baob(self) -> U32Vector4: ...
    @property
    def baoa(self) -> U32Vector4: ...
    @property
    def baoo(self) -> U32Vector4: ...
    @property
    def baol(self) -> U32Vector4: ...
    @property
    def balr(self) -> U32Vector4: ...
    @property
    def balg(self) -> U32Vector4: ...
    @property
    def balb(self) -> U32Vector4: ...
    @property
    def bala(self) -> U32Vector4: ...
    @property
    def balo(self) -> U32Vector4: ...
    @property
    def ball(self) -> U32Vector4: ...
    @property
    def borr(self) -> U32Vector4: ...
    @property
    def borg(self) -> U32Vector4: ...
    @property
    def borb(self) -> U32Vector4: ...
    @property
    def bora(self) -> U32Vector4: ...
    @property
    def boro(self) -> U32Vector4: ...
    @property
    def borl(self) -> U32Vector4: ...
    @property
    def bogr(self) -> U32Vector4: ...
    @property
    def bogg(self) -> U32Vector4: ...
    @property
    def bogb(self) -> U32Vector4: ...
    @property
    def boga(self) -> U32Vector4: ...
    @property
    def bogo(self) -> U32Vector4: ...
    @property
    def bogl(self) -> U32Vector4: ...
    @property
    def bobr(self) -> U32Vector4: ...
    @property
    def bobg(self) -> U32Vector4: ...
    @property
    def bobb(self) -> U32Vector4: ...
    @property
    def boba(self) -> U32Vector4: ...
    @property
    def bobo(self) -> U32Vector4: ...
    @property
    def bobl(self) -> U32Vector4: ...
    @property
    def boar(self) -> U32Vector4: ...
    @property
    def boag(self) -> U32Vector4: ...
    @property
    def boab(self) -> U32Vector4: ...
    @property
    def boaa(self) -> U32Vector4: ...
    @property
    def boao(self) -> U32Vector4: ...
    @property
    def boal(self) -> U32Vector4: ...
    @property
    def boor(self) -> U32Vector4: ...
    @property
    def boog(self) -> U32Vector4: ...
    @property
    def boob(self) -> U32Vector4: ...
    @property
    def booa(self) -> U32Vector4: ...
    @property
    def booo(self) -> U32Vector4: ...
    @property
    def bool(self) -> U32Vector4: ...
    @property
    def bolr(self) -> U32Vector4: ...
    @property
    def bolg(self) -> U32Vector4: ...
    @property
    def bolb(self) -> U32Vector4: ...
    @property
    def bola(self) -> U32Vector4: ...
    @property
    def bolo(self) -> U32Vector4: ...
    @property
    def boll(self) -> U32Vector4: ...
    @property
    def blrr(self) -> U32Vector4: ...
    @property
    def blrg(self) -> U32Vector4: ...
    @property
    def blrb(self) -> U32Vector4: ...
    @property
    def blra(self) -> U32Vector4: ...
    @property
    def blro(self) -> U32Vector4: ...
    @property
    def blrl(self) -> U32Vector4: ...
    @property
    def blgr(self) -> U32Vector4: ...
    @property
    def blgg(self) -> U32Vector4: ...
    @property
    def blgb(self) -> U32Vector4: ...
    @property
    def blga(self) -> U32Vector4: ...
    @property
    def blgo(self) -> U32Vector4: ...
    @property
    def blgl(self) -> U32Vector4: ...
    @property
    def blbr(self) -> U32Vector4: ...
    @property
    def blbg(self) -> U32Vector4: ...
    @property
    def blbb(self) -> U32Vector4: ...
    @property
    def blba(self) -> U32Vector4: ...
    @property
    def blbo(self) -> U32Vector4: ...
    @property
    def blbl(self) -> U32Vector4: ...
    @property
    def blar(self) -> U32Vector4: ...
    @property
    def blag(self) -> U32Vector4: ...
    @property
    def blab(self) -> U32Vector4: ...
    @property
    def blaa(self) -> U32Vector4: ...
    @property
    def blao(self) -> U32Vector4: ...
    @property
    def blal(self) -> U32Vector4: ...
    @property
    def blor(self) -> U32Vector4: ...
    @property
    def blog(self) -> U32Vector4: ...
    @property
    def blob(self) -> U32Vector4: ...
    @property
    def bloa(self) -> U32Vector4: ...
    @property
    def bloo(self) -> U32Vector4: ...
    @property
    def blol(self) -> U32Vector4: ...
    @property
    def bllr(self) -> U32Vector4: ...
    @property
    def bllg(self) -> U32Vector4: ...
    @property
    def bllb(self) -> U32Vector4: ...
    @property
    def blla(self) -> U32Vector4: ...
    @property
    def bllo(self) -> U32Vector4: ...
    @property
    def blll(self) -> U32Vector4: ...
    @property
    def arrr(self) -> U32Vector4: ...
    @property
    def arrg(self) -> U32Vector4: ...
    @property
    def arrb(self) -> U32Vector4: ...
    @property
    def arra(self) -> U32Vector4: ...
    @property
    def arro(self) -> U32Vector4: ...
    @property
    def arrl(self) -> U32Vector4: ...
    @property
    def argr(self) -> U32Vector4: ...
    @property
    def argg(self) -> U32Vector4: ...
    @property
    def argb(self) -> U32Vector4: ...
    @property
    def arga(self) -> U32Vector4: ...
    @property
    def argo(self) -> U32Vector4: ...
    @property
    def argl(self) -> U32Vector4: ...
    @property
    def arbr(self) -> U32Vector4: ...
    @property
    def arbg(self) -> U32Vector4: ...
    @property
    def arbb(self) -> U32Vector4: ...
    @property
    def arba(self) -> U32Vector4: ...
    @property
    def arbo(self) -> U32Vector4: ...
    @property
    def arbl(self) -> U32Vector4: ...
    @property
    def arar(self) -> U32Vector4: ...
    @property
    def arag(self) -> U32Vector4: ...
    @property
    def arab(self) -> U32Vector4: ...
    @property
    def araa(self) -> U32Vector4: ...
    @property
    def arao(self) -> U32Vector4: ...
    @property
    def aral(self) -> U32Vector4: ...
    @property
    def aror(self) -> U32Vector4: ...
    @property
    def arog(self) -> U32Vector4: ...
    @property
    def arob(self) -> U32Vector4: ...
    @property
    def aroa(self) -> U32Vector4: ...
    @property
    def aroo(self) -> U32Vector4: ...
    @property
    def arol(self) -> U32Vector4: ...
    @property
    def arlr(self) -> U32Vector4: ...
    @property
    def arlg(self) -> U32Vector4: ...
    @property
    def arlb(self) -> U32Vector4: ...
    @property
    def arla(self) -> U32Vector4: ...
    @property
    def arlo(self) -> U32Vector4: ...
    @property
    def arll(self) -> U32Vector4: ...
    @property
    def agrr(self) -> U32Vector4: ...
    @property
    def agrg(self) -> U32Vector4: ...
    @property
    def agrb(self) -> U32Vector4: ...
    @property
    def agra(self) -> U32Vector4: ...
    @property
    def agro(self) -> U32Vector4: ...
    @property
    def agrl(self) -> U32Vector4: ...
    @property
    def aggr(self) -> U32Vector4: ...
    @property
    def aggg(self) -> U32Vector4: ...
    @property
    def aggb(self) -> U32Vector4: ...
    @property
    def agga(self) -> U32Vector4: ...
    @property
    def aggo(self) -> U32Vector4: ...
    @property
    def aggl(self) -> U32Vector4: ...
    @property
    def agbr(self) -> U32Vector4: ...
    @property
    def agbg(self) -> U32Vector4: ...
    @property
    def agbb(self) -> U32Vector4: ...
    @property
    def agba(self) -> U32Vector4: ...
    @property
    def agbo(self) -> U32Vector4: ...
    @property
    def agbl(self) -> U32Vector4: ...
    @property
    def agar(self) -> U32Vector4: ...
    @property
    def agag(self) -> U32Vector4: ...
    @property
    def agab(self) -> U32Vector4: ...
    @property
    def agaa(self) -> U32Vector4: ...
    @property
    def agao(self) -> U32Vector4: ...
    @property
    def agal(self) -> U32Vector4: ...
    @property
    def agor(self) -> U32Vector4: ...
    @property
    def agog(self) -> U32Vector4: ...
    @property
    def agob(self) -> U32Vector4: ...
    @property
    def agoa(self) -> U32Vector4: ...
    @property
    def agoo(self) -> U32Vector4: ...
    @property
    def agol(self) -> U32Vector4: ...
    @property
    def aglr(self) -> U32Vector4: ...
    @property
    def aglg(self) -> U32Vector4: ...
    @property
    def aglb(self) -> U32Vector4: ...
    @property
    def agla(self) -> U32Vector4: ...
    @property
    def aglo(self) -> U32Vector4: ...
    @property
    def agll(self) -> U32Vector4: ...
    @property
    def abrr(self) -> U32Vector4: ...
    @property
    def abrg(self) -> U32Vector4: ...
    @property
    def abrb(self) -> U32Vector4: ...
    @property
    def abra(self) -> U32Vector4: ...
    @property
    def abro(self) -> U32Vector4: ...
    @property
    def abrl(self) -> U32Vector4: ...
    @property
    def abgr(self) -> U32Vector4: ...
    @property
    def abgg(self) -> U32Vector4: ...
    @property
    def abgb(self) -> U32Vector4: ...
    @property
    def abga(self) -> U32Vector4: ...
    @property
    def abgo(self) -> U32Vector4: ...
    @property
    def abgl(self) -> U32Vector4: ...
    @property
    def abbr(self) -> U32Vector4: ...
    @property
    def abbg(self) -> U32Vector4: ...
    @property
    def abbb(self) -> U32Vector4: ...
    @property
    def abba(self) -> U32Vector4: ...
    @property
    def abbo(self) -> U32Vector4: ...
    @property
    def abbl(self) -> U32Vector4: ...
    @property
    def abar(self) -> U32Vector4: ...
    @property
    def abag(self) -> U32Vector4: ...
    @property
    def abab(self) -> U32Vector4: ...
    @property
    def abaa(self) -> U32Vector4: ...
    @property
    def abao(self) -> U32Vector4: ...
    @property
    def abal(self) -> U32Vector4: ...
    @property
    def abor(self) -> U32Vector4: ...
    @property
    def abog(self) -> U32Vector4: ...
    @property
    def abob(self) -> U32Vector4: ...
    @property
    def aboa(self) -> U32Vector4: ...
    @property
    def aboo(self) -> U32Vector4: ...
    @property
    def abol(self) -> U32Vector4: ...
    @property
    def ablr(self) -> U32Vector4: ...
    @property
    def ablg(self) -> U32Vector4: ...
    @property
    def ablb(self) -> U32Vector4: ...
    @property
    def abla(self) -> U32Vector4: ...
    @property
    def ablo(self) -> U32Vector4: ...
    @property
    def abll(self) -> U32Vector4: ...
    @property
    def aarr(self) -> U32Vector4: ...
    @property
    def aarg(self) -> U32Vector4: ...
    @property
    def aarb(self) -> U32Vector4: ...
    @property
    def aara(self) -> U32Vector4: ...
    @property
    def aaro(self) -> U32Vector4: ...
    @property
    def aarl(self) -> U32Vector4: ...
    @property
    def aagr(self) -> U32Vector4: ...
    @property
    def aagg(self) -> U32Vector4: ...
    @property
    def aagb(self) -> U32Vector4: ...
    @property
    def aaga(self) -> U32Vector4: ...
    @property
    def aago(self) -> U32Vector4: ...
    @property
    def aagl(self) -> U32Vector4: ...
    @property
    def aabr(self) -> U32Vector4: ...
    @property
    def aabg(self) -> U32Vector4: ...
    @property
    def aabb(self) -> U32Vector4: ...
    @property
    def aaba(self) -> U32Vector4: ...
    @property
    def aabo(self) -> U32Vector4: ...
    @property
    def aabl(self) -> U32Vector4: ...
    @property
    def aaar(self) -> U32Vector4: ...
    @property
    def aaag(self) -> U32Vector4: ...
    @property
    def aaab(self) -> U32Vector4: ...
    @property
    def aaaa(self) -> U32Vector4: ...
    @property
    def aaao(self) -> U32Vector4: ...
    @property
    def aaal(self) -> U32Vector4: ...
    @property
    def aaor(self) -> U32Vector4: ...
    @property
    def aaog(self) -> U32Vector4: ...
    @property
    def aaob(self) -> U32Vector4: ...
    @property
    def aaoa(self) -> U32Vector4: ...
    @property
    def aaoo(self) -> U32Vector4: ...
    @property
    def aaol(self) -> U32Vector4: ...
    @property
    def aalr(self) -> U32Vector4: ...
    @property
    def aalg(self) -> U32Vector4: ...
    @property
    def aalb(self) -> U32Vector4: ...
    @property
    def aala(self) -> U32Vector4: ...
    @property
    def aalo(self) -> U32Vector4: ...
    @property
    def aall(self) -> U32Vector4: ...
    @property
    def aorr(self) -> U32Vector4: ...
    @property
    def aorg(self) -> U32Vector4: ...
    @property
    def aorb(self) -> U32Vector4: ...
    @property
    def aora(self) -> U32Vector4: ...
    @property
    def aoro(self) -> U32Vector4: ...
    @property
    def aorl(self) -> U32Vector4: ...
    @property
    def aogr(self) -> U32Vector4: ...
    @property
    def aogg(self) -> U32Vector4: ...
    @property
    def aogb(self) -> U32Vector4: ...
    @property
    def aoga(self) -> U32Vector4: ...
    @property
    def aogo(self) -> U32Vector4: ...
    @property
    def aogl(self) -> U32Vector4: ...
    @property
    def aobr(self) -> U32Vector4: ...
    @property
    def aobg(self) -> U32Vector4: ...
    @property
    def aobb(self) -> U32Vector4: ...
    @property
    def aoba(self) -> U32Vector4: ...
    @property
    def aobo(self) -> U32Vector4: ...
    @property
    def aobl(self) -> U32Vector4: ...
    @property
    def aoar(self) -> U32Vector4: ...
    @property
    def aoag(self) -> U32Vector4: ...
    @property
    def aoab(self) -> U32Vector4: ...
    @property
    def aoaa(self) -> U32Vector4: ...
    @property
    def aoao(self) -> U32Vector4: ...
    @property
    def aoal(self) -> U32Vector4: ...
    @property
    def aoor(self) -> U32Vector4: ...
    @property
    def aoog(self) -> U32Vector4: ...
    @property
    def aoob(self) -> U32Vector4: ...
    @property
    def aooa(self) -> U32Vector4: ...
    @property
    def aooo(self) -> U32Vector4: ...
    @property
    def aool(self) -> U32Vector4: ...
    @property
    def aolr(self) -> U32Vector4: ...
    @property
    def aolg(self) -> U32Vector4: ...
    @property
    def aolb(self) -> U32Vector4: ...
    @property
    def aola(self) -> U32Vector4: ...
    @property
    def aolo(self) -> U32Vector4: ...
    @property
    def aoll(self) -> U32Vector4: ...
    @property
    def alrr(self) -> U32Vector4: ...
    @property
    def alrg(self) -> U32Vector4: ...
    @property
    def alrb(self) -> U32Vector4: ...
    @property
    def alra(self) -> U32Vector4: ...
    @property
    def alro(self) -> U32Vector4: ...
    @property
    def alrl(self) -> U32Vector4: ...
    @property
    def algr(self) -> U32Vector4: ...
    @property
    def algg(self) -> U32Vector4: ...
    @property
    def algb(self) -> U32Vector4: ...
    @property
    def alga(self) -> U32Vector4: ...
    @property
    def algo(self) -> U32Vector4: ...
    @property
    def algl(self) -> U32Vector4: ...
    @property
    def albr(self) -> U32Vector4: ...
    @property
    def albg(self) -> U32Vector4: ...
    @property
    def albb(self) -> U32Vector4: ...
    @property
    def alba(self) -> U32Vector4: ...
    @property
    def albo(self) -> U32Vector4: ...
    @property
    def albl(self) -> U32Vector4: ...
    @property
    def alar(self) -> U32Vector4: ...
    @property
    def alag(self) -> U32Vector4: ...
    @property
    def alab(self) -> U32Vector4: ...
    @property
    def alaa(self) -> U32Vector4: ...
    @property
    def alao(self) -> U32Vector4: ...
    @property
    def alal(self) -> U32Vector4: ...
    @property
    def alor(self) -> U32Vector4: ...
    @property
    def alog(self) -> U32Vector4: ...
    @property
    def alob(self) -> U32Vector4: ...
    @property
    def aloa(self) -> U32Vector4: ...
    @property
    def aloo(self) -> U32Vector4: ...
    @property
    def alol(self) -> U32Vector4: ...
    @property
    def allr(self) -> U32Vector4: ...
    @property
    def allg(self) -> U32Vector4: ...
    @property
    def allb(self) -> U32Vector4: ...
    @property
    def alla(self) -> U32Vector4: ...
    @property
    def allo(self) -> U32Vector4: ...
    @property
    def alll(self) -> U32Vector4: ...
    @property
    def orrr(self) -> U32Vector4: ...
    @property
    def orrg(self) -> U32Vector4: ...
    @property
    def orrb(self) -> U32Vector4: ...
    @property
    def orra(self) -> U32Vector4: ...
    @property
    def orro(self) -> U32Vector4: ...
    @property
    def orrl(self) -> U32Vector4: ...
    @property
    def orgr(self) -> U32Vector4: ...
    @property
    def orgg(self) -> U32Vector4: ...
    @property
    def orgb(self) -> U32Vector4: ...
    @property
    def orga(self) -> U32Vector4: ...
    @property
    def orgo(self) -> U32Vector4: ...
    @property
    def orgl(self) -> U32Vector4: ...
    @property
    def orbr(self) -> U32Vector4: ...
    @property
    def orbg(self) -> U32Vector4: ...
    @property
    def orbb(self) -> U32Vector4: ...
    @property
    def orba(self) -> U32Vector4: ...
    @property
    def orbo(self) -> U32Vector4: ...
    @property
    def orbl(self) -> U32Vector4: ...
    @property
    def orar(self) -> U32Vector4: ...
    @property
    def orag(self) -> U32Vector4: ...
    @property
    def orab(self) -> U32Vector4: ...
    @property
    def oraa(self) -> U32Vector4: ...
    @property
    def orao(self) -> U32Vector4: ...
    @property
    def oral(self) -> U32Vector4: ...
    @property
    def oror(self) -> U32Vector4: ...
    @property
    def orog(self) -> U32Vector4: ...
    @property
    def orob(self) -> U32Vector4: ...
    @property
    def oroa(self) -> U32Vector4: ...
    @property
    def oroo(self) -> U32Vector4: ...
    @property
    def orol(self) -> U32Vector4: ...
    @property
    def orlr(self) -> U32Vector4: ...
    @property
    def orlg(self) -> U32Vector4: ...
    @property
    def orlb(self) -> U32Vector4: ...
    @property
    def orla(self) -> U32Vector4: ...
    @property
    def orlo(self) -> U32Vector4: ...
    @property
    def orll(self) -> U32Vector4: ...
    @property
    def ogrr(self) -> U32Vector4: ...
    @property
    def ogrg(self) -> U32Vector4: ...
    @property
    def ogrb(self) -> U32Vector4: ...
    @property
    def ogra(self) -> U32Vector4: ...
    @property
    def ogro(self) -> U32Vector4: ...
    @property
    def ogrl(self) -> U32Vector4: ...
    @property
    def oggr(self) -> U32Vector4: ...
    @property
    def oggg(self) -> U32Vector4: ...
    @property
    def oggb(self) -> U32Vector4: ...
    @property
    def ogga(self) -> U32Vector4: ...
    @property
    def oggo(self) -> U32Vector4: ...
    @property
    def oggl(self) -> U32Vector4: ...
    @property
    def ogbr(self) -> U32Vector4: ...
    @property
    def ogbg(self) -> U32Vector4: ...
    @property
    def ogbb(self) -> U32Vector4: ...
    @property
    def ogba(self) -> U32Vector4: ...
    @property
    def ogbo(self) -> U32Vector4: ...
    @property
    def ogbl(self) -> U32Vector4: ...
    @property
    def ogar(self) -> U32Vector4: ...
    @property
    def ogag(self) -> U32Vector4: ...
    @property
    def ogab(self) -> U32Vector4: ...
    @property
    def ogaa(self) -> U32Vector4: ...
    @property
    def ogao(self) -> U32Vector4: ...
    @property
    def ogal(self) -> U32Vector4: ...
    @property
    def ogor(self) -> U32Vector4: ...
    @property
    def ogog(self) -> U32Vector4: ...
    @property
    def ogob(self) -> U32Vector4: ...
    @property
    def ogoa(self) -> U32Vector4: ...
    @property
    def ogoo(self) -> U32Vector4: ...
    @property
    def ogol(self) -> U32Vector4: ...
    @property
    def oglr(self) -> U32Vector4: ...
    @property
    def oglg(self) -> U32Vector4: ...
    @property
    def oglb(self) -> U32Vector4: ...
    @property
    def ogla(self) -> U32Vector4: ...
    @property
    def oglo(self) -> U32Vector4: ...
    @property
    def ogll(self) -> U32Vector4: ...
    @property
    def obrr(self) -> U32Vector4: ...
    @property
    def obrg(self) -> U32Vector4: ...
    @property
    def obrb(self) -> U32Vector4: ...
    @property
    def obra(self) -> U32Vector4: ...
    @property
    def obro(self) -> U32Vector4: ...
    @property
    def obrl(self) -> U32Vector4: ...
    @property
    def obgr(self) -> U32Vector4: ...
    @property
    def obgg(self) -> U32Vector4: ...
    @property
    def obgb(self) -> U32Vector4: ...
    @property
    def obga(self) -> U32Vector4: ...
    @property
    def obgo(self) -> U32Vector4: ...
    @property
    def obgl(self) -> U32Vector4: ...
    @property
    def obbr(self) -> U32Vector4: ...
    @property
    def obbg(self) -> U32Vector4: ...
    @property
    def obbb(self) -> U32Vector4: ...
    @property
    def obba(self) -> U32Vector4: ...
    @property
    def obbo(self) -> U32Vector4: ...
    @property
    def obbl(self) -> U32Vector4: ...
    @property
    def obar(self) -> U32Vector4: ...
    @property
    def obag(self) -> U32Vector4: ...
    @property
    def obab(self) -> U32Vector4: ...
    @property
    def obaa(self) -> U32Vector4: ...
    @property
    def obao(self) -> U32Vector4: ...
    @property
    def obal(self) -> U32Vector4: ...
    @property
    def obor(self) -> U32Vector4: ...
    @property
    def obog(self) -> U32Vector4: ...
    @property
    def obob(self) -> U32Vector4: ...
    @property
    def oboa(self) -> U32Vector4: ...
    @property
    def oboo(self) -> U32Vector4: ...
    @property
    def obol(self) -> U32Vector4: ...
    @property
    def oblr(self) -> U32Vector4: ...
    @property
    def oblg(self) -> U32Vector4: ...
    @property
    def oblb(self) -> U32Vector4: ...
    @property
    def obla(self) -> U32Vector4: ...
    @property
    def oblo(self) -> U32Vector4: ...
    @property
    def obll(self) -> U32Vector4: ...
    @property
    def oarr(self) -> U32Vector4: ...
    @property
    def oarg(self) -> U32Vector4: ...
    @property
    def oarb(self) -> U32Vector4: ...
    @property
    def oara(self) -> U32Vector4: ...
    @property
    def oaro(self) -> U32Vector4: ...
    @property
    def oarl(self) -> U32Vector4: ...
    @property
    def oagr(self) -> U32Vector4: ...
    @property
    def oagg(self) -> U32Vector4: ...
    @property
    def oagb(self) -> U32Vector4: ...
    @property
    def oaga(self) -> U32Vector4: ...
    @property
    def oago(self) -> U32Vector4: ...
    @property
    def oagl(self) -> U32Vector4: ...
    @property
    def oabr(self) -> U32Vector4: ...
    @property
    def oabg(self) -> U32Vector4: ...
    @property
    def oabb(self) -> U32Vector4: ...
    @property
    def oaba(self) -> U32Vector4: ...
    @property
    def oabo(self) -> U32Vector4: ...
    @property
    def oabl(self) -> U32Vector4: ...
    @property
    def oaar(self) -> U32Vector4: ...
    @property
    def oaag(self) -> U32Vector4: ...
    @property
    def oaab(self) -> U32Vector4: ...
    @property
    def oaaa(self) -> U32Vector4: ...
    @property
    def oaao(self) -> U32Vector4: ...
    @property
    def oaal(self) -> U32Vector4: ...
    @property
    def oaor(self) -> U32Vector4: ...
    @property
    def oaog(self) -> U32Vector4: ...
    @property
    def oaob(self) -> U32Vector4: ...
    @property
    def oaoa(self) -> U32Vector4: ...
    @property
    def oaoo(self) -> U32Vector4: ...
    @property
    def oaol(self) -> U32Vector4: ...
    @property
    def oalr(self) -> U32Vector4: ...
    @property
    def oalg(self) -> U32Vector4: ...
    @property
    def oalb(self) -> U32Vector4: ...
    @property
    def oala(self) -> U32Vector4: ...
    @property
    def oalo(self) -> U32Vector4: ...
    @property
    def oall(self) -> U32Vector4: ...
    @property
    def oorr(self) -> U32Vector4: ...
    @property
    def oorg(self) -> U32Vector4: ...
    @property
    def oorb(self) -> U32Vector4: ...
    @property
    def oora(self) -> U32Vector4: ...
    @property
    def ooro(self) -> U32Vector4: ...
    @property
    def oorl(self) -> U32Vector4: ...
    @property
    def oogr(self) -> U32Vector4: ...
    @property
    def oogg(self) -> U32Vector4: ...
    @property
    def oogb(self) -> U32Vector4: ...
    @property
    def ooga(self) -> U32Vector4: ...
    @property
    def oogo(self) -> U32Vector4: ...
    @property
    def oogl(self) -> U32Vector4: ...
    @property
    def oobr(self) -> U32Vector4: ...
    @property
    def oobg(self) -> U32Vector4: ...
    @property
    def oobb(self) -> U32Vector4: ...
    @property
    def ooba(self) -> U32Vector4: ...
    @property
    def oobo(self) -> U32Vector4: ...
    @property
    def oobl(self) -> U32Vector4: ...
    @property
    def ooar(self) -> U32Vector4: ...
    @property
    def ooag(self) -> U32Vector4: ...
    @property
    def ooab(self) -> U32Vector4: ...
    @property
    def ooaa(self) -> U32Vector4: ...
    @property
    def ooao(self) -> U32Vector4: ...
    @property
    def ooal(self) -> U32Vector4: ...
    @property
    def ooor(self) -> U32Vector4: ...
    @property
    def ooog(self) -> U32Vector4: ...
    @property
    def ooob(self) -> U32Vector4: ...
    @property
    def oooa(self) -> U32Vector4: ...
    @property
    def oolr(self) -> U32Vector4: ...
    @property
    def oolg(self) -> U32Vector4: ...
    @property
    def oolb(self) -> U32Vector4: ...
    @property
    def oola(self) -> U32Vector4: ...
    @property
    def olrr(self) -> U32Vector4: ...
    @property
    def olrg(self) -> U32Vector4: ...
    @property
    def olrb(self) -> U32Vector4: ...
    @property
    def olra(self) -> U32Vector4: ...
    @property
    def olro(self) -> U32Vector4: ...
    @property
    def olrl(self) -> U32Vector4: ...
    @property
    def olgr(self) -> U32Vector4: ...
    @property
    def olgg(self) -> U32Vector4: ...
    @property
    def olgb(self) -> U32Vector4: ...
    @property
    def olga(self) -> U32Vector4: ...
    @property
    def olgo(self) -> U32Vector4: ...
    @property
    def olgl(self) -> U32Vector4: ...
    @property
    def olbr(self) -> U32Vector4: ...
    @property
    def olbg(self) -> U32Vector4: ...
    @property
    def olbb(self) -> U32Vector4: ...
    @property
    def olba(self) -> U32Vector4: ...
    @property
    def olbo(self) -> U32Vector4: ...
    @property
    def olbl(self) -> U32Vector4: ...
    @property
    def olar(self) -> U32Vector4: ...
    @property
    def olag(self) -> U32Vector4: ...
    @property
    def olab(self) -> U32Vector4: ...
    @property
    def olaa(self) -> U32Vector4: ...
    @property
    def olao(self) -> U32Vector4: ...
    @property
    def olal(self) -> U32Vector4: ...
    @property
    def olor(self) -> U32Vector4: ...
    @property
    def olog(self) -> U32Vector4: ...
    @property
    def olob(self) -> U32Vector4: ...
    @property
    def oloa(self) -> U32Vector4: ...
    @property
    def ollr(self) -> U32Vector4: ...
    @property
    def ollg(self) -> U32Vector4: ...
    @property
    def ollb(self) -> U32Vector4: ...
    @property
    def olla(self) -> U32Vector4: ...
    @property
    def lrrr(self) -> U32Vector4: ...
    @property
    def lrrg(self) -> U32Vector4: ...
    @property
    def lrrb(self) -> U32Vector4: ...
    @property
    def lrra(self) -> U32Vector4: ...
    @property
    def lrro(self) -> U32Vector4: ...
    @property
    def lrrl(self) -> U32Vector4: ...
    @property
    def lrgr(self) -> U32Vector4: ...
    @property
    def lrgg(self) -> U32Vector4: ...
    @property
    def lrgb(self) -> U32Vector4: ...
    @property
    def lrga(self) -> U32Vector4: ...
    @property
    def lrgo(self) -> U32Vector4: ...
    @property
    def lrgl(self) -> U32Vector4: ...
    @property
    def lrbr(self) -> U32Vector4: ...
    @property
    def lrbg(self) -> U32Vector4: ...
    @property
    def lrbb(self) -> U32Vector4: ...
    @property
    def lrba(self) -> U32Vector4: ...
    @property
    def lrbo(self) -> U32Vector4: ...
    @property
    def lrbl(self) -> U32Vector4: ...
    @property
    def lrar(self) -> U32Vector4: ...
    @property
    def lrag(self) -> U32Vector4: ...
    @property
    def lrab(self) -> U32Vector4: ...
    @property
    def lraa(self) -> U32Vector4: ...
    @property
    def lrao(self) -> U32Vector4: ...
    @property
    def lral(self) -> U32Vector4: ...
    @property
    def lror(self) -> U32Vector4: ...
    @property
    def lrog(self) -> U32Vector4: ...
    @property
    def lrob(self) -> U32Vector4: ...
    @property
    def lroa(self) -> U32Vector4: ...
    @property
    def lroo(self) -> U32Vector4: ...
    @property
    def lrol(self) -> U32Vector4: ...
    @property
    def lrlr(self) -> U32Vector4: ...
    @property
    def lrlg(self) -> U32Vector4: ...
    @property
    def lrlb(self) -> U32Vector4: ...
    @property
    def lrla(self) -> U32Vector4: ...
    @property
    def lrlo(self) -> U32Vector4: ...
    @property
    def lrll(self) -> U32Vector4: ...
    @property
    def lgrr(self) -> U32Vector4: ...
    @property
    def lgrg(self) -> U32Vector4: ...
    @property
    def lgrb(self) -> U32Vector4: ...
    @property
    def lgra(self) -> U32Vector4: ...
    @property
    def lgro(self) -> U32Vector4: ...
    @property
    def lgrl(self) -> U32Vector4: ...
    @property
    def lggr(self) -> U32Vector4: ...
    @property
    def lggg(self) -> U32Vector4: ...
    @property
    def lggb(self) -> U32Vector4: ...
    @property
    def lgga(self) -> U32Vector4: ...
    @property
    def lggo(self) -> U32Vector4: ...
    @property
    def lggl(self) -> U32Vector4: ...
    @property
    def lgbr(self) -> U32Vector4: ...
    @property
    def lgbg(self) -> U32Vector4: ...
    @property
    def lgbb(self) -> U32Vector4: ...
    @property
    def lgba(self) -> U32Vector4: ...
    @property
    def lgbo(self) -> U32Vector4: ...
    @property
    def lgbl(self) -> U32Vector4: ...
    @property
    def lgar(self) -> U32Vector4: ...
    @property
    def lgag(self) -> U32Vector4: ...
    @property
    def lgab(self) -> U32Vector4: ...
    @property
    def lgaa(self) -> U32Vector4: ...
    @property
    def lgao(self) -> U32Vector4: ...
    @property
    def lgal(self) -> U32Vector4: ...
    @property
    def lgor(self) -> U32Vector4: ...
    @property
    def lgog(self) -> U32Vector4: ...
    @property
    def lgob(self) -> U32Vector4: ...
    @property
    def lgoa(self) -> U32Vector4: ...
    @property
    def lgoo(self) -> U32Vector4: ...
    @property
    def lgol(self) -> U32Vector4: ...
    @property
    def lglr(self) -> U32Vector4: ...
    @property
    def lglg(self) -> U32Vector4: ...
    @property
    def lglb(self) -> U32Vector4: ...
    @property
    def lgla(self) -> U32Vector4: ...
    @property
    def lglo(self) -> U32Vector4: ...
    @property
    def lgll(self) -> U32Vector4: ...
    @property
    def lbrr(self) -> U32Vector4: ...
    @property
    def lbrg(self) -> U32Vector4: ...
    @property
    def lbrb(self) -> U32Vector4: ...
    @property
    def lbra(self) -> U32Vector4: ...
    @property
    def lbro(self) -> U32Vector4: ...
    @property
    def lbrl(self) -> U32Vector4: ...
    @property
    def lbgr(self) -> U32Vector4: ...
    @property
    def lbgg(self) -> U32Vector4: ...
    @property
    def lbgb(self) -> U32Vector4: ...
    @property
    def lbga(self) -> U32Vector4: ...
    @property
    def lbgo(self) -> U32Vector4: ...
    @property
    def lbgl(self) -> U32Vector4: ...
    @property
    def lbbr(self) -> U32Vector4: ...
    @property
    def lbbg(self) -> U32Vector4: ...
    @property
    def lbbb(self) -> U32Vector4: ...
    @property
    def lbba(self) -> U32Vector4: ...
    @property
    def lbbo(self) -> U32Vector4: ...
    @property
    def lbbl(self) -> U32Vector4: ...
    @property
    def lbar(self) -> U32Vector4: ...
    @property
    def lbag(self) -> U32Vector4: ...
    @property
    def lbab(self) -> U32Vector4: ...
    @property
    def lbaa(self) -> U32Vector4: ...
    @property
    def lbao(self) -> U32Vector4: ...
    @property
    def lbal(self) -> U32Vector4: ...
    @property
    def lbor(self) -> U32Vector4: ...
    @property
    def lbog(self) -> U32Vector4: ...
    @property
    def lbob(self) -> U32Vector4: ...
    @property
    def lboa(self) -> U32Vector4: ...
    @property
    def lboo(self) -> U32Vector4: ...
    @property
    def lbol(self) -> U32Vector4: ...
    @property
    def lblr(self) -> U32Vector4: ...
    @property
    def lblg(self) -> U32Vector4: ...
    @property
    def lblb(self) -> U32Vector4: ...
    @property
    def lbla(self) -> U32Vector4: ...
    @property
    def lblo(self) -> U32Vector4: ...
    @property
    def lbll(self) -> U32Vector4: ...
    @property
    def larr(self) -> U32Vector4: ...
    @property
    def larg(self) -> U32Vector4: ...
    @property
    def larb(self) -> U32Vector4: ...
    @property
    def lara(self) -> U32Vector4: ...
    @property
    def laro(self) -> U32Vector4: ...
    @property
    def larl(self) -> U32Vector4: ...
    @property
    def lagr(self) -> U32Vector4: ...
    @property
    def lagg(self) -> U32Vector4: ...
    @property
    def lagb(self) -> U32Vector4: ...
    @property
    def laga(self) -> U32Vector4: ...
    @property
    def lago(self) -> U32Vector4: ...
    @property
    def lagl(self) -> U32Vector4: ...
    @property
    def labr(self) -> U32Vector4: ...
    @property
    def labg(self) -> U32Vector4: ...
    @property
    def labb(self) -> U32Vector4: ...
    @property
    def laba(self) -> U32Vector4: ...
    @property
    def labo(self) -> U32Vector4: ...
    @property
    def labl(self) -> U32Vector4: ...
    @property
    def laar(self) -> U32Vector4: ...
    @property
    def laag(self) -> U32Vector4: ...
    @property
    def laab(self) -> U32Vector4: ...
    @property
    def laaa(self) -> U32Vector4: ...
    @property
    def laao(self) -> U32Vector4: ...
    @property
    def laal(self) -> U32Vector4: ...
    @property
    def laor(self) -> U32Vector4: ...
    @property
    def laog(self) -> U32Vector4: ...
    @property
    def laob(self) -> U32Vector4: ...
    @property
    def laoa(self) -> U32Vector4: ...
    @property
    def laoo(self) -> U32Vector4: ...
    @property
    def laol(self) -> U32Vector4: ...
    @property
    def lalr(self) -> U32Vector4: ...
    @property
    def lalg(self) -> U32Vector4: ...
    @property
    def lalb(self) -> U32Vector4: ...
    @property
    def lala(self) -> U32Vector4: ...
    @property
    def lalo(self) -> U32Vector4: ...
    @property
    def lall(self) -> U32Vector4: ...
    @property
    def lorr(self) -> U32Vector4: ...
    @property
    def lorg(self) -> U32Vector4: ...
    @property
    def lorb(self) -> U32Vector4: ...
    @property
    def lora(self) -> U32Vector4: ...
    @property
    def loro(self) -> U32Vector4: ...
    @property
    def lorl(self) -> U32Vector4: ...
    @property
    def logr(self) -> U32Vector4: ...
    @property
    def logg(self) -> U32Vector4: ...
    @property
    def logb(self) -> U32Vector4: ...
    @property
    def loga(self) -> U32Vector4: ...
    @property
    def logo(self) -> U32Vector4: ...
    @property
    def logl(self) -> U32Vector4: ...
    @property
    def lobr(self) -> U32Vector4: ...
    @property
    def lobg(self) -> U32Vector4: ...
    @property
    def lobb(self) -> U32Vector4: ...
    @property
    def loba(self) -> U32Vector4: ...
    @property
    def lobo(self) -> U32Vector4: ...
    @property
    def lobl(self) -> U32Vector4: ...
    @property
    def loar(self) -> U32Vector4: ...
    @property
    def loag(self) -> U32Vector4: ...
    @property
    def loab(self) -> U32Vector4: ...
    @property
    def loaa(self) -> U32Vector4: ...
    @property
    def loao(self) -> U32Vector4: ...
    @property
    def loal(self) -> U32Vector4: ...
    @property
    def loor(self) -> U32Vector4: ...
    @property
    def loog(self) -> U32Vector4: ...
    @property
    def loob(self) -> U32Vector4: ...
    @property
    def looa(self) -> U32Vector4: ...
    @property
    def lolr(self) -> U32Vector4: ...
    @property
    def lolg(self) -> U32Vector4: ...
    @property
    def lolb(self) -> U32Vector4: ...
    @property
    def lola(self) -> U32Vector4: ...
    @property
    def llrr(self) -> U32Vector4: ...
    @property
    def llrg(self) -> U32Vector4: ...
    @property
    def llrb(self) -> U32Vector4: ...
    @property
    def llra(self) -> U32Vector4: ...
    @property
    def llro(self) -> U32Vector4: ...
    @property
    def llrl(self) -> U32Vector4: ...
    @property
    def llgr(self) -> U32Vector4: ...
    @property
    def llgg(self) -> U32Vector4: ...
    @property
    def llgb(self) -> U32Vector4: ...
    @property
    def llga(self) -> U32Vector4: ...
    @property
    def llgo(self) -> U32Vector4: ...
    @property
    def llgl(self) -> U32Vector4: ...
    @property
    def llbr(self) -> U32Vector4: ...
    @property
    def llbg(self) -> U32Vector4: ...
    @property
    def llbb(self) -> U32Vector4: ...
    @property
    def llba(self) -> U32Vector4: ...
    @property
    def llbo(self) -> U32Vector4: ...
    @property
    def llbl(self) -> U32Vector4: ...
    @property
    def llar(self) -> U32Vector4: ...
    @property
    def llag(self) -> U32Vector4: ...
    @property
    def llab(self) -> U32Vector4: ...
    @property
    def llaa(self) -> U32Vector4: ...
    @property
    def llao(self) -> U32Vector4: ...
    @property
    def llal(self) -> U32Vector4: ...
    @property
    def llor(self) -> U32Vector4: ...
    @property
    def llog(self) -> U32Vector4: ...
    @property
    def llob(self) -> U32Vector4: ...
    @property
    def lloa(self) -> U32Vector4: ...
    @property
    def lllr(self) -> U32Vector4: ...
    @property
    def lllg(self) -> U32Vector4: ...
    @property
    def lllb(self) -> U32Vector4: ...
    @property
    def llla(self) -> U32Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> U32Vector2: ...
    @property
    def st(self) -> U32Vector2: ...
    @property
    def sq(self) -> U32Vector2: ...
    @property
    def sp(self) -> U32Vector2: ...
    @property
    def so(self) -> U32Vector2: ...
    @property
    def sl(self) -> U32Vector2: ...
    @property
    def ts(self) -> U32Vector2: ...
    @property
    def tt(self) -> U32Vector2: ...
    @property
    def tq(self) -> U32Vector2: ...
    @property
    def tp(self) -> U32Vector2: ...
    @property
    def to(self) -> U32Vector2: ...
    @property
    def tl(self) -> U32Vector2: ...
    @property
    def qs(self) -> U32Vector2: ...
    @property
    def qt(self) -> U32Vector2: ...
    @property
    def qq(self) -> U32Vector2: ...
    @property
    def qp(self) -> U32Vector2: ...
    @property
    def qo(self) -> U32Vector2: ...
    @property
    def ql(self) -> U32Vector2: ...
    @property
    def ps(self) -> U32Vector2: ...
    @property
    def pt(self) -> U32Vector2: ...
    @property
    def pq(self) -> U32Vector2: ...
    @property
    def pp(self) -> U32Vector2: ...
    @property
    def po(self) -> U32Vector2: ...
    @property
    def pl(self) -> U32Vector2: ...
    @property
    def os(self) -> U32Vector2: ...
    @property
    def ot(self) -> U32Vector2: ...
    @property
    def oq(self) -> U32Vector2: ...
    @property
    def op(self) -> U32Vector2: ...
    @property
    def ls(self) -> U32Vector2: ...
    @property
    def lt(self) -> U32Vector2: ...
    @property
    def lq(self) -> U32Vector2: ...
    @property
    def lp(self) -> U32Vector2: ...
    @property
    def sss(self) -> U32Vector3: ...
    @property
    def sst(self) -> U32Vector3: ...
    @property
    def ssq(self) -> U32Vector3: ...
    @property
    def ssp(self) -> U32Vector3: ...
    @property
    def sso(self) -> U32Vector3: ...
    @property
    def ssl(self) -> U32Vector3: ...
    @property
    def sts(self) -> U32Vector3: ...
    @property
    def stt(self) -> U32Vector3: ...
    @property
    def stq(self) -> U32Vector3: ...
    @property
    def stp(self) -> U32Vector3: ...
    @property
    def sto(self) -> U32Vector3: ...
    @property
    def stl(self) -> U32Vector3: ...
    @property
    def sqs(self) -> U32Vector3: ...
    @property
    def sqt(self) -> U32Vector3: ...
    @property
    def sqq(self) -> U32Vector3: ...
    @property
    def sqp(self) -> U32Vector3: ...
    @property
    def sqo(self) -> U32Vector3: ...
    @property
    def sql(self) -> U32Vector3: ...
    @property
    def sps(self) -> U32Vector3: ...
    @property
    def spt(self) -> U32Vector3: ...
    @property
    def spq(self) -> U32Vector3: ...
    @property
    def spp(self) -> U32Vector3: ...
    @property
    def spo(self) -> U32Vector3: ...
    @property
    def spl(self) -> U32Vector3: ...
    @property
    def sos(self) -> U32Vector3: ...
    @property
    def sot(self) -> U32Vector3: ...
    @property
    def soq(self) -> U32Vector3: ...
    @property
    def sop(self) -> U32Vector3: ...
    @property
    def soo(self) -> U32Vector3: ...
    @property
    def sol(self) -> U32Vector3: ...
    @property
    def sls(self) -> U32Vector3: ...
    @property
    def slt(self) -> U32Vector3: ...
    @property
    def slq(self) -> U32Vector3: ...
    @property
    def slp(self) -> U32Vector3: ...
    @property
    def slo(self) -> U32Vector3: ...
    @property
    def sll(self) -> U32Vector3: ...
    @property
    def tss(self) -> U32Vector3: ...
    @property
    def tst(self) -> U32Vector3: ...
    @property
    def tsq(self) -> U32Vector3: ...
    @property
    def tsp(self) -> U32Vector3: ...
    @property
    def tso(self) -> U32Vector3: ...
    @property
    def tsl(self) -> U32Vector3: ...
    @property
    def tts(self) -> U32Vector3: ...
    @property
    def ttt(self) -> U32Vector3: ...
    @property
    def ttq(self) -> U32Vector3: ...
    @property
    def ttp(self) -> U32Vector3: ...
    @property
    def tto(self) -> U32Vector3: ...
    @property
    def ttl(self) -> U32Vector3: ...
    @property
    def tqs(self) -> U32Vector3: ...
    @property
    def tqt(self) -> U32Vector3: ...
    @property
    def tqq(self) -> U32Vector3: ...
    @property
    def tqp(self) -> U32Vector3: ...
    @property
    def tqo(self) -> U32Vector3: ...
    @property
    def tql(self) -> U32Vector3: ...
    @property
    def tps(self) -> U32Vector3: ...
    @property
    def tpt(self) -> U32Vector3: ...
    @property
    def tpq(self) -> U32Vector3: ...
    @property
    def tpp(self) -> U32Vector3: ...
    @property
    def tpo(self) -> U32Vector3: ...
    @property
    def tpl(self) -> U32Vector3: ...
    @property
    def tos(self) -> U32Vector3: ...
    @property
    def tot(self) -> U32Vector3: ...
    @property
    def toq(self) -> U32Vector3: ...
    @property
    def top(self) -> U32Vector3: ...
    @property
    def too(self) -> U32Vector3: ...
    @property
    def tol(self) -> U32Vector3: ...
    @property
    def tls(self) -> U32Vector3: ...
    @property
    def tlt(self) -> U32Vector3: ...
    @property
    def tlq(self) -> U32Vector3: ...
    @property
    def tlp(self) -> U32Vector3: ...
    @property
    def tlo(self) -> U32Vector3: ...
    @property
    def tll(self) -> U32Vector3: ...
    @property
    def qss(self) -> U32Vector3: ...
    @property
    def qst(self) -> U32Vector3: ...
    @property
    def qsq(self) -> U32Vector3: ...
    @property
    def qsp(self) -> U32Vector3: ...
    @property
    def qso(self) -> U32Vector3: ...
    @property
    def qsl(self) -> U32Vector3: ...
    @property
    def qts(self) -> U32Vector3: ...
    @property
    def qtt(self) -> U32Vector3: ...
    @property
    def qtq(self) -> U32Vector3: ...
    @property
    def qtp(self) -> U32Vector3: ...
    @property
    def qto(self) -> U32Vector3: ...
    @property
    def qtl(self) -> U32Vector3: ...
    @property
    def qqs(self) -> U32Vector3: ...
    @property
    def qqt(self) -> U32Vector3: ...
    @property
    def qqq(self) -> U32Vector3: ...
    @property
    def qqp(self) -> U32Vector3: ...
    @property
    def qqo(self) -> U32Vector3: ...
    @property
    def qql(self) -> U32Vector3: ...
    @property
    def qps(self) -> U32Vector3: ...
    @property
    def qpt(self) -> U32Vector3: ...
    @property
    def qpq(self) -> U32Vector3: ...
    @property
    def qpp(self) -> U32Vector3: ...
    @property
    def qpo(self) -> U32Vector3: ...
    @property
    def qpl(self) -> U32Vector3: ...
    @property
    def qos(self) -> U32Vector3: ...
    @property
    def qot(self) -> U32Vector3: ...
    @property
    def qoq(self) -> U32Vector3: ...
    @property
    def qop(self) -> U32Vector3: ...
    @property
    def qoo(self) -> U32Vector3: ...
    @property
    def qol(self) -> U32Vector3: ...
    @property
    def qls(self) -> U32Vector3: ...
    @property
    def qlt(self) -> U32Vector3: ...
    @property
    def qlq(self) -> U32Vector3: ...
    @property
    def qlp(self) -> U32Vector3: ...
    @property
    def qlo(self) -> U32Vector3: ...
    @property
    def qll(self) -> U32Vector3: ...
    @property
    def pss(self) -> U32Vector3: ...
    @property
    def pst(self) -> U32Vector3: ...
    @property
    def psq(self) -> U32Vector3: ...
    @property
    def psp(self) -> U32Vector3: ...
    @property
    def pso(self) -> U32Vector3: ...
    @property
    def psl(self) -> U32Vector3: ...
    @property
    def pts(self) -> U32Vector3: ...
    @property
    def ptt(self) -> U32Vector3: ...
    @property
    def ptq(self) -> U32Vector3: ...
    @property
    def ptp(self) -> U32Vector3: ...
    @property
    def pto(self) -> U32Vector3: ...
    @property
    def ptl(self) -> U32Vector3: ...
    @property
    def pqs(self) -> U32Vector3: ...
    @property
    def pqt(self) -> U32Vector3: ...
    @property
    def pqq(self) -> U32Vector3: ...
    @property
    def pqp(self) -> U32Vector3: ...
    @property
    def pqo(self) -> U32Vector3: ...
    @property
    def pql(self) -> U32Vector3: ...
    @property
    def pps(self) -> U32Vector3: ...
    @property
    def ppt(self) -> U32Vector3: ...
    @property
    def ppq(self) -> U32Vector3: ...
    @property
    def ppp(self) -> U32Vector3: ...
    @property
    def ppo(self) -> U32Vector3: ...
    @property
    def ppl(self) -> U32Vector3: ...
    @property
    def pos(self) -> U32Vector3: ...
    @property
    def pot(self) -> U32Vector3: ...
    @property
    def poq(self) -> U32Vector3: ...
    @property
    def pop(self) -> U32Vector3: ...
    @property
    def poo(self) -> U32Vector3: ...
    @property
    def pol(self) -> U32Vector3: ...
    @property
    def pls(self) -> U32Vector3: ...
    @property
    def plt(self) -> U32Vector3: ...
    @property
    def plq(self) -> U32Vector3: ...
    @property
    def plp(self) -> U32Vector3: ...
    @property
    def plo(self) -> U32Vector3: ...
    @property
    def pll(self) -> U32Vector3: ...
    @property
    def oss(self) -> U32Vector3: ...
    @property
    def ost(self) -> U32Vector3: ...
    @property
    def osq(self) -> U32Vector3: ...
    @property
    def osp(self) -> U32Vector3: ...
    @property
    def oso(self) -> U32Vector3: ...
    @property
    def osl(self) -> U32Vector3: ...
    @property
    def ots(self) -> U32Vector3: ...
    @property
    def ott(self) -> U32Vector3: ...
    @property
    def otq(self) -> U32Vector3: ...
    @property
    def otp(self) -> U32Vector3: ...
    @property
    def oto(self) -> U32Vector3: ...
    @property
    def otl(self) -> U32Vector3: ...
    @property
    def oqs(self) -> U32Vector3: ...
    @property
    def oqt(self) -> U32Vector3: ...
    @property
    def oqq(self) -> U32Vector3: ...
    @property
    def oqp(self) -> U32Vector3: ...
    @property
    def oqo(self) -> U32Vector3: ...
    @property
    def oql(self) -> U32Vector3: ...
    @property
    def ops(self) -> U32Vector3: ...
    @property
    def opt(self) -> U32Vector3: ...
    @property
    def opq(self) -> U32Vector3: ...
    @property
    def opp(self) -> U32Vector3: ...
    @property
    def opo(self) -> U32Vector3: ...
    @property
    def opl(self) -> U32Vector3: ...
    @property
    def oos(self) -> U32Vector3: ...
    @property
    def oot(self) -> U32Vector3: ...
    @property
    def ooq(self) -> U32Vector3: ...
    @property
    def oop(self) -> U32Vector3: ...
    @property
    def ols(self) -> U32Vector3: ...
    @property
    def olt(self) -> U32Vector3: ...
    @property
    def olq(self) -> U32Vector3: ...
    @property
    def olp(self) -> U32Vector3: ...
    @property
    def lss(self) -> U32Vector3: ...
    @property
    def lst(self) -> U32Vector3: ...
    @property
    def lsq(self) -> U32Vector3: ...
    @property
    def lsp(self) -> U32Vector3: ...
    @property
    def lso(self) -> U32Vector3: ...
    @property
    def lsl(self) -> U32Vector3: ...
    @property
    def lts(self) -> U32Vector3: ...
    @property
    def ltt(self) -> U32Vector3: ...
    @property
    def ltq(self) -> U32Vector3: ...
    @property
    def ltp(self) -> U32Vector3: ...
    @property
    def lto(self) -> U32Vector3: ...
    @property
    def ltl(self) -> U32Vector3: ...
    @property
    def lqs(self) -> U32Vector3: ...
    @property
    def lqt(self) -> U32Vector3: ...
    @property
    def lqq(self) -> U32Vector3: ...
    @property
    def lqp(self) -> U32Vector3: ...
    @property
    def lqo(self) -> U32Vector3: ...
    @property
    def lql(self) -> U32Vector3: ...
    @property
    def lps(self) -> U32Vector3: ...
    @property
    def lpt(self) -> U32Vector3: ...
    @property
    def lpq(self) -> U32Vector3: ...
    @property
    def lpp(self) -> U32Vector3: ...
    @property
    def lpo(self) -> U32Vector3: ...
    @property
    def lpl(self) -> U32Vector3: ...
    @property
    def los(self) -> U32Vector3: ...
    @property
    def lot(self) -> U32Vector3: ...
    @property
    def loq(self) -> U32Vector3: ...
    @property
    def lop(self) -> U32Vector3: ...
    @property
    def lls(self) -> U32Vector3: ...
    @property
    def llt(self) -> U32Vector3: ...
    @property
    def llq(self) -> U32Vector3: ...
    @property
    def llp(self) -> U32Vector3: ...
    @property
    def ssss(self) -> U32Vector4: ...
    @property
    def ssst(self) -> U32Vector4: ...
    @property
    def sssq(self) -> U32Vector4: ...
    @property
    def sssp(self) -> U32Vector4: ...
    @property
    def ssso(self) -> U32Vector4: ...
    @property
    def sssl(self) -> U32Vector4: ...
    @property
    def ssts(self) -> U32Vector4: ...
    @property
    def sstt(self) -> U32Vector4: ...
    @property
    def sstq(self) -> U32Vector4: ...
    @property
    def sstp(self) -> U32Vector4: ...
    @property
    def ssto(self) -> U32Vector4: ...
    @property
    def sstl(self) -> U32Vector4: ...
    @property
    def ssqs(self) -> U32Vector4: ...
    @property
    def ssqt(self) -> U32Vector4: ...
    @property
    def ssqq(self) -> U32Vector4: ...
    @property
    def ssqp(self) -> U32Vector4: ...
    @property
    def ssqo(self) -> U32Vector4: ...
    @property
    def ssql(self) -> U32Vector4: ...
    @property
    def ssps(self) -> U32Vector4: ...
    @property
    def sspt(self) -> U32Vector4: ...
    @property
    def sspq(self) -> U32Vector4: ...
    @property
    def sspp(self) -> U32Vector4: ...
    @property
    def sspo(self) -> U32Vector4: ...
    @property
    def sspl(self) -> U32Vector4: ...
    @property
    def ssos(self) -> U32Vector4: ...
    @property
    def ssot(self) -> U32Vector4: ...
    @property
    def ssoq(self) -> U32Vector4: ...
    @property
    def ssop(self) -> U32Vector4: ...
    @property
    def ssoo(self) -> U32Vector4: ...
    @property
    def ssol(self) -> U32Vector4: ...
    @property
    def ssls(self) -> U32Vector4: ...
    @property
    def sslt(self) -> U32Vector4: ...
    @property
    def sslq(self) -> U32Vector4: ...
    @property
    def sslp(self) -> U32Vector4: ...
    @property
    def sslo(self) -> U32Vector4: ...
    @property
    def ssll(self) -> U32Vector4: ...
    @property
    def stss(self) -> U32Vector4: ...
    @property
    def stst(self) -> U32Vector4: ...
    @property
    def stsq(self) -> U32Vector4: ...
    @property
    def stsp(self) -> U32Vector4: ...
    @property
    def stso(self) -> U32Vector4: ...
    @property
    def stsl(self) -> U32Vector4: ...
    @property
    def stts(self) -> U32Vector4: ...
    @property
    def sttt(self) -> U32Vector4: ...
    @property
    def sttq(self) -> U32Vector4: ...
    @property
    def sttp(self) -> U32Vector4: ...
    @property
    def stto(self) -> U32Vector4: ...
    @property
    def sttl(self) -> U32Vector4: ...
    @property
    def stqs(self) -> U32Vector4: ...
    @property
    def stqt(self) -> U32Vector4: ...
    @property
    def stqq(self) -> U32Vector4: ...
    @property
    def stqp(self) -> U32Vector4: ...
    @property
    def stqo(self) -> U32Vector4: ...
    @property
    def stql(self) -> U32Vector4: ...
    @property
    def stps(self) -> U32Vector4: ...
    @property
    def stpt(self) -> U32Vector4: ...
    @property
    def stpq(self) -> U32Vector4: ...
    @property
    def stpp(self) -> U32Vector4: ...
    @property
    def stpo(self) -> U32Vector4: ...
    @property
    def stpl(self) -> U32Vector4: ...
    @property
    def stos(self) -> U32Vector4: ...
    @property
    def stot(self) -> U32Vector4: ...
    @property
    def stoq(self) -> U32Vector4: ...
    @property
    def stop(self) -> U32Vector4: ...
    @property
    def stoo(self) -> U32Vector4: ...
    @property
    def stol(self) -> U32Vector4: ...
    @property
    def stls(self) -> U32Vector4: ...
    @property
    def stlt(self) -> U32Vector4: ...
    @property
    def stlq(self) -> U32Vector4: ...
    @property
    def stlp(self) -> U32Vector4: ...
    @property
    def stlo(self) -> U32Vector4: ...
    @property
    def stll(self) -> U32Vector4: ...
    @property
    def sqss(self) -> U32Vector4: ...
    @property
    def sqst(self) -> U32Vector4: ...
    @property
    def sqsq(self) -> U32Vector4: ...
    @property
    def sqsp(self) -> U32Vector4: ...
    @property
    def sqso(self) -> U32Vector4: ...
    @property
    def sqsl(self) -> U32Vector4: ...
    @property
    def sqts(self) -> U32Vector4: ...
    @property
    def sqtt(self) -> U32Vector4: ...
    @property
    def sqtq(self) -> U32Vector4: ...
    @property
    def sqtp(self) -> U32Vector4: ...
    @property
    def sqto(self) -> U32Vector4: ...
    @property
    def sqtl(self) -> U32Vector4: ...
    @property
    def sqqs(self) -> U32Vector4: ...
    @property
    def sqqt(self) -> U32Vector4: ...
    @property
    def sqqq(self) -> U32Vector4: ...
    @property
    def sqqp(self) -> U32Vector4: ...
    @property
    def sqqo(self) -> U32Vector4: ...
    @property
    def sqql(self) -> U32Vector4: ...
    @property
    def sqps(self) -> U32Vector4: ...
    @property
    def sqpt(self) -> U32Vector4: ...
    @property
    def sqpq(self) -> U32Vector4: ...
    @property
    def sqpp(self) -> U32Vector4: ...
    @property
    def sqpo(self) -> U32Vector4: ...
    @property
    def sqpl(self) -> U32Vector4: ...
    @property
    def sqos(self) -> U32Vector4: ...
    @property
    def sqot(self) -> U32Vector4: ...
    @property
    def sqoq(self) -> U32Vector4: ...
    @property
    def sqop(self) -> U32Vector4: ...
    @property
    def sqoo(self) -> U32Vector4: ...
    @property
    def sqol(self) -> U32Vector4: ...
    @property
    def sqls(self) -> U32Vector4: ...
    @property
    def sqlt(self) -> U32Vector4: ...
    @property
    def sqlq(self) -> U32Vector4: ...
    @property
    def sqlp(self) -> U32Vector4: ...
    @property
    def sqlo(self) -> U32Vector4: ...
    @property
    def sqll(self) -> U32Vector4: ...
    @property
    def spss(self) -> U32Vector4: ...
    @property
    def spst(self) -> U32Vector4: ...
    @property
    def spsq(self) -> U32Vector4: ...
    @property
    def spsp(self) -> U32Vector4: ...
    @property
    def spso(self) -> U32Vector4: ...
    @property
    def spsl(self) -> U32Vector4: ...
    @property
    def spts(self) -> U32Vector4: ...
    @property
    def sptt(self) -> U32Vector4: ...
    @property
    def sptq(self) -> U32Vector4: ...
    @property
    def sptp(self) -> U32Vector4: ...
    @property
    def spto(self) -> U32Vector4: ...
    @property
    def sptl(self) -> U32Vector4: ...
    @property
    def spqs(self) -> U32Vector4: ...
    @property
    def spqt(self) -> U32Vector4: ...
    @property
    def spqq(self) -> U32Vector4: ...
    @property
    def spqp(self) -> U32Vector4: ...
    @property
    def spqo(self) -> U32Vector4: ...
    @property
    def spql(self) -> U32Vector4: ...
    @property
    def spps(self) -> U32Vector4: ...
    @property
    def sppt(self) -> U32Vector4: ...
    @property
    def sppq(self) -> U32Vector4: ...
    @property
    def sppp(self) -> U32Vector4: ...
    @property
    def sppo(self) -> U32Vector4: ...
    @property
    def sppl(self) -> U32Vector4: ...
    @property
    def spos(self) -> U32Vector4: ...
    @property
    def spot(self) -> U32Vector4: ...
    @property
    def spoq(self) -> U32Vector4: ...
    @property
    def spop(self) -> U32Vector4: ...
    @property
    def spoo(self) -> U32Vector4: ...
    @property
    def spol(self) -> U32Vector4: ...
    @property
    def spls(self) -> U32Vector4: ...
    @property
    def splt(self) -> U32Vector4: ...
    @property
    def splq(self) -> U32Vector4: ...
    @property
    def splp(self) -> U32Vector4: ...
    @property
    def splo(self) -> U32Vector4: ...
    @property
    def spll(self) -> U32Vector4: ...
    @property
    def soss(self) -> U32Vector4: ...
    @property
    def sost(self) -> U32Vector4: ...
    @property
    def sosq(self) -> U32Vector4: ...
    @property
    def sosp(self) -> U32Vector4: ...
    @property
    def soso(self) -> U32Vector4: ...
    @property
    def sosl(self) -> U32Vector4: ...
    @property
    def sots(self) -> U32Vector4: ...
    @property
    def sott(self) -> U32Vector4: ...
    @property
    def sotq(self) -> U32Vector4: ...
    @property
    def sotp(self) -> U32Vector4: ...
    @property
    def soto(self) -> U32Vector4: ...
    @property
    def sotl(self) -> U32Vector4: ...
    @property
    def soqs(self) -> U32Vector4: ...
    @property
    def soqt(self) -> U32Vector4: ...
    @property
    def soqq(self) -> U32Vector4: ...
    @property
    def soqp(self) -> U32Vector4: ...
    @property
    def soqo(self) -> U32Vector4: ...
    @property
    def soql(self) -> U32Vector4: ...
    @property
    def sops(self) -> U32Vector4: ...
    @property
    def sopt(self) -> U32Vector4: ...
    @property
    def sopq(self) -> U32Vector4: ...
    @property
    def sopp(self) -> U32Vector4: ...
    @property
    def sopo(self) -> U32Vector4: ...
    @property
    def sopl(self) -> U32Vector4: ...
    @property
    def soos(self) -> U32Vector4: ...
    @property
    def soot(self) -> U32Vector4: ...
    @property
    def sooq(self) -> U32Vector4: ...
    @property
    def soop(self) -> U32Vector4: ...
    @property
    def sooo(self) -> U32Vector4: ...
    @property
    def sool(self) -> U32Vector4: ...
    @property
    def sols(self) -> U32Vector4: ...
    @property
    def solt(self) -> U32Vector4: ...
    @property
    def solq(self) -> U32Vector4: ...
    @property
    def solp(self) -> U32Vector4: ...
    @property
    def solo(self) -> U32Vector4: ...
    @property
    def soll(self) -> U32Vector4: ...
    @property
    def slss(self) -> U32Vector4: ...
    @property
    def slst(self) -> U32Vector4: ...
    @property
    def slsq(self) -> U32Vector4: ...
    @property
    def slsp(self) -> U32Vector4: ...
    @property
    def slso(self) -> U32Vector4: ...
    @property
    def slsl(self) -> U32Vector4: ...
    @property
    def slts(self) -> U32Vector4: ...
    @property
    def sltt(self) -> U32Vector4: ...
    @property
    def sltq(self) -> U32Vector4: ...
    @property
    def sltp(self) -> U32Vector4: ...
    @property
    def slto(self) -> U32Vector4: ...
    @property
    def sltl(self) -> U32Vector4: ...
    @property
    def slqs(self) -> U32Vector4: ...
    @property
    def slqt(self) -> U32Vector4: ...
    @property
    def slqq(self) -> U32Vector4: ...
    @property
    def slqp(self) -> U32Vector4: ...
    @property
    def slqo(self) -> U32Vector4: ...
    @property
    def slql(self) -> U32Vector4: ...
    @property
    def slps(self) -> U32Vector4: ...
    @property
    def slpt(self) -> U32Vector4: ...
    @property
    def slpq(self) -> U32Vector4: ...
    @property
    def slpp(self) -> U32Vector4: ...
    @property
    def slpo(self) -> U32Vector4: ...
    @property
    def slpl(self) -> U32Vector4: ...
    @property
    def slos(self) -> U32Vector4: ...
    @property
    def slot(self) -> U32Vector4: ...
    @property
    def sloq(self) -> U32Vector4: ...
    @property
    def slop(self) -> U32Vector4: ...
    @property
    def sloo(self) -> U32Vector4: ...
    @property
    def slol(self) -> U32Vector4: ...
    @property
    def slls(self) -> U32Vector4: ...
    @property
    def sllt(self) -> U32Vector4: ...
    @property
    def sllq(self) -> U32Vector4: ...
    @property
    def sllp(self) -> U32Vector4: ...
    @property
    def sllo(self) -> U32Vector4: ...
    @property
    def slll(self) -> U32Vector4: ...
    @property
    def tsss(self) -> U32Vector4: ...
    @property
    def tsst(self) -> U32Vector4: ...
    @property
    def tssq(self) -> U32Vector4: ...
    @property
    def tssp(self) -> U32Vector4: ...
    @property
    def tsso(self) -> U32Vector4: ...
    @property
    def tssl(self) -> U32Vector4: ...
    @property
    def tsts(self) -> U32Vector4: ...
    @property
    def tstt(self) -> U32Vector4: ...
    @property
    def tstq(self) -> U32Vector4: ...
    @property
    def tstp(self) -> U32Vector4: ...
    @property
    def tsto(self) -> U32Vector4: ...
    @property
    def tstl(self) -> U32Vector4: ...
    @property
    def tsqs(self) -> U32Vector4: ...
    @property
    def tsqt(self) -> U32Vector4: ...
    @property
    def tsqq(self) -> U32Vector4: ...
    @property
    def tsqp(self) -> U32Vector4: ...
    @property
    def tsqo(self) -> U32Vector4: ...
    @property
    def tsql(self) -> U32Vector4: ...
    @property
    def tsps(self) -> U32Vector4: ...
    @property
    def tspt(self) -> U32Vector4: ...
    @property
    def tspq(self) -> U32Vector4: ...
    @property
    def tspp(self) -> U32Vector4: ...
    @property
    def tspo(self) -> U32Vector4: ...
    @property
    def tspl(self) -> U32Vector4: ...
    @property
    def tsos(self) -> U32Vector4: ...
    @property
    def tsot(self) -> U32Vector4: ...
    @property
    def tsoq(self) -> U32Vector4: ...
    @property
    def tsop(self) -> U32Vector4: ...
    @property
    def tsoo(self) -> U32Vector4: ...
    @property
    def tsol(self) -> U32Vector4: ...
    @property
    def tsls(self) -> U32Vector4: ...
    @property
    def tslt(self) -> U32Vector4: ...
    @property
    def tslq(self) -> U32Vector4: ...
    @property
    def tslp(self) -> U32Vector4: ...
    @property
    def tslo(self) -> U32Vector4: ...
    @property
    def tsll(self) -> U32Vector4: ...
    @property
    def ttss(self) -> U32Vector4: ...
    @property
    def ttst(self) -> U32Vector4: ...
    @property
    def ttsq(self) -> U32Vector4: ...
    @property
    def ttsp(self) -> U32Vector4: ...
    @property
    def ttso(self) -> U32Vector4: ...
    @property
    def ttsl(self) -> U32Vector4: ...
    @property
    def ttts(self) -> U32Vector4: ...
    @property
    def tttt(self) -> U32Vector4: ...
    @property
    def tttq(self) -> U32Vector4: ...
    @property
    def tttp(self) -> U32Vector4: ...
    @property
    def ttto(self) -> U32Vector4: ...
    @property
    def tttl(self) -> U32Vector4: ...
    @property
    def ttqs(self) -> U32Vector4: ...
    @property
    def ttqt(self) -> U32Vector4: ...
    @property
    def ttqq(self) -> U32Vector4: ...
    @property
    def ttqp(self) -> U32Vector4: ...
    @property
    def ttqo(self) -> U32Vector4: ...
    @property
    def ttql(self) -> U32Vector4: ...
    @property
    def ttps(self) -> U32Vector4: ...
    @property
    def ttpt(self) -> U32Vector4: ...
    @property
    def ttpq(self) -> U32Vector4: ...
    @property
    def ttpp(self) -> U32Vector4: ...
    @property
    def ttpo(self) -> U32Vector4: ...
    @property
    def ttpl(self) -> U32Vector4: ...
    @property
    def ttos(self) -> U32Vector4: ...
    @property
    def ttot(self) -> U32Vector4: ...
    @property
    def ttoq(self) -> U32Vector4: ...
    @property
    def ttop(self) -> U32Vector4: ...
    @property
    def ttoo(self) -> U32Vector4: ...
    @property
    def ttol(self) -> U32Vector4: ...
    @property
    def ttls(self) -> U32Vector4: ...
    @property
    def ttlt(self) -> U32Vector4: ...
    @property
    def ttlq(self) -> U32Vector4: ...
    @property
    def ttlp(self) -> U32Vector4: ...
    @property
    def ttlo(self) -> U32Vector4: ...
    @property
    def ttll(self) -> U32Vector4: ...
    @property
    def tqss(self) -> U32Vector4: ...
    @property
    def tqst(self) -> U32Vector4: ...
    @property
    def tqsq(self) -> U32Vector4: ...
    @property
    def tqsp(self) -> U32Vector4: ...
    @property
    def tqso(self) -> U32Vector4: ...
    @property
    def tqsl(self) -> U32Vector4: ...
    @property
    def tqts(self) -> U32Vector4: ...
    @property
    def tqtt(self) -> U32Vector4: ...
    @property
    def tqtq(self) -> U32Vector4: ...
    @property
    def tqtp(self) -> U32Vector4: ...
    @property
    def tqto(self) -> U32Vector4: ...
    @property
    def tqtl(self) -> U32Vector4: ...
    @property
    def tqqs(self) -> U32Vector4: ...
    @property
    def tqqt(self) -> U32Vector4: ...
    @property
    def tqqq(self) -> U32Vector4: ...
    @property
    def tqqp(self) -> U32Vector4: ...
    @property
    def tqqo(self) -> U32Vector4: ...
    @property
    def tqql(self) -> U32Vector4: ...
    @property
    def tqps(self) -> U32Vector4: ...
    @property
    def tqpt(self) -> U32Vector4: ...
    @property
    def tqpq(self) -> U32Vector4: ...
    @property
    def tqpp(self) -> U32Vector4: ...
    @property
    def tqpo(self) -> U32Vector4: ...
    @property
    def tqpl(self) -> U32Vector4: ...
    @property
    def tqos(self) -> U32Vector4: ...
    @property
    def tqot(self) -> U32Vector4: ...
    @property
    def tqoq(self) -> U32Vector4: ...
    @property
    def tqop(self) -> U32Vector4: ...
    @property
    def tqoo(self) -> U32Vector4: ...
    @property
    def tqol(self) -> U32Vector4: ...
    @property
    def tqls(self) -> U32Vector4: ...
    @property
    def tqlt(self) -> U32Vector4: ...
    @property
    def tqlq(self) -> U32Vector4: ...
    @property
    def tqlp(self) -> U32Vector4: ...
    @property
    def tqlo(self) -> U32Vector4: ...
    @property
    def tqll(self) -> U32Vector4: ...
    @property
    def tpss(self) -> U32Vector4: ...
    @property
    def tpst(self) -> U32Vector4: ...
    @property
    def tpsq(self) -> U32Vector4: ...
    @property
    def tpsp(self) -> U32Vector4: ...
    @property
    def tpso(self) -> U32Vector4: ...
    @property
    def tpsl(self) -> U32Vector4: ...
    @property
    def tpts(self) -> U32Vector4: ...
    @property
    def tptt(self) -> U32Vector4: ...
    @property
    def tptq(self) -> U32Vector4: ...
    @property
    def tptp(self) -> U32Vector4: ...
    @property
    def tpto(self) -> U32Vector4: ...
    @property
    def tptl(self) -> U32Vector4: ...
    @property
    def tpqs(self) -> U32Vector4: ...
    @property
    def tpqt(self) -> U32Vector4: ...
    @property
    def tpqq(self) -> U32Vector4: ...
    @property
    def tpqp(self) -> U32Vector4: ...
    @property
    def tpqo(self) -> U32Vector4: ...
    @property
    def tpql(self) -> U32Vector4: ...
    @property
    def tpps(self) -> U32Vector4: ...
    @property
    def tppt(self) -> U32Vector4: ...
    @property
    def tppq(self) -> U32Vector4: ...
    @property
    def tppp(self) -> U32Vector4: ...
    @property
    def tppo(self) -> U32Vector4: ...
    @property
    def tppl(self) -> U32Vector4: ...
    @property
    def tpos(self) -> U32Vector4: ...
    @property
    def tpot(self) -> U32Vector4: ...
    @property
    def tpoq(self) -> U32Vector4: ...
    @property
    def tpop(self) -> U32Vector4: ...
    @property
    def tpoo(self) -> U32Vector4: ...
    @property
    def tpol(self) -> U32Vector4: ...
    @property
    def tpls(self) -> U32Vector4: ...
    @property
    def tplt(self) -> U32Vector4: ...
    @property
    def tplq(self) -> U32Vector4: ...
    @property
    def tplp(self) -> U32Vector4: ...
    @property
    def tplo(self) -> U32Vector4: ...
    @property
    def tpll(self) -> U32Vector4: ...
    @property
    def toss(self) -> U32Vector4: ...
    @property
    def tost(self) -> U32Vector4: ...
    @property
    def tosq(self) -> U32Vector4: ...
    @property
    def tosp(self) -> U32Vector4: ...
    @property
    def toso(self) -> U32Vector4: ...
    @property
    def tosl(self) -> U32Vector4: ...
    @property
    def tots(self) -> U32Vector4: ...
    @property
    def tott(self) -> U32Vector4: ...
    @property
    def totq(self) -> U32Vector4: ...
    @property
    def totp(self) -> U32Vector4: ...
    @property
    def toto(self) -> U32Vector4: ...
    @property
    def totl(self) -> U32Vector4: ...
    @property
    def toqs(self) -> U32Vector4: ...
    @property
    def toqt(self) -> U32Vector4: ...
    @property
    def toqq(self) -> U32Vector4: ...
    @property
    def toqp(self) -> U32Vector4: ...
    @property
    def toqo(self) -> U32Vector4: ...
    @property
    def toql(self) -> U32Vector4: ...
    @property
    def tops(self) -> U32Vector4: ...
    @property
    def topt(self) -> U32Vector4: ...
    @property
    def topq(self) -> U32Vector4: ...
    @property
    def topp(self) -> U32Vector4: ...
    @property
    def topo(self) -> U32Vector4: ...
    @property
    def topl(self) -> U32Vector4: ...
    @property
    def toos(self) -> U32Vector4: ...
    @property
    def toot(self) -> U32Vector4: ...
    @property
    def tooq(self) -> U32Vector4: ...
    @property
    def toop(self) -> U32Vector4: ...
    @property
    def tooo(self) -> U32Vector4: ...
    @property
    def tool(self) -> U32Vector4: ...
    @property
    def tols(self) -> U32Vector4: ...
    @property
    def tolt(self) -> U32Vector4: ...
    @property
    def tolq(self) -> U32Vector4: ...
    @property
    def tolp(self) -> U32Vector4: ...
    @property
    def tolo(self) -> U32Vector4: ...
    @property
    def toll(self) -> U32Vector4: ...
    @property
    def tlss(self) -> U32Vector4: ...
    @property
    def tlst(self) -> U32Vector4: ...
    @property
    def tlsq(self) -> U32Vector4: ...
    @property
    def tlsp(self) -> U32Vector4: ...
    @property
    def tlso(self) -> U32Vector4: ...
    @property
    def tlsl(self) -> U32Vector4: ...
    @property
    def tlts(self) -> U32Vector4: ...
    @property
    def tltt(self) -> U32Vector4: ...
    @property
    def tltq(self) -> U32Vector4: ...
    @property
    def tltp(self) -> U32Vector4: ...
    @property
    def tlto(self) -> U32Vector4: ...
    @property
    def tltl(self) -> U32Vector4: ...
    @property
    def tlqs(self) -> U32Vector4: ...
    @property
    def tlqt(self) -> U32Vector4: ...
    @property
    def tlqq(self) -> U32Vector4: ...
    @property
    def tlqp(self) -> U32Vector4: ...
    @property
    def tlqo(self) -> U32Vector4: ...
    @property
    def tlql(self) -> U32Vector4: ...
    @property
    def tlps(self) -> U32Vector4: ...
    @property
    def tlpt(self) -> U32Vector4: ...
    @property
    def tlpq(self) -> U32Vector4: ...
    @property
    def tlpp(self) -> U32Vector4: ...
    @property
    def tlpo(self) -> U32Vector4: ...
    @property
    def tlpl(self) -> U32Vector4: ...
    @property
    def tlos(self) -> U32Vector4: ...
    @property
    def tlot(self) -> U32Vector4: ...
    @property
    def tloq(self) -> U32Vector4: ...
    @property
    def tlop(self) -> U32Vector4: ...
    @property
    def tloo(self) -> U32Vector4: ...
    @property
    def tlol(self) -> U32Vector4: ...
    @property
    def tlls(self) -> U32Vector4: ...
    @property
    def tllt(self) -> U32Vector4: ...
    @property
    def tllq(self) -> U32Vector4: ...
    @property
    def tllp(self) -> U32Vector4: ...
    @property
    def tllo(self) -> U32Vector4: ...
    @property
    def tlll(self) -> U32Vector4: ...
    @property
    def qsss(self) -> U32Vector4: ...
    @property
    def qsst(self) -> U32Vector4: ...
    @property
    def qssq(self) -> U32Vector4: ...
    @property
    def qssp(self) -> U32Vector4: ...
    @property
    def qsso(self) -> U32Vector4: ...
    @property
    def qssl(self) -> U32Vector4: ...
    @property
    def qsts(self) -> U32Vector4: ...
    @property
    def qstt(self) -> U32Vector4: ...
    @property
    def qstq(self) -> U32Vector4: ...
    @property
    def qstp(self) -> U32Vector4: ...
    @property
    def qsto(self) -> U32Vector4: ...
    @property
    def qstl(self) -> U32Vector4: ...
    @property
    def qsqs(self) -> U32Vector4: ...
    @property
    def qsqt(self) -> U32Vector4: ...
    @property
    def qsqq(self) -> U32Vector4: ...
    @property
    def qsqp(self) -> U32Vector4: ...
    @property
    def qsqo(self) -> U32Vector4: ...
    @property
    def qsql(self) -> U32Vector4: ...
    @property
    def qsps(self) -> U32Vector4: ...
    @property
    def qspt(self) -> U32Vector4: ...
    @property
    def qspq(self) -> U32Vector4: ...
    @property
    def qspp(self) -> U32Vector4: ...
    @property
    def qspo(self) -> U32Vector4: ...
    @property
    def qspl(self) -> U32Vector4: ...
    @property
    def qsos(self) -> U32Vector4: ...
    @property
    def qsot(self) -> U32Vector4: ...
    @property
    def qsoq(self) -> U32Vector4: ...
    @property
    def qsop(self) -> U32Vector4: ...
    @property
    def qsoo(self) -> U32Vector4: ...
    @property
    def qsol(self) -> U32Vector4: ...
    @property
    def qsls(self) -> U32Vector4: ...
    @property
    def qslt(self) -> U32Vector4: ...
    @property
    def qslq(self) -> U32Vector4: ...
    @property
    def qslp(self) -> U32Vector4: ...
    @property
    def qslo(self) -> U32Vector4: ...
    @property
    def qsll(self) -> U32Vector4: ...
    @property
    def qtss(self) -> U32Vector4: ...
    @property
    def qtst(self) -> U32Vector4: ...
    @property
    def qtsq(self) -> U32Vector4: ...
    @property
    def qtsp(self) -> U32Vector4: ...
    @property
    def qtso(self) -> U32Vector4: ...
    @property
    def qtsl(self) -> U32Vector4: ...
    @property
    def qtts(self) -> U32Vector4: ...
    @property
    def qttt(self) -> U32Vector4: ...
    @property
    def qttq(self) -> U32Vector4: ...
    @property
    def qttp(self) -> U32Vector4: ...
    @property
    def qtto(self) -> U32Vector4: ...
    @property
    def qttl(self) -> U32Vector4: ...
    @property
    def qtqs(self) -> U32Vector4: ...
    @property
    def qtqt(self) -> U32Vector4: ...
    @property
    def qtqq(self) -> U32Vector4: ...
    @property
    def qtqp(self) -> U32Vector4: ...
    @property
    def qtqo(self) -> U32Vector4: ...
    @property
    def qtql(self) -> U32Vector4: ...
    @property
    def qtps(self) -> U32Vector4: ...
    @property
    def qtpt(self) -> U32Vector4: ...
    @property
    def qtpq(self) -> U32Vector4: ...
    @property
    def qtpp(self) -> U32Vector4: ...
    @property
    def qtpo(self) -> U32Vector4: ...
    @property
    def qtpl(self) -> U32Vector4: ...
    @property
    def qtos(self) -> U32Vector4: ...
    @property
    def qtot(self) -> U32Vector4: ...
    @property
    def qtoq(self) -> U32Vector4: ...
    @property
    def qtop(self) -> U32Vector4: ...
    @property
    def qtoo(self) -> U32Vector4: ...
    @property
    def qtol(self) -> U32Vector4: ...
    @property
    def qtls(self) -> U32Vector4: ...
    @property
    def qtlt(self) -> U32Vector4: ...
    @property
    def qtlq(self) -> U32Vector4: ...
    @property
    def qtlp(self) -> U32Vector4: ...
    @property
    def qtlo(self) -> U32Vector4: ...
    @property
    def qtll(self) -> U32Vector4: ...
    @property
    def qqss(self) -> U32Vector4: ...
    @property
    def qqst(self) -> U32Vector4: ...
    @property
    def qqsq(self) -> U32Vector4: ...
    @property
    def qqsp(self) -> U32Vector4: ...
    @property
    def qqso(self) -> U32Vector4: ...
    @property
    def qqsl(self) -> U32Vector4: ...
    @property
    def qqts(self) -> U32Vector4: ...
    @property
    def qqtt(self) -> U32Vector4: ...
    @property
    def qqtq(self) -> U32Vector4: ...
    @property
    def qqtp(self) -> U32Vector4: ...
    @property
    def qqto(self) -> U32Vector4: ...
    @property
    def qqtl(self) -> U32Vector4: ...
    @property
    def qqqs(self) -> U32Vector4: ...
    @property
    def qqqt(self) -> U32Vector4: ...
    @property
    def qqqq(self) -> U32Vector4: ...
    @property
    def qqqp(self) -> U32Vector4: ...
    @property
    def qqqo(self) -> U32Vector4: ...
    @property
    def qqql(self) -> U32Vector4: ...
    @property
    def qqps(self) -> U32Vector4: ...
    @property
    def qqpt(self) -> U32Vector4: ...
    @property
    def qqpq(self) -> U32Vector4: ...
    @property
    def qqpp(self) -> U32Vector4: ...
    @property
    def qqpo(self) -> U32Vector4: ...
    @property
    def qqpl(self) -> U32Vector4: ...
    @property
    def qqos(self) -> U32Vector4: ...
    @property
    def qqot(self) -> U32Vector4: ...
    @property
    def qqoq(self) -> U32Vector4: ...
    @property
    def qqop(self) -> U32Vector4: ...
    @property
    def qqoo(self) -> U32Vector4: ...
    @property
    def qqol(self) -> U32Vector4: ...
    @property
    def qqls(self) -> U32Vector4: ...
    @property
    def qqlt(self) -> U32Vector4: ...
    @property
    def qqlq(self) -> U32Vector4: ...
    @property
    def qqlp(self) -> U32Vector4: ...
    @property
    def qqlo(self) -> U32Vector4: ...
    @property
    def qqll(self) -> U32Vector4: ...
    @property
    def qpss(self) -> U32Vector4: ...
    @property
    def qpst(self) -> U32Vector4: ...
    @property
    def qpsq(self) -> U32Vector4: ...
    @property
    def qpsp(self) -> U32Vector4: ...
    @property
    def qpso(self) -> U32Vector4: ...
    @property
    def qpsl(self) -> U32Vector4: ...
    @property
    def qpts(self) -> U32Vector4: ...
    @property
    def qptt(self) -> U32Vector4: ...
    @property
    def qptq(self) -> U32Vector4: ...
    @property
    def qptp(self) -> U32Vector4: ...
    @property
    def qpto(self) -> U32Vector4: ...
    @property
    def qptl(self) -> U32Vector4: ...
    @property
    def qpqs(self) -> U32Vector4: ...
    @property
    def qpqt(self) -> U32Vector4: ...
    @property
    def qpqq(self) -> U32Vector4: ...
    @property
    def qpqp(self) -> U32Vector4: ...
    @property
    def qpqo(self) -> U32Vector4: ...
    @property
    def qpql(self) -> U32Vector4: ...
    @property
    def qpps(self) -> U32Vector4: ...
    @property
    def qppt(self) -> U32Vector4: ...
    @property
    def qppq(self) -> U32Vector4: ...
    @property
    def qppp(self) -> U32Vector4: ...
    @property
    def qppo(self) -> U32Vector4: ...
    @property
    def qppl(self) -> U32Vector4: ...
    @property
    def qpos(self) -> U32Vector4: ...
    @property
    def qpot(self) -> U32Vector4: ...
    @property
    def qpoq(self) -> U32Vector4: ...
    @property
    def qpop(self) -> U32Vector4: ...
    @property
    def qpoo(self) -> U32Vector4: ...
    @property
    def qpol(self) -> U32Vector4: ...
    @property
    def qpls(self) -> U32Vector4: ...
    @property
    def qplt(self) -> U32Vector4: ...
    @property
    def qplq(self) -> U32Vector4: ...
    @property
    def qplp(self) -> U32Vector4: ...
    @property
    def qplo(self) -> U32Vector4: ...
    @property
    def qpll(self) -> U32Vector4: ...
    @property
    def qoss(self) -> U32Vector4: ...
    @property
    def qost(self) -> U32Vector4: ...
    @property
    def qosq(self) -> U32Vector4: ...
    @property
    def qosp(self) -> U32Vector4: ...
    @property
    def qoso(self) -> U32Vector4: ...
    @property
    def qosl(self) -> U32Vector4: ...
    @property
    def qots(self) -> U32Vector4: ...
    @property
    def qott(self) -> U32Vector4: ...
    @property
    def qotq(self) -> U32Vector4: ...
    @property
    def qotp(self) -> U32Vector4: ...
    @property
    def qoto(self) -> U32Vector4: ...
    @property
    def qotl(self) -> U32Vector4: ...
    @property
    def qoqs(self) -> U32Vector4: ...
    @property
    def qoqt(self) -> U32Vector4: ...
    @property
    def qoqq(self) -> U32Vector4: ...
    @property
    def qoqp(self) -> U32Vector4: ...
    @property
    def qoqo(self) -> U32Vector4: ...
    @property
    def qoql(self) -> U32Vector4: ...
    @property
    def qops(self) -> U32Vector4: ...
    @property
    def qopt(self) -> U32Vector4: ...
    @property
    def qopq(self) -> U32Vector4: ...
    @property
    def qopp(self) -> U32Vector4: ...
    @property
    def qopo(self) -> U32Vector4: ...
    @property
    def qopl(self) -> U32Vector4: ...
    @property
    def qoos(self) -> U32Vector4: ...
    @property
    def qoot(self) -> U32Vector4: ...
    @property
    def qooq(self) -> U32Vector4: ...
    @property
    def qoop(self) -> U32Vector4: ...
    @property
    def qooo(self) -> U32Vector4: ...
    @property
    def qool(self) -> U32Vector4: ...
    @property
    def qols(self) -> U32Vector4: ...
    @property
    def qolt(self) -> U32Vector4: ...
    @property
    def qolq(self) -> U32Vector4: ...
    @property
    def qolp(self) -> U32Vector4: ...
    @property
    def qolo(self) -> U32Vector4: ...
    @property
    def qoll(self) -> U32Vector4: ...
    @property
    def qlss(self) -> U32Vector4: ...
    @property
    def qlst(self) -> U32Vector4: ...
    @property
    def qlsq(self) -> U32Vector4: ...
    @property
    def qlsp(self) -> U32Vector4: ...
    @property
    def qlso(self) -> U32Vector4: ...
    @property
    def qlsl(self) -> U32Vector4: ...
    @property
    def qlts(self) -> U32Vector4: ...
    @property
    def qltt(self) -> U32Vector4: ...
    @property
    def qltq(self) -> U32Vector4: ...
    @property
    def qltp(self) -> U32Vector4: ...
    @property
    def qlto(self) -> U32Vector4: ...
    @property
    def qltl(self) -> U32Vector4: ...
    @property
    def qlqs(self) -> U32Vector4: ...
    @property
    def qlqt(self) -> U32Vector4: ...
    @property
    def qlqq(self) -> U32Vector4: ...
    @property
    def qlqp(self) -> U32Vector4: ...
    @property
    def qlqo(self) -> U32Vector4: ...
    @property
    def qlql(self) -> U32Vector4: ...
    @property
    def qlps(self) -> U32Vector4: ...
    @property
    def qlpt(self) -> U32Vector4: ...
    @property
    def qlpq(self) -> U32Vector4: ...
    @property
    def qlpp(self) -> U32Vector4: ...
    @property
    def qlpo(self) -> U32Vector4: ...
    @property
    def qlpl(self) -> U32Vector4: ...
    @property
    def qlos(self) -> U32Vector4: ...
    @property
    def qlot(self) -> U32Vector4: ...
    @property
    def qloq(self) -> U32Vector4: ...
    @property
    def qlop(self) -> U32Vector4: ...
    @property
    def qloo(self) -> U32Vector4: ...
    @property
    def qlol(self) -> U32Vector4: ...
    @property
    def qlls(self) -> U32Vector4: ...
    @property
    def qllt(self) -> U32Vector4: ...
    @property
    def qllq(self) -> U32Vector4: ...
    @property
    def qllp(self) -> U32Vector4: ...
    @property
    def qllo(self) -> U32Vector4: ...
    @property
    def qlll(self) -> U32Vector4: ...
    @property
    def psss(self) -> U32Vector4: ...
    @property
    def psst(self) -> U32Vector4: ...
    @property
    def pssq(self) -> U32Vector4: ...
    @property
    def pssp(self) -> U32Vector4: ...
    @property
    def psso(self) -> U32Vector4: ...
    @property
    def pssl(self) -> U32Vector4: ...
    @property
    def psts(self) -> U32Vector4: ...
    @property
    def pstt(self) -> U32Vector4: ...
    @property
    def pstq(self) -> U32Vector4: ...
    @property
    def pstp(self) -> U32Vector4: ...
    @property
    def psto(self) -> U32Vector4: ...
    @property
    def pstl(self) -> U32Vector4: ...
    @property
    def psqs(self) -> U32Vector4: ...
    @property
    def psqt(self) -> U32Vector4: ...
    @property
    def psqq(self) -> U32Vector4: ...
    @property
    def psqp(self) -> U32Vector4: ...
    @property
    def psqo(self) -> U32Vector4: ...
    @property
    def psql(self) -> U32Vector4: ...
    @property
    def psps(self) -> U32Vector4: ...
    @property
    def pspt(self) -> U32Vector4: ...
    @property
    def pspq(self) -> U32Vector4: ...
    @property
    def pspp(self) -> U32Vector4: ...
    @property
    def pspo(self) -> U32Vector4: ...
    @property
    def pspl(self) -> U32Vector4: ...
    @property
    def psos(self) -> U32Vector4: ...
    @property
    def psot(self) -> U32Vector4: ...
    @property
    def psoq(self) -> U32Vector4: ...
    @property
    def psop(self) -> U32Vector4: ...
    @property
    def psoo(self) -> U32Vector4: ...
    @property
    def psol(self) -> U32Vector4: ...
    @property
    def psls(self) -> U32Vector4: ...
    @property
    def pslt(self) -> U32Vector4: ...
    @property
    def pslq(self) -> U32Vector4: ...
    @property
    def pslp(self) -> U32Vector4: ...
    @property
    def pslo(self) -> U32Vector4: ...
    @property
    def psll(self) -> U32Vector4: ...
    @property
    def ptss(self) -> U32Vector4: ...
    @property
    def ptst(self) -> U32Vector4: ...
    @property
    def ptsq(self) -> U32Vector4: ...
    @property
    def ptsp(self) -> U32Vector4: ...
    @property
    def ptso(self) -> U32Vector4: ...
    @property
    def ptsl(self) -> U32Vector4: ...
    @property
    def ptts(self) -> U32Vector4: ...
    @property
    def pttt(self) -> U32Vector4: ...
    @property
    def pttq(self) -> U32Vector4: ...
    @property
    def pttp(self) -> U32Vector4: ...
    @property
    def ptto(self) -> U32Vector4: ...
    @property
    def pttl(self) -> U32Vector4: ...
    @property
    def ptqs(self) -> U32Vector4: ...
    @property
    def ptqt(self) -> U32Vector4: ...
    @property
    def ptqq(self) -> U32Vector4: ...
    @property
    def ptqp(self) -> U32Vector4: ...
    @property
    def ptqo(self) -> U32Vector4: ...
    @property
    def ptql(self) -> U32Vector4: ...
    @property
    def ptps(self) -> U32Vector4: ...
    @property
    def ptpt(self) -> U32Vector4: ...
    @property
    def ptpq(self) -> U32Vector4: ...
    @property
    def ptpp(self) -> U32Vector4: ...
    @property
    def ptpo(self) -> U32Vector4: ...
    @property
    def ptpl(self) -> U32Vector4: ...
    @property
    def ptos(self) -> U32Vector4: ...
    @property
    def ptot(self) -> U32Vector4: ...
    @property
    def ptoq(self) -> U32Vector4: ...
    @property
    def ptop(self) -> U32Vector4: ...
    @property
    def ptoo(self) -> U32Vector4: ...
    @property
    def ptol(self) -> U32Vector4: ...
    @property
    def ptls(self) -> U32Vector4: ...
    @property
    def ptlt(self) -> U32Vector4: ...
    @property
    def ptlq(self) -> U32Vector4: ...
    @property
    def ptlp(self) -> U32Vector4: ...
    @property
    def ptlo(self) -> U32Vector4: ...
    @property
    def ptll(self) -> U32Vector4: ...
    @property
    def pqss(self) -> U32Vector4: ...
    @property
    def pqst(self) -> U32Vector4: ...
    @property
    def pqsq(self) -> U32Vector4: ...
    @property
    def pqsp(self) -> U32Vector4: ...
    @property
    def pqso(self) -> U32Vector4: ...
    @property
    def pqsl(self) -> U32Vector4: ...
    @property
    def pqts(self) -> U32Vector4: ...
    @property
    def pqtt(self) -> U32Vector4: ...
    @property
    def pqtq(self) -> U32Vector4: ...
    @property
    def pqtp(self) -> U32Vector4: ...
    @property
    def pqto(self) -> U32Vector4: ...
    @property
    def pqtl(self) -> U32Vector4: ...
    @property
    def pqqs(self) -> U32Vector4: ...
    @property
    def pqqt(self) -> U32Vector4: ...
    @property
    def pqqq(self) -> U32Vector4: ...
    @property
    def pqqp(self) -> U32Vector4: ...
    @property
    def pqqo(self) -> U32Vector4: ...
    @property
    def pqql(self) -> U32Vector4: ...
    @property
    def pqps(self) -> U32Vector4: ...
    @property
    def pqpt(self) -> U32Vector4: ...
    @property
    def pqpq(self) -> U32Vector4: ...
    @property
    def pqpp(self) -> U32Vector4: ...
    @property
    def pqpo(self) -> U32Vector4: ...
    @property
    def pqpl(self) -> U32Vector4: ...
    @property
    def pqos(self) -> U32Vector4: ...
    @property
    def pqot(self) -> U32Vector4: ...
    @property
    def pqoq(self) -> U32Vector4: ...
    @property
    def pqop(self) -> U32Vector4: ...
    @property
    def pqoo(self) -> U32Vector4: ...
    @property
    def pqol(self) -> U32Vector4: ...
    @property
    def pqls(self) -> U32Vector4: ...
    @property
    def pqlt(self) -> U32Vector4: ...
    @property
    def pqlq(self) -> U32Vector4: ...
    @property
    def pqlp(self) -> U32Vector4: ...
    @property
    def pqlo(self) -> U32Vector4: ...
    @property
    def pqll(self) -> U32Vector4: ...
    @property
    def ppss(self) -> U32Vector4: ...
    @property
    def ppst(self) -> U32Vector4: ...
    @property
    def ppsq(self) -> U32Vector4: ...
    @property
    def ppsp(self) -> U32Vector4: ...
    @property
    def ppso(self) -> U32Vector4: ...
    @property
    def ppsl(self) -> U32Vector4: ...
    @property
    def ppts(self) -> U32Vector4: ...
    @property
    def pptt(self) -> U32Vector4: ...
    @property
    def pptq(self) -> U32Vector4: ...
    @property
    def pptp(self) -> U32Vector4: ...
    @property
    def ppto(self) -> U32Vector4: ...
    @property
    def pptl(self) -> U32Vector4: ...
    @property
    def ppqs(self) -> U32Vector4: ...
    @property
    def ppqt(self) -> U32Vector4: ...
    @property
    def ppqq(self) -> U32Vector4: ...
    @property
    def ppqp(self) -> U32Vector4: ...
    @property
    def ppqo(self) -> U32Vector4: ...
    @property
    def ppql(self) -> U32Vector4: ...
    @property
    def ppps(self) -> U32Vector4: ...
    @property
    def pppt(self) -> U32Vector4: ...
    @property
    def pppq(self) -> U32Vector4: ...
    @property
    def pppp(self) -> U32Vector4: ...
    @property
    def pppo(self) -> U32Vector4: ...
    @property
    def pppl(self) -> U32Vector4: ...
    @property
    def ppos(self) -> U32Vector4: ...
    @property
    def ppot(self) -> U32Vector4: ...
    @property
    def ppoq(self) -> U32Vector4: ...
    @property
    def ppop(self) -> U32Vector4: ...
    @property
    def ppoo(self) -> U32Vector4: ...
    @property
    def ppol(self) -> U32Vector4: ...
    @property
    def ppls(self) -> U32Vector4: ...
    @property
    def pplt(self) -> U32Vector4: ...
    @property
    def pplq(self) -> U32Vector4: ...
    @property
    def pplp(self) -> U32Vector4: ...
    @property
    def pplo(self) -> U32Vector4: ...
    @property
    def ppll(self) -> U32Vector4: ...
    @property
    def poss(self) -> U32Vector4: ...
    @property
    def post(self) -> U32Vector4: ...
    @property
    def posq(self) -> U32Vector4: ...
    @property
    def posp(self) -> U32Vector4: ...
    @property
    def poso(self) -> U32Vector4: ...
    @property
    def posl(self) -> U32Vector4: ...
    @property
    def pots(self) -> U32Vector4: ...
    @property
    def pott(self) -> U32Vector4: ...
    @property
    def potq(self) -> U32Vector4: ...
    @property
    def potp(self) -> U32Vector4: ...
    @property
    def poto(self) -> U32Vector4: ...
    @property
    def potl(self) -> U32Vector4: ...
    @property
    def poqs(self) -> U32Vector4: ...
    @property
    def poqt(self) -> U32Vector4: ...
    @property
    def poqq(self) -> U32Vector4: ...
    @property
    def poqp(self) -> U32Vector4: ...
    @property
    def poqo(self) -> U32Vector4: ...
    @property
    def poql(self) -> U32Vector4: ...
    @property
    def pops(self) -> U32Vector4: ...
    @property
    def popt(self) -> U32Vector4: ...
    @property
    def popq(self) -> U32Vector4: ...
    @property
    def popp(self) -> U32Vector4: ...
    @property
    def popo(self) -> U32Vector4: ...
    @property
    def popl(self) -> U32Vector4: ...
    @property
    def poos(self) -> U32Vector4: ...
    @property
    def poot(self) -> U32Vector4: ...
    @property
    def pooq(self) -> U32Vector4: ...
    @property
    def poop(self) -> U32Vector4: ...
    @property
    def pooo(self) -> U32Vector4: ...
    @property
    def pool(self) -> U32Vector4: ...
    @property
    def pols(self) -> U32Vector4: ...
    @property
    def polt(self) -> U32Vector4: ...
    @property
    def polq(self) -> U32Vector4: ...
    @property
    def polp(self) -> U32Vector4: ...
    @property
    def polo(self) -> U32Vector4: ...
    @property
    def poll(self) -> U32Vector4: ...
    @property
    def plss(self) -> U32Vector4: ...
    @property
    def plst(self) -> U32Vector4: ...
    @property
    def plsq(self) -> U32Vector4: ...
    @property
    def plsp(self) -> U32Vector4: ...
    @property
    def plso(self) -> U32Vector4: ...
    @property
    def plsl(self) -> U32Vector4: ...
    @property
    def plts(self) -> U32Vector4: ...
    @property
    def pltt(self) -> U32Vector4: ...
    @property
    def pltq(self) -> U32Vector4: ...
    @property
    def pltp(self) -> U32Vector4: ...
    @property
    def plto(self) -> U32Vector4: ...
    @property
    def pltl(self) -> U32Vector4: ...
    @property
    def plqs(self) -> U32Vector4: ...
    @property
    def plqt(self) -> U32Vector4: ...
    @property
    def plqq(self) -> U32Vector4: ...
    @property
    def plqp(self) -> U32Vector4: ...
    @property
    def plqo(self) -> U32Vector4: ...
    @property
    def plql(self) -> U32Vector4: ...
    @property
    def plps(self) -> U32Vector4: ...
    @property
    def plpt(self) -> U32Vector4: ...
    @property
    def plpq(self) -> U32Vector4: ...
    @property
    def plpp(self) -> U32Vector4: ...
    @property
    def plpo(self) -> U32Vector4: ...
    @property
    def plpl(self) -> U32Vector4: ...
    @property
    def plos(self) -> U32Vector4: ...
    @property
    def plot(self) -> U32Vector4: ...
    @property
    def ploq(self) -> U32Vector4: ...
    @property
    def plop(self) -> U32Vector4: ...
    @property
    def ploo(self) -> U32Vector4: ...
    @property
    def plol(self) -> U32Vector4: ...
    @property
    def plls(self) -> U32Vector4: ...
    @property
    def pllt(self) -> U32Vector4: ...
    @property
    def pllq(self) -> U32Vector4: ...
    @property
    def pllp(self) -> U32Vector4: ...
    @property
    def pllo(self) -> U32Vector4: ...
    @property
    def plll(self) -> U32Vector4: ...
    @property
    def osss(self) -> U32Vector4: ...
    @property
    def osst(self) -> U32Vector4: ...
    @property
    def ossq(self) -> U32Vector4: ...
    @property
    def ossp(self) -> U32Vector4: ...
    @property
    def osso(self) -> U32Vector4: ...
    @property
    def ossl(self) -> U32Vector4: ...
    @property
    def osts(self) -> U32Vector4: ...
    @property
    def ostt(self) -> U32Vector4: ...
    @property
    def ostq(self) -> U32Vector4: ...
    @property
    def ostp(self) -> U32Vector4: ...
    @property
    def osto(self) -> U32Vector4: ...
    @property
    def ostl(self) -> U32Vector4: ...
    @property
    def osqs(self) -> U32Vector4: ...
    @property
    def osqt(self) -> U32Vector4: ...
    @property
    def osqq(self) -> U32Vector4: ...
    @property
    def osqp(self) -> U32Vector4: ...
    @property
    def osqo(self) -> U32Vector4: ...
    @property
    def osql(self) -> U32Vector4: ...
    @property
    def osps(self) -> U32Vector4: ...
    @property
    def ospt(self) -> U32Vector4: ...
    @property
    def ospq(self) -> U32Vector4: ...
    @property
    def ospp(self) -> U32Vector4: ...
    @property
    def ospo(self) -> U32Vector4: ...
    @property
    def ospl(self) -> U32Vector4: ...
    @property
    def osos(self) -> U32Vector4: ...
    @property
    def osot(self) -> U32Vector4: ...
    @property
    def osoq(self) -> U32Vector4: ...
    @property
    def osop(self) -> U32Vector4: ...
    @property
    def osoo(self) -> U32Vector4: ...
    @property
    def osol(self) -> U32Vector4: ...
    @property
    def osls(self) -> U32Vector4: ...
    @property
    def oslt(self) -> U32Vector4: ...
    @property
    def oslq(self) -> U32Vector4: ...
    @property
    def oslp(self) -> U32Vector4: ...
    @property
    def oslo(self) -> U32Vector4: ...
    @property
    def osll(self) -> U32Vector4: ...
    @property
    def otss(self) -> U32Vector4: ...
    @property
    def otst(self) -> U32Vector4: ...
    @property
    def otsq(self) -> U32Vector4: ...
    @property
    def otsp(self) -> U32Vector4: ...
    @property
    def otso(self) -> U32Vector4: ...
    @property
    def otsl(self) -> U32Vector4: ...
    @property
    def otts(self) -> U32Vector4: ...
    @property
    def ottt(self) -> U32Vector4: ...
    @property
    def ottq(self) -> U32Vector4: ...
    @property
    def ottp(self) -> U32Vector4: ...
    @property
    def otto(self) -> U32Vector4: ...
    @property
    def ottl(self) -> U32Vector4: ...
    @property
    def otqs(self) -> U32Vector4: ...
    @property
    def otqt(self) -> U32Vector4: ...
    @property
    def otqq(self) -> U32Vector4: ...
    @property
    def otqp(self) -> U32Vector4: ...
    @property
    def otqo(self) -> U32Vector4: ...
    @property
    def otql(self) -> U32Vector4: ...
    @property
    def otps(self) -> U32Vector4: ...
    @property
    def otpt(self) -> U32Vector4: ...
    @property
    def otpq(self) -> U32Vector4: ...
    @property
    def otpp(self) -> U32Vector4: ...
    @property
    def otpo(self) -> U32Vector4: ...
    @property
    def otpl(self) -> U32Vector4: ...
    @property
    def otos(self) -> U32Vector4: ...
    @property
    def otot(self) -> U32Vector4: ...
    @property
    def otoq(self) -> U32Vector4: ...
    @property
    def otop(self) -> U32Vector4: ...
    @property
    def otoo(self) -> U32Vector4: ...
    @property
    def otol(self) -> U32Vector4: ...
    @property
    def otls(self) -> U32Vector4: ...
    @property
    def otlt(self) -> U32Vector4: ...
    @property
    def otlq(self) -> U32Vector4: ...
    @property
    def otlp(self) -> U32Vector4: ...
    @property
    def otlo(self) -> U32Vector4: ...
    @property
    def otll(self) -> U32Vector4: ...
    @property
    def oqss(self) -> U32Vector4: ...
    @property
    def oqst(self) -> U32Vector4: ...
    @property
    def oqsq(self) -> U32Vector4: ...
    @property
    def oqsp(self) -> U32Vector4: ...
    @property
    def oqso(self) -> U32Vector4: ...
    @property
    def oqsl(self) -> U32Vector4: ...
    @property
    def oqts(self) -> U32Vector4: ...
    @property
    def oqtt(self) -> U32Vector4: ...
    @property
    def oqtq(self) -> U32Vector4: ...
    @property
    def oqtp(self) -> U32Vector4: ...
    @property
    def oqto(self) -> U32Vector4: ...
    @property
    def oqtl(self) -> U32Vector4: ...
    @property
    def oqqs(self) -> U32Vector4: ...
    @property
    def oqqt(self) -> U32Vector4: ...
    @property
    def oqqq(self) -> U32Vector4: ...
    @property
    def oqqp(self) -> U32Vector4: ...
    @property
    def oqqo(self) -> U32Vector4: ...
    @property
    def oqql(self) -> U32Vector4: ...
    @property
    def oqps(self) -> U32Vector4: ...
    @property
    def oqpt(self) -> U32Vector4: ...
    @property
    def oqpq(self) -> U32Vector4: ...
    @property
    def oqpp(self) -> U32Vector4: ...
    @property
    def oqpo(self) -> U32Vector4: ...
    @property
    def oqpl(self) -> U32Vector4: ...
    @property
    def oqos(self) -> U32Vector4: ...
    @property
    def oqot(self) -> U32Vector4: ...
    @property
    def oqoq(self) -> U32Vector4: ...
    @property
    def oqop(self) -> U32Vector4: ...
    @property
    def oqoo(self) -> U32Vector4: ...
    @property
    def oqol(self) -> U32Vector4: ...
    @property
    def oqls(self) -> U32Vector4: ...
    @property
    def oqlt(self) -> U32Vector4: ...
    @property
    def oqlq(self) -> U32Vector4: ...
    @property
    def oqlp(self) -> U32Vector4: ...
    @property
    def oqlo(self) -> U32Vector4: ...
    @property
    def oqll(self) -> U32Vector4: ...
    @property
    def opss(self) -> U32Vector4: ...
    @property
    def opst(self) -> U32Vector4: ...
    @property
    def opsq(self) -> U32Vector4: ...
    @property
    def opsp(self) -> U32Vector4: ...
    @property
    def opso(self) -> U32Vector4: ...
    @property
    def opsl(self) -> U32Vector4: ...
    @property
    def opts(self) -> U32Vector4: ...
    @property
    def optt(self) -> U32Vector4: ...
    @property
    def optq(self) -> U32Vector4: ...
    @property
    def optp(self) -> U32Vector4: ...
    @property
    def opto(self) -> U32Vector4: ...
    @property
    def optl(self) -> U32Vector4: ...
    @property
    def opqs(self) -> U32Vector4: ...
    @property
    def opqt(self) -> U32Vector4: ...
    @property
    def opqq(self) -> U32Vector4: ...
    @property
    def opqp(self) -> U32Vector4: ...
    @property
    def opqo(self) -> U32Vector4: ...
    @property
    def opql(self) -> U32Vector4: ...
    @property
    def opps(self) -> U32Vector4: ...
    @property
    def oppt(self) -> U32Vector4: ...
    @property
    def oppq(self) -> U32Vector4: ...
    @property
    def oppp(self) -> U32Vector4: ...
    @property
    def oppo(self) -> U32Vector4: ...
    @property
    def oppl(self) -> U32Vector4: ...
    @property
    def opos(self) -> U32Vector4: ...
    @property
    def opot(self) -> U32Vector4: ...
    @property
    def opoq(self) -> U32Vector4: ...
    @property
    def opop(self) -> U32Vector4: ...
    @property
    def opoo(self) -> U32Vector4: ...
    @property
    def opol(self) -> U32Vector4: ...
    @property
    def opls(self) -> U32Vector4: ...
    @property
    def oplt(self) -> U32Vector4: ...
    @property
    def oplq(self) -> U32Vector4: ...
    @property
    def oplp(self) -> U32Vector4: ...
    @property
    def oplo(self) -> U32Vector4: ...
    @property
    def opll(self) -> U32Vector4: ...
    @property
    def ooss(self) -> U32Vector4: ...
    @property
    def oost(self) -> U32Vector4: ...
    @property
    def oosq(self) -> U32Vector4: ...
    @property
    def oosp(self) -> U32Vector4: ...
    @property
    def ooso(self) -> U32Vector4: ...
    @property
    def oosl(self) -> U32Vector4: ...
    @property
    def oots(self) -> U32Vector4: ...
    @property
    def oott(self) -> U32Vector4: ...
    @property
    def ootq(self) -> U32Vector4: ...
    @property
    def ootp(self) -> U32Vector4: ...
    @property
    def ooto(self) -> U32Vector4: ...
    @property
    def ootl(self) -> U32Vector4: ...
    @property
    def ooqs(self) -> U32Vector4: ...
    @property
    def ooqt(self) -> U32Vector4: ...
    @property
    def ooqq(self) -> U32Vector4: ...
    @property
    def ooqp(self) -> U32Vector4: ...
    @property
    def ooqo(self) -> U32Vector4: ...
    @property
    def ooql(self) -> U32Vector4: ...
    @property
    def oops(self) -> U32Vector4: ...
    @property
    def oopt(self) -> U32Vector4: ...
    @property
    def oopq(self) -> U32Vector4: ...
    @property
    def oopp(self) -> U32Vector4: ...
    @property
    def oopo(self) -> U32Vector4: ...
    @property
    def oopl(self) -> U32Vector4: ...
    @property
    def ooos(self) -> U32Vector4: ...
    @property
    def ooot(self) -> U32Vector4: ...
    @property
    def oooq(self) -> U32Vector4: ...
    @property
    def ooop(self) -> U32Vector4: ...
    @property
    def ools(self) -> U32Vector4: ...
    @property
    def oolt(self) -> U32Vector4: ...
    @property
    def oolq(self) -> U32Vector4: ...
    @property
    def oolp(self) -> U32Vector4: ...
    @property
    def olss(self) -> U32Vector4: ...
    @property
    def olst(self) -> U32Vector4: ...
    @property
    def olsq(self) -> U32Vector4: ...
    @property
    def olsp(self) -> U32Vector4: ...
    @property
    def olso(self) -> U32Vector4: ...
    @property
    def olsl(self) -> U32Vector4: ...
    @property
    def olts(self) -> U32Vector4: ...
    @property
    def oltt(self) -> U32Vector4: ...
    @property
    def oltq(self) -> U32Vector4: ...
    @property
    def oltp(self) -> U32Vector4: ...
    @property
    def olto(self) -> U32Vector4: ...
    @property
    def oltl(self) -> U32Vector4: ...
    @property
    def olqs(self) -> U32Vector4: ...
    @property
    def olqt(self) -> U32Vector4: ...
    @property
    def olqq(self) -> U32Vector4: ...
    @property
    def olqp(self) -> U32Vector4: ...
    @property
    def olqo(self) -> U32Vector4: ...
    @property
    def olql(self) -> U32Vector4: ...
    @property
    def olps(self) -> U32Vector4: ...
    @property
    def olpt(self) -> U32Vector4: ...
    @property
    def olpq(self) -> U32Vector4: ...
    @property
    def olpp(self) -> U32Vector4: ...
    @property
    def olpo(self) -> U32Vector4: ...
    @property
    def olpl(self) -> U32Vector4: ...
    @property
    def olos(self) -> U32Vector4: ...
    @property
    def olot(self) -> U32Vector4: ...
    @property
    def oloq(self) -> U32Vector4: ...
    @property
    def olop(self) -> U32Vector4: ...
    @property
    def olls(self) -> U32Vector4: ...
    @property
    def ollt(self) -> U32Vector4: ...
    @property
    def ollq(self) -> U32Vector4: ...
    @property
    def ollp(self) -> U32Vector4: ...
    @property
    def lsss(self) -> U32Vector4: ...
    @property
    def lsst(self) -> U32Vector4: ...
    @property
    def lssq(self) -> U32Vector4: ...
    @property
    def lssp(self) -> U32Vector4: ...
    @property
    def lsso(self) -> U32Vector4: ...
    @property
    def lssl(self) -> U32Vector4: ...
    @property
    def lsts(self) -> U32Vector4: ...
    @property
    def lstt(self) -> U32Vector4: ...
    @property
    def lstq(self) -> U32Vector4: ...
    @property
    def lstp(self) -> U32Vector4: ...
    @property
    def lsto(self) -> U32Vector4: ...
    @property
    def lstl(self) -> U32Vector4: ...
    @property
    def lsqs(self) -> U32Vector4: ...
    @property
    def lsqt(self) -> U32Vector4: ...
    @property
    def lsqq(self) -> U32Vector4: ...
    @property
    def lsqp(self) -> U32Vector4: ...
    @property
    def lsqo(self) -> U32Vector4: ...
    @property
    def lsql(self) -> U32Vector4: ...
    @property
    def lsps(self) -> U32Vector4: ...
    @property
    def lspt(self) -> U32Vector4: ...
    @property
    def lspq(self) -> U32Vector4: ...
    @property
    def lspp(self) -> U32Vector4: ...
    @property
    def lspo(self) -> U32Vector4: ...
    @property
    def lspl(self) -> U32Vector4: ...
    @property
    def lsos(self) -> U32Vector4: ...
    @property
    def lsot(self) -> U32Vector4: ...
    @property
    def lsoq(self) -> U32Vector4: ...
    @property
    def lsop(self) -> U32Vector4: ...
    @property
    def lsoo(self) -> U32Vector4: ...
    @property
    def lsol(self) -> U32Vector4: ...
    @property
    def lsls(self) -> U32Vector4: ...
    @property
    def lslt(self) -> U32Vector4: ...
    @property
    def lslq(self) -> U32Vector4: ...
    @property
    def lslp(self) -> U32Vector4: ...
    @property
    def lslo(self) -> U32Vector4: ...
    @property
    def lsll(self) -> U32Vector4: ...
    @property
    def ltss(self) -> U32Vector4: ...
    @property
    def ltst(self) -> U32Vector4: ...
    @property
    def ltsq(self) -> U32Vector4: ...
    @property
    def ltsp(self) -> U32Vector4: ...
    @property
    def ltso(self) -> U32Vector4: ...
    @property
    def ltsl(self) -> U32Vector4: ...
    @property
    def ltts(self) -> U32Vector4: ...
    @property
    def lttt(self) -> U32Vector4: ...
    @property
    def lttq(self) -> U32Vector4: ...
    @property
    def lttp(self) -> U32Vector4: ...
    @property
    def ltto(self) -> U32Vector4: ...
    @property
    def lttl(self) -> U32Vector4: ...
    @property
    def ltqs(self) -> U32Vector4: ...
    @property
    def ltqt(self) -> U32Vector4: ...
    @property
    def ltqq(self) -> U32Vector4: ...
    @property
    def ltqp(self) -> U32Vector4: ...
    @property
    def ltqo(self) -> U32Vector4: ...
    @property
    def ltql(self) -> U32Vector4: ...
    @property
    def ltps(self) -> U32Vector4: ...
    @property
    def ltpt(self) -> U32Vector4: ...
    @property
    def ltpq(self) -> U32Vector4: ...
    @property
    def ltpp(self) -> U32Vector4: ...
    @property
    def ltpo(self) -> U32Vector4: ...
    @property
    def ltpl(self) -> U32Vector4: ...
    @property
    def ltos(self) -> U32Vector4: ...
    @property
    def ltot(self) -> U32Vector4: ...
    @property
    def ltoq(self) -> U32Vector4: ...
    @property
    def ltop(self) -> U32Vector4: ...
    @property
    def ltoo(self) -> U32Vector4: ...
    @property
    def ltol(self) -> U32Vector4: ...
    @property
    def ltls(self) -> U32Vector4: ...
    @property
    def ltlt(self) -> U32Vector4: ...
    @property
    def ltlq(self) -> U32Vector4: ...
    @property
    def ltlp(self) -> U32Vector4: ...
    @property
    def ltlo(self) -> U32Vector4: ...
    @property
    def ltll(self) -> U32Vector4: ...
    @property
    def lqss(self) -> U32Vector4: ...
    @property
    def lqst(self) -> U32Vector4: ...
    @property
    def lqsq(self) -> U32Vector4: ...
    @property
    def lqsp(self) -> U32Vector4: ...
    @property
    def lqso(self) -> U32Vector4: ...
    @property
    def lqsl(self) -> U32Vector4: ...
    @property
    def lqts(self) -> U32Vector4: ...
    @property
    def lqtt(self) -> U32Vector4: ...
    @property
    def lqtq(self) -> U32Vector4: ...
    @property
    def lqtp(self) -> U32Vector4: ...
    @property
    def lqto(self) -> U32Vector4: ...
    @property
    def lqtl(self) -> U32Vector4: ...
    @property
    def lqqs(self) -> U32Vector4: ...
    @property
    def lqqt(self) -> U32Vector4: ...
    @property
    def lqqq(self) -> U32Vector4: ...
    @property
    def lqqp(self) -> U32Vector4: ...
    @property
    def lqqo(self) -> U32Vector4: ...
    @property
    def lqql(self) -> U32Vector4: ...
    @property
    def lqps(self) -> U32Vector4: ...
    @property
    def lqpt(self) -> U32Vector4: ...
    @property
    def lqpq(self) -> U32Vector4: ...
    @property
    def lqpp(self) -> U32Vector4: ...
    @property
    def lqpo(self) -> U32Vector4: ...
    @property
    def lqpl(self) -> U32Vector4: ...
    @property
    def lqos(self) -> U32Vector4: ...
    @property
    def lqot(self) -> U32Vector4: ...
    @property
    def lqoq(self) -> U32Vector4: ...
    @property
    def lqop(self) -> U32Vector4: ...
    @property
    def lqoo(self) -> U32Vector4: ...
    @property
    def lqol(self) -> U32Vector4: ...
    @property
    def lqls(self) -> U32Vector4: ...
    @property
    def lqlt(self) -> U32Vector4: ...
    @property
    def lqlq(self) -> U32Vector4: ...
    @property
    def lqlp(self) -> U32Vector4: ...
    @property
    def lqlo(self) -> U32Vector4: ...
    @property
    def lqll(self) -> U32Vector4: ...
    @property
    def lpss(self) -> U32Vector4: ...
    @property
    def lpst(self) -> U32Vector4: ...
    @property
    def lpsq(self) -> U32Vector4: ...
    @property
    def lpsp(self) -> U32Vector4: ...
    @property
    def lpso(self) -> U32Vector4: ...
    @property
    def lpsl(self) -> U32Vector4: ...
    @property
    def lpts(self) -> U32Vector4: ...
    @property
    def lptt(self) -> U32Vector4: ...
    @property
    def lptq(self) -> U32Vector4: ...
    @property
    def lptp(self) -> U32Vector4: ...
    @property
    def lpto(self) -> U32Vector4: ...
    @property
    def lptl(self) -> U32Vector4: ...
    @property
    def lpqs(self) -> U32Vector4: ...
    @property
    def lpqt(self) -> U32Vector4: ...
    @property
    def lpqq(self) -> U32Vector4: ...
    @property
    def lpqp(self) -> U32Vector4: ...
    @property
    def lpqo(self) -> U32Vector4: ...
    @property
    def lpql(self) -> U32Vector4: ...
    @property
    def lpps(self) -> U32Vector4: ...
    @property
    def lppt(self) -> U32Vector4: ...
    @property
    def lppq(self) -> U32Vector4: ...
    @property
    def lppp(self) -> U32Vector4: ...
    @property
    def lppo(self) -> U32Vector4: ...
    @property
    def lppl(self) -> U32Vector4: ...
    @property
    def lpos(self) -> U32Vector4: ...
    @property
    def lpot(self) -> U32Vector4: ...
    @property
    def lpoq(self) -> U32Vector4: ...
    @property
    def lpop(self) -> U32Vector4: ...
    @property
    def lpoo(self) -> U32Vector4: ...
    @property
    def lpol(self) -> U32Vector4: ...
    @property
    def lpls(self) -> U32Vector4: ...
    @property
    def lplt(self) -> U32Vector4: ...
    @property
    def lplq(self) -> U32Vector4: ...
    @property
    def lplp(self) -> U32Vector4: ...
    @property
    def lplo(self) -> U32Vector4: ...
    @property
    def lpll(self) -> U32Vector4: ...
    @property
    def loss(self) -> U32Vector4: ...
    @property
    def lost(self) -> U32Vector4: ...
    @property
    def losq(self) -> U32Vector4: ...
    @property
    def losp(self) -> U32Vector4: ...
    @property
    def loso(self) -> U32Vector4: ...
    @property
    def losl(self) -> U32Vector4: ...
    @property
    def lots(self) -> U32Vector4: ...
    @property
    def lott(self) -> U32Vector4: ...
    @property
    def lotq(self) -> U32Vector4: ...
    @property
    def lotp(self) -> U32Vector4: ...
    @property
    def loto(self) -> U32Vector4: ...
    @property
    def lotl(self) -> U32Vector4: ...
    @property
    def loqs(self) -> U32Vector4: ...
    @property
    def loqt(self) -> U32Vector4: ...
    @property
    def loqq(self) -> U32Vector4: ...
    @property
    def loqp(self) -> U32Vector4: ...
    @property
    def loqo(self) -> U32Vector4: ...
    @property
    def loql(self) -> U32Vector4: ...
    @property
    def lops(self) -> U32Vector4: ...
    @property
    def lopt(self) -> U32Vector4: ...
    @property
    def lopq(self) -> U32Vector4: ...
    @property
    def lopp(self) -> U32Vector4: ...
    @property
    def lopo(self) -> U32Vector4: ...
    @property
    def lopl(self) -> U32Vector4: ...
    @property
    def loos(self) -> U32Vector4: ...
    @property
    def loot(self) -> U32Vector4: ...
    @property
    def looq(self) -> U32Vector4: ...
    @property
    def loop(self) -> U32Vector4: ...
    @property
    def lols(self) -> U32Vector4: ...
    @property
    def lolt(self) -> U32Vector4: ...
    @property
    def lolq(self) -> U32Vector4: ...
    @property
    def lolp(self) -> U32Vector4: ...
    @property
    def llss(self) -> U32Vector4: ...
    @property
    def llst(self) -> U32Vector4: ...
    @property
    def llsq(self) -> U32Vector4: ...
    @property
    def llsp(self) -> U32Vector4: ...
    @property
    def llso(self) -> U32Vector4: ...
    @property
    def llsl(self) -> U32Vector4: ...
    @property
    def llts(self) -> U32Vector4: ...
    @property
    def lltt(self) -> U32Vector4: ...
    @property
    def lltq(self) -> U32Vector4: ...
    @property
    def lltp(self) -> U32Vector4: ...
    @property
    def llto(self) -> U32Vector4: ...
    @property
    def lltl(self) -> U32Vector4: ...
    @property
    def llqs(self) -> U32Vector4: ...
    @property
    def llqt(self) -> U32Vector4: ...
    @property
    def llqq(self) -> U32Vector4: ...
    @property
    def llqp(self) -> U32Vector4: ...
    @property
    def llqo(self) -> U32Vector4: ...
    @property
    def llql(self) -> U32Vector4: ...
    @property
    def llps(self) -> U32Vector4: ...
    @property
    def llpt(self) -> U32Vector4: ...
    @property
    def llpq(self) -> U32Vector4: ...
    @property
    def llpp(self) -> U32Vector4: ...
    @property
    def llpo(self) -> U32Vector4: ...
    @property
    def llpl(self) -> U32Vector4: ...
    @property
    def llos(self) -> U32Vector4: ...
    @property
    def llot(self) -> U32Vector4: ...
    @property
    def lloq(self) -> U32Vector4: ...
    @property
    def llop(self) -> U32Vector4: ...
    @property
    def llls(self) -> U32Vector4: ...
    @property
    def lllt(self) -> U32Vector4: ...
    @property
    def lllq(self) -> U32Vector4: ...
    @property
    def lllp(self) -> U32Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U32Vector2: ...
    @property
    def uv(self) -> U32Vector2: ...
    @property
    def uo(self) -> U32Vector2: ...
    @property
    def ul(self) -> U32Vector2: ...
    @property
    def vu(self) -> U32Vector2: ...
    @property
    def vv(self) -> U32Vector2: ...
    @property
    def vo(self) -> U32Vector2: ...
    @property
    def vl(self) -> U32Vector2: ...
    @property
    def ou(self) -> U32Vector2: ...
    @property
    def ov(self) -> U32Vector2: ...
    @property
    def lu(self) -> U32Vector2: ...
    @property
    def lv(self) -> U32Vector2: ...
    @property
    def uuu(self) -> U32Vector3: ...
    @property
    def uuv(self) -> U32Vector3: ...
    @property
    def uuo(self) -> U32Vector3: ...
    @property
    def uul(self) -> U32Vector3: ...
    @property
    def uvu(self) -> U32Vector3: ...
    @property
    def uvv(self) -> U32Vector3: ...
    @property
    def uvo(self) -> U32Vector3: ...
    @property
    def uvl(self) -> U32Vector3: ...
    @property
    def uou(self) -> U32Vector3: ...
    @property
    def uov(self) -> U32Vector3: ...
    @property
    def uoo(self) -> U32Vector3: ...
    @property
    def uol(self) -> U32Vector3: ...
    @property
    def ulu(self) -> U32Vector3: ...
    @property
    def ulv(self) -> U32Vector3: ...
    @property
    def ulo(self) -> U32Vector3: ...
    @property
    def ull(self) -> U32Vector3: ...
    @property
    def vuu(self) -> U32Vector3: ...
    @property
    def vuv(self) -> U32Vector3: ...
    @property
    def vuo(self) -> U32Vector3: ...
    @property
    def vul(self) -> U32Vector3: ...
    @property
    def vvu(self) -> U32Vector3: ...
    @property
    def vvv(self) -> U32Vector3: ...
    @property
    def vvo(self) -> U32Vector3: ...
    @property
    def vvl(self) -> U32Vector3: ...
    @property
    def vou(self) -> U32Vector3: ...
    @property
    def vov(self) -> U32Vector3: ...
    @property
    def voo(self) -> U32Vector3: ...
    @property
    def vol(self) -> U32Vector3: ...
    @property
    def vlu(self) -> U32Vector3: ...
    @property
    def vlv(self) -> U32Vector3: ...
    @property
    def vlo(self) -> U32Vector3: ...
    @property
    def vll(self) -> U32Vector3: ...
    @property
    def ouu(self) -> U32Vector3: ...
    @property
    def ouv(self) -> U32Vector3: ...
    @property
    def ouo(self) -> U32Vector3: ...
    @property
    def oul(self) -> U32Vector3: ...
    @property
    def ovu(self) -> U32Vector3: ...
    @property
    def ovv(self) -> U32Vector3: ...
    @property
    def ovo(self) -> U32Vector3: ...
    @property
    def ovl(self) -> U32Vector3: ...
    @property
    def oou(self) -> U32Vector3: ...
    @property
    def oov(self) -> U32Vector3: ...
    @property
    def olu(self) -> U32Vector3: ...
    @property
    def olv(self) -> U32Vector3: ...
    @property
    def luu(self) -> U32Vector3: ...
    @property
    def luv(self) -> U32Vector3: ...
    @property
    def luo(self) -> U32Vector3: ...
    @property
    def lul(self) -> U32Vector3: ...
    @property
    def lvu(self) -> U32Vector3: ...
    @property
    def lvv(self) -> U32Vector3: ...
    @property
    def lvo(self) -> U32Vector3: ...
    @property
    def lvl(self) -> U32Vector3: ...
    @property
    def lou(self) -> U32Vector3: ...
    @property
    def lov(self) -> U32Vector3: ...
    @property
    def llu(self) -> U32Vector3: ...
    @property
    def llv(self) -> U32Vector3: ...
    @property
    def uuuu(self) -> U32Vector4: ...
    @property
    def uuuv(self) -> U32Vector4: ...
    @property
    def uuuo(self) -> U32Vector4: ...
    @property
    def uuul(self) -> U32Vector4: ...
    @property
    def uuvu(self) -> U32Vector4: ...
    @property
    def uuvv(self) -> U32Vector4: ...
    @property
    def uuvo(self) -> U32Vector4: ...
    @property
    def uuvl(self) -> U32Vector4: ...
    @property
    def uuou(self) -> U32Vector4: ...
    @property
    def uuov(self) -> U32Vector4: ...
    @property
    def uuoo(self) -> U32Vector4: ...
    @property
    def uuol(self) -> U32Vector4: ...
    @property
    def uulu(self) -> U32Vector4: ...
    @property
    def uulv(self) -> U32Vector4: ...
    @property
    def uulo(self) -> U32Vector4: ...
    @property
    def uull(self) -> U32Vector4: ...
    @property
    def uvuu(self) -> U32Vector4: ...
    @property
    def uvuv(self) -> U32Vector4: ...
    @property
    def uvuo(self) -> U32Vector4: ...
    @property
    def uvul(self) -> U32Vector4: ...
    @property
    def uvvu(self) -> U32Vector4: ...
    @property
    def uvvv(self) -> U32Vector4: ...
    @property
    def uvvo(self) -> U32Vector4: ...
    @property
    def uvvl(self) -> U32Vector4: ...
    @property
    def uvou(self) -> U32Vector4: ...
    @property
    def uvov(self) -> U32Vector4: ...
    @property
    def uvoo(self) -> U32Vector4: ...
    @property
    def uvol(self) -> U32Vector4: ...
    @property
    def uvlu(self) -> U32Vector4: ...
    @property
    def uvlv(self) -> U32Vector4: ...
    @property
    def uvlo(self) -> U32Vector4: ...
    @property
    def uvll(self) -> U32Vector4: ...
    @property
    def uouu(self) -> U32Vector4: ...
    @property
    def uouv(self) -> U32Vector4: ...
    @property
    def uouo(self) -> U32Vector4: ...
    @property
    def uoul(self) -> U32Vector4: ...
    @property
    def uovu(self) -> U32Vector4: ...
    @property
    def uovv(self) -> U32Vector4: ...
    @property
    def uovo(self) -> U32Vector4: ...
    @property
    def uovl(self) -> U32Vector4: ...
    @property
    def uoou(self) -> U32Vector4: ...
    @property
    def uoov(self) -> U32Vector4: ...
    @property
    def uooo(self) -> U32Vector4: ...
    @property
    def uool(self) -> U32Vector4: ...
    @property
    def uolu(self) -> U32Vector4: ...
    @property
    def uolv(self) -> U32Vector4: ...
    @property
    def uolo(self) -> U32Vector4: ...
    @property
    def uoll(self) -> U32Vector4: ...
    @property
    def uluu(self) -> U32Vector4: ...
    @property
    def uluv(self) -> U32Vector4: ...
    @property
    def uluo(self) -> U32Vector4: ...
    @property
    def ulul(self) -> U32Vector4: ...
    @property
    def ulvu(self) -> U32Vector4: ...
    @property
    def ulvv(self) -> U32Vector4: ...
    @property
    def ulvo(self) -> U32Vector4: ...
    @property
    def ulvl(self) -> U32Vector4: ...
    @property
    def ulou(self) -> U32Vector4: ...
    @property
    def ulov(self) -> U32Vector4: ...
    @property
    def uloo(self) -> U32Vector4: ...
    @property
    def ulol(self) -> U32Vector4: ...
    @property
    def ullu(self) -> U32Vector4: ...
    @property
    def ullv(self) -> U32Vector4: ...
    @property
    def ullo(self) -> U32Vector4: ...
    @property
    def ulll(self) -> U32Vector4: ...
    @property
    def vuuu(self) -> U32Vector4: ...
    @property
    def vuuv(self) -> U32Vector4: ...
    @property
    def vuuo(self) -> U32Vector4: ...
    @property
    def vuul(self) -> U32Vector4: ...
    @property
    def vuvu(self) -> U32Vector4: ...
    @property
    def vuvv(self) -> U32Vector4: ...
    @property
    def vuvo(self) -> U32Vector4: ...
    @property
    def vuvl(self) -> U32Vector4: ...
    @property
    def vuou(self) -> U32Vector4: ...
    @property
    def vuov(self) -> U32Vector4: ...
    @property
    def vuoo(self) -> U32Vector4: ...
    @property
    def vuol(self) -> U32Vector4: ...
    @property
    def vulu(self) -> U32Vector4: ...
    @property
    def vulv(self) -> U32Vector4: ...
    @property
    def vulo(self) -> U32Vector4: ...
    @property
    def vull(self) -> U32Vector4: ...
    @property
    def vvuu(self) -> U32Vector4: ...
    @property
    def vvuv(self) -> U32Vector4: ...
    @property
    def vvuo(self) -> U32Vector4: ...
    @property
    def vvul(self) -> U32Vector4: ...
    @property
    def vvvu(self) -> U32Vector4: ...
    @property
    def vvvv(self) -> U32Vector4: ...
    @property
    def vvvo(self) -> U32Vector4: ...
    @property
    def vvvl(self) -> U32Vector4: ...
    @property
    def vvou(self) -> U32Vector4: ...
    @property
    def vvov(self) -> U32Vector4: ...
    @property
    def vvoo(self) -> U32Vector4: ...
    @property
    def vvol(self) -> U32Vector4: ...
    @property
    def vvlu(self) -> U32Vector4: ...
    @property
    def vvlv(self) -> U32Vector4: ...
    @property
    def vvlo(self) -> U32Vector4: ...
    @property
    def vvll(self) -> U32Vector4: ...
    @property
    def vouu(self) -> U32Vector4: ...
    @property
    def vouv(self) -> U32Vector4: ...
    @property
    def vouo(self) -> U32Vector4: ...
    @property
    def voul(self) -> U32Vector4: ...
    @property
    def vovu(self) -> U32Vector4: ...
    @property
    def vovv(self) -> U32Vector4: ...
    @property
    def vovo(self) -> U32Vector4: ...
    @property
    def vovl(self) -> U32Vector4: ...
    @property
    def voou(self) -> U32Vector4: ...
    @property
    def voov(self) -> U32Vector4: ...
    @property
    def vooo(self) -> U32Vector4: ...
    @property
    def vool(self) -> U32Vector4: ...
    @property
    def volu(self) -> U32Vector4: ...
    @property
    def volv(self) -> U32Vector4: ...
    @property
    def volo(self) -> U32Vector4: ...
    @property
    def voll(self) -> U32Vector4: ...
    @property
    def vluu(self) -> U32Vector4: ...
    @property
    def vluv(self) -> U32Vector4: ...
    @property
    def vluo(self) -> U32Vector4: ...
    @property
    def vlul(self) -> U32Vector4: ...
    @property
    def vlvu(self) -> U32Vector4: ...
    @property
    def vlvv(self) -> U32Vector4: ...
    @property
    def vlvo(self) -> U32Vector4: ...
    @property
    def vlvl(self) -> U32Vector4: ...
    @property
    def vlou(self) -> U32Vector4: ...
    @property
    def vlov(self) -> U32Vector4: ...
    @property
    def vloo(self) -> U32Vector4: ...
    @property
    def vlol(self) -> U32Vector4: ...
    @property
    def vllu(self) -> U32Vector4: ...
    @property
    def vllv(self) -> U32Vector4: ...
    @property
    def vllo(self) -> U32Vector4: ...
    @property
    def vlll(self) -> U32Vector4: ...
    @property
    def ouuu(self) -> U32Vector4: ...
    @property
    def ouuv(self) -> U32Vector4: ...
    @property
    def ouuo(self) -> U32Vector4: ...
    @property
    def ouul(self) -> U32Vector4: ...
    @property
    def ouvu(self) -> U32Vector4: ...
    @property
    def ouvv(self) -> U32Vector4: ...
    @property
    def ouvo(self) -> U32Vector4: ...
    @property
    def ouvl(self) -> U32Vector4: ...
    @property
    def ouou(self) -> U32Vector4: ...
    @property
    def ouov(self) -> U32Vector4: ...
    @property
    def ouoo(self) -> U32Vector4: ...
    @property
    def ouol(self) -> U32Vector4: ...
    @property
    def oulu(self) -> U32Vector4: ...
    @property
    def oulv(self) -> U32Vector4: ...
    @property
    def oulo(self) -> U32Vector4: ...
    @property
    def oull(self) -> U32Vector4: ...
    @property
    def ovuu(self) -> U32Vector4: ...
    @property
    def ovuv(self) -> U32Vector4: ...
    @property
    def ovuo(self) -> U32Vector4: ...
    @property
    def ovul(self) -> U32Vector4: ...
    @property
    def ovvu(self) -> U32Vector4: ...
    @property
    def ovvv(self) -> U32Vector4: ...
    @property
    def ovvo(self) -> U32Vector4: ...
    @property
    def ovvl(self) -> U32Vector4: ...
    @property
    def ovou(self) -> U32Vector4: ...
    @property
    def ovov(self) -> U32Vector4: ...
    @property
    def ovoo(self) -> U32Vector4: ...
    @property
    def ovol(self) -> U32Vector4: ...
    @property
    def ovlu(self) -> U32Vector4: ...
    @property
    def ovlv(self) -> U32Vector4: ...
    @property
    def ovlo(self) -> U32Vector4: ...
    @property
    def ovll(self) -> U32Vector4: ...
    @property
    def oouu(self) -> U32Vector4: ...
    @property
    def oouv(self) -> U32Vector4: ...
    @property
    def oouo(self) -> U32Vector4: ...
    @property
    def ooul(self) -> U32Vector4: ...
    @property
    def oovu(self) -> U32Vector4: ...
    @property
    def oovv(self) -> U32Vector4: ...
    @property
    def oovo(self) -> U32Vector4: ...
    @property
    def oovl(self) -> U32Vector4: ...
    @property
    def ooou(self) -> U32Vector4: ...
    @property
    def ooov(self) -> U32Vector4: ...
    @property
    def oolu(self) -> U32Vector4: ...
    @property
    def oolv(self) -> U32Vector4: ...
    @property
    def oluu(self) -> U32Vector4: ...
    @property
    def oluv(self) -> U32Vector4: ...
    @property
    def oluo(self) -> U32Vector4: ...
    @property
    def olul(self) -> U32Vector4: ...
    @property
    def olvu(self) -> U32Vector4: ...
    @property
    def olvv(self) -> U32Vector4: ...
    @property
    def olvo(self) -> U32Vector4: ...
    @property
    def olvl(self) -> U32Vector4: ...
    @property
    def olou(self) -> U32Vector4: ...
    @property
    def olov(self) -> U32Vector4: ...
    @property
    def ollu(self) -> U32Vector4: ...
    @property
    def ollv(self) -> U32Vector4: ...
    @property
    def luuu(self) -> U32Vector4: ...
    @property
    def luuv(self) -> U32Vector4: ...
    @property
    def luuo(self) -> U32Vector4: ...
    @property
    def luul(self) -> U32Vector4: ...
    @property
    def luvu(self) -> U32Vector4: ...
    @property
    def luvv(self) -> U32Vector4: ...
    @property
    def luvo(self) -> U32Vector4: ...
    @property
    def luvl(self) -> U32Vector4: ...
    @property
    def luou(self) -> U32Vector4: ...
    @property
    def luov(self) -> U32Vector4: ...
    @property
    def luoo(self) -> U32Vector4: ...
    @property
    def luol(self) -> U32Vector4: ...
    @property
    def lulu(self) -> U32Vector4: ...
    @property
    def lulv(self) -> U32Vector4: ...
    @property
    def lulo(self) -> U32Vector4: ...
    @property
    def lull(self) -> U32Vector4: ...
    @property
    def lvuu(self) -> U32Vector4: ...
    @property
    def lvuv(self) -> U32Vector4: ...
    @property
    def lvuo(self) -> U32Vector4: ...
    @property
    def lvul(self) -> U32Vector4: ...
    @property
    def lvvu(self) -> U32Vector4: ...
    @property
    def lvvv(self) -> U32Vector4: ...
    @property
    def lvvo(self) -> U32Vector4: ...
    @property
    def lvvl(self) -> U32Vector4: ...
    @property
    def lvou(self) -> U32Vector4: ...
    @property
    def lvov(self) -> U32Vector4: ...
    @property
    def lvoo(self) -> U32Vector4: ...
    @property
    def lvol(self) -> U32Vector4: ...
    @property
    def lvlu(self) -> U32Vector4: ...
    @property
    def lvlv(self) -> U32Vector4: ...
    @property
    def lvlo(self) -> U32Vector4: ...
    @property
    def lvll(self) -> U32Vector4: ...
    @property
    def louu(self) -> U32Vector4: ...
    @property
    def louv(self) -> U32Vector4: ...
    @property
    def louo(self) -> U32Vector4: ...
    @property
    def loul(self) -> U32Vector4: ...
    @property
    def lovu(self) -> U32Vector4: ...
    @property
    def lovv(self) -> U32Vector4: ...
    @property
    def lovo(self) -> U32Vector4: ...
    @property
    def lovl(self) -> U32Vector4: ...
    @property
    def loou(self) -> U32Vector4: ...
    @property
    def loov(self) -> U32Vector4: ...
    @property
    def lolu(self) -> U32Vector4: ...
    @property
    def lolv(self) -> U32Vector4: ...
    @property
    def lluu(self) -> U32Vector4: ...
    @property
    def lluv(self) -> U32Vector4: ...
    @property
    def lluo(self) -> U32Vector4: ...
    @property
    def llul(self) -> U32Vector4: ...
    @property
    def llvu(self) -> U32Vector4: ...
    @property
    def llvv(self) -> U32Vector4: ...
    @property
    def llvo(self) -> U32Vector4: ...
    @property
    def llvl(self) -> U32Vector4: ...
    @property
    def llou(self) -> U32Vector4: ...
    @property
    def llov(self) -> U32Vector4: ...
    @property
    def lllu(self) -> U32Vector4: ...
    @property
    def lllv(self) -> U32Vector4: ...
    @property
    def oo(self) -> U32Vector2: ...
    @property
    def ol(self) -> U32Vector2: ...
    @property
    def lo(self) -> U32Vector2: ...
    @property
    def ll(self) -> U32Vector2: ...
    @property
    def ooo(self) -> U32Vector3: ...
    @property
    def ool(self) -> U32Vector3: ...
    @property
    def olo(self) -> U32Vector3: ...
    @property
    def oll(self) -> U32Vector3: ...
    @property
    def loo(self) -> U32Vector3: ...
    @property
    def lol(self) -> U32Vector3: ...
    @property
    def llo(self) -> U32Vector3: ...
    @property
    def lll(self) -> U32Vector3: ...
    @property
    def oooo(self) -> U32Vector4: ...
    @property
    def oool(self) -> U32Vector4: ...
    @property
    def oolo(self) -> U32Vector4: ...
    @property
    def ooll(self) -> U32Vector4: ...
    @property
    def oloo(self) -> U32Vector4: ...
    @property
    def olol(self) -> U32Vector4: ...
    @property
    def ollo(self) -> U32Vector4: ...
    @property
    def olll(self) -> U32Vector4: ...
    @property
    def looo(self) -> U32Vector4: ...
    @property
    def lool(self) -> U32Vector4: ...
    @property
    def lolo(self) -> U32Vector4: ...
    @property
    def loll(self) -> U32Vector4: ...
    @property
    def lloo(self) -> U32Vector4: ...
    @property
    def llol(self) -> U32Vector4: ...
    @property
    def lllo(self) -> U32Vector4: ...
    @property
    def llll(self) -> U32Vector4: ...
    def min(self, n: Number, /) -> U32Vector4: ...
    def max(self, n: Number, /) -> U32Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> U32Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U32Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector4: ...

@final
class U32Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U32Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U32Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> U32Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> U32Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U32Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[U32Vector4]: ...

@final
class IVector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: IVector4) -> IVector4: ...
    @overload
    def __add__(self, other: Number) -> IVector4: ...
    @overload
    def __radd__(self, other: IVector4) -> IVector4: ...
    @overload
    def __radd__(self, other: Number) -> IVector4: ...
    @overload
    def __sub__(self, other: IVector4) -> IVector4: ...
    @overload
    def __sub__(self, other: Number) -> IVector4: ...
    @overload
    def __rsub__(self, other: IVector4) -> IVector4: ...
    @overload
    def __rsub__(self, other: Number) -> IVector4: ...
    @overload
    def __mul__(self, other: IVector4) -> IVector4: ...
    @overload
    def __mul__(self, other: Number) -> IVector4: ...
    @overload
    def __rmul__(self, other: IVector4) -> IVector4: ...
    @overload
    def __rmul__(self, other: Number) -> IVector4: ...
    @overload
    def __truediv__(self, other: IVector4) -> IVector4: ...
    @overload
    def __truediv__(self, other: Number) -> IVector4: ...
    @overload
    def __rtruediv__(self, other: IVector4) -> IVector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> IVector4: ...
    def __neg__(self) -> IVector4: ...
    def __abs__(self) -> IVector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> IVector2: ...
    @property
    def xy(self) -> IVector2: ...
    @property
    def xz(self) -> IVector2: ...
    @property
    def xw(self) -> IVector2: ...
    @property
    def xo(self) -> IVector2: ...
    @property
    def xl(self) -> IVector2: ...
    @property
    def yx(self) -> IVector2: ...
    @property
    def yy(self) -> IVector2: ...
    @property
    def yz(self) -> IVector2: ...
    @property
    def yw(self) -> IVector2: ...
    @property
    def yo(self) -> IVector2: ...
    @property
    def yl(self) -> IVector2: ...
    @property
    def zx(self) -> IVector2: ...
    @property
    def zy(self) -> IVector2: ...
    @property
    def zz(self) -> IVector2: ...
    @property
    def zw(self) -> IVector2: ...
    @property
    def zo(self) -> IVector2: ...
    @property
    def zl(self) -> IVector2: ...
    @property
    def wx(self) -> IVector2: ...
    @property
    def wy(self) -> IVector2: ...
    @property
    def wz(self) -> IVector2: ...
    @property
    def ww(self) -> IVector2: ...
    @property
    def wo(self) -> IVector2: ...
    @property
    def wl(self) -> IVector2: ...
    @property
    def ox(self) -> IVector2: ...
    @property
    def oy(self) -> IVector2: ...
    @property
    def oz(self) -> IVector2: ...
    @property
    def ow(self) -> IVector2: ...
    @property
    def lx(self) -> IVector2: ...
    @property
    def ly(self) -> IVector2: ...
    @property
    def lz(self) -> IVector2: ...
    @property
    def lw(self) -> IVector2: ...
    @property
    def xxx(self) -> IVector3: ...
    @property
    def xxy(self) -> IVector3: ...
    @property
    def xxz(self) -> IVector3: ...
    @property
    def xxw(self) -> IVector3: ...
    @property
    def xxo(self) -> IVector3: ...
    @property
    def xxl(self) -> IVector3: ...
    @property
    def xyx(self) -> IVector3: ...
    @property
    def xyy(self) -> IVector3: ...
    @property
    def xyz(self) -> IVector3: ...
    @property
    def xyw(self) -> IVector3: ...
    @property
    def xyo(self) -> IVector3: ...
    @property
    def xyl(self) -> IVector3: ...
    @property
    def xzx(self) -> IVector3: ...
    @property
    def xzy(self) -> IVector3: ...
    @property
    def xzz(self) -> IVector3: ...
    @property
    def xzw(self) -> IVector3: ...
    @property
    def xzo(self) -> IVector3: ...
    @property
    def xzl(self) -> IVector3: ...
    @property
    def xwx(self) -> IVector3: ...
    @property
    def xwy(self) -> IVector3: ...
    @property
    def xwz(self) -> IVector3: ...
    @property
    def xww(self) -> IVector3: ...
    @property
    def xwo(self) -> IVector3: ...
    @property
    def xwl(self) -> IVector3: ...
    @property
    def xox(self) -> IVector3: ...
    @property
    def xoy(self) -> IVector3: ...
    @property
    def xoz(self) -> IVector3: ...
    @property
    def xow(self) -> IVector3: ...
    @property
    def xoo(self) -> IVector3: ...
    @property
    def xol(self) -> IVector3: ...
    @property
    def xlx(self) -> IVector3: ...
    @property
    def xly(self) -> IVector3: ...
    @property
    def xlz(self) -> IVector3: ...
    @property
    def xlw(self) -> IVector3: ...
    @property
    def xlo(self) -> IVector3: ...
    @property
    def xll(self) -> IVector3: ...
    @property
    def yxx(self) -> IVector3: ...
    @property
    def yxy(self) -> IVector3: ...
    @property
    def yxz(self) -> IVector3: ...
    @property
    def yxw(self) -> IVector3: ...
    @property
    def yxo(self) -> IVector3: ...
    @property
    def yxl(self) -> IVector3: ...
    @property
    def yyx(self) -> IVector3: ...
    @property
    def yyy(self) -> IVector3: ...
    @property
    def yyz(self) -> IVector3: ...
    @property
    def yyw(self) -> IVector3: ...
    @property
    def yyo(self) -> IVector3: ...
    @property
    def yyl(self) -> IVector3: ...
    @property
    def yzx(self) -> IVector3: ...
    @property
    def yzy(self) -> IVector3: ...
    @property
    def yzz(self) -> IVector3: ...
    @property
    def yzw(self) -> IVector3: ...
    @property
    def yzo(self) -> IVector3: ...
    @property
    def yzl(self) -> IVector3: ...
    @property
    def ywx(self) -> IVector3: ...
    @property
    def ywy(self) -> IVector3: ...
    @property
    def ywz(self) -> IVector3: ...
    @property
    def yww(self) -> IVector3: ...
    @property
    def ywo(self) -> IVector3: ...
    @property
    def ywl(self) -> IVector3: ...
    @property
    def yox(self) -> IVector3: ...
    @property
    def yoy(self) -> IVector3: ...
    @property
    def yoz(self) -> IVector3: ...
    @property
    def yow(self) -> IVector3: ...
    @property
    def yoo(self) -> IVector3: ...
    @property
    def yol(self) -> IVector3: ...
    @property
    def ylx(self) -> IVector3: ...
    @property
    def yly(self) -> IVector3: ...
    @property
    def ylz(self) -> IVector3: ...
    @property
    def ylw(self) -> IVector3: ...
    @property
    def ylo(self) -> IVector3: ...
    @property
    def yll(self) -> IVector3: ...
    @property
    def zxx(self) -> IVector3: ...
    @property
    def zxy(self) -> IVector3: ...
    @property
    def zxz(self) -> IVector3: ...
    @property
    def zxw(self) -> IVector3: ...
    @property
    def zxo(self) -> IVector3: ...
    @property
    def zxl(self) -> IVector3: ...
    @property
    def zyx(self) -> IVector3: ...
    @property
    def zyy(self) -> IVector3: ...
    @property
    def zyz(self) -> IVector3: ...
    @property
    def zyw(self) -> IVector3: ...
    @property
    def zyo(self) -> IVector3: ...
    @property
    def zyl(self) -> IVector3: ...
    @property
    def zzx(self) -> IVector3: ...
    @property
    def zzy(self) -> IVector3: ...
    @property
    def zzz(self) -> IVector3: ...
    @property
    def zzw(self) -> IVector3: ...
    @property
    def zzo(self) -> IVector3: ...
    @property
    def zzl(self) -> IVector3: ...
    @property
    def zwx(self) -> IVector3: ...
    @property
    def zwy(self) -> IVector3: ...
    @property
    def zwz(self) -> IVector3: ...
    @property
    def zww(self) -> IVector3: ...
    @property
    def zwo(self) -> IVector3: ...
    @property
    def zwl(self) -> IVector3: ...
    @property
    def zox(self) -> IVector3: ...
    @property
    def zoy(self) -> IVector3: ...
    @property
    def zoz(self) -> IVector3: ...
    @property
    def zow(self) -> IVector3: ...
    @property
    def zoo(self) -> IVector3: ...
    @property
    def zol(self) -> IVector3: ...
    @property
    def zlx(self) -> IVector3: ...
    @property
    def zly(self) -> IVector3: ...
    @property
    def zlz(self) -> IVector3: ...
    @property
    def zlw(self) -> IVector3: ...
    @property
    def zlo(self) -> IVector3: ...
    @property
    def zll(self) -> IVector3: ...
    @property
    def wxx(self) -> IVector3: ...
    @property
    def wxy(self) -> IVector3: ...
    @property
    def wxz(self) -> IVector3: ...
    @property
    def wxw(self) -> IVector3: ...
    @property
    def wxo(self) -> IVector3: ...
    @property
    def wxl(self) -> IVector3: ...
    @property
    def wyx(self) -> IVector3: ...
    @property
    def wyy(self) -> IVector3: ...
    @property
    def wyz(self) -> IVector3: ...
    @property
    def wyw(self) -> IVector3: ...
    @property
    def wyo(self) -> IVector3: ...
    @property
    def wyl(self) -> IVector3: ...
    @property
    def wzx(self) -> IVector3: ...
    @property
    def wzy(self) -> IVector3: ...
    @property
    def wzz(self) -> IVector3: ...
    @property
    def wzw(self) -> IVector3: ...
    @property
    def wzo(self) -> IVector3: ...
    @property
    def wzl(self) -> IVector3: ...
    @property
    def wwx(self) -> IVector3: ...
    @property
    def wwy(self) -> IVector3: ...
    @property
    def wwz(self) -> IVector3: ...
    @property
    def www(self) -> IVector3: ...
    @property
    def wwo(self) -> IVector3: ...
    @property
    def wwl(self) -> IVector3: ...
    @property
    def wox(self) -> IVector3: ...
    @property
    def woy(self) -> IVector3: ...
    @property
    def woz(self) -> IVector3: ...
    @property
    def wow(self) -> IVector3: ...
    @property
    def woo(self) -> IVector3: ...
    @property
    def wol(self) -> IVector3: ...
    @property
    def wlx(self) -> IVector3: ...
    @property
    def wly(self) -> IVector3: ...
    @property
    def wlz(self) -> IVector3: ...
    @property
    def wlw(self) -> IVector3: ...
    @property
    def wlo(self) -> IVector3: ...
    @property
    def wll(self) -> IVector3: ...
    @property
    def oxx(self) -> IVector3: ...
    @property
    def oxy(self) -> IVector3: ...
    @property
    def oxz(self) -> IVector3: ...
    @property
    def oxw(self) -> IVector3: ...
    @property
    def oxo(self) -> IVector3: ...
    @property
    def oxl(self) -> IVector3: ...
    @property
    def oyx(self) -> IVector3: ...
    @property
    def oyy(self) -> IVector3: ...
    @property
    def oyz(self) -> IVector3: ...
    @property
    def oyw(self) -> IVector3: ...
    @property
    def oyo(self) -> IVector3: ...
    @property
    def oyl(self) -> IVector3: ...
    @property
    def ozx(self) -> IVector3: ...
    @property
    def ozy(self) -> IVector3: ...
    @property
    def ozz(self) -> IVector3: ...
    @property
    def ozw(self) -> IVector3: ...
    @property
    def ozo(self) -> IVector3: ...
    @property
    def ozl(self) -> IVector3: ...
    @property
    def owx(self) -> IVector3: ...
    @property
    def owy(self) -> IVector3: ...
    @property
    def owz(self) -> IVector3: ...
    @property
    def oww(self) -> IVector3: ...
    @property
    def owo(self) -> IVector3: ...
    @property
    def owl(self) -> IVector3: ...
    @property
    def oox(self) -> IVector3: ...
    @property
    def ooy(self) -> IVector3: ...
    @property
    def ooz(self) -> IVector3: ...
    @property
    def oow(self) -> IVector3: ...
    @property
    def olx(self) -> IVector3: ...
    @property
    def oly(self) -> IVector3: ...
    @property
    def olz(self) -> IVector3: ...
    @property
    def olw(self) -> IVector3: ...
    @property
    def lxx(self) -> IVector3: ...
    @property
    def lxy(self) -> IVector3: ...
    @property
    def lxz(self) -> IVector3: ...
    @property
    def lxw(self) -> IVector3: ...
    @property
    def lxo(self) -> IVector3: ...
    @property
    def lxl(self) -> IVector3: ...
    @property
    def lyx(self) -> IVector3: ...
    @property
    def lyy(self) -> IVector3: ...
    @property
    def lyz(self) -> IVector3: ...
    @property
    def lyw(self) -> IVector3: ...
    @property
    def lyo(self) -> IVector3: ...
    @property
    def lyl(self) -> IVector3: ...
    @property
    def lzx(self) -> IVector3: ...
    @property
    def lzy(self) -> IVector3: ...
    @property
    def lzz(self) -> IVector3: ...
    @property
    def lzw(self) -> IVector3: ...
    @property
    def lzo(self) -> IVector3: ...
    @property
    def lzl(self) -> IVector3: ...
    @property
    def lwx(self) -> IVector3: ...
    @property
    def lwy(self) -> IVector3: ...
    @property
    def lwz(self) -> IVector3: ...
    @property
    def lww(self) -> IVector3: ...
    @property
    def lwo(self) -> IVector3: ...
    @property
    def lwl(self) -> IVector3: ...
    @property
    def lox(self) -> IVector3: ...
    @property
    def loy(self) -> IVector3: ...
    @property
    def loz(self) -> IVector3: ...
    @property
    def low(self) -> IVector3: ...
    @property
    def llx(self) -> IVector3: ...
    @property
    def lly(self) -> IVector3: ...
    @property
    def llz(self) -> IVector3: ...
    @property
    def llw(self) -> IVector3: ...
    @property
    def xxxx(self) -> IVector4: ...
    @property
    def xxxy(self) -> IVector4: ...
    @property
    def xxxz(self) -> IVector4: ...
    @property
    def xxxw(self) -> IVector4: ...
    @property
    def xxxo(self) -> IVector4: ...
    @property
    def xxxl(self) -> IVector4: ...
    @property
    def xxyx(self) -> IVector4: ...
    @property
    def xxyy(self) -> IVector4: ...
    @property
    def xxyz(self) -> IVector4: ...
    @property
    def xxyw(self) -> IVector4: ...
    @property
    def xxyo(self) -> IVector4: ...
    @property
    def xxyl(self) -> IVector4: ...
    @property
    def xxzx(self) -> IVector4: ...
    @property
    def xxzy(self) -> IVector4: ...
    @property
    def xxzz(self) -> IVector4: ...
    @property
    def xxzw(self) -> IVector4: ...
    @property
    def xxzo(self) -> IVector4: ...
    @property
    def xxzl(self) -> IVector4: ...
    @property
    def xxwx(self) -> IVector4: ...
    @property
    def xxwy(self) -> IVector4: ...
    @property
    def xxwz(self) -> IVector4: ...
    @property
    def xxww(self) -> IVector4: ...
    @property
    def xxwo(self) -> IVector4: ...
    @property
    def xxwl(self) -> IVector4: ...
    @property
    def xxox(self) -> IVector4: ...
    @property
    def xxoy(self) -> IVector4: ...
    @property
    def xxoz(self) -> IVector4: ...
    @property
    def xxow(self) -> IVector4: ...
    @property
    def xxoo(self) -> IVector4: ...
    @property
    def xxol(self) -> IVector4: ...
    @property
    def xxlx(self) -> IVector4: ...
    @property
    def xxly(self) -> IVector4: ...
    @property
    def xxlz(self) -> IVector4: ...
    @property
    def xxlw(self) -> IVector4: ...
    @property
    def xxlo(self) -> IVector4: ...
    @property
    def xxll(self) -> IVector4: ...
    @property
    def xyxx(self) -> IVector4: ...
    @property
    def xyxy(self) -> IVector4: ...
    @property
    def xyxz(self) -> IVector4: ...
    @property
    def xyxw(self) -> IVector4: ...
    @property
    def xyxo(self) -> IVector4: ...
    @property
    def xyxl(self) -> IVector4: ...
    @property
    def xyyx(self) -> IVector4: ...
    @property
    def xyyy(self) -> IVector4: ...
    @property
    def xyyz(self) -> IVector4: ...
    @property
    def xyyw(self) -> IVector4: ...
    @property
    def xyyo(self) -> IVector4: ...
    @property
    def xyyl(self) -> IVector4: ...
    @property
    def xyzx(self) -> IVector4: ...
    @property
    def xyzy(self) -> IVector4: ...
    @property
    def xyzz(self) -> IVector4: ...
    @property
    def xyzw(self) -> IVector4: ...
    @property
    def xyzo(self) -> IVector4: ...
    @property
    def xyzl(self) -> IVector4: ...
    @property
    def xywx(self) -> IVector4: ...
    @property
    def xywy(self) -> IVector4: ...
    @property
    def xywz(self) -> IVector4: ...
    @property
    def xyww(self) -> IVector4: ...
    @property
    def xywo(self) -> IVector4: ...
    @property
    def xywl(self) -> IVector4: ...
    @property
    def xyox(self) -> IVector4: ...
    @property
    def xyoy(self) -> IVector4: ...
    @property
    def xyoz(self) -> IVector4: ...
    @property
    def xyow(self) -> IVector4: ...
    @property
    def xyoo(self) -> IVector4: ...
    @property
    def xyol(self) -> IVector4: ...
    @property
    def xylx(self) -> IVector4: ...
    @property
    def xyly(self) -> IVector4: ...
    @property
    def xylz(self) -> IVector4: ...
    @property
    def xylw(self) -> IVector4: ...
    @property
    def xylo(self) -> IVector4: ...
    @property
    def xyll(self) -> IVector4: ...
    @property
    def xzxx(self) -> IVector4: ...
    @property
    def xzxy(self) -> IVector4: ...
    @property
    def xzxz(self) -> IVector4: ...
    @property
    def xzxw(self) -> IVector4: ...
    @property
    def xzxo(self) -> IVector4: ...
    @property
    def xzxl(self) -> IVector4: ...
    @property
    def xzyx(self) -> IVector4: ...
    @property
    def xzyy(self) -> IVector4: ...
    @property
    def xzyz(self) -> IVector4: ...
    @property
    def xzyw(self) -> IVector4: ...
    @property
    def xzyo(self) -> IVector4: ...
    @property
    def xzyl(self) -> IVector4: ...
    @property
    def xzzx(self) -> IVector4: ...
    @property
    def xzzy(self) -> IVector4: ...
    @property
    def xzzz(self) -> IVector4: ...
    @property
    def xzzw(self) -> IVector4: ...
    @property
    def xzzo(self) -> IVector4: ...
    @property
    def xzzl(self) -> IVector4: ...
    @property
    def xzwx(self) -> IVector4: ...
    @property
    def xzwy(self) -> IVector4: ...
    @property
    def xzwz(self) -> IVector4: ...
    @property
    def xzww(self) -> IVector4: ...
    @property
    def xzwo(self) -> IVector4: ...
    @property
    def xzwl(self) -> IVector4: ...
    @property
    def xzox(self) -> IVector4: ...
    @property
    def xzoy(self) -> IVector4: ...
    @property
    def xzoz(self) -> IVector4: ...
    @property
    def xzow(self) -> IVector4: ...
    @property
    def xzoo(self) -> IVector4: ...
    @property
    def xzol(self) -> IVector4: ...
    @property
    def xzlx(self) -> IVector4: ...
    @property
    def xzly(self) -> IVector4: ...
    @property
    def xzlz(self) -> IVector4: ...
    @property
    def xzlw(self) -> IVector4: ...
    @property
    def xzlo(self) -> IVector4: ...
    @property
    def xzll(self) -> IVector4: ...
    @property
    def xwxx(self) -> IVector4: ...
    @property
    def xwxy(self) -> IVector4: ...
    @property
    def xwxz(self) -> IVector4: ...
    @property
    def xwxw(self) -> IVector4: ...
    @property
    def xwxo(self) -> IVector4: ...
    @property
    def xwxl(self) -> IVector4: ...
    @property
    def xwyx(self) -> IVector4: ...
    @property
    def xwyy(self) -> IVector4: ...
    @property
    def xwyz(self) -> IVector4: ...
    @property
    def xwyw(self) -> IVector4: ...
    @property
    def xwyo(self) -> IVector4: ...
    @property
    def xwyl(self) -> IVector4: ...
    @property
    def xwzx(self) -> IVector4: ...
    @property
    def xwzy(self) -> IVector4: ...
    @property
    def xwzz(self) -> IVector4: ...
    @property
    def xwzw(self) -> IVector4: ...
    @property
    def xwzo(self) -> IVector4: ...
    @property
    def xwzl(self) -> IVector4: ...
    @property
    def xwwx(self) -> IVector4: ...
    @property
    def xwwy(self) -> IVector4: ...
    @property
    def xwwz(self) -> IVector4: ...
    @property
    def xwww(self) -> IVector4: ...
    @property
    def xwwo(self) -> IVector4: ...
    @property
    def xwwl(self) -> IVector4: ...
    @property
    def xwox(self) -> IVector4: ...
    @property
    def xwoy(self) -> IVector4: ...
    @property
    def xwoz(self) -> IVector4: ...
    @property
    def xwow(self) -> IVector4: ...
    @property
    def xwoo(self) -> IVector4: ...
    @property
    def xwol(self) -> IVector4: ...
    @property
    def xwlx(self) -> IVector4: ...
    @property
    def xwly(self) -> IVector4: ...
    @property
    def xwlz(self) -> IVector4: ...
    @property
    def xwlw(self) -> IVector4: ...
    @property
    def xwlo(self) -> IVector4: ...
    @property
    def xwll(self) -> IVector4: ...
    @property
    def xoxx(self) -> IVector4: ...
    @property
    def xoxy(self) -> IVector4: ...
    @property
    def xoxz(self) -> IVector4: ...
    @property
    def xoxw(self) -> IVector4: ...
    @property
    def xoxo(self) -> IVector4: ...
    @property
    def xoxl(self) -> IVector4: ...
    @property
    def xoyx(self) -> IVector4: ...
    @property
    def xoyy(self) -> IVector4: ...
    @property
    def xoyz(self) -> IVector4: ...
    @property
    def xoyw(self) -> IVector4: ...
    @property
    def xoyo(self) -> IVector4: ...
    @property
    def xoyl(self) -> IVector4: ...
    @property
    def xozx(self) -> IVector4: ...
    @property
    def xozy(self) -> IVector4: ...
    @property
    def xozz(self) -> IVector4: ...
    @property
    def xozw(self) -> IVector4: ...
    @property
    def xozo(self) -> IVector4: ...
    @property
    def xozl(self) -> IVector4: ...
    @property
    def xowx(self) -> IVector4: ...
    @property
    def xowy(self) -> IVector4: ...
    @property
    def xowz(self) -> IVector4: ...
    @property
    def xoww(self) -> IVector4: ...
    @property
    def xowo(self) -> IVector4: ...
    @property
    def xowl(self) -> IVector4: ...
    @property
    def xoox(self) -> IVector4: ...
    @property
    def xooy(self) -> IVector4: ...
    @property
    def xooz(self) -> IVector4: ...
    @property
    def xoow(self) -> IVector4: ...
    @property
    def xooo(self) -> IVector4: ...
    @property
    def xool(self) -> IVector4: ...
    @property
    def xolx(self) -> IVector4: ...
    @property
    def xoly(self) -> IVector4: ...
    @property
    def xolz(self) -> IVector4: ...
    @property
    def xolw(self) -> IVector4: ...
    @property
    def xolo(self) -> IVector4: ...
    @property
    def xoll(self) -> IVector4: ...
    @property
    def xlxx(self) -> IVector4: ...
    @property
    def xlxy(self) -> IVector4: ...
    @property
    def xlxz(self) -> IVector4: ...
    @property
    def xlxw(self) -> IVector4: ...
    @property
    def xlxo(self) -> IVector4: ...
    @property
    def xlxl(self) -> IVector4: ...
    @property
    def xlyx(self) -> IVector4: ...
    @property
    def xlyy(self) -> IVector4: ...
    @property
    def xlyz(self) -> IVector4: ...
    @property
    def xlyw(self) -> IVector4: ...
    @property
    def xlyo(self) -> IVector4: ...
    @property
    def xlyl(self) -> IVector4: ...
    @property
    def xlzx(self) -> IVector4: ...
    @property
    def xlzy(self) -> IVector4: ...
    @property
    def xlzz(self) -> IVector4: ...
    @property
    def xlzw(self) -> IVector4: ...
    @property
    def xlzo(self) -> IVector4: ...
    @property
    def xlzl(self) -> IVector4: ...
    @property
    def xlwx(self) -> IVector4: ...
    @property
    def xlwy(self) -> IVector4: ...
    @property
    def xlwz(self) -> IVector4: ...
    @property
    def xlww(self) -> IVector4: ...
    @property
    def xlwo(self) -> IVector4: ...
    @property
    def xlwl(self) -> IVector4: ...
    @property
    def xlox(self) -> IVector4: ...
    @property
    def xloy(self) -> IVector4: ...
    @property
    def xloz(self) -> IVector4: ...
    @property
    def xlow(self) -> IVector4: ...
    @property
    def xloo(self) -> IVector4: ...
    @property
    def xlol(self) -> IVector4: ...
    @property
    def xllx(self) -> IVector4: ...
    @property
    def xlly(self) -> IVector4: ...
    @property
    def xllz(self) -> IVector4: ...
    @property
    def xllw(self) -> IVector4: ...
    @property
    def xllo(self) -> IVector4: ...
    @property
    def xlll(self) -> IVector4: ...
    @property
    def yxxx(self) -> IVector4: ...
    @property
    def yxxy(self) -> IVector4: ...
    @property
    def yxxz(self) -> IVector4: ...
    @property
    def yxxw(self) -> IVector4: ...
    @property
    def yxxo(self) -> IVector4: ...
    @property
    def yxxl(self) -> IVector4: ...
    @property
    def yxyx(self) -> IVector4: ...
    @property
    def yxyy(self) -> IVector4: ...
    @property
    def yxyz(self) -> IVector4: ...
    @property
    def yxyw(self) -> IVector4: ...
    @property
    def yxyo(self) -> IVector4: ...
    @property
    def yxyl(self) -> IVector4: ...
    @property
    def yxzx(self) -> IVector4: ...
    @property
    def yxzy(self) -> IVector4: ...
    @property
    def yxzz(self) -> IVector4: ...
    @property
    def yxzw(self) -> IVector4: ...
    @property
    def yxzo(self) -> IVector4: ...
    @property
    def yxzl(self) -> IVector4: ...
    @property
    def yxwx(self) -> IVector4: ...
    @property
    def yxwy(self) -> IVector4: ...
    @property
    def yxwz(self) -> IVector4: ...
    @property
    def yxww(self) -> IVector4: ...
    @property
    def yxwo(self) -> IVector4: ...
    @property
    def yxwl(self) -> IVector4: ...
    @property
    def yxox(self) -> IVector4: ...
    @property
    def yxoy(self) -> IVector4: ...
    @property
    def yxoz(self) -> IVector4: ...
    @property
    def yxow(self) -> IVector4: ...
    @property
    def yxoo(self) -> IVector4: ...
    @property
    def yxol(self) -> IVector4: ...
    @property
    def yxlx(self) -> IVector4: ...
    @property
    def yxly(self) -> IVector4: ...
    @property
    def yxlz(self) -> IVector4: ...
    @property
    def yxlw(self) -> IVector4: ...
    @property
    def yxlo(self) -> IVector4: ...
    @property
    def yxll(self) -> IVector4: ...
    @property
    def yyxx(self) -> IVector4: ...
    @property
    def yyxy(self) -> IVector4: ...
    @property
    def yyxz(self) -> IVector4: ...
    @property
    def yyxw(self) -> IVector4: ...
    @property
    def yyxo(self) -> IVector4: ...
    @property
    def yyxl(self) -> IVector4: ...
    @property
    def yyyx(self) -> IVector4: ...
    @property
    def yyyy(self) -> IVector4: ...
    @property
    def yyyz(self) -> IVector4: ...
    @property
    def yyyw(self) -> IVector4: ...
    @property
    def yyyo(self) -> IVector4: ...
    @property
    def yyyl(self) -> IVector4: ...
    @property
    def yyzx(self) -> IVector4: ...
    @property
    def yyzy(self) -> IVector4: ...
    @property
    def yyzz(self) -> IVector4: ...
    @property
    def yyzw(self) -> IVector4: ...
    @property
    def yyzo(self) -> IVector4: ...
    @property
    def yyzl(self) -> IVector4: ...
    @property
    def yywx(self) -> IVector4: ...
    @property
    def yywy(self) -> IVector4: ...
    @property
    def yywz(self) -> IVector4: ...
    @property
    def yyww(self) -> IVector4: ...
    @property
    def yywo(self) -> IVector4: ...
    @property
    def yywl(self) -> IVector4: ...
    @property
    def yyox(self) -> IVector4: ...
    @property
    def yyoy(self) -> IVector4: ...
    @property
    def yyoz(self) -> IVector4: ...
    @property
    def yyow(self) -> IVector4: ...
    @property
    def yyoo(self) -> IVector4: ...
    @property
    def yyol(self) -> IVector4: ...
    @property
    def yylx(self) -> IVector4: ...
    @property
    def yyly(self) -> IVector4: ...
    @property
    def yylz(self) -> IVector4: ...
    @property
    def yylw(self) -> IVector4: ...
    @property
    def yylo(self) -> IVector4: ...
    @property
    def yyll(self) -> IVector4: ...
    @property
    def yzxx(self) -> IVector4: ...
    @property
    def yzxy(self) -> IVector4: ...
    @property
    def yzxz(self) -> IVector4: ...
    @property
    def yzxw(self) -> IVector4: ...
    @property
    def yzxo(self) -> IVector4: ...
    @property
    def yzxl(self) -> IVector4: ...
    @property
    def yzyx(self) -> IVector4: ...
    @property
    def yzyy(self) -> IVector4: ...
    @property
    def yzyz(self) -> IVector4: ...
    @property
    def yzyw(self) -> IVector4: ...
    @property
    def yzyo(self) -> IVector4: ...
    @property
    def yzyl(self) -> IVector4: ...
    @property
    def yzzx(self) -> IVector4: ...
    @property
    def yzzy(self) -> IVector4: ...
    @property
    def yzzz(self) -> IVector4: ...
    @property
    def yzzw(self) -> IVector4: ...
    @property
    def yzzo(self) -> IVector4: ...
    @property
    def yzzl(self) -> IVector4: ...
    @property
    def yzwx(self) -> IVector4: ...
    @property
    def yzwy(self) -> IVector4: ...
    @property
    def yzwz(self) -> IVector4: ...
    @property
    def yzww(self) -> IVector4: ...
    @property
    def yzwo(self) -> IVector4: ...
    @property
    def yzwl(self) -> IVector4: ...
    @property
    def yzox(self) -> IVector4: ...
    @property
    def yzoy(self) -> IVector4: ...
    @property
    def yzoz(self) -> IVector4: ...
    @property
    def yzow(self) -> IVector4: ...
    @property
    def yzoo(self) -> IVector4: ...
    @property
    def yzol(self) -> IVector4: ...
    @property
    def yzlx(self) -> IVector4: ...
    @property
    def yzly(self) -> IVector4: ...
    @property
    def yzlz(self) -> IVector4: ...
    @property
    def yzlw(self) -> IVector4: ...
    @property
    def yzlo(self) -> IVector4: ...
    @property
    def yzll(self) -> IVector4: ...
    @property
    def ywxx(self) -> IVector4: ...
    @property
    def ywxy(self) -> IVector4: ...
    @property
    def ywxz(self) -> IVector4: ...
    @property
    def ywxw(self) -> IVector4: ...
    @property
    def ywxo(self) -> IVector4: ...
    @property
    def ywxl(self) -> IVector4: ...
    @property
    def ywyx(self) -> IVector4: ...
    @property
    def ywyy(self) -> IVector4: ...
    @property
    def ywyz(self) -> IVector4: ...
    @property
    def ywyw(self) -> IVector4: ...
    @property
    def ywyo(self) -> IVector4: ...
    @property
    def ywyl(self) -> IVector4: ...
    @property
    def ywzx(self) -> IVector4: ...
    @property
    def ywzy(self) -> IVector4: ...
    @property
    def ywzz(self) -> IVector4: ...
    @property
    def ywzw(self) -> IVector4: ...
    @property
    def ywzo(self) -> IVector4: ...
    @property
    def ywzl(self) -> IVector4: ...
    @property
    def ywwx(self) -> IVector4: ...
    @property
    def ywwy(self) -> IVector4: ...
    @property
    def ywwz(self) -> IVector4: ...
    @property
    def ywww(self) -> IVector4: ...
    @property
    def ywwo(self) -> IVector4: ...
    @property
    def ywwl(self) -> IVector4: ...
    @property
    def ywox(self) -> IVector4: ...
    @property
    def ywoy(self) -> IVector4: ...
    @property
    def ywoz(self) -> IVector4: ...
    @property
    def ywow(self) -> IVector4: ...
    @property
    def ywoo(self) -> IVector4: ...
    @property
    def ywol(self) -> IVector4: ...
    @property
    def ywlx(self) -> IVector4: ...
    @property
    def ywly(self) -> IVector4: ...
    @property
    def ywlz(self) -> IVector4: ...
    @property
    def ywlw(self) -> IVector4: ...
    @property
    def ywlo(self) -> IVector4: ...
    @property
    def ywll(self) -> IVector4: ...
    @property
    def yoxx(self) -> IVector4: ...
    @property
    def yoxy(self) -> IVector4: ...
    @property
    def yoxz(self) -> IVector4: ...
    @property
    def yoxw(self) -> IVector4: ...
    @property
    def yoxo(self) -> IVector4: ...
    @property
    def yoxl(self) -> IVector4: ...
    @property
    def yoyx(self) -> IVector4: ...
    @property
    def yoyy(self) -> IVector4: ...
    @property
    def yoyz(self) -> IVector4: ...
    @property
    def yoyw(self) -> IVector4: ...
    @property
    def yoyo(self) -> IVector4: ...
    @property
    def yoyl(self) -> IVector4: ...
    @property
    def yozx(self) -> IVector4: ...
    @property
    def yozy(self) -> IVector4: ...
    @property
    def yozz(self) -> IVector4: ...
    @property
    def yozw(self) -> IVector4: ...
    @property
    def yozo(self) -> IVector4: ...
    @property
    def yozl(self) -> IVector4: ...
    @property
    def yowx(self) -> IVector4: ...
    @property
    def yowy(self) -> IVector4: ...
    @property
    def yowz(self) -> IVector4: ...
    @property
    def yoww(self) -> IVector4: ...
    @property
    def yowo(self) -> IVector4: ...
    @property
    def yowl(self) -> IVector4: ...
    @property
    def yoox(self) -> IVector4: ...
    @property
    def yooy(self) -> IVector4: ...
    @property
    def yooz(self) -> IVector4: ...
    @property
    def yoow(self) -> IVector4: ...
    @property
    def yooo(self) -> IVector4: ...
    @property
    def yool(self) -> IVector4: ...
    @property
    def yolx(self) -> IVector4: ...
    @property
    def yoly(self) -> IVector4: ...
    @property
    def yolz(self) -> IVector4: ...
    @property
    def yolw(self) -> IVector4: ...
    @property
    def yolo(self) -> IVector4: ...
    @property
    def yoll(self) -> IVector4: ...
    @property
    def ylxx(self) -> IVector4: ...
    @property
    def ylxy(self) -> IVector4: ...
    @property
    def ylxz(self) -> IVector4: ...
    @property
    def ylxw(self) -> IVector4: ...
    @property
    def ylxo(self) -> IVector4: ...
    @property
    def ylxl(self) -> IVector4: ...
    @property
    def ylyx(self) -> IVector4: ...
    @property
    def ylyy(self) -> IVector4: ...
    @property
    def ylyz(self) -> IVector4: ...
    @property
    def ylyw(self) -> IVector4: ...
    @property
    def ylyo(self) -> IVector4: ...
    @property
    def ylyl(self) -> IVector4: ...
    @property
    def ylzx(self) -> IVector4: ...
    @property
    def ylzy(self) -> IVector4: ...
    @property
    def ylzz(self) -> IVector4: ...
    @property
    def ylzw(self) -> IVector4: ...
    @property
    def ylzo(self) -> IVector4: ...
    @property
    def ylzl(self) -> IVector4: ...
    @property
    def ylwx(self) -> IVector4: ...
    @property
    def ylwy(self) -> IVector4: ...
    @property
    def ylwz(self) -> IVector4: ...
    @property
    def ylww(self) -> IVector4: ...
    @property
    def ylwo(self) -> IVector4: ...
    @property
    def ylwl(self) -> IVector4: ...
    @property
    def ylox(self) -> IVector4: ...
    @property
    def yloy(self) -> IVector4: ...
    @property
    def yloz(self) -> IVector4: ...
    @property
    def ylow(self) -> IVector4: ...
    @property
    def yloo(self) -> IVector4: ...
    @property
    def ylol(self) -> IVector4: ...
    @property
    def yllx(self) -> IVector4: ...
    @property
    def ylly(self) -> IVector4: ...
    @property
    def yllz(self) -> IVector4: ...
    @property
    def yllw(self) -> IVector4: ...
    @property
    def yllo(self) -> IVector4: ...
    @property
    def ylll(self) -> IVector4: ...
    @property
    def zxxx(self) -> IVector4: ...
    @property
    def zxxy(self) -> IVector4: ...
    @property
    def zxxz(self) -> IVector4: ...
    @property
    def zxxw(self) -> IVector4: ...
    @property
    def zxxo(self) -> IVector4: ...
    @property
    def zxxl(self) -> IVector4: ...
    @property
    def zxyx(self) -> IVector4: ...
    @property
    def zxyy(self) -> IVector4: ...
    @property
    def zxyz(self) -> IVector4: ...
    @property
    def zxyw(self) -> IVector4: ...
    @property
    def zxyo(self) -> IVector4: ...
    @property
    def zxyl(self) -> IVector4: ...
    @property
    def zxzx(self) -> IVector4: ...
    @property
    def zxzy(self) -> IVector4: ...
    @property
    def zxzz(self) -> IVector4: ...
    @property
    def zxzw(self) -> IVector4: ...
    @property
    def zxzo(self) -> IVector4: ...
    @property
    def zxzl(self) -> IVector4: ...
    @property
    def zxwx(self) -> IVector4: ...
    @property
    def zxwy(self) -> IVector4: ...
    @property
    def zxwz(self) -> IVector4: ...
    @property
    def zxww(self) -> IVector4: ...
    @property
    def zxwo(self) -> IVector4: ...
    @property
    def zxwl(self) -> IVector4: ...
    @property
    def zxox(self) -> IVector4: ...
    @property
    def zxoy(self) -> IVector4: ...
    @property
    def zxoz(self) -> IVector4: ...
    @property
    def zxow(self) -> IVector4: ...
    @property
    def zxoo(self) -> IVector4: ...
    @property
    def zxol(self) -> IVector4: ...
    @property
    def zxlx(self) -> IVector4: ...
    @property
    def zxly(self) -> IVector4: ...
    @property
    def zxlz(self) -> IVector4: ...
    @property
    def zxlw(self) -> IVector4: ...
    @property
    def zxlo(self) -> IVector4: ...
    @property
    def zxll(self) -> IVector4: ...
    @property
    def zyxx(self) -> IVector4: ...
    @property
    def zyxy(self) -> IVector4: ...
    @property
    def zyxz(self) -> IVector4: ...
    @property
    def zyxw(self) -> IVector4: ...
    @property
    def zyxo(self) -> IVector4: ...
    @property
    def zyxl(self) -> IVector4: ...
    @property
    def zyyx(self) -> IVector4: ...
    @property
    def zyyy(self) -> IVector4: ...
    @property
    def zyyz(self) -> IVector4: ...
    @property
    def zyyw(self) -> IVector4: ...
    @property
    def zyyo(self) -> IVector4: ...
    @property
    def zyyl(self) -> IVector4: ...
    @property
    def zyzx(self) -> IVector4: ...
    @property
    def zyzy(self) -> IVector4: ...
    @property
    def zyzz(self) -> IVector4: ...
    @property
    def zyzw(self) -> IVector4: ...
    @property
    def zyzo(self) -> IVector4: ...
    @property
    def zyzl(self) -> IVector4: ...
    @property
    def zywx(self) -> IVector4: ...
    @property
    def zywy(self) -> IVector4: ...
    @property
    def zywz(self) -> IVector4: ...
    @property
    def zyww(self) -> IVector4: ...
    @property
    def zywo(self) -> IVector4: ...
    @property
    def zywl(self) -> IVector4: ...
    @property
    def zyox(self) -> IVector4: ...
    @property
    def zyoy(self) -> IVector4: ...
    @property
    def zyoz(self) -> IVector4: ...
    @property
    def zyow(self) -> IVector4: ...
    @property
    def zyoo(self) -> IVector4: ...
    @property
    def zyol(self) -> IVector4: ...
    @property
    def zylx(self) -> IVector4: ...
    @property
    def zyly(self) -> IVector4: ...
    @property
    def zylz(self) -> IVector4: ...
    @property
    def zylw(self) -> IVector4: ...
    @property
    def zylo(self) -> IVector4: ...
    @property
    def zyll(self) -> IVector4: ...
    @property
    def zzxx(self) -> IVector4: ...
    @property
    def zzxy(self) -> IVector4: ...
    @property
    def zzxz(self) -> IVector4: ...
    @property
    def zzxw(self) -> IVector4: ...
    @property
    def zzxo(self) -> IVector4: ...
    @property
    def zzxl(self) -> IVector4: ...
    @property
    def zzyx(self) -> IVector4: ...
    @property
    def zzyy(self) -> IVector4: ...
    @property
    def zzyz(self) -> IVector4: ...
    @property
    def zzyw(self) -> IVector4: ...
    @property
    def zzyo(self) -> IVector4: ...
    @property
    def zzyl(self) -> IVector4: ...
    @property
    def zzzx(self) -> IVector4: ...
    @property
    def zzzy(self) -> IVector4: ...
    @property
    def zzzz(self) -> IVector4: ...
    @property
    def zzzw(self) -> IVector4: ...
    @property
    def zzzo(self) -> IVector4: ...
    @property
    def zzzl(self) -> IVector4: ...
    @property
    def zzwx(self) -> IVector4: ...
    @property
    def zzwy(self) -> IVector4: ...
    @property
    def zzwz(self) -> IVector4: ...
    @property
    def zzww(self) -> IVector4: ...
    @property
    def zzwo(self) -> IVector4: ...
    @property
    def zzwl(self) -> IVector4: ...
    @property
    def zzox(self) -> IVector4: ...
    @property
    def zzoy(self) -> IVector4: ...
    @property
    def zzoz(self) -> IVector4: ...
    @property
    def zzow(self) -> IVector4: ...
    @property
    def zzoo(self) -> IVector4: ...
    @property
    def zzol(self) -> IVector4: ...
    @property
    def zzlx(self) -> IVector4: ...
    @property
    def zzly(self) -> IVector4: ...
    @property
    def zzlz(self) -> IVector4: ...
    @property
    def zzlw(self) -> IVector4: ...
    @property
    def zzlo(self) -> IVector4: ...
    @property
    def zzll(self) -> IVector4: ...
    @property
    def zwxx(self) -> IVector4: ...
    @property
    def zwxy(self) -> IVector4: ...
    @property
    def zwxz(self) -> IVector4: ...
    @property
    def zwxw(self) -> IVector4: ...
    @property
    def zwxo(self) -> IVector4: ...
    @property
    def zwxl(self) -> IVector4: ...
    @property
    def zwyx(self) -> IVector4: ...
    @property
    def zwyy(self) -> IVector4: ...
    @property
    def zwyz(self) -> IVector4: ...
    @property
    def zwyw(self) -> IVector4: ...
    @property
    def zwyo(self) -> IVector4: ...
    @property
    def zwyl(self) -> IVector4: ...
    @property
    def zwzx(self) -> IVector4: ...
    @property
    def zwzy(self) -> IVector4: ...
    @property
    def zwzz(self) -> IVector4: ...
    @property
    def zwzw(self) -> IVector4: ...
    @property
    def zwzo(self) -> IVector4: ...
    @property
    def zwzl(self) -> IVector4: ...
    @property
    def zwwx(self) -> IVector4: ...
    @property
    def zwwy(self) -> IVector4: ...
    @property
    def zwwz(self) -> IVector4: ...
    @property
    def zwww(self) -> IVector4: ...
    @property
    def zwwo(self) -> IVector4: ...
    @property
    def zwwl(self) -> IVector4: ...
    @property
    def zwox(self) -> IVector4: ...
    @property
    def zwoy(self) -> IVector4: ...
    @property
    def zwoz(self) -> IVector4: ...
    @property
    def zwow(self) -> IVector4: ...
    @property
    def zwoo(self) -> IVector4: ...
    @property
    def zwol(self) -> IVector4: ...
    @property
    def zwlx(self) -> IVector4: ...
    @property
    def zwly(self) -> IVector4: ...
    @property
    def zwlz(self) -> IVector4: ...
    @property
    def zwlw(self) -> IVector4: ...
    @property
    def zwlo(self) -> IVector4: ...
    @property
    def zwll(self) -> IVector4: ...
    @property
    def zoxx(self) -> IVector4: ...
    @property
    def zoxy(self) -> IVector4: ...
    @property
    def zoxz(self) -> IVector4: ...
    @property
    def zoxw(self) -> IVector4: ...
    @property
    def zoxo(self) -> IVector4: ...
    @property
    def zoxl(self) -> IVector4: ...
    @property
    def zoyx(self) -> IVector4: ...
    @property
    def zoyy(self) -> IVector4: ...
    @property
    def zoyz(self) -> IVector4: ...
    @property
    def zoyw(self) -> IVector4: ...
    @property
    def zoyo(self) -> IVector4: ...
    @property
    def zoyl(self) -> IVector4: ...
    @property
    def zozx(self) -> IVector4: ...
    @property
    def zozy(self) -> IVector4: ...
    @property
    def zozz(self) -> IVector4: ...
    @property
    def zozw(self) -> IVector4: ...
    @property
    def zozo(self) -> IVector4: ...
    @property
    def zozl(self) -> IVector4: ...
    @property
    def zowx(self) -> IVector4: ...
    @property
    def zowy(self) -> IVector4: ...
    @property
    def zowz(self) -> IVector4: ...
    @property
    def zoww(self) -> IVector4: ...
    @property
    def zowo(self) -> IVector4: ...
    @property
    def zowl(self) -> IVector4: ...
    @property
    def zoox(self) -> IVector4: ...
    @property
    def zooy(self) -> IVector4: ...
    @property
    def zooz(self) -> IVector4: ...
    @property
    def zoow(self) -> IVector4: ...
    @property
    def zooo(self) -> IVector4: ...
    @property
    def zool(self) -> IVector4: ...
    @property
    def zolx(self) -> IVector4: ...
    @property
    def zoly(self) -> IVector4: ...
    @property
    def zolz(self) -> IVector4: ...
    @property
    def zolw(self) -> IVector4: ...
    @property
    def zolo(self) -> IVector4: ...
    @property
    def zoll(self) -> IVector4: ...
    @property
    def zlxx(self) -> IVector4: ...
    @property
    def zlxy(self) -> IVector4: ...
    @property
    def zlxz(self) -> IVector4: ...
    @property
    def zlxw(self) -> IVector4: ...
    @property
    def zlxo(self) -> IVector4: ...
    @property
    def zlxl(self) -> IVector4: ...
    @property
    def zlyx(self) -> IVector4: ...
    @property
    def zlyy(self) -> IVector4: ...
    @property
    def zlyz(self) -> IVector4: ...
    @property
    def zlyw(self) -> IVector4: ...
    @property
    def zlyo(self) -> IVector4: ...
    @property
    def zlyl(self) -> IVector4: ...
    @property
    def zlzx(self) -> IVector4: ...
    @property
    def zlzy(self) -> IVector4: ...
    @property
    def zlzz(self) -> IVector4: ...
    @property
    def zlzw(self) -> IVector4: ...
    @property
    def zlzo(self) -> IVector4: ...
    @property
    def zlzl(self) -> IVector4: ...
    @property
    def zlwx(self) -> IVector4: ...
    @property
    def zlwy(self) -> IVector4: ...
    @property
    def zlwz(self) -> IVector4: ...
    @property
    def zlww(self) -> IVector4: ...
    @property
    def zlwo(self) -> IVector4: ...
    @property
    def zlwl(self) -> IVector4: ...
    @property
    def zlox(self) -> IVector4: ...
    @property
    def zloy(self) -> IVector4: ...
    @property
    def zloz(self) -> IVector4: ...
    @property
    def zlow(self) -> IVector4: ...
    @property
    def zloo(self) -> IVector4: ...
    @property
    def zlol(self) -> IVector4: ...
    @property
    def zllx(self) -> IVector4: ...
    @property
    def zlly(self) -> IVector4: ...
    @property
    def zllz(self) -> IVector4: ...
    @property
    def zllw(self) -> IVector4: ...
    @property
    def zllo(self) -> IVector4: ...
    @property
    def zlll(self) -> IVector4: ...
    @property
    def wxxx(self) -> IVector4: ...
    @property
    def wxxy(self) -> IVector4: ...
    @property
    def wxxz(self) -> IVector4: ...
    @property
    def wxxw(self) -> IVector4: ...
    @property
    def wxxo(self) -> IVector4: ...
    @property
    def wxxl(self) -> IVector4: ...
    @property
    def wxyx(self) -> IVector4: ...
    @property
    def wxyy(self) -> IVector4: ...
    @property
    def wxyz(self) -> IVector4: ...
    @property
    def wxyw(self) -> IVector4: ...
    @property
    def wxyo(self) -> IVector4: ...
    @property
    def wxyl(self) -> IVector4: ...
    @property
    def wxzx(self) -> IVector4: ...
    @property
    def wxzy(self) -> IVector4: ...
    @property
    def wxzz(self) -> IVector4: ...
    @property
    def wxzw(self) -> IVector4: ...
    @property
    def wxzo(self) -> IVector4: ...
    @property
    def wxzl(self) -> IVector4: ...
    @property
    def wxwx(self) -> IVector4: ...
    @property
    def wxwy(self) -> IVector4: ...
    @property
    def wxwz(self) -> IVector4: ...
    @property
    def wxww(self) -> IVector4: ...
    @property
    def wxwo(self) -> IVector4: ...
    @property
    def wxwl(self) -> IVector4: ...
    @property
    def wxox(self) -> IVector4: ...
    @property
    def wxoy(self) -> IVector4: ...
    @property
    def wxoz(self) -> IVector4: ...
    @property
    def wxow(self) -> IVector4: ...
    @property
    def wxoo(self) -> IVector4: ...
    @property
    def wxol(self) -> IVector4: ...
    @property
    def wxlx(self) -> IVector4: ...
    @property
    def wxly(self) -> IVector4: ...
    @property
    def wxlz(self) -> IVector4: ...
    @property
    def wxlw(self) -> IVector4: ...
    @property
    def wxlo(self) -> IVector4: ...
    @property
    def wxll(self) -> IVector4: ...
    @property
    def wyxx(self) -> IVector4: ...
    @property
    def wyxy(self) -> IVector4: ...
    @property
    def wyxz(self) -> IVector4: ...
    @property
    def wyxw(self) -> IVector4: ...
    @property
    def wyxo(self) -> IVector4: ...
    @property
    def wyxl(self) -> IVector4: ...
    @property
    def wyyx(self) -> IVector4: ...
    @property
    def wyyy(self) -> IVector4: ...
    @property
    def wyyz(self) -> IVector4: ...
    @property
    def wyyw(self) -> IVector4: ...
    @property
    def wyyo(self) -> IVector4: ...
    @property
    def wyyl(self) -> IVector4: ...
    @property
    def wyzx(self) -> IVector4: ...
    @property
    def wyzy(self) -> IVector4: ...
    @property
    def wyzz(self) -> IVector4: ...
    @property
    def wyzw(self) -> IVector4: ...
    @property
    def wyzo(self) -> IVector4: ...
    @property
    def wyzl(self) -> IVector4: ...
    @property
    def wywx(self) -> IVector4: ...
    @property
    def wywy(self) -> IVector4: ...
    @property
    def wywz(self) -> IVector4: ...
    @property
    def wyww(self) -> IVector4: ...
    @property
    def wywo(self) -> IVector4: ...
    @property
    def wywl(self) -> IVector4: ...
    @property
    def wyox(self) -> IVector4: ...
    @property
    def wyoy(self) -> IVector4: ...
    @property
    def wyoz(self) -> IVector4: ...
    @property
    def wyow(self) -> IVector4: ...
    @property
    def wyoo(self) -> IVector4: ...
    @property
    def wyol(self) -> IVector4: ...
    @property
    def wylx(self) -> IVector4: ...
    @property
    def wyly(self) -> IVector4: ...
    @property
    def wylz(self) -> IVector4: ...
    @property
    def wylw(self) -> IVector4: ...
    @property
    def wylo(self) -> IVector4: ...
    @property
    def wyll(self) -> IVector4: ...
    @property
    def wzxx(self) -> IVector4: ...
    @property
    def wzxy(self) -> IVector4: ...
    @property
    def wzxz(self) -> IVector4: ...
    @property
    def wzxw(self) -> IVector4: ...
    @property
    def wzxo(self) -> IVector4: ...
    @property
    def wzxl(self) -> IVector4: ...
    @property
    def wzyx(self) -> IVector4: ...
    @property
    def wzyy(self) -> IVector4: ...
    @property
    def wzyz(self) -> IVector4: ...
    @property
    def wzyw(self) -> IVector4: ...
    @property
    def wzyo(self) -> IVector4: ...
    @property
    def wzyl(self) -> IVector4: ...
    @property
    def wzzx(self) -> IVector4: ...
    @property
    def wzzy(self) -> IVector4: ...
    @property
    def wzzz(self) -> IVector4: ...
    @property
    def wzzw(self) -> IVector4: ...
    @property
    def wzzo(self) -> IVector4: ...
    @property
    def wzzl(self) -> IVector4: ...
    @property
    def wzwx(self) -> IVector4: ...
    @property
    def wzwy(self) -> IVector4: ...
    @property
    def wzwz(self) -> IVector4: ...
    @property
    def wzww(self) -> IVector4: ...
    @property
    def wzwo(self) -> IVector4: ...
    @property
    def wzwl(self) -> IVector4: ...
    @property
    def wzox(self) -> IVector4: ...
    @property
    def wzoy(self) -> IVector4: ...
    @property
    def wzoz(self) -> IVector4: ...
    @property
    def wzow(self) -> IVector4: ...
    @property
    def wzoo(self) -> IVector4: ...
    @property
    def wzol(self) -> IVector4: ...
    @property
    def wzlx(self) -> IVector4: ...
    @property
    def wzly(self) -> IVector4: ...
    @property
    def wzlz(self) -> IVector4: ...
    @property
    def wzlw(self) -> IVector4: ...
    @property
    def wzlo(self) -> IVector4: ...
    @property
    def wzll(self) -> IVector4: ...
    @property
    def wwxx(self) -> IVector4: ...
    @property
    def wwxy(self) -> IVector4: ...
    @property
    def wwxz(self) -> IVector4: ...
    @property
    def wwxw(self) -> IVector4: ...
    @property
    def wwxo(self) -> IVector4: ...
    @property
    def wwxl(self) -> IVector4: ...
    @property
    def wwyx(self) -> IVector4: ...
    @property
    def wwyy(self) -> IVector4: ...
    @property
    def wwyz(self) -> IVector4: ...
    @property
    def wwyw(self) -> IVector4: ...
    @property
    def wwyo(self) -> IVector4: ...
    @property
    def wwyl(self) -> IVector4: ...
    @property
    def wwzx(self) -> IVector4: ...
    @property
    def wwzy(self) -> IVector4: ...
    @property
    def wwzz(self) -> IVector4: ...
    @property
    def wwzw(self) -> IVector4: ...
    @property
    def wwzo(self) -> IVector4: ...
    @property
    def wwzl(self) -> IVector4: ...
    @property
    def wwwx(self) -> IVector4: ...
    @property
    def wwwy(self) -> IVector4: ...
    @property
    def wwwz(self) -> IVector4: ...
    @property
    def wwww(self) -> IVector4: ...
    @property
    def wwwo(self) -> IVector4: ...
    @property
    def wwwl(self) -> IVector4: ...
    @property
    def wwox(self) -> IVector4: ...
    @property
    def wwoy(self) -> IVector4: ...
    @property
    def wwoz(self) -> IVector4: ...
    @property
    def wwow(self) -> IVector4: ...
    @property
    def wwoo(self) -> IVector4: ...
    @property
    def wwol(self) -> IVector4: ...
    @property
    def wwlx(self) -> IVector4: ...
    @property
    def wwly(self) -> IVector4: ...
    @property
    def wwlz(self) -> IVector4: ...
    @property
    def wwlw(self) -> IVector4: ...
    @property
    def wwlo(self) -> IVector4: ...
    @property
    def wwll(self) -> IVector4: ...
    @property
    def woxx(self) -> IVector4: ...
    @property
    def woxy(self) -> IVector4: ...
    @property
    def woxz(self) -> IVector4: ...
    @property
    def woxw(self) -> IVector4: ...
    @property
    def woxo(self) -> IVector4: ...
    @property
    def woxl(self) -> IVector4: ...
    @property
    def woyx(self) -> IVector4: ...
    @property
    def woyy(self) -> IVector4: ...
    @property
    def woyz(self) -> IVector4: ...
    @property
    def woyw(self) -> IVector4: ...
    @property
    def woyo(self) -> IVector4: ...
    @property
    def woyl(self) -> IVector4: ...
    @property
    def wozx(self) -> IVector4: ...
    @property
    def wozy(self) -> IVector4: ...
    @property
    def wozz(self) -> IVector4: ...
    @property
    def wozw(self) -> IVector4: ...
    @property
    def wozo(self) -> IVector4: ...
    @property
    def wozl(self) -> IVector4: ...
    @property
    def wowx(self) -> IVector4: ...
    @property
    def wowy(self) -> IVector4: ...
    @property
    def wowz(self) -> IVector4: ...
    @property
    def woww(self) -> IVector4: ...
    @property
    def wowo(self) -> IVector4: ...
    @property
    def wowl(self) -> IVector4: ...
    @property
    def woox(self) -> IVector4: ...
    @property
    def wooy(self) -> IVector4: ...
    @property
    def wooz(self) -> IVector4: ...
    @property
    def woow(self) -> IVector4: ...
    @property
    def wooo(self) -> IVector4: ...
    @property
    def wool(self) -> IVector4: ...
    @property
    def wolx(self) -> IVector4: ...
    @property
    def woly(self) -> IVector4: ...
    @property
    def wolz(self) -> IVector4: ...
    @property
    def wolw(self) -> IVector4: ...
    @property
    def wolo(self) -> IVector4: ...
    @property
    def woll(self) -> IVector4: ...
    @property
    def wlxx(self) -> IVector4: ...
    @property
    def wlxy(self) -> IVector4: ...
    @property
    def wlxz(self) -> IVector4: ...
    @property
    def wlxw(self) -> IVector4: ...
    @property
    def wlxo(self) -> IVector4: ...
    @property
    def wlxl(self) -> IVector4: ...
    @property
    def wlyx(self) -> IVector4: ...
    @property
    def wlyy(self) -> IVector4: ...
    @property
    def wlyz(self) -> IVector4: ...
    @property
    def wlyw(self) -> IVector4: ...
    @property
    def wlyo(self) -> IVector4: ...
    @property
    def wlyl(self) -> IVector4: ...
    @property
    def wlzx(self) -> IVector4: ...
    @property
    def wlzy(self) -> IVector4: ...
    @property
    def wlzz(self) -> IVector4: ...
    @property
    def wlzw(self) -> IVector4: ...
    @property
    def wlzo(self) -> IVector4: ...
    @property
    def wlzl(self) -> IVector4: ...
    @property
    def wlwx(self) -> IVector4: ...
    @property
    def wlwy(self) -> IVector4: ...
    @property
    def wlwz(self) -> IVector4: ...
    @property
    def wlww(self) -> IVector4: ...
    @property
    def wlwo(self) -> IVector4: ...
    @property
    def wlwl(self) -> IVector4: ...
    @property
    def wlox(self) -> IVector4: ...
    @property
    def wloy(self) -> IVector4: ...
    @property
    def wloz(self) -> IVector4: ...
    @property
    def wlow(self) -> IVector4: ...
    @property
    def wloo(self) -> IVector4: ...
    @property
    def wlol(self) -> IVector4: ...
    @property
    def wllx(self) -> IVector4: ...
    @property
    def wlly(self) -> IVector4: ...
    @property
    def wllz(self) -> IVector4: ...
    @property
    def wllw(self) -> IVector4: ...
    @property
    def wllo(self) -> IVector4: ...
    @property
    def wlll(self) -> IVector4: ...
    @property
    def oxxx(self) -> IVector4: ...
    @property
    def oxxy(self) -> IVector4: ...
    @property
    def oxxz(self) -> IVector4: ...
    @property
    def oxxw(self) -> IVector4: ...
    @property
    def oxxo(self) -> IVector4: ...
    @property
    def oxxl(self) -> IVector4: ...
    @property
    def oxyx(self) -> IVector4: ...
    @property
    def oxyy(self) -> IVector4: ...
    @property
    def oxyz(self) -> IVector4: ...
    @property
    def oxyw(self) -> IVector4: ...
    @property
    def oxyo(self) -> IVector4: ...
    @property
    def oxyl(self) -> IVector4: ...
    @property
    def oxzx(self) -> IVector4: ...
    @property
    def oxzy(self) -> IVector4: ...
    @property
    def oxzz(self) -> IVector4: ...
    @property
    def oxzw(self) -> IVector4: ...
    @property
    def oxzo(self) -> IVector4: ...
    @property
    def oxzl(self) -> IVector4: ...
    @property
    def oxwx(self) -> IVector4: ...
    @property
    def oxwy(self) -> IVector4: ...
    @property
    def oxwz(self) -> IVector4: ...
    @property
    def oxww(self) -> IVector4: ...
    @property
    def oxwo(self) -> IVector4: ...
    @property
    def oxwl(self) -> IVector4: ...
    @property
    def oxox(self) -> IVector4: ...
    @property
    def oxoy(self) -> IVector4: ...
    @property
    def oxoz(self) -> IVector4: ...
    @property
    def oxow(self) -> IVector4: ...
    @property
    def oxoo(self) -> IVector4: ...
    @property
    def oxol(self) -> IVector4: ...
    @property
    def oxlx(self) -> IVector4: ...
    @property
    def oxly(self) -> IVector4: ...
    @property
    def oxlz(self) -> IVector4: ...
    @property
    def oxlw(self) -> IVector4: ...
    @property
    def oxlo(self) -> IVector4: ...
    @property
    def oxll(self) -> IVector4: ...
    @property
    def oyxx(self) -> IVector4: ...
    @property
    def oyxy(self) -> IVector4: ...
    @property
    def oyxz(self) -> IVector4: ...
    @property
    def oyxw(self) -> IVector4: ...
    @property
    def oyxo(self) -> IVector4: ...
    @property
    def oyxl(self) -> IVector4: ...
    @property
    def oyyx(self) -> IVector4: ...
    @property
    def oyyy(self) -> IVector4: ...
    @property
    def oyyz(self) -> IVector4: ...
    @property
    def oyyw(self) -> IVector4: ...
    @property
    def oyyo(self) -> IVector4: ...
    @property
    def oyyl(self) -> IVector4: ...
    @property
    def oyzx(self) -> IVector4: ...
    @property
    def oyzy(self) -> IVector4: ...
    @property
    def oyzz(self) -> IVector4: ...
    @property
    def oyzw(self) -> IVector4: ...
    @property
    def oyzo(self) -> IVector4: ...
    @property
    def oyzl(self) -> IVector4: ...
    @property
    def oywx(self) -> IVector4: ...
    @property
    def oywy(self) -> IVector4: ...
    @property
    def oywz(self) -> IVector4: ...
    @property
    def oyww(self) -> IVector4: ...
    @property
    def oywo(self) -> IVector4: ...
    @property
    def oywl(self) -> IVector4: ...
    @property
    def oyox(self) -> IVector4: ...
    @property
    def oyoy(self) -> IVector4: ...
    @property
    def oyoz(self) -> IVector4: ...
    @property
    def oyow(self) -> IVector4: ...
    @property
    def oyoo(self) -> IVector4: ...
    @property
    def oyol(self) -> IVector4: ...
    @property
    def oylx(self) -> IVector4: ...
    @property
    def oyly(self) -> IVector4: ...
    @property
    def oylz(self) -> IVector4: ...
    @property
    def oylw(self) -> IVector4: ...
    @property
    def oylo(self) -> IVector4: ...
    @property
    def oyll(self) -> IVector4: ...
    @property
    def ozxx(self) -> IVector4: ...
    @property
    def ozxy(self) -> IVector4: ...
    @property
    def ozxz(self) -> IVector4: ...
    @property
    def ozxw(self) -> IVector4: ...
    @property
    def ozxo(self) -> IVector4: ...
    @property
    def ozxl(self) -> IVector4: ...
    @property
    def ozyx(self) -> IVector4: ...
    @property
    def ozyy(self) -> IVector4: ...
    @property
    def ozyz(self) -> IVector4: ...
    @property
    def ozyw(self) -> IVector4: ...
    @property
    def ozyo(self) -> IVector4: ...
    @property
    def ozyl(self) -> IVector4: ...
    @property
    def ozzx(self) -> IVector4: ...
    @property
    def ozzy(self) -> IVector4: ...
    @property
    def ozzz(self) -> IVector4: ...
    @property
    def ozzw(self) -> IVector4: ...
    @property
    def ozzo(self) -> IVector4: ...
    @property
    def ozzl(self) -> IVector4: ...
    @property
    def ozwx(self) -> IVector4: ...
    @property
    def ozwy(self) -> IVector4: ...
    @property
    def ozwz(self) -> IVector4: ...
    @property
    def ozww(self) -> IVector4: ...
    @property
    def ozwo(self) -> IVector4: ...
    @property
    def ozwl(self) -> IVector4: ...
    @property
    def ozox(self) -> IVector4: ...
    @property
    def ozoy(self) -> IVector4: ...
    @property
    def ozoz(self) -> IVector4: ...
    @property
    def ozow(self) -> IVector4: ...
    @property
    def ozoo(self) -> IVector4: ...
    @property
    def ozol(self) -> IVector4: ...
    @property
    def ozlx(self) -> IVector4: ...
    @property
    def ozly(self) -> IVector4: ...
    @property
    def ozlz(self) -> IVector4: ...
    @property
    def ozlw(self) -> IVector4: ...
    @property
    def ozlo(self) -> IVector4: ...
    @property
    def ozll(self) -> IVector4: ...
    @property
    def owxx(self) -> IVector4: ...
    @property
    def owxy(self) -> IVector4: ...
    @property
    def owxz(self) -> IVector4: ...
    @property
    def owxw(self) -> IVector4: ...
    @property
    def owxo(self) -> IVector4: ...
    @property
    def owxl(self) -> IVector4: ...
    @property
    def owyx(self) -> IVector4: ...
    @property
    def owyy(self) -> IVector4: ...
    @property
    def owyz(self) -> IVector4: ...
    @property
    def owyw(self) -> IVector4: ...
    @property
    def owyo(self) -> IVector4: ...
    @property
    def owyl(self) -> IVector4: ...
    @property
    def owzx(self) -> IVector4: ...
    @property
    def owzy(self) -> IVector4: ...
    @property
    def owzz(self) -> IVector4: ...
    @property
    def owzw(self) -> IVector4: ...
    @property
    def owzo(self) -> IVector4: ...
    @property
    def owzl(self) -> IVector4: ...
    @property
    def owwx(self) -> IVector4: ...
    @property
    def owwy(self) -> IVector4: ...
    @property
    def owwz(self) -> IVector4: ...
    @property
    def owww(self) -> IVector4: ...
    @property
    def owwo(self) -> IVector4: ...
    @property
    def owwl(self) -> IVector4: ...
    @property
    def owox(self) -> IVector4: ...
    @property
    def owoy(self) -> IVector4: ...
    @property
    def owoz(self) -> IVector4: ...
    @property
    def owow(self) -> IVector4: ...
    @property
    def owoo(self) -> IVector4: ...
    @property
    def owol(self) -> IVector4: ...
    @property
    def owlx(self) -> IVector4: ...
    @property
    def owly(self) -> IVector4: ...
    @property
    def owlz(self) -> IVector4: ...
    @property
    def owlw(self) -> IVector4: ...
    @property
    def owlo(self) -> IVector4: ...
    @property
    def owll(self) -> IVector4: ...
    @property
    def ooxx(self) -> IVector4: ...
    @property
    def ooxy(self) -> IVector4: ...
    @property
    def ooxz(self) -> IVector4: ...
    @property
    def ooxw(self) -> IVector4: ...
    @property
    def ooxo(self) -> IVector4: ...
    @property
    def ooxl(self) -> IVector4: ...
    @property
    def ooyx(self) -> IVector4: ...
    @property
    def ooyy(self) -> IVector4: ...
    @property
    def ooyz(self) -> IVector4: ...
    @property
    def ooyw(self) -> IVector4: ...
    @property
    def ooyo(self) -> IVector4: ...
    @property
    def ooyl(self) -> IVector4: ...
    @property
    def oozx(self) -> IVector4: ...
    @property
    def oozy(self) -> IVector4: ...
    @property
    def oozz(self) -> IVector4: ...
    @property
    def oozw(self) -> IVector4: ...
    @property
    def oozo(self) -> IVector4: ...
    @property
    def oozl(self) -> IVector4: ...
    @property
    def oowx(self) -> IVector4: ...
    @property
    def oowy(self) -> IVector4: ...
    @property
    def oowz(self) -> IVector4: ...
    @property
    def ooww(self) -> IVector4: ...
    @property
    def oowo(self) -> IVector4: ...
    @property
    def oowl(self) -> IVector4: ...
    @property
    def ooox(self) -> IVector4: ...
    @property
    def oooy(self) -> IVector4: ...
    @property
    def oooz(self) -> IVector4: ...
    @property
    def ooow(self) -> IVector4: ...
    @property
    def oolx(self) -> IVector4: ...
    @property
    def ooly(self) -> IVector4: ...
    @property
    def oolz(self) -> IVector4: ...
    @property
    def oolw(self) -> IVector4: ...
    @property
    def olxx(self) -> IVector4: ...
    @property
    def olxy(self) -> IVector4: ...
    @property
    def olxz(self) -> IVector4: ...
    @property
    def olxw(self) -> IVector4: ...
    @property
    def olxo(self) -> IVector4: ...
    @property
    def olxl(self) -> IVector4: ...
    @property
    def olyx(self) -> IVector4: ...
    @property
    def olyy(self) -> IVector4: ...
    @property
    def olyz(self) -> IVector4: ...
    @property
    def olyw(self) -> IVector4: ...
    @property
    def olyo(self) -> IVector4: ...
    @property
    def olyl(self) -> IVector4: ...
    @property
    def olzx(self) -> IVector4: ...
    @property
    def olzy(self) -> IVector4: ...
    @property
    def olzz(self) -> IVector4: ...
    @property
    def olzw(self) -> IVector4: ...
    @property
    def olzo(self) -> IVector4: ...
    @property
    def olzl(self) -> IVector4: ...
    @property
    def olwx(self) -> IVector4: ...
    @property
    def olwy(self) -> IVector4: ...
    @property
    def olwz(self) -> IVector4: ...
    @property
    def olww(self) -> IVector4: ...
    @property
    def olwo(self) -> IVector4: ...
    @property
    def olwl(self) -> IVector4: ...
    @property
    def olox(self) -> IVector4: ...
    @property
    def oloy(self) -> IVector4: ...
    @property
    def oloz(self) -> IVector4: ...
    @property
    def olow(self) -> IVector4: ...
    @property
    def ollx(self) -> IVector4: ...
    @property
    def olly(self) -> IVector4: ...
    @property
    def ollz(self) -> IVector4: ...
    @property
    def ollw(self) -> IVector4: ...
    @property
    def lxxx(self) -> IVector4: ...
    @property
    def lxxy(self) -> IVector4: ...
    @property
    def lxxz(self) -> IVector4: ...
    @property
    def lxxw(self) -> IVector4: ...
    @property
    def lxxo(self) -> IVector4: ...
    @property
    def lxxl(self) -> IVector4: ...
    @property
    def lxyx(self) -> IVector4: ...
    @property
    def lxyy(self) -> IVector4: ...
    @property
    def lxyz(self) -> IVector4: ...
    @property
    def lxyw(self) -> IVector4: ...
    @property
    def lxyo(self) -> IVector4: ...
    @property
    def lxyl(self) -> IVector4: ...
    @property
    def lxzx(self) -> IVector4: ...
    @property
    def lxzy(self) -> IVector4: ...
    @property
    def lxzz(self) -> IVector4: ...
    @property
    def lxzw(self) -> IVector4: ...
    @property
    def lxzo(self) -> IVector4: ...
    @property
    def lxzl(self) -> IVector4: ...
    @property
    def lxwx(self) -> IVector4: ...
    @property
    def lxwy(self) -> IVector4: ...
    @property
    def lxwz(self) -> IVector4: ...
    @property
    def lxww(self) -> IVector4: ...
    @property
    def lxwo(self) -> IVector4: ...
    @property
    def lxwl(self) -> IVector4: ...
    @property
    def lxox(self) -> IVector4: ...
    @property
    def lxoy(self) -> IVector4: ...
    @property
    def lxoz(self) -> IVector4: ...
    @property
    def lxow(self) -> IVector4: ...
    @property
    def lxoo(self) -> IVector4: ...
    @property
    def lxol(self) -> IVector4: ...
    @property
    def lxlx(self) -> IVector4: ...
    @property
    def lxly(self) -> IVector4: ...
    @property
    def lxlz(self) -> IVector4: ...
    @property
    def lxlw(self) -> IVector4: ...
    @property
    def lxlo(self) -> IVector4: ...
    @property
    def lxll(self) -> IVector4: ...
    @property
    def lyxx(self) -> IVector4: ...
    @property
    def lyxy(self) -> IVector4: ...
    @property
    def lyxz(self) -> IVector4: ...
    @property
    def lyxw(self) -> IVector4: ...
    @property
    def lyxo(self) -> IVector4: ...
    @property
    def lyxl(self) -> IVector4: ...
    @property
    def lyyx(self) -> IVector4: ...
    @property
    def lyyy(self) -> IVector4: ...
    @property
    def lyyz(self) -> IVector4: ...
    @property
    def lyyw(self) -> IVector4: ...
    @property
    def lyyo(self) -> IVector4: ...
    @property
    def lyyl(self) -> IVector4: ...
    @property
    def lyzx(self) -> IVector4: ...
    @property
    def lyzy(self) -> IVector4: ...
    @property
    def lyzz(self) -> IVector4: ...
    @property
    def lyzw(self) -> IVector4: ...
    @property
    def lyzo(self) -> IVector4: ...
    @property
    def lyzl(self) -> IVector4: ...
    @property
    def lywx(self) -> IVector4: ...
    @property
    def lywy(self) -> IVector4: ...
    @property
    def lywz(self) -> IVector4: ...
    @property
    def lyww(self) -> IVector4: ...
    @property
    def lywo(self) -> IVector4: ...
    @property
    def lywl(self) -> IVector4: ...
    @property
    def lyox(self) -> IVector4: ...
    @property
    def lyoy(self) -> IVector4: ...
    @property
    def lyoz(self) -> IVector4: ...
    @property
    def lyow(self) -> IVector4: ...
    @property
    def lyoo(self) -> IVector4: ...
    @property
    def lyol(self) -> IVector4: ...
    @property
    def lylx(self) -> IVector4: ...
    @property
    def lyly(self) -> IVector4: ...
    @property
    def lylz(self) -> IVector4: ...
    @property
    def lylw(self) -> IVector4: ...
    @property
    def lylo(self) -> IVector4: ...
    @property
    def lyll(self) -> IVector4: ...
    @property
    def lzxx(self) -> IVector4: ...
    @property
    def lzxy(self) -> IVector4: ...
    @property
    def lzxz(self) -> IVector4: ...
    @property
    def lzxw(self) -> IVector4: ...
    @property
    def lzxo(self) -> IVector4: ...
    @property
    def lzxl(self) -> IVector4: ...
    @property
    def lzyx(self) -> IVector4: ...
    @property
    def lzyy(self) -> IVector4: ...
    @property
    def lzyz(self) -> IVector4: ...
    @property
    def lzyw(self) -> IVector4: ...
    @property
    def lzyo(self) -> IVector4: ...
    @property
    def lzyl(self) -> IVector4: ...
    @property
    def lzzx(self) -> IVector4: ...
    @property
    def lzzy(self) -> IVector4: ...
    @property
    def lzzz(self) -> IVector4: ...
    @property
    def lzzw(self) -> IVector4: ...
    @property
    def lzzo(self) -> IVector4: ...
    @property
    def lzzl(self) -> IVector4: ...
    @property
    def lzwx(self) -> IVector4: ...
    @property
    def lzwy(self) -> IVector4: ...
    @property
    def lzwz(self) -> IVector4: ...
    @property
    def lzww(self) -> IVector4: ...
    @property
    def lzwo(self) -> IVector4: ...
    @property
    def lzwl(self) -> IVector4: ...
    @property
    def lzox(self) -> IVector4: ...
    @property
    def lzoy(self) -> IVector4: ...
    @property
    def lzoz(self) -> IVector4: ...
    @property
    def lzow(self) -> IVector4: ...
    @property
    def lzoo(self) -> IVector4: ...
    @property
    def lzol(self) -> IVector4: ...
    @property
    def lzlx(self) -> IVector4: ...
    @property
    def lzly(self) -> IVector4: ...
    @property
    def lzlz(self) -> IVector4: ...
    @property
    def lzlw(self) -> IVector4: ...
    @property
    def lzlo(self) -> IVector4: ...
    @property
    def lzll(self) -> IVector4: ...
    @property
    def lwxx(self) -> IVector4: ...
    @property
    def lwxy(self) -> IVector4: ...
    @property
    def lwxz(self) -> IVector4: ...
    @property
    def lwxw(self) -> IVector4: ...
    @property
    def lwxo(self) -> IVector4: ...
    @property
    def lwxl(self) -> IVector4: ...
    @property
    def lwyx(self) -> IVector4: ...
    @property
    def lwyy(self) -> IVector4: ...
    @property
    def lwyz(self) -> IVector4: ...
    @property
    def lwyw(self) -> IVector4: ...
    @property
    def lwyo(self) -> IVector4: ...
    @property
    def lwyl(self) -> IVector4: ...
    @property
    def lwzx(self) -> IVector4: ...
    @property
    def lwzy(self) -> IVector4: ...
    @property
    def lwzz(self) -> IVector4: ...
    @property
    def lwzw(self) -> IVector4: ...
    @property
    def lwzo(self) -> IVector4: ...
    @property
    def lwzl(self) -> IVector4: ...
    @property
    def lwwx(self) -> IVector4: ...
    @property
    def lwwy(self) -> IVector4: ...
    @property
    def lwwz(self) -> IVector4: ...
    @property
    def lwww(self) -> IVector4: ...
    @property
    def lwwo(self) -> IVector4: ...
    @property
    def lwwl(self) -> IVector4: ...
    @property
    def lwox(self) -> IVector4: ...
    @property
    def lwoy(self) -> IVector4: ...
    @property
    def lwoz(self) -> IVector4: ...
    @property
    def lwow(self) -> IVector4: ...
    @property
    def lwoo(self) -> IVector4: ...
    @property
    def lwol(self) -> IVector4: ...
    @property
    def lwlx(self) -> IVector4: ...
    @property
    def lwly(self) -> IVector4: ...
    @property
    def lwlz(self) -> IVector4: ...
    @property
    def lwlw(self) -> IVector4: ...
    @property
    def lwlo(self) -> IVector4: ...
    @property
    def lwll(self) -> IVector4: ...
    @property
    def loxx(self) -> IVector4: ...
    @property
    def loxy(self) -> IVector4: ...
    @property
    def loxz(self) -> IVector4: ...
    @property
    def loxw(self) -> IVector4: ...
    @property
    def loxo(self) -> IVector4: ...
    @property
    def loxl(self) -> IVector4: ...
    @property
    def loyx(self) -> IVector4: ...
    @property
    def loyy(self) -> IVector4: ...
    @property
    def loyz(self) -> IVector4: ...
    @property
    def loyw(self) -> IVector4: ...
    @property
    def loyo(self) -> IVector4: ...
    @property
    def loyl(self) -> IVector4: ...
    @property
    def lozx(self) -> IVector4: ...
    @property
    def lozy(self) -> IVector4: ...
    @property
    def lozz(self) -> IVector4: ...
    @property
    def lozw(self) -> IVector4: ...
    @property
    def lozo(self) -> IVector4: ...
    @property
    def lozl(self) -> IVector4: ...
    @property
    def lowx(self) -> IVector4: ...
    @property
    def lowy(self) -> IVector4: ...
    @property
    def lowz(self) -> IVector4: ...
    @property
    def loww(self) -> IVector4: ...
    @property
    def lowo(self) -> IVector4: ...
    @property
    def lowl(self) -> IVector4: ...
    @property
    def loox(self) -> IVector4: ...
    @property
    def looy(self) -> IVector4: ...
    @property
    def looz(self) -> IVector4: ...
    @property
    def loow(self) -> IVector4: ...
    @property
    def lolx(self) -> IVector4: ...
    @property
    def loly(self) -> IVector4: ...
    @property
    def lolz(self) -> IVector4: ...
    @property
    def lolw(self) -> IVector4: ...
    @property
    def llxx(self) -> IVector4: ...
    @property
    def llxy(self) -> IVector4: ...
    @property
    def llxz(self) -> IVector4: ...
    @property
    def llxw(self) -> IVector4: ...
    @property
    def llxo(self) -> IVector4: ...
    @property
    def llxl(self) -> IVector4: ...
    @property
    def llyx(self) -> IVector4: ...
    @property
    def llyy(self) -> IVector4: ...
    @property
    def llyz(self) -> IVector4: ...
    @property
    def llyw(self) -> IVector4: ...
    @property
    def llyo(self) -> IVector4: ...
    @property
    def llyl(self) -> IVector4: ...
    @property
    def llzx(self) -> IVector4: ...
    @property
    def llzy(self) -> IVector4: ...
    @property
    def llzz(self) -> IVector4: ...
    @property
    def llzw(self) -> IVector4: ...
    @property
    def llzo(self) -> IVector4: ...
    @property
    def llzl(self) -> IVector4: ...
    @property
    def llwx(self) -> IVector4: ...
    @property
    def llwy(self) -> IVector4: ...
    @property
    def llwz(self) -> IVector4: ...
    @property
    def llww(self) -> IVector4: ...
    @property
    def llwo(self) -> IVector4: ...
    @property
    def llwl(self) -> IVector4: ...
    @property
    def llox(self) -> IVector4: ...
    @property
    def lloy(self) -> IVector4: ...
    @property
    def lloz(self) -> IVector4: ...
    @property
    def llow(self) -> IVector4: ...
    @property
    def lllx(self) -> IVector4: ...
    @property
    def llly(self) -> IVector4: ...
    @property
    def lllz(self) -> IVector4: ...
    @property
    def lllw(self) -> IVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> IVector2: ...
    @property
    def rg(self) -> IVector2: ...
    @property
    def rb(self) -> IVector2: ...
    @property
    def ra(self) -> IVector2: ...
    @property
    def ro(self) -> IVector2: ...
    @property
    def rl(self) -> IVector2: ...
    @property
    def gr(self) -> IVector2: ...
    @property
    def gg(self) -> IVector2: ...
    @property
    def gb(self) -> IVector2: ...
    @property
    def ga(self) -> IVector2: ...
    @property
    def go(self) -> IVector2: ...
    @property
    def gl(self) -> IVector2: ...
    @property
    def br(self) -> IVector2: ...
    @property
    def bg(self) -> IVector2: ...
    @property
    def bb(self) -> IVector2: ...
    @property
    def ba(self) -> IVector2: ...
    @property
    def bo(self) -> IVector2: ...
    @property
    def bl(self) -> IVector2: ...
    @property
    def ar(self) -> IVector2: ...
    @property
    def ag(self) -> IVector2: ...
    @property
    def ab(self) -> IVector2: ...
    @property
    def aa(self) -> IVector2: ...
    @property
    def ao(self) -> IVector2: ...
    @property
    def al(self) -> IVector2: ...
    @property
    def og(self) -> IVector2: ...
    @property
    def ob(self) -> IVector2: ...
    @property
    def oa(self) -> IVector2: ...
    @property
    def lr(self) -> IVector2: ...
    @property
    def lg(self) -> IVector2: ...
    @property
    def lb(self) -> IVector2: ...
    @property
    def la(self) -> IVector2: ...
    @property
    def rrr(self) -> IVector3: ...
    @property
    def rrg(self) -> IVector3: ...
    @property
    def rrb(self) -> IVector3: ...
    @property
    def rra(self) -> IVector3: ...
    @property
    def rro(self) -> IVector3: ...
    @property
    def rrl(self) -> IVector3: ...
    @property
    def rgr(self) -> IVector3: ...
    @property
    def rgg(self) -> IVector3: ...
    @property
    def rgb(self) -> IVector3: ...
    @property
    def rga(self) -> IVector3: ...
    @property
    def rgo(self) -> IVector3: ...
    @property
    def rgl(self) -> IVector3: ...
    @property
    def rbr(self) -> IVector3: ...
    @property
    def rbg(self) -> IVector3: ...
    @property
    def rbb(self) -> IVector3: ...
    @property
    def rba(self) -> IVector3: ...
    @property
    def rbo(self) -> IVector3: ...
    @property
    def rbl(self) -> IVector3: ...
    @property
    def rar(self) -> IVector3: ...
    @property
    def rag(self) -> IVector3: ...
    @property
    def rab(self) -> IVector3: ...
    @property
    def raa(self) -> IVector3: ...
    @property
    def rao(self) -> IVector3: ...
    @property
    def ral(self) -> IVector3: ...
    @property
    def ror(self) -> IVector3: ...
    @property
    def rog(self) -> IVector3: ...
    @property
    def rob(self) -> IVector3: ...
    @property
    def roa(self) -> IVector3: ...
    @property
    def roo(self) -> IVector3: ...
    @property
    def rol(self) -> IVector3: ...
    @property
    def rlr(self) -> IVector3: ...
    @property
    def rlg(self) -> IVector3: ...
    @property
    def rlb(self) -> IVector3: ...
    @property
    def rla(self) -> IVector3: ...
    @property
    def rlo(self) -> IVector3: ...
    @property
    def rll(self) -> IVector3: ...
    @property
    def grr(self) -> IVector3: ...
    @property
    def grg(self) -> IVector3: ...
    @property
    def grb(self) -> IVector3: ...
    @property
    def gra(self) -> IVector3: ...
    @property
    def gro(self) -> IVector3: ...
    @property
    def grl(self) -> IVector3: ...
    @property
    def ggr(self) -> IVector3: ...
    @property
    def ggg(self) -> IVector3: ...
    @property
    def ggb(self) -> IVector3: ...
    @property
    def gga(self) -> IVector3: ...
    @property
    def ggo(self) -> IVector3: ...
    @property
    def ggl(self) -> IVector3: ...
    @property
    def gbr(self) -> IVector3: ...
    @property
    def gbg(self) -> IVector3: ...
    @property
    def gbb(self) -> IVector3: ...
    @property
    def gba(self) -> IVector3: ...
    @property
    def gbo(self) -> IVector3: ...
    @property
    def gbl(self) -> IVector3: ...
    @property
    def gar(self) -> IVector3: ...
    @property
    def gag(self) -> IVector3: ...
    @property
    def gab(self) -> IVector3: ...
    @property
    def gaa(self) -> IVector3: ...
    @property
    def gao(self) -> IVector3: ...
    @property
    def gal(self) -> IVector3: ...
    @property
    def gor(self) -> IVector3: ...
    @property
    def gog(self) -> IVector3: ...
    @property
    def gob(self) -> IVector3: ...
    @property
    def goa(self) -> IVector3: ...
    @property
    def goo(self) -> IVector3: ...
    @property
    def gol(self) -> IVector3: ...
    @property
    def glr(self) -> IVector3: ...
    @property
    def glg(self) -> IVector3: ...
    @property
    def glb(self) -> IVector3: ...
    @property
    def gla(self) -> IVector3: ...
    @property
    def glo(self) -> IVector3: ...
    @property
    def gll(self) -> IVector3: ...
    @property
    def brr(self) -> IVector3: ...
    @property
    def brg(self) -> IVector3: ...
    @property
    def brb(self) -> IVector3: ...
    @property
    def bra(self) -> IVector3: ...
    @property
    def bro(self) -> IVector3: ...
    @property
    def brl(self) -> IVector3: ...
    @property
    def bgr(self) -> IVector3: ...
    @property
    def bgg(self) -> IVector3: ...
    @property
    def bgb(self) -> IVector3: ...
    @property
    def bga(self) -> IVector3: ...
    @property
    def bgo(self) -> IVector3: ...
    @property
    def bgl(self) -> IVector3: ...
    @property
    def bbr(self) -> IVector3: ...
    @property
    def bbg(self) -> IVector3: ...
    @property
    def bbb(self) -> IVector3: ...
    @property
    def bba(self) -> IVector3: ...
    @property
    def bbo(self) -> IVector3: ...
    @property
    def bbl(self) -> IVector3: ...
    @property
    def bar(self) -> IVector3: ...
    @property
    def bag(self) -> IVector3: ...
    @property
    def bab(self) -> IVector3: ...
    @property
    def baa(self) -> IVector3: ...
    @property
    def bao(self) -> IVector3: ...
    @property
    def bal(self) -> IVector3: ...
    @property
    def bor(self) -> IVector3: ...
    @property
    def bog(self) -> IVector3: ...
    @property
    def bob(self) -> IVector3: ...
    @property
    def boa(self) -> IVector3: ...
    @property
    def boo(self) -> IVector3: ...
    @property
    def bol(self) -> IVector3: ...
    @property
    def blr(self) -> IVector3: ...
    @property
    def blg(self) -> IVector3: ...
    @property
    def blb(self) -> IVector3: ...
    @property
    def bla(self) -> IVector3: ...
    @property
    def blo(self) -> IVector3: ...
    @property
    def bll(self) -> IVector3: ...
    @property
    def arr(self) -> IVector3: ...
    @property
    def arg(self) -> IVector3: ...
    @property
    def arb(self) -> IVector3: ...
    @property
    def ara(self) -> IVector3: ...
    @property
    def aro(self) -> IVector3: ...
    @property
    def arl(self) -> IVector3: ...
    @property
    def agr(self) -> IVector3: ...
    @property
    def agg(self) -> IVector3: ...
    @property
    def agb(self) -> IVector3: ...
    @property
    def aga(self) -> IVector3: ...
    @property
    def ago(self) -> IVector3: ...
    @property
    def agl(self) -> IVector3: ...
    @property
    def abr(self) -> IVector3: ...
    @property
    def abg(self) -> IVector3: ...
    @property
    def abb(self) -> IVector3: ...
    @property
    def aba(self) -> IVector3: ...
    @property
    def abo(self) -> IVector3: ...
    @property
    def abl(self) -> IVector3: ...
    @property
    def aar(self) -> IVector3: ...
    @property
    def aag(self) -> IVector3: ...
    @property
    def aab(self) -> IVector3: ...
    @property
    def aaa(self) -> IVector3: ...
    @property
    def aao(self) -> IVector3: ...
    @property
    def aal(self) -> IVector3: ...
    @property
    def aor(self) -> IVector3: ...
    @property
    def aog(self) -> IVector3: ...
    @property
    def aob(self) -> IVector3: ...
    @property
    def aoa(self) -> IVector3: ...
    @property
    def aoo(self) -> IVector3: ...
    @property
    def aol(self) -> IVector3: ...
    @property
    def alr(self) -> IVector3: ...
    @property
    def alg(self) -> IVector3: ...
    @property
    def alb(self) -> IVector3: ...
    @property
    def ala(self) -> IVector3: ...
    @property
    def alo(self) -> IVector3: ...
    @property
    def all(self) -> IVector3: ...
    @property
    def orr(self) -> IVector3: ...
    @property
    def org(self) -> IVector3: ...
    @property
    def orb(self) -> IVector3: ...
    @property
    def ora(self) -> IVector3: ...
    @property
    def oro(self) -> IVector3: ...
    @property
    def orl(self) -> IVector3: ...
    @property
    def ogr(self) -> IVector3: ...
    @property
    def ogg(self) -> IVector3: ...
    @property
    def ogb(self) -> IVector3: ...
    @property
    def oga(self) -> IVector3: ...
    @property
    def ogo(self) -> IVector3: ...
    @property
    def ogl(self) -> IVector3: ...
    @property
    def obr(self) -> IVector3: ...
    @property
    def obg(self) -> IVector3: ...
    @property
    def obb(self) -> IVector3: ...
    @property
    def oba(self) -> IVector3: ...
    @property
    def obo(self) -> IVector3: ...
    @property
    def obl(self) -> IVector3: ...
    @property
    def oar(self) -> IVector3: ...
    @property
    def oag(self) -> IVector3: ...
    @property
    def oab(self) -> IVector3: ...
    @property
    def oaa(self) -> IVector3: ...
    @property
    def oao(self) -> IVector3: ...
    @property
    def oal(self) -> IVector3: ...
    @property
    def oor(self) -> IVector3: ...
    @property
    def oog(self) -> IVector3: ...
    @property
    def oob(self) -> IVector3: ...
    @property
    def ooa(self) -> IVector3: ...
    @property
    def olr(self) -> IVector3: ...
    @property
    def olg(self) -> IVector3: ...
    @property
    def olb(self) -> IVector3: ...
    @property
    def ola(self) -> IVector3: ...
    @property
    def lrr(self) -> IVector3: ...
    @property
    def lrg(self) -> IVector3: ...
    @property
    def lrb(self) -> IVector3: ...
    @property
    def lra(self) -> IVector3: ...
    @property
    def lro(self) -> IVector3: ...
    @property
    def lrl(self) -> IVector3: ...
    @property
    def lgr(self) -> IVector3: ...
    @property
    def lgg(self) -> IVector3: ...
    @property
    def lgb(self) -> IVector3: ...
    @property
    def lga(self) -> IVector3: ...
    @property
    def lgo(self) -> IVector3: ...
    @property
    def lgl(self) -> IVector3: ...
    @property
    def lbr(self) -> IVector3: ...
    @property
    def lbg(self) -> IVector3: ...
    @property
    def lbb(self) -> IVector3: ...
    @property
    def lba(self) -> IVector3: ...
    @property
    def lbo(self) -> IVector3: ...
    @property
    def lbl(self) -> IVector3: ...
    @property
    def lar(self) -> IVector3: ...
    @property
    def lag(self) -> IVector3: ...
    @property
    def lab(self) -> IVector3: ...
    @property
    def laa(self) -> IVector3: ...
    @property
    def lao(self) -> IVector3: ...
    @property
    def lal(self) -> IVector3: ...
    @property
    def lor(self) -> IVector3: ...
    @property
    def log(self) -> IVector3: ...
    @property
    def lob(self) -> IVector3: ...
    @property
    def loa(self) -> IVector3: ...
    @property
    def llr(self) -> IVector3: ...
    @property
    def llg(self) -> IVector3: ...
    @property
    def llb(self) -> IVector3: ...
    @property
    def lla(self) -> IVector3: ...
    @property
    def rrrr(self) -> IVector4: ...
    @property
    def rrrg(self) -> IVector4: ...
    @property
    def rrrb(self) -> IVector4: ...
    @property
    def rrra(self) -> IVector4: ...
    @property
    def rrro(self) -> IVector4: ...
    @property
    def rrrl(self) -> IVector4: ...
    @property
    def rrgr(self) -> IVector4: ...
    @property
    def rrgg(self) -> IVector4: ...
    @property
    def rrgb(self) -> IVector4: ...
    @property
    def rrga(self) -> IVector4: ...
    @property
    def rrgo(self) -> IVector4: ...
    @property
    def rrgl(self) -> IVector4: ...
    @property
    def rrbr(self) -> IVector4: ...
    @property
    def rrbg(self) -> IVector4: ...
    @property
    def rrbb(self) -> IVector4: ...
    @property
    def rrba(self) -> IVector4: ...
    @property
    def rrbo(self) -> IVector4: ...
    @property
    def rrbl(self) -> IVector4: ...
    @property
    def rrar(self) -> IVector4: ...
    @property
    def rrag(self) -> IVector4: ...
    @property
    def rrab(self) -> IVector4: ...
    @property
    def rraa(self) -> IVector4: ...
    @property
    def rrao(self) -> IVector4: ...
    @property
    def rral(self) -> IVector4: ...
    @property
    def rror(self) -> IVector4: ...
    @property
    def rrog(self) -> IVector4: ...
    @property
    def rrob(self) -> IVector4: ...
    @property
    def rroa(self) -> IVector4: ...
    @property
    def rroo(self) -> IVector4: ...
    @property
    def rrol(self) -> IVector4: ...
    @property
    def rrlr(self) -> IVector4: ...
    @property
    def rrlg(self) -> IVector4: ...
    @property
    def rrlb(self) -> IVector4: ...
    @property
    def rrla(self) -> IVector4: ...
    @property
    def rrlo(self) -> IVector4: ...
    @property
    def rrll(self) -> IVector4: ...
    @property
    def rgrr(self) -> IVector4: ...
    @property
    def rgrg(self) -> IVector4: ...
    @property
    def rgrb(self) -> IVector4: ...
    @property
    def rgra(self) -> IVector4: ...
    @property
    def rgro(self) -> IVector4: ...
    @property
    def rgrl(self) -> IVector4: ...
    @property
    def rggr(self) -> IVector4: ...
    @property
    def rggg(self) -> IVector4: ...
    @property
    def rggb(self) -> IVector4: ...
    @property
    def rgga(self) -> IVector4: ...
    @property
    def rggo(self) -> IVector4: ...
    @property
    def rggl(self) -> IVector4: ...
    @property
    def rgbr(self) -> IVector4: ...
    @property
    def rgbg(self) -> IVector4: ...
    @property
    def rgbb(self) -> IVector4: ...
    @property
    def rgba(self) -> IVector4: ...
    @property
    def rgbo(self) -> IVector4: ...
    @property
    def rgbl(self) -> IVector4: ...
    @property
    def rgar(self) -> IVector4: ...
    @property
    def rgag(self) -> IVector4: ...
    @property
    def rgab(self) -> IVector4: ...
    @property
    def rgaa(self) -> IVector4: ...
    @property
    def rgao(self) -> IVector4: ...
    @property
    def rgal(self) -> IVector4: ...
    @property
    def rgor(self) -> IVector4: ...
    @property
    def rgog(self) -> IVector4: ...
    @property
    def rgob(self) -> IVector4: ...
    @property
    def rgoa(self) -> IVector4: ...
    @property
    def rgoo(self) -> IVector4: ...
    @property
    def rgol(self) -> IVector4: ...
    @property
    def rglr(self) -> IVector4: ...
    @property
    def rglg(self) -> IVector4: ...
    @property
    def rglb(self) -> IVector4: ...
    @property
    def rgla(self) -> IVector4: ...
    @property
    def rglo(self) -> IVector4: ...
    @property
    def rgll(self) -> IVector4: ...
    @property
    def rbrr(self) -> IVector4: ...
    @property
    def rbrg(self) -> IVector4: ...
    @property
    def rbrb(self) -> IVector4: ...
    @property
    def rbra(self) -> IVector4: ...
    @property
    def rbro(self) -> IVector4: ...
    @property
    def rbrl(self) -> IVector4: ...
    @property
    def rbgr(self) -> IVector4: ...
    @property
    def rbgg(self) -> IVector4: ...
    @property
    def rbgb(self) -> IVector4: ...
    @property
    def rbga(self) -> IVector4: ...
    @property
    def rbgo(self) -> IVector4: ...
    @property
    def rbgl(self) -> IVector4: ...
    @property
    def rbbr(self) -> IVector4: ...
    @property
    def rbbg(self) -> IVector4: ...
    @property
    def rbbb(self) -> IVector4: ...
    @property
    def rbba(self) -> IVector4: ...
    @property
    def rbbo(self) -> IVector4: ...
    @property
    def rbbl(self) -> IVector4: ...
    @property
    def rbar(self) -> IVector4: ...
    @property
    def rbag(self) -> IVector4: ...
    @property
    def rbab(self) -> IVector4: ...
    @property
    def rbaa(self) -> IVector4: ...
    @property
    def rbao(self) -> IVector4: ...
    @property
    def rbal(self) -> IVector4: ...
    @property
    def rbor(self) -> IVector4: ...
    @property
    def rbog(self) -> IVector4: ...
    @property
    def rbob(self) -> IVector4: ...
    @property
    def rboa(self) -> IVector4: ...
    @property
    def rboo(self) -> IVector4: ...
    @property
    def rbol(self) -> IVector4: ...
    @property
    def rblr(self) -> IVector4: ...
    @property
    def rblg(self) -> IVector4: ...
    @property
    def rblb(self) -> IVector4: ...
    @property
    def rbla(self) -> IVector4: ...
    @property
    def rblo(self) -> IVector4: ...
    @property
    def rbll(self) -> IVector4: ...
    @property
    def rarr(self) -> IVector4: ...
    @property
    def rarg(self) -> IVector4: ...
    @property
    def rarb(self) -> IVector4: ...
    @property
    def rara(self) -> IVector4: ...
    @property
    def raro(self) -> IVector4: ...
    @property
    def rarl(self) -> IVector4: ...
    @property
    def ragr(self) -> IVector4: ...
    @property
    def ragg(self) -> IVector4: ...
    @property
    def ragb(self) -> IVector4: ...
    @property
    def raga(self) -> IVector4: ...
    @property
    def rago(self) -> IVector4: ...
    @property
    def ragl(self) -> IVector4: ...
    @property
    def rabr(self) -> IVector4: ...
    @property
    def rabg(self) -> IVector4: ...
    @property
    def rabb(self) -> IVector4: ...
    @property
    def raba(self) -> IVector4: ...
    @property
    def rabo(self) -> IVector4: ...
    @property
    def rabl(self) -> IVector4: ...
    @property
    def raar(self) -> IVector4: ...
    @property
    def raag(self) -> IVector4: ...
    @property
    def raab(self) -> IVector4: ...
    @property
    def raaa(self) -> IVector4: ...
    @property
    def raao(self) -> IVector4: ...
    @property
    def raal(self) -> IVector4: ...
    @property
    def raor(self) -> IVector4: ...
    @property
    def raog(self) -> IVector4: ...
    @property
    def raob(self) -> IVector4: ...
    @property
    def raoa(self) -> IVector4: ...
    @property
    def raoo(self) -> IVector4: ...
    @property
    def raol(self) -> IVector4: ...
    @property
    def ralr(self) -> IVector4: ...
    @property
    def ralg(self) -> IVector4: ...
    @property
    def ralb(self) -> IVector4: ...
    @property
    def rala(self) -> IVector4: ...
    @property
    def ralo(self) -> IVector4: ...
    @property
    def rall(self) -> IVector4: ...
    @property
    def rorr(self) -> IVector4: ...
    @property
    def rorg(self) -> IVector4: ...
    @property
    def rorb(self) -> IVector4: ...
    @property
    def rora(self) -> IVector4: ...
    @property
    def roro(self) -> IVector4: ...
    @property
    def rorl(self) -> IVector4: ...
    @property
    def rogr(self) -> IVector4: ...
    @property
    def rogg(self) -> IVector4: ...
    @property
    def rogb(self) -> IVector4: ...
    @property
    def roga(self) -> IVector4: ...
    @property
    def rogo(self) -> IVector4: ...
    @property
    def rogl(self) -> IVector4: ...
    @property
    def robr(self) -> IVector4: ...
    @property
    def robg(self) -> IVector4: ...
    @property
    def robb(self) -> IVector4: ...
    @property
    def roba(self) -> IVector4: ...
    @property
    def robo(self) -> IVector4: ...
    @property
    def robl(self) -> IVector4: ...
    @property
    def roar(self) -> IVector4: ...
    @property
    def roag(self) -> IVector4: ...
    @property
    def roab(self) -> IVector4: ...
    @property
    def roaa(self) -> IVector4: ...
    @property
    def roao(self) -> IVector4: ...
    @property
    def roal(self) -> IVector4: ...
    @property
    def roor(self) -> IVector4: ...
    @property
    def roog(self) -> IVector4: ...
    @property
    def roob(self) -> IVector4: ...
    @property
    def rooa(self) -> IVector4: ...
    @property
    def rooo(self) -> IVector4: ...
    @property
    def rool(self) -> IVector4: ...
    @property
    def rolr(self) -> IVector4: ...
    @property
    def rolg(self) -> IVector4: ...
    @property
    def rolb(self) -> IVector4: ...
    @property
    def rola(self) -> IVector4: ...
    @property
    def rolo(self) -> IVector4: ...
    @property
    def roll(self) -> IVector4: ...
    @property
    def rlrr(self) -> IVector4: ...
    @property
    def rlrg(self) -> IVector4: ...
    @property
    def rlrb(self) -> IVector4: ...
    @property
    def rlra(self) -> IVector4: ...
    @property
    def rlro(self) -> IVector4: ...
    @property
    def rlrl(self) -> IVector4: ...
    @property
    def rlgr(self) -> IVector4: ...
    @property
    def rlgg(self) -> IVector4: ...
    @property
    def rlgb(self) -> IVector4: ...
    @property
    def rlga(self) -> IVector4: ...
    @property
    def rlgo(self) -> IVector4: ...
    @property
    def rlgl(self) -> IVector4: ...
    @property
    def rlbr(self) -> IVector4: ...
    @property
    def rlbg(self) -> IVector4: ...
    @property
    def rlbb(self) -> IVector4: ...
    @property
    def rlba(self) -> IVector4: ...
    @property
    def rlbo(self) -> IVector4: ...
    @property
    def rlbl(self) -> IVector4: ...
    @property
    def rlar(self) -> IVector4: ...
    @property
    def rlag(self) -> IVector4: ...
    @property
    def rlab(self) -> IVector4: ...
    @property
    def rlaa(self) -> IVector4: ...
    @property
    def rlao(self) -> IVector4: ...
    @property
    def rlal(self) -> IVector4: ...
    @property
    def rlor(self) -> IVector4: ...
    @property
    def rlog(self) -> IVector4: ...
    @property
    def rlob(self) -> IVector4: ...
    @property
    def rloa(self) -> IVector4: ...
    @property
    def rloo(self) -> IVector4: ...
    @property
    def rlol(self) -> IVector4: ...
    @property
    def rllr(self) -> IVector4: ...
    @property
    def rllg(self) -> IVector4: ...
    @property
    def rllb(self) -> IVector4: ...
    @property
    def rlla(self) -> IVector4: ...
    @property
    def rllo(self) -> IVector4: ...
    @property
    def rlll(self) -> IVector4: ...
    @property
    def grrr(self) -> IVector4: ...
    @property
    def grrg(self) -> IVector4: ...
    @property
    def grrb(self) -> IVector4: ...
    @property
    def grra(self) -> IVector4: ...
    @property
    def grro(self) -> IVector4: ...
    @property
    def grrl(self) -> IVector4: ...
    @property
    def grgr(self) -> IVector4: ...
    @property
    def grgg(self) -> IVector4: ...
    @property
    def grgb(self) -> IVector4: ...
    @property
    def grga(self) -> IVector4: ...
    @property
    def grgo(self) -> IVector4: ...
    @property
    def grgl(self) -> IVector4: ...
    @property
    def grbr(self) -> IVector4: ...
    @property
    def grbg(self) -> IVector4: ...
    @property
    def grbb(self) -> IVector4: ...
    @property
    def grba(self) -> IVector4: ...
    @property
    def grbo(self) -> IVector4: ...
    @property
    def grbl(self) -> IVector4: ...
    @property
    def grar(self) -> IVector4: ...
    @property
    def grag(self) -> IVector4: ...
    @property
    def grab(self) -> IVector4: ...
    @property
    def graa(self) -> IVector4: ...
    @property
    def grao(self) -> IVector4: ...
    @property
    def gral(self) -> IVector4: ...
    @property
    def gror(self) -> IVector4: ...
    @property
    def grog(self) -> IVector4: ...
    @property
    def grob(self) -> IVector4: ...
    @property
    def groa(self) -> IVector4: ...
    @property
    def groo(self) -> IVector4: ...
    @property
    def grol(self) -> IVector4: ...
    @property
    def grlr(self) -> IVector4: ...
    @property
    def grlg(self) -> IVector4: ...
    @property
    def grlb(self) -> IVector4: ...
    @property
    def grla(self) -> IVector4: ...
    @property
    def grlo(self) -> IVector4: ...
    @property
    def grll(self) -> IVector4: ...
    @property
    def ggrr(self) -> IVector4: ...
    @property
    def ggrg(self) -> IVector4: ...
    @property
    def ggrb(self) -> IVector4: ...
    @property
    def ggra(self) -> IVector4: ...
    @property
    def ggro(self) -> IVector4: ...
    @property
    def ggrl(self) -> IVector4: ...
    @property
    def gggr(self) -> IVector4: ...
    @property
    def gggg(self) -> IVector4: ...
    @property
    def gggb(self) -> IVector4: ...
    @property
    def ggga(self) -> IVector4: ...
    @property
    def gggo(self) -> IVector4: ...
    @property
    def gggl(self) -> IVector4: ...
    @property
    def ggbr(self) -> IVector4: ...
    @property
    def ggbg(self) -> IVector4: ...
    @property
    def ggbb(self) -> IVector4: ...
    @property
    def ggba(self) -> IVector4: ...
    @property
    def ggbo(self) -> IVector4: ...
    @property
    def ggbl(self) -> IVector4: ...
    @property
    def ggar(self) -> IVector4: ...
    @property
    def ggag(self) -> IVector4: ...
    @property
    def ggab(self) -> IVector4: ...
    @property
    def ggaa(self) -> IVector4: ...
    @property
    def ggao(self) -> IVector4: ...
    @property
    def ggal(self) -> IVector4: ...
    @property
    def ggor(self) -> IVector4: ...
    @property
    def ggog(self) -> IVector4: ...
    @property
    def ggob(self) -> IVector4: ...
    @property
    def ggoa(self) -> IVector4: ...
    @property
    def ggoo(self) -> IVector4: ...
    @property
    def ggol(self) -> IVector4: ...
    @property
    def gglr(self) -> IVector4: ...
    @property
    def gglg(self) -> IVector4: ...
    @property
    def gglb(self) -> IVector4: ...
    @property
    def ggla(self) -> IVector4: ...
    @property
    def gglo(self) -> IVector4: ...
    @property
    def ggll(self) -> IVector4: ...
    @property
    def gbrr(self) -> IVector4: ...
    @property
    def gbrg(self) -> IVector4: ...
    @property
    def gbrb(self) -> IVector4: ...
    @property
    def gbra(self) -> IVector4: ...
    @property
    def gbro(self) -> IVector4: ...
    @property
    def gbrl(self) -> IVector4: ...
    @property
    def gbgr(self) -> IVector4: ...
    @property
    def gbgg(self) -> IVector4: ...
    @property
    def gbgb(self) -> IVector4: ...
    @property
    def gbga(self) -> IVector4: ...
    @property
    def gbgo(self) -> IVector4: ...
    @property
    def gbgl(self) -> IVector4: ...
    @property
    def gbbr(self) -> IVector4: ...
    @property
    def gbbg(self) -> IVector4: ...
    @property
    def gbbb(self) -> IVector4: ...
    @property
    def gbba(self) -> IVector4: ...
    @property
    def gbbo(self) -> IVector4: ...
    @property
    def gbbl(self) -> IVector4: ...
    @property
    def gbar(self) -> IVector4: ...
    @property
    def gbag(self) -> IVector4: ...
    @property
    def gbab(self) -> IVector4: ...
    @property
    def gbaa(self) -> IVector4: ...
    @property
    def gbao(self) -> IVector4: ...
    @property
    def gbal(self) -> IVector4: ...
    @property
    def gbor(self) -> IVector4: ...
    @property
    def gbog(self) -> IVector4: ...
    @property
    def gbob(self) -> IVector4: ...
    @property
    def gboa(self) -> IVector4: ...
    @property
    def gboo(self) -> IVector4: ...
    @property
    def gbol(self) -> IVector4: ...
    @property
    def gblr(self) -> IVector4: ...
    @property
    def gblg(self) -> IVector4: ...
    @property
    def gblb(self) -> IVector4: ...
    @property
    def gbla(self) -> IVector4: ...
    @property
    def gblo(self) -> IVector4: ...
    @property
    def gbll(self) -> IVector4: ...
    @property
    def garr(self) -> IVector4: ...
    @property
    def garg(self) -> IVector4: ...
    @property
    def garb(self) -> IVector4: ...
    @property
    def gara(self) -> IVector4: ...
    @property
    def garo(self) -> IVector4: ...
    @property
    def garl(self) -> IVector4: ...
    @property
    def gagr(self) -> IVector4: ...
    @property
    def gagg(self) -> IVector4: ...
    @property
    def gagb(self) -> IVector4: ...
    @property
    def gaga(self) -> IVector4: ...
    @property
    def gago(self) -> IVector4: ...
    @property
    def gagl(self) -> IVector4: ...
    @property
    def gabr(self) -> IVector4: ...
    @property
    def gabg(self) -> IVector4: ...
    @property
    def gabb(self) -> IVector4: ...
    @property
    def gaba(self) -> IVector4: ...
    @property
    def gabo(self) -> IVector4: ...
    @property
    def gabl(self) -> IVector4: ...
    @property
    def gaar(self) -> IVector4: ...
    @property
    def gaag(self) -> IVector4: ...
    @property
    def gaab(self) -> IVector4: ...
    @property
    def gaaa(self) -> IVector4: ...
    @property
    def gaao(self) -> IVector4: ...
    @property
    def gaal(self) -> IVector4: ...
    @property
    def gaor(self) -> IVector4: ...
    @property
    def gaog(self) -> IVector4: ...
    @property
    def gaob(self) -> IVector4: ...
    @property
    def gaoa(self) -> IVector4: ...
    @property
    def gaoo(self) -> IVector4: ...
    @property
    def gaol(self) -> IVector4: ...
    @property
    def galr(self) -> IVector4: ...
    @property
    def galg(self) -> IVector4: ...
    @property
    def galb(self) -> IVector4: ...
    @property
    def gala(self) -> IVector4: ...
    @property
    def galo(self) -> IVector4: ...
    @property
    def gall(self) -> IVector4: ...
    @property
    def gorr(self) -> IVector4: ...
    @property
    def gorg(self) -> IVector4: ...
    @property
    def gorb(self) -> IVector4: ...
    @property
    def gora(self) -> IVector4: ...
    @property
    def goro(self) -> IVector4: ...
    @property
    def gorl(self) -> IVector4: ...
    @property
    def gogr(self) -> IVector4: ...
    @property
    def gogg(self) -> IVector4: ...
    @property
    def gogb(self) -> IVector4: ...
    @property
    def goga(self) -> IVector4: ...
    @property
    def gogo(self) -> IVector4: ...
    @property
    def gogl(self) -> IVector4: ...
    @property
    def gobr(self) -> IVector4: ...
    @property
    def gobg(self) -> IVector4: ...
    @property
    def gobb(self) -> IVector4: ...
    @property
    def goba(self) -> IVector4: ...
    @property
    def gobo(self) -> IVector4: ...
    @property
    def gobl(self) -> IVector4: ...
    @property
    def goar(self) -> IVector4: ...
    @property
    def goag(self) -> IVector4: ...
    @property
    def goab(self) -> IVector4: ...
    @property
    def goaa(self) -> IVector4: ...
    @property
    def goao(self) -> IVector4: ...
    @property
    def goal(self) -> IVector4: ...
    @property
    def goor(self) -> IVector4: ...
    @property
    def goog(self) -> IVector4: ...
    @property
    def goob(self) -> IVector4: ...
    @property
    def gooa(self) -> IVector4: ...
    @property
    def gooo(self) -> IVector4: ...
    @property
    def gool(self) -> IVector4: ...
    @property
    def golr(self) -> IVector4: ...
    @property
    def golg(self) -> IVector4: ...
    @property
    def golb(self) -> IVector4: ...
    @property
    def gola(self) -> IVector4: ...
    @property
    def golo(self) -> IVector4: ...
    @property
    def goll(self) -> IVector4: ...
    @property
    def glrr(self) -> IVector4: ...
    @property
    def glrg(self) -> IVector4: ...
    @property
    def glrb(self) -> IVector4: ...
    @property
    def glra(self) -> IVector4: ...
    @property
    def glro(self) -> IVector4: ...
    @property
    def glrl(self) -> IVector4: ...
    @property
    def glgr(self) -> IVector4: ...
    @property
    def glgg(self) -> IVector4: ...
    @property
    def glgb(self) -> IVector4: ...
    @property
    def glga(self) -> IVector4: ...
    @property
    def glgo(self) -> IVector4: ...
    @property
    def glgl(self) -> IVector4: ...
    @property
    def glbr(self) -> IVector4: ...
    @property
    def glbg(self) -> IVector4: ...
    @property
    def glbb(self) -> IVector4: ...
    @property
    def glba(self) -> IVector4: ...
    @property
    def glbo(self) -> IVector4: ...
    @property
    def glbl(self) -> IVector4: ...
    @property
    def glar(self) -> IVector4: ...
    @property
    def glag(self) -> IVector4: ...
    @property
    def glab(self) -> IVector4: ...
    @property
    def glaa(self) -> IVector4: ...
    @property
    def glao(self) -> IVector4: ...
    @property
    def glal(self) -> IVector4: ...
    @property
    def glor(self) -> IVector4: ...
    @property
    def glog(self) -> IVector4: ...
    @property
    def glob(self) -> IVector4: ...
    @property
    def gloa(self) -> IVector4: ...
    @property
    def gloo(self) -> IVector4: ...
    @property
    def glol(self) -> IVector4: ...
    @property
    def gllr(self) -> IVector4: ...
    @property
    def gllg(self) -> IVector4: ...
    @property
    def gllb(self) -> IVector4: ...
    @property
    def glla(self) -> IVector4: ...
    @property
    def gllo(self) -> IVector4: ...
    @property
    def glll(self) -> IVector4: ...
    @property
    def brrr(self) -> IVector4: ...
    @property
    def brrg(self) -> IVector4: ...
    @property
    def brrb(self) -> IVector4: ...
    @property
    def brra(self) -> IVector4: ...
    @property
    def brro(self) -> IVector4: ...
    @property
    def brrl(self) -> IVector4: ...
    @property
    def brgr(self) -> IVector4: ...
    @property
    def brgg(self) -> IVector4: ...
    @property
    def brgb(self) -> IVector4: ...
    @property
    def brga(self) -> IVector4: ...
    @property
    def brgo(self) -> IVector4: ...
    @property
    def brgl(self) -> IVector4: ...
    @property
    def brbr(self) -> IVector4: ...
    @property
    def brbg(self) -> IVector4: ...
    @property
    def brbb(self) -> IVector4: ...
    @property
    def brba(self) -> IVector4: ...
    @property
    def brbo(self) -> IVector4: ...
    @property
    def brbl(self) -> IVector4: ...
    @property
    def brar(self) -> IVector4: ...
    @property
    def brag(self) -> IVector4: ...
    @property
    def brab(self) -> IVector4: ...
    @property
    def braa(self) -> IVector4: ...
    @property
    def brao(self) -> IVector4: ...
    @property
    def bral(self) -> IVector4: ...
    @property
    def bror(self) -> IVector4: ...
    @property
    def brog(self) -> IVector4: ...
    @property
    def brob(self) -> IVector4: ...
    @property
    def broa(self) -> IVector4: ...
    @property
    def broo(self) -> IVector4: ...
    @property
    def brol(self) -> IVector4: ...
    @property
    def brlr(self) -> IVector4: ...
    @property
    def brlg(self) -> IVector4: ...
    @property
    def brlb(self) -> IVector4: ...
    @property
    def brla(self) -> IVector4: ...
    @property
    def brlo(self) -> IVector4: ...
    @property
    def brll(self) -> IVector4: ...
    @property
    def bgrr(self) -> IVector4: ...
    @property
    def bgrg(self) -> IVector4: ...
    @property
    def bgrb(self) -> IVector4: ...
    @property
    def bgra(self) -> IVector4: ...
    @property
    def bgro(self) -> IVector4: ...
    @property
    def bgrl(self) -> IVector4: ...
    @property
    def bggr(self) -> IVector4: ...
    @property
    def bggg(self) -> IVector4: ...
    @property
    def bggb(self) -> IVector4: ...
    @property
    def bgga(self) -> IVector4: ...
    @property
    def bggo(self) -> IVector4: ...
    @property
    def bggl(self) -> IVector4: ...
    @property
    def bgbr(self) -> IVector4: ...
    @property
    def bgbg(self) -> IVector4: ...
    @property
    def bgbb(self) -> IVector4: ...
    @property
    def bgba(self) -> IVector4: ...
    @property
    def bgbo(self) -> IVector4: ...
    @property
    def bgbl(self) -> IVector4: ...
    @property
    def bgar(self) -> IVector4: ...
    @property
    def bgag(self) -> IVector4: ...
    @property
    def bgab(self) -> IVector4: ...
    @property
    def bgaa(self) -> IVector4: ...
    @property
    def bgao(self) -> IVector4: ...
    @property
    def bgal(self) -> IVector4: ...
    @property
    def bgor(self) -> IVector4: ...
    @property
    def bgog(self) -> IVector4: ...
    @property
    def bgob(self) -> IVector4: ...
    @property
    def bgoa(self) -> IVector4: ...
    @property
    def bgoo(self) -> IVector4: ...
    @property
    def bgol(self) -> IVector4: ...
    @property
    def bglr(self) -> IVector4: ...
    @property
    def bglg(self) -> IVector4: ...
    @property
    def bglb(self) -> IVector4: ...
    @property
    def bgla(self) -> IVector4: ...
    @property
    def bglo(self) -> IVector4: ...
    @property
    def bgll(self) -> IVector4: ...
    @property
    def bbrr(self) -> IVector4: ...
    @property
    def bbrg(self) -> IVector4: ...
    @property
    def bbrb(self) -> IVector4: ...
    @property
    def bbra(self) -> IVector4: ...
    @property
    def bbro(self) -> IVector4: ...
    @property
    def bbrl(self) -> IVector4: ...
    @property
    def bbgr(self) -> IVector4: ...
    @property
    def bbgg(self) -> IVector4: ...
    @property
    def bbgb(self) -> IVector4: ...
    @property
    def bbga(self) -> IVector4: ...
    @property
    def bbgo(self) -> IVector4: ...
    @property
    def bbgl(self) -> IVector4: ...
    @property
    def bbbr(self) -> IVector4: ...
    @property
    def bbbg(self) -> IVector4: ...
    @property
    def bbbb(self) -> IVector4: ...
    @property
    def bbba(self) -> IVector4: ...
    @property
    def bbbo(self) -> IVector4: ...
    @property
    def bbbl(self) -> IVector4: ...
    @property
    def bbar(self) -> IVector4: ...
    @property
    def bbag(self) -> IVector4: ...
    @property
    def bbab(self) -> IVector4: ...
    @property
    def bbaa(self) -> IVector4: ...
    @property
    def bbao(self) -> IVector4: ...
    @property
    def bbal(self) -> IVector4: ...
    @property
    def bbor(self) -> IVector4: ...
    @property
    def bbog(self) -> IVector4: ...
    @property
    def bbob(self) -> IVector4: ...
    @property
    def bboa(self) -> IVector4: ...
    @property
    def bboo(self) -> IVector4: ...
    @property
    def bbol(self) -> IVector4: ...
    @property
    def bblr(self) -> IVector4: ...
    @property
    def bblg(self) -> IVector4: ...
    @property
    def bblb(self) -> IVector4: ...
    @property
    def bbla(self) -> IVector4: ...
    @property
    def bblo(self) -> IVector4: ...
    @property
    def bbll(self) -> IVector4: ...
    @property
    def barr(self) -> IVector4: ...
    @property
    def barg(self) -> IVector4: ...
    @property
    def barb(self) -> IVector4: ...
    @property
    def bara(self) -> IVector4: ...
    @property
    def baro(self) -> IVector4: ...
    @property
    def barl(self) -> IVector4: ...
    @property
    def bagr(self) -> IVector4: ...
    @property
    def bagg(self) -> IVector4: ...
    @property
    def bagb(self) -> IVector4: ...
    @property
    def baga(self) -> IVector4: ...
    @property
    def bago(self) -> IVector4: ...
    @property
    def bagl(self) -> IVector4: ...
    @property
    def babr(self) -> IVector4: ...
    @property
    def babg(self) -> IVector4: ...
    @property
    def babb(self) -> IVector4: ...
    @property
    def baba(self) -> IVector4: ...
    @property
    def babo(self) -> IVector4: ...
    @property
    def babl(self) -> IVector4: ...
    @property
    def baar(self) -> IVector4: ...
    @property
    def baag(self) -> IVector4: ...
    @property
    def baab(self) -> IVector4: ...
    @property
    def baaa(self) -> IVector4: ...
    @property
    def baao(self) -> IVector4: ...
    @property
    def baal(self) -> IVector4: ...
    @property
    def baor(self) -> IVector4: ...
    @property
    def baog(self) -> IVector4: ...
    @property
    def baob(self) -> IVector4: ...
    @property
    def baoa(self) -> IVector4: ...
    @property
    def baoo(self) -> IVector4: ...
    @property
    def baol(self) -> IVector4: ...
    @property
    def balr(self) -> IVector4: ...
    @property
    def balg(self) -> IVector4: ...
    @property
    def balb(self) -> IVector4: ...
    @property
    def bala(self) -> IVector4: ...
    @property
    def balo(self) -> IVector4: ...
    @property
    def ball(self) -> IVector4: ...
    @property
    def borr(self) -> IVector4: ...
    @property
    def borg(self) -> IVector4: ...
    @property
    def borb(self) -> IVector4: ...
    @property
    def bora(self) -> IVector4: ...
    @property
    def boro(self) -> IVector4: ...
    @property
    def borl(self) -> IVector4: ...
    @property
    def bogr(self) -> IVector4: ...
    @property
    def bogg(self) -> IVector4: ...
    @property
    def bogb(self) -> IVector4: ...
    @property
    def boga(self) -> IVector4: ...
    @property
    def bogo(self) -> IVector4: ...
    @property
    def bogl(self) -> IVector4: ...
    @property
    def bobr(self) -> IVector4: ...
    @property
    def bobg(self) -> IVector4: ...
    @property
    def bobb(self) -> IVector4: ...
    @property
    def boba(self) -> IVector4: ...
    @property
    def bobo(self) -> IVector4: ...
    @property
    def bobl(self) -> IVector4: ...
    @property
    def boar(self) -> IVector4: ...
    @property
    def boag(self) -> IVector4: ...
    @property
    def boab(self) -> IVector4: ...
    @property
    def boaa(self) -> IVector4: ...
    @property
    def boao(self) -> IVector4: ...
    @property
    def boal(self) -> IVector4: ...
    @property
    def boor(self) -> IVector4: ...
    @property
    def boog(self) -> IVector4: ...
    @property
    def boob(self) -> IVector4: ...
    @property
    def booa(self) -> IVector4: ...
    @property
    def booo(self) -> IVector4: ...
    @property
    def bool(self) -> IVector4: ...
    @property
    def bolr(self) -> IVector4: ...
    @property
    def bolg(self) -> IVector4: ...
    @property
    def bolb(self) -> IVector4: ...
    @property
    def bola(self) -> IVector4: ...
    @property
    def bolo(self) -> IVector4: ...
    @property
    def boll(self) -> IVector4: ...
    @property
    def blrr(self) -> IVector4: ...
    @property
    def blrg(self) -> IVector4: ...
    @property
    def blrb(self) -> IVector4: ...
    @property
    def blra(self) -> IVector4: ...
    @property
    def blro(self) -> IVector4: ...
    @property
    def blrl(self) -> IVector4: ...
    @property
    def blgr(self) -> IVector4: ...
    @property
    def blgg(self) -> IVector4: ...
    @property
    def blgb(self) -> IVector4: ...
    @property
    def blga(self) -> IVector4: ...
    @property
    def blgo(self) -> IVector4: ...
    @property
    def blgl(self) -> IVector4: ...
    @property
    def blbr(self) -> IVector4: ...
    @property
    def blbg(self) -> IVector4: ...
    @property
    def blbb(self) -> IVector4: ...
    @property
    def blba(self) -> IVector4: ...
    @property
    def blbo(self) -> IVector4: ...
    @property
    def blbl(self) -> IVector4: ...
    @property
    def blar(self) -> IVector4: ...
    @property
    def blag(self) -> IVector4: ...
    @property
    def blab(self) -> IVector4: ...
    @property
    def blaa(self) -> IVector4: ...
    @property
    def blao(self) -> IVector4: ...
    @property
    def blal(self) -> IVector4: ...
    @property
    def blor(self) -> IVector4: ...
    @property
    def blog(self) -> IVector4: ...
    @property
    def blob(self) -> IVector4: ...
    @property
    def bloa(self) -> IVector4: ...
    @property
    def bloo(self) -> IVector4: ...
    @property
    def blol(self) -> IVector4: ...
    @property
    def bllr(self) -> IVector4: ...
    @property
    def bllg(self) -> IVector4: ...
    @property
    def bllb(self) -> IVector4: ...
    @property
    def blla(self) -> IVector4: ...
    @property
    def bllo(self) -> IVector4: ...
    @property
    def blll(self) -> IVector4: ...
    @property
    def arrr(self) -> IVector4: ...
    @property
    def arrg(self) -> IVector4: ...
    @property
    def arrb(self) -> IVector4: ...
    @property
    def arra(self) -> IVector4: ...
    @property
    def arro(self) -> IVector4: ...
    @property
    def arrl(self) -> IVector4: ...
    @property
    def argr(self) -> IVector4: ...
    @property
    def argg(self) -> IVector4: ...
    @property
    def argb(self) -> IVector4: ...
    @property
    def arga(self) -> IVector4: ...
    @property
    def argo(self) -> IVector4: ...
    @property
    def argl(self) -> IVector4: ...
    @property
    def arbr(self) -> IVector4: ...
    @property
    def arbg(self) -> IVector4: ...
    @property
    def arbb(self) -> IVector4: ...
    @property
    def arba(self) -> IVector4: ...
    @property
    def arbo(self) -> IVector4: ...
    @property
    def arbl(self) -> IVector4: ...
    @property
    def arar(self) -> IVector4: ...
    @property
    def arag(self) -> IVector4: ...
    @property
    def arab(self) -> IVector4: ...
    @property
    def araa(self) -> IVector4: ...
    @property
    def arao(self) -> IVector4: ...
    @property
    def aral(self) -> IVector4: ...
    @property
    def aror(self) -> IVector4: ...
    @property
    def arog(self) -> IVector4: ...
    @property
    def arob(self) -> IVector4: ...
    @property
    def aroa(self) -> IVector4: ...
    @property
    def aroo(self) -> IVector4: ...
    @property
    def arol(self) -> IVector4: ...
    @property
    def arlr(self) -> IVector4: ...
    @property
    def arlg(self) -> IVector4: ...
    @property
    def arlb(self) -> IVector4: ...
    @property
    def arla(self) -> IVector4: ...
    @property
    def arlo(self) -> IVector4: ...
    @property
    def arll(self) -> IVector4: ...
    @property
    def agrr(self) -> IVector4: ...
    @property
    def agrg(self) -> IVector4: ...
    @property
    def agrb(self) -> IVector4: ...
    @property
    def agra(self) -> IVector4: ...
    @property
    def agro(self) -> IVector4: ...
    @property
    def agrl(self) -> IVector4: ...
    @property
    def aggr(self) -> IVector4: ...
    @property
    def aggg(self) -> IVector4: ...
    @property
    def aggb(self) -> IVector4: ...
    @property
    def agga(self) -> IVector4: ...
    @property
    def aggo(self) -> IVector4: ...
    @property
    def aggl(self) -> IVector4: ...
    @property
    def agbr(self) -> IVector4: ...
    @property
    def agbg(self) -> IVector4: ...
    @property
    def agbb(self) -> IVector4: ...
    @property
    def agba(self) -> IVector4: ...
    @property
    def agbo(self) -> IVector4: ...
    @property
    def agbl(self) -> IVector4: ...
    @property
    def agar(self) -> IVector4: ...
    @property
    def agag(self) -> IVector4: ...
    @property
    def agab(self) -> IVector4: ...
    @property
    def agaa(self) -> IVector4: ...
    @property
    def agao(self) -> IVector4: ...
    @property
    def agal(self) -> IVector4: ...
    @property
    def agor(self) -> IVector4: ...
    @property
    def agog(self) -> IVector4: ...
    @property
    def agob(self) -> IVector4: ...
    @property
    def agoa(self) -> IVector4: ...
    @property
    def agoo(self) -> IVector4: ...
    @property
    def agol(self) -> IVector4: ...
    @property
    def aglr(self) -> IVector4: ...
    @property
    def aglg(self) -> IVector4: ...
    @property
    def aglb(self) -> IVector4: ...
    @property
    def agla(self) -> IVector4: ...
    @property
    def aglo(self) -> IVector4: ...
    @property
    def agll(self) -> IVector4: ...
    @property
    def abrr(self) -> IVector4: ...
    @property
    def abrg(self) -> IVector4: ...
    @property
    def abrb(self) -> IVector4: ...
    @property
    def abra(self) -> IVector4: ...
    @property
    def abro(self) -> IVector4: ...
    @property
    def abrl(self) -> IVector4: ...
    @property
    def abgr(self) -> IVector4: ...
    @property
    def abgg(self) -> IVector4: ...
    @property
    def abgb(self) -> IVector4: ...
    @property
    def abga(self) -> IVector4: ...
    @property
    def abgo(self) -> IVector4: ...
    @property
    def abgl(self) -> IVector4: ...
    @property
    def abbr(self) -> IVector4: ...
    @property
    def abbg(self) -> IVector4: ...
    @property
    def abbb(self) -> IVector4: ...
    @property
    def abba(self) -> IVector4: ...
    @property
    def abbo(self) -> IVector4: ...
    @property
    def abbl(self) -> IVector4: ...
    @property
    def abar(self) -> IVector4: ...
    @property
    def abag(self) -> IVector4: ...
    @property
    def abab(self) -> IVector4: ...
    @property
    def abaa(self) -> IVector4: ...
    @property
    def abao(self) -> IVector4: ...
    @property
    def abal(self) -> IVector4: ...
    @property
    def abor(self) -> IVector4: ...
    @property
    def abog(self) -> IVector4: ...
    @property
    def abob(self) -> IVector4: ...
    @property
    def aboa(self) -> IVector4: ...
    @property
    def aboo(self) -> IVector4: ...
    @property
    def abol(self) -> IVector4: ...
    @property
    def ablr(self) -> IVector4: ...
    @property
    def ablg(self) -> IVector4: ...
    @property
    def ablb(self) -> IVector4: ...
    @property
    def abla(self) -> IVector4: ...
    @property
    def ablo(self) -> IVector4: ...
    @property
    def abll(self) -> IVector4: ...
    @property
    def aarr(self) -> IVector4: ...
    @property
    def aarg(self) -> IVector4: ...
    @property
    def aarb(self) -> IVector4: ...
    @property
    def aara(self) -> IVector4: ...
    @property
    def aaro(self) -> IVector4: ...
    @property
    def aarl(self) -> IVector4: ...
    @property
    def aagr(self) -> IVector4: ...
    @property
    def aagg(self) -> IVector4: ...
    @property
    def aagb(self) -> IVector4: ...
    @property
    def aaga(self) -> IVector4: ...
    @property
    def aago(self) -> IVector4: ...
    @property
    def aagl(self) -> IVector4: ...
    @property
    def aabr(self) -> IVector4: ...
    @property
    def aabg(self) -> IVector4: ...
    @property
    def aabb(self) -> IVector4: ...
    @property
    def aaba(self) -> IVector4: ...
    @property
    def aabo(self) -> IVector4: ...
    @property
    def aabl(self) -> IVector4: ...
    @property
    def aaar(self) -> IVector4: ...
    @property
    def aaag(self) -> IVector4: ...
    @property
    def aaab(self) -> IVector4: ...
    @property
    def aaaa(self) -> IVector4: ...
    @property
    def aaao(self) -> IVector4: ...
    @property
    def aaal(self) -> IVector4: ...
    @property
    def aaor(self) -> IVector4: ...
    @property
    def aaog(self) -> IVector4: ...
    @property
    def aaob(self) -> IVector4: ...
    @property
    def aaoa(self) -> IVector4: ...
    @property
    def aaoo(self) -> IVector4: ...
    @property
    def aaol(self) -> IVector4: ...
    @property
    def aalr(self) -> IVector4: ...
    @property
    def aalg(self) -> IVector4: ...
    @property
    def aalb(self) -> IVector4: ...
    @property
    def aala(self) -> IVector4: ...
    @property
    def aalo(self) -> IVector4: ...
    @property
    def aall(self) -> IVector4: ...
    @property
    def aorr(self) -> IVector4: ...
    @property
    def aorg(self) -> IVector4: ...
    @property
    def aorb(self) -> IVector4: ...
    @property
    def aora(self) -> IVector4: ...
    @property
    def aoro(self) -> IVector4: ...
    @property
    def aorl(self) -> IVector4: ...
    @property
    def aogr(self) -> IVector4: ...
    @property
    def aogg(self) -> IVector4: ...
    @property
    def aogb(self) -> IVector4: ...
    @property
    def aoga(self) -> IVector4: ...
    @property
    def aogo(self) -> IVector4: ...
    @property
    def aogl(self) -> IVector4: ...
    @property
    def aobr(self) -> IVector4: ...
    @property
    def aobg(self) -> IVector4: ...
    @property
    def aobb(self) -> IVector4: ...
    @property
    def aoba(self) -> IVector4: ...
    @property
    def aobo(self) -> IVector4: ...
    @property
    def aobl(self) -> IVector4: ...
    @property
    def aoar(self) -> IVector4: ...
    @property
    def aoag(self) -> IVector4: ...
    @property
    def aoab(self) -> IVector4: ...
    @property
    def aoaa(self) -> IVector4: ...
    @property
    def aoao(self) -> IVector4: ...
    @property
    def aoal(self) -> IVector4: ...
    @property
    def aoor(self) -> IVector4: ...
    @property
    def aoog(self) -> IVector4: ...
    @property
    def aoob(self) -> IVector4: ...
    @property
    def aooa(self) -> IVector4: ...
    @property
    def aooo(self) -> IVector4: ...
    @property
    def aool(self) -> IVector4: ...
    @property
    def aolr(self) -> IVector4: ...
    @property
    def aolg(self) -> IVector4: ...
    @property
    def aolb(self) -> IVector4: ...
    @property
    def aola(self) -> IVector4: ...
    @property
    def aolo(self) -> IVector4: ...
    @property
    def aoll(self) -> IVector4: ...
    @property
    def alrr(self) -> IVector4: ...
    @property
    def alrg(self) -> IVector4: ...
    @property
    def alrb(self) -> IVector4: ...
    @property
    def alra(self) -> IVector4: ...
    @property
    def alro(self) -> IVector4: ...
    @property
    def alrl(self) -> IVector4: ...
    @property
    def algr(self) -> IVector4: ...
    @property
    def algg(self) -> IVector4: ...
    @property
    def algb(self) -> IVector4: ...
    @property
    def alga(self) -> IVector4: ...
    @property
    def algo(self) -> IVector4: ...
    @property
    def algl(self) -> IVector4: ...
    @property
    def albr(self) -> IVector4: ...
    @property
    def albg(self) -> IVector4: ...
    @property
    def albb(self) -> IVector4: ...
    @property
    def alba(self) -> IVector4: ...
    @property
    def albo(self) -> IVector4: ...
    @property
    def albl(self) -> IVector4: ...
    @property
    def alar(self) -> IVector4: ...
    @property
    def alag(self) -> IVector4: ...
    @property
    def alab(self) -> IVector4: ...
    @property
    def alaa(self) -> IVector4: ...
    @property
    def alao(self) -> IVector4: ...
    @property
    def alal(self) -> IVector4: ...
    @property
    def alor(self) -> IVector4: ...
    @property
    def alog(self) -> IVector4: ...
    @property
    def alob(self) -> IVector4: ...
    @property
    def aloa(self) -> IVector4: ...
    @property
    def aloo(self) -> IVector4: ...
    @property
    def alol(self) -> IVector4: ...
    @property
    def allr(self) -> IVector4: ...
    @property
    def allg(self) -> IVector4: ...
    @property
    def allb(self) -> IVector4: ...
    @property
    def alla(self) -> IVector4: ...
    @property
    def allo(self) -> IVector4: ...
    @property
    def alll(self) -> IVector4: ...
    @property
    def orrr(self) -> IVector4: ...
    @property
    def orrg(self) -> IVector4: ...
    @property
    def orrb(self) -> IVector4: ...
    @property
    def orra(self) -> IVector4: ...
    @property
    def orro(self) -> IVector4: ...
    @property
    def orrl(self) -> IVector4: ...
    @property
    def orgr(self) -> IVector4: ...
    @property
    def orgg(self) -> IVector4: ...
    @property
    def orgb(self) -> IVector4: ...
    @property
    def orga(self) -> IVector4: ...
    @property
    def orgo(self) -> IVector4: ...
    @property
    def orgl(self) -> IVector4: ...
    @property
    def orbr(self) -> IVector4: ...
    @property
    def orbg(self) -> IVector4: ...
    @property
    def orbb(self) -> IVector4: ...
    @property
    def orba(self) -> IVector4: ...
    @property
    def orbo(self) -> IVector4: ...
    @property
    def orbl(self) -> IVector4: ...
    @property
    def orar(self) -> IVector4: ...
    @property
    def orag(self) -> IVector4: ...
    @property
    def orab(self) -> IVector4: ...
    @property
    def oraa(self) -> IVector4: ...
    @property
    def orao(self) -> IVector4: ...
    @property
    def oral(self) -> IVector4: ...
    @property
    def oror(self) -> IVector4: ...
    @property
    def orog(self) -> IVector4: ...
    @property
    def orob(self) -> IVector4: ...
    @property
    def oroa(self) -> IVector4: ...
    @property
    def oroo(self) -> IVector4: ...
    @property
    def orol(self) -> IVector4: ...
    @property
    def orlr(self) -> IVector4: ...
    @property
    def orlg(self) -> IVector4: ...
    @property
    def orlb(self) -> IVector4: ...
    @property
    def orla(self) -> IVector4: ...
    @property
    def orlo(self) -> IVector4: ...
    @property
    def orll(self) -> IVector4: ...
    @property
    def ogrr(self) -> IVector4: ...
    @property
    def ogrg(self) -> IVector4: ...
    @property
    def ogrb(self) -> IVector4: ...
    @property
    def ogra(self) -> IVector4: ...
    @property
    def ogro(self) -> IVector4: ...
    @property
    def ogrl(self) -> IVector4: ...
    @property
    def oggr(self) -> IVector4: ...
    @property
    def oggg(self) -> IVector4: ...
    @property
    def oggb(self) -> IVector4: ...
    @property
    def ogga(self) -> IVector4: ...
    @property
    def oggo(self) -> IVector4: ...
    @property
    def oggl(self) -> IVector4: ...
    @property
    def ogbr(self) -> IVector4: ...
    @property
    def ogbg(self) -> IVector4: ...
    @property
    def ogbb(self) -> IVector4: ...
    @property
    def ogba(self) -> IVector4: ...
    @property
    def ogbo(self) -> IVector4: ...
    @property
    def ogbl(self) -> IVector4: ...
    @property
    def ogar(self) -> IVector4: ...
    @property
    def ogag(self) -> IVector4: ...
    @property
    def ogab(self) -> IVector4: ...
    @property
    def ogaa(self) -> IVector4: ...
    @property
    def ogao(self) -> IVector4: ...
    @property
    def ogal(self) -> IVector4: ...
    @property
    def ogor(self) -> IVector4: ...
    @property
    def ogog(self) -> IVector4: ...
    @property
    def ogob(self) -> IVector4: ...
    @property
    def ogoa(self) -> IVector4: ...
    @property
    def ogoo(self) -> IVector4: ...
    @property
    def ogol(self) -> IVector4: ...
    @property
    def oglr(self) -> IVector4: ...
    @property
    def oglg(self) -> IVector4: ...
    @property
    def oglb(self) -> IVector4: ...
    @property
    def ogla(self) -> IVector4: ...
    @property
    def oglo(self) -> IVector4: ...
    @property
    def ogll(self) -> IVector4: ...
    @property
    def obrr(self) -> IVector4: ...
    @property
    def obrg(self) -> IVector4: ...
    @property
    def obrb(self) -> IVector4: ...
    @property
    def obra(self) -> IVector4: ...
    @property
    def obro(self) -> IVector4: ...
    @property
    def obrl(self) -> IVector4: ...
    @property
    def obgr(self) -> IVector4: ...
    @property
    def obgg(self) -> IVector4: ...
    @property
    def obgb(self) -> IVector4: ...
    @property
    def obga(self) -> IVector4: ...
    @property
    def obgo(self) -> IVector4: ...
    @property
    def obgl(self) -> IVector4: ...
    @property
    def obbr(self) -> IVector4: ...
    @property
    def obbg(self) -> IVector4: ...
    @property
    def obbb(self) -> IVector4: ...
    @property
    def obba(self) -> IVector4: ...
    @property
    def obbo(self) -> IVector4: ...
    @property
    def obbl(self) -> IVector4: ...
    @property
    def obar(self) -> IVector4: ...
    @property
    def obag(self) -> IVector4: ...
    @property
    def obab(self) -> IVector4: ...
    @property
    def obaa(self) -> IVector4: ...
    @property
    def obao(self) -> IVector4: ...
    @property
    def obal(self) -> IVector4: ...
    @property
    def obor(self) -> IVector4: ...
    @property
    def obog(self) -> IVector4: ...
    @property
    def obob(self) -> IVector4: ...
    @property
    def oboa(self) -> IVector4: ...
    @property
    def oboo(self) -> IVector4: ...
    @property
    def obol(self) -> IVector4: ...
    @property
    def oblr(self) -> IVector4: ...
    @property
    def oblg(self) -> IVector4: ...
    @property
    def oblb(self) -> IVector4: ...
    @property
    def obla(self) -> IVector4: ...
    @property
    def oblo(self) -> IVector4: ...
    @property
    def obll(self) -> IVector4: ...
    @property
    def oarr(self) -> IVector4: ...
    @property
    def oarg(self) -> IVector4: ...
    @property
    def oarb(self) -> IVector4: ...
    @property
    def oara(self) -> IVector4: ...
    @property
    def oaro(self) -> IVector4: ...
    @property
    def oarl(self) -> IVector4: ...
    @property
    def oagr(self) -> IVector4: ...
    @property
    def oagg(self) -> IVector4: ...
    @property
    def oagb(self) -> IVector4: ...
    @property
    def oaga(self) -> IVector4: ...
    @property
    def oago(self) -> IVector4: ...
    @property
    def oagl(self) -> IVector4: ...
    @property
    def oabr(self) -> IVector4: ...
    @property
    def oabg(self) -> IVector4: ...
    @property
    def oabb(self) -> IVector4: ...
    @property
    def oaba(self) -> IVector4: ...
    @property
    def oabo(self) -> IVector4: ...
    @property
    def oabl(self) -> IVector4: ...
    @property
    def oaar(self) -> IVector4: ...
    @property
    def oaag(self) -> IVector4: ...
    @property
    def oaab(self) -> IVector4: ...
    @property
    def oaaa(self) -> IVector4: ...
    @property
    def oaao(self) -> IVector4: ...
    @property
    def oaal(self) -> IVector4: ...
    @property
    def oaor(self) -> IVector4: ...
    @property
    def oaog(self) -> IVector4: ...
    @property
    def oaob(self) -> IVector4: ...
    @property
    def oaoa(self) -> IVector4: ...
    @property
    def oaoo(self) -> IVector4: ...
    @property
    def oaol(self) -> IVector4: ...
    @property
    def oalr(self) -> IVector4: ...
    @property
    def oalg(self) -> IVector4: ...
    @property
    def oalb(self) -> IVector4: ...
    @property
    def oala(self) -> IVector4: ...
    @property
    def oalo(self) -> IVector4: ...
    @property
    def oall(self) -> IVector4: ...
    @property
    def oorr(self) -> IVector4: ...
    @property
    def oorg(self) -> IVector4: ...
    @property
    def oorb(self) -> IVector4: ...
    @property
    def oora(self) -> IVector4: ...
    @property
    def ooro(self) -> IVector4: ...
    @property
    def oorl(self) -> IVector4: ...
    @property
    def oogr(self) -> IVector4: ...
    @property
    def oogg(self) -> IVector4: ...
    @property
    def oogb(self) -> IVector4: ...
    @property
    def ooga(self) -> IVector4: ...
    @property
    def oogo(self) -> IVector4: ...
    @property
    def oogl(self) -> IVector4: ...
    @property
    def oobr(self) -> IVector4: ...
    @property
    def oobg(self) -> IVector4: ...
    @property
    def oobb(self) -> IVector4: ...
    @property
    def ooba(self) -> IVector4: ...
    @property
    def oobo(self) -> IVector4: ...
    @property
    def oobl(self) -> IVector4: ...
    @property
    def ooar(self) -> IVector4: ...
    @property
    def ooag(self) -> IVector4: ...
    @property
    def ooab(self) -> IVector4: ...
    @property
    def ooaa(self) -> IVector4: ...
    @property
    def ooao(self) -> IVector4: ...
    @property
    def ooal(self) -> IVector4: ...
    @property
    def ooor(self) -> IVector4: ...
    @property
    def ooog(self) -> IVector4: ...
    @property
    def ooob(self) -> IVector4: ...
    @property
    def oooa(self) -> IVector4: ...
    @property
    def oolr(self) -> IVector4: ...
    @property
    def oolg(self) -> IVector4: ...
    @property
    def oolb(self) -> IVector4: ...
    @property
    def oola(self) -> IVector4: ...
    @property
    def olrr(self) -> IVector4: ...
    @property
    def olrg(self) -> IVector4: ...
    @property
    def olrb(self) -> IVector4: ...
    @property
    def olra(self) -> IVector4: ...
    @property
    def olro(self) -> IVector4: ...
    @property
    def olrl(self) -> IVector4: ...
    @property
    def olgr(self) -> IVector4: ...
    @property
    def olgg(self) -> IVector4: ...
    @property
    def olgb(self) -> IVector4: ...
    @property
    def olga(self) -> IVector4: ...
    @property
    def olgo(self) -> IVector4: ...
    @property
    def olgl(self) -> IVector4: ...
    @property
    def olbr(self) -> IVector4: ...
    @property
    def olbg(self) -> IVector4: ...
    @property
    def olbb(self) -> IVector4: ...
    @property
    def olba(self) -> IVector4: ...
    @property
    def olbo(self) -> IVector4: ...
    @property
    def olbl(self) -> IVector4: ...
    @property
    def olar(self) -> IVector4: ...
    @property
    def olag(self) -> IVector4: ...
    @property
    def olab(self) -> IVector4: ...
    @property
    def olaa(self) -> IVector4: ...
    @property
    def olao(self) -> IVector4: ...
    @property
    def olal(self) -> IVector4: ...
    @property
    def olor(self) -> IVector4: ...
    @property
    def olog(self) -> IVector4: ...
    @property
    def olob(self) -> IVector4: ...
    @property
    def oloa(self) -> IVector4: ...
    @property
    def ollr(self) -> IVector4: ...
    @property
    def ollg(self) -> IVector4: ...
    @property
    def ollb(self) -> IVector4: ...
    @property
    def olla(self) -> IVector4: ...
    @property
    def lrrr(self) -> IVector4: ...
    @property
    def lrrg(self) -> IVector4: ...
    @property
    def lrrb(self) -> IVector4: ...
    @property
    def lrra(self) -> IVector4: ...
    @property
    def lrro(self) -> IVector4: ...
    @property
    def lrrl(self) -> IVector4: ...
    @property
    def lrgr(self) -> IVector4: ...
    @property
    def lrgg(self) -> IVector4: ...
    @property
    def lrgb(self) -> IVector4: ...
    @property
    def lrga(self) -> IVector4: ...
    @property
    def lrgo(self) -> IVector4: ...
    @property
    def lrgl(self) -> IVector4: ...
    @property
    def lrbr(self) -> IVector4: ...
    @property
    def lrbg(self) -> IVector4: ...
    @property
    def lrbb(self) -> IVector4: ...
    @property
    def lrba(self) -> IVector4: ...
    @property
    def lrbo(self) -> IVector4: ...
    @property
    def lrbl(self) -> IVector4: ...
    @property
    def lrar(self) -> IVector4: ...
    @property
    def lrag(self) -> IVector4: ...
    @property
    def lrab(self) -> IVector4: ...
    @property
    def lraa(self) -> IVector4: ...
    @property
    def lrao(self) -> IVector4: ...
    @property
    def lral(self) -> IVector4: ...
    @property
    def lror(self) -> IVector4: ...
    @property
    def lrog(self) -> IVector4: ...
    @property
    def lrob(self) -> IVector4: ...
    @property
    def lroa(self) -> IVector4: ...
    @property
    def lroo(self) -> IVector4: ...
    @property
    def lrol(self) -> IVector4: ...
    @property
    def lrlr(self) -> IVector4: ...
    @property
    def lrlg(self) -> IVector4: ...
    @property
    def lrlb(self) -> IVector4: ...
    @property
    def lrla(self) -> IVector4: ...
    @property
    def lrlo(self) -> IVector4: ...
    @property
    def lrll(self) -> IVector4: ...
    @property
    def lgrr(self) -> IVector4: ...
    @property
    def lgrg(self) -> IVector4: ...
    @property
    def lgrb(self) -> IVector4: ...
    @property
    def lgra(self) -> IVector4: ...
    @property
    def lgro(self) -> IVector4: ...
    @property
    def lgrl(self) -> IVector4: ...
    @property
    def lggr(self) -> IVector4: ...
    @property
    def lggg(self) -> IVector4: ...
    @property
    def lggb(self) -> IVector4: ...
    @property
    def lgga(self) -> IVector4: ...
    @property
    def lggo(self) -> IVector4: ...
    @property
    def lggl(self) -> IVector4: ...
    @property
    def lgbr(self) -> IVector4: ...
    @property
    def lgbg(self) -> IVector4: ...
    @property
    def lgbb(self) -> IVector4: ...
    @property
    def lgba(self) -> IVector4: ...
    @property
    def lgbo(self) -> IVector4: ...
    @property
    def lgbl(self) -> IVector4: ...
    @property
    def lgar(self) -> IVector4: ...
    @property
    def lgag(self) -> IVector4: ...
    @property
    def lgab(self) -> IVector4: ...
    @property
    def lgaa(self) -> IVector4: ...
    @property
    def lgao(self) -> IVector4: ...
    @property
    def lgal(self) -> IVector4: ...
    @property
    def lgor(self) -> IVector4: ...
    @property
    def lgog(self) -> IVector4: ...
    @property
    def lgob(self) -> IVector4: ...
    @property
    def lgoa(self) -> IVector4: ...
    @property
    def lgoo(self) -> IVector4: ...
    @property
    def lgol(self) -> IVector4: ...
    @property
    def lglr(self) -> IVector4: ...
    @property
    def lglg(self) -> IVector4: ...
    @property
    def lglb(self) -> IVector4: ...
    @property
    def lgla(self) -> IVector4: ...
    @property
    def lglo(self) -> IVector4: ...
    @property
    def lgll(self) -> IVector4: ...
    @property
    def lbrr(self) -> IVector4: ...
    @property
    def lbrg(self) -> IVector4: ...
    @property
    def lbrb(self) -> IVector4: ...
    @property
    def lbra(self) -> IVector4: ...
    @property
    def lbro(self) -> IVector4: ...
    @property
    def lbrl(self) -> IVector4: ...
    @property
    def lbgr(self) -> IVector4: ...
    @property
    def lbgg(self) -> IVector4: ...
    @property
    def lbgb(self) -> IVector4: ...
    @property
    def lbga(self) -> IVector4: ...
    @property
    def lbgo(self) -> IVector4: ...
    @property
    def lbgl(self) -> IVector4: ...
    @property
    def lbbr(self) -> IVector4: ...
    @property
    def lbbg(self) -> IVector4: ...
    @property
    def lbbb(self) -> IVector4: ...
    @property
    def lbba(self) -> IVector4: ...
    @property
    def lbbo(self) -> IVector4: ...
    @property
    def lbbl(self) -> IVector4: ...
    @property
    def lbar(self) -> IVector4: ...
    @property
    def lbag(self) -> IVector4: ...
    @property
    def lbab(self) -> IVector4: ...
    @property
    def lbaa(self) -> IVector4: ...
    @property
    def lbao(self) -> IVector4: ...
    @property
    def lbal(self) -> IVector4: ...
    @property
    def lbor(self) -> IVector4: ...
    @property
    def lbog(self) -> IVector4: ...
    @property
    def lbob(self) -> IVector4: ...
    @property
    def lboa(self) -> IVector4: ...
    @property
    def lboo(self) -> IVector4: ...
    @property
    def lbol(self) -> IVector4: ...
    @property
    def lblr(self) -> IVector4: ...
    @property
    def lblg(self) -> IVector4: ...
    @property
    def lblb(self) -> IVector4: ...
    @property
    def lbla(self) -> IVector4: ...
    @property
    def lblo(self) -> IVector4: ...
    @property
    def lbll(self) -> IVector4: ...
    @property
    def larr(self) -> IVector4: ...
    @property
    def larg(self) -> IVector4: ...
    @property
    def larb(self) -> IVector4: ...
    @property
    def lara(self) -> IVector4: ...
    @property
    def laro(self) -> IVector4: ...
    @property
    def larl(self) -> IVector4: ...
    @property
    def lagr(self) -> IVector4: ...
    @property
    def lagg(self) -> IVector4: ...
    @property
    def lagb(self) -> IVector4: ...
    @property
    def laga(self) -> IVector4: ...
    @property
    def lago(self) -> IVector4: ...
    @property
    def lagl(self) -> IVector4: ...
    @property
    def labr(self) -> IVector4: ...
    @property
    def labg(self) -> IVector4: ...
    @property
    def labb(self) -> IVector4: ...
    @property
    def laba(self) -> IVector4: ...
    @property
    def labo(self) -> IVector4: ...
    @property
    def labl(self) -> IVector4: ...
    @property
    def laar(self) -> IVector4: ...
    @property
    def laag(self) -> IVector4: ...
    @property
    def laab(self) -> IVector4: ...
    @property
    def laaa(self) -> IVector4: ...
    @property
    def laao(self) -> IVector4: ...
    @property
    def laal(self) -> IVector4: ...
    @property
    def laor(self) -> IVector4: ...
    @property
    def laog(self) -> IVector4: ...
    @property
    def laob(self) -> IVector4: ...
    @property
    def laoa(self) -> IVector4: ...
    @property
    def laoo(self) -> IVector4: ...
    @property
    def laol(self) -> IVector4: ...
    @property
    def lalr(self) -> IVector4: ...
    @property
    def lalg(self) -> IVector4: ...
    @property
    def lalb(self) -> IVector4: ...
    @property
    def lala(self) -> IVector4: ...
    @property
    def lalo(self) -> IVector4: ...
    @property
    def lall(self) -> IVector4: ...
    @property
    def lorr(self) -> IVector4: ...
    @property
    def lorg(self) -> IVector4: ...
    @property
    def lorb(self) -> IVector4: ...
    @property
    def lora(self) -> IVector4: ...
    @property
    def loro(self) -> IVector4: ...
    @property
    def lorl(self) -> IVector4: ...
    @property
    def logr(self) -> IVector4: ...
    @property
    def logg(self) -> IVector4: ...
    @property
    def logb(self) -> IVector4: ...
    @property
    def loga(self) -> IVector4: ...
    @property
    def logo(self) -> IVector4: ...
    @property
    def logl(self) -> IVector4: ...
    @property
    def lobr(self) -> IVector4: ...
    @property
    def lobg(self) -> IVector4: ...
    @property
    def lobb(self) -> IVector4: ...
    @property
    def loba(self) -> IVector4: ...
    @property
    def lobo(self) -> IVector4: ...
    @property
    def lobl(self) -> IVector4: ...
    @property
    def loar(self) -> IVector4: ...
    @property
    def loag(self) -> IVector4: ...
    @property
    def loab(self) -> IVector4: ...
    @property
    def loaa(self) -> IVector4: ...
    @property
    def loao(self) -> IVector4: ...
    @property
    def loal(self) -> IVector4: ...
    @property
    def loor(self) -> IVector4: ...
    @property
    def loog(self) -> IVector4: ...
    @property
    def loob(self) -> IVector4: ...
    @property
    def looa(self) -> IVector4: ...
    @property
    def lolr(self) -> IVector4: ...
    @property
    def lolg(self) -> IVector4: ...
    @property
    def lolb(self) -> IVector4: ...
    @property
    def lola(self) -> IVector4: ...
    @property
    def llrr(self) -> IVector4: ...
    @property
    def llrg(self) -> IVector4: ...
    @property
    def llrb(self) -> IVector4: ...
    @property
    def llra(self) -> IVector4: ...
    @property
    def llro(self) -> IVector4: ...
    @property
    def llrl(self) -> IVector4: ...
    @property
    def llgr(self) -> IVector4: ...
    @property
    def llgg(self) -> IVector4: ...
    @property
    def llgb(self) -> IVector4: ...
    @property
    def llga(self) -> IVector4: ...
    @property
    def llgo(self) -> IVector4: ...
    @property
    def llgl(self) -> IVector4: ...
    @property
    def llbr(self) -> IVector4: ...
    @property
    def llbg(self) -> IVector4: ...
    @property
    def llbb(self) -> IVector4: ...
    @property
    def llba(self) -> IVector4: ...
    @property
    def llbo(self) -> IVector4: ...
    @property
    def llbl(self) -> IVector4: ...
    @property
    def llar(self) -> IVector4: ...
    @property
    def llag(self) -> IVector4: ...
    @property
    def llab(self) -> IVector4: ...
    @property
    def llaa(self) -> IVector4: ...
    @property
    def llao(self) -> IVector4: ...
    @property
    def llal(self) -> IVector4: ...
    @property
    def llor(self) -> IVector4: ...
    @property
    def llog(self) -> IVector4: ...
    @property
    def llob(self) -> IVector4: ...
    @property
    def lloa(self) -> IVector4: ...
    @property
    def lllr(self) -> IVector4: ...
    @property
    def lllg(self) -> IVector4: ...
    @property
    def lllb(self) -> IVector4: ...
    @property
    def llla(self) -> IVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> IVector2: ...
    @property
    def st(self) -> IVector2: ...
    @property
    def sq(self) -> IVector2: ...
    @property
    def sp(self) -> IVector2: ...
    @property
    def so(self) -> IVector2: ...
    @property
    def sl(self) -> IVector2: ...
    @property
    def ts(self) -> IVector2: ...
    @property
    def tt(self) -> IVector2: ...
    @property
    def tq(self) -> IVector2: ...
    @property
    def tp(self) -> IVector2: ...
    @property
    def to(self) -> IVector2: ...
    @property
    def tl(self) -> IVector2: ...
    @property
    def qs(self) -> IVector2: ...
    @property
    def qt(self) -> IVector2: ...
    @property
    def qq(self) -> IVector2: ...
    @property
    def qp(self) -> IVector2: ...
    @property
    def qo(self) -> IVector2: ...
    @property
    def ql(self) -> IVector2: ...
    @property
    def ps(self) -> IVector2: ...
    @property
    def pt(self) -> IVector2: ...
    @property
    def pq(self) -> IVector2: ...
    @property
    def pp(self) -> IVector2: ...
    @property
    def po(self) -> IVector2: ...
    @property
    def pl(self) -> IVector2: ...
    @property
    def os(self) -> IVector2: ...
    @property
    def ot(self) -> IVector2: ...
    @property
    def oq(self) -> IVector2: ...
    @property
    def op(self) -> IVector2: ...
    @property
    def ls(self) -> IVector2: ...
    @property
    def lt(self) -> IVector2: ...
    @property
    def lq(self) -> IVector2: ...
    @property
    def lp(self) -> IVector2: ...
    @property
    def sss(self) -> IVector3: ...
    @property
    def sst(self) -> IVector3: ...
    @property
    def ssq(self) -> IVector3: ...
    @property
    def ssp(self) -> IVector3: ...
    @property
    def sso(self) -> IVector3: ...
    @property
    def ssl(self) -> IVector3: ...
    @property
    def sts(self) -> IVector3: ...
    @property
    def stt(self) -> IVector3: ...
    @property
    def stq(self) -> IVector3: ...
    @property
    def stp(self) -> IVector3: ...
    @property
    def sto(self) -> IVector3: ...
    @property
    def stl(self) -> IVector3: ...
    @property
    def sqs(self) -> IVector3: ...
    @property
    def sqt(self) -> IVector3: ...
    @property
    def sqq(self) -> IVector3: ...
    @property
    def sqp(self) -> IVector3: ...
    @property
    def sqo(self) -> IVector3: ...
    @property
    def sql(self) -> IVector3: ...
    @property
    def sps(self) -> IVector3: ...
    @property
    def spt(self) -> IVector3: ...
    @property
    def spq(self) -> IVector3: ...
    @property
    def spp(self) -> IVector3: ...
    @property
    def spo(self) -> IVector3: ...
    @property
    def spl(self) -> IVector3: ...
    @property
    def sos(self) -> IVector3: ...
    @property
    def sot(self) -> IVector3: ...
    @property
    def soq(self) -> IVector3: ...
    @property
    def sop(self) -> IVector3: ...
    @property
    def soo(self) -> IVector3: ...
    @property
    def sol(self) -> IVector3: ...
    @property
    def sls(self) -> IVector3: ...
    @property
    def slt(self) -> IVector3: ...
    @property
    def slq(self) -> IVector3: ...
    @property
    def slp(self) -> IVector3: ...
    @property
    def slo(self) -> IVector3: ...
    @property
    def sll(self) -> IVector3: ...
    @property
    def tss(self) -> IVector3: ...
    @property
    def tst(self) -> IVector3: ...
    @property
    def tsq(self) -> IVector3: ...
    @property
    def tsp(self) -> IVector3: ...
    @property
    def tso(self) -> IVector3: ...
    @property
    def tsl(self) -> IVector3: ...
    @property
    def tts(self) -> IVector3: ...
    @property
    def ttt(self) -> IVector3: ...
    @property
    def ttq(self) -> IVector3: ...
    @property
    def ttp(self) -> IVector3: ...
    @property
    def tto(self) -> IVector3: ...
    @property
    def ttl(self) -> IVector3: ...
    @property
    def tqs(self) -> IVector3: ...
    @property
    def tqt(self) -> IVector3: ...
    @property
    def tqq(self) -> IVector3: ...
    @property
    def tqp(self) -> IVector3: ...
    @property
    def tqo(self) -> IVector3: ...
    @property
    def tql(self) -> IVector3: ...
    @property
    def tps(self) -> IVector3: ...
    @property
    def tpt(self) -> IVector3: ...
    @property
    def tpq(self) -> IVector3: ...
    @property
    def tpp(self) -> IVector3: ...
    @property
    def tpo(self) -> IVector3: ...
    @property
    def tpl(self) -> IVector3: ...
    @property
    def tos(self) -> IVector3: ...
    @property
    def tot(self) -> IVector3: ...
    @property
    def toq(self) -> IVector3: ...
    @property
    def top(self) -> IVector3: ...
    @property
    def too(self) -> IVector3: ...
    @property
    def tol(self) -> IVector3: ...
    @property
    def tls(self) -> IVector3: ...
    @property
    def tlt(self) -> IVector3: ...
    @property
    def tlq(self) -> IVector3: ...
    @property
    def tlp(self) -> IVector3: ...
    @property
    def tlo(self) -> IVector3: ...
    @property
    def tll(self) -> IVector3: ...
    @property
    def qss(self) -> IVector3: ...
    @property
    def qst(self) -> IVector3: ...
    @property
    def qsq(self) -> IVector3: ...
    @property
    def qsp(self) -> IVector3: ...
    @property
    def qso(self) -> IVector3: ...
    @property
    def qsl(self) -> IVector3: ...
    @property
    def qts(self) -> IVector3: ...
    @property
    def qtt(self) -> IVector3: ...
    @property
    def qtq(self) -> IVector3: ...
    @property
    def qtp(self) -> IVector3: ...
    @property
    def qto(self) -> IVector3: ...
    @property
    def qtl(self) -> IVector3: ...
    @property
    def qqs(self) -> IVector3: ...
    @property
    def qqt(self) -> IVector3: ...
    @property
    def qqq(self) -> IVector3: ...
    @property
    def qqp(self) -> IVector3: ...
    @property
    def qqo(self) -> IVector3: ...
    @property
    def qql(self) -> IVector3: ...
    @property
    def qps(self) -> IVector3: ...
    @property
    def qpt(self) -> IVector3: ...
    @property
    def qpq(self) -> IVector3: ...
    @property
    def qpp(self) -> IVector3: ...
    @property
    def qpo(self) -> IVector3: ...
    @property
    def qpl(self) -> IVector3: ...
    @property
    def qos(self) -> IVector3: ...
    @property
    def qot(self) -> IVector3: ...
    @property
    def qoq(self) -> IVector3: ...
    @property
    def qop(self) -> IVector3: ...
    @property
    def qoo(self) -> IVector3: ...
    @property
    def qol(self) -> IVector3: ...
    @property
    def qls(self) -> IVector3: ...
    @property
    def qlt(self) -> IVector3: ...
    @property
    def qlq(self) -> IVector3: ...
    @property
    def qlp(self) -> IVector3: ...
    @property
    def qlo(self) -> IVector3: ...
    @property
    def qll(self) -> IVector3: ...
    @property
    def pss(self) -> IVector3: ...
    @property
    def pst(self) -> IVector3: ...
    @property
    def psq(self) -> IVector3: ...
    @property
    def psp(self) -> IVector3: ...
    @property
    def pso(self) -> IVector3: ...
    @property
    def psl(self) -> IVector3: ...
    @property
    def pts(self) -> IVector3: ...
    @property
    def ptt(self) -> IVector3: ...
    @property
    def ptq(self) -> IVector3: ...
    @property
    def ptp(self) -> IVector3: ...
    @property
    def pto(self) -> IVector3: ...
    @property
    def ptl(self) -> IVector3: ...
    @property
    def pqs(self) -> IVector3: ...
    @property
    def pqt(self) -> IVector3: ...
    @property
    def pqq(self) -> IVector3: ...
    @property
    def pqp(self) -> IVector3: ...
    @property
    def pqo(self) -> IVector3: ...
    @property
    def pql(self) -> IVector3: ...
    @property
    def pps(self) -> IVector3: ...
    @property
    def ppt(self) -> IVector3: ...
    @property
    def ppq(self) -> IVector3: ...
    @property
    def ppp(self) -> IVector3: ...
    @property
    def ppo(self) -> IVector3: ...
    @property
    def ppl(self) -> IVector3: ...
    @property
    def pos(self) -> IVector3: ...
    @property
    def pot(self) -> IVector3: ...
    @property
    def poq(self) -> IVector3: ...
    @property
    def pop(self) -> IVector3: ...
    @property
    def poo(self) -> IVector3: ...
    @property
    def pol(self) -> IVector3: ...
    @property
    def pls(self) -> IVector3: ...
    @property
    def plt(self) -> IVector3: ...
    @property
    def plq(self) -> IVector3: ...
    @property
    def plp(self) -> IVector3: ...
    @property
    def plo(self) -> IVector3: ...
    @property
    def pll(self) -> IVector3: ...
    @property
    def oss(self) -> IVector3: ...
    @property
    def ost(self) -> IVector3: ...
    @property
    def osq(self) -> IVector3: ...
    @property
    def osp(self) -> IVector3: ...
    @property
    def oso(self) -> IVector3: ...
    @property
    def osl(self) -> IVector3: ...
    @property
    def ots(self) -> IVector3: ...
    @property
    def ott(self) -> IVector3: ...
    @property
    def otq(self) -> IVector3: ...
    @property
    def otp(self) -> IVector3: ...
    @property
    def oto(self) -> IVector3: ...
    @property
    def otl(self) -> IVector3: ...
    @property
    def oqs(self) -> IVector3: ...
    @property
    def oqt(self) -> IVector3: ...
    @property
    def oqq(self) -> IVector3: ...
    @property
    def oqp(self) -> IVector3: ...
    @property
    def oqo(self) -> IVector3: ...
    @property
    def oql(self) -> IVector3: ...
    @property
    def ops(self) -> IVector3: ...
    @property
    def opt(self) -> IVector3: ...
    @property
    def opq(self) -> IVector3: ...
    @property
    def opp(self) -> IVector3: ...
    @property
    def opo(self) -> IVector3: ...
    @property
    def opl(self) -> IVector3: ...
    @property
    def oos(self) -> IVector3: ...
    @property
    def oot(self) -> IVector3: ...
    @property
    def ooq(self) -> IVector3: ...
    @property
    def oop(self) -> IVector3: ...
    @property
    def ols(self) -> IVector3: ...
    @property
    def olt(self) -> IVector3: ...
    @property
    def olq(self) -> IVector3: ...
    @property
    def olp(self) -> IVector3: ...
    @property
    def lss(self) -> IVector3: ...
    @property
    def lst(self) -> IVector3: ...
    @property
    def lsq(self) -> IVector3: ...
    @property
    def lsp(self) -> IVector3: ...
    @property
    def lso(self) -> IVector3: ...
    @property
    def lsl(self) -> IVector3: ...
    @property
    def lts(self) -> IVector3: ...
    @property
    def ltt(self) -> IVector3: ...
    @property
    def ltq(self) -> IVector3: ...
    @property
    def ltp(self) -> IVector3: ...
    @property
    def lto(self) -> IVector3: ...
    @property
    def ltl(self) -> IVector3: ...
    @property
    def lqs(self) -> IVector3: ...
    @property
    def lqt(self) -> IVector3: ...
    @property
    def lqq(self) -> IVector3: ...
    @property
    def lqp(self) -> IVector3: ...
    @property
    def lqo(self) -> IVector3: ...
    @property
    def lql(self) -> IVector3: ...
    @property
    def lps(self) -> IVector3: ...
    @property
    def lpt(self) -> IVector3: ...
    @property
    def lpq(self) -> IVector3: ...
    @property
    def lpp(self) -> IVector3: ...
    @property
    def lpo(self) -> IVector3: ...
    @property
    def lpl(self) -> IVector3: ...
    @property
    def los(self) -> IVector3: ...
    @property
    def lot(self) -> IVector3: ...
    @property
    def loq(self) -> IVector3: ...
    @property
    def lop(self) -> IVector3: ...
    @property
    def lls(self) -> IVector3: ...
    @property
    def llt(self) -> IVector3: ...
    @property
    def llq(self) -> IVector3: ...
    @property
    def llp(self) -> IVector3: ...
    @property
    def ssss(self) -> IVector4: ...
    @property
    def ssst(self) -> IVector4: ...
    @property
    def sssq(self) -> IVector4: ...
    @property
    def sssp(self) -> IVector4: ...
    @property
    def ssso(self) -> IVector4: ...
    @property
    def sssl(self) -> IVector4: ...
    @property
    def ssts(self) -> IVector4: ...
    @property
    def sstt(self) -> IVector4: ...
    @property
    def sstq(self) -> IVector4: ...
    @property
    def sstp(self) -> IVector4: ...
    @property
    def ssto(self) -> IVector4: ...
    @property
    def sstl(self) -> IVector4: ...
    @property
    def ssqs(self) -> IVector4: ...
    @property
    def ssqt(self) -> IVector4: ...
    @property
    def ssqq(self) -> IVector4: ...
    @property
    def ssqp(self) -> IVector4: ...
    @property
    def ssqo(self) -> IVector4: ...
    @property
    def ssql(self) -> IVector4: ...
    @property
    def ssps(self) -> IVector4: ...
    @property
    def sspt(self) -> IVector4: ...
    @property
    def sspq(self) -> IVector4: ...
    @property
    def sspp(self) -> IVector4: ...
    @property
    def sspo(self) -> IVector4: ...
    @property
    def sspl(self) -> IVector4: ...
    @property
    def ssos(self) -> IVector4: ...
    @property
    def ssot(self) -> IVector4: ...
    @property
    def ssoq(self) -> IVector4: ...
    @property
    def ssop(self) -> IVector4: ...
    @property
    def ssoo(self) -> IVector4: ...
    @property
    def ssol(self) -> IVector4: ...
    @property
    def ssls(self) -> IVector4: ...
    @property
    def sslt(self) -> IVector4: ...
    @property
    def sslq(self) -> IVector4: ...
    @property
    def sslp(self) -> IVector4: ...
    @property
    def sslo(self) -> IVector4: ...
    @property
    def ssll(self) -> IVector4: ...
    @property
    def stss(self) -> IVector4: ...
    @property
    def stst(self) -> IVector4: ...
    @property
    def stsq(self) -> IVector4: ...
    @property
    def stsp(self) -> IVector4: ...
    @property
    def stso(self) -> IVector4: ...
    @property
    def stsl(self) -> IVector4: ...
    @property
    def stts(self) -> IVector4: ...
    @property
    def sttt(self) -> IVector4: ...
    @property
    def sttq(self) -> IVector4: ...
    @property
    def sttp(self) -> IVector4: ...
    @property
    def stto(self) -> IVector4: ...
    @property
    def sttl(self) -> IVector4: ...
    @property
    def stqs(self) -> IVector4: ...
    @property
    def stqt(self) -> IVector4: ...
    @property
    def stqq(self) -> IVector4: ...
    @property
    def stqp(self) -> IVector4: ...
    @property
    def stqo(self) -> IVector4: ...
    @property
    def stql(self) -> IVector4: ...
    @property
    def stps(self) -> IVector4: ...
    @property
    def stpt(self) -> IVector4: ...
    @property
    def stpq(self) -> IVector4: ...
    @property
    def stpp(self) -> IVector4: ...
    @property
    def stpo(self) -> IVector4: ...
    @property
    def stpl(self) -> IVector4: ...
    @property
    def stos(self) -> IVector4: ...
    @property
    def stot(self) -> IVector4: ...
    @property
    def stoq(self) -> IVector4: ...
    @property
    def stop(self) -> IVector4: ...
    @property
    def stoo(self) -> IVector4: ...
    @property
    def stol(self) -> IVector4: ...
    @property
    def stls(self) -> IVector4: ...
    @property
    def stlt(self) -> IVector4: ...
    @property
    def stlq(self) -> IVector4: ...
    @property
    def stlp(self) -> IVector4: ...
    @property
    def stlo(self) -> IVector4: ...
    @property
    def stll(self) -> IVector4: ...
    @property
    def sqss(self) -> IVector4: ...
    @property
    def sqst(self) -> IVector4: ...
    @property
    def sqsq(self) -> IVector4: ...
    @property
    def sqsp(self) -> IVector4: ...
    @property
    def sqso(self) -> IVector4: ...
    @property
    def sqsl(self) -> IVector4: ...
    @property
    def sqts(self) -> IVector4: ...
    @property
    def sqtt(self) -> IVector4: ...
    @property
    def sqtq(self) -> IVector4: ...
    @property
    def sqtp(self) -> IVector4: ...
    @property
    def sqto(self) -> IVector4: ...
    @property
    def sqtl(self) -> IVector4: ...
    @property
    def sqqs(self) -> IVector4: ...
    @property
    def sqqt(self) -> IVector4: ...
    @property
    def sqqq(self) -> IVector4: ...
    @property
    def sqqp(self) -> IVector4: ...
    @property
    def sqqo(self) -> IVector4: ...
    @property
    def sqql(self) -> IVector4: ...
    @property
    def sqps(self) -> IVector4: ...
    @property
    def sqpt(self) -> IVector4: ...
    @property
    def sqpq(self) -> IVector4: ...
    @property
    def sqpp(self) -> IVector4: ...
    @property
    def sqpo(self) -> IVector4: ...
    @property
    def sqpl(self) -> IVector4: ...
    @property
    def sqos(self) -> IVector4: ...
    @property
    def sqot(self) -> IVector4: ...
    @property
    def sqoq(self) -> IVector4: ...
    @property
    def sqop(self) -> IVector4: ...
    @property
    def sqoo(self) -> IVector4: ...
    @property
    def sqol(self) -> IVector4: ...
    @property
    def sqls(self) -> IVector4: ...
    @property
    def sqlt(self) -> IVector4: ...
    @property
    def sqlq(self) -> IVector4: ...
    @property
    def sqlp(self) -> IVector4: ...
    @property
    def sqlo(self) -> IVector4: ...
    @property
    def sqll(self) -> IVector4: ...
    @property
    def spss(self) -> IVector4: ...
    @property
    def spst(self) -> IVector4: ...
    @property
    def spsq(self) -> IVector4: ...
    @property
    def spsp(self) -> IVector4: ...
    @property
    def spso(self) -> IVector4: ...
    @property
    def spsl(self) -> IVector4: ...
    @property
    def spts(self) -> IVector4: ...
    @property
    def sptt(self) -> IVector4: ...
    @property
    def sptq(self) -> IVector4: ...
    @property
    def sptp(self) -> IVector4: ...
    @property
    def spto(self) -> IVector4: ...
    @property
    def sptl(self) -> IVector4: ...
    @property
    def spqs(self) -> IVector4: ...
    @property
    def spqt(self) -> IVector4: ...
    @property
    def spqq(self) -> IVector4: ...
    @property
    def spqp(self) -> IVector4: ...
    @property
    def spqo(self) -> IVector4: ...
    @property
    def spql(self) -> IVector4: ...
    @property
    def spps(self) -> IVector4: ...
    @property
    def sppt(self) -> IVector4: ...
    @property
    def sppq(self) -> IVector4: ...
    @property
    def sppp(self) -> IVector4: ...
    @property
    def sppo(self) -> IVector4: ...
    @property
    def sppl(self) -> IVector4: ...
    @property
    def spos(self) -> IVector4: ...
    @property
    def spot(self) -> IVector4: ...
    @property
    def spoq(self) -> IVector4: ...
    @property
    def spop(self) -> IVector4: ...
    @property
    def spoo(self) -> IVector4: ...
    @property
    def spol(self) -> IVector4: ...
    @property
    def spls(self) -> IVector4: ...
    @property
    def splt(self) -> IVector4: ...
    @property
    def splq(self) -> IVector4: ...
    @property
    def splp(self) -> IVector4: ...
    @property
    def splo(self) -> IVector4: ...
    @property
    def spll(self) -> IVector4: ...
    @property
    def soss(self) -> IVector4: ...
    @property
    def sost(self) -> IVector4: ...
    @property
    def sosq(self) -> IVector4: ...
    @property
    def sosp(self) -> IVector4: ...
    @property
    def soso(self) -> IVector4: ...
    @property
    def sosl(self) -> IVector4: ...
    @property
    def sots(self) -> IVector4: ...
    @property
    def sott(self) -> IVector4: ...
    @property
    def sotq(self) -> IVector4: ...
    @property
    def sotp(self) -> IVector4: ...
    @property
    def soto(self) -> IVector4: ...
    @property
    def sotl(self) -> IVector4: ...
    @property
    def soqs(self) -> IVector4: ...
    @property
    def soqt(self) -> IVector4: ...
    @property
    def soqq(self) -> IVector4: ...
    @property
    def soqp(self) -> IVector4: ...
    @property
    def soqo(self) -> IVector4: ...
    @property
    def soql(self) -> IVector4: ...
    @property
    def sops(self) -> IVector4: ...
    @property
    def sopt(self) -> IVector4: ...
    @property
    def sopq(self) -> IVector4: ...
    @property
    def sopp(self) -> IVector4: ...
    @property
    def sopo(self) -> IVector4: ...
    @property
    def sopl(self) -> IVector4: ...
    @property
    def soos(self) -> IVector4: ...
    @property
    def soot(self) -> IVector4: ...
    @property
    def sooq(self) -> IVector4: ...
    @property
    def soop(self) -> IVector4: ...
    @property
    def sooo(self) -> IVector4: ...
    @property
    def sool(self) -> IVector4: ...
    @property
    def sols(self) -> IVector4: ...
    @property
    def solt(self) -> IVector4: ...
    @property
    def solq(self) -> IVector4: ...
    @property
    def solp(self) -> IVector4: ...
    @property
    def solo(self) -> IVector4: ...
    @property
    def soll(self) -> IVector4: ...
    @property
    def slss(self) -> IVector4: ...
    @property
    def slst(self) -> IVector4: ...
    @property
    def slsq(self) -> IVector4: ...
    @property
    def slsp(self) -> IVector4: ...
    @property
    def slso(self) -> IVector4: ...
    @property
    def slsl(self) -> IVector4: ...
    @property
    def slts(self) -> IVector4: ...
    @property
    def sltt(self) -> IVector4: ...
    @property
    def sltq(self) -> IVector4: ...
    @property
    def sltp(self) -> IVector4: ...
    @property
    def slto(self) -> IVector4: ...
    @property
    def sltl(self) -> IVector4: ...
    @property
    def slqs(self) -> IVector4: ...
    @property
    def slqt(self) -> IVector4: ...
    @property
    def slqq(self) -> IVector4: ...
    @property
    def slqp(self) -> IVector4: ...
    @property
    def slqo(self) -> IVector4: ...
    @property
    def slql(self) -> IVector4: ...
    @property
    def slps(self) -> IVector4: ...
    @property
    def slpt(self) -> IVector4: ...
    @property
    def slpq(self) -> IVector4: ...
    @property
    def slpp(self) -> IVector4: ...
    @property
    def slpo(self) -> IVector4: ...
    @property
    def slpl(self) -> IVector4: ...
    @property
    def slos(self) -> IVector4: ...
    @property
    def slot(self) -> IVector4: ...
    @property
    def sloq(self) -> IVector4: ...
    @property
    def slop(self) -> IVector4: ...
    @property
    def sloo(self) -> IVector4: ...
    @property
    def slol(self) -> IVector4: ...
    @property
    def slls(self) -> IVector4: ...
    @property
    def sllt(self) -> IVector4: ...
    @property
    def sllq(self) -> IVector4: ...
    @property
    def sllp(self) -> IVector4: ...
    @property
    def sllo(self) -> IVector4: ...
    @property
    def slll(self) -> IVector4: ...
    @property
    def tsss(self) -> IVector4: ...
    @property
    def tsst(self) -> IVector4: ...
    @property
    def tssq(self) -> IVector4: ...
    @property
    def tssp(self) -> IVector4: ...
    @property
    def tsso(self) -> IVector4: ...
    @property
    def tssl(self) -> IVector4: ...
    @property
    def tsts(self) -> IVector4: ...
    @property
    def tstt(self) -> IVector4: ...
    @property
    def tstq(self) -> IVector4: ...
    @property
    def tstp(self) -> IVector4: ...
    @property
    def tsto(self) -> IVector4: ...
    @property
    def tstl(self) -> IVector4: ...
    @property
    def tsqs(self) -> IVector4: ...
    @property
    def tsqt(self) -> IVector4: ...
    @property
    def tsqq(self) -> IVector4: ...
    @property
    def tsqp(self) -> IVector4: ...
    @property
    def tsqo(self) -> IVector4: ...
    @property
    def tsql(self) -> IVector4: ...
    @property
    def tsps(self) -> IVector4: ...
    @property
    def tspt(self) -> IVector4: ...
    @property
    def tspq(self) -> IVector4: ...
    @property
    def tspp(self) -> IVector4: ...
    @property
    def tspo(self) -> IVector4: ...
    @property
    def tspl(self) -> IVector4: ...
    @property
    def tsos(self) -> IVector4: ...
    @property
    def tsot(self) -> IVector4: ...
    @property
    def tsoq(self) -> IVector4: ...
    @property
    def tsop(self) -> IVector4: ...
    @property
    def tsoo(self) -> IVector4: ...
    @property
    def tsol(self) -> IVector4: ...
    @property
    def tsls(self) -> IVector4: ...
    @property
    def tslt(self) -> IVector4: ...
    @property
    def tslq(self) -> IVector4: ...
    @property
    def tslp(self) -> IVector4: ...
    @property
    def tslo(self) -> IVector4: ...
    @property
    def tsll(self) -> IVector4: ...
    @property
    def ttss(self) -> IVector4: ...
    @property
    def ttst(self) -> IVector4: ...
    @property
    def ttsq(self) -> IVector4: ...
    @property
    def ttsp(self) -> IVector4: ...
    @property
    def ttso(self) -> IVector4: ...
    @property
    def ttsl(self) -> IVector4: ...
    @property
    def ttts(self) -> IVector4: ...
    @property
    def tttt(self) -> IVector4: ...
    @property
    def tttq(self) -> IVector4: ...
    @property
    def tttp(self) -> IVector4: ...
    @property
    def ttto(self) -> IVector4: ...
    @property
    def tttl(self) -> IVector4: ...
    @property
    def ttqs(self) -> IVector4: ...
    @property
    def ttqt(self) -> IVector4: ...
    @property
    def ttqq(self) -> IVector4: ...
    @property
    def ttqp(self) -> IVector4: ...
    @property
    def ttqo(self) -> IVector4: ...
    @property
    def ttql(self) -> IVector4: ...
    @property
    def ttps(self) -> IVector4: ...
    @property
    def ttpt(self) -> IVector4: ...
    @property
    def ttpq(self) -> IVector4: ...
    @property
    def ttpp(self) -> IVector4: ...
    @property
    def ttpo(self) -> IVector4: ...
    @property
    def ttpl(self) -> IVector4: ...
    @property
    def ttos(self) -> IVector4: ...
    @property
    def ttot(self) -> IVector4: ...
    @property
    def ttoq(self) -> IVector4: ...
    @property
    def ttop(self) -> IVector4: ...
    @property
    def ttoo(self) -> IVector4: ...
    @property
    def ttol(self) -> IVector4: ...
    @property
    def ttls(self) -> IVector4: ...
    @property
    def ttlt(self) -> IVector4: ...
    @property
    def ttlq(self) -> IVector4: ...
    @property
    def ttlp(self) -> IVector4: ...
    @property
    def ttlo(self) -> IVector4: ...
    @property
    def ttll(self) -> IVector4: ...
    @property
    def tqss(self) -> IVector4: ...
    @property
    def tqst(self) -> IVector4: ...
    @property
    def tqsq(self) -> IVector4: ...
    @property
    def tqsp(self) -> IVector4: ...
    @property
    def tqso(self) -> IVector4: ...
    @property
    def tqsl(self) -> IVector4: ...
    @property
    def tqts(self) -> IVector4: ...
    @property
    def tqtt(self) -> IVector4: ...
    @property
    def tqtq(self) -> IVector4: ...
    @property
    def tqtp(self) -> IVector4: ...
    @property
    def tqto(self) -> IVector4: ...
    @property
    def tqtl(self) -> IVector4: ...
    @property
    def tqqs(self) -> IVector4: ...
    @property
    def tqqt(self) -> IVector4: ...
    @property
    def tqqq(self) -> IVector4: ...
    @property
    def tqqp(self) -> IVector4: ...
    @property
    def tqqo(self) -> IVector4: ...
    @property
    def tqql(self) -> IVector4: ...
    @property
    def tqps(self) -> IVector4: ...
    @property
    def tqpt(self) -> IVector4: ...
    @property
    def tqpq(self) -> IVector4: ...
    @property
    def tqpp(self) -> IVector4: ...
    @property
    def tqpo(self) -> IVector4: ...
    @property
    def tqpl(self) -> IVector4: ...
    @property
    def tqos(self) -> IVector4: ...
    @property
    def tqot(self) -> IVector4: ...
    @property
    def tqoq(self) -> IVector4: ...
    @property
    def tqop(self) -> IVector4: ...
    @property
    def tqoo(self) -> IVector4: ...
    @property
    def tqol(self) -> IVector4: ...
    @property
    def tqls(self) -> IVector4: ...
    @property
    def tqlt(self) -> IVector4: ...
    @property
    def tqlq(self) -> IVector4: ...
    @property
    def tqlp(self) -> IVector4: ...
    @property
    def tqlo(self) -> IVector4: ...
    @property
    def tqll(self) -> IVector4: ...
    @property
    def tpss(self) -> IVector4: ...
    @property
    def tpst(self) -> IVector4: ...
    @property
    def tpsq(self) -> IVector4: ...
    @property
    def tpsp(self) -> IVector4: ...
    @property
    def tpso(self) -> IVector4: ...
    @property
    def tpsl(self) -> IVector4: ...
    @property
    def tpts(self) -> IVector4: ...
    @property
    def tptt(self) -> IVector4: ...
    @property
    def tptq(self) -> IVector4: ...
    @property
    def tptp(self) -> IVector4: ...
    @property
    def tpto(self) -> IVector4: ...
    @property
    def tptl(self) -> IVector4: ...
    @property
    def tpqs(self) -> IVector4: ...
    @property
    def tpqt(self) -> IVector4: ...
    @property
    def tpqq(self) -> IVector4: ...
    @property
    def tpqp(self) -> IVector4: ...
    @property
    def tpqo(self) -> IVector4: ...
    @property
    def tpql(self) -> IVector4: ...
    @property
    def tpps(self) -> IVector4: ...
    @property
    def tppt(self) -> IVector4: ...
    @property
    def tppq(self) -> IVector4: ...
    @property
    def tppp(self) -> IVector4: ...
    @property
    def tppo(self) -> IVector4: ...
    @property
    def tppl(self) -> IVector4: ...
    @property
    def tpos(self) -> IVector4: ...
    @property
    def tpot(self) -> IVector4: ...
    @property
    def tpoq(self) -> IVector4: ...
    @property
    def tpop(self) -> IVector4: ...
    @property
    def tpoo(self) -> IVector4: ...
    @property
    def tpol(self) -> IVector4: ...
    @property
    def tpls(self) -> IVector4: ...
    @property
    def tplt(self) -> IVector4: ...
    @property
    def tplq(self) -> IVector4: ...
    @property
    def tplp(self) -> IVector4: ...
    @property
    def tplo(self) -> IVector4: ...
    @property
    def tpll(self) -> IVector4: ...
    @property
    def toss(self) -> IVector4: ...
    @property
    def tost(self) -> IVector4: ...
    @property
    def tosq(self) -> IVector4: ...
    @property
    def tosp(self) -> IVector4: ...
    @property
    def toso(self) -> IVector4: ...
    @property
    def tosl(self) -> IVector4: ...
    @property
    def tots(self) -> IVector4: ...
    @property
    def tott(self) -> IVector4: ...
    @property
    def totq(self) -> IVector4: ...
    @property
    def totp(self) -> IVector4: ...
    @property
    def toto(self) -> IVector4: ...
    @property
    def totl(self) -> IVector4: ...
    @property
    def toqs(self) -> IVector4: ...
    @property
    def toqt(self) -> IVector4: ...
    @property
    def toqq(self) -> IVector4: ...
    @property
    def toqp(self) -> IVector4: ...
    @property
    def toqo(self) -> IVector4: ...
    @property
    def toql(self) -> IVector4: ...
    @property
    def tops(self) -> IVector4: ...
    @property
    def topt(self) -> IVector4: ...
    @property
    def topq(self) -> IVector4: ...
    @property
    def topp(self) -> IVector4: ...
    @property
    def topo(self) -> IVector4: ...
    @property
    def topl(self) -> IVector4: ...
    @property
    def toos(self) -> IVector4: ...
    @property
    def toot(self) -> IVector4: ...
    @property
    def tooq(self) -> IVector4: ...
    @property
    def toop(self) -> IVector4: ...
    @property
    def tooo(self) -> IVector4: ...
    @property
    def tool(self) -> IVector4: ...
    @property
    def tols(self) -> IVector4: ...
    @property
    def tolt(self) -> IVector4: ...
    @property
    def tolq(self) -> IVector4: ...
    @property
    def tolp(self) -> IVector4: ...
    @property
    def tolo(self) -> IVector4: ...
    @property
    def toll(self) -> IVector4: ...
    @property
    def tlss(self) -> IVector4: ...
    @property
    def tlst(self) -> IVector4: ...
    @property
    def tlsq(self) -> IVector4: ...
    @property
    def tlsp(self) -> IVector4: ...
    @property
    def tlso(self) -> IVector4: ...
    @property
    def tlsl(self) -> IVector4: ...
    @property
    def tlts(self) -> IVector4: ...
    @property
    def tltt(self) -> IVector4: ...
    @property
    def tltq(self) -> IVector4: ...
    @property
    def tltp(self) -> IVector4: ...
    @property
    def tlto(self) -> IVector4: ...
    @property
    def tltl(self) -> IVector4: ...
    @property
    def tlqs(self) -> IVector4: ...
    @property
    def tlqt(self) -> IVector4: ...
    @property
    def tlqq(self) -> IVector4: ...
    @property
    def tlqp(self) -> IVector4: ...
    @property
    def tlqo(self) -> IVector4: ...
    @property
    def tlql(self) -> IVector4: ...
    @property
    def tlps(self) -> IVector4: ...
    @property
    def tlpt(self) -> IVector4: ...
    @property
    def tlpq(self) -> IVector4: ...
    @property
    def tlpp(self) -> IVector4: ...
    @property
    def tlpo(self) -> IVector4: ...
    @property
    def tlpl(self) -> IVector4: ...
    @property
    def tlos(self) -> IVector4: ...
    @property
    def tlot(self) -> IVector4: ...
    @property
    def tloq(self) -> IVector4: ...
    @property
    def tlop(self) -> IVector4: ...
    @property
    def tloo(self) -> IVector4: ...
    @property
    def tlol(self) -> IVector4: ...
    @property
    def tlls(self) -> IVector4: ...
    @property
    def tllt(self) -> IVector4: ...
    @property
    def tllq(self) -> IVector4: ...
    @property
    def tllp(self) -> IVector4: ...
    @property
    def tllo(self) -> IVector4: ...
    @property
    def tlll(self) -> IVector4: ...
    @property
    def qsss(self) -> IVector4: ...
    @property
    def qsst(self) -> IVector4: ...
    @property
    def qssq(self) -> IVector4: ...
    @property
    def qssp(self) -> IVector4: ...
    @property
    def qsso(self) -> IVector4: ...
    @property
    def qssl(self) -> IVector4: ...
    @property
    def qsts(self) -> IVector4: ...
    @property
    def qstt(self) -> IVector4: ...
    @property
    def qstq(self) -> IVector4: ...
    @property
    def qstp(self) -> IVector4: ...
    @property
    def qsto(self) -> IVector4: ...
    @property
    def qstl(self) -> IVector4: ...
    @property
    def qsqs(self) -> IVector4: ...
    @property
    def qsqt(self) -> IVector4: ...
    @property
    def qsqq(self) -> IVector4: ...
    @property
    def qsqp(self) -> IVector4: ...
    @property
    def qsqo(self) -> IVector4: ...
    @property
    def qsql(self) -> IVector4: ...
    @property
    def qsps(self) -> IVector4: ...
    @property
    def qspt(self) -> IVector4: ...
    @property
    def qspq(self) -> IVector4: ...
    @property
    def qspp(self) -> IVector4: ...
    @property
    def qspo(self) -> IVector4: ...
    @property
    def qspl(self) -> IVector4: ...
    @property
    def qsos(self) -> IVector4: ...
    @property
    def qsot(self) -> IVector4: ...
    @property
    def qsoq(self) -> IVector4: ...
    @property
    def qsop(self) -> IVector4: ...
    @property
    def qsoo(self) -> IVector4: ...
    @property
    def qsol(self) -> IVector4: ...
    @property
    def qsls(self) -> IVector4: ...
    @property
    def qslt(self) -> IVector4: ...
    @property
    def qslq(self) -> IVector4: ...
    @property
    def qslp(self) -> IVector4: ...
    @property
    def qslo(self) -> IVector4: ...
    @property
    def qsll(self) -> IVector4: ...
    @property
    def qtss(self) -> IVector4: ...
    @property
    def qtst(self) -> IVector4: ...
    @property
    def qtsq(self) -> IVector4: ...
    @property
    def qtsp(self) -> IVector4: ...
    @property
    def qtso(self) -> IVector4: ...
    @property
    def qtsl(self) -> IVector4: ...
    @property
    def qtts(self) -> IVector4: ...
    @property
    def qttt(self) -> IVector4: ...
    @property
    def qttq(self) -> IVector4: ...
    @property
    def qttp(self) -> IVector4: ...
    @property
    def qtto(self) -> IVector4: ...
    @property
    def qttl(self) -> IVector4: ...
    @property
    def qtqs(self) -> IVector4: ...
    @property
    def qtqt(self) -> IVector4: ...
    @property
    def qtqq(self) -> IVector4: ...
    @property
    def qtqp(self) -> IVector4: ...
    @property
    def qtqo(self) -> IVector4: ...
    @property
    def qtql(self) -> IVector4: ...
    @property
    def qtps(self) -> IVector4: ...
    @property
    def qtpt(self) -> IVector4: ...
    @property
    def qtpq(self) -> IVector4: ...
    @property
    def qtpp(self) -> IVector4: ...
    @property
    def qtpo(self) -> IVector4: ...
    @property
    def qtpl(self) -> IVector4: ...
    @property
    def qtos(self) -> IVector4: ...
    @property
    def qtot(self) -> IVector4: ...
    @property
    def qtoq(self) -> IVector4: ...
    @property
    def qtop(self) -> IVector4: ...
    @property
    def qtoo(self) -> IVector4: ...
    @property
    def qtol(self) -> IVector4: ...
    @property
    def qtls(self) -> IVector4: ...
    @property
    def qtlt(self) -> IVector4: ...
    @property
    def qtlq(self) -> IVector4: ...
    @property
    def qtlp(self) -> IVector4: ...
    @property
    def qtlo(self) -> IVector4: ...
    @property
    def qtll(self) -> IVector4: ...
    @property
    def qqss(self) -> IVector4: ...
    @property
    def qqst(self) -> IVector4: ...
    @property
    def qqsq(self) -> IVector4: ...
    @property
    def qqsp(self) -> IVector4: ...
    @property
    def qqso(self) -> IVector4: ...
    @property
    def qqsl(self) -> IVector4: ...
    @property
    def qqts(self) -> IVector4: ...
    @property
    def qqtt(self) -> IVector4: ...
    @property
    def qqtq(self) -> IVector4: ...
    @property
    def qqtp(self) -> IVector4: ...
    @property
    def qqto(self) -> IVector4: ...
    @property
    def qqtl(self) -> IVector4: ...
    @property
    def qqqs(self) -> IVector4: ...
    @property
    def qqqt(self) -> IVector4: ...
    @property
    def qqqq(self) -> IVector4: ...
    @property
    def qqqp(self) -> IVector4: ...
    @property
    def qqqo(self) -> IVector4: ...
    @property
    def qqql(self) -> IVector4: ...
    @property
    def qqps(self) -> IVector4: ...
    @property
    def qqpt(self) -> IVector4: ...
    @property
    def qqpq(self) -> IVector4: ...
    @property
    def qqpp(self) -> IVector4: ...
    @property
    def qqpo(self) -> IVector4: ...
    @property
    def qqpl(self) -> IVector4: ...
    @property
    def qqos(self) -> IVector4: ...
    @property
    def qqot(self) -> IVector4: ...
    @property
    def qqoq(self) -> IVector4: ...
    @property
    def qqop(self) -> IVector4: ...
    @property
    def qqoo(self) -> IVector4: ...
    @property
    def qqol(self) -> IVector4: ...
    @property
    def qqls(self) -> IVector4: ...
    @property
    def qqlt(self) -> IVector4: ...
    @property
    def qqlq(self) -> IVector4: ...
    @property
    def qqlp(self) -> IVector4: ...
    @property
    def qqlo(self) -> IVector4: ...
    @property
    def qqll(self) -> IVector4: ...
    @property
    def qpss(self) -> IVector4: ...
    @property
    def qpst(self) -> IVector4: ...
    @property
    def qpsq(self) -> IVector4: ...
    @property
    def qpsp(self) -> IVector4: ...
    @property
    def qpso(self) -> IVector4: ...
    @property
    def qpsl(self) -> IVector4: ...
    @property
    def qpts(self) -> IVector4: ...
    @property
    def qptt(self) -> IVector4: ...
    @property
    def qptq(self) -> IVector4: ...
    @property
    def qptp(self) -> IVector4: ...
    @property
    def qpto(self) -> IVector4: ...
    @property
    def qptl(self) -> IVector4: ...
    @property
    def qpqs(self) -> IVector4: ...
    @property
    def qpqt(self) -> IVector4: ...
    @property
    def qpqq(self) -> IVector4: ...
    @property
    def qpqp(self) -> IVector4: ...
    @property
    def qpqo(self) -> IVector4: ...
    @property
    def qpql(self) -> IVector4: ...
    @property
    def qpps(self) -> IVector4: ...
    @property
    def qppt(self) -> IVector4: ...
    @property
    def qppq(self) -> IVector4: ...
    @property
    def qppp(self) -> IVector4: ...
    @property
    def qppo(self) -> IVector4: ...
    @property
    def qppl(self) -> IVector4: ...
    @property
    def qpos(self) -> IVector4: ...
    @property
    def qpot(self) -> IVector4: ...
    @property
    def qpoq(self) -> IVector4: ...
    @property
    def qpop(self) -> IVector4: ...
    @property
    def qpoo(self) -> IVector4: ...
    @property
    def qpol(self) -> IVector4: ...
    @property
    def qpls(self) -> IVector4: ...
    @property
    def qplt(self) -> IVector4: ...
    @property
    def qplq(self) -> IVector4: ...
    @property
    def qplp(self) -> IVector4: ...
    @property
    def qplo(self) -> IVector4: ...
    @property
    def qpll(self) -> IVector4: ...
    @property
    def qoss(self) -> IVector4: ...
    @property
    def qost(self) -> IVector4: ...
    @property
    def qosq(self) -> IVector4: ...
    @property
    def qosp(self) -> IVector4: ...
    @property
    def qoso(self) -> IVector4: ...
    @property
    def qosl(self) -> IVector4: ...
    @property
    def qots(self) -> IVector4: ...
    @property
    def qott(self) -> IVector4: ...
    @property
    def qotq(self) -> IVector4: ...
    @property
    def qotp(self) -> IVector4: ...
    @property
    def qoto(self) -> IVector4: ...
    @property
    def qotl(self) -> IVector4: ...
    @property
    def qoqs(self) -> IVector4: ...
    @property
    def qoqt(self) -> IVector4: ...
    @property
    def qoqq(self) -> IVector4: ...
    @property
    def qoqp(self) -> IVector4: ...
    @property
    def qoqo(self) -> IVector4: ...
    @property
    def qoql(self) -> IVector4: ...
    @property
    def qops(self) -> IVector4: ...
    @property
    def qopt(self) -> IVector4: ...
    @property
    def qopq(self) -> IVector4: ...
    @property
    def qopp(self) -> IVector4: ...
    @property
    def qopo(self) -> IVector4: ...
    @property
    def qopl(self) -> IVector4: ...
    @property
    def qoos(self) -> IVector4: ...
    @property
    def qoot(self) -> IVector4: ...
    @property
    def qooq(self) -> IVector4: ...
    @property
    def qoop(self) -> IVector4: ...
    @property
    def qooo(self) -> IVector4: ...
    @property
    def qool(self) -> IVector4: ...
    @property
    def qols(self) -> IVector4: ...
    @property
    def qolt(self) -> IVector4: ...
    @property
    def qolq(self) -> IVector4: ...
    @property
    def qolp(self) -> IVector4: ...
    @property
    def qolo(self) -> IVector4: ...
    @property
    def qoll(self) -> IVector4: ...
    @property
    def qlss(self) -> IVector4: ...
    @property
    def qlst(self) -> IVector4: ...
    @property
    def qlsq(self) -> IVector4: ...
    @property
    def qlsp(self) -> IVector4: ...
    @property
    def qlso(self) -> IVector4: ...
    @property
    def qlsl(self) -> IVector4: ...
    @property
    def qlts(self) -> IVector4: ...
    @property
    def qltt(self) -> IVector4: ...
    @property
    def qltq(self) -> IVector4: ...
    @property
    def qltp(self) -> IVector4: ...
    @property
    def qlto(self) -> IVector4: ...
    @property
    def qltl(self) -> IVector4: ...
    @property
    def qlqs(self) -> IVector4: ...
    @property
    def qlqt(self) -> IVector4: ...
    @property
    def qlqq(self) -> IVector4: ...
    @property
    def qlqp(self) -> IVector4: ...
    @property
    def qlqo(self) -> IVector4: ...
    @property
    def qlql(self) -> IVector4: ...
    @property
    def qlps(self) -> IVector4: ...
    @property
    def qlpt(self) -> IVector4: ...
    @property
    def qlpq(self) -> IVector4: ...
    @property
    def qlpp(self) -> IVector4: ...
    @property
    def qlpo(self) -> IVector4: ...
    @property
    def qlpl(self) -> IVector4: ...
    @property
    def qlos(self) -> IVector4: ...
    @property
    def qlot(self) -> IVector4: ...
    @property
    def qloq(self) -> IVector4: ...
    @property
    def qlop(self) -> IVector4: ...
    @property
    def qloo(self) -> IVector4: ...
    @property
    def qlol(self) -> IVector4: ...
    @property
    def qlls(self) -> IVector4: ...
    @property
    def qllt(self) -> IVector4: ...
    @property
    def qllq(self) -> IVector4: ...
    @property
    def qllp(self) -> IVector4: ...
    @property
    def qllo(self) -> IVector4: ...
    @property
    def qlll(self) -> IVector4: ...
    @property
    def psss(self) -> IVector4: ...
    @property
    def psst(self) -> IVector4: ...
    @property
    def pssq(self) -> IVector4: ...
    @property
    def pssp(self) -> IVector4: ...
    @property
    def psso(self) -> IVector4: ...
    @property
    def pssl(self) -> IVector4: ...
    @property
    def psts(self) -> IVector4: ...
    @property
    def pstt(self) -> IVector4: ...
    @property
    def pstq(self) -> IVector4: ...
    @property
    def pstp(self) -> IVector4: ...
    @property
    def psto(self) -> IVector4: ...
    @property
    def pstl(self) -> IVector4: ...
    @property
    def psqs(self) -> IVector4: ...
    @property
    def psqt(self) -> IVector4: ...
    @property
    def psqq(self) -> IVector4: ...
    @property
    def psqp(self) -> IVector4: ...
    @property
    def psqo(self) -> IVector4: ...
    @property
    def psql(self) -> IVector4: ...
    @property
    def psps(self) -> IVector4: ...
    @property
    def pspt(self) -> IVector4: ...
    @property
    def pspq(self) -> IVector4: ...
    @property
    def pspp(self) -> IVector4: ...
    @property
    def pspo(self) -> IVector4: ...
    @property
    def pspl(self) -> IVector4: ...
    @property
    def psos(self) -> IVector4: ...
    @property
    def psot(self) -> IVector4: ...
    @property
    def psoq(self) -> IVector4: ...
    @property
    def psop(self) -> IVector4: ...
    @property
    def psoo(self) -> IVector4: ...
    @property
    def psol(self) -> IVector4: ...
    @property
    def psls(self) -> IVector4: ...
    @property
    def pslt(self) -> IVector4: ...
    @property
    def pslq(self) -> IVector4: ...
    @property
    def pslp(self) -> IVector4: ...
    @property
    def pslo(self) -> IVector4: ...
    @property
    def psll(self) -> IVector4: ...
    @property
    def ptss(self) -> IVector4: ...
    @property
    def ptst(self) -> IVector4: ...
    @property
    def ptsq(self) -> IVector4: ...
    @property
    def ptsp(self) -> IVector4: ...
    @property
    def ptso(self) -> IVector4: ...
    @property
    def ptsl(self) -> IVector4: ...
    @property
    def ptts(self) -> IVector4: ...
    @property
    def pttt(self) -> IVector4: ...
    @property
    def pttq(self) -> IVector4: ...
    @property
    def pttp(self) -> IVector4: ...
    @property
    def ptto(self) -> IVector4: ...
    @property
    def pttl(self) -> IVector4: ...
    @property
    def ptqs(self) -> IVector4: ...
    @property
    def ptqt(self) -> IVector4: ...
    @property
    def ptqq(self) -> IVector4: ...
    @property
    def ptqp(self) -> IVector4: ...
    @property
    def ptqo(self) -> IVector4: ...
    @property
    def ptql(self) -> IVector4: ...
    @property
    def ptps(self) -> IVector4: ...
    @property
    def ptpt(self) -> IVector4: ...
    @property
    def ptpq(self) -> IVector4: ...
    @property
    def ptpp(self) -> IVector4: ...
    @property
    def ptpo(self) -> IVector4: ...
    @property
    def ptpl(self) -> IVector4: ...
    @property
    def ptos(self) -> IVector4: ...
    @property
    def ptot(self) -> IVector4: ...
    @property
    def ptoq(self) -> IVector4: ...
    @property
    def ptop(self) -> IVector4: ...
    @property
    def ptoo(self) -> IVector4: ...
    @property
    def ptol(self) -> IVector4: ...
    @property
    def ptls(self) -> IVector4: ...
    @property
    def ptlt(self) -> IVector4: ...
    @property
    def ptlq(self) -> IVector4: ...
    @property
    def ptlp(self) -> IVector4: ...
    @property
    def ptlo(self) -> IVector4: ...
    @property
    def ptll(self) -> IVector4: ...
    @property
    def pqss(self) -> IVector4: ...
    @property
    def pqst(self) -> IVector4: ...
    @property
    def pqsq(self) -> IVector4: ...
    @property
    def pqsp(self) -> IVector4: ...
    @property
    def pqso(self) -> IVector4: ...
    @property
    def pqsl(self) -> IVector4: ...
    @property
    def pqts(self) -> IVector4: ...
    @property
    def pqtt(self) -> IVector4: ...
    @property
    def pqtq(self) -> IVector4: ...
    @property
    def pqtp(self) -> IVector4: ...
    @property
    def pqto(self) -> IVector4: ...
    @property
    def pqtl(self) -> IVector4: ...
    @property
    def pqqs(self) -> IVector4: ...
    @property
    def pqqt(self) -> IVector4: ...
    @property
    def pqqq(self) -> IVector4: ...
    @property
    def pqqp(self) -> IVector4: ...
    @property
    def pqqo(self) -> IVector4: ...
    @property
    def pqql(self) -> IVector4: ...
    @property
    def pqps(self) -> IVector4: ...
    @property
    def pqpt(self) -> IVector4: ...
    @property
    def pqpq(self) -> IVector4: ...
    @property
    def pqpp(self) -> IVector4: ...
    @property
    def pqpo(self) -> IVector4: ...
    @property
    def pqpl(self) -> IVector4: ...
    @property
    def pqos(self) -> IVector4: ...
    @property
    def pqot(self) -> IVector4: ...
    @property
    def pqoq(self) -> IVector4: ...
    @property
    def pqop(self) -> IVector4: ...
    @property
    def pqoo(self) -> IVector4: ...
    @property
    def pqol(self) -> IVector4: ...
    @property
    def pqls(self) -> IVector4: ...
    @property
    def pqlt(self) -> IVector4: ...
    @property
    def pqlq(self) -> IVector4: ...
    @property
    def pqlp(self) -> IVector4: ...
    @property
    def pqlo(self) -> IVector4: ...
    @property
    def pqll(self) -> IVector4: ...
    @property
    def ppss(self) -> IVector4: ...
    @property
    def ppst(self) -> IVector4: ...
    @property
    def ppsq(self) -> IVector4: ...
    @property
    def ppsp(self) -> IVector4: ...
    @property
    def ppso(self) -> IVector4: ...
    @property
    def ppsl(self) -> IVector4: ...
    @property
    def ppts(self) -> IVector4: ...
    @property
    def pptt(self) -> IVector4: ...
    @property
    def pptq(self) -> IVector4: ...
    @property
    def pptp(self) -> IVector4: ...
    @property
    def ppto(self) -> IVector4: ...
    @property
    def pptl(self) -> IVector4: ...
    @property
    def ppqs(self) -> IVector4: ...
    @property
    def ppqt(self) -> IVector4: ...
    @property
    def ppqq(self) -> IVector4: ...
    @property
    def ppqp(self) -> IVector4: ...
    @property
    def ppqo(self) -> IVector4: ...
    @property
    def ppql(self) -> IVector4: ...
    @property
    def ppps(self) -> IVector4: ...
    @property
    def pppt(self) -> IVector4: ...
    @property
    def pppq(self) -> IVector4: ...
    @property
    def pppp(self) -> IVector4: ...
    @property
    def pppo(self) -> IVector4: ...
    @property
    def pppl(self) -> IVector4: ...
    @property
    def ppos(self) -> IVector4: ...
    @property
    def ppot(self) -> IVector4: ...
    @property
    def ppoq(self) -> IVector4: ...
    @property
    def ppop(self) -> IVector4: ...
    @property
    def ppoo(self) -> IVector4: ...
    @property
    def ppol(self) -> IVector4: ...
    @property
    def ppls(self) -> IVector4: ...
    @property
    def pplt(self) -> IVector4: ...
    @property
    def pplq(self) -> IVector4: ...
    @property
    def pplp(self) -> IVector4: ...
    @property
    def pplo(self) -> IVector4: ...
    @property
    def ppll(self) -> IVector4: ...
    @property
    def poss(self) -> IVector4: ...
    @property
    def post(self) -> IVector4: ...
    @property
    def posq(self) -> IVector4: ...
    @property
    def posp(self) -> IVector4: ...
    @property
    def poso(self) -> IVector4: ...
    @property
    def posl(self) -> IVector4: ...
    @property
    def pots(self) -> IVector4: ...
    @property
    def pott(self) -> IVector4: ...
    @property
    def potq(self) -> IVector4: ...
    @property
    def potp(self) -> IVector4: ...
    @property
    def poto(self) -> IVector4: ...
    @property
    def potl(self) -> IVector4: ...
    @property
    def poqs(self) -> IVector4: ...
    @property
    def poqt(self) -> IVector4: ...
    @property
    def poqq(self) -> IVector4: ...
    @property
    def poqp(self) -> IVector4: ...
    @property
    def poqo(self) -> IVector4: ...
    @property
    def poql(self) -> IVector4: ...
    @property
    def pops(self) -> IVector4: ...
    @property
    def popt(self) -> IVector4: ...
    @property
    def popq(self) -> IVector4: ...
    @property
    def popp(self) -> IVector4: ...
    @property
    def popo(self) -> IVector4: ...
    @property
    def popl(self) -> IVector4: ...
    @property
    def poos(self) -> IVector4: ...
    @property
    def poot(self) -> IVector4: ...
    @property
    def pooq(self) -> IVector4: ...
    @property
    def poop(self) -> IVector4: ...
    @property
    def pooo(self) -> IVector4: ...
    @property
    def pool(self) -> IVector4: ...
    @property
    def pols(self) -> IVector4: ...
    @property
    def polt(self) -> IVector4: ...
    @property
    def polq(self) -> IVector4: ...
    @property
    def polp(self) -> IVector4: ...
    @property
    def polo(self) -> IVector4: ...
    @property
    def poll(self) -> IVector4: ...
    @property
    def plss(self) -> IVector4: ...
    @property
    def plst(self) -> IVector4: ...
    @property
    def plsq(self) -> IVector4: ...
    @property
    def plsp(self) -> IVector4: ...
    @property
    def plso(self) -> IVector4: ...
    @property
    def plsl(self) -> IVector4: ...
    @property
    def plts(self) -> IVector4: ...
    @property
    def pltt(self) -> IVector4: ...
    @property
    def pltq(self) -> IVector4: ...
    @property
    def pltp(self) -> IVector4: ...
    @property
    def plto(self) -> IVector4: ...
    @property
    def pltl(self) -> IVector4: ...
    @property
    def plqs(self) -> IVector4: ...
    @property
    def plqt(self) -> IVector4: ...
    @property
    def plqq(self) -> IVector4: ...
    @property
    def plqp(self) -> IVector4: ...
    @property
    def plqo(self) -> IVector4: ...
    @property
    def plql(self) -> IVector4: ...
    @property
    def plps(self) -> IVector4: ...
    @property
    def plpt(self) -> IVector4: ...
    @property
    def plpq(self) -> IVector4: ...
    @property
    def plpp(self) -> IVector4: ...
    @property
    def plpo(self) -> IVector4: ...
    @property
    def plpl(self) -> IVector4: ...
    @property
    def plos(self) -> IVector4: ...
    @property
    def plot(self) -> IVector4: ...
    @property
    def ploq(self) -> IVector4: ...
    @property
    def plop(self) -> IVector4: ...
    @property
    def ploo(self) -> IVector4: ...
    @property
    def plol(self) -> IVector4: ...
    @property
    def plls(self) -> IVector4: ...
    @property
    def pllt(self) -> IVector4: ...
    @property
    def pllq(self) -> IVector4: ...
    @property
    def pllp(self) -> IVector4: ...
    @property
    def pllo(self) -> IVector4: ...
    @property
    def plll(self) -> IVector4: ...
    @property
    def osss(self) -> IVector4: ...
    @property
    def osst(self) -> IVector4: ...
    @property
    def ossq(self) -> IVector4: ...
    @property
    def ossp(self) -> IVector4: ...
    @property
    def osso(self) -> IVector4: ...
    @property
    def ossl(self) -> IVector4: ...
    @property
    def osts(self) -> IVector4: ...
    @property
    def ostt(self) -> IVector4: ...
    @property
    def ostq(self) -> IVector4: ...
    @property
    def ostp(self) -> IVector4: ...
    @property
    def osto(self) -> IVector4: ...
    @property
    def ostl(self) -> IVector4: ...
    @property
    def osqs(self) -> IVector4: ...
    @property
    def osqt(self) -> IVector4: ...
    @property
    def osqq(self) -> IVector4: ...
    @property
    def osqp(self) -> IVector4: ...
    @property
    def osqo(self) -> IVector4: ...
    @property
    def osql(self) -> IVector4: ...
    @property
    def osps(self) -> IVector4: ...
    @property
    def ospt(self) -> IVector4: ...
    @property
    def ospq(self) -> IVector4: ...
    @property
    def ospp(self) -> IVector4: ...
    @property
    def ospo(self) -> IVector4: ...
    @property
    def ospl(self) -> IVector4: ...
    @property
    def osos(self) -> IVector4: ...
    @property
    def osot(self) -> IVector4: ...
    @property
    def osoq(self) -> IVector4: ...
    @property
    def osop(self) -> IVector4: ...
    @property
    def osoo(self) -> IVector4: ...
    @property
    def osol(self) -> IVector4: ...
    @property
    def osls(self) -> IVector4: ...
    @property
    def oslt(self) -> IVector4: ...
    @property
    def oslq(self) -> IVector4: ...
    @property
    def oslp(self) -> IVector4: ...
    @property
    def oslo(self) -> IVector4: ...
    @property
    def osll(self) -> IVector4: ...
    @property
    def otss(self) -> IVector4: ...
    @property
    def otst(self) -> IVector4: ...
    @property
    def otsq(self) -> IVector4: ...
    @property
    def otsp(self) -> IVector4: ...
    @property
    def otso(self) -> IVector4: ...
    @property
    def otsl(self) -> IVector4: ...
    @property
    def otts(self) -> IVector4: ...
    @property
    def ottt(self) -> IVector4: ...
    @property
    def ottq(self) -> IVector4: ...
    @property
    def ottp(self) -> IVector4: ...
    @property
    def otto(self) -> IVector4: ...
    @property
    def ottl(self) -> IVector4: ...
    @property
    def otqs(self) -> IVector4: ...
    @property
    def otqt(self) -> IVector4: ...
    @property
    def otqq(self) -> IVector4: ...
    @property
    def otqp(self) -> IVector4: ...
    @property
    def otqo(self) -> IVector4: ...
    @property
    def otql(self) -> IVector4: ...
    @property
    def otps(self) -> IVector4: ...
    @property
    def otpt(self) -> IVector4: ...
    @property
    def otpq(self) -> IVector4: ...
    @property
    def otpp(self) -> IVector4: ...
    @property
    def otpo(self) -> IVector4: ...
    @property
    def otpl(self) -> IVector4: ...
    @property
    def otos(self) -> IVector4: ...
    @property
    def otot(self) -> IVector4: ...
    @property
    def otoq(self) -> IVector4: ...
    @property
    def otop(self) -> IVector4: ...
    @property
    def otoo(self) -> IVector4: ...
    @property
    def otol(self) -> IVector4: ...
    @property
    def otls(self) -> IVector4: ...
    @property
    def otlt(self) -> IVector4: ...
    @property
    def otlq(self) -> IVector4: ...
    @property
    def otlp(self) -> IVector4: ...
    @property
    def otlo(self) -> IVector4: ...
    @property
    def otll(self) -> IVector4: ...
    @property
    def oqss(self) -> IVector4: ...
    @property
    def oqst(self) -> IVector4: ...
    @property
    def oqsq(self) -> IVector4: ...
    @property
    def oqsp(self) -> IVector4: ...
    @property
    def oqso(self) -> IVector4: ...
    @property
    def oqsl(self) -> IVector4: ...
    @property
    def oqts(self) -> IVector4: ...
    @property
    def oqtt(self) -> IVector4: ...
    @property
    def oqtq(self) -> IVector4: ...
    @property
    def oqtp(self) -> IVector4: ...
    @property
    def oqto(self) -> IVector4: ...
    @property
    def oqtl(self) -> IVector4: ...
    @property
    def oqqs(self) -> IVector4: ...
    @property
    def oqqt(self) -> IVector4: ...
    @property
    def oqqq(self) -> IVector4: ...
    @property
    def oqqp(self) -> IVector4: ...
    @property
    def oqqo(self) -> IVector4: ...
    @property
    def oqql(self) -> IVector4: ...
    @property
    def oqps(self) -> IVector4: ...
    @property
    def oqpt(self) -> IVector4: ...
    @property
    def oqpq(self) -> IVector4: ...
    @property
    def oqpp(self) -> IVector4: ...
    @property
    def oqpo(self) -> IVector4: ...
    @property
    def oqpl(self) -> IVector4: ...
    @property
    def oqos(self) -> IVector4: ...
    @property
    def oqot(self) -> IVector4: ...
    @property
    def oqoq(self) -> IVector4: ...
    @property
    def oqop(self) -> IVector4: ...
    @property
    def oqoo(self) -> IVector4: ...
    @property
    def oqol(self) -> IVector4: ...
    @property
    def oqls(self) -> IVector4: ...
    @property
    def oqlt(self) -> IVector4: ...
    @property
    def oqlq(self) -> IVector4: ...
    @property
    def oqlp(self) -> IVector4: ...
    @property
    def oqlo(self) -> IVector4: ...
    @property
    def oqll(self) -> IVector4: ...
    @property
    def opss(self) -> IVector4: ...
    @property
    def opst(self) -> IVector4: ...
    @property
    def opsq(self) -> IVector4: ...
    @property
    def opsp(self) -> IVector4: ...
    @property
    def opso(self) -> IVector4: ...
    @property
    def opsl(self) -> IVector4: ...
    @property
    def opts(self) -> IVector4: ...
    @property
    def optt(self) -> IVector4: ...
    @property
    def optq(self) -> IVector4: ...
    @property
    def optp(self) -> IVector4: ...
    @property
    def opto(self) -> IVector4: ...
    @property
    def optl(self) -> IVector4: ...
    @property
    def opqs(self) -> IVector4: ...
    @property
    def opqt(self) -> IVector4: ...
    @property
    def opqq(self) -> IVector4: ...
    @property
    def opqp(self) -> IVector4: ...
    @property
    def opqo(self) -> IVector4: ...
    @property
    def opql(self) -> IVector4: ...
    @property
    def opps(self) -> IVector4: ...
    @property
    def oppt(self) -> IVector4: ...
    @property
    def oppq(self) -> IVector4: ...
    @property
    def oppp(self) -> IVector4: ...
    @property
    def oppo(self) -> IVector4: ...
    @property
    def oppl(self) -> IVector4: ...
    @property
    def opos(self) -> IVector4: ...
    @property
    def opot(self) -> IVector4: ...
    @property
    def opoq(self) -> IVector4: ...
    @property
    def opop(self) -> IVector4: ...
    @property
    def opoo(self) -> IVector4: ...
    @property
    def opol(self) -> IVector4: ...
    @property
    def opls(self) -> IVector4: ...
    @property
    def oplt(self) -> IVector4: ...
    @property
    def oplq(self) -> IVector4: ...
    @property
    def oplp(self) -> IVector4: ...
    @property
    def oplo(self) -> IVector4: ...
    @property
    def opll(self) -> IVector4: ...
    @property
    def ooss(self) -> IVector4: ...
    @property
    def oost(self) -> IVector4: ...
    @property
    def oosq(self) -> IVector4: ...
    @property
    def oosp(self) -> IVector4: ...
    @property
    def ooso(self) -> IVector4: ...
    @property
    def oosl(self) -> IVector4: ...
    @property
    def oots(self) -> IVector4: ...
    @property
    def oott(self) -> IVector4: ...
    @property
    def ootq(self) -> IVector4: ...
    @property
    def ootp(self) -> IVector4: ...
    @property
    def ooto(self) -> IVector4: ...
    @property
    def ootl(self) -> IVector4: ...
    @property
    def ooqs(self) -> IVector4: ...
    @property
    def ooqt(self) -> IVector4: ...
    @property
    def ooqq(self) -> IVector4: ...
    @property
    def ooqp(self) -> IVector4: ...
    @property
    def ooqo(self) -> IVector4: ...
    @property
    def ooql(self) -> IVector4: ...
    @property
    def oops(self) -> IVector4: ...
    @property
    def oopt(self) -> IVector4: ...
    @property
    def oopq(self) -> IVector4: ...
    @property
    def oopp(self) -> IVector4: ...
    @property
    def oopo(self) -> IVector4: ...
    @property
    def oopl(self) -> IVector4: ...
    @property
    def ooos(self) -> IVector4: ...
    @property
    def ooot(self) -> IVector4: ...
    @property
    def oooq(self) -> IVector4: ...
    @property
    def ooop(self) -> IVector4: ...
    @property
    def ools(self) -> IVector4: ...
    @property
    def oolt(self) -> IVector4: ...
    @property
    def oolq(self) -> IVector4: ...
    @property
    def oolp(self) -> IVector4: ...
    @property
    def olss(self) -> IVector4: ...
    @property
    def olst(self) -> IVector4: ...
    @property
    def olsq(self) -> IVector4: ...
    @property
    def olsp(self) -> IVector4: ...
    @property
    def olso(self) -> IVector4: ...
    @property
    def olsl(self) -> IVector4: ...
    @property
    def olts(self) -> IVector4: ...
    @property
    def oltt(self) -> IVector4: ...
    @property
    def oltq(self) -> IVector4: ...
    @property
    def oltp(self) -> IVector4: ...
    @property
    def olto(self) -> IVector4: ...
    @property
    def oltl(self) -> IVector4: ...
    @property
    def olqs(self) -> IVector4: ...
    @property
    def olqt(self) -> IVector4: ...
    @property
    def olqq(self) -> IVector4: ...
    @property
    def olqp(self) -> IVector4: ...
    @property
    def olqo(self) -> IVector4: ...
    @property
    def olql(self) -> IVector4: ...
    @property
    def olps(self) -> IVector4: ...
    @property
    def olpt(self) -> IVector4: ...
    @property
    def olpq(self) -> IVector4: ...
    @property
    def olpp(self) -> IVector4: ...
    @property
    def olpo(self) -> IVector4: ...
    @property
    def olpl(self) -> IVector4: ...
    @property
    def olos(self) -> IVector4: ...
    @property
    def olot(self) -> IVector4: ...
    @property
    def oloq(self) -> IVector4: ...
    @property
    def olop(self) -> IVector4: ...
    @property
    def olls(self) -> IVector4: ...
    @property
    def ollt(self) -> IVector4: ...
    @property
    def ollq(self) -> IVector4: ...
    @property
    def ollp(self) -> IVector4: ...
    @property
    def lsss(self) -> IVector4: ...
    @property
    def lsst(self) -> IVector4: ...
    @property
    def lssq(self) -> IVector4: ...
    @property
    def lssp(self) -> IVector4: ...
    @property
    def lsso(self) -> IVector4: ...
    @property
    def lssl(self) -> IVector4: ...
    @property
    def lsts(self) -> IVector4: ...
    @property
    def lstt(self) -> IVector4: ...
    @property
    def lstq(self) -> IVector4: ...
    @property
    def lstp(self) -> IVector4: ...
    @property
    def lsto(self) -> IVector4: ...
    @property
    def lstl(self) -> IVector4: ...
    @property
    def lsqs(self) -> IVector4: ...
    @property
    def lsqt(self) -> IVector4: ...
    @property
    def lsqq(self) -> IVector4: ...
    @property
    def lsqp(self) -> IVector4: ...
    @property
    def lsqo(self) -> IVector4: ...
    @property
    def lsql(self) -> IVector4: ...
    @property
    def lsps(self) -> IVector4: ...
    @property
    def lspt(self) -> IVector4: ...
    @property
    def lspq(self) -> IVector4: ...
    @property
    def lspp(self) -> IVector4: ...
    @property
    def lspo(self) -> IVector4: ...
    @property
    def lspl(self) -> IVector4: ...
    @property
    def lsos(self) -> IVector4: ...
    @property
    def lsot(self) -> IVector4: ...
    @property
    def lsoq(self) -> IVector4: ...
    @property
    def lsop(self) -> IVector4: ...
    @property
    def lsoo(self) -> IVector4: ...
    @property
    def lsol(self) -> IVector4: ...
    @property
    def lsls(self) -> IVector4: ...
    @property
    def lslt(self) -> IVector4: ...
    @property
    def lslq(self) -> IVector4: ...
    @property
    def lslp(self) -> IVector4: ...
    @property
    def lslo(self) -> IVector4: ...
    @property
    def lsll(self) -> IVector4: ...
    @property
    def ltss(self) -> IVector4: ...
    @property
    def ltst(self) -> IVector4: ...
    @property
    def ltsq(self) -> IVector4: ...
    @property
    def ltsp(self) -> IVector4: ...
    @property
    def ltso(self) -> IVector4: ...
    @property
    def ltsl(self) -> IVector4: ...
    @property
    def ltts(self) -> IVector4: ...
    @property
    def lttt(self) -> IVector4: ...
    @property
    def lttq(self) -> IVector4: ...
    @property
    def lttp(self) -> IVector4: ...
    @property
    def ltto(self) -> IVector4: ...
    @property
    def lttl(self) -> IVector4: ...
    @property
    def ltqs(self) -> IVector4: ...
    @property
    def ltqt(self) -> IVector4: ...
    @property
    def ltqq(self) -> IVector4: ...
    @property
    def ltqp(self) -> IVector4: ...
    @property
    def ltqo(self) -> IVector4: ...
    @property
    def ltql(self) -> IVector4: ...
    @property
    def ltps(self) -> IVector4: ...
    @property
    def ltpt(self) -> IVector4: ...
    @property
    def ltpq(self) -> IVector4: ...
    @property
    def ltpp(self) -> IVector4: ...
    @property
    def ltpo(self) -> IVector4: ...
    @property
    def ltpl(self) -> IVector4: ...
    @property
    def ltos(self) -> IVector4: ...
    @property
    def ltot(self) -> IVector4: ...
    @property
    def ltoq(self) -> IVector4: ...
    @property
    def ltop(self) -> IVector4: ...
    @property
    def ltoo(self) -> IVector4: ...
    @property
    def ltol(self) -> IVector4: ...
    @property
    def ltls(self) -> IVector4: ...
    @property
    def ltlt(self) -> IVector4: ...
    @property
    def ltlq(self) -> IVector4: ...
    @property
    def ltlp(self) -> IVector4: ...
    @property
    def ltlo(self) -> IVector4: ...
    @property
    def ltll(self) -> IVector4: ...
    @property
    def lqss(self) -> IVector4: ...
    @property
    def lqst(self) -> IVector4: ...
    @property
    def lqsq(self) -> IVector4: ...
    @property
    def lqsp(self) -> IVector4: ...
    @property
    def lqso(self) -> IVector4: ...
    @property
    def lqsl(self) -> IVector4: ...
    @property
    def lqts(self) -> IVector4: ...
    @property
    def lqtt(self) -> IVector4: ...
    @property
    def lqtq(self) -> IVector4: ...
    @property
    def lqtp(self) -> IVector4: ...
    @property
    def lqto(self) -> IVector4: ...
    @property
    def lqtl(self) -> IVector4: ...
    @property
    def lqqs(self) -> IVector4: ...
    @property
    def lqqt(self) -> IVector4: ...
    @property
    def lqqq(self) -> IVector4: ...
    @property
    def lqqp(self) -> IVector4: ...
    @property
    def lqqo(self) -> IVector4: ...
    @property
    def lqql(self) -> IVector4: ...
    @property
    def lqps(self) -> IVector4: ...
    @property
    def lqpt(self) -> IVector4: ...
    @property
    def lqpq(self) -> IVector4: ...
    @property
    def lqpp(self) -> IVector4: ...
    @property
    def lqpo(self) -> IVector4: ...
    @property
    def lqpl(self) -> IVector4: ...
    @property
    def lqos(self) -> IVector4: ...
    @property
    def lqot(self) -> IVector4: ...
    @property
    def lqoq(self) -> IVector4: ...
    @property
    def lqop(self) -> IVector4: ...
    @property
    def lqoo(self) -> IVector4: ...
    @property
    def lqol(self) -> IVector4: ...
    @property
    def lqls(self) -> IVector4: ...
    @property
    def lqlt(self) -> IVector4: ...
    @property
    def lqlq(self) -> IVector4: ...
    @property
    def lqlp(self) -> IVector4: ...
    @property
    def lqlo(self) -> IVector4: ...
    @property
    def lqll(self) -> IVector4: ...
    @property
    def lpss(self) -> IVector4: ...
    @property
    def lpst(self) -> IVector4: ...
    @property
    def lpsq(self) -> IVector4: ...
    @property
    def lpsp(self) -> IVector4: ...
    @property
    def lpso(self) -> IVector4: ...
    @property
    def lpsl(self) -> IVector4: ...
    @property
    def lpts(self) -> IVector4: ...
    @property
    def lptt(self) -> IVector4: ...
    @property
    def lptq(self) -> IVector4: ...
    @property
    def lptp(self) -> IVector4: ...
    @property
    def lpto(self) -> IVector4: ...
    @property
    def lptl(self) -> IVector4: ...
    @property
    def lpqs(self) -> IVector4: ...
    @property
    def lpqt(self) -> IVector4: ...
    @property
    def lpqq(self) -> IVector4: ...
    @property
    def lpqp(self) -> IVector4: ...
    @property
    def lpqo(self) -> IVector4: ...
    @property
    def lpql(self) -> IVector4: ...
    @property
    def lpps(self) -> IVector4: ...
    @property
    def lppt(self) -> IVector4: ...
    @property
    def lppq(self) -> IVector4: ...
    @property
    def lppp(self) -> IVector4: ...
    @property
    def lppo(self) -> IVector4: ...
    @property
    def lppl(self) -> IVector4: ...
    @property
    def lpos(self) -> IVector4: ...
    @property
    def lpot(self) -> IVector4: ...
    @property
    def lpoq(self) -> IVector4: ...
    @property
    def lpop(self) -> IVector4: ...
    @property
    def lpoo(self) -> IVector4: ...
    @property
    def lpol(self) -> IVector4: ...
    @property
    def lpls(self) -> IVector4: ...
    @property
    def lplt(self) -> IVector4: ...
    @property
    def lplq(self) -> IVector4: ...
    @property
    def lplp(self) -> IVector4: ...
    @property
    def lplo(self) -> IVector4: ...
    @property
    def lpll(self) -> IVector4: ...
    @property
    def loss(self) -> IVector4: ...
    @property
    def lost(self) -> IVector4: ...
    @property
    def losq(self) -> IVector4: ...
    @property
    def losp(self) -> IVector4: ...
    @property
    def loso(self) -> IVector4: ...
    @property
    def losl(self) -> IVector4: ...
    @property
    def lots(self) -> IVector4: ...
    @property
    def lott(self) -> IVector4: ...
    @property
    def lotq(self) -> IVector4: ...
    @property
    def lotp(self) -> IVector4: ...
    @property
    def loto(self) -> IVector4: ...
    @property
    def lotl(self) -> IVector4: ...
    @property
    def loqs(self) -> IVector4: ...
    @property
    def loqt(self) -> IVector4: ...
    @property
    def loqq(self) -> IVector4: ...
    @property
    def loqp(self) -> IVector4: ...
    @property
    def loqo(self) -> IVector4: ...
    @property
    def loql(self) -> IVector4: ...
    @property
    def lops(self) -> IVector4: ...
    @property
    def lopt(self) -> IVector4: ...
    @property
    def lopq(self) -> IVector4: ...
    @property
    def lopp(self) -> IVector4: ...
    @property
    def lopo(self) -> IVector4: ...
    @property
    def lopl(self) -> IVector4: ...
    @property
    def loos(self) -> IVector4: ...
    @property
    def loot(self) -> IVector4: ...
    @property
    def looq(self) -> IVector4: ...
    @property
    def loop(self) -> IVector4: ...
    @property
    def lols(self) -> IVector4: ...
    @property
    def lolt(self) -> IVector4: ...
    @property
    def lolq(self) -> IVector4: ...
    @property
    def lolp(self) -> IVector4: ...
    @property
    def llss(self) -> IVector4: ...
    @property
    def llst(self) -> IVector4: ...
    @property
    def llsq(self) -> IVector4: ...
    @property
    def llsp(self) -> IVector4: ...
    @property
    def llso(self) -> IVector4: ...
    @property
    def llsl(self) -> IVector4: ...
    @property
    def llts(self) -> IVector4: ...
    @property
    def lltt(self) -> IVector4: ...
    @property
    def lltq(self) -> IVector4: ...
    @property
    def lltp(self) -> IVector4: ...
    @property
    def llto(self) -> IVector4: ...
    @property
    def lltl(self) -> IVector4: ...
    @property
    def llqs(self) -> IVector4: ...
    @property
    def llqt(self) -> IVector4: ...
    @property
    def llqq(self) -> IVector4: ...
    @property
    def llqp(self) -> IVector4: ...
    @property
    def llqo(self) -> IVector4: ...
    @property
    def llql(self) -> IVector4: ...
    @property
    def llps(self) -> IVector4: ...
    @property
    def llpt(self) -> IVector4: ...
    @property
    def llpq(self) -> IVector4: ...
    @property
    def llpp(self) -> IVector4: ...
    @property
    def llpo(self) -> IVector4: ...
    @property
    def llpl(self) -> IVector4: ...
    @property
    def llos(self) -> IVector4: ...
    @property
    def llot(self) -> IVector4: ...
    @property
    def lloq(self) -> IVector4: ...
    @property
    def llop(self) -> IVector4: ...
    @property
    def llls(self) -> IVector4: ...
    @property
    def lllt(self) -> IVector4: ...
    @property
    def lllq(self) -> IVector4: ...
    @property
    def lllp(self) -> IVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> IVector2: ...
    @property
    def uv(self) -> IVector2: ...
    @property
    def uo(self) -> IVector2: ...
    @property
    def ul(self) -> IVector2: ...
    @property
    def vu(self) -> IVector2: ...
    @property
    def vv(self) -> IVector2: ...
    @property
    def vo(self) -> IVector2: ...
    @property
    def vl(self) -> IVector2: ...
    @property
    def ou(self) -> IVector2: ...
    @property
    def ov(self) -> IVector2: ...
    @property
    def lu(self) -> IVector2: ...
    @property
    def lv(self) -> IVector2: ...
    @property
    def uuu(self) -> IVector3: ...
    @property
    def uuv(self) -> IVector3: ...
    @property
    def uuo(self) -> IVector3: ...
    @property
    def uul(self) -> IVector3: ...
    @property
    def uvu(self) -> IVector3: ...
    @property
    def uvv(self) -> IVector3: ...
    @property
    def uvo(self) -> IVector3: ...
    @property
    def uvl(self) -> IVector3: ...
    @property
    def uou(self) -> IVector3: ...
    @property
    def uov(self) -> IVector3: ...
    @property
    def uoo(self) -> IVector3: ...
    @property
    def uol(self) -> IVector3: ...
    @property
    def ulu(self) -> IVector3: ...
    @property
    def ulv(self) -> IVector3: ...
    @property
    def ulo(self) -> IVector3: ...
    @property
    def ull(self) -> IVector3: ...
    @property
    def vuu(self) -> IVector3: ...
    @property
    def vuv(self) -> IVector3: ...
    @property
    def vuo(self) -> IVector3: ...
    @property
    def vul(self) -> IVector3: ...
    @property
    def vvu(self) -> IVector3: ...
    @property
    def vvv(self) -> IVector3: ...
    @property
    def vvo(self) -> IVector3: ...
    @property
    def vvl(self) -> IVector3: ...
    @property
    def vou(self) -> IVector3: ...
    @property
    def vov(self) -> IVector3: ...
    @property
    def voo(self) -> IVector3: ...
    @property
    def vol(self) -> IVector3: ...
    @property
    def vlu(self) -> IVector3: ...
    @property
    def vlv(self) -> IVector3: ...
    @property
    def vlo(self) -> IVector3: ...
    @property
    def vll(self) -> IVector3: ...
    @property
    def ouu(self) -> IVector3: ...
    @property
    def ouv(self) -> IVector3: ...
    @property
    def ouo(self) -> IVector3: ...
    @property
    def oul(self) -> IVector3: ...
    @property
    def ovu(self) -> IVector3: ...
    @property
    def ovv(self) -> IVector3: ...
    @property
    def ovo(self) -> IVector3: ...
    @property
    def ovl(self) -> IVector3: ...
    @property
    def oou(self) -> IVector3: ...
    @property
    def oov(self) -> IVector3: ...
    @property
    def olu(self) -> IVector3: ...
    @property
    def olv(self) -> IVector3: ...
    @property
    def luu(self) -> IVector3: ...
    @property
    def luv(self) -> IVector3: ...
    @property
    def luo(self) -> IVector3: ...
    @property
    def lul(self) -> IVector3: ...
    @property
    def lvu(self) -> IVector3: ...
    @property
    def lvv(self) -> IVector3: ...
    @property
    def lvo(self) -> IVector3: ...
    @property
    def lvl(self) -> IVector3: ...
    @property
    def lou(self) -> IVector3: ...
    @property
    def lov(self) -> IVector3: ...
    @property
    def llu(self) -> IVector3: ...
    @property
    def llv(self) -> IVector3: ...
    @property
    def uuuu(self) -> IVector4: ...
    @property
    def uuuv(self) -> IVector4: ...
    @property
    def uuuo(self) -> IVector4: ...
    @property
    def uuul(self) -> IVector4: ...
    @property
    def uuvu(self) -> IVector4: ...
    @property
    def uuvv(self) -> IVector4: ...
    @property
    def uuvo(self) -> IVector4: ...
    @property
    def uuvl(self) -> IVector4: ...
    @property
    def uuou(self) -> IVector4: ...
    @property
    def uuov(self) -> IVector4: ...
    @property
    def uuoo(self) -> IVector4: ...
    @property
    def uuol(self) -> IVector4: ...
    @property
    def uulu(self) -> IVector4: ...
    @property
    def uulv(self) -> IVector4: ...
    @property
    def uulo(self) -> IVector4: ...
    @property
    def uull(self) -> IVector4: ...
    @property
    def uvuu(self) -> IVector4: ...
    @property
    def uvuv(self) -> IVector4: ...
    @property
    def uvuo(self) -> IVector4: ...
    @property
    def uvul(self) -> IVector4: ...
    @property
    def uvvu(self) -> IVector4: ...
    @property
    def uvvv(self) -> IVector4: ...
    @property
    def uvvo(self) -> IVector4: ...
    @property
    def uvvl(self) -> IVector4: ...
    @property
    def uvou(self) -> IVector4: ...
    @property
    def uvov(self) -> IVector4: ...
    @property
    def uvoo(self) -> IVector4: ...
    @property
    def uvol(self) -> IVector4: ...
    @property
    def uvlu(self) -> IVector4: ...
    @property
    def uvlv(self) -> IVector4: ...
    @property
    def uvlo(self) -> IVector4: ...
    @property
    def uvll(self) -> IVector4: ...
    @property
    def uouu(self) -> IVector4: ...
    @property
    def uouv(self) -> IVector4: ...
    @property
    def uouo(self) -> IVector4: ...
    @property
    def uoul(self) -> IVector4: ...
    @property
    def uovu(self) -> IVector4: ...
    @property
    def uovv(self) -> IVector4: ...
    @property
    def uovo(self) -> IVector4: ...
    @property
    def uovl(self) -> IVector4: ...
    @property
    def uoou(self) -> IVector4: ...
    @property
    def uoov(self) -> IVector4: ...
    @property
    def uooo(self) -> IVector4: ...
    @property
    def uool(self) -> IVector4: ...
    @property
    def uolu(self) -> IVector4: ...
    @property
    def uolv(self) -> IVector4: ...
    @property
    def uolo(self) -> IVector4: ...
    @property
    def uoll(self) -> IVector4: ...
    @property
    def uluu(self) -> IVector4: ...
    @property
    def uluv(self) -> IVector4: ...
    @property
    def uluo(self) -> IVector4: ...
    @property
    def ulul(self) -> IVector4: ...
    @property
    def ulvu(self) -> IVector4: ...
    @property
    def ulvv(self) -> IVector4: ...
    @property
    def ulvo(self) -> IVector4: ...
    @property
    def ulvl(self) -> IVector4: ...
    @property
    def ulou(self) -> IVector4: ...
    @property
    def ulov(self) -> IVector4: ...
    @property
    def uloo(self) -> IVector4: ...
    @property
    def ulol(self) -> IVector4: ...
    @property
    def ullu(self) -> IVector4: ...
    @property
    def ullv(self) -> IVector4: ...
    @property
    def ullo(self) -> IVector4: ...
    @property
    def ulll(self) -> IVector4: ...
    @property
    def vuuu(self) -> IVector4: ...
    @property
    def vuuv(self) -> IVector4: ...
    @property
    def vuuo(self) -> IVector4: ...
    @property
    def vuul(self) -> IVector4: ...
    @property
    def vuvu(self) -> IVector4: ...
    @property
    def vuvv(self) -> IVector4: ...
    @property
    def vuvo(self) -> IVector4: ...
    @property
    def vuvl(self) -> IVector4: ...
    @property
    def vuou(self) -> IVector4: ...
    @property
    def vuov(self) -> IVector4: ...
    @property
    def vuoo(self) -> IVector4: ...
    @property
    def vuol(self) -> IVector4: ...
    @property
    def vulu(self) -> IVector4: ...
    @property
    def vulv(self) -> IVector4: ...
    @property
    def vulo(self) -> IVector4: ...
    @property
    def vull(self) -> IVector4: ...
    @property
    def vvuu(self) -> IVector4: ...
    @property
    def vvuv(self) -> IVector4: ...
    @property
    def vvuo(self) -> IVector4: ...
    @property
    def vvul(self) -> IVector4: ...
    @property
    def vvvu(self) -> IVector4: ...
    @property
    def vvvv(self) -> IVector4: ...
    @property
    def vvvo(self) -> IVector4: ...
    @property
    def vvvl(self) -> IVector4: ...
    @property
    def vvou(self) -> IVector4: ...
    @property
    def vvov(self) -> IVector4: ...
    @property
    def vvoo(self) -> IVector4: ...
    @property
    def vvol(self) -> IVector4: ...
    @property
    def vvlu(self) -> IVector4: ...
    @property
    def vvlv(self) -> IVector4: ...
    @property
    def vvlo(self) -> IVector4: ...
    @property
    def vvll(self) -> IVector4: ...
    @property
    def vouu(self) -> IVector4: ...
    @property
    def vouv(self) -> IVector4: ...
    @property
    def vouo(self) -> IVector4: ...
    @property
    def voul(self) -> IVector4: ...
    @property
    def vovu(self) -> IVector4: ...
    @property
    def vovv(self) -> IVector4: ...
    @property
    def vovo(self) -> IVector4: ...
    @property
    def vovl(self) -> IVector4: ...
    @property
    def voou(self) -> IVector4: ...
    @property
    def voov(self) -> IVector4: ...
    @property
    def vooo(self) -> IVector4: ...
    @property
    def vool(self) -> IVector4: ...
    @property
    def volu(self) -> IVector4: ...
    @property
    def volv(self) -> IVector4: ...
    @property
    def volo(self) -> IVector4: ...
    @property
    def voll(self) -> IVector4: ...
    @property
    def vluu(self) -> IVector4: ...
    @property
    def vluv(self) -> IVector4: ...
    @property
    def vluo(self) -> IVector4: ...
    @property
    def vlul(self) -> IVector4: ...
    @property
    def vlvu(self) -> IVector4: ...
    @property
    def vlvv(self) -> IVector4: ...
    @property
    def vlvo(self) -> IVector4: ...
    @property
    def vlvl(self) -> IVector4: ...
    @property
    def vlou(self) -> IVector4: ...
    @property
    def vlov(self) -> IVector4: ...
    @property
    def vloo(self) -> IVector4: ...
    @property
    def vlol(self) -> IVector4: ...
    @property
    def vllu(self) -> IVector4: ...
    @property
    def vllv(self) -> IVector4: ...
    @property
    def vllo(self) -> IVector4: ...
    @property
    def vlll(self) -> IVector4: ...
    @property
    def ouuu(self) -> IVector4: ...
    @property
    def ouuv(self) -> IVector4: ...
    @property
    def ouuo(self) -> IVector4: ...
    @property
    def ouul(self) -> IVector4: ...
    @property
    def ouvu(self) -> IVector4: ...
    @property
    def ouvv(self) -> IVector4: ...
    @property
    def ouvo(self) -> IVector4: ...
    @property
    def ouvl(self) -> IVector4: ...
    @property
    def ouou(self) -> IVector4: ...
    @property
    def ouov(self) -> IVector4: ...
    @property
    def ouoo(self) -> IVector4: ...
    @property
    def ouol(self) -> IVector4: ...
    @property
    def oulu(self) -> IVector4: ...
    @property
    def oulv(self) -> IVector4: ...
    @property
    def oulo(self) -> IVector4: ...
    @property
    def oull(self) -> IVector4: ...
    @property
    def ovuu(self) -> IVector4: ...
    @property
    def ovuv(self) -> IVector4: ...
    @property
    def ovuo(self) -> IVector4: ...
    @property
    def ovul(self) -> IVector4: ...
    @property
    def ovvu(self) -> IVector4: ...
    @property
    def ovvv(self) -> IVector4: ...
    @property
    def ovvo(self) -> IVector4: ...
    @property
    def ovvl(self) -> IVector4: ...
    @property
    def ovou(self) -> IVector4: ...
    @property
    def ovov(self) -> IVector4: ...
    @property
    def ovoo(self) -> IVector4: ...
    @property
    def ovol(self) -> IVector4: ...
    @property
    def ovlu(self) -> IVector4: ...
    @property
    def ovlv(self) -> IVector4: ...
    @property
    def ovlo(self) -> IVector4: ...
    @property
    def ovll(self) -> IVector4: ...
    @property
    def oouu(self) -> IVector4: ...
    @property
    def oouv(self) -> IVector4: ...
    @property
    def oouo(self) -> IVector4: ...
    @property
    def ooul(self) -> IVector4: ...
    @property
    def oovu(self) -> IVector4: ...
    @property
    def oovv(self) -> IVector4: ...
    @property
    def oovo(self) -> IVector4: ...
    @property
    def oovl(self) -> IVector4: ...
    @property
    def ooou(self) -> IVector4: ...
    @property
    def ooov(self) -> IVector4: ...
    @property
    def oolu(self) -> IVector4: ...
    @property
    def oolv(self) -> IVector4: ...
    @property
    def oluu(self) -> IVector4: ...
    @property
    def oluv(self) -> IVector4: ...
    @property
    def oluo(self) -> IVector4: ...
    @property
    def olul(self) -> IVector4: ...
    @property
    def olvu(self) -> IVector4: ...
    @property
    def olvv(self) -> IVector4: ...
    @property
    def olvo(self) -> IVector4: ...
    @property
    def olvl(self) -> IVector4: ...
    @property
    def olou(self) -> IVector4: ...
    @property
    def olov(self) -> IVector4: ...
    @property
    def ollu(self) -> IVector4: ...
    @property
    def ollv(self) -> IVector4: ...
    @property
    def luuu(self) -> IVector4: ...
    @property
    def luuv(self) -> IVector4: ...
    @property
    def luuo(self) -> IVector4: ...
    @property
    def luul(self) -> IVector4: ...
    @property
    def luvu(self) -> IVector4: ...
    @property
    def luvv(self) -> IVector4: ...
    @property
    def luvo(self) -> IVector4: ...
    @property
    def luvl(self) -> IVector4: ...
    @property
    def luou(self) -> IVector4: ...
    @property
    def luov(self) -> IVector4: ...
    @property
    def luoo(self) -> IVector4: ...
    @property
    def luol(self) -> IVector4: ...
    @property
    def lulu(self) -> IVector4: ...
    @property
    def lulv(self) -> IVector4: ...
    @property
    def lulo(self) -> IVector4: ...
    @property
    def lull(self) -> IVector4: ...
    @property
    def lvuu(self) -> IVector4: ...
    @property
    def lvuv(self) -> IVector4: ...
    @property
    def lvuo(self) -> IVector4: ...
    @property
    def lvul(self) -> IVector4: ...
    @property
    def lvvu(self) -> IVector4: ...
    @property
    def lvvv(self) -> IVector4: ...
    @property
    def lvvo(self) -> IVector4: ...
    @property
    def lvvl(self) -> IVector4: ...
    @property
    def lvou(self) -> IVector4: ...
    @property
    def lvov(self) -> IVector4: ...
    @property
    def lvoo(self) -> IVector4: ...
    @property
    def lvol(self) -> IVector4: ...
    @property
    def lvlu(self) -> IVector4: ...
    @property
    def lvlv(self) -> IVector4: ...
    @property
    def lvlo(self) -> IVector4: ...
    @property
    def lvll(self) -> IVector4: ...
    @property
    def louu(self) -> IVector4: ...
    @property
    def louv(self) -> IVector4: ...
    @property
    def louo(self) -> IVector4: ...
    @property
    def loul(self) -> IVector4: ...
    @property
    def lovu(self) -> IVector4: ...
    @property
    def lovv(self) -> IVector4: ...
    @property
    def lovo(self) -> IVector4: ...
    @property
    def lovl(self) -> IVector4: ...
    @property
    def loou(self) -> IVector4: ...
    @property
    def loov(self) -> IVector4: ...
    @property
    def lolu(self) -> IVector4: ...
    @property
    def lolv(self) -> IVector4: ...
    @property
    def lluu(self) -> IVector4: ...
    @property
    def lluv(self) -> IVector4: ...
    @property
    def lluo(self) -> IVector4: ...
    @property
    def llul(self) -> IVector4: ...
    @property
    def llvu(self) -> IVector4: ...
    @property
    def llvv(self) -> IVector4: ...
    @property
    def llvo(self) -> IVector4: ...
    @property
    def llvl(self) -> IVector4: ...
    @property
    def llou(self) -> IVector4: ...
    @property
    def llov(self) -> IVector4: ...
    @property
    def lllu(self) -> IVector4: ...
    @property
    def lllv(self) -> IVector4: ...
    @property
    def oo(self) -> IVector2: ...
    @property
    def ol(self) -> IVector2: ...
    @property
    def lo(self) -> IVector2: ...
    @property
    def ll(self) -> IVector2: ...
    @property
    def ooo(self) -> IVector3: ...
    @property
    def ool(self) -> IVector3: ...
    @property
    def olo(self) -> IVector3: ...
    @property
    def oll(self) -> IVector3: ...
    @property
    def loo(self) -> IVector3: ...
    @property
    def lol(self) -> IVector3: ...
    @property
    def llo(self) -> IVector3: ...
    @property
    def lll(self) -> IVector3: ...
    @property
    def oooo(self) -> IVector4: ...
    @property
    def oool(self) -> IVector4: ...
    @property
    def oolo(self) -> IVector4: ...
    @property
    def ooll(self) -> IVector4: ...
    @property
    def oloo(self) -> IVector4: ...
    @property
    def olol(self) -> IVector4: ...
    @property
    def ollo(self) -> IVector4: ...
    @property
    def olll(self) -> IVector4: ...
    @property
    def looo(self) -> IVector4: ...
    @property
    def lool(self) -> IVector4: ...
    @property
    def lolo(self) -> IVector4: ...
    @property
    def loll(self) -> IVector4: ...
    @property
    def lloo(self) -> IVector4: ...
    @property
    def llol(self) -> IVector4: ...
    @property
    def lllo(self) -> IVector4: ...
    @property
    def llll(self) -> IVector4: ...
    def min(self, n: Number, /) -> IVector4: ...
    def max(self, n: Number, /) -> IVector4: ...
    def clamp(self, min: Number, max: Number, /) -> IVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[IVector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector4: ...

@final
class IVector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: IVector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[IVector4]: ...
    @overload
    def __getitem__(self, index: int) -> IVector4: ...
    @overload
    def __getitem__(self, index: slice) -> IVector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> IVector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[IVector4]: ...

@final
class UVector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: UVector4) -> UVector4: ...
    @overload
    def __add__(self, other: Number) -> UVector4: ...
    @overload
    def __radd__(self, other: UVector4) -> UVector4: ...
    @overload
    def __radd__(self, other: Number) -> UVector4: ...
    @overload
    def __sub__(self, other: UVector4) -> UVector4: ...
    @overload
    def __sub__(self, other: Number) -> UVector4: ...
    @overload
    def __rsub__(self, other: UVector4) -> UVector4: ...
    @overload
    def __rsub__(self, other: Number) -> UVector4: ...
    @overload
    def __mul__(self, other: UVector4) -> UVector4: ...
    @overload
    def __mul__(self, other: Number) -> UVector4: ...
    @overload
    def __rmul__(self, other: UVector4) -> UVector4: ...
    @overload
    def __rmul__(self, other: Number) -> UVector4: ...
    @overload
    def __truediv__(self, other: UVector4) -> UVector4: ...
    @overload
    def __truediv__(self, other: Number) -> UVector4: ...
    @overload
    def __rtruediv__(self, other: UVector4) -> UVector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> UVector4: ...
    def __abs__(self) -> UVector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> UVector2: ...
    @property
    def xy(self) -> UVector2: ...
    @property
    def xz(self) -> UVector2: ...
    @property
    def xw(self) -> UVector2: ...
    @property
    def xo(self) -> UVector2: ...
    @property
    def xl(self) -> UVector2: ...
    @property
    def yx(self) -> UVector2: ...
    @property
    def yy(self) -> UVector2: ...
    @property
    def yz(self) -> UVector2: ...
    @property
    def yw(self) -> UVector2: ...
    @property
    def yo(self) -> UVector2: ...
    @property
    def yl(self) -> UVector2: ...
    @property
    def zx(self) -> UVector2: ...
    @property
    def zy(self) -> UVector2: ...
    @property
    def zz(self) -> UVector2: ...
    @property
    def zw(self) -> UVector2: ...
    @property
    def zo(self) -> UVector2: ...
    @property
    def zl(self) -> UVector2: ...
    @property
    def wx(self) -> UVector2: ...
    @property
    def wy(self) -> UVector2: ...
    @property
    def wz(self) -> UVector2: ...
    @property
    def ww(self) -> UVector2: ...
    @property
    def wo(self) -> UVector2: ...
    @property
    def wl(self) -> UVector2: ...
    @property
    def ox(self) -> UVector2: ...
    @property
    def oy(self) -> UVector2: ...
    @property
    def oz(self) -> UVector2: ...
    @property
    def ow(self) -> UVector2: ...
    @property
    def lx(self) -> UVector2: ...
    @property
    def ly(self) -> UVector2: ...
    @property
    def lz(self) -> UVector2: ...
    @property
    def lw(self) -> UVector2: ...
    @property
    def xxx(self) -> UVector3: ...
    @property
    def xxy(self) -> UVector3: ...
    @property
    def xxz(self) -> UVector3: ...
    @property
    def xxw(self) -> UVector3: ...
    @property
    def xxo(self) -> UVector3: ...
    @property
    def xxl(self) -> UVector3: ...
    @property
    def xyx(self) -> UVector3: ...
    @property
    def xyy(self) -> UVector3: ...
    @property
    def xyz(self) -> UVector3: ...
    @property
    def xyw(self) -> UVector3: ...
    @property
    def xyo(self) -> UVector3: ...
    @property
    def xyl(self) -> UVector3: ...
    @property
    def xzx(self) -> UVector3: ...
    @property
    def xzy(self) -> UVector3: ...
    @property
    def xzz(self) -> UVector3: ...
    @property
    def xzw(self) -> UVector3: ...
    @property
    def xzo(self) -> UVector3: ...
    @property
    def xzl(self) -> UVector3: ...
    @property
    def xwx(self) -> UVector3: ...
    @property
    def xwy(self) -> UVector3: ...
    @property
    def xwz(self) -> UVector3: ...
    @property
    def xww(self) -> UVector3: ...
    @property
    def xwo(self) -> UVector3: ...
    @property
    def xwl(self) -> UVector3: ...
    @property
    def xox(self) -> UVector3: ...
    @property
    def xoy(self) -> UVector3: ...
    @property
    def xoz(self) -> UVector3: ...
    @property
    def xow(self) -> UVector3: ...
    @property
    def xoo(self) -> UVector3: ...
    @property
    def xol(self) -> UVector3: ...
    @property
    def xlx(self) -> UVector3: ...
    @property
    def xly(self) -> UVector3: ...
    @property
    def xlz(self) -> UVector3: ...
    @property
    def xlw(self) -> UVector3: ...
    @property
    def xlo(self) -> UVector3: ...
    @property
    def xll(self) -> UVector3: ...
    @property
    def yxx(self) -> UVector3: ...
    @property
    def yxy(self) -> UVector3: ...
    @property
    def yxz(self) -> UVector3: ...
    @property
    def yxw(self) -> UVector3: ...
    @property
    def yxo(self) -> UVector3: ...
    @property
    def yxl(self) -> UVector3: ...
    @property
    def yyx(self) -> UVector3: ...
    @property
    def yyy(self) -> UVector3: ...
    @property
    def yyz(self) -> UVector3: ...
    @property
    def yyw(self) -> UVector3: ...
    @property
    def yyo(self) -> UVector3: ...
    @property
    def yyl(self) -> UVector3: ...
    @property
    def yzx(self) -> UVector3: ...
    @property
    def yzy(self) -> UVector3: ...
    @property
    def yzz(self) -> UVector3: ...
    @property
    def yzw(self) -> UVector3: ...
    @property
    def yzo(self) -> UVector3: ...
    @property
    def yzl(self) -> UVector3: ...
    @property
    def ywx(self) -> UVector3: ...
    @property
    def ywy(self) -> UVector3: ...
    @property
    def ywz(self) -> UVector3: ...
    @property
    def yww(self) -> UVector3: ...
    @property
    def ywo(self) -> UVector3: ...
    @property
    def ywl(self) -> UVector3: ...
    @property
    def yox(self) -> UVector3: ...
    @property
    def yoy(self) -> UVector3: ...
    @property
    def yoz(self) -> UVector3: ...
    @property
    def yow(self) -> UVector3: ...
    @property
    def yoo(self) -> UVector3: ...
    @property
    def yol(self) -> UVector3: ...
    @property
    def ylx(self) -> UVector3: ...
    @property
    def yly(self) -> UVector3: ...
    @property
    def ylz(self) -> UVector3: ...
    @property
    def ylw(self) -> UVector3: ...
    @property
    def ylo(self) -> UVector3: ...
    @property
    def yll(self) -> UVector3: ...
    @property
    def zxx(self) -> UVector3: ...
    @property
    def zxy(self) -> UVector3: ...
    @property
    def zxz(self) -> UVector3: ...
    @property
    def zxw(self) -> UVector3: ...
    @property
    def zxo(self) -> UVector3: ...
    @property
    def zxl(self) -> UVector3: ...
    @property
    def zyx(self) -> UVector3: ...
    @property
    def zyy(self) -> UVector3: ...
    @property
    def zyz(self) -> UVector3: ...
    @property
    def zyw(self) -> UVector3: ...
    @property
    def zyo(self) -> UVector3: ...
    @property
    def zyl(self) -> UVector3: ...
    @property
    def zzx(self) -> UVector3: ...
    @property
    def zzy(self) -> UVector3: ...
    @property
    def zzz(self) -> UVector3: ...
    @property
    def zzw(self) -> UVector3: ...
    @property
    def zzo(self) -> UVector3: ...
    @property
    def zzl(self) -> UVector3: ...
    @property
    def zwx(self) -> UVector3: ...
    @property
    def zwy(self) -> UVector3: ...
    @property
    def zwz(self) -> UVector3: ...
    @property
    def zww(self) -> UVector3: ...
    @property
    def zwo(self) -> UVector3: ...
    @property
    def zwl(self) -> UVector3: ...
    @property
    def zox(self) -> UVector3: ...
    @property
    def zoy(self) -> UVector3: ...
    @property
    def zoz(self) -> UVector3: ...
    @property
    def zow(self) -> UVector3: ...
    @property
    def zoo(self) -> UVector3: ...
    @property
    def zol(self) -> UVector3: ...
    @property
    def zlx(self) -> UVector3: ...
    @property
    def zly(self) -> UVector3: ...
    @property
    def zlz(self) -> UVector3: ...
    @property
    def zlw(self) -> UVector3: ...
    @property
    def zlo(self) -> UVector3: ...
    @property
    def zll(self) -> UVector3: ...
    @property
    def wxx(self) -> UVector3: ...
    @property
    def wxy(self) -> UVector3: ...
    @property
    def wxz(self) -> UVector3: ...
    @property
    def wxw(self) -> UVector3: ...
    @property
    def wxo(self) -> UVector3: ...
    @property
    def wxl(self) -> UVector3: ...
    @property
    def wyx(self) -> UVector3: ...
    @property
    def wyy(self) -> UVector3: ...
    @property
    def wyz(self) -> UVector3: ...
    @property
    def wyw(self) -> UVector3: ...
    @property
    def wyo(self) -> UVector3: ...
    @property
    def wyl(self) -> UVector3: ...
    @property
    def wzx(self) -> UVector3: ...
    @property
    def wzy(self) -> UVector3: ...
    @property
    def wzz(self) -> UVector3: ...
    @property
    def wzw(self) -> UVector3: ...
    @property
    def wzo(self) -> UVector3: ...
    @property
    def wzl(self) -> UVector3: ...
    @property
    def wwx(self) -> UVector3: ...
    @property
    def wwy(self) -> UVector3: ...
    @property
    def wwz(self) -> UVector3: ...
    @property
    def www(self) -> UVector3: ...
    @property
    def wwo(self) -> UVector3: ...
    @property
    def wwl(self) -> UVector3: ...
    @property
    def wox(self) -> UVector3: ...
    @property
    def woy(self) -> UVector3: ...
    @property
    def woz(self) -> UVector3: ...
    @property
    def wow(self) -> UVector3: ...
    @property
    def woo(self) -> UVector3: ...
    @property
    def wol(self) -> UVector3: ...
    @property
    def wlx(self) -> UVector3: ...
    @property
    def wly(self) -> UVector3: ...
    @property
    def wlz(self) -> UVector3: ...
    @property
    def wlw(self) -> UVector3: ...
    @property
    def wlo(self) -> UVector3: ...
    @property
    def wll(self) -> UVector3: ...
    @property
    def oxx(self) -> UVector3: ...
    @property
    def oxy(self) -> UVector3: ...
    @property
    def oxz(self) -> UVector3: ...
    @property
    def oxw(self) -> UVector3: ...
    @property
    def oxo(self) -> UVector3: ...
    @property
    def oxl(self) -> UVector3: ...
    @property
    def oyx(self) -> UVector3: ...
    @property
    def oyy(self) -> UVector3: ...
    @property
    def oyz(self) -> UVector3: ...
    @property
    def oyw(self) -> UVector3: ...
    @property
    def oyo(self) -> UVector3: ...
    @property
    def oyl(self) -> UVector3: ...
    @property
    def ozx(self) -> UVector3: ...
    @property
    def ozy(self) -> UVector3: ...
    @property
    def ozz(self) -> UVector3: ...
    @property
    def ozw(self) -> UVector3: ...
    @property
    def ozo(self) -> UVector3: ...
    @property
    def ozl(self) -> UVector3: ...
    @property
    def owx(self) -> UVector3: ...
    @property
    def owy(self) -> UVector3: ...
    @property
    def owz(self) -> UVector3: ...
    @property
    def oww(self) -> UVector3: ...
    @property
    def owo(self) -> UVector3: ...
    @property
    def owl(self) -> UVector3: ...
    @property
    def oox(self) -> UVector3: ...
    @property
    def ooy(self) -> UVector3: ...
    @property
    def ooz(self) -> UVector3: ...
    @property
    def oow(self) -> UVector3: ...
    @property
    def olx(self) -> UVector3: ...
    @property
    def oly(self) -> UVector3: ...
    @property
    def olz(self) -> UVector3: ...
    @property
    def olw(self) -> UVector3: ...
    @property
    def lxx(self) -> UVector3: ...
    @property
    def lxy(self) -> UVector3: ...
    @property
    def lxz(self) -> UVector3: ...
    @property
    def lxw(self) -> UVector3: ...
    @property
    def lxo(self) -> UVector3: ...
    @property
    def lxl(self) -> UVector3: ...
    @property
    def lyx(self) -> UVector3: ...
    @property
    def lyy(self) -> UVector3: ...
    @property
    def lyz(self) -> UVector3: ...
    @property
    def lyw(self) -> UVector3: ...
    @property
    def lyo(self) -> UVector3: ...
    @property
    def lyl(self) -> UVector3: ...
    @property
    def lzx(self) -> UVector3: ...
    @property
    def lzy(self) -> UVector3: ...
    @property
    def lzz(self) -> UVector3: ...
    @property
    def lzw(self) -> UVector3: ...
    @property
    def lzo(self) -> UVector3: ...
    @property
    def lzl(self) -> UVector3: ...
    @property
    def lwx(self) -> UVector3: ...
    @property
    def lwy(self) -> UVector3: ...
    @property
    def lwz(self) -> UVector3: ...
    @property
    def lww(self) -> UVector3: ...
    @property
    def lwo(self) -> UVector3: ...
    @property
    def lwl(self) -> UVector3: ...
    @property
    def lox(self) -> UVector3: ...
    @property
    def loy(self) -> UVector3: ...
    @property
    def loz(self) -> UVector3: ...
    @property
    def low(self) -> UVector3: ...
    @property
    def llx(self) -> UVector3: ...
    @property
    def lly(self) -> UVector3: ...
    @property
    def llz(self) -> UVector3: ...
    @property
    def llw(self) -> UVector3: ...
    @property
    def xxxx(self) -> UVector4: ...
    @property
    def xxxy(self) -> UVector4: ...
    @property
    def xxxz(self) -> UVector4: ...
    @property
    def xxxw(self) -> UVector4: ...
    @property
    def xxxo(self) -> UVector4: ...
    @property
    def xxxl(self) -> UVector4: ...
    @property
    def xxyx(self) -> UVector4: ...
    @property
    def xxyy(self) -> UVector4: ...
    @property
    def xxyz(self) -> UVector4: ...
    @property
    def xxyw(self) -> UVector4: ...
    @property
    def xxyo(self) -> UVector4: ...
    @property
    def xxyl(self) -> UVector4: ...
    @property
    def xxzx(self) -> UVector4: ...
    @property
    def xxzy(self) -> UVector4: ...
    @property
    def xxzz(self) -> UVector4: ...
    @property
    def xxzw(self) -> UVector4: ...
    @property
    def xxzo(self) -> UVector4: ...
    @property
    def xxzl(self) -> UVector4: ...
    @property
    def xxwx(self) -> UVector4: ...
    @property
    def xxwy(self) -> UVector4: ...
    @property
    def xxwz(self) -> UVector4: ...
    @property
    def xxww(self) -> UVector4: ...
    @property
    def xxwo(self) -> UVector4: ...
    @property
    def xxwl(self) -> UVector4: ...
    @property
    def xxox(self) -> UVector4: ...
    @property
    def xxoy(self) -> UVector4: ...
    @property
    def xxoz(self) -> UVector4: ...
    @property
    def xxow(self) -> UVector4: ...
    @property
    def xxoo(self) -> UVector4: ...
    @property
    def xxol(self) -> UVector4: ...
    @property
    def xxlx(self) -> UVector4: ...
    @property
    def xxly(self) -> UVector4: ...
    @property
    def xxlz(self) -> UVector4: ...
    @property
    def xxlw(self) -> UVector4: ...
    @property
    def xxlo(self) -> UVector4: ...
    @property
    def xxll(self) -> UVector4: ...
    @property
    def xyxx(self) -> UVector4: ...
    @property
    def xyxy(self) -> UVector4: ...
    @property
    def xyxz(self) -> UVector4: ...
    @property
    def xyxw(self) -> UVector4: ...
    @property
    def xyxo(self) -> UVector4: ...
    @property
    def xyxl(self) -> UVector4: ...
    @property
    def xyyx(self) -> UVector4: ...
    @property
    def xyyy(self) -> UVector4: ...
    @property
    def xyyz(self) -> UVector4: ...
    @property
    def xyyw(self) -> UVector4: ...
    @property
    def xyyo(self) -> UVector4: ...
    @property
    def xyyl(self) -> UVector4: ...
    @property
    def xyzx(self) -> UVector4: ...
    @property
    def xyzy(self) -> UVector4: ...
    @property
    def xyzz(self) -> UVector4: ...
    @property
    def xyzw(self) -> UVector4: ...
    @property
    def xyzo(self) -> UVector4: ...
    @property
    def xyzl(self) -> UVector4: ...
    @property
    def xywx(self) -> UVector4: ...
    @property
    def xywy(self) -> UVector4: ...
    @property
    def xywz(self) -> UVector4: ...
    @property
    def xyww(self) -> UVector4: ...
    @property
    def xywo(self) -> UVector4: ...
    @property
    def xywl(self) -> UVector4: ...
    @property
    def xyox(self) -> UVector4: ...
    @property
    def xyoy(self) -> UVector4: ...
    @property
    def xyoz(self) -> UVector4: ...
    @property
    def xyow(self) -> UVector4: ...
    @property
    def xyoo(self) -> UVector4: ...
    @property
    def xyol(self) -> UVector4: ...
    @property
    def xylx(self) -> UVector4: ...
    @property
    def xyly(self) -> UVector4: ...
    @property
    def xylz(self) -> UVector4: ...
    @property
    def xylw(self) -> UVector4: ...
    @property
    def xylo(self) -> UVector4: ...
    @property
    def xyll(self) -> UVector4: ...
    @property
    def xzxx(self) -> UVector4: ...
    @property
    def xzxy(self) -> UVector4: ...
    @property
    def xzxz(self) -> UVector4: ...
    @property
    def xzxw(self) -> UVector4: ...
    @property
    def xzxo(self) -> UVector4: ...
    @property
    def xzxl(self) -> UVector4: ...
    @property
    def xzyx(self) -> UVector4: ...
    @property
    def xzyy(self) -> UVector4: ...
    @property
    def xzyz(self) -> UVector4: ...
    @property
    def xzyw(self) -> UVector4: ...
    @property
    def xzyo(self) -> UVector4: ...
    @property
    def xzyl(self) -> UVector4: ...
    @property
    def xzzx(self) -> UVector4: ...
    @property
    def xzzy(self) -> UVector4: ...
    @property
    def xzzz(self) -> UVector4: ...
    @property
    def xzzw(self) -> UVector4: ...
    @property
    def xzzo(self) -> UVector4: ...
    @property
    def xzzl(self) -> UVector4: ...
    @property
    def xzwx(self) -> UVector4: ...
    @property
    def xzwy(self) -> UVector4: ...
    @property
    def xzwz(self) -> UVector4: ...
    @property
    def xzww(self) -> UVector4: ...
    @property
    def xzwo(self) -> UVector4: ...
    @property
    def xzwl(self) -> UVector4: ...
    @property
    def xzox(self) -> UVector4: ...
    @property
    def xzoy(self) -> UVector4: ...
    @property
    def xzoz(self) -> UVector4: ...
    @property
    def xzow(self) -> UVector4: ...
    @property
    def xzoo(self) -> UVector4: ...
    @property
    def xzol(self) -> UVector4: ...
    @property
    def xzlx(self) -> UVector4: ...
    @property
    def xzly(self) -> UVector4: ...
    @property
    def xzlz(self) -> UVector4: ...
    @property
    def xzlw(self) -> UVector4: ...
    @property
    def xzlo(self) -> UVector4: ...
    @property
    def xzll(self) -> UVector4: ...
    @property
    def xwxx(self) -> UVector4: ...
    @property
    def xwxy(self) -> UVector4: ...
    @property
    def xwxz(self) -> UVector4: ...
    @property
    def xwxw(self) -> UVector4: ...
    @property
    def xwxo(self) -> UVector4: ...
    @property
    def xwxl(self) -> UVector4: ...
    @property
    def xwyx(self) -> UVector4: ...
    @property
    def xwyy(self) -> UVector4: ...
    @property
    def xwyz(self) -> UVector4: ...
    @property
    def xwyw(self) -> UVector4: ...
    @property
    def xwyo(self) -> UVector4: ...
    @property
    def xwyl(self) -> UVector4: ...
    @property
    def xwzx(self) -> UVector4: ...
    @property
    def xwzy(self) -> UVector4: ...
    @property
    def xwzz(self) -> UVector4: ...
    @property
    def xwzw(self) -> UVector4: ...
    @property
    def xwzo(self) -> UVector4: ...
    @property
    def xwzl(self) -> UVector4: ...
    @property
    def xwwx(self) -> UVector4: ...
    @property
    def xwwy(self) -> UVector4: ...
    @property
    def xwwz(self) -> UVector4: ...
    @property
    def xwww(self) -> UVector4: ...
    @property
    def xwwo(self) -> UVector4: ...
    @property
    def xwwl(self) -> UVector4: ...
    @property
    def xwox(self) -> UVector4: ...
    @property
    def xwoy(self) -> UVector4: ...
    @property
    def xwoz(self) -> UVector4: ...
    @property
    def xwow(self) -> UVector4: ...
    @property
    def xwoo(self) -> UVector4: ...
    @property
    def xwol(self) -> UVector4: ...
    @property
    def xwlx(self) -> UVector4: ...
    @property
    def xwly(self) -> UVector4: ...
    @property
    def xwlz(self) -> UVector4: ...
    @property
    def xwlw(self) -> UVector4: ...
    @property
    def xwlo(self) -> UVector4: ...
    @property
    def xwll(self) -> UVector4: ...
    @property
    def xoxx(self) -> UVector4: ...
    @property
    def xoxy(self) -> UVector4: ...
    @property
    def xoxz(self) -> UVector4: ...
    @property
    def xoxw(self) -> UVector4: ...
    @property
    def xoxo(self) -> UVector4: ...
    @property
    def xoxl(self) -> UVector4: ...
    @property
    def xoyx(self) -> UVector4: ...
    @property
    def xoyy(self) -> UVector4: ...
    @property
    def xoyz(self) -> UVector4: ...
    @property
    def xoyw(self) -> UVector4: ...
    @property
    def xoyo(self) -> UVector4: ...
    @property
    def xoyl(self) -> UVector4: ...
    @property
    def xozx(self) -> UVector4: ...
    @property
    def xozy(self) -> UVector4: ...
    @property
    def xozz(self) -> UVector4: ...
    @property
    def xozw(self) -> UVector4: ...
    @property
    def xozo(self) -> UVector4: ...
    @property
    def xozl(self) -> UVector4: ...
    @property
    def xowx(self) -> UVector4: ...
    @property
    def xowy(self) -> UVector4: ...
    @property
    def xowz(self) -> UVector4: ...
    @property
    def xoww(self) -> UVector4: ...
    @property
    def xowo(self) -> UVector4: ...
    @property
    def xowl(self) -> UVector4: ...
    @property
    def xoox(self) -> UVector4: ...
    @property
    def xooy(self) -> UVector4: ...
    @property
    def xooz(self) -> UVector4: ...
    @property
    def xoow(self) -> UVector4: ...
    @property
    def xooo(self) -> UVector4: ...
    @property
    def xool(self) -> UVector4: ...
    @property
    def xolx(self) -> UVector4: ...
    @property
    def xoly(self) -> UVector4: ...
    @property
    def xolz(self) -> UVector4: ...
    @property
    def xolw(self) -> UVector4: ...
    @property
    def xolo(self) -> UVector4: ...
    @property
    def xoll(self) -> UVector4: ...
    @property
    def xlxx(self) -> UVector4: ...
    @property
    def xlxy(self) -> UVector4: ...
    @property
    def xlxz(self) -> UVector4: ...
    @property
    def xlxw(self) -> UVector4: ...
    @property
    def xlxo(self) -> UVector4: ...
    @property
    def xlxl(self) -> UVector4: ...
    @property
    def xlyx(self) -> UVector4: ...
    @property
    def xlyy(self) -> UVector4: ...
    @property
    def xlyz(self) -> UVector4: ...
    @property
    def xlyw(self) -> UVector4: ...
    @property
    def xlyo(self) -> UVector4: ...
    @property
    def xlyl(self) -> UVector4: ...
    @property
    def xlzx(self) -> UVector4: ...
    @property
    def xlzy(self) -> UVector4: ...
    @property
    def xlzz(self) -> UVector4: ...
    @property
    def xlzw(self) -> UVector4: ...
    @property
    def xlzo(self) -> UVector4: ...
    @property
    def xlzl(self) -> UVector4: ...
    @property
    def xlwx(self) -> UVector4: ...
    @property
    def xlwy(self) -> UVector4: ...
    @property
    def xlwz(self) -> UVector4: ...
    @property
    def xlww(self) -> UVector4: ...
    @property
    def xlwo(self) -> UVector4: ...
    @property
    def xlwl(self) -> UVector4: ...
    @property
    def xlox(self) -> UVector4: ...
    @property
    def xloy(self) -> UVector4: ...
    @property
    def xloz(self) -> UVector4: ...
    @property
    def xlow(self) -> UVector4: ...
    @property
    def xloo(self) -> UVector4: ...
    @property
    def xlol(self) -> UVector4: ...
    @property
    def xllx(self) -> UVector4: ...
    @property
    def xlly(self) -> UVector4: ...
    @property
    def xllz(self) -> UVector4: ...
    @property
    def xllw(self) -> UVector4: ...
    @property
    def xllo(self) -> UVector4: ...
    @property
    def xlll(self) -> UVector4: ...
    @property
    def yxxx(self) -> UVector4: ...
    @property
    def yxxy(self) -> UVector4: ...
    @property
    def yxxz(self) -> UVector4: ...
    @property
    def yxxw(self) -> UVector4: ...
    @property
    def yxxo(self) -> UVector4: ...
    @property
    def yxxl(self) -> UVector4: ...
    @property
    def yxyx(self) -> UVector4: ...
    @property
    def yxyy(self) -> UVector4: ...
    @property
    def yxyz(self) -> UVector4: ...
    @property
    def yxyw(self) -> UVector4: ...
    @property
    def yxyo(self) -> UVector4: ...
    @property
    def yxyl(self) -> UVector4: ...
    @property
    def yxzx(self) -> UVector4: ...
    @property
    def yxzy(self) -> UVector4: ...
    @property
    def yxzz(self) -> UVector4: ...
    @property
    def yxzw(self) -> UVector4: ...
    @property
    def yxzo(self) -> UVector4: ...
    @property
    def yxzl(self) -> UVector4: ...
    @property
    def yxwx(self) -> UVector4: ...
    @property
    def yxwy(self) -> UVector4: ...
    @property
    def yxwz(self) -> UVector4: ...
    @property
    def yxww(self) -> UVector4: ...
    @property
    def yxwo(self) -> UVector4: ...
    @property
    def yxwl(self) -> UVector4: ...
    @property
    def yxox(self) -> UVector4: ...
    @property
    def yxoy(self) -> UVector4: ...
    @property
    def yxoz(self) -> UVector4: ...
    @property
    def yxow(self) -> UVector4: ...
    @property
    def yxoo(self) -> UVector4: ...
    @property
    def yxol(self) -> UVector4: ...
    @property
    def yxlx(self) -> UVector4: ...
    @property
    def yxly(self) -> UVector4: ...
    @property
    def yxlz(self) -> UVector4: ...
    @property
    def yxlw(self) -> UVector4: ...
    @property
    def yxlo(self) -> UVector4: ...
    @property
    def yxll(self) -> UVector4: ...
    @property
    def yyxx(self) -> UVector4: ...
    @property
    def yyxy(self) -> UVector4: ...
    @property
    def yyxz(self) -> UVector4: ...
    @property
    def yyxw(self) -> UVector4: ...
    @property
    def yyxo(self) -> UVector4: ...
    @property
    def yyxl(self) -> UVector4: ...
    @property
    def yyyx(self) -> UVector4: ...
    @property
    def yyyy(self) -> UVector4: ...
    @property
    def yyyz(self) -> UVector4: ...
    @property
    def yyyw(self) -> UVector4: ...
    @property
    def yyyo(self) -> UVector4: ...
    @property
    def yyyl(self) -> UVector4: ...
    @property
    def yyzx(self) -> UVector4: ...
    @property
    def yyzy(self) -> UVector4: ...
    @property
    def yyzz(self) -> UVector4: ...
    @property
    def yyzw(self) -> UVector4: ...
    @property
    def yyzo(self) -> UVector4: ...
    @property
    def yyzl(self) -> UVector4: ...
    @property
    def yywx(self) -> UVector4: ...
    @property
    def yywy(self) -> UVector4: ...
    @property
    def yywz(self) -> UVector4: ...
    @property
    def yyww(self) -> UVector4: ...
    @property
    def yywo(self) -> UVector4: ...
    @property
    def yywl(self) -> UVector4: ...
    @property
    def yyox(self) -> UVector4: ...
    @property
    def yyoy(self) -> UVector4: ...
    @property
    def yyoz(self) -> UVector4: ...
    @property
    def yyow(self) -> UVector4: ...
    @property
    def yyoo(self) -> UVector4: ...
    @property
    def yyol(self) -> UVector4: ...
    @property
    def yylx(self) -> UVector4: ...
    @property
    def yyly(self) -> UVector4: ...
    @property
    def yylz(self) -> UVector4: ...
    @property
    def yylw(self) -> UVector4: ...
    @property
    def yylo(self) -> UVector4: ...
    @property
    def yyll(self) -> UVector4: ...
    @property
    def yzxx(self) -> UVector4: ...
    @property
    def yzxy(self) -> UVector4: ...
    @property
    def yzxz(self) -> UVector4: ...
    @property
    def yzxw(self) -> UVector4: ...
    @property
    def yzxo(self) -> UVector4: ...
    @property
    def yzxl(self) -> UVector4: ...
    @property
    def yzyx(self) -> UVector4: ...
    @property
    def yzyy(self) -> UVector4: ...
    @property
    def yzyz(self) -> UVector4: ...
    @property
    def yzyw(self) -> UVector4: ...
    @property
    def yzyo(self) -> UVector4: ...
    @property
    def yzyl(self) -> UVector4: ...
    @property
    def yzzx(self) -> UVector4: ...
    @property
    def yzzy(self) -> UVector4: ...
    @property
    def yzzz(self) -> UVector4: ...
    @property
    def yzzw(self) -> UVector4: ...
    @property
    def yzzo(self) -> UVector4: ...
    @property
    def yzzl(self) -> UVector4: ...
    @property
    def yzwx(self) -> UVector4: ...
    @property
    def yzwy(self) -> UVector4: ...
    @property
    def yzwz(self) -> UVector4: ...
    @property
    def yzww(self) -> UVector4: ...
    @property
    def yzwo(self) -> UVector4: ...
    @property
    def yzwl(self) -> UVector4: ...
    @property
    def yzox(self) -> UVector4: ...
    @property
    def yzoy(self) -> UVector4: ...
    @property
    def yzoz(self) -> UVector4: ...
    @property
    def yzow(self) -> UVector4: ...
    @property
    def yzoo(self) -> UVector4: ...
    @property
    def yzol(self) -> UVector4: ...
    @property
    def yzlx(self) -> UVector4: ...
    @property
    def yzly(self) -> UVector4: ...
    @property
    def yzlz(self) -> UVector4: ...
    @property
    def yzlw(self) -> UVector4: ...
    @property
    def yzlo(self) -> UVector4: ...
    @property
    def yzll(self) -> UVector4: ...
    @property
    def ywxx(self) -> UVector4: ...
    @property
    def ywxy(self) -> UVector4: ...
    @property
    def ywxz(self) -> UVector4: ...
    @property
    def ywxw(self) -> UVector4: ...
    @property
    def ywxo(self) -> UVector4: ...
    @property
    def ywxl(self) -> UVector4: ...
    @property
    def ywyx(self) -> UVector4: ...
    @property
    def ywyy(self) -> UVector4: ...
    @property
    def ywyz(self) -> UVector4: ...
    @property
    def ywyw(self) -> UVector4: ...
    @property
    def ywyo(self) -> UVector4: ...
    @property
    def ywyl(self) -> UVector4: ...
    @property
    def ywzx(self) -> UVector4: ...
    @property
    def ywzy(self) -> UVector4: ...
    @property
    def ywzz(self) -> UVector4: ...
    @property
    def ywzw(self) -> UVector4: ...
    @property
    def ywzo(self) -> UVector4: ...
    @property
    def ywzl(self) -> UVector4: ...
    @property
    def ywwx(self) -> UVector4: ...
    @property
    def ywwy(self) -> UVector4: ...
    @property
    def ywwz(self) -> UVector4: ...
    @property
    def ywww(self) -> UVector4: ...
    @property
    def ywwo(self) -> UVector4: ...
    @property
    def ywwl(self) -> UVector4: ...
    @property
    def ywox(self) -> UVector4: ...
    @property
    def ywoy(self) -> UVector4: ...
    @property
    def ywoz(self) -> UVector4: ...
    @property
    def ywow(self) -> UVector4: ...
    @property
    def ywoo(self) -> UVector4: ...
    @property
    def ywol(self) -> UVector4: ...
    @property
    def ywlx(self) -> UVector4: ...
    @property
    def ywly(self) -> UVector4: ...
    @property
    def ywlz(self) -> UVector4: ...
    @property
    def ywlw(self) -> UVector4: ...
    @property
    def ywlo(self) -> UVector4: ...
    @property
    def ywll(self) -> UVector4: ...
    @property
    def yoxx(self) -> UVector4: ...
    @property
    def yoxy(self) -> UVector4: ...
    @property
    def yoxz(self) -> UVector4: ...
    @property
    def yoxw(self) -> UVector4: ...
    @property
    def yoxo(self) -> UVector4: ...
    @property
    def yoxl(self) -> UVector4: ...
    @property
    def yoyx(self) -> UVector4: ...
    @property
    def yoyy(self) -> UVector4: ...
    @property
    def yoyz(self) -> UVector4: ...
    @property
    def yoyw(self) -> UVector4: ...
    @property
    def yoyo(self) -> UVector4: ...
    @property
    def yoyl(self) -> UVector4: ...
    @property
    def yozx(self) -> UVector4: ...
    @property
    def yozy(self) -> UVector4: ...
    @property
    def yozz(self) -> UVector4: ...
    @property
    def yozw(self) -> UVector4: ...
    @property
    def yozo(self) -> UVector4: ...
    @property
    def yozl(self) -> UVector4: ...
    @property
    def yowx(self) -> UVector4: ...
    @property
    def yowy(self) -> UVector4: ...
    @property
    def yowz(self) -> UVector4: ...
    @property
    def yoww(self) -> UVector4: ...
    @property
    def yowo(self) -> UVector4: ...
    @property
    def yowl(self) -> UVector4: ...
    @property
    def yoox(self) -> UVector4: ...
    @property
    def yooy(self) -> UVector4: ...
    @property
    def yooz(self) -> UVector4: ...
    @property
    def yoow(self) -> UVector4: ...
    @property
    def yooo(self) -> UVector4: ...
    @property
    def yool(self) -> UVector4: ...
    @property
    def yolx(self) -> UVector4: ...
    @property
    def yoly(self) -> UVector4: ...
    @property
    def yolz(self) -> UVector4: ...
    @property
    def yolw(self) -> UVector4: ...
    @property
    def yolo(self) -> UVector4: ...
    @property
    def yoll(self) -> UVector4: ...
    @property
    def ylxx(self) -> UVector4: ...
    @property
    def ylxy(self) -> UVector4: ...
    @property
    def ylxz(self) -> UVector4: ...
    @property
    def ylxw(self) -> UVector4: ...
    @property
    def ylxo(self) -> UVector4: ...
    @property
    def ylxl(self) -> UVector4: ...
    @property
    def ylyx(self) -> UVector4: ...
    @property
    def ylyy(self) -> UVector4: ...
    @property
    def ylyz(self) -> UVector4: ...
    @property
    def ylyw(self) -> UVector4: ...
    @property
    def ylyo(self) -> UVector4: ...
    @property
    def ylyl(self) -> UVector4: ...
    @property
    def ylzx(self) -> UVector4: ...
    @property
    def ylzy(self) -> UVector4: ...
    @property
    def ylzz(self) -> UVector4: ...
    @property
    def ylzw(self) -> UVector4: ...
    @property
    def ylzo(self) -> UVector4: ...
    @property
    def ylzl(self) -> UVector4: ...
    @property
    def ylwx(self) -> UVector4: ...
    @property
    def ylwy(self) -> UVector4: ...
    @property
    def ylwz(self) -> UVector4: ...
    @property
    def ylww(self) -> UVector4: ...
    @property
    def ylwo(self) -> UVector4: ...
    @property
    def ylwl(self) -> UVector4: ...
    @property
    def ylox(self) -> UVector4: ...
    @property
    def yloy(self) -> UVector4: ...
    @property
    def yloz(self) -> UVector4: ...
    @property
    def ylow(self) -> UVector4: ...
    @property
    def yloo(self) -> UVector4: ...
    @property
    def ylol(self) -> UVector4: ...
    @property
    def yllx(self) -> UVector4: ...
    @property
    def ylly(self) -> UVector4: ...
    @property
    def yllz(self) -> UVector4: ...
    @property
    def yllw(self) -> UVector4: ...
    @property
    def yllo(self) -> UVector4: ...
    @property
    def ylll(self) -> UVector4: ...
    @property
    def zxxx(self) -> UVector4: ...
    @property
    def zxxy(self) -> UVector4: ...
    @property
    def zxxz(self) -> UVector4: ...
    @property
    def zxxw(self) -> UVector4: ...
    @property
    def zxxo(self) -> UVector4: ...
    @property
    def zxxl(self) -> UVector4: ...
    @property
    def zxyx(self) -> UVector4: ...
    @property
    def zxyy(self) -> UVector4: ...
    @property
    def zxyz(self) -> UVector4: ...
    @property
    def zxyw(self) -> UVector4: ...
    @property
    def zxyo(self) -> UVector4: ...
    @property
    def zxyl(self) -> UVector4: ...
    @property
    def zxzx(self) -> UVector4: ...
    @property
    def zxzy(self) -> UVector4: ...
    @property
    def zxzz(self) -> UVector4: ...
    @property
    def zxzw(self) -> UVector4: ...
    @property
    def zxzo(self) -> UVector4: ...
    @property
    def zxzl(self) -> UVector4: ...
    @property
    def zxwx(self) -> UVector4: ...
    @property
    def zxwy(self) -> UVector4: ...
    @property
    def zxwz(self) -> UVector4: ...
    @property
    def zxww(self) -> UVector4: ...
    @property
    def zxwo(self) -> UVector4: ...
    @property
    def zxwl(self) -> UVector4: ...
    @property
    def zxox(self) -> UVector4: ...
    @property
    def zxoy(self) -> UVector4: ...
    @property
    def zxoz(self) -> UVector4: ...
    @property
    def zxow(self) -> UVector4: ...
    @property
    def zxoo(self) -> UVector4: ...
    @property
    def zxol(self) -> UVector4: ...
    @property
    def zxlx(self) -> UVector4: ...
    @property
    def zxly(self) -> UVector4: ...
    @property
    def zxlz(self) -> UVector4: ...
    @property
    def zxlw(self) -> UVector4: ...
    @property
    def zxlo(self) -> UVector4: ...
    @property
    def zxll(self) -> UVector4: ...
    @property
    def zyxx(self) -> UVector4: ...
    @property
    def zyxy(self) -> UVector4: ...
    @property
    def zyxz(self) -> UVector4: ...
    @property
    def zyxw(self) -> UVector4: ...
    @property
    def zyxo(self) -> UVector4: ...
    @property
    def zyxl(self) -> UVector4: ...
    @property
    def zyyx(self) -> UVector4: ...
    @property
    def zyyy(self) -> UVector4: ...
    @property
    def zyyz(self) -> UVector4: ...
    @property
    def zyyw(self) -> UVector4: ...
    @property
    def zyyo(self) -> UVector4: ...
    @property
    def zyyl(self) -> UVector4: ...
    @property
    def zyzx(self) -> UVector4: ...
    @property
    def zyzy(self) -> UVector4: ...
    @property
    def zyzz(self) -> UVector4: ...
    @property
    def zyzw(self) -> UVector4: ...
    @property
    def zyzo(self) -> UVector4: ...
    @property
    def zyzl(self) -> UVector4: ...
    @property
    def zywx(self) -> UVector4: ...
    @property
    def zywy(self) -> UVector4: ...
    @property
    def zywz(self) -> UVector4: ...
    @property
    def zyww(self) -> UVector4: ...
    @property
    def zywo(self) -> UVector4: ...
    @property
    def zywl(self) -> UVector4: ...
    @property
    def zyox(self) -> UVector4: ...
    @property
    def zyoy(self) -> UVector4: ...
    @property
    def zyoz(self) -> UVector4: ...
    @property
    def zyow(self) -> UVector4: ...
    @property
    def zyoo(self) -> UVector4: ...
    @property
    def zyol(self) -> UVector4: ...
    @property
    def zylx(self) -> UVector4: ...
    @property
    def zyly(self) -> UVector4: ...
    @property
    def zylz(self) -> UVector4: ...
    @property
    def zylw(self) -> UVector4: ...
    @property
    def zylo(self) -> UVector4: ...
    @property
    def zyll(self) -> UVector4: ...
    @property
    def zzxx(self) -> UVector4: ...
    @property
    def zzxy(self) -> UVector4: ...
    @property
    def zzxz(self) -> UVector4: ...
    @property
    def zzxw(self) -> UVector4: ...
    @property
    def zzxo(self) -> UVector4: ...
    @property
    def zzxl(self) -> UVector4: ...
    @property
    def zzyx(self) -> UVector4: ...
    @property
    def zzyy(self) -> UVector4: ...
    @property
    def zzyz(self) -> UVector4: ...
    @property
    def zzyw(self) -> UVector4: ...
    @property
    def zzyo(self) -> UVector4: ...
    @property
    def zzyl(self) -> UVector4: ...
    @property
    def zzzx(self) -> UVector4: ...
    @property
    def zzzy(self) -> UVector4: ...
    @property
    def zzzz(self) -> UVector4: ...
    @property
    def zzzw(self) -> UVector4: ...
    @property
    def zzzo(self) -> UVector4: ...
    @property
    def zzzl(self) -> UVector4: ...
    @property
    def zzwx(self) -> UVector4: ...
    @property
    def zzwy(self) -> UVector4: ...
    @property
    def zzwz(self) -> UVector4: ...
    @property
    def zzww(self) -> UVector4: ...
    @property
    def zzwo(self) -> UVector4: ...
    @property
    def zzwl(self) -> UVector4: ...
    @property
    def zzox(self) -> UVector4: ...
    @property
    def zzoy(self) -> UVector4: ...
    @property
    def zzoz(self) -> UVector4: ...
    @property
    def zzow(self) -> UVector4: ...
    @property
    def zzoo(self) -> UVector4: ...
    @property
    def zzol(self) -> UVector4: ...
    @property
    def zzlx(self) -> UVector4: ...
    @property
    def zzly(self) -> UVector4: ...
    @property
    def zzlz(self) -> UVector4: ...
    @property
    def zzlw(self) -> UVector4: ...
    @property
    def zzlo(self) -> UVector4: ...
    @property
    def zzll(self) -> UVector4: ...
    @property
    def zwxx(self) -> UVector4: ...
    @property
    def zwxy(self) -> UVector4: ...
    @property
    def zwxz(self) -> UVector4: ...
    @property
    def zwxw(self) -> UVector4: ...
    @property
    def zwxo(self) -> UVector4: ...
    @property
    def zwxl(self) -> UVector4: ...
    @property
    def zwyx(self) -> UVector4: ...
    @property
    def zwyy(self) -> UVector4: ...
    @property
    def zwyz(self) -> UVector4: ...
    @property
    def zwyw(self) -> UVector4: ...
    @property
    def zwyo(self) -> UVector4: ...
    @property
    def zwyl(self) -> UVector4: ...
    @property
    def zwzx(self) -> UVector4: ...
    @property
    def zwzy(self) -> UVector4: ...
    @property
    def zwzz(self) -> UVector4: ...
    @property
    def zwzw(self) -> UVector4: ...
    @property
    def zwzo(self) -> UVector4: ...
    @property
    def zwzl(self) -> UVector4: ...
    @property
    def zwwx(self) -> UVector4: ...
    @property
    def zwwy(self) -> UVector4: ...
    @property
    def zwwz(self) -> UVector4: ...
    @property
    def zwww(self) -> UVector4: ...
    @property
    def zwwo(self) -> UVector4: ...
    @property
    def zwwl(self) -> UVector4: ...
    @property
    def zwox(self) -> UVector4: ...
    @property
    def zwoy(self) -> UVector4: ...
    @property
    def zwoz(self) -> UVector4: ...
    @property
    def zwow(self) -> UVector4: ...
    @property
    def zwoo(self) -> UVector4: ...
    @property
    def zwol(self) -> UVector4: ...
    @property
    def zwlx(self) -> UVector4: ...
    @property
    def zwly(self) -> UVector4: ...
    @property
    def zwlz(self) -> UVector4: ...
    @property
    def zwlw(self) -> UVector4: ...
    @property
    def zwlo(self) -> UVector4: ...
    @property
    def zwll(self) -> UVector4: ...
    @property
    def zoxx(self) -> UVector4: ...
    @property
    def zoxy(self) -> UVector4: ...
    @property
    def zoxz(self) -> UVector4: ...
    @property
    def zoxw(self) -> UVector4: ...
    @property
    def zoxo(self) -> UVector4: ...
    @property
    def zoxl(self) -> UVector4: ...
    @property
    def zoyx(self) -> UVector4: ...
    @property
    def zoyy(self) -> UVector4: ...
    @property
    def zoyz(self) -> UVector4: ...
    @property
    def zoyw(self) -> UVector4: ...
    @property
    def zoyo(self) -> UVector4: ...
    @property
    def zoyl(self) -> UVector4: ...
    @property
    def zozx(self) -> UVector4: ...
    @property
    def zozy(self) -> UVector4: ...
    @property
    def zozz(self) -> UVector4: ...
    @property
    def zozw(self) -> UVector4: ...
    @property
    def zozo(self) -> UVector4: ...
    @property
    def zozl(self) -> UVector4: ...
    @property
    def zowx(self) -> UVector4: ...
    @property
    def zowy(self) -> UVector4: ...
    @property
    def zowz(self) -> UVector4: ...
    @property
    def zoww(self) -> UVector4: ...
    @property
    def zowo(self) -> UVector4: ...
    @property
    def zowl(self) -> UVector4: ...
    @property
    def zoox(self) -> UVector4: ...
    @property
    def zooy(self) -> UVector4: ...
    @property
    def zooz(self) -> UVector4: ...
    @property
    def zoow(self) -> UVector4: ...
    @property
    def zooo(self) -> UVector4: ...
    @property
    def zool(self) -> UVector4: ...
    @property
    def zolx(self) -> UVector4: ...
    @property
    def zoly(self) -> UVector4: ...
    @property
    def zolz(self) -> UVector4: ...
    @property
    def zolw(self) -> UVector4: ...
    @property
    def zolo(self) -> UVector4: ...
    @property
    def zoll(self) -> UVector4: ...
    @property
    def zlxx(self) -> UVector4: ...
    @property
    def zlxy(self) -> UVector4: ...
    @property
    def zlxz(self) -> UVector4: ...
    @property
    def zlxw(self) -> UVector4: ...
    @property
    def zlxo(self) -> UVector4: ...
    @property
    def zlxl(self) -> UVector4: ...
    @property
    def zlyx(self) -> UVector4: ...
    @property
    def zlyy(self) -> UVector4: ...
    @property
    def zlyz(self) -> UVector4: ...
    @property
    def zlyw(self) -> UVector4: ...
    @property
    def zlyo(self) -> UVector4: ...
    @property
    def zlyl(self) -> UVector4: ...
    @property
    def zlzx(self) -> UVector4: ...
    @property
    def zlzy(self) -> UVector4: ...
    @property
    def zlzz(self) -> UVector4: ...
    @property
    def zlzw(self) -> UVector4: ...
    @property
    def zlzo(self) -> UVector4: ...
    @property
    def zlzl(self) -> UVector4: ...
    @property
    def zlwx(self) -> UVector4: ...
    @property
    def zlwy(self) -> UVector4: ...
    @property
    def zlwz(self) -> UVector4: ...
    @property
    def zlww(self) -> UVector4: ...
    @property
    def zlwo(self) -> UVector4: ...
    @property
    def zlwl(self) -> UVector4: ...
    @property
    def zlox(self) -> UVector4: ...
    @property
    def zloy(self) -> UVector4: ...
    @property
    def zloz(self) -> UVector4: ...
    @property
    def zlow(self) -> UVector4: ...
    @property
    def zloo(self) -> UVector4: ...
    @property
    def zlol(self) -> UVector4: ...
    @property
    def zllx(self) -> UVector4: ...
    @property
    def zlly(self) -> UVector4: ...
    @property
    def zllz(self) -> UVector4: ...
    @property
    def zllw(self) -> UVector4: ...
    @property
    def zllo(self) -> UVector4: ...
    @property
    def zlll(self) -> UVector4: ...
    @property
    def wxxx(self) -> UVector4: ...
    @property
    def wxxy(self) -> UVector4: ...
    @property
    def wxxz(self) -> UVector4: ...
    @property
    def wxxw(self) -> UVector4: ...
    @property
    def wxxo(self) -> UVector4: ...
    @property
    def wxxl(self) -> UVector4: ...
    @property
    def wxyx(self) -> UVector4: ...
    @property
    def wxyy(self) -> UVector4: ...
    @property
    def wxyz(self) -> UVector4: ...
    @property
    def wxyw(self) -> UVector4: ...
    @property
    def wxyo(self) -> UVector4: ...
    @property
    def wxyl(self) -> UVector4: ...
    @property
    def wxzx(self) -> UVector4: ...
    @property
    def wxzy(self) -> UVector4: ...
    @property
    def wxzz(self) -> UVector4: ...
    @property
    def wxzw(self) -> UVector4: ...
    @property
    def wxzo(self) -> UVector4: ...
    @property
    def wxzl(self) -> UVector4: ...
    @property
    def wxwx(self) -> UVector4: ...
    @property
    def wxwy(self) -> UVector4: ...
    @property
    def wxwz(self) -> UVector4: ...
    @property
    def wxww(self) -> UVector4: ...
    @property
    def wxwo(self) -> UVector4: ...
    @property
    def wxwl(self) -> UVector4: ...
    @property
    def wxox(self) -> UVector4: ...
    @property
    def wxoy(self) -> UVector4: ...
    @property
    def wxoz(self) -> UVector4: ...
    @property
    def wxow(self) -> UVector4: ...
    @property
    def wxoo(self) -> UVector4: ...
    @property
    def wxol(self) -> UVector4: ...
    @property
    def wxlx(self) -> UVector4: ...
    @property
    def wxly(self) -> UVector4: ...
    @property
    def wxlz(self) -> UVector4: ...
    @property
    def wxlw(self) -> UVector4: ...
    @property
    def wxlo(self) -> UVector4: ...
    @property
    def wxll(self) -> UVector4: ...
    @property
    def wyxx(self) -> UVector4: ...
    @property
    def wyxy(self) -> UVector4: ...
    @property
    def wyxz(self) -> UVector4: ...
    @property
    def wyxw(self) -> UVector4: ...
    @property
    def wyxo(self) -> UVector4: ...
    @property
    def wyxl(self) -> UVector4: ...
    @property
    def wyyx(self) -> UVector4: ...
    @property
    def wyyy(self) -> UVector4: ...
    @property
    def wyyz(self) -> UVector4: ...
    @property
    def wyyw(self) -> UVector4: ...
    @property
    def wyyo(self) -> UVector4: ...
    @property
    def wyyl(self) -> UVector4: ...
    @property
    def wyzx(self) -> UVector4: ...
    @property
    def wyzy(self) -> UVector4: ...
    @property
    def wyzz(self) -> UVector4: ...
    @property
    def wyzw(self) -> UVector4: ...
    @property
    def wyzo(self) -> UVector4: ...
    @property
    def wyzl(self) -> UVector4: ...
    @property
    def wywx(self) -> UVector4: ...
    @property
    def wywy(self) -> UVector4: ...
    @property
    def wywz(self) -> UVector4: ...
    @property
    def wyww(self) -> UVector4: ...
    @property
    def wywo(self) -> UVector4: ...
    @property
    def wywl(self) -> UVector4: ...
    @property
    def wyox(self) -> UVector4: ...
    @property
    def wyoy(self) -> UVector4: ...
    @property
    def wyoz(self) -> UVector4: ...
    @property
    def wyow(self) -> UVector4: ...
    @property
    def wyoo(self) -> UVector4: ...
    @property
    def wyol(self) -> UVector4: ...
    @property
    def wylx(self) -> UVector4: ...
    @property
    def wyly(self) -> UVector4: ...
    @property
    def wylz(self) -> UVector4: ...
    @property
    def wylw(self) -> UVector4: ...
    @property
    def wylo(self) -> UVector4: ...
    @property
    def wyll(self) -> UVector4: ...
    @property
    def wzxx(self) -> UVector4: ...
    @property
    def wzxy(self) -> UVector4: ...
    @property
    def wzxz(self) -> UVector4: ...
    @property
    def wzxw(self) -> UVector4: ...
    @property
    def wzxo(self) -> UVector4: ...
    @property
    def wzxl(self) -> UVector4: ...
    @property
    def wzyx(self) -> UVector4: ...
    @property
    def wzyy(self) -> UVector4: ...
    @property
    def wzyz(self) -> UVector4: ...
    @property
    def wzyw(self) -> UVector4: ...
    @property
    def wzyo(self) -> UVector4: ...
    @property
    def wzyl(self) -> UVector4: ...
    @property
    def wzzx(self) -> UVector4: ...
    @property
    def wzzy(self) -> UVector4: ...
    @property
    def wzzz(self) -> UVector4: ...
    @property
    def wzzw(self) -> UVector4: ...
    @property
    def wzzo(self) -> UVector4: ...
    @property
    def wzzl(self) -> UVector4: ...
    @property
    def wzwx(self) -> UVector4: ...
    @property
    def wzwy(self) -> UVector4: ...
    @property
    def wzwz(self) -> UVector4: ...
    @property
    def wzww(self) -> UVector4: ...
    @property
    def wzwo(self) -> UVector4: ...
    @property
    def wzwl(self) -> UVector4: ...
    @property
    def wzox(self) -> UVector4: ...
    @property
    def wzoy(self) -> UVector4: ...
    @property
    def wzoz(self) -> UVector4: ...
    @property
    def wzow(self) -> UVector4: ...
    @property
    def wzoo(self) -> UVector4: ...
    @property
    def wzol(self) -> UVector4: ...
    @property
    def wzlx(self) -> UVector4: ...
    @property
    def wzly(self) -> UVector4: ...
    @property
    def wzlz(self) -> UVector4: ...
    @property
    def wzlw(self) -> UVector4: ...
    @property
    def wzlo(self) -> UVector4: ...
    @property
    def wzll(self) -> UVector4: ...
    @property
    def wwxx(self) -> UVector4: ...
    @property
    def wwxy(self) -> UVector4: ...
    @property
    def wwxz(self) -> UVector4: ...
    @property
    def wwxw(self) -> UVector4: ...
    @property
    def wwxo(self) -> UVector4: ...
    @property
    def wwxl(self) -> UVector4: ...
    @property
    def wwyx(self) -> UVector4: ...
    @property
    def wwyy(self) -> UVector4: ...
    @property
    def wwyz(self) -> UVector4: ...
    @property
    def wwyw(self) -> UVector4: ...
    @property
    def wwyo(self) -> UVector4: ...
    @property
    def wwyl(self) -> UVector4: ...
    @property
    def wwzx(self) -> UVector4: ...
    @property
    def wwzy(self) -> UVector4: ...
    @property
    def wwzz(self) -> UVector4: ...
    @property
    def wwzw(self) -> UVector4: ...
    @property
    def wwzo(self) -> UVector4: ...
    @property
    def wwzl(self) -> UVector4: ...
    @property
    def wwwx(self) -> UVector4: ...
    @property
    def wwwy(self) -> UVector4: ...
    @property
    def wwwz(self) -> UVector4: ...
    @property
    def wwww(self) -> UVector4: ...
    @property
    def wwwo(self) -> UVector4: ...
    @property
    def wwwl(self) -> UVector4: ...
    @property
    def wwox(self) -> UVector4: ...
    @property
    def wwoy(self) -> UVector4: ...
    @property
    def wwoz(self) -> UVector4: ...
    @property
    def wwow(self) -> UVector4: ...
    @property
    def wwoo(self) -> UVector4: ...
    @property
    def wwol(self) -> UVector4: ...
    @property
    def wwlx(self) -> UVector4: ...
    @property
    def wwly(self) -> UVector4: ...
    @property
    def wwlz(self) -> UVector4: ...
    @property
    def wwlw(self) -> UVector4: ...
    @property
    def wwlo(self) -> UVector4: ...
    @property
    def wwll(self) -> UVector4: ...
    @property
    def woxx(self) -> UVector4: ...
    @property
    def woxy(self) -> UVector4: ...
    @property
    def woxz(self) -> UVector4: ...
    @property
    def woxw(self) -> UVector4: ...
    @property
    def woxo(self) -> UVector4: ...
    @property
    def woxl(self) -> UVector4: ...
    @property
    def woyx(self) -> UVector4: ...
    @property
    def woyy(self) -> UVector4: ...
    @property
    def woyz(self) -> UVector4: ...
    @property
    def woyw(self) -> UVector4: ...
    @property
    def woyo(self) -> UVector4: ...
    @property
    def woyl(self) -> UVector4: ...
    @property
    def wozx(self) -> UVector4: ...
    @property
    def wozy(self) -> UVector4: ...
    @property
    def wozz(self) -> UVector4: ...
    @property
    def wozw(self) -> UVector4: ...
    @property
    def wozo(self) -> UVector4: ...
    @property
    def wozl(self) -> UVector4: ...
    @property
    def wowx(self) -> UVector4: ...
    @property
    def wowy(self) -> UVector4: ...
    @property
    def wowz(self) -> UVector4: ...
    @property
    def woww(self) -> UVector4: ...
    @property
    def wowo(self) -> UVector4: ...
    @property
    def wowl(self) -> UVector4: ...
    @property
    def woox(self) -> UVector4: ...
    @property
    def wooy(self) -> UVector4: ...
    @property
    def wooz(self) -> UVector4: ...
    @property
    def woow(self) -> UVector4: ...
    @property
    def wooo(self) -> UVector4: ...
    @property
    def wool(self) -> UVector4: ...
    @property
    def wolx(self) -> UVector4: ...
    @property
    def woly(self) -> UVector4: ...
    @property
    def wolz(self) -> UVector4: ...
    @property
    def wolw(self) -> UVector4: ...
    @property
    def wolo(self) -> UVector4: ...
    @property
    def woll(self) -> UVector4: ...
    @property
    def wlxx(self) -> UVector4: ...
    @property
    def wlxy(self) -> UVector4: ...
    @property
    def wlxz(self) -> UVector4: ...
    @property
    def wlxw(self) -> UVector4: ...
    @property
    def wlxo(self) -> UVector4: ...
    @property
    def wlxl(self) -> UVector4: ...
    @property
    def wlyx(self) -> UVector4: ...
    @property
    def wlyy(self) -> UVector4: ...
    @property
    def wlyz(self) -> UVector4: ...
    @property
    def wlyw(self) -> UVector4: ...
    @property
    def wlyo(self) -> UVector4: ...
    @property
    def wlyl(self) -> UVector4: ...
    @property
    def wlzx(self) -> UVector4: ...
    @property
    def wlzy(self) -> UVector4: ...
    @property
    def wlzz(self) -> UVector4: ...
    @property
    def wlzw(self) -> UVector4: ...
    @property
    def wlzo(self) -> UVector4: ...
    @property
    def wlzl(self) -> UVector4: ...
    @property
    def wlwx(self) -> UVector4: ...
    @property
    def wlwy(self) -> UVector4: ...
    @property
    def wlwz(self) -> UVector4: ...
    @property
    def wlww(self) -> UVector4: ...
    @property
    def wlwo(self) -> UVector4: ...
    @property
    def wlwl(self) -> UVector4: ...
    @property
    def wlox(self) -> UVector4: ...
    @property
    def wloy(self) -> UVector4: ...
    @property
    def wloz(self) -> UVector4: ...
    @property
    def wlow(self) -> UVector4: ...
    @property
    def wloo(self) -> UVector4: ...
    @property
    def wlol(self) -> UVector4: ...
    @property
    def wllx(self) -> UVector4: ...
    @property
    def wlly(self) -> UVector4: ...
    @property
    def wllz(self) -> UVector4: ...
    @property
    def wllw(self) -> UVector4: ...
    @property
    def wllo(self) -> UVector4: ...
    @property
    def wlll(self) -> UVector4: ...
    @property
    def oxxx(self) -> UVector4: ...
    @property
    def oxxy(self) -> UVector4: ...
    @property
    def oxxz(self) -> UVector4: ...
    @property
    def oxxw(self) -> UVector4: ...
    @property
    def oxxo(self) -> UVector4: ...
    @property
    def oxxl(self) -> UVector4: ...
    @property
    def oxyx(self) -> UVector4: ...
    @property
    def oxyy(self) -> UVector4: ...
    @property
    def oxyz(self) -> UVector4: ...
    @property
    def oxyw(self) -> UVector4: ...
    @property
    def oxyo(self) -> UVector4: ...
    @property
    def oxyl(self) -> UVector4: ...
    @property
    def oxzx(self) -> UVector4: ...
    @property
    def oxzy(self) -> UVector4: ...
    @property
    def oxzz(self) -> UVector4: ...
    @property
    def oxzw(self) -> UVector4: ...
    @property
    def oxzo(self) -> UVector4: ...
    @property
    def oxzl(self) -> UVector4: ...
    @property
    def oxwx(self) -> UVector4: ...
    @property
    def oxwy(self) -> UVector4: ...
    @property
    def oxwz(self) -> UVector4: ...
    @property
    def oxww(self) -> UVector4: ...
    @property
    def oxwo(self) -> UVector4: ...
    @property
    def oxwl(self) -> UVector4: ...
    @property
    def oxox(self) -> UVector4: ...
    @property
    def oxoy(self) -> UVector4: ...
    @property
    def oxoz(self) -> UVector4: ...
    @property
    def oxow(self) -> UVector4: ...
    @property
    def oxoo(self) -> UVector4: ...
    @property
    def oxol(self) -> UVector4: ...
    @property
    def oxlx(self) -> UVector4: ...
    @property
    def oxly(self) -> UVector4: ...
    @property
    def oxlz(self) -> UVector4: ...
    @property
    def oxlw(self) -> UVector4: ...
    @property
    def oxlo(self) -> UVector4: ...
    @property
    def oxll(self) -> UVector4: ...
    @property
    def oyxx(self) -> UVector4: ...
    @property
    def oyxy(self) -> UVector4: ...
    @property
    def oyxz(self) -> UVector4: ...
    @property
    def oyxw(self) -> UVector4: ...
    @property
    def oyxo(self) -> UVector4: ...
    @property
    def oyxl(self) -> UVector4: ...
    @property
    def oyyx(self) -> UVector4: ...
    @property
    def oyyy(self) -> UVector4: ...
    @property
    def oyyz(self) -> UVector4: ...
    @property
    def oyyw(self) -> UVector4: ...
    @property
    def oyyo(self) -> UVector4: ...
    @property
    def oyyl(self) -> UVector4: ...
    @property
    def oyzx(self) -> UVector4: ...
    @property
    def oyzy(self) -> UVector4: ...
    @property
    def oyzz(self) -> UVector4: ...
    @property
    def oyzw(self) -> UVector4: ...
    @property
    def oyzo(self) -> UVector4: ...
    @property
    def oyzl(self) -> UVector4: ...
    @property
    def oywx(self) -> UVector4: ...
    @property
    def oywy(self) -> UVector4: ...
    @property
    def oywz(self) -> UVector4: ...
    @property
    def oyww(self) -> UVector4: ...
    @property
    def oywo(self) -> UVector4: ...
    @property
    def oywl(self) -> UVector4: ...
    @property
    def oyox(self) -> UVector4: ...
    @property
    def oyoy(self) -> UVector4: ...
    @property
    def oyoz(self) -> UVector4: ...
    @property
    def oyow(self) -> UVector4: ...
    @property
    def oyoo(self) -> UVector4: ...
    @property
    def oyol(self) -> UVector4: ...
    @property
    def oylx(self) -> UVector4: ...
    @property
    def oyly(self) -> UVector4: ...
    @property
    def oylz(self) -> UVector4: ...
    @property
    def oylw(self) -> UVector4: ...
    @property
    def oylo(self) -> UVector4: ...
    @property
    def oyll(self) -> UVector4: ...
    @property
    def ozxx(self) -> UVector4: ...
    @property
    def ozxy(self) -> UVector4: ...
    @property
    def ozxz(self) -> UVector4: ...
    @property
    def ozxw(self) -> UVector4: ...
    @property
    def ozxo(self) -> UVector4: ...
    @property
    def ozxl(self) -> UVector4: ...
    @property
    def ozyx(self) -> UVector4: ...
    @property
    def ozyy(self) -> UVector4: ...
    @property
    def ozyz(self) -> UVector4: ...
    @property
    def ozyw(self) -> UVector4: ...
    @property
    def ozyo(self) -> UVector4: ...
    @property
    def ozyl(self) -> UVector4: ...
    @property
    def ozzx(self) -> UVector4: ...
    @property
    def ozzy(self) -> UVector4: ...
    @property
    def ozzz(self) -> UVector4: ...
    @property
    def ozzw(self) -> UVector4: ...
    @property
    def ozzo(self) -> UVector4: ...
    @property
    def ozzl(self) -> UVector4: ...
    @property
    def ozwx(self) -> UVector4: ...
    @property
    def ozwy(self) -> UVector4: ...
    @property
    def ozwz(self) -> UVector4: ...
    @property
    def ozww(self) -> UVector4: ...
    @property
    def ozwo(self) -> UVector4: ...
    @property
    def ozwl(self) -> UVector4: ...
    @property
    def ozox(self) -> UVector4: ...
    @property
    def ozoy(self) -> UVector4: ...
    @property
    def ozoz(self) -> UVector4: ...
    @property
    def ozow(self) -> UVector4: ...
    @property
    def ozoo(self) -> UVector4: ...
    @property
    def ozol(self) -> UVector4: ...
    @property
    def ozlx(self) -> UVector4: ...
    @property
    def ozly(self) -> UVector4: ...
    @property
    def ozlz(self) -> UVector4: ...
    @property
    def ozlw(self) -> UVector4: ...
    @property
    def ozlo(self) -> UVector4: ...
    @property
    def ozll(self) -> UVector4: ...
    @property
    def owxx(self) -> UVector4: ...
    @property
    def owxy(self) -> UVector4: ...
    @property
    def owxz(self) -> UVector4: ...
    @property
    def owxw(self) -> UVector4: ...
    @property
    def owxo(self) -> UVector4: ...
    @property
    def owxl(self) -> UVector4: ...
    @property
    def owyx(self) -> UVector4: ...
    @property
    def owyy(self) -> UVector4: ...
    @property
    def owyz(self) -> UVector4: ...
    @property
    def owyw(self) -> UVector4: ...
    @property
    def owyo(self) -> UVector4: ...
    @property
    def owyl(self) -> UVector4: ...
    @property
    def owzx(self) -> UVector4: ...
    @property
    def owzy(self) -> UVector4: ...
    @property
    def owzz(self) -> UVector4: ...
    @property
    def owzw(self) -> UVector4: ...
    @property
    def owzo(self) -> UVector4: ...
    @property
    def owzl(self) -> UVector4: ...
    @property
    def owwx(self) -> UVector4: ...
    @property
    def owwy(self) -> UVector4: ...
    @property
    def owwz(self) -> UVector4: ...
    @property
    def owww(self) -> UVector4: ...
    @property
    def owwo(self) -> UVector4: ...
    @property
    def owwl(self) -> UVector4: ...
    @property
    def owox(self) -> UVector4: ...
    @property
    def owoy(self) -> UVector4: ...
    @property
    def owoz(self) -> UVector4: ...
    @property
    def owow(self) -> UVector4: ...
    @property
    def owoo(self) -> UVector4: ...
    @property
    def owol(self) -> UVector4: ...
    @property
    def owlx(self) -> UVector4: ...
    @property
    def owly(self) -> UVector4: ...
    @property
    def owlz(self) -> UVector4: ...
    @property
    def owlw(self) -> UVector4: ...
    @property
    def owlo(self) -> UVector4: ...
    @property
    def owll(self) -> UVector4: ...
    @property
    def ooxx(self) -> UVector4: ...
    @property
    def ooxy(self) -> UVector4: ...
    @property
    def ooxz(self) -> UVector4: ...
    @property
    def ooxw(self) -> UVector4: ...
    @property
    def ooxo(self) -> UVector4: ...
    @property
    def ooxl(self) -> UVector4: ...
    @property
    def ooyx(self) -> UVector4: ...
    @property
    def ooyy(self) -> UVector4: ...
    @property
    def ooyz(self) -> UVector4: ...
    @property
    def ooyw(self) -> UVector4: ...
    @property
    def ooyo(self) -> UVector4: ...
    @property
    def ooyl(self) -> UVector4: ...
    @property
    def oozx(self) -> UVector4: ...
    @property
    def oozy(self) -> UVector4: ...
    @property
    def oozz(self) -> UVector4: ...
    @property
    def oozw(self) -> UVector4: ...
    @property
    def oozo(self) -> UVector4: ...
    @property
    def oozl(self) -> UVector4: ...
    @property
    def oowx(self) -> UVector4: ...
    @property
    def oowy(self) -> UVector4: ...
    @property
    def oowz(self) -> UVector4: ...
    @property
    def ooww(self) -> UVector4: ...
    @property
    def oowo(self) -> UVector4: ...
    @property
    def oowl(self) -> UVector4: ...
    @property
    def ooox(self) -> UVector4: ...
    @property
    def oooy(self) -> UVector4: ...
    @property
    def oooz(self) -> UVector4: ...
    @property
    def ooow(self) -> UVector4: ...
    @property
    def oolx(self) -> UVector4: ...
    @property
    def ooly(self) -> UVector4: ...
    @property
    def oolz(self) -> UVector4: ...
    @property
    def oolw(self) -> UVector4: ...
    @property
    def olxx(self) -> UVector4: ...
    @property
    def olxy(self) -> UVector4: ...
    @property
    def olxz(self) -> UVector4: ...
    @property
    def olxw(self) -> UVector4: ...
    @property
    def olxo(self) -> UVector4: ...
    @property
    def olxl(self) -> UVector4: ...
    @property
    def olyx(self) -> UVector4: ...
    @property
    def olyy(self) -> UVector4: ...
    @property
    def olyz(self) -> UVector4: ...
    @property
    def olyw(self) -> UVector4: ...
    @property
    def olyo(self) -> UVector4: ...
    @property
    def olyl(self) -> UVector4: ...
    @property
    def olzx(self) -> UVector4: ...
    @property
    def olzy(self) -> UVector4: ...
    @property
    def olzz(self) -> UVector4: ...
    @property
    def olzw(self) -> UVector4: ...
    @property
    def olzo(self) -> UVector4: ...
    @property
    def olzl(self) -> UVector4: ...
    @property
    def olwx(self) -> UVector4: ...
    @property
    def olwy(self) -> UVector4: ...
    @property
    def olwz(self) -> UVector4: ...
    @property
    def olww(self) -> UVector4: ...
    @property
    def olwo(self) -> UVector4: ...
    @property
    def olwl(self) -> UVector4: ...
    @property
    def olox(self) -> UVector4: ...
    @property
    def oloy(self) -> UVector4: ...
    @property
    def oloz(self) -> UVector4: ...
    @property
    def olow(self) -> UVector4: ...
    @property
    def ollx(self) -> UVector4: ...
    @property
    def olly(self) -> UVector4: ...
    @property
    def ollz(self) -> UVector4: ...
    @property
    def ollw(self) -> UVector4: ...
    @property
    def lxxx(self) -> UVector4: ...
    @property
    def lxxy(self) -> UVector4: ...
    @property
    def lxxz(self) -> UVector4: ...
    @property
    def lxxw(self) -> UVector4: ...
    @property
    def lxxo(self) -> UVector4: ...
    @property
    def lxxl(self) -> UVector4: ...
    @property
    def lxyx(self) -> UVector4: ...
    @property
    def lxyy(self) -> UVector4: ...
    @property
    def lxyz(self) -> UVector4: ...
    @property
    def lxyw(self) -> UVector4: ...
    @property
    def lxyo(self) -> UVector4: ...
    @property
    def lxyl(self) -> UVector4: ...
    @property
    def lxzx(self) -> UVector4: ...
    @property
    def lxzy(self) -> UVector4: ...
    @property
    def lxzz(self) -> UVector4: ...
    @property
    def lxzw(self) -> UVector4: ...
    @property
    def lxzo(self) -> UVector4: ...
    @property
    def lxzl(self) -> UVector4: ...
    @property
    def lxwx(self) -> UVector4: ...
    @property
    def lxwy(self) -> UVector4: ...
    @property
    def lxwz(self) -> UVector4: ...
    @property
    def lxww(self) -> UVector4: ...
    @property
    def lxwo(self) -> UVector4: ...
    @property
    def lxwl(self) -> UVector4: ...
    @property
    def lxox(self) -> UVector4: ...
    @property
    def lxoy(self) -> UVector4: ...
    @property
    def lxoz(self) -> UVector4: ...
    @property
    def lxow(self) -> UVector4: ...
    @property
    def lxoo(self) -> UVector4: ...
    @property
    def lxol(self) -> UVector4: ...
    @property
    def lxlx(self) -> UVector4: ...
    @property
    def lxly(self) -> UVector4: ...
    @property
    def lxlz(self) -> UVector4: ...
    @property
    def lxlw(self) -> UVector4: ...
    @property
    def lxlo(self) -> UVector4: ...
    @property
    def lxll(self) -> UVector4: ...
    @property
    def lyxx(self) -> UVector4: ...
    @property
    def lyxy(self) -> UVector4: ...
    @property
    def lyxz(self) -> UVector4: ...
    @property
    def lyxw(self) -> UVector4: ...
    @property
    def lyxo(self) -> UVector4: ...
    @property
    def lyxl(self) -> UVector4: ...
    @property
    def lyyx(self) -> UVector4: ...
    @property
    def lyyy(self) -> UVector4: ...
    @property
    def lyyz(self) -> UVector4: ...
    @property
    def lyyw(self) -> UVector4: ...
    @property
    def lyyo(self) -> UVector4: ...
    @property
    def lyyl(self) -> UVector4: ...
    @property
    def lyzx(self) -> UVector4: ...
    @property
    def lyzy(self) -> UVector4: ...
    @property
    def lyzz(self) -> UVector4: ...
    @property
    def lyzw(self) -> UVector4: ...
    @property
    def lyzo(self) -> UVector4: ...
    @property
    def lyzl(self) -> UVector4: ...
    @property
    def lywx(self) -> UVector4: ...
    @property
    def lywy(self) -> UVector4: ...
    @property
    def lywz(self) -> UVector4: ...
    @property
    def lyww(self) -> UVector4: ...
    @property
    def lywo(self) -> UVector4: ...
    @property
    def lywl(self) -> UVector4: ...
    @property
    def lyox(self) -> UVector4: ...
    @property
    def lyoy(self) -> UVector4: ...
    @property
    def lyoz(self) -> UVector4: ...
    @property
    def lyow(self) -> UVector4: ...
    @property
    def lyoo(self) -> UVector4: ...
    @property
    def lyol(self) -> UVector4: ...
    @property
    def lylx(self) -> UVector4: ...
    @property
    def lyly(self) -> UVector4: ...
    @property
    def lylz(self) -> UVector4: ...
    @property
    def lylw(self) -> UVector4: ...
    @property
    def lylo(self) -> UVector4: ...
    @property
    def lyll(self) -> UVector4: ...
    @property
    def lzxx(self) -> UVector4: ...
    @property
    def lzxy(self) -> UVector4: ...
    @property
    def lzxz(self) -> UVector4: ...
    @property
    def lzxw(self) -> UVector4: ...
    @property
    def lzxo(self) -> UVector4: ...
    @property
    def lzxl(self) -> UVector4: ...
    @property
    def lzyx(self) -> UVector4: ...
    @property
    def lzyy(self) -> UVector4: ...
    @property
    def lzyz(self) -> UVector4: ...
    @property
    def lzyw(self) -> UVector4: ...
    @property
    def lzyo(self) -> UVector4: ...
    @property
    def lzyl(self) -> UVector4: ...
    @property
    def lzzx(self) -> UVector4: ...
    @property
    def lzzy(self) -> UVector4: ...
    @property
    def lzzz(self) -> UVector4: ...
    @property
    def lzzw(self) -> UVector4: ...
    @property
    def lzzo(self) -> UVector4: ...
    @property
    def lzzl(self) -> UVector4: ...
    @property
    def lzwx(self) -> UVector4: ...
    @property
    def lzwy(self) -> UVector4: ...
    @property
    def lzwz(self) -> UVector4: ...
    @property
    def lzww(self) -> UVector4: ...
    @property
    def lzwo(self) -> UVector4: ...
    @property
    def lzwl(self) -> UVector4: ...
    @property
    def lzox(self) -> UVector4: ...
    @property
    def lzoy(self) -> UVector4: ...
    @property
    def lzoz(self) -> UVector4: ...
    @property
    def lzow(self) -> UVector4: ...
    @property
    def lzoo(self) -> UVector4: ...
    @property
    def lzol(self) -> UVector4: ...
    @property
    def lzlx(self) -> UVector4: ...
    @property
    def lzly(self) -> UVector4: ...
    @property
    def lzlz(self) -> UVector4: ...
    @property
    def lzlw(self) -> UVector4: ...
    @property
    def lzlo(self) -> UVector4: ...
    @property
    def lzll(self) -> UVector4: ...
    @property
    def lwxx(self) -> UVector4: ...
    @property
    def lwxy(self) -> UVector4: ...
    @property
    def lwxz(self) -> UVector4: ...
    @property
    def lwxw(self) -> UVector4: ...
    @property
    def lwxo(self) -> UVector4: ...
    @property
    def lwxl(self) -> UVector4: ...
    @property
    def lwyx(self) -> UVector4: ...
    @property
    def lwyy(self) -> UVector4: ...
    @property
    def lwyz(self) -> UVector4: ...
    @property
    def lwyw(self) -> UVector4: ...
    @property
    def lwyo(self) -> UVector4: ...
    @property
    def lwyl(self) -> UVector4: ...
    @property
    def lwzx(self) -> UVector4: ...
    @property
    def lwzy(self) -> UVector4: ...
    @property
    def lwzz(self) -> UVector4: ...
    @property
    def lwzw(self) -> UVector4: ...
    @property
    def lwzo(self) -> UVector4: ...
    @property
    def lwzl(self) -> UVector4: ...
    @property
    def lwwx(self) -> UVector4: ...
    @property
    def lwwy(self) -> UVector4: ...
    @property
    def lwwz(self) -> UVector4: ...
    @property
    def lwww(self) -> UVector4: ...
    @property
    def lwwo(self) -> UVector4: ...
    @property
    def lwwl(self) -> UVector4: ...
    @property
    def lwox(self) -> UVector4: ...
    @property
    def lwoy(self) -> UVector4: ...
    @property
    def lwoz(self) -> UVector4: ...
    @property
    def lwow(self) -> UVector4: ...
    @property
    def lwoo(self) -> UVector4: ...
    @property
    def lwol(self) -> UVector4: ...
    @property
    def lwlx(self) -> UVector4: ...
    @property
    def lwly(self) -> UVector4: ...
    @property
    def lwlz(self) -> UVector4: ...
    @property
    def lwlw(self) -> UVector4: ...
    @property
    def lwlo(self) -> UVector4: ...
    @property
    def lwll(self) -> UVector4: ...
    @property
    def loxx(self) -> UVector4: ...
    @property
    def loxy(self) -> UVector4: ...
    @property
    def loxz(self) -> UVector4: ...
    @property
    def loxw(self) -> UVector4: ...
    @property
    def loxo(self) -> UVector4: ...
    @property
    def loxl(self) -> UVector4: ...
    @property
    def loyx(self) -> UVector4: ...
    @property
    def loyy(self) -> UVector4: ...
    @property
    def loyz(self) -> UVector4: ...
    @property
    def loyw(self) -> UVector4: ...
    @property
    def loyo(self) -> UVector4: ...
    @property
    def loyl(self) -> UVector4: ...
    @property
    def lozx(self) -> UVector4: ...
    @property
    def lozy(self) -> UVector4: ...
    @property
    def lozz(self) -> UVector4: ...
    @property
    def lozw(self) -> UVector4: ...
    @property
    def lozo(self) -> UVector4: ...
    @property
    def lozl(self) -> UVector4: ...
    @property
    def lowx(self) -> UVector4: ...
    @property
    def lowy(self) -> UVector4: ...
    @property
    def lowz(self) -> UVector4: ...
    @property
    def loww(self) -> UVector4: ...
    @property
    def lowo(self) -> UVector4: ...
    @property
    def lowl(self) -> UVector4: ...
    @property
    def loox(self) -> UVector4: ...
    @property
    def looy(self) -> UVector4: ...
    @property
    def looz(self) -> UVector4: ...
    @property
    def loow(self) -> UVector4: ...
    @property
    def lolx(self) -> UVector4: ...
    @property
    def loly(self) -> UVector4: ...
    @property
    def lolz(self) -> UVector4: ...
    @property
    def lolw(self) -> UVector4: ...
    @property
    def llxx(self) -> UVector4: ...
    @property
    def llxy(self) -> UVector4: ...
    @property
    def llxz(self) -> UVector4: ...
    @property
    def llxw(self) -> UVector4: ...
    @property
    def llxo(self) -> UVector4: ...
    @property
    def llxl(self) -> UVector4: ...
    @property
    def llyx(self) -> UVector4: ...
    @property
    def llyy(self) -> UVector4: ...
    @property
    def llyz(self) -> UVector4: ...
    @property
    def llyw(self) -> UVector4: ...
    @property
    def llyo(self) -> UVector4: ...
    @property
    def llyl(self) -> UVector4: ...
    @property
    def llzx(self) -> UVector4: ...
    @property
    def llzy(self) -> UVector4: ...
    @property
    def llzz(self) -> UVector4: ...
    @property
    def llzw(self) -> UVector4: ...
    @property
    def llzo(self) -> UVector4: ...
    @property
    def llzl(self) -> UVector4: ...
    @property
    def llwx(self) -> UVector4: ...
    @property
    def llwy(self) -> UVector4: ...
    @property
    def llwz(self) -> UVector4: ...
    @property
    def llww(self) -> UVector4: ...
    @property
    def llwo(self) -> UVector4: ...
    @property
    def llwl(self) -> UVector4: ...
    @property
    def llox(self) -> UVector4: ...
    @property
    def lloy(self) -> UVector4: ...
    @property
    def lloz(self) -> UVector4: ...
    @property
    def llow(self) -> UVector4: ...
    @property
    def lllx(self) -> UVector4: ...
    @property
    def llly(self) -> UVector4: ...
    @property
    def lllz(self) -> UVector4: ...
    @property
    def lllw(self) -> UVector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> UVector2: ...
    @property
    def rg(self) -> UVector2: ...
    @property
    def rb(self) -> UVector2: ...
    @property
    def ra(self) -> UVector2: ...
    @property
    def ro(self) -> UVector2: ...
    @property
    def rl(self) -> UVector2: ...
    @property
    def gr(self) -> UVector2: ...
    @property
    def gg(self) -> UVector2: ...
    @property
    def gb(self) -> UVector2: ...
    @property
    def ga(self) -> UVector2: ...
    @property
    def go(self) -> UVector2: ...
    @property
    def gl(self) -> UVector2: ...
    @property
    def br(self) -> UVector2: ...
    @property
    def bg(self) -> UVector2: ...
    @property
    def bb(self) -> UVector2: ...
    @property
    def ba(self) -> UVector2: ...
    @property
    def bo(self) -> UVector2: ...
    @property
    def bl(self) -> UVector2: ...
    @property
    def ar(self) -> UVector2: ...
    @property
    def ag(self) -> UVector2: ...
    @property
    def ab(self) -> UVector2: ...
    @property
    def aa(self) -> UVector2: ...
    @property
    def ao(self) -> UVector2: ...
    @property
    def al(self) -> UVector2: ...
    @property
    def og(self) -> UVector2: ...
    @property
    def ob(self) -> UVector2: ...
    @property
    def oa(self) -> UVector2: ...
    @property
    def lr(self) -> UVector2: ...
    @property
    def lg(self) -> UVector2: ...
    @property
    def lb(self) -> UVector2: ...
    @property
    def la(self) -> UVector2: ...
    @property
    def rrr(self) -> UVector3: ...
    @property
    def rrg(self) -> UVector3: ...
    @property
    def rrb(self) -> UVector3: ...
    @property
    def rra(self) -> UVector3: ...
    @property
    def rro(self) -> UVector3: ...
    @property
    def rrl(self) -> UVector3: ...
    @property
    def rgr(self) -> UVector3: ...
    @property
    def rgg(self) -> UVector3: ...
    @property
    def rgb(self) -> UVector3: ...
    @property
    def rga(self) -> UVector3: ...
    @property
    def rgo(self) -> UVector3: ...
    @property
    def rgl(self) -> UVector3: ...
    @property
    def rbr(self) -> UVector3: ...
    @property
    def rbg(self) -> UVector3: ...
    @property
    def rbb(self) -> UVector3: ...
    @property
    def rba(self) -> UVector3: ...
    @property
    def rbo(self) -> UVector3: ...
    @property
    def rbl(self) -> UVector3: ...
    @property
    def rar(self) -> UVector3: ...
    @property
    def rag(self) -> UVector3: ...
    @property
    def rab(self) -> UVector3: ...
    @property
    def raa(self) -> UVector3: ...
    @property
    def rao(self) -> UVector3: ...
    @property
    def ral(self) -> UVector3: ...
    @property
    def ror(self) -> UVector3: ...
    @property
    def rog(self) -> UVector3: ...
    @property
    def rob(self) -> UVector3: ...
    @property
    def roa(self) -> UVector3: ...
    @property
    def roo(self) -> UVector3: ...
    @property
    def rol(self) -> UVector3: ...
    @property
    def rlr(self) -> UVector3: ...
    @property
    def rlg(self) -> UVector3: ...
    @property
    def rlb(self) -> UVector3: ...
    @property
    def rla(self) -> UVector3: ...
    @property
    def rlo(self) -> UVector3: ...
    @property
    def rll(self) -> UVector3: ...
    @property
    def grr(self) -> UVector3: ...
    @property
    def grg(self) -> UVector3: ...
    @property
    def grb(self) -> UVector3: ...
    @property
    def gra(self) -> UVector3: ...
    @property
    def gro(self) -> UVector3: ...
    @property
    def grl(self) -> UVector3: ...
    @property
    def ggr(self) -> UVector3: ...
    @property
    def ggg(self) -> UVector3: ...
    @property
    def ggb(self) -> UVector3: ...
    @property
    def gga(self) -> UVector3: ...
    @property
    def ggo(self) -> UVector3: ...
    @property
    def ggl(self) -> UVector3: ...
    @property
    def gbr(self) -> UVector3: ...
    @property
    def gbg(self) -> UVector3: ...
    @property
    def gbb(self) -> UVector3: ...
    @property
    def gba(self) -> UVector3: ...
    @property
    def gbo(self) -> UVector3: ...
    @property
    def gbl(self) -> UVector3: ...
    @property
    def gar(self) -> UVector3: ...
    @property
    def gag(self) -> UVector3: ...
    @property
    def gab(self) -> UVector3: ...
    @property
    def gaa(self) -> UVector3: ...
    @property
    def gao(self) -> UVector3: ...
    @property
    def gal(self) -> UVector3: ...
    @property
    def gor(self) -> UVector3: ...
    @property
    def gog(self) -> UVector3: ...
    @property
    def gob(self) -> UVector3: ...
    @property
    def goa(self) -> UVector3: ...
    @property
    def goo(self) -> UVector3: ...
    @property
    def gol(self) -> UVector3: ...
    @property
    def glr(self) -> UVector3: ...
    @property
    def glg(self) -> UVector3: ...
    @property
    def glb(self) -> UVector3: ...
    @property
    def gla(self) -> UVector3: ...
    @property
    def glo(self) -> UVector3: ...
    @property
    def gll(self) -> UVector3: ...
    @property
    def brr(self) -> UVector3: ...
    @property
    def brg(self) -> UVector3: ...
    @property
    def brb(self) -> UVector3: ...
    @property
    def bra(self) -> UVector3: ...
    @property
    def bro(self) -> UVector3: ...
    @property
    def brl(self) -> UVector3: ...
    @property
    def bgr(self) -> UVector3: ...
    @property
    def bgg(self) -> UVector3: ...
    @property
    def bgb(self) -> UVector3: ...
    @property
    def bga(self) -> UVector3: ...
    @property
    def bgo(self) -> UVector3: ...
    @property
    def bgl(self) -> UVector3: ...
    @property
    def bbr(self) -> UVector3: ...
    @property
    def bbg(self) -> UVector3: ...
    @property
    def bbb(self) -> UVector3: ...
    @property
    def bba(self) -> UVector3: ...
    @property
    def bbo(self) -> UVector3: ...
    @property
    def bbl(self) -> UVector3: ...
    @property
    def bar(self) -> UVector3: ...
    @property
    def bag(self) -> UVector3: ...
    @property
    def bab(self) -> UVector3: ...
    @property
    def baa(self) -> UVector3: ...
    @property
    def bao(self) -> UVector3: ...
    @property
    def bal(self) -> UVector3: ...
    @property
    def bor(self) -> UVector3: ...
    @property
    def bog(self) -> UVector3: ...
    @property
    def bob(self) -> UVector3: ...
    @property
    def boa(self) -> UVector3: ...
    @property
    def boo(self) -> UVector3: ...
    @property
    def bol(self) -> UVector3: ...
    @property
    def blr(self) -> UVector3: ...
    @property
    def blg(self) -> UVector3: ...
    @property
    def blb(self) -> UVector3: ...
    @property
    def bla(self) -> UVector3: ...
    @property
    def blo(self) -> UVector3: ...
    @property
    def bll(self) -> UVector3: ...
    @property
    def arr(self) -> UVector3: ...
    @property
    def arg(self) -> UVector3: ...
    @property
    def arb(self) -> UVector3: ...
    @property
    def ara(self) -> UVector3: ...
    @property
    def aro(self) -> UVector3: ...
    @property
    def arl(self) -> UVector3: ...
    @property
    def agr(self) -> UVector3: ...
    @property
    def agg(self) -> UVector3: ...
    @property
    def agb(self) -> UVector3: ...
    @property
    def aga(self) -> UVector3: ...
    @property
    def ago(self) -> UVector3: ...
    @property
    def agl(self) -> UVector3: ...
    @property
    def abr(self) -> UVector3: ...
    @property
    def abg(self) -> UVector3: ...
    @property
    def abb(self) -> UVector3: ...
    @property
    def aba(self) -> UVector3: ...
    @property
    def abo(self) -> UVector3: ...
    @property
    def abl(self) -> UVector3: ...
    @property
    def aar(self) -> UVector3: ...
    @property
    def aag(self) -> UVector3: ...
    @property
    def aab(self) -> UVector3: ...
    @property
    def aaa(self) -> UVector3: ...
    @property
    def aao(self) -> UVector3: ...
    @property
    def aal(self) -> UVector3: ...
    @property
    def aor(self) -> UVector3: ...
    @property
    def aog(self) -> UVector3: ...
    @property
    def aob(self) -> UVector3: ...
    @property
    def aoa(self) -> UVector3: ...
    @property
    def aoo(self) -> UVector3: ...
    @property
    def aol(self) -> UVector3: ...
    @property
    def alr(self) -> UVector3: ...
    @property
    def alg(self) -> UVector3: ...
    @property
    def alb(self) -> UVector3: ...
    @property
    def ala(self) -> UVector3: ...
    @property
    def alo(self) -> UVector3: ...
    @property
    def all(self) -> UVector3: ...
    @property
    def orr(self) -> UVector3: ...
    @property
    def org(self) -> UVector3: ...
    @property
    def orb(self) -> UVector3: ...
    @property
    def ora(self) -> UVector3: ...
    @property
    def oro(self) -> UVector3: ...
    @property
    def orl(self) -> UVector3: ...
    @property
    def ogr(self) -> UVector3: ...
    @property
    def ogg(self) -> UVector3: ...
    @property
    def ogb(self) -> UVector3: ...
    @property
    def oga(self) -> UVector3: ...
    @property
    def ogo(self) -> UVector3: ...
    @property
    def ogl(self) -> UVector3: ...
    @property
    def obr(self) -> UVector3: ...
    @property
    def obg(self) -> UVector3: ...
    @property
    def obb(self) -> UVector3: ...
    @property
    def oba(self) -> UVector3: ...
    @property
    def obo(self) -> UVector3: ...
    @property
    def obl(self) -> UVector3: ...
    @property
    def oar(self) -> UVector3: ...
    @property
    def oag(self) -> UVector3: ...
    @property
    def oab(self) -> UVector3: ...
    @property
    def oaa(self) -> UVector3: ...
    @property
    def oao(self) -> UVector3: ...
    @property
    def oal(self) -> UVector3: ...
    @property
    def oor(self) -> UVector3: ...
    @property
    def oog(self) -> UVector3: ...
    @property
    def oob(self) -> UVector3: ...
    @property
    def ooa(self) -> UVector3: ...
    @property
    def olr(self) -> UVector3: ...
    @property
    def olg(self) -> UVector3: ...
    @property
    def olb(self) -> UVector3: ...
    @property
    def ola(self) -> UVector3: ...
    @property
    def lrr(self) -> UVector3: ...
    @property
    def lrg(self) -> UVector3: ...
    @property
    def lrb(self) -> UVector3: ...
    @property
    def lra(self) -> UVector3: ...
    @property
    def lro(self) -> UVector3: ...
    @property
    def lrl(self) -> UVector3: ...
    @property
    def lgr(self) -> UVector3: ...
    @property
    def lgg(self) -> UVector3: ...
    @property
    def lgb(self) -> UVector3: ...
    @property
    def lga(self) -> UVector3: ...
    @property
    def lgo(self) -> UVector3: ...
    @property
    def lgl(self) -> UVector3: ...
    @property
    def lbr(self) -> UVector3: ...
    @property
    def lbg(self) -> UVector3: ...
    @property
    def lbb(self) -> UVector3: ...
    @property
    def lba(self) -> UVector3: ...
    @property
    def lbo(self) -> UVector3: ...
    @property
    def lbl(self) -> UVector3: ...
    @property
    def lar(self) -> UVector3: ...
    @property
    def lag(self) -> UVector3: ...
    @property
    def lab(self) -> UVector3: ...
    @property
    def laa(self) -> UVector3: ...
    @property
    def lao(self) -> UVector3: ...
    @property
    def lal(self) -> UVector3: ...
    @property
    def lor(self) -> UVector3: ...
    @property
    def log(self) -> UVector3: ...
    @property
    def lob(self) -> UVector3: ...
    @property
    def loa(self) -> UVector3: ...
    @property
    def llr(self) -> UVector3: ...
    @property
    def llg(self) -> UVector3: ...
    @property
    def llb(self) -> UVector3: ...
    @property
    def lla(self) -> UVector3: ...
    @property
    def rrrr(self) -> UVector4: ...
    @property
    def rrrg(self) -> UVector4: ...
    @property
    def rrrb(self) -> UVector4: ...
    @property
    def rrra(self) -> UVector4: ...
    @property
    def rrro(self) -> UVector4: ...
    @property
    def rrrl(self) -> UVector4: ...
    @property
    def rrgr(self) -> UVector4: ...
    @property
    def rrgg(self) -> UVector4: ...
    @property
    def rrgb(self) -> UVector4: ...
    @property
    def rrga(self) -> UVector4: ...
    @property
    def rrgo(self) -> UVector4: ...
    @property
    def rrgl(self) -> UVector4: ...
    @property
    def rrbr(self) -> UVector4: ...
    @property
    def rrbg(self) -> UVector4: ...
    @property
    def rrbb(self) -> UVector4: ...
    @property
    def rrba(self) -> UVector4: ...
    @property
    def rrbo(self) -> UVector4: ...
    @property
    def rrbl(self) -> UVector4: ...
    @property
    def rrar(self) -> UVector4: ...
    @property
    def rrag(self) -> UVector4: ...
    @property
    def rrab(self) -> UVector4: ...
    @property
    def rraa(self) -> UVector4: ...
    @property
    def rrao(self) -> UVector4: ...
    @property
    def rral(self) -> UVector4: ...
    @property
    def rror(self) -> UVector4: ...
    @property
    def rrog(self) -> UVector4: ...
    @property
    def rrob(self) -> UVector4: ...
    @property
    def rroa(self) -> UVector4: ...
    @property
    def rroo(self) -> UVector4: ...
    @property
    def rrol(self) -> UVector4: ...
    @property
    def rrlr(self) -> UVector4: ...
    @property
    def rrlg(self) -> UVector4: ...
    @property
    def rrlb(self) -> UVector4: ...
    @property
    def rrla(self) -> UVector4: ...
    @property
    def rrlo(self) -> UVector4: ...
    @property
    def rrll(self) -> UVector4: ...
    @property
    def rgrr(self) -> UVector4: ...
    @property
    def rgrg(self) -> UVector4: ...
    @property
    def rgrb(self) -> UVector4: ...
    @property
    def rgra(self) -> UVector4: ...
    @property
    def rgro(self) -> UVector4: ...
    @property
    def rgrl(self) -> UVector4: ...
    @property
    def rggr(self) -> UVector4: ...
    @property
    def rggg(self) -> UVector4: ...
    @property
    def rggb(self) -> UVector4: ...
    @property
    def rgga(self) -> UVector4: ...
    @property
    def rggo(self) -> UVector4: ...
    @property
    def rggl(self) -> UVector4: ...
    @property
    def rgbr(self) -> UVector4: ...
    @property
    def rgbg(self) -> UVector4: ...
    @property
    def rgbb(self) -> UVector4: ...
    @property
    def rgba(self) -> UVector4: ...
    @property
    def rgbo(self) -> UVector4: ...
    @property
    def rgbl(self) -> UVector4: ...
    @property
    def rgar(self) -> UVector4: ...
    @property
    def rgag(self) -> UVector4: ...
    @property
    def rgab(self) -> UVector4: ...
    @property
    def rgaa(self) -> UVector4: ...
    @property
    def rgao(self) -> UVector4: ...
    @property
    def rgal(self) -> UVector4: ...
    @property
    def rgor(self) -> UVector4: ...
    @property
    def rgog(self) -> UVector4: ...
    @property
    def rgob(self) -> UVector4: ...
    @property
    def rgoa(self) -> UVector4: ...
    @property
    def rgoo(self) -> UVector4: ...
    @property
    def rgol(self) -> UVector4: ...
    @property
    def rglr(self) -> UVector4: ...
    @property
    def rglg(self) -> UVector4: ...
    @property
    def rglb(self) -> UVector4: ...
    @property
    def rgla(self) -> UVector4: ...
    @property
    def rglo(self) -> UVector4: ...
    @property
    def rgll(self) -> UVector4: ...
    @property
    def rbrr(self) -> UVector4: ...
    @property
    def rbrg(self) -> UVector4: ...
    @property
    def rbrb(self) -> UVector4: ...
    @property
    def rbra(self) -> UVector4: ...
    @property
    def rbro(self) -> UVector4: ...
    @property
    def rbrl(self) -> UVector4: ...
    @property
    def rbgr(self) -> UVector4: ...
    @property
    def rbgg(self) -> UVector4: ...
    @property
    def rbgb(self) -> UVector4: ...
    @property
    def rbga(self) -> UVector4: ...
    @property
    def rbgo(self) -> UVector4: ...
    @property
    def rbgl(self) -> UVector4: ...
    @property
    def rbbr(self) -> UVector4: ...
    @property
    def rbbg(self) -> UVector4: ...
    @property
    def rbbb(self) -> UVector4: ...
    @property
    def rbba(self) -> UVector4: ...
    @property
    def rbbo(self) -> UVector4: ...
    @property
    def rbbl(self) -> UVector4: ...
    @property
    def rbar(self) -> UVector4: ...
    @property
    def rbag(self) -> UVector4: ...
    @property
    def rbab(self) -> UVector4: ...
    @property
    def rbaa(self) -> UVector4: ...
    @property
    def rbao(self) -> UVector4: ...
    @property
    def rbal(self) -> UVector4: ...
    @property
    def rbor(self) -> UVector4: ...
    @property
    def rbog(self) -> UVector4: ...
    @property
    def rbob(self) -> UVector4: ...
    @property
    def rboa(self) -> UVector4: ...
    @property
    def rboo(self) -> UVector4: ...
    @property
    def rbol(self) -> UVector4: ...
    @property
    def rblr(self) -> UVector4: ...
    @property
    def rblg(self) -> UVector4: ...
    @property
    def rblb(self) -> UVector4: ...
    @property
    def rbla(self) -> UVector4: ...
    @property
    def rblo(self) -> UVector4: ...
    @property
    def rbll(self) -> UVector4: ...
    @property
    def rarr(self) -> UVector4: ...
    @property
    def rarg(self) -> UVector4: ...
    @property
    def rarb(self) -> UVector4: ...
    @property
    def rara(self) -> UVector4: ...
    @property
    def raro(self) -> UVector4: ...
    @property
    def rarl(self) -> UVector4: ...
    @property
    def ragr(self) -> UVector4: ...
    @property
    def ragg(self) -> UVector4: ...
    @property
    def ragb(self) -> UVector4: ...
    @property
    def raga(self) -> UVector4: ...
    @property
    def rago(self) -> UVector4: ...
    @property
    def ragl(self) -> UVector4: ...
    @property
    def rabr(self) -> UVector4: ...
    @property
    def rabg(self) -> UVector4: ...
    @property
    def rabb(self) -> UVector4: ...
    @property
    def raba(self) -> UVector4: ...
    @property
    def rabo(self) -> UVector4: ...
    @property
    def rabl(self) -> UVector4: ...
    @property
    def raar(self) -> UVector4: ...
    @property
    def raag(self) -> UVector4: ...
    @property
    def raab(self) -> UVector4: ...
    @property
    def raaa(self) -> UVector4: ...
    @property
    def raao(self) -> UVector4: ...
    @property
    def raal(self) -> UVector4: ...
    @property
    def raor(self) -> UVector4: ...
    @property
    def raog(self) -> UVector4: ...
    @property
    def raob(self) -> UVector4: ...
    @property
    def raoa(self) -> UVector4: ...
    @property
    def raoo(self) -> UVector4: ...
    @property
    def raol(self) -> UVector4: ...
    @property
    def ralr(self) -> UVector4: ...
    @property
    def ralg(self) -> UVector4: ...
    @property
    def ralb(self) -> UVector4: ...
    @property
    def rala(self) -> UVector4: ...
    @property
    def ralo(self) -> UVector4: ...
    @property
    def rall(self) -> UVector4: ...
    @property
    def rorr(self) -> UVector4: ...
    @property
    def rorg(self) -> UVector4: ...
    @property
    def rorb(self) -> UVector4: ...
    @property
    def rora(self) -> UVector4: ...
    @property
    def roro(self) -> UVector4: ...
    @property
    def rorl(self) -> UVector4: ...
    @property
    def rogr(self) -> UVector4: ...
    @property
    def rogg(self) -> UVector4: ...
    @property
    def rogb(self) -> UVector4: ...
    @property
    def roga(self) -> UVector4: ...
    @property
    def rogo(self) -> UVector4: ...
    @property
    def rogl(self) -> UVector4: ...
    @property
    def robr(self) -> UVector4: ...
    @property
    def robg(self) -> UVector4: ...
    @property
    def robb(self) -> UVector4: ...
    @property
    def roba(self) -> UVector4: ...
    @property
    def robo(self) -> UVector4: ...
    @property
    def robl(self) -> UVector4: ...
    @property
    def roar(self) -> UVector4: ...
    @property
    def roag(self) -> UVector4: ...
    @property
    def roab(self) -> UVector4: ...
    @property
    def roaa(self) -> UVector4: ...
    @property
    def roao(self) -> UVector4: ...
    @property
    def roal(self) -> UVector4: ...
    @property
    def roor(self) -> UVector4: ...
    @property
    def roog(self) -> UVector4: ...
    @property
    def roob(self) -> UVector4: ...
    @property
    def rooa(self) -> UVector4: ...
    @property
    def rooo(self) -> UVector4: ...
    @property
    def rool(self) -> UVector4: ...
    @property
    def rolr(self) -> UVector4: ...
    @property
    def rolg(self) -> UVector4: ...
    @property
    def rolb(self) -> UVector4: ...
    @property
    def rola(self) -> UVector4: ...
    @property
    def rolo(self) -> UVector4: ...
    @property
    def roll(self) -> UVector4: ...
    @property
    def rlrr(self) -> UVector4: ...
    @property
    def rlrg(self) -> UVector4: ...
    @property
    def rlrb(self) -> UVector4: ...
    @property
    def rlra(self) -> UVector4: ...
    @property
    def rlro(self) -> UVector4: ...
    @property
    def rlrl(self) -> UVector4: ...
    @property
    def rlgr(self) -> UVector4: ...
    @property
    def rlgg(self) -> UVector4: ...
    @property
    def rlgb(self) -> UVector4: ...
    @property
    def rlga(self) -> UVector4: ...
    @property
    def rlgo(self) -> UVector4: ...
    @property
    def rlgl(self) -> UVector4: ...
    @property
    def rlbr(self) -> UVector4: ...
    @property
    def rlbg(self) -> UVector4: ...
    @property
    def rlbb(self) -> UVector4: ...
    @property
    def rlba(self) -> UVector4: ...
    @property
    def rlbo(self) -> UVector4: ...
    @property
    def rlbl(self) -> UVector4: ...
    @property
    def rlar(self) -> UVector4: ...
    @property
    def rlag(self) -> UVector4: ...
    @property
    def rlab(self) -> UVector4: ...
    @property
    def rlaa(self) -> UVector4: ...
    @property
    def rlao(self) -> UVector4: ...
    @property
    def rlal(self) -> UVector4: ...
    @property
    def rlor(self) -> UVector4: ...
    @property
    def rlog(self) -> UVector4: ...
    @property
    def rlob(self) -> UVector4: ...
    @property
    def rloa(self) -> UVector4: ...
    @property
    def rloo(self) -> UVector4: ...
    @property
    def rlol(self) -> UVector4: ...
    @property
    def rllr(self) -> UVector4: ...
    @property
    def rllg(self) -> UVector4: ...
    @property
    def rllb(self) -> UVector4: ...
    @property
    def rlla(self) -> UVector4: ...
    @property
    def rllo(self) -> UVector4: ...
    @property
    def rlll(self) -> UVector4: ...
    @property
    def grrr(self) -> UVector4: ...
    @property
    def grrg(self) -> UVector4: ...
    @property
    def grrb(self) -> UVector4: ...
    @property
    def grra(self) -> UVector4: ...
    @property
    def grro(self) -> UVector4: ...
    @property
    def grrl(self) -> UVector4: ...
    @property
    def grgr(self) -> UVector4: ...
    @property
    def grgg(self) -> UVector4: ...
    @property
    def grgb(self) -> UVector4: ...
    @property
    def grga(self) -> UVector4: ...
    @property
    def grgo(self) -> UVector4: ...
    @property
    def grgl(self) -> UVector4: ...
    @property
    def grbr(self) -> UVector4: ...
    @property
    def grbg(self) -> UVector4: ...
    @property
    def grbb(self) -> UVector4: ...
    @property
    def grba(self) -> UVector4: ...
    @property
    def grbo(self) -> UVector4: ...
    @property
    def grbl(self) -> UVector4: ...
    @property
    def grar(self) -> UVector4: ...
    @property
    def grag(self) -> UVector4: ...
    @property
    def grab(self) -> UVector4: ...
    @property
    def graa(self) -> UVector4: ...
    @property
    def grao(self) -> UVector4: ...
    @property
    def gral(self) -> UVector4: ...
    @property
    def gror(self) -> UVector4: ...
    @property
    def grog(self) -> UVector4: ...
    @property
    def grob(self) -> UVector4: ...
    @property
    def groa(self) -> UVector4: ...
    @property
    def groo(self) -> UVector4: ...
    @property
    def grol(self) -> UVector4: ...
    @property
    def grlr(self) -> UVector4: ...
    @property
    def grlg(self) -> UVector4: ...
    @property
    def grlb(self) -> UVector4: ...
    @property
    def grla(self) -> UVector4: ...
    @property
    def grlo(self) -> UVector4: ...
    @property
    def grll(self) -> UVector4: ...
    @property
    def ggrr(self) -> UVector4: ...
    @property
    def ggrg(self) -> UVector4: ...
    @property
    def ggrb(self) -> UVector4: ...
    @property
    def ggra(self) -> UVector4: ...
    @property
    def ggro(self) -> UVector4: ...
    @property
    def ggrl(self) -> UVector4: ...
    @property
    def gggr(self) -> UVector4: ...
    @property
    def gggg(self) -> UVector4: ...
    @property
    def gggb(self) -> UVector4: ...
    @property
    def ggga(self) -> UVector4: ...
    @property
    def gggo(self) -> UVector4: ...
    @property
    def gggl(self) -> UVector4: ...
    @property
    def ggbr(self) -> UVector4: ...
    @property
    def ggbg(self) -> UVector4: ...
    @property
    def ggbb(self) -> UVector4: ...
    @property
    def ggba(self) -> UVector4: ...
    @property
    def ggbo(self) -> UVector4: ...
    @property
    def ggbl(self) -> UVector4: ...
    @property
    def ggar(self) -> UVector4: ...
    @property
    def ggag(self) -> UVector4: ...
    @property
    def ggab(self) -> UVector4: ...
    @property
    def ggaa(self) -> UVector4: ...
    @property
    def ggao(self) -> UVector4: ...
    @property
    def ggal(self) -> UVector4: ...
    @property
    def ggor(self) -> UVector4: ...
    @property
    def ggog(self) -> UVector4: ...
    @property
    def ggob(self) -> UVector4: ...
    @property
    def ggoa(self) -> UVector4: ...
    @property
    def ggoo(self) -> UVector4: ...
    @property
    def ggol(self) -> UVector4: ...
    @property
    def gglr(self) -> UVector4: ...
    @property
    def gglg(self) -> UVector4: ...
    @property
    def gglb(self) -> UVector4: ...
    @property
    def ggla(self) -> UVector4: ...
    @property
    def gglo(self) -> UVector4: ...
    @property
    def ggll(self) -> UVector4: ...
    @property
    def gbrr(self) -> UVector4: ...
    @property
    def gbrg(self) -> UVector4: ...
    @property
    def gbrb(self) -> UVector4: ...
    @property
    def gbra(self) -> UVector4: ...
    @property
    def gbro(self) -> UVector4: ...
    @property
    def gbrl(self) -> UVector4: ...
    @property
    def gbgr(self) -> UVector4: ...
    @property
    def gbgg(self) -> UVector4: ...
    @property
    def gbgb(self) -> UVector4: ...
    @property
    def gbga(self) -> UVector4: ...
    @property
    def gbgo(self) -> UVector4: ...
    @property
    def gbgl(self) -> UVector4: ...
    @property
    def gbbr(self) -> UVector4: ...
    @property
    def gbbg(self) -> UVector4: ...
    @property
    def gbbb(self) -> UVector4: ...
    @property
    def gbba(self) -> UVector4: ...
    @property
    def gbbo(self) -> UVector4: ...
    @property
    def gbbl(self) -> UVector4: ...
    @property
    def gbar(self) -> UVector4: ...
    @property
    def gbag(self) -> UVector4: ...
    @property
    def gbab(self) -> UVector4: ...
    @property
    def gbaa(self) -> UVector4: ...
    @property
    def gbao(self) -> UVector4: ...
    @property
    def gbal(self) -> UVector4: ...
    @property
    def gbor(self) -> UVector4: ...
    @property
    def gbog(self) -> UVector4: ...
    @property
    def gbob(self) -> UVector4: ...
    @property
    def gboa(self) -> UVector4: ...
    @property
    def gboo(self) -> UVector4: ...
    @property
    def gbol(self) -> UVector4: ...
    @property
    def gblr(self) -> UVector4: ...
    @property
    def gblg(self) -> UVector4: ...
    @property
    def gblb(self) -> UVector4: ...
    @property
    def gbla(self) -> UVector4: ...
    @property
    def gblo(self) -> UVector4: ...
    @property
    def gbll(self) -> UVector4: ...
    @property
    def garr(self) -> UVector4: ...
    @property
    def garg(self) -> UVector4: ...
    @property
    def garb(self) -> UVector4: ...
    @property
    def gara(self) -> UVector4: ...
    @property
    def garo(self) -> UVector4: ...
    @property
    def garl(self) -> UVector4: ...
    @property
    def gagr(self) -> UVector4: ...
    @property
    def gagg(self) -> UVector4: ...
    @property
    def gagb(self) -> UVector4: ...
    @property
    def gaga(self) -> UVector4: ...
    @property
    def gago(self) -> UVector4: ...
    @property
    def gagl(self) -> UVector4: ...
    @property
    def gabr(self) -> UVector4: ...
    @property
    def gabg(self) -> UVector4: ...
    @property
    def gabb(self) -> UVector4: ...
    @property
    def gaba(self) -> UVector4: ...
    @property
    def gabo(self) -> UVector4: ...
    @property
    def gabl(self) -> UVector4: ...
    @property
    def gaar(self) -> UVector4: ...
    @property
    def gaag(self) -> UVector4: ...
    @property
    def gaab(self) -> UVector4: ...
    @property
    def gaaa(self) -> UVector4: ...
    @property
    def gaao(self) -> UVector4: ...
    @property
    def gaal(self) -> UVector4: ...
    @property
    def gaor(self) -> UVector4: ...
    @property
    def gaog(self) -> UVector4: ...
    @property
    def gaob(self) -> UVector4: ...
    @property
    def gaoa(self) -> UVector4: ...
    @property
    def gaoo(self) -> UVector4: ...
    @property
    def gaol(self) -> UVector4: ...
    @property
    def galr(self) -> UVector4: ...
    @property
    def galg(self) -> UVector4: ...
    @property
    def galb(self) -> UVector4: ...
    @property
    def gala(self) -> UVector4: ...
    @property
    def galo(self) -> UVector4: ...
    @property
    def gall(self) -> UVector4: ...
    @property
    def gorr(self) -> UVector4: ...
    @property
    def gorg(self) -> UVector4: ...
    @property
    def gorb(self) -> UVector4: ...
    @property
    def gora(self) -> UVector4: ...
    @property
    def goro(self) -> UVector4: ...
    @property
    def gorl(self) -> UVector4: ...
    @property
    def gogr(self) -> UVector4: ...
    @property
    def gogg(self) -> UVector4: ...
    @property
    def gogb(self) -> UVector4: ...
    @property
    def goga(self) -> UVector4: ...
    @property
    def gogo(self) -> UVector4: ...
    @property
    def gogl(self) -> UVector4: ...
    @property
    def gobr(self) -> UVector4: ...
    @property
    def gobg(self) -> UVector4: ...
    @property
    def gobb(self) -> UVector4: ...
    @property
    def goba(self) -> UVector4: ...
    @property
    def gobo(self) -> UVector4: ...
    @property
    def gobl(self) -> UVector4: ...
    @property
    def goar(self) -> UVector4: ...
    @property
    def goag(self) -> UVector4: ...
    @property
    def goab(self) -> UVector4: ...
    @property
    def goaa(self) -> UVector4: ...
    @property
    def goao(self) -> UVector4: ...
    @property
    def goal(self) -> UVector4: ...
    @property
    def goor(self) -> UVector4: ...
    @property
    def goog(self) -> UVector4: ...
    @property
    def goob(self) -> UVector4: ...
    @property
    def gooa(self) -> UVector4: ...
    @property
    def gooo(self) -> UVector4: ...
    @property
    def gool(self) -> UVector4: ...
    @property
    def golr(self) -> UVector4: ...
    @property
    def golg(self) -> UVector4: ...
    @property
    def golb(self) -> UVector4: ...
    @property
    def gola(self) -> UVector4: ...
    @property
    def golo(self) -> UVector4: ...
    @property
    def goll(self) -> UVector4: ...
    @property
    def glrr(self) -> UVector4: ...
    @property
    def glrg(self) -> UVector4: ...
    @property
    def glrb(self) -> UVector4: ...
    @property
    def glra(self) -> UVector4: ...
    @property
    def glro(self) -> UVector4: ...
    @property
    def glrl(self) -> UVector4: ...
    @property
    def glgr(self) -> UVector4: ...
    @property
    def glgg(self) -> UVector4: ...
    @property
    def glgb(self) -> UVector4: ...
    @property
    def glga(self) -> UVector4: ...
    @property
    def glgo(self) -> UVector4: ...
    @property
    def glgl(self) -> UVector4: ...
    @property
    def glbr(self) -> UVector4: ...
    @property
    def glbg(self) -> UVector4: ...
    @property
    def glbb(self) -> UVector4: ...
    @property
    def glba(self) -> UVector4: ...
    @property
    def glbo(self) -> UVector4: ...
    @property
    def glbl(self) -> UVector4: ...
    @property
    def glar(self) -> UVector4: ...
    @property
    def glag(self) -> UVector4: ...
    @property
    def glab(self) -> UVector4: ...
    @property
    def glaa(self) -> UVector4: ...
    @property
    def glao(self) -> UVector4: ...
    @property
    def glal(self) -> UVector4: ...
    @property
    def glor(self) -> UVector4: ...
    @property
    def glog(self) -> UVector4: ...
    @property
    def glob(self) -> UVector4: ...
    @property
    def gloa(self) -> UVector4: ...
    @property
    def gloo(self) -> UVector4: ...
    @property
    def glol(self) -> UVector4: ...
    @property
    def gllr(self) -> UVector4: ...
    @property
    def gllg(self) -> UVector4: ...
    @property
    def gllb(self) -> UVector4: ...
    @property
    def glla(self) -> UVector4: ...
    @property
    def gllo(self) -> UVector4: ...
    @property
    def glll(self) -> UVector4: ...
    @property
    def brrr(self) -> UVector4: ...
    @property
    def brrg(self) -> UVector4: ...
    @property
    def brrb(self) -> UVector4: ...
    @property
    def brra(self) -> UVector4: ...
    @property
    def brro(self) -> UVector4: ...
    @property
    def brrl(self) -> UVector4: ...
    @property
    def brgr(self) -> UVector4: ...
    @property
    def brgg(self) -> UVector4: ...
    @property
    def brgb(self) -> UVector4: ...
    @property
    def brga(self) -> UVector4: ...
    @property
    def brgo(self) -> UVector4: ...
    @property
    def brgl(self) -> UVector4: ...
    @property
    def brbr(self) -> UVector4: ...
    @property
    def brbg(self) -> UVector4: ...
    @property
    def brbb(self) -> UVector4: ...
    @property
    def brba(self) -> UVector4: ...
    @property
    def brbo(self) -> UVector4: ...
    @property
    def brbl(self) -> UVector4: ...
    @property
    def brar(self) -> UVector4: ...
    @property
    def brag(self) -> UVector4: ...
    @property
    def brab(self) -> UVector4: ...
    @property
    def braa(self) -> UVector4: ...
    @property
    def brao(self) -> UVector4: ...
    @property
    def bral(self) -> UVector4: ...
    @property
    def bror(self) -> UVector4: ...
    @property
    def brog(self) -> UVector4: ...
    @property
    def brob(self) -> UVector4: ...
    @property
    def broa(self) -> UVector4: ...
    @property
    def broo(self) -> UVector4: ...
    @property
    def brol(self) -> UVector4: ...
    @property
    def brlr(self) -> UVector4: ...
    @property
    def brlg(self) -> UVector4: ...
    @property
    def brlb(self) -> UVector4: ...
    @property
    def brla(self) -> UVector4: ...
    @property
    def brlo(self) -> UVector4: ...
    @property
    def brll(self) -> UVector4: ...
    @property
    def bgrr(self) -> UVector4: ...
    @property
    def bgrg(self) -> UVector4: ...
    @property
    def bgrb(self) -> UVector4: ...
    @property
    def bgra(self) -> UVector4: ...
    @property
    def bgro(self) -> UVector4: ...
    @property
    def bgrl(self) -> UVector4: ...
    @property
    def bggr(self) -> UVector4: ...
    @property
    def bggg(self) -> UVector4: ...
    @property
    def bggb(self) -> UVector4: ...
    @property
    def bgga(self) -> UVector4: ...
    @property
    def bggo(self) -> UVector4: ...
    @property
    def bggl(self) -> UVector4: ...
    @property
    def bgbr(self) -> UVector4: ...
    @property
    def bgbg(self) -> UVector4: ...
    @property
    def bgbb(self) -> UVector4: ...
    @property
    def bgba(self) -> UVector4: ...
    @property
    def bgbo(self) -> UVector4: ...
    @property
    def bgbl(self) -> UVector4: ...
    @property
    def bgar(self) -> UVector4: ...
    @property
    def bgag(self) -> UVector4: ...
    @property
    def bgab(self) -> UVector4: ...
    @property
    def bgaa(self) -> UVector4: ...
    @property
    def bgao(self) -> UVector4: ...
    @property
    def bgal(self) -> UVector4: ...
    @property
    def bgor(self) -> UVector4: ...
    @property
    def bgog(self) -> UVector4: ...
    @property
    def bgob(self) -> UVector4: ...
    @property
    def bgoa(self) -> UVector4: ...
    @property
    def bgoo(self) -> UVector4: ...
    @property
    def bgol(self) -> UVector4: ...
    @property
    def bglr(self) -> UVector4: ...
    @property
    def bglg(self) -> UVector4: ...
    @property
    def bglb(self) -> UVector4: ...
    @property
    def bgla(self) -> UVector4: ...
    @property
    def bglo(self) -> UVector4: ...
    @property
    def bgll(self) -> UVector4: ...
    @property
    def bbrr(self) -> UVector4: ...
    @property
    def bbrg(self) -> UVector4: ...
    @property
    def bbrb(self) -> UVector4: ...
    @property
    def bbra(self) -> UVector4: ...
    @property
    def bbro(self) -> UVector4: ...
    @property
    def bbrl(self) -> UVector4: ...
    @property
    def bbgr(self) -> UVector4: ...
    @property
    def bbgg(self) -> UVector4: ...
    @property
    def bbgb(self) -> UVector4: ...
    @property
    def bbga(self) -> UVector4: ...
    @property
    def bbgo(self) -> UVector4: ...
    @property
    def bbgl(self) -> UVector4: ...
    @property
    def bbbr(self) -> UVector4: ...
    @property
    def bbbg(self) -> UVector4: ...
    @property
    def bbbb(self) -> UVector4: ...
    @property
    def bbba(self) -> UVector4: ...
    @property
    def bbbo(self) -> UVector4: ...
    @property
    def bbbl(self) -> UVector4: ...
    @property
    def bbar(self) -> UVector4: ...
    @property
    def bbag(self) -> UVector4: ...
    @property
    def bbab(self) -> UVector4: ...
    @property
    def bbaa(self) -> UVector4: ...
    @property
    def bbao(self) -> UVector4: ...
    @property
    def bbal(self) -> UVector4: ...
    @property
    def bbor(self) -> UVector4: ...
    @property
    def bbog(self) -> UVector4: ...
    @property
    def bbob(self) -> UVector4: ...
    @property
    def bboa(self) -> UVector4: ...
    @property
    def bboo(self) -> UVector4: ...
    @property
    def bbol(self) -> UVector4: ...
    @property
    def bblr(self) -> UVector4: ...
    @property
    def bblg(self) -> UVector4: ...
    @property
    def bblb(self) -> UVector4: ...
    @property
    def bbla(self) -> UVector4: ...
    @property
    def bblo(self) -> UVector4: ...
    @property
    def bbll(self) -> UVector4: ...
    @property
    def barr(self) -> UVector4: ...
    @property
    def barg(self) -> UVector4: ...
    @property
    def barb(self) -> UVector4: ...
    @property
    def bara(self) -> UVector4: ...
    @property
    def baro(self) -> UVector4: ...
    @property
    def barl(self) -> UVector4: ...
    @property
    def bagr(self) -> UVector4: ...
    @property
    def bagg(self) -> UVector4: ...
    @property
    def bagb(self) -> UVector4: ...
    @property
    def baga(self) -> UVector4: ...
    @property
    def bago(self) -> UVector4: ...
    @property
    def bagl(self) -> UVector4: ...
    @property
    def babr(self) -> UVector4: ...
    @property
    def babg(self) -> UVector4: ...
    @property
    def babb(self) -> UVector4: ...
    @property
    def baba(self) -> UVector4: ...
    @property
    def babo(self) -> UVector4: ...
    @property
    def babl(self) -> UVector4: ...
    @property
    def baar(self) -> UVector4: ...
    @property
    def baag(self) -> UVector4: ...
    @property
    def baab(self) -> UVector4: ...
    @property
    def baaa(self) -> UVector4: ...
    @property
    def baao(self) -> UVector4: ...
    @property
    def baal(self) -> UVector4: ...
    @property
    def baor(self) -> UVector4: ...
    @property
    def baog(self) -> UVector4: ...
    @property
    def baob(self) -> UVector4: ...
    @property
    def baoa(self) -> UVector4: ...
    @property
    def baoo(self) -> UVector4: ...
    @property
    def baol(self) -> UVector4: ...
    @property
    def balr(self) -> UVector4: ...
    @property
    def balg(self) -> UVector4: ...
    @property
    def balb(self) -> UVector4: ...
    @property
    def bala(self) -> UVector4: ...
    @property
    def balo(self) -> UVector4: ...
    @property
    def ball(self) -> UVector4: ...
    @property
    def borr(self) -> UVector4: ...
    @property
    def borg(self) -> UVector4: ...
    @property
    def borb(self) -> UVector4: ...
    @property
    def bora(self) -> UVector4: ...
    @property
    def boro(self) -> UVector4: ...
    @property
    def borl(self) -> UVector4: ...
    @property
    def bogr(self) -> UVector4: ...
    @property
    def bogg(self) -> UVector4: ...
    @property
    def bogb(self) -> UVector4: ...
    @property
    def boga(self) -> UVector4: ...
    @property
    def bogo(self) -> UVector4: ...
    @property
    def bogl(self) -> UVector4: ...
    @property
    def bobr(self) -> UVector4: ...
    @property
    def bobg(self) -> UVector4: ...
    @property
    def bobb(self) -> UVector4: ...
    @property
    def boba(self) -> UVector4: ...
    @property
    def bobo(self) -> UVector4: ...
    @property
    def bobl(self) -> UVector4: ...
    @property
    def boar(self) -> UVector4: ...
    @property
    def boag(self) -> UVector4: ...
    @property
    def boab(self) -> UVector4: ...
    @property
    def boaa(self) -> UVector4: ...
    @property
    def boao(self) -> UVector4: ...
    @property
    def boal(self) -> UVector4: ...
    @property
    def boor(self) -> UVector4: ...
    @property
    def boog(self) -> UVector4: ...
    @property
    def boob(self) -> UVector4: ...
    @property
    def booa(self) -> UVector4: ...
    @property
    def booo(self) -> UVector4: ...
    @property
    def bool(self) -> UVector4: ...
    @property
    def bolr(self) -> UVector4: ...
    @property
    def bolg(self) -> UVector4: ...
    @property
    def bolb(self) -> UVector4: ...
    @property
    def bola(self) -> UVector4: ...
    @property
    def bolo(self) -> UVector4: ...
    @property
    def boll(self) -> UVector4: ...
    @property
    def blrr(self) -> UVector4: ...
    @property
    def blrg(self) -> UVector4: ...
    @property
    def blrb(self) -> UVector4: ...
    @property
    def blra(self) -> UVector4: ...
    @property
    def blro(self) -> UVector4: ...
    @property
    def blrl(self) -> UVector4: ...
    @property
    def blgr(self) -> UVector4: ...
    @property
    def blgg(self) -> UVector4: ...
    @property
    def blgb(self) -> UVector4: ...
    @property
    def blga(self) -> UVector4: ...
    @property
    def blgo(self) -> UVector4: ...
    @property
    def blgl(self) -> UVector4: ...
    @property
    def blbr(self) -> UVector4: ...
    @property
    def blbg(self) -> UVector4: ...
    @property
    def blbb(self) -> UVector4: ...
    @property
    def blba(self) -> UVector4: ...
    @property
    def blbo(self) -> UVector4: ...
    @property
    def blbl(self) -> UVector4: ...
    @property
    def blar(self) -> UVector4: ...
    @property
    def blag(self) -> UVector4: ...
    @property
    def blab(self) -> UVector4: ...
    @property
    def blaa(self) -> UVector4: ...
    @property
    def blao(self) -> UVector4: ...
    @property
    def blal(self) -> UVector4: ...
    @property
    def blor(self) -> UVector4: ...
    @property
    def blog(self) -> UVector4: ...
    @property
    def blob(self) -> UVector4: ...
    @property
    def bloa(self) -> UVector4: ...
    @property
    def bloo(self) -> UVector4: ...
    @property
    def blol(self) -> UVector4: ...
    @property
    def bllr(self) -> UVector4: ...
    @property
    def bllg(self) -> UVector4: ...
    @property
    def bllb(self) -> UVector4: ...
    @property
    def blla(self) -> UVector4: ...
    @property
    def bllo(self) -> UVector4: ...
    @property
    def blll(self) -> UVector4: ...
    @property
    def arrr(self) -> UVector4: ...
    @property
    def arrg(self) -> UVector4: ...
    @property
    def arrb(self) -> UVector4: ...
    @property
    def arra(self) -> UVector4: ...
    @property
    def arro(self) -> UVector4: ...
    @property
    def arrl(self) -> UVector4: ...
    @property
    def argr(self) -> UVector4: ...
    @property
    def argg(self) -> UVector4: ...
    @property
    def argb(self) -> UVector4: ...
    @property
    def arga(self) -> UVector4: ...
    @property
    def argo(self) -> UVector4: ...
    @property
    def argl(self) -> UVector4: ...
    @property
    def arbr(self) -> UVector4: ...
    @property
    def arbg(self) -> UVector4: ...
    @property
    def arbb(self) -> UVector4: ...
    @property
    def arba(self) -> UVector4: ...
    @property
    def arbo(self) -> UVector4: ...
    @property
    def arbl(self) -> UVector4: ...
    @property
    def arar(self) -> UVector4: ...
    @property
    def arag(self) -> UVector4: ...
    @property
    def arab(self) -> UVector4: ...
    @property
    def araa(self) -> UVector4: ...
    @property
    def arao(self) -> UVector4: ...
    @property
    def aral(self) -> UVector4: ...
    @property
    def aror(self) -> UVector4: ...
    @property
    def arog(self) -> UVector4: ...
    @property
    def arob(self) -> UVector4: ...
    @property
    def aroa(self) -> UVector4: ...
    @property
    def aroo(self) -> UVector4: ...
    @property
    def arol(self) -> UVector4: ...
    @property
    def arlr(self) -> UVector4: ...
    @property
    def arlg(self) -> UVector4: ...
    @property
    def arlb(self) -> UVector4: ...
    @property
    def arla(self) -> UVector4: ...
    @property
    def arlo(self) -> UVector4: ...
    @property
    def arll(self) -> UVector4: ...
    @property
    def agrr(self) -> UVector4: ...
    @property
    def agrg(self) -> UVector4: ...
    @property
    def agrb(self) -> UVector4: ...
    @property
    def agra(self) -> UVector4: ...
    @property
    def agro(self) -> UVector4: ...
    @property
    def agrl(self) -> UVector4: ...
    @property
    def aggr(self) -> UVector4: ...
    @property
    def aggg(self) -> UVector4: ...
    @property
    def aggb(self) -> UVector4: ...
    @property
    def agga(self) -> UVector4: ...
    @property
    def aggo(self) -> UVector4: ...
    @property
    def aggl(self) -> UVector4: ...
    @property
    def agbr(self) -> UVector4: ...
    @property
    def agbg(self) -> UVector4: ...
    @property
    def agbb(self) -> UVector4: ...
    @property
    def agba(self) -> UVector4: ...
    @property
    def agbo(self) -> UVector4: ...
    @property
    def agbl(self) -> UVector4: ...
    @property
    def agar(self) -> UVector4: ...
    @property
    def agag(self) -> UVector4: ...
    @property
    def agab(self) -> UVector4: ...
    @property
    def agaa(self) -> UVector4: ...
    @property
    def agao(self) -> UVector4: ...
    @property
    def agal(self) -> UVector4: ...
    @property
    def agor(self) -> UVector4: ...
    @property
    def agog(self) -> UVector4: ...
    @property
    def agob(self) -> UVector4: ...
    @property
    def agoa(self) -> UVector4: ...
    @property
    def agoo(self) -> UVector4: ...
    @property
    def agol(self) -> UVector4: ...
    @property
    def aglr(self) -> UVector4: ...
    @property
    def aglg(self) -> UVector4: ...
    @property
    def aglb(self) -> UVector4: ...
    @property
    def agla(self) -> UVector4: ...
    @property
    def aglo(self) -> UVector4: ...
    @property
    def agll(self) -> UVector4: ...
    @property
    def abrr(self) -> UVector4: ...
    @property
    def abrg(self) -> UVector4: ...
    @property
    def abrb(self) -> UVector4: ...
    @property
    def abra(self) -> UVector4: ...
    @property
    def abro(self) -> UVector4: ...
    @property
    def abrl(self) -> UVector4: ...
    @property
    def abgr(self) -> UVector4: ...
    @property
    def abgg(self) -> UVector4: ...
    @property
    def abgb(self) -> UVector4: ...
    @property
    def abga(self) -> UVector4: ...
    @property
    def abgo(self) -> UVector4: ...
    @property
    def abgl(self) -> UVector4: ...
    @property
    def abbr(self) -> UVector4: ...
    @property
    def abbg(self) -> UVector4: ...
    @property
    def abbb(self) -> UVector4: ...
    @property
    def abba(self) -> UVector4: ...
    @property
    def abbo(self) -> UVector4: ...
    @property
    def abbl(self) -> UVector4: ...
    @property
    def abar(self) -> UVector4: ...
    @property
    def abag(self) -> UVector4: ...
    @property
    def abab(self) -> UVector4: ...
    @property
    def abaa(self) -> UVector4: ...
    @property
    def abao(self) -> UVector4: ...
    @property
    def abal(self) -> UVector4: ...
    @property
    def abor(self) -> UVector4: ...
    @property
    def abog(self) -> UVector4: ...
    @property
    def abob(self) -> UVector4: ...
    @property
    def aboa(self) -> UVector4: ...
    @property
    def aboo(self) -> UVector4: ...
    @property
    def abol(self) -> UVector4: ...
    @property
    def ablr(self) -> UVector4: ...
    @property
    def ablg(self) -> UVector4: ...
    @property
    def ablb(self) -> UVector4: ...
    @property
    def abla(self) -> UVector4: ...
    @property
    def ablo(self) -> UVector4: ...
    @property
    def abll(self) -> UVector4: ...
    @property
    def aarr(self) -> UVector4: ...
    @property
    def aarg(self) -> UVector4: ...
    @property
    def aarb(self) -> UVector4: ...
    @property
    def aara(self) -> UVector4: ...
    @property
    def aaro(self) -> UVector4: ...
    @property
    def aarl(self) -> UVector4: ...
    @property
    def aagr(self) -> UVector4: ...
    @property
    def aagg(self) -> UVector4: ...
    @property
    def aagb(self) -> UVector4: ...
    @property
    def aaga(self) -> UVector4: ...
    @property
    def aago(self) -> UVector4: ...
    @property
    def aagl(self) -> UVector4: ...
    @property
    def aabr(self) -> UVector4: ...
    @property
    def aabg(self) -> UVector4: ...
    @property
    def aabb(self) -> UVector4: ...
    @property
    def aaba(self) -> UVector4: ...
    @property
    def aabo(self) -> UVector4: ...
    @property
    def aabl(self) -> UVector4: ...
    @property
    def aaar(self) -> UVector4: ...
    @property
    def aaag(self) -> UVector4: ...
    @property
    def aaab(self) -> UVector4: ...
    @property
    def aaaa(self) -> UVector4: ...
    @property
    def aaao(self) -> UVector4: ...
    @property
    def aaal(self) -> UVector4: ...
    @property
    def aaor(self) -> UVector4: ...
    @property
    def aaog(self) -> UVector4: ...
    @property
    def aaob(self) -> UVector4: ...
    @property
    def aaoa(self) -> UVector4: ...
    @property
    def aaoo(self) -> UVector4: ...
    @property
    def aaol(self) -> UVector4: ...
    @property
    def aalr(self) -> UVector4: ...
    @property
    def aalg(self) -> UVector4: ...
    @property
    def aalb(self) -> UVector4: ...
    @property
    def aala(self) -> UVector4: ...
    @property
    def aalo(self) -> UVector4: ...
    @property
    def aall(self) -> UVector4: ...
    @property
    def aorr(self) -> UVector4: ...
    @property
    def aorg(self) -> UVector4: ...
    @property
    def aorb(self) -> UVector4: ...
    @property
    def aora(self) -> UVector4: ...
    @property
    def aoro(self) -> UVector4: ...
    @property
    def aorl(self) -> UVector4: ...
    @property
    def aogr(self) -> UVector4: ...
    @property
    def aogg(self) -> UVector4: ...
    @property
    def aogb(self) -> UVector4: ...
    @property
    def aoga(self) -> UVector4: ...
    @property
    def aogo(self) -> UVector4: ...
    @property
    def aogl(self) -> UVector4: ...
    @property
    def aobr(self) -> UVector4: ...
    @property
    def aobg(self) -> UVector4: ...
    @property
    def aobb(self) -> UVector4: ...
    @property
    def aoba(self) -> UVector4: ...
    @property
    def aobo(self) -> UVector4: ...
    @property
    def aobl(self) -> UVector4: ...
    @property
    def aoar(self) -> UVector4: ...
    @property
    def aoag(self) -> UVector4: ...
    @property
    def aoab(self) -> UVector4: ...
    @property
    def aoaa(self) -> UVector4: ...
    @property
    def aoao(self) -> UVector4: ...
    @property
    def aoal(self) -> UVector4: ...
    @property
    def aoor(self) -> UVector4: ...
    @property
    def aoog(self) -> UVector4: ...
    @property
    def aoob(self) -> UVector4: ...
    @property
    def aooa(self) -> UVector4: ...
    @property
    def aooo(self) -> UVector4: ...
    @property
    def aool(self) -> UVector4: ...
    @property
    def aolr(self) -> UVector4: ...
    @property
    def aolg(self) -> UVector4: ...
    @property
    def aolb(self) -> UVector4: ...
    @property
    def aola(self) -> UVector4: ...
    @property
    def aolo(self) -> UVector4: ...
    @property
    def aoll(self) -> UVector4: ...
    @property
    def alrr(self) -> UVector4: ...
    @property
    def alrg(self) -> UVector4: ...
    @property
    def alrb(self) -> UVector4: ...
    @property
    def alra(self) -> UVector4: ...
    @property
    def alro(self) -> UVector4: ...
    @property
    def alrl(self) -> UVector4: ...
    @property
    def algr(self) -> UVector4: ...
    @property
    def algg(self) -> UVector4: ...
    @property
    def algb(self) -> UVector4: ...
    @property
    def alga(self) -> UVector4: ...
    @property
    def algo(self) -> UVector4: ...
    @property
    def algl(self) -> UVector4: ...
    @property
    def albr(self) -> UVector4: ...
    @property
    def albg(self) -> UVector4: ...
    @property
    def albb(self) -> UVector4: ...
    @property
    def alba(self) -> UVector4: ...
    @property
    def albo(self) -> UVector4: ...
    @property
    def albl(self) -> UVector4: ...
    @property
    def alar(self) -> UVector4: ...
    @property
    def alag(self) -> UVector4: ...
    @property
    def alab(self) -> UVector4: ...
    @property
    def alaa(self) -> UVector4: ...
    @property
    def alao(self) -> UVector4: ...
    @property
    def alal(self) -> UVector4: ...
    @property
    def alor(self) -> UVector4: ...
    @property
    def alog(self) -> UVector4: ...
    @property
    def alob(self) -> UVector4: ...
    @property
    def aloa(self) -> UVector4: ...
    @property
    def aloo(self) -> UVector4: ...
    @property
    def alol(self) -> UVector4: ...
    @property
    def allr(self) -> UVector4: ...
    @property
    def allg(self) -> UVector4: ...
    @property
    def allb(self) -> UVector4: ...
    @property
    def alla(self) -> UVector4: ...
    @property
    def allo(self) -> UVector4: ...
    @property
    def alll(self) -> UVector4: ...
    @property
    def orrr(self) -> UVector4: ...
    @property
    def orrg(self) -> UVector4: ...
    @property
    def orrb(self) -> UVector4: ...
    @property
    def orra(self) -> UVector4: ...
    @property
    def orro(self) -> UVector4: ...
    @property
    def orrl(self) -> UVector4: ...
    @property
    def orgr(self) -> UVector4: ...
    @property
    def orgg(self) -> UVector4: ...
    @property
    def orgb(self) -> UVector4: ...
    @property
    def orga(self) -> UVector4: ...
    @property
    def orgo(self) -> UVector4: ...
    @property
    def orgl(self) -> UVector4: ...
    @property
    def orbr(self) -> UVector4: ...
    @property
    def orbg(self) -> UVector4: ...
    @property
    def orbb(self) -> UVector4: ...
    @property
    def orba(self) -> UVector4: ...
    @property
    def orbo(self) -> UVector4: ...
    @property
    def orbl(self) -> UVector4: ...
    @property
    def orar(self) -> UVector4: ...
    @property
    def orag(self) -> UVector4: ...
    @property
    def orab(self) -> UVector4: ...
    @property
    def oraa(self) -> UVector4: ...
    @property
    def orao(self) -> UVector4: ...
    @property
    def oral(self) -> UVector4: ...
    @property
    def oror(self) -> UVector4: ...
    @property
    def orog(self) -> UVector4: ...
    @property
    def orob(self) -> UVector4: ...
    @property
    def oroa(self) -> UVector4: ...
    @property
    def oroo(self) -> UVector4: ...
    @property
    def orol(self) -> UVector4: ...
    @property
    def orlr(self) -> UVector4: ...
    @property
    def orlg(self) -> UVector4: ...
    @property
    def orlb(self) -> UVector4: ...
    @property
    def orla(self) -> UVector4: ...
    @property
    def orlo(self) -> UVector4: ...
    @property
    def orll(self) -> UVector4: ...
    @property
    def ogrr(self) -> UVector4: ...
    @property
    def ogrg(self) -> UVector4: ...
    @property
    def ogrb(self) -> UVector4: ...
    @property
    def ogra(self) -> UVector4: ...
    @property
    def ogro(self) -> UVector4: ...
    @property
    def ogrl(self) -> UVector4: ...
    @property
    def oggr(self) -> UVector4: ...
    @property
    def oggg(self) -> UVector4: ...
    @property
    def oggb(self) -> UVector4: ...
    @property
    def ogga(self) -> UVector4: ...
    @property
    def oggo(self) -> UVector4: ...
    @property
    def oggl(self) -> UVector4: ...
    @property
    def ogbr(self) -> UVector4: ...
    @property
    def ogbg(self) -> UVector4: ...
    @property
    def ogbb(self) -> UVector4: ...
    @property
    def ogba(self) -> UVector4: ...
    @property
    def ogbo(self) -> UVector4: ...
    @property
    def ogbl(self) -> UVector4: ...
    @property
    def ogar(self) -> UVector4: ...
    @property
    def ogag(self) -> UVector4: ...
    @property
    def ogab(self) -> UVector4: ...
    @property
    def ogaa(self) -> UVector4: ...
    @property
    def ogao(self) -> UVector4: ...
    @property
    def ogal(self) -> UVector4: ...
    @property
    def ogor(self) -> UVector4: ...
    @property
    def ogog(self) -> UVector4: ...
    @property
    def ogob(self) -> UVector4: ...
    @property
    def ogoa(self) -> UVector4: ...
    @property
    def ogoo(self) -> UVector4: ...
    @property
    def ogol(self) -> UVector4: ...
    @property
    def oglr(self) -> UVector4: ...
    @property
    def oglg(self) -> UVector4: ...
    @property
    def oglb(self) -> UVector4: ...
    @property
    def ogla(self) -> UVector4: ...
    @property
    def oglo(self) -> UVector4: ...
    @property
    def ogll(self) -> UVector4: ...
    @property
    def obrr(self) -> UVector4: ...
    @property
    def obrg(self) -> UVector4: ...
    @property
    def obrb(self) -> UVector4: ...
    @property
    def obra(self) -> UVector4: ...
    @property
    def obro(self) -> UVector4: ...
    @property
    def obrl(self) -> UVector4: ...
    @property
    def obgr(self) -> UVector4: ...
    @property
    def obgg(self) -> UVector4: ...
    @property
    def obgb(self) -> UVector4: ...
    @property
    def obga(self) -> UVector4: ...
    @property
    def obgo(self) -> UVector4: ...
    @property
    def obgl(self) -> UVector4: ...
    @property
    def obbr(self) -> UVector4: ...
    @property
    def obbg(self) -> UVector4: ...
    @property
    def obbb(self) -> UVector4: ...
    @property
    def obba(self) -> UVector4: ...
    @property
    def obbo(self) -> UVector4: ...
    @property
    def obbl(self) -> UVector4: ...
    @property
    def obar(self) -> UVector4: ...
    @property
    def obag(self) -> UVector4: ...
    @property
    def obab(self) -> UVector4: ...
    @property
    def obaa(self) -> UVector4: ...
    @property
    def obao(self) -> UVector4: ...
    @property
    def obal(self) -> UVector4: ...
    @property
    def obor(self) -> UVector4: ...
    @property
    def obog(self) -> UVector4: ...
    @property
    def obob(self) -> UVector4: ...
    @property
    def oboa(self) -> UVector4: ...
    @property
    def oboo(self) -> UVector4: ...
    @property
    def obol(self) -> UVector4: ...
    @property
    def oblr(self) -> UVector4: ...
    @property
    def oblg(self) -> UVector4: ...
    @property
    def oblb(self) -> UVector4: ...
    @property
    def obla(self) -> UVector4: ...
    @property
    def oblo(self) -> UVector4: ...
    @property
    def obll(self) -> UVector4: ...
    @property
    def oarr(self) -> UVector4: ...
    @property
    def oarg(self) -> UVector4: ...
    @property
    def oarb(self) -> UVector4: ...
    @property
    def oara(self) -> UVector4: ...
    @property
    def oaro(self) -> UVector4: ...
    @property
    def oarl(self) -> UVector4: ...
    @property
    def oagr(self) -> UVector4: ...
    @property
    def oagg(self) -> UVector4: ...
    @property
    def oagb(self) -> UVector4: ...
    @property
    def oaga(self) -> UVector4: ...
    @property
    def oago(self) -> UVector4: ...
    @property
    def oagl(self) -> UVector4: ...
    @property
    def oabr(self) -> UVector4: ...
    @property
    def oabg(self) -> UVector4: ...
    @property
    def oabb(self) -> UVector4: ...
    @property
    def oaba(self) -> UVector4: ...
    @property
    def oabo(self) -> UVector4: ...
    @property
    def oabl(self) -> UVector4: ...
    @property
    def oaar(self) -> UVector4: ...
    @property
    def oaag(self) -> UVector4: ...
    @property
    def oaab(self) -> UVector4: ...
    @property
    def oaaa(self) -> UVector4: ...
    @property
    def oaao(self) -> UVector4: ...
    @property
    def oaal(self) -> UVector4: ...
    @property
    def oaor(self) -> UVector4: ...
    @property
    def oaog(self) -> UVector4: ...
    @property
    def oaob(self) -> UVector4: ...
    @property
    def oaoa(self) -> UVector4: ...
    @property
    def oaoo(self) -> UVector4: ...
    @property
    def oaol(self) -> UVector4: ...
    @property
    def oalr(self) -> UVector4: ...
    @property
    def oalg(self) -> UVector4: ...
    @property
    def oalb(self) -> UVector4: ...
    @property
    def oala(self) -> UVector4: ...
    @property
    def oalo(self) -> UVector4: ...
    @property
    def oall(self) -> UVector4: ...
    @property
    def oorr(self) -> UVector4: ...
    @property
    def oorg(self) -> UVector4: ...
    @property
    def oorb(self) -> UVector4: ...
    @property
    def oora(self) -> UVector4: ...
    @property
    def ooro(self) -> UVector4: ...
    @property
    def oorl(self) -> UVector4: ...
    @property
    def oogr(self) -> UVector4: ...
    @property
    def oogg(self) -> UVector4: ...
    @property
    def oogb(self) -> UVector4: ...
    @property
    def ooga(self) -> UVector4: ...
    @property
    def oogo(self) -> UVector4: ...
    @property
    def oogl(self) -> UVector4: ...
    @property
    def oobr(self) -> UVector4: ...
    @property
    def oobg(self) -> UVector4: ...
    @property
    def oobb(self) -> UVector4: ...
    @property
    def ooba(self) -> UVector4: ...
    @property
    def oobo(self) -> UVector4: ...
    @property
    def oobl(self) -> UVector4: ...
    @property
    def ooar(self) -> UVector4: ...
    @property
    def ooag(self) -> UVector4: ...
    @property
    def ooab(self) -> UVector4: ...
    @property
    def ooaa(self) -> UVector4: ...
    @property
    def ooao(self) -> UVector4: ...
    @property
    def ooal(self) -> UVector4: ...
    @property
    def ooor(self) -> UVector4: ...
    @property
    def ooog(self) -> UVector4: ...
    @property
    def ooob(self) -> UVector4: ...
    @property
    def oooa(self) -> UVector4: ...
    @property
    def oolr(self) -> UVector4: ...
    @property
    def oolg(self) -> UVector4: ...
    @property
    def oolb(self) -> UVector4: ...
    @property
    def oola(self) -> UVector4: ...
    @property
    def olrr(self) -> UVector4: ...
    @property
    def olrg(self) -> UVector4: ...
    @property
    def olrb(self) -> UVector4: ...
    @property
    def olra(self) -> UVector4: ...
    @property
    def olro(self) -> UVector4: ...
    @property
    def olrl(self) -> UVector4: ...
    @property
    def olgr(self) -> UVector4: ...
    @property
    def olgg(self) -> UVector4: ...
    @property
    def olgb(self) -> UVector4: ...
    @property
    def olga(self) -> UVector4: ...
    @property
    def olgo(self) -> UVector4: ...
    @property
    def olgl(self) -> UVector4: ...
    @property
    def olbr(self) -> UVector4: ...
    @property
    def olbg(self) -> UVector4: ...
    @property
    def olbb(self) -> UVector4: ...
    @property
    def olba(self) -> UVector4: ...
    @property
    def olbo(self) -> UVector4: ...
    @property
    def olbl(self) -> UVector4: ...
    @property
    def olar(self) -> UVector4: ...
    @property
    def olag(self) -> UVector4: ...
    @property
    def olab(self) -> UVector4: ...
    @property
    def olaa(self) -> UVector4: ...
    @property
    def olao(self) -> UVector4: ...
    @property
    def olal(self) -> UVector4: ...
    @property
    def olor(self) -> UVector4: ...
    @property
    def olog(self) -> UVector4: ...
    @property
    def olob(self) -> UVector4: ...
    @property
    def oloa(self) -> UVector4: ...
    @property
    def ollr(self) -> UVector4: ...
    @property
    def ollg(self) -> UVector4: ...
    @property
    def ollb(self) -> UVector4: ...
    @property
    def olla(self) -> UVector4: ...
    @property
    def lrrr(self) -> UVector4: ...
    @property
    def lrrg(self) -> UVector4: ...
    @property
    def lrrb(self) -> UVector4: ...
    @property
    def lrra(self) -> UVector4: ...
    @property
    def lrro(self) -> UVector4: ...
    @property
    def lrrl(self) -> UVector4: ...
    @property
    def lrgr(self) -> UVector4: ...
    @property
    def lrgg(self) -> UVector4: ...
    @property
    def lrgb(self) -> UVector4: ...
    @property
    def lrga(self) -> UVector4: ...
    @property
    def lrgo(self) -> UVector4: ...
    @property
    def lrgl(self) -> UVector4: ...
    @property
    def lrbr(self) -> UVector4: ...
    @property
    def lrbg(self) -> UVector4: ...
    @property
    def lrbb(self) -> UVector4: ...
    @property
    def lrba(self) -> UVector4: ...
    @property
    def lrbo(self) -> UVector4: ...
    @property
    def lrbl(self) -> UVector4: ...
    @property
    def lrar(self) -> UVector4: ...
    @property
    def lrag(self) -> UVector4: ...
    @property
    def lrab(self) -> UVector4: ...
    @property
    def lraa(self) -> UVector4: ...
    @property
    def lrao(self) -> UVector4: ...
    @property
    def lral(self) -> UVector4: ...
    @property
    def lror(self) -> UVector4: ...
    @property
    def lrog(self) -> UVector4: ...
    @property
    def lrob(self) -> UVector4: ...
    @property
    def lroa(self) -> UVector4: ...
    @property
    def lroo(self) -> UVector4: ...
    @property
    def lrol(self) -> UVector4: ...
    @property
    def lrlr(self) -> UVector4: ...
    @property
    def lrlg(self) -> UVector4: ...
    @property
    def lrlb(self) -> UVector4: ...
    @property
    def lrla(self) -> UVector4: ...
    @property
    def lrlo(self) -> UVector4: ...
    @property
    def lrll(self) -> UVector4: ...
    @property
    def lgrr(self) -> UVector4: ...
    @property
    def lgrg(self) -> UVector4: ...
    @property
    def lgrb(self) -> UVector4: ...
    @property
    def lgra(self) -> UVector4: ...
    @property
    def lgro(self) -> UVector4: ...
    @property
    def lgrl(self) -> UVector4: ...
    @property
    def lggr(self) -> UVector4: ...
    @property
    def lggg(self) -> UVector4: ...
    @property
    def lggb(self) -> UVector4: ...
    @property
    def lgga(self) -> UVector4: ...
    @property
    def lggo(self) -> UVector4: ...
    @property
    def lggl(self) -> UVector4: ...
    @property
    def lgbr(self) -> UVector4: ...
    @property
    def lgbg(self) -> UVector4: ...
    @property
    def lgbb(self) -> UVector4: ...
    @property
    def lgba(self) -> UVector4: ...
    @property
    def lgbo(self) -> UVector4: ...
    @property
    def lgbl(self) -> UVector4: ...
    @property
    def lgar(self) -> UVector4: ...
    @property
    def lgag(self) -> UVector4: ...
    @property
    def lgab(self) -> UVector4: ...
    @property
    def lgaa(self) -> UVector4: ...
    @property
    def lgao(self) -> UVector4: ...
    @property
    def lgal(self) -> UVector4: ...
    @property
    def lgor(self) -> UVector4: ...
    @property
    def lgog(self) -> UVector4: ...
    @property
    def lgob(self) -> UVector4: ...
    @property
    def lgoa(self) -> UVector4: ...
    @property
    def lgoo(self) -> UVector4: ...
    @property
    def lgol(self) -> UVector4: ...
    @property
    def lglr(self) -> UVector4: ...
    @property
    def lglg(self) -> UVector4: ...
    @property
    def lglb(self) -> UVector4: ...
    @property
    def lgla(self) -> UVector4: ...
    @property
    def lglo(self) -> UVector4: ...
    @property
    def lgll(self) -> UVector4: ...
    @property
    def lbrr(self) -> UVector4: ...
    @property
    def lbrg(self) -> UVector4: ...
    @property
    def lbrb(self) -> UVector4: ...
    @property
    def lbra(self) -> UVector4: ...
    @property
    def lbro(self) -> UVector4: ...
    @property
    def lbrl(self) -> UVector4: ...
    @property
    def lbgr(self) -> UVector4: ...
    @property
    def lbgg(self) -> UVector4: ...
    @property
    def lbgb(self) -> UVector4: ...
    @property
    def lbga(self) -> UVector4: ...
    @property
    def lbgo(self) -> UVector4: ...
    @property
    def lbgl(self) -> UVector4: ...
    @property
    def lbbr(self) -> UVector4: ...
    @property
    def lbbg(self) -> UVector4: ...
    @property
    def lbbb(self) -> UVector4: ...
    @property
    def lbba(self) -> UVector4: ...
    @property
    def lbbo(self) -> UVector4: ...
    @property
    def lbbl(self) -> UVector4: ...
    @property
    def lbar(self) -> UVector4: ...
    @property
    def lbag(self) -> UVector4: ...
    @property
    def lbab(self) -> UVector4: ...
    @property
    def lbaa(self) -> UVector4: ...
    @property
    def lbao(self) -> UVector4: ...
    @property
    def lbal(self) -> UVector4: ...
    @property
    def lbor(self) -> UVector4: ...
    @property
    def lbog(self) -> UVector4: ...
    @property
    def lbob(self) -> UVector4: ...
    @property
    def lboa(self) -> UVector4: ...
    @property
    def lboo(self) -> UVector4: ...
    @property
    def lbol(self) -> UVector4: ...
    @property
    def lblr(self) -> UVector4: ...
    @property
    def lblg(self) -> UVector4: ...
    @property
    def lblb(self) -> UVector4: ...
    @property
    def lbla(self) -> UVector4: ...
    @property
    def lblo(self) -> UVector4: ...
    @property
    def lbll(self) -> UVector4: ...
    @property
    def larr(self) -> UVector4: ...
    @property
    def larg(self) -> UVector4: ...
    @property
    def larb(self) -> UVector4: ...
    @property
    def lara(self) -> UVector4: ...
    @property
    def laro(self) -> UVector4: ...
    @property
    def larl(self) -> UVector4: ...
    @property
    def lagr(self) -> UVector4: ...
    @property
    def lagg(self) -> UVector4: ...
    @property
    def lagb(self) -> UVector4: ...
    @property
    def laga(self) -> UVector4: ...
    @property
    def lago(self) -> UVector4: ...
    @property
    def lagl(self) -> UVector4: ...
    @property
    def labr(self) -> UVector4: ...
    @property
    def labg(self) -> UVector4: ...
    @property
    def labb(self) -> UVector4: ...
    @property
    def laba(self) -> UVector4: ...
    @property
    def labo(self) -> UVector4: ...
    @property
    def labl(self) -> UVector4: ...
    @property
    def laar(self) -> UVector4: ...
    @property
    def laag(self) -> UVector4: ...
    @property
    def laab(self) -> UVector4: ...
    @property
    def laaa(self) -> UVector4: ...
    @property
    def laao(self) -> UVector4: ...
    @property
    def laal(self) -> UVector4: ...
    @property
    def laor(self) -> UVector4: ...
    @property
    def laog(self) -> UVector4: ...
    @property
    def laob(self) -> UVector4: ...
    @property
    def laoa(self) -> UVector4: ...
    @property
    def laoo(self) -> UVector4: ...
    @property
    def laol(self) -> UVector4: ...
    @property
    def lalr(self) -> UVector4: ...
    @property
    def lalg(self) -> UVector4: ...
    @property
    def lalb(self) -> UVector4: ...
    @property
    def lala(self) -> UVector4: ...
    @property
    def lalo(self) -> UVector4: ...
    @property
    def lall(self) -> UVector4: ...
    @property
    def lorr(self) -> UVector4: ...
    @property
    def lorg(self) -> UVector4: ...
    @property
    def lorb(self) -> UVector4: ...
    @property
    def lora(self) -> UVector4: ...
    @property
    def loro(self) -> UVector4: ...
    @property
    def lorl(self) -> UVector4: ...
    @property
    def logr(self) -> UVector4: ...
    @property
    def logg(self) -> UVector4: ...
    @property
    def logb(self) -> UVector4: ...
    @property
    def loga(self) -> UVector4: ...
    @property
    def logo(self) -> UVector4: ...
    @property
    def logl(self) -> UVector4: ...
    @property
    def lobr(self) -> UVector4: ...
    @property
    def lobg(self) -> UVector4: ...
    @property
    def lobb(self) -> UVector4: ...
    @property
    def loba(self) -> UVector4: ...
    @property
    def lobo(self) -> UVector4: ...
    @property
    def lobl(self) -> UVector4: ...
    @property
    def loar(self) -> UVector4: ...
    @property
    def loag(self) -> UVector4: ...
    @property
    def loab(self) -> UVector4: ...
    @property
    def loaa(self) -> UVector4: ...
    @property
    def loao(self) -> UVector4: ...
    @property
    def loal(self) -> UVector4: ...
    @property
    def loor(self) -> UVector4: ...
    @property
    def loog(self) -> UVector4: ...
    @property
    def loob(self) -> UVector4: ...
    @property
    def looa(self) -> UVector4: ...
    @property
    def lolr(self) -> UVector4: ...
    @property
    def lolg(self) -> UVector4: ...
    @property
    def lolb(self) -> UVector4: ...
    @property
    def lola(self) -> UVector4: ...
    @property
    def llrr(self) -> UVector4: ...
    @property
    def llrg(self) -> UVector4: ...
    @property
    def llrb(self) -> UVector4: ...
    @property
    def llra(self) -> UVector4: ...
    @property
    def llro(self) -> UVector4: ...
    @property
    def llrl(self) -> UVector4: ...
    @property
    def llgr(self) -> UVector4: ...
    @property
    def llgg(self) -> UVector4: ...
    @property
    def llgb(self) -> UVector4: ...
    @property
    def llga(self) -> UVector4: ...
    @property
    def llgo(self) -> UVector4: ...
    @property
    def llgl(self) -> UVector4: ...
    @property
    def llbr(self) -> UVector4: ...
    @property
    def llbg(self) -> UVector4: ...
    @property
    def llbb(self) -> UVector4: ...
    @property
    def llba(self) -> UVector4: ...
    @property
    def llbo(self) -> UVector4: ...
    @property
    def llbl(self) -> UVector4: ...
    @property
    def llar(self) -> UVector4: ...
    @property
    def llag(self) -> UVector4: ...
    @property
    def llab(self) -> UVector4: ...
    @property
    def llaa(self) -> UVector4: ...
    @property
    def llao(self) -> UVector4: ...
    @property
    def llal(self) -> UVector4: ...
    @property
    def llor(self) -> UVector4: ...
    @property
    def llog(self) -> UVector4: ...
    @property
    def llob(self) -> UVector4: ...
    @property
    def lloa(self) -> UVector4: ...
    @property
    def lllr(self) -> UVector4: ...
    @property
    def lllg(self) -> UVector4: ...
    @property
    def lllb(self) -> UVector4: ...
    @property
    def llla(self) -> UVector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> UVector2: ...
    @property
    def st(self) -> UVector2: ...
    @property
    def sq(self) -> UVector2: ...
    @property
    def sp(self) -> UVector2: ...
    @property
    def so(self) -> UVector2: ...
    @property
    def sl(self) -> UVector2: ...
    @property
    def ts(self) -> UVector2: ...
    @property
    def tt(self) -> UVector2: ...
    @property
    def tq(self) -> UVector2: ...
    @property
    def tp(self) -> UVector2: ...
    @property
    def to(self) -> UVector2: ...
    @property
    def tl(self) -> UVector2: ...
    @property
    def qs(self) -> UVector2: ...
    @property
    def qt(self) -> UVector2: ...
    @property
    def qq(self) -> UVector2: ...
    @property
    def qp(self) -> UVector2: ...
    @property
    def qo(self) -> UVector2: ...
    @property
    def ql(self) -> UVector2: ...
    @property
    def ps(self) -> UVector2: ...
    @property
    def pt(self) -> UVector2: ...
    @property
    def pq(self) -> UVector2: ...
    @property
    def pp(self) -> UVector2: ...
    @property
    def po(self) -> UVector2: ...
    @property
    def pl(self) -> UVector2: ...
    @property
    def os(self) -> UVector2: ...
    @property
    def ot(self) -> UVector2: ...
    @property
    def oq(self) -> UVector2: ...
    @property
    def op(self) -> UVector2: ...
    @property
    def ls(self) -> UVector2: ...
    @property
    def lt(self) -> UVector2: ...
    @property
    def lq(self) -> UVector2: ...
    @property
    def lp(self) -> UVector2: ...
    @property
    def sss(self) -> UVector3: ...
    @property
    def sst(self) -> UVector3: ...
    @property
    def ssq(self) -> UVector3: ...
    @property
    def ssp(self) -> UVector3: ...
    @property
    def sso(self) -> UVector3: ...
    @property
    def ssl(self) -> UVector3: ...
    @property
    def sts(self) -> UVector3: ...
    @property
    def stt(self) -> UVector3: ...
    @property
    def stq(self) -> UVector3: ...
    @property
    def stp(self) -> UVector3: ...
    @property
    def sto(self) -> UVector3: ...
    @property
    def stl(self) -> UVector3: ...
    @property
    def sqs(self) -> UVector3: ...
    @property
    def sqt(self) -> UVector3: ...
    @property
    def sqq(self) -> UVector3: ...
    @property
    def sqp(self) -> UVector3: ...
    @property
    def sqo(self) -> UVector3: ...
    @property
    def sql(self) -> UVector3: ...
    @property
    def sps(self) -> UVector3: ...
    @property
    def spt(self) -> UVector3: ...
    @property
    def spq(self) -> UVector3: ...
    @property
    def spp(self) -> UVector3: ...
    @property
    def spo(self) -> UVector3: ...
    @property
    def spl(self) -> UVector3: ...
    @property
    def sos(self) -> UVector3: ...
    @property
    def sot(self) -> UVector3: ...
    @property
    def soq(self) -> UVector3: ...
    @property
    def sop(self) -> UVector3: ...
    @property
    def soo(self) -> UVector3: ...
    @property
    def sol(self) -> UVector3: ...
    @property
    def sls(self) -> UVector3: ...
    @property
    def slt(self) -> UVector3: ...
    @property
    def slq(self) -> UVector3: ...
    @property
    def slp(self) -> UVector3: ...
    @property
    def slo(self) -> UVector3: ...
    @property
    def sll(self) -> UVector3: ...
    @property
    def tss(self) -> UVector3: ...
    @property
    def tst(self) -> UVector3: ...
    @property
    def tsq(self) -> UVector3: ...
    @property
    def tsp(self) -> UVector3: ...
    @property
    def tso(self) -> UVector3: ...
    @property
    def tsl(self) -> UVector3: ...
    @property
    def tts(self) -> UVector3: ...
    @property
    def ttt(self) -> UVector3: ...
    @property
    def ttq(self) -> UVector3: ...
    @property
    def ttp(self) -> UVector3: ...
    @property
    def tto(self) -> UVector3: ...
    @property
    def ttl(self) -> UVector3: ...
    @property
    def tqs(self) -> UVector3: ...
    @property
    def tqt(self) -> UVector3: ...
    @property
    def tqq(self) -> UVector3: ...
    @property
    def tqp(self) -> UVector3: ...
    @property
    def tqo(self) -> UVector3: ...
    @property
    def tql(self) -> UVector3: ...
    @property
    def tps(self) -> UVector3: ...
    @property
    def tpt(self) -> UVector3: ...
    @property
    def tpq(self) -> UVector3: ...
    @property
    def tpp(self) -> UVector3: ...
    @property
    def tpo(self) -> UVector3: ...
    @property
    def tpl(self) -> UVector3: ...
    @property
    def tos(self) -> UVector3: ...
    @property
    def tot(self) -> UVector3: ...
    @property
    def toq(self) -> UVector3: ...
    @property
    def top(self) -> UVector3: ...
    @property
    def too(self) -> UVector3: ...
    @property
    def tol(self) -> UVector3: ...
    @property
    def tls(self) -> UVector3: ...
    @property
    def tlt(self) -> UVector3: ...
    @property
    def tlq(self) -> UVector3: ...
    @property
    def tlp(self) -> UVector3: ...
    @property
    def tlo(self) -> UVector3: ...
    @property
    def tll(self) -> UVector3: ...
    @property
    def qss(self) -> UVector3: ...
    @property
    def qst(self) -> UVector3: ...
    @property
    def qsq(self) -> UVector3: ...
    @property
    def qsp(self) -> UVector3: ...
    @property
    def qso(self) -> UVector3: ...
    @property
    def qsl(self) -> UVector3: ...
    @property
    def qts(self) -> UVector3: ...
    @property
    def qtt(self) -> UVector3: ...
    @property
    def qtq(self) -> UVector3: ...
    @property
    def qtp(self) -> UVector3: ...
    @property
    def qto(self) -> UVector3: ...
    @property
    def qtl(self) -> UVector3: ...
    @property
    def qqs(self) -> UVector3: ...
    @property
    def qqt(self) -> UVector3: ...
    @property
    def qqq(self) -> UVector3: ...
    @property
    def qqp(self) -> UVector3: ...
    @property
    def qqo(self) -> UVector3: ...
    @property
    def qql(self) -> UVector3: ...
    @property
    def qps(self) -> UVector3: ...
    @property
    def qpt(self) -> UVector3: ...
    @property
    def qpq(self) -> UVector3: ...
    @property
    def qpp(self) -> UVector3: ...
    @property
    def qpo(self) -> UVector3: ...
    @property
    def qpl(self) -> UVector3: ...
    @property
    def qos(self) -> UVector3: ...
    @property
    def qot(self) -> UVector3: ...
    @property
    def qoq(self) -> UVector3: ...
    @property
    def qop(self) -> UVector3: ...
    @property
    def qoo(self) -> UVector3: ...
    @property
    def qol(self) -> UVector3: ...
    @property
    def qls(self) -> UVector3: ...
    @property
    def qlt(self) -> UVector3: ...
    @property
    def qlq(self) -> UVector3: ...
    @property
    def qlp(self) -> UVector3: ...
    @property
    def qlo(self) -> UVector3: ...
    @property
    def qll(self) -> UVector3: ...
    @property
    def pss(self) -> UVector3: ...
    @property
    def pst(self) -> UVector3: ...
    @property
    def psq(self) -> UVector3: ...
    @property
    def psp(self) -> UVector3: ...
    @property
    def pso(self) -> UVector3: ...
    @property
    def psl(self) -> UVector3: ...
    @property
    def pts(self) -> UVector3: ...
    @property
    def ptt(self) -> UVector3: ...
    @property
    def ptq(self) -> UVector3: ...
    @property
    def ptp(self) -> UVector3: ...
    @property
    def pto(self) -> UVector3: ...
    @property
    def ptl(self) -> UVector3: ...
    @property
    def pqs(self) -> UVector3: ...
    @property
    def pqt(self) -> UVector3: ...
    @property
    def pqq(self) -> UVector3: ...
    @property
    def pqp(self) -> UVector3: ...
    @property
    def pqo(self) -> UVector3: ...
    @property
    def pql(self) -> UVector3: ...
    @property
    def pps(self) -> UVector3: ...
    @property
    def ppt(self) -> UVector3: ...
    @property
    def ppq(self) -> UVector3: ...
    @property
    def ppp(self) -> UVector3: ...
    @property
    def ppo(self) -> UVector3: ...
    @property
    def ppl(self) -> UVector3: ...
    @property
    def pos(self) -> UVector3: ...
    @property
    def pot(self) -> UVector3: ...
    @property
    def poq(self) -> UVector3: ...
    @property
    def pop(self) -> UVector3: ...
    @property
    def poo(self) -> UVector3: ...
    @property
    def pol(self) -> UVector3: ...
    @property
    def pls(self) -> UVector3: ...
    @property
    def plt(self) -> UVector3: ...
    @property
    def plq(self) -> UVector3: ...
    @property
    def plp(self) -> UVector3: ...
    @property
    def plo(self) -> UVector3: ...
    @property
    def pll(self) -> UVector3: ...
    @property
    def oss(self) -> UVector3: ...
    @property
    def ost(self) -> UVector3: ...
    @property
    def osq(self) -> UVector3: ...
    @property
    def osp(self) -> UVector3: ...
    @property
    def oso(self) -> UVector3: ...
    @property
    def osl(self) -> UVector3: ...
    @property
    def ots(self) -> UVector3: ...
    @property
    def ott(self) -> UVector3: ...
    @property
    def otq(self) -> UVector3: ...
    @property
    def otp(self) -> UVector3: ...
    @property
    def oto(self) -> UVector3: ...
    @property
    def otl(self) -> UVector3: ...
    @property
    def oqs(self) -> UVector3: ...
    @property
    def oqt(self) -> UVector3: ...
    @property
    def oqq(self) -> UVector3: ...
    @property
    def oqp(self) -> UVector3: ...
    @property
    def oqo(self) -> UVector3: ...
    @property
    def oql(self) -> UVector3: ...
    @property
    def ops(self) -> UVector3: ...
    @property
    def opt(self) -> UVector3: ...
    @property
    def opq(self) -> UVector3: ...
    @property
    def opp(self) -> UVector3: ...
    @property
    def opo(self) -> UVector3: ...
    @property
    def opl(self) -> UVector3: ...
    @property
    def oos(self) -> UVector3: ...
    @property
    def oot(self) -> UVector3: ...
    @property
    def ooq(self) -> UVector3: ...
    @property
    def oop(self) -> UVector3: ...
    @property
    def ols(self) -> UVector3: ...
    @property
    def olt(self) -> UVector3: ...
    @property
    def olq(self) -> UVector3: ...
    @property
    def olp(self) -> UVector3: ...
    @property
    def lss(self) -> UVector3: ...
    @property
    def lst(self) -> UVector3: ...
    @property
    def lsq(self) -> UVector3: ...
    @property
    def lsp(self) -> UVector3: ...
    @property
    def lso(self) -> UVector3: ...
    @property
    def lsl(self) -> UVector3: ...
    @property
    def lts(self) -> UVector3: ...
    @property
    def ltt(self) -> UVector3: ...
    @property
    def ltq(self) -> UVector3: ...
    @property
    def ltp(self) -> UVector3: ...
    @property
    def lto(self) -> UVector3: ...
    @property
    def ltl(self) -> UVector3: ...
    @property
    def lqs(self) -> UVector3: ...
    @property
    def lqt(self) -> UVector3: ...
    @property
    def lqq(self) -> UVector3: ...
    @property
    def lqp(self) -> UVector3: ...
    @property
    def lqo(self) -> UVector3: ...
    @property
    def lql(self) -> UVector3: ...
    @property
    def lps(self) -> UVector3: ...
    @property
    def lpt(self) -> UVector3: ...
    @property
    def lpq(self) -> UVector3: ...
    @property
    def lpp(self) -> UVector3: ...
    @property
    def lpo(self) -> UVector3: ...
    @property
    def lpl(self) -> UVector3: ...
    @property
    def los(self) -> UVector3: ...
    @property
    def lot(self) -> UVector3: ...
    @property
    def loq(self) -> UVector3: ...
    @property
    def lop(self) -> UVector3: ...
    @property
    def lls(self) -> UVector3: ...
    @property
    def llt(self) -> UVector3: ...
    @property
    def llq(self) -> UVector3: ...
    @property
    def llp(self) -> UVector3: ...
    @property
    def ssss(self) -> UVector4: ...
    @property
    def ssst(self) -> UVector4: ...
    @property
    def sssq(self) -> UVector4: ...
    @property
    def sssp(self) -> UVector4: ...
    @property
    def ssso(self) -> UVector4: ...
    @property
    def sssl(self) -> UVector4: ...
    @property
    def ssts(self) -> UVector4: ...
    @property
    def sstt(self) -> UVector4: ...
    @property
    def sstq(self) -> UVector4: ...
    @property
    def sstp(self) -> UVector4: ...
    @property
    def ssto(self) -> UVector4: ...
    @property
    def sstl(self) -> UVector4: ...
    @property
    def ssqs(self) -> UVector4: ...
    @property
    def ssqt(self) -> UVector4: ...
    @property
    def ssqq(self) -> UVector4: ...
    @property
    def ssqp(self) -> UVector4: ...
    @property
    def ssqo(self) -> UVector4: ...
    @property
    def ssql(self) -> UVector4: ...
    @property
    def ssps(self) -> UVector4: ...
    @property
    def sspt(self) -> UVector4: ...
    @property
    def sspq(self) -> UVector4: ...
    @property
    def sspp(self) -> UVector4: ...
    @property
    def sspo(self) -> UVector4: ...
    @property
    def sspl(self) -> UVector4: ...
    @property
    def ssos(self) -> UVector4: ...
    @property
    def ssot(self) -> UVector4: ...
    @property
    def ssoq(self) -> UVector4: ...
    @property
    def ssop(self) -> UVector4: ...
    @property
    def ssoo(self) -> UVector4: ...
    @property
    def ssol(self) -> UVector4: ...
    @property
    def ssls(self) -> UVector4: ...
    @property
    def sslt(self) -> UVector4: ...
    @property
    def sslq(self) -> UVector4: ...
    @property
    def sslp(self) -> UVector4: ...
    @property
    def sslo(self) -> UVector4: ...
    @property
    def ssll(self) -> UVector4: ...
    @property
    def stss(self) -> UVector4: ...
    @property
    def stst(self) -> UVector4: ...
    @property
    def stsq(self) -> UVector4: ...
    @property
    def stsp(self) -> UVector4: ...
    @property
    def stso(self) -> UVector4: ...
    @property
    def stsl(self) -> UVector4: ...
    @property
    def stts(self) -> UVector4: ...
    @property
    def sttt(self) -> UVector4: ...
    @property
    def sttq(self) -> UVector4: ...
    @property
    def sttp(self) -> UVector4: ...
    @property
    def stto(self) -> UVector4: ...
    @property
    def sttl(self) -> UVector4: ...
    @property
    def stqs(self) -> UVector4: ...
    @property
    def stqt(self) -> UVector4: ...
    @property
    def stqq(self) -> UVector4: ...
    @property
    def stqp(self) -> UVector4: ...
    @property
    def stqo(self) -> UVector4: ...
    @property
    def stql(self) -> UVector4: ...
    @property
    def stps(self) -> UVector4: ...
    @property
    def stpt(self) -> UVector4: ...
    @property
    def stpq(self) -> UVector4: ...
    @property
    def stpp(self) -> UVector4: ...
    @property
    def stpo(self) -> UVector4: ...
    @property
    def stpl(self) -> UVector4: ...
    @property
    def stos(self) -> UVector4: ...
    @property
    def stot(self) -> UVector4: ...
    @property
    def stoq(self) -> UVector4: ...
    @property
    def stop(self) -> UVector4: ...
    @property
    def stoo(self) -> UVector4: ...
    @property
    def stol(self) -> UVector4: ...
    @property
    def stls(self) -> UVector4: ...
    @property
    def stlt(self) -> UVector4: ...
    @property
    def stlq(self) -> UVector4: ...
    @property
    def stlp(self) -> UVector4: ...
    @property
    def stlo(self) -> UVector4: ...
    @property
    def stll(self) -> UVector4: ...
    @property
    def sqss(self) -> UVector4: ...
    @property
    def sqst(self) -> UVector4: ...
    @property
    def sqsq(self) -> UVector4: ...
    @property
    def sqsp(self) -> UVector4: ...
    @property
    def sqso(self) -> UVector4: ...
    @property
    def sqsl(self) -> UVector4: ...
    @property
    def sqts(self) -> UVector4: ...
    @property
    def sqtt(self) -> UVector4: ...
    @property
    def sqtq(self) -> UVector4: ...
    @property
    def sqtp(self) -> UVector4: ...
    @property
    def sqto(self) -> UVector4: ...
    @property
    def sqtl(self) -> UVector4: ...
    @property
    def sqqs(self) -> UVector4: ...
    @property
    def sqqt(self) -> UVector4: ...
    @property
    def sqqq(self) -> UVector4: ...
    @property
    def sqqp(self) -> UVector4: ...
    @property
    def sqqo(self) -> UVector4: ...
    @property
    def sqql(self) -> UVector4: ...
    @property
    def sqps(self) -> UVector4: ...
    @property
    def sqpt(self) -> UVector4: ...
    @property
    def sqpq(self) -> UVector4: ...
    @property
    def sqpp(self) -> UVector4: ...
    @property
    def sqpo(self) -> UVector4: ...
    @property
    def sqpl(self) -> UVector4: ...
    @property
    def sqos(self) -> UVector4: ...
    @property
    def sqot(self) -> UVector4: ...
    @property
    def sqoq(self) -> UVector4: ...
    @property
    def sqop(self) -> UVector4: ...
    @property
    def sqoo(self) -> UVector4: ...
    @property
    def sqol(self) -> UVector4: ...
    @property
    def sqls(self) -> UVector4: ...
    @property
    def sqlt(self) -> UVector4: ...
    @property
    def sqlq(self) -> UVector4: ...
    @property
    def sqlp(self) -> UVector4: ...
    @property
    def sqlo(self) -> UVector4: ...
    @property
    def sqll(self) -> UVector4: ...
    @property
    def spss(self) -> UVector4: ...
    @property
    def spst(self) -> UVector4: ...
    @property
    def spsq(self) -> UVector4: ...
    @property
    def spsp(self) -> UVector4: ...
    @property
    def spso(self) -> UVector4: ...
    @property
    def spsl(self) -> UVector4: ...
    @property
    def spts(self) -> UVector4: ...
    @property
    def sptt(self) -> UVector4: ...
    @property
    def sptq(self) -> UVector4: ...
    @property
    def sptp(self) -> UVector4: ...
    @property
    def spto(self) -> UVector4: ...
    @property
    def sptl(self) -> UVector4: ...
    @property
    def spqs(self) -> UVector4: ...
    @property
    def spqt(self) -> UVector4: ...
    @property
    def spqq(self) -> UVector4: ...
    @property
    def spqp(self) -> UVector4: ...
    @property
    def spqo(self) -> UVector4: ...
    @property
    def spql(self) -> UVector4: ...
    @property
    def spps(self) -> UVector4: ...
    @property
    def sppt(self) -> UVector4: ...
    @property
    def sppq(self) -> UVector4: ...
    @property
    def sppp(self) -> UVector4: ...
    @property
    def sppo(self) -> UVector4: ...
    @property
    def sppl(self) -> UVector4: ...
    @property
    def spos(self) -> UVector4: ...
    @property
    def spot(self) -> UVector4: ...
    @property
    def spoq(self) -> UVector4: ...
    @property
    def spop(self) -> UVector4: ...
    @property
    def spoo(self) -> UVector4: ...
    @property
    def spol(self) -> UVector4: ...
    @property
    def spls(self) -> UVector4: ...
    @property
    def splt(self) -> UVector4: ...
    @property
    def splq(self) -> UVector4: ...
    @property
    def splp(self) -> UVector4: ...
    @property
    def splo(self) -> UVector4: ...
    @property
    def spll(self) -> UVector4: ...
    @property
    def soss(self) -> UVector4: ...
    @property
    def sost(self) -> UVector4: ...
    @property
    def sosq(self) -> UVector4: ...
    @property
    def sosp(self) -> UVector4: ...
    @property
    def soso(self) -> UVector4: ...
    @property
    def sosl(self) -> UVector4: ...
    @property
    def sots(self) -> UVector4: ...
    @property
    def sott(self) -> UVector4: ...
    @property
    def sotq(self) -> UVector4: ...
    @property
    def sotp(self) -> UVector4: ...
    @property
    def soto(self) -> UVector4: ...
    @property
    def sotl(self) -> UVector4: ...
    @property
    def soqs(self) -> UVector4: ...
    @property
    def soqt(self) -> UVector4: ...
    @property
    def soqq(self) -> UVector4: ...
    @property
    def soqp(self) -> UVector4: ...
    @property
    def soqo(self) -> UVector4: ...
    @property
    def soql(self) -> UVector4: ...
    @property
    def sops(self) -> UVector4: ...
    @property
    def sopt(self) -> UVector4: ...
    @property
    def sopq(self) -> UVector4: ...
    @property
    def sopp(self) -> UVector4: ...
    @property
    def sopo(self) -> UVector4: ...
    @property
    def sopl(self) -> UVector4: ...
    @property
    def soos(self) -> UVector4: ...
    @property
    def soot(self) -> UVector4: ...
    @property
    def sooq(self) -> UVector4: ...
    @property
    def soop(self) -> UVector4: ...
    @property
    def sooo(self) -> UVector4: ...
    @property
    def sool(self) -> UVector4: ...
    @property
    def sols(self) -> UVector4: ...
    @property
    def solt(self) -> UVector4: ...
    @property
    def solq(self) -> UVector4: ...
    @property
    def solp(self) -> UVector4: ...
    @property
    def solo(self) -> UVector4: ...
    @property
    def soll(self) -> UVector4: ...
    @property
    def slss(self) -> UVector4: ...
    @property
    def slst(self) -> UVector4: ...
    @property
    def slsq(self) -> UVector4: ...
    @property
    def slsp(self) -> UVector4: ...
    @property
    def slso(self) -> UVector4: ...
    @property
    def slsl(self) -> UVector4: ...
    @property
    def slts(self) -> UVector4: ...
    @property
    def sltt(self) -> UVector4: ...
    @property
    def sltq(self) -> UVector4: ...
    @property
    def sltp(self) -> UVector4: ...
    @property
    def slto(self) -> UVector4: ...
    @property
    def sltl(self) -> UVector4: ...
    @property
    def slqs(self) -> UVector4: ...
    @property
    def slqt(self) -> UVector4: ...
    @property
    def slqq(self) -> UVector4: ...
    @property
    def slqp(self) -> UVector4: ...
    @property
    def slqo(self) -> UVector4: ...
    @property
    def slql(self) -> UVector4: ...
    @property
    def slps(self) -> UVector4: ...
    @property
    def slpt(self) -> UVector4: ...
    @property
    def slpq(self) -> UVector4: ...
    @property
    def slpp(self) -> UVector4: ...
    @property
    def slpo(self) -> UVector4: ...
    @property
    def slpl(self) -> UVector4: ...
    @property
    def slos(self) -> UVector4: ...
    @property
    def slot(self) -> UVector4: ...
    @property
    def sloq(self) -> UVector4: ...
    @property
    def slop(self) -> UVector4: ...
    @property
    def sloo(self) -> UVector4: ...
    @property
    def slol(self) -> UVector4: ...
    @property
    def slls(self) -> UVector4: ...
    @property
    def sllt(self) -> UVector4: ...
    @property
    def sllq(self) -> UVector4: ...
    @property
    def sllp(self) -> UVector4: ...
    @property
    def sllo(self) -> UVector4: ...
    @property
    def slll(self) -> UVector4: ...
    @property
    def tsss(self) -> UVector4: ...
    @property
    def tsst(self) -> UVector4: ...
    @property
    def tssq(self) -> UVector4: ...
    @property
    def tssp(self) -> UVector4: ...
    @property
    def tsso(self) -> UVector4: ...
    @property
    def tssl(self) -> UVector4: ...
    @property
    def tsts(self) -> UVector4: ...
    @property
    def tstt(self) -> UVector4: ...
    @property
    def tstq(self) -> UVector4: ...
    @property
    def tstp(self) -> UVector4: ...
    @property
    def tsto(self) -> UVector4: ...
    @property
    def tstl(self) -> UVector4: ...
    @property
    def tsqs(self) -> UVector4: ...
    @property
    def tsqt(self) -> UVector4: ...
    @property
    def tsqq(self) -> UVector4: ...
    @property
    def tsqp(self) -> UVector4: ...
    @property
    def tsqo(self) -> UVector4: ...
    @property
    def tsql(self) -> UVector4: ...
    @property
    def tsps(self) -> UVector4: ...
    @property
    def tspt(self) -> UVector4: ...
    @property
    def tspq(self) -> UVector4: ...
    @property
    def tspp(self) -> UVector4: ...
    @property
    def tspo(self) -> UVector4: ...
    @property
    def tspl(self) -> UVector4: ...
    @property
    def tsos(self) -> UVector4: ...
    @property
    def tsot(self) -> UVector4: ...
    @property
    def tsoq(self) -> UVector4: ...
    @property
    def tsop(self) -> UVector4: ...
    @property
    def tsoo(self) -> UVector4: ...
    @property
    def tsol(self) -> UVector4: ...
    @property
    def tsls(self) -> UVector4: ...
    @property
    def tslt(self) -> UVector4: ...
    @property
    def tslq(self) -> UVector4: ...
    @property
    def tslp(self) -> UVector4: ...
    @property
    def tslo(self) -> UVector4: ...
    @property
    def tsll(self) -> UVector4: ...
    @property
    def ttss(self) -> UVector4: ...
    @property
    def ttst(self) -> UVector4: ...
    @property
    def ttsq(self) -> UVector4: ...
    @property
    def ttsp(self) -> UVector4: ...
    @property
    def ttso(self) -> UVector4: ...
    @property
    def ttsl(self) -> UVector4: ...
    @property
    def ttts(self) -> UVector4: ...
    @property
    def tttt(self) -> UVector4: ...
    @property
    def tttq(self) -> UVector4: ...
    @property
    def tttp(self) -> UVector4: ...
    @property
    def ttto(self) -> UVector4: ...
    @property
    def tttl(self) -> UVector4: ...
    @property
    def ttqs(self) -> UVector4: ...
    @property
    def ttqt(self) -> UVector4: ...
    @property
    def ttqq(self) -> UVector4: ...
    @property
    def ttqp(self) -> UVector4: ...
    @property
    def ttqo(self) -> UVector4: ...
    @property
    def ttql(self) -> UVector4: ...
    @property
    def ttps(self) -> UVector4: ...
    @property
    def ttpt(self) -> UVector4: ...
    @property
    def ttpq(self) -> UVector4: ...
    @property
    def ttpp(self) -> UVector4: ...
    @property
    def ttpo(self) -> UVector4: ...
    @property
    def ttpl(self) -> UVector4: ...
    @property
    def ttos(self) -> UVector4: ...
    @property
    def ttot(self) -> UVector4: ...
    @property
    def ttoq(self) -> UVector4: ...
    @property
    def ttop(self) -> UVector4: ...
    @property
    def ttoo(self) -> UVector4: ...
    @property
    def ttol(self) -> UVector4: ...
    @property
    def ttls(self) -> UVector4: ...
    @property
    def ttlt(self) -> UVector4: ...
    @property
    def ttlq(self) -> UVector4: ...
    @property
    def ttlp(self) -> UVector4: ...
    @property
    def ttlo(self) -> UVector4: ...
    @property
    def ttll(self) -> UVector4: ...
    @property
    def tqss(self) -> UVector4: ...
    @property
    def tqst(self) -> UVector4: ...
    @property
    def tqsq(self) -> UVector4: ...
    @property
    def tqsp(self) -> UVector4: ...
    @property
    def tqso(self) -> UVector4: ...
    @property
    def tqsl(self) -> UVector4: ...
    @property
    def tqts(self) -> UVector4: ...
    @property
    def tqtt(self) -> UVector4: ...
    @property
    def tqtq(self) -> UVector4: ...
    @property
    def tqtp(self) -> UVector4: ...
    @property
    def tqto(self) -> UVector4: ...
    @property
    def tqtl(self) -> UVector4: ...
    @property
    def tqqs(self) -> UVector4: ...
    @property
    def tqqt(self) -> UVector4: ...
    @property
    def tqqq(self) -> UVector4: ...
    @property
    def tqqp(self) -> UVector4: ...
    @property
    def tqqo(self) -> UVector4: ...
    @property
    def tqql(self) -> UVector4: ...
    @property
    def tqps(self) -> UVector4: ...
    @property
    def tqpt(self) -> UVector4: ...
    @property
    def tqpq(self) -> UVector4: ...
    @property
    def tqpp(self) -> UVector4: ...
    @property
    def tqpo(self) -> UVector4: ...
    @property
    def tqpl(self) -> UVector4: ...
    @property
    def tqos(self) -> UVector4: ...
    @property
    def tqot(self) -> UVector4: ...
    @property
    def tqoq(self) -> UVector4: ...
    @property
    def tqop(self) -> UVector4: ...
    @property
    def tqoo(self) -> UVector4: ...
    @property
    def tqol(self) -> UVector4: ...
    @property
    def tqls(self) -> UVector4: ...
    @property
    def tqlt(self) -> UVector4: ...
    @property
    def tqlq(self) -> UVector4: ...
    @property
    def tqlp(self) -> UVector4: ...
    @property
    def tqlo(self) -> UVector4: ...
    @property
    def tqll(self) -> UVector4: ...
    @property
    def tpss(self) -> UVector4: ...
    @property
    def tpst(self) -> UVector4: ...
    @property
    def tpsq(self) -> UVector4: ...
    @property
    def tpsp(self) -> UVector4: ...
    @property
    def tpso(self) -> UVector4: ...
    @property
    def tpsl(self) -> UVector4: ...
    @property
    def tpts(self) -> UVector4: ...
    @property
    def tptt(self) -> UVector4: ...
    @property
    def tptq(self) -> UVector4: ...
    @property
    def tptp(self) -> UVector4: ...
    @property
    def tpto(self) -> UVector4: ...
    @property
    def tptl(self) -> UVector4: ...
    @property
    def tpqs(self) -> UVector4: ...
    @property
    def tpqt(self) -> UVector4: ...
    @property
    def tpqq(self) -> UVector4: ...
    @property
    def tpqp(self) -> UVector4: ...
    @property
    def tpqo(self) -> UVector4: ...
    @property
    def tpql(self) -> UVector4: ...
    @property
    def tpps(self) -> UVector4: ...
    @property
    def tppt(self) -> UVector4: ...
    @property
    def tppq(self) -> UVector4: ...
    @property
    def tppp(self) -> UVector4: ...
    @property
    def tppo(self) -> UVector4: ...
    @property
    def tppl(self) -> UVector4: ...
    @property
    def tpos(self) -> UVector4: ...
    @property
    def tpot(self) -> UVector4: ...
    @property
    def tpoq(self) -> UVector4: ...
    @property
    def tpop(self) -> UVector4: ...
    @property
    def tpoo(self) -> UVector4: ...
    @property
    def tpol(self) -> UVector4: ...
    @property
    def tpls(self) -> UVector4: ...
    @property
    def tplt(self) -> UVector4: ...
    @property
    def tplq(self) -> UVector4: ...
    @property
    def tplp(self) -> UVector4: ...
    @property
    def tplo(self) -> UVector4: ...
    @property
    def tpll(self) -> UVector4: ...
    @property
    def toss(self) -> UVector4: ...
    @property
    def tost(self) -> UVector4: ...
    @property
    def tosq(self) -> UVector4: ...
    @property
    def tosp(self) -> UVector4: ...
    @property
    def toso(self) -> UVector4: ...
    @property
    def tosl(self) -> UVector4: ...
    @property
    def tots(self) -> UVector4: ...
    @property
    def tott(self) -> UVector4: ...
    @property
    def totq(self) -> UVector4: ...
    @property
    def totp(self) -> UVector4: ...
    @property
    def toto(self) -> UVector4: ...
    @property
    def totl(self) -> UVector4: ...
    @property
    def toqs(self) -> UVector4: ...
    @property
    def toqt(self) -> UVector4: ...
    @property
    def toqq(self) -> UVector4: ...
    @property
    def toqp(self) -> UVector4: ...
    @property
    def toqo(self) -> UVector4: ...
    @property
    def toql(self) -> UVector4: ...
    @property
    def tops(self) -> UVector4: ...
    @property
    def topt(self) -> UVector4: ...
    @property
    def topq(self) -> UVector4: ...
    @property
    def topp(self) -> UVector4: ...
    @property
    def topo(self) -> UVector4: ...
    @property
    def topl(self) -> UVector4: ...
    @property
    def toos(self) -> UVector4: ...
    @property
    def toot(self) -> UVector4: ...
    @property
    def tooq(self) -> UVector4: ...
    @property
    def toop(self) -> UVector4: ...
    @property
    def tooo(self) -> UVector4: ...
    @property
    def tool(self) -> UVector4: ...
    @property
    def tols(self) -> UVector4: ...
    @property
    def tolt(self) -> UVector4: ...
    @property
    def tolq(self) -> UVector4: ...
    @property
    def tolp(self) -> UVector4: ...
    @property
    def tolo(self) -> UVector4: ...
    @property
    def toll(self) -> UVector4: ...
    @property
    def tlss(self) -> UVector4: ...
    @property
    def tlst(self) -> UVector4: ...
    @property
    def tlsq(self) -> UVector4: ...
    @property
    def tlsp(self) -> UVector4: ...
    @property
    def tlso(self) -> UVector4: ...
    @property
    def tlsl(self) -> UVector4: ...
    @property
    def tlts(self) -> UVector4: ...
    @property
    def tltt(self) -> UVector4: ...
    @property
    def tltq(self) -> UVector4: ...
    @property
    def tltp(self) -> UVector4: ...
    @property
    def tlto(self) -> UVector4: ...
    @property
    def tltl(self) -> UVector4: ...
    @property
    def tlqs(self) -> UVector4: ...
    @property
    def tlqt(self) -> UVector4: ...
    @property
    def tlqq(self) -> UVector4: ...
    @property
    def tlqp(self) -> UVector4: ...
    @property
    def tlqo(self) -> UVector4: ...
    @property
    def tlql(self) -> UVector4: ...
    @property
    def tlps(self) -> UVector4: ...
    @property
    def tlpt(self) -> UVector4: ...
    @property
    def tlpq(self) -> UVector4: ...
    @property
    def tlpp(self) -> UVector4: ...
    @property
    def tlpo(self) -> UVector4: ...
    @property
    def tlpl(self) -> UVector4: ...
    @property
    def tlos(self) -> UVector4: ...
    @property
    def tlot(self) -> UVector4: ...
    @property
    def tloq(self) -> UVector4: ...
    @property
    def tlop(self) -> UVector4: ...
    @property
    def tloo(self) -> UVector4: ...
    @property
    def tlol(self) -> UVector4: ...
    @property
    def tlls(self) -> UVector4: ...
    @property
    def tllt(self) -> UVector4: ...
    @property
    def tllq(self) -> UVector4: ...
    @property
    def tllp(self) -> UVector4: ...
    @property
    def tllo(self) -> UVector4: ...
    @property
    def tlll(self) -> UVector4: ...
    @property
    def qsss(self) -> UVector4: ...
    @property
    def qsst(self) -> UVector4: ...
    @property
    def qssq(self) -> UVector4: ...
    @property
    def qssp(self) -> UVector4: ...
    @property
    def qsso(self) -> UVector4: ...
    @property
    def qssl(self) -> UVector4: ...
    @property
    def qsts(self) -> UVector4: ...
    @property
    def qstt(self) -> UVector4: ...
    @property
    def qstq(self) -> UVector4: ...
    @property
    def qstp(self) -> UVector4: ...
    @property
    def qsto(self) -> UVector4: ...
    @property
    def qstl(self) -> UVector4: ...
    @property
    def qsqs(self) -> UVector4: ...
    @property
    def qsqt(self) -> UVector4: ...
    @property
    def qsqq(self) -> UVector4: ...
    @property
    def qsqp(self) -> UVector4: ...
    @property
    def qsqo(self) -> UVector4: ...
    @property
    def qsql(self) -> UVector4: ...
    @property
    def qsps(self) -> UVector4: ...
    @property
    def qspt(self) -> UVector4: ...
    @property
    def qspq(self) -> UVector4: ...
    @property
    def qspp(self) -> UVector4: ...
    @property
    def qspo(self) -> UVector4: ...
    @property
    def qspl(self) -> UVector4: ...
    @property
    def qsos(self) -> UVector4: ...
    @property
    def qsot(self) -> UVector4: ...
    @property
    def qsoq(self) -> UVector4: ...
    @property
    def qsop(self) -> UVector4: ...
    @property
    def qsoo(self) -> UVector4: ...
    @property
    def qsol(self) -> UVector4: ...
    @property
    def qsls(self) -> UVector4: ...
    @property
    def qslt(self) -> UVector4: ...
    @property
    def qslq(self) -> UVector4: ...
    @property
    def qslp(self) -> UVector4: ...
    @property
    def qslo(self) -> UVector4: ...
    @property
    def qsll(self) -> UVector4: ...
    @property
    def qtss(self) -> UVector4: ...
    @property
    def qtst(self) -> UVector4: ...
    @property
    def qtsq(self) -> UVector4: ...
    @property
    def qtsp(self) -> UVector4: ...
    @property
    def qtso(self) -> UVector4: ...
    @property
    def qtsl(self) -> UVector4: ...
    @property
    def qtts(self) -> UVector4: ...
    @property
    def qttt(self) -> UVector4: ...
    @property
    def qttq(self) -> UVector4: ...
    @property
    def qttp(self) -> UVector4: ...
    @property
    def qtto(self) -> UVector4: ...
    @property
    def qttl(self) -> UVector4: ...
    @property
    def qtqs(self) -> UVector4: ...
    @property
    def qtqt(self) -> UVector4: ...
    @property
    def qtqq(self) -> UVector4: ...
    @property
    def qtqp(self) -> UVector4: ...
    @property
    def qtqo(self) -> UVector4: ...
    @property
    def qtql(self) -> UVector4: ...
    @property
    def qtps(self) -> UVector4: ...
    @property
    def qtpt(self) -> UVector4: ...
    @property
    def qtpq(self) -> UVector4: ...
    @property
    def qtpp(self) -> UVector4: ...
    @property
    def qtpo(self) -> UVector4: ...
    @property
    def qtpl(self) -> UVector4: ...
    @property
    def qtos(self) -> UVector4: ...
    @property
    def qtot(self) -> UVector4: ...
    @property
    def qtoq(self) -> UVector4: ...
    @property
    def qtop(self) -> UVector4: ...
    @property
    def qtoo(self) -> UVector4: ...
    @property
    def qtol(self) -> UVector4: ...
    @property
    def qtls(self) -> UVector4: ...
    @property
    def qtlt(self) -> UVector4: ...
    @property
    def qtlq(self) -> UVector4: ...
    @property
    def qtlp(self) -> UVector4: ...
    @property
    def qtlo(self) -> UVector4: ...
    @property
    def qtll(self) -> UVector4: ...
    @property
    def qqss(self) -> UVector4: ...
    @property
    def qqst(self) -> UVector4: ...
    @property
    def qqsq(self) -> UVector4: ...
    @property
    def qqsp(self) -> UVector4: ...
    @property
    def qqso(self) -> UVector4: ...
    @property
    def qqsl(self) -> UVector4: ...
    @property
    def qqts(self) -> UVector4: ...
    @property
    def qqtt(self) -> UVector4: ...
    @property
    def qqtq(self) -> UVector4: ...
    @property
    def qqtp(self) -> UVector4: ...
    @property
    def qqto(self) -> UVector4: ...
    @property
    def qqtl(self) -> UVector4: ...
    @property
    def qqqs(self) -> UVector4: ...
    @property
    def qqqt(self) -> UVector4: ...
    @property
    def qqqq(self) -> UVector4: ...
    @property
    def qqqp(self) -> UVector4: ...
    @property
    def qqqo(self) -> UVector4: ...
    @property
    def qqql(self) -> UVector4: ...
    @property
    def qqps(self) -> UVector4: ...
    @property
    def qqpt(self) -> UVector4: ...
    @property
    def qqpq(self) -> UVector4: ...
    @property
    def qqpp(self) -> UVector4: ...
    @property
    def qqpo(self) -> UVector4: ...
    @property
    def qqpl(self) -> UVector4: ...
    @property
    def qqos(self) -> UVector4: ...
    @property
    def qqot(self) -> UVector4: ...
    @property
    def qqoq(self) -> UVector4: ...
    @property
    def qqop(self) -> UVector4: ...
    @property
    def qqoo(self) -> UVector4: ...
    @property
    def qqol(self) -> UVector4: ...
    @property
    def qqls(self) -> UVector4: ...
    @property
    def qqlt(self) -> UVector4: ...
    @property
    def qqlq(self) -> UVector4: ...
    @property
    def qqlp(self) -> UVector4: ...
    @property
    def qqlo(self) -> UVector4: ...
    @property
    def qqll(self) -> UVector4: ...
    @property
    def qpss(self) -> UVector4: ...
    @property
    def qpst(self) -> UVector4: ...
    @property
    def qpsq(self) -> UVector4: ...
    @property
    def qpsp(self) -> UVector4: ...
    @property
    def qpso(self) -> UVector4: ...
    @property
    def qpsl(self) -> UVector4: ...
    @property
    def qpts(self) -> UVector4: ...
    @property
    def qptt(self) -> UVector4: ...
    @property
    def qptq(self) -> UVector4: ...
    @property
    def qptp(self) -> UVector4: ...
    @property
    def qpto(self) -> UVector4: ...
    @property
    def qptl(self) -> UVector4: ...
    @property
    def qpqs(self) -> UVector4: ...
    @property
    def qpqt(self) -> UVector4: ...
    @property
    def qpqq(self) -> UVector4: ...
    @property
    def qpqp(self) -> UVector4: ...
    @property
    def qpqo(self) -> UVector4: ...
    @property
    def qpql(self) -> UVector4: ...
    @property
    def qpps(self) -> UVector4: ...
    @property
    def qppt(self) -> UVector4: ...
    @property
    def qppq(self) -> UVector4: ...
    @property
    def qppp(self) -> UVector4: ...
    @property
    def qppo(self) -> UVector4: ...
    @property
    def qppl(self) -> UVector4: ...
    @property
    def qpos(self) -> UVector4: ...
    @property
    def qpot(self) -> UVector4: ...
    @property
    def qpoq(self) -> UVector4: ...
    @property
    def qpop(self) -> UVector4: ...
    @property
    def qpoo(self) -> UVector4: ...
    @property
    def qpol(self) -> UVector4: ...
    @property
    def qpls(self) -> UVector4: ...
    @property
    def qplt(self) -> UVector4: ...
    @property
    def qplq(self) -> UVector4: ...
    @property
    def qplp(self) -> UVector4: ...
    @property
    def qplo(self) -> UVector4: ...
    @property
    def qpll(self) -> UVector4: ...
    @property
    def qoss(self) -> UVector4: ...
    @property
    def qost(self) -> UVector4: ...
    @property
    def qosq(self) -> UVector4: ...
    @property
    def qosp(self) -> UVector4: ...
    @property
    def qoso(self) -> UVector4: ...
    @property
    def qosl(self) -> UVector4: ...
    @property
    def qots(self) -> UVector4: ...
    @property
    def qott(self) -> UVector4: ...
    @property
    def qotq(self) -> UVector4: ...
    @property
    def qotp(self) -> UVector4: ...
    @property
    def qoto(self) -> UVector4: ...
    @property
    def qotl(self) -> UVector4: ...
    @property
    def qoqs(self) -> UVector4: ...
    @property
    def qoqt(self) -> UVector4: ...
    @property
    def qoqq(self) -> UVector4: ...
    @property
    def qoqp(self) -> UVector4: ...
    @property
    def qoqo(self) -> UVector4: ...
    @property
    def qoql(self) -> UVector4: ...
    @property
    def qops(self) -> UVector4: ...
    @property
    def qopt(self) -> UVector4: ...
    @property
    def qopq(self) -> UVector4: ...
    @property
    def qopp(self) -> UVector4: ...
    @property
    def qopo(self) -> UVector4: ...
    @property
    def qopl(self) -> UVector4: ...
    @property
    def qoos(self) -> UVector4: ...
    @property
    def qoot(self) -> UVector4: ...
    @property
    def qooq(self) -> UVector4: ...
    @property
    def qoop(self) -> UVector4: ...
    @property
    def qooo(self) -> UVector4: ...
    @property
    def qool(self) -> UVector4: ...
    @property
    def qols(self) -> UVector4: ...
    @property
    def qolt(self) -> UVector4: ...
    @property
    def qolq(self) -> UVector4: ...
    @property
    def qolp(self) -> UVector4: ...
    @property
    def qolo(self) -> UVector4: ...
    @property
    def qoll(self) -> UVector4: ...
    @property
    def qlss(self) -> UVector4: ...
    @property
    def qlst(self) -> UVector4: ...
    @property
    def qlsq(self) -> UVector4: ...
    @property
    def qlsp(self) -> UVector4: ...
    @property
    def qlso(self) -> UVector4: ...
    @property
    def qlsl(self) -> UVector4: ...
    @property
    def qlts(self) -> UVector4: ...
    @property
    def qltt(self) -> UVector4: ...
    @property
    def qltq(self) -> UVector4: ...
    @property
    def qltp(self) -> UVector4: ...
    @property
    def qlto(self) -> UVector4: ...
    @property
    def qltl(self) -> UVector4: ...
    @property
    def qlqs(self) -> UVector4: ...
    @property
    def qlqt(self) -> UVector4: ...
    @property
    def qlqq(self) -> UVector4: ...
    @property
    def qlqp(self) -> UVector4: ...
    @property
    def qlqo(self) -> UVector4: ...
    @property
    def qlql(self) -> UVector4: ...
    @property
    def qlps(self) -> UVector4: ...
    @property
    def qlpt(self) -> UVector4: ...
    @property
    def qlpq(self) -> UVector4: ...
    @property
    def qlpp(self) -> UVector4: ...
    @property
    def qlpo(self) -> UVector4: ...
    @property
    def qlpl(self) -> UVector4: ...
    @property
    def qlos(self) -> UVector4: ...
    @property
    def qlot(self) -> UVector4: ...
    @property
    def qloq(self) -> UVector4: ...
    @property
    def qlop(self) -> UVector4: ...
    @property
    def qloo(self) -> UVector4: ...
    @property
    def qlol(self) -> UVector4: ...
    @property
    def qlls(self) -> UVector4: ...
    @property
    def qllt(self) -> UVector4: ...
    @property
    def qllq(self) -> UVector4: ...
    @property
    def qllp(self) -> UVector4: ...
    @property
    def qllo(self) -> UVector4: ...
    @property
    def qlll(self) -> UVector4: ...
    @property
    def psss(self) -> UVector4: ...
    @property
    def psst(self) -> UVector4: ...
    @property
    def pssq(self) -> UVector4: ...
    @property
    def pssp(self) -> UVector4: ...
    @property
    def psso(self) -> UVector4: ...
    @property
    def pssl(self) -> UVector4: ...
    @property
    def psts(self) -> UVector4: ...
    @property
    def pstt(self) -> UVector4: ...
    @property
    def pstq(self) -> UVector4: ...
    @property
    def pstp(self) -> UVector4: ...
    @property
    def psto(self) -> UVector4: ...
    @property
    def pstl(self) -> UVector4: ...
    @property
    def psqs(self) -> UVector4: ...
    @property
    def psqt(self) -> UVector4: ...
    @property
    def psqq(self) -> UVector4: ...
    @property
    def psqp(self) -> UVector4: ...
    @property
    def psqo(self) -> UVector4: ...
    @property
    def psql(self) -> UVector4: ...
    @property
    def psps(self) -> UVector4: ...
    @property
    def pspt(self) -> UVector4: ...
    @property
    def pspq(self) -> UVector4: ...
    @property
    def pspp(self) -> UVector4: ...
    @property
    def pspo(self) -> UVector4: ...
    @property
    def pspl(self) -> UVector4: ...
    @property
    def psos(self) -> UVector4: ...
    @property
    def psot(self) -> UVector4: ...
    @property
    def psoq(self) -> UVector4: ...
    @property
    def psop(self) -> UVector4: ...
    @property
    def psoo(self) -> UVector4: ...
    @property
    def psol(self) -> UVector4: ...
    @property
    def psls(self) -> UVector4: ...
    @property
    def pslt(self) -> UVector4: ...
    @property
    def pslq(self) -> UVector4: ...
    @property
    def pslp(self) -> UVector4: ...
    @property
    def pslo(self) -> UVector4: ...
    @property
    def psll(self) -> UVector4: ...
    @property
    def ptss(self) -> UVector4: ...
    @property
    def ptst(self) -> UVector4: ...
    @property
    def ptsq(self) -> UVector4: ...
    @property
    def ptsp(self) -> UVector4: ...
    @property
    def ptso(self) -> UVector4: ...
    @property
    def ptsl(self) -> UVector4: ...
    @property
    def ptts(self) -> UVector4: ...
    @property
    def pttt(self) -> UVector4: ...
    @property
    def pttq(self) -> UVector4: ...
    @property
    def pttp(self) -> UVector4: ...
    @property
    def ptto(self) -> UVector4: ...
    @property
    def pttl(self) -> UVector4: ...
    @property
    def ptqs(self) -> UVector4: ...
    @property
    def ptqt(self) -> UVector4: ...
    @property
    def ptqq(self) -> UVector4: ...
    @property
    def ptqp(self) -> UVector4: ...
    @property
    def ptqo(self) -> UVector4: ...
    @property
    def ptql(self) -> UVector4: ...
    @property
    def ptps(self) -> UVector4: ...
    @property
    def ptpt(self) -> UVector4: ...
    @property
    def ptpq(self) -> UVector4: ...
    @property
    def ptpp(self) -> UVector4: ...
    @property
    def ptpo(self) -> UVector4: ...
    @property
    def ptpl(self) -> UVector4: ...
    @property
    def ptos(self) -> UVector4: ...
    @property
    def ptot(self) -> UVector4: ...
    @property
    def ptoq(self) -> UVector4: ...
    @property
    def ptop(self) -> UVector4: ...
    @property
    def ptoo(self) -> UVector4: ...
    @property
    def ptol(self) -> UVector4: ...
    @property
    def ptls(self) -> UVector4: ...
    @property
    def ptlt(self) -> UVector4: ...
    @property
    def ptlq(self) -> UVector4: ...
    @property
    def ptlp(self) -> UVector4: ...
    @property
    def ptlo(self) -> UVector4: ...
    @property
    def ptll(self) -> UVector4: ...
    @property
    def pqss(self) -> UVector4: ...
    @property
    def pqst(self) -> UVector4: ...
    @property
    def pqsq(self) -> UVector4: ...
    @property
    def pqsp(self) -> UVector4: ...
    @property
    def pqso(self) -> UVector4: ...
    @property
    def pqsl(self) -> UVector4: ...
    @property
    def pqts(self) -> UVector4: ...
    @property
    def pqtt(self) -> UVector4: ...
    @property
    def pqtq(self) -> UVector4: ...
    @property
    def pqtp(self) -> UVector4: ...
    @property
    def pqto(self) -> UVector4: ...
    @property
    def pqtl(self) -> UVector4: ...
    @property
    def pqqs(self) -> UVector4: ...
    @property
    def pqqt(self) -> UVector4: ...
    @property
    def pqqq(self) -> UVector4: ...
    @property
    def pqqp(self) -> UVector4: ...
    @property
    def pqqo(self) -> UVector4: ...
    @property
    def pqql(self) -> UVector4: ...
    @property
    def pqps(self) -> UVector4: ...
    @property
    def pqpt(self) -> UVector4: ...
    @property
    def pqpq(self) -> UVector4: ...
    @property
    def pqpp(self) -> UVector4: ...
    @property
    def pqpo(self) -> UVector4: ...
    @property
    def pqpl(self) -> UVector4: ...
    @property
    def pqos(self) -> UVector4: ...
    @property
    def pqot(self) -> UVector4: ...
    @property
    def pqoq(self) -> UVector4: ...
    @property
    def pqop(self) -> UVector4: ...
    @property
    def pqoo(self) -> UVector4: ...
    @property
    def pqol(self) -> UVector4: ...
    @property
    def pqls(self) -> UVector4: ...
    @property
    def pqlt(self) -> UVector4: ...
    @property
    def pqlq(self) -> UVector4: ...
    @property
    def pqlp(self) -> UVector4: ...
    @property
    def pqlo(self) -> UVector4: ...
    @property
    def pqll(self) -> UVector4: ...
    @property
    def ppss(self) -> UVector4: ...
    @property
    def ppst(self) -> UVector4: ...
    @property
    def ppsq(self) -> UVector4: ...
    @property
    def ppsp(self) -> UVector4: ...
    @property
    def ppso(self) -> UVector4: ...
    @property
    def ppsl(self) -> UVector4: ...
    @property
    def ppts(self) -> UVector4: ...
    @property
    def pptt(self) -> UVector4: ...
    @property
    def pptq(self) -> UVector4: ...
    @property
    def pptp(self) -> UVector4: ...
    @property
    def ppto(self) -> UVector4: ...
    @property
    def pptl(self) -> UVector4: ...
    @property
    def ppqs(self) -> UVector4: ...
    @property
    def ppqt(self) -> UVector4: ...
    @property
    def ppqq(self) -> UVector4: ...
    @property
    def ppqp(self) -> UVector4: ...
    @property
    def ppqo(self) -> UVector4: ...
    @property
    def ppql(self) -> UVector4: ...
    @property
    def ppps(self) -> UVector4: ...
    @property
    def pppt(self) -> UVector4: ...
    @property
    def pppq(self) -> UVector4: ...
    @property
    def pppp(self) -> UVector4: ...
    @property
    def pppo(self) -> UVector4: ...
    @property
    def pppl(self) -> UVector4: ...
    @property
    def ppos(self) -> UVector4: ...
    @property
    def ppot(self) -> UVector4: ...
    @property
    def ppoq(self) -> UVector4: ...
    @property
    def ppop(self) -> UVector4: ...
    @property
    def ppoo(self) -> UVector4: ...
    @property
    def ppol(self) -> UVector4: ...
    @property
    def ppls(self) -> UVector4: ...
    @property
    def pplt(self) -> UVector4: ...
    @property
    def pplq(self) -> UVector4: ...
    @property
    def pplp(self) -> UVector4: ...
    @property
    def pplo(self) -> UVector4: ...
    @property
    def ppll(self) -> UVector4: ...
    @property
    def poss(self) -> UVector4: ...
    @property
    def post(self) -> UVector4: ...
    @property
    def posq(self) -> UVector4: ...
    @property
    def posp(self) -> UVector4: ...
    @property
    def poso(self) -> UVector4: ...
    @property
    def posl(self) -> UVector4: ...
    @property
    def pots(self) -> UVector4: ...
    @property
    def pott(self) -> UVector4: ...
    @property
    def potq(self) -> UVector4: ...
    @property
    def potp(self) -> UVector4: ...
    @property
    def poto(self) -> UVector4: ...
    @property
    def potl(self) -> UVector4: ...
    @property
    def poqs(self) -> UVector4: ...
    @property
    def poqt(self) -> UVector4: ...
    @property
    def poqq(self) -> UVector4: ...
    @property
    def poqp(self) -> UVector4: ...
    @property
    def poqo(self) -> UVector4: ...
    @property
    def poql(self) -> UVector4: ...
    @property
    def pops(self) -> UVector4: ...
    @property
    def popt(self) -> UVector4: ...
    @property
    def popq(self) -> UVector4: ...
    @property
    def popp(self) -> UVector4: ...
    @property
    def popo(self) -> UVector4: ...
    @property
    def popl(self) -> UVector4: ...
    @property
    def poos(self) -> UVector4: ...
    @property
    def poot(self) -> UVector4: ...
    @property
    def pooq(self) -> UVector4: ...
    @property
    def poop(self) -> UVector4: ...
    @property
    def pooo(self) -> UVector4: ...
    @property
    def pool(self) -> UVector4: ...
    @property
    def pols(self) -> UVector4: ...
    @property
    def polt(self) -> UVector4: ...
    @property
    def polq(self) -> UVector4: ...
    @property
    def polp(self) -> UVector4: ...
    @property
    def polo(self) -> UVector4: ...
    @property
    def poll(self) -> UVector4: ...
    @property
    def plss(self) -> UVector4: ...
    @property
    def plst(self) -> UVector4: ...
    @property
    def plsq(self) -> UVector4: ...
    @property
    def plsp(self) -> UVector4: ...
    @property
    def plso(self) -> UVector4: ...
    @property
    def plsl(self) -> UVector4: ...
    @property
    def plts(self) -> UVector4: ...
    @property
    def pltt(self) -> UVector4: ...
    @property
    def pltq(self) -> UVector4: ...
    @property
    def pltp(self) -> UVector4: ...
    @property
    def plto(self) -> UVector4: ...
    @property
    def pltl(self) -> UVector4: ...
    @property
    def plqs(self) -> UVector4: ...
    @property
    def plqt(self) -> UVector4: ...
    @property
    def plqq(self) -> UVector4: ...
    @property
    def plqp(self) -> UVector4: ...
    @property
    def plqo(self) -> UVector4: ...
    @property
    def plql(self) -> UVector4: ...
    @property
    def plps(self) -> UVector4: ...
    @property
    def plpt(self) -> UVector4: ...
    @property
    def plpq(self) -> UVector4: ...
    @property
    def plpp(self) -> UVector4: ...
    @property
    def plpo(self) -> UVector4: ...
    @property
    def plpl(self) -> UVector4: ...
    @property
    def plos(self) -> UVector4: ...
    @property
    def plot(self) -> UVector4: ...
    @property
    def ploq(self) -> UVector4: ...
    @property
    def plop(self) -> UVector4: ...
    @property
    def ploo(self) -> UVector4: ...
    @property
    def plol(self) -> UVector4: ...
    @property
    def plls(self) -> UVector4: ...
    @property
    def pllt(self) -> UVector4: ...
    @property
    def pllq(self) -> UVector4: ...
    @property
    def pllp(self) -> UVector4: ...
    @property
    def pllo(self) -> UVector4: ...
    @property
    def plll(self) -> UVector4: ...
    @property
    def osss(self) -> UVector4: ...
    @property
    def osst(self) -> UVector4: ...
    @property
    def ossq(self) -> UVector4: ...
    @property
    def ossp(self) -> UVector4: ...
    @property
    def osso(self) -> UVector4: ...
    @property
    def ossl(self) -> UVector4: ...
    @property
    def osts(self) -> UVector4: ...
    @property
    def ostt(self) -> UVector4: ...
    @property
    def ostq(self) -> UVector4: ...
    @property
    def ostp(self) -> UVector4: ...
    @property
    def osto(self) -> UVector4: ...
    @property
    def ostl(self) -> UVector4: ...
    @property
    def osqs(self) -> UVector4: ...
    @property
    def osqt(self) -> UVector4: ...
    @property
    def osqq(self) -> UVector4: ...
    @property
    def osqp(self) -> UVector4: ...
    @property
    def osqo(self) -> UVector4: ...
    @property
    def osql(self) -> UVector4: ...
    @property
    def osps(self) -> UVector4: ...
    @property
    def ospt(self) -> UVector4: ...
    @property
    def ospq(self) -> UVector4: ...
    @property
    def ospp(self) -> UVector4: ...
    @property
    def ospo(self) -> UVector4: ...
    @property
    def ospl(self) -> UVector4: ...
    @property
    def osos(self) -> UVector4: ...
    @property
    def osot(self) -> UVector4: ...
    @property
    def osoq(self) -> UVector4: ...
    @property
    def osop(self) -> UVector4: ...
    @property
    def osoo(self) -> UVector4: ...
    @property
    def osol(self) -> UVector4: ...
    @property
    def osls(self) -> UVector4: ...
    @property
    def oslt(self) -> UVector4: ...
    @property
    def oslq(self) -> UVector4: ...
    @property
    def oslp(self) -> UVector4: ...
    @property
    def oslo(self) -> UVector4: ...
    @property
    def osll(self) -> UVector4: ...
    @property
    def otss(self) -> UVector4: ...
    @property
    def otst(self) -> UVector4: ...
    @property
    def otsq(self) -> UVector4: ...
    @property
    def otsp(self) -> UVector4: ...
    @property
    def otso(self) -> UVector4: ...
    @property
    def otsl(self) -> UVector4: ...
    @property
    def otts(self) -> UVector4: ...
    @property
    def ottt(self) -> UVector4: ...
    @property
    def ottq(self) -> UVector4: ...
    @property
    def ottp(self) -> UVector4: ...
    @property
    def otto(self) -> UVector4: ...
    @property
    def ottl(self) -> UVector4: ...
    @property
    def otqs(self) -> UVector4: ...
    @property
    def otqt(self) -> UVector4: ...
    @property
    def otqq(self) -> UVector4: ...
    @property
    def otqp(self) -> UVector4: ...
    @property
    def otqo(self) -> UVector4: ...
    @property
    def otql(self) -> UVector4: ...
    @property
    def otps(self) -> UVector4: ...
    @property
    def otpt(self) -> UVector4: ...
    @property
    def otpq(self) -> UVector4: ...
    @property
    def otpp(self) -> UVector4: ...
    @property
    def otpo(self) -> UVector4: ...
    @property
    def otpl(self) -> UVector4: ...
    @property
    def otos(self) -> UVector4: ...
    @property
    def otot(self) -> UVector4: ...
    @property
    def otoq(self) -> UVector4: ...
    @property
    def otop(self) -> UVector4: ...
    @property
    def otoo(self) -> UVector4: ...
    @property
    def otol(self) -> UVector4: ...
    @property
    def otls(self) -> UVector4: ...
    @property
    def otlt(self) -> UVector4: ...
    @property
    def otlq(self) -> UVector4: ...
    @property
    def otlp(self) -> UVector4: ...
    @property
    def otlo(self) -> UVector4: ...
    @property
    def otll(self) -> UVector4: ...
    @property
    def oqss(self) -> UVector4: ...
    @property
    def oqst(self) -> UVector4: ...
    @property
    def oqsq(self) -> UVector4: ...
    @property
    def oqsp(self) -> UVector4: ...
    @property
    def oqso(self) -> UVector4: ...
    @property
    def oqsl(self) -> UVector4: ...
    @property
    def oqts(self) -> UVector4: ...
    @property
    def oqtt(self) -> UVector4: ...
    @property
    def oqtq(self) -> UVector4: ...
    @property
    def oqtp(self) -> UVector4: ...
    @property
    def oqto(self) -> UVector4: ...
    @property
    def oqtl(self) -> UVector4: ...
    @property
    def oqqs(self) -> UVector4: ...
    @property
    def oqqt(self) -> UVector4: ...
    @property
    def oqqq(self) -> UVector4: ...
    @property
    def oqqp(self) -> UVector4: ...
    @property
    def oqqo(self) -> UVector4: ...
    @property
    def oqql(self) -> UVector4: ...
    @property
    def oqps(self) -> UVector4: ...
    @property
    def oqpt(self) -> UVector4: ...
    @property
    def oqpq(self) -> UVector4: ...
    @property
    def oqpp(self) -> UVector4: ...
    @property
    def oqpo(self) -> UVector4: ...
    @property
    def oqpl(self) -> UVector4: ...
    @property
    def oqos(self) -> UVector4: ...
    @property
    def oqot(self) -> UVector4: ...
    @property
    def oqoq(self) -> UVector4: ...
    @property
    def oqop(self) -> UVector4: ...
    @property
    def oqoo(self) -> UVector4: ...
    @property
    def oqol(self) -> UVector4: ...
    @property
    def oqls(self) -> UVector4: ...
    @property
    def oqlt(self) -> UVector4: ...
    @property
    def oqlq(self) -> UVector4: ...
    @property
    def oqlp(self) -> UVector4: ...
    @property
    def oqlo(self) -> UVector4: ...
    @property
    def oqll(self) -> UVector4: ...
    @property
    def opss(self) -> UVector4: ...
    @property
    def opst(self) -> UVector4: ...
    @property
    def opsq(self) -> UVector4: ...
    @property
    def opsp(self) -> UVector4: ...
    @property
    def opso(self) -> UVector4: ...
    @property
    def opsl(self) -> UVector4: ...
    @property
    def opts(self) -> UVector4: ...
    @property
    def optt(self) -> UVector4: ...
    @property
    def optq(self) -> UVector4: ...
    @property
    def optp(self) -> UVector4: ...
    @property
    def opto(self) -> UVector4: ...
    @property
    def optl(self) -> UVector4: ...
    @property
    def opqs(self) -> UVector4: ...
    @property
    def opqt(self) -> UVector4: ...
    @property
    def opqq(self) -> UVector4: ...
    @property
    def opqp(self) -> UVector4: ...
    @property
    def opqo(self) -> UVector4: ...
    @property
    def opql(self) -> UVector4: ...
    @property
    def opps(self) -> UVector4: ...
    @property
    def oppt(self) -> UVector4: ...
    @property
    def oppq(self) -> UVector4: ...
    @property
    def oppp(self) -> UVector4: ...
    @property
    def oppo(self) -> UVector4: ...
    @property
    def oppl(self) -> UVector4: ...
    @property
    def opos(self) -> UVector4: ...
    @property
    def opot(self) -> UVector4: ...
    @property
    def opoq(self) -> UVector4: ...
    @property
    def opop(self) -> UVector4: ...
    @property
    def opoo(self) -> UVector4: ...
    @property
    def opol(self) -> UVector4: ...
    @property
    def opls(self) -> UVector4: ...
    @property
    def oplt(self) -> UVector4: ...
    @property
    def oplq(self) -> UVector4: ...
    @property
    def oplp(self) -> UVector4: ...
    @property
    def oplo(self) -> UVector4: ...
    @property
    def opll(self) -> UVector4: ...
    @property
    def ooss(self) -> UVector4: ...
    @property
    def oost(self) -> UVector4: ...
    @property
    def oosq(self) -> UVector4: ...
    @property
    def oosp(self) -> UVector4: ...
    @property
    def ooso(self) -> UVector4: ...
    @property
    def oosl(self) -> UVector4: ...
    @property
    def oots(self) -> UVector4: ...
    @property
    def oott(self) -> UVector4: ...
    @property
    def ootq(self) -> UVector4: ...
    @property
    def ootp(self) -> UVector4: ...
    @property
    def ooto(self) -> UVector4: ...
    @property
    def ootl(self) -> UVector4: ...
    @property
    def ooqs(self) -> UVector4: ...
    @property
    def ooqt(self) -> UVector4: ...
    @property
    def ooqq(self) -> UVector4: ...
    @property
    def ooqp(self) -> UVector4: ...
    @property
    def ooqo(self) -> UVector4: ...
    @property
    def ooql(self) -> UVector4: ...
    @property
    def oops(self) -> UVector4: ...
    @property
    def oopt(self) -> UVector4: ...
    @property
    def oopq(self) -> UVector4: ...
    @property
    def oopp(self) -> UVector4: ...
    @property
    def oopo(self) -> UVector4: ...
    @property
    def oopl(self) -> UVector4: ...
    @property
    def ooos(self) -> UVector4: ...
    @property
    def ooot(self) -> UVector4: ...
    @property
    def oooq(self) -> UVector4: ...
    @property
    def ooop(self) -> UVector4: ...
    @property
    def ools(self) -> UVector4: ...
    @property
    def oolt(self) -> UVector4: ...
    @property
    def oolq(self) -> UVector4: ...
    @property
    def oolp(self) -> UVector4: ...
    @property
    def olss(self) -> UVector4: ...
    @property
    def olst(self) -> UVector4: ...
    @property
    def olsq(self) -> UVector4: ...
    @property
    def olsp(self) -> UVector4: ...
    @property
    def olso(self) -> UVector4: ...
    @property
    def olsl(self) -> UVector4: ...
    @property
    def olts(self) -> UVector4: ...
    @property
    def oltt(self) -> UVector4: ...
    @property
    def oltq(self) -> UVector4: ...
    @property
    def oltp(self) -> UVector4: ...
    @property
    def olto(self) -> UVector4: ...
    @property
    def oltl(self) -> UVector4: ...
    @property
    def olqs(self) -> UVector4: ...
    @property
    def olqt(self) -> UVector4: ...
    @property
    def olqq(self) -> UVector4: ...
    @property
    def olqp(self) -> UVector4: ...
    @property
    def olqo(self) -> UVector4: ...
    @property
    def olql(self) -> UVector4: ...
    @property
    def olps(self) -> UVector4: ...
    @property
    def olpt(self) -> UVector4: ...
    @property
    def olpq(self) -> UVector4: ...
    @property
    def olpp(self) -> UVector4: ...
    @property
    def olpo(self) -> UVector4: ...
    @property
    def olpl(self) -> UVector4: ...
    @property
    def olos(self) -> UVector4: ...
    @property
    def olot(self) -> UVector4: ...
    @property
    def oloq(self) -> UVector4: ...
    @property
    def olop(self) -> UVector4: ...
    @property
    def olls(self) -> UVector4: ...
    @property
    def ollt(self) -> UVector4: ...
    @property
    def ollq(self) -> UVector4: ...
    @property
    def ollp(self) -> UVector4: ...
    @property
    def lsss(self) -> UVector4: ...
    @property
    def lsst(self) -> UVector4: ...
    @property
    def lssq(self) -> UVector4: ...
    @property
    def lssp(self) -> UVector4: ...
    @property
    def lsso(self) -> UVector4: ...
    @property
    def lssl(self) -> UVector4: ...
    @property
    def lsts(self) -> UVector4: ...
    @property
    def lstt(self) -> UVector4: ...
    @property
    def lstq(self) -> UVector4: ...
    @property
    def lstp(self) -> UVector4: ...
    @property
    def lsto(self) -> UVector4: ...
    @property
    def lstl(self) -> UVector4: ...
    @property
    def lsqs(self) -> UVector4: ...
    @property
    def lsqt(self) -> UVector4: ...
    @property
    def lsqq(self) -> UVector4: ...
    @property
    def lsqp(self) -> UVector4: ...
    @property
    def lsqo(self) -> UVector4: ...
    @property
    def lsql(self) -> UVector4: ...
    @property
    def lsps(self) -> UVector4: ...
    @property
    def lspt(self) -> UVector4: ...
    @property
    def lspq(self) -> UVector4: ...
    @property
    def lspp(self) -> UVector4: ...
    @property
    def lspo(self) -> UVector4: ...
    @property
    def lspl(self) -> UVector4: ...
    @property
    def lsos(self) -> UVector4: ...
    @property
    def lsot(self) -> UVector4: ...
    @property
    def lsoq(self) -> UVector4: ...
    @property
    def lsop(self) -> UVector4: ...
    @property
    def lsoo(self) -> UVector4: ...
    @property
    def lsol(self) -> UVector4: ...
    @property
    def lsls(self) -> UVector4: ...
    @property
    def lslt(self) -> UVector4: ...
    @property
    def lslq(self) -> UVector4: ...
    @property
    def lslp(self) -> UVector4: ...
    @property
    def lslo(self) -> UVector4: ...
    @property
    def lsll(self) -> UVector4: ...
    @property
    def ltss(self) -> UVector4: ...
    @property
    def ltst(self) -> UVector4: ...
    @property
    def ltsq(self) -> UVector4: ...
    @property
    def ltsp(self) -> UVector4: ...
    @property
    def ltso(self) -> UVector4: ...
    @property
    def ltsl(self) -> UVector4: ...
    @property
    def ltts(self) -> UVector4: ...
    @property
    def lttt(self) -> UVector4: ...
    @property
    def lttq(self) -> UVector4: ...
    @property
    def lttp(self) -> UVector4: ...
    @property
    def ltto(self) -> UVector4: ...
    @property
    def lttl(self) -> UVector4: ...
    @property
    def ltqs(self) -> UVector4: ...
    @property
    def ltqt(self) -> UVector4: ...
    @property
    def ltqq(self) -> UVector4: ...
    @property
    def ltqp(self) -> UVector4: ...
    @property
    def ltqo(self) -> UVector4: ...
    @property
    def ltql(self) -> UVector4: ...
    @property
    def ltps(self) -> UVector4: ...
    @property
    def ltpt(self) -> UVector4: ...
    @property
    def ltpq(self) -> UVector4: ...
    @property
    def ltpp(self) -> UVector4: ...
    @property
    def ltpo(self) -> UVector4: ...
    @property
    def ltpl(self) -> UVector4: ...
    @property
    def ltos(self) -> UVector4: ...
    @property
    def ltot(self) -> UVector4: ...
    @property
    def ltoq(self) -> UVector4: ...
    @property
    def ltop(self) -> UVector4: ...
    @property
    def ltoo(self) -> UVector4: ...
    @property
    def ltol(self) -> UVector4: ...
    @property
    def ltls(self) -> UVector4: ...
    @property
    def ltlt(self) -> UVector4: ...
    @property
    def ltlq(self) -> UVector4: ...
    @property
    def ltlp(self) -> UVector4: ...
    @property
    def ltlo(self) -> UVector4: ...
    @property
    def ltll(self) -> UVector4: ...
    @property
    def lqss(self) -> UVector4: ...
    @property
    def lqst(self) -> UVector4: ...
    @property
    def lqsq(self) -> UVector4: ...
    @property
    def lqsp(self) -> UVector4: ...
    @property
    def lqso(self) -> UVector4: ...
    @property
    def lqsl(self) -> UVector4: ...
    @property
    def lqts(self) -> UVector4: ...
    @property
    def lqtt(self) -> UVector4: ...
    @property
    def lqtq(self) -> UVector4: ...
    @property
    def lqtp(self) -> UVector4: ...
    @property
    def lqto(self) -> UVector4: ...
    @property
    def lqtl(self) -> UVector4: ...
    @property
    def lqqs(self) -> UVector4: ...
    @property
    def lqqt(self) -> UVector4: ...
    @property
    def lqqq(self) -> UVector4: ...
    @property
    def lqqp(self) -> UVector4: ...
    @property
    def lqqo(self) -> UVector4: ...
    @property
    def lqql(self) -> UVector4: ...
    @property
    def lqps(self) -> UVector4: ...
    @property
    def lqpt(self) -> UVector4: ...
    @property
    def lqpq(self) -> UVector4: ...
    @property
    def lqpp(self) -> UVector4: ...
    @property
    def lqpo(self) -> UVector4: ...
    @property
    def lqpl(self) -> UVector4: ...
    @property
    def lqos(self) -> UVector4: ...
    @property
    def lqot(self) -> UVector4: ...
    @property
    def lqoq(self) -> UVector4: ...
    @property
    def lqop(self) -> UVector4: ...
    @property
    def lqoo(self) -> UVector4: ...
    @property
    def lqol(self) -> UVector4: ...
    @property
    def lqls(self) -> UVector4: ...
    @property
    def lqlt(self) -> UVector4: ...
    @property
    def lqlq(self) -> UVector4: ...
    @property
    def lqlp(self) -> UVector4: ...
    @property
    def lqlo(self) -> UVector4: ...
    @property
    def lqll(self) -> UVector4: ...
    @property
    def lpss(self) -> UVector4: ...
    @property
    def lpst(self) -> UVector4: ...
    @property
    def lpsq(self) -> UVector4: ...
    @property
    def lpsp(self) -> UVector4: ...
    @property
    def lpso(self) -> UVector4: ...
    @property
    def lpsl(self) -> UVector4: ...
    @property
    def lpts(self) -> UVector4: ...
    @property
    def lptt(self) -> UVector4: ...
    @property
    def lptq(self) -> UVector4: ...
    @property
    def lptp(self) -> UVector4: ...
    @property
    def lpto(self) -> UVector4: ...
    @property
    def lptl(self) -> UVector4: ...
    @property
    def lpqs(self) -> UVector4: ...
    @property
    def lpqt(self) -> UVector4: ...
    @property
    def lpqq(self) -> UVector4: ...
    @property
    def lpqp(self) -> UVector4: ...
    @property
    def lpqo(self) -> UVector4: ...
    @property
    def lpql(self) -> UVector4: ...
    @property
    def lpps(self) -> UVector4: ...
    @property
    def lppt(self) -> UVector4: ...
    @property
    def lppq(self) -> UVector4: ...
    @property
    def lppp(self) -> UVector4: ...
    @property
    def lppo(self) -> UVector4: ...
    @property
    def lppl(self) -> UVector4: ...
    @property
    def lpos(self) -> UVector4: ...
    @property
    def lpot(self) -> UVector4: ...
    @property
    def lpoq(self) -> UVector4: ...
    @property
    def lpop(self) -> UVector4: ...
    @property
    def lpoo(self) -> UVector4: ...
    @property
    def lpol(self) -> UVector4: ...
    @property
    def lpls(self) -> UVector4: ...
    @property
    def lplt(self) -> UVector4: ...
    @property
    def lplq(self) -> UVector4: ...
    @property
    def lplp(self) -> UVector4: ...
    @property
    def lplo(self) -> UVector4: ...
    @property
    def lpll(self) -> UVector4: ...
    @property
    def loss(self) -> UVector4: ...
    @property
    def lost(self) -> UVector4: ...
    @property
    def losq(self) -> UVector4: ...
    @property
    def losp(self) -> UVector4: ...
    @property
    def loso(self) -> UVector4: ...
    @property
    def losl(self) -> UVector4: ...
    @property
    def lots(self) -> UVector4: ...
    @property
    def lott(self) -> UVector4: ...
    @property
    def lotq(self) -> UVector4: ...
    @property
    def lotp(self) -> UVector4: ...
    @property
    def loto(self) -> UVector4: ...
    @property
    def lotl(self) -> UVector4: ...
    @property
    def loqs(self) -> UVector4: ...
    @property
    def loqt(self) -> UVector4: ...
    @property
    def loqq(self) -> UVector4: ...
    @property
    def loqp(self) -> UVector4: ...
    @property
    def loqo(self) -> UVector4: ...
    @property
    def loql(self) -> UVector4: ...
    @property
    def lops(self) -> UVector4: ...
    @property
    def lopt(self) -> UVector4: ...
    @property
    def lopq(self) -> UVector4: ...
    @property
    def lopp(self) -> UVector4: ...
    @property
    def lopo(self) -> UVector4: ...
    @property
    def lopl(self) -> UVector4: ...
    @property
    def loos(self) -> UVector4: ...
    @property
    def loot(self) -> UVector4: ...
    @property
    def looq(self) -> UVector4: ...
    @property
    def loop(self) -> UVector4: ...
    @property
    def lols(self) -> UVector4: ...
    @property
    def lolt(self) -> UVector4: ...
    @property
    def lolq(self) -> UVector4: ...
    @property
    def lolp(self) -> UVector4: ...
    @property
    def llss(self) -> UVector4: ...
    @property
    def llst(self) -> UVector4: ...
    @property
    def llsq(self) -> UVector4: ...
    @property
    def llsp(self) -> UVector4: ...
    @property
    def llso(self) -> UVector4: ...
    @property
    def llsl(self) -> UVector4: ...
    @property
    def llts(self) -> UVector4: ...
    @property
    def lltt(self) -> UVector4: ...
    @property
    def lltq(self) -> UVector4: ...
    @property
    def lltp(self) -> UVector4: ...
    @property
    def llto(self) -> UVector4: ...
    @property
    def lltl(self) -> UVector4: ...
    @property
    def llqs(self) -> UVector4: ...
    @property
    def llqt(self) -> UVector4: ...
    @property
    def llqq(self) -> UVector4: ...
    @property
    def llqp(self) -> UVector4: ...
    @property
    def llqo(self) -> UVector4: ...
    @property
    def llql(self) -> UVector4: ...
    @property
    def llps(self) -> UVector4: ...
    @property
    def llpt(self) -> UVector4: ...
    @property
    def llpq(self) -> UVector4: ...
    @property
    def llpp(self) -> UVector4: ...
    @property
    def llpo(self) -> UVector4: ...
    @property
    def llpl(self) -> UVector4: ...
    @property
    def llos(self) -> UVector4: ...
    @property
    def llot(self) -> UVector4: ...
    @property
    def lloq(self) -> UVector4: ...
    @property
    def llop(self) -> UVector4: ...
    @property
    def llls(self) -> UVector4: ...
    @property
    def lllt(self) -> UVector4: ...
    @property
    def lllq(self) -> UVector4: ...
    @property
    def lllp(self) -> UVector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> UVector2: ...
    @property
    def uv(self) -> UVector2: ...
    @property
    def uo(self) -> UVector2: ...
    @property
    def ul(self) -> UVector2: ...
    @property
    def vu(self) -> UVector2: ...
    @property
    def vv(self) -> UVector2: ...
    @property
    def vo(self) -> UVector2: ...
    @property
    def vl(self) -> UVector2: ...
    @property
    def ou(self) -> UVector2: ...
    @property
    def ov(self) -> UVector2: ...
    @property
    def lu(self) -> UVector2: ...
    @property
    def lv(self) -> UVector2: ...
    @property
    def uuu(self) -> UVector3: ...
    @property
    def uuv(self) -> UVector3: ...
    @property
    def uuo(self) -> UVector3: ...
    @property
    def uul(self) -> UVector3: ...
    @property
    def uvu(self) -> UVector3: ...
    @property
    def uvv(self) -> UVector3: ...
    @property
    def uvo(self) -> UVector3: ...
    @property
    def uvl(self) -> UVector3: ...
    @property
    def uou(self) -> UVector3: ...
    @property
    def uov(self) -> UVector3: ...
    @property
    def uoo(self) -> UVector3: ...
    @property
    def uol(self) -> UVector3: ...
    @property
    def ulu(self) -> UVector3: ...
    @property
    def ulv(self) -> UVector3: ...
    @property
    def ulo(self) -> UVector3: ...
    @property
    def ull(self) -> UVector3: ...
    @property
    def vuu(self) -> UVector3: ...
    @property
    def vuv(self) -> UVector3: ...
    @property
    def vuo(self) -> UVector3: ...
    @property
    def vul(self) -> UVector3: ...
    @property
    def vvu(self) -> UVector3: ...
    @property
    def vvv(self) -> UVector3: ...
    @property
    def vvo(self) -> UVector3: ...
    @property
    def vvl(self) -> UVector3: ...
    @property
    def vou(self) -> UVector3: ...
    @property
    def vov(self) -> UVector3: ...
    @property
    def voo(self) -> UVector3: ...
    @property
    def vol(self) -> UVector3: ...
    @property
    def vlu(self) -> UVector3: ...
    @property
    def vlv(self) -> UVector3: ...
    @property
    def vlo(self) -> UVector3: ...
    @property
    def vll(self) -> UVector3: ...
    @property
    def ouu(self) -> UVector3: ...
    @property
    def ouv(self) -> UVector3: ...
    @property
    def ouo(self) -> UVector3: ...
    @property
    def oul(self) -> UVector3: ...
    @property
    def ovu(self) -> UVector3: ...
    @property
    def ovv(self) -> UVector3: ...
    @property
    def ovo(self) -> UVector3: ...
    @property
    def ovl(self) -> UVector3: ...
    @property
    def oou(self) -> UVector3: ...
    @property
    def oov(self) -> UVector3: ...
    @property
    def olu(self) -> UVector3: ...
    @property
    def olv(self) -> UVector3: ...
    @property
    def luu(self) -> UVector3: ...
    @property
    def luv(self) -> UVector3: ...
    @property
    def luo(self) -> UVector3: ...
    @property
    def lul(self) -> UVector3: ...
    @property
    def lvu(self) -> UVector3: ...
    @property
    def lvv(self) -> UVector3: ...
    @property
    def lvo(self) -> UVector3: ...
    @property
    def lvl(self) -> UVector3: ...
    @property
    def lou(self) -> UVector3: ...
    @property
    def lov(self) -> UVector3: ...
    @property
    def llu(self) -> UVector3: ...
    @property
    def llv(self) -> UVector3: ...
    @property
    def uuuu(self) -> UVector4: ...
    @property
    def uuuv(self) -> UVector4: ...
    @property
    def uuuo(self) -> UVector4: ...
    @property
    def uuul(self) -> UVector4: ...
    @property
    def uuvu(self) -> UVector4: ...
    @property
    def uuvv(self) -> UVector4: ...
    @property
    def uuvo(self) -> UVector4: ...
    @property
    def uuvl(self) -> UVector4: ...
    @property
    def uuou(self) -> UVector4: ...
    @property
    def uuov(self) -> UVector4: ...
    @property
    def uuoo(self) -> UVector4: ...
    @property
    def uuol(self) -> UVector4: ...
    @property
    def uulu(self) -> UVector4: ...
    @property
    def uulv(self) -> UVector4: ...
    @property
    def uulo(self) -> UVector4: ...
    @property
    def uull(self) -> UVector4: ...
    @property
    def uvuu(self) -> UVector4: ...
    @property
    def uvuv(self) -> UVector4: ...
    @property
    def uvuo(self) -> UVector4: ...
    @property
    def uvul(self) -> UVector4: ...
    @property
    def uvvu(self) -> UVector4: ...
    @property
    def uvvv(self) -> UVector4: ...
    @property
    def uvvo(self) -> UVector4: ...
    @property
    def uvvl(self) -> UVector4: ...
    @property
    def uvou(self) -> UVector4: ...
    @property
    def uvov(self) -> UVector4: ...
    @property
    def uvoo(self) -> UVector4: ...
    @property
    def uvol(self) -> UVector4: ...
    @property
    def uvlu(self) -> UVector4: ...
    @property
    def uvlv(self) -> UVector4: ...
    @property
    def uvlo(self) -> UVector4: ...
    @property
    def uvll(self) -> UVector4: ...
    @property
    def uouu(self) -> UVector4: ...
    @property
    def uouv(self) -> UVector4: ...
    @property
    def uouo(self) -> UVector4: ...
    @property
    def uoul(self) -> UVector4: ...
    @property
    def uovu(self) -> UVector4: ...
    @property
    def uovv(self) -> UVector4: ...
    @property
    def uovo(self) -> UVector4: ...
    @property
    def uovl(self) -> UVector4: ...
    @property
    def uoou(self) -> UVector4: ...
    @property
    def uoov(self) -> UVector4: ...
    @property
    def uooo(self) -> UVector4: ...
    @property
    def uool(self) -> UVector4: ...
    @property
    def uolu(self) -> UVector4: ...
    @property
    def uolv(self) -> UVector4: ...
    @property
    def uolo(self) -> UVector4: ...
    @property
    def uoll(self) -> UVector4: ...
    @property
    def uluu(self) -> UVector4: ...
    @property
    def uluv(self) -> UVector4: ...
    @property
    def uluo(self) -> UVector4: ...
    @property
    def ulul(self) -> UVector4: ...
    @property
    def ulvu(self) -> UVector4: ...
    @property
    def ulvv(self) -> UVector4: ...
    @property
    def ulvo(self) -> UVector4: ...
    @property
    def ulvl(self) -> UVector4: ...
    @property
    def ulou(self) -> UVector4: ...
    @property
    def ulov(self) -> UVector4: ...
    @property
    def uloo(self) -> UVector4: ...
    @property
    def ulol(self) -> UVector4: ...
    @property
    def ullu(self) -> UVector4: ...
    @property
    def ullv(self) -> UVector4: ...
    @property
    def ullo(self) -> UVector4: ...
    @property
    def ulll(self) -> UVector4: ...
    @property
    def vuuu(self) -> UVector4: ...
    @property
    def vuuv(self) -> UVector4: ...
    @property
    def vuuo(self) -> UVector4: ...
    @property
    def vuul(self) -> UVector4: ...
    @property
    def vuvu(self) -> UVector4: ...
    @property
    def vuvv(self) -> UVector4: ...
    @property
    def vuvo(self) -> UVector4: ...
    @property
    def vuvl(self) -> UVector4: ...
    @property
    def vuou(self) -> UVector4: ...
    @property
    def vuov(self) -> UVector4: ...
    @property
    def vuoo(self) -> UVector4: ...
    @property
    def vuol(self) -> UVector4: ...
    @property
    def vulu(self) -> UVector4: ...
    @property
    def vulv(self) -> UVector4: ...
    @property
    def vulo(self) -> UVector4: ...
    @property
    def vull(self) -> UVector4: ...
    @property
    def vvuu(self) -> UVector4: ...
    @property
    def vvuv(self) -> UVector4: ...
    @property
    def vvuo(self) -> UVector4: ...
    @property
    def vvul(self) -> UVector4: ...
    @property
    def vvvu(self) -> UVector4: ...
    @property
    def vvvv(self) -> UVector4: ...
    @property
    def vvvo(self) -> UVector4: ...
    @property
    def vvvl(self) -> UVector4: ...
    @property
    def vvou(self) -> UVector4: ...
    @property
    def vvov(self) -> UVector4: ...
    @property
    def vvoo(self) -> UVector4: ...
    @property
    def vvol(self) -> UVector4: ...
    @property
    def vvlu(self) -> UVector4: ...
    @property
    def vvlv(self) -> UVector4: ...
    @property
    def vvlo(self) -> UVector4: ...
    @property
    def vvll(self) -> UVector4: ...
    @property
    def vouu(self) -> UVector4: ...
    @property
    def vouv(self) -> UVector4: ...
    @property
    def vouo(self) -> UVector4: ...
    @property
    def voul(self) -> UVector4: ...
    @property
    def vovu(self) -> UVector4: ...
    @property
    def vovv(self) -> UVector4: ...
    @property
    def vovo(self) -> UVector4: ...
    @property
    def vovl(self) -> UVector4: ...
    @property
    def voou(self) -> UVector4: ...
    @property
    def voov(self) -> UVector4: ...
    @property
    def vooo(self) -> UVector4: ...
    @property
    def vool(self) -> UVector4: ...
    @property
    def volu(self) -> UVector4: ...
    @property
    def volv(self) -> UVector4: ...
    @property
    def volo(self) -> UVector4: ...
    @property
    def voll(self) -> UVector4: ...
    @property
    def vluu(self) -> UVector4: ...
    @property
    def vluv(self) -> UVector4: ...
    @property
    def vluo(self) -> UVector4: ...
    @property
    def vlul(self) -> UVector4: ...
    @property
    def vlvu(self) -> UVector4: ...
    @property
    def vlvv(self) -> UVector4: ...
    @property
    def vlvo(self) -> UVector4: ...
    @property
    def vlvl(self) -> UVector4: ...
    @property
    def vlou(self) -> UVector4: ...
    @property
    def vlov(self) -> UVector4: ...
    @property
    def vloo(self) -> UVector4: ...
    @property
    def vlol(self) -> UVector4: ...
    @property
    def vllu(self) -> UVector4: ...
    @property
    def vllv(self) -> UVector4: ...
    @property
    def vllo(self) -> UVector4: ...
    @property
    def vlll(self) -> UVector4: ...
    @property
    def ouuu(self) -> UVector4: ...
    @property
    def ouuv(self) -> UVector4: ...
    @property
    def ouuo(self) -> UVector4: ...
    @property
    def ouul(self) -> UVector4: ...
    @property
    def ouvu(self) -> UVector4: ...
    @property
    def ouvv(self) -> UVector4: ...
    @property
    def ouvo(self) -> UVector4: ...
    @property
    def ouvl(self) -> UVector4: ...
    @property
    def ouou(self) -> UVector4: ...
    @property
    def ouov(self) -> UVector4: ...
    @property
    def ouoo(self) -> UVector4: ...
    @property
    def ouol(self) -> UVector4: ...
    @property
    def oulu(self) -> UVector4: ...
    @property
    def oulv(self) -> UVector4: ...
    @property
    def oulo(self) -> UVector4: ...
    @property
    def oull(self) -> UVector4: ...
    @property
    def ovuu(self) -> UVector4: ...
    @property
    def ovuv(self) -> UVector4: ...
    @property
    def ovuo(self) -> UVector4: ...
    @property
    def ovul(self) -> UVector4: ...
    @property
    def ovvu(self) -> UVector4: ...
    @property
    def ovvv(self) -> UVector4: ...
    @property
    def ovvo(self) -> UVector4: ...
    @property
    def ovvl(self) -> UVector4: ...
    @property
    def ovou(self) -> UVector4: ...
    @property
    def ovov(self) -> UVector4: ...
    @property
    def ovoo(self) -> UVector4: ...
    @property
    def ovol(self) -> UVector4: ...
    @property
    def ovlu(self) -> UVector4: ...
    @property
    def ovlv(self) -> UVector4: ...
    @property
    def ovlo(self) -> UVector4: ...
    @property
    def ovll(self) -> UVector4: ...
    @property
    def oouu(self) -> UVector4: ...
    @property
    def oouv(self) -> UVector4: ...
    @property
    def oouo(self) -> UVector4: ...
    @property
    def ooul(self) -> UVector4: ...
    @property
    def oovu(self) -> UVector4: ...
    @property
    def oovv(self) -> UVector4: ...
    @property
    def oovo(self) -> UVector4: ...
    @property
    def oovl(self) -> UVector4: ...
    @property
    def ooou(self) -> UVector4: ...
    @property
    def ooov(self) -> UVector4: ...
    @property
    def oolu(self) -> UVector4: ...
    @property
    def oolv(self) -> UVector4: ...
    @property
    def oluu(self) -> UVector4: ...
    @property
    def oluv(self) -> UVector4: ...
    @property
    def oluo(self) -> UVector4: ...
    @property
    def olul(self) -> UVector4: ...
    @property
    def olvu(self) -> UVector4: ...
    @property
    def olvv(self) -> UVector4: ...
    @property
    def olvo(self) -> UVector4: ...
    @property
    def olvl(self) -> UVector4: ...
    @property
    def olou(self) -> UVector4: ...
    @property
    def olov(self) -> UVector4: ...
    @property
    def ollu(self) -> UVector4: ...
    @property
    def ollv(self) -> UVector4: ...
    @property
    def luuu(self) -> UVector4: ...
    @property
    def luuv(self) -> UVector4: ...
    @property
    def luuo(self) -> UVector4: ...
    @property
    def luul(self) -> UVector4: ...
    @property
    def luvu(self) -> UVector4: ...
    @property
    def luvv(self) -> UVector4: ...
    @property
    def luvo(self) -> UVector4: ...
    @property
    def luvl(self) -> UVector4: ...
    @property
    def luou(self) -> UVector4: ...
    @property
    def luov(self) -> UVector4: ...
    @property
    def luoo(self) -> UVector4: ...
    @property
    def luol(self) -> UVector4: ...
    @property
    def lulu(self) -> UVector4: ...
    @property
    def lulv(self) -> UVector4: ...
    @property
    def lulo(self) -> UVector4: ...
    @property
    def lull(self) -> UVector4: ...
    @property
    def lvuu(self) -> UVector4: ...
    @property
    def lvuv(self) -> UVector4: ...
    @property
    def lvuo(self) -> UVector4: ...
    @property
    def lvul(self) -> UVector4: ...
    @property
    def lvvu(self) -> UVector4: ...
    @property
    def lvvv(self) -> UVector4: ...
    @property
    def lvvo(self) -> UVector4: ...
    @property
    def lvvl(self) -> UVector4: ...
    @property
    def lvou(self) -> UVector4: ...
    @property
    def lvov(self) -> UVector4: ...
    @property
    def lvoo(self) -> UVector4: ...
    @property
    def lvol(self) -> UVector4: ...
    @property
    def lvlu(self) -> UVector4: ...
    @property
    def lvlv(self) -> UVector4: ...
    @property
    def lvlo(self) -> UVector4: ...
    @property
    def lvll(self) -> UVector4: ...
    @property
    def louu(self) -> UVector4: ...
    @property
    def louv(self) -> UVector4: ...
    @property
    def louo(self) -> UVector4: ...
    @property
    def loul(self) -> UVector4: ...
    @property
    def lovu(self) -> UVector4: ...
    @property
    def lovv(self) -> UVector4: ...
    @property
    def lovo(self) -> UVector4: ...
    @property
    def lovl(self) -> UVector4: ...
    @property
    def loou(self) -> UVector4: ...
    @property
    def loov(self) -> UVector4: ...
    @property
    def lolu(self) -> UVector4: ...
    @property
    def lolv(self) -> UVector4: ...
    @property
    def lluu(self) -> UVector4: ...
    @property
    def lluv(self) -> UVector4: ...
    @property
    def lluo(self) -> UVector4: ...
    @property
    def llul(self) -> UVector4: ...
    @property
    def llvu(self) -> UVector4: ...
    @property
    def llvv(self) -> UVector4: ...
    @property
    def llvo(self) -> UVector4: ...
    @property
    def llvl(self) -> UVector4: ...
    @property
    def llou(self) -> UVector4: ...
    @property
    def llov(self) -> UVector4: ...
    @property
    def lllu(self) -> UVector4: ...
    @property
    def lllv(self) -> UVector4: ...
    @property
    def oo(self) -> UVector2: ...
    @property
    def ol(self) -> UVector2: ...
    @property
    def lo(self) -> UVector2: ...
    @property
    def ll(self) -> UVector2: ...
    @property
    def ooo(self) -> UVector3: ...
    @property
    def ool(self) -> UVector3: ...
    @property
    def olo(self) -> UVector3: ...
    @property
    def oll(self) -> UVector3: ...
    @property
    def loo(self) -> UVector3: ...
    @property
    def lol(self) -> UVector3: ...
    @property
    def llo(self) -> UVector3: ...
    @property
    def lll(self) -> UVector3: ...
    @property
    def oooo(self) -> UVector4: ...
    @property
    def oool(self) -> UVector4: ...
    @property
    def oolo(self) -> UVector4: ...
    @property
    def ooll(self) -> UVector4: ...
    @property
    def oloo(self) -> UVector4: ...
    @property
    def olol(self) -> UVector4: ...
    @property
    def ollo(self) -> UVector4: ...
    @property
    def olll(self) -> UVector4: ...
    @property
    def looo(self) -> UVector4: ...
    @property
    def lool(self) -> UVector4: ...
    @property
    def lolo(self) -> UVector4: ...
    @property
    def loll(self) -> UVector4: ...
    @property
    def lloo(self) -> UVector4: ...
    @property
    def llol(self) -> UVector4: ...
    @property
    def lllo(self) -> UVector4: ...
    @property
    def llll(self) -> UVector4: ...
    def min(self, n: Number, /) -> UVector4: ...
    def max(self, n: Number, /) -> UVector4: ...
    def clamp(self, min: Number, max: Number, /) -> UVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[UVector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector4: ...

@final
class UVector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: UVector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[UVector4]: ...
    @overload
    def __getitem__(self, index: int) -> UVector4: ...
    @overload
    def __getitem__(self, index: slice) -> UVector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> UVector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[UVector4]: ...

@final
class I64Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __add__(self, other: Number) -> I64Vector4: ...
    @overload
    def __radd__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __radd__(self, other: Number) -> I64Vector4: ...
    @overload
    def __sub__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __sub__(self, other: Number) -> I64Vector4: ...
    @overload
    def __rsub__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> I64Vector4: ...
    @overload
    def __mul__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __mul__(self, other: Number) -> I64Vector4: ...
    @overload
    def __rmul__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> I64Vector4: ...
    @overload
    def __truediv__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> I64Vector4: ...
    @overload
    def __rtruediv__(self, other: I64Vector4) -> I64Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> I64Vector4: ...
    def __neg__(self) -> I64Vector4: ...
    def __abs__(self) -> I64Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> I64Vector2: ...
    @property
    def xy(self) -> I64Vector2: ...
    @property
    def xz(self) -> I64Vector2: ...
    @property
    def xw(self) -> I64Vector2: ...
    @property
    def xo(self) -> I64Vector2: ...
    @property
    def xl(self) -> I64Vector2: ...
    @property
    def yx(self) -> I64Vector2: ...
    @property
    def yy(self) -> I64Vector2: ...
    @property
    def yz(self) -> I64Vector2: ...
    @property
    def yw(self) -> I64Vector2: ...
    @property
    def yo(self) -> I64Vector2: ...
    @property
    def yl(self) -> I64Vector2: ...
    @property
    def zx(self) -> I64Vector2: ...
    @property
    def zy(self) -> I64Vector2: ...
    @property
    def zz(self) -> I64Vector2: ...
    @property
    def zw(self) -> I64Vector2: ...
    @property
    def zo(self) -> I64Vector2: ...
    @property
    def zl(self) -> I64Vector2: ...
    @property
    def wx(self) -> I64Vector2: ...
    @property
    def wy(self) -> I64Vector2: ...
    @property
    def wz(self) -> I64Vector2: ...
    @property
    def ww(self) -> I64Vector2: ...
    @property
    def wo(self) -> I64Vector2: ...
    @property
    def wl(self) -> I64Vector2: ...
    @property
    def ox(self) -> I64Vector2: ...
    @property
    def oy(self) -> I64Vector2: ...
    @property
    def oz(self) -> I64Vector2: ...
    @property
    def ow(self) -> I64Vector2: ...
    @property
    def lx(self) -> I64Vector2: ...
    @property
    def ly(self) -> I64Vector2: ...
    @property
    def lz(self) -> I64Vector2: ...
    @property
    def lw(self) -> I64Vector2: ...
    @property
    def xxx(self) -> I64Vector3: ...
    @property
    def xxy(self) -> I64Vector3: ...
    @property
    def xxz(self) -> I64Vector3: ...
    @property
    def xxw(self) -> I64Vector3: ...
    @property
    def xxo(self) -> I64Vector3: ...
    @property
    def xxl(self) -> I64Vector3: ...
    @property
    def xyx(self) -> I64Vector3: ...
    @property
    def xyy(self) -> I64Vector3: ...
    @property
    def xyz(self) -> I64Vector3: ...
    @property
    def xyw(self) -> I64Vector3: ...
    @property
    def xyo(self) -> I64Vector3: ...
    @property
    def xyl(self) -> I64Vector3: ...
    @property
    def xzx(self) -> I64Vector3: ...
    @property
    def xzy(self) -> I64Vector3: ...
    @property
    def xzz(self) -> I64Vector3: ...
    @property
    def xzw(self) -> I64Vector3: ...
    @property
    def xzo(self) -> I64Vector3: ...
    @property
    def xzl(self) -> I64Vector3: ...
    @property
    def xwx(self) -> I64Vector3: ...
    @property
    def xwy(self) -> I64Vector3: ...
    @property
    def xwz(self) -> I64Vector3: ...
    @property
    def xww(self) -> I64Vector3: ...
    @property
    def xwo(self) -> I64Vector3: ...
    @property
    def xwl(self) -> I64Vector3: ...
    @property
    def xox(self) -> I64Vector3: ...
    @property
    def xoy(self) -> I64Vector3: ...
    @property
    def xoz(self) -> I64Vector3: ...
    @property
    def xow(self) -> I64Vector3: ...
    @property
    def xoo(self) -> I64Vector3: ...
    @property
    def xol(self) -> I64Vector3: ...
    @property
    def xlx(self) -> I64Vector3: ...
    @property
    def xly(self) -> I64Vector3: ...
    @property
    def xlz(self) -> I64Vector3: ...
    @property
    def xlw(self) -> I64Vector3: ...
    @property
    def xlo(self) -> I64Vector3: ...
    @property
    def xll(self) -> I64Vector3: ...
    @property
    def yxx(self) -> I64Vector3: ...
    @property
    def yxy(self) -> I64Vector3: ...
    @property
    def yxz(self) -> I64Vector3: ...
    @property
    def yxw(self) -> I64Vector3: ...
    @property
    def yxo(self) -> I64Vector3: ...
    @property
    def yxl(self) -> I64Vector3: ...
    @property
    def yyx(self) -> I64Vector3: ...
    @property
    def yyy(self) -> I64Vector3: ...
    @property
    def yyz(self) -> I64Vector3: ...
    @property
    def yyw(self) -> I64Vector3: ...
    @property
    def yyo(self) -> I64Vector3: ...
    @property
    def yyl(self) -> I64Vector3: ...
    @property
    def yzx(self) -> I64Vector3: ...
    @property
    def yzy(self) -> I64Vector3: ...
    @property
    def yzz(self) -> I64Vector3: ...
    @property
    def yzw(self) -> I64Vector3: ...
    @property
    def yzo(self) -> I64Vector3: ...
    @property
    def yzl(self) -> I64Vector3: ...
    @property
    def ywx(self) -> I64Vector3: ...
    @property
    def ywy(self) -> I64Vector3: ...
    @property
    def ywz(self) -> I64Vector3: ...
    @property
    def yww(self) -> I64Vector3: ...
    @property
    def ywo(self) -> I64Vector3: ...
    @property
    def ywl(self) -> I64Vector3: ...
    @property
    def yox(self) -> I64Vector3: ...
    @property
    def yoy(self) -> I64Vector3: ...
    @property
    def yoz(self) -> I64Vector3: ...
    @property
    def yow(self) -> I64Vector3: ...
    @property
    def yoo(self) -> I64Vector3: ...
    @property
    def yol(self) -> I64Vector3: ...
    @property
    def ylx(self) -> I64Vector3: ...
    @property
    def yly(self) -> I64Vector3: ...
    @property
    def ylz(self) -> I64Vector3: ...
    @property
    def ylw(self) -> I64Vector3: ...
    @property
    def ylo(self) -> I64Vector3: ...
    @property
    def yll(self) -> I64Vector3: ...
    @property
    def zxx(self) -> I64Vector3: ...
    @property
    def zxy(self) -> I64Vector3: ...
    @property
    def zxz(self) -> I64Vector3: ...
    @property
    def zxw(self) -> I64Vector3: ...
    @property
    def zxo(self) -> I64Vector3: ...
    @property
    def zxl(self) -> I64Vector3: ...
    @property
    def zyx(self) -> I64Vector3: ...
    @property
    def zyy(self) -> I64Vector3: ...
    @property
    def zyz(self) -> I64Vector3: ...
    @property
    def zyw(self) -> I64Vector3: ...
    @property
    def zyo(self) -> I64Vector3: ...
    @property
    def zyl(self) -> I64Vector3: ...
    @property
    def zzx(self) -> I64Vector3: ...
    @property
    def zzy(self) -> I64Vector3: ...
    @property
    def zzz(self) -> I64Vector3: ...
    @property
    def zzw(self) -> I64Vector3: ...
    @property
    def zzo(self) -> I64Vector3: ...
    @property
    def zzl(self) -> I64Vector3: ...
    @property
    def zwx(self) -> I64Vector3: ...
    @property
    def zwy(self) -> I64Vector3: ...
    @property
    def zwz(self) -> I64Vector3: ...
    @property
    def zww(self) -> I64Vector3: ...
    @property
    def zwo(self) -> I64Vector3: ...
    @property
    def zwl(self) -> I64Vector3: ...
    @property
    def zox(self) -> I64Vector3: ...
    @property
    def zoy(self) -> I64Vector3: ...
    @property
    def zoz(self) -> I64Vector3: ...
    @property
    def zow(self) -> I64Vector3: ...
    @property
    def zoo(self) -> I64Vector3: ...
    @property
    def zol(self) -> I64Vector3: ...
    @property
    def zlx(self) -> I64Vector3: ...
    @property
    def zly(self) -> I64Vector3: ...
    @property
    def zlz(self) -> I64Vector3: ...
    @property
    def zlw(self) -> I64Vector3: ...
    @property
    def zlo(self) -> I64Vector3: ...
    @property
    def zll(self) -> I64Vector3: ...
    @property
    def wxx(self) -> I64Vector3: ...
    @property
    def wxy(self) -> I64Vector3: ...
    @property
    def wxz(self) -> I64Vector3: ...
    @property
    def wxw(self) -> I64Vector3: ...
    @property
    def wxo(self) -> I64Vector3: ...
    @property
    def wxl(self) -> I64Vector3: ...
    @property
    def wyx(self) -> I64Vector3: ...
    @property
    def wyy(self) -> I64Vector3: ...
    @property
    def wyz(self) -> I64Vector3: ...
    @property
    def wyw(self) -> I64Vector3: ...
    @property
    def wyo(self) -> I64Vector3: ...
    @property
    def wyl(self) -> I64Vector3: ...
    @property
    def wzx(self) -> I64Vector3: ...
    @property
    def wzy(self) -> I64Vector3: ...
    @property
    def wzz(self) -> I64Vector3: ...
    @property
    def wzw(self) -> I64Vector3: ...
    @property
    def wzo(self) -> I64Vector3: ...
    @property
    def wzl(self) -> I64Vector3: ...
    @property
    def wwx(self) -> I64Vector3: ...
    @property
    def wwy(self) -> I64Vector3: ...
    @property
    def wwz(self) -> I64Vector3: ...
    @property
    def www(self) -> I64Vector3: ...
    @property
    def wwo(self) -> I64Vector3: ...
    @property
    def wwl(self) -> I64Vector3: ...
    @property
    def wox(self) -> I64Vector3: ...
    @property
    def woy(self) -> I64Vector3: ...
    @property
    def woz(self) -> I64Vector3: ...
    @property
    def wow(self) -> I64Vector3: ...
    @property
    def woo(self) -> I64Vector3: ...
    @property
    def wol(self) -> I64Vector3: ...
    @property
    def wlx(self) -> I64Vector3: ...
    @property
    def wly(self) -> I64Vector3: ...
    @property
    def wlz(self) -> I64Vector3: ...
    @property
    def wlw(self) -> I64Vector3: ...
    @property
    def wlo(self) -> I64Vector3: ...
    @property
    def wll(self) -> I64Vector3: ...
    @property
    def oxx(self) -> I64Vector3: ...
    @property
    def oxy(self) -> I64Vector3: ...
    @property
    def oxz(self) -> I64Vector3: ...
    @property
    def oxw(self) -> I64Vector3: ...
    @property
    def oxo(self) -> I64Vector3: ...
    @property
    def oxl(self) -> I64Vector3: ...
    @property
    def oyx(self) -> I64Vector3: ...
    @property
    def oyy(self) -> I64Vector3: ...
    @property
    def oyz(self) -> I64Vector3: ...
    @property
    def oyw(self) -> I64Vector3: ...
    @property
    def oyo(self) -> I64Vector3: ...
    @property
    def oyl(self) -> I64Vector3: ...
    @property
    def ozx(self) -> I64Vector3: ...
    @property
    def ozy(self) -> I64Vector3: ...
    @property
    def ozz(self) -> I64Vector3: ...
    @property
    def ozw(self) -> I64Vector3: ...
    @property
    def ozo(self) -> I64Vector3: ...
    @property
    def ozl(self) -> I64Vector3: ...
    @property
    def owx(self) -> I64Vector3: ...
    @property
    def owy(self) -> I64Vector3: ...
    @property
    def owz(self) -> I64Vector3: ...
    @property
    def oww(self) -> I64Vector3: ...
    @property
    def owo(self) -> I64Vector3: ...
    @property
    def owl(self) -> I64Vector3: ...
    @property
    def oox(self) -> I64Vector3: ...
    @property
    def ooy(self) -> I64Vector3: ...
    @property
    def ooz(self) -> I64Vector3: ...
    @property
    def oow(self) -> I64Vector3: ...
    @property
    def olx(self) -> I64Vector3: ...
    @property
    def oly(self) -> I64Vector3: ...
    @property
    def olz(self) -> I64Vector3: ...
    @property
    def olw(self) -> I64Vector3: ...
    @property
    def lxx(self) -> I64Vector3: ...
    @property
    def lxy(self) -> I64Vector3: ...
    @property
    def lxz(self) -> I64Vector3: ...
    @property
    def lxw(self) -> I64Vector3: ...
    @property
    def lxo(self) -> I64Vector3: ...
    @property
    def lxl(self) -> I64Vector3: ...
    @property
    def lyx(self) -> I64Vector3: ...
    @property
    def lyy(self) -> I64Vector3: ...
    @property
    def lyz(self) -> I64Vector3: ...
    @property
    def lyw(self) -> I64Vector3: ...
    @property
    def lyo(self) -> I64Vector3: ...
    @property
    def lyl(self) -> I64Vector3: ...
    @property
    def lzx(self) -> I64Vector3: ...
    @property
    def lzy(self) -> I64Vector3: ...
    @property
    def lzz(self) -> I64Vector3: ...
    @property
    def lzw(self) -> I64Vector3: ...
    @property
    def lzo(self) -> I64Vector3: ...
    @property
    def lzl(self) -> I64Vector3: ...
    @property
    def lwx(self) -> I64Vector3: ...
    @property
    def lwy(self) -> I64Vector3: ...
    @property
    def lwz(self) -> I64Vector3: ...
    @property
    def lww(self) -> I64Vector3: ...
    @property
    def lwo(self) -> I64Vector3: ...
    @property
    def lwl(self) -> I64Vector3: ...
    @property
    def lox(self) -> I64Vector3: ...
    @property
    def loy(self) -> I64Vector3: ...
    @property
    def loz(self) -> I64Vector3: ...
    @property
    def low(self) -> I64Vector3: ...
    @property
    def llx(self) -> I64Vector3: ...
    @property
    def lly(self) -> I64Vector3: ...
    @property
    def llz(self) -> I64Vector3: ...
    @property
    def llw(self) -> I64Vector3: ...
    @property
    def xxxx(self) -> I64Vector4: ...
    @property
    def xxxy(self) -> I64Vector4: ...
    @property
    def xxxz(self) -> I64Vector4: ...
    @property
    def xxxw(self) -> I64Vector4: ...
    @property
    def xxxo(self) -> I64Vector4: ...
    @property
    def xxxl(self) -> I64Vector4: ...
    @property
    def xxyx(self) -> I64Vector4: ...
    @property
    def xxyy(self) -> I64Vector4: ...
    @property
    def xxyz(self) -> I64Vector4: ...
    @property
    def xxyw(self) -> I64Vector4: ...
    @property
    def xxyo(self) -> I64Vector4: ...
    @property
    def xxyl(self) -> I64Vector4: ...
    @property
    def xxzx(self) -> I64Vector4: ...
    @property
    def xxzy(self) -> I64Vector4: ...
    @property
    def xxzz(self) -> I64Vector4: ...
    @property
    def xxzw(self) -> I64Vector4: ...
    @property
    def xxzo(self) -> I64Vector4: ...
    @property
    def xxzl(self) -> I64Vector4: ...
    @property
    def xxwx(self) -> I64Vector4: ...
    @property
    def xxwy(self) -> I64Vector4: ...
    @property
    def xxwz(self) -> I64Vector4: ...
    @property
    def xxww(self) -> I64Vector4: ...
    @property
    def xxwo(self) -> I64Vector4: ...
    @property
    def xxwl(self) -> I64Vector4: ...
    @property
    def xxox(self) -> I64Vector4: ...
    @property
    def xxoy(self) -> I64Vector4: ...
    @property
    def xxoz(self) -> I64Vector4: ...
    @property
    def xxow(self) -> I64Vector4: ...
    @property
    def xxoo(self) -> I64Vector4: ...
    @property
    def xxol(self) -> I64Vector4: ...
    @property
    def xxlx(self) -> I64Vector4: ...
    @property
    def xxly(self) -> I64Vector4: ...
    @property
    def xxlz(self) -> I64Vector4: ...
    @property
    def xxlw(self) -> I64Vector4: ...
    @property
    def xxlo(self) -> I64Vector4: ...
    @property
    def xxll(self) -> I64Vector4: ...
    @property
    def xyxx(self) -> I64Vector4: ...
    @property
    def xyxy(self) -> I64Vector4: ...
    @property
    def xyxz(self) -> I64Vector4: ...
    @property
    def xyxw(self) -> I64Vector4: ...
    @property
    def xyxo(self) -> I64Vector4: ...
    @property
    def xyxl(self) -> I64Vector4: ...
    @property
    def xyyx(self) -> I64Vector4: ...
    @property
    def xyyy(self) -> I64Vector4: ...
    @property
    def xyyz(self) -> I64Vector4: ...
    @property
    def xyyw(self) -> I64Vector4: ...
    @property
    def xyyo(self) -> I64Vector4: ...
    @property
    def xyyl(self) -> I64Vector4: ...
    @property
    def xyzx(self) -> I64Vector4: ...
    @property
    def xyzy(self) -> I64Vector4: ...
    @property
    def xyzz(self) -> I64Vector4: ...
    @property
    def xyzw(self) -> I64Vector4: ...
    @property
    def xyzo(self) -> I64Vector4: ...
    @property
    def xyzl(self) -> I64Vector4: ...
    @property
    def xywx(self) -> I64Vector4: ...
    @property
    def xywy(self) -> I64Vector4: ...
    @property
    def xywz(self) -> I64Vector4: ...
    @property
    def xyww(self) -> I64Vector4: ...
    @property
    def xywo(self) -> I64Vector4: ...
    @property
    def xywl(self) -> I64Vector4: ...
    @property
    def xyox(self) -> I64Vector4: ...
    @property
    def xyoy(self) -> I64Vector4: ...
    @property
    def xyoz(self) -> I64Vector4: ...
    @property
    def xyow(self) -> I64Vector4: ...
    @property
    def xyoo(self) -> I64Vector4: ...
    @property
    def xyol(self) -> I64Vector4: ...
    @property
    def xylx(self) -> I64Vector4: ...
    @property
    def xyly(self) -> I64Vector4: ...
    @property
    def xylz(self) -> I64Vector4: ...
    @property
    def xylw(self) -> I64Vector4: ...
    @property
    def xylo(self) -> I64Vector4: ...
    @property
    def xyll(self) -> I64Vector4: ...
    @property
    def xzxx(self) -> I64Vector4: ...
    @property
    def xzxy(self) -> I64Vector4: ...
    @property
    def xzxz(self) -> I64Vector4: ...
    @property
    def xzxw(self) -> I64Vector4: ...
    @property
    def xzxo(self) -> I64Vector4: ...
    @property
    def xzxl(self) -> I64Vector4: ...
    @property
    def xzyx(self) -> I64Vector4: ...
    @property
    def xzyy(self) -> I64Vector4: ...
    @property
    def xzyz(self) -> I64Vector4: ...
    @property
    def xzyw(self) -> I64Vector4: ...
    @property
    def xzyo(self) -> I64Vector4: ...
    @property
    def xzyl(self) -> I64Vector4: ...
    @property
    def xzzx(self) -> I64Vector4: ...
    @property
    def xzzy(self) -> I64Vector4: ...
    @property
    def xzzz(self) -> I64Vector4: ...
    @property
    def xzzw(self) -> I64Vector4: ...
    @property
    def xzzo(self) -> I64Vector4: ...
    @property
    def xzzl(self) -> I64Vector4: ...
    @property
    def xzwx(self) -> I64Vector4: ...
    @property
    def xzwy(self) -> I64Vector4: ...
    @property
    def xzwz(self) -> I64Vector4: ...
    @property
    def xzww(self) -> I64Vector4: ...
    @property
    def xzwo(self) -> I64Vector4: ...
    @property
    def xzwl(self) -> I64Vector4: ...
    @property
    def xzox(self) -> I64Vector4: ...
    @property
    def xzoy(self) -> I64Vector4: ...
    @property
    def xzoz(self) -> I64Vector4: ...
    @property
    def xzow(self) -> I64Vector4: ...
    @property
    def xzoo(self) -> I64Vector4: ...
    @property
    def xzol(self) -> I64Vector4: ...
    @property
    def xzlx(self) -> I64Vector4: ...
    @property
    def xzly(self) -> I64Vector4: ...
    @property
    def xzlz(self) -> I64Vector4: ...
    @property
    def xzlw(self) -> I64Vector4: ...
    @property
    def xzlo(self) -> I64Vector4: ...
    @property
    def xzll(self) -> I64Vector4: ...
    @property
    def xwxx(self) -> I64Vector4: ...
    @property
    def xwxy(self) -> I64Vector4: ...
    @property
    def xwxz(self) -> I64Vector4: ...
    @property
    def xwxw(self) -> I64Vector4: ...
    @property
    def xwxo(self) -> I64Vector4: ...
    @property
    def xwxl(self) -> I64Vector4: ...
    @property
    def xwyx(self) -> I64Vector4: ...
    @property
    def xwyy(self) -> I64Vector4: ...
    @property
    def xwyz(self) -> I64Vector4: ...
    @property
    def xwyw(self) -> I64Vector4: ...
    @property
    def xwyo(self) -> I64Vector4: ...
    @property
    def xwyl(self) -> I64Vector4: ...
    @property
    def xwzx(self) -> I64Vector4: ...
    @property
    def xwzy(self) -> I64Vector4: ...
    @property
    def xwzz(self) -> I64Vector4: ...
    @property
    def xwzw(self) -> I64Vector4: ...
    @property
    def xwzo(self) -> I64Vector4: ...
    @property
    def xwzl(self) -> I64Vector4: ...
    @property
    def xwwx(self) -> I64Vector4: ...
    @property
    def xwwy(self) -> I64Vector4: ...
    @property
    def xwwz(self) -> I64Vector4: ...
    @property
    def xwww(self) -> I64Vector4: ...
    @property
    def xwwo(self) -> I64Vector4: ...
    @property
    def xwwl(self) -> I64Vector4: ...
    @property
    def xwox(self) -> I64Vector4: ...
    @property
    def xwoy(self) -> I64Vector4: ...
    @property
    def xwoz(self) -> I64Vector4: ...
    @property
    def xwow(self) -> I64Vector4: ...
    @property
    def xwoo(self) -> I64Vector4: ...
    @property
    def xwol(self) -> I64Vector4: ...
    @property
    def xwlx(self) -> I64Vector4: ...
    @property
    def xwly(self) -> I64Vector4: ...
    @property
    def xwlz(self) -> I64Vector4: ...
    @property
    def xwlw(self) -> I64Vector4: ...
    @property
    def xwlo(self) -> I64Vector4: ...
    @property
    def xwll(self) -> I64Vector4: ...
    @property
    def xoxx(self) -> I64Vector4: ...
    @property
    def xoxy(self) -> I64Vector4: ...
    @property
    def xoxz(self) -> I64Vector4: ...
    @property
    def xoxw(self) -> I64Vector4: ...
    @property
    def xoxo(self) -> I64Vector4: ...
    @property
    def xoxl(self) -> I64Vector4: ...
    @property
    def xoyx(self) -> I64Vector4: ...
    @property
    def xoyy(self) -> I64Vector4: ...
    @property
    def xoyz(self) -> I64Vector4: ...
    @property
    def xoyw(self) -> I64Vector4: ...
    @property
    def xoyo(self) -> I64Vector4: ...
    @property
    def xoyl(self) -> I64Vector4: ...
    @property
    def xozx(self) -> I64Vector4: ...
    @property
    def xozy(self) -> I64Vector4: ...
    @property
    def xozz(self) -> I64Vector4: ...
    @property
    def xozw(self) -> I64Vector4: ...
    @property
    def xozo(self) -> I64Vector4: ...
    @property
    def xozl(self) -> I64Vector4: ...
    @property
    def xowx(self) -> I64Vector4: ...
    @property
    def xowy(self) -> I64Vector4: ...
    @property
    def xowz(self) -> I64Vector4: ...
    @property
    def xoww(self) -> I64Vector4: ...
    @property
    def xowo(self) -> I64Vector4: ...
    @property
    def xowl(self) -> I64Vector4: ...
    @property
    def xoox(self) -> I64Vector4: ...
    @property
    def xooy(self) -> I64Vector4: ...
    @property
    def xooz(self) -> I64Vector4: ...
    @property
    def xoow(self) -> I64Vector4: ...
    @property
    def xooo(self) -> I64Vector4: ...
    @property
    def xool(self) -> I64Vector4: ...
    @property
    def xolx(self) -> I64Vector4: ...
    @property
    def xoly(self) -> I64Vector4: ...
    @property
    def xolz(self) -> I64Vector4: ...
    @property
    def xolw(self) -> I64Vector4: ...
    @property
    def xolo(self) -> I64Vector4: ...
    @property
    def xoll(self) -> I64Vector4: ...
    @property
    def xlxx(self) -> I64Vector4: ...
    @property
    def xlxy(self) -> I64Vector4: ...
    @property
    def xlxz(self) -> I64Vector4: ...
    @property
    def xlxw(self) -> I64Vector4: ...
    @property
    def xlxo(self) -> I64Vector4: ...
    @property
    def xlxl(self) -> I64Vector4: ...
    @property
    def xlyx(self) -> I64Vector4: ...
    @property
    def xlyy(self) -> I64Vector4: ...
    @property
    def xlyz(self) -> I64Vector4: ...
    @property
    def xlyw(self) -> I64Vector4: ...
    @property
    def xlyo(self) -> I64Vector4: ...
    @property
    def xlyl(self) -> I64Vector4: ...
    @property
    def xlzx(self) -> I64Vector4: ...
    @property
    def xlzy(self) -> I64Vector4: ...
    @property
    def xlzz(self) -> I64Vector4: ...
    @property
    def xlzw(self) -> I64Vector4: ...
    @property
    def xlzo(self) -> I64Vector4: ...
    @property
    def xlzl(self) -> I64Vector4: ...
    @property
    def xlwx(self) -> I64Vector4: ...
    @property
    def xlwy(self) -> I64Vector4: ...
    @property
    def xlwz(self) -> I64Vector4: ...
    @property
    def xlww(self) -> I64Vector4: ...
    @property
    def xlwo(self) -> I64Vector4: ...
    @property
    def xlwl(self) -> I64Vector4: ...
    @property
    def xlox(self) -> I64Vector4: ...
    @property
    def xloy(self) -> I64Vector4: ...
    @property
    def xloz(self) -> I64Vector4: ...
    @property
    def xlow(self) -> I64Vector4: ...
    @property
    def xloo(self) -> I64Vector4: ...
    @property
    def xlol(self) -> I64Vector4: ...
    @property
    def xllx(self) -> I64Vector4: ...
    @property
    def xlly(self) -> I64Vector4: ...
    @property
    def xllz(self) -> I64Vector4: ...
    @property
    def xllw(self) -> I64Vector4: ...
    @property
    def xllo(self) -> I64Vector4: ...
    @property
    def xlll(self) -> I64Vector4: ...
    @property
    def yxxx(self) -> I64Vector4: ...
    @property
    def yxxy(self) -> I64Vector4: ...
    @property
    def yxxz(self) -> I64Vector4: ...
    @property
    def yxxw(self) -> I64Vector4: ...
    @property
    def yxxo(self) -> I64Vector4: ...
    @property
    def yxxl(self) -> I64Vector4: ...
    @property
    def yxyx(self) -> I64Vector4: ...
    @property
    def yxyy(self) -> I64Vector4: ...
    @property
    def yxyz(self) -> I64Vector4: ...
    @property
    def yxyw(self) -> I64Vector4: ...
    @property
    def yxyo(self) -> I64Vector4: ...
    @property
    def yxyl(self) -> I64Vector4: ...
    @property
    def yxzx(self) -> I64Vector4: ...
    @property
    def yxzy(self) -> I64Vector4: ...
    @property
    def yxzz(self) -> I64Vector4: ...
    @property
    def yxzw(self) -> I64Vector4: ...
    @property
    def yxzo(self) -> I64Vector4: ...
    @property
    def yxzl(self) -> I64Vector4: ...
    @property
    def yxwx(self) -> I64Vector4: ...
    @property
    def yxwy(self) -> I64Vector4: ...
    @property
    def yxwz(self) -> I64Vector4: ...
    @property
    def yxww(self) -> I64Vector4: ...
    @property
    def yxwo(self) -> I64Vector4: ...
    @property
    def yxwl(self) -> I64Vector4: ...
    @property
    def yxox(self) -> I64Vector4: ...
    @property
    def yxoy(self) -> I64Vector4: ...
    @property
    def yxoz(self) -> I64Vector4: ...
    @property
    def yxow(self) -> I64Vector4: ...
    @property
    def yxoo(self) -> I64Vector4: ...
    @property
    def yxol(self) -> I64Vector4: ...
    @property
    def yxlx(self) -> I64Vector4: ...
    @property
    def yxly(self) -> I64Vector4: ...
    @property
    def yxlz(self) -> I64Vector4: ...
    @property
    def yxlw(self) -> I64Vector4: ...
    @property
    def yxlo(self) -> I64Vector4: ...
    @property
    def yxll(self) -> I64Vector4: ...
    @property
    def yyxx(self) -> I64Vector4: ...
    @property
    def yyxy(self) -> I64Vector4: ...
    @property
    def yyxz(self) -> I64Vector4: ...
    @property
    def yyxw(self) -> I64Vector4: ...
    @property
    def yyxo(self) -> I64Vector4: ...
    @property
    def yyxl(self) -> I64Vector4: ...
    @property
    def yyyx(self) -> I64Vector4: ...
    @property
    def yyyy(self) -> I64Vector4: ...
    @property
    def yyyz(self) -> I64Vector4: ...
    @property
    def yyyw(self) -> I64Vector4: ...
    @property
    def yyyo(self) -> I64Vector4: ...
    @property
    def yyyl(self) -> I64Vector4: ...
    @property
    def yyzx(self) -> I64Vector4: ...
    @property
    def yyzy(self) -> I64Vector4: ...
    @property
    def yyzz(self) -> I64Vector4: ...
    @property
    def yyzw(self) -> I64Vector4: ...
    @property
    def yyzo(self) -> I64Vector4: ...
    @property
    def yyzl(self) -> I64Vector4: ...
    @property
    def yywx(self) -> I64Vector4: ...
    @property
    def yywy(self) -> I64Vector4: ...
    @property
    def yywz(self) -> I64Vector4: ...
    @property
    def yyww(self) -> I64Vector4: ...
    @property
    def yywo(self) -> I64Vector4: ...
    @property
    def yywl(self) -> I64Vector4: ...
    @property
    def yyox(self) -> I64Vector4: ...
    @property
    def yyoy(self) -> I64Vector4: ...
    @property
    def yyoz(self) -> I64Vector4: ...
    @property
    def yyow(self) -> I64Vector4: ...
    @property
    def yyoo(self) -> I64Vector4: ...
    @property
    def yyol(self) -> I64Vector4: ...
    @property
    def yylx(self) -> I64Vector4: ...
    @property
    def yyly(self) -> I64Vector4: ...
    @property
    def yylz(self) -> I64Vector4: ...
    @property
    def yylw(self) -> I64Vector4: ...
    @property
    def yylo(self) -> I64Vector4: ...
    @property
    def yyll(self) -> I64Vector4: ...
    @property
    def yzxx(self) -> I64Vector4: ...
    @property
    def yzxy(self) -> I64Vector4: ...
    @property
    def yzxz(self) -> I64Vector4: ...
    @property
    def yzxw(self) -> I64Vector4: ...
    @property
    def yzxo(self) -> I64Vector4: ...
    @property
    def yzxl(self) -> I64Vector4: ...
    @property
    def yzyx(self) -> I64Vector4: ...
    @property
    def yzyy(self) -> I64Vector4: ...
    @property
    def yzyz(self) -> I64Vector4: ...
    @property
    def yzyw(self) -> I64Vector4: ...
    @property
    def yzyo(self) -> I64Vector4: ...
    @property
    def yzyl(self) -> I64Vector4: ...
    @property
    def yzzx(self) -> I64Vector4: ...
    @property
    def yzzy(self) -> I64Vector4: ...
    @property
    def yzzz(self) -> I64Vector4: ...
    @property
    def yzzw(self) -> I64Vector4: ...
    @property
    def yzzo(self) -> I64Vector4: ...
    @property
    def yzzl(self) -> I64Vector4: ...
    @property
    def yzwx(self) -> I64Vector4: ...
    @property
    def yzwy(self) -> I64Vector4: ...
    @property
    def yzwz(self) -> I64Vector4: ...
    @property
    def yzww(self) -> I64Vector4: ...
    @property
    def yzwo(self) -> I64Vector4: ...
    @property
    def yzwl(self) -> I64Vector4: ...
    @property
    def yzox(self) -> I64Vector4: ...
    @property
    def yzoy(self) -> I64Vector4: ...
    @property
    def yzoz(self) -> I64Vector4: ...
    @property
    def yzow(self) -> I64Vector4: ...
    @property
    def yzoo(self) -> I64Vector4: ...
    @property
    def yzol(self) -> I64Vector4: ...
    @property
    def yzlx(self) -> I64Vector4: ...
    @property
    def yzly(self) -> I64Vector4: ...
    @property
    def yzlz(self) -> I64Vector4: ...
    @property
    def yzlw(self) -> I64Vector4: ...
    @property
    def yzlo(self) -> I64Vector4: ...
    @property
    def yzll(self) -> I64Vector4: ...
    @property
    def ywxx(self) -> I64Vector4: ...
    @property
    def ywxy(self) -> I64Vector4: ...
    @property
    def ywxz(self) -> I64Vector4: ...
    @property
    def ywxw(self) -> I64Vector4: ...
    @property
    def ywxo(self) -> I64Vector4: ...
    @property
    def ywxl(self) -> I64Vector4: ...
    @property
    def ywyx(self) -> I64Vector4: ...
    @property
    def ywyy(self) -> I64Vector4: ...
    @property
    def ywyz(self) -> I64Vector4: ...
    @property
    def ywyw(self) -> I64Vector4: ...
    @property
    def ywyo(self) -> I64Vector4: ...
    @property
    def ywyl(self) -> I64Vector4: ...
    @property
    def ywzx(self) -> I64Vector4: ...
    @property
    def ywzy(self) -> I64Vector4: ...
    @property
    def ywzz(self) -> I64Vector4: ...
    @property
    def ywzw(self) -> I64Vector4: ...
    @property
    def ywzo(self) -> I64Vector4: ...
    @property
    def ywzl(self) -> I64Vector4: ...
    @property
    def ywwx(self) -> I64Vector4: ...
    @property
    def ywwy(self) -> I64Vector4: ...
    @property
    def ywwz(self) -> I64Vector4: ...
    @property
    def ywww(self) -> I64Vector4: ...
    @property
    def ywwo(self) -> I64Vector4: ...
    @property
    def ywwl(self) -> I64Vector4: ...
    @property
    def ywox(self) -> I64Vector4: ...
    @property
    def ywoy(self) -> I64Vector4: ...
    @property
    def ywoz(self) -> I64Vector4: ...
    @property
    def ywow(self) -> I64Vector4: ...
    @property
    def ywoo(self) -> I64Vector4: ...
    @property
    def ywol(self) -> I64Vector4: ...
    @property
    def ywlx(self) -> I64Vector4: ...
    @property
    def ywly(self) -> I64Vector4: ...
    @property
    def ywlz(self) -> I64Vector4: ...
    @property
    def ywlw(self) -> I64Vector4: ...
    @property
    def ywlo(self) -> I64Vector4: ...
    @property
    def ywll(self) -> I64Vector4: ...
    @property
    def yoxx(self) -> I64Vector4: ...
    @property
    def yoxy(self) -> I64Vector4: ...
    @property
    def yoxz(self) -> I64Vector4: ...
    @property
    def yoxw(self) -> I64Vector4: ...
    @property
    def yoxo(self) -> I64Vector4: ...
    @property
    def yoxl(self) -> I64Vector4: ...
    @property
    def yoyx(self) -> I64Vector4: ...
    @property
    def yoyy(self) -> I64Vector4: ...
    @property
    def yoyz(self) -> I64Vector4: ...
    @property
    def yoyw(self) -> I64Vector4: ...
    @property
    def yoyo(self) -> I64Vector4: ...
    @property
    def yoyl(self) -> I64Vector4: ...
    @property
    def yozx(self) -> I64Vector4: ...
    @property
    def yozy(self) -> I64Vector4: ...
    @property
    def yozz(self) -> I64Vector4: ...
    @property
    def yozw(self) -> I64Vector4: ...
    @property
    def yozo(self) -> I64Vector4: ...
    @property
    def yozl(self) -> I64Vector4: ...
    @property
    def yowx(self) -> I64Vector4: ...
    @property
    def yowy(self) -> I64Vector4: ...
    @property
    def yowz(self) -> I64Vector4: ...
    @property
    def yoww(self) -> I64Vector4: ...
    @property
    def yowo(self) -> I64Vector4: ...
    @property
    def yowl(self) -> I64Vector4: ...
    @property
    def yoox(self) -> I64Vector4: ...
    @property
    def yooy(self) -> I64Vector4: ...
    @property
    def yooz(self) -> I64Vector4: ...
    @property
    def yoow(self) -> I64Vector4: ...
    @property
    def yooo(self) -> I64Vector4: ...
    @property
    def yool(self) -> I64Vector4: ...
    @property
    def yolx(self) -> I64Vector4: ...
    @property
    def yoly(self) -> I64Vector4: ...
    @property
    def yolz(self) -> I64Vector4: ...
    @property
    def yolw(self) -> I64Vector4: ...
    @property
    def yolo(self) -> I64Vector4: ...
    @property
    def yoll(self) -> I64Vector4: ...
    @property
    def ylxx(self) -> I64Vector4: ...
    @property
    def ylxy(self) -> I64Vector4: ...
    @property
    def ylxz(self) -> I64Vector4: ...
    @property
    def ylxw(self) -> I64Vector4: ...
    @property
    def ylxo(self) -> I64Vector4: ...
    @property
    def ylxl(self) -> I64Vector4: ...
    @property
    def ylyx(self) -> I64Vector4: ...
    @property
    def ylyy(self) -> I64Vector4: ...
    @property
    def ylyz(self) -> I64Vector4: ...
    @property
    def ylyw(self) -> I64Vector4: ...
    @property
    def ylyo(self) -> I64Vector4: ...
    @property
    def ylyl(self) -> I64Vector4: ...
    @property
    def ylzx(self) -> I64Vector4: ...
    @property
    def ylzy(self) -> I64Vector4: ...
    @property
    def ylzz(self) -> I64Vector4: ...
    @property
    def ylzw(self) -> I64Vector4: ...
    @property
    def ylzo(self) -> I64Vector4: ...
    @property
    def ylzl(self) -> I64Vector4: ...
    @property
    def ylwx(self) -> I64Vector4: ...
    @property
    def ylwy(self) -> I64Vector4: ...
    @property
    def ylwz(self) -> I64Vector4: ...
    @property
    def ylww(self) -> I64Vector4: ...
    @property
    def ylwo(self) -> I64Vector4: ...
    @property
    def ylwl(self) -> I64Vector4: ...
    @property
    def ylox(self) -> I64Vector4: ...
    @property
    def yloy(self) -> I64Vector4: ...
    @property
    def yloz(self) -> I64Vector4: ...
    @property
    def ylow(self) -> I64Vector4: ...
    @property
    def yloo(self) -> I64Vector4: ...
    @property
    def ylol(self) -> I64Vector4: ...
    @property
    def yllx(self) -> I64Vector4: ...
    @property
    def ylly(self) -> I64Vector4: ...
    @property
    def yllz(self) -> I64Vector4: ...
    @property
    def yllw(self) -> I64Vector4: ...
    @property
    def yllo(self) -> I64Vector4: ...
    @property
    def ylll(self) -> I64Vector4: ...
    @property
    def zxxx(self) -> I64Vector4: ...
    @property
    def zxxy(self) -> I64Vector4: ...
    @property
    def zxxz(self) -> I64Vector4: ...
    @property
    def zxxw(self) -> I64Vector4: ...
    @property
    def zxxo(self) -> I64Vector4: ...
    @property
    def zxxl(self) -> I64Vector4: ...
    @property
    def zxyx(self) -> I64Vector4: ...
    @property
    def zxyy(self) -> I64Vector4: ...
    @property
    def zxyz(self) -> I64Vector4: ...
    @property
    def zxyw(self) -> I64Vector4: ...
    @property
    def zxyo(self) -> I64Vector4: ...
    @property
    def zxyl(self) -> I64Vector4: ...
    @property
    def zxzx(self) -> I64Vector4: ...
    @property
    def zxzy(self) -> I64Vector4: ...
    @property
    def zxzz(self) -> I64Vector4: ...
    @property
    def zxzw(self) -> I64Vector4: ...
    @property
    def zxzo(self) -> I64Vector4: ...
    @property
    def zxzl(self) -> I64Vector4: ...
    @property
    def zxwx(self) -> I64Vector4: ...
    @property
    def zxwy(self) -> I64Vector4: ...
    @property
    def zxwz(self) -> I64Vector4: ...
    @property
    def zxww(self) -> I64Vector4: ...
    @property
    def zxwo(self) -> I64Vector4: ...
    @property
    def zxwl(self) -> I64Vector4: ...
    @property
    def zxox(self) -> I64Vector4: ...
    @property
    def zxoy(self) -> I64Vector4: ...
    @property
    def zxoz(self) -> I64Vector4: ...
    @property
    def zxow(self) -> I64Vector4: ...
    @property
    def zxoo(self) -> I64Vector4: ...
    @property
    def zxol(self) -> I64Vector4: ...
    @property
    def zxlx(self) -> I64Vector4: ...
    @property
    def zxly(self) -> I64Vector4: ...
    @property
    def zxlz(self) -> I64Vector4: ...
    @property
    def zxlw(self) -> I64Vector4: ...
    @property
    def zxlo(self) -> I64Vector4: ...
    @property
    def zxll(self) -> I64Vector4: ...
    @property
    def zyxx(self) -> I64Vector4: ...
    @property
    def zyxy(self) -> I64Vector4: ...
    @property
    def zyxz(self) -> I64Vector4: ...
    @property
    def zyxw(self) -> I64Vector4: ...
    @property
    def zyxo(self) -> I64Vector4: ...
    @property
    def zyxl(self) -> I64Vector4: ...
    @property
    def zyyx(self) -> I64Vector4: ...
    @property
    def zyyy(self) -> I64Vector4: ...
    @property
    def zyyz(self) -> I64Vector4: ...
    @property
    def zyyw(self) -> I64Vector4: ...
    @property
    def zyyo(self) -> I64Vector4: ...
    @property
    def zyyl(self) -> I64Vector4: ...
    @property
    def zyzx(self) -> I64Vector4: ...
    @property
    def zyzy(self) -> I64Vector4: ...
    @property
    def zyzz(self) -> I64Vector4: ...
    @property
    def zyzw(self) -> I64Vector4: ...
    @property
    def zyzo(self) -> I64Vector4: ...
    @property
    def zyzl(self) -> I64Vector4: ...
    @property
    def zywx(self) -> I64Vector4: ...
    @property
    def zywy(self) -> I64Vector4: ...
    @property
    def zywz(self) -> I64Vector4: ...
    @property
    def zyww(self) -> I64Vector4: ...
    @property
    def zywo(self) -> I64Vector4: ...
    @property
    def zywl(self) -> I64Vector4: ...
    @property
    def zyox(self) -> I64Vector4: ...
    @property
    def zyoy(self) -> I64Vector4: ...
    @property
    def zyoz(self) -> I64Vector4: ...
    @property
    def zyow(self) -> I64Vector4: ...
    @property
    def zyoo(self) -> I64Vector4: ...
    @property
    def zyol(self) -> I64Vector4: ...
    @property
    def zylx(self) -> I64Vector4: ...
    @property
    def zyly(self) -> I64Vector4: ...
    @property
    def zylz(self) -> I64Vector4: ...
    @property
    def zylw(self) -> I64Vector4: ...
    @property
    def zylo(self) -> I64Vector4: ...
    @property
    def zyll(self) -> I64Vector4: ...
    @property
    def zzxx(self) -> I64Vector4: ...
    @property
    def zzxy(self) -> I64Vector4: ...
    @property
    def zzxz(self) -> I64Vector4: ...
    @property
    def zzxw(self) -> I64Vector4: ...
    @property
    def zzxo(self) -> I64Vector4: ...
    @property
    def zzxl(self) -> I64Vector4: ...
    @property
    def zzyx(self) -> I64Vector4: ...
    @property
    def zzyy(self) -> I64Vector4: ...
    @property
    def zzyz(self) -> I64Vector4: ...
    @property
    def zzyw(self) -> I64Vector4: ...
    @property
    def zzyo(self) -> I64Vector4: ...
    @property
    def zzyl(self) -> I64Vector4: ...
    @property
    def zzzx(self) -> I64Vector4: ...
    @property
    def zzzy(self) -> I64Vector4: ...
    @property
    def zzzz(self) -> I64Vector4: ...
    @property
    def zzzw(self) -> I64Vector4: ...
    @property
    def zzzo(self) -> I64Vector4: ...
    @property
    def zzzl(self) -> I64Vector4: ...
    @property
    def zzwx(self) -> I64Vector4: ...
    @property
    def zzwy(self) -> I64Vector4: ...
    @property
    def zzwz(self) -> I64Vector4: ...
    @property
    def zzww(self) -> I64Vector4: ...
    @property
    def zzwo(self) -> I64Vector4: ...
    @property
    def zzwl(self) -> I64Vector4: ...
    @property
    def zzox(self) -> I64Vector4: ...
    @property
    def zzoy(self) -> I64Vector4: ...
    @property
    def zzoz(self) -> I64Vector4: ...
    @property
    def zzow(self) -> I64Vector4: ...
    @property
    def zzoo(self) -> I64Vector4: ...
    @property
    def zzol(self) -> I64Vector4: ...
    @property
    def zzlx(self) -> I64Vector4: ...
    @property
    def zzly(self) -> I64Vector4: ...
    @property
    def zzlz(self) -> I64Vector4: ...
    @property
    def zzlw(self) -> I64Vector4: ...
    @property
    def zzlo(self) -> I64Vector4: ...
    @property
    def zzll(self) -> I64Vector4: ...
    @property
    def zwxx(self) -> I64Vector4: ...
    @property
    def zwxy(self) -> I64Vector4: ...
    @property
    def zwxz(self) -> I64Vector4: ...
    @property
    def zwxw(self) -> I64Vector4: ...
    @property
    def zwxo(self) -> I64Vector4: ...
    @property
    def zwxl(self) -> I64Vector4: ...
    @property
    def zwyx(self) -> I64Vector4: ...
    @property
    def zwyy(self) -> I64Vector4: ...
    @property
    def zwyz(self) -> I64Vector4: ...
    @property
    def zwyw(self) -> I64Vector4: ...
    @property
    def zwyo(self) -> I64Vector4: ...
    @property
    def zwyl(self) -> I64Vector4: ...
    @property
    def zwzx(self) -> I64Vector4: ...
    @property
    def zwzy(self) -> I64Vector4: ...
    @property
    def zwzz(self) -> I64Vector4: ...
    @property
    def zwzw(self) -> I64Vector4: ...
    @property
    def zwzo(self) -> I64Vector4: ...
    @property
    def zwzl(self) -> I64Vector4: ...
    @property
    def zwwx(self) -> I64Vector4: ...
    @property
    def zwwy(self) -> I64Vector4: ...
    @property
    def zwwz(self) -> I64Vector4: ...
    @property
    def zwww(self) -> I64Vector4: ...
    @property
    def zwwo(self) -> I64Vector4: ...
    @property
    def zwwl(self) -> I64Vector4: ...
    @property
    def zwox(self) -> I64Vector4: ...
    @property
    def zwoy(self) -> I64Vector4: ...
    @property
    def zwoz(self) -> I64Vector4: ...
    @property
    def zwow(self) -> I64Vector4: ...
    @property
    def zwoo(self) -> I64Vector4: ...
    @property
    def zwol(self) -> I64Vector4: ...
    @property
    def zwlx(self) -> I64Vector4: ...
    @property
    def zwly(self) -> I64Vector4: ...
    @property
    def zwlz(self) -> I64Vector4: ...
    @property
    def zwlw(self) -> I64Vector4: ...
    @property
    def zwlo(self) -> I64Vector4: ...
    @property
    def zwll(self) -> I64Vector4: ...
    @property
    def zoxx(self) -> I64Vector4: ...
    @property
    def zoxy(self) -> I64Vector4: ...
    @property
    def zoxz(self) -> I64Vector4: ...
    @property
    def zoxw(self) -> I64Vector4: ...
    @property
    def zoxo(self) -> I64Vector4: ...
    @property
    def zoxl(self) -> I64Vector4: ...
    @property
    def zoyx(self) -> I64Vector4: ...
    @property
    def zoyy(self) -> I64Vector4: ...
    @property
    def zoyz(self) -> I64Vector4: ...
    @property
    def zoyw(self) -> I64Vector4: ...
    @property
    def zoyo(self) -> I64Vector4: ...
    @property
    def zoyl(self) -> I64Vector4: ...
    @property
    def zozx(self) -> I64Vector4: ...
    @property
    def zozy(self) -> I64Vector4: ...
    @property
    def zozz(self) -> I64Vector4: ...
    @property
    def zozw(self) -> I64Vector4: ...
    @property
    def zozo(self) -> I64Vector4: ...
    @property
    def zozl(self) -> I64Vector4: ...
    @property
    def zowx(self) -> I64Vector4: ...
    @property
    def zowy(self) -> I64Vector4: ...
    @property
    def zowz(self) -> I64Vector4: ...
    @property
    def zoww(self) -> I64Vector4: ...
    @property
    def zowo(self) -> I64Vector4: ...
    @property
    def zowl(self) -> I64Vector4: ...
    @property
    def zoox(self) -> I64Vector4: ...
    @property
    def zooy(self) -> I64Vector4: ...
    @property
    def zooz(self) -> I64Vector4: ...
    @property
    def zoow(self) -> I64Vector4: ...
    @property
    def zooo(self) -> I64Vector4: ...
    @property
    def zool(self) -> I64Vector4: ...
    @property
    def zolx(self) -> I64Vector4: ...
    @property
    def zoly(self) -> I64Vector4: ...
    @property
    def zolz(self) -> I64Vector4: ...
    @property
    def zolw(self) -> I64Vector4: ...
    @property
    def zolo(self) -> I64Vector4: ...
    @property
    def zoll(self) -> I64Vector4: ...
    @property
    def zlxx(self) -> I64Vector4: ...
    @property
    def zlxy(self) -> I64Vector4: ...
    @property
    def zlxz(self) -> I64Vector4: ...
    @property
    def zlxw(self) -> I64Vector4: ...
    @property
    def zlxo(self) -> I64Vector4: ...
    @property
    def zlxl(self) -> I64Vector4: ...
    @property
    def zlyx(self) -> I64Vector4: ...
    @property
    def zlyy(self) -> I64Vector4: ...
    @property
    def zlyz(self) -> I64Vector4: ...
    @property
    def zlyw(self) -> I64Vector4: ...
    @property
    def zlyo(self) -> I64Vector4: ...
    @property
    def zlyl(self) -> I64Vector4: ...
    @property
    def zlzx(self) -> I64Vector4: ...
    @property
    def zlzy(self) -> I64Vector4: ...
    @property
    def zlzz(self) -> I64Vector4: ...
    @property
    def zlzw(self) -> I64Vector4: ...
    @property
    def zlzo(self) -> I64Vector4: ...
    @property
    def zlzl(self) -> I64Vector4: ...
    @property
    def zlwx(self) -> I64Vector4: ...
    @property
    def zlwy(self) -> I64Vector4: ...
    @property
    def zlwz(self) -> I64Vector4: ...
    @property
    def zlww(self) -> I64Vector4: ...
    @property
    def zlwo(self) -> I64Vector4: ...
    @property
    def zlwl(self) -> I64Vector4: ...
    @property
    def zlox(self) -> I64Vector4: ...
    @property
    def zloy(self) -> I64Vector4: ...
    @property
    def zloz(self) -> I64Vector4: ...
    @property
    def zlow(self) -> I64Vector4: ...
    @property
    def zloo(self) -> I64Vector4: ...
    @property
    def zlol(self) -> I64Vector4: ...
    @property
    def zllx(self) -> I64Vector4: ...
    @property
    def zlly(self) -> I64Vector4: ...
    @property
    def zllz(self) -> I64Vector4: ...
    @property
    def zllw(self) -> I64Vector4: ...
    @property
    def zllo(self) -> I64Vector4: ...
    @property
    def zlll(self) -> I64Vector4: ...
    @property
    def wxxx(self) -> I64Vector4: ...
    @property
    def wxxy(self) -> I64Vector4: ...
    @property
    def wxxz(self) -> I64Vector4: ...
    @property
    def wxxw(self) -> I64Vector4: ...
    @property
    def wxxo(self) -> I64Vector4: ...
    @property
    def wxxl(self) -> I64Vector4: ...
    @property
    def wxyx(self) -> I64Vector4: ...
    @property
    def wxyy(self) -> I64Vector4: ...
    @property
    def wxyz(self) -> I64Vector4: ...
    @property
    def wxyw(self) -> I64Vector4: ...
    @property
    def wxyo(self) -> I64Vector4: ...
    @property
    def wxyl(self) -> I64Vector4: ...
    @property
    def wxzx(self) -> I64Vector4: ...
    @property
    def wxzy(self) -> I64Vector4: ...
    @property
    def wxzz(self) -> I64Vector4: ...
    @property
    def wxzw(self) -> I64Vector4: ...
    @property
    def wxzo(self) -> I64Vector4: ...
    @property
    def wxzl(self) -> I64Vector4: ...
    @property
    def wxwx(self) -> I64Vector4: ...
    @property
    def wxwy(self) -> I64Vector4: ...
    @property
    def wxwz(self) -> I64Vector4: ...
    @property
    def wxww(self) -> I64Vector4: ...
    @property
    def wxwo(self) -> I64Vector4: ...
    @property
    def wxwl(self) -> I64Vector4: ...
    @property
    def wxox(self) -> I64Vector4: ...
    @property
    def wxoy(self) -> I64Vector4: ...
    @property
    def wxoz(self) -> I64Vector4: ...
    @property
    def wxow(self) -> I64Vector4: ...
    @property
    def wxoo(self) -> I64Vector4: ...
    @property
    def wxol(self) -> I64Vector4: ...
    @property
    def wxlx(self) -> I64Vector4: ...
    @property
    def wxly(self) -> I64Vector4: ...
    @property
    def wxlz(self) -> I64Vector4: ...
    @property
    def wxlw(self) -> I64Vector4: ...
    @property
    def wxlo(self) -> I64Vector4: ...
    @property
    def wxll(self) -> I64Vector4: ...
    @property
    def wyxx(self) -> I64Vector4: ...
    @property
    def wyxy(self) -> I64Vector4: ...
    @property
    def wyxz(self) -> I64Vector4: ...
    @property
    def wyxw(self) -> I64Vector4: ...
    @property
    def wyxo(self) -> I64Vector4: ...
    @property
    def wyxl(self) -> I64Vector4: ...
    @property
    def wyyx(self) -> I64Vector4: ...
    @property
    def wyyy(self) -> I64Vector4: ...
    @property
    def wyyz(self) -> I64Vector4: ...
    @property
    def wyyw(self) -> I64Vector4: ...
    @property
    def wyyo(self) -> I64Vector4: ...
    @property
    def wyyl(self) -> I64Vector4: ...
    @property
    def wyzx(self) -> I64Vector4: ...
    @property
    def wyzy(self) -> I64Vector4: ...
    @property
    def wyzz(self) -> I64Vector4: ...
    @property
    def wyzw(self) -> I64Vector4: ...
    @property
    def wyzo(self) -> I64Vector4: ...
    @property
    def wyzl(self) -> I64Vector4: ...
    @property
    def wywx(self) -> I64Vector4: ...
    @property
    def wywy(self) -> I64Vector4: ...
    @property
    def wywz(self) -> I64Vector4: ...
    @property
    def wyww(self) -> I64Vector4: ...
    @property
    def wywo(self) -> I64Vector4: ...
    @property
    def wywl(self) -> I64Vector4: ...
    @property
    def wyox(self) -> I64Vector4: ...
    @property
    def wyoy(self) -> I64Vector4: ...
    @property
    def wyoz(self) -> I64Vector4: ...
    @property
    def wyow(self) -> I64Vector4: ...
    @property
    def wyoo(self) -> I64Vector4: ...
    @property
    def wyol(self) -> I64Vector4: ...
    @property
    def wylx(self) -> I64Vector4: ...
    @property
    def wyly(self) -> I64Vector4: ...
    @property
    def wylz(self) -> I64Vector4: ...
    @property
    def wylw(self) -> I64Vector4: ...
    @property
    def wylo(self) -> I64Vector4: ...
    @property
    def wyll(self) -> I64Vector4: ...
    @property
    def wzxx(self) -> I64Vector4: ...
    @property
    def wzxy(self) -> I64Vector4: ...
    @property
    def wzxz(self) -> I64Vector4: ...
    @property
    def wzxw(self) -> I64Vector4: ...
    @property
    def wzxo(self) -> I64Vector4: ...
    @property
    def wzxl(self) -> I64Vector4: ...
    @property
    def wzyx(self) -> I64Vector4: ...
    @property
    def wzyy(self) -> I64Vector4: ...
    @property
    def wzyz(self) -> I64Vector4: ...
    @property
    def wzyw(self) -> I64Vector4: ...
    @property
    def wzyo(self) -> I64Vector4: ...
    @property
    def wzyl(self) -> I64Vector4: ...
    @property
    def wzzx(self) -> I64Vector4: ...
    @property
    def wzzy(self) -> I64Vector4: ...
    @property
    def wzzz(self) -> I64Vector4: ...
    @property
    def wzzw(self) -> I64Vector4: ...
    @property
    def wzzo(self) -> I64Vector4: ...
    @property
    def wzzl(self) -> I64Vector4: ...
    @property
    def wzwx(self) -> I64Vector4: ...
    @property
    def wzwy(self) -> I64Vector4: ...
    @property
    def wzwz(self) -> I64Vector4: ...
    @property
    def wzww(self) -> I64Vector4: ...
    @property
    def wzwo(self) -> I64Vector4: ...
    @property
    def wzwl(self) -> I64Vector4: ...
    @property
    def wzox(self) -> I64Vector4: ...
    @property
    def wzoy(self) -> I64Vector4: ...
    @property
    def wzoz(self) -> I64Vector4: ...
    @property
    def wzow(self) -> I64Vector4: ...
    @property
    def wzoo(self) -> I64Vector4: ...
    @property
    def wzol(self) -> I64Vector4: ...
    @property
    def wzlx(self) -> I64Vector4: ...
    @property
    def wzly(self) -> I64Vector4: ...
    @property
    def wzlz(self) -> I64Vector4: ...
    @property
    def wzlw(self) -> I64Vector4: ...
    @property
    def wzlo(self) -> I64Vector4: ...
    @property
    def wzll(self) -> I64Vector4: ...
    @property
    def wwxx(self) -> I64Vector4: ...
    @property
    def wwxy(self) -> I64Vector4: ...
    @property
    def wwxz(self) -> I64Vector4: ...
    @property
    def wwxw(self) -> I64Vector4: ...
    @property
    def wwxo(self) -> I64Vector4: ...
    @property
    def wwxl(self) -> I64Vector4: ...
    @property
    def wwyx(self) -> I64Vector4: ...
    @property
    def wwyy(self) -> I64Vector4: ...
    @property
    def wwyz(self) -> I64Vector4: ...
    @property
    def wwyw(self) -> I64Vector4: ...
    @property
    def wwyo(self) -> I64Vector4: ...
    @property
    def wwyl(self) -> I64Vector4: ...
    @property
    def wwzx(self) -> I64Vector4: ...
    @property
    def wwzy(self) -> I64Vector4: ...
    @property
    def wwzz(self) -> I64Vector4: ...
    @property
    def wwzw(self) -> I64Vector4: ...
    @property
    def wwzo(self) -> I64Vector4: ...
    @property
    def wwzl(self) -> I64Vector4: ...
    @property
    def wwwx(self) -> I64Vector4: ...
    @property
    def wwwy(self) -> I64Vector4: ...
    @property
    def wwwz(self) -> I64Vector4: ...
    @property
    def wwww(self) -> I64Vector4: ...
    @property
    def wwwo(self) -> I64Vector4: ...
    @property
    def wwwl(self) -> I64Vector4: ...
    @property
    def wwox(self) -> I64Vector4: ...
    @property
    def wwoy(self) -> I64Vector4: ...
    @property
    def wwoz(self) -> I64Vector4: ...
    @property
    def wwow(self) -> I64Vector4: ...
    @property
    def wwoo(self) -> I64Vector4: ...
    @property
    def wwol(self) -> I64Vector4: ...
    @property
    def wwlx(self) -> I64Vector4: ...
    @property
    def wwly(self) -> I64Vector4: ...
    @property
    def wwlz(self) -> I64Vector4: ...
    @property
    def wwlw(self) -> I64Vector4: ...
    @property
    def wwlo(self) -> I64Vector4: ...
    @property
    def wwll(self) -> I64Vector4: ...
    @property
    def woxx(self) -> I64Vector4: ...
    @property
    def woxy(self) -> I64Vector4: ...
    @property
    def woxz(self) -> I64Vector4: ...
    @property
    def woxw(self) -> I64Vector4: ...
    @property
    def woxo(self) -> I64Vector4: ...
    @property
    def woxl(self) -> I64Vector4: ...
    @property
    def woyx(self) -> I64Vector4: ...
    @property
    def woyy(self) -> I64Vector4: ...
    @property
    def woyz(self) -> I64Vector4: ...
    @property
    def woyw(self) -> I64Vector4: ...
    @property
    def woyo(self) -> I64Vector4: ...
    @property
    def woyl(self) -> I64Vector4: ...
    @property
    def wozx(self) -> I64Vector4: ...
    @property
    def wozy(self) -> I64Vector4: ...
    @property
    def wozz(self) -> I64Vector4: ...
    @property
    def wozw(self) -> I64Vector4: ...
    @property
    def wozo(self) -> I64Vector4: ...
    @property
    def wozl(self) -> I64Vector4: ...
    @property
    def wowx(self) -> I64Vector4: ...
    @property
    def wowy(self) -> I64Vector4: ...
    @property
    def wowz(self) -> I64Vector4: ...
    @property
    def woww(self) -> I64Vector4: ...
    @property
    def wowo(self) -> I64Vector4: ...
    @property
    def wowl(self) -> I64Vector4: ...
    @property
    def woox(self) -> I64Vector4: ...
    @property
    def wooy(self) -> I64Vector4: ...
    @property
    def wooz(self) -> I64Vector4: ...
    @property
    def woow(self) -> I64Vector4: ...
    @property
    def wooo(self) -> I64Vector4: ...
    @property
    def wool(self) -> I64Vector4: ...
    @property
    def wolx(self) -> I64Vector4: ...
    @property
    def woly(self) -> I64Vector4: ...
    @property
    def wolz(self) -> I64Vector4: ...
    @property
    def wolw(self) -> I64Vector4: ...
    @property
    def wolo(self) -> I64Vector4: ...
    @property
    def woll(self) -> I64Vector4: ...
    @property
    def wlxx(self) -> I64Vector4: ...
    @property
    def wlxy(self) -> I64Vector4: ...
    @property
    def wlxz(self) -> I64Vector4: ...
    @property
    def wlxw(self) -> I64Vector4: ...
    @property
    def wlxo(self) -> I64Vector4: ...
    @property
    def wlxl(self) -> I64Vector4: ...
    @property
    def wlyx(self) -> I64Vector4: ...
    @property
    def wlyy(self) -> I64Vector4: ...
    @property
    def wlyz(self) -> I64Vector4: ...
    @property
    def wlyw(self) -> I64Vector4: ...
    @property
    def wlyo(self) -> I64Vector4: ...
    @property
    def wlyl(self) -> I64Vector4: ...
    @property
    def wlzx(self) -> I64Vector4: ...
    @property
    def wlzy(self) -> I64Vector4: ...
    @property
    def wlzz(self) -> I64Vector4: ...
    @property
    def wlzw(self) -> I64Vector4: ...
    @property
    def wlzo(self) -> I64Vector4: ...
    @property
    def wlzl(self) -> I64Vector4: ...
    @property
    def wlwx(self) -> I64Vector4: ...
    @property
    def wlwy(self) -> I64Vector4: ...
    @property
    def wlwz(self) -> I64Vector4: ...
    @property
    def wlww(self) -> I64Vector4: ...
    @property
    def wlwo(self) -> I64Vector4: ...
    @property
    def wlwl(self) -> I64Vector4: ...
    @property
    def wlox(self) -> I64Vector4: ...
    @property
    def wloy(self) -> I64Vector4: ...
    @property
    def wloz(self) -> I64Vector4: ...
    @property
    def wlow(self) -> I64Vector4: ...
    @property
    def wloo(self) -> I64Vector4: ...
    @property
    def wlol(self) -> I64Vector4: ...
    @property
    def wllx(self) -> I64Vector4: ...
    @property
    def wlly(self) -> I64Vector4: ...
    @property
    def wllz(self) -> I64Vector4: ...
    @property
    def wllw(self) -> I64Vector4: ...
    @property
    def wllo(self) -> I64Vector4: ...
    @property
    def wlll(self) -> I64Vector4: ...
    @property
    def oxxx(self) -> I64Vector4: ...
    @property
    def oxxy(self) -> I64Vector4: ...
    @property
    def oxxz(self) -> I64Vector4: ...
    @property
    def oxxw(self) -> I64Vector4: ...
    @property
    def oxxo(self) -> I64Vector4: ...
    @property
    def oxxl(self) -> I64Vector4: ...
    @property
    def oxyx(self) -> I64Vector4: ...
    @property
    def oxyy(self) -> I64Vector4: ...
    @property
    def oxyz(self) -> I64Vector4: ...
    @property
    def oxyw(self) -> I64Vector4: ...
    @property
    def oxyo(self) -> I64Vector4: ...
    @property
    def oxyl(self) -> I64Vector4: ...
    @property
    def oxzx(self) -> I64Vector4: ...
    @property
    def oxzy(self) -> I64Vector4: ...
    @property
    def oxzz(self) -> I64Vector4: ...
    @property
    def oxzw(self) -> I64Vector4: ...
    @property
    def oxzo(self) -> I64Vector4: ...
    @property
    def oxzl(self) -> I64Vector4: ...
    @property
    def oxwx(self) -> I64Vector4: ...
    @property
    def oxwy(self) -> I64Vector4: ...
    @property
    def oxwz(self) -> I64Vector4: ...
    @property
    def oxww(self) -> I64Vector4: ...
    @property
    def oxwo(self) -> I64Vector4: ...
    @property
    def oxwl(self) -> I64Vector4: ...
    @property
    def oxox(self) -> I64Vector4: ...
    @property
    def oxoy(self) -> I64Vector4: ...
    @property
    def oxoz(self) -> I64Vector4: ...
    @property
    def oxow(self) -> I64Vector4: ...
    @property
    def oxoo(self) -> I64Vector4: ...
    @property
    def oxol(self) -> I64Vector4: ...
    @property
    def oxlx(self) -> I64Vector4: ...
    @property
    def oxly(self) -> I64Vector4: ...
    @property
    def oxlz(self) -> I64Vector4: ...
    @property
    def oxlw(self) -> I64Vector4: ...
    @property
    def oxlo(self) -> I64Vector4: ...
    @property
    def oxll(self) -> I64Vector4: ...
    @property
    def oyxx(self) -> I64Vector4: ...
    @property
    def oyxy(self) -> I64Vector4: ...
    @property
    def oyxz(self) -> I64Vector4: ...
    @property
    def oyxw(self) -> I64Vector4: ...
    @property
    def oyxo(self) -> I64Vector4: ...
    @property
    def oyxl(self) -> I64Vector4: ...
    @property
    def oyyx(self) -> I64Vector4: ...
    @property
    def oyyy(self) -> I64Vector4: ...
    @property
    def oyyz(self) -> I64Vector4: ...
    @property
    def oyyw(self) -> I64Vector4: ...
    @property
    def oyyo(self) -> I64Vector4: ...
    @property
    def oyyl(self) -> I64Vector4: ...
    @property
    def oyzx(self) -> I64Vector4: ...
    @property
    def oyzy(self) -> I64Vector4: ...
    @property
    def oyzz(self) -> I64Vector4: ...
    @property
    def oyzw(self) -> I64Vector4: ...
    @property
    def oyzo(self) -> I64Vector4: ...
    @property
    def oyzl(self) -> I64Vector4: ...
    @property
    def oywx(self) -> I64Vector4: ...
    @property
    def oywy(self) -> I64Vector4: ...
    @property
    def oywz(self) -> I64Vector4: ...
    @property
    def oyww(self) -> I64Vector4: ...
    @property
    def oywo(self) -> I64Vector4: ...
    @property
    def oywl(self) -> I64Vector4: ...
    @property
    def oyox(self) -> I64Vector4: ...
    @property
    def oyoy(self) -> I64Vector4: ...
    @property
    def oyoz(self) -> I64Vector4: ...
    @property
    def oyow(self) -> I64Vector4: ...
    @property
    def oyoo(self) -> I64Vector4: ...
    @property
    def oyol(self) -> I64Vector4: ...
    @property
    def oylx(self) -> I64Vector4: ...
    @property
    def oyly(self) -> I64Vector4: ...
    @property
    def oylz(self) -> I64Vector4: ...
    @property
    def oylw(self) -> I64Vector4: ...
    @property
    def oylo(self) -> I64Vector4: ...
    @property
    def oyll(self) -> I64Vector4: ...
    @property
    def ozxx(self) -> I64Vector4: ...
    @property
    def ozxy(self) -> I64Vector4: ...
    @property
    def ozxz(self) -> I64Vector4: ...
    @property
    def ozxw(self) -> I64Vector4: ...
    @property
    def ozxo(self) -> I64Vector4: ...
    @property
    def ozxl(self) -> I64Vector4: ...
    @property
    def ozyx(self) -> I64Vector4: ...
    @property
    def ozyy(self) -> I64Vector4: ...
    @property
    def ozyz(self) -> I64Vector4: ...
    @property
    def ozyw(self) -> I64Vector4: ...
    @property
    def ozyo(self) -> I64Vector4: ...
    @property
    def ozyl(self) -> I64Vector4: ...
    @property
    def ozzx(self) -> I64Vector4: ...
    @property
    def ozzy(self) -> I64Vector4: ...
    @property
    def ozzz(self) -> I64Vector4: ...
    @property
    def ozzw(self) -> I64Vector4: ...
    @property
    def ozzo(self) -> I64Vector4: ...
    @property
    def ozzl(self) -> I64Vector4: ...
    @property
    def ozwx(self) -> I64Vector4: ...
    @property
    def ozwy(self) -> I64Vector4: ...
    @property
    def ozwz(self) -> I64Vector4: ...
    @property
    def ozww(self) -> I64Vector4: ...
    @property
    def ozwo(self) -> I64Vector4: ...
    @property
    def ozwl(self) -> I64Vector4: ...
    @property
    def ozox(self) -> I64Vector4: ...
    @property
    def ozoy(self) -> I64Vector4: ...
    @property
    def ozoz(self) -> I64Vector4: ...
    @property
    def ozow(self) -> I64Vector4: ...
    @property
    def ozoo(self) -> I64Vector4: ...
    @property
    def ozol(self) -> I64Vector4: ...
    @property
    def ozlx(self) -> I64Vector4: ...
    @property
    def ozly(self) -> I64Vector4: ...
    @property
    def ozlz(self) -> I64Vector4: ...
    @property
    def ozlw(self) -> I64Vector4: ...
    @property
    def ozlo(self) -> I64Vector4: ...
    @property
    def ozll(self) -> I64Vector4: ...
    @property
    def owxx(self) -> I64Vector4: ...
    @property
    def owxy(self) -> I64Vector4: ...
    @property
    def owxz(self) -> I64Vector4: ...
    @property
    def owxw(self) -> I64Vector4: ...
    @property
    def owxo(self) -> I64Vector4: ...
    @property
    def owxl(self) -> I64Vector4: ...
    @property
    def owyx(self) -> I64Vector4: ...
    @property
    def owyy(self) -> I64Vector4: ...
    @property
    def owyz(self) -> I64Vector4: ...
    @property
    def owyw(self) -> I64Vector4: ...
    @property
    def owyo(self) -> I64Vector4: ...
    @property
    def owyl(self) -> I64Vector4: ...
    @property
    def owzx(self) -> I64Vector4: ...
    @property
    def owzy(self) -> I64Vector4: ...
    @property
    def owzz(self) -> I64Vector4: ...
    @property
    def owzw(self) -> I64Vector4: ...
    @property
    def owzo(self) -> I64Vector4: ...
    @property
    def owzl(self) -> I64Vector4: ...
    @property
    def owwx(self) -> I64Vector4: ...
    @property
    def owwy(self) -> I64Vector4: ...
    @property
    def owwz(self) -> I64Vector4: ...
    @property
    def owww(self) -> I64Vector4: ...
    @property
    def owwo(self) -> I64Vector4: ...
    @property
    def owwl(self) -> I64Vector4: ...
    @property
    def owox(self) -> I64Vector4: ...
    @property
    def owoy(self) -> I64Vector4: ...
    @property
    def owoz(self) -> I64Vector4: ...
    @property
    def owow(self) -> I64Vector4: ...
    @property
    def owoo(self) -> I64Vector4: ...
    @property
    def owol(self) -> I64Vector4: ...
    @property
    def owlx(self) -> I64Vector4: ...
    @property
    def owly(self) -> I64Vector4: ...
    @property
    def owlz(self) -> I64Vector4: ...
    @property
    def owlw(self) -> I64Vector4: ...
    @property
    def owlo(self) -> I64Vector4: ...
    @property
    def owll(self) -> I64Vector4: ...
    @property
    def ooxx(self) -> I64Vector4: ...
    @property
    def ooxy(self) -> I64Vector4: ...
    @property
    def ooxz(self) -> I64Vector4: ...
    @property
    def ooxw(self) -> I64Vector4: ...
    @property
    def ooxo(self) -> I64Vector4: ...
    @property
    def ooxl(self) -> I64Vector4: ...
    @property
    def ooyx(self) -> I64Vector4: ...
    @property
    def ooyy(self) -> I64Vector4: ...
    @property
    def ooyz(self) -> I64Vector4: ...
    @property
    def ooyw(self) -> I64Vector4: ...
    @property
    def ooyo(self) -> I64Vector4: ...
    @property
    def ooyl(self) -> I64Vector4: ...
    @property
    def oozx(self) -> I64Vector4: ...
    @property
    def oozy(self) -> I64Vector4: ...
    @property
    def oozz(self) -> I64Vector4: ...
    @property
    def oozw(self) -> I64Vector4: ...
    @property
    def oozo(self) -> I64Vector4: ...
    @property
    def oozl(self) -> I64Vector4: ...
    @property
    def oowx(self) -> I64Vector4: ...
    @property
    def oowy(self) -> I64Vector4: ...
    @property
    def oowz(self) -> I64Vector4: ...
    @property
    def ooww(self) -> I64Vector4: ...
    @property
    def oowo(self) -> I64Vector4: ...
    @property
    def oowl(self) -> I64Vector4: ...
    @property
    def ooox(self) -> I64Vector4: ...
    @property
    def oooy(self) -> I64Vector4: ...
    @property
    def oooz(self) -> I64Vector4: ...
    @property
    def ooow(self) -> I64Vector4: ...
    @property
    def oolx(self) -> I64Vector4: ...
    @property
    def ooly(self) -> I64Vector4: ...
    @property
    def oolz(self) -> I64Vector4: ...
    @property
    def oolw(self) -> I64Vector4: ...
    @property
    def olxx(self) -> I64Vector4: ...
    @property
    def olxy(self) -> I64Vector4: ...
    @property
    def olxz(self) -> I64Vector4: ...
    @property
    def olxw(self) -> I64Vector4: ...
    @property
    def olxo(self) -> I64Vector4: ...
    @property
    def olxl(self) -> I64Vector4: ...
    @property
    def olyx(self) -> I64Vector4: ...
    @property
    def olyy(self) -> I64Vector4: ...
    @property
    def olyz(self) -> I64Vector4: ...
    @property
    def olyw(self) -> I64Vector4: ...
    @property
    def olyo(self) -> I64Vector4: ...
    @property
    def olyl(self) -> I64Vector4: ...
    @property
    def olzx(self) -> I64Vector4: ...
    @property
    def olzy(self) -> I64Vector4: ...
    @property
    def olzz(self) -> I64Vector4: ...
    @property
    def olzw(self) -> I64Vector4: ...
    @property
    def olzo(self) -> I64Vector4: ...
    @property
    def olzl(self) -> I64Vector4: ...
    @property
    def olwx(self) -> I64Vector4: ...
    @property
    def olwy(self) -> I64Vector4: ...
    @property
    def olwz(self) -> I64Vector4: ...
    @property
    def olww(self) -> I64Vector4: ...
    @property
    def olwo(self) -> I64Vector4: ...
    @property
    def olwl(self) -> I64Vector4: ...
    @property
    def olox(self) -> I64Vector4: ...
    @property
    def oloy(self) -> I64Vector4: ...
    @property
    def oloz(self) -> I64Vector4: ...
    @property
    def olow(self) -> I64Vector4: ...
    @property
    def ollx(self) -> I64Vector4: ...
    @property
    def olly(self) -> I64Vector4: ...
    @property
    def ollz(self) -> I64Vector4: ...
    @property
    def ollw(self) -> I64Vector4: ...
    @property
    def lxxx(self) -> I64Vector4: ...
    @property
    def lxxy(self) -> I64Vector4: ...
    @property
    def lxxz(self) -> I64Vector4: ...
    @property
    def lxxw(self) -> I64Vector4: ...
    @property
    def lxxo(self) -> I64Vector4: ...
    @property
    def lxxl(self) -> I64Vector4: ...
    @property
    def lxyx(self) -> I64Vector4: ...
    @property
    def lxyy(self) -> I64Vector4: ...
    @property
    def lxyz(self) -> I64Vector4: ...
    @property
    def lxyw(self) -> I64Vector4: ...
    @property
    def lxyo(self) -> I64Vector4: ...
    @property
    def lxyl(self) -> I64Vector4: ...
    @property
    def lxzx(self) -> I64Vector4: ...
    @property
    def lxzy(self) -> I64Vector4: ...
    @property
    def lxzz(self) -> I64Vector4: ...
    @property
    def lxzw(self) -> I64Vector4: ...
    @property
    def lxzo(self) -> I64Vector4: ...
    @property
    def lxzl(self) -> I64Vector4: ...
    @property
    def lxwx(self) -> I64Vector4: ...
    @property
    def lxwy(self) -> I64Vector4: ...
    @property
    def lxwz(self) -> I64Vector4: ...
    @property
    def lxww(self) -> I64Vector4: ...
    @property
    def lxwo(self) -> I64Vector4: ...
    @property
    def lxwl(self) -> I64Vector4: ...
    @property
    def lxox(self) -> I64Vector4: ...
    @property
    def lxoy(self) -> I64Vector4: ...
    @property
    def lxoz(self) -> I64Vector4: ...
    @property
    def lxow(self) -> I64Vector4: ...
    @property
    def lxoo(self) -> I64Vector4: ...
    @property
    def lxol(self) -> I64Vector4: ...
    @property
    def lxlx(self) -> I64Vector4: ...
    @property
    def lxly(self) -> I64Vector4: ...
    @property
    def lxlz(self) -> I64Vector4: ...
    @property
    def lxlw(self) -> I64Vector4: ...
    @property
    def lxlo(self) -> I64Vector4: ...
    @property
    def lxll(self) -> I64Vector4: ...
    @property
    def lyxx(self) -> I64Vector4: ...
    @property
    def lyxy(self) -> I64Vector4: ...
    @property
    def lyxz(self) -> I64Vector4: ...
    @property
    def lyxw(self) -> I64Vector4: ...
    @property
    def lyxo(self) -> I64Vector4: ...
    @property
    def lyxl(self) -> I64Vector4: ...
    @property
    def lyyx(self) -> I64Vector4: ...
    @property
    def lyyy(self) -> I64Vector4: ...
    @property
    def lyyz(self) -> I64Vector4: ...
    @property
    def lyyw(self) -> I64Vector4: ...
    @property
    def lyyo(self) -> I64Vector4: ...
    @property
    def lyyl(self) -> I64Vector4: ...
    @property
    def lyzx(self) -> I64Vector4: ...
    @property
    def lyzy(self) -> I64Vector4: ...
    @property
    def lyzz(self) -> I64Vector4: ...
    @property
    def lyzw(self) -> I64Vector4: ...
    @property
    def lyzo(self) -> I64Vector4: ...
    @property
    def lyzl(self) -> I64Vector4: ...
    @property
    def lywx(self) -> I64Vector4: ...
    @property
    def lywy(self) -> I64Vector4: ...
    @property
    def lywz(self) -> I64Vector4: ...
    @property
    def lyww(self) -> I64Vector4: ...
    @property
    def lywo(self) -> I64Vector4: ...
    @property
    def lywl(self) -> I64Vector4: ...
    @property
    def lyox(self) -> I64Vector4: ...
    @property
    def lyoy(self) -> I64Vector4: ...
    @property
    def lyoz(self) -> I64Vector4: ...
    @property
    def lyow(self) -> I64Vector4: ...
    @property
    def lyoo(self) -> I64Vector4: ...
    @property
    def lyol(self) -> I64Vector4: ...
    @property
    def lylx(self) -> I64Vector4: ...
    @property
    def lyly(self) -> I64Vector4: ...
    @property
    def lylz(self) -> I64Vector4: ...
    @property
    def lylw(self) -> I64Vector4: ...
    @property
    def lylo(self) -> I64Vector4: ...
    @property
    def lyll(self) -> I64Vector4: ...
    @property
    def lzxx(self) -> I64Vector4: ...
    @property
    def lzxy(self) -> I64Vector4: ...
    @property
    def lzxz(self) -> I64Vector4: ...
    @property
    def lzxw(self) -> I64Vector4: ...
    @property
    def lzxo(self) -> I64Vector4: ...
    @property
    def lzxl(self) -> I64Vector4: ...
    @property
    def lzyx(self) -> I64Vector4: ...
    @property
    def lzyy(self) -> I64Vector4: ...
    @property
    def lzyz(self) -> I64Vector4: ...
    @property
    def lzyw(self) -> I64Vector4: ...
    @property
    def lzyo(self) -> I64Vector4: ...
    @property
    def lzyl(self) -> I64Vector4: ...
    @property
    def lzzx(self) -> I64Vector4: ...
    @property
    def lzzy(self) -> I64Vector4: ...
    @property
    def lzzz(self) -> I64Vector4: ...
    @property
    def lzzw(self) -> I64Vector4: ...
    @property
    def lzzo(self) -> I64Vector4: ...
    @property
    def lzzl(self) -> I64Vector4: ...
    @property
    def lzwx(self) -> I64Vector4: ...
    @property
    def lzwy(self) -> I64Vector4: ...
    @property
    def lzwz(self) -> I64Vector4: ...
    @property
    def lzww(self) -> I64Vector4: ...
    @property
    def lzwo(self) -> I64Vector4: ...
    @property
    def lzwl(self) -> I64Vector4: ...
    @property
    def lzox(self) -> I64Vector4: ...
    @property
    def lzoy(self) -> I64Vector4: ...
    @property
    def lzoz(self) -> I64Vector4: ...
    @property
    def lzow(self) -> I64Vector4: ...
    @property
    def lzoo(self) -> I64Vector4: ...
    @property
    def lzol(self) -> I64Vector4: ...
    @property
    def lzlx(self) -> I64Vector4: ...
    @property
    def lzly(self) -> I64Vector4: ...
    @property
    def lzlz(self) -> I64Vector4: ...
    @property
    def lzlw(self) -> I64Vector4: ...
    @property
    def lzlo(self) -> I64Vector4: ...
    @property
    def lzll(self) -> I64Vector4: ...
    @property
    def lwxx(self) -> I64Vector4: ...
    @property
    def lwxy(self) -> I64Vector4: ...
    @property
    def lwxz(self) -> I64Vector4: ...
    @property
    def lwxw(self) -> I64Vector4: ...
    @property
    def lwxo(self) -> I64Vector4: ...
    @property
    def lwxl(self) -> I64Vector4: ...
    @property
    def lwyx(self) -> I64Vector4: ...
    @property
    def lwyy(self) -> I64Vector4: ...
    @property
    def lwyz(self) -> I64Vector4: ...
    @property
    def lwyw(self) -> I64Vector4: ...
    @property
    def lwyo(self) -> I64Vector4: ...
    @property
    def lwyl(self) -> I64Vector4: ...
    @property
    def lwzx(self) -> I64Vector4: ...
    @property
    def lwzy(self) -> I64Vector4: ...
    @property
    def lwzz(self) -> I64Vector4: ...
    @property
    def lwzw(self) -> I64Vector4: ...
    @property
    def lwzo(self) -> I64Vector4: ...
    @property
    def lwzl(self) -> I64Vector4: ...
    @property
    def lwwx(self) -> I64Vector4: ...
    @property
    def lwwy(self) -> I64Vector4: ...
    @property
    def lwwz(self) -> I64Vector4: ...
    @property
    def lwww(self) -> I64Vector4: ...
    @property
    def lwwo(self) -> I64Vector4: ...
    @property
    def lwwl(self) -> I64Vector4: ...
    @property
    def lwox(self) -> I64Vector4: ...
    @property
    def lwoy(self) -> I64Vector4: ...
    @property
    def lwoz(self) -> I64Vector4: ...
    @property
    def lwow(self) -> I64Vector4: ...
    @property
    def lwoo(self) -> I64Vector4: ...
    @property
    def lwol(self) -> I64Vector4: ...
    @property
    def lwlx(self) -> I64Vector4: ...
    @property
    def lwly(self) -> I64Vector4: ...
    @property
    def lwlz(self) -> I64Vector4: ...
    @property
    def lwlw(self) -> I64Vector4: ...
    @property
    def lwlo(self) -> I64Vector4: ...
    @property
    def lwll(self) -> I64Vector4: ...
    @property
    def loxx(self) -> I64Vector4: ...
    @property
    def loxy(self) -> I64Vector4: ...
    @property
    def loxz(self) -> I64Vector4: ...
    @property
    def loxw(self) -> I64Vector4: ...
    @property
    def loxo(self) -> I64Vector4: ...
    @property
    def loxl(self) -> I64Vector4: ...
    @property
    def loyx(self) -> I64Vector4: ...
    @property
    def loyy(self) -> I64Vector4: ...
    @property
    def loyz(self) -> I64Vector4: ...
    @property
    def loyw(self) -> I64Vector4: ...
    @property
    def loyo(self) -> I64Vector4: ...
    @property
    def loyl(self) -> I64Vector4: ...
    @property
    def lozx(self) -> I64Vector4: ...
    @property
    def lozy(self) -> I64Vector4: ...
    @property
    def lozz(self) -> I64Vector4: ...
    @property
    def lozw(self) -> I64Vector4: ...
    @property
    def lozo(self) -> I64Vector4: ...
    @property
    def lozl(self) -> I64Vector4: ...
    @property
    def lowx(self) -> I64Vector4: ...
    @property
    def lowy(self) -> I64Vector4: ...
    @property
    def lowz(self) -> I64Vector4: ...
    @property
    def loww(self) -> I64Vector4: ...
    @property
    def lowo(self) -> I64Vector4: ...
    @property
    def lowl(self) -> I64Vector4: ...
    @property
    def loox(self) -> I64Vector4: ...
    @property
    def looy(self) -> I64Vector4: ...
    @property
    def looz(self) -> I64Vector4: ...
    @property
    def loow(self) -> I64Vector4: ...
    @property
    def lolx(self) -> I64Vector4: ...
    @property
    def loly(self) -> I64Vector4: ...
    @property
    def lolz(self) -> I64Vector4: ...
    @property
    def lolw(self) -> I64Vector4: ...
    @property
    def llxx(self) -> I64Vector4: ...
    @property
    def llxy(self) -> I64Vector4: ...
    @property
    def llxz(self) -> I64Vector4: ...
    @property
    def llxw(self) -> I64Vector4: ...
    @property
    def llxo(self) -> I64Vector4: ...
    @property
    def llxl(self) -> I64Vector4: ...
    @property
    def llyx(self) -> I64Vector4: ...
    @property
    def llyy(self) -> I64Vector4: ...
    @property
    def llyz(self) -> I64Vector4: ...
    @property
    def llyw(self) -> I64Vector4: ...
    @property
    def llyo(self) -> I64Vector4: ...
    @property
    def llyl(self) -> I64Vector4: ...
    @property
    def llzx(self) -> I64Vector4: ...
    @property
    def llzy(self) -> I64Vector4: ...
    @property
    def llzz(self) -> I64Vector4: ...
    @property
    def llzw(self) -> I64Vector4: ...
    @property
    def llzo(self) -> I64Vector4: ...
    @property
    def llzl(self) -> I64Vector4: ...
    @property
    def llwx(self) -> I64Vector4: ...
    @property
    def llwy(self) -> I64Vector4: ...
    @property
    def llwz(self) -> I64Vector4: ...
    @property
    def llww(self) -> I64Vector4: ...
    @property
    def llwo(self) -> I64Vector4: ...
    @property
    def llwl(self) -> I64Vector4: ...
    @property
    def llox(self) -> I64Vector4: ...
    @property
    def lloy(self) -> I64Vector4: ...
    @property
    def lloz(self) -> I64Vector4: ...
    @property
    def llow(self) -> I64Vector4: ...
    @property
    def lllx(self) -> I64Vector4: ...
    @property
    def llly(self) -> I64Vector4: ...
    @property
    def lllz(self) -> I64Vector4: ...
    @property
    def lllw(self) -> I64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> I64Vector2: ...
    @property
    def rg(self) -> I64Vector2: ...
    @property
    def rb(self) -> I64Vector2: ...
    @property
    def ra(self) -> I64Vector2: ...
    @property
    def ro(self) -> I64Vector2: ...
    @property
    def rl(self) -> I64Vector2: ...
    @property
    def gr(self) -> I64Vector2: ...
    @property
    def gg(self) -> I64Vector2: ...
    @property
    def gb(self) -> I64Vector2: ...
    @property
    def ga(self) -> I64Vector2: ...
    @property
    def go(self) -> I64Vector2: ...
    @property
    def gl(self) -> I64Vector2: ...
    @property
    def br(self) -> I64Vector2: ...
    @property
    def bg(self) -> I64Vector2: ...
    @property
    def bb(self) -> I64Vector2: ...
    @property
    def ba(self) -> I64Vector2: ...
    @property
    def bo(self) -> I64Vector2: ...
    @property
    def bl(self) -> I64Vector2: ...
    @property
    def ar(self) -> I64Vector2: ...
    @property
    def ag(self) -> I64Vector2: ...
    @property
    def ab(self) -> I64Vector2: ...
    @property
    def aa(self) -> I64Vector2: ...
    @property
    def ao(self) -> I64Vector2: ...
    @property
    def al(self) -> I64Vector2: ...
    @property
    def og(self) -> I64Vector2: ...
    @property
    def ob(self) -> I64Vector2: ...
    @property
    def oa(self) -> I64Vector2: ...
    @property
    def lr(self) -> I64Vector2: ...
    @property
    def lg(self) -> I64Vector2: ...
    @property
    def lb(self) -> I64Vector2: ...
    @property
    def la(self) -> I64Vector2: ...
    @property
    def rrr(self) -> I64Vector3: ...
    @property
    def rrg(self) -> I64Vector3: ...
    @property
    def rrb(self) -> I64Vector3: ...
    @property
    def rra(self) -> I64Vector3: ...
    @property
    def rro(self) -> I64Vector3: ...
    @property
    def rrl(self) -> I64Vector3: ...
    @property
    def rgr(self) -> I64Vector3: ...
    @property
    def rgg(self) -> I64Vector3: ...
    @property
    def rgb(self) -> I64Vector3: ...
    @property
    def rga(self) -> I64Vector3: ...
    @property
    def rgo(self) -> I64Vector3: ...
    @property
    def rgl(self) -> I64Vector3: ...
    @property
    def rbr(self) -> I64Vector3: ...
    @property
    def rbg(self) -> I64Vector3: ...
    @property
    def rbb(self) -> I64Vector3: ...
    @property
    def rba(self) -> I64Vector3: ...
    @property
    def rbo(self) -> I64Vector3: ...
    @property
    def rbl(self) -> I64Vector3: ...
    @property
    def rar(self) -> I64Vector3: ...
    @property
    def rag(self) -> I64Vector3: ...
    @property
    def rab(self) -> I64Vector3: ...
    @property
    def raa(self) -> I64Vector3: ...
    @property
    def rao(self) -> I64Vector3: ...
    @property
    def ral(self) -> I64Vector3: ...
    @property
    def ror(self) -> I64Vector3: ...
    @property
    def rog(self) -> I64Vector3: ...
    @property
    def rob(self) -> I64Vector3: ...
    @property
    def roa(self) -> I64Vector3: ...
    @property
    def roo(self) -> I64Vector3: ...
    @property
    def rol(self) -> I64Vector3: ...
    @property
    def rlr(self) -> I64Vector3: ...
    @property
    def rlg(self) -> I64Vector3: ...
    @property
    def rlb(self) -> I64Vector3: ...
    @property
    def rla(self) -> I64Vector3: ...
    @property
    def rlo(self) -> I64Vector3: ...
    @property
    def rll(self) -> I64Vector3: ...
    @property
    def grr(self) -> I64Vector3: ...
    @property
    def grg(self) -> I64Vector3: ...
    @property
    def grb(self) -> I64Vector3: ...
    @property
    def gra(self) -> I64Vector3: ...
    @property
    def gro(self) -> I64Vector3: ...
    @property
    def grl(self) -> I64Vector3: ...
    @property
    def ggr(self) -> I64Vector3: ...
    @property
    def ggg(self) -> I64Vector3: ...
    @property
    def ggb(self) -> I64Vector3: ...
    @property
    def gga(self) -> I64Vector3: ...
    @property
    def ggo(self) -> I64Vector3: ...
    @property
    def ggl(self) -> I64Vector3: ...
    @property
    def gbr(self) -> I64Vector3: ...
    @property
    def gbg(self) -> I64Vector3: ...
    @property
    def gbb(self) -> I64Vector3: ...
    @property
    def gba(self) -> I64Vector3: ...
    @property
    def gbo(self) -> I64Vector3: ...
    @property
    def gbl(self) -> I64Vector3: ...
    @property
    def gar(self) -> I64Vector3: ...
    @property
    def gag(self) -> I64Vector3: ...
    @property
    def gab(self) -> I64Vector3: ...
    @property
    def gaa(self) -> I64Vector3: ...
    @property
    def gao(self) -> I64Vector3: ...
    @property
    def gal(self) -> I64Vector3: ...
    @property
    def gor(self) -> I64Vector3: ...
    @property
    def gog(self) -> I64Vector3: ...
    @property
    def gob(self) -> I64Vector3: ...
    @property
    def goa(self) -> I64Vector3: ...
    @property
    def goo(self) -> I64Vector3: ...
    @property
    def gol(self) -> I64Vector3: ...
    @property
    def glr(self) -> I64Vector3: ...
    @property
    def glg(self) -> I64Vector3: ...
    @property
    def glb(self) -> I64Vector3: ...
    @property
    def gla(self) -> I64Vector3: ...
    @property
    def glo(self) -> I64Vector3: ...
    @property
    def gll(self) -> I64Vector3: ...
    @property
    def brr(self) -> I64Vector3: ...
    @property
    def brg(self) -> I64Vector3: ...
    @property
    def brb(self) -> I64Vector3: ...
    @property
    def bra(self) -> I64Vector3: ...
    @property
    def bro(self) -> I64Vector3: ...
    @property
    def brl(self) -> I64Vector3: ...
    @property
    def bgr(self) -> I64Vector3: ...
    @property
    def bgg(self) -> I64Vector3: ...
    @property
    def bgb(self) -> I64Vector3: ...
    @property
    def bga(self) -> I64Vector3: ...
    @property
    def bgo(self) -> I64Vector3: ...
    @property
    def bgl(self) -> I64Vector3: ...
    @property
    def bbr(self) -> I64Vector3: ...
    @property
    def bbg(self) -> I64Vector3: ...
    @property
    def bbb(self) -> I64Vector3: ...
    @property
    def bba(self) -> I64Vector3: ...
    @property
    def bbo(self) -> I64Vector3: ...
    @property
    def bbl(self) -> I64Vector3: ...
    @property
    def bar(self) -> I64Vector3: ...
    @property
    def bag(self) -> I64Vector3: ...
    @property
    def bab(self) -> I64Vector3: ...
    @property
    def baa(self) -> I64Vector3: ...
    @property
    def bao(self) -> I64Vector3: ...
    @property
    def bal(self) -> I64Vector3: ...
    @property
    def bor(self) -> I64Vector3: ...
    @property
    def bog(self) -> I64Vector3: ...
    @property
    def bob(self) -> I64Vector3: ...
    @property
    def boa(self) -> I64Vector3: ...
    @property
    def boo(self) -> I64Vector3: ...
    @property
    def bol(self) -> I64Vector3: ...
    @property
    def blr(self) -> I64Vector3: ...
    @property
    def blg(self) -> I64Vector3: ...
    @property
    def blb(self) -> I64Vector3: ...
    @property
    def bla(self) -> I64Vector3: ...
    @property
    def blo(self) -> I64Vector3: ...
    @property
    def bll(self) -> I64Vector3: ...
    @property
    def arr(self) -> I64Vector3: ...
    @property
    def arg(self) -> I64Vector3: ...
    @property
    def arb(self) -> I64Vector3: ...
    @property
    def ara(self) -> I64Vector3: ...
    @property
    def aro(self) -> I64Vector3: ...
    @property
    def arl(self) -> I64Vector3: ...
    @property
    def agr(self) -> I64Vector3: ...
    @property
    def agg(self) -> I64Vector3: ...
    @property
    def agb(self) -> I64Vector3: ...
    @property
    def aga(self) -> I64Vector3: ...
    @property
    def ago(self) -> I64Vector3: ...
    @property
    def agl(self) -> I64Vector3: ...
    @property
    def abr(self) -> I64Vector3: ...
    @property
    def abg(self) -> I64Vector3: ...
    @property
    def abb(self) -> I64Vector3: ...
    @property
    def aba(self) -> I64Vector3: ...
    @property
    def abo(self) -> I64Vector3: ...
    @property
    def abl(self) -> I64Vector3: ...
    @property
    def aar(self) -> I64Vector3: ...
    @property
    def aag(self) -> I64Vector3: ...
    @property
    def aab(self) -> I64Vector3: ...
    @property
    def aaa(self) -> I64Vector3: ...
    @property
    def aao(self) -> I64Vector3: ...
    @property
    def aal(self) -> I64Vector3: ...
    @property
    def aor(self) -> I64Vector3: ...
    @property
    def aog(self) -> I64Vector3: ...
    @property
    def aob(self) -> I64Vector3: ...
    @property
    def aoa(self) -> I64Vector3: ...
    @property
    def aoo(self) -> I64Vector3: ...
    @property
    def aol(self) -> I64Vector3: ...
    @property
    def alr(self) -> I64Vector3: ...
    @property
    def alg(self) -> I64Vector3: ...
    @property
    def alb(self) -> I64Vector3: ...
    @property
    def ala(self) -> I64Vector3: ...
    @property
    def alo(self) -> I64Vector3: ...
    @property
    def all(self) -> I64Vector3: ...
    @property
    def orr(self) -> I64Vector3: ...
    @property
    def org(self) -> I64Vector3: ...
    @property
    def orb(self) -> I64Vector3: ...
    @property
    def ora(self) -> I64Vector3: ...
    @property
    def oro(self) -> I64Vector3: ...
    @property
    def orl(self) -> I64Vector3: ...
    @property
    def ogr(self) -> I64Vector3: ...
    @property
    def ogg(self) -> I64Vector3: ...
    @property
    def ogb(self) -> I64Vector3: ...
    @property
    def oga(self) -> I64Vector3: ...
    @property
    def ogo(self) -> I64Vector3: ...
    @property
    def ogl(self) -> I64Vector3: ...
    @property
    def obr(self) -> I64Vector3: ...
    @property
    def obg(self) -> I64Vector3: ...
    @property
    def obb(self) -> I64Vector3: ...
    @property
    def oba(self) -> I64Vector3: ...
    @property
    def obo(self) -> I64Vector3: ...
    @property
    def obl(self) -> I64Vector3: ...
    @property
    def oar(self) -> I64Vector3: ...
    @property
    def oag(self) -> I64Vector3: ...
    @property
    def oab(self) -> I64Vector3: ...
    @property
    def oaa(self) -> I64Vector3: ...
    @property
    def oao(self) -> I64Vector3: ...
    @property
    def oal(self) -> I64Vector3: ...
    @property
    def oor(self) -> I64Vector3: ...
    @property
    def oog(self) -> I64Vector3: ...
    @property
    def oob(self) -> I64Vector3: ...
    @property
    def ooa(self) -> I64Vector3: ...
    @property
    def olr(self) -> I64Vector3: ...
    @property
    def olg(self) -> I64Vector3: ...
    @property
    def olb(self) -> I64Vector3: ...
    @property
    def ola(self) -> I64Vector3: ...
    @property
    def lrr(self) -> I64Vector3: ...
    @property
    def lrg(self) -> I64Vector3: ...
    @property
    def lrb(self) -> I64Vector3: ...
    @property
    def lra(self) -> I64Vector3: ...
    @property
    def lro(self) -> I64Vector3: ...
    @property
    def lrl(self) -> I64Vector3: ...
    @property
    def lgr(self) -> I64Vector3: ...
    @property
    def lgg(self) -> I64Vector3: ...
    @property
    def lgb(self) -> I64Vector3: ...
    @property
    def lga(self) -> I64Vector3: ...
    @property
    def lgo(self) -> I64Vector3: ...
    @property
    def lgl(self) -> I64Vector3: ...
    @property
    def lbr(self) -> I64Vector3: ...
    @property
    def lbg(self) -> I64Vector3: ...
    @property
    def lbb(self) -> I64Vector3: ...
    @property
    def lba(self) -> I64Vector3: ...
    @property
    def lbo(self) -> I64Vector3: ...
    @property
    def lbl(self) -> I64Vector3: ...
    @property
    def lar(self) -> I64Vector3: ...
    @property
    def lag(self) -> I64Vector3: ...
    @property
    def lab(self) -> I64Vector3: ...
    @property
    def laa(self) -> I64Vector3: ...
    @property
    def lao(self) -> I64Vector3: ...
    @property
    def lal(self) -> I64Vector3: ...
    @property
    def lor(self) -> I64Vector3: ...
    @property
    def log(self) -> I64Vector3: ...
    @property
    def lob(self) -> I64Vector3: ...
    @property
    def loa(self) -> I64Vector3: ...
    @property
    def llr(self) -> I64Vector3: ...
    @property
    def llg(self) -> I64Vector3: ...
    @property
    def llb(self) -> I64Vector3: ...
    @property
    def lla(self) -> I64Vector3: ...
    @property
    def rrrr(self) -> I64Vector4: ...
    @property
    def rrrg(self) -> I64Vector4: ...
    @property
    def rrrb(self) -> I64Vector4: ...
    @property
    def rrra(self) -> I64Vector4: ...
    @property
    def rrro(self) -> I64Vector4: ...
    @property
    def rrrl(self) -> I64Vector4: ...
    @property
    def rrgr(self) -> I64Vector4: ...
    @property
    def rrgg(self) -> I64Vector4: ...
    @property
    def rrgb(self) -> I64Vector4: ...
    @property
    def rrga(self) -> I64Vector4: ...
    @property
    def rrgo(self) -> I64Vector4: ...
    @property
    def rrgl(self) -> I64Vector4: ...
    @property
    def rrbr(self) -> I64Vector4: ...
    @property
    def rrbg(self) -> I64Vector4: ...
    @property
    def rrbb(self) -> I64Vector4: ...
    @property
    def rrba(self) -> I64Vector4: ...
    @property
    def rrbo(self) -> I64Vector4: ...
    @property
    def rrbl(self) -> I64Vector4: ...
    @property
    def rrar(self) -> I64Vector4: ...
    @property
    def rrag(self) -> I64Vector4: ...
    @property
    def rrab(self) -> I64Vector4: ...
    @property
    def rraa(self) -> I64Vector4: ...
    @property
    def rrao(self) -> I64Vector4: ...
    @property
    def rral(self) -> I64Vector4: ...
    @property
    def rror(self) -> I64Vector4: ...
    @property
    def rrog(self) -> I64Vector4: ...
    @property
    def rrob(self) -> I64Vector4: ...
    @property
    def rroa(self) -> I64Vector4: ...
    @property
    def rroo(self) -> I64Vector4: ...
    @property
    def rrol(self) -> I64Vector4: ...
    @property
    def rrlr(self) -> I64Vector4: ...
    @property
    def rrlg(self) -> I64Vector4: ...
    @property
    def rrlb(self) -> I64Vector4: ...
    @property
    def rrla(self) -> I64Vector4: ...
    @property
    def rrlo(self) -> I64Vector4: ...
    @property
    def rrll(self) -> I64Vector4: ...
    @property
    def rgrr(self) -> I64Vector4: ...
    @property
    def rgrg(self) -> I64Vector4: ...
    @property
    def rgrb(self) -> I64Vector4: ...
    @property
    def rgra(self) -> I64Vector4: ...
    @property
    def rgro(self) -> I64Vector4: ...
    @property
    def rgrl(self) -> I64Vector4: ...
    @property
    def rggr(self) -> I64Vector4: ...
    @property
    def rggg(self) -> I64Vector4: ...
    @property
    def rggb(self) -> I64Vector4: ...
    @property
    def rgga(self) -> I64Vector4: ...
    @property
    def rggo(self) -> I64Vector4: ...
    @property
    def rggl(self) -> I64Vector4: ...
    @property
    def rgbr(self) -> I64Vector4: ...
    @property
    def rgbg(self) -> I64Vector4: ...
    @property
    def rgbb(self) -> I64Vector4: ...
    @property
    def rgba(self) -> I64Vector4: ...
    @property
    def rgbo(self) -> I64Vector4: ...
    @property
    def rgbl(self) -> I64Vector4: ...
    @property
    def rgar(self) -> I64Vector4: ...
    @property
    def rgag(self) -> I64Vector4: ...
    @property
    def rgab(self) -> I64Vector4: ...
    @property
    def rgaa(self) -> I64Vector4: ...
    @property
    def rgao(self) -> I64Vector4: ...
    @property
    def rgal(self) -> I64Vector4: ...
    @property
    def rgor(self) -> I64Vector4: ...
    @property
    def rgog(self) -> I64Vector4: ...
    @property
    def rgob(self) -> I64Vector4: ...
    @property
    def rgoa(self) -> I64Vector4: ...
    @property
    def rgoo(self) -> I64Vector4: ...
    @property
    def rgol(self) -> I64Vector4: ...
    @property
    def rglr(self) -> I64Vector4: ...
    @property
    def rglg(self) -> I64Vector4: ...
    @property
    def rglb(self) -> I64Vector4: ...
    @property
    def rgla(self) -> I64Vector4: ...
    @property
    def rglo(self) -> I64Vector4: ...
    @property
    def rgll(self) -> I64Vector4: ...
    @property
    def rbrr(self) -> I64Vector4: ...
    @property
    def rbrg(self) -> I64Vector4: ...
    @property
    def rbrb(self) -> I64Vector4: ...
    @property
    def rbra(self) -> I64Vector4: ...
    @property
    def rbro(self) -> I64Vector4: ...
    @property
    def rbrl(self) -> I64Vector4: ...
    @property
    def rbgr(self) -> I64Vector4: ...
    @property
    def rbgg(self) -> I64Vector4: ...
    @property
    def rbgb(self) -> I64Vector4: ...
    @property
    def rbga(self) -> I64Vector4: ...
    @property
    def rbgo(self) -> I64Vector4: ...
    @property
    def rbgl(self) -> I64Vector4: ...
    @property
    def rbbr(self) -> I64Vector4: ...
    @property
    def rbbg(self) -> I64Vector4: ...
    @property
    def rbbb(self) -> I64Vector4: ...
    @property
    def rbba(self) -> I64Vector4: ...
    @property
    def rbbo(self) -> I64Vector4: ...
    @property
    def rbbl(self) -> I64Vector4: ...
    @property
    def rbar(self) -> I64Vector4: ...
    @property
    def rbag(self) -> I64Vector4: ...
    @property
    def rbab(self) -> I64Vector4: ...
    @property
    def rbaa(self) -> I64Vector4: ...
    @property
    def rbao(self) -> I64Vector4: ...
    @property
    def rbal(self) -> I64Vector4: ...
    @property
    def rbor(self) -> I64Vector4: ...
    @property
    def rbog(self) -> I64Vector4: ...
    @property
    def rbob(self) -> I64Vector4: ...
    @property
    def rboa(self) -> I64Vector4: ...
    @property
    def rboo(self) -> I64Vector4: ...
    @property
    def rbol(self) -> I64Vector4: ...
    @property
    def rblr(self) -> I64Vector4: ...
    @property
    def rblg(self) -> I64Vector4: ...
    @property
    def rblb(self) -> I64Vector4: ...
    @property
    def rbla(self) -> I64Vector4: ...
    @property
    def rblo(self) -> I64Vector4: ...
    @property
    def rbll(self) -> I64Vector4: ...
    @property
    def rarr(self) -> I64Vector4: ...
    @property
    def rarg(self) -> I64Vector4: ...
    @property
    def rarb(self) -> I64Vector4: ...
    @property
    def rara(self) -> I64Vector4: ...
    @property
    def raro(self) -> I64Vector4: ...
    @property
    def rarl(self) -> I64Vector4: ...
    @property
    def ragr(self) -> I64Vector4: ...
    @property
    def ragg(self) -> I64Vector4: ...
    @property
    def ragb(self) -> I64Vector4: ...
    @property
    def raga(self) -> I64Vector4: ...
    @property
    def rago(self) -> I64Vector4: ...
    @property
    def ragl(self) -> I64Vector4: ...
    @property
    def rabr(self) -> I64Vector4: ...
    @property
    def rabg(self) -> I64Vector4: ...
    @property
    def rabb(self) -> I64Vector4: ...
    @property
    def raba(self) -> I64Vector4: ...
    @property
    def rabo(self) -> I64Vector4: ...
    @property
    def rabl(self) -> I64Vector4: ...
    @property
    def raar(self) -> I64Vector4: ...
    @property
    def raag(self) -> I64Vector4: ...
    @property
    def raab(self) -> I64Vector4: ...
    @property
    def raaa(self) -> I64Vector4: ...
    @property
    def raao(self) -> I64Vector4: ...
    @property
    def raal(self) -> I64Vector4: ...
    @property
    def raor(self) -> I64Vector4: ...
    @property
    def raog(self) -> I64Vector4: ...
    @property
    def raob(self) -> I64Vector4: ...
    @property
    def raoa(self) -> I64Vector4: ...
    @property
    def raoo(self) -> I64Vector4: ...
    @property
    def raol(self) -> I64Vector4: ...
    @property
    def ralr(self) -> I64Vector4: ...
    @property
    def ralg(self) -> I64Vector4: ...
    @property
    def ralb(self) -> I64Vector4: ...
    @property
    def rala(self) -> I64Vector4: ...
    @property
    def ralo(self) -> I64Vector4: ...
    @property
    def rall(self) -> I64Vector4: ...
    @property
    def rorr(self) -> I64Vector4: ...
    @property
    def rorg(self) -> I64Vector4: ...
    @property
    def rorb(self) -> I64Vector4: ...
    @property
    def rora(self) -> I64Vector4: ...
    @property
    def roro(self) -> I64Vector4: ...
    @property
    def rorl(self) -> I64Vector4: ...
    @property
    def rogr(self) -> I64Vector4: ...
    @property
    def rogg(self) -> I64Vector4: ...
    @property
    def rogb(self) -> I64Vector4: ...
    @property
    def roga(self) -> I64Vector4: ...
    @property
    def rogo(self) -> I64Vector4: ...
    @property
    def rogl(self) -> I64Vector4: ...
    @property
    def robr(self) -> I64Vector4: ...
    @property
    def robg(self) -> I64Vector4: ...
    @property
    def robb(self) -> I64Vector4: ...
    @property
    def roba(self) -> I64Vector4: ...
    @property
    def robo(self) -> I64Vector4: ...
    @property
    def robl(self) -> I64Vector4: ...
    @property
    def roar(self) -> I64Vector4: ...
    @property
    def roag(self) -> I64Vector4: ...
    @property
    def roab(self) -> I64Vector4: ...
    @property
    def roaa(self) -> I64Vector4: ...
    @property
    def roao(self) -> I64Vector4: ...
    @property
    def roal(self) -> I64Vector4: ...
    @property
    def roor(self) -> I64Vector4: ...
    @property
    def roog(self) -> I64Vector4: ...
    @property
    def roob(self) -> I64Vector4: ...
    @property
    def rooa(self) -> I64Vector4: ...
    @property
    def rooo(self) -> I64Vector4: ...
    @property
    def rool(self) -> I64Vector4: ...
    @property
    def rolr(self) -> I64Vector4: ...
    @property
    def rolg(self) -> I64Vector4: ...
    @property
    def rolb(self) -> I64Vector4: ...
    @property
    def rola(self) -> I64Vector4: ...
    @property
    def rolo(self) -> I64Vector4: ...
    @property
    def roll(self) -> I64Vector4: ...
    @property
    def rlrr(self) -> I64Vector4: ...
    @property
    def rlrg(self) -> I64Vector4: ...
    @property
    def rlrb(self) -> I64Vector4: ...
    @property
    def rlra(self) -> I64Vector4: ...
    @property
    def rlro(self) -> I64Vector4: ...
    @property
    def rlrl(self) -> I64Vector4: ...
    @property
    def rlgr(self) -> I64Vector4: ...
    @property
    def rlgg(self) -> I64Vector4: ...
    @property
    def rlgb(self) -> I64Vector4: ...
    @property
    def rlga(self) -> I64Vector4: ...
    @property
    def rlgo(self) -> I64Vector4: ...
    @property
    def rlgl(self) -> I64Vector4: ...
    @property
    def rlbr(self) -> I64Vector4: ...
    @property
    def rlbg(self) -> I64Vector4: ...
    @property
    def rlbb(self) -> I64Vector4: ...
    @property
    def rlba(self) -> I64Vector4: ...
    @property
    def rlbo(self) -> I64Vector4: ...
    @property
    def rlbl(self) -> I64Vector4: ...
    @property
    def rlar(self) -> I64Vector4: ...
    @property
    def rlag(self) -> I64Vector4: ...
    @property
    def rlab(self) -> I64Vector4: ...
    @property
    def rlaa(self) -> I64Vector4: ...
    @property
    def rlao(self) -> I64Vector4: ...
    @property
    def rlal(self) -> I64Vector4: ...
    @property
    def rlor(self) -> I64Vector4: ...
    @property
    def rlog(self) -> I64Vector4: ...
    @property
    def rlob(self) -> I64Vector4: ...
    @property
    def rloa(self) -> I64Vector4: ...
    @property
    def rloo(self) -> I64Vector4: ...
    @property
    def rlol(self) -> I64Vector4: ...
    @property
    def rllr(self) -> I64Vector4: ...
    @property
    def rllg(self) -> I64Vector4: ...
    @property
    def rllb(self) -> I64Vector4: ...
    @property
    def rlla(self) -> I64Vector4: ...
    @property
    def rllo(self) -> I64Vector4: ...
    @property
    def rlll(self) -> I64Vector4: ...
    @property
    def grrr(self) -> I64Vector4: ...
    @property
    def grrg(self) -> I64Vector4: ...
    @property
    def grrb(self) -> I64Vector4: ...
    @property
    def grra(self) -> I64Vector4: ...
    @property
    def grro(self) -> I64Vector4: ...
    @property
    def grrl(self) -> I64Vector4: ...
    @property
    def grgr(self) -> I64Vector4: ...
    @property
    def grgg(self) -> I64Vector4: ...
    @property
    def grgb(self) -> I64Vector4: ...
    @property
    def grga(self) -> I64Vector4: ...
    @property
    def grgo(self) -> I64Vector4: ...
    @property
    def grgl(self) -> I64Vector4: ...
    @property
    def grbr(self) -> I64Vector4: ...
    @property
    def grbg(self) -> I64Vector4: ...
    @property
    def grbb(self) -> I64Vector4: ...
    @property
    def grba(self) -> I64Vector4: ...
    @property
    def grbo(self) -> I64Vector4: ...
    @property
    def grbl(self) -> I64Vector4: ...
    @property
    def grar(self) -> I64Vector4: ...
    @property
    def grag(self) -> I64Vector4: ...
    @property
    def grab(self) -> I64Vector4: ...
    @property
    def graa(self) -> I64Vector4: ...
    @property
    def grao(self) -> I64Vector4: ...
    @property
    def gral(self) -> I64Vector4: ...
    @property
    def gror(self) -> I64Vector4: ...
    @property
    def grog(self) -> I64Vector4: ...
    @property
    def grob(self) -> I64Vector4: ...
    @property
    def groa(self) -> I64Vector4: ...
    @property
    def groo(self) -> I64Vector4: ...
    @property
    def grol(self) -> I64Vector4: ...
    @property
    def grlr(self) -> I64Vector4: ...
    @property
    def grlg(self) -> I64Vector4: ...
    @property
    def grlb(self) -> I64Vector4: ...
    @property
    def grla(self) -> I64Vector4: ...
    @property
    def grlo(self) -> I64Vector4: ...
    @property
    def grll(self) -> I64Vector4: ...
    @property
    def ggrr(self) -> I64Vector4: ...
    @property
    def ggrg(self) -> I64Vector4: ...
    @property
    def ggrb(self) -> I64Vector4: ...
    @property
    def ggra(self) -> I64Vector4: ...
    @property
    def ggro(self) -> I64Vector4: ...
    @property
    def ggrl(self) -> I64Vector4: ...
    @property
    def gggr(self) -> I64Vector4: ...
    @property
    def gggg(self) -> I64Vector4: ...
    @property
    def gggb(self) -> I64Vector4: ...
    @property
    def ggga(self) -> I64Vector4: ...
    @property
    def gggo(self) -> I64Vector4: ...
    @property
    def gggl(self) -> I64Vector4: ...
    @property
    def ggbr(self) -> I64Vector4: ...
    @property
    def ggbg(self) -> I64Vector4: ...
    @property
    def ggbb(self) -> I64Vector4: ...
    @property
    def ggba(self) -> I64Vector4: ...
    @property
    def ggbo(self) -> I64Vector4: ...
    @property
    def ggbl(self) -> I64Vector4: ...
    @property
    def ggar(self) -> I64Vector4: ...
    @property
    def ggag(self) -> I64Vector4: ...
    @property
    def ggab(self) -> I64Vector4: ...
    @property
    def ggaa(self) -> I64Vector4: ...
    @property
    def ggao(self) -> I64Vector4: ...
    @property
    def ggal(self) -> I64Vector4: ...
    @property
    def ggor(self) -> I64Vector4: ...
    @property
    def ggog(self) -> I64Vector4: ...
    @property
    def ggob(self) -> I64Vector4: ...
    @property
    def ggoa(self) -> I64Vector4: ...
    @property
    def ggoo(self) -> I64Vector4: ...
    @property
    def ggol(self) -> I64Vector4: ...
    @property
    def gglr(self) -> I64Vector4: ...
    @property
    def gglg(self) -> I64Vector4: ...
    @property
    def gglb(self) -> I64Vector4: ...
    @property
    def ggla(self) -> I64Vector4: ...
    @property
    def gglo(self) -> I64Vector4: ...
    @property
    def ggll(self) -> I64Vector4: ...
    @property
    def gbrr(self) -> I64Vector4: ...
    @property
    def gbrg(self) -> I64Vector4: ...
    @property
    def gbrb(self) -> I64Vector4: ...
    @property
    def gbra(self) -> I64Vector4: ...
    @property
    def gbro(self) -> I64Vector4: ...
    @property
    def gbrl(self) -> I64Vector4: ...
    @property
    def gbgr(self) -> I64Vector4: ...
    @property
    def gbgg(self) -> I64Vector4: ...
    @property
    def gbgb(self) -> I64Vector4: ...
    @property
    def gbga(self) -> I64Vector4: ...
    @property
    def gbgo(self) -> I64Vector4: ...
    @property
    def gbgl(self) -> I64Vector4: ...
    @property
    def gbbr(self) -> I64Vector4: ...
    @property
    def gbbg(self) -> I64Vector4: ...
    @property
    def gbbb(self) -> I64Vector4: ...
    @property
    def gbba(self) -> I64Vector4: ...
    @property
    def gbbo(self) -> I64Vector4: ...
    @property
    def gbbl(self) -> I64Vector4: ...
    @property
    def gbar(self) -> I64Vector4: ...
    @property
    def gbag(self) -> I64Vector4: ...
    @property
    def gbab(self) -> I64Vector4: ...
    @property
    def gbaa(self) -> I64Vector4: ...
    @property
    def gbao(self) -> I64Vector4: ...
    @property
    def gbal(self) -> I64Vector4: ...
    @property
    def gbor(self) -> I64Vector4: ...
    @property
    def gbog(self) -> I64Vector4: ...
    @property
    def gbob(self) -> I64Vector4: ...
    @property
    def gboa(self) -> I64Vector4: ...
    @property
    def gboo(self) -> I64Vector4: ...
    @property
    def gbol(self) -> I64Vector4: ...
    @property
    def gblr(self) -> I64Vector4: ...
    @property
    def gblg(self) -> I64Vector4: ...
    @property
    def gblb(self) -> I64Vector4: ...
    @property
    def gbla(self) -> I64Vector4: ...
    @property
    def gblo(self) -> I64Vector4: ...
    @property
    def gbll(self) -> I64Vector4: ...
    @property
    def garr(self) -> I64Vector4: ...
    @property
    def garg(self) -> I64Vector4: ...
    @property
    def garb(self) -> I64Vector4: ...
    @property
    def gara(self) -> I64Vector4: ...
    @property
    def garo(self) -> I64Vector4: ...
    @property
    def garl(self) -> I64Vector4: ...
    @property
    def gagr(self) -> I64Vector4: ...
    @property
    def gagg(self) -> I64Vector4: ...
    @property
    def gagb(self) -> I64Vector4: ...
    @property
    def gaga(self) -> I64Vector4: ...
    @property
    def gago(self) -> I64Vector4: ...
    @property
    def gagl(self) -> I64Vector4: ...
    @property
    def gabr(self) -> I64Vector4: ...
    @property
    def gabg(self) -> I64Vector4: ...
    @property
    def gabb(self) -> I64Vector4: ...
    @property
    def gaba(self) -> I64Vector4: ...
    @property
    def gabo(self) -> I64Vector4: ...
    @property
    def gabl(self) -> I64Vector4: ...
    @property
    def gaar(self) -> I64Vector4: ...
    @property
    def gaag(self) -> I64Vector4: ...
    @property
    def gaab(self) -> I64Vector4: ...
    @property
    def gaaa(self) -> I64Vector4: ...
    @property
    def gaao(self) -> I64Vector4: ...
    @property
    def gaal(self) -> I64Vector4: ...
    @property
    def gaor(self) -> I64Vector4: ...
    @property
    def gaog(self) -> I64Vector4: ...
    @property
    def gaob(self) -> I64Vector4: ...
    @property
    def gaoa(self) -> I64Vector4: ...
    @property
    def gaoo(self) -> I64Vector4: ...
    @property
    def gaol(self) -> I64Vector4: ...
    @property
    def galr(self) -> I64Vector4: ...
    @property
    def galg(self) -> I64Vector4: ...
    @property
    def galb(self) -> I64Vector4: ...
    @property
    def gala(self) -> I64Vector4: ...
    @property
    def galo(self) -> I64Vector4: ...
    @property
    def gall(self) -> I64Vector4: ...
    @property
    def gorr(self) -> I64Vector4: ...
    @property
    def gorg(self) -> I64Vector4: ...
    @property
    def gorb(self) -> I64Vector4: ...
    @property
    def gora(self) -> I64Vector4: ...
    @property
    def goro(self) -> I64Vector4: ...
    @property
    def gorl(self) -> I64Vector4: ...
    @property
    def gogr(self) -> I64Vector4: ...
    @property
    def gogg(self) -> I64Vector4: ...
    @property
    def gogb(self) -> I64Vector4: ...
    @property
    def goga(self) -> I64Vector4: ...
    @property
    def gogo(self) -> I64Vector4: ...
    @property
    def gogl(self) -> I64Vector4: ...
    @property
    def gobr(self) -> I64Vector4: ...
    @property
    def gobg(self) -> I64Vector4: ...
    @property
    def gobb(self) -> I64Vector4: ...
    @property
    def goba(self) -> I64Vector4: ...
    @property
    def gobo(self) -> I64Vector4: ...
    @property
    def gobl(self) -> I64Vector4: ...
    @property
    def goar(self) -> I64Vector4: ...
    @property
    def goag(self) -> I64Vector4: ...
    @property
    def goab(self) -> I64Vector4: ...
    @property
    def goaa(self) -> I64Vector4: ...
    @property
    def goao(self) -> I64Vector4: ...
    @property
    def goal(self) -> I64Vector4: ...
    @property
    def goor(self) -> I64Vector4: ...
    @property
    def goog(self) -> I64Vector4: ...
    @property
    def goob(self) -> I64Vector4: ...
    @property
    def gooa(self) -> I64Vector4: ...
    @property
    def gooo(self) -> I64Vector4: ...
    @property
    def gool(self) -> I64Vector4: ...
    @property
    def golr(self) -> I64Vector4: ...
    @property
    def golg(self) -> I64Vector4: ...
    @property
    def golb(self) -> I64Vector4: ...
    @property
    def gola(self) -> I64Vector4: ...
    @property
    def golo(self) -> I64Vector4: ...
    @property
    def goll(self) -> I64Vector4: ...
    @property
    def glrr(self) -> I64Vector4: ...
    @property
    def glrg(self) -> I64Vector4: ...
    @property
    def glrb(self) -> I64Vector4: ...
    @property
    def glra(self) -> I64Vector4: ...
    @property
    def glro(self) -> I64Vector4: ...
    @property
    def glrl(self) -> I64Vector4: ...
    @property
    def glgr(self) -> I64Vector4: ...
    @property
    def glgg(self) -> I64Vector4: ...
    @property
    def glgb(self) -> I64Vector4: ...
    @property
    def glga(self) -> I64Vector4: ...
    @property
    def glgo(self) -> I64Vector4: ...
    @property
    def glgl(self) -> I64Vector4: ...
    @property
    def glbr(self) -> I64Vector4: ...
    @property
    def glbg(self) -> I64Vector4: ...
    @property
    def glbb(self) -> I64Vector4: ...
    @property
    def glba(self) -> I64Vector4: ...
    @property
    def glbo(self) -> I64Vector4: ...
    @property
    def glbl(self) -> I64Vector4: ...
    @property
    def glar(self) -> I64Vector4: ...
    @property
    def glag(self) -> I64Vector4: ...
    @property
    def glab(self) -> I64Vector4: ...
    @property
    def glaa(self) -> I64Vector4: ...
    @property
    def glao(self) -> I64Vector4: ...
    @property
    def glal(self) -> I64Vector4: ...
    @property
    def glor(self) -> I64Vector4: ...
    @property
    def glog(self) -> I64Vector4: ...
    @property
    def glob(self) -> I64Vector4: ...
    @property
    def gloa(self) -> I64Vector4: ...
    @property
    def gloo(self) -> I64Vector4: ...
    @property
    def glol(self) -> I64Vector4: ...
    @property
    def gllr(self) -> I64Vector4: ...
    @property
    def gllg(self) -> I64Vector4: ...
    @property
    def gllb(self) -> I64Vector4: ...
    @property
    def glla(self) -> I64Vector4: ...
    @property
    def gllo(self) -> I64Vector4: ...
    @property
    def glll(self) -> I64Vector4: ...
    @property
    def brrr(self) -> I64Vector4: ...
    @property
    def brrg(self) -> I64Vector4: ...
    @property
    def brrb(self) -> I64Vector4: ...
    @property
    def brra(self) -> I64Vector4: ...
    @property
    def brro(self) -> I64Vector4: ...
    @property
    def brrl(self) -> I64Vector4: ...
    @property
    def brgr(self) -> I64Vector4: ...
    @property
    def brgg(self) -> I64Vector4: ...
    @property
    def brgb(self) -> I64Vector4: ...
    @property
    def brga(self) -> I64Vector4: ...
    @property
    def brgo(self) -> I64Vector4: ...
    @property
    def brgl(self) -> I64Vector4: ...
    @property
    def brbr(self) -> I64Vector4: ...
    @property
    def brbg(self) -> I64Vector4: ...
    @property
    def brbb(self) -> I64Vector4: ...
    @property
    def brba(self) -> I64Vector4: ...
    @property
    def brbo(self) -> I64Vector4: ...
    @property
    def brbl(self) -> I64Vector4: ...
    @property
    def brar(self) -> I64Vector4: ...
    @property
    def brag(self) -> I64Vector4: ...
    @property
    def brab(self) -> I64Vector4: ...
    @property
    def braa(self) -> I64Vector4: ...
    @property
    def brao(self) -> I64Vector4: ...
    @property
    def bral(self) -> I64Vector4: ...
    @property
    def bror(self) -> I64Vector4: ...
    @property
    def brog(self) -> I64Vector4: ...
    @property
    def brob(self) -> I64Vector4: ...
    @property
    def broa(self) -> I64Vector4: ...
    @property
    def broo(self) -> I64Vector4: ...
    @property
    def brol(self) -> I64Vector4: ...
    @property
    def brlr(self) -> I64Vector4: ...
    @property
    def brlg(self) -> I64Vector4: ...
    @property
    def brlb(self) -> I64Vector4: ...
    @property
    def brla(self) -> I64Vector4: ...
    @property
    def brlo(self) -> I64Vector4: ...
    @property
    def brll(self) -> I64Vector4: ...
    @property
    def bgrr(self) -> I64Vector4: ...
    @property
    def bgrg(self) -> I64Vector4: ...
    @property
    def bgrb(self) -> I64Vector4: ...
    @property
    def bgra(self) -> I64Vector4: ...
    @property
    def bgro(self) -> I64Vector4: ...
    @property
    def bgrl(self) -> I64Vector4: ...
    @property
    def bggr(self) -> I64Vector4: ...
    @property
    def bggg(self) -> I64Vector4: ...
    @property
    def bggb(self) -> I64Vector4: ...
    @property
    def bgga(self) -> I64Vector4: ...
    @property
    def bggo(self) -> I64Vector4: ...
    @property
    def bggl(self) -> I64Vector4: ...
    @property
    def bgbr(self) -> I64Vector4: ...
    @property
    def bgbg(self) -> I64Vector4: ...
    @property
    def bgbb(self) -> I64Vector4: ...
    @property
    def bgba(self) -> I64Vector4: ...
    @property
    def bgbo(self) -> I64Vector4: ...
    @property
    def bgbl(self) -> I64Vector4: ...
    @property
    def bgar(self) -> I64Vector4: ...
    @property
    def bgag(self) -> I64Vector4: ...
    @property
    def bgab(self) -> I64Vector4: ...
    @property
    def bgaa(self) -> I64Vector4: ...
    @property
    def bgao(self) -> I64Vector4: ...
    @property
    def bgal(self) -> I64Vector4: ...
    @property
    def bgor(self) -> I64Vector4: ...
    @property
    def bgog(self) -> I64Vector4: ...
    @property
    def bgob(self) -> I64Vector4: ...
    @property
    def bgoa(self) -> I64Vector4: ...
    @property
    def bgoo(self) -> I64Vector4: ...
    @property
    def bgol(self) -> I64Vector4: ...
    @property
    def bglr(self) -> I64Vector4: ...
    @property
    def bglg(self) -> I64Vector4: ...
    @property
    def bglb(self) -> I64Vector4: ...
    @property
    def bgla(self) -> I64Vector4: ...
    @property
    def bglo(self) -> I64Vector4: ...
    @property
    def bgll(self) -> I64Vector4: ...
    @property
    def bbrr(self) -> I64Vector4: ...
    @property
    def bbrg(self) -> I64Vector4: ...
    @property
    def bbrb(self) -> I64Vector4: ...
    @property
    def bbra(self) -> I64Vector4: ...
    @property
    def bbro(self) -> I64Vector4: ...
    @property
    def bbrl(self) -> I64Vector4: ...
    @property
    def bbgr(self) -> I64Vector4: ...
    @property
    def bbgg(self) -> I64Vector4: ...
    @property
    def bbgb(self) -> I64Vector4: ...
    @property
    def bbga(self) -> I64Vector4: ...
    @property
    def bbgo(self) -> I64Vector4: ...
    @property
    def bbgl(self) -> I64Vector4: ...
    @property
    def bbbr(self) -> I64Vector4: ...
    @property
    def bbbg(self) -> I64Vector4: ...
    @property
    def bbbb(self) -> I64Vector4: ...
    @property
    def bbba(self) -> I64Vector4: ...
    @property
    def bbbo(self) -> I64Vector4: ...
    @property
    def bbbl(self) -> I64Vector4: ...
    @property
    def bbar(self) -> I64Vector4: ...
    @property
    def bbag(self) -> I64Vector4: ...
    @property
    def bbab(self) -> I64Vector4: ...
    @property
    def bbaa(self) -> I64Vector4: ...
    @property
    def bbao(self) -> I64Vector4: ...
    @property
    def bbal(self) -> I64Vector4: ...
    @property
    def bbor(self) -> I64Vector4: ...
    @property
    def bbog(self) -> I64Vector4: ...
    @property
    def bbob(self) -> I64Vector4: ...
    @property
    def bboa(self) -> I64Vector4: ...
    @property
    def bboo(self) -> I64Vector4: ...
    @property
    def bbol(self) -> I64Vector4: ...
    @property
    def bblr(self) -> I64Vector4: ...
    @property
    def bblg(self) -> I64Vector4: ...
    @property
    def bblb(self) -> I64Vector4: ...
    @property
    def bbla(self) -> I64Vector4: ...
    @property
    def bblo(self) -> I64Vector4: ...
    @property
    def bbll(self) -> I64Vector4: ...
    @property
    def barr(self) -> I64Vector4: ...
    @property
    def barg(self) -> I64Vector4: ...
    @property
    def barb(self) -> I64Vector4: ...
    @property
    def bara(self) -> I64Vector4: ...
    @property
    def baro(self) -> I64Vector4: ...
    @property
    def barl(self) -> I64Vector4: ...
    @property
    def bagr(self) -> I64Vector4: ...
    @property
    def bagg(self) -> I64Vector4: ...
    @property
    def bagb(self) -> I64Vector4: ...
    @property
    def baga(self) -> I64Vector4: ...
    @property
    def bago(self) -> I64Vector4: ...
    @property
    def bagl(self) -> I64Vector4: ...
    @property
    def babr(self) -> I64Vector4: ...
    @property
    def babg(self) -> I64Vector4: ...
    @property
    def babb(self) -> I64Vector4: ...
    @property
    def baba(self) -> I64Vector4: ...
    @property
    def babo(self) -> I64Vector4: ...
    @property
    def babl(self) -> I64Vector4: ...
    @property
    def baar(self) -> I64Vector4: ...
    @property
    def baag(self) -> I64Vector4: ...
    @property
    def baab(self) -> I64Vector4: ...
    @property
    def baaa(self) -> I64Vector4: ...
    @property
    def baao(self) -> I64Vector4: ...
    @property
    def baal(self) -> I64Vector4: ...
    @property
    def baor(self) -> I64Vector4: ...
    @property
    def baog(self) -> I64Vector4: ...
    @property
    def baob(self) -> I64Vector4: ...
    @property
    def baoa(self) -> I64Vector4: ...
    @property
    def baoo(self) -> I64Vector4: ...
    @property
    def baol(self) -> I64Vector4: ...
    @property
    def balr(self) -> I64Vector4: ...
    @property
    def balg(self) -> I64Vector4: ...
    @property
    def balb(self) -> I64Vector4: ...
    @property
    def bala(self) -> I64Vector4: ...
    @property
    def balo(self) -> I64Vector4: ...
    @property
    def ball(self) -> I64Vector4: ...
    @property
    def borr(self) -> I64Vector4: ...
    @property
    def borg(self) -> I64Vector4: ...
    @property
    def borb(self) -> I64Vector4: ...
    @property
    def bora(self) -> I64Vector4: ...
    @property
    def boro(self) -> I64Vector4: ...
    @property
    def borl(self) -> I64Vector4: ...
    @property
    def bogr(self) -> I64Vector4: ...
    @property
    def bogg(self) -> I64Vector4: ...
    @property
    def bogb(self) -> I64Vector4: ...
    @property
    def boga(self) -> I64Vector4: ...
    @property
    def bogo(self) -> I64Vector4: ...
    @property
    def bogl(self) -> I64Vector4: ...
    @property
    def bobr(self) -> I64Vector4: ...
    @property
    def bobg(self) -> I64Vector4: ...
    @property
    def bobb(self) -> I64Vector4: ...
    @property
    def boba(self) -> I64Vector4: ...
    @property
    def bobo(self) -> I64Vector4: ...
    @property
    def bobl(self) -> I64Vector4: ...
    @property
    def boar(self) -> I64Vector4: ...
    @property
    def boag(self) -> I64Vector4: ...
    @property
    def boab(self) -> I64Vector4: ...
    @property
    def boaa(self) -> I64Vector4: ...
    @property
    def boao(self) -> I64Vector4: ...
    @property
    def boal(self) -> I64Vector4: ...
    @property
    def boor(self) -> I64Vector4: ...
    @property
    def boog(self) -> I64Vector4: ...
    @property
    def boob(self) -> I64Vector4: ...
    @property
    def booa(self) -> I64Vector4: ...
    @property
    def booo(self) -> I64Vector4: ...
    @property
    def bool(self) -> I64Vector4: ...
    @property
    def bolr(self) -> I64Vector4: ...
    @property
    def bolg(self) -> I64Vector4: ...
    @property
    def bolb(self) -> I64Vector4: ...
    @property
    def bola(self) -> I64Vector4: ...
    @property
    def bolo(self) -> I64Vector4: ...
    @property
    def boll(self) -> I64Vector4: ...
    @property
    def blrr(self) -> I64Vector4: ...
    @property
    def blrg(self) -> I64Vector4: ...
    @property
    def blrb(self) -> I64Vector4: ...
    @property
    def blra(self) -> I64Vector4: ...
    @property
    def blro(self) -> I64Vector4: ...
    @property
    def blrl(self) -> I64Vector4: ...
    @property
    def blgr(self) -> I64Vector4: ...
    @property
    def blgg(self) -> I64Vector4: ...
    @property
    def blgb(self) -> I64Vector4: ...
    @property
    def blga(self) -> I64Vector4: ...
    @property
    def blgo(self) -> I64Vector4: ...
    @property
    def blgl(self) -> I64Vector4: ...
    @property
    def blbr(self) -> I64Vector4: ...
    @property
    def blbg(self) -> I64Vector4: ...
    @property
    def blbb(self) -> I64Vector4: ...
    @property
    def blba(self) -> I64Vector4: ...
    @property
    def blbo(self) -> I64Vector4: ...
    @property
    def blbl(self) -> I64Vector4: ...
    @property
    def blar(self) -> I64Vector4: ...
    @property
    def blag(self) -> I64Vector4: ...
    @property
    def blab(self) -> I64Vector4: ...
    @property
    def blaa(self) -> I64Vector4: ...
    @property
    def blao(self) -> I64Vector4: ...
    @property
    def blal(self) -> I64Vector4: ...
    @property
    def blor(self) -> I64Vector4: ...
    @property
    def blog(self) -> I64Vector4: ...
    @property
    def blob(self) -> I64Vector4: ...
    @property
    def bloa(self) -> I64Vector4: ...
    @property
    def bloo(self) -> I64Vector4: ...
    @property
    def blol(self) -> I64Vector4: ...
    @property
    def bllr(self) -> I64Vector4: ...
    @property
    def bllg(self) -> I64Vector4: ...
    @property
    def bllb(self) -> I64Vector4: ...
    @property
    def blla(self) -> I64Vector4: ...
    @property
    def bllo(self) -> I64Vector4: ...
    @property
    def blll(self) -> I64Vector4: ...
    @property
    def arrr(self) -> I64Vector4: ...
    @property
    def arrg(self) -> I64Vector4: ...
    @property
    def arrb(self) -> I64Vector4: ...
    @property
    def arra(self) -> I64Vector4: ...
    @property
    def arro(self) -> I64Vector4: ...
    @property
    def arrl(self) -> I64Vector4: ...
    @property
    def argr(self) -> I64Vector4: ...
    @property
    def argg(self) -> I64Vector4: ...
    @property
    def argb(self) -> I64Vector4: ...
    @property
    def arga(self) -> I64Vector4: ...
    @property
    def argo(self) -> I64Vector4: ...
    @property
    def argl(self) -> I64Vector4: ...
    @property
    def arbr(self) -> I64Vector4: ...
    @property
    def arbg(self) -> I64Vector4: ...
    @property
    def arbb(self) -> I64Vector4: ...
    @property
    def arba(self) -> I64Vector4: ...
    @property
    def arbo(self) -> I64Vector4: ...
    @property
    def arbl(self) -> I64Vector4: ...
    @property
    def arar(self) -> I64Vector4: ...
    @property
    def arag(self) -> I64Vector4: ...
    @property
    def arab(self) -> I64Vector4: ...
    @property
    def araa(self) -> I64Vector4: ...
    @property
    def arao(self) -> I64Vector4: ...
    @property
    def aral(self) -> I64Vector4: ...
    @property
    def aror(self) -> I64Vector4: ...
    @property
    def arog(self) -> I64Vector4: ...
    @property
    def arob(self) -> I64Vector4: ...
    @property
    def aroa(self) -> I64Vector4: ...
    @property
    def aroo(self) -> I64Vector4: ...
    @property
    def arol(self) -> I64Vector4: ...
    @property
    def arlr(self) -> I64Vector4: ...
    @property
    def arlg(self) -> I64Vector4: ...
    @property
    def arlb(self) -> I64Vector4: ...
    @property
    def arla(self) -> I64Vector4: ...
    @property
    def arlo(self) -> I64Vector4: ...
    @property
    def arll(self) -> I64Vector4: ...
    @property
    def agrr(self) -> I64Vector4: ...
    @property
    def agrg(self) -> I64Vector4: ...
    @property
    def agrb(self) -> I64Vector4: ...
    @property
    def agra(self) -> I64Vector4: ...
    @property
    def agro(self) -> I64Vector4: ...
    @property
    def agrl(self) -> I64Vector4: ...
    @property
    def aggr(self) -> I64Vector4: ...
    @property
    def aggg(self) -> I64Vector4: ...
    @property
    def aggb(self) -> I64Vector4: ...
    @property
    def agga(self) -> I64Vector4: ...
    @property
    def aggo(self) -> I64Vector4: ...
    @property
    def aggl(self) -> I64Vector4: ...
    @property
    def agbr(self) -> I64Vector4: ...
    @property
    def agbg(self) -> I64Vector4: ...
    @property
    def agbb(self) -> I64Vector4: ...
    @property
    def agba(self) -> I64Vector4: ...
    @property
    def agbo(self) -> I64Vector4: ...
    @property
    def agbl(self) -> I64Vector4: ...
    @property
    def agar(self) -> I64Vector4: ...
    @property
    def agag(self) -> I64Vector4: ...
    @property
    def agab(self) -> I64Vector4: ...
    @property
    def agaa(self) -> I64Vector4: ...
    @property
    def agao(self) -> I64Vector4: ...
    @property
    def agal(self) -> I64Vector4: ...
    @property
    def agor(self) -> I64Vector4: ...
    @property
    def agog(self) -> I64Vector4: ...
    @property
    def agob(self) -> I64Vector4: ...
    @property
    def agoa(self) -> I64Vector4: ...
    @property
    def agoo(self) -> I64Vector4: ...
    @property
    def agol(self) -> I64Vector4: ...
    @property
    def aglr(self) -> I64Vector4: ...
    @property
    def aglg(self) -> I64Vector4: ...
    @property
    def aglb(self) -> I64Vector4: ...
    @property
    def agla(self) -> I64Vector4: ...
    @property
    def aglo(self) -> I64Vector4: ...
    @property
    def agll(self) -> I64Vector4: ...
    @property
    def abrr(self) -> I64Vector4: ...
    @property
    def abrg(self) -> I64Vector4: ...
    @property
    def abrb(self) -> I64Vector4: ...
    @property
    def abra(self) -> I64Vector4: ...
    @property
    def abro(self) -> I64Vector4: ...
    @property
    def abrl(self) -> I64Vector4: ...
    @property
    def abgr(self) -> I64Vector4: ...
    @property
    def abgg(self) -> I64Vector4: ...
    @property
    def abgb(self) -> I64Vector4: ...
    @property
    def abga(self) -> I64Vector4: ...
    @property
    def abgo(self) -> I64Vector4: ...
    @property
    def abgl(self) -> I64Vector4: ...
    @property
    def abbr(self) -> I64Vector4: ...
    @property
    def abbg(self) -> I64Vector4: ...
    @property
    def abbb(self) -> I64Vector4: ...
    @property
    def abba(self) -> I64Vector4: ...
    @property
    def abbo(self) -> I64Vector4: ...
    @property
    def abbl(self) -> I64Vector4: ...
    @property
    def abar(self) -> I64Vector4: ...
    @property
    def abag(self) -> I64Vector4: ...
    @property
    def abab(self) -> I64Vector4: ...
    @property
    def abaa(self) -> I64Vector4: ...
    @property
    def abao(self) -> I64Vector4: ...
    @property
    def abal(self) -> I64Vector4: ...
    @property
    def abor(self) -> I64Vector4: ...
    @property
    def abog(self) -> I64Vector4: ...
    @property
    def abob(self) -> I64Vector4: ...
    @property
    def aboa(self) -> I64Vector4: ...
    @property
    def aboo(self) -> I64Vector4: ...
    @property
    def abol(self) -> I64Vector4: ...
    @property
    def ablr(self) -> I64Vector4: ...
    @property
    def ablg(self) -> I64Vector4: ...
    @property
    def ablb(self) -> I64Vector4: ...
    @property
    def abla(self) -> I64Vector4: ...
    @property
    def ablo(self) -> I64Vector4: ...
    @property
    def abll(self) -> I64Vector4: ...
    @property
    def aarr(self) -> I64Vector4: ...
    @property
    def aarg(self) -> I64Vector4: ...
    @property
    def aarb(self) -> I64Vector4: ...
    @property
    def aara(self) -> I64Vector4: ...
    @property
    def aaro(self) -> I64Vector4: ...
    @property
    def aarl(self) -> I64Vector4: ...
    @property
    def aagr(self) -> I64Vector4: ...
    @property
    def aagg(self) -> I64Vector4: ...
    @property
    def aagb(self) -> I64Vector4: ...
    @property
    def aaga(self) -> I64Vector4: ...
    @property
    def aago(self) -> I64Vector4: ...
    @property
    def aagl(self) -> I64Vector4: ...
    @property
    def aabr(self) -> I64Vector4: ...
    @property
    def aabg(self) -> I64Vector4: ...
    @property
    def aabb(self) -> I64Vector4: ...
    @property
    def aaba(self) -> I64Vector4: ...
    @property
    def aabo(self) -> I64Vector4: ...
    @property
    def aabl(self) -> I64Vector4: ...
    @property
    def aaar(self) -> I64Vector4: ...
    @property
    def aaag(self) -> I64Vector4: ...
    @property
    def aaab(self) -> I64Vector4: ...
    @property
    def aaaa(self) -> I64Vector4: ...
    @property
    def aaao(self) -> I64Vector4: ...
    @property
    def aaal(self) -> I64Vector4: ...
    @property
    def aaor(self) -> I64Vector4: ...
    @property
    def aaog(self) -> I64Vector4: ...
    @property
    def aaob(self) -> I64Vector4: ...
    @property
    def aaoa(self) -> I64Vector4: ...
    @property
    def aaoo(self) -> I64Vector4: ...
    @property
    def aaol(self) -> I64Vector4: ...
    @property
    def aalr(self) -> I64Vector4: ...
    @property
    def aalg(self) -> I64Vector4: ...
    @property
    def aalb(self) -> I64Vector4: ...
    @property
    def aala(self) -> I64Vector4: ...
    @property
    def aalo(self) -> I64Vector4: ...
    @property
    def aall(self) -> I64Vector4: ...
    @property
    def aorr(self) -> I64Vector4: ...
    @property
    def aorg(self) -> I64Vector4: ...
    @property
    def aorb(self) -> I64Vector4: ...
    @property
    def aora(self) -> I64Vector4: ...
    @property
    def aoro(self) -> I64Vector4: ...
    @property
    def aorl(self) -> I64Vector4: ...
    @property
    def aogr(self) -> I64Vector4: ...
    @property
    def aogg(self) -> I64Vector4: ...
    @property
    def aogb(self) -> I64Vector4: ...
    @property
    def aoga(self) -> I64Vector4: ...
    @property
    def aogo(self) -> I64Vector4: ...
    @property
    def aogl(self) -> I64Vector4: ...
    @property
    def aobr(self) -> I64Vector4: ...
    @property
    def aobg(self) -> I64Vector4: ...
    @property
    def aobb(self) -> I64Vector4: ...
    @property
    def aoba(self) -> I64Vector4: ...
    @property
    def aobo(self) -> I64Vector4: ...
    @property
    def aobl(self) -> I64Vector4: ...
    @property
    def aoar(self) -> I64Vector4: ...
    @property
    def aoag(self) -> I64Vector4: ...
    @property
    def aoab(self) -> I64Vector4: ...
    @property
    def aoaa(self) -> I64Vector4: ...
    @property
    def aoao(self) -> I64Vector4: ...
    @property
    def aoal(self) -> I64Vector4: ...
    @property
    def aoor(self) -> I64Vector4: ...
    @property
    def aoog(self) -> I64Vector4: ...
    @property
    def aoob(self) -> I64Vector4: ...
    @property
    def aooa(self) -> I64Vector4: ...
    @property
    def aooo(self) -> I64Vector4: ...
    @property
    def aool(self) -> I64Vector4: ...
    @property
    def aolr(self) -> I64Vector4: ...
    @property
    def aolg(self) -> I64Vector4: ...
    @property
    def aolb(self) -> I64Vector4: ...
    @property
    def aola(self) -> I64Vector4: ...
    @property
    def aolo(self) -> I64Vector4: ...
    @property
    def aoll(self) -> I64Vector4: ...
    @property
    def alrr(self) -> I64Vector4: ...
    @property
    def alrg(self) -> I64Vector4: ...
    @property
    def alrb(self) -> I64Vector4: ...
    @property
    def alra(self) -> I64Vector4: ...
    @property
    def alro(self) -> I64Vector4: ...
    @property
    def alrl(self) -> I64Vector4: ...
    @property
    def algr(self) -> I64Vector4: ...
    @property
    def algg(self) -> I64Vector4: ...
    @property
    def algb(self) -> I64Vector4: ...
    @property
    def alga(self) -> I64Vector4: ...
    @property
    def algo(self) -> I64Vector4: ...
    @property
    def algl(self) -> I64Vector4: ...
    @property
    def albr(self) -> I64Vector4: ...
    @property
    def albg(self) -> I64Vector4: ...
    @property
    def albb(self) -> I64Vector4: ...
    @property
    def alba(self) -> I64Vector4: ...
    @property
    def albo(self) -> I64Vector4: ...
    @property
    def albl(self) -> I64Vector4: ...
    @property
    def alar(self) -> I64Vector4: ...
    @property
    def alag(self) -> I64Vector4: ...
    @property
    def alab(self) -> I64Vector4: ...
    @property
    def alaa(self) -> I64Vector4: ...
    @property
    def alao(self) -> I64Vector4: ...
    @property
    def alal(self) -> I64Vector4: ...
    @property
    def alor(self) -> I64Vector4: ...
    @property
    def alog(self) -> I64Vector4: ...
    @property
    def alob(self) -> I64Vector4: ...
    @property
    def aloa(self) -> I64Vector4: ...
    @property
    def aloo(self) -> I64Vector4: ...
    @property
    def alol(self) -> I64Vector4: ...
    @property
    def allr(self) -> I64Vector4: ...
    @property
    def allg(self) -> I64Vector4: ...
    @property
    def allb(self) -> I64Vector4: ...
    @property
    def alla(self) -> I64Vector4: ...
    @property
    def allo(self) -> I64Vector4: ...
    @property
    def alll(self) -> I64Vector4: ...
    @property
    def orrr(self) -> I64Vector4: ...
    @property
    def orrg(self) -> I64Vector4: ...
    @property
    def orrb(self) -> I64Vector4: ...
    @property
    def orra(self) -> I64Vector4: ...
    @property
    def orro(self) -> I64Vector4: ...
    @property
    def orrl(self) -> I64Vector4: ...
    @property
    def orgr(self) -> I64Vector4: ...
    @property
    def orgg(self) -> I64Vector4: ...
    @property
    def orgb(self) -> I64Vector4: ...
    @property
    def orga(self) -> I64Vector4: ...
    @property
    def orgo(self) -> I64Vector4: ...
    @property
    def orgl(self) -> I64Vector4: ...
    @property
    def orbr(self) -> I64Vector4: ...
    @property
    def orbg(self) -> I64Vector4: ...
    @property
    def orbb(self) -> I64Vector4: ...
    @property
    def orba(self) -> I64Vector4: ...
    @property
    def orbo(self) -> I64Vector4: ...
    @property
    def orbl(self) -> I64Vector4: ...
    @property
    def orar(self) -> I64Vector4: ...
    @property
    def orag(self) -> I64Vector4: ...
    @property
    def orab(self) -> I64Vector4: ...
    @property
    def oraa(self) -> I64Vector4: ...
    @property
    def orao(self) -> I64Vector4: ...
    @property
    def oral(self) -> I64Vector4: ...
    @property
    def oror(self) -> I64Vector4: ...
    @property
    def orog(self) -> I64Vector4: ...
    @property
    def orob(self) -> I64Vector4: ...
    @property
    def oroa(self) -> I64Vector4: ...
    @property
    def oroo(self) -> I64Vector4: ...
    @property
    def orol(self) -> I64Vector4: ...
    @property
    def orlr(self) -> I64Vector4: ...
    @property
    def orlg(self) -> I64Vector4: ...
    @property
    def orlb(self) -> I64Vector4: ...
    @property
    def orla(self) -> I64Vector4: ...
    @property
    def orlo(self) -> I64Vector4: ...
    @property
    def orll(self) -> I64Vector4: ...
    @property
    def ogrr(self) -> I64Vector4: ...
    @property
    def ogrg(self) -> I64Vector4: ...
    @property
    def ogrb(self) -> I64Vector4: ...
    @property
    def ogra(self) -> I64Vector4: ...
    @property
    def ogro(self) -> I64Vector4: ...
    @property
    def ogrl(self) -> I64Vector4: ...
    @property
    def oggr(self) -> I64Vector4: ...
    @property
    def oggg(self) -> I64Vector4: ...
    @property
    def oggb(self) -> I64Vector4: ...
    @property
    def ogga(self) -> I64Vector4: ...
    @property
    def oggo(self) -> I64Vector4: ...
    @property
    def oggl(self) -> I64Vector4: ...
    @property
    def ogbr(self) -> I64Vector4: ...
    @property
    def ogbg(self) -> I64Vector4: ...
    @property
    def ogbb(self) -> I64Vector4: ...
    @property
    def ogba(self) -> I64Vector4: ...
    @property
    def ogbo(self) -> I64Vector4: ...
    @property
    def ogbl(self) -> I64Vector4: ...
    @property
    def ogar(self) -> I64Vector4: ...
    @property
    def ogag(self) -> I64Vector4: ...
    @property
    def ogab(self) -> I64Vector4: ...
    @property
    def ogaa(self) -> I64Vector4: ...
    @property
    def ogao(self) -> I64Vector4: ...
    @property
    def ogal(self) -> I64Vector4: ...
    @property
    def ogor(self) -> I64Vector4: ...
    @property
    def ogog(self) -> I64Vector4: ...
    @property
    def ogob(self) -> I64Vector4: ...
    @property
    def ogoa(self) -> I64Vector4: ...
    @property
    def ogoo(self) -> I64Vector4: ...
    @property
    def ogol(self) -> I64Vector4: ...
    @property
    def oglr(self) -> I64Vector4: ...
    @property
    def oglg(self) -> I64Vector4: ...
    @property
    def oglb(self) -> I64Vector4: ...
    @property
    def ogla(self) -> I64Vector4: ...
    @property
    def oglo(self) -> I64Vector4: ...
    @property
    def ogll(self) -> I64Vector4: ...
    @property
    def obrr(self) -> I64Vector4: ...
    @property
    def obrg(self) -> I64Vector4: ...
    @property
    def obrb(self) -> I64Vector4: ...
    @property
    def obra(self) -> I64Vector4: ...
    @property
    def obro(self) -> I64Vector4: ...
    @property
    def obrl(self) -> I64Vector4: ...
    @property
    def obgr(self) -> I64Vector4: ...
    @property
    def obgg(self) -> I64Vector4: ...
    @property
    def obgb(self) -> I64Vector4: ...
    @property
    def obga(self) -> I64Vector4: ...
    @property
    def obgo(self) -> I64Vector4: ...
    @property
    def obgl(self) -> I64Vector4: ...
    @property
    def obbr(self) -> I64Vector4: ...
    @property
    def obbg(self) -> I64Vector4: ...
    @property
    def obbb(self) -> I64Vector4: ...
    @property
    def obba(self) -> I64Vector4: ...
    @property
    def obbo(self) -> I64Vector4: ...
    @property
    def obbl(self) -> I64Vector4: ...
    @property
    def obar(self) -> I64Vector4: ...
    @property
    def obag(self) -> I64Vector4: ...
    @property
    def obab(self) -> I64Vector4: ...
    @property
    def obaa(self) -> I64Vector4: ...
    @property
    def obao(self) -> I64Vector4: ...
    @property
    def obal(self) -> I64Vector4: ...
    @property
    def obor(self) -> I64Vector4: ...
    @property
    def obog(self) -> I64Vector4: ...
    @property
    def obob(self) -> I64Vector4: ...
    @property
    def oboa(self) -> I64Vector4: ...
    @property
    def oboo(self) -> I64Vector4: ...
    @property
    def obol(self) -> I64Vector4: ...
    @property
    def oblr(self) -> I64Vector4: ...
    @property
    def oblg(self) -> I64Vector4: ...
    @property
    def oblb(self) -> I64Vector4: ...
    @property
    def obla(self) -> I64Vector4: ...
    @property
    def oblo(self) -> I64Vector4: ...
    @property
    def obll(self) -> I64Vector4: ...
    @property
    def oarr(self) -> I64Vector4: ...
    @property
    def oarg(self) -> I64Vector4: ...
    @property
    def oarb(self) -> I64Vector4: ...
    @property
    def oara(self) -> I64Vector4: ...
    @property
    def oaro(self) -> I64Vector4: ...
    @property
    def oarl(self) -> I64Vector4: ...
    @property
    def oagr(self) -> I64Vector4: ...
    @property
    def oagg(self) -> I64Vector4: ...
    @property
    def oagb(self) -> I64Vector4: ...
    @property
    def oaga(self) -> I64Vector4: ...
    @property
    def oago(self) -> I64Vector4: ...
    @property
    def oagl(self) -> I64Vector4: ...
    @property
    def oabr(self) -> I64Vector4: ...
    @property
    def oabg(self) -> I64Vector4: ...
    @property
    def oabb(self) -> I64Vector4: ...
    @property
    def oaba(self) -> I64Vector4: ...
    @property
    def oabo(self) -> I64Vector4: ...
    @property
    def oabl(self) -> I64Vector4: ...
    @property
    def oaar(self) -> I64Vector4: ...
    @property
    def oaag(self) -> I64Vector4: ...
    @property
    def oaab(self) -> I64Vector4: ...
    @property
    def oaaa(self) -> I64Vector4: ...
    @property
    def oaao(self) -> I64Vector4: ...
    @property
    def oaal(self) -> I64Vector4: ...
    @property
    def oaor(self) -> I64Vector4: ...
    @property
    def oaog(self) -> I64Vector4: ...
    @property
    def oaob(self) -> I64Vector4: ...
    @property
    def oaoa(self) -> I64Vector4: ...
    @property
    def oaoo(self) -> I64Vector4: ...
    @property
    def oaol(self) -> I64Vector4: ...
    @property
    def oalr(self) -> I64Vector4: ...
    @property
    def oalg(self) -> I64Vector4: ...
    @property
    def oalb(self) -> I64Vector4: ...
    @property
    def oala(self) -> I64Vector4: ...
    @property
    def oalo(self) -> I64Vector4: ...
    @property
    def oall(self) -> I64Vector4: ...
    @property
    def oorr(self) -> I64Vector4: ...
    @property
    def oorg(self) -> I64Vector4: ...
    @property
    def oorb(self) -> I64Vector4: ...
    @property
    def oora(self) -> I64Vector4: ...
    @property
    def ooro(self) -> I64Vector4: ...
    @property
    def oorl(self) -> I64Vector4: ...
    @property
    def oogr(self) -> I64Vector4: ...
    @property
    def oogg(self) -> I64Vector4: ...
    @property
    def oogb(self) -> I64Vector4: ...
    @property
    def ooga(self) -> I64Vector4: ...
    @property
    def oogo(self) -> I64Vector4: ...
    @property
    def oogl(self) -> I64Vector4: ...
    @property
    def oobr(self) -> I64Vector4: ...
    @property
    def oobg(self) -> I64Vector4: ...
    @property
    def oobb(self) -> I64Vector4: ...
    @property
    def ooba(self) -> I64Vector4: ...
    @property
    def oobo(self) -> I64Vector4: ...
    @property
    def oobl(self) -> I64Vector4: ...
    @property
    def ooar(self) -> I64Vector4: ...
    @property
    def ooag(self) -> I64Vector4: ...
    @property
    def ooab(self) -> I64Vector4: ...
    @property
    def ooaa(self) -> I64Vector4: ...
    @property
    def ooao(self) -> I64Vector4: ...
    @property
    def ooal(self) -> I64Vector4: ...
    @property
    def ooor(self) -> I64Vector4: ...
    @property
    def ooog(self) -> I64Vector4: ...
    @property
    def ooob(self) -> I64Vector4: ...
    @property
    def oooa(self) -> I64Vector4: ...
    @property
    def oolr(self) -> I64Vector4: ...
    @property
    def oolg(self) -> I64Vector4: ...
    @property
    def oolb(self) -> I64Vector4: ...
    @property
    def oola(self) -> I64Vector4: ...
    @property
    def olrr(self) -> I64Vector4: ...
    @property
    def olrg(self) -> I64Vector4: ...
    @property
    def olrb(self) -> I64Vector4: ...
    @property
    def olra(self) -> I64Vector4: ...
    @property
    def olro(self) -> I64Vector4: ...
    @property
    def olrl(self) -> I64Vector4: ...
    @property
    def olgr(self) -> I64Vector4: ...
    @property
    def olgg(self) -> I64Vector4: ...
    @property
    def olgb(self) -> I64Vector4: ...
    @property
    def olga(self) -> I64Vector4: ...
    @property
    def olgo(self) -> I64Vector4: ...
    @property
    def olgl(self) -> I64Vector4: ...
    @property
    def olbr(self) -> I64Vector4: ...
    @property
    def olbg(self) -> I64Vector4: ...
    @property
    def olbb(self) -> I64Vector4: ...
    @property
    def olba(self) -> I64Vector4: ...
    @property
    def olbo(self) -> I64Vector4: ...
    @property
    def olbl(self) -> I64Vector4: ...
    @property
    def olar(self) -> I64Vector4: ...
    @property
    def olag(self) -> I64Vector4: ...
    @property
    def olab(self) -> I64Vector4: ...
    @property
    def olaa(self) -> I64Vector4: ...
    @property
    def olao(self) -> I64Vector4: ...
    @property
    def olal(self) -> I64Vector4: ...
    @property
    def olor(self) -> I64Vector4: ...
    @property
    def olog(self) -> I64Vector4: ...
    @property
    def olob(self) -> I64Vector4: ...
    @property
    def oloa(self) -> I64Vector4: ...
    @property
    def ollr(self) -> I64Vector4: ...
    @property
    def ollg(self) -> I64Vector4: ...
    @property
    def ollb(self) -> I64Vector4: ...
    @property
    def olla(self) -> I64Vector4: ...
    @property
    def lrrr(self) -> I64Vector4: ...
    @property
    def lrrg(self) -> I64Vector4: ...
    @property
    def lrrb(self) -> I64Vector4: ...
    @property
    def lrra(self) -> I64Vector4: ...
    @property
    def lrro(self) -> I64Vector4: ...
    @property
    def lrrl(self) -> I64Vector4: ...
    @property
    def lrgr(self) -> I64Vector4: ...
    @property
    def lrgg(self) -> I64Vector4: ...
    @property
    def lrgb(self) -> I64Vector4: ...
    @property
    def lrga(self) -> I64Vector4: ...
    @property
    def lrgo(self) -> I64Vector4: ...
    @property
    def lrgl(self) -> I64Vector4: ...
    @property
    def lrbr(self) -> I64Vector4: ...
    @property
    def lrbg(self) -> I64Vector4: ...
    @property
    def lrbb(self) -> I64Vector4: ...
    @property
    def lrba(self) -> I64Vector4: ...
    @property
    def lrbo(self) -> I64Vector4: ...
    @property
    def lrbl(self) -> I64Vector4: ...
    @property
    def lrar(self) -> I64Vector4: ...
    @property
    def lrag(self) -> I64Vector4: ...
    @property
    def lrab(self) -> I64Vector4: ...
    @property
    def lraa(self) -> I64Vector4: ...
    @property
    def lrao(self) -> I64Vector4: ...
    @property
    def lral(self) -> I64Vector4: ...
    @property
    def lror(self) -> I64Vector4: ...
    @property
    def lrog(self) -> I64Vector4: ...
    @property
    def lrob(self) -> I64Vector4: ...
    @property
    def lroa(self) -> I64Vector4: ...
    @property
    def lroo(self) -> I64Vector4: ...
    @property
    def lrol(self) -> I64Vector4: ...
    @property
    def lrlr(self) -> I64Vector4: ...
    @property
    def lrlg(self) -> I64Vector4: ...
    @property
    def lrlb(self) -> I64Vector4: ...
    @property
    def lrla(self) -> I64Vector4: ...
    @property
    def lrlo(self) -> I64Vector4: ...
    @property
    def lrll(self) -> I64Vector4: ...
    @property
    def lgrr(self) -> I64Vector4: ...
    @property
    def lgrg(self) -> I64Vector4: ...
    @property
    def lgrb(self) -> I64Vector4: ...
    @property
    def lgra(self) -> I64Vector4: ...
    @property
    def lgro(self) -> I64Vector4: ...
    @property
    def lgrl(self) -> I64Vector4: ...
    @property
    def lggr(self) -> I64Vector4: ...
    @property
    def lggg(self) -> I64Vector4: ...
    @property
    def lggb(self) -> I64Vector4: ...
    @property
    def lgga(self) -> I64Vector4: ...
    @property
    def lggo(self) -> I64Vector4: ...
    @property
    def lggl(self) -> I64Vector4: ...
    @property
    def lgbr(self) -> I64Vector4: ...
    @property
    def lgbg(self) -> I64Vector4: ...
    @property
    def lgbb(self) -> I64Vector4: ...
    @property
    def lgba(self) -> I64Vector4: ...
    @property
    def lgbo(self) -> I64Vector4: ...
    @property
    def lgbl(self) -> I64Vector4: ...
    @property
    def lgar(self) -> I64Vector4: ...
    @property
    def lgag(self) -> I64Vector4: ...
    @property
    def lgab(self) -> I64Vector4: ...
    @property
    def lgaa(self) -> I64Vector4: ...
    @property
    def lgao(self) -> I64Vector4: ...
    @property
    def lgal(self) -> I64Vector4: ...
    @property
    def lgor(self) -> I64Vector4: ...
    @property
    def lgog(self) -> I64Vector4: ...
    @property
    def lgob(self) -> I64Vector4: ...
    @property
    def lgoa(self) -> I64Vector4: ...
    @property
    def lgoo(self) -> I64Vector4: ...
    @property
    def lgol(self) -> I64Vector4: ...
    @property
    def lglr(self) -> I64Vector4: ...
    @property
    def lglg(self) -> I64Vector4: ...
    @property
    def lglb(self) -> I64Vector4: ...
    @property
    def lgla(self) -> I64Vector4: ...
    @property
    def lglo(self) -> I64Vector4: ...
    @property
    def lgll(self) -> I64Vector4: ...
    @property
    def lbrr(self) -> I64Vector4: ...
    @property
    def lbrg(self) -> I64Vector4: ...
    @property
    def lbrb(self) -> I64Vector4: ...
    @property
    def lbra(self) -> I64Vector4: ...
    @property
    def lbro(self) -> I64Vector4: ...
    @property
    def lbrl(self) -> I64Vector4: ...
    @property
    def lbgr(self) -> I64Vector4: ...
    @property
    def lbgg(self) -> I64Vector4: ...
    @property
    def lbgb(self) -> I64Vector4: ...
    @property
    def lbga(self) -> I64Vector4: ...
    @property
    def lbgo(self) -> I64Vector4: ...
    @property
    def lbgl(self) -> I64Vector4: ...
    @property
    def lbbr(self) -> I64Vector4: ...
    @property
    def lbbg(self) -> I64Vector4: ...
    @property
    def lbbb(self) -> I64Vector4: ...
    @property
    def lbba(self) -> I64Vector4: ...
    @property
    def lbbo(self) -> I64Vector4: ...
    @property
    def lbbl(self) -> I64Vector4: ...
    @property
    def lbar(self) -> I64Vector4: ...
    @property
    def lbag(self) -> I64Vector4: ...
    @property
    def lbab(self) -> I64Vector4: ...
    @property
    def lbaa(self) -> I64Vector4: ...
    @property
    def lbao(self) -> I64Vector4: ...
    @property
    def lbal(self) -> I64Vector4: ...
    @property
    def lbor(self) -> I64Vector4: ...
    @property
    def lbog(self) -> I64Vector4: ...
    @property
    def lbob(self) -> I64Vector4: ...
    @property
    def lboa(self) -> I64Vector4: ...
    @property
    def lboo(self) -> I64Vector4: ...
    @property
    def lbol(self) -> I64Vector4: ...
    @property
    def lblr(self) -> I64Vector4: ...
    @property
    def lblg(self) -> I64Vector4: ...
    @property
    def lblb(self) -> I64Vector4: ...
    @property
    def lbla(self) -> I64Vector4: ...
    @property
    def lblo(self) -> I64Vector4: ...
    @property
    def lbll(self) -> I64Vector4: ...
    @property
    def larr(self) -> I64Vector4: ...
    @property
    def larg(self) -> I64Vector4: ...
    @property
    def larb(self) -> I64Vector4: ...
    @property
    def lara(self) -> I64Vector4: ...
    @property
    def laro(self) -> I64Vector4: ...
    @property
    def larl(self) -> I64Vector4: ...
    @property
    def lagr(self) -> I64Vector4: ...
    @property
    def lagg(self) -> I64Vector4: ...
    @property
    def lagb(self) -> I64Vector4: ...
    @property
    def laga(self) -> I64Vector4: ...
    @property
    def lago(self) -> I64Vector4: ...
    @property
    def lagl(self) -> I64Vector4: ...
    @property
    def labr(self) -> I64Vector4: ...
    @property
    def labg(self) -> I64Vector4: ...
    @property
    def labb(self) -> I64Vector4: ...
    @property
    def laba(self) -> I64Vector4: ...
    @property
    def labo(self) -> I64Vector4: ...
    @property
    def labl(self) -> I64Vector4: ...
    @property
    def laar(self) -> I64Vector4: ...
    @property
    def laag(self) -> I64Vector4: ...
    @property
    def laab(self) -> I64Vector4: ...
    @property
    def laaa(self) -> I64Vector4: ...
    @property
    def laao(self) -> I64Vector4: ...
    @property
    def laal(self) -> I64Vector4: ...
    @property
    def laor(self) -> I64Vector4: ...
    @property
    def laog(self) -> I64Vector4: ...
    @property
    def laob(self) -> I64Vector4: ...
    @property
    def laoa(self) -> I64Vector4: ...
    @property
    def laoo(self) -> I64Vector4: ...
    @property
    def laol(self) -> I64Vector4: ...
    @property
    def lalr(self) -> I64Vector4: ...
    @property
    def lalg(self) -> I64Vector4: ...
    @property
    def lalb(self) -> I64Vector4: ...
    @property
    def lala(self) -> I64Vector4: ...
    @property
    def lalo(self) -> I64Vector4: ...
    @property
    def lall(self) -> I64Vector4: ...
    @property
    def lorr(self) -> I64Vector4: ...
    @property
    def lorg(self) -> I64Vector4: ...
    @property
    def lorb(self) -> I64Vector4: ...
    @property
    def lora(self) -> I64Vector4: ...
    @property
    def loro(self) -> I64Vector4: ...
    @property
    def lorl(self) -> I64Vector4: ...
    @property
    def logr(self) -> I64Vector4: ...
    @property
    def logg(self) -> I64Vector4: ...
    @property
    def logb(self) -> I64Vector4: ...
    @property
    def loga(self) -> I64Vector4: ...
    @property
    def logo(self) -> I64Vector4: ...
    @property
    def logl(self) -> I64Vector4: ...
    @property
    def lobr(self) -> I64Vector4: ...
    @property
    def lobg(self) -> I64Vector4: ...
    @property
    def lobb(self) -> I64Vector4: ...
    @property
    def loba(self) -> I64Vector4: ...
    @property
    def lobo(self) -> I64Vector4: ...
    @property
    def lobl(self) -> I64Vector4: ...
    @property
    def loar(self) -> I64Vector4: ...
    @property
    def loag(self) -> I64Vector4: ...
    @property
    def loab(self) -> I64Vector4: ...
    @property
    def loaa(self) -> I64Vector4: ...
    @property
    def loao(self) -> I64Vector4: ...
    @property
    def loal(self) -> I64Vector4: ...
    @property
    def loor(self) -> I64Vector4: ...
    @property
    def loog(self) -> I64Vector4: ...
    @property
    def loob(self) -> I64Vector4: ...
    @property
    def looa(self) -> I64Vector4: ...
    @property
    def lolr(self) -> I64Vector4: ...
    @property
    def lolg(self) -> I64Vector4: ...
    @property
    def lolb(self) -> I64Vector4: ...
    @property
    def lola(self) -> I64Vector4: ...
    @property
    def llrr(self) -> I64Vector4: ...
    @property
    def llrg(self) -> I64Vector4: ...
    @property
    def llrb(self) -> I64Vector4: ...
    @property
    def llra(self) -> I64Vector4: ...
    @property
    def llro(self) -> I64Vector4: ...
    @property
    def llrl(self) -> I64Vector4: ...
    @property
    def llgr(self) -> I64Vector4: ...
    @property
    def llgg(self) -> I64Vector4: ...
    @property
    def llgb(self) -> I64Vector4: ...
    @property
    def llga(self) -> I64Vector4: ...
    @property
    def llgo(self) -> I64Vector4: ...
    @property
    def llgl(self) -> I64Vector4: ...
    @property
    def llbr(self) -> I64Vector4: ...
    @property
    def llbg(self) -> I64Vector4: ...
    @property
    def llbb(self) -> I64Vector4: ...
    @property
    def llba(self) -> I64Vector4: ...
    @property
    def llbo(self) -> I64Vector4: ...
    @property
    def llbl(self) -> I64Vector4: ...
    @property
    def llar(self) -> I64Vector4: ...
    @property
    def llag(self) -> I64Vector4: ...
    @property
    def llab(self) -> I64Vector4: ...
    @property
    def llaa(self) -> I64Vector4: ...
    @property
    def llao(self) -> I64Vector4: ...
    @property
    def llal(self) -> I64Vector4: ...
    @property
    def llor(self) -> I64Vector4: ...
    @property
    def llog(self) -> I64Vector4: ...
    @property
    def llob(self) -> I64Vector4: ...
    @property
    def lloa(self) -> I64Vector4: ...
    @property
    def lllr(self) -> I64Vector4: ...
    @property
    def lllg(self) -> I64Vector4: ...
    @property
    def lllb(self) -> I64Vector4: ...
    @property
    def llla(self) -> I64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> I64Vector2: ...
    @property
    def st(self) -> I64Vector2: ...
    @property
    def sq(self) -> I64Vector2: ...
    @property
    def sp(self) -> I64Vector2: ...
    @property
    def so(self) -> I64Vector2: ...
    @property
    def sl(self) -> I64Vector2: ...
    @property
    def ts(self) -> I64Vector2: ...
    @property
    def tt(self) -> I64Vector2: ...
    @property
    def tq(self) -> I64Vector2: ...
    @property
    def tp(self) -> I64Vector2: ...
    @property
    def to(self) -> I64Vector2: ...
    @property
    def tl(self) -> I64Vector2: ...
    @property
    def qs(self) -> I64Vector2: ...
    @property
    def qt(self) -> I64Vector2: ...
    @property
    def qq(self) -> I64Vector2: ...
    @property
    def qp(self) -> I64Vector2: ...
    @property
    def qo(self) -> I64Vector2: ...
    @property
    def ql(self) -> I64Vector2: ...
    @property
    def ps(self) -> I64Vector2: ...
    @property
    def pt(self) -> I64Vector2: ...
    @property
    def pq(self) -> I64Vector2: ...
    @property
    def pp(self) -> I64Vector2: ...
    @property
    def po(self) -> I64Vector2: ...
    @property
    def pl(self) -> I64Vector2: ...
    @property
    def os(self) -> I64Vector2: ...
    @property
    def ot(self) -> I64Vector2: ...
    @property
    def oq(self) -> I64Vector2: ...
    @property
    def op(self) -> I64Vector2: ...
    @property
    def ls(self) -> I64Vector2: ...
    @property
    def lt(self) -> I64Vector2: ...
    @property
    def lq(self) -> I64Vector2: ...
    @property
    def lp(self) -> I64Vector2: ...
    @property
    def sss(self) -> I64Vector3: ...
    @property
    def sst(self) -> I64Vector3: ...
    @property
    def ssq(self) -> I64Vector3: ...
    @property
    def ssp(self) -> I64Vector3: ...
    @property
    def sso(self) -> I64Vector3: ...
    @property
    def ssl(self) -> I64Vector3: ...
    @property
    def sts(self) -> I64Vector3: ...
    @property
    def stt(self) -> I64Vector3: ...
    @property
    def stq(self) -> I64Vector3: ...
    @property
    def stp(self) -> I64Vector3: ...
    @property
    def sto(self) -> I64Vector3: ...
    @property
    def stl(self) -> I64Vector3: ...
    @property
    def sqs(self) -> I64Vector3: ...
    @property
    def sqt(self) -> I64Vector3: ...
    @property
    def sqq(self) -> I64Vector3: ...
    @property
    def sqp(self) -> I64Vector3: ...
    @property
    def sqo(self) -> I64Vector3: ...
    @property
    def sql(self) -> I64Vector3: ...
    @property
    def sps(self) -> I64Vector3: ...
    @property
    def spt(self) -> I64Vector3: ...
    @property
    def spq(self) -> I64Vector3: ...
    @property
    def spp(self) -> I64Vector3: ...
    @property
    def spo(self) -> I64Vector3: ...
    @property
    def spl(self) -> I64Vector3: ...
    @property
    def sos(self) -> I64Vector3: ...
    @property
    def sot(self) -> I64Vector3: ...
    @property
    def soq(self) -> I64Vector3: ...
    @property
    def sop(self) -> I64Vector3: ...
    @property
    def soo(self) -> I64Vector3: ...
    @property
    def sol(self) -> I64Vector3: ...
    @property
    def sls(self) -> I64Vector3: ...
    @property
    def slt(self) -> I64Vector3: ...
    @property
    def slq(self) -> I64Vector3: ...
    @property
    def slp(self) -> I64Vector3: ...
    @property
    def slo(self) -> I64Vector3: ...
    @property
    def sll(self) -> I64Vector3: ...
    @property
    def tss(self) -> I64Vector3: ...
    @property
    def tst(self) -> I64Vector3: ...
    @property
    def tsq(self) -> I64Vector3: ...
    @property
    def tsp(self) -> I64Vector3: ...
    @property
    def tso(self) -> I64Vector3: ...
    @property
    def tsl(self) -> I64Vector3: ...
    @property
    def tts(self) -> I64Vector3: ...
    @property
    def ttt(self) -> I64Vector3: ...
    @property
    def ttq(self) -> I64Vector3: ...
    @property
    def ttp(self) -> I64Vector3: ...
    @property
    def tto(self) -> I64Vector3: ...
    @property
    def ttl(self) -> I64Vector3: ...
    @property
    def tqs(self) -> I64Vector3: ...
    @property
    def tqt(self) -> I64Vector3: ...
    @property
    def tqq(self) -> I64Vector3: ...
    @property
    def tqp(self) -> I64Vector3: ...
    @property
    def tqo(self) -> I64Vector3: ...
    @property
    def tql(self) -> I64Vector3: ...
    @property
    def tps(self) -> I64Vector3: ...
    @property
    def tpt(self) -> I64Vector3: ...
    @property
    def tpq(self) -> I64Vector3: ...
    @property
    def tpp(self) -> I64Vector3: ...
    @property
    def tpo(self) -> I64Vector3: ...
    @property
    def tpl(self) -> I64Vector3: ...
    @property
    def tos(self) -> I64Vector3: ...
    @property
    def tot(self) -> I64Vector3: ...
    @property
    def toq(self) -> I64Vector3: ...
    @property
    def top(self) -> I64Vector3: ...
    @property
    def too(self) -> I64Vector3: ...
    @property
    def tol(self) -> I64Vector3: ...
    @property
    def tls(self) -> I64Vector3: ...
    @property
    def tlt(self) -> I64Vector3: ...
    @property
    def tlq(self) -> I64Vector3: ...
    @property
    def tlp(self) -> I64Vector3: ...
    @property
    def tlo(self) -> I64Vector3: ...
    @property
    def tll(self) -> I64Vector3: ...
    @property
    def qss(self) -> I64Vector3: ...
    @property
    def qst(self) -> I64Vector3: ...
    @property
    def qsq(self) -> I64Vector3: ...
    @property
    def qsp(self) -> I64Vector3: ...
    @property
    def qso(self) -> I64Vector3: ...
    @property
    def qsl(self) -> I64Vector3: ...
    @property
    def qts(self) -> I64Vector3: ...
    @property
    def qtt(self) -> I64Vector3: ...
    @property
    def qtq(self) -> I64Vector3: ...
    @property
    def qtp(self) -> I64Vector3: ...
    @property
    def qto(self) -> I64Vector3: ...
    @property
    def qtl(self) -> I64Vector3: ...
    @property
    def qqs(self) -> I64Vector3: ...
    @property
    def qqt(self) -> I64Vector3: ...
    @property
    def qqq(self) -> I64Vector3: ...
    @property
    def qqp(self) -> I64Vector3: ...
    @property
    def qqo(self) -> I64Vector3: ...
    @property
    def qql(self) -> I64Vector3: ...
    @property
    def qps(self) -> I64Vector3: ...
    @property
    def qpt(self) -> I64Vector3: ...
    @property
    def qpq(self) -> I64Vector3: ...
    @property
    def qpp(self) -> I64Vector3: ...
    @property
    def qpo(self) -> I64Vector3: ...
    @property
    def qpl(self) -> I64Vector3: ...
    @property
    def qos(self) -> I64Vector3: ...
    @property
    def qot(self) -> I64Vector3: ...
    @property
    def qoq(self) -> I64Vector3: ...
    @property
    def qop(self) -> I64Vector3: ...
    @property
    def qoo(self) -> I64Vector3: ...
    @property
    def qol(self) -> I64Vector3: ...
    @property
    def qls(self) -> I64Vector3: ...
    @property
    def qlt(self) -> I64Vector3: ...
    @property
    def qlq(self) -> I64Vector3: ...
    @property
    def qlp(self) -> I64Vector3: ...
    @property
    def qlo(self) -> I64Vector3: ...
    @property
    def qll(self) -> I64Vector3: ...
    @property
    def pss(self) -> I64Vector3: ...
    @property
    def pst(self) -> I64Vector3: ...
    @property
    def psq(self) -> I64Vector3: ...
    @property
    def psp(self) -> I64Vector3: ...
    @property
    def pso(self) -> I64Vector3: ...
    @property
    def psl(self) -> I64Vector3: ...
    @property
    def pts(self) -> I64Vector3: ...
    @property
    def ptt(self) -> I64Vector3: ...
    @property
    def ptq(self) -> I64Vector3: ...
    @property
    def ptp(self) -> I64Vector3: ...
    @property
    def pto(self) -> I64Vector3: ...
    @property
    def ptl(self) -> I64Vector3: ...
    @property
    def pqs(self) -> I64Vector3: ...
    @property
    def pqt(self) -> I64Vector3: ...
    @property
    def pqq(self) -> I64Vector3: ...
    @property
    def pqp(self) -> I64Vector3: ...
    @property
    def pqo(self) -> I64Vector3: ...
    @property
    def pql(self) -> I64Vector3: ...
    @property
    def pps(self) -> I64Vector3: ...
    @property
    def ppt(self) -> I64Vector3: ...
    @property
    def ppq(self) -> I64Vector3: ...
    @property
    def ppp(self) -> I64Vector3: ...
    @property
    def ppo(self) -> I64Vector3: ...
    @property
    def ppl(self) -> I64Vector3: ...
    @property
    def pos(self) -> I64Vector3: ...
    @property
    def pot(self) -> I64Vector3: ...
    @property
    def poq(self) -> I64Vector3: ...
    @property
    def pop(self) -> I64Vector3: ...
    @property
    def poo(self) -> I64Vector3: ...
    @property
    def pol(self) -> I64Vector3: ...
    @property
    def pls(self) -> I64Vector3: ...
    @property
    def plt(self) -> I64Vector3: ...
    @property
    def plq(self) -> I64Vector3: ...
    @property
    def plp(self) -> I64Vector3: ...
    @property
    def plo(self) -> I64Vector3: ...
    @property
    def pll(self) -> I64Vector3: ...
    @property
    def oss(self) -> I64Vector3: ...
    @property
    def ost(self) -> I64Vector3: ...
    @property
    def osq(self) -> I64Vector3: ...
    @property
    def osp(self) -> I64Vector3: ...
    @property
    def oso(self) -> I64Vector3: ...
    @property
    def osl(self) -> I64Vector3: ...
    @property
    def ots(self) -> I64Vector3: ...
    @property
    def ott(self) -> I64Vector3: ...
    @property
    def otq(self) -> I64Vector3: ...
    @property
    def otp(self) -> I64Vector3: ...
    @property
    def oto(self) -> I64Vector3: ...
    @property
    def otl(self) -> I64Vector3: ...
    @property
    def oqs(self) -> I64Vector3: ...
    @property
    def oqt(self) -> I64Vector3: ...
    @property
    def oqq(self) -> I64Vector3: ...
    @property
    def oqp(self) -> I64Vector3: ...
    @property
    def oqo(self) -> I64Vector3: ...
    @property
    def oql(self) -> I64Vector3: ...
    @property
    def ops(self) -> I64Vector3: ...
    @property
    def opt(self) -> I64Vector3: ...
    @property
    def opq(self) -> I64Vector3: ...
    @property
    def opp(self) -> I64Vector3: ...
    @property
    def opo(self) -> I64Vector3: ...
    @property
    def opl(self) -> I64Vector3: ...
    @property
    def oos(self) -> I64Vector3: ...
    @property
    def oot(self) -> I64Vector3: ...
    @property
    def ooq(self) -> I64Vector3: ...
    @property
    def oop(self) -> I64Vector3: ...
    @property
    def ols(self) -> I64Vector3: ...
    @property
    def olt(self) -> I64Vector3: ...
    @property
    def olq(self) -> I64Vector3: ...
    @property
    def olp(self) -> I64Vector3: ...
    @property
    def lss(self) -> I64Vector3: ...
    @property
    def lst(self) -> I64Vector3: ...
    @property
    def lsq(self) -> I64Vector3: ...
    @property
    def lsp(self) -> I64Vector3: ...
    @property
    def lso(self) -> I64Vector3: ...
    @property
    def lsl(self) -> I64Vector3: ...
    @property
    def lts(self) -> I64Vector3: ...
    @property
    def ltt(self) -> I64Vector3: ...
    @property
    def ltq(self) -> I64Vector3: ...
    @property
    def ltp(self) -> I64Vector3: ...
    @property
    def lto(self) -> I64Vector3: ...
    @property
    def ltl(self) -> I64Vector3: ...
    @property
    def lqs(self) -> I64Vector3: ...
    @property
    def lqt(self) -> I64Vector3: ...
    @property
    def lqq(self) -> I64Vector3: ...
    @property
    def lqp(self) -> I64Vector3: ...
    @property
    def lqo(self) -> I64Vector3: ...
    @property
    def lql(self) -> I64Vector3: ...
    @property
    def lps(self) -> I64Vector3: ...
    @property
    def lpt(self) -> I64Vector3: ...
    @property
    def lpq(self) -> I64Vector3: ...
    @property
    def lpp(self) -> I64Vector3: ...
    @property
    def lpo(self) -> I64Vector3: ...
    @property
    def lpl(self) -> I64Vector3: ...
    @property
    def los(self) -> I64Vector3: ...
    @property
    def lot(self) -> I64Vector3: ...
    @property
    def loq(self) -> I64Vector3: ...
    @property
    def lop(self) -> I64Vector3: ...
    @property
    def lls(self) -> I64Vector3: ...
    @property
    def llt(self) -> I64Vector3: ...
    @property
    def llq(self) -> I64Vector3: ...
    @property
    def llp(self) -> I64Vector3: ...
    @property
    def ssss(self) -> I64Vector4: ...
    @property
    def ssst(self) -> I64Vector4: ...
    @property
    def sssq(self) -> I64Vector4: ...
    @property
    def sssp(self) -> I64Vector4: ...
    @property
    def ssso(self) -> I64Vector4: ...
    @property
    def sssl(self) -> I64Vector4: ...
    @property
    def ssts(self) -> I64Vector4: ...
    @property
    def sstt(self) -> I64Vector4: ...
    @property
    def sstq(self) -> I64Vector4: ...
    @property
    def sstp(self) -> I64Vector4: ...
    @property
    def ssto(self) -> I64Vector4: ...
    @property
    def sstl(self) -> I64Vector4: ...
    @property
    def ssqs(self) -> I64Vector4: ...
    @property
    def ssqt(self) -> I64Vector4: ...
    @property
    def ssqq(self) -> I64Vector4: ...
    @property
    def ssqp(self) -> I64Vector4: ...
    @property
    def ssqo(self) -> I64Vector4: ...
    @property
    def ssql(self) -> I64Vector4: ...
    @property
    def ssps(self) -> I64Vector4: ...
    @property
    def sspt(self) -> I64Vector4: ...
    @property
    def sspq(self) -> I64Vector4: ...
    @property
    def sspp(self) -> I64Vector4: ...
    @property
    def sspo(self) -> I64Vector4: ...
    @property
    def sspl(self) -> I64Vector4: ...
    @property
    def ssos(self) -> I64Vector4: ...
    @property
    def ssot(self) -> I64Vector4: ...
    @property
    def ssoq(self) -> I64Vector4: ...
    @property
    def ssop(self) -> I64Vector4: ...
    @property
    def ssoo(self) -> I64Vector4: ...
    @property
    def ssol(self) -> I64Vector4: ...
    @property
    def ssls(self) -> I64Vector4: ...
    @property
    def sslt(self) -> I64Vector4: ...
    @property
    def sslq(self) -> I64Vector4: ...
    @property
    def sslp(self) -> I64Vector4: ...
    @property
    def sslo(self) -> I64Vector4: ...
    @property
    def ssll(self) -> I64Vector4: ...
    @property
    def stss(self) -> I64Vector4: ...
    @property
    def stst(self) -> I64Vector4: ...
    @property
    def stsq(self) -> I64Vector4: ...
    @property
    def stsp(self) -> I64Vector4: ...
    @property
    def stso(self) -> I64Vector4: ...
    @property
    def stsl(self) -> I64Vector4: ...
    @property
    def stts(self) -> I64Vector4: ...
    @property
    def sttt(self) -> I64Vector4: ...
    @property
    def sttq(self) -> I64Vector4: ...
    @property
    def sttp(self) -> I64Vector4: ...
    @property
    def stto(self) -> I64Vector4: ...
    @property
    def sttl(self) -> I64Vector4: ...
    @property
    def stqs(self) -> I64Vector4: ...
    @property
    def stqt(self) -> I64Vector4: ...
    @property
    def stqq(self) -> I64Vector4: ...
    @property
    def stqp(self) -> I64Vector4: ...
    @property
    def stqo(self) -> I64Vector4: ...
    @property
    def stql(self) -> I64Vector4: ...
    @property
    def stps(self) -> I64Vector4: ...
    @property
    def stpt(self) -> I64Vector4: ...
    @property
    def stpq(self) -> I64Vector4: ...
    @property
    def stpp(self) -> I64Vector4: ...
    @property
    def stpo(self) -> I64Vector4: ...
    @property
    def stpl(self) -> I64Vector4: ...
    @property
    def stos(self) -> I64Vector4: ...
    @property
    def stot(self) -> I64Vector4: ...
    @property
    def stoq(self) -> I64Vector4: ...
    @property
    def stop(self) -> I64Vector4: ...
    @property
    def stoo(self) -> I64Vector4: ...
    @property
    def stol(self) -> I64Vector4: ...
    @property
    def stls(self) -> I64Vector4: ...
    @property
    def stlt(self) -> I64Vector4: ...
    @property
    def stlq(self) -> I64Vector4: ...
    @property
    def stlp(self) -> I64Vector4: ...
    @property
    def stlo(self) -> I64Vector4: ...
    @property
    def stll(self) -> I64Vector4: ...
    @property
    def sqss(self) -> I64Vector4: ...
    @property
    def sqst(self) -> I64Vector4: ...
    @property
    def sqsq(self) -> I64Vector4: ...
    @property
    def sqsp(self) -> I64Vector4: ...
    @property
    def sqso(self) -> I64Vector4: ...
    @property
    def sqsl(self) -> I64Vector4: ...
    @property
    def sqts(self) -> I64Vector4: ...
    @property
    def sqtt(self) -> I64Vector4: ...
    @property
    def sqtq(self) -> I64Vector4: ...
    @property
    def sqtp(self) -> I64Vector4: ...
    @property
    def sqto(self) -> I64Vector4: ...
    @property
    def sqtl(self) -> I64Vector4: ...
    @property
    def sqqs(self) -> I64Vector4: ...
    @property
    def sqqt(self) -> I64Vector4: ...
    @property
    def sqqq(self) -> I64Vector4: ...
    @property
    def sqqp(self) -> I64Vector4: ...
    @property
    def sqqo(self) -> I64Vector4: ...
    @property
    def sqql(self) -> I64Vector4: ...
    @property
    def sqps(self) -> I64Vector4: ...
    @property
    def sqpt(self) -> I64Vector4: ...
    @property
    def sqpq(self) -> I64Vector4: ...
    @property
    def sqpp(self) -> I64Vector4: ...
    @property
    def sqpo(self) -> I64Vector4: ...
    @property
    def sqpl(self) -> I64Vector4: ...
    @property
    def sqos(self) -> I64Vector4: ...
    @property
    def sqot(self) -> I64Vector4: ...
    @property
    def sqoq(self) -> I64Vector4: ...
    @property
    def sqop(self) -> I64Vector4: ...
    @property
    def sqoo(self) -> I64Vector4: ...
    @property
    def sqol(self) -> I64Vector4: ...
    @property
    def sqls(self) -> I64Vector4: ...
    @property
    def sqlt(self) -> I64Vector4: ...
    @property
    def sqlq(self) -> I64Vector4: ...
    @property
    def sqlp(self) -> I64Vector4: ...
    @property
    def sqlo(self) -> I64Vector4: ...
    @property
    def sqll(self) -> I64Vector4: ...
    @property
    def spss(self) -> I64Vector4: ...
    @property
    def spst(self) -> I64Vector4: ...
    @property
    def spsq(self) -> I64Vector4: ...
    @property
    def spsp(self) -> I64Vector4: ...
    @property
    def spso(self) -> I64Vector4: ...
    @property
    def spsl(self) -> I64Vector4: ...
    @property
    def spts(self) -> I64Vector4: ...
    @property
    def sptt(self) -> I64Vector4: ...
    @property
    def sptq(self) -> I64Vector4: ...
    @property
    def sptp(self) -> I64Vector4: ...
    @property
    def spto(self) -> I64Vector4: ...
    @property
    def sptl(self) -> I64Vector4: ...
    @property
    def spqs(self) -> I64Vector4: ...
    @property
    def spqt(self) -> I64Vector4: ...
    @property
    def spqq(self) -> I64Vector4: ...
    @property
    def spqp(self) -> I64Vector4: ...
    @property
    def spqo(self) -> I64Vector4: ...
    @property
    def spql(self) -> I64Vector4: ...
    @property
    def spps(self) -> I64Vector4: ...
    @property
    def sppt(self) -> I64Vector4: ...
    @property
    def sppq(self) -> I64Vector4: ...
    @property
    def sppp(self) -> I64Vector4: ...
    @property
    def sppo(self) -> I64Vector4: ...
    @property
    def sppl(self) -> I64Vector4: ...
    @property
    def spos(self) -> I64Vector4: ...
    @property
    def spot(self) -> I64Vector4: ...
    @property
    def spoq(self) -> I64Vector4: ...
    @property
    def spop(self) -> I64Vector4: ...
    @property
    def spoo(self) -> I64Vector4: ...
    @property
    def spol(self) -> I64Vector4: ...
    @property
    def spls(self) -> I64Vector4: ...
    @property
    def splt(self) -> I64Vector4: ...
    @property
    def splq(self) -> I64Vector4: ...
    @property
    def splp(self) -> I64Vector4: ...
    @property
    def splo(self) -> I64Vector4: ...
    @property
    def spll(self) -> I64Vector4: ...
    @property
    def soss(self) -> I64Vector4: ...
    @property
    def sost(self) -> I64Vector4: ...
    @property
    def sosq(self) -> I64Vector4: ...
    @property
    def sosp(self) -> I64Vector4: ...
    @property
    def soso(self) -> I64Vector4: ...
    @property
    def sosl(self) -> I64Vector4: ...
    @property
    def sots(self) -> I64Vector4: ...
    @property
    def sott(self) -> I64Vector4: ...
    @property
    def sotq(self) -> I64Vector4: ...
    @property
    def sotp(self) -> I64Vector4: ...
    @property
    def soto(self) -> I64Vector4: ...
    @property
    def sotl(self) -> I64Vector4: ...
    @property
    def soqs(self) -> I64Vector4: ...
    @property
    def soqt(self) -> I64Vector4: ...
    @property
    def soqq(self) -> I64Vector4: ...
    @property
    def soqp(self) -> I64Vector4: ...
    @property
    def soqo(self) -> I64Vector4: ...
    @property
    def soql(self) -> I64Vector4: ...
    @property
    def sops(self) -> I64Vector4: ...
    @property
    def sopt(self) -> I64Vector4: ...
    @property
    def sopq(self) -> I64Vector4: ...
    @property
    def sopp(self) -> I64Vector4: ...
    @property
    def sopo(self) -> I64Vector4: ...
    @property
    def sopl(self) -> I64Vector4: ...
    @property
    def soos(self) -> I64Vector4: ...
    @property
    def soot(self) -> I64Vector4: ...
    @property
    def sooq(self) -> I64Vector4: ...
    @property
    def soop(self) -> I64Vector4: ...
    @property
    def sooo(self) -> I64Vector4: ...
    @property
    def sool(self) -> I64Vector4: ...
    @property
    def sols(self) -> I64Vector4: ...
    @property
    def solt(self) -> I64Vector4: ...
    @property
    def solq(self) -> I64Vector4: ...
    @property
    def solp(self) -> I64Vector4: ...
    @property
    def solo(self) -> I64Vector4: ...
    @property
    def soll(self) -> I64Vector4: ...
    @property
    def slss(self) -> I64Vector4: ...
    @property
    def slst(self) -> I64Vector4: ...
    @property
    def slsq(self) -> I64Vector4: ...
    @property
    def slsp(self) -> I64Vector4: ...
    @property
    def slso(self) -> I64Vector4: ...
    @property
    def slsl(self) -> I64Vector4: ...
    @property
    def slts(self) -> I64Vector4: ...
    @property
    def sltt(self) -> I64Vector4: ...
    @property
    def sltq(self) -> I64Vector4: ...
    @property
    def sltp(self) -> I64Vector4: ...
    @property
    def slto(self) -> I64Vector4: ...
    @property
    def sltl(self) -> I64Vector4: ...
    @property
    def slqs(self) -> I64Vector4: ...
    @property
    def slqt(self) -> I64Vector4: ...
    @property
    def slqq(self) -> I64Vector4: ...
    @property
    def slqp(self) -> I64Vector4: ...
    @property
    def slqo(self) -> I64Vector4: ...
    @property
    def slql(self) -> I64Vector4: ...
    @property
    def slps(self) -> I64Vector4: ...
    @property
    def slpt(self) -> I64Vector4: ...
    @property
    def slpq(self) -> I64Vector4: ...
    @property
    def slpp(self) -> I64Vector4: ...
    @property
    def slpo(self) -> I64Vector4: ...
    @property
    def slpl(self) -> I64Vector4: ...
    @property
    def slos(self) -> I64Vector4: ...
    @property
    def slot(self) -> I64Vector4: ...
    @property
    def sloq(self) -> I64Vector4: ...
    @property
    def slop(self) -> I64Vector4: ...
    @property
    def sloo(self) -> I64Vector4: ...
    @property
    def slol(self) -> I64Vector4: ...
    @property
    def slls(self) -> I64Vector4: ...
    @property
    def sllt(self) -> I64Vector4: ...
    @property
    def sllq(self) -> I64Vector4: ...
    @property
    def sllp(self) -> I64Vector4: ...
    @property
    def sllo(self) -> I64Vector4: ...
    @property
    def slll(self) -> I64Vector4: ...
    @property
    def tsss(self) -> I64Vector4: ...
    @property
    def tsst(self) -> I64Vector4: ...
    @property
    def tssq(self) -> I64Vector4: ...
    @property
    def tssp(self) -> I64Vector4: ...
    @property
    def tsso(self) -> I64Vector4: ...
    @property
    def tssl(self) -> I64Vector4: ...
    @property
    def tsts(self) -> I64Vector4: ...
    @property
    def tstt(self) -> I64Vector4: ...
    @property
    def tstq(self) -> I64Vector4: ...
    @property
    def tstp(self) -> I64Vector4: ...
    @property
    def tsto(self) -> I64Vector4: ...
    @property
    def tstl(self) -> I64Vector4: ...
    @property
    def tsqs(self) -> I64Vector4: ...
    @property
    def tsqt(self) -> I64Vector4: ...
    @property
    def tsqq(self) -> I64Vector4: ...
    @property
    def tsqp(self) -> I64Vector4: ...
    @property
    def tsqo(self) -> I64Vector4: ...
    @property
    def tsql(self) -> I64Vector4: ...
    @property
    def tsps(self) -> I64Vector4: ...
    @property
    def tspt(self) -> I64Vector4: ...
    @property
    def tspq(self) -> I64Vector4: ...
    @property
    def tspp(self) -> I64Vector4: ...
    @property
    def tspo(self) -> I64Vector4: ...
    @property
    def tspl(self) -> I64Vector4: ...
    @property
    def tsos(self) -> I64Vector4: ...
    @property
    def tsot(self) -> I64Vector4: ...
    @property
    def tsoq(self) -> I64Vector4: ...
    @property
    def tsop(self) -> I64Vector4: ...
    @property
    def tsoo(self) -> I64Vector4: ...
    @property
    def tsol(self) -> I64Vector4: ...
    @property
    def tsls(self) -> I64Vector4: ...
    @property
    def tslt(self) -> I64Vector4: ...
    @property
    def tslq(self) -> I64Vector4: ...
    @property
    def tslp(self) -> I64Vector4: ...
    @property
    def tslo(self) -> I64Vector4: ...
    @property
    def tsll(self) -> I64Vector4: ...
    @property
    def ttss(self) -> I64Vector4: ...
    @property
    def ttst(self) -> I64Vector4: ...
    @property
    def ttsq(self) -> I64Vector4: ...
    @property
    def ttsp(self) -> I64Vector4: ...
    @property
    def ttso(self) -> I64Vector4: ...
    @property
    def ttsl(self) -> I64Vector4: ...
    @property
    def ttts(self) -> I64Vector4: ...
    @property
    def tttt(self) -> I64Vector4: ...
    @property
    def tttq(self) -> I64Vector4: ...
    @property
    def tttp(self) -> I64Vector4: ...
    @property
    def ttto(self) -> I64Vector4: ...
    @property
    def tttl(self) -> I64Vector4: ...
    @property
    def ttqs(self) -> I64Vector4: ...
    @property
    def ttqt(self) -> I64Vector4: ...
    @property
    def ttqq(self) -> I64Vector4: ...
    @property
    def ttqp(self) -> I64Vector4: ...
    @property
    def ttqo(self) -> I64Vector4: ...
    @property
    def ttql(self) -> I64Vector4: ...
    @property
    def ttps(self) -> I64Vector4: ...
    @property
    def ttpt(self) -> I64Vector4: ...
    @property
    def ttpq(self) -> I64Vector4: ...
    @property
    def ttpp(self) -> I64Vector4: ...
    @property
    def ttpo(self) -> I64Vector4: ...
    @property
    def ttpl(self) -> I64Vector4: ...
    @property
    def ttos(self) -> I64Vector4: ...
    @property
    def ttot(self) -> I64Vector4: ...
    @property
    def ttoq(self) -> I64Vector4: ...
    @property
    def ttop(self) -> I64Vector4: ...
    @property
    def ttoo(self) -> I64Vector4: ...
    @property
    def ttol(self) -> I64Vector4: ...
    @property
    def ttls(self) -> I64Vector4: ...
    @property
    def ttlt(self) -> I64Vector4: ...
    @property
    def ttlq(self) -> I64Vector4: ...
    @property
    def ttlp(self) -> I64Vector4: ...
    @property
    def ttlo(self) -> I64Vector4: ...
    @property
    def ttll(self) -> I64Vector4: ...
    @property
    def tqss(self) -> I64Vector4: ...
    @property
    def tqst(self) -> I64Vector4: ...
    @property
    def tqsq(self) -> I64Vector4: ...
    @property
    def tqsp(self) -> I64Vector4: ...
    @property
    def tqso(self) -> I64Vector4: ...
    @property
    def tqsl(self) -> I64Vector4: ...
    @property
    def tqts(self) -> I64Vector4: ...
    @property
    def tqtt(self) -> I64Vector4: ...
    @property
    def tqtq(self) -> I64Vector4: ...
    @property
    def tqtp(self) -> I64Vector4: ...
    @property
    def tqto(self) -> I64Vector4: ...
    @property
    def tqtl(self) -> I64Vector4: ...
    @property
    def tqqs(self) -> I64Vector4: ...
    @property
    def tqqt(self) -> I64Vector4: ...
    @property
    def tqqq(self) -> I64Vector4: ...
    @property
    def tqqp(self) -> I64Vector4: ...
    @property
    def tqqo(self) -> I64Vector4: ...
    @property
    def tqql(self) -> I64Vector4: ...
    @property
    def tqps(self) -> I64Vector4: ...
    @property
    def tqpt(self) -> I64Vector4: ...
    @property
    def tqpq(self) -> I64Vector4: ...
    @property
    def tqpp(self) -> I64Vector4: ...
    @property
    def tqpo(self) -> I64Vector4: ...
    @property
    def tqpl(self) -> I64Vector4: ...
    @property
    def tqos(self) -> I64Vector4: ...
    @property
    def tqot(self) -> I64Vector4: ...
    @property
    def tqoq(self) -> I64Vector4: ...
    @property
    def tqop(self) -> I64Vector4: ...
    @property
    def tqoo(self) -> I64Vector4: ...
    @property
    def tqol(self) -> I64Vector4: ...
    @property
    def tqls(self) -> I64Vector4: ...
    @property
    def tqlt(self) -> I64Vector4: ...
    @property
    def tqlq(self) -> I64Vector4: ...
    @property
    def tqlp(self) -> I64Vector4: ...
    @property
    def tqlo(self) -> I64Vector4: ...
    @property
    def tqll(self) -> I64Vector4: ...
    @property
    def tpss(self) -> I64Vector4: ...
    @property
    def tpst(self) -> I64Vector4: ...
    @property
    def tpsq(self) -> I64Vector4: ...
    @property
    def tpsp(self) -> I64Vector4: ...
    @property
    def tpso(self) -> I64Vector4: ...
    @property
    def tpsl(self) -> I64Vector4: ...
    @property
    def tpts(self) -> I64Vector4: ...
    @property
    def tptt(self) -> I64Vector4: ...
    @property
    def tptq(self) -> I64Vector4: ...
    @property
    def tptp(self) -> I64Vector4: ...
    @property
    def tpto(self) -> I64Vector4: ...
    @property
    def tptl(self) -> I64Vector4: ...
    @property
    def tpqs(self) -> I64Vector4: ...
    @property
    def tpqt(self) -> I64Vector4: ...
    @property
    def tpqq(self) -> I64Vector4: ...
    @property
    def tpqp(self) -> I64Vector4: ...
    @property
    def tpqo(self) -> I64Vector4: ...
    @property
    def tpql(self) -> I64Vector4: ...
    @property
    def tpps(self) -> I64Vector4: ...
    @property
    def tppt(self) -> I64Vector4: ...
    @property
    def tppq(self) -> I64Vector4: ...
    @property
    def tppp(self) -> I64Vector4: ...
    @property
    def tppo(self) -> I64Vector4: ...
    @property
    def tppl(self) -> I64Vector4: ...
    @property
    def tpos(self) -> I64Vector4: ...
    @property
    def tpot(self) -> I64Vector4: ...
    @property
    def tpoq(self) -> I64Vector4: ...
    @property
    def tpop(self) -> I64Vector4: ...
    @property
    def tpoo(self) -> I64Vector4: ...
    @property
    def tpol(self) -> I64Vector4: ...
    @property
    def tpls(self) -> I64Vector4: ...
    @property
    def tplt(self) -> I64Vector4: ...
    @property
    def tplq(self) -> I64Vector4: ...
    @property
    def tplp(self) -> I64Vector4: ...
    @property
    def tplo(self) -> I64Vector4: ...
    @property
    def tpll(self) -> I64Vector4: ...
    @property
    def toss(self) -> I64Vector4: ...
    @property
    def tost(self) -> I64Vector4: ...
    @property
    def tosq(self) -> I64Vector4: ...
    @property
    def tosp(self) -> I64Vector4: ...
    @property
    def toso(self) -> I64Vector4: ...
    @property
    def tosl(self) -> I64Vector4: ...
    @property
    def tots(self) -> I64Vector4: ...
    @property
    def tott(self) -> I64Vector4: ...
    @property
    def totq(self) -> I64Vector4: ...
    @property
    def totp(self) -> I64Vector4: ...
    @property
    def toto(self) -> I64Vector4: ...
    @property
    def totl(self) -> I64Vector4: ...
    @property
    def toqs(self) -> I64Vector4: ...
    @property
    def toqt(self) -> I64Vector4: ...
    @property
    def toqq(self) -> I64Vector4: ...
    @property
    def toqp(self) -> I64Vector4: ...
    @property
    def toqo(self) -> I64Vector4: ...
    @property
    def toql(self) -> I64Vector4: ...
    @property
    def tops(self) -> I64Vector4: ...
    @property
    def topt(self) -> I64Vector4: ...
    @property
    def topq(self) -> I64Vector4: ...
    @property
    def topp(self) -> I64Vector4: ...
    @property
    def topo(self) -> I64Vector4: ...
    @property
    def topl(self) -> I64Vector4: ...
    @property
    def toos(self) -> I64Vector4: ...
    @property
    def toot(self) -> I64Vector4: ...
    @property
    def tooq(self) -> I64Vector4: ...
    @property
    def toop(self) -> I64Vector4: ...
    @property
    def tooo(self) -> I64Vector4: ...
    @property
    def tool(self) -> I64Vector4: ...
    @property
    def tols(self) -> I64Vector4: ...
    @property
    def tolt(self) -> I64Vector4: ...
    @property
    def tolq(self) -> I64Vector4: ...
    @property
    def tolp(self) -> I64Vector4: ...
    @property
    def tolo(self) -> I64Vector4: ...
    @property
    def toll(self) -> I64Vector4: ...
    @property
    def tlss(self) -> I64Vector4: ...
    @property
    def tlst(self) -> I64Vector4: ...
    @property
    def tlsq(self) -> I64Vector4: ...
    @property
    def tlsp(self) -> I64Vector4: ...
    @property
    def tlso(self) -> I64Vector4: ...
    @property
    def tlsl(self) -> I64Vector4: ...
    @property
    def tlts(self) -> I64Vector4: ...
    @property
    def tltt(self) -> I64Vector4: ...
    @property
    def tltq(self) -> I64Vector4: ...
    @property
    def tltp(self) -> I64Vector4: ...
    @property
    def tlto(self) -> I64Vector4: ...
    @property
    def tltl(self) -> I64Vector4: ...
    @property
    def tlqs(self) -> I64Vector4: ...
    @property
    def tlqt(self) -> I64Vector4: ...
    @property
    def tlqq(self) -> I64Vector4: ...
    @property
    def tlqp(self) -> I64Vector4: ...
    @property
    def tlqo(self) -> I64Vector4: ...
    @property
    def tlql(self) -> I64Vector4: ...
    @property
    def tlps(self) -> I64Vector4: ...
    @property
    def tlpt(self) -> I64Vector4: ...
    @property
    def tlpq(self) -> I64Vector4: ...
    @property
    def tlpp(self) -> I64Vector4: ...
    @property
    def tlpo(self) -> I64Vector4: ...
    @property
    def tlpl(self) -> I64Vector4: ...
    @property
    def tlos(self) -> I64Vector4: ...
    @property
    def tlot(self) -> I64Vector4: ...
    @property
    def tloq(self) -> I64Vector4: ...
    @property
    def tlop(self) -> I64Vector4: ...
    @property
    def tloo(self) -> I64Vector4: ...
    @property
    def tlol(self) -> I64Vector4: ...
    @property
    def tlls(self) -> I64Vector4: ...
    @property
    def tllt(self) -> I64Vector4: ...
    @property
    def tllq(self) -> I64Vector4: ...
    @property
    def tllp(self) -> I64Vector4: ...
    @property
    def tllo(self) -> I64Vector4: ...
    @property
    def tlll(self) -> I64Vector4: ...
    @property
    def qsss(self) -> I64Vector4: ...
    @property
    def qsst(self) -> I64Vector4: ...
    @property
    def qssq(self) -> I64Vector4: ...
    @property
    def qssp(self) -> I64Vector4: ...
    @property
    def qsso(self) -> I64Vector4: ...
    @property
    def qssl(self) -> I64Vector4: ...
    @property
    def qsts(self) -> I64Vector4: ...
    @property
    def qstt(self) -> I64Vector4: ...
    @property
    def qstq(self) -> I64Vector4: ...
    @property
    def qstp(self) -> I64Vector4: ...
    @property
    def qsto(self) -> I64Vector4: ...
    @property
    def qstl(self) -> I64Vector4: ...
    @property
    def qsqs(self) -> I64Vector4: ...
    @property
    def qsqt(self) -> I64Vector4: ...
    @property
    def qsqq(self) -> I64Vector4: ...
    @property
    def qsqp(self) -> I64Vector4: ...
    @property
    def qsqo(self) -> I64Vector4: ...
    @property
    def qsql(self) -> I64Vector4: ...
    @property
    def qsps(self) -> I64Vector4: ...
    @property
    def qspt(self) -> I64Vector4: ...
    @property
    def qspq(self) -> I64Vector4: ...
    @property
    def qspp(self) -> I64Vector4: ...
    @property
    def qspo(self) -> I64Vector4: ...
    @property
    def qspl(self) -> I64Vector4: ...
    @property
    def qsos(self) -> I64Vector4: ...
    @property
    def qsot(self) -> I64Vector4: ...
    @property
    def qsoq(self) -> I64Vector4: ...
    @property
    def qsop(self) -> I64Vector4: ...
    @property
    def qsoo(self) -> I64Vector4: ...
    @property
    def qsol(self) -> I64Vector4: ...
    @property
    def qsls(self) -> I64Vector4: ...
    @property
    def qslt(self) -> I64Vector4: ...
    @property
    def qslq(self) -> I64Vector4: ...
    @property
    def qslp(self) -> I64Vector4: ...
    @property
    def qslo(self) -> I64Vector4: ...
    @property
    def qsll(self) -> I64Vector4: ...
    @property
    def qtss(self) -> I64Vector4: ...
    @property
    def qtst(self) -> I64Vector4: ...
    @property
    def qtsq(self) -> I64Vector4: ...
    @property
    def qtsp(self) -> I64Vector4: ...
    @property
    def qtso(self) -> I64Vector4: ...
    @property
    def qtsl(self) -> I64Vector4: ...
    @property
    def qtts(self) -> I64Vector4: ...
    @property
    def qttt(self) -> I64Vector4: ...
    @property
    def qttq(self) -> I64Vector4: ...
    @property
    def qttp(self) -> I64Vector4: ...
    @property
    def qtto(self) -> I64Vector4: ...
    @property
    def qttl(self) -> I64Vector4: ...
    @property
    def qtqs(self) -> I64Vector4: ...
    @property
    def qtqt(self) -> I64Vector4: ...
    @property
    def qtqq(self) -> I64Vector4: ...
    @property
    def qtqp(self) -> I64Vector4: ...
    @property
    def qtqo(self) -> I64Vector4: ...
    @property
    def qtql(self) -> I64Vector4: ...
    @property
    def qtps(self) -> I64Vector4: ...
    @property
    def qtpt(self) -> I64Vector4: ...
    @property
    def qtpq(self) -> I64Vector4: ...
    @property
    def qtpp(self) -> I64Vector4: ...
    @property
    def qtpo(self) -> I64Vector4: ...
    @property
    def qtpl(self) -> I64Vector4: ...
    @property
    def qtos(self) -> I64Vector4: ...
    @property
    def qtot(self) -> I64Vector4: ...
    @property
    def qtoq(self) -> I64Vector4: ...
    @property
    def qtop(self) -> I64Vector4: ...
    @property
    def qtoo(self) -> I64Vector4: ...
    @property
    def qtol(self) -> I64Vector4: ...
    @property
    def qtls(self) -> I64Vector4: ...
    @property
    def qtlt(self) -> I64Vector4: ...
    @property
    def qtlq(self) -> I64Vector4: ...
    @property
    def qtlp(self) -> I64Vector4: ...
    @property
    def qtlo(self) -> I64Vector4: ...
    @property
    def qtll(self) -> I64Vector4: ...
    @property
    def qqss(self) -> I64Vector4: ...
    @property
    def qqst(self) -> I64Vector4: ...
    @property
    def qqsq(self) -> I64Vector4: ...
    @property
    def qqsp(self) -> I64Vector4: ...
    @property
    def qqso(self) -> I64Vector4: ...
    @property
    def qqsl(self) -> I64Vector4: ...
    @property
    def qqts(self) -> I64Vector4: ...
    @property
    def qqtt(self) -> I64Vector4: ...
    @property
    def qqtq(self) -> I64Vector4: ...
    @property
    def qqtp(self) -> I64Vector4: ...
    @property
    def qqto(self) -> I64Vector4: ...
    @property
    def qqtl(self) -> I64Vector4: ...
    @property
    def qqqs(self) -> I64Vector4: ...
    @property
    def qqqt(self) -> I64Vector4: ...
    @property
    def qqqq(self) -> I64Vector4: ...
    @property
    def qqqp(self) -> I64Vector4: ...
    @property
    def qqqo(self) -> I64Vector4: ...
    @property
    def qqql(self) -> I64Vector4: ...
    @property
    def qqps(self) -> I64Vector4: ...
    @property
    def qqpt(self) -> I64Vector4: ...
    @property
    def qqpq(self) -> I64Vector4: ...
    @property
    def qqpp(self) -> I64Vector4: ...
    @property
    def qqpo(self) -> I64Vector4: ...
    @property
    def qqpl(self) -> I64Vector4: ...
    @property
    def qqos(self) -> I64Vector4: ...
    @property
    def qqot(self) -> I64Vector4: ...
    @property
    def qqoq(self) -> I64Vector4: ...
    @property
    def qqop(self) -> I64Vector4: ...
    @property
    def qqoo(self) -> I64Vector4: ...
    @property
    def qqol(self) -> I64Vector4: ...
    @property
    def qqls(self) -> I64Vector4: ...
    @property
    def qqlt(self) -> I64Vector4: ...
    @property
    def qqlq(self) -> I64Vector4: ...
    @property
    def qqlp(self) -> I64Vector4: ...
    @property
    def qqlo(self) -> I64Vector4: ...
    @property
    def qqll(self) -> I64Vector4: ...
    @property
    def qpss(self) -> I64Vector4: ...
    @property
    def qpst(self) -> I64Vector4: ...
    @property
    def qpsq(self) -> I64Vector4: ...
    @property
    def qpsp(self) -> I64Vector4: ...
    @property
    def qpso(self) -> I64Vector4: ...
    @property
    def qpsl(self) -> I64Vector4: ...
    @property
    def qpts(self) -> I64Vector4: ...
    @property
    def qptt(self) -> I64Vector4: ...
    @property
    def qptq(self) -> I64Vector4: ...
    @property
    def qptp(self) -> I64Vector4: ...
    @property
    def qpto(self) -> I64Vector4: ...
    @property
    def qptl(self) -> I64Vector4: ...
    @property
    def qpqs(self) -> I64Vector4: ...
    @property
    def qpqt(self) -> I64Vector4: ...
    @property
    def qpqq(self) -> I64Vector4: ...
    @property
    def qpqp(self) -> I64Vector4: ...
    @property
    def qpqo(self) -> I64Vector4: ...
    @property
    def qpql(self) -> I64Vector4: ...
    @property
    def qpps(self) -> I64Vector4: ...
    @property
    def qppt(self) -> I64Vector4: ...
    @property
    def qppq(self) -> I64Vector4: ...
    @property
    def qppp(self) -> I64Vector4: ...
    @property
    def qppo(self) -> I64Vector4: ...
    @property
    def qppl(self) -> I64Vector4: ...
    @property
    def qpos(self) -> I64Vector4: ...
    @property
    def qpot(self) -> I64Vector4: ...
    @property
    def qpoq(self) -> I64Vector4: ...
    @property
    def qpop(self) -> I64Vector4: ...
    @property
    def qpoo(self) -> I64Vector4: ...
    @property
    def qpol(self) -> I64Vector4: ...
    @property
    def qpls(self) -> I64Vector4: ...
    @property
    def qplt(self) -> I64Vector4: ...
    @property
    def qplq(self) -> I64Vector4: ...
    @property
    def qplp(self) -> I64Vector4: ...
    @property
    def qplo(self) -> I64Vector4: ...
    @property
    def qpll(self) -> I64Vector4: ...
    @property
    def qoss(self) -> I64Vector4: ...
    @property
    def qost(self) -> I64Vector4: ...
    @property
    def qosq(self) -> I64Vector4: ...
    @property
    def qosp(self) -> I64Vector4: ...
    @property
    def qoso(self) -> I64Vector4: ...
    @property
    def qosl(self) -> I64Vector4: ...
    @property
    def qots(self) -> I64Vector4: ...
    @property
    def qott(self) -> I64Vector4: ...
    @property
    def qotq(self) -> I64Vector4: ...
    @property
    def qotp(self) -> I64Vector4: ...
    @property
    def qoto(self) -> I64Vector4: ...
    @property
    def qotl(self) -> I64Vector4: ...
    @property
    def qoqs(self) -> I64Vector4: ...
    @property
    def qoqt(self) -> I64Vector4: ...
    @property
    def qoqq(self) -> I64Vector4: ...
    @property
    def qoqp(self) -> I64Vector4: ...
    @property
    def qoqo(self) -> I64Vector4: ...
    @property
    def qoql(self) -> I64Vector4: ...
    @property
    def qops(self) -> I64Vector4: ...
    @property
    def qopt(self) -> I64Vector4: ...
    @property
    def qopq(self) -> I64Vector4: ...
    @property
    def qopp(self) -> I64Vector4: ...
    @property
    def qopo(self) -> I64Vector4: ...
    @property
    def qopl(self) -> I64Vector4: ...
    @property
    def qoos(self) -> I64Vector4: ...
    @property
    def qoot(self) -> I64Vector4: ...
    @property
    def qooq(self) -> I64Vector4: ...
    @property
    def qoop(self) -> I64Vector4: ...
    @property
    def qooo(self) -> I64Vector4: ...
    @property
    def qool(self) -> I64Vector4: ...
    @property
    def qols(self) -> I64Vector4: ...
    @property
    def qolt(self) -> I64Vector4: ...
    @property
    def qolq(self) -> I64Vector4: ...
    @property
    def qolp(self) -> I64Vector4: ...
    @property
    def qolo(self) -> I64Vector4: ...
    @property
    def qoll(self) -> I64Vector4: ...
    @property
    def qlss(self) -> I64Vector4: ...
    @property
    def qlst(self) -> I64Vector4: ...
    @property
    def qlsq(self) -> I64Vector4: ...
    @property
    def qlsp(self) -> I64Vector4: ...
    @property
    def qlso(self) -> I64Vector4: ...
    @property
    def qlsl(self) -> I64Vector4: ...
    @property
    def qlts(self) -> I64Vector4: ...
    @property
    def qltt(self) -> I64Vector4: ...
    @property
    def qltq(self) -> I64Vector4: ...
    @property
    def qltp(self) -> I64Vector4: ...
    @property
    def qlto(self) -> I64Vector4: ...
    @property
    def qltl(self) -> I64Vector4: ...
    @property
    def qlqs(self) -> I64Vector4: ...
    @property
    def qlqt(self) -> I64Vector4: ...
    @property
    def qlqq(self) -> I64Vector4: ...
    @property
    def qlqp(self) -> I64Vector4: ...
    @property
    def qlqo(self) -> I64Vector4: ...
    @property
    def qlql(self) -> I64Vector4: ...
    @property
    def qlps(self) -> I64Vector4: ...
    @property
    def qlpt(self) -> I64Vector4: ...
    @property
    def qlpq(self) -> I64Vector4: ...
    @property
    def qlpp(self) -> I64Vector4: ...
    @property
    def qlpo(self) -> I64Vector4: ...
    @property
    def qlpl(self) -> I64Vector4: ...
    @property
    def qlos(self) -> I64Vector4: ...
    @property
    def qlot(self) -> I64Vector4: ...
    @property
    def qloq(self) -> I64Vector4: ...
    @property
    def qlop(self) -> I64Vector4: ...
    @property
    def qloo(self) -> I64Vector4: ...
    @property
    def qlol(self) -> I64Vector4: ...
    @property
    def qlls(self) -> I64Vector4: ...
    @property
    def qllt(self) -> I64Vector4: ...
    @property
    def qllq(self) -> I64Vector4: ...
    @property
    def qllp(self) -> I64Vector4: ...
    @property
    def qllo(self) -> I64Vector4: ...
    @property
    def qlll(self) -> I64Vector4: ...
    @property
    def psss(self) -> I64Vector4: ...
    @property
    def psst(self) -> I64Vector4: ...
    @property
    def pssq(self) -> I64Vector4: ...
    @property
    def pssp(self) -> I64Vector4: ...
    @property
    def psso(self) -> I64Vector4: ...
    @property
    def pssl(self) -> I64Vector4: ...
    @property
    def psts(self) -> I64Vector4: ...
    @property
    def pstt(self) -> I64Vector4: ...
    @property
    def pstq(self) -> I64Vector4: ...
    @property
    def pstp(self) -> I64Vector4: ...
    @property
    def psto(self) -> I64Vector4: ...
    @property
    def pstl(self) -> I64Vector4: ...
    @property
    def psqs(self) -> I64Vector4: ...
    @property
    def psqt(self) -> I64Vector4: ...
    @property
    def psqq(self) -> I64Vector4: ...
    @property
    def psqp(self) -> I64Vector4: ...
    @property
    def psqo(self) -> I64Vector4: ...
    @property
    def psql(self) -> I64Vector4: ...
    @property
    def psps(self) -> I64Vector4: ...
    @property
    def pspt(self) -> I64Vector4: ...
    @property
    def pspq(self) -> I64Vector4: ...
    @property
    def pspp(self) -> I64Vector4: ...
    @property
    def pspo(self) -> I64Vector4: ...
    @property
    def pspl(self) -> I64Vector4: ...
    @property
    def psos(self) -> I64Vector4: ...
    @property
    def psot(self) -> I64Vector4: ...
    @property
    def psoq(self) -> I64Vector4: ...
    @property
    def psop(self) -> I64Vector4: ...
    @property
    def psoo(self) -> I64Vector4: ...
    @property
    def psol(self) -> I64Vector4: ...
    @property
    def psls(self) -> I64Vector4: ...
    @property
    def pslt(self) -> I64Vector4: ...
    @property
    def pslq(self) -> I64Vector4: ...
    @property
    def pslp(self) -> I64Vector4: ...
    @property
    def pslo(self) -> I64Vector4: ...
    @property
    def psll(self) -> I64Vector4: ...
    @property
    def ptss(self) -> I64Vector4: ...
    @property
    def ptst(self) -> I64Vector4: ...
    @property
    def ptsq(self) -> I64Vector4: ...
    @property
    def ptsp(self) -> I64Vector4: ...
    @property
    def ptso(self) -> I64Vector4: ...
    @property
    def ptsl(self) -> I64Vector4: ...
    @property
    def ptts(self) -> I64Vector4: ...
    @property
    def pttt(self) -> I64Vector4: ...
    @property
    def pttq(self) -> I64Vector4: ...
    @property
    def pttp(self) -> I64Vector4: ...
    @property
    def ptto(self) -> I64Vector4: ...
    @property
    def pttl(self) -> I64Vector4: ...
    @property
    def ptqs(self) -> I64Vector4: ...
    @property
    def ptqt(self) -> I64Vector4: ...
    @property
    def ptqq(self) -> I64Vector4: ...
    @property
    def ptqp(self) -> I64Vector4: ...
    @property
    def ptqo(self) -> I64Vector4: ...
    @property
    def ptql(self) -> I64Vector4: ...
    @property
    def ptps(self) -> I64Vector4: ...
    @property
    def ptpt(self) -> I64Vector4: ...
    @property
    def ptpq(self) -> I64Vector4: ...
    @property
    def ptpp(self) -> I64Vector4: ...
    @property
    def ptpo(self) -> I64Vector4: ...
    @property
    def ptpl(self) -> I64Vector4: ...
    @property
    def ptos(self) -> I64Vector4: ...
    @property
    def ptot(self) -> I64Vector4: ...
    @property
    def ptoq(self) -> I64Vector4: ...
    @property
    def ptop(self) -> I64Vector4: ...
    @property
    def ptoo(self) -> I64Vector4: ...
    @property
    def ptol(self) -> I64Vector4: ...
    @property
    def ptls(self) -> I64Vector4: ...
    @property
    def ptlt(self) -> I64Vector4: ...
    @property
    def ptlq(self) -> I64Vector4: ...
    @property
    def ptlp(self) -> I64Vector4: ...
    @property
    def ptlo(self) -> I64Vector4: ...
    @property
    def ptll(self) -> I64Vector4: ...
    @property
    def pqss(self) -> I64Vector4: ...
    @property
    def pqst(self) -> I64Vector4: ...
    @property
    def pqsq(self) -> I64Vector4: ...
    @property
    def pqsp(self) -> I64Vector4: ...
    @property
    def pqso(self) -> I64Vector4: ...
    @property
    def pqsl(self) -> I64Vector4: ...
    @property
    def pqts(self) -> I64Vector4: ...
    @property
    def pqtt(self) -> I64Vector4: ...
    @property
    def pqtq(self) -> I64Vector4: ...
    @property
    def pqtp(self) -> I64Vector4: ...
    @property
    def pqto(self) -> I64Vector4: ...
    @property
    def pqtl(self) -> I64Vector4: ...
    @property
    def pqqs(self) -> I64Vector4: ...
    @property
    def pqqt(self) -> I64Vector4: ...
    @property
    def pqqq(self) -> I64Vector4: ...
    @property
    def pqqp(self) -> I64Vector4: ...
    @property
    def pqqo(self) -> I64Vector4: ...
    @property
    def pqql(self) -> I64Vector4: ...
    @property
    def pqps(self) -> I64Vector4: ...
    @property
    def pqpt(self) -> I64Vector4: ...
    @property
    def pqpq(self) -> I64Vector4: ...
    @property
    def pqpp(self) -> I64Vector4: ...
    @property
    def pqpo(self) -> I64Vector4: ...
    @property
    def pqpl(self) -> I64Vector4: ...
    @property
    def pqos(self) -> I64Vector4: ...
    @property
    def pqot(self) -> I64Vector4: ...
    @property
    def pqoq(self) -> I64Vector4: ...
    @property
    def pqop(self) -> I64Vector4: ...
    @property
    def pqoo(self) -> I64Vector4: ...
    @property
    def pqol(self) -> I64Vector4: ...
    @property
    def pqls(self) -> I64Vector4: ...
    @property
    def pqlt(self) -> I64Vector4: ...
    @property
    def pqlq(self) -> I64Vector4: ...
    @property
    def pqlp(self) -> I64Vector4: ...
    @property
    def pqlo(self) -> I64Vector4: ...
    @property
    def pqll(self) -> I64Vector4: ...
    @property
    def ppss(self) -> I64Vector4: ...
    @property
    def ppst(self) -> I64Vector4: ...
    @property
    def ppsq(self) -> I64Vector4: ...
    @property
    def ppsp(self) -> I64Vector4: ...
    @property
    def ppso(self) -> I64Vector4: ...
    @property
    def ppsl(self) -> I64Vector4: ...
    @property
    def ppts(self) -> I64Vector4: ...
    @property
    def pptt(self) -> I64Vector4: ...
    @property
    def pptq(self) -> I64Vector4: ...
    @property
    def pptp(self) -> I64Vector4: ...
    @property
    def ppto(self) -> I64Vector4: ...
    @property
    def pptl(self) -> I64Vector4: ...
    @property
    def ppqs(self) -> I64Vector4: ...
    @property
    def ppqt(self) -> I64Vector4: ...
    @property
    def ppqq(self) -> I64Vector4: ...
    @property
    def ppqp(self) -> I64Vector4: ...
    @property
    def ppqo(self) -> I64Vector4: ...
    @property
    def ppql(self) -> I64Vector4: ...
    @property
    def ppps(self) -> I64Vector4: ...
    @property
    def pppt(self) -> I64Vector4: ...
    @property
    def pppq(self) -> I64Vector4: ...
    @property
    def pppp(self) -> I64Vector4: ...
    @property
    def pppo(self) -> I64Vector4: ...
    @property
    def pppl(self) -> I64Vector4: ...
    @property
    def ppos(self) -> I64Vector4: ...
    @property
    def ppot(self) -> I64Vector4: ...
    @property
    def ppoq(self) -> I64Vector4: ...
    @property
    def ppop(self) -> I64Vector4: ...
    @property
    def ppoo(self) -> I64Vector4: ...
    @property
    def ppol(self) -> I64Vector4: ...
    @property
    def ppls(self) -> I64Vector4: ...
    @property
    def pplt(self) -> I64Vector4: ...
    @property
    def pplq(self) -> I64Vector4: ...
    @property
    def pplp(self) -> I64Vector4: ...
    @property
    def pplo(self) -> I64Vector4: ...
    @property
    def ppll(self) -> I64Vector4: ...
    @property
    def poss(self) -> I64Vector4: ...
    @property
    def post(self) -> I64Vector4: ...
    @property
    def posq(self) -> I64Vector4: ...
    @property
    def posp(self) -> I64Vector4: ...
    @property
    def poso(self) -> I64Vector4: ...
    @property
    def posl(self) -> I64Vector4: ...
    @property
    def pots(self) -> I64Vector4: ...
    @property
    def pott(self) -> I64Vector4: ...
    @property
    def potq(self) -> I64Vector4: ...
    @property
    def potp(self) -> I64Vector4: ...
    @property
    def poto(self) -> I64Vector4: ...
    @property
    def potl(self) -> I64Vector4: ...
    @property
    def poqs(self) -> I64Vector4: ...
    @property
    def poqt(self) -> I64Vector4: ...
    @property
    def poqq(self) -> I64Vector4: ...
    @property
    def poqp(self) -> I64Vector4: ...
    @property
    def poqo(self) -> I64Vector4: ...
    @property
    def poql(self) -> I64Vector4: ...
    @property
    def pops(self) -> I64Vector4: ...
    @property
    def popt(self) -> I64Vector4: ...
    @property
    def popq(self) -> I64Vector4: ...
    @property
    def popp(self) -> I64Vector4: ...
    @property
    def popo(self) -> I64Vector4: ...
    @property
    def popl(self) -> I64Vector4: ...
    @property
    def poos(self) -> I64Vector4: ...
    @property
    def poot(self) -> I64Vector4: ...
    @property
    def pooq(self) -> I64Vector4: ...
    @property
    def poop(self) -> I64Vector4: ...
    @property
    def pooo(self) -> I64Vector4: ...
    @property
    def pool(self) -> I64Vector4: ...
    @property
    def pols(self) -> I64Vector4: ...
    @property
    def polt(self) -> I64Vector4: ...
    @property
    def polq(self) -> I64Vector4: ...
    @property
    def polp(self) -> I64Vector4: ...
    @property
    def polo(self) -> I64Vector4: ...
    @property
    def poll(self) -> I64Vector4: ...
    @property
    def plss(self) -> I64Vector4: ...
    @property
    def plst(self) -> I64Vector4: ...
    @property
    def plsq(self) -> I64Vector4: ...
    @property
    def plsp(self) -> I64Vector4: ...
    @property
    def plso(self) -> I64Vector4: ...
    @property
    def plsl(self) -> I64Vector4: ...
    @property
    def plts(self) -> I64Vector4: ...
    @property
    def pltt(self) -> I64Vector4: ...
    @property
    def pltq(self) -> I64Vector4: ...
    @property
    def pltp(self) -> I64Vector4: ...
    @property
    def plto(self) -> I64Vector4: ...
    @property
    def pltl(self) -> I64Vector4: ...
    @property
    def plqs(self) -> I64Vector4: ...
    @property
    def plqt(self) -> I64Vector4: ...
    @property
    def plqq(self) -> I64Vector4: ...
    @property
    def plqp(self) -> I64Vector4: ...
    @property
    def plqo(self) -> I64Vector4: ...
    @property
    def plql(self) -> I64Vector4: ...
    @property
    def plps(self) -> I64Vector4: ...
    @property
    def plpt(self) -> I64Vector4: ...
    @property
    def plpq(self) -> I64Vector4: ...
    @property
    def plpp(self) -> I64Vector4: ...
    @property
    def plpo(self) -> I64Vector4: ...
    @property
    def plpl(self) -> I64Vector4: ...
    @property
    def plos(self) -> I64Vector4: ...
    @property
    def plot(self) -> I64Vector4: ...
    @property
    def ploq(self) -> I64Vector4: ...
    @property
    def plop(self) -> I64Vector4: ...
    @property
    def ploo(self) -> I64Vector4: ...
    @property
    def plol(self) -> I64Vector4: ...
    @property
    def plls(self) -> I64Vector4: ...
    @property
    def pllt(self) -> I64Vector4: ...
    @property
    def pllq(self) -> I64Vector4: ...
    @property
    def pllp(self) -> I64Vector4: ...
    @property
    def pllo(self) -> I64Vector4: ...
    @property
    def plll(self) -> I64Vector4: ...
    @property
    def osss(self) -> I64Vector4: ...
    @property
    def osst(self) -> I64Vector4: ...
    @property
    def ossq(self) -> I64Vector4: ...
    @property
    def ossp(self) -> I64Vector4: ...
    @property
    def osso(self) -> I64Vector4: ...
    @property
    def ossl(self) -> I64Vector4: ...
    @property
    def osts(self) -> I64Vector4: ...
    @property
    def ostt(self) -> I64Vector4: ...
    @property
    def ostq(self) -> I64Vector4: ...
    @property
    def ostp(self) -> I64Vector4: ...
    @property
    def osto(self) -> I64Vector4: ...
    @property
    def ostl(self) -> I64Vector4: ...
    @property
    def osqs(self) -> I64Vector4: ...
    @property
    def osqt(self) -> I64Vector4: ...
    @property
    def osqq(self) -> I64Vector4: ...
    @property
    def osqp(self) -> I64Vector4: ...
    @property
    def osqo(self) -> I64Vector4: ...
    @property
    def osql(self) -> I64Vector4: ...
    @property
    def osps(self) -> I64Vector4: ...
    @property
    def ospt(self) -> I64Vector4: ...
    @property
    def ospq(self) -> I64Vector4: ...
    @property
    def ospp(self) -> I64Vector4: ...
    @property
    def ospo(self) -> I64Vector4: ...
    @property
    def ospl(self) -> I64Vector4: ...
    @property
    def osos(self) -> I64Vector4: ...
    @property
    def osot(self) -> I64Vector4: ...
    @property
    def osoq(self) -> I64Vector4: ...
    @property
    def osop(self) -> I64Vector4: ...
    @property
    def osoo(self) -> I64Vector4: ...
    @property
    def osol(self) -> I64Vector4: ...
    @property
    def osls(self) -> I64Vector4: ...
    @property
    def oslt(self) -> I64Vector4: ...
    @property
    def oslq(self) -> I64Vector4: ...
    @property
    def oslp(self) -> I64Vector4: ...
    @property
    def oslo(self) -> I64Vector4: ...
    @property
    def osll(self) -> I64Vector4: ...
    @property
    def otss(self) -> I64Vector4: ...
    @property
    def otst(self) -> I64Vector4: ...
    @property
    def otsq(self) -> I64Vector4: ...
    @property
    def otsp(self) -> I64Vector4: ...
    @property
    def otso(self) -> I64Vector4: ...
    @property
    def otsl(self) -> I64Vector4: ...
    @property
    def otts(self) -> I64Vector4: ...
    @property
    def ottt(self) -> I64Vector4: ...
    @property
    def ottq(self) -> I64Vector4: ...
    @property
    def ottp(self) -> I64Vector4: ...
    @property
    def otto(self) -> I64Vector4: ...
    @property
    def ottl(self) -> I64Vector4: ...
    @property
    def otqs(self) -> I64Vector4: ...
    @property
    def otqt(self) -> I64Vector4: ...
    @property
    def otqq(self) -> I64Vector4: ...
    @property
    def otqp(self) -> I64Vector4: ...
    @property
    def otqo(self) -> I64Vector4: ...
    @property
    def otql(self) -> I64Vector4: ...
    @property
    def otps(self) -> I64Vector4: ...
    @property
    def otpt(self) -> I64Vector4: ...
    @property
    def otpq(self) -> I64Vector4: ...
    @property
    def otpp(self) -> I64Vector4: ...
    @property
    def otpo(self) -> I64Vector4: ...
    @property
    def otpl(self) -> I64Vector4: ...
    @property
    def otos(self) -> I64Vector4: ...
    @property
    def otot(self) -> I64Vector4: ...
    @property
    def otoq(self) -> I64Vector4: ...
    @property
    def otop(self) -> I64Vector4: ...
    @property
    def otoo(self) -> I64Vector4: ...
    @property
    def otol(self) -> I64Vector4: ...
    @property
    def otls(self) -> I64Vector4: ...
    @property
    def otlt(self) -> I64Vector4: ...
    @property
    def otlq(self) -> I64Vector4: ...
    @property
    def otlp(self) -> I64Vector4: ...
    @property
    def otlo(self) -> I64Vector4: ...
    @property
    def otll(self) -> I64Vector4: ...
    @property
    def oqss(self) -> I64Vector4: ...
    @property
    def oqst(self) -> I64Vector4: ...
    @property
    def oqsq(self) -> I64Vector4: ...
    @property
    def oqsp(self) -> I64Vector4: ...
    @property
    def oqso(self) -> I64Vector4: ...
    @property
    def oqsl(self) -> I64Vector4: ...
    @property
    def oqts(self) -> I64Vector4: ...
    @property
    def oqtt(self) -> I64Vector4: ...
    @property
    def oqtq(self) -> I64Vector4: ...
    @property
    def oqtp(self) -> I64Vector4: ...
    @property
    def oqto(self) -> I64Vector4: ...
    @property
    def oqtl(self) -> I64Vector4: ...
    @property
    def oqqs(self) -> I64Vector4: ...
    @property
    def oqqt(self) -> I64Vector4: ...
    @property
    def oqqq(self) -> I64Vector4: ...
    @property
    def oqqp(self) -> I64Vector4: ...
    @property
    def oqqo(self) -> I64Vector4: ...
    @property
    def oqql(self) -> I64Vector4: ...
    @property
    def oqps(self) -> I64Vector4: ...
    @property
    def oqpt(self) -> I64Vector4: ...
    @property
    def oqpq(self) -> I64Vector4: ...
    @property
    def oqpp(self) -> I64Vector4: ...
    @property
    def oqpo(self) -> I64Vector4: ...
    @property
    def oqpl(self) -> I64Vector4: ...
    @property
    def oqos(self) -> I64Vector4: ...
    @property
    def oqot(self) -> I64Vector4: ...
    @property
    def oqoq(self) -> I64Vector4: ...
    @property
    def oqop(self) -> I64Vector4: ...
    @property
    def oqoo(self) -> I64Vector4: ...
    @property
    def oqol(self) -> I64Vector4: ...
    @property
    def oqls(self) -> I64Vector4: ...
    @property
    def oqlt(self) -> I64Vector4: ...
    @property
    def oqlq(self) -> I64Vector4: ...
    @property
    def oqlp(self) -> I64Vector4: ...
    @property
    def oqlo(self) -> I64Vector4: ...
    @property
    def oqll(self) -> I64Vector4: ...
    @property
    def opss(self) -> I64Vector4: ...
    @property
    def opst(self) -> I64Vector4: ...
    @property
    def opsq(self) -> I64Vector4: ...
    @property
    def opsp(self) -> I64Vector4: ...
    @property
    def opso(self) -> I64Vector4: ...
    @property
    def opsl(self) -> I64Vector4: ...
    @property
    def opts(self) -> I64Vector4: ...
    @property
    def optt(self) -> I64Vector4: ...
    @property
    def optq(self) -> I64Vector4: ...
    @property
    def optp(self) -> I64Vector4: ...
    @property
    def opto(self) -> I64Vector4: ...
    @property
    def optl(self) -> I64Vector4: ...
    @property
    def opqs(self) -> I64Vector4: ...
    @property
    def opqt(self) -> I64Vector4: ...
    @property
    def opqq(self) -> I64Vector4: ...
    @property
    def opqp(self) -> I64Vector4: ...
    @property
    def opqo(self) -> I64Vector4: ...
    @property
    def opql(self) -> I64Vector4: ...
    @property
    def opps(self) -> I64Vector4: ...
    @property
    def oppt(self) -> I64Vector4: ...
    @property
    def oppq(self) -> I64Vector4: ...
    @property
    def oppp(self) -> I64Vector4: ...
    @property
    def oppo(self) -> I64Vector4: ...
    @property
    def oppl(self) -> I64Vector4: ...
    @property
    def opos(self) -> I64Vector4: ...
    @property
    def opot(self) -> I64Vector4: ...
    @property
    def opoq(self) -> I64Vector4: ...
    @property
    def opop(self) -> I64Vector4: ...
    @property
    def opoo(self) -> I64Vector4: ...
    @property
    def opol(self) -> I64Vector4: ...
    @property
    def opls(self) -> I64Vector4: ...
    @property
    def oplt(self) -> I64Vector4: ...
    @property
    def oplq(self) -> I64Vector4: ...
    @property
    def oplp(self) -> I64Vector4: ...
    @property
    def oplo(self) -> I64Vector4: ...
    @property
    def opll(self) -> I64Vector4: ...
    @property
    def ooss(self) -> I64Vector4: ...
    @property
    def oost(self) -> I64Vector4: ...
    @property
    def oosq(self) -> I64Vector4: ...
    @property
    def oosp(self) -> I64Vector4: ...
    @property
    def ooso(self) -> I64Vector4: ...
    @property
    def oosl(self) -> I64Vector4: ...
    @property
    def oots(self) -> I64Vector4: ...
    @property
    def oott(self) -> I64Vector4: ...
    @property
    def ootq(self) -> I64Vector4: ...
    @property
    def ootp(self) -> I64Vector4: ...
    @property
    def ooto(self) -> I64Vector4: ...
    @property
    def ootl(self) -> I64Vector4: ...
    @property
    def ooqs(self) -> I64Vector4: ...
    @property
    def ooqt(self) -> I64Vector4: ...
    @property
    def ooqq(self) -> I64Vector4: ...
    @property
    def ooqp(self) -> I64Vector4: ...
    @property
    def ooqo(self) -> I64Vector4: ...
    @property
    def ooql(self) -> I64Vector4: ...
    @property
    def oops(self) -> I64Vector4: ...
    @property
    def oopt(self) -> I64Vector4: ...
    @property
    def oopq(self) -> I64Vector4: ...
    @property
    def oopp(self) -> I64Vector4: ...
    @property
    def oopo(self) -> I64Vector4: ...
    @property
    def oopl(self) -> I64Vector4: ...
    @property
    def ooos(self) -> I64Vector4: ...
    @property
    def ooot(self) -> I64Vector4: ...
    @property
    def oooq(self) -> I64Vector4: ...
    @property
    def ooop(self) -> I64Vector4: ...
    @property
    def ools(self) -> I64Vector4: ...
    @property
    def oolt(self) -> I64Vector4: ...
    @property
    def oolq(self) -> I64Vector4: ...
    @property
    def oolp(self) -> I64Vector4: ...
    @property
    def olss(self) -> I64Vector4: ...
    @property
    def olst(self) -> I64Vector4: ...
    @property
    def olsq(self) -> I64Vector4: ...
    @property
    def olsp(self) -> I64Vector4: ...
    @property
    def olso(self) -> I64Vector4: ...
    @property
    def olsl(self) -> I64Vector4: ...
    @property
    def olts(self) -> I64Vector4: ...
    @property
    def oltt(self) -> I64Vector4: ...
    @property
    def oltq(self) -> I64Vector4: ...
    @property
    def oltp(self) -> I64Vector4: ...
    @property
    def olto(self) -> I64Vector4: ...
    @property
    def oltl(self) -> I64Vector4: ...
    @property
    def olqs(self) -> I64Vector4: ...
    @property
    def olqt(self) -> I64Vector4: ...
    @property
    def olqq(self) -> I64Vector4: ...
    @property
    def olqp(self) -> I64Vector4: ...
    @property
    def olqo(self) -> I64Vector4: ...
    @property
    def olql(self) -> I64Vector4: ...
    @property
    def olps(self) -> I64Vector4: ...
    @property
    def olpt(self) -> I64Vector4: ...
    @property
    def olpq(self) -> I64Vector4: ...
    @property
    def olpp(self) -> I64Vector4: ...
    @property
    def olpo(self) -> I64Vector4: ...
    @property
    def olpl(self) -> I64Vector4: ...
    @property
    def olos(self) -> I64Vector4: ...
    @property
    def olot(self) -> I64Vector4: ...
    @property
    def oloq(self) -> I64Vector4: ...
    @property
    def olop(self) -> I64Vector4: ...
    @property
    def olls(self) -> I64Vector4: ...
    @property
    def ollt(self) -> I64Vector4: ...
    @property
    def ollq(self) -> I64Vector4: ...
    @property
    def ollp(self) -> I64Vector4: ...
    @property
    def lsss(self) -> I64Vector4: ...
    @property
    def lsst(self) -> I64Vector4: ...
    @property
    def lssq(self) -> I64Vector4: ...
    @property
    def lssp(self) -> I64Vector4: ...
    @property
    def lsso(self) -> I64Vector4: ...
    @property
    def lssl(self) -> I64Vector4: ...
    @property
    def lsts(self) -> I64Vector4: ...
    @property
    def lstt(self) -> I64Vector4: ...
    @property
    def lstq(self) -> I64Vector4: ...
    @property
    def lstp(self) -> I64Vector4: ...
    @property
    def lsto(self) -> I64Vector4: ...
    @property
    def lstl(self) -> I64Vector4: ...
    @property
    def lsqs(self) -> I64Vector4: ...
    @property
    def lsqt(self) -> I64Vector4: ...
    @property
    def lsqq(self) -> I64Vector4: ...
    @property
    def lsqp(self) -> I64Vector4: ...
    @property
    def lsqo(self) -> I64Vector4: ...
    @property
    def lsql(self) -> I64Vector4: ...
    @property
    def lsps(self) -> I64Vector4: ...
    @property
    def lspt(self) -> I64Vector4: ...
    @property
    def lspq(self) -> I64Vector4: ...
    @property
    def lspp(self) -> I64Vector4: ...
    @property
    def lspo(self) -> I64Vector4: ...
    @property
    def lspl(self) -> I64Vector4: ...
    @property
    def lsos(self) -> I64Vector4: ...
    @property
    def lsot(self) -> I64Vector4: ...
    @property
    def lsoq(self) -> I64Vector4: ...
    @property
    def lsop(self) -> I64Vector4: ...
    @property
    def lsoo(self) -> I64Vector4: ...
    @property
    def lsol(self) -> I64Vector4: ...
    @property
    def lsls(self) -> I64Vector4: ...
    @property
    def lslt(self) -> I64Vector4: ...
    @property
    def lslq(self) -> I64Vector4: ...
    @property
    def lslp(self) -> I64Vector4: ...
    @property
    def lslo(self) -> I64Vector4: ...
    @property
    def lsll(self) -> I64Vector4: ...
    @property
    def ltss(self) -> I64Vector4: ...
    @property
    def ltst(self) -> I64Vector4: ...
    @property
    def ltsq(self) -> I64Vector4: ...
    @property
    def ltsp(self) -> I64Vector4: ...
    @property
    def ltso(self) -> I64Vector4: ...
    @property
    def ltsl(self) -> I64Vector4: ...
    @property
    def ltts(self) -> I64Vector4: ...
    @property
    def lttt(self) -> I64Vector4: ...
    @property
    def lttq(self) -> I64Vector4: ...
    @property
    def lttp(self) -> I64Vector4: ...
    @property
    def ltto(self) -> I64Vector4: ...
    @property
    def lttl(self) -> I64Vector4: ...
    @property
    def ltqs(self) -> I64Vector4: ...
    @property
    def ltqt(self) -> I64Vector4: ...
    @property
    def ltqq(self) -> I64Vector4: ...
    @property
    def ltqp(self) -> I64Vector4: ...
    @property
    def ltqo(self) -> I64Vector4: ...
    @property
    def ltql(self) -> I64Vector4: ...
    @property
    def ltps(self) -> I64Vector4: ...
    @property
    def ltpt(self) -> I64Vector4: ...
    @property
    def ltpq(self) -> I64Vector4: ...
    @property
    def ltpp(self) -> I64Vector4: ...
    @property
    def ltpo(self) -> I64Vector4: ...
    @property
    def ltpl(self) -> I64Vector4: ...
    @property
    def ltos(self) -> I64Vector4: ...
    @property
    def ltot(self) -> I64Vector4: ...
    @property
    def ltoq(self) -> I64Vector4: ...
    @property
    def ltop(self) -> I64Vector4: ...
    @property
    def ltoo(self) -> I64Vector4: ...
    @property
    def ltol(self) -> I64Vector4: ...
    @property
    def ltls(self) -> I64Vector4: ...
    @property
    def ltlt(self) -> I64Vector4: ...
    @property
    def ltlq(self) -> I64Vector4: ...
    @property
    def ltlp(self) -> I64Vector4: ...
    @property
    def ltlo(self) -> I64Vector4: ...
    @property
    def ltll(self) -> I64Vector4: ...
    @property
    def lqss(self) -> I64Vector4: ...
    @property
    def lqst(self) -> I64Vector4: ...
    @property
    def lqsq(self) -> I64Vector4: ...
    @property
    def lqsp(self) -> I64Vector4: ...
    @property
    def lqso(self) -> I64Vector4: ...
    @property
    def lqsl(self) -> I64Vector4: ...
    @property
    def lqts(self) -> I64Vector4: ...
    @property
    def lqtt(self) -> I64Vector4: ...
    @property
    def lqtq(self) -> I64Vector4: ...
    @property
    def lqtp(self) -> I64Vector4: ...
    @property
    def lqto(self) -> I64Vector4: ...
    @property
    def lqtl(self) -> I64Vector4: ...
    @property
    def lqqs(self) -> I64Vector4: ...
    @property
    def lqqt(self) -> I64Vector4: ...
    @property
    def lqqq(self) -> I64Vector4: ...
    @property
    def lqqp(self) -> I64Vector4: ...
    @property
    def lqqo(self) -> I64Vector4: ...
    @property
    def lqql(self) -> I64Vector4: ...
    @property
    def lqps(self) -> I64Vector4: ...
    @property
    def lqpt(self) -> I64Vector4: ...
    @property
    def lqpq(self) -> I64Vector4: ...
    @property
    def lqpp(self) -> I64Vector4: ...
    @property
    def lqpo(self) -> I64Vector4: ...
    @property
    def lqpl(self) -> I64Vector4: ...
    @property
    def lqos(self) -> I64Vector4: ...
    @property
    def lqot(self) -> I64Vector4: ...
    @property
    def lqoq(self) -> I64Vector4: ...
    @property
    def lqop(self) -> I64Vector4: ...
    @property
    def lqoo(self) -> I64Vector4: ...
    @property
    def lqol(self) -> I64Vector4: ...
    @property
    def lqls(self) -> I64Vector4: ...
    @property
    def lqlt(self) -> I64Vector4: ...
    @property
    def lqlq(self) -> I64Vector4: ...
    @property
    def lqlp(self) -> I64Vector4: ...
    @property
    def lqlo(self) -> I64Vector4: ...
    @property
    def lqll(self) -> I64Vector4: ...
    @property
    def lpss(self) -> I64Vector4: ...
    @property
    def lpst(self) -> I64Vector4: ...
    @property
    def lpsq(self) -> I64Vector4: ...
    @property
    def lpsp(self) -> I64Vector4: ...
    @property
    def lpso(self) -> I64Vector4: ...
    @property
    def lpsl(self) -> I64Vector4: ...
    @property
    def lpts(self) -> I64Vector4: ...
    @property
    def lptt(self) -> I64Vector4: ...
    @property
    def lptq(self) -> I64Vector4: ...
    @property
    def lptp(self) -> I64Vector4: ...
    @property
    def lpto(self) -> I64Vector4: ...
    @property
    def lptl(self) -> I64Vector4: ...
    @property
    def lpqs(self) -> I64Vector4: ...
    @property
    def lpqt(self) -> I64Vector4: ...
    @property
    def lpqq(self) -> I64Vector4: ...
    @property
    def lpqp(self) -> I64Vector4: ...
    @property
    def lpqo(self) -> I64Vector4: ...
    @property
    def lpql(self) -> I64Vector4: ...
    @property
    def lpps(self) -> I64Vector4: ...
    @property
    def lppt(self) -> I64Vector4: ...
    @property
    def lppq(self) -> I64Vector4: ...
    @property
    def lppp(self) -> I64Vector4: ...
    @property
    def lppo(self) -> I64Vector4: ...
    @property
    def lppl(self) -> I64Vector4: ...
    @property
    def lpos(self) -> I64Vector4: ...
    @property
    def lpot(self) -> I64Vector4: ...
    @property
    def lpoq(self) -> I64Vector4: ...
    @property
    def lpop(self) -> I64Vector4: ...
    @property
    def lpoo(self) -> I64Vector4: ...
    @property
    def lpol(self) -> I64Vector4: ...
    @property
    def lpls(self) -> I64Vector4: ...
    @property
    def lplt(self) -> I64Vector4: ...
    @property
    def lplq(self) -> I64Vector4: ...
    @property
    def lplp(self) -> I64Vector4: ...
    @property
    def lplo(self) -> I64Vector4: ...
    @property
    def lpll(self) -> I64Vector4: ...
    @property
    def loss(self) -> I64Vector4: ...
    @property
    def lost(self) -> I64Vector4: ...
    @property
    def losq(self) -> I64Vector4: ...
    @property
    def losp(self) -> I64Vector4: ...
    @property
    def loso(self) -> I64Vector4: ...
    @property
    def losl(self) -> I64Vector4: ...
    @property
    def lots(self) -> I64Vector4: ...
    @property
    def lott(self) -> I64Vector4: ...
    @property
    def lotq(self) -> I64Vector4: ...
    @property
    def lotp(self) -> I64Vector4: ...
    @property
    def loto(self) -> I64Vector4: ...
    @property
    def lotl(self) -> I64Vector4: ...
    @property
    def loqs(self) -> I64Vector4: ...
    @property
    def loqt(self) -> I64Vector4: ...
    @property
    def loqq(self) -> I64Vector4: ...
    @property
    def loqp(self) -> I64Vector4: ...
    @property
    def loqo(self) -> I64Vector4: ...
    @property
    def loql(self) -> I64Vector4: ...
    @property
    def lops(self) -> I64Vector4: ...
    @property
    def lopt(self) -> I64Vector4: ...
    @property
    def lopq(self) -> I64Vector4: ...
    @property
    def lopp(self) -> I64Vector4: ...
    @property
    def lopo(self) -> I64Vector4: ...
    @property
    def lopl(self) -> I64Vector4: ...
    @property
    def loos(self) -> I64Vector4: ...
    @property
    def loot(self) -> I64Vector4: ...
    @property
    def looq(self) -> I64Vector4: ...
    @property
    def loop(self) -> I64Vector4: ...
    @property
    def lols(self) -> I64Vector4: ...
    @property
    def lolt(self) -> I64Vector4: ...
    @property
    def lolq(self) -> I64Vector4: ...
    @property
    def lolp(self) -> I64Vector4: ...
    @property
    def llss(self) -> I64Vector4: ...
    @property
    def llst(self) -> I64Vector4: ...
    @property
    def llsq(self) -> I64Vector4: ...
    @property
    def llsp(self) -> I64Vector4: ...
    @property
    def llso(self) -> I64Vector4: ...
    @property
    def llsl(self) -> I64Vector4: ...
    @property
    def llts(self) -> I64Vector4: ...
    @property
    def lltt(self) -> I64Vector4: ...
    @property
    def lltq(self) -> I64Vector4: ...
    @property
    def lltp(self) -> I64Vector4: ...
    @property
    def llto(self) -> I64Vector4: ...
    @property
    def lltl(self) -> I64Vector4: ...
    @property
    def llqs(self) -> I64Vector4: ...
    @property
    def llqt(self) -> I64Vector4: ...
    @property
    def llqq(self) -> I64Vector4: ...
    @property
    def llqp(self) -> I64Vector4: ...
    @property
    def llqo(self) -> I64Vector4: ...
    @property
    def llql(self) -> I64Vector4: ...
    @property
    def llps(self) -> I64Vector4: ...
    @property
    def llpt(self) -> I64Vector4: ...
    @property
    def llpq(self) -> I64Vector4: ...
    @property
    def llpp(self) -> I64Vector4: ...
    @property
    def llpo(self) -> I64Vector4: ...
    @property
    def llpl(self) -> I64Vector4: ...
    @property
    def llos(self) -> I64Vector4: ...
    @property
    def llot(self) -> I64Vector4: ...
    @property
    def lloq(self) -> I64Vector4: ...
    @property
    def llop(self) -> I64Vector4: ...
    @property
    def llls(self) -> I64Vector4: ...
    @property
    def lllt(self) -> I64Vector4: ...
    @property
    def lllq(self) -> I64Vector4: ...
    @property
    def lllp(self) -> I64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> I64Vector2: ...
    @property
    def uv(self) -> I64Vector2: ...
    @property
    def uo(self) -> I64Vector2: ...
    @property
    def ul(self) -> I64Vector2: ...
    @property
    def vu(self) -> I64Vector2: ...
    @property
    def vv(self) -> I64Vector2: ...
    @property
    def vo(self) -> I64Vector2: ...
    @property
    def vl(self) -> I64Vector2: ...
    @property
    def ou(self) -> I64Vector2: ...
    @property
    def ov(self) -> I64Vector2: ...
    @property
    def lu(self) -> I64Vector2: ...
    @property
    def lv(self) -> I64Vector2: ...
    @property
    def uuu(self) -> I64Vector3: ...
    @property
    def uuv(self) -> I64Vector3: ...
    @property
    def uuo(self) -> I64Vector3: ...
    @property
    def uul(self) -> I64Vector3: ...
    @property
    def uvu(self) -> I64Vector3: ...
    @property
    def uvv(self) -> I64Vector3: ...
    @property
    def uvo(self) -> I64Vector3: ...
    @property
    def uvl(self) -> I64Vector3: ...
    @property
    def uou(self) -> I64Vector3: ...
    @property
    def uov(self) -> I64Vector3: ...
    @property
    def uoo(self) -> I64Vector3: ...
    @property
    def uol(self) -> I64Vector3: ...
    @property
    def ulu(self) -> I64Vector3: ...
    @property
    def ulv(self) -> I64Vector3: ...
    @property
    def ulo(self) -> I64Vector3: ...
    @property
    def ull(self) -> I64Vector3: ...
    @property
    def vuu(self) -> I64Vector3: ...
    @property
    def vuv(self) -> I64Vector3: ...
    @property
    def vuo(self) -> I64Vector3: ...
    @property
    def vul(self) -> I64Vector3: ...
    @property
    def vvu(self) -> I64Vector3: ...
    @property
    def vvv(self) -> I64Vector3: ...
    @property
    def vvo(self) -> I64Vector3: ...
    @property
    def vvl(self) -> I64Vector3: ...
    @property
    def vou(self) -> I64Vector3: ...
    @property
    def vov(self) -> I64Vector3: ...
    @property
    def voo(self) -> I64Vector3: ...
    @property
    def vol(self) -> I64Vector3: ...
    @property
    def vlu(self) -> I64Vector3: ...
    @property
    def vlv(self) -> I64Vector3: ...
    @property
    def vlo(self) -> I64Vector3: ...
    @property
    def vll(self) -> I64Vector3: ...
    @property
    def ouu(self) -> I64Vector3: ...
    @property
    def ouv(self) -> I64Vector3: ...
    @property
    def ouo(self) -> I64Vector3: ...
    @property
    def oul(self) -> I64Vector3: ...
    @property
    def ovu(self) -> I64Vector3: ...
    @property
    def ovv(self) -> I64Vector3: ...
    @property
    def ovo(self) -> I64Vector3: ...
    @property
    def ovl(self) -> I64Vector3: ...
    @property
    def oou(self) -> I64Vector3: ...
    @property
    def oov(self) -> I64Vector3: ...
    @property
    def olu(self) -> I64Vector3: ...
    @property
    def olv(self) -> I64Vector3: ...
    @property
    def luu(self) -> I64Vector3: ...
    @property
    def luv(self) -> I64Vector3: ...
    @property
    def luo(self) -> I64Vector3: ...
    @property
    def lul(self) -> I64Vector3: ...
    @property
    def lvu(self) -> I64Vector3: ...
    @property
    def lvv(self) -> I64Vector3: ...
    @property
    def lvo(self) -> I64Vector3: ...
    @property
    def lvl(self) -> I64Vector3: ...
    @property
    def lou(self) -> I64Vector3: ...
    @property
    def lov(self) -> I64Vector3: ...
    @property
    def llu(self) -> I64Vector3: ...
    @property
    def llv(self) -> I64Vector3: ...
    @property
    def uuuu(self) -> I64Vector4: ...
    @property
    def uuuv(self) -> I64Vector4: ...
    @property
    def uuuo(self) -> I64Vector4: ...
    @property
    def uuul(self) -> I64Vector4: ...
    @property
    def uuvu(self) -> I64Vector4: ...
    @property
    def uuvv(self) -> I64Vector4: ...
    @property
    def uuvo(self) -> I64Vector4: ...
    @property
    def uuvl(self) -> I64Vector4: ...
    @property
    def uuou(self) -> I64Vector4: ...
    @property
    def uuov(self) -> I64Vector4: ...
    @property
    def uuoo(self) -> I64Vector4: ...
    @property
    def uuol(self) -> I64Vector4: ...
    @property
    def uulu(self) -> I64Vector4: ...
    @property
    def uulv(self) -> I64Vector4: ...
    @property
    def uulo(self) -> I64Vector4: ...
    @property
    def uull(self) -> I64Vector4: ...
    @property
    def uvuu(self) -> I64Vector4: ...
    @property
    def uvuv(self) -> I64Vector4: ...
    @property
    def uvuo(self) -> I64Vector4: ...
    @property
    def uvul(self) -> I64Vector4: ...
    @property
    def uvvu(self) -> I64Vector4: ...
    @property
    def uvvv(self) -> I64Vector4: ...
    @property
    def uvvo(self) -> I64Vector4: ...
    @property
    def uvvl(self) -> I64Vector4: ...
    @property
    def uvou(self) -> I64Vector4: ...
    @property
    def uvov(self) -> I64Vector4: ...
    @property
    def uvoo(self) -> I64Vector4: ...
    @property
    def uvol(self) -> I64Vector4: ...
    @property
    def uvlu(self) -> I64Vector4: ...
    @property
    def uvlv(self) -> I64Vector4: ...
    @property
    def uvlo(self) -> I64Vector4: ...
    @property
    def uvll(self) -> I64Vector4: ...
    @property
    def uouu(self) -> I64Vector4: ...
    @property
    def uouv(self) -> I64Vector4: ...
    @property
    def uouo(self) -> I64Vector4: ...
    @property
    def uoul(self) -> I64Vector4: ...
    @property
    def uovu(self) -> I64Vector4: ...
    @property
    def uovv(self) -> I64Vector4: ...
    @property
    def uovo(self) -> I64Vector4: ...
    @property
    def uovl(self) -> I64Vector4: ...
    @property
    def uoou(self) -> I64Vector4: ...
    @property
    def uoov(self) -> I64Vector4: ...
    @property
    def uooo(self) -> I64Vector4: ...
    @property
    def uool(self) -> I64Vector4: ...
    @property
    def uolu(self) -> I64Vector4: ...
    @property
    def uolv(self) -> I64Vector4: ...
    @property
    def uolo(self) -> I64Vector4: ...
    @property
    def uoll(self) -> I64Vector4: ...
    @property
    def uluu(self) -> I64Vector4: ...
    @property
    def uluv(self) -> I64Vector4: ...
    @property
    def uluo(self) -> I64Vector4: ...
    @property
    def ulul(self) -> I64Vector4: ...
    @property
    def ulvu(self) -> I64Vector4: ...
    @property
    def ulvv(self) -> I64Vector4: ...
    @property
    def ulvo(self) -> I64Vector4: ...
    @property
    def ulvl(self) -> I64Vector4: ...
    @property
    def ulou(self) -> I64Vector4: ...
    @property
    def ulov(self) -> I64Vector4: ...
    @property
    def uloo(self) -> I64Vector4: ...
    @property
    def ulol(self) -> I64Vector4: ...
    @property
    def ullu(self) -> I64Vector4: ...
    @property
    def ullv(self) -> I64Vector4: ...
    @property
    def ullo(self) -> I64Vector4: ...
    @property
    def ulll(self) -> I64Vector4: ...
    @property
    def vuuu(self) -> I64Vector4: ...
    @property
    def vuuv(self) -> I64Vector4: ...
    @property
    def vuuo(self) -> I64Vector4: ...
    @property
    def vuul(self) -> I64Vector4: ...
    @property
    def vuvu(self) -> I64Vector4: ...
    @property
    def vuvv(self) -> I64Vector4: ...
    @property
    def vuvo(self) -> I64Vector4: ...
    @property
    def vuvl(self) -> I64Vector4: ...
    @property
    def vuou(self) -> I64Vector4: ...
    @property
    def vuov(self) -> I64Vector4: ...
    @property
    def vuoo(self) -> I64Vector4: ...
    @property
    def vuol(self) -> I64Vector4: ...
    @property
    def vulu(self) -> I64Vector4: ...
    @property
    def vulv(self) -> I64Vector4: ...
    @property
    def vulo(self) -> I64Vector4: ...
    @property
    def vull(self) -> I64Vector4: ...
    @property
    def vvuu(self) -> I64Vector4: ...
    @property
    def vvuv(self) -> I64Vector4: ...
    @property
    def vvuo(self) -> I64Vector4: ...
    @property
    def vvul(self) -> I64Vector4: ...
    @property
    def vvvu(self) -> I64Vector4: ...
    @property
    def vvvv(self) -> I64Vector4: ...
    @property
    def vvvo(self) -> I64Vector4: ...
    @property
    def vvvl(self) -> I64Vector4: ...
    @property
    def vvou(self) -> I64Vector4: ...
    @property
    def vvov(self) -> I64Vector4: ...
    @property
    def vvoo(self) -> I64Vector4: ...
    @property
    def vvol(self) -> I64Vector4: ...
    @property
    def vvlu(self) -> I64Vector4: ...
    @property
    def vvlv(self) -> I64Vector4: ...
    @property
    def vvlo(self) -> I64Vector4: ...
    @property
    def vvll(self) -> I64Vector4: ...
    @property
    def vouu(self) -> I64Vector4: ...
    @property
    def vouv(self) -> I64Vector4: ...
    @property
    def vouo(self) -> I64Vector4: ...
    @property
    def voul(self) -> I64Vector4: ...
    @property
    def vovu(self) -> I64Vector4: ...
    @property
    def vovv(self) -> I64Vector4: ...
    @property
    def vovo(self) -> I64Vector4: ...
    @property
    def vovl(self) -> I64Vector4: ...
    @property
    def voou(self) -> I64Vector4: ...
    @property
    def voov(self) -> I64Vector4: ...
    @property
    def vooo(self) -> I64Vector4: ...
    @property
    def vool(self) -> I64Vector4: ...
    @property
    def volu(self) -> I64Vector4: ...
    @property
    def volv(self) -> I64Vector4: ...
    @property
    def volo(self) -> I64Vector4: ...
    @property
    def voll(self) -> I64Vector4: ...
    @property
    def vluu(self) -> I64Vector4: ...
    @property
    def vluv(self) -> I64Vector4: ...
    @property
    def vluo(self) -> I64Vector4: ...
    @property
    def vlul(self) -> I64Vector4: ...
    @property
    def vlvu(self) -> I64Vector4: ...
    @property
    def vlvv(self) -> I64Vector4: ...
    @property
    def vlvo(self) -> I64Vector4: ...
    @property
    def vlvl(self) -> I64Vector4: ...
    @property
    def vlou(self) -> I64Vector4: ...
    @property
    def vlov(self) -> I64Vector4: ...
    @property
    def vloo(self) -> I64Vector4: ...
    @property
    def vlol(self) -> I64Vector4: ...
    @property
    def vllu(self) -> I64Vector4: ...
    @property
    def vllv(self) -> I64Vector4: ...
    @property
    def vllo(self) -> I64Vector4: ...
    @property
    def vlll(self) -> I64Vector4: ...
    @property
    def ouuu(self) -> I64Vector4: ...
    @property
    def ouuv(self) -> I64Vector4: ...
    @property
    def ouuo(self) -> I64Vector4: ...
    @property
    def ouul(self) -> I64Vector4: ...
    @property
    def ouvu(self) -> I64Vector4: ...
    @property
    def ouvv(self) -> I64Vector4: ...
    @property
    def ouvo(self) -> I64Vector4: ...
    @property
    def ouvl(self) -> I64Vector4: ...
    @property
    def ouou(self) -> I64Vector4: ...
    @property
    def ouov(self) -> I64Vector4: ...
    @property
    def ouoo(self) -> I64Vector4: ...
    @property
    def ouol(self) -> I64Vector4: ...
    @property
    def oulu(self) -> I64Vector4: ...
    @property
    def oulv(self) -> I64Vector4: ...
    @property
    def oulo(self) -> I64Vector4: ...
    @property
    def oull(self) -> I64Vector4: ...
    @property
    def ovuu(self) -> I64Vector4: ...
    @property
    def ovuv(self) -> I64Vector4: ...
    @property
    def ovuo(self) -> I64Vector4: ...
    @property
    def ovul(self) -> I64Vector4: ...
    @property
    def ovvu(self) -> I64Vector4: ...
    @property
    def ovvv(self) -> I64Vector4: ...
    @property
    def ovvo(self) -> I64Vector4: ...
    @property
    def ovvl(self) -> I64Vector4: ...
    @property
    def ovou(self) -> I64Vector4: ...
    @property
    def ovov(self) -> I64Vector4: ...
    @property
    def ovoo(self) -> I64Vector4: ...
    @property
    def ovol(self) -> I64Vector4: ...
    @property
    def ovlu(self) -> I64Vector4: ...
    @property
    def ovlv(self) -> I64Vector4: ...
    @property
    def ovlo(self) -> I64Vector4: ...
    @property
    def ovll(self) -> I64Vector4: ...
    @property
    def oouu(self) -> I64Vector4: ...
    @property
    def oouv(self) -> I64Vector4: ...
    @property
    def oouo(self) -> I64Vector4: ...
    @property
    def ooul(self) -> I64Vector4: ...
    @property
    def oovu(self) -> I64Vector4: ...
    @property
    def oovv(self) -> I64Vector4: ...
    @property
    def oovo(self) -> I64Vector4: ...
    @property
    def oovl(self) -> I64Vector4: ...
    @property
    def ooou(self) -> I64Vector4: ...
    @property
    def ooov(self) -> I64Vector4: ...
    @property
    def oolu(self) -> I64Vector4: ...
    @property
    def oolv(self) -> I64Vector4: ...
    @property
    def oluu(self) -> I64Vector4: ...
    @property
    def oluv(self) -> I64Vector4: ...
    @property
    def oluo(self) -> I64Vector4: ...
    @property
    def olul(self) -> I64Vector4: ...
    @property
    def olvu(self) -> I64Vector4: ...
    @property
    def olvv(self) -> I64Vector4: ...
    @property
    def olvo(self) -> I64Vector4: ...
    @property
    def olvl(self) -> I64Vector4: ...
    @property
    def olou(self) -> I64Vector4: ...
    @property
    def olov(self) -> I64Vector4: ...
    @property
    def ollu(self) -> I64Vector4: ...
    @property
    def ollv(self) -> I64Vector4: ...
    @property
    def luuu(self) -> I64Vector4: ...
    @property
    def luuv(self) -> I64Vector4: ...
    @property
    def luuo(self) -> I64Vector4: ...
    @property
    def luul(self) -> I64Vector4: ...
    @property
    def luvu(self) -> I64Vector4: ...
    @property
    def luvv(self) -> I64Vector4: ...
    @property
    def luvo(self) -> I64Vector4: ...
    @property
    def luvl(self) -> I64Vector4: ...
    @property
    def luou(self) -> I64Vector4: ...
    @property
    def luov(self) -> I64Vector4: ...
    @property
    def luoo(self) -> I64Vector4: ...
    @property
    def luol(self) -> I64Vector4: ...
    @property
    def lulu(self) -> I64Vector4: ...
    @property
    def lulv(self) -> I64Vector4: ...
    @property
    def lulo(self) -> I64Vector4: ...
    @property
    def lull(self) -> I64Vector4: ...
    @property
    def lvuu(self) -> I64Vector4: ...
    @property
    def lvuv(self) -> I64Vector4: ...
    @property
    def lvuo(self) -> I64Vector4: ...
    @property
    def lvul(self) -> I64Vector4: ...
    @property
    def lvvu(self) -> I64Vector4: ...
    @property
    def lvvv(self) -> I64Vector4: ...
    @property
    def lvvo(self) -> I64Vector4: ...
    @property
    def lvvl(self) -> I64Vector4: ...
    @property
    def lvou(self) -> I64Vector4: ...
    @property
    def lvov(self) -> I64Vector4: ...
    @property
    def lvoo(self) -> I64Vector4: ...
    @property
    def lvol(self) -> I64Vector4: ...
    @property
    def lvlu(self) -> I64Vector4: ...
    @property
    def lvlv(self) -> I64Vector4: ...
    @property
    def lvlo(self) -> I64Vector4: ...
    @property
    def lvll(self) -> I64Vector4: ...
    @property
    def louu(self) -> I64Vector4: ...
    @property
    def louv(self) -> I64Vector4: ...
    @property
    def louo(self) -> I64Vector4: ...
    @property
    def loul(self) -> I64Vector4: ...
    @property
    def lovu(self) -> I64Vector4: ...
    @property
    def lovv(self) -> I64Vector4: ...
    @property
    def lovo(self) -> I64Vector4: ...
    @property
    def lovl(self) -> I64Vector4: ...
    @property
    def loou(self) -> I64Vector4: ...
    @property
    def loov(self) -> I64Vector4: ...
    @property
    def lolu(self) -> I64Vector4: ...
    @property
    def lolv(self) -> I64Vector4: ...
    @property
    def lluu(self) -> I64Vector4: ...
    @property
    def lluv(self) -> I64Vector4: ...
    @property
    def lluo(self) -> I64Vector4: ...
    @property
    def llul(self) -> I64Vector4: ...
    @property
    def llvu(self) -> I64Vector4: ...
    @property
    def llvv(self) -> I64Vector4: ...
    @property
    def llvo(self) -> I64Vector4: ...
    @property
    def llvl(self) -> I64Vector4: ...
    @property
    def llou(self) -> I64Vector4: ...
    @property
    def llov(self) -> I64Vector4: ...
    @property
    def lllu(self) -> I64Vector4: ...
    @property
    def lllv(self) -> I64Vector4: ...
    @property
    def oo(self) -> I64Vector2: ...
    @property
    def ol(self) -> I64Vector2: ...
    @property
    def lo(self) -> I64Vector2: ...
    @property
    def ll(self) -> I64Vector2: ...
    @property
    def ooo(self) -> I64Vector3: ...
    @property
    def ool(self) -> I64Vector3: ...
    @property
    def olo(self) -> I64Vector3: ...
    @property
    def oll(self) -> I64Vector3: ...
    @property
    def loo(self) -> I64Vector3: ...
    @property
    def lol(self) -> I64Vector3: ...
    @property
    def llo(self) -> I64Vector3: ...
    @property
    def lll(self) -> I64Vector3: ...
    @property
    def oooo(self) -> I64Vector4: ...
    @property
    def oool(self) -> I64Vector4: ...
    @property
    def oolo(self) -> I64Vector4: ...
    @property
    def ooll(self) -> I64Vector4: ...
    @property
    def oloo(self) -> I64Vector4: ...
    @property
    def olol(self) -> I64Vector4: ...
    @property
    def ollo(self) -> I64Vector4: ...
    @property
    def olll(self) -> I64Vector4: ...
    @property
    def looo(self) -> I64Vector4: ...
    @property
    def lool(self) -> I64Vector4: ...
    @property
    def lolo(self) -> I64Vector4: ...
    @property
    def loll(self) -> I64Vector4: ...
    @property
    def lloo(self) -> I64Vector4: ...
    @property
    def llol(self) -> I64Vector4: ...
    @property
    def lllo(self) -> I64Vector4: ...
    @property
    def llll(self) -> I64Vector4: ...
    def min(self, n: Number, /) -> I64Vector4: ...
    def max(self, n: Number, /) -> I64Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> I64Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[I64Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector4: ...

@final
class I64Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: I64Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[I64Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> I64Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> I64Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> I64Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[I64Vector4]: ...

@final
class U64Vector4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __iter__(self) -> Iterator[int]: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: int) -> int: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __lt__(self, other: Any) -> _bool: ...
    def __le__(self, other: Any) -> _bool: ...
    def __gt__(self, other: Any) -> _bool: ...
    def __ge__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __add__(self, other: Number) -> U64Vector4: ...
    @overload
    def __radd__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __radd__(self, other: Number) -> U64Vector4: ...
    @overload
    def __sub__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __sub__(self, other: Number) -> U64Vector4: ...
    @overload
    def __rsub__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __rsub__(self, other: Number) -> U64Vector4: ...
    @overload
    def __mul__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __mul__(self, other: Number) -> U64Vector4: ...
    @overload
    def __rmul__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __rmul__(self, other: Number) -> U64Vector4: ...
    @overload
    def __truediv__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __truediv__(self, other: Number) -> U64Vector4: ...
    @overload
    def __rtruediv__(self, other: U64Vector4) -> U64Vector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> U64Vector4: ...
    def __abs__(self) -> U64Vector4: ...
    def __bool__(self) -> _bool: ...
    @property
    def x(self) -> int: ...
    @property
    def y(self) -> int: ...
    @property
    def z(self) -> int: ...
    @property
    def w(self) -> int: ...
    @property
    def xx(self) -> U64Vector2: ...
    @property
    def xy(self) -> U64Vector2: ...
    @property
    def xz(self) -> U64Vector2: ...
    @property
    def xw(self) -> U64Vector2: ...
    @property
    def xo(self) -> U64Vector2: ...
    @property
    def xl(self) -> U64Vector2: ...
    @property
    def yx(self) -> U64Vector2: ...
    @property
    def yy(self) -> U64Vector2: ...
    @property
    def yz(self) -> U64Vector2: ...
    @property
    def yw(self) -> U64Vector2: ...
    @property
    def yo(self) -> U64Vector2: ...
    @property
    def yl(self) -> U64Vector2: ...
    @property
    def zx(self) -> U64Vector2: ...
    @property
    def zy(self) -> U64Vector2: ...
    @property
    def zz(self) -> U64Vector2: ...
    @property
    def zw(self) -> U64Vector2: ...
    @property
    def zo(self) -> U64Vector2: ...
    @property
    def zl(self) -> U64Vector2: ...
    @property
    def wx(self) -> U64Vector2: ...
    @property
    def wy(self) -> U64Vector2: ...
    @property
    def wz(self) -> U64Vector2: ...
    @property
    def ww(self) -> U64Vector2: ...
    @property
    def wo(self) -> U64Vector2: ...
    @property
    def wl(self) -> U64Vector2: ...
    @property
    def ox(self) -> U64Vector2: ...
    @property
    def oy(self) -> U64Vector2: ...
    @property
    def oz(self) -> U64Vector2: ...
    @property
    def ow(self) -> U64Vector2: ...
    @property
    def lx(self) -> U64Vector2: ...
    @property
    def ly(self) -> U64Vector2: ...
    @property
    def lz(self) -> U64Vector2: ...
    @property
    def lw(self) -> U64Vector2: ...
    @property
    def xxx(self) -> U64Vector3: ...
    @property
    def xxy(self) -> U64Vector3: ...
    @property
    def xxz(self) -> U64Vector3: ...
    @property
    def xxw(self) -> U64Vector3: ...
    @property
    def xxo(self) -> U64Vector3: ...
    @property
    def xxl(self) -> U64Vector3: ...
    @property
    def xyx(self) -> U64Vector3: ...
    @property
    def xyy(self) -> U64Vector3: ...
    @property
    def xyz(self) -> U64Vector3: ...
    @property
    def xyw(self) -> U64Vector3: ...
    @property
    def xyo(self) -> U64Vector3: ...
    @property
    def xyl(self) -> U64Vector3: ...
    @property
    def xzx(self) -> U64Vector3: ...
    @property
    def xzy(self) -> U64Vector3: ...
    @property
    def xzz(self) -> U64Vector3: ...
    @property
    def xzw(self) -> U64Vector3: ...
    @property
    def xzo(self) -> U64Vector3: ...
    @property
    def xzl(self) -> U64Vector3: ...
    @property
    def xwx(self) -> U64Vector3: ...
    @property
    def xwy(self) -> U64Vector3: ...
    @property
    def xwz(self) -> U64Vector3: ...
    @property
    def xww(self) -> U64Vector3: ...
    @property
    def xwo(self) -> U64Vector3: ...
    @property
    def xwl(self) -> U64Vector3: ...
    @property
    def xox(self) -> U64Vector3: ...
    @property
    def xoy(self) -> U64Vector3: ...
    @property
    def xoz(self) -> U64Vector3: ...
    @property
    def xow(self) -> U64Vector3: ...
    @property
    def xoo(self) -> U64Vector3: ...
    @property
    def xol(self) -> U64Vector3: ...
    @property
    def xlx(self) -> U64Vector3: ...
    @property
    def xly(self) -> U64Vector3: ...
    @property
    def xlz(self) -> U64Vector3: ...
    @property
    def xlw(self) -> U64Vector3: ...
    @property
    def xlo(self) -> U64Vector3: ...
    @property
    def xll(self) -> U64Vector3: ...
    @property
    def yxx(self) -> U64Vector3: ...
    @property
    def yxy(self) -> U64Vector3: ...
    @property
    def yxz(self) -> U64Vector3: ...
    @property
    def yxw(self) -> U64Vector3: ...
    @property
    def yxo(self) -> U64Vector3: ...
    @property
    def yxl(self) -> U64Vector3: ...
    @property
    def yyx(self) -> U64Vector3: ...
    @property
    def yyy(self) -> U64Vector3: ...
    @property
    def yyz(self) -> U64Vector3: ...
    @property
    def yyw(self) -> U64Vector3: ...
    @property
    def yyo(self) -> U64Vector3: ...
    @property
    def yyl(self) -> U64Vector3: ...
    @property
    def yzx(self) -> U64Vector3: ...
    @property
    def yzy(self) -> U64Vector3: ...
    @property
    def yzz(self) -> U64Vector3: ...
    @property
    def yzw(self) -> U64Vector3: ...
    @property
    def yzo(self) -> U64Vector3: ...
    @property
    def yzl(self) -> U64Vector3: ...
    @property
    def ywx(self) -> U64Vector3: ...
    @property
    def ywy(self) -> U64Vector3: ...
    @property
    def ywz(self) -> U64Vector3: ...
    @property
    def yww(self) -> U64Vector3: ...
    @property
    def ywo(self) -> U64Vector3: ...
    @property
    def ywl(self) -> U64Vector3: ...
    @property
    def yox(self) -> U64Vector3: ...
    @property
    def yoy(self) -> U64Vector3: ...
    @property
    def yoz(self) -> U64Vector3: ...
    @property
    def yow(self) -> U64Vector3: ...
    @property
    def yoo(self) -> U64Vector3: ...
    @property
    def yol(self) -> U64Vector3: ...
    @property
    def ylx(self) -> U64Vector3: ...
    @property
    def yly(self) -> U64Vector3: ...
    @property
    def ylz(self) -> U64Vector3: ...
    @property
    def ylw(self) -> U64Vector3: ...
    @property
    def ylo(self) -> U64Vector3: ...
    @property
    def yll(self) -> U64Vector3: ...
    @property
    def zxx(self) -> U64Vector3: ...
    @property
    def zxy(self) -> U64Vector3: ...
    @property
    def zxz(self) -> U64Vector3: ...
    @property
    def zxw(self) -> U64Vector3: ...
    @property
    def zxo(self) -> U64Vector3: ...
    @property
    def zxl(self) -> U64Vector3: ...
    @property
    def zyx(self) -> U64Vector3: ...
    @property
    def zyy(self) -> U64Vector3: ...
    @property
    def zyz(self) -> U64Vector3: ...
    @property
    def zyw(self) -> U64Vector3: ...
    @property
    def zyo(self) -> U64Vector3: ...
    @property
    def zyl(self) -> U64Vector3: ...
    @property
    def zzx(self) -> U64Vector3: ...
    @property
    def zzy(self) -> U64Vector3: ...
    @property
    def zzz(self) -> U64Vector3: ...
    @property
    def zzw(self) -> U64Vector3: ...
    @property
    def zzo(self) -> U64Vector3: ...
    @property
    def zzl(self) -> U64Vector3: ...
    @property
    def zwx(self) -> U64Vector3: ...
    @property
    def zwy(self) -> U64Vector3: ...
    @property
    def zwz(self) -> U64Vector3: ...
    @property
    def zww(self) -> U64Vector3: ...
    @property
    def zwo(self) -> U64Vector3: ...
    @property
    def zwl(self) -> U64Vector3: ...
    @property
    def zox(self) -> U64Vector3: ...
    @property
    def zoy(self) -> U64Vector3: ...
    @property
    def zoz(self) -> U64Vector3: ...
    @property
    def zow(self) -> U64Vector3: ...
    @property
    def zoo(self) -> U64Vector3: ...
    @property
    def zol(self) -> U64Vector3: ...
    @property
    def zlx(self) -> U64Vector3: ...
    @property
    def zly(self) -> U64Vector3: ...
    @property
    def zlz(self) -> U64Vector3: ...
    @property
    def zlw(self) -> U64Vector3: ...
    @property
    def zlo(self) -> U64Vector3: ...
    @property
    def zll(self) -> U64Vector3: ...
    @property
    def wxx(self) -> U64Vector3: ...
    @property
    def wxy(self) -> U64Vector3: ...
    @property
    def wxz(self) -> U64Vector3: ...
    @property
    def wxw(self) -> U64Vector3: ...
    @property
    def wxo(self) -> U64Vector3: ...
    @property
    def wxl(self) -> U64Vector3: ...
    @property
    def wyx(self) -> U64Vector3: ...
    @property
    def wyy(self) -> U64Vector3: ...
    @property
    def wyz(self) -> U64Vector3: ...
    @property
    def wyw(self) -> U64Vector3: ...
    @property
    def wyo(self) -> U64Vector3: ...
    @property
    def wyl(self) -> U64Vector3: ...
    @property
    def wzx(self) -> U64Vector3: ...
    @property
    def wzy(self) -> U64Vector3: ...
    @property
    def wzz(self) -> U64Vector3: ...
    @property
    def wzw(self) -> U64Vector3: ...
    @property
    def wzo(self) -> U64Vector3: ...
    @property
    def wzl(self) -> U64Vector3: ...
    @property
    def wwx(self) -> U64Vector3: ...
    @property
    def wwy(self) -> U64Vector3: ...
    @property
    def wwz(self) -> U64Vector3: ...
    @property
    def www(self) -> U64Vector3: ...
    @property
    def wwo(self) -> U64Vector3: ...
    @property
    def wwl(self) -> U64Vector3: ...
    @property
    def wox(self) -> U64Vector3: ...
    @property
    def woy(self) -> U64Vector3: ...
    @property
    def woz(self) -> U64Vector3: ...
    @property
    def wow(self) -> U64Vector3: ...
    @property
    def woo(self) -> U64Vector3: ...
    @property
    def wol(self) -> U64Vector3: ...
    @property
    def wlx(self) -> U64Vector3: ...
    @property
    def wly(self) -> U64Vector3: ...
    @property
    def wlz(self) -> U64Vector3: ...
    @property
    def wlw(self) -> U64Vector3: ...
    @property
    def wlo(self) -> U64Vector3: ...
    @property
    def wll(self) -> U64Vector3: ...
    @property
    def oxx(self) -> U64Vector3: ...
    @property
    def oxy(self) -> U64Vector3: ...
    @property
    def oxz(self) -> U64Vector3: ...
    @property
    def oxw(self) -> U64Vector3: ...
    @property
    def oxo(self) -> U64Vector3: ...
    @property
    def oxl(self) -> U64Vector3: ...
    @property
    def oyx(self) -> U64Vector3: ...
    @property
    def oyy(self) -> U64Vector3: ...
    @property
    def oyz(self) -> U64Vector3: ...
    @property
    def oyw(self) -> U64Vector3: ...
    @property
    def oyo(self) -> U64Vector3: ...
    @property
    def oyl(self) -> U64Vector3: ...
    @property
    def ozx(self) -> U64Vector3: ...
    @property
    def ozy(self) -> U64Vector3: ...
    @property
    def ozz(self) -> U64Vector3: ...
    @property
    def ozw(self) -> U64Vector3: ...
    @property
    def ozo(self) -> U64Vector3: ...
    @property
    def ozl(self) -> U64Vector3: ...
    @property
    def owx(self) -> U64Vector3: ...
    @property
    def owy(self) -> U64Vector3: ...
    @property
    def owz(self) -> U64Vector3: ...
    @property
    def oww(self) -> U64Vector3: ...
    @property
    def owo(self) -> U64Vector3: ...
    @property
    def owl(self) -> U64Vector3: ...
    @property
    def oox(self) -> U64Vector3: ...
    @property
    def ooy(self) -> U64Vector3: ...
    @property
    def ooz(self) -> U64Vector3: ...
    @property
    def oow(self) -> U64Vector3: ...
    @property
    def olx(self) -> U64Vector3: ...
    @property
    def oly(self) -> U64Vector3: ...
    @property
    def olz(self) -> U64Vector3: ...
    @property
    def olw(self) -> U64Vector3: ...
    @property
    def lxx(self) -> U64Vector3: ...
    @property
    def lxy(self) -> U64Vector3: ...
    @property
    def lxz(self) -> U64Vector3: ...
    @property
    def lxw(self) -> U64Vector3: ...
    @property
    def lxo(self) -> U64Vector3: ...
    @property
    def lxl(self) -> U64Vector3: ...
    @property
    def lyx(self) -> U64Vector3: ...
    @property
    def lyy(self) -> U64Vector3: ...
    @property
    def lyz(self) -> U64Vector3: ...
    @property
    def lyw(self) -> U64Vector3: ...
    @property
    def lyo(self) -> U64Vector3: ...
    @property
    def lyl(self) -> U64Vector3: ...
    @property
    def lzx(self) -> U64Vector3: ...
    @property
    def lzy(self) -> U64Vector3: ...
    @property
    def lzz(self) -> U64Vector3: ...
    @property
    def lzw(self) -> U64Vector3: ...
    @property
    def lzo(self) -> U64Vector3: ...
    @property
    def lzl(self) -> U64Vector3: ...
    @property
    def lwx(self) -> U64Vector3: ...
    @property
    def lwy(self) -> U64Vector3: ...
    @property
    def lwz(self) -> U64Vector3: ...
    @property
    def lww(self) -> U64Vector3: ...
    @property
    def lwo(self) -> U64Vector3: ...
    @property
    def lwl(self) -> U64Vector3: ...
    @property
    def lox(self) -> U64Vector3: ...
    @property
    def loy(self) -> U64Vector3: ...
    @property
    def loz(self) -> U64Vector3: ...
    @property
    def low(self) -> U64Vector3: ...
    @property
    def llx(self) -> U64Vector3: ...
    @property
    def lly(self) -> U64Vector3: ...
    @property
    def llz(self) -> U64Vector3: ...
    @property
    def llw(self) -> U64Vector3: ...
    @property
    def xxxx(self) -> U64Vector4: ...
    @property
    def xxxy(self) -> U64Vector4: ...
    @property
    def xxxz(self) -> U64Vector4: ...
    @property
    def xxxw(self) -> U64Vector4: ...
    @property
    def xxxo(self) -> U64Vector4: ...
    @property
    def xxxl(self) -> U64Vector4: ...
    @property
    def xxyx(self) -> U64Vector4: ...
    @property
    def xxyy(self) -> U64Vector4: ...
    @property
    def xxyz(self) -> U64Vector4: ...
    @property
    def xxyw(self) -> U64Vector4: ...
    @property
    def xxyo(self) -> U64Vector4: ...
    @property
    def xxyl(self) -> U64Vector4: ...
    @property
    def xxzx(self) -> U64Vector4: ...
    @property
    def xxzy(self) -> U64Vector4: ...
    @property
    def xxzz(self) -> U64Vector4: ...
    @property
    def xxzw(self) -> U64Vector4: ...
    @property
    def xxzo(self) -> U64Vector4: ...
    @property
    def xxzl(self) -> U64Vector4: ...
    @property
    def xxwx(self) -> U64Vector4: ...
    @property
    def xxwy(self) -> U64Vector4: ...
    @property
    def xxwz(self) -> U64Vector4: ...
    @property
    def xxww(self) -> U64Vector4: ...
    @property
    def xxwo(self) -> U64Vector4: ...
    @property
    def xxwl(self) -> U64Vector4: ...
    @property
    def xxox(self) -> U64Vector4: ...
    @property
    def xxoy(self) -> U64Vector4: ...
    @property
    def xxoz(self) -> U64Vector4: ...
    @property
    def xxow(self) -> U64Vector4: ...
    @property
    def xxoo(self) -> U64Vector4: ...
    @property
    def xxol(self) -> U64Vector4: ...
    @property
    def xxlx(self) -> U64Vector4: ...
    @property
    def xxly(self) -> U64Vector4: ...
    @property
    def xxlz(self) -> U64Vector4: ...
    @property
    def xxlw(self) -> U64Vector4: ...
    @property
    def xxlo(self) -> U64Vector4: ...
    @property
    def xxll(self) -> U64Vector4: ...
    @property
    def xyxx(self) -> U64Vector4: ...
    @property
    def xyxy(self) -> U64Vector4: ...
    @property
    def xyxz(self) -> U64Vector4: ...
    @property
    def xyxw(self) -> U64Vector4: ...
    @property
    def xyxo(self) -> U64Vector4: ...
    @property
    def xyxl(self) -> U64Vector4: ...
    @property
    def xyyx(self) -> U64Vector4: ...
    @property
    def xyyy(self) -> U64Vector4: ...
    @property
    def xyyz(self) -> U64Vector4: ...
    @property
    def xyyw(self) -> U64Vector4: ...
    @property
    def xyyo(self) -> U64Vector4: ...
    @property
    def xyyl(self) -> U64Vector4: ...
    @property
    def xyzx(self) -> U64Vector4: ...
    @property
    def xyzy(self) -> U64Vector4: ...
    @property
    def xyzz(self) -> U64Vector4: ...
    @property
    def xyzw(self) -> U64Vector4: ...
    @property
    def xyzo(self) -> U64Vector4: ...
    @property
    def xyzl(self) -> U64Vector4: ...
    @property
    def xywx(self) -> U64Vector4: ...
    @property
    def xywy(self) -> U64Vector4: ...
    @property
    def xywz(self) -> U64Vector4: ...
    @property
    def xyww(self) -> U64Vector4: ...
    @property
    def xywo(self) -> U64Vector4: ...
    @property
    def xywl(self) -> U64Vector4: ...
    @property
    def xyox(self) -> U64Vector4: ...
    @property
    def xyoy(self) -> U64Vector4: ...
    @property
    def xyoz(self) -> U64Vector4: ...
    @property
    def xyow(self) -> U64Vector4: ...
    @property
    def xyoo(self) -> U64Vector4: ...
    @property
    def xyol(self) -> U64Vector4: ...
    @property
    def xylx(self) -> U64Vector4: ...
    @property
    def xyly(self) -> U64Vector4: ...
    @property
    def xylz(self) -> U64Vector4: ...
    @property
    def xylw(self) -> U64Vector4: ...
    @property
    def xylo(self) -> U64Vector4: ...
    @property
    def xyll(self) -> U64Vector4: ...
    @property
    def xzxx(self) -> U64Vector4: ...
    @property
    def xzxy(self) -> U64Vector4: ...
    @property
    def xzxz(self) -> U64Vector4: ...
    @property
    def xzxw(self) -> U64Vector4: ...
    @property
    def xzxo(self) -> U64Vector4: ...
    @property
    def xzxl(self) -> U64Vector4: ...
    @property
    def xzyx(self) -> U64Vector4: ...
    @property
    def xzyy(self) -> U64Vector4: ...
    @property
    def xzyz(self) -> U64Vector4: ...
    @property
    def xzyw(self) -> U64Vector4: ...
    @property
    def xzyo(self) -> U64Vector4: ...
    @property
    def xzyl(self) -> U64Vector4: ...
    @property
    def xzzx(self) -> U64Vector4: ...
    @property
    def xzzy(self) -> U64Vector4: ...
    @property
    def xzzz(self) -> U64Vector4: ...
    @property
    def xzzw(self) -> U64Vector4: ...
    @property
    def xzzo(self) -> U64Vector4: ...
    @property
    def xzzl(self) -> U64Vector4: ...
    @property
    def xzwx(self) -> U64Vector4: ...
    @property
    def xzwy(self) -> U64Vector4: ...
    @property
    def xzwz(self) -> U64Vector4: ...
    @property
    def xzww(self) -> U64Vector4: ...
    @property
    def xzwo(self) -> U64Vector4: ...
    @property
    def xzwl(self) -> U64Vector4: ...
    @property
    def xzox(self) -> U64Vector4: ...
    @property
    def xzoy(self) -> U64Vector4: ...
    @property
    def xzoz(self) -> U64Vector4: ...
    @property
    def xzow(self) -> U64Vector4: ...
    @property
    def xzoo(self) -> U64Vector4: ...
    @property
    def xzol(self) -> U64Vector4: ...
    @property
    def xzlx(self) -> U64Vector4: ...
    @property
    def xzly(self) -> U64Vector4: ...
    @property
    def xzlz(self) -> U64Vector4: ...
    @property
    def xzlw(self) -> U64Vector4: ...
    @property
    def xzlo(self) -> U64Vector4: ...
    @property
    def xzll(self) -> U64Vector4: ...
    @property
    def xwxx(self) -> U64Vector4: ...
    @property
    def xwxy(self) -> U64Vector4: ...
    @property
    def xwxz(self) -> U64Vector4: ...
    @property
    def xwxw(self) -> U64Vector4: ...
    @property
    def xwxo(self) -> U64Vector4: ...
    @property
    def xwxl(self) -> U64Vector4: ...
    @property
    def xwyx(self) -> U64Vector4: ...
    @property
    def xwyy(self) -> U64Vector4: ...
    @property
    def xwyz(self) -> U64Vector4: ...
    @property
    def xwyw(self) -> U64Vector4: ...
    @property
    def xwyo(self) -> U64Vector4: ...
    @property
    def xwyl(self) -> U64Vector4: ...
    @property
    def xwzx(self) -> U64Vector4: ...
    @property
    def xwzy(self) -> U64Vector4: ...
    @property
    def xwzz(self) -> U64Vector4: ...
    @property
    def xwzw(self) -> U64Vector4: ...
    @property
    def xwzo(self) -> U64Vector4: ...
    @property
    def xwzl(self) -> U64Vector4: ...
    @property
    def xwwx(self) -> U64Vector4: ...
    @property
    def xwwy(self) -> U64Vector4: ...
    @property
    def xwwz(self) -> U64Vector4: ...
    @property
    def xwww(self) -> U64Vector4: ...
    @property
    def xwwo(self) -> U64Vector4: ...
    @property
    def xwwl(self) -> U64Vector4: ...
    @property
    def xwox(self) -> U64Vector4: ...
    @property
    def xwoy(self) -> U64Vector4: ...
    @property
    def xwoz(self) -> U64Vector4: ...
    @property
    def xwow(self) -> U64Vector4: ...
    @property
    def xwoo(self) -> U64Vector4: ...
    @property
    def xwol(self) -> U64Vector4: ...
    @property
    def xwlx(self) -> U64Vector4: ...
    @property
    def xwly(self) -> U64Vector4: ...
    @property
    def xwlz(self) -> U64Vector4: ...
    @property
    def xwlw(self) -> U64Vector4: ...
    @property
    def xwlo(self) -> U64Vector4: ...
    @property
    def xwll(self) -> U64Vector4: ...
    @property
    def xoxx(self) -> U64Vector4: ...
    @property
    def xoxy(self) -> U64Vector4: ...
    @property
    def xoxz(self) -> U64Vector4: ...
    @property
    def xoxw(self) -> U64Vector4: ...
    @property
    def xoxo(self) -> U64Vector4: ...
    @property
    def xoxl(self) -> U64Vector4: ...
    @property
    def xoyx(self) -> U64Vector4: ...
    @property
    def xoyy(self) -> U64Vector4: ...
    @property
    def xoyz(self) -> U64Vector4: ...
    @property
    def xoyw(self) -> U64Vector4: ...
    @property
    def xoyo(self) -> U64Vector4: ...
    @property
    def xoyl(self) -> U64Vector4: ...
    @property
    def xozx(self) -> U64Vector4: ...
    @property
    def xozy(self) -> U64Vector4: ...
    @property
    def xozz(self) -> U64Vector4: ...
    @property
    def xozw(self) -> U64Vector4: ...
    @property
    def xozo(self) -> U64Vector4: ...
    @property
    def xozl(self) -> U64Vector4: ...
    @property
    def xowx(self) -> U64Vector4: ...
    @property
    def xowy(self) -> U64Vector4: ...
    @property
    def xowz(self) -> U64Vector4: ...
    @property
    def xoww(self) -> U64Vector4: ...
    @property
    def xowo(self) -> U64Vector4: ...
    @property
    def xowl(self) -> U64Vector4: ...
    @property
    def xoox(self) -> U64Vector4: ...
    @property
    def xooy(self) -> U64Vector4: ...
    @property
    def xooz(self) -> U64Vector4: ...
    @property
    def xoow(self) -> U64Vector4: ...
    @property
    def xooo(self) -> U64Vector4: ...
    @property
    def xool(self) -> U64Vector4: ...
    @property
    def xolx(self) -> U64Vector4: ...
    @property
    def xoly(self) -> U64Vector4: ...
    @property
    def xolz(self) -> U64Vector4: ...
    @property
    def xolw(self) -> U64Vector4: ...
    @property
    def xolo(self) -> U64Vector4: ...
    @property
    def xoll(self) -> U64Vector4: ...
    @property
    def xlxx(self) -> U64Vector4: ...
    @property
    def xlxy(self) -> U64Vector4: ...
    @property
    def xlxz(self) -> U64Vector4: ...
    @property
    def xlxw(self) -> U64Vector4: ...
    @property
    def xlxo(self) -> U64Vector4: ...
    @property
    def xlxl(self) -> U64Vector4: ...
    @property
    def xlyx(self) -> U64Vector4: ...
    @property
    def xlyy(self) -> U64Vector4: ...
    @property
    def xlyz(self) -> U64Vector4: ...
    @property
    def xlyw(self) -> U64Vector4: ...
    @property
    def xlyo(self) -> U64Vector4: ...
    @property
    def xlyl(self) -> U64Vector4: ...
    @property
    def xlzx(self) -> U64Vector4: ...
    @property
    def xlzy(self) -> U64Vector4: ...
    @property
    def xlzz(self) -> U64Vector4: ...
    @property
    def xlzw(self) -> U64Vector4: ...
    @property
    def xlzo(self) -> U64Vector4: ...
    @property
    def xlzl(self) -> U64Vector4: ...
    @property
    def xlwx(self) -> U64Vector4: ...
    @property
    def xlwy(self) -> U64Vector4: ...
    @property
    def xlwz(self) -> U64Vector4: ...
    @property
    def xlww(self) -> U64Vector4: ...
    @property
    def xlwo(self) -> U64Vector4: ...
    @property
    def xlwl(self) -> U64Vector4: ...
    @property
    def xlox(self) -> U64Vector4: ...
    @property
    def xloy(self) -> U64Vector4: ...
    @property
    def xloz(self) -> U64Vector4: ...
    @property
    def xlow(self) -> U64Vector4: ...
    @property
    def xloo(self) -> U64Vector4: ...
    @property
    def xlol(self) -> U64Vector4: ...
    @property
    def xllx(self) -> U64Vector4: ...
    @property
    def xlly(self) -> U64Vector4: ...
    @property
    def xllz(self) -> U64Vector4: ...
    @property
    def xllw(self) -> U64Vector4: ...
    @property
    def xllo(self) -> U64Vector4: ...
    @property
    def xlll(self) -> U64Vector4: ...
    @property
    def yxxx(self) -> U64Vector4: ...
    @property
    def yxxy(self) -> U64Vector4: ...
    @property
    def yxxz(self) -> U64Vector4: ...
    @property
    def yxxw(self) -> U64Vector4: ...
    @property
    def yxxo(self) -> U64Vector4: ...
    @property
    def yxxl(self) -> U64Vector4: ...
    @property
    def yxyx(self) -> U64Vector4: ...
    @property
    def yxyy(self) -> U64Vector4: ...
    @property
    def yxyz(self) -> U64Vector4: ...
    @property
    def yxyw(self) -> U64Vector4: ...
    @property
    def yxyo(self) -> U64Vector4: ...
    @property
    def yxyl(self) -> U64Vector4: ...
    @property
    def yxzx(self) -> U64Vector4: ...
    @property
    def yxzy(self) -> U64Vector4: ...
    @property
    def yxzz(self) -> U64Vector4: ...
    @property
    def yxzw(self) -> U64Vector4: ...
    @property
    def yxzo(self) -> U64Vector4: ...
    @property
    def yxzl(self) -> U64Vector4: ...
    @property
    def yxwx(self) -> U64Vector4: ...
    @property
    def yxwy(self) -> U64Vector4: ...
    @property
    def yxwz(self) -> U64Vector4: ...
    @property
    def yxww(self) -> U64Vector4: ...
    @property
    def yxwo(self) -> U64Vector4: ...
    @property
    def yxwl(self) -> U64Vector4: ...
    @property
    def yxox(self) -> U64Vector4: ...
    @property
    def yxoy(self) -> U64Vector4: ...
    @property
    def yxoz(self) -> U64Vector4: ...
    @property
    def yxow(self) -> U64Vector4: ...
    @property
    def yxoo(self) -> U64Vector4: ...
    @property
    def yxol(self) -> U64Vector4: ...
    @property
    def yxlx(self) -> U64Vector4: ...
    @property
    def yxly(self) -> U64Vector4: ...
    @property
    def yxlz(self) -> U64Vector4: ...
    @property
    def yxlw(self) -> U64Vector4: ...
    @property
    def yxlo(self) -> U64Vector4: ...
    @property
    def yxll(self) -> U64Vector4: ...
    @property
    def yyxx(self) -> U64Vector4: ...
    @property
    def yyxy(self) -> U64Vector4: ...
    @property
    def yyxz(self) -> U64Vector4: ...
    @property
    def yyxw(self) -> U64Vector4: ...
    @property
    def yyxo(self) -> U64Vector4: ...
    @property
    def yyxl(self) -> U64Vector4: ...
    @property
    def yyyx(self) -> U64Vector4: ...
    @property
    def yyyy(self) -> U64Vector4: ...
    @property
    def yyyz(self) -> U64Vector4: ...
    @property
    def yyyw(self) -> U64Vector4: ...
    @property
    def yyyo(self) -> U64Vector4: ...
    @property
    def yyyl(self) -> U64Vector4: ...
    @property
    def yyzx(self) -> U64Vector4: ...
    @property
    def yyzy(self) -> U64Vector4: ...
    @property
    def yyzz(self) -> U64Vector4: ...
    @property
    def yyzw(self) -> U64Vector4: ...
    @property
    def yyzo(self) -> U64Vector4: ...
    @property
    def yyzl(self) -> U64Vector4: ...
    @property
    def yywx(self) -> U64Vector4: ...
    @property
    def yywy(self) -> U64Vector4: ...
    @property
    def yywz(self) -> U64Vector4: ...
    @property
    def yyww(self) -> U64Vector4: ...
    @property
    def yywo(self) -> U64Vector4: ...
    @property
    def yywl(self) -> U64Vector4: ...
    @property
    def yyox(self) -> U64Vector4: ...
    @property
    def yyoy(self) -> U64Vector4: ...
    @property
    def yyoz(self) -> U64Vector4: ...
    @property
    def yyow(self) -> U64Vector4: ...
    @property
    def yyoo(self) -> U64Vector4: ...
    @property
    def yyol(self) -> U64Vector4: ...
    @property
    def yylx(self) -> U64Vector4: ...
    @property
    def yyly(self) -> U64Vector4: ...
    @property
    def yylz(self) -> U64Vector4: ...
    @property
    def yylw(self) -> U64Vector4: ...
    @property
    def yylo(self) -> U64Vector4: ...
    @property
    def yyll(self) -> U64Vector4: ...
    @property
    def yzxx(self) -> U64Vector4: ...
    @property
    def yzxy(self) -> U64Vector4: ...
    @property
    def yzxz(self) -> U64Vector4: ...
    @property
    def yzxw(self) -> U64Vector4: ...
    @property
    def yzxo(self) -> U64Vector4: ...
    @property
    def yzxl(self) -> U64Vector4: ...
    @property
    def yzyx(self) -> U64Vector4: ...
    @property
    def yzyy(self) -> U64Vector4: ...
    @property
    def yzyz(self) -> U64Vector4: ...
    @property
    def yzyw(self) -> U64Vector4: ...
    @property
    def yzyo(self) -> U64Vector4: ...
    @property
    def yzyl(self) -> U64Vector4: ...
    @property
    def yzzx(self) -> U64Vector4: ...
    @property
    def yzzy(self) -> U64Vector4: ...
    @property
    def yzzz(self) -> U64Vector4: ...
    @property
    def yzzw(self) -> U64Vector4: ...
    @property
    def yzzo(self) -> U64Vector4: ...
    @property
    def yzzl(self) -> U64Vector4: ...
    @property
    def yzwx(self) -> U64Vector4: ...
    @property
    def yzwy(self) -> U64Vector4: ...
    @property
    def yzwz(self) -> U64Vector4: ...
    @property
    def yzww(self) -> U64Vector4: ...
    @property
    def yzwo(self) -> U64Vector4: ...
    @property
    def yzwl(self) -> U64Vector4: ...
    @property
    def yzox(self) -> U64Vector4: ...
    @property
    def yzoy(self) -> U64Vector4: ...
    @property
    def yzoz(self) -> U64Vector4: ...
    @property
    def yzow(self) -> U64Vector4: ...
    @property
    def yzoo(self) -> U64Vector4: ...
    @property
    def yzol(self) -> U64Vector4: ...
    @property
    def yzlx(self) -> U64Vector4: ...
    @property
    def yzly(self) -> U64Vector4: ...
    @property
    def yzlz(self) -> U64Vector4: ...
    @property
    def yzlw(self) -> U64Vector4: ...
    @property
    def yzlo(self) -> U64Vector4: ...
    @property
    def yzll(self) -> U64Vector4: ...
    @property
    def ywxx(self) -> U64Vector4: ...
    @property
    def ywxy(self) -> U64Vector4: ...
    @property
    def ywxz(self) -> U64Vector4: ...
    @property
    def ywxw(self) -> U64Vector4: ...
    @property
    def ywxo(self) -> U64Vector4: ...
    @property
    def ywxl(self) -> U64Vector4: ...
    @property
    def ywyx(self) -> U64Vector4: ...
    @property
    def ywyy(self) -> U64Vector4: ...
    @property
    def ywyz(self) -> U64Vector4: ...
    @property
    def ywyw(self) -> U64Vector4: ...
    @property
    def ywyo(self) -> U64Vector4: ...
    @property
    def ywyl(self) -> U64Vector4: ...
    @property
    def ywzx(self) -> U64Vector4: ...
    @property
    def ywzy(self) -> U64Vector4: ...
    @property
    def ywzz(self) -> U64Vector4: ...
    @property
    def ywzw(self) -> U64Vector4: ...
    @property
    def ywzo(self) -> U64Vector4: ...
    @property
    def ywzl(self) -> U64Vector4: ...
    @property
    def ywwx(self) -> U64Vector4: ...
    @property
    def ywwy(self) -> U64Vector4: ...
    @property
    def ywwz(self) -> U64Vector4: ...
    @property
    def ywww(self) -> U64Vector4: ...
    @property
    def ywwo(self) -> U64Vector4: ...
    @property
    def ywwl(self) -> U64Vector4: ...
    @property
    def ywox(self) -> U64Vector4: ...
    @property
    def ywoy(self) -> U64Vector4: ...
    @property
    def ywoz(self) -> U64Vector4: ...
    @property
    def ywow(self) -> U64Vector4: ...
    @property
    def ywoo(self) -> U64Vector4: ...
    @property
    def ywol(self) -> U64Vector4: ...
    @property
    def ywlx(self) -> U64Vector4: ...
    @property
    def ywly(self) -> U64Vector4: ...
    @property
    def ywlz(self) -> U64Vector4: ...
    @property
    def ywlw(self) -> U64Vector4: ...
    @property
    def ywlo(self) -> U64Vector4: ...
    @property
    def ywll(self) -> U64Vector4: ...
    @property
    def yoxx(self) -> U64Vector4: ...
    @property
    def yoxy(self) -> U64Vector4: ...
    @property
    def yoxz(self) -> U64Vector4: ...
    @property
    def yoxw(self) -> U64Vector4: ...
    @property
    def yoxo(self) -> U64Vector4: ...
    @property
    def yoxl(self) -> U64Vector4: ...
    @property
    def yoyx(self) -> U64Vector4: ...
    @property
    def yoyy(self) -> U64Vector4: ...
    @property
    def yoyz(self) -> U64Vector4: ...
    @property
    def yoyw(self) -> U64Vector4: ...
    @property
    def yoyo(self) -> U64Vector4: ...
    @property
    def yoyl(self) -> U64Vector4: ...
    @property
    def yozx(self) -> U64Vector4: ...
    @property
    def yozy(self) -> U64Vector4: ...
    @property
    def yozz(self) -> U64Vector4: ...
    @property
    def yozw(self) -> U64Vector4: ...
    @property
    def yozo(self) -> U64Vector4: ...
    @property
    def yozl(self) -> U64Vector4: ...
    @property
    def yowx(self) -> U64Vector4: ...
    @property
    def yowy(self) -> U64Vector4: ...
    @property
    def yowz(self) -> U64Vector4: ...
    @property
    def yoww(self) -> U64Vector4: ...
    @property
    def yowo(self) -> U64Vector4: ...
    @property
    def yowl(self) -> U64Vector4: ...
    @property
    def yoox(self) -> U64Vector4: ...
    @property
    def yooy(self) -> U64Vector4: ...
    @property
    def yooz(self) -> U64Vector4: ...
    @property
    def yoow(self) -> U64Vector4: ...
    @property
    def yooo(self) -> U64Vector4: ...
    @property
    def yool(self) -> U64Vector4: ...
    @property
    def yolx(self) -> U64Vector4: ...
    @property
    def yoly(self) -> U64Vector4: ...
    @property
    def yolz(self) -> U64Vector4: ...
    @property
    def yolw(self) -> U64Vector4: ...
    @property
    def yolo(self) -> U64Vector4: ...
    @property
    def yoll(self) -> U64Vector4: ...
    @property
    def ylxx(self) -> U64Vector4: ...
    @property
    def ylxy(self) -> U64Vector4: ...
    @property
    def ylxz(self) -> U64Vector4: ...
    @property
    def ylxw(self) -> U64Vector4: ...
    @property
    def ylxo(self) -> U64Vector4: ...
    @property
    def ylxl(self) -> U64Vector4: ...
    @property
    def ylyx(self) -> U64Vector4: ...
    @property
    def ylyy(self) -> U64Vector4: ...
    @property
    def ylyz(self) -> U64Vector4: ...
    @property
    def ylyw(self) -> U64Vector4: ...
    @property
    def ylyo(self) -> U64Vector4: ...
    @property
    def ylyl(self) -> U64Vector4: ...
    @property
    def ylzx(self) -> U64Vector4: ...
    @property
    def ylzy(self) -> U64Vector4: ...
    @property
    def ylzz(self) -> U64Vector4: ...
    @property
    def ylzw(self) -> U64Vector4: ...
    @property
    def ylzo(self) -> U64Vector4: ...
    @property
    def ylzl(self) -> U64Vector4: ...
    @property
    def ylwx(self) -> U64Vector4: ...
    @property
    def ylwy(self) -> U64Vector4: ...
    @property
    def ylwz(self) -> U64Vector4: ...
    @property
    def ylww(self) -> U64Vector4: ...
    @property
    def ylwo(self) -> U64Vector4: ...
    @property
    def ylwl(self) -> U64Vector4: ...
    @property
    def ylox(self) -> U64Vector4: ...
    @property
    def yloy(self) -> U64Vector4: ...
    @property
    def yloz(self) -> U64Vector4: ...
    @property
    def ylow(self) -> U64Vector4: ...
    @property
    def yloo(self) -> U64Vector4: ...
    @property
    def ylol(self) -> U64Vector4: ...
    @property
    def yllx(self) -> U64Vector4: ...
    @property
    def ylly(self) -> U64Vector4: ...
    @property
    def yllz(self) -> U64Vector4: ...
    @property
    def yllw(self) -> U64Vector4: ...
    @property
    def yllo(self) -> U64Vector4: ...
    @property
    def ylll(self) -> U64Vector4: ...
    @property
    def zxxx(self) -> U64Vector4: ...
    @property
    def zxxy(self) -> U64Vector4: ...
    @property
    def zxxz(self) -> U64Vector4: ...
    @property
    def zxxw(self) -> U64Vector4: ...
    @property
    def zxxo(self) -> U64Vector4: ...
    @property
    def zxxl(self) -> U64Vector4: ...
    @property
    def zxyx(self) -> U64Vector4: ...
    @property
    def zxyy(self) -> U64Vector4: ...
    @property
    def zxyz(self) -> U64Vector4: ...
    @property
    def zxyw(self) -> U64Vector4: ...
    @property
    def zxyo(self) -> U64Vector4: ...
    @property
    def zxyl(self) -> U64Vector4: ...
    @property
    def zxzx(self) -> U64Vector4: ...
    @property
    def zxzy(self) -> U64Vector4: ...
    @property
    def zxzz(self) -> U64Vector4: ...
    @property
    def zxzw(self) -> U64Vector4: ...
    @property
    def zxzo(self) -> U64Vector4: ...
    @property
    def zxzl(self) -> U64Vector4: ...
    @property
    def zxwx(self) -> U64Vector4: ...
    @property
    def zxwy(self) -> U64Vector4: ...
    @property
    def zxwz(self) -> U64Vector4: ...
    @property
    def zxww(self) -> U64Vector4: ...
    @property
    def zxwo(self) -> U64Vector4: ...
    @property
    def zxwl(self) -> U64Vector4: ...
    @property
    def zxox(self) -> U64Vector4: ...
    @property
    def zxoy(self) -> U64Vector4: ...
    @property
    def zxoz(self) -> U64Vector4: ...
    @property
    def zxow(self) -> U64Vector4: ...
    @property
    def zxoo(self) -> U64Vector4: ...
    @property
    def zxol(self) -> U64Vector4: ...
    @property
    def zxlx(self) -> U64Vector4: ...
    @property
    def zxly(self) -> U64Vector4: ...
    @property
    def zxlz(self) -> U64Vector4: ...
    @property
    def zxlw(self) -> U64Vector4: ...
    @property
    def zxlo(self) -> U64Vector4: ...
    @property
    def zxll(self) -> U64Vector4: ...
    @property
    def zyxx(self) -> U64Vector4: ...
    @property
    def zyxy(self) -> U64Vector4: ...
    @property
    def zyxz(self) -> U64Vector4: ...
    @property
    def zyxw(self) -> U64Vector4: ...
    @property
    def zyxo(self) -> U64Vector4: ...
    @property
    def zyxl(self) -> U64Vector4: ...
    @property
    def zyyx(self) -> U64Vector4: ...
    @property
    def zyyy(self) -> U64Vector4: ...
    @property
    def zyyz(self) -> U64Vector4: ...
    @property
    def zyyw(self) -> U64Vector4: ...
    @property
    def zyyo(self) -> U64Vector4: ...
    @property
    def zyyl(self) -> U64Vector4: ...
    @property
    def zyzx(self) -> U64Vector4: ...
    @property
    def zyzy(self) -> U64Vector4: ...
    @property
    def zyzz(self) -> U64Vector4: ...
    @property
    def zyzw(self) -> U64Vector4: ...
    @property
    def zyzo(self) -> U64Vector4: ...
    @property
    def zyzl(self) -> U64Vector4: ...
    @property
    def zywx(self) -> U64Vector4: ...
    @property
    def zywy(self) -> U64Vector4: ...
    @property
    def zywz(self) -> U64Vector4: ...
    @property
    def zyww(self) -> U64Vector4: ...
    @property
    def zywo(self) -> U64Vector4: ...
    @property
    def zywl(self) -> U64Vector4: ...
    @property
    def zyox(self) -> U64Vector4: ...
    @property
    def zyoy(self) -> U64Vector4: ...
    @property
    def zyoz(self) -> U64Vector4: ...
    @property
    def zyow(self) -> U64Vector4: ...
    @property
    def zyoo(self) -> U64Vector4: ...
    @property
    def zyol(self) -> U64Vector4: ...
    @property
    def zylx(self) -> U64Vector4: ...
    @property
    def zyly(self) -> U64Vector4: ...
    @property
    def zylz(self) -> U64Vector4: ...
    @property
    def zylw(self) -> U64Vector4: ...
    @property
    def zylo(self) -> U64Vector4: ...
    @property
    def zyll(self) -> U64Vector4: ...
    @property
    def zzxx(self) -> U64Vector4: ...
    @property
    def zzxy(self) -> U64Vector4: ...
    @property
    def zzxz(self) -> U64Vector4: ...
    @property
    def zzxw(self) -> U64Vector4: ...
    @property
    def zzxo(self) -> U64Vector4: ...
    @property
    def zzxl(self) -> U64Vector4: ...
    @property
    def zzyx(self) -> U64Vector4: ...
    @property
    def zzyy(self) -> U64Vector4: ...
    @property
    def zzyz(self) -> U64Vector4: ...
    @property
    def zzyw(self) -> U64Vector4: ...
    @property
    def zzyo(self) -> U64Vector4: ...
    @property
    def zzyl(self) -> U64Vector4: ...
    @property
    def zzzx(self) -> U64Vector4: ...
    @property
    def zzzy(self) -> U64Vector4: ...
    @property
    def zzzz(self) -> U64Vector4: ...
    @property
    def zzzw(self) -> U64Vector4: ...
    @property
    def zzzo(self) -> U64Vector4: ...
    @property
    def zzzl(self) -> U64Vector4: ...
    @property
    def zzwx(self) -> U64Vector4: ...
    @property
    def zzwy(self) -> U64Vector4: ...
    @property
    def zzwz(self) -> U64Vector4: ...
    @property
    def zzww(self) -> U64Vector4: ...
    @property
    def zzwo(self) -> U64Vector4: ...
    @property
    def zzwl(self) -> U64Vector4: ...
    @property
    def zzox(self) -> U64Vector4: ...
    @property
    def zzoy(self) -> U64Vector4: ...
    @property
    def zzoz(self) -> U64Vector4: ...
    @property
    def zzow(self) -> U64Vector4: ...
    @property
    def zzoo(self) -> U64Vector4: ...
    @property
    def zzol(self) -> U64Vector4: ...
    @property
    def zzlx(self) -> U64Vector4: ...
    @property
    def zzly(self) -> U64Vector4: ...
    @property
    def zzlz(self) -> U64Vector4: ...
    @property
    def zzlw(self) -> U64Vector4: ...
    @property
    def zzlo(self) -> U64Vector4: ...
    @property
    def zzll(self) -> U64Vector4: ...
    @property
    def zwxx(self) -> U64Vector4: ...
    @property
    def zwxy(self) -> U64Vector4: ...
    @property
    def zwxz(self) -> U64Vector4: ...
    @property
    def zwxw(self) -> U64Vector4: ...
    @property
    def zwxo(self) -> U64Vector4: ...
    @property
    def zwxl(self) -> U64Vector4: ...
    @property
    def zwyx(self) -> U64Vector4: ...
    @property
    def zwyy(self) -> U64Vector4: ...
    @property
    def zwyz(self) -> U64Vector4: ...
    @property
    def zwyw(self) -> U64Vector4: ...
    @property
    def zwyo(self) -> U64Vector4: ...
    @property
    def zwyl(self) -> U64Vector4: ...
    @property
    def zwzx(self) -> U64Vector4: ...
    @property
    def zwzy(self) -> U64Vector4: ...
    @property
    def zwzz(self) -> U64Vector4: ...
    @property
    def zwzw(self) -> U64Vector4: ...
    @property
    def zwzo(self) -> U64Vector4: ...
    @property
    def zwzl(self) -> U64Vector4: ...
    @property
    def zwwx(self) -> U64Vector4: ...
    @property
    def zwwy(self) -> U64Vector4: ...
    @property
    def zwwz(self) -> U64Vector4: ...
    @property
    def zwww(self) -> U64Vector4: ...
    @property
    def zwwo(self) -> U64Vector4: ...
    @property
    def zwwl(self) -> U64Vector4: ...
    @property
    def zwox(self) -> U64Vector4: ...
    @property
    def zwoy(self) -> U64Vector4: ...
    @property
    def zwoz(self) -> U64Vector4: ...
    @property
    def zwow(self) -> U64Vector4: ...
    @property
    def zwoo(self) -> U64Vector4: ...
    @property
    def zwol(self) -> U64Vector4: ...
    @property
    def zwlx(self) -> U64Vector4: ...
    @property
    def zwly(self) -> U64Vector4: ...
    @property
    def zwlz(self) -> U64Vector4: ...
    @property
    def zwlw(self) -> U64Vector4: ...
    @property
    def zwlo(self) -> U64Vector4: ...
    @property
    def zwll(self) -> U64Vector4: ...
    @property
    def zoxx(self) -> U64Vector4: ...
    @property
    def zoxy(self) -> U64Vector4: ...
    @property
    def zoxz(self) -> U64Vector4: ...
    @property
    def zoxw(self) -> U64Vector4: ...
    @property
    def zoxo(self) -> U64Vector4: ...
    @property
    def zoxl(self) -> U64Vector4: ...
    @property
    def zoyx(self) -> U64Vector4: ...
    @property
    def zoyy(self) -> U64Vector4: ...
    @property
    def zoyz(self) -> U64Vector4: ...
    @property
    def zoyw(self) -> U64Vector4: ...
    @property
    def zoyo(self) -> U64Vector4: ...
    @property
    def zoyl(self) -> U64Vector4: ...
    @property
    def zozx(self) -> U64Vector4: ...
    @property
    def zozy(self) -> U64Vector4: ...
    @property
    def zozz(self) -> U64Vector4: ...
    @property
    def zozw(self) -> U64Vector4: ...
    @property
    def zozo(self) -> U64Vector4: ...
    @property
    def zozl(self) -> U64Vector4: ...
    @property
    def zowx(self) -> U64Vector4: ...
    @property
    def zowy(self) -> U64Vector4: ...
    @property
    def zowz(self) -> U64Vector4: ...
    @property
    def zoww(self) -> U64Vector4: ...
    @property
    def zowo(self) -> U64Vector4: ...
    @property
    def zowl(self) -> U64Vector4: ...
    @property
    def zoox(self) -> U64Vector4: ...
    @property
    def zooy(self) -> U64Vector4: ...
    @property
    def zooz(self) -> U64Vector4: ...
    @property
    def zoow(self) -> U64Vector4: ...
    @property
    def zooo(self) -> U64Vector4: ...
    @property
    def zool(self) -> U64Vector4: ...
    @property
    def zolx(self) -> U64Vector4: ...
    @property
    def zoly(self) -> U64Vector4: ...
    @property
    def zolz(self) -> U64Vector4: ...
    @property
    def zolw(self) -> U64Vector4: ...
    @property
    def zolo(self) -> U64Vector4: ...
    @property
    def zoll(self) -> U64Vector4: ...
    @property
    def zlxx(self) -> U64Vector4: ...
    @property
    def zlxy(self) -> U64Vector4: ...
    @property
    def zlxz(self) -> U64Vector4: ...
    @property
    def zlxw(self) -> U64Vector4: ...
    @property
    def zlxo(self) -> U64Vector4: ...
    @property
    def zlxl(self) -> U64Vector4: ...
    @property
    def zlyx(self) -> U64Vector4: ...
    @property
    def zlyy(self) -> U64Vector4: ...
    @property
    def zlyz(self) -> U64Vector4: ...
    @property
    def zlyw(self) -> U64Vector4: ...
    @property
    def zlyo(self) -> U64Vector4: ...
    @property
    def zlyl(self) -> U64Vector4: ...
    @property
    def zlzx(self) -> U64Vector4: ...
    @property
    def zlzy(self) -> U64Vector4: ...
    @property
    def zlzz(self) -> U64Vector4: ...
    @property
    def zlzw(self) -> U64Vector4: ...
    @property
    def zlzo(self) -> U64Vector4: ...
    @property
    def zlzl(self) -> U64Vector4: ...
    @property
    def zlwx(self) -> U64Vector4: ...
    @property
    def zlwy(self) -> U64Vector4: ...
    @property
    def zlwz(self) -> U64Vector4: ...
    @property
    def zlww(self) -> U64Vector4: ...
    @property
    def zlwo(self) -> U64Vector4: ...
    @property
    def zlwl(self) -> U64Vector4: ...
    @property
    def zlox(self) -> U64Vector4: ...
    @property
    def zloy(self) -> U64Vector4: ...
    @property
    def zloz(self) -> U64Vector4: ...
    @property
    def zlow(self) -> U64Vector4: ...
    @property
    def zloo(self) -> U64Vector4: ...
    @property
    def zlol(self) -> U64Vector4: ...
    @property
    def zllx(self) -> U64Vector4: ...
    @property
    def zlly(self) -> U64Vector4: ...
    @property
    def zllz(self) -> U64Vector4: ...
    @property
    def zllw(self) -> U64Vector4: ...
    @property
    def zllo(self) -> U64Vector4: ...
    @property
    def zlll(self) -> U64Vector4: ...
    @property
    def wxxx(self) -> U64Vector4: ...
    @property
    def wxxy(self) -> U64Vector4: ...
    @property
    def wxxz(self) -> U64Vector4: ...
    @property
    def wxxw(self) -> U64Vector4: ...
    @property
    def wxxo(self) -> U64Vector4: ...
    @property
    def wxxl(self) -> U64Vector4: ...
    @property
    def wxyx(self) -> U64Vector4: ...
    @property
    def wxyy(self) -> U64Vector4: ...
    @property
    def wxyz(self) -> U64Vector4: ...
    @property
    def wxyw(self) -> U64Vector4: ...
    @property
    def wxyo(self) -> U64Vector4: ...
    @property
    def wxyl(self) -> U64Vector4: ...
    @property
    def wxzx(self) -> U64Vector4: ...
    @property
    def wxzy(self) -> U64Vector4: ...
    @property
    def wxzz(self) -> U64Vector4: ...
    @property
    def wxzw(self) -> U64Vector4: ...
    @property
    def wxzo(self) -> U64Vector4: ...
    @property
    def wxzl(self) -> U64Vector4: ...
    @property
    def wxwx(self) -> U64Vector4: ...
    @property
    def wxwy(self) -> U64Vector4: ...
    @property
    def wxwz(self) -> U64Vector4: ...
    @property
    def wxww(self) -> U64Vector4: ...
    @property
    def wxwo(self) -> U64Vector4: ...
    @property
    def wxwl(self) -> U64Vector4: ...
    @property
    def wxox(self) -> U64Vector4: ...
    @property
    def wxoy(self) -> U64Vector4: ...
    @property
    def wxoz(self) -> U64Vector4: ...
    @property
    def wxow(self) -> U64Vector4: ...
    @property
    def wxoo(self) -> U64Vector4: ...
    @property
    def wxol(self) -> U64Vector4: ...
    @property
    def wxlx(self) -> U64Vector4: ...
    @property
    def wxly(self) -> U64Vector4: ...
    @property
    def wxlz(self) -> U64Vector4: ...
    @property
    def wxlw(self) -> U64Vector4: ...
    @property
    def wxlo(self) -> U64Vector4: ...
    @property
    def wxll(self) -> U64Vector4: ...
    @property
    def wyxx(self) -> U64Vector4: ...
    @property
    def wyxy(self) -> U64Vector4: ...
    @property
    def wyxz(self) -> U64Vector4: ...
    @property
    def wyxw(self) -> U64Vector4: ...
    @property
    def wyxo(self) -> U64Vector4: ...
    @property
    def wyxl(self) -> U64Vector4: ...
    @property
    def wyyx(self) -> U64Vector4: ...
    @property
    def wyyy(self) -> U64Vector4: ...
    @property
    def wyyz(self) -> U64Vector4: ...
    @property
    def wyyw(self) -> U64Vector4: ...
    @property
    def wyyo(self) -> U64Vector4: ...
    @property
    def wyyl(self) -> U64Vector4: ...
    @property
    def wyzx(self) -> U64Vector4: ...
    @property
    def wyzy(self) -> U64Vector4: ...
    @property
    def wyzz(self) -> U64Vector4: ...
    @property
    def wyzw(self) -> U64Vector4: ...
    @property
    def wyzo(self) -> U64Vector4: ...
    @property
    def wyzl(self) -> U64Vector4: ...
    @property
    def wywx(self) -> U64Vector4: ...
    @property
    def wywy(self) -> U64Vector4: ...
    @property
    def wywz(self) -> U64Vector4: ...
    @property
    def wyww(self) -> U64Vector4: ...
    @property
    def wywo(self) -> U64Vector4: ...
    @property
    def wywl(self) -> U64Vector4: ...
    @property
    def wyox(self) -> U64Vector4: ...
    @property
    def wyoy(self) -> U64Vector4: ...
    @property
    def wyoz(self) -> U64Vector4: ...
    @property
    def wyow(self) -> U64Vector4: ...
    @property
    def wyoo(self) -> U64Vector4: ...
    @property
    def wyol(self) -> U64Vector4: ...
    @property
    def wylx(self) -> U64Vector4: ...
    @property
    def wyly(self) -> U64Vector4: ...
    @property
    def wylz(self) -> U64Vector4: ...
    @property
    def wylw(self) -> U64Vector4: ...
    @property
    def wylo(self) -> U64Vector4: ...
    @property
    def wyll(self) -> U64Vector4: ...
    @property
    def wzxx(self) -> U64Vector4: ...
    @property
    def wzxy(self) -> U64Vector4: ...
    @property
    def wzxz(self) -> U64Vector4: ...
    @property
    def wzxw(self) -> U64Vector4: ...
    @property
    def wzxo(self) -> U64Vector4: ...
    @property
    def wzxl(self) -> U64Vector4: ...
    @property
    def wzyx(self) -> U64Vector4: ...
    @property
    def wzyy(self) -> U64Vector4: ...
    @property
    def wzyz(self) -> U64Vector4: ...
    @property
    def wzyw(self) -> U64Vector4: ...
    @property
    def wzyo(self) -> U64Vector4: ...
    @property
    def wzyl(self) -> U64Vector4: ...
    @property
    def wzzx(self) -> U64Vector4: ...
    @property
    def wzzy(self) -> U64Vector4: ...
    @property
    def wzzz(self) -> U64Vector4: ...
    @property
    def wzzw(self) -> U64Vector4: ...
    @property
    def wzzo(self) -> U64Vector4: ...
    @property
    def wzzl(self) -> U64Vector4: ...
    @property
    def wzwx(self) -> U64Vector4: ...
    @property
    def wzwy(self) -> U64Vector4: ...
    @property
    def wzwz(self) -> U64Vector4: ...
    @property
    def wzww(self) -> U64Vector4: ...
    @property
    def wzwo(self) -> U64Vector4: ...
    @property
    def wzwl(self) -> U64Vector4: ...
    @property
    def wzox(self) -> U64Vector4: ...
    @property
    def wzoy(self) -> U64Vector4: ...
    @property
    def wzoz(self) -> U64Vector4: ...
    @property
    def wzow(self) -> U64Vector4: ...
    @property
    def wzoo(self) -> U64Vector4: ...
    @property
    def wzol(self) -> U64Vector4: ...
    @property
    def wzlx(self) -> U64Vector4: ...
    @property
    def wzly(self) -> U64Vector4: ...
    @property
    def wzlz(self) -> U64Vector4: ...
    @property
    def wzlw(self) -> U64Vector4: ...
    @property
    def wzlo(self) -> U64Vector4: ...
    @property
    def wzll(self) -> U64Vector4: ...
    @property
    def wwxx(self) -> U64Vector4: ...
    @property
    def wwxy(self) -> U64Vector4: ...
    @property
    def wwxz(self) -> U64Vector4: ...
    @property
    def wwxw(self) -> U64Vector4: ...
    @property
    def wwxo(self) -> U64Vector4: ...
    @property
    def wwxl(self) -> U64Vector4: ...
    @property
    def wwyx(self) -> U64Vector4: ...
    @property
    def wwyy(self) -> U64Vector4: ...
    @property
    def wwyz(self) -> U64Vector4: ...
    @property
    def wwyw(self) -> U64Vector4: ...
    @property
    def wwyo(self) -> U64Vector4: ...
    @property
    def wwyl(self) -> U64Vector4: ...
    @property
    def wwzx(self) -> U64Vector4: ...
    @property
    def wwzy(self) -> U64Vector4: ...
    @property
    def wwzz(self) -> U64Vector4: ...
    @property
    def wwzw(self) -> U64Vector4: ...
    @property
    def wwzo(self) -> U64Vector4: ...
    @property
    def wwzl(self) -> U64Vector4: ...
    @property
    def wwwx(self) -> U64Vector4: ...
    @property
    def wwwy(self) -> U64Vector4: ...
    @property
    def wwwz(self) -> U64Vector4: ...
    @property
    def wwww(self) -> U64Vector4: ...
    @property
    def wwwo(self) -> U64Vector4: ...
    @property
    def wwwl(self) -> U64Vector4: ...
    @property
    def wwox(self) -> U64Vector4: ...
    @property
    def wwoy(self) -> U64Vector4: ...
    @property
    def wwoz(self) -> U64Vector4: ...
    @property
    def wwow(self) -> U64Vector4: ...
    @property
    def wwoo(self) -> U64Vector4: ...
    @property
    def wwol(self) -> U64Vector4: ...
    @property
    def wwlx(self) -> U64Vector4: ...
    @property
    def wwly(self) -> U64Vector4: ...
    @property
    def wwlz(self) -> U64Vector4: ...
    @property
    def wwlw(self) -> U64Vector4: ...
    @property
    def wwlo(self) -> U64Vector4: ...
    @property
    def wwll(self) -> U64Vector4: ...
    @property
    def woxx(self) -> U64Vector4: ...
    @property
    def woxy(self) -> U64Vector4: ...
    @property
    def woxz(self) -> U64Vector4: ...
    @property
    def woxw(self) -> U64Vector4: ...
    @property
    def woxo(self) -> U64Vector4: ...
    @property
    def woxl(self) -> U64Vector4: ...
    @property
    def woyx(self) -> U64Vector4: ...
    @property
    def woyy(self) -> U64Vector4: ...
    @property
    def woyz(self) -> U64Vector4: ...
    @property
    def woyw(self) -> U64Vector4: ...
    @property
    def woyo(self) -> U64Vector4: ...
    @property
    def woyl(self) -> U64Vector4: ...
    @property
    def wozx(self) -> U64Vector4: ...
    @property
    def wozy(self) -> U64Vector4: ...
    @property
    def wozz(self) -> U64Vector4: ...
    @property
    def wozw(self) -> U64Vector4: ...
    @property
    def wozo(self) -> U64Vector4: ...
    @property
    def wozl(self) -> U64Vector4: ...
    @property
    def wowx(self) -> U64Vector4: ...
    @property
    def wowy(self) -> U64Vector4: ...
    @property
    def wowz(self) -> U64Vector4: ...
    @property
    def woww(self) -> U64Vector4: ...
    @property
    def wowo(self) -> U64Vector4: ...
    @property
    def wowl(self) -> U64Vector4: ...
    @property
    def woox(self) -> U64Vector4: ...
    @property
    def wooy(self) -> U64Vector4: ...
    @property
    def wooz(self) -> U64Vector4: ...
    @property
    def woow(self) -> U64Vector4: ...
    @property
    def wooo(self) -> U64Vector4: ...
    @property
    def wool(self) -> U64Vector4: ...
    @property
    def wolx(self) -> U64Vector4: ...
    @property
    def woly(self) -> U64Vector4: ...
    @property
    def wolz(self) -> U64Vector4: ...
    @property
    def wolw(self) -> U64Vector4: ...
    @property
    def wolo(self) -> U64Vector4: ...
    @property
    def woll(self) -> U64Vector4: ...
    @property
    def wlxx(self) -> U64Vector4: ...
    @property
    def wlxy(self) -> U64Vector4: ...
    @property
    def wlxz(self) -> U64Vector4: ...
    @property
    def wlxw(self) -> U64Vector4: ...
    @property
    def wlxo(self) -> U64Vector4: ...
    @property
    def wlxl(self) -> U64Vector4: ...
    @property
    def wlyx(self) -> U64Vector4: ...
    @property
    def wlyy(self) -> U64Vector4: ...
    @property
    def wlyz(self) -> U64Vector4: ...
    @property
    def wlyw(self) -> U64Vector4: ...
    @property
    def wlyo(self) -> U64Vector4: ...
    @property
    def wlyl(self) -> U64Vector4: ...
    @property
    def wlzx(self) -> U64Vector4: ...
    @property
    def wlzy(self) -> U64Vector4: ...
    @property
    def wlzz(self) -> U64Vector4: ...
    @property
    def wlzw(self) -> U64Vector4: ...
    @property
    def wlzo(self) -> U64Vector4: ...
    @property
    def wlzl(self) -> U64Vector4: ...
    @property
    def wlwx(self) -> U64Vector4: ...
    @property
    def wlwy(self) -> U64Vector4: ...
    @property
    def wlwz(self) -> U64Vector4: ...
    @property
    def wlww(self) -> U64Vector4: ...
    @property
    def wlwo(self) -> U64Vector4: ...
    @property
    def wlwl(self) -> U64Vector4: ...
    @property
    def wlox(self) -> U64Vector4: ...
    @property
    def wloy(self) -> U64Vector4: ...
    @property
    def wloz(self) -> U64Vector4: ...
    @property
    def wlow(self) -> U64Vector4: ...
    @property
    def wloo(self) -> U64Vector4: ...
    @property
    def wlol(self) -> U64Vector4: ...
    @property
    def wllx(self) -> U64Vector4: ...
    @property
    def wlly(self) -> U64Vector4: ...
    @property
    def wllz(self) -> U64Vector4: ...
    @property
    def wllw(self) -> U64Vector4: ...
    @property
    def wllo(self) -> U64Vector4: ...
    @property
    def wlll(self) -> U64Vector4: ...
    @property
    def oxxx(self) -> U64Vector4: ...
    @property
    def oxxy(self) -> U64Vector4: ...
    @property
    def oxxz(self) -> U64Vector4: ...
    @property
    def oxxw(self) -> U64Vector4: ...
    @property
    def oxxo(self) -> U64Vector4: ...
    @property
    def oxxl(self) -> U64Vector4: ...
    @property
    def oxyx(self) -> U64Vector4: ...
    @property
    def oxyy(self) -> U64Vector4: ...
    @property
    def oxyz(self) -> U64Vector4: ...
    @property
    def oxyw(self) -> U64Vector4: ...
    @property
    def oxyo(self) -> U64Vector4: ...
    @property
    def oxyl(self) -> U64Vector4: ...
    @property
    def oxzx(self) -> U64Vector4: ...
    @property
    def oxzy(self) -> U64Vector4: ...
    @property
    def oxzz(self) -> U64Vector4: ...
    @property
    def oxzw(self) -> U64Vector4: ...
    @property
    def oxzo(self) -> U64Vector4: ...
    @property
    def oxzl(self) -> U64Vector4: ...
    @property
    def oxwx(self) -> U64Vector4: ...
    @property
    def oxwy(self) -> U64Vector4: ...
    @property
    def oxwz(self) -> U64Vector4: ...
    @property
    def oxww(self) -> U64Vector4: ...
    @property
    def oxwo(self) -> U64Vector4: ...
    @property
    def oxwl(self) -> U64Vector4: ...
    @property
    def oxox(self) -> U64Vector4: ...
    @property
    def oxoy(self) -> U64Vector4: ...
    @property
    def oxoz(self) -> U64Vector4: ...
    @property
    def oxow(self) -> U64Vector4: ...
    @property
    def oxoo(self) -> U64Vector4: ...
    @property
    def oxol(self) -> U64Vector4: ...
    @property
    def oxlx(self) -> U64Vector4: ...
    @property
    def oxly(self) -> U64Vector4: ...
    @property
    def oxlz(self) -> U64Vector4: ...
    @property
    def oxlw(self) -> U64Vector4: ...
    @property
    def oxlo(self) -> U64Vector4: ...
    @property
    def oxll(self) -> U64Vector4: ...
    @property
    def oyxx(self) -> U64Vector4: ...
    @property
    def oyxy(self) -> U64Vector4: ...
    @property
    def oyxz(self) -> U64Vector4: ...
    @property
    def oyxw(self) -> U64Vector4: ...
    @property
    def oyxo(self) -> U64Vector4: ...
    @property
    def oyxl(self) -> U64Vector4: ...
    @property
    def oyyx(self) -> U64Vector4: ...
    @property
    def oyyy(self) -> U64Vector4: ...
    @property
    def oyyz(self) -> U64Vector4: ...
    @property
    def oyyw(self) -> U64Vector4: ...
    @property
    def oyyo(self) -> U64Vector4: ...
    @property
    def oyyl(self) -> U64Vector4: ...
    @property
    def oyzx(self) -> U64Vector4: ...
    @property
    def oyzy(self) -> U64Vector4: ...
    @property
    def oyzz(self) -> U64Vector4: ...
    @property
    def oyzw(self) -> U64Vector4: ...
    @property
    def oyzo(self) -> U64Vector4: ...
    @property
    def oyzl(self) -> U64Vector4: ...
    @property
    def oywx(self) -> U64Vector4: ...
    @property
    def oywy(self) -> U64Vector4: ...
    @property
    def oywz(self) -> U64Vector4: ...
    @property
    def oyww(self) -> U64Vector4: ...
    @property
    def oywo(self) -> U64Vector4: ...
    @property
    def oywl(self) -> U64Vector4: ...
    @property
    def oyox(self) -> U64Vector4: ...
    @property
    def oyoy(self) -> U64Vector4: ...
    @property
    def oyoz(self) -> U64Vector4: ...
    @property
    def oyow(self) -> U64Vector4: ...
    @property
    def oyoo(self) -> U64Vector4: ...
    @property
    def oyol(self) -> U64Vector4: ...
    @property
    def oylx(self) -> U64Vector4: ...
    @property
    def oyly(self) -> U64Vector4: ...
    @property
    def oylz(self) -> U64Vector4: ...
    @property
    def oylw(self) -> U64Vector4: ...
    @property
    def oylo(self) -> U64Vector4: ...
    @property
    def oyll(self) -> U64Vector4: ...
    @property
    def ozxx(self) -> U64Vector4: ...
    @property
    def ozxy(self) -> U64Vector4: ...
    @property
    def ozxz(self) -> U64Vector4: ...
    @property
    def ozxw(self) -> U64Vector4: ...
    @property
    def ozxo(self) -> U64Vector4: ...
    @property
    def ozxl(self) -> U64Vector4: ...
    @property
    def ozyx(self) -> U64Vector4: ...
    @property
    def ozyy(self) -> U64Vector4: ...
    @property
    def ozyz(self) -> U64Vector4: ...
    @property
    def ozyw(self) -> U64Vector4: ...
    @property
    def ozyo(self) -> U64Vector4: ...
    @property
    def ozyl(self) -> U64Vector4: ...
    @property
    def ozzx(self) -> U64Vector4: ...
    @property
    def ozzy(self) -> U64Vector4: ...
    @property
    def ozzz(self) -> U64Vector4: ...
    @property
    def ozzw(self) -> U64Vector4: ...
    @property
    def ozzo(self) -> U64Vector4: ...
    @property
    def ozzl(self) -> U64Vector4: ...
    @property
    def ozwx(self) -> U64Vector4: ...
    @property
    def ozwy(self) -> U64Vector4: ...
    @property
    def ozwz(self) -> U64Vector4: ...
    @property
    def ozww(self) -> U64Vector4: ...
    @property
    def ozwo(self) -> U64Vector4: ...
    @property
    def ozwl(self) -> U64Vector4: ...
    @property
    def ozox(self) -> U64Vector4: ...
    @property
    def ozoy(self) -> U64Vector4: ...
    @property
    def ozoz(self) -> U64Vector4: ...
    @property
    def ozow(self) -> U64Vector4: ...
    @property
    def ozoo(self) -> U64Vector4: ...
    @property
    def ozol(self) -> U64Vector4: ...
    @property
    def ozlx(self) -> U64Vector4: ...
    @property
    def ozly(self) -> U64Vector4: ...
    @property
    def ozlz(self) -> U64Vector4: ...
    @property
    def ozlw(self) -> U64Vector4: ...
    @property
    def ozlo(self) -> U64Vector4: ...
    @property
    def ozll(self) -> U64Vector4: ...
    @property
    def owxx(self) -> U64Vector4: ...
    @property
    def owxy(self) -> U64Vector4: ...
    @property
    def owxz(self) -> U64Vector4: ...
    @property
    def owxw(self) -> U64Vector4: ...
    @property
    def owxo(self) -> U64Vector4: ...
    @property
    def owxl(self) -> U64Vector4: ...
    @property
    def owyx(self) -> U64Vector4: ...
    @property
    def owyy(self) -> U64Vector4: ...
    @property
    def owyz(self) -> U64Vector4: ...
    @property
    def owyw(self) -> U64Vector4: ...
    @property
    def owyo(self) -> U64Vector4: ...
    @property
    def owyl(self) -> U64Vector4: ...
    @property
    def owzx(self) -> U64Vector4: ...
    @property
    def owzy(self) -> U64Vector4: ...
    @property
    def owzz(self) -> U64Vector4: ...
    @property
    def owzw(self) -> U64Vector4: ...
    @property
    def owzo(self) -> U64Vector4: ...
    @property
    def owzl(self) -> U64Vector4: ...
    @property
    def owwx(self) -> U64Vector4: ...
    @property
    def owwy(self) -> U64Vector4: ...
    @property
    def owwz(self) -> U64Vector4: ...
    @property
    def owww(self) -> U64Vector4: ...
    @property
    def owwo(self) -> U64Vector4: ...
    @property
    def owwl(self) -> U64Vector4: ...
    @property
    def owox(self) -> U64Vector4: ...
    @property
    def owoy(self) -> U64Vector4: ...
    @property
    def owoz(self) -> U64Vector4: ...
    @property
    def owow(self) -> U64Vector4: ...
    @property
    def owoo(self) -> U64Vector4: ...
    @property
    def owol(self) -> U64Vector4: ...
    @property
    def owlx(self) -> U64Vector4: ...
    @property
    def owly(self) -> U64Vector4: ...
    @property
    def owlz(self) -> U64Vector4: ...
    @property
    def owlw(self) -> U64Vector4: ...
    @property
    def owlo(self) -> U64Vector4: ...
    @property
    def owll(self) -> U64Vector4: ...
    @property
    def ooxx(self) -> U64Vector4: ...
    @property
    def ooxy(self) -> U64Vector4: ...
    @property
    def ooxz(self) -> U64Vector4: ...
    @property
    def ooxw(self) -> U64Vector4: ...
    @property
    def ooxo(self) -> U64Vector4: ...
    @property
    def ooxl(self) -> U64Vector4: ...
    @property
    def ooyx(self) -> U64Vector4: ...
    @property
    def ooyy(self) -> U64Vector4: ...
    @property
    def ooyz(self) -> U64Vector4: ...
    @property
    def ooyw(self) -> U64Vector4: ...
    @property
    def ooyo(self) -> U64Vector4: ...
    @property
    def ooyl(self) -> U64Vector4: ...
    @property
    def oozx(self) -> U64Vector4: ...
    @property
    def oozy(self) -> U64Vector4: ...
    @property
    def oozz(self) -> U64Vector4: ...
    @property
    def oozw(self) -> U64Vector4: ...
    @property
    def oozo(self) -> U64Vector4: ...
    @property
    def oozl(self) -> U64Vector4: ...
    @property
    def oowx(self) -> U64Vector4: ...
    @property
    def oowy(self) -> U64Vector4: ...
    @property
    def oowz(self) -> U64Vector4: ...
    @property
    def ooww(self) -> U64Vector4: ...
    @property
    def oowo(self) -> U64Vector4: ...
    @property
    def oowl(self) -> U64Vector4: ...
    @property
    def ooox(self) -> U64Vector4: ...
    @property
    def oooy(self) -> U64Vector4: ...
    @property
    def oooz(self) -> U64Vector4: ...
    @property
    def ooow(self) -> U64Vector4: ...
    @property
    def oolx(self) -> U64Vector4: ...
    @property
    def ooly(self) -> U64Vector4: ...
    @property
    def oolz(self) -> U64Vector4: ...
    @property
    def oolw(self) -> U64Vector4: ...
    @property
    def olxx(self) -> U64Vector4: ...
    @property
    def olxy(self) -> U64Vector4: ...
    @property
    def olxz(self) -> U64Vector4: ...
    @property
    def olxw(self) -> U64Vector4: ...
    @property
    def olxo(self) -> U64Vector4: ...
    @property
    def olxl(self) -> U64Vector4: ...
    @property
    def olyx(self) -> U64Vector4: ...
    @property
    def olyy(self) -> U64Vector4: ...
    @property
    def olyz(self) -> U64Vector4: ...
    @property
    def olyw(self) -> U64Vector4: ...
    @property
    def olyo(self) -> U64Vector4: ...
    @property
    def olyl(self) -> U64Vector4: ...
    @property
    def olzx(self) -> U64Vector4: ...
    @property
    def olzy(self) -> U64Vector4: ...
    @property
    def olzz(self) -> U64Vector4: ...
    @property
    def olzw(self) -> U64Vector4: ...
    @property
    def olzo(self) -> U64Vector4: ...
    @property
    def olzl(self) -> U64Vector4: ...
    @property
    def olwx(self) -> U64Vector4: ...
    @property
    def olwy(self) -> U64Vector4: ...
    @property
    def olwz(self) -> U64Vector4: ...
    @property
    def olww(self) -> U64Vector4: ...
    @property
    def olwo(self) -> U64Vector4: ...
    @property
    def olwl(self) -> U64Vector4: ...
    @property
    def olox(self) -> U64Vector4: ...
    @property
    def oloy(self) -> U64Vector4: ...
    @property
    def oloz(self) -> U64Vector4: ...
    @property
    def olow(self) -> U64Vector4: ...
    @property
    def ollx(self) -> U64Vector4: ...
    @property
    def olly(self) -> U64Vector4: ...
    @property
    def ollz(self) -> U64Vector4: ...
    @property
    def ollw(self) -> U64Vector4: ...
    @property
    def lxxx(self) -> U64Vector4: ...
    @property
    def lxxy(self) -> U64Vector4: ...
    @property
    def lxxz(self) -> U64Vector4: ...
    @property
    def lxxw(self) -> U64Vector4: ...
    @property
    def lxxo(self) -> U64Vector4: ...
    @property
    def lxxl(self) -> U64Vector4: ...
    @property
    def lxyx(self) -> U64Vector4: ...
    @property
    def lxyy(self) -> U64Vector4: ...
    @property
    def lxyz(self) -> U64Vector4: ...
    @property
    def lxyw(self) -> U64Vector4: ...
    @property
    def lxyo(self) -> U64Vector4: ...
    @property
    def lxyl(self) -> U64Vector4: ...
    @property
    def lxzx(self) -> U64Vector4: ...
    @property
    def lxzy(self) -> U64Vector4: ...
    @property
    def lxzz(self) -> U64Vector4: ...
    @property
    def lxzw(self) -> U64Vector4: ...
    @property
    def lxzo(self) -> U64Vector4: ...
    @property
    def lxzl(self) -> U64Vector4: ...
    @property
    def lxwx(self) -> U64Vector4: ...
    @property
    def lxwy(self) -> U64Vector4: ...
    @property
    def lxwz(self) -> U64Vector4: ...
    @property
    def lxww(self) -> U64Vector4: ...
    @property
    def lxwo(self) -> U64Vector4: ...
    @property
    def lxwl(self) -> U64Vector4: ...
    @property
    def lxox(self) -> U64Vector4: ...
    @property
    def lxoy(self) -> U64Vector4: ...
    @property
    def lxoz(self) -> U64Vector4: ...
    @property
    def lxow(self) -> U64Vector4: ...
    @property
    def lxoo(self) -> U64Vector4: ...
    @property
    def lxol(self) -> U64Vector4: ...
    @property
    def lxlx(self) -> U64Vector4: ...
    @property
    def lxly(self) -> U64Vector4: ...
    @property
    def lxlz(self) -> U64Vector4: ...
    @property
    def lxlw(self) -> U64Vector4: ...
    @property
    def lxlo(self) -> U64Vector4: ...
    @property
    def lxll(self) -> U64Vector4: ...
    @property
    def lyxx(self) -> U64Vector4: ...
    @property
    def lyxy(self) -> U64Vector4: ...
    @property
    def lyxz(self) -> U64Vector4: ...
    @property
    def lyxw(self) -> U64Vector4: ...
    @property
    def lyxo(self) -> U64Vector4: ...
    @property
    def lyxl(self) -> U64Vector4: ...
    @property
    def lyyx(self) -> U64Vector4: ...
    @property
    def lyyy(self) -> U64Vector4: ...
    @property
    def lyyz(self) -> U64Vector4: ...
    @property
    def lyyw(self) -> U64Vector4: ...
    @property
    def lyyo(self) -> U64Vector4: ...
    @property
    def lyyl(self) -> U64Vector4: ...
    @property
    def lyzx(self) -> U64Vector4: ...
    @property
    def lyzy(self) -> U64Vector4: ...
    @property
    def lyzz(self) -> U64Vector4: ...
    @property
    def lyzw(self) -> U64Vector4: ...
    @property
    def lyzo(self) -> U64Vector4: ...
    @property
    def lyzl(self) -> U64Vector4: ...
    @property
    def lywx(self) -> U64Vector4: ...
    @property
    def lywy(self) -> U64Vector4: ...
    @property
    def lywz(self) -> U64Vector4: ...
    @property
    def lyww(self) -> U64Vector4: ...
    @property
    def lywo(self) -> U64Vector4: ...
    @property
    def lywl(self) -> U64Vector4: ...
    @property
    def lyox(self) -> U64Vector4: ...
    @property
    def lyoy(self) -> U64Vector4: ...
    @property
    def lyoz(self) -> U64Vector4: ...
    @property
    def lyow(self) -> U64Vector4: ...
    @property
    def lyoo(self) -> U64Vector4: ...
    @property
    def lyol(self) -> U64Vector4: ...
    @property
    def lylx(self) -> U64Vector4: ...
    @property
    def lyly(self) -> U64Vector4: ...
    @property
    def lylz(self) -> U64Vector4: ...
    @property
    def lylw(self) -> U64Vector4: ...
    @property
    def lylo(self) -> U64Vector4: ...
    @property
    def lyll(self) -> U64Vector4: ...
    @property
    def lzxx(self) -> U64Vector4: ...
    @property
    def lzxy(self) -> U64Vector4: ...
    @property
    def lzxz(self) -> U64Vector4: ...
    @property
    def lzxw(self) -> U64Vector4: ...
    @property
    def lzxo(self) -> U64Vector4: ...
    @property
    def lzxl(self) -> U64Vector4: ...
    @property
    def lzyx(self) -> U64Vector4: ...
    @property
    def lzyy(self) -> U64Vector4: ...
    @property
    def lzyz(self) -> U64Vector4: ...
    @property
    def lzyw(self) -> U64Vector4: ...
    @property
    def lzyo(self) -> U64Vector4: ...
    @property
    def lzyl(self) -> U64Vector4: ...
    @property
    def lzzx(self) -> U64Vector4: ...
    @property
    def lzzy(self) -> U64Vector4: ...
    @property
    def lzzz(self) -> U64Vector4: ...
    @property
    def lzzw(self) -> U64Vector4: ...
    @property
    def lzzo(self) -> U64Vector4: ...
    @property
    def lzzl(self) -> U64Vector4: ...
    @property
    def lzwx(self) -> U64Vector4: ...
    @property
    def lzwy(self) -> U64Vector4: ...
    @property
    def lzwz(self) -> U64Vector4: ...
    @property
    def lzww(self) -> U64Vector4: ...
    @property
    def lzwo(self) -> U64Vector4: ...
    @property
    def lzwl(self) -> U64Vector4: ...
    @property
    def lzox(self) -> U64Vector4: ...
    @property
    def lzoy(self) -> U64Vector4: ...
    @property
    def lzoz(self) -> U64Vector4: ...
    @property
    def lzow(self) -> U64Vector4: ...
    @property
    def lzoo(self) -> U64Vector4: ...
    @property
    def lzol(self) -> U64Vector4: ...
    @property
    def lzlx(self) -> U64Vector4: ...
    @property
    def lzly(self) -> U64Vector4: ...
    @property
    def lzlz(self) -> U64Vector4: ...
    @property
    def lzlw(self) -> U64Vector4: ...
    @property
    def lzlo(self) -> U64Vector4: ...
    @property
    def lzll(self) -> U64Vector4: ...
    @property
    def lwxx(self) -> U64Vector4: ...
    @property
    def lwxy(self) -> U64Vector4: ...
    @property
    def lwxz(self) -> U64Vector4: ...
    @property
    def lwxw(self) -> U64Vector4: ...
    @property
    def lwxo(self) -> U64Vector4: ...
    @property
    def lwxl(self) -> U64Vector4: ...
    @property
    def lwyx(self) -> U64Vector4: ...
    @property
    def lwyy(self) -> U64Vector4: ...
    @property
    def lwyz(self) -> U64Vector4: ...
    @property
    def lwyw(self) -> U64Vector4: ...
    @property
    def lwyo(self) -> U64Vector4: ...
    @property
    def lwyl(self) -> U64Vector4: ...
    @property
    def lwzx(self) -> U64Vector4: ...
    @property
    def lwzy(self) -> U64Vector4: ...
    @property
    def lwzz(self) -> U64Vector4: ...
    @property
    def lwzw(self) -> U64Vector4: ...
    @property
    def lwzo(self) -> U64Vector4: ...
    @property
    def lwzl(self) -> U64Vector4: ...
    @property
    def lwwx(self) -> U64Vector4: ...
    @property
    def lwwy(self) -> U64Vector4: ...
    @property
    def lwwz(self) -> U64Vector4: ...
    @property
    def lwww(self) -> U64Vector4: ...
    @property
    def lwwo(self) -> U64Vector4: ...
    @property
    def lwwl(self) -> U64Vector4: ...
    @property
    def lwox(self) -> U64Vector4: ...
    @property
    def lwoy(self) -> U64Vector4: ...
    @property
    def lwoz(self) -> U64Vector4: ...
    @property
    def lwow(self) -> U64Vector4: ...
    @property
    def lwoo(self) -> U64Vector4: ...
    @property
    def lwol(self) -> U64Vector4: ...
    @property
    def lwlx(self) -> U64Vector4: ...
    @property
    def lwly(self) -> U64Vector4: ...
    @property
    def lwlz(self) -> U64Vector4: ...
    @property
    def lwlw(self) -> U64Vector4: ...
    @property
    def lwlo(self) -> U64Vector4: ...
    @property
    def lwll(self) -> U64Vector4: ...
    @property
    def loxx(self) -> U64Vector4: ...
    @property
    def loxy(self) -> U64Vector4: ...
    @property
    def loxz(self) -> U64Vector4: ...
    @property
    def loxw(self) -> U64Vector4: ...
    @property
    def loxo(self) -> U64Vector4: ...
    @property
    def loxl(self) -> U64Vector4: ...
    @property
    def loyx(self) -> U64Vector4: ...
    @property
    def loyy(self) -> U64Vector4: ...
    @property
    def loyz(self) -> U64Vector4: ...
    @property
    def loyw(self) -> U64Vector4: ...
    @property
    def loyo(self) -> U64Vector4: ...
    @property
    def loyl(self) -> U64Vector4: ...
    @property
    def lozx(self) -> U64Vector4: ...
    @property
    def lozy(self) -> U64Vector4: ...
    @property
    def lozz(self) -> U64Vector4: ...
    @property
    def lozw(self) -> U64Vector4: ...
    @property
    def lozo(self) -> U64Vector4: ...
    @property
    def lozl(self) -> U64Vector4: ...
    @property
    def lowx(self) -> U64Vector4: ...
    @property
    def lowy(self) -> U64Vector4: ...
    @property
    def lowz(self) -> U64Vector4: ...
    @property
    def loww(self) -> U64Vector4: ...
    @property
    def lowo(self) -> U64Vector4: ...
    @property
    def lowl(self) -> U64Vector4: ...
    @property
    def loox(self) -> U64Vector4: ...
    @property
    def looy(self) -> U64Vector4: ...
    @property
    def looz(self) -> U64Vector4: ...
    @property
    def loow(self) -> U64Vector4: ...
    @property
    def lolx(self) -> U64Vector4: ...
    @property
    def loly(self) -> U64Vector4: ...
    @property
    def lolz(self) -> U64Vector4: ...
    @property
    def lolw(self) -> U64Vector4: ...
    @property
    def llxx(self) -> U64Vector4: ...
    @property
    def llxy(self) -> U64Vector4: ...
    @property
    def llxz(self) -> U64Vector4: ...
    @property
    def llxw(self) -> U64Vector4: ...
    @property
    def llxo(self) -> U64Vector4: ...
    @property
    def llxl(self) -> U64Vector4: ...
    @property
    def llyx(self) -> U64Vector4: ...
    @property
    def llyy(self) -> U64Vector4: ...
    @property
    def llyz(self) -> U64Vector4: ...
    @property
    def llyw(self) -> U64Vector4: ...
    @property
    def llyo(self) -> U64Vector4: ...
    @property
    def llyl(self) -> U64Vector4: ...
    @property
    def llzx(self) -> U64Vector4: ...
    @property
    def llzy(self) -> U64Vector4: ...
    @property
    def llzz(self) -> U64Vector4: ...
    @property
    def llzw(self) -> U64Vector4: ...
    @property
    def llzo(self) -> U64Vector4: ...
    @property
    def llzl(self) -> U64Vector4: ...
    @property
    def llwx(self) -> U64Vector4: ...
    @property
    def llwy(self) -> U64Vector4: ...
    @property
    def llwz(self) -> U64Vector4: ...
    @property
    def llww(self) -> U64Vector4: ...
    @property
    def llwo(self) -> U64Vector4: ...
    @property
    def llwl(self) -> U64Vector4: ...
    @property
    def llox(self) -> U64Vector4: ...
    @property
    def lloy(self) -> U64Vector4: ...
    @property
    def lloz(self) -> U64Vector4: ...
    @property
    def llow(self) -> U64Vector4: ...
    @property
    def lllx(self) -> U64Vector4: ...
    @property
    def llly(self) -> U64Vector4: ...
    @property
    def lllz(self) -> U64Vector4: ...
    @property
    def lllw(self) -> U64Vector4: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    @property
    def a(self) -> int: ...
    @property
    def rr(self) -> U64Vector2: ...
    @property
    def rg(self) -> U64Vector2: ...
    @property
    def rb(self) -> U64Vector2: ...
    @property
    def ra(self) -> U64Vector2: ...
    @property
    def ro(self) -> U64Vector2: ...
    @property
    def rl(self) -> U64Vector2: ...
    @property
    def gr(self) -> U64Vector2: ...
    @property
    def gg(self) -> U64Vector2: ...
    @property
    def gb(self) -> U64Vector2: ...
    @property
    def ga(self) -> U64Vector2: ...
    @property
    def go(self) -> U64Vector2: ...
    @property
    def gl(self) -> U64Vector2: ...
    @property
    def br(self) -> U64Vector2: ...
    @property
    def bg(self) -> U64Vector2: ...
    @property
    def bb(self) -> U64Vector2: ...
    @property
    def ba(self) -> U64Vector2: ...
    @property
    def bo(self) -> U64Vector2: ...
    @property
    def bl(self) -> U64Vector2: ...
    @property
    def ar(self) -> U64Vector2: ...
    @property
    def ag(self) -> U64Vector2: ...
    @property
    def ab(self) -> U64Vector2: ...
    @property
    def aa(self) -> U64Vector2: ...
    @property
    def ao(self) -> U64Vector2: ...
    @property
    def al(self) -> U64Vector2: ...
    @property
    def og(self) -> U64Vector2: ...
    @property
    def ob(self) -> U64Vector2: ...
    @property
    def oa(self) -> U64Vector2: ...
    @property
    def lr(self) -> U64Vector2: ...
    @property
    def lg(self) -> U64Vector2: ...
    @property
    def lb(self) -> U64Vector2: ...
    @property
    def la(self) -> U64Vector2: ...
    @property
    def rrr(self) -> U64Vector3: ...
    @property
    def rrg(self) -> U64Vector3: ...
    @property
    def rrb(self) -> U64Vector3: ...
    @property
    def rra(self) -> U64Vector3: ...
    @property
    def rro(self) -> U64Vector3: ...
    @property
    def rrl(self) -> U64Vector3: ...
    @property
    def rgr(self) -> U64Vector3: ...
    @property
    def rgg(self) -> U64Vector3: ...
    @property
    def rgb(self) -> U64Vector3: ...
    @property
    def rga(self) -> U64Vector3: ...
    @property
    def rgo(self) -> U64Vector3: ...
    @property
    def rgl(self) -> U64Vector3: ...
    @property
    def rbr(self) -> U64Vector3: ...
    @property
    def rbg(self) -> U64Vector3: ...
    @property
    def rbb(self) -> U64Vector3: ...
    @property
    def rba(self) -> U64Vector3: ...
    @property
    def rbo(self) -> U64Vector3: ...
    @property
    def rbl(self) -> U64Vector3: ...
    @property
    def rar(self) -> U64Vector3: ...
    @property
    def rag(self) -> U64Vector3: ...
    @property
    def rab(self) -> U64Vector3: ...
    @property
    def raa(self) -> U64Vector3: ...
    @property
    def rao(self) -> U64Vector3: ...
    @property
    def ral(self) -> U64Vector3: ...
    @property
    def ror(self) -> U64Vector3: ...
    @property
    def rog(self) -> U64Vector3: ...
    @property
    def rob(self) -> U64Vector3: ...
    @property
    def roa(self) -> U64Vector3: ...
    @property
    def roo(self) -> U64Vector3: ...
    @property
    def rol(self) -> U64Vector3: ...
    @property
    def rlr(self) -> U64Vector3: ...
    @property
    def rlg(self) -> U64Vector3: ...
    @property
    def rlb(self) -> U64Vector3: ...
    @property
    def rla(self) -> U64Vector3: ...
    @property
    def rlo(self) -> U64Vector3: ...
    @property
    def rll(self) -> U64Vector3: ...
    @property
    def grr(self) -> U64Vector3: ...
    @property
    def grg(self) -> U64Vector3: ...
    @property
    def grb(self) -> U64Vector3: ...
    @property
    def gra(self) -> U64Vector3: ...
    @property
    def gro(self) -> U64Vector3: ...
    @property
    def grl(self) -> U64Vector3: ...
    @property
    def ggr(self) -> U64Vector3: ...
    @property
    def ggg(self) -> U64Vector3: ...
    @property
    def ggb(self) -> U64Vector3: ...
    @property
    def gga(self) -> U64Vector3: ...
    @property
    def ggo(self) -> U64Vector3: ...
    @property
    def ggl(self) -> U64Vector3: ...
    @property
    def gbr(self) -> U64Vector3: ...
    @property
    def gbg(self) -> U64Vector3: ...
    @property
    def gbb(self) -> U64Vector3: ...
    @property
    def gba(self) -> U64Vector3: ...
    @property
    def gbo(self) -> U64Vector3: ...
    @property
    def gbl(self) -> U64Vector3: ...
    @property
    def gar(self) -> U64Vector3: ...
    @property
    def gag(self) -> U64Vector3: ...
    @property
    def gab(self) -> U64Vector3: ...
    @property
    def gaa(self) -> U64Vector3: ...
    @property
    def gao(self) -> U64Vector3: ...
    @property
    def gal(self) -> U64Vector3: ...
    @property
    def gor(self) -> U64Vector3: ...
    @property
    def gog(self) -> U64Vector3: ...
    @property
    def gob(self) -> U64Vector3: ...
    @property
    def goa(self) -> U64Vector3: ...
    @property
    def goo(self) -> U64Vector3: ...
    @property
    def gol(self) -> U64Vector3: ...
    @property
    def glr(self) -> U64Vector3: ...
    @property
    def glg(self) -> U64Vector3: ...
    @property
    def glb(self) -> U64Vector3: ...
    @property
    def gla(self) -> U64Vector3: ...
    @property
    def glo(self) -> U64Vector3: ...
    @property
    def gll(self) -> U64Vector3: ...
    @property
    def brr(self) -> U64Vector3: ...
    @property
    def brg(self) -> U64Vector3: ...
    @property
    def brb(self) -> U64Vector3: ...
    @property
    def bra(self) -> U64Vector3: ...
    @property
    def bro(self) -> U64Vector3: ...
    @property
    def brl(self) -> U64Vector3: ...
    @property
    def bgr(self) -> U64Vector3: ...
    @property
    def bgg(self) -> U64Vector3: ...
    @property
    def bgb(self) -> U64Vector3: ...
    @property
    def bga(self) -> U64Vector3: ...
    @property
    def bgo(self) -> U64Vector3: ...
    @property
    def bgl(self) -> U64Vector3: ...
    @property
    def bbr(self) -> U64Vector3: ...
    @property
    def bbg(self) -> U64Vector3: ...
    @property
    def bbb(self) -> U64Vector3: ...
    @property
    def bba(self) -> U64Vector3: ...
    @property
    def bbo(self) -> U64Vector3: ...
    @property
    def bbl(self) -> U64Vector3: ...
    @property
    def bar(self) -> U64Vector3: ...
    @property
    def bag(self) -> U64Vector3: ...
    @property
    def bab(self) -> U64Vector3: ...
    @property
    def baa(self) -> U64Vector3: ...
    @property
    def bao(self) -> U64Vector3: ...
    @property
    def bal(self) -> U64Vector3: ...
    @property
    def bor(self) -> U64Vector3: ...
    @property
    def bog(self) -> U64Vector3: ...
    @property
    def bob(self) -> U64Vector3: ...
    @property
    def boa(self) -> U64Vector3: ...
    @property
    def boo(self) -> U64Vector3: ...
    @property
    def bol(self) -> U64Vector3: ...
    @property
    def blr(self) -> U64Vector3: ...
    @property
    def blg(self) -> U64Vector3: ...
    @property
    def blb(self) -> U64Vector3: ...
    @property
    def bla(self) -> U64Vector3: ...
    @property
    def blo(self) -> U64Vector3: ...
    @property
    def bll(self) -> U64Vector3: ...
    @property
    def arr(self) -> U64Vector3: ...
    @property
    def arg(self) -> U64Vector3: ...
    @property
    def arb(self) -> U64Vector3: ...
    @property
    def ara(self) -> U64Vector3: ...
    @property
    def aro(self) -> U64Vector3: ...
    @property
    def arl(self) -> U64Vector3: ...
    @property
    def agr(self) -> U64Vector3: ...
    @property
    def agg(self) -> U64Vector3: ...
    @property
    def agb(self) -> U64Vector3: ...
    @property
    def aga(self) -> U64Vector3: ...
    @property
    def ago(self) -> U64Vector3: ...
    @property
    def agl(self) -> U64Vector3: ...
    @property
    def abr(self) -> U64Vector3: ...
    @property
    def abg(self) -> U64Vector3: ...
    @property
    def abb(self) -> U64Vector3: ...
    @property
    def aba(self) -> U64Vector3: ...
    @property
    def abo(self) -> U64Vector3: ...
    @property
    def abl(self) -> U64Vector3: ...
    @property
    def aar(self) -> U64Vector3: ...
    @property
    def aag(self) -> U64Vector3: ...
    @property
    def aab(self) -> U64Vector3: ...
    @property
    def aaa(self) -> U64Vector3: ...
    @property
    def aao(self) -> U64Vector3: ...
    @property
    def aal(self) -> U64Vector3: ...
    @property
    def aor(self) -> U64Vector3: ...
    @property
    def aog(self) -> U64Vector3: ...
    @property
    def aob(self) -> U64Vector3: ...
    @property
    def aoa(self) -> U64Vector3: ...
    @property
    def aoo(self) -> U64Vector3: ...
    @property
    def aol(self) -> U64Vector3: ...
    @property
    def alr(self) -> U64Vector3: ...
    @property
    def alg(self) -> U64Vector3: ...
    @property
    def alb(self) -> U64Vector3: ...
    @property
    def ala(self) -> U64Vector3: ...
    @property
    def alo(self) -> U64Vector3: ...
    @property
    def all(self) -> U64Vector3: ...
    @property
    def orr(self) -> U64Vector3: ...
    @property
    def org(self) -> U64Vector3: ...
    @property
    def orb(self) -> U64Vector3: ...
    @property
    def ora(self) -> U64Vector3: ...
    @property
    def oro(self) -> U64Vector3: ...
    @property
    def orl(self) -> U64Vector3: ...
    @property
    def ogr(self) -> U64Vector3: ...
    @property
    def ogg(self) -> U64Vector3: ...
    @property
    def ogb(self) -> U64Vector3: ...
    @property
    def oga(self) -> U64Vector3: ...
    @property
    def ogo(self) -> U64Vector3: ...
    @property
    def ogl(self) -> U64Vector3: ...
    @property
    def obr(self) -> U64Vector3: ...
    @property
    def obg(self) -> U64Vector3: ...
    @property
    def obb(self) -> U64Vector3: ...
    @property
    def oba(self) -> U64Vector3: ...
    @property
    def obo(self) -> U64Vector3: ...
    @property
    def obl(self) -> U64Vector3: ...
    @property
    def oar(self) -> U64Vector3: ...
    @property
    def oag(self) -> U64Vector3: ...
    @property
    def oab(self) -> U64Vector3: ...
    @property
    def oaa(self) -> U64Vector3: ...
    @property
    def oao(self) -> U64Vector3: ...
    @property
    def oal(self) -> U64Vector3: ...
    @property
    def oor(self) -> U64Vector3: ...
    @property
    def oog(self) -> U64Vector3: ...
    @property
    def oob(self) -> U64Vector3: ...
    @property
    def ooa(self) -> U64Vector3: ...
    @property
    def olr(self) -> U64Vector3: ...
    @property
    def olg(self) -> U64Vector3: ...
    @property
    def olb(self) -> U64Vector3: ...
    @property
    def ola(self) -> U64Vector3: ...
    @property
    def lrr(self) -> U64Vector3: ...
    @property
    def lrg(self) -> U64Vector3: ...
    @property
    def lrb(self) -> U64Vector3: ...
    @property
    def lra(self) -> U64Vector3: ...
    @property
    def lro(self) -> U64Vector3: ...
    @property
    def lrl(self) -> U64Vector3: ...
    @property
    def lgr(self) -> U64Vector3: ...
    @property
    def lgg(self) -> U64Vector3: ...
    @property
    def lgb(self) -> U64Vector3: ...
    @property
    def lga(self) -> U64Vector3: ...
    @property
    def lgo(self) -> U64Vector3: ...
    @property
    def lgl(self) -> U64Vector3: ...
    @property
    def lbr(self) -> U64Vector3: ...
    @property
    def lbg(self) -> U64Vector3: ...
    @property
    def lbb(self) -> U64Vector3: ...
    @property
    def lba(self) -> U64Vector3: ...
    @property
    def lbo(self) -> U64Vector3: ...
    @property
    def lbl(self) -> U64Vector3: ...
    @property
    def lar(self) -> U64Vector3: ...
    @property
    def lag(self) -> U64Vector3: ...
    @property
    def lab(self) -> U64Vector3: ...
    @property
    def laa(self) -> U64Vector3: ...
    @property
    def lao(self) -> U64Vector3: ...
    @property
    def lal(self) -> U64Vector3: ...
    @property
    def lor(self) -> U64Vector3: ...
    @property
    def log(self) -> U64Vector3: ...
    @property
    def lob(self) -> U64Vector3: ...
    @property
    def loa(self) -> U64Vector3: ...
    @property
    def llr(self) -> U64Vector3: ...
    @property
    def llg(self) -> U64Vector3: ...
    @property
    def llb(self) -> U64Vector3: ...
    @property
    def lla(self) -> U64Vector3: ...
    @property
    def rrrr(self) -> U64Vector4: ...
    @property
    def rrrg(self) -> U64Vector4: ...
    @property
    def rrrb(self) -> U64Vector4: ...
    @property
    def rrra(self) -> U64Vector4: ...
    @property
    def rrro(self) -> U64Vector4: ...
    @property
    def rrrl(self) -> U64Vector4: ...
    @property
    def rrgr(self) -> U64Vector4: ...
    @property
    def rrgg(self) -> U64Vector4: ...
    @property
    def rrgb(self) -> U64Vector4: ...
    @property
    def rrga(self) -> U64Vector4: ...
    @property
    def rrgo(self) -> U64Vector4: ...
    @property
    def rrgl(self) -> U64Vector4: ...
    @property
    def rrbr(self) -> U64Vector4: ...
    @property
    def rrbg(self) -> U64Vector4: ...
    @property
    def rrbb(self) -> U64Vector4: ...
    @property
    def rrba(self) -> U64Vector4: ...
    @property
    def rrbo(self) -> U64Vector4: ...
    @property
    def rrbl(self) -> U64Vector4: ...
    @property
    def rrar(self) -> U64Vector4: ...
    @property
    def rrag(self) -> U64Vector4: ...
    @property
    def rrab(self) -> U64Vector4: ...
    @property
    def rraa(self) -> U64Vector4: ...
    @property
    def rrao(self) -> U64Vector4: ...
    @property
    def rral(self) -> U64Vector4: ...
    @property
    def rror(self) -> U64Vector4: ...
    @property
    def rrog(self) -> U64Vector4: ...
    @property
    def rrob(self) -> U64Vector4: ...
    @property
    def rroa(self) -> U64Vector4: ...
    @property
    def rroo(self) -> U64Vector4: ...
    @property
    def rrol(self) -> U64Vector4: ...
    @property
    def rrlr(self) -> U64Vector4: ...
    @property
    def rrlg(self) -> U64Vector4: ...
    @property
    def rrlb(self) -> U64Vector4: ...
    @property
    def rrla(self) -> U64Vector4: ...
    @property
    def rrlo(self) -> U64Vector4: ...
    @property
    def rrll(self) -> U64Vector4: ...
    @property
    def rgrr(self) -> U64Vector4: ...
    @property
    def rgrg(self) -> U64Vector4: ...
    @property
    def rgrb(self) -> U64Vector4: ...
    @property
    def rgra(self) -> U64Vector4: ...
    @property
    def rgro(self) -> U64Vector4: ...
    @property
    def rgrl(self) -> U64Vector4: ...
    @property
    def rggr(self) -> U64Vector4: ...
    @property
    def rggg(self) -> U64Vector4: ...
    @property
    def rggb(self) -> U64Vector4: ...
    @property
    def rgga(self) -> U64Vector4: ...
    @property
    def rggo(self) -> U64Vector4: ...
    @property
    def rggl(self) -> U64Vector4: ...
    @property
    def rgbr(self) -> U64Vector4: ...
    @property
    def rgbg(self) -> U64Vector4: ...
    @property
    def rgbb(self) -> U64Vector4: ...
    @property
    def rgba(self) -> U64Vector4: ...
    @property
    def rgbo(self) -> U64Vector4: ...
    @property
    def rgbl(self) -> U64Vector4: ...
    @property
    def rgar(self) -> U64Vector4: ...
    @property
    def rgag(self) -> U64Vector4: ...
    @property
    def rgab(self) -> U64Vector4: ...
    @property
    def rgaa(self) -> U64Vector4: ...
    @property
    def rgao(self) -> U64Vector4: ...
    @property
    def rgal(self) -> U64Vector4: ...
    @property
    def rgor(self) -> U64Vector4: ...
    @property
    def rgog(self) -> U64Vector4: ...
    @property
    def rgob(self) -> U64Vector4: ...
    @property
    def rgoa(self) -> U64Vector4: ...
    @property
    def rgoo(self) -> U64Vector4: ...
    @property
    def rgol(self) -> U64Vector4: ...
    @property
    def rglr(self) -> U64Vector4: ...
    @property
    def rglg(self) -> U64Vector4: ...
    @property
    def rglb(self) -> U64Vector4: ...
    @property
    def rgla(self) -> U64Vector4: ...
    @property
    def rglo(self) -> U64Vector4: ...
    @property
    def rgll(self) -> U64Vector4: ...
    @property
    def rbrr(self) -> U64Vector4: ...
    @property
    def rbrg(self) -> U64Vector4: ...
    @property
    def rbrb(self) -> U64Vector4: ...
    @property
    def rbra(self) -> U64Vector4: ...
    @property
    def rbro(self) -> U64Vector4: ...
    @property
    def rbrl(self) -> U64Vector4: ...
    @property
    def rbgr(self) -> U64Vector4: ...
    @property
    def rbgg(self) -> U64Vector4: ...
    @property
    def rbgb(self) -> U64Vector4: ...
    @property
    def rbga(self) -> U64Vector4: ...
    @property
    def rbgo(self) -> U64Vector4: ...
    @property
    def rbgl(self) -> U64Vector4: ...
    @property
    def rbbr(self) -> U64Vector4: ...
    @property
    def rbbg(self) -> U64Vector4: ...
    @property
    def rbbb(self) -> U64Vector4: ...
    @property
    def rbba(self) -> U64Vector4: ...
    @property
    def rbbo(self) -> U64Vector4: ...
    @property
    def rbbl(self) -> U64Vector4: ...
    @property
    def rbar(self) -> U64Vector4: ...
    @property
    def rbag(self) -> U64Vector4: ...
    @property
    def rbab(self) -> U64Vector4: ...
    @property
    def rbaa(self) -> U64Vector4: ...
    @property
    def rbao(self) -> U64Vector4: ...
    @property
    def rbal(self) -> U64Vector4: ...
    @property
    def rbor(self) -> U64Vector4: ...
    @property
    def rbog(self) -> U64Vector4: ...
    @property
    def rbob(self) -> U64Vector4: ...
    @property
    def rboa(self) -> U64Vector4: ...
    @property
    def rboo(self) -> U64Vector4: ...
    @property
    def rbol(self) -> U64Vector4: ...
    @property
    def rblr(self) -> U64Vector4: ...
    @property
    def rblg(self) -> U64Vector4: ...
    @property
    def rblb(self) -> U64Vector4: ...
    @property
    def rbla(self) -> U64Vector4: ...
    @property
    def rblo(self) -> U64Vector4: ...
    @property
    def rbll(self) -> U64Vector4: ...
    @property
    def rarr(self) -> U64Vector4: ...
    @property
    def rarg(self) -> U64Vector4: ...
    @property
    def rarb(self) -> U64Vector4: ...
    @property
    def rara(self) -> U64Vector4: ...
    @property
    def raro(self) -> U64Vector4: ...
    @property
    def rarl(self) -> U64Vector4: ...
    @property
    def ragr(self) -> U64Vector4: ...
    @property
    def ragg(self) -> U64Vector4: ...
    @property
    def ragb(self) -> U64Vector4: ...
    @property
    def raga(self) -> U64Vector4: ...
    @property
    def rago(self) -> U64Vector4: ...
    @property
    def ragl(self) -> U64Vector4: ...
    @property
    def rabr(self) -> U64Vector4: ...
    @property
    def rabg(self) -> U64Vector4: ...
    @property
    def rabb(self) -> U64Vector4: ...
    @property
    def raba(self) -> U64Vector4: ...
    @property
    def rabo(self) -> U64Vector4: ...
    @property
    def rabl(self) -> U64Vector4: ...
    @property
    def raar(self) -> U64Vector4: ...
    @property
    def raag(self) -> U64Vector4: ...
    @property
    def raab(self) -> U64Vector4: ...
    @property
    def raaa(self) -> U64Vector4: ...
    @property
    def raao(self) -> U64Vector4: ...
    @property
    def raal(self) -> U64Vector4: ...
    @property
    def raor(self) -> U64Vector4: ...
    @property
    def raog(self) -> U64Vector4: ...
    @property
    def raob(self) -> U64Vector4: ...
    @property
    def raoa(self) -> U64Vector4: ...
    @property
    def raoo(self) -> U64Vector4: ...
    @property
    def raol(self) -> U64Vector4: ...
    @property
    def ralr(self) -> U64Vector4: ...
    @property
    def ralg(self) -> U64Vector4: ...
    @property
    def ralb(self) -> U64Vector4: ...
    @property
    def rala(self) -> U64Vector4: ...
    @property
    def ralo(self) -> U64Vector4: ...
    @property
    def rall(self) -> U64Vector4: ...
    @property
    def rorr(self) -> U64Vector4: ...
    @property
    def rorg(self) -> U64Vector4: ...
    @property
    def rorb(self) -> U64Vector4: ...
    @property
    def rora(self) -> U64Vector4: ...
    @property
    def roro(self) -> U64Vector4: ...
    @property
    def rorl(self) -> U64Vector4: ...
    @property
    def rogr(self) -> U64Vector4: ...
    @property
    def rogg(self) -> U64Vector4: ...
    @property
    def rogb(self) -> U64Vector4: ...
    @property
    def roga(self) -> U64Vector4: ...
    @property
    def rogo(self) -> U64Vector4: ...
    @property
    def rogl(self) -> U64Vector4: ...
    @property
    def robr(self) -> U64Vector4: ...
    @property
    def robg(self) -> U64Vector4: ...
    @property
    def robb(self) -> U64Vector4: ...
    @property
    def roba(self) -> U64Vector4: ...
    @property
    def robo(self) -> U64Vector4: ...
    @property
    def robl(self) -> U64Vector4: ...
    @property
    def roar(self) -> U64Vector4: ...
    @property
    def roag(self) -> U64Vector4: ...
    @property
    def roab(self) -> U64Vector4: ...
    @property
    def roaa(self) -> U64Vector4: ...
    @property
    def roao(self) -> U64Vector4: ...
    @property
    def roal(self) -> U64Vector4: ...
    @property
    def roor(self) -> U64Vector4: ...
    @property
    def roog(self) -> U64Vector4: ...
    @property
    def roob(self) -> U64Vector4: ...
    @property
    def rooa(self) -> U64Vector4: ...
    @property
    def rooo(self) -> U64Vector4: ...
    @property
    def rool(self) -> U64Vector4: ...
    @property
    def rolr(self) -> U64Vector4: ...
    @property
    def rolg(self) -> U64Vector4: ...
    @property
    def rolb(self) -> U64Vector4: ...
    @property
    def rola(self) -> U64Vector4: ...
    @property
    def rolo(self) -> U64Vector4: ...
    @property
    def roll(self) -> U64Vector4: ...
    @property
    def rlrr(self) -> U64Vector4: ...
    @property
    def rlrg(self) -> U64Vector4: ...
    @property
    def rlrb(self) -> U64Vector4: ...
    @property
    def rlra(self) -> U64Vector4: ...
    @property
    def rlro(self) -> U64Vector4: ...
    @property
    def rlrl(self) -> U64Vector4: ...
    @property
    def rlgr(self) -> U64Vector4: ...
    @property
    def rlgg(self) -> U64Vector4: ...
    @property
    def rlgb(self) -> U64Vector4: ...
    @property
    def rlga(self) -> U64Vector4: ...
    @property
    def rlgo(self) -> U64Vector4: ...
    @property
    def rlgl(self) -> U64Vector4: ...
    @property
    def rlbr(self) -> U64Vector4: ...
    @property
    def rlbg(self) -> U64Vector4: ...
    @property
    def rlbb(self) -> U64Vector4: ...
    @property
    def rlba(self) -> U64Vector4: ...
    @property
    def rlbo(self) -> U64Vector4: ...
    @property
    def rlbl(self) -> U64Vector4: ...
    @property
    def rlar(self) -> U64Vector4: ...
    @property
    def rlag(self) -> U64Vector4: ...
    @property
    def rlab(self) -> U64Vector4: ...
    @property
    def rlaa(self) -> U64Vector4: ...
    @property
    def rlao(self) -> U64Vector4: ...
    @property
    def rlal(self) -> U64Vector4: ...
    @property
    def rlor(self) -> U64Vector4: ...
    @property
    def rlog(self) -> U64Vector4: ...
    @property
    def rlob(self) -> U64Vector4: ...
    @property
    def rloa(self) -> U64Vector4: ...
    @property
    def rloo(self) -> U64Vector4: ...
    @property
    def rlol(self) -> U64Vector4: ...
    @property
    def rllr(self) -> U64Vector4: ...
    @property
    def rllg(self) -> U64Vector4: ...
    @property
    def rllb(self) -> U64Vector4: ...
    @property
    def rlla(self) -> U64Vector4: ...
    @property
    def rllo(self) -> U64Vector4: ...
    @property
    def rlll(self) -> U64Vector4: ...
    @property
    def grrr(self) -> U64Vector4: ...
    @property
    def grrg(self) -> U64Vector4: ...
    @property
    def grrb(self) -> U64Vector4: ...
    @property
    def grra(self) -> U64Vector4: ...
    @property
    def grro(self) -> U64Vector4: ...
    @property
    def grrl(self) -> U64Vector4: ...
    @property
    def grgr(self) -> U64Vector4: ...
    @property
    def grgg(self) -> U64Vector4: ...
    @property
    def grgb(self) -> U64Vector4: ...
    @property
    def grga(self) -> U64Vector4: ...
    @property
    def grgo(self) -> U64Vector4: ...
    @property
    def grgl(self) -> U64Vector4: ...
    @property
    def grbr(self) -> U64Vector4: ...
    @property
    def grbg(self) -> U64Vector4: ...
    @property
    def grbb(self) -> U64Vector4: ...
    @property
    def grba(self) -> U64Vector4: ...
    @property
    def grbo(self) -> U64Vector4: ...
    @property
    def grbl(self) -> U64Vector4: ...
    @property
    def grar(self) -> U64Vector4: ...
    @property
    def grag(self) -> U64Vector4: ...
    @property
    def grab(self) -> U64Vector4: ...
    @property
    def graa(self) -> U64Vector4: ...
    @property
    def grao(self) -> U64Vector4: ...
    @property
    def gral(self) -> U64Vector4: ...
    @property
    def gror(self) -> U64Vector4: ...
    @property
    def grog(self) -> U64Vector4: ...
    @property
    def grob(self) -> U64Vector4: ...
    @property
    def groa(self) -> U64Vector4: ...
    @property
    def groo(self) -> U64Vector4: ...
    @property
    def grol(self) -> U64Vector4: ...
    @property
    def grlr(self) -> U64Vector4: ...
    @property
    def grlg(self) -> U64Vector4: ...
    @property
    def grlb(self) -> U64Vector4: ...
    @property
    def grla(self) -> U64Vector4: ...
    @property
    def grlo(self) -> U64Vector4: ...
    @property
    def grll(self) -> U64Vector4: ...
    @property
    def ggrr(self) -> U64Vector4: ...
    @property
    def ggrg(self) -> U64Vector4: ...
    @property
    def ggrb(self) -> U64Vector4: ...
    @property
    def ggra(self) -> U64Vector4: ...
    @property
    def ggro(self) -> U64Vector4: ...
    @property
    def ggrl(self) -> U64Vector4: ...
    @property
    def gggr(self) -> U64Vector4: ...
    @property
    def gggg(self) -> U64Vector4: ...
    @property
    def gggb(self) -> U64Vector4: ...
    @property
    def ggga(self) -> U64Vector4: ...
    @property
    def gggo(self) -> U64Vector4: ...
    @property
    def gggl(self) -> U64Vector4: ...
    @property
    def ggbr(self) -> U64Vector4: ...
    @property
    def ggbg(self) -> U64Vector4: ...
    @property
    def ggbb(self) -> U64Vector4: ...
    @property
    def ggba(self) -> U64Vector4: ...
    @property
    def ggbo(self) -> U64Vector4: ...
    @property
    def ggbl(self) -> U64Vector4: ...
    @property
    def ggar(self) -> U64Vector4: ...
    @property
    def ggag(self) -> U64Vector4: ...
    @property
    def ggab(self) -> U64Vector4: ...
    @property
    def ggaa(self) -> U64Vector4: ...
    @property
    def ggao(self) -> U64Vector4: ...
    @property
    def ggal(self) -> U64Vector4: ...
    @property
    def ggor(self) -> U64Vector4: ...
    @property
    def ggog(self) -> U64Vector4: ...
    @property
    def ggob(self) -> U64Vector4: ...
    @property
    def ggoa(self) -> U64Vector4: ...
    @property
    def ggoo(self) -> U64Vector4: ...
    @property
    def ggol(self) -> U64Vector4: ...
    @property
    def gglr(self) -> U64Vector4: ...
    @property
    def gglg(self) -> U64Vector4: ...
    @property
    def gglb(self) -> U64Vector4: ...
    @property
    def ggla(self) -> U64Vector4: ...
    @property
    def gglo(self) -> U64Vector4: ...
    @property
    def ggll(self) -> U64Vector4: ...
    @property
    def gbrr(self) -> U64Vector4: ...
    @property
    def gbrg(self) -> U64Vector4: ...
    @property
    def gbrb(self) -> U64Vector4: ...
    @property
    def gbra(self) -> U64Vector4: ...
    @property
    def gbro(self) -> U64Vector4: ...
    @property
    def gbrl(self) -> U64Vector4: ...
    @property
    def gbgr(self) -> U64Vector4: ...
    @property
    def gbgg(self) -> U64Vector4: ...
    @property
    def gbgb(self) -> U64Vector4: ...
    @property
    def gbga(self) -> U64Vector4: ...
    @property
    def gbgo(self) -> U64Vector4: ...
    @property
    def gbgl(self) -> U64Vector4: ...
    @property
    def gbbr(self) -> U64Vector4: ...
    @property
    def gbbg(self) -> U64Vector4: ...
    @property
    def gbbb(self) -> U64Vector4: ...
    @property
    def gbba(self) -> U64Vector4: ...
    @property
    def gbbo(self) -> U64Vector4: ...
    @property
    def gbbl(self) -> U64Vector4: ...
    @property
    def gbar(self) -> U64Vector4: ...
    @property
    def gbag(self) -> U64Vector4: ...
    @property
    def gbab(self) -> U64Vector4: ...
    @property
    def gbaa(self) -> U64Vector4: ...
    @property
    def gbao(self) -> U64Vector4: ...
    @property
    def gbal(self) -> U64Vector4: ...
    @property
    def gbor(self) -> U64Vector4: ...
    @property
    def gbog(self) -> U64Vector4: ...
    @property
    def gbob(self) -> U64Vector4: ...
    @property
    def gboa(self) -> U64Vector4: ...
    @property
    def gboo(self) -> U64Vector4: ...
    @property
    def gbol(self) -> U64Vector4: ...
    @property
    def gblr(self) -> U64Vector4: ...
    @property
    def gblg(self) -> U64Vector4: ...
    @property
    def gblb(self) -> U64Vector4: ...
    @property
    def gbla(self) -> U64Vector4: ...
    @property
    def gblo(self) -> U64Vector4: ...
    @property
    def gbll(self) -> U64Vector4: ...
    @property
    def garr(self) -> U64Vector4: ...
    @property
    def garg(self) -> U64Vector4: ...
    @property
    def garb(self) -> U64Vector4: ...
    @property
    def gara(self) -> U64Vector4: ...
    @property
    def garo(self) -> U64Vector4: ...
    @property
    def garl(self) -> U64Vector4: ...
    @property
    def gagr(self) -> U64Vector4: ...
    @property
    def gagg(self) -> U64Vector4: ...
    @property
    def gagb(self) -> U64Vector4: ...
    @property
    def gaga(self) -> U64Vector4: ...
    @property
    def gago(self) -> U64Vector4: ...
    @property
    def gagl(self) -> U64Vector4: ...
    @property
    def gabr(self) -> U64Vector4: ...
    @property
    def gabg(self) -> U64Vector4: ...
    @property
    def gabb(self) -> U64Vector4: ...
    @property
    def gaba(self) -> U64Vector4: ...
    @property
    def gabo(self) -> U64Vector4: ...
    @property
    def gabl(self) -> U64Vector4: ...
    @property
    def gaar(self) -> U64Vector4: ...
    @property
    def gaag(self) -> U64Vector4: ...
    @property
    def gaab(self) -> U64Vector4: ...
    @property
    def gaaa(self) -> U64Vector4: ...
    @property
    def gaao(self) -> U64Vector4: ...
    @property
    def gaal(self) -> U64Vector4: ...
    @property
    def gaor(self) -> U64Vector4: ...
    @property
    def gaog(self) -> U64Vector4: ...
    @property
    def gaob(self) -> U64Vector4: ...
    @property
    def gaoa(self) -> U64Vector4: ...
    @property
    def gaoo(self) -> U64Vector4: ...
    @property
    def gaol(self) -> U64Vector4: ...
    @property
    def galr(self) -> U64Vector4: ...
    @property
    def galg(self) -> U64Vector4: ...
    @property
    def galb(self) -> U64Vector4: ...
    @property
    def gala(self) -> U64Vector4: ...
    @property
    def galo(self) -> U64Vector4: ...
    @property
    def gall(self) -> U64Vector4: ...
    @property
    def gorr(self) -> U64Vector4: ...
    @property
    def gorg(self) -> U64Vector4: ...
    @property
    def gorb(self) -> U64Vector4: ...
    @property
    def gora(self) -> U64Vector4: ...
    @property
    def goro(self) -> U64Vector4: ...
    @property
    def gorl(self) -> U64Vector4: ...
    @property
    def gogr(self) -> U64Vector4: ...
    @property
    def gogg(self) -> U64Vector4: ...
    @property
    def gogb(self) -> U64Vector4: ...
    @property
    def goga(self) -> U64Vector4: ...
    @property
    def gogo(self) -> U64Vector4: ...
    @property
    def gogl(self) -> U64Vector4: ...
    @property
    def gobr(self) -> U64Vector4: ...
    @property
    def gobg(self) -> U64Vector4: ...
    @property
    def gobb(self) -> U64Vector4: ...
    @property
    def goba(self) -> U64Vector4: ...
    @property
    def gobo(self) -> U64Vector4: ...
    @property
    def gobl(self) -> U64Vector4: ...
    @property
    def goar(self) -> U64Vector4: ...
    @property
    def goag(self) -> U64Vector4: ...
    @property
    def goab(self) -> U64Vector4: ...
    @property
    def goaa(self) -> U64Vector4: ...
    @property
    def goao(self) -> U64Vector4: ...
    @property
    def goal(self) -> U64Vector4: ...
    @property
    def goor(self) -> U64Vector4: ...
    @property
    def goog(self) -> U64Vector4: ...
    @property
    def goob(self) -> U64Vector4: ...
    @property
    def gooa(self) -> U64Vector4: ...
    @property
    def gooo(self) -> U64Vector4: ...
    @property
    def gool(self) -> U64Vector4: ...
    @property
    def golr(self) -> U64Vector4: ...
    @property
    def golg(self) -> U64Vector4: ...
    @property
    def golb(self) -> U64Vector4: ...
    @property
    def gola(self) -> U64Vector4: ...
    @property
    def golo(self) -> U64Vector4: ...
    @property
    def goll(self) -> U64Vector4: ...
    @property
    def glrr(self) -> U64Vector4: ...
    @property
    def glrg(self) -> U64Vector4: ...
    @property
    def glrb(self) -> U64Vector4: ...
    @property
    def glra(self) -> U64Vector4: ...
    @property
    def glro(self) -> U64Vector4: ...
    @property
    def glrl(self) -> U64Vector4: ...
    @property
    def glgr(self) -> U64Vector4: ...
    @property
    def glgg(self) -> U64Vector4: ...
    @property
    def glgb(self) -> U64Vector4: ...
    @property
    def glga(self) -> U64Vector4: ...
    @property
    def glgo(self) -> U64Vector4: ...
    @property
    def glgl(self) -> U64Vector4: ...
    @property
    def glbr(self) -> U64Vector4: ...
    @property
    def glbg(self) -> U64Vector4: ...
    @property
    def glbb(self) -> U64Vector4: ...
    @property
    def glba(self) -> U64Vector4: ...
    @property
    def glbo(self) -> U64Vector4: ...
    @property
    def glbl(self) -> U64Vector4: ...
    @property
    def glar(self) -> U64Vector4: ...
    @property
    def glag(self) -> U64Vector4: ...
    @property
    def glab(self) -> U64Vector4: ...
    @property
    def glaa(self) -> U64Vector4: ...
    @property
    def glao(self) -> U64Vector4: ...
    @property
    def glal(self) -> U64Vector4: ...
    @property
    def glor(self) -> U64Vector4: ...
    @property
    def glog(self) -> U64Vector4: ...
    @property
    def glob(self) -> U64Vector4: ...
    @property
    def gloa(self) -> U64Vector4: ...
    @property
    def gloo(self) -> U64Vector4: ...
    @property
    def glol(self) -> U64Vector4: ...
    @property
    def gllr(self) -> U64Vector4: ...
    @property
    def gllg(self) -> U64Vector4: ...
    @property
    def gllb(self) -> U64Vector4: ...
    @property
    def glla(self) -> U64Vector4: ...
    @property
    def gllo(self) -> U64Vector4: ...
    @property
    def glll(self) -> U64Vector4: ...
    @property
    def brrr(self) -> U64Vector4: ...
    @property
    def brrg(self) -> U64Vector4: ...
    @property
    def brrb(self) -> U64Vector4: ...
    @property
    def brra(self) -> U64Vector4: ...
    @property
    def brro(self) -> U64Vector4: ...
    @property
    def brrl(self) -> U64Vector4: ...
    @property
    def brgr(self) -> U64Vector4: ...
    @property
    def brgg(self) -> U64Vector4: ...
    @property
    def brgb(self) -> U64Vector4: ...
    @property
    def brga(self) -> U64Vector4: ...
    @property
    def brgo(self) -> U64Vector4: ...
    @property
    def brgl(self) -> U64Vector4: ...
    @property
    def brbr(self) -> U64Vector4: ...
    @property
    def brbg(self) -> U64Vector4: ...
    @property
    def brbb(self) -> U64Vector4: ...
    @property
    def brba(self) -> U64Vector4: ...
    @property
    def brbo(self) -> U64Vector4: ...
    @property
    def brbl(self) -> U64Vector4: ...
    @property
    def brar(self) -> U64Vector4: ...
    @property
    def brag(self) -> U64Vector4: ...
    @property
    def brab(self) -> U64Vector4: ...
    @property
    def braa(self) -> U64Vector4: ...
    @property
    def brao(self) -> U64Vector4: ...
    @property
    def bral(self) -> U64Vector4: ...
    @property
    def bror(self) -> U64Vector4: ...
    @property
    def brog(self) -> U64Vector4: ...
    @property
    def brob(self) -> U64Vector4: ...
    @property
    def broa(self) -> U64Vector4: ...
    @property
    def broo(self) -> U64Vector4: ...
    @property
    def brol(self) -> U64Vector4: ...
    @property
    def brlr(self) -> U64Vector4: ...
    @property
    def brlg(self) -> U64Vector4: ...
    @property
    def brlb(self) -> U64Vector4: ...
    @property
    def brla(self) -> U64Vector4: ...
    @property
    def brlo(self) -> U64Vector4: ...
    @property
    def brll(self) -> U64Vector4: ...
    @property
    def bgrr(self) -> U64Vector4: ...
    @property
    def bgrg(self) -> U64Vector4: ...
    @property
    def bgrb(self) -> U64Vector4: ...
    @property
    def bgra(self) -> U64Vector4: ...
    @property
    def bgro(self) -> U64Vector4: ...
    @property
    def bgrl(self) -> U64Vector4: ...
    @property
    def bggr(self) -> U64Vector4: ...
    @property
    def bggg(self) -> U64Vector4: ...
    @property
    def bggb(self) -> U64Vector4: ...
    @property
    def bgga(self) -> U64Vector4: ...
    @property
    def bggo(self) -> U64Vector4: ...
    @property
    def bggl(self) -> U64Vector4: ...
    @property
    def bgbr(self) -> U64Vector4: ...
    @property
    def bgbg(self) -> U64Vector4: ...
    @property
    def bgbb(self) -> U64Vector4: ...
    @property
    def bgba(self) -> U64Vector4: ...
    @property
    def bgbo(self) -> U64Vector4: ...
    @property
    def bgbl(self) -> U64Vector4: ...
    @property
    def bgar(self) -> U64Vector4: ...
    @property
    def bgag(self) -> U64Vector4: ...
    @property
    def bgab(self) -> U64Vector4: ...
    @property
    def bgaa(self) -> U64Vector4: ...
    @property
    def bgao(self) -> U64Vector4: ...
    @property
    def bgal(self) -> U64Vector4: ...
    @property
    def bgor(self) -> U64Vector4: ...
    @property
    def bgog(self) -> U64Vector4: ...
    @property
    def bgob(self) -> U64Vector4: ...
    @property
    def bgoa(self) -> U64Vector4: ...
    @property
    def bgoo(self) -> U64Vector4: ...
    @property
    def bgol(self) -> U64Vector4: ...
    @property
    def bglr(self) -> U64Vector4: ...
    @property
    def bglg(self) -> U64Vector4: ...
    @property
    def bglb(self) -> U64Vector4: ...
    @property
    def bgla(self) -> U64Vector4: ...
    @property
    def bglo(self) -> U64Vector4: ...
    @property
    def bgll(self) -> U64Vector4: ...
    @property
    def bbrr(self) -> U64Vector4: ...
    @property
    def bbrg(self) -> U64Vector4: ...
    @property
    def bbrb(self) -> U64Vector4: ...
    @property
    def bbra(self) -> U64Vector4: ...
    @property
    def bbro(self) -> U64Vector4: ...
    @property
    def bbrl(self) -> U64Vector4: ...
    @property
    def bbgr(self) -> U64Vector4: ...
    @property
    def bbgg(self) -> U64Vector4: ...
    @property
    def bbgb(self) -> U64Vector4: ...
    @property
    def bbga(self) -> U64Vector4: ...
    @property
    def bbgo(self) -> U64Vector4: ...
    @property
    def bbgl(self) -> U64Vector4: ...
    @property
    def bbbr(self) -> U64Vector4: ...
    @property
    def bbbg(self) -> U64Vector4: ...
    @property
    def bbbb(self) -> U64Vector4: ...
    @property
    def bbba(self) -> U64Vector4: ...
    @property
    def bbbo(self) -> U64Vector4: ...
    @property
    def bbbl(self) -> U64Vector4: ...
    @property
    def bbar(self) -> U64Vector4: ...
    @property
    def bbag(self) -> U64Vector4: ...
    @property
    def bbab(self) -> U64Vector4: ...
    @property
    def bbaa(self) -> U64Vector4: ...
    @property
    def bbao(self) -> U64Vector4: ...
    @property
    def bbal(self) -> U64Vector4: ...
    @property
    def bbor(self) -> U64Vector4: ...
    @property
    def bbog(self) -> U64Vector4: ...
    @property
    def bbob(self) -> U64Vector4: ...
    @property
    def bboa(self) -> U64Vector4: ...
    @property
    def bboo(self) -> U64Vector4: ...
    @property
    def bbol(self) -> U64Vector4: ...
    @property
    def bblr(self) -> U64Vector4: ...
    @property
    def bblg(self) -> U64Vector4: ...
    @property
    def bblb(self) -> U64Vector4: ...
    @property
    def bbla(self) -> U64Vector4: ...
    @property
    def bblo(self) -> U64Vector4: ...
    @property
    def bbll(self) -> U64Vector4: ...
    @property
    def barr(self) -> U64Vector4: ...
    @property
    def barg(self) -> U64Vector4: ...
    @property
    def barb(self) -> U64Vector4: ...
    @property
    def bara(self) -> U64Vector4: ...
    @property
    def baro(self) -> U64Vector4: ...
    @property
    def barl(self) -> U64Vector4: ...
    @property
    def bagr(self) -> U64Vector4: ...
    @property
    def bagg(self) -> U64Vector4: ...
    @property
    def bagb(self) -> U64Vector4: ...
    @property
    def baga(self) -> U64Vector4: ...
    @property
    def bago(self) -> U64Vector4: ...
    @property
    def bagl(self) -> U64Vector4: ...
    @property
    def babr(self) -> U64Vector4: ...
    @property
    def babg(self) -> U64Vector4: ...
    @property
    def babb(self) -> U64Vector4: ...
    @property
    def baba(self) -> U64Vector4: ...
    @property
    def babo(self) -> U64Vector4: ...
    @property
    def babl(self) -> U64Vector4: ...
    @property
    def baar(self) -> U64Vector4: ...
    @property
    def baag(self) -> U64Vector4: ...
    @property
    def baab(self) -> U64Vector4: ...
    @property
    def baaa(self) -> U64Vector4: ...
    @property
    def baao(self) -> U64Vector4: ...
    @property
    def baal(self) -> U64Vector4: ...
    @property
    def baor(self) -> U64Vector4: ...
    @property
    def baog(self) -> U64Vector4: ...
    @property
    def baob(self) -> U64Vector4: ...
    @property
    def baoa(self) -> U64Vector4: ...
    @property
    def baoo(self) -> U64Vector4: ...
    @property
    def baol(self) -> U64Vector4: ...
    @property
    def balr(self) -> U64Vector4: ...
    @property
    def balg(self) -> U64Vector4: ...
    @property
    def balb(self) -> U64Vector4: ...
    @property
    def bala(self) -> U64Vector4: ...
    @property
    def balo(self) -> U64Vector4: ...
    @property
    def ball(self) -> U64Vector4: ...
    @property
    def borr(self) -> U64Vector4: ...
    @property
    def borg(self) -> U64Vector4: ...
    @property
    def borb(self) -> U64Vector4: ...
    @property
    def bora(self) -> U64Vector4: ...
    @property
    def boro(self) -> U64Vector4: ...
    @property
    def borl(self) -> U64Vector4: ...
    @property
    def bogr(self) -> U64Vector4: ...
    @property
    def bogg(self) -> U64Vector4: ...
    @property
    def bogb(self) -> U64Vector4: ...
    @property
    def boga(self) -> U64Vector4: ...
    @property
    def bogo(self) -> U64Vector4: ...
    @property
    def bogl(self) -> U64Vector4: ...
    @property
    def bobr(self) -> U64Vector4: ...
    @property
    def bobg(self) -> U64Vector4: ...
    @property
    def bobb(self) -> U64Vector4: ...
    @property
    def boba(self) -> U64Vector4: ...
    @property
    def bobo(self) -> U64Vector4: ...
    @property
    def bobl(self) -> U64Vector4: ...
    @property
    def boar(self) -> U64Vector4: ...
    @property
    def boag(self) -> U64Vector4: ...
    @property
    def boab(self) -> U64Vector4: ...
    @property
    def boaa(self) -> U64Vector4: ...
    @property
    def boao(self) -> U64Vector4: ...
    @property
    def boal(self) -> U64Vector4: ...
    @property
    def boor(self) -> U64Vector4: ...
    @property
    def boog(self) -> U64Vector4: ...
    @property
    def boob(self) -> U64Vector4: ...
    @property
    def booa(self) -> U64Vector4: ...
    @property
    def booo(self) -> U64Vector4: ...
    @property
    def bool(self) -> U64Vector4: ...
    @property
    def bolr(self) -> U64Vector4: ...
    @property
    def bolg(self) -> U64Vector4: ...
    @property
    def bolb(self) -> U64Vector4: ...
    @property
    def bola(self) -> U64Vector4: ...
    @property
    def bolo(self) -> U64Vector4: ...
    @property
    def boll(self) -> U64Vector4: ...
    @property
    def blrr(self) -> U64Vector4: ...
    @property
    def blrg(self) -> U64Vector4: ...
    @property
    def blrb(self) -> U64Vector4: ...
    @property
    def blra(self) -> U64Vector4: ...
    @property
    def blro(self) -> U64Vector4: ...
    @property
    def blrl(self) -> U64Vector4: ...
    @property
    def blgr(self) -> U64Vector4: ...
    @property
    def blgg(self) -> U64Vector4: ...
    @property
    def blgb(self) -> U64Vector4: ...
    @property
    def blga(self) -> U64Vector4: ...
    @property
    def blgo(self) -> U64Vector4: ...
    @property
    def blgl(self) -> U64Vector4: ...
    @property
    def blbr(self) -> U64Vector4: ...
    @property
    def blbg(self) -> U64Vector4: ...
    @property
    def blbb(self) -> U64Vector4: ...
    @property
    def blba(self) -> U64Vector4: ...
    @property
    def blbo(self) -> U64Vector4: ...
    @property
    def blbl(self) -> U64Vector4: ...
    @property
    def blar(self) -> U64Vector4: ...
    @property
    def blag(self) -> U64Vector4: ...
    @property
    def blab(self) -> U64Vector4: ...
    @property
    def blaa(self) -> U64Vector4: ...
    @property
    def blao(self) -> U64Vector4: ...
    @property
    def blal(self) -> U64Vector4: ...
    @property
    def blor(self) -> U64Vector4: ...
    @property
    def blog(self) -> U64Vector4: ...
    @property
    def blob(self) -> U64Vector4: ...
    @property
    def bloa(self) -> U64Vector4: ...
    @property
    def bloo(self) -> U64Vector4: ...
    @property
    def blol(self) -> U64Vector4: ...
    @property
    def bllr(self) -> U64Vector4: ...
    @property
    def bllg(self) -> U64Vector4: ...
    @property
    def bllb(self) -> U64Vector4: ...
    @property
    def blla(self) -> U64Vector4: ...
    @property
    def bllo(self) -> U64Vector4: ...
    @property
    def blll(self) -> U64Vector4: ...
    @property
    def arrr(self) -> U64Vector4: ...
    @property
    def arrg(self) -> U64Vector4: ...
    @property
    def arrb(self) -> U64Vector4: ...
    @property
    def arra(self) -> U64Vector4: ...
    @property
    def arro(self) -> U64Vector4: ...
    @property
    def arrl(self) -> U64Vector4: ...
    @property
    def argr(self) -> U64Vector4: ...
    @property
    def argg(self) -> U64Vector4: ...
    @property
    def argb(self) -> U64Vector4: ...
    @property
    def arga(self) -> U64Vector4: ...
    @property
    def argo(self) -> U64Vector4: ...
    @property
    def argl(self) -> U64Vector4: ...
    @property
    def arbr(self) -> U64Vector4: ...
    @property
    def arbg(self) -> U64Vector4: ...
    @property
    def arbb(self) -> U64Vector4: ...
    @property
    def arba(self) -> U64Vector4: ...
    @property
    def arbo(self) -> U64Vector4: ...
    @property
    def arbl(self) -> U64Vector4: ...
    @property
    def arar(self) -> U64Vector4: ...
    @property
    def arag(self) -> U64Vector4: ...
    @property
    def arab(self) -> U64Vector4: ...
    @property
    def araa(self) -> U64Vector4: ...
    @property
    def arao(self) -> U64Vector4: ...
    @property
    def aral(self) -> U64Vector4: ...
    @property
    def aror(self) -> U64Vector4: ...
    @property
    def arog(self) -> U64Vector4: ...
    @property
    def arob(self) -> U64Vector4: ...
    @property
    def aroa(self) -> U64Vector4: ...
    @property
    def aroo(self) -> U64Vector4: ...
    @property
    def arol(self) -> U64Vector4: ...
    @property
    def arlr(self) -> U64Vector4: ...
    @property
    def arlg(self) -> U64Vector4: ...
    @property
    def arlb(self) -> U64Vector4: ...
    @property
    def arla(self) -> U64Vector4: ...
    @property
    def arlo(self) -> U64Vector4: ...
    @property
    def arll(self) -> U64Vector4: ...
    @property
    def agrr(self) -> U64Vector4: ...
    @property
    def agrg(self) -> U64Vector4: ...
    @property
    def agrb(self) -> U64Vector4: ...
    @property
    def agra(self) -> U64Vector4: ...
    @property
    def agro(self) -> U64Vector4: ...
    @property
    def agrl(self) -> U64Vector4: ...
    @property
    def aggr(self) -> U64Vector4: ...
    @property
    def aggg(self) -> U64Vector4: ...
    @property
    def aggb(self) -> U64Vector4: ...
    @property
    def agga(self) -> U64Vector4: ...
    @property
    def aggo(self) -> U64Vector4: ...
    @property
    def aggl(self) -> U64Vector4: ...
    @property
    def agbr(self) -> U64Vector4: ...
    @property
    def agbg(self) -> U64Vector4: ...
    @property
    def agbb(self) -> U64Vector4: ...
    @property
    def agba(self) -> U64Vector4: ...
    @property
    def agbo(self) -> U64Vector4: ...
    @property
    def agbl(self) -> U64Vector4: ...
    @property
    def agar(self) -> U64Vector4: ...
    @property
    def agag(self) -> U64Vector4: ...
    @property
    def agab(self) -> U64Vector4: ...
    @property
    def agaa(self) -> U64Vector4: ...
    @property
    def agao(self) -> U64Vector4: ...
    @property
    def agal(self) -> U64Vector4: ...
    @property
    def agor(self) -> U64Vector4: ...
    @property
    def agog(self) -> U64Vector4: ...
    @property
    def agob(self) -> U64Vector4: ...
    @property
    def agoa(self) -> U64Vector4: ...
    @property
    def agoo(self) -> U64Vector4: ...
    @property
    def agol(self) -> U64Vector4: ...
    @property
    def aglr(self) -> U64Vector4: ...
    @property
    def aglg(self) -> U64Vector4: ...
    @property
    def aglb(self) -> U64Vector4: ...
    @property
    def agla(self) -> U64Vector4: ...
    @property
    def aglo(self) -> U64Vector4: ...
    @property
    def agll(self) -> U64Vector4: ...
    @property
    def abrr(self) -> U64Vector4: ...
    @property
    def abrg(self) -> U64Vector4: ...
    @property
    def abrb(self) -> U64Vector4: ...
    @property
    def abra(self) -> U64Vector4: ...
    @property
    def abro(self) -> U64Vector4: ...
    @property
    def abrl(self) -> U64Vector4: ...
    @property
    def abgr(self) -> U64Vector4: ...
    @property
    def abgg(self) -> U64Vector4: ...
    @property
    def abgb(self) -> U64Vector4: ...
    @property
    def abga(self) -> U64Vector4: ...
    @property
    def abgo(self) -> U64Vector4: ...
    @property
    def abgl(self) -> U64Vector4: ...
    @property
    def abbr(self) -> U64Vector4: ...
    @property
    def abbg(self) -> U64Vector4: ...
    @property
    def abbb(self) -> U64Vector4: ...
    @property
    def abba(self) -> U64Vector4: ...
    @property
    def abbo(self) -> U64Vector4: ...
    @property
    def abbl(self) -> U64Vector4: ...
    @property
    def abar(self) -> U64Vector4: ...
    @property
    def abag(self) -> U64Vector4: ...
    @property
    def abab(self) -> U64Vector4: ...
    @property
    def abaa(self) -> U64Vector4: ...
    @property
    def abao(self) -> U64Vector4: ...
    @property
    def abal(self) -> U64Vector4: ...
    @property
    def abor(self) -> U64Vector4: ...
    @property
    def abog(self) -> U64Vector4: ...
    @property
    def abob(self) -> U64Vector4: ...
    @property
    def aboa(self) -> U64Vector4: ...
    @property
    def aboo(self) -> U64Vector4: ...
    @property
    def abol(self) -> U64Vector4: ...
    @property
    def ablr(self) -> U64Vector4: ...
    @property
    def ablg(self) -> U64Vector4: ...
    @property
    def ablb(self) -> U64Vector4: ...
    @property
    def abla(self) -> U64Vector4: ...
    @property
    def ablo(self) -> U64Vector4: ...
    @property
    def abll(self) -> U64Vector4: ...
    @property
    def aarr(self) -> U64Vector4: ...
    @property
    def aarg(self) -> U64Vector4: ...
    @property
    def aarb(self) -> U64Vector4: ...
    @property
    def aara(self) -> U64Vector4: ...
    @property
    def aaro(self) -> U64Vector4: ...
    @property
    def aarl(self) -> U64Vector4: ...
    @property
    def aagr(self) -> U64Vector4: ...
    @property
    def aagg(self) -> U64Vector4: ...
    @property
    def aagb(self) -> U64Vector4: ...
    @property
    def aaga(self) -> U64Vector4: ...
    @property
    def aago(self) -> U64Vector4: ...
    @property
    def aagl(self) -> U64Vector4: ...
    @property
    def aabr(self) -> U64Vector4: ...
    @property
    def aabg(self) -> U64Vector4: ...
    @property
    def aabb(self) -> U64Vector4: ...
    @property
    def aaba(self) -> U64Vector4: ...
    @property
    def aabo(self) -> U64Vector4: ...
    @property
    def aabl(self) -> U64Vector4: ...
    @property
    def aaar(self) -> U64Vector4: ...
    @property
    def aaag(self) -> U64Vector4: ...
    @property
    def aaab(self) -> U64Vector4: ...
    @property
    def aaaa(self) -> U64Vector4: ...
    @property
    def aaao(self) -> U64Vector4: ...
    @property
    def aaal(self) -> U64Vector4: ...
    @property
    def aaor(self) -> U64Vector4: ...
    @property
    def aaog(self) -> U64Vector4: ...
    @property
    def aaob(self) -> U64Vector4: ...
    @property
    def aaoa(self) -> U64Vector4: ...
    @property
    def aaoo(self) -> U64Vector4: ...
    @property
    def aaol(self) -> U64Vector4: ...
    @property
    def aalr(self) -> U64Vector4: ...
    @property
    def aalg(self) -> U64Vector4: ...
    @property
    def aalb(self) -> U64Vector4: ...
    @property
    def aala(self) -> U64Vector4: ...
    @property
    def aalo(self) -> U64Vector4: ...
    @property
    def aall(self) -> U64Vector4: ...
    @property
    def aorr(self) -> U64Vector4: ...
    @property
    def aorg(self) -> U64Vector4: ...
    @property
    def aorb(self) -> U64Vector4: ...
    @property
    def aora(self) -> U64Vector4: ...
    @property
    def aoro(self) -> U64Vector4: ...
    @property
    def aorl(self) -> U64Vector4: ...
    @property
    def aogr(self) -> U64Vector4: ...
    @property
    def aogg(self) -> U64Vector4: ...
    @property
    def aogb(self) -> U64Vector4: ...
    @property
    def aoga(self) -> U64Vector4: ...
    @property
    def aogo(self) -> U64Vector4: ...
    @property
    def aogl(self) -> U64Vector4: ...
    @property
    def aobr(self) -> U64Vector4: ...
    @property
    def aobg(self) -> U64Vector4: ...
    @property
    def aobb(self) -> U64Vector4: ...
    @property
    def aoba(self) -> U64Vector4: ...
    @property
    def aobo(self) -> U64Vector4: ...
    @property
    def aobl(self) -> U64Vector4: ...
    @property
    def aoar(self) -> U64Vector4: ...
    @property
    def aoag(self) -> U64Vector4: ...
    @property
    def aoab(self) -> U64Vector4: ...
    @property
    def aoaa(self) -> U64Vector4: ...
    @property
    def aoao(self) -> U64Vector4: ...
    @property
    def aoal(self) -> U64Vector4: ...
    @property
    def aoor(self) -> U64Vector4: ...
    @property
    def aoog(self) -> U64Vector4: ...
    @property
    def aoob(self) -> U64Vector4: ...
    @property
    def aooa(self) -> U64Vector4: ...
    @property
    def aooo(self) -> U64Vector4: ...
    @property
    def aool(self) -> U64Vector4: ...
    @property
    def aolr(self) -> U64Vector4: ...
    @property
    def aolg(self) -> U64Vector4: ...
    @property
    def aolb(self) -> U64Vector4: ...
    @property
    def aola(self) -> U64Vector4: ...
    @property
    def aolo(self) -> U64Vector4: ...
    @property
    def aoll(self) -> U64Vector4: ...
    @property
    def alrr(self) -> U64Vector4: ...
    @property
    def alrg(self) -> U64Vector4: ...
    @property
    def alrb(self) -> U64Vector4: ...
    @property
    def alra(self) -> U64Vector4: ...
    @property
    def alro(self) -> U64Vector4: ...
    @property
    def alrl(self) -> U64Vector4: ...
    @property
    def algr(self) -> U64Vector4: ...
    @property
    def algg(self) -> U64Vector4: ...
    @property
    def algb(self) -> U64Vector4: ...
    @property
    def alga(self) -> U64Vector4: ...
    @property
    def algo(self) -> U64Vector4: ...
    @property
    def algl(self) -> U64Vector4: ...
    @property
    def albr(self) -> U64Vector4: ...
    @property
    def albg(self) -> U64Vector4: ...
    @property
    def albb(self) -> U64Vector4: ...
    @property
    def alba(self) -> U64Vector4: ...
    @property
    def albo(self) -> U64Vector4: ...
    @property
    def albl(self) -> U64Vector4: ...
    @property
    def alar(self) -> U64Vector4: ...
    @property
    def alag(self) -> U64Vector4: ...
    @property
    def alab(self) -> U64Vector4: ...
    @property
    def alaa(self) -> U64Vector4: ...
    @property
    def alao(self) -> U64Vector4: ...
    @property
    def alal(self) -> U64Vector4: ...
    @property
    def alor(self) -> U64Vector4: ...
    @property
    def alog(self) -> U64Vector4: ...
    @property
    def alob(self) -> U64Vector4: ...
    @property
    def aloa(self) -> U64Vector4: ...
    @property
    def aloo(self) -> U64Vector4: ...
    @property
    def alol(self) -> U64Vector4: ...
    @property
    def allr(self) -> U64Vector4: ...
    @property
    def allg(self) -> U64Vector4: ...
    @property
    def allb(self) -> U64Vector4: ...
    @property
    def alla(self) -> U64Vector4: ...
    @property
    def allo(self) -> U64Vector4: ...
    @property
    def alll(self) -> U64Vector4: ...
    @property
    def orrr(self) -> U64Vector4: ...
    @property
    def orrg(self) -> U64Vector4: ...
    @property
    def orrb(self) -> U64Vector4: ...
    @property
    def orra(self) -> U64Vector4: ...
    @property
    def orro(self) -> U64Vector4: ...
    @property
    def orrl(self) -> U64Vector4: ...
    @property
    def orgr(self) -> U64Vector4: ...
    @property
    def orgg(self) -> U64Vector4: ...
    @property
    def orgb(self) -> U64Vector4: ...
    @property
    def orga(self) -> U64Vector4: ...
    @property
    def orgo(self) -> U64Vector4: ...
    @property
    def orgl(self) -> U64Vector4: ...
    @property
    def orbr(self) -> U64Vector4: ...
    @property
    def orbg(self) -> U64Vector4: ...
    @property
    def orbb(self) -> U64Vector4: ...
    @property
    def orba(self) -> U64Vector4: ...
    @property
    def orbo(self) -> U64Vector4: ...
    @property
    def orbl(self) -> U64Vector4: ...
    @property
    def orar(self) -> U64Vector4: ...
    @property
    def orag(self) -> U64Vector4: ...
    @property
    def orab(self) -> U64Vector4: ...
    @property
    def oraa(self) -> U64Vector4: ...
    @property
    def orao(self) -> U64Vector4: ...
    @property
    def oral(self) -> U64Vector4: ...
    @property
    def oror(self) -> U64Vector4: ...
    @property
    def orog(self) -> U64Vector4: ...
    @property
    def orob(self) -> U64Vector4: ...
    @property
    def oroa(self) -> U64Vector4: ...
    @property
    def oroo(self) -> U64Vector4: ...
    @property
    def orol(self) -> U64Vector4: ...
    @property
    def orlr(self) -> U64Vector4: ...
    @property
    def orlg(self) -> U64Vector4: ...
    @property
    def orlb(self) -> U64Vector4: ...
    @property
    def orla(self) -> U64Vector4: ...
    @property
    def orlo(self) -> U64Vector4: ...
    @property
    def orll(self) -> U64Vector4: ...
    @property
    def ogrr(self) -> U64Vector4: ...
    @property
    def ogrg(self) -> U64Vector4: ...
    @property
    def ogrb(self) -> U64Vector4: ...
    @property
    def ogra(self) -> U64Vector4: ...
    @property
    def ogro(self) -> U64Vector4: ...
    @property
    def ogrl(self) -> U64Vector4: ...
    @property
    def oggr(self) -> U64Vector4: ...
    @property
    def oggg(self) -> U64Vector4: ...
    @property
    def oggb(self) -> U64Vector4: ...
    @property
    def ogga(self) -> U64Vector4: ...
    @property
    def oggo(self) -> U64Vector4: ...
    @property
    def oggl(self) -> U64Vector4: ...
    @property
    def ogbr(self) -> U64Vector4: ...
    @property
    def ogbg(self) -> U64Vector4: ...
    @property
    def ogbb(self) -> U64Vector4: ...
    @property
    def ogba(self) -> U64Vector4: ...
    @property
    def ogbo(self) -> U64Vector4: ...
    @property
    def ogbl(self) -> U64Vector4: ...
    @property
    def ogar(self) -> U64Vector4: ...
    @property
    def ogag(self) -> U64Vector4: ...
    @property
    def ogab(self) -> U64Vector4: ...
    @property
    def ogaa(self) -> U64Vector4: ...
    @property
    def ogao(self) -> U64Vector4: ...
    @property
    def ogal(self) -> U64Vector4: ...
    @property
    def ogor(self) -> U64Vector4: ...
    @property
    def ogog(self) -> U64Vector4: ...
    @property
    def ogob(self) -> U64Vector4: ...
    @property
    def ogoa(self) -> U64Vector4: ...
    @property
    def ogoo(self) -> U64Vector4: ...
    @property
    def ogol(self) -> U64Vector4: ...
    @property
    def oglr(self) -> U64Vector4: ...
    @property
    def oglg(self) -> U64Vector4: ...
    @property
    def oglb(self) -> U64Vector4: ...
    @property
    def ogla(self) -> U64Vector4: ...
    @property
    def oglo(self) -> U64Vector4: ...
    @property
    def ogll(self) -> U64Vector4: ...
    @property
    def obrr(self) -> U64Vector4: ...
    @property
    def obrg(self) -> U64Vector4: ...
    @property
    def obrb(self) -> U64Vector4: ...
    @property
    def obra(self) -> U64Vector4: ...
    @property
    def obro(self) -> U64Vector4: ...
    @property
    def obrl(self) -> U64Vector4: ...
    @property
    def obgr(self) -> U64Vector4: ...
    @property
    def obgg(self) -> U64Vector4: ...
    @property
    def obgb(self) -> U64Vector4: ...
    @property
    def obga(self) -> U64Vector4: ...
    @property
    def obgo(self) -> U64Vector4: ...
    @property
    def obgl(self) -> U64Vector4: ...
    @property
    def obbr(self) -> U64Vector4: ...
    @property
    def obbg(self) -> U64Vector4: ...
    @property
    def obbb(self) -> U64Vector4: ...
    @property
    def obba(self) -> U64Vector4: ...
    @property
    def obbo(self) -> U64Vector4: ...
    @property
    def obbl(self) -> U64Vector4: ...
    @property
    def obar(self) -> U64Vector4: ...
    @property
    def obag(self) -> U64Vector4: ...
    @property
    def obab(self) -> U64Vector4: ...
    @property
    def obaa(self) -> U64Vector4: ...
    @property
    def obao(self) -> U64Vector4: ...
    @property
    def obal(self) -> U64Vector4: ...
    @property
    def obor(self) -> U64Vector4: ...
    @property
    def obog(self) -> U64Vector4: ...
    @property
    def obob(self) -> U64Vector4: ...
    @property
    def oboa(self) -> U64Vector4: ...
    @property
    def oboo(self) -> U64Vector4: ...
    @property
    def obol(self) -> U64Vector4: ...
    @property
    def oblr(self) -> U64Vector4: ...
    @property
    def oblg(self) -> U64Vector4: ...
    @property
    def oblb(self) -> U64Vector4: ...
    @property
    def obla(self) -> U64Vector4: ...
    @property
    def oblo(self) -> U64Vector4: ...
    @property
    def obll(self) -> U64Vector4: ...
    @property
    def oarr(self) -> U64Vector4: ...
    @property
    def oarg(self) -> U64Vector4: ...
    @property
    def oarb(self) -> U64Vector4: ...
    @property
    def oara(self) -> U64Vector4: ...
    @property
    def oaro(self) -> U64Vector4: ...
    @property
    def oarl(self) -> U64Vector4: ...
    @property
    def oagr(self) -> U64Vector4: ...
    @property
    def oagg(self) -> U64Vector4: ...
    @property
    def oagb(self) -> U64Vector4: ...
    @property
    def oaga(self) -> U64Vector4: ...
    @property
    def oago(self) -> U64Vector4: ...
    @property
    def oagl(self) -> U64Vector4: ...
    @property
    def oabr(self) -> U64Vector4: ...
    @property
    def oabg(self) -> U64Vector4: ...
    @property
    def oabb(self) -> U64Vector4: ...
    @property
    def oaba(self) -> U64Vector4: ...
    @property
    def oabo(self) -> U64Vector4: ...
    @property
    def oabl(self) -> U64Vector4: ...
    @property
    def oaar(self) -> U64Vector4: ...
    @property
    def oaag(self) -> U64Vector4: ...
    @property
    def oaab(self) -> U64Vector4: ...
    @property
    def oaaa(self) -> U64Vector4: ...
    @property
    def oaao(self) -> U64Vector4: ...
    @property
    def oaal(self) -> U64Vector4: ...
    @property
    def oaor(self) -> U64Vector4: ...
    @property
    def oaog(self) -> U64Vector4: ...
    @property
    def oaob(self) -> U64Vector4: ...
    @property
    def oaoa(self) -> U64Vector4: ...
    @property
    def oaoo(self) -> U64Vector4: ...
    @property
    def oaol(self) -> U64Vector4: ...
    @property
    def oalr(self) -> U64Vector4: ...
    @property
    def oalg(self) -> U64Vector4: ...
    @property
    def oalb(self) -> U64Vector4: ...
    @property
    def oala(self) -> U64Vector4: ...
    @property
    def oalo(self) -> U64Vector4: ...
    @property
    def oall(self) -> U64Vector4: ...
    @property
    def oorr(self) -> U64Vector4: ...
    @property
    def oorg(self) -> U64Vector4: ...
    @property
    def oorb(self) -> U64Vector4: ...
    @property
    def oora(self) -> U64Vector4: ...
    @property
    def ooro(self) -> U64Vector4: ...
    @property
    def oorl(self) -> U64Vector4: ...
    @property
    def oogr(self) -> U64Vector4: ...
    @property
    def oogg(self) -> U64Vector4: ...
    @property
    def oogb(self) -> U64Vector4: ...
    @property
    def ooga(self) -> U64Vector4: ...
    @property
    def oogo(self) -> U64Vector4: ...
    @property
    def oogl(self) -> U64Vector4: ...
    @property
    def oobr(self) -> U64Vector4: ...
    @property
    def oobg(self) -> U64Vector4: ...
    @property
    def oobb(self) -> U64Vector4: ...
    @property
    def ooba(self) -> U64Vector4: ...
    @property
    def oobo(self) -> U64Vector4: ...
    @property
    def oobl(self) -> U64Vector4: ...
    @property
    def ooar(self) -> U64Vector4: ...
    @property
    def ooag(self) -> U64Vector4: ...
    @property
    def ooab(self) -> U64Vector4: ...
    @property
    def ooaa(self) -> U64Vector4: ...
    @property
    def ooao(self) -> U64Vector4: ...
    @property
    def ooal(self) -> U64Vector4: ...
    @property
    def ooor(self) -> U64Vector4: ...
    @property
    def ooog(self) -> U64Vector4: ...
    @property
    def ooob(self) -> U64Vector4: ...
    @property
    def oooa(self) -> U64Vector4: ...
    @property
    def oolr(self) -> U64Vector4: ...
    @property
    def oolg(self) -> U64Vector4: ...
    @property
    def oolb(self) -> U64Vector4: ...
    @property
    def oola(self) -> U64Vector4: ...
    @property
    def olrr(self) -> U64Vector4: ...
    @property
    def olrg(self) -> U64Vector4: ...
    @property
    def olrb(self) -> U64Vector4: ...
    @property
    def olra(self) -> U64Vector4: ...
    @property
    def olro(self) -> U64Vector4: ...
    @property
    def olrl(self) -> U64Vector4: ...
    @property
    def olgr(self) -> U64Vector4: ...
    @property
    def olgg(self) -> U64Vector4: ...
    @property
    def olgb(self) -> U64Vector4: ...
    @property
    def olga(self) -> U64Vector4: ...
    @property
    def olgo(self) -> U64Vector4: ...
    @property
    def olgl(self) -> U64Vector4: ...
    @property
    def olbr(self) -> U64Vector4: ...
    @property
    def olbg(self) -> U64Vector4: ...
    @property
    def olbb(self) -> U64Vector4: ...
    @property
    def olba(self) -> U64Vector4: ...
    @property
    def olbo(self) -> U64Vector4: ...
    @property
    def olbl(self) -> U64Vector4: ...
    @property
    def olar(self) -> U64Vector4: ...
    @property
    def olag(self) -> U64Vector4: ...
    @property
    def olab(self) -> U64Vector4: ...
    @property
    def olaa(self) -> U64Vector4: ...
    @property
    def olao(self) -> U64Vector4: ...
    @property
    def olal(self) -> U64Vector4: ...
    @property
    def olor(self) -> U64Vector4: ...
    @property
    def olog(self) -> U64Vector4: ...
    @property
    def olob(self) -> U64Vector4: ...
    @property
    def oloa(self) -> U64Vector4: ...
    @property
    def ollr(self) -> U64Vector4: ...
    @property
    def ollg(self) -> U64Vector4: ...
    @property
    def ollb(self) -> U64Vector4: ...
    @property
    def olla(self) -> U64Vector4: ...
    @property
    def lrrr(self) -> U64Vector4: ...
    @property
    def lrrg(self) -> U64Vector4: ...
    @property
    def lrrb(self) -> U64Vector4: ...
    @property
    def lrra(self) -> U64Vector4: ...
    @property
    def lrro(self) -> U64Vector4: ...
    @property
    def lrrl(self) -> U64Vector4: ...
    @property
    def lrgr(self) -> U64Vector4: ...
    @property
    def lrgg(self) -> U64Vector4: ...
    @property
    def lrgb(self) -> U64Vector4: ...
    @property
    def lrga(self) -> U64Vector4: ...
    @property
    def lrgo(self) -> U64Vector4: ...
    @property
    def lrgl(self) -> U64Vector4: ...
    @property
    def lrbr(self) -> U64Vector4: ...
    @property
    def lrbg(self) -> U64Vector4: ...
    @property
    def lrbb(self) -> U64Vector4: ...
    @property
    def lrba(self) -> U64Vector4: ...
    @property
    def lrbo(self) -> U64Vector4: ...
    @property
    def lrbl(self) -> U64Vector4: ...
    @property
    def lrar(self) -> U64Vector4: ...
    @property
    def lrag(self) -> U64Vector4: ...
    @property
    def lrab(self) -> U64Vector4: ...
    @property
    def lraa(self) -> U64Vector4: ...
    @property
    def lrao(self) -> U64Vector4: ...
    @property
    def lral(self) -> U64Vector4: ...
    @property
    def lror(self) -> U64Vector4: ...
    @property
    def lrog(self) -> U64Vector4: ...
    @property
    def lrob(self) -> U64Vector4: ...
    @property
    def lroa(self) -> U64Vector4: ...
    @property
    def lroo(self) -> U64Vector4: ...
    @property
    def lrol(self) -> U64Vector4: ...
    @property
    def lrlr(self) -> U64Vector4: ...
    @property
    def lrlg(self) -> U64Vector4: ...
    @property
    def lrlb(self) -> U64Vector4: ...
    @property
    def lrla(self) -> U64Vector4: ...
    @property
    def lrlo(self) -> U64Vector4: ...
    @property
    def lrll(self) -> U64Vector4: ...
    @property
    def lgrr(self) -> U64Vector4: ...
    @property
    def lgrg(self) -> U64Vector4: ...
    @property
    def lgrb(self) -> U64Vector4: ...
    @property
    def lgra(self) -> U64Vector4: ...
    @property
    def lgro(self) -> U64Vector4: ...
    @property
    def lgrl(self) -> U64Vector4: ...
    @property
    def lggr(self) -> U64Vector4: ...
    @property
    def lggg(self) -> U64Vector4: ...
    @property
    def lggb(self) -> U64Vector4: ...
    @property
    def lgga(self) -> U64Vector4: ...
    @property
    def lggo(self) -> U64Vector4: ...
    @property
    def lggl(self) -> U64Vector4: ...
    @property
    def lgbr(self) -> U64Vector4: ...
    @property
    def lgbg(self) -> U64Vector4: ...
    @property
    def lgbb(self) -> U64Vector4: ...
    @property
    def lgba(self) -> U64Vector4: ...
    @property
    def lgbo(self) -> U64Vector4: ...
    @property
    def lgbl(self) -> U64Vector4: ...
    @property
    def lgar(self) -> U64Vector4: ...
    @property
    def lgag(self) -> U64Vector4: ...
    @property
    def lgab(self) -> U64Vector4: ...
    @property
    def lgaa(self) -> U64Vector4: ...
    @property
    def lgao(self) -> U64Vector4: ...
    @property
    def lgal(self) -> U64Vector4: ...
    @property
    def lgor(self) -> U64Vector4: ...
    @property
    def lgog(self) -> U64Vector4: ...
    @property
    def lgob(self) -> U64Vector4: ...
    @property
    def lgoa(self) -> U64Vector4: ...
    @property
    def lgoo(self) -> U64Vector4: ...
    @property
    def lgol(self) -> U64Vector4: ...
    @property
    def lglr(self) -> U64Vector4: ...
    @property
    def lglg(self) -> U64Vector4: ...
    @property
    def lglb(self) -> U64Vector4: ...
    @property
    def lgla(self) -> U64Vector4: ...
    @property
    def lglo(self) -> U64Vector4: ...
    @property
    def lgll(self) -> U64Vector4: ...
    @property
    def lbrr(self) -> U64Vector4: ...
    @property
    def lbrg(self) -> U64Vector4: ...
    @property
    def lbrb(self) -> U64Vector4: ...
    @property
    def lbra(self) -> U64Vector4: ...
    @property
    def lbro(self) -> U64Vector4: ...
    @property
    def lbrl(self) -> U64Vector4: ...
    @property
    def lbgr(self) -> U64Vector4: ...
    @property
    def lbgg(self) -> U64Vector4: ...
    @property
    def lbgb(self) -> U64Vector4: ...
    @property
    def lbga(self) -> U64Vector4: ...
    @property
    def lbgo(self) -> U64Vector4: ...
    @property
    def lbgl(self) -> U64Vector4: ...
    @property
    def lbbr(self) -> U64Vector4: ...
    @property
    def lbbg(self) -> U64Vector4: ...
    @property
    def lbbb(self) -> U64Vector4: ...
    @property
    def lbba(self) -> U64Vector4: ...
    @property
    def lbbo(self) -> U64Vector4: ...
    @property
    def lbbl(self) -> U64Vector4: ...
    @property
    def lbar(self) -> U64Vector4: ...
    @property
    def lbag(self) -> U64Vector4: ...
    @property
    def lbab(self) -> U64Vector4: ...
    @property
    def lbaa(self) -> U64Vector4: ...
    @property
    def lbao(self) -> U64Vector4: ...
    @property
    def lbal(self) -> U64Vector4: ...
    @property
    def lbor(self) -> U64Vector4: ...
    @property
    def lbog(self) -> U64Vector4: ...
    @property
    def lbob(self) -> U64Vector4: ...
    @property
    def lboa(self) -> U64Vector4: ...
    @property
    def lboo(self) -> U64Vector4: ...
    @property
    def lbol(self) -> U64Vector4: ...
    @property
    def lblr(self) -> U64Vector4: ...
    @property
    def lblg(self) -> U64Vector4: ...
    @property
    def lblb(self) -> U64Vector4: ...
    @property
    def lbla(self) -> U64Vector4: ...
    @property
    def lblo(self) -> U64Vector4: ...
    @property
    def lbll(self) -> U64Vector4: ...
    @property
    def larr(self) -> U64Vector4: ...
    @property
    def larg(self) -> U64Vector4: ...
    @property
    def larb(self) -> U64Vector4: ...
    @property
    def lara(self) -> U64Vector4: ...
    @property
    def laro(self) -> U64Vector4: ...
    @property
    def larl(self) -> U64Vector4: ...
    @property
    def lagr(self) -> U64Vector4: ...
    @property
    def lagg(self) -> U64Vector4: ...
    @property
    def lagb(self) -> U64Vector4: ...
    @property
    def laga(self) -> U64Vector4: ...
    @property
    def lago(self) -> U64Vector4: ...
    @property
    def lagl(self) -> U64Vector4: ...
    @property
    def labr(self) -> U64Vector4: ...
    @property
    def labg(self) -> U64Vector4: ...
    @property
    def labb(self) -> U64Vector4: ...
    @property
    def laba(self) -> U64Vector4: ...
    @property
    def labo(self) -> U64Vector4: ...
    @property
    def labl(self) -> U64Vector4: ...
    @property
    def laar(self) -> U64Vector4: ...
    @property
    def laag(self) -> U64Vector4: ...
    @property
    def laab(self) -> U64Vector4: ...
    @property
    def laaa(self) -> U64Vector4: ...
    @property
    def laao(self) -> U64Vector4: ...
    @property
    def laal(self) -> U64Vector4: ...
    @property
    def laor(self) -> U64Vector4: ...
    @property
    def laog(self) -> U64Vector4: ...
    @property
    def laob(self) -> U64Vector4: ...
    @property
    def laoa(self) -> U64Vector4: ...
    @property
    def laoo(self) -> U64Vector4: ...
    @property
    def laol(self) -> U64Vector4: ...
    @property
    def lalr(self) -> U64Vector4: ...
    @property
    def lalg(self) -> U64Vector4: ...
    @property
    def lalb(self) -> U64Vector4: ...
    @property
    def lala(self) -> U64Vector4: ...
    @property
    def lalo(self) -> U64Vector4: ...
    @property
    def lall(self) -> U64Vector4: ...
    @property
    def lorr(self) -> U64Vector4: ...
    @property
    def lorg(self) -> U64Vector4: ...
    @property
    def lorb(self) -> U64Vector4: ...
    @property
    def lora(self) -> U64Vector4: ...
    @property
    def loro(self) -> U64Vector4: ...
    @property
    def lorl(self) -> U64Vector4: ...
    @property
    def logr(self) -> U64Vector4: ...
    @property
    def logg(self) -> U64Vector4: ...
    @property
    def logb(self) -> U64Vector4: ...
    @property
    def loga(self) -> U64Vector4: ...
    @property
    def logo(self) -> U64Vector4: ...
    @property
    def logl(self) -> U64Vector4: ...
    @property
    def lobr(self) -> U64Vector4: ...
    @property
    def lobg(self) -> U64Vector4: ...
    @property
    def lobb(self) -> U64Vector4: ...
    @property
    def loba(self) -> U64Vector4: ...
    @property
    def lobo(self) -> U64Vector4: ...
    @property
    def lobl(self) -> U64Vector4: ...
    @property
    def loar(self) -> U64Vector4: ...
    @property
    def loag(self) -> U64Vector4: ...
    @property
    def loab(self) -> U64Vector4: ...
    @property
    def loaa(self) -> U64Vector4: ...
    @property
    def loao(self) -> U64Vector4: ...
    @property
    def loal(self) -> U64Vector4: ...
    @property
    def loor(self) -> U64Vector4: ...
    @property
    def loog(self) -> U64Vector4: ...
    @property
    def loob(self) -> U64Vector4: ...
    @property
    def looa(self) -> U64Vector4: ...
    @property
    def lolr(self) -> U64Vector4: ...
    @property
    def lolg(self) -> U64Vector4: ...
    @property
    def lolb(self) -> U64Vector4: ...
    @property
    def lola(self) -> U64Vector4: ...
    @property
    def llrr(self) -> U64Vector4: ...
    @property
    def llrg(self) -> U64Vector4: ...
    @property
    def llrb(self) -> U64Vector4: ...
    @property
    def llra(self) -> U64Vector4: ...
    @property
    def llro(self) -> U64Vector4: ...
    @property
    def llrl(self) -> U64Vector4: ...
    @property
    def llgr(self) -> U64Vector4: ...
    @property
    def llgg(self) -> U64Vector4: ...
    @property
    def llgb(self) -> U64Vector4: ...
    @property
    def llga(self) -> U64Vector4: ...
    @property
    def llgo(self) -> U64Vector4: ...
    @property
    def llgl(self) -> U64Vector4: ...
    @property
    def llbr(self) -> U64Vector4: ...
    @property
    def llbg(self) -> U64Vector4: ...
    @property
    def llbb(self) -> U64Vector4: ...
    @property
    def llba(self) -> U64Vector4: ...
    @property
    def llbo(self) -> U64Vector4: ...
    @property
    def llbl(self) -> U64Vector4: ...
    @property
    def llar(self) -> U64Vector4: ...
    @property
    def llag(self) -> U64Vector4: ...
    @property
    def llab(self) -> U64Vector4: ...
    @property
    def llaa(self) -> U64Vector4: ...
    @property
    def llao(self) -> U64Vector4: ...
    @property
    def llal(self) -> U64Vector4: ...
    @property
    def llor(self) -> U64Vector4: ...
    @property
    def llog(self) -> U64Vector4: ...
    @property
    def llob(self) -> U64Vector4: ...
    @property
    def lloa(self) -> U64Vector4: ...
    @property
    def lllr(self) -> U64Vector4: ...
    @property
    def lllg(self) -> U64Vector4: ...
    @property
    def lllb(self) -> U64Vector4: ...
    @property
    def llla(self) -> U64Vector4: ...
    @property
    def s(self) -> int: ...
    @property
    def t(self) -> int: ...
    @property
    def q(self) -> int: ...
    @property
    def p(self) -> int: ...
    @property
    def ss(self) -> U64Vector2: ...
    @property
    def st(self) -> U64Vector2: ...
    @property
    def sq(self) -> U64Vector2: ...
    @property
    def sp(self) -> U64Vector2: ...
    @property
    def so(self) -> U64Vector2: ...
    @property
    def sl(self) -> U64Vector2: ...
    @property
    def ts(self) -> U64Vector2: ...
    @property
    def tt(self) -> U64Vector2: ...
    @property
    def tq(self) -> U64Vector2: ...
    @property
    def tp(self) -> U64Vector2: ...
    @property
    def to(self) -> U64Vector2: ...
    @property
    def tl(self) -> U64Vector2: ...
    @property
    def qs(self) -> U64Vector2: ...
    @property
    def qt(self) -> U64Vector2: ...
    @property
    def qq(self) -> U64Vector2: ...
    @property
    def qp(self) -> U64Vector2: ...
    @property
    def qo(self) -> U64Vector2: ...
    @property
    def ql(self) -> U64Vector2: ...
    @property
    def ps(self) -> U64Vector2: ...
    @property
    def pt(self) -> U64Vector2: ...
    @property
    def pq(self) -> U64Vector2: ...
    @property
    def pp(self) -> U64Vector2: ...
    @property
    def po(self) -> U64Vector2: ...
    @property
    def pl(self) -> U64Vector2: ...
    @property
    def os(self) -> U64Vector2: ...
    @property
    def ot(self) -> U64Vector2: ...
    @property
    def oq(self) -> U64Vector2: ...
    @property
    def op(self) -> U64Vector2: ...
    @property
    def ls(self) -> U64Vector2: ...
    @property
    def lt(self) -> U64Vector2: ...
    @property
    def lq(self) -> U64Vector2: ...
    @property
    def lp(self) -> U64Vector2: ...
    @property
    def sss(self) -> U64Vector3: ...
    @property
    def sst(self) -> U64Vector3: ...
    @property
    def ssq(self) -> U64Vector3: ...
    @property
    def ssp(self) -> U64Vector3: ...
    @property
    def sso(self) -> U64Vector3: ...
    @property
    def ssl(self) -> U64Vector3: ...
    @property
    def sts(self) -> U64Vector3: ...
    @property
    def stt(self) -> U64Vector3: ...
    @property
    def stq(self) -> U64Vector3: ...
    @property
    def stp(self) -> U64Vector3: ...
    @property
    def sto(self) -> U64Vector3: ...
    @property
    def stl(self) -> U64Vector3: ...
    @property
    def sqs(self) -> U64Vector3: ...
    @property
    def sqt(self) -> U64Vector3: ...
    @property
    def sqq(self) -> U64Vector3: ...
    @property
    def sqp(self) -> U64Vector3: ...
    @property
    def sqo(self) -> U64Vector3: ...
    @property
    def sql(self) -> U64Vector3: ...
    @property
    def sps(self) -> U64Vector3: ...
    @property
    def spt(self) -> U64Vector3: ...
    @property
    def spq(self) -> U64Vector3: ...
    @property
    def spp(self) -> U64Vector3: ...
    @property
    def spo(self) -> U64Vector3: ...
    @property
    def spl(self) -> U64Vector3: ...
    @property
    def sos(self) -> U64Vector3: ...
    @property
    def sot(self) -> U64Vector3: ...
    @property
    def soq(self) -> U64Vector3: ...
    @property
    def sop(self) -> U64Vector3: ...
    @property
    def soo(self) -> U64Vector3: ...
    @property
    def sol(self) -> U64Vector3: ...
    @property
    def sls(self) -> U64Vector3: ...
    @property
    def slt(self) -> U64Vector3: ...
    @property
    def slq(self) -> U64Vector3: ...
    @property
    def slp(self) -> U64Vector3: ...
    @property
    def slo(self) -> U64Vector3: ...
    @property
    def sll(self) -> U64Vector3: ...
    @property
    def tss(self) -> U64Vector3: ...
    @property
    def tst(self) -> U64Vector3: ...
    @property
    def tsq(self) -> U64Vector3: ...
    @property
    def tsp(self) -> U64Vector3: ...
    @property
    def tso(self) -> U64Vector3: ...
    @property
    def tsl(self) -> U64Vector3: ...
    @property
    def tts(self) -> U64Vector3: ...
    @property
    def ttt(self) -> U64Vector3: ...
    @property
    def ttq(self) -> U64Vector3: ...
    @property
    def ttp(self) -> U64Vector3: ...
    @property
    def tto(self) -> U64Vector3: ...
    @property
    def ttl(self) -> U64Vector3: ...
    @property
    def tqs(self) -> U64Vector3: ...
    @property
    def tqt(self) -> U64Vector3: ...
    @property
    def tqq(self) -> U64Vector3: ...
    @property
    def tqp(self) -> U64Vector3: ...
    @property
    def tqo(self) -> U64Vector3: ...
    @property
    def tql(self) -> U64Vector3: ...
    @property
    def tps(self) -> U64Vector3: ...
    @property
    def tpt(self) -> U64Vector3: ...
    @property
    def tpq(self) -> U64Vector3: ...
    @property
    def tpp(self) -> U64Vector3: ...
    @property
    def tpo(self) -> U64Vector3: ...
    @property
    def tpl(self) -> U64Vector3: ...
    @property
    def tos(self) -> U64Vector3: ...
    @property
    def tot(self) -> U64Vector3: ...
    @property
    def toq(self) -> U64Vector3: ...
    @property
    def top(self) -> U64Vector3: ...
    @property
    def too(self) -> U64Vector3: ...
    @property
    def tol(self) -> U64Vector3: ...
    @property
    def tls(self) -> U64Vector3: ...
    @property
    def tlt(self) -> U64Vector3: ...
    @property
    def tlq(self) -> U64Vector3: ...
    @property
    def tlp(self) -> U64Vector3: ...
    @property
    def tlo(self) -> U64Vector3: ...
    @property
    def tll(self) -> U64Vector3: ...
    @property
    def qss(self) -> U64Vector3: ...
    @property
    def qst(self) -> U64Vector3: ...
    @property
    def qsq(self) -> U64Vector3: ...
    @property
    def qsp(self) -> U64Vector3: ...
    @property
    def qso(self) -> U64Vector3: ...
    @property
    def qsl(self) -> U64Vector3: ...
    @property
    def qts(self) -> U64Vector3: ...
    @property
    def qtt(self) -> U64Vector3: ...
    @property
    def qtq(self) -> U64Vector3: ...
    @property
    def qtp(self) -> U64Vector3: ...
    @property
    def qto(self) -> U64Vector3: ...
    @property
    def qtl(self) -> U64Vector3: ...
    @property
    def qqs(self) -> U64Vector3: ...
    @property
    def qqt(self) -> U64Vector3: ...
    @property
    def qqq(self) -> U64Vector3: ...
    @property
    def qqp(self) -> U64Vector3: ...
    @property
    def qqo(self) -> U64Vector3: ...
    @property
    def qql(self) -> U64Vector3: ...
    @property
    def qps(self) -> U64Vector3: ...
    @property
    def qpt(self) -> U64Vector3: ...
    @property
    def qpq(self) -> U64Vector3: ...
    @property
    def qpp(self) -> U64Vector3: ...
    @property
    def qpo(self) -> U64Vector3: ...
    @property
    def qpl(self) -> U64Vector3: ...
    @property
    def qos(self) -> U64Vector3: ...
    @property
    def qot(self) -> U64Vector3: ...
    @property
    def qoq(self) -> U64Vector3: ...
    @property
    def qop(self) -> U64Vector3: ...
    @property
    def qoo(self) -> U64Vector3: ...
    @property
    def qol(self) -> U64Vector3: ...
    @property
    def qls(self) -> U64Vector3: ...
    @property
    def qlt(self) -> U64Vector3: ...
    @property
    def qlq(self) -> U64Vector3: ...
    @property
    def qlp(self) -> U64Vector3: ...
    @property
    def qlo(self) -> U64Vector3: ...
    @property
    def qll(self) -> U64Vector3: ...
    @property
    def pss(self) -> U64Vector3: ...
    @property
    def pst(self) -> U64Vector3: ...
    @property
    def psq(self) -> U64Vector3: ...
    @property
    def psp(self) -> U64Vector3: ...
    @property
    def pso(self) -> U64Vector3: ...
    @property
    def psl(self) -> U64Vector3: ...
    @property
    def pts(self) -> U64Vector3: ...
    @property
    def ptt(self) -> U64Vector3: ...
    @property
    def ptq(self) -> U64Vector3: ...
    @property
    def ptp(self) -> U64Vector3: ...
    @property
    def pto(self) -> U64Vector3: ...
    @property
    def ptl(self) -> U64Vector3: ...
    @property
    def pqs(self) -> U64Vector3: ...
    @property
    def pqt(self) -> U64Vector3: ...
    @property
    def pqq(self) -> U64Vector3: ...
    @property
    def pqp(self) -> U64Vector3: ...
    @property
    def pqo(self) -> U64Vector3: ...
    @property
    def pql(self) -> U64Vector3: ...
    @property
    def pps(self) -> U64Vector3: ...
    @property
    def ppt(self) -> U64Vector3: ...
    @property
    def ppq(self) -> U64Vector3: ...
    @property
    def ppp(self) -> U64Vector3: ...
    @property
    def ppo(self) -> U64Vector3: ...
    @property
    def ppl(self) -> U64Vector3: ...
    @property
    def pos(self) -> U64Vector3: ...
    @property
    def pot(self) -> U64Vector3: ...
    @property
    def poq(self) -> U64Vector3: ...
    @property
    def pop(self) -> U64Vector3: ...
    @property
    def poo(self) -> U64Vector3: ...
    @property
    def pol(self) -> U64Vector3: ...
    @property
    def pls(self) -> U64Vector3: ...
    @property
    def plt(self) -> U64Vector3: ...
    @property
    def plq(self) -> U64Vector3: ...
    @property
    def plp(self) -> U64Vector3: ...
    @property
    def plo(self) -> U64Vector3: ...
    @property
    def pll(self) -> U64Vector3: ...
    @property
    def oss(self) -> U64Vector3: ...
    @property
    def ost(self) -> U64Vector3: ...
    @property
    def osq(self) -> U64Vector3: ...
    @property
    def osp(self) -> U64Vector3: ...
    @property
    def oso(self) -> U64Vector3: ...
    @property
    def osl(self) -> U64Vector3: ...
    @property
    def ots(self) -> U64Vector3: ...
    @property
    def ott(self) -> U64Vector3: ...
    @property
    def otq(self) -> U64Vector3: ...
    @property
    def otp(self) -> U64Vector3: ...
    @property
    def oto(self) -> U64Vector3: ...
    @property
    def otl(self) -> U64Vector3: ...
    @property
    def oqs(self) -> U64Vector3: ...
    @property
    def oqt(self) -> U64Vector3: ...
    @property
    def oqq(self) -> U64Vector3: ...
    @property
    def oqp(self) -> U64Vector3: ...
    @property
    def oqo(self) -> U64Vector3: ...
    @property
    def oql(self) -> U64Vector3: ...
    @property
    def ops(self) -> U64Vector3: ...
    @property
    def opt(self) -> U64Vector3: ...
    @property
    def opq(self) -> U64Vector3: ...
    @property
    def opp(self) -> U64Vector3: ...
    @property
    def opo(self) -> U64Vector3: ...
    @property
    def opl(self) -> U64Vector3: ...
    @property
    def oos(self) -> U64Vector3: ...
    @property
    def oot(self) -> U64Vector3: ...
    @property
    def ooq(self) -> U64Vector3: ...
    @property
    def oop(self) -> U64Vector3: ...
    @property
    def ols(self) -> U64Vector3: ...
    @property
    def olt(self) -> U64Vector3: ...
    @property
    def olq(self) -> U64Vector3: ...
    @property
    def olp(self) -> U64Vector3: ...
    @property
    def lss(self) -> U64Vector3: ...
    @property
    def lst(self) -> U64Vector3: ...
    @property
    def lsq(self) -> U64Vector3: ...
    @property
    def lsp(self) -> U64Vector3: ...
    @property
    def lso(self) -> U64Vector3: ...
    @property
    def lsl(self) -> U64Vector3: ...
    @property
    def lts(self) -> U64Vector3: ...
    @property
    def ltt(self) -> U64Vector3: ...
    @property
    def ltq(self) -> U64Vector3: ...
    @property
    def ltp(self) -> U64Vector3: ...
    @property
    def lto(self) -> U64Vector3: ...
    @property
    def ltl(self) -> U64Vector3: ...
    @property
    def lqs(self) -> U64Vector3: ...
    @property
    def lqt(self) -> U64Vector3: ...
    @property
    def lqq(self) -> U64Vector3: ...
    @property
    def lqp(self) -> U64Vector3: ...
    @property
    def lqo(self) -> U64Vector3: ...
    @property
    def lql(self) -> U64Vector3: ...
    @property
    def lps(self) -> U64Vector3: ...
    @property
    def lpt(self) -> U64Vector3: ...
    @property
    def lpq(self) -> U64Vector3: ...
    @property
    def lpp(self) -> U64Vector3: ...
    @property
    def lpo(self) -> U64Vector3: ...
    @property
    def lpl(self) -> U64Vector3: ...
    @property
    def los(self) -> U64Vector3: ...
    @property
    def lot(self) -> U64Vector3: ...
    @property
    def loq(self) -> U64Vector3: ...
    @property
    def lop(self) -> U64Vector3: ...
    @property
    def lls(self) -> U64Vector3: ...
    @property
    def llt(self) -> U64Vector3: ...
    @property
    def llq(self) -> U64Vector3: ...
    @property
    def llp(self) -> U64Vector3: ...
    @property
    def ssss(self) -> U64Vector4: ...
    @property
    def ssst(self) -> U64Vector4: ...
    @property
    def sssq(self) -> U64Vector4: ...
    @property
    def sssp(self) -> U64Vector4: ...
    @property
    def ssso(self) -> U64Vector4: ...
    @property
    def sssl(self) -> U64Vector4: ...
    @property
    def ssts(self) -> U64Vector4: ...
    @property
    def sstt(self) -> U64Vector4: ...
    @property
    def sstq(self) -> U64Vector4: ...
    @property
    def sstp(self) -> U64Vector4: ...
    @property
    def ssto(self) -> U64Vector4: ...
    @property
    def sstl(self) -> U64Vector4: ...
    @property
    def ssqs(self) -> U64Vector4: ...
    @property
    def ssqt(self) -> U64Vector4: ...
    @property
    def ssqq(self) -> U64Vector4: ...
    @property
    def ssqp(self) -> U64Vector4: ...
    @property
    def ssqo(self) -> U64Vector4: ...
    @property
    def ssql(self) -> U64Vector4: ...
    @property
    def ssps(self) -> U64Vector4: ...
    @property
    def sspt(self) -> U64Vector4: ...
    @property
    def sspq(self) -> U64Vector4: ...
    @property
    def sspp(self) -> U64Vector4: ...
    @property
    def sspo(self) -> U64Vector4: ...
    @property
    def sspl(self) -> U64Vector4: ...
    @property
    def ssos(self) -> U64Vector4: ...
    @property
    def ssot(self) -> U64Vector4: ...
    @property
    def ssoq(self) -> U64Vector4: ...
    @property
    def ssop(self) -> U64Vector4: ...
    @property
    def ssoo(self) -> U64Vector4: ...
    @property
    def ssol(self) -> U64Vector4: ...
    @property
    def ssls(self) -> U64Vector4: ...
    @property
    def sslt(self) -> U64Vector4: ...
    @property
    def sslq(self) -> U64Vector4: ...
    @property
    def sslp(self) -> U64Vector4: ...
    @property
    def sslo(self) -> U64Vector4: ...
    @property
    def ssll(self) -> U64Vector4: ...
    @property
    def stss(self) -> U64Vector4: ...
    @property
    def stst(self) -> U64Vector4: ...
    @property
    def stsq(self) -> U64Vector4: ...
    @property
    def stsp(self) -> U64Vector4: ...
    @property
    def stso(self) -> U64Vector4: ...
    @property
    def stsl(self) -> U64Vector4: ...
    @property
    def stts(self) -> U64Vector4: ...
    @property
    def sttt(self) -> U64Vector4: ...
    @property
    def sttq(self) -> U64Vector4: ...
    @property
    def sttp(self) -> U64Vector4: ...
    @property
    def stto(self) -> U64Vector4: ...
    @property
    def sttl(self) -> U64Vector4: ...
    @property
    def stqs(self) -> U64Vector4: ...
    @property
    def stqt(self) -> U64Vector4: ...
    @property
    def stqq(self) -> U64Vector4: ...
    @property
    def stqp(self) -> U64Vector4: ...
    @property
    def stqo(self) -> U64Vector4: ...
    @property
    def stql(self) -> U64Vector4: ...
    @property
    def stps(self) -> U64Vector4: ...
    @property
    def stpt(self) -> U64Vector4: ...
    @property
    def stpq(self) -> U64Vector4: ...
    @property
    def stpp(self) -> U64Vector4: ...
    @property
    def stpo(self) -> U64Vector4: ...
    @property
    def stpl(self) -> U64Vector4: ...
    @property
    def stos(self) -> U64Vector4: ...
    @property
    def stot(self) -> U64Vector4: ...
    @property
    def stoq(self) -> U64Vector4: ...
    @property
    def stop(self) -> U64Vector4: ...
    @property
    def stoo(self) -> U64Vector4: ...
    @property
    def stol(self) -> U64Vector4: ...
    @property
    def stls(self) -> U64Vector4: ...
    @property
    def stlt(self) -> U64Vector4: ...
    @property
    def stlq(self) -> U64Vector4: ...
    @property
    def stlp(self) -> U64Vector4: ...
    @property
    def stlo(self) -> U64Vector4: ...
    @property
    def stll(self) -> U64Vector4: ...
    @property
    def sqss(self) -> U64Vector4: ...
    @property
    def sqst(self) -> U64Vector4: ...
    @property
    def sqsq(self) -> U64Vector4: ...
    @property
    def sqsp(self) -> U64Vector4: ...
    @property
    def sqso(self) -> U64Vector4: ...
    @property
    def sqsl(self) -> U64Vector4: ...
    @property
    def sqts(self) -> U64Vector4: ...
    @property
    def sqtt(self) -> U64Vector4: ...
    @property
    def sqtq(self) -> U64Vector4: ...
    @property
    def sqtp(self) -> U64Vector4: ...
    @property
    def sqto(self) -> U64Vector4: ...
    @property
    def sqtl(self) -> U64Vector4: ...
    @property
    def sqqs(self) -> U64Vector4: ...
    @property
    def sqqt(self) -> U64Vector4: ...
    @property
    def sqqq(self) -> U64Vector4: ...
    @property
    def sqqp(self) -> U64Vector4: ...
    @property
    def sqqo(self) -> U64Vector4: ...
    @property
    def sqql(self) -> U64Vector4: ...
    @property
    def sqps(self) -> U64Vector4: ...
    @property
    def sqpt(self) -> U64Vector4: ...
    @property
    def sqpq(self) -> U64Vector4: ...
    @property
    def sqpp(self) -> U64Vector4: ...
    @property
    def sqpo(self) -> U64Vector4: ...
    @property
    def sqpl(self) -> U64Vector4: ...
    @property
    def sqos(self) -> U64Vector4: ...
    @property
    def sqot(self) -> U64Vector4: ...
    @property
    def sqoq(self) -> U64Vector4: ...
    @property
    def sqop(self) -> U64Vector4: ...
    @property
    def sqoo(self) -> U64Vector4: ...
    @property
    def sqol(self) -> U64Vector4: ...
    @property
    def sqls(self) -> U64Vector4: ...
    @property
    def sqlt(self) -> U64Vector4: ...
    @property
    def sqlq(self) -> U64Vector4: ...
    @property
    def sqlp(self) -> U64Vector4: ...
    @property
    def sqlo(self) -> U64Vector4: ...
    @property
    def sqll(self) -> U64Vector4: ...
    @property
    def spss(self) -> U64Vector4: ...
    @property
    def spst(self) -> U64Vector4: ...
    @property
    def spsq(self) -> U64Vector4: ...
    @property
    def spsp(self) -> U64Vector4: ...
    @property
    def spso(self) -> U64Vector4: ...
    @property
    def spsl(self) -> U64Vector4: ...
    @property
    def spts(self) -> U64Vector4: ...
    @property
    def sptt(self) -> U64Vector4: ...
    @property
    def sptq(self) -> U64Vector4: ...
    @property
    def sptp(self) -> U64Vector4: ...
    @property
    def spto(self) -> U64Vector4: ...
    @property
    def sptl(self) -> U64Vector4: ...
    @property
    def spqs(self) -> U64Vector4: ...
    @property
    def spqt(self) -> U64Vector4: ...
    @property
    def spqq(self) -> U64Vector4: ...
    @property
    def spqp(self) -> U64Vector4: ...
    @property
    def spqo(self) -> U64Vector4: ...
    @property
    def spql(self) -> U64Vector4: ...
    @property
    def spps(self) -> U64Vector4: ...
    @property
    def sppt(self) -> U64Vector4: ...
    @property
    def sppq(self) -> U64Vector4: ...
    @property
    def sppp(self) -> U64Vector4: ...
    @property
    def sppo(self) -> U64Vector4: ...
    @property
    def sppl(self) -> U64Vector4: ...
    @property
    def spos(self) -> U64Vector4: ...
    @property
    def spot(self) -> U64Vector4: ...
    @property
    def spoq(self) -> U64Vector4: ...
    @property
    def spop(self) -> U64Vector4: ...
    @property
    def spoo(self) -> U64Vector4: ...
    @property
    def spol(self) -> U64Vector4: ...
    @property
    def spls(self) -> U64Vector4: ...
    @property
    def splt(self) -> U64Vector4: ...
    @property
    def splq(self) -> U64Vector4: ...
    @property
    def splp(self) -> U64Vector4: ...
    @property
    def splo(self) -> U64Vector4: ...
    @property
    def spll(self) -> U64Vector4: ...
    @property
    def soss(self) -> U64Vector4: ...
    @property
    def sost(self) -> U64Vector4: ...
    @property
    def sosq(self) -> U64Vector4: ...
    @property
    def sosp(self) -> U64Vector4: ...
    @property
    def soso(self) -> U64Vector4: ...
    @property
    def sosl(self) -> U64Vector4: ...
    @property
    def sots(self) -> U64Vector4: ...
    @property
    def sott(self) -> U64Vector4: ...
    @property
    def sotq(self) -> U64Vector4: ...
    @property
    def sotp(self) -> U64Vector4: ...
    @property
    def soto(self) -> U64Vector4: ...
    @property
    def sotl(self) -> U64Vector4: ...
    @property
    def soqs(self) -> U64Vector4: ...
    @property
    def soqt(self) -> U64Vector4: ...
    @property
    def soqq(self) -> U64Vector4: ...
    @property
    def soqp(self) -> U64Vector4: ...
    @property
    def soqo(self) -> U64Vector4: ...
    @property
    def soql(self) -> U64Vector4: ...
    @property
    def sops(self) -> U64Vector4: ...
    @property
    def sopt(self) -> U64Vector4: ...
    @property
    def sopq(self) -> U64Vector4: ...
    @property
    def sopp(self) -> U64Vector4: ...
    @property
    def sopo(self) -> U64Vector4: ...
    @property
    def sopl(self) -> U64Vector4: ...
    @property
    def soos(self) -> U64Vector4: ...
    @property
    def soot(self) -> U64Vector4: ...
    @property
    def sooq(self) -> U64Vector4: ...
    @property
    def soop(self) -> U64Vector4: ...
    @property
    def sooo(self) -> U64Vector4: ...
    @property
    def sool(self) -> U64Vector4: ...
    @property
    def sols(self) -> U64Vector4: ...
    @property
    def solt(self) -> U64Vector4: ...
    @property
    def solq(self) -> U64Vector4: ...
    @property
    def solp(self) -> U64Vector4: ...
    @property
    def solo(self) -> U64Vector4: ...
    @property
    def soll(self) -> U64Vector4: ...
    @property
    def slss(self) -> U64Vector4: ...
    @property
    def slst(self) -> U64Vector4: ...
    @property
    def slsq(self) -> U64Vector4: ...
    @property
    def slsp(self) -> U64Vector4: ...
    @property
    def slso(self) -> U64Vector4: ...
    @property
    def slsl(self) -> U64Vector4: ...
    @property
    def slts(self) -> U64Vector4: ...
    @property
    def sltt(self) -> U64Vector4: ...
    @property
    def sltq(self) -> U64Vector4: ...
    @property
    def sltp(self) -> U64Vector4: ...
    @property
    def slto(self) -> U64Vector4: ...
    @property
    def sltl(self) -> U64Vector4: ...
    @property
    def slqs(self) -> U64Vector4: ...
    @property
    def slqt(self) -> U64Vector4: ...
    @property
    def slqq(self) -> U64Vector4: ...
    @property
    def slqp(self) -> U64Vector4: ...
    @property
    def slqo(self) -> U64Vector4: ...
    @property
    def slql(self) -> U64Vector4: ...
    @property
    def slps(self) -> U64Vector4: ...
    @property
    def slpt(self) -> U64Vector4: ...
    @property
    def slpq(self) -> U64Vector4: ...
    @property
    def slpp(self) -> U64Vector4: ...
    @property
    def slpo(self) -> U64Vector4: ...
    @property
    def slpl(self) -> U64Vector4: ...
    @property
    def slos(self) -> U64Vector4: ...
    @property
    def slot(self) -> U64Vector4: ...
    @property
    def sloq(self) -> U64Vector4: ...
    @property
    def slop(self) -> U64Vector4: ...
    @property
    def sloo(self) -> U64Vector4: ...
    @property
    def slol(self) -> U64Vector4: ...
    @property
    def slls(self) -> U64Vector4: ...
    @property
    def sllt(self) -> U64Vector4: ...
    @property
    def sllq(self) -> U64Vector4: ...
    @property
    def sllp(self) -> U64Vector4: ...
    @property
    def sllo(self) -> U64Vector4: ...
    @property
    def slll(self) -> U64Vector4: ...
    @property
    def tsss(self) -> U64Vector4: ...
    @property
    def tsst(self) -> U64Vector4: ...
    @property
    def tssq(self) -> U64Vector4: ...
    @property
    def tssp(self) -> U64Vector4: ...
    @property
    def tsso(self) -> U64Vector4: ...
    @property
    def tssl(self) -> U64Vector4: ...
    @property
    def tsts(self) -> U64Vector4: ...
    @property
    def tstt(self) -> U64Vector4: ...
    @property
    def tstq(self) -> U64Vector4: ...
    @property
    def tstp(self) -> U64Vector4: ...
    @property
    def tsto(self) -> U64Vector4: ...
    @property
    def tstl(self) -> U64Vector4: ...
    @property
    def tsqs(self) -> U64Vector4: ...
    @property
    def tsqt(self) -> U64Vector4: ...
    @property
    def tsqq(self) -> U64Vector4: ...
    @property
    def tsqp(self) -> U64Vector4: ...
    @property
    def tsqo(self) -> U64Vector4: ...
    @property
    def tsql(self) -> U64Vector4: ...
    @property
    def tsps(self) -> U64Vector4: ...
    @property
    def tspt(self) -> U64Vector4: ...
    @property
    def tspq(self) -> U64Vector4: ...
    @property
    def tspp(self) -> U64Vector4: ...
    @property
    def tspo(self) -> U64Vector4: ...
    @property
    def tspl(self) -> U64Vector4: ...
    @property
    def tsos(self) -> U64Vector4: ...
    @property
    def tsot(self) -> U64Vector4: ...
    @property
    def tsoq(self) -> U64Vector4: ...
    @property
    def tsop(self) -> U64Vector4: ...
    @property
    def tsoo(self) -> U64Vector4: ...
    @property
    def tsol(self) -> U64Vector4: ...
    @property
    def tsls(self) -> U64Vector4: ...
    @property
    def tslt(self) -> U64Vector4: ...
    @property
    def tslq(self) -> U64Vector4: ...
    @property
    def tslp(self) -> U64Vector4: ...
    @property
    def tslo(self) -> U64Vector4: ...
    @property
    def tsll(self) -> U64Vector4: ...
    @property
    def ttss(self) -> U64Vector4: ...
    @property
    def ttst(self) -> U64Vector4: ...
    @property
    def ttsq(self) -> U64Vector4: ...
    @property
    def ttsp(self) -> U64Vector4: ...
    @property
    def ttso(self) -> U64Vector4: ...
    @property
    def ttsl(self) -> U64Vector4: ...
    @property
    def ttts(self) -> U64Vector4: ...
    @property
    def tttt(self) -> U64Vector4: ...
    @property
    def tttq(self) -> U64Vector4: ...
    @property
    def tttp(self) -> U64Vector4: ...
    @property
    def ttto(self) -> U64Vector4: ...
    @property
    def tttl(self) -> U64Vector4: ...
    @property
    def ttqs(self) -> U64Vector4: ...
    @property
    def ttqt(self) -> U64Vector4: ...
    @property
    def ttqq(self) -> U64Vector4: ...
    @property
    def ttqp(self) -> U64Vector4: ...
    @property
    def ttqo(self) -> U64Vector4: ...
    @property
    def ttql(self) -> U64Vector4: ...
    @property
    def ttps(self) -> U64Vector4: ...
    @property
    def ttpt(self) -> U64Vector4: ...
    @property
    def ttpq(self) -> U64Vector4: ...
    @property
    def ttpp(self) -> U64Vector4: ...
    @property
    def ttpo(self) -> U64Vector4: ...
    @property
    def ttpl(self) -> U64Vector4: ...
    @property
    def ttos(self) -> U64Vector4: ...
    @property
    def ttot(self) -> U64Vector4: ...
    @property
    def ttoq(self) -> U64Vector4: ...
    @property
    def ttop(self) -> U64Vector4: ...
    @property
    def ttoo(self) -> U64Vector4: ...
    @property
    def ttol(self) -> U64Vector4: ...
    @property
    def ttls(self) -> U64Vector4: ...
    @property
    def ttlt(self) -> U64Vector4: ...
    @property
    def ttlq(self) -> U64Vector4: ...
    @property
    def ttlp(self) -> U64Vector4: ...
    @property
    def ttlo(self) -> U64Vector4: ...
    @property
    def ttll(self) -> U64Vector4: ...
    @property
    def tqss(self) -> U64Vector4: ...
    @property
    def tqst(self) -> U64Vector4: ...
    @property
    def tqsq(self) -> U64Vector4: ...
    @property
    def tqsp(self) -> U64Vector4: ...
    @property
    def tqso(self) -> U64Vector4: ...
    @property
    def tqsl(self) -> U64Vector4: ...
    @property
    def tqts(self) -> U64Vector4: ...
    @property
    def tqtt(self) -> U64Vector4: ...
    @property
    def tqtq(self) -> U64Vector4: ...
    @property
    def tqtp(self) -> U64Vector4: ...
    @property
    def tqto(self) -> U64Vector4: ...
    @property
    def tqtl(self) -> U64Vector4: ...
    @property
    def tqqs(self) -> U64Vector4: ...
    @property
    def tqqt(self) -> U64Vector4: ...
    @property
    def tqqq(self) -> U64Vector4: ...
    @property
    def tqqp(self) -> U64Vector4: ...
    @property
    def tqqo(self) -> U64Vector4: ...
    @property
    def tqql(self) -> U64Vector4: ...
    @property
    def tqps(self) -> U64Vector4: ...
    @property
    def tqpt(self) -> U64Vector4: ...
    @property
    def tqpq(self) -> U64Vector4: ...
    @property
    def tqpp(self) -> U64Vector4: ...
    @property
    def tqpo(self) -> U64Vector4: ...
    @property
    def tqpl(self) -> U64Vector4: ...
    @property
    def tqos(self) -> U64Vector4: ...
    @property
    def tqot(self) -> U64Vector4: ...
    @property
    def tqoq(self) -> U64Vector4: ...
    @property
    def tqop(self) -> U64Vector4: ...
    @property
    def tqoo(self) -> U64Vector4: ...
    @property
    def tqol(self) -> U64Vector4: ...
    @property
    def tqls(self) -> U64Vector4: ...
    @property
    def tqlt(self) -> U64Vector4: ...
    @property
    def tqlq(self) -> U64Vector4: ...
    @property
    def tqlp(self) -> U64Vector4: ...
    @property
    def tqlo(self) -> U64Vector4: ...
    @property
    def tqll(self) -> U64Vector4: ...
    @property
    def tpss(self) -> U64Vector4: ...
    @property
    def tpst(self) -> U64Vector4: ...
    @property
    def tpsq(self) -> U64Vector4: ...
    @property
    def tpsp(self) -> U64Vector4: ...
    @property
    def tpso(self) -> U64Vector4: ...
    @property
    def tpsl(self) -> U64Vector4: ...
    @property
    def tpts(self) -> U64Vector4: ...
    @property
    def tptt(self) -> U64Vector4: ...
    @property
    def tptq(self) -> U64Vector4: ...
    @property
    def tptp(self) -> U64Vector4: ...
    @property
    def tpto(self) -> U64Vector4: ...
    @property
    def tptl(self) -> U64Vector4: ...
    @property
    def tpqs(self) -> U64Vector4: ...
    @property
    def tpqt(self) -> U64Vector4: ...
    @property
    def tpqq(self) -> U64Vector4: ...
    @property
    def tpqp(self) -> U64Vector4: ...
    @property
    def tpqo(self) -> U64Vector4: ...
    @property
    def tpql(self) -> U64Vector4: ...
    @property
    def tpps(self) -> U64Vector4: ...
    @property
    def tppt(self) -> U64Vector4: ...
    @property
    def tppq(self) -> U64Vector4: ...
    @property
    def tppp(self) -> U64Vector4: ...
    @property
    def tppo(self) -> U64Vector4: ...
    @property
    def tppl(self) -> U64Vector4: ...
    @property
    def tpos(self) -> U64Vector4: ...
    @property
    def tpot(self) -> U64Vector4: ...
    @property
    def tpoq(self) -> U64Vector4: ...
    @property
    def tpop(self) -> U64Vector4: ...
    @property
    def tpoo(self) -> U64Vector4: ...
    @property
    def tpol(self) -> U64Vector4: ...
    @property
    def tpls(self) -> U64Vector4: ...
    @property
    def tplt(self) -> U64Vector4: ...
    @property
    def tplq(self) -> U64Vector4: ...
    @property
    def tplp(self) -> U64Vector4: ...
    @property
    def tplo(self) -> U64Vector4: ...
    @property
    def tpll(self) -> U64Vector4: ...
    @property
    def toss(self) -> U64Vector4: ...
    @property
    def tost(self) -> U64Vector4: ...
    @property
    def tosq(self) -> U64Vector4: ...
    @property
    def tosp(self) -> U64Vector4: ...
    @property
    def toso(self) -> U64Vector4: ...
    @property
    def tosl(self) -> U64Vector4: ...
    @property
    def tots(self) -> U64Vector4: ...
    @property
    def tott(self) -> U64Vector4: ...
    @property
    def totq(self) -> U64Vector4: ...
    @property
    def totp(self) -> U64Vector4: ...
    @property
    def toto(self) -> U64Vector4: ...
    @property
    def totl(self) -> U64Vector4: ...
    @property
    def toqs(self) -> U64Vector4: ...
    @property
    def toqt(self) -> U64Vector4: ...
    @property
    def toqq(self) -> U64Vector4: ...
    @property
    def toqp(self) -> U64Vector4: ...
    @property
    def toqo(self) -> U64Vector4: ...
    @property
    def toql(self) -> U64Vector4: ...
    @property
    def tops(self) -> U64Vector4: ...
    @property
    def topt(self) -> U64Vector4: ...
    @property
    def topq(self) -> U64Vector4: ...
    @property
    def topp(self) -> U64Vector4: ...
    @property
    def topo(self) -> U64Vector4: ...
    @property
    def topl(self) -> U64Vector4: ...
    @property
    def toos(self) -> U64Vector4: ...
    @property
    def toot(self) -> U64Vector4: ...
    @property
    def tooq(self) -> U64Vector4: ...
    @property
    def toop(self) -> U64Vector4: ...
    @property
    def tooo(self) -> U64Vector4: ...
    @property
    def tool(self) -> U64Vector4: ...
    @property
    def tols(self) -> U64Vector4: ...
    @property
    def tolt(self) -> U64Vector4: ...
    @property
    def tolq(self) -> U64Vector4: ...
    @property
    def tolp(self) -> U64Vector4: ...
    @property
    def tolo(self) -> U64Vector4: ...
    @property
    def toll(self) -> U64Vector4: ...
    @property
    def tlss(self) -> U64Vector4: ...
    @property
    def tlst(self) -> U64Vector4: ...
    @property
    def tlsq(self) -> U64Vector4: ...
    @property
    def tlsp(self) -> U64Vector4: ...
    @property
    def tlso(self) -> U64Vector4: ...
    @property
    def tlsl(self) -> U64Vector4: ...
    @property
    def tlts(self) -> U64Vector4: ...
    @property
    def tltt(self) -> U64Vector4: ...
    @property
    def tltq(self) -> U64Vector4: ...
    @property
    def tltp(self) -> U64Vector4: ...
    @property
    def tlto(self) -> U64Vector4: ...
    @property
    def tltl(self) -> U64Vector4: ...
    @property
    def tlqs(self) -> U64Vector4: ...
    @property
    def tlqt(self) -> U64Vector4: ...
    @property
    def tlqq(self) -> U64Vector4: ...
    @property
    def tlqp(self) -> U64Vector4: ...
    @property
    def tlqo(self) -> U64Vector4: ...
    @property
    def tlql(self) -> U64Vector4: ...
    @property
    def tlps(self) -> U64Vector4: ...
    @property
    def tlpt(self) -> U64Vector4: ...
    @property
    def tlpq(self) -> U64Vector4: ...
    @property
    def tlpp(self) -> U64Vector4: ...
    @property
    def tlpo(self) -> U64Vector4: ...
    @property
    def tlpl(self) -> U64Vector4: ...
    @property
    def tlos(self) -> U64Vector4: ...
    @property
    def tlot(self) -> U64Vector4: ...
    @property
    def tloq(self) -> U64Vector4: ...
    @property
    def tlop(self) -> U64Vector4: ...
    @property
    def tloo(self) -> U64Vector4: ...
    @property
    def tlol(self) -> U64Vector4: ...
    @property
    def tlls(self) -> U64Vector4: ...
    @property
    def tllt(self) -> U64Vector4: ...
    @property
    def tllq(self) -> U64Vector4: ...
    @property
    def tllp(self) -> U64Vector4: ...
    @property
    def tllo(self) -> U64Vector4: ...
    @property
    def tlll(self) -> U64Vector4: ...
    @property
    def qsss(self) -> U64Vector4: ...
    @property
    def qsst(self) -> U64Vector4: ...
    @property
    def qssq(self) -> U64Vector4: ...
    @property
    def qssp(self) -> U64Vector4: ...
    @property
    def qsso(self) -> U64Vector4: ...
    @property
    def qssl(self) -> U64Vector4: ...
    @property
    def qsts(self) -> U64Vector4: ...
    @property
    def qstt(self) -> U64Vector4: ...
    @property
    def qstq(self) -> U64Vector4: ...
    @property
    def qstp(self) -> U64Vector4: ...
    @property
    def qsto(self) -> U64Vector4: ...
    @property
    def qstl(self) -> U64Vector4: ...
    @property
    def qsqs(self) -> U64Vector4: ...
    @property
    def qsqt(self) -> U64Vector4: ...
    @property
    def qsqq(self) -> U64Vector4: ...
    @property
    def qsqp(self) -> U64Vector4: ...
    @property
    def qsqo(self) -> U64Vector4: ...
    @property
    def qsql(self) -> U64Vector4: ...
    @property
    def qsps(self) -> U64Vector4: ...
    @property
    def qspt(self) -> U64Vector4: ...
    @property
    def qspq(self) -> U64Vector4: ...
    @property
    def qspp(self) -> U64Vector4: ...
    @property
    def qspo(self) -> U64Vector4: ...
    @property
    def qspl(self) -> U64Vector4: ...
    @property
    def qsos(self) -> U64Vector4: ...
    @property
    def qsot(self) -> U64Vector4: ...
    @property
    def qsoq(self) -> U64Vector4: ...
    @property
    def qsop(self) -> U64Vector4: ...
    @property
    def qsoo(self) -> U64Vector4: ...
    @property
    def qsol(self) -> U64Vector4: ...
    @property
    def qsls(self) -> U64Vector4: ...
    @property
    def qslt(self) -> U64Vector4: ...
    @property
    def qslq(self) -> U64Vector4: ...
    @property
    def qslp(self) -> U64Vector4: ...
    @property
    def qslo(self) -> U64Vector4: ...
    @property
    def qsll(self) -> U64Vector4: ...
    @property
    def qtss(self) -> U64Vector4: ...
    @property
    def qtst(self) -> U64Vector4: ...
    @property
    def qtsq(self) -> U64Vector4: ...
    @property
    def qtsp(self) -> U64Vector4: ...
    @property
    def qtso(self) -> U64Vector4: ...
    @property
    def qtsl(self) -> U64Vector4: ...
    @property
    def qtts(self) -> U64Vector4: ...
    @property
    def qttt(self) -> U64Vector4: ...
    @property
    def qttq(self) -> U64Vector4: ...
    @property
    def qttp(self) -> U64Vector4: ...
    @property
    def qtto(self) -> U64Vector4: ...
    @property
    def qttl(self) -> U64Vector4: ...
    @property
    def qtqs(self) -> U64Vector4: ...
    @property
    def qtqt(self) -> U64Vector4: ...
    @property
    def qtqq(self) -> U64Vector4: ...
    @property
    def qtqp(self) -> U64Vector4: ...
    @property
    def qtqo(self) -> U64Vector4: ...
    @property
    def qtql(self) -> U64Vector4: ...
    @property
    def qtps(self) -> U64Vector4: ...
    @property
    def qtpt(self) -> U64Vector4: ...
    @property
    def qtpq(self) -> U64Vector4: ...
    @property
    def qtpp(self) -> U64Vector4: ...
    @property
    def qtpo(self) -> U64Vector4: ...
    @property
    def qtpl(self) -> U64Vector4: ...
    @property
    def qtos(self) -> U64Vector4: ...
    @property
    def qtot(self) -> U64Vector4: ...
    @property
    def qtoq(self) -> U64Vector4: ...
    @property
    def qtop(self) -> U64Vector4: ...
    @property
    def qtoo(self) -> U64Vector4: ...
    @property
    def qtol(self) -> U64Vector4: ...
    @property
    def qtls(self) -> U64Vector4: ...
    @property
    def qtlt(self) -> U64Vector4: ...
    @property
    def qtlq(self) -> U64Vector4: ...
    @property
    def qtlp(self) -> U64Vector4: ...
    @property
    def qtlo(self) -> U64Vector4: ...
    @property
    def qtll(self) -> U64Vector4: ...
    @property
    def qqss(self) -> U64Vector4: ...
    @property
    def qqst(self) -> U64Vector4: ...
    @property
    def qqsq(self) -> U64Vector4: ...
    @property
    def qqsp(self) -> U64Vector4: ...
    @property
    def qqso(self) -> U64Vector4: ...
    @property
    def qqsl(self) -> U64Vector4: ...
    @property
    def qqts(self) -> U64Vector4: ...
    @property
    def qqtt(self) -> U64Vector4: ...
    @property
    def qqtq(self) -> U64Vector4: ...
    @property
    def qqtp(self) -> U64Vector4: ...
    @property
    def qqto(self) -> U64Vector4: ...
    @property
    def qqtl(self) -> U64Vector4: ...
    @property
    def qqqs(self) -> U64Vector4: ...
    @property
    def qqqt(self) -> U64Vector4: ...
    @property
    def qqqq(self) -> U64Vector4: ...
    @property
    def qqqp(self) -> U64Vector4: ...
    @property
    def qqqo(self) -> U64Vector4: ...
    @property
    def qqql(self) -> U64Vector4: ...
    @property
    def qqps(self) -> U64Vector4: ...
    @property
    def qqpt(self) -> U64Vector4: ...
    @property
    def qqpq(self) -> U64Vector4: ...
    @property
    def qqpp(self) -> U64Vector4: ...
    @property
    def qqpo(self) -> U64Vector4: ...
    @property
    def qqpl(self) -> U64Vector4: ...
    @property
    def qqos(self) -> U64Vector4: ...
    @property
    def qqot(self) -> U64Vector4: ...
    @property
    def qqoq(self) -> U64Vector4: ...
    @property
    def qqop(self) -> U64Vector4: ...
    @property
    def qqoo(self) -> U64Vector4: ...
    @property
    def qqol(self) -> U64Vector4: ...
    @property
    def qqls(self) -> U64Vector4: ...
    @property
    def qqlt(self) -> U64Vector4: ...
    @property
    def qqlq(self) -> U64Vector4: ...
    @property
    def qqlp(self) -> U64Vector4: ...
    @property
    def qqlo(self) -> U64Vector4: ...
    @property
    def qqll(self) -> U64Vector4: ...
    @property
    def qpss(self) -> U64Vector4: ...
    @property
    def qpst(self) -> U64Vector4: ...
    @property
    def qpsq(self) -> U64Vector4: ...
    @property
    def qpsp(self) -> U64Vector4: ...
    @property
    def qpso(self) -> U64Vector4: ...
    @property
    def qpsl(self) -> U64Vector4: ...
    @property
    def qpts(self) -> U64Vector4: ...
    @property
    def qptt(self) -> U64Vector4: ...
    @property
    def qptq(self) -> U64Vector4: ...
    @property
    def qptp(self) -> U64Vector4: ...
    @property
    def qpto(self) -> U64Vector4: ...
    @property
    def qptl(self) -> U64Vector4: ...
    @property
    def qpqs(self) -> U64Vector4: ...
    @property
    def qpqt(self) -> U64Vector4: ...
    @property
    def qpqq(self) -> U64Vector4: ...
    @property
    def qpqp(self) -> U64Vector4: ...
    @property
    def qpqo(self) -> U64Vector4: ...
    @property
    def qpql(self) -> U64Vector4: ...
    @property
    def qpps(self) -> U64Vector4: ...
    @property
    def qppt(self) -> U64Vector4: ...
    @property
    def qppq(self) -> U64Vector4: ...
    @property
    def qppp(self) -> U64Vector4: ...
    @property
    def qppo(self) -> U64Vector4: ...
    @property
    def qppl(self) -> U64Vector4: ...
    @property
    def qpos(self) -> U64Vector4: ...
    @property
    def qpot(self) -> U64Vector4: ...
    @property
    def qpoq(self) -> U64Vector4: ...
    @property
    def qpop(self) -> U64Vector4: ...
    @property
    def qpoo(self) -> U64Vector4: ...
    @property
    def qpol(self) -> U64Vector4: ...
    @property
    def qpls(self) -> U64Vector4: ...
    @property
    def qplt(self) -> U64Vector4: ...
    @property
    def qplq(self) -> U64Vector4: ...
    @property
    def qplp(self) -> U64Vector4: ...
    @property
    def qplo(self) -> U64Vector4: ...
    @property
    def qpll(self) -> U64Vector4: ...
    @property
    def qoss(self) -> U64Vector4: ...
    @property
    def qost(self) -> U64Vector4: ...
    @property
    def qosq(self) -> U64Vector4: ...
    @property
    def qosp(self) -> U64Vector4: ...
    @property
    def qoso(self) -> U64Vector4: ...
    @property
    def qosl(self) -> U64Vector4: ...
    @property
    def qots(self) -> U64Vector4: ...
    @property
    def qott(self) -> U64Vector4: ...
    @property
    def qotq(self) -> U64Vector4: ...
    @property
    def qotp(self) -> U64Vector4: ...
    @property
    def qoto(self) -> U64Vector4: ...
    @property
    def qotl(self) -> U64Vector4: ...
    @property
    def qoqs(self) -> U64Vector4: ...
    @property
    def qoqt(self) -> U64Vector4: ...
    @property
    def qoqq(self) -> U64Vector4: ...
    @property
    def qoqp(self) -> U64Vector4: ...
    @property
    def qoqo(self) -> U64Vector4: ...
    @property
    def qoql(self) -> U64Vector4: ...
    @property
    def qops(self) -> U64Vector4: ...
    @property
    def qopt(self) -> U64Vector4: ...
    @property
    def qopq(self) -> U64Vector4: ...
    @property
    def qopp(self) -> U64Vector4: ...
    @property
    def qopo(self) -> U64Vector4: ...
    @property
    def qopl(self) -> U64Vector4: ...
    @property
    def qoos(self) -> U64Vector4: ...
    @property
    def qoot(self) -> U64Vector4: ...
    @property
    def qooq(self) -> U64Vector4: ...
    @property
    def qoop(self) -> U64Vector4: ...
    @property
    def qooo(self) -> U64Vector4: ...
    @property
    def qool(self) -> U64Vector4: ...
    @property
    def qols(self) -> U64Vector4: ...
    @property
    def qolt(self) -> U64Vector4: ...
    @property
    def qolq(self) -> U64Vector4: ...
    @property
    def qolp(self) -> U64Vector4: ...
    @property
    def qolo(self) -> U64Vector4: ...
    @property
    def qoll(self) -> U64Vector4: ...
    @property
    def qlss(self) -> U64Vector4: ...
    @property
    def qlst(self) -> U64Vector4: ...
    @property
    def qlsq(self) -> U64Vector4: ...
    @property
    def qlsp(self) -> U64Vector4: ...
    @property
    def qlso(self) -> U64Vector4: ...
    @property
    def qlsl(self) -> U64Vector4: ...
    @property
    def qlts(self) -> U64Vector4: ...
    @property
    def qltt(self) -> U64Vector4: ...
    @property
    def qltq(self) -> U64Vector4: ...
    @property
    def qltp(self) -> U64Vector4: ...
    @property
    def qlto(self) -> U64Vector4: ...
    @property
    def qltl(self) -> U64Vector4: ...
    @property
    def qlqs(self) -> U64Vector4: ...
    @property
    def qlqt(self) -> U64Vector4: ...
    @property
    def qlqq(self) -> U64Vector4: ...
    @property
    def qlqp(self) -> U64Vector4: ...
    @property
    def qlqo(self) -> U64Vector4: ...
    @property
    def qlql(self) -> U64Vector4: ...
    @property
    def qlps(self) -> U64Vector4: ...
    @property
    def qlpt(self) -> U64Vector4: ...
    @property
    def qlpq(self) -> U64Vector4: ...
    @property
    def qlpp(self) -> U64Vector4: ...
    @property
    def qlpo(self) -> U64Vector4: ...
    @property
    def qlpl(self) -> U64Vector4: ...
    @property
    def qlos(self) -> U64Vector4: ...
    @property
    def qlot(self) -> U64Vector4: ...
    @property
    def qloq(self) -> U64Vector4: ...
    @property
    def qlop(self) -> U64Vector4: ...
    @property
    def qloo(self) -> U64Vector4: ...
    @property
    def qlol(self) -> U64Vector4: ...
    @property
    def qlls(self) -> U64Vector4: ...
    @property
    def qllt(self) -> U64Vector4: ...
    @property
    def qllq(self) -> U64Vector4: ...
    @property
    def qllp(self) -> U64Vector4: ...
    @property
    def qllo(self) -> U64Vector4: ...
    @property
    def qlll(self) -> U64Vector4: ...
    @property
    def psss(self) -> U64Vector4: ...
    @property
    def psst(self) -> U64Vector4: ...
    @property
    def pssq(self) -> U64Vector4: ...
    @property
    def pssp(self) -> U64Vector4: ...
    @property
    def psso(self) -> U64Vector4: ...
    @property
    def pssl(self) -> U64Vector4: ...
    @property
    def psts(self) -> U64Vector4: ...
    @property
    def pstt(self) -> U64Vector4: ...
    @property
    def pstq(self) -> U64Vector4: ...
    @property
    def pstp(self) -> U64Vector4: ...
    @property
    def psto(self) -> U64Vector4: ...
    @property
    def pstl(self) -> U64Vector4: ...
    @property
    def psqs(self) -> U64Vector4: ...
    @property
    def psqt(self) -> U64Vector4: ...
    @property
    def psqq(self) -> U64Vector4: ...
    @property
    def psqp(self) -> U64Vector4: ...
    @property
    def psqo(self) -> U64Vector4: ...
    @property
    def psql(self) -> U64Vector4: ...
    @property
    def psps(self) -> U64Vector4: ...
    @property
    def pspt(self) -> U64Vector4: ...
    @property
    def pspq(self) -> U64Vector4: ...
    @property
    def pspp(self) -> U64Vector4: ...
    @property
    def pspo(self) -> U64Vector4: ...
    @property
    def pspl(self) -> U64Vector4: ...
    @property
    def psos(self) -> U64Vector4: ...
    @property
    def psot(self) -> U64Vector4: ...
    @property
    def psoq(self) -> U64Vector4: ...
    @property
    def psop(self) -> U64Vector4: ...
    @property
    def psoo(self) -> U64Vector4: ...
    @property
    def psol(self) -> U64Vector4: ...
    @property
    def psls(self) -> U64Vector4: ...
    @property
    def pslt(self) -> U64Vector4: ...
    @property
    def pslq(self) -> U64Vector4: ...
    @property
    def pslp(self) -> U64Vector4: ...
    @property
    def pslo(self) -> U64Vector4: ...
    @property
    def psll(self) -> U64Vector4: ...
    @property
    def ptss(self) -> U64Vector4: ...
    @property
    def ptst(self) -> U64Vector4: ...
    @property
    def ptsq(self) -> U64Vector4: ...
    @property
    def ptsp(self) -> U64Vector4: ...
    @property
    def ptso(self) -> U64Vector4: ...
    @property
    def ptsl(self) -> U64Vector4: ...
    @property
    def ptts(self) -> U64Vector4: ...
    @property
    def pttt(self) -> U64Vector4: ...
    @property
    def pttq(self) -> U64Vector4: ...
    @property
    def pttp(self) -> U64Vector4: ...
    @property
    def ptto(self) -> U64Vector4: ...
    @property
    def pttl(self) -> U64Vector4: ...
    @property
    def ptqs(self) -> U64Vector4: ...
    @property
    def ptqt(self) -> U64Vector4: ...
    @property
    def ptqq(self) -> U64Vector4: ...
    @property
    def ptqp(self) -> U64Vector4: ...
    @property
    def ptqo(self) -> U64Vector4: ...
    @property
    def ptql(self) -> U64Vector4: ...
    @property
    def ptps(self) -> U64Vector4: ...
    @property
    def ptpt(self) -> U64Vector4: ...
    @property
    def ptpq(self) -> U64Vector4: ...
    @property
    def ptpp(self) -> U64Vector4: ...
    @property
    def ptpo(self) -> U64Vector4: ...
    @property
    def ptpl(self) -> U64Vector4: ...
    @property
    def ptos(self) -> U64Vector4: ...
    @property
    def ptot(self) -> U64Vector4: ...
    @property
    def ptoq(self) -> U64Vector4: ...
    @property
    def ptop(self) -> U64Vector4: ...
    @property
    def ptoo(self) -> U64Vector4: ...
    @property
    def ptol(self) -> U64Vector4: ...
    @property
    def ptls(self) -> U64Vector4: ...
    @property
    def ptlt(self) -> U64Vector4: ...
    @property
    def ptlq(self) -> U64Vector4: ...
    @property
    def ptlp(self) -> U64Vector4: ...
    @property
    def ptlo(self) -> U64Vector4: ...
    @property
    def ptll(self) -> U64Vector4: ...
    @property
    def pqss(self) -> U64Vector4: ...
    @property
    def pqst(self) -> U64Vector4: ...
    @property
    def pqsq(self) -> U64Vector4: ...
    @property
    def pqsp(self) -> U64Vector4: ...
    @property
    def pqso(self) -> U64Vector4: ...
    @property
    def pqsl(self) -> U64Vector4: ...
    @property
    def pqts(self) -> U64Vector4: ...
    @property
    def pqtt(self) -> U64Vector4: ...
    @property
    def pqtq(self) -> U64Vector4: ...
    @property
    def pqtp(self) -> U64Vector4: ...
    @property
    def pqto(self) -> U64Vector4: ...
    @property
    def pqtl(self) -> U64Vector4: ...
    @property
    def pqqs(self) -> U64Vector4: ...
    @property
    def pqqt(self) -> U64Vector4: ...
    @property
    def pqqq(self) -> U64Vector4: ...
    @property
    def pqqp(self) -> U64Vector4: ...
    @property
    def pqqo(self) -> U64Vector4: ...
    @property
    def pqql(self) -> U64Vector4: ...
    @property
    def pqps(self) -> U64Vector4: ...
    @property
    def pqpt(self) -> U64Vector4: ...
    @property
    def pqpq(self) -> U64Vector4: ...
    @property
    def pqpp(self) -> U64Vector4: ...
    @property
    def pqpo(self) -> U64Vector4: ...
    @property
    def pqpl(self) -> U64Vector4: ...
    @property
    def pqos(self) -> U64Vector4: ...
    @property
    def pqot(self) -> U64Vector4: ...
    @property
    def pqoq(self) -> U64Vector4: ...
    @property
    def pqop(self) -> U64Vector4: ...
    @property
    def pqoo(self) -> U64Vector4: ...
    @property
    def pqol(self) -> U64Vector4: ...
    @property
    def pqls(self) -> U64Vector4: ...
    @property
    def pqlt(self) -> U64Vector4: ...
    @property
    def pqlq(self) -> U64Vector4: ...
    @property
    def pqlp(self) -> U64Vector4: ...
    @property
    def pqlo(self) -> U64Vector4: ...
    @property
    def pqll(self) -> U64Vector4: ...
    @property
    def ppss(self) -> U64Vector4: ...
    @property
    def ppst(self) -> U64Vector4: ...
    @property
    def ppsq(self) -> U64Vector4: ...
    @property
    def ppsp(self) -> U64Vector4: ...
    @property
    def ppso(self) -> U64Vector4: ...
    @property
    def ppsl(self) -> U64Vector4: ...
    @property
    def ppts(self) -> U64Vector4: ...
    @property
    def pptt(self) -> U64Vector4: ...
    @property
    def pptq(self) -> U64Vector4: ...
    @property
    def pptp(self) -> U64Vector4: ...
    @property
    def ppto(self) -> U64Vector4: ...
    @property
    def pptl(self) -> U64Vector4: ...
    @property
    def ppqs(self) -> U64Vector4: ...
    @property
    def ppqt(self) -> U64Vector4: ...
    @property
    def ppqq(self) -> U64Vector4: ...
    @property
    def ppqp(self) -> U64Vector4: ...
    @property
    def ppqo(self) -> U64Vector4: ...
    @property
    def ppql(self) -> U64Vector4: ...
    @property
    def ppps(self) -> U64Vector4: ...
    @property
    def pppt(self) -> U64Vector4: ...
    @property
    def pppq(self) -> U64Vector4: ...
    @property
    def pppp(self) -> U64Vector4: ...
    @property
    def pppo(self) -> U64Vector4: ...
    @property
    def pppl(self) -> U64Vector4: ...
    @property
    def ppos(self) -> U64Vector4: ...
    @property
    def ppot(self) -> U64Vector4: ...
    @property
    def ppoq(self) -> U64Vector4: ...
    @property
    def ppop(self) -> U64Vector4: ...
    @property
    def ppoo(self) -> U64Vector4: ...
    @property
    def ppol(self) -> U64Vector4: ...
    @property
    def ppls(self) -> U64Vector4: ...
    @property
    def pplt(self) -> U64Vector4: ...
    @property
    def pplq(self) -> U64Vector4: ...
    @property
    def pplp(self) -> U64Vector4: ...
    @property
    def pplo(self) -> U64Vector4: ...
    @property
    def ppll(self) -> U64Vector4: ...
    @property
    def poss(self) -> U64Vector4: ...
    @property
    def post(self) -> U64Vector4: ...
    @property
    def posq(self) -> U64Vector4: ...
    @property
    def posp(self) -> U64Vector4: ...
    @property
    def poso(self) -> U64Vector4: ...
    @property
    def posl(self) -> U64Vector4: ...
    @property
    def pots(self) -> U64Vector4: ...
    @property
    def pott(self) -> U64Vector4: ...
    @property
    def potq(self) -> U64Vector4: ...
    @property
    def potp(self) -> U64Vector4: ...
    @property
    def poto(self) -> U64Vector4: ...
    @property
    def potl(self) -> U64Vector4: ...
    @property
    def poqs(self) -> U64Vector4: ...
    @property
    def poqt(self) -> U64Vector4: ...
    @property
    def poqq(self) -> U64Vector4: ...
    @property
    def poqp(self) -> U64Vector4: ...
    @property
    def poqo(self) -> U64Vector4: ...
    @property
    def poql(self) -> U64Vector4: ...
    @property
    def pops(self) -> U64Vector4: ...
    @property
    def popt(self) -> U64Vector4: ...
    @property
    def popq(self) -> U64Vector4: ...
    @property
    def popp(self) -> U64Vector4: ...
    @property
    def popo(self) -> U64Vector4: ...
    @property
    def popl(self) -> U64Vector4: ...
    @property
    def poos(self) -> U64Vector4: ...
    @property
    def poot(self) -> U64Vector4: ...
    @property
    def pooq(self) -> U64Vector4: ...
    @property
    def poop(self) -> U64Vector4: ...
    @property
    def pooo(self) -> U64Vector4: ...
    @property
    def pool(self) -> U64Vector4: ...
    @property
    def pols(self) -> U64Vector4: ...
    @property
    def polt(self) -> U64Vector4: ...
    @property
    def polq(self) -> U64Vector4: ...
    @property
    def polp(self) -> U64Vector4: ...
    @property
    def polo(self) -> U64Vector4: ...
    @property
    def poll(self) -> U64Vector4: ...
    @property
    def plss(self) -> U64Vector4: ...
    @property
    def plst(self) -> U64Vector4: ...
    @property
    def plsq(self) -> U64Vector4: ...
    @property
    def plsp(self) -> U64Vector4: ...
    @property
    def plso(self) -> U64Vector4: ...
    @property
    def plsl(self) -> U64Vector4: ...
    @property
    def plts(self) -> U64Vector4: ...
    @property
    def pltt(self) -> U64Vector4: ...
    @property
    def pltq(self) -> U64Vector4: ...
    @property
    def pltp(self) -> U64Vector4: ...
    @property
    def plto(self) -> U64Vector4: ...
    @property
    def pltl(self) -> U64Vector4: ...
    @property
    def plqs(self) -> U64Vector4: ...
    @property
    def plqt(self) -> U64Vector4: ...
    @property
    def plqq(self) -> U64Vector4: ...
    @property
    def plqp(self) -> U64Vector4: ...
    @property
    def plqo(self) -> U64Vector4: ...
    @property
    def plql(self) -> U64Vector4: ...
    @property
    def plps(self) -> U64Vector4: ...
    @property
    def plpt(self) -> U64Vector4: ...
    @property
    def plpq(self) -> U64Vector4: ...
    @property
    def plpp(self) -> U64Vector4: ...
    @property
    def plpo(self) -> U64Vector4: ...
    @property
    def plpl(self) -> U64Vector4: ...
    @property
    def plos(self) -> U64Vector4: ...
    @property
    def plot(self) -> U64Vector4: ...
    @property
    def ploq(self) -> U64Vector4: ...
    @property
    def plop(self) -> U64Vector4: ...
    @property
    def ploo(self) -> U64Vector4: ...
    @property
    def plol(self) -> U64Vector4: ...
    @property
    def plls(self) -> U64Vector4: ...
    @property
    def pllt(self) -> U64Vector4: ...
    @property
    def pllq(self) -> U64Vector4: ...
    @property
    def pllp(self) -> U64Vector4: ...
    @property
    def pllo(self) -> U64Vector4: ...
    @property
    def plll(self) -> U64Vector4: ...
    @property
    def osss(self) -> U64Vector4: ...
    @property
    def osst(self) -> U64Vector4: ...
    @property
    def ossq(self) -> U64Vector4: ...
    @property
    def ossp(self) -> U64Vector4: ...
    @property
    def osso(self) -> U64Vector4: ...
    @property
    def ossl(self) -> U64Vector4: ...
    @property
    def osts(self) -> U64Vector4: ...
    @property
    def ostt(self) -> U64Vector4: ...
    @property
    def ostq(self) -> U64Vector4: ...
    @property
    def ostp(self) -> U64Vector4: ...
    @property
    def osto(self) -> U64Vector4: ...
    @property
    def ostl(self) -> U64Vector4: ...
    @property
    def osqs(self) -> U64Vector4: ...
    @property
    def osqt(self) -> U64Vector4: ...
    @property
    def osqq(self) -> U64Vector4: ...
    @property
    def osqp(self) -> U64Vector4: ...
    @property
    def osqo(self) -> U64Vector4: ...
    @property
    def osql(self) -> U64Vector4: ...
    @property
    def osps(self) -> U64Vector4: ...
    @property
    def ospt(self) -> U64Vector4: ...
    @property
    def ospq(self) -> U64Vector4: ...
    @property
    def ospp(self) -> U64Vector4: ...
    @property
    def ospo(self) -> U64Vector4: ...
    @property
    def ospl(self) -> U64Vector4: ...
    @property
    def osos(self) -> U64Vector4: ...
    @property
    def osot(self) -> U64Vector4: ...
    @property
    def osoq(self) -> U64Vector4: ...
    @property
    def osop(self) -> U64Vector4: ...
    @property
    def osoo(self) -> U64Vector4: ...
    @property
    def osol(self) -> U64Vector4: ...
    @property
    def osls(self) -> U64Vector4: ...
    @property
    def oslt(self) -> U64Vector4: ...
    @property
    def oslq(self) -> U64Vector4: ...
    @property
    def oslp(self) -> U64Vector4: ...
    @property
    def oslo(self) -> U64Vector4: ...
    @property
    def osll(self) -> U64Vector4: ...
    @property
    def otss(self) -> U64Vector4: ...
    @property
    def otst(self) -> U64Vector4: ...
    @property
    def otsq(self) -> U64Vector4: ...
    @property
    def otsp(self) -> U64Vector4: ...
    @property
    def otso(self) -> U64Vector4: ...
    @property
    def otsl(self) -> U64Vector4: ...
    @property
    def otts(self) -> U64Vector4: ...
    @property
    def ottt(self) -> U64Vector4: ...
    @property
    def ottq(self) -> U64Vector4: ...
    @property
    def ottp(self) -> U64Vector4: ...
    @property
    def otto(self) -> U64Vector4: ...
    @property
    def ottl(self) -> U64Vector4: ...
    @property
    def otqs(self) -> U64Vector4: ...
    @property
    def otqt(self) -> U64Vector4: ...
    @property
    def otqq(self) -> U64Vector4: ...
    @property
    def otqp(self) -> U64Vector4: ...
    @property
    def otqo(self) -> U64Vector4: ...
    @property
    def otql(self) -> U64Vector4: ...
    @property
    def otps(self) -> U64Vector4: ...
    @property
    def otpt(self) -> U64Vector4: ...
    @property
    def otpq(self) -> U64Vector4: ...
    @property
    def otpp(self) -> U64Vector4: ...
    @property
    def otpo(self) -> U64Vector4: ...
    @property
    def otpl(self) -> U64Vector4: ...
    @property
    def otos(self) -> U64Vector4: ...
    @property
    def otot(self) -> U64Vector4: ...
    @property
    def otoq(self) -> U64Vector4: ...
    @property
    def otop(self) -> U64Vector4: ...
    @property
    def otoo(self) -> U64Vector4: ...
    @property
    def otol(self) -> U64Vector4: ...
    @property
    def otls(self) -> U64Vector4: ...
    @property
    def otlt(self) -> U64Vector4: ...
    @property
    def otlq(self) -> U64Vector4: ...
    @property
    def otlp(self) -> U64Vector4: ...
    @property
    def otlo(self) -> U64Vector4: ...
    @property
    def otll(self) -> U64Vector4: ...
    @property
    def oqss(self) -> U64Vector4: ...
    @property
    def oqst(self) -> U64Vector4: ...
    @property
    def oqsq(self) -> U64Vector4: ...
    @property
    def oqsp(self) -> U64Vector4: ...
    @property
    def oqso(self) -> U64Vector4: ...
    @property
    def oqsl(self) -> U64Vector4: ...
    @property
    def oqts(self) -> U64Vector4: ...
    @property
    def oqtt(self) -> U64Vector4: ...
    @property
    def oqtq(self) -> U64Vector4: ...
    @property
    def oqtp(self) -> U64Vector4: ...
    @property
    def oqto(self) -> U64Vector4: ...
    @property
    def oqtl(self) -> U64Vector4: ...
    @property
    def oqqs(self) -> U64Vector4: ...
    @property
    def oqqt(self) -> U64Vector4: ...
    @property
    def oqqq(self) -> U64Vector4: ...
    @property
    def oqqp(self) -> U64Vector4: ...
    @property
    def oqqo(self) -> U64Vector4: ...
    @property
    def oqql(self) -> U64Vector4: ...
    @property
    def oqps(self) -> U64Vector4: ...
    @property
    def oqpt(self) -> U64Vector4: ...
    @property
    def oqpq(self) -> U64Vector4: ...
    @property
    def oqpp(self) -> U64Vector4: ...
    @property
    def oqpo(self) -> U64Vector4: ...
    @property
    def oqpl(self) -> U64Vector4: ...
    @property
    def oqos(self) -> U64Vector4: ...
    @property
    def oqot(self) -> U64Vector4: ...
    @property
    def oqoq(self) -> U64Vector4: ...
    @property
    def oqop(self) -> U64Vector4: ...
    @property
    def oqoo(self) -> U64Vector4: ...
    @property
    def oqol(self) -> U64Vector4: ...
    @property
    def oqls(self) -> U64Vector4: ...
    @property
    def oqlt(self) -> U64Vector4: ...
    @property
    def oqlq(self) -> U64Vector4: ...
    @property
    def oqlp(self) -> U64Vector4: ...
    @property
    def oqlo(self) -> U64Vector4: ...
    @property
    def oqll(self) -> U64Vector4: ...
    @property
    def opss(self) -> U64Vector4: ...
    @property
    def opst(self) -> U64Vector4: ...
    @property
    def opsq(self) -> U64Vector4: ...
    @property
    def opsp(self) -> U64Vector4: ...
    @property
    def opso(self) -> U64Vector4: ...
    @property
    def opsl(self) -> U64Vector4: ...
    @property
    def opts(self) -> U64Vector4: ...
    @property
    def optt(self) -> U64Vector4: ...
    @property
    def optq(self) -> U64Vector4: ...
    @property
    def optp(self) -> U64Vector4: ...
    @property
    def opto(self) -> U64Vector4: ...
    @property
    def optl(self) -> U64Vector4: ...
    @property
    def opqs(self) -> U64Vector4: ...
    @property
    def opqt(self) -> U64Vector4: ...
    @property
    def opqq(self) -> U64Vector4: ...
    @property
    def opqp(self) -> U64Vector4: ...
    @property
    def opqo(self) -> U64Vector4: ...
    @property
    def opql(self) -> U64Vector4: ...
    @property
    def opps(self) -> U64Vector4: ...
    @property
    def oppt(self) -> U64Vector4: ...
    @property
    def oppq(self) -> U64Vector4: ...
    @property
    def oppp(self) -> U64Vector4: ...
    @property
    def oppo(self) -> U64Vector4: ...
    @property
    def oppl(self) -> U64Vector4: ...
    @property
    def opos(self) -> U64Vector4: ...
    @property
    def opot(self) -> U64Vector4: ...
    @property
    def opoq(self) -> U64Vector4: ...
    @property
    def opop(self) -> U64Vector4: ...
    @property
    def opoo(self) -> U64Vector4: ...
    @property
    def opol(self) -> U64Vector4: ...
    @property
    def opls(self) -> U64Vector4: ...
    @property
    def oplt(self) -> U64Vector4: ...
    @property
    def oplq(self) -> U64Vector4: ...
    @property
    def oplp(self) -> U64Vector4: ...
    @property
    def oplo(self) -> U64Vector4: ...
    @property
    def opll(self) -> U64Vector4: ...
    @property
    def ooss(self) -> U64Vector4: ...
    @property
    def oost(self) -> U64Vector4: ...
    @property
    def oosq(self) -> U64Vector4: ...
    @property
    def oosp(self) -> U64Vector4: ...
    @property
    def ooso(self) -> U64Vector4: ...
    @property
    def oosl(self) -> U64Vector4: ...
    @property
    def oots(self) -> U64Vector4: ...
    @property
    def oott(self) -> U64Vector4: ...
    @property
    def ootq(self) -> U64Vector4: ...
    @property
    def ootp(self) -> U64Vector4: ...
    @property
    def ooto(self) -> U64Vector4: ...
    @property
    def ootl(self) -> U64Vector4: ...
    @property
    def ooqs(self) -> U64Vector4: ...
    @property
    def ooqt(self) -> U64Vector4: ...
    @property
    def ooqq(self) -> U64Vector4: ...
    @property
    def ooqp(self) -> U64Vector4: ...
    @property
    def ooqo(self) -> U64Vector4: ...
    @property
    def ooql(self) -> U64Vector4: ...
    @property
    def oops(self) -> U64Vector4: ...
    @property
    def oopt(self) -> U64Vector4: ...
    @property
    def oopq(self) -> U64Vector4: ...
    @property
    def oopp(self) -> U64Vector4: ...
    @property
    def oopo(self) -> U64Vector4: ...
    @property
    def oopl(self) -> U64Vector4: ...
    @property
    def ooos(self) -> U64Vector4: ...
    @property
    def ooot(self) -> U64Vector4: ...
    @property
    def oooq(self) -> U64Vector4: ...
    @property
    def ooop(self) -> U64Vector4: ...
    @property
    def ools(self) -> U64Vector4: ...
    @property
    def oolt(self) -> U64Vector4: ...
    @property
    def oolq(self) -> U64Vector4: ...
    @property
    def oolp(self) -> U64Vector4: ...
    @property
    def olss(self) -> U64Vector4: ...
    @property
    def olst(self) -> U64Vector4: ...
    @property
    def olsq(self) -> U64Vector4: ...
    @property
    def olsp(self) -> U64Vector4: ...
    @property
    def olso(self) -> U64Vector4: ...
    @property
    def olsl(self) -> U64Vector4: ...
    @property
    def olts(self) -> U64Vector4: ...
    @property
    def oltt(self) -> U64Vector4: ...
    @property
    def oltq(self) -> U64Vector4: ...
    @property
    def oltp(self) -> U64Vector4: ...
    @property
    def olto(self) -> U64Vector4: ...
    @property
    def oltl(self) -> U64Vector4: ...
    @property
    def olqs(self) -> U64Vector4: ...
    @property
    def olqt(self) -> U64Vector4: ...
    @property
    def olqq(self) -> U64Vector4: ...
    @property
    def olqp(self) -> U64Vector4: ...
    @property
    def olqo(self) -> U64Vector4: ...
    @property
    def olql(self) -> U64Vector4: ...
    @property
    def olps(self) -> U64Vector4: ...
    @property
    def olpt(self) -> U64Vector4: ...
    @property
    def olpq(self) -> U64Vector4: ...
    @property
    def olpp(self) -> U64Vector4: ...
    @property
    def olpo(self) -> U64Vector4: ...
    @property
    def olpl(self) -> U64Vector4: ...
    @property
    def olos(self) -> U64Vector4: ...
    @property
    def olot(self) -> U64Vector4: ...
    @property
    def oloq(self) -> U64Vector4: ...
    @property
    def olop(self) -> U64Vector4: ...
    @property
    def olls(self) -> U64Vector4: ...
    @property
    def ollt(self) -> U64Vector4: ...
    @property
    def ollq(self) -> U64Vector4: ...
    @property
    def ollp(self) -> U64Vector4: ...
    @property
    def lsss(self) -> U64Vector4: ...
    @property
    def lsst(self) -> U64Vector4: ...
    @property
    def lssq(self) -> U64Vector4: ...
    @property
    def lssp(self) -> U64Vector4: ...
    @property
    def lsso(self) -> U64Vector4: ...
    @property
    def lssl(self) -> U64Vector4: ...
    @property
    def lsts(self) -> U64Vector4: ...
    @property
    def lstt(self) -> U64Vector4: ...
    @property
    def lstq(self) -> U64Vector4: ...
    @property
    def lstp(self) -> U64Vector4: ...
    @property
    def lsto(self) -> U64Vector4: ...
    @property
    def lstl(self) -> U64Vector4: ...
    @property
    def lsqs(self) -> U64Vector4: ...
    @property
    def lsqt(self) -> U64Vector4: ...
    @property
    def lsqq(self) -> U64Vector4: ...
    @property
    def lsqp(self) -> U64Vector4: ...
    @property
    def lsqo(self) -> U64Vector4: ...
    @property
    def lsql(self) -> U64Vector4: ...
    @property
    def lsps(self) -> U64Vector4: ...
    @property
    def lspt(self) -> U64Vector4: ...
    @property
    def lspq(self) -> U64Vector4: ...
    @property
    def lspp(self) -> U64Vector4: ...
    @property
    def lspo(self) -> U64Vector4: ...
    @property
    def lspl(self) -> U64Vector4: ...
    @property
    def lsos(self) -> U64Vector4: ...
    @property
    def lsot(self) -> U64Vector4: ...
    @property
    def lsoq(self) -> U64Vector4: ...
    @property
    def lsop(self) -> U64Vector4: ...
    @property
    def lsoo(self) -> U64Vector4: ...
    @property
    def lsol(self) -> U64Vector4: ...
    @property
    def lsls(self) -> U64Vector4: ...
    @property
    def lslt(self) -> U64Vector4: ...
    @property
    def lslq(self) -> U64Vector4: ...
    @property
    def lslp(self) -> U64Vector4: ...
    @property
    def lslo(self) -> U64Vector4: ...
    @property
    def lsll(self) -> U64Vector4: ...
    @property
    def ltss(self) -> U64Vector4: ...
    @property
    def ltst(self) -> U64Vector4: ...
    @property
    def ltsq(self) -> U64Vector4: ...
    @property
    def ltsp(self) -> U64Vector4: ...
    @property
    def ltso(self) -> U64Vector4: ...
    @property
    def ltsl(self) -> U64Vector4: ...
    @property
    def ltts(self) -> U64Vector4: ...
    @property
    def lttt(self) -> U64Vector4: ...
    @property
    def lttq(self) -> U64Vector4: ...
    @property
    def lttp(self) -> U64Vector4: ...
    @property
    def ltto(self) -> U64Vector4: ...
    @property
    def lttl(self) -> U64Vector4: ...
    @property
    def ltqs(self) -> U64Vector4: ...
    @property
    def ltqt(self) -> U64Vector4: ...
    @property
    def ltqq(self) -> U64Vector4: ...
    @property
    def ltqp(self) -> U64Vector4: ...
    @property
    def ltqo(self) -> U64Vector4: ...
    @property
    def ltql(self) -> U64Vector4: ...
    @property
    def ltps(self) -> U64Vector4: ...
    @property
    def ltpt(self) -> U64Vector4: ...
    @property
    def ltpq(self) -> U64Vector4: ...
    @property
    def ltpp(self) -> U64Vector4: ...
    @property
    def ltpo(self) -> U64Vector4: ...
    @property
    def ltpl(self) -> U64Vector4: ...
    @property
    def ltos(self) -> U64Vector4: ...
    @property
    def ltot(self) -> U64Vector4: ...
    @property
    def ltoq(self) -> U64Vector4: ...
    @property
    def ltop(self) -> U64Vector4: ...
    @property
    def ltoo(self) -> U64Vector4: ...
    @property
    def ltol(self) -> U64Vector4: ...
    @property
    def ltls(self) -> U64Vector4: ...
    @property
    def ltlt(self) -> U64Vector4: ...
    @property
    def ltlq(self) -> U64Vector4: ...
    @property
    def ltlp(self) -> U64Vector4: ...
    @property
    def ltlo(self) -> U64Vector4: ...
    @property
    def ltll(self) -> U64Vector4: ...
    @property
    def lqss(self) -> U64Vector4: ...
    @property
    def lqst(self) -> U64Vector4: ...
    @property
    def lqsq(self) -> U64Vector4: ...
    @property
    def lqsp(self) -> U64Vector4: ...
    @property
    def lqso(self) -> U64Vector4: ...
    @property
    def lqsl(self) -> U64Vector4: ...
    @property
    def lqts(self) -> U64Vector4: ...
    @property
    def lqtt(self) -> U64Vector4: ...
    @property
    def lqtq(self) -> U64Vector4: ...
    @property
    def lqtp(self) -> U64Vector4: ...
    @property
    def lqto(self) -> U64Vector4: ...
    @property
    def lqtl(self) -> U64Vector4: ...
    @property
    def lqqs(self) -> U64Vector4: ...
    @property
    def lqqt(self) -> U64Vector4: ...
    @property
    def lqqq(self) -> U64Vector4: ...
    @property
    def lqqp(self) -> U64Vector4: ...
    @property
    def lqqo(self) -> U64Vector4: ...
    @property
    def lqql(self) -> U64Vector4: ...
    @property
    def lqps(self) -> U64Vector4: ...
    @property
    def lqpt(self) -> U64Vector4: ...
    @property
    def lqpq(self) -> U64Vector4: ...
    @property
    def lqpp(self) -> U64Vector4: ...
    @property
    def lqpo(self) -> U64Vector4: ...
    @property
    def lqpl(self) -> U64Vector4: ...
    @property
    def lqos(self) -> U64Vector4: ...
    @property
    def lqot(self) -> U64Vector4: ...
    @property
    def lqoq(self) -> U64Vector4: ...
    @property
    def lqop(self) -> U64Vector4: ...
    @property
    def lqoo(self) -> U64Vector4: ...
    @property
    def lqol(self) -> U64Vector4: ...
    @property
    def lqls(self) -> U64Vector4: ...
    @property
    def lqlt(self) -> U64Vector4: ...
    @property
    def lqlq(self) -> U64Vector4: ...
    @property
    def lqlp(self) -> U64Vector4: ...
    @property
    def lqlo(self) -> U64Vector4: ...
    @property
    def lqll(self) -> U64Vector4: ...
    @property
    def lpss(self) -> U64Vector4: ...
    @property
    def lpst(self) -> U64Vector4: ...
    @property
    def lpsq(self) -> U64Vector4: ...
    @property
    def lpsp(self) -> U64Vector4: ...
    @property
    def lpso(self) -> U64Vector4: ...
    @property
    def lpsl(self) -> U64Vector4: ...
    @property
    def lpts(self) -> U64Vector4: ...
    @property
    def lptt(self) -> U64Vector4: ...
    @property
    def lptq(self) -> U64Vector4: ...
    @property
    def lptp(self) -> U64Vector4: ...
    @property
    def lpto(self) -> U64Vector4: ...
    @property
    def lptl(self) -> U64Vector4: ...
    @property
    def lpqs(self) -> U64Vector4: ...
    @property
    def lpqt(self) -> U64Vector4: ...
    @property
    def lpqq(self) -> U64Vector4: ...
    @property
    def lpqp(self) -> U64Vector4: ...
    @property
    def lpqo(self) -> U64Vector4: ...
    @property
    def lpql(self) -> U64Vector4: ...
    @property
    def lpps(self) -> U64Vector4: ...
    @property
    def lppt(self) -> U64Vector4: ...
    @property
    def lppq(self) -> U64Vector4: ...
    @property
    def lppp(self) -> U64Vector4: ...
    @property
    def lppo(self) -> U64Vector4: ...
    @property
    def lppl(self) -> U64Vector4: ...
    @property
    def lpos(self) -> U64Vector4: ...
    @property
    def lpot(self) -> U64Vector4: ...
    @property
    def lpoq(self) -> U64Vector4: ...
    @property
    def lpop(self) -> U64Vector4: ...
    @property
    def lpoo(self) -> U64Vector4: ...
    @property
    def lpol(self) -> U64Vector4: ...
    @property
    def lpls(self) -> U64Vector4: ...
    @property
    def lplt(self) -> U64Vector4: ...
    @property
    def lplq(self) -> U64Vector4: ...
    @property
    def lplp(self) -> U64Vector4: ...
    @property
    def lplo(self) -> U64Vector4: ...
    @property
    def lpll(self) -> U64Vector4: ...
    @property
    def loss(self) -> U64Vector4: ...
    @property
    def lost(self) -> U64Vector4: ...
    @property
    def losq(self) -> U64Vector4: ...
    @property
    def losp(self) -> U64Vector4: ...
    @property
    def loso(self) -> U64Vector4: ...
    @property
    def losl(self) -> U64Vector4: ...
    @property
    def lots(self) -> U64Vector4: ...
    @property
    def lott(self) -> U64Vector4: ...
    @property
    def lotq(self) -> U64Vector4: ...
    @property
    def lotp(self) -> U64Vector4: ...
    @property
    def loto(self) -> U64Vector4: ...
    @property
    def lotl(self) -> U64Vector4: ...
    @property
    def loqs(self) -> U64Vector4: ...
    @property
    def loqt(self) -> U64Vector4: ...
    @property
    def loqq(self) -> U64Vector4: ...
    @property
    def loqp(self) -> U64Vector4: ...
    @property
    def loqo(self) -> U64Vector4: ...
    @property
    def loql(self) -> U64Vector4: ...
    @property
    def lops(self) -> U64Vector4: ...
    @property
    def lopt(self) -> U64Vector4: ...
    @property
    def lopq(self) -> U64Vector4: ...
    @property
    def lopp(self) -> U64Vector4: ...
    @property
    def lopo(self) -> U64Vector4: ...
    @property
    def lopl(self) -> U64Vector4: ...
    @property
    def loos(self) -> U64Vector4: ...
    @property
    def loot(self) -> U64Vector4: ...
    @property
    def looq(self) -> U64Vector4: ...
    @property
    def loop(self) -> U64Vector4: ...
    @property
    def lols(self) -> U64Vector4: ...
    @property
    def lolt(self) -> U64Vector4: ...
    @property
    def lolq(self) -> U64Vector4: ...
    @property
    def lolp(self) -> U64Vector4: ...
    @property
    def llss(self) -> U64Vector4: ...
    @property
    def llst(self) -> U64Vector4: ...
    @property
    def llsq(self) -> U64Vector4: ...
    @property
    def llsp(self) -> U64Vector4: ...
    @property
    def llso(self) -> U64Vector4: ...
    @property
    def llsl(self) -> U64Vector4: ...
    @property
    def llts(self) -> U64Vector4: ...
    @property
    def lltt(self) -> U64Vector4: ...
    @property
    def lltq(self) -> U64Vector4: ...
    @property
    def lltp(self) -> U64Vector4: ...
    @property
    def llto(self) -> U64Vector4: ...
    @property
    def lltl(self) -> U64Vector4: ...
    @property
    def llqs(self) -> U64Vector4: ...
    @property
    def llqt(self) -> U64Vector4: ...
    @property
    def llqq(self) -> U64Vector4: ...
    @property
    def llqp(self) -> U64Vector4: ...
    @property
    def llqo(self) -> U64Vector4: ...
    @property
    def llql(self) -> U64Vector4: ...
    @property
    def llps(self) -> U64Vector4: ...
    @property
    def llpt(self) -> U64Vector4: ...
    @property
    def llpq(self) -> U64Vector4: ...
    @property
    def llpp(self) -> U64Vector4: ...
    @property
    def llpo(self) -> U64Vector4: ...
    @property
    def llpl(self) -> U64Vector4: ...
    @property
    def llos(self) -> U64Vector4: ...
    @property
    def llot(self) -> U64Vector4: ...
    @property
    def lloq(self) -> U64Vector4: ...
    @property
    def llop(self) -> U64Vector4: ...
    @property
    def llls(self) -> U64Vector4: ...
    @property
    def lllt(self) -> U64Vector4: ...
    @property
    def lllq(self) -> U64Vector4: ...
    @property
    def lllp(self) -> U64Vector4: ...
    @property
    def u(self) -> int: ...
    @property
    def v(self) -> int: ...
    @property
    def uu(self) -> U64Vector2: ...
    @property
    def uv(self) -> U64Vector2: ...
    @property
    def uo(self) -> U64Vector2: ...
    @property
    def ul(self) -> U64Vector2: ...
    @property
    def vu(self) -> U64Vector2: ...
    @property
    def vv(self) -> U64Vector2: ...
    @property
    def vo(self) -> U64Vector2: ...
    @property
    def vl(self) -> U64Vector2: ...
    @property
    def ou(self) -> U64Vector2: ...
    @property
    def ov(self) -> U64Vector2: ...
    @property
    def lu(self) -> U64Vector2: ...
    @property
    def lv(self) -> U64Vector2: ...
    @property
    def uuu(self) -> U64Vector3: ...
    @property
    def uuv(self) -> U64Vector3: ...
    @property
    def uuo(self) -> U64Vector3: ...
    @property
    def uul(self) -> U64Vector3: ...
    @property
    def uvu(self) -> U64Vector3: ...
    @property
    def uvv(self) -> U64Vector3: ...
    @property
    def uvo(self) -> U64Vector3: ...
    @property
    def uvl(self) -> U64Vector3: ...
    @property
    def uou(self) -> U64Vector3: ...
    @property
    def uov(self) -> U64Vector3: ...
    @property
    def uoo(self) -> U64Vector3: ...
    @property
    def uol(self) -> U64Vector3: ...
    @property
    def ulu(self) -> U64Vector3: ...
    @property
    def ulv(self) -> U64Vector3: ...
    @property
    def ulo(self) -> U64Vector3: ...
    @property
    def ull(self) -> U64Vector3: ...
    @property
    def vuu(self) -> U64Vector3: ...
    @property
    def vuv(self) -> U64Vector3: ...
    @property
    def vuo(self) -> U64Vector3: ...
    @property
    def vul(self) -> U64Vector3: ...
    @property
    def vvu(self) -> U64Vector3: ...
    @property
    def vvv(self) -> U64Vector3: ...
    @property
    def vvo(self) -> U64Vector3: ...
    @property
    def vvl(self) -> U64Vector3: ...
    @property
    def vou(self) -> U64Vector3: ...
    @property
    def vov(self) -> U64Vector3: ...
    @property
    def voo(self) -> U64Vector3: ...
    @property
    def vol(self) -> U64Vector3: ...
    @property
    def vlu(self) -> U64Vector3: ...
    @property
    def vlv(self) -> U64Vector3: ...
    @property
    def vlo(self) -> U64Vector3: ...
    @property
    def vll(self) -> U64Vector3: ...
    @property
    def ouu(self) -> U64Vector3: ...
    @property
    def ouv(self) -> U64Vector3: ...
    @property
    def ouo(self) -> U64Vector3: ...
    @property
    def oul(self) -> U64Vector3: ...
    @property
    def ovu(self) -> U64Vector3: ...
    @property
    def ovv(self) -> U64Vector3: ...
    @property
    def ovo(self) -> U64Vector3: ...
    @property
    def ovl(self) -> U64Vector3: ...
    @property
    def oou(self) -> U64Vector3: ...
    @property
    def oov(self) -> U64Vector3: ...
    @property
    def olu(self) -> U64Vector3: ...
    @property
    def olv(self) -> U64Vector3: ...
    @property
    def luu(self) -> U64Vector3: ...
    @property
    def luv(self) -> U64Vector3: ...
    @property
    def luo(self) -> U64Vector3: ...
    @property
    def lul(self) -> U64Vector3: ...
    @property
    def lvu(self) -> U64Vector3: ...
    @property
    def lvv(self) -> U64Vector3: ...
    @property
    def lvo(self) -> U64Vector3: ...
    @property
    def lvl(self) -> U64Vector3: ...
    @property
    def lou(self) -> U64Vector3: ...
    @property
    def lov(self) -> U64Vector3: ...
    @property
    def llu(self) -> U64Vector3: ...
    @property
    def llv(self) -> U64Vector3: ...
    @property
    def uuuu(self) -> U64Vector4: ...
    @property
    def uuuv(self) -> U64Vector4: ...
    @property
    def uuuo(self) -> U64Vector4: ...
    @property
    def uuul(self) -> U64Vector4: ...
    @property
    def uuvu(self) -> U64Vector4: ...
    @property
    def uuvv(self) -> U64Vector4: ...
    @property
    def uuvo(self) -> U64Vector4: ...
    @property
    def uuvl(self) -> U64Vector4: ...
    @property
    def uuou(self) -> U64Vector4: ...
    @property
    def uuov(self) -> U64Vector4: ...
    @property
    def uuoo(self) -> U64Vector4: ...
    @property
    def uuol(self) -> U64Vector4: ...
    @property
    def uulu(self) -> U64Vector4: ...
    @property
    def uulv(self) -> U64Vector4: ...
    @property
    def uulo(self) -> U64Vector4: ...
    @property
    def uull(self) -> U64Vector4: ...
    @property
    def uvuu(self) -> U64Vector4: ...
    @property
    def uvuv(self) -> U64Vector4: ...
    @property
    def uvuo(self) -> U64Vector4: ...
    @property
    def uvul(self) -> U64Vector4: ...
    @property
    def uvvu(self) -> U64Vector4: ...
    @property
    def uvvv(self) -> U64Vector4: ...
    @property
    def uvvo(self) -> U64Vector4: ...
    @property
    def uvvl(self) -> U64Vector4: ...
    @property
    def uvou(self) -> U64Vector4: ...
    @property
    def uvov(self) -> U64Vector4: ...
    @property
    def uvoo(self) -> U64Vector4: ...
    @property
    def uvol(self) -> U64Vector4: ...
    @property
    def uvlu(self) -> U64Vector4: ...
    @property
    def uvlv(self) -> U64Vector4: ...
    @property
    def uvlo(self) -> U64Vector4: ...
    @property
    def uvll(self) -> U64Vector4: ...
    @property
    def uouu(self) -> U64Vector4: ...
    @property
    def uouv(self) -> U64Vector4: ...
    @property
    def uouo(self) -> U64Vector4: ...
    @property
    def uoul(self) -> U64Vector4: ...
    @property
    def uovu(self) -> U64Vector4: ...
    @property
    def uovv(self) -> U64Vector4: ...
    @property
    def uovo(self) -> U64Vector4: ...
    @property
    def uovl(self) -> U64Vector4: ...
    @property
    def uoou(self) -> U64Vector4: ...
    @property
    def uoov(self) -> U64Vector4: ...
    @property
    def uooo(self) -> U64Vector4: ...
    @property
    def uool(self) -> U64Vector4: ...
    @property
    def uolu(self) -> U64Vector4: ...
    @property
    def uolv(self) -> U64Vector4: ...
    @property
    def uolo(self) -> U64Vector4: ...
    @property
    def uoll(self) -> U64Vector4: ...
    @property
    def uluu(self) -> U64Vector4: ...
    @property
    def uluv(self) -> U64Vector4: ...
    @property
    def uluo(self) -> U64Vector4: ...
    @property
    def ulul(self) -> U64Vector4: ...
    @property
    def ulvu(self) -> U64Vector4: ...
    @property
    def ulvv(self) -> U64Vector4: ...
    @property
    def ulvo(self) -> U64Vector4: ...
    @property
    def ulvl(self) -> U64Vector4: ...
    @property
    def ulou(self) -> U64Vector4: ...
    @property
    def ulov(self) -> U64Vector4: ...
    @property
    def uloo(self) -> U64Vector4: ...
    @property
    def ulol(self) -> U64Vector4: ...
    @property
    def ullu(self) -> U64Vector4: ...
    @property
    def ullv(self) -> U64Vector4: ...
    @property
    def ullo(self) -> U64Vector4: ...
    @property
    def ulll(self) -> U64Vector4: ...
    @property
    def vuuu(self) -> U64Vector4: ...
    @property
    def vuuv(self) -> U64Vector4: ...
    @property
    def vuuo(self) -> U64Vector4: ...
    @property
    def vuul(self) -> U64Vector4: ...
    @property
    def vuvu(self) -> U64Vector4: ...
    @property
    def vuvv(self) -> U64Vector4: ...
    @property
    def vuvo(self) -> U64Vector4: ...
    @property
    def vuvl(self) -> U64Vector4: ...
    @property
    def vuou(self) -> U64Vector4: ...
    @property
    def vuov(self) -> U64Vector4: ...
    @property
    def vuoo(self) -> U64Vector4: ...
    @property
    def vuol(self) -> U64Vector4: ...
    @property
    def vulu(self) -> U64Vector4: ...
    @property
    def vulv(self) -> U64Vector4: ...
    @property
    def vulo(self) -> U64Vector4: ...
    @property
    def vull(self) -> U64Vector4: ...
    @property
    def vvuu(self) -> U64Vector4: ...
    @property
    def vvuv(self) -> U64Vector4: ...
    @property
    def vvuo(self) -> U64Vector4: ...
    @property
    def vvul(self) -> U64Vector4: ...
    @property
    def vvvu(self) -> U64Vector4: ...
    @property
    def vvvv(self) -> U64Vector4: ...
    @property
    def vvvo(self) -> U64Vector4: ...
    @property
    def vvvl(self) -> U64Vector4: ...
    @property
    def vvou(self) -> U64Vector4: ...
    @property
    def vvov(self) -> U64Vector4: ...
    @property
    def vvoo(self) -> U64Vector4: ...
    @property
    def vvol(self) -> U64Vector4: ...
    @property
    def vvlu(self) -> U64Vector4: ...
    @property
    def vvlv(self) -> U64Vector4: ...
    @property
    def vvlo(self) -> U64Vector4: ...
    @property
    def vvll(self) -> U64Vector4: ...
    @property
    def vouu(self) -> U64Vector4: ...
    @property
    def vouv(self) -> U64Vector4: ...
    @property
    def vouo(self) -> U64Vector4: ...
    @property
    def voul(self) -> U64Vector4: ...
    @property
    def vovu(self) -> U64Vector4: ...
    @property
    def vovv(self) -> U64Vector4: ...
    @property
    def vovo(self) -> U64Vector4: ...
    @property
    def vovl(self) -> U64Vector4: ...
    @property
    def voou(self) -> U64Vector4: ...
    @property
    def voov(self) -> U64Vector4: ...
    @property
    def vooo(self) -> U64Vector4: ...
    @property
    def vool(self) -> U64Vector4: ...
    @property
    def volu(self) -> U64Vector4: ...
    @property
    def volv(self) -> U64Vector4: ...
    @property
    def volo(self) -> U64Vector4: ...
    @property
    def voll(self) -> U64Vector4: ...
    @property
    def vluu(self) -> U64Vector4: ...
    @property
    def vluv(self) -> U64Vector4: ...
    @property
    def vluo(self) -> U64Vector4: ...
    @property
    def vlul(self) -> U64Vector4: ...
    @property
    def vlvu(self) -> U64Vector4: ...
    @property
    def vlvv(self) -> U64Vector4: ...
    @property
    def vlvo(self) -> U64Vector4: ...
    @property
    def vlvl(self) -> U64Vector4: ...
    @property
    def vlou(self) -> U64Vector4: ...
    @property
    def vlov(self) -> U64Vector4: ...
    @property
    def vloo(self) -> U64Vector4: ...
    @property
    def vlol(self) -> U64Vector4: ...
    @property
    def vllu(self) -> U64Vector4: ...
    @property
    def vllv(self) -> U64Vector4: ...
    @property
    def vllo(self) -> U64Vector4: ...
    @property
    def vlll(self) -> U64Vector4: ...
    @property
    def ouuu(self) -> U64Vector4: ...
    @property
    def ouuv(self) -> U64Vector4: ...
    @property
    def ouuo(self) -> U64Vector4: ...
    @property
    def ouul(self) -> U64Vector4: ...
    @property
    def ouvu(self) -> U64Vector4: ...
    @property
    def ouvv(self) -> U64Vector4: ...
    @property
    def ouvo(self) -> U64Vector4: ...
    @property
    def ouvl(self) -> U64Vector4: ...
    @property
    def ouou(self) -> U64Vector4: ...
    @property
    def ouov(self) -> U64Vector4: ...
    @property
    def ouoo(self) -> U64Vector4: ...
    @property
    def ouol(self) -> U64Vector4: ...
    @property
    def oulu(self) -> U64Vector4: ...
    @property
    def oulv(self) -> U64Vector4: ...
    @property
    def oulo(self) -> U64Vector4: ...
    @property
    def oull(self) -> U64Vector4: ...
    @property
    def ovuu(self) -> U64Vector4: ...
    @property
    def ovuv(self) -> U64Vector4: ...
    @property
    def ovuo(self) -> U64Vector4: ...
    @property
    def ovul(self) -> U64Vector4: ...
    @property
    def ovvu(self) -> U64Vector4: ...
    @property
    def ovvv(self) -> U64Vector4: ...
    @property
    def ovvo(self) -> U64Vector4: ...
    @property
    def ovvl(self) -> U64Vector4: ...
    @property
    def ovou(self) -> U64Vector4: ...
    @property
    def ovov(self) -> U64Vector4: ...
    @property
    def ovoo(self) -> U64Vector4: ...
    @property
    def ovol(self) -> U64Vector4: ...
    @property
    def ovlu(self) -> U64Vector4: ...
    @property
    def ovlv(self) -> U64Vector4: ...
    @property
    def ovlo(self) -> U64Vector4: ...
    @property
    def ovll(self) -> U64Vector4: ...
    @property
    def oouu(self) -> U64Vector4: ...
    @property
    def oouv(self) -> U64Vector4: ...
    @property
    def oouo(self) -> U64Vector4: ...
    @property
    def ooul(self) -> U64Vector4: ...
    @property
    def oovu(self) -> U64Vector4: ...
    @property
    def oovv(self) -> U64Vector4: ...
    @property
    def oovo(self) -> U64Vector4: ...
    @property
    def oovl(self) -> U64Vector4: ...
    @property
    def ooou(self) -> U64Vector4: ...
    @property
    def ooov(self) -> U64Vector4: ...
    @property
    def oolu(self) -> U64Vector4: ...
    @property
    def oolv(self) -> U64Vector4: ...
    @property
    def oluu(self) -> U64Vector4: ...
    @property
    def oluv(self) -> U64Vector4: ...
    @property
    def oluo(self) -> U64Vector4: ...
    @property
    def olul(self) -> U64Vector4: ...
    @property
    def olvu(self) -> U64Vector4: ...
    @property
    def olvv(self) -> U64Vector4: ...
    @property
    def olvo(self) -> U64Vector4: ...
    @property
    def olvl(self) -> U64Vector4: ...
    @property
    def olou(self) -> U64Vector4: ...
    @property
    def olov(self) -> U64Vector4: ...
    @property
    def ollu(self) -> U64Vector4: ...
    @property
    def ollv(self) -> U64Vector4: ...
    @property
    def luuu(self) -> U64Vector4: ...
    @property
    def luuv(self) -> U64Vector4: ...
    @property
    def luuo(self) -> U64Vector4: ...
    @property
    def luul(self) -> U64Vector4: ...
    @property
    def luvu(self) -> U64Vector4: ...
    @property
    def luvv(self) -> U64Vector4: ...
    @property
    def luvo(self) -> U64Vector4: ...
    @property
    def luvl(self) -> U64Vector4: ...
    @property
    def luou(self) -> U64Vector4: ...
    @property
    def luov(self) -> U64Vector4: ...
    @property
    def luoo(self) -> U64Vector4: ...
    @property
    def luol(self) -> U64Vector4: ...
    @property
    def lulu(self) -> U64Vector4: ...
    @property
    def lulv(self) -> U64Vector4: ...
    @property
    def lulo(self) -> U64Vector4: ...
    @property
    def lull(self) -> U64Vector4: ...
    @property
    def lvuu(self) -> U64Vector4: ...
    @property
    def lvuv(self) -> U64Vector4: ...
    @property
    def lvuo(self) -> U64Vector4: ...
    @property
    def lvul(self) -> U64Vector4: ...
    @property
    def lvvu(self) -> U64Vector4: ...
    @property
    def lvvv(self) -> U64Vector4: ...
    @property
    def lvvo(self) -> U64Vector4: ...
    @property
    def lvvl(self) -> U64Vector4: ...
    @property
    def lvou(self) -> U64Vector4: ...
    @property
    def lvov(self) -> U64Vector4: ...
    @property
    def lvoo(self) -> U64Vector4: ...
    @property
    def lvol(self) -> U64Vector4: ...
    @property
    def lvlu(self) -> U64Vector4: ...
    @property
    def lvlv(self) -> U64Vector4: ...
    @property
    def lvlo(self) -> U64Vector4: ...
    @property
    def lvll(self) -> U64Vector4: ...
    @property
    def louu(self) -> U64Vector4: ...
    @property
    def louv(self) -> U64Vector4: ...
    @property
    def louo(self) -> U64Vector4: ...
    @property
    def loul(self) -> U64Vector4: ...
    @property
    def lovu(self) -> U64Vector4: ...
    @property
    def lovv(self) -> U64Vector4: ...
    @property
    def lovo(self) -> U64Vector4: ...
    @property
    def lovl(self) -> U64Vector4: ...
    @property
    def loou(self) -> U64Vector4: ...
    @property
    def loov(self) -> U64Vector4: ...
    @property
    def lolu(self) -> U64Vector4: ...
    @property
    def lolv(self) -> U64Vector4: ...
    @property
    def lluu(self) -> U64Vector4: ...
    @property
    def lluv(self) -> U64Vector4: ...
    @property
    def lluo(self) -> U64Vector4: ...
    @property
    def llul(self) -> U64Vector4: ...
    @property
    def llvu(self) -> U64Vector4: ...
    @property
    def llvv(self) -> U64Vector4: ...
    @property
    def llvo(self) -> U64Vector4: ...
    @property
    def llvl(self) -> U64Vector4: ...
    @property
    def llou(self) -> U64Vector4: ...
    @property
    def llov(self) -> U64Vector4: ...
    @property
    def lllu(self) -> U64Vector4: ...
    @property
    def lllv(self) -> U64Vector4: ...
    @property
    def oo(self) -> U64Vector2: ...
    @property
    def ol(self) -> U64Vector2: ...
    @property
    def lo(self) -> U64Vector2: ...
    @property
    def ll(self) -> U64Vector2: ...
    @property
    def ooo(self) -> U64Vector3: ...
    @property
    def ool(self) -> U64Vector3: ...
    @property
    def olo(self) -> U64Vector3: ...
    @property
    def oll(self) -> U64Vector3: ...
    @property
    def loo(self) -> U64Vector3: ...
    @property
    def lol(self) -> U64Vector3: ...
    @property
    def llo(self) -> U64Vector3: ...
    @property
    def lll(self) -> U64Vector3: ...
    @property
    def oooo(self) -> U64Vector4: ...
    @property
    def oool(self) -> U64Vector4: ...
    @property
    def oolo(self) -> U64Vector4: ...
    @property
    def ooll(self) -> U64Vector4: ...
    @property
    def oloo(self) -> U64Vector4: ...
    @property
    def olol(self) -> U64Vector4: ...
    @property
    def ollo(self) -> U64Vector4: ...
    @property
    def olll(self) -> U64Vector4: ...
    @property
    def looo(self) -> U64Vector4: ...
    @property
    def lool(self) -> U64Vector4: ...
    @property
    def lolo(self) -> U64Vector4: ...
    @property
    def loll(self) -> U64Vector4: ...
    @property
    def lloo(self) -> U64Vector4: ...
    @property
    def llol(self) -> U64Vector4: ...
    @property
    def lllo(self) -> U64Vector4: ...
    @property
    def llll(self) -> U64Vector4: ...
    def min(self, n: Number, /) -> U64Vector4: ...
    def max(self, n: Number, /) -> U64Vector4: ...
    def clamp(self, min: Number, max: Number, /) -> U64Vector4: ...
    @classmethod
    def get_limits(cls) -> tuple[int, int]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> type[U64Vector4Array]: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector4: ...

@final
class U64Vector4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: U64Vector4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[U64Vector4]: ...
    @overload
    def __getitem__(self, index: int) -> U64Vector4: ...
    @overload
    def __getitem__(self, index: slice) -> U64Vector4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer, /) -> U64Vector4Array: ...
    @classmethod
    def get_component_type(cls) -> type[U64Vector4]: ...

@final
class DMatrix2x2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector2, _1: DVector2, /): ...
    @overload
    def __init__(self, _0: Number, _1: Number, _2: Number, _3: Number, /): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector2]: ...
    def __getitem__(self, index: int) -> DVector2: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix2x2) -> DMatrix2x2: ...
    @overload
    def __add__(self, other: Number) -> DMatrix2x2: ...
    @overload
    def __radd__(self, other: DMatrix2x2) -> DMatrix2x2: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix2x2: ...
    @overload
    def __sub__(self, other: DMatrix2x2) -> DMatrix2x2: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix2x2: ...
    @overload
    def __rsub__(self, other: DMatrix2x2) -> DMatrix2x2: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix2x2: ...
    def __mul__(self, other: Number) -> DMatrix2x2: ...
    def __rmul__(self, other: Number) -> DMatrix2x2: ...
    @overload
    def __matmul__(self, other: DMatrix2x2) -> DMatrix2x2: ...
    @overload
    def __matmul__(self, other: DMatrix3x2) -> DMatrix3x2: ...
    @overload
    def __matmul__(self, other: DMatrix4x2) -> DMatrix4x2: ...
    @overload
    def __matmul__(self, other: DVector2) -> DVector2: ...
    def __rmatmul__(self, other: DVector2) -> DVector2: ...
    @overload
    def __truediv__(self, other: Number) -> DMatrix2x2: ...
    @overload
    def __truediv__(self, other: DMatrix2x2) -> DMatrix2x2: ...
    @overload
    def __truediv__(self, other: DVector2) -> DVector2: ...
    @overload
    def __rtruediv__(self, other: DVector2) -> DVector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> DMatrix2x2: ...
    def __neg__(self) -> DMatrix2x2: ...
    def __bool__(self) -> _bool: ...
    def inverse(self) -> DMatrix2x2: ...
    def transpose(self) -> DMatrix2x2: ...
    def get_row(self, index: int, /) -> DVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix2x2Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix2x2: ...

@final
class DMatrix2x2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix2x2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix2x2]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix2x2: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix2x2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix2x2Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix2x2]: ...

@final
class FMatrix2x2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector2, _1: FVector2, /): ...
    @overload
    def __init__(self, _0: Number, _1: Number, _2: Number, _3: Number, /): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector2]: ...
    def __getitem__(self, index: int) -> FVector2: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix2x2) -> FMatrix2x2: ...
    @overload
    def __add__(self, other: Number) -> FMatrix2x2: ...
    @overload
    def __radd__(self, other: FMatrix2x2) -> FMatrix2x2: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix2x2: ...
    @overload
    def __sub__(self, other: FMatrix2x2) -> FMatrix2x2: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix2x2: ...
    @overload
    def __rsub__(self, other: FMatrix2x2) -> FMatrix2x2: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix2x2: ...
    def __mul__(self, other: Number) -> FMatrix2x2: ...
    def __rmul__(self, other: Number) -> FMatrix2x2: ...
    @overload
    def __matmul__(self, other: FMatrix2x2) -> FMatrix2x2: ...
    @overload
    def __matmul__(self, other: FMatrix3x2) -> FMatrix3x2: ...
    @overload
    def __matmul__(self, other: FMatrix4x2) -> FMatrix4x2: ...
    @overload
    def __matmul__(self, other: FVector2) -> FVector2: ...
    def __rmatmul__(self, other: FVector2) -> FVector2: ...
    @overload
    def __truediv__(self, other: Number) -> FMatrix2x2: ...
    @overload
    def __truediv__(self, other: FMatrix2x2) -> FMatrix2x2: ...
    @overload
    def __truediv__(self, other: FVector2) -> FVector2: ...
    @overload
    def __rtruediv__(self, other: FVector2) -> FVector2: ...
    @overload
    def __rtruediv__(self, other: Number) -> FMatrix2x2: ...
    def __neg__(self) -> FMatrix2x2: ...
    def __bool__(self) -> _bool: ...
    def inverse(self) -> FMatrix2x2: ...
    def transpose(self) -> FMatrix2x2: ...
    def get_row(self, index: int, /) -> FVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix2x2Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix2x2: ...

@final
class FMatrix2x2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix2x2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix2x2]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix2x2: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix2x2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix2x2Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix2x2]: ...

@final
class DMatrix2x3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector3, _1: DVector3, /): ...
    @overload
    def __init__(
        self, _0: Number, _1: Number, _2: Number, _3: Number, _4: Number, _5: Number, /
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector3]: ...
    def __getitem__(self, index: int) -> DVector3: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix2x3) -> DMatrix2x3: ...
    @overload
    def __add__(self, other: Number) -> DMatrix2x3: ...
    @overload
    def __radd__(self, other: DMatrix2x3) -> DMatrix2x3: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix2x3: ...
    @overload
    def __sub__(self, other: DMatrix2x3) -> DMatrix2x3: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix2x3: ...
    @overload
    def __rsub__(self, other: DMatrix2x3) -> DMatrix2x3: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix2x3: ...
    def __mul__(self, other: Number) -> DMatrix2x3: ...
    def __rmul__(self, other: Number) -> DMatrix2x3: ...
    @overload
    def __matmul__(self, other: DMatrix2x2) -> DMatrix2x3: ...
    @overload
    def __matmul__(self, other: DMatrix3x2) -> DMatrix3x3: ...
    @overload
    def __matmul__(self, other: DMatrix4x2) -> DMatrix4x3: ...
    @overload
    def __matmul__(self, other: DVector2) -> DVector3: ...
    def __rmatmul__(self, other: DVector3) -> DVector2: ...
    def __truediv__(self, other: Number) -> DMatrix2x3: ...
    def __rtruediv__(self, other: Number) -> DMatrix2x3: ...
    def __neg__(self) -> DMatrix2x3: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> DMatrix3x2: ...
    def get_row(self, index: int, /) -> DVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix2x3Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix2x3: ...

@final
class DMatrix2x3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix2x3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix2x3]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix2x3: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix2x3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix2x3Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix2x3]: ...

@final
class FMatrix2x3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector3, _1: FVector3, /): ...
    @overload
    def __init__(
        self, _0: Number, _1: Number, _2: Number, _3: Number, _4: Number, _5: Number, /
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector3]: ...
    def __getitem__(self, index: int) -> FVector3: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix2x3) -> FMatrix2x3: ...
    @overload
    def __add__(self, other: Number) -> FMatrix2x3: ...
    @overload
    def __radd__(self, other: FMatrix2x3) -> FMatrix2x3: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix2x3: ...
    @overload
    def __sub__(self, other: FMatrix2x3) -> FMatrix2x3: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix2x3: ...
    @overload
    def __rsub__(self, other: FMatrix2x3) -> FMatrix2x3: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix2x3: ...
    def __mul__(self, other: Number) -> FMatrix2x3: ...
    def __rmul__(self, other: Number) -> FMatrix2x3: ...
    @overload
    def __matmul__(self, other: FMatrix2x2) -> FMatrix2x3: ...
    @overload
    def __matmul__(self, other: FMatrix3x2) -> FMatrix3x3: ...
    @overload
    def __matmul__(self, other: FMatrix4x2) -> FMatrix4x3: ...
    @overload
    def __matmul__(self, other: FVector2) -> FVector3: ...
    def __rmatmul__(self, other: FVector3) -> FVector2: ...
    def __truediv__(self, other: Number) -> FMatrix2x3: ...
    def __rtruediv__(self, other: Number) -> FMatrix2x3: ...
    def __neg__(self) -> FMatrix2x3: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> FMatrix3x2: ...
    def get_row(self, index: int, /) -> FVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix2x3Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix2x3: ...

@final
class FMatrix2x3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix2x3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix2x3]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix2x3: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix2x3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix2x3Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix2x3]: ...

@final
class DMatrix2x4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector4, _1: DVector4, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector4]: ...
    def __getitem__(self, index: int) -> DVector4: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix2x4) -> DMatrix2x4: ...
    @overload
    def __add__(self, other: Number) -> DMatrix2x4: ...
    @overload
    def __radd__(self, other: DMatrix2x4) -> DMatrix2x4: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix2x4: ...
    @overload
    def __sub__(self, other: DMatrix2x4) -> DMatrix2x4: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix2x4: ...
    @overload
    def __rsub__(self, other: DMatrix2x4) -> DMatrix2x4: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix2x4: ...
    def __mul__(self, other: Number) -> DMatrix2x4: ...
    def __rmul__(self, other: Number) -> DMatrix2x4: ...
    @overload
    def __matmul__(self, other: DMatrix2x2) -> DMatrix2x4: ...
    @overload
    def __matmul__(self, other: DMatrix3x2) -> DMatrix3x4: ...
    @overload
    def __matmul__(self, other: DMatrix4x2) -> DMatrix4x4: ...
    @overload
    def __matmul__(self, other: DVector2) -> DVector4: ...
    def __rmatmul__(self, other: DVector4) -> DVector2: ...
    def __truediv__(self, other: Number) -> DMatrix2x4: ...
    def __rtruediv__(self, other: Number) -> DMatrix2x4: ...
    def __neg__(self) -> DMatrix2x4: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> DMatrix4x2: ...
    def get_row(self, index: int, /) -> DVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix2x4Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix2x4: ...

@final
class DMatrix2x4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix2x4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix2x4]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix2x4: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix2x4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix2x4Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix2x4]: ...

@final
class FMatrix2x4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector4, _1: FVector4, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector4]: ...
    def __getitem__(self, index: int) -> FVector4: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix2x4) -> FMatrix2x4: ...
    @overload
    def __add__(self, other: Number) -> FMatrix2x4: ...
    @overload
    def __radd__(self, other: FMatrix2x4) -> FMatrix2x4: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix2x4: ...
    @overload
    def __sub__(self, other: FMatrix2x4) -> FMatrix2x4: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix2x4: ...
    @overload
    def __rsub__(self, other: FMatrix2x4) -> FMatrix2x4: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix2x4: ...
    def __mul__(self, other: Number) -> FMatrix2x4: ...
    def __rmul__(self, other: Number) -> FMatrix2x4: ...
    @overload
    def __matmul__(self, other: FMatrix2x2) -> FMatrix2x4: ...
    @overload
    def __matmul__(self, other: FMatrix3x2) -> FMatrix3x4: ...
    @overload
    def __matmul__(self, other: FMatrix4x2) -> FMatrix4x4: ...
    @overload
    def __matmul__(self, other: FVector2) -> FVector4: ...
    def __rmatmul__(self, other: FVector4) -> FVector2: ...
    def __truediv__(self, other: Number) -> FMatrix2x4: ...
    def __rtruediv__(self, other: Number) -> FMatrix2x4: ...
    def __neg__(self) -> FMatrix2x4: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> FMatrix4x2: ...
    def get_row(self, index: int, /) -> FVector2: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix2x4Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix2x4: ...

@final
class FMatrix2x4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix2x4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix2x4]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix2x4: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix2x4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix2x4Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix2x4]: ...

@final
class DMatrix3x2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector2, _1: DVector2, _2: DVector2, /): ...
    @overload
    def __init__(
        self, _0: Number, _1: Number, _2: Number, _3: Number, _4: Number, _5: Number, /
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector2]: ...
    def __getitem__(self, index: int) -> DVector2: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix3x2) -> DMatrix3x2: ...
    @overload
    def __add__(self, other: Number) -> DMatrix3x2: ...
    @overload
    def __radd__(self, other: DMatrix3x2) -> DMatrix3x2: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix3x2: ...
    @overload
    def __sub__(self, other: DMatrix3x2) -> DMatrix3x2: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix3x2: ...
    @overload
    def __rsub__(self, other: DMatrix3x2) -> DMatrix3x2: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix3x2: ...
    def __mul__(self, other: Number) -> DMatrix3x2: ...
    def __rmul__(self, other: Number) -> DMatrix3x2: ...
    @overload
    def __matmul__(self, other: DMatrix2x3) -> DMatrix2x2: ...
    @overload
    def __matmul__(self, other: DMatrix3x3) -> DMatrix3x2: ...
    @overload
    def __matmul__(self, other: DMatrix4x3) -> DMatrix4x2: ...
    @overload
    def __matmul__(self, other: DVector3) -> DVector2: ...
    def __rmatmul__(self, other: DVector2) -> DVector3: ...
    def __truediv__(self, other: Number) -> DMatrix3x2: ...
    def __rtruediv__(self, other: Number) -> DMatrix3x2: ...
    def __neg__(self) -> DMatrix3x2: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> DMatrix2x3: ...
    def get_row(self, index: int, /) -> DVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix3x2Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix3x2: ...

@final
class DMatrix3x2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix3x2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix3x2]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix3x2: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix3x2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix3x2Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix3x2]: ...

@final
class FMatrix3x2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector2, _1: FVector2, _2: FVector2, /): ...
    @overload
    def __init__(
        self, _0: Number, _1: Number, _2: Number, _3: Number, _4: Number, _5: Number, /
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector2]: ...
    def __getitem__(self, index: int) -> FVector2: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix3x2) -> FMatrix3x2: ...
    @overload
    def __add__(self, other: Number) -> FMatrix3x2: ...
    @overload
    def __radd__(self, other: FMatrix3x2) -> FMatrix3x2: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix3x2: ...
    @overload
    def __sub__(self, other: FMatrix3x2) -> FMatrix3x2: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix3x2: ...
    @overload
    def __rsub__(self, other: FMatrix3x2) -> FMatrix3x2: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix3x2: ...
    def __mul__(self, other: Number) -> FMatrix3x2: ...
    def __rmul__(self, other: Number) -> FMatrix3x2: ...
    @overload
    def __matmul__(self, other: FMatrix2x3) -> FMatrix2x2: ...
    @overload
    def __matmul__(self, other: FMatrix3x3) -> FMatrix3x2: ...
    @overload
    def __matmul__(self, other: FMatrix4x3) -> FMatrix4x2: ...
    @overload
    def __matmul__(self, other: FVector3) -> FVector2: ...
    def __rmatmul__(self, other: FVector2) -> FVector3: ...
    def __truediv__(self, other: Number) -> FMatrix3x2: ...
    def __rtruediv__(self, other: Number) -> FMatrix3x2: ...
    def __neg__(self) -> FMatrix3x2: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> FMatrix2x3: ...
    def get_row(self, index: int, /) -> FVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix3x2Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix3x2: ...

@final
class FMatrix3x2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix3x2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix3x2]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix3x2: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix3x2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix3x2Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix3x2]: ...

@final
class DMatrix3x3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector3, _1: DVector3, _2: DVector3, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector3]: ...
    def __getitem__(self, index: int) -> DVector3: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix3x3) -> DMatrix3x3: ...
    @overload
    def __add__(self, other: Number) -> DMatrix3x3: ...
    @overload
    def __radd__(self, other: DMatrix3x3) -> DMatrix3x3: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix3x3: ...
    @overload
    def __sub__(self, other: DMatrix3x3) -> DMatrix3x3: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix3x3: ...
    @overload
    def __rsub__(self, other: DMatrix3x3) -> DMatrix3x3: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix3x3: ...
    def __mul__(self, other: Number) -> DMatrix3x3: ...
    def __rmul__(self, other: Number) -> DMatrix3x3: ...
    @overload
    def __matmul__(self, other: DMatrix2x3) -> DMatrix2x3: ...
    @overload
    def __matmul__(self, other: DMatrix3x3) -> DMatrix3x3: ...
    @overload
    def __matmul__(self, other: DMatrix4x3) -> DMatrix4x3: ...
    @overload
    def __matmul__(self, other: DVector3) -> DVector3: ...
    def __rmatmul__(self, other: DVector3) -> DVector3: ...
    @overload
    def __truediv__(self, other: Number) -> DMatrix3x3: ...
    @overload
    def __truediv__(self, other: DMatrix3x3) -> DMatrix3x3: ...
    @overload
    def __truediv__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rtruediv__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> DMatrix3x3: ...
    def __neg__(self) -> DMatrix3x3: ...
    def __bool__(self) -> _bool: ...
    def inverse(self) -> DMatrix3x3: ...
    def transpose(self) -> DMatrix3x3: ...
    def to_quaternion(self) -> DQuaternion: ...
    def get_row(self, index: int, /) -> DVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix3x3Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix3x3: ...

@final
class DMatrix3x3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix3x3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix3x3]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix3x3: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix3x3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix3x3Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix3x3]: ...

@final
class FMatrix3x3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector3, _1: FVector3, _2: FVector3, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector3]: ...
    def __getitem__(self, index: int) -> FVector3: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix3x3) -> FMatrix3x3: ...
    @overload
    def __add__(self, other: Number) -> FMatrix3x3: ...
    @overload
    def __radd__(self, other: FMatrix3x3) -> FMatrix3x3: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix3x3: ...
    @overload
    def __sub__(self, other: FMatrix3x3) -> FMatrix3x3: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix3x3: ...
    @overload
    def __rsub__(self, other: FMatrix3x3) -> FMatrix3x3: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix3x3: ...
    def __mul__(self, other: Number) -> FMatrix3x3: ...
    def __rmul__(self, other: Number) -> FMatrix3x3: ...
    @overload
    def __matmul__(self, other: FMatrix2x3) -> FMatrix2x3: ...
    @overload
    def __matmul__(self, other: FMatrix3x3) -> FMatrix3x3: ...
    @overload
    def __matmul__(self, other: FMatrix4x3) -> FMatrix4x3: ...
    @overload
    def __matmul__(self, other: FVector3) -> FVector3: ...
    def __rmatmul__(self, other: FVector3) -> FVector3: ...
    @overload
    def __truediv__(self, other: Number) -> FMatrix3x3: ...
    @overload
    def __truediv__(self, other: FMatrix3x3) -> FMatrix3x3: ...
    @overload
    def __truediv__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rtruediv__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rtruediv__(self, other: Number) -> FMatrix3x3: ...
    def __neg__(self) -> FMatrix3x3: ...
    def __bool__(self) -> _bool: ...
    def inverse(self) -> FMatrix3x3: ...
    def transpose(self) -> FMatrix3x3: ...
    def to_quaternion(self) -> FQuaternion: ...
    def get_row(self, index: int, /) -> FVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix3x3Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix3x3: ...

@final
class FMatrix3x3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix3x3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix3x3]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix3x3: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix3x3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix3x3Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix3x3]: ...

@final
class DMatrix3x4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector4, _1: DVector4, _2: DVector4, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        _9: Number,
        _10: Number,
        _11: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector4]: ...
    def __getitem__(self, index: int) -> DVector4: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix3x4) -> DMatrix3x4: ...
    @overload
    def __add__(self, other: Number) -> DMatrix3x4: ...
    @overload
    def __radd__(self, other: DMatrix3x4) -> DMatrix3x4: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix3x4: ...
    @overload
    def __sub__(self, other: DMatrix3x4) -> DMatrix3x4: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix3x4: ...
    @overload
    def __rsub__(self, other: DMatrix3x4) -> DMatrix3x4: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix3x4: ...
    def __mul__(self, other: Number) -> DMatrix3x4: ...
    def __rmul__(self, other: Number) -> DMatrix3x4: ...
    @overload
    def __matmul__(self, other: DMatrix2x3) -> DMatrix2x4: ...
    @overload
    def __matmul__(self, other: DMatrix3x3) -> DMatrix3x4: ...
    @overload
    def __matmul__(self, other: DMatrix4x3) -> DMatrix4x4: ...
    @overload
    def __matmul__(self, other: DVector3) -> DVector4: ...
    def __rmatmul__(self, other: DVector4) -> DVector3: ...
    def __truediv__(self, other: Number) -> DMatrix3x4: ...
    def __rtruediv__(self, other: Number) -> DMatrix3x4: ...
    def __neg__(self) -> DMatrix3x4: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> DMatrix4x3: ...
    def get_row(self, index: int, /) -> DVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix3x4Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix3x4: ...

@final
class DMatrix3x4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix3x4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix3x4]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix3x4: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix3x4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix3x4Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix3x4]: ...

@final
class FMatrix3x4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector4, _1: FVector4, _2: FVector4, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        _9: Number,
        _10: Number,
        _11: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector4]: ...
    def __getitem__(self, index: int) -> FVector4: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix3x4) -> FMatrix3x4: ...
    @overload
    def __add__(self, other: Number) -> FMatrix3x4: ...
    @overload
    def __radd__(self, other: FMatrix3x4) -> FMatrix3x4: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix3x4: ...
    @overload
    def __sub__(self, other: FMatrix3x4) -> FMatrix3x4: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix3x4: ...
    @overload
    def __rsub__(self, other: FMatrix3x4) -> FMatrix3x4: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix3x4: ...
    def __mul__(self, other: Number) -> FMatrix3x4: ...
    def __rmul__(self, other: Number) -> FMatrix3x4: ...
    @overload
    def __matmul__(self, other: FMatrix2x3) -> FMatrix2x4: ...
    @overload
    def __matmul__(self, other: FMatrix3x3) -> FMatrix3x4: ...
    @overload
    def __matmul__(self, other: FMatrix4x3) -> FMatrix4x4: ...
    @overload
    def __matmul__(self, other: FVector3) -> FVector4: ...
    def __rmatmul__(self, other: FVector4) -> FVector3: ...
    def __truediv__(self, other: Number) -> FMatrix3x4: ...
    def __rtruediv__(self, other: Number) -> FMatrix3x4: ...
    def __neg__(self) -> FMatrix3x4: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> FMatrix4x3: ...
    def get_row(self, index: int, /) -> FVector3: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix3x4Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix3x4: ...

@final
class FMatrix3x4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix3x4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix3x4]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix3x4: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix3x4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix3x4Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix3x4]: ...

@final
class DMatrix4x2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector2, _1: DVector2, _2: DVector2, _3: DVector2, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector2]: ...
    def __getitem__(self, index: int) -> DVector2: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix4x2) -> DMatrix4x2: ...
    @overload
    def __add__(self, other: Number) -> DMatrix4x2: ...
    @overload
    def __radd__(self, other: DMatrix4x2) -> DMatrix4x2: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix4x2: ...
    @overload
    def __sub__(self, other: DMatrix4x2) -> DMatrix4x2: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix4x2: ...
    @overload
    def __rsub__(self, other: DMatrix4x2) -> DMatrix4x2: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix4x2: ...
    def __mul__(self, other: Number) -> DMatrix4x2: ...
    def __rmul__(self, other: Number) -> DMatrix4x2: ...
    @overload
    def __matmul__(self, other: DMatrix2x4) -> DMatrix2x2: ...
    @overload
    def __matmul__(self, other: DMatrix3x4) -> DMatrix3x2: ...
    @overload
    def __matmul__(self, other: DMatrix4x4) -> DMatrix4x2: ...
    @overload
    def __matmul__(self, other: DVector4) -> DVector2: ...
    def __rmatmul__(self, other: DVector2) -> DVector4: ...
    def __truediv__(self, other: Number) -> DMatrix4x2: ...
    def __rtruediv__(self, other: Number) -> DMatrix4x2: ...
    def __neg__(self) -> DMatrix4x2: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> DMatrix2x4: ...
    def get_row(self, index: int, /) -> DVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix4x2Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix4x2: ...

@final
class DMatrix4x2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix4x2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix4x2]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix4x2: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix4x2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix4x2Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix4x2]: ...

@final
class FMatrix4x2:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector2, _1: FVector2, _2: FVector2, _3: FVector2, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector2]: ...
    def __getitem__(self, index: int) -> FVector2: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix4x2) -> FMatrix4x2: ...
    @overload
    def __add__(self, other: Number) -> FMatrix4x2: ...
    @overload
    def __radd__(self, other: FMatrix4x2) -> FMatrix4x2: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix4x2: ...
    @overload
    def __sub__(self, other: FMatrix4x2) -> FMatrix4x2: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix4x2: ...
    @overload
    def __rsub__(self, other: FMatrix4x2) -> FMatrix4x2: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix4x2: ...
    def __mul__(self, other: Number) -> FMatrix4x2: ...
    def __rmul__(self, other: Number) -> FMatrix4x2: ...
    @overload
    def __matmul__(self, other: FMatrix2x4) -> FMatrix2x2: ...
    @overload
    def __matmul__(self, other: FMatrix3x4) -> FMatrix3x2: ...
    @overload
    def __matmul__(self, other: FMatrix4x4) -> FMatrix4x2: ...
    @overload
    def __matmul__(self, other: FVector4) -> FVector2: ...
    def __rmatmul__(self, other: FVector2) -> FVector4: ...
    def __truediv__(self, other: Number) -> FMatrix4x2: ...
    def __rtruediv__(self, other: Number) -> FMatrix4x2: ...
    def __neg__(self) -> FMatrix4x2: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> FMatrix2x4: ...
    def get_row(self, index: int, /) -> FVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix4x2Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix4x2: ...

@final
class FMatrix4x2Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix4x2): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix4x2]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix4x2: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix4x2Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix4x2Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix4x2]: ...

@final
class DMatrix4x3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector3, _1: DVector3, _2: DVector3, _3: DVector3, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        _9: Number,
        _10: Number,
        _11: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector3]: ...
    def __getitem__(self, index: int) -> DVector3: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix4x3) -> DMatrix4x3: ...
    @overload
    def __add__(self, other: Number) -> DMatrix4x3: ...
    @overload
    def __radd__(self, other: DMatrix4x3) -> DMatrix4x3: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix4x3: ...
    @overload
    def __sub__(self, other: DMatrix4x3) -> DMatrix4x3: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix4x3: ...
    @overload
    def __rsub__(self, other: DMatrix4x3) -> DMatrix4x3: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix4x3: ...
    def __mul__(self, other: Number) -> DMatrix4x3: ...
    def __rmul__(self, other: Number) -> DMatrix4x3: ...
    @overload
    def __matmul__(self, other: DMatrix2x4) -> DMatrix2x3: ...
    @overload
    def __matmul__(self, other: DMatrix3x4) -> DMatrix3x3: ...
    @overload
    def __matmul__(self, other: DMatrix4x4) -> DMatrix4x3: ...
    @overload
    def __matmul__(self, other: DVector4) -> DVector3: ...
    def __rmatmul__(self, other: DVector3) -> DVector4: ...
    def __truediv__(self, other: Number) -> DMatrix4x3: ...
    def __rtruediv__(self, other: Number) -> DMatrix4x3: ...
    def __neg__(self) -> DMatrix4x3: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> DMatrix3x4: ...
    def get_row(self, index: int, /) -> DVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix4x3Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix4x3: ...

@final
class DMatrix4x3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix4x3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix4x3]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix4x3: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix4x3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix4x3Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix4x3]: ...

@final
class FMatrix4x3:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector3, _1: FVector3, _2: FVector3, _3: FVector3, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        _9: Number,
        _10: Number,
        _11: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector3]: ...
    def __getitem__(self, index: int) -> FVector3: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix4x3) -> FMatrix4x3: ...
    @overload
    def __add__(self, other: Number) -> FMatrix4x3: ...
    @overload
    def __radd__(self, other: FMatrix4x3) -> FMatrix4x3: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix4x3: ...
    @overload
    def __sub__(self, other: FMatrix4x3) -> FMatrix4x3: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix4x3: ...
    @overload
    def __rsub__(self, other: FMatrix4x3) -> FMatrix4x3: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix4x3: ...
    def __mul__(self, other: Number) -> FMatrix4x3: ...
    def __rmul__(self, other: Number) -> FMatrix4x3: ...
    @overload
    def __matmul__(self, other: FMatrix2x4) -> FMatrix2x3: ...
    @overload
    def __matmul__(self, other: FMatrix3x4) -> FMatrix3x3: ...
    @overload
    def __matmul__(self, other: FMatrix4x4) -> FMatrix4x3: ...
    @overload
    def __matmul__(self, other: FVector4) -> FVector3: ...
    def __rmatmul__(self, other: FVector3) -> FVector4: ...
    def __truediv__(self, other: Number) -> FMatrix4x3: ...
    def __rtruediv__(self, other: Number) -> FMatrix4x3: ...
    def __neg__(self) -> FMatrix4x3: ...
    def __bool__(self) -> _bool: ...
    def transpose(self) -> FMatrix3x4: ...
    def get_row(self, index: int, /) -> FVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix4x3Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix4x3: ...

@final
class FMatrix4x3Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix4x3): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix4x3]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix4x3: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix4x3Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix4x3Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix4x3]: ...

@final
class DMatrix4x4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: DVector4, _1: DVector4, _2: DVector4, _3: DVector4, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        _9: Number,
        _10: Number,
        _11: Number,
        _12: Number,
        _13: Number,
        _14: Number,
        _15: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DVector4]: ...
    def __getitem__(self, index: int) -> DVector4: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: DMatrix4x4) -> DMatrix4x4: ...
    @overload
    def __add__(self, other: Number) -> DMatrix4x4: ...
    @overload
    def __radd__(self, other: DMatrix4x4) -> DMatrix4x4: ...
    @overload
    def __radd__(self, other: Number) -> DMatrix4x4: ...
    @overload
    def __sub__(self, other: DMatrix4x4) -> DMatrix4x4: ...
    @overload
    def __sub__(self, other: Number) -> DMatrix4x4: ...
    @overload
    def __rsub__(self, other: DMatrix4x4) -> DMatrix4x4: ...
    @overload
    def __rsub__(self, other: Number) -> DMatrix4x4: ...
    def __mul__(self, other: Number) -> DMatrix4x4: ...
    def __rmul__(self, other: Number) -> DMatrix4x4: ...
    @overload
    def __matmul__(self, other: DMatrix2x4) -> DMatrix2x4: ...
    @overload
    def __matmul__(self, other: DMatrix3x4) -> DMatrix3x4: ...
    @overload
    def __matmul__(self, other: DMatrix4x4) -> DMatrix4x4: ...
    @overload
    def __matmul__(self, other: DVector3) -> DVector3: ...
    @overload
    def __matmul__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rmatmul__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rmatmul__(self, other: DVector4) -> DVector4: ...
    @overload
    def __truediv__(self, other: Number) -> DMatrix4x4: ...
    @overload
    def __truediv__(self, other: DMatrix4x4) -> DMatrix4x4: ...
    @overload
    def __truediv__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rtruediv__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> DMatrix4x4: ...
    def __neg__(self) -> DMatrix4x4: ...
    def __bool__(self) -> _bool: ...
    def inverse(self) -> DMatrix4x4: ...
    def transpose(self) -> DMatrix4x4: ...
    def rotate(self, angle: float, axis: DVector3, /) -> DMatrix4x4: ...
    def scale(self, scaling: DVector3, /) -> DMatrix4x4: ...
    def translate(self, translation: DVector3, /) -> DMatrix4x4: ...
    @classmethod
    def perspective(
        cls, fov: float, aspect_ratio: float, near: float, far: float, /
    ) -> DMatrix4x4: ...
    @classmethod
    def orthographic(
        cls, left: float, right: float, top: float, bottom: float, near: float, far: float, /
    ) -> DMatrix4x4: ...
    @classmethod
    def look_at(cls, eye: DVector3, center: DVector3, up: DVector3, /) -> DMatrix4x4: ...
    def to_matrix3(self) -> DMatrix3x3: ...
    def to_quaternion(self) -> DQuaternion: ...
    def get_row(self, index: int, /) -> DVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DMatrix4x4Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix4x4: ...

@final
class DMatrix4x4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: DMatrix4x4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DMatrix4x4]: ...
    @overload
    def __getitem__(self, index: int) -> DMatrix4x4: ...
    @overload
    def __getitem__(self, index: slice) -> DMatrix4x4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DMatrix4x4Array: ...
    @classmethod
    def get_component_type(cls) -> type[DMatrix4x4]: ...

@final
class FMatrix4x4:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, all: Number, /): ...
    @overload
    def __init__(self, _0: FVector4, _1: FVector4, _2: FVector4, _3: FVector4, /): ...
    @overload
    def __init__(
        self,
        _0: Number,
        _1: Number,
        _2: Number,
        _3: Number,
        _4: Number,
        _5: Number,
        _6: Number,
        _7: Number,
        _8: Number,
        _9: Number,
        _10: Number,
        _11: Number,
        _12: Number,
        _13: Number,
        _14: Number,
        _15: Number,
        /,
    ): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FVector4]: ...
    def __getitem__(self, index: int) -> FVector4: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @overload
    def __add__(self, other: FMatrix4x4) -> FMatrix4x4: ...
    @overload
    def __add__(self, other: Number) -> FMatrix4x4: ...
    @overload
    def __radd__(self, other: FMatrix4x4) -> FMatrix4x4: ...
    @overload
    def __radd__(self, other: Number) -> FMatrix4x4: ...
    @overload
    def __sub__(self, other: FMatrix4x4) -> FMatrix4x4: ...
    @overload
    def __sub__(self, other: Number) -> FMatrix4x4: ...
    @overload
    def __rsub__(self, other: FMatrix4x4) -> FMatrix4x4: ...
    @overload
    def __rsub__(self, other: Number) -> FMatrix4x4: ...
    def __mul__(self, other: Number) -> FMatrix4x4: ...
    def __rmul__(self, other: Number) -> FMatrix4x4: ...
    @overload
    def __matmul__(self, other: FMatrix2x4) -> FMatrix2x4: ...
    @overload
    def __matmul__(self, other: FMatrix3x4) -> FMatrix3x4: ...
    @overload
    def __matmul__(self, other: FMatrix4x4) -> FMatrix4x4: ...
    @overload
    def __matmul__(self, other: FVector3) -> FVector3: ...
    @overload
    def __matmul__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rmatmul__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rmatmul__(self, other: FVector4) -> FVector4: ...
    @overload
    def __truediv__(self, other: Number) -> FMatrix4x4: ...
    @overload
    def __truediv__(self, other: FMatrix4x4) -> FMatrix4x4: ...
    @overload
    def __truediv__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rtruediv__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rtruediv__(self, other: Number) -> FMatrix4x4: ...
    def __neg__(self) -> FMatrix4x4: ...
    def __bool__(self) -> _bool: ...
    def inverse(self) -> FMatrix4x4: ...
    def transpose(self) -> FMatrix4x4: ...
    def rotate(self, angle: float, axis: FVector3, /) -> FMatrix4x4: ...
    def scale(self, scaling: FVector3, /) -> FMatrix4x4: ...
    def translate(self, translation: FVector3, /) -> FMatrix4x4: ...
    @classmethod
    def perspective(
        cls, fov: float, aspect_ratio: float, near: float, far: float, /
    ) -> FMatrix4x4: ...
    @classmethod
    def orthographic(
        cls, left: float, right: float, top: float, bottom: float, near: float, far: float, /
    ) -> FMatrix4x4: ...
    @classmethod
    def look_at(cls, eye: FVector3, center: FVector3, up: FVector3, /) -> FMatrix4x4: ...
    def to_matrix3(self) -> FMatrix3x3: ...
    def to_quaternion(self) -> FQuaternion: ...
    def get_row(self, index: int, /) -> FVector4: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FMatrix4x4Array: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix4x4: ...

@final
class FMatrix4x4Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *matrices: FMatrix4x4): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FMatrix4x4]: ...
    @overload
    def __getitem__(self, index: int) -> FMatrix4x4: ...
    @overload
    def __getitem__(self, index: slice) -> FMatrix4x4Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FMatrix4x4Array: ...
    @classmethod
    def get_component_type(cls) -> type[FMatrix4x4]: ...

@final
class DQuaternion:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, w: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[float]: ...
    def __getitem__(self, index: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    def __add__(self, other: DQuaternion) -> DQuaternion: ...
    def __radd__(self, other: DQuaternion) -> DQuaternion: ...
    def __sub__(self, other: DQuaternion) -> DQuaternion: ...
    def __rsub__(self, other: DQuaternion) -> DQuaternion: ...
    @overload
    def __mul__(self, other: DQuaternion) -> DQuaternion: ...
    @overload
    def __mul__(self, other: Number) -> DQuaternion: ...
    @overload
    def __rmul__(self, other: DQuaternion) -> DQuaternion: ...
    @overload
    def __rmul__(self, other: Number) -> DQuaternion: ...
    @overload
    def __matmul__(self, other: DQuaternion) -> float: ...
    @overload
    def __matmul__(self, other: DVector3) -> DVector3: ...
    @overload
    def __matmul__(self, other: DVector4) -> DVector4: ...
    @overload
    def __rmatmul__(self, other: DQuaternion) -> float: ...
    @overload
    def __rmatmul__(self, other: DVector3) -> DVector3: ...
    @overload
    def __rmatmul__(self, other: DVector4) -> DVector4: ...
    def __truediv__(self, other: Number) -> DQuaternion: ...
    def __neg__(self) -> DQuaternion: ...
    def __bool__(self) -> _bool: ...
    @property
    def w(self) -> float: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float: ...
    @property
    def magnitude(self) -> float: ...
    def cross(self, other: DQuaternion, /) -> DQuaternion: ...
    def to_matrix3(self) -> DMatrix3x3: ...
    def to_matrix4(self) -> DMatrix4x4: ...
    def normalize(self) -> DQuaternion: ...
    def inverse(self) -> DQuaternion: ...
    def rotate(self, angle: float, axis: DVector3) -> DQuaternion: ...
    def lerp(self, other: DQuaternion, x: float, /) -> DQuaternion: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> DQuaternionArray: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DQuaternion: ...
    @classmethod
    def get_component_type(cls) -> type[DQuaternion]: ...

@final
class DQuaternionArray:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: DQuaternion): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[DQuaternion]: ...
    @overload
    def __getitem__(self, index: int) -> DQuaternion: ...
    @overload
    def __getitem__(self, index: slice) -> DQuaternionArray: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DQuaternionArray: ...
    @classmethod
    def get_component_type(cls) -> type[DQuaternion]: ...

@final
class FQuaternion:
    __slots__ = ["__weakref__"]

    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, w: Number, /): ...
    @overload
    def __init__(self, x: Number, y: Number, z: Number, w: Number, /): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[float]: ...
    def __getitem__(self, index: int) -> float: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    def __add__(self, other: FQuaternion) -> FQuaternion: ...
    def __radd__(self, other: FQuaternion) -> FQuaternion: ...
    def __sub__(self, other: FQuaternion) -> FQuaternion: ...
    def __rsub__(self, other: FQuaternion) -> FQuaternion: ...
    @overload
    def __mul__(self, other: FQuaternion) -> FQuaternion: ...
    @overload
    def __mul__(self, other: Number) -> FQuaternion: ...
    @overload
    def __rmul__(self, other: FQuaternion) -> FQuaternion: ...
    @overload
    def __rmul__(self, other: Number) -> FQuaternion: ...
    @overload
    def __matmul__(self, other: FQuaternion) -> float: ...
    @overload
    def __matmul__(self, other: FVector3) -> FVector3: ...
    @overload
    def __matmul__(self, other: FVector4) -> FVector4: ...
    @overload
    def __rmatmul__(self, other: FQuaternion) -> float: ...
    @overload
    def __rmatmul__(self, other: FVector3) -> FVector3: ...
    @overload
    def __rmatmul__(self, other: FVector4) -> FVector4: ...
    def __truediv__(self, other: Number) -> FQuaternion: ...
    def __neg__(self) -> FQuaternion: ...
    def __bool__(self) -> _bool: ...
    @property
    def w(self) -> float: ...
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float: ...
    @property
    def magnitude(self) -> float: ...
    def cross(self, other: FQuaternion, /) -> FQuaternion: ...
    def to_matrix3(self) -> FMatrix3x3: ...
    def to_matrix4(self) -> FMatrix4x4: ...
    def normalize(self) -> FQuaternion: ...
    def inverse(self) -> FQuaternion: ...
    def rotate(self, angle: float, axis: FVector3) -> FQuaternion: ...
    def lerp(self, other: FQuaternion, x: float, /) -> FQuaternion: ...
    @classmethod
    def get_limits(cls) -> tuple[float, float]: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @classmethod
    def get_size(cls) -> int: ...
    @classmethod
    def get_array_type(cls) -> FQuaternionArray: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FQuaternion: ...
    @classmethod
    def get_component_type(cls) -> type[FQuaternion]: ...

@final
class FQuaternionArray:
    __slots__ = ["__weakref__"]

    def __init__(self, *vectors: FQuaternion): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[FQuaternion]: ...
    @overload
    def __getitem__(self, index: int) -> FQuaternion: ...
    @overload
    def __getitem__(self, index: slice) -> FQuaternionArray: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FQuaternionArray: ...
    @classmethod
    def get_component_type(cls) -> type[FQuaternion]: ...

@final
class BArray:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: _bool): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_bool]: ...
    @overload
    def __getitem__(self, index: int) -> _bool: ...
    @overload
    def __getitem__(self, index: slice) -> BArray: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_bool]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> BArray: ...

@final
class DArray:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: float): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[float]: ...
    @overload
    def __getitem__(self, index: int) -> float: ...
    @overload
    def __getitem__(self, index: slice) -> DArray: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_double]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> DArray: ...

@final
class FArray:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: float): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[float]: ...
    @overload
    def __getitem__(self, index: int) -> float: ...
    @overload
    def __getitem__(self, index: slice) -> FArray: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_float]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> FArray: ...

@final
class I8Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> I8Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> I8Array: ...

@final
class U8Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> U8Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint8]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> U8Array: ...

@final
class I16Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> I16Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> I16Array: ...

@final
class U16Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> U16Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint16]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> U16Array: ...

@final
class I32Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> I32Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> I32Array: ...

@final
class U32Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> U32Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint32]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> U32Array: ...

@final
class IArray:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> IArray: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> IArray: ...

@final
class UArray:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> UArray: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> UArray: ...

@final
class I64Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> I64Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_int64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> I64Array: ...

@final
class U64Array:
    __slots__ = ["__weakref__"]

    def __init__(self, *pods: int): ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[int]: ...
    @overload
    def __getitem__(self, index: int) -> int: ...
    @overload
    def __getitem__(self, index: slice) -> U64Array: ...
    def __eq__(self, other: Any) -> _bool: ...
    def __ne__(self, other: Any) -> _bool: ...
    def __bool__(self) -> _bool: ...
    def __buffer__(self, flags: int) -> memoryview: ...
    def __release_buffer__(self, view: memoryview) -> None: ...
    @property
    def pointer(self) -> ctypes._Pointer[ctypes.c_uint64]: ...
    @property
    def size(self) -> int: ...
    @classmethod
    def from_buffer(cls, buffer: Buffer) -> U64Array: ...
